%%
%% This is file `roman8.def',
%% generated with the docstrip utility.
%%
%% The original source files were:
%%
%% umlaute.dtx  (with options: `roman8')
%% 
%% Copyright (C) 1994-2006 Axel Sommerfeldt (caption@sommerfee.de)
%% 
%% --------------------------------------------------------------------------
%% 
%% This work may be distributed and/or modified under the
%% conditions of the LaTeX Project Public License, either version 1.3
%% of this license or (at your option) any later version.
%% The latest version of this license is in
%%   http://www.latex-project.org/lppl.txt
%% and version 1.3 or later is part of all distributions of LaTeX
%% version 2003/12/01 or later.
%% 
%% This work has the LPPL maintenance status "maintained".
%% 
%% This Current Maintainer of this work is Axel Sommerfeldt.
%% 
%% This work consists of the files umlaute.ins, umlaute.dtx
%% and the derived files umlaute.sty, atari.def, pc850.def,
%% isolatin.def, mac.def, and roman8.def.
%% 
\ProvidesFile{roman8.def}[2006/01/29 v2.1 Input encoding file (AS)]
\makeatletter
\ProvideTextCommandDefault{\textdegree}{\ensuremath{{^\circ}}}
\ProvideTextCommandDefault{\textonehalf}{\ensuremath{\frac12}}
\ProvideTextCommandDefault{\textonequarter}{\ensuremath{\frac14}}
\ProvideTextCommandDefault{\textthreequarters}{\ensuremath{\frac34}}
\ProvideTextCommandDefault{\texttrademark}{\ensuremath{^\mathrm{TM}}}
\ProvideTextCommandDefault{\textflorin}{\textit{f}}
\ProvideTextCommandDefault{\textpeseta}{Pt}
\ProvideTextCommandDefault{\textblacksquare}
   {\vrule \@width .3em \@height .4em \@depth -.1em\relax}
\ProvideTextCommandDefault{\textcent}
   {\TextSymbolUnavailable\textcent}
\ProvideTextCommandDefault{\textyen}
   {\TextSymbolUnavailable\textyen}
\ProvideTextCommandDefault{\textcurrency}
   {\TextSymbolUnavailable\textcurrency}
\ProvideTextCommandDefault{\textbrokenbar}
   {\TextSymbolUnavailable\textbrokenbar}
\providecommand{\mathonesuperior}{{^1}}
\providecommand{\mathtwosuperior}{{^2}}
\providecommand{\maththreesuperior}{{^3}}
\providecommand{\mathnsuperior}{{^n}}
\DeclareInputText{160}{~}
\DeclareInputText{161}{\@tabacckludge`A}
\DeclareInputText{162}{\^A}
\DeclareInputText{163}{\@tabacckludge`E}
\DeclareInputText{164}{\^E}
\DeclareInputText{165}{\"E}
\DeclareInputText{166}{\^I}
\DeclareInputText{167}{\"I}
\DeclareInputText{168}{\'{}}
\DeclareInputText{169}{\`{}}
\DeclareInputText{170}{\^{}}
\DeclareInputText{171}{\"{}}
\DeclareInputText{172}{\~{}}
\DeclareInputText{173}{\@tabacckludge`U}
\DeclareInputText{174}{\^U}
\DeclareInputText{175}{\textlira}
\DeclareInputText{176}{\={}}
\DeclareInputText{177}{\@tabacckludge'Y}
\DeclareInputText{178}{\@tabacckludge'y}
\DeclareInputMath{179}{\mathdegree}
\DeclareInputText{180}{{\c C}}
\DeclareInputText{181}{{\c c}}
\DeclareInputText{182}{\~N}
\DeclareInputText{183}{\~n}
\DeclareInputText{184}{\textexclamdown}
\DeclareInputText{185}{\textquestiondown}
\DeclareInputText{186}{\textcurrency}
\DeclareInputText{187}{\textsterling}
\DeclareInputText{188}{\textyen}
\DeclareInputText{189}{\S}
\DeclareInputText{190}{\textgulden}
\DeclareInputText{191}{\textcent}
\DeclareInputText{192}{\^a}
\DeclareInputText{193}{\^e}
\DeclareInputText{194}{\^o}
\DeclareInputText{195}{\^u}
\DeclareInputText{196}{\@tabacckludge'a}
\DeclareInputText{197}{\@tabacckludge'e}
\DeclareInputText{198}{\@tabacckludge'o}
\DeclareInputText{199}{\@tabacckludge'u}
\DeclareInputText{200}{\@tabacckludge`a}
\DeclareInputText{201}{\@tabacckludge`e}
\DeclareInputText{202}{\@tabacckludge`o}
\DeclareInputText{203}{\@tabacckludge`u}
\DeclareInputText{204}{\"a}
\DeclareInputText{205}{\"e}
\DeclareInputText{206}{\"o}
\DeclareInputText{207}{\"u}
\DeclareInputText{208}{\AA}
\DeclareInputText{209}{\^\i}
\DeclareInputText{210}{\O}
\DeclareInputText{211}{\AE}
\DeclareInputText{212}{\aa}
\DeclareInputText{213}{\@tabacckludge'\i}
\DeclareInputText{214}{\o}
\DeclareInputText{215}{\ae}
\DeclareInputText{216}{\"A}
\DeclareInputText{217}{\@tabacckludge`\i}
\DeclareInputText{218}{\"O}
\DeclareInputText{219}{\"U}
\DeclareInputText{220}{\@tabacckludge'E}
\DeclareInputText{221}{\"\i}
\DeclareInputText{222}{\ss}
\DeclareInputText{223}{\^O}
\DeclareInputText{224}{\@tabacckludge'A}
\DeclareInputText{225}{\~A}
\DeclareInputText{226}{\~a}
\DeclareInputText{227}{\DH}
\DeclareInputText{228}{\dh}
\DeclareInputText{229}{\@tabacckludge'I}
\DeclareInputText{230}{\@tabacckludge`I}
\DeclareInputText{231}{\@tabacckludge'O}
\DeclareInputText{232}{\@tabacckludge`O}
\DeclareInputText{233}{\~O}
\DeclareInputText{234}{\~o}
\DeclareInputText{235}{\v{S}}
\DeclareInputText{236}{\v{s}}
\DeclareInputText{237}{\@tabacckludge'U}
\DeclareInputText{238}{\"Y}
\DeclareInputText{239}{\"y}
\DeclareInputText{240}{\TH}
\DeclareInputText{241}{\th}
\DeclareInputText{242}{\textperiodcentered}
\DeclareInputMath{243}{\mu}
\DeclareInputText{244}{\P}
\DeclareInputText{245}{\textthreequarters}
\DeclareInputText{246}{\textemdash}
\DeclareInputText{247}{\textonequarter}
\DeclareInputText{248}{\textonehalf}
\DeclareInputMath{249}{\mathordmasculine}
\DeclareInputMath{250}{\mathordfeminine}
\DeclareInputText{251}{\guillemotleft}
\DeclareInputText{252}{\textblacksquare}
\DeclareInputText{253}{\guillemotright}
\DeclareInputMath{254}{\pm}
\DeclareInputText{255}{\nobreakspace}
\makeatother
\endinput
%%
%% End of file `roman8.def'.
