\qt@newcmd\qurantrans@de@i{Im Namen Gottes, des Erbarmers, des Barmherzigen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@ii{Lob sei Gott, dem Herrn der Welten,\qt@no{2}}%
\qt@newcmd\qurantrans@de@iii{Dem Erbarmer, dem Barmherzigen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@iv{Der Verfügungsgewalt besitzt über den Tag des Gerichtes!\qt@no{4}}%
\qt@newcmd\qurantrans@de@v{Dir dienen wir, und Dich bitten wir um Hilfe.\qt@no{5}}%
\qt@newcmd\qurantrans@de@vi{Führe uns den geraden Weg,\qt@no{6}}%
\qt@newcmd\qurantrans@de@vii{Den Weg derer, die Du begnadet hast, die nicht dem Zorn verfallen und nicht irregehen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@viii{\basmalahde Alif Lam Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@ix{Dies ist das Buch, an ihm ist kein Zweifel möglich, es ist eine Rechtleitung für die Gottesfürchtigen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@x{Die an das Unsichtbare glauben und das Gebet verrichten und von dem, was Wir ihnen beschert haben, spenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@xi{Und die an das glauben, was zu dir herabgesandt und was vor dir herabgesandt wurde, und die über das Jenseits Gewißheit hegen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@xii{Diese folgen einer Rechtleitung von ihrem Herrn, und das sind die, denen es wohl ergeht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@xiii{Denen, die ungläubig sind, ist es gleich, ob du sie warnst oder ob du sie nicht warnst; sie glauben nicht.\qt@no{6}}%
\qt@newcmd\qurantrans@de@xiv{Versiegelt hat Gott ihre Herzen und ihr Gehör, und über ihrem Augenlicht liegt eine Hülle. Und bestimmt ist für sie eine gewaltige Pein.\qt@no{7}}%
\qt@newcmd\qurantrans@de@xv{Unter den Menschen gibt es welche, die sagen: «Wir glauben an Gott und an den Jüngsten Tag.» Doch sie sind keine Gläubigen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@xvi{Sie versuchen, Gott und diejenigen, die glauben, zu betrügen. Sie betrügen aber (letztlich) nur sich selbst, und sie merken es nicht.\qt@no{9}}%
\qt@newcmd\qurantrans@de@xvii{In ihren Herzen ist Krankheit, und Gott hat ihre Krankheit noch vermehrt. Und für sie ist eine schmerzhafte Pein bestimmt dafür, daß sie zu lügen pflegten.\qt@no{10}}%
\qt@newcmd\qurantrans@de@xviii{Und wenn ihnen gesagt wird: «Stiftet nicht Unheil auf der Erde», sagen sie: «Wir schaffen ja nur Ordnung.»\qt@no{11}}%
\qt@newcmd\qurantrans@de@xix{Und doch sind eben sie die Unheilstifter, aber sie merken es nicht.\qt@no{12}}%
\qt@newcmd\qurantrans@de@xx{Und wenn ihnen gesagt wird: «Glaubt, wie die (anderen) Menschen glauben», sagen sie: «Sollen wir denn glauben, wie die Toren glauben!» Und doch sind eben sie die Toren, aber sie wissen es nicht.\qt@no{13}}%
\qt@newcmd\qurantrans@de@xxi{Und wenn sie diejenigen, die glauben, treffen, sagen sie: «Wir glauben..» Und wenn sie mit ihren teuflischen Anführern allein sind, sagen sie: «Wir stehen auf eurer Seite; wir treiben ja nur Spott.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@xxii{Gott ist es, der ihrer spottet und sie im Übermaß ihres Frevels verharren läßt, so daß sie blind umherirren.\qt@no{15}}%
\qt@newcmd\qurantrans@de@xxiii{Das sind die, die den Irrtum um die Rechtleitung erkauft haben. Doch bringt ihr Handel keinen Gewinn, noch folgen sie der Rechtleitung.\qt@no{16}}%
\qt@newcmd\qurantrans@de@xxiv{Mit ihnen ist es wie mit dem, der ein Feuer anzündete. Als es erleuchtete, was um ihn herum war, nahm Gott ihnen das Licht hinweg und ließ sie in Finsternissen zurück, so daß sie nicht sehen können.\qt@no{17}}%
\qt@newcmd\qurantrans@de@xxv{Taub, stumm, blind sind sie: Sie werden nicht umkehren.\qt@no{18}}%
\qt@newcmd\qurantrans@de@xxvi{Oder es ist wie ein heftiger Regen vom Himmel mit Finsternissen und Donner und Blitz. Sie stecken sich die Finger in die Ohren vor den Donnerschlägen aus Angst vor dem Tod. Und Gott umgreift die Ungläubigen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@xxvii{Der Blitz raubt ihnen beinahe das Augenlicht. Sooft er ihnen leuchtet, gehen sie in seinem Licht; läßt er es über ihnen dunkel werden, bleiben sie stehen. Und wenn Gott wollte, würde Er ihr Gehör und ihr Augenlicht fortnehmen. Gott hat Macht zu allen Dingen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@xxviii{O ihr Menschen, dienet eurem Herrn, der euch und die, die vor euch lebten, erschuf, auf daß ihr gottesfürchtig werdet,\qt@no{21}}%
\qt@newcmd\qurantrans@de@xxix{Der euch die Erde zu einer Unterlage und den Himmel zu einem Bau machte, und der vom Himmel Wasser herabkommen ließ und dadurch Früchte als Lebensunterhalt für euch hervorbrachte. So stellt Gott keine anderen als Gegenpart zur Seite, wo ihr (es) doch wißt.\qt@no{22}}%
\qt@newcmd\qurantrans@de@xxx{Und wenn ihr im Zweifel seid über das, was Wir auf unseren Diener hinabgesandt haben, dann bringt eine Sure gleicher Art bei und ruft eure Zeugen anstelle Gottes an, so ihr die Wahrheit sagt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@xxxi{Wenn ihr es nicht tut - und ihr werdet es nie tun können -, dann hütet euch vor dem Feuer, dessen Brennstoff Menschen und Steine sind und das für die Ungläubigen bereitet ist.\qt@no{24}}%
\qt@newcmd\qurantrans@de@xxxii{Und verkünde denen, die glauben und die guten Werke tun, daß für sie Gärten bestimmt sind, unter denen Bäche fließen. Sooft ihnen daraus eine Frucht als Lebensunterhalt beschert wird, sagen sie: «Das ist, was uns vorher beschert wurde»; es wird ihnen aber nur Ähnliches gebracht. Und sie haben darin geläuterte Gattinnen. Und sie werden darin ewig weilen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@xxxiii{Gott schämt sich nicht, als Gleichnis eine Mücke oder das, was darüber hinausgeht, zu nehmen. Diejenigen nun, die glauben, wissen, daß es die Wahrheit von ihrem Herrn ist. Diejenigen, die ungläubig sind, sagen: «Was will Gott mit einem solchen Gleichnis?» Er führt damit viele irre und leitet damit viele recht. Und Er führt damit nur die Frevler irre,\qt@no{26}}%
\qt@newcmd\qurantrans@de@xxxiv{Die den Bund Gottes, nachdem er geschlossen worden ist, brechen und das zerschneiden, was Gott befohlen hat zu verbinden, und die auf der Erde Unheil stiften. Das sind die Verlierer.\qt@no{27}}%
\qt@newcmd\qurantrans@de@xxxv{Wie könnt ihr Gott verleugnen, wo ihr tot waret und Er euch lebendig gemacht hat? Dann läßt Er euch sterben und macht euch wieder lebendig, und dann werdet ihr zu Ihm zurückgebracht.\qt@no{28}}%
\qt@newcmd\qurantrans@de@xxxvi{Er ist es, der für euch alles, was auf der Erde ist, erschaffen hat, dann hat Er sich zum Himmel aufgerichtet und ihn zu sieben Himmeln gestaltet. Und Er weiß über alle Dinge Bescheid.\qt@no{29}}%
\qt@newcmd\qurantrans@de@xxxvii{Und als dein Herr zu den Engeln sprach: «Ich werde auf der Erde einen Nachfolger einsetzen.» Sie sagten: «Willst Du auf ihr einen einsetzen, der auf ihr Unheil stiftet und Blut vergießt, während wir dein Lob singen und deine Heiligkeit rühmen?» Er sprach: «Ich weiß, was ihr nicht wißt.»\qt@no{30}}%
\qt@newcmd\qurantrans@de@xxxviii{Und Er lehrte Adam alle Dinge samt ihren Namen. Dann führte Er sie den Engeln vor und sprach: «Tut mir die Namen dieser kund, so ihr die Wahrheit sagt.»\qt@no{31}}%
\qt@newcmd\qurantrans@de@xxxix{Sie sagten: «Preis sei Dir! Wir haben kein Wissen außer dem, was Du uns gelehrt hast. Du bist der, der alles weiß und weise ist.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@xl{Er sprach: «O Adam, tu ihnen ihre Namen kund.» Als er ihre Namen kundgetan hatte, sprach Er: «Habe Ich euch nicht gesagt: Ich weiß das Unsichtbare der Himmel und der Erde, und Ich weiß, was ihr offenlegt und was ihr verschweigt?»\qt@no{33}}%
\qt@newcmd\qurantrans@de@xli{Und als Wir zu den Engeln sprachen: «Werft euch vor Adam nieder.» Da warfen sie sich nieder, außer Iblis. Der weigerte sich und verhielt sich hochmütig, und er war einer der Ungläubigen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@xlii{Und Wir sprachen: «O Adam, bewohne du und deine Gattin das Paradies. Eßt reichlich von ihm zu eurem Wohl, wo ihr wollt. Aber naht euch nicht diesem Baum, sonst gehört ihr zu denen, die Unrecht tun.»\qt@no{35}}%
\qt@newcmd\qurantrans@de@xliii{Da ließ sie Satan beide vom Paradies fallen und vertrieb sie vom Ort, wo sie waren. Und Wir sprachen: «Geht hinunter. Die einen von euch sind Feinde der anderen. Ihr habt auf der Erde Aufenthalt und Nutznießung für eine Weile.»\qt@no{36}}%
\qt@newcmd\qurantrans@de@xliv{Da nahm Adam von seinem Herrn Worte (der Umkehr) entgegen, so wandte Er sich ihm gnädig zu. Er ist der, der sich gnädig zuwendet, der Barmherzige.\qt@no{37}}%
\qt@newcmd\qurantrans@de@xlv{Wir sprachen: «Geht von ihm alle hinunter. Wenn dann von Mir eine Rechtleitung zu euch kommt, dann haben diejenigen, die meiner Rechtleitung folgen, nichts zu befürchten, und sie werden nicht traurig sein.\qt@no{38}}%
\qt@newcmd\qurantrans@de@xlvi{Diejenigen aber, die nicht glauben und unsere Zeichen für Lüge erklären, das sind die Gefährten des Feuers; sie werden darin ewig weilen.»\qt@no{39}}%
\qt@newcmd\qurantrans@de@xlvii{O ihr Kinder Israels, gedenket meiner Gnade, mit der Ich euch begnadet habe, und erfüllt euren Bund mit Mir, so will Ich meinen Bund mit euch erfüllen. Vor Mir sollt ihr Ehrfurcht haben.\qt@no{40}}%
\qt@newcmd\qurantrans@de@xlviii{Und glaubt an das, was Ich hinabgesandt habe zur Bestätigung dessen, was bei euch ist. Und seid nicht die ersten, die es verleugnen. Und verkauft nicht meine Zeichen für einen geringen Preis. Mich sollt ihr fürchten.\qt@no{41}}%
\qt@newcmd\qurantrans@de@xlix{Und verkleidet nicht das Wahre mit dem Falschen, und verschweigt nicht die Wahrheit, wo ihr es doch wißt.\qt@no{42}}%
\qt@newcmd\qurantrans@de@l{Und verrichtet das Gebet und entrichtet die Abgabe, und verneigt euch mit denen, die sich verneigen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@li{Wollt ihr denn den Menschen die Frömmigkeit gebieten und dabei euch selbst vergessen, wo ihr doch das Buch verlest? Habt ihr denn keinen Verstand?\qt@no{44}}%
\qt@newcmd\qurantrans@de@lii{Und sucht Hilfe in der Geduld und im Gebet. Und das ist ja schwer, außer für die Demütigen,\qt@no{45}}%
\qt@newcmd\qurantrans@de@liii{Die damit rechnen, daß sie ihrem Herrn begegnen und daß sie zu Ihm zurückkehren werden.\qt@no{46}}%
\qt@newcmd\qurantrans@de@liv{O ihr Kinder Israels, gedenket meiner Gnade, mit der Ich euch begnadet habe, und daß Ich euch vor den Weltenbewohnern bevorzugt habe.\qt@no{47}}%
\qt@newcmd\qurantrans@de@lv{Und hütet euch vor einem Tag, an dem keine Seele für eine andere etwas begleichen kann, keine Fürsprache von ihr angenommen und kein Lösegeld von ihr genommen wird, und an dem sie keine Unterstützung erfahren.\qt@no{48}}%
\qt@newcmd\qurantrans@de@lvi{Und als Wir euch von den Leuten des Pharao erretteten, die euch schlimme Pein zufügten, indem sie eure Söhne abschlachteten und nur eure Frauen am Leben ließen - darin war für euch eine gewaltige Prüfung von eurem Herrn.\qt@no{49}}%
\qt@newcmd\qurantrans@de@lvii{Und als Wir euretwegen das Meer spalteten und euch so retteten und die Leute des Pharao ertrinken ließen, während ihr zuschautet.\qt@no{50}}%
\qt@newcmd\qurantrans@de@lviii{Und als Wir uns mit Mose für vierzig Nächte verabredeten. Dann nahmt ihr euch, nachdem er weggegangen war, das Kalb und habt so Unrecht getan.\qt@no{51}}%
\qt@newcmd\qurantrans@de@lix{Dann verziehen Wir euch nach diesem (Frevel), auf daß ihr dankbar würdet.\qt@no{52}}%
\qt@newcmd\qurantrans@de@lx{Und als Wir Mose das Buch und die Unterscheidungsnorm zukommen ließen, auf daß ihr der Rechtleitung folget.\qt@no{53}}%
\qt@newcmd\qurantrans@de@lxi{Und als Mose zu seinem Volk sagte: «O mein Volk, ihr habt euch selbst Unrecht getan, indem ihr euch das Kalb genommen habt. Kehrt zu eurem Schöpfer um und tötet einander. Das ist besser für euch in den Augen eures Schöpfers.» Da wandte Er sich euch wieder gnädig zu. Er ist der, der sich gnädig zuwendet, der Barmherzige.\qt@no{54}}%
\qt@newcmd\qurantrans@de@lxii{Und als ihr sagtet: «O Mose, wir werden dir nicht glauben, bis wir Gott offen sehen.» Da ergriff euch der Donnerschlag, während ihr zuschautet.\qt@no{55}}%
\qt@newcmd\qurantrans@de@lxiii{Dann erweckten Wir euch nach eurem Tod, auf daß ihr dankbar würdet.\qt@no{56}}%
\qt@newcmd\qurantrans@de@lxiv{Und Wir ließen die Wolken euch überschatten und sandten auf euch das Manna und die Wachteln hinunter: «Eßt von den köstlichen Dingen, die Wir euch beschert haben.» Und nicht Uns taten sie Unrecht, sondern sich selbst haben sie Unrecht getan.\qt@no{57}}%
\qt@newcmd\qurantrans@de@lxv{Und als Wir sprachen: «Tretet in diese Stadt ein und eßt reichlich davon zu eurem Wohl, wo ihr wollt; und betretet das Tor in der Haltung der Niederwerfung und sagt: Entlastung!, dann vergeben Wir euch eure Verfehlungen. Und Wir werden den Rechtschaffenen noch mehr geben.\qt@no{58}}%
\qt@newcmd\qurantrans@de@lxvi{Dann vertauschten es diejenigen, die Unrecht taten, mit einem Ausspruch, der anders war als das, was ihnen gesagt worden war. Da sandten Wir über diejenigen, die Unrecht taten, ein Zorngericht vom Himmel herab dafür, daß sie frevelten.\qt@no{59}}%
\qt@newcmd\qurantrans@de@lxvii{Und als Mose um Wasser für sein Volk bat. Da sprachen Wir: «Schlag mit deinem Stab auf den Stein.» Da brachen aus ihm zwölf Quellen hervor, so daß jede Menschengruppe wußte, wo ihre Trinkstelle war. - «Eßt und trinkt von dem, was Gott beschert hat, und verbreitet nicht Unheil auf der Erde.»\qt@no{60}}%
\qt@newcmd\qurantrans@de@lxviii{Und als ihr sagtet: «O Mose, wir werden es nicht aushalten, nur eine einzige Speise zu haben. So rufe für uns deinen Herrn an, daß Er uns etwas hervorbringe von dem, was die Erde sonst wachsen läßt an Gemüse, Gurken, Knoblauch, Linsen und Zwiebeln.» Er sagte: «Wollt ihr denn das Minderwertige in Tausch gegen das Bessere nehmen? Zieht hinab nach Ägypten. Ihr habt dort, was ihr erbittet.» Und über sie wurden Erniedrigung und Elend gelegt, und sie zogen sich den Zorn Gottes zu. Dies dafür, daß sie immer wieder die Zeichen Gottes verleugneten und die Propheten zu Unrecht töteten; dies dafür, daß sie ungehorsam waren und immer wieder Übertretungen begingen.\qt@no{61}}%
\qt@newcmd\qurantrans@de@lxix{Diejenigen, die glauben, und diejenigen, die Juden sind, und die Christen und die Säbier, all die, die an Gott und den Jüngsten Tag glauben und Gutes tun, erhalten ihren Lohn bei ihrem Herrn, sie haben nichts zu befürchten, und sie werden nicht traurig sein.\qt@no{62}}%
\qt@newcmd\qurantrans@de@lxx{Und als Wir eure Verpflichtung entgegennahmen und über euch den Berg emporhoben: «Nehmt, was Wir euch zukommen ließen, mit aller Kraft und gedenket dessen, was darin steht, auf daß ihr gottesfürchtig seid.»\qt@no{63}}%
\qt@newcmd\qurantrans@de@lxxi{Aber ihr, ihr kehrtet euch danach ab. Ohne die Huld Gottes gegen euch und seine Barmherzigkeit wäret ihr gewiß unter den Verlierern.\qt@no{64}}%
\qt@newcmd\qurantrans@de@lxxii{Und ihr wißt um diejenigen von euch, die gegen den Sabbat eine Übertretung begangen haben, worauf Wir zu ihnen sprachen: «Werdet zu verabscheuten Affen.»\qt@no{65}}%
\qt@newcmd\qurantrans@de@lxxiii{So machten Wir es zu einem warnenden Beispiel für die Zeitgenossen und die Nachfolgenden und zu einer Ermahnung für die Gottesfürchtigen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@lxxiv{Und als Mose zu seinem Volk sagte: «Gott befiehlt euch, eine Kuh zu schächten.» Sie sagten: «Nimmst du uns zum Gegenstand des Spottes?» Er sagte: «Ich suche Zuflucht bei Gott davor, daß ich zu den Törichten gehöre.»\qt@no{67}}%
\qt@newcmd\qurantrans@de@lxxv{Sie sagten: «Rufe für uns deinen Herrn an, daß Er uns deutlich mache, wie sie sein soll.» Er sagte: «Er spricht: Es soll eine Kuh sein, die weder alt noch zu jung zum Kalben ist, eben mittleren Alters dazwischen. Nun tut, was euch befohlen wird.»\qt@no{68}}%
\qt@newcmd\qurantrans@de@lxxvi{Sie sagten: «Rufe für uns deinen Herrn an, daß Er uns deutlich mache, welcher Farbe sie sein soll.» Er sagte: «Er spricht: Es soll eine gelbe Kuh sein, deren Farbe intensiv ist und die den Zuschauern Freude macht.»\qt@no{69}}%
\qt@newcmd\qurantrans@de@lxxvii{Sie sagten: «Rufe für uns deinen Herrn an, daß Er uns deutlich mache, wie sie sein soll, denn die Kühe kommen uns doch ähnlich vor. Aber wir werden gewiß, so Gott will, die richtige finden.»\qt@no{70}}%
\qt@newcmd\qurantrans@de@lxxviii{Er sagte: «Er spricht: Es soll eine Kuh sein, die weder zum Pflügen des Ackers noch zum Bewässern der Felder gezwungen wurde, fehlerfrei und ohne Farbmischung.» Sie sagten: «Jetzt hast du die Wahrheit gebracht.» Sie schachteten sie. Doch beinahe hätten sie es nicht getan.\qt@no{71}}%
\qt@newcmd\qurantrans@de@lxxix{Und als ihr jemanden getötet hattet und über ihn miteinander Streit bekamt. Aber Gott bringt doch heraus, was ihr verschwiegen hattet.\qt@no{72}}%
\qt@newcmd\qurantrans@de@lxxx{Da sprachen Wir: «Schlagt ihn mit einem Stück von ihr.» So macht Gott die Toten wieder lebendig und läßt euch seine Zeichen sehen, auf daß ihr verständig werdet.\qt@no{73}}%
\qt@newcmd\qurantrans@de@lxxxi{Nach diesem Geschehen verhärteten sich eure Herzen, so daß sie wie Steine waren oder noch härter. Denn unter den Steinen gibt es welche, aus denen die Bäche hervorbrechen; und es gibt welche, die sich spalten, so daß das Wasser aus ihnen herauskommt; und es gibt welche, die aus Ehrfurcht vor Gott herunterfallen. Und Gott läßt nicht unbeachtet, was ihr tut.\qt@no{74}}%
\qt@newcmd\qurantrans@de@lxxxii{Erhofft ihr etwa, daß sie mit euch glauben, wo doch ein Teil von ihnen das Wort Gottes hörte, es aber dann wissentlich entstellte, nachdem er es verstanden hatte?\qt@no{75}}%
\qt@newcmd\qurantrans@de@lxxxiii{Wenn sie diejenigen, die glauben, treffen, sagen sie: «Wir glauben.» Und wenn sie miteinander allein sind, sagen sie: «Wollt ihr denn mit ihnen über das reden, was Gott euch eröffnet hat, so daß sie damit vor eurem Herrn mit euch streiten? Habt ihr denn keinen Verstand?»\qt@no{76}}%
\qt@newcmd\qurantrans@de@lxxxiv{Wissen sie denn nicht, daß Gott weiß, was sie geheimhalten und was sie offenlegen?\qt@no{77}}%
\qt@newcmd\qurantrans@de@lxxxv{Unter ihnen gibt es Ungelehrte, die das Buch nicht kennen, sondern nur Wunschvorstellungen heranziehen. Sie stellen ja nur Mutmaßungen an.\qt@no{78}}%
\qt@newcmd\qurantrans@de@lxxxvi{Aber wehe denen, die das Buch mit ihren Händen schreiben und dann sagen: «Dies ist von Gott her», um es für einen geringen Preis zu verkaufen! Wehe ihnen wegen dessen, was ihre Hände geschrieben haben, und wehe ihnen wegen dessen, was sie erwerben!\qt@no{79}}%
\qt@newcmd\qurantrans@de@lxxxvii{Und sie sagen: «Das Feuer wird uns sicher nur für eine Anzahl von Tagen berühren.» Sprich: Habt ihr bei Gott ein (solches) Versprechen erhalten? - dann wird Gott sein Versprechen niemals brechen -, oder sagt ihr über Gott, was ihr nicht wißt?\qt@no{80}}%
\qt@newcmd\qurantrans@de@lxxxviii{Nein, diejenigen, die eine schlechte Tat erworben haben und die ihre Schuld umfängt, das sind die Gefährten des Feuers; sie werden darin ewig weilen.\qt@no{81}}%
\qt@newcmd\qurantrans@de@lxxxix{Und diejenigen, die glauben und die guten Werke tun, das sind die Gefährten des Paradieses; sie werden darin ewig weilen.\qt@no{82}}%
\qt@newcmd\qurantrans@de@xc{Und als Wir die Verpflichtung der Kinder Israels entgegennahmen: «Ihr sollt nur Gott dienen, die Eltern gut behandeln, und auch die Verwandten, die Waisen und die Bedürftigen. Sprecht freundlich zu den Menschen. Verrichtet das Gebet und entrichtet die Abgabe.» Danach habt ihr - bis auf wenige von euch - den Rücken gekehrt und euch abgewandt.\qt@no{83}}%
\qt@newcmd\qurantrans@de@xci{Und als Wir eure Verpflichtung entgegennahmen: «Ihr sollt nicht gegenseitig euer Blut vergießen und nicht einander aus euren Wohnstätten vertreiben.» Da habt ihr sie bekräftigt, indem ihr dies bezeugt habt.\qt@no{84}}%
\qt@newcmd\qurantrans@de@xcii{Dennoch tötet gerade ihr einander und vertreibt einen Teil von euch aus ihren Wohnstätten, indem ihr einander in Schuld und Übertretung gegen sie unterstützt. Und wenn sie als Gefangene zu euch kommen, so löst ihr sie aus. Dabei ist euch doch verboten, sie zu vertreiben. Glaubt ihr denn nur an einen Teil des Buches und verleugnet den anderen? Die Vergeltung für diejenigen unter euch, die dies tun, ist nichts als Schande im diesseitigen Leben, und am Tag der Auferstehung werden sie der härtesten Pein zugeführt werden. Und Gott läßt nicht unbeachtet, was ihr tut.\qt@no{85}}%
\qt@newcmd\qurantrans@de@xciii{Das sind die, die das diesseitige Leben gegen das Jenseits erkauft haben. So wird ihnen die Pein nicht erleichtert, und sie werden keine Unterstützung erfahren.\qt@no{86}}%
\qt@newcmd\qurantrans@de@xciv{Und Wir ließen dem Mose das Buch zukommen und nach ihm die Gesandten folgen. Und Wir ließen Jesus, dem Sohn Marias, die deutlichen Zeichen zukommen und stärkten ihn mit dem Geist der Heiligkeit. Wollt ihr euch denn jedesmal, wenn euch ein Gesandter etwas bringt, was ihr nicht mögt, hochmütig verhalten und einen Teil (von ihnen) der Lüge zeihen und einen (anderen) Teil töten?\qt@no{87}}%
\qt@newcmd\qurantrans@de@xcv{Und sie sagen: «Unsere Herzen sind unbeschnitten.» Aber nein! Gott hat sie wegen ihres Unglaubens verflucht. Darum sind sie sowenig gläubig.\qt@no{88}}%
\qt@newcmd\qurantrans@de@xcvi{Und als ein Buch von Gott zu ihnen kam, das bestätigte, was bei ihnen war - zuvor pflegten sie um den Sieg über die zu bitten, die nicht glauben -, als nun zu ihnen kam, was sie kannten, da verleugneten sie es. So komme Gottes Fluch über die Ungläubigen!\qt@no{89}}%
\qt@newcmd\qurantrans@de@xcvii{Schlimm ist das, wofür sie ihre Seelen verkauft haben, daß sie verleugneten, was Gott herabgesandt hat, aus ungerechter Ablehnung, daß Gott etwas von seiner Huld herabsendet, auf wen von seinen Dienern Er will. So haben sie sich Zorn über Zorn zugezogen. Und die Ungläubigen erhalten eine schmähliche Pein.\qt@no{90}}%
\qt@newcmd\qurantrans@de@xcviii{Und wenn zu ihnen gesagt wird: «Glaubt an das, was Gott herabgesandt hat», sagen sie: «Wir glauben an das, was auf uns herabgesandt wurde.» Sie verleugnen aber, was nachher kam, obwohl das die Wahrheit ist, das bestätigt, was bei ihnen ist. Sprich: Warum habt ihr früher die Propheten Gottes getötet, wenn ihr gläubig seid?\qt@no{91}}%
\qt@newcmd\qurantrans@de@xcix{Und Mose kam doch zu euch mit den deutlichen Zeichen. Dann aber nahmt ihr euch, nachdem er weggegangen war, das Kalb und habt so Unrecht getan.\qt@no{92}}%
\qt@newcmd\qurantrans@de@c{Und als Wir eure Verpflichtung entgegennahmen und den Berg über euch emporhoben: «Nehmt, was Wir euch zukommen ließen, mit aller Kraft und hört darauf.» Sie sagten: «Wir hören, und wir gehorchen nicht.» Und für ihren Unglauben mußten sie das Kalb wie ein Getränk in ihre Herzen aufnehmen. Sprich: Schlimm ist das, was euer Glaube euch befiehlt, wenn ihr (überhaupt) gläubig seid.\qt@no{93}}%
\qt@newcmd\qurantrans@de@ci{Sprich: Wenn die jenseitige Wohnstätte bei Gott euch unter Ausschluß der anderen Menschen zusteht, dann wünscht euch den Tod, so ihr die Wahrheit sagt.\qt@no{94}}%
\qt@newcmd\qurantrans@de@cii{Aber niemals werden sie ihn sich wünschen wegen der Werke, die ihre Hände vorausgeschickt haben. Und Gott weiß über die, die Unrecht tun, Bescheid.\qt@no{95}}%
\qt@newcmd\qurantrans@de@ciii{Und du wirst sicher finden, daß unter den Menschen sie am meisten am Leben hängen - auch mehr als die Polytheisten. Manch einer von ihnen möchte gern tausend Jahre alt werden. Aber das Altwerden wird ihn gewiß nicht von der Pein wegrücken. Und Gott sieht wohl, was sie tun.\qt@no{96}}%
\qt@newcmd\qurantrans@de@civ{Sprich: Wenn einer dem Gabriel ein Feind ist - denn er hat ihn auf dein Herz herabkommen lassen mit der Erlaubnis Gottes als Bestätigung dessen, was vor ihm vorhanden war und als Rechtleitung und Frohbotschaft für die Gläubigen -,\qt@no{97}}%
\qt@newcmd\qurantrans@de@cv{Wenn einer ein Feind ist Gott und seinen Engeln und seinen Gesandten, und Gabriel und Michael, dann ist Gott den Ungläubigen ein Feind.\qt@no{98}}%
\qt@newcmd\qurantrans@de@cvi{Und Wir sandten zu dir gewiß deutliche Zeichen hinab. Nur die Frevler verleugnen sie.\qt@no{99}}%
\qt@newcmd\qurantrans@de@cvii{Will ein Teil von ihnen denn jedesmal, wenn sie einen Bund schließen, ihn verwerfen? Nein, die meisten von ihnen glauben nicht.\qt@no{100}}%
\qt@newcmd\qurantrans@de@cviii{Und als zu ihnen ein Gesandter von Gott her kam, der bestätigte, was bei ihnen war, warf ein Teil derer, denen das Buch zugekommen war, das Buch Gottes hinter seinen Rücken, als ob sie von nichts wußten.\qt@no{101}}%
\qt@newcmd\qurantrans@de@cix{Und sie folgten dem, was die Satane unter der Herrschaft Salomos vortrugen. Aber nicht Salomo war ungläubig, sondern die Satane waren ungläubig; sie lehrten die Menschen die Zauberei und das, was auf die Engel in Babel, Haarut und Maarut, herabgesandt worden war. Diese jedoch lehrten niemanden, ohne zu sagen: «Wir sind nur eine Versuchung, so werde nicht ungläubig.» So lernten sie von ihnen das, womit sie zwischen dem Mann und seiner Frau Zwietracht stifteten. Und sie vermögen niemanden damit zu schädigen, es sei denn mit Gottes Erlaubnis. Und sie lernten, was ihnen schadete und nicht nützte. Und sie wußten, daß derjenige, der dies erkauft, im Jenseits keinen Anteil hat. Schlimm ist das, wofür sie ihre Seele verkauft haben. Wenn sie es doch wüßten!\qt@no{102}}%
\qt@newcmd\qurantrans@de@cx{Und wenn sie geglaubt hätten und gottesfürchtig gewesen wären!... Eine Belohnung von Gott ist doch besser, wenn sie es doch wüßten!\qt@no{103}}%
\qt@newcmd\qurantrans@de@cxi{O ihr, die ihr glaubt, sagt nicht: «Achte auf uns» (raainaa), sondern sagt: «Schau auf uns» (unzurnaa). Und hört darauf. Und für die Ungläubigen ist eine schmerzhafte Pein bestimmt.\qt@no{104}}%
\qt@newcmd\qurantrans@de@cxii{Diejenigen von den Leuten des Buches und von den Polytheisten, die nicht glauben, mögen es nicht gern, daß auf euch etwas Gutes von eurem Herrn herabgesandt wird. Gott aber schenkt seine Barmherzigkeit in besonderer Weise, wem Er will. Und Gott besitzt große Huld.\qt@no{105}}%
\qt@newcmd\qurantrans@de@cxiii{Was Wir auch an Zeichen aufheben oder der Vergessenheit preisgeben Wir bringen dafür ein besseres oder ein gleiches. Weißt du denn nicht, daß Gott Macht hat zu allen Dingen?\qt@no{106}}%
\qt@newcmd\qurantrans@de@cxiv{Weißt du denn nicht, daß Gott die Königsherrschaft der Himmel und der Erde hat? Außer Gott habt ihr weder Freund noch Helfer.\qt@no{107}}%
\qt@newcmd\qurantrans@de@cxv{Oder wollt ihr von eurem Gesandten das gleiche fordern, was früher von Mose gefordert wurde? Wer den Unglauben gegen den Glauben eintauscht, ist vom rechten Weg abgeirrt.\qt@no{108}}%
\qt@newcmd\qurantrans@de@cxvi{Viele von den Leuten des Buches möchten gerne euch, nachdem ihr gläubig geworden seid, wieder zu Ungläubigen machen, da sie von sich aus Neid empfinden, nachdem ihnen die Wahrheit deutlich wurde. So verzeiht und seid nachsichtig, bis Gott mit seinem Befehl eintrifft. Gott hat Macht zu allen Dingen.\qt@no{109}}%
\qt@newcmd\qurantrans@de@cxvii{Verrichtet das Gebet und entrichtet die Abgabe. Und was ihr für euch an Gutem vorausschickt, das werdet ihr bei Gott vorfinden. Gott sieht wohl, was ihr tut.\qt@no{110}}%
\qt@newcmd\qurantrans@de@cxviii{Sie sagen: «Es werden das Paradies nur die betreten, die Juden oder Christen sind.» Das sind ihre Wünsche. Sprich: Bringt her euren Beweis, so ihr die Wahrheit sagt.\qt@no{111}}%
\qt@newcmd\qurantrans@de@cxix{Nein, wer sich völlig Gott hingibt und dabei rechtschaffen ist, der hat seinen Lohn bei seinem Herrn. Diese haben nichts zu befürchten, und sie werden nicht traurig sein.\qt@no{112}}%
\qt@newcmd\qurantrans@de@cxx{Die Juden sagen: «Die Christen haben keine Grundlage.» Und die Christen sagen: «Die Juden haben keine Grundlage.» Dabei lesen sie (alle) das Buch. Auch diejenigen, die unwissend sind, äußern sich in der gleichen Weise. Gott wird am Tage der Auferstehung zwischen ihnen über das urteilen, worüber sie uneins waren.\qt@no{113}}%
\qt@newcmd\qurantrans@de@cxxi{Und wer ist ungerechter als der, der verhindert, daß in den Anbetungsstätten Gottes seines Namens gedacht wird, und bestrebt ist, sie zu zerstören? Gerade diese dürfen sie nicht anders als voller Furcht betreten. Bestimmt ist für sie im Diesseits Schande und im Jenseits eine gewaltige Pein.\qt@no{114}}%
\qt@newcmd\qurantrans@de@cxxii{Gottes ist der Osten und der Westen. Wohin ihr euch auch wenden möget, dort ist das Antlitz Gottes. Gott umfaßt und weiß alles.\qt@no{115}}%
\qt@newcmd\qurantrans@de@cxxiii{Und sie sagen: «Gott hat sich ein Kind genommen.» Preis sei Ihm! Ihm gehört doch, was in den Himmeln und auf der Erde ist. Alle sind Ihm demütig ergeben.\qt@no{116}}%
\qt@newcmd\qurantrans@de@cxxiv{Er ist der Schöpfer der Himmel und der Erde. Wenn Er eine Sache beschlossen hat, sagt Er zu ihr nur: Sei!, und sie ist.\qt@no{117}}%
\qt@newcmd\qurantrans@de@cxxv{Diejenigen, die unwissend sind, sagen: «Wenn doch Gott zu uns spräche oder ein Zeichen zu uns käme!» Auch diejenigen, die vor ihnen lebten, äußerten sich in der gleichen Weise. Ihre Herzen sind einander ähnlich. Wir haben die Zeichen deutlich gemacht für Leute, die Gewißheit hegen.\qt@no{118}}%
\qt@newcmd\qurantrans@de@cxxvi{Wir sandten dich mit der Wahrheit als Freudenboten und Warner. Und du hast dich nicht für die Gefährten der Hölle zu verantworten.\qt@no{119}}%
\qt@newcmd\qurantrans@de@cxxvii{Weder die Juden noch die Christen werden mit dir zufrieden sein, bis du ihrer Glaubensrichtung folgst. Sprich: Nur die Rechtleitung Gottes ist die (wahre) Rechtleitung. Und wenn du ihren Neigungen folgst nach dem, was dir an Wissen zugekommen ist, so wirst du vor Gott weder Freund noch Helfer haben.\qt@no{120}}%
\qt@newcmd\qurantrans@de@cxxviii{Diejenigen, denen Wir das Buch zukommen ließen und die es lesen, wie es richtig gelesen werden soll, glauben daran. Diejenigen, die nicht daran glauben, das sind die Verlierer.\qt@no{121}}%
\qt@newcmd\qurantrans@de@cxxix{O ihr Kinder Israels, gedenket meiner Gnade, mit der Ich euch begnadet habe, und daß Ich euch vor den Weltenbewohnern bevorzugt habe.\qt@no{122}}%
\qt@newcmd\qurantrans@de@cxxx{Und hütet euch vor einem Tag, an dem keine Seele für eine andere etwas begleichen kann, kein Lösegeld von ihr angenommen wird und keine Fürsprache ihr nützt, und an dem sie keine Unterstützung erfahren.\qt@no{123}}%
\qt@newcmd\qurantrans@de@cxxxi{Und als Abraham von seinem Herrn durch Worte auf die Probe gestellt wurde und er sie erfüllte. Er sprach: «Ich mache dich zum Vorbild für die Menschen.» Er sagte: «Und auch welche von meiner Nachkommenschaft.» Er sprach: «Mein Bund erstreckt sich aber nicht auf die, die Unrecht tun.»\qt@no{124}}%
\qt@newcmd\qurantrans@de@cxxxii{Und als Wir das Haus zu einem Versammlungsort für die Menschen und zu einer sicheren Stätte machten: «Nehmt euch die Stätte Abrahams zu einem Gebetsort.» Und Wir erlegten Abraham und Ismael auf: «Reinigt mein Haus für diejenigen, die den Umlauf vollziehen und die eine Einkehrzeit einlegen und die sich verneigen und niederwerfen.»\qt@no{125}}%
\qt@newcmd\qurantrans@de@cxxxiii{Und als Abraham sagte: «Mein Herr, mach dieses zu einem sicheren Gebiet und beschere seinen Bewohnern Früchte, denen von ihnen, die an Gott und den Jüngsten Tag glauben.» Er sprach: «Und wer nicht glaubt, den lasse Ich ein wenig genießen, alsdann zwinge Ich ihn in die Pein des Feuers - welch schlimmes Ende!»\qt@no{126}}%
\qt@newcmd\qurantrans@de@cxxxiv{Und als Abraham dabei war, vom Haus die Fundamente hochzuziehen, (er) und Ismael. (Sie beteten:) «Unser Herr, nimm es von uns an. Du bist der, der alles hört und weiß.\qt@no{127}}%
\qt@newcmd\qurantrans@de@cxxxv{Unser Herr, mache uns beide Dir ergeben und (mache) aus unserer Nachkommenschaft eine Gemeinschaft, die Dir ergeben ist. Und zeige uns unsere Riten, und wende Dich uns gnädig zu. Du bist der, der sich gnädig zuwendet, der Barmherzige.\qt@no{128}}%
\qt@newcmd\qurantrans@de@cxxxvi{Unser Herr, laß unter ihnen einen Gesandten aus ihrer Mitte erstehen, der ihnen deine Zeichen verliest und sie das Buch und die Weisheit lehrt und sie läutert. Du bist der Mächtige, der Weise.»\qt@no{129}}%
\qt@newcmd\qurantrans@de@cxxxvii{Und wer verschmäht die Glaubensrichtung Abrahams außer dem, der seine Seele geringschätzt? Wir haben ihn ja im Diesseits auserwählt. Und im Jenseits gehört er zu den Rechtschaffenen.\qt@no{130}}%
\qt@newcmd\qurantrans@de@cxxxviii{Und als sein Herr zu ihm sprach: «Sei (Mir) ergeben.» Er sagte: «Ich ergebe mich dem Herrn der Welten.»\qt@no{131}}%
\qt@newcmd\qurantrans@de@cxxxix{Und Abraham hat es seinen Söhnen aufgetragen, er und auch Jakob: «O meine Söhne, Gott hat für euch die (reine) Religion erwählt. So sollt ihr nur als Gottergebene sterben.»\qt@no{132}}%
\qt@newcmd\qurantrans@de@cxl{Oder wart ihr zugegen, als Jakob im Sterben lag? Als er zu seinen Söhnen sagte: «Wem werdet ihr dienen nach mir?» Sie sagten: «Dienen werden wir deinem Gott und dem Gott deiner Väter Abraham, Ismael und Isaak, dem einzigen Gott. Und wir sind Ihm ergeben.»\qt@no{133}}%
\qt@newcmd\qurantrans@de@cxli{Das ist eine Gemeinschaft, die dahingegangen ist. Sie erhält, was sie erworben hat, und ihr erhaltet, was ihr erworben habt. Und ihr habt nicht zu verantworten, was sie zu tun pflegten.\qt@no{134}}%
\qt@newcmd\qurantrans@de@cxlii{Und sie sagen: «Werdet Juden oder Christen, so folgt ihr der Rechtleitung.» Sprich: Nein, (wir folgen) der Glaubensrichtung Abrahams, als Anhänger des reinen Glaubens; und er gehörte nicht zu den Polytheisten.\qt@no{135}}%
\qt@newcmd\qurantrans@de@cxliii{Sprecht: Wir glauben an Gott und an das, was zu uns herabgesandt wurde, und an das, was herabgesandt wurde zu Abraham, Ismael, Isaak, Jakob und den Stämmen, und an das, was Mose und Jesus zugekommen ist, und an das, was den (anderen) Propheten von ihrem Herrn zugekommen ist. Wir machen bei keinem von ihnen einen Unterschied. Und wir sind Ihm ergeben.\qt@no{136}}%
\qt@newcmd\qurantrans@de@cxliv{Wenn sie an das gleiche glauben, woran ihr glaubt, so folgen sie der Rechtleitung. Wenn sie sich abkehren, so befinden sie sich in Widerstreit. Gott wird dich vor ihnen schützen. Er ist der, der alles hört und weiß.\qt@no{137}}%
\qt@newcmd\qurantrans@de@cxlv{Nehmt an) das Kennzeichen Gottes. Und wer hat ein schöneres Kennzeichen als Gott? Ihm (allein) dienen wir.\qt@no{138}}%
\qt@newcmd\qurantrans@de@cxlvi{Sprich: Was streitet ihr mit uns über Gott, wo Er unser Herr und euer Herr ist? Wir haben unsere Werke, und ihr habt eure Werke (zu verantworten). Und wir sind zu Ihm aufrichtig.\qt@no{139}}%
\qt@newcmd\qurantrans@de@cxlvii{Oder wollt ihr sagen, daß Abraham, Ismael, Isaak, Jakob und die Stämme Juden oder Christen gewesen sind? Sprich: Wißt ihr es besser oder Gott? Wer ist denn ungerechter als der, der ein Zeugnis, das er von Gott hat, verschweigt? Aber Gott läßt nicht unbeachtet, was ihr tut.\qt@no{140}}%
\qt@newcmd\qurantrans@de@cxlviii{Das ist eine Gemeinschaft, die dahingegangen ist. Sie erhält, was sie erworben hat, und ihr erhaltet, was ihr erworben habt. Und ihr habt nicht zu verantworten, was sie zu tun pflegten.\qt@no{141}}%
\qt@newcmd\qurantrans@de@cxlix{Die Toren unter den Menschen werden sagen: «Was hat sie von ihrer Gebetsrichtung abgebracht, die sie (bisher) eingehalten haben?» Sprich: Gottes ist der Osten und der Westen. Er führt, wen Er will, zu einem geraden Weg.\qt@no{142}}%
\qt@newcmd\qurantrans@de@cl{Und so haben Wir euch zu einer in der Mitte stehenden Gemeinschaft gemacht, auf daß ihr Zeugen seid über die Menschen und daß der Gesandte Zeuge sei über euch. Und Wir haben die Gebetsrichtung, die du eingehalten hast, nur eingesetzt, um zu erfahren, wer dem Gesandten folgt, (und um ihn zu unterscheiden) von dem, der auf seinen Fersen kehrtmacht. Wahrlich, das ist schwer, außer für die, die Gott rechtleitet. Nimmer wird Gott es zulassen, daß euer Glaube umsonst gewesen ist. Gott hat Mitleid mit den Menschen und ist barmherzig.\qt@no{143}}%
\qt@newcmd\qurantrans@de@cli{Wir sehen, wie du dein Gesicht zum Himmel hin und her richtest. So werden Wir dir eine Gebetsrichtung festlegen, mit der du zufrieden sein wirst. Wende also dein Gesicht in Richtung der heiligen Moschee. Und wo immer ihr seid, wendet euer Gesicht in ihre Richtung. Diejenigen, denen das Buch zugekommen ist, wissen bestimmt, daß es die Wahrheit von ihrem Herrn ist. Gott läßt nicht unbeachtet, was sie tun.\qt@no{144}}%
\qt@newcmd\qurantrans@de@clii{Du magst zu denen, denen das Buch zugekommen ist, mit jedem Zeichen kommen, sie werden deiner Gebetsrichtung nicht folgen. Und auch du wirst ihrer Gebetsrichtung nicht folgen. Keiner von ihnen wird der Gebetsrichtung der anderen folgen. Und wenn du ihren Neigungen folgst nach dem, was an Wissen zu dir gekommen ist, gehörst du gewiß zu denen, die Unrecht tun.\qt@no{145}}%
\qt@newcmd\qurantrans@de@cliii{Diejenigen, denen Wir das Buch zukommen ließen, kennen es, wie sie ihre Söhne kennen. Aber ein Teil von ihnen verschweigt wissentlich die Wahrheit.\qt@no{146}}%
\qt@newcmd\qurantrans@de@cliv{Es ist die Wahrheit von deinem Herrn. So sei nicht einer der Zweifler.\qt@no{147}}%
\qt@newcmd\qurantrans@de@clv{Jeder hat eine Richtung, zu der er sich wendet. So eilt zu den guten Dingen um die Wette. Wo immer ihr euch befindet, Gott wird euch alle zusammenbringen. Gott hat Macht zu allen Dingen.\qt@no{148}}%
\qt@newcmd\qurantrans@de@clvi{Von wo du auch herausgehst, wende dein Gesicht in Richtung der heiligen Moschee. Es ist wirklich die Wahrheit von deinem Herrn. Gott läßt nicht unbeachtet, was ihr tut.\qt@no{149}}%
\qt@newcmd\qurantrans@de@clvii{Und von wo du auch herausgehst, wende dein Gesicht in Richtung der heiligen Moschee. Und wo immer ihr auch seid, wendet euer Gesicht in ihre Richtung, damit die Menschen keinen Beweisgrund gegen euch haben, außer denen von ihnen, die Unrecht tun - fürchtet sie nicht, sondern fürchtet Mich -, und damit Ich meine Gnade an euch vollende und auf daß ihr die Rechtleitung findet,\qt@no{150}}%
\qt@newcmd\qurantrans@de@clviii{So wie Wir auch unter euch einen Gesandten aus eurer Mitte entsandt haben, der euch unsere Zeichen verliest, euch läutert und euch das Buch und die Weisheit lehrt und euch das lehrt, was ihr nicht wußtet.\qt@no{151}}%
\qt@newcmd\qurantrans@de@clix{Darum gedenket Meiner, dann gedenke Ich euer, und danket Mir und seid nicht undankbar gegen Mich.\qt@no{152}}%
\qt@newcmd\qurantrans@de@clx{O ihr, die ihr glaubt, sucht Hilfe in der Geduld und im Gebet. Gott ist mit den Geduldigen.\qt@no{153}}%
\qt@newcmd\qurantrans@de@clxi{Und sagt nicht von denen, die auf dem Weg Gottes getötet werden, sie seien tot. Sie sind vielmehr lebendig, aber ihr merkt es nicht.\qt@no{154}}%
\qt@newcmd\qurantrans@de@clxii{Und Wir werden euch sicher Prüfungen aussetzen mit ein wenig Furcht und Hunger und mit Verlust an Vermögen, Seelen und Früchten. Und verkünde den Geduldigen frohe Botschaft,\qt@no{155}}%
\qt@newcmd\qurantrans@de@clxiii{Die, wenn ein Unglück sie trifft, sagen: «Wir gehören Gott, und wir kehren zu Ihm zurück.»\qt@no{156}}%
\qt@newcmd\qurantrans@de@clxiv{Auf sie kommen Segnungen und Barmherzigkeit von ihrem Herrn herab. Das sind die, die der Rechtleitung folgen.\qt@no{157}}%
\qt@newcmd\qurantrans@de@clxv{Siehe, Safaa und Marwa gehören zu den Kultzeichen Gottes. Wer also die Wallfahrt zum Haus (Gottes) oder den Pilgerbesuch vollzieht, für den ist es kein Vergehen, zwischen ihnen den Lauf zu verrichten. Und wenn einer freiwillig Gutes tut, so zeigt sich Gott erkenntlich und weiß Bescheid.\qt@no{158}}%
\qt@newcmd\qurantrans@de@clxvi{Diejenigen, die verschweigen, was Wir an deutlichen Zeichen und Rechtleitung hinabgesandt haben, nachdem Wir es den Menschen im Buch deutlich gemacht haben, diese wird Gott verfluchen, und verfluchen werden sie auch die Fluchenden,\qt@no{159}}%
\qt@newcmd\qurantrans@de@clxvii{Außer denen, die umkehren und Besserung zeigen und (alles) offenlegen. Denen wende Ich Mich gnädig zu. Ich bin der, der sich gnädig zuwendet, der Barmherzige.\qt@no{160}}%
\qt@newcmd\qurantrans@de@clxviii{Über diejenigen, die nicht glauben und als Ungläubige sterben, kommt der Fluch Gottes und der Engel und der Menschen allesamt.\qt@no{161}}%
\qt@newcmd\qurantrans@de@clxix{Sie werden darin ewig weilen. Ihnen wird die Pein nicht erleichtert, und ihnen wird kein Aufschub gewährt.\qt@no{162}}%
\qt@newcmd\qurantrans@de@clxx{Und euer Gott ist ein einziger Gott, es gibt keinen Gott außer Ihm, dem Erbarmer, dem Barmherzigen.\qt@no{163}}%
\qt@newcmd\qurantrans@de@clxxi{In der Erschaffung der Himmel und der Erde; im Aufeinanderfolgen von Nacht und Tag; in den Schiffen, die auf dem Meer fahren mit dem, was den Menschen nützt; im Wasser, das Gott vom Himmel herabkommen läßt und mit dem Er die Erde nach ihrem Absterben wieder belebt und auf ihr allerlei Getier sich ausbreiten läßt; im Wechsel der Winde und der zwischen Himmel und Erde in Dienst gestellten Wolken, (in alledem) sind Zeichen für Leute, die verständig sind.\qt@no{164}}%
\qt@newcmd\qurantrans@de@clxxii{Unter den Menschen sind welche, die sich neben Gott andere als Gegenpart nehmen, die sie lieben, wie man Gott liebt. Doch diejenigen, die glauben, lieben Gott noch mehr. Wenn nur diejenigen, die Unrecht tun, angesichts der Pein sehen würden, daß alle Kraft Gott gehört und daß Gott harte Pein verhängt!\qt@no{165}}%
\qt@newcmd\qurantrans@de@clxxiii{Und dies), wenn diejenigen, denen man folgte, sich von denen, die ihnen folgten, lossagen, sie die Pein sehen, die Verbindungen für sie abgeschnitten sind,\qt@no{166}}%
\qt@newcmd\qurantrans@de@clxxiv{Und diejenigen, die (ihnen) folgten, sagen: «Hätten wir doch eine Möglichkeit zur Rückkehr, damit wir uns von ihnen lossagen, so wie sie sich von uns losgesagt haben!» Auf diese Weise läßt Gott sie ihre Werke sehen als Grund zu vielfachem Bedauern für sie. Und sie werden aus dem Feuer nicht herauskommen.\qt@no{167}}%
\qt@newcmd\qurantrans@de@clxxv{O ihr Menschen, eßt von dem, was es auf der Erde gibt, so es erlaubt und köstlich ist. Und folgt nicht den Fußstapfen des Satans. Er ist euch ein offenkundiger Feind.\qt@no{168}}%
\qt@newcmd\qurantrans@de@clxxvi{Er befiehlt euch nur Böses und Schändliches, und daß ihr über Gott sagt, was ihr nicht wißt.\qt@no{169}}%
\qt@newcmd\qurantrans@de@clxxvii{Und wenn ihnen gesagt wird: «Folgt dem, was Gott herabgesandt hat», sagen sie: «Wir folgen lieber dem, was wir bei unseren Vätern vorgefunden haben.» Was denn, auch wenn ihre Väter nichts verstanden haben und der Rechtleitung nicht gefolgt sind?\qt@no{170}}%
\qt@newcmd\qurantrans@de@clxxviii{Mit denen, die ungläubig sind, ist es, wie wenn einer etwas anschreit, was nur Rufen und Zurufen hört. Taub, stumm, blind sind sie, so haben sie keinen Verstand.\qt@no{171}}%
\qt@newcmd\qurantrans@de@clxxix{O ihr, die ihr glaubt, eßt von den köstlichen Dingen, die Wir euch beschert haben, und danket Gott, so ihr (wirklich) Ihm dient.\qt@no{172}}%
\qt@newcmd\qurantrans@de@clxxx{Verboten hat Er euch nur Verendetes, Blut, Schweinefleisch und das, worüber ein anderer als Gott angerufen worden ist. Wer aber gezwungen wird, ohne daß er Auflehnung oder Übertretung begeht, den trifft keine Schuld. Gott ist voller Vergebung und barmherzig.\qt@no{173}}%
\qt@newcmd\qurantrans@de@clxxxi{Diejenigen, die verschweigen, was Gott vom Buch herabgesandt hat, und es für einen geringen Preis verkaufen, werden in ihrem Bauch nichts als Feuer verzehren. Gott wird sie am Tag der Auferstehung nicht ansprechen und nicht für rein erklären. Und für sie ist eine schmerzhafte Pein bestimmt.\qt@no{174}}%
\qt@newcmd\qurantrans@de@clxxxii{Das sind die, die den Irrtum gegen die Rechtleitung erkauft haben, und auch die Pein gegen die Vergebung. Wie können sie sich dem Feuer gegenüber so standhaft zeigen!\qt@no{175}}%
\qt@newcmd\qurantrans@de@clxxxiii{Dies geschieht, weil Gott das Buch mit der Wahrheit herabgesandt hat. Und diejenigen, die über das Buch uneins sind, befinden sich in einem tiefen Widerstreit.\qt@no{176}}%
\qt@newcmd\qurantrans@de@clxxxiv{Frömmigkeit besteht nicht darin, daß ihr euer Gesicht nach Osten und Westen wendet. Frömmigkeit besteht darin, daß man an Gott, den Jüngsten Tag, die Engel, das Buch und die Propheten glaubt, daß man, aus Liebe zu Ihm, den Verwandten, den Waisen, den Bedürftigen, dem Reisenden und den Bettlern Geld zukommen läßt und (es) für den Loskauf der Sklaven und Gefangenen (ausgibt), und daß man das Gebet verrichtet und die Abgabe entrichtet. (Fromm sind auch) die, die ihre eingegangenen Verpflichtungen erfüllen, und die, die in Not und Leid und zur Zeit der Gewalt geduldig sind. Sie sind es, die wahrhaftig sind, und sie sind die Gottesfürchtigen.\qt@no{177}}%
\qt@newcmd\qurantrans@de@clxxxv{O ihr, die ihr glaubt, vorgeschrieben ist euch bei Totschlag die Wiedervergeltung: der Freie für den Freien, der Sklave für den Sklaven, das Weib für das Weib. Wenn einem von seinem Bruder etwas nachgelassen wird, dann soll die Beitreibung auf rechtliche Weise und die Leistung an ihn auf gute Weise erfolgen. Dies ist eine Erleichterung von Seiten eures Herrn und eine Barmherzigkeit. Wer danach Übertretungen begeht, für den ist eine schmerzhafte Pein bestimmt.\qt@no{178}}%
\qt@newcmd\qurantrans@de@clxxxvi{In der Wiedervergeltung liegt für euch Leben, o ihr Einsichtigen, auf daß ihr gottesfürchtig werdet.\qt@no{179}}%
\qt@newcmd\qurantrans@de@clxxxvii{Vorgeschrieben ist euch, wenn einer von euch im Sterben liegt, falls er Vermögen hinterläßt, eine Verfügung zugunsten der Eltern und der Angehörigen auf rechtliche Weise zu machen. Das ist eine Rechtspflicht für die Gottesfürchtigen.\qt@no{180}}%
\qt@newcmd\qurantrans@de@clxxxviii{Wenn jemand es abändert, nachdem er es gehört hat, so lastet die Schuld daran auf denjenigen, die es abändern. Gott hört und weiß alles.\qt@no{181}}%
\qt@newcmd\qurantrans@de@clxxxix{Wer aber von Seiten eines Erblassers eine Unrechtmäßigkeit oder ein schuldhaftes Verhalten befürchtet und dann zwischen ihnen Aussöhnung schafft, den trifft keine Schuld. Gott ist voller Vergebung und barmherzig.\qt@no{182}}%
\qt@newcmd\qurantrans@de@cxc{O ihr, die ihr glaubt, vorgeschrieben ist euch, zu fasten, so wie es denen vorgeschrieben worden ist, die vor euch lebten, auf daß ihr gottesfürchtig werdet,\qt@no{183}}%
\qt@newcmd\qurantrans@de@cxci{und dies) für eine Anzahl von Tagen. Wer von euch krank ist oder sich auf einer Reise befindet, für den gilt eine Anzahl anderer Tage. Denjenigen aber, die es (eigentlich einhalten) können, ist als Ersatzleistung die Speisung eines Bedürftigen auferlegt. Wenn einer freiwillig Gutes tut, so ist es besser für ihn. Und daß ihr fastet, ist besser für euch, wenn ihr Bescheid wißt.\qt@no{184}}%
\qt@newcmd\qurantrans@de@cxcii{Der Monat Ramadan ist es, in dem der Koran herabgesandt wurde als Rechtleitung für die Menschen und als deutliche Zeichen der Rechtleitung und der Unterscheidungsnorm. Wer von euch nun in dem Monat anwesend ist, der soll in ihm fasten. Und wer krank ist oder sich auf einer Reise befindet, für den gilt eine Anzahl anderer Tage. Gott will für euch Erleichterung, Er will für euch nicht Erschwernis, und daß ihr die Zahl (der Tage) vollendet und Gott dafür hochpreiset, daß Er euch rechtgeleitet hat, und daß ihr wohl dankbar werdet.\qt@no{185}}%
\qt@newcmd\qurantrans@de@cxciii{Wenn dich meine Diener nach Mir fragen, so bin Ich nahe, und Ich erhöre den Ruf des Rufenden, wenn er Mich anruft. Sie sollen nun auf Mich hören, und sie sollen an Mich glauben, auf daß sie einen rechten Wandel zeigen.\qt@no{186}}%
\qt@newcmd\qurantrans@de@cxciv{Erlaubt ist euch, in der Nacht während der Fastenzeit Umgang mit euren Frauen zu haben. Sie sind eine Bekleidung für euch, und ihr seid eine Bekleidung für sie. Gott weiß, daß ihr euch immer wieder selbst betrogen habt, so hat Er sich euch gnädig wieder zugewandt und euch verziehen. Verkehrt nunmehr mit ihnen und trachtet nach dem, was Gott euch vorgeschrieben hat. Und eßt und trinkt, bis ihr in der Morgendämmerung den weißen Faden vom schwarzen Faden unterscheiden könnt. Danach vollzieht das Fasten bis zur Nacht. Und verkehrt nicht mit ihnen, während ihr in den Moscheen eine Einkehrzeit einlegt. Dies sind die Bestimmungen Gottes, tretet ihnen nicht zu nahe. So macht Gott den Menschen seine Zeichen deutlich, auf daß sie gottesfürchtig werden.\qt@no{187}}%
\qt@newcmd\qurantrans@de@cxcv{Und verzehrt nicht untereinander euer Vermögen durch Betrug, und übergebt es nicht den Richtern, um einen Teil des Vermögens der Menschen in sündhafter Weise wissentlich zu verzehren.\qt@no{188}}%
\qt@newcmd\qurantrans@de@cxcvi{Sie fragen dich nach den Neumonden. Sprich: Sie sind Zeitbestimmungen für die Menschen und für die Wallfahrt. Und Frömmigkeit besteht nicht darin, daß ihr durch die Hinterseite in die Häuser geht. Frömmigkeit besteht darin, daß man gottesfürchtig ist. Geht also in die Häuser durch ihre Türen. Und fürchtet Gott, auf daß es euch wohl ergehe.\qt@no{189}}%
\qt@newcmd\qurantrans@de@cxcvii{Und kämpft auf dem Weg Gottes gegen diejenigen, die gegen euch kämpfen, und begeht keine Übertretungen. Gott liebt die nicht, die Übertretungen begehen.\qt@no{190}}%
\qt@newcmd\qurantrans@de@cxcviii{Und tötet sie, wo immer ihr sie trefft, und vertreibt sie, von wo sie euch vertrieben haben. Denn Verführen ist schlimmer als Töten. Kämpft nicht gegen sie bei der heiligen Moschee, bis sie dort gegen euch kämpfen. Wenn sie gegen euch kämpfen, dann tötet sie. So ist die Vergeltung für die Ungläubigen.\qt@no{191}}%
\qt@newcmd\qurantrans@de@cxcix{Wenn sie aufhören, so ist Gott voller Vergebung und barmherzig.\qt@no{192}}%
\qt@newcmd\qurantrans@de@cc{Kämpft gegen sie, bis es keine Verführung mehr gibt und bis die Religion nur noch Gott gehört. Wenn sie aufhören, dann darf es keine Übertretung geben, es sei denn gegen die, die Unrecht tun.\qt@no{193}}%
\qt@newcmd\qurantrans@de@cci{Ein heiliger Monat (darf zur Vergeltung dienen) für einen heiligen Monat. Bei den heiligen Dingen gilt die Wiedervergeltung. Wer sich gegen euch vergeht, gegen den dürft ihr euch ähnlich vergehen, wie er sich gegen euch vergeht. Und fürchtet Gott und wißt, daß Gott mit den Gottesfürchtigen ist.\qt@no{194}}%
\qt@newcmd\qurantrans@de@ccii{Und spendet auf dem Weg Gottes und streckt nicht eure Hände nach dem Verderben aus, und tut Gutes. Gott liebt die, die Gutes tun.\qt@no{195}}%
\qt@newcmd\qurantrans@de@cciii{Und vollzieht die Wallfahrt und den Pilgerbesuch für Gott. Wenn ihr behindert seid, dann gebt, was an Opfertieren erschwinglich ist. Und schert euch nicht den Kopf, bis die Opfertiere ihren erlaubten Schlachtort erreicht haben. Wer von euch krank ist oder ein Leiden am Kopf hat, der hat eine Ersatzleistung zu bringen: Fasten oder Almosen oder Tieropfer. Wenn ihr nun in Sicherheit seid, so soll derjenige, der nach dem Pilgerbesuch (das normale Leben) bis zur (Zeit der) Wallfahrt genossen hat, darbringen, was an Opfertieren erschwinglich ist. Wer es nicht vermag, der soll während der Wallfahrt drei Tage fasten und sieben, nachdem ihr zurückgekehrt seid: Das sind insgesamt zehn. Dies gilt für den, dessen Angehörige nicht im Bezirk der heiligen Moschee wohnen. Fürchtet Gott und wißt, daß Gott eine harte Strafe verhängt.\qt@no{196}}%
\qt@newcmd\qurantrans@de@cciv{Die Wallfahrt findet in bekannten Monaten statt. Wer sich in ihnen die Wallfahrt auferlegt, hat sich während der Wallfahrt des Geschlechtsumgangs, des Frevels und des Streites zu enthalten. Und was ihr an Gutem tut, das weiß Gott. Und versorgt euch mit Wegzehrung. Aber die beste Wegzehrung ist die Gottesfurcht. Und fürchtet Mich, o ihr Einsichtigen.\qt@no{197}}%
\qt@newcmd\qurantrans@de@ccv{Es ist für euch kein Vergehen, daß ihr nach einer Huld von eurem Herrn strebt. Und wenn ihr von ``Arafat herabeilt, dann gedenket Gottes vor der heiligen Kultstätte. Gedenket Seiner, wie Er euch rechtgeleitet hat. Ihr gehörtet ja vordem zu denen, die irregehen.\qt@no{198}}%
\qt@newcmd\qurantrans@de@ccvi{Dann eilt herab, von wo die Menschen herabeilen, und bittet Gott um Vergebung. Gott ist voller Vergebung und barmherzig.\qt@no{199}}%
\qt@newcmd\qurantrans@de@ccvii{Und wenn ihr eure Riten beendet habt, dann gedenket Gottes, wie ihr eurer Väter (früher) gedacht habt oder mit noch innigerem Gedenken. Unter den Menschen gibt es welche, die sagen: «Unser Herr, beschenke uns im Diesseits.» Diese haben im Jenseits keinen Anteil.\qt@no{200}}%
\qt@newcmd\qurantrans@de@ccviii{Und unter ihnen gibt es welche, die sagen: «Unser Herr, schenke uns im Diesseits Gutes und auch im Jenseits Gutes, und bewahre uns vor der Pein des Feuers.»\qt@no{201}}%
\qt@newcmd\qurantrans@de@ccix{Diese erhalten einen Anteil von dem, was sie sich erworben haben. Gott ist schnell im Abrechnen.\qt@no{202}}%
\qt@newcmd\qurantrans@de@ccx{Und gedenket Gottes in einer bestimmten Anzahl von Tagen. Wer sich beeilt und es in zwei Tagen durchführt, den trifft keine Schuld. Und wer längere Zeit braucht, den trifft auch keine Schuld. (Dies gilt) für den, der gottesfürchtig ist. Fürchtet Gott und wißt, daß ihr zu Ihm versammelt werdet.\qt@no{203}}%
\qt@newcmd\qurantrans@de@ccxi{Und unter den Menschen gibt es manch einen, dessen Reden über das diesseitige Leben dir gefällt und der Gott zum Zeugen anruft für das, was in seinem Herzen ist. Dabei ist er der streitsüchtigste Widersacher.\qt@no{204}}%
\qt@newcmd\qurantrans@de@ccxii{Und wenn er an der Macht ist, reist er auf der Erde umher, um Unheil auf ihr zu stiften und Saatfelder und Nachwuchs zu verderben. Aber Gott liebt das Unheil nicht.\qt@no{205}}%
\qt@newcmd\qurantrans@de@ccxiii{Und wenn man zu ihm sagt: «Fürchte Gott», so führt ihn die Überheblichkeit zur Schuld. So ist die Hölle das, was für ihn eben ausreicht. Welch schlimme Lagerstätte!\qt@no{206}}%
\qt@newcmd\qurantrans@de@ccxiv{Und unter den Menschen gibt es auch manch einen, der sich selbst veräußern würde im Streben nach dem Wohlwollen Gottes. Und Gott hat Mitleid mit den Dienern.\qt@no{207}}%
\qt@newcmd\qurantrans@de@ccxv{O ihr, die ihr glaubt, tretet allesamt in die Ergebung und folgt nicht den Fußstapfen des Satans. Er ist euch ein offenkundiger Feind.\qt@no{208}}%
\qt@newcmd\qurantrans@de@ccxvi{Wenn ihr aber strauchelt, nachdem die deutlichen Zeichen zu euch gekommen sind, so wißt, daß Gott mächtig und weise ist.\qt@no{209}}%
\qt@newcmd\qurantrans@de@ccxvii{Erwarten sie denn etwas anderes, als daß Gott zu ihnen in überschattenden Wolken kommt, und auch die Engel? Die Angelegenheit ist dann entschieden. Und zu Gott werden alle Angelegenheiten zurückgebracht.\qt@no{210}}%
\qt@newcmd\qurantrans@de@ccxviii{Frag die Kinder Israels, wie viele deutliche Zeichen Wir ihnen zukommen ließen. Wenn aber einer die Gnade Gottes eintauscht, nachdem sie zu ihm gekommen ist, so verhängt Gott eine harte Strafe.\qt@no{211}}%
\qt@newcmd\qurantrans@de@ccxix{Das diesseitige Leben ist denen, die ungläubig sind, verlockend gemacht worden, und sie verhöhnen diejenigen, die glauben. Aber diejenigen, die gottesfürchtig sind, stehen am Tag der Auferstehung über ihnen. Und Gott beschert den Lebensunterhalt, wem Er will, ohne (viel) zu rechnen.\qt@no{212}}%
\qt@newcmd\qurantrans@de@ccxx{Die Menschen waren eine einzige Gemeinschaft. Dann ließ Gott die Propheten als Freudenboten und Warner erstehen. Er sandte mit ihnen das Buch mit der Wahrheit herab, damit es zwischen den Menschen über das urteile, worüber sie uneins waren. Und nur jene, denen es zuteil wurde, waren darüber uneins, nachdem die deutlichen Zeichen zu ihnen gekommen waren - dies aus ungerechter Auflehnung untereinander. Nun hat Gott die, die glauben, mit seiner Erlaubnis zu der Wahrheit geleitet, über die sie uneins waren. Und Gott führt, wen Er will, zu einem geraden Weg.\qt@no{213}}%
\qt@newcmd\qurantrans@de@ccxxi{Oder meint ihr, daß ihr ins Paradies eingehen werdet, noch ehe euch das gleiche widerfahren ist wie denen, die vor euch dahingegangen sind? Not und Leid berührten sie, und sie wurden hin und her geschüttelt, so daß der Gesandte und diejenigen, die mit ihm gläubig waren, dann sagten: Wann kommt die Unterstützung Gottes? Wahrlich, die Unterstützung Gottes ist nahe.\qt@no{214}}%
\qt@newcmd\qurantrans@de@ccxxii{Sie fragen dich, was sie spenden sollen. Sprich: Was ihr an Gutem spendet, das sei für die Eltern, die Angehörigen, die Waisen, die Bedürftigen und den Reisenden. Und was ihr an Gutem tut, Gott weiß es.\qt@no{215}}%
\qt@newcmd\qurantrans@de@ccxxiii{Vorgeschrieben ist euch der Kampf, obwohl er euch zuwider ist. Aber vielleicht ist euch etwas zuwider, während es gut für euch ist. Und vielleicht liebt ihr etwas, während es schlecht für euch ist. Und Gott weiß, ihr aber wißt nicht Bescheid.\qt@no{216}}%
\qt@newcmd\qurantrans@de@ccxxiv{Sie fragen dich nach dem heiligen Monat, nach dem Kampf in ihm. Sprich: Der Kampf in ihm ist schwerwiegend; aber (die Menschen) vom Wege Gottes abweisen, an Ihn nicht glauben, den Zugang zur heiligen Moschee verwehren und deren Anwohner daraus vertreiben, (all das) wiegt bei Gott schwerer. Verführen wiegt schwerer als Töten. Sie hören nicht auf, gegen euch zu kämpfen, bis sie euch von eurer Religion abbringen, wenn sie (es) können. Diejenigen von euch, die sich nun von ihrer Religion abwenden und als Ungläubige sterben, deren Werke sind im Diesseits und Jenseits wertlos. Das sind die Gefährten des Feuers; sie werden darin ewig weilen.\qt@no{217}}%
\qt@newcmd\qurantrans@de@ccxxv{Diejenigen, die glauben, und diejenigen, die ausgewandert sind und sich auf dem Weg Gottes eingesetzt haben, dürfen auf die Barmherzigkeit Gottes hoffen. Und Gott ist voller Vergebung und barmherzig.\qt@no{218}}%
\qt@newcmd\qurantrans@de@ccxxvi{Sie fragen dich nach dem Wein und dem Glücksspiel. Sprich: In ihnen liegt eine große Sünde und auch vielfacher Nutzen für die Menschen. Aber die Sünde in ihnen ist größer als der Nutzen. Und sie fragen dich, was sie spenden sollen. Sprich: Das Entbehrliche. So macht Gott euch die Zeichen deutlich, auf daß ihr nachdenkt\qt@no{219}}%
\qt@newcmd\qurantrans@de@ccxxvii{über das Diesseits und das Jenseits. Und sie fragen dich nach den Waisen. Sprich: Für sie Ordnung schaffen ist besser. Aber wenn ihr den Umgang mit ihnen pflegt, so sind sie eure Brüder. Und Gott weiß den Unheilstifter zu unterscheiden von dem, der Ordnung schafft. Und wenn Gott gewollt hätte, hätte Er euch in Bedrängnis gebracht. Gott ist mächtig und weise.\qt@no{220}}%
\qt@newcmd\qurantrans@de@ccxxviii{Und heiratet nicht polytheistische Frauen, bis sie gläubig geworden sind. Wahrlich, eine gläubige Sklavin ist besser als eine polytheistische Frau, auch wenn sie euch gefallen sollte. Und laßt die Polytheisten nicht zur Heirat zu, bis sie gläubig geworden sind. Wahrlich, ein gläubiger Sklave ist besser als ein Polytheist, auch wenn er euch gefallen sollte. Jene rufen zum Feuer. Gott aber ruft zum Paradies und zur Vergebung mit seiner Erlaubnis. Und Er macht den Menschen seine Zeichen deutlich, auf daß sie es bedenken.\qt@no{221}}%
\qt@newcmd\qurantrans@de@ccxxix{Und sie fragen dich nach der Menstruation. Sprich: Sie ist ein Leiden. So haltet euch von den Frauen während der Menstruation fern und nähert euch ihnen nicht, bis sie wieder rein sind. Wenn sie sich nun gereinigt haben, dann geht zu ihnen, wie Gott es euch befohlen hat. Gott liebt die Bußfertigen, und Er liebt die, die sich reinigen.\qt@no{222}}%
\qt@newcmd\qurantrans@de@ccxxx{Eure Frauen sind für euch ein Saatfeld. Geht zu eurem Saatfeld, wo immer ihr wollt. Und schickt für euch (etwas Gutes) voraus. Und fürchtet Gott und wißt, daß ihr Ihm begegnen werdet. Und verkünde den Gläubigen frohe Botschaft.\qt@no{223}}%
\qt@newcmd\qurantrans@de@ccxxxi{Und macht Gott nicht bei euren Eiden zu einem Hinderungsgrund, Pietät zu üben und gottesfürchtig zu sein und Frieden unter den Menschen zu stiften. Und Gott hört und weiß alles.\qt@no{224}}%
\qt@newcmd\qurantrans@de@ccxxxii{Gott belangt euch nicht wegen unbedachter Rede in euren Eiden. Aber Er belangt euch wegen dessen, was eure Herzen begehen. Und Gott ist voller Vergebung und langmütig.\qt@no{225}}%
\qt@newcmd\qurantrans@de@ccxxxiii{Diejenigen, die schwören, sich ihrer Frauen zu enthalten, haben vier Monate zu warten. Wenn sie es aber zurücknehmen, so ist Gott voller Vergebung und barmherzig.\qt@no{226}}%
\qt@newcmd\qurantrans@de@ccxxxiv{Und wenn sie sich zur Entlassung entschließen - siehe, Gott hört und weiß alles.\qt@no{227}}%
\qt@newcmd\qurantrans@de@ccxxxv{Die entlassenen Frauen haben drei Perioden lang zu warten. Es ist ihnen nicht erlaubt, zu verschweigen, was Gott in ihrem Schoß erschaffen hat, so sie an Gott und den Jüngsten Tag glauben. Ihre Gatten haben eher das Recht, sie während dieser Zeit zurückzunehmen, wenn sie eine Aussöhnung anstreben. Und sie haben Anspruch auf dasgleiche, was ihnen obliegt, und dies auf rechtliche Weise. Die Männer stehen eine Stufe über ihnen. Und Gott ist mächtig und weise.\qt@no{228}}%
\qt@newcmd\qurantrans@de@ccxxxvi{Die Entlassung darf zweimal erfolgen. Dann müssen sie entweder in rechtlicher Weise behalten oder im Guten freigegeben werden. Und es ist euch nicht erlaubt, etwas von dem, was ihr ihnen zukommen ließet, zu nehmen, es sei denn, beide fürchten, die Bestimmungen Gottes nicht einzuhalten. Und wenn ihr fürchtet, daß die beiden die Bestimmungen Gottes nicht einhalten werden, so besteht für sie beide kein Vergehen in bezug auf das, womit sie sich loskauft. Dies sind die Bestimmungen Gottes, übertretet sie nicht. Diejenigen, die Gottes Bestimmungen übertreten, das sind die, die Unrecht tun.\qt@no{229}}%
\qt@newcmd\qurantrans@de@ccxxxvii{Wenn er sie entläßt, so ist sie ihm nicht mehr erlaubt, ehe sie nicht einen anderen Gatten geheiratet hat. Wenn dieser sie entläßt, dann ist es für sie kein Vergehen, wieder zueinander zurückzukehren, wenn sie meinen, die Bestimmungen Gottes einhalten zu können. Dies sind die Bestimmungen Gottes. Er macht sie deutlich für Leute, die Bescheid wissen.\qt@no{230}}%
\qt@newcmd\qurantrans@de@ccxxxviii{Und wenn ihr die Frauen entlaßt und sie das Ende ihrer Frist erreichen, dann behaltet sie in rechtlicher Weise oder gebt sie in rechtlicher Weise frei. Behaltet sie aber nicht aus Schadenslust, um Übertretungen zu begehen. Wer dies tut, der hat sich selbst Unrecht getan. Und nehmt euch nicht die Zeichen Gottes zum Gegenstand des Spottes, und gedenket der Gnade Gottes zu euch und dessen, was Er von dem Buch und der Weisheit auf euch herabgesandt hat, um euch damit zu ermahnen. Und fürchtet Gott und wißt, daß Gott über alle Dinge Bescheid weiß.\qt@no{231}}%
\qt@newcmd\qurantrans@de@ccxxxix{Und wenn ihr die Frauen entlaßt und sie das Ende ihrer Frist erreichen, dann hindert sie nicht, ihre Gatten wieder zu heiraten, falls sie sich in rechtlicher Weise geeinigt haben. Damit wird derjenige von euch ermahnt, der an Gott und den Jüngsten Tag glaubt. Dies ist lauterer und reiner für euch. Und Gott weiß, ihr aber wißt nicht Bescheid.\qt@no{232}}%
\qt@newcmd\qurantrans@de@ccxl{Und die Mütter sollen ihre Kinder zwei volle Jahre stillen. Das gilt für den, der das Stillen bis zum Ende führen will. Und derjenige, dem (das Kind) geboren wurde, hat für ihren Lebensunterhalt und ihre Kleidung in rechtlicher Weise zu sorgen. Von niemandem wird mehr gefordert, als er vermag. Einer Mutter darf nicht wegen ihres Kindes Schaden zugefügt werden, und auch nicht einem Vater wegen seines Kindes. Und der Erbe hat die gleichen Verpflichtungen. Wenn sie sich jedoch in beiderseitigem Einvernehmen und nach Beratung für die Entwöhnung entscheiden, so ist das für sie kein Vergehen. Und wenn ihr eure Kinder stillen lassen wollt, so ist das für euch kein Vergehen, sofern ihr das, was ihr (an Lohn) ausgesetzt habt, in rechtlicher Weise übergebt. Und fürchtet Gott. Und wißt, Gott sieht wohl, was ihr tut.\qt@no{233}}%
\qt@newcmd\qurantrans@de@ccxli{Und wenn welche von euch abberufen werden und Gattinnen zurücklassen, so sollen diese vier Monate und zehn Tage zuwarten. Und wenn sie das Ende ihrer Frist erreicht haben, so besteht für euch kein Vergehen, wenn sie über sich in rechtlicher Weise verfügen. Und Gott hat Kenntnis von dem, was ihr tut.\qt@no{234}}%
\qt@newcmd\qurantrans@de@ccxlii{Und es ist für euch kein Vergehen, wenn ihr Heiratsabsichten gegenüber solchen Frauen andeutet oder bei euch innerlich hegt. Gott weiß, daß ihr an sie denken werdet. Aber vereinbart nichts heimlich mit ihnen, es sei denn, ihr sagt etwas, was sich geziemt. Und entscheidet euch nicht, die Ehe zu schließen, bis die vorgeschriebene Frist zu Ende gegangen ist. Und wißt, daß Gott weiß, was in eurem Innern ist. So nehmt euch vor Ihm in acht. Und wißt, daß Gott voller Vergebung und langmütig ist.\qt@no{235}}%
\qt@newcmd\qurantrans@de@ccxliii{Es ist für euch kein Vergehen, wenn ihr die Frauen entlaßt, solange ihr sie noch nicht berührt oder für sie noch keine Morgengabe ausgesetzt habt. Und sichert ihnen - der Wohlhabende nach seinem Maß und der Unbemittelte nach seinem Maß - eine Versorgung auf rechtliche Weise. (Dies gilt) als Rechtspflicht für die Rechtschaffenen.\qt@no{236}}%
\qt@newcmd\qurantrans@de@ccxliv{Und wenn ihr sie entlaßt, noch ehe ihr sie berührt habt, während ihr für sie eine Morgengabe ausgesetzt habt, so (steht ihnen) die Hälfte von dem (zu), was ihr ausgesetzt habt, es sei denn, sie lassen (etwas davon) nach oder der, unter dessen Obhut die Eheschließung steht, läßt (etwas davon) nach. Und daß ihr (etwas) nachlaßt, entspricht eher der Gottesfurcht. Und vergeßt die Großmut untereinander nicht. Gott sieht wohl, was ihr tut.\qt@no{237}}%
\qt@newcmd\qurantrans@de@ccxlv{Haltet die Gebete ein, und das mittlere Gebet. Und steht vor Gott in demütiger Ergebenheit.\qt@no{238}}%
\qt@newcmd\qurantrans@de@ccxlvi{Und wenn ihr (etwas) befürchtet, (so betet), ob ihr nun zu Fuß seid oder reitet. Und so ihr in Sicherheit seid, dann gedenket Gottes, wie Er euch gelehrt hat, was ihr nicht wußtet.\qt@no{239}}%
\qt@newcmd\qurantrans@de@ccxlvii{Und diejenigen von euch, die abberufen werden und Gattinnen zurücklassen, haben ihren Gattinnen eine Versorgung für ein Jahr zu vermachen, ohne sie (aus dem Haus) auszuweisen. Wenn sie aber von sich aus ausziehen, so besteht für euch kein Vergehen, wenn sie über sich in rechtlicher Weise verfügen. Und Gott ist mächtig und weise.\qt@no{240}}%
\qt@newcmd\qurantrans@de@ccxlviii{Und den entlassenen Frauen steht eine Versorgung in rechtlicher Weise zu. (Das gilt) als Rechtspflicht für die Gottesfürchtigen.\qt@no{241}}%
\qt@newcmd\qurantrans@de@ccxlix{So macht Gott euch seine Zeichen deutlich, auf daß ihr verständig werdet.\qt@no{242}}%
\qt@newcmd\qurantrans@de@ccl{Hast du nicht auf jene geschaut, die zu Tausenden aus ihren Wohnstätten auszogen aus Angst vor dem Tod? Gott sprach zu ihnen: «Sterbet.» Dann machte Er sie wieder lebendig. Gott ist voller Huld gegen die Menschen. Aber die meisten Menschen sind nicht dankbar.\qt@no{243}}%
\qt@newcmd\qurantrans@de@ccli{Und kämpft auf dem Weg Gottes. Und wißt, daß Gott alles hört und weiß.\qt@no{244}}%
\qt@newcmd\qurantrans@de@cclii{Wer ist es, der Gott ein gutes Darlehen leiht? Er wird es ihm vielfach verdoppeln. Und Gott teilt bemessen und auch großzügig zu. Und zu Ihm werdet ihr zurückgebracht.\qt@no{245}}%
\qt@newcmd\qurantrans@de@ccliii{Hast du nicht auf die Vornehmen unter den Kindern Israels nach Mose geschaut, als sie zu einem ihrer Propheten sagten: «Setz uns einen König ein, damit wir auf dem Weg Gottes kämpfen.» Er sagte: «Kann es aber möglich sein, daß ihr, wenn euch vorgeschrieben wird zu kämpfen, doch nicht kämpft?» Sie sagten: «Warum sollten wir denn nicht auf dem Weg Gottes kämpfen, wo wir doch aus unseren Wohnstätten und von unseren Söhnen vertrieben worden sind?» Als ihnen aber vorgeschrieben wurde zu kämpfen, kehrten sie sich ab bis auf wenige von ihnen. Und Gott weiß Bescheid über die, die Unrecht tun.\qt@no{246}}%
\qt@newcmd\qurantrans@de@ccliv{Und ihr Prophet sagte zu ihnen: «Gott hat euch Talut zum König eingesetzt.» Sie sagten: «Wie sollte er die Königsherrschaft über uns erhalten, wo wir doch eher Recht auf die Königsherrschaft haben als er und ihm kein beachtliches Vermögen zuteil wurde?» Er sagte: «Gott hat ihn vor euch auserwählt und ihm darüber hinaus ein größeres Maß an Wissen und Körperstatur gegeben. Und Gott läßt seine Königsherrschaft zukommen, wem Er will. Und Gott umfaßt und weiß alles.\qt@no{247}}%
\qt@newcmd\qurantrans@de@cclv{Und ihr Prophet sagte zu ihnen: «Das Zeichen seiner Königsherrschaft wird sein, daß die Lade, in der eine Ruhe spendende Gegenwart von eurem Herrn ist und ein Rest von dem, was die Sippe Moses und die Sippe Aarons hinterlassen haben, zu euch kommt, getragen von den Engeln. Darin ist für euch ein Zeichen, so ihr gläubig seid.»\qt@no{248}}%
\qt@newcmd\qurantrans@de@cclvi{Und als Taalut mit den Truppen ausrückte, sagte er: «Gott wird euch mit einem Fluß prüfen. Wer daraus trinkt, gehört nicht zu mir, und wer davon nicht kostet, gehört zu mir, ausgenommen, wer nur eine Handvoll schöpft.» Da tranken sie daraus bis auf wenige von ihnen. Und als er und diejenigen, die mit ihm gläubig waren, den Fluß überquert hatten, sagten sie: «Wir haben heute keine Kraft gegen Goliat und seine Truppen.» Da sagten diejenigen, die damit rechneten, daß sie Gott begegnen werden: «Wie manche geringe Schar hat doch schon mit Gottes Erlaubnis eine große Schar besiegt! Und Gott ist mit den Standhaften.»\qt@no{249}}%
\qt@newcmd\qurantrans@de@cclvii{Und als sie sich gegen Goliat und seine Truppen stellten, sagten sie: «Unser Herr, gieße Standhaftigkeit über uns aus, festige unsere Schritte und unterstütze uns gegen die ungläubigen Leute.»\qt@no{250}}%
\qt@newcmd\qurantrans@de@cclviii{Und sie schlugen sie mit Gottes Erlaubnis. Und David tötete Goliat. Und ihm ließ Gott die Königsherrschaft und die Weisheit zukommen, und Er lehrte ihn manches, was Er eben wollte. Und würde Gott nicht die einen Menschen durch die anderen abwehren, so würde die Erde voller Unheil sein. Aber Gott ist voller Huld gegen die Weltenbewohner.\qt@no{251}}%
\qt@newcmd\qurantrans@de@cclix{Das sind die Zeichen Gottes. Wir verlesen sie dir der Wahrheit entsprechend. Und du bist gewiß einer der Gesandten.\qt@no{252}}%
\qt@newcmd\qurantrans@de@cclx{Das sind die Gesandten. Wir haben die einen von ihnen vor den anderen bevorzugt. Unter ihnen sind welche, mit denen Gott gesprochen hat. Einige von ihnen hat Er um Rangstufen erhöht. Und Wir haben Jesus, dem Sohn Marias, die deutlichen Zeichen zukommen lassen und ihn mit dem Geist der Heiligkeit gestärkt. Und wenn Gott gewollt hätte, hätten diejenigen, die nach ihnen lebten, einander nicht bekämpft, nachdem die deutlichen Zeichen zu ihnen gekommen waren. Aber sie sind uneins geworden. Unter ihnen sind solche, die geglaubt haben, und solche, die ungläubig geworden sind. Und wenn Gott gewollt hätte, hätten sie einander nicht bekämpft. Aber Gott tut, was Er will.\qt@no{253}}%
\qt@newcmd\qurantrans@de@cclxi{O ihr, die ihr glaubt, spendet von dem, was Wir euch beschert haben, bevor ein Tag kommt, an dem es weder Kaufgeschäft noch Freundschaft, noch Fürsprache gibt. Die Ungläubigen sind die, die Unrecht tun.\qt@no{254}}%
\qt@newcmd\qurantrans@de@cclxii{Gott, es gibt keinen Gott außer Ihm, dem Lebendigen, dem Beständigen. Nicht überkommt Ihn Schlummer und nicht Schlaf. Ihm gehört, was in den Himmeln und was auf der Erde ist. Wer ist es, der bei Ihm Fürsprache einlegen kann, es sei denn mit seiner Erlaubnis? Er weiß, was vor ihnen und was hinter ihnen liegt, während sie nichts von seinem Wissen erfassen, außer was Er will. Sein Thron umfaßt die Himmel und die Erde, und es fällt Ihm nicht schwer, sie zu bewahren. Er ist der Erhabene, der Majestätische.\qt@no{255}}%
\qt@newcmd\qurantrans@de@cclxiii{Es gibt keinen Zwang in der Religion. Der rechte Wandel unterscheidet sich nunmehr klar vom Irrweg. Wer also die Götzen verleugnet und an Gott glaubt, der hält sich an der festesten Handhabe, bei der es kein Reißen gibt. Und Gott hört und weiß alles.\qt@no{256}}%
\qt@newcmd\qurantrans@de@cclxiv{Gott ist der Freund derer, die glauben; Er führt sie aus den Finsternissen hinaus ins Licht. Diejenigen, die nicht glauben, haben die Götzen zu Freunden; sie führen sie aus dem Licht hinaus in die Finsternisse. Das sind die Gefährten des Feuers, sie werden darin ewig weilen.\qt@no{257}}%
\qt@newcmd\qurantrans@de@cclxv{Hast du nicht auf den geschaut, der mit Abraham über seinen Herrn stritt (aus dem Grund), daß Gott ihm die Königsherrschaft zukommen ließ? Als Abraham sagte: «Mein Herr ist es, der lebendig macht und sterben läßt.» Er sagte: «Ich mache lebendig und lasse sterben.» Abraham sagte: «Gott bringt die Sonne vom Osten her. Bring du sie vom Westen.» Da war der, der ungläubig war, verwirrt. Und Gott leitet die ungerechten Leute nicht recht.\qt@no{258}}%
\qt@newcmd\qurantrans@de@cclxvi{Oder es ist wie mit dem, der an einer Stadt vorbeikam, die über ihren Dächern verödet war. Er sagte: «Wie kann Gott diese wieder lebendig machen, nachdem sie ausgestorben ist?» Da ließ ihn Gott sterben, hundert Jahre, dann erweckte Er ihn. Er sprach: «Wie lange hast du verweilt?» Er sagte: «Verweilt habe ich einen Tag oder einen Teil von einem Tag.» Er sprach: «Nein, du hast hundert Jahre verweilt. Schau auf deine Speise und deinen Trank, sie sind nicht verfault. Und schau auf deinen Esel. Das ist, damit Wir dich zu einem Zeichen für die Menschen machen. Und schau auf die Gebeine, wie Wir sie aufrichten und sie dann mit Fleisch überziehen.» Als (dies) ihm deutlich wurde, sagte er: «Ich weiß nun, daß Gott Macht hat zu allen Dingen.»\qt@no{259}}%
\qt@newcmd\qurantrans@de@cclxvii{Und als Abraham sagte: «Mein Herr, zeig mir, wie Du die Toten wieder lebendig machst.» Er sprach: «Glaubst du denn nicht?» Er sagte: «Doch. Aber mein Herz soll Ruhe finden.» Er sprach: «Dann nimm vier Vögel, richte sie auf dich zu (und schlachte sie). Dann lege auf jeden Berg ein Stück von ihnen, und dann rufe sie. Sie werden zu dir eilends kommen. Und wisse, daß Gott mächtig und weise ist.»\qt@no{260}}%
\qt@newcmd\qurantrans@de@cclxviii{Mit denen, die ihr Vermögen auf dem Weg Gottes spenden, ist es wie mit einem Saatkorn, das sieben Ähren wachsen läßt mit hundert Körnern in jeder Ähre. Gott gibt das Doppelte, wem Er will. Gott umfaßt und weiß alles.\qt@no{261}}%
\qt@newcmd\qurantrans@de@cclxix{Diejenigen, die ihr Vermögen auf dem Weg Gottes spenden und, nachdem sie gespendet haben, nicht auf ihr Verdienst pochen und nicht Ungemach zufügen, haben ihren Lohn bei ihrem Herrn, sie haben nichts zu befürchten, und sie werden nicht traurig sein.\qt@no{262}}%
\qt@newcmd\qurantrans@de@cclxx{Freundliche Worte und Verzeihen sind besser als ein Almosen, dem Ungemach folgt. Gott ist auf niemanden angewiesen und langmütig.\qt@no{263}}%
\qt@newcmd\qurantrans@de@cclxxi{O ihr, die ihr glaubt, vereitelt nicht eure Almosen, indem ihr auf euer Verdienst pocht und Ungemach zufügt, gleich dem, der sein Vermögen spendet, um von den Menschen gesehen zu werden, und nicht an Gott und den Jüngsten Tag glaubt. Mit ihm ist es wie mit einem Felsen, der von Erdreich bedeckt ist. Es trifft ihn ein Platzregen und macht ihn zu einem kahlen Ding. Sie verfügen über nichts von dem, was sie erworben haben. Und Gott leitet die ungläubigen Leute nicht recht.\qt@no{264}}%
\qt@newcmd\qurantrans@de@cclxxii{Mit denen, die ihr Vermögen spenden im Streben nach dem Wohlwollen Gottes und zur Festigung ihrer Seelen, ist es wie mit einem Garten auf einer Anhöhe. Es trifft ihn ein Platzregen, und er bringt den doppelten Ernteertrag. Und wenn ihn kein Platzregen trifft, dann ist es der Tau. Und Gott sieht wohl, was ihr tut.\qt@no{265}}%
\qt@newcmd\qurantrans@de@cclxxiii{Möchte einer von euch einen Garten von Palmen und Weinstöcken haben, unter dem Bäche fließen und in dem er allerlei Früchte hat? Dann trifft ihn das Alter, während er (noch) schwache Nachkommen hat. Nun trifft ihn ein Wirbelsturm mit Feuer, er verbrennt. So macht Gott euch die Zeichen deutlich, damit ihr nachdenkt.\qt@no{266}}%
\qt@newcmd\qurantrans@de@cclxxiv{O ihr, die ihr glaubt, spendet von den köstlichen Dingen, die ihr erworben habt, und von dem, was Wir für euch aus der Erde hervorgebracht haben. Und sucht nicht das Schlechte aus, um davon zu spenden, wo ihr es nicht nehmen würdet, es sei denn, ihr würdet ein Auge zudrücken. Und wißt, daß Gott auf niemanden angewiesen ist und des Lobes würdig.\qt@no{267}}%
\qt@newcmd\qurantrans@de@cclxxv{Der Satan droht euch Armut an und befiehlt euch Schändliches. Gott verheißt euch Vergebung von seiner Seite und Huld. Gott umfaßt und weiß alles.\qt@no{268}}%
\qt@newcmd\qurantrans@de@cclxxvi{Er schenkt die Weisheit, wem Er will. Und wem die Weisheit geschenkt wird, dem wird viel Gutes geschenkt. Jedoch bedenken es nur die Einsichtigen.\qt@no{269}}%
\qt@newcmd\qurantrans@de@cclxxvii{Was ihr an Spenden spendet oder an Gelübden gelobt, Gott weiß es. Diejenigen, die Unrecht tun, haben keine Helfer.\qt@no{270}}%
\qt@newcmd\qurantrans@de@cclxxviii{Wenn ihr die Almosen offen zeigt, so ist es schön. Wenn ihr sie geheimhaltet und den Armen zukommen laßt, so ist es besser für euch, und Er sühnt euch etwas von euren Missetaten. Gott hat Kenntnis von dem, was ihr tut.\qt@no{271}}%
\qt@newcmd\qurantrans@de@cclxxix{Es ist nicht deine Aufgabe, sie rechtzuleiten, sondern Gott leitet recht, wen Er will. Und was ihr an Gutem spendet, es ist zu eurem Vorteil. Und ihr spendet nur in der Suche nach dem Antlitz Gottes. Und was ihr an Gutem spendet, wird euch voll zurückerstattet, und euch wird nicht Unrecht getan.\qt@no{272}}%
\qt@newcmd\qurantrans@de@cclxxx{Die Spenden sind) für die Armen, die auf dem Weg Gottes Behinderung erleiden, so daß sie nicht im Land umherwandern können. Der Törichte hält sie für reich wegen ihrer Zurückhaltung. Du erkennst sie an ihrem Merkmal. Sie betteln die Menschen nicht in aufdringlicher Weise an. Und was ihr an Gutem spendet, Gott weiß es.\qt@no{273}}%
\qt@newcmd\qurantrans@de@cclxxxi{Diejenigen, die ihr Vermögen bei Nacht und Tag, geheim oder offen, spenden, haben ihren Lohn bei ihrem Herrn, sie haben nichts zu befürchten, und sie werden nicht traurig sein.\qt@no{274}}%
\qt@newcmd\qurantrans@de@cclxxxii{Diejenigen, die den Zins verzehren, werden nur so aufstehen, wie der aufsteht, den der Satan packt und verprügelt. Dies, weil sie sagen: Das Verkaufen ist gleich dem Zinsnehmen. Aber Gott hat das Verkaufen erlaubt und das Zinsnehmen verboten. Wer eine Ermahnung von seinem Herrn bekommt und dann aufhört, darf das, was vorher geschah, behalten. Seine Angelegenheit wird Gott überlassen. Diejenigen aber, die es von neuem tun, das sind die Gefährten des Feuers; darin werden sie ewig weilen.\qt@no{275}}%
\qt@newcmd\qurantrans@de@cclxxxiii{Gott vernichtet das Zinsnehmen, und Er verzinst die Almosen. Gott liebt keinen, der sehr ungläubig und sündig ist.\qt@no{276}}%
\qt@newcmd\qurantrans@de@cclxxxiv{Diejenigen, die glauben, die guten Werke tun, das Gebet verrichten, die Abgabe entrichten, haben ihren Lohn bei ihrem Herrn, sie haben nichts zu befürchten, und sie werden nicht traurig sein.\qt@no{277}}%
\qt@newcmd\qurantrans@de@cclxxxv{O ihr, die ihr glaubt, fürchtet Gott und laßt, was künftig an Zinsnehmen anfällt, bleiben, so ihr gläubig seid.\qt@no{278}}%
\qt@newcmd\qurantrans@de@cclxxxvi{Wenn ihr es nicht tut, so erwartet Krieg von Gott und seinem Gesandten. Wenn ihr umkehrt, steht euch euer Kapital zu; so, tut ihr kein Unrecht, und es wird euch kein Unrecht getan.\qt@no{279}}%
\qt@newcmd\qurantrans@de@cclxxxvii{Und wenn (ein Schuldner) in Bedrängnis ist, dann gewährt ihm Aufschub, bis sich (bei ihm) Erleichterung einstellt. Daß ihr (es ihm) aber als Almosen erlaßt, ist besser für euch, so ihr Bescheid wißt.\qt@no{280}}%
\qt@newcmd\qurantrans@de@cclxxxviii{Und hütet euch vor einem Tag, an dem ihr zu Gott zurückgebracht werdet. Dann wird jeder Seele voll zurückerstattet, was sie erworben hat. Und ihnen wird nicht Unrecht getan.\qt@no{281}}%
\qt@newcmd\qurantrans@de@cclxxxix{O ihr, die ihr glaubt, wenn es unter euch um eine Schuld auf eine bestimmte Frist geht, dann schreibt es auf. Ein Schreiber soll (es) in eurem Beisein der Gerechtigkeit gemäß aufschreiben. Kein Schreiber soll sich weigern, zu schreiben, wie Gott ihn gelehrt hat. Er soll schreiben, und der, gegen den das Recht besteht, soll (es) diktieren, und er soll Gott, seinen Herrn, fürchten und nichts davon abziehen. Und wenn derjenige, gegen den das Recht besteht, schwachsinnig oder hilflos ist oder nicht selbst zu diktieren vermag, so soll sein Sachwalter der Gerechtigkeit gemäß diktieren. Und laßt zwei Zeugen aus den Reihen eurer Männer (es) bezeugen. Wenn es aber keine zwei Männer gibt, dann sollen es ein Mann und zwei Frauen sein aus den Reihen der Zeugen, mit denen ihr einverstanden seid, so daß, wenn eine der beiden sich irrt, die eine von ihnen die andere erinnern kann. Die Zeugen sollen sich nicht weigern, wenn sie dazu aufgerufen werden. Und unterlaßt nicht aus Verdruß, es aufzuschreiben, ob es klein oder groß ist, (um es festzulegen) bis zu seiner Frist. Das ist für euch gerechter bei Gott und richtiger für das Zeugnis und bewirkt eher, daß ihr keine Zweifel hegt. Es sei denn, es handelt sich um eine an Ort und Stelle vorhandene Ware, die ihr untereinander aushändigt, dann ist es für euch kein Vergehen, wenn ihr sie nicht aufschreibt. Und nehmt Zeugen, wenn ihr miteinander Kaufgeschäfte abschließt. Kein Schreiber und kein Zeuge soll einem Schaden ausgesetzt werden. Wenn ihr (es) aber tut, so ist das ein Frevel von euch. Und fürchtet Gott. Gott lehrt euch, und Gott weiß alle Dinge.\qt@no{282}}%
\qt@newcmd\qurantrans@de@ccxc{Und wenn ihr auf einer Reise seid und keinen Schreiber findet, so soll ein Pfand genommen werden. Und wenn die einen von euch den anderen etwas anvertrauen, so soll derjenige, dem etwas anvertraut wurde, das ihm Anvertraute zurückgeben, und er soll Gott, seinen Herrn, fürchten. Und verschweigt nicht das Zeugnis. Wer es verschweigt, dessen Herz ist sündig. Gott weiß, was ihr tut.\qt@no{283}}%
\qt@newcmd\qurantrans@de@ccxci{Gott gehört, was in den Himmeln und was auf der Erde ist. Und ob ihr das, was in eurem Inneren ist, offenlegt oder geheimhaltet, Gott rechnet mit euch darüber ab. Er vergibt, wem Er will, und Er peinigt, wen Er will. Und Gott hat Macht zu allen Dingen.\qt@no{284}}%
\qt@newcmd\qurantrans@de@ccxcii{Der Gesandte glaubt an das, was zu ihm von seinem Herrn herabgesandt wurde, und ebenso die Gläubigen. Jeder glaubt an Gott und seine Engel und seine Bücher und seine Gesandten. Wir machen bei keinem seiner Gesandten einen Unterschied. Und sie sagen: «Wir hören, und wir gehorchen. Schenke uns deine Vergebung, unser Herr. Zu Dir führt der Lebensweg.\qt@no{285}}%
\qt@newcmd\qurantrans@de@ccxciii{Gott fordert von niemandem mehr, als er vermag. Ihm gereicht zum Vorteil, was er erworben hat, und ihm gereicht zum Schaden, was er begangen hat. Unser Herr, belange uns nicht, wenn wir vergessen oder sündigen. Unser Herr, lege auf uns keine Last, wie Du sie auf die gelegt hast, die vor uns lebten. Unser Herr, lade uns nichts auf, wozu wir keine Kraft haben. Verzeihe uns, vergib uns und erbarme dich unser. Du bist unser Schutzherr, so unterstütze uns gegen die ungläubigen Leute.»\qt@no{286}}%
\qt@newcmd\qurantrans@de@ccxciv{\basmalahde Alif Lam Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@ccxcv{Gott, es gibt keinen Gott außer Ihm, dem Lebendigen, dem Beständigen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@ccxcvi{Er hat auf dich das Buch mit der Wahrheit herabgesandt als Bestätigung dessen, was vor ihm vorhanden war. Und Er hat die Tora und das Evangelium herabgesandt\qt@no{3}}%
\qt@newcmd\qurantrans@de@ccxcvii{Zuvor als Rechtleitung für die Menschen, und Er hat die Unterscheidungsnorm herabgesandt. Diejenigen, die die Zeichen Gottes verleugnen, erhalten eine harte Pein. Und Gott ist mächtig und übt Rache.\qt@no{4}}%
\qt@newcmd\qurantrans@de@ccxcviii{Vor Gott ist nichts verborgen, weder auf der Erde noch im Himmel.\qt@no{5}}%
\qt@newcmd\qurantrans@de@ccxcix{Er ist es, der euch im Mutterschoß gestaltet, wie Er will. Es gibt keinen Gott außer Ihm, dem Mächtigen, dem Weisen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@ccc{Er ist es, der das Buch auf dich herabgesandt hat. In ihm gibt es eindeutig festgelegte Zeichen - sie sind die Urnorm des Buches - und andere, mehrdeutige. Diejenigen, in deren Herzen Abweichen von der Wahrheit steckt, folgen dem, was in ihm mehrdeutig ist, im Trachten danach, (die Menschen) zu verführen, und im Trachten danach, es (eigener) Deutung zu unterziehen. Um seine Deutung aber weiß niemand außer Gott. Und diejenigen, die im Wissen fest gegründet sind, sagen: «Wir glauben; das eine und das andere ist von unserem Herrn.» Jedoch bedenken (es) nur die Einsichtigen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@ccci{Unser Herr, laß unsere Herzen nicht abweichen, nachdem Du uns rechtgeleitet hast. Und schenke uns von Dir Barmherzigkeit. Du bist ja der Freigebige.\qt@no{8}}%
\qt@newcmd\qurantrans@de@cccii{Unser Herr, Du wirst die Menschen auf einen Tag versammeln, an dem kein Zweifel möglich ist. Gott bricht das Versprechen nicht.\qt@no{9}}%
\qt@newcmd\qurantrans@de@ccciii{Denen, die ungläubig sind, werden weder ihr Vermögen noch ihre Kinder vor Gott etwas nützen. Sie sind der Brennstoff des Feuers.\qt@no{10}}%
\qt@newcmd\qurantrans@de@ccciv{Es ist wie mit den Leuten des Pharao und denen, die vor ihnen lebten. Sie erklärten unsere Zeichen für Lüge, so ergriff sie Gott wegen ihrer Sünden. Und Gott verhängt harte Strafen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@cccv{Sprich zu denen, die ungläubig sind: Ihr werdet besiegt und zur Hölle versammelt werden - welch schlimme Lagerstätte!\qt@no{12}}%
\qt@newcmd\qurantrans@de@cccvi{Ihr hattet ein Zeichen in zwei Gruppen, die aufeinandertrafen: Die eine Gruppe kämpfte auf dem Weg Gottes, die andere war ungläubig. Sie sahen mit eigenen Augen, daß jene zweimal so zahlreich waren wie sie. Aber Gott stärkt mit seiner Unterstützung, wen Er will. Darin ist eine Lehre für die Einsichtigen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@cccvii{Verlockend ist den Menschen gemacht worden die Liebe zu dem, was man begehrt: Frauen, Söhne, ganze Zentner von Gold und Silber, gekennzeichnete Pferde, Vieh und Ackerland. Dies ist Nutznießung des diesseitigen Lebens. Aber bei Gott ist die schöne Heimstatt.\qt@no{14}}%
\qt@newcmd\qurantrans@de@cccviii{Sprich: Soll ich euch etwas kundtun, was besser ist als dieses? Für diejenigen, die gottesfürchtig sind, sind bei ihrem Herrn Gärten, unter denen Bäche fließen und in denen sie ewig weilen werden, und geläuterte Gattinnen und Wohlgefallen von Gott - Gott sieht wohl die Diener -,\qt@no{15}}%
\qt@newcmd\qurantrans@de@cccix{für sie), die sagen: «Unser Herr, wir glauben, so vergib uns unsere Sünden und bewahre uns vor der Pein des Feuers»,\qt@no{16}}%
\qt@newcmd\qurantrans@de@cccx{Die geduldig, wahrhaftig und demütig ergeben sind, die Spenden geben und die in der Morgendämmerung um Vergebung bitten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@cccxi{Gott bezeugt, daß es keinen Gott gibt außer Ihm, ebenso die Engel und diejenigen, die das Wissen besitzen. Er steht für die Gerechtigkeit ein. Es gibt keinen Gott außer Ihm, dem Mächtigen, dem Weisen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@cccxii{Die Religion bei Gott ist der Islam. Diejenigen, denen das Buch zugekommen ist, sind erst uneins geworden, nachdem das Wissen zu ihnen gekommen war, dies aus ungerechter Auflehnung untereinander. Wenn aber jemand die Zeichen Gottes verleugnet - siehe, Gott ist schnell im Abrechnen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@cccxiii{Wenn sie mit dir streiten, dann sprich: Ich ergebe mich Gott völlig, und auch die, die mir folgen. Und sprich zu denen, denen das Buch zugekommen ist, und zu den Ungelehrten: Werdet ihr nun Muslime werden? Wenn sie Muslime werden, folgen sie der Rechtleitung. Wenn sie sich aber abkehren, so obliegt dir nur die Ausrichtung (der Botschaft). Und Gott sieht wohl die Diener.\qt@no{20}}%
\qt@newcmd\qurantrans@de@cccxiv{Denen, die die Zeichen Gottes verleugnen und die Propheten zu Unrecht töten und diejenigen unter den Menschen töten, die die Gerechtigkeit gebieten, verkünde eine schmerzhafte Pein.\qt@no{21}}%
\qt@newcmd\qurantrans@de@cccxv{Das sind die, deren Werke im Diesseits und Jenseits wertlos sind, und sie werden keine Helfer haben.\qt@no{22}}%
\qt@newcmd\qurantrans@de@cccxvi{Hast du nicht auf jene geschaut, denen ein Anteil vom Buch zugekommen ist, wie sie zum Buch Gottes gerufen werden, damit es zwischen ihnen urteilt, und wie dann ein Teil von ihnen den Rücken kehrt und sich abwendet?\qt@no{23}}%
\qt@newcmd\qurantrans@de@cccxvii{Dies, weil sie sagen: «Das Feuer wird uns sicher nur einige gezählte Tage berühren.» Siehe, betört hat sie in ihrer Religion, was sie immer wieder erdichtet haben.\qt@no{24}}%
\qt@newcmd\qurantrans@de@cccxviii{Wie wird es wohl sein, wenn Wir sie auf einen Tag versammeln, an dem kein Zweifel möglich ist, und wenn jeder Seele voll erstattet wird, was sie erworben hat? Und ihnen wird nicht Unrecht getan.\qt@no{25}}%
\qt@newcmd\qurantrans@de@cccxix{Sprich: O Gott, der Du über die Königsherrschaft verfügst, Du gibst die Königsherrschaft, wem Du willst, und Du nimmst die Königsherrschaft, wem Du willst. Du verleihst Macht, wem Du willst, und Du erniedrigst, wen Du willst. In deiner Hand liegt das Gute. Du hast Macht zu allen Dingen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@cccxx{Du läßt die Nacht in den Tag übergehen, und Du läßt den Tag in die Nacht übergehen. Du bringst das Lebendige aus dem Toten, und Du bringst das Tote aus dem Lebendigen hervor, und Du bescherst Unterhalt, wem Du willst, ohne (viel) zu rechnen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@cccxxi{Die Gläubigen sollen sich nicht die Ungläubigen anstelle der Gläubigen zu Freunden nehmen. Wer das tut, hat keine Gemeinschaft mit Gott, es sei denn, ihr hütet euch wirklich vor ihnen. Gott warnt euch vor sich selbst. Und zu Gott führt der Lebensweg.\qt@no{28}}%
\qt@newcmd\qurantrans@de@cccxxii{Sprich: Ob ihr das geheimhaltet, was in eurem Inneren ist, oder es offenlegt, Gott weiß es. Und Er weiß, was in den Himmeln und was auf der Erde ist. Und Gott hat Macht zu allen Dingen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@cccxxiii{An dem Tag, da jede Seele das vorgebracht findet, was sie an Gutem getan hat und was sie an Bösem getan hat. Sie wird wünschen, es läge zwischen ihr und ihm eine weite Entfernung. Und Gott warnt euch vor sich selbst. Und Gott hat Mitleid mit den Dienern.\qt@no{30}}%
\qt@newcmd\qurantrans@de@cccxxiv{Sprich: Wenn ihr Gott liebt, dann folgt mir, so wird Gott euch lieben und euch eure Sünden vergeben. Und Gott ist voller Vergebung und barmherzig.\qt@no{31}}%
\qt@newcmd\qurantrans@de@cccxxv{Sprich: Gehorchet Gott und dem Gesandten. Wenn sie sich abkehren - siehe, Gott liebt die Ungläubigen nicht.\qt@no{32}}%
\qt@newcmd\qurantrans@de@cccxxvi{Gott hat sich Adam, Noach, die Sippe Abrahams und die Sippe Imraans vor den Weltenbewohnern erwählt,\qt@no{33}}%
\qt@newcmd\qurantrans@de@cccxxvii{Eine Nachkommenschaft, von der die einen von den anderen stammen. Und Gott hört und weiß alles.\qt@no{34}}%
\qt@newcmd\qurantrans@de@cccxxviii{Als die Frau Imraans sagte: «Mein Herr, ich gelobe Dir das, was in meinem Leib ist, und weihe es Dir. Nimm es von mir an. Du bist der, der alles hört und weiß.»\qt@no{35}}%
\qt@newcmd\qurantrans@de@cccxxix{Als sie mit ihr niederkam, sagte sie: «Mein Herr, ich habe da ein Mädchen geboren.» - Gott wußte doch besser, was sie geboren hatte, ein männliches Kind ist eben nicht wie ein weibliches. - «Und ich habe sie Maria genannt. Und ich suche bei Dir Zuflucht für sie und ihre Nachkommenschaft vor dem gesteinigten Satan.»\qt@no{36}}%
\qt@newcmd\qurantrans@de@cccxxx{Da nahm sie ihr Herr auf schöne Weise an und ließ sie auf schöne Weise heranwachsen. Er vertraute sie Zakaria an. Sooft Zakaria zu ihr in das Heiligtum trat, fand er bei ihr Lebensunterhalt. Er sagte: «O Maria, woher hast du das?» Sie sagte: «Von Gott. Gott beschert Unterhalt, wem Er will, ohne (viel) zu rechnen.»\qt@no{37}}%
\qt@newcmd\qurantrans@de@cccxxxi{Dort rief Zakaria zu seinem Herrn und sagte: «Mein Herr, schenke mir von Dir her eine gute Nachkommenschaft. Du erhörst ja das Gebet.»\qt@no{38}}%
\qt@newcmd\qurantrans@de@cccxxxii{Da riefen ihm die Engel, während er im Heiligtum stand und betete, zu: «Gott verkündet dir Yahyaa; er wird ein Wort von Gott für wahr halten und wird Herrscher, Asket und Prophet sein, einer von den Rechtschaffenen.»\qt@no{39}}%
\qt@newcmd\qurantrans@de@cccxxxiii{Er sagte: «Mein Herr, wie soll ich einen Knaben haben, wo ich ein hohes Alter erreicht habe und meine Frau unfruchtbar ist?» Er sprach: «So ist es; Gott tut, was Er will.»\qt@no{40}}%
\qt@newcmd\qurantrans@de@cccxxxiv{Er sagte: «Mein Herr, setze mir ein Zeichen.» Er sprach: «Dein Zeichen ist, daß du drei Tage lang zu den Menschen nicht sprechen wirst, außer durch Winken. Und gedenke viel deines Herrn und preise (Ihn) am Abend und am Morgen.»\qt@no{41}}%
\qt@newcmd\qurantrans@de@cccxxxv{Als die Engel sagten: «O Maria, Gott hat dich auserwählt und rein gemacht, und Er hat dich vor den Frauen der Weltenbewohner auserwählt.\qt@no{42}}%
\qt@newcmd\qurantrans@de@cccxxxvi{O Maria, sei deinem Herrn demütig ergeben, wirf dich nieder und verneige dich mit denen, die sich verneigen.»\qt@no{43}}%
\qt@newcmd\qurantrans@de@cccxxxvii{Dies gehört zu den Berichten über das Unsichtbare, die Wir dir offenbaren. Du warst ja nicht bei ihnen, als sie ihre Losstäbe warfen, wer von ihnen Maria betreuen solle. Und du warst nicht bei ihnen, als sie miteinander stritten.\qt@no{44}}%
\qt@newcmd\qurantrans@de@cccxxxviii{Als die Engel sagten: «O Maria, Gott verkündet dir ein Wort von Ihm, dessen Name Christus Jesus, der Sohn Marias, ist; er wird angesehen sein im Diesseits und Jenseits, und einer von denen, die in die Nähe (Gottes) zugelassen werden.\qt@no{45}}%
\qt@newcmd\qurantrans@de@cccxxxix{Er wird zu den Menschen sprechen in der Wiege und als Erwachsener und einer der Rechtschaffenen sein.»\qt@no{46}}%
\qt@newcmd\qurantrans@de@cccxl{Sie sagte: «Mein Herr, wie soll ich ein Kind bekommen, wo mich kein Mensch berührt hat?» Er sprach: «So ist es; Gott schafft, was Er will. Wenn Er eine Sache beschlossen hat, sagt Er zu ihr nur: Sei!, und sie ist.»\qt@no{47}}%
\qt@newcmd\qurantrans@de@cccxli{Und Er wird ihn lehren das Buch, die Weisheit, die Tora und das Evangelium.\qt@no{48}}%
\qt@newcmd\qurantrans@de@cccxlii{Und (Er wird ihn) zu einem Gesandten an die Kinder Israels (machen): «Ich komme zu euch mit einem Zeichen von eurem Herrn: Ich schaffe euch aus Ton etwas wie eine Vogelgestalt, dann blase ich hinein, und es wird zu einem Vogel mit Gottes Erlaubnis; und ich heile Blinde und Aussätzige und mache Tote wieder lebendig mit Gottes Erlaubnis; und ich tue euch kund, was ihr eßt und in euren Häusern aufspeichert. Darin ist für euch ein Zeichen, so ihr gläubig seid.\qt@no{49}}%
\qt@newcmd\qurantrans@de@cccxliii{Und (ich komme), das zu bestätigen, was von der Tora vor mir vorhanden war, und um euch einiges von dem zu erlauben, was euch verboten wurde. So komme ich zu euch mit einem Zeichen von eurem Herrn. Daher fürchtet Gott und gehorchet mir.\qt@no{50}}%
\qt@newcmd\qurantrans@de@cccxliv{Gott ist mein Herr und euer Herr, so dienet Ihm. Das ist ein gerader Weg.»\qt@no{51}}%
\qt@newcmd\qurantrans@de@cccxlv{Als Jesus Unglauben von ihrer Seite spürte, sagte er: «Wer sind meine Helfer (auf dem Weg) zu Gott hin?» Die Jünger sagten: «Wir sind die Helfer Gottes. Wir glauben an Gott. Bezeuge, daß wir gottergeben sind.\qt@no{52}}%
\qt@newcmd\qurantrans@de@cccxlvi{Unser Herr, wir glauben an das, was Du herabgesandt hast, und wir folgen dem Gesandten. So verzeichne uns unter denen, die bezeugen.»\qt@no{53}}%
\qt@newcmd\qurantrans@de@cccxlvii{Sie schmiedeten Ränke, und Gott schmiedete Ränke. Gott ist der beste derer, die Ränke schmieden.\qt@no{54}}%
\qt@newcmd\qurantrans@de@cccxlviii{Als Gott sprach: «O Jesus, Ich werde dich abberufen und zu Mir erheben und dich von denen, die ungläubig sind, rein machen. Und Ich werde diejenigen, die dir folgen, über die, die ungläubig sind, stellen bis zum Tag der Auferstehung. Dann wird zu Mir eure Rückkehr sein, und Ich werde zwischen euch über das urteilen, worüber ihr uneins waret.\qt@no{55}}%
\qt@newcmd\qurantrans@de@cccxlix{Diejenigen, die ungläubig sind, werde Ich mit einer harten Pein peinigen im Diesseits und Jenseits, und sie werden keine Helfer haben.»\qt@no{56}}%
\qt@newcmd\qurantrans@de@cccl{Denjenigen aber, die glauben und die guten Werke tun, wird Er ihren Lohn voll erstatten. Gott liebt die nicht, die Unrecht tun.\qt@no{57}}%
\qt@newcmd\qurantrans@de@cccli{Das verlesen Wir dir von den Zeichen und der weisen Ermahnung.\qt@no{58}}%
\qt@newcmd\qurantrans@de@ccclii{Mit Jesus ist es vor Gott wie mit Adam. Er erschuf ihn aus Erde, dann sagte Er zu ihm: Sei!, und er war.\qt@no{59}}%
\qt@newcmd\qurantrans@de@cccliii{Es ist die Wahrheit von deinem Herrn. Darum sei nicht einer von den Zweiflern.\qt@no{60}}%
\qt@newcmd\qurantrans@de@cccliv{Und wenn man mit dir darüber streitet nach dem, was zu dir an Wissen gekommen ist, dann sprich: Kommt her, laßt uns unsere Söhne und eure Söhne, unsere Frauen und eure Frauen, uns selbst und euch selbst zusammenrufen und dann den Gemeinschaftseid leisten und den Fluch Gottes auf die Lügner herabkommen lassen.\qt@no{61}}%
\qt@newcmd\qurantrans@de@ccclv{Das ist gewiß der wahre Bericht. Und es gibt keinen Gott außer Gott. Gott ist der Mächtige, der Weise.\qt@no{62}}%
\qt@newcmd\qurantrans@de@ccclvi{Wenn sie sich abkehren, so weiß Gott über die Unheilstifter Bescheid.\qt@no{63}}%
\qt@newcmd\qurantrans@de@ccclvii{Sprich: O ihr Leute des Buches, kommt her zu einem zwischen uns und euch gleich angenommenen Wort: daß wir Gott allein dienen und Ihm nichts beigesellen, und daß wir nicht einander zu Herren nehmen neben Gott. Doch wenn sie sich abkehren, dann sagt: «Bezeugt, daß wir gottergeben sind.»\qt@no{64}}%
\qt@newcmd\qurantrans@de@ccclviii{O ihr Leute des Buches, warum streitet ihr über Abraham, wo doch die Tora und das Evangelium erst nach ihm herabgesandt wurden? Habt ihr denn keinen Verstand?\qt@no{65}}%
\qt@newcmd\qurantrans@de@ccclix{Siehe, ihr habt über etwas gestritten, wovon ihr Wissen habt. Warum streitet ihr denn nun über das, wovon ihr kein Wissen habt? Gott weiß, ihr aber wißt nicht Bescheid.\qt@no{66}}%
\qt@newcmd\qurantrans@de@ccclx{Abraham war weder Jude noch Christ, sondern er war Anhänger des reinen Glaubens, ein Gottergebener, und er gehörte nicht zu den Polytheisten.\qt@no{67}}%
\qt@newcmd\qurantrans@de@ccclxi{Diejenigen unter den Menschen, die am ehesten Abraham beanspruchen dürfen, sind die, die ihm gefolgt sind, und dieser Prophet und diejenigen, die glauben. Und Gott ist der Freund der Gläubigen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@ccclxii{Eine Gruppe von den Leuten des Buches möchte euch gern in die Irre führen. Aber sie führen nur sich selbst in die Irre, und sie merken es nicht.\qt@no{69}}%
\qt@newcmd\qurantrans@de@ccclxiii{O ihr Leute des Buches, warum verleugnet ihr die Zeichen Gottes, wo ihr sie selbst bezeugt?\qt@no{70}}%
\qt@newcmd\qurantrans@de@ccclxiv{O ihr Leute des Buches, warum verkleidet ihr die Wahrheit mit dem Falschen und verschweigt die Wahrheit, wo ihr es wißt?\qt@no{71}}%
\qt@newcmd\qurantrans@de@ccclxv{Eine Gruppe von den Leuten des Buches sagt: «Glaubt an das, was auf diejenigen, die glauben, herabgesandt wurde, nur am Anfang des Tages, und verleugnet es an seinem Ende. Vielleicht werden sie umkehren.\qt@no{72}}%
\qt@newcmd\qurantrans@de@ccclxvi{Und glaubt nur denen, die eurer Religion folgen.» Sprich: Die Rechtleitung ist ja die Rechtleitung Gottes. (Fürchtet ihr,) daß jemandem (anderen) das gleiche zukommt, was euch zugekommen ist, oder daß sie mit euch vor eurem Herrn streiten? Sprich: Die Huld liegt in der Hand Gottes, Er läßt sie zukommen, wem Er will. Gott umfaßt und weiß alles.\qt@no{73}}%
\qt@newcmd\qurantrans@de@ccclxvii{Er schenkt seine Barmherzigkeit besonders, wem Er will. Und Gott besitzt große Huld.\qt@no{74}}%
\qt@newcmd\qurantrans@de@ccclxviii{Unter den Leuten des Buches gibt es manch einen, der, wenn du ihm einen Zentner anvertraust, ihn dir zurückgibt, und manch einen, der, wenn du ihm einen (einzigen) Dinar anvertraust, ihn dir nur dann zurückgibt, wenn du ihm dauernd zusetzt. Das rührt daher, daß sie sagen: «Man kann uns in bezug auf die Ungelehrten nicht belangen.» Damit sagen sie über Gott eine Lüge, und sie wissen es.\qt@no{75}}%
\qt@newcmd\qurantrans@de@ccclxix{Nein! Wenn einer seine Verpflichtung erfüllt und gottesfürchtig ist - siehe, Gott liebt die Gottesfürchtigen.\qt@no{76}}%
\qt@newcmd\qurantrans@de@ccclxx{Diejenigen, die den Bund Gottes und ihre Eide für einen geringen Preis verkaufen, haben am Jenseits keinen Anteil. Gott wird sie nicht ansprechen und nicht zu ihnen schauen am Tag der Auferstehung, und sie auch nicht für rein erklären. Und für sie ist eine schmerzhafte Pein bestimmt.\qt@no{77}}%
\qt@newcmd\qurantrans@de@ccclxxi{Unter ihnen gibt es eine Gruppe, die ihre Zungen beim Lesen des Buches verdrehen, damit ihr meint, es gehöre zum Buch, während es nicht zum Buch gehört, und die sagen, es sei von Gott her, während es nicht von Gott kommt. Damit sagen sie gegen Gott eine Lüge aus, und sie wissen es.\qt@no{78}}%
\qt@newcmd\qurantrans@de@ccclxxii{Es steht keinem Menschen zu, daß Gott ihm das Buch, die Urteilskraft und die Prophetie zukommen läßt und daß er dann zu den Menschen sagt: «Seid meine Diener anstelle Gottes.» Vielmehr (wird er sagen): «Seid Gottesgelehrte, da ihr das Buch lehrt und da ihr es erforscht.»\qt@no{79}}%
\qt@newcmd\qurantrans@de@ccclxxiii{Und auch nicht, daß er euch befiehlt, die Engel und die Propheten zu Herren zu nehmen. Kann er euch den Unglauben befehlen, nachdem ihr Gottergebene geworden seid?\qt@no{80}}%
\qt@newcmd\qurantrans@de@ccclxxiv{Und als Gott die Verpflichtung der Propheten entgegennahm: «Was immer Ich euch für ein Buch und eine Weisheit zukommen lasse, wenn danach ein Gesandter zu euch kommt, der bestätigt, was bei euch ist, dann müßt ihr an ihn glauben und ihn unterstützen.» Er sprach: «Erkennt ihr es an und betrachtet ihr euch Mir gegenüber daran gebunden?» Sie sagten: «Wir erkennen es an.» Er sprach: «So bezeugt es, und Ich gehöre mit euch zu den Zeugen.»\qt@no{81}}%
\qt@newcmd\qurantrans@de@ccclxxv{Diejenigen, die sich danach abkehren, das sind die Frevler.\qt@no{82}}%
\qt@newcmd\qurantrans@de@ccclxxvi{Suchen sie sich etwa eine andere Religion als die Religion Gottes, wo Ihm ergeben ist, was in den Himmeln und auf der Erde ist, ob freiwillig oder widerwillig, und wo sie (alle) zu Ihm zurückgebracht werden?\qt@no{83}}%
\qt@newcmd\qurantrans@de@ccclxxvii{Sprich: Wir glauben an Gott und an das, was auf uns herabgesandt wurde, und an das, was herabgesandt wurde auf Abraham, Ismael, Isaak, Jakob und die Stämme, und an das, was Mose und Jesus und den Propheten von ihrem Herrn zugekommen ist. Wir machen bei keinem von ihnen einen Unterschied. Und wir sind Ihm ergeben.\qt@no{84}}%
\qt@newcmd\qurantrans@de@ccclxxviii{Wer eine andere Religion als den Islam sucht, von dem wird es nicht angenommen werden. Und im Jenseits gehört er zu den Verlierern.\qt@no{85}}%
\qt@newcmd\qurantrans@de@ccclxxix{Wie sollte Gott Leute rechtleiten, die ungläubig geworden sind, nachdem sie gläubig waren und bezeugt haben, daß der Gesandte wahrhaftig ist, und nachdem die deutlichen Zeichen zu ihnen gekommen sind? Gott leitet die ungerechten Leute nicht recht.\qt@no{86}}%
\qt@newcmd\qurantrans@de@ccclxxx{Die Vergeltung für sie ist, daß der Fluch Gottes und der Engel und der Menschen allesamt über sie kommt.\qt@no{87}}%
\qt@newcmd\qurantrans@de@ccclxxxi{Sie werden darin ewig weilen. Ihnen wird die Pein nicht erleichtert, und ihnen wird kein Aufschub gewährt,\qt@no{88}}%
\qt@newcmd\qurantrans@de@ccclxxxii{Außer denen, die danach umkehren und Besserung zeigen. Denn Gott ist voller Vergebung und barmherzig.\qt@no{89}}%
\qt@newcmd\qurantrans@de@ccclxxxiii{Von denen, die, nachdem sie gläubig waren, ungläubig werden und an Unglauben zunehmen, wird ihre Reue nicht angenommen werden. Das sind die, die irregehen.\qt@no{90}}%
\qt@newcmd\qurantrans@de@ccclxxxiv{Von denen, die ungläubig geworden sind und als Ungläubige sterben - nicht die Erde voll Gold würde von einem von ihnen angenommen, auch wenn er sich damit loskaufen wollte. Für sie ist eine schmerzhafte Pein bestimmt, und sie werden keine Helfer haben.\qt@no{91}}%
\qt@newcmd\qurantrans@de@ccclxxxv{Ihr werdet die (wahre) Frömmigkeit nicht erlangen, bis ihr von dem spendet, was ihr liebt. Und was immer ihr spendet, Gott weiß es.\qt@no{92}}%
\qt@newcmd\qurantrans@de@ccclxxxvi{Alle Speisen waren den Kindern Israels erlaubt, außer dem, was Israel sich selbst verboten hat, bevor die Tora herabgesandt wurde. Sprich: Bringt die Tora her und lest sie vor, so ihr die Wahrheit sagt.\qt@no{93}}%
\qt@newcmd\qurantrans@de@ccclxxxvii{Diejenigen, die nach diesem gegen Gott Lügen erdichten, das sind die, die Unrecht tun.\qt@no{94}}%
\qt@newcmd\qurantrans@de@ccclxxxviii{Sprich: Gott sagt die Wahrheit. So folgt der Glaubensrichtung Abrahams, als Anhänger des reinen Glaubens, und er gehörte nicht zu den Polytheisten.\qt@no{95}}%
\qt@newcmd\qurantrans@de@ccclxxxix{Das erste Haus, das für die Menschen errichtet wurde, ist gewiß dasjenige in Bakka; voller Segen ist es und Rechtleitung für die Weltenbewohner.\qt@no{96}}%
\qt@newcmd\qurantrans@de@cccxc{In ihm sind deutliche Zeichen. Es ist die Stätte Abrahams, und wer es betritt, ist in Sicherheit. Und Gott hat den Menschen die Pflicht zur Wallfahrt nach dem Haus auferlegt, allen, die dazu eine Möglichkeit finden. Und wenn einer ungläubig ist, so ist Gott auf die Weltenbewohner nicht angewiesen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@cccxci{Sprich: O ihr Leute des Buches, warum verleugnet ihr die Zeichen Gottes? Gott ist Zeuge über das, was ihr tut.\qt@no{98}}%
\qt@newcmd\qurantrans@de@cccxcii{Sprich: O ihr Leute des Buches, warum weist ihr den, der glaubt, vom Weg Gottes ab, indem ihr euch ihn krumm wünscht, wo ihr doch Zeugen seid? Und Gott läßt nicht unbeachtet, was ihr tut.\qt@no{99}}%
\qt@newcmd\qurantrans@de@cccxciii{O ihr, die ihr glaubt, wenn ihr einer Gruppe derer gehorcht, denen das Buch zugekommen ist, werden sie euch, nachdem ihr gläubig geworden seid, wieder zu Ungläubigen machen.\qt@no{100}}%
\qt@newcmd\qurantrans@de@cccxciv{Wie könnt ihr ungläubig werden, wo euch die Zeichen Gottes verlesen werden und sein Gesandter unter euch ist? Wer an Gott festhält, wird zu einem geraden Weg geleitet.\qt@no{101}}%
\qt@newcmd\qurantrans@de@cccxcv{O ihr, die ihr glaubt, fürchtet Gott, wie Er richtig gefürchtet werden soll, und sterbt nicht anders denn als Gottergebene.\qt@no{102}}%
\qt@newcmd\qurantrans@de@cccxcvi{Und haltet allesamt am Seil Gottes fest und spaltet euch nicht. Und gedenket der Gnade Gottes zu euch, als ihr Feinde waret und Er Vertrautheit zwischen euren Herzen stiftete, so daß ihr durch seine Gnade Brüder wurdet; und als ihr euch am Rande einer Feuergrube befandet und Er euch davor rettete. So macht euch Gott seine Zeichen deutlich, auf daß ihr der Rechtleitung folgt.\qt@no{103}}%
\qt@newcmd\qurantrans@de@cccxcvii{Aus euch soll eine Gemeinschaft (von Gläubigen) entstehen, die zum Guten aufrufen, das Rechte gebieten und das Verwerfliche verbieten. Das sind die, denen es wohl ergeht.\qt@no{104}}%
\qt@newcmd\qurantrans@de@cccxcviii{Und seid nicht wie diejenigen, die sich gespalten haben und uneins geworden sind, nachdem die deutlichen Zeichen zu ihnen gekommen waren. Bestimmt ist für sie eine gewaltige Pein,\qt@no{105}}%
\qt@newcmd\qurantrans@de@cccxcix{Am Tag, da einige Gesichter weiß und andere Gesichter schwarz sein werden. Zu denen, deren Gesichter schwarz sein werden, (wird gesprochen): «Wieso seid ihr ungläubig geworden, nachdem ihr gläubig waret? Kostet nun die Pein dafür, daß ihr ungläubig waret.»\qt@no{106}}%
\qt@newcmd\qurantrans@de@cd{Diejenigen aber, deren Gesichter weiß sein werden, befinden sich in der Barmherzigkeit Gottes; sie werden darin ewig weilen.\qt@no{107}}%
\qt@newcmd\qurantrans@de@cdi{Das sind die Zeichen Gottes. Wir verlesen sie dir der Wahrheit entsprechend. Und Gott will kein Unrecht für die Weltenbewohner.\qt@no{108}}%
\qt@newcmd\qurantrans@de@cdii{Gott gehört, was in den Himmeln und was auf der Erde ist. Und zu Ihm werden die Angelegenheiten zurückgebracht.\qt@no{109}}%
\qt@newcmd\qurantrans@de@cdiii{Ihr seid die beste Gemeinschaft, die je unter den Menschen hervorgebracht worden ist. Ihr gebietet das Rechte und verbietet das Verwerfliche und glaubt an Gott. Würden die Leute des Buches glauben, es wäre besser für sie. Unter ihnen gibt es Gläubige, aber die meisten von ihnen sind Frevler.\qt@no{110}}%
\qt@newcmd\qurantrans@de@cdiv{Sie werden euch keinen Schaden, nur geringes Leid zufügen. Und wenn sie gegen euch kämpfen, werden sie euch den Rücken kehren. Und dann werden sie keine Unterstützung erfahren.\qt@no{111}}%
\qt@newcmd\qurantrans@de@cdv{Erniedrigung überdeckt sie, wo immer sie angetroffen werden, es sei denn, sie stehen unter dem Schutz einer Verbindung mit Gott und einer Verbindung mit Menschen. Und sie ziehen sich den Zorn Gottes zu. Und Elend überdeckt sie. Dies dafür, daß sie immer wieder die Zeichen Gottes verleugneten und die Propheten zu Unrecht töteten; dies dafür, daß sie ungehorsam waren und immer wieder Übertretungen begingen.\qt@no{112}}%
\qt@newcmd\qurantrans@de@cdvi{Sie sind nicht (alle) gleich. Unter den Leuten des Buches gibt es eine aufrechte Gemeinschaft. Sie verlesen die Zeichen Gottes zu (verschiedenen) Nachtzeiten, während sie sich niederwerfen.\qt@no{113}}%
\qt@newcmd\qurantrans@de@cdvii{Sie glauben an Gott und an den Jüngsten Tag. Sie gebieten das Rechte und verbieten das Verwerfliche und eilen zu den guten Dingen um die Wette. Sie gehören zu den Rechtschaffenen.\qt@no{114}}%
\qt@newcmd\qurantrans@de@cdviii{Was immer sie an Gutem tun, sie werden dafür nicht Undank ernten. Gott weiß über die Gottesfürchtigen Bescheid.\qt@no{115}}%
\qt@newcmd\qurantrans@de@cdix{Denen, die ungläubig sind, werden weder ihr Vermögen noch ihre Kinder vor Gott etwas nützen. Das sind die Gefährten des Feuers; sie werden darin ewig weilen.\qt@no{116}}%
\qt@newcmd\qurantrans@de@cdx{Mit dem, was sie in diesem irdischen Leben ausgeben, ist es wie mit einem eisigen Wind, der das Saatfeld von Leuten traf, die sich selbst Unrecht getan hatten, und es vernichtete. Und nicht Gott hat ihnen Unrecht getan, sondern sie tun sich selbst Unrecht.\qt@no{117}}%
\qt@newcmd\qurantrans@de@cdxi{O ihr, die ihr glaubt, nehmt euch keine Vertrauten unter denen, die nicht zu euch gehören. Sie werden euch kein Unheil ersparen. Sie möchten gern, ihr würdet in Bedrängnis geraten. Der Haß hat sich aus ihrem Munde kundgetan, und das, was ihre Brust verbirgt, ist schlimmer. Wir haben euch die Zeichen deutlich gemacht, so ihr verständig seid.\qt@no{118}}%
\qt@newcmd\qurantrans@de@cdxii{Seht, ihr liebt sie, sie aber lieben euch nicht. Ihr glaubt an das gesamte Buch. Wenn sie euch treffen, sagen sie: «Wir glauben.» Wenn sie allein sind, beißen sie sich gegen euch die Fingerspitzen vor Groll. Sprich: Sterbt an eurem Groll. Gott weiß über das innere Geheimnis Bescheid.\qt@no{119}}%
\qt@newcmd\qurantrans@de@cdxiii{Wenn euch Gutes widerfährt, tut es ihnen leid, und wenn euch Schlimmes trifft, freuen sie sich darüber. Wenn ihr euch geduldig und gottesfürchtig zeigt, wird ihre List euch nichts schaden. Gott umgreift, was sie tun.\qt@no{120}}%
\qt@newcmd\qurantrans@de@cdxiv{Und als du von deinen Angehörigen in der Frühe weggingst, um die Gläubigen in Stellungen zum Kampf einzuweisen. Und Gott hört und weiß alles.\qt@no{121}}%
\qt@newcmd\qurantrans@de@cdxv{Als zwei Gruppen von euch im Begriff waren, den Mut zu verlieren, wo doch Gott ihr Sachwalter war. Auf Gott sollen die Gläubigen vertrauen.\qt@no{122}}%
\qt@newcmd\qurantrans@de@cdxvi{Gott hat euch doch in Badr unterstützt, als ihr unterlegen waret. So fürchtet Gott, auf daß ihr dankbar werdet.\qt@no{123}}%
\qt@newcmd\qurantrans@de@cdxvii{Als du zu den Gläubigen sagtest: «Genügt es euch denn nicht, daß euer Herr euch mit dreitausend herabgesandten Engeln beisteht?\qt@no{124}}%
\qt@newcmd\qurantrans@de@cdxviii{Ja, wenn ihr standhaft und gottesfürchtig seid und sie sogleich gegen euch vorrücken, steht euch euer Herr bei mit fünftausend stürmenden Engeln.»\qt@no{125}}%
\qt@newcmd\qurantrans@de@cdxix{Und Gott hat es nur deswegen gemacht, damit es für euch eine Frohbotschaft sei und damit eure Herzen dadurch Ruhe finden - der Sieg kommt nur von Gott, dem Mächtigen, dem Weisen -,\qt@no{126}}%
\qt@newcmd\qurantrans@de@cdxx{Dies, damit Er einen Teil derer, die ungläubig sind, ausmerze oder sie niederwerfe, so daß sie enttäuscht zurückkehren\qt@no{127}}%
\qt@newcmd\qurantrans@de@cdxxi{- dir steht in dieser Angelegenheit keine Entscheidung zu -, oder damit Er sich ihnen wieder zuwende oder sie peinige, denn sie sind Leute, die Unrecht tun.\qt@no{128}}%
\qt@newcmd\qurantrans@de@cdxxii{Gott gehört, was in den Himmeln und was auf der Erde ist. Er vergibt, wem Er will, und Er peinigt, wen Er will. Und Gott ist voller Vergebung und barmherzig.\qt@no{129}}%
\qt@newcmd\qurantrans@de@cdxxiii{O ihr, die ihr glaubt, verzehrt nicht den Zins in mehrfach verdoppelten Beträgen und fürchtet Gott, auf daß es euch wohl ergehe.\qt@no{130}}%
\qt@newcmd\qurantrans@de@cdxxiv{Und hütet euch vor dem Feuer, das für die Ungläubigen bereitet ist.\qt@no{131}}%
\qt@newcmd\qurantrans@de@cdxxv{Und gehorchet Gott und dem Gesandten, auf daß ihr Erbarmen findet.\qt@no{132}}%
\qt@newcmd\qurantrans@de@cdxxvi{Wetteifert nach einer Vergebung von eurem Herrn und zu einem Garten, der so breit ist wie die Himmel und die Erde, der für die Gottesfürchtigen bereitet ist,\qt@no{133}}%
\qt@newcmd\qurantrans@de@cdxxvii{Die in guten und schlechten Tagen spenden, ihren Groll unterdrücken und den Menschen verzeihen - Gott liebt die Rechtschaffenen -,\qt@no{134}}%
\qt@newcmd\qurantrans@de@cdxxviii{Und die, wenn sie etwas Schändliches begangen oder sich selbst Unrecht getan haben, Gottes gedenken und um Vergebung für ihre Sünden bitten - und wer vergibt die Sünden außer Gott? - und auf dem, was sie getan haben, nicht beharren, wo sie es doch wissen.\qt@no{135}}%
\qt@newcmd\qurantrans@de@cdxxix{Deren Lohn ist Vergebung von ihrem Herrn und Gärten, unter denen Bäche fließen; darin werden sie ewig weilen. Vorzüglich ist doch der Lohn derer, die (gut) handeln.\qt@no{136}}%
\qt@newcmd\qurantrans@de@cdxxx{Schon vor euch wurde (mit den Menschen) beispielhaft verfahren. Geht auf der Erde umher und schaut, wie das Ende derer war, die (die Botschaft) für Lüge erklärt haben.\qt@no{137}}%
\qt@newcmd\qurantrans@de@cdxxxi{Dies ist eine Darlegung für die Menschen und eine Rechtleitung und eine Ermahnung für die Gottesfürchtigen.\qt@no{138}}%
\qt@newcmd\qurantrans@de@cdxxxii{Und erlahmt nicht und werdet nicht traurig, wo ihr doch die Oberhand haben werdet, so ihr gläubig seid.\qt@no{139}}%
\qt@newcmd\qurantrans@de@cdxxxiii{Wenn ihr Wunden erlitten habt, so haben die (feindlichen) Leute ähnliche Wunden erlitten. Solche Tage teilen Wir den Menschen abwechselnd zu. (Dies geschieht) auch, damit Gott diejenigen in Erfahrung bringe, die glauben, und sich aus euren Reihen Zeugen nehme - und Gott liebt die nicht, die Unrecht tun -,\qt@no{140}}%
\qt@newcmd\qurantrans@de@cdxxxiv{Und damit Gott diejenigen, die glauben, läutere und die Ungläubigen vernichte,\qt@no{141}}%
\qt@newcmd\qurantrans@de@cdxxxv{- oder meint ihr, daß ihr ins Paradies eingehen werdet, noch ehe Gott in Erfahrung gebracht hat, wer von ihnen sich eingesetzt hat? - und damit Er in Erfahrung bringe, wer die Standhaften sind.\qt@no{142}}%
\qt@newcmd\qurantrans@de@cdxxxvi{Und ihr pflegtet euch den Tod zu wünschen, bevor ihr ihm begegnet seid. Nun habt ihr ihn gesehen, indem ihr zuschautet.\qt@no{143}}%
\qt@newcmd\qurantrans@de@cdxxxvii{Muhammad ist nur ein Gesandter. Vor ihm sind etliche Gesandte dahingegangen. Werdet ihr denn, wenn er stirbt oder getötet wird, auf euren Fersen kehrtmachen? Wer auf seinen Fersen kehrtmacht, wird Gott nichts schaden können. Und Gott wird (es) den Dankbaren vergelten.\qt@no{144}}%
\qt@newcmd\qurantrans@de@cdxxxviii{Und niemand kann sterben außer mit der Erlaubnis Gottes gemäß einer Schrift mit festgelegter Frist. Und wer die Belohnung des Diesseits will, dem geben Wir etwas von ihm. Und wer die Belohnung des Jenseits will, dem geben Wir etwas von ihm. Und Wir werden (es) den Dankbaren vergelten.\qt@no{145}}%
\qt@newcmd\qurantrans@de@cdxxxix{Und so manchen Propheten gab es, mit dem viele Scharen gekämpft haben. Sie erlahmten nicht wegen dessen, was sie auf dem Weg Gottes traf, und sie wurden nicht schwach, und sie gaben nicht nach. Und Gott liebt die Standhaften.\qt@no{146}}%
\qt@newcmd\qurantrans@de@cdxl{Und nichts anderes war ihre Rede, als daß sie sagten: «Unser Herr, vergib uns unsere Schuld und unsere Maßlosigkeit in unserem Anliegen. Festige unsere Schritte und unterstütze uns gegen die ungläubigen Leute.»\qt@no{147}}%
\qt@newcmd\qurantrans@de@cdxli{So gab ihnen Gott den Lohn des Diesseits und den schönen Lohn des Jenseits. Und Gott liebt die Rechtschaffenen.\qt@no{148}}%
\qt@newcmd\qurantrans@de@cdxlii{O ihr, die ihr glaubt, wenn ihr denen gehorcht, die nicht glauben, lassen sie euch auf den Fersen kehrtmachen und als Verlierer zurückkehren.\qt@no{149}}%
\qt@newcmd\qurantrans@de@cdxliii{Nein, Gott ist euer Schutzherr, und Er ist der beste Helfer.\qt@no{150}}%
\qt@newcmd\qurantrans@de@cdxliv{Wir werden den Herzen derer, die ungläubig sind, Schrecken einjagen dafür, daß sie Gott solche beigesellen, für die Er keine Ermächtigung herabgesandt hat. Ihre Heimstätte ist das Feuer. Schlimm ist die Bleibe derer, die Unrecht tun.\qt@no{151}}%
\qt@newcmd\qurantrans@de@cdxlv{Und Gott hat euch sein Versprechen wahr gemacht, als ihr sie mit seiner Erlaubnis vernichtend schlüget, bis ihr verzagtet, miteinander über die Angelegenheit strittet und ungehorsam wäret, nachdem Er euch hatte sehen lassen, was ihr liebt - unter euch gibt es welche, die das Diesseits wollen, und unter euch gibt es welche, die das Jenseits wollen; dann wies Er euch von ihnen ab, um euch zu prüfen. Und Er hat euch nunmehr verziehen. Gott ist voller Huld gegen die Gläubigen.\qt@no{152}}%
\qt@newcmd\qurantrans@de@cdxlvi{Als ihr wegliefet, ohne euch nach jemandem umzuwenden, während der Gesandte euch weit hinten zurief. Da belohnte Er euch mit Kummer für (den dem Propheten gemachten) Kummer, damit ihr nicht traurig seid über das, was euch entgangen war, und auch nicht über das, was euch getroffen hatte. Und Gott hat Kenntnis von dem, was ihr tut.\qt@no{153}}%
\qt@newcmd\qurantrans@de@cdxlvii{Dann, nach dem Kummer, sandte Er auf euch Sicherheit herab, einen Schlaf, der eine Gruppe von euch überkam, während eine (andere) Gruppe sich um sich selbst kümmerte, indem sie von Gott etwas anderes als die Wahrheit dachte, wie in der Zeit der Unwissenheit gedacht wurde. Sie sagten: «Haben wir bei der Angelegenheit etwas zu entscheiden?» Sprich: Gott allein entscheidet in der ganzen Angelegenheit. Sie halten in ihrem Inneren geheim, was sie dir nicht offenlegen. Sie sagen: «Wenn wir bei der Angelegenheit etwas zu entscheiden gehabt hätten, wären wir nicht hier getötet worden.» Sprich: Auch wenn ihr in euren Häusern geblieben wäret, so wären diejenigen, für die es bestimmt war, getötet zu werden, dennoch zu ihren Sterbestätten hinausgezogen. (Dies geschieht), damit Gott prüft, was in eurer Brust ist, und läutert, was in euren Herzen ist. Und Gott weiß über das innere Geheimnis Bescheid.\qt@no{154}}%
\qt@newcmd\qurantrans@de@cdxlviii{Diejenigen von euch, die sich abkehrten am Tag, da die beiden Scharen aufeinandertrafen, die hat der Satan straucheln lassen wegen etwas von dem, was sie erworben haben. Gott hat ihnen nunmehr verziehen. Gott ist voller Vergebung und langmütig.\qt@no{155}}%
\qt@newcmd\qurantrans@de@cdxlix{O ihr, die ihr glaubt, seid nicht wie diejenigen, die nicht glauben und die von ihren Brüdern, wenn diese auf der Erde umherzogen oder sich auf einem Feldzug befanden, sagen: «Wären sie bei uns gewesen, wären sie nicht gestorben und nicht getötet worden.» Gott will dies zu einem Grund zu (tiefem) Bedauern in ihren Herzen machen. Und Gott macht lebendig und läßt sterben. Und Gott sieht wohl, was ihr tut.\qt@no{156}}%
\qt@newcmd\qurantrans@de@cdl{Und wenn ihr auf dem Weg Gottes getötet werdet oder sterbt, so ist Vergebung und Barmherzigkeit von Gott besser als das, was sie zusammentragen.\qt@no{157}}%
\qt@newcmd\qurantrans@de@cdli{Und wenn ihr sterbt oder getötet werdet, so werdet ihr gewiß zu Gott versammelt werden.\qt@no{158}}%
\qt@newcmd\qurantrans@de@cdlii{Es ist um der Barmherzigkeit Gottes willen, daß du ihnen gegenüber umgänglich warst. Wärest du grob und hartherzig gewesen, wären sie rings um dich fortgelaufen. So verzeihe ihnen und bitte für sie um Vergebung und ziehe sie zu Rate in den Angelegenheiten. Und wenn du dich entschlossen hast, dann vertraue auf Gott. Gott liebt ja die, die vertrauen.\qt@no{159}}%
\qt@newcmd\qurantrans@de@cdliii{Wenn Gott euch unterstützt, dann kann niemand euch besiegen. Und wenn Er euch im Stich läßt, wer ist es, der euch daraufhin unterstützen könnte? Auf Gott sollen also die Gläubigen vertrauen.\qt@no{160}}%
\qt@newcmd\qurantrans@de@cdliv{Es steht einem Propheten nicht zu, zu veruntreuen. Wer veruntreut, wird das, was er veruntreut hat, am Tag der Auferstehung beibringen. Dann wird jeder Seele voll erstattet, was sie erworben hat. Und ihnen wird nicht Unrecht getan.\qt@no{161}}%
\qt@newcmd\qurantrans@de@cdlv{Ist denn derjenige, der dem Wohlgefallen Gottes folgt, wie der, der sich den Groll Gottes zuzieht und dessen Heimstätte die Hölle ist? Welch schlimmes Ende!\qt@no{162}}%
\qt@newcmd\qurantrans@de@cdlvi{Sie nehmen verschiedene Rangstufen bei Gott ein. Und Gott sieht wohl, was sie tun.\qt@no{163}}%
\qt@newcmd\qurantrans@de@cdlvii{Gott hat den Gläubigen eine Wohltat erwiesen, als Er unter ihnen einen Gesandten aus ihrer Mitte hat erstehen lassen, der ihnen seine Zeichen verliest, sie läutert und sie das Buch und die Weisheit lehrt. Sie befanden sich ja vorher in einem offenkundigen Irrtum.\qt@no{164}}%
\qt@newcmd\qurantrans@de@cdlviii{Wie konntet ihr, als euch ein Unglück traf, das ihr (den Feinden) doppelt so arg zugefügt hattet, sagen: «Woher kommt das?» Sprich: Es kommt von euch selbst. Gott hat Macht zu allen Dingen.\qt@no{165}}%
\qt@newcmd\qurantrans@de@cdlix{Und was euch traf an dem Tag, da die beiden Scharen aufeinandertrafen, das geschah mit der Erlaubnis Gottes, und damit Er die Gläubigen in Erfahrung bringe,\qt@no{166}}%
\qt@newcmd\qurantrans@de@cdlx{Und auch die in Erfahrung bringe, die heucheln. Und es wurde zu ihnen gesagt: «Kommt her, kämpft auf dem Weg Gottes oder wehrt die Feinde ab.» Sie sagten: «Wenn wir wüßten, daß es einen Kampf geben würde, würden wir euch sicherlich folgen.» An jenem Tag waren sie dem Unglauben näher als dem Glauben, sie sagten ja mit ihrem Munde, was nicht in ihrem Herzen war. Und Gott weiß besser, was sie verschweigen.\qt@no{167}}%
\qt@newcmd\qurantrans@de@cdlxi{Das sind diejenigen, die, während sie selbst zurückblieben, von ihren Brüdern sagten: «Hätten sie uns gehorcht, wären sie nicht getötet worden.» Sprich: Wehrt doch den Tod von euch ab, so ihr die Wahrheit sagt.\qt@no{168}}%
\qt@newcmd\qurantrans@de@cdlxii{Halte diejenigen, die auf dem Weg Gottes getötet wurden, nicht für tot. Sie sind vielmehr lebendig bei ihrem Herrn, und sie werden versorgt,\qt@no{169}}%
\qt@newcmd\qurantrans@de@cdlxiii{Und sie freuen sich dabei über das, was Gott ihnen von seiner Huld zukommen ließ. Und sie erwarten die, die hinter ihnen nachgekommen sind und sie noch nicht eingeholt haben, voll Freude darüber, daß auch sie nichts zu befürchten haben und nicht traurig sein werden.\qt@no{170}}%
\qt@newcmd\qurantrans@de@cdlxiv{Sie empfangen mit Freude Gnade und Huld von Gott und (freuen sich) darüber, daß Gott den Lohn der Gläubigen nicht verlorengehen läßt.\qt@no{171}}%
\qt@newcmd\qurantrans@de@cdlxv{Für diejenigen, die auf Gott und den Gesandten gehört haben, nachdem sie die Wunde erlitten hatten, - für diejenigen von ihnen, die rechtschaffen und gottesfürchtig waren, ist ein großartiger Lohn bestimmt.\qt@no{172}}%
\qt@newcmd\qurantrans@de@cdlxvi{Zu ihnen haben die Menschen gesagt: «Die Menschen haben sich gegen euch versammelt, daher fürchtet euch vor ihnen.» Aber das hat ihren Glauben nur verstärkt, und sie sagten: «Gott genügt uns. Welch vorzüglicher Sachwalter!»\qt@no{173}}%
\qt@newcmd\qurantrans@de@cdlxvii{Sie kehrten mit Gnade und Huld von Gott zurück, nichts Böses berührte sie, und sie folgten dem Wohlgefallen Gottes. Und Gott besitzt große Huld.\qt@no{174}}%
\qt@newcmd\qurantrans@de@cdlxviii{Das ist der Satan. Er will (euch) vor seinen Freunden angst machen. Habt keine Angst vor ihnen, Mich sollt ihr fürchten, so ihr gläubig seid.\qt@no{175}}%
\qt@newcmd\qurantrans@de@cdlxix{Laß dich nicht durch die betrüben, die im Unglauben miteinander wetteifern. Sie können Gott nicht schaden. Gott will für sie im Jenseits keinen Anteil bereithalten. Und bestimmt ist für sie eine gewaltige Pein.\qt@no{176}}%
\qt@newcmd\qurantrans@de@cdlxx{Diejenigen, die den Unglauben für den Glauben erkauft haben, können Gott nichts schaden. Und bestimmt ist für sie eine schmerzhafte Pein.\qt@no{177}}%
\qt@newcmd\qurantrans@de@cdlxxi{Und diejenigen, die ungläubig sind, sollen nicht meinen, es sei besser für sie, daß Wir ihnen Aufschub gewähren. Wir gewähren ihnen ja Aufschub, damit sie noch mehr Schuld auf sich laden. Und bestimmt ist für sie eine schmähliche Pein.\qt@no{178}}%
\qt@newcmd\qurantrans@de@cdlxxii{Nimmer wird Gott die Gläubigen in dem Zustand belassen, in dem ihr euch befindet; Er will nur das Schlechte vom Guten unterscheiden. Und nimmer wird Gott euch über das Unsichtbare Auskunft geben. Aber Gott erwählt von seinen Gesandten, wen Er will. So glaubt an Gott und seine Gesandten. Und wenn ihr glaubt und gottesfürchtig seid, ist für euch ein großartiger Lohn bestimmt.\qt@no{179}}%
\qt@newcmd\qurantrans@de@cdlxxiii{Diejenigen, die mit dem geizen, was Gott ihnen von seiner Huld beschert hat, sollen nicht meinen, das sei besser für sie. Nein, es ist schlechter für sie. Am Tag der Auferstehung wird ihnen das, womit sie gegeizt haben, als Halskette umgelegt werden. Gott gehört das Erbe der Himmel und der Erde. Und Gott hat Kenntnis von dem, was ihr tut.\qt@no{180}}%
\qt@newcmd\qurantrans@de@cdlxxiv{Gott hat die Rede derer gehört, die da sagten: «Gott ist arm, wir sind reich.» Wir werden aufschreiben, was sie sagten, und daß sie die Propheten zu Unrecht töteten, und Wir werden sprechen: Kostet die Pein des Höllenbrandes.\qt@no{181}}%
\qt@newcmd\qurantrans@de@cdlxxv{Dies für das, was eure Hände vorausgeschickt haben, und weil Gott den Dienern kein Unrecht tut.\qt@no{182}}%
\qt@newcmd\qurantrans@de@cdlxxvi{Das sind die, die sagten: «Gott hat uns auferlegt, an keinen Gesandten zu glauben, bis er uns ein Opfer bringt, das das Feuer verzehrt.» Sprich: Es sind vor mir Gesandte mit den deutlichen Zeichen und mit dem, was ihr gesagt habt, zu euch gekommen. Warum habt ihr sie dann getötet, so ihr die Wahrheit sagt?\qt@no{183}}%
\qt@newcmd\qurantrans@de@cdlxxvii{Wenn sie dich der Lüge zeihen, so sind schon vor dir Gesandte der Lüge geziehen worden, die mit den deutlichen Zeichen gekommen waren, und mit den Schriften und dem erleuchtenden Buch.\qt@no{184}}%
\qt@newcmd\qurantrans@de@cdlxxviii{Jeder wird den Tod erleiden. Euch wird euer Lohn am Tag der Auferstehung voll erstattet. Wer vom Feuer weggerückt und ins Paradies geführt wird, der hat das Ziel erreicht. Das diesseitige Leben ist ja nur eine betörende Nutznießung.\qt@no{185}}%
\qt@newcmd\qurantrans@de@cdlxxix{Ihr werdet sicherlich an eurem Vermögen und an euch selbst geprüft werden, und ihr werdet gewiß von denen, denen das Buch vor euch zugekommen ist, und von den Polytheisten viel Ungemach hören. Wenn ihr euch aber geduldig und gottesfürchtig verhaltet, so gehört dies zur Entschlossenheit in den Anliegen.\qt@no{186}}%
\qt@newcmd\qurantrans@de@cdlxxx{Und als Gott die Verpflichtung derer, denen das Buch zugekommen ist, entgegennahm: «Ihr müßt es den Menschen deutlich machen und dürft es nicht verschweigen.» Da warfen sie es hinter ihren Rücken und verkauften es für einen geringen Preis. Schlecht ist, was sie erkaufen.\qt@no{187}}%
\qt@newcmd\qurantrans@de@cdlxxxi{Und meine nicht, daß diejenigen, die sich freuen über das, was sie vollbracht haben, und es lieben, für das gelobt zu werden, was sie nicht getan haben - meine nicht, sie würden der Pein entrinnen. Bestimmt ist für sie eine schmerzhafte Pein.\qt@no{188}}%
\qt@newcmd\qurantrans@de@cdlxxxii{Und Gott gehört die Königsherrschaft der Himmel und der Erde. Und Gott hat Macht zu allen Dingen.\qt@no{189}}%
\qt@newcmd\qurantrans@de@cdlxxxiii{In der Erschaffung der Himmel und der Erde und im Aufeinanderfolgen von Nacht und Tag sind Zeichen für die Einsichtigen,\qt@no{190}}%
\qt@newcmd\qurantrans@de@cdlxxxiv{Die Gottes gedenken, im Stehen und Sitzen und auf ihren Seiten liegend, und über die Erschaffung der Himmel und der Erde nachdenken: «Unser Herr, Du hast dies nicht umsonst erschaffen. Preis sei Dir! Bewahre uns vor der Pein des Feuers.\qt@no{191}}%
\qt@newcmd\qurantrans@de@cdlxxxv{Unser Herr, wen Du ins Feuer eingehen läßt, den hast Du zuschanden gemacht. Diejenigen, die Unrecht tun, werden keine Helfer haben.\qt@no{192}}%
\qt@newcmd\qurantrans@de@cdlxxxvi{Unser Herr, wir haben einen Rufer gehört, der zum Glauben ruft: Glaubt an euren Herrn. Da haben wir geglaubt. Unser Herr, vergib uns unsere Sünden und sühne uns unsere Missetaten, und berufe uns mit den Frommen ab.\qt@no{193}}%
\qt@newcmd\qurantrans@de@cdlxxxvii{Unser Herr, gib uns das, was Du uns durch deine Gesandten verheißen hast, und mache uns am Tag der Auferstehung nicht zuschanden. Du brichst das Versprechen nicht.»\qt@no{194}}%
\qt@newcmd\qurantrans@de@cdlxxxviii{Da erhörte sie ihr Herr: «Ich lasse keine Tat verlorengehen, die einer von euch getan hat, ob Mann oder Weib. Die einen stammen ja von den anderen. Denjenigen, die ausgewandert und aus ihren Wohnstätten vertrieben worden sind und auf meinem Weg Leid erlitten haben, die gekämpft haben und getötet worden sind, werde Ich ihre Missetaten sühnen und sie in Gärten eingehen lassen, unter denen Bäche fließen, als Belohnung von Gott.» Und bei Gott ist die schöne Belohnung.\qt@no{195}}%
\qt@newcmd\qurantrans@de@cdlxxxix{Laß dich nicht betören durch das Umherziehen der Ungläubigen im Land.\qt@no{196}}%
\qt@newcmd\qurantrans@de@cdxc{Das ist eine geringe Nutznießung. Dann wird ihre Heimstätte die Hölle sein - welch schlimme Lagerstätte!\qt@no{197}}%
\qt@newcmd\qurantrans@de@cdxci{Aber für diejenigen, die ihren Herrn fürchten, sind Gärten bestimmt, unter denen Bäche fließen; darin werden sie ewig weilen, als Herberge von seiten Gottes. Und was bei Gott ist, ist besser für die Frommen.\qt@no{198}}%
\qt@newcmd\qurantrans@de@cdxcii{Unter den Leuten des Buches gibt es welche, die an Gott glauben und an das, was zu euch herabgesandt wurde, und an das, was zu ihnen herabgesandt wurde, und so zeigen sie sich demütig gegen Gott. Und sie verkaufen nicht die Zeichen Gottes für einen geringen Preis. Jene haben ihren Lohn bei ihrem Herrn. Gott ist schnell im Abrechnen.\qt@no{199}}%
\qt@newcmd\qurantrans@de@cdxciii{O ihr, die ihr glaubt, seid geduldig und miteinander standhaft und einsatzbereit. Und fürchtet Gott, auf daß es euch wohl ergehe.\qt@no{200}}%
\qt@newcmd\qurantrans@de@cdxciv{\basmalahde O ihr Menschen, fürchtet euren Herrn, der euch aus einem einzigen Wesen erschuf, aus ihm seine Gattin erschuf und aus ihnen beiden viele Männer und Frauen entstehen und sich ausbreiten ließ. Und fürchtet Gott, in dessen Namen ihr einander bittet, und (achtet) die Verwandtschaftsbande. Gott ist Wächter über euch.\qt@no{1}}%
\qt@newcmd\qurantrans@de@cdxcv{Und gebt den Waisen ihr Vermögen und tauscht nicht Schlechtes gegen Gutes aus. Und zehrt nicht ihr Vermögen auf zu eurem Vermögen hinzu. Das wäre eine große Sünde.\qt@no{2}}%
\qt@newcmd\qurantrans@de@cdxcvi{Und wenn ihr fürchtet, gegenüber den Waisen nicht gerecht zu sein, dann heiratet, was euch an Frauen beliebt, zwei, drei oder vier. Wenn ihr aber fürchtet, (sie) nicht gleich zu behandeln, dann nur eine, oder was eure rechte Hand (an Sklavinnen) besitzt. Das bewirkt es eher, daß ihr euch vor Ungerechtigkeit bewahrt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@cdxcvii{Und gebt den Frauen ihre Morgengabe als Geschenk. Wenn sie euch freiwillig etwas davon überlassen, so könnt ihr es verbrauchen, und es wird euch zur Freude und zum Wohl sein.\qt@no{4}}%
\qt@newcmd\qurantrans@de@cdxcviii{Und gebt nicht den Toren euer Vermögen, durch das Gott euch einen Unterhalt verschafft hat. Versorgt sie damit und kleidet sie, und sagt zu ihnen, was sich geziemt.\qt@no{5}}%
\qt@newcmd\qurantrans@de@cdxcix{Und prüft die Waisen. Sobald sie das Heiratsalter erreicht haben, übergebt ihnen ihr Vermögen, wenn ihr bei ihnen einen rechten Wandel feststellt. Und zehrt es nicht verschwenderisch und voreilig auf, bevor sie älter werden. Und wer nicht darauf angewiesen ist, soll sich enthalten; wer arm ist, soll in rechtlicher Weise davon zehren. Und wenn ihr ihnen ihr Vermögen übergebt, dann laßt es gegen sie bezeugen. Und Gott genügt als der, der abrechnet.\qt@no{6}}%
\qt@newcmd\qurantrans@de@d{Den Männern steht ein Teil von dem, was die Eltern und die Angehörigen hinterlassen, zu, und den Frauen steht ein Teil von dem, was die Eltern und die Angehörigen hinterlassen, zu, mag es wenig oder viel sein; (dies gilt) als Pflichtanteil.\qt@no{7}}%
\qt@newcmd\qurantrans@de@di{Und wenn die Verwandten, die Waisen und die Bedürftigen bei der Teilung zugegen sind, so gebt ihnen etwas davon und sagt zu ihnen, was sich geziemt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@dii{Fürchten sollen sich diejenigen, die, wenn sie selbst schwache Nachkommen hinterlassen würden, besorgt um sie wären. Sie sollen Gott fürchten und zutreffende Worte sprechen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@diii{Diejenigen, die das Vermögen der Waisen zu Unrecht verzehren, verzehren nur Feuer in ihrem Bauch. Und sie werden in einem Feuerbrand brennen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@div{Gott trägt euch in bezug auf eure Kinder (folgendes) auf: Einem männlichen Kind steht soviel wie der Anteil von zwei weiblichen zu; sind es nur Frauen, über zwei an der Zahl, so stehen ihnen zwei Drittel dessen, was er hinterläßt, zu; ist es nur eine, so steht ihr die Hälfte zu. Den beiden Eltern steht jedem von ihnen ein Sechstel dessen, was er hinterläßt, zu, wenn er Kinder hat; wenn er keine Kinder hat und seine Eltern ihn beerben, so steht seiner Mutter ein Drittel zu. Hat er Brüder, so steht seiner Mutter ein Sechstel zu. (Dies gilt) nach Berücksichtigung eines Testamentes, das er etwa gemacht hat, oder einer (bestehenden) Schuld. - Eure Väter und eure Söhne: Ihr wißt nicht, wer von ihnen euch im Nutzen nähersteht. - (Dies ist) eine Pflicht von seiten Gottes. Gott weiß Bescheid und ist weise.\qt@no{11}}%
\qt@newcmd\qurantrans@de@dv{Euch steht die Hälfte dessen, was eure Gattinnen hinterlassen, zu, wenn sie keine Kinder haben. Wenn sie Kinder haben, dann steht euch ein Viertel dessen, was sie hinterlassen, zu, und zwar nach Berücksichtigung eines Testamentes, das sie etwa gemacht haben, oder einer (bestehenden) Schuld. Und es steht ihnen ein Viertel dessen, was ihr hinterlaßt, zu, wenn ihr keine Kinder habt. Wenn ihr Kinder habt, dann steht ihnen ein Achtel dessen, was ihr hinterlaßt, zu, und zwar nach Berücksichtigung eines Testamentes, das ihr etwa gemacht habt, oder einer (bestehenden) Schuld. Und wenn ein Mann oder eine Frau von seitlichen Verwandten beerbt wird und er einen Bruder oder eine Schwester hat, dann steht einem jeden von ihnen ein Sechstel zu. Sind es mehr, dann teilen sie sich in ein Drittel, und zwar nach Berücksichtigung eines Testamentes, das er etwa gemacht hat, oder einer (bestehenden) Schuld. Es soll kein Schaden zugefügt werden. (Dies ist) ein Auftrag von Seiten Gottes. Gott weiß Bescheid und ist langmütig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@dvi{Das sind die Rechtsbestimmungen Gottes. Wer Gott und seinem Gesandten gehorcht, den läßt Er in Gärten eingehen, unter denen Bäche fließen; darin werden sie ewig weilen. Das ist der großartige Erfolg.\qt@no{13}}%
\qt@newcmd\qurantrans@de@dvii{Und wer gegen Gott und seinen Gesandten ungehorsam ist und seine Rechtsbestimmungen übertritt, den läßt Er in ein Feuer eingehen; darin wird er ewig weilen. Und bestimmt ist für ihn eine schmähliche Pein.\qt@no{14}}%
\qt@newcmd\qurantrans@de@dviii{Gegen diejenigen von euren Frauen, die Schändliches begehen, müßt ihr vier von euch zeugen lassen. Wenn sie es bezeugen, dann haltet sie in den Häusern fest, bis der Tod sie abberuft oder Gott ihnen einen Ausweg verschafft.\qt@no{15}}%
\qt@newcmd\qurantrans@de@dix{Und wenn zwei von euch es begehen, dann fügt ihnen beiden Leid zu. Wenn sie bereuen und Besserung zeigen, dann laßt von ihnen ab. Gott schenkt Zuwendung und ist barmherzig.\qt@no{16}}%
\qt@newcmd\qurantrans@de@dx{Gott obliegt es, sich denen (gnädig) zuzuwenden, die aus Unwissenheit das Böse tun und die dann bald darauf bereuen. Diesen schenkt Gott Zuwendung. Gott weiß Bescheid und ist weise.\qt@no{17}}%
\qt@newcmd\qurantrans@de@dxi{Die gnädige Zuwendung gilt aber nicht für die, welche die bösen Taten begehen, so daß erst, wenn der Tod einem von ihnen naht, dieser sagt: «Ich bereue jetzt»; und auch nicht für die, die als Ungläubige sterben. Diesen haben Wir eine schmerzliche Pein bereitet.\qt@no{18}}%
\qt@newcmd\qurantrans@de@dxii{O ihr, die ihr glaubt, es ist euch nicht erlaubt, die Frauen wider ihren Willen zu erben. Und setzt ihnen nicht zu, um etwas von dem zu nehmen, was ihr ihnen zukommen ließet, es sei denn, sie begehen eine eindeutige schändliche Tat. Und geht mit ihnen in rechtlicher Weise um. Wenn sie euch zuwider sind, so ist euch vielleicht etwas zuwider, während Gott viel Gutes in es hineinlegt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@dxiii{Und wenn ihr eine Gattin gegen eine (andere) Gattin eintauschen wollt und ihr der einen von ihnen einen Zentner habt zukommen lassen, so nehmt nichts davon. Wollt ihr es denn durch Verleumdung und offenkundige Sünde nehmen?\qt@no{20}}%
\qt@newcmd\qurantrans@de@dxiv{Wie könnt ihr es nehmen, wo ihr bereits zueinander eingegangen seid und wo sie von euch eine feste Verpflichtung entgegengenommen haben?\qt@no{21}}%
\qt@newcmd\qurantrans@de@dxv{Und heiratet nicht solche Frauen, die (vorher) eure Väter geheiratet haben, abgesehen von dem, was bereits geschehen ist. Das ist etwas Schändliches und Abscheuliches und ein übler Weg.\qt@no{22}}%
\qt@newcmd\qurantrans@de@dxvi{Verboten ist auch, zu heiraten eure Mütter, eure Töchter, eure Schwestern, eure Tanten väterlicherseits und eure Tanten mütterlicherseits, die Töchter des Bruders und die Töchter der Schwester, eure Mütter, die euch gestillt haben, und eure Milchschwestern, die Mütter eurer Frauen, eure Stieftöchter, die sich in eurem Schutz befinden und von euren Frauen stammen, zu denen ihr eingegangen seid - wenn ihr zu ihnen noch nicht eingegangen seid, dann ist es für euch kein Vergehen -, und die Ehefrauen eurer Söhne, die aus euren Lenden stammen. (Verboten ist) auch, daß ihr zwei Schwestern zur Frau zusammen habt, abgesehen von dem, was bereits geschehen ist. Gott ist voller Vergebung und barmherzig.\qt@no{23}}%
\qt@newcmd\qurantrans@de@dxvii{Und (verboten ist, zu heiraten) die unter Schutz Gestellten unter den Frauen, ausgenommen das, was eure rechte Hand (an Sklavinnen) besitzt. Das ist die Vorschrift Gottes für euch. Erlaubt ist euch, was jenseits dieser (Gruppe) liegt, daß ihr euch mit eurem Vermögen (Frauen) sucht in der Absicht, (sie) unter Schutz zu stellen und nicht Unzucht zu treiben. Denen unter ihnen, die ihr genossen habt, sollt ihr - das ist eine Rechtspflicht - ihren Lohn geben. Es besteht für euch kein Vergehen, wenn ihr, nachdem die Rechtspflicht festgesetzt ist, darüber hinaus etwas in gegenseitigem Einvernehmen vereinbart. Gott weiß Bescheid und ist weise.\qt@no{24}}%
\qt@newcmd\qurantrans@de@dxviii{Wer von euch keine Mittel besitzt, um unter Schutz gestellte gläubige Frauen zu heiraten, der soll Frauen heiraten aus den Reihen der gläubigen Mägde, die eure rechte Hand besitzt. Gott weiß besser Bescheid über euren Glauben. Die einen von euch stammen ja von den anderen. So heiratet sie mit der Erlaubnis ihrer Herren und gebt ihnen ihren Lohn in rechtlicher Weise, als unter Schutz gestellten Frauen, die nicht Unzucht treiben und sich keine Liebhaber nehmen. Wenn sie unter Schutz gestellt worden sind und dann Schändliches begehen, steht für sie darauf an Pein die Hälfte dessen, was für die unter Schutz gestellten Frauen steht. Dies gilt für denjenigen von euch, der die Bedrängnis fürchtet. Und wenn ihr euch geduldig zeigt, so ist es besser für euch. Und Gott ist voller Vergebung und barmherzig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@dxix{Gott will es euch deutlich machen und euch zu den Verfahrensweisen derer, die vor euch lebten, leiten und sich euch zuwenden. Gott weiß Bescheid und ist weise.\qt@no{26}}%
\qt@newcmd\qurantrans@de@dxx{Und Gott will sich euch zuwenden; diejenigen aber, die den Begierden folgen, wollen, daß ihr eine gewaltige Abweichung vollzieht.\qt@no{27}}%
\qt@newcmd\qurantrans@de@dxxi{Gott will euch Erleichterung gewähren. Der Mensch ist ja schwach erschaffen worden.\qt@no{28}}%
\qt@newcmd\qurantrans@de@dxxii{O ihr, die ihr glaubt, verzehrt nicht untereinander euer Vermögen durch Betrug, es sei denn, es geht um einen Handel in gegenseitigem Einvernehmen. Und tötet nicht einander. Gott ist barmherzig zu euch.\qt@no{29}}%
\qt@newcmd\qurantrans@de@dxxiii{Wer es (doch) in Übertretung und Unrecht tut, den werden Wir in einem Feuer brennen lassen. Das ist Gott ein leichtes.\qt@no{30}}%
\qt@newcmd\qurantrans@de@dxxiv{Wenn ihr die schweren Vergehen meidet von dem, was euch untersagt ist, sühnen Wir euch eure Missetaten und gewähren euch einen ehrenvollen Eingang (ins Paradies).\qt@no{31}}%
\qt@newcmd\qurantrans@de@dxxv{Und wünscht euch nicht das, womit Gott die einen von euch vor den anderen bevorzugt hat. Die Männer erhalten einen Anteil von dem, was sie erworben haben, und die Frauen erhalten einen Anteil von dem, was sie erworben haben. Und bittet Gott um etwas von seiner Huld. Gott weiß über alle Dinge Bescheid.\qt@no{32}}%
\qt@newcmd\qurantrans@de@dxxvi{Einem jeden haben Wir Erbfolger gegeben in bezug auf das, was die Eltern und die Angehörigen hinterlassen. Und gebt denjenigen, mit denen euch eure Eide verbinden, ihren Anteil. Gott ist ja über alle Dinge Zeuge.\qt@no{33}}%
\qt@newcmd\qurantrans@de@dxxvii{Die Männer haben Vollmacht und Verantwortung gegenüber den Frauen, weil Gott die einen vor den anderen bevorzugt hat und weil sie von ihrem Vermögen (für die Frauen) ausgeben. Die rechtschaffenen Frauen sind demütig ergeben und bewahren das, was geheimgehalten werden soll, da Gott es geheimhält. Ermahnt diejenigen, von denen ihr Widerspenstigkeit befürchtet, und entfernt euch von ihnen in den Schlafgemächern und schlagt sie. Wenn sie euch gehorchen, dann wendet nichts Weiteres gegen sie an. Gott ist erhaben und groß.\qt@no{34}}%
\qt@newcmd\qurantrans@de@dxxviii{Und wenn ihr ein Zerwürfnis zwischen beiden (Ehepartnern) befürchtet, dann bestellt einen Schiedsrichter aus seiner Familie und einen Schiedsrichter aus ihrer Familie. Wenn sie sich aussöhnen wollen, wird Gott ihnen Eintracht schenken. Gott weiß Bescheid und hat Kenntnis von allem.\qt@no{35}}%
\qt@newcmd\qurantrans@de@dxxix{Und dient Gott und gesellt Ihm nichts bei. Und behandelt die Eltern gut und die Verwandten, die Waisen, die Bedürftigen, den verwandten Beisassen, den fremden Beisassen, den Gefährten an eurer Seite, den Reisenden und das, was eure rechte Hand besitzt. Gott liebt die nicht, die eingebildet und prahlerisch sind,\qt@no{36}}%
\qt@newcmd\qurantrans@de@dxxx{Die geizen und den Leuten befehlen, geizig zu sein, und die verschweigen, was Gott ihnen von seiner Huld hat zukommen lassen - Wir haben den Ungläubigen eine schmähliche Pein bereitet -,\qt@no{37}}%
\qt@newcmd\qurantrans@de@dxxxi{Und die ihr Vermögen spenden, um von den Menschen gesehen zu werden, und nicht an Gott glauben, und auch nicht an den Jüngsten Tag. Und wenn einer den Satan zum Gesellen hat, so ist das ein schlimmer Geselle.\qt@no{38}}%
\qt@newcmd\qurantrans@de@dxxxii{Was würde ihnen (Schlimmes) widerfahren, wenn sie an Gott und den Jüngsten Tag glaubten und von dem spendeten, was Gott ihnen beschert hat? Und Gott weiß über sie Bescheid.\qt@no{39}}%
\qt@newcmd\qurantrans@de@dxxxiii{Gott tut nicht einmal im Gewicht eines Stäubchens Unrecht. Und wenn es eine gute Tat ist, so wird Er sie verdoppeln und von sich her einen großartigen Lohn zukommen lassen.\qt@no{40}}%
\qt@newcmd\qurantrans@de@dxxxiv{Wie wird es sein, wenn Wir von jeder Gemeinschaft einen Zeugen beibringen und dich als Zeugen über diese beibringen?\qt@no{41}}%
\qt@newcmd\qurantrans@de@dxxxv{An jenem Tag werden diejenigen, die ungläubig und gegen den Gesandten ungehorsam waren, wünschen, sie würden dem Erdboden gleichgemacht, und sie werden vor Gott keine Aussage verschweigen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@dxxxvi{O ihr, die ihr glaubt, kommt nicht zum Gebet, während ihr betrunken seid, bis ihr wißt, was ihr sagt, und auch nicht sexuell verunreinigt - es sei denn, ihr geht vorbei -, bis ihr euch gewaschen habt. Und wenn ihr krank oder auf Reisen seid, oder wenn einer von euch vom Abort kommt oder wenn ihr die Frauen berührt habt und ihr kein Wasser findet, dann sucht --einen sauberen Boden und streicht euch über das Gesicht und die Hände. Gott ist voller Verzeihung und Vergebung.\qt@no{43}}%
\qt@newcmd\qurantrans@de@dxxxvii{Hast du nicht auf jene geschaut, denen ein Anteil am Buch zugekommen ist, wie sie den Irrtum erkaufen und wollen, daß ihr vom Weg abirrt?\qt@no{44}}%
\qt@newcmd\qurantrans@de@dxxxviii{Gott weiß besser über eure Feinde Bescheid. Und Gott genügt als Freund, und Gott genügt als Helfer.\qt@no{45}}%
\qt@newcmd\qurantrans@de@dxxxix{Unter denen, die Juden sind, entstellen einige den Sinn der Worte und sagen: «Wir hören, und wir gehorchen nicht», und: «Höre zu, ohne daß du hören kannst», und: «Achte auf uns» (raainaa); sie verdrehen dabei ihre Zungen und greifen die Religion an. Hätten sie gesagt: «Wir hören, und wir gehorchen», und: «Höre», und: «Schau auf uns» (unzurnaa), wäre es besser und richtiger für sie. Aber Gott hat sie wegen ihres Unglaubens verflucht, so glauben sie nur wenig.\qt@no{46}}%
\qt@newcmd\qurantrans@de@dxl{O ihr, denen das Buch zugekommen ist, glaubt an das, was Wir hinabgesandt haben zur Bestätigung dessen, was bei euch ist, bevor Wir bestimmte Gesichter auswischen und sie auf ihren Rücken kehren oder sie verfluchen, wie Wir die Gefährten des Sabbats verflucht haben. Und der Befehl Gottes wird ausgeführt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@dxli{Gott vergibt nicht, daß Ihm beigesellt wird, und Er vergibt, was darunter liegt, wem Er will. Und wer Gott (andere) beigesellt, hat eine gewaltige Sünde erdichtet.\qt@no{48}}%
\qt@newcmd\qurantrans@de@dxlii{Hast du nicht auf jene geschaut, die sich selbst für rein erklären? Nein, Gott erklärt rein, wen Er will, und ihnen wird nicht ein Dattelfädchen Unrecht getan.\qt@no{49}}%
\qt@newcmd\qurantrans@de@dxliii{Schau, wie sie gegen Gott Lügen erdichten! Dies genügt als offenkundige Sünde.\qt@no{50}}%
\qt@newcmd\qurantrans@de@dxliv{Hast du nicht auf jene geschaut, denen ein Anteil am Buch zugekommen ist, wie sie an die nutzlose Magie und an die Götzen glauben und von denen, die ungläubig sind, sagen: «Diese folgen einem rechteren Weg als die, die glauben»?\qt@no{51}}%
\qt@newcmd\qurantrans@de@dxlv{Das sind die, die Gott verflucht hat. Und wen Gott verflucht, für den wirst du keinen Helfer finden.\qt@no{52}}%
\qt@newcmd\qurantrans@de@dxlvi{Oder haben sie etwa einen Anteil an der Königsherrschaft? Auch dann würden sie den Menschen nicht einmal ein Dattelgrübchen zukommen lassen.\qt@no{53}}%
\qt@newcmd\qurantrans@de@dxlvii{Oder beneiden sie die Menschen um das, was ihnen Gott von seiner Huld hat zukommen lassen? Wir ließen ja der Sippe Abrahams das Buch und die Weisheit zukommen, und Wir ließen ihnen eine gewaltige Königsherrschaft zukommen.\qt@no{54}}%
\qt@newcmd\qurantrans@de@dxlviii{Unter ihnen gab es solche, die daran glaubten, und unter ihnen gab es solche, die sich davon abwandten. Und die Hölle genügt als Feuerbrand.\qt@no{55}}%
\qt@newcmd\qurantrans@de@dxlix{Diejenigen, die unsere Zeichen verleugnen, werden Wir in einem Feuer brennen lassen. Sooft ihre Häute gar sind, tauschen Wir ihnen andere Häute (dagegen) ein, damit sie die Pein kosten. Gott ist mächtig und weise.\qt@no{56}}%
\qt@newcmd\qurantrans@de@dl{Und diejenigen, die glauben und die guten Werke tun, werden Wir in Gärten eingehen lassen, unter denen Bäche fließen; darin werden sie auf immer ewig weilen. Darin sind geläuterte Gattinnen. Und Wir werden sie in einen ausgedehnten Schatten eingehen lassen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@dli{Gott befiehlt euch, anvertraute Güter ihren Eigentümern zurückzugeben und, wenn ihr unter den Menschen urteilt, nach Gerechtigkeit zu urteilen. Wie trefflich ist das, womit Gott euch ermahnt! Gott sieht und hört alles.\qt@no{58}}%
\qt@newcmd\qurantrans@de@dlii{O ihr, die ihr glaubt, gehorchet Gott und gehorchet dem Gesandten und den Zuständigen unter euch. Wenn ihr über etwas streitet, so bringt es vor Gott und den Gesandten, so ihr an Gott und den Jüngsten Tag glaubt. Das ist besser und führt zu einem schöneren Ergebnis.\qt@no{59}}%
\qt@newcmd\qurantrans@de@dliii{Hast du nicht auf jene geschaut, die behaupten, sie glaubten an das, was zu dir herabgesandt wurde, und an das, was vor dir herabgesandt wurde, wie sie die Streitsachen vor die Götzen bringen wollen, wo ihnen doch befohlen wurde, sie zu verleugnen? Und der Satan will sie weit abirren lassen.\qt@no{60}}%
\qt@newcmd\qurantrans@de@dliv{Und wenn zu ihnen gesagt wird: «Kommt her zu dem, was Gott herabgesandt hat, und zum Gesandten», siehst du die Heuchler sich klar von dir abwenden.\qt@no{61}}%
\qt@newcmd\qurantrans@de@dlv{Wie wird es wohl sein, wenn sie ein Unglück trifft für das, was ihre Hände vorausgeschickt haben, und sie zu dir kommen und bei Gott schwören: «Wir wollten es ja nur gutmachen und Einvernehmen schaffen»?\qt@no{62}}%
\qt@newcmd\qurantrans@de@dlvi{Das sind die, von denen Gott weiß, was in ihrem Herzen ist. Wende dich von ihnen ab, ermahne sie und sprich zu ihnen über sie selbst eindringliche Worte.\qt@no{63}}%
\qt@newcmd\qurantrans@de@dlvii{Und Wir haben die Gesandten nur deswegen entsandt, damit man ihnen gehorcht mit der Erlaubnis Gottes. Würden sie, da sie gegen sich selbst Unrecht verübt haben, zu dir kommen und Gott um Vergebung bitten, und würde der Gesandte für sie um Vergebung bitten, so würden sie sicher finden, daß Gott sich gnädig zuwendet und barmherzig ist.\qt@no{64}}%
\qt@newcmd\qurantrans@de@dlviii{Nein, bei deinem Herrn, sie glauben nicht (wirklich), bis sie dich zum Schiedsrichter nehmen über das, was zwischen ihnen umstritten ist, und danach wegen deiner Entscheidung keine Bedrängnis in ihrem Inneren spüren, sondern sich in völliger Ergebung fügen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@dlix{Und wenn Wir ihnen vorgeschrieben hätten: Tötet einander oder zieht aus euren Wohnstätten aus, sie hätten es nicht getan, bis auf wenige von ihnen. Würden sie doch das tun, wozu sie ermahnt werden, es wäre sicher besser für sie und würde sie stärker festigen,\qt@no{66}}%
\qt@newcmd\qurantrans@de@dlx{Und Wir würden ihnen von Uns her einen großartigen Lohn zukommen lassen,\qt@no{67}}%
\qt@newcmd\qurantrans@de@dlxi{Und Wir würden sie einen geraden Weg führen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@dlxii{Diejenigen, die Gott und dem Gesandten gehorchen, befinden sich mit denen, die Gott begnadet hat, von den Propheten, den Wahrhaftigen, den Märtyrern und den Rechtschaffenen. Welch treffliche Gefährten sind es!\qt@no{69}}%
\qt@newcmd\qurantrans@de@dlxiii{Das ist die Huld Gottes. Und Gott genügt als der, der Bescheid weiß.\qt@no{70}}%
\qt@newcmd\qurantrans@de@dlxiv{O ihr, die ihr glaubt, seid auf eurer Hut, und rückt in Trupps aus oder rückt zusammen aus.\qt@no{71}}%
\qt@newcmd\qurantrans@de@dlxv{Unter euch gibt es manch einen, der langsam ausrückt; und wenn euch ein Unglück trifft, sagt er: «Gott hat mir Gnade erwiesen, da ich nicht mit ihnen dabei war.»\qt@no{72}}%
\qt@newcmd\qurantrans@de@dlxvi{Und wenn euch eine Huld von Gott her zuteil wird, sagt er sicher, als hätte zwischen euch und ihm nie Liebe bestanden: «O wäre ich doch mit ihnen gewesen, so hätte ich einen großartigen Erfolg erlangt!»\qt@no{73}}%
\qt@newcmd\qurantrans@de@dlxvii{So sollen diejenigen, die das diesseitige Leben gegen das Jenseits verkaufen, auf dem Weg Gottes kämpfen. Und wer auf dem Weg Gottes kämpft und daraufhin getötet wird oder siegt, dem werden Wir einen großartigen Lohn zukommen lassen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@dlxviii{Was hindert euch daran, zu kämpfen auf dem Weg Gottes und für diejenigen unter den Männern, den Frauen und den Kindern, die wie Schwache behandelt werden und die sagen: «Unser Herr, führe uns aus dieser Stadt hinaus, deren Einwohner Unrecht tun, und bestelle uns von Dir her einen Freund, und bestelle uns von Dir her einen Helfer»?\qt@no{75}}%
\qt@newcmd\qurantrans@de@dlxix{Diejenigen, die glauben, kämpfen auf dem Weg Gottes. Und diejenigen, die ungläubig sind, kämpfen auf dem Weg der Götzen. So kämpft gegen die Freunde des Satans. Die List des Satans ist schwach.\qt@no{76}}%
\qt@newcmd\qurantrans@de@dlxx{Hast du nicht auf jene geschaut, zu denen gesagt wurde: «Haltet eure Hände zurück und verrichtet das Gebet und entrichtet die Abgabe»? Als ihnen dann der Kampf vorgeschrieben wurde, hatte plötzlich ein Teil von ihnen eine solche Furcht vor den Menschen wie die Furcht vor Gott oder eine noch stärkere Furcht. Und sie sagten: «Unser Herr, warum hast Du uns den Kampf vorgeschrieben? Hättest Du uns doch für eine kurze Frist zurückgestellt!» Sprich: Die Nutznießung des Diesseits ist gering, und das Jenseits ist besser für den, der gottesfürchtig ist. Und euch wird nicht ein Dattelfädchen Unrecht getan.\qt@no{77}}%
\qt@newcmd\qurantrans@de@dlxxi{Wo immer ihr seid, der Tod wird euch erreichen, auch wenn ihr in hochgebauten Burgen wäret. Wenn sie etwas Gutes trifft, sagen sie: «Das ist von Gott.» Und wenn sie etwas Schlechtes trifft, sagen sie: «Das ist von dir.» Sprich: Alles ist von Gott. Was ist mit diesen Leuten los, daß sie kaum eine Aussage begreifen?\qt@no{78}}%
\qt@newcmd\qurantrans@de@dlxxii{Was dich an Gutem trifft, ist von Gott. Und was dich an Schlechtem trifft, ist von dir selbst. Und Wir haben dich zum Gesandten für die Menschen entsandt. Und Gott genügt als Zeuge.\qt@no{79}}%
\qt@newcmd\qurantrans@de@dlxxiii{Wer dem Gesandten gehorcht, gehorcht Gott. Und wenn jemand sich abkehrt - siehe, Wir haben dich nicht als Hüter über sie gesandt.\qt@no{80}}%
\qt@newcmd\qurantrans@de@dlxxiv{Und sie sagen: «(Wir leisten) Gehorsam.» Wenn sie jedoch von dir weggehen, beschließt eine Gruppe von ihnen im Dunkeln etwas anderes als das, was du sagst. Und Gott schreibt, was sie im Dunkeln beschließen, auf. Darum wende dich von ihnen ab und vertrau auf Gott. Gott genügt als Sachwalter.\qt@no{81}}%
\qt@newcmd\qurantrans@de@dlxxv{Betrachten sie denn nicht sorgfältig den Koran? Wenn er von einem anderen als Gott wäre, würden sie in ihm viel Widerspruch finden.\qt@no{82}}%
\qt@newcmd\qurantrans@de@dlxxvi{Und wenn zu ihnen etwas durchdringt, das Sicherheit oder Angst hervorruft, verbreiten sie es. Würden sie es aber vor den Gesandten und die Zuständigen unter ihnen bringen, so würden es diejenigen von ihnen, die es herauszubekommen verstehen, (zu beurteilen) wissen. Und ohne die Huld Gottes gegen euch und seine Barmherzigkeit wäret ihr bis auf wenige dem Satan gefolgt.\qt@no{83}}%
\qt@newcmd\qurantrans@de@dlxxvii{So kämpfe auf dem Weg Gottes. Du hast dich nur für dich selbst zu verantworten. Und sporne die Gläubigen an, vielleicht wird Gott die Schlagkraft derer, die ungläubig sind, zurückhalten. Und Gott besitzt eine noch stärkere Schlagkraft und verhängt eine noch härtere Strafe.\qt@no{84}}%
\qt@newcmd\qurantrans@de@dlxxviii{Wer eine gute Fürbitte einlegt, erhält einen Anteil daran. Und wer eine schlechte Fürbitte einlegt, erhält einen entsprechenden Anteil daran. Gott umsorgt und überwacht alle Dinge.\qt@no{85}}%
\qt@newcmd\qurantrans@de@dlxxix{Wenn ihr mit einem Gruß begrüßt werdet, dann grüßt mit einem noch schöneren Gruß, oder erwidert ihn. Und Gott rechnet über alle Dinge ab.\qt@no{86}}%
\qt@newcmd\qurantrans@de@dlxxx{Gott, es gibt keinen Gott außer Ihm. Er wird euch sicher zu dem Tag der Auferstehung versammeln, an dem kein Zweifel möglich ist. Und wer ist wahrhaftiger als Gott in seinen Aussagen?\qt@no{87}}%
\qt@newcmd\qurantrans@de@dlxxxi{Wieso seid ihr im Hinblick auf die Heuchler zwei Gruppen, wo Gott sie (in den alten Zustand) zurückversetzt hat wegen dessen, was sie erworben haben? Wollt ihr denn rechtleiten, wen Gott irregeführt hat? Wen Gott irreführt, für den findest du keinen Weg.\qt@no{88}}%
\qt@newcmd\qurantrans@de@dlxxxii{Sie möchten gern, ihr würdet ungläubig, wie sie ungläubig sind, so daß ihr ihnen gleich würdet. So nehmt euch niemanden von ihnen zum Freund, bis sie auf dem Weg Gottes auswandern. Wenn sie sich abkehren, dann greift sie und tötet sie, wo immer ihr sie findet, und nehmt euch niemanden von ihnen zum Freund oder Helfer,\qt@no{89}}%
\qt@newcmd\qurantrans@de@dlxxxiii{Mit Ausnahme derer, die zu Leuten gelangen, zwischen denen und euch ein Vertrag besteht, oder zu euch kommen, weil Beklommenheit ihre Brust befallen hat, gegen euch zu kämpfen oder gegen ihre (eigenen) Leute zu kämpfen - und wenn Gott gewollt hätte, hätte Er ihnen Gewalt über euch verliehen, und dann hätten sie gewiß gegen euch gekämpft. Wenn sie sich von euch fernhalten und nicht gegen euch kämpfen und euch Frieden anbieten, dann erlaubt euch Gott nicht, gegen sie vorzugehen.\qt@no{90}}%
\qt@newcmd\qurantrans@de@dlxxxiv{Ihr werdet andere finden, die vor euch Sicherheit und auch vor ihren (eigenen) Leuten Sicherheit haben wollen. Jedesmal, wenn sie der Verführung unterworfen werden, werden sie dadurch (in den alten Zustand) zurückversetzt. Wenn sie sich nicht von euch fernhalten und euch nicht den Frieden anbieten und ihre Hände nicht zurückhalten, dann greift sie und tötet sie, wo immer ihr sie trefft. Über solche Leute haben Wir euch eine offenkundige Gewalt verliehen.\qt@no{91}}%
\qt@newcmd\qurantrans@de@dlxxxv{Es steht einem Gläubigen nicht zu, einen Gläubigen zu töten, es sei denn, (es geschieht) aus Versehen. Wer einen Gläubigen aus Versehen tötet, hat einen gläubigen Sklaven zu befreien oder ein Blutgeld an seine Angehörigen zu übergeben, es sei denn, sie erlassen es als Almosen. Wenn er zu Leuten gehört, die eure Feinde sind, während er ein Gläubiger ist, so ist ein gläubiger Sklave zu befreien. Und wenn er zu Leuten gehört, zwischen denen und euch ein Vertrag besteht, dann ist ein Blutgeld an seine Angehörigen auszuhändigen und ein gläubiger Sklave zu befreien. Wer es nicht vermag, der hat zwei Monate hintereinander zu fasten. Das ist eine Zuwendung von Seiten Gottes. Und Gott weiß Bescheid und ist weise.\qt@no{92}}%
\qt@newcmd\qurantrans@de@dlxxxvi{Und wer einen Gläubigen vorsätzlich tötet, dessen Lohn ist die Hölle; darin wird er ewig weilen. Und Gott zürnt ihm und verflucht ihn und bereitet ihm eine gewaltige Pein.\qt@no{93}}%
\qt@newcmd\qurantrans@de@dlxxxvii{O ihr, die ihr glaubt, wenn ihr auf dem Weg Gottes im Land umherwandert, so stellt die Lage eindeutig fest und sagt nicht zu dem, der euch den Frieden anbietet: «Du bist kein Gläubiger», im Trachten nach den Gütern des diesseitigen Lebens. Gott schafft doch viele Möglichkeiten, Beute zu erzielen. So seid ihr früher gewesen, da hat Gott euch eine Wohltat erwiesen. Stellt also die Lage eindeutig fest. Gott hat Kenntnis von dem, was ihr tut.\qt@no{94}}%
\qt@newcmd\qurantrans@de@dlxxxviii{Nicht gleich sind diejenigen unter den Gläubigen, die daheim sitzen, ohne ein Gebrechen zu haben, und diejenigen, die sich auf dem Weg Gottes mit ihrem Vermögen und mit ihrer eigenen Person einsetzen. Gott bevorzugt diejenigen, die sich mit ihrem Vermögen und mit ihrer eigenen Person einsetzen, um eine ganze Stufe vor denen, die daheim sitzen. Einem jeden hat Gott das Beste versprochen. Doch hat Gott diejenigen, die sich einsetzen, vor denen, die daheim sitzen, bevorzugt mit einem großartigen Lohn,\qt@no{95}}%
\qt@newcmd\qurantrans@de@dlxxxix{Mit Rangstufen von Ihm her, mit Vergebung und Barmherzigkeit. Gott ist voller Vergebung und barmherzig.\qt@no{96}}%
\qt@newcmd\qurantrans@de@dxc{Zu denen, die die Engel abberufen, während sie gegen sich selbst Unrecht verübt haben, sagen diese: «Wie war euer Zustand?» Sie sagen: «Wir wurden auf der Erde wie Schwache behandelt.» Sie sagen: «War denn die Erde Gottes nicht weit genug, so daß ihr auf ihr hättet auswandern können?» Diese haben die Hölle zur Heimstätte - welch schlimmes Ende!\qt@no{97}}%
\qt@newcmd\qurantrans@de@dxci{- mit Ausnahme derer unter den Männern, Frauen und Kindern, die wie Schwache behandelt wurden und sich keine List erdenken konnten und keinen rechten Weg fanden.\qt@no{98}}%
\qt@newcmd\qurantrans@de@dxcii{Diesen möge der Herr verzeihen! Und Gott ist voller Verzeihung und Vergebung.\qt@no{99}}%
\qt@newcmd\qurantrans@de@dxciii{Wer auf dem Weg Gottes auswandert, findet auf der Erde viele Auswege und ergiebige Versorgung. Und wer aus seinem Haus hinausgeht, um zu Gott und seinem Gesandten auszuwandern, und dann vom Tod ereilt wird, dessen Lohn obliegt Gott. Und Gott ist voller Vergebung und barmherzig.\qt@no{100}}%
\qt@newcmd\qurantrans@de@dxciv{Und wenn ihr im Land umherwandert, ist es für euch kein Vergehen, das Gebet abzukürzen, falls ihr Angst habt, daß diejenigen, die ungläubig sind, euch der Anfechtung aussetzen. Die Ungläubigen sind euch ja ein offenkundiger Feind.\qt@no{101}}%
\qt@newcmd\qurantrans@de@dxcv{Und wenn du unter ihnen weilst und für sie das Gebet anführst, so soll sich eine Gruppe von ihnen mit dir aufstellen und ihre Waffen ergreifen. Wenn sie sich niederwerfen, so sollen die anderen hinter euch sein. Dann soll eine andere Gruppe, die noch nicht gebetet hat, kommen und mit dir beten, und sie sollen auf ihrer Hut sein und ihre Waffen ergreifen. Diejenigen, die ungläubig sind, möchten gern, ihr würdet auf eure Waffen und eure Sachen nicht achtgeben, so daß sie euch auf einmal überfallen. Und es ist für euch kein Vergehen, wenn ihr unter dem Regen zu leiden habt und krank seid, eure Waffen abzulegen. Und seid auf eurer Hut. Gott hat den Ungläubigen eine schmähliche Pein bereitet.\qt@no{102}}%
\qt@newcmd\qurantrans@de@dxcvi{Und wenn ihr das Gebet beendet habt, dann gedenket Gottes im Stehen und Sitzen und auf euren Seiten liegend. Und wenn ihr Ruhe habt, dann verrichtet das Gebet. Das Gebet ist für die Gläubigen eine für bestimmte Zeiten festgesetzte Vorschrift.\qt@no{103}}%
\qt@newcmd\qurantrans@de@dxcvii{Und erlahmt nicht in der Verfolgung der Leute. Wenn ihr leidet, so leiden sie auch, wie ihr leidet; ihr aber erhofft von Gott, was sie nicht erhoffen. Gott weiß Bescheid und ist weise.\qt@no{104}}%
\qt@newcmd\qurantrans@de@dxcviii{Wir haben zu dir das Buch mit der Wahrheit hinabgesandt, damit du zwischen den Menschen nach dem urteilst, was Gott dich hat sehen lassen. Und sei nicht ein Anwalt der Verräter,\qt@no{105}}%
\qt@newcmd\qurantrans@de@dxcix{Und bitte Gott um Vergebung. Gott ist voller Vergebung und barmherzig.\qt@no{106}}%
\qt@newcmd\qurantrans@de@dc{Und streite nicht zugunsten derer, die sich selbst betrügen. Gott liebt den nicht, der ein Betrüger und ein Sünder ist.\qt@no{107}}%
\qt@newcmd\qurantrans@de@dci{Sie verbergen sich vor den Menschen, sie können sich aber vor Gott nicht verbergen, wo Er bei ihnen ist, wenn sie im Dunkeln Verabredungen treffen, die Ihm nicht gefallen. Und Gott umfaßt, was sie tun.\qt@no{108}}%
\qt@newcmd\qurantrans@de@dcii{Da habt ihr zu ihren Gunsten im diesseitigen Leben gestritten. Wer wird denn am Tag der Auferstehung mit Gott zu ihren Gunsten streiten, oder wer wird als Sachwalter über sie eingesetzt?\qt@no{109}}%
\qt@newcmd\qurantrans@de@dciii{Wer Böses begeht oder sich selbst Unrecht tut und dann Gott um Vergebung bittet, der wird finden, daß Gott voller Vergebung und barmherzig ist.\qt@no{110}}%
\qt@newcmd\qurantrans@de@dciv{Und wer eine Sünde erwirbt, erwirbt sie zu seinem eigenen Schaden. Und Gott weiß Bescheid und ist weise.\qt@no{111}}%
\qt@newcmd\qurantrans@de@dcv{Und wer eine Verfehlung oder eine Sünde erwirbt und sie dann einem Unschuldigen vorwirft, der lädt auf sich eine Verleumdung und eine offenkundige Sünde.\qt@no{112}}%
\qt@newcmd\qurantrans@de@dcvi{Und ohne die Huld Gottes gegen dich und seine Barmherzigkeit wäre eine Gruppe von ihnen im Begriff gewesen, dich irrezuführen; aber sie führen nur sich selbst in die Irre, und sie schaden dir nichts. Und Gott hat auf dich das Buch und die Weisheit herabgesandt und dich gelehrt, was du nicht wußtest. Und die Huld Gottes gegen dich ist gewaltig.\qt@no{113}}%
\qt@newcmd\qurantrans@de@dcvii{Nichts Gutes liegt in einem großen Teil ihrer vertraulichen Gespräche, es sei denn, wenn einer zu einem Almosen oder zu einer rechten Tat oder zur Aussöhnung zwischen den Menschen auffordert. Und wer dies im Streben nach dem Wohlgefallen Gottes tut, dem werden Wir einen großartigen Lohn zukommen lassen.\qt@no{114}}%
\qt@newcmd\qurantrans@de@dcviii{Wer sich dem Gesandten widersetzt, nachdem ihm die Rechtleitung deutlich geworden ist, und einem anderen Weg als dem der Gläubigen folgt, den lassen Wir verfolgen, was er verfolgt hat, und in der Hölle brennen - welch schlimmes Ende!\qt@no{115}}%
\qt@newcmd\qurantrans@de@dcix{Gott vergibt nicht, daß Ihm (etwas) beigesellt wird, und Er vergibt, was darunter liegt, wem Er will. Und wer Gott (andere) beigesellt, der ist weit abgeirrt.\qt@no{116}}%
\qt@newcmd\qurantrans@de@dcx{Sie rufen ja statt Seiner weibliche Wesen an, sie rufen ja einen rebellischen Satan an,\qt@no{117}}%
\qt@newcmd\qurantrans@de@dcxi{Den Gott verfluchte und der sagte: «Ich werde mir von deinen Dienern einen festgesetzten Anteil nehmen,\qt@no{118}}%
\qt@newcmd\qurantrans@de@dcxii{Und ich werde sie irreführen, und ich werde sie Wünschen nachjagen lassen, und ich werde ihnen befehlen, und sie werden die Ohren der Herdentiere abschneiden, und ich werde ihnen befehlen, und sie werden die Schöpfung Gottes verändern.» Und wer sich den Satan anstelle Gottes zum Freund nimmt, der wird einen offenkundigen Verlust erleiden.\qt@no{119}}%
\qt@newcmd\qurantrans@de@dcxiii{Er macht ihnen Versprechungen und läßt sie Wünschen nachjagen. Und der Satan verspricht ihnen nur Betörung.\qt@no{120}}%
\qt@newcmd\qurantrans@de@dcxiv{Diese haben die Hölle zur Heimstätte, und sie werden kein Mittel finden, ihr zu entrinnen.\qt@no{121}}%
\qt@newcmd\qurantrans@de@dcxv{Diejenigen, die glauben und die guten Werke tun, werden Wir in Gärten eingehen lassen, unter denen Bäche fließen; darin werden sie auf immer ewig weilen. Das ist das Versprechen Gottes in Wahrheit, und wer ist wahrhaftiger als Gott in seiner Aussage?\qt@no{122}}%
\qt@newcmd\qurantrans@de@dcxvi{Weder nach euren Wünschen noch nach den Wünschen der Leute des Buches geht es. Wer Böses tut, dem wird danach vergolten, und er wird für sich anstelle Gottes weder Freund noch Helfer finden.\qt@no{123}}%
\qt@newcmd\qurantrans@de@dcxvii{Diejenigen, die etwas von den guten Werken tun, ob Mann oder Weib, und dabei gläubig sind, werden ins Paradies eingehen, und ihnen wird nicht ein Dattelgrübchen Unrecht getan.\qt@no{124}}%
\qt@newcmd\qurantrans@de@dcxviii{Und wer hat eine schönere Religion als der, der sich völlig Gott hingibt und dabei rechtschaffen ist und der Glaubensrichtung Abrahams, als Anhänger des reinen Glaubens, folgt? Gott hat sich Abraham ja zum Vertrauten genommen.\qt@no{125}}%
\qt@newcmd\qurantrans@de@dcxix{Und Gott gehört, was in den Himmeln und was auf der Erde ist. Und Gott umfaßt alle Dinge.\qt@no{126}}%
\qt@newcmd\qurantrans@de@dcxx{Sie fragen dich um Rechtsauskunft über die Frauen. Sprich: Rechtsauskunft über sie gibt euch Gott, und auch das, was euch im Buch verlesen wird über die weiblichen Waisen, denen ihr nicht das zukommen laßt, was ihnen vorschriftsgemäß zusteht, und die ihr nicht heiraten wollt, und auch über die von den Kindern, die wie Schwache behandelt werden; und ihr sollt für die Gerechtigkeit gegenüber den Waisen eintreten. Was ihr an Gutem tut, Gott weiß es.\qt@no{127}}%
\qt@newcmd\qurantrans@de@dcxxi{Und wenn eine Frau von ihrem Gemahl Widerspenstigkeit oder Abwendung befürchtet, so ist es für sie beide kein Vergehen, untereinander Aussöhnung zu schaffen, und die Aussöhnung ist besser. Und die Menschen sind dem Geiz ständig verfallen. Und wenn ihr rechtschaffen und gottesfürchtig seid, so hat Gott Kenntnis von dem, was ihr tut.\qt@no{128}}%
\qt@newcmd\qurantrans@de@dcxxii{Und ihr werdet es nicht schaffen, die Frauen gleich zu behandeln, ihr mögt euch noch so sehr bemühen. Aber wendet euch nicht (von der einen) gänzlich ab, so daß ihr sie in der Schwebe laßt. Und wenn ihr nach Aussöhnung strebt und Gott fürchtet, so ist Gott voller Vergebung und barmherzig.\qt@no{129}}%
\qt@newcmd\qurantrans@de@dcxxiii{Und wenn die beiden sich trennen, wird Gott aus seinem umfassenden Reichtum jeden unabhängig machen. Und Gott umfaßt alles und ist weise.\qt@no{130}}%
\qt@newcmd\qurantrans@de@dcxxiv{Und Gott gehört, was in den Himmeln und auf der Erde ist. Wir haben denen, denen das Buch vor euch zugekommen ist, und auch euch aufgetragen: Fürchtet Gott. Wenn ihr ungläubig seid, so gehört Gott doch (alles), was in den Himmeln und auf der Erde ist. Gott ist auf niemanden angewiesen und des Lobes würdig.\qt@no{131}}%
\qt@newcmd\qurantrans@de@dcxxv{Und Gott gehört, was in den Himmeln und was auf der Erde ist. Gott genügt als Sachwalter.\qt@no{132}}%
\qt@newcmd\qurantrans@de@dcxxvi{Wenn Er will, läßt Er euch, ihr Menschen, fortgehen und andere nachrücken. Gott hat dazu die Macht.\qt@no{133}}%
\qt@newcmd\qurantrans@de@dcxxvii{Wenn einer den Lohn des Diesseits will, so besteht bei Gott der Lohn des Diesseits und des Jenseits. Gott hört und sieht alles.\qt@no{134}}%
\qt@newcmd\qurantrans@de@dcxxviii{O ihr, die ihr glaubt, tretet für die Gerechtigkeit ein und legt Zeugnis für Gott ab, auch wenn es gegen euch selbst oder gegen die Eltern und die Angehörigen sein sollte. Wenn es sich um einen Reichen oder einen Armen handelt, so hat Gott eher Anspruch auf beide; folgt also nicht (euren) Neigungen, anstatt gerecht zu sein. Wenn ihr (das Zeugnis) verdreht oder davon ablaßt, so hat Gott Kenntnis von dem, was ihr tut.\qt@no{135}}%
\qt@newcmd\qurantrans@de@dcxxix{O ihr, die ihr glaubt, glaubt an Gott und seinen Gesandten, und das Buch, das Er auf seinen Gesandten herabgesandt hat, und das Buch, das Er zuvor herabgesandt hat. Wer Gott verleugnet, und seine Engel, seine Bücher, seine Gesandten und den Jüngsten Tag, der ist weit abgeirrt.\qt@no{136}}%
\qt@newcmd\qurantrans@de@dcxxx{Denen, die glauben und dann ungläubig werden, dann wieder glauben und dann wieder ungläubig werden und dann im Unglauben zunehmen, denen wird Gott unmöglich vergeben, und Er wird sie unmöglich einen rechten Weg führen.\qt@no{137}}%
\qt@newcmd\qurantrans@de@dcxxxi{Verkünde den Heuchlern, daß für sie eine schmerzhafte Pein bestimmt ist,\qt@no{138}}%
\qt@newcmd\qurantrans@de@dcxxxii{sie), die sich die Ungläubigen anstelle der Gläubigen zu Freunden nehmen. Suchen sie denn bei ihnen die Macht? Alle Macht gehört Gott.\qt@no{139}}%
\qt@newcmd\qurantrans@de@dcxxxiii{Er hat auf euch im Buch herabgesandt, ihr sollt, wenn ihr hört, daß Leute die Zeichen Gottes verleugnen und über sie spotten, euch nicht zu ihnen setzen, bis sie auf ein anderes Gespräch eingehen. Sonst seid ihr ihnen gleich. Gott wird die Heuchler und die Ungläubigen allesamt in der Hölle versammeln,\qt@no{140}}%
\qt@newcmd\qurantrans@de@dcxxxiv{Die euch gegenüber ja abwarten: Wenn euch ein Erfolg von Gott her zufällt, sagen sie: «Sind wir nicht mit euch gewesen?»; und wenn den Ungläubigen ein Glück zufällt, sagen sie: «Haben wir euch nicht in der Hand gehabt, euch aber vor den Gläubigen geschützt?» Gott wird am Tag der Auferstehung zwischen euch urteilen. Und Gott wird nie den Ungläubigen eine Möglichkeit geben, gegen die Gläubigen vorzugehen.\qt@no{141}}%
\qt@newcmd\qurantrans@de@dcxxxv{Die Heuchler versuchen, Gott zu betrügen; Er ist es aber, der sie betrügt. Und wenn sie sich zum Gebet hinstellen, stellen sie sich nachlässig hin, wobei sie von den Menschen gesehen werden wollen, und sie gedenken Gottes nur wenig.\qt@no{142}}%
\qt@newcmd\qurantrans@de@dcxxxvi{Sie schwanken dazwischen und schließen sich weder diesen noch jenen an. Und wen Gott irreführt, für den wirst du keinen Weg finden.\qt@no{143}}%
\qt@newcmd\qurantrans@de@dcxxxvii{O ihr, die ihr glaubt, nehmt euch nicht die Ungläubigen anstelle der Gläubigen zu Freunden. Wollt ihr denn Gott eine offenkundige Handhabe gegen euch liefern?\qt@no{144}}%
\qt@newcmd\qurantrans@de@dcxxxviii{Die Heuchler befinden sich im untersten Grund des Feuers, und du wirst für sie keinen Helfer finden,\qt@no{145}}%
\qt@newcmd\qurantrans@de@dcxxxix{Außer denen, die umkehren und Besserung zeigen, an Gott festhalten und gegenüber Gott aufrichtig in ihrer Religion sind. Jene zählen zu den Gläubigen. Und Gott wird den Gläubigen einen großartigen Lohn zukommen lassen.\qt@no{146}}%
\qt@newcmd\qurantrans@de@dcxl{Warum sollte Gott euch peinigen, wenn ihr dankbar und gläubig seid? Und Gott zeigt sich erkenntlich und weiß Bescheid.\qt@no{147}}%
\qt@newcmd\qurantrans@de@dcxli{Gott liebt es nicht, daß jemand über das Böse öffentlich redet, es sei denn, es wurde ihm Unrecht getan. Gott hört und weiß alles.\qt@no{148}}%
\qt@newcmd\qurantrans@de@dcxlii{Ob ihr etwas Gutes offen zeigt oder geheimhaltet oder etwas Böses verzeiht, Gott ist voller Verzeihung und mächtig.\qt@no{149}}%
\qt@newcmd\qurantrans@de@dcxliii{Diejenigen, die Gott und seine Gesandten verleugnen und zwischen Gott und seinen Gesandten unterscheiden wollen und sagen: «Wir glauben an die einen, verleugnen aber die anderen», und einen Weg dazwischen einschlagen wollen,\qt@no{150}}%
\qt@newcmd\qurantrans@de@dcxliv{Das sind die wahren Ungläubigen. Und Wir haben für die Ungläubigen eine schmähliche Pein bereitet.\qt@no{151}}%
\qt@newcmd\qurantrans@de@dcxlv{Denen, die an Gott und seine Gesandten glauben und bei keinem von ihnen einen Unterschied machen, wird Er ihren Lohn zukommen lassen. Gott ist voller Vergebung und barmherzig.\qt@no{152}}%
\qt@newcmd\qurantrans@de@dcxlvi{Die Leute des Buches fordern von dir, daß du auf sie ein Buch vom Himmel herabsenden läßt. Sie haben von Mose etwas noch Größeres als dieses gefordert und gesagt: «Laß uns Gott offen sehen.» Da ergriff sie der Donnerschlag wegen ihrer Ungerechtigkeit. Dann nahmen sie sich das Kalb, nachdem die deutlichen Zeichen zu ihnen gekommen waren. Wir verziehen das, und Wir gaben Mose eine offenkundige Ermächtigung.\qt@no{153}}%
\qt@newcmd\qurantrans@de@dcxlvii{Und Wir hoben den Berg über sie bei (der Entgegennahme) ihrer Verpflichtung empor und sprachen zu ihnen: «Betretet das Tor in der Haltung der Niederwerfung.» Und Wir sprachen zu ihnen: «Begeht am Sabbat keine Übertretungen.» Und Wir nahmen von ihnen eine feste Verpflichtung entgegen.\qt@no{154}}%
\qt@newcmd\qurantrans@de@dcxlviii{Verflucht wurden sie,) weil sie ihre Verpflichtung brachen, die Zeichen Gottes verleugneten, die Propheten zu Unrecht töteten und sagten: «Unsere Herzen sind unbeschnitten» - vielmehr hat Gott sie wegen ihres Unglaubens versiegelt, so daß sie nur wenig glauben;\qt@no{155}}%
\qt@newcmd\qurantrans@de@dcxlix{Und weil sie ungläubig waren und gegen Maria eine gewaltige Verleumdung aussprachen;\qt@no{156}}%
\qt@newcmd\qurantrans@de@dcl{Und weil sie sagten: «Wir haben Christus Jesus, den Sohn Marias, den Gesandten Gottes, getötet.» - Sie haben ihn aber nicht getötet, und sie haben ihn nicht gekreuzigt, sondern es erschien ihnen eine ihm ähnliche Gestalt. Diejenigen, die über ihn uneins sind, sind im Zweifel über ihn. Sie haben kein Wissen über ihn, außer daß sie Vermutungen folgen. Und sie haben ihn nicht mit Gewißheit getötet,\qt@no{157}}%
\qt@newcmd\qurantrans@de@dcli{Sondern Gott hat ihn zu sich erhoben. Gott ist mächtig und weise.\qt@no{158}}%
\qt@newcmd\qurantrans@de@dclii{Und es gibt keinen unter den Leuten des Buches, der nicht noch vor seinem Tod an ihn glauben würde. Am Tag der Auferstehung wird er über sie Zeuge sein. -\qt@no{159}}%
\qt@newcmd\qurantrans@de@dcliii{Und wegen der Ungerechtigkeit derer, die Juden sind, haben Wir ihnen köstliche Dinge verboten, die ihnen (sonst) erlaubt waren, und weil sie viele vom Weg Gottes nachdrücklich abweisen;\qt@no{160}}%
\qt@newcmd\qurantrans@de@dcliv{Und weil sie Zins nehmen, obwohl er ihnen verboten ist; und weil sie das Vermögen der Menschen durch Betrug verzehren. Und Wir haben den Ungläubigen unter ihnen eine schmerzhafte Pein bereitet.\qt@no{161}}%
\qt@newcmd\qurantrans@de@dclv{Aber diejenigen von ihnen, die im Wissen fest gegründet sind, und auch die Gläubigen glauben an das, was zu dir herabgesandt wurde und was vor dir herabgesandt wurde. Und denen, die das Gebet verrichten und die Abgabe entrichten und an Gott und den Jüngsten Tag glauben, denen werden Wir einen großartigen Lohn zukommen lassen.\qt@no{162}}%
\qt@newcmd\qurantrans@de@dclvi{Wir gaben dir eine Offenbarung, wie Wir Noach und den Propheten nach ihm offenbart haben. Und Wir offenbarten (auch) Abraham, Ismael, Isaak, Jakob und den Stämmen, Jesus, Ijob, Jonas, Aaron und Salomo. Und Wir ließen David eine Schrift zukommen.\qt@no{163}}%
\qt@newcmd\qurantrans@de@dclvii{Und (Wir schickten) Gesandte, von denen Wir dir früher erzählt haben, und (auch) Gesandte, von denen Wir dir nicht erzählt haben - und Gott hat mit Mose wahrhaftig gesprochen -,\qt@no{164}}%
\qt@newcmd\qurantrans@de@dclviii{Gesandte als Freudenboten und Warner, damit die Menschen nach dem Auftreten der Gesandten keinen Beweisgrund gegen Gott haben. Und Gott ist mächtig und weise.\qt@no{165}}%
\qt@newcmd\qurantrans@de@dclix{Gott bezeugt, was Er zu dir herabgesandt hat. Er hat es mit seinem Wissen herabgesandt. Auch die Engel bezeugen es. Und Gott genügt als Zeuge.\qt@no{166}}%
\qt@newcmd\qurantrans@de@dclx{Diejenigen, die ungläubig sind und vom Weg Gottes abweisen, sind weit abgeirrt.\qt@no{167}}%
\qt@newcmd\qurantrans@de@dclxi{Denen, die ungläubig sind und Unrecht tun, wird Gott unmöglich vergeben, und Er wird sie unmöglich einen rechten Weg führen,\qt@no{168}}%
\qt@newcmd\qurantrans@de@dclxii{Es sei denn den Weg zur Hölle, darin werden sie auf immer ewig weilen. Und dies ist Gott ein leichtes.\qt@no{169}}%
\qt@newcmd\qurantrans@de@dclxiii{O ihr Menschen, der Gesandte ist von eurem Herrn mit der Wahrheit zu euch gekommen. So glaubt, das ist besser für euch. Wenn ihr ungläubig bleibt, so gehört Gott doch (alles), was in den Himmeln und auf der Erde ist. Gott weiß Bescheid und ist weise.\qt@no{170}}%
\qt@newcmd\qurantrans@de@dclxiv{O ihr Leute des Buches, übertreibt nicht in eurer Religion und sagt über Gott nur die Wahrheit. Christus Jesus, der Sohn Marias, ist doch nur der Gesandte Gottes und sein Wort, das Er zu Maria hinüberbrachte, und ein Geist von Ihm. So glaubt an Gott und seine Gesandten. Und sagt nicht: Drei. Hört auf, das ist besser für euch. Gott ist doch ein einziger Gott. Preis sei Ihm, und erhaben ist Er darüber, daß Er ein Kind habe. Er hat, was in den Himmeln und was auf der Erde ist. Und Gott genügt als Sachwalter.\qt@no{171}}%
\qt@newcmd\qurantrans@de@dclxv{Christus wird es sicher nicht aus Widerwillen ablehnen, Diener Gottes zu sein, und auch nicht die in die Nähe (Gottes) zugelassenen Engel. Wenn einer es aus Widerwillen ablehnt, Ihm zu dienen, und sich hochmütig zeigt, so wird Gott doch sie allesamt zu sich versammeln.\qt@no{172}}%
\qt@newcmd\qurantrans@de@dclxvi{Denen nun, die glauben und die guten Werke tun, wird Er ihren Lohn voll erstatten, und Er wird ihnen von seiner Huld noch mehr geben. Diejenigen aber, die Widerwillen zeigen und sich hochmütig verhalten, wird Er mit einer schmerzhaften Pein peinigen. Und sie werden für sich anstelle Gottes weder Freund noch Helfer finden.\qt@no{173}}%
\qt@newcmd\qurantrans@de@dclxvii{O ihr Menschen, gekommen ist nunmehr zu euch ein Beweis von eurem Herrn. Und Wir haben zu euch ein offenkundiges Licht hinabgesandt.\qt@no{174}}%
\qt@newcmd\qurantrans@de@dclxviii{Diejenigen nun, die an Gott glauben und an Ihm festhalten, wird Er in seine Barmherzigkeit und Huld eingehen lassen und auf einem geraden Weg zu sich führen.\qt@no{175}}%
\qt@newcmd\qurantrans@de@dclxix{Sie fragen dich um Rechtsauskunft. Sprich: Gott gibt euch eine Rechtsauskunft über die seitliche Verwandtschaft. Wenn jemand stirbt, ohne ein Kind zu haben, und er eine Schwester hat, dann steht ihr die Hälfte dessen, was er hinterläßt, zu. Und er beerbt sie, wenn sie kein Kind hat. Wenn es zwei (Schwestern) sind, stehen ihnen zwei Drittel dessen, was er hinterläßt, zu. Wenn es Geschwister sind, Männer und Frauen, dann steht dem männlichen Geschwister soviel wie der Anteil von zwei weiblichen zu. Gott macht es euch deutlich, damit ihr nicht irregeht. Und Gott weiß über alle Dinge Bescheid.\qt@no{176}}%
\qt@newcmd\qurantrans@de@dclxx{\basmalahde O ihr, die ihr glaubt, erfüllt die Verträge. Erlaubt ist euch das Herdenvieh außer dem, was euch bekanntgemacht wird; und ihr sollt, während ihr im Weihezustand seid, das Jagdwild nicht für erlaubt halten. Gott entscheidet, was Er will.\qt@no{1}}%
\qt@newcmd\qurantrans@de@dclxxi{O ihr, die ihr glaubt, verletzt den sakralen Charakter weder der Kultzeichen Gottes noch des heiligen Monats, noch der Opfertiere und der Halsgehänge noch derer, die sich zum heiligen Haus begeben im Streben nach Huld und Wohlgefallen von ihrem Herrn. Wenn ihr den Weihezustand abgelegt habt, dann dürft ihr jagen. Und der Haß gegen bestimmte Leute, weil sie euch von der heiligen Moschee abgewiesen haben, soll euch nicht dazu verleiten, Übertretungen zu begehen. Helft einander zur Frömmigkeit und Gottesfurcht, und helft einander nicht zur Sünde und Übertretung. Und fürchtet Gott. Gott verhängt eine harte Strafe.\qt@no{2}}%
\qt@newcmd\qurantrans@de@dclxxii{Verboten ist euch Verendetes, Blut, Schweinefleisch und das, worüber ein anderer als Gott angerufen worden ist, und Ersticktes, Erschlagenes, Gestürztes, Gestoßenes und das, was ein wildes Tier angefressen hat - ausgenommen das, was ihr schachtet -, und das, was auf Opfersteinen geschlachtet worden ist. Und (verboten ist) auch, daß ihr mit Pfeilen das Los werft. Das ist Frevel. Heute sind diejenigen, die ungläubig sind, an eurer Religion verzweifelt, so fürchtet sie nicht, sondern fürchtet Mich. Heute habe Ich euch eure Religion vervollkommnet und meine Gnade an euch vollendet, und Ich habe daran Gefallen, daß der Islam eure Religion sei. Wenn aber einer aus Hunger gezwungen wird, ohne zu einer Sünde hinzuneigen, so ist Gott voller Vergebung und barmherzig.\qt@no{3}}%
\qt@newcmd\qurantrans@de@dclxxiii{Sie fragen dich, was ihnen erlaubt ist. Sprich: Erlaubt sind euch die köstlichen Dinge. Wenn ihr Jagdtiere abrichtet, indem ihr sie von dem lehrt, was Gott euch gelehrt hat, dann eßt von dem, was sie für euch fassen, und erwähnt den Namen Gottes darüber. Und fürchtet Gott. Gott ist schnell im Abrechnen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@dclxxiv{Heute sind euch die köstlichen Dinge erlaubt. Die Speise derer, denen das Buch zugekommen ist, ist euch erlaubt, und eure Speise ist ihnen erlaubt. (Erlaubt sind) auch die unter Schutz gestellten gläubigen Frauen und die unter Schutz gestellten Frauen aus den Reihen derer, denen vor euch das Buch zugekommen ist, wenn ihr ihnen ihren Lohn zukommen laßt und mit ihnen in der Absicht lebt, (sie) unter Schutz zu stellen, nicht Unzucht zu treiben und (sie) nicht als heimliche Konkubinen zu nehmen. Und wer den Glauben leugnet, dessen Werk ist wertlos, und im Jenseits gehört er zu den Verlierern.\qt@no{5}}%
\qt@newcmd\qurantrans@de@dclxxv{O ihr, die ihr glaubt, wenn ihr euch zum Gebet hinstellt, so wascht (vorher) euer Gesicht und eure Hände bis zu den Ellbogen und streicht euch über den Kopf, und (wascht) eure Füße bis zu den Knöcheln. Und wenn ihr sexuell verunreinigt seid, dann reinigt euch. Und wenn ihr krank oder auf Reisen seid, oder wenn einer von euch vom Abort kommt, oder wenn ihr die Frauen berührt habt und ihr kein Wasser findet, dann sucht einen sauberen Boden und streicht euch davon über das Gesicht und die Hände. Gott will euch keine Bedrängnis auferlegen, sondern Er will euch rein machen und seine Gnade an euch vollenden, auf daß ihr dankbar seid.\qt@no{6}}%
\qt@newcmd\qurantrans@de@dclxxvi{Und gedenket der Gnade Gottes zu euch und der Verpflichtung, durch die Er euch gebunden hat, als ihr sagtet: «Wir hören und wir gehorchen.» Und fürchtet Gott. Gott weiß über das innere Geheimnis Bescheid.\qt@no{7}}%
\qt@newcmd\qurantrans@de@dclxxvii{O ihr, die ihr glaubt, tretet für Gott ein und legt Zeugnis für die Gerechtigkeit ab. Und der Haß gegen bestimmte Leute soll euch nicht dazu verleiten, nicht gerecht zu sein. Seid gerecht, das entspricht eher der Gottesfurcht. Und fürchtet Gott. Gott hat Kenntnis von dem, was ihr tut.\qt@no{8}}%
\qt@newcmd\qurantrans@de@dclxxviii{Gott hat denen, die glauben und die guten Werke tun, versprochen: Bestimmt ist für sie Vergebung und großartiger Lohn.\qt@no{9}}%
\qt@newcmd\qurantrans@de@dclxxix{Und diejenigen, die nicht glauben und unsere Zeichen für Lüge erklären, das sind die Gefährten der Hölle.\qt@no{10}}%
\qt@newcmd\qurantrans@de@dclxxx{O ihr, die ihr glaubt, gedenket der Gnade Gottes zu euch, als gewisse Leute im Begriff waren, ihre Hände nach euch auszustrecken, und Er ihre Hände von euch zurückhielt. Und fürchtet Gott. Auf Gott sollen die Gläubigen vertrauen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@dclxxxi{Gott nahm die Verpflichtung der Kinder Israels entgegen. Und Wir ließen aus ihren Reihen zwölf Vorgesetzte erstehen. Und Gott sprach: «Ich bin mit euch. Wenn ihr das Gebet verrichtet und die Abgabe entrichtet, an meine Gesandten glaubt und ihnen beisteht und Gott ein schönes Darlehn leiht, werde Ich euch eure Missetaten sühnen, und Ich werde euch in Gärten eingehen lassen, unter denen Bäche fließen. Wer von euch hierauf ungläubig wird, der ist vom rechten Weg abgeirrt.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@dclxxxii{Weil sie aber ihre Verpflichtung brachen, haben Wir sie verflucht und ihre Herzen verstockt gemacht. Sie entstellen den Sinn der Worte. Und sie vergaßen einen Teil von dem, womit sie ermahnt worden waren. Und du wirst immer wieder Verrat von ihrer Seite erfahren - bis auf wenige von ihnen. Aber verzeih ihnen und laß es ihnen nach. Gott liebt die Rechtschaffenen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@dclxxxiii{Und von denen, die sagen: «Wir sind Christen», nahmen Wir ihre Verpflichtung entgegen. Sie vergaßen einen Teil von dem, womit sie ermahnt worden waren. So erregten Wir unter ihnen Feindschaft und Haß bis zum Tag der Auferstehung. Gott wird ihnen kundtun, was sie zu machen pflegten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@dclxxxiv{O ihr Leute des Buches, unser Gesandter ist nunmehr zu euch gekommen, um euch vieles von dem, was ihr vom Buch geheimgehalten habt, deutlich zu machen und um vieles zu übergehen. Gekommen ist zu euch von Gott ein Licht und ein offenkundiges Buch,\qt@no{15}}%
\qt@newcmd\qurantrans@de@dclxxxv{Mit dem Gott diejenigen, die seinem Wohlgefallen nachgehen, die Wege des Friedens leitet und sie aus den Finsternissen ins Licht herausbringt mit seiner Erlaubnis; und Er leitet sie zu einem geraden Weg.\qt@no{16}}%
\qt@newcmd\qurantrans@de@dclxxxvi{Ungläubig sind gewiß diejenigen, die sagen: «Gott ist Christus, der Sohn Marias.» Sprich: Wer vermag denn gegen Gott überhaupt etwas auszurichten, wenn Er Christus, den Sohn Marias, und seine Mutter und diejenigen, die auf der Erde sind, allesamt verderben lassen will? Gott gehört die Königsherrschaft der Himmel und der Erde und dessen, was dazwischen ist. Er erschafft, was Er will. Und Gott hat Macht zu allen Dingen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@dclxxxvii{Die Juden und die Christen sagen: «Wir sind die Söhne Gottes und seine Lieblinge.» Sprich: Warum peinigt Er euch dann für eure Sünden? Nein, ihr seid Menschen von denen, die Er erschaffen hat. Er vergibt, wem Er will, und Er peinigt, wen Er will. Und Gott gehört die Königsherrschaft der Himmel und der Erde und dessen, was dazwischen ist. Zu Ihm führt der Lebensweg.\qt@no{18}}%
\qt@newcmd\qurantrans@de@dclxxxviii{O ihr Leute des Buches, unser Gesandter ist nunmehr zu euch gekommen, um euch in einer Zeit, in der die Propheten ausgeblieben sind, Klarheit zu bringen, damit ihr nicht sagt: «Zu uns ist kein Freudenbote und kein Warner gekommen.» Zu euch ist ein Freudenbote und ein Warner gekommen. Und Gott hat Macht zu allen Dingen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@dclxxxix{Und als Mose zu seinem Volk sagte: «Mein Volk, gedenket der Gnade Gottes zu euch, als Er in eurer Mitte Propheten einsetzte und euch zu Königen machte und euch zukommen ließ, was Er niemandem von den Weltenbewohnern hat zukommen lassen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@dcxc{O mein Volk, tretet in das heilige Land ein, das Gott euch bestimmt hat. Und kehrt nicht den Rücken, sonst würdet ihr als Verlierer umkehren.»\qt@no{21}}%
\qt@newcmd\qurantrans@de@dcxci{Sie sagten: «O Mose, in ihm sind gewaltige Leute. Wir werden es nie betreten, solange sie nicht aus ihm herausgehen. Wenn sie aus ihm herausgehen, dann treten wir ein.»\qt@no{22}}%
\qt@newcmd\qurantrans@de@dcxcii{Zwei Männer von den Gottesfürchtigen, welche Gott begnadet hatte, sagten: «Tretet durch das Tor gegen sie ein; wenn ihr dadurch eintretet, werdet ihr Sieger sein. Und auf Gott sollt ihr vertrauen, so ihr Gläubige seid.»\qt@no{23}}%
\qt@newcmd\qurantrans@de@dcxciii{Sie sagten: «O Mose, wir werden es nie und nimmer betreten, solange sie darin sind. Geh du und dein Herr und kämpft. Wir bleiben hier sitzen.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@dcxciv{Er sagte: «Mein Herr, ich habe nur mich selbst und meinen Bruder. So scheide uns von den frevlerischen Leuten.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@dcxcv{Er sprach: «Verwehrt sei es ihnen vierzig Jahre, in denen sie auf der Erde umherziehen. So sei nicht betrübt über die frevlerischen Leute.»\qt@no{26}}%
\qt@newcmd\qurantrans@de@dcxcvi{Und verlies ihnen den Bericht über die zwei Söhne Adams der Wahrheit entsprechend. Als sie ein Opfer darbrachten. Es wurde von dem einen angenommen und von dem anderen nicht angenommen. Der sagte: «Ich schlage dich tot.» Er sagte: «Gott nimmt es an nur von den Gottesfürchtigen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@dcxcvii{Auch wenn du deine Hand nach mir ausstreckst, um mich zu töten, so werde ich meine Hand nicht nach dir ausstrecken, um dich zu töten. Ich fürchte Gott, den Herrn der Welten.\qt@no{28}}%
\qt@newcmd\qurantrans@de@dcxcviii{Ich will, daß du meine Sünde und deine Sünde auf dich lädst und so zu den Gefährten des Feuers gehörst. Das ist die Vergeltung für die, die Unrecht tun.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@dcxcix{Seine Seele machte ihn willig, seinen Bruder zu töten. Er tötete ihn und wurde einer der Verlierer.\qt@no{30}}%
\qt@newcmd\qurantrans@de@dcc{Gott schickte einen Raben, der in der Erde scharrte, um ihm zu zeigen, wie er die Leiche seines Bruders bedecken könne. Er sagte: «Wehe mir! Bin ich nicht fähig, wie dieser Rabe zu sein und die Leiche meines Bruders zu bedecken?» So wurde er einer von denen, die bereuen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@dcci{Aus diesem Grund haben Wir den Kindern Israels vorgeschrieben: Wenn einer jemanden tötet, jedoch nicht wegen eines Mordes oder weil er auf der Erde Unheil stiftet, so ist es, als hätte er die Menschen alle getötet. Und wenn jemand ihn am Leben erhält, so ist es, als hätte er die Menschen alle am Leben erhalten. Unsere Gesandten kamen zu ihnen mit den deutlichen Zeichen. Aber viele von ihnen verhalten sich nach alledem maßlos auf der Erde.\qt@no{32}}%
\qt@newcmd\qurantrans@de@dccii{Die Vergeltung für die, die gegen Gott und seinen Gesandten Krieg führen und auf der Erde umherreisen, um Unheil zu stiften, soll dies sein, daß sie getötet oder gekreuzigt werden, oder daß ihnen Hände und Füße wechselseitig abgehackt werden, oder daß sie aus dem Land verbannt werden. Das ist für sie eine Schande im Diesseits, und im Jenseits ist für sie eine gewaltige Pein bestimmt,\qt@no{33}}%
\qt@newcmd\qurantrans@de@dcciii{Außer denen, die umkehren, bevor ihr euch ihrer bemächtigt. Und wißt, daß Gott voller Vergebung und barmherzig ist.\qt@no{34}}%
\qt@newcmd\qurantrans@de@dcciv{O ihr, die ihr glaubt, fürchtet Gott und sucht ein Mittel, zu Ihm zu gelangen, und setzt euch auf seinem Weg ein, auf daß es euch wohl ergehe.\qt@no{35}}%
\qt@newcmd\qurantrans@de@dccv{Wenn diejenigen, die ungläubig sind, alles hätten, was auf der Erde ist, und noch einmal das gleiche dazu, um sich damit von der Pein des Tages der Auferstehung loszukaufen, es würde nicht von ihnen angenommen werden. Und bestimmt ist für sie eine schmerzhafte Pein.\qt@no{36}}%
\qt@newcmd\qurantrans@de@dccvi{Sie wollen aus dem Feuer herauskommen. Sie können aber nicht aus ihm herauskommen. Und bestimmt ist für sie eine beständige Pein.\qt@no{37}}%
\qt@newcmd\qurantrans@de@dccvii{Und hackt dem Dieb und der Diebin die Hände ab zur Vergeltung für das, was sie erworben haben, dies als abschreckende Strafe von Seiten Gottes. Und Gott ist mächtig und weise.\qt@no{38}}%
\qt@newcmd\qurantrans@de@dccviii{Wenn aber einer, nachdem er Unrecht getan hat, umkehrt und Besserung zeigt, wird Gott sich gewiß ihm zuwenden. Gott ist ja voller Vergebung und barmherzig.\qt@no{39}}%
\qt@newcmd\qurantrans@de@dccix{Weißt du nicht, daß Gott die Königsherrschaft der Himmel und der Erde gehört? Er peinigt, wen Er will, und Er vergibt, wem Er will. Und Gott hat Macht zu allen Dingen.\qt@no{40}}%
\qt@newcmd\qurantrans@de@dccx{O Gesandter, laß dich nicht durch die betrüben, die im Unglauben miteinander wetteifern, aus den Reihen derer, die mit dem Mund sagen: «Wir glauben», während ihre Herzen nicht glauben. Unter denen, die Juden sind, gibt es welche, die auf Lügen hören und auf andere Leute, die nicht zu dir gekommen sind, hören. Sie entstellen den Sinn der Worte und sagen: «Wenn euch dies gebracht wird, nehmt es an; wenn es euch aber nicht gebracht wird, dann seid auf der Hut.» Wen Gott der Versuchung preisgeben will, für den vermagst du gegen Gott überhaupt nichts auszurichten. Das sind die, deren Herzen Gott nicht rein machen will. Bestimmt ist für sie im Diesseits Schande und im Jenseits eine gewaltige Pein.\qt@no{41}}%
\qt@newcmd\qurantrans@de@dccxi{Sie hören auf Lügen, und sie verzehren unrechtmäßig erworbenes Gut. Wenn sie zu dir kommen, so urteile zwischen ihnen oder wende dich von ihnen ab. Wenn du dich von ihnen abwendest, werden sie dir nichts schaden; wenn du urteilst, dann urteile zwischen ihnen nach Gerechtigkeit. Gott liebt die, die gerecht handeln.\qt@no{42}}%
\qt@newcmd\qurantrans@de@dccxii{Wie können sie dich zum Schiedsrichter machen, wo sie doch die Tora besitzen, in der das Urteil Gottes enthalten ist, und sich hierauf nach alledem abkehren? Diese sind keine (richtigen) Gläubigen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@dccxiii{Wir haben die Tora hinabgesandt, in der Rechtleitung und Licht enthalten sind, damit die Propheten, die gottergeben waren, für die, die Juden sind, (danach) urteilen, und so auch die Rabbiner und die Gelehrten, aufgrund dessen, was ihnen vom Buche Gottes anvertraut wurde und worüber sie Zeugen waren. So fürchtet nicht die Menschen, sondern fürchtet Mich. Und verkauft nicht meine Zeichen für einen geringen Preis. Diejenigen, die nicht nach dem urteilen, was Gott herabgesandt hat, das sind die Ungläubigen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@dccxiv{Und wir haben ihnen darin vorgeschrieben: Leben um Leben, Auge um Auge, Nase um Nase, Ohr um Ohr, Zahn um Zahn; und auch für Verwundungen gilt die Wiedervergeltung. Wer aber dies als Almosen erläßt, dem ist es eine Sühne. Diejenigen, die nicht nach dem urteilen, was Gott herabgesandt hat, das sind die, die Unrecht tun.\qt@no{45}}%
\qt@newcmd\qurantrans@de@dccxv{Und Wir ließen nach ihnen Jesus, den Sohn Marias, folgen, damit er bestätige, was von der Tora vor ihm vorhanden war. Und Wir ließen ihm das Evangelium zukommen, das Rechtleitung und Licht enthält und das bestätigt, was von der Tora vor ihm vorhanden war, und als Rechtleitung und Ermahnung für die Gottesfürchtigen.\qt@no{46}}%
\qt@newcmd\qurantrans@de@dccxvi{Die Leute des Evangeliums sollen nach dem urteilen, was Gott darin herabgesandt hat. Und diejenigen, die nicht nach dem urteilen, was Gott herabgesandt hat, das sind die Frevler.\qt@no{47}}%
\qt@newcmd\qurantrans@de@dccxvii{Und Wir haben zu dir das Buch mit der Wahrheit hinabgesandt, damit es bestätige, was vom Buch vor ihm vorhanden war, und alles, was darin steht, fest in der Hand habe. Urteile nun zwischen ihnen nach dem, was Gott herabgesandt hat, und folge nicht ihren Neigungen, damit du nicht von dem abweichst, was von der Wahrheit zu dir gekommen ist. Für jeden von euch haben Wir eine Richtung und einen Weg festgelegt. Und wenn Gott gewollt hätte, hätte Er euch zu einer einzigen Gemeinschaft gemacht. Doch will Er euch prüfen in dem, was Er euch hat zukommen lassen. So eilt zu den guten Dingen um die Wette. Zu Gott werdet ihr allesamt zurückkehren, dann wird Er euch das kundtun, worüber ihr uneins waret.\qt@no{48}}%
\qt@newcmd\qurantrans@de@dccxviii{Und urteile zwischen ihnen nach dem, was Gott herabgesandt hat, und folge nicht ihren Neigungen. Und hüte dich vor ihnen, daß sie dich nicht verführen und abweichen lassen von einem Teil dessen, was Wir zu dir hinabgesandt haben. Wenn sie sich abkehren, so wisse, daß Gott sie wegen eines Teiles ihrer Sünden treffen will. Und gewiß sind viele von den Menschen Frevler.\qt@no{49}}%
\qt@newcmd\qurantrans@de@dccxix{Wünschen sie etwa die Urteilsnorm der Zeit der Unwissenheit? Wer hat denn eine bessere Urteilsnorm als Gott für Leute, die Gewißheit hegen?\qt@no{50}}%
\qt@newcmd\qurantrans@de@dccxx{O ihr, die ihr glaubt, nehmt euch nicht die Juden und die Christen zu Freunden. Sie sind untereinander Freunde. Wer von euch sie zu Freunden nimmt, gehört zu ihnen. Gott leitet ungerechte Leute gewiß nicht recht.\qt@no{51}}%
\qt@newcmd\qurantrans@de@dccxxi{Und du siehst, daß diejenigen, in deren Herzen Krankheit ist, sich eilig um sie bemühen; sie sagen: «Wir fürchten, daß uns eine Schicksalswende trifft.» Möge Gott den Erfolg oder einen Befehl von sich her bringen, so daß sie bereuen, was sie in ihrem Inneren geheimhalten!\qt@no{52}}%
\qt@newcmd\qurantrans@de@dccxxii{Und diejenigen, die glauben, werden sagen: «Sind das die, die bei Gott ihren eifrigsten Eid geschworen haben, sie seien auf eurer Seite?» Ihre Werke sind wertlos, so daß sie nun zu Verlierern geworden sind.\qt@no{53}}%
\qt@newcmd\qurantrans@de@dccxxiii{O ihr, die ihr glaubt, wenn einer von euch von seiner Religion abfällt, so wird Gott (anstelle der Abgefallenen) Leute bringen, die Er liebt und die Ihn lieben, die den Gläubigen gegenüber sich umgänglich zeigen, den Ungläubigen gegenüber aber mit Kraft auftreten, die sich auf dem Weg Gottes einsetzen und den Tadel des Tadelnden nicht fürchten. Das ist die Huld Gottes; Er läßt sie zukommen, wem Er will. Gott umfaßt und weiß alles.\qt@no{54}}%
\qt@newcmd\qurantrans@de@dccxxiv{Euer Freund ist Gott, und sein Gesandter, und auch diejenigen, die glauben, die das Gebet verrichten und die Abgabe entrichten, während sie sich verneigen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@dccxxv{Wer sich Gott und seinen Gesandten und diejenigen, die glauben, zu Freunden nimmt (gehört zu ihnen): Die Partei Gottes sind die Obsiegenden.\qt@no{56}}%
\qt@newcmd\qurantrans@de@dccxxvi{O ihr, die ihr glaubt, nehmt euch aus den Reihen derer, denen das Buch vor euch zugekommen ist, nicht diejenigen, die eure Religion zum Gegenstand von Spott und Spiel nehmen, und auch nicht die Ungläubigen zu Freunden. Und fürchtet Gott, so ihr gläubig seid.\qt@no{57}}%
\qt@newcmd\qurantrans@de@dccxxvii{Wenn ihr zum Gebet ruft, nehmen sie es zum Gegenstand von Spott und Spiel. Dies, weil sie Leute sind, die keinen Verstand haben.\qt@no{58}}%
\qt@newcmd\qurantrans@de@dccxxviii{Sprich: O ihr Leute des Buches, was anderes läßt euch uns grollen, als daß wir an Gott glauben und an das, was zu uns herabgesandt wurde, und das, was zuvor herabgesandt wurde, und daß die meisten von euch Frevler sind?\qt@no{59}}%
\qt@newcmd\qurantrans@de@dccxxix{Sprich: Soll ich euch kundtun, was schlimmer ist als dies in bezug auf die Belohnung bei Gott? Diejenigen, die Gott verflucht hat und denen Er zürnt und aus deren Reihen Er einige zu Affen und Schweinen gemacht hat, und die den Götzen dienen, diese befinden sich in einer schlimmeren Lage und sind vom rechten Weg weiter abgeirrt.\qt@no{60}}%
\qt@newcmd\qurantrans@de@dccxxx{Und wenn sie zu euch kommen, sagen sie: «Wir glauben.» Aber sie treten als Ungläubige ein und gehen als solche hinaus. Und Gott weiß besser Bescheid über das, was sie verschweigen.\qt@no{61}}%
\qt@newcmd\qurantrans@de@dccxxxi{Und du siehst, daß viele von ihnen in Sünde und Übertretung und im Verzehren des unrechtmäßig Erworbenen wetteifern. Schlimm ist das, was sie zu tun pflegen.\qt@no{62}}%
\qt@newcmd\qurantrans@de@dccxxxii{Würden doch die Rabbiner und die Gelehrten ihnen die sündhaften Aussagen und das Verzehren des unrechtmäßig Erworbenen verbieten! Ja, schlimm ist das, was sie zu machen pflegen.\qt@no{63}}%
\qt@newcmd\qurantrans@de@dccxxxiii{Und die Juden sagen: «Die Hand Gottes ist gefesselt.» Ihre Hände seien gefesselt und sie seien verflucht für das, was sie sagen! Nein, seine Hände sind ausgebreitet, und Er spendet, wie Er will. Und was zu dir von deinem Herrn herabgesandt wurde, wird sicher bei vielen von ihnen das Übermaß ihres Frevels und den Unglauben noch mehren. Und Wir erregten unter ihnen Feindschaft und Haß bis zum Tag der Auferstehung. Sooft sie ein Feuer zum Krieg entfachen, löscht Gott es aus. Und sie reisen auf der Erde umher, um Unheil zu stiften. Gott liebt die Unheilstifter nicht.\qt@no{64}}%
\qt@newcmd\qurantrans@de@dccxxxiv{Würden die Leute des Buches glauben und gottesfürchtig sein, Wir würden ihnen ihre Missetaten sühnen und sie in die Gärten der Wonne eingehen lassen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@dccxxxv{Und würden sie die Tora und das Evangelium und das, was zu ihnen von ihrem Herrn herabgesandt wurde, einhalten, sie würden von oben und unter ihren Füßen zu essen bekommen. Unter ihnen gibt es eine Gemeinde mit maßvollem Wandel. Was aber viele von ihnen betrifft, so ist schlimm, was sie tun.\qt@no{66}}%
\qt@newcmd\qurantrans@de@dccxxxvi{O Gesandter, richte aus, was zu dir von deinem Herrn herabgesandt wurde. Wenn du es nicht tust, dann hast du deine Botschaft nicht ausgerichtet. Gott schützt dich vor den Menschen. Gott leitet gewiß die ungläubigen Leute nicht recht.\qt@no{67}}%
\qt@newcmd\qurantrans@de@dccxxxvii{Sprich: O ihr Leute des Buches, ihr entbehrt jeder Grundlage, bis ihr die Tora und das Evangelium und das, was zu euch von eurem Herrn herabgesandt wurde, einhaltet. Und was von deinem Herrn zu dir herabgesandt wurde, wird sicher bei vielen von ihnen das Übermaß ihres Frevels und den Unglauben noch mehren. So sei nicht betrübt über die ungläubigen Leute.\qt@no{68}}%
\qt@newcmd\qurantrans@de@dccxxxviii{Diejenigen, die glauben, und diejenigen, die Juden sind, und die Säbier und die Christen, all die, die an Gott und den Jüngsten Tag glauben und Gutes tun, haben nichts zu befürchten, und sie werden nicht traurig sein.\qt@no{69}}%
\qt@newcmd\qurantrans@de@dccxxxix{Und Wir nahmen die Verpflichtung der Kinder Israels entgegen und entsandten ihnen Gesandte. Jedesmal, wenn ihnen ein Gesandter etwas brachte, was sie nicht mochten, ziehen sie einen Teil (von ihnen) der Lüge und töteten einen (anderen) Teil.\qt@no{70}}%
\qt@newcmd\qurantrans@de@dccxl{Und sie meinten, es käme keine Versuchung. Sie waren blind und taub. Hierauf wandte sich Gott ihnen gnädig zu. Dann wurden sie (wieder) blind und taub, und zwar viele von ihnen. Und Gott sieht wohl, was sie tun.\qt@no{71}}%
\qt@newcmd\qurantrans@de@dccxli{Ungläubig sind diejenigen, die sagen: «Gott ist Christus, der Sohn Marias», wo doch Christus gesagt hat: «O ihr Kinder Israels, dienet Gott, meinem Herrn und eurem Herrn.» Wer Gott (andere) beigesellt, dem verwehrt Gott das Paradies. Seine Heimstätte ist das Feuer. Und die, die Unrecht tun, werden keine Helfer haben.\qt@no{72}}%
\qt@newcmd\qurantrans@de@dccxlii{Ungläubig sind diejenigen, die sagen: «Gott ist der Dritte von dreien», wo es doch keinen Gott gibt außer einem einzigen Gott. Wenn sie mit dem, was sie sagen, nicht aufhören, so wird diejenigen von ihnen, die ungläubig sind, eine schmerzhafte Pein treffen.\qt@no{73}}%
\qt@newcmd\qurantrans@de@dccxliii{Wollen sie sich denn nicht reumütig Gott zuwenden und Ihn um Vergebung bitten? Gott ist voller Vergebung und barmherzig.\qt@no{74}}%
\qt@newcmd\qurantrans@de@dccxliv{Christus, der Sohn Marias, ist nichts anderes als ein Gesandter; vor ihm sind etliche Gesandte dahingegangen. Seine Mutter ist eine Wahrhaftige. Beide pflegten, Speise zu essen. Siehe, wie Wir ihnen die Zeichen deutlich machen, und dann siehe, wie sie sich abwenden lassen.\qt@no{75}}%
\qt@newcmd\qurantrans@de@dccxlv{Sprich: Wie könnt ihr anstelle Gottes dem dienen, was euch weder Schaden noch Nutzen bringen kann? Und Gott ist der, der alles hört und weiß.\qt@no{76}}%
\qt@newcmd\qurantrans@de@dccxlvi{Sprich: O ihr Leute des Buches, übertreibt nicht in eurer Religion über die Wahrheit hinaus und folgt nicht den Neigungen von Leuten, die früher irregegangen sind und viele irregeführt haben und vom rechten Weg abgeirrt sind.\qt@no{77}}%
\qt@newcmd\qurantrans@de@dccxlvii{Verflucht wurden diejenigen von den Kindern Israels, die ungläubig waren, durch den Mund Davids und Jesu, des Sohnes Marias. Dies dafür, daß sie ungehorsam waren und immer wieder Übertretungen begingen.\qt@no{78}}%
\qt@newcmd\qurantrans@de@dccxlviii{Sie pflegten nicht einander das Verwerfliche, das sie taten, zu verbieten. Schlimm ist, was sie zu tun pflegten.\qt@no{79}}%
\qt@newcmd\qurantrans@de@dccxlix{Du siehst viele von ihnen die Ungläubigen zu Freunden nehmen. Schlimm ist das, was sie sich selbst vorausgeschickt haben, so daß Gott ihnen grollt; und sie werden in der Pein ewig weilen.\qt@no{80}}%
\qt@newcmd\qurantrans@de@dccl{Würden sie an Gott und den Propheten und an das, was zu ihm herabgesandt wurde, glauben, hätten sie sie nicht zu Freunden genommen. Aber viele von ihnen sind Frevler.\qt@no{81}}%
\qt@newcmd\qurantrans@de@dccli{Du wirst sicher finden, daß unter den Menschen diejenigen, die den Gläubigen am stärksten Feindschaft zeigen, die Juden und die Polytheisten sind. Und du wirst sicher finden, daß unter ihnen diejenigen, die den Gläubigen in Liebe am nächsten stehen, die sind, welche sagen: «Wir sind Christen.» Dies deshalb, weil es unter ihnen Priester und Mönche gibt und weil sie nicht hochmütig sind.\qt@no{82}}%
\qt@newcmd\qurantrans@de@dcclii{Wenn sie hören, was zu dem Gesandten herabgesandt wurde, siehst du ihre Augen von Tränen überfließen wegen dessen, was sie nun von der Wahrheit kennen. Sie sagen: «Unser Herr, wir glauben. Verzeichne uns unter den Zeugen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@dccliii{Warum sollten wir nicht an Gott glauben und an das, was von der Wahrheit zu uns gekommen ist, und nicht erhoffen, daß unser Herr uns Eingang gewährt mit den rechtschaffenen Leuten?»\qt@no{84}}%
\qt@newcmd\qurantrans@de@dccliv{Nun belohnt sie Gott für das, was sie gesagt haben mit Gärten, unter denen Bäche fließen; darin werden sie ewig weilen. Dies ist die Entlohnung der Rechtschaffenen.\qt@no{85}}%
\qt@newcmd\qurantrans@de@dcclv{Diejenigen, die ungläubig sind und unsere Zeichen für Lüge erklären, das sind die Gefährten der Hölle.\qt@no{86}}%
\qt@newcmd\qurantrans@de@dcclvi{O ihr, die ihr glaubt, erklärt nicht für verboten die köstlichen Dinge, die Gott euch erlaubt hat, und begeht keine Übertretungen. Gott liebt die nicht, die Übertretungen begehen.\qt@no{87}}%
\qt@newcmd\qurantrans@de@dcclvii{Und eßt von dem, was Gott euch beschert hat, so es erlaubt und köstlich ist. Und fürchtet Gott, an den ihr eben glaubt.\qt@no{88}}%
\qt@newcmd\qurantrans@de@dcclviii{Gott belangt euch nicht wegen unbedachter Rede in euren Eiden. Aber Er belangt euch, wenn ihr euch in den Eiden fest bindet. Die Sühne dafür besteht darin, zehn Bedürftige mit dem zu beköstigen, womit ihr sonst eure Angehörigen beköstigt, oder sie zu kleiden oder einen Sklaven freizulassen. Wer es nicht vermag, der soll drei Tage fasten. Das ist die Sühne für eure Eide, wenn ihr schwört. Und haltet eure Eide. So macht Gott euch seine Zeichen deutlich, auf daß ihr dankbar seid.\qt@no{89}}%
\qt@newcmd\qurantrans@de@dcclix{O ihr, die ihr glaubt, der Wein, das Glücksspiel, die Opfersteine und die Lospfeile sind ein Greuel von Satans Werk. Meidet es, auf daß es euch wohl ergehe.\qt@no{90}}%
\qt@newcmd\qurantrans@de@dcclx{Der Satan will ja durch Wein und Glücksspiel Feindschaft und Haß zwischen euch erregen und euch vom Gedenken Gottes und vom Gebet abbringen. Werdet ihr wohl nun aufhören?\qt@no{91}}%
\qt@newcmd\qurantrans@de@dcclxi{Und gehorchet Gott und gehorchet dem Gesandten, und seid auf der Hut. Wenn ihr euch abkehrt, so wißt, unserem Gesandten obliegt nur die deutliche Ausrichtung (der Botschaft).\qt@no{92}}%
\qt@newcmd\qurantrans@de@dcclxii{Diejenigen, die glauben und die guten Werke tun, begehen im Hinblick auf das, was sie essen, kein Vergehen, wenn sie nur gottesfürchtig sind und glauben und die guten Werke tun, und weiter gottesfürchtig sind und glauben, und dann auch gottesfürchtig und rechtschaffen sind. Und Gott liebt die Rechtschaffenen.\qt@no{93}}%
\qt@newcmd\qurantrans@de@dcclxiii{O ihr, die ihr glaubt, Gott wird euch prüfen im Hinblick auf das Jagdwild, das eure Hände und eure Lanzen treffen, damit Gott feststellt, wer Ihn im Verborgenen fürchtet. Wer danach Übertretungen begeht, für den ist eine schmerzhafte Pein bestimmt.\qt@no{94}}%
\qt@newcmd\qurantrans@de@dcclxiv{O ihr, die ihr glaubt, tötet nicht das Jagdwild, während ihr im Weihezustand seid. Wer es von euch vorsätzlich tötet, hat als Auflage ein Stück Vieh abzugeben, gleich dem, was er getötet hat - darüber sollen zwei gerechte Leute von euch urteilen -, dies als Opfertier, das die Kaba erreichen soll. Oder er hat als Sühne Bedürftige zu speisen oder das Entsprechende an Fasten zu leisten. (Dies,) damit er die schweren Folgen seines Verhaltens kostet. Gott verzeiht, was vorher geschah. Wer es aber wieder tut, an dem rächt sich Gott. Und Gott ist mächtig und übt Rache.\qt@no{95}}%
\qt@newcmd\qurantrans@de@dcclxv{Erlaubt ist euch, die Tiere des Meeres zu jagen und sie zu essen zum Nutzen für euch und für die Reisenden, aber verboten ist euch, die Tiere des Festlandes zu jagen, während ihr im Weihezustand seid. Und fürchtet Gott, zu dem ihr versammelt werdet.\qt@no{96}}%
\qt@newcmd\qurantrans@de@dcclxvi{Gott hat die Kaba, das heilige Haus, für die Menschen zum Erfüllungsort ihrer Kulthandlungen gemacht, sowie den heiligen Monat, die Opfertiere und die Halsgehänge. Dies, damit ihr wißt, daß Gott weiß, was in den Himmeln und was auf der Erde ist, und daß Gott über alle Dinge Bescheid weiß.\qt@no{97}}%
\qt@newcmd\qurantrans@de@dcclxvii{Wißt, Gott verhängt eine harte Strafe, und Gott ist voller Vergebung und barmherzig.\qt@no{98}}%
\qt@newcmd\qurantrans@de@dcclxviii{Dem Gesandten obliegt nur die Ausrichtung (der Botschaft). Und Gott weiß, was ihr offenlegt und was ihr verschweigt.\qt@no{99}}%
\qt@newcmd\qurantrans@de@dcclxix{Sprich: Nicht gleich sind das Verdorbene und das Köstliche, auch wenn die Menge des Verdorbenen dir gefallen sollte. So fürchtet Gott, ihr Einsichtigen, auf daß es euch wohl ergehe.\qt@no{100}}%
\qt@newcmd\qurantrans@de@dcclxx{O ihr, die ihr glaubt, fragt nicht nach Dingen, die, wenn sie euch offengelegt würden, euch weh täten, und die, wenn ihr zu der Zeit, da der Koran herabgesandt wird, danach fragt, euch offengelegt werden. Gott hat sie übergangen. Und Gott ist voller Vergebung und mild.\qt@no{101}}%
\qt@newcmd\qurantrans@de@dcclxxi{Danach haben Leute vor euch gefragt, und dann haben sie sie verleugnet.\qt@no{102}}%
\qt@newcmd\qurantrans@de@dcclxxii{Gott hat keine Vorschriften festgelegt in bezug auf ohrgeschlitzte oder freiweidende Kamelstuten oder auf Schafe, die weiblichen Nachwuchs nacheinander werfen, oder auf Kamelhengste, die (dank ihrer Leistung) ihren Rücken (vor Traglast) schützen. Aber diejenigen, die ungläubig sind, erdichten Lügen gegen Gott, und die meisten von ihnen haben keinen Verstand.\qt@no{103}}%
\qt@newcmd\qurantrans@de@dcclxxiii{Und wenn ihnen gesagt wird: «Kommt her zu dem, was Gott herabgesandt hat, und zum Gesandten», sagen sie: «Uns genügt, was wir bei unseren Vätern vorgefunden haben.» Was denn, auch wenn ihre Väter nichts gewußt haben und der Rechtleitung nicht gefolgt sind?\qt@no{104}}%
\qt@newcmd\qurantrans@de@dcclxxiv{O ihr, die ihr glaubt, kümmert euch um euch selbst. Wer abirrt, kann euch nicht schaden, wenn ihr der Rechtleitung folgt. Zu Gott ist eure Heimkehr allesamt. Dann wird Er euch kundtun, was ihr zu tun pflegtet.\qt@no{105}}%
\qt@newcmd\qurantrans@de@dcclxxv{O ihr, die ihr glaubt, wenn einer von euch im Sterben liegt und sein Testament machen will, so soll das Zeugnis unter euch erfolgen durch zwei gerechte Leute von euch - oder durch zwei andere, die nicht von euch sind, wenn ihr im Land umherwandert und euch das Todesunglück trifft. Ihr sollt sie nach dem Gebet festhalten, und sie sollen dann, wenn ihr Zweifel habt, bei Gott schwören: «Wir verkaufen es nicht für irgendeinen Preis, auch wenn es sich um einen Verwandten handelt. Und wir verschweigen das Zeugnis Gottes nicht, sonst gehörten wir zu denen, die sich versündigen.»\qt@no{106}}%
\qt@newcmd\qurantrans@de@dcclxxvi{Wenn man entdeckt, daß sie sich einer Sünde schuldig gemacht haben, sollen aus den Reihen derer, gegen die man sich versündigt hat, zwei andere, deren Erbrecht am ehesten feststeht, an ihre Stelle treten und bei Gott schwören: «Unser Zeugnis ist bestimmt wahrhaftiger als deren Zeugnis. Und wir haben keine Übertretung begangen, sonst gehörten wir zu denen, die Unrecht tun.»\qt@no{107}}%
\qt@newcmd\qurantrans@de@dcclxxvii{Das bewirkt eher, daß sie das Zeugnis seinem Inhalt entsprechend vorbringen oder eben fürchten, daß nach ihren Eiden (andere) Eide zugelassen werden. Und fürchtet Gott und hört zu. Gott leitet die frevlerischen Leute nicht recht.\qt@no{108}}%
\qt@newcmd\qurantrans@de@dcclxxviii{Am Tag, da Gott die Gesandten versammelt und dann spricht: «Was wurde euch geantwortet?» Sie sagen: «Wir haben kein Wissen (davon). Du bist der, der die unsichtbaren Dinge alle weiß.»\qt@no{109}}%
\qt@newcmd\qurantrans@de@dcclxxix{Und als Gott sprach: «O Jesus, Sohn Marias, gedenke meiner Gnade zu dir und zu deiner Mutter, als Ich dich mit dem Geist der Heiligkeit stärkte, so daß du zu den Menschen in der Wiege und als Erwachsener sprachst; und als Ich dich das Buch, die Weisheit, die Tora und das Evangelium lehrte; und als du aus Ton etwas wie eine Vogelgestalt mit meiner Erlaubnis schufest und dann hineinbliesest und es mit meiner Erlaubnis zu einem Vogel wurde; und als du Blinde und Aussätzige mit meiner Erlaubnis heiltest und Tote mit meiner Erlaubnis herauskommen ließest; und als Ich die Kinder Israels von dir zurückhielt, als du mit den deutlichen Zeichen zu ihnen kamst, worauf diejenigen von ihnen, die ungläubig waren, sagten: >Das ist nichts als eine offenkundige Zauberei.\qt@no{110}}%
\qt@newcmd\qurantrans@de@dcclxxx{Und als Ich den Jüngern offenbarte: «Glaubt an Mich und an meinen Gesandten.» Sie sagten: «Wir glauben. Bezeuge, daß wir gottergeben sind.»\qt@no{111}}%
\qt@newcmd\qurantrans@de@dcclxxxi{Als die Jünger sagten: «O Jesus, Sohn Marias, kann dein Herr uns einen Tisch vom Himmel herabsenden?» Er sagte: «Fürchtet Gott, so ihr gläubig seid.»\qt@no{112}}%
\qt@newcmd\qurantrans@de@dcclxxxii{Sie sagten: «Wir wollen davon essen, so daß unsere Herzen Ruhe finden und daß wir wissen, daß du uns die Wahrheit gesagt hast, und daß wir zu denen gehören, die darüber Zeugnis geben.»\qt@no{113}}%
\qt@newcmd\qurantrans@de@dcclxxxiii{Jesus, der Sohn Marias, sagte: «O Gott, unser Herr, sende auf uns einen Tisch vom Himmel herab, daß er für uns, für den ersten von uns und den letzten von uns, ein Fest sei, und ein Zeichen von Dir. Und versorge uns. Du bist der beste Versorger.»\qt@no{114}}%
\qt@newcmd\qurantrans@de@dcclxxxiv{Gott sprach: «Ich werde ihn auf euch hinabsenden. Wer von euch hernach ungläubig wird, den werde Ich mit einer Pein peinigen, mit der Ich keinen von den Weltenbewohnern peinige.»\qt@no{115}}%
\qt@newcmd\qurantrans@de@dcclxxxv{Und als Gott sprach: «O Jesus, Sohn Marias, warst du es, der zu den Menschen sagte: >Nehmt euch neben Gott mich und meine Mutter zu Göttern?\qt@no{116}}%
\qt@newcmd\qurantrans@de@dcclxxxvi{Ich habe ihnen nichts anderes gesagt als das, was Du mir befohlen hast, nämlich: >Dienet Gott, meinem Herrn und eurem Herrn. < Ich war Zeuge über sie, solange ich unter ihnen weilte. Als Du mich abberufen hast, warst Du der Wächter über sie. Und Du bist über alle Dinge Zeuge.\qt@no{117}}%
\qt@newcmd\qurantrans@de@dcclxxxvii{Wenn Du sie peinigst, so sind sie deine Diener. Wenn Du ihnen vergibst, so bist Du der Mächtige, der Weise.»\qt@no{118}}%
\qt@newcmd\qurantrans@de@dcclxxxviii{Gott sprach: «Das ist der Tag, an dem den Wahrhaftigen ihre Wahrhaftigkeit nützen wird. Bestimmt sind für sie Gärten, unter denen Bäche fließen, darin werden sie auf immer ewig weilen. Gott hat Wohlgefallen an ihnen, und sie haben Wohlgefallen an Ihm. Dies ist der großartige Erfolg.»\qt@no{119}}%
\qt@newcmd\qurantrans@de@dcclxxxix{Gott gehört die Königsherrschaft der Himmel und der Erde und dessen, was in ihnen ist. Und Er hat Macht zu allen Dingen.\qt@no{120}}%
\qt@newcmd\qurantrans@de@dccxc{\basmalahde Lob sei Gott, der die Himmel und die Erde erschaffen und die Finsternisse und das Licht gemacht hat! Dennoch setzen diejenigen, die ungläubig sind, ihrem Herrn (andere) gleich.\qt@no{1}}%
\qt@newcmd\qurantrans@de@dccxci{Er ist es, der euch aus Ton erschaffen und dann eine Frist beschlossen hat. Und eine bestimmte Frist ist bei Ihm festgesetzt. Dennoch zweifelt ihr daran.\qt@no{2}}%
\qt@newcmd\qurantrans@de@dccxcii{Er ist Gott in den Himmeln und auf der Erde. Er weiß, was ihr geheimhaltet und was ihr offen bekundet, und Er weiß, was ihr erwerbt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@dccxciii{Kein Zeichen von den Zeichen ihres Herrn kommt zu ihnen, ohne daß sie sich davon abwenden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@dccxciv{So erklärten sie die Wahrheit für Lüge, als sie zu ihnen kam. Aber zu ihnen werden die Berichte gelangen über das, worüber sie immer wieder gespottet haben.\qt@no{5}}%
\qt@newcmd\qurantrans@de@dccxcv{Haben sie denn nicht gesehen, wie viele Generationen Wir vor ihnen haben verderben lassen? Wir haben ihnen auf der Erde eine feste Stellung verliehen, die Wir euch nicht verliehen haben. Und Wir haben den Himmel über sie ergiebig regnen lassen und die Bäche unter ihnen fließen lassen. Aber Wir haben sie für ihre Sünden verderben und nach ihnen eine andere Generation entstehen lassen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@dccxcvi{Hätten Wir auf dich ein Buch auf Papyrus hinabgesandt und würden sie es mit ihren Händen berühren, würden diejenigen, die ungläubig sind, dennoch sagen: «Dies ist nichts als offenkundige Zauberei.»\qt@no{7}}%
\qt@newcmd\qurantrans@de@dccxcvii{Und sie sagen: «Wäre ein Engel auf ihn herabgesandt worden!» Würden Wir einen Engel hinabsenden, wäre die Angelegenheit dann entschieden, und ihnen wird dann kein Aufschub gewährt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@dccxcviii{Auch wenn Wir ihn zu einem Engel gemacht hätten, Wir hätten ihn (doch) zu einem Mann gemacht und hätten ihnen (noch weiter) verhüllt, was sie sich ja selbst verhüllen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@dccxcix{Gespottet wurde schon vor dir über Gesandte. Da umschloß diejenigen, die sie verhöhnt hatten, das, worüber sie spotteten.\qt@no{10}}%
\qt@newcmd\qurantrans@de@dccc{Sprich: Zieht auf der Erde umher und schaut, wie das Ende derer war, die (die Botschaft) für Lüge erklärt haben.\qt@no{11}}%
\qt@newcmd\qurantrans@de@dccci{Sprich: Wem gehört, was in den Himmeln und auf der Erde ist? Sprich: (Es gehört) Gott. Vorgeschrieben hat Er sich selbst die Barmherzigkeit. Er wird euch zum Tag der Auferstehung versammeln, an dem kein Zweifel möglich ist. Diejenigen, die sich selbst verloren haben, die glauben eben nicht.\qt@no{12}}%
\qt@newcmd\qurantrans@de@dcccii{Ihm gehört, was in der Nacht und am Tag besteht. Er ist der, der alles hört und weiß.\qt@no{13}}%
\qt@newcmd\qurantrans@de@dccciii{Sprich: Sollte ich mir zum Freund einen anderen als Gott nehmen, den Schöpfer der Himmel und der Erde, der Speise gibt und dem niemand Speise geben kann? Sprich: Mir ist befohlen worden, der erste von denen zu sein, die gottergeben sind, und: «Sei nicht einer der Polytheisten.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@dccciv{Sprich: Ich fürchte, wenn ich gegen meinen Herrn ungehorsam bin, die Pein eines gewaltigen Tages.\qt@no{15}}%
\qt@newcmd\qurantrans@de@dcccv{Von wem sie an jenem Tag abgewehrt wird, dessen hat Er sich erbarmt. Und das ist der offenkundige Erfolg.\qt@no{16}}%
\qt@newcmd\qurantrans@de@dcccvi{Wenn Gott dich einen Schaden erleiden läßt, so gibt es niemanden, der ihn beheben könnte, außer Ihm. Und wenn Er dich Gutes erfahren läßt, so hat Er Macht zu allen Dingen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@dcccvii{Er ist der, der bezwingende Macht über seine Diener besitzt, und Er ist der, der weise ist und Kenntnis von allem hat.\qt@no{18}}%
\qt@newcmd\qurantrans@de@dcccviii{Sprich: Was gilt als das größte Zeugnis? Sprich: Gott ist Zeuge zwischen mir und euch. Dieser Koran ist mir offenbart, damit ich euch und auch jeden, den er erreicht, durch ihn warne. Wollt ihr wirklich bezeugen, daß es neben Gott andere Götter gibt? Sprich: Ich bezeuge (es) nicht. Sprich: Er ist nur ein einziger Gott, und ich bin unschuldig an dem, was ihr (Ihm) beigesellt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@dcccix{Diejenigen, denen Wir das Buch zukommen ließen, kennen es, wie sie ihre Söhne kennen. Diejenigen, die sich selbst verloren haben, die glauben eben nicht.\qt@no{20}}%
\qt@newcmd\qurantrans@de@dcccx{Und wer ist denn ungerechter, als wer gegen Gott eine Lüge erdichtet oder seine Zeichen für Lüge erklärt? Denen, die Unrecht tun, wird es sicher nicht wohl ergehen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@dcccxi{Und am Tag, da Wir sie alle versammeln. Dann sprechen Wir zu denen, die Polytheisten waren: Wo sind denn eure Teilhaber, die ihr immer wieder angegeben habt?\qt@no{22}}%
\qt@newcmd\qurantrans@de@dcccxii{Dann wird die Folge ihrer Verwirrung nur sein, daß sie sagen: «Bei Gott, unserem Herrn, wir waren keine Polytheisten.»\qt@no{23}}%
\qt@newcmd\qurantrans@de@dcccxiii{Schau, wie sie sich selbst belügen und wie ihnen entschwunden ist, was sie zu erdichten pflegten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@dcccxiv{Und unter ihnen gibt es welche, die dir zuhören. Aber Wir haben auf ihre Herzen Hüllen gelegt, so daß sie es nicht begreifen, und in ihre Ohren Schwerhörigkeit. Sie mögen jedes Zeichen sehen, sie glauben nicht daran. Und so, wenn sie zu dir kommen, um mit dir zu streiten, sagen diejenigen, die ungläubig sind: «Das sind nichts als die Fabeln der Früheren.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@dcccxv{Und sie verwehren es (den Leuten), und sie selbst entfernen sich davon. Sie bringen nur Verderben über sich selbst, und sie merken es nicht.\qt@no{26}}%
\qt@newcmd\qurantrans@de@dcccxvi{Könntest du nur zuschauen, wenn sie vor das Feuer gestellt werden und sagen: «O würden wir doch zurückgebracht werden! Wir würden dann nicht die Zeichen unseres Herrn für Lüge erklären, und wir würden zu den Gläubigen gehören»!\qt@no{27}}%
\qt@newcmd\qurantrans@de@dcccxvii{Vielmehr ist ihnen klar geworden, was sie früher geheimgehalten haben. Würden sie zurückgebracht, sie würden zu dem zurückkehren, was ihnen verboten wurde. Sie sind ja Lügner.\qt@no{28}}%
\qt@newcmd\qurantrans@de@dcccxviii{Und sie sagen: «Es gibt nur unser diesseitiges Leben, und wir werden nicht auferweckt.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@dcccxix{Könntest du nur zuschauen, wenn sie vor ihren Herrn gestellt werden und Er spricht: «Ist dies nicht die Wahrheit?» Sie sagen: «Doch, bei unserem Herrn!» Er spricht: «Kostet nun die Pein dafür, daß ihr ungläubig waret.»\qt@no{30}}%
\qt@newcmd\qurantrans@de@dcccxx{Den Verlust haben diejenigen, die die Begegnung mit Gott für Lüge erklären, so daß sie, wenn die Stunde plötzlich über sie kommt, sagen: «O welch ein Bedauern, daß wir sie nicht beachtet haben!» Sie tragen ihre Lasten auf ihrem Rücken. O schlimm ist das, was sie da tragen werden!\qt@no{31}}%
\qt@newcmd\qurantrans@de@dcccxxi{Das diesseitige Leben ist nur Spiel und Zerstreuung. Die jenseitige Wohnstätte ist gewiß besser für die, die gottesfürchtig sind. Habt ihr denn keinen Verstand?\qt@no{32}}%
\qt@newcmd\qurantrans@de@dcccxxii{Wir wissen wohl, daß dich betrübt, was sie sagen. Aber nicht dich zeihen sie der Lüge, sondern die, die Unrecht tun, leugnen ja die Zeichen Gottes.\qt@no{33}}%
\qt@newcmd\qurantrans@de@dcccxxiii{Der Lüge wurden schon vor dir Gesandte geziehen. Und sie ertrugen es mit Geduld, daß sie der Lüge geziehen wurden und daß ihnen Leid zugefügt wurde, bis unsere Unterstützung zu ihnen kam. Niemand wird die Worte Gottes abändern können. Zu dir ist doch etwas vom Bericht über die Gesandten gekommen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@dcccxxiv{Und wenn du schwer daran trägst, daß sie sich abwenden, und wenn du imstande bist, einen Schacht in die Erde oder eine Leiter in den Himmel zu suchen und ihnen dann ein Zeichen vorzubringen... Wenn Gott gewollt hätte, hätte Er sie in der Rechtleitung vereint. Sei also nicht einer der Törichten.\qt@no{35}}%
\qt@newcmd\qurantrans@de@dcccxxv{Nur jene können antworten, die zuhören. Die Toten wird Gott auferwecken. Dann werden sie zu Ihm zurückgebracht.\qt@no{36}}%
\qt@newcmd\qurantrans@de@dcccxxvi{Und sie sagen: «Wenn doch ein Zeichen von seinem Herrn auf ihn herabgesandt würde!» Sprich: Gott hat dazu die Macht, ein Zeichen herabzusenden. Aber die meisten von ihnen wissen nicht Bescheid.\qt@no{37}}%
\qt@newcmd\qurantrans@de@dcccxxvii{Es gibt keine Tiere auf der Erde und keine Vögel, die mit ihren Flügeln fliegen, die nicht Gemeinschaften wären gleich euch. Wir haben im Buch nichts übergangen. Dann werden sie zu ihrem Herrn versammelt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@dcccxxviii{Und diejenigen, die unsere Zeichen für Lüge erklären, sind taub, stumm, in Finsternissen. Gott führt irre, wen Er will, und wen Er will, den bringt Er auf einen geraden Weg.\qt@no{39}}%
\qt@newcmd\qurantrans@de@dcccxxix{Sprich: Was meint ihr? Wenn die Pein Gottes über euch kommt oder die Stunde über euch kommt, werdet ihr dann andere anrufen als Gott, so ihr die Wahrheit sagt?\qt@no{40}}%
\qt@newcmd\qurantrans@de@dcccxxx{Nein, Ihn werdet ihr anrufen. Dann wird Er das beheben, weswegen ihr (Ihn) anruft, wenn Er will, und ihr werdet vergessen, was ihr (Ihm) beigesellt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@dcccxxxi{Und Wir haben vor dir zu Gemeinschaften (Gesandte) geschickt und sie mit Not und Leid heimgesucht, auf daß sie sich vielleicht demütigen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@dcccxxxii{Hätten sie sich doch gedemütigt, als unsere Schlagkraft über sie kam! Aber ihre Herzen verhärteten sich, und der Satan machte ihnen verlockend, was sie zu tun pflegten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@dcccxxxiii{Und als sie das vergaßen, womit sie ermahnt worden waren, öffneten Wir ihnen die Tore zu allen Dingen. Als sie sich dann über das freuten, was ihnen zuteil wurde, suchten Wir sie plötzlich heim. Da waren sie ganz verzweifelt.\qt@no{44}}%
\qt@newcmd\qurantrans@de@dcccxxxiv{So wurde der letzte Rest der Leute, die Unrecht taten, ausgemerzt. Und Lob sei Gott, dem Herrn der Welten!\qt@no{45}}%
\qt@newcmd\qurantrans@de@dcccxxxv{Sprich: Was meint ihr? Wenn Gott euer Gehör und euer Augenlicht fortnähme und euer Herz versiegelte, welcher Gott außer Gott könnte es euch zurückbringen? Schau, wie Wir die Zeichen auf verschiedene Weise darlegen, dennoch kehren sie sich ab.\qt@no{46}}%
\qt@newcmd\qurantrans@de@dcccxxxvi{Sprich: Was meint ihr? Wenn die Pein Gottes plötzlich oder offen über euch kommt, wer wird wohl verderben außer den Leuten, die Unrecht tun?\qt@no{47}}%
\qt@newcmd\qurantrans@de@dcccxxxvii{Und Wir senden die Gesandten nur als Freudenboten und Warner. Diejenigen nun, die glauben und Besserung bringen, haben nichts zu befürchten, und sie werden nicht traurig sein.\qt@no{48}}%
\qt@newcmd\qurantrans@de@dcccxxxviii{Diejenigen aber, die unsere Zeichen für Lüge erklären, wird die Pein treffen dafür, daß sie immer wieder gefrevelt haben.\qt@no{49}}%
\qt@newcmd\qurantrans@de@dcccxxxix{Sprich: Ich sage euch nicht, ich hätte die Vorratskammern Gottes, und ich kenne auch nicht das Unsichtbare. Und ich sage euch nicht, ich sei ein Engel. Ich folge nur dem, was mir offenbart wird. Sprich: Sind etwa der Blinde und der Sehende gleich? Wollt ihr denn nicht nachdenken?\qt@no{50}}%
\qt@newcmd\qurantrans@de@dcccxl{Und warne damit diejenigen, die fürchten, zu ihrem Herrn versammelt zu werden - außer Ihm haben sie weder Freund noch Fürsprecher -, auf daß sie gottesfürchtig werden.\qt@no{51}}%
\qt@newcmd\qurantrans@de@dcccxli{Und vertreibe diejenigen nicht, die morgens und abends ihren Herrn anrufen in der Suche nach seinem Antlitz. Dir obliegt in keiner Weise, sie zur Rechenschaft zu ziehen, und ihnen obliegt in keiner Weise, dich zur Rechenschaft zu ziehen, so daß du sie vertreiben dürftest. Sonst würdest du zu denen gehören, die Unrecht tun.\qt@no{52}}%
\qt@newcmd\qurantrans@de@dcccxlii{Und so haben Wir die einen durch die anderen der Versuchung ausgesetzt, damit sie sagen: «Hat Gott gerade diesen da aus unserer Mitte eine Wohltat erwiesen?» Weiß nicht Gott besser Bescheid über die, die dankbar sind?\qt@no{53}}%
\qt@newcmd\qurantrans@de@dcccxliii{Und wenn diejenigen, die an unsere Zeichen glauben, zu dir kommen, so sprich: Friede über euch! Euer Herr hat sich selbst die Barmherzigkeit vorgeschrieben: Wenn nun einer von euch aus Unwissenheit Böses tut, aber danach umkehrt und Besserung zeigt, so ist Er voller Vergebung und barmherzig.\qt@no{54}}%
\qt@newcmd\qurantrans@de@dcccxliv{So legen Wir die Zeichen im einzelnen dar, auch damit der Weg der Übeltäter offenkundig wird.\qt@no{55}}%
\qt@newcmd\qurantrans@de@dcccxlv{Sprich: Mir ist verboten worden, denen zu dienen, die ihr anstelle Gottes anruft. Sprich: Ich folge nicht euren Neigungen. Sonst wäre ich abgeirrt, und ich würde nicht zu denen zählen, die der Rechtleitung folgen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@dcccxlvi{Sprich: Ich berufe mich auf ein deutliches Zeichen von meinem Herrn, aber ihr erklärt es für Lüge. Ich verfüge nicht über das, was ihr zu beschleunigen wünscht. Das Urteil gehört Gott allein. Er erzählt die Wahrheit, und Er ist der Beste derer, die entscheiden.\qt@no{57}}%
\qt@newcmd\qurantrans@de@dcccxlvii{Sprich: Würde ich über das verfügen, was ihr zu beschleunigen wünscht, wäre die Angelegenheit zwischen mir und euch bereits entschieden. Und Gott weiß besser Bescheid über die, die Unrecht tun.\qt@no{58}}%
\qt@newcmd\qurantrans@de@dcccxlviii{Er verfügt über die Schlüssel des Unsichtbaren. Niemand außer Ihm weiß über sie Bescheid. Und Er weiß, was auf dem Festland und im Meer ist. Kein Blatt fällt, ohne daß Er darüber Bescheid weiß; und es gibt kein Korn in den Finsternissen der Erde und nichts Feuchtes und nichts Trockenes, ohne daß es in einem offenkundigen Buch stünde.\qt@no{59}}%
\qt@newcmd\qurantrans@de@dcccxlix{Er ist es, der euch bei Nacht abberuft und weiß, was ihr bei Tag begangen habt; und dann erweckt Er euch an ihm, damit eine bestimmte Frist erfüllt wird. Dann wird eure Rückkehr zu Ihm sein, und dann wird Er euch kundtun, was ihr zu tun pflegtet.\qt@no{60}}%
\qt@newcmd\qurantrans@de@dcccl{Er ist der, der bezwingende Macht über seine Diener besitzt. Und Er entsendet Hüter über euch, so daß, wenn der Tod zu einem von euch kommt, unsere Boten ihn abberufen, und sie übergehen nichts.\qt@no{61}}%
\qt@newcmd\qurantrans@de@dcccli{Dann werden sie zu Gott, ihrem wahren Herrscher, zurückgebracht. Ja, Ihm gehört das Urteil, und Er ist der Schnellste derer, die abrechnen.\qt@no{62}}%
\qt@newcmd\qurantrans@de@dccclii{Sprich: Wer errettet euch aus den Finsternissen des Festlandes und des Meeres, wenn ihr Ihn in (sichtbarer) Demut und im Verborgenen anruft: «Wenn Er uns hieraus rettet, werden wir sicher zu den Dankbaren gehören»?\qt@no{63}}%
\qt@newcmd\qurantrans@de@dcccliii{Sprich: Gott errettet euch daraus und aus jeder Trübsal. Ihr aber gesellt (Ihm) weiterhin (andere) bei.\qt@no{64}}%
\qt@newcmd\qurantrans@de@dcccliv{Sprich: Er ist es, der dazu die Macht hat, über euch eine Pein von oben oder unter euren Füßen zu schicken, oder euch in Parteien durcheinanderzubringen und die einen von euch die Schlagkraft der anderen erleiden zu lassen. Schau, wie Wir die Zeichen auf verschiedene Weise darlegen, auf daß sie die Sache begreifen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@dccclv{Dein Volk erklärt es für Lüge, obwohl es die Wahrheit ist. Sprich: Ich bin nicht als Sachwalter über euch eingesetzt.\qt@no{66}}%
\qt@newcmd\qurantrans@de@dccclvi{Jeder Bericht wird seine Bestätigung finden, und ihr werdet es noch zu wissen bekommen.\qt@no{67}}%
\qt@newcmd\qurantrans@de@dccclvii{Und wenn du diejenigen siehst, die auf unsere Zeichen (spottend) eingehen, dann wende dich von ihnen ab, bis sie auf ein anderes Gespräch eingehen. Und wenn dich der Satan vergessen läßt, dann sollst du, nachdem du dich wieder daran erinnert hast, nicht mit den Leuten zusammensitzen, die Unrecht tun.\qt@no{68}}%
\qt@newcmd\qurantrans@de@dccclviii{Denjenigen, die gottesfürchtig sind, obliegt in keiner Weise, sie zur Rechenschaft zu ziehen. Es ist nur eine Ermahnung, auf daß sie gottesfürchtig werden.\qt@no{69}}%
\qt@newcmd\qurantrans@de@dccclix{Und laß diejenigen sitzen, die ihre Religion zum Gegenstand von Spiel und Zerstreuung nehmen und die das diesseitige Leben betört. Und ermahne durch ihn, auf daß niemand dem Verderben preisgegeben wird für das, was er erworben hat. Er hat dann außer Gott weder Freund noch Fürsprecher. Und würde er auch jedes Lösegeld vorlegen, es wird von ihm nicht angenommen werden. Das sind die, die dem Verderben preisgegeben werden für das, was sie erworben haben. Bestimmt ist für sie ein Getränk aus heißem Wasser und eine schmerzhafte Pein dafür, daß sie ungläubig waren.\qt@no{70}}%
\qt@newcmd\qurantrans@de@dccclx{Sprich: Sollen wir statt zu Gott zu etwas rufen, was uns weder nützt noch schadet, und, nachdem Gott uns rechtgeleitet hat, auf unseren Fersen kehrtmachen, gleich jenem, den die Satane im Land weggelockt haben? Ratlos (ist er); er hat Gefährten, die ihn zur Rechtleitung rufen: «Komm zu uns.» Sprich: Nur die Rechtleitung Gottes ist die (wahre) Rechtleitung. Und uns wurde befohlen, uns dem Herrn der Welten zu ergeben.\qt@no{71}}%
\qt@newcmd\qurantrans@de@dccclxi{Und auch: Verrichtet das Gebet und fürchtet Ihn. Er ist es, zu dem ihr versammelt werdet.\qt@no{72}}%
\qt@newcmd\qurantrans@de@dccclxii{Er ist es, der die Himmel und die Erde der Wahrheit entsprechend erschaffen hat. Und am Tag, da Er sagt: Sei!, und es ist. Sein Wort ist die Wahrheit. Und Ihm gehört die Königsherrschaft am Tag, da in die Trompete geblasen wird. Er weiß über das Unsichtbare und das Offenbare Bescheid, und Er ist der Weise, der Kenntnis von allem hat.\qt@no{73}}%
\qt@newcmd\qurantrans@de@dccclxiii{Und als Abraham zu seinem Vater Aazar sagte: «Nimmst du dir denn Götzen zu Göttern? Ich sehe dich und dein Volk in einem offenkundigen Irrtum.»\qt@no{74}}%
\qt@newcmd\qurantrans@de@dccclxiv{Und so zeigten Wir Abraham das Reich der Himmel und der Erde, damit er einer von denen sei, die Gewißheit hegen.\qt@no{75}}%
\qt@newcmd\qurantrans@de@dccclxv{Als nun die Nacht ihn umhüllte, sah er einen Stern. Er sagte: «Das ist mein Herr.» Als der aber verschwand, sagte er: «Ich liebe die nicht, die verschwinden.»\qt@no{76}}%
\qt@newcmd\qurantrans@de@dccclxvi{Als er dann den Mond aufgehen sah, sagte er: «Das ist mein Herr.» Als der aber verschwand, sagte er: «Wenn mein Herr mich nicht rechtleitet, werde ich gewiß zu den abgeirrten Leuten gehören.»\qt@no{77}}%
\qt@newcmd\qurantrans@de@dccclxvii{Als er dann die Sonne aufgehen sah, sagte er: «Das ist mein Herr. Das ist ja größer.» Als sie aber verschwand, sagte er: «O mein Volk, ich bin unschuldig an dem, was ihr (Gott) beigesellt.\qt@no{78}}%
\qt@newcmd\qurantrans@de@dccclxviii{Ich richte mein Gesicht zu dem, der die Himmel und die Erde erschaffen hat, als Anhänger des reinen Glaubens, und ich gehöre nicht zu den Polytheisten.»\qt@no{79}}%
\qt@newcmd\qurantrans@de@dccclxix{Sein Volk stritt mit ihm. Er sagte: «Wollt ihr mit mir über Gott streiten, wo Er mich rechtgeleitet hat? Ich fürchte nicht, was ihr Ihm beigesellt, es sei denn, es ist mein Herr, der etwas will. Mein Herr umfaßt alle Dinge in seinem Wissen. Wollt ihr es nicht bedenken?»\qt@no{80}}%
\qt@newcmd\qurantrans@de@dccclxx{«Und wie sollte ich fürchten, was ihr (Ihm) beigesellt, wo ihr euch nicht fürchtet, Gott solche beizugesellen, für die Er auf euch keine Ermächtigung herabgesandt hat? Welche von beiden Gruppen hat eher Recht auf Sicherheit, so ihr es wißt?»\qt@no{81}}%
\qt@newcmd\qurantrans@de@dccclxxi{Denjenigen, die glauben und ihren Glauben nicht mit unrechtem Handeln verhüllen, gehört die Sicherheit, und sie folgen der Rechtleitung.\qt@no{82}}%
\qt@newcmd\qurantrans@de@dccclxxii{Das ist unser Beweisgrund, den Wir Abraham gegen sein Volk zukommen ließen. Wir erhöhen, wen Wir wollen, um Rangstufen. Dein Herr ist weise und weiß Bescheid.\qt@no{83}}%
\qt@newcmd\qurantrans@de@dccclxxiii{Und Wir schenkten ihm Isaak und Jakob; jeden (von ihnen) haben wir rechtgeleitet. Auch Noach haben Wir zuvor rechtgeleitet, sowie aus seiner Nachkommenschaft David und Salomo, Ijob, Josef, Mose und Aaron - so entlohnen Wir die, die Gutes tun -;\qt@no{84}}%
\qt@newcmd\qurantrans@de@dccclxxiv{Und Zakaria, Yahyaa -, Jesus und Elias: jeder von ihnen gehört zu den Rechtschaffenen;\qt@no{85}}%
\qt@newcmd\qurantrans@de@dccclxxv{Und Ismael, Elischa, Jonas und Lot: jeden (von ihnen) haben Wir vor den Weltenbewohnern bevorzugt;\qt@no{86}}%
\qt@newcmd\qurantrans@de@dccclxxvi{Und auch manche von ihren Vätern, ihren Nachkommen und ihren Brüdern. Wir haben sie erwählt und zu einem geraden Weg geleitet.\qt@no{87}}%
\qt@newcmd\qurantrans@de@dccclxxvii{Das ist die Rechtleitung Gottes. Er leitet damit recht, wen von seinen Dienern Er will. Und hätten sie (Gott andere) beigesellt, so wäre (auch) ihnen wertlos geworden, was sie zu tun pflegten.\qt@no{88}}%
\qt@newcmd\qurantrans@de@dccclxxviii{Das sind die, denen Wir das Buch, die Urteilsvollmacht und die Prophetie zukommen ließen. Wenn diese da sie verleugnen, so haben Wir sie Leuten anvertraut, die sie nicht verleugnen.\qt@no{89}}%
\qt@newcmd\qurantrans@de@dccclxxix{Das sind die, die Gott rechtgeleitet hat. Richte dich nun nach ihrer Rechtleitung. Sprich: Ich verlange von euch keinen Lohn dafür. Es ist nichts als eine Ermahnung für die Weltenbewohner.\qt@no{90}}%
\qt@newcmd\qurantrans@de@dccclxxx{Und sie haben Gott nicht so eingeschätzt, wie Er eingeschätzt werden soll, als sie sagten: «Gott hat nichts auf einen Menschen herabgesandt.» Sprich: Wer hat denn das Buch herabgesandt, das Mose als Licht und Rechtleitung für die Menschen gebracht hat? Ihr macht ja daraus Papyrusblätter, die ihr offen zeigt - und ihr haltet vieles geheim; und euch wurde beigebracht, was ihr nicht wußtet, weder ihr noch eure Väter. Sprich: Gott. Und dann laß sie in ihren ausschweifenden Reden ihr Spiel treiben.\qt@no{91}}%
\qt@newcmd\qurantrans@de@dccclxxxi{Und dies ist ein Buch, das Wir hinabgesandt haben, ein gesegnetes (Buch), das bestätigt, was vor ihm vorhanden war, damit du die Mutter der Städte und die Menschen in ihrer Umgebung warnest. Diejenigen, die an das Jenseits glauben, glauben auch daran, und sie halten ihr Gebet ein.\qt@no{92}}%
\qt@newcmd\qurantrans@de@dccclxxxii{Und wer ist ungerechter, als wer gegen Gott eine Lüge erdichtet oder sagt: «Mir ist offenbart worden», während ihm nichts offenbart worden ist, und wer sagt: «Ich werde herabsenden, was dem gleich ist, was Gott herabgesandt hat»? Könntest du nur zuschauen, wenn die Ungerechten sich in den Fluten des Todes befinden und die Engel ihre Hände ausstrecken: «Gebt eure Seelen heraus. Heute wird euch mit der Pein der Schmach vergolten dafür, daß ihr immer wieder gegen Gott etwas anderes als die Wahrheit gesagt und euch gegenüber seinen Zeichen hochmütig verhalten habt»!\qt@no{93}}%
\qt@newcmd\qurantrans@de@dccclxxxiii{Ihr seid nun einzeln zu Uns gekommen, so wie Wir euch das erste Mal erschaffen haben, und ihr habt hinter euch zurückgelassen, was Wir euch verliehen haben. Und Wir sehen nicht bei euch eure Fürsprecher, von denen ihr behauptet habt, sie hätten Anteil an euch. Abgeschnitten ist das Band zwischen euch, und entschwunden ist euch, was ihr immer wieder behauptet habt.\qt@no{94}}%
\qt@newcmd\qurantrans@de@dccclxxxiv{Gott ist es, der die Körner und die Kerne spaltet. So bringt Er das Lebendige aus dem Toten und das Tote aus dem Lebendigen hervor. Das ist Gott - wie leicht laßt ihr euch doch abwenden! -,\qt@no{95}}%
\qt@newcmd\qurantrans@de@dccclxxxv{Er, der die Morgendämmerung anbrechen läßt. Er hat die Nacht zur Ruhezeit und die Sonne und den Mond zur Zeitberechnung gemacht. Das ist das Dekret dessen, der mächtig ist und Bescheid weiß.\qt@no{96}}%
\qt@newcmd\qurantrans@de@dccclxxxvi{Er ist es, der euch die Sterne gemacht hat, damit ihr euch durch sie die gute Richtung in den Finsternissen des Festlandes und des Meeres findet. Wir haben die Zeichen im einzelnen dargelegt für Leute, die Bescheid wissen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@dccclxxxvii{Und Er ist es, der euch aus einem einzigen Wesen hat entstehen lassen. Dann gibt es einen Aufenthaltsort und einen Aufbewahrungsort. Wir haben die Zeichen im einzelnen dargelegt für Leute, die begreifen.\qt@no{98}}%
\qt@newcmd\qurantrans@de@dccclxxxviii{Und Er ist es, der vom Himmel Wasser herabkommen läßt. Und Wir bringen damit Pflanzen jeglicher (Art) hervor; und dann bringen Wir aus ihnen Grün hervor, aus dem Wir übereinandergereihte Körner hervorbringen - und aus den Palmen, aus ihren Blütenscheiden entstehen herabhängende Dattelbüschel -, und (auch) Gärten mit Weinstöcken, und die Öl- und die Granatapfelbäume, die einander ähnlich und unähnlich sind. Schaut auf ihre Früchte, wenn sie Früchte tragen, und auf deren Reifen. Darin sind Zeichen für Leute, die glauben.\qt@no{99}}%
\qt@newcmd\qurantrans@de@dccclxxxix{Und sie haben Gott Teilhaber gegeben: die Djinn, wo Er sie doch erschaffen hat. Und sie haben Ihm Söhne und Töchter angedichtet, ohne (richtiges) Wissen. Preis sei Ihm! Er ist erhaben über das, was sie da schildern.\qt@no{100}}%
\qt@newcmd\qurantrans@de@dcccxc{Der Schöpfer der Himmel und der Erde, woher soll Er ein Kind haben, wo Er doch keine Gefährtin hat und Er (sonst) alles erschaffen hat? Und Er weiß über alle Dinge Bescheid.\qt@no{101}}%
\qt@newcmd\qurantrans@de@dcccxci{Das ist eben Gott, euer Herr. Es gibt keinen Gott außer Ihm, dem Schöpfer aller Dinge. So dienet Ihm. Er ist Sachwalter über alle Dinge.\qt@no{102}}%
\qt@newcmd\qurantrans@de@dcccxcii{Die Blicke erreichen Ihn nicht, Er aber erreicht die Blicke. Und Er ist der Feinfühlige, der Kenntnis von allem hat.\qt@no{103}}%
\qt@newcmd\qurantrans@de@dcccxciii{Zu euch sind nun einsichtbringende Zeichen von eurem Herrn gekommen. Wer einsichtig wird, ist es zu seinem eigenen Vorteil, und wer blind ist, ist es zu seinem eigenen Schaden. Und ich bin nicht Hüter über euch.\qt@no{104}}%
\qt@newcmd\qurantrans@de@dcccxciv{So legen Wir die Zeichen auf verschiedene Weise dar. Dies, damit sie sagen können: «Du hast danach geforscht», und damit Wir es Leuten deutlich machen, die Bescheid wissen.\qt@no{105}}%
\qt@newcmd\qurantrans@de@dcccxcv{Folge dem, was dir von deinem Herrn offenbart worden ist. Es gibt keinen Gott außer Ihm. Und wende dich von den Polytheisten ab.\qt@no{106}}%
\qt@newcmd\qurantrans@de@dcccxcvi{Wenn Gott gewollt hätte, wären sie nicht Polytheisten geworden. Und Wir haben dich nicht zum Hüter über sie gemacht, und du bist nicht als Sachwalter über sie eingesetzt.\qt@no{107}}%
\qt@newcmd\qurantrans@de@dcccxcvii{Und schmäht nicht diejenigen, die sie anstelle Gottes anrufen, damit sie nicht in Übertretung ohne (richtiges) Wissen Gott schmähen. So haben Wir jeder Gemeinschaft ihr Tun verlockend gemacht. Alsdann wird ihre Rückkehr zu ihrem Herrn sein, und Er wird ihnen kundtun, was sie zu tun pflegten.\qt@no{108}}%
\qt@newcmd\qurantrans@de@dcccxcviii{Und sie haben bei Gott ihren eifrigsten Eid geschworen, sie würden, wenn ein Zeichen zu ihnen käme, sicher daran glauben. Sprich: Die Zeichen stehen bei Gott allein. Wie solltet ihr auch merken, daß, wenn es tatsächlich käme, sie doch nicht glauben?\qt@no{109}}%
\qt@newcmd\qurantrans@de@dcccxcix{Und Wir kehren ihre Herzen und ihr Augenlicht um, ebenso wie sie das erste Mal nicht daran geglaubt haben. Und Wir lassen sie im Übermaß ihres Frevels blind umherirren.\qt@no{110}}%
\qt@newcmd\qurantrans@de@cm{Würden Wir auch zu ihnen die Engel hinabsenden, würden die Toten auch zu ihnen sprechen und Wir alle Dinge vor ihren Augen versammeln, sie würden unmöglich glauben, es sei denn, Gott will es. Aber die meisten von ihnen sind töricht.\qt@no{111}}%
\qt@newcmd\qurantrans@de@cmi{Und so haben Wir jedem Propheten einen Feind gegeben: die Satane der Menschen und der Djinn, von denen die einen den anderen prunkvolles Gerede eingeben, um sie zu betören - und wenn dein Herr gewollt hätte, hätten sie es nicht getan; so laß sie sitzen mit dem, was sie erdichten -,\qt@no{112}}%
\qt@newcmd\qurantrans@de@cmii{Und damit die Herzen derer, die an das Jenseits nicht glauben, ihm zuneigen, und damit sie daran Gefallen haben und begehen, was sie begehen.\qt@no{113}}%
\qt@newcmd\qurantrans@de@cmiii{Sollte ich mir einen anderen Schiedsrichter als Gott suchen, wo Er es doch ist, der das Buch, im einzelnen dargelegt, zu euch herabgesandt hat? Diejenigen, denen Wir das Buch zukommen ließen, wissen, daß es von deinem Herrn mit der Wahrheit herabgesandt wurde. So sei nicht einer der Zweifler.\qt@no{114}}%
\qt@newcmd\qurantrans@de@cmiv{Und das Wort deines Herrn hat sich in Wahrhaftigkeit und Gerechtigkeit erfüllt. Niemand kann seine Worte abändern. Er ist der, der alles hört und weiß.\qt@no{115}}%
\qt@newcmd\qurantrans@de@cmv{Und wenn du der Mehrzahl derer, die auf der Erde sind, folgst, werden sie dich vom Weg Gottes abirren lassen. Sie folgen ja nur Vermutungen, und sie stellen nur Schätzungen an.\qt@no{116}}%
\qt@newcmd\qurantrans@de@cmvi{Dein Herr weiß besser, wer von seinem Weg abirrt, und Er weiß besser über die Bescheid, die der Rechtleitung folgen.\qt@no{117}}%
\qt@newcmd\qurantrans@de@cmvii{Eßt von dem, worüber der Name Gottes ausgesprochen worden ist, so ihr an seine Zeichen glaubt.\qt@no{118}}%
\qt@newcmd\qurantrans@de@cmviii{Warum solltet ihr denn nicht von dem essen, worüber der Name Gottes ausgesprochen worden ist, wo Er euch im einzelnen dargelegt hat, was Er euch verboten hat, ausgenommen das, wozu ihr gezwungen werdet? Viele führen (andere) durch ihre Neigungen ohne (richtiges) Wissen in die Irre. Aber dein Herr weiß besser über die Bescheid, die Übertretungen begehen.\qt@no{119}}%
\qt@newcmd\qurantrans@de@cmix{Unterlaßt die Sünde, ob sie offen oder verborgen ist. Denen, die Sünden erwerben, wird vergolten für das, was sie begangen haben.\qt@no{120}}%
\qt@newcmd\qurantrans@de@cmx{Und eßt nicht von dem, worüber der Name Gottes nicht ausgesprochen worden ist. Das ist Frevel. Die Satane geben ihren Freunden ein, mit euch zu streiten. Wenn ihr ihnen gehorcht, seid ihr sogleich Polytheisten.\qt@no{121}}%
\qt@newcmd\qurantrans@de@cmxi{Ist denn der, der tot war, den Wir aber dann wieder lebendig gemacht und dem Wir ein Licht gegeben haben, daß er darin unter den Menschen umhergehen kann, mit dem zu vergleichen, der in den Finsternissen ist und nicht aus ihnen herauskommen kann? So ist den Ungläubigen verlockend gemacht worden, was sie zu tun pflegten.\qt@no{122}}%
\qt@newcmd\qurantrans@de@cmxii{Und so haben Wir in jeder Stadt ihre größten Übeltäter bestellt, damit sie in ihr Ränke schmieden. Sie schmieden aber Ränke nur gegen sich selbst, und sie merken es nicht.\qt@no{123}}%
\qt@newcmd\qurantrans@de@cmxiii{Wenn ein Zeichen zu ihnen kommt, sagen sie: «Wir werden nicht glauben, bis uns das gleiche zukommt, was den Gesandten Gottes zugekommen ist.» Gott weiß besser, wo Er seine Botschaft anbringt. Diejenigen, die Übeltaten begehen, wird bei Gott Erniedrigung und eine harte Pein treffen dafür, daß sie Ränke schmiedeten.\qt@no{124}}%
\qt@newcmd\qurantrans@de@cmxiv{Wen Gott rechtleiten will, dem weitet Er die Brust für den Islam. Und wen Er irreführen will, dem macht Er die Brust beklommen und bedrückt, so als ob er in den Himmel hochsteigen würde. Auf diese Weise legt Gott das Greuel auf diejenigen, die nicht glauben.\qt@no{125}}%
\qt@newcmd\qurantrans@de@cmxv{Und dies ist der Weg deines Herrn, er ist gerade. Wir haben die Zeichen im einzelnen dargelegt für Leute, die (sie) bedenken.\qt@no{126}}%
\qt@newcmd\qurantrans@de@cmxvi{Bestimmt ist für sie bei ihrem Herrn die Wohnstätte des Friedens. Er ist ihr Freund wegen dessen, was sie zu tun pflegten.\qt@no{127}}%
\qt@newcmd\qurantrans@de@cmxvii{Und am Tag, da Er sie alle versammelt: «O Gemeinschaft der Djinn, ihr habt euch viele Menschen angeworben.» Und ihre Freunde unter den Menschen sagen: «Unser Herr, wir haben Nutzen voneinander gehabt und haben nun unsere Frist erreicht, die Du uns gesetzt hast.» Er spricht: «Das Feuer ist eure Bleibe; darin werdet ihr ewig weilen, es sei denn, Gott will es anders.» Dein Herr ist weise und weiß Bescheid.\qt@no{128}}%
\qt@newcmd\qurantrans@de@cmxviii{So setzen Wir die einen von denen, die Unrecht tun, über die anderen wegen dessen, was sie erworben haben.\qt@no{129}}%
\qt@newcmd\qurantrans@de@cmxix{«O Gemeinschaft der Djinn und der Menschen, sind nicht Gesandte aus eurer Mitte zu euch gekommen, um euch von meinen Zeichen zu erzählen und euch die Begegnung mit diesem eurem Tag warnend zu verkünden?» Sie sagen: «Wir zeugen gegen uns selbst.» Betört hat sie das diesseitige Leben, und sie zeugen gegen sich selbst, daß sie ungläubig waren.\qt@no{130}}%
\qt@newcmd\qurantrans@de@cmxx{Das ist so, weil dein Herr die Städte nicht zu Unrecht dem Verderben preisgibt, während ihre Bewohner nichts ahnen.\qt@no{131}}%
\qt@newcmd\qurantrans@de@cmxxi{Für alle sind Rangstufen bestimmt für das, was sie getan haben. Und dein Herr läßt nicht unbeachtet, was sie tun.\qt@no{132}}%
\qt@newcmd\qurantrans@de@cmxxii{Und dein Herr ist der, der auf niemanden angewiesen ist und die Barmherzigkeit besitzt. Wenn Er will, läßt Er euch fortgehen und nach euch folgen, was Er will, so wie Er euch aus der Nachkommenschaft anderer Leute entstehen ließ.\qt@no{133}}%
\qt@newcmd\qurantrans@de@cmxxiii{Was euch angedroht wird, wird sicher kommen, und ihr könnt es nicht vereiteln.\qt@no{134}}%
\qt@newcmd\qurantrans@de@cmxxiv{Sprich: O mein Volk, handelt nach eurem Standpunkt. Ich werde auch so handeln. Dann werdet ihr wissen, wem die jenseitige Wohnstätte gehört. Wahrlich, denen, die Unrecht tun, wird es nicht wohl ergehen.\qt@no{135}}%
\qt@newcmd\qurantrans@de@cmxxv{Und sie haben für Gott einen Anteil festgesetzt von dem, was Er an Feldernte und Vieh geschaffen hat. Und sie sagen: «Dies ist für Gott» - so behaupten sie -, «und dies ist für unsere Teilhaber.» Was für ihre Teilhaber bestimmt ist, gelangt nicht zu Gott, was aber für Gott bestimmt ist, gelangt zu ihren Teilhabern. Schlecht ist doch ihr Urteil.\qt@no{136}}%
\qt@newcmd\qurantrans@de@cmxxvi{Und auch vielen von den Polytheisten haben ihre Teilhaber verlockend gemacht, ihre Kinder zu töten, um sie zu verderben und sie in ihrer Religion zu verwirren. Wenn Gott gewollt hätte, hätten sie es nicht getan. So laß sie da sitzen mit dem, was sie erdichten.\qt@no{137}}%
\qt@newcmd\qurantrans@de@cmxxvii{Und sie sagen: «Das sind Tiere und Ernte, die tabu sind. Niemand darf davon essen, außer dem, wen wir wollen.» So behaupten sie. - Es sind Tiere, deren Rücken (für Lasten) verwehrt sind, und Tiere, über die sie den Namen Gottes nicht aussprechen; so erdichten sie eine Lüge gegen Ihn. Er wird ihnen vergelten für das, was sie immer wieder erdichteten.\qt@no{138}}%
\qt@newcmd\qurantrans@de@cmxxviii{Und sie sagen: «Was im Leib dieser Tiere ist, ist den Männern unter uns vorbehalten, für unsere Gattinnen aber verboten.» Wird es aber tot (geboren), so haben sie (alle) daran teil. Er wird ihnen für ihre Schilderung vergelten. Er ist weise und weiß Bescheid.\qt@no{139}}%
\qt@newcmd\qurantrans@de@cmxxix{Den Verlust haben diejenigen, die ihre Kinder aus Torheit ohne (richtiges) Wissen töten und das für verboten erklären, was Gott ihnen beschert, indem sie gegen Gott eine Lüge erdichten. Sie sind abgeirrt und folgen nicht der Rechtleitung.\qt@no{140}}%
\qt@newcmd\qurantrans@de@cmxxx{Und Er ist es, der Gärten mit Spalieren und ohne Spaliere entstehen läßt, sowie die Palmen und das Getreide verschiedener Erntesorten, und die Öl- und Granatapfelbäume, die einander ähnlich und unähnlich sind. Eßt von ihren Früchten, wenn sie Früchte tragen, und entrichtet am Tag ihrer Ernte, was als Rechtspflicht darauf steht, aber seid nicht maßlos - Er liebt ja die Maßlosen nicht.\qt@no{141}}%
\qt@newcmd\qurantrans@de@cmxxxi{Und an Vieh (läßt Er entstehen) Lasttiere und Kleintiere. Eßt von dem, was Gott euch beschert hat, und folgt nicht den Fußstapfen des Satans. Er ist euch ein offenkundiger Feind.\qt@no{142}}%
\qt@newcmd\qurantrans@de@cmxxxii{Und auch) acht Tiere zu Paaren: von den Schafen zwei und von den Ziegen zwei. - Sprich: Sind es die beiden Männchen, die Er verboten hat, oder die beiden Weibchen, oder was der Leib der beiden Weibchen enthält? Tut (es) mir denn aufgrund eines (richtigen) Wissens kund, so ihr die Wahrheit sagt.\qt@no{143}}%
\qt@newcmd\qurantrans@de@cmxxxiii{Und auch von den Kamelen zwei und von den Rindern zwei. - Sprich: Sind es die beiden Männchen, die Er verboten hat, oder die beiden Weibchen, oder was der Leib der Weibchen enthält? Oder waret ihr zugegen, als Gott euch dies auftrug? Wer ist denn ungerechter, als wer gegen Gott eine Lüge erdichtet, um die Menschen ohne (richtiges) Wissen irrezuführen? Gott leitet die ungerechten Leute nicht recht.\qt@no{144}}%
\qt@newcmd\qurantrans@de@cmxxxiv{Sprich: In dem, was mir offenbart wurde, finde ich nicht, daß etwas für den Essenden zu essen verboten wäre, es sei denn, es ist Verendetes oder ausgeflossenes Blut oder Schweinefleisch - es ist ein Greuel - oder ein Frevel, worüber ein anderer als Gott angerufen worden ist. Wenn aber einer gezwungen wird, wobei er weder Auflehnung noch Übertretung begeht, so ist dein Herr voller Vergebung und barmherzig.\qt@no{145}}%
\qt@newcmd\qurantrans@de@cmxxxv{Und denen, die Juden sind, haben Wir alles verboten, was Krallen hat; und von den Rindern und den Schafen haben Wir ihnen das Fett verboten, ausgenommen das, was ihre Rücken und ihre Eingeweide tragen oder was mit Knochen verwachsen ist. Damit haben Wir ihnen für ihre Auflehnung vergolten, und Wir sagen ja die Wahrheit.\qt@no{146}}%
\qt@newcmd\qurantrans@de@cmxxxvi{Wenn sie dich der Lüge zeihen, dann sprich: Euer Herr besitzt eine umfassende Barmherzigkeit, aber niemand kann seine Schlagkraft von den Leuten abwenden, die Übeltäter sind.\qt@no{147}}%
\qt@newcmd\qurantrans@de@cmxxxvii{Diejenigen, die (Ihm) beigesellen, werden sagen: «Wenn Gott gewollt hätte, hätten wir nicht beigesellt, und auch nicht unsere Väter, und wir hätten nichts verboten.» Auf diese Weise haben diejenigen, die vor ihnen lebten, (ihre Gesandten) der Lüge geziehen, bis sie unsere Schlagkraft zu spüren bekamen. Sprich: Besitzt ihr irgendein Wissen, das ihr uns vorbringen könnt? Ihr folgt ja nur Vermutungen, und ihr stellt nur Schätzungen an.\qt@no{148}}%
\qt@newcmd\qurantrans@de@cmxxxviii{Sprich: Gott verfügt über den überzeugenden Beweisgrund. Wenn Er wollte, würde Er euch allesamt rechtleiten.\qt@no{149}}%
\qt@newcmd\qurantrans@de@cmxxxix{Sprich: Bringt eure Zeugen her, die bezeugen, daß Gott dies verboten hat. Wenn sie es auch bezeugen, so bezeuge es nicht mit ihnen, und folge nicht den Neigungen derer, die unsere Zeichen für Lüge erklären und die nicht an das Jenseits glauben und ihrem Herrn (andere) gleichsetzen.\qt@no{150}}%
\qt@newcmd\qurantrans@de@cmxl{Sprich: Kommt her, daß ich verlese, was euer Herr euch verboten hat: Ihr sollt Ihm nichts beigesellen, und die Eltern gut behandeln; und tötet nicht eure Kinder aus Angst vor Verarmung - euch und ihnen bescheren Wir doch den Lebensunterhalt; und nähert euch nicht den schändlichen Taten, was von ihnen offen und was verborgen ist; und tötet nicht den Menschen, den Gott für unantastbar erklärt hat, es sei denn bei vorliegender Berechtigung. Dies hat Er euch aufgetragen, auf daß ihr verständig werdet.\qt@no{151}}%
\qt@newcmd\qurantrans@de@cmxli{Und nähert euch nicht dem Vermögen des Waisenkindes, es sei denn auf die beste Art, bis es seine Vollkraft erreicht hat. Und gebt volles Maß und Gewicht nach Gerechtigkeit. Wir fordern von den Menschen nur das, was sie vermögen. Und wenn ihr aussagt, dann seid gerecht, auch wenn es um einen Verwandten geht. Und erfüllt den Bund Gottes. Dies hat Er euch aufgetragen, auf daß ihr es bedenket.\qt@no{152}}%
\qt@newcmd\qurantrans@de@cmxlii{Und dies ist mein Weg, er ist gerade. Folgt ihm. Und folgt nicht den verschiedenen Wegen, daß sie euch nicht in verschiedene Richtungen von seinem Weg wegführen. Dies hat Er euch aufgetragen, auf daß ihr gottesfürchtig werdet.\qt@no{153}}%
\qt@newcmd\qurantrans@de@cmxliii{Alsdann ließen Wir dem Mose das Buch zukommen, um (alles) zu vervollständigen für den, der Gutes getan hat, als eine ins einzelne gehende Darlegung aller Dinge und als Rechtleitung und Barmherzigkeit, auf daß sie an die Begegnung mit ihrem Herrn glauben.\qt@no{154}}%
\qt@newcmd\qurantrans@de@cmxliv{Und dies ist ein Buch, das Wir hinabgesandt haben, ein gesegnetes (Buch). Folgt ihm und seid gottesfürchtig, auf daß ihr Erbarmen findet.\qt@no{155}}%
\qt@newcmd\qurantrans@de@cmxlv{Damit ihr nicht sagt: «Das Buch wurde nur auf zwei Gemeinden vor uns herabgesandt, und wir ahnten nichts von dem, was sie erforscht haben.»\qt@no{156}}%
\qt@newcmd\qurantrans@de@cmxlvi{Oder damit ihr nicht sagt: «Wenn das Buch auf uns herabgesandt worden wäre, hätten wir uns besser leiten lassen als sie.» Nunmehr ist ein deutliches Zeichen von eurem Herrn und Rechtleitung und Barmherzigkeit zu euch gekommen. Wer ist denn ungerechter als der, der die Zeichen Gottes für Lüge erklärt und sich von ihnen abwendet? Wir werden denjenigen, die sich von unseren Zeichen abwenden, mit einer schlimmen Pein dafür vergelten, daß sie sich abwandten.\qt@no{157}}%
\qt@newcmd\qurantrans@de@cmxlvii{Erwarten sie denn etwas anderes, als daß die Engel zu ihnen kommen, oder daß dein Herr kommt, oder daß einige von den Zeichen deines Herrn kommen? Am Tag, da einige von den Zeichen deines Herrn kommen, nützt niemandem sein Glaube, wenn er nicht vorher geglaubt oder in seinem Glauben Gutes erworben hat. Sprich: Wartet ab. Wir warten selbst auch ab.\qt@no{158}}%
\qt@newcmd\qurantrans@de@cmxlviii{Mit denen, die ihre Religion spalteten und zu Parteien wurden, hast du nichts zu schaffen. Ihre Angelegenheit wird Gott zurückgebracht. Alsdann wird Er ihnen kundtun, was sie zu tun pflegten.\qt@no{159}}%
\qt@newcmd\qurantrans@de@cmxlix{Wer mit einer guten Tat kommt, erhält zehnmal soviel. Und wer mit einer schlechten Tat kommt, dem wird nur gleichviel vergolten. Und ihnen wird dabei nicht Unrecht getan.\qt@no{160}}%
\qt@newcmd\qurantrans@de@cml{Sprich: Mich hat mein Herr zu einem geraden Weg geleitet, einer richtigen Religion, der Glaubensrichtung Abrahams als Anhänger des reinen Glaubens, und er gehörte nicht zu den Polytheisten.\qt@no{161}}%
\qt@newcmd\qurantrans@de@cmli{Sprich: Mein Gebet und meine Kulthandlung, mein Leben und mein Sterben gehören Gott, dem Herrn der Welten.\qt@no{162}}%
\qt@newcmd\qurantrans@de@cmlii{Er hat keinen Teilhaber. Dies ist mir befohlen worden, und ich bin der erste der Gottergebenen.\qt@no{163}}%
\qt@newcmd\qurantrans@de@cmliii{Sprich: Sollte ich mir einen anderen Herrn suchen als Gott, wo Er doch der Herr aller Dinge ist? Jede Seele erwirbt (das Böse) nur zu ihrem eigenen Schaden. Und keine lasttragende (Seele) trägt die Last einer anderen. Alsdann wird eure Rückkehr zu eurem Herrn sein, und Er wird euch das kundtun, worüber ihr uneins waret.\qt@no{164}}%
\qt@newcmd\qurantrans@de@cmliv{Er ist es, der euch zu aufeinanderfolgenden Generationen auf der Erde gemacht und die einen von euch über die anderen um Rangstufen erhöht hat, um euch zu prüfen in dem, was Er euch zukommen ließ. Dein Herr ist schnell im Bestrafen, und Er ist voller Vergebung und barmherzig.\qt@no{165}}%
\qt@newcmd\qurantrans@de@cmlv{\basmalahde Alif Lam Mim Saad.\qt@no{1}}%
\qt@newcmd\qurantrans@de@cmlvi{Dies ist ein Buch, das zu dir herabgesandt worden ist - es soll in deiner Brust keine Bedrängnis seinetwegen sein -, damit du mit ihm warnst, und auch als Ermahnung für die Gläubigen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@cmlvii{Folgt dem, was zu euch von eurem Herrn herabgesandt worden ist, und folgt nicht an seiner Stelle (anderen) Freunden. Ihr laßt euch wenig ermahnen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@cmlviii{Wie manche Stadt haben Wir verderben lassen! Da kam unsere Schlagkraft über sie nachts, oder während sie zu Mittag ruhten.\qt@no{4}}%
\qt@newcmd\qurantrans@de@cmlix{So war ihr Rufen, als unsere Schlagkraft über sie kam, nur, daß sie sagten: «Wir haben gewiß Unrecht getan.»\qt@no{5}}%
\qt@newcmd\qurantrans@de@cmlx{Ja, Wir werden gewiß die, zu denen Gesandte geschickt worden sind, zur Verantwortung ziehen, und Wir werden gewiß die Gesandten zur Verantwortung ziehen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@cmlxi{Wir werden ihnen bestimmt aus (richtigem) Wissen (alles) erzählen, denn Wir waren ja nicht abwesend.\qt@no{7}}%
\qt@newcmd\qurantrans@de@cmlxii{Das Wägen an jenem Tag erfolgt der Wahrheit entsprechend. Diejenigen, deren Waagschalen schwer sind, das sind die, denen es wohl ergeht.\qt@no{8}}%
\qt@newcmd\qurantrans@de@cmlxiii{Diejenigen, deren Waagschalen leicht sind, das sind die, die sich selbst verloren haben, weil sie sich gegenüber unseren Zeichen ungerecht verhielten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@cmlxiv{Und Wir haben euch auf der Erde eine feste Stellung gegeben, und Wir haben für euch auf ihr Unterhaltsmöglichkeiten bereitet. Ihr seid aber wenig dankbar.\qt@no{10}}%
\qt@newcmd\qurantrans@de@cmlxv{Und Wir haben euch erschaffen. Dann haben Wir euch gestaltet. Dann haben Wir zu den Engeln gesprochen: «Werft euch vor Adam nieder.» Da warfen sie sich nieder, außer Iblis: er gehörte nicht zu denen, die sich niederwarfen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@cmlxvi{Er sprach: «Was hat dich daran gehindert, dich niederzuwerfen, als Ich (es) dir befohlen habe?» Er sagte: «Ich bin besser als er. Du hast mich aus Feuer erschaffen, ihn aber hast Du aus Ton erschaffen.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@cmlxvii{Er sprach: «Geh aus ihm hinunter. Dir steht es nicht zu, dich darin hochmütig zu verhalten. Geh hinaus. Du gehörst zu den Erniedrigten.»\qt@no{13}}%
\qt@newcmd\qurantrans@de@cmlxviii{Er sagte: «Gewähre mir Aufschub bis zu dem Tag, da sie erweckt werden.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@cmlxix{Er sprach: «Du sollst nun zu denen gehören, denen Aufschub gewährt wird.»\qt@no{15}}%
\qt@newcmd\qurantrans@de@cmlxx{Er sagte: «Weil Du mich hast abirren lassen, werde ich, ich schwöre es, ihnen auf deinem geraden Weg auflauern.\qt@no{16}}%
\qt@newcmd\qurantrans@de@cmlxxi{Dann werde ich zu ihnen treten von vorn und von hinten, von ihrer rechten und von ihrer linken Seite. Und Du wirst die meisten von ihnen nicht dankbar finden.»\qt@no{17}}%
\qt@newcmd\qurantrans@de@cmlxxii{Er sprach: «Geh aus ihm hinaus, verabscheut und verstoßen. Wer auch immer von ihnen dir folgt, füllen werde Ich die Hölle mit euch allen.»\qt@no{18}}%
\qt@newcmd\qurantrans@de@cmlxxiii{«O Adam, bewohne, du und deine Gattin, das Paradies. Eßt, wo ihr wollt, und nähert euch nicht diesem Baum, sonst gehört ihr zu denen, die Unrecht tun.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@cmlxxiv{Der Satan flüsterte ihnen ein, um ihnen zu zeigen, was ihnen von ihrer Blöße verborgen geblieben war. Und er sagte: «Nur deswegen hat euch euer Herr diesen Baum verboten, damit ihr nicht zu Engeln werdet oder zu denen gehöret, die ewig leben.»\qt@no{20}}%
\qt@newcmd\qurantrans@de@cmlxxv{Und er schwor ihnen: «Ich bin zu euch einer von denen, die (euch) gut raten.»\qt@no{21}}%
\qt@newcmd\qurantrans@de@cmlxxvi{Er ließ sie durch Betörung abfallen. Und als sie dann von dem Baum gekostet hatten, wurde ihnen ihre Blöße offenbar, und sie begannen, Blätter des Paradieses über sich zusammenzuheften. Und ihr Herr rief ihnen zu: «Habe ich euch nicht jenen Baum verboten und euch gesagt: Der Satan ist euch ein offenkundiger Feind?»\qt@no{22}}%
\qt@newcmd\qurantrans@de@cmlxxvii{Sie sagten: «Unser Herr, wir haben uns selbst Unrecht getan. Und wenn Du uns nicht vergibst und dich unser erbarmst, werden wir bestimmt zu den Verlierern gehören.»\qt@no{23}}%
\qt@newcmd\qurantrans@de@cmlxxviii{Er sprach: «Geht hinunter. Die einen von euch sind Feinde der anderen. Ihr habt auf der Erde Aufenthalt und Nutznießung auf eine Weile.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@cmlxxix{Er sprach: «Auf ihr werdet ihr leben, und auf ihr werdet ihr sterben, und aus ihr werdet ihr hervorgebracht werden.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@cmlxxx{O Kinder Adams, Wir haben auf euch Kleidung hinabgesandt, die eure Blöße bedeckt, und auch Prunkgewänder. Aber die Kleidung der Gottesfurcht, die ist besser. Das gehört zu den Zeichen Gottes, auf daß sie es bedenken.\qt@no{26}}%
\qt@newcmd\qurantrans@de@cmlxxxi{O Kinder Adams, der Satan soll euch bloß nicht verführen, wie er eure Eltern aus dem Paradies vertrieben hat, indem er ihnen die Kleidung wegnahm, um sie ihre Blöße sehen zu lassen. Er sieht euch, er und seine Genossen, von wo ihr sie nicht seht. Wir haben die Satane denen zu Freunden gemacht, die nicht glauben.\qt@no{27}}%
\qt@newcmd\qurantrans@de@cmlxxxii{Und wenn sie etwas Schändliches tun, sagen sie: «Wir haben es bei unseren Vätern vorgefunden, und Gott hat es uns geboten.» Sprich: Gott gebietet nicht das Schändliche. Wollt ihr denn über Gott sagen, was ihr nicht wißt?\qt@no{28}}%
\qt@newcmd\qurantrans@de@cmlxxxiii{Sprich: Mein Herr hat die Gerechtigkeit geboten, und daß ihr euer Gesicht bei jeder Moschee aufrichtet und ihn anruft, wobei ihr Ihm gegenüber aufrichtig in der Religion seid. So wie Er euch anfangs gemacht hat, werdet ihr zurückkehren.\qt@no{29}}%
\qt@newcmd\qurantrans@de@cmlxxxiv{Einen Teil hat Er rechtgeleitet, über einen Teil ist der Irrtum zu Recht gekommen, denn sie haben sich die Satane anstelle Gottes zu Freunden genommen und meinen, sie seien der Rechtleitung gefolgt.\qt@no{30}}%
\qt@newcmd\qurantrans@de@cmlxxxv{O Kinder Adams, legt euren Schmuck bei jeder Moschee an, und eßt und trinkt, aber seid nicht maßlos. Er liebt ja die Maßlosen nicht.\qt@no{31}}%
\qt@newcmd\qurantrans@de@cmlxxxvi{Sprich: Wer hat denn den Schmuck verboten, den Gott für seine Diener hervorgebracht hat, und auch die köstlichen Dinge des Lebensunterhalts? Sprich: Sie sind im diesseitigen Leben für die bestimmt, die glauben, und am Tag der Auferstehung (ihnen) besonders vorbehalten. So legen Wir die Zeichen im einzelnen dar für Leute, die Bescheid wissen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@cmlxxxvii{Sprich: Siehe, mein Herr hat die schändlichen Taten verboten, was von ihnen offen und was verborgen ist; und auch die Sünde und die Gewaltanwendung ohne vorliegende Berechtigung, und auch, daß ihr Gott solche beigesellt, für die er keine Ermächtigung herabgesandt hat, und daß ihr über Gott das sagt, was ihr nicht wißt.\qt@no{33}}%
\qt@newcmd\qurantrans@de@cmlxxxviii{Für jede Gemeinschaft ist eine Frist festgesetzt. Und wenn ihre Frist kommt, können sie nicht einmal eine Stunde zurückbleiben oder vorausgehen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@cmlxxxix{O Kinder Adams, wenn Gesandte aus eurer Mitte zu euch kommen, um euch von meinen Zeichen zu erzählen, dann haben diejenigen, die gottesfürchtig sind und Besserung bringen, nichts zu befürchten, und sie werden nicht traurig sein.\qt@no{35}}%
\qt@newcmd\qurantrans@de@cmxc{Und diejenigen, die unsere Zeichen für Lüge erklären und sich ihnen gegenüber hochmütig verhalten, das sind die Gefährten der Hölle; darin werden sie ewig weilen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@cmxci{Und wer ist denn ungerechter als der, der gegen Gott eine Lüge erdichtet oder seine Zeichen für Lüge erklärt? Jene wird ihr Anteil dem Buch gemäß erreichen. Und wenn dann unsere Boten zu ihnen kommen, um sie abzuberufen, sagen sie: «Wo sind diejenigen, die ihr anstelle Gottes anzurufen pflegtet?» Sie sagen: «Sie sind uns entschwunden.» Und sie bezeugen gegen sich selbst, daß sie ungläubig waren.\qt@no{37}}%
\qt@newcmd\qurantrans@de@cmxcii{Er spricht: «Geht ein ins Feuer mit Gemeinschaften von den Djinn und den Menschen, die vor euch dahingegangen sind.» Sooft eine Gemeinschaft hineingeht, verflucht sie ihresgleichen. Und wenn sie dann alle darin einander eingeholt haben, sagt die letzte von ihnen in bezug auf die erste: «Unser Herr, diese da haben uns in die Irre geführt. So laß ihnen eine doppelte Pein vom Feuer zukommen.» Er spricht: «Jeder erhält das Doppelte, aber ihr wißt nicht Bescheid.»\qt@no{38}}%
\qt@newcmd\qurantrans@de@cmxciii{Und die erste von ihnen sagt zu der letzten: «Ihr habt nun keinen Vorzug gegenüber uns. Kostet die Pein für das, was ihr erworben habt.»\qt@no{39}}%
\qt@newcmd\qurantrans@de@cmxciv{Denen, die unsere Zeichen für Lüge erklären und sich ihnen gegenüber hochmütig verhalten, werden die Tore des Himmels nicht geöffnet, und sie werden nicht ins Paradies eingehen, ehe denn ein Kamel durch ein Nadelöhr geht. So vergelten Wir denen, die Übeltäter sind.\qt@no{40}}%
\qt@newcmd\qurantrans@de@cmxcv{Ihnen wird die Hölle zur Lagerstätte, und über ihnen sind Decken. So vergelten Wir denen, die Unrecht tun.\qt@no{41}}%
\qt@newcmd\qurantrans@de@cmxcvi{Diejenigen, die glauben und die guten Werke tun - Wir fordern von einem jeden nur das, was er vermag -, das sind die Gefährten des Paradieses; darin werden sie ewig weilen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@cmxcvii{Und Wir nehmen weg, was in ihrer Brust an Groll dasein mag. Es fließen unter ihnen Bäche. Und sie sagen: «Lob sei Gott, der uns hierher geleitet hat! Wir hätten unmöglich die Rechtleitung gefunden, hätte uns Gott nicht rechtgeleitet. Die Gesandten unseres Herrn sind wirklich mit der Wahrheit gekommen.» Und es wird ihnen zugerufen: «Dies ist das Paradies. Es ist euch zum Erbe gegeben worden für das, was ihr zu tun pflegtet.»\qt@no{43}}%
\qt@newcmd\qurantrans@de@cmxcviii{Die Gefährten des Paradieses rufen den Gefährten des Feuers zu: «Wir haben gefunden, daß das, was uns unser Herr versprochen hat, wahr ist. Habt Ihr auch gefunden, daß das, was euch euer Herr versprochen hat, wahr ist?» Sie sagen: «Ja.» Da ruft ein Rufer unter ihnen aus: «Gottes Zorn komme über die, die Unrecht tun,\qt@no{44}}%
\qt@newcmd\qurantrans@de@cmxcix{Die vom Weg Gottes abweisen und sich ihn krumm wünschen, und die ja das Jenseits verleugnen.»\qt@no{45}}%
\qt@newcmd\qurantrans@de@m{Und zwischen ihnen ist ein Vorhang. Und auf dem Bergkamm sind Männer, die jeden an seinem Merkmal erkennen. Sie rufen den Gefährten des Paradieses zu: «Friede sei über euch!» Sie selbst aber sind nicht hineingegangen, obwohl sie es begehren.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mi{Und wenn ihre Blicke sich den Gefährten des Feuers zuwenden, sagen sie: «Unser Herr, stelle uns nicht zu den Leuten, die Unrecht tun.»\qt@no{47}}%
\qt@newcmd\qurantrans@de@mii{Und die Gefährten des Bergkammes rufen Männern, die sie an ihrem Merkmal erkennen, zu. Sie sagen: «Es hat euch nicht genützt, daß ihr (Vermögen und Freunde) gesammelt und euch hochmütig verhalten habt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@miii{Sind das diejenigen, von denen ihr geschworen habt, Gott werde ihnen keine Barmherzigkeit erweisen? - Geht (ihr) ins Paradies ein. Ihr habt nichts zu befürchten, und ihr werdet nicht traurig sein.»\qt@no{49}}%
\qt@newcmd\qurantrans@de@miv{Die Gefährten des Feuers rufen den Gefährten des Paradieses zu: «Schüttet auf uns etwas Wasser aus oder etwas von dem, was Gott euch beschert hat.» Sie sagen: «Gott hat beides den Ungläubigen verwehrt,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mv{Die ihre Religion zum Gegenstand von Zerstreuung und Spiel genommen haben und die das diesseitige Leben betört hat.» Heute werden Wir sie vergessen, wie sie die Begegnung mit diesem ihrem Tag vergaßen und wie sie unsere Zeichen zu leugnen pflegten.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mvi{Wir haben ihnen ein Buch gebracht, das Wir mit Wissen im einzelnen dargelegt haben, als Rechtleitung und Barmherzigkeit für Leute, die glauben.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mvii{Erwarten sie denn etwas anderes als seine Deutung? An dem Tag, an dem seine Deutung eintrifft, werden diejenigen, die es vorher vergessen haben, sagen: «Die Gesandten unseres Herrn sind mit der Wahrheit gekommen. Haben wir denn Fürsprecher, daß sie für uns Fürsprache einlegen? Oder können wir zurückgebracht werden, daß wir anders handeln, als wir gehandelt haben?» Sie haben sich selbst verloren, und es ist ihnen entschwunden, was sie immer wieder erdichtet haben.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mviii{Euer Herr ist Gott, der die Himmel und die Erde in sechs Tagen erschuf und sich dann auf dem Thron zurechtsetzte. Er läßt die Nacht den Tag überdecken, wobei sie ihn eilig einzuholen sucht. (Er erschuf auch) die Sonne, den Mond und die Sterne, welche durch seinen Befehl dienstbar gemacht wurden. Siehe, Ihm allein steht das Erschaffen und der Befehl zu. Gesegnet sei Gott, der Herr der Welten!\qt@no{54}}%
\qt@newcmd\qurantrans@de@mix{Und ruft euren Herrn in (sichtbarer) Demut und im Verborgenen an. Er liebt die nicht, die Übertretungen begehen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mx{Und stiftet nicht Unheil auf der Erde, nachdem sie in Ordnung gebracht worden ist. Und ruft Ihn in Furcht und Begehren an. Die Barmherzigkeit Gottes ist den Rechtschaffenen nahe.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mxi{Und Er ist es, der die Winde als frohe Kunde seiner Barmherzigkeit vorausschickt. Wenn sie dann eine schwere Bewölkung herbeitragen, treiben Wir sie zu einem abgestorbenen Land, senden dadurch das Wasser hernieder und bringen dadurch allerlei Früchte hervor. So bringen Wir (auch) die Toten hervor, auf daß ihr es bedenket.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mxii{Und das gute Land bringt seine Pflanzen mit der Erlaubnis seines Herrn hervor. Und das (Land), das schlecht ist, bringt nur mühsam etwas hervor. So legen Wir die Zeichen auf verschiedene Weise dar für Leute, die dankbar sind.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mxiii{Wir sandten Noach zu seinem Volk. Er sagte: «O mein Volk, dienet Gott. Ihr habt keinen Gott außer Ihm. Ich fürchte für euch die Pein eines gewaltigen Tages.»\qt@no{59}}%
\qt@newcmd\qurantrans@de@mxiv{Die Vornehmen aus seinem Volk sagten: «Wir sehen, daß du dich in einem offenkundigen Irrtum befindest.»\qt@no{60}}%
\qt@newcmd\qurantrans@de@mxv{Er sagte: «O mein Volk, bei mir befindet sich kein Irrtum, sondern ich bin ein Gesandter vom Herrn der Welten.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mxvi{Ich richte euch die Botschaften meines Herrn aus und rate euch gut. Und ich weiß von Gott her, was ihr nicht wißt.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mxvii{Wundert ihr euch etwa darüber, daß eine Ermahnung von eurem Herrn zu euch gekommen ist durch einen Mann aus eurer Mitte, damit er euch warne und damit ihr gottesfürchtig werdet, auf daß ihr Erbarmen findet?»\qt@no{63}}%
\qt@newcmd\qurantrans@de@mxviii{Sie aber ziehen ihn der Lüge. Da retteten Wir ihn und diejenigen, die mit ihm waren, im Schiff. Und Wir ließen diejenigen ertrinken, die unsere Zeichen für Lüge erklärten; sie waren ja ein blindes Volk.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mxix{Und (Wir sandten) zu Aad ihren Bruder Hud. Er sagte: «O mein Volk, dienet Gott. Ihr habt keinen Gott außer Ihm. Wollt ihr nicht gottesfürchtig sein?»\qt@no{65}}%
\qt@newcmd\qurantrans@de@mxx{Die Vornehmen aus seinem Volk, die ungläubig waren, sagten: «Wir sehen, daß du der Torheit verfallen bist, und wir meinen, daß du zu den Lügnern gehörst.»\qt@no{66}}%
\qt@newcmd\qurantrans@de@mxxi{Er sagte: «O mein Volk, bei mir befindet sich keine Torheit, sondern ich bin ein Gesandter vom Herrn der Welten.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mxxii{Ich richte euch die Botschaften meines Herrn aus, und ich bin für euch ein treuer Ratgeber.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mxxiii{Wundert ihr euch etwa darüber, daß eine Ermahnung von eurem Herrn zu euch gekommen ist durch einen Mann aus eurer Mitte, damit er euch warne? Gedenket, als Er euch zu Nachfolgern nach dem Volk des Noach machte und euch eine beachtlichere Stellung in seiner Schöpfung verlieh. Gedenket also der Wohltaten Gottes, auf daß es euch wohl ergehe.»\qt@no{69}}%
\qt@newcmd\qurantrans@de@mxxiv{Sie sagten: «Bist du zu uns gekommen, damit wir Gott allein dienen und das verlassen, was unsere Väter verehrt haben? Bring uns doch her, was du uns androhst, so du zu denen gehörst, die die Wahrheit sagen.»\qt@no{70}}%
\qt@newcmd\qurantrans@de@mxxv{Er sagte: «Es überfallen euch Greuel und Zorn von eurem Herrn. Streitet ihr denn mit mir über Namen, die ihr genannt habt, ihr und eure Väter, für die aber Gott keine Ermächtigung herabgesandt hat? So wartet ab. Ich bin mit euch einer von denen, die abwarten.»\qt@no{71}}%
\qt@newcmd\qurantrans@de@mxxvi{Da retteten Wir ihn und diejenigen, die mit ihm waren, aus Barmherzigkeit von Uns. Und Wir merzten den letzten Rest derer aus, die unsere Zeichen für Lüge erklärten und nicht gläubig waren.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mxxvii{Und (Wir sandten) zu Thamud ihren Bruder Salih. Er sagte: «O mein Volk, dienet Gott. Ihr habt keinen Gott außer Ihm. Ein deutliches Zeichen ist von eurem Herrn zu euch gekommen. Dies ist die Kamelstute Gottes, euch zum Zeichen. Laßt sie auf Gottes Erde weiden und rührt sie nicht mit etwas Bösem an, sonst ergreift euch eine schmerzhafte Pein.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mxxviii{Und gedenket, als Er euch zu Nachfolgern nach, Aad machte und euch auf der Erde in Stätten einwies, so daß ihr euch in ihren Ebenen Schlösser nahmt und die Berge zu Häusern meißeltet. Gedenket also der Wohltaten Gottes und verbreitet nicht Unheil auf der Erde.»\qt@no{74}}%
\qt@newcmd\qurantrans@de@mxxix{Die Vornehmen aus seinem Volk, die sich hochmütig verhielten, sagten zu denen, die wie Schwache behandelt wurden, ja zu denen von ihnen, die gläubig waren: «Wißt ihr sicher, daß Salih von seinem Herrn gesandt worden ist?» Sie sagten: «Wir glauben gewiß an das, womit er gesandt worden ist.»\qt@no{75}}%
\qt@newcmd\qurantrans@de@mxxx{Diejenigen, die sich hochmütig verhielten, sagten: «Wir verleugnen das, woran ihr glaubt.»\qt@no{76}}%
\qt@newcmd\qurantrans@de@mxxxi{Sie schnitten der Kamelstute die Flechsen durch und stachen sie, und sie rebellierten gegen den Befehl ihres Herrn. Und sie sagten: «O Salih, bring uns doch her, was du uns androhst, so du einer der Gesandten bist.»\qt@no{77}}%
\qt@newcmd\qurantrans@de@mxxxii{Da ergriff sie das Beben, und am Morgen lagen sie in ihrer Wohnstätte nieder.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mxxxiii{So kehrte er sich von ihnen ab und sagte: «O mein Volk, ich habe euch die Botschaft meines Herrn ausgerichtet, und ich habe euch gut geraten. Aber ihr liebt die nicht, die gut raten.»\qt@no{79}}%
\qt@newcmd\qurantrans@de@mxxxiv{Und (Wir sandten) Lot. Als er zu seinem Volk sagte: «Wollt ihr denn das Schändliche begehen, wie es vor euch keiner von den Weltenbewohnern begangen hat?\qt@no{80}}%
\qt@newcmd\qurantrans@de@mxxxv{Ihr geht in Begierde zu den Männern, statt zu den Frauen. Nein, ihr seid maßlose Leute.»\qt@no{81}}%
\qt@newcmd\qurantrans@de@mxxxvi{Die Antwort seines Volkes war nur, daß sie sagten: «Vertreibt sie aus eurer Stadt. Das sind Menschen, die sich rein stellen.»\qt@no{82}}%
\qt@newcmd\qurantrans@de@mxxxvii{Da retteten Wir ihn und seine Angehörigen, außer seiner Frau. Sie gehörte zu denen, die zurückblieben und dem Verderben anheimfielen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mxxxviii{Und Wir ließen einen Regen auf sie niedergehen. Schau, wie das Ende der Übeltäter war.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mxxxix{Und (Wir sandten) zu Madyan ihren Bruder Shuayb. Er sagte: «O mein Volk, dienet Gott. Ihr habt keinen Gott außer Ihm. Ein deutliches Zeichen ist von eurem Herrn zu euch gekommen, so gebt volles Maß und Gewicht und zieht den Menschen nichts ab, was ihnen gehört, und stiftet nicht Unheil auf der Erde, nachdem sie in Ordnung gebracht worden ist. Das ist bestimmt besser für euch, so ihr gläubig seid.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mxl{Und lauert nicht auf jedem Weg, indem ihr droht und vom Weg Gottes den abweist, der an Ihn glaubt, und euch ihn krumm wünscht. Und gedenket, als ihr wenige waret und Er euch zu vielen machte. Und schaut, wie das Ende der Unheilstifter war.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mxli{Und wenn eine Gruppe von euch an das glaubt, womit ich gesandt worden bin, eine andere Gruppe aber nicht glaubt, so geduldet euch, bis Gott zwischen uns urteilt. Er ist der Beste derer, die Urteile fällen.»\qt@no{87}}%
\qt@newcmd\qurantrans@de@mxlii{Die Vornehmen aus seinem Volk, die sich hochmütig verhielten, sagten: «Wir werden dich, o Shuayb, und diejenigen, die mit dir glauben, bestimmt aus unserer Stadt vertreiben, oder ihr kehrt zu unserer Glaubensrichtung zurück.» Er sagte: «Was denn, auch wenn es uns zuwider ist?\qt@no{88}}%
\qt@newcmd\qurantrans@de@mxliii{Wir würden gegen Gott eine Lüge erdichten, sollten wir zu eurer Glaubensrichtung zurückkehren, nachdem uns Gott aus ihr errettet hat. Wir können unmöglich zu ihr zurückkehren, es sei denn, Gott unser Herr, wollte es. Unser Herr umfaßt alle Dinge in seinem Wissen. Auf Gott vertrauen wir. Unser Herr, richte zwischen uns und unserem Volk nach der Wahrheit. Du bist der beste Richter.»\qt@no{89}}%
\qt@newcmd\qurantrans@de@mxliv{Und die Vornehmen aus seinem Volk, die ungläubig waren, sagten: «Wenn ihr dem Shuayb folgt, werdet ihr bestimmt Verlierer sein.»\qt@no{90}}%
\qt@newcmd\qurantrans@de@mxlv{Da ergriff sie das Beben, und am Morgen lagen sie in ihrer Wohnstätte nieder.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mxlvi{Diejenigen, die Shuayb der Lüge ziehen, waren, als hätten sie nicht darin lange gewohnt. Diejenigen, die Shuayb der Lüge ziehen, waren selbst die Verlierer.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mxlvii{So kehrte er sich von ihnen ab und sagte: «O mein Volk, ich habe euch die Botschaften meines Herrn ausgerichtet, und ich habe euch gut geraten. Wie könnte ich betrübt sein über ungläubige Leute?»\qt@no{93}}%
\qt@newcmd\qurantrans@de@mxlviii{Und Wir haben keinen Propheten in eine Stadt gesandt, ohne daß Wir ihre Bewohner durch Not und Leid heimgesucht hätten, auf daß sie sich vielleicht demütigen.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mxlix{Dann tauschten Wir anstelle des Schlimmen Gutes ein, bis sie sich vermehrten und sagten: «Auch unsere Väter haben Leid und Erfreuliches erfahren.» So ergriffen Wir sie plötzlich, ohne daß sie es merkten.\qt@no{95}}%
\qt@newcmd\qurantrans@de@ml{Hätten die Bewohner der Städte geglaubt und wären sie gottesfürchtig gewesen, hätten Wir ihnen die Segnungen vom Himmel und von der Erde aufgetan. Aber sie erklärten (die Botschaft) für Lüge, so ergriffen Wir sie für das, was sie erworben haben.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mli{Wähnen sich denn die Bewohner der Städte in Sicherheit davor, daß unsere Schlagkraft nachts über sie kommt, während sie schlafen?\qt@no{97}}%
\qt@newcmd\qurantrans@de@mlii{Oder wähnen sich die Bewohner der Städte in Sicherheit davor, daß unsere Schlagkraft am helllichten Tag über sie kommt, während sie sich dem Spiel hingeben?\qt@no{98}}%
\qt@newcmd\qurantrans@de@mliii{Wähnen sie sich in Sicherheit vor den Ränken Gottes? In Sicherheit vor den Ränken Gottes wähnen sich nur die Leute, die Verlierer sind.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mliv{Ist denen, die die Erde nach ihren Bewohnern erben, nicht deutlich geworden, daß, wenn Wir es wollten, Wir sie für ihre Sünden treffen? Und Wir versiegeln ihre Herzen, so daß sie nicht hören.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mlv{Über diese Städte erzählen Wir dir einiges von ihren Berichten. Ihre Gesandten kamen zu ihnen mit den deutlichen Zeichen, sie vermochten aber nicht an das zu glauben, was sie früher für Lüge erklärt hatten. So versiegelt Gott die Herzen der Ungläubigen.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mlvi{Und bei den meisten von ihnen haben Wir keine Vertragstreue gefunden. Und Wir haben gefunden, daß die meisten von ihnen Frevler sind.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mlvii{Dann entsandten Wir nach ihnen Mose mit unseren Zeichen zu Pharao und seinen Vornehmen. Sie handelten an ihnen ungerecht. So schau, wie das Ende der Unheilstifter war.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mlviii{Und Mose sagte: «O Pharao, ich bin ein Gesandter vom Herrn der Welten,\qt@no{104}}%
\qt@newcmd\qurantrans@de@mlix{Verpflichtet und darauf bedacht, über Gott nur die Wahrheit zu sagen. Ich bin zu euch mit einem deutlichen Zeichen von eurem Herrn gekommen. So schick die Kinder Israels mit mir weg.»\qt@no{105}}%
\qt@newcmd\qurantrans@de@mlx{Er sagte: «Wenn du mit einem Zeichen gekommen bist, dann bring es her, so du zu denen gehörst, die die Wahrheit sagen.»\qt@no{106}}%
\qt@newcmd\qurantrans@de@mlxi{Er warf seinen Stab, und da war er eine offenkundige Schlange.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mlxii{Und er zog seine Hand heraus, da war sie weiß für die Zuschauer.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mlxiii{Die Vornehmen aus dem Volk Pharaos sagten: «Dieser ist ja ein erfahrener Zauberer,\qt@no{109}}%
\qt@newcmd\qurantrans@de@mlxiv{Der euch aus eurem Land vertreiben will. Was befehlt ihr nun?»\qt@no{110}}%
\qt@newcmd\qurantrans@de@mlxv{Sie sagten: «Stell ihn und seinen Bruder zurück, und schick zu den Städten Leute, die sie versammeln,\qt@no{111}}%
\qt@newcmd\qurantrans@de@mlxvi{Damit sie dir jeden erfahrenen Zauberer herbringen.»\qt@no{112}}%
\qt@newcmd\qurantrans@de@mlxvii{Und die Zauberer kamen zu Pharao. Sie sagten: «Wir bekommen wohl eine Belohnung, wenn wir es sind, die siegen?»\qt@no{113}}%
\qt@newcmd\qurantrans@de@mlxviii{Er sagte: «Ja. Und ihr werdet auch zu denen gehören, die in (meine) Nähe zugelassen werden.»\qt@no{114}}%
\qt@newcmd\qurantrans@de@mlxix{Sie sagten: «O Mose, entweder wirfst du, oder wir sind es, die (zuerst) werfen.»\qt@no{115}}%
\qt@newcmd\qurantrans@de@mlxx{Er sagte: «Werft ihr (zuerst).» Als sie nun warfen, bezauberten sie die Augen der Menschen und jagten ihnen Angst ein, und sie brachten einen gewaltigen Zauber vor.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mlxxi{Und Wir gaben dem Mose ein: «Wirf deinen Stab.» Da fing er an zu verschlingen, was sie vorgaukelten.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mlxxii{So bestätigte sich die Wahrheit, und das, was sie machten, erwies sich als falsch.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mlxxiii{Sie wurden dort besiegt und kehrten als Erniedrigte zurück.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mlxxiv{Und die Zauberer wurden in Anbetung zu Boden geworfen.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mlxxv{Sie sagten: «Wir glauben an den Herrn der Welten,\qt@no{121}}%
\qt@newcmd\qurantrans@de@mlxxvi{Den Herrn von Mose und Aaron.»\qt@no{122}}%
\qt@newcmd\qurantrans@de@mlxxvii{Pharao sagte: «Ihr glaubt an ihn, bevor ich es euch erlaube? Das sind ja Ränke, die ihr in der Stadt geschmiedet habt, um ihre Bewohner aus ihr zu vertreiben. Aber ihr werdet es zu wissen bekommen.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mlxxviii{Ich werde eure Hände und eure Füße wechselseitig abhacken, und dann werde ich euch allesamt kreuzigen lassen.»\qt@no{124}}%
\qt@newcmd\qurantrans@de@mlxxix{Sie sagten: «Wir kehren zu unserem Herrn zurück.\qt@no{125}}%
\qt@newcmd\qurantrans@de@mlxxx{Nichts anderes läßt dich uns grollen, als daß wir an die Zeichen unseres Herrn glaubten, als sie zu uns kamen. Unser Herr, gieße Geduld über uns aus und berufe uns als Gottergebene ab.»\qt@no{126}}%
\qt@newcmd\qurantrans@de@mlxxxi{Die Vornehmen aus dem Volk Pharaos sagten: «Willst du zulassen, daß Mose und sein Volk auf der Erde Unheil stiften und daß er dich und deine Götter verläßt?» Er sagte: «Wir werden ihre Söhne ermorden und nur ihre Frauen am Leben lassen. Wir sind ja Zwingherrscher über sie.»\qt@no{127}}%
\qt@newcmd\qurantrans@de@mlxxxii{Mose sagte zu seinem Volk: «Sucht Hilfe bei Gott, und seid geduldig. Die Erde gehört Gott, Er gibt sie zum Erbe, wem von seinen Dienern Er will. Und das (gute) Ende gehört den Gottesfürchtigen.»\qt@no{128}}%
\qt@newcmd\qurantrans@de@mlxxxiii{Sie sagten: «Uns ist Leid zugefügt worden, bevor du zu uns kamst und nachdem du zu uns gekommen bist.» Er sagte: «Möge euer Herr euren Feind verderben lassen und euch zu Nachfolgern auf der Erde einsetzen und dann sehen, wie ihr handelt.»\qt@no{129}}%
\qt@newcmd\qurantrans@de@mlxxxiv{Und Wir ließen über die Leute des Pharao Dürrejahre und Mangel an Früchten kommen, auf daß sie es bedenken.\qt@no{130}}%
\qt@newcmd\qurantrans@de@mlxxxv{Wenn ihnen dann etwas Gutes zufiel, sagten sie: «Wir haben es verdient»; und wenn sie etwas Übles traf, sahen sie ein böses Omen in Mose und denen, die mit ihm waren. Aber ihr Omen ist bei Gott, jedoch wissen die meisten von ihnen nicht Bescheid.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mlxxxvi{Und sie sagten: «Welches Zeichen du auch vorbringen magst, um uns damit zu bezaubern, wir werden dir nicht glauben.»\qt@no{132}}%
\qt@newcmd\qurantrans@de@mlxxxvii{So schickten Wir über sie die Flut, die Heuschrecken, die Läuse, die Frösche und das Blut als einzeln vorgebrachte Zeichen. Da verhielten sie sich hochmütig und waren Leute, die Übeltäter waren.\qt@no{133}}%
\qt@newcmd\qurantrans@de@mlxxxviii{Als nun das Zorngericht sie überfiel, sagten sie: «O Mose, rufe für uns deinen Herrn an aufgrund seines Bundes mit dir. Wenn du das Zorngericht von uns aufhebst, dann werden wir dir glauben und die Kinder Israels mit dir wegschicken.»\qt@no{134}}%
\qt@newcmd\qurantrans@de@mlxxxix{Als Wir dann das Zorngericht von ihnen aufhoben auf eine Frist, die sie ja erreichen sollten, brachen sie gleich ihr Wort.\qt@no{135}}%
\qt@newcmd\qurantrans@de@mxc{Da rächten Wir uns an ihnen und ließen sie im Meer ertrinken dafür, daß sie unsere Zeichen für Lüge erklärten und sie unbeachtet ließen.\qt@no{136}}%
\qt@newcmd\qurantrans@de@mxci{Und Wir gaben den Leuten, die wie Schwache behandelt wurden, zum Erbe die östlichen und die westlichen Gegenden der Erde, die Wir mit Segen bedacht haben. Und das schönste Wort deines Herrn erfüllte sich an den Kindern Israels dafür, daß sie geduldig waren. Und Wir zerstörten, was Pharao und sein Volk zu machen und was sie aufzurichten pflegten.\qt@no{137}}%
\qt@newcmd\qurantrans@de@mxcii{Und Wir ließen die Kinder Israels das Meer überqueren. Sie trafen auf Leute, die sich dem Dienst an ihren Götzen widmeten. Sie sagten: «O Mose, mache uns einen Gott, wie sie ja Götter haben.» Er sagte: «Ihr seid Leute, die töricht sind.\qt@no{138}}%
\qt@newcmd\qurantrans@de@mxciii{Was die da vollziehen, wird dem Verderben anheimfallen, und zunichte wird, was sie zu tun pflegten.»\qt@no{139}}%
\qt@newcmd\qurantrans@de@mxciv{Er sagte: «Sollte ich euch einen anderen Gott wünschen als Gott, wo Er euch doch vor den Weltenbewohnern bevorzugt hat?»\qt@no{140}}%
\qt@newcmd\qurantrans@de@mxcv{Und als Wir euch vor den Leuten Pharaos retteten, als sie euch eine schlimme Pein zufügten, indem sie eure Söhne ermordeten und nur eure Frauen am Leben ließen. Darin war für euch eine gewaltige Prüfung von eurem Herrn.\qt@no{141}}%
\qt@newcmd\qurantrans@de@mxcvi{Und wir verabredeten uns mit Mose dreißig Nächte lang und ergänzten sie mit weiteren zehn. So vervollständigte sich der Termin seines Herrn auf vierzig Nächte. Und Mose sagte zu seinem Bruder Aaron: «Sei mein Nachfolger bei meinem Volk, sorge für Ordnung und folge nicht dem Weg der Unheilstifter.»\qt@no{142}}%
\qt@newcmd\qurantrans@de@mxcvii{Als Mose zu unserem Termin kam und sein Herr zu ihm sprach, sagte er: «Mein Herr, zeige (Dich) mir, daß ich zu Dir schaue.» Er sprach: «Du wirst Mich nicht sehen. Aber schau zu dem Berg. Wenn er an seiner Stelle festbleibt, wirst du Mich sehen.» Als sein Herr sich vor dem Berg enthüllte, machte Er ihn zu Staub, und Mose fiel zu Boden wie vom Blitz getroffen. Als er aufwachte, sagte er: «Preis sei Dir! Ich wende mich Dir zu, und ich bin der erste der Gläubigen.»\qt@no{143}}%
\qt@newcmd\qurantrans@de@mxcviii{Er sprach: «O Mose, Ich habe dich durch meine Botschaften und mein Gespräch (mit dir) vor den Menschen auserwählt. So nimm, was Ich dir zukommen lasse, und sei einer der Dankbaren.»\qt@no{144}}%
\qt@newcmd\qurantrans@de@mxcix{Und Wir schrieben ihm auf den Tafeln über alle Dinge, eine Ermahnung und eine ins einzelne gehende Darlegung aller Dinge. «So nimm sie mit voller Kraft und befiehl deinem Volk, sich an das Schönste in ihnen zu halten. Ich werde euch die Wohnstätte der Frevler zeigen.»\qt@no{145}}%
\qt@newcmd\qurantrans@de@mc{Abweisen werde Ich von meinen Zeichen diejenigen, die sich auf der Erde zu Unrecht hochmütig verhalten. Wenn sie auch jedes Zeichen sehen, glauben sie nicht daran. Und wenn sie den Weg des rechten Wandels sehen, nehmen sie ihn sich nicht zum Weg. Wenn sie den Weg der Verirrung sehen, nehmen sie ihn sich zum Weg. Dies, weil sie unsere Zeichen für Lüge erklären und sie unbeachtet lassen.\qt@no{146}}%
\qt@newcmd\qurantrans@de@mci{Diejenigen, die unsere Zeichen und die Begegnung mit dem Jenseits für Lüge erklären, deren Werke sind wertlos. Wird ihnen denn für etwas anderes vergolten als für das, was sie taten?\qt@no{147}}%
\qt@newcmd\qurantrans@de@mcii{Und die Leute des Mose machten sich, nachdem er weggegangen war, aus ihren Schmucksachen ein Kalb als Leib, der blökte. Sahen sie denn nicht, daß es nicht zu ihnen sprechen und sie nicht den Weg führen konnte? Sie machten es sich und taten Unrecht.\qt@no{148}}%
\qt@newcmd\qurantrans@de@mciii{Und als sich bei ihnen die Reue einstellte und sie einsahen, daß sie irregegangen waren, sagten sie: «Wenn unser Herr sich nicht unser erbarmt und uns vergibt, werden wir sicher zu den Verlierern gehören.»\qt@no{149}}%
\qt@newcmd\qurantrans@de@mciv{Als Mose zornig und voller Bedauern zu seinem Volk zurückkam, sagte er: «Schlimm ist das, was ihr, nachdem ich weggegangen war, begangen habt. Wolltet ihr den Befehl eures Herrn beschleunigen?» Er warf die Tafeln nieder und packte den Kopf seines Bruders und zog ihn an sich. Dieser sagte: «Sohn meiner Mutter, die Leute behandelten mich wie einen Schwachen und hätten mich beinahe getötet. So laß nicht die Feinde Schadenfreude über mich haben und stelle mich nicht zu den Leuten, die Unrecht tun.»\qt@no{150}}%
\qt@newcmd\qurantrans@de@mcv{Er sagte: «Mein Herr, vergib mir und meinem Bruder, und laß uns in deine Barmherzigkeit eingehen. Du bist der Barmherzigste der Barmherzigen.»\qt@no{151}}%
\qt@newcmd\qurantrans@de@mcvi{Diejenigen, die sich das Kalb nahmen, wird Zorn von ihrem Herrn und Erniedrigung im diesseitigen Leben treffen. So vergelten Wir denen, die Lügen erdichten.\qt@no{152}}%
\qt@newcmd\qurantrans@de@mcvii{Diejenigen, die die bösen Taten begehen, aber danach umkehren und glauben - siehe, dein Herr ist danach voller Vergebung und barmherzig.\qt@no{153}}%
\qt@newcmd\qurantrans@de@mcviii{Und als sich der Zorn in Mose beruhigt hatte, nahm er die Tafeln. In ihrer Abschrift ist Rechtleitung und Barmherzigkeit für die, die vor ihrem Herrn Ehrfurcht haben.\qt@no{154}}%
\qt@newcmd\qurantrans@de@mcix{Und Mose wählte zu unserem Termin aus seinem Volk siebzig Männer. Als nun das Beben sie ergriff, sagte er: «Mein Herr, wenn Du gewollt hättest, hättest Du sie vorher verderben lassen, und mich auch. Willst Du uns verderben lassen für das, was die Toren unter uns getan haben? Es ist doch deine Versuchung, mit der Du irreführst, wen Du willst, und rechtleitest, wen Du willst. Du bist unser Freund, so vergib uns und erbarm dich unser. Du bist der Beste derer, die vergeben.\qt@no{155}}%
\qt@newcmd\qurantrans@de@mcx{Und bestimme für uns im Diesseits Gutes, und auch im Jenseits. Wir sind zu Dir reumütig zurückgekehrt.» Er sprach: «Mit meiner Pein treffe Ich, wen Ich will. Und meine Barmherzigkeit umfaßt alle Dinge. Ich werde sie für die bestimmen, die gottesfürchtig sind und die Abgabe entrichten, und die an unsere Zeichen glauben»,\qt@no{156}}%
\qt@newcmd\qurantrans@de@mcxi{Die dem Gesandten, dem ungelehrten Propheten, folgen, den sie bei sich in der Tora und im Evangelium verzeichnet finden. Er befiehlt ihnen das Rechte und verbietet ihnen das Verwerfliche, er erlaubt ihnen die köstlichen Dinge und verbietet ihnen die schlechten, und er nimmt ihnen ihre Last und die Fesseln, die auf ihnen lagen, ab. Diejenigen nun, die an ihn glauben, ihm beistehen, ihn unterstützen und dem Licht, das mit ihm herabgesandt worden ist, folgen, das sind die, denen es wohl ergeht.\qt@no{157}}%
\qt@newcmd\qurantrans@de@mcxii{Sprich: O Menschen, ich bin an euch alle der Gesandte Gottes, dem die Königsherrschaft der Himmel und der Erde gehört. Es gibt keinen Gott außer Ihm. Er macht lebendig und läßt sterben. So glaubt an Gott und seinen Gesandten, den ungelehrten Propheten, der an Gott und seine Worte glaubt, und folgt ihm, auf daß ihr die Rechtleitung findet.\qt@no{158}}%
\qt@newcmd\qurantrans@de@mcxiii{Und unter dem Volk Moses ist eine Gemeinschaft, die nach der Wahrheit leitet und nach ihr gerecht handelt.\qt@no{159}}%
\qt@newcmd\qurantrans@de@mcxiv{Und Wir zerteilten sie in zwölf Stämme und Gemeinschaften. Und Wir gaben dem Mose, als sein Volk ihn um Wasser zu trinken bat, ein: «Schlag mit deinem Stab auf den Stein.» Da traten aus ihm zwölf Quellen heraus, und jede Menschengruppe wußte nun, wo ihre Trinkstelle war. Und Wir ließen die Wolken sie überschatten und sandten auf sie das Manna und die Wachteln hinab: «Eßt von den köstlichen Dingen, die Wir euch beschert haben.» Und nicht Uns taten sie Unrecht, sondern sich selbst haben sie Unrecht getan.\qt@no{160}}%
\qt@newcmd\qurantrans@de@mcxv{Und als zu ihnen gesagt wurde: «Bewohnet diese Stadt und eßt davon, wo ihr wollt. Und sagt: Entlastung!, und betretet das Tor in der Haltung der Niederwerfung, dann vergeben Wir euch eure Verfehlungen. Und Wir werden den Rechtschaffenen noch mehr geben.\qt@no{161}}%
\qt@newcmd\qurantrans@de@mcxvi{Da vertauschten es diejenigen von ihnen, die Unrecht taten, mit einem Ausspruch, der anders war als das, was ihnen gesagt worden war. Da sandten Wir über sie ein Zorngericht vom Himmel herab dafür, daß sie Unrecht taten.\qt@no{162}}%
\qt@newcmd\qurantrans@de@mcxvii{Und frag sie nach der Stadt, die am Meer lag, als sie am Sabbat Übertretungen begingen, wie ihre Fische zu ihnen sichtbar geschwommen kamen am Tag, an dem sie Sabbat hatten, und wie sie am Tag, an dem sie den Sabbat nicht hielten, zu ihnen nicht kamen. So prüften Wir sie dafür, daß sie frevelten.\qt@no{163}}%
\qt@newcmd\qurantrans@de@mcxviii{Und als eine Gemeinschaft von ihnen sagte: «Warum ermahnt ihr Leute, die Gott verderben oder mit einer harten Pein peinigen wird?» Sie sagten: «Um eine Entschuldigung bei eurem Herrn zu haben, und auf daß sie vielleicht gottesfürchtig werden.»\qt@no{164}}%
\qt@newcmd\qurantrans@de@mcxix{Und als sie vergessen hatten, womit sie ermahnt worden waren, retteten Wir diejenigen, die das Böse verboten, und ergriffen diejenigen, die Unrecht taten, mit einer schlimmen Pein dafür, daß sie frevelten.\qt@no{165}}%
\qt@newcmd\qurantrans@de@mcxx{Als sie sich rebellisch weigerten, das zu unterlassen, was ihnen verboten war, sprachen Wir zu ihnen: «Werdet zu verabscheuten Affen.»\qt@no{166}}%
\qt@newcmd\qurantrans@de@mcxxi{Und als dein Herr ankündigte, Er werde gegen sie bis zum Tag der Auferstehung Leute schicken, die ihnen eine schlimme Pein zufügen. Dein Herr ist schnell im Bestrafen, und Er ist voller Vergebung und barmherzig.\qt@no{167}}%
\qt@newcmd\qurantrans@de@mcxxii{Und Wir zerteilten sie auf der Erde in Gemeinschaften. Unter ihnen gab es Rechtschaffene und solche, die es nicht waren. Und Wir prüften sie mit Gutem und Bösem, auf daß sie umkehren.\qt@no{168}}%
\qt@newcmd\qurantrans@de@mcxxiii{Auf sie folgten Nachfolger, die das Buch erbten. Sie greifen nach den Gütern des Diesseits und sagen: «Es wird uns vergeben.» Und wenn sich ihnen gleiche Güter bieten, greifen sie danach. Wurde nicht von ihnen die Verpflichtung des Buches entgegengenommen, sie sollen über Gott nur die Wahrheit sagen? Sie haben doch das, was darin steht, erforscht. Und die jenseitige Wohnstätte ist gewiß besser für die, die gottesfürchtig sind. Habt ihr denn keinen Verstand?\qt@no{169}}%
\qt@newcmd\qurantrans@de@mcxxiv{Und diejenigen, die am Buch festhalten und das Gebet verrichten - siehe, Wir lassen den Lohn derer, die Besserung zeigen, nicht verlorengehen.\qt@no{170}}%
\qt@newcmd\qurantrans@de@mcxxv{Und als Wir über sie den Berg schüttelten, als wäre er eine überschattende Hülle, und sie meinten, er würde auf sie fallen: «Nehmt, was Wir euch zukommen ließen, mit aller Kraft und gedenket dessen, was darin steht, auf daß ihr gottesfürchtig werdet.»\qt@no{171}}%
\qt@newcmd\qurantrans@de@mcxxvi{Und als dein Herr aus den Lenden der Kinder Adams ihre Nachkommenschaft nahm und gegen sich selbst zeugen ließ: «Bin Ich nicht euer Herr?» Sie sagten: «Jawohl, wir bezeugen es.» (Dies,) damit ihr nicht am Tag der Auferstehung sagt: «Wir ahnten nichts davon»,\qt@no{172}}%
\qt@newcmd\qurantrans@de@mcxxvii{Oder auch nicht sagt: «Unsere Väter waren doch zuvor Polytheisten, und wir sind nur eine Nachkommenschaft nach ihnen. Willst Du uns denn verderben für die Taten derer, die Falsches tun?»\qt@no{173}}%
\qt@newcmd\qurantrans@de@mcxxviii{So legen Wir die Zeichen im einzelnen dar, auf daß sie umkehren.\qt@no{174}}%
\qt@newcmd\qurantrans@de@mcxxix{Und verlies ihnen den Bericht über den, dem Wir unsere Zeichen zukommen ließen und der sich dann ihrer entledigte. Da holte ihn der Satan ein, und er wurde einer von denen, die irregegangen sind.\qt@no{175}}%
\qt@newcmd\qurantrans@de@mcxxx{Und wenn Wir gewollt hätten, hätten Wir ihn durch sie erhöht. Aber er wandte sich der Erde zu und folgte seiner Neigung. So ist es mit ihm wie mit einem Hund: Gehst du auf ihn los, hängt er die Zunge heraus; läßt du ihn in Ruhe, hängt er auch die Zunge heraus. So ist es mit den Leuten, die unsere Zeichen für Lüge erklären. Erzähle also, was es zu erzählen gibt, auf daß sie nachdenken.\qt@no{176}}%
\qt@newcmd\qurantrans@de@mcxxxi{Schlimm ist es mit den Leuten, die unsere Zeichen für Lüge erklären und sich selbst Unrecht tun.\qt@no{177}}%
\qt@newcmd\qurantrans@de@mcxxxii{Wen Gott rechtleitet, der ist es, der der Rechtleitung folgt. Und die, die Er irreführt, das sind die Verlierer.\qt@no{178}}%
\qt@newcmd\qurantrans@de@mcxxxiii{Wir haben für die Hölle viele von den Djinn und den Menschen geschaffen. Sie haben Herzen, mit denen sie nicht begreifen; sie haben Augen, mit denen sie nicht sehen; und sie haben Ohren, mit denen sie nicht hören. Sie sind wie das Vieh, ja sie irren noch mehr ab. Das sind die, die (alles) unbeachtet lassen.\qt@no{179}}%
\qt@newcmd\qurantrans@de@mcxxxiv{Gott gehören die schönsten Namen. So ruft Ihn damit an und laßt die stehen, die über seine Namen abwegig denken. Ihnen wird vergolten für das, was sie taten.\qt@no{180}}%
\qt@newcmd\qurantrans@de@mcxxxv{Und unter denen, die Wir erschaffen haben, ist eine Gemeinschaft (von Menschen), die nach der Wahrheit leiten und nach ihr gerecht handeln.\qt@no{181}}%
\qt@newcmd\qurantrans@de@mcxxxvi{Und diejenigen, die unsere Zeichen für Lüge erklären, werden Wir Schritt für Schritt (dem Verderben) näherbringen, von wo sie es nicht wissen.\qt@no{182}}%
\qt@newcmd\qurantrans@de@mcxxxvii{Und Ich gewähre ihnen Aufschub. Meine List ist fest.\qt@no{183}}%
\qt@newcmd\qurantrans@de@mcxxxviii{Denken sie denn nicht darüber nach? Ihr Gefährte leidet doch nicht an Besessenheit; er ist nur ein deutlicher Warner.\qt@no{184}}%
\qt@newcmd\qurantrans@de@mcxxxix{Haben sie denn nicht das Reich der Himmel und der Erde und all das, was Gott erschaffen hat, betrachtet, und auch daß ihre Frist vielleicht nahegekommen ist? An welche Botschaft nach dieser wollen sie denn sonst glauben?\qt@no{185}}%
\qt@newcmd\qurantrans@de@mcxl{Wen Gott irreführt, der hat niemanden, der ihn rechtleiten könnte; und Er läßt sie im Übermaß ihres Frevels blind umherirren.\qt@no{186}}%
\qt@newcmd\qurantrans@de@mcxli{Sie fragen dich nach der Stunde, wann sie feststehen wird. Sprich: Nur mein Herr weiß über sie Bescheid. Nur Er wird sie zu ihrer Zeit erscheinen lassen. Schwer lastet sie in den Himmeln und auf der Erde. Sie wird euch plötzlich überkommen. Sie fragen dich, als ob du eindringlich um Auskunft über sie bittest. Sprich: Nur Gott weiß über sie Bescheid. Aber die meisten Menschen wissen nicht Bescheid.\qt@no{187}}%
\qt@newcmd\qurantrans@de@mcxlii{Sprich: Ich kann mir selbst weder Nutzen noch Schaden bringen, außer was Gott will. Wenn ich über das Unsichtbare Bescheid wüßte, würde ich mir selbst viel Gutes verschaffen, und das Böse würde mich nicht berühren. Ich bin nur ein Warner und ein Freudenbote für Leute, die glauben.\qt@no{188}}%
\qt@newcmd\qurantrans@de@mcxliii{Er ist es, der euch aus einem einzigen Wesen erschaffen hat, und Er hat aus ihm seine Gattin gemacht, damit er bei ihr wohne. Und als er sie beschlafen hatte, trug sie dann eine leichte Leibesfrucht und verbrachte damit eine Zeit. Als sie schwer zu tragen hatte, riefen die beiden Gott, ihren Herrn, an: «Wenn Du uns ein gesundes Kind schenkst, werden wir gewiß zu den Dankbaren gehören.»\qt@no{189}}%
\qt@newcmd\qurantrans@de@mcxliv{Als Er ihnen dann ein gesundes Kind schenkte, stellten sie Ihm Gefährten zur Seite, die an dem teilhaben sollten, was Er ihnen geschenkt hatte. Aber Gott ist erhaben über das, was sie (Ihm) beigesellen.\qt@no{190}}%
\qt@newcmd\qurantrans@de@mcxlv{Wollen sie (Ihm) denn solche beigesellen, die nichts erschaffen, aber selbst erschaffen sind,\qt@no{191}}%
\qt@newcmd\qurantrans@de@mcxlvi{Und die ihnen keine Unterstützung gewähren, noch sich selbst helfen können?\qt@no{192}}%
\qt@newcmd\qurantrans@de@mcxlvii{Und wenn ihr sie zur Rechtleitung ruft, folgen sie euch nicht. Es ist für euch gleich, ob ihr sie ruft oder ob ihr schweigt.\qt@no{193}}%
\qt@newcmd\qurantrans@de@mcxlviii{Diejenigen, die ihr anstelle Gottes anruft, sind nur Diener wie ihr selbst. So ruft sie doch an, daß sie euch erhören, so ihr die Wahrheit sagt.\qt@no{194}}%
\qt@newcmd\qurantrans@de@mcxlix{Haben sie denn (überhaupt) Füße, mit denen sie gehen, oder haben sie Hände, mit denen sie gewaltig zugreifen, oder haben sie Augen, mit denen sie sehen, oder haben sie Ohren, mit denen sie hören? Sprich: Ruft eure Teilhaber an, und dann geht gegen mich mit eurer List vor und gewährt mir keinen Aufschub.\qt@no{195}}%
\qt@newcmd\qurantrans@de@mcl{Mein Freund ist Gott, der das Buch herabgesandt hat, und Er schenkt seine Freundschaft den Rechtschaffenen.\qt@no{196}}%
\qt@newcmd\qurantrans@de@mcli{Diejenigen, die ihr anstelle Gottes anruft, können euch keine Unterstützung gewähren, noch können sie sich selbst helfen.\qt@no{197}}%
\qt@newcmd\qurantrans@de@mclii{Und wenn ihr sie zur Rechtleitung ruft, hören sie nicht. Du siehst, wie sie dich anschauen, aber sie sehen nicht.\qt@no{198}}%
\qt@newcmd\qurantrans@de@mcliii{Nimm das Gute und Leichte, gebiete das Rechte und wende dich von den Törichten ab.\qt@no{199}}%
\qt@newcmd\qurantrans@de@mcliv{Und wenn dich vom Satan her ein Stachel aufstachelt, dann suche Zuflucht bei Gott. Er hört und weiß alles.\qt@no{200}}%
\qt@newcmd\qurantrans@de@mclv{Diejenigen, die gottesfürchtig sind, wenn sie eine Heimsuchung vom Satan her trifft, erinnern sich, und sogleich werden sie einsichtig.\qt@no{201}}%
\qt@newcmd\qurantrans@de@mclvi{Ihre Brüder aber bestärken sie im Irrtum, und dann lassen sie darin nicht nach.\qt@no{202}}%
\qt@newcmd\qurantrans@de@mclvii{Und wenn du ihnen kein Zeichen vorbringst, sagen sie: «Hättest du es dir doch selbst ausgesucht!» Sprich: Ich folge nur dem, was mir von meinem Herrn offenbart wird. Dies sind einsichtbringende Zeichen von eurem Herrn und eine Rechtleitung und eine Barmherzigkeit für Leute, die glauben.\qt@no{203}}%
\qt@newcmd\qurantrans@de@mclviii{Und wenn der Koran verlesen wird, dann hört zu und seid still, auf daß ihr Erbarmen findet.\qt@no{204}}%
\qt@newcmd\qurantrans@de@mclix{Und gedenke deines Herrn in deinem Inneren in Demut und Furcht und ohne lautes Aussprechen, am Morgen und am Abend. Und sei nicht einer von denen, die (dies) unbeachtet lassen.\qt@no{205}}%
\qt@newcmd\qurantrans@de@mclx{Diejenigen, die bei deinem Herrn sind, weigern sich nicht hochmütig, Ihm zu dienen. Sie preisen Ihn und werfen sich vor Ihm nieder.\qt@no{206}}%
\qt@newcmd\qurantrans@de@mclxi{\basmalahde Sie fragen dich nach der Beute. Sprich: Die Beute gehört Gott und dem Gesandten. So fürchtet Gott und stiftet Aussöhnung untereinander, und gehorchet Gott und seinem Gesandten, so ihr gläubig seid.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mclxii{Die Gläubigen, das sind diejenigen, deren Herzen sich ängstigen, wenn Gottes gedacht wird, und die, wenn ihnen seine Zeichen verlesen werden, dadurch in ihrem Glauben bestärkt werden und die auf ihren Herrn vertrauen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mclxiii{Die das Gebet verrichten und von dem spenden, was Wir ihnen beschert haben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mclxiv{Das sind die wahren Gläubigen. Bestimmt sind für sie bei ihrem Herrn Rangstufen und Vergebung und trefflicher Unterhalt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mclxv{So wie dein Herr dich aus deinem Haus ausziehen ließ der richtigen Entscheidung gemäß, während es einem Teil der Gläubigen zuwider war.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mclxvi{Sie stritten ja mit dir über das Richtige, nachdem es deutlich geworden war, als würden sie in den Tod getrieben, während sie zuschauen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mclxvii{Und als Gott euch versprach, daß die eine der beiden Gruppen für euch bestimmt sei, ihr aber wünschtet, daß diejenige ohne Wehrkraft für euch bestimmt sei. Jedoch wollte Gott mit seinen Worten die Wahrheit bestätigen und den letzten Rest der Ungläubigen ausmerzen,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mclxviii{Um die Wahrheit zu bestätigen und das Falsche nichtig zu machen, auch wenn es den Übeltätern zuwider war.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mclxix{Als ihr euren Herrn um Rettung batet, und Er euch erhörte: «Ich werde euch mit tausend hintereinander reitenden Engeln beistehen.»\qt@no{9}}%
\qt@newcmd\qurantrans@de@mclxx{Und Gott machte es nur deswegen, damit es eine Frohbotschaft sei und damit eure Herzen dadurch Ruhe finden. Der Sieg kommt ja nur von Gott. Gott ist mächtig und weise.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mclxxi{Als Er Schläfrigkeit euch überkommen ließ als Sicherheit von Ihm her und Wasser vom Himmel auf euch herabsandte, um euch damit rein zu machen und die Unreinheit des Satans von euch zu entfernen, und um eure Herzen zu stärken und eure Schritte damit zu festigen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mclxxii{Als dein Herr den Engeln eingab: «Ich bin mit euch. Festigt diejenigen, die glauben. Ich werde den Herzen derer, die ungläubig sind, Schrecken einjagen. So schlagt auf die Nacken und schlagt auf jeden Finger von ihnen.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mclxxiii{Dies dafür, daß sie sich Gott und seinem Gesandten widersetzten. Und wenn jemand sich Gott und seinem Gesandten widersetzt, so verhängt Gott eine harte Strafe.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mclxxiv{Das ist sie, so kostet sie. Und wißt: Bestimmt ist für die Ungläubigen die Pein des Feuers.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mclxxv{O ihr, die ihr glaubt, wenn ihr auf die, die ungläubig sind, trefft, während sie zur Schlacht anrücken, dann kehret ihnen nicht den Rücken.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mclxxvi{Wer ihnen an jenem Tag den Rücken kehrt - es sei denn, er setzt sich ab zum Kampf, oder er stößt zu einer anderen Schar -, zieht sich den Zorn Gottes zu. Seine Heimstätte ist die Hölle - welch schlimmes Ende!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mclxxvii{Nicht ihr habt sie getötet, sondern Gott hat sie getötet. Und nicht du hast geworfen, als du geworfen hast, sondern Gott hat geworfen. Und Er wollte die Gläubigen einer schönen Prüfung unterziehen. Gott hört und weiß alles.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mclxxviii{Das ist es. Und so wollte Gott die List der Ungläubigen wirkungslos machen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mclxxix{Wenn ihr einen Richterspruch verlangt, so ist der Richterspruch zu euch gekommen. Und wenn ihr aufhört, so ist es besser für euch. Und wenn ihr (dazu) zurückkehrt, kehren Wir (auch) zurück. Und eure Schar wird nichts von euch abwenden können, auch wenn sie zahlreich sein sollte. Und siehe, Gott ist mit den Gläubigen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mclxxx{O ihr, die ihr glaubt, gehorchet Gott und seinem Gesandten, und kehrt euch nicht von ihm ab, wo ihr doch hört.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mclxxxi{Und seid nicht wie diejenigen, die sagen: «Wir hören», wo sie doch nicht hören.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mclxxxii{Die schlimmsten Tiere bei Gott sind die tauben und stummen, die keinen Verstand haben.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mclxxxiii{Hätte Gott bei ihnen etwas Gutes festgestellt, hätte Er sie hören lassen. Und wenn Er sie hätte hören lassen, so hätten sie sich (dennoch) abgekehrt und abgewandt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mclxxxiv{O ihr, die ihr glaubt, erhört Gott und den Gesandten, wenn er euch zu dem aufruft, was euch Leben gibt. Und wißt, daß Gott zwischen dem Menschen und seinem Herzen trennt, und daß ihr zu Ihm versammelt werdet.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mclxxxv{Und hütet euch vor einer Versuchung, die gewiß nicht ausschließlich diejenigen von euch treffen wird, die Unrecht tun. Und wißt, Gott verhängt eine harte Strafe.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mclxxxvi{Und gedenket, als ihr auf der Erde wäret und als Schwache behandelt wurdet und fürchtetet, daß euch die Menschen hinwegraffen würden. Da hat Er euch untergebracht und euch mit seiner Unterstützung gestärkt und euch (einiges) von den köstlichen Dingen beschert, auf daß ihr dankbar seid.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mclxxxvii{O ihr, die ihr glaubt, seid nicht treulos gegenüber Gott und dem Gesandten, und seid nicht treulos in bezug auf die euch anvertrauten Dinge, wo ihr es wißt.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mclxxxviii{Und wißt, daß euer Vermögen und eure Kinder eine Versuchung sind und daß Gott einen großartigen Lohn bereithält.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mclxxxix{O ihr, die ihr glaubt, wenn ihr Gott fürchtet, bestellt Er euch eine Unterscheidungsnorm, sühnt euch eure Missetaten und vergibt euch. Und Gott besitzt große Huld.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcxc{Und als diejenigen, die ungläubig sind, gegen dich Ränke schmiedeten, um dich festzunehmen oder zu töten oder zu vertreiben. Sie schmiedeten Ränke, und Gott schmiedete Ränke. Gott ist der Beste derer, die Ränke schmieden.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcxci{Und wenn ihnen unsere Zeichen verlesen werden, sagen sie: «Wir haben es gehört. Wenn wir wollten, könnten auch wir etwas Derartiges sagen. Das sind nichts als Fabeln der Früheren.»\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcxcii{Und als sie sagten: «O unser Gott, wenn dies die Wahrheit von Dir ist, so laß auf uns Steine vom Himmel regnen, oder bring über uns eine schmerzhafte Pein.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcxciii{Aber Gott konnte sie wohl unmöglich peinigen, während du noch in ihrer Mitte warst; und Er konnte sie wohl unmöglich peinigen, während sie um Vergebung baten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcxciv{Und warum sollte Gott sie nicht peinigen, wo sie (euch) von der heiligen Moschee abweisen und sie nicht seine Freunde sind? Seine Freunde sind ja nur die Gottesfürchtigen. Aber die meisten von ihnen wissen nicht Bescheid.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcxcv{Und ihr Gebet beim Haus ist nur Pfeifen und Klatschen. So kostet die Pein dafür, daß ihr ungläubig waret.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcxcvi{Diejenigen, die ungläubig sind, spenden ihr Vermögen, um vom Weg Gottes abzuweisen. Sie werden es spenden, und dann wird es ein Bedauern für sie sein. Dann werden sie besiegt werden. Und diejenigen, die ungläubig sind, werden zur Hölle versammelt werden,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcxcvii{Damit Gott das Schlechte vom Guten unterscheide und das Schlechte übereinander tue, es allesamt zusammenhäufe und in die Hölle stelle. Das sind die Verlierer.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcxcviii{Sprich zu denen, die ungläubig sind: Wenn sie aufhören, wird ihnen vergeben, was vorher geschah. Wenn sie (dazu) zurückkehren, so steht fest, wie an den Früheren gehandelt wurde.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcxcix{Und kämpft gegen sie, bis es keine Verführung mehr gibt und bis die Religion gänzlich nur noch Gott gehört. Wenn sie aufhören, so sieht Gott wohl, was sie tun.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcc{Und wenn sie sich abkehren, so wißt, daß Gott euer Schutzherr ist: Welch vorzüglicher Schutzherr und welch vorzüglicher Helfer!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcci{Und wißt: Wenn ihr etwas erbeutet, so gehört ein Fünftel davon Gott und dem Gesandten, und den Verwandten, den Waisen, den Bedürftigen, dem Reisenden, so ihr an Gott glaubt und an das, was Wir auf unseren Diener am Tag der Unterscheidung hinabgesandt haben, am Tag, da die beiden Scharen aufeinandertrafen. Und Gott hat Macht zu allen Dingen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mccii{Als ihr auf der näheren Talseite wäret, sie auf der ferneren Talseite und die Reittiere unterhalb von euch. Und wenn ihr euch verabredet hättet, wäret ihr über die Verabredung nicht einig geworden, aber Gott wollte eine Angelegenheit entscheiden, die zur Ausführung kommen sollte, damit diejenigen, die umkamen, aufgrund eines deutlichen Zeichens umkämen, und diejenigen, die am Leben blieben, aufgrund eines deutlichen Zeichens am Leben blieben. Und Gott hört und weiß alles.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcciii{Als Gott dich in deinem Traum sie als wenige sehen ließ. Hätte Er dich sie als viele sehen lassen, ihr wäret verzagt und hättet miteinander über die Angelegenheit gestritten. Aber Gott hat euch (davor) bewahrt. Er weiß über das innere Geheimnis Bescheid.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcciv{Und als Er sie, als ihr aufeinandertraft, in euren Augen als wenige erscheinen ließ, und (auch) euch in ihren Augen weniger machte, damit Gott eine Angelegenheit entscheide, die zur Ausführung kommen sollte. Und zu Gott werden alle Angelegenheiten zurückgebracht.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mccv{O ihr, die ihr glaubt, wenn ihr auf eine Schar trefft, so steht fest und gedenket Gottes viel, auf daß es euch wohl ergehe.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mccvi{Und gehorchet Gott und seinem Gesandten, und streitet nicht miteinander, sonst würdet ihr verzagen, und eure Durchsetzungskraft würde auch schwinden. Und seid standhaft. Gott ist mit den Standhaften.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mccvii{Und seid nicht wie diejenigen, die aus ihren Wohnstätten auszogen, aus Übermut und um von den Menschen gesehen zu werden, und die vom Weg Gottes abweisen. Gott umfaßt, was sie tun.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mccviii{Und als der Satan ihnen ihre Taten verlockend machte und sagte: «Es gibt niemanden unter den Menschen, der euch heute besiegen könnte. Und ich bin euch ein (hilfsbereiter) Nachbar.» Als aber die beiden Scharen einander sahen, machte er auf seinen Fersen kehrt und sagte: «Ich bin euer ledig. Ich sehe, was ihr nicht seht. Ich fürchte Gott. Gott verhängt eine harte Strafe.»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mccix{Als die Heuchler und die, in deren Herzen Krankheit ist, sagten: «Betört hat diese da ihre Religion.» Wenn einer aber auf Gott vertraut, so ist Gott mächtig und weise.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mccx{Und wenn du nur zuschauen könntest, wenn die Engel diejenigen abberufen, die ungläubig sind: Sie schlagen sie ins Gesicht und auf das Hinterteil: «Kostet die Pein des Feuerbrandes.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mccxi{Dies für das, was eure Hände vorausgeschickt haben, und weil Gott den Dienern kein Unrecht tut.»\qt@no{51}}%
\qt@newcmd\qurantrans@de@mccxii{Es ist wie mit den Leuten des Pharao und denen, die vor ihnen lebten. Sie verleugneten die Zeichen Gottes, so suchte sie Gott wegen ihrer Sünden heim. Gott ist stark und verhängt eine harte Strafe.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mccxiii{Dies, weil Gott niemals seine Gnade, mit der Er ein Volk begnadet hat, ändert, bis sie selbst das ändern, was in ihrem Innern ist, und weil Gott alles hört und weiß.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mccxiv{Es ist wie mit den Leuten des Pharao und denen, die vor ihnen lebten: Sie erklärten die Zeichen ihres Herrn für Lüge, so ließen Wir sie für ihre Sünden verderben, und Wir ließen die Leute des Pharao ertrinken. Sie waren ja alle Leute, die Unrecht taten.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mccxv{Die schlimmsten Tiere bei Gott sind die, die ungläubig sind und weiterhin nicht glauben,\qt@no{55}}%
\qt@newcmd\qurantrans@de@mccxvi{Mit denen du einen Vertrag geschlossen hast, die aber dann ihren Vertrag jedesmal brechen und nicht gottesfürchtig sind.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mccxvii{Und wenn du sie im Krieg triffst, dann verscheuche mit ihnen diejenigen, die hinter ihnen stehen, auf daß sie es bedenken.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mccxviii{Und wenn du von bestimmten Leuten Verrat fürchtest, so kündige ihnen (den Vertrag) so eindeutig auf, daß Gleichheit zwischen euch besteht. Gott liebt ja die Verräter nicht.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mccxix{Und diejenigen, die ungläubig sind, sollen nicht meinen, sie seien (euch) voraus. Sie werden nichts vereiteln können.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mccxx{Und rüstet gegen sie, was ihr an Kraft und an einsatzbereiten Pferden haben könnt, um damit den Feinden Gottes und euren Feinden Angst zu machen, sowie anderen außer ihnen, die ihr nicht kennt; Gott aber kennt sie. Und was ihr auch auf dem Weg Gottes spendet, wird euch voll zurückerstattet, und euch wird nicht Unrecht getan.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mccxxi{Und wenn sie sich dem Frieden zuneigen, dann neige auch du dich ihm zu und vertrau auf Gott. Er ist der, der alles hört und weiß.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mccxxii{Und wenn sie dich betrügen wollen, so genügt dir Gott. Er ist es, der dich mit seiner Unterstützung und mit den Gläubigen gestärkt\qt@no{62}}%
\qt@newcmd\qurantrans@de@mccxxiii{Und zwischen ihren Herzen Vertrautheit gestiftet hat. Wenn du alles, was auf Erden ist, (dafür) ausgegeben hättest, hättest du nicht zwischen ihren Herzen Vertrautheit stiften können. Aber Gott hat zwischen ihnen Vertrautheit gestiftet. Er ist mächtig und weise.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mccxxiv{O Prophet, Gott genügt dir und auch denen von den Gläubigen, die dir folgen.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mccxxv{O Prophet, sporne die Gläubigen zum Kampf an. Wenn es unter euch auch nur zwanzig gibt, die standhaft sind, werden sie zweihundert besiegen. Und wenn es unter euch hundert gibt, werden sie tausend von denen, die ungläubig sind, besiegen. Dies dafür, daß sie Leute sind, die nicht begreifen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mccxxvi{Jetzt hat Gott euch Erleichterung gewährt. Er weiß, daß in euren Reihen Schwachheit vorkommt. Wenn es unter euch hundert Standhafte gibt, werden sie zweihundert besiegen, und wenn es unter euch tausend gibt, werden sie zweitausend besiegen, mit Gottes Erlaubnis. Und Gott ist mit den Standhaften.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mccxxvii{Es steht einem Propheten nicht zu, Gefangene zu haben, bis er auf der Erde stark gewütet hat. Ihr wollt die Güter des Diesseits, und Gott will das Jenseits. Gott ist mächtig und weise.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mccxxviii{Gäbe es nicht eine früher ergangene Bestimmung von Gott, so hätte euch für das, was ihr genommen habt, eine gewaltige Pein ergriffen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mccxxix{Eßt nun von dem, was ihr erbeutet habt, so es erlaubt und köstlich ist, und fürchtet Gott. Gott ist voller Vergebung und barmherzig.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mccxxx{O Prophet, sprich zu denen von den Gefangenen, die sich in eurer Gewalt befinden: «Wenn Gott in euren Herzen etwas Gutes feststellt, läßt Er euch etwas Besseres zukommen als das, was euch genommen worden ist, und vergibt euch. Und Gott ist voller Vergebung und barmherzig.»\qt@no{70}}%
\qt@newcmd\qurantrans@de@mccxxxi{Und wenn sie dich verraten wollen, so haben sie zuvor Gott verraten, und Er hat sie dann (eurer) Gewalt preisgegeben. Und Gott weiß Bescheid und ist weise.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mccxxxii{Diejenigen, die glaubten und ausgewandert sind und sich mit ihrem Vermögen und mit ihrer eigenen Person auf dem Weg Gottes eingesetzt haben, und diejenigen, die (jene) untergebracht und unterstützt haben, sind untereinander Freunde. Mit denen aber, die glauben und nicht ausgewandert sind, habt ihr keine Freundschaft zu pflegen, bis sie auswandern. Wenn sie euch jedoch um Unterstützung wegen der (euch gemeinsamen) Religion bitten, so habt ihr die Pflicht zur Unterstützung, außer gegen Leute, zwischen denen und euch eine Vertragspflicht besteht. Und Gott sieht wohl, was ihr tut.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mccxxxiii{Und diejenigen, die ungläubig sind, sind untereinander Freunde. Wenn ihr es nicht tut, wird es im Land Verführung und großes Unheil geben.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mccxxxiv{Und diejenigen, die glauben und ausgewandert sind und sich auf dem Weg Gottes eingesetzt haben, und diejenigen, die (jene) untergebracht und unterstützt haben, das sind die wahren Gläubigen. Bestimmt ist für sie Vergebung und ein trefflicher Unterhalt.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mccxxxv{Und diejenigen, die danach geglaubt haben und ausgewandert sind und sich mit euch eingesetzt haben, sie gehören zu euch. Aber die Blutsverwandten haben eher Anspruch aufeinander; dies steht im Buch Gottes. Gott weiß über alle Dinge Bescheid.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mccxxxvi{\basmalahde Aufkündigung von Seiten Gottes und seines Gesandten an diejenigen unter den Polytheisten, mit denen ihr einen Vertrag abgeschlossen habt.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mccxxxvii{Nun zieht im Land vier Monate umher und wißt, daß ihr Gottes Willen nicht vereiteln könnt, und daß Gott die Ungläubigen zuschanden macht.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mccxxxviii{Und Bekanntmachung von Seiten Gottes und seines Gesandten an die Menschen am Tag der großen Wallfahrt: Gott ist der Polytheisten ledig, und auch sein Gesandter. Wenn ihr umkehrt, ist es besser für euch. Wenn ihr abkehrt, so wißt, daß ihr Gottes Willen nicht vereiteln könnt. Und verkünde denen, die ungläubig sind, eine schmerzhafte Pein.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mccxxxix{Mit Ausnahme derer von den Polytheisten, mit denen ihr einen Vertrag geschlossen habt und die euch in nichts fehlen lassen und niemandem gegen euch beigestanden haben. So erfüllt ihnen gegenüber ihren Vertrag bis zu der ihnen eingeräumten Frist. Gott liebt die Gottesfürchtigen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mccxl{Wenn die heiligen Monate abgelaufen sind, dann tötet die Polytheisten, wo immer ihr sie findet, greift sie, belagert sie und lauert ihnen auf jedem Weg auf. Wenn sie umkehren, das Gebet verrichten und die Abgabe entrichten, dann laßt sie ihres Weges ziehen: Gott ist voller Vergebung und barmherzig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mccxli{Und wenn einer von den Polytheisten dich um Schutz bittet, so gewähre ihm Schutz, bis er das Wort Gottes hört. Danach laß ihn den Ort erreichen, in dem er in Sicherheit ist. Dies, weil sie Leute sind, die nicht Bescheid wissen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mccxlii{Wie sollten die Polytheisten Gott und seinem Gesandten gegenüber einen Vertrag geltend machen, ausgenommen die, mit denen ihr bei der heiligen Moschee einen Vertrag abgeschlossen habt? Solange sie sich euch gegenüber recht verhalten, verhaltet auch ihr euch ihnen gegenüber recht. Gott liebt die Gottesfürchtigen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mccxliii{Wie sollten sie dies tun, wo sie doch, wenn sie die Oberhand über euch bekommen, euch gegenüber weder Verwandtschaft noch Schutzbund beachten? Sie stellen euch zufrieden mit ihrem Munde, aber ihre Herzen sind voller Ablehnung. Und die meisten von ihnen sind Frevler.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mccxliv{Sie haben die Zeichen Gottes für einen geringen Preis verkauft und so die Menschen von seinem Weg abgewiesen. Schlimm ist, was sie immer wieder getan haben.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mccxlv{Sie beachten einem Gläubigen gegenüber weder Verwandtschaft noch Schutzbund. Das sind die, die Übertretungen begehen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mccxlvi{Wenn sie umkehren, das Gebet verrichten und die Abgabe entrichten, dann sind sie eure Brüder in der Religion. Wir legen die Zeichen im einzelnen dar für Leute, die Bescheid wissen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mccxlvii{Wenn sie aber nach Vertragsabschluß ihre Eide brechen und eure Religion angreifen, dann kämpft gegen die Anführer des Unglaubens. Für sie gibt es ja keine Eide. Vielleicht werden sie aufhören.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mccxlviii{Wollt ihr nicht gegen Leute kämpfen, die ihre Eide gebrochen haben und im Begriff gewesen sind, den Gesandten zu vertreiben, wobei sie zuerst gegen euch vorgegangen sind? Fürchtet ihr sie? Gott hat eher darauf Anspruch, daß ihr Ihn fürchtet, so ihr gläubig seid.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mccxlix{Kämpft gegen sie, so wird Gott sie durch eure Hände peinigen, sie zuschanden machen und euch gegen sie unterstützen, die Brust gläubiger Leute wieder heil machen\qt@no{14}}%
\qt@newcmd\qurantrans@de@mccl{Und den Groll ihrer Herzen entfernen. Und Gott wendet sich, wem Er will, wieder zu. Und Gott weiß Bescheid und ist weise.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mccli{Oder meint ihr, daß ihr in Ruhe gelassen werdet, bevor Gott in Erfahrung gebracht hat, wer von euch sich eingesetzt und sich außer Gott, seinem Gesandten und den Gläubigen keinen Freund genommen hat? Und Gott hat Kenntnis von dem, was ihr tut.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mcclii{Es steht den Polytheisten nicht zu, in den Moscheen Gottes zu verweilen und Dienst zu tun, wo sie gegen sich selbst bezeugen, ungläubig zu sein. Deren Werke sind wertlos, und sie werden im Feuer ewig weilen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mccliii{In den Moscheen Gottes verweilen und Dienst tun dürfen nur die, die an Gott und den Jüngsten Tag glauben, das Gebet verrichten, die Abgabe entrichten und niemanden fürchten außer Gott. Mögen diese zu denen gehören, die der Rechtleitung folgen!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mccliv{Wollt ihr etwa die Tränkung der Pilger und den Dienst in der heiligen Moschee so bewerten, wie (die Werke dessen), der an Gott und den Jüngsten Tag glaubt und sich auf dem Weg Gottes einsetzt? Sie sind bei Gott nicht gleich. Und Gott leitet die ungerechten Leute nicht recht.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mcclv{Diejenigen, die glauben und ausgewandert sind und sich auf dem Weg Gottes mit ihrem Vermögen und mit ihrer eigenen Person eingesetzt haben, haben eine höhere Rangstufe bei Gott. Das sind die Erfolgreichen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mcclvi{Ihr Herr verkündet ihnen eine Barmherzigkeit von sich und Wohlgefallen und Gärten, in denen sie beständige Wonne haben;\qt@no{21}}%
\qt@newcmd\qurantrans@de@mcclvii{Darin werden sie auf immer ewig weilen. Bei Gott steht ein großartiger Lohn bereit.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mcclviii{O ihr, die ihr glaubt, nehmt euch nicht eure Väter und eure Brüder zu Freunden, wenn sie den Unglauben dem Glauben vorziehen. Diejenigen von euch, die sie zu Freunden nehmen, das sind die, die Unrecht tun.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mcclix{Sprich: Wenn eure Väter, eure Söhne, eure Brüder, eure Gattinnen und eure Verwandten, ein Vermögen, das ihr erworben habt, eine Handelsware, die ihr fürchtet nicht loszuwerden, und Wohnungen, die euch gefallen, euch lieber sind als Gott und sein Gesandter und der Einsatz auf seinem Weg, dann wartet ab, bis Gott mit seinem Befehl kommt. Gott leitet die frevlerischen Leute nicht recht.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mcclx{Gott hat euch an vielen Orten unterstützt, und auch am Tag von Hunayn, als eure große Zahl euch gefiel, von euch aber nichts abwenden konnte. Die Erde wurde euch eng trotz ihrer Weite. Daraufhin kehrtet ihr den Rücken.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcclxi{Dann sandte Gott seine ruhespendende Gegenwart auf seinen Gesandten und auf die Gläubigen herab. Und Er sandte Truppen, die ihr nicht sehen konntet, herab und peinigte diejenigen, die ungläubig waren. Das ist die Vergeltung für die Ungläubigen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mcclxii{Und Gott wendet sich danach gnädig zu, wem Er will. Und Gott ist voller Vergebung und barmherzig.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mcclxiii{O ihr, die ihr glaubt, die Polytheisten sind unrein, so sollen sie nach diesem ihrem Jahr sich der heiligen Moschee nicht mehr nähern. Und wenn ihr fürchtet, (deswegen) arm zu werden, so wird Gott euch von seiner Huld reich machen, wenn Er will. Gott weiß Bescheid und ist weise.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mcclxiv{Kämpft gegen diejenigen, die nicht an Gott und nicht an den Jüngsten Tag glauben und nicht verbieten, was Gott und sein Gesandter verboten haben, und nicht der Religion der Wahrheit angehören - von denen, denen das Buch zugekommen ist, bis sie von dem, was ihre Hand besitzt, Tribut entrichten als Erniedrigte.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcclxv{Die Juden sagen: «Uzayr ist Gottes Sohn.» Und die Christen sagen: «Christus ist Gottes Sohn.» Das ist ihre Rede aus ihrem eigenen Munde. Damit reden sie wie die, die vorher ungläubig waren. Gott bekämpfe sie! Wie leicht lassen sie sich doch abwenden!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcclxvi{Sie nahmen sich ihre Gelehrten und ihre Mönche zu Herren neben Gott, sowie auch Christus, den Sohn Marias. Dabei wurde ihnen doch nur befohlen, einem einzigen Gott zu dienen. Es gibt keinen Gott außer Ihm. Preis sei Ihm! Erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcclxvii{Sie wollen das Licht Gottes mit ihrem Mund auslöschen. Aber Gott besteht darauf, sein Licht zu vollenden, auch wenn es den Ungläubigen zuwider ist.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcclxviii{Er ist es, der seinen Gesandten mit der Rechtleitung und der Religion der Wahrheit gesandt hat, um ihr die Oberhand zu verleihen über alle Religion, auch wenn es den Polytheisten zuwider ist.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcclxix{O ihr, die ihr glaubt, viele von den Gelehrten und den Mönchen verzehren das Vermögen der Menschen durch Betrug und weisen (sie) vom Weg Gottes ab. Denjenigen, die Gold und Silber horten und es nicht auf dem Weg Gottes spenden, verkünde eine schmerzhafte Pein,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcclxx{Am Tag, da im Feuer der Hölle darüber angeheizt wird und damit ihre Stirn, ihre Seiten und ihr Rücken gebrandmarkt werden. Dies ist, was ihr für euch selbst gehortet habt. So kostet, was ihr immer wieder gehortet habt.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcclxxi{Die Zahl der Monate bei Gott ist zwölf Monate, im Buch Gottes festgestellt am Tag, da Er die Himmel und die Erde erschaffen hat. Vier davon sind heilig. Das ist die richtige Religion. So tut euch selbst in ihnen kein Unrecht. Und kämpft gegen die Polytheisten allesamt, wie sie gegen euch allesamt kämpfen. Und wißt, Gott ist mit den Gottesfürchtigen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcclxxii{Der Brauch des verschobenen Monats zeugt von mehr Unglauben. Damit werden diejenigen, die ungläubig sind, irregeführt, so daß sie ihn in einem Jahr für profan und in einem (anderen) Jahr für heilig erklären, um die Zahl dessen, was Gott für heilig erklärt hat, auszugleichen; so erklären sie für profan, was Gott für heilig erklärt hat. Ihr schlechtes Tun ist ihnen verlockend gemacht worden. Gott leitet die ungläubigen Leute nicht recht.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcclxxiii{O ihr, die ihr glaubt, was ist mit euch, daß ihr, wenn zu euch gesagt wird: «Rückt aus auf dem Weg Gottes», euch schwer bis zur Erde neigt? Gefällt euch das diesseitige Leben mehr als das jenseitige? Aber die Nutznießung des diesseitigen Lebens ist im (Vergleich mit dem) Jenseits nur gering (zuschätzen).\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcclxxiv{Wenn ihr nicht ausrückt, peinigt Er euch mit einer schmerzhaften Pein und nimmt an eurer Stelle ein anderes Volk, und ihr könnt Ihm keinen Schaden zufügen. Gott hat Macht zu allen Dingen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcclxxv{Wenn ihr ihn nicht unterstützt, so hat Gott ihn (schon damals) unterstützt, als diejenigen, die ungläubig sind, ihn zusammen mit einem zweiten Mann vertrieben haben. Sie waren beide in der Höhle, und er sagte zu seinem Gefährten: «Sei nicht traurig. Gott ist mit uns.» Da sandte Gott seine Ruhe spendende Gegenwart auf ihn herab und stärkte ihn mit Truppen, die ihr nicht sehen konntet. Und Er machte das Wort derer, die ungläubig sind, unterlegen. Siehe, Gottes Wort ist überlegen. Und Gott ist mächtig und weise.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcclxxvi{Rückt aus, ob leicht oder schwer, und setzt euch mit eurem Vermögen und mit eurer eigenen Person auf dem Weg Gottes ein. Das ist besser für euch, so ihr Bescheid wißt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mcclxxvii{Ginge es um nahe Güter oder eine mäßige Reise, würden sie dir folgen. Aber die Entfernung ist ihnen zu weit. Und sie werden bei Gott schwören: «Wenn wir es könnten, würden wir mit euch hinausziehen.» Sie stürzen sich dabei selbst ins Verderben. Und Gott weiß, daß sie ja lügen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcclxxviii{Gott verzeihe dir! Warum hast du sie befreit? (Hättest du nur gewartet), bis dir deutlich geworden war, wer die Wahrheit sagt, und du die Lügner in Erfahrung gebracht hast!\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcclxxix{Diejenigen, die an Gott und den Jüngsten Tag glauben, bitten dich nicht um Befreiung davon, sich mit ihrem Vermögen und mit ihrer eigenen Person einzusetzen. Und Gott weiß über die Gottesfürchtigen Bescheid.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mcclxxx{Um Befreiung bitten dich nur diejenigen, die an Gott und den Jüngsten Tag nicht glauben und deren Herzen zweifeln; und in ihrem Zweifel zögern sie.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mcclxxxi{Hätten sie wirklich ausziehen wollen, hätten sie sich dafür gerüstet. Aber ihr Ausmarsch war Gott zuwider, so hielt Er sie zurück. Und es wurde gesagt: «Sitzt daheim mit denen, die daheim sitzen.»\qt@no{46}}%
\qt@newcmd\qurantrans@de@mcclxxxii{Würden sie mit euch ausziehen, würden sie euch nur noch Verschlechterung bringen und unter euch schnell umherlaufen im Trachten danach, (euch) der Versuchung auszusetzen. Dennoch gibt es unter euch welche, die nur auf sie hören. Und Gott weiß über die Bescheid, die Unrecht tun.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mcclxxxiii{Sie haben schon früher danach getrachtet, euch der Versuchung auszusetzen, und gegen dich Intrigen getrieben, bis die Wahrheit kam und der Befehl Gottes erschien, obwohl es ihnen zuwider war.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mcclxxxiv{Unter ihnen gibt es welche, die sagen: «Befreie mich und führe mich nicht in Versuchung.» In Versuchung sind sie doch gefallen. Und die Hölle umfaßt die Ungläubigen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mcclxxxv{Wenn dich Gutes trifft, tut es ihnen leid; und wenn dich ein Unglück trifft, sagen sie: «Wir haben unsere Angelegenheit schon vorher selbst übernommen.» Und sie kehren sich erfreut ab.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mcclxxxvi{Sprich: Uns wird nur das treffen, was Gott uns bestimmt hat. Er ist unser Schutzherr. Auf Gott sollen die Gläubigen vertrauen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mcclxxxvii{Sprich: Erwartet ihr für uns etwas anderes als eine der beiden schönsten Sachen? Wir erwarten für euch, daß Gott euch trifft mit einer Pein von Ihm oder durch unsere Hände. So wartet nur ab, wir warten mit euch ab.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mcclxxxviii{Sprich: Ihr mögt freiwillig oder widerwillig spenden, es wird von euch doch nicht angenommen werden. Ihr seid ja frevlerische Leute.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mcclxxxix{Und nichts anderes verhindert, daß ihre Spenden von ihnen angenommen werden, als daß sie Gott und seinen Gesandten verleugnen, nur nachlässig zum Gebet hingehen und nur widerwillig spenden.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mccxc{Nicht sollen dir ihr Vermögen und ihre Kinder gefallen. Gott will sie ja im diesseitigen Leben damit peinigen, und daß ihre Seele (im Tod) dahinschwindet, während sie ungläubig sind.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mccxci{Und sie schwören bei Gott, daß sie zu euch gehören. Sie gehören aber nicht zu euch, sondern sie sind Leute, die ängstlich sind.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mccxcii{Würden sie einen Zufluchtsort oder Höhlen oder einen Schlupfgang finden, sie würden schnellstens dorthin flüchten.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mccxciii{Und unter ihnen gibt es welche, die gegen dich wegen der Almosen nörgeln. Wenn ihnen etwas davon gegeben wird, sind sie zufrieden. Wenn ihnen nichts davon gegeben wird, geraten sie gleich in Groll.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mccxciv{Wären sie doch mit dem zufrieden, was Gott und sein Gesandter ihnen zukommen ließen, und würden sie doch sagen: «Gott genügt uns. Gott wird uns etwas von seiner Huld zukommen lassen, und auch sein Gesandter. Auf Gott richten wir unsere Wünsche»!\qt@no{59}}%
\qt@newcmd\qurantrans@de@mccxcv{Die Almosen sind bestimmt für die Armen, die Bedürftigen, die, die damit befaßt sind, die, deren Herzen vertraut gemacht werden sollen, die Gefangenen, die Verschuldeten, für den Einsatz auf dem Weg Gottes und für den Reisenden. Es ist eine Rechtspflicht von seiten Gottes. Und Gott weiß Bescheid und ist weise.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mccxcvi{Und unter ihnen gibt es welche, die dem Propheten Leid zufügen und sagen: «Er ist (nur) Ohr.» Sprich: Ein Ohr zum Guten für euch. Er glaubt an Gott und glaubt den Gläubigen, und (er ist) eine Barmherzigkeit für die von euch, die gläubig sind. Für diejenigen aber, die dem Gesandten Gottes Leid zufügen, ist eine schmerzhafte Pein bestimmt.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mccxcvii{Sie schwören euch bei Gott, um euch zufriedenzustellen. Aber Gott - und (auch) sein Gesandter - hat eher darauf Anspruch, daß sie Ihn zufriedenstellen, so sie gläubig sind.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mccxcviii{Wissen sie denn nicht, daß für den, der sich Gott und seinem Gesandten widersetzt, das Feuer der Hölle bestimmt ist, in dem er ewig weilen wird? Das ist die gewaltige Schande.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mccxcix{Die Heuchler befürchten, daß eine Sure auf sie herabgesandt wird, die ihnen das kundtut, was in ihren Herzen ist. Sprich: Spottet nur! Gott wird ans Licht bringen, was ihr befürchtet.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mccc{Und wenn du sie fragst, sagen sie sicherlich: «Wir hielten ausschweifende Reden und trieben nur unser Spiel.» Sprich: Wolltet ihr denn über Gott und seine Zeichen und seinen Gesandten spotten?\qt@no{65}}%
\qt@newcmd\qurantrans@de@mccci{Entschuldigt euch nicht! Ihr seid ungläubig geworden, nachdem ihr geglaubt hattet. Wenn Wir auch einer Gruppe von euch verzeihen, so peinigen Wir eine andere Gruppe dafür, daß sie Übeltäter waren.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mcccii{Die Heuchler und die Heuchlerinnen stammen voneinander. Sie gebieten das Verwerfliche und verbieten das Rechte und halten ihre Hände geschlossen. Vergessen haben sie Gott, und so hat Er sie vergessen. Die Heuchler sind die wahren Frevler.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mccciii{Versprochen hat Gott den Heuchlern und den Heuchlerinnen und den Ungläubigen das Feuer der Hölle, darin werden sie ewig weilen. Es ist ihr Genüge. Und Gott hat sie verflucht, und bestimmt ist für sie eine beständige Pein.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mccciv{Es ist wie mit denen, die vor euch lebten. Sie hatten eine stärkere Kraft als ihr und mehr Vermögen und Kinder. Sie nützten ihren Anteil aus, dann habt ihr euren Anteil ausgenützt, wie die, die vor euch lebten, ihren Anteil ausgenützt haben. Und ihr habt ausschweifende Reden gehalten wie die Reden, die sie gehalten haben. Deren Werke sind wertlos im Diesseits und Jenseits. Das sind die Verlierer.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mcccv{Ist denn der Bericht über die, die vor ihnen lebten, nicht zu ihnen gelangt, das Volk Noachs, die, Aad und Thamud, das Volk Abrahams und die Gefährten von Madyan und die verschwundenen Städte? Ihre Gesandten kamen zu ihnen mit den deutlichen Zeichen. Und es ist bestimmt nicht Gott, der ihnen Unrecht getan hat, sondern sie haben sich selbst Unrecht getan.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mcccvi{Die gläubigen Männer und Frauen sind untereinander Freunde. Sie gebieten das Rechte und verbieten das Verwerfliche, verrichten das Gebet und entrichten die Abgabe und gehorchen Gott und seinem Gesandten. Siehe, Gott wird sich ihrer erbarmen. Gott ist mächtig und weise.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mcccvii{Gott hat den gläubigen Männern und Frauen Gärten versprochen, unter denen Bäche fließen und in denen sie ewig weilen werden, und gute Wohnungen in den Gärten von Eden. Ein Wohlgefallen von Gott ist aber größer. Das ist der großartige Erfolg.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mcccviii{O Prophet, setz dich gegen die Ungläubigen und die Heuchler ein und fasse sie hart an. Ihre Heimstätte ist die Hölle - welch schlimmes Ende!\qt@no{73}}%
\qt@newcmd\qurantrans@de@mcccix{Sie schwören bei Gott, sie hätten es nicht gesagt. Aber sie haben wohl das Wort des Unglaubens gesagt und sind, nachdem sie den Islam angenommen hatten, ungläubig geworden. Sie waren im Begriff, das auszuführen, was sie (doch) nicht erreicht haben. Und nichts ließ sie grollen, als daß Gott es war - und (auch) sein Gesandter -, der sie von seiner Huld reich gemacht hat. Wenn sie aber umkehren, ist es besser für sie. Und wenn sie sich abkehren, wird Gott sie mit einer schmerzhaften Pein peinigen im Diesseits und Jenseits. Und sie werden auf der Erde weder Freund noch Helfer haben.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mcccx{Unter ihnen gibt es welche, die ein bindendes Versprechen mit Gott eingegangen waren: «Wenn Er uns etwas von seiner Huld zukommen läßt, dann werden wir Almosen geben und zu den Rechtschaffenen gehören.»\qt@no{75}}%
\qt@newcmd\qurantrans@de@mcccxi{Als Er ihnen nun etwas von seiner Huld hatte zukommen lassen, geizten sie damit, und sie machten kehrt und wandten sich ab.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mcccxii{Als Folge davon setzte Er in ihre Herzen Heuchelei bis zum Tag, an dem sie Ihm begegnen werden. Dies dafür, daß sie Gott gegenüber brachen, was sie Ihm versprochen hatten, und daß sie logen.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mcccxiii{Wissen sie denn nicht, daß Gott über ihre Geheimnisse und ihre vertraulichen Gespräche Bescheid weiß, und daß Gott die unsichtbaren Dinge alle weiß?\qt@no{78}}%
\qt@newcmd\qurantrans@de@mcccxiv{Diejenigen, die gegen die Freiwilligen unter den Gläubigen wegen der Almosen nörgeln und auch gegen die, die nichts als ihren Einsatz zu leisten vermögen, mäkeln und sie verhöhnen - Gott verhöhnt sie, und bestimmt ist für sie eine schmerzhafte Pein.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mcccxv{Bitte um Vergebung für sie, oder bitte nicht um Vergebung für sie. Wenn du auch siebzigmal um Vergebung für sie bittest, Gott wird ihnen niemals vergeben. Dies, weil sie Gott und seinen Gesandten verleugneten. Und Gott leitet die frevlerischen Leute nicht recht.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mcccxvi{Es freuen sich die Zurückgelassenen darüber, daß sie im Gegensatz zum Gesandten Gottes daheim geblieben sind, und es ist ihnen zuwider, sich mit ihrem Vermögen und mit ihrer eigenen Person auf dem Weg Gottes einzusetzen. Und sie sagen: «Rückt nicht in der Hitze aus.» Sprich: Das Feuer der Hölle ist noch heißer; wenn sie es doch begreifen könnten!\qt@no{81}}%
\qt@newcmd\qurantrans@de@mcccxvii{Sie sollen ein wenig lachen, und sie sollen viel weinen zur Vergeltung für das, was sie erworben haben.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mcccxviii{Wenn Gott dich zu einer Gruppe von ihnen zurückkehren läßt und sie dich um Erlaubnis bitten hinauszuziehen, dann sprich: Niemals werdet ihr mit mir ausziehen, und nie werdet ihr mit mir gegen einen Feind kämpfen. Ihr habt das erste Mal daran Gefallen gefunden, daheim zu sitzen. So sitzet daheim mit den Zurückgebliebenen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mcccxix{Und bete niemals über einen von ihnen, der gestorben ist, und stehe nicht bei seinem Grab. Sie haben Gott und seinen Gesandten verleugnet, und sie starben als Frevler.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mcccxx{Nicht sollen ihr Vermögen und ihre Kinder dir gefallen. Gott will sie ja im Diesseits damit peinigen, und auch daß ihre Seele (im Tod) dahinschwindet, während sie ungläubig sind.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mcccxxi{Und wenn eine Sure herabgesandt wird: «Glaubt an Gott und setzt euch mit seinem Gesandten ein», dann bitten dich die Wohlhabenden unter ihnen um Befreiung und sagen: «Laß uns mit denen bleiben, die daheim sitzen.»\qt@no{86}}%
\qt@newcmd\qurantrans@de@mcccxxii{Sie finden daran Gefallen, mit den Zurückgebliebenen zu sein, und versiegelt wurden ihre Herzen, so daß sie nicht begreifen.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mcccxxiii{Aber der Gesandte und diejenigen, die mit ihm glauben, setzen sich mit ihrem Vermögen und mit ihrer eigenen Person ein. Für sie sind die guten Dinge bestimmt, und das sind die, denen es wohl ergeht.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mcccxxiv{Bereitet hat Gott für sie Gärten, unter denen Bäche fließen; darin werden sie ewig weilen. Das ist der großartige Erfolg.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mcccxxv{Und diejenigen von den arabischen Beduinen, die sich entschuldigen wollen, kommen her, um Befreiung zu erbitten. Und daheim sitzen diejenigen, die Gott und seinen Gesandten belügen. Treffen wird diejenigen unter ihnen, die ungläubig sind, eine schmerzhafte Pein.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mcccxxvi{Für die Schwachen, die Kranken und für diejenigen, die nichts zum Spenden haben, ist es kein Grund zur Bedrängnis, wenn sie sich gegenüber Gott und seinem Gesandten aufrichtig verhalten. Die Rechtschaffenen können nicht belangt werden - Gott ist voller Vergebung und barmherzig -,\qt@no{91}}%
\qt@newcmd\qurantrans@de@mcccxxvii{Und auch nicht diejenigen, die, wenn sie zu dir kommen, damit du sie mitreiten läßt, und du sagst: «Ich finde keine Tiere, daß ich euch mitreiten lassen kann», sich abkehren, während ihre Augen von Tränen überfließen aus Traurigkeit darüber, daß sie nichts zum Spenden haben.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mcccxxviii{Belangt werden diejenigen, die dich um Befreiung bitten, obwohl sie reich sind. Sie finden daran Gefallen, mit den Zurückgebliebenen zu sein. Gott hat ihre Herzen versiegelt, so daß sie nicht Bescheid wissen.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mcccxxix{Sie entschuldigen sich bei euch, wenn ihr zu ihnen zurückkommt. Sprich: Entschuldigt euch nicht. Wir glauben es euch doch nicht. Gott hat uns etwas von den Berichten über euch kundgetan. Und Gott wird euer Tun sehen, und auch sein Gesandter. Dann werdet ihr zu dem, der über das Unsichtbare und das Offenbare Bescheid weiß, zurückgebracht, und Er wird euch kundtun, was ihr zu tun pflegtet.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mcccxxx{Sie werden euch bei Gott schwören, wenn ihr zu ihnen zurückkehrt, damit ihr euch von ihnen abwendet. So wendet euch von ihnen ab, sie sind ein Greuel. Ihre Heimstätte ist die Hölle zur Vergeltung für das, was sie erworben haben.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mcccxxxi{Sie schwören euch, damit ihr mit ihnen zufrieden seid. Auch wenn ihr mit ihnen zufrieden seid, so ist Gott nicht zufrieden mit den frevlerischen Leuten.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mcccxxxii{Die arabischen Beduinen sind stärker dem Unglauben und der Heuchelei verfallen, und es paßt eher zu ihnen, daß sie die Bestimmungen dessen, was Gott auf seinen Gesandten herabgesandt hat, nicht kennen. Und Gott weiß Bescheid und ist weise.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcccxxxiii{Und unter den arabischen Beduinen gibt es welche, die das, was sie spenden, als erzwungene Zahlung ansehen und für euch die Schicksalswendungen erwarten. Über sie wird die Schicksalswendung des Unheils kommen: Und Gott hört und weiß alles.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mcccxxxiv{Und unter den arabischen Beduinen gibt es welche, die an Gott und den Jüngsten Tag glauben und das, was sie spenden, als Mittel ansehen, Gott näherzukommen und die Gebete des Gesandten zu erhalten. Es ist sicher für sie ein Mittel, (Gott) näherzukommen. Gott wird sie in seine Barmherzigkeit eingehen lassen. Gott ist voller Vergebung und barmherzig.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mcccxxxv{Mit den Allerersten der Auswanderer und der Helfer und denjenigen, die ihnen in Rechtschaffenheit gefolgt sind, ist Gott zufrieden, und sie sind mit Ihm zufrieden. Und Er hat für sie Gärten bereitet, unter denen Bäche fließen; darin werden sie auf immer ewig weilen. Das ist der großartige Erfolg.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mcccxxxvi{Und unter den arabischen Beduinen in eurer Umgebung und auch unter den Bewohnern von Medina gibt es Heuchler, die in der Heuchelei geübt sind. Du kennst sie nicht, aber Wir kennen sie. Wir werden sie zweimal peinigen, dann werden sie einer gewaltigen Pein zugeführt werden.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mcccxxxvii{Andere bekennen ihre Sünden, sie vermischen eine gute mit einer anderen schlechten Tat. Möge Gott sich ihnen gnädig zuwenden! Gott ist voller Vergebung und barmherzig.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mcccxxxviii{Nimm von ihrem Vermögen ein Almosen, mit dem du sie rein machst und läuterst, und bitte um Segen für sie. Dein Gebet ist für sie eine Beruhigung. Und Gott hört und weiß alles.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mcccxxxix{Wissen sie denn nicht, daß es Gott ist, der die Umkehr von seinen Dienern annimmt und die Almosen nimmt, und daß Gott der ist, der sich gnädig zuwendet, und der Barmherzige?\qt@no{104}}%
\qt@newcmd\qurantrans@de@mcccxl{Und sprich: Handelt doch. Gott wird euer Tun sehen, und auch sein Gesandter und die Gläubigen. Und ihr werdet zu dem, der über das Unsichtbare und das Offenbare Bescheid weiß, zurückgebracht, und Er wird euch kundtun, was ihr zu tun pflegtet.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mcccxli{Andere werden zurückgestellt, bis der Befehl Gottes eintrifft. Entweder peinigt Er sie, oder Er wendet sich ihnen gnädig zu. Und Gott weiß Bescheid und ist weise.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mcccxlii{Es gibt auch) diejenigen, die sich eine (eigene) Moschee genommen haben aus Schadenslust und Unglauben, zum Anstiften von Zwietracht zwischen den Gläubigen und als Beobachtungsort für den, der zuvor gegen Gott und seinen Gesandten Krieg geführt hat. Sie werden sicher schwören: «Wir haben nur das Beste gewollt.» Aber Gott bezeugt, daß sie ja nur Lügner sind.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mcccxliii{Stell dich niemals in ihr zum Gebet hin. Eine Moschee, die vom ersten Tag an auf die Gottesfurcht gegründet worden ist, hat eher darauf Anspruch, daß du dich in ihr hinstellst. In ihr sind Männer, die es lieben, sich zu reinigen. Und Gott liebt die, die sich reinigen.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mcccxliv{Ist der, der seinen Bau auf die Furcht Gottes und sein Wohlgefallen gegründet hat, besser oder der, der seinen Bau auf den Rand eines brüchigen Hanges gegründet hat, worauf er mit ihm ins Feuer der Hölle abstürzt? Und Gott leitet die ungerechten Leute nicht recht.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mcccxlv{Ihr Bau, den sie gebaut haben, wird unablässig Zweifel in ihren Herzen hervorrufen, bis ihre Herzen zerreißen. Und Gott weiß Bescheid und ist weise.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mcccxlvi{Gott hat von den Gläubigen ihre eigene Person und ihr Vermögen dafür erkauft, daß ihnen das Paradies gehört, insofern sie auf dem Weg Gottes kämpfen und so töten oder getötet werden. Das ist ein Ihm obliegendes Versprechen in Wahrheit in der Tora und im Evangelium und im Koran. Und wer hält seine Abmachung treuer ein als Gott? So seid froh über das Kaufgeschäft, das ihr abgeschlossen habt. Und das ist der großartige Erfolg.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mcccxlvii{Diejenigen, die umkehren, (Gott) dienen, loben, umherziehen, sich verneigen, sich niederwerfen, das Rechte gebieten und das Verwerfliche verbieten, die Bestimmungen Gottes einhalten... Und verkünde den Gläubigen eine Frohbotschaft.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mcccxlviii{Der Prophet und diejenigen, die glauben, haben nicht für die Polytheisten um Vergebung zu bitten, auch wenn es Verwandte wären, nachdem es ihnen deutlich geworden ist, daß sie Gefährten der Hölle sind.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mcccxlix{Die Bitte Abrahams um Vergebung für seinen Vater erfolgte nur aufgrund eines Versprechens, das er ihm gegeben hatte. Als es ihm aber deutlich wurde, daß er ein Feind Gottes war, sagte er sich von ihm los. Abraham war voller Trauer und langmütig.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mcccl{Gott kann unmöglich Leute irreführen, nachdem Er sie recht geleitet hat, bis Er ihnen deutlich macht, wovor sie sich hüten sollen. Gott weiß über alle Dinge Bescheid.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mcccli{Gott gehört die Königsherrschaft der Himmel und der Erde. Er macht lebendig und läßt sterben. Ihr habt außer Gott weder Freund noch Helfer.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mccclii{Gott hat sich dem Propheten, den Auswanderern und den Helfern zugewandt, die ihm in der Stunde der Bedrängnis gefolgt sind, nachdem die Herzen einer Gruppe von ihnen fast abgewichen wären. Ihnen hat Er sich dann zugewandt - Er hat Mitleid mit ihnen und ist barmherzig -,\qt@no{117}}%
\qt@newcmd\qurantrans@de@mcccliii{Und auch den dreien, die zurückgelassen wurden, bis die Erde ihnen trotz ihrer Weite eng wurde und ihre Seelen sich beengt fühlten und sie begriffen, daß es vor Gott keine andere Zuflucht gibt als zu Ihm. Darauf wandte Er sich ihnen zu, damit sie umkehren. Gott ist der, der sich zuwendet, der Barmherzige.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mcccliv{O ihr, die ihr glaubt, fürchtet Gott und seid mit denen, die die Wahrheit sagen.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mccclv{Die Bewohner von Medina und die arabischen Beduinen in ihrer Umgebung dürfen nicht hinter dem Gesandten Gottes zurückbleiben und sich selbst ihm vorziehen. Dies, weil sie weder auf dem Weg Gottes Durst oder Mühsal oder Hunger erleiden noch einen Schritt unternehmen, der die Ungläubigen in Groll versetzt, noch einem Feind etwas zufügen, ohne daß dadurch ein gutes Werk für sie aufgezeichnet würde. Gott läßt den Lohn der Rechtschaffenen nicht verlorengehen.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mccclvi{Und sie geben keine Spende, ob klein oder groß, und sie überqueren kein Tal, ohne daß es für sie aufgezeichnet würde, damit Gott ihnen das Beste vergelte von dem, was sie getan haben.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mccclvii{Die Gläubigen dürfen nicht allesamt ausrücken. Möge doch von jeder Abteilung von ihnen eine Gruppe ausrücken, so daß sie selbst sich in der Religion belehren lassen und ihre Leute, wenn sie zu ihnen zurückkehren, warnen, auf daß sie auf der Hut seien.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mccclviii{O ihr, die ihr glaubt, kämpft gegen diejenigen von den Ungläubigen, die in eurer Nähe sind. Sie sollen von eurer Seite Härte spüren. Und wißt, daß Gott mit den Gottesfürchtigen ist.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mccclix{Wenn eine Sure herabgesandt wird, dann gibt es unter ihnen welche, die sagen: «Wen von euch hat denn diese (Sure) im Glauben bestärkt?» Im Glauben bestärkt hat diese wohl diejenigen, die glauben, und sie sind froh.\qt@no{124}}%
\qt@newcmd\qurantrans@de@mccclx{Aber diejenigen, in deren Herzen Krankheit ist, macht sie zu einem noch größeren Greuel, als sie es schon sind. Und sie sterben als Ungläubige.\qt@no{125}}%
\qt@newcmd\qurantrans@de@mccclxi{Sehen sie denn nicht, daß sie jedes Jahr einmal oder zweimal der Versuchung ausgesetzt werden? Aber dann kehren sie nicht um, und sie bedenken es nicht.\qt@no{126}}%
\qt@newcmd\qurantrans@de@mccclxii{Und wenn eine Sure herabgesandt wird, schauen sie einander an: «Sieht euch jemand?», dann entfernen sie sich. Möge Gott ihre Herzen (von der Rechtleitung) fernhalten, weil sie ja Leute sind, die nicht begreifen!\qt@no{127}}%
\qt@newcmd\qurantrans@de@mccclxiii{Certainly an Apostle has come to you from among yourselves; grievous to him is your falling into distress, excessively solicitous respecting you; to the believers (he is) compassionate,\qt@no{128}}%
\qt@newcmd\qurantrans@de@mccclxiv{But if they turn back, say: Allah is sufficient for me, there is no god but He; on Him do I rely, and He is the Lord of mighty power.\qt@no{129}}%
\qt@newcmd\qurantrans@de@mccclxv{\basmalahde Alif Lam Ra. Dies sind die Zeichen des weisen Buches.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mccclxvi{Scheint es denn den Menschen verwunderlich, daß Wir einem Mann aus ihrer Mitte offenbart haben:. «Warne die Menschen und verkünde denen, die glauben, daß sie bei ihrem Herrn einen wahrhaftigen Vorrang haben»? Die Ungläubigen sagen: «Das ist nur eine offenkundige Zauberei.»\qt@no{2}}%
\qt@newcmd\qurantrans@de@mccclxvii{Euer Herr ist Gott, der die Himmel und die Erde in sechs Tagen erschuf und sich dann auf dem Thron zurechtsetzte, um die Angelegenheit zu regeln. Es gibt keinen Fürsprecher außer nach seiner Erlaubnis. So ist Gott, euer Herr. Dienet Ihm. Wollt ihr es denn nicht bedenken?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mccclxviii{Zu Ihm kommt ihr alle zurück. Das ist das Versprechen Gottes in Wahrheit. Er macht die Schöpfung am Anfang, und Er wiederholt sie, um diejenigen, die glauben und die guten Werke tun, in Gerechtigkeit zu entlohnen. Für die, die ungläubig sind, sind ein Getränk aus heißem Wasser und eine schmerzhafte Pein bestimmt dafür, daß sie ungläubig waren.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mccclxix{Er ist es, der die Sonne zur Leuchte und den Mond zum Licht gemacht und ihm Stationen zugemessen hat, damit ihr die Zahl der Jahre und die Zeitrechnung wißt. Gott hat ja dies nur in Wahrheit erschaffen. Er legt die Zeichen im einzelnen dar für Leute, die Bescheid wissen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mccclxx{Im Aufeinanderfolgen von Nacht und Tag und in dem, was Gott in den Himmeln und auf der Erde erschaffen hat, sind gewiß Zeichen für Leute, die gottesfürchtig sind.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mccclxxi{Diejenigen, die nicht erwarten, Uns zu begegnen, die mit dem diesseitigen Leben zufrieden sind und sich darin wohl fühlen, und die unsere Zeichen unbeachtet lassen,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mccclxxii{Diese haben das Feuer zur Heimstätte für das, was sie erworben haben.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mccclxxiii{Diejenigen, die glauben und die guten Werke tun, leitet ihr Herr wegen ihres Glaubens recht. Unter ihnen werden Bäche fließen in den Gärten der Wonne.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mccclxxiv{Ihr Rufen darin wird sein: «Preis sei Dir, unser Gott!», und ihre Begrüßung darin: «Friede!» Ihr abschließender Ruf: «Lob sei Gott, dem Herrn der Welten!»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mccclxxv{Würde Gott den Menschen das Böse so beschleunigen, wie sie das Gute zu beschleunigen wünschen, wäre für sie die Frist erfüllt. Aber Wir lassen diejenigen, die nicht erwarten, Uns zu begegnen, im Übermaß ihres Frevels umherirren.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mccclxxvi{Und wenn ein Schaden den Menschen trifft, ruft er (Uns) an auf der Seite (liegend), im Sitzen oder Stehen. Wenn Wir ihm seinen Schaden behoben haben, geht er vorbei, als hätte er Uns nicht gegen einen Schaden, der ihn getroffen hat, angerufen. So wird den Maßlosen verlockend gemacht, was sie tun.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mccclxxvii{Wir haben viele Generationen vor euch verderben lassen, als sie Unrecht taten, da ihre Gesandten mit den deutlichen Zeichen zu ihnen kamen, und sie nicht glauben mochten. So vergelten Wir Leuten, die Übeltäter sind.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mccclxxviii{Dann haben Wir euch nach ihnen zu Nachfolgern auf der Erde eingesetzt, um zu sehen, wie ihr handeln würdet.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mccclxxix{Wenn ihnen unsere Zeichen als deutliche Beweise verlesen werden, sagen diejenigen, die nicht erwarten, Uns zu begegnen: «Bring einen anderen Koran als diesen, oder ändere ihn ab.» Sprich: Es steht mir nicht zu, ihn von mir aus abzuändern. Ich folge nur dem, was mir offenbart wird. Ich fürchte, wenn ich gegen meinen Herrn ungehorsam bin, die Pein eines gewaltigen Tages.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mccclxxx{Sprich: Wenn Gott wollte, würde ich ihn euch nicht verlesen, und Er würde euch nicht davon Kenntnis geben. Ich habe doch davor ein Leben lang unter euch verweilt. Habt ihr denn keinen Verstand?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mccclxxxi{Wer ist denn ungerechter, als wer gegen Gott eine Lüge erdichtet oder seine Zeichen für Lüge erklärt? Den Übeltätern wird es gewiß nicht wohl ergehen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mccclxxxii{Sie verehren anstelle Gottes, was ihnen weder schadet noch nützt, und sagen: «Das sind unsere Fürsprecher bei Gott.» Sprich: Wollt ihr denn Gott etwas kundtun, was Er nicht kennt, weder in den Himmeln noch auf der Erde? Preis sei Ihm, und erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mccclxxxiii{Die Menschen waren nur eine einzige Gemeinschaft. Dann wurden sie uneins. Und gäbe es nicht einen früher ergangenen Spruch von deinem Herrn, so wäre zwischen ihnen entschieden über das, worüber sie uneins sind.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mccclxxxiv{Und sie sagen: «Wenn doch auf ihn ein Zeichen von seinem Herrn herabgesandt würde!» Sprich: Das Unsichtbare gehört Gott. So wartet ab. Ich bin mit euch einer von denen, die abwarten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mccclxxxv{Wenn Wir die Menschen nach einem Leid, das sie traf, Barmherzigkeit kosten lassen, schmieden sie gleich Ränke gegen unsere Zeichen. Sprich: Gott kann noch schneller Ränke schmieden. Unsere Boten schreiben auf, was ihr an Ränken schmiedet.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mccclxxxvi{Er ist es, der euch auf dem Festland und auf dem Meer reisen läßt. Wenn ihr dann auf den Schiffen seid und diese mit ihnen bei einem guten Wind dahinfahren und sie sich darüber freuen, dann kommt über sie ein stürmischer Wind, die Wellen kommen über sie von überall her, und sie meinen, daß sie rings umschlossen werden. Da rufen sie Gott an, wobei sie Ihm gegenüber aufrichtig in der Religion sind: «Wenn Du uns hieraus rettest, werden wir zu den Dankbaren zählen.»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mccclxxxvii{Aber wenn Er sie gerettet hat, fangen sie gleich an, zu Unrecht gewalttätig auf der Erde zu handeln. O ihr Menschen, eure Gewalttätigkeit gereicht doch euch selbst zum Schaden. (Es ist doch nur) Nutznießung des diesseitigen Lebens. Dann wird eure Rückkehr zu Uns sein, da werden Wir euch kundtun, was ihr zu tun pflegtet.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mccclxxxviii{Mit dem diesseitigen Leben ist es wie mit dem Wasser, das Wir vom. Himmel herabkommen lassen, worauf die Pflanzen der Erde, wie sie die Menschen und das Vieh verzehren, sich damit vermengen. Wenn dann die Erde ihren Prunk angenommen und sich geschmückt hat und ihre Bewohner meinen, sie verfügen nun über sie, kommt unser Befehl über sie in der Nacht oder am Tag, und Wir machen sie zum abgemähten Land, als ob sie am Tag zuvor nicht in Blüte gestanden hätte. So legen Wir die Zeichen im einzelnen dar für Leute, die nachdenken.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mccclxxxix{Gott, ruft zur Wohnstätte des Friedens, und Er leitet, wen Er will, zu einem geraden Weg.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcccxc{Diejenigen, die rechtschaffen sind, erhalten das Beste und noch mehr. Ihre Gesichter werden weder Ruß noch Erniedrigung bedecken. Das sind die Gefährten des Paradieses; darin werden sie ewig weilen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mcccxci{Diejenigen, die die bösen Taten erwerben, erhalten zur Vergeltung für eine (jede) schlechte Tat das ihr Entsprechende, und Erniedrigung wird schwer auf ihnen lasten - sie haben niemanden, der sie vor Gott schützen könnte -, als wären ihre Gesichter von Fetzen einer finsteren Nacht bedeckt. Das sind die Gefährten des Feuers; darin werden sie ewig weilen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mcccxcii{Und am Tag, da Wir sie alle versammeln. Dann sprechen Wir zu denen, die Polytheisten waren: «Bleibt an eurem Platz stehen, ihr und eure Teilhaber.» Wir scheiden sie dann voneinander. Ihre Teilhaber sagen: «Nicht uns habt ihr verehrt.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mcccxciii{Gott genügt als Zeuge zwischen uns und euch. Wir haben eure Verehrung unbeachtet gelassen.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcccxciv{Dort wird jede Seele erfahren, was sie früher getan hat. Und sie werden zu Gott, ihrem wahren Herrscher, zurückgebracht, und entschwunden ist ihnen dann, was sie zu erdichten pflegten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcccxcv{Sprich: Wer versorgt euch vom Himmel und von der Erde, oder wer verfügt über Gehör und Augenlicht? Und wer bringt das Lebendige aus dem Toten und bringt das Tote aus dem Lebendigen hervor? Und wer regelt die Angelegenheit? Sie werden sagen: «Gott.» Sprich: Wollt ihr denn nicht gottesfürchtig sein?\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcccxcvi{Das ist eben Gott, euer wahrer Herr. Was gibt es denn jenseits der Wahrheit als den Irrtum? Wie leicht laßt ihr euch doch abbringen!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcccxcvii{So ist der Spruch deines Herrn zu Recht gegen die, die freveln, fällig geworden, nämlich daß sie nicht glauben.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcccxcviii{Sprich: Gibt es unter euren Teilhabern einen, der die Schöpfung am Anfang macht und sie dann wiederholt? Sprich: Gott macht die Schöpfung am Anfang und wiederholt sie dann. Wie leicht laßt ihr euch doch abwenden!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcccxcix{Sprich: Gibt es unter euren Teilhabern einen, der zur Wahrheit rechtleitet? Sprich: Gott leitet zur Wahrheit recht. Hat der, der zur Wahrheit rechtleitet, eher Anspruch auf Gefolgschaft, oder der, der nur dann die Rechtleitung findet, wenn er selbst rechtgeleitet wird? Was ist mit euch? Wie urteilt ihr denn?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcd{Und die meisten von ihnen folgen ja nur einer Vermutung. Die Vermutung aber nützt in bezug auf die Wahrheit nichts. Gott weiß, was sie tun.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcdi{Dieser Koran kann unmöglich ohne Gott erdichtet werden. Er ist vielmehr die Bestätigung dessen, was vor ihm vorhanden war, und die ins einzelne gehende Darlegung des Buches. Kein Zweifel an ihm ist möglich; er ist vom Herrn der Welten.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcdii{Oder sagen sie: «Er hat ihn erdichtet»? Sprich: Dann bringt eine Sure, die ihm gleich ist, bei und ruft, wen ihr könnt, anstelle Gottes an, so ihr die Wahrheit sagt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcdiii{Nein, sie erklären für Lüge das, wovon sie kein umfassendes Wissen haben, und bevor seine Deutung zu ihnen gekommen ist. So haben es auch diejenigen, die vor ihnen lebten, für Lüge erklärt. So schau, wie das Ende ``derer war, die Unrecht taten.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcdiv{Und unter ihnen sind welche, die an ihn glauben, und unter ihnen sind welche, die an ihn nicht glauben. Und dein Herr weiß besser über die Unheilstifter Bescheid.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcdv{Und wenn sie dich der Lüge zeihen, dann sprich: Mir kommt mein Tun zu und euch euer Tun. Ihr seid unschuldig an dem, was ich tue; und ich bin unschuldig an dem, was ihr tut.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mcdvi{Und unter ihnen sind welche, die dir zuhören. Bist du es etwa, der die Tauben hören läßt, auch wenn sie keinen Verstand haben?\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcdvii{Und unter ihnen sind welche, die zu dir hinschauen. Bist du es etwa, der die Blinden rechtleitet, auch wenn sie nicht sehen?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcdviii{Gott tut den Menschen kein Unrecht. Vielmehr tun die Menschen sich selbst Unrecht.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mcdix{Und am Tag, da Er sie versammelt, als hätten sie nur eine Stunde vom Tag verweilt, um einander kennenzulernen. Den Verlust haben diejenigen, die die Begegnung mit Gott für Lüge erklärt haben und der Rechtleitung nicht gefolgt sind.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mcdx{Ob Wir dich einen Teil dessen, was Wir ihnen androhen, sehen lassen oder dich abberufen, zu Uns wird ihre Rückkehr sein. Dann ist Gott Zeuge über das, was sie tun.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mcdxi{Und jede Gemeinschaft hat einen Gesandten. Wenn ihr Gesandter zu ihnen kommt, wird zwischen ihnen in Gerechtigkeit entschieden, und ihnen wird nicht Unrecht getan.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mcdxii{Und sie sagen: «Wann wird diese Androhung eintreten, so ihr die Wahrheit sagt?»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mcdxiii{Sprich: Ich kann mir selbst weder Nutzen noch Schaden bringen, außer was Gott will. Für jede Gemeinschaft ist eine Frist festgesetzt. Und wenn ihre Frist kommt, können sie nicht einmal eine Stunde zurückbleiben oder vorausgehen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mcdxiv{Sprich: Was meint ihr? Wenn seine Pein bei Nacht oder bei Tag über euch kommt, was werden die Übeltäter davon zu beschleunigen wünschen?\qt@no{50}}%
\qt@newcmd\qurantrans@de@mcdxv{Werdet ihr, wenn sie hereinbricht, dann daran glauben? Wie? Erst jetzt? Und dabei habt ihr sie doch zu beschleunigen gewünscht.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mcdxvi{Dann wird zu denen, die Unrecht getan haben, gesprochen: «Kostet die ewige Pein. Wird euch denn für etwas anderes vergolten als für das, was ihr erworben habt?»\qt@no{52}}%
\qt@newcmd\qurantrans@de@mcdxvii{Und sie erkundigen sich bei dir: «Ist es wahr?» Sprich: Ja, bei meinem Herrn, es ist bestimmt wahr, und ihr könnt es nicht vereiteln.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mcdxviii{Und würde jeder, der Unrecht getan hat, das besitzen, was auf der Erde ist, er würde sich damit loskaufen. Sie empfinden insgeheim Reue, wenn sie die Pein sehen. Und zwischen ihnen wird in Gerechtigkeit entschieden, und ihnen wird nicht Unrecht getan.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mcdxix{Siehe, Gott gehört, was in den Himmeln und auf der Erde ist. Siehe, das Versprechen Gottes ist wahr. Aber die meisten von ihnen wissen nicht Bescheid.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mcdxx{Er macht lebendig und läßt sterben. Und zu Ihm werdet ihr zurückgebracht.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mcdxxi{O ihr Menschen, zu euch ist nunmehr eine Ermahnung von eurem Herrn gekommen und eine Heilung für euer Inneres, eine Rechtleitung und Barmherzigkeit für die Gläubigen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mcdxxii{Sprich: Über die Huld Gottes und über seine Barmherzigkeit, ja darüber sollen sie sich freuen. Das ist besser als das, was sie zusammentragen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mcdxxiii{Sprich: Was meint ihr? Das, was Gott für euch an Lebensunterhalt herabgesandt hat und was ihr in Verbotenes und Erlaubtes eingeteilt habt, - sprich: Hat Gott es euch erlaubt, oder erdichtet ihr etwas gegen Gott?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mcdxxiv{Was wird die Meinung derer, die gegen Gott Lügen erdichten, am Tag der Auferstehung sein? Gott ist voller Huld gegen die Menschen. Aber die meisten von ihnen sind nicht dankbar.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mcdxxv{Du befaßt dich mit keiner Angelegenheit, und du verliest darüber keinen Koran, und ihr vollzieht keine Handlung, ohne daß Wir Zeugen über euch sind, wenn ihr euch ausgiebig damit beschäftigt. Und es entgeht deinem Herrn nicht das Gewicht eines Stäubchens, weder auf der Erde noch im Himmel. Und es gibt nichts, was kleiner ist als dies oder größer, das nicht in einem deutlichen Buch stünde.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mcdxxvi{Siehe, die Freunde Gottes haben nichts zu befürchten, und sie werden nicht traurig sein,\qt@no{62}}%
\qt@newcmd\qurantrans@de@mcdxxvii{Sie, die glauben und gottesfürchtig sind.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mcdxxviii{Ihnen gilt die frohe Botschaft im diesseitigen Leben und im Jenseits. Unabänderlich sind die Worte Gottes. Das ist der großartige Erfolg.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mcdxxix{Das, was sie sagen, soll dich nicht betrüben. Alle Macht gehört Gott. Er ist es, der alles hört und weiß.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mcdxxx{Siehe, Gott gehört, wer in den Himmeln und wer auf der Erde ist. Gewiß folgen diejenigen, die anstelle Gottes Teilhaber anrufen, ja sie folgen nur Vermutungen, und sie stellen nur Schätzungen an.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mcdxxxi{Er ist es, der euch die Nacht gemacht hat, damit ihr in ihr ruht, und den Tag, an dem man sehen kann. Darin sind Zeichen für Leute, die hören.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mcdxxxii{Sie sagen: «Gott hat sich ein Kind genommen.» Preis sei Ihm! Er ist auf niemanden angewiesen. Ihm gehört, was in den Himmeln und was auf der Erde ist. Ihr habt dafür keine Ermächtigung. Wollt ihr denn über Gott sagen, was ihr nicht wißt?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mcdxxxiii{Sprich: Denen, die gegen Gott Lügen erdichten, wird es nicht wohl ergehen.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mcdxxxiv{Sie haben) auf der Erde eine Nutznießung, dann wird ihre Rückkehr zu Uns sein. Dann lassen Wir sie die harte Pein kosten dafür, daß sie ungläubig waren.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mcdxxxv{Und verlies ihnen den Bericht über Noach, als er zu seinem Volk sagte: «O mein Volk, wenn ihr schwer ertragen könnt, daß ich mich hinstelle und euch mit den Zeichen Gottes ermahne, so vertraue ich auf Gott. Einigt euch über eure Angelegenheit, ihr und eure Teilhaber, und euer Entschluß soll für euch nicht unklar sein; dann führt ihn an mir aus und gewährt mir keinen Aufschub.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mcdxxxvi{Wenn ihr euch abkehrt, so habe ich von euch keinen Lohn verlangt. Mein Lohn obliegt Gott allein. Und mir ist befohlen worden, einer der Gottergebenen zu sein.»\qt@no{72}}%
\qt@newcmd\qurantrans@de@mcdxxxvii{Sie ziehen ihn aber der Lüge. Da erretteten Wir ihn und diejenigen, die mit ihm waren, im Schiff. Und Wir machten sie zu aufeinanderfolgenden Generationen und ließen die, die unsere Zeichen für Lüge erklärten, ertrinken. So schau, wie das Ende derer war, die gewarnt worden sind.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mcdxxxviii{Dann schickten Wir nach ihm Gesandte zu ihrem (jeweiligen) Volk; sie kamen mit den deutlichen Zeichen zu ihnen. Aber sie vermochten nicht, an das zu glauben, was sie früher für Lüge erklärt hatten. So versiegeln Wir die Herzen derer, die Übertretungen begehen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mcdxxxix{Dann schickten Wir nach ihnen Mose und Aaron mit unseren Zeichen zu Pharao und seinen Vornehmen. Sie verhielten sich hochmütig und waren Übeltäter.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mcdxl{Als nun die Wahrheit von Uns zu ihnen kam, sagten sie: «Das ist ja eine offenkundige Zauberei.»\qt@no{76}}%
\qt@newcmd\qurantrans@de@mcdxli{Mose sagte: «Wie könnt ihr denn (das) von der Wahrheit sagen, nachdem sie zu euch gekommen ist? Ist das etwa Zauberei? Den Zauberern wird es nicht wohl ergehen.»\qt@no{77}}%
\qt@newcmd\qurantrans@de@mcdxlii{Sie sagten: «Bist du zu uns gekommen, um uns von dem abzubringen, was wir bei unseren Vätern vorgefunden haben, und damit die Oberhoheit auf der Erde euch zufalle? Wir werden euch gewiß nicht glauben.»\qt@no{78}}%
\qt@newcmd\qurantrans@de@mcdxliii{Und der Pharao sagte: «Bringt mir jeden erfahrenen Zauberer.»\qt@no{79}}%
\qt@newcmd\qurantrans@de@mcdxliv{Als die Zauberer kamen, sagte Mose zu ihnen: «Werft, was ihr werfen wollt.»\qt@no{80}}%
\qt@newcmd\qurantrans@de@mcdxlv{Als sie geworfen hatten, sagte Mose: «Was ihr vorgebracht habt, ist Zauberei. Gott wird sie zunichte machen. Gott läßt das Tun der Unheilstifter nicht als gut gelten.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mcdxlvi{Und Gott bestätigt die Wahrheit mit seinen Worten, auch wenn es den Übeltätern zuwider ist.»\qt@no{82}}%
\qt@newcmd\qurantrans@de@mcdxlvii{Und dem Mose glaubten nur junge Leute aus seinem Volk, dabei hatten sie Angst vor dem Pharao und ihren Vornehmen, daß er sie der Versuchung aussetzen würde. Der Pharao hatte ja hohe Gewalt im Land und war einer der Maßlosen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mcdxlviii{Und Mose sagte: «O mein Volk, wenn ihr an Gott glaubt, dann vertraut auf Ihn, so ihr gottergeben seid.»\qt@no{84}}%
\qt@newcmd\qurantrans@de@mcdxlix{Sie sagten: «Auf Gott vertrauen wir. Unser Herr, mach uns nicht zu einer Versuchung für die Leute, die Unrecht tun,\qt@no{85}}%
\qt@newcmd\qurantrans@de@mcdl{Und errette uns in deiner Barmherzigkeit von den ungläubigen Leuten.»\qt@no{86}}%
\qt@newcmd\qurantrans@de@mcdli{Und Wir offenbarten dem Mose und seinem Bruder: «Weiset eurem Volk in Ägypten Häuser zu und macht, daß eure Häuser einander gegenüberstehen, und verrichtet das Gebet. Und verkünde den Gläubigen eine frohe Botschaft.»\qt@no{87}}%
\qt@newcmd\qurantrans@de@mcdlii{Und Mose sagte: «Unser Herr, du hast Pharao und seinen Vornehmen im diesseitigen Leben Pracht und Vermögen zukommen lassen, unser Herr, damit sie (die Leute) von deinem Weg abirren lassen. Unser Herr, wisch ihr Vermögen aus und schnüre ihre Herzen fest, so daß sie nicht glauben, bis sie die schmerzhafte Pein sehen.»\qt@no{88}}%
\qt@newcmd\qurantrans@de@mcdliii{Er sprach: «Eure Bitte ist erhört. So verhaltet euch recht und folgt nicht dem Weg derer, die nicht Bescheid wissen.»\qt@no{89}}%
\qt@newcmd\qurantrans@de@mcdliv{Und Wir ließen die Kinder Israels das Meer überqueren. Da verfolgten sie der Pharao und seine Truppen in Auflehnung und Übertretung. Als er am Ertrinken war, sagte er: «Ich glaube, daß es keinen Gott gibt außer dem, an den die Kinder Israels glauben. Und ich gehöre nun zu den Gottergebenen.»\qt@no{90}}%
\qt@newcmd\qurantrans@de@mcdlv{«Wie? Erst jetzt, wo du zuvor ungehorsam warst und zu den Unheilstiftern gehörtest?\qt@no{91}}%
\qt@newcmd\qurantrans@de@mcdlvi{Heute wollen Wir dich mit deinem Leib erretten, damit du für die, die nach dir kommen, ein Zeichen seist.» Und viele von den Menschen lassen unsere Zeichen unbeachtet.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mcdlvii{Und Wir haben den Kindern Israels einen wahrhaftigen Aufenthaltsort zugewiesen und ihnen von den köstlichen Dingen beschert. Sie sind aber erst uneins geworden, nachdem das Wissen zu ihnen gekommen war. Dein Herr wird am Tag der Auferstehung zwischen ihnen über das entscheiden, worüber sie uneins waren.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mcdlviii{Wenn du über das, was Wir zu dir hinabgesandt haben, im Zweifel bist, dann frag diejenigen, die (bereits) vor dir das Buch lesen. Wahrlich, zu dir ist die Wahrheit von deinem Herrn gekommen, so sei nicht einer von den Zweiflern,\qt@no{94}}%
\qt@newcmd\qurantrans@de@mcdlix{Und sei nicht einer von denen, die die Zeichen Gottes für Lüge erklärt haben, sonst wirst du zu den Verlierern gehören.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mcdlx{Diejenigen, gegen die der Spruch deines Herrn fällig geworden ist, glauben nicht,\qt@no{96}}%
\qt@newcmd\qurantrans@de@mcdlxi{Auch wenn jedes Zeichen zu ihnen käme, bis sie die schmerzhafte Pein sehen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcdlxii{Wenn doch (irgend) eine Stadt geglaubt hätte, so daß ihr Glaube ihr genützt hätte! (Keine tat es), außer dem Volk des Jonas. Als diese geglaubt haben, haben Wir die Pein der Schande im diesseitigen Leben von ihnen aufgehoben und ihnen eine Nutznießung für eine Weile gewährt.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mcdlxiii{Wenn dein Herr wollte, würden die, die auf der Erde sind, alle zusammen gläubig werden. Bist du es etwa, der die Menschen zwingen kann, gläubig zu werden?\qt@no{99}}%
\qt@newcmd\qurantrans@de@mcdlxiv{Niemand kann glauben, es sei denn mit der Erlaubnis Gottes. Und Er legt Greuel auf diejenigen, die keinen Verstand haben.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mcdlxv{Sprich: Schaut, was in den Himmeln und auf der Erde ist. Aber die Zeichen und die Warnungen nützen den Leuten, die nicht glauben, nicht.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mcdlxvi{Erwarten sie denn etwas anderes, als was den Tagen derer gleicht, die vor ihnen dahingegangen sind? Sprich: Wartet nur ab, ich bin mit euch einer von denen, die abwarten.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mcdlxvii{Dann erretten Wir unsere Gesandten und die, die glauben. Ebenso - es ist eine Uns obliegende Pflicht - erretten Wir die Gläubigen.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mcdlxviii{Sprich: O ihr Menschen, wenn ihr über meine Religion im Zweifel seid, so diene ich nicht denen, denen ihr anstelle Gottes dient, sondern ich diene Gott, der euch abberuft. Und mir wurde befohlen, einer der Gläubigen zu sein.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mcdlxix{Und: Richte dein Gesicht auf die Religion als Anhänger des reinen Glaubens, und sei nicht einer der Polytheisten.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mcdlxx{Und rufe nicht anstelle Gottes an, was dir weder nützt noch schadet. Wenn du es tust, dann gehörst du zu denen, die Unrecht tun.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mcdlxxi{Wenn Gott dich mit einem Schaden trifft, dann kann niemand ihn beheben, außer Ihm. Und wenn Er für dich etwas Gutes will, dann kann niemand seine Huld zurückweisen. Er trifft damit, wen von seinen Dienern Er will. Und Er ist voller Vergebung und barmherzig.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mcdlxxii{Sprich: O ihr Menschen, zu euch ist die Wahrheit von eurem Herrn gekommen. Wer der Rechtleitung folgt, folgt ihr zu seinem eigenen Vorteil. Und wer irregeht, geht irre zu seinem eigenen Schaden. Und ich bin nicht euer Sachwalter.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mcdlxxiii{Und folge dem, was dir offenbart wird, und sei geduldig, bis Gott sein Urteil fällt. Er ist der Beste derer, die Urteile fällen.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mcdlxxiv{\basmalahde Alif Lam Ra. (Dies ist) ein Buch, dessen Zeichen eindeutig festgefügt und dann im einzelnen dargelegt sind von einem Weisen und Kundigen:\qt@no{1}}%
\qt@newcmd\qurantrans@de@mcdlxxv{«Ihr sollt Gott allein dienen» - ich bin euch von Ihm her ein Warner und Freudenbote -,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mcdlxxvi{Und: «Bittet euren Herrn um Vergebung, dann wendet euch Ihm zu, so wird Er euch eine schöne Nutznießung auf eine bestimmte Frist geben, und Er wird jedem, der ein Verdienst (erworben) hat, sein Verdienst zukommen lassen.» Wenn ihr euch abkehrt, so fürchte ich für euch die Pein eines schweren Tages.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mcdlxxvii{Zu Gott wird eure Rückkehr sein, und Er hat Macht zu allen Dingen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mcdlxxviii{Aber siehe, sie falten ihre Brust zusammen, um sich vor Ihm zu verstecken. Siehe, wenn sie sich (auch) mit ihren Gewändern überdecken, Er weiß doch, was sie geheimhalten und was sie offenlegen. Er weiß über das innere Geheimnis Bescheid.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mcdlxxix{Und es gibt kein Tier auf der Erde, ohne daß Gott für seinen Unterhalt sorgen und seinen Aufenthaltsort und seinen Aufbewahrungsort kennen würde. Alles steht in einem deutlichen Buch.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mcdlxxx{Er ist es, der die Himmel und die Erde in sechs Tagen erschaffen hat, während sein Thron auf dem Wasser war, um euch zu prüfen (und festzustellen), wer von euch am besten handelt. Und wenn du sagst: «Ihr werdet nach dem Tod auferweckt werden», sagen diejenigen, die ungläubig sind, bestimmt: «Das ist ja offenkundige Zauberei.»\qt@no{7}}%
\qt@newcmd\qurantrans@de@mcdlxxxi{Und wenn Wir ihnen die Pein für eine bestimmte Weile zurückstellen, sagen sie wohl: «Was hält sie zurück?» Siehe, am Tag, da sie über sie kommt, kann sie nicht von ihnen abgewehrt werden, und es wird sie das umschließen, worüber sie spotteten.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mcdlxxxii{Und wenn Wir den Menschen von uns her Barmherzigkeit kosten lassen und sie ihm dann wegnehmen, ist er sehr verzweifelt und undankbar.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mcdlxxxiii{Und wenn Wir ihn nach einem Leid, das ihn erfaßt hat, Angenehmes kosten lassen, sagt er gewiß: «Das Übel ist von mir gewichen.» Und er ist froh und prahlerisch,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mcdlxxxiv{Mit Ausnahme derer, die geduldig sind und die guten Werke tun; für sie ist Vergebung und ein großer Lohn bestimmt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mcdlxxxv{Vielleicht möchtest du einen Teil von dem, was dir offenbart wird, auslassen und fühlst dadurch deine Brust beklommen, und dies, weil sie sagen: «Wäre doch ein Schatz auf ihn herabgesandt worden oder ein Engel mit ihm gekommen!» Du aber bist nur ein Warner. Und Gott ist Sachwalter über alle Dinge.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mcdlxxxvi{Oder sagen sie: «Er hat ihn erdichtet»? Sprich: Dann bringt zehn Suren bei, die ihm gleich wären und die erdichtet sind, und ruft, wen ihr könnt, anstelle Gottes an, so ihr die Wahrheit sagt.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mcdlxxxvii{Wenn sie euch nicht erhören, dann wißt, daß es mit Gottes Wissen herabgesandt worden ist und daß es keinen Gott gibt außer ihm. Werdet ihr nun Gottergebene sein?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mcdlxxxviii{Denen, die das diesseitige Leben und seinen Schmuck begehren, erstatten Wir in ihm ihre Taten voll, und ihnen wird in ihm nichts abgezogen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mcdlxxxix{Das sind die, für die im Jenseits nichts bestimmt ist als das Feuer. Wertlos ist, was sie in ihm vollbracht haben, und nichtig ist, was sie zu tun pflegten.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mcdxc{Ist denn der, der von seinem Herrn einen deutlichen Beweis hat, auf den auch ein Zeuge von Ihm folgt und dem das Buch des Mose vorausging als Vorbild und Barmherzigkeit... (den anderen gleich)? Diese glauben daran. Wer aber von den Parteien ihn verleugnet, dessen Verabredungsort ist das Feuer. So sei nicht über ihn im Zweifel, er ist die Wahrheit von deinem Herrn. Aber die meisten Menschen glauben nicht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mcdxci{Und wer ist ungerechter als der, der gegen Gott eine Lüge erdichtet? Jene werden ihrem Herrn vorgeführt, und die Zeugen werden sagen: «Das sind die, die gegen ihren Herrn gelogen haben.» Möge der Fluch Gottes über die kommen, die Unrecht tun,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mcdxcii{Die vom Weg Gottes abweisen und sich ihn krumm wünschen, und die ja das Jenseits verleugnen!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mcdxciii{Jene konnten (Gottes Willen) auf der Erde nicht vereiteln, und sie haben außer Gott keine Freunde. Die Pein wird ihnen verdoppelt. Sie vermochten nicht zu hören, und sie konnten nicht sehen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mcdxciv{Das sind die, die sich selbst verloren haben, und es ist ihnen entschwunden, was sie erdichteten.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mcdxcv{Zweifellos sind sie im Jenseits die größten Verlierer.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mcdxcvi{Diejenigen, die glauben und die guten Werke tun und sich vor ihrem Herrn demütigen, das sind die Gefährten des Paradieses; darin werden sie ewig weilen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mcdxcvii{Es ist mit den beiden Gruppen wie mit dem Blinden und dem Tauben und dem, der sehen, und dem, der hören kann. Sind die beiden etwa einander gleichzusetzen? Wollt ihr es nicht bedenken?\qt@no{24}}%
\qt@newcmd\qurantrans@de@mcdxcviii{Und Wir sandten Noach zu seinem Volk: «Ich bin euch ein deutlicher Warner:\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcdxcix{Ihr sollt Gott allein dienen. Ich fürchte für euch die Pein eines schmerzvollen Tages.»\qt@no{26}}%
\qt@newcmd\qurantrans@de@md{Die Vornehmen aus seinem Volk, die ungläubig waren, sagten: «Wir sehen, daß du nur ein Mensch bist wie wir. Und wir sehen, daß nur die dir folgen, die unsere Niedrigsten sind, und zwar ohne reifliche Überlegung. Und wir sehen bei euch keinen Vorzug über uns. Vielmehr meinen wir, daß ihr lügt.»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdi{Er sagte: «O mein Volk, was meint ihr? Wenn ich einen deutlichen Beweis von meinem Herrn habe und Er mir eine Barmherzigkeit von sich hat zukommen lassen, die eurem Blick entzogen wurde, könnten wir sie euch da aufzwingen, wo sie euch zuwider ist?\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdii{Und, o mein Volk, ich verlange von euch dafür kein Geld. Mein Lohn obliegt Gott allein. Und ich werde nicht diejenigen, die glauben, verstoßen; sie werden ihrem Herrn begegnen. Aber ich sehe, daß ihr Leute seid, die töricht sind.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdiii{Und, o mein Volk, wer wird mir denn vor Gott Unterstützung gewähren, wenn ich sie verstoßen sollte? Wollt ihr es nicht bedenken?\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdiv{Und ich sage euch nicht, ich hätte die Vorratskammern Gottes, und ich kenne auch nicht das Unsichtbare. Und ich sage nicht, ich sei ein Engel. Und ich sage nicht (von denen), die eure Augen verachten, Gott werde ihnen niemals etwas Gutes zukommen lassen - Gott weiß besser, was in ihrem Inneren ist -, sonst würde ich zu denen gehören, die Unrecht tun.»\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdv{Sie sagten: «O Noach, du hast mit uns gestritten und den Streit mit uns lange geführt. So bring uns doch her, was du uns androhst, so du zu denen gehörst, die die Wahrheit sagen.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdvi{Er sagte: «Bringen wird es euch Gott, wenn Er will, und ihr könnt es nicht vereiteln.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdvii{Und nicht nützt euch mein Rat, so ich euch raten will, wenn Gott euch abirren lassen will. Er ist euer Herr, und zu Ihm werdet ihr zurückgebracht.»\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdviii{Oder sagen sie wohl: «Er hat ihn erdichtet»? Sprich: Wenn ich ihn erdichtet habe, so lastet meine Übeltat auf mir, und ich bin unschuldig an dem, was ihr verübt.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdix{Und dem Noach wurde offenbart: «Niemand aus deinem Volk wird glauben außer denen, die bereits geglaubt haben. So fühle dich nicht elend wegen dessen, was sie taten.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdx{Und verfertige das Schiff vor unseren Augen und nach unserer Offenbarung. Und sprich Mich nicht an zugunsten derer, die Unrecht getan haben. Sie werden sicher ertränkt werden.»\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdxi{Er ging daran, das Schiff zu verfertigen. Und sooft Vornehme aus seinem Volk an ihm vorbeikamen, verhöhnten sie ihn. Er sagte: «Wenn ihr uns jetzt verhöhnt, werden auch wir euch verhöhnen, so wie ihr (uns jetzt) verhöhnt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdxii{Und ihr werdet zu wissen bekommen, über wen eine Pein kommen wird, die ihn zuschanden macht, und (über wen) eine beständige Pein hereinbricht.»\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdxiii{Als nun unser Befehl kam und der Ofen brodelte, sprachen Wir: «Lade darin ein Paar von jeder Art und deine Angehörigen außer dem, gegen den der Spruch vorher erging, und diejenigen, die glauben.» Mit ihm glaubten aber nur wenige.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdxiv{Und er sagte: «Steigt hinein. Im Namen Gottes erfolge seine Fahrt und seine Landung. Mein Herr ist voller Vergebung und barmherzig.»\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdxv{Es fuhr mit ihnen dahin auf Wogen wie Berge. Noach rief seinem Sohn zu, der abseits stand: «O mein Sohn, steig ein mit uns und sei nicht mit den Ungläubigen.»\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdxvi{Er sagte: «Ich werde Unterkunft finden auf einem Berg, der mich vor dem Wasser schützt.» Er sagte: «Es gibt heute niemanden, der vor dem Befehl Gottes schützen könnte, außer für den, dessen Er sich erbarmt.» Die Woge trennte sie beide, und so war er einer von denen, die ertränkt wurden.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdxvii{Und es wurde gesprochen: «O Erde, verschlinge dein Wasser. O Himmel, halt ein.» Das Wasser nahm ab, und die Angelegenheit war entschieden. Es hielt auf dem Djudi an. Und es wurde gesprochen: «Weg mit den Leuten, die Unrecht tun!»\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdxviii{Und Noach rief seinen Herrn an und sagte: «Mein Herr, mein Sohn gehört zu meinen Angehörigen, und dein Versprechen ist die Wahrheit, und du bist der Weiseste derer, die Urteile fällen.»\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdxix{Er sprach: «O Noach, er gehört nicht zu deinen Angehörigen. Es ist eine Tat, die nicht gut ist. So bitte Mich nicht um etwas, wovon du kein Wissen hast. Ich ermahne dich, nicht einer der Törichten zu sein.»\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdxx{Er sagte: «O mein Herr, ich suche Zuflucht bei Dir davor, daß ich Dich um etwas bitte, wovon ich kein Wissen habe. Wenn Du mir nicht vergibst und dich meiner (nicht) erbarmst, werde ich zu den Verlierern gehören.»\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdxxi{Es wurde gesprochen: «O Noach, steig hinunter in unserem Frieden und mit unseren Segnungen über dich und über Gemeinschaften von denen, die mit dir sind. Es gibt Gemeinschaften, denen Wir eine Nutznießung geben werden, dann wird sie eine schmerzhafte Pein von Uns erfassen.»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdxxii{Dies gehört zu den Berichten über das Unsichtbare, die Wir dir offenbaren. Du wußtest sie vorher nicht, weder du noch dein Volk. Sei nun geduldig. Das Ende gehört den Gottesfürchtigen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdxxiii{Und (Wir sandten) zu Aad ihren Bruder Hud. Er sagte: «O mein Volk, dienet Gott. Ihr habt keinen Gott außer Ihm. Ihr erdichtet ja nur Lügen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdxxiv{O mein Volk, ich verlange von euch keinen Lohn. Mein Lohn obliegt dem, der mich geschaffen hat. Habt ihr denn keinen Verstand?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdxxv{Und, o mein Volk, bittet euren Herrn um Vergebung, dann wendet euch Ihm zu, so wird Er den Himmel über euch ergiebig regnen lassen und euch zu eurer Kraft noch mehr Kraft hinzu verleihen. Und kehrt euch nicht als Übeltäter ab.»\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdxxvi{Sie sagten: «O Hud, du hast uns keinen deutlichen Beweis gebracht. Wir werden nicht unsere Götter auf dein Wort hin verlassen, und wir werden dir nicht glauben.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mdxxvii{Wir können nur sagen, daß einer unserer Götter dir etwas Böses zugefügt hat.» Er sagte: «Ich nehme Gott zum Zeugen, und auch ihr sollt bezeugen, daß ich unschuldig bin an dem, was ihr (Ihm) beigesellt\qt@no{54}}%
\qt@newcmd\qurantrans@de@mdxxviii{und) neben Ihn (stellt). Geht gegen mich allesamt mit eurer List vor und gewährt mir keinen Aufschub.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mdxxix{Ich vertraue auf Gott, meinen Herrn und euren Herrn. Es gibt kein Tier, das Er nicht an seinem Schopf halten würde. Mein Herr befindet sich auf einem geraden Weg.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mdxxx{Wenn ihr euch abkehrt, so habe ich euch das ausgerichtet, womit ich zu euch gesandt worden bin. Mein Herr wird ein anderes Volk nach euch folgen lassen, und ihr könnt Ihm nichts schaden. Mein Herr ist Hüter über alle Dinge.»\qt@no{57}}%
\qt@newcmd\qurantrans@de@mdxxxi{Als nun unser Befehl eintraf, erretteten Wir Hud und diejenigen, die mit ihm glaubten, aus Barmherzigkeit von Uns, und Wir erretteten sie vor einer schweren Pein.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mdxxxii{Das waren die Aad. Sie leugneten die Zeichen ihres Herrn und waren gegen seine Gesandten ungehorsam und folgten dem Befehl eines jeden widerspenstigen Gewaltherrschers.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mdxxxiii{Aber der Fluch verfolgte sie im Diesseits und (so auch) am Tag der Auferstehung. Die Aad waren undankbar gegen ihren Herrn. Ja, weg mit Aad, dem Volk von Hud!\qt@no{60}}%
\qt@newcmd\qurantrans@de@mdxxxiv{Und (Wir sandten) zu Thamud ihren Bruder Salih. Er sagte: «O mein Volk, dienet Gott. Ihr habt keinen Gott außer Ihm. Er hat euch aus der Erde entstehen lassen und sie euch zu bebauen und zu bestellen gegeben. So bittet Ihn um Vergebung, dann wendet euch Ihm zu. Mein Herr ist nahe und bereit zu erhören.»\qt@no{61}}%
\qt@newcmd\qurantrans@de@mdxxxv{Sie sagten: «O Salih, du warst vorher in unserer Mitte einer, auf den man Hoffnung setzte. Willst du uns denn verbieten, zu verehren, was unsere Väter immer verehrt haben? Wir hegen über das, wozu du uns aufrufst, einen starken Zweifel.»\qt@no{62}}%
\qt@newcmd\qurantrans@de@mdxxxvi{Er sagte: «O mein Volk, was meint ihr? Wenn ich einen deutlichen Beweis von meinem Herrn habe und Er mir eine Barmherzigkeit von sich hat zukommen lassen, wer wird mir vor Gott Unterstützung gewähren, wenn ich gegen Ihn ungehorsam bin? Ihr bringt mir nur noch einen größeren Verlust.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mdxxxvii{Und, o mein Volk, dies ist die Kamelstute Gottes, euch zum Zeichen. Laßt sie auf Gottes Erde weiden und rührt sie nicht mit etwas Bösem an. Sonst ergreift euch eine baldige Pein.»\qt@no{64}}%
\qt@newcmd\qurantrans@de@mdxxxviii{Sie schnitten ihr die Flechsen durch und stachen sie. Da sagte er: «Genießt (euer Leben) in eurer Wohnstätte noch drei Tage lang. Dies ist eine Androhung, die nicht erlogen ist.»\qt@no{65}}%
\qt@newcmd\qurantrans@de@mdxxxix{Als nun unser Befehl eintraf, erretteten Wir Salih und diejenigen, die mit ihm glaubten, aus Barmherzigkeit von Uns, auch vor der Schande jenes Tages. Dein Herr ist ja der Starke, der Mächtige.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mdxl{Da ergriff diejenigen, die Unrecht taten, der Schrei, und am Morgen lagen sie in ihren Wohnstätten auf ihren Gesichtern,\qt@no{67}}%
\qt@newcmd\qurantrans@de@mdxli{Als hätten sie nicht lange darin gewohnt. Die Thamud waren undankbar gegen ihren Herrn. Ja, weg mit Thamud!\qt@no{68}}%
\qt@newcmd\qurantrans@de@mdxlii{Unsere Boten kamen zu Abraham mit der frohen Botschaft. Sie sagten: «Frieden!» Er sagte: «Frieden!» Es dauerte nicht lange, da brachte er ein geschmortes Kalb herbei.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mdxliii{Und als er sah, daß ihre Hände nicht hinlangten, fand er es von ihnen befremdlich, und er empfand Angst vor ihnen. Sie sagten: «Hab keine Angst. Wir sind zu dem Volk von Lot gesandt.»\qt@no{70}}%
\qt@newcmd\qurantrans@de@mdxliv{Seine Frau stand da. Da lachte sie. Da verkündeten Wir ihr Isaak, und nach Isaak Jakob.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mdxlv{Sie sagte: «O wehe mir, soll ich noch gebären, wo ich doch alt bin und dieser da mein Mann ist, ein Greis? Das ist doch eine verwunderliche Sache.»\qt@no{72}}%
\qt@newcmd\qurantrans@de@mdxlvi{Sie sagten: «Bist du verwundert über den Befehl Gottes? Die Barmherzigkeit Gottes und seine Segnungen kommen auf euch, ihr Leute des Hauses! Er ist des Lobes und der Ehre würdig.»\qt@no{73}}%
\qt@newcmd\qurantrans@de@mdxlvii{Als der Schrecken von Abraham gewichen und die frohe Botschaft zu ihm gekommen war, begann er mit Uns über das Volk von Lot zu streiten.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mdxlviii{Abraham war ja langmütig, voller Trauer und bereit, sich Gott reumütig zuzuwenden.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mdxlix{«O Abraham, laß davon ab. Der Befehl deines Herrn ist eingetroffen. Über sie kommt eine Pein, die unabwendbar ist.»\qt@no{76}}%
\qt@newcmd\qurantrans@de@mdl{Und als unsere Boten zu Lot kamen, geriet er ihretwegen in eine böse Lage und wußte daraus keinen Ausweg. Er sagte: «Das ist ein drangsalvoller Tag.»\qt@no{77}}%
\qt@newcmd\qurantrans@de@mdli{Seine Leute kamen zu ihm geeilt. Zuvor pflegten sie Missetaten zu verüben. Er sagte: «O mein Volk, da sind meine Töchter, sie sind reiner für euch. So fürchtet Gott und bringt nicht Schande über mich in Zusammenhang mit meinen Gästen. Gibt es denn unter euch keinen vernünftigen Mann?»\qt@no{78}}%
\qt@newcmd\qurantrans@de@mdlii{Sie sagten: «Du weißt genau, daß wir kein Recht auf deine Töchter haben, und du weißt wohl, was wir wollen.»\qt@no{79}}%
\qt@newcmd\qurantrans@de@mdliii{Er sagte: «Hätte ich doch Kraft genug, um euch zu widerstehen, oder könnte ich nur auf einem starken Stützpunkt Unterkunft finden!»\qt@no{80}}%
\qt@newcmd\qurantrans@de@mdliv{Sie sagten: «O Lot, wir sind die Boten deines Herrn. Sie werden nicht zu dir gelangen. So zieh mit deinen Angehörigen in einem Teil der Nacht fort, und keiner von euch soll sich umdrehen. Ausgenommen deine Frau; treffen wird sie das, was ihnen zustoßen wird. Ihre Verabredungszeit ist der Morgen. Ist nicht der Morgen schon nahe?»\qt@no{81}}%
\qt@newcmd\qurantrans@de@mdlv{Als nun unser Befehl eintraf, kehrten Wir in ihrer Stadt das Oberste zuunterst und ließen auf sie Backsteine aus übereinandergeschichtetem Ton regnen,\qt@no{82}}%
\qt@newcmd\qurantrans@de@mdlvi{Bei deinem Herrn gekennzeichnete (Steine). Und sie liegt denen, die Unrecht tun, gewiß nicht fern.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mdlvii{Und (Wir sandten) zu Madyan ihren Bruder Shuayb. Er sagte: «O mein Volk, dienet Gott. Ihr habt keinen Gott außer Ihm. Laßt an Maß und Gewicht nichts fehlen. Ich sehe, es geht euch gut. Ich fürchte aber für euch die Pein eines umgreifenden Tages.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mdlviii{Und, o mein Volk, gebt volles Maß und Gewicht nach Gerechtigkeit und zieht den Menschen nichts ab, was ihnen gehört, und verbreitet nicht Unheil auf der Erde.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mdlix{Was bei Gott bleibt, ist besser für euch, so ihr gläubig seid. Und ich bin nicht Hüter über euch.»\qt@no{86}}%
\qt@newcmd\qurantrans@de@mdlx{Sie sagten: «O Shuayb, befehlen dir denn deine Gebete, daß wir das verlassen, was unsere Väter verehrt haben, oder davon abstehen, mit unserem Vermögen zu tun, was wir wollen? Du bist ja wohl der Langmütige, der Vernünftige!»\qt@no{87}}%
\qt@newcmd\qurantrans@de@mdlxi{Er sagte: «O mein Volk, was meint ihr, wenn ich einen deutlichen Beweis von meinem Herrn habe und Er mir einen schönen Lebensunterhalt beschert hat? Ich will auch nicht (einfach) euch widersprechen im Hinblick auf das, was ich euch verbiete. Ich will nur Ordnung schaffen, soweit ich es vermag. Durch Gott allein wird mir das Gelingen beschieden. Auf Ihn vertraue ich, und Ihm wende ich mich reumütig zu.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mdlxii{Und, o mein Volk, mein Zerwürfnis (mit euch) soll euch nicht dazu führen, daß euch das trifft, was früher das Volk von Noach oder das Volk von Hud oder das Volk von Salih getroffen hat. Und das Volk von Lot liegt nicht fernab von euch.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mdlxiii{Und bittet euren Herrn um Vergebung und dann wendet euch Ihm zu. Mein Herr ist barmherzig und liebevoll.»\qt@no{90}}%
\qt@newcmd\qurantrans@de@mdlxiv{Sie sagten: «O Shuayb, wir begreifen nicht viel von dem, was du sagst. Und wir sehen, daß du in unserer Mitte schwach bist. Wenn dein Anhang nicht wäre, hätten wir dich gesteinigt. Vor dir haben wir ja keinen Respekt.»\qt@no{91}}%
\qt@newcmd\qurantrans@de@mdlxv{Er sagte: «O mein Volk, habt ihr vor meinem Anhang eher Respekt als vor Gott, und habt ihr Ihn hinter eurem Rücken zurückgelassen? Mein Herr umfaßt, was ihr tut.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mdlxvi{Und, o mein Volk, handelt nach eurem Standpunkt, ich werde auch so handeln. Ihr werdet zu wissen bekommen, über wen eine Pein kommen wird, die ihn zuschanden macht, und wer ein Lügner ist. Und wartet ab, ich warte mit euch ab.»\qt@no{93}}%
\qt@newcmd\qurantrans@de@mdlxvii{Als nun unser Befehl eintraf, erretteten Wir Shuayb und diejenigen, die mit ihm glaubten, aus Barmherzigkeit von Uns. Da ergriff diejenigen, die Unrecht taten, der Schrei, und am Morgen lagen sie in ihren Wohnstätten auf ihren Gesichtern,\qt@no{94}}%
\qt@newcmd\qurantrans@de@mdlxviii{Als hätten sie nicht lange darin gewohnt. Ja, weg mit Madyan, wie auch die Thamud weggerafft wurden!\qt@no{95}}%
\qt@newcmd\qurantrans@de@mdlxix{Und Wir sandten Mose mit unseren Zeichen und einer offenkundigen Ermächtigung\qt@no{96}}%
\qt@newcmd\qurantrans@de@mdlxx{Zu Pharao und seinen Vornehmen. Diese folgten dem Befehl des Pharao; der Befehl des Pharao war aber nicht richtig.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mdlxxi{Er wird seinem Volk am Tag der Auferstehung vorangehen. Er führt sie wie zur Tränke ins Feuer hinab - welch schlimme Tränke, zu der sie hinabgeführt werden!\qt@no{98}}%
\qt@newcmd\qurantrans@de@mdlxxii{Und der Fluch verfolgte sie im Diesseits und (so auch) am Tag der Auferstehung - welch schlimmes Geschenk, das ihnen geschenkt wird!\qt@no{99}}%
\qt@newcmd\qurantrans@de@mdlxxiii{Dies gehört zu den Berichten über die Städte; Wir erzählen es dir. Einige von ihnen stehen noch, andere sind abgemäht.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mdlxxiv{Nicht Wir haben ihnen Unrecht getan, sondern sie haben sich selbst Unrecht getan. Ihre Götter, die sie anstelle Gottes anrufen, haben ihnen nichts genützt, als der Befehl deines Herrn eintraf, und sie brachten ihnen nur noch ein größeres Verderben.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mdlxxv{So ist die Art deines Herrn heimzusuchen, wenn Er die Städte heimsucht, während sie Unrecht tun. Seine Heimsuchung ist schmerzhaft und hart.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mdlxxvi{Darin ist ein Zeichen für den, der die Pein des Jenseits fürchtet. Das ist ein Tag, auf den die Menschen versammelt werden. Das ist ein Tag, den sie alle erleben werden.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mdlxxvii{Und Wir stellen ihn nur für eine bestimmte Frist zurück.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mdlxxviii{Am Tag, da er eintrifft, wird niemand sprechen, außer mit seiner Erlaubnis. Einige von ihnen werden dann unglücklich und andere selig sein.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mdlxxix{Diejenigen, die unglücklich sind, werden dann im Feuer sein; darin werden sie seufzen und schluchzen,\qt@no{106}}%
\qt@newcmd\qurantrans@de@mdlxxx{Und sie werden darin ewig weilen, solange die Himmel und die Erde währen, außer was dein Herr will. Dein Herr tut ja, was Er will.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mdlxxxi{Diejenigen, die selig sind, werden im Paradies sein; darin werden sie ewig weilen, solange die Himmel und die Erde währen, außer was dein Herr will, als Gabe, die nicht verringert wird.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mdlxxxii{So sei nicht über das, was diese da verehren, im Zweifel. Sie verehren nur, wie ihre Väter zuvor verehrt haben. Und Wir werden ihnen ihren Anteil unverkürzt erstatten.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mdlxxxiii{Und Wir ließen dem Mose das Buch zukommen. Da wurden sie darüber uneins. Und gäbe es nicht einen früher ergangenen Spruch von deinem Herrn, so wäre zwischen ihnen entschieden worden. Und sie sind darüber in einem starken Zweifel.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mdlxxxiv{Und allen wird dein Herr bestimmt ihre Taten voll vergelten. Er hat Kenntnis von dem, was sie tun.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mdlxxxv{So verhalte dich recht, wie dir befohlen wurde, du und diejenigen, die mit dir umkehren, und zeigt kein Übermaß an Frevel. Er sieht wohl, was ihr tut.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mdlxxxvi{Und sucht nicht eine Stütze bei denen, die Unrecht tun, sonst erfaßt euch das Feuer. Dann werdet ihr keine Freunde haben außer Gott, und dann werdet ihr keine Unterstützung erfahren.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mdlxxxvii{Und verrichte das Gebet an beiden Enden des Tages und zu Nachtzeiten. Die guten Taten vertreiben die Missetaten. Das ist eine Erinnerung für die, die (Gottes) gedenken.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mdlxxxviii{Und sei geduldig. Gott läßt den Lohn der Rechtschaffenen nicht verlorengehen.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mdlxxxix{Wenn es doch unter den Generationen vor euch einen tugendhaften Rest von Leuten gäbe, die verbieten, auf der Erde Unheil zu stiften - abgesehen von einigen wenigen von ihnen, die Wir gerettet haben! Diejenigen, die Unrecht taten, folgten dem ihnen verliehenen üppigen Leben und wurden Übeltäter.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mdxc{Und dein Herr hätte unmöglich Städte zu Unrecht dem Verderben preisgegeben, während ihre Bewohner Besserung zeigten.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mdxci{Und wenn dein Herr gewollt hätte, hätte Er die Menschen zu einer einzigen Gemeinschaft gemacht. Aber sie sind immer noch uneins,\qt@no{118}}%
\qt@newcmd\qurantrans@de@mdxcii{Außer denen, derer sich dein Herr erbarmt hat. Dazu hat Er sie erschaffen. Und so erfüllt sich der Spruch deines Herrn: «Ich werde die Hölle mit den Djinn und den Menschen allen füllen.»\qt@no{119}}%
\qt@newcmd\qurantrans@de@mdxciii{Alles erzählen Wir dir von den Berichten über die Gesandten,, um dein Herz damit zu festigen. Darin ist die Wahrheit zu dir gekommen, und eine Ermahnung und Erinnerung für die Gläubigen.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mdxciv{Und sprich zu denen, die nicht glauben: Handelt nach eurem Standpunkt, wir werden (auch so) handeln.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mdxcv{Und wartet ab, wir warten auch ab.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mdxcvi{Und Gott gehört das Unsichtbare der Himmel und der Erde, und zu Ihm wird die ganze Angelegenheit zurückgebracht. So diene Ihm und vertraue auf Ihn. Und dein Herr läßt nicht unbeachtet, was ihr tut.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mdxcvii{\basmalahde Alif Lam Ra. Dies sind die Zeichen des deutlichen Buches.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdxcviii{Wir haben es als einen arabischen Koran hinabgesandt, auf daß ihr verständig werdet.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdxcix{Wir erzählen dir die schönste Erzählung dadurch, daß Wir dir diesen Koran offenbart haben. Du warst vordem einer von denen, die (davon) keine Ahnung hatten.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdc{Als Josef zu seinem Vater sagte: «O mein Vater, ich sah elf Sterne und die Sonne und den Mond, ich sah sie vor mir niederfallen.»\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdci{Er sagte: «O mein Sohn, erzähle von deinem Traumgesicht nicht deinen Brüdern, sonst werden sie eine List gegen dich ausführen. Der Satan ist den Menschen ein offenkundiger Feind.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdcii{Und so wird dein Herr dich erwählen und dich etwas von der Deutung der Geschichten lehren und seine Gnade an dir und an der Sippe Jakobs vollenden, wie Er sie vorher an seinen beiden Vätern Abraham und Isaak vollendet hat. Dein Herr weiß Bescheid und ist weise.»\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdciii{Siehe, in Josef und seinen Brüdern sind Zeichen für die, die (nach der Wahrheit) fragen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdciv{Als sie sagten: «Josef und sein Bruder sind unserem Vater bestimmt lieber als wir, obwohl wir eine (beachtliche) Gruppe sind. Unser Vater befindet sich in einem offenkundigen Irrtum.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdcv{Tötet Josef oder werft ihn ins Land hinaus, so wird das Gesicht eures Vaters nur noch auf euch schauen, und danach werdet ihr Leute sein, die rechtschaffen sind.»\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdcvi{Ein Sprecher unter ihnen sagte: «Tötet Josef nicht, werft ihn (lieber) in die verborgene Tiefe der Zisterne, dann wird ihn schon der eine oder andere Reisende aufnehmen, wenn ihr doch etwas tun wollt.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdcvii{Sie sagten: «O unser Vater, warum vertraust du uns Josef nicht an? Wir werden ihm sicher gut raten.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdcviii{Schick ihn morgen mit uns, daß er sich frei bewege und spiele. Wir werden ihn sicher behüten.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdcix{Er sagte: «Es macht mich traurig, daß ihr ihn mitnehmen wollt. Und ich fürchte, daß ihn der Wolf frißt, während ihr nicht auf ihn achtgebt.»\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdcx{Sie sagten: «Sollte ihn der Wolf fressen, wo wir doch eine (beachtliche) Gruppe sind, dann werden wir gewiß Verlust davon tragen.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdcxi{Als sie ihn mitnahmen und übereinkamen, ihn in die verborgene Tiefe der Zisterne hinunterzulassen... - Und Wir offenbarten ihm: «Du wirst ihnen noch das, was sie hier getan haben, kundtun, ohne daß sie es merken.»\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdcxii{Und am Abend kamen sie weinend zu ihrem Vater.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdcxiii{Sie sagten: «O unser Vater, wir gingen, um einen Wettlauf zu machen, und ließen Josef bei unseren Sachen zurück. Da fraß ihn der Wolf. Du glaubst uns wohl nicht, auch wenn wir die Wahrheit sagen.»\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdcxiv{Sie trugen auf sein Hemd falsches Blut auf. Er sagte: «Nein, eure Seele hat euch etwas eingeredet. (Es gilt) schöne Geduld (zu üben). Gott ist der, der um Hilfe gebeten wird gegen das, was ihr beschreibt.»\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdcxv{Reisende kamen vorbei. Sie schickten ihren Wasserschöpfer, und er ließ seinen Eimer hinunter. Er sagte: «O gute Nachricht! Da ist ein Junge.» Sie versteckten ihn als Ware. Und Gott wußte wohl, was sie taten.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdcxvi{Und sie verkauften ihn für einen zu niedrigen Preis, einige gezählte Drachmen. Und sie übten Verzicht in bezug auf ihn.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdcxvii{Und derjenige aus Ägypten, der ihn gekauft hatte, sagte zu seiner Frau: «Bereite ihm eine freundliche Bleibe. Möge er uns Nutzen bringen, oder vielleicht nehmen wir ihn als Kind an.» Und Wir gaben dem Josef eine angesehene Stellung im Land. Und Wir wollten ihn die Deutung der Geschichten lehren. Und Gott ist in seiner Angelegenheit überlegen. Aber die meisten Menschen wissen nicht Bescheid.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdcxviii{Als er seine Vollkraft erreicht hatte, ließen Wir ihm Urteilskraft und Wissen zukommen. So entlohnen Wir die Rechtschaffenen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdcxix{Und die, in deren Haus er war, versuchte, ihn zu verführen. Sie schloß die Türen ab und sagte: «Komm her.» Er sagte: «Gott behüte! Er, mein Besitzer, hat mir eine schöne Bleibe bereitet. Denen, die Unrecht tun, wird es nicht wohl ergehen.»\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdcxx{Sie hätte sich beinahe mit ihm eingelassen, und er hätte sich beinahe mit ihr eingelassen, hätte er nicht den Beweis seines Herrn gesehen. Dies (geschah), damit Wir das Böse und das Schändliche von ihm abwehrten. Er gehört ja zu unseren auserwählten Dienern.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdcxxi{Sie suchten beide als erster die Tür zu erreichen. Sie zerriß ihm von hinten das Hemd. Und sie trafen auf ihren Herrn bei der Tür. Sie sagte: «Der Lohn dessen, der deiner Familie Böses antun wollte, ist ja wohl das Gefängnis oder eine schmerzhafte Pein.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdcxxii{Er sagte: «Sie war es, die versucht hat, mich zu verführen.» Und ein Zeuge aus ihrer Familie bezeugte: «Wenn sein Hemd vorn zerrissen ist, hat sie die Wahrheit gesagt, und er ist einer von denen, die lügen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdcxxiii{Und wenn sein Hemd hinten zerrissen ist, hat sie gelogen, und er ist einer von denen, die die Wahrheit sagen.»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdcxxiv{Als er nun sah, daß sein Hemd hinten zerrissen war, sagte er: «Das ist eine List von euch. Eure List ist gewaltig.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdcxxv{Josef, wende dich davon ab. Und (du), bitte um Vergebung für deine Schuld. Du gehörst ja zu denen, die sich versündigt haben.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdcxxvi{Nun sagten Frauen in der Stadt: «Die Gemahlin des Hochmögenden versucht, ihren Knecht zu verführen. Er hat sie in leidenschaftliche Liebe versetzt. Wir sehen, sie befindet sich in einem offenkundigen Irrtum.»\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdcxxvii{Als sie von ihren Ränken hörte, schickte sie zu ihnen und bereitete ihnen ein Gelage. Sie ließ einer jeden von ihnen ein Messer geben und sagte (zu Josef): «Komm zu ihnen heraus.» Als sie ihn sahen, fanden sie ihn außerordentlich, und sie schnitten sich in die Hände und sagten: «Gott bewahre! Das ist nicht ein Mensch. Das ist nur ein edler Engel.»\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdcxxviii{Sie sagte: «Das ist der, dessentwegen ihr mich getadelt habt. Ich habe versucht, ihn zu verführen. Er aber hielt an seiner Unschuld fest. Und wenn er nicht tut, was ich ihm befehle, wird er bestimmt ins Gefängnis geworfen werden, und er wird zu denen gehören, die erniedrigt werden.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdcxxix{Er sagte: «Mein Herr, mir ist das Gefängnis lieber als das, wozu sie mich auffordern. Und wenn du ihre List von mir nicht abwehrst, werde ich mich zu ihnen hingezogen fühlen und einer der Törichten sein.»\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdcxxx{Sein Herr erhörte ihn und wehrte ihre List von ihm ab. Er ist es, der alles hört und weiß.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdcxxxi{Dann, nachdem sie die Zeichen gesehen hatten, schien es ihnen angebracht, ihn eine Zeitlang ins Gefängnis zu werfen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdcxxxii{Mit ihm kamen zwei Knechte ins Gefängnis. Der eine von ihnen sagte: «Ich sah mich Wein keltern.» Der andere sagte: «Ich sah mich auf dem Kopf Brot tragen, von dem die Vögel fraßen. So tu uns kund, wie dies zu deuten ist. Wir sehen es, du gehörst zu den Rechtschaffenen.»\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdcxxxiii{Er sagte: «Es wird euch das Essen, mit dem ihr versorgt werdet, nicht gebracht, ohne daß ich euch kundgetan habe, wie es zu deuten ist, bevor es euch gebracht wird. Das ist etwas von dem, was mich mein Herr gelehrt hat. Verlassen habe ich die Glaubensrichtung von Leuten, die nicht an Gott glauben und die das Jenseits verleugnen,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdcxxxiv{Und ich bin der Glaubensrichtung meiner Väter Abraham, Isaak und Jakob gefolgt. Wir dürfen Gott nichts beigesellen. Das ist etwas von der Huld Gottes zu uns und zu den Menschen. Aber die meisten Menschen sind nicht dankbar.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdcxxxv{O ihr beiden Insassen des Gefängnisses! Sind verschiedene Herren besser, oder der eine Gott, der bezwingende Macht besitzt?\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdcxxxvi{Ihr dient außer Ihm nur Namen, die ihr genannt habt, ihr und eure Väter, für die aber Gott keine Ermächtigung herabgesandt hat. Das Urteil gehört Gott allein. Er hat befohlen, daß ihr nur Ihm dienen sollt. Das ist die richtige Religion. Aber die meisten Menschen wissen nicht Bescheid.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdcxxxvii{O ihr beiden Insassen des Gefängnisses! Der eine von euch wird seinem Herrn Wein zu trinken geben. Der andere aber wird gekreuzigt, und die Vögel werden von seinem Kopf fressen. Entschieden ist die Angelegenheit, über die ihr um Auskunft fragt.»\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdcxxxviii{Und er sagte zu dem von ihnen, mit dessen Rettung er rechnete: «Gedenke meiner bei deinem Herrn.» Aber der Satan ließ ihn vergessen, ihn bei seinem Herrn zu erwähnen. So blieb er noch einige Jahre im Gefängnis.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdcxxxix{Und der König sagte: «Ich sah sieben fette Kühe, die von sieben mageren gefressen wurden, und sieben grüne Ähren und (sieben) andere, die verdorrt waren. O ihr Vornehmen, gebt mir Auskunft über mein Traumgesicht, so ihr das Traumgesicht auslegen könnt.»\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdcxl{Sie sagten: «Wirres Bündel von Träumen. Wir wissen über die Deutung der Träume nicht Bescheid.»\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdcxli{Derjenige von ihnen, der gerettet wurde und sich nach einer Weile erinnerte, sagte: «Ich werde euch seine Deutung kundtun. Schickt mich los.»\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdcxlii{«Josef, du Wahrhaftiger, gib uns Auskunft über sieben fette Kühe, die von sieben mageren gefressen werden, und von sieben grünen Ähren und (sieben) anderen, die verdorrt sind. So mag ich zu den Menschen zurückkehren, auf daß sie Bescheid wissen.»\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdcxliii{Er sagte: «Ihr werdet sieben Jahre wie gewohnt säen. Was ihr aber erntet, das laßt in seinen Ähren, bis auf einen geringen Teil von dem, was ihr verzehrt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdcxliv{Danach werden dann sieben harte (Jahre) kommen, die das verzehren werden, was ihr für sie vorher eingebracht habt, bis auf einen geringen Teil von dem, was ihr aufbewahrt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdcxlv{Danach wird dann ein Jahr kommen, in dem die Menschen Regen haben und in dem sie keltern werden.»\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdcxlvi{Der König sagte: «Bringt ihn zu mir.» Als der Bote zu ihm kam, sagte er: «Kehr zu deinem Herrn zurück und frag ihn, wie es mit den Frauen steht, die sich in ihre Hände geschnitten haben. Mein Herr weiß doch über ihre List Bescheid.»\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdcxlvii{Er sagte: «Was war da mit euch, als ihr versucht habt, Josef zu verführen?» Sie sagten: «Gott bewahre! Wir wissen gegen ihn nichts Böses (anzugeben).» Die Frau des Hochmögenden sagte: «Jetzt ist die Wahrheit offenbar geworden. Ich habe versucht, ihn zu verführen. Und er gehört zu denen, die die Wahrheit sagen.»\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdcxlviii{Josef sagte:) «Dies ist, damit er weiß, daß ich ihn nicht in seiner Abwesenheit verraten habe und daß Gott die List der Verräter nicht gelingen läßt.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdcxlix{Und ich erkläre mich nicht selbst für unschuldig. Die Seele gebietet ja mit Nachdruck das Böse, es sei denn, mein Herr erbarmt sich. Mein Herr ist voller Vergebung und barmherzig.»\qt@no{53}}%
\qt@newcmd\qurantrans@de@mdcl{Und der König sagte: «Bringt ihn zu mir. Ich will ihn ausschließlich für mich haben.» Als er mit ihm gesprochen hatte, sagte er: «Heute bist du bei uns in angesehener Stellung und genießt unser Vertrauen.»\qt@no{54}}%
\qt@newcmd\qurantrans@de@mdcli{Er sagte: «Setze mich über die Vorratskammern des Landes ein. Ich bin ein (guter) Hüter und weiß Bescheid.»\qt@no{55}}%
\qt@newcmd\qurantrans@de@mdclii{So haben Wir dem Josef eine angesehene Stellung im Land gegeben, so daß er darin sich aufhalten konnte, wo er wollte. Wir treffen mit unserer Barmherzigkeit, wen Wir wollen, und Wir lassen den Lohn der Rechtschaffenen nicht verlorengehen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mdcliii{Und wahrlich, der Lohn des Jenseits ist besser für die, die glauben und gottesfürchtig sind.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mdcliv{Und die Brüder Josefs kamen und traten bei ihm ein. Er erkannte sie, während sie ihn für einen Unbekannten hielten.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mdclv{Als er sie nun mit ihrem Bedarf ausgestattet hatte, sagte er: «Bringt mir einen Bruder von euch, (einen) von eurem Vater. Seht ihr nicht, daß ich das Maß voll erstatte und daß ich der beste der Gastgeber bin?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mdclvi{Wenn ihr ihn mir nicht bringt, so bekommt ihr bei mir kein Maß mehr, und ihr sollt nicht in meine Nähe treten.»\qt@no{60}}%
\qt@newcmd\qurantrans@de@mdclvii{Sie sagten: «Wir werden versuchen, seinen Vater in bezug auf ihn zu überreden, und wir werden es bestimmt tun.»\qt@no{61}}%
\qt@newcmd\qurantrans@de@mdclviii{Und er sagte zu seinen Knechten: «Steckt auch ihre Tauschware in ihr Gepäck, daß sie sie (wieder) erkennen, wenn sie zu ihren Angehörigen heimgekehrt sind. Vielleicht werden sie dann auch zurückkommen.»\qt@no{62}}%
\qt@newcmd\qurantrans@de@mdclix{Als sie zu ihrem Vater zurückkamen, sagten sie: «O unser Vater, die (nächste) Zuteilung wurde uns verwehrt. So schick unseren Bruder mit uns, damit wir eine Zuteilung zugemessen bekommen. Und wir werden ihn bestimmt behüten.»\qt@no{63}}%
\qt@newcmd\qurantrans@de@mdclx{Er sagte: «Kann ich ihn euch etwa anders anvertrauen, als ich euch zuvor seinen Bruder anvertraut habe? Gott ist der beste Hüter, und Er ist der Barmherzigste der Barmherzigen.»\qt@no{64}}%
\qt@newcmd\qurantrans@de@mdclxi{Und als sie ihre Sachen öffneten, fanden sie, daß ihre Tauschware ihnen zurückgegeben worden war. Sie sagten: «O unser Vater, was wünschen wir mehr? Das ist unsere Tauschware, sie ist uns zurückgegeben worden. Wir werden Vorrat für unsere Angehörigen bringen, unseren Bruder behüten und die Last eines Kamels mehr zugemessen bekommen. Das ist ein leicht zu erhaltendes Maß.»\qt@no{65}}%
\qt@newcmd\qurantrans@de@mdclxii{Er sagte: «Ich werde ihn nicht mit euch schicken, bis ihr mir ein verbindliches Versprechen vor Gott gebt, daß ihr ihn mir zurückbringt, es sei denn, ihr werdet umringt.» Als sie ihm ihr verbindliches Versprechen gegeben hatten, sagte er: «Gott ist Sachwalter über das, was wir (hier) sagen.»\qt@no{66}}%
\qt@newcmd\qurantrans@de@mdclxiii{Und er sagte: «O meine Söhne, geht nicht durch ein einziges Tor hinein. Geht durch verschiedene Tore hinein. Ich kann euch vor Gott nichts nützen. Das Urteil gehört Gott allein. Auf Ihn vertraue ich. Auf Ihn sollen die vertrauen, die (überhaupt auf jemanden) vertrauen.»\qt@no{67}}%
\qt@newcmd\qurantrans@de@mdclxiv{Als sie hineingingen, wie ihr Vater ihnen befohlen hatte, hat es ihnen vor Gott nichts genützt. Es war nur ein Bedürfnis in der Seele Jakobs, das er (damit) erfüllte. Und er besaß Wissen, weil Wir es ihn gelehrt hatten. Aber die meisten Menschen wissen nicht Bescheid.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mdclxv{Als sie bei Josef eintraten, zog er seinen Bruder zu sich. Er sagte: «Ich, ich bin dein Bruder. So sei nicht betrübt wegen dessen, was sie taten.»\qt@no{69}}%
\qt@newcmd\qurantrans@de@mdclxvi{Als er sie nun mit ihrem Bedarf ausgestattet hatte, tat er das Trinkgefäß in das Gepäck seines Bruders. Dann rief ein Rufer aus: «Ihr da von der Karawane, ihr seid ja Diebe.»\qt@no{70}}%
\qt@newcmd\qurantrans@de@mdclxvii{Sie sagten, während sie auf sie zugingen: «Was vermißt ihr?»\qt@no{71}}%
\qt@newcmd\qurantrans@de@mdclxviii{Sie sagten: «Wir vermissen den Pokal des Königs. Wer ihn zurückbringt, erhält die Last eines Kamels, und dafür bin ich Bürge.»\qt@no{72}}%
\qt@newcmd\qurantrans@de@mdclxix{Sie sagten: «Bei Gott, ihr wißt es, wir sind nicht gekommen, um im Land Unheil zu stiften, und wir sind keine Diebe.»\qt@no{73}}%
\qt@newcmd\qurantrans@de@mdclxx{Sie sagten: «Was ist die Vergeltung dafür, wenn ihr lügt?»\qt@no{74}}%
\qt@newcmd\qurantrans@de@mdclxxi{Sie sagten: «Die Vergeltung dafür ist, daß der, in dessen Gepäck er gefunden wird, selbst als Entgelt dafür dienen soll. So vergelten wir denen, die Unrecht tun.»\qt@no{75}}%
\qt@newcmd\qurantrans@de@mdclxxii{Er begann (zu suchen) in ihren Behältern vor dem Behälter seines Bruders. Dann holte er ihn aus dem Behälter seines Bruders. So führten Wir für Josef eine List aus. Nach der Religion des Königs hätte er unmöglich seinen Bruder (als Sklaven) nehmen können, es sei denn, daß es Gott wollte. Wir erhöhen, wen Wir wollen, um Rangstufen. Und über jedem, der Wissen besitzt, steht einer, der (noch mehr) weiß.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mdclxxiii{Sie sagten: «Wenn er stiehlt, so hat auch ein Bruder von ihm zuvor gestohlen.» Josef hielt es in seinem Inneren geheim und zeigte es ihnen nicht offen. Er sagte: «Ihr seid noch schlimmer daran. Und Gott weiß besser, was ihr beschreibt.»\qt@no{77}}%
\qt@newcmd\qurantrans@de@mdclxxiv{Sie sagten: «O Hochmögender, er hat einen Vater, einen hochbetagten Greis. So nimm einen von uns an seiner Stelle. Wir sehen, daß du einer der Rechtschaffenen bist.»\qt@no{78}}%
\qt@newcmd\qurantrans@de@mdclxxv{Er sagte: «Gott behüte, daß wir einen anderen nehmen als den, bei dem wir unsere Sachen gefunden haben! Sonst würden wir zu denen gehören, die Unrecht tun.»\qt@no{79}}%
\qt@newcmd\qurantrans@de@mdclxxvi{Als sie an ihm jede Hoffnung verloren hatten, gingen sie zu einem vertraulichen Gespräch unter sich. Der Älteste von ihnen sagte: «Wißt ihr nicht, daß euer Vater von euch ein verbindliches Versprechen vor Gott entgegengenommen hat, und daß ihr zuvor eure Pflicht in bezug auf Josef nicht erfüllt habt? Ich werde das Land nicht verlassen, bis mein Vater es mir erlaubt oder Gott ein Urteil für mich fällt, und Er ist der Beste derer, die Urteile fällen.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mdclxxvii{Kehrt zu eurem Vater zurück und sagt: >O unser Vater, dein Sohn hat gestohlen, und wir bezeugen nur das, was wir wissen, und wir können nicht Hüter sein über das, was verborgen ist.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mdclxxviii{Und frag die Stadt, in der wir waren, und die Karawane, mit der wir gekommen sind. Wir sagen ja die Wahrheit.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mdclxxix{Er sagte: «Eure Seele hat euch etwas eingeredet. (Es gilt) schöne Geduld (zu üben). Möge Gott sie mir alle zurückbringen! Er ist der, der alles weiß und weise ist.»\qt@no{83}}%
\qt@newcmd\qurantrans@de@mdclxxx{Und er kehrte sich von ihnen ab und sagte: «O wie voller Gram bin ich um Josef!» Und seine Augen wurden weiß vor Trauer, und er unterdrückte (seinen Groll).\qt@no{84}}%
\qt@newcmd\qurantrans@de@mdclxxxi{Sie sagten: «Bei Gott, du hörst nicht auf, des Josef zu gedenken, bis du bald zusammenbrichst oder zu denen gehörst, die zugrunde gehen.»\qt@no{85}}%
\qt@newcmd\qurantrans@de@mdclxxxii{Er sagte: «Ich klage ja meinen Kummer und meine Trauer Gott allein, und ich weiß von Gott, was ihr nicht wißt.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mdclxxxiii{O meine Söhne, geht und erkundigt euch über Josef und seinen Bruder. Und verliert nicht die Hoffnung, daß Gott Aufatmen verschafft. Die Hoffnung, daß Gott Aufatmen verschafft, verlieren nur die ungläubigen Leute.»\qt@no{87}}%
\qt@newcmd\qurantrans@de@mdclxxxiv{Als sie (wieder) bei ihm eintraten, sagten sie: «O Hochmögender, Not hat uns und unsere Angehörigen erfaßt. Und wir haben (nur) eine zusammengewürfelte Ware gebracht. So erstatte uns (dennoch) volles Maß und gib es uns als Almosen. Gott vergilt denen, die Almosen geben.»\qt@no{88}}%
\qt@newcmd\qurantrans@de@mdclxxxv{Er sagte: «Wißt ihr (noch), was ihr Josef und seinem Bruder angetan habt, als ihr töricht gehandelt habt?»\qt@no{89}}%
\qt@newcmd\qurantrans@de@mdclxxxvi{Sie sagten: «Bist du denn wirklich Josef?» Er sagte: «Ich bin Josef, und das ist mein Bruder. Gott hat uns eine Wohltat erwiesen. Wahrlich, wenn einer gottesfürchtig und geduldig ist, so läßt Gott den Lohn der Rechtschaffenen nicht verlorengehen.»\qt@no{90}}%
\qt@newcmd\qurantrans@de@mdclxxxvii{Sie sagten: «Bei Gott, Gott hat dich vor uns bevorzugt. Und wir haben bestimmt gesündigt.»\qt@no{91}}%
\qt@newcmd\qurantrans@de@mdclxxxviii{Er sagte: «Keine Schelte soll heute über euch kommen. Gott vergibt euch, Er ist ja der Barmherzigste der Barmherzigen.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mdclxxxix{Nehmt dieses mein Hemd mit und legt es auf das Gesicht meines Vaters, dann wird er wieder sehen können. Und bringt alle eure Angehörigen zu mir.»\qt@no{93}}%
\qt@newcmd\qurantrans@de@mdcxc{Als nun die Karawane aufbrach, sagte ihr Vater: «Wahrlich, ich spüre Josefs Geruch. Wenn ihr nur nicht (meine Worte) als dummes Gerede zurückweisen würdet!»\qt@no{94}}%
\qt@newcmd\qurantrans@de@mdcxci{Sie sagten: «Bei Gott, du befindest dich in deinem alten Irrtum.»\qt@no{95}}%
\qt@newcmd\qurantrans@de@mdcxcii{Als nun der Freudenbote kam, legte er es auf sein Gesicht, und er konnte wieder sehen. Er sagte: «Habe ich euch nicht gesagt, daß ich von Gott weiß, was ihr nicht wißt?»\qt@no{96}}%
\qt@newcmd\qurantrans@de@mdcxciii{Sie sagten: «O unser Vater, bitte für uns um Vergebung unserer Sünden. Wir haben ja gesündigt.»\qt@no{97}}%
\qt@newcmd\qurantrans@de@mdcxciv{Er sagte: «Ich werde meinen Herrn um Vergebung für euch bitten. Er ist der, der voller Vergebung und barmherzig ist.»\qt@no{98}}%
\qt@newcmd\qurantrans@de@mdcxcv{Als sie nun bei Josef eintraten, zog er seine Eltern an sich und sagte: «Betretet Ägypten, so Gott will, in Sicherheit.»\qt@no{99}}%
\qt@newcmd\qurantrans@de@mdcxcvi{Und er erhob seine Eltern auf den Thron. Und sie warfen sich vor ihm nieder. Er sagte: «O mein Vater, das ist die Deutung meines Traumgesichts von früher. Mein Herr hat es wahr gemacht. Und Er hat mir Gutes erwiesen, als Er mich aus dem Gefängnis herauskommen ließ und euch aus der Steppe hierher brachte, nachdem der Satan zwischen mir und meinen Brüdern (zu Zwietracht) aufgestachelt hatte. Mein Herr weiß zu erreichen, was Er will. Er ist der, der alles weiß und weise ist.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mdcxcvii{Mein Herr, du hast mir etwas von der Königsherrschaft zukommen lassen und mich etwas von der Deutung der Geschichten gelehrt. Du Schöpfer der Himmel und der Erde, du bist mein Freund im Diesseits und Jenseits. Berufe mich als gottergeben ab und stelle mich zu den Rechtschaffenen.»\qt@no{101}}%
\qt@newcmd\qurantrans@de@mdcxcviii{Dies gehört zu den Berichten über das Unsichtbare, das Wir dir offenbaren. Du warst nicht bei ihnen, als sie sich verbanden und Ränke schmiedeten.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mdcxcix{Und die meisten Menschen sind nicht gläubig, du magst dich noch so sehr bemühen.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mdcc{Und du verlangst von ihnen keinen Lohn dafür. Es ist nur eine Ermahnung für die Weltenbewohner.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mdcci{Wie viele Zeichen gibt es in den Himmeln und auf der Erde, an denen sie vorbeigehen, indem sie sich von ihnen abwenden?\qt@no{105}}%
\qt@newcmd\qurantrans@de@mdccii{Und die meisten von ihnen glauben nicht an Gott, ohne (Ihm andere) beizugesellen.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mdcciii{Wähnen sie sich denn in Sicherheit davor, daß eine überdeckende (Strafe) von der Pein Gottes über sie kommt, oder daß plötzlich die Stunde über sie kommt, ohne daß sie es merken?\qt@no{107}}%
\qt@newcmd\qurantrans@de@mdcciv{Sprich: Das ist mein Weg. Ich rufe zu Gott aufgrund eines einsichtbringenden Beweises, ich und diejenigen, die mir folgen. Preis sei Gott! Und ich gehöre nicht zu den Polytheisten.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mdccv{Und Wir haben vor dir von den Bewohnern der Städte nur Männer gesandt, denen Wir Offenbarungen eingegeben haben. Sind sie denn nicht auf der Erde umhergegangen und haben geschaut, wie das Ende derer war, die vor ihnen lebten? Wahrlich, die Wohnstätte des Jenseits ist besser für die, die gottesfürchtig sind. Habt ihr denn keinen Verstand?\qt@no{109}}%
\qt@newcmd\qurantrans@de@mdccvi{Als dann die Gesandten die Hoffnung verloren hatten und sie meinten, sie seien belogen worden, kam unsere Unterstützung zu ihnen. Und so wird errettet, wen Wir wollen. Und niemand kann unsere Schlagkraft von den Leuten zurückhalten, die Übeltäter sind.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mdccvii{Wahrlich, in der Erzählung über sie ist eine Lehre für die Einsichtigen. Es ist keine Geschichte, die erdichtet wird, sondern die Bestätigung dessen, was vor ihm vorhanden war, und eine ins einzelne gehende Darlegung aller Dinge, und eine Rechtleitung und Barmherzigkeit für Leute, die glauben..\qt@no{111}}%
\qt@newcmd\qurantrans@de@mdccviii{\basmalahde Alif Lam Mim Ra. Dies sind die Zeichen des Buches. Und was zu dir von deinem Herrn herabgesandt worden ist, ist die Wahrheit. Aber die meisten Menschen glauben nicht.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdccix{Gott ist es, der die Himmel ohne Stützen, die ihr sehen könntet, emporgehoben und sich dann auf dem Thron zurechtgesetzt hat. Er hat die Sonne und den Mond dienstbar gemacht - jedes läuft auf eine festgesetzte Frist. Er regelt die Angelegenheit. Er legt die Zeichen im einzelnen dar, auf daß ihr über die Begegnung mit eurem Herrn Gewißheit heget.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdccx{Und Er ist es, der die Erde ausgebreitet und auf ihr festgegründete Berge und Flüsse gemacht hat. Und von allen Früchten hat Er auf ihr ein Paar gemacht. Er läßt die Nacht den Tag überdecken. Darin sind Zeichen für Leute, die nachdenken.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdccxi{Und auf der Erde sind nebeneinanderliegende Landstrecken und Gärten mit Weinstöcken, und Getreide und Palmen mit mehreren und mit einzelnen Stämmen aus der einen Wurzel, welche (alle) mit ein und demselben Wasser bewässert werden. Wir lassen die einen von ihnen die anderen im Ernteertrag übertreffen. Darin sind Zeichen für Leute, die Verstand haben.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdccxii{Und wenn du dich schon verwunderst, so ist ihre Rede verwunderlich: «Sollen wir, wenn wir zu Staub geworden sind, wirklich neu erschaffen werden?» Das sind die, die ihren Herrn verleugnen; das sind die, die Fesseln an ihrem Hals haben; das sind die Gefährten des Feuers, darin werden sie ewig weilen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdccxiii{Sie wünschen von dir, daß ihnen das Schlechte vor dem Guten beschleunigt wird, obwohl doch vor ihnen beispielhafte Strafen verhängt wurden. Und dein Herr hält den Menschen, auch wenn sie Unrecht tun, Vergebung bereit. Und dein Herr verhängt eine harte Strafe.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdccxiv{Und diejenigen, die ungläubig sind, sagen: «Wenn doch ein Zeichen von seinem Herrn auf ihn herabgesandt würde!» Du bist aber nur ein Warner. Und jedes Volk hat einen, der es rechtleitet.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdccxv{Gott weiß, was jedes Weib trägt und um wieviel der Mutterschoß die Tragzeit verkürzt und um wieviel er sie verlängert. Und jedes Ding hat bei Ihm ein Maß.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdccxvi{Er ist der, der über das Unsichtbare und das Offenbare Bescheid weiß, der Große, der Erhabene.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdccxvii{Es ist gleich, ob einer von euch seine Worte im geheimen spricht oder laut äußert, und ob einer sich in der Nacht versteckt oder am Tag hinausgeht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdccxviii{Vor sich und hinter sich hat er Begleiter, die ihn auf Gottes Befehl behüten. Gott verändert nicht den Zustand eines Volkes, bis sie selbst ihren eigenen Zustand verändern. Und wenn Gott einem Volk Böses will, so kann es nicht zurückgewiesen werden. Und sie haben außer Ihm keinen Schutzherrn.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdccxix{Er ist es, der euch den Blitz als Grund zur Angst und zum Begehren sehen und die schweren Wolken entstehen läßt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdccxx{Und der Donner singt sein Lob, und auch die Engel aus Furcht vor Ihm. Und Er schickt die Donnerschläge und trifft damit, wen Er will. Aber (nein), sie streiten über Gott, und Er kann (dem Gegner) im Streit stark zusetzen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdccxxi{Ihm gebührt die wahre Anrufung. Diejenigen, die sie an seiner Stelle anrufen, erhören sie in keinem Anliegen. Es ist nur wie mit einem, der seine Hände nach Wasser ausstreckt, damit es seinen Mund erreicht, aber es erreicht ihn nicht. Und das Rufen der Ungläubigen geht gewiß in die Irre.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdccxxii{Und vor Gott wirft sich, wer in den Himmeln und auf der Erde ist, nieder, ob freiwillig oder widerwillig, und auch ihre Schatten, am Morgen und am Abend.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdccxxiii{Sprich: Wer ist der Herr der Himmel und der Erde? Sprich: Gott. Sprich: Nehmt ihr euch denn außer Ihm Freunde, die sich selbst weder Nutzen noch Schaden bringen können? Sprich: Sind etwa der Blinde und der Sehende gleich? Oder sind etwa die Finsternisse und das Licht gleich? Oder haben sie Gott solche Teilhaber (zur Seite) gestellt, die erschaffen haben, wie Er erschaffen hat, so daß ihnen die Schöpfung gleichartig erscheint? Sprich: Gott ist der Schöpfer aller Dinge, und Er ist der Eine, der bezwingende Macht besitzt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdccxxiv{Er sendet vom Himmel Wasser herab, und da fließen Täler nach ihrem Maß, und die Flut trägt Schaum an der Oberfläche. Ein ähnlicher Schaum tritt aus dem aus, worüber man das Feuer brennen läßt, um Schmuck oder Gerät anzufertigen. So führt Gott (im Gleichnis) das Wahre und das Falsche an. Was den Schaum betrifft, so vergeht er nutzlos. Was aber den Menschen nützt, bleibt in der Erde. So führt Gott die Gleichnisse an.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdccxxv{Diejenigen, die auf ihren Herrn hören, erhalten das Beste. Diejenigen, die nicht auf ihn hören, würden wohl, wenn sie alles, was auf der Erde ist, besäßen und noch einmal soviel dazu, sich damit loskaufen wollen. Sie erwartet eine böse Abrechnung, und ihre Heimstätte ist die Hölle - welch schlimme Lagerstätte!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdccxxvi{Ist etwa derjenige, der weiß, daß das, was zu dir von deinem Herrn herabgesandt worden ist, die Wahrheit ist, wie der, der blind ist? Jedoch bedenken es nur die Einsichtigen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdccxxvii{Diejenigen, die den Bund Gottes halten und die Verpflichtung nicht brechen,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdccxxviii{Und die verbinden, was Gott zu verbinden befohlen hat, ihren Herrn fürchten und Angst vor einer bösen Abrechnung haben,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdccxxix{Und die geduldig sind in der Suche nach dem Antlitz ihres Herrn, das Gebet verrichten und von dem, was Wir ihnen beschert haben, geheim und offen spenden, und das Böse mit dem Guten abwehren, diese werden die jenseitige Wohnstätte erhalten,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdccxxx{Die Gärten von Eden, in die sie eingehen werden, sie und diejenigen von ihren Vätern, ihren Gattinnen und ihrer Nachkommenschaft, die Gutes getan haben. Und die Engel treten zu ihnen ein durch alle Tore:\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdccxxxi{«Friede sei über euch dafür, daß ihr geduldig wäret!» Welch vorzügliche jenseitige Wohnstätte!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdccxxxii{Diejenigen, die den Bund Gottes, nachdem er geschlossen worden ist, brechen, und das zerschneiden, was Gott befohlen hat zu verbinden, und auf der Erde Unheil stiften, die werden den Fluch Gottes und eine schlimme Wohnstätte erhalten.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdccxxxiii{Gott teilt den Lebensunterhalt großzügig, wem Er will, und auch bemessen zu. Und sie freuen sich über das diesseitige Leben; das diesseitige Leben ist aber im Vergleich mit dem Jenseits nur Nutznießung.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdccxxxiv{Und diejenigen, die ungläubig sind, sagen: «Wenn doch ein Zeichen von seinem Herrn auf ihn herabgesandt würde!» Sprich: Gott führt irre, wen Er will, und leitet zu sich, wer sich Ihm reumütig zuwendet.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdccxxxv{Diejenigen, die glauben und deren Herzen im Gedenken Gottes Ruhe finden - ja, im Gedenken Gottes finden die Herzen Ruhe -,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdccxxxvi{Diejenigen, die glauben und die guten Werke tun - selig sind sie, und sie werden eine schöne Heimstatt erhalten.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdccxxxvii{So haben Wir dich in eine Gemeinschaft gesandt, vor der Gemeinschaften dahingegangen sind, damit du ihnen verliest, was Wir dir offenbart haben. Aber sie verleugnen den Erbarmer. Sprich: Er ist mein Herr. Es gibt keinen Gott außer Ihm. Auf Ihn vertraue ich, und Ihm wende ich mich zu.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdccxxxviii{Auch wenn ein Koran käme, mit dem die Berge versetzt oder die Erde zerstückelt oder zu den Toten gesprochen werden könnte... Nein, bei der ganzen Angelegenheit hat Gott allein zu entscheiden. Wissen denn diejenigen, die glauben, nicht, daß Gott, wenn Er wollte, die Menschen alle rechtleiten würde? Diejenigen, die ungläubig sind, wird immer eine Katastrophe treffen für das, was sie gemacht haben, oder sie wird in der Nähe ihrer Wohnstätten niedergehen, bis das Versprechen Gottes eintrifft. Und Gott bricht das Versprechen nicht.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdccxxxix{Gespottet wurde schon vor dir über Gesandte. Da gewährte Ich denen, die ungläubig waren, Aufschub, dann ergriff Ich sie. Und wie war dann meine Strafe!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdccxl{Ist denn der, der über jede Seele Macht ausübt, um ihr zu vergelten für das, was sie getan hat, (den Götzen gleich)? Sie stellen Gott Teilhaber (zur Seite). Sprich: Nennt sie. Oder wollt ihr Ihm etwas kundtun, das Er auf der Erde nicht wüßte, oder etwas, das eine offenkundige Rede ist? Nein, denen, die ungläubig sind, sind ihre Ränke verlockend gemacht worden, und sie sind vom Weg abgewiesen worden. Wen Gott irreführt, der hat niemanden, der ihn rechtleiten könnte.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdccxli{Für sie ist bestimmt eine Pein im diesseitigen Leben. Die Pein des Jenseits ist jedoch härter. Und sie haben niemanden, der sie vor Gott schützen könnte.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdccxlii{Mit dem Paradies, das den Gottesfürchtigen versprochen ist, ist es wie folgt: Unter ihm fließen Bäche, und es hat ständigen Ernteertrag und Schatten. Das ist das, was im Jenseits für die Gottesfürchtigen folgt. Und was im Jenseits für die Ungläubigen folgt, ist das Feuer.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdccxliii{Diejenigen, denen Wir das Buch zukommen ließen, freuen sich über das, was zu dir herabgesandt worden ist. Unter den Parteien gibt es welche, die einen Teil davon verwerfen. Sprich: Mir ist befohlen worden, Gott zu dienen und Ihm nicht (andere) beizugesellen. Zu Ihm rufe ich, und zu Ihm ist meine Rückkehr.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdccxliv{Und so haben Wir ihn als eine Urteilsnorm in arabischer Sprache hinabgesandt. Wenn du ihren Neigungen folgst nach dem, was dir an Wissen zugekommen ist, so hast du vor Gott weder Freund noch Beschützer.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdccxlv{Und Wir haben vor dir Gesandte entsandt, und Wir haben ihnen Gattinnen und Nachkommenschaft gegeben. Kein Gesandter kann ein Zeichen bringen außer mit der Erlaubnis Gottes. Jede Frist steht fest in einem Buch.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdccxlvi{Gott löscht aus, und Er bestätigt, was Er will. Bei Ihm steht die Urnorm des Buches.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdccxlvii{Ob Wir dich einen Teil dessen, was Wir ihnen androhen, sehen lassen oder dich abberufen, dir obliegt nur die Ausrichtung (der Botschaft). Und Uns obliegt die Abrechnung.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdccxlviii{Haben sie denn nicht gesehen, daß Wir über das Land kommen und es an seinen Enden kürzen? Gott (allein) urteilt, und niemand kann danach sein Urteil rückgängig machen. Und Er ist schnell im Abrechnen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdccxlix{Ränke haben schon diejenigen, die vor ihnen lebten, geschmiedet. Aber Gott schmiedet die (wirksamen) Ränke alle. Er weiß, was jede Seele erwirbt. Und die Ungläubigen werden zu wissen bekommen, für wen die jenseitige Wohnstätte bestimmt ist.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdccl{Diejenigen, die ungläubig sind, sagen: «Du bist kein Gesandter.» Sprich: Gott genügt als Zeuge zwischen mir und euch, und auch diejenigen, die das Wissen des Buches besitzen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdccli{\basmalahde Alif Lam, Ra. Dies ist ein Buch, das Wir zu dir hinabgesandt haben, damit du die Menschen mit der Erlaubnis ihres Herrn aus den Finsternissen ins Licht hinausführst, zum Weg dessen, der mächtig und des Lobes würdig ist,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdcclii{zum Weg) Gottes, dem gehört, was in den Himmeln und was auf der Erde ist. Wehe den Ungläubigen vor einer harten Pein!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdccliii{Diejenigen, die das diesseitige Leben mehr lieben als das Jenseits und vom Weg Gottes abweisen und sich ihn krumm wünschen, die sind weit abgeirrt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdccliv{Und Wir haben keinen Gesandten entsandt, außer (mit einer Botschaft) in der Sprache seines Volkes, damit er (sie) ihnen deutlich macht. Gott führt dann irre, wen Er will, und Er leitet recht, wen Er will. Und Er ist der Mächtige, der Weise.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdcclv{Und Wir haben Mose mit unseren Zeichen gesandt: «Führe dein Volk aus den Finsternissen ins Licht hinaus, und erinnere sie an die Tage Gottes.» Darin sind Zeichen für jeden, der sehr standhaft und sehr dankbar ist.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdcclvi{Und als Mose zu seinem Volk sagte: «Gedenket der Gnade Gottes zu euch, als Er euch vor den Leuten des Pharao rettete, die euch schlimme Pein zufügten, eure Söhne abschlachteten und nur eure Frauen am Leben ließen - darin war für euch eine gewaltige Prüfung von eurem Herrn.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdcclvii{Und als euer Herr ankündigte: >Wenn ihr dankbar seid, werde Ich euch noch mehr Gnade erweisen. Und wenn ihr undankbar seid, so ist meine Pein hart. >»\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdcclviii{Und Mose sagte: «Wenn ihr undankbar seid, ihr und alle, die auf der Erde sind, so ist Gott auf niemanden angewiesen und des Lobes würdig.»\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdcclix{Ist denn nicht der Bericht über die, die vor euch lebten, zu euch gelangt, das Volk Noachs, die ``Aad und Thamud und die, die nach ihnen lebten, und die nur Gott kennt? Ihre Gesandten kamen zu ihnen mit den deutlichen Zeichen. Sie aber steckten ihre Hände in den Mund und sagten: «Wir verleugnen das, womit ihr gesandt seid, und wir hegen über das, wozu ihr uns aufruft, einen starken Zweifel.»\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdcclx{Ihre Gesandten sagten: «Ist denn ein Zweifel möglich über Gott, den Schöpfer der Himmel und der Erde? Er ruft euch, um euch etwas von euren Sünden zu vergeben und euch für eine bestimmte Frist zurückzustellen.» Sie sagten: «Ihr seid nur Menschen wie wir. Ihr wollt uns von dem abbringen, was unsere Väter verehrten. So bringt uns eine offenkundige Ermächtigung.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdcclxi{Ihre Gesandten sagten zu ihnen: «Wir sind zwar nur Menschen wie ihr. Aber Gott erweist seine Wohltaten, wem von seinen Dienern Er will. Und wir können euch keine Ermächtigung bringen außer mit der Erlaubnis Gottes. Auf Gott sollen die Gläubigen vertrauen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdcclxii{Warum sollten wir nicht auf Gott vertrauen, wo Er uns unsere Wege geführt hat? Wir werden das Leid, das ihr uns zufügt, geduldig ertragen, und auf Gott sollen die vertrauen, die (überhaupt auf jemanden) vertrauen.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdcclxiii{Diejenigen, die ungläubig waren, sagten zu ihren Gesandten: «Wir werden euch bestimmt aus unserem Land vertreiben, oder ihr kehrt zu unserer Glaubensrichtung zurück.» Da offenbarte ihnen ihr Herr: «Verderben werden Wir die, die Unrecht tun.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdcclxiv{Und Wir werden euch nach ihnen das Land bewohnen lassen. Dies gilt für den, der meinen Stand fürchtet und meine Androhung fürchtet.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdcclxv{Und sie baten um einen Richterspruch. Und enttäuscht wurde jeder widerspenstige Gewaltherrscher.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdcclxvi{Hintendrein steht für ihn die Hölle bereit, und er bekommt eitriges Wasser zu trinken,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdcclxvii{Das er schluckt, aber kaum hinunterbringt. Und der Tod kommt zu ihm von überall her, aber er stirbt nicht. Und danach steht eine schwere Pein.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdcclxviii{Mit denen, die ihren Herrn verleugnen, ist es so: Ihre Werke sind wie die Asche, auf die der Wind an einem stürmischen Tag heftig bläst. Sie verfügen über nichts von dem, was sie erworben haben. Das ist der tiefe Irrtum.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdcclxix{Siehst du denn nicht, daß Gott die Himmel und die Erde in Wahrheit erschaffen hat? Wenn Er will, läßt Er euch fortgehen und bringt eine neue Schöpfung hervor.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdcclxx{Dies fällt Gott sicher nicht schwer.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdcclxxi{Und sie treten vor Gott allesamt. Da sagen die Schwachen zu denen, die sich hochmütig verhielten: «Wir waren doch eure Gefolgsleute. Könnt ihr uns vor Gottes Pein etwas nützen?» Sie sagen: «Wenn Gott uns rechtgeleitet hätte, hätten (auch) wir euch rechtgeleitet. Es ist für uns gleich, ob wir uns mutlos oder geduldig zeigen; für uns gibt es kein Entrinnen.»\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdcclxxii{Und der Satan sagt, nachdem die Angelegenheit entschieden ist: «Gott hat euch ein wahres Versprechen gegeben. Auch ich habe euch (etwas versprochen), es aber dann nicht gehalten. Und ich hatte keine Macht über euch; ich habe euch nur gerufen, und ihr habt auf mich gehört. So tadelt mich nicht, tadelt euch selbst. Ich kann euch nicht helfen, und ihr könnt mir nicht helfen. Ich weise es zurück, daß ihr mich zuvor (Gott) beigesellt habt.» Für die, die Unrecht tun, ist eine schmerzhafte Pein bestimmt.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdcclxxiii{Aber diejenigen, die glauben und die guten Werke tun, werden in Gärten geführt, unter denen Bäche fließen; darin werden sie ewig weilen, mit der Erlaubnis ihres Herrn. Ihre Begrüßung darin wird sein: «Frieden!»\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdcclxxiv{Hast du nicht gesehen, wie Gott ein Gleichnis von einem gefälligen Wort anführt? Es ist wie ein guter Baum, dessen Stamm fest ist und dessen Zweige in den Himmel reichen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdcclxxv{Er bringt seinen Ernteertrag zu jeder Zeit, mit der Erlaubnis seines Herrn. Und Gott führt für die Menschen Gleichnisse an, auf daß sie es bedenken.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdcclxxvi{Mit einem schlechten Wort ist es wie mit einem schlechten Baum, der über der Erde herausgerissen wurde und keinen festen Grund mehr hat.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdcclxxvii{Gott festigt diejenigen, die glauben, durch die feste Aussage im diesseitigen Leben und im Jenseits. Und Gott führt die, die Unrecht tun, in die Irre. Gott tut, was Er will.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdcclxxviii{Hast du nicht auf jene geschaut, die die Gnade Gottes gegen den Unglauben eingetauscht und ihr Volk in die Wohnstätte des Verderbens versetzt haben,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdcclxxix{In die Hölle, in der sie brennen? - Welch schlimmer Aufenthalt!\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdcclxxx{Und sie haben Gott andere als Gegenpart zur Seite gestellt, um (die Menschen) von seinem Weg abirren zu lassen. Sprich: Genießet nur, ihr treibt ja ins Feuer.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdcclxxxi{Sag zu meinen Dienern, die glauben, sie sollen das Gebet verrichten und von dem, was Wir ihnen beschert haben, geheim und offen spenden, bevor ein Tag kommt, an dem es weder Handel noch Freundschaft gibt.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdcclxxxii{Gott ist es, der die Himmel und die Erde erschuf und vom Himmel Wasser herabkommen ließ und dadurch von den Früchten einen Lebensunterhalt für euch hervorbrachte. Und Er stellte in euren Dienst die Schiffe, damit sie auf dem Meer auf seinen Befehl fahren. Und Er stellte in euren Dienst die Flüsse.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdcclxxxiii{Er stellte in euren Dienst die Sonne und den Mond in unablässigem Lauf. Und Er stellte in euren Dienst die Nacht und den Tag.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdcclxxxiv{Und Er ließ euch etwas zukommen von allem, worum ihr batet. Wenn ihr die Gnade Gottes aufzählen wolltet, könntet ihr sie nicht erfassen. Wahrlich, der Mensch neigt sehr zum Unrecht und ist sehr undankbar.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdcclxxxv{Als Abraham sagte: «Mein Herr, mache dieses Gebiet sicher, und laß mich und meine Söhne es meiden, den Götzen zu dienen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdcclxxxvi{Mein Herr, sie haben viele Menschen irregeführt. Wer mir nun folgt, gehört zu mir, und wenn einer gegen mich ungehorsam ist, so bist Du voller Vergebung und barmherzig.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdcclxxxvii{Unser Herr, ich habe einige aus meiner Nachkommenschaft in einem Tal ohne Saat bei deinem heiligen Haus wohnen lassen, unser Herr, damit sie das Gebet verrichten. So laß die Herzen einiger Menschen sich ihnen zuneigen und beschere ihnen etwas von den Früchten, auf daß sie dankbar seien.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdcclxxxviii{Unser Herr, Du weißt, was wir verbergen und was wir offenlegen, und vor Gott ist nichts verborgen, weder auf der Erde noch im Himmel.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdcclxxxix{Lob sei Gott, der mir trotz meines Alters Ismael und Isaak geschenkt hat! Mein Herr erhört das Rufen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdccxc{Mein Herr, laß mich und die aus meiner Nachkommenschaft das Gebet verrichten, unser Herr, und nimm unser Rufen an.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdccxci{Unser Herr, vergib mir und meinen Eltern und den Gläubigen am Tag, da die Abrechnung heraufkommen wird.»\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdccxcii{Und du darfst nicht meinen, daß Gott das, was die Ungerechten tun, unbeachtet läßt. Er stellt sie nur zurück bis zu einem Tag, an dem die Blicke starr werden,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdccxciii{Mit gerecktem Hals und erhobenem Haupt; ihr Blick kehrt nicht zu ihnen zurück, und ihre Herzen sind leer.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdccxciv{Und warne die Menschen vor dem Tag, an dem die Pein über sie kommt. Da werden diejenigen, die Unrecht taten, sagen: «Unser Herr, stelle uns auf eine kurze Frist zurück, so werden wir auf deinen Ruf hören und den Gesandten folgen.» - «Hattet ihr denn nicht vorher geschworen, ihr würdet nicht vergehen?\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdccxcv{Ihr habt noch in den Wohnungen derer gewohnt, die sich selbst Unrecht getan haben, und es ist euch deutlich geworden, wie Wir an ihnen gehandelt haben. Wir haben euch doch Beispiele gegeben.»\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdccxcvi{Und sie haben ihre Ränke geschmiedet, aber über ihre Ränke hat Gott allein zu entscheiden, auch wenn ihre Ränke derart sind, daß davor die Berge vergehen.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdccxcvii{So darfst du nicht meinen, daß Gott sein Versprechen an die Gesandten bricht. Gott ist mächtig und übt Rache.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdccxcviii{Am Tag, da die Erde gegen eine andere Erde eingetauscht wird, und auch die Himmel, und da sie vor Gott treten, den Einen, der bezwingende Macht besitzt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdccxcix{An jenem Tag siehst du die Übeltäter in Ketten aneinandergebunden.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdccc{Ihre Kleider sind aus Pech, und das Feuer überdeckt ihre Gesichter,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdccci{Damit Gott einem jeden vergelte, was er erworben hat. Gott ist schnell im Abrechnen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdcccii{Das ist eine Botschaft an die Menschen, damit sie dadurch gewarnt werden und damit sie wissen, daß Er ein einziger Gott ist, und damit es die Einsichtigen bedenken.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdccciii{\basmalahde Auf Lam Ra. Dies sind die Zeichen des Buches und eines deutlichen Korans.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdccciv{Vielleicht werden diejenigen, die ungläubig sind, wünschen, sie wären Muslime gewesen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdcccv{Laß sie nur essen und genießen und sich durch die Hoffnung ablenken lassen. Sie werden es noch zu wissen bekommen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdcccvi{Und Wir haben keine Stadt verderben lassen, ohne daß sie eine festgelegte Vorherbestimmung gehabt hätte.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdcccvii{Keine Gemeinschaft kann ihrer Frist vorausgehen, noch kann sie hinter ihr zurückbleiben.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdcccviii{Und sie sagen: «O du, auf den die Ermahnung herabgesandt worden sein soll, du bist ja besessen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdcccix{Würdest du uns doch die Engel bringen, so du zu denen gehörst, die die Wahrheit sagen!»\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdcccx{Wir senden die Engel nur mit der Wahrheit hinab. Dann wird ihnen kein Aufschub gewährt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdcccxi{Wir, ja Wir haben die Ermahnung hinabgesandt, und Wir werden sie gewiß bewahren.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdcccxii{Wir haben vor dir zu den Parteien der Früheren entsandt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdcccxiii{Und kein Gesandter kam zu ihnen, ohne daß sie ihn verspottet hätten.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdcccxiv{So lassen Wir ihn in die Herzen der Übeltäter eingehen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdcccxv{Sie glauben nicht daran, und es wird (an ihnen) nach dem Beispiel der Früheren verfahren.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdcccxvi{Auch wenn Wir ihnen ein Tor vom Himmel öffneten und sie ständig dadurch emporstiegen,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdcccxvii{Würden sie sagen: «Unsere Blicke sind ja verschlossen. Nein, wir sind Leute, die einem Zauber verfallen sind.»\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdcccxviii{Und Wir haben im Himmel Sternzeichen gesetzt und ihn für die Zuschauer geschmückt,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdcccxix{Und Wir haben ihn vor jedem gesteinigten Satan bewahrt,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdcccxx{Außer dem, der wie ein Dieb horcht, worauf ihn eine deutlich erkennbare Sternschnuppe verfolgt.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdcccxxi{Auch die Erde haben Wir ausgebreitet und auf ihr festgegründete Berge angebracht. Und Wir haben auf ihr allerlei Dinge im rechten Maß wachsen lassen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdcccxxii{Und Wir haben auf ihr für euch Unterhaltsmöglichkeiten bereitet, und (auch) für diejenigen, die ihr nicht versorgt.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdcccxxiii{Und es gibt nichts, von dem Wir nicht einen Vorrat angelegt hätten. Und Wir senden es nur in festgelegtem Maß hinab.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdcccxxiv{Und Wir haben die befruchtenden Winde gesandt. Und Wir haben dann vom Himmel Wasser hinabkommen lassen und es euch zu trinken gegeben. Ihr aber hättet davon keinen Vorrat anlegen können.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdcccxxv{Und Wir sind es, die lebendig machen und sterben lassen, und Wir sind es, die (alles) erben.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdcccxxvi{Und Wir kennen diejenigen unter euch, die vorausgehen, und Wir kennen die, die zurückbleiben.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdcccxxvii{Und siehe, dein Herr wird sie versammeln. Er ist weise und weiß Bescheid.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdcccxxviii{Und Wir haben den Menschen aus einer Trockenmasse, aus einem gestaltbaren schwarzen Schlamm erschaffen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdcccxxix{Und die Djinn haben Wir vorher aus dem Feuer des glühenden Windes erschaffen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdcccxxx{Und als dein Herr zu den Engeln sprach: «Ich. werde einen Menschen aus einer Trockenmasse, aus einem gestaltbaren schwarzen Schlamm erschaffen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdcccxxxi{Wenn Ich ihn geformt und ihm von meinem Geist eingeblasen habe, dann fallt und werft euch vor ihm nieder.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdcccxxxii{Da warfen sich die Engel alle zusammen nieder,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdcccxxxiii{Außer Iblis; er weigerte sich, zu denen zu gehören, die sich niederwarfen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdcccxxxiv{Er sprach: «O Iblis, was ist mit dir, daß du nicht zu denen gehörst, die sich niederwarfen?»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdcccxxxv{Er sagte: «Ich kann mich unmöglich vor einem Menschen niederwerfen, den Du aus einer Trockenmasse, aus einem gestaltbaren schwarzen Schlamm erschaffen hast.»\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdcccxxxvi{Er sprach: «Dann geh aus ihm hinaus. Du bist ja der Steinigung würdig.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdcccxxxvii{Und auf dir liegt der Fluch bis zum Tag des Gerichtes.»\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdcccxxxviii{Er sagte: «Mein Herr, gewähre mir Aufschub bis zu dem Tag, da sie auferweckt werden.»\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdcccxxxix{Er sprach: «Siehe, du gehörst zu denen, denen Aufschub gewährt wird\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdcccxl{Bis zu dem Tag der festgelegten Zeit.»\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdcccxli{Er sagte: «Mein Herr, weil Du mich irregeführt hast, werde ich, ich schwöre es, ihnen auf der Erde Verlockungen bereiten und sie allesamt abirren lassen,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdcccxlii{Außer deinen auserwählten Dienern unter ihnen.»\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdcccxliii{Er sprach: «Das ist ein gerader Weg, der mir obliegt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdcccxliv{Was meine Diener betrifft, so hast du über sie keine Macht, außer denen unter den Abgeirrten, die dir folgen.»\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdcccxlv{Und die Hölle ist der Verabredungsort für sie alle.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdcccxlvi{Sie hat sieben Tore, und jedem Tor wird ein Teil von ihnen zugewiesen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdcccxlvii{Die Gottesfürchtigen aber werden in Gärten und an Quellen sein:\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdcccxlviii{«Geht hinein in Frieden und Sicherheit.»\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdcccxlix{Und Wir nehmen weg, was in ihrer Brust an Groll dasein mag, so daß sie wie Brüder auf Liegen ruhen, einander gegenüber.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdcccl{Darin erfaßt sie keine Mühsal, und sie werden nicht daraus vertrieben.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdcccli{Tu meinen Dienern kund, daß Ich der bin, der voller Vergebung und barmherzig ist,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdccclii{Und daß meine Pein die schmerzhafte Pein ist.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdcccliii{Und berichte ihnen von den Gästen Abrahams.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdcccliv{Als sie bei ihm eintraten und sagten: «Frieden!» Er sagte: «Wir haben Angst vor euch.»\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdccclv{Sie sagten: «Hab keine Angst. Wir verkünden dir einen klugen Knaben.»\qt@no{53}}%
\qt@newcmd\qurantrans@de@mdccclvi{Er sagte: «Ihr verkündet (es) mir, obwohl mich das Alter erfaßt hat! Was verkündet ihr mir denn?»\qt@no{54}}%
\qt@newcmd\qurantrans@de@mdccclvii{Sie sagten: «Wir verkünden (es) dir in Wahrheit. So sei nicht einer von denen, die die Hoffnung aufgeben.»\qt@no{55}}%
\qt@newcmd\qurantrans@de@mdccclviii{Er sagte: «Nur die Abgeirrten geben die Hoffnung auf die Barmherzigkeit ihres Herrn auf.»\qt@no{56}}%
\qt@newcmd\qurantrans@de@mdccclix{Er sagte: «Was ist euer Anliegen, ihr Boten?»\qt@no{57}}%
\qt@newcmd\qurantrans@de@mdccclx{Sie sagten: «Wir sind zu Leuten gesandt, die Übeltäter sind,\qt@no{58}}%
\qt@newcmd\qurantrans@de@mdccclxi{Ausgenommen die Sippe Lots. Diese werden Wir sicher alle erretten,\qt@no{59}}%
\qt@newcmd\qurantrans@de@mdccclxii{Außer seiner Frau. Wir haben (es so) bestimmt, sie gehört zu denen, die zurückbleiben und dem Verderben anheimfallen.»\qt@no{60}}%
\qt@newcmd\qurantrans@de@mdccclxiii{Als nun die Boten zu der Sippe Lots kamen,\qt@no{61}}%
\qt@newcmd\qurantrans@de@mdccclxiv{Sagte er: «Ihr seid unbekannte Leute.»\qt@no{62}}%
\qt@newcmd\qurantrans@de@mdccclxv{Sie sagten: «Nein, wir kommen zu dir mit dem, was sie immer wieder bezweifelt haben.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mdccclxvi{Und wir kommen zu dir mit der Wahrheit. Und wir sagen, was wahr ist.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mdccclxvii{So zieh mit deinen Angehörigen in einem Teil der Nacht fort, und folge du hinterdrein. Und keiner von euch soll sich umdrehen. Und geht, wohin euch befohlen wird.»\qt@no{65}}%
\qt@newcmd\qurantrans@de@mdccclxviii{Und Wir haben ihm diesen Befehl mitgeteilt, daß der letzte Rest dieser Leute am Morgen ausgemerzt werde.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mdccclxix{Und die Bewohner der Stadt kamen frohlockend.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mdccclxx{Er sagte: «Diese sind meine Gäste, so stellt mich nicht bloß.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mdccclxxi{Und fürchtet Gott und bringt keine Schande über mich.»\qt@no{69}}%
\qt@newcmd\qurantrans@de@mdccclxxii{Sie sagten: «Haben wir dir nicht verboten, mit den Weltenbewohnern Umgang zu pflegen?»\qt@no{70}}%
\qt@newcmd\qurantrans@de@mdccclxxiii{Er sagte: «Da sind meine Töchter, so ihr etwas tun wollt.»\qt@no{71}}%
\qt@newcmd\qurantrans@de@mdccclxxiv{Bei deinem Leben, sie irrten in ihrer Trunkenheit umher.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mdccclxxv{So ergriff sie der Schrei bei Sonnenaufgang,\qt@no{73}}%
\qt@newcmd\qurantrans@de@mdccclxxvi{Und Wir kehrten in ihrer Stadt das Oberste zuunterst und ließen auf sie Steine aus Ton herabregnen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mdccclxxvii{Darin sind wahrlich Zeichen für die Betrachtenden.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mdccclxxviii{Und sie liegt an einem noch bestehenden Weg.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mdccclxxix{Darin ist wahrlich ein Zeichen für die Gläubigen.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mdccclxxx{Und die Gefährten des Waldes taten Unrecht,\qt@no{78}}%
\qt@newcmd\qurantrans@de@mdccclxxxi{So rächten Wir uns an ihnen. Beide liegen an einem offenkundigen Wegweiser.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mdccclxxxii{Und die Gefährten von al-Hidjr ziehen die Gesandten der Lüge.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mdccclxxxiii{Wir ließen ihnen unsere Zeichen zukommen, aber sie wandten sich von ihnen ab.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mdccclxxxiv{Und sie hauten aus den Bergen Häuser aus, um in Sicherheit zu leben.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mdccclxxxv{Da ergriff sie der Schrei am Morgen,\qt@no{83}}%
\qt@newcmd\qurantrans@de@mdccclxxxvi{Und so nützte ihnen, was sie zu erwerben pflegten, nicht.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mdccclxxxvii{Siehe, Wir haben die Himmel und die Erde, und was dazwischen ist, in Wahrheit erschaffen. Und die Stunde wird sicher eintreffen. So übe schöne Nachsicht.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mdccclxxxviii{Dein Herr ist es, der alles erschafft und Bescheid weiß.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mdccclxxxix{Und Wir haben dir sieben aus den sich wiederholenden Versen zukommen lassen, und auch den gewaltigen Koran.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mdcccxc{Richte nicht gierig deine Augen auf das, was Wir einigen von ihnen zur Nutznießung verliehen haben. Und sei nicht betrübt über sie. Und senke deinen Flügel für die Gläubigen,\qt@no{88}}%
\qt@newcmd\qurantrans@de@mdcccxci{Und sprich: Ich bin ja der deutliche Warner.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mdcccxcii{Wie Wir auf die hinabgesandt haben, die aufteilen,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mdcccxciii{Die den Koran zergliedert haben.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mdcccxciv{Bei deinem Herrn! Wir werden sie alle zur Verantwortung ziehen\qt@no{92}}%
\qt@newcmd\qurantrans@de@mdcccxcv{über das, was sie taten.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mdcccxcvi{Und verkünde laut, was dir befohlen wird, und wende dich von den Polytheisten ab.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mdcccxcvii{Wir schützen dich vor den Spöttern,\qt@no{95}}%
\qt@newcmd\qurantrans@de@mdcccxcviii{Die Gott einen anderen Gott zur Seite stellen. Sie werden es noch zu wissen bekommen.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mdcccxcix{Wir wissen ja, daß deine Brust beklommen ist wegen dessen, was sie sagen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcm{Aber sing das Lob deines Herrn und sei einer von denen, die sich niederwerfen,\qt@no{98}}%
\qt@newcmd\qurantrans@de@mcmi{Und diene deinem Herrn, bis das sichere Los dich ereilt.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mcmii{\basmalahde Der Befehl Gottes ist eingetroffen, so wünscht nicht, ihn zu beschleunigen. Preis sei Ihm, und erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mcmiii{Er sendet die Engel mit dem Geist von seinem Befehl herab, auf wen von seinen Dienern Er will: «Warnt (und verkündet), daß es keinen Gott gibt außer Mir. Ihr sollt Mich (allein) fürchten.»\qt@no{2}}%
\qt@newcmd\qurantrans@de@mcmiv{Er hat die Himmel und die Erde in Wahrheit erschaffen. Erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mcmv{Den Menschen hat Er aus einem Tropfen erschaffen. Und doch ist er gleich offenkundig streitsüchtig.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mcmvi{Auch hat Er die Herdentiere erschaffen. An ihnen habt ihr Wärme und, allerlei Nutzen; und ihr könnt davon essen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mcmvii{Und ihr habt an ihnen Schönes, wenn ihr (sie) abends eintreibt und wenn ihr (sie) morgens austreibt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mcmviii{Und sie tragen eure Lasten in ein Land, das ihr (sonst) nur mit größter Mühe hättet erreichen können. Euer Herr hat ja Mitleid und ist barmherzig.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mcmix{Und (erschaffen hat Er) die Pferde, die Maultiere und die Esel, damit ihr auf ihnen reitet, und auch (euch) zur Zierde. Und Er erschafft, was ihr nicht wißt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mcmx{Gott obliegt es, den richtigen Weg zu weisen. Es gibt ja welche, die abweichen. Wenn Er gewollt hätte, hätte Er euch allesamt rechtgeleitet.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mcmxi{Er ist es, der vom Himmel Wasser hat herabkommen lassen. Davon habt ihr etwas zu trinken, und davon wachsen Sträucher, in denen ihr weiden lassen könnt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mcmxii{Er läßt euch dadurch Getreide sprießen, und Ölbäume, Palmen, Weinstöcke und allerlei Früchte. Darin ist ein Zeichen für Leute, die nachdenken.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mcmxiii{Und Er hat euch die Nacht und den Tag, die Sonne und den Mond dienstbar gemacht. Auch die Sterne sind durch seinen Befehl dienstbar gemacht worden. Darin sind Zeichen für Leute, die verständig sind.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mcmxiv{Und (da ist) auch, was Er euch auf der Erde in verschiedenen Arten geschaffen hat. Darin ist ein Zeichen für Leute, die es bedenken.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mcmxv{Und Er ist es, der euch das Meer dienstbar gemacht hat, damit ihr frisches Fleisch daraus eßt und Schmuck aus ihm herausholt, um ihn anzulegen. Und du siehst die Schiffe es durchspalten, ja, damit ihr nach etwas von seiner Huld strebt, auf daß ihr dankbar werdet.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mcmxvi{Und Er hat auf der Erde festgegründete Berge gelegt, daß sie nicht mit euch schwanke, und Flüsse und Wege - auf daß ihr der Rechtleitung folget -\qt@no{15}}%
\qt@newcmd\qurantrans@de@mcmxvii{Und Wegzeichen. Und mit Hilfe der Sterne finden sie die Richtung.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mcmxviii{Ist denn der, der erschafft, wie der, der nicht erschafft? Wollt ihr es nicht bedenken?\qt@no{17}}%
\qt@newcmd\qurantrans@de@mcmxix{Und wenn ihr die Gnade Gottes aufzählen wolltet, könntet ihr sie nicht erfassen. Wahrlich, Gott ist voller Vergebung und barmherzig.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mcmxx{Und Gott weiß, was ihr geheimhaltet und was ihr offenlegt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mcmxxi{Und diejenigen, die sie anstelle Gottes anrufen, erschaffen nichts; sie werden aber selbst erschaffen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mcmxxii{Tot sind sie, nicht lebendig, und sie merken nicht, wann sie erweckt werden.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mcmxxiii{Euer Gott ist ein einziger Gott. Diejenigen, die nicht an das Jenseits glauben, weisen es in ihren Herzen ab und zeigen sich hochmütig.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mcmxxiv{Wahrlich, Gott weiß, was sie geheimhalten und was sie offenlegen. Er liebt die nicht, die sich hochmütig zeigen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mcmxxv{Und wenn zu ihnen gesagt wird: «Was hat euer Herr herabgesandt?», sagen sie: «Die Fabeln der Früheren.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mcmxxvi{So sollen sie am Tag der Auferstehung ihre Lasten vollständig tragen, und auch einen Teil der Lasten derer, die sie ohne (richtiges) Wissen irreleiten. O schlimm ist, was auf ihnen lastet!\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcmxxvii{Ränke haben diejenigen, die vor ihnen lebten, geschmiedet. Da erfaßte Gott ihren Bau an den Grundmauern, und die Decke über ihnen stürzte auf sie herab, und die Pein kam über sie, von wo sie es nicht merkten.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mcmxxviii{Dann am Tag der Auferstehung wird Er sie zuschanden machen und sagen: «Wo sind meine Teilhaber, derentwegen ihr euch widersetztet?» Diejenigen, denen das Wissen zugekommen ist, sagen: «Schande und Unheil kommen heute über die Ungläubigen»,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mcmxxix{Die von den Engeln abberufen werden, während sie sich selbst Unrecht getan haben. Sie bieten den Frieden an: «Wir pflegten nichts Böses zu tun.» - «Doch, Gott weiß Bescheid über das, was ihr zu tun pflegtet.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mcmxxx{Betretet nun die Tore der Hölle; darin werdet ihr ewig weilen. Schlimm ist ja die Bleibe der Hochmütigen.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcmxxxi{Zu denen, die gottesfürchtig sind, wird gesagt: «Was hat euer Herr herabgesandt?» Sie sagen: «Gutes.» Diejenigen, die rechtschaffen sind, erhalten hier im Diesseits Gutes. Aber die Wohnstätte des Jenseits ist gewiß besser. Vorzüglich ist die Wohnstätte der Gottesfürchtigen:\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcmxxxii{Die Gärten von Eden, in die sie eingehen, unter denen Bäche fließen und in denen sie (alles) haben, was sie wollen. So entlohnt Gott die Gottesfürchtigen,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcmxxxiii{Die die Engel abberufen, während sie gute (Menschen) gewesen sind. Sie sagen: «Friede sei über euch! Geht ins Paradies ein für das, was ihr zu tun pflegtet.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcmxxxiv{Erwarten Sie denn etwas anderes, als daß die Engel zu ihnen kommen, oder daß der Befehl deines Herrn eintrifft? So handelten auch diejenigen, die vor ihnen lebten. Und nicht Gott hat ihnen Unrecht getan, sondern sie haben sich selbst Unrecht getan.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcmxxxv{Getroffen haben sie (schließlich) die bösen Taten, die sie begangen haben, und umschlossen hat sie das, worüber sie spotteten.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcmxxxvi{Diejenigen, die Polytheisten sind, sagen: «Wenn Gott gewollt hätte, hätten wir nichts an seiner Stelle verehrt, weder wir noch unsere Väter, und wir hätten nichts an seiner Stelle verboten.» So handelten auch diejenigen, die vor ihnen lebten. Obliegt denn dem Gesandten etwas anderes als die deutliche Ausrichtung (der Botschaft)?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcmxxxvii{Und Wir haben aus der Mitte jeder Gemeinschaft einen Gesandten erstehen lassen: «Dienet Gott und meidet die Götzen.» Unter ihnen sind welche, die Gott rechtgeleitet hat, und unter ihnen sind welche, über die der Irrtum zu Recht fällig geworden ist. Geht auf der Erde umher und schaut, wie das Ende derer war, die (die Gesandten) der Lüge geziehen haben.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcmxxxviii{Du magst dich noch so sehr bemühen, sie rechtzuleiten, Gott leitet nicht recht, wen Er (nun) irreführt. Und sie werden keinen Helfer haben.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcmxxxix{Sie schwören bei Gott ihren eifrigsten Eid, Gott werde die nicht auferwecken, die sterben. Doch, das ist ein Ihm obliegendes Versprechen in Wahrheit - aber die meisten Menschen wissen nicht Bescheid -,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcmxl{Damit Er ihnen das deutlich macht, worüber sie uneins sind, und damit diejenigen, die ungläubig sind, wissen, daß sie Lügner waren.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcmxli{Unsere Rede zu einer Sache, wenn Wir sie wollen, ist, zu ihr zu sprechen: Sei!, und sie ist.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcmxlii{Und die, die um Gottes willen ausgewandert sind, nachdem ihnen Unrecht getan wurde, werden Wir gewiß im Diesseits in einen schönen Stand einweisen. Aber der Lohn des Jenseits ist gewiß größer, wenn sie es nur wüßten!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mcmxliii{Sie), die geduldig sind und auf ihren Herrn vertrauen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcmxliv{Und Wir haben vor dir nur Männer gesandt, denen Wir Offenbarungen eingegeben haben. So fragt die Besitzer der Ermahnung, wenn ihr nicht Bescheid wißt.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcmxlv{Wir haben sie gesandt) mit den deutlichen Zeichen und den Schriften. Und Wir haben zu dir die Ermahnung herabgesandt, damit du den Menschen deutlich machst, was zu ihnen herabgesandt worden ist, und damit sie vielleicht nachdenken.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mcmxlvi{Wähnen sich denn diejenigen, die böse Ränke schmieden, in Sicherheit davor, daß Gott die Erde mit ihnen versinken läßt, oder daß die Pein über sie kommt, von wo sie es nicht merken,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mcmxlvii{Oder daß Er sie in ihrem Umherziehen ergreift - und sie können es nicht vereiteln -,\qt@no{46}}%
\qt@newcmd\qurantrans@de@mcmxlviii{Oder daß Er sie ergreift, während sie Angst haben? Euer Herr hat Mitleid und ist barmherzig.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mcmxlix{Haben sie nicht auf die Dinge geschaut, die Gott erschaffen hat? Ihre Schatten wenden sich rechts und links, und sie werfen sich demütig vor Gott nieder.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mcml{Vor Gott wirft sich nieder, was in den Himmeln und was auf der Erde ist, ob Tiere oder Engel, und sie verhalten sich nicht hochmütig.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mcmli{Sie fürchten ihren Herrn, der über ihnen steht, und sie tun, was ihnen befohlen wird.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mcmlii{Und Gott hat gesprochen: Nehmt euch nicht zwei Götter. Er ist nur ein einziger Gott. Vor Mir sollt ihr Ehrfurcht haben.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mcmliii{Ihm gehört, was in den Himmeln und auf der Erde ist. Und Ihm gehört die religiöse Verehrung in ständiger Weise. Wollt ihr denn einen anderen als Gott fürchten?\qt@no{52}}%
\qt@newcmd\qurantrans@de@mcmliv{Was ihr an Gnade erfahrt, ist von Gott. Wenn euch dann ein Schaden trifft, so schreit ihr zu Ihm um Hilfe.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mcmlv{Wenn Er dann euch den Schaden behoben hat, da ist gleich ein Teil von euch dabei, ihrem Herrn (andere) beizugesellen,\qt@no{54}}%
\qt@newcmd\qurantrans@de@mcmlvi{Um undankbar zu sein für das, was Wir ihnen zukommen ließen. So genießet nur. Ihr werdet es noch zu wissen bekommen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mcmlvii{Und sie bestimmen für das, von dem sie kein Wissen haben, einen Anteil von dem, was Wir ihnen beschert haben. Bei Gott, ihr werdet zu verantworten haben, was ihr zu erdichten pflegtet.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mcmlviii{Und sie geben Gott Töchter - Preis sei Ihm! - und sich selbst, was sie begehren.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mcmlix{Wenn einer von ihnen von der Geburt eines Mädchens benachrichtigt wird, bleibt sein Gesicht finster, und er unterdrückt (seinen Groll).\qt@no{58}}%
\qt@newcmd\qurantrans@de@mcmlx{Er verbirgt sich vor den Leuten wegen der schlimmen Nachricht. Solle er es nun trotz der Schmach behalten oder es im Boden verscharren. Übel ist, wie sie da urteilen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mcmlxi{Diejenigen, die an das Jenseits nicht glauben, besitzen die Eigenschaft des Bösen. Gott besitzt die höchste Eigenschaft, und er ist der Mächtige, der Weise.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mcmlxii{Wenn Gott die Menschen für ihre Ungerechtigkeit belangen wollte, würde Er auf ihr kein Tier übriglassen. Aber Er stellt sie auf eine festgesetzte Frist zurück. Und wenn ihre Frist eintrifft, können sie nicht einmal eine Stunde zurückbleiben oder vorausgehen.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mcmlxiii{Und sie geben Gott, was sie verabscheuen. Und ihre Zungen behaupten lügnerisch, daß für sie das Beste bestimmt ist. Kein Zweifel, daß für sie das Feuer bestimmt ist und daß sie zuerst hineingestürzt werden.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mcmlxiv{Bei Gott, Wir haben (Propheten) zu Gemeinschaften vor dir gesandt. Da machte ihnen der Satan ihre Taten verlockend. So ist er heute ihr Freund, und bestimmt ist für sie eine schmerzhafte Pein.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mcmlxv{Und Wir haben auf dich das Buch nur deswegen hinabgesandt, damit du ihnen das deutlich machst, worüber sie uneins waren, und als Rechtleitung und Barmherzigkeit für Leute, die glauben.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mcmlxvi{Und Gott hat vom Himmel Wasser herabkommen lassen und mit ihm die Erde nach ihrem Absterben wieder belebt. Darin ist ein Zeichen für Leute, die hören können.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mcmlxvii{Einen Grund zum Nachdenken habt ihr in den Herdentieren. Wir geben euch von dem, was in ihrem Leib zwischen Kot und Blut ist, zu trinken, reine Milch, bekömmlich für die, die (sie) trinken.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mcmlxviii{Und (Wir geben euch) von den Früchten der Palmen und der Weinstöcke, woraus ihr euch Rauschgetränk und einen schönen Lebensunterhalt nehmt. Darin ist ein Zeichen für Leute, die verständig sind.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mcmlxix{Und dein Herr hat der Biene eingegeben: «Nimm dir Häuser in den Bergen, in den Bäumen und in den Spalieren.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mcmlxx{Dann iß von allen Früchten, wandle auf den Wegen deines Herrn, die (dir) leicht gemacht sind.» Aus ihren Leibern kommt ein Trank von verschiedenen Arten, in dem Heilung für die Menschen ist. Darin ist ein Zeichen für Leute, die nachdenken.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mcmlxxi{Und Gott hat euch erschaffen, dann beruft Er euch ab. Und manch einer von euch wird in das schlimmste Alter gebracht, so daß er nach (vorherigem) Wissen nun nichts (mehr) weiß. Gott weiß Bescheid und ist mächtig.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mcmlxxii{Und Gott hat einige von euch im Lebensunterhalt vor den anderen bevorzugt. Doch geben diejenigen, die bevorzugt werden, ihren Lebensunterhalt nicht an die zurück, die ihre rechte Hand besitzt, so daß sie alle darin gleich werden. Wollen sie denn die Gnade Gottes leugnen?\qt@no{71}}%
\qt@newcmd\qurantrans@de@mcmlxxiii{Und Gott hat euch aus euch selbst Gattinnen gemacht, und von euren Gattinnen Söhne und Enkel gemacht. Und Er hat euch (einiges) von den köstlichen Dingen beschert. Wollen sie denn an das Falsche glauben und die Gnade Gottes verleugnen,\qt@no{72}}%
\qt@newcmd\qurantrans@de@mcmlxxiv{Und denen anstelle Gottes dienen, die ihnen keinen Lebensunterhalt von den Himmeln und der Erde bescheren können und (sonst) nichts vermögen?\qt@no{73}}%
\qt@newcmd\qurantrans@de@mcmlxxv{So führt für Gott keine Gleichnisse an. Gott weiß, ihr aber wißt nicht Bescheid.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mcmlxxvi{Gott führt als Gleichnis einen leibeigenen Sklaven an, der über nichts Gewalt hat, und einen, dem Wir von uns her einen schönen Lebensunterhalt beschert haben, so daß er davon geheim und offen spendet. Sind sie etwa gleich? Lob sei Gott! Aber die meisten von ihnen wissen nicht Bescheid.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mcmlxxvii{Und Gott führt als Gleichnis zwei Männer an. Der eine ist stumm und hat über nichts Gewalt; er ist seinem Herrn eine Last; wo er ihn auch hinschickt, bringt er nichts Gutes. Ist er etwa dem gleich, der die Gerechtigkeit gebietet, wobei er einem geraden Weg folgt?\qt@no{76}}%
\qt@newcmd\qurantrans@de@mcmlxxviii{Und Gott gehört das Unsichtbare der Himmel und der Erde. Und die Angelegenheit der Stunde ist nur wie ein Augenblick, oder sie ist noch näher. Gott hat Macht zu allen Dingen.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mcmlxxix{Und Gott hat euch aus dem Leib eurer Mütter hervorgebracht, während ihr nichts wußtet. Und Er hat euch Gehör, Augenlicht und Herz gegeben, auf daß ihr dankbar seid.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mcmlxxx{Haben sie nicht auf die Vögel geschaut, die im Luftraum des Himmels dienstbar gemacht worden sind? Nur Gott hält sie oben. Darin sind Zeichen für Leute, die glauben.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mcmlxxxi{Und Gott hat euch aus euren Häusern eine Ruhestätte gemacht, und Er hat euch aus den Häuten des Viehs Behausungen gemacht, die ihr am Tag eures Aufbrechens und am Tag eures Aufenthaltes leicht benutzen könnt, und aus ihrer Wolle, ihren Fellhärchen und ihrem Haar Ausstattung und Nutznießung für eine Weile.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mcmlxxxii{Und Gott hat euch aus dem, was Er erschaffen hat, schattenspendende Dinge gemacht. Und Er hat euch aus den Bergen Verstecke gemacht. Und Er hat euch Gewänder gemacht, die euch vor der Hitze schützen, und Gewänder, die euch vor eurer Schlagkraft (gegeneinander) schützen. So vollendet Er seine Gnade an euch, auf daß ihr gottergeben seid.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mcmlxxxiii{Wenn sie sich abkehren, so obliegt dir nur die deutliche Ausrichtung (der Botschaft).\qt@no{82}}%
\qt@newcmd\qurantrans@de@mcmlxxxiv{Sie kennen die Gnade Gottes, und dann verwerfen sie sie. Die meisten von ihnen sind ja Ungläubige.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mcmlxxxv{Und am Tag, da Wir von jeder Gemeinschaft einen Zeugen erwecken. Dann wird denen, die ungläubig sind, keine Erlaubnis gegeben, und es wird von ihnen keine entschuldigende Umkehr (mehr) angenommen.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mcmlxxxvi{Und wenn diejenigen, die Unrecht taten, die Pein sehen, dann wird ihnen weder Erleichterung noch Aufschub gewährt.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mcmlxxxvii{Und wenn diejenigen, die Polytheisten waren, ihre Teilhaber sehen, sagen sie: «Unser Herr, das sind unsere Teilhaber, die wir an deiner Stelle anzurufen pflegten.» Sie aber richteten an sie das Wort: «Ihr seid ja Lügner.»\qt@no{86}}%
\qt@newcmd\qurantrans@de@mcmlxxxviii{Sie bieten Gott an jenem Tag Ergebenheit an, und entschwunden ist ihnen, was sie zu erdichten pflegten.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mcmlxxxix{Diejenigen, die ungläubig sind und vom Weg Gottes abweisen, lassen Wir eine noch größere Pein über ihre Pein erleiden dafür, daß sie Unheil stifteten.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mcmxc{Und am Tag, da Wir in jeder Gemeinschaft aus ihren eigenen Reihen einen Zeugen über sie erwecken. Dich bringen Wir zum Zeugen über diese da. Und Wir haben das Buch auf dich hinabgesandt, um alles deutlich zu machen, und als Rechtleitung, Barmherzigkeit und Frohbotschaft für die Gottergebenen.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mcmxci{Gott gebietet, Gerechtigkeit zu üben, Gutes zu tun und die Verwandten zu beschenken. Er verbietet das Schändliche, das Verwerfliche und die Gewalttätigkeit. Er ermahnt euch, auf daß ihr es bedenket.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mcmxcii{Und haltet den Bund Gottes, wenn ihr einen Bund geschlossen habt, und brecht nicht die Eide nach ihrer Bekräftigung, wo ihr Gott zum Bürgen über euch gemacht habt. Gott weiß, was ihr tut.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mcmxciii{Und seid nicht wie jene, die ihr Garn, nachdem es fest gesponnen war, wieder in aufgelöste Strähnen bricht, indem ihr eure Eide untereinander als Mittel des Betrugs nehmt, weil ja eine Gemeinschaft zahlreicher ist als eine andere Gemeinschaft. Gott prüft euch damit. Und Er wird euch am Tag der Auferstehung gewiß das deutlich machen, worüber ihr uneins wäret.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mcmxciv{Und wenn Gott gewollt hätte, hätte Er euch zu einer einzigen Gemeinschaft gemacht. Aber Er führt irre, wen Er will, und Er leitet recht, wen Er will, und ihr werdet das zu verantworten haben, was ihr zu tun pflegtet.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mcmxcv{Und nehmt euch nicht eure Eide als Mittel des Betrugs untereinander, sonst könnte der eine Fuß, nachdem er fest gestanden hat, straucheln, und ihr würdet Böses erleiden dafür, daß ihr vom Weg Gottes abgewiesen habt. Bestimmt ist dann für euch eine gewaltige Pein.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mcmxcvi{Und verkauft den Bund Gottes nicht für einen geringen Preis. Was bei Gott ist, ist besser für euch, wenn ihr Bescheid wißt.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mcmxcvii{Was bei euch ist, geht zu Ende; was bei Gott ist, hat Bestand. Wir werden bestimmt denen, die geduldig sind, mit ihrem Lohn vergelten für das Beste von dem, was sie taten.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mcmxcviii{Wer Gutes tut, ob Mann oder Weib, und dabei gläubig ist, den werden Wir bestimmt ein angenehmes Leben leben lassen. Und Wir werden ihnen bestimmt mit ihrem Lohn vergelten für das Beste von dem, was sie taten.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcmxcix{Und so du den Koran verliest, suche Zuflucht bei Gott vor dem gesteinigten Satan.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mm{Er hat keine Macht über diejenigen, die glauben und auf ihren Herrn vertrauen.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmi{Er hat Macht nur über diejenigen, die ihn zum Freund nehmen und die Ihm (andere) beigesellen.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmii{Und wenn Wir ein Zeichen anstelle eines (anderen) Zeichens eintauschen - und Gott weiß besser, was Er herabsendet -, sagen sie: «(Das) erdichtest du nur.» Aber nein, die meisten von ihnen wissen nicht Bescheid.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmiii{Sprich: Herabgesandt hat ihn der Geist der Heiligkeit von deinem Herrn mit der Wahrheit, um diejenigen, die glauben, zu festigen, und als Rechtleitung und Frohbotschaft für die Gottergebenen.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmiv{Und wir wissen ja doch, daß sie sagen: «Es lehrt ihn gewiß ein Mensch.» Die Sprache dessen, auf den sie hinweisen, ist eine fremde, und dies hier ist eine deutliche arabische Sprache.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmv{Diejenigen, die nicht an die Zeichen Gottes glauben, leitet Gott nicht recht, und bestimmt ist für sie eine schmerzhafte Pein.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmvi{Lügen erdichten nur diejenigen, die nicht an die Zeichen Gottes glauben. Das sind die (wahren) Lügner.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmvii{Wer Gott verleugnet, nachdem er gläubig war - außer dem, der gezwungen wird, während sein Herz im Glauben Ruhe gefunden hat -, nein, diejenigen, die ihre Brust dem Unglauben öffnen, über die kommt ein Zorn von Gott, und bestimmt ist für sie eine gewaltige Pein.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmviii{Dies, weil sie das diesseitige Leben mehr lieben als das Jenseits und weil Gott die ungläubigen Leute nicht rechtleitet.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmix{Das sind diejenigen, deren Herz, Gehör und Augenlicht Gott versiegelt hat. Und das sind die, die alles unbeachtet lassen.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmx{Zweifellos sind sie im Jenseits die Verlierer.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmxi{Dein Herr ist zu denen, die (doch) ausgewandert sind, nachdem sie der Versuchung ausgesetzt wurden, und sich dann eingesetzt haben und standhaft gewesen sind - ja, dein Herr ist nach diesem (zu ihnen) voller Vergebung und barmherzig.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmxii{Am Tag, da jede Seele kommt, um für sich selbst zu streiten, und da jeder Seele voll erstattet wird, was sie getan hat. Und ihnen wird kein Unrecht getan.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmxiii{Gott führt als Gleichnis eine Stadt an, die in Sicherheit und Ruhe lebte. Ihr Lebensunterhalt kam zu ihr reichlich von überall her. Da wurde sie gegenüber den Wohltaten Gottes undankbar. So ließ sie Gott das Kleid des Hungers und der Angst erleiden für das, was sie machten.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmxiv{Ein Gesandter aus ihrer Mitte kam doch zu ihnen. Sie ziehen ihn der Lüge. So ergriff sie die Pein, während sie Unrecht taten.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmxv{Eßt nun von dem, was Gott euch beschert hat, so es erlaubt und köstlich ist. Und seid dankbar für die Gnade Gottes, so ihr wirklich Ihm dient.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmxvi{Verboten hat Er euch Verendetes, Blut, Schweinefleisch und das, worüber ein anderer als Gott angerufen worden ist. Wenn aber einer gezwungen wird, wobei er weder Auflehnung noch Übertretung begeht, so ist Gott voller Vergebung und barmherzig.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmxvii{Und sagt nicht von dem, was eure Zungen lügnerisch behaupten: «Dies ist erlaubt, und dies ist verboten», um gegen Gott Lügen zu erdichten. Denen, die gegen Gott Lügen erdichten, wird es nicht wohl ergehen.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmxviii{Es ist nur eine geringe Nutznießung. Und bestimmt ist für sie eine schmerzhafte Pein.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmxix{Und denen, die Juden sind, haben Wir das verboten, worüber Wir dir zuvor berichtet haben. Und nicht Wir taten ihnen Unrecht, sondern sie selbst haben sich Unrecht getan.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmxx{Dein Herr ist zu denen, die aus Unwissenheit Böses tun, aber danach umkehren und Besserung zeigen - ja, dein Herr ist danach (zu ihnen) voller Vergebung und barmherzig.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmxxi{Abraham war ein Vorbild, Gott demütig ergeben und Anhänger des reinen Glaubens, und er gehörte nicht zu den Polytheisten;\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmxxii{Dankbar (war er) für seine Wohltaten. Er hat ihn erwählt und zu einem geraden Weg geleitet.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmxxiii{Und Wir haben ihm im Diesseits Gutes zukommen lassen. Und im Jenseits gehört er gewiß zu den Rechtschaffenen.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmxxiv{Und Wir haben dir offenbart: «Folge der Glaubensrichtung Abrahams, als Anhänger des reinen Glaubens, und er gehörte nicht zu den Polytheisten.»\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmxxv{Und der Sabbat ist denen auferlegt worden, die über ihn uneins waren. Dein Herr wird bestimmt am Tag der Auferstehung zwischen ihnen über das urteilen, worüber sie uneins waren.\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmxxvi{Ruf zum Weg deines Herrn mit Weisheit und schöner Ermahnung, und streite mit ihnen auf die beste Art. Dein Herr weiß besser, wer von seinem Weg abirrt, und Er weiß besser, wer die sind, die der Rechtleitung folgten.\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmxxvii{Und wenn ihr bestraft, so bestraft im gleichen Maße, wie ihr bestraft wurdet. Und wenn ihr euch geduldig zeigt, so ist es besser für die Geduldigen.\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmxxviii{Sei geduldig. Deine Geduld ist nur durch Gott möglich. Sei nicht betrübt über sie, und sei nicht in Bedrängnis wegen der Ränke, die sie schmieden.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmxxix{Und Gott ist mit denen, die (Ihn) fürchten und die rechtschaffen sind.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmxxx{\basmalahde Preis sei dem, der seinen Diener bei Nacht von der heiligen Moschee zur fernsten Moschee, die Wir ringsum gesegnet haben, reisen ließ, damit Wir ihm etwas von unseren Zeichen zeigen. Er ist der, der alles hört und sieht.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmxxxi{Und Wir ließen dem Mose das Buch zukommen und machten es zu einer Rechtleitung für die Kinder Israels: «Nehmt euch außer Mir keinen Sachwalter,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmxxxii{Ihr, die Nachkommenschaft derer, die Wir mit Noach getragen haben. Er war ein dankbarer Diener.»\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmxxxiii{Und Wir haben für die Kinder Israels im Buch bestimmt: «Wahrlich, ihr werdet zweimal auf der Erde Unheil stiften, und ihr werdet große Macht erringen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmxxxiv{Wenn nun die Drohung vom ersten Mal eintrifft, schicken Wir gegen euch Diener von Uns, die eine starke Schlagkraft besitzen. Sie dringen durch die Wohnstätten ein. Und dies ist eine Drohung, die ausgeführt sein wird.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmxxxv{Dann geben Wir euch wieder die Oberhand über sie, und Wir stehen euch bei mit Vermögen und Söhnen und machen euch zu einer zahlreicheren Schar.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmxxxvi{Wenn ihr Gutes tut, tut ihr Gutes für euch selbst. Wenn ihr Böses tut, ist es auch für euch selbst. - Wenn dann die Drohung vom letzten Mal eintrifft, (kommen sie), um euch schlimm ins Gesicht zu treffen und die Moschee zu betreten, wie sie sie beim ersten Mal betraten, und das, worüber sie Macht errungen haben, dem völligen Verderben preiszugeben.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmxxxvii{Möge euer Herr sich euer erbarmen! Und wenn ihr (dazu) zurückkehrt, kehren Wir (auch) zurück. Und Wir haben die Hölle zum Gefängnis für die Ungläubigen gemacht.»\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmxxxviii{Dieser Koran leitet zu dem, was richtiger ist, und verkündet den Gläubigen, die die guten Werke tun, daß für sie ein großer Lohn bestimmt ist,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmxxxix{Und daß Wir denen, die an das Jenseits nicht glauben, eine schmerzhafte Pein bereitet haben.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmxl{Der Mensch erbittet das Böse, wie er das Gute erbittet. Der Mensch ist ja eilfertig.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmxli{Und Wir haben die Nacht und den Tag zu zwei Zeichen gemacht. Das Zeichen der Nacht haben Wir gelöscht, und Wir haben das Zeichen des Tages so gemacht, daß man (an ihm) sehen kann, damit ihr nach einer Huld von eurem Herrn strebt, und damit ihr die Zahl der Jahre und die Zeitrechnung wißt. Jedes Ding haben Wir im einzelnen dargelegt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmxlii{Und jedem Menschen haben Wir sein Omen an seinem Hals befestigt. Und am Tag der Auferstehung holen Wir ihm ein Buch heraus, das er aufgeschlagen vorfindet:\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmxliii{«Lies dein Buch. Du selbst genügst heute, um mit dir selbst abzurechnen.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmxliv{Wer der Rechtleitung folgt, folgt ihr zu seinem eigenen Vorteil. Und wer irregeht, der geht irre zu seinem eigenen Schaden. Und keine lasttragende (Seele) trägt die Last einer anderen. Und Wir peinigen nicht, ehe Wir einen Gesandten haben erstehen lassen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmxlv{Und wenn Wir eine Stadt verderben wollen, befehlen Wir denen, die in ihr üppig leben, (zu freveln), und sie freveln in ihr. Somit wird der Spruch zu Recht gegen sie fällig, und Wir zerstören sie vollständig.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmxlvi{Und wie viele Generationen nach Noach haben Wir verderben lassen! Dein Herr kennt und sieht die Sünden seiner Diener zur Genüge.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmxlvii{Wer das schnell Eintreffende will, dem gewähren Wir darin schnell, was Wir wollen - dem, den Wir wollen. Dann bestimmen Wir für ihn die Hölle; darin wird er brennen, gescholten und verstoßen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmxlviii{Denen, die das Jenseits wollen und sich entsprechend darum bemühen, wobei sie gläubig sind, denen wird für ihr Mühen gedankt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmxlix{Ihnen allen, diesen und jenen, stehen Wir bei mit etwas von der Gabe deines Herrn. Die Gabe deines Herrn wird nicht verwehrt.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mml{Schau, wie Wir die einen von ihnen vor den anderen bevorzugen. Im Jenseits gibt es sicher höhere Rangstufen und größere Auszeichnungen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmli{Setze Gott keinen anderen Gott zur Seite, sonst wirst du dasitzen, gescholten und im Stich gelassen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmlii{Und dein Herr hat bestimmt, daß ihr nur Ihm dienen sollt, und daß man die Eltern gut behandeln soll. Wenn eines von ihnen oder beide bei dir ein hohes Alter erreichen, so sag nicht zu ihnen: «Pfui!», und fahre sie nicht an, sondern sprich zu ihnen ehrerbietige Worte.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmliii{Und senke für sie aus Barmherzigkeit den Flügel der Untergebenheit und sag: «Mein Herr, erbarme dich ihrer, wie sie mich aufgezogen haben, als ich klein war.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmliv{Euer Herr weiß besser, was in eurem Inneren ist. Wenn ihr rechtschaffen seid, so ist Er für die, die immer wieder umkehren, voller Vergebung.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmlv{Und laß dem Verwandten sein Recht zukommen, ebenso dem Bedürftigen und dem Reisenden, aber handle nicht ganz verschwenderisch.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmlvi{Diejenigen, die verschwenderisch sind, sind Brüder der Satane; und der Satan ist gegenüber seinem Herrn sehr undankbar.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmlvii{Und falls du dich von ihnen abwendest im Streben nach einer von dir erhofften Barmherzigkeit von deinem Herrn, so sprich zu ihnen milde Worte.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmlviii{Und laß deine Hand nicht an deinem Hals gefesselt sein, aber strecke sie auch nicht vollständig aus. Sonst würdest du getadelt und verarmt dasitzen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmlix{Dein Herr teilt den Lebensunterhalt großzügig, wem Er will, und auch bemessen zu. Er hat Kenntnis von seinen Dienern, und Er sieht sie wohl.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmlx{Und tötet nicht eure Kinder aus Furcht vor Verarmung. Ihnen und euch bescheren Wir doch den Lebensunterhalt. Sie töten ist eine große Sünde.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmlxi{Und nähert euch nicht der Unzucht. Sie ist etwas Schändliches, und sie ist ein übler Weg.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmlxii{Und tötet nicht den Menschen, den Gott für unantastbar erklärt hat, es sei denn bei vorliegender Berechtigung. Wird jemand ungerechterweise getötet, so geben Wir seinem nächsten Verwandten Vollmacht (ihn zu rächen). Nur soll er nicht maßlos im Töten sein; siehe, er wird Beistand finden.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmlxiii{Und nähert euch nicht dem Vermögen des Waisenkindes, es sei denn auf die beste Art, bis es seine Vollkraft erreicht hat. Und erfüllt eingegangene Verpflichtungen. Ober die Verpflichtungen wird Rechenschaft gefordert.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmlxiv{Und gebt volles Maß, wenn ihr meßt. Und wägt mit der richtigen Waage. Das ist besser und führt zu einem schöneren Ergebnis.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmlxv{Und verfolge nicht das, wovon du kein Wissen hast. Ober Gehör, Augenlicht und Herz, über all das wird Rechenschaft gefordert.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmlxvi{Und schreite nicht unbekümmert auf der Erde einher. Du wirst ja die Erde nicht durchbohren und die Berge an Höhe nicht erreichen können.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmlxvii{Das schlechte Verhalten in alledem ist bei deinem Herrn verpönt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmlxviii{Das ist etwas von dem, was dir dein Herr an Weisheit offenbart hat. Und setze Gott keinen anderen Gott zur Seite, sonst wirst du in die Hölle geworfen, getadelt und verstoßen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmlxix{Hat denn euer Herr für euch die Söhne erwählt und sich selbst aus den Reihen der Engel Töchter genommen? Ihr sagt da etwas Ungeheuerliches.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmlxx{Wir haben in diesem Koran (die Botschaft) auf verschiedene Weise dargestellt, damit sie es bedenken. Aber das mehrt in ihnen nur die Abneigung.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmlxxi{Sprich: Gäbe es neben Ihm noch (andere) Götter, wie sie sagen, dann würden sie nach einem Weg suchen, zum Herrn des Thrones zu gelangen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmlxxii{Preis sei Ihm, und überaus hoch erhaben ist Er über das, was sie sagen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmlxxiii{Ihn preisen die sieben Himmel und die Erde, und wer in ihnen ist. Es gibt nichts, was nicht sein Lob singen würde. Aber ihr begreift ihren Preisgesang nicht. Er ist langmütig und voller Vergebung.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmlxxiv{Und wenn du den Koran verliest, stellen Wir zwischen dich und die, die an das Jenseits nicht glauben, einen unsichtbaren Vorhang.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmlxxv{Und Wir legen auf ihre Herzen Hüllen, so daß sie ihn nicht begreifen, und in ihre Ohren Schwerhörigkeit. Und wenn du im Koran allein deinen Herrn erwähnst, kehren sie aus Abneigung den Rücken.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmlxxvi{Wir wissen besser, womit sie zuhören, wenn sie dir zuhören, und (auch) wenn sie sich in vertraulichem Gespräch befinden, da die, die Unrecht tun, sagen: «Ihr folgt doch nur einem Mann, der einem Zauber verfallen ist.»\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmlxxvii{Schau, wie sie dir Gleichnisse anführen, und so abgeirrt sind, daß sie keinen Weg mehr finden.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmlxxviii{Und sie sagen: «Sollen wir, wenn wir (bereits) Knochen geworden und auseinandergefallen sind, wirklich wieder als neue Schöpfung erweckt werden?»\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmlxxix{Sprich: «(Ja), ob ihr nun Steine oder Eisen seid,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmlxxx{Oder etwas Erschaffenes von der Art, die in eurer Vorstellung kaum möglich wäre.» Sie werden sagen: «Wer wird uns (ins Leben) zurückbringen?» Sprich: «Der, der euch das erste Mal erschaffen hat.» Dann werden sie vor dir den Kopf schütteln und sagen: «Wann wird das sein?» Sprich: «Vielleicht wird es bald sein.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmlxxxi{Am Tag, da Er euch rufen wird, und da ihr mit seinem Lob antworten und meinen werdet, ihr hättet nur ein wenig verweilt.»\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmlxxxii{Und sag meinen Dienern, sie sollen die besten Worte sprechen. Der Satan stachelt zwischen ihnen auf. Der Satan ist dem Menschen ein offenkundiger Feind.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmlxxxiii{Euer Herr weiß besser über euch Bescheid. Wenn Er will, erbarmt Er sich euer, und wenn Er will, peinigt Er euch. Und Wir haben dich nicht als Sachwalter über sie gesandt.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmlxxxiv{Und dein Herr weiß besser über die Bescheid, die in den Himmeln und auf der Erde sind. Und Wir haben einige der Propheten vor den anderen bevorzugt. Und Wir haben David eine Schrift zukommen lassen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmlxxxv{Sprich: Ruft die, die ihr anstelle Gottes angebt, an. Sie vermögen doch von euch den Schaden weder zu beheben noch abzuwenden.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmlxxxvi{Jene, die sie anrufen, suchen selbst ein Mittel, zu ihrem Herrn zu gelangen, (und wetteifern), wer von ihnen am nächsten sei; und sie hoffen auf seine Barmherzigkeit und haben Angst vor seiner Pein. Die Pein deines Herrn ist etwas, wovor man auf der Hut sein muß.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmlxxxvii{Und es gibt keine Stadt, die Wir nicht vor dem Tag der Auferstehung verderben oder mit einer harten Pein peinigen würden. Dies steht im Buch zeilenweise niedergeschrieben.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmlxxxviii{Und nichts anderes hinderte Uns daran, (Propheten) mit den Zeichen zu senden, als daß die Früheren sie für Lüge erklärten. Und Wir ließen den Thamud die Kamelstute als sichtbares Zeichen zukommen. Sie aber handelten an ihr ungerecht. Und Wir senden (die Propheten) mit den Zeichen nur zur Abschreckung.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmlxxxix{Und als Wir zu dir sprachen: «Dein Herr umfängt die Menschen.» Und Wir haben das Traumgesicht, das Wir dich sehen ließen, nur zu einer Versuchung für die Menschen gemacht, ebenso den verfluchten Baum im Koran. Wir machen ihnen angst, aber es steigert bei ihnen nur um so mehr das Übermaß ihres Frevels.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmxc{Und als Wir zu den Engeln sprachen: «Werft euch vor Adam nieder.» Da warfen sie sich nieder, außer Iblis. Er sagte: «Soll ich mich vor dem niederwerfen, den Du aus Ton erschaffen hast?»\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmxci{Er sagte: «Was meinst Du von diesem, dem Du mehr Ehre verliehen hast als mir? Wenn du mir bis zum Tag der Auferstehung Aufschub gewährst, werde ich seiner Nachkommenschaft die Zügel anlegen, bis auf wenige.»\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmxcii{Er sprach: «Geh weg. Wenn einer von ihnen dir folgt, so ist die Hölle euer Lohn, ein reichlicher Lohn.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmxciii{Und schrecke mit deiner Stimme auf, wen von ihnen du vermagst, und biete gegen sie deine Pferde und dein Fußvolk auf, und nimm von ihnen einen Anteil an Vermögen und Kindern, und mach ihnen Versprechungen.» - Der Satan verspricht ihnen nur Betörung. -\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmxciv{«Über meine Diener hast du keine Macht.» Und dein Herr genügt als Sachwalter.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmxcv{Euer Herr ist es, der für euch die Schiffe auf dem Meer treibt, damit ihr nach etwas von seiner Huld strebt. Er ist ja zu euch barmherzig.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmxcvi{Wenn euch auf dem Meer ein Schaden trifft, da irren die, die ihr außer Ihm anruft, (weit) weg. Hat Er euch dann ans Land errettet, wendet ihr euch ab. Der Mensch ist eben undankbar.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmxcvii{Wähnt ihr euch denn in Sicherheit davor, daß Er eine Ecke des Festlandes mit euch versinken läßt oder einen Sandsturm gegen euch schickt, und daß ihr dann für euch keinen Sachwalter findet?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmxcviii{Oder wähnt ihr euch in Sicherheit davor, daß Er euch ein anderes Mal aufs Meer zurückbringt und einen verheerenden Wind gegen euch schickt und euch ertrinken läßt, weil ihr ungläubig seid, und daß ihr dann für euch niemanden findet, der Uns deswegen belangen könnte?\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmxcix{Und Wir haben den Kindern Adams Ehre erwiesen; Wir haben sie auf dem Festland und auf dem Meer getragen und ihnen (einiges) von den köstlichen Dingen beschert, und Wir haben sie vor vielen von denen, die Wir erschaffen haben, eindeutig bevorzugt.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmc{Am Tag, da Wir jede Menschengruppe mit ihrem Vorsteher rufen werden. Diejenigen, denen dann ihr Buch in ihre Rechte gereicht wird, werden ihr Buch lesen, und ihnen wird nicht ein Dattelfädchen Unrecht getan.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmci{Und wer in diesem (Leben) blind ist, der ist (auch) im Jenseits blind und irrt noch mehr vom Weg ab.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcii{Fast hätten sie dich verführt (und) von dem (abgebracht), was Wir dir offenbart haben, damit du gegen Uns etwas anderes erdichtest. Dann hätten sie dich zum Vertrauten genommen.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmciii{Hätten Wir dich nicht gefestigt, du hättest wohl bei ihnen ein wenig Stütze gesucht.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmciv{Dann hätten Wir dich doppeltes Leben und doppelten Tod erleiden lassen, und dann würdest du für dich keinen Helfer gegen Uns finden.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcv{Und fast hätten sie dich aus dem Land aufgeschreckt, um dich daraus zu vertreiben. Dann würden sie nach dir nur ein wenig verweilen können.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcvi{Nach diesem Beispiel wurde verfahren mit denen von unseren Gesandten, die Wir vor dir gesandt haben. Und du wirst keine Veränderungen bei unserem Verfahren finden.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcvii{Verrichte das Gebet beim Neigen der Sonne bis zum Dunkel der Nacht, und (auch) die Koranlesung bei Tagesanbruch. Der Koranlesung bei Tagesanbruch soll man beiwohnen.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmcviii{Und wache einen Teil der Nacht damit, als zusätzliche Tat für dich. Dein Herr möge dich zu einer lobenswerten Rangstellung erwecken.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmcix{Und sprich: Mein Herr, gewähre mir einen wahrhaftigen Eingang und gewähre mir einen wahrhaftigen Ausgang, und schaffe mir von Dir her eine unterstützende Macht.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmcx{Und sprich: Die Wahrheit ist gekommen, und das Falsche schwindet dahin. Das Falsche schwindet ja schnell dahin.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmcxi{Und Wir senden vom Koran hinab, was den Gläubigen Heilung und Barmherzigkeit bringt; denen aber, die Unrecht tun, bringt es nur noch mehr Verlust.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmcxii{Wenn Wir dem Menschen Gnade erweisen, wendet er sich ab und entfernt sich beiseite. Und wenn ihn das Böse trifft, ist er sehr verzweifelt.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmcxiii{Sprich: Jeder handelt nach seiner Weise. Euer Herr weiß besser, wer dem richtigeren Weg folgt.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmcxiv{Und sie fragen dich nach dem Geist. Sprich: Der Geist ist vom Befehl meines Herrn. Und euch ist vom Wissen nur wenig zugekommen.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmcxv{Und wenn Wir wollten, würden Wir sicherlich wegnehmen, was Wir dir offenbart haben. Du könntest dann für dich in dieser Sache keinen Sachwalter gegen Uns finden.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmcxvi{Es ist nichts als Barmherzigkeit von deinem Herrn. Seine Huld zu dir ist ja groß.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmcxvii{Sprich: Wenn die Menschen und die Djinn zusammenkämen, um etwas beizubringen, was diesem Koran gleich wäre, sie brächten nicht seinesgleichen bei, auch wenn sie einander helfen würden.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmcxviii{Und Wir haben den Menschen in diesem Koran verschiedene Gleichnisse dargelegt. Doch bestehen die meisten Menschen auf dem Unglauben.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmcxix{Und sie sagen: «Wir werden dir nicht glauben, bis du uns aus der Erde eine Quelle hervorbrechen läßt,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmcxx{Oder bis du einen Garten von Palmen und Weinstöcken hast und durch ihn Bäche ausgiebig hervorbrechen läßt,\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmcxxi{Oder bis du den Himmel auf uns in Stücken herabfallen läßt, wie du behauptet hast, oder Gott und die Engel vor unsere Augen bringst,\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmcxxii{Oder bis du ein Haus aus Gold besitzt oder in den Himmel hochsteigst. Und wir werden nicht glauben, daß du hochgestiegen bist, bis du auf uns ein Buch herabsendest, das wir lesen können.» Sprich: Preis sei meinem Herrn! Bin ich etwas anderes als ein Mensch und ein Gesandter?\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmcxxiii{Und was die Menschen daran hinderte, als die Rechtleitung zu ihnen kam, zu glauben, ist nichts anderes, als daß sie sagten: «Hat denn Gott einen Menschen als Gesandten erstehen lassen?»\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmcxxiv{Sprich: Wenn es auf der Erde Engel gäbe, die da in Ruhe umhergingen, dann hätten Wir ihnen vom Himmel ja einen Engel als Gesandten hinabgeschickt.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmcxxv{Sprich: Gott genügt als Zeuge zwischen mir und euch. Er hat Kenntnis von seinen Dienern, und Er sieht sie wohl.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmcxxvi{Wen Gott rechtleitet, der ist es, der der Rechtleitung folgt. Und für die, die Er irreführt, wirst du außer Ihm keine Freunde finden. Und Wir werden sie am Tag der Auferstehung versammeln, (sie wandeln) vor sich hin, blind, stumm, taub. Ihre Heimstätte ist die Hölle. Sooft sie schwächer wird, mehren Wir ihnen den Feuerbrand.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmcxxvii{Das ist ihr Lohn dafür, daß sie unsere Zeichen verleugnet und gesagt haben: «Sollen wir, wenn wir (bereits) Knochen geworden und auseinandergefallen sind, wirklich wieder als neue Schöpfung auferweckt werden?»\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmcxxviii{Haben sie denn nicht gesehen, daß Gott, der die Himmel und die Erde erschaffen hat, die Macht hat, ihresgleichen zu erschaffen? Und Er hat ihnen eine Frist gesetzt, an der kein Zweifel möglich ist. Doch bestehen die, die Unrecht tun, auf dem Unglauben.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmcxxix{Sprich: Würdet ihr über die Vorratskammern der Barmherzigkeit meines Herrn verfügen, ihr würdet aus Furcht vor dem Ausgeben zurückhaltend sein. Der Mensch ist ja geizig.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmcxxx{Und Wir haben dem Mose neun deutliche Zeichen zukommen lassen. So frag die Kinder Israels. Als er zu ihnen kam, sagte Pharao zu ihm: «Ich bin der Meinung, o Mose, daß du einem Zauber verfallen bist.»\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmcxxxi{Er sagte: «Du weißt, niemand anderes als der Herr der Himmel und der Erde hat diese als Einsicht bringende Zeichen herabgesandt. Und ich bin der Meinung, o Pharao, daß du dem Verderben verfallen bist.»\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmcxxxii{Da wollte er sie aus dem Land aufschrecken. Wir aber ließen ihn ertrinken, und auch alle, die mit ihm waren.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmcxxxiii{Und nach seinem Ableben sprachen Wir zu den Kindern Israels: «Bewohnt das Land. Wenn dann das Versprechen des Jenseits eintrifft, bringen Wir euch als gebündelte Schar herbei.»\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmcxxxiv{Mit der Wahrheit haben Wir ihn hinabgesandt, und mit der Wahrheit ist er hinabgekommen. Und Wir haben dich nur als Freudenboten und Warner gesandt.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmcxxxv{Und Wir haben ihn als Koran in Abschnitte geteilt, damit du ihn den Menschen ohne Hast verliest. Und Wir haben ihn nach und nach hinabgesandt.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmcxxxvi{Sprich: Glaubt daran oder glaubt eben nicht. Diejenigen, denen vor ihm das Wissen zugekommen ist, wenn er ihnen verlesen wird, fallen in Anbetung auf ihr Kinn nieder\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmcxxxvii{Und sagen: «Preis sei unserem Herrn! Das Versprechen unseres Herrn ist ausgeführt.»\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmcxxxviii{Und sie werfen sich auf ihr Kinn weinend nieder, und es mehrt in ihnen die Demut.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmcxxxix{Sprich: Ruft Gott oder ruft den Erbarmer an. Welchen ihr auch anruft, Ihm gehören die schönsten Namen. Und sei nicht laut beim Gebet, und auch nicht leise dabei. Suche einen Weg dazwischen.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmcxl{Und sprich: Lob sei Gott, der sich kein Kind genommen hat, und der keinen Teilhaber an der Königsherrschaft hat und keinen Freund als Helfer aus der Erniedrigung! Und preise mit Nachdruck seine Größe.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmcxli{\basmalahde Lob sei Gott, der das Buch auf seinen Diener herabgesandt und daran nichts Krummes gemacht hat,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcxlii{ein Buch,) das richtig ist, damit er vor einem harten Schlag von Ihm her warne und den Gläubigen, die die guten Werke tun, verkünde, daß für sie ein schöner Lohn bestimmt ist -\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcxliii{Darin werden sie ewig verbleiben -,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmcxliv{Und damit er diejenigen warne, die sagen: «Gott hat sich ein Kind genommen.»\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcxlv{Sie haben kein (richtiges) Wissen davon, und auch nicht (früher) ihre Väter. Es ist ein ungeheuerliches Wort, das aus ihrem Munde herauskommt. Nichts als Lüge sagen sie da.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcxlvi{Vielleicht magst du, wenn sie an diese Botschaft nicht glauben, aus Gram noch dich selbst umbringen, nachdem sie sich abgewandt haben.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcxlvii{Wir haben das, was auf der Erde ist, zu einem Schmuck für sie gemacht, um sie zu prüfen (und festzustellen), wer von ihnen am besten handelt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcxlviii{Und Wir werden gewiß das, was auf ihr ist, zu einem dürren Boden machen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcxlix{Oder meinst du, daß die Leute der Höhle und der Inschrift ein (besonderes) Wunder unter unseren Zeichen sind?\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcl{Als die Jünglinge in der Höhle Unterkunft suchten und sagten: «Unser Herr, laß uns Barmherzigkeit von Dir zukommen und bereite uns aus unserer Angelegenheit einen guten Ausweg.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcli{Da schlugen Wir in der Höhle auf ihre Ohren für eine Anzahl von Jahren.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmclii{Dann erweckten Wir sie, um zu erfahren, welche von den beiden Parteien die Dauer ihres Verbleibens am richtigsten erfaßt hat.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcliii{Wir erzählen dir ihre Geschichte in Wahrheit. Sie waren Jünglinge, die an ihren Herrn glaubten und denen Wir noch mehr Rechtleitung schenkten.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcliv{Und Wir festigten ihre Herzen, als sie aufstanden und sagten: «Unser Herr ist der Herr der Himmel und der Erde. Wir werden niemals außer Ihm einen (anderen) Gott anrufen, sonst würden wir etwas Abwegiges sagen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmclv{Diese unsere Volksgenossen haben sich an seiner Stelle Götter genommen. Würden sie doch für sie eine offenkundige Ermächtigung bringen! Wer ist denn ungerechter als der, der gegen Gott eine Lüge erdichtet?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmclvi{Und so ihr euch von ihnen und von dem, was sie außer Gott anbeten, absetzt, dann sucht Unterkunft in der Höhle, so wird euer Herr über euch einiges von seiner Barmherzigkeit ausbreiten und euch in eurer Angelegenheit eine milde Behandlung bereiten.»\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmclvii{Und du siehst die Sonne, wenn sie aufgeht, sich von ihrer Höhle zur Rechten wegneigen, und wenn sie untergeht, an ihnen zur Linken vorbeigehen, während sie sich darin in einem Raum befinden. Das gehört zu den Zeichen Gottes. Wen Gott rechtleitet, der ist es, der der Rechtleitung folgt, und wen Er irreführt, für den wirst du keinen Freund finden, der ihn den rechten Weg weisen könnte.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmclviii{Und du meinst, sie seien wach, obwohl sie schlafen. Und Wir drehen sie nach rechts und nach links um. Und ihr Hund streckt seine Vorderbeine im Vorraum aus. Würdest du sie erblicken, du würdest vor ihnen zur Flucht kehrtmachen und du würdest vor ihnen mit Schrecken erfüllt sein.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmclix{Und so erweckten Wir sie, damit sie sich untereinander fragten. Ein Sprecher unter ihnen sagte: «Wie lange habt ihr verweilt?» Sie sagten: «Verweilt haben wir einen Tag oder einen Teil von einem Tag.» Sie sagten: «Euer Herr weiß besser, wie lange ihr verweilt habt. So schickt einen von euch mit diesen euren Silbermünzen in die Stadt; er soll sehen, welche ihrer Speisen am reinsten ist, und euch davon eine Versorgung bringen. Er soll behutsam sein und niemanden etwas von euch merken lassen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmclx{Denn wenn sie von euch erfahren, werden sie euch steinigen oder euch zur Rückkehr zu ihrer Glaubensrichtung zwingen; dann wird es euch nie mehr wohl ergehen.»\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmclxi{Auf diese Weise ließen Wir (die Menschen) sie doch entdecken, damit sie wissen, daß das Versprechen Gottes wahr ist und daß an der Stunde kein Zweifel möglich ist. Als sie untereinander über ihre Angelegenheit stritten, da sagten sie: «Baut über ihnen ein Gebäude.» Ihr Herr weiß besser über sie Bescheid. Diejenigen, die in ihrer Angelegenheit überlegen waren, sagten: «Wir werden uns über ihnen eine Anbetungsstätte einrichten.»\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmclxii{Manche werden sagen: «Es sind drei, und ihr Hund ist der vierte von ihnen.» Und manche sagen: «Fünf sind sie, und der sechste von ihnen ist ihr Hund.» - Ein Herumraten in bezug auf das Unsichtbare. Und manche sagen: «Sieben, und der achte von ihnen ist ihr Hund.» Sprich: Mein Herr weiß besser über ihre Zahl Bescheid. Und nur wenige wissen über sie Bescheid. Darum streite über sie nur aufgrund eines offensichtlichen Beweises, und frage niemanden von ihnen um Auskunft über sie.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmclxiii{Und sag nicht von einer Sache: «Ich werde dies morgen tun»,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmclxiv{Es sei denn (du fügst hinzu): «So Gott will.» Und gedenke deines Herrn, wenn du es vergessen hast, und sag: «Mein Herr möge mich zu etwas rechtleiten, was der richtigen Handlungsweise eher entspricht als dies!»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmclxv{Und sie verweilten in ihrer Höhle dreihundert Jahre und noch neun dazu.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmclxvi{Sprich: Gott weiß besser, wie lange sie verweilt haben. Ihm gehört das Unsichtbare der Himmel und der Erde. Wie vorzüglich sieht Er, und wie vorzüglich hört Er! Sie haben außer Ihm keinen Freund. Und Er beteiligt niemanden an seiner Urteilsgewalt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmclxvii{Und verlies, was dir vom Buch deines Herrn offenbart worden ist. Niemand wird seine Worte abändern können. Und du wirst außer Ihm keine Zuflucht finden.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmclxviii{Und halte dich geduldig zurück zusammen mit denen, die morgens und abends ihren Herrn anrufen, in der Suche nach seinem Antlitz. Und deine Augen sollen nicht über sie hinwegsehen, indem du nach dem Schmuck des diesseitigen Lebens trachtest. Und gehorche nicht dem, dessen Herz Wir unserem Gedenken gegenüber achtlos gemacht haben, der seiner Neigung folgt und dessen Angelegenheit sich durch Maßlosigkeit auszeichnet.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmclxix{Und sprich: Es ist die Wahrheit von eurem Herrn. Wer nun will, möge glauben, und wer will, möge ungläubig sein. Wir haben denen, die Unrecht tun, ein Feuer bereitet, dessen Zeltdecke sie umschließt. Und wenn sie um Hilfe rufen, wird ihnen mit Wasser gleich geschmolzenem Erz geholfen, das die Gesichter verbrennt. Welch schlimmer Trank und welch schlechter Lagerplatz!\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmclxx{Diejenigen, die glauben und die guten Werke tun - siehe, Wir lassen den Lohn derer, die in ihrem Handeln rechtschaffen sind, nicht verlorengehen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmclxxi{Jene sind es, für die die Gärten von Eden bestimmt sind. Unter ihnen fließen Bäche. Geschmückt sind sie darin mit Armringen aus Gold, und sie tragen grüne Gewänder aus Seide und Brokat, indem sie sich darin auf Liegen lehnen. Wie vorzüglich ist der Lohn und wie schön der Lagerplatz!\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmclxxii{Und führe ihnen als Gleichnis zwei Männer an. Dem einen von ihnen gaben Wir zwei Gärten mit Weinstöcken, und Wir umgaben sie mit Palmen und legten dazwischen Saatfelder an.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmclxxiii{Heide Gärten brachten ihren Ernteertrag und ließen nichts davon fehlen. Und dazwischen ließen Wir einen Bach hervorbrechen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmclxxiv{Er erhielt (daraus) Früchte. Da sagte er zu seinem Gefährten, während er sich mit ihm unterhielt: «Ich habe mehr Vermögen als du und auch eine stärkere Schar.»\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmclxxv{Und er betrat seinen Garten, indem er sich selbst Unrecht tat. Er sagte: «Ich glaube nicht, daß dieser (Garten) jemals verschwinden wird,\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmclxxvi{Und ich glaube nicht, daß die Stunde heraufkommen wird. Und wenn ich zu meinem Herrn zurückgebracht werde, werde ich sicher als Rückzugsort etwas Besseres als ihn finden.»\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmclxxvii{Sein Gefährte sagte zu ihm, während er sich mit ihm unterhielt: «Willst du denn den verleugnen, der dich aus Erde, dann aus einem Tropfen erschaffen und dann dich zu einem Mann gebildet hat?\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmclxxviii{Aber, was mich betrifft: Er, Gott, ist mein Herr, und ich geselle meinem Herrn niemanden bei.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmclxxix{Hättest du doch, als du deinen Garten betreten hast, gesagt: > (Es ist), was Gott will; es gibt keine Kraft außer durch Gott\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmclxxx{So möge mein Herr mir etwas Besseres als deinen Garten geben und über ihn aufeinanderfolgende Pfeile vom Himmel schicken, so daß er zu einem schlüpfrigen Boden wird,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmclxxxi{Oder daß sein Wasser versiegt, so daß du es nicht mehr wirst finden können.»\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmclxxxii{Seine Früchte wurden ringsum erfaßt. Da begann er, seine Handflächen umzudrehen über das, was er für ihn ausgegeben hatte, während er bis zu den Wipfeln verödet war, und zu sagen: «O hätte ich doch meinem Herrn niemanden beigesellt!»\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmclxxxiii{Und er hatte keine Schar, die ihn anstelle Gottes unterstützte, und er fand auch selbst keine Unterstützung.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmclxxxiv{In dem Fall gehört allein dem wahren Gott die Möglichkeit, Freundschaft zu gewähren. Er ist der Beste im Belohnen, und Er verschafft den besten Ausgang.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmclxxxv{Und führe ihnen das Gleichnis vom diesseitigen Leben an. Es ist wie Wasser, das Wir vom Himmel herabkommen lassen, worauf sich die Pflanzen der Erde damit vermengen. Dann werden sie zu dürrem Zeug, das die Winde verwehen. Und Gott hat ja Macht zu allen Dingen.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmclxxxvi{Das Vermögen und die Söhne sind der Schmuck des diesseitigen Lebens. Was aber bleibt, die guten Werke - sie bringen bei deinem Herrn einen besseren Lohn und begründen eine bessere Hoffnung.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmclxxxvii{Und an dem Tag, da Wir die Berge versetzen und du die Erde herauskommen siehst und Wir sie versammeln, ohne jemanden von ihnen auszulassen,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmclxxxviii{Und da sie deinem Herrn in einer Reihe vorgeführt werden: «Ihr seid zu Uns gekommen, so wie Wir euch das erste Mal erschaffen haben. Ihr aber habt behauptet, Wir würden für euch keine Verabredung festlegen.»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmclxxxix{Und das Buch wird hingelegt. Da siehst du die Übeltäter erschrocken vor dem, was darin steht. Und sie sagen: «O wehe uns! Was ist mit diesem Buch, daß es nichts, weder klein noch groß, ausläßt, ohne es zu erfassen?» Und sie finden (alles), was sie taten, vorliegen. Und dein Herr tut niemandem Unrecht.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcxc{Und als Wir zu den Engeln sprachen: «Werft euch vor Adam nieder.» Da warfen sie sich nieder, außer Iblis. Er gehörte zu den Djinn. So frevelte er gegen den Befehl seines Herrn. Wollt ihr denn ihn und seine Nachkommenschaft euch zu Freunden an meiner Stelle nehmen, wo sie euch doch Feind sind? Welch schlimmer Tausch für die, die Unrecht tun!\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcxci{Ich habe sie der Erschaffung der Himmel und der Erde nicht beiwohnen lassen, und auch nicht ihrer eigenen Erschaffung. Ich nehme mir niemals die Irreführenden als Beistand.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcxcii{Und an dem Tag, da Er spricht: «Ruft meine Teilhaber, die ihr angegeben habt, an.» Sie rufen sie an, aber sie erhören sie nicht. Und Wir setzen zwischen sie einen Abgrund.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcxciii{Und die Übeltäter sehen das Feuer und erwarten, daß sie hineinfallen, und finden kein Mittel, ihm zu entrinnen.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcxciv{Und Wir haben den Menschen in diesem Koran verschiedene Gleichnisse dargelegt. Von allen Wesen ist aber der Mensch am streitsüchtigsten.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcxcv{Und was die Menschen daran hinderte, zu glauben und ihren Herrn um Vergebung zu bitten, als die Rechtleitung zu ihnen kam, ist nichts anderes als (die anmaßende Forderung), daß an ihnen nach dem Beispiel der Früheren verfahren werde, oder daß die Pein über sie komme vor ihren Augen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcxcvi{Wir senden die Gesandten nur als Freudenboten und Warner. Diejenigen, die nicht glauben, streiten mit dem Falschen, um damit die Wahrheit zu widerlegen. Und sie nehmen meine Zeichen und das, womit sie gewarnt wurden, zum Gegenstand des Spottes.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcxcvii{Wer ist denn ungerechter als der, der mit den Zeichen seines Herrn ermahnt wird und sich dann von ihnen abwendet und vergißt, was seine Hände vorausgeschickt haben. Wir haben auf ihre Herzen Hüllen gelegt, so daß sie ihn nicht begreifen, und in ihre Ohren Schwerhörigkeit. Auch wenn du sie zur Rechtleitung rufst, werden sie niemals der Rechtleitung folgen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcxcviii{Und dein Herr ist voller Vergebung, und Er besitzt die Barmherzigkeit. Würde Er sie für das belangen, was sie erworben haben, würde Er für sie die Pein beschleunigen. Aber sie haben eine Verabredung, vor der sie keine Zuflucht finden werden.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcxcix{Das sind die Städte, die Wir verderben ließen, als sie Unrecht taten. Und Wir haben für ihr Verderben eine Verabredung festgelegt.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcc{Und als Mose zu seinem Knecht sagte: «Ich werde nicht ablassen, bis ich den Zusammenfluß der beiden Meere erreicht habe, und sollte ich lange Zeit unterwegs sein.»\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcci{Als sie deren Zusammenfluß erreicht hatten, vergaßen sie ihren Fisch, so nahm er seinen Weg ins Meer wie einen Tauchpfad.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmccii{Als sie vorbeigegangen waren, sagte er zu seinem Knecht: «Bring uns unser Mittagessen. Wir haben ja auf dieser unserer Reise (viel) Mühsal erlitten.»\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcciii{Er sagte: «Was meinst du? Als wir beim Felsen Rast gemacht haben, da habe ich den Fisch vergessen. Vergessen ließ mich ihn nur der Satan, so daß ich nicht mehr an ihn dachte. Und er nahm seinen Weg ins Meer auf wunderbare Weise.»\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcciv{Er sagte: «Das ist es, was wir suchten.» Da kehrten sie beide zurück und folgten ihren eigenen Spuren.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmccv{Sie trafen einen von unseren Dienern, dem Wir Barmherzigkeit von Uns hatten zukommen lassen und den Wir Wissen von Uns gelehrt hatten.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmccvi{Mose sagte zu ihm: «Darf ich dir folgen, auf daß du mich von dem lehrst, was du über den rechten Weg belehrt worden bist?»\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmccvii{Er sagte: «Nimmer wirst du bei mir aushalten können.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmccviii{Wie willst du das aushalten, wovon du keine umfassende Kenntnis hast?»\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmccix{Er sagte: «Du wirst finden, so Gott will, daß ich standhaft bin, und ich werde gegen keinen Befehl von dir ungehorsam sein.»\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmccx{Er sagte: «Wenn du mir folgst, dann frage mich nach nichts, bis ich selbst mit dir zuerst darüber rede.»\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmccxi{Da zogen sie beide weiter. Als sie nun das Schiff bestiegen, schlug er darin ein Loch. Er sagte: «Wie konntest du ein Loch darin schlagen, um seine Besatzung ertrinken zu lassen? Du hast da eine grauenhafte Sache begangen.»\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmccxii{Er sagte: «Habe ich nicht gesagt, daß du nimmer bei mir wirst aushalten können?»\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmccxiii{Er sagte: «Belange mich nicht dafür, daß ich vergessen habe, und bedrücke mich in meiner Angelegenheit nicht mit einer schweren Last.»\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmccxiv{Da zogen sie beide weiter. Als sie dann einen Jungen trafen, tötete er ihn. Er sagte: «Wie konntest du einen unschuldigen Menschen töten, und zwar nicht als Wiedervergeltung für einen (anderen) Menschen? Du hast da eine verwerfliche Sache begangen.»\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmccxv{Er sagte: «Habe ich nicht zu dir gesagt, daß du nimmer bei mir wirst aushalten können?»\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmccxvi{Er sagte: «Wenn ich dich nach diesem noch einmal nach irgend etwas frage, dann laß mich dich nicht mehr begleiten. Du hast dann von mir aus bereits eine Entschuldigung erhalten.»\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmccxvii{Da zogen sie beide weiter. Als sie dann zu den Bewohnern einer Stadt kamen, baten sie ihre Bewohner um etwas zu essen. Sie weigerten sich, sie zu bewirten. Da fanden sie in ihr eine Mauer, die einzustürzen drohte. Er richtete sie auf. Er sagte: «Wenn du gewollt hättest, hättest du dafür einen Lohn nehmen können.»\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmccxviii{Er sagte: «Jetzt ist die Trennung zwischen mir und dir fällig. Ich werde dir die Deutung dessen kundgeben, was du nicht aushalten konntest.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmccxix{Was das Schiff betrifft, so gehörte es armen Leuten, die auf dem Meer arbeiteten. Ich wollte es schadhaft machen, denn ein König war hinter ihnen her, der jedes Schiff mit Gewalt nahm.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmccxx{Was den Jungen betrifft, so waren seine Eltern gläubige (Menschen). Da fürchteten wir, er würde sie durch das Übermaß seines Frevels und durch seinen Unglauben bedrücken.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmccxxi{So wollten wir, daß ihr Herr ihnen einen zum Tausch gebe, der besser wäre als er in der Lauterkeit und anhänglicher in der Pietät.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmccxxii{Und was die Mauer betrifft, so gehörte sie zwei Waisenjungen in der Stadt. Unter ihr befand sich ein Schatz, der ihnen gehörte. Ihr Vater war rechtschaffen. Da wollte dein Herr, daß sie (erst) ihre Vollkraft erreichen und ihren Schatz herausholen, aus Barmherzigkeit von deinem Herrn. Ich tat es ja nicht aus eigenem Entschluß. Das ist die Deutung dessen, was du nicht aushalten konntest.»\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmccxxiii{Und sie fragen dich nach dem mit den zwei Hörnern. Sprich: Ich werde euch eine Geschichte über ihn verlesen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmccxxiv{Wir gaben ihm auf der Erde eine angesehene Stellung und eröffneten ihm in jedem Bereich einen Weg.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmccxxv{Da folgte er einem Weg.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmccxxvi{Als er nun den Ort des Sonnenuntergangs erreichte, fand er, daß sie in einer verschlammten Quelle unterging. Und er fand bei ihr ein Volk. Wir sprachen: «O du mit den zwei Hörnern, entweder peinigst du (sie), oder du entscheidest dich für Güte gegen sie.»\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmccxxvii{Er sagte: «Was den, der Unrecht tut, betrifft, so werden wir ihn peinigen, dann wird er zu seinem Herrn zurückgebracht, und Er wird ihn eine entsetzliche Pein erleiden lassen.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmccxxviii{Wer aber glaubt und Gutes tut, für den ist das Beste als Lohn bestimmt, und Wir werden ihm aus unserem Gebot etwas sagen, was Erleichterung bringt.»\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmccxxix{Dann folgte er einem Weg.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmccxxx{Als er nun den Ort des Sonnenaufgangs erreichte, fand er, daß sie über Leuten aufgeht, denen Wir keine Bedeckung vor ihr geschaffen hatten.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmccxxxi{So war es. Und Wir haben eine umfassende Kenntnis von dem, was ihn betrifft.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmccxxxii{Dann folgte er einem Weg.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmccxxxiii{Als er nun den Ort zwischen den beiden Sperrmauern erreichte, fand er neben ihnen Leute, die kaum ein Wort verstehen konnten.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmccxxxiv{Sie sagten: «O du mit den zwei Hörnern, Gog und Magog stiften Unheil auf der Erde. Sollen wir dir einen Tribut aussetzen, daß du zwischen uns und ihnen eine Sperrmauer errichtest?»\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmccxxxv{Er sagte: «Die angesehene Macht, die mir mein Herr verliehen hat, ist besser. Nun helft mir mit Kraft, so errichte ich zwischen euch und ihnen einen aufgeschütteten Wall.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmccxxxvi{Bringt mir die Eisenstücke.» Als er nun zwischen den beiden Berghängen gleich hoch aufgeschüttet hatte, sagte er: «Blast (jetzt).» Als er es zum Glühen gebracht hatte, sagte er: «Bringt mir (geschmolzenes) Kupfer, daß ich es darüber gieße.»\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmccxxxvii{So konnten sie ihn nicht überwinden, und sie konnten auch nicht darin eine Bresche schlagen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmccxxxviii{Er sagte: «Das ist eine Barmherzigkeit von meinem Herrn. Wenn dann das Versprechen meines Herrn eintrifft, macht Er ihn zu Staub. Und das Versprechen meines Herrn ist wahr.»\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmccxxxix{Und Wir lassen sie an jenem Tag durcheinander in Wallung geraten. Es wird in die Trompete geblasen. Dann versammeln Wir sie alle.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmccxl{Und Wir führen den Ungläubigen an jenem Tag die Hölle nachdrücklich vor,\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmccxli{ihnen), deren Augen vor meiner Ermahnung bedeckt waren, und die auch nicht hören konnten.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmccxlii{Meinen denn diejenigen, die ungläubig sind, daß sie sich meine Diener an meiner Stelle zu Freunden nehmen können? Wir haben den Ungläubigen die Hölle als Herberge bereitet.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmccxliii{Sprich: Sollen Wir euch die bekanntgeben, die an ihren Werken am meisten verlieren,\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmccxliv{Deren Mühen im diesseitigen Leben fehlgeht, während sie meinen, sie würden rechtschaffen handeln?\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmccxlv{Das sind diejenigen, die die Zeichen ihres Herrn und die Begegnung mit Ihm verleugnen. Ihre Werke sind wertlos, und Wir werden ihnen am Tag der Auferstehung kein Gewicht beimessen.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmccxlvi{Das ist ihr Lohn, die Hölle, dafür, daß sie ungläubig waren und meine Zeichen und meine Gesandten zum Gegenstand des Spottes genommen haben.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmccxlvii{Diejenigen, die glauben und die guten Werke tun, werden die Gärten des Paradieses zur Herberge haben,\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmccxlviii{Sie werden darin ewig weilen und sich nicht wünschen, davon wegzugehen.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmccxlix{Sprich: Wenn das Meer Tinte für die Worte meines Herrn wäre, würde das Meer zu Ende gehen, bevor die Worte meines Herrn zu Ende gehen, auch wenn Wir noch einmal soviel hinzubrächten.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmccl{Sprich: Ich bin ja nur ein Mensch wie ihr; mir wird offenbart, daß euer Gott ein einziger Gott ist. Wer nun auf die Begegnung mit seinem Herrn hofft, der soll gute Werke tun und bei der Anbetung seines Herrn (Ihm) niemanden beigesellen.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmccli{\basmalahde Kaaf Haa Yaa Ayn Saad.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcclii{Zum Gedenken an die Barmherzigkeit deines Herrn zu seinem Diener Zakaria.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmccliii{Als er seinen Herrn im Verborgenen anrief.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmccliv{Er sagte: «Mein Herr, schwach ist mir das Gebein geworden und altersgrau das Haupt. Und ich war gewiß im Rufen zu Dir, mein Herr, nicht unglücklich.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcclv{Ich fürchte die Verwandten nach mir, und meine Frau ist unfruchtbar. So schenke mir von Dir einen nahen Verwandten,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcclvi{Der mich beerbt und von der Sippe Jakobs erbt, und mach ihn, mein Herr, (Dir) wohlgefällig.»\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcclvii{«O Zakaria, Wir verkünden dir einen Knaben, sein Name ist Yahyaa, wie Wir zuvor noch niemanden gleich ihm genannt haben.»\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcclviii{Er sagte: «Mein Herr, wie soll ich einen Knaben haben, wo meine Frau unfruchtbar ist und ich vom hohen Alter einen Dürrezustand erreicht habe?»\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcclix{Er sprach: «So wird es sein. Dein Herr spricht: Das ist Mir ein leichtes. Auch dich habe Ich vorher erschaffen, als du noch nichts warst.»\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcclx{Er sagte: «Mein Herr, setze mir ein Zeichen.» Er sprach: «Dein Zeichen ist, daß du, obwohl gesund, drei Nächte lang nicht zu den Menschen sprechen wirst.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcclxi{Er kam zu seinen Leuten aus dem Tempel heraus und bedeutete ihnen: «Preiset morgens und abends.»\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmcclxii{«O Yahyaa, nimm das Buch mit aller Kraft.» Und Wir ließen ihm noch als Kind die Urteilskraft zukommen,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcclxiii{Und Mitgefühl von Uns und Lauterkeit. Er war gottesfürchtig\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcclxiv{Und pietätvoll gegen seine Eltern, er war aber nicht ein widerspenstiger Gewaltherrscher.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmcclxv{Und Friede sei über ihm am Tag, da er geboren wurde, und am Tag, da er stirbt, und am Tag, da er wieder zum Leben erweckt wird.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmcclxvi{Und gedenke im Buch der Maria, als sie sich von ihren Angehörigen an einen östlichen Ort zurückzog.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmcclxvii{Sie nahm sich einen Vorhang vor ihnen. Da sandten Wir unseren Geist zu ihr. Er erschien ihr im Bildnis eines wohlgestalteten Menschen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmcclxviii{Sie sagte: «Ich suche beim Erbarmer Zuflucht vor dir, so du gottesfürchtig bist.»\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmcclxix{Er sagte: «Ich bin der Bote deines Herrn, um dir einen lauteren Knaben zu schenken.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmcclxx{Sie sagte: «Wie soll ich einen Knaben bekommen? Es hat mich doch kein Mensch berührt, und ich bin keine Hure.»\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmcclxxi{Er sagte: «So wird es sein. Dein Herr spricht: Das ist Mir ein leichtes. Wir wollen ihn zu einem Zeichen für die Menschen und zu einer Barmherzigkeit von Uns machen. Und es ist eine beschlossene Sache.»\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmcclxxii{So empfing sie ihn. Und sie zog sich mit ihm zu einem entlegenen Ort zurück.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmcclxxiii{Die Wehen ließen sie zum Stamm der Palme gehen. Sie sagte: «O wäre ich doch vorher gestorben und ganz und gar in Vergessenheit geraten!»\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmcclxxiv{Da rief er ihr von unten her zu: «Sei nicht betrübt. Dein Herr hat unter dir Wasser fließen lassen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmcclxxv{Und schüttle den Stamm der Palme gegen dich, so läßt sie frische, reife Datteln auf dich herunterfallen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmcclxxvi{Dann iß und trink und sei frohen Mutes. Und wenn du jemanden von den Menschen siehst, dann sag: Ich habe dem Erbarmer ein Fasten gelobt, so werde ich heute mit keinem Menschen reden.»\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmcclxxvii{Dann kam sie mit ihm zu ihrem Volk, indem sie ihn trug. Sie sagten: «O Maria, du hast eine unerhörte Sache begangen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmcclxxviii{O Schwester Aarons, nicht war dein Vater ein schlechter Mann, und nicht war deine Mutter eine Hure.»\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmcclxxix{Sie zeigte auf ihn. Sie sagten: «Wie können wir mit dem reden, der noch ein Kind in der Wiege ist?»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmcclxxx{Er sagte: «Ich bin der Diener Gottes. Er ließ mir das Buch zukommen und machte mich zu einem Propheten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmcclxxxi{Und Er machte mich gesegnet, wo immer ich bin. Und Er trug mir auf, das Gebet und die Abgabe (zu erfüllen), solange ich lebe,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmcclxxxii{Und pietätvoll gegen meine Mutter zu sein. Und Er machte mich nicht zu einem unglückseligen Gewaltherrscher.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmcclxxxiii{Und Friede sei über mir am Tag, da ich geboren wurde, und am Tag, da ich sterbe, und am Tag, da ich wieder zum Leben erweckt werde.»\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmcclxxxiv{Das ist Jesus, der Sohn Marias. Es ist das Wort der Wahrheit, woran sie zweifeln.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmcclxxxv{Es steht Gott nicht an, sich ein Kind zu nehmen. Preis sei Ihm! Wenn Er eine Sache beschlossen hat, sagt Er zu ihr: Sei!, und sie ist.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmcclxxxvi{«Und Gott ist mein Herr und euer Herr; so dienet Ihm. Das ist ein gerader Weg.»\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmcclxxxvii{Dann wurden die Parteien untereinander uneins. Wehe denen, die nicht glauben, vor dem Erleben eines gewaltigen Tages!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmcclxxxviii{O wie gut werden sie hören und schauen am Tag, an dem sie zu Uns kommen werden! Aber die, die Unrecht tun, befinden sich heute in einem offenkundigen Irrtum.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmcclxxxix{Warne sie vor dem Tag des Bedauerns, wenn die Angelegenheit entschieden sein wird, während sie alles unbeachtet lassen, und während sie (noch) ungläubig sind.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmccxc{Wir werden gewiß die Erde und alle, die auf ihr sind, erben. Und zu Uns werden sie zurückgebracht.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmccxci{Und gedenke im Buch des Abraham. Er war ein Wahrhaftiger und ein Prophet.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmccxcii{Als er zu seinem Vater sagte: «O mein Vater, warum verehrst du das, was nicht hört und nicht sieht und dir nichts nützt?\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmccxciii{O mein Vater, zu mir ist vom Wissen gekommen, was nicht zu dir gekommen ist. So folge mir, dann führe ich dich einen ebenen Weg.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmccxciv{O mein Vater, diene nicht, dem Satan. Der Satan ist gegen den Erbarmer widerspenstig.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmccxcv{O mein Vater, ich fürchte, daß dich vom Erbarmer eine Pein erfaßt, und daß du so zum Freund des Satans wirst.»\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmccxcvi{Er sagte: «Verschmähst du meine Götter, o Abraham? Wenn du nicht aufhörst, werde ich dich bestimmt steinigen. Und entferne dich von mir auf lange Zeit.»\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmccxcvii{Er sagte: «Friede sei über dir. Ich werde meinen Herrn für dich um Vergebung bitten. Er ist zu mir sehr entgegenkommend.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmccxcviii{Ich sondere mich von euch ab und von dem, was ihr anstelle Gottes anruft. Ich rufe meinen Herrn an. Möge ich im Rufen zu meinem Herrn nicht unglücklich werden!»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmccxcix{Als er sich von ihnen und von dem, was sie anstelle Gottes verehrten, abgesondert hatte, schenkten Wir ihm Isaak und Jakob; und beide machten Wir zu Propheten.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmccc{Und Wir schenkten ihnen etwas von unserer Barmherzigkeit, und Wir gaben ihnen einen hohen wahrhaftigen Ruf.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmccci{Und gedenke im Buch des Mose. Er war auserwählt, und er war ein Gesandter und Prophet.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcccii{Und Wir riefen ihn von der rechten Seite des Berges und ließen ihn zu vertraulichem Gespräch nähertreten.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmccciii{Und Wir schenkten ihm in unserer Barmherzigkeit seinen Bruder Aaron als Propheten.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmccciv{Und gedenke im Buch des Ismael. Er war treu zu seinem Versprechen, und er war ein Gesandter und Prophet.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcccv{Und er befahl seinen Angehörigen das Gebet und die Abgabe, und er war seinem Herrn wohlgefällig.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcccvi{Und gedenke im Buch des Idris. Er war ein Wahrhaftiger und Prophet.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcccvii{Und Wir erhoben ihn an einen hohen Ort.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcccviii{Das sind die unter den Propheten, die Gott begnadet hat, aus der Nachkommenschaft Adams und von denen, die Wir mit Noach getragen haben, und aus der Nachkommenschaft Abrahams und Israels, und von denen, die Wir rechtgeleitet und erwählt haben. Wenn ihnen die Zeichen des Erbarmers verlesen werden, fallen sie anbetend und weinend nieder.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcccix{Es folgten dann nach ihnen Nachfolger, die das Gebet vernachlässigten und den Begierden nachgingen. So werden sie den Untergang finden,\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcccx{Außer denen, die umkehren und glauben und Gutes tun. Diese gehen ins Paradies ein - und ihnen wird in nichts Unrecht getan -,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcccxi{In die Gärten von Eden, die der Erbarmer seinen Dienern im Bereich des Unsichtbaren versprochen hat. Sein Versprechen wird bestimmt herbeigeführt.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcccxii{Sie hören darin keine unbedachte Rede, sondern nur: «Frieden!» Und sie haben darin ihren Unterhalt morgens und abends.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcccxiii{Das ist das Paradies, das Wir denen von unseren Dienern zum Erbe geben, die gottesfürchtig sind.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcccxiv{Und wir kommen nur auf Befehl deines Herrn herab. Ihm gehört, was vor uns und was hinter uns und was dazwischen liegt. Und dein Herr ist gewiß nicht vergeßlich,\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcccxv{Er), der Herr der Himmel und der Erde und dessen, was zwischen ihnen ist. So diene Ihm und sei beharrlich in seinem Dienst. Weißt du etwa einen, der Ihm namensgleich wäre?\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcccxvi{Der Mensch sagt: «Werde ich, wenn ich gestorben bin, (wirklich) lebendig hervorgebracht werden?»\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcccxvii{Bedenkt der Mensch denn nicht, daß Wir ihn zuvor erschaffen haben, da er nichts war?\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmcccxviii{Bei deinem Herrn, Wir werden sie samt den Satanen versammeln. Dann werden Wir sie rings um die Hölle auf den Knien vortreten lassen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmcccxix{Dann werden Wir aus jeder Partei die herausgreifen, die gegen den Erbarmer am heftigsten rebelliert haben.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmcccxx{Dann wissen gerade Wir am besten über die Bescheid, die es am ehesten verdienen, darin zu brennen.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmcccxxi{Und es gibt keinen unter euch, der nicht dahin wie zur Tränke gehen würde? Dies obliegt deinem Herrn als eine unabwendbare beschlossene Sache.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmcccxxii{Dann erretten Wir diejenigen, die gottesfürchtig waren, und lassen die, die Unrecht tun, in ihr auf den Knien zurück.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcccxxiii{Und wenn ihnen unsere Zeichen als deutliche Beweise verlesen werden, sagen diejenigen, die ungläubig sind, zu denen, die glauben: «Welche von den beiden Gruppen hat eine bessere Stellung und eine schönere Gesellschaft?»\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmcccxxiv{Wie viele Generationen haben Wir vor ihnen verderben lassen, die sie in Ausstattung und Aussehen übertrafen!\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmcccxxv{Sprich: Den, der sich im Irrtum befindet, möge der Erbarmer lange darin gewähren lassen! Wenn sie schließlich das sehen, was ihnen angedroht ist, die Pein oder die Stunde, da werden sie wissen, wer die schlimmere Stätte und die schwächere Heerschar hat.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcccxxvi{Und Gott läßt diejenigen, die der Rechtleitung folgen, in der Rechtleitung zunehmen. Was bleibt, die guten Werke - sie bringen bei deinem Herrn einen besseren Lohn und sichern einen besseren Ort der Rückkehr.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcccxxvii{Hast du den gesehen, der unsere Zeichen verleugnet und sagt: «Mir werden gewiß Vermögen und Kinder zuteil»?\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcccxxviii{Hat er denn Kenntnis vom Unsichtbaren erworben, oder hat er vom Erbarmer ein verbindliches Versprechen erhalten?\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmcccxxix{Nein, Wir werden aufschreiben, was er sagt, und ihm die Pein noch verlängern.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmcccxxx{Und Wir erben von ihm das, wovon er spricht. Und allein kommt er zu Uns.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmcccxxxi{Und sie haben sich anstelle Gottes Götter genommen, damit sie ihnen zum Stolz gereichen.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmcccxxxii{Nein, sie werden deren Verehrung verleugnen und werden ihnen Widersacher sein.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmcccxxxiii{Hast du nicht gesehen, daß Wir die Satane gegen die Ungläubigen gesandt haben, damit sie sie heftig aufstacheln?\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmcccxxxiv{Nichts sollst du gegen sie zu beschleunigen wünschen. Wir zählen ihnen schon (die Tage) ab.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmcccxxxv{Am Tag, da Wir die Gottesfürchtigen wie eine Reiterabordnung zum Erbarmer versammeln,\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmcccxxxvi{Und die Übeltäter zur Hölle wie eine Herde zur Tränke treiben,\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmcccxxxvii{Verfügen sie über keine Fürsprache, außer dem, der vom Erbarmer ein verbindliches Versprechen erhalten hat.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmcccxxxviii{Und sie sagen: «Der Erbarmer hat sich ein Kind genommen.»\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmcccxxxix{Ihr habt da eine ungeheuerliche Sache begangen.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmcccxl{Die Himmel brechen bald auseinander, und die Erde spaltet sich, und die Berge stürzen in Trümmern darüber,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmcccxli{Daß sie dem Erbarmer ein Kind zuschreiben.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmcccxlii{Es ziemt doch dem Erbarmer nicht, sich ein Kind zu nehmen.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmcccxliii{Niemand in den Himmeln und auf der Erde wird zum Erbarmer anders denn als Diener kommen können.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmcccxliv{Er hat sie erfaßt und genau aufgezählt.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmcccxlv{Und sie alle werden zu Ihm am Tag der Auferstehung allein kommen.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmcccxlvi{Denen, die glauben und die guten Werke tun, wird der Erbarmer Liebe bereiten.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmcccxlvii{Wir haben ihn leicht gemacht in deiner Sprache, damit du durch ihn den Gottesfürchtigen frohe Botschaft verkündest und durch ihn streitsüchtige Leute warnst.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmcccxlviii{Und wie viele Generationen haben Wir vor ihnen verderben lassen! Nimmst du noch irgendeinen von ihnen wahr, oder hörst du von ihnen noch den geringsten Laut?\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmcccxlix{\basmalahde Taa Haa.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcccl{Wir haben den Koran nicht auf dich hinabgesandt, damit du unglücklich bist,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcccli{Sondern als Erinnerung für den, der gottesfürchtig ist,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmccclii{Herabgesandt von dem, der die Erde und die erhabenen Himmel erschaffen hat.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcccliii{Der Erbarmer hat sich auf dem Thron zurechtgesetzt.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcccliv{Ihm gehört, was in den Himmeln und was auf der Erde ist, und was zwischen ihnen und was unter dem Erdboden ist.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmccclv{Und wenn du deine Worte laut äußerst, so weiß Er ja das Geheimnis, und was noch verborgener ist.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmccclvi{Gott, es gibt keinen Gott außer Ihm. Ihm gehören die schönsten Namen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmccclvii{Ist denn die Geschichte von Mose zu dir gelangt?\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmccclviii{Als er ein Feuer sah und zu seinen Angehörigen sagte: «Bleibt hier. Ich habe ein Feuer wahrgenommen, vielleicht kann ich euch davon eine Fackel bringen oder beim Feuer eine Wegweisung finden.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmccclix{Als er dort ankam, wurde ihm zugerufen: «O Mose!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmccclx{Siehe, Ich bin dein Herr, so ziehe deine Sandalen aus. Du befindest dich ja im heiligen Tal Tuwaa.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmccclxi{Ja, Ich habe dich erwählt. So höre auf das, was offenbart wird.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmccclxii{Siehe, Ich bin Gott. Es gibt keinen Gott außer Mir. So diene Mir und verrichte das Gebet zu meinem Gedächtnis.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmccclxiii{Wahrlich, die Stunde kommt - Ich halte sie fast verborgen -, damit jeder Seele vergolten wird für das, worum sie sich bemüht.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmccclxiv{So laß nicht den, der nicht an sie glaubt und seiner Neigung folgt, dich von ihr abweisen, sonst würdest du zugrunde gehen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmccclxv{Und was ist das da in deiner Rechten, o Mose?»\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmccclxvi{Er sagte: «Das ist mein Stab. Darauf stütze ich mich, und damit schlage ich für meine Schafe Blätter ab, und ich gebrauche ihn auch sonst zu anderen Zwecken.»\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmccclxvii{Er sprach: «Wirf ihn hin, o Mose!»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmccclxviii{Er warf ihn hin, da war er eine Schlange, die lief.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmccclxix{Er sprach: «Nimm sie, und hab keine Angst. Wir werden sie zu ihrem ursprünglichen Zustand zurückbringen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmccclxx{Und lege deine Hand dicht an deine Seite, so kommt sie weiß, jedoch nicht von Übel befallen, heraus. (Dies) als weiteres Zeichen,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmccclxxi{Auf das Wir dich etwas von unseren größten Zeichen sehen lassen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmccclxxii{Geh zu Pharao, er zeigt ein Übermaß an Frevel.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmccclxxiii{Er sagte: «Mein Herr, weite mir meine Brust,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmccclxxiv{Und mach mir meine Angelegenheit leicht.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmccclxxv{Und löse einen Knoten von meiner Zunge,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmccclxxvi{So daß sie meine Worte begreifen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmccclxxvii{Und bestelle mir aus den Reihen meiner Angehörigen einen, der (die Last) mitträgt,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmccclxxviii{Aaron, meinen Bruder.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmccclxxix{Festige durch ihn meine Kraft,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmccclxxx{Und laß ihn an meiner Angelegenheit teilhaben,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmccclxxxi{Damit wir Dich viel preisen\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmccclxxxii{Und Deiner viel gedenken.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmccclxxxiii{Du siehst uns ja wohl.»\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmccclxxxiv{Er sprach: «Deine Bitte ist dir gewährt, o Mose!\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmccclxxxv{Und Wir haben dir (bereits) ein anderes Mal eine Wohltat erwiesen,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmccclxxxvi{Als Wir deiner Mutter eingegeben haben, was eingegeben werden sollte:\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmccclxxxvii{>Leg ihn in den Kasten und wirf ihn ins Meer, und das Meer soll ihn ans Ufer legen, so daß ihn ein Feind von Mir und Feind von ihm aufnimmt < Und Ich habe über dich eine Liebe von Mir gelegt, ja, auf daß du vor meinem Auge aufgezogen wirst.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmccclxxxviii{Als deine Schwester herging und sagte: >Soll ich euch auf jemanden hinweisen, der ihn betreuen würde? < So haben Wir dich zu deiner Mutter zurückkehren lassen, damit sie frohen Mutes und nicht betrübt sei. Und du tötetest einen Menschen. Da erretteten Wir dich aus dem Kummer, und Wir unterzogen dich einer harten Versuchung. So verweiltest du jahrelang unter den Leuten von Madyan. Dann kamst du, o Mose, zu einer vorausbestimmten Zeit.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmccclxxxix{Und Ich habe dich für Mich aufgezogen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmcccxc{Geh du und dein Bruder mit meinen Zeichen, und seid nicht nachlässig in meinem Gedenken.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmcccxci{Geht zu Pharao, er zeigt ja ein Übermaß an Frevel.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmcccxcii{So sprecht zu ihm in sanfter Rede, vielleicht bedenkt er es, oder er fürchtet sich.»\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmcccxciii{Sie sagten: «Unser Herr, wir fürchten, daß er übereilig gegen uns vorgeht oder daß er ein Übermaß an Frevel zeigt.»\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmcccxciv{Er sprach: «Fürchtet euch nicht. Ich bin mit euch und höre und sehe, (was geschieht).\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmcccxcv{So geht zu ihm und sagt: >Wir sind Gesandte deines Herrn. Schick die Kinder Israels mit uns weg, und peinige sie nicht. Wir sind zu dir mit einem Zeichen von deinem Herrn gekommen. Und Friede sei auf dem, der der Rechtleitung folgt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmcccxcvi{Uns ist ja offenbart worden, daß die Pein den überkommt, der (die Botschaft) für Lüge erklärt und sich abkehrt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmcccxcvii{Er sagte: «Wer ist denn euer Herr, o Mose?»\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcccxcviii{Er sagte: «Unser Herr ist der, der jedem Ding seine Natur gegeben und dann die Rechtleitung gebracht hat.»\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcccxcix{Er sagte: «Wie steht es denn mit den früheren Generationen?»\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcd{Er sagte: «Über sie weiß mein Herr Bescheid, es steht in einem Buch. Mein Herr irrt nicht, und Er vergißt nicht.»\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcdi{Er, der euch die Erde zu einer Lagerstätte gemacht und euch auf ihr Wege geebnet hat und Wasser vom Himmel hat herabkommen lassen. So haben Wir dadurch Arten verschiedener Pflanzen hervorgebracht.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcdii{Eßt und weidet euer Vieh. Darin sind Zeichen für Leute, die Vernunft haben.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcdiii{Aus ihr haben Wir euch erschaffen, und in sie lassen Wir euch zurückkehren, und aus ihr bringen Wir euch ein anderes Mal hervor.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcdiv{Und Wir ließen ihn all unsere Zeichen schauen, aber er erklärte (sie) für Lüge und weigerte sich.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcdv{Er sagte: «Bist du zu uns gekommen, um uns mit deiner Zauberei aus unserem Land zu vertreiben, o Mose?\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcdvi{Wir werden dir gewiß mit einer gleichen Zauberei kommen. So setze zwischen uns und dir eine Verabredungszeit fest, die weder wir noch du versäumen werden, an einem Ort, der (uns allen) gleichermaßen recht ist.»\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcdvii{Er sagte: «Eure Verabredungszeit soll am Tag des Schmuckfestes sein, und die Menschen sollen am späten Vormittag versammelt sein.»\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcdviii{Pharao kehrte sich ab. Da nahm er seine ganze List zusammen, und dann kam er.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcdix{Mose sagte zu ihnen: «Wehe euch! Erdichtet gegen Gott keine Lüge, daß Er euch nicht durch eine Pein vertilgt. Enttäuscht wird ja, wer Lügen erdichtet.»\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcdx{Da stritten sie untereinander über ihre Angelegenheit und führten insgeheim vertrauliche Gespräche.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcdxi{Sie sagten: «Diese beiden sind gewiß Zauberer, die euch mit ihrem Zauber aus eurem Land vertreiben und euren vorbildlichen Weg beseitigen wollen.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcdxii{So nehmt einmütig eure ganze List zusammen, dann kommt in einer Reihe. Heute wird es dem wohl ergehen, der überlegen ist.»\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcdxiii{Sie sagten: «O Mose, entweder wirfst du, oder wir sind es, die zuerst werfen.»\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcdxiv{Er sagte: «Nein, werft ihr (zuerst).» Und siehe da, ihre Stricke und Stäbe - es kam ihm ja durch ihre Zauberei vor, als ob sie liefen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcdxv{Und er, Mose, empfand in seiner Seele Furcht.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmcdxvi{Wir sprachen: «Fürchte dich nicht. Du, ja du wirst überlegen sein.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmcdxvii{Wirf, was in deiner Rechten ist, so verschlingt es das, was sie gemacht haben. Was sie gemacht haben, ist nur die List eines Zauberers, und dem Zauberer ergeht es nicht wohl, woher er auch kommen mag.»\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmcdxviii{Und die Zauberer wurden in Anbetung zu Boden geworfen. Sie sagten: «Wir glauben an den Herrn Aarons und Moses.»\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmcdxix{Er sagte: «Ihr glaubt an ihn, bevor ich es euch erlaube? Er ist bestimmt euer Ranghöchster, der euch die Zauberei gelehrt hat. Ich werde eure Hände und Füße wechselseitig abhacken, und ich werde euch an Palmstämmen kreuzigen lassen. Und ihr werdet sicher erfahren, wer von uns eine härtere und nachhaltigere Pein verhängt.»\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmcdxx{Sie sagten: «Wir werden dir nicht den Vorzug geben vor dem, was an deutlichen Zeichen zu uns gekommen ist, und (vor) dem, der uns erschaffen hat. So entscheide, was du entscheiden magst. Du entscheidest nur über dieses irdische Leben.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcdxxi{Wir glauben an unseren Herrn, damit Er uns unsere Verfehlungen vergebe und auch die Zauberei, zu der du uns gezwungen hast. Gott ist besser und hat eher Bestand.»\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmcdxxii{Siehe, wer als Übeltäter zu seinem Herrn kommt, erhält die Hölle; darin wird er weder sterben noch leben.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmcdxxiii{Diejenigen, die zu Ihm als Gläubige kommen, welche die guten Werke getan haben, erhalten die höchsten Rangstufen,\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcdxxiv{Die Gärten von Eden, unter denen die Bäche fließen; darin werden sie ewig weilen. Das ist der Lohn derer, die sich läutern.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcdxxv{Und Wir haben dem Mose offenbart: «Zieh bei Nacht mit meinen Dienern fort und schlag ihnen einen trockenen Weg durch das Meer, so daß du nicht Angst davor haben mußt, eingeholt zu werden, und du nichts zu befürchten brauchst.»\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcdxxvi{Pharao verfolgte sie mit seinen Truppen, und es überdeckte sie vom Meer, was sie überdeckte.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmcdxxvii{Pharao hat sein Volk irregeführt und nicht rechtgeleitet.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmcdxxviii{O Kinder Israels, Wir retteten euch vor eurem Feind, Wir verabredeten Uns mit euch auf der rechten Seite des Berges, und Wir sandten das Manna und die Wachteln auf euch hinab:\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmcdxxix{«Eßt von den köstlichen Dingen dessen, was Wir euch beschert haben, und zeigt nicht darin ein Übermaß an Frevel, sonst läßt sich mein Zorn auf euch nieder. Der, auf den sich mein Zorn niederläßt, wird stürzen.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmcdxxx{Doch siehe, Ich bin voller Vergebung für den, der umkehrt und glaubt und Gutes tut und dann der Rechtleitung folgt.»\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmcdxxxi{«Und was veranlaßt dich, von deinem Volk fortzueilen, o Mose?»\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmcdxxxii{Er sagte: «Siehe, sie folgen mir auf der Spur. Und ich bin zu dir geeilt, mein Herr, damit Du (an mir) Wohlgefallen hast.»\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmcdxxxiii{Er sprach: «Wir haben dein Volk, nachdem du weggegangen warst, einer Versuchung ausgesetzt, und der Samiri hat sie irregeführt.»\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmcdxxxiv{Da kam Mose zu seinem Volk zornig und voller Bedauern. Er sagte: «O mein Volk, hat euch euer Herr nicht ein schönes Versprechen gegeben? Hat es euch mit dem Bund zu lange gedauert, oder wolltet ihr, daß sich ein Zorn von eurem Herrn auf euch niederläßt, daß ihr die Vereinbarung mit mir gebrochen habt?»\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmcdxxxv{Sie sagten: «Nicht aus freier Entscheidung haben wir die Vereinbarung mit dir gebrochen, sondern wir hatten ganze Lasten von den Schmucksachen der Leute zu tragen. Da haben wir sie hineingeworfen. Ebenso hat (sie) der Samiri hineingelegt.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmcdxxxvi{So brachte er ihnen ein Kalb hervor als Leib, der blökte. Sie sagten: >Das ist euer Gott und der Gott Moses. Aber er hat vergessene»\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmcdxxxvii{Sehen sie denn nicht, daß er ihnen kein Wort erwidert und ihnen weder Schaden noch Nutzen bringen kann?\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmcdxxxviii{Aaron hatte ihnen doch vorher gesagt: «O mein Volk, ihr seid damit einer Versuchung ausgesetzt worden. Und euer Herr ist der Erbarmer. So folgt mir und gehorcht meinem Befehl.»\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmcdxxxix{Sie sagten: «Wir werden nicht aufhören, ihn zu verehren, bis Mose zu uns zurückkehrt.»\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmcdxl{Er sagte: «O Aaron, was hat dich, als du sie irregehen sahst, daran gehindert,\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmcdxli{Mir zu folgen? Bist du denn gegen meinen Befehl ungehorsam gewesen?»\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmcdxlii{Er sagte: «O Sohn meiner Mutter, pack mich nicht am Bart und nicht am Kopf. Ich fürchtete, du würdest sagen: Du hast unter den Kindern Israels Zwietracht gestiftet und du hast mein Wort nicht beachtet.»\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmcdxliii{Er sagte: «Und was ist mit dir, o Samiri?»\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmcdxliv{Er sagte: «Ich habe erblickt, was sie nicht erblickt haben. So habe ich eine Handvoll Erde von der Spur des Gesandten gefaßt und sie dann hingeworfen. Auf diese Weise hat es mir meine Seele eingeredet.»\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmcdxlv{Er sagte: «Geh weg. Dein Los im Leben ist, zu sagen: >Ihr dürft mich nicht berühren. < Und du hast eine Verabredung, die man dir sicher nicht brechen wird. Und schau zu deinem Gott, den du mit Beharrlichkeit verehrt hast. Wir werden ihn wahrlich verbrennen, und dann werden wir ihn wahrlich ins Meer streuen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmcdxlvi{Euer Gott ist allein Gott, außer dem es keinen Gott gibt. Er umfaßt alle Dinge in seinem Wissen.»\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmcdxlvii{Auf diese Weise erzählen Wir dir etwas von den Berichten dessen, was früher geschah. Und Wir haben dir eine Ermahnung von Uns zukommen lassen.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmcdxlviii{Wer sich dann abwendet, wird am lag der Auferstehung eine Last tragen.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmcdxlix{Darin werden sie ewig weilen. Schlimm wird für sie am Tag der Auferstehung das sein, was sie tragen.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmcdl{Am Tag, da in die Trompete geblasen wird. An jenem Tag versammeln Wir die Übeltäter wie Leute mit blauen Augen.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmcdli{Sie werden zueinander leise sprechen: «Ihr habt nur zehn (Nächte) verweilt.»\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmcdlii{Wir wissen besser, was sie sagen, wenn derjenige, der von ihnen der Vorbildlichste in seinem Verhalten ist, sagt: «Ihr habt ja nur einen Tag verweilt.»\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmcdliii{Und sie fragen dich nach den Bergen. Sprich: Mein Herr wird sie in den Wind streuen\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmcdliv{Und sie als ebenen Grund zurücklassen,\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmcdlv{So daß du in ihnen keine Krümmungen und keine Erhebungen mehr siehst.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmcdlvi{An jenem Tag folgen sie dem Rufer, bei dem nichts Krummes ist; die Stimmen wenden sich in demütiger Andacht zum Erbarmer, und du hörst nichts als Flüstern.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmcdlvii{An jenem Tag nützt die Fürsprache nicht, außer dem, dem es der Erbarmer erlaubt und dessen Worte Ihm Wohlgefallen.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmcdlviii{Er weiß, was vor ihnen und was hinter ihnen liegt, sie aber umfassen es nicht mit ihrem Wissen.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmcdlix{Die Gesichter werden sich demütig senken vor dem Lebendigen, dem Beständigen. Und enttäuscht wird der, dessen Last Unrecht ist.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmcdlx{Wer etwas von den guten Werken tut und dabei gläubig ist, der wird kein Unrecht und keine Lohnminderung befürchten.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmcdlxi{Und so haben Wir ihn als einen arabischen Koran hinabgesandt. Und Wir haben darin verschiedene Drohungen dargelegt, auf daß sie gottesfürchtig werden oder er ihnen eine Ermahnung bringe.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmcdlxii{Erhaben ist Gott, der wahre König. Und übereile dich nicht mit dem Koran, bevor er dir zu Ende offenbart worden ist. Und sprich: Mein Herr, gib mir mehr Wissen.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmcdlxiii{Und Wir hatten früher Adam (eine Verpflichtung) auferlegt. Aber er vergaß (sie). Und wir fanden bei ihm keine Entschlossenheit.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmcdlxiv{Und als Wir zu den Engeln sprachen: «Werft euch vor Adam nieder.» Sie warfen sich nieder, außer Iblis. Er weigerte sich.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmcdlxv{Wir sprachen: «O Adam, dieser (da) ist dir und deiner Gattin ein Feind. Daß er euch nicht aus dem Paradies vertreibt! Sonst wirst du unglücklich sein.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmcdlxvi{Es ist dir gewährt, daß du darin nicht hungerst und nicht nackt bist,\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmcdlxvii{Und daß du darin nicht dürstest und nicht unter der Sonnenhitze leidest.»\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmcdlxviii{Da flüsterte ihm der Satan ein, er sagte: «O Adam, soll ich dich auf den Baum der Ewigkeit hinweisen und auf eine Königsherrschaft, die nicht vergeht?»\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmcdlxix{Sie aßen beide davon, da wurde ihnen ihre Blöße offenbar, und sie begannen, Blätter des Paradieses über sich zusammenzuheften. Adam war gegen seinen Herrn ungehorsam, und so irrte er ab.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmcdlxx{Dann erwählte ihn sein Herr, und Er wandte sich ihm zu und leitete (ihn) recht.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmcdlxxi{Er sprach: «Geht von ihm hinunter. Die einen von euch sind Feinde der anderen. Wenn dann von Mir eine Rechtleitung zu euch kommt, dann wird der, der meiner Rechtleitung folgt, nicht irregehen und nicht unglücklich sein.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmcdlxxii{Und der, der sich von meiner Ermahnung abwendet, wird ein beengtes Leben führen. Und am Tag der Auferstehung versammeln Wir ihn blind (zu den anderen).\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmcdlxxiii{Er sagt: «Mein Herr, warum hast du mich blind (zu den anderen) versammelt, wo ich doch sehen konnte?»\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmcdlxxiv{Er spricht: «Zu dir sind doch unsere Zeichen gekommen, und du hast sie vergessen. Ebenso wirst du heute vergessen.»\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmcdlxxv{So vergelten Wir dem, der maßlos ist und nicht an die Zeichen seines Herrn glaubt. Die Pein des Jenseits ist ja härter und nachhaltiger.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmcdlxxvi{Ist es ihnen nicht aufgegangen, wie viele Generationen, in deren Wohnungen sie nun schreiten, Wir vor ihnen haben verderben lassen? Darin sind Zeichen für Leute, die Vernunft haben.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmcdlxxvii{Und gäbe es nicht einen früher ergangenen Spruch deines Herrn und eine festgesetzte Frist, wäre dieses Los (auch jetzt) unabwendbar.\qt@no{129}}%
\qt@newcmd\qurantrans@de@mmcdlxxviii{So ertrage mit Geduld, was sie sagen. Und singe das Lob deines Herrn vor dem Aufgang der Sonne und vor ihrem Untergang. Und preise (Ihn) zu (verschiedenen) Nachtzeiten und an den Enden des Tages, auf daß du zufrieden bist.\qt@no{130}}%
\qt@newcmd\qurantrans@de@mmcdlxxix{Und richte nicht gierig deine Augen auf das, was Wir einigen von ihnen zur Nutznießung verliehen haben - das ist der Glanz des diesseitigen Lebens -, um sie darin der Versuchung auszusetzen. Der Lebensunterhalt, den dein Herr beschert, ist besser und hat eher Bestand.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mmcdlxxx{Und befiehl deinen Angehörigen, das Gebet (zu verrichten). Und harre du darin aus. Wir fordern keinen Lebensunterhalt von dir. Wir bescheren dir doch den Lebensunterhalt. Und das (gute) Ende gehört der Frömmigkeit.\qt@no{132}}%
\qt@newcmd\qurantrans@de@mmcdlxxxi{Und sie sagen: «Wenn er doch ein Zeichen von seinem Herrn bringen würde!» Ist nicht zu ihnen der deutliche Beweis dessen gekommen, was in den früheren Blättern steht?\qt@no{133}}%
\qt@newcmd\qurantrans@de@mmcdlxxxii{Hätten Wir sie vor ihm durch eine Pein verderben lassen, hätten sie gesagt: «Unser Herr, hättest Du doch einen Gesandten zu uns geschickt, so daß wir deinen Zeichen hätten folgen können, bevor wir Erniedrigung und Schande erlitten!»\qt@no{134}}%
\qt@newcmd\qurantrans@de@mmcdlxxxiii{Sprich: Jeder wartet ab. So wartet auch ihr ab. Ihr werdet erfahren, wer die Leute des ebenen Weges sind und wer der Rechtleitung folgt.\qt@no{135}}%
\qt@newcmd\qurantrans@de@mmcdlxxxiv{\basmalahde Nahegerückt ist den Menschen ihre Abrechnung, während sie sich in Achtlosigkeit abwenden.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcdlxxxv{Keine neue Ermahnung kommt von ihrem Herrn zu ihnen, ohne daß sie sie hören, während sie sich dem Spiel hingeben\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcdlxxxvi{Und ihre Herzen zerstreut sind. Und sie äußern insgeheim im vertraulichen Gespräch - sie, die Unrecht tun: «Ist dieser etwas anderes als ein Mensch wie ihr? Wollt ihr euch denn sehenden Auges der Zauberei hingeben?»\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmcdlxxxvii{Sprich: Mein Herr weiß, was im Himmel und auf der Erde gesagt wird. Er ist der, der alles hört und weiß.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcdlxxxviii{Aber nein, sie sagen: «Wirres Bündel von Träumen. Nein, er hat ihn erdichtet. Nein, er ist ja ein Dichter. Er soll uns ein Zeichen bringen, sowie die Früheren gesandt worden sind.»\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcdlxxxix{Nie hat vor ihnen eine Stadt, die Wir verderben ließen, geglaubt. Werden gerade sie nun glauben?\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcdxc{Und Wir haben vor dir nur Männer gesandt, denen Wir Offenbarungen eingegeben haben. So fragt die Besitzer der Ermahnung, wenn ihr nicht Bescheid wißt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcdxci{Wir haben die nicht zu Körpern gemacht, die keine Speise essen, und sie waren nicht ewig.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcdxcii{Dann haben Wir ihnen das Versprechen wahr gemacht. So haben Wir sie und die, die Wir wollen, gerettet und die Maßlosen vernichtet.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcdxciii{Wir haben ein Buch zu euch hinabgesandt, in dem eure Ermahnung steht. Habt ihr denn keinen Verstand?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcdxciv{Wie manche Stadt, die Unrecht tat, haben Wir zerstört und haben nach ihr ein anderes Volk entstehen lassen!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmcdxcv{Und als sie unsere Schlagkraft spürten, da fingen sie an, davor wegzulaufen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcdxcvi{«Lauft nicht weg und kehrt zu dem euch verliehenen üppigen Leben und zu euren Wohnungen zurück. Vielleicht werdet ihr zur Verantwortung gezogen.»\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcdxcvii{Sie sagten: «Wehe uns! Wir haben ja Unrecht getan.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmcdxcviii{Das war ständig ihr Ruf, bis Wir sie zum abgemähten Land machten und sie ausgelöscht dalagen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmcdxcix{Und Wir haben den Himmel und die Erde, und was zwischen ihnen ist, nicht zum Spiel erschaffen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmd{Hätten Wir uns eine Zerstreuung nehmen wollen, dann hätten Wir sie von Uns aus genommen - wenn Wir das überhaupt hätten tun wollen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdi{Nein, Wir schleudern die Wahrheit gegen das Falsche, und sie zerschmettert ihm das Haupt, und siehe da, es schwindet dahin. Und wehe euch wegen dessen, was ihr da schildert!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdii{Ihm gehört, wer in den Himmeln und auf der Erde ist. Diejenigen, die bei Ihm sind, weigern sich nicht hochmütig, Ihm zu dienen, und werden darin nicht müde.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdiii{Sie preisen (Ihn) Nacht und Tag, und sie lassen nicht nach.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdiv{Oder haben sie sich Götter aus der Erde genommen, die auferstehen lassen können?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdv{Gäbe es in ihnen beiden andere Götter als Gott, würden sie (beide) verderben. Preis sei Gott, dem Herrn des Thrones! (Er ist erhaben) über das, was sie schildern.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdvi{Er wird nicht zur Verantwortung gezogen für das, was Er tut; sie aber werden zur Verantwortung gezogen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdvii{Oder haben sie sich außer Ihm Götter genommen? Sprich: Bringt her euren Beweis. Das ist die Ermahnung derer, die mit mir sind, und die Ermahnung derer, die vor mir lebten. Aber die meisten von ihnen kennen die Wahrheit nicht, so wenden sie sich ab.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdviii{Und Wir haben keinen Gesandten vor dir geschickt, dem Wir nicht offenbart hätten: «Es gibt keinen Gott außer Mir, so dienet Mir.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdix{Und sie sagen: «Der Erbarmer hat sich ein Kind genommen.» Preis sei Ihm! Nein, es sind nur Diener, denen Ehre erwiesen worden ist.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdx{Sie kommen Ihm im Sprechen nicht zuvor, und nach seinem Befehl handeln sie.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdxi{Er weiß, was vor ihnen und was hinter ihnen liegt, und sie legen Fürsprache nur für den ein, der Ihm genehm ist. Und sie erschrecken aus Furcht vor Ihm.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdxii{Und wer von ihnen sagen sollte: «Ich bin Gott neben Ihm», dem vergelten Wir mit der Hölle. So vergelten Wir denen, die Unrecht tun.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdxiii{Haben denn diejenigen, die ungläubig sind, nicht gesehen, daß die Himmel und die Erde eine einzige Masse waren? Da haben Wir sie getrennt und alles Lebendige aus dem Wasser gemacht. Wollen sie denn nicht glauben?\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdxiv{Und Wir haben auf der Erde festgegründete Berge gemacht, daß sie nicht mit ihnen schwanke. Und Wir haben auf ihr breite Durchgänge als Wege gemacht, auf daß sie der Rechtleitung folgen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdxv{Und Wir haben den Himmel zu einer wohlbehüteten Decke gemacht. Aber sie wenden sich von seinen Zeichen ab.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdxvi{Und Er ist es, der die Nacht und den Tag, die Sonne und den Mond erschaffen hat; jedes Gestirn nimmt seinen Lauf in einer (eigenen) Sphäre.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdxvii{Und Wir haben für keinen Menschen vor dir bestimmt, ewig zu leben. Wenn du nun stirbst, sollten sie dann ewig leben?\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdxviii{Jeder wird den Tod erleiden. Und Wir prüfen euch mit Bösem und Gutem und setzen euch damit der Versuchung aus. Und zu Uns werdet ihr zurückgebracht.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdxix{Und wenn diejenigen, die ungläubig sind, dich sehen, nehmen sie dich nur zum Gegenstand des Spottes: «Ist das der, der über eure Götter spricht?» Und sie, sie glauben nicht an die Ermahnung des Erbarmers.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdxx{Der Mensch ist aus Eilfertigkeit erschaffen worden. Ich werde euch schon meine Zeichen sehen lassen; so wünscht nicht von Mir, es zu beschleunigen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdxxi{Und sie sagen: «Wann wird diese Androhung eintreten, so ihr die Wahrheit sagt?»\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdxxii{Wenn diejenigen, die ungläubig sind, es nur wüßten, wenn sie das Feuer weder von ihren Gesichtern noch von ihren Rücken abhalten können und auch keine Unterstützung erfahren!\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdxxiii{Nein, es wird sie plötzlich überkommen und sie überraschen. Sie werden es nicht abwehren können, und es wird ihnen kein Aufschub gewährt.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdxxiv{Gespottet wurde schon vor dir über Gesandte. Da umschloß diejenigen, die sie verhöhnt hatten, das, worüber sie spotteten.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdxxv{Sprich: Wer behütet euch bei Nacht und bei Tag vor dem Erbarmer? Aber nein, sie wenden sich von der Ermahnung ihres Herrn ab.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdxxvi{Oder haben sie etwa Götter, die sie vor Uns schützen können? Sie können doch sich selbst nicht helfen, und sie werden keinen Beistand gegen Uns erhalten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdxxvii{Nein, Wir haben diese da und ihre Väter genießen lassen, bis sie ein hohes Alter erreicht haben. Haben sie denn nicht gesehen, daß Wir über das Land kommen und es an seinen Enden kürzen? Werden nun sie die Sieger sein?\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdxxviii{Sprich: Ich warne euch nur mit der Offenbarung. Aber Taube hören nicht den Ruf, wenn sie gewarnt werden.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdxxix{Und wenn ein Hauch von der Pein deines Herrn sie berührt, sagen sie gewiß: «O wehe uns! Wir haben ja Unrecht getan.»\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdxxx{Und Wir stellen die gerechten Waagen für den Tag der Auferstehung auf. So wird keiner Seele in irgend etwas Unrecht getan. Und wäre es auch das Gewicht eines Senfkornes, Wir bringen es bei. Und Wir genügen für die Abrechnung.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdxxxi{Und Wir haben Mose und Aaron die Unterscheidungsnorm zukommen lassen, und ein Licht und eine Ermahnung für die Gottesfürchtigen,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdxxxii{Die ihren Herrn im Verborgenen fürchten und vor der Stunde erschrecken.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdxxxiii{Und dies ist eine gesegnete Ermahnung, die Wir hinabgesandt haben. Wollt ihr sie denn verwerfen?\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdxxxiv{Und Wir haben zuvor Abraham zu seinem rechten Verhalten geleitet. Und Wir wußten über ihn Bescheid.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdxxxv{Als er zu seinem Vater und seinem Volk sagte: «Was sind das für Bildwerke, die ihr verehrt?»\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdxxxvi{Sie sagten: «Wir fanden, daß bereits unsere Väter ihnen dienten.»\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdxxxvii{Er sagte: «Ihr und eure Väter befindet euch in einem offenkundigen Irrtum.»\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdxxxviii{Sie sagten: «Bringst du uns die Wahrheit, oder gehörst du zu denen, die ihr Spiel treiben?»\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdxxxix{Er sagte: «Nein, euer Herr ist der Herr der Himmel und der Erde, der sie erschaffen hat. Und ich bin einer von denen, die euch das bezeugen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdxl{Und bei Gott, ich werde gegen eure Götter mit einer List vorgehen, nachdem ihr den Rücken gekehrt habt.»\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdxli{Da schlug er sie in Stücke, außer einem großen unter ihnen, auf daß sie sich zu ihm wandten.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdxlii{Sie sagten: «Wer hat dies mit unseren Göttern getan? Wahrlich, er gehört zu denen, die Unrecht tun.»\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdxliii{Sie sagten: «Wir hörten einen Jüngling von ihnen sprechen; man nennt ihn Abraham.»\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdxliv{Sie sagten: «Bringt ihn her vor den Augen der Menschen. Vielleicht bezeugen sie es.»\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdxlv{Sie sagten: «Hast du dies mit unseren Göttern getan, o Abraham?»\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdxlvi{Er sagte: «Nein, getan hat das dieser da, der Größte unter ihnen. Fragt sie, so sie reden können.»\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdxlvii{Sie kamen wieder zu sich und sagten: «Ihr seid ja die, die Unrecht tun.»\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdxlviii{Dann machten sie eine Kehrtwende: «Du weißt doch, daß diese nicht reden können.»\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmdxlix{Er sagte: «Wie könnt ihr anstelle Gottes das verehren, was euch nichts nützen und nichts schaden kann?\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmdl{Pfui über euch und über das, was ihr anstelle Gottes verehrt! Habt ihr denn keinen Verstand?»\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmdli{Sie sagten: «Verbrennt ihn und helft euren Göttern, so ihr etwas tun wollt.»\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmdlii{Wir sprachen: «O Feuer, sei kühl und harmlos für Abraham.»\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmdliii{Sie wollten mit einer List gegen ihn vorgehen. Da machten Wir, daß sie die waren, die den größten Verlust hatten.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmdliv{Und Wir erretteten ihn und Lot in das Land, das Wir für die Weltenbewohner gesegnet haben.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmdlv{Und Wir schenkten ihm Isaak und Jakob dazu. Und jeden machten Wir rechtschaffen.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmdlvi{Und Wir machten sie zu Vorbildern, die (die Menschen) nach unserem Befehl leiteten. Und Wir offenbarten ihnen, die guten Werke zu tun, das Gebet zu verrichten und die Abgabe zu entrichten. Und Uns haben sie gedient.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmdlvii{Und dem Lot ließen Wir Urteilskraft und Wissen zukommen. Und Wir erretteten ihn aus der Stadt, die die schlechten Taten zu begehen pflegte. Sie waren ja böse Leute und Frevler.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmdlviii{Und Wir ließen ihn in unsere Barmherzigkeit eingehen. Wahrlich, er gehört zu den Rechtschaffenen.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmdlix{Und (erwähne) Noach, als er zuvor rief. Da erhörten Wir ihn und erretteten ihn und seine Angehörigen aus der großen Drangsal.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmdlx{Und Wir unterstützten ihn gegen die Leute, die unsere Zeichen für Lüge erklärten. Sie waren ja böse Leute. So ließen Wir sie allesamt ertrinken.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmdlxi{Und (erwähne) David und Salomo, als sie über das Saatfeld urteilten. Darin waren die Schafe der Leute nachts eingebrochen und hatten sich ausgebreitet. Und Wir waren bei ihrem Urteil zugegen.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmdlxii{Und Wir ließen Salomo die Sache begreifen. Und jedem ließen Wir Urteilskraft und Wissen zukommen. Und Wir machten dem David die Berge dienstbar, daß sie (Uns) preisen, und (auch) die Vögel. Ja, Wir haben es getan.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmdlxiii{Und Wir lehrten ihn die Herstellung von Panzerkleidung für euch, damit sie euch vor eurer (gegenseitigen) Schlagkraft schütze. Wollt ihr denn nicht dankbar sein?\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmdlxiv{Und dem Salomo (machten Wir dienstbar) den Wind als Sturm, daß er nach seinem Befehl zum Land eile, das Wir gesegnet haben - und Wir wußten über alle Dinge Bescheid -,\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmdlxv{Und auch unter den Satanen welche, die für ihn tauchten und noch andere Arbeiten verrichteten; und Wir überwachten sie.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmdlxvi{Und (erwähne) Ijob, als er zu seinem Herrn rief: «Mich hat Schaden getroffen, und Du bist der Barmherzigste der Barmherzigen.»\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmdlxvii{Da erhörten Wir ihn und behoben den Schaden, den er erlitten hatte. Und Wir gaben ihm seine Angehörigen (wieder) und noch einmal die gleiche Zahl dazu, aus Barmherzigkeit von Uns und als Ermahnung für die, die (Uns) dienen.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmdlxviii{Und (erwähne) Ismael und Idris und Dhul-Kifl. Jeder (von ihnen) gehörte zu den Geduldigen.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmdlxix{Und Wir ließen sie in unsere Barmherzigkeit eingehen. Wahrlich, sie gehören zu den Rechtschaffenen.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmdlxx{Und (erwähne) den Mann mit dem Fisch. Als er erzürnt wegging und meinte, Wir könnten ihn nicht überwältigen. Und er rief in den Finsternissen: «Es gibt keinen Gott außer Dir. Preis sei Dir! Ich war einer von denen, die Unrecht tun.»\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmdlxxi{Da erhörten Wir ihn und erretteten ihn aus der Trübsal. So retten Wir die Gläubigen.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmdlxxii{Und (erwähne) Zakaria, als er zu seinem Herrn rief: «Mein Herr, laß mich nicht kinderlos bleiben. Und Du bist der beste der Erben.»\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmdlxxiii{Da erhörten Wir ihn und schenkten ihm Yahyaa und machten ihm seine Gattin wieder fruchtbar. Sie eilten zu den guten Dingen um die Wette und riefen zu Uns in Verlangen und Ehrfurcht, und sie waren vor Uns demütig.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmdlxxiv{Und (erwähne) die, die ihre Scham unter Schutz stellte. Da bliesen Wir in sie von unserem Geist, und Wir machten sie und ihren Sohn zu einem Zeichen für die Weltenbewohner.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmdlxxv{«Diese eure Gemeinschaft ist eine einzige Gemeinschaft. Und ich bin euer Herr, so dienet Mir.»\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmdlxxvi{Aber sie spalteten sich in ihrer Angelegenheit untereinander. Doch sie werden alle zu Uns zurückkehren.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmdlxxvii{Wer etwas von den guten Werken tut und dabei gläubig ist, der wird für sein Mühen nicht Undank ernten; Wir schreiben es ihm gut.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmdlxxviii{Und ein Verbot liegt über einer Stadt, die Wir verderben ließen: Sie kehren nicht zurück,\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmdlxxix{Bis der (Damm von) Gog und Magog eröffnet wird, sie von allen Anhöhen herbeieilen\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmdlxxx{Und die wahrhaftige Drohung naherückt. Dann werden die Blicke derer, die ungläubig sind, starr werden: «O wehe uns! Wir ließen es unbeachtet. Ja, wir haben Unrecht getan.»\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmdlxxxi{«Ihr und das, was ihr anstelle Gottes verehrt, seid Brennstoff der Hölle. Ihr werdet dahin wie zur Tränke gehen.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmdlxxxii{Wären diese da Götter, wären sie nicht darin wie zur Tränke gegangen.» Und alle werden darin ewig weilen.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmdlxxxiii{Sie werden darin seufzen, aber selbst darin nichts hören.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmdlxxxiv{Diejenigen, für die das Beste von Uns her vorausbestimmt ist, werden von ihr ferngehalten,\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmdlxxxv{Sie hören nicht (einmal) ihr leisestes Geräusch. Und sie werden in dem, was ihre Seelen begehrt haben, ewig weilen.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmdlxxxvi{Der große Schrecken macht sie nicht traurig, und die Engel empfangen sie: «Das ist euer Tag, der euch versprochen wurde.»\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmdlxxxvii{Am Tag, an dem Wir den Himmel zusammenfalten, wie das Urkundenbuch die Schriftstücke zusammenfaltet. Wie Wir eine erste Schöpfung am Anfang gemacht haben, wiederholen Wir sie. Das ist ein Uns obliegendes Versprechen. Ja, Wir werden es tun.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmdlxxxviii{Und Wir haben in der Schrift nach der Ermahnung geschrieben, daß meine rechtschaffenen Diener das Land erben werden.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmdlxxxix{In diesem ist eine Botschaft an Leute, die (Uns) dienen.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmdxc{Und Wir haben dich nur als eine Barmherzigkeit für die Weltenbewohner gesandt.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmdxci{Sprich: Mir wird wahrlich offenbart, daß euer Gott ein einziger Gott ist. Wollt ihr denn nicht gottergeben sein?\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmdxcii{Wenn sie sich abkehren, dann sprich: Ich habe es euch so angekündigt, daß zwischen uns Gleichheit besteht. Und ich weiß nicht, ob das, was euch angedroht wird, nahe ist oder fernliegt.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmdxciii{Er weiß, was an Worten laut geäußert wird, und Er weiß, was ihr verschweigt.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmdxciv{Und ich weiß nicht, ob es vielleicht nur eine Versuchung für euch ist und eine Nutznießung für eine Weile.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmdxcv{Er sprach: Mein Herr, urteile nach der Wahrheit. Und unser Herr ist der Erbarmer, der um Hilfe gebeten wird gegen das, was ihr schildert.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmdxcvi{\basmalahde O ihr Menschen, fürchtet euren Herrn. Das Beben der Stunde ist eine gewaltige Sache.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdxcvii{Am Tag, da ihr es seht, wird jede Stillende aus Entsetzen übersehen, was sie eben gestillt hat, und jede Schwangere wird mit dem niederkommen, was sie trägt. Und du siehst die Menschen trunken, obwohl sie nicht betrunken sind. Aber die Pein Gottes ist hart.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdxcviii{Und unter den Menschen ist manch einer, der über Gott ohne (richtiges) Wissen streitet und jedem rebellischen Satan folgt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdxcix{Gegen den geschrieben steht, daß er den, der ihn zum Freund nimmt, irreführen und zur Pein des Höllenbrandes leiten wird.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdc{O ihr Menschen, wenn ihr über die Auferstehung im Zweifel seid, so bedenket, daß Wir euch aus Erde erschaffen haben, dann aus einem Tropfen, dann aus einem Embryo, dann aus einem Fötus, gestaltet und nicht gestaltet, um es euch deutlich zu machen. Und Wir lassen, was Wir wollen, im Mutterleib auf eine festgesetzte Frist ruhen. Dann lassen Wir euch als Kind herauskommen. Dann (sorgen Wir für euch), damit ihr eure Vollkraft erreicht. Und manch einer von euch wird abberufen, und manch einer von euch wird in das schlimmste Alter gebracht, so daß er, nachdem er vorher Wissen besessen hat, nichts (mehr) weiß. Und du siehst die Erde regungslos. Wenn Wir aber Wasser auf sie herabkommen lassen, regt sie sich, wächst zu und läßt verschiedene erfreuliche Pflanzenarten sprießen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdci{Dies, weil Gott die Wahrheit ist, und weil Er die Toten lebendig macht und weil Er Macht hat zu allen Dingen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdcii{Und weil die Stunde kommt - an ihr ist kein Zweifel möglich -, und weil Gott (all) die erwecken wird, die in den Gräbern sind.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdciii{Und unter den Menschen gibt es manch einen, der über Gott streitet ohne (richtiges) Wissen, ohne Rechtleitung und ohne erleuchtendes Buch;\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdciv{Er wendet sich zur Seite, um (die Menschen) vom Wege Gottes abirren zu lassen. Bestimmt ist für ihn Schande im Diesseits, und Wir lassen ihn am Tag der Auferstehung die Pein des Höllenbrandes kosten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdcv{«Dies für das, was deine Hände vorausgeschickt haben, und weil Gott den Dienern kein Unrecht tut.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdcvi{Und unter den Menschen gibt es manch einen, der Gott nur beiläufig dient. Wenn ihn etwas Gutes trifft, fühlt er sich wohl darin. Und wenn ihn eine Versuchung trifft, macht er eine Kehrtwende. Er verliert das Diesseits und das Jenseits. Das ist der offenkundige Verlust.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdcvii{Er ruft anstelle Gottes das an, was ihm weder schaden noch nützen kann. Das ist der tiefe Irrtum.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdcviii{Er ruft den an, von dem eher Schaden zu erwarten ist als Nutzen. Welch schlimmer Schutzherr und welch schlimmer Gefährte!\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdcix{Gott läßt diejenigen, die glauben und die guten Werke tun, in Gärten eingehen, unter denen Bäche fließen. Gott tut, was Er will.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdcx{Wer meint, daß Gott ihn im Diesseits und Jenseits niemals unterstützen werde, der strecke doch ein Seil zum Himmel, dann schneide er es ab. So schaue er, ob seine List das entfernt, was (seinen) Groll hervorruft.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdcxi{Und so haben Wir ihn als deutliche Zeichen hinabgesandt. (Es ist so,) daß Gott rechtleitet, wen Er will.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdcxii{Diejenigen, die glauben, und diejenigen, die Juden sind, und die Säbier und die Christen und die Magier und diejenigen, die Polytheisten sind - siehe, Gott wird am Tag der Auferstehung zwischen ihnen entscheiden. Gott ist ja Zeuge über alle Dinge.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdcxiii{Hast du nicht gesehen, daß sich vor Gott niederwirft, wer in den Himmeln und wer auf der Erde ist, und (auch) die Sonne, der Mond und die Sterne, die Berge, die Bäume und die Tiere, und viele von den Menschen? Und gegen viele ist die Pein zu Recht fällig geworden. Und wen Gott der Schmach aussetzt, dem kann niemand Ehre erweisen. Wahrlich, Gott tut, was Er will.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdcxiv{Das sind zwei Streitparteien, die miteinander über ihren Herrn streiten. Für diejenigen, die ungläubig sind, sind Gewänder aus Feuer zugeschnitten; über ihre Köpfe wird heißes Wasser gegossen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdcxv{Dadurch wird zum Schmelzen gebracht, was sie in ihrem Bauch haben, und ebenso die Haut.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdcxvi{Und für sie sind Keulen aus Eisen bestimmt.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdcxvii{Sooft sie vor Kummer aus ihm herauskommen wollen, werden sie zu ihm zurückgebracht, und (es wird zu ihnen gesagt): «Kostet die Pein des Höllenbrandes.»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdcxviii{Gott läßt diejenigen, die glauben und die guten Werke tun, in Gärten eingehen, unter denen Bäche fließen. Geschmückt werden sie darin mit Armringen aus Gold und mit Perlen, und ihre Kleidung darin ist aus Seide.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdcxix{Rechtgeleitet sind sie zu den Worten, die gut sind; und rechtgeleitet sind sie zum Weg dessen, der des Lobes würdig ist.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdcxx{Diejenigen, die ungläubig sind und vom Wege Gottes abweisen und auch von der heiligen Moschee, die Wir für die Menschen bestimmt haben, gleich ob sie dort oder in der Wüste wohnen... Wer sucht, darin etwas Krummes zu Unrecht zu verüben, den werden Wir eine schmerzhafte Pein kosten lassen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdcxxi{Und als Wir den Abraham in die Stätte des Hauses eingewiesen haben: «Geselle mir nichts bei. Und reinige mein Haus für diejenigen, die den Umlauf vollziehen, und für die, die aufrecht stehen, sich verneigen und niederwerfen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdcxxii{Und ruf unter den Menschen zur Wallfahrt auf, so werden sie zu dir kommen zu Fuß und auf vielen hageren Kamelen, die aus jedem tiefen Paßweg daherkommen,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdcxxiii{Damit sie allerlei Nutzen für sich erfahren und den Namen Gottes an den bekannten Tagen über dem erwähnen, was Er ihnen an Herdenvieh beschert hat. - Eßt davon und gebt dem Notleidenden und Armen zu essen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdcxxiv{Dann wollen sie ihre Ungepflegtheit beenden, ihre Gelübde erfüllen und zum Umlauf um das altehrwürdige Haus geführt werden.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdcxxv{So ist es. Und wenn einer die heiligen Dinge Gottes hochhält, ist es besser für ihn bei seinem Herrn. Erlaubt ist euch das Vieh, außer dem, was euch bekanntgemacht wird. So meidet den Greuel der Götzen, und meidet die falsche Aussage,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdcxxvi{Als Anhänger des rechten Glaubens gegenüber Gott, die Ihm nichts beigesellen. Und wenn einer Gott (etwas) beigesellt, es ist, als würde er vom Himmel herunterfallen, von den Vögeln weggeschnappt oder vom Wind an einen fernen Ort hinabgestürzt werden.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdcxxvii{So ist es. Und wenn einer die Opfertiere Gottes hochhält, ist es ein Ausdruck der Frömmigkeit der Herzen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdcxxviii{In ihnen habt ihr allerlei Nutzen auf eine bestimmte Frist, dann liegt ihr erlaubter Schlachtort beim altehrwürdigen Haus.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdcxxix{Und für jede Gemeinschaft haben Wir einen Ritus festgelegt, damit sie den Namen Gottes über dem erwähnen, was er ihnen an Herdenvieh beschert hat. Und euer Gott ist ein einziger Gott, Ihm müßt ihr ergeben sein. Und verkünde frohe Botschaft denen, die sich demütigen,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdcxxx{Deren Herzen sich ängstigen, wenn Gottes gedacht wird, die das, was sie trifft, geduldig ertragen, das Gebet verrichten und von dem, was Wir ihnen beschert haben, spenden.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdcxxxi{Und die Opferkamele haben wir euch zu Kultzeichen Gottes gemacht. An ihnen habt ihr etwas Gutes. So erwähnt den Namen Gottes über ihnen, wenn sie mit gebundenen Beinen dastehen. Und wenn sie auf die Seite umgefallen sind, dann eßt davon und gebt dem bescheidenen und dem fordernden (Armen) zu essen. So haben Wir sie euch dienstbar gemacht, auf daß ihr dankbar seid.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdcxxxii{Weder ihr Fleisch noch ihr Blut erreicht Gott, aber Ihn erreicht eure Frömmigkeit. So hat Er sie euch dienstbar gemacht, damit ihr Gott hochpreiset, daß Er euch rechtgeleitet hat. Und verkünde denen, die Gutes tun, frohe Botschaft.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdcxxxiii{Gott verteidigt diejenigen, die glauben. Gott liebt keinen undankbaren Treulosen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdcxxxiv{Erlaubnis (zum Kampf) ist denen gegeben, die bekämpft werden, weil ihnen ja Unrecht getan wurde - und Gott hat gewiß die Macht, sie zu unterstützen -,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdcxxxv{ihnen), die zu Unrecht aus ihren Wohnstätten vertrieben wurden, nur weil sie sagen: Unser Herr ist Gott. Und hätte Gott nicht die einen Menschen durch die anderen abgewehrt, so wären gewiß Mönchsklausen, Kirchen, Gebetsstätten und Moscheen zerstört worden, in denen des Namens Gottes viel gedacht wird. - Und Gott wird bestimmt die unterstützen, die Ihn unterstützen. Gott ist stark und mächtig.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdcxxxvi{Ihnen), die, wenn Wir ihnen eine angesehene Stellung auf der Erde geben, das Gebet verrichten und die Abgabe entrichten, das Rechte gebieten und das Verwerfliche verbieten. Und Gott gehört das Ende der Angelegenheiten.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdcxxxvii{Wenn sie dich der Lüge zeihen, so haben auch vor ihnen das Volk Noachs, die Aad und die Thamud (ihre Gesandten) der Lüge geziehen,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdcxxxviii{Und auch das Volk Abrahams und das Volk Lots,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdcxxxix{Und die Bewohner von Madyan. Auch Mose wurde der Lüge geziehen. Da gewährte Ich den Ungläubigen Aufschub, dann ergriff Ich sie. Und wie war dann meine Mißbilligung!\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdcxl{Wie so manche Stadt, die Unrecht tat, haben Wir verderben lassen, so daß sie bis zu den Dächern verödet war, und wie manchen (nun) verlassenen Brunnen und manches aufgerichtete Schloß!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdcxli{Sind sie denn nicht auf der Erde umhergegangen, daß sie Herzen bekommen, mit denen sie verstehen, oder Ohren, mit denen sie hören? Nicht die Blicke sind blind, blind sind die Herzen, die in der Brust sind.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdcxlii{Und sie wünschen von dir, die Pein zu beschleunigen. Gott wird sein Versprechen nicht brechen. Und siehe, ein Tag bei deinem Herrn ist wie tausend Jahre nach eurer Berechnung.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdcxliii{Und wie mancher Stadt, die Unrecht tat, habe Ich Aufschub gewährt! Dann habe Ich sie ergriffen. Und zu Mir führt der Lebensweg.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdcxliv{Sprich: O ihr Menschen, ich bin euch nur ein deutlicher Warner.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdcxlv{Für diejenigen nun, die glauben und die guten Werke tun, sind Vergebung und trefflicher Unterhalt bestimmt.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdcxlvi{Und diejenigen, die eifrig gegen unsere Zeichen vorgehen und ihnen Machtlosigkeit nachweisen wollen, das sind die Gefährten der Hölle.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdcxlvii{Und Wir haben vor dir keinen Gesandten oder Propheten geschickt, ohne daß ihm, wenn er etwas wünschte, der Satan in seinen Wunsch etwas dazwischen geworfen hätte. Aber Gott hebt auf, was der Satan dazwischen wirft. Dann legt Gott seine Zeichen eindeutig fest. Und Gott weiß Bescheid und ist weise.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdcxlviii{Das ist so), damit Er das, was der Satan dazwischen wirft, zu einer Versuchung für diejenigen macht, in deren Herzen Krankheit ist und deren Herzen verhärtet sind - wahrlich, die Unrecht tun, befinden sich in weitgehendem Widerstreit -,\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdcxlix{Und damit diejenigen, denen das Wissen zugekommen ist, wissen, daß es die Wahrheit von deinem Herrn ist, so daß sie daran glauben und ihre Herzen sich vor Ihm demütigen. Und wahrlich, Gott führt diejenigen, die glauben, zu einem geraden Weg.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdcl{Und diejenigen, die ungläubig sind, beharren auf ihrem Zweifel daran, bis die Stunde sie plötzlich überkommt oder bis die Pein eines unfruchtbaren Tages über sie kommt.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdcli{An jenem Tag gehört die Königsherrschaft Gott (allein). Er wird zwischen ihnen urteilen. Dann werden diejenigen, die glauben und die guten Werke tun, in den Gärten der Wonne sein.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdclii{Diejenigen aber, die ungläubig sind und unsere Zeichen für Lüge erklären, erhalten eine schmähliche Pein.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdcliii{Denen, die auf dem Weg Gottes ausgewandert sind, dann getötet werden oder sterben, wird Gott bestimmt einen schönen Unterhalt bescheren. Gott ist ja der beste der Versorger.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdcliv{Er wird sie bestimmt in einen Ort eingehen lassen, der ihnen genehm ist. Und Gott weiß Bescheid und ist langmütig.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdclv{So ist es. Und wer im gleichen Maße bestraft, wie er bestraft wurde, dann aber wieder Gegenstand von Übergriffen wird, den wird Gott sicher unterstützen. Gott ist voller Verzeihung und Vergebung.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdclvi{Dies, weil Gott die Nacht in den Tag übergehen und den Tag in die Nacht übergehen läßt, und weil Gott alles hört und sieht.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdclvii{Dies, weil Gott die Wahrheit ist und weil das, was sie an seiner Stelle anrufen, das Falsche ist, und weil Gott der Erhabene und der Große ist.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdclviii{Hast du denn nicht gesehen, daß Gott Wasser vom Himmel herabkommen läßt, so daß die Erde grün wird? Gott ist feinfühlig und hat Kenntnis von allem.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdclix{Ihm gehört, was in den Himmeln und was auf der Erde ist. Gott ist der, der auf niemanden angewiesen und des Lobes würdig ist.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdclx{Hast du denn nicht gesehen, daß Gott euch das, was auf der Erde ist, dienstbar gemacht hat, ebenso die Schiffe, die nach seinem Befehl auf dem Meer fahren? Und Er hält den Himmel, daß er nicht auf die Erde fällt, außer mit seiner Erlaubnis. Siehe, Gott hat Mitleid mit den Menschen und ist barmherzig.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmdclxi{Und Er ist es, der euch lebendig gemacht hat. Dann läßt Er euch sterben, dann macht Er euch wieder lebendig. Wahrlich, der Mensch ist undankbar.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmdclxii{Für jede Gemeinschaft haben Wir einen Ritus festgelegt, den sie zu vollziehen haben. So sollen sie nicht mit dir über die Angelegenheit Streit führen. Und rufe zu deinem Herrn. Siehe, du folgst einer geraden Rechtleitung.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmdclxiii{Und wenn sie doch mit dir streiten, dann sprich: Gott weiß besser, was ihr tut.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmdclxiv{Gott wird am Tag der Auferstehung zwischen euch über das urteilen, worüber ihr uneins wart.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmdclxv{Weißt du denn nicht, daß Gott weiß, was im Himmel und auf der Erde ist? Das steht in einem Buch. Das ist Gott ein leichtes.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmdclxvi{Und sie verehren anstelle Gottes das, wofür Er keine Ermächtigung herabgesandt hat und wovon sie kein Wissen haben. Für die, die Unrecht tun, gibt es keinen Helfer.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmdclxvii{Und wenn ihnen unsere Zeichen als deutliche Beweise verlesen werden, dann erkennst du im Gesicht derer, die ungläubig sind, die Mißbilligung. Am liebsten würden sie über die herfallen, die ihnen unsere Zeichen verlesen. Sprich: Soll ich euch kundtun, was schlimmer ist als dies? Es ist das Feuer, das Gott denen angedroht hat, die ungläubig sind - welch schlimmes Ende!\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmdclxviii{O ihr Menschen, ein Gleichnis wird (euch) vorgetragen, so hört darauf. Diejenigen, die ihr anstelle Gottes anruft, können niemals auch nur eine Fliege erschaffen, auch wenn sie sich dafür zusammentun. Und wenn die Fliege ihnen etwas raubte, könnten sie es ihr nicht entreißen. Schwach ist (hier) der, der sucht, und das, was gesucht wird.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmdclxix{Sie haben Gott nicht so eingeschätzt, wie Er eingeschätzt werden soll. Wahrlich, Gott ist stark und mächtig.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmdclxx{Gott erwählt sich aus den Engeln Boten, und (auch) aus den Menschen. Gott hört und sieht alles.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmdclxxi{Er weiß, was vor ihnen und was hinter ihnen liegt. Und zu Gott werden die Angelegenheiten zurückgebracht.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmdclxxii{O ihr, die ihr glaubt, verneigt euch, werft euch nieder und dient eurem Herrn, und tut das Gute, auf daß es euch wohl ergehe.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmdclxxiii{Und setzt euch für Gott ein, wie der richtige Einsatz für Ihn sein soll. Er hat euch erwählt. Und Er hat euch in der Religion keine Bedrängnis auferlegt; so ist die Glaubensrichtung eures Vaters Abraham. Er hat euch Muslime genannt, früher und (nunmehr) in diesem (Buch), auf daß der Gesandte Zeuge über euch sei und ihr Zeugen über die Menschen seid. So verrichtet das Gebet, entrichtet die Abgabe und haltet an Gott fest. Er ist euer Schutzherr. Welch vorzüglicher Schutzherr und welch vorzüglicher Helfer!\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmdclxxiv{\basmalahde Wohl ergeht es den Gläubigen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdclxxv{Die in ihrem Gebet demütig sind,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdclxxvi{Und die sich von unbedachter Rede abwenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdclxxvii{Und die die Abgabe entrichten,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdclxxviii{Und die ihre Scham bewahren,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdclxxix{Außer gegenüber ihren Gattinnen, oder was ihre rechte Hand (an Sklavinnen) besitzt, dann sind sie nicht zu tadeln\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdclxxx{- diejenigen aber, die darüber hinaus (andere) begehren, das sind die, die Übertretungen begehen -,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdclxxxi{Und die auf das ihnen Anvertraute und ihre Verpflichtungen achtgeben,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdclxxxii{Und die ihr Gebet einhalten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdclxxxiii{Das sind die Erben,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdclxxxiv{Die das Paradies erben werden; darin werden sie ewig weilen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdclxxxv{Und wahrlich, Wir schufen den Menschen aus einem entnommenen Ton.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdclxxxvi{Dann machten Wir ihn zu einem Tropfen in einem festen Aufenthaltsort.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdclxxxvii{Dann schufen Wir den Tropfen zu einem Embryo, und Wir schufen den Embryo zu einem Fötus, und Wir schufen den Fötus zu Knochen. Und Wir bekleideten die Knochen mit Fleisch. Dann ließen Wir ihn als eine weitere Schöpfung entstehen. Gott sei gesegnet, der beste Schöpfer!\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdclxxxviii{Dann werdet ihr nach all diesem sterben.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdclxxxix{Dann werdet ihr am Tag der Auferstehung auferweckt werden.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdcxc{Und Wir schufen über euch sieben Bahnen, und Wir lassen die Schöpfung nicht unbeachtet.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdcxci{Und Wir ließen vom Himmel Wasser in einem bestimmten Maß herabkommen und ließen es sich in der Erde aufhalten. Und Wir vermögen wohl, es wieder wegzunehmen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdcxcii{Und Wir ließen euch dadurch Gärten mit Palmen und Weinstöcken entstehen, in denen ihr viele Früchte habt und von denen ihr essen könnt,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdcxciii{Und einen Baum, der aus dem Berg Sinai herauskommt, und der Fett hervorbringt und auch Tunke für die, die essen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdcxciv{Und ihr habt in den Herdentieren einen Grund zum Nachdenken. Wir geben euch von dem, was in ihrem Leib ist, zu trinken. Ihr habt an ihnen allerlei Nutzen, und ihr könnt davon essen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdcxcv{Und auf ihnen und (ebenso) auf den Schiffen werdet ihr getragen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdcxcvi{Und Wir sandten Noach zu seinem Volk. Er sagte: ``O mein Volk, dienet Gott. Ihr habt keinen Gott außer Ihm. Wollt ihr nicht gottesfürchtig sein?"\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdcxcvii{Da sagten die Vornehmen aus seinem Volk, die ungläubig waren: ``Dieser ist nur ein Mensch wie ihr, der einen Vorzug euch gegenüber haben will. Und wenn Gott gewollt hätte, hätte Er Engel herabgesandt. Wir haben bei unseren Vorvätern so etwas nicht gehört.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdcxcviii{Er ist nur ein Mann, der an Besessenheit leidet. So wartet mit ihm eine Weile ab."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdcxcix{Er sagte: ``Mein Herr, unterstütze mich, wo sie mich der Lüge zeihen."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdcc{Da offenbarten Wir ihm: ``Verfertige das Schiff vor unseren Augen und nach unserer Offenbarung. Und wenn unser Befehl kommt und der Ofen brodelt, so laß ein Paar von jeder Art und deine Familie hineingehen, außer dem, gegen den der Spruch vorher erging. Und sprich Mich nicht wegen derer an, die Unrecht tun; sie werden sicher ertränkt werden.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdcci{Und wenn du dich auf dem Schiff gut eingerichtet hast, du und diejenigen, die mit dir sind, dann sag: Lob sei Gott, der uns von den Leuten errettet hat, die Unrecht tun!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdccii{Und sag: Mein Herr, gewähre mir einen gesegnete Unterkunft. Du bist der Beste derer, die Unterkunft gewähren."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdcciii{Darin sind Zeichen. Wir haben (sie) damit der Prüfung ausgesetzt.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdcciv{Dann ließen Wir nach ihnen eine andere Generation entstehen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdccv{Und Wir schickten unter sie einen Gesandten aus ihrer Mitte: «Dienet Gott. Ihr habt keinen Gott außer Ihm. Wollt ihr nicht gottesfürchtig sein?»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdccvi{Und die Vornehmen aus seinem Volk, die ungläubig waren, die die Begegnung mit dem Jenseits für Lüge erklärten und denen Wir im diesseitigen Leben üppigen Wohlstand geschenkt hatten, sagten: «Dieser ist nur ein Mensch wie ihr. Er ißt von dem, was ihr eßt, und trinkt von dem, was ihr trinkt.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdccvii{Und wenn ihr einem Menschen euresgleichen gehorcht, dann werdet ihr gewiß Verlierer sein.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdccviii{Verspricht er euch wirklich, daß ihr, wenn ihr gestorben und zu Staub und Knochen geworden seid, wieder hervorgebracht werdet?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdccix{Weit, weit gefehlt ist das, was euch versprochen wird.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdccx{Es gibt nur unser diesseitiges Leben: Wir sterben, und wir leben (hier), und wir werden nicht auferweckt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdccxi{Er ist nur ein Mann, der eine Lüge gegen Gott erdichtet hat. Und wir glauben ihm nicht.»\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdccxii{Er sagte: «Mein Herr, unterstütze mich, wo sie mich der Lüge zeihen.»\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdccxiii{Er sprach: «Über ein kleines werden sie es bereuen.»\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdccxiv{Da ergriff sie zu Recht der Schrei, und Wir machten sie zur Spreu. Weg mit den Leuten, die Unrecht tun!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdccxv{Dann ließen Wir nach ihnen eine andere Generation entstehen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdccxvi{Keine Gemeinschaft geht ihrer Frist voraus, noch bleibt sie zurück.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdccxvii{Dann schickten Wir unsere Gesandten, einen nach dem anderen. Jedesmal, wenn zu einer Gemeinschaft ihr Gesandter kam, ziehen sie ihn der Lüge. So ließen Wir die einen von ihnen auf die anderen folgen und machten sie zum Gegenstand von Geschichten. Weg mit Leuten, die nicht glauben!\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdccxviii{Dann sandten Wir Mose und seinen Bruder Aaron mit unseren Zeichen und einer offenkundigen Ermächtigung\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdccxix{Zu Pharao und seinen Vornehmen. Sie aber verhielten sich hochmütig. Sie waren Leute mit hoher Gewalt.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdccxx{Sie sagten: «Sollen wir denn zwei Menschen unseresgleichen glauben, wo ihr Volk in unserem Dienst steht?»\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdccxxi{Und sie ziehen sie der Lüge. So gehörten sie zu denen, die ins Verderben gestürzt wurden.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdccxxii{Und Wir ließen Mose das Buch zukommen, auf daß sie der Rechtleitung folgen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdccxxiii{Und Wir machten den Sohn Marias und seine Mutter zu einem Zeichen. Und Wir gaben ihnen Unterkunft auf einer Anhöhe mit Grund und Quellwasser.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdccxxiv{«O ihr Gesandten, eßt von den köstlichen Dingen und tut Gutes. Ich weiß Bescheid über das, was ihr tut.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdccxxv{Diese eure Gemeinschaft ist eine einzige Gemeinschaft. Und Ich bin euer Herr, so fürchtet Mich.»\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdccxxvi{Aber sie spalteten sich in ihrer Angelegenheit untereinander nach verschiedenen Büchern, und jede Partei war froh über das, was sie besaß.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdccxxvii{Laß sie für eine Weile in ihrem Abgrund.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdccxxviii{Meinen sie, daß Wir, wenn Wir ihnen mit Vermögen und Söhnen beistehen,\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdccxxix{Wetteifern, ihnen gute Dinge zu bescheren? Aber sie merken (es) nicht.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdccxxx{Diejenigen, die aus Furcht vor ihrem Herrn erschrecken,\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdccxxxi{Und die an die Zeichen ihres Herrn glauben,\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdccxxxii{Und die ihrem Herrn nicht (andere) beigesellen,\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdccxxxiii{Und die spenden, was sie spenden, während ihre Herzen sich (davor) ängstigen, daß sie zu ihrem Herrn zurückkehren werden,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdccxxxiv{Diese eilen zu den guten Dingen um die Wette und werden darin die Allerersten sein.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdccxxxv{Und Wir fordern von niemandem mehr, als er vermag. Und bei Uns ist ein Buch, das die Wahrheit redet. Und ihnen wird nicht Unrecht getan.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdccxxxvi{Nein, ihre Herzen befinden sich davor in abgründiger Achtlosigkeit. Und da gibt es bei ihnen Werke, die anders sind als diese, und die sie vollbringen.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdccxxxvii{Wenn Wir dann diejenigen von ihnen, die üppig leben, mit der Pein ergreifen, schreien sie um Hilfe.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdccxxxviii{«Schreit nicht um Hilfe heute. Ihr werdet von Uns keine Unterstützung erfahren.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmdccxxxix{Meine Zeichen wurden euch verlesen. Aber ihr pflegtet auf euren Fersen kehrtzumachen,\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmdccxl{Indem ihr euch hochmütig verhieltet und über ihn wie Nachtplauderer faseltet.»\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmdccxli{Haben sie denn nicht die Aussage betrachtet, oder ist zu ihnen gekommen, was nicht zu ihren Vorvätern kam?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmdccxlii{Oder haben sie nicht ihren Gesandten erkannt, so daß sie ihn (nun) verwerfen?\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmdccxliii{Oder sagen sie: «Er leidet an Besessenheit»? Nein, er kam zu ihnen mit der Wahrheit, aber die meisten von ihnen verabscheuen die Wahrheit.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmdccxliv{Würde die Wahrheit ihren Neigungen folgen, verderben würden die Himmel und die Erde, und wer in ihnen ist. Nein, Wir kamen zu ihnen mit ihrer Ermahnung, sie aber wenden sich von ihrer Ermahnung ab.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmdccxlv{Oder verlangst du von ihnen einen Lohn? Der Lohn deines Herrn ist besser. Er ist der beste Versorger.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmdccxlvi{Und wahrlich, du rufst sie zu einem geraden Weg.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmdccxlvii{Jene, die an das Jenseits nicht glauben, weichen vom Weg ab.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmdccxlviii{Würden Wir uns ihrer erbarmen und den Schaden, den sie erleiden, beheben, sie würden im Übermaß ihres Frevels verharren und blind umherirren.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmdccxlix{Und Wir haben sie mit der Pein ergriffen. Sie aber gaben ihrem Herrn nicht nach, und sie demütigten sich nicht.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmdccl{Wenn Wir ihnen dann ein Tor zu einer harten Pein öffnen, sind sie darüber ganz verzweifelt.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmdccli{Er ist es, der euch Gehör, Augenlicht und Herz entstehen ließ. Ihr aber seid wenig dankbar.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmdcclii{Und Er ist es, der euch auf der Erde schuf und vermehrte. Und zu Ihm werdet ihr versammelt werden.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmdccliii{Und Er ist es, der lebendig macht und sterben läßt. Und in seiner Macht steht das Aufeinanderfolgen von Nacht und Tag. Habt ihr denn keinen Verstand?\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmdccliv{Aber nein, sie sagen das gleiche, was die Früheren gesagt haben.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmdcclv{Sie sagen: «Sollen wir, wenn wir gestorben und zu Staub und Knochen geworden sind, auferweckt werden?\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmdcclvi{Dies ist uns und zuvor unseren Vätern versprochen worden. Das sind nichts als die Fabeln der Früheren.»\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmdcclvii{Sprich: Wem gehört die Erde, und wer auf ihr ist, so ihr es wißt?\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmdcclviii{Sie werden sagen: «Gott.» Sprich: Wollt ihr es nicht bedenken?\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmdcclix{Sprich: Wer ist der Herr der sieben Himmel und der Herr des majestätischen Thrones?\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmdcclx{Sie werden sagen: «(Alles) gehört Gott.» Sprich: Wollt ihr nicht gottesfürchtig sein?\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmdcclxi{Sprich: In wessen Hand ist die Herrschaft über alle Dinge, der Schutz gewährt und gegen den kein Schutz gewährt werden kann, so ihr es wißt?\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmdcclxii{Sie werden sagen: «(Alles) gehört Gott.» Sprich: Wieso seid ihr einem Zauber verfallen?\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmdcclxiii{Nein, Wir sind zu ihnen mit der Wahrheit gekommen. Sie aber lügen.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmdcclxiv{Gott hat sich kein Kind genommen. Und es gibt keinen Gott neben Ihm, sonst würde jeder Gott das wegnehmen, was Er geschaffen hat, und die einen von ihnen würden sich den anderen gegenüber überheblich zeigen. Preis sei Gott, (der erhaben ist) über das, was sie da schildern,\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmdcclxv{Der über das Unsichtbare und das Offenbare Bescheid weiß! Erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmdcclxvi{Sprich: Mein Herr, wenn Du mich sehen läßt, was ihnen angedroht wird,\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmdcclxvii{Mein Herr, dann stelle mich nicht unter die Leute, die Unrecht tun.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmdcclxviii{Wahrlich, Wir haben die Macht dazu, dich sehen zu lassen, was Wir ihnen androhen.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmdcclxix{Wehre die schlechte Tat ab mit einer Tat, die besser ist. Wir wissen besser, was sie da schildern.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmdcclxx{Und sprich: Mein Herr, ich suche bei Dir Zuflucht vor den Aufstachelungen der Satane.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmdcclxxi{Und ich suche bei Dir, mein Herr, Zuflucht davor, daß sie mich aufsuchen.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmdcclxxii{Wenn dann der Tod zu einem von ihnen kommt, sagt er: «Mein Herr, bringt mich zurück,\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmdcclxxiii{Vielleicht tue ich Gutes in dem, was ich hinterlassen habe.» Nein, das ist nur ein Wort, das er (so) sagt. Hinter ihnen ist eine Schranke bis zu dem Tag, da sie auferweckt werden.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmdcclxxiv{Und wenn in die Trompete geblasen wird, dann gibt es an jenem Tag zwischen ihnen keine Verwandtschaft mehr, und sie befragen nicht mehr einander.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmdcclxxv{Diejenigen, deren Waagschalen schwer sind, das sind die, denen es wohl ergeht.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmdcclxxvi{Und diejenigen, deren Waagschalen leicht sind, das sind die, die sich selbst verloren haben; in der Hölle werden sie ewig weilen.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmdcclxxvii{Das Feuer schlägt auf ihre Gesichter, und sie fletschen die Zähne.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmdcclxxviii{«Wurden euch nicht meine Zeichen verlesen, ihr aber pflegtet sie für Lüge zu erklären?»\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmdcclxxix{Sie sagen: «Unser Herr, unser Unglück hat uns besiegt, und wir waren abgeirrte Leute.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmdcclxxx{Unser Herr, bring uns aus ihr heraus. Wenn wir rückfällig werden, dann sind wir (wirklich) Leute, die Unrecht tun.»\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmdcclxxxi{Er spricht: «Seid darin verabscheut, und sprecht Mich nicht an.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmdcclxxxii{Eine Gruppe von meinen Dienern pflegte zu sagen: >Unser Herr, wir glauben, so vergib uns und erbarme dich unser. Du bist der Beste derer, die sich erbarmen.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmdcclxxxiii{Ihr aber nahmt sie zum Gegenstand des Spottes, bis ihr ihretwegen meine Ermahnung vergaßet, und ihr lachtet über sie.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmdcclxxxiv{Ich habe sie heute dafür, daß sie geduldig waren, damit entlohnt, daß sie die Erfolgreichen sind.»\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmdcclxxxv{Er spricht: «Wie viele Jahre habt ihr in der Erde verweilt?»\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmdcclxxxvi{Sie sagen: «Verweilt haben wir einen Tag oder einen Teil von einem Tag. Frag die, die rechnen können.»\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmdcclxxxvii{Er spricht: «Ihr habt nur kurz verweilt, wenn ihr es nur wüßtet!\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmdcclxxxviii{Meint ihr denn, Wir hätten euch zum sinnlosen Spiel erschaffen und ihr würdet nicht zu Uns zurückgebracht?»\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmdcclxxxix{Erhaben ist Gott, der wahre König. Es gibt keinen Gott außer Ihm, dem Herrn des ehrwürdigen Thrones.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmdccxc{Und wer neben Gott einen anderen Gott anruft, für den er keinen Beweis hat, der wird bei seinem Herrn Rechenschaft ablegen. Den Ungläubigen wird es nicht wohl ergehen.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmdccxci{Und sprich: Mein Herr, vergib und erbarme dich. Und Du bist der Beste derer, die sich erbarmen.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmdccxcii{\basmalahde Das ist eine Sure, die Wir hinabgesandt und verpflichtend gemacht haben. Und Wir haben darin deutliche Zeichen hinabgesandt, auf daß ihr es bedenket.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdccxciii{Wenn eine Frau und ein Mann Unzucht begehen, dann geißelt jeden von ihnen mit hundert Hieben. Habt kein Mitleid mit ihnen angesichts (der Rechtsbestimmungen) der Religion Gottes, so ihr an Gott und den Jüngsten Tag glaubt. Und bei der Vollstreckung der Pein an ihnen soll eine Gruppe von den Gläubigen zugegen sein.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdccxciv{Der Mann, der Unzucht begangen hat, darf nur eine Frau, die Unzucht begangen hat, oder eine Polytheisten heiraten. Die Frau, die Unzucht begangen hat, darf nur ein Mann, der Unzucht begangen hat, oder ein Polytheist heiraten. Den Gläubigen ist dies verboten.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdccxcv{Diejenigen, die den unter Schutz gestellten Frauen Untreue vorwerfen und hierauf nicht vier Zeugen beibringen, die sollt ihr mit achtzig Hieben geißeln. Nehmt von ihnen nie mehr eine Zeugenaussage an - das sind die (wahren) Frevler -,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdccxcvi{Mit Ausnahme derer, die danach umkehren und Besserung zeigen. Denn Gott ist voller Vergebung und barmherzig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdccxcvii{Im Falle derer, die ihren Gattinnen Untreue vorwerfen, aber keine Zeugen haben außer sich selbst, besteht die Zeugenaussage eines solchen Mannes darin, daß er viermal bei Gott bezeugt, er gehöre zu denen, die die Wahrheit sagen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdccxcviii{Und zum fünften Mal (bezeugt), der Fluch Gottes komme über ihn, wenn er ein Lügner sein sollte.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdccxcix{Von ihr wehrt es die Pein ab, daß sie viermal bei Gott bezeugt, er sei ein Lügner,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdccc{Und zum fünften Mal (bezeugt), der Zorn Gottes komme über sie, wenn er zu denen gehören sollte, die die Wahrheit sagen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdccci{Und ohne die Huld Gottes gegen euch und seine Barmherzigkeit, und wäre Gott nicht der, der sich zuwendet und weise ist...\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdcccii{Diejenigen, die die Lüge vorgebracht haben, sind eine gewisse Gruppe von euch. Betrachtet es nicht als ein Übel für euch; nein, es ist etwas Gutes für euch. Einem jeden von ihnen wird zuteil, was er an Schuld erworben hat. Und für den, der den Hauptanteil daran auf sich genommen hat, ist eine gewaltige Pein bestimmt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdccciii{Hätten doch, als ihr es hörtet, die gläubigen Männer und Frauen eine gute Meinung voneinander gehabt und gesagt: «Es ist doch eine offenkundige Lüge»!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdccciv{Hätten sie doch darüber vier Zeugen beigebracht! Da sie aber die Zeugen nicht beigebracht haben, sind sie eben bei Gott die, die lügen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdcccv{Und ohne die Huld Gottes gegen euch und seine Barmherzigkeit im Diesseits und Jenseits würde euch für das, was ihr so breit beredet habt, eine gewaltige Pein erfassen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdcccvi{Als ihr es mit eurer Zunge aufgegriffen habt und ihr mit eurem Munde das gesagt habt, wovon ihr kein Wissen hattet, und es als eine leichte Sache betrachtet habt, während es bei Gott eine ungeheuerliche Sache ist.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdcccvii{Hättet ihr doch, als ihr es hörtet, gesagt: «Es steht uns nicht zu, darüber zu reden. Preis sei Dir! Das ist doch eine gewaltige Verleumdung»!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdcccviii{Gott ermahnt euch, nie wieder so etwas zu tun, so ihr gläubig seid.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdcccix{Und Gott macht euch die Zeichen deutlich. Und Gott weiß Bescheid und ist weise.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdcccx{Für diejenigen, die es gern möchten, daß sich das Schändliche unter den Gläubigen verbreitet, ist eine schmerzhafte Pein bestimmt im Diesseits und Jenseits. Und Gott weiß, ihr aber wißt nicht Bescheid.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdcccxi{Und ohne die Huld Gottes gegen euch und seine Barmherzigkeit, und hätte Gott nicht Mitleid und wäre Er nicht barmherzig...\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdcccxii{O ihr, die ihr glaubt, folgt nicht den Fußstapfen des Satans. Wer den Fußstapfen des Satans folgt, der gebietet das Schändliche und Verwerfliche. Und ohne die Huld Gottes gegen euch und seine Barmherzigkeit würde keiner von euch jemals geläutert sein. Aber Gott läutert, wen Er will. Und Gott hört und weiß alles.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdcccxiii{Und diejenigen, die Überfluß und beachtliches Vermögen besitzen, sollen nicht schwören, sie würden den Verwandten, den Bedürftigen und denen, die auf dem Weg Gottes ausgewandert sind, nichts zukommen lassen. Sie sollen verzeihen und nachlassen. Liebt ihr es selbst nicht, daß Gott euch vergibt? Gott ist voller Vergebung und barmherzig.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdcccxiv{Diejenigen, die den unter Schutz gestellten, nichts ahnenden gläubigen Frauen Untreue vorwerfen, sind verflucht im Diesseits und Jenseits. Bestimmt ist für sie eine gewaltige Pein,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdcccxv{Am Tag, da ihre Zungen und ihre Hände und ihre Füße gegen sie Zeugnis ablegen werden über das, was sie zu tun pflegten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdcccxvi{An jenem Tag wird Gott es ihnen entsprechend dem gegen sie ergangenen wahren Gerichtsurteil voll erstatten. Und sie werden zu wissen bekommen, daß Gott die offenkundige Wahrheit ist.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdcccxvii{Schlechte Frauen gehören zu schlechten Männern, und schlechte Männer gehören zu schlechten Frauen. Gute Frauen gehören zu guten Männern, und gute Männer gehören zu guten Frauen. Diese werden freigesprochen von dem, was man (über sie) redet. Bestimmt ist für sie Vergebung und trefflicher Unterhalt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdcccxviii{O ihr, die ihr glaubt, betretet nicht Häuser, die nicht eure (eigenen) Häuser sind, bis ihr euch bemerkbar gemacht und ihre Bewohner begrüßt habt. Das ist besser für euch, auf daß ihr es bedenket.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdcccxix{Wenn ihr niemanden darin findet, dann tretet nicht ein, bis man es euch erlaubt. Und wenn man zu euch sagt: «Kehrt um», dann sollt ihr umkehren. Das ist lauterer für euch. Und Gott weiß, was ihr tut.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdcccxx{Es ist für euch kein Vergehen, unbewohnte Häuser zu betreten, in denen sich eine Nutznießung für euch befindet. Und Gott weiß, was ihr offenlegt und was ihr verschweigt.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdcccxxi{Sprich zu den gläubigen Männern, sie sollen ihre Blicke senken und ihre Scham bewahren. Das ist lauterer für sie. Gott hat Kenntnis von dem, was sie machen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdcccxxii{Und sprich zu den gläubigen Frauen, sie sollen ihre Blicke senken und ihre Scham bewahren, ihren Schmuck nicht offen zeigen, mit Ausnahme dessen, was sonst sichtbar ist. Sie sollen ihren Schleier auf den Kleiderausschnitt schlagen und ihren Schmuck nicht offen zeigen, es sei denn ihren Ehegatten, ihren Vätern, den Vätern ihrer Ehegatten, ihren Söhnen, den Söhnen ihrer Ehegatten, ihren Brüdern, den Söhnen ihrer Brüder und den Söhnen ihrer Schwestern, ihren Frauen, denen, die ihre rechte Hand besitzt, den männlichen Gefolgsleuten, die keinen Trieb mehr haben, den Kindern, die die Blöße der Frauen nicht beachten. Sie sollen ihre Füße nicht aneinanderschlagen, damit man gewahr wird, was für einen Schmuck sie verborgen tragen. Bekehrt euch allesamt zu Gott, ihr Gläubigen, auf daß es euch wohl ergehe.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdcccxxiii{Und verheiratet die Ledigen unter euch und die Rechtschaffenen von euren Sklaven und euren Sklavinnen. Wenn sie arm sind, wird Gott sie durch seine Huld reich machen. Und Gott umfaßt und weiß alles.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdcccxxiv{Diejenigen, die keine Möglichkeit zum Heiraten finden, sollen keusch bleiben, bis Gott sie durch seine Huld reich macht. Und denjenigen aus der Reihe derer, die eure rechte Hand besitzt, die einen Freibrief zu erhalten suchen, sollt ihr einen Freibrief ausstellen, falls ihr etwas Gutes an ihnen feststellt. Und gebt ihnen etwas vom Vermögen Gottes, das Er euch gegeben hat. Und zwingt nicht eure Sklavinnen, wenn sie sich unter Schutz stellen wollen, zur Hurerei im Trachten nach den Gütern des diesseitigen Lebens. Und wenn einer sie dazu zwingt, so ist Gott, nachdem sie gezwungen worden sind, voller Vergebung und barmherzig.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdcccxxv{Und Wir haben zu euch Zeichen hinabgesandt, die (alles) deutlich machen, und ein Beispiel aus den Berichten über die, die vor euch dahingegangen sind, und eine Ermahnung für die Gottesfürchtigen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdcccxxvi{Gott ist das Licht der Himmel und der Erde. Sein Licht ist einer Nische vergleichbar, in der eine Lampe ist. Die Lampe ist in einem Glas. Das Glas ist, als wäre es ein funkelnder Stern. Es wird angezündet von einem gesegneten Baum, einem Ölbaum, weder östlich noch westlich, dessen Öl fast schon leuchtet, auch ohne daß das Feuer es berührt hätte. Licht über Licht. Gott führt zu seinem Licht, wen Er will, und Gott führt den Menschen die Gleichnisse an. Und Gott weiß über alle Dinge Bescheid.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdcccxxvii{Das steht) in Häusern, für die Gott erlaubt hat, daß sie errichtet werden und daß darin seines Namens gedacht wird. Ihn preisen darin, am Morgen und am Abend,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdcccxxviii{Männer, die weder Handel noch Kaufgeschäft ablenken vom Gedenken Gottes, von der Verrichtung des Gebets und der Entrichtung der Abgabe, die einen Tag fürchten, an dem Herzen und Augenlicht umgekehrt werden,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdcccxxix{Damit Gott ihnen das Beste vergelte von dem, was sie getan haben, und ihnen von seiner Huld noch mehr gebe. Und Gott beschert den Lebensunterhalt, wem Er will, ohne (viel) zu rechnen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdcccxxx{Die Werke derer, die ungläubig sind, sind wie eine Luftspiegelung in einer Ebene. Der Durstige hält sie für Wasser. Kommt er aber dorthin, findet er, daß es nichts ist. Er findet aber Gott da. Und Er erstattet ihm seine Rechnung voll. Gott ist schnell im Abrechnen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdcccxxxi{Oder (sie sind) wie Finsternisse in einem tiefen Meer, das von einer Woge überdeckt ist, über der eine Woge liegt, über der wiederum eine Wolke liegt: Finsternisse, eine über der anderen. Wenn er seine Hand ausstreckt, kann er sie kaum sehen. Und wem Gott kein Licht verschafft, für den gibt es kein Licht.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdcccxxxii{Hast du nicht gesehen, daß (alle) Gott preisen, die in den Himmeln und auf der Erde sind, und die Vögel mit ausgebreiteten Flügeln? Jeder kennt sein Gebet und seinen Lobpreis. Und Gott weiß, was sie tun.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdcccxxxiii{Und Gott gehört die Königsherrschaft der Himmel und der Erde. Und zu Gott führt der Lebensweg.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdcccxxxiv{Hast du nicht gesehen, daß Gott die Wolken dahertreibt, sie dann zusammenfügt und dann zu einem Haufen macht? Da siehst du den Platzregen dazwischen herauskommen. Und Er sendet vom Himmel Wolkenberge herab, mit Hagel darin; damit trifft Er, wen Er will, und Er wehrt ihn ab, von wem Er will. Das Aufleuchten seiner Blitze nimmt beinahe das Augenlicht hinweg.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdcccxxxv{Gott läßt Nacht und Tag sich abwechseln. Darin ist eine Lehre für Leute, die Einsicht haben.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdcccxxxvi{Und Gott hat jedes Getier aus Wasser geschaffen. Es gibt unter ihnen solche, die auf ihrem Bauch gehen, und solche, die auf zwei Beinen gehen, und wieder solche, die auf vieren gehen. Gott erschafft, was Er will. Gott hat Macht zu allen Dingen.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdcccxxxvii{Wir haben Zeichen hinabgesandt, die (alles) deutlich machen. Und Gott führt, wen Er will, zu einem geraden Weg.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdcccxxxviii{Und sie sagen: «Wir glauben an Gott und an den Gesandten, und wir gehorchen.» Dann, nach diesem (Wort), kehrt sich ein Teil von ihnen ab. Das sind ja keine Gläubigen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdcccxxxix{Und wenn sie zu Gott und seinem Gesandten gerufen werden, damit er zwischen ihnen urteile, wendet sich gleich ein Teil von ihnen ab.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdcccxl{Wenn aber das Recht auf ihrer Seite ist, dann kommen sie zu ihm, bereit, sich zu unterwerfen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdcccxli{Ist in ihren Herzen Krankheit, oder haben sie Zweifel, oder fürchten sie, daß Gott gegen sie ungerecht sein könnte, und auch sein Gesandter? Nein, sie sind eben die, die Unrecht tun.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdcccxlii{Die Rede der Gläubigen, wenn sie zu Gott und seinem Gesandten gerufen werden, damit er zwischen ihnen urteile, besteht darin, daß sie sagen: «Wir hören, und wir gehorchen.» Das sind die, denen es wohl ergeht.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdcccxliii{Diejenigen, die Gott und seinem Gesandten gehorchen, Gott fürchten und sich vor Ihm hüten, das sind die Erfolgreichen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdcccxliv{Und sie haben bei Gott ihren eifrigsten Eid geschworen, sie würden, wenn du es ihnen befiehlst, hinausziehen. Sprich: Schwört nicht, (leistet lieber) einen geziemenden Gehorsam. Gott hat Kenntnis von dem, was ihr tut.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdcccxlv{Sprich: Gehorchet Gott und gehorchet dem Gesandten. Wenn ihr euch abkehrt, dann obliegt ihm nur das, was ihm auferlegt ist, und euch obliegt, was euch auferlegt ist. Wenn ihr ihm gehorcht, folgt ihr der Rechtleitung. Und dem Gesandten obliegt nur die deutliche Ausrichtung (der Botschaft).\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdcccxlvi{Gott hat denen von euch, die glauben und die guten Werke tun, versprochen, daß Er sie zu Nachfolgern auf der Erde bestellen wird, wie Er diejenigen, die vor ihnen lebten, zu Nachfolgern bestellt hat; daß Er ihnen ihrer Religion, die Er mit Gefallen für sie festgelegt hat, zu einer angesehenen Stellung verhelfen wird; und daß Er ihnen, nachdem sie in Angst gelebt haben, statt, dessen Sicherheit gewähren wird. Sie dienen Mir und gesellen Mir nichts bei. Diejenigen, die hernach ungläubig sind, das sind die (wahren) Frevler.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdcccxlvii{Und verrichtet das Gebet und entrichtet die Abgabe, und gehorchet dem Gesandten, auf daß ihr Erbarmen findet.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdcccxlviii{Und du sollst nicht meinen, daß diejenigen, die ungläubig sind, auf der Erde etwas vereiteln können. Ihre Heimstätte ist das Feuer - welch schlimmes Ende!\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdcccxlix{Ö ihr, die ihr glaubt, diejenigen, die eure rechte Hand besitzt, und diejenigen von euch, die noch nicht das Unterscheidungsalter erreicht haben, sollen euch zu drei Zeiten um Erlaubnis bitten: vor dem Gebet am Frühmorgen, wenn ihr zur Mittagszeit eure Kleider ablegt, und nach dem Abendgebet. Das sind drei Blößen von euch. Es ist nach diesen (Zeiten) für euch und für sie kein Vergehen. Sie gehen ja oft unter euch umher, und das tut ihr untereinander. So macht Gott euch die Zeichen deutlich. Und Gott weiß Bescheid und ist weise.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdcccl{Und wenn die Kinder unter euch das Unterscheidungsalter erreicht haben, dann sollen sie euch um Erlaubnis bitten, wie diejenigen um Erlaubnis gebeten haben, die vor ihnen lebten. So macht Gott euch seine Zeichen deutlich. Und Gott weiß Bescheid und ist weise.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdcccli{Und für die unter den Frauen, die sich zur Ruhe gesetzt haben und nicht mehr zu heiraten hoffen, ist es kein Vergehen, wenn sie ihre Kleider ablegen, ohne daß sie jedoch ihren Schmuck zur Schau stellen. Und besser für sie wäre, daß sie sich dessen enthalten. Und Gott hört und weiß alles.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdccclii{Es ist für den Blinden kein Grund zur Bedrängnis, es ist für den Krüppel kein Grund zur Bedrängnis, es ist für den Kranken kein Grund zur Bedrängnis, es ist auch für euch kein Grund zur Bedrängnis, wenn ihr in euren Häusern eßt, in den Häusern eurer Väter oder in den Häusern eurer Mütter, in den Häusern eurer Brüder oder in den Häusern eurer Schwestern, in den Häusern eurer Onkel väterlicherseits oder in den Häusern eurer Tanten väterlicherseits, in den Häusern eurer Onkel mütterlicherseits oder in den Häusern eurer Tanten mütterlicherseits, in einem Haus, dessen Schlüssel in eurem Besitz ist, oder (im Haus) eines Freundes. Es ist für euch kein Vergehen, gemeinsam oder getrennt zu essen. Wenn ihr nun Häuser betretet, so grüßt einander mit einem gesegneten, guten Gruß, der von Gott kommt. So macht Gott euch die Zeichen deutlich, auf daß ihr verständig werdet.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdcccliii{Die Gläubigen, das sind diejenigen, die an Gott und seinen Gesandten glauben und, wenn sie in einer gemeinsamen Angelegenheit bei ihm sind, nicht weggehen, ohne ihn um Erlaubnis zu bitten. Diejenigen, die dich um Erlaubnis bitten, das sind die, die an Gott und seinen Gesandten glauben. Wenn sie dich nun wegen eines ihrer Anliegen um Erlaubnis bitten, so gib, wem von ihnen du willst, Erlaubnis. Und bitte um Vergebung für sie. Gott ist voller Vergebung und barmherzig.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdcccliv{Ihr sollt unter euch den Aufruf des Gesandten nicht wie den Aufruf eines von euch an die anderen bewerten. Gott kennt wohl diejenigen von euch, die sich unbemerkt davonstehlen, indem sie sich verstecken. Diejenigen, die seinem Befehl zuwiderhandeln, sollen sich in acht nehmen, daß nicht eine Versuchung sie trifft, oder eine schmerzhafte Pein sie trifft.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdccclv{Seht, Gott gehört, was in den Himmeln und auf der Erde ist. Er weiß, in welchem Zustand ihr euch befindet. Und am Tag, da sie zu Ihm zurückgebracht werden, wird Er ihnen kundtun, was sie getan haben. Und Gott weiß über alle Dinge Bescheid.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdccclvi{\basmalahde Gesegnet sei der, der auf seinen Diener die Unterscheidungsnorm herabgesandt hat, damit er den Weltenbewohnern ein Warner sei,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdccclvii{Er, dem die Königsherrschaft der Himmel und der Erde gehört, der sich kein Kind genommen hat und der keinen Teilhaber an der Königsherrschaft hat und jedes Ding erschaffen und ihm sein Maß gegeben hat.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdccclviii{Und sie haben sich an seiner Stelle Götter genommen, die nichts erschaffen, aber selbst erschaffen werden, und die sich selbst weder Schaden noch Nutzen bringen können, und die weder über Tod noch über Leben, noch über Auferweckung verfügen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdccclix{Und diejenigen, die ungläubig sind, sagen: «Das ist ja nichts als eine Lüge, die er erdichtet hat und bei der andere Leute ihm geholfen haben.» Sie begehen da Ungerechtigkeit und Falschaussage.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdccclx{Und sie sagen: «Es sind die Fabeln der Früheren, die er sich aufgeschrieben hat. Sie werden ihm doch morgens und abends diktiert.»\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdccclxi{Sprich: Herabgesandt hat ihn der, der weiß, was in den Himmeln und auf der Erde geheim ist. Er ist voller Vergebung und barmherzig.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdccclxii{Und sie sagen: «Was ist mit diesem Gesandten, daß er Speise ißt und auf den Märkten umhergeht? Wäre doch zu ihm ein Engel herabgesandt worden, daß er mit ihm ein Warner sei!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdccclxiii{Oder wäre doch ihm ein Schatz überbracht worden, oder hätte er doch einen Garten, von dem er essen könnte!» Und die, die Unrecht tun, sagen: «Ihr folgt doch nur einem Mann, der einem Zauber verfallen ist.»\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdccclxiv{Schau, wie sie dir Gleichnisse anführen. Dabei sind sie abgeirrt und können keinen Weg mehr finden.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdccclxv{Gesegnet sei der, der dir, wenn Er will, etwas Besseres als dies zuteilen kann: Gärten, unter denen Bäche fließen, und der dir Schlösser zuteilen kann.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdccclxvi{Nein, sie erklären die Stunde für Lüge. Und Wir haben für die, die die Stunde für Lüge erklären, einen Feuerbrand bereitet.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdccclxvii{Wenn er sie aus der Ferne sieht, hören sie bereits sein Grollen und Aufheulen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdccclxviii{Und wenn sie aneinandergebunden da in einen engen Ort geworfen werden, rufen sie dort: O weh, was für ein Verderben!\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdccclxix{«Ruft heute nicht nur einmal: O weh, was für ein Verderben!, sondern ruft vielmal: O weh, was für ein Verderben!»\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdccclxx{Sprich: Ist das besser oder der Garten der Ewigkeit, der den Gottesfürchtigen versprochen ist, und der für sie Belohnung und Reiseziel ist?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdccclxxi{Sie haben darin, was sie wollen, und sie werden darin ewig weilen. Das ist ein Versprechen, das deinem Herrn obliegt und eingefordert werden kann.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdccclxxii{Und am Tag, da Er sie und das, was sie anstelle Gottes verehren, versammelt und Er spricht: «Seid ihr es gewesen, die diese meine Diener irregeführt haben, oder sind sie selbst vom Weg abgeirrt?»\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdccclxxiii{Sie sagen: «Preis sei Dir! Es ziemte uns nicht, uns an deiner Stelle andere Freunde zu nehmen. Aber Du hast sie und ihre Väter genießen lassen, so daß sie die Ermahnung vergessen haben und ein verlorenes Volk geworden sind.»\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdccclxxiv{- «Nun haben sie euch in dem, was ihr sagt, der Lüge geziehen. Da könnt ihr weder etwas abwenden noch Unterstützung erfahren. Und wer von euch Unrecht tut, den lassen Wir eine große Pein kosten.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdccclxxv{Und Wir haben vor dir keine Gesandten geschickt mit einem anderen Verhalten, als daß sie Speise gegessen haben und auf den Märkten umhergegangen sind. Und Wir haben die einen von euch zur Versuchung für die anderen gemacht, (um festzustellen), ob ihr geduldig seid. Und dein Herr sieht alles.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdccclxxvi{Und diejenigen, die nicht erwarten, Uns zu begegnen, sagen: «Wären doch die Engel auf uns herabgesandt worden, oder könnten wir doch unseren Herrn sehen!» Sie sind hochmütig in bezug auf sich selbst, und sie erheben sich in großer Rebellion.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdccclxxvii{Am Tag, da sie die Engel sehen, an dem Tag gibt es für die Übeltäter keine frohe Botschaft, und sie sagen: «Tabu, das sei verwehrt!»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdccclxxviii{Und Wir wenden uns den Werken, die sie getan haben, zu und machen sie zu verwehtem Staub.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdccclxxix{Die Gefährten des Paradieses haben an jenem Tag einen besseren Aufenthalt und einen schöneren Ruheplatz.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdccclxxx{Und am Tag, da sich der Himmel durch die Wolken hindurch spaltet und die Engel eindrucksvoll herabgesandt werden,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdccclxxxi{An dem Tag gehört die wahre Königsherrschaft dem Erbarmer. Es ist für die Ungläubigen ein schwerer Tag.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdccclxxxii{Und am Tag, da derjenige, der Unrecht tut, sich in die Hände beißt und sagt: «O hätte ich doch den Weg mit dem Gesandten eingeschlagen!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdccclxxxiii{O weh mir! Hätte ich mir doch nicht den Soundso zum Vertrauten genommen!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdccclxxxiv{Er hat mich ja von der Ermahnung abirren lassen, nachdem sie zu mir gekommen war.» Und der Satan läßt den Menschen im Stich.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdccclxxxv{Und der Gesandte sagt: «O mein Herr, mein Volk hält diesen Koran für etwas, was gemieden werden soll.»\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdccclxxxvi{So haben Wir für jeden Propheten einen Feind aus den Reihen der Übeltäter bestellt. Und dein Herr genügt als Führer und Helfer.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdccclxxxvii{Und diejenigen, die ungläubig sind, sagen: «Wäre doch der Koran auf ihn als Ganzes herabgesandt worden!» So (geschieht es aber), damit Wir dein Herz mit ihm festigen. Und Wir haben ihn Abschnitt für Abschnitt vorgetragen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdccclxxxviii{Und sie kommen zu dir mit keinem beschriebenen Fall, ohne daß Wir dir die Wahrheit und eine schönere Erläuterung brächten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdccclxxxix{Diejenigen, die auf ihren Gesichtern zur Hölle versammelt werden, befinden sich in einer schlimmeren Lage und sind vom (rechten) Weg weiter abgeirrt.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdcccxc{Und Wir ließen Mose das Buch zukommen und bestellten ihm seinen Bruder Aaron, (die Last) mitzutragen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdcccxci{Da sprachen Wir: «Geht zu den Leuten, die unsere Zeichen für Lüge erklärt haben.» Und Wir zerstörten sie völlig.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdcccxcii{Und auch die Leute des Noach, als sie die Gesandten der Lüge ziehen, ließen Wir ertrinken und machten sie zu einem Zeichen für die Menschen. Und Wir haben denen, die Unrecht tun, eine schmerzhafte Pein bereitet.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdcccxciii{Desgleichen auch die Aad, die Thamud, die Leute des Brunnens und viele Generationen dazwischen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdcccxciv{Einem jeden Volk führten Wir Gleichnisse an, und Wir gaben jedes (von ihnen) dem völligen Verderben preis.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdcccxcv{Sie kamen doch an der Stadt vorbei, auf die der Unheilsregen niederging. Haben sie sie denn nicht gesehen? Nein, sie erwarten keine Auferweckung.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdcccxcvi{Und wenn sie dich sehen, nehmen sie dich nur zum Gegenstand des Spottes: «Ist das der, den Gott zum Gesandten hat erstehen lassen?\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdcccxcvii{Beinahe hätte er uns von unseren Göttern abirren lassen, wenn wir nicht beharrlich an ihnen festgehalten hätten.» Sie werden, wenn sie die Pein sehen, zu wissen bekommen, wer weiter vom Weg abgeirrt ist.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdcccxcviii{Hast du den gesehen, der sich seine Neigung zu seinem Gott nimmt? Willst du denn ein Sachwalter über ihn sein?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdcccxcix{Oder meinst du, daß die meisten von ihnen hören oder Verstand haben? Sie sind doch nur wie das Vieh, nein, sie irren noch weiter vom Weg ab.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmcm{Hast du denn nicht zu deinem Herrn geschaut, wie Er den Schatten lang werden läßt? Wenn Er gewollt hätte, hätte Er ihn stillstehen lassen. Alsdann machen Wir die Sonne zu einem Hinweis auf ihn.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmcmi{Dann ziehen Wir ihn zu Uns in einem geringen Maß ein.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmcmii{Und Er ist es, der euch die Nacht zur Kleidung und den Schlaf zum Ausruhen macht, und der den Tag zum Aufstehen macht.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmcmiii{Und Er ist es, der die Winde als frohe Kunde seiner Barmherzigkeit vorausschickt. Und Wir lassen vom Himmel ein reines Wasser herabkommen,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmcmiv{Um damit eine abgestorbene Ortschaft zu beleben und um es vielen von dem, was Wir erschaffen haben, Vieh und Menschen, zu trinken zu geben.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcmv{Und Wir haben es unter ihnen auf verschiedene Weise dargestellt, damit sie es bedenken. Doch bestehen die meisten Menschen auf dem Unglauben.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcmvi{Und wenn Wir gewollt hätten, hätten Wir in jeder Stadt einen Warner erstehen lassen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcmvii{So gehorche nicht den Ungläubigen und setze dich damit gegen sie ein mit großem Einsatz.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcmviii{Und Er ist es, der die beiden Meere zugleich hat entstehen lassen: das eine süß und erfrischend, das andere salzig und bitter. Und Er hat zwischen ihnen beiden eine Schranke und eine verwehrte Absperrung errichtet.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcmix{Und Er ist es, der aus Wasser einen Menschen erschaffen und ihn zu einer Sippe von Verwandten und Schwägern gemacht hat. Und dein Herr ist allmächtig.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcmx{Und sie verehren anstelle Gottes, was ihnen weder nützt noch schadet. Und der Ungläubige leistet Beistand gegen seinen Herrn.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcmxi{Und Wir haben dich nur als Freudenboten und Warner gesandt.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcmxii{Sprich: Ich verlange von euch keinen Lohn dafür, es sei denn, einer will einen Weg zu seinem Herrn einschlagen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcmxiii{Und vertraue auf den Lebendigen, der nicht stirbt, und singe sein Lob. Er genügt als der, der Kenntnis von den Sünden seiner Diener hat,\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcmxiv{Der die Himmel und die Erde, und was dazwischen ist, in sechs Tagen erschaffen und sich dann auf dem Thron zurechtgesetzt hat, Er, der Erbarmer. So frag einen, der von Ihm Kenntnis hat.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcmxv{Und wenn zu ihnen gesagt wird: «Werft euch vor dem Erbarmer nieder», sagen sie: «Was ist denn der Erbarmer? Sollen wir uns (einfach) vor dem niederwerfen, was du uns befiehlst?» Und das mehrt in ihnen nur die Abneigung.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcmxvi{Gesegnet sei der, der im Himmel Sternzeichen gesetzt und darin eine Leuchte und einen hellen Mond gesetzt hat!\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcmxvii{Und Er ist es, der die Nacht und den Tag gemacht hat, so daß sie aufeinanderfolgen, für den, der es bedenken oder Dankbarkeit zeigen will.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcmxviii{Und die Diener des Erbarmers sind die, die demütig auf der Erde umhergehen und, wenn die Törichten sie anreden, sagen: «Frieden!»\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcmxix{Und die, welche die Nacht vor ihrem Herrn verbringen, indem sie sich niederwerfen und aufrecht stehen.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcmxx{Und die, die sagen: «Unser Herr, wende ab von uns die Pein der Hölle.» - Ihre Pein lastet ja unabwendbar;\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcmxxi{Sie ist schlimm als Aufenthalt und Bleibe.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcmxxii{Und die, die, wenn sie spenden, weder verschwenderisch noch zurückhaltend sind, sondern die Mitte dazwischen halten.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmcmxxiii{Und die, die neben Gott keinen anderen Gott anrufen und den Menschen nicht töten, den Gott für unantastbar erklärt hat, es sei denn bei vorliegender Berechtigung, und die keine Unzucht begehen. - Wer das tut, hat die Folge der Sünde zu erleiden;\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmcmxxiv{Die Pein wird ihm am Tag der Auferstehung verdoppelt, darin wird er in Schmach ewig weilen,\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmcmxxv{Außer dem, der umkehrt, glaubt und gute Werke tut; Gott wird ihnen ihre schlechten Taten gegen gute eintauschen; und Gott ist voller Vergebung und barmherzig.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmcmxxvi{Und wer umkehrt und Gutes tut, der wendet sich in wahrhaftiger Umkehr Gott zu.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmcmxxvii{Und (auch) die, die das Falsche nicht bezeugen und, wenn sie unbedachte Rede im Vorbeigehen hören, würdevoll weitergehen.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcmxxviii{Und die, die, wenn sie mit den Zeichen ihres Herrn ermahnt werden, ihnen gegenüber nicht taub und blind niederfallen.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmcmxxix{Und die, die sagen: «Unser Herr, schenke uns an unseren Gattinnen und unseren Nachkommenschaften Grund zur Freude, und mache uns zu einem Vorbild für die Gottesfürchtigen.»\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmcmxxx{All diese werden mit dem Obergemach dafür belohnt werden,, daß sie geduldig waren; und ihnen wird darin Gruß und Friede entgegengebracht;\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcmxxxi{Darin werden sie ewig weilen. Schön ist es als Aufenthalt und Bleibe.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcmxxxii{Sprich: Mein Herr würde sich nicht um euch kümmern, würdet ihr nicht (zu Ihm) rufen. Ihr habt doch (alles) für Lüge erklärt, und nun wird es unabwendbar.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcmxxxiii{\basmalahde Taa Sien Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcmxxxiv{Dies sind die Zeichen des deutlichen Buches.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcmxxxv{Du magst dich noch selbst umbringen, weil sie nicht gläubig sind.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmcmxxxvi{Wenn Wir wollten, könnten Wir vom Himmel ein Zeichen auf sie hinabsenden, so daß sich ihre Nacken davor unterwerfen würden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcmxxxvii{Keine neue Ermahnung kommt vom Erbarmer zu ihnen, ohne daß sie sich davon abwenden.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcmxxxviii{Sie haben (sie) ja für Lüge erklärt. So wird zu ihnen die Kunde kommen von dem, worüber sie immer wieder gespottet haben.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcmxxxix{Haben sie nicht auf die Erde geschaut, wie viele treffliche Arten Wir auf ihr haben wachsen lassen?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcmxl{Darin ist wahrlich ein Zeichen. Aber die meisten von ihnen sind nicht gläubig.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcmxli{Und dein Herr ist der Mächtige, der Barmherzige.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcmxlii{Als dein Herr Mose rief: «Geh zum Volk, das Unrecht tut,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcmxliii{Zum Volk Pharaos, ob sie nicht gottesfürchtig sein wollen.»\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmcmxliv{Er sagte: «Mein Herr, ich fürchte, daß sie mich der Lüge zeihen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcmxlv{Und meine Brust ist eng, und meine Zunge ist nicht gelöst. So schicke Aaron.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcmxlvi{Auch haben sie gegen mich eine Sünde geltend zu machen; so fürchte ich, daß sie mich töten.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmcmxlvii{Er sprach: «Nein. Geht beide hin mit unseren Zeichen. Wir sind mit euch und hören zu.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmcmxlviii{Geht zu Pharao und sagt: >Wir sind der Gesandte des Herrn der Welten,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmcmxlix{Du sollst die Kinder Israels mit uns wegschicken.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmcml{Er sagte: «Haben wir dich nicht als Kind unter uns aufgezogen, und hast du nicht viele Jahre deines Lebens unter uns verbracht?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmcmli{Und du hast deine Tat, die du (damals) getan hast, verübt und bist einer der Undankbaren geworden.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmcmlii{Er sagte: «Ich habe sie da verübt, als ich (noch) einer der Irrenden war.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmcmliii{So bin ich vor euch geflohen, als ich Angst vor euch bekommen hatte. Da hat mir mein Herr Urteilskraft geschenkt und mich zu einem der Gesandten gemacht.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmcmliv{Ist es denn eine Gnade, die nun du mir erweisest, daß du die Kinder Israels zu Sklaven gemacht hast?»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmcmlv{Pharao sagte: «Was ist denn der Herr der Welten?»\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmcmlvi{Er sagte: «Der Herr der Himmel und der Erde und dessen, was dazwischen ist, so ihr Gewißheit hegt.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmcmlvii{Er sagte zu denen, die um ihn waren: «Hört ihr nicht zu?»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmcmlviii{Er sagte: «Er ist euer Herr und der Herr eurer Vorväter.»\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmcmlix{Er sagte: «Euer Gesandter, der zu euch gesandt worden ist, ist ja besessen.»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmcmlx{Er sagte: «Er ist der Herr des Ostens und des Westens und dessen, was dazwischen ist, so ihr Verstand habt.»\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmcmlxi{Er sagte: «Wenn du dir einen anderen Gott als mich nimmst, werde ich dich sicher zu einem der Gefangenen machen.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmcmlxii{Er sagte: «Was aber, wenn ich dir eine offenkundige Sache bringe?»\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmcmlxiii{Er sagte: «Dann bring sie her, so du zu denen gehörst, die die Wahrheit sagen.»\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmcmlxiv{Er warf seinen Stab, da war er eine offenkundige Schlange.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmcmlxv{Und er zog seine Hand heraus, da war sie weiß für die Zuschauer.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmcmlxvi{Er sagte zu den Vornehmen um ihn: «Dieser ist ja ein erfahrener Zauberer,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmcmlxvii{Der euch mit seiner Zauberei aus eurem Land vertreiben will. Was befehlt ihr nun?»\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmcmlxviii{Sie sagten: «Stell ihn und seinen Bruder zurück und schick zu den Städten Leute, die sie versammeln,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmcmlxix{Damit sie dir jeden erfahrenen Zauberer herbringen.»\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmcmlxx{So wurden die Zauberer auf den Termin eines bestimmten Tages zusammengeführt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmcmlxxi{Und es wurde zu den Menschen gesagt: «Werdet ihr euch auch versammeln?\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmcmlxxii{So könnten wir den Zauberern folgen, wenn sie die Sieger sind.»\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmcmlxxiii{Als die Zauberer kamen, sagten sie zu Pharao: «Wir bekommen wohl eine Belohnung, wenn wir es sind, die siegen?»\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmcmlxxiv{Er sagte: «Ja. Und ihr werdet auch zu denen gehören, die in (meine) Nähe zugelassen werden.»\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmcmlxxv{Mose sagte zu ihnen: «Werft, was ihr werfen wollt.»\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmcmlxxvi{Sie warfen ihre Stricke und Stäbe und sagten: «Bei der Macht Pharaos, wir werden gewiß die Sieger sein.»\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmcmlxxvii{Mose warf seinen Stab, da fing er an zu verschlingen, was sie vorgaukelten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmcmlxxviii{So wurden die Zauberer in Anbetung zu Boden geworfen.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmcmlxxix{Sie sagten: «Wir glauben an den Herrn der Welten,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmcmlxxx{Den Herrn von Mose und Aaron.»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmcmlxxxi{Er sagte: «Ihr glaubt ihm, bevor ich es euch erlaube? Er ist bestimmt euer Ranghöchster, der euch die Zauberei gelehrt hat. Ihr werdet es zu wissen bekommen. Ich werde eure Hände und eure Füße wechselseitig abhacken, und ich werde euch allesamt kreuzigen lassen.»\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcmlxxxii{Sie sagten: «So schlimm ist es nicht, wir kehren zu unserem Herrn zurück.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcmlxxxiii{Wir erhoffen, daß unser Herr uns unsere Verfehlungen vergebe dafür, daß wir (nun) die ersten der Gläubigen sind.»\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcmlxxxiv{Und Wir haben dem Mose offenbart: «Zieh bei Nacht mit meinen Dienern fort; ihr werdet da verfolgt werden.»\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcmlxxxv{Da schickte Pharao zu den Städten Leute, die sie versammelten:\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcmlxxxvi{«Diese sind ein kleiner Rest;\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcmlxxxvii{Dennoch versetzen sie uns in Groll.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcmlxxxviii{Aber wir sind eine zusammenrückende Gruppe auf der Hut.»\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcmlxxxix{So ließen Wir sie fortziehen von Gärten und Quellen,\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcmxc{Von Schätzen und von trefflicher Stätte.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcmxci{So war es. Und Wir gaben sie den Kindern Israels zum Erbe.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcmxcii{Sie verfolgten sie also bei Sonnenaufgang.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcmxciii{Als die beiden Scharen einander sahen, sagten die Gefährten des Mose: «Wir werden eingeholt.»\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcmxciv{Er sagte: «Nein. Mit mir ist mein Herr. Er wird mich rechtleiten.»\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcmxcv{Da offenbarten Wir dem Mose: «Schlag mit deinem Stab das Meer.» So spaltete es sich, und jeder Teil war wie ein gewaltiger Berg.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcmxcvi{Und Wir ließen die anderen dort herankommen.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcmxcvii{Und Wir retteten Mose und die, die mit ihm waren, alle.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcmxcviii{Dann ließen Wir die anderen ertrinken.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcmxcix{Darin ist wahrlich ein Zeichen. Aber die meisten von ihnen sind nicht gläubig.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmm{Und dein Herr ist der Mächtige, der Barmherzige.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmi{Und verlies ihnen den Bericht über Abraham.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmii{Als er zu seinem Vater und seinem Volk sagte: «Was betet ihr denn an?»\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmiii{Sie sagten: «Wir beten Götzen an, und wir verehren sie beharrlich.»\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmiv{Er sagte: «Hören sie denn euch, wenn ihr ruft?\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmv{Oder können sie euch nützen oder schaden?»\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmvi{Sie sagten: «Aber wir fanden, daß bereits unsere Väter so handelten.»\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmvii{Er sagte: «Seht ihr wohl das, was ihr anzubeten pflegtet,\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmviii{Ihr und eure Vorväter?\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmix{Feind sind sie mir (alle), nicht so der Herr der Welten,\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmx{Der mich erschaffen hat und mich nun rechtleitet,\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmxi{Und der mir zu essen und zu trinken gibt\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmxii{Und, wenn ich krank bin, mich heilt,\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmxiii{Und der mich sterben läßt und dann wieder lebendig macht,\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmxiv{Und von dem ich erhoffe, daß Er mir am Tag des Gerichtes meine Verfehlung vergebe.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmxv{Mein Herr, schenke mir Urteilskraft, und stelle mich zu denen, die Gutes tun.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmxvi{Und gib mir einen wahrhaftigen Ruf unter den späteren (Generationen).\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmxvii{Und mach mich zu einem der Erben des Gartens der Wonne.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmxviii{Und vergib meinem Vater, er gehört ja zu den Irrenden.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmxix{Und mach mich nicht zuschanden am Tag, da sie erweckt werden,\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmxx{Am Tag, da weder Vermögen noch Söhne nützen,\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmxxi{Sondern nur, wenn einer mit einem gesunden Herzen zu Gott kommt.»\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmxxii{Und das Paradies wird an die Gottesfürchtigen herangebracht.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmxxiii{Und sichtbar gemacht wird die Hölle denen, die irregegangen sind.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmxxiv{Und es wird zu ihnen gesagt: «Wo ist denn das, was ihr anzubeten pflegtet\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmxxv{Anstelle Gottes? Können sie euch unterstützen oder sich selbst unterstützen?»\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmxxvi{Dann werden sie hineingestürzt, sie und die, die irregegangen sind,\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmmxxvii{Und die Truppen des Iblis allesamt.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmmxxviii{Sie sagen, während sie darin miteinander streiten:\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmmxxix{«Bei Gott, wir befanden uns in einem offenkundigen Irrtum,\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmmxxx{Als wir euch dem Herrn der Welten gleichsetzten.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmmxxxi{Es waren nur die Übeltäter, die uns irregeführt haben.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmmxxxii{So haben wir nun keine Fürsprecher,\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmmxxxiii{Und auch keinen warmherzigen Freund.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmmxxxiv{Hätten wir doch eine Möglichkeit zur Rückkehr, damit wir zu den Gläubigen gehörten!»\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmmxxxv{Darin ist wahrlich ein Zeichen. Aber die meisten von ihnen sind nicht gläubig.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmmxxxvi{Und dein Herr ist der Mächtige, der Barmherzige.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmmxxxvii{Das Volk Noachs zieh die Gesandten der Lüge.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmmxxxviii{Als ihr Bruder Noach zu ihnen sagte: «Wollt ihr denn nicht gottesfürchtig sein?\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmmxxxix{Ich bin für euch ein treuer Gesandter.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmmxl{So fürchtet Gott und gehorcht mir.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmmxli{Ich verlange von euch dafür keinen Lohn. Mein Lohn obliegt nur dem Herrn der Welten.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmmxlii{So fürchtet Gott und gehorcht mir.»\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmmxliii{Sie sagten: «Sollen wir dir glauben, wo dir (nur) die Niedrigsten folgen?»\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmmxliv{Er sagte: «Was bedeutet es denn, wenn ich weiß, was sie zu tun pflegten?\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmmxlv{Mit ihnen abzurechnen obliegt nur meinem Herrn, wenn ihr es doch merken würdet!\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmmxlvi{Und ich werde die Gläubigen nicht verstoßen!\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmmxlvii{Ich bin nur ein offenkundiger Warner.»\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmmxlviii{Sie sagten: «Wenn du nicht aufhörst, o Noach, wirst du bestimmt zu denen gehören, die gesteinigt werden.»\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmmxlix{Er sagte: «Mein Herr, mein Volk hat mich der Lüge geziehen.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmml{So richte zwischen mir und ihnen und errette mich und die von den Gläubigen, die mit mir sind.»\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmmli{Da retteten Wir ihn und die, die mit ihm waren, im voll beladenen Schiff.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmmlii{Dann ließen Wir hiernach die übrigen ertrinken.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmmliii{Darin ist wahrlich ein Zeichen. Aber die meisten von ihnen sind nicht gläubig.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmmliv{Und dein Herr ist der Mächtige, der Barmherzige.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmmlv{Die Aad ziehen die Gesandten der Lüge.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmmlvi{Als ihr Bruder Hud zu ihnen sagte: «Wollt ihr denn nicht gottesfürchtig sein?\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmmlvii{Ich bin für euch ein treuer Gesandter.\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmmlviii{So fürchtet Gott und gehorcht mir.\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmmlix{Ich verlange von euch dafür keinen Lohn. Mein Lohn obliegt nur dem Herrn der Welten.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmmlx{Wollt ihr denn weiter auf jeder Anhöhe ein Wahrzeichen bauen und ein sinnloses Spiel treiben\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmmlxi{Und euch Bauwerke nehmen in der Hoffnung, daß ihr ewig weilen würdet?\qt@no{129}}%
\qt@newcmd\qurantrans@de@mmmlxii{Und, wenn ihr zugreift, greift ihr gewalttätig zu.\qt@no{130}}%
\qt@newcmd\qurantrans@de@mmmlxiii{So fürchtet Gott und gehorcht mir.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mmmlxiv{Und fürchtet den, der euch beigestanden hat mit dem, was ihr wißt,\qt@no{132}}%
\qt@newcmd\qurantrans@de@mmmlxv{Der euch beigestanden hat mit Vieh und Söhnen,\qt@no{133}}%
\qt@newcmd\qurantrans@de@mmmlxvi{Und Gärten und Quellen.\qt@no{134}}%
\qt@newcmd\qurantrans@de@mmmlxvii{Ich fürchte für euch die Pein eines gewaltigen Tages.»\qt@no{135}}%
\qt@newcmd\qurantrans@de@mmmlxviii{Sie sagten: «Es ist uns gleich, ob du ermahnst oder ob du nicht zu denen gehörst, die ermahnen.\qt@no{136}}%
\qt@newcmd\qurantrans@de@mmmlxix{Das hier ist nichts als die Sitte der Früheren.\qt@no{137}}%
\qt@newcmd\qurantrans@de@mmmlxx{Und wir werden bestimmt nicht gepeinigt werden.»\qt@no{138}}%
\qt@newcmd\qurantrans@de@mmmlxxi{So ziehen sie ihn der Lüge. Da ließen Wir sie verderben. Darin ist wahrlich ein Zeichen. Aber die meisten von ihnen sind nicht gläubig.\qt@no{139}}%
\qt@newcmd\qurantrans@de@mmmlxxii{Und dein Herr ist der Mächtige, der Barmherzige.\qt@no{140}}%
\qt@newcmd\qurantrans@de@mmmlxxiii{Die Thamud ziehen die Gesandten der Lüge.\qt@no{141}}%
\qt@newcmd\qurantrans@de@mmmlxxiv{Als ihr Bruder Salih zu ihnen sagte: «Wollt ihr denn nicht gottesfürchtig sein?\qt@no{142}}%
\qt@newcmd\qurantrans@de@mmmlxxv{Ich bin für euch ein treuer Gesandter.\qt@no{143}}%
\qt@newcmd\qurantrans@de@mmmlxxvi{So fürchtet Gott und gehorcht mir.\qt@no{144}}%
\qt@newcmd\qurantrans@de@mmmlxxvii{Ich verlange von euch keinen Lohn. Mein Lohn obliegt nur dem Herrn der Welten.\qt@no{145}}%
\qt@newcmd\qurantrans@de@mmmlxxviii{Werdet ihr etwa in dem, was hier ist, in Sicherheit gelassen,\qt@no{146}}%
\qt@newcmd\qurantrans@de@mmmlxxix{In Gärten und an Quellen,\qt@no{147}}%
\qt@newcmd\qurantrans@de@mmmlxxx{In Getreidefeldern und Palmen, deren Blütenscheiden zart sind?\qt@no{148}}%
\qt@newcmd\qurantrans@de@mmmlxxxi{Und werdet ihr weiter geschickt aus den Bergen Häuser meißeln?\qt@no{149}}%
\qt@newcmd\qurantrans@de@mmmlxxxii{So fürchtet Gott und gehorcht mir.\qt@no{150}}%
\qt@newcmd\qurantrans@de@mmmlxxxiii{Und gehorcht nicht dem Befehl der Maßlosen,\qt@no{151}}%
\qt@newcmd\qurantrans@de@mmmlxxxiv{Die Unheil auf der Erde stiften und keine Besserung bringen.»\qt@no{152}}%
\qt@newcmd\qurantrans@de@mmmlxxxv{Sie sagten: «Du bist ja einer von denen, die einem Zauber verfallen sind.\qt@no{153}}%
\qt@newcmd\qurantrans@de@mmmlxxxvi{Du bist nur ein Mensch wie wir. So bring ein Zeichen her, so du zu denen gehörst, die die Wahrheit sagen.»\qt@no{154}}%
\qt@newcmd\qurantrans@de@mmmlxxxvii{Er sagte: «Dies ist eine Kamelstute; sie hat eine Trinkzeit, und ihr habt eine Trinkzeit an einem bestimmten Tag.\qt@no{155}}%
\qt@newcmd\qurantrans@de@mmmlxxxviii{Und rührt sie nicht mit etwas Bösem an, sonst ergreift euch die Pein eines gewaltigen Tages.»\qt@no{156}}%
\qt@newcmd\qurantrans@de@mmmlxxxix{Und sie schnitten ihr die Flechsen durch und stachen sie. So wurden sie zu Leuten, die (ihre Tat) bereuen.\qt@no{157}}%
\qt@newcmd\qurantrans@de@mmmxc{Da ergriff sie die Pein. Darin ist wahrlich ein Zeichen. Aber die meisten von ihnen sind nicht gläubig.\qt@no{158}}%
\qt@newcmd\qurantrans@de@mmmxci{Und dein Herr ist der Mächtige, der Barmherzige.\qt@no{159}}%
\qt@newcmd\qurantrans@de@mmmxcii{Das Volk Lots zieh die Gesandten der Lüge.\qt@no{160}}%
\qt@newcmd\qurantrans@de@mmmxciii{Als ihr Bruder Lot zu ihnen sagte: «Wollt ihr denn nicht gottesfürchtig sein?\qt@no{161}}%
\qt@newcmd\qurantrans@de@mmmxciv{Ich bin für euch ein treuer Gesandter.\qt@no{162}}%
\qt@newcmd\qurantrans@de@mmmxcv{So fürchtet Gott und gehorcht mir.\qt@no{163}}%
\qt@newcmd\qurantrans@de@mmmxcvi{Ich verlange von euch dafür keinen Lohn. Mein Lohn obliegt nur dem Herrn der Welten.\qt@no{164}}%
\qt@newcmd\qurantrans@de@mmmxcvii{Wie könnt ihr denn zu den Männern unter den Weltenbewohnern gehen\qt@no{165}}%
\qt@newcmd\qurantrans@de@mmmxcviii{Und, was euch euer Herr an Gattinnen erschaffen hat, liegen lassen? Nein, ihr seid Leute, die Übertretungen begehen.»\qt@no{166}}%
\qt@newcmd\qurantrans@de@mmmxcix{Sie sagten: «Wenn du nicht aufhörst, o Lot, wirst du zu denen gehören, die vertrieben werden.»\qt@no{167}}%
\qt@newcmd\qurantrans@de@mmmc{Er sagte: «Ich verabscheue eure Tat.\qt@no{168}}%
\qt@newcmd\qurantrans@de@mmmci{Mein Herr, errette mich und meine Angehörigen von dem, was sie tun.»\qt@no{169}}%
\qt@newcmd\qurantrans@de@mmmcii{Da erretteten Wir ihn und seine Angehörigen alle,\qt@no{170}}%
\qt@newcmd\qurantrans@de@mmmciii{Bis auf eine alte Frau unter denen, die zurückblieben und dem Verderben anheimfielen.\qt@no{171}}%
\qt@newcmd\qurantrans@de@mmmciv{Dann zerstörten Wir die anderen.\qt@no{172}}%
\qt@newcmd\qurantrans@de@mmmcv{Und Wir ließen einen Regen auf sie niedergehen. Schlimm war der Regen, der die Gewarnten traf.\qt@no{173}}%
\qt@newcmd\qurantrans@de@mmmcvi{Darin ist wahrlich ein Zeichen. Aber die meisten von ihnen sind nicht gläubig.\qt@no{174}}%
\qt@newcmd\qurantrans@de@mmmcvii{Und dein Herr ist der Mächtige, der Barmherzige.\qt@no{175}}%
\qt@newcmd\qurantrans@de@mmmcviii{Die Gefährten des Waldes ziehen die Gesandten der Lüge.\qt@no{176}}%
\qt@newcmd\qurantrans@de@mmmcix{Als Shuayb zu ihnen sagte: «Wollt ihr denn nicht gottesfürchtig sein?\qt@no{177}}%
\qt@newcmd\qurantrans@de@mmmcx{Ich bin für euch ein treuer Gesandter.\qt@no{178}}%
\qt@newcmd\qurantrans@de@mmmcxi{So fürchtet Gott und gehorcht mir.\qt@no{179}}%
\qt@newcmd\qurantrans@de@mmmcxii{Ich verlange von euch dafür keinen Lohn. Mein Lohn obliegt nur dem Herrn der Welten.\qt@no{180}}%
\qt@newcmd\qurantrans@de@mmmcxiii{Gebt volles Maß und seid nicht solche, die Verlust verursachen.\qt@no{181}}%
\qt@newcmd\qurantrans@de@mmmcxiv{Und wägt mit der richtigen Waage.\qt@no{182}}%
\qt@newcmd\qurantrans@de@mmmcxv{Und zieht den Menschen nicht ab, was ihnen gehört, und verbreitet nicht Unheil auf der Erde.\qt@no{183}}%
\qt@newcmd\qurantrans@de@mmmcxvi{Und fürchtet den, der euch und die früheren Geschöpfe erschaffen hat.»\qt@no{184}}%
\qt@newcmd\qurantrans@de@mmmcxvii{Sie sagten: «Du bist ja einer von denen, die einem Zauber verfallen sind.\qt@no{185}}%
\qt@newcmd\qurantrans@de@mmmcxviii{Du bist nur ein Mensch wie wir. Wir meinen, daß du zu den Lügnern gehörst.\qt@no{186}}%
\qt@newcmd\qurantrans@de@mmmcxix{Laß doch Stücke vom Himmel auf uns herabfallen, so du zu denen gehörst, die die Wahrheit sagen.»\qt@no{187}}%
\qt@newcmd\qurantrans@de@mmmcxx{Er sagte: «Mein Herr weiß besser, was ihr tut.»\qt@no{188}}%
\qt@newcmd\qurantrans@de@mmmcxxi{Sie ziehen ihn der Lüge. Da ergriff sie die Pein des Tages der Überschattung. Es war die Pein eines gewaltigen Tages.\qt@no{189}}%
\qt@newcmd\qurantrans@de@mmmcxxii{Darin ist wahrlich ein Zeichen. Aber die meisten von ihnen sind nicht gläubig.\qt@no{190}}%
\qt@newcmd\qurantrans@de@mmmcxxiii{Und dein Herr ist der Mächtige, der Barmherzige.\qt@no{191}}%
\qt@newcmd\qurantrans@de@mmmcxxiv{Und er ist eine Herabsendung des Herrn der Welten;\qt@no{192}}%
\qt@newcmd\qurantrans@de@mmmcxxv{Mit ihm ist der treue Geist herabgestiegen\qt@no{193}}%
\qt@newcmd\qurantrans@de@mmmcxxvi{Auf dein Herz, damit du einer der Warner seist,\qt@no{194}}%
\qt@newcmd\qurantrans@de@mmmcxxvii{In deutlicher arabischer Sprache.\qt@no{195}}%
\qt@newcmd\qurantrans@de@mmmcxxviii{Und er ist in den Schriften der Früheren (erwähnt).\qt@no{196}}%
\qt@newcmd\qurantrans@de@mmmcxxix{War es ihnen denn nicht ein Zeichen, daß die Gelehrten der Kinder Israels über ihn Bescheid wissen?\qt@no{197}}%
\qt@newcmd\qurantrans@de@mmmcxxx{Wenn Wir ihn auf einen der Nichtaraber hinabgesandt hätten\qt@no{198}}%
\qt@newcmd\qurantrans@de@mmmcxxxi{Und er ihn ihnen vorgelesen hätte, hätten sie nicht an ihn geglaubt.\qt@no{199}}%
\qt@newcmd\qurantrans@de@mmmcxxxii{So haben Wir ihn in die Herzen der Übeltäter eingehen lassen.\qt@no{200}}%
\qt@newcmd\qurantrans@de@mmmcxxxiii{Sie glauben nicht daran, bis sie die schmerzhafte Pein sehen,\qt@no{201}}%
\qt@newcmd\qurantrans@de@mmmcxxxiv{Bis diese über sie plötzlich kommt, ohne daß sie es merken,\qt@no{202}}%
\qt@newcmd\qurantrans@de@mmmcxxxv{Und bis sie sagen: «Wird uns Aufschub gewährt?»\qt@no{203}}%
\qt@newcmd\qurantrans@de@mmmcxxxvi{Wünschen sie denn unsere Pein zu beschleunigen?\qt@no{204}}%
\qt@newcmd\qurantrans@de@mmmcxxxvii{Was meinst du? Wenn Wir sie auf Jahre genießen lassen,\qt@no{205}}%
\qt@newcmd\qurantrans@de@mmmcxxxviii{Wenn dann zu ihnen kommt, was ihnen angedroht wurde,\qt@no{206}}%
\qt@newcmd\qurantrans@de@mmmcxxxix{Dann wird ihnen nicht nützen, was ihnen an Nutznießung gewährt wurde.\qt@no{207}}%
\qt@newcmd\qurantrans@de@mmmcxl{Und Wir haben keine Stadt verderben lassen, ohne daß sie Warner gehabt hätte,\qt@no{208}}%
\qt@newcmd\qurantrans@de@mmmcxli{Dies als Ermahnung. Und Wir tun kein Unrecht.\qt@no{209}}%
\qt@newcmd\qurantrans@de@mmmcxlii{Nicht die Satane sind mit ihm herabgestiegen;\qt@no{210}}%
\qt@newcmd\qurantrans@de@mmmcxliii{Es ziemt ihnen nicht, und sie vermögen es nicht.\qt@no{211}}%
\qt@newcmd\qurantrans@de@mmmcxliv{Sie sind vom Hören ausgeschlossen.\qt@no{212}}%
\qt@newcmd\qurantrans@de@mmmcxlv{So rufe neben Gott keinen anderen Gott an, sonst wirst du zu den Gepeinigten gehören.\qt@no{213}}%
\qt@newcmd\qurantrans@de@mmmcxlvi{Und warne deine nächsten Sippenmitglieder.\qt@no{214}}%
\qt@newcmd\qurantrans@de@mmmcxlvii{Und senke deinen Flügel für die unter den Gläubigen, die dir folgen.\qt@no{215}}%
\qt@newcmd\qurantrans@de@mmmcxlviii{Wenn sie gegen dich ungehorsam sind, dann sprich: «Ich bin unschuldig an dem, was ihr tut.»\qt@no{216}}%
\qt@newcmd\qurantrans@de@mmmcxlix{Und vertrau auf den Mächtigen, den Barmherzigen,\qt@no{217}}%
\qt@newcmd\qurantrans@de@mmmcl{Der dich sieht, wenn du aufrecht stehst,\qt@no{218}}%
\qt@newcmd\qurantrans@de@mmmcli{Und auch wie du dich hin und her wendest unter denen, die sich niederwerfen.\qt@no{219}}%
\qt@newcmd\qurantrans@de@mmmclii{Er ist der, der alles hört und weiß.\qt@no{220}}%
\qt@newcmd\qurantrans@de@mmmcliii{Soll ich euch kundtun, auf wen die Satane herabsteigen?\qt@no{221}}%
\qt@newcmd\qurantrans@de@mmmcliv{Sie steigen auf jeden Lügner und Sünder herab.\qt@no{222}}%
\qt@newcmd\qurantrans@de@mmmclv{Sie hören hin, aber die meisten von ihnen sind Lügner.\qt@no{223}}%
\qt@newcmd\qurantrans@de@mmmclvi{Und den Dichtern folgen die Abgeirrten.\qt@no{224}}%
\qt@newcmd\qurantrans@de@mmmclvii{Hast du nicht gesehen, daß sie in jedem Tal richtungslos wandern,\qt@no{225}}%
\qt@newcmd\qurantrans@de@mmmclviii{Und daß sie sagen, was sie nicht tun?\qt@no{226}}%
\qt@newcmd\qurantrans@de@mmmclix{Nicht so diejenigen, die glauben, die guten Werke tun und Gottes viel gedenken, und (erst) sich selbst helfen, nachdem ihnen Unrecht getan wurde. Und diejenigen, die Unrecht tun, werden erfahren, was für eine Rückkehr sie haben werden.\qt@no{227}}%
\qt@newcmd\qurantrans@de@mmmclx{\basmalahde Taa Sien. Dies sind die Zeichen des Korans und eines deutlichen Buches,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmclxi{Rechtleitung und Frohbotschaft für die Gläubigen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmclxii{Die das Gebet verrichten und die Abgabe entrichten, und die über das Jenseits Gewißheit hegen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmclxiii{Denjenigen, die nicht an das Jenseits glauben, haben Wir ihre Werke verlockend gemacht, so daß sie blind umherirren.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmclxiv{Das sind die, für die eine schlimme Pein bestimmt ist, und im Jenseits sind sie die größten Verlierer.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmclxv{Und wahrlich, dir wird der Koran zum Empfang überbracht von Seiten eines Weisen und Allwissenden.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmclxvi{Als Mose zu seinen Angehörigen sagte: «Ich habe ein Feuer wahrgenommen. Ich werde euch davon eine Nachricht bringen, oder ich bringe euch eine brennende Fackel, auf daß ihr euch (daran) wärmen könnt.»\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmclxvii{Als er dort ankam, wurde ihm zugerufen: «Gesegnet sei der, der im Feuer und der in seiner Umgebung ist, und Preis sei Gott, dem Herrn der Welten!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmclxviii{O Mose, Ich bin, ja Ich bin Gott, der Mächtige, der Weise.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmclxix{Und wirf deinen Stab.» Als er nun sah, daß er sich schüttelte, als wäre er eine flinke Schlange, kehrte er den Rücken und wandte sich nicht mehr um. «O Mose, hab keine Angst. Ja, bei Mir brauchen die Gesandten keine Angst zu haben,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmclxx{Außer wenn einer Unrecht tut. Tauscht er dann aber nach der bösen Tat etwas Gutes ein, dann bin Ich voller Vergebung und barmherzig.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmclxxi{Und stecke deine Hand in deinen Hemdausschnitt, so kommt sie weiß, jedoch nicht von Übel befallen, heraus. Dies zählt zu neun Zeichen, gerichtet an Pharao und sein Volk. Sie sind ja ein frevlerisches Volk.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmclxxii{Als unsere Zeichen deutlich sichtbar zu ihnen kamen, sagten sie: «Das ist eine offenkundige Zauberei.»\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmclxxiii{Und sie verleugneten sie, obwohl ihr Inneres darüber Gewißheit hegte, aus Ungerechtigkeit und Überheblichkeit. So schau, wie das Ende der Unheilstifter war.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmclxxiv{Und Wir ließen David und Salomo Wissen zukommen. Und sie sagten: «Lob sei Gott, der uns vor vielen seiner gläubigen Diener bevorzugt hat!»\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmclxxv{Und Salomo beerbte David. Und er sagte: «O ihr Menschen, uns ist die Sprache der Vögel gelehrt worden, und uns wurde von allen Dingen zugeteilt. Das ist wahrlich die offenkundige Huld (Gottes).»\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmclxxvi{Und zusammengeführt wurden zu Salomo seine Truppen - Djinn, Menschen und Vögel -, und sie wurden in Abteilungen aufgestellt.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmclxxvii{Als sie dann zum Ameisental kamen, sagte eine Ameise: «O ihr Ameisen, geht in eure Wohnungen hinein, daß euch nicht Salomo und seine Truppen zermalmen, ohne es zu merken.»\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmclxxviii{Da lächelte er erheitert über ihre Rede und sagte: «Mein Herr, gib mir ein, für deine Gnade zu danken, mit der Du mich und meine Eltern begnadet hast, und etwas Gutes zu tun, das Dir wohlgefällt. Und laß mich durch deine Barmherzigkeit eingehen in die Reihen deiner rechtschaffenen Diener.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmclxxix{Und er schaute bei den Vögeln nach Vermißten. Da sagte er: «Wie kommt es, daß ich den Wiedehopf nicht sehe? Befindet er sich etwa unter den Abwesenden?\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmclxxx{Wahrlich, ich werde ihn mit einer harten Pein peinigen. Oder ich werde ihn wahrlich schlachten, es sei denn, er bringt mir eine offenkundige Ermächtigung.»\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmclxxxi{Er blieb nicht lange. Da sagte er: «Ich habe erfaßt, was du nicht erfaßt hast. Und ich bringe dir aus Saba eine sichere Nachricht.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmclxxxii{Ich habe herausgefunden, daß eine Frau über sie herrscht, daß ihr von allen Dingen zugeteilt worden ist, und daß sie einen gewaltigen Thron hat.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmclxxxiii{Und ich habe herausgefunden, daß sie und ihr Volk vor der Sonne niederfallen, statt vor Gott. Und der Satan hat ihnen ihre Werke verlockend gemacht und sie vom Weg abgewiesen, so daß sie der Rechtleitung nicht folgen,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmclxxxiv{dies), damit sie nicht vor Gott niederfallen, der das Verborgene in den Himmeln und auf der Erde herausbringt und weiß, was ihr verbergt, und was ihr offenlegt.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmclxxxv{Gott, es gibt keinen Gott außer Ihm, dem Herrn des majestätischen Thrones.»\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmclxxxvi{Er sagte: «Wir werden schauen, ob du die Wahrheit sagst oder ob du zu den Lügnern gehörst.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmclxxxvii{Geh mit diesem meinem Schreiben und überbring es ihnen. Dann kehr dich von ihnen ab und schau, was sie erwidern.»\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmclxxxviii{Sie sagte: «O ihr Vornehmen, mir ist ein edles Schreiben überbracht worden.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmclxxxix{Es ist von Salomo. Und es lautet: >Im Namen Gottes, des Erbarmers, des Barmherzigen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmcxc{Seid mir gegenüber nicht überheblich und kommt ergeben zu mir.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmcxci{Sie sagte: «O ihr Vornehmen, gebt mir Auskunft über meine Angelegenheit. Ich entscheide ja keine Angelegenheit, es sei denn in eurer Gegenwart.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmcxcii{Sie sagten: «Wir besitzen eine Streitmacht und besitzen eine starke Schlagkraft. Aber dir gehört es, zu befehlen. So sieh zu, was du befehlen willst.»\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmcxciii{Sie sagte: «Wenn Könige eine Stadt betreten, stürzen sie sie ins Unheil und machen die Mächtigen unter ihren Bewohnern zu gedemütigten (Menschen). Ja, so handeln sie.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmcxciv{Ich werde (Leute) zu ihnen schicken mit einem Geschenk, und dann werde ich abwarten, was die Boten (für eine Antwort) zurückbringen.»\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmcxcv{Als er zu Salomo kam, sagte dieser: «Ihr wollt mir mit Geld beistehen? Aber das, was mir Gott zukommen ließ, ist besser als das, was Er euch zukommen ließ. Nein, ihr seid es, die ihr euch über euer Geschenk freut.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmcxcvi{Kehr zu ihnen zurück. Wahrlich, wir werden zu ihnen mit Truppen kommen, denen sie nichts entgegenzusetzen haben. Und wahrlich, wir werden sie aus ihrer Stadt vertreiben als Gedemütigte und Erniedrigte.»\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmcxcvii{Er sagte: «O ihr Vornehmen, wer von euch bringt mir ihren Thron, bevor sie ergeben zu mir kommen?»\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmcxcviii{Ein listiger Kraftprotz unter den Djinn sagte: «Ich bringe ihn dir, bevor du dich von deiner Stelle erhebst. Und ich habe gewiß die Stärke dazu, und ich bin vertrauenswürdig.»\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmcxcix{Der, der Wissen aus dem Buch besaß, sagte: «Ich bringe ihn dir, bevor dein Blick zu dir zurückkehrt.» Als er ihn da bei sich stehen sah, sagte er: «Dies ist von der Huld meines Herrn, damit Er mich prüft, ob ich dankbar oder undankbar bin. Und wer dankbar ist, ist dankbar zu seinem eigenen Vorteil. Und wenn einer undankbar ist, so ist mein Herr auf niemanden angewiesen und freigebig.»\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmcc{Er sagte: «Macht ihr ihren Thron unkenntlich, wir wollen schauen, ob sie der Rechtleitung folgt oder ob sie zu denen gehört, die der Rechtleitung nicht folgen.»\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmcci{Als sie kam, wurde (zu ihr) gesagt: «War dein Thron so?» Sie sagte: «Es ist so, als wäre er es. Und uns wurde schon vor dem das Wissen zuteil, und wir waren gottergeben.»\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmccii{Und abgewiesen hat sie, was sie anstelle Gottes verehrte. Sie gehörte ja zu ungläubigen Leuten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmcciii{Es wurde zu ihr gesagt: «Tritt in den offenen Hochbau ein.» Als sie ihn sah, hielt sie ihn für ein tiefes Wasser und entblößte ihre Beine. Er sagte: «Es ist ein mit Glas ausgelegter Hochbau.» Sie sagte: «Mein Herr, ich habe mir selbst Unrecht getan. Und ich ergebe mich, zusammen mit Salomo, Gott, dem Herrn der Welten.»\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmcciv{Und Wir haben zu Thamud ihren Bruder Salih gesandt: «Dienet Gott.» Da waren es plötzlich zwei Gruppen, die miteinander stritten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmccv{Er sagte: «O mein Volk, warum wünscht ihr, das Schlechte vor dem Guten zu beschleunigen? Könntet ihr doch Gott um Vergebung bitten, auf daß ihr Erbarmen findet!»\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmccvi{Sie sagten: «Wir sehen ein böses Omen in dir und in denen, die mit dir sind.» Er sagte: «Euer Omen ist bei Gott. Aber ihr seid ja Leute, die der Versuchung ausgesetzt sind.»\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmccvii{Nun befanden sich in der Stadt neun Mitglieder einer Bande, die auf der Erde Unheil stifteten und keine Besserung brachten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmccviii{Sie sagten: «Schwört voreinander bei Gott: Wir werden ihn und seine Angehörigen nachts überfallen, dann werden wir zu seinem nächsten Verwandten sagen: >Wir waren bei der Vernichtung seiner Angehörigen nicht zugegen, und wir sagen (hier) die Wahrheit.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmccix{Sie schmiedeten Ränke, und Wir schmiedeten Ränke, ohne daß sie es merkten.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmccx{Schau, wie die Folge ihrer Ränke war: Wir zerstörten sie und ihr Volk allesamt.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmccxi{Da sind ihre Häuser verödet dafür, daß sie Unrecht taten. Darin ist ein Zeichen für Leute, die Bescheid wissen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmccxii{Und Wir retteten diejenigen, die glaubten und gottesfürchtig waren.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmccxiii{Und (gesandt haben Wir) Lot. Als er zu seinem Volk sagte: «Wollt ihr denn das Schändliche begehen, wo ihr es doch seht?\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmccxiv{Wollt ihr denn in Begierde zu den Männern gehen statt zu den Frauen? Nein, ihr seid Leute, die töricht sind.»\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmccxv{Die Antwort seines Volkes war nur, daß sie sagten: «Vertreibt die Sippe Lots aus eurer Stadt. Das sind Menschen, die sich rein stellen.»\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmccxvi{Da retteten Wir ihn und seine Angehörigen, außer seiner Frau. Wir bestimmten, daß sie zu denen gehörte, die zurückblieben und dem Verderben anheimfielen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmccxvii{Und Wir ließen einen Regen auf sie niedergehen. Schlimm ist der Regen, der die Gewarnten traf.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmccxviii{Sprich: Lob sei Gott! Und Friede sei über seinen Dienern, die Er sich erwählt hat! Ist Gott besser oder das, was sie (Ihm) beigesellen?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmccxix{Oder wer hat die Himmel und die Erde erschaffen und euch vom Himmel Wasser herabkommen lassen? Dadurch haben Wir Gärten wachsen lassen, die Freude bereiten. Ihr hättet unmöglich deren Bäume wachsen lassen können. Gibt es denn einen (anderen) Gott neben Gott? Nein, ihr seid ja Leute, die (Gott andere) gleichsetzen.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmccxx{Oder wer hat die Erde zu einem festen Grund gemacht und Flüsse durch sie gemacht und auf ihr festgegründete Berge gemacht und zwischen den beiden Meeren eine Schranke gemacht? Gibt es denn einen (anderen) Gott neben Gott? Nein, die meisten von ihnen wissen nicht Bescheid.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmccxxi{Oder wer erhört den Bedrängten, wenn er zu Ihm ruft, und behebt das Böse und macht euch zu Nachfolgern auf der Erde? Gibt es denn einen (anderen) Gott neben Gott? Aber ihr bedenkt es wenig.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmccxxii{Oder wer führt euch in den Finsternissen des Festlandes und des Meeres (den rechten Weg)? Und wer schickt seiner Barmherzigkeit die Winde als frohe Botschaft voraus? Gibt es denn einen (anderen) Gott neben Gott? Erhaben ist Gott über das, was sie (Ihm) beigesellen.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmccxxiii{Und wer macht die Schöpfung am Anfang und wiederholt sie? Und wer versorgt euch vom Himmel und von der Erde? Gibt es denn einen (anderen) Gott neben Gott? Sprich: Bringt her euren Beweis, so ihr die Wahrheit sagt.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmccxxiv{Sprich: Diejenigen, die in den Himmeln und auf der Erde sind, wissen über das Unsichtbare nicht Bescheid, außer Gott. Und sie merken nicht, wann sie auferweckt werden.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmccxxv{Nein, ihr Wissen über das Jenseits ist da erschöpft. Mehr noch, sie hegen darüber Zweifel. Mehr noch, sie sind ihm gegenüber blind.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmccxxvi{Und diejenigen, die ungläubig sind, sagen: «Wenn wir und unsere Väter zu Staub geworden sind, sollen wir dann wirklich wieder hervorgebracht werden?\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmccxxvii{Dies ist uns und zuvor unseren Vätern versprochen worden. Das sind (doch) nichts als die Fabeln der Früheren.»\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmccxxviii{Sprich: Geht auf der Erde umher und schaut, wie das Ende der Übeltäter war.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmccxxix{Und sei nicht betrübt über sie, und sei nicht in Bedrängnis wegen der Ränke, die sie schmieden.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmccxxx{Und sie sagen: «Wann wird diese Androhung eintreten, so ihr die Wahrheit sagt?»\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmccxxxi{Sprich: Vielleicht ist einiges von dem, was ihr zu beschleunigen wünscht, dicht hinter euch.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmccxxxii{Und dein Herr ist voller Huld gegen die Menschen. Aber die meisten von ihnen sind nicht dankbar.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmccxxxiii{Und dein Herr weiß gewiß, was ihre Brust verhüllt und was sie offenlegen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmccxxxiv{Und es gibt nichts Unsichtbares im Himmel und auf der Erde, was nicht in einem deutlichen Buch stünde.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmccxxxv{Dieser Koran erzählt den Kindern Israels das meiste von dem, worüber sie uneins sind.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmccxxxvi{Und es ist eine Rechtleitung und Barmherzigkeit für die Gläubigen.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmccxxxvii{Dein Herr richtet zwischen ihnen durch sein Urteil. Und Er ist der Mächtige, der alles weiß.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmccxxxviii{So vertrau auf Gott. Du folgst der offenkundigen Wahrheit.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmccxxxix{Du kannst nicht die Toten hören lassen und auch nicht die Tauben den Zuruf hören lassen, wenn sie den Rücken kehren.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmccxl{Und du kannst nicht die Blinden aus ihrem Irrtum herausführen. Hören lassen kannst du nur die, die an unsere Zeichen Klauben und somit gottergeben sind.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmccxli{Und wenn der Spruch über sie fällig wird, bringen Wir ihnen ein Tier aus der Erde hervor, das zu ihnen spricht; (dies), weil die Menschen über unsere Zeichen keine Gewißheit hegen.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmccxlii{Und am Tag, da Wir aus jeder Gemeinschaft eine Schar von denen versammeln, die unsere Zeichen für Lüge erklären, und sie dann in Abteilungen aufgestellt werden.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmccxliii{Wenn sie dann kommen, spricht Er: Habt ihr denn meine Zeichen für Lüge erklärt, obwohl ihr davon kein umfassendes Wissen hattet, oder was habt ihr denn getan?»\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmccxliv{Und der Spruch wird über sie fällig dafür, daß sie Unrecht taten, so daß sie nicht reden werden.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmccxlv{Haben sie nicht gesehen, daß Wir die Nacht gemacht haben, damit sie in ihr ruhen, und den Tag, an dem man sehen kann? Darin sind Zeichen für Leute, die glauben.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmccxlvi{Und am Tag, da in die Trompete geblasen wird und die erschrecken, die in den Himmeln und die auf der Erde sind, außer denen, die Gott will. Und alle kommen demütig zu Ihm.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmccxlvii{Und du siehst die Berge: Du meinst, sie stünden fest, während sie wie Wolken vorbeiziehen. (Es ist) das Werk Gottes, der alles sorgfältig gemacht hat. Er hat Kenntnis von dem, was ihr tut.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmccxlviii{Diejenigen, die mit einer guten Tat kommen, erhalten etwas Besseres als siel Und sie sind an jenem Tag vor dem Schrecken in Sicherheit.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmccxlix{Und diejenigen, die mit einer schlechten Tat kommen, werden auf ihren Gesichtern ins Feuer gestürzt: «Wird euch denn für etwas anderes vergolten als das, was ihr zu tun pflegtet?»\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmccl{Wahrlich, mir ist befohlen worden, dem Herrn dieser Ortschaft zu dienen, der sie für heilig erklärt hat und dem alles gehört. Und mir ist befohlen worden, einer der Gottergebenen zu sein,\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmccli{Und den Koran zu verlesen. Wer der Rechtleitung folgt, folgt ihr zu seinem eigenen Vorteil. Und wenn einer irregeht, dann sprich: Ich bin ja nur einer der Warner.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmcclii{Und sprich: Lob sei Gott! Er wird euch seine Zeichen sehen lassen, so werdet ihr sie erkennen. Und Gott läßt nicht unbeachtet, was ihr tut.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmccliii{\basmalahde Taa Sien Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmccliv{Dies sind die Zeichen des deutlichen Buches.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcclv{Wir verlesen dir aus dem Bericht über Mose und Pharao der Wahrheit entsprechend, für Leute, die glauben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcclvi{Pharao hatte hohe Gewalt im Land und machte seine Bewohner zu Parteien, von denen er eine Gruppe wie Schwache behandelte, indem er ihre Söhne abschlachtete und nur ihre Frauen am Leben ließ. Er gehörte zu den Unheilstiftern.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcclvii{Wir aber wollten denen, die im Land wie Schwache behandelt wurden, eine Wohltat erweisen und sie zu Vorbildern machen und zu Erben machen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcclviii{Ihnen eine angesehene Stellung im Land geben und durch sie Pharao, Haamaan und deren Truppen das erfahren lassen, was sie befürchteten.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcclix{Und Wir gaben der Mutter des Mose ein: «Stille ihn. Und falls du Angst um ihn hast, so leg ihn ins Meer. Hab keine Angst und sei nicht betrübt. Wir werden ihn dir zurückbringen und ihn zu einem der Gesandten machen.»\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcclx{Da lasen ihn die Angehörigen Pharaos auf, damit er ihnen ein Feind und Grund zum Kummer sei. Pharao, Haamaan und deren Truppen handelten verfehlt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcclxi{Die Frau Pharaos sagte: «Er wird für mich und dich ein Grund zur Freude sein. Tötet ihn nicht. Möge er Nutzen bringen, oder vielleicht nehmen wir ihn als Kind an.» Dabei merkten sie nicht (was das ihnen bringen würde).\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcclxii{Und das Herz der Mutter des Mose war leer. Fast hätte sie ihn offen bekanntgegeben, wenn Wir nicht ihr Herz gefestigt hätten, damit sie zu den Gläubigen gehöre.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcclxiii{Und sie sagte zu seiner Schwester: «Folge seiner Spur.» Sie beobachtete ihn beiläufig, ohne daß sie es merkten.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcclxiv{Nun hatten Wir ihm zuvor Ammenbrüste verwehrt. Da sagte sie: «Soll ich euch auf Hausleute hinweisen, die ihn für euch betreuen und ihm wohlgesinnt sein würden?»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcclxv{So brachten Wir ihn zu seiner Mutter zurück, damit sie durch ihn frohen Mutes und nicht betrübt sei, und sie sollte wissen, daß das Versprechen Gottes wahr ist. Aber die meisten von ihnen wissen nicht Bescheid.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcclxvi{Und als er seine Vollkraft und seine volle Gestalt erreicht hatte, ließen Wir ihm Urteilskraft und Wissen zukommen. So entlohnen Wir die Rechtschaffenen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcclxvii{Und er trat in die Stadt ein zu einer Zeit, da ihre Bewohner darauf nicht achteten. Da fand er darin zwei Männer, die miteinander kämpften, der eine war von seiner eigenen Partei, der andere von seinen Feinden. Der, der von seiner Partei war, rief ihn zu Hilfe gegen den, der von den Feinden war. Mose schlug ihn und brachte ihn so um. Er sagte: «Das gehört zum Werk Satans. Er ist ja ein offenkundiger Feind, der irreführt.»\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcclxviii{Er sagte: «Mein Herr, ich habe mir selbst Unrecht getan. So vergib mir.» Da vergab Er ihm. Er ist ja der, der voller Vergebung und barmherzig ist.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcclxix{Er sagte: «Mein Herr, weil Du mich begnadet hast, werde ich nie den Übeltätern Beistand leisten.»\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcclxx{Am Morgen war er in der Stadt voller Angst und sah sich immer wieder um. Und siehe, der ihn am Tag zuvor um Unterstützung gebeten hatte, schrie zu ihm um Hilfe. Mose sagte zu ihm: «Du bist offenkundig stark abgeirrt.»\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcclxxi{Als er nun mit Gewalt den greifen wollte, der ihrer beider Feind war, sagte dieser: «O Mose, willst du denn mich töten, wie du gestern einen Menschen getötet hast? Du willst nichts anderes, als ein Gewalttäter im Land zu sein, und du willst nicht zu denen gehören, die Besserung bringen.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcclxxii{Ein Mann kam vom äußersten Ende der Stadt gelaufen. Er sagte: «O Mose, die Vornehmen beraten über dich, um dich zu töten. So geh fort, ich gehöre zu denen, die dir gut raten.»\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcclxxiii{Er ging aus ihr fort voller Angst und sah sich immer wieder um. Er sagte: «Mein Herr, errette mich von den Leuten, die Unrecht tun.»\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcclxxiv{Und als er sich in Richtung Madyan begab, sagte er: «Möge mein Herr mich den rechten Weg führen!»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcclxxv{Als er nun zum Wasser von Madyan kam, fand er dort eine (ganze) Gemeinschaft von Menschen, die (ihr Vieh) tränkten. Und er fand außer ihnen zwei Frauen, die (ihre Tiere) zurückhielten. Er sagte: «Was ist mit euch?» Sie sagten: «Wir tränken (unsere Tiere) nicht, bis die Hirten (ihr Vieh) zurückgetrieben haben. Und unser Vater ist (auch) ein hochbetagter Greis.»\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcclxxvi{Da tränkte er ihnen (ihre Tiere). Dann zog er sich zurück in den Schatten und sagte: «Mein Herr, ich bin dessen bedürftig, was Du auch immer an Gutem zu mir herabsendest.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcclxxvii{Die eine von den beiden kam zu ihm, sie ging verschämt. Sie sagte: «Mein Vater ruft dich, um dir den Lohn dafür zu geben, daß du uns (die Tiere) getränkt hast.» Als er zu ihm kam und ihm die Geschichte erzählte, sagte er: «Hab keine Angst. Du bist den Leuten entkommen, die Unrecht tun.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcclxxviii{Die eine von den beiden sagte: «O mein Vater, dinge ihn. Der Beste, den du dingen kannst, ist der, der stark und vertrauenswürdig ist.»\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcclxxix{Er sagte: «Ich will dir eine dieser meiner beiden Töchter zur Frau geben unter der Bedingung, daß du acht Jahre in meinen Dienst trittst. Wenn du sie aber auf zehn vollmachst, so ist das deine Entscheidung. Ich will gegen dich keine Härte zeigen. Du wirst, so Gott will, finden, daß ich zu den Rechtschaffenen gehöre.»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcclxxx{Er sagte: «Dies sei zwischen mir und dir abgemacht. Welche der beiden Fristen ich auch erfülle, es darf keine Übertretung gegen mich geben. Und Gott ist Sachwalter über das, was wir sagen.»\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcclxxxi{Als Mose die Frist erfüllte und mit seinen Angehörigen fortzog, nahm er auf der Seite des Berges ein Feuer wahr. Er sagte zu seinen Angehörigen: «Bleibt hier. Ich habe ein Feuer wahrgenommen. Vielleicht kann ich euch davon eine Nachricht oder einen Scheit aus dem Feuer bringen, so daß ihr euch wärmen könnt.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmcclxxxii{Als er dort ankam, wurde ihm vom rechten Ufer des Tals im gesegneten Land aus dem Baum zugerufen: «O Mose, wahrlich, Ich bin Gott, der Herr der Welten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmcclxxxiii{Und wirf deinen Stab.» Als er nun sah, daß er sich schüttelte, als wäre er eine flinke Schlange, kehrte er den Rücken und wandte sich nicht mehr um. - «O Mose, komm her und hab keine Angst. Du sollst zu denen gehören, die in Sicherheit sind.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmcclxxxiv{Und stecke deine Hand in deinen Hemdausschnitt, so kommt sie weiß, jedoch nicht von Übel befallen, heraus. Und zieh deinen Arm an dich aus Erschrecken. Das sind zwei Beweise von deinem Herrn, gerichtet an Pharao und seine Vornehmen. Sie sind ja ein frevlerisches Volk.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmcclxxxv{Er sagte: «Mein Herr, ich habe einen von ihnen getötet. So fürchte ich, daß sie mich töten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmcclxxxvi{Auch hat mein Bruder Aaron eine redegewandtere Zunge. So sende ihn mit mir zur Unterstützung, daß er mich bestätige. Ich fürchte, daß sie mich der Lüge zeihen.»\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmcclxxxvii{Er sprach: «Wir werden deinen Arm durch deinen Bruder festigen, und Wir erteilen euch beiden eine Ermächtigung, so daß sie nicht zu euch gelangen können. (Geht also) mit unseren Zeichen. Ihr und diejenigen, die euch folgen, werden die Sieger sein.»\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmcclxxxviii{Als nun Mose mit unseren Zeichen als deutlichen Beweisen kam, sagten sie: «Das ist nichts anderes als eine erdichtete Zauberei. Und wir haben bei unseren Vorvätern so etwas nicht gehört.»\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmcclxxxix{Mose sagte: «Mein Herr weiß besser, wer von Ihm her die Rechtleitung bringt und wem die jenseitige Wohnstätte gehört. Wahrlich, denen, die Unrecht tun, wird es nicht wohl ergehen.»\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmccxc{Und Pharao sagte: «O ihr Vornehmen, ich weiß euch keinen anderen Gott als mich. So entfache mir, o Haamaan, einen Brand auf Lehm, und mache mir einen offenen Hochbau, auf daß ich zum Gott des Mose emporsteige. Siehe, ich meine, daß er zu den Lügnern gehört.»\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmccxci{Er und seine Truppen zeigten sich auf der Erde zu Unrecht hochmütig, und sie meinten, daß sie nicht zu Uns zurückgebracht werden.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmccxcii{Da ergriffen Wir ihn und seine Truppen und warfen sie ins Meer. So schau, wie das Ende derer war, die Unrecht tun.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmccxciii{Und Wir machten sie zu Anführern, die zum Feuer rufen. Und am Tag der Auferstehung werden sie keine Unterstützung erfahren.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmccxciv{Und Wir ließen ihnen einen Fluch im Diesseits folgen, und am Tag der Auferstehung gehören sie zu den Verabscheuten.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmccxcv{Und Wir ließen Mose das Buch zukommen, nachdem Wir die früheren Generationen haben verderben lassen, als einsichtbringende Zeichen für die Menschen und als eine Rechtleitung und Barmherzigkeit, auf daß sie es bedenken.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmccxcvi{Und du warst nicht auf der Seite des westlichen Ortes, als Wir Mose die Angelegenheit auftrugen, und du gehörtest nicht zu den Zeugen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmccxcvii{Wir aber ließen Generationen entstehen, und sie erreichten ein hohes Alter. Und du hattest dich nicht unter den Leuten von Madyan niedergelassen, um ihnen unsere Zeichen zu verlesen. Wir aber entsandten (die Propheten).\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmccxcviii{Und du warst nicht auf der Seite des Berges, als Wir zuriefen. Aber (es geschah) als Barmherzigkeit von deinem Herrn, damit du Leute warnst, zu denen vor dir noch kein Warner gekommen ist, auf daß sie es bedenken.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmccxcix{Wir hätten es auch nicht getan), würden sie nicht, wenn sie ein Unglück trifft wegen dessen, was ihre Hände vorausgeschickt haben, sagen: «Unser Herr, wenn Du doch einen Gesandten zu uns geschickt hättest, so daß wir deinen Zeichen hätten folgen können und zu den Gläubigen gehört hätten!»\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmccc{Als nun von Uns her die Wahrheit zu ihnen kam, sagten sie: «Wäre ihm doch das gleiche zugekommen, was Mose zugekommen ist!» Haben sie denn nicht das verleugnet, was zuvor Mose zugekommen ist? Sie sagten: «Zwei Zauberwerke, die einander beistehen.» Und sie sagten: «Wir verleugnen sie alle beide.»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmccci{Sprich: Dann bringt ein Buch von Gott bei, das eine bessere Rechtleitung enthält als diese beiden, so will ich ihm folgen, wenn ihr die Wahrheit sagt.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmcccii{Wenn sie dich nicht erhören, dann sollst du wissen, daß sie nur ihren Neigungen folgen. Und wer ist weiter abgeirrt als der, der seiner Neigung folgt ohne eine Rechtleitung von Gott. Gott leitet die ungerechten Leute nicht recht.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmccciii{Und Wir haben ihnen das Wort Stück für Stück übermittelt, auf daß sie es bedenken.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmccciv{Diejenigen, denen Wir vor ihm das Buch zukommen ließen, sie glauben an ihn.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmcccv{Und wenn er ihnen verlesen wird, sagen sie: «Wir glauben an ihn. Es ist die Wahrheit von unserem Herrn. Wir waren schon vor ihm gottergeben.»\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmcccvi{Diese erhalten ihren Lohn zweifach dafür, daß sie geduldig waren. Und sie wehren das Böse mit dem Guten ab und spenden von dem, was Wir ihnen beschert haben.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmcccvii{Und wenn sie unbedachte Rede hören, wenden sie sich davon ab und sagen: «Wir haben unsere Werke und ihr habt eure Werke (zu verantworten). Friede sei über euch! Wir suchen nicht den Umgang mit den Törichten.»\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmcccviii{Du kannst nicht rechtleiten, wen du gern möchtest. Gott ist es, der rechtleitet, wen Er will. Er weiß besser, wer der Rechtleitung folgt.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmcccix{Und sie sagen: «Wenn wir mit dir der Rechtleitung folgen, werden wir von unserem Land weggerafft.» Haben Wir ihnen denn nicht als feste Stätte einen sicheren heiligen Bezirk gegeben, zu dem die verschiedenartigen Früchte zusammengetragen werden als Versorgung von Uns her? Aber die meisten von ihnen wissen nicht Bescheid.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmcccx{Und wie manche Stadt, die sich maßlos ihres Lebensunterhaltes erfreute, haben Wir verderben lassen! Da sind nun ihre Wohnungen, sie wurden nach ihnen nicht mehr bewohnt, bis auf einige wenige. Und Wir sind es, die (alles) geerbt haben.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmcccxi{Und dein Herr hätte unmöglich die Städte verderben lassen, ohne in ihrer Hauptstadt einen Gesandten erstehen zu lassen, der ihnen unsere Zeichen verliest. Und Wir hätten die Städte nie verderben lassen, wenn ihre Bewohner nicht Unrecht getan hätten.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmcccxii{Und was immer euch zuteil geworden ist, ist Nutznießung und Schmuck des diesseitigen Lebens. Was aber bei Gott ist, ist besser und beständiger. Habt ihr denn keinen Verstand?\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmcccxiii{Ist denn der, dem Wir ein schönes Versprechen gegeben haben und der es auch vorfinden wird, dem gleich, den Wir die Nutznießung des diesseitigen Lebens genießen lassen, der aber am Tag der Auferstehung zu denen gehören wird, die vorgeführt werden?\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmcccxiv{Und am Tag, da Er sie ruft und spricht: «Wo sind meine Teilhaber, die ihr immer wieder angegeben habt?»\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmcccxv{Diejenigen, gegen die der Spruch zu Recht fällig geworden ist, sagen: «Unser Herr, diese, die wir haben abirren lassen, haben wir abirren lassen, wie wir selbst abgeirrt waren. (Von ihnen) sagen wir uns vor Dir los. Nicht uns verehrten sie ja.»\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmcccxvi{Und es wird gesprochen: «Ruft eure Teilhaber.» Sie rufen sie an, aber sie erhören sie nicht. Und sie sehen die Pein. Wären sie doch der Rechtleitung gefolgt!\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmcccxvii{Und am Tag, da Er sie ruft und spricht: «Was habt ihr den Gesandten geantwortet?»\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmcccxviii{An jenem Tag ist ihnen entschwunden, was sie zu berichten haben, und sie befragen (auch) nicht einander.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmcccxix{Wer aber umkehrt und glaubt und Gutes tut, der möge zu denen gehören, denen es wohl ergeht.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmcccxx{Und dein Herr erschafft und wählt, was Er will. Sie aber haben nicht die Möglichkeit zu wählen. Preis sei Gott! Und erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmcccxxi{Und dein Herr weiß, was ihre Brust verhüllt und was sie offenlegen.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmcccxxii{Und Er ist Gott. Es gibt keinen Gott außer Ihm. Lob sei Ihm in der diesseitigen und der jenseitigen Welt! Ihm gehört das Urteil, und zu Ihm werdet ihr zurückgebracht.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmcccxxiii{Sprich: Was meint ihr? Würde Gott auf euch die Nacht als Dauerzustand legen bis zum Tag der Auferstehung, welcher Gott außer Gott würde euch Licht bringen? Wollt ihr denn nicht hören?\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmcccxxiv{Sprich: Was meint ihr? Würde Gott auf euch den Tag als Dauerzustand legen bis zum Tag der Auferstehung, welcher Gott außer Gott würde euch eine Nacht bringen, in der ihr ruhen könnt? Wollt ihr denn nicht einsichtig sein?\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmcccxxv{In seiner Barmherzigkeit hat Er euch die Nacht und den Tag gemacht, damit ihr darin ruht und auch nach etwas von seiner Huld strebt, auf daß ihr dankbar seid.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmcccxxvi{Und am Tag, da Er sie ruft und spricht: «Wo sind denn meine Teilhaber, die ihr immer wieder angegeben habt?»\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmcccxxvii{Und Wir ziehen aus jeder Gemeinschaft einen Zeugen heraus. Da sprechen Wir: «Bringt her euren Beweis.» So wissen sie, daß die Wahrheit Gott gehört. Und entschwunden ist ihnen, was sie zu erdichten pflegten.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmcccxxviii{Qarun gehörte zum Volk des Mose. Er behandelte sie mit ungerechter Gewalt. Und Wir ließen ihm solche Schätze zukommen, daß deren Schlüssel eine schwere Last für eine Schar kräftiger Männer gewesen wären. Als seine Leute zu ihm sagten: «Sei nicht so froh, Gott liebt die nicht, die zu sehr froh sind.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmcccxxix{Und strebe mit dem, was Gott dir zukommen ließ, nach der jenseitigen Wohnstätte, und vergiß auch nicht deinen Anteil am Diesseits. Und tu Gutes, so wie Gott dir Gutes getan hat. Und suche nicht das Unheil auf der Erde, Gott liebt ja nicht die Unheilstifter.»\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmcccxxx{Er sagte: «Es ist mir zugekommen aufgrund von Wissen, das ich besitze.» Wußte er denn nicht, daß Gott vor ihm solche Generationen hat verderben lassen, die eine stärkere Kraft besaßen und mehr Reichtum gesammelt hatten? Und die Übeltäter werden nicht nach ihren (einzelnen) Sünden gefragt.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmcccxxxi{Und er kam zu seinem Volk in seinem Schmuck heraus. Diejenigen, die das diesseitige Leben begehrten, sagten: «O hätten doch auch wir das gleiche wie das, was Qarun zugekommen ist! Er hat großes Glück.»\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmcccxxxii{Und diejenigen, denen das Wissen zugekommen war, sagten: «Wehe euch! Der Lohn Gottes ist besser für den, der glaubt und Gutes tut. Und es wird nur den Geduldigen dargeboten.»\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmcccxxxiii{Da ließen Wir die Erde mit ihm und mit seiner Wohnstätte versinken. Und da hatte er keine Schar, die ihn anstelle Gottes unterstützte, und er erfuhr auch selber keine Unterstützung.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmcccxxxiv{Und diejenigen, die sich am Tag zuvor seine Stelle gewünscht hatten, fingen schon am Morgen an zu sagen: «O weh, Gott teilt ja den Lebensunterhalt großzügig, wem von seinen Dienern Er will, und auch bemessen zu. Hätte Gott uns nicht eine Wohltat erwiesen, so hätte Er uns versinken lassen. O weh, den Ungläubigen ergeht es ja nicht wohl.»\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmcccxxxv{Das ist die jenseitige Wohnstätte. Wir bestimmen sie für diejenigen, die nicht hohe Macht auf der Erde und nicht Unheil suchen. Und das Ende gehört den Gottesfürchtigen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmcccxxxvi{Wer mit einer guten Tat kommt, erhält etwas Besseres als sie. Wenn aber einer mit einer schlechten Tat kommt, so wird denen, die bösen Taten begehen, nur das vergolten, was sie zu tun pflegten.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmcccxxxvii{Der dir den Koran verpflichtend gemacht hat, wird dich zu einem Ort der Wiederkehr zurückkehren lassen. Sprich: Mein Herr weiß besser, wer die Rechtleitung bringt und wer sich in einem offenkundigen Irrtum befindet.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmcccxxxviii{Und du hattest nicht erwartet, daß das Buch dir überbracht würde. Es ist nur aus Barmherzigkeit von deinem Herrn. So leiste den Ungläubigen keinen Beistand.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmcccxxxix{Und sie sollen dich von den Zeichen Gottes nicht abweisen, nachdem sie nun zu dir herabgesandt worden sind. Und ruf zu deinem Herrn, und sei nicht einer der Polytheisten.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmcccxl{Und rufe neben Gott keinen anderen Gott an. Es gibt keinen Gott außer Ihm. Alle Dinge werden untergehen, nur sein Antlitz nicht. Ihm gehört das Urteil, und zu Ihm werdet ihr zurückgebracht.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmcccxli{\basmalahde Alif Lam Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcccxlii{Meinen die Menschen, daß sie in Ruhe gelassen werden, nur weil sie sagen: «Wir glauben», ohne daß sie der Versuchung ausgesetzt werden?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcccxliii{Wir haben schon diejenigen, die vor ihnen lebten, der Versuchung ausgesetzt. Gott wird gewiß in Erfahrung bringen, wer die Wahrheit sagt, und Er wird gewiß in Erfahrung bringen, wer die Lügner sind.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcccxliv{Oder meinen diejenigen, die schlechte Taten begehen, sie seien uns voraus? Schlecht ist ihr Urteil.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcccxlv{Wenn einer die Begegnung mit Gott erwartet, so wird die Frist Gottes sicher eintreffen. Und Er ist der der alles hört und weiß.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcccxlvi{Und wer sich einsetzt, setzt sich ein zu seinem eigenen Vorteil. Gott ist ja auf die Weltenbewohner nicht angewiesen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcccxlvii{Denjenigen, die glauben und die guten Werke tun, werden Wir ihre Missetaten sühnen, und Wir werden ihnen das Beste vergelten von dem, was sie taten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcccxlviii{Und Wir haben dem Menschen aufgetragen, seine Eltern gut zu behandeln. Wenn sie dich aber bedrängen, Mir das beizugesellen, wovon du kein Wissen hast, dann gehorche ihnen nicht. Zu Mir wird eure Rückkehr sein, da werde Ich euch kundtun, was ihr zu tun pflegtet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcccxlix{Diejenigen, die glauben und die guten Werke tun, lassen Wir in die Reihe der Rechtschaffenen eingehen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcccl{Und unter den Menschen ist manch einer, der sagt: «Wir glauben an Gott.» Wenn er aber um Gottes willen etwas erleidet, setzt er die Anfechtung durch die Menschen der Pein Gottes gleich. Wenn jedoch von deinem Herrn eine Unterstützung kommt, sagen sie bestimmt: «Wir sind doch mit euch gewesen.» Weiß Gott denn nicht besser Bescheid über das, was im Inneren der Weltenbewohner steckt?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcccli{Und Gott wird gewiß in Erfahrung bringen, wer glaubt, und Er wird gewiß in Erfahrung bringen, wer die Heuchler sind.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmccclii{Und diejenigen, die ungläubig sind, sagen zu denen, die glauben: «Folgt unserem Weg. Laßt uns eure Verfehlungen tragen.» Tragen werden sie aber nichts von ihren Verfehlungen. Sie sind ja Lügner.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcccliii{Tragen werden sie ihre (eigenen) Lasten, und auch (weitere) Lasten zu ihren eigenen Lasten hinzu. Und sie werden am Tag der Auferstehung zu verantworten haben, was sie zu erdichten pflegten.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcccliv{Und Wir sandten Noach zu seinem Volk. Er verweilte unter ihnen tausend Jahre weniger fünfzig Jahre. Da ergriff sie die Flut, während sie Unrecht taten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmccclv{Und Wir retteten ihn und die Insassen des Schiffes, und Wir machten es zu einem Zeichen für die Weltenbewohner.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmccclvi{Und (Wir sandten) Abraham. Als er zu seinem Volk sagte: «Dienet Gott und fürchtet Ihn. Das ist besser für euch, so ihr Bescheid wißt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmccclvii{Ihr dient anstelle Gottes Götzen und schafft (dabei) nur Lüge. Die, denen ihr anstelle Gottes dient, können euch keinen Lebensunterhalt bringen. So sucht den Lebensunterhalt bei Gott und dienet Ihm und danket Ihm. Zu Ihm werdet ihr zurückgebracht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmccclviii{Und wenn ihr (die Botschaft) für Lüge erklärt, so haben vor euch (manche) Gemeinschaften (sie) für Lüge erklärt. Und dem Gesandten obliegt nur die deutliche Ausrichtung (der Botschaft).»\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmccclix{Haben sie denn nicht gesehen, wie Gott die Schöpfung am Anfang macht und sie dann wiederholt? Dies ist für Gott ein leichtes.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmccclx{Sprich: Geht auf der Erde umher und schaut, wie Er die Schöpfung am Anfang gemacht hat. Dann läßt Gott die letzte Schöpfung entstehen. Gott hat Macht zu allen Dingen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmccclxi{Er peinigt, wen Er will, und Er erbarmt sich, wessen Er will. Und zu Ihm werdet ihr zurückgebracht.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmccclxii{Und ihr könnt weder auf der Erde noch im Himmel etwas vereiteln. Und ihr habt außer Gott weder Freund noch Helfer.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmccclxiii{Und diejenigen, die die Zeichen Gottes und die Begegnung mit Ihm verleugnen, diese haben die Hoffnung auf meine Barmherzigkeit verloren, und für sie ist eine schmerzhafte Pein bestimmt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmccclxiv{Die Antwort seines Volkes war nur, daß sie sagten: «Tötet ihn oder verbrennt ihn.» Da rettete ihn Gott aus dem Feuer. Darin sind Zeichen für Leute, die glauben.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmccclxv{Und er sagte: «Ihr habt euch ja anstelle Gottes Götzen genommen aus Liebe zueinander im diesseitigen Leben. Aber dann, am Tag der Auferstehung verleugnet ihr einander und verflucht ihr einander. Eure Heimstätte ist das Feuer, und ihr werdet keine Helfer haben.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmccclxvi{Da glaubte Lot ihm und sagte: «Ich wandere zu meinem Herrn aus. Er ist der Mächtige, der Weise.»\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmccclxvii{Und Wir schenkten ihm Isaak und Jakob und ließen in seiner Nachkommenschaft die Prophetie und das Buch auftreten, und Wir ließen ihm seinen Lohn im Diesseits zukommen. Und im Jenseits gehört er zu den Rechtschaffenen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmccclxviii{Und (Wir sandten) Lot. Als er zu seinem Volk sagte: «Ihr begeht das Schändliche, wie es vor euch keiner von den Weltenbewohnern getan hat.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmccclxix{Wollt ihr denn zu den Männern gehen, den Weg zu Raubzwecken abschneiden und in eurer Versammlung das Verwerfliche begehen?» Die Antwort seines Volkes war nur, daß sie sagten: «Bring uns doch die Pein Gottes her, so du einer von denen bist, die die Wahrheit sagen.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmccclxx{Er sagte: «Mein Herr, unterstütze mich gegen die Leute, die Unheil stiften.»\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmccclxxi{Und als unsere Boten zu Abraham mit der frohen Botschaft kamen, sagten sie: «Wir werden die Bewohner dieser Stadt verderben lassen. Ihre Bewohner tun ja Unrecht.»\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmccclxxii{Er sagte: «Aber Lot befindet sich in ihr.» Sie sagten: «Wir wissen besser, wer sich in ihr befindet. Wir werden ihn gewiß erretten, ihn und seine Angehörigen, außer seiner Frau. Sie gehört zu denen, die zurückbleiben und dem Verderben anheimfallen.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmccclxxiii{Und als unsere Boten zu Lot kamen, geriet er ihretwegen in eine böse Lage und wußte daraus keinen Ausweg. Sie sagten: «Hab keine Angst und sei nicht betrübt. Wir werden dich erretten, dich und deine Angehörigen, außer deiner Frau. Sie gehört zu denen, die zurückbleiben und dem Verderben anheimfallen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmccclxxiv{Wir werden über die Bewohner dieser Stadt ein Zorngericht vom Himmel herabsenden dafür, daß sie gefrevelt haben.»\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmccclxxv{Und Wir ließen von ihr ein offenkundiges Zeichen zurück für Leute, die Verstand haben.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmccclxxvi{Und (Wir sandten) zu Madyan ihren Bruder Shuayb. Er sagte: «O mein Volk, dienet Gott, erwartet den Jüngsten Tag und verbreitet nicht Unheil auf der Erde.»\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmccclxxvii{Aber sie ziehen ihn der Lüge. Da ergriff sie das Beben, und am Morgen lagen sie in ihrer Wohnstätte nieder.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmccclxxviii{Und (erwähne auch) die, Aad und die Thamud. (Ihr Los) ist euch deutlich geworden an ihren Wohnungen. Der Satan machte ihnen ihre Werke verlockend und wies sie so vom Weg ab, obwohl sie Einsicht besaßen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmccclxxix{Und Qarun und Pharao und Haamaan. Mose kam zu ihnen mit den deutlichen Zeichen. Sie verhielten sich hochmütig auf der Erde; sie konnten (Uns) aber nicht vorauseilen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmccclxxx{Einen jeden ergriffen Wir wegen seiner Sünde. So schickten Wir gegen einige von ihnen einen Sandsturm, andere ergriff der Schrei, mit anderen ließen Wir die Erde versinken, andere ließen Wir ertrinken. Bestimmt nicht Gott hat ihnen Unrecht getan, sondern sie haben sich selbst Unrecht getan.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmccclxxxi{Mit denen, die sich anstelle Gottes Freunde nehmen, ist es wie mit der Spinne, die sich ein Haus genommen hat. Das schwächste Haus ist ja das Haus der Spinne, wenn sie es doch wüßten!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmccclxxxii{Gott weiß all das, was sie an seiner Stelle anrufen. Und Er ist der Mächtige, der Weise.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmccclxxxiii{Diese Gleichnisse führen Wir den Menschen an. Aber nur die verstehen sie, die Wissen besitzen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmccclxxxiv{Gott hat die Himmel und die Erde in Wahrheit erschaffen. Darin ist ein Zeichen für die Gläubigen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmccclxxxv{Verlies, was dir vom Buch offenbart wird, und verrichte das Gebet. Das Gebet verbietet das Schändliche und das Verwerfliche. Und wahrlich, das Gedenken Gottes ist größer. Und Gott weiß, was ihr macht.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmccclxxxvi{Und streitet mit den Leuten des Buches nur auf die beste Art, mit Ausnahme derer von ihnen, die Unrecht tun. Und sagt: «Wir glauben an das, was zu uns herabgesandt und zu euch herabgesandt wurde. Unser Gott und euer Gott ist einer. Und wir sind Ihm ergeben.»\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmccclxxxvii{Und so haben Wir das Buch zu dir hinabgesandt. Diejenigen, denen Wir das Buch haben zukommen lassen, glauben daran. Und auch unter diesen da gibt es welche, die daran glauben. Und nur die Ungläubigen verleugnen unsere Zeichen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmccclxxxviii{Und du hast vordem kein Buch verlesen und es auch nicht mit deiner rechten Hand geschrieben. Sonst würden die zweifeln, die (es) für falsch erklären.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmccclxxxix{Nein, es enthält deutliche Zeichen in der Brust derer, denen das Wissen zugekommen ist. Und nur die, die Unrecht tun, verleugnen unsere Zeichen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmcccxc{Und sie sagen: «Wenn doch Zeichen von seinem Herrn auf ihn herabgesandt würden!» Sprich: Über die Zeichen verfügt Gott. Ich aber bin nur ein deutlicher Warner.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmcccxci{Genügt es ihnen denn nicht, daß Wir das Buch auf dich hinabgesandt haben, das ihnen verlesen wird? Darin ist eine Barmherzigkeit und eine Ermahnung für Leute, die glauben.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmcccxcii{Sprich: Gott genügt als Zeuge zwischen mir und euch. Er weiß, was in den Himmeln und auf Erden ist. Und diejenigen, die an das Falsche glauben und Gott verleugnen, das sind die Verlierer.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmcccxciii{Und sie wünschen von dir, die Pein zu beschleunigen. Gäbe es nicht eine festgesetzte Frist, wäre die Pein zu ihnen gekommen. Wahrlich, sie wird plötzlich über sie kommen, ohne daß sie es merken.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmcccxciv{Sie wünschen von dir, die Pein zu beschleunigen. Wahrlich, die Hölle umfaßt die Ungläubigen.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmcccxcv{Am Tag, da die Pein sie überdeckt von oben und von unter ihren Füßen her, und Er spricht: «Kostet, was ihr zu tun pflegtet.»\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmcccxcvi{O ihr meine Diener, die ihr glaubt, meine Erde ist weit. Mir, ja Mir sollt ihr dienen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmcccxcvii{Jeder wird den Tod erleiden. Und dann werdet ihr zu Uns zurückgebracht.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmcccxcviii{Diejenigen, die glauben und die guten Werke tun, werden Wir im Paradies in Obergemächer einweisen, unter denen Bäche fließen; darin werden sie ewig weilen. Vorzüglich ist der Lohn derer, die (gut) handeln,\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmcccxcix{Die geduldig sind und auf ihren Herrn vertrauen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmcd{Und wie viele Tiere gibt es, die nicht ihren eigenen Unterhalt herbeitragen. Gott beschert ihnen und euch den Unterhalt. Und Er ist der, der alles hört und weiß.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmcdi{Und wenn du sie fragst, wer die Himmel und die Erde erschaffen und die Sonne und den Mond dienstbar gemacht hat, sagen sie bestimmt: «Gott.» Wie leicht lassen sie sich doch abwenden!\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmcdii{Gott teilt den Lebensunterhalt großzügig, wem von seinen Dienern Er will, und teilt ihm auch bemessen zu. Gott weiß über alle Dinge Bescheid.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmcdiii{Und wenn du sie fragst, wer Wasser vom Himmel herabkommen läßt und die Erde damit nach ihrem Absterben belebt, sagen sie bestimmt: «Gott.» Sprich: Lob sei Gott! Aber die meisten von ihnen haben keinen Verstand.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmcdiv{Das diesseitige Leben ist nur Zerstreuung und Spiel. Die jenseitige Wohnstätte ist wahrlich das (eigentliche) Leben, wenn sie es nur wüßten!\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmcdv{Wenn sie in ein Schiff einsteigen, rufen sie Gott an, wobei sie Ihm gegenüber aufrichtig in der Religion sind. Kaum hat Er sie ans Land errettet, da gesellen sie (Ihm wieder andere) bei,\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmcdvi{Um sich undankbar zu zeigen für das, was Wir ihnen haben zukommen lassen, und um zu genießen. Sie werden es noch zu wissen bekommen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmcdvii{Haben sie denn nicht gesehen, daß Wir einen sicheren heiligen Bezirk gemacht haben, während um ihn die Menschen weggerafft werden? Wollen sie denn an das Falsche glauben und die Gnade Gottes verleugnen?\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmcdviii{Und wer ist ungerechter als der, der gegen Gott eine Lüge erdichtet oder die Wahrheit, als sie zu ihm kam, für Lüge erklärt? Ist nicht in der Hölle eine Bleibe für die Ungläubigen?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmcdix{Und diejenigen, die sich um Unseretwillen einsetzen, werden Wir gewiß unsere Wege führen. Und Gott ist mit denen, die Gutes tun.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmcdx{\basmalahde Alif Lam Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcdxi{Die Byzantiner sind besiegt worden\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcdxii{Im nächstliegenden Land. Aber sie werden nach ihrer Niederlage selbst siegen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcdxiii{In einigen Jahren. Gott gehört der Befehl vorher und nachher. An jenem Tag werden die Gläubigen sich freuen\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcdxiv{über die Unterstützung Gottes. Gott unterstützt, wen Er will. Und Er ist der Mächtige, der Barmherzige.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcdxv{Das ist ein Versprechen Gottes. Gott bricht sein Versprechen nicht. Aber die meisten Menschen wissen nicht Bescheid.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcdxvi{Sie wissen nur das Äußere vom diesseitigen Leben. Das Jenseits aber lassen sie unbeachtet.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcdxvii{Denken sie denn in ihrem Inneren nicht darüber nach? Gott hat die Himmel und die Erde, und was dazwischen ist, nur in Wahrheit und auf eine festgesetzte Frist erschaffen. Aber viele von den Menschen verleugnen die Begegnung mit ihrem Herrn.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcdxviii{Sind sie nicht auf der Erde umhergegangen und haben geschaut, wie das Ende derer war, die vor ihnen lebten? Sie hatten eine stärkere Kraft als sie, pflügten und bebauten das Land noch mehr, als sie es bebauten. Und ihre Gesandten kamen zu ihnen mit den deutlichen Zeichen. Nicht Gott hat ihnen Unrecht getan, sondern sie haben sich selbst Unrecht getan.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcdxix{Dann war das Ende derer, die Böses getan haben, das Schlechteste dafür, daß sie die Zeichen Gottes für Lüge erklärt hatten und über sie zu spotten pflegten.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcdxx{Gott macht die Schöpfung am Anfang, und dann wiederholt Er sie. Dann werdet ihr zu Ihm zurückgebracht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcdxxi{Und am Tag, da die Stunde heraufkommt, werden die Übeltäter ganz verzweifelt sein.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcdxxii{Sie haben dann an ihren Teilhabern keine Fürsprecher, und sie verleugnen ihre Teilhaber.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcdxxiii{Am Tag, da die Stunde heraufkommt, an jenem Tag werden sie sich in Gruppen teilen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcdxxiv{Was nun diejenigen betrifft, die geglaubt und die guten Werke getan haben, so wird ihnen in einem Garten Freude bereitet.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcdxxv{Diejenigen aber, die ungläubig waren und unsere Zeichen und die Begegnung mit dem Jenseits für Lüge erklärt haben, sie werden zur Pein vorgeführt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcdxxvi{Preis sei Gott, wenn ihr den Abend und wenn ihr den Morgen erreicht!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcdxxvii{Und Lob sei Ihm in den Himmeln und auf der Erde, am Abend, und wenn ihr den Mittag erreicht!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcdxxviii{Er bringt das Lebendige aus dem Toten, und Er bringt das Tote aus dem Lebendigen hervor. Und Er belebt die Erde nach ihrem Absterben. Und so werdet auch ihr hervorgebracht.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcdxxix{Und es gehört zu seinen Zeichen, daß Er euch aus Erde erschaffen hat, da waret ihr Menschen, die sich ausbreiten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcdxxx{Und es gehört zu seinen Zeichen, daß Er euch aus euch selbst Gattinnen erschaffen hat, damit ihr bei ihnen wohnet. Und Er hat Liebe und Barmherzigkeit zwischen euch gemacht. Darin sind Zeichen für Leute, die nachdenken.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcdxxxi{Zu seinen Zeichen gehört die Erschaffung der Himmel und der Erde, und auch die Verschiedenheit eurer Sprachen und Arten. Darin sind Zeichen für die Wissenden.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcdxxxii{Und zu seinen Zeichen gehört euer Schlaf, und auch euer Streben nach etwas von seiner Huld, und (dies) in der Nacht und am Tag. Darin sind Zeichen für Leute, die hören.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcdxxxiii{Und es gehört zu seinen Zeichen, daß Er euch den Blitz als Grund zur Angst und zum Begehren sehen läßt, und daß Er Wasser vom Himmel herabkommen läßt und damit die Erde nach ihrem Absterben belebt. Darin sind Zeichen für Leute, die Verstand haben.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcdxxxiv{Und es gehört zu seinen Zeichen, daß der Himmel und die Erde durch seinen Befehl bestehen. Dann, wenn Er euch mit einem Ruf aus der Erde ruft, da kommt ihr hervor.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcdxxxv{Ihm gehört, wer in den Himmeln und auf der Erde ist. Alle sind Ihm demütig ergeben.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcdxxxvi{Er ist es, der die Schöpfung am Anfang macht und sie dann wiederholt. Und das ist für Ihn noch leichter. Er besitzt die höchste Eigenschaft in den Himmeln und auf der Erde, und Er ist der Mächtige, der Weise.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcdxxxvii{Er führt euch aus eurem Lebensbereich ein Gleichnis an. Habt ihr denn aus den Reihen derer, die eure rechte Hand besitzt, welche, die an dem teilhaben, was Wir euch beschert haben, daß ihr darin gleich wäret und ihr sie fürchten müßtet, wie ihr einander fürchtet? So legen Wir die Zeichen dar für Leute, die Verstand haben.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcdxxxviii{Nein, diejenigen, die Unrecht tun, folgen ihren Neigungen ohne (richtiges) Wissen. Wer kann den rechtleiten, den Gott irreführt? Und sie haben keine Helfer.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmcdxxxix{Und richte dein Gesicht auf die Religion als Anhänger des reinen Glaubens. Das ist die Schöpfung Gottes, die Er für die Menschen festgelegt hat. Die Schöpfung Gottes kann nicht abgeändert werden. Das ist die richtige Religion. Aber die meisten Menschen wissen nicht Bescheid.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmcdxl{Haltet daran fest), indem ihr euch Ihm reumütig zuwendet, und fürchtet Ihn und verrichtet das Gebet. Und ihr sollt nicht zu den Polytheisten gehören,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmcdxli{Zu denen, die ihre Religion spalteten und zu Parteien wurden, wobei jede Partei froh ist über das, was sie hat.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmcdxlii{Und wenn ein Schaden die Menschen erfaßt, rufen sie zu ihrem Herrn, indem sie sich Ihm reumütig zuwenden. Wenn Er sie dann eine Barmherzigkeit von sich kosten läßt, da ist gleich ein Teil von ihnen dabei, ihrem Herrn (andere) beizugesellen,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmcdxliii{Um undankbar zu sein für das, was Wir ihnen haben zukommen lassen. - Genießt nur, ihr werdet es noch zu wissen bekommen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmcdxliv{Oder haben Wir etwa auf sie eine Ermächtigung hinabgesandt, die über das geredet hätte, was sie Ihm beigesellen?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmcdxlv{Und wenn Wir die Menschen Barmherzigkeit kosten lassen, freuen sie sich darüber. Wenn sie aber etwas Böses trifft für das, was ihre Hände vorausgeschickt haben, geben sie gleich die Hoffnung auf.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmcdxlvi{Haben sie denn nicht gesehen, daß Gott den Lebensunterhalt großzügig zuteilt, wem Er will, und auch bemessen? Darin sind Zeichen für Leute, die glauben.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmcdxlvii{Laß dem Verwandten sein Recht zukommen, ebenso dem Bedürftigen und dem Reisenden. Das ist besser für die, die die Antlitz Gottes suchen. Das sind die, denen es wohl ergeht,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmcdxlviii{Und was ihr auf Zins ausleiht, damit es sich aus dem Vermögen der Menschen vermehre, es vermehrt sich bei Gott nicht. Und was ihr an Almosen gebt in der Suche nach dem Antlitz Gottes... - das sind die, die das Doppelte erzielen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmcdxlix{Gott ist es der euch erschafft und dann versorgt. Dann läßt Er euch sterben, dann macht Er euch wieder lebendig. Gibt es unter euren Teilhabern einen, der überhaupt etwas von alledem tun kann? Preis sei Ihm, und erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmcdl{Unheil ist auf dem Festland und auf dem Meer erschienen aufgrund dessen, was die Hände der Menschen erworben haben. Er will sie damit einiges kosten lassen von dem, was sie getan haben, auf daß sie (dann) umkehren.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmcdli{Sprich: Geht auf der Erde umher und schaut, wie das Ende derer war, die früher lebten. Die meisten von ihnen waren Polytheisten.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmcdlii{Richte nun dein Gesicht auf die richtige Religion, bevor ein Tag kommt, der von Gott nicht zurückgewiesen wird. An jenem Tag werden sie sich (in Gruppen) spalten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmcdliii{Wer ungläubig ist, dessen Unglaube lastet auf ihm. Und diejenigen, die Gutes tun, bereiten sich selbst die Lagerstätte vor,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmcdliv{Daß Er denen, die glauben und die guten Werke tun, aus seiner Huld vergelte. Er liebt nicht die Ungläubigen.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmcdlv{Und es gehört zu seinen Zeichen, daß Er die Winde als Freudenboten schickt, damit Er euch etwas von seiner Barmherzigkeit kosten läßt, und damit die Schiffe auf seinen Befehl fahren, und damit ihr nach etwas von seiner Huld strebt, auf daß ihr dankbar seid.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmcdlvi{Und Wir haben schon vor dir Gesandte an ihr (jeweiliges) Volk geschickt. Sie kamen zu ihnen mit den deutlichen Zeichen. Da rächten Wir uns an denen, die Übeltäter waren. Und es war für Uns eine Verpflichtung, die Gläubigen zu unterstützen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmcdlvii{Gott ist es, der die Winde schickt, und sie wühlen die Wolken auf. Dann breitet Er sie im Himmel aus, wie Er will, und macht sie zu Stücken. Da siehst du den Platzregen dazwischen heraus- kommen. Wenn Er damit die von seinen Dienern, die Er will, trifft, da sind sie froh,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmcdlviii{Obwohl sie vorher, bevor auf sie herabgesandt wurde, ganz verzweifelt waren.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmcdlix{Schau auf die Spuren der Barmherzigkeit Gottes, wie Er die Erde nach ihrem Absterben wieder belebt. Ein solcher (Gott) kann wahrlich (auch) die Toten wieder lebendig machen. Und Er hat Macht zu allen Dingen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmcdlx{Und wenn Wir einen Wind schickten und sie dann alles gelb werden sähen, sie blieben trotzdem danach ungläubig.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmcdlxi{Du kannst nicht die Toten hören lassen, und auch nicht die Tauben den Zuruf hören lassen, wenn sie den Rücken kehren.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmcdlxii{Und du kannst nicht die Blinden aus ihrem Irrtum herausführen. Hören lassen kannst du nur die, die an unsere Zeichen glauben und somit gottergeben sind.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmcdlxiii{Gott ist es, der euch (zuerst) schwach erschafft. Dann bringt Er nach der Schwäche Kraft. Dann bringt Er nach der Kraft Schwäche und graues Haar. Er erschafft, was Er will. Und Er ist der, der alles weiß und allmächtig ist.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmcdlxiv{Und am Tag, da die Stunde heraufkommt, schwören die Übeltäter, sie hätten nur eine Stunde verweilt. So lassen sie sich immer wieder täuschen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmcdlxv{Und diejenigen, denen Wissen und Glauben zugekommen sind, sagen: «Nach dem Buch Gottes habt ihr bis zum Tag der Auferweckung verweilt. Und das ist der Tag der Auferweckung. Aber ihr wußtet es ja nicht.»\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmcdlxvi{An jenem Tag nützt denen, die Unrecht taten, ihre Entschuldigung nicht, und es wird von ihnen keine entschuldigende Umkehr (mehr) angenommen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmcdlxvii{Und Wir haben den Menschen in diesem Koran verschiedene Gleichnisse angeführt. Und wenn du zu ihnen mit einem Zeichen kommst, sagen diejenigen, die ungläubig sind, bestimmt: «Ihr bringt nur Falsches.»\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmcdlxviii{So versiegelt Gott die Herzen derer, die nicht Bescheid wissen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmcdlxix{Sei nun geduldig. Das Versprechen Gottes ist wahr. Und diejenigen, die keine Gewißheit hegen, sollen dich nicht abirren lassen.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmcdlxx{\basmalahde Alif Lam Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcdlxxi{Dies sind die Zeichen des weisen Buches,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcdlxxii{Eine Rechtleitung und Barmherzigkeit für die Rechtschaffenen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcdlxxiii{Die das Gebet verrichten und die Abgabe entrichten, und die über das Jenseits Gewißheit hegen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcdlxxiv{Diese folgen einer Rechtleitung von ihrem Herrn, und das sind die, denen es wohl ergeht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcdlxxv{Unter den Menschen gibt es manch einen, der ergötzende Unterhaltung einhandelt, um (die Menschen) vom Weg Gottes ohne (rechtes) Wissen abirren zu lassen und ihn zum Gegenstand des Spottes zu nehmen. Für solche ist eine schmähliche Pein bestimmt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcdlxxvi{Und wenn ihm unsere Zeichen verlesen werden, kehrt er sich hochmütig ab, als hätte er sie nicht gehört, als wäre eine Schwerhörigkeit in seinen Ohren. So verkünde ihm eine schmerzhafte Pein.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcdlxxvii{Für diejenigen, die glauben und die guten Werke tun, sind die Gärten der Wonne bestimmt;\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcdlxxviii{Darin werden sie ewig weilen. Dies ist das Versprechen Gottes in Wahrheit. Und Er ist der Mächtige, der Weise.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcdlxxix{Er hat die Himmel erschaffen ohne Stützen, die ihr sehen könntet. Und Er hat auf der Erde festgegründete Berge gelegt, daß sie nicht mit euch schwanke. Und Er hat auf ihr allerlei Getier sich ausbreiten lassen. Und Wir haben Wasser vom Himmel herabkommen und damit viele treffliche Arten auf ihr wachsen lassen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcdlxxx{Das ist die Schöpfung Gottes. Zeigt mir nun, was die geschaffen haben, die es außer Ihm geben soll. Nein, diejenigen, die Unrecht tun, befinden sich in einem offenkundigen Irrtum.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxi{Und Wir ließen den Luqmaan Weisheit zukommen: «Du sollst gegen Gott dankbar sein.» Und wer dankbar ist, ist dankbar zu seinem eigenen Vorteil. Und wenn einer undankbar ist, so ist Gott auf niemanden angewiesen und des Lobes würdig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxii{Und als Luqmaan zu seinem Sohn sagte, indem er ihn ermahnte: «O mein lieber Sohn, geselle Gott nichts bei. Die Beigesellung ist ein gewaltiges Unrecht.»\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxiii{Und Wir haben dem Menschen aufgetragen, seine Eltern gut zu behandeln. Seine Mutter hat ihn ja unter wiederholter Schwäche getragen. Und seine Entwöhnung erfolgt binnen zwei Jahren. - «Sei dankbar Mir und deinen Eltern. Zu Mir führt der Lebensweg.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxiv{Wenn sie dich bedrängen, Mir das beizugesellen, wovon du kein Wissen hast, dann gehorche ihnen nicht. Und geh mit ihnen im Diesseits in rechtlicher Weise um. Und folge dem Weg derer, die sich Mir reumütig zuwenden. Zu Mir wird dann eure Rückkehr sein, da werde Ich euch kundtun, was ihr zu tun pflegtet.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxv{O mein lieber Sohn, wäre es auch das Gewicht eines Senfkornes und befände es sich in einem Felsen oder in den Himmeln oder auf der Erde, Gott bringt es bei. Gott ist feinfühlig und hat Kenntnis von allem.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxvi{O mein lieber Sohn, verrichte das Gebet, gebiete das Rechte und verbiete das Verwerfliche, ertrage geduldig, was dich trifft. Das gehört zur Entschlossenheit in den Anliegen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxvii{Und zeige den Menschen nicht hochnäsig die Wange, und schreite nicht unbekümmert auf der Erde umher. Gott liebt niemanden, der eingebildet und prahlerisch ist.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxviii{Halte das rechte Maß in deinem Gang. Und dämpfe deine Stimme. Die widerlichste unter den Stimmen ist die Stimme der Esel.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxix{Habt ihr nicht gesehen, daß Gott euch das, was in den Himmeln und auf der Erde ist, dienstbar gemacht hat, und daß Er über euch seine Gnade ausgegossen hat äußerlich und innerlich? Und unter den Menschen gibt es welche, die über Gott streiten ohne (richtiges) Wissen, ohne Rechtleitung und ohne erleuchtendes Buch.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcdxc{Und wenn zu ihnen gesagt wird: «Folgt dem, was Gott herabgesandt hat», sagen sie: «Wir folgen lieber dem, was wir bei unseren Vätern vorgefunden haben.» Was denn, auch wenn der Satan sie zur Pein des Feuerbrandes ruft?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcdxci{Wer sich Gott völlig hingibt und dabei rechtschaffen ist, der hält sich an der festesten Handhabe. Und zu Gott führt das Ende der Angelegenheiten.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcdxcii{Und wer ungläubig ist, dessen Unglaube soll dich nicht betrüben. Zu Uns wird ihre Rückkehr sein, da werden Wir ihnen kundtun, was sie taten. Gott weiß über das innere Geheimnis Bescheid.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcdxciii{Wir lassen sie ein wenig genießen, alsdann zwingen Wir sie in eine schwere Pein.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcdxciv{Und wenn du sie fragst, wer die Himmel und die Erde erschaffen hat, sagen sie bestimmt: «Gott.» Sprich: Lob sei Gott! Aber die meisten von ihnen wissen nicht Bescheid.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcdxcv{Gott gehört, was in den Himmeln und auf der Erde ist. Gott ist der, der auf niemanden angewiesen und des Lobes würdig ist.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcdxcvi{Und wenn das, was es auf der Erde an Bäumen gibt, Schreibrohre wären, und das Meer (als Tinte) bereits einmal leer gemacht wäre und noch sieben weitere Meere dazu erhielte, würden die Worte Gottes nicht zu Ende gehen. Wahrlich, Gott ist mächtig und weise.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcdxcvii{Euch (alle) erschaffen und auferwecken ist nur so wie bei einem einzigen Menschen. Gott hört und sieht alles.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcdxcviii{Hast du nicht gesehen, daß Gott die Nacht in den Tag übergehen und den Tag in die Nacht übergehen läßt und die Sonne und den Mond dienstbar gemacht hat, so daß jedes seinen Lauf zu einer festgelegten Frist nimmt, und daß Gott Kenntnis hat von dem, was ihr tut?\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmcdxcix{Dies, weil Gott die Wahrheit ist und weil das, was sie an seiner Stelle anrufen, das Falsche ist, und weil Gott der Erhabene und Große ist.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmd{Hast du nicht gesehen, daß die Schiffe durch die Gnade Gottes auf dem Meer fahren, damit Er euch etwas von seinen Zeichen sehen läßt? Darin sind wahrlich Zeichen für jeden, der sehr geduldig und sehr dankbar ist.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdi{Und wenn Wellen wie überschattende Hüllen sie überdecken, rufen sie zu Gott, wobei sie Ihm gegenüber aufrichtig in der Religion sind. Wenn Er sie ans Land errettet, zeigen nur einige von ihnen einen maßvollen Wandel. Und nur der verleugnet unsere Zeichen, der ganz treulos und sehr undankbar ist.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdii{O ihr Menschen, fürchtet euren Herrn und habt Angst vor einem Tag, an dem weder der Vater etwas für sein Kind begleichen kann, noch das Kind für seinen Vater etwas begleichen kann. Das Versprechen Gottes ist wahr. So soll euch das diesseitige Leben nicht betören. Und nicht betören soll euch in bezug auf Gott der Betörer.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdiii{Gott (allein) weiß über die Stunde Bescheid. Er läßt den Regen herabkommen. Und Er weiß, was im Mutterschoß ist. Niemand weiß, was er morgen erwerben wird, und niemand weiß, in welchem Land er sterben wird. Gott weiß Bescheid und hat Kenntnis von allem.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdiv{\basmalahde Alif Lam Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdv{Die Herabsendung des Buches - an ihm ist kein Zweifel möglich - ist vom Herrn der Welten.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdvi{Oder sagen sie: «Er hat es erdichtet»? Nein, es ist die Wahrheit vor deinem Herrn, damit du Leute warnst, zu denen vor dir noch kein Warner gekommen ist, auf daß sie der Rechtleitung folgen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdvii{Gott ist es, der die Himmel und die Erde und das, was dazwischen ist, in sechs Tagen erschaffen und sich dann auf dem Thron zurechtgesetzt hat. Ihr habt außer Ihm weder Freund noch Fürsprecher. Wollt ihr es denn nicht bedenken?\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdviii{Er regelt die Angelegenheit vom Himmel bis zur Erde. Dann steigt sie zu Ihm empor an einem Tag, dessen Ausmaß nach eurer Berechnung tausend Jahre sind.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdix{Jener ist der, der über das Unsichtbare und das Offenbare Bescheid weiß, der Mächtige, der Barmherzige,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdx{Der alles, was Er erschaffen hat, gut gemacht hat. Zuerst erschuf Er den Menschen aus Ton,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdxi{Dann machte Er seine Nachkommenschaft aus dem Erguß eines verächtlichen Wassers.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdxii{Dann formte Er ihn und blies ihm von seinem Geist ein. Und Er machte euch Gehör, Augenlicht und Herz. Ihr seid aber wenig dankbar.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdxiii{Und sie sagen: «Sollen wir, wenn wir uns in der Erde verirrt haben, wirklich wieder neu erschaffen werden?» Nein, sie verleugnen die Begegnung mit ihrem Herrn.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdxiv{Sprich: Abberufen wird euch der Engel des Todes, der mit euch betraut ist. Dann werdet ihr zu eurem Herrn zurückgebracht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdxv{Könntest du nur sehen, wenn die Übeltäter vor ihrem Herrn die Köpfe hängenlassen: «Unser Herr, jetzt haben wir gesehen und gehört. Bring uns zurück, so wollen wir Gutes tun. Wir hegen nun Gewißheit»!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdxvi{Und wenn Wir gewollt hätten, hätten Wir jedem Menschen seine Rechtleitung zukommen lassen. Aber der Spruch von Mir ist zu Recht fällig geworden: «Füllen werde Ich die Hölle mit den Djinn und den Menschen allen.»\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdxvii{So kostet es dafür, daß ihr die Begegnung mit diesem eurem Tag vergessen habt. Wir haben euch (auch) vergessen. Kostet die Pein der Ewigkeit für das, was ihr zu tun pflegtet.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdxviii{Nur die glauben an unsere Zeichen, die, wenn sie damit ermahnt werden, in Anbetung niederfallen und das Lob ihres Herrn singen, und die nicht hochmütig sind.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdxix{Ihre Seiten halten sich fern von den Schlafstätten. Sie rufen ihren Herrn in Furcht und Begehren an und spenden von dem, was Wir ihnen beschert haben.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdxx{Und niemand weiß, welche Freuden für sie im Verborgenen bestimmt sind als Lohn für das, was sie zu tun pflegten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdxxi{Ist denn derjenige, der gläubig ist, wie der, der ein Frevler ist? Sie sind eben nicht gleich.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdxxii{Diejenigen, die glauben und die guten Werke tun, erhalten die Gärten der Heimstätte als Herberge für das, was sie zu tun pflegten.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdxxiii{Denjenigen aber, die freveln, dient das Feuer zur Heimstätte. Sooft sie aus ihm herauskommen wollen, werden sie zu ihm zurückgebracht, und es wird zu ihnen gesagt: «Kostet die Pein des Feuers, die ihr immer wieder für Lüge erklärt habt.»\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdxxiv{Und Wir werden sie vor der größeren Pein etwas von der diesseitigen Pein kosten lassen, auf daß sie dann umkehren.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdxxv{Und wer ist ungerechter als der, der mit den Zeichen seines Herrn ermahnt wird und sich dann von ihnen abwendet? Wir werden uns an den Übeltätern rächen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdxxvi{Und Wir haben Mose das Buch zukommen lassen. So sei nicht im Zweifel über die Begegnung mit Ihm. Und Wir haben es zu einer Rechtleitung für die Kinder Israels gemacht.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdxxvii{Und Wir haben aus ihren Reihen Vorbilder bestellt, die (sie) nach unserem Befehl leiteten, als sie sich geduldig gezeigt hatten und über unsere Zeichen Gewißheit hegten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdxxviii{Wahrlich, dein Herr wird um Tag der Auferstehung zwischen ihnen über das entscheiden, worüber sie uneins waren.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdxxix{Ist es ihnen nicht aufgegangen, wie viele Generationen, in deren Wohnungen sie nun schreiten, Wir vor ihnen haben verderben lassen? Darin sind Zeichen. Wollen sie denn nicht hören?\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdxxx{Haben sie nicht gesehen, daß Wir das Wasser zum dürren Land treiben und dadurch Getreide hervorbringen, das ihr Vieh und sie selbst verzehren? Wollen sie denn nicht einsichtig sein?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdxxxi{Und sie sagen: «Wann wird dieser Richterspruch eintreten, so ihr die Wahrheit sagt?»\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdxxxii{Sprich: Am Tag des Richterspruchs wird denen, die ungläubig waren, ihr Glaube nicht (mehr) nützen. Und ihnen wird kein Aufschub gewährt.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdxxxiii{So wende dich ab von ihnen und warte ab. Sie warten auch selbst ab.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdxxxiv{\basmalahde O Prophet, fürchte Gott und gehorche nicht den Ungläubigen und den Heuchlern. Gott weiß Bescheid und ist weise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdxxxv{Und folge dem, was dir von deinem Herrn offenbart wird. Gott hat Kenntnis von dem, was ihr tut.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdxxxvi{Und vertrau auf Gott. Gott genügt als Sachwalter.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdxxxvii{Gott hat keinem Mann zwei Herzen in seinem Inneren gemacht. Und Er hat eure Gattinnen, von denen ihr euch durch den Rückenspruch trennt, nicht (wirklich) zu euren Müttern gemacht. Und Er hat eure Adoptivsöhne nicht (wirklich) zu euren Söhnen gemacht. Das ist eure Rede aus eurem Munde. Aber Gott sagt die Wahrheit, und Er führt den (rechten) Weg.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdxxxviii{Nennt sie nach ihren Vätern. Das ist gerechter in den Augen Gottes. Wenn ihr ihre Väter nicht kennt, dann gelten sie als eure Brüder in der Religion und eure Schützlinge. Und es wird euch nicht als Vergehen angerechnet, was ihr hier gefehlt habt, sondern was eure Herzen vorsätzlich anstreben. Und Gott ist voller Vergebung und barmherzig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdxxxix{Der Prophet hat eher Anspruch auf die Gläubigen als sie selbst. Und seine Gattinnen sind ihre Mütter. Und die Blutsverwandten haben nach dem Buch Gottes eher Anspruch auf gegenseitige Freundschaft als die Gläubigen und Ausgewanderten, es sei denn, ihr möchtet euren Freunden Gutes tun. Dies ist im Buch Gottes zeilenweise niedergeschrieben.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdxl{Und als Wir von den Propheten ihre Verpflichtung entgegennahmen, und auch von dir und von Noach, Abraham, Mose und Jesus, dem Sohn Marias. Wir nahmen von ihnen eine schwere Verpflichtung entgegen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdxli{Damit Er ja die Wahrhaftigen nach ihrem wahrhaftigen Wandel frage. Und Er hat für die Ungläubigen eine schmerzhafte Pein bereitet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdxlii{O ihr, die ihr glaubt, gedenket der Gnade Gottes zu euch, als Truppen zu euch kamen. Da sandten Wir gegen sie einen Wind und auch Truppen, die ihr nicht sehen konntet. Und Gott sieht wohl, was ihr tut.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdxliii{Als sie von oben und von unten her zu euch kamen, und als die Blicke wichen und die Herzen die Kehle erreichten und ihr von Gott verschiedene Meinungen hattet.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdxliv{Dort wurden die Gläubigen geprüft, und sie wurden heftig hin und her geschüttelt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdxlv{Als die Heuchler und die, in deren Herzen Krankheit ist, sagten: «Gott und sein Gesandter haben uns nur Trügerisches versprochen.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdxlvi{Und als eine Gruppe von ihnen sagte: «O ihr Leute von Yathrib, ihr könnt euch (hier) nicht aufhalten. Kehrt zurück.» Und ein Teil von ihnen bat den Propheten um Erlaubnis; sie sagten: «Unsere Häuser sind ohne Schutz.» Dabei waren sie nicht ohne Schutz, sie wollten nur fliehen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdxlvii{Wäre man darin von ihren verschiedenen Bezirken her zu ihnen eingedrungen, und wären sie dann aufgefordert worden, der Versuchung nachzugeben, sie hätten es begangen, und sie hätten dabei nur kurze Zeit gezögert.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdxlviii{Dabei hatten sie sich vorher Gott gegenüber verpflichtet, sie würden nicht den Rücken kehren. Über die Verpflichtung Gott gegenüber wird Rechenschaft gefordert.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdxlix{Sprich: Die Flucht wird euch nicht nützen, wenn ihr davor flieht, zu sterben oder getötet zu werden. Und dann wird euch nur noch ein wenig Nutznießung gewährt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdl{Sprich: Wer ist es denn, der euch vor Gott schützen könnte, wenn Er euch Böses will oder wenn Er euch Barmherzigkeit (erweisen) will? Und sie werden für sich außer Gott weder Freund noch Helfer finden.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdli{Gott kennt wohl diejenigen von euch, die (die anderen) behindern und die zu ihren Brüdern sagen: «Kommt her zu uns.» Und sie lassen sich nur wenig auf den Kampf ein\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdlii{Und sind dabei geizig euch gegenüber. Wenn aber die Angst sich einstellt, siehst du, wie sie zu dir mit kreisenden Augen blicken wie einer, der vor dem Tod ohnmächtig wird. Ist dann die Angst vorüber, verbrühen sie euch mit scharfen Zungen aus Gier nach dem (erbeuteten) Gut. Das sind keine Gläubigen. Gott macht ihre Werke wertlos. Und dies ist Gott ein leichtes.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdliii{Sie meinen, die Parteien seien nicht weggegangen. Und sollten die Parteien (wieder) kommen, möchten sie gerne, sie wären in der Wüste unter den arabischen Beduinen und würden sich über euch erkundigen. Und wenn sie in eurer Mitte wären, würden sie nur wenig kämpfen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdliv{Ihr habt im Gesandten Gottes ein schönes Vorbild, (und zwar) für jeden, der auf Gott und den Jüngsten Tag hofft und Gottes viel gedenkt.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdlv{Und als die Gläubigen die Parteien sahen, sagten sie: «Das ist, was Gott und sein Gesandter uns versprochen haben. Gott und sein Gesandter sagen die Wahrheit.» Und es hat ihren Glauben und ihre Ergebenheit nur vermehrt.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdlvi{Unter den Gläubigen gibt es Männer, die das wahr gemacht haben, wozu sie sich Gott gegenüber verpflichtet haben. Unter ihnen sind welche, die ihre Lebensaufgabe erfüllt haben; und unter ihnen sind welche, die noch warten müssen. Und sie haben keine Änderung vorgenommen;\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdlvii{Auf daß Gott die Wahrhaftigen für ihre Wahrhaftigkeit entlohne und die Heuchler peinige, wenn Er will, oder sich ihnen (gnädig) zuwende. Gott ist voller Vergebung und barmherzig.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdlviii{Und Gott wies diejenigen, die ungläubig sind, mit ihrem Groll zurück, ohne daß sie etwas Gutes erlangt hätten. Und Gott befreite die Gläubigen vom Kampf. Gott ist stark und mächtig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdlix{Und Er ließ diejenigen von den Leuten des Buches, die ihnen Beistand geleistet hatten, aus ihren Burgen heruntersteigen. Und Er jagte ihren Herzen Schrecken ein, so daß ihr einen Teil (von ihnen) getötet und einen Teil gefangengenommen habt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdlx{Und Er gab euch zum Erbe ihr Land, ihre Wohnstätten und ihren Besitz, und auch ein Land, das ihr (vorher) nicht betreten hattet. Und Gott hat Macht zu allen Dingen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdlxi{O Prophet, sprich zu deinen Gattinnen: Wenn ihr das diesseitige Leben und seinen Schmuck begehrt, dann kommt her, ich werde euch eine Versorgung sichern und euch auf eine schöne Weise freigeben.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdlxii{Und wenn ihr Gott und seinen Gesandten und die jenseitige Wohnstätte begehrt, so hat Gott für die Rechtschaffenen von euch einen großartigen Lohn bereitet.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdlxiii{O ihr Frauen des Propheten, wenn eine von euch eine eindeutige Schandtat begeht, wird ihr die Pein verdoppelt. Und das ist Gott ein leichtes.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdlxiv{Und wenn eine von euch Gott und seinem Gesandten ergeben ist und Gutes tut, lassen Wir ihr ihren Lohn zweimal zukommen. Und Wir haben für sie einen trefflichen Unterhalt bereitet.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdlxv{O ihr Frauen des Propheten, ihr seid nicht wie irgendeine von den Frauen. Wenn ihr gottesfürchtig seid, dann seid nicht unterwürfig im Reden, damit nicht derjenige, in dessen Herzen Krankheit ist, sich Hoffnungen macht. Und sprecht geziemende Worte.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdlxvi{Haltet euch in euren Häusern auf. Und stellt nicht euren Schmuck zur Schau wie in der Zeit der früheren Unwissenheit. Verrichtet das Gebet und entrichtet die Abgabe und gehorcht Gott und seinem Gesandten. Gott will die Unreinheit von euch entfernen, ihr Leute des Hauses, und euch völlig rein machen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdlxvii{Und gedenkt dessen, was von den Zeichen Gottes und von der Weisheit in euren Häusern verlesen wird. Gott ist feinfühlig und hat Kenntnis von allem.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdlxviii{Für die muslimischen Männer und Frauen, Männer und Frauen, die gläubig, ergeben, wahrhaftig, geduldig, demütig sind, die Almosen geben, fasten, ihre Scham bewahren und Gottes viel gedenken - für sie hat Gott Vergebung und einen großartigen Lohn bereitet.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdlxix{Ein Gläubiger oder eine Gläubige darf, wenn Gott und sein Gesandter eine Angelegenheit entschieden haben, nicht die Möglichkeit haben, in ihrer Angelegenheit frei zu wählen. Und wer gegen Gott und seinen Gesandten ungehorsam ist, der befindet sich in einem offenkundigen Irrtum.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdlxx{Und als du zu dem, dem Gott Gnade erwiesen hatte und dem auch du Gnade erwiesen hattest, sagtest: «Behalte deine Gattin für dich und fürchte Gott», und in deinem Inneren geheimhieltest, was (Gott doch offenlegt, und die Menschen fürchtetest, während Gott eher darauf Anspruch hat, daß du Ihn fürchtest. Als dann Zayd seinen Wunsch an ihr erfüllt hatte, gaben Wir sie dir zur Gattin, damit für die Gläubigen kein Grund zur Bedrängnis bestehe in bezug auf die Gattinnen ihrer Adoptivsöhne, wenn diese ihren Wunsch an ihnen erfüllt haben. Und der Befehl Gottes wird ausgeführt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdlxxi{Es besteht für den Propheten kein Grund zur Bedrängnis in dem, was Gott für ihn festgelegt hat. So war das beispielhafte Verhalten Gottes mit denen, die vorher dahingegangen sind - und der Befehl Gottes ist ein fest gefaßter Beschluß -,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdlxxii{Die die Botschaft Gottes ausrichten, Ihn fürchten und niemanden fürchten außer Gott. Und Gott genügt als der, der abrechnet.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdlxxiii{Muhammad ist nicht der Vater irgendeines von euren Männern, sondern der Gesandte Gottes und das Siegel der Propheten. Und Gott weiß über alle Dinge Bescheid.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdlxxiv{O ihr, die ihr glaubt, gedenket Gottes in häufigem Gedenken\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdlxxv{Und preiset Ihn morgens und abends.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdlxxvi{Er ist es, der über euch den Segen spricht - und auch seine Engel -, damit Er euch aus den Finsternissen ins Licht hinausführt. Und Er ist barmherzig zu den Gläubigen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdlxxvii{Ihre Begrüßung am Tag, da sie Ihm begegnen, wird sein: «Friede!» Und Er hat für sie einen trefflichen Lohn bereitet.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdlxxviii{O Prophet, Wir haben dich gesandt als Zeugen, als Freudenboten und als Warner,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdlxxix{Und als einen, der zu Gott mit seiner Erlaubnis ruft, und als eine helle Leuchte.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdlxxx{Und verkünde den Gläubigen, daß für sie von Gott eine große Huld bestimmt ist.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdlxxxi{Und gehorche nicht den Ungläubigen und den Heuchlern. Kümmere dich nicht um das Leid, das sie dir zufügen, und vertrau auf Gott. Und Gott genügt als Sachwalter.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdlxxxii{O ihr, die ihr glaubt, wenn ihr gläubige Frauen heiratet und sie hierauf entlaßt, bevor ihr sie berührt habt, dann dürft ihr ihnen nicht eine von euch berechnete Wartezeit auferlegen. Sichert ihnen eine Versorgung und laßt sie auf eine schöne Weise frei.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdlxxxiii{O Prophet, Wir haben dir für erlaubt erklärt zu heiraten: deine Gattinnen, denen du ihren Lohn hast zukommen lassen; das, was deine rechte Hand besitzt von dem, was Gott dir als Beute zugeteilt hat; die Töchter deines Onkels und die Töchter deiner Tanten väterlicherseits, die Töchter deines Onkels und die Töchter deiner Tanten mütterlicherseits, welche mit dir ausgewandert sind; auch jede gläubige Frau, wenn sie sich dem Propheten (ohne Gegenforderung) schenkt und falls der Prophet sie heiraten will: Dies ist dir vorbehalten im Unterschied von den Gläubigen - Wir wissen wohl, was Wir ihnen in bezug auf ihre Gattinnen und auf das, was ihre rechte Hand besitzt, verpflichtend gemacht haben -, damit für dich kein Grund zur Bedrängnis bestehe. Und Gott ist voller Vergebung und barmherzig.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdlxxxiv{Du darfst zurückstellen, wen von ihnen du willst, und du darrst bei dir aufnehmen, wen du willst. Und wenn du doch eine von denen haben möchtest, die du abgewiesen hast, dann ist das für dich kein Vergehen. Das bewirkt eher, daß sie frohen Mutes, nicht betrübt und alle mit dem zufrieden sind, was du ihnen zukommen läßt. Gott weiß, was in euren Herzen ist. Gott weiß Bescheid und ist langmütig.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdlxxxv{Es ist dir danach nicht mehr erlaubt, Frauen zu heiraten und sie gegen (andere) Gattinnen einzutauschen, auch wenn ihre Schönheit dir gefallen sollte, mit Ausnahme dessen, was deine rechte Hand besitzt. Und Gott ist Wächter über alle Dinge.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdlxxxvi{O ihr, die ihr glaubt, tretet nicht in die Häuser des Propheten ein - es sei denn, das wird euch erlaubt - zur Teilnahme an einem Essen, ohne auf die Essenszeit zu warten. Wenn ihr dann hereingerufen werdet, dann tretet ein, und, wenn ihr gegessen habt, dann geht auseinander, und (dies) ohne euch einer Unterhaltung hinzugeben. Damit fügt ihr dem Propheten Leid zu, aber er schämt sich vor euch. Gott aber schämt sich nicht vor der Wahrheit. Und wenn ihr sie um einen Gegenstand bittet, so bittet sie von hinter einem Vorhang. Das ist reiner für eure Herzen und ihre Herzen. Und es steht euch nicht zu, dem Gesandten Gottes Leid zuzufügen, und auch nicht jemals seine Gattinnen nach ihm zu heiraten. Das wäre bei Gott etwas Ungeheuerliches.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdlxxxvii{Ob ihr etwas offenlegt oder geheimhaltet, Gott weiß über alle Dinge Bescheid.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdlxxxviii{Es besteht für sie kein Vergehen in bezug auf ihre Väter, ihre Söhne, ihre Brüder, die Söhne ihrer Brüder, die Söhne ihrer Schwestern, ihre Frauen und die, die ihre rechte Hand besitzt. Und fürchtet Gott. Gott ist Zeuge über alle Dinge.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmdlxxxix{Gott und seine Engel sprechen den Segen über den Propheten. O ihr, die ihr glaubt, sprecht den Segen über ihn und grüßt ihn mit gehörigem Gruß.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmdxc{Diejenigen, die Gott und seinem Gesandten Leid zufügen, verflucht Gott im Diesseits und Jenseits, und für sie hat Er eine schmähliche Pein bereitet.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmdxci{Und diejenigen, die den gläubigen Männern und Frauen Leid zufügen für etwas, was sie nicht begangen haben, laden auf sich eine Verleumdung und eine offenkundige Schuld.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmdxcii{O Prophet, sag deinen Gattinnen und deinen Töchtern und den Frauen der Gläubigen, sie sollen etwas von ihrem Überwurf über sich herunterziehen. Das bewirkt eher, daß sie erkannt werden und daß sie nicht belästigt werden. Und Gott ist voller Vergebung und barmherzig.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmdxciii{Wenn die Heuchler und diejenigen, in deren Herzen Krankheit ist, und diejenigen, die beunruhigende Falschmeldungen in der Stadt verbreiten, (damit) nicht aufhören, werden Wir dich bestimmt gegen sie antreiben. Dann werden sie nur noch kurze Zeit in deiner Nachbarschaft darin wohnen.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmdxciv{Verflucht sind sie. Wo immer man sie trifft, wird man sie ergreifen und unerbittlich töten.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmdxcv{So war das beispielhafte Verfahren Gottes mit denen, die vorher dahingegangen sind. Und du wirst im Verfahren Gottes keine Veränderung finden.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmdxcvi{Die Menschen fragen dich nach der Stunde. Sprich: Nur Gott weiß über sie Bescheid. Woher willst du es wissen? Vielleicht steht die Stunde nahe bevor.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmdxcvii{Gott hat die Ungläubigen verflucht, und Er hat für sie einen Feuerbrand bereitet;\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmdxcviii{Darin werden sie auf immer ewig weilen; und sie werden weder Freund noch Helfer finden.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmdxcix{Am Tag, da ihre Gesichter im Feuer gewendet werden, sagen sie: «O hätten wir doch Gott gehorcht und hätten wir doch dem Gesandten gehorcht!»\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmdc{Und sie sagen: «Unser Herr, wir haben unseren Herrschern und den Großen unter uns gehorcht, da haben sie uns vom Weg abirren lassen.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmdci{Unser Herr, laß ihnen ein Doppelmaß an Pein zukommen, und verfluche sie mit einem großen Fluch.»\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmdcii{O ihr, die ihr glaubt, seid nicht wie diejenigen, die Mose Leid zugefügt haben, worauf Gott ihn freisprach von dem, was sie (über ihn) sagten. Er war bei Gott angesehen.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmdciii{O ihr, die ihr glaubt, fürchtet Gott und sprecht zutreffende Worte,\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmdciv{Dann läßt Er eure Werke als gut gelten und vergibt euch eure Sünden. Und wer Gott und seinem Gesandten gehorcht, der erringt einen großartigen Erfolg.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmdcv{Wir haben das Vertrauenspfand den Himmeln und der Erde und den Bergen angeboten, sie aber weigerten sich, es zu tragen, sie waren erschrocken davor. Der Mensch trug es - er tut wirklich Unrecht und ist sehr töricht.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmdcvi{So wird Gott nun die Heuchler, Männer und Frauen, und die Polytheisten, Männer und Frauen, peinigen, und so wird Gott sich den Gläubigen, Männern und Frauen, zuwenden. Gott ist voller Vergebung und barmherzig.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmdcvii{\basmalahde Lob sei Gott, dem gehört, was in den Himmeln und was auf der Erde ist, und Lob sei Ihm im Jenseits! Er ist der, der weise ist und Kenntnis von allem hat.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdcviii{Er weiß, was in die Erde eingeht und was aus ihr herauskommt, was vom Himmel herabkommt und was darin emporsteigt. Und Er ist der, der barmherzig und voller Vergebung ist.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdcix{Diejenigen, die ungläubig sind, sagen: «Die Stunde kommt nicht über uns.» Sprich: Doch, sie kommt bestimmt über euch, bei meinem Herrn, der das Unsichtbare weiß! Es entgeht Ihm nicht das Gewicht eines Stäubchens, weder in den Himmeln noch auf der Erde. Und es gibt nichts, was kleiner ist als dies oder größer, das nicht in einem deutlichen Buch stünde,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdcx{Damit Er diejenigen, die glauben und die guten Werke tun, entlohne. Bestimmt sind für diese Vergebung und ein trefflicher Unterhalt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdcxi{Für diejenigen, die eifrig gegen unsere Zeichen vorgehen und ihnen Machtlosigkeit nachweisen wollen, für die ist eine Pein durch ein schmerzhaftes Zorngericht bestimmt.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdcxii{Und diejenigen, denen das Wissen zugekommen ist, sehen, daß das, was zu dir von deinem Herrn herabgesandt worden ist, die Wahrheit ist und zum Weg dessen führt, der mächtig und des Lobes würdig ist.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdcxiii{Und diejenigen, die ungläubig sind, sagen: «Sollen wir euch auf einen Mann hinweisen, der euch kundtut, daß ihr, wenn ihr in Stücke gerissen worden seid, doch neu erschaffen werdet?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdcxiv{Hat er gegen Gott eine Lüge erdichtet, oder leidet er an Besessenheit?» Aber nein, diejenigen, die an das Jenseits nicht glauben, stecken in der Pein und befinden sich im tiefen Irrtum.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdcxv{Haben sie denn nicht auf das geschaut, was vom Himmel und von der Erde vor ihnen und was hinter ihnen ist? Wenn Wir wollen, lassen Wir die Erde mit ihnen versinken oder Stücke vom Himmel auf sie herabfallen. Darin ist ein Zeichen für jeden Diener, der sich (Gott) reumütig zuwendet.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdcxvi{Und Wir ließen David eine Huld von Uns zukommen. - «Ihr Berge, singt Kehrverse mit ihm, und auch ihr Vögel.» Und Wir machten für ihn das Eisen geschmeidig.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdcxvii{- «Fertige Panzergewänder an und webe im richtigen Maß die Panzermaschen aneinander. Und tut Gutes. Ich sehe wohl, was ihr tut.»\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdcxviii{Und dem Salomo (machten Wir) den Wind (dienstbar), dessen Morgenlauf einen Monat und dessen Abendlauf einen Monat dauert. Und Wir ließen die Quelle des geschmolzenen Erzes für ihn fließen. Und unter den Djinn gab es welche, die mit der Erlaubnis seines Herrn vor ihm tätig waren. Wer von ihnen von unserem Befehl abweicht, den lassen Wir etwas von der Pein des Feuerbrandes kosten.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdcxix{Sie machten ihm, was er wollte an Heiligtümern, Bildwerken, Schüsseln wie Trögen und feststehenden Kesseln. - «Verrichtet, ihr Sippe Davids, eure Arbeit in Dankbarkeit.» Ja, nur wenige von meinen Dienern sind dankbar.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdcxx{Und als Wir für ihn den Tod bestimmt hatten, hat sie auf sein Ableben nur das Kriechtier der Erde hingewiesen, das seinen Stab fraß. Als er nun zu Boden fiel, wurde den Djinn deutlich, daß sie, wenn sie das Unsichtbare gewußt hätten, nicht (weiter) in der schmählichen Pein geblieben wären.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdcxxi{Die Saba hatten in ihrem Wohngebiet ein Zeichen: zwei Gärten zur Rechten und zur Linken. - «Eßt von dem, was euer Herr beschert hat, und danket Ihm. Eine gute Ortschaft (ist es), und ein Herr voller Vergebung.»\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdcxxii{Aber sie wandten sich ab. Da sandten Wir gegen sie die Flut des Staudammes, und Wir vertauschten ihnen ihre zwei Gärten gegen zwei Gärten mit bitterem Ernteertrag und Tamarisken und einigen wenigen Zizyphusbäumen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdcxxiii{Das haben Wir ihnen vergolten dafür, daß sie undankbar waren. Vergelten Wir denn sonst jemandem anderem als dem Undankbaren?\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdcxxiv{Und Wir legten zwischen ihnen und den Städten, die Wir gesegnet haben, sichtbare Städte an. Und Wir legten das Maß der Reise zwischen ihnen fest: «Zieht in ihnen Nächte und Tage in Sicherheit umher.»\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdcxxv{Sie aber sagten: «Unser Herr, lege größere Entfernungen zwischen unsere Reisen.» Sie taten sich selbst Unrecht. So machten Wir sie zum Gegenstand von Geschichten. Und Wir zerrissen sie in Stücke. Darin sind Zeichen für jeden, der sehr geduldig und sehr dankbar ist.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdcxxvi{Wahrlich, Iblis fand seine Meinung von ihnen bestätigt. Sie folgten ihm, mit Ausnahme eines Teils der Gläubigen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdcxxvii{Und er hatte keine Macht über sie. Es geschah nur, damit Wir feststellen, wer an das Jenseits glaubt, und (ihn unterscheiden) von dem, der darüber Zweifel hegt. Dein Herr ist Hüter aller Dinge.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdcxxviii{Sprich: Ruft die an, die ihr anstelle Gottes angebt. Sie verfügen nicht einmal über das Gewicht eines Stäubchens weder in den Himmeln noch auf der Erde. Und sie haben an ihnen beiden keinen Anteil. Und Er hat unter ihnen keinen, der Ihm Beistand leisten könnte.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdcxxix{Auch nützt bei Ihm die Fürsprache nicht, außer wenn Er es jemandem erlaubt. Wenn dann die Angst von ihren Herzen entfernt worden ist, sagen sie: «Was hat euer Herr gesagt?» Sie sagen: «Die Wahrheit.» Und Er ist der Erhabene, der Große.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdcxxx{Sprich: Wer versorgt euch von den Himmeln und der Erde? Sprich: Gott. Entweder wir folgen einer Rechtleitung oder ihr, oder wir befinden uns in einem offenkundigen Irrtum oder ihr.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdcxxxi{Sprich: Ihr habt nicht zu verantworten, was wir verübt haben, und wir haben nicht zu verantworten, was ihr tut.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdcxxxii{Sprich: Zusammenbringen wird uns unser Herr. Dann richtet Er zwischen uns nach der Wahrheit. Er ist der, der wahrhaft richtet und Bescheid weiß.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdcxxxiii{Sprich: Laßt mich die sehen, die ihr Ihm als Teilhaber beigegeben habt. Nein, Er ist Gott, der Mächtige, der Weise.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdcxxxiv{Und Wir haben dich für die Menschen allesamt nur als Freudenboten und Warner gesandt. Aber die meisten Menschen wissen nicht Bescheid.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdcxxxv{Und sie sagen: «Wann wird diese Androhung eintreten, so ihr die Wahrheit sagt?»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdcxxxvi{Sprich: Euch ist eine Verabredung auf einen Tag festgelegt, hinter dem ihr nicht einmal eine Stunde zurückbleiben noch ihm vorausgehen könnt.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdcxxxvii{Und diejenigen, die ungläubig sind, sagen: «Nie werden wir an diesen Koran glauben, und auch nicht an das, was vor ihm vorhanden war.» Könntest du nur sehen, wenn die, die Unrecht tun, vor ihren Herrn gestellt werden und untereinander die Worte wechseln! Diejenigen, die wie Schwache behandelt wurden, sagen zu denen, die sich hochmütig verhielten: «Wenn ihr nicht gewesen wäret, so wären wir gläubig gewesen.»\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdcxxxviii{Diejenigen, die sich hochmütig verhielten, sagen zu denen, die wie Schwache behandelt wurden: «Sind wir es gewesen, die euch von der Rechtleitung abgewiesen haben, nachdem sie zu euch gekommen war? Nein, ihr wart Übeltäter.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdcxxxix{Diejenigen, die wie Schwache behandelt wurden, sagen zu denen, die sich hochmütig verhielten: «Es waren eher eure Ränke bei Nacht und bei Tag, da ihr uns befohlen habt, Gott zu verleugnen und Ihm andere als Gegenpart zur Seite zu stellen.» Und sie hegen insgeheim Reue, wenn sie die Pein sehen. Und Wir bringen die Fesseln an den Hals derer an, die ungläubig waren. Wird ihnen denn etwas anderes vergolten als das, was sie zu tun pflegten?\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdcxl{Und Wir haben in keine Stadt einen Warner gesandt, ohne daß die, die in ihr üppig lebten, gesagt hätten: «Das, womit ihr gesandt seid, das verleugnen wir.»\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdcxli{Und sie sagen: «Wir haben mehr Vermögen und Kinder. Und wir werden gewiß nicht gepeinigt werden.»\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdcxlii{Sprich: Mein Herr teilt den Lebensunterhalt großzügig, wem Er will, und auch bemessen zu. Aber die meisten Menschen wissen nicht Bescheid.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdcxliii{Es ist nicht euer Vermögen, und es sind auch nicht eure Kinder, die euch Zutritt in unsere Nähe verschaffen, mit Ausnahme derer, die glauben und Gutes tun. Diese erhalten einen doppelten Lohn für das, was sie getan haben, und sie werden in den Obergemächern in Sicherheit sein.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdcxliv{Diejenigen aber, die eifrig gegen unsere Zeichen vorgehen und ihnen Machtlosigkeit nachweisen wollen, werden zur Pein vorgeführt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdcxlv{Sprich: Mein Herr teilt den Lebensunterhalt großzügig, wem von seinen Dienern Er will, und auch bemessen zu. Und was ihr auch immer spendet, Er wird es euch ersetzen. Er ist der beste der Versorger.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdcxlvi{Und am Tag, da Er sie alle versammelt. Dann spricht Er zu den Engeln: «Diese da, pflegten sie euch zu verehren?»\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdcxlvii{Sie sagen: «Preis sei Dir! Du bist unser Freund, nicht sie. Nein, sie verehrten die Djinn. Die meisten von ihnen glauben an sie.»\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdcxlviii{«Heute könnt ihr einander weder Nutzen noch Schaden bringen.» Und Wir sprechen zu denen, die Unrecht getan haben: «Kostet die Pein des Feuers, das ihr immer wieder für Lüge erklärt habt.»\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdcxlix{Und wenn ihnen unsere Zeichen als deutliche Beweise verlesen werden, sagen sie: «Dieser ist nur ein Mann, der euch von dem abbringen will, was eure Väter verehrten.» Und sie sagen: «Das ist nichts als eine erdichtete Lüge.» Und diejenigen, die ungläubig sind, sagen von der Wahrheit, als sie zu ihnen kam: «Das ist nichts als eine offenkundige Zauberei.»\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdcl{Und Wir haben ihnen keine Bücher zukommen lassen, die sie hätten erforschen können. Und Wir haben vor dir keinen Warner zu ihnen gesandt.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdcli{Für Lüge haben es (auch) die erklärt, die vor ihnen lebten, wobei diese nicht einmal das Zehntel dessen erreicht haben, was Wir jenen zukommen ließen. Sie haben meine Gesandten der Lüge geziehen. Wie war aber dann meine Mißbilligung!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdclii{Sprich: Ich ermahne euch, eines (zu beachten): daß ihr euch zu zweit und einzeln vor Gott hinstellt und dann nachdenkt. Euer Gefährte leidet nicht an Besessenheit. Er ist euch nur ein Warner vor dem Eintreten einer harten Pein.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdcliii{Was ich auch immer an Lohn hätte verlangen können, das gehört euch. Mein Lohn obliegt Gott allein. Und er ist Zeuge über alle Dinge.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdcliv{Sprich: Mein Herr läßt die Wahrheit überbringen, Er, der die unsichtbaren Dinge alle weiß.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdclv{Sprich: Die Wahrheit ist gekommen. Das Falsche kann weder etwas neu erschaffen noch es wiederholen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdclvi{Sprich: Wenn ich irregehe, gehe ich irre zu meinem eigenen Schaden. Und wenn ich der Rechtleitung folge, so ist es durch das, was mir mein Herr offenbart. Er hört alles und ist nahe.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdclvii{Könntest du nur sehen, wenn sie erschrecken! Da gibt es kein Entrinnen, und sie werden aus einem nahen Ort weggerafft.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdclviii{Und sie sagen: «Wir glauben daran.» Aber wie sollte es ihnen gelingen, ihn aus einem fernen Ort zu erlangen,\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdclix{Wo sie ihn doch vorher verleugnet haben? Und sie raten in bezug auf das Unsichtbare herum aus einem fernen Ort.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdclx{Und es wird eine Trennung vorgenommen zwischen ihnen und dem, was sie begehren, so wie es vorher mit ihresgleichen gemacht wurde. Sie waren ja in einem starken Zweifel.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdclxi{\basmalahde Lob sei Gott, dem Schöpfer der Himmel und der Erde, der die Engel zu Boten gemacht hat mit Flügeln, je zwei, drei und vier! Er fügt der Schöpfung hinzu, was Er will. Gott hat Macht zu allen Dingen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdclxii{Was Gott den Menschen an Barmherzigkeit eröffnet, kann niemand zurückhalten. Und was Er zurückhält, kann niemand nach Ihm zur Verfügung stellen. Und Er ist der Mächtige, der Weise.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdclxiii{O ihr Menschen, gedenket der Gnade Gottes zu euch. Gibt es denn einen anderen Schöpfer als Gott, der euch vom Himmel und von der Erde versorgt? Es gibt keinen Gott außer Ihm. Wie leicht laßt ihr euch doch abwenden!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdclxiv{Wenn sie dich der Lüge zeihen, so wurden vor dir Gesandte der Lüge geziehen. Und zu Gott werden die Angelegenheiten zurückgebracht.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdclxv{O ihr Menschen, das Versprechen Gottes ist wahr. So soll euch das diesseitige Leben nicht betören. Und nicht betören soll euch in bezug auf Gott der Betörer.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdclxvi{Der Satan ist euch ein Feind. So nehmt auch ihr ihn euch zum Feind. Er ruft ja seine Anhänger dazu, zu den Gefährten des Feuerbrandes zu gehören.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdclxvii{Bestimmt ist für diejenigen, die ungläubig sind, eine harte Pein, und für diejenigen, die glauben und die guten Werke tun, Vergebung und ein großer Lohn.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdclxviii{Soll der, dem sein böses Tun verlockend gemacht worden ist und der es dann auch schön findet, (rechtgeleitet sein)? Gott führt irre, wen Er will, und Er leitet recht, wen Er will. So soll deine Seele nicht über sie in Bedauern zergehen. Gott weiß, was sie machen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdclxix{Und Gott ist es, der die Winde schickt, und sie wühlen die Wolken auf. Dann treiben Wir sie zu einem abgestorbenen Land und beleben damit die Erde nach ihrem Absterben wieder. So ist es auch mit der Auferstehung.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdclxx{Wenn einer die Macht sucht, so gehört alle Macht Gott. Zu Ihm steigt das gefällige Wort, und die gute Tat läßt Er (zu sich) aufsteigen. Und für die, die böse Ränke schmieden, ist eine harte Pein bestimmt. Und ihre Ränke werden dem Verderben anheimfallen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdclxxi{Und Gott hat euch aus Erde, dann aus einem Tropfen erschaffen und euch dann zu Paaren gemacht. Kein Weib wird schwanger oder kommt nieder, es sei denn mit seinem Wissen. Und keinem, der alt wird, wird das Altwerden oder eine Verkürzung seiner Lebenszeit zuteil, ohne daß es in einem Buch stünde. Dies ist Gott ein leichtes.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdclxxii{Die zwei Meere sind nicht gleich. Das eine ist süß, erfrischend und bekömmlich zu trinken; das andere ist salzig und bitter. Und aus beiden eßt ihr frisches Fleisch und holt Schmuck heraus, den ihr anlegt. Und du siehst die Schiffe es durchspalten, damit ihr nach etwas von seiner Huld strebt und auf daß ihr dankbar werdet.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdclxxiii{Er läßt die Nacht in den Tag übergehen, und den Tag in die Nacht. Und Er hat die Sonne und den Mond dienstbar gemacht, so daß jedes seinen Lauf auf eine bestimmte Frist nimmt. So ist Gott, euer Herr. Ihm gehört die Königsherrschaft. Die aber, die ihr an seiner Stelle anruft, verfügen nicht einmal über das Häutchen eines Dattelkernes.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdclxxiv{Wenn ihr sie anruft, hören sie euer Rufen nicht. Und würden sie (es) doch hören, sie würden euch nicht erhören. Und am Tag der Auferstehung verleugnen sie eure Beigesellung. Und niemand kann dir Kunde geben wie Einer, der Kenntnis von allem hat.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdclxxv{O ihr Menschen, ihr seid es, die Gottes bedürftig sind. Und Gott ist der, der auf niemanden angewiesen und des Lobes würdig ist.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdclxxvi{Wenn Er will, läßt Er euch fortgehen und eine neue Schöpfung kommen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdclxxvii{Und dies fällt Gott nicht schwer.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdclxxviii{Und keine lasttragende Seele trägt die Last einer anderen. Und wenn eine Schwerbeladene (zum Mittragen) ihrer Last aufruft, wird nichts davon (für sie) getragen, und ginge es auch dabei um einen Verwandten. Du kannst nur die warnen, die ihren Herrn im Verborgenen fürchten und das Gebet verrichten. Und wer sich läutert, läutert sich zu seinem eigenen Vorteil. Und zu Gott führt der Lebensweg.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdclxxix{Nicht gleich sind der Blinde und der Sehende,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdclxxx{Noch die Finsternisse und das Licht,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdclxxxi{Noch der Schatten und die Sommerhitze.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdclxxxii{Und nicht gleich sind auch die Lebenden und die Toten. Gott läßt hören, wen Er will. Du bist es nicht, der die hören läßt, die in den Gräbern sind.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdclxxxiii{Du bist nur ein Warner.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdclxxxiv{Wir haben dich mit der Wahrheit gesandt als Freudenboten und als Warner. Und es gibt keine Gemeinschaft, bei der nicht früher ein Warner aufgetreten wäre.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdclxxxv{Und wenn sie dich der Lüge zeihen, so haben auch diejenigen, die vor ihnen lebten, (die Gesandten) der Lüge geziehen. Ihre Gesandten kamen zu ihnen mit den deutlichen Zeichen und mit den Schriften und dem erleuchtenden Buch.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdclxxxvi{Dann ergriff Ich die, die ungläubig waren. Und wie war meine Mißbilligung!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdclxxxvii{Hast du nicht gesehen, daß Gott vom Himmel Wasser herabkommen läßt? Dann bringen Wir dadurch Früchte von verschiedenen Arten hervor. Und bei den Bergen gibt es Züge von verschiedenen Farben, weiße und rote und rabenschwarze.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdclxxxviii{Und bei den Menschen und den Tieren und dem Vieh gibt es Arten von verschiedener Farbe. So ist es. Wahrlich, Gott fürchten unter seinen Dienern eben die Gelehrten. Gott ist mächtig und voller Vergebung.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdclxxxix{Diejenigen, die das Buch Gottes verlesen, das Gebet verrichten und von dem, was Wir ihnen beschert haben, geheim und offen spenden, hoffen auf einen Handel, der niemals dem Verderben anheimfallen wird,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdcxc{Damit Er ihnen ihren Lohn voll erstatte und ihnen von seiner Huld noch mehr gebe. Er ist voller Vergebung und zeigt sich erkenntlich.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdcxci{Und was Wir dir vom Buch offenbart haben, ist die Wahrheit als Bestätigung dessen, was vor ihm vorhanden war. Gott hat Kenntnis von seinen Dienern, und Er sieht sie wohl.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdcxcii{Dann haben Wir das Buch denen von unseren Dienern, die Wir auserwählt haben, zum Erbe gegeben. Manch einer von ihnen tut sich selbst Unrecht, manch anderer von ihnen zeigt einen maßvollen Wandel, und manch anderer von ihnen ist (sogar) im Wettlauf nach den guten Dingen (den anderen) voraus mit der Erlaubnis Gottes. Das ist die große Huld:\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdcxciii{Die Gärten von Eden, in die sie eingehen. Geschmückt werden sie mit Armringen aus Gold und mit Perlen, und ihre Kleidung darin ist aus Seide.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdcxciv{Und sie sagen: «Lob sei Gott, der die Betrübtheit von uns weggenommen hat! Unser Herr ist wahrlich voller Vergebung und zeigt sich erkenntlich.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdcxcv{Er, der uns durch seine Huld in die Wohnstätte des (ewigen) Aufenthaltes versetzt hat, in der uns keine Ermüdung befällt, und in der uns keine Ermattung befällt.»\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdcxcvi{Für diejenigen, die ungläubig sind, ist das Feuer der Hölle bestimmt. Darin wird mit ihnen kein Ende gemacht, so daß sie sterben, und es wird ihnen auch seine Pein nicht erleichtert. So vergelten Wir jedem, der sehr ungläubig ist.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdcxcvii{Und sie schreien darin: «Unser Herr, laß uns herauskommen, so werden wir Gutes tun und nicht das, was wir zu tun pflegten.» - «Haben Wir euch nicht so alt werden lassen, daß jeder, der es hätte bedenken wollen, es hätte bedenken können? Und ist nicht der Warner zu euch gekommen? So kostet (es) nun. Die, die Unrecht tun, haben keinen Helfer.»\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdcxcviii{Gott ist der, der das Unsichtbare der Himmel und der Erde weiß. Er weiß über das innere Geheimnis Bescheid.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdcxcix{Er ist es, der euch zu aufeinanderfolgenden Generationen auf der Erde gemacht hat. Wer nun ungläubig ist, dessen Unglaube lastet auf ihm. Und den Ungläubigen bringt ihr Unglaube bei ihrem Herrn nur noch mehr Abscheu. Und den Ungläubigen bringt ihr Unglaube nur noch mehr Verlust.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdcc{Sprich: Habt ihr eure Teilhaber gesehen, die ihr anstelle Gottes anruft? Zeigt mir doch, was sie von der Erde erschaffen haben. Oder haben sie etwa einen Anteil an den Himmeln? Oder haben Wir ihnen (sonst) ein Buch zukommen lassen, so daß sie von daher einen deutlichen Beweis hätten? Nein, die, die Unrecht tun, versprechen einander nur Betörung.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdcci{Gott hält die Himmel und die Erde, daß sie nicht vergehen. Würden sie vergehen, so könnte niemand nach Ihm sie festhalten. Er ist langmütig und voller Vergebung.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdccii{Und sie haben bei Gott ihren eifrigsten Eid geschworen, sie würden, wenn ein Warner zu ihnen käme, treuer der Rechtleitung folgen als irgendeine von den Gemeinschaften. Als dann aber ein Warner zu ihnen kam, mehrte es in ihnen nur noch die Abneigung,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdcciii{Da sie sich auf der Erde hochmütig verhielten und böse Ränke schmiedeten. Aber die bösen Ränke umschließen nur ihre Urheber. Erwarten sie denn (für sich) etwas anderes als die Art, wie mit den Früheren verfahren wurde? Du wirst bei dem Verfahren Gottes keine Veränderung finden, und du wirst bei dem Verfahren Gottes keine Verwandlung finden.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdcciv{Sind sie denn nicht auf der Erde umhergezogen und haben geschaut, wie das Ende derer war, die vor ihnen lebten? Sie hatten eine stärkere Kraft als sie. Und Gottes Wille wird durch nichts vereitelt weder in den Himmeln noch auf der Erde. Er weiß Bescheid und ist mächtig.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdccv{Und wenn Gott die Menschen für das, was sie erworben haben, belangen wollte, würde Er auf ihrer Oberfläche kein Tier übriglassen. Aber Er stellt sie zu einer bestimmten Frist zurück. Und wenn ihre Frist kommt - wahrlich, Gott sieht wohl seine Diener.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdccvi{\basmalahde Yaa Sien.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdccvii{Beim weisen Koran,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdccviii{Du bist einer der Gesandten\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdccix{Auf einem geraden Weg.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdccx{Er ist die Herabsendung des Mächtigen, des Barmherzigen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdccxi{Damit du Leute warnst, deren Väter nicht gewarnt wurden, so daß sie (alles) unbeachtet lassen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdccxii{Der Spruch ist über die meisten von ihnen fällig geworden, so glauben sie nicht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdccxiii{Wir haben an ihren Hals Fesseln angebracht, die bis zum Kinn reichen, so daß sie den Kopf hochhalten müssen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdccxiv{Und Wir haben vor ihnen eine Sperrmauer und hinter ihnen eine Sperrmauer gemacht und sie umhüllt, so daß sie nicht sehen können.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdccxv{Und es ist ihnen gleich, ob du sie warnst oder ob du sie nicht warnst; sie glauben nicht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdccxvi{Du kannst nur den warnen, der der Ermahnung folgt und den Erbarmer im Verborgenen fürchtet. So verkünde ihm Vergebung und trefflichen Lohn.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdccxvii{Wir, ja Wir machen die Toten wieder lebendig. Und Wir schreiben auf, was sie vorausgeschickt haben und auch ihre Spuren. Alle Dinge haben Wir in einer deutlichen Anzeigeschrift erfaßt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdccxviii{Und führe ihnen als Gleichnis die Leute der Stadt an, als die Gesandten zu ihr kamen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdccxix{Als Wir zwei zu ihnen sandten, da ziehen sie beide der Lüge. Wir brachten durch einen dritten Verstärkung. Sie sagten: «Wir sind zu euch gesandt.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdccxx{Sie sagten: «Ihr seid nur Menschen wie wir. Der Erbarmer hat nichts herabgesandt. Ihr lügt nur.»\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdccxxi{Sie sagten: «Unser Herr weiß es, wir sind wirklich zu euch gesandt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdccxxii{Und uns obliegt nur die deutliche Ausrichtung (der Botschaft).»\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdccxxiii{Sie sagten: «Wir sehen in euch ein böses Omen. Wenn ihr nicht aufhört, werden wir euch bestimmt steinigen, und euch wird bestimmt eine schmerzhafte Pein von uns treffen.»\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdccxxiv{Sie sagten: «Euer Omen ist bei euch selbst. (Redet ihr denn so,) wenn ihr ermahnt werdet? Nein, ihr seid eher maßlose Leute.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdccxxv{Und vom äußersten Ende der Stadt kam ein Mann gelaufen. Er sagte: «O mein Volk, folgt den Gesandten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdccxxvi{Folgt denen, die von euch keinen Lohn verlangen und der Rechtleitung folgen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdccxxvii{Und warum sollte ich dem nicht dienen, der mich erschaffen hat, und zu dem ihr zurückgebracht werdet?\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdccxxviii{Soll ich mir etwa an seiner Stelle andere Götter nehmen, deren Fürbitte, falls der Erbarmer mir Schaden zufügen will, mir nichts nützt, und die mich nicht retten können?\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdccxxix{Ich würde mich in einem offenkundigen Irrtum befinden.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdccxxx{Ich glaube an euren Herrn, so hört auf mich.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdccxxxi{Es wurde (zu ihm) gesprochen: «Geh ins Paradies ein.» Er sagte: «O wüßte doch mein Volk davon,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdccxxxii{Daß mein Herr mir vergeben und mich zu denen gestellt hat, die ehrenvoll behandelt werden!»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdccxxxiii{Nach ihm sandten Wir kein Heer vom Himmel gegen sein Volk herab; ja Wir sandten auch sonst nichts (gegen sie) herab.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdccxxxiv{Es genügte ein einziger Schrei, da waren sie plötzlich ausgelöscht.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdccxxxv{Schade um die Diener! Kein Gesandter kommt zu ihnen, ohne daß sie ihn verspotten würden.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdccxxxvi{Haben sie denn nicht gesehen, wie viele Generationen Wir vor ihnen haben verderben lassen, und daß sie zu ihnen nicht zurückkehren?\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdccxxxvii{Und sie werden unausweichlich alle insgesamt bei Uns vorgeführt.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdccxxxviii{Und ein Zeichen ist für sie die abgestorbene Erde. Wir haben sie belebt und aus ihr Körner hervorgebracht, von denen sie essen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdccxxxix{Und Wir haben auf ihr Gärten von Palmen und Weinstöcken angelegt und auf ihr Quellen hervorbrechen lassen,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdccxl{Damit sie von ihren Früchten essen können. Nicht ihre Hände haben sie geschaffen. Wollen sie denn nicht dankbar sein?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdccxli{Preis sei dem, der die Paare alle erschaffen hat: bei dem, was die Erde wachsen läßt, bei ihnen selbst und bei dem, was sie nicht wissen!\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdccxlii{Und ein Zeichen ist für sie die Nacht. Wir ziehen von ihr den Tag weg, und schon befinden sie sich im Dunkeln.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdccxliii{Und die Sonne läuft zu einem für sie bestimmten Aufenthaltsort. Das ist die Bestimmung dessen, der mächtig ist und Bescheid weiß.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdccxliv{Und den Mond haben Wir in Stationen gemessen, bis er abnimmt und wie ein alter Palmstiel wird.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdccxlv{Weder darf die Sonne den Mond einholen, noch kommt die Nacht dem Tag zuvor. Und jedes Gestirn nimmt seinen Lauf in einer (eigenen) Sphäre.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdccxlvi{Und ein Zeichen ist es für sie, daß Wir ihre Nachkommenschaft auf dem vollbeladenen Schiff getragen haben.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdccxlvii{Und Wir haben ihnen etwas Gleiches geschaffen, was sie besteigen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdccxlviii{Und wenn Wir wollen, lassen Wir sie ertrinken, dann gibt es niemanden, zu dem sie um Hilfe schreien könnten, und sie werden nicht gerettet,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdccxlix{Es sei denn aus Barmherzigkeit von Uns und zur Nutznießung für eine Weile.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdccl{Und wenn zu ihnen gesagt wird: «Hütet euch vor dem, was vor euch, und dem, was hinter euch ist, auf daß ihr Erbarmen findet.»...\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdccli{Kein Zeichen von den Zeichen ihres Herrn kommt zu ihnen, ohne daß sie sich davon abwenden.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdcclii{Und wenn zu ihnen gesagt wird: «Spendet von dem, was Gott euch beschert hat», sagen diejenigen, die ungläubig sind, zu denen, die glauben: «Sollen wir den ernähren, den Gott, wenn Er wollte, ernähren würde? Ihr befindet euch in einem offenkundigen Irrtum.»\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdccliii{Und sie sagen: «Wann wird diese Androhung eintreten, so ihr die Wahrheit sagt?»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdccliv{Sie warten wohl nur auf einen einzigen Schrei, der sie ergreift, während sie noch miteinander streiten.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdcclv{Da werden sie kein Testament machen und auch nicht zu ihren Angehörigen zurückkehren können.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdcclvi{Es wird in die Trompete geblasen, und gleich eilen sie aus den Gräbern zu ihrem Herrn herbei.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdcclvii{Sie sagen: «O wehe uns! Wer hat uns von unserer Schlafstätte erweckt? Das ist, was der Erbarmer versprochen hat, und die Gesandten haben die Wahrheit gesagt.»\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdcclviii{Es genügt nur ein einziger Schrei, da werden sie allesamt bei Uns vorgeführt.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdcclix{«Heute wird niemandem irgendein Unrecht getan. Und euch wird nur das vergolten, was ihr zu tun pflegtet.»\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdcclx{Die Gefährten des Paradieses finden heute Beschäftigung und Wohlbehagen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmdcclxi{Sie und ihre Gattinnen befinden sich im Schatten und lehnen sich auf Liegen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmdcclxii{Sie haben darin Früchte, und sie haben, was sie für sich wünschen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmdcclxiii{«Friede!», als Anrede von einem barmherzigen Herrn.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmdcclxiv{«Scheidet euch heute (von den Gottesfürchtigen), ihr Übeltäter.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmdcclxv{Habe Ich euch, o ihr Kinder Adams, nicht auferlegt, ihr sollt nicht dem Satan dienen - er ist euch ja ein offenkundiger Feind -,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmdcclxvi{Ihr sollt Mir dienen - das ist ein gerader Weg?\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmdcclxvii{Er hat doch viele Geschöpfe aus euren Reihen irregeführt. Hattet ihr denn keinen Verstand?\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmdcclxviii{Das ist die Hölle, die euch immer wieder angedroht wurde.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmdcclxix{Ihr sollt heute darin brennen dafür, daß ihr ungläubig waret.»\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmdcclxx{Heute versiegeln Wir ihnen den Mund. Ihre Hände werden zu Uns sprechen, und ihre Füße Zeugnis ablegen über das, was sie erworben haben.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmdcclxxi{Und wenn Wir wollten, würden Wir ihre Augen auswischen. Und sie eilen dann zum Weg um die Wette, aber wie könnten sie da sehen?\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmdcclxxii{Wenn Wir wollten, würden Wir sie an ihrem Ort verwandeln, so daß sie nicht mehr vorangehen können, und auch nicht zurückkehren.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmdcclxxiii{Wen Wir alt werden lassen, den lassen Wir in seiner körperlichen Verfassung eine Kehrtwende machen. Haben sie denn keinen Verstand?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmdcclxxiv{Und Wir haben ihn nicht das Dichten gelehrt, und es ziemt ihm nicht. Das ist doch nur eine Ermahnung und ein deutlicher Koran,\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmdcclxxv{Damit er diejenigen warne, die (da) leben, und der Spruch fällig werde gegen die Ungläubigen.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmdcclxxvi{Haben sie denn nicht gesehen, daß Wir ihnen unter dem, was unsere Hände gemacht haben, Herdentiere erschaffen haben, über die sie verfügen?\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmdcclxxvii{Und Wir haben sie ihnen unterworfen. Einige von ihnen dienen ihnen als Reittiere, von anderen können sie essen.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmdcclxxviii{Und sie haben an ihnen allerlei Nutzen und etwas zu trinken. Wollen sie denn nicht dankbar sein?\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmdcclxxix{Und sie haben sich anstelle Gottes Götter genommen, in der Hoffnung, Unterstützung zu finden.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmdcclxxx{Sie können doch ihnen keine Unterstützung gewähren, und sie sind für sie Truppen, die selbst vorgeführt werden.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxi{Ihre Worte sollen dich nicht betrüben. Wir wissen ja, was sie geheimhalten und was sie offenlegen.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxii{Hat denn der Mensch nicht gesehen, daß Wir ihn aus einem Tropfen erschaffen haben, und doch ist er ein offenkundiger Widerstreiter.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxiii{Er führt Uns ein Gleichnis an und vergißt, daß er erschaffen ist. Er sagt: «Wer macht diese Gebeine wieder lebendig, wenn sie auseinandergefallen sind?»\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxiv{Sprich: Wieder lebendig macht sie der, der sie das erste Mal hat entstehen lassen. Und Er weiß über alle Geschöpfe Bescheid.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxv{Er, der euch aus grünen Bäumen Feuer gemacht hat, so daß ihr gleich damit anzünden könnt.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxvi{Hat nicht der, der die Himmel und die Erde erschaffen hat, auch Macht, ihresgleichen zu erschaffen? Ja doch. Und Er ist der, der alles erschafft und Bescheid weiß.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxvii{Sein Befehl, wenn Er etwas will, ist, dazu nur zu sagen: Sei!, und es ist.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxviii{Preis sei dem, in dessen Hand die Herrschaft über alle Dinge ist und zu dem ihr zurückgebracht werdet!\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxix{\basmalahde Bei denen, die reihenweise sich reihen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdccxc{Und denen, die einen Schelteschrei ausstoßen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdccxci{Und denen, die eine Ermahnung verlesen!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdccxcii{Euer Gott ist ein Einziger,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdccxciii{Der Herr der Himmel und der Erde und dessen, was dazwischen ist, und der Herr der östlichen Gegenden.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdccxciv{Wir haben den untersten Himmel mit einem Schmuck geziert: mit den Sternen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdccxcv{Und dies zum Schutz vor jedem rebellischen Satan.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdccxcvi{So können sie der obersten Ratsversammlung nicht lauschen, und sie werden von allen Seiten beworfen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdccxcvii{Damit sie zurückgestoßen werden - und für sie ist eine ständige Pein bestimmt -,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdccxcviii{Außer dem, der etwas aufschnappt und den dann eine leuchtende Sternschnuppe verfolgt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdccxcix{Frag sie doch um ihre Meinung: Ist es etwa schwerer, sie zu erschaffen als die, die Wir (sonst) erschaffen haben? Wir haben sie ja aus haftendem Ton erschaffen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdccc{Nein, du wunderst dich, sie aber höhnen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdccci{Und wenn sie ermahnt werden, bedenken sie es nicht.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdcccii{Und wenn sie ein Zeichen sehen, verhöhnen sie es miteinander.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdccciii{Und sie sagen: «Das ist nur eine offenkundige Zauberei.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdccciv{Wenn wir gestorben und Staub und Knochen geworden sind, sollen wir dann wirklich auferweckt werden?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdcccv{Und auch unsere Vorväter?»\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdcccvi{Sprich: Ja, und ihr werdet euch demütig verhalten.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdcccvii{Es genügt ein einziger Schelteschrei, da schauen sie schon hin.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdcccviii{Und sie sagen: «O weh uns! Das ist der Tag des Gerichtes!»\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdcccix{- «Das ist der Tag der Scheidung, den ihr für Lüge zu erklären pflegtet.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdcccx{- Versammelt nun diejenigen, die Unrecht getan haben, ihre Partner und das, was sie verehrten\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdcccxi{Anstelle Gottes. Dann führt sie zum Weg der Hölle\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdcccxii{Und stellt sie auf, sie werden zur Verantwortung gezogen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdcccxiii{- Was ist mit euch, daß ihr nicht einander unterstützt?»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdcccxiv{Nein, heute ergeben sie sich.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdcccxv{Und sie gehen aufeinander zu, um sich gegenseitig zu befragen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdcccxvi{Sie sagen: «Ihr kamt doch von rechts her zu uns.»\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdcccxvii{Sie sagen: «Nein, ihr wäret ja nicht gläubig.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdcccxviii{Wir aber hatten keine Macht über euch, sondern ihr waret Leute, die ein Übermaß an Frevel zeigten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdcccxix{So ist der Spruch unseres Herrn gegen uns zu Recht fällig geworden. Wir werden bestimmt (die Pein) kosten.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdcccxx{Und so haben wir euch abirren lassen. Wir waren ja selbst abgeirrt.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdcccxxi{An jenem Tag werden sie an derselben Pein teilhaben.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdcccxxii{So verfahren Wir mit den Übeltätern.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdcccxxiii{Wenn zu ihnen gesagt wurde: «Es gibt keinen Gott außer Gott», verhielten sie sich hochmütig\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdcccxxiv{Und sagten: «Sollen wir denn unsere Götter verlassen wegen eines besessenen Dichters?»\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdcccxxv{Aber nein, er ist mit der Wahrheit gekommen und hat die Gesandten bestätigt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdcccxxvi{Ihr werdet bestimmt die schmerzhafte Pein kosten,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdcccxxvii{Und euch wird nur für das vergolten, was ihr zu tun pflegtet,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdcccxxviii{Ausgenommen den auserwählten Dienern Gottes.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdcccxxix{Für diese ist ein festgelegter Unterhalt bestimmt:\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdcccxxx{Früchte, und sie werden ehrenvoll behandelt\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxi{In den Gärten der Wonne\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxii{Auf Liegen, einander gegenüber.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxiii{Dabei wird ihnen ein Becher aus einem Quell herumgereicht,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxiv{Weiß, genußvoll für die, die (daraus) trinken.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxv{Darin steckt keine heimtückische Beeinträchtigung, und dadurch werden sie nicht berauscht.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxvi{Und bei ihnen sind (Huri), die ihre Blicke zurückhalten und schöne, große Augen haben,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxvii{Als ob sie wohlverwahrte Eier wären.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxviii{Und sie gehen aufeinander zu, um sich gegenseitig zu befragen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxix{Ein Sprecher unter ihnen sagt: «Ich hatte einen Gesellen,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdcccxl{Der sagte: «Gehörst du wirklich zu denen, die es für wahr halten?\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdcccxli{Sollen wir, wenn wir gestorben und Staub und Knochen geworden sind, dann wirklich gerichtet werden?\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdcccxlii{Er sagt (weiter): «Wollt ihr denn hinabschauen?»\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdcccxliii{Er schaut selbst hinab und sieht ihn mitten in der Hölle.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmdcccxliv{Er sagt: «Bei Gott, beinahe hättest du mich ins Verderben gestürzt.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmdcccxlv{Und ohne die Gnade meines Herrn würde ich nun zu denen gehören, die vorgeführt werden.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmdcccxlvi{Ist es nicht wirklich so, daß wir sterben\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmdcccxlvii{Nur durch unseren ersten Tod, und daß wir nicht gepeinigt werden?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmdcccxlviii{Das ist doch der großartige Erfolg!\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmdcccxlix{Für ein solches Ziel sollen diejenigen, die handeln, (ihre Werke) tun.»\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmdcccl{Ist dies als Herberge besser oder der Zaqqum-Baum?\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmdcccli{Den haben Wir zu einer Versuchung für die gemacht, die Unrecht tun.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmdccclii{Es ist ein Baum, der aus dem Grund der Hölle herauskommt\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmdcccliii{Und dessen Fruchtscheide so ist, als wären es Köpfe von Satanen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmdcccliv{Sie essen davon und füllen sich die Bäuche.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmdccclv{Dann erhalten sie darauf eine Mischung von heißem Wasser.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmdccclvi{Dann kehren sie zur Hölle zurück.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmdccclvii{Sie haben ihre Väter im Irrtum vorgefunden,\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmdccclviii{Da werden sie auf ihren Spuren eilig getrieben.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmdccclix{Vor ihnen sind bereits die meisten unter den Früheren abgeirrt,\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmdccclx{Und Wir haben unter sie Warner gesandt.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmdccclxi{So schau, wie das Ende der Gewarnten war,\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmdccclxii{Ausgenommen der auserwählten Diener Gottes.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmdccclxiii{Und wahrlich, Noach rief Uns an - welch vorzüglicher Erhörer sind Wir!\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmdccclxiv{Und Wir erretteten ihn und seine Angehörigen aus der großen Drangsal\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmdccclxv{Und machten seine Nachkommenschaft zu den (einzig) Überlebenden.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmdccclxvi{Und Wir ließen seinen Ruf unter den späteren Generationen fortbestehen.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmdccclxvii{Friede sei über Noach unter den Weltenbewohnern!\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmdccclxviii{So vergelten Wir den Rechtschaffenen.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmdccclxix{Er gehört zu unseren gläubigen Dienern.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmdccclxx{Dann ließen Wir die anderen ertrinken.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmdccclxxi{Zu seiner Gemeinde gehört Abraham.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmdccclxxii{Als er mit gesundem Herzen zu seinem Herrn kam.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmdccclxxiii{Als er zu seinem Vater und seinem Volk sagte: «Was verehrt ihr da?\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmdccclxxiv{Sucht ihr eine Lüge: Götter anstelle Gottes?\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmdccclxxv{Welche Meinung habt ihr denn vom Herrn der Welten?»\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmdccclxxvi{Und er warf einen Blick zu den Sternen\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmdccclxxvii{Und sagte: «Ich bin krank.»\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmdccclxxviii{Da kehrten sie ihm den Rücken.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmdccclxxix{Nun schlich er sich zu ihren Göttern und sagte: «Wollt ihr nicht essen?\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmdccclxxx{Was ist mit euch, daß ihr nicht redet?»\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxi{Und er wandte sich und schlug auf sie mit der Rechten ein.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxii{Da kamen sie auf ihn zu geeilt.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxiii{Er sagte: «Wie könnt ihr denn das verehren, was ihr selbst meißelt,\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxiv{Wo doch Gott euch und das, was ihr tut, erschaffen hat?»\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxv{Sie sagten: «Baut für ihn einen Bau und werft ihn in die Hölle.»\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxvi{Sie wollten gegen ihn mit einer List vorgehen. Da machten Wir sie zu den Unterlegenen.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxvii{Er sagte: «Ich gehe zu meinem Herrn, Er wird mich rechtleiten.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxviii{O mein Herr, schenk mir einen von den Rechtschaffenen.»\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxix{Da verkündeten Wir ihm einen langmütigen Knaben.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmmdcccxc{Als dieser das Alter erreichte, daß er mit ihm laufen konnte, sagte er: «Mein lieber Sohn, ich sehe im Schlaf, daß ich dich schlachte. Schau jetzt, was du meinst.» Er sagte: «O mein Vater, tu, was dir befohlen wird. Du wirst finden, so Gott will, daß ich zu den Standhaften gehöre.»\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmmdcccxci{Als sie sich beide ergeben gezeigt hatten und er ihn auf die Stirn niedergeworfen hatte,\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmmdcccxcii{Da riefen Wir ihm zu: «O Abraham,\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmmdcccxciii{Du hast das Traumgesicht wahr gemacht.» So vergelten Wir den Rechtschaffenen.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmmdcccxciv{Das ist die offenkundige Prüfung.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmmdcccxcv{Und Wir lösten ihn mit einem großen Schlachtopfer aus.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmmdcccxcvi{Und Wir ließen seinen Ruf unter den späteren Generationen fortbestehen.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmmdcccxcvii{Friede sei über Abraham!\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmmdcccxcviii{So vergelten Wir den Rechtschaffenen.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmmdcccxcix{Er gehört zu unseren gläubigen Dienern.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmmcm{Und Wir verkündeten ihm Isaak als einen Propheten von den Rechtschaffenen.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmmcmi{Und Wir segneten ihn und Isaak. Unter ihren Nachkommen gibt es welche, die rechtschaffen sind, und welche, die sich selbst offenkundig Unrecht tun.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmmcmii{Und Wir erwiesen Mose und Aaron eine Wohltat\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmmcmiii{Und erretteten sie beide und ihr Volk aus der großen Drangsal.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmmcmiv{Und Wir unterstützten sie, da waren sie es, die die Sieger wurden.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmmcmv{Und Wir ließen ihnen beiden das deutliche Buch zukommen\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmmcmvi{Und führten sie den geraden Weg.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmmcmvii{Und Wir ließen ihren Ruf unter den späteren Generationen fortbestehen.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmmcmviii{Friede sei über Mose und Aaron!\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmmcmix{So vergelten Wir den Rechtschaffenen.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmmcmx{Sie beide gehören zu unseren gläubigen Dienern.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmmcmxi{Auch Elias war einer der Gesandten.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmmcmxii{Als er zu seinem Volk sagte: «Wollt ihr nicht gottesfürchtig sein?\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmmcmxiii{Wie könnt ihr den Baal anrufen und den Besten der Schöpfer verlassen,\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmmcmxiv{Gott, euren Herrn und den Herrn eurer Vorväter?»\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmmcmxv{Da ziehen sie ihn der Lüge. So werden sie gewiß vorgeführt werden,\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmmcmxvi{Ausgenommen die auserwählten Diener Gottes.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmmcmxvii{Und Wir ließen seinen Ruf unter den späteren Generationen fortbestehen.\qt@no{129}}%
\qt@newcmd\qurantrans@de@mmmcmxviii{Friede sei über Elias!\qt@no{130}}%
\qt@newcmd\qurantrans@de@mmmcmxix{So vergelten Wir den Rechtschaffenen.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mmmcmxx{Er gehört zu unseren gläubigen Dienern.\qt@no{132}}%
\qt@newcmd\qurantrans@de@mmmcmxxi{Auch Lot war einer der Gesandten.\qt@no{133}}%
\qt@newcmd\qurantrans@de@mmmcmxxii{Als Wir ihn und alle seine Angehörigen erretteten,\qt@no{134}}%
\qt@newcmd\qurantrans@de@mmmcmxxiii{Außer einer alten Frau unter denen, die zurückblieben und dem Verderben anheimfielen.\qt@no{135}}%
\qt@newcmd\qurantrans@de@mmmcmxxiv{Dann zerstörten Wir die anderen.\qt@no{136}}%
\qt@newcmd\qurantrans@de@mmmcmxxv{Ihr kommt ja an ihnen vorbei, am Morgen\qt@no{137}}%
\qt@newcmd\qurantrans@de@mmmcmxxvi{Und in der Nacht. Habt ihr denn keinen Verstand?\qt@no{138}}%
\qt@newcmd\qurantrans@de@mmmcmxxvii{Auch Jonas war einer der Gesandten.\qt@no{139}}%
\qt@newcmd\qurantrans@de@mmmcmxxviii{Als er zum vollbeladenen Schiff davonlief.\qt@no{140}}%
\qt@newcmd\qurantrans@de@mmmcmxxix{Er warf Lose und wurde einer der Unterlegenen.\qt@no{141}}%
\qt@newcmd\qurantrans@de@mmmcmxxx{Der Fisch verschlang ihn, der sich Tadel zugezogen hatte.\qt@no{142}}%
\qt@newcmd\qurantrans@de@mmmcmxxxi{Und wäre er nicht einer von denen geworden, die (Gott) preisen,\qt@no{143}}%
\qt@newcmd\qurantrans@de@mmmcmxxxii{Wäre er in seinem Bauch geblieben bis zu dem Tag, an dem sie auferweckt werden.\qt@no{144}}%
\qt@newcmd\qurantrans@de@mmmcmxxxiii{Da warfen Wir ihn auf das kahle Land; dabei war er krank.\qt@no{145}}%
\qt@newcmd\qurantrans@de@mmmcmxxxiv{Und Wir ließen eine Kürbisstaude über ihm wachsen.\qt@no{146}}%
\qt@newcmd\qurantrans@de@mmmcmxxxv{Und Wir sandten ihn zu Hunderttausend oder gar mehr.\qt@no{147}}%
\qt@newcmd\qurantrans@de@mmmcmxxxvi{Da glaubten sie. Und Wir gewährten ihnen Nutznießung für eine Weile.\qt@no{148}}%
\qt@newcmd\qurantrans@de@mmmcmxxxvii{Frag sie um ihre Meinung: Gehören deinem Herrn etwa die Töchter, und ihnen die Söhne?\qt@no{149}}%
\qt@newcmd\qurantrans@de@mmmcmxxxviii{Oder haben Wir die Engel als weibliche Wesen erschaffen, während sie zugegen waren?\qt@no{150}}%
\qt@newcmd\qurantrans@de@mmmcmxxxix{Es ist aus Lügenhaftigkeit, daß sie sagen:\qt@no{151}}%
\qt@newcmd\qurantrans@de@mmmcmxl{«Gott hat gezeugt.» Wahrlich, sie sind Lügner.\qt@no{152}}%
\qt@newcmd\qurantrans@de@mmmcmxli{Hat Er sich denn die Töchter vor den Söhnen auserwählt?\qt@no{153}}%
\qt@newcmd\qurantrans@de@mmmcmxlii{Was ist mit euch? Wie urteilt ihr nur?\qt@no{154}}%
\qt@newcmd\qurantrans@de@mmmcmxliii{Wollt ihr es nicht bedenken?\qt@no{155}}%
\qt@newcmd\qurantrans@de@mmmcmxliv{Oder habt ihr eine offenkundige Ermächtigung?\qt@no{156}}%
\qt@newcmd\qurantrans@de@mmmcmxlv{Dann bringt euer Buch herbei, so ihr die Wahrheit sagt.\qt@no{157}}%
\qt@newcmd\qurantrans@de@mmmcmxlvi{Und sie stellen zwischen Ihm und den Djinn eine Verwandtschaft her. Aber die Djinn wissen, daß sie vorgeführt werden -\qt@no{158}}%
\qt@newcmd\qurantrans@de@mmmcmxlvii{Preis sei Gott! (Er ist erhaben) über das, was sie da schildern -,\qt@no{159}}%
\qt@newcmd\qurantrans@de@mmmcmxlviii{Ausgenommen die auserwählten Diener Gottes.\qt@no{160}}%
\qt@newcmd\qurantrans@de@mmmcmxlix{Ihr und das, was ihr verehrt,\qt@no{161}}%
\qt@newcmd\qurantrans@de@mmmcml{Könnt niemanden gegen Ihn verführen,\qt@no{162}}%
\qt@newcmd\qurantrans@de@mmmcmli{Außer dem, der in der Hölle brennen wird.\qt@no{163}}%
\qt@newcmd\qurantrans@de@mmmcmlii{Und es gibt niemanden von uns, der nicht einen gekennzeichneten Rang hätte.\qt@no{164}}%
\qt@newcmd\qurantrans@de@mmmcmliii{Wahrlich, wir sind es, die reihenweise sich reihen,\qt@no{165}}%
\qt@newcmd\qurantrans@de@mmmcmliv{Und wahrlich, wir sind es, die lobpreisen.\qt@no{166}}%
\qt@newcmd\qurantrans@de@mmmcmlv{Und sie pflegten zu sagen:\qt@no{167}}%
\qt@newcmd\qurantrans@de@mmmcmlvi{«Wenn wir nur eine Ermahnung gleich der der Früheren hätten,\qt@no{168}}%
\qt@newcmd\qurantrans@de@mmmcmlvii{Dann wären wir die auserwählten Diener Gottes.»\qt@no{169}}%
\qt@newcmd\qurantrans@de@mmmcmlviii{Und doch verleugnen sie Ihn. Aber sie werden es zu wissen bekommen.\qt@no{170}}%
\qt@newcmd\qurantrans@de@mmmcmlix{Schon früher ist unser Spruch an unsere gesandten Diener ergangen:\qt@no{171}}%
\qt@newcmd\qurantrans@de@mmmcmlx{Sie sind es, die Unterstützung erfahren werden.\qt@no{172}}%
\qt@newcmd\qurantrans@de@mmmcmlxi{Und unsere Heerscharen werden die Sieger sein.\qt@no{173}}%
\qt@newcmd\qurantrans@de@mmmcmlxii{So kehre dich für eine Weile von ihnen ab\qt@no{174}}%
\qt@newcmd\qurantrans@de@mmmcmlxiii{Und schau auf sie. Auch sie werden es schauen.\qt@no{175}}%
\qt@newcmd\qurantrans@de@mmmcmlxiv{Wünschen sie denn unsere Pein zu beschleunigen?\qt@no{176}}%
\qt@newcmd\qurantrans@de@mmmcmlxv{Wenn sie sich aber in ihrem Gebiet niederläßt, dann wird der Morgen der Gewarnten schlimm sein.\qt@no{177}}%
\qt@newcmd\qurantrans@de@mmmcmlxvi{Und kehre dich für eine Weile von ihnen ab\qt@no{178}}%
\qt@newcmd\qurantrans@de@mmmcmlxvii{Und schau zu. Auch sie werden es schauen.\qt@no{179}}%
\qt@newcmd\qurantrans@de@mmmcmlxviii{Preis sei deinem Herrn, dem Herrn der Macht! (Er ist erhaben) über das, was sie schildern.\qt@no{180}}%
\qt@newcmd\qurantrans@de@mmmcmlxix{Und Friede sei über den Gesandten!\qt@no{181}}%
\qt@newcmd\qurantrans@de@mmmcmlxx{Und Lob sei Gott, dem Herrn der Welten!\qt@no{182}}%
\qt@newcmd\qurantrans@de@mmmcmlxxi{\basmalahde Saad. Beim Koran, der die Ermahnung enthält!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcmlxxii{Nein, diejenigen, die ungläubig sind, zeigen Stolz und befinden sich im Widerstreit.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcmlxxiii{Wie manche Generation haben Wir vor ihnen verderben lassen. Sie riefen, da es doch keine Zeit mehr zum Entrinnen war.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcmlxxiv{Sie wundern sich darüber, daß ein Warner aus ihrer Mitte zu ihnen gekommen ist. Die Ungläubigen sagen: «Dies ist ein Zauberer, der lügt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcmlxxv{Will er denn die Götter zu einem einzigen Gott machen? Das ist eine verwunderliche Sache.»\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcmlxxvi{Und die Vornehmen unter ihnen gingen fort: «Geht hin und haltet an euren Göttern beharrlich fest. Das ist eine Sache, die gefordert wird.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcmlxxvii{Wir haben nicht gehört, daß es bei der vorherigen Religionsgemeinschaft so etwas gegeben hat. Das ist nur eine Erfindung.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcmlxxviii{Ist die Ermahnung wirklich gerade auf ihn aus unserer Mitte herabgesandt worden?» Nein, sie hegen über meine Ermahnung Zweifel. Nein, sie haben eben meine Pein noch nicht gekostet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcmlxxix{Oder besitzen sie etwa die Vorratskammern der Barmherzigkeit deines Herrn, des Mächtigen, des Freigebigen?\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcmlxxx{Oder gehört etwa ihnen die Königsherrschaft der Himmel und der Erde und dessen, was dazwischen ist? So sollen sie doch in den Gängen emporsteigen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxi{Solch eine Heerschar von den Parteien, die da geschlagen steht!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxii{Der Lüge ziehen (ihre Gesandten) schon vor ihnen das Volk Noachs, die Aad und Pharao, der Besitzer der Pfähle,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxiii{Und die Thamud, das Volk Lots und die Leute des Waldes. Das sind die Parteien.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxiv{Alle ausnahmslos ziehen die Propheten der Lüge, so wurde meine Strafe zu Recht fällig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxv{Diese warten wohl nur auf einen einzigen Schrei, der keinen Aufschub duldet.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxvi{Und sie sagen: «Unser Herr, laß unseren Anteil uns ereilen noch vor dem Tag der Abrechnung.»\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxvii{Ertrag mit Geduld, was sie sagen. Und gedenke unseres Dieners David, des Kraftvollen. Er war bereit zur Umkehr.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxviii{Und Wir machten die Berge dienstbar, daß sie zusammen mit ihm am Abend und bei Sonnenaufgang lobpreisen,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxix{Und auch die Vögel in Scharen. Alle waren bereit, zu Ihm umzukehren.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcmxc{Und Wir festigten seine Königsherrschaft und ließen ihm die Weisheit und die Fähigkeit zu entscheidendem Spruch zukommen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcmxci{Ist der Bericht über die Streitigkeiten zu dir gelangt? Als sie über die Mauern in die Räume einstiegen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcmxcii{Als sie bei David eintraten. Da hatte er Angst vor ihnen. Sie sagten: «Fürchte dich nicht. Wir sind zwei Streitparteien, von denen die eine gegen die andere Übergriffe begangen hat. So urteile zwischen uns nach der Wahrheit, handle nicht ungerecht und führe uns zum rechten Weg.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcmxciii{Dieser mein Bruder besitzt neunundneunzig Schafe, ich aber ein einziges Schaf. Nun sagte er: >Vertraue es mir an\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcmxciv{Er sagte: «Er hat dir Unrecht getan, daß er dein Schaf zu seinen Schafen hinzu verlangte. Viele von den Partnern begehen gegeneinander Übergriffe, ausgenommen diejenigen, die glauben und die guten Werke tun - und das sind nur wenige.» Und David verstand, daß Wir ihn der Versuchung ausgesetzt hatten. Da bat er seinen Herrn um Vergebung und warf sich in Verneigung nieder und wandte sich (Ihm) reumütig zu.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcmxcv{Da vergaben Wir ihm dies. Bestimmt ist für ihn der Zutritt in unsere Nähe und eine schöne Heimstatt.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcmxcvi{O David, Wir haben dich zum Nachfolger auf der Erde bestellt. So urteile zwischen den Menschen nach der Wahrheit und folge nicht der (eigenen) Neigung, daß sie dich nicht vom Weg Gottes abirren läßt. Für diejenigen, die vom Weg Gottes abirren, ist eine harte Pein bestimmt dafür, daß sie den Tag der Abrechnung vergessen haben.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcmxcvii{Und Wir haben den Himmel und die Erde und das, was dazwischen ist, nicht umsonst erschaffen. Das ist die Meinung derer, die ungläubig sind. Wehe aber denen, die ungläubig sind, vor dem Feuer!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcmxcviii{Oder sollen Wir etwa diejenigen, die glauben und die guten Werke tun, den Unheilstiftern auf Erden gleichstellen, oder die Gottesfürchtigen denen, die voller Laster sind?\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcmxcix{Dies ist ein gesegnetes Buch, das Wir zu dir hinabgesandt haben, damit sie seine Zeichen betrachten und damit die Einsichtigen es bedenken.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmm{Und Wir schenkten David den Salomo. Welch trefflicher Diener! Er war immer bereit zur Umkehr.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmi{Als ihm zur Abendzeit die auf drei Füßen stehenden, schnellen Pferde vorgeführt wurden.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmii{Da sagte er: «Ich habe mich der Liebe der (irdischen) Güter hingegeben und darüber übersehen, meines Herrn zu gedenken, bis sie hinter dem Vorhang verschwand.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmiii{Bringt sie mir her.» Da begann er, ihnen über Beine und Hals zu streichen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmiv{Und Wir unterwarfen Salomo der Versuchung und setzten ein leibhaftiges Wesen auf seinen Stuhl. Dann bekehrte er sich.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmv{Er sagte: «Mein Herr, vergib mir und schenke mir eine Königsherrschaft, wie sie keinem nach mir geziemt. Du bist ja der Freigebige.»\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmvi{Da machten Wir ihm den Wind dienstbar, daß er nach seinem Befehl sanft eilte, wohin er es für treffend hielt;\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmvii{Und auch die Satane, jeden Bauarbeiter und Taucher,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmviii{Und andere, in Ketten aneinandergebunden.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmix{«Das ist unsere Gabe. So sei wohltätig oder zurückhaltend, und dies, ohne zu rechnen.»\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmx{Bestimmt ist für ihn der Zutritt in unsere Nähe und eine schöne Heimstatt.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmxi{Und gedenke unseres Dieners Ijob. Als er zu seinem Herrn rief: «Mich hat der Satan mit Mühsal und Pein berührt.»\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmxii{- «Stampfe mit dem Fuß. Das ist kühles Wasser zum Waschen und zum Trinken.»\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmxiii{Und Wir schenkten ihm seine Angehörigen (wieder) und noch einmal die gleiche Zahl dazu, aus Barmherzigkeit von Uns und als Ermahnung für die Einsichtigen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmxiv{Und: «Nimm in deine Hand ein Bündel (Zweige) und schlag damit zu und sei nicht eidbrüchig.» Wir fanden ihn geduldig. Welch trefflicher Diener! Er war immer bereit zur Umkehr.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmxv{Und gedenke unserer Diener Abraham, Isaak und Jakob, die Kraft und Einsicht besaßen.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmxvi{Wir haben sie mit einer besonderen Erwählung ausgezeichnet, mit dem Gedenken an die (jenseitige) Wohnstätte.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmxvii{Sie gehören bei Uns zu den Auserwählten und Guten.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmxviii{Und gedenke Ismaels, Elischas und des Dhu1-Kifl. Alle gehören zu den Guten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmxix{Dies ist eine Ermahnung. Für die Gottesfürchtigen ist eine schöne Heimstatt bestimmt,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmxx{Die Gärten von Eden, deren Tore ihnen geöffnet stehen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmxxi{Sie lehnen sich darin, und sie rufen darin nach vielen Früchten und nach Getränk.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmxxii{Und bei ihnen sind gleichaltrige (Huri), die ihre Blicke zurückhalten.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmxxiii{Das ist, was euch für den Tag der Abrechnung verheißen ist.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmxxiv{Das ist unsere Versorgung; sie geht nicht zu Ende.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmxxv{So ist es. Für die, die ein Übermaß an Frevel zeigen, ist eine böse Heimstätte bestimmt,\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmxxvi{Die Hölle, in der sie brennen. Welch schlimme Lagerstätte!\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmxxvii{Das ist - sie sollen es kosten - heißes Wasser und stinkender Eiter\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmxxviii{Und anderes dergleichen in verschiedenen Arten.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmxxix{- «Da ist eine Schar, die sich mit euch hineinstürzt.» - «Sie sind nicht willkommen. Sie werden im Feuer brennen.»\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmxxx{Sie sagen: «Nein, ihr seid es, die ihr nicht willkommen seid. Das habt ihr uns doch verursacht - ein schlimmer Aufenthalt.»\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmxxxi{Sie sagen: «Unser Herr, mehre dem, der uns dies verursacht hat, die Pein, eine doppelte (Pein) im Feuer.»\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmxxxii{Und sie sagen: «Warum sehen wir nicht gewisse Männer, die wir zu den Bösen zählten?\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmxxxiii{Haben wir sie (etwa zu Unrecht) zum Gegenstand des Spottes genommen? Oder sind die Blicke von ihnen gewichen?»\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmxxxiv{Das ist wahr, so streiten die Bewohner des Feuers miteinander.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmxxxv{Sprich: Ich bin nur ein Warner. Und es gibt keinen Gott außer Gott, dem Einen, der bezwingende Macht besitzt,\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmxxxvi{Dem Herrn der Himmel und der Erde und dessen, was dazwischen ist, dem Mächtigen, der voller Vergebung ist.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmxxxvii{Sprich: Das ist ein gewaltiger Bericht,\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmxxxviii{Von dem ihr euch abwendet.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmxxxix{Ich hatte kein Wissen über die oberste Ratsversammlung, als sie miteinander stritten.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmxl{Mir wird nur offenbart, daß ich ja ein offenkundiger Warner bin.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmxli{Als dein Herr zu den Engeln sprach: «Ich werde einen Menschen aus Ton erschaffen.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmxlii{Wenn Ich ihn geformt und ihm von meinem Geist eingeblasen habe, dann fallt und werft euch vor ihm nieder.»\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmxliii{Da warfen sich die Engel alle zusammen nieder,\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmxliv{Außer Iblis; er verhielt sich hochmütig und war einer der Ungläubigen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmxlv{Er sprach: «O Iblis, was hat dich daran gehindert, dich vor dem niederzuwerfen, was Ich mit meinen Händen erschaffen habe? Verhältst du dich (jetzt) hochmütig, oder gehörst du etwa zu den Ranghohen?»\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmxlvi{Er sagte: «Ich bin besser als er. Mich hast Du aus Feuer erschaffen, ihn hast Du (nur) aus Ton erschaffen.»\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmxlvii{Er sprach: «Dann geh aus ihm hinaus. Du bist der Steinigung würdig,\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmxlviii{Und auf dir liegt mein Fluch bis zum Tag des Gerichtes.»\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmxlix{Er sagte: «Mein Herr, gewähre mir Aufschub bis zu dem Tag, da sie auferweckt werden.»\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmml{Er sprach: «Siehe, du gehörst nun zu denen, denen Aufschub gewährt wird,\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmli{Bis zum Tag der bestimmten Zeit.»\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmlii{Er sagte: «Bei deiner Macht, ich werde sie allesamt abirren lassen,\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmliii{Außer deinen auserwählten Dienern unter ihnen.»\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmliv{Er sprach: «Es ist die Wahrheit - und Ich sage ja die Wahrheit;\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmlv{Ich werde die Hölle füllen mit dir und mit all denen von ihnen, die dir folgen.»\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmlvi{Sprich: Ich verlange von euch keinen Lohn dafür. Und ich gehöre nicht zu denen, die unbefugt handeln.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmmlvii{Es ist nur eine Ermahnung für die Weltenbewohner.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmmlviii{Und ihr werdet den Bericht darüber nach einer Weile erfahren.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmmlix{\basmalahde Herabsendung des Buches von Gott, dem Mächtigen, dem Weisen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmlx{Wir haben das Buch mit der Wahrheit zu dir hinabgesandt. So diene Gott und sei Ihm gegenüber aufrichtig in der Religion.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmlxi{Wahrlich, Gott gehört die aufrichtige Religion. Und zwischen denen, die sich an seiner Stelle Freunde nehmen - «Wir dienen ihnen nur», (sagen sie), «damit sie uns Zutritt in die Nähe Gottes verschaffen» -, wird Gott urteilen über das, worüber sie uneins sind. Gott leitet den nicht recht, der ein Lügner und sehr ungläubig ist.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmlxii{Hätte Gott sich ein Kind nehmen wollen, hätte Er aus dem, was Er erschaffen hat, sich auserwählt, was Er will. Preis sei Ihm! Er ist Gott, der Eine, der bezwingende Macht besitzt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmlxiii{Er hat die Himmel und die Erde in Wahrheit erschaffen. Er legt die Nacht über den Tag, und Er legt den Tag über die Nacht. Und Er hat die Sonne und den Mond dienstbar gemacht - jedes läuft auf eine bestimmte Frist. Siehe, Er ist der Mächtige, der voller Vergebung ist.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmlxiv{Er hat euch aus einem einzigen Wesen erschaffen, dann machte Er aus ihm seine Gattin. Und Er hat für euch an Vieh acht (Tiere) in Paaren herabgesandt. Er erschafft euch im Schoß eurer Mütter, eine Schöpfung nach der anderen in dreifacher Finsternis. Das ist eben Gott, euer Herr. Ihm gehört die Königsherrschaft. Es gibt keinen Gott außer Ihm. Wie leicht laßt ihr euch doch abbringen!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmlxv{Wenn ihr ungläubig seid, so ist Gott nicht auf euch angewiesen. Er findet kein Gefallen am Unglauben für seine Diener. Wenn ihr dankbar seid, so findet Er daran Gefallen für euch. Und keine lasttragende (Seele) trägt die Last einer anderen. Alsdann wird eure Rückkehr zu eurem Herrn sein. Dann wird Er euch kundtun, was ihr zu tun pflegtet. Er weiß wohl über das innere Geheimnis Bescheid.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmlxvi{Und wenn ein Schaden den Menschen trifft, ruft er zu seinem Herrn, indem er sich Ihm reumütig zuwendet. Wenn Er ihm Gnade von sich gewährt, vergißt er, wofür er vorher gebetet hat, und er stellt Gott andere als Gegenpart zur Seite, um (die Menschen) von seinem Weg abirren zu lassen. Sprich: Genieße deinen Unglauben ein wenig; du gehörst zu den Gefährten des Feuers.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmlxvii{Ist etwa einer, der sich zu (verschiedenen) Nachtzeiten in demütiger Andacht befindet, ob er sich niederwirft oder aufrecht steht, der sich vor dem Jenseits in acht nimmt und die Barmherzigkeit seines Herrn erhofft...? - Sprich: Sind etwa diejenigen, die wissen, und diejenigen, die nicht wissen, gleich? Doch bedenken es nur die Einsichtigen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmlxviii{Sprich: O meine Diener, die ihr gläubig seid, fürchtet euren Herrn. Diejenigen, die rechtschaffen sind, erhalten hier im Diesseits Gutes. Und Gottes Erde ist weit. Den Geduldigen wird ihr Lohn voll erstattet, ohne daß es (viel) gerechnet wird.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmlxix{Sprich: Mir wurde befohlen, Gott zu dienen und dabei Ihm gegenüber aufrichtig in der Religion zu sein.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmlxx{Und mir wurde befohlen, der erste der Gottergebenen zu sein.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmlxxi{Sprich: Ich fürchte, wenn ich gegen meinen Herrn ungehorsam bin, die Pein eines gewaltigen Tages.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmlxxii{Sprich: Gott (allein) diene ich, und ich bin dabei Ihm gegenüber aufrichtig in meiner Religion.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmlxxiii{Ihr mögt auch an seiner Stelle verehren, was ihr wollt. Sprich: Die (wahren) Verlierer sind die, die am Tag der Auferstehung sich selbst und ihre Angehörigen verlieren. Ja, das ist der offenkundige Verlust.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmlxxiv{Sie haben über sich überschattende Hüllen von Feuer und unter sich (ebensolche) überschattende Hüllen. Damit macht Gott seinen Dienern angst: «O meine Diener, fürchtet Mich also.»\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmlxxv{Und denen, die die Götter meiden, um ihnen nicht zu dienen, und sich Gott reumütig zuwenden, gilt die frohe Botschaft. So verkünde die frohe Botschaft meinen Dienern,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmlxxvi{Die auf das Wort hören und dem Besten davon folgen. Das sind die, die Gott rechtleitet, und das sind die Einsichtigen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmlxxvii{Denjenigen, gegen den der Spruch der Pein zu Recht fällig geworden ist... - Willst du etwa den retten, der im Feuer ist?\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmlxxviii{Aber für die, die ihren Herrn fürchten, sind Obergemächer bestimmt, über denen (andere) Obergemächer aufgebaut sind, unter denen Bäche fließen. Das ist das Versprechen Gottes. Gott bricht das Versprechen nicht.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmlxxix{Hast du denn nicht gesehen, daß Gott vom Himmel Wasser herabkommen und es als Quellen in der Erde seine Bahn nehmen läßt? Dann bringt Er dadurch Getreide verschiedener Art hervor. Dann verdorrt es, und du siehst es gelb werden. Dann macht Er es zu zermalmtem Zeug. Darin ist eine Ermahnung für die Einsichtigen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmlxxx{Ist denn der, dem Gott die Brust für den Islam weitet, so daß er in einem Licht von seinem Herrn wandert, (einem Irrenden gleich)? So wehe denen, deren Herzen gegen die Ermahnung Gottes verhärtet sind! Sie befinden sich in einem offenkundigen Irrtum.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmlxxxi{Gott hat die beste Botschaft herabgesandt, ein Buch mit gleichartigen, sich wiederholenden Versen, vor dem die Haut derer, die ihren Herrn fürchten, erschauert. Dann werden ihre Haut und ihr Herz weich und neigen sich dem Gedenken Gottes zu. Das ist die Rechtleitung Gottes. Er leitet damit recht, wen Er will. Und wen Gott irreführt, der hat niemanden, der ihn rechtleiten könnte.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmlxxxii{Ist denn einer, der sich mit seinem Gesicht vor der schlimmen Pein am Tag der Auferstehung hütet,... (einem Unbekümmerten gleich)? Und es wird zu denen, die Unrecht tun, gesprochen: «Kostet, was ihr erworben habt.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmlxxxiii{Für Lüge haben es diejenigen erklärt, die vor ihnen lebten. Da kam die Pein über sie, von wo sie es nicht merkten.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmlxxxiv{Und Gott ließ sie im diesseitigen Leben die Schande erleiden. Aber die Pein des Jenseits ist größer, wenn sie es nur wüßten!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmlxxxv{Und Wir haben den Menschen in diesem Koran allerlei Gleichnisse angeführt - auf daß sie es bedenken -,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmlxxxvi{Als einem arabischen Koran, an dem nichts Krummes ist, auf daß sie gottesfürchtig werden.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmlxxxvii{Gott führt als Gleichnis einen Mann an, der Herren gehört, welche miteinander in Zank liegen, und einen Mann, der nur einem Herrn gehört. Sind die beiden im Gleichnis etwa gleich? Lob sei Gott! Aber die meisten von ihnen wissen nicht Bescheid.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmlxxxviii{Du wirst gewiß sterben, auch sie werden sterben.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmlxxxix{Dann werdet ihr am Tag der Auferstehung bei eurem Herrn miteinander streiten.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmxc{Wer ist denn ungerechter als der, der gegen Gott lügt und die Wahrheit, als sie zu ihm kam, für Lüge erklärt? Ist nicht in der Hölle eine Bleibe für die Ungläubigen?\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmxci{Diejenigen aber, die mit der Wahrheit kommen und sie für wahr halten, das sind die Gottesfürchtigen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmxcii{Sie haben bei ihrem Herrn, was sie wollen. Das ist der Lohn der Rechtschaffenen,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmxciii{Auf daß Gott ihnen das Schlechteste von dem, was sie getan haben, sühne und ihnen ihren Lohn vergelte für das Beste von dem, was sie taten.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmxciv{Genügt Gott seinem Diener nicht? Und sie wollen dir mit denen, die es außer Ihm geben soll, angst machen. Wen Gott irreführt, der hat niemanden, der ihn rechtleiten könnte.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmxcv{Und wen Gott rechtleitet, den kann niemand irreführen. Ist nicht Gott mächtig, und übt Er nicht Rache?\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmxcvi{Und wenn du sie fragst, wer die Himmel und die Erde erschaffen hat, sagen sie bestimmt: «Gott.» Sprich: Habt ihr denn das, was ihr anstelle Gottes anruft, betrachtet? Wenn Gott für mich Schaden will, können sie denn seinen Schaden beheben? Oder wenn Er für mich Barmherzigkeit will, könnten sie seine Barmherzigkeit zurückhalten? Sprich: Mir genügt Gott. Auf Ihn vertrauen die, die (überhaupt auf jemanden) vertrauen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmxcvii{Sprich: O mein Volk, handelt nach eurem Standpunkt, ich werde auch so handeln. Ihr werdet erfahren,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmxcviii{über wen eine Pein, die ihn zuschanden macht, kommen und auf den eine beständige Pein niedergehen wird.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmxcix{Wir haben für die Menschen das Buch mit der Wahrheit auf dich hinabgesandt. Wer der Rechtleitung folgt, tut das zu seinem eigenen Vorteil. Wer irregeht, geht irre zu seinem eigenen Schaden. Und du bist nicht ihr Sachwalter.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmc{Gott beruft die Seelen zur Zeit ihres Todes ab, und auch die, die nicht gestorben sind, während ihres Schlafs. Er hält die eine, für die Er den Tod beschlossen hat, zurück, und Er läßt die andere auf eine bestimmte Frist laufen. Darin sind Zeichen für Leute, die nachdenken.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmci{Oder haben sie sich anstelle Gottes Fürsprecher genommen? Sprich: Was denn, auch wenn sie über nichts verfügen und keinen Verstand haben?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcii{Sprich: Alle Fürsprache gehört Gott allein. Ihm gehört die Königsherrschaft der Himmel und der Erde. Dann werdet ihr zu Ihm zurückgebracht.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmciii{Und wenn Gott allein erwähnt wird, empfinden die Herzen derer, die nicht an das Jenseits glauben, Abneigung. Und wenn die erwähnt werden, die es außer Ihm geben soll, da sind sie plötzlich froh.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmciv{Sprich: O mein Gott, Schöpfer der Himmel und der Erde, der über das Unsichtbare und das Offenbare Bescheid weiß. Du wirst zwischen deinen Dienern urteilen über das, worüber sie uneins waren.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcv{Wenn diejenigen, die Unrecht tun, alles hätten, was auf der Erde ist, und noch einmal das gleiche dazu, sie würden sich damit von der schlimmen Pein am Tag der Auferstehung loskaufen. Ihnen wird dann nämlich von Gott her klar, womit sie nicht gerechnet haben.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcvi{Und ihnen werden die Missetaten, die sie erworben haben, klar, und es umschließt sie das, worüber sie spotteten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcvii{Wenn ein Schaden den Menschen berührt, ruft er zu Uns. Wenn Wir ihm dann Gnade von Uns gewähren, sagt er: «Es ist mir aufgrund von Wissen zugekommen.» Nein, es ist eine Versuchung. Aber die meisten von ihnen wissen nicht Bescheid.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcviii{Das haben schon diejenigen, die vor ihnen lebten, gesagt. Aber nicht nützte ihnen, was sie zu erwerben pflegten.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcix{Und die Missetaten, die sie erworben hatten, trafen sie. Und auch diejenigen unter diesen da, die Unrecht tun, werden die Missetaten, die sie erworben haben, treffen. Und sie können es nicht vereiteln.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcx{Wissen sie denn nicht, daß Gott den Lebensunterhalt großzügig zuteilt, wem Er will, und auch bemessen? Darin sind Zeichen für Leute, die glauben.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcxi{Sprich: O meine Diener, die ihr gegen euch selbst Übertretungen begangen habt, gebt die Hoffnung auf die Barmherzigkeit Gottes nicht auf. Gott vergibt die Sünden alle. Er ist ja der, der voller Vergebung und barmherzig ist.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcxii{Und wendet euch eurem Herrn reumütig zu und seid Ihm ergeben, bevor die Pein über euch kommt. Dann werdet ihr keine Unterstützung erfahren.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcxiii{Und folgt dem Besten von dem, was zu euch von eurem Herrn herabgesandt worden ist, bevor die Pein plötzlich über euch kommt, ohne daß ihr es merkt.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcxiv{Sonst würde einer sagen: «Was für ein Grund zum Bedauern ist es für mich, daß ich Gott wenig Beachtung schenkte. Ich gehörte ja zu den Spöttern.»\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcxv{Oder er könnte sagen: «Hätte Gott mich rechtgeleitet, wäre ich bestimmt einer der Gottesfürchtigen.»\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcxvi{Oder auch könnte er, wenn er die Pein sieht, sagen: «Hätte ich doch eine Möglichkeit zur Rückkehr, damit ich zu den Rechtschaffenen gehörte!»\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcxvii{- «Aber meine Zeichen sind doch zu dir gekommen. Da hast du sie für Lüge erklärt und dich hochmütig gezeigt, und du bist einer der Ungläubigen gewesen.»\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcxviii{Und am Tag der Auferstehung siehst du die Gesichter derer, die gegen Gott gelogen haben, schwarz werden. Ist nicht in der Hölle eine Bleibe für die Hochmütigen?\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmcxix{Und Gott errettet diejenigen, die gottesfürchtig waren, indem Er sie entrinnen läßt. Das Böse wird sie nicht berühren, und sie werden nicht traurig sein.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmcxx{Gott ist der Schöpfer aller Dinge. Und Er ist der Sachwalter über alle Dinge.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmcxxi{Er hat die Schlüssel der Himmel und der Erde. Diejenigen, die die Zeichen Gottes verleugnen, das sind die Verlierer.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmcxxii{Sprich: Wollt ihr mir wirklich befehlen, einem anderen als Gott zu dienen, ihr Törichten?\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcxxiii{Dir und denen, die vor dir lebten, ist es offenbart worden: «Wenn du (Gott andere) beigesellst, ist dein Werk wertlos und du gehörst zu den Verlierern.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcxxiv{Nein, Gott allein sollst du dienen und zu den Dankbaren gehören.»\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcxxv{Und sie haben Gott nicht eingeschätzt, wie Er eingeschätzt werden soll. Die ganze Erde wird am Tag der Auferstehung in seiner Hand gehalten, und auch die Himmel zusammengefaltet in seiner Rechten. Preis sei Ihm, und erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcxxvi{Und es wird in die Trompete geblasen, und da stürzt, wie vom Blitz getroffen, wer in den Himmeln und wer auf der Erde ist, außer denen, die Gott will. Dann wird ein zweites Mal hineingeblasen, und da stehen sie schon auf und schauen hin.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmcxxvii{Und die Erde erstrahlt im Lichte ihres Herrn. Das Buch wird hingelegt. Die Propheten und die Zeugen werden herbeigebracht. Und es wird zwischen ihnen nach der Wahrheit entschieden, und ihnen wird kein Unrecht getan.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcxxviii{Und jeder Seele wird voll zurückerstattet, was sie getan hat. Und Er weiß besser Bescheid über das, was sie tun.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcxxix{Und diejenigen, die ungläubig waren, werden in Scharen zur Hölle getrieben. Und wenn sie dort ankommen, werden ihre Tore geöffnet. Und ihre Wärter sagen zu ihnen: «Sind nicht Gesandte aus eurer Mitte zu euch gekommen, um euch die Zeichen eures Herrn zu verlesen und euch die Begegnung mit diesem eurem Tag warnend zu verkünden?» Sie sagen: «Ja doch. Aber der Spruch der Pein ist gegen die Ungläubigen zu Recht fällig geworden.»\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmcxxx{Es wird gesprochen: «Betretet die Tore der Hölle, darin werdet ihr ewig weilen. Wie schlimm ist die Bleibe der Hochmütigen!»\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmcxxxi{Und diejenigen, die ihren Herrn fürchteten, werden in Scharen ins Paradies geführt. Und wenn sie dort ankommen, und seine Tore geöffnet werden und seine Wärter zu ihnen sagen: «Friede sei über euch! Gut wart ihr, so betretet es; darin werdet ihr ewig weilen.»...\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmcxxxii{Und sie sagen: «Lob sei Gott, der sein Versprechen an uns wahr gemacht und uns die Erde zum Erbe gegeben hat, sodaß wir uns im Paradies aufhalten können, wo wir wollen! Wie trefflich ist der Lohn derer, die (gut) handeln!»\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmcxxxiii{Und du siehst die Engel sich um den Thron hinstellen und das Lob ihres Herrn singen. Und es ist zwischen ihnen nach der Wahrheit entschieden. Und es wird gesagt: «Lob sei Gott, dem Herrn der Welten!»\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmcxxxiv{\basmalahde Haa Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcxxxv{Herabsendung des Buches von Gott, dem Mächtigen, der Bescheid weiß,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcxxxvi{Der die Sünde vergibt und die Reue annimmt, der strenge Strafe verhängt und dauernde Gnade besitzt. Es gibt keinen Gott außer Ihm. Zu Ihm führt der Lebensweg.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcxxxvii{Über die Zeichen Gottes streiten nur diejenigen, die ungläubig sind. Laß dich durch ihr Umherziehen in den Landstrichen nicht betören.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcxxxviii{Schon vor ihnen haben das Volk Noachs und nach diesen die Parteien (ihre Gesandten) der Lüge geziehen. Jede Gemeinschaft war schon dabei, gegen ihren Gesandten vorzugehen, um ihn zu ergreifen. Und sie stritten mit dem Falschen, um damit die Wahrheit zu widerlegen. Da habe Ich sie ergriffen. Wie war da meine Strafe!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcxxxix{Und so wurde der Spruch deines Herrn gegen diejenigen, die ungläubig sind, zu Recht fällig, nämlich daß sie Gefährten des Feuers sind.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcxl{Diejenigen, die den Thron tragen, und diejenigen, die um ihn stehen, singen das Lob ihres Herrn und glauben an Ihn. Sie bitten um Vergebung für die, die glauben: «Unser Herr, Du umschließt alle Dinge in deiner Barmherzigkeit und deinem Wissen. So vergib denen, die sich bekehrt haben und deinem Weg gefolgt sind, und bewahre sie vor der Pein der Hölle.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcxli{Unser Herr, laß sie in die Gärten Edens, die Du ihnen verheißen hast, eingehen, ebenso die Rechtschaffenen von ihren Vätern, ihren Gattinnen und ihren Nachkommen, Du bist der Mächtige, der Weise.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcxlii{Und bewahre sie vor den Missetaten. An jenem Tag, wen Du vor den Missetaten bewahrst, dessen hast Du dich erbarmt, und das ist der großartige Erfolg.»\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcxliii{Denen, die ungläubig waren, wird zugerufen: «Der Abscheu Gottes ist gewiß größer als euer Abscheu vor euch selbst, da ihr zum Glauben aufgerufen worden, aber ungläubig geblieben seid.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcxliv{Sie sagen: «Unser Herr, Du hast uns zweimal sterben lassen und zweimal lebendig gemacht. So bekennen wir unsere Sünden. Gibt es denn einen Weg, hier herauszukommen?»\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcxlv{«Dies ist so, weil ihr, wenn immer Gott allein angerufen wurde, ungläubig geblieben seid, ihr aber, wenn Ihm (andere) beigesellt wurden, geglaubt habt. Das Urteil gehört Gott, dem Erhabenen, dem Großen.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcxlvi{Er ist es, der euch seine Zeichen sehen läßt und euch vom Himmel einen Lebensunterhalt herabsendet. Und doch bedenkt es nur der, der zur Umkehr bereit ist.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcxlvii{So ruft Gott an, indem ihr Ihm gegenüber aufrichtig in der Religion seid, auch wenn es den Ungläubigen zuwider ist.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcxlviii{Er hat hohe Rangstufen und ist Herr des Thrones. Er legt den Geist von seinem Befehl, auf wen von seinen Dienern Er will, damit er vor dem Tag der Begegnung warne,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcxlix{Dem Tag, an dem sie vortreten, wobei nichts von ihnen vor Gott verborgen bleibt. - «Wem gehört heute die Königsherrschaft?» - Gott, dem Einen, der bezwingende Macht besitzt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcl{Heute wird jeder Seele vergolten für das, was sie erworben hat. Heute geschieht kein Unrecht. Gott ist schnell im Abrechnen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcli{Und warne sie vor dem Tag der bevorstehenden Auferstehung, da die Herzen voller Gram die Kehle erreichen. Die, die Unrecht tun, haben dann keinen warmherzigen Freund und keinen Fürsprecher, dem man gehorchen würde.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmclii{Er weiß, was sich hinter den Augen an Verrat versteckt und was die Brust verbirgt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcliii{Und Gott entscheidet nach der Wahrheit. Diejenigen aber, die sie an seiner Stelle anrufen, entscheiden ohne jede Grundlage. Gott ist der, der alles hört und sieht.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcliv{Sind sie denn nicht auf der Erde umhergegangen und haben geschaut, wie das Ende derer war, die vor ihnen lebten? Sie hatten eine stärkere Kraft und hinterließen mehr Spuren auf der Erde als sie. Da ergriff sie Gott wegen ihrer Sünden. Und es gab für sie niemanden, der sie vor Gott hätte schützen können.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmclv{Dies, weil ihre Gesandten immer wieder mit den deutlichen Zeichen zu ihnen kamen, sie aber ungläubig blieben. So ergriff sie Gott. Er ist stark und verhängt eine harte Strafe.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmclvi{Und Wir haben Mose gesandt mit unseren Zeichen und mit einer offenkundigen Ermächtigung\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmclvii{Zu Pharao und Haamaan und Qarun. Sie aber sagten: «Ein Zauberer, der lügt.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmclviii{Als er zu ihnen mit der Wahrheit von Uns kam, sagten sie: «Tötet die Söhne derer, die mit ihm glauben, und laßt ihre Frauen am Leben.» Aber die List der Ungläubigen geht gewiß ins Leere.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmclix{Pharao sagte: «Laßt mich Mose töten. Er soll seinen Herrn anrufen. Ich fürchte, er wird sonst eure Religion abändern oder Unheil im Land hervorrufen.»\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmclx{Und Mose sagte: «Ich suche Zuflucht bei meinem Herrn und eurem Herrn vor jedem Hochmütigen, der nicht an den Tag der Abrechnung glaubt.»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmclxi{Ein gläubiger Mann von den Leuten des Pharao, der seinen Glauben verschwieg, sagte: «Wollt ihr denn einen Mann töten, weil er sagt: >Mein Herr ist Gott\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmclxii{O mein Volk, zwar gehört euch heute die Herrschaft, und ihr habt die Oberhand im Land. Wer wird uns aber gegen die Schlagkraft Gottes unterstützen, wenn sie über uns kommt?» Pharao sagte: «Ich zeige euch nur, was ich sehe, und ich weise euch nur den Weg des rechten Wandels.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmclxiii{Der, der glaubte, sagte: «O mein Volk, ich fürchte für euch etwas Ähnliches wie den Tag der Parteien,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmclxiv{Etwas Ähnliches wie das, was dem Volk Noachs, den Aad, den Thamud und denen, die nach ihnen lebten, widerfuhr. Und Gott will den Dienern kein Unrecht tun.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmclxv{O mein Volk, ich fürchte für euch den Tag des gegenseitigen Zurufens,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmclxvi{Den Tag, an dem ihr den Rücken kehren werdet. Dabei werdet ihr niemanden haben, der euch vor Gott schützen könnte. Und wen Gott irreführt, der hat niemanden, der ihn rechtleiten könnte.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmclxvii{Früher kam Joseph zu euch mit den deutlichen Zeichen. Ihr aber verharrtet in eurem Zweifel über das, was er euch gebracht hatte. Als er dann starb, sagtet ihr: >Gott wird nach ihm keinen Gesandten mehr erstehen lassen. < So führt Gott in die Irre den, der maßlos und voller Zweifel ist.»\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmclxviii{Diejenigen, die über die Zeichen Gottes streiten, ohne eine Ermächtigung erhalten zu haben, erregen damit großen Abscheu bei Gott und bei denen, die gläubig sind. So versiegelt Gott das Herz eines jeden, der hochmütig und gewalttätig ist.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmclxix{Und Pharao sagte: «O Haamaan, errichte mir einen offenen Hochbau, vielleicht kann ich die Gänge erreichen,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmclxx{Die Gänge der Himmel, daß ich zum Gott des Mose emporsteige. Ich halte ihn ja für einen Lügner.» So wurde dem Pharao sein böses Tun verlockend gemacht, und er wurde vom (rechten) Weg abgewiesen. Die List Pharaos ist dem Verderben erlegen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmclxxi{Derjenige, der gläubig war, sagte: «O mein Volk, folgt mir, dann weise ich euch den Weg des rechten Wandels.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmclxxii{O mein Volk, dieses irdische Leben ist nur Nutznießung. Das Jenseits aber ist die Wohnstätte zum Bleiben.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmclxxiii{Wer etwas Schlechtes tut, dem wird gleichviel vergolten. Und diejenigen, die Gutes tun, ob Mann oder Weib, und dabei gläubig sind, werden ins Paradies eingehen, wo ihnen Unterhalt beschert wird ohne Abrechnung.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmclxxiv{O mein Volk, wie kommt es, daß ich euch zur Rettung rufe, ihr aber mich zum Feuer ruft?\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmclxxv{Ihr ruft mich dazu auf, Gott zu verleugnen und Ihm das beizugesellen, wovon ich kein Wissen habe. Ich aber rufe euch zum Mächtigen, der voller Vergebung ist.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmclxxvi{Wahrlich, es steht dem, zu dem ihr mich ruft, keine Anrufung zu, weder im Diesseits noch im Jenseits. Zu Gott werden wir ja zurückgebracht. Die Maßlosen sind die Gefährten des Feuers.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmclxxvii{Ihr werdet euch an das erinnern, was ich euch sage. Ich überlasse Gott meine Angelegenheit. Gott sieht wohl die Diener.»\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmclxxviii{So bewahrte ihn Gott vor ihrer bösen Arglist, und die Leute Pharaos umschloß die böse Pein,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmclxxix{Das Feuer, dem sie morgens und abends vorgeführt werden. Und am Tag, da die Stunde heraufkommt (heißt es): «Laßt die Leute Pharaos in die härteste Pein eingehen.»\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmclxxx{Wenn sie nun im Feuer miteinander streiten. Dann sagen die Schwachen zu denen, die sich hochmütig verhielten: «Wir waren doch eure Gefolgsleute. Könnt ihr nun einen Teil des Feuers von uns fernhalten?»\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmclxxxi{Diejenigen, die sich hochmütig verhielten, sagen: «Wir befinden uns doch alle darin. Gott hat nun zwischen den Dienern geurteilt.»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmclxxxii{Diejenigen, die sich im Feuer befinden, sagen zu den Wärtern der Hölle: «Ruft zu eurem Herrn, daß Er uns einen Tag der Erleichterung von der Pein gewähre.»\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmclxxxiii{Sie sagen: «Sind nicht eure Gesandten immer wieder mit den deutlichen Zeichen zu euch gekommen?» Sie sagen: «Doch.» Sie sagen: «So ruft ihr selbst.» Aber das Rufen der Ungläubigen geht gewiß ins Leere.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmclxxxiv{Wahrlich, Wir unterstützen unsere Gesandten und diejenigen, die glauben, im diesseitigen Leben und am Tag, da die Zeugen auftreten,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmclxxxv{Am Tag, da denen, die Unrecht tun, ihre Entschuldigung nicht nützt. Sie werden den Fluch und sie werden eine schlimme Wohnstätte erhalten.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmclxxxvi{Wir haben Mose die Rechtleitung zukommen und die Kinder Israels das Buch erben lassen\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmclxxxvii{Als Rechtleitung und Ermahnung für die Einsichtigen.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmclxxxviii{So sei geduldig. Das Versprechen Gottes ist wahr. Und bitte um Vergebung für deine Sünde. Und sing das Lob deines Herrn am Abend und am frühen Morgen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmclxxxix{Diejenigen, die über die Zeichen Gottes streiten, ohne daß sie eine Ermächtigung erhalten hätten, haben in ihrer Brust nichts als Überheblichkeit. Sie werden sie nicht befriedigen können. So suche Zuflucht bei Gott. Er ist der, der alles hört und sieht.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcxc{Die Erschaffung der Himmel und der Erde ist wahrlich größer als die Erschaffung der Menschen. Aber die meisten Menschen wissen nicht Bescheid.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcxci{Nicht gleich sind der Blinde und der Sehende, und auch nicht diejenigen, die glauben und die guten Werke tun, und der Missetäter. Aber ihr bedenkt es wenig.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcxcii{Die Stunde kommt bestimmt, an ihr ist kein Zweifel möglich. Aber die meisten Menschen glauben nicht.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcxciii{Euer Herr spricht: «Ruft zu Mir, so erhöre Ich euch. Diejenigen, die sich aus Hochmut weigern, Mir zu dienen, werden in die Hölle als Gedemütigte eingehen.»\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmcxciv{Gott ist es, der euch die Nacht gemacht hat, damit ihr in ihr ruht, und den Tag, an dem man sehen kann. Gott ist voller Huld gegen die Menschen. Aber die meisten Menschen sind nicht dankbar.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmcxcv{So ist Gott, euer Herr, der Schöpfer aller Dinge. Es gibt keinen Gott außer Ihm. Wie leicht laßt ihr euch doch abwenden!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmcxcvi{Auf diese Weise lassen sich abwenden, die die Zeichen Gottes zu verleugnen pflegen.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmcxcvii{Gott ist es, der euch die Erde zu einem festen Grund und den Himmel zu einem Bau gemacht, euch gestaltet und eure Gestalten schön geformt und euch von den köstlichen Dingen beschert hat. So ist Gott, euer Herr. Gesegnet sei Gott, der Herr der Welten!\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcxcviii{Er ist der Lebendige. Es gibt keinen Gott außer Ihm. So ruft zu Ihm, indem ihr Ihm gegenüber aufrichtig in der Religion seid. Lob sei Gott, dem Herrn der Welten!\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcxcix{Sprich: Mir wurde, als die deutlichen Zeichen von meinem Herrn zu mir kamen, verboten, denen zu dienen, die ihr anstelle Gottes anruft. Und mir wurde befohlen, mich dem Herrn der Welten zu ergeben.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcc{Er ist es, der euch aus Erde erschaffen hat, dann aus einem Tropfen, dann aus einem Embryo. Dann läßt Er euch als Kind herauskommen. Dann (sorgt Er für euch), damit ihr eure Vollkraft erreicht und damit ihr dann Greise werdet - unter euch sind (allerdings) einige, die Er vorher abberuft - und damit ihr eine bestimmte Frist erreicht, auf daß ihr verständig werdet.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcci{Er ist es, der lebendig macht und sterben läßt. Und wenn Er eine Sache beschlossen hat, sagt Er zu ihr nur: «Sei!», und sie ist.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmccii{Hast du nicht auf jene geschaut, die über die Zeichen Gottes streiten? Wie leicht lassen sie sich doch abbringen!\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcciii{Sie, die das Buch und das, womit Wir unsere Gesandten geschickt haben, für Lüge erklären. Sie werden es zu wissen bekommen,\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcciv{Wenn die Fesseln und die Ketten an ihrem Hals angebracht werden und sie hineingezerrt werden\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmccv{Ins heiße Wasser und dann ins Feuer als Brennstoff geworfen werden.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmccvi{Dann wird zu ihnen gesagt: «Wo ist das, was ihr (Gott) beigesellt habt (und verehrt habt)\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmccvii{Anstelle Gottes?» Sie sagen: «Sie sind uns entschwunden. Nein, wir riefen früher nichts an.» So führt Gott die Ungläubigen in die Irre.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmccviii{Dies, weil ihr euch auf der Erde zu Unrecht der Freude hinzugeben und unbekümmert zu leben pflegtet.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmccix{Betretet nun die Tore der Hölle, darin werdet ihr ewig weilen. Schlimm ist die Bleibe der Hochmütigen.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmccx{So sei geduldig. Das Versprechen Gottes ist wahr. Ob Wir dich einen Teil dessen, was Wir ihnen androhen, sehen lassen oder dich abberufen, zu Uns werden sie (auf jeden Fall) zurückgebracht.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmccxi{Und Wir haben schon vor dir Gesandte geschickt. Unter ihnen sind manche, von denen Wir dir erzählt haben, und unter ihnen sind manche, von denen Wir dir nicht erzählt haben. Und kein Gesandter kann ein Zeichen bringen, außer mit der Erlaubnis Gottes. Wenn der Befehl Gottes eintrifft, wird nach der Wahrheit entschieden, und dann haben diejenigen den Verlust, die (die Botschaft) für falsch erklären.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmccxii{Gott ist es, der für euch die Herdentiere bestellt hat, damit ihr auf ihnen reiten könnt; und ihr könnt auch von ihnen essen.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmmccxiii{An ihnen habt ihr sonst allerlei Nutzen. Und damit ihr, auf ihnen reitend, etwas, was ihr in eurer Brust hegt und anstrebt, erreichen könnt. (Endlich) werdet ihr auf ihnen und auf den Schiffen getragen.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmccxiv{Und Er zeigt euch seine Zeichen. Welches von den Zeichen Gottes wollt ihr nun verleugnen?\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmccxv{Sind sie denn nicht auf der Erde umhergegangen und haben geschaut, wie das Ende derer war, die vor ihnen lebten? Sie waren zahlreicher als sie, hatten eine stärkere Kraft und hinterließen mehr Spuren auf der Erde. Und nicht nützte ihnen, was sie zu erwerben pflegten.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmccxvi{Als nun ihre Gesandten mit den deutlichen Zeichen zu ihnen kamen, freuten sie sich über das Wissen, das sie besaßen. Und es umschloß sie das, worüber sie spotteten.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmccxvii{Als sie dann unsere Schlagkraft sahen, sagten sie: «Wir glauben an Gott allein und verleugnen das, was wir Ihm beigesellten.»\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmccxviii{Aber ihr Glaube konnte ihnen nicht nützen, als sie unsere Schlagkraft sahen - so ist Gott mit seinen Dienern in der Vergangenheit verfahren. Den Verlust hatten da die Ungläubigen.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmccxix{\basmalahde Haa Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmccxx{Herabsendung von dem Erbarmer, dem Barmherzigen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmccxxi{Ein Buch, dessen Zeichen im einzelnen dargelegt sind, als arabischer Koran, für Leute, die Bescheid wissen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmccxxii{Als Freudenbote und Warner. Aber die meisten von ihnen wenden sich ab, so daß sie nicht hören.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmccxxiii{Sie sagen: «Unsere Herzen liegen vor dem, wozu du uns aufrufst, in Hüllen, und in unseren Ohren ist Schwerhörigkeit, und zwischen uns und dir ist ein Vorhang. Handle also (wie du magst), wir werden auch ebenfalls handeln.»\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmccxxiv{Sprich: Ich bin nur ein Mensch wie ihr; mir wird offenbart, daß euer Gott ein einziger Gott ist. So verhaltet euch Ihm gegenüber recht und bittet Ihn um Vergebung. Und wehe den Polytheisten,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmccxxv{Die die Abgabe nicht entrichten und das Jenseits verleugnen!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmccxxvi{Für die, die glauben und die guten Werke tun, ist ein Lohn bestimmt, der nicht aufhört.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmccxxvii{Sprich: Wollt ihr wirklich den verleugnen, der die Erde in zwei Tagen erschaffen hat, und Ihm andere als Gegenpart zur Seite stellen? Das ist doch der Herr der Welten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmccxxviii{Er hat auf ihr festgegründete Berge gemacht, die über ihr aufragen, und Er hat sie gesegnet und in ihr Nahrung nach Maß festgelegt - in vier Tagen - gleichmäßig für die, die danach verlangen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmccxxix{Dann richtete Er sich zum Himmel auf, welcher noch aus Rauch bestand, und sprach zu ihm und zur Erde: «Kommt her, freiwillig oder widerwillig.» Sie sagten: «Wir kommen freiwillig.»\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmccxxx{So gestaltete Er sie nach seiner Entscheidung zu sieben Himmeln in zwei Tagen und wies jedem Himmel seine Aufgabe zu. Und Wir schmückten den untersten Himmel mit Leuchten, (die auch) zum Schutz (dienen). Das ist die Bestimmung dessen, der mächtig ist und Bescheid weiß.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmccxxxi{Wenn sie sich also abwenden, dann sprich: Ich warne euch vor einem Donnerschlag gleich dem Donnerschlag der Aad und der Thamud.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmccxxxii{Als die Gesandten zu ihnen kamen von vorn und von hinten: «Dienet Gott allein», sagten sie: «Wenn unser Herr gewollt hätte, hätte Er Engel herabgesandt. So verleugnen wir das, womit ihr gesandt worden seid.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmccxxxiii{Was die Aad betrifft, so verhielten sie sich auf der Erde zu Unrecht hochmütig und sagten: «Wer hat eine stärkere Kraft als wir?» Haben sie denn nicht gesehen, daß Gott, der sie erschaffen hat, eine stärkere Kraft hat als sie? Aber sie verleugneten dauernd unsere Zeichen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmccxxxiv{Da sandten Wir gegen sie an unheilvollen Tagen einen eiskalten Wind, um sie die Pein der Schande im diesseitigen Leben kosten zu lassen. Aber die Pein des Jenseits ist noch schändlicher. Und sie werden keine Unterstützung erfahren.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmccxxxv{Und was die Thamud betrifft, so wollten Wir sie rechtleiten, sie aber zogen die Blindheit der Rechtleitung vor. Da ergriff sie der Donnerschlag der schändlichen Pein wegen dessen, was sie zu erwerben pflegten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmccxxxvi{Und Wir erretteten diejenigen, die glaubten und gottesfürchtig waren.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmccxxxvii{Und am Tag, da die Feinde Gottes zum Feuer versammelt und in Abteilungen aufgestellt werden.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmccxxxviii{Wenn sie dann dort angekommen sind, legen ihr Gehör, ihre Augen und ihre Häute gegen sie Zeugnis ab über das, was sie zu tun pflegten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmccxxxix{Sie sagen zu ihren Häuten: «Warum habt ihr gegen uns Zeugnis abgelegt?» Sie sagen: «Gott, der alle Dinge reden läßt, hat uns reden lassen. Er ist es, der euch das erste Mal erschaffen hat, und zu Ihm werdet ihr zurückgebracht.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmccxl{Und ihr pflegtet euch nicht so zu verdecken, daß nicht euer Gehör, eure Augen und eure Häute Zeugnis wegen euch ablegten. Aber ihr meintet, daß Gott nicht viel wisse von dem, was ihr tut.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmccxli{Und diese eure Meinung, die ihr von eurem Herrn hattet, hat euch ins Verderben gestürzt, so daß ihr jetzt zu den Verlierern gehört.»\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmccxlii{Wenn sie Geduld zeigen, so ist doch das Feuer ihre Bleibe. Und wenn sie eine entschuldigende Umkehr zeigen, so werden sie doch nicht zu denen gehören, deren entschuldigende Umkehr angenommen wird.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmccxliii{Und Wir haben für sie Gesellen abgestellt, die ihnen verlockend machten, was vor ihnen und was hinter ihnen lag. Und fällig wurde der Spruch gegen sie, wie gegen Gemeinschaften von Djinn und Menschen, die vor ihnen dahingegangen waren. Sie waren ja Verlierer.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmccxliv{Diejenigen, die ungläubig sind, sagen: «Hört nicht auf diesen Koran, und führt dazwischen unbedachte Reden, auf daß ihr siegt.»\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmccxlv{Wir werden diejenigen, die ungläubig sind, bestimmt eine harte Pein kosten lassen, und Wir werden ihnen vergelten das Schlimmste von dem, was sie zu tun pflegten.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmccxlvi{Das ist der Lohn der Feinde Gottes: das Feuer, in dem sie eine ewige Wohnstätte haben, als Vergeltung dafür, daß sie unsere Zeichen immer wieder verleugnet haben.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmccxlvii{Und diejenigen, die ungläubig sind, sagen: «Unser Herr, laß uns die zwei Gruppen - von den Djinn und den Menschen - sehen, die uns irregeführt haben, daß wir sie unter unsere Füße legen, damit sie zu den Unterlegenen gehören.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmccxlviii{Auf diejenigen, die sagen: «Unser Herr ist Gott», und sich dann recht verhalten, kommen die Engel herab: «Fürchtet euch nicht, seid nicht traurig und freut euch auf das Paradies, das euch immer wieder versprochen wurde.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmccxlix{Wir sind eure Freunde im diesseitigen Leben und im Jenseits. Ihr habt darin, was eure Seele begehrt, und ihr habt darin, was ihr beansprucht,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmccl{- eine Herberge von einem, der voller Vergebung und barmherzig ist.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmccli{Und wer spricht bessere Worte als der, der zu Gott ruft, Gutes tut und sagt: «Ich gehöre zu den Gottergebenen»?\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcclii{Nicht gleich sind die gute und die schlechte Tat. Wehre ab mit einer Tat, die besser ist, da wird der, zwischen dem und dir eine Feindschaft besteht, so, als wäre er ein warmherziger Freund.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmccliii{Aber dies wird nur denen verliehen, die geduldig sind, ja es wird nur dem verliehen, der ein gewaltiges Glück hat.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmccliv{Und wenn dich vom Satan ein Stachel aufstachelt, dann suche Zuflucht bei Gott. Er ist der, der alles hört und weiß.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcclv{Und zu seinen Zeichen gehören die Nacht und der Tag, die Sonne und der Mond. Werft euch weder vor der Sonne noch vor dem Mond nieder. Werft euch nieder vor Gott, der sie erschaffen hat, so ihr Ihm dienen wollt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcclvi{Wenn sie sich hochmütig weigern - diejenigen, die bei deinem Herrn sind, preisen Ihn doch Nacht und Tag, ohne Überdruß zu empfinden.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcclvii{Zu seinen Zeichen gehört es, daß du die Erde regungslos siehst. Wenn Wir aber Wasser auf sie herabkommen lassen, regt sie sich und wächst zu. Der sie wieder belebt, wird die Toten wieder lebendig machen. Er hat Macht zu allen Dingen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcclviii{Diejenigen, die über unsere Zeichen abwegig denken, sind Uns nicht verborgen. Ist der, der ins Feuer geworfen wird, besser oder der, der am Tag der Auferstehung in Sicherheit herantritt. Tut, was ihr wollt; Er sieht wohl, was ihr tut.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcclix{Diejenigen, die die Ermahnung verleugnen, nachdem sie zu ihnen gekommen ist... Es ist ein hehres Buch,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcclx{An das das Falsche weder von vorn noch von hinten herankommt, eine Herabsendung von einem, der weise und des Lobes würdig ist.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcclxi{Es wird dir nur das gesagt, was schon den Gesandten vor dir gesagt wurde. Dein Herr schenkt Vergebung und verhängt eine schmerzhafte Strafe.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcclxii{Hätten Wir ihn zu einem fremdsprachigen Koran gemacht, hätten sie gesagt: «Wären doch seine Zeichen im einzelnen dargelegt worden! Wie, ein fremdsprachiger (Koran) und ein Araber!» Sprich: Er ist für diejenigen, die glauben, eine Rechtleitung und eine Heilung. Diejenigen, die nicht glauben, haben Schwerhörigkeit in ihren Ohren, und er ist ihrem Blick entzogen. Diese sind, als würde ihnen aus einem fernen Ort zugerufen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcclxiii{Und Wir ließen dem Mose das Buch zukommen. Da wurde man darüber uneins. Und gäbe es nicht einen früher ergangenen Spruch von deinem Herrn, so wäre zwischen ihnen entschieden worden. Und sie sind darüber in einem starken Zweifel.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcclxiv{Wer Gutes tut, tut es zu seinem eigenen Vorteil. Und wer Böses tut, tut es zu seinem eigenen Schaden. Und dein Herr tut den Dienern kein Unrecht.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcclxv{Ihm ist das Wissen um die Stunde (des Gerichtes) anheimgestellt. Keine Früchte kommen aus ihren Hüllen hervor und kein Weib empfängt und kommt nieder, es sei denn mit seinem Wissen. Und am Tag, da Er ihnen zuruft: «Wo sind denn meine Teilhaber?», sagen sie: «Wir geben Dir bekannt, es gibt keinen unter uns, der davon zeugen würde.»\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcclxvi{Und ihnen ist entschwunden, was sie früher anzurufen pflegten, und sie erwarten, daß es für sie kein Entrinnen gibt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcclxvii{Der Mensch wird nicht überdrüssig, um das Gute zu bitten. Wenn das Böse ihn berührt, dann ist er verzweifelt und gibt die Hoffnung auf.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcclxviii{Und wenn Wir ihn eine Barmherzigkeit von Uns kosten lassen, nachdem ihn ein Schaden berührt hat, sagt er bestimmt: «Das steht mir zu. Und ich glaube nicht, daß die Stunde heraufkommen wird. Und wenn ich zu meinem Herrn zurückgebracht werde, werde ich sicher das Beste bei Ihm erhalten.» Doch Wir werden denen, die ungläubig sind, kundtun, was sie getan haben, und sie eine schwere Pein kosten lassen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcclxix{Und wenn Wir dem Menschen Gnade erweisen, wendet er sich ab und entfernt sich beiseite. Und wenn das Böse ihn berührt, ergeht er sich in ausführlichem Beten.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcclxx{Sprich: Was meint ihr, wenn er doch von Gott stammt und ihr dann ihn doch verleugnet? Wer ist weiter abgeirrt als der, der sich in einem tiefen Widerstreit befindet?\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcclxxi{Wir werden sie an den Horizonten und in ihnen selbst unsere Zeichen sehen lassen, bis es ihnen deutlich wird, daß es die Wahrheit ist. Genügt es denn nicht, daß dein Herr Zeuge ist über alle Dinge?\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcclxxii{Aber siehe, sie hegen Zweifel über die Begegnung mit ihrem Herrn. Siehe, Er umfaßt doch alle Dinge.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcclxxiii{\basmalahde Haa Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcclxxiv{Ayn Sien Qaaf.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcclxxv{So offenbart Gott dir, so wie (früher) denjenigen, die vor dir lebten, (Er), der Mächtige, der Weise.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcclxxvi{Ihm gehört, was in den Himmeln und was auf der Erde ist, und Er ist der Erhabene, der Majestätische.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcclxxvii{Die Himmel brechen bald auseinander von oben her. Und die Engel singen das Lob ihres Herrn und bitten Ihn um Vergebung für die, die auf der Erde sind. Ja, Gott ist der, der voller Vergebung und barmherzig ist.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcclxxviii{Über diejenigen, die sich an seiner Stelle Freunde nehmen, ist Gott Hüter, und nicht bist du ihr Sachwalter.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcclxxix{Und so haben Wir dir einen arabischen Koran offenbart, damit du die Mutter der Städte und die Menschen in ihrer Umgebung warnest, und damit du vor dem Tag der Versammlung warnest; an ihm ist kein Zweifel möglich. Ein Teil wird im Paradies sein und ein Teil im Feuerbrand.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcclxxx{Und wenn Gott gewollt hätte, hätte Er sie zu einer einzigen Gemeinschaft gemacht. Aber Er läßt in seine Barmherzigkeit eingehen, wen Er will. Und die, die Unrecht tun, haben weder Freund noch Helfer.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxi{Oder haben sie sich an seiner Stelle Freunde genommen? Gott (allein) ist doch der Freund. Er macht die Toten wieder lebendig, und Er hat Macht zu allen Dingen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxii{Und worüber ihr auch uneins seid, das Urteil darüber steht Gott (allein) zu. Das ist eben Gott, mein Herr. Auf Ihn vertraue ich, und zu Ihm wende ich mich reumütig.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxiii{Er ist) der Schöpfer der Himmel und der Erde. Er hat euch aus euch selbst Gattinnen gemacht, und auch aus den Tieren Paare, so daß Er euch dadurch vermehrt. Nichts ist Ihm gleich. Er ist der, der alles hört und sieht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxiv{Er hat die Schlüssel der Himmel und der Erde. Er teilt den Lebensunterhalt großzügig, wem Er will, und auch bemessen zu. Siehe, Er weiß über alle Dinge Bescheid.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxv{Er hat euch von der Religion verordnet, was Er Noach aufgetragen hat, und was Wir dir offenbart haben, und was Wir Abraham, Mose und Jesus aufgetragen haben: Haltet die (Bestimmungen der) Religion ein und bringt keine Spaltungen hinein. Den Polytheisten fällt das schwer, wozu du sie aufrufst. Gott erwählt dazu, wen Er will, und Er führt dazu, wer sich (Ihm) reumütig zuwendet.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxvi{Und sie spalteten sich darin erst, nachdem das Wissen zu ihnen gekommen war, dies aus ungerechter Auflehnung untereinander. Und gäbe es nicht einen früher ergangenen Spruch von deinem Herrn auf eine bestimmte Frist, so wäre zwischen ihnen entschieden worden. Siehe, diejenigen, denen nach ihnen das Buch zum Erbe übergeben wurde, hegen darüber starken Zweifel.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxvii{Darum rufe du auf und verhalte dich recht, wie dir befohlen worden ist. Und folge nicht ihren Neigungen, sondern sprich: Ich glaube an das, was Gott an Büchern herabgesandt hat, und mir ist befohlen worden, unter euch Gerechtigkeit zu üben. Gott ist unser Herr und euer Herr. Wir haben unsere Werke, und ihr habt eure Werke (zu verantworten). Es gibt keinen Streitgrund zwischen uns und euch. Gott wird uns zusammenbringen. Und zu Ihm führt der Lebensweg.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxviii{Diejenigen, die über Gott streiten, nachdem auf Ihn gehört worden ist, deren Streitgrund wird bei ihrem Herrn widerlegt. Auf ihnen liegt Zorn, und bestimmt ist für sie eine harte Pein.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxix{Gott ist es, der das Buch mit der Wahrheit herabgesandt hat, und auch die Waage. Woher willst du es wissen? Vielleicht steht die Stunde nahe bevor.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmccxc{Diejenigen, die nicht an sie glauben, wünschen, sie zu beschleunigen. Die aber, die glauben, sind erschrocken vor ihr und wissen, daß sie Wirklichkeit ist. Aber diejenigen, die über die Stunde streiten, befinden sich in tiefem Irrtum.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmccxci{Gott ist feinfühlig zu seinen Dienern. Er beschert Lebensunterhalt, wem Er will. Und Er ist der Starke, der Mächtige.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmccxcii{Wer die Saaternte des Jenseits haben will, dem mehren Wir noch seine Saaternte. Und wer die Saaternte des Diesseits haben will, dem lassen Wir etwas davon zukommen, er erhält aber im Jenseits keinen Anteil.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmccxciii{Oder haben sie etwa Teilhaber, die ihnen als Religion verordnet haben, was Gott nicht erlaubt hat? Und gäbe es nicht den Urteilsspruch, wäre es zwischen ihnen entschieden worden. Und für die, die Unrecht tun, ist eine schmerzhafte Pein bestimmt.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmccxciv{Du siehst dann die, die Unrecht tun, erschrocken vor dem, was sie erworben haben, und es wird über sie hereinbrechen. Diejenigen aber, die glauben und die guten Werke tun, befinden sich in den Gärten des Paradieses. Sie haben bei ihrem Herrn, was sie wollen. Das ist die große Huld.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmccxcv{Das ist die Frohbotschaft, die Gott seinen Dienern, die glauben und die guten Werke tun, verkündet. Sprich: Ich verlange von euch keinen Lohn dafür, es sei denn die Liebe wie zu den Verwandten. Und wer ein gutes Werk tut, dem schenken Wir dafür noch mehr Gutes. Gott ist voller Vergebung und zeigt sich erkenntlich.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmccxcvi{Oder sagen sie etwa: «Er hat eine Lüge gegen Gott erdichtet»? Wenn nun Gott will, kann Er dir das Herz versiegeln. Und Gott löscht das Falsche aus und bestätigt die Wahrheit mit seinen Worten. Er weiß über das innere Geheimnis Bescheid.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmccxcvii{Er ist es, der die Umkehr von seinen Dienern annimmt und die Missetaten verzeiht. Und Er weiß, was ihr tut.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmccxcviii{Und Er erhört diejenigen, die glauben und die guten Werke tun, und gibt ihnen noch mehr von seiner Huld. Für die Ungläubigen aber ist eine harte Pein bestimmt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmccxcix{Und würde Gott seinen Dienern den Lebensunterhalt großzügig zuteilen, würden sie auf der Erde Ungerechtigkeit verüben. Aber Gott läßt im richtigen Maße herabkommen, was Er will. Er hat Kenntnis von seinen Dienern und sieht sie wohl.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmccc{Und Er ist es, der den Regen herabkommen läßt, nachdem sie die Hoffnung aufgegeben haben, und der seine Barmherzigkeit ausbreitet. Und Er ist der Freund und des Lobes würdig.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmccci{Und zu seinen Zeichen gehört es, daß Er die Himmel und die Erde erschaffen hat und in ihnen beiden allerlei Getier sich ausbreiten läßt. Und Er hat die Macht, sie zu versammeln, wenn Er will.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmcccii{Und was immer euch an Unglück trifft, es ist für das, was eure I lande erworben haben. Und Er übergeht wohl vieles.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmccciii{Ihr könnt auf der Erde nichts vereiteln. Und ihr habt außer Gott weder Freund noch Helfer.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmccciv{Und zu seinen Zeichen gehören die Schiffe, die wie Berge auf dem Meer fahren.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmcccv{Wenn Er will, läßt Er den Wind sich legen, so daß sie nun bewegungslos auf der Wasserfläche stehenbleiben. Darin sind Zeichen für jeden, der sehr geduldig und sehr dankbar ist.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcccvi{Oder Er läßt sie untergehen für das, was sie erworben haben. Und Er übergeht wohl vieles.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmcccvii{Deshalb sollen diejenigen, die über unsere Zeichen streiten, wissen, daß es für sie kein Entrinnen gibt.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmcccviii{Und was immer euch zuteil geworden ist, ist Nutznießung des diesseitigen Lebens. Was aber bei Gott ist, ist besser und hat eher Bestand für die, die glauben und auf ihren Herrn vertrauen,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcccix{Die die schweren Sünden und die schändlichen Taten meiden und, wenn sie in Zorn geraten, (lieber) vergeben,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcccx{Und die auf ihren Herrn hören und das Gebet verrichten, ihre Angelegenheiten durch Beratung regeln und von dem, was Wir ihnen beschert haben, spenden,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcccxi{Und die, wenn ihnen Ungerechtigkeit widerfährt, sich selbst helfen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcccxii{Eine schlechte Tat soll mit etwas gleich Bösem vergolten werden. Wer aber verzeiht und Besserung schafft, dessen Lohn obliegt Gott. Er liebt ja die nicht, die Unrecht tun.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcccxiii{Und die, die sich selbst helfen, nachdem ihnen Unrecht getan wurde, können nicht belangt werden.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcccxiv{Belangt werden die, die den Menschen Unrecht tun und auf der Erde unberechtigterweise ungerecht handeln. Für sie ist eine schmerzhafte Pein bestimmt.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcccxv{Wahrlich, wenn einer geduldig ist und vergibt, so gehört dies zur Entschlossenheit in den Anliegen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcccxvi{Wen Gott irreführt, der hat daraufhin keinen Freund. Und du siehst, wie die, die Unrecht tun, wenn sie die Pein sehen, sagen: «Gibt es denn einen Weg, zurückzukehren?»\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcccxvii{Und du siehst, wie sie ihm vorgeführt werden, demütig vor Erniedrigung und mit verstohlenen Blicken dreinschauend. Und diejenigen, die glauben, sagen: «Die (wahren) Verlierer sind die, die am Tag der Auferstehung sich selbst und ihre Angehörigen verlieren.» Ja, die, die Unrecht tun, befinden sich in einer beständigen Pein.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcccxviii{Und sie haben keine Freunde, die sie anstelle Gottes unterstützen könnten. Wen Gott irreführt, für den gibt es keinen Ausweg.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcccxix{Hört auf euren Herrn, bevor ein Tag kommt, der von Seiten Gottes nicht zurückgewiesen wird. An jenem Tag habt ihr keinen Zufluchtsort und keine Möglichkeit, (irgend etwas) abzustreiten.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcccxx{Wenn sie sich abwenden, so haben Wir dich nicht als Hüter über sie gesandt. Dir obliegt nur die Ausrichtung (der Botschaft). Und siehe, wenn Wir den Menschen Barmherzigkeit von Uns kosten lassen, freut er sich darüber. Wenn aber etwas Böses sie trifft für das, was ihre Hände vorausgeschickt haben, dann ist der Mensch sehr undankbar.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcccxxi{Gott gehört die Königsherrschaft der Himmel und der Erde. Er erschafft, was Er will. Er schenkt, wem Er will, weibliche, und Er schenkt, wem Er will, männliche Nachkommen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcccxxii{Oder Er schenkt beides, männliche und weibliche Nachkommen. Und Er macht, wen Er will, unfruchtbar. Er weiß Bescheid und ist mächtig.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcccxxiii{Und es steht keinem Menschen zu, daß Gott zu ihm spricht, es sei denn durch Offenbarung oder hinter einem Vorhang, oder indem Er einen Boten sendet, der (ihm) dann mit seiner Erlaubnis offenbart, was Er will. Er ist erhaben und weise.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcccxxiv{Und so haben Wir dir Geist von unserem Befehl offenbart. Du wußtest nicht (vorher), was das Buch und was der Glaube ist. Und doch haben Wir es zu einem Licht gemacht, mit dem Wir rechtleiten, wen von unseren Dienern Wir wollen. Und wahrlich, du führst zu einem geraden Weg,\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcccxxv{Dem Weg Gottes, dem gehört, was in den Himmeln und was auf der Erde ist. Ja, zu Gott gelangen die Angelegenheiten.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcccxxvi{\basmalahde Haa Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcccxxvii{Beim deutlichen Buch!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcccxxviii{Wir haben es zu einem arabischen Koran gemacht, auf daß ihr verständig werdet.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcccxxix{Er ist aufgezeichnet in der Urnorm des Buches bei Uns, erhaben und weise.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcccxxx{Sollen Wir denn euch die Ermahnung vorenthalten, wenn ihr maßlose Leute seid?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxi{Und wie so manchen Propheten haben Wir zu den früheren Generationen gesandt!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxii{Und kein Gesandter kam zu ihnen, ohne daß sie ihn verspottet hätten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxiii{Und doch haben Wir solche verderben lassen, die eine stärkere Gewalt besaßen als diese da. Und fest steht, wie an den Früheren gehandelt wurde.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxiv{Und wenn du sie fragst, wer die Himmel und die Erde erschaffen hat, sagen sie bestimmt: «Erschaffen hat sie der Mächtige, der Bescheid weiß.»\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxv{Er, der euch die Erde zu einer Lagerstätte gemacht und euch auf ihr Wege gemacht hat, auf daß ihr der Rechtleitung folget,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxvi{Und der euch Wasser vom Himmel in einem bestimmten Maß herabkommen läßt. Damit beleben Wir eine abgestorbene Ortschaft. Auf diese Weise werdet auch ihr hervorgebracht werden.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxvii{Er, der die Paare alle erschaffen und euch Schiffe und Herdentiere gemacht hat, auf die ihr steigen könnt,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxviii{Damit ihr euch auf ihrem Rücken zurechtsetzt und dann, wenn ihr euch darauf zurechtgesetzt habt, der Gnade eures Herrn gedenkt und sagt: «Preis sei dem, der uns diese dienstbar gemacht hat! Wir hätten es allein gewiß nicht schaffen können.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxix{Und wir werden bestimmt zu unserem Herrn zurückkehren.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcccxl{Und sie schreiben Ihm einen Teil von seinen Dienern zu. Der Mensch ist ja offenkundig sehr undankbar.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcccxli{Oder hat Er sich etwa aus den Reihen seiner Geschöpfe Töchter genommen und für euch die Söhne erwählt?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcccxlii{Wenn einem von ihnen verkündet wird, (ihm sei geboren), was er dem Erbarmer zum Beispiel zuschreibt, bleibt sein Gesicht finster, und er ist voller Gram.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcccxliii{Wollen sie denn wirklich solche (Gott zuschreiben), die im Schmuck aufgezogen werden und sich im Streit nicht deutlich genug ausdrücken?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcccxliv{Und sie machen die Engel, welche ja die Diener des Erbarmers sind, zu weiblichen Wesen. Waren sie denn bei ihrer Erschaffung zugegen? Ihr Zeugnis wird aufgeschrieben, und sie werden zur Verantwortung gezogen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcccxlv{Und sie sagen: «Wenn der Erbarmer gewollt hätte, hätten wir sie nicht verehrt.» Sie haben ja kein Wissen darüber. Sie stellen nur Schätzungen an.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcccxlvi{Oder haben Wir ihnen etwa vordem ein Buch zukommen lassen, an dem sie sich festhalten könnten?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcccxlvii{Nein, vielmehr sagen sie: «Wir haben bei unseren Vätern eine bestimmte Glaubensrichtung vorgefunden, und in ihren Fußstapfen folgen wir der Rechtleitung.»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcccxlviii{So haben Wir auch vor dir in keine Stadt einen Warner gesandt, ohne daß die, die in ihr üppig lebten, gesagt hätten: «Wir haben bei unseren Vätern eine bestimmte Glaubensrichtung vorgefunden, und wir treten in ihre Fußstapfen.»\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcccxlix{Er sagte: «Was denn, auch wenn ich euch bringe, was eine bessere Rechtleitung beinhaltet als das, was ihr bei euren Vätern vorgefunden habt?» Sie sagten: «Wir verleugnen das, womit ihr gesandt worden seid.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcccl{Da rächten Wir uns an ihnen. So schau, wie das Ende derer war, die (die Gesandten) der Lüge geziehen haben.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcccli{Und als Abraham zu seinem Vater und seinem Volk sagte: «Ich bin unschuldig an dem, was ihr verehrt,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmccclii{Außer dem, der mich erschaffen hat; Er wird mich rechtleiten.»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmcccliii{Und er machte es zu einem bleibenden Ausspruch unter denen, die nach ihm kommen sollten, auf daß sie umkehren.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmcccliv{Nein, Ich habe diese da und ihre Väter genießen lassen, bis die Wahrheit und ein offenkundiger Gesandter zu ihnen kamen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmccclv{Als nun die Wahrheit zu ihnen kam, sagten sie: «Das ist Zauberei; wir glauben nicht daran.»\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmccclvi{Und sie sagen: «Wenn doch dieser Koran auf einen mächtigen Mann aus den zwei Städten herabgesandt worden wäre!»\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmccclvii{Sind sie es etwa, die die Barmherzigkeit deines Herrn verteilen? Wir haben doch unter ihnen ihren Lebensunterhalt im diesseitigen Leben verteilt und die einen von ihnen über die anderen um Rangstufen erhöht, damit die einen von ihnen die anderen in ihren Dienst nehmen. Aber die Barmherzigkeit deines Herrn ist besser als das, was sie zusammentragen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmccclviii{Und wären nicht die Menschen eine einzige Gemeinschaft, hätten Wir denen, die den Erbarmer verleugnen, Decken aus Silber für ihre Häuser gemacht, und auch Treppen, auf denen sie hinaufsteigen können,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmccclix{Und Türen für ihre Häuser, und Betten, auf denen sie liegen können,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmccclx{Und weiteren Prunk. All das ist nichts als Nutznießung des diesseitigen Lebens. Das Jenseits bei deinem Herrn ist aber für die Gottesfürchtigen bestimmt.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmccclxi{Wer sich gegenüber der Ermahnung des Erbarmers wie blind verhält, für den bestellen Wir einen Satan, der ihm dann zum Gesellen wird.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmccclxii{Und siehe, sie weisen sie vom Weg ab, sie aber meinen, sie würden der Rechtleitung folgen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmccclxiii{Wenn er schließlich zu Uns kommt, sagt er: «O wäre doch zwischen mir und dir eine Entfernung wie zwischen Osten und Westen!» Welch schlimmer Geselle!\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmccclxiv{Heute, da ihr Unrecht getan habt, nützt euch nicht, daß ihr an derselben Pein teilhabt.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmccclxv{Willst du denn die Tauben hören lassen oder die Blinden und die, die sich in einem offenkundigen Irrtum befinden, rechtleiten?\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmccclxvi{Sollten Wir dich fortnehmen, dann werden Wir uns an ihnen rächen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmccclxvii{Oder Wir lassen dich sehen, was Wir ihnen angedroht haben, denn Wir haben ja völlig Macht über sie.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmccclxviii{Halte nun fest an dem, was dir offenbart wurde; du befindest dich auf einem geraden Weg.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmccclxix{Das ist eine Ermahnung für dich und dein Volk. Und ihr werdet zur Verantwortung gezogen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmccclxx{Und frage jene unserer Gesandten, die Wir vor dir gesandt haben, ob Wir anstelle des Erbarmers (andere) Götter zum Gegenstand der Verehrung eingesetzt haben.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmccclxxi{Und Wir haben Mose mit unseren Zeichen zu Pharao und seinen Vornehmen gesandt. Er sagte: «Ich bin der Gesandte des Herrn der Welten.»\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmccclxxii{Als er mit unseren Zeichen zu ihnen kam, da lachten sie gleich über sie.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmccclxxiii{Und Wir zeigten ihnen kein Zeichen, das nicht größer gewesen wäre als das andere. Und Wir ergriffen sie mit der Pein, auf daß sie umkehrten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmccclxxiv{Und sie sagten: «O du Zauberer, rufe für uns deinen Herrn an aufgrund seines Bundes mit dir, dann werden wir der Rechtleitung folgen.»\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmccclxxv{Als Wir dann die Pein von ihnen aufhoben, da brachen sie gleich ihr Wort.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmccclxxvi{Und Pharao ließ unter seinem Volk ausrufen; er sagte: «O mein Volk, gehört mir nicht die Königsherrschaft Ägyptens, und auch dieser Bäche, die unter mir fließen? Wollt ihr nicht einsichtig sein?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmccclxxvii{Oder bin ich nicht besser als dieser da, der verächtlich ist und sich kaum deutlich ausdrückt?\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmccclxxviii{Wären ihm doch Armringe aus Gold überbracht worden, oder die Engel mit ihm als Begleitung gekommen!»\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmccclxxix{Er ließ sein Volk abirren, und sie gehorchten ihm. Sie waren ja frevlerische Leute.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmccclxxx{Als sie unseren Zorn erregten, rächten Wir uns an ihnen und ließen sie alle ertrinken.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxi{Und Wir machten sie zu einem dahingegangenen Volk und zu einem Beispiel für die anderen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxii{Und als der Sohn Marias als Beispiel angeführt wurde, da erging sich dein Volk gleich in lautem Spott.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxiii{Und sie sagten: «Wer ist besser, unsere Götter oder er?» Sie führten ihn dir nur zum Streiten an. Nein, sie sind streitsüchtige Leute.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxiv{Er ist nichts als ein Diener, den Wir begnadet und zu einem Beispiel für die Kinder Israels gemacht haben.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxv{Und wenn Wir wollten, könnten Wir aus euren Reihen Engel bestellen, die als (eure) Nachfolger auf der Erde leben würden.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxvi{Und er ist ein Erkennungszeichen für die Stunde (des Gerichtes). So hegt keine Zweifel über sie und folgt mir. Das ist ein gerader Weg.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxvii{Und der Satan soll euch nicht abweisen. Er ist euch ein offenkundiger Feind.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxviii{Und als Jesus mit den deutlichen Zeichen kam, sagte er: «Ich komme zu euch mit der Weisheit, und um euch einiges von dem, worüber ihr uneins seid, deutlich zu machen. So fürchtet Gott und gehorcht mir.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxix{Gott ist mein Herr und euer Herr. So dienet Ihm. Das ist ein gerader Weg.»\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcccxc{Dann wurden die Parteien untereinander uneins. Wehe denen, die Unrecht tun, vor der Pein eines schmerzhaften Tages!\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcccxci{Erwarten sie denn etwas anderes, als daß die Stunde plötzlich über sie kommt, ohne daß sie es merken?\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcccxcii{Vertraute werden an jenem Tag einer des anderen Feind sein, außer den Gottesfürchtigen.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcccxciii{«O meine Diener, ihr habt heute nichts zu befürchten, und ihr werdet nicht traurig sein.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmcccxciv{Ihr, die ihr an unsere Zeichen glaubtet und gottergeben waret.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcccxcv{Geht ins Paradies ein, ihr und eure Gattinnen, euch wird Freude bereitet.»\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcccxcvi{Es werden ihnen Schüsseln aus Gold und Becher herumgereicht. Und darin gibt es, was die Seele begehrt und für die Augen eine Wonne ist. «Und ihr werdet darin ewig weilen\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmcccxcvii{Das ist das Paradies, das euch zum Erbe gegeben worden ist für das, was ihr zu tun pflegtet.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmcccxcviii{Darin sind viele Früchte für euch, von denen ihr essen könnt.»\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmcccxcix{Die Übeltäter aber werden in der Pein der Hölle ewig weilen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmcd{Es wird ihnen keine Erleichterung gewährt, und sie werden darin voller Verzweiflung sein.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmcdi{Und nicht Wir haben ihnen Unrecht getan, sondern sie sind es, die Unrecht getan haben.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmcdii{Und sie rufen: «O Malik, dein Herr soll mit uns ein Ende machen.» Er sagt: «Ihr werdet (hier) bleiben.»\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmcdiii{Wir haben euch die Wahrheit gebracht. Aber die meisten von euch verabscheuen die Wahrheit.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmcdiv{Oder haben sie Ränke geschmiedet? Auch Wir können ebensolche schmieden.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmmcdv{Oder meinen sie, Wir würden nicht hören, was sie insgeheim und in vertraulichem Gespräch sagen? Doch, und auch unsere Boten schreiben bei ihnen (alles) auf.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmcdvi{Sprich: Wenn der Erbarmer ein Kind hätte, wäre ich der erste derer, die (es) anbeten.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmcdvii{Preis sei dem Herrn der Himmel und der Erde, dem Herrn des Thrones! (Erhaben ist Er) über das, was sie da schildern.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmcdviii{Laß sie schweifende Reden halten und ihr Spiel treiben, bis sie ihrem Tag begegnen, der ihnen angedroht ist.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmcdix{Er ist Gott im Himmel und Gott auf der Erde. Er ist der, der weise ist und Bescheid weiß.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmcdx{Und gesegnet sei der, dem die Königsherrschaft der Himmel und der Erde und dessen, was dazwischen ist, gehört, der das Wissen über die Stunde besitzt und zu dem ihr zurückgebracht werdet!\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmcdxi{Und diejenigen, die sie an seiner Stelle anrufen, verfügen nicht darüber, Fürsprache einzulegen, ausgenommen die, welche die Wahrheit bezeugen und die Bescheid wissen.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmmcdxii{Wenn du sie fragst, wer sie erschaffen hat, sagen sie bestimmt: «Gott». Wie leicht lassen sie sich doch abwenden!\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmmcdxiii{Und bei seinem Ausspruch: «O mein Herr, das sind Leute, die nicht glauben.»\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmmcdxiv{Übe Nachsicht mit ihnen und sprich: «Frieden!» Sie werden es noch zu wissen bekommen.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmmcdxv{\basmalahde Haa Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcdxvi{Beim deutlichen Buch!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcdxvii{Wir haben es in einer gesegneten Nacht hinabgesandt - Wir haben ja (die Menschen) immer wieder gewarnt -,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcdxviii{In der jede weise Angelegenheit einzeln entschieden wird\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcdxix{Als eine Angelegenheit von unserer Seite - ja, Wir haben immer wieder (Warner) gesandt -,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcdxx{Als eine Barmherzigkeit von deinem Herrn - Er ist der, der alles hört und weiß -,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcdxxi{Dem Herrn der Himmel und der Erde und dessen, was dazwischen ist, so ihr Gewißheit hegt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcdxxii{Es gibt keinen Gott außer Ihm. Er macht lebendig und läßt sterben, (Er), euer Herr und der Herr eurer Vorväter.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcdxxiii{Aber nein, sie hegen Zweifel und treiben ihr Spiel.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcdxxiv{Erwarte nun den Tag, an dem der Himmel einen offenkundigen Rauch herabkommen läßt,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcdxxv{Der die Menschen überdecken wird. Das ist eine schmerzhafte Pein.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcdxxvi{«Unser Herr, hebe die Pein von uns auf. Wir wollen gläubig sein.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcdxxvii{Wie soll ihnen da die Ermahnung nützen, wo doch ein offenkundiger Gesandter zu ihnen kam,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcdxxviii{Sie sich aber von ihm abkehrten und sagten: «Einer, den ein anderer belehrt, ein Besessener»?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcdxxix{Wir werden die Pein ein wenig aufheben. Ihr aber werdet rückfällig werden.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcdxxx{Am Tag, da Wir mit großer Gewalt zugreifen werden, da werden Wir uns rächen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxi{Und Wir haben schon vor ihnen das Volk des Pharao der Versuchung ausgesetzt. Zu ihnen kam ein edler Gesandter:\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxii{«Übergebt mir die Diener Gottes. Ich bin euch ein treuer Gesandter.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxiii{Und zeigt euch Gott gegenüber nicht überheblich. Ich komme zu euch mit einer offenkundigen Ermächtigung.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxiv{Und ich suche Zuflucht bei meinem Herrn und eurem Herrn davor, daß ihr mich steinigt.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxv{Und wenn ihr mir nicht glaubt, dann haltet euch fern von mir.»\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxvi{Da rief er zu seinem Herrn: «Das sind Leute, die Übeltäter sind.»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxvii{- «Zieh bei Nacht mit meinen Dienern fort; ihr werdet da verfolgt werden.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxviii{Und verlaß das Meer als ruhige breite Bahn. Siehe, sie sind ein Heer, das ertränkt werden soll.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxix{Wie viele Gärten und Quellen ließen sie zurück,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcdxl{Und Saatfelder und eine treffliche Stätte,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmcdxli{Und ein angenehmes Leben, in dem sie es sich wohl sein ließen!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmcdxlii{So war es. Und Wir gaben es anderen Leuten zum Erbe.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmcdxliii{Weder der Himmel noch die Erde weinte über sie. Und ihnen wurde kein Aufschub gewährt.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmcdxliv{Und Wir erretteten die Kinder Israels von der schmählichen Pein,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmcdxlv{Von Pharao. Er hatte hohe Gewalt und war einer der Maßlosen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmcdxlvi{Und Wir erwählten sie aufgrund von Wissen vor den Weltenbewohnern\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmcdxlvii{Und ließen ihnen von den Zeichen solche zukommen, die eine offenkundige Prüfung enthielten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcdxlviii{Wahrlich, diese da sagen:\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmcdxlix{«Es gibt nur unseren ersten Tod. Und wir werden nicht wiedererweckt werden.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmcdl{Bringt doch unsere Väter herbei, so ihr die Wahrheit sagt.»\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcdli{Sind sie etwa besser, oder das Volk des Tubbaá und jene, die vor ihnen lebten? Wir haben sie verderben lassen, denn sie waren Übeltäter.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcdlii{Und Wir haben die Himmel und die Erde und das, was dazwischen ist, nicht zum Spiel erschaffen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcdliii{Wir haben sie in Wahrheit erschaffen. Aber die meisten von ihnen wissen nicht Bescheid.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcdliv{Der Tag der Scheidung ist der Termin für sie alle,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcdlv{Der Tag, an dem kein Schutzherr seinem Schützling etwas nützen kann und sie keine Unterstützung erfahren werden,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcdlvi{Ausgenommen wessen sich Gott erbarmt. Er ist der Mächtige, der Barmherzige.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcdlvii{Der Zaqqum-Baum\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcdlviii{Ist die Speise des Sünders;\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcdlix{Wie geschmolzenes Erz kocht er in den Bäuchen,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcdlx{Wie das heiße Wasser kocht.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcdlxi{«Nehmt ihn und zerrt ihn mitten in die Hölle.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcdlxii{Dann gießt über seinen Kopf etwas von der Pein des heißen Wassers.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcdlxiii{Koste es: Du bist doch der Mächtige, der Treffliche!»\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcdlxiv{«Das ist, worüber ihr Zweifel hegtet.»\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcdlxv{Die Gottesfürchtigen befinden sich an einer sicheren Stätte,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcdlxvi{In Gärten und an Quellen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcdlxvii{Sie tragen Gewänder aus Seide und Brokat, und (sie liegen) einander gegenüber.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcdlxviii{So ist es. Und Wir geben ihnen als Gattinnen Huri mit schönen, großen Augen.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcdlxix{Sie rufen darin nach Früchten jeder Art (und leben) in Sicherheit.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcdlxx{Sie erleiden darin nicht den Tod, außer dem ersten Tod. Und Er hütet sie vor der Pein der Hölle,\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxi{Als Huld von deinem Herrn. Das ist der großartige Erfolg.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxii{Wir haben ihn durch deine Zunge leicht gemacht, auf daß sie es bedenken.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxiii{Warte nun ab; auch sie warten ab.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxiv{\basmalahde Haa Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxv{Herabsendung des Buches von Gott, dem Mächtigen, dem Weisen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxvi{In den Himmeln und auf der Erde sind Zeichen für die Gläubigen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxvii{Und in eurer Erschaffung und in dem, was Er an Getier sich ausbreiten läßt, sind Zeichen für Leute, die Gewißheit hegen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxviii{Und auch im Aufeinanderfolgen von Nacht und Tag und in dem, was Gott an Lebensunterhalt vom Himmel herabkommen läßt und dadurch die Erde nach ihrem Absterben belebt, und im Wechsel der Winde sind Zeichen für Leute, die Verstand haben.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxix{Dies sind die Zeichen Gottes. Wir verlesen sie dir der Wahrheit entsprechend. An welche Botschaft, nachdem Gott und seine Zeichen kundgetan worden sind, wollen sie denn sonst glauben?\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxx{Wehe jedem Lügner und Sünder,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxi{Der hört, wie ihm die Zeichen Gottes verlesen werden, und dann hochmütig (in seinem Unglauben) verharrt, als hätte er sie nie gehört! Verkünde ihm eine schmerzhafte Pein.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxii{Und wenn er etwas von unseren Zeichen erfährt, nimmt er es zum Gegenstand des Spottes. Für solche Leute ist eine schmähliche Pein bestimmt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxiii{Hinterdrein steht für sie die Hölle bereit. Und es nützt ihnen nichts, was sie erworben und was sie sich anstelle Gottes zu Freunden genommen haben. Für sie ist eine gewaltige Pein bestimmt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxiv{Dies ist die Rechtleitung. Für diejenigen aber, die die Zeichen ihres Herrn verleugnen, ist eine Pein durch ein schmerzhaftes Zorngericht bestimmt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxv{Gott ist es, der euch das Meer dienstbar gemacht hat, daß die Schiffe nach seinem Befehl darauf fahren, daß ihr nach etwas von seiner Huld strebt und daß ihr dankbar werdet.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxvi{Und Er hat euch von sich aus alles dienstbar gemacht, was in den Himmeln und was auf der Erde ist. Darin sind Zeichen für Leute, die nachdenken.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxvii{Sag denen, die glauben, sie sollen denen vergeben, die nicht erwarten, die Tage Gottes zu erleben, auf daß Er den Leuten vergelte für das, was sie erworben haben.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxviii{Wer Gutes tut, tut es zu seinem eigenen Vorteil. Und wer Böses tut, tut es zu seinem eigenen Schaden. Zu eurem Herrn werdet ihr dann zurückgebracht.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxix{Und Wir haben den Kindern Israels das Buch, die Urteilsvollmacht und die Prophetie zukommen lassen und ihnen allerlei köstliche Dinge beschert und sie bevorzugt vor den Weltenbewohnern.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcdxc{Und Wir haben ihnen in der Angelegenheit (der Religion) deutliche Beweise zukommen lassen. Sie wurden aber erst uneins, nachdem das Wissen zu ihnen gekommen war, dies aus ungerechter Auflehnung untereinander. Dein Herr wird am Tag der Auferstehung zwischen ihnen über das entscheiden, worüber sie uneins waren.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcdxci{Dann stellten Wir für dich eine Richtung in der Angelegenheit (der Religion) fest. So folge ihr und folge nicht den Neigungen derer, die nicht Bescheid wissen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcdxcii{Sie können dir vor Gott nichts nützen. Diejenigen, die Unrecht tun, sind untereinander Freunde. Gott aber ist der Freund der Gottesfürchtigen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcdxciii{Dies sind einsichtbringende Zeichen für die Menschen und eine Rechtleitung und Barmherzigkeit für Leute, die Gewißheit hegen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcdxciv{Oder meinen diejenigen, die die Missetaten verüben, daß Wir sie denen, die glauben und die guten Werke tun, gleichstellen sowohl in ihrem Leben als in ihrem Sterben? Schlimm ist es, wie sie urteilen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcdxcv{Gott hat die Himmel und die Erde der Wahrheit entsprechend erschaffen. Und jeder Seele soll vergolten werden für das, was sie erworben hat. Und ihnen wird nicht Unrecht getan.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcdxcvi{Hast du den gesehen, der sich zu seinem Gott seine Neigung gemacht hat, den Gott aufgrund von Wissen irregeführt, dem Er das Gehör und das Herz versiegelt und auf dessen Augenlicht eine Hülle gelegt hat? Wer nach Gott könnte ihn rechtleiten? Wollt ihr es nicht bedenken?\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcdxcvii{Und sie sagen: «Es gibt nur unser diesseitiges Leben. Wir sterben und leben (hier), und nur die Zeit läßt uns verderben.» Sie haben doch kein Wissen darüber, sie stellen nur Mutmaßungen an.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcdxcviii{Und wenn ihnen unsere Zeichen als deutliche Beweise verlesen werden, führen sie kein anderes Argument an, als nur zu sagen: «Bringt unsere Väter herbei, wenn ihr die Wahrheit sagt.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcdxcix{Sprich: Gott macht euch lebendig und läßt euch dann sterben. Dann versammelt Er euch zum Tag der Auferstehung, an dem kein Zweifel möglich ist. Aber die meisten Menschen wissen nicht Bescheid.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmd{Gott gehört die Königsherrschaft der Himmel und der Erde. Und am Tag, da die Stunde heraufkommt, an jenem Tag werden die verlieren, die (die Botschaft) für falsch erklären.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdi{Und du siehst jede Gemeinschaft auf den Knien sitzen. Jede Gemeinschaft wird zu ihrem Buch gerufen: «Heute wird euch das vergolten, was ihr zu tun pflegtet.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdii{Dies ist unser Buch. Es spricht die Wahrheit gegen euch aus. Wir ließen aufschreiben, was ihr zu tun pflegtet.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdiii{Diejenigen, die geglaubt und die guten Werke getan haben, läßt ihr Herr in seine Barmherzigkeit eingehen. Das ist der offenkundige Erfolg.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdiv{Diejenigen aber, die ungläubig waren, (bekommen zu hören): «Sind euch nicht meine Zeichen immer wieder verlesen worden? Ihr aber habt euch hochmütig verhalten und seid Leute gewesen, die Übeltäter waren.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdv{Und wenn gesagt wurde: >Das Versprechen Gottes ist wahr, und an der Stunde ist kein Zweifel möglichWir wissen nicht, was überhaupt die Stunde ist. Wir stellen nur Mutmaßungen an und hegen darüber keine Gewißheit.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdvi{Und ihnen werden ihre Missetaten, die sie begangen haben, klar, und es umschließt sie das, worüber sie spotteten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdvii{Und es wird gesprochen: «Heute werden Wir euch vergessen, wie ihr die Begegnung mit diesem eurem Tag vergessen habt. Eure Heimstätte ist das Feuer, und ihr habt keine Helfer.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdviii{Dies dafür, daß ihr die Zeichen Gottes zum Gegenstand des Spottes genommen habt und daß euch das diesseitige Leben betört hat.» Heute werden sie nicht mehr daraus herausgebracht, und es wird von ihnen keine entschuldigende Umkehr mehr angenommen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdix{Lob sei nun Gott, dem Herrn der Himmel und dem Herrn der Erde, dem Herrn der Welten!\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdx{Ihm gehört die Hoheit in den Himmeln und auf der Erde. Er ist der Mächtige, der Weise.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdxi{\basmalahde Haa Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdxii{Herabsendung des Buches von Gott, dem Mächtigen, dem Weisen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdxiii{Wir haben die Himmel und die Erde und das, was dazwischen ist, nur in Wahrheit und auf eine festgesetzte Frist erschaffen. Aber diejenigen, die ungläubig sind, wenden sich von der Warnung, die sie erhielten, ab.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdxiv{Sprich: Was meint ihr von dem, was ihr anstelle Gottes anruft? Zeigt mir, was sie von der Erde erschaffen haben. Oder haben sie etwa einen Anteil an den Himmeln? Bringt mir doch ein Buch her, das vor diesem wäre, oder auch nur eine Spur von Wissen, so ihr die Wahrheit sagt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdxv{Und wer ist weiter abgeirrt als der, der anstelle Gottes solche anruft, die ihn bis zum Tag der Auferstehung nicht erhören, ihr Rufen nicht beachten\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdxvi{Und, wenn die Menschen versammelt werden, ihnen Feind sind und ihre Verehrung verleugnen?\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdxvii{Und wenn ihnen unsere Zeichen als deutliche Beweise verlesen werden, sagen diejenigen, die ungläubig sind, von der Wahrheit, als sie zu ihnen kam: «Das ist offenkundige Zauberei.»\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdxviii{Oder sagen sie wohl: «Er hat ihn erdichtet»? Sprich: Wenn ich ihn erdichtet habe, dann könnt ihr mir vor Gott keinen Nutzen bringen. Er weiß besser, womit ihr euch ausgiebig beschäftigt. Er genügt als Zeuge zwischen mir und euch. Und Er ist der, der voller Vergebung und barmherzig ist.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdxix{Sprich: Ich bin keine Neuerscheinung unter den Gesandten. Und ich weiß nicht, was mit mir, und auch nicht, was mit euch geschehen wird. Ich folge nur dem, was mir offenbart wird. Und ich bin nur ein deutlicher Warner.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdxx{Sprich: Was meint ihr, wenn er doch von Gott stammt und ihr ihn aber verleugnet, während ein Zeuge von den Kindern Israels etwas bezeugt, was ihm gleich ist, und nunmehr an ihn glaubt, während ihr euch hochmütig zeigt? Gott leitet die ungerechten Leute nicht recht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdxxi{Und diejenigen, die ungläubig sind, sagen von denen, die glauben: «Wäre er etwas Gutes, hätten sie ihn nicht vor uns erlangt.» Und da sie sich nicht durch ihn rechtleiten lassen, werden sie ja sagen: «Das ist eine alte Lüge.»\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdxxii{Und ihm ging das Buch des Mose als Vorbild und Barmherzigkeit voraus. Und dies ist ein Buch zur Bestätigung in arabischer Sprache, um diejenigen, die Unrecht tun, zu warnen, und als Botschaft für die Rechtschaffenen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdxxiii{Diejenigen, die sagen: «Unser Herr ist Gott», und sich dann recht verhalten, haben nichts zu befürchten, und sie werden nicht traurig sein.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdxxiv{Das sind die Gefährten des Paradieses; darin werden sie ewig weilen als Lohn für das, was sie zu tun pflegten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdxxv{Und Wir haben dem Menschen aufgetragen, seine Eltern gut zu behandeln. Seine Mutter hat ihn unter widrigen Umständen getragen und unter widrigen Umständen geboren. Die Zeit von der Schwangerschaft bis zur Entwöhnung beträgt dreißig Monate. Wenn er dann seine Vollkraft erreicht hat und (auch) das Alter von vierzig Jahren erreicht hat, sagt er: «Mein Herr, gib mir ein, für deine Gnade zu danken, mit der Du mich und meine Eltern begnadet hast, und etwas Gutes zu tun, was Dir wohlgefällt. Und schenke mir Gutes in meiner Nachkommenschaft. Ich wende mich Dir zu, und ich gehöre zu denen, die sich (Dir) ergeben.»\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdxxvi{Das sind die, von denen Wir das Beste von dem, was sie getan haben, annehmen und deren Missetaten Wir übergehen, unter den Gefährten des Paradieses. Dies ist das wahrhaftige Versprechen, das ihnen immer wieder gegeben wurde.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdxxvii{Und der, der zu seinen Eltern sagt: «Pfui über euch! Wollt ihr denn mir versprechen, ich würde hervorgebracht werden, wo vor mir Generationen dahingegangen sind?»... Sie aber bitten Gott um Hilfe: «Wehe dir, glaube doch. Das Versprechen Gottes ist wahr.» Da sagt er: «Das sind nichts als die Fabeln der Früheren.»\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdxxviii{Das sind die, gegen die der Spruch fällig wird, wie gegen Gemeinschaften von Djinn und Menschen, die vor ihnen dahingegangen sind. Sie sind ja die Verlierer.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdxxix{Für alle sind Rangstufen bestimmt für das, was sie getan haben, und dies, damit Er ihnen ihre Taten voll erstatte. Und ihnen wird nicht Unrecht getan.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdxxx{Und am Tag, da diejenigen, die ungläubig sind, dem Feuer vorgeführt werden: «Ihr habt eure köstlichen Dinge im diesseitigen Leben aufgezehrt und genossen. Heute wird euch mit der schmählichen Pein vergolten dafür, daß ihr euch auf der Erde immer wieder zu Unrecht hochmütig verhalten und daß ihr immer wieder gefrevelt habt.»\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdxxxi{Und gedenke des Bruders der Aad, als er bei den Dünen sein Volk warnte - Warner gab es vor ihm und nach ihm -: «Ihr sollt Gott allein dienen. Ich fürchte für euch die Pein eines gewaltigen Tages.»\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdxxxii{Sie sagten: «Bist du gekommen, um uns von unseren Göttern abzuwenden? Bring uns doch her, was du uns androhst, so du zu denen gehörst, die die Wahrheit sagen.»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdxxxiii{Er sagte: «Nur Gott weiß darüber Bescheid. Ich aber richte euch das aus, womit ich gesandt worden bin. Aber ich sehe, daß ihr Leute seid, die töricht sind.»\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdxxxiv{Als sie es als heraufziehende Wolke sahen, die sich ihren Tälern näherte, sagten sie: «Das ist eine heraufziehende Wolke, die uns Regen bringt.» - «Nein, vielmehr ist das, was ihr zu beschleunigen gewünscht habt, ein Wind, der eine schmerzhafte Pein birgt,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdxxxv{Der auf den Befehl seines Herrn alles zerstört.» Am Morgen waren sie so, daß man nur noch ihre Wohnungen sah. So vergelten Wir den Leuten, die Übeltäter sind.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdxxxvi{Und Wir hatten ihnen doch eine angesehene Stellung verliehen, die Wir euch nicht verliehen haben. Und Wir haben ihnen Gehör, Augenlicht und Herz gegeben. Aber ihr Gehör, ihr Augenlicht und ihr Herz nützten ihnen nichts, da sie die Zeichen Gottes verleugneten. Und es umschloß sie, worüber sie spotteten.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdxxxvii{Und Wir haben die Städte rings um euch auch verderben lassen. Und Wir haben verschiedene Zeichen dargelegt, auf daß sie umkehren.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdxxxviii{Hätten ihnen doch diejenigen Unterstützung geleistet, die sie sich anstelle Gottes zu Göttern nahmen, (angeblich) um sich Zutritt in seine Nähe zu verschaffen! Nein, sie sind ihnen entschwunden. Das ist ihre Lüge und das, was sie zu erdichten pflegten.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdxxxix{Und als Wir eine Schar Djinn veranlaßten, sich dir zuzuwenden und dem Koran zuzuhören. Als sie eingetroffen waren, sagten sie: «Schweigt und hört zu.» Als er zu Ende war, kehrten sie zu ihrem Volk zurück, um sie zu warnen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdxl{Sie sagten: «O unser Volk, wir haben ein Buch gehört, das nach Mose herabgesandt wurde, zu bestätigen, was vor ihm vorhanden war, und das zur Wahrheit führt und zu einem geraden Weg.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdxli{O unser Volk, hört auf den Rufer Gottes und glaubt an ihn, so vergibt Er euch etwas von euren Sünden und schützt euch vor einer schmerzhaften Pein.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdxlii{Wer auf den Rufer Gottes nicht hört, kann doch (seinen Willen) auf der Erde nicht vereiteln, und er hat außer Ihm keine Freunde. Solche befinden sich in einem offenkundigen Irrtum.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdxliii{Haben sie denn nicht gesehen, daß Gott, der die Himmel und die Erde erschaffen hat und bei ihrer Erschaffung nicht überfordert war, auch die Macht hat, die Toten wieder lebendig zu machen? Ja, Er hat Macht zu allen Dingen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdxliv{Und am Tag, da diejenigen, die ungläubig sind, dem Feuer vorgeführt werden: «Ist dies nicht die Wahrheit?» Sie sagen: «Doch, bei unserem Herrn!» Er spricht: «Dann kostet die Pein dafür, daß ihr ungläubig wäret.»\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdxlv{Sei nun geduldig, wie diejenigen unter den Gesandten, die Entschlossenheit besaßen, geduldig waren. Und wünsche nichts gegen sie zu beschleunigen. Am Tag, da sie sehen, was ihnen angedroht wird, wird ihnen sein, als hätten sie nur eine Stunde vom Tag verweilt. Hiermit wird die Botschaft ausgerichtet. Wird denn jemand anderes ins Verderben gestürzt als die frevlerischen Leute?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdxlvi{\basmalahde Denen, die ungläubig sind und vom Weg Gottes abweisen, läßt Er ihre Werke fehlgehen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdxlvii{Denen aber, die glauben und die guten Werke tun und an das glauben, was auf Muhammad herabgesandt worden ist - es ist ja die Wahrheit von ihrem Herrn -, denen sühnt Er ihre Missetaten und bringt ihre Angelegenheiten in Ordnung.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdxlviii{Dies, weil die, die ungläubig sind, dem Falschen folgen, und die, die glauben, der Wahrheit von ihrem Herrn folgen. Auf diese Weise legt Gott den Menschen ihren Zustand in Gleichnissen dar.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdxlix{Wenn ihr auf die, die ungläubig sind, trefft, dann schlagt (ihnen) auf die Nacken. Wenn ihr sie schließlich schwer niedergekämpft habt, dann schnürt (ihnen) die Fesseln fest. Danach gilt es, sie aus Gnade oder gegen Lösegeld zu entlassen. (Handelt so), bis der Krieg seine Waffenlasten ablegt. So ist es. Und wenn Gott wollte, würde Er sie selbst strafen. Aber Er möchte die einen von euch durch die anderen prüfen. Denen, die auf dem Weg Gottes getötet werden, läßt Er ihre Werke niemals fehlgehen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdl{Er wird sie rechtleiten und ihre Angelegenheiten in Ordnung bringen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdli{Sie ins Paradies eingehen lassen, das Er ihnen zu erkennen gegeben hat.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdlii{O ihr, die ihr glaubt, wenn ihr Gott unterstützt, unterstützt Er euch und festigt eure Schritte.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdliii{Was aber diejenigen betrifft, die ungläubig sind, so wehe ihnen! Und Er wird ihre Werke fehlgehen lassen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdliv{Dies, weil sie verabscheuen, was Gott herabgesandt hat. So macht Er ihre Werke wertlos.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdlv{Sind sie denn nicht auf der Erde umhergezogen und haben geschaut, wie das Ende derer war, die vor ihnen lebten? Gott hat über sie Zerstörung gebracht. Den Ungläubigen wird etwas Ähnliches widerfahren.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdlvi{Dies, weil Gott der Schutzherr derer ist, die glauben, und weil die Ungläubigen keinen Schutzherrn haben.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdlvii{Gott läßt diejenigen, die glauben und die guten Werke tun, in Gärten eingehen, unter denen Bäche fließen. Diejenigen aber, die ungläubig sind, genießen und essen, wie das Vieh frißt; das Feuer ist ihre Bleibe.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdlviii{Und wie manche Stadt, die eine stärkere Kraft hatte als deine Stadt, die dich vertrieben hat, haben Wir verderben lassen; und sie hatten keine Helfer.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdlix{Ist denn der, der einen deutlichen Beweis von seinem Herrn hat, denen gleich, denen ihr böses Tun verlockend gemacht wird und die ihren Neigungen folgen?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdlx{Mit dem Paradies, das den Gottesfürchtigen versprochen ist, ist es wie folgt: Darin sind Bäche mit Wasser, das nicht faul ist, und Bäche mit Milch, deren Geschmack sich nicht ändert, und Bäche mit Wein, der genußvoll ist für die, die davon trinken, und Bäche mit gefiltertem Honig. Und sie haben darin allerlei Früchte und Vergebung von ihrem Herrn. (Sind diese etwa) denen gleich, die im Feuer ewig weilen und heißes Wasser zu trinken bekommen, das ihre Eingeweide zerreißt?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdlxi{Und unter ihnen gibt es welche, die dir zuhören. Wenn sie aber dann von dir weggehen, sagen sie zu denen, denen das Wissen zugekommen ist: «Was hat er soeben gesagt?» Das sind die, deren Herzen Gott versiegelt hat und die ihren Neigungen folgen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdlxii{Und diejenigen, die der Rechtleitung folgen, läßt Er in der Rechtleitung zunehmen, und Er verleiht ihnen ihre Gottesfurcht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdlxiii{Erwarten sie denn etwas anderes, als daß die Stunde plötzlich über sie kommt? Ihre Voraussetzungen sind bereits eingetroffen. Wenn sie über sie kommt, was nützt ihnen dann noch ihre Ermahnung?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdlxiv{Wisse nun, daß es keinen Gott gibt außer Gott. Und bitte um Vergebung für deine Sünde und für die gläubigen Männer und die gläubigen Frauen. Gott weiß, wo ihr umherzieht und wo ihr bleibt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdlxv{Und diejenigen, die glauben, sagen: «Wäre doch eine Sure herabgesandt worden!» Wenn aber eine eindeutige Sure herabgesandt wird und darin vom Kampf die Rede ist, siehst du diejenigen, in deren Herzen Krankheit ist, zu dir schauen, wie einer schaut, der vor dem Tod ohnmächtig wird. Recht geschieht ihnen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdlxvi{Gehorchen (sollen sie) und geziemende Worte sagen. Wenn die Sache beschlossen ist, dann wäre es besser für sie, sie würden Gott gegenüber aufrichtig sein.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdlxvii{Ob ihr wohl, wenn ihr euch abkehrt, auf der Erde Unheil stiftet und die Verwandtschaftsbande zerreißt?\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdlxviii{Das sind die, die Gott verflucht: Er macht sie somit taub und läßt ihr Augenlicht erblinden.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdlxix{Betrachten sie denn nicht sorgfältig den Koran? Oder sind an ihren Herzen Verriegelungen angebracht?\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdlxx{Denen, die den Rücken kehren, nachdem ihnen die Rechtleitung deutlich geworden ist, hat der Satan etwas eingeredet und Hoffnung auf Aufschub gemacht.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdlxxi{Dies, weil sie zu denen, die verabscheuen, was Gott herabgesandt hat, sagen: «Wir werden euch in einigen Angelegenheiten gehorchen.» Aber Gott weiß, was sie im geheimen reden.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdlxxii{Wie wird es wohl sein, wenn die Engel sie abberufen und sie dabei ins Gesicht und auf den Rücken schlagen?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdlxxiii{Dies, weil sie dem folgen, was Gott erzürnt, und sein Wohlgefallen verabscheuen. So macht Er ihre Werke wertlos.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdlxxiv{Oder meinen diejenigen, in deren Herzen Krankheit ist, Gott würde ihren Groll nicht ans Licht bringen?\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdlxxv{Und wenn Wir wollten, würden Wir sie dir zeigen, so daß du sie an ihrem Merkmal erkennst. Und du wirst sie bestimmt an ihrer falschen Art zu sprechen erkennen. Und Gott weiß über eure Werke Bescheid.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdlxxvi{Und Wir werden euch bestimmt prüfen, bis Wir feststellen, welche von euch sich einsetzen und standhaft sind, und bis Wir die Berichte über euch prüfen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdlxxvii{Diejenigen, die ungläubig sind, vom Weg Gottes abweisen und sich dem Gesandten widersetzen, nachdem ihnen die Rechtleitung deutlich geworden ist, können Gott nichts schaden. Und Er wird ihre Werke wertlos machen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdlxxviii{O ihr, die ihr glaubt, gehorcht Gott und gehorcht dem Gesandten und laßt eure Werke nicht fehlgehen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdlxxix{Denen, die ungläubig sind und vom Weg Gottes abweisen und dann als Ungläubige sterben, wird Gott niemals vergeben.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdlxxx{So erlahmt nicht und ruft nicht zum Frieden, wo ihr die Oberhand haben werdet. Und Gott ist mit euch, und Er wird euch eure Werke niemals schmälern.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxi{Das diesseitige Leben ist nur Spiel und Zerstreuung. Wenn ihr glaubt und gottesfürchtig seid, läßt Er euch euren Lohn zukommen und verlangt von euch nicht euer Vermögen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxii{Würde Er es von euch verlangen und euch drängen, ihr würdet damit geizen, und Er würde euren Groll ans Licht bringen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxiii{Siehe, ihr werdet aufgerufen, auf dem Weg Gottes zu spenden. Jedoch gibt es unter euch welche, die sich geizig zeigen. Wer sich aber geizig zeigt, ist geizig gegen sich selbst. Gott ist der, der auf niemanden angewiesen ist, ihr aber seid die Armen. Wenn ihr euch abkehrt, nimmt Er an eurer Stelle ein anderes Volk, und sie werden dann nicht so sein wie ihr.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxiv{\basmalahde Wir haben dir einen offenkundigen Erfolg verliehen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxv{Damit Gott dir deine Sünden vergebe, die früheren und die späteren, und damit Er seine Gnade an dir vollende und dich einen geraden Weg führe,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxvi{Und damit Gott dich mit einem mächtigen Beistand unterstütze.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxvii{Er ist es, der die Ruhe spendende Gegenwart in die Herzen der Gläubigen herabgesandt hat, daß sie in ihrem Glauben noch an Glauben zunehmen. Und Gott gehören die Heerscharen der Himmel und der Erde. Und Gott weiß Bescheid und ist weise.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxviii{Dies, damit Er die gläubigen Männer und die gläubigen Frauen in Gärten eingehen lasse, unter denen Bäche fließen - darin werden sie ewig weilen - und ihnen ihre Missetaten sühne. Und das ist bei Gott ein großartiger Erfolg.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxix{Und damit Er die Heuchler, Männer und Frauen, und die Polytheisten, Männer und Frauen, peinige, die von Gott eine schlechte Meinung haben. Über sie kommt die böse Schicksalswendung. Gott zürnt ihnen, verflucht sie und bereitet ihnen die Hölle - welch schlimmes Ende!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdxc{Und Gott gehören die Heerscharen der Himmel und der Erde. Und Gott ist mächtig und weise.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdxci{Wir haben dich als Zeugen, als Freudenboten und als Warner gesandt,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdxcii{Damit ihr an Gott und seinen Gesandten glaubt, ihm beisteht und ihn ehrt, und (damit ihr) Ihn preist morgens und abends.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdxciii{Diejenigen, die dir Treue geloben, geloben Gott Treue. Gottes Hand liegt über ihren Händen. Wer nun (sein Wort) bricht, bricht es zu seinem eigenen Schaden. Und wer das erfüllt, wozu er sich Gott gegenüber verpflichtet hat, dem läßt Er einen großartigen Lohn zukommen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdxciv{Die Zurückgelassenen unter den arabischen Beduinen werden zu dir sagen: «Unser Vermögen und unsere Angehörigen haben uns zu sehr in Anspruch genommen. So bitte für uns um Vergebung.» Sie sagen mit ihrer Zunge, was nicht in ihrem Herzen ist. Sprich: Wer kann für euch vor Gott überhaupt etwas bewirken, wenn Er euch schaden will oder wenn Er euch nützen will? Nein, Gott hat Kenntnis von dem, was ihr tut.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdxcv{Nein, ihr meintet, daß der Gesandte und die Gläubigen nie mehr zu ihren Angehörigen heimkehren würden, und das ist euch in euren Herzen verlockend gemacht worden. Und ihr hattet eine schlechte Meinung und waret ein verlorenes Volk.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdxcvi{Und wenn einer an Gott und seinen Gesandten nicht glaubt, so haben Wir für die Ungläubigen einen Feuerbrand bereitet.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdxcvii{Und Gott gehört die Königsherrschaft der Himmel und der Erde. Er vergibt, wem Er will, und Er peinigt, wen Er will. Und Gott ist voller Vergebung und barmherzig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdxcviii{Die Zurückgelassenen werden, wenn ihr auszieht, um Beute zu machen, sagen: «Laßt uns euch folgen.» Sie möchten gern die Worte Gottes abändern. Sprich: «Ihr dürft uns nicht folgen. So hat Gott schon vorher gesprochen.» Sie werden sagen: «Nein, aber ihr hegt Neid uns gegenüber.» Nein, sie begreifen ja nur wenig.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdxcix{Sprich zu den Zurückgelassenen unter den arabischen Beduinen: «Ihr werdet dazu aufgerufen, gegen Leute, die eine starke Schlagkraft besitzen, zu kämpfen, es sei denn, sie ergeben sich. Wenn ihr gehorcht, läßt Gott euch einen schönen Lohn zukommen. Wenn ihr euch aber abkehrt, wie ihr euch vorher abgekehrt habt, dann peinigt Er euch mit einer schmerzhaften Pein.»\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdc{Es gibt keinen Grund zur Bedrängnis für den Blinden; es gibt keinen Grund zur Bedrängnis für den Krüppel; es gibt keinen Grund zur Bedrängnis für den Kranken. Wer Gott und seinem Gesandten gehorcht, den läßt Er in Gärten eingehen, unter denen Bäche fließen. Und wer sich abkehrt, den peinigt Er mit einer schmerzhaften Pein.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdci{Gott hat an den Gläubigen Wohlgefallen gefunden, als sie dir unter dem Baum Treue gelobten. Da wußte Er, was in ihren Herzen war, und Er sandte die Ruhe spendende Gegenwart auf sie herab und schenkte ihnen als Lohn einen nahen Erfolg\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcii{Und viel Beute, die sie nehmen würden. Und Gott ist mächtig und weise.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdciii{Gott hat euch versprochen, daß ihr viel Beute machen würdet. So hat Er euch erst diese eilig zufallen lassen und die Hände der Menschen von euch zurückgehalten. Dies, damit es ein Zeichen für die Gläubigen sei und Er euch einen geraden Weg führe.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdciv{Auch eine andere (Beute), die ihr nicht zu erreichen vermochtet, hat Gott schon umfangen. Gott hat Macht zu allen Dingen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcv{Und hätten diejenigen, die ungläubig sind, gegen euch gekämpft, sie hätten den Rücken gekehrt. Und dann finden sie weder Freund noch Helfer.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcvi{So war das beispielhafte Verfahren Gottes, das früher angewandt wurde. Und du wirst im Verfahren Gottes keine Veränderung finden.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcvii{Und Er ist es, der im Tal von Mekka ihre Hände von euch und eure Hände von ihnen zurückgehalten hat, nachdem Er euch den Sieg über sie verliehen hatte. Und Gott sieht wohl, was ihr tut.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcviii{Sie sind es, die ungläubig sind und euch von der heiligen Moschee abgewiesen und verhindert haben, daß die Opfertiere ihren Schlachtort erreichen. Und hätte es nicht gläubige Männer und gläubige Frauen gegeben, die ihr nicht kanntet, derentwegen aber euch verwehrt wurde, sie niederzutreten und dadurch unwissentlich ihretwegen Schuld auf euch zu laden... Dies, damit Gott in seine Barmherzigkeit eingehen läßt, wen Er will. Wenn diese sich abgesetzt hätten, hätten Wir diejenigen von ihnen, die ungläubig sind, mit einer schmerzhaften Pein gepeinigt.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcix{Als diejenigen, die ungläubig sind, in ihren Herzen den Verbandsgeist entfachten - den Verbandsgeist der Zeit der Unwissenheit -, worauf Gott seine Ruhe spendende Gegenwart auf seinen Gesandten und auf die Gläubigen herabsandte und sie auf das Wort der Gottesfurcht verpflichtete. Sie hatten ja eher den berechtigten Anspruch darauf und waren seiner würdig. Und Gott weiß über alle Dinge Bescheid.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdcx{Gott hat seinem Gesandten das Traumgesicht der Wahrheit entsprechend wahr gemacht: Ihr werdet gewiß, wenn Gott will, die heilige Moschee betreten in Sicherheit, sowohl mit geschorenem Kopf als auch mit gestutztem Haar, und ohne Angst zu haben. Und Er wußte, was ihr nicht wußtet, und Er bestimmte (für euch) außerdem einen nahen Erfolg.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcxi{Er ist es, der seinen Gesandten mit der Rechtleitung und der Religion der Wahrheit gesandt hat, um ihr die Oberhand zu verleihen über alle Religion. Und Gott genügt als Zeuge.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcxii{Muhammad ist der Gesandte Gottes. Und diejenigen, die mit ihm sind, sind den Ungläubigen gegenüber heftig, gegeneinander aber barmherzig. Du siehst, wie sie sich verneigen und niederwerfen im Streben nach Gottes Huld und Wohlgefallen. Ihr Merkmal steht auf ihrem Gesicht als Spur der Niederwerfung. Das ist ihre Beschreibung in der Tora. Beschrieben werden sie im Evangelium wie ein Saatfeld, das seine Triebe hervorbringt und stärker werden läßt, so daß sie verdicken und auf den Halmen stehen, zum Gefallen derer, die gesät haben. (Dies), damit Er die Ungläubigen durch sie in Wut versetze. Gott hat diejenigen von ihnen, die glauben und die guten Werke tun, Vergebung und großartigen Lohn versprochen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdcxiii{\basmalahde O ihr, die ihr glaubt, kommt nicht Gott und seinem Gesandten zuvor und fürchtet Gott. Gott hört und weiß alles.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcxiv{O ihr, die ihr glaubt, erhebt nicht eure Stimme über die Stimme des Propheten, und sprecht nicht so laut zu ihm, wie ihr laut zueinander sprecht, damit nicht eure Werke wertlos werden, ohne daß ihr es merkt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcxv{Diejenigen, die ihre Stimme bei dem Gesandten Gottes dämpfen, das sind die, deren Herzen Gott auf die Gottesfurcht geprüft hat. Bestimmt ist für sie Vergebung und großartiger Lohn.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcxvi{Von denen, die dich rufen, während du dich in den Gemächern aufhältst, haben die meisten keinen Verstand.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcxvii{Würden sie sich gedulden, bis du zu ihnen herauskommst, wäre es besser für sie. Und Gott ist voller Vergebung und barmherzig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdcxviii{O ihr, die ihr glaubt, wenn ein Frevler mit einer Nachricht zu euch kommt, so stellt es eindeutig fest, damit ihr nicht (einigen) Leuten in Unwissenheit etwas antut und dann bereuen müßt, was ihr getan habt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdcxix{Und wißt, daß der Gesandte Gottes unter euch weilt. Würde er in vielen Angelegenheiten euch gehorchen, würdet ihr in Bedrängnis geraten. Aber Gott hat euch den Glauben lieb und in euren Herzen anziehend gemacht, und Er hat euch den Unglauben, den Frevel und den Ungehorsam verabscheuen lassen. Das sind die, die den rechten Wandel zeigen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdcxx{Dies als Huld und Gnade von Gott. Und Gott weiß Bescheid und ist weise.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdcxxi{Und wenn zwei Gruppen von den Gläubigen einander bekämpfen, so stiftet Frieden zwischen ihnen. Wenn die eine von ihnen gegen die andere in ungerechter Weise vorgeht, dann kämpft gegen diejenige, die in ungerechter Weise vorgeht, bis sie zum Befehl Gottes umkehrt. Wenn sie umkehrt, dann stiftet Frieden zwischen ihnen nach Gerechtigkeit und handelt dabei gerecht. Gott liebt die, die gerecht handeln.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdcxxii{Die Gläubigen sind ja Brüder. So stiftet Frieden zwischen euren beiden Brüdern und fürchtet Gott, auf daß ihr Erbarmen findet.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdcxxiii{O ihr, die ihr glaubt, die einen sollen nicht die anderen verhöhnen, vielleicht sind diese eben besser als sie. Auch sollen nicht Frauen andere Frauen verhöhnen, vielleicht sind diese eben besser als sie. Und nörgelt nicht untereinander und gebt einander keine Schimpfnamen. Welch schlimmer Name, der des Frevels, nach der Annahme des Glaubens! Diejenigen, die nicht umkehren, sind die, die Unrecht tun.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdcxxiv{O ihr, die ihr glaubt, meidet viel von den Mutmaßungen. Manche Mutmaßung ist Sünde. Spioniert nicht und führt nicht üble Nachrede übereinander. Möchte denn einer von euch das Fleisch seines Bruders, wenn er tot ist, essen? Es wäre euch doch zuwider. Fürchtet Gott. Gott wendet sich gnädig zu und ist barmherzig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdcxxv{O ihr Menschen, Wir haben euch von einem männlichen und einem weiblichen Wesen erschaffen, und Wir haben euch zu Verbänden und Stämmen gemacht, damit ihr einander kennenlernt. Der Angesehenste von euch bei Gott, das ist der Gottesfürchtigste von euch. Gott weiß Bescheid und hat Kenntnis von allem.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdcxxvi{Die arabischen Beduinen sagen: «Wir glauben.» Sprich: Ihr glaubt nicht (wirklich). Sagt vielmehr: Wir sind Muslime geworden. Der Glaube ist ja noch nicht in eure Herzen gedrungen. Wenn ihr aber Gott und seinem Gesandten gehorcht, verringert Er euch nichts von euren Werken. Gott ist voller Vergebung und barmherzig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdcxxvii{Die Gläubigen, das sind diejenigen, die an Gott und seinen Gesandten glauben und dann nicht zweifeln und sich mit ihrem Vermögen und mit ihrer eigenen Person auf dem Weg Gottes einsetzen. Das sind die Wahrhaftigen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdcxxviii{Sprich: Wollt ihr denn Gott über eure Religion belehren, wo Gott weiß, was in den Himmeln und was auf der Erde ist? Und Gott weiß über alle Dinge Bescheid.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcxxix{Sie halten es für eine Wohltat gegen dich, daß sie Muslime geworden sind. Sprich: Haltet nicht eure Annahme des Islams für eine Wohltat gegen mich. Nein, Gott hat euch eine Wohltat erwiesen dadurch, daß Er euch zum Glauben geführt hat, wenn ihr wahrhaftig seid.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcxxx{Gott kennt das Unsichtbare der Himmel und der Erde. Und Gott sieht wohl, was ihr tut.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxi{\basmalahde Qaaf. Beim glorreichen Koran!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxii{Aber nein, sie wundern sich darüber, daß ein Warner aus ihrer Mitte zu ihnen kam. Da sagen die Ungläubigen: «Das ist eine verwunderliche Sache.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxiii{Sollen wir wirklich, wenn wir gestorben und zu Staub geworden sind...? Das ist doch eine Rückkehr von weiter Entfernung.»\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxiv{Wir wissen wohl, was die Erde von ihnen aufzehrt. Und bei Uns ist ein Buch, das alles aufbewahrt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxv{Aber nein, für Lüge erklärten sie die Wahrheit, als sie zu ihnen kam, und so befinden sie sich in einer verworrenen Lage.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxvi{Haben sie nicht zum Himmel über ihnen emporgeschaut, wie Wir ihn aufgebaut und geschmückt haben, und daß er keine Risse aufweist?\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxvii{Auch die Erde haben Wir ausgebreitet und auf ihr festgegründete Berge angebracht, und Wir haben auf ihr verschiedene erfreuliche Pflanzenarten wachsen lassen,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxviii{Um Einsicht zu bringen und als Ermahnung für jeden Diener, der sich (Gott) reumütig zuwendet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxix{Und Wir haben vom Himmel gesegnetes Wasser herabkommen und damit Gärten und Korn für die Ernte wachsen lassen,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdcxl{Und Palmen, hochragend und mit übereinander gereihten Blütenscheiden,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdcxli{Als Lebensunterhalt für die Diener. Und Wir haben damit abgestorbene Ortschaften wieder belebt. So erfolgt auch das Herauskommen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdcxlii{Der Lüge ziehen (ihre Gesandten) schon vor ihnen das Volk Noachs, die Leute des Brunnens und die Thamud,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdcxliii{Und die Aad, Pharao und die Brüder Lots,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdcxliv{Die Leute des Waldes und das Volk des Tubbaá. Alle haben die Gesandten der Lüge geziehen, so ist meine Androhung zu Recht fällig geworden.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdcxlv{Waren Wir denn mit der ersten Schöpfung überfordert? Nein, sie aber sind im unklaren über eine neue Schöpfung.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdcxlvi{Wir haben doch den Menschen erschaffen und wissen, was ihm seine Seele einflüstert. Und Wir sind ihm näher als die Halsschlagader.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcxlvii{Wenn die beiden, die entgegenzunehmen haben, entgegennehmen, zur Rechten und zur Linken sitzend,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcxlviii{Wird er kein Wort sprechen, ohne daß bei ihm ein Bewacher bereitstünde.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcxlix{Und die Trunkenheit des Todes bringt die Wahrheit: «Das ist das, wovor du auszuweichen pflegtest.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdcl{Und es wird in die Trompete geblasen. Das ist der angedrohte Tag.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcli{Und jede Seele kommt, und mit ihr ein Treiber und ein Zeuge.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdclii{«Du hast dies unbeachtet gelassen. Nun haben Wir deine Decke von dir weggezogen, so daß dein Augenlicht heute scharf ist.»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcliii{Und sein Geselle sagt: «Das ist, was bei mir bereitliegt.»\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcliv{- «Werft, ihr beide, in die Hölle jeden, der sehr ungläubig und widerspenstig ist,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdclv{Das Gute verweigert, Übertretungen begeht und Zweifel hegt,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdclvi{Und der Gott einen anderen Gott zur Seite stellt. So werft ihn in die harte Pein.»\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdclvii{Sein Geselle sagt: «Unser Herr, ich habe ihn nicht zu übermäßigem Frevel verleitet, sondern er befand sich selbst in tiefem Irrtum.»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdclviii{Er spricht: «Streitet euch nicht vor Mir, wo Ich euch die Androhung vorausgeschickt habe.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdclix{Der Spruch wird bei Mir nicht abgeändert. Und Ich tue den Dienern kein Unrecht.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdclx{Dies) am Tag, da Wir zur Hölle sprechen: «Bist du bereits voll?», und sie sagt: «Gibt es noch mehr?»\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdclxi{Und das Paradies wird an die Gottesfürchtigen herangebracht, nicht weit entfernt:\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdclxii{«Das ist, was euch versprochen worden ist für jeden, der bereit zur Umkehr ist und sich selbst hütet,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdclxiii{Der den Erbarmer im verborgenen fürchtet und mit reumütigem Herzen (zu Ihm) kommt.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdclxiv{Geht hinein in Frieden. Das ist der Tag der Ewigkeit.»\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdclxv{Sie haben darin, was sie wollen. Und bei Uns steht noch mehr.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdclxvi{Wie viele Generationen haben Wir vor ihnen verderben lassen, die eine stärkere Gewalt besaßen als sie und in den Ländern umherzogen! Gibt es ein Entrinnen?\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdclxvii{Darin ist eine Ermahnung für den, der Herz hat oder hinhört, während er zugegen ist.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdclxviii{Und Wir haben die Himmel und die Erde und das, was dazwischen ist, in sechs Tagen erschaffen. Und dabei hat Uns keine Ermattung befallen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdclxix{So ertrage mit Geduld, was sie sagen. Und singe das Lob deines Herrn vor dem Aufgang der Sonne und vor dem Untergang,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdclxx{Und preise Ihn in der Nacht und jeweils am Ende der Niederwerfung.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdclxxi{Und höre zu am Tag, da der Rufer aus einem nahen Ort ruft,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdclxxii{Am Tag, da sie den Schrei in Wahrheit hören. Das ist der Tag des Herauskommens.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdclxxiii{Wahrlich, Wir allein machen wieder lebendig und lassen sterben. Und zu Uns führt der Lebensweg.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdclxxiv{Am Tag, da die Erde sich vor ihnen spaltet und sie eilig (hervorkommen), das ist ein Versammeln, das Uns ein leichtes ist.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdclxxv{Wir wissen besser, was sie sagen. Und du bist nicht ein Zwingherr über sie. Ermahne nur mit dem Koran die, welche meine Drohung fürchten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdclxxvi{\basmalahde Bei denen, die heftig aufwirbeln,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdclxxvii{Bei denen, die eine Last tragen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdclxxviii{Bei denen, die leicht dahinziehen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdclxxix{Bei denen, die verschiedene Befehle ausführen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdclxxx{Siehe, was euch angedroht ist, ist wahr,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxi{Und das Gericht wird eintreffen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxii{Beim Himmel voller Adern,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxiii{Ihr haltet unterschiedliche Reden.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxiv{Der wird davon abwendig gemacht, wer sich abwenden läßt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxv{Dem Tod geweiht seien die, die nur Schätzungen anstellen,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxvi{Die sich in abgründiger Achtlosigkeit befinden.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxvii{Sie fragen: «Wann wird denn der Tag des Gerichtes sein?»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxviii{- «Am Tag, da sie im Feuer geprüft werden.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxix{Kostet nun eure Prüfung. Das ist es, was ihr zu beschleunigen wünschtet.»\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdcxc{Die Gottesfürchtigen aber werden in Gärten und an Quellen sein,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdcxci{Sie nehmen, was ihr Herr ihnen zukommen läßt. Sie waren ja vordem rechtschaffen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcxcii{Nur ein wenig schliefen sie in der Nacht,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcxciii{Und in der Morgendämmerung baten sie um Vergebung,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcxciv{Und sie räumten dem Bettler und dem Unbemittelten ein Recht auf ihr Vermögen ein.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdcxcv{Und auf der Erde gibt es Zeichen für die, die Gewißheit hegen,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcxcvi{Und auch in euch selbst. Wollt ihr denn nicht sehen?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcxcvii{Und im Himmel ist euer Lebensunterhalt und das, was euch versprochen wird.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcxcviii{Beim Herrn des Himmels und der Erde, das ist so wahr, wie ihr reden könnt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcxcix{Ist denn die Geschichte von den geehrten Gästen Abrahams zu dir gelangt?\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcc{Als sie bei ihm eintraten und sagten: «Friede!» Er sagte: «Friede! Lauter unbekannte Leute.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcci{Er schlich sich zu seinen Angehörigen. Er brachte ein (gebratenes) Mastkalb herbei\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdccii{Und setzte es ihnen vor. Er sagte: «Wollt ihr nicht essen?»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcciii{Er empfand Angst vor ihnen. Sie sagten: «Hab keine Angst.» Und sie verkündeten ihm einen klugen Knaben.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcciv{Da kam seine Frau schreiend herbei. Sie schlug sich ins Gesicht und sagte: «Eine unfruchtbare alte Frau!»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdccv{Sie sagten: «So hat dein Herr gesprochen. Er ist der, der weise ist und Bescheid weiß.»\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdccvi{Er sagte: «Was ist euer Anliegen, ihr Boten?»\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdccvii{Sie sagten: «Wir sind gesandt zu Leuten, die Übeltäter sind,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdccviii{Um über sie Steine aus Ton niederkommen zu lassen,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdccix{Die bei deinem Herrn gekennzeichnet sind für die Maßlosen.»\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdccx{Und Wir brachten die, die in ihr gläubig waren, hinaus.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdccxi{Aber Wir fanden in ihr nur ein (einziges) Haus von Gottergebenen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdccxii{Und Wir hinterließen in ihr ein Zeichen für diejenigen, die die schmerzhafte Pein fürchten.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdccxiii{Ein Zeichen ist) auch in Mose, als Wir ihn mit einer offenkundigen Ermächtigung zu Pharao sandten.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdccxiv{Der kehrte sich ab samt seiner Streitmacht und sagte: «Ein Zauberer oder ein Besessener.»\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdccxv{Da ergriffen Wir ihn und seine Truppen und warfen sie ins Meer. Er hatte sich Tadel zugezogen.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdccxvi{Und auch in den Aad, als Wir über sie den lebensfeindlichen Wind sandten,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdccxvii{Der nichts von allem, worüber er kam, übrigließ, ohne es werden zu lassen wie etwas, was auseinanderfällt.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdccxviii{Und auch in den Thamud, als zu ihnen gesprochen wurde: «Genießt für eine Weile.»\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdccxix{Sie rebellierten gegen den Befehl ihres Herrn. Da ergriff sie der Donnerschlag, während sie hinschauten,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdccxx{Und sie konnten sich nicht mehr aufrichten und erfuhren keine Unterstützung mehr.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdccxxi{Verderben ließen Wir) früher auch das Volk Noachs. Das waren frevlerische Leute.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdccxxii{Und den Himmel haben Wir mit Kraft aufgebaut. Und Wir verfügen über breite Möglichkeiten.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdccxxiii{Und die Erde haben Wir ausgebreitet. Wie trefflich haben Wir sie geebnet!\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdccxxiv{Und von allem haben Wir ein Paar erschaffen, auf daß ihr es bedenket.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdccxxv{Flüchtet nun zu Gott. Ich bin euch von Ihm ein deutlicher Warner.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmdccxxvi{Und setzt nicht Gott einen anderen Gott zur Seite. Ich bin euch von Ihm ein deutlicher Warner.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmdccxxvii{So ist auch zu denen, die vor ihnen lebten, kein Gesandter gekommen, ohne daß sie gesagt hätten: «Ein Zauberer oder ein Besessener.»\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmdccxxviii{Haben sie es denn einander (als Vermächtnis) aufgetragen? Nein, sie sind Leute, die ein Übermaß an Frevel zeigen.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmdccxxix{So kehre dich von ihnen ab. Du ziehst dir damit keinen Tadel zu.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmdccxxx{Und ermahne, denn die Ermahnung nützt den Gläubigen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxi{Und Ich habe die Djinn und die Menschen nur dazu erschaffen, daß sie Mir dienen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxii{Ich will von ihnen keinen Unterhalt, und Ich will nicht, daß sie Mir zu essen geben.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxiii{Gott ist es, der Unterhalt beschert und Kraft und Festigkeit besitzt.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxiv{Für diejenigen, die Unrecht tun, ist ein Los bestimmt wie das Los ihrer Gefährten. So sollen sie Mich nicht um Beschleunigung bitten.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxv{Wehe denen, die ungläubig sind, vor ihrem Tag, der ihnen angedroht ist!\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxvi{\basmalahde Beim Berg;\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxvii{Bei einem Buch, das zeilenweise niedergeschrieben ist\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxviii{Auf ausgebreitetem Pergament;\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxix{Bei dem (zur Pilgerzeit) besuchten Haus\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdccxl{Und bei der hochgezogenen Decke,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdccxli{Und beim angefüllten Meer!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdccxlii{Die Pein deines Herrn wird eintreffen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdccxliii{Es gibt niemanden, der sie abwehren könnte.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdccxliv{Am Tag, da der Himmel in heftiges Schwanken gerät\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdccxlv{Und die Berge sich fortbewegen,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdccxlvi{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdccxlvii{Die in ausschweifenden Reden ihr Spiel treiben!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdccxlviii{Am Tag, da sie unerbittlich ins Feuer der Hölle gestoßen werden:\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdccxlix{«Das ist das Feuer, das ihr immer wieder für Lüge erklärt habt.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdccl{Ist dies nun Zauberei, oder könnt ihr nicht sehen?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdccli{Ihr sollt darin brennen. Es ist für euch gleich, ob ihr es geduldig ertragt oder nicht. Euch wird das vergolten, was ihr zu tun pflegtet.»\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcclii{Die Gottesfürchtigen befinden sich in Gärten und Wonne.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdccliii{Sie empfinden Wohlbehagen an dem, was ihnen ihr Herr zukommen läßt. Und ihr Herr bewahrt sie vor der Pein der Hölle.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdccliv{«Eßt und trinkt zu eurem Wohl für das, was ihr zu tun pflegtet.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdcclv{Sie lehnen sich auf gereihten Betten. Und Wir vermählen sie mit großäugigen Huri.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcclvi{Und zu denen, die glauben und denen ihre Nachkommenschaft im Glauben folgt, lassen Wir ihre Nachkommenschaft stoßen. Und Wir verringern ihnen nichts von ihren Werken. Ein jeder haftet für das, was er erworben hat.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcclvii{Und Wir versorgen sie mit Früchten und Fleisch von dem, was sie begehren.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcclviii{Darin greifen sie untereinander nach einem Becher, der nicht zu unbedachter Rede verleitet und in dem nichts Sündhaftes steckt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcclix{Und unter ihnen machen die Runde Jünglinge, die zu ihnen gehören, als wären sie wohlverwahrte Perlen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcclx{Und sie treten aneinander heran und fragen sich gegenseitig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcclxi{Sie sagen: «Früher waren wir inmitten unserer Angehörigen erschrocken.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdcclxii{Da erwies Gott uns eine Wohltat und bewahrte uns vor der Pein des glühenden Windes.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcclxiii{Früher pflegten wir Ihn anzurufen. Er ist der, der gütig und barmherzig ist.»\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcclxiv{Ermahne nun; du bist dank der Gnade deines Herrn weder ein Wahrsager noch ein Besessener.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdcclxv{Oder sagen sie etwa: «Ein Dichter. Wir warten ab, was für ein Unheil das Schicksal ihm bringen wird»?\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdcclxvi{Sprich: Wartet ab, ich gehöre mit euch zu denen, die abwarten.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdcclxvii{Oder befiehlt ihnen etwa ihr Verstand dies, oder sind sie vielmehr Leute, die ein Übermaß an Frevel zeigen?\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdcclxviii{Oder sagen sie: «Er hat ihn sich selbst in den Mund gelegt»? Nein, vielmehr glauben sie nicht.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdcclxix{Sie sollen doch eine Botschaft gleicher Art beibringen, so sie die Wahrheit sagen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdcclxx{Oder sind sie aus etwas anderem erschaffen worden, oder sind sie gar selbst die Schöpfer?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxi{Oder haben sie etwa die Himmel und die Erde erschaffen? Nein, vielmehr hegen sie keine Gewißheit.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxii{Oder besitzen sie etwa die Vorratskammern deines Herrn, oder sind sie es, die alles fest in der Hand haben?\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxiii{Oder haben sie eine Leiter, auf der sie (emporsteigen und) lauschen können? Dann soll doch ihr Lauscher eine offenkundige Ermächtigung beibringen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxiv{Oder sind für Ihn die Töchter und für euch die Söhne bestimmt?\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxv{Oder verlangst du etwa einen Lohn von ihnen, so daß sie mit Schulden belastet wären?\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxvi{Oder liegt ihnen das Unsichtbare vor, so daß sie (es) aufschreiben?\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxvii{Oder wollen sie arglistig handeln? Diejenigen, die ungläubig sind, sind es, die der List erliegen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxviii{Oder haben sie einen anderen Gott als Gott? Preis sei Gott! (Erhaben ist Er) über das, was sie (Ihm) beigesellen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxix{Und sähen sie ein Stück vom Himmel herabfallen, sie würden sagen: «Eine aufgeschichtete Wolke.»\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxx{Laß sie nur, bis sie ihrem Tag begegnen, an dem sie vom Donnerschlag getroffen werden,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxi{Dem Tag, da ihre List ihnen nichts nützt und sie keine Unterstützung erfahren.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxii{Für diejenigen, die Unrecht tun, ist eine Pein außer dieser bestimmt, aber die meisten von ihnen wissen nicht Bescheid.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxiii{Sei geduldig, bis dein Herr sein Urteil fällt. Du stehst vor unseren Augen. Und sing das Lob deines Herrn, wenn du dich hinstellst.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxiv{Und preise Ihn in der Nacht und beim Schwinden der Sterne.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxv{\basmalahde Beim Stern, wenn er fällt!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxvi{Euer Gefährte geht nicht irre und ist nicht einem Irrtum erlegen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxvii{Und er redet nicht aus eigener Neigung.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxviii{Es ist nichts anderes als eine Offenbarung, die offenbart wird.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxix{Belehrt hat ihn einer, der starke Kräfte hat,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdccxc{Der Macht besitzt. Er stand aufrecht da,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdccxci{Am obersten Horizont.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdccxcii{Dann kam er näher und stieg nach unten,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdccxciii{So daß er (nur) zwei Bogenlängen entfernt war oder noch näher.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdccxciv{Da offenbarte Er seinem Diener, was Er offenbarte.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdccxcv{Sein Herz hat nicht gelogen, was er sah.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdccxcvi{Wollt ihr denn mit ihm streiten über das, was er sieht?\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdccxcvii{Und er sah ihn ein anderes Mal herabkommen,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdccxcviii{Beim Zizyphusbaum am Ende des Weges,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdccxcix{Bei dem der Garten der Heimstätte ist.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdccc{Als den Zizyphusbaum bedeckte, was (ihn) bedeckte,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdccci{Da wich der Blick nicht ab, und er überschritt das Maß nicht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcccii{Wahrlich, er sah etwas von den größten Zeichen seines Herrn.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdccciii{Habt ihr Laat und, Uzzaa gesehen,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdccciv{Und auch Manaat, diese andere, die dritte?\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcccv{Ist denn für euch das, was männlich ist, und für Ihn das, was weiblich ist, bestimmt?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcccvi{Das wäre dann eine ungerechte Verteilung.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcccvii{Das sind nur Namen, die ihr genannt habt, ihr und eure Väter, für die Gott aber keine Ermächtigung herabgesandt hat. Sie folgen nur Vermutungen und dem, wozu ihre Seelen neigen, wo doch die Rechtleitung von ihrem Herrn zu ihnen gekommen ist.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcccviii{Oder kann denn der Mensch haben, was er wünscht?\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcccix{Ja, Gott gehört das Jenseits und das Diesseits.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcccx{Und wie manche Engel gibt es in den Himmeln, deren Fürsprache nichts nützt, außer nachdem Gott es einem erlaubt hat, den Er will und der sein Gefallen hat.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdcccxi{Diejenigen, die an das Jenseits nicht glauben, geben den Engeln Namen, wie man sie weiblichen Wesen gibt.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcccxii{Sie haben jedoch kein Wissen darüber. Sie folgen nur Vermutungen. Aber die Vermutungen nützen in bezug auf die Wahrheit nichts.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcccxiii{Wende dich nun ab von dem, der sich von unserer Ermahnung abkehrt und nur das diesseitige Leben will.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdcccxiv{Das ist, was sie vom Wissen erreicht haben. Dein Herr weiß besser, wer von seinem Weg abirrt, und Er weiß besser, wer der Rechtleitung folgt.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdcccxv{Gott gehört, was in den Himmeln und was auf der Erde ist, auf daß Er denen, die Böses tun, vergelte für das, was sie tun, und daß Er denen, die Gutes tun, mit dem Allerbesten vergelte.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdcccxvi{Diejenigen, die die schweren Sünden und die schändlichen Taten meiden, abgesehen von leichten Verfehlungen... Wahrlich, dein Herr hat eine umfassende Vergebung. Er weiß besser über euch Bescheid, als Er euch aus der Erde entstehen ließ und als ihr Embryos im Leib eurer Mütter waret. So erklärt nicht euch selbst für rein. Er weiß besser, wer gottesfürchtig ist.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdcccxvii{Hast du den gesehen, der sich abkehrt\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdcccxviii{Und nur wenig gibt und dann (auch) aufhört?\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdcccxix{Hat er etwa Wissen über das Unsichtbare, daß er (es) sehen könnte?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdcccxx{Oder wurde ihm nicht verkündet, was da steht in den Blättern des Mose\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxi{Und des Abraham, der (seine Pflichten) erfüllte?\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxii{Nämlich), daß keine lasttragende (Seele) die Last einer anderen tragen wird.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxiii{Daß für den Menschen nur das bestimmt ist, wonach er strebt,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxiv{Daß sein Streben sichtbar werden wird\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxv{Und daß ihm hierauf voll dafür vergolten wird.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxvi{Daß das Ende des Weges zu Gott führt,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxvii{Daß Er lachen und weinen läßt\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxviii{Und daß Er sterben läßt und wieder lebendig macht.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxix{Daß Er beide Geschlechter, das männliche und das weibliche, erschafft\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxx{Aus einem Samentropfen, wenn er sich ergießt,\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxi{Und daß Ihm die zweite Schöpfung obliegt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxii{Daß Er reich macht und Besitz schenkt,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxiii{Und daß Er der Herr des Sirius ist.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxiv{Und daß Er die Aad, die damals lebten, verderben ließ,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxv{Und auch die Thamud, wobei Er nichts übrigließ,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxvi{Und vorher das Volk Noachs - sie waren ja Leute, die noch mehr Unrecht taten und das Übermaß ihres Frevels noch steigerten.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxvii{Und daß Er die verschwundene Stadt stürzen ließ,\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxviii{So daß sie bedeckte, was (sie dann) bedeckte.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxix{Welche der Wohltaten deines Herrn willst du nun bestreiten?\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmdcccxl{Das ist ein Warner nach Art der früheren Warner.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmdcccxli{Die nahende Stunde steht bevor.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlii{Niemand außer Gott kann sie beheben.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmdcccxliii{Wundert ihr euch denn über diese Botschaft\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmdcccxliv{Und lacht, statt zu weinen,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlv{Während ihr euch mit etwas anderem beschäftigt?\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlvi{Werft euch doch vor Gott nieder und dienet (Ihm).\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlvii{\basmalahde Nahegerückt ist die Stunde, und gespalten hat sich der Mond.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlviii{Und wenn sie ein Zeichen sehen, wenden sie sich ab und sagen: «Eine ständige Zauberei.»\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlix{Und sie erklären (es) für Lüge und folgen ihren Neigungen. Doch jede Angelegenheit wird bestätigt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcccl{Zu ihnen ist doch von den Berichten gekommen, was eine Zurechtweisung enthält,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcccli{Eine triftige Weisheit. Aber da nützen die Warnungen nicht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdccclii{So kehre dich von ihnen ab. Am Tag, da der Rufer zu etwas Unangenehmem ruft\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdcccliii{Und sie aus den Gräbern wie ausschwärmende Heuschrecken herauskommen, mit gesenkten Blicken,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdcccliv{Den Hals nach dem Rufer gereckt, da sagen die Ungläubigen: «Das ist ein schwerer Tag.»\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdccclv{Schon vor ihnen hat das Volk Noachs (ihren Gesandten) der Lüge geziehen. Sie ziehen unseren Diener der Lüge und sagten: «Ein Besessener.» Und er wurde (von ihnen) zurechtgewiesen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdccclvi{Da rief er zu seinem Herrn: «Ich bin unterlegen, so unterstütze Du (mich).»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdccclvii{Da öffneten Wir die Tore des Himmels durch ein niederströmendes Wasser\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdccclviii{Und ließen aus der Erde Quellen hervorbrechen. Und das Wasser traf zusammen nach testgelegtem Befehl.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdccclix{Und Wir trugen ihn auf einem Schiff mit Planken und Nägeln,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdccclx{Das vor unseren Augen dahinfuhr. Dies als Lohn für den, der Verleugnung erfahren hatte.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdccclxi{Und Wir ließen es als Zeichen zurück. Aber gibt es denn jemanden, der es bedenkt?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdccclxii{Und wie waren dann meine Pein und meine Warnungen!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdccclxiii{Und Wir haben den Koran leicht zu bedenken gemacht. Aber gibt es denn jemanden, der es bedenkt?\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdccclxiv{Die Aad haben (ihren Gesandten) der Lüge geziehen. Und wie waren dann meine Pein und meine Warnungen!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdccclxv{Wir sandten gegen sie an einem langwährenden unheilvollen Tag einen eiskalten Wind,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdccclxvi{Der die Menschen fortriß, als wären sie Stämme entwurzelter Palmen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdccclxvii{Wie waren dann meine Pein und meine Warnungen!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdccclxviii{Und Wir haben den Koran leicht zu bedenken gemacht. Aber gibt es denn jemanden, der es bedenkt?\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdccclxix{Die Thamud haben die Warner der Lüge geziehen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdccclxx{Sie sagten: «Sollen wir denn einem Menschen aus unseren Reihen, der allein dasteht, folgen? Dann befänden wir uns im Irrtum und litten an Wahnsinn.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxi{Ist die Ermahnung wirklich gerade ihm aus unserer Mitte überbracht worden? Nein, vielmehr ist er ein überheblicher Lügner.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxii{Morgen werden sie zu wissen bekommen, wer der überhebliche Lügner ist.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxiii{Wir werden die Kamelstute schicken als Versuchung für sie. So beobachte sie und sei geduldig.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxiv{Und gib ihnen kund, daß das Wasser zwischen ihnen zu teilen ist. Jede Trinkzeit soll (dann abwechselnd) wahrgenommen werden.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxv{Sie aber riefen ihren Gefährten her. Er griff zu und schnitt (der Kamelstute) die Flechsen durch und stach sie.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxvi{Wie waren dann meine Pein und meine Warnungen!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxvii{Wir sandten über sie einen einzigen Schrei, da waren sie plötzlich wie das dürre Zeug dessen, der Tiergehege baut.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxviii{Und Wir haben den Koran leicht zu bedenken gemacht. Aber gibt es denn jemanden, der es bedenkt?\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxix{Das Volk Lots hat die Warner der Lüge geziehen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxx{Wir sandten einen Steinregen über sie, ausgenommen die Angehörigen Lots. Wir erretteten sie im Morgengrauen,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxi{Aus Gnade von Uns. So entlohnen Wir den, der dankbar ist.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxii{Er hatte sie vor unserem gewaltigen Zugriff gewarnt. Aber sie bestritten die Warnungen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxiii{Sie versuchten ihn in bezug auf seine Gäste zu überreden. Da wischten Wir ihre Augen aus. «Kostet nun meine Pein und meine Warnungen.»\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxiv{Und am Morgen ereilte sie eine beständige Pein.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxv{«Kostet nun meine Pein und meine Warnungen.»\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxvi{Und Wir haben den Koran leicht zu bedenken gemacht. Aber gibt es denn jemanden, der es bedenkt?\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxvii{Und (auch) zu den Leuten Pharaos kamen die Warnungen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxviii{Aber sie erklärten alle unsere Zeichen für Lüge. Da ergriffen Wir sie, wie ein Starker und Mächtiger ergreift.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxix{Sind etwa eure Ungläubigen besser als jene? Oder habt ihr einen Freispruch in den Schriften?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdcccxc{Oder sagen sie: «Wir sind eine Schar, die siegen wird»?\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdcccxci{Die Schar wird in die Flucht geschlagen werden, und sie werden den Rücken kehren.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcii{Nein, die Stunde ist ihre Verabredungszeit. Und die Stunde ist noch unheilvoller und bitterer.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdcccxciii{Die Übeltäter befinden sich im Irrtum und leiden an Wahnsinn.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdcccxciv{Am Tag, da sie auf ihren Gesichtern ins Feuer hineingezerrt werden: «Kostet die Berührung des Höllenbrandes.»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcv{Wir haben alle Dinge nach Maß erschaffen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcvi{Und unser Befehl ist ein einziges Wort, gleich einem schnellen Blick.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcvii{Und Wir haben doch (früher) euresgleichen verderben lassen. Gibt es denn jemanden, der es bedenkt?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcviii{Alles, was sie getan haben, steht in den Schriften.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcix{Alles, ob klein oder groß, ist zeilenweise niedergeschrieben.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcm{Die Gottesfürchtigen werden in Gärten und an Bächen sein,\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcmi{Auf einem wahrhaftigen Sitzplatz, bei einem mächtigen König.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcmii{\basmalahde Der Erbarmer\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcmiii{Hat den Koran gelehrt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcmiv{Er hat den Menschen erschaffen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcmv{Er hat ihn deutliche Rede gelehrt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcmvi{Die Sonne und der Mond laufen nach Berechnung.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcmvii{Der Stern und die Bäume werfen sich nieder.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcmviii{Den Himmel hat Er emporgehoben und die Waage aufgestellt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcmix{Ihr sollt beim Wägen nicht das Maß überschreiten,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcmx{Das Gewicht nach Gerechtigkeit messen und beim Wägen nicht weniger geben.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcmxi{Und die Erde hat Er für die Geschöpfe gelegt;\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcmxii{Auf ihr gibt es Früchte, Palmen mit Blütenscheiden,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcmxiii{Korn auf Halmen und duftende Pflanzen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcmxiv{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcmxv{Er hat den Menschen aus einer Trockenmasse wie dem Töpferton erschaffen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcmxvi{Und Er hat die Djinn aus einer Feuerflamme erschaffen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcmxvii{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcmxviii{Er ist) der Herr der beiden Gegenden des Ostens und der Herr der beiden Gegenden des Westens.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcmxix{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcmxx{Er hat die beiden Meere zugleich entstehen lassen, die zusammentreffen,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcmxxi{Zwischen denen aber eine Schranke steht, daß sie nicht überlaufen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcmxxii{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcmxxiii{Aus ihnen beiden kommen Perlen und Korallen hervor.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcmxxiv{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcmxxv{Ihm gehören die gebauten Schiffe, die wie Berge auf dem Meer fahren.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcmxxvi{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcmxxvii{Alle, die auf ihr sind, werden vergehen;\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmcmxxviii{Bleiben wird nur das Antlitz deines Herrn, das erhabene und ehrwürdige.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmcmxxix{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmcmxxx{Ihn bitten diejenigen, die in den Himmeln und auf der Erde sind. Jeden Tag hat Er mit einem Anliegen zu tun.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxi{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxii{Wir werden Uns mit euch befassen, ihr beiden Machtbereiche.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxiii{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxiv{O ihr Verband der Djinn und der Menschen, wenn ihr aus den Regionen der Himmel und der Erde herauskommen könnt, so kommt doch heraus. Ihr werdet nicht herauskommen, außer mit einer Ermächtigung.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxv{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxvi{Über euch beide wird eine Feuerflamme und Qualm geschickt, und ihr werdet euch selbst nicht helfen können.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxvii{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxviii{Wenn der Himmel sich spaltet und rosig wie rotes Leder wird\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxix{- welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären? -,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcmxl{An jenem Tag werden weder die Menschen noch die Djinn nach ihren (einzelnen) Sünden gefragt.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcmxli{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcmxlii{Die Übeltäter werden an ihrem Merkmal erkannt und dann am Schopf und an den Füßen gepackt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcmxliii{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcmxliv{Das ist die Hölle, die die Übeltäter für Lüge erklären.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcmxlv{Sie machen zwischen ihr und kochend heißem Wasser die Runde.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcmxlvi{Welche der Wohltaten eures Herrn wollt ihr für Lüge erklären?\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcmxlvii{Für den, der den Stand seines Herrn fürchtet, sind zwei Gärten bestimmt\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcmxlviii{- welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären? -,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcmxlix{Die Zweige haben.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcml{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcmli{Darin fließen zwei Quellen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcmlii{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcmliii{Darin gibt es von jeder Frucht ein Paar.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcmliv{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcmlv{Sie lehnen sich auf Unterlagen, deren Futter aus Brokat sind. Und die Ernte der beiden Gärten hängt herab.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcmlvi{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcmlvii{Darin sind (Frauen), die ihre Blicke zurückhalten, die vor ihnen weder Mensch noch Djinn beschlafen hat -\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcmlviii{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären? -,\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcmlix{Als wären sie Hyazinth und Korallen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcmlx{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcmlxi{Kann einer, der Gutes tut, anders entlohnt werden als dadurch, daß ihm Gutes getan wird?\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmcmlxii{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmcmlxiii{Außer ihnen beiden gibt es zwei andere Gärten\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmcmlxiv{- welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären? -,\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmcmlxv{Dunkelgrüne.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcmlxvi{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcmlxvii{Darin sind zwei ergiebig sprudelnde Quellen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcmlxviii{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcmlxix{Darin sind Früchte und Palmen und Granatapfelbäume.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmcmlxx{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxi{Darin sind gute und schöne Frauen\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxii{- welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären? -,\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxiii{Huri, die in den Zelten zurückgezogen wohnen.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxiv{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxv{Vor ihnen hat sie weder Mensch noch Djinn beschlafen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxvi{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxvii{Sie lehnen sich auf grünen Decken und schönen Teppichen.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxviii{Welche der Wohltaten eures Herrn wollt ihr beide für Lüge erklären?\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxix{Gesegnet sei der Name deines Herrn, des Erhabenen und Ehrwürdigen!\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxx{\basmalahde Wenn die (Stunde) eintrifft, die eintreffen wird\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxi{- es gibt niemanden, der ihr Eintreffen leugnen könnte -,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxii{Wird sie einiges niedrig machen und einiges erhöhen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxiii{Wenn die Erde heftig geschüttelt wird\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxiv{Und die Berge völlig zertrümmert werden,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxv{Und zu verstreutem Staub werden,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxvi{Und ihr in drei Gruppen aufgeteilt werdet.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxvii{Die von der rechten Seite - was sind die von der rechten Seite?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxviii{Und die von der unglückseligen Seite - was sind die von der unglückseligen Seite?\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxix{Und die Allerersten, ja sie werden die Allerersten sein.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcmxc{Das sind die, die in die Nähe (Gottes) zugelassen werden,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcmxci{In den Gärten der Wonne.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcmxcii{Eine große Schar von den früheren\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcmxciii{Und wenige von den Späteren.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcmxciv{Auf durchwobenen Betten\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcmxcv{Lehnen sie sich einander gegenüber.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcmxcvi{Unter ihnen machen ewig junge Knaben die Runde\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcmxcvii{Mit Humpen und Krügen und einem Becher aus einem Quell,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcmxcviii{Von dem sie weder Kopfweh bekommen noch sich berauschen,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcmxcix{Und mit Früchten von dem, was sie sich auswählen,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmm{Und Fleisch von Geflügel von dem, was sie begehren.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmi{Und (darin sind) großäugige Huri,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmii{Gleich wohlverwahrten Perlen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmiii{Dies) als Lohn für das, was sie zu tun pflegten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmiv{Sie hören darin keine unbedachte Rede und nichts Sündhaftes,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmv{Sondern nur das Wort: «Friede! Friede!»\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmvi{Die von der rechten Seite - was sind die von der rechten Seite?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmvii{Sie sind unter Zizyphusbäumen ohne Dornen\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmviii{Und übereinandergereihten Bananen\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmix{Und ausgestrecktem Schatten,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmx{An Wasser, das sich ergießt,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmxi{Mit vielen Früchten,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmxii{Die weder aufhören noch verwehrt sind,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmxiii{Und auf erhöhten Unterlagen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmxiv{Wir haben sie eigens entstehen lassen\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmxv{Und sie zu Jungfrauen gemacht,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmxvi{Liebevoll und gleichaltrig,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmxvii{Für die von der rechten Seite.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmxviii{Eine große Schar von den Früheren,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmxix{Und eine große Schar von den Späteren.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmxx{Und die von der linken Seite - was sind die von der linken Seite?\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmxxi{Sie sind in glühendem Wind und heißem Wasser,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmxxii{Und in Schatten aus schwarzem Rauch,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmxxiii{Der weder kühl noch trefflich ist.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmxxiv{Sie lebten ja vordem üppig\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmxxv{Und verharrten in der gewaltigen Untreue.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmxxvi{Und sie sagten: «Wenn wir gestorben und zu Staub und Knochen geworden sind, sollen wir dann wirklich auferweckt werden?\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmxxvii{Und auch unsere Vorväter?»\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmxxviii{Sprich: Die Früheren und die Späteren\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmxxix{Werden versammelt zum Termin eines bestimmten Tages.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmxxx{Dann werdet ihr, die ihr irregeht und (die Botschaft) für Lüge erklärt,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmxxxi{Von Zaqqum-Bäumen essen\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmxxxii{Und davon die Bäuche füllen,\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmmxxxiii{Und darauf heißes Wasser trinken;\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmmxxxiv{Trinken werdet ihr, wie durstkranke Kamele trinken.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmmxxxv{Das ist ihre Bewirtung am Tag des Gerichtes.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmmxxxvi{Wir sind es doch, die euch erschaffen haben. Würdet ihr es doch für wahr halten!\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmmxxxvii{Habt ihr gesehen, was (für einen Samen) ihr zum Erguß bringt?\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmmxxxviii{Seid ihr es, die ihn erschaffen oder sind nicht vielmehr Wir es, die (ihn) erschaffen?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmmxxxix{Wir haben für euch den Tod festgelegt. Und niemand kann Uns voraus sein (und abwenden),\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmmxl{Daß Wir (gegen euch) euresgleichen eintauschen und euch in einen Zustand entstehen lassen, den ihr nicht kennt.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmmxli{Ihr wißt doch um die erste Schöpfung. Würdet ihr es doch bedenken!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmmxlii{Habt ihr gesehen, was ihr (an Feldern) bestellt?\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmmxliii{Sät ihr darin, oder sind nicht vielmehr Wir es, die säen?\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmmxliv{Wenn Wir wollten, Wir könnten es zu zermalmtem Zeug machen, und ihr würdet verwundert darüber reden:\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmmxlv{«Wir sind mit Schulden belastet.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmmxlvi{Nein, wir entbehren viel- mehr alles.»\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmmxlvii{Habt ihr denn das Wasser gesehen, das ihr trinkt?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmmxlviii{Habt ihr es von den Wolken herabkommen lassen, oder sind nicht vielmehr Wir es, die (es) herabkommen lassen?\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmmxlix{Wenn Wir wollten, Wir könnten es bitter machen. Würdet ihr doch dankbar sein!\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmml{Habt ihr denn das Feuer gesehen, das ihr zündet?\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmmli{Habt ihr etwa den dazu nötigen Baum entstehen lassen, oder sind nicht vielmehr Wir es, die (ihn) entstehen lassen?\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmmlii{Wir haben es zur Ermahnung und zur Nutznießung für die Wüstenwanderer gemacht.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmmliii{So preise den Namen deines majestätischen Herrn.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmmliv{Nein, Ich schwöre bei den Niedergangsorten der Sterne\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmmlv{- das ist, wenn ihr es nur wüßtet, ein gewaltiger Schwur:\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmmlvi{Das ist wahrlich ein trefflicher Koran\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmmlvii{In einem wohlverwahrten Buch,\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmmlviii{Das nur die berühren dürfen, die rein gemacht worden sind;\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmmmlix{Herabsendung vom Herrn der Welten.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmmlx{Wollt ihr denn über diese Botschaft nur schöne Worte machen\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmmlxi{Und es zu eurer täglichen Beschäftigung machen, sie für Lüge zu erklären?\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmmlxii{Könntet ihr denn, wenn die Seele die Kehle erreicht,\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmmlxiii{Während ihr da zuschaut\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmmlxiv{Und Wir ihm näher sind als ihr - aber ihr könnt es nicht sehen -,\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmmlxv{Könntet ihr denn, wenn ihr doch nicht dem Gericht unterzogen werden sollt,\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmmmlxvi{Sie zurückbringen, so ihr die Wahrheit sagt?\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmmmlxvii{Wenn er nun zu denen gehört, die in die Nähe Gottes zugelassen werden,\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmmmlxviii{Dann sind für ihn (bereit) eine angenehme Brise, duftende Pflanzen und Gärten der Wonne.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmmmlxix{Und wenn er zu denen von der rechten Seite gehört,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmmmlxx{Dann wird er mit «Friede sei dir!» begrüßt von denen, die von der rechten Seite sind.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmmmlxxi{Wenn er aber zu denen gehört, die (die Botschaft) für Lüge erklären und irregehen,\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmmmlxxii{Dann wird ihm eine Bewirtung aus heißem Wasser (zuteil),\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmmmlxxiii{Und das Erleiden des Höllenbrandes.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmmmmlxxiv{Wahrlich, dies ist die Wahrheit, die gewiß ist.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmmmmlxxv{So preise den Namen deines majestätischen Herrn.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmmmmlxxvi{\basmalahde Gott preist, was in den Himmeln und auf der Erde ist. Und Er ist der Mächtige, der Weise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmlxxvii{Ihm gehört die Königsherrschaft der Himmel und der Erde. Er macht lebendig und läßt sterben. Und Er hat Macht zu allen Dingen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmlxxviii{Er ist der Erste und der Letzte, der Sichtbare und der Verborgene. Und Er weiß über alle Dinge Bescheid.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmlxxix{Er ist es, der die Himmel und die Erde in sechs Tagen erschuf und sich dann auf dem Thron zurechtsetzte. Er weiß, was in die Erde eingeht und was aus ihr herauskommt, was vom Himmel herabkommt und was darin emporsteigt. Und Er ist mit euch, wo immer ihr auch seid. Und Gott sieht wohl, was ihr tut.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmlxxx{Ihm gehört die Königsherrschaft der Himmel und der Erde. Und zu Gott werden die Angelegenheiten zurückgebracht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxi{Er läßt die Nacht in den Tag übergehen, und Er läßt den Tag in die Nacht übergehen. Und Er weiß über das innere Geheimnis Bescheid.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxii{Glaubt an Gott und seinen Gesandten und spendet von dem, worüber Er euch zu Nachfolgern eingesetzt hat. Für diejenigen von euch, die glauben und spenden, ist ein großer Lohn bestimmt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxiii{Warum wollt ihr denn nicht an Gott glauben, wo doch der Gesandte euch dazu aufruft, an euren Herrn zu glauben, und wo Er eure Verpflichtungen entgegengenommen hat, so ihr gläubig seid?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxiv{Er ist es, der deutliche Zeichen auf seinen Diener herabsendet, um euch aus den Finsternissen ins Licht hinauszuführen. Und Gott hat Mitleid mit euch und ist barmherzig.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxv{Warum wollt ihr nicht auf dem Weg Gottes spenden, wo doch das Erbe der Himmel und der Erde Gott gehört. Diejenigen von euch, die vor dem Erfolg gespendet und gekämpft haben, sind (den anderen) nicht gleichzusetzen. Diese haben eine höhere Rangstufe als die, die erst nachher gespendet und gekämpft haben. Allen aber hat Gott das Beste versprochen. Und Gott hat Kenntnis von dem, was ihr tut.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxvi{Wer ist es, der Gott ein schönes Darlehen leiht? Er wird es ihm vervielfachen. Und bestimmt ist für ihn ein trefflicher Lohn.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxvii{Am Tag, da du siehst, wie den gläubigen Männern und den gläubigen Frauen ihr Licht vor ihnen und zu ihrer Rechten eilt: «Eine frohe Botschaft für euch heute: Gärten, unter denen Bäche fließen; darin werdet ihr ewig weilen. Das ist der großartige Erfolg.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxviii{Am Tag, da die Heuchler und die Heuchlerinnen zu denen, die glauben, sagen: «Wartet auf uns, daß wir Licht von eurem Licht nehmen.» Es wird (zu ihnen) gesagt: «Geht doch nach hinten zurück und sucht dort nach Licht.» Da wird zwischen ihnen eine Mauer errichtet mit einem Tor. Nach innen ist die Barmherzigkeit und nach außen, davor, ist die Pein.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxix{Sie rufen ihnen zu: «Waren wir nicht auf eurer Seite?» Sie sagen: «Doch, aber ihr habt euch selbst der Versuchung ausgesetzt, ihr habt abgewartet und gezweifelt, und die (eitlen) Wünsche haben euch betört, bis der Befehl Gottes kam. Und betört hat euch in bezug auf Gott der Betörer.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmxc{Heute wird weder von euch noch von denen, die ungläubig sind, ein Lösegeld angenommen. Eure Heimstätte ist das Feuer. Es ist euer Schutzherr - welch schlimmes Ende!\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmxci{Ist es nicht für die, die glauben, Zeit, daß ihre Herzen demütig werden vor der Ermahnung Gottes und vor dem, was von der Wahrheit herabgekommen ist, und daß sie sich nicht wie diejenigen verhalten, denen früher das Buch zugekommen ist, es aber zu lang gedauert hat, so daß ihre Herzen sich verhärtet haben und viele von ihnen Frevler wurden?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmxcii{Wißt, daß Gott die Erde nach ihrem Absterben wieder belebt. Wir machen euch die Zeichen deutlich, auf daß ihr verständig seid.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmxciii{Denen, Männern und Frauen, die Almosen geben und damit Gott ein schönes Darlehen leihen, wird Er es vervielfachen. Und bestimmt ist für sie ein trefflicher Lohn.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmxciv{Diejenigen, die an Gott und seine Gesandten glauben, das sind die Wahrhaftigen und die Zeugen vor ihrem Herrn. Sie erhalten ihren Lohn und ihr Licht. Und diejenigen, die ungläubig sind und unsere Zeichen für Lüge erklären, das sind die Gefährten der Hölle.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmxcv{Wißt, daß das diesseitige Leben nur Spiel und Zerstreuung ist, Schmuck und Prahlerei unter euch und Wetteifern nach mehr Vermögen und Kindern. Es ist wie mit einem Regen, der durch die (von ihm hervorgebrachten) Pflanzen den Säern gefällt. Dann aber verdorren sie, und du siehst, wie sie gelb werden. Dann wird alles zu zermalmtem Zeug. Im Jenseits gibt es eine harte Pein, und auch Vergebung von Gott und Wohlgefallen. Und das diesseitige Leben ist nur betörende Nutznießung.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmxcvi{Eilt zu einer Vergebung von eurem Herrn um die Wette und zu einem Garten, der so breit ist wie der Himmel und die Erde, und der für die bereitet ist, die an Gott und seine Gesandten glauben. Das ist die Huld Gottes, Er läßt sie zukommen, wem Er will. Und Gott besitzt große Huld.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmxcvii{Kein Unglück trifft ein auf der Erde oder bei euch selbst, ohne daß es in einem Buch stünde, bevor Wir es erschaffen. Dies ist Gott ein leichtes.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmxcviii{Dies), damit ihr nicht traurig seid über das, was euch entgangen ist, und euch nicht (zu sehr) der Freude hingebt über das, was Er euch hat zukommen lassen. Und Gott liebt niemanden, der eingebildet und prahlerisch ist,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmxcix{Die, die geizen und den Menschen befehlen, geizig zu sein. Und wenn einer sich abkehrt, so ist Gott der, der auf niemanden angewiesen und des Lobes würdig ist.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmc{Wir haben unsere Gesandten mit den deutlichen Zeichen gesandt und mit ihnen das Buch und die Waage herabkommen lassen, damit die Menschen für die Gerechtigkeit eintreten. Und Wir haben das Eisen herabkommen lassen. In ihm ist heftige Schlagkraft und vielerlei Nutzen für die Menschen. Gott wollte feststellen, wer Ihn und seine Gesandten unterstützt, auch in Abwesenheit. Gott ist stark und mächtig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmci{Und Wir sandten Noach und Abraham und ließen in ihrer Nachkommenschaft die Prophetie und das Buch auftreten. Einige von ihnen folgten der Rechtleitung, und viele von ihnen waren Frevler.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcii{Dann ließen Wir nach ihnen unsere Gesandten folgen. Und Wir ließen Jesus, den Sohn Marias, folgen und ihm das Evangelium zukommen. Und Wir setzten in die Herzen derer, die ihm folgten, Mitleid und Barmherzigkeit, und auch Mönchtum, das sie erfanden - Wir haben es ihnen nicht vorgeschrieben -, dies nur im Trachten nach dem Wohlgefallen Gottes. Sie beobachteten es jedoch nicht in der rechten Weise. Und so ließen Wir denjenigen von ihnen, die glaubten, ihren Lohn zukommen. Aber viele von ihnen waren Frevler.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmciii{O ihr, die ihr glaubt, fürchtet Gott und glaubt an seinen Gesandten, dann läßt Er euch einen doppelten Anteil an seiner Barmherzigkeit zukommen, macht euch ein Licht, in dem ihr wandeln könnt, und vergibt euch. Gott ist voller Vergebung und barmherzig.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmciv{Die Leute des Buches sollen nun wissen, daß sie über nichts von der Huld Gottes verfügen, sondern daß die Huld in der Hand Gottes liegt. Er läßt sie zukommen, wem Er will. Und Gott besitzt große Huld.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcv{\basmalahde Gehört hat Gott die Aussage jener, die mit dir über ihren Gatten streitet und bei Gott Klage erhebt. Gott hört euren Wortwechsel. Gott hört und sieht alles.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcvi{Für diejenigen von euch, die sich von ihren Frauen durch den Rückenspruch scheiden... Sie sind doch nicht ihre Mütter. Ihre Mütter sind die, die sie geboren haben. Sie sagen da ein verwerfliches Wort und etwas Falsches. Und Gott ist voller Verzeihung und Vergebung.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcvii{Diejenigen, die sich von ihren Frauen durch den Rückenspruch scheiden und dann doch zu dem zurückkehren, wovon sie reden, haben einen Sklaven freizulassen, bevor sie beide einander berühren. Damit werdet ihr ermahnt. Und Gott hat Kenntnis von dem, was ihr tut.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcviii{Wer es nicht vermag, der hat zwei Monate hintereinander zu fasten, bevor sie beide einander berühren. Wer es nicht vermag, der hat sechzig Bedürftige zu beköstigen. Dies, damit ihr an Gott und seinen Gesandten glaubt. Das sind die Rechtsbestimmungen Gottes. Für die Ungläubigen ist eine schmerzhafte Pein bestimmt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcix{Diejenigen, die sich Gott und seinem Gesandten widersetzen, werden niedergeworfen, wie die niedergeworfen wurden, die vor ihnen lebten. Wir haben doch deutliche Zeichen hinabgesandt. Und für die Ungläubigen ist eine schmähliche Pein bestimmt.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcx{Am Tag, da Gott sie alle auferweckt und ihnen kundtut, was sie getan haben. Gott hat es erfaßt, während sie es vergessen haben. Und Gott ist Zeuge über alle Dinge.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcxi{Hast du nicht gesehen, daß Gott weiß, was in den Himmeln und was auf der Erde ist? Es gibt kein vertrauliches Gespräch zwischen dreien, ohne daß Er der vierte in ihrer Runde wäre, und keines zwischen fünfen, ohne daß Er der Sechste in ihrer Runde wäre, und auch nicht zwischen weniger oder mehr als dieser (Zahl), ohne daß Er mit ihnen wäre, wo immer sie sein mögen. Dann tut Er ihnen am Tag der Auferstehung kund, was sie getan haben. Gott weiß über alle Dinge Bescheid.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcxii{Hast du nicht auf jene geschaut, denen verboten wurde, vertrauliche Gespräche zu führen, die aber dann zurückkehren zu dem, was ihnen verboten wurde und miteinander über Sünde, Übertretung und Ungehorsam gegen den Gesandten vertraulich sprechen? Und wenn sie zu dir kommen, grüßen sie dich, wie nicht einmal Gott dich grüßt, und sagen bei sich: «Könnte Gott uns nur peinigen für das, was wir sagen!» Ihnen genügt doch die Hölle, in der sie brennen worden - welch schlimmes Ende!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcxiii{O ihr, die ihr glaubt, wenn ihr vertraulich miteinander sprecht, dann sprecht nicht vertraulich miteinander über Sünde, Übertretung und Ungehorsam gegen den Gesandten, sondern sprecht vertraulich miteinander über Frömmigkeit und Gottesfurcht. Und fürchtet Gott, zu dem ihr versammelt werdet.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcxiv{Vertrauliche Gespräche sind vom Satan, der damit die betrüben will, die gläubig sind. Aber er kann ihnen keinen Schaden zufügen, außer mit der Erlaubnis Gottes. Auf Gott sollen die Gläubigen vertrauen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcxv{O ihr, die ihr glaubt, wenn zu euch gesagt wird: «Macht Platz in den Versammlungen.», dann macht Platz, so macht auch Gott euch Platz. Und wenn gesagt wird: «Erhebt euch.», dann erhebt euch eben, so erhöht auch Gott diejenigen von euch, die glauben, und die, denen das Wissen zugekommen ist, um Rangstufen. Und Gott hat Kenntnis von dem, was ihr tut.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcxvi{O ihr, die ihr glaubt, wenn ihr mit dem Gesandten vertraulich sprechen wollt, dann gebt vor eurem vertraulichen Gespräch im voraus ein Almosen. Das ist besser und reiner für euch. Wenn ihr es nicht vermögt, dann ist Gott voller Vergebung und barmherzig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcxvii{Seid ihr erschrocken davor, daß ihr vor eurem vertraulichen Gespräch im voraus Almosen geben sollt? Nun, wenn ihr es nicht tut und Gott sich euch wieder gnädig zuwendet, dann verrichtet das Gebet und entrichtet die Abgabe, und gehorcht Gott und seinem Gesandten. Und Gott hat Kenntnis von dem, was ihr tut.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcxviii{Hast du nicht auf jene geschaut, die sich Leute zu Freunden nehmen, auf die Gott zornig ist? Sie gehören nicht zu euch, und auch nicht zu ihnen. Und sie leisten wissentlich falsche Eide.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcxix{Gott hat für sie eine harte Pein bereitet. Schlimm ist das, was sie zu tun pflegen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcxx{Sie haben sich ihre Eide zu einem Schutzschild genommen und so (die Menschen) vom Weg Gottes abgewiesen. Bestimmt ist für sie eine schmähliche Pein.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcxxi{Ihr Vermögen und ihre Kinder werden ihnen vor Gott nichts nützen. Das sind die Gefährten des Feuers; darin werden sie ewig weilen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcxxii{Am Tag, da Gott sie alle auferweckt. Da werden sie Ihm schwören, wie sie euch schwören, und meinen, sie hätten nun eine Grundlage. Siehe, sie sind ja doch Lügner.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcxxiii{Der Satan hat von ihnen Besitz ergriffen und sie die Ermahnung Gottes vergessen lassen. Sie sind die Partei des Satans. Siehe, die Partei des Satans sind ja doch die Verlierer.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcxxiv{Diejenigen, die sich Gott und seinem Gesandten widersetzen, die gehören zu den Niedrigsten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcxxv{Gott hat vorgeschrieben: «Siegen werde Ich, Ich und meine Gesandten.» Gott ist stark und mächtig.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcxxvi{Du wirst nicht feststellen, daß Leute, die an Gott und den Jüngsten Tag glauben, denen Liebe zeigen, die sich Gott und seinem Gesandten widersetzen, auch wenn sie ihre Väter wären oder ihre Söhne, ihre Brüder oder ihre Sippenmitglieder. In deren Herzen hat Er den Glauben geschrieben und sie mit einem Geist von sich gestärkt. Er wird sie in Gärten eingehen lassen, unter denen Bäche fließen; darin werden sie ewig weilen. Gott hat Wohlgefallen an ihnen, und sie haben Wohlgefallen an Ihm. Sie sind die Partei Gottes. Siehe, die Partei Gottes sind die, denen es wohl ergeht.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcxxvii{\basmalahde Gott preist, was in den Himmeln und was auf der Erde ist. Und Er ist der Mächtige, der Weise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcxxviii{Er ist es, der diejenigen von den Leuten des Buches, die ungläubig sind, aus ihren Wohnstätten zur ersten Versammlung vertrieben hat. Ihr habt nicht geglaubt, daß sie fortziehen würden; auch sie meinten, ihre Festungen würden sie vor Gott schützen. Da kam Gott über sie, von wo sie nicht damit rechneten, und jagte ihren Herzen Schrecken ein, so daß sie ihre Häuser mit ihren eigenen Händen und durch die Hände der Gläubigen zerstörten. Zieht nun die Lehre daraus, ihr Einsichtigen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcxxix{Und hätte Gott ihnen nicht die Verbannung vorgeschrieben, Er hätte sie im Diesseits gepeinigt. Im Jenseits aber erhalten sie die Pein des Feuers.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcxxx{Dies dafür, daß sie sich Gott und seinem Gesandten widersetzten. Und wenn jemand sich Gott widersetzt, so verhängt Gott eine harte Strafe.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxi{Was ihr an Palmen umgehauen habt oder auf ihrem Stamm habt stehenlassen, es geschah mit der Erlaubnis Gottes, auch damit Er die Frevler zuschanden mache.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxii{Und für das, was Gott seinem Gesandten von ihnen als Beute zugeteilt hat, habt ihr weder Pferde noch Kamele anspornen müssen. Gott verleiht vielmehr seinen Gesandten Gewalt, über wen Er will. Und Gott hat Macht zu allen Dingen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxiii{Was Gott seinem Gesandten von den Bewohnern der Städte als Beute zugeteilt hat, gehört Gott und seinem Gesandten, und den Verwandten, den Waisen, den Bedürftigen, dem Reisenden. Dies, damit es nicht eben unter den Reichen von euch die Runde macht. Und was der Gesandte euch zukommen läßt, das sollt ihr nehmen. Und was er euch verwehrt, davon sollt ihr euch fernhalten. Und fürchtet Gott. Gott verhängt eine harte Strafe.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxiv{Das gehört) den armen Auswanderern, die aus ihren Wohnstätten und von ihrem Besitz vertrieben wurden, die nach Gottes Huld und Wohlgefallen streben und Gott und seinen Gesandten unterstützen. Das sind die Wahrhaftigen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxv{Und diejenigen, die vor ihnen in der Wohnstätte und im Glauben zu Hause waren, lieben die, die zu ihnen ausgewandert sind; sie empfinden in ihrem Inneren kein Bedürfnis für das, was diesen zugekommen ist, und sie bevorzugen sie vor sich selbst, auch wenn sie selbst Not leiden. Und diejenigen, die vor ihrer eigenen Habsucht bewahrt bleiben, sind die, denen es wohl ergeht.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxvi{Und diejenigen, die nach ihnen gekommen sind, sagen: «Unser Herr, vergib uns und unseren Brüdern, die uns im Glauben vorangegangen sind. Und setze in unsere Herzen keinen Groll gegen die, die glauben. Unser Herr, du hast Mitleid und bist barmherzig.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxvii{Hast du nicht auf jene geschaut, die Heuchler sind? Sie sagen zu ihren Brüdern von den Leuten des Buches, welche ungläubig sind: «Wenn ihr vertrieben werdet, werden wir mit euch fortziehen. Und wir werden niemals jemandem gegen euch gehorchen. Und wenn gegen euch gekämpft wird, dann werden wir euch bestimmt unterstützen.» Und Gott bezeugt, daß sie ja lügen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxviii{Wenn sie vertrieben werden, werden sie nicht mit ihnen fortziehen. Und wenn gegen sie gekämpft wird, werden sie sie nicht unterstützen. Und sollten sie sie unterstützen, dann werden sie den Rücken kehren, und dann werden sie selbst keine Unterstützung erfahren.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxix{Ihr flößt ihnen mehr Angst ein als Gott. Dies, weil sie Leute sind, die nicht begreifen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcxl{Sie kämpfen nicht alle zusammen gegen euch, es sei denn in befestigten Städten oder hinter Mauern. Ihre Schlagkraft ist untereinander gewaltig. Du meinst, sie bilden eine Einheit, aber ihre Herzen sind geteilt. Dies, weil sie Leute sind, die keinen Verstand haben.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcxli{Es ist wie mit denen, die es kurz vor ihnen erlebten. Sie erlitten die schweren Folgen ihres Verhaltens, und für sie ist eine schmerzhafte Pein bestimmt.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcxlii{Es ist wie mit dem Satan, als er zum Menschen sagte: «Sei ungläubig.» Als er ungläubig wurde, sagte er: «Ich bin unschuldig an dir. Ich fürchte Gott, den Herrn der Welten.»\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcxliii{Das Ende von beiden war, daß sie nun im Feuer sind; darin werden sie ewig weilen. Und das ist die Vergeltung für die, die Unrecht tun.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcxliv{O ihr, die ihr glaubt, fürchtet Gott. Und jeder schaue, was er für morgen vorausschickt. Und fürchtet Gott. Gott hat Kenntnis von dem, was ihr tut.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcxlv{Und seid nicht wie diejenigen, die Gott vergessen haben und die Er dann sich selbst vergessen ließ. Das sind die Frevler.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcxlvi{Nicht gleich sind die Gefährten des Feuers und die Gefährten des Paradieses. Die Gefährten des Paradieses sind es, die den Erfolg erzielen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcxlvii{Hätten Wir diesen Koran auf einen Berg hinabgesandt, du hättest gesehen, wie er aus Furcht vor Gott demütig innehält und sich spaltet. Diese Gleichnisse führen Wir den Menschen an, auf daß sie nachdenken.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcxlviii{Er ist Gott, außer dem es keinen Gott gibt, der über das Unsichtbare und das Offenbare Bescheid weiß. Er ist der Erbarmer, der Barmherzige.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcxlix{Er ist Gott, außer dem es keinen Gott gibt, der König, der Heilige, der Inbegriff des Friedens, der Stifter der Sicherheit, der alles fest in der Hand hat, der Mächtige, der Gewaltige, der Stolze. Preis sei Gott! (Er ist erhaben) über das, was sie (Ihm) beigesellen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcl{Er ist Gott, der Schöpfer, der Erschaffer, der Bildner. Sein sind die schönsten Namen. Ihn preist, was in den Himmeln und auf der Erde ist. Und Er ist der Mächtige, der Weise.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcli{\basmalahde O ihr, die ihr glaubt, nehmt euch nicht meine Feinde und eure Feinde zu Freunden, indem ihr ihnen Liebe entgegenbringt, wo sie doch das verleugnen, was von der Wahrheit zu euch gekommen ist, und den Gesandten und euch selbst vertreiben, weil ihr an Gott, euren Herrn, glaubt. (Haltet euch daran), wenn ihr wirklich ausgezogen seid zum Einsatz auf meinem Weg und im Streben nach meinem Wohlgefallen. Ihr zeigt ihnen heimlich Liebe, wo Ich doch besser weiß, was ihr verbergt und was ihr offenlegt. Und wer von euch das tut, der ist vom rechten Weg abgeirrt.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmclii{Wenn sie euch treffen, sind sie euch feind und strecken gegen euch ihre Hände und ihre Zungen zum Bösen aus. Sie wünschten, ihr wäret ungläubig.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcliii{Weder eure Verwandtschaftsbande noch eure Kinder werden euch nützen. Am Tag der Auferstehung wird Gott zwischen euch entscheiden. Und Gott sieht wohl, was ihr tut.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcliv{Ihr habt doch ein schönes Beispiel in Abraham und denen, die mit ihm waren, als sie zu ihrem Volk sagten: «Wir sind unschuldig an euch und an dem, was ihr anstelle Gottes verehrt. Wir verleugnen euch. Und zwischen uns und euch sind Feindschaft und Haß auf immer sichtbar geworden, bis ihr an Gott allein glaubt.» Dies, bis auf das Wort Abrahams zu seinem Vater: «Ich werde für dich um Vergebung bitten. Und ich kann dir vor Gott keinen Nutzen bringen. Unser Herr, auf Dich vertrauen wir, und Dir wenden wir uns reumütig zu. Und zu Dir führt der Lebensweg.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmclv{Unser Herr, mach uns nicht zu einer Versuchung für die, die ungläubig sind. Und vergib uns, unser Herr. Du bist der Mächtige, der Weise.»\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmclvi{Ihr habt in ihnen ein schönes Beispiel, und zwar jeder, der auf Gott hofft und den Jüngsten Tag erwartet. Wenn einer sich abkehrt, so ist Gott der, der auf niemanden angewiesen und des Lobes würdig ist.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmclvii{Möge Gott zwischen euch und die von ihnen, mit denen ihr verfeindet seid, Liebe setzen! Und Gott ist mächtig. Und Gott ist voller Vergebung und barmherzig.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmclviii{Gott verbietet euch nicht, denen, die nicht gegen euch der Religion wegen gekämpft und euch nicht aus euren Wohnstätten vertrieben haben, Pietät zu zeigen und Gerechtigkeit angedeihen zu lassen. Gott liebt ja die, die gerecht handeln.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmclix{Gott verbietet Euch, die, die gegen euch der Religion wegen gekämpft und euch aus euren Wohnstätten vertrieben und zu eurer Vertreibung Beistand geleistet haben, zu Freunden zu nehmen. Diejenigen, die sie zu Freunden nehmen, das sind die, die Unrecht tun.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmclx{O ihr, die ihr glaubt, wenn gläubige Frauen als Auswanderer zu euch kommen, dann prüft sie. Gott weiß besser über ihren Glauben Bescheid. Wenn ihr feststellt, daß sie gläubig sind, dann schickt sie nicht zu den Ungläubigen zurück. Zur Ehe sind weder diese (Frauen) ihnen erlaubt, noch sind sie diesen (Frauen) erlaubt. Und laßt ihnen jedoch zukommen, was sie (seinerzeit für diese Frauen) ausgegeben haben. Es ist für euch kein Vergehen, sie zu heiraten, wenn ihr ihnen ihren Lohn zukommen laßt. Und haltet nicht am Eheband mit den ungläubigen Frauen fest, und fordert, was ihr (bei der Eheschließung) ausgegeben habt, zurück. Auch sie sollen zurückfordern, was sie ausgegeben haben. Das ist das Urteil Gottes; Er urteilt zwischen euch. Und Gott weiß Bescheid und ist weise.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmclxi{Und wenn euch etwas von euren Gattinnen an die Ungläubigen abhanden kommt und ihr nach einer Strafaktion Beute macht, dann laßt denen, deren Gattinnen fortgegangen sind, soviel zukommen, wie sie ausgegeben haben. Und fürchtet Gott, an den ihr glaubt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmclxii{O Prophet, wenn gläubige Frauen zu dir kommen, um dir Treue zu geloben, daß sie Gott nichts beigesellen, nicht stehlen, keinen Ehebruch begehen, ihre Kinder nicht töten, keine Verleumdung vorbringen, die sie vor ihren eigenen Händen und Füßen erdichten, und gegen dich nicht ungehorsam sind in dem, was recht ist, dann nimm ihr Treueversprechen an und bitte Gott für sie um Vergebung. Gott ist voller Vergebung und barmherzig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmclxiii{O ihr, die ihr glaubt, nehmt nicht Leute zu Freunden, auf die Gott zornig ist. Sie haben die Hoffnung auf das Jenseits aufgegeben, so wie die Ungläubigen die Hoffnung aufgegeben haben in bezug auf die (Toten) in den Gräbern.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmclxiv{\basmalahde Gott preist, was in den Himmeln und was auf der Erde ist. Und Er ist der Mächtige, der Weise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmclxv{O ihr, die ihr glaubt, warum sagt ihr, was ihr nicht tut?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmclxvi{Großen Abscheu erregt es bei Gott, daß ihr sagt, was ihr nicht tut.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmclxvii{Gott liebt die, die auf seinem Weg kämpfen in einer Reihe, als wären sie ein festgefügter Bau.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmclxviii{Und als Mose zu seinem Volk sagte: «O mein Volk, warum fügt ihr mir Leid zu, wo ihr doch wißt, daß ich der Gesandte Gottes an euch bin?» Als sie nun abwichen, ließ Gott ihre Herzen abweichen. Und Gott leitet die frevlerischen Leute nicht recht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmclxix{Und als Jesus, der Sohn Marias, sagte: «O Kinder Israels, ich bin der Gesandte Gottes an euch, um zu bestätigen, was von der Tora vor mir vorhanden war, und einen Gesandten zu verkünden, der nach mir kommt: sein Name ist Ahmad.» Als er nun mit den deutlichen Zeichen zu ihnen kam, sagten sie: «Das ist eine offenkundige Zauberei.»\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmclxx{Und wer ist ungerechter als der, der gegen Gott Lügen erdichtet, während er zur Gottergebenheit aufgerufen wird? Und Gott leitet die ungerechten Leute nicht recht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmclxxi{Sie wollen das Licht Gottes mit ihrem Mund auslöschen. Aber Gott wird sein Licht vollenden, auch wenn es den Ungläubigen zuwider ist.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmclxxii{Er ist es, der seinen Gesandten mit der Rechtleitung und der Religion der Wahrheit gesandt hat, um ihr die Oberhand zu verleihen über alle Religion, auch wenn es den Polytheisten zuwider ist.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmclxxiii{O ihr, die ihr glaubt, soll ich euch auf einen Handel hinweisen, der euch vor einer schmerzhaften Pein rettet?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmclxxiv{Ihr sollt an Gott und seinen Gesandten glauben, euch auf dem Weg Gottes mit eurem Vermögen und mit eigener Person einsetzen - das ist besser für euch, so ihr Bescheid wißt -,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmclxxv{Dann wird Er euch eure Sünden vergeben und euch eingehen lassen in Gärten, unter denen Bäche fließen, und in angenehme Wohnungen in den Gärten von Eden. Das ist der großartige Erfolg.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmclxxvi{Und noch eine andere (Huld), die ihr liebt, (steht bereit): eine Unterstützung von Gott und ein naher Erfolg. Verkünde den Gläubigen eine Frohbotschaft.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmclxxvii{O ihr, die ihr glaubt, seid die Helfer Gottes, so wie Jesus, der Sohn Marias, zu den Jüngern gesagt hat: «Wer sind meine Helfer (auf dem Weg) zu Gott hin?» Die Jünger sagten: «Wir sind die Helfer Gottes.» Eine Gruppe der Kinder Israels glaubte, und eine (andere) Gruppe war ungläubig. Da stärkten Wir diejenigen, die glaubten, gegen ihre Feinde, und sie bekamen die Oberhand.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmclxxviii{\basmalahde Gott preist, was in den Himmeln und was auf der Erde ist, (Ihn), den König, den Heiligen, den Mächtigen, den Weisen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmclxxix{Er ist es, der unter den Ungelehrten einen Gesandten aus ihrer Mitte hat erstehen lassen, der ihnen seine Zeichen verliest, sie läutert und sie das Buch und die Weisheit lehrt - sie befanden sich ja zuvor in einem offenkundigen Irrtum -,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmclxxx{sie) und andere vor ihnen, die sie noch nicht eingeholt haben. Und Er ist der Mächtige, der Weise.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxi{Das ist die Huld Gottes. Er läßt sie zukommen, wem Er will. Und Gott besitzt große Huld.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxii{Es ist mit denen, denen die Tora auferlegt worden ist, die sie aber dann doch nicht getragen haben, wie mit einem Esel, der Bücher trägt. Schlimm ist es mit den Leuten, die die Zeichen Gottes für Lüge erklären. Und Gott leitet die ungerechten Leute nicht recht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxiii{Sprich: O ihr, die ihr Juden seid, wenn ihr behauptet, ihr seid die Freunde Gottes unter Ausschluß der anderen Menschen, dann wünscht euch den Tod, so ihr die Wahrheit sagt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxiv{Aber niemals werden sie ihn sich wünschen wegen dessen, was ihre Hände vorausgeschickt haben. Und Gott weiß über die, die Unrecht tun, Bescheid.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxv{Sprich: Der Tod, vor dem ihr flieht, wird euch erreichen. Dann werdet ihr zu dem, der das Unsichtbare und das Offenbare kennt, zurückgebracht, und Er wird euch kundtun, was ihr zu tun pflegtet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxvi{O ihr, die ihr glaubt, wenn am Freitag zum Gebet gerufen wird, dann eilt zum Gedenken Gottes und laßt das Kaufgeschäft ruhen. Das ist besser für euch, so ihr Bescheid wißt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxvii{Wenn das Gebet beendet ist, dann breitet euch im Land aus und strebt nach etwas von der Huld Gottes. Und gedenkt Gottes viel, auf daß es euch wohl ergehe.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxviii{Und wenn sie einen Handel oder eine Gelegenheit zur Zerstreuung sehen, laufen sie hin und lassen dich stehen. Sprich: Was bei Gott ist, ist doch besser als Zerstreuung und Handel. Und Gott ist der beste Versorger.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxix{\basmalahde Wenn die Heuchler zu dir kommen, sagen sie: «Wir bezeugen, daß du der Gesandte Gottes bist.» Gott weiß wohl, daß du sein Gesandter bist. Und Gott bezeugt, daß die Heuchler doch lügen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcxc{Sie nehmen sich ihre Eide zu einem Schutzschild und weisen so (die Menschen) vom Wege Gottes ab. Schlimm ist das, was sie zu tun pflegen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcxci{Dies, weil sie erst gläubig waren, dann aber ungläubig wurden. Da wurden ihre Herzen versiegelt, so daß sie nicht begreifen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcxcii{Und wenn du sie siehst, gefallen dir ihre Gestalten. Und wenn sie sprechen, hörst du ihren Worten zu. Sie sind wie angelehnte Bretter. Sie meinen, jeder Schrei sei gegen sie gerichtet. Sie sind die (wahren) Feinde. Nimm dich vor ihnen in acht. Gott bekämpfe sie! Wie leicht lassen sie sich doch abwenden!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcxciii{Und wenn zu ihnen gesagt wird: «Kommt her, daß der Gesandte Gottes für euch um Vergebung bittet», wenden sie den Kopf zur Seite, und du siehst, wie sie sich in Hochmut abwenden.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcxciv{Für sie ist es gleich, ob du für sie um Vergebung bittest oder nicht bittest. Gott wird ihnen nicht vergeben. Gott leitet die frevlerischen Leute nicht recht.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcxcv{Sie sind es, die sagen: «Spendet nicht für diejenigen, die beim Gesandten Gottes sind, bis sie davoneilen.» Doch Gott gehören die Vorratskammern der Himmel und der Erde. Aber die Heuchler begreifen es nicht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcxcvi{Sie sagen: «Wenn wir nach Medina zurückkehren, werden sicher die Mächtigen die Schwächeren aus ihr vertreiben.» Doch Gott gehört die Macht, und auch seinem Gesandten und den Gläubigen. Aber die Heuchler wissen nicht Bescheid.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcxcvii{O ihr, die ihr glaubt, euer Vermögen und eure Kinder sollen euch nicht vom Gedenken Gottes ablenken. Diejenigen, die dies tun, das sind die Verlierer.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcxcviii{Und spendet von dem, was Wir euch beschert haben, bevor der Tod einen von euch ereilt und er dann sagt: «Mein Herr, könntest Du mich doch auf eine kurze Frist zurückstellen! Ich würde Almosen geben und einer der Rechtschaffenen sein.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcxcix{Gott wird aber niemanden zurückstellen, wenn seine Frist kommt. Und Gott hat Kenntnis von dem, was ihr tut.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcc{\basmalahde Gott preist, was in den Himmeln und was auf der Erde ist. Ihm gehört die Königsherrschaft, und Lob sei Ihm! Er hat Macht zu allen Dingen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcci{Er ist es, der euch erschaffen hat. Die einen von euch sind ungläubig, die anderen gläubig. Und Gott sieht wohl, was ihr tut.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccii{Er hat die Himmel und die Erde in Wahrheit erschaffen, und Er hat euch gestaltet und eure Gestalten schön gemacht. Und zu Ihm führt der Lebensweg.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcciii{Er weiß, was in den Himmeln und auf der Erde ist, und Er weiß, was ihr im geheimen redet und was ihr offenlegt. Und Gott weiß über das innere Geheimnis Bescheid.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcciv{Ist denn nicht der Bericht über die, die zuvor ungläubig waren, zu euch gelangt? Sie erlitten die schweren Folgen ihres Verhaltens, und bestimmt ist für sie eine schmerzhafte Pein.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccv{Dies, weil ihre Gesandten mit den deutlichen Zeichen zu ihnen kamen, sie aber sagten: «Sollen etwa Menschen uns rechtleiten?» Da wurden sie ungläubig und kehrten sich ab. Doch Gott bedurfte ihrer nicht. Gott ist auf niemanden angewiesen und des Lobes würdig.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccvi{Diejenigen, die ungläubig sind, behaupten, daß sie nicht auferweckt werden. Doch, bei meinem Herrn, ihr werdet auferweckt werden. Dann wird euch das kundgetan, was ihr getan habt. Und dies ist Gott ein leichtes.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccvii{Darum glaubt an Gott und seinen Gesandten und das Licht, das Wir hinabgesandt haben. Und Gott hat Kenntnis von dem, was ihr tut.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccviii{Am Tag, da Er euch zum Tag der Versammlung versammelt. Das ist der Tag der Übervorteilung. Wer an Gott glaubt und Gutes tut, dem sühnt Er seine Missetaten, und den läßt Er in Gärten eingehen, unter denen Bäche fließen; darin werden sie auf immer ewig weilen. Das ist der großartige Erfolg.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccix{Und diejenigen, die ungläubig sind und unsere Zeichen für Lüge erklären, das sind die Gefährten des Feuers; darin werden sie ewig weilen - welch schlimmes Ende!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccx{Kein Unglück trifft (jemanden), es sei denn mit der Erlaubnis Gottes. Und wer an Gott glaubt, dessen Herz leitet Er recht. Und Gott weiß über alle Dinge Bescheid.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccxi{Und gehorcht Gott und gehorcht dem Gesandten. Wenn ihr euch abkehrt, so obliegt unserem Gesandten nur die deutliche Ausrichtung (der Botschaft).\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccxii{Gott, es gibt keinen Gott außer Ihm. Auf Gott sollen die Gläubigen vertrauen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmccxiii{O ihr, die ihr glaubt, unter euren Gattinnen und euren Kindern sind welche, die euch feind sind. So nehmt euch vor ihnen in acht. Wenn ihr verzeiht, nachsichtig seid und vergebt, so ist Gott voller Vergebung und barmherzig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmccxiv{Euer Vermögen und eure Kinder sind eine Versuchung. Aber Gott hält einen großartigen Lohn bereit.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmccxv{Fürchtet nun Gott, soviel ihr eben könnt. Und hört zu und gehorcht. Und spendet: Es ist besser für euch. Und diejenigen, die vor ihrer eigenen Habsucht bewahrt bleiben, das sind die, denen es wohl ergeht.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmccxvi{Wenn ihr Gott ein schönes Darlehen leiht, wird Er es euch vervielfachen und euch vergeben. Und Gott zeigt sich erkenntlich und ist langmütig.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmccxvii{Er), der über das Unsichtbare und das Offenbare Bescheid weiß, der Mächtige, der Weise.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmccxviii{\basmalahde O Prophet, wenn ihr die Frauen entlaßt, dann entlaßt sie unter Beachtung ihrer Wartezeit, und berechnet die Wartezeit (sorgfältig). Und fürchtet Gott, euren Herrn. Weiset sie nicht aus ihren Häusern aus. Sie sollen auch nicht selbst ausziehen, es sei denn, sie begehen eine offenkundige schändliche Tat. Das sind die Rechtsbestimmungen Gottes. Wer die Rechtsbestimmungen Gottes übertritt, der tut sich selbst Unrecht. Du weißt nicht, ob nicht Gott vielleicht danach eine neue Lage herbeiführt.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccxix{Wenn sie das Ende ihrer Frist erreichen, dann behaltet sie in rechtlicher Weise oder trennt euch von ihnen in rechtlicher Weise. Und nehmt zwei gerechte Leute von euch zu Zeugen, und erhaltet das Zeugnis Gottes aufrecht. Damit wird derjenige ermahnt, der an Gott und den Jüngsten Tag glaubt. Und wer Gott fürchtet, dem schafft Er einen Ausweg\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccxx{Und beschert Unterhalt, von wo er damit nicht rechnet. Und wer auf Gott vertraut, dem genügt Er. Gott erzielt sein Anliegen. Gott legt für jede Sache ein Maß fest.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccxxi{Und für die von euren Frauen, die keine Menstruation mehr erwarten, falls ihr da Zweifel hegt, gilt eine Wartezeit von drei Monaten. Und ebenso für die, die keine Menstruation haben. Für die, die schwanger sind, ist die Frist erreicht, wenn sie gebären, was sie (in ihrem Leib) tragen. Und wer Gott fürchtet, dem schafft Er Erleichterung in seiner Angelegenheit.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccxxii{Das ist die Anordnung Gottes, die Er zu euch herabgesandt hat. Und wer Gott fürchtet, dem sühnt Er seine Missetaten und dem verleiht Er einen großartigen Lohn.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccxxiii{Laßt sie dort wohnen, wo ihr selbst wohnt, euren Möglichkeiten entsprechend. Und fügt ihnen keinen Schaden zu, um sie in Bedrängnis zu bringen. Und wenn sie schwanger sind, dann übernehmt für sie die nötigen Aufwendungen, bis sie gebären, was sie (in ihrem Leib) tragen. Wenn sie für euch das Kind stillen, dann laßt ihnen ihren Lohn zukommen, und beratet euch untereinander (darüber) in rechtlicher Weise. Und wenn ihr euch schwer einigen könnt, dann wird wohl eine andere (das Kind) für ihn stillen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccxxiv{Der Wohlhabende soll entsprechend seinem Vermögen die Aufwendungen gestalten. Und wem der Unterhalt bemessen zugeteilt wurde, der soll von dem ausgeben, was Gott ihm hat zukommen lassen. Gott fordert von einem nur das, was Er ihm hat zukommen lassen. Gott wird nach der schwierigen Lage Erleichterung schaffen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccxxv{Und wie manche Stadt rebellierte gegen den Befehl ihres Herrn und seiner Gesandten, worauf Wir sie einer harten Abrechnung unterzogen und mit einer entsetzlichen Pein peinigten.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccxxvi{Da erlitt sie die schweren Folgen ihres Verhaltens. Und das Ende ihres Verhaltens war (nur) Verlust.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccxxvii{Gott hat für sie eine harte Pein bereitet. So fürchtet Gott, ihr Einsichtigen, die ihr glaubt. Gott hat zu euch eine Ermahnung herabgesandt,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccxxviii{Einen Gesandten, der euch die Zeichen Gottes, die alles deutlich machen, verliest, um die, die glauben und die guten Werke tun, aus den Finsternissen ins Licht hinauszuführen. Und wer an Gott glaubt und Gutes tut, den wird Er in Gärten eingehen lassen, unter denen Bäche fließen; darin werden sie auf immer ewig weilen. Gott hat ihm (damit) einen schönen Unterhalt gewährt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccxxix{Gott ist es, der sieben Himmel erschaffen hat, und von der Erde gleich viel. Der Befehl (Gottes) kommt zwischen ihnen herab, damit ihr wißt, daß Gott Macht hat zu allen Dingen und daß Gott alle Dinge mit seinem Wissen umfaßt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccxxx{\basmalahde O Prophet, warum erklärst du für verboten, was Gott dir erlaubt hat, im Streben danach, deinen Gattinnen zu gefallen? Gott ist voller Vergebung und barmherzig.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxi{Gott hat für euch festgelegt, wie eure Eide zu lösen sind. Und Gott ist euer Schutzherr, und Er ist der, der Bescheid weiß und weise ist.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxii{Als der Prophet einer seiner Gattinnen einige Worte im geheimen anvertraute. Als sie sie dann mitteilte und Gott ihm darüber Auskunft gab, gab er einen Teil davon bekannt und überging einen anderen Teil. Als er es ihr nun mitteilte, sagte sie: «Wer hat dir das kundgetan?» Er sagte: «Kundgetan hat (es) mir der, der Bescheid weiß und Kenntnis (von allem) hat.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxiii{Wenn ihr beide zu Gott umkehrt - eure Herzen waren ja dem Gerede zugeneigt -... Und wenn ihr einander gegen ihn Beistand leistet, so ist Gott sein Schutzherr, und Gabriel, die Rechtschaffenen von den Gläubigen und außerdem die Engel werden (ihm) Beistand leisten.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxiv{Möge sein Herr ihm, wenn er euch (alle) entläßt, zum Tausch andere Gattinnen geben, die besser sind als ihr: muslimische Frauen, die gläubig und demütig ergeben sind, bereit zur Umkehr sind, (Gott) dienen und umherziehen, ob sie früher verheiratet waren oder Jungfrauen sind.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxv{O ihr, die ihr glaubt, hütet euch selbst und eure Angehörigen vor einem Feuer, dessen Brennstoff Menschen und Steine sind und über das hartherzige und strenge Engel gesetzt sind, die gegen Gott nicht ungehorsam sind in dem, was Er ihnen befiehlt, sondern tun, was ihnen befohlen wird.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxvi{O ihr, die ihr ungläubig seid, entschuldigt euch heute nicht. Euch wird nur das vergolten, was ihr zu tun pflegtet.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxvii{O ihr, die ihr glaubt, kehrt zu Gott um in aufrichtiger Umkehr. Möge Gott euch eure Missetaten sühnen und euch in Gärten eingehen lassen, unter denen Bäche fließen, am Tag, da Gott den Propheten und die, die mit ihm gläubig sind, nicht zuschanden macht! Ihr Licht eilt vor ihnen und zu ihrer Rechten her. Sie sagen: «Unser Herr, vollende für uns unser Licht und vergib uns. Du hast Macht zu allen Dingen.»\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxviii{O Prophet, setze dich gegen die Ungläubigen und die Heuchler ein und fasse sie hart an. Ihre Heimstätte ist die Hölle - welch schlimmes Ende!\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxix{Gott hat für die, die ungläubig sind, die Frau Noachs und die Frau Lots als Beispiel angeführt. Sie beide unterstanden zwei Dienern von unseren rechtschaffenen Dienern. Aber sie waren treulos zu ihnen, so konnten ihnen diese vor Gott nichts nützen. Und es wurde gesagt: «Geht ins Feuer ein mit denen, die hineingehen.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccxl{Und Gott hat für die, die glauben, die Frau des Pharao als Beispiel angeführt. Als sie sagte: «Mein Herr, baue mir ein Haus bei Dir im Paradies, und errette mich von Pharao und seinem Handeln, und errette mich von den Leuten, die Unrecht tun.»\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccxli{Und (auch) Maria, die Tochter, Imraans, die ihre Scham unter Schutz stellte, worauf Wir in sie von unserem Geist bliesen. Und sie hielt die Worte ihres Herrn und seine Bücher für wahr und gehörte zu denen, die (Gott) demütig ergeben sind.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccxlii{\basmalahde Gesegnet sei der, in dessen Hand die Königsherrschaft ist! Und Er hat Macht zu allen Dingen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccxliii{Er), der den Tod und das Leben erschaffen hat, um euch zu prüfen (und festzustellen), wer von euch am besten handelt. Und Er ist der, der mächtig und voller Vergebung ist.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccxliv{Er), der sieben Himmel in Schichten erschaffen hat. Und du kannst an der Schöpfung des Erbarmers kein Mißverhältnis sehen. Wende deinen Blick zurück: Siehst du irgendeinen Mangel?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccxlv{Dann wende deinen Blick zweimal zurück. Dein Blick kehrt zu dir beschämt und ermüdet zurück.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccxlvi{Und Wir haben den untersten Himmel mit Leuchten geschmückt und haben sie zu Wurfsteinen gegen die Satane gemacht. Und Wir haben für sie die Pein des Feuerbrandes bereitet.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccxlvii{Und bestimmt ist für die, die ihren Herrn verleugnen, die Pein der Hölle - welch schlimmes Ende!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccxlviii{Wenn sie hineingeworfen werden, hören sie sie aufheulen, während sie brodelt,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccxlix{Als würde sie bald vor Wut platzen. Sooft eine Schar hineingeworfen wird, werden ihre Wärter sie fragen: «Ist denn kein Warner zu euch gekommen?»\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccl{Sie sagen: «Doch, ein Warner ist zu uns gekommen. Aber wir haben (ihn) der Lüge geziehen und gesagt: >Gott hat nichts herabgesandt; ihr befindet euch ja in großem Irrtum.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccli{Und sie sagen: «Hätten wir nur gehört und Verstand gehabt, wären wir nun nicht unter den Gefährten des Höllenbrandes.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcclii{Und sie bekennen ihre Schuld. Weg mit den Gefährten des Höllenbrandes!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccliii{Für diejenigen, die ihren Herrn im Verborgenen fürchten, ist Vergebung und ein großer Lohn bestimmt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccliv{Ihr mögt eure Worte im geheimen sprechen oder offen äußern, Er weiß über das innere Geheimnis Bescheid.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcclv{Sollte Er denn nicht über die Bescheid wissen, die Er erschaffen hat? Und Er ist der Feinfühlige, der Kenntnis von allem hat.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcclvi{Er ist es, der euch die Erde gefügig gemacht hat. So schreitet auf ihrem Rücken umher und eßt von dem, was Er (euch) beschert. Und zu Ihm führt die Auferstehung.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcclvii{Wähnt ihr euch denn in Sicherheit davor, daß der, der im Himmel ist, die Erde mit euch versinken läßt, so daß sie ins Schwanken gerät?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcclviii{Oder wähnt ihr euch in Sicherheit davor, daß der, der im Himmel ist, einen Sandsturm gegen euch schickt? Da werdet ihr erfahren, wie seine Warnung ist.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcclix{Auch diejenigen, die vor ihnen lebten, haben (die Gesandten) der Lüge geziehen. Und wie war meine Mißbilligung!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcclx{Haben sie nicht auf die Vögel über ihnen geschaut, wie sie ihre Flügel ausbreiten und einziehen? Nur der Erbarmer hält sie. Er sieht wohl alle Dinge.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcclxi{Oder wer ist anstelle des Erbarmers derjenige, der euch eine Heerschar wäre, die euch unterstützen könnte? Die Ungläubigen fallen nur der Betörung anheim.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcclxii{Oder wer ist denn der, der euch Lebensunterhalt bescheren könnte, wenn Er seine Versorgung zurückhält? Aber nein, sie verharren in Rebellion und Abneigung.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcclxiii{Folgt denn einer, der, auf sein Gesicht gestürzt, umherschreitet, eher der Rechtleitung, oder einer, der aufrecht auf einem geraden Weg umherschreitet?\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcclxiv{Sprich: Er ist es, der euch entstehen ließ und euch Gehör, Augenlicht und Herz machte. Ihr seid aber wenig dankbar.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcclxv{Sprich: Er ist es, der euch auf der Erde vermehrte. Und zu Ihm werdet ihr versammelt werden.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcclxvi{Und sie sagen: «Wann wird diese Androhung eintreffen, so ihr die Wahrheit sagt?»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcclxvii{Sprich: Nur Gott weiß darüber Bescheid. Und ich bin ein deutlicher Warner.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcclxviii{Wenn sie es aber aus der Nähe sehen, dann werden die Gesichter derer, die ungläubig sind, schlimm betroffen. Und es wird gesprochen: «Das ist das, was ihr herbeizurufen suchtet.»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcclxix{Sprich: Was meint ihr, wenn Gott mich und die, die mit mir sind, verderben lassen oder wenn Er sich unser erbarmen sollte, wer wird dann die Ungläubigen schützen vor einer schmerzhaften Pein?\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcclxx{Sprich: Er ist der Erbarmer. An Ihn glauben Wir, und auf Ihn vertrauen wir. Ihr werdet noch erfahren, wer sich im offenkundigen Irrtum befindet.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxi{Sprich: Was meint ihr, wenn euer Wasser versiegen sollte, wer könnte euch sprudelndes Wasser bringen?\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxii{\basmalahde Nun. Bei dem Schreibrohr und (bei) dem, was sie zeilenweise niederschreiben.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxiii{Du bist dank der Gnade deines Herrn kein Besessener.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxiv{Bestimmt ist für dich ein Lohn, der nicht aufhört.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxv{Und du besitzt großartige Charakterzüge.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxvi{Du wirst schauen, und auch sie werden schauen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxvii{Wer von euch der Versuchung ausgesetzt ist.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxviii{Dein Herr weiß besser, wer von seinem Weg abirrt, und Er weiß besser, wer die sind, die der Rechtleitung folgen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxix{So gehorche nicht denen, die (die Botschaft) für Lüge erklären.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxx{Sie möchten gern, daß du nur schöne Worte machst, so daß auch sie schöne Worte machen können.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxi{Und gehorche niemandem, der dauernd schwört und verächtlich dasteht,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxii{Der ständig stichelt und Verleumdungen verbreitet,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxiii{Der das Gute verweigert, Übertretungen begeht und Schuld auf sich lädt,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxiv{Der außerdem grobes Benehmen zeigt und ein Eindringling ist,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxv{Auch wenn er Vermögen und Söhne hat.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxvi{Wenn ihm unsere Zeichen verlesen werden, sagt er: «Die Fabeln der Früheren.»\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxvii{Wir werden ihn auf der Nase mit einem Merkmal brandmarken.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxviii{Wir haben sie Prüfungen unterworfen, wie Wir die Besitzer des Gartens Prüfungen unterworfen haben, als sie schworen, sie würden ihn am Morgen abernten,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxix{Ohne einen Vorbehalt zu machen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmccxc{Da machte darin eine verheerende Erscheinung von deinem Herrn die Runde, während sie schliefen,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmccxci{Und er war am Morgen wie abgeerntet.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmccxcii{Da riefen sie am Morgen einander zu:\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmccxciii{«Geht in der Frühe zu eurem Saatfeld, wenn ihr ernten wollt.»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmccxciv{Sie machten sich auf den Weg und flüsterten dabei einander zu:\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmccxcv{«Heute darf kein Bedürftiger zu euch hereinkommen.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmccxcvi{Und sie gingen hin in der Frühe in der Meinung, sie könnten (den Armen ihren Anteil) verwehren.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmccxcvii{Als sie ihn aber sahen, sagten sie: «Wir befinden uns im Irrtum.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmccxcviii{Nein, wir entbehren vielmehr alles.»\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmccxcix{Derjenige von ihnen, der in der Mitte stand, sagte: «Habe ich nicht zu euch gesagt: Würdet ihr doch Gott preisen!»\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmccc{Sie sagten: «Preis sei unserem Herrn! Wir haben gewiß Unrecht getan.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmccci{Da gingen sie aufeinander zu und machten sich gegenseitig Vorwürfe.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcccii{Sie sagten: «O wehe uns! Wir haben ein Übermaß an Frevel gezeigt.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmccciii{Möge unser Herr uns zum Tausch dafür einen besseren (Garten) als ihn geben! Auf Gott richten wir unsere Wünsche.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmccciv{So ist die Pein. Doch die Pein des Jenseits ist größer, wenn sie es nur wüßten!\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmcccv{Für die Gottesfürchtigen sind bei ihrem Herrn Gärten der Wonne bestimmt.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmcccvi{Sollen Wir etwa die Gottergebenen den Übeltätern gleichsetzen?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmcccvii{Was ist mit euch? Wie urteilt ihr nur?\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmcccviii{Oder habt ihr ein Buch, in dem ihr nachforschen (und finden) könnt,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmcccix{Daß euch bestimmt ist, was ihr euch auswählt?\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmcccx{Oder haben Wir uns euch gegenüber eidlich verpflichtet bis zum Tag der Auferstehung, daß ihr bekommt, was ihr entscheidet?\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmcccxi{Frage sie, wer von ihnen dafür bürgt.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmcccxii{Oder haben sie etwa Teilhaber? Sie sollen doch ihre Teilhaber herbeibringen, so sie die Wahrheit sagen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmcccxiii{Am Tag, da die Angelegenheit ernst wird und sie aufgerufen werden, sich in Anbetung niederzuwerfen, sie aber es nicht vermögen,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmcccxiv{Während ihre Blicke gesenkt sind und Erniedrigung auf ihnen schwer lastet. Dabei wurden sie doch aufgefordert, sich in Anbetung niederzuwerfen, als sie gesund waren.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmcccxv{Laß Mich in Ruhe mit denen, die diese Botschaft für Lüge erklären. Wir werden sie nach und nach kommen lassen, von wo sie es nicht wissen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmcccxvi{Und Ich gewähre ihnen Aufschub. Meine List ist fest.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmcccxvii{Oder verlangst du etwa einen Lohn von ihnen, so daß sie mit Schulden belastet würden?\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmcccxviii{Oder liegt ihnen das Unsichtbare vor, so daß sie (es) aufschreiben?\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmcccxix{So sei geduldig, bis das Urteil deines Herrn eintrifft. Und sei nicht wie der mit dem Fisch, als er voller Gram (zu Gott) rief.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmcccxx{Und hätte ihn nicht eine Gnade von seinem Herrn rechtzeitig erreicht, wäre er auf das kahle Land geworfen worden und hätte sich Tadel zugezogen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxi{Da erwählte ihn sein Herr und machte ihn zu einem der Rechtschaffenen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxii{Diejenigen, die ungläubig sind, würden dich, wenn sie die Ermahnung hören, mit ihren Blicken beinahe ins Straucheln bringen. Und sie sagen: «Er ist ja ein Besessener.»\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxiii{Doch es ist nichts als eine Ermahnung für die Weltenbewohner.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxiv{\basmalahde Die fällig wird,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxv{Was ist die, die fällig wird?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxvi{Woher sollst du wissen, was die ist, die fällig wird?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxvii{Die Thamud und die Aad erklärten die Katastrophe für Lüge.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxviii{Die Thamud nun wurden durch den hereinbrechenden Schrei vernichtet.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxix{Die Aad aber wurden durch einen gewaltigen eiskalten Wind vernichtet,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxx{Den Er sieben Nächte und acht Tage lang immer wieder gegen sie einsetzte. Da hättest du sehen können, wie die Leute dort auf dem Boden lagen, als wären sie Stämme hohler Palmen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxi{Kannst du denn etwas sehen, was von ihnen übriggeblieben wäre?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxii{Und Pharao und diejenigen, die vor ihm lebten, und die verschwundenen Städte zeigten sündiges Verhallen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxiii{Sie waren gegen den Gesandten ihres Herrn ungehorsam. I) a ergriff Er sie mit einem sich steigernden heftigen Griff.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxiv{Als das Wasser (alles) überflutete, trugen Wir euch auf dem dahinfahrenden (Schiff),\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxv{Um es für euch zu einer Erinnerung zu machen, und damit es von jedem aufnahmefähigen Ohr aufgenommen wird.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxvi{Und wenn in die Trompete nur einmal geblasen wird\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxvii{Und Erde und Berge gehoben und auf einen Schlag zu Staub gemacht werden,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxviii{An jenem Tag trifft (die Stunde) ein, die eintreffen wird,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxix{Und der Himmel spaltet sich, so daß er an jenem Tag brüchig wird,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcccxl{Und die Engel stehen an seinem Rand. Über ihnen tragen an jenem Tag acht den Thron deines Herrn.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcccxli{An jenem Tag werdet ihr vorgeführt, und nichts von euch bleibt verborgen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlii{Wem dann sein Buch in seine Rechte gereicht wird, der sagt: «Nehmt und lest mein Buch.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcccxliii{Ich rechnete ja damit, daß ich meiner Abrechnung begegnen werde.»\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcccxliv{Er befindet sich da in einem zufriedenen Leben,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlv{In einem hochgelegenen Garten,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlvi{Dessen Früchte herabhängen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlvii{«Eßt und trinkt zu eurem Wohl für das, was ihr früher in den vergangenen Tagen getan habt.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlviii{Wem aber sein Buch in seine Linke gereicht wird, der sagt: «O wäre mir doch mein Buch nicht gereicht worden!\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlix{Und hätte ich doch nicht erfahren, wie meine Abrechnung aussieht!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcccl{O wäre dies nur das Ende!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcccli{Nicht nützt mir mein Vermögen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmccclii{Vernichtet ist meine Machtfülle.»\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcccliii{- «Nehmt ihn und fesselt ihn.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcccliv{Dann laßt ihn in der Hölle brennen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmccclv{Dann legt ihn in eine Kette, deren Länge siebzig Ellen ist.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmccclvi{Er glaubte nämlich nicht an Gott, den Majestätischen,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmccclvii{Und er hielt nicht zur Speisung des Bedürftigen an.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmccclviii{Hier hat er heute keinen warmherzigen Freund,\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmccclix{Und auch keine Speise außer Eiterflüssigkeit,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmccclx{Die nur die Sünder essen.»\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmccclxi{O nein, Ich schwöre bei dem, was ihr erblickt,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmccclxii{Und dem, was ihr nicht erblickt.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmccclxiii{Das ist die Rede eines edlen Gesandten,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmccclxiv{Das ist nicht die Rede eines Dichters. Aber ihr seid ja so wenig gläubig.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmccclxv{Das ist auch nicht die Rede eines Wahrsagers. Aber ihr bedenkt es ja so wenig.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmccclxvi{Es ist eine Herabsendung vom Herrn der Welten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmccclxvii{Und hätte er sich gegen Uns einige Äußerungen selbst in den Mund gelegt,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmccclxviii{Wir hätten ihn gewiß an der Rechten gefaßt\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmccclxix{Und ihm dann die Herzader durchschnitten,\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmccclxx{Und keiner von euch hätte (Uns) von ihm abhalten können.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxi{Wahrlich, es ist eine Erinnerung für die Gottesfürchtigen.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxii{Und wahrlich, Wir wissen, daß einige von euch (es) für Lüge erklären.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxiii{Und wahrlich, es ist ein Grund zum Bedauern für die Ungläubigen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxiv{Und wahrlich, es ist die Wahrheit, die gewiß ist.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxv{So preise den Namen deines Herrn, des Majestätischen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxvi{\basmalahde Ein Fragesteller fragt nach einer Pein, die eintreffen wird\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxvii{Für die Ungläubigen, die niemand abwehren kann,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxviii{Von Gott her, dem Herrn der Himmelsleiter.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxix{Die Engel und der Geist steigen zu Ihm empor an einem Tag, dessen Ausmaß fünfzigtausend Jahre sind.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxx{Übe darum schöne Geduld.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxi{Sie sehen sie weit entfernt,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxii{Wir aber sehen sie nahe.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxiii{Am Tag, da der Himmel wie geschmolzenes Erz\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxiv{Und die Berge wie Wolleflocken sein werden,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxv{Und kein warmherziger Freund seinen Freund (nach seinem Befinden) fragt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxvi{Sie werden sie gewiß zu sehen bekommen. Der Übeltäter möchte sich gern von der Pein jenes Tages loskaufen mit seinen Söhnen\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxvii{Und seiner Gefährtin und seinem Bruder\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxviii{Und seiner Sippe, die ihm eine Heimstätte bietet,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxix{Und allen, die auf der Erde sind. Er wünscht, dies möge ihn dann retten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcccxc{Aber nein, es ist ein loderndes Feuer,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcccxci{Das die Gliedmaßen wegnimmt,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcii{Das den ruft, der den Rücken kehrt und fortgeht,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcccxciii{Vermögen) zusammenträgt und in Behältern hortet.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcccxciv{Der Mensch ist als kleinmütig erschaffen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcv{Wenn das Böse ihn trifft, ist er sehr mutlos;\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcvi{Und wenn ihm Gutes widerfährt, verweigert er es anderen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcvii{Ausgenommen sind die, die beten\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcviii{Und in ihrem Gebet beharrlich sind,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcix{Und die ein bestimmtes Recht auf ihr Vermögen einräumen\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcd{Dem Bettler und dem Unbemittelten,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcdi{Und die den Tag des Gerichtes für wahr halten,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcdii{Und die vor der Pein ihres Herrn erschrocken sind\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcdiii{- vor der Pein ihres Herrn befindet sich niemand in Sicherheit -,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcdiv{Und die ihre Scham bewahren,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcdv{Außer gegenüber ihren Gattinnen, oder was ihre Rechte (an Sklavinnen) besitzt - dann sind sie nicht zu tadeln;\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcdvi{Diejenigen aber, die darüber hinaus (andere) begehren, das sind die, die Übertretungen begehen -,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmcdvii{Und die auf das ihnen Anvertraute und ihre Verpflichtung achtgeben\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmcdviii{Und die ihr Zeugnis ablegen,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmcdix{Und die ihr Gebet einhalten.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmcdx{Diese befinden sich in Gärten und werden (darin) ehrenvoll behandelt.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmcdxi{Was ist mit denen, die ungläubig sind, daß sie den Hals nach dir recken\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmcdxii{Von rechts und von links, in Gruppen aufgeteilt?\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmcdxiii{Erhofft (wirklich) jeder von ihnen, in einen Garten der Wonne eingehen zu können?\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmcdxiv{Aber nein, Wir haben sie doch aus dem erschaffen, was sie wissen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmcdxv{Nein, Ich schwöre beim Herrn der östlichen und der westlichen Gegenden, Wir haben dazu die Macht,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmcdxvi{Sie durch andere zu ersetzen, die besser sind als sie. Und niemand kann Uns voraus sein.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmcdxvii{So laß sie ausschweifende Reden halten und ihr Spiel treiben, bis sie ihrem Tag begegnen, der ihnen angedroht ist,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmcdxviii{Dem Tag, da sie aus den Gräbern hastig herauskommen, als würden sie zu den Opfersteinen eilen,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmcdxix{Während ihre Blicke gesenkt sind und Erniedrigung auf ihnen schwer lastet. Das ist der Tag, der ihnen immer wieder angedroht wurde.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmcdxx{\basmalahde Wir haben Noach zu seinem Volk gesandt: «Warne dein Volk, bevor eine schmerzhafte Pein über sie kommt.»\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxi{Er sagte: «O mein Volk, ich bin euch ein deutlicher Warner:\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxii{Dienet Gott und fürchtet Ihn, und gehorchet mir,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxiii{Dann vergibt Er euch etwas von euren Sünden und stellt euch zu einer bestimmten Frist zurück. Die Frist Gottes, wenn sie eintrifft, kann nicht aufgeschoben werden, wenn ihr es nur wüßtet!»\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxiv{Er sagte: «Mein Herr, ich habe mein Volk bei Nacht und bei Tag gerufen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxv{Mein Rufen hat sie aber nur in ihrer Flucht bestärkt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxvi{Sooft ich sie rief, damit Du ihnen vergibst, steckten sie ihre Finger in die Ohren, hüllten sich in Gewänder, blieben verstockt und verhielten sich sehr hochmütig.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxvii{Dann richtete ich an sie den Ruf öffentlich.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxviii{Dann sprach ich zu ihnen offen und im geheimen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxix{Ich sagte: Bittet euren Herrn um Vergebung - Er vergibt ja viel -,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxx{Dann wird Er den Himmel über euch ergiebig regnen lassen\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxi{Und euch mit Vermögen und Söhnen beistehen, für euch Gärten machen und für euch Bäche machen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxii{Was ist mit euch, daß ihr nicht von Gott erwartet, daß Er würdevoll handelt,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxiii{Wo Er euch doch in Phasen erschaffen hat?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxiv{Habt ihr nicht gesehen, wie Gott sieben Himmel in Schichten erschaffen hat,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxv{Den Mond darin zu einem Licht gemacht und die Sonne zu einer Leuchte gemacht hat?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxvi{Gott hat euch aus der Erde wie Pflanzen wachsen lassen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxvii{Dann wird Er euch zu ihr zurückkehren lassen und (wieder aus ihr) hervorbringen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxviii{Und Gott hat euch die Erde zu einer ausgebreiteten Unterlage gemacht,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxix{Damit ihr auf ihr Wege als breite Durchgänge begehen könnt.»\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcdxl{Noach sagte: «Mein Herr, sie sind gegen mich ungehorsam geworden und sind einem gefolgt, der durch sein Vermögen und seine Kinder nur noch größeren Verlust hat.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcdxli{Und sie haben ungeheure Ränke geschmiedet.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlii{Und sie sagen: >Verlaßt doch nicht eure Götter. Verlaßt doch nicht Wadd, noch Suwaa, noch Yaghuth, und auch nicht Ya uq und Nasr.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcdxliii{Sie haben viele irregeführt. So laß die, die Unrecht tun, noch tiefer in die Irre gehen.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcdxliv{Wegen ihrer Sünden wurden sie ertränkt und in ein Feuer geführt. Da fanden sie für sich außer Gott keine Helfer.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlv{Und Noach sagte: «Mein Herr, laß keinen von den Ungläubigen weiter auf der Erde wohnen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlvi{Wenn Du sie (leben) läßt, führen sie deine Diener irre und bringen zur Welt nur solche, die voller Laster und sehr ungläubig sind.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlvii{Mein Herr, vergib mir und meinen Eltern und dem, der als Gläubiger mein Haus betritt, und den gläubigen Männern und den gläubigen Frauen. Und stürze die, die Unrecht tun, noch tiefer ins Verderben.»\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlviii{\basmalahde Sprich: Mir ist offenbart worden, daß eine Schar Djinn zuhörte. Sie sagten: «Wir haben einen erstaunlichen Koran gehört,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlix{Der zum rechten Wandel leitet. So haben wir an ihn geglaubt. Und wir werden unserem Herrn niemanden beigesellen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdl{Und erhaben ist die Majestät unseres Herrn. Er hat sich weder eine Gefährtin noch ein Kind genommen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdli{Und der Tor unter uns pflegte gegen Gott etwas Abwegiges zu sagen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcdlii{Und wir meinten, daß weder die Menschen noch die Djinn gegen Gott eine Lüge sagen würden.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcdliii{Und einige Männer unter den Menschen suchten Zuflucht bei einigen Männern unter den Djinn und mehrten somit bei ihnen ihre Gewalttätigkeit.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcdliv{Und sie meinten, wie ihr ja meint, daß Gott niemanden auferwecken würde.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcdlv{Und wir haben den Himmel abgesucht und festgestellt, daß er mit starken Wächtern und Sternschnuppen voll ist.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcdlvi{Und wir pflegten uns an verschiedenen Stellen von ihm zu setzen, um zu lauschen. Wer aber jetzt horcht, der findet, daß eine Sternschnuppe auf ihn lauert.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcdlvii{Und wir wissen nicht, ob für die Bewohner der Erde Böses vorgesehen ist, oder ob ihr Herr vorhat, sie zum rechten Wandel zu führen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcdlviii{Und unter uns gibt es Rechtschaffene; und unter uns gibt es solche, die es nicht sind. Wir folgten getrennten Wegen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcdlix{Und wir meinen, daß wir auf der Erde Gottes Willen nicht vereiteln können, und daß wir ihn nicht durch Flucht vereiteln können.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcdlx{Und als wir die Rechtleitung hörten, glaubten wir an sie. Wer an seinen Herrn glaubt, der befürchtet keinen Lohnabzug und keine Gewaltanwendung.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxi{Und unter uns gibt es solche, die gottergeben sind; und unter uns gibt es solche, die dem Falschen zuneigen. Diejenigen nun, die gottergeben sind, bemühen sich um den rechten Wandel.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxii{Diejenigen aber, die dem Falschen zuneigen, werden Brennholz für die Hölle sein.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxiii{Und würden sie den rechten Weg einschlagen, würden Wir ihnen reichlich Wasser zu trinken geben,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxiv{Um sie dadurch der Versuchung auszusetzen. Und wer sich von der Ermahnung seines Herrn abwendet, den wird Er in eine überwältigende Pein eingehen lassen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxv{Und die Anbetungsstätten gehören Gott; so ruft neben Gott niemanden an.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxvi{Und als der Diener Gottes aufstand, um Ihn anzurufen, hätten sie ihn im Gedränge fast erdrückt.»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxvii{Sprich: Ich rufe meinen Herrn an, und ich geselle Ihm niemanden bei.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxviii{Sprich: Ich verfüge für euch weder über Schaden noch über die rechte Unterweisung.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxix{Sprich: Niemand kann mich gegen Gott schützen, und ich kann außer bei Ihm keinen Ort finden, wohin ich mich begeben könnte.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxx{Ich habe nur etwas auszurichten von Gott und seinen Botschaften. Und für diejenigen, die gegen Gott und seinen Gesandten ungehorsam sind, ist das Feuer der Hölle bestimmt; darin werden sie auf immer ewig weilen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxi{Wenn sie dann sehen, was ihnen angedroht ist, werden sie wissen, wer die schwächeren Helfer und die geringere Zahl hat.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxii{Sprich: Ich weiß nicht, ob das, was euch angedroht ist, nahe ist, oder ob mein Herr ihm noch eine Frist setzt,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxiii{Er), der über das Unsichtbare Bescheid weiß und niemandem über das, was bei Ihm verborgen ist, Auskunft gibt,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxiv{Außer dem Gesandten, der Ihm genehm ist; dann läßt Er vor ihm und hinter ihm Wächter gehen, die ihn belauern,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxv{Um festzustellen, ob sie die Botschaften ihres Herrn ausgerichtet haben. Er umfaßt, was bei ihnen ist, und Er erfaßt die Zahl aller Dinge.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxvi{\basmalahde Der du dich eingehüllt hast,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxvii{Steh auf (zum Gebet) die ganze Nacht, bis auf einen kleinen Teil,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxviii{Die halbe Nacht, oder verringere sie um einen kleinen Teil,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxix{Oder füge etwas hinzu. Und trage den Koran Abschnitt für Abschnitt vor.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxx{Wir werden dir Worte auferlegen, die schwer zu tragen sind.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxi{Das Aufstehen in der Nacht hinterläßt einen stärkeren Eindruck und läßt eher die rechten Worte finden.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxii{Du wirst am Tag lange beansprucht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxiii{Und so gedenke des Namens deines Herrn und widme dich Ihm allein.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxiv{Er ist) der Herr des Ostens und des Westens. Es gibt keinen Gott außer Ihm. So nimm Ihn dir zum Sachwalter.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxv{Und ertrage mit Geduld, was sie sagen. Und entferne dich auf schöne Weise von ihnen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxvi{Laß Mich in Ruhe mit denen, die (die Botschaft) für Lüge erklären und die ein angenehmes Leben haben. Und gewähre ihnen noch eine kurze Frist.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxvii{Bei Uns stehen schwere Fesseln und eine Hölle bereit,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxviii{Und auch eine Speise, die im Hals steckenbleibt, und eine schmerzhafte Pein,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxix{Am Tag, da die Erde und die Berge beben und die Berge (wie) ein zerrinnender Sandhaufen sein werden.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcdxc{Wir haben zu euch einen Gesandten als Zeugen über euch geschickt, so wie Wir zu Pharao einen Gesandten geschickt haben.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcdxci{Pharao war aber ungehorsam gegen den Gesandten. Da ergriffen Wir ihn mit einem harten Griff.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcii{Wie wollt ihr, wenn ihr ungläubig seid, euch vor einem Tag hüten, der die Kinder zu Greisen werden läßt?\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcdxciii{Der Himmel wird sich an ihm spalten. Seine Drohung wird ausgeführt.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcdxciv{Dies ist nur eine Erinnerung. Wer nun will, schlägt einen Weg zu seinem Herrn ein.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcv{Dein Herr weiß, daß du (zum Gebet) aufstehst beinahe zwei Drittel der Nacht oder die Hälfte oder ein Drittel davon, und ebenso eine Gruppe von denen, die mit dir sind. Und Gott bestimmt das Maß der Nacht und des Tages. Er weiß, daß ihr es nicht (selbst) würdet erfassen können. Da wandte Er sich euch zu. So verlest aus dem Koran, was leicht (zu bewältigen) ist. Er weiß, daß es unter euch Kranke geben würde, und andere, die im Land umherwandern im Streben nach der Huld Gottes, und (wieder) andere, die auf dem Weg Gottes kämpfen. Verlest also daraus, was leicht (zu bewältigen) ist, und verrichtet das Gebet und entrichtet die Abgabe und leiht Gott ein schönes Darlehen. Und was ihr für euch selbst an Gutem vorausschickt, das werdet ihr bei Gott vorfinden als noch besser und großartiger belohnt. Und bittet Gott um Vergebung. Gott ist ja voller Vergebung und barmherzig.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcvi{\basmalahde Der du dich zugedeckt hast,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcvii{Steh auf und warne,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcviii{Und preise die Größe deines Herrn,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcix{Und reinige deine Kleider,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmd{Und entferne dich von der Unreinheit (des Götzendienstes),\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdi{Und poch nicht auf dein Verdienst, um mehr zu erhalten,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdii{Und sei geduldig, bis dein Herr sein Urteil fällt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdiii{Wenn dann in das Horn gestoßen wird,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdiv{Dann ist es an jenem Tag ein schwerer Tag,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdv{Für die Ungläubigen nicht leicht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdvi{Laß Mich in Ruhe mit dem, den Ich allein erschaffen habe,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdvii{Und dem Ich ausgedehntes Vermögen verliehen habe,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdviii{Und auch Söhne, die als Zeugen da sind,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdix{Und dem Ich alles schön geebnet habe,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdx{Der aber erhofft, daß Ich (ihm) noch mehr gebe.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdxi{Nein, er zeigte sich widerspenstig gegen meine Zeichen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdxii{Ich werde Mühsal auf ihm schwer lasten lassen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdxiii{Er dachte nach und wog ab.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdxiv{Dem Tod geweiht sei er, wie seine Abwägung war!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdxv{Noch einmal: Dem Tod geweiht sei er, wie seine Abwägung war!\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdxvi{Dann schaute er,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdxvii{Dann runzelte er die Stirn und zeigte Widerwillen,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdxviii{Dann kehrte er den Rücken und verhielt sich hochmütig,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdxix{Und er sagte: «Das ist doch nur eine Zauberei, die (sonst) überliefert wird.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdxx{Das sind nur die Worte von Menschen.»\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdxxi{Ich werde ihn im Höllenfeuer brennen lassen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdxxii{Woher sollst du wissen, was das Höllenfeuer ist?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdxxiii{Es läßt keinen Rest bestehen, und es läßt nichts übrigbleiben.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdxxiv{Es versengt die Haut.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdxxv{Es hat über sich neunzehn (Wächter).\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdxxvi{Als Wächter des Feuers haben Wir nur Engel eingesetzt. Und ihre Zahl haben Wir zu einer Versuchung gemacht für die, die ungläubig sind, damit diejenigen Gewißheit erlangen, denen das Buch zugekommen ist, und damit diejenigen, die glauben, an Glauben zunehmen, und damit diejenigen, denen das Buch zugekommen ist, und auch die Gläubigen keinen Zweifel hegen, und damit diejenigen, in deren Herzen Krankheit ist, und (auch) die Ungläubigen sagen: «Was will denn Gott mit einem solchen Gleichnis?» Auf diese Weise führt Gott irre, wen Er will, und leitet recht, wen Er will. Und über die Heerscharen deines Herrn weiß nur Er Bescheid. Und es ist nur eine Ermahnung für die Menschen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdxxvii{Nein, beim Mond,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdxxviii{Bei der Nacht, wenn sie den Rücken kehrt,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdxxix{Beim Morgen, wenn er anbricht,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdxxx{Siehe, sie ist eine der größten (Gestalten des Verderbens),\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxi{Zur Warnung für die Menschen,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxii{Für jeden von euch, der vorankommen oder zurückbleiben will.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxiii{Ein jeder haftet für das, was er erworben hat,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxiv{Ausgenommen die von der Rechten;\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxv{Sie befinden sich in Gärten, und sie fragen einander\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxvi{Nach den Übeltätern:\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxvii{«Was hat euch ins Höllenfeuer getrieben?»\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxviii{Sie sagen: «Wir gehörten nicht zu denen, die beteten,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxix{Und wir pflegten nicht den Bedürftigen zu speisen,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmdxl{Und wir gingen auf ausschweifende Reden ein mit denen, die solche hielten,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmdxli{Und wir erklärten den Tag des Gerichtes für Lüge,\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmdxlii{Bis das sichere Ende uns ereilte.»\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmdxliii{Nun nützt ihnen die Fürsprache der Fürsprechenden nicht.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmdxliv{Was ist mit ihnen, daß sie sich von der Ermahnung abwenden,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmdxlv{Als wären sie aufgeschreckte Wildesel,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmdxlvi{Die vor einem jagenden Löwen fliehen?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmdxlvii{Aber nein, jeder von ihnen will, daß ihm aufgeschlagene Blätter zukommen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmdxlviii{Nein, sie fürchten sich nicht vor dem Jenseits.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmmdxlix{Nein, er ist eine Erinnerung.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmmdl{Wer will, gedenkt seiner.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmmdli{Sie werden aber (seiner) nicht gedenken, es sei denn, Gott will es. Ihm gebührt, daß man Ihn fürchtet, und Ihm gebührt zu vergeben.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmmdlii{\basmalahde Nein, Ich schwöre beim Tag der Auferstehung.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdliii{Nein, Ich schwöre bei der Seele, die sich selbst Tadel ausspricht.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdliv{Meint denn der Mensch, daß Wir seine Gebeine nicht zusammenfügen werden?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdlv{Ja doch, Wir sind imstande, seine Fingerspitzen zurechtzubilden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdlvi{Doch der Mensch will voller Laster sein, sein Leben lang.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdlvii{Er fragt: «Wann wird der Tag der Auferstehung sein?»\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdlviii{Wenn dann das Augenlicht geblendet ist\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdlix{Und der Mond sich verfinstert\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdlx{Und Sonne und Mond miteinander vereinigt werden,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdlxi{An jenem Tag sagt der Mensch: «Wohin nun fliehen?»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdlxii{Nein, es gibt keine Zuflucht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdlxiii{Zu deinem Herrn führt an jenem Tag der Weg zum Aufenthaltsort.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdlxiv{An jenem Tag wird dem Menschen kundgetan, was er vorausgeschickt und zurückgestellt hat.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdlxv{Nein, der Mensch bringt einsichtige Beweise gegen sich selbst,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdlxvi{Auch wenn er seine Entschuldigungen vorbringt.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdlxvii{Bewege deine Zunge nicht damit, um dich damit zu übereilen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdlxviii{Uns obliegt es, ihn zusammenzustellen und ihn vorzulesen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdlxix{Und (erst) wenn Wir ihn vorgelesen haben, dann folge du der Art, ihn vorzulesen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdlxx{Dann obliegt es Uns, ihn zu erläutern.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxi{Aber nein, ihr liebt eher das, was schnell eintrifft,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxii{Und vernachlässigt das Jenseits.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxiii{An jenem Tag gibt es strahlende Gesichter,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxiv{Die zu ihrem Herrn schauen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxv{Und es gibt an jenem Tag finstere Gesichter;\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxvi{Du möchtest meinen, es widerfahre ihnen ein das Rückgrat brechendes Unglück.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxvii{Nein, wenn sie das Schlüsselbein erreicht\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxviii{Und gesagt wird: «Wer kann als Beschwörer auftreten?»,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxix{Und wenn er meint, es sei (jetzt) der Abschied,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxx{Und wenn (bei ihm) das eine Bein auf das andere trifft,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxi{An jenem Tag wird er zu deinem Herrn getrieben.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxii{Denn er hielt nicht (die Botschaft) für wahr, und er betete nicht,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxiii{Sondern er erklärte (sie) für Lüge, und er kehrte sich ab.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxiv{Dann ging er zu seinen Angehörigen mit stolzen Schritten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxv{- «Recht geschieht dir, ja recht geschieht es.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxvi{Noch einmal: Recht geschieht dir, ja recht geschieht es.»\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxvii{Meint denn der Mensch, daß er unbeachtet gelassen wird?\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxviii{Ist er nicht ein Samentropfen, der sich ergießt,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxix{Dann ein Embryo gewesen? Da hat Er erschaffen und gebildet,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdxc{Und daraus das Paar gemacht: den Mann und das Weib.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdxci{Ist ein solcher (Gott) denn nicht imstande, die Toten wieder lebendig zu machen?\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdxcii{\basmalahde Gab es (nicht) für den Menschen eine Zeitspanne, in der er kein nennenswertes Ding war?\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdxciii{Wir haben den Menschen aus einem Tropfen, einem Gemisch erschaffen, um ihn zu prüfen. Und Wir haben ihn mit Gehör und Augenlicht versehen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdxciv{Wir haben ihn den (rechten) Weg geführt, ob er nun dankbar oder undankbar ist.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdxcv{Wir halten für die Ungläubigen Ketten und Fesseln und einen Feuerbrand bereit.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdxcvi{Die Frommen trinken aus einem Becher, in dem Kampfer beigemischt ist,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdxcvii{Aus einer Quelle, aus der die Diener Gottes trinken, die sie sprudelnd hervorbrechen lassen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdxcviii{Sie erfüllen das Gelübde und fürchten einen Tag, dessen Unheil sich ausbreitet,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdxcix{Und sie geben, aus Liebe zu Ihm, Speise zu essen dem Bedürftigen, der Waise und dem Gefangenen:\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdc{«Wir speisen euch in der Suche nach dem Antlitz Gottes. Wir wollen von euch weder Lohn noch Dank.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdci{Wir fürchten von Seiten unseres Herrn einen finsteren, schauderhaften Tag.»\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcii{So bewahrt sie Gott vor dem Unheil jenes Tages und läßt sie strahlendes Glück und Freude vorfinden,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdciii{Und Er vergibt ihnen dafür, daß sie geduldig sind, mit einem Garten und mit Seide.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdciv{Sie lehnen sich darin auf Liegen, und sie leiden darin weder unter heißer Sonne noch unter bitterer Kälte.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcv{Seine Schatten hängen über ihnen herab, und seine Früchte sind ihnen sehr leicht erreichbar gemacht.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcvi{Herumgereicht werden ihnen Gefäße aus Silber und Humpen wie Gläser,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcvii{Gläser aus Silber, die sie nach Maß bemessen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcviii{Und darin wird ihnen ein Becher zu trinken gegeben, in dem Ingwer beigemischt ist,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcix{Aus einer Quelle darin, die Salsabiel genannt wird.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcx{Und unter ihnen machen ewig junge Knaben die Runde. Wenn du sie siehst, meinst du, sie seien ausgestreute Perlen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcxi{Und wenn du dort hinschaust, siehst du Wonne und großes Königreich.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcxii{Sie haben grüne Gewänder aus Seide und auch Brokat an, und sie sind mit Armringen aus Silber geschmückt. Und ihr Herr gibt ihnen ein reines Getränk zu trinken.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcxiii{«Das ist der Lohn für euch, und euch wird für euer Mühen gedankt.»\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcxiv{Wir, ja Wir haben den Koran auf dich nach und nach hinabgesandt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcxv{So sei geduldig, bis das Urteil deines Herrn eintrifft, und gehorche keinem von ihnen, der ein Sünder oder ein Ungläubiger ist.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcxvi{Und gedenke des Namens deines Herrn morgens und abends,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcxvii{Und wirf dich in der Nacht vor Ihm nieder und preise Ihn lange zur Nachtzeit.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcxviii{Diese da lieben das, was schnell eintrifft, und vernachlässigen einen Tag, der schwer lasten wird.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcxix{Wir haben sie doch erschaffen und ihre Glieder gefestigt. Und wenn Wir wollen, ersetzen Wir sie durch andere ihresgleichen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcxx{Dies ist eine Erinnerung. Wer nun will, schlägt einen Weg zu seinem Herrn ein.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxi{Und ihr wollt nicht, es sei denn, Gott will es. Gott weiß Bescheid und ist weise.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxii{Er läßt, wen Er will, in seine Barmherzigkeit eingehen. Für die, die Unrecht tun, hält Er eine schmerzhafte Pein bereit.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxiii{\basmalahde Bei denen, die nacheinander wie eine Mähne gesandt werden\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxiv{Und die im Sturm daherstürmen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxv{Bei denen, die offen ausbreiten\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxvi{Und die deutlich unterscheiden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxvii{Und die eine Ermahnung überbringen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxviii{Als Entschuldigung oder als Warnung!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxix{Was euch angedroht wird, wird gewiß eintreffen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxx{Wenn die Sterne ausgewischt werden,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxi{Und wenn der Himmel Risse aufweist,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxii{Und wenn die Berge (in den Wind) gestreut werden,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxiii{Und wenn für die Gesandten der Termin festgelegt wird.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxiv{Auf was für einen Tag wird die Frist gesetzt?\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxv{Für den Tag der Scheidung.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxvi{Woher sollst du wissen, was der Tag der Scheidung ist?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxvii{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären!\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxviii{Haben Wir nicht die Früheren verderben lassen?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxix{Dann lassen Wir ihnen die Späteren folgen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcxl{So handeln Wir an den Übeltätern.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcxli{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlii{Haben Wir euch nicht aus verächtlichem Wasser erschaffen,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcxliii{Das Wir in einen festen Aufenthaltsort gelegt haben,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcxliv{Bis zu einer bekannten Frist?\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlv{Wir haben sie bemessen. Wie trefflich setzen Wir das Maß fest!\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlvi{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlvii{Haben Wir nicht die Erde gemacht, zu sammeln\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlviii{Die Lebenden und die Toten,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlix{Und auf ihr festgegründete, hochragende (Berge) gemacht und euch erfrischendes Wasser zu trinken gegeben?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcl{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcli{- «Begebt euch zu dem, was ihr immer wieder für Lüge erklärt habt,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdclii{Begebt euch in einen Schatten mit drei Verzweigungen,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcliii{Der keinen (wirklichen) Schatten spendet und auch nicht vor den Flammen schützt.»\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdcliv{Sie sprüht Funken (hoch) wie ein Schloß,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdclv{Als wären sie gelbe Kamele.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdclvi{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären!,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdclvii{Das ist der Tag, an dem sie nicht reden\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdclviii{Und ihnen nicht erlaubt wird, Entschuldigungen vorzubringen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdclix{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdclx{- «Das ist der Tag der Scheidung. Wir haben euch zu den Früheren versammelt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdclxi{Wenn ihr nun eine List habt, so führt sie gegen Mich aus.»\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdclxii{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdclxiii{Die Gottesfürchtigen befinden sich inmitten von Schatten und Quellen,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdclxiv{Und Früchten von dem, was sie begehren.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdclxv{- «Eßt und trinkt zu eurem Wohl (als Entlohnung) für das, was ihr zu tun pflegtet.»\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmdclxvi{So lohnen Wir den Rechtschaffenen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmdclxvii{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmdclxviii{Eßt und genießt ein wenig, ihr seid Übeltäter.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmdclxix{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären!\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmdclxx{Und wenn zu ihnen gesprochen wird: «Verneigt euch», verneigen sie sich nicht.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxi{Wehe an jenem Tag denen, die (die Botschaft) für Lüge erklären!\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxii{An welche Botschaft nach dieser wollen sie denn sonst glauben?\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxiii{\basmalahde Wonach fragen sie sich untereinander?\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxiv{Nach dem gewaltigen Bericht,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxv{über den sie uneins sind.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxvi{Nein, sie werden es zu wissen bekommen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxvii{Noch einmal: Nein, sie werden es zu wissen bekommen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxviii{Haben Wir nicht die Erde zu einer Lagerstätte gemacht\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxix{Und die Berge zu Pflöcken?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxx{Und Wir haben euch als Paare erschaffen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxi{Und Wir haben euren Schlaf zum Ausruhen gemacht.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxii{Und Wir haben die Nacht zu einem Kleid gemacht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxiii{Und Wir haben den Tag zum Erwerb des Lebensunterhalts gemacht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxiv{Und Wir haben über euch sieben feste (Himmelsschichten) aufgebaut.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxv{Und Wir haben eine hell brennende Leuchte gemacht.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxvi{Und Wir haben von den Regenwolken strömendes Wasser herabkommen lassen,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxvii{Um damit Korn und Pflanzen hervorzubringen\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxviii{Und Gärten mit dichtem Bestand.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxix{Wahrlich, der Tag der Scheidung ist ein (festgelegter) Termin.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcxc{Am Tag, da in die Trompete geblasen wird und ihr in Scharen herbeikommt,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcxci{Und da der Himmel geöffnet und zu (weiten) Toren wird,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcii{Und da die Berge versetzt und zu einer Luftspiegelung werden.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcxciii{Die Hölle liegt auf der Lauer,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcxciv{Als Heimstatt für die, die ein Übermaß an Frevel zeigen;\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcv{Darin werden sie endlose Zeiten verweilen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcvi{Sie werden darin weder Kühle noch ein Getränk kosten,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcvii{Sondern nur heißes Wasser und stinkenden Eiter,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcviii{Als angemessene Vergeltung.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcix{Sie erwarteten ja keine Abrechnung\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcc{Und haben unsere Zeichen beharrlich für Lüge erklärt.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcci{Alles haben Wir in einem Buch erfaßt.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdccii{So kostet (es); Wir werden euch nur noch eure Pein mehren.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcciii{Für die Gottesfürchtigen ist ein Ort des Erfolges bestimmt,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdcciv{Gärten und Weinstöcke,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccv{Und gleichaltrige Frauen mit schwellenden Brüsten,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccvi{Und ein randvoller Becher.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccvii{Sie hören darin keine unbedachte Rede und keine Bezeichnung der Botschaft als Lüge.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccviii{Dies zum Lohn - als Geschenk entsprechend der Abrechnung - von deinem Herrn,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccix{Dem Herrn der Himmel und der Erde und dessen, was dazwischen ist, dem Erbarmer. Vor Ihm verfügen sie über keine Möglichkeit zu reden,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdccx{Am Tag, da der Geist und die Engel in einer Reihe stehen. Sie werden nicht sprechen, ausgenommen der, dem der Erbarmer es erlaubt und der das Richtige sagt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdccxi{Das ist der Tag der Wahrheit. Wer nun will, unternimmt die Rückkehr zu seinem Herrn.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdccxii{Wir warnen euch vor einer nahe bevorstehenden Pein, am Tag, da der Mensch erblickt, was seine Hände vorausgeschickt haben, und der Ungläubige ruft: «O wäre ich doch nur Staub!»\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdccxiii{\basmalahde Bei denen, die mit Gewalt entreißen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdccxiv{Und denen, die mit Leichtigkeit herausziehen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdccxv{Und denen, die schnell dahinschwimmen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccxvi{Und allen vorauseilen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdccxvii{Und die Angelegenheiten regeln!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdccxviii{Am Tag, da das Beben einsetzt\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdccxix{Und das nächste darauf folgt,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdccxx{An jenem Tag werden die Herzen in unruhiger Angst dastehen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxi{Mit gesenkten Blicken.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxii{Sie sagen: «Sollen wir wirklich in den früheren Zustand zurückgebracht werden?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxiii{Auch wenn wir zu morschen Knochen geworden sind?»\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxiv{Sie sagen: «Das wäre dann eine verlustreiche Rückkehr.»\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxv{Es wird ein einziger Schelteschrei sein,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxvi{Und schon befinden sie sich auf der Erdoberfläche.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxvii{Ist die Geschichte von Mose zu dir gelangt?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxviii{Als sein Herr ihm im heiligen Tal Tuwaa zurief:\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxix{«Geh zu Pharao. Er zeigt ein Übermaß an Frevel.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxx{Und sag: Möchtest du dich nicht läutern,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxi{Und daß ich dich zu deinem Herrn rechtleite, so daß du gottesfürchtig wirst?»\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxii{Er ließ ihn das größte Zeichen sehen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxiii{Er aber erklärte (es) für Lüge und war ungehorsam.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxiv{Dann kehrte er den Rücken und eilte davon.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxv{Er versammelte (sein Volk) und rief (ihnen) zu.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxvi{Er sagte: «Ich bin euer Herr, der Höchste.»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxvii{Da ergriff ihn Gott mit der Strafe des Jenseits und des Diesseits.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxviii{Darin ist eine Lehre für die Gottesfürchtigen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxix{Ist es etwa schwerer, euch zu erschaffen, als den Himmel, den Er aufgebaut hat?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdccxl{Er hob sein Dach empor und gestaltete ihn.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdccxli{Und Er ließ seine Nacht dunkel werden und seinen Morgen hervorkommen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlii{Und Er breitete danach die Erde aus.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdccxliii{Er ließ ihr Wasser und ihre Weideflächen aus ihr hervorkommen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdccxliv{Und Er gründete die Berge fest.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlv{Dies als Nutznießung für euch und euer Vieh.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlvi{Wenn dann die große, alles überwältigende Katastrophe kommt,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlvii{Am Tag, da der Mensch sich an das erinnert, was er erstrebt hat,\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlviii{Und die Hölle denen sichtbar gemacht wird, die sehen können,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlix{Dann wird dem, der ein Übermaß an Frevel gezeigt\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdccl{Und das diesseitige Leben vorgezogen hat,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdccli{Die Hölle zur Heimstätte.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdcclii{Und dem, der den Stand seines Herrn gefürchtet und seiner Seele ihre (bösen) Neigungen verwehrt hat,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdccliii{Wird das Paradies zur Heimstätte.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdccliv{Sie fragen dich nach der Stunde, wann sie feststehen wird.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdcclv{Was hast du denn davon, sie öfter zu erwähnen?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmdcclvi{Ihre Schlußetappe führt zu deinem Herrn.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmdcclvii{Du hast nur die zu warnen, die sie fürchten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmdcclviii{Am Tag, da sie sie sehen werden, ist es so, als hätten sie nur einen Abend verweilt oder den Morgen darauf.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmdcclix{\basmalahde Er runzelte die Stirn und kehrte sich ab,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcclx{Weil der Blinde zu ihm kam.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxi{Woher sollst du es wissen, vielleicht will er sich läutern\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxii{Oder bedenken, so daß ihm die Ermahnung nützt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxiii{Wer sich verhält, als wäre er auf niemanden angewiesen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxiv{Dem widmest du dich,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxv{Und es kümmert dich nicht, daß er sich nicht läutern will.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxvi{Wer aber zu dir eilig kommt\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxvii{Und dabei gottesfürchtig ist,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxviii{Dem schenkst du keine Aufmerksamkeit.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxix{Nein, es ist eine Erinnerung.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxx{Wer nun will, gedenkt seiner.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxi{Er steht) auf Blättern, die in Ehren gehalten werden,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxii{Emporgehoben und rein gehalten,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxiii{In den Händen von Schreibern,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxiv{Die edel und fromm sind.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxv{Dem Tod geweiht sei der Mensch, wie undankbar ist er!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxvi{Aus was für einem Stoff hat Er ihn erschaffen?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxvii{Aus einem Tropfen hat Er ihn erschaffen und ihm sein Maß gesetzt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxviii{Dann macht Er ihm seinen Weg leicht.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxix{Dann läßt Er ihn sterben und bringt ihn ins Grab.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxx{Dann, wenn Er will, erweckt Er ihn auf.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxi{Nein, er hat noch nicht ausgeführt, was Er ihm befohlen hat.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxii{Der Mensch soll auf seine Nahrung schauen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxiii{Wir gießen doch Wasser in Strömen,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxiv{Dann spalten Wir die Erde auf\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxv{Und lassen auf ihr Korn wachsen,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxvi{Und Weinstöcke und Gemüse,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxvii{Und Ölbäume und Palmen,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxviii{Und Gärten mit großen dicken Bäumen,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxix{Und Früchten und Gras,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdccxc{Als Nutznießung für euch und euer Vieh.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccxci{Wenn die ohrenbetäubende (Stunde) kommt,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcii{Am Tag, da einer flieht vor seinem Bruder,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccxciii{Und seiner Mutter und seinem Vater,\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccxciv{Und seiner Gefährtin und seinen Söhnen,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcv{An jenem Tag hat jeder von ihnen eine Angelegenheit, die ihn voll beansprucht,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcvi{An jenem Tag wird es strahlende Gesichter geben,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcvii{Die lachen und sich freuen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcviii{Und es wird an jenem Tag Gesichter geben, auf denen Staub liegt,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcix{Und die von Ruß bedeckt sind.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdccc{Das sind die, die ungläubig und voller Laster sind.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdccci{\basmalahde Wenn die Sonne (von einer Hülle) umwunden wird,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcccii{Und wenn die Sterne herabstürzen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdccciii{Und wenn die Berge versetzt werden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccciv{Und wenn die im zehnten Monat stehenden Kamelstuten vernachlässigt werden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcccv{Und wenn die wilden Tiere versammelt werden,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccvi{Und wenn die Meere angefüllt werden,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcccvii{Und wenn die Seelen zu ihren Partnern gesellt werden,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcccviii{Und wenn das Mädchen, das verscharrt wurde, gefragt wird,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcccix{Wegen welcher Sünde es denn getötet wurde,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcccx{Und wenn die Blätter ausgebreitet werden,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxi{Und wenn der Himmel (wie ein Fell) abgezogen wird,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxii{Und wenn die Hölle angefacht wird,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxiii{Und wenn das Paradies herangebracht wird,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxiv{Dann wird jeder erfahren, was er vorgebracht hat.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxv{Nein, Ich schwöre bei den rückläufigen Planeten,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxvi{Die dahineilen und sich in ihrem Bau verbergen,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxvii{Und bei der Nacht, wenn sie an einem Ende steht,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxviii{Und beim Morgen, wenn er anbricht:\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxix{Das ist die Rede eines edlen Gesandten,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxx{Der Kraft besitzt und beim Herrn des Thrones hochgestellt ist,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxi{Dem man gehorcht und der treu ist.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxii{Euer Gefährte ist kein Besessener.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxiii{Er hat ihn gewiß am deutlichen Horizont gesehen,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxiv{Und er hält nicht aus Geiz das (ihm offenbarte) Verborgene zurück.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxv{Das ist nicht die Rede eines gesteinigten Satans.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxvi{Wo geht ihr denn hin?\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxvii{Das ist nur eine Ermahnung für die Weltenbewohner,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxviii{Für die von euch, die sich recht verhalten wollen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxix{Und ihr wollt nicht, es sei denn, Gott will es, (Er), der Herr der Welten.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxx{\basmalahde Wenn der Himmel zerbricht,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxi{Und wenn die Sterne sich zerstreuen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxii{Und wenn die Meere zum Ausbrechen gebracht werden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxiii{Und wenn die Gräber umgewühlt werden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxiv{Dann wird jeder erfahren, was er vorausgeschickt und zurückgestellt hat.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxv{O Mensch, was hat dich gegen deinen edelmütigen Herrn betört,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxvi{Der dich erschaffen und gebildet und zurechtgeformt hat,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxvii{Und dich in der Gestalt, die Er wollte, zusammengefügt hat?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxviii{Nein, ihr erklärt lieber das Gericht für Lüge.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxix{Über euch sind Hüter eingesetzt,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxl{Vortreffliche, die (alles) aufschreiben,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxli{Und die wissen, was ihr tut.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlii{Die Frommen leben in Wonne.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxliii{Und diejenigen, die voller Laster sind, leben in einer Hölle,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxliv{In der sie am Tag des Gerichtes brennen,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlv{Und der sie nicht entkommen können.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlvi{Woher sollst du wissen, was der Tag des Gerichtes ist?\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlvii{Noch einmal: Woher sollst du wissen, was der Tag des Gerichtes ist?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlviii{Am Tag, da niemand für einen anderen etwas bewirken kann. Und die Angelegenheit steht an jenem Tag bei Gott (allein).\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlix{\basmalahde Wehe denen, die das Maß verkürzen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcccl{Die, wenn sie sich von den Menschen zumessen lassen, volles Maß verlangen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcccli{Wenn sie ihnen aber zumessen oder abwägen, weniger geben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccclii{Rechnen diese nicht damit, daß sie auferweckt werden\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcccliii{Zu einem gewaltigen Tag,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccliv{Am Tag, da die Menschen sich vor dem Herrn der Welten hinstellen werden?\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdccclv{Nein, das Buch derer, die voller Laster sind, befindet sich im tiefen Kerker (Sidjjin).\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdccclvi{Und woher sollst du wissen, was der tiefe Kerker ist?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdccclvii{Es ist ein deutlich geschriebenes Buch.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdccclviii{Wehe an jenem Tag denen, die für Lüge erklären,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdccclix{Die den Tag des Gerichtes für Lüge erklären!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdccclx{Und nur der erklärt ihn für Lüge, der Übertretungen begeht und Schuld auf sich lädt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxi{Wenn ihm unsere Zeichen verlesen werden, sagt er: «Die Fabeln der Früheren.»\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxii{Nein, aber das, was sie zu erwerben pflegten, hat sich (wie Rost) über ihre Herzen gelegt.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxiii{Nein, sie werden an jenem Tag nicht zu ihrem Herrn zugelassen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxiv{Dann werden sie in der Hölle brennen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxv{Dann wird gesprochen: «Das ist das, was ihr immer wieder für Lüge erklärt habt.»\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxvi{Nein, das Buch der Frommen befindet sich in der Hohen Stätte.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxvii{Und woher sollst du wissen, was die Hohe Stätte ist?\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxviii{Es ist ein deutlich geschriebenes Buch,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxix{Das diejenigen schauen, die in die Nähe (Gottes) zugelassen sind.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxx{Die Frommen leben in Wonne,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxi{Auf Liegen (gelehnt), und halten Ausschau.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxii{Du erkennst auf ihren Gesichtern das strahlende Glück der Wonne.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxiii{Ihnen wird ein versiegeltes reines Getränk zu trinken gegeben,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxiv{Dessen Siegel aus Moschus ist - darum sollen die Wettkämpfer um die Wette streiten -,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxv{Und mit Tasniem beigemischt,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxvi{Aus einer Quelle, aus der diejenigen trinken, die in die Nähe (Gottes) zugelassen sind.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxvii{Die Übeltäter lachten über die, die glauben.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxviii{Und, wenn sie sie im Vorbeigehen trafen, zwinkerten sie einander zu.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxix{Und, wenn sie zu ihren Angehörigen zurückkehrten, kehrten sie mit Wohlbehagen zurück.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxx{Und, wenn sie sie sahen, sagten sie: «Diese da gehen in die Irre.»\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxi{Dabei waren sie doch nicht als Hüter über sie gesandt.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxii{Heute lachen die, die glauben, über die Ungläubigen,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxiii{Auf Liegen (gelehnt), und halten Ausschau.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxiv{Sind die Ungläubigen wohl entlohnt worden für das, was sie zu tun pflegten?\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxv{\basmalahde Wenn der Himmel sich spaltet\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxvi{Und auf seinen Herrn hört - und es obliegt ihm ja -,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxvii{Und wenn die Erde ausgebreitet wird\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxviii{Und auswirft, was in ihr ist, und somit ganz leer wird\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxix{Und auf ihren Herrn hört - und es obliegt ihr ja...\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxc{O Mensch, du strebst mühevoll deinem Herrn zu, und du wirst Ihm begegnen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxci{Wem nun sein Buch in seine Rechte gereicht wird,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcii{Für den wird eine leichte Abrechnung erfolgen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxciii{Und er wird froh zu seinen Angehörigen zurückkehren.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxciv{Wem aber sein Buch hinter seinem Rücken gereicht wird,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcv{Der wird rufen: «Was für ein Verderben!»,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcvi{Und er wird im Feuerbrand brennen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcvii{Er war froh inmitten seiner Angehörigen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcviii{Er meinte, er würde nicht zurückkehren.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcix{Doch, sein Herr sieht ihn ja wohl.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcm{Nein, Ich schwöre bei der Abenddämmerung\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmi{Und der Nacht und dem, was sie zusammentreibt,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmii{Und dem Mond, wenn er voll wird.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmiii{Ihr werdet eine Etappe nach der anderen durchlaufen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmiv{Was ist mit ihnen, daß sie nicht glauben\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcmv{Und, wenn ihnen der Koran verlesen wird, sich nicht niederwerfen?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcmvi{Nein, diejenigen, die ungläubig sind, erklären (ihn) für Lüge.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcmvii{Und Gott weiß besser, was sie (in ihrem. Herzen) bergen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcmviii{So verkünde ihnen eine schmerzhafte Pein,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcmix{Außer denen, die glauben und die guten Werke tun: Sie empfangen einen Lohn, der nicht aufhört.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcmx{\basmalahde Bei dem Himmel mit den Sternzeichen\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmxi{Und (bei) dem angedrohten Tag,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmxii{Und dem Zeugen und dem, was bezeugt ist!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmxiii{Dem Tod geweiht seien die Leute des Grabens,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmxiv{Des Feuers mit seinem Brennstoff,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmxv{Wenn sie daran sitzen\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmxvi{Und bezeugen, was sie den Gläubigen angetan haben.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmxvii{Und nichts anderes läßt sie ihnen grollen, als daß sie an Gott glauben, den Mächtigen, der des Lobes würdig ist,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmxviii{Dem die Königsherrschaft der Himmel und der Erde gehört. Und Gott ist Zeuge über alle Dinge.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmxix{Für diejenigen, die die gläubigen Männer und die gläubigen Frauen in Versuchung bringen und danach nicht umkehren, ist die Pein der Hölle bestimmt, ja, für sie ist die Pein des Feuerbrandes bestimmt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmxx{Für die aber, die glauben und die guten Werke tun, sind Gärten bestimmt, unter denen Bäche fließen. Das ist der große Erfolg.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxi{Der gewaltige Zugriff deines Herrn ist hart.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxii{Er macht (die Schöpfung) am Anfang, und Er wiederholt (sie).\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxiii{Er ist der, der viel vergibt und liebevoll ist,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxiv{Der glorreiche Herr des Thrones.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxv{Er tut mit Entschiedenheit, was Er will.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxvi{Ist die Geschichte von den Heerscharen zu dir gelangt,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxvii{Von Pharao und den Thamud?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxviii{Nein, diejenigen, die ungläubig sind, beharren darauf, (die Botschaft) für Lüge zu erklären.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxix{Und Gott umschließt sie von hinten her.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxx{Nein, es ist ein glorreicher Koran\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxi{Auf einer wohlverwahrten Tafel.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxii{\basmalahde Bei dem Himmel und dem Nachtstern!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxiii{Und woher sollst du wissen, was der Nachtstern ist?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxiv{Es ist) der leuchtende Stern.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxv{Es gibt niemanden, über den nicht ein Hüter eingesetzt ist.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxvi{Der Mensch soll doch betrachten, aus was für einem Stoff er erschaffen ist.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxvii{Er ist aus einem sich ergießenden Wasser erschaffen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxviii{Das zwischen Lende und Rippen herauskommt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxix{Er hat die Macht, ihn zurückzubringen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmxl{Am Tag, da die inneren Geheimnisse geprüft werden.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmxli{Dann hat er keine Kraft und keinen Helfer.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlii{Beim Himmel, mit der Wiederkehr (seiner Erscheinungen)\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmxliii{Und (bei) der Erde mit ihrem Sprießen!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmxliv{Er ist eine entscheidende Botschaft.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlv{Er ist nicht ein Scherz.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlvi{Sie führen eine List aus,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlvii{Und Ich führe eine List aus.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlviii{So gewähre den Ungläubigen noch eine Frist. Gewähre ihnen noch eine kurze Frist.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlix{\basmalahde Preise den Namen deines Herrn, des Allerhöchsten,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcml{Der erschafft und zurechtformt,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmli{Und der das Maß setzt und rechtleitet,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmlii{Und der das Weidegras hervorbringt\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmliii{Und es dann zu grauer Spreu macht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmliv{Wir werden dich lesen lassen, und du wirst nichts vergessen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmlv{Außer dem, was Gott will. Er weiß, was offenliegt und was verborgen bleibt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmlvi{Und Wir werden dir das Gute leicht machen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmlvii{Ermahne nun, so die Ermahnung etwas nützt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmlviii{Der wird es bedenken, der gottesfürchtig ist.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmlix{Meiden wird es aber der Übelste,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmlx{Der im großen Feuer brennen wird;\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxi{Darin wird er dann weder sterben noch leben.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxii{Es wird dem wohl ergehen, der sich läutert,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxiii{Des Namens seines Herrn gedenkt und betet.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxiv{Aber ihr zieht das diesseitige Leben vor,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxv{Wo doch das Jenseits besser ist und eher Bestand hat.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxvi{Dies steht in den früheren Blättern,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxvii{Den Blättern von Abraham und Mose.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxviii{\basmalahde Ist die Geschichte von der alles bedeckenden Katastrophe zu dir. gelangt?\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxix{An jenem Tag wird es gesenkte Gesichter geben,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxx{Die sich abarbeiten und Mühsal erleiden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxi{In einem glühenden Feuer. brennen\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxii{Und aus einer siedenden Quelle zu trinken bekommen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxiii{Ihre Speise besteht nur aus trockenen Dornen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxiv{Die weder fett machen noch gegen den Hunger helfen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxv{An jenem Tag wird es (auch) fröhliche Gesichter geben,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxvi{Die mit ihrem Mühen zufrieden sind\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxvii{Und in einem hochgelegenen Garten sind,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxviii{In dem sie keine unbedachten Reden hören.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxix{Es gibt darin eine fließende Quelle,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxx{Und es gibt darin erhöhte Betten,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxi{Hingestellte Humpen,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxii{In Reihen gelegte Kissen\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxiii{Und ausgebreitete Teppiche.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxiv{Schauen sie denn nicht zu den Kamelen, wie sie erschaffen wurden,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxv{Und zum Himmel, wie er emporgehoben wurde,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxvi{Und zu den Bergen, wie sie aufgerichtet wurden,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxvii{Und zu der Erde, wie sie als Fläche gemacht wurde?\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxviii{So ermahne. Du bist ja ein Mahner.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxix{Du hast sie nicht fest in der Hand.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcmxc{Wer sich aber abkehrt und ungläubig bleibt,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcmxci{Den peinigt Gott mit der größten Pein.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcii{Zu Uns erfolgt ihre Rückkehr,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcmxciii{Und dann obliegt es Uns, mit ihnen abzurechnen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcmxciv{\basmalahde Bei der Morgenröte\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcv{Und den zehn Nächten,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcvi{Und der geraden und der ungeraden (Zahl),\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcvii{Und der Nacht, wenn sie dahingeht!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcviii{Ist darin wohl ein (ausreichender) Schwur für den, der Verstand hat?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcix{Hast du nicht gesehen, wie dein Herr an den Aad gehandelt hat,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmm{der Stadt) Iram, berühmt für ihre Säulen,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmi{Dergleichen nicht erschaffen wurde in den Ländern?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmii{Und an den Thamud, die im Tal die Felsen aushauten?\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmiii{Und an Pharao, dem Besitzer der Pfähle?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmiv{Sie zeigten im Land ein Übermaß an Frevel\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmv{Und stifteten darin viel Unheil.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmvi{Da ließ dein Herr auf sie die Geißel einer (schlimmen) Pein fallen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmvii{Dein Herr liegt auf der Lauer.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmviii{Aber der Mensch, wenn sein Herr ihn prüft und großzügig behandelt und ihm ein angenehmes Leben schenkt, sagt: «Mein Herr behandelt mich großzügig.»\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmix{Wenn Er ihn aber prüft und ihm seinen Lebensunterhalt bemessen zuteilt, sagt er: «Mein Herr läßt mich Schmach erleiden.»\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmx{Aber nein, ihr behandelt die Waise nicht großzügig,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmxi{Und ihr haltet nicht zur Speisung des Bedürftigen an.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmxii{Ihr verzehrt das Erbe ohne Unterschied.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmxiii{Und ihr hegt für den Besitz eine allzu große Liebe.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmmxiv{Nein, wenn die Erde ganz zu Staub gemacht wird\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmmxv{Und dein Herr kommt, und die Engel in Reihen, die eine hinter der anderen,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmmxvi{Und die Hölle an jenem Tag herbeigebracht wird, an jenem Tag wird der Mensch es bedenken. Was soll ihm aber dann das Bedenken?\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmmxvii{Er sagt: «O hätte ich doch für mein Leben (hier) etwas vorausgeschickt!»\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmmxviii{Niemand kann an jenem Tag peinigen, so wie Er peinigt,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmmxix{Und niemand kann fesseln, so wie Er fesselt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmmxx{- O du Seele, die du Ruhe gefunden hast,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmmxxi{Kehre zu deinem Herrn zufrieden und von seinem Wohlgefallen begleitet zurück.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmmxxii{Tritt in die Reihen meiner Diener ein,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmmxxiii{Und tritt ein in mein Paradies.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmmxxiv{\basmalahde Nein, Ich schwöre bei diesem Gebiet,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmxxv{Wo du dich in diesem Gebiet aufhältst,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmxxvi{Und bei jedem Vater und dem, was er zeugt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmxxvii{Wir haben den Menschen zur Mühsal erschaffen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmxxviii{Meint er denn, daß niemand ihn überwältigen kann?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmxxix{Er sagt: «Ich habe ein großes Vermögen verbraucht.»\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmxxx{Meint er, daß niemand ihn gesehen hat?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxi{Haben Wir ihm nicht zwei Augen gemacht,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxii{Eine Zunge und zwei Lippen,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxiii{Und ihm beide Wege gewiesen?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxiv{Würde er doch den steilen Weg hinaufstürmen!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxv{Woher sollst du wissen, was der steile Weg ist?\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxvi{Es ist) die Befreiung eines Sklaven\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxvii{Oder, am Tag der Hungersnot, die Speisung\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxviii{Einer verwandten Waise\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxix{Oder eines Bedürftigen, der im Staub liegt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmxl{Und daß man außerdem zu denen gehört, die glauben, einander die Geduld nahelegen und einander die Barmherzigkeit nahelegen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmxli{Das sind die von der rechten Seite.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmxlii{Die aber, die unsere Zeichen verleugnen, sind die von der unglückseligen Seite.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmxliii{Über ihnen liegt ein Feuer zugeschlagen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmmxliv{\basmalahde Bei der Sonne und ihrem Morgenlicht,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmxlv{Und dem Mond, wenn er ihr folgt,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmxlvi{Und dem Tag, wenn er sie hell scheinen läßt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmxlvii{Und der Nacht, wenn sie sie bedeckt,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmxlviii{Und dem Himmel und dem, was ihn aufbaut,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmxlix{Und der Erde und dem, was sie ausbreitet,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmml{Und der Seele und dem, was sie zurechtformt\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmli{Und ihr ihre Lasterhaftigkeit und ihre Frömmigkeit eingibt!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmlii{Dem wird wohl ergehen, der sie läutert,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmliii{Und der wird enttäuscht sein, der sie mit Missetaten überdeckt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmliv{Die Thamud haben im Übermaß ihres Frevels (die Botschaft) für Lüge erklärt,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmlv{Als sich der Übelste unter ihnen erhob.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmlvi{Der Gesandte Gottes sagte zu ihnen: «Gebt acht auf die Kamelstute Gottes und ihre Trinkzeit.»\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmlvii{Sie aber ziehen ihn der Lüge und schnitten ihr die Flechsen durch und stachen sie. Da überzog sie ihr Herr für ihre Schuld mit Verderben und machte sie dem Erdboden gleich,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmlviii{Ohne die Folgen davon fürchten zu müssen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmlix{\basmalahde Bei der Nacht, wenn sie (alles) bedeckt,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmlx{Und dem Tag, wenn er hell scheint,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmlxi{Und dem, was den Mann und das Weib erschaffen hat!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmlxii{Euer Streben ist wahrlich verschieden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmlxiii{Wer nun gibt und gottesfürchtig ist\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmlxiv{Und das Allerbeste für wahr hält,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmlxv{Dem werden Wir das Gute leicht machen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmlxvi{Wer aber geizig ist und sich verhält, als wäre er auf niemanden angewiesen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmlxvii{Und das Allerbeste für Lüge erklärt,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmlxviii{Dem werden Wir die Drangsal leicht machen,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmlxix{Und sein Vermögen wird ihm nicht nützen, wenn er zugrunde geht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmlxx{Uns obliegt die Rechtleitung.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxi{Und Uns gehört das Jenseits und das Diesseits.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxii{Ich warne euch vor einem lodernden Feuer,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxiii{In dem nur der Übelste brennt,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxiv{Der (die Botschaft) für Lüge erklärt und sich abkehrt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxv{Ferngehalten davon wird der Gottesfürchtigste werden,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxvi{Der sein Vermögen hergibt, um sich zu läutern,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxvii{Nicht daß jemand bei ihm einen Gunsterweis guthätte, der vergolten werden müßte,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxviii{Sondern in der Suche nach dem Antlitz seines Herrn, des Allerhöchsten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxix{Und wahrlich, er wird zufrieden sein.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxx{\basmalahde Bei dem Morgen\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxi{Und (bei) der Nacht, wenn sie still ist!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxii{Dein Herr hat dir nicht den Abschied gegeben und haßt (dich) nicht.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxiii{Wahrlich, das Jenseits ist besser für dich als das Diesseits.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxiv{Und wahrlich, dein Herr wird dir geben, und du wirst zufrieden sein.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxv{Hat Er dich nicht als Waise gefunden und dir Unterkunft besorgt,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxvi{Und dich abgeirrt gefunden und rechtgeleitet,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxvii{Und bedürftig gefunden und reich gemacht?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxviii{So unterdrücke die Waise nicht,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxix{Und fahre den Bettler nicht an,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmxc{Und erzähle von der Gnade deines Herrn.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmxci{\basmalahde Haben Wir dir nicht deine Brust geweitet\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmxcii{Und dir deine Last abgenommen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmxciii{Die deinen Rücken schwer erdrückte,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmxciv{Und dir deinen Ruf erhöht?\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmxcv{Wahrlich, mit der Erschwernis gibt es eine Erleichterung,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmxcvi{Ja, mit der Erschwernis gibt es eine Erleichterung.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmxcvii{Wenn du fertig bist, dann mühe dich ab\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmxcviii{Und richte deine Wünsche auf deinen Herrn aus.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmxcix{\basmalahde Bei dem Feigenbaum und dem Olivenbaum,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmc{Und dem Berg Sinai,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmci{Und diesem sicheren Gebiet!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcii{Wir haben den Menschen in schönster ebenmäßiger Gestalt erschaffen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmciii{Dann haben Wir ihn in den niedrigsten der niedrigen Stände gebracht,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmciv{Ausgenommen die, die glauben und die guten Werke tun: Sie empfangen einen Lohn, der nicht aufhört.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcv{Was läßt dich da noch das Gericht für Lüge erklären?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcvi{Ist nicht Gott der Weiseste all derer, die urteilen?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcvii{\basmalahde Lies im Namen deines Herrn, der erschaffen hat,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcviii{Den Menschen erschaffen hat aus einem Embryo.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcix{Lies. Dein Herr ist der Edelmütigste,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcx{Der durch das Schreibrohr gelehrt hat,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxi{Den Menschen gelehrt hat, was er nicht wußte.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxii{Nein, der Mensch zeigt ein Übermaß an Frevel,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcxiii{Daß er meint, er wäre auf niemanden angewiesen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcxiv{Zu deinem Herrn erfolgt die Rückkehr.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcxv{Hast du den gesehen, der da wehrt\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmcxvi{Einem Diener, wenn er betet?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmcxvii{Was meinst du? Ob er da der Rechtleitung folgt,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmcxviii{Oder ob er die Gottesfurcht gebietet?\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmcxix{Was meinst du? Ob er wohl (die Botschaft) für Lüge erklärt und sich abkehrt?\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmcxx{Weiß er denn nicht, daß Gott (alles) sieht?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxi{Nein, wenn er nicht aufhört, werden Wir ihn gewiß am Schopf packen und ziehen,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxii{Einem lügnerischen, sündigen Schopf.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxiii{Er soll doch da seine Mitstreiter herbeirufen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxiv{Wir werden die Schergen herbeirufen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxv{Nein, gehorche ihm nicht. Wirf dich vielmehr nieder und suche die Nähe (Gottes).\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxvi{\basmalahde Wir haben ihn in der Nacht der Bestimmung hinabgesandt.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxvii{Woher sollst du wissen, was die Nacht der Bestimmung ist?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxviii{Die Nacht der Bestimmung ist besser als tausend Monate.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxix{Die Engel und der Geist kommen in ihr mit der Erlaubnis ihres Herrn herab mit jedem Anliegen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxx{Voller Frieden ist sie bis zum Aufgang der Morgenröte.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxi{\basmalahde Diejenigen von den Leuten des Buches und den Polytheisten, die ungläubig sind, werden nicht aufhören, bis das deutliche Zeichen zu ihnen kommt,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxii{Ein Gesandter von Gott, der gereinigte Blätter verliest,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxiii{In denen Schriften mit richtigem Inhalt stehen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxiv{Und diejenigen, denen das Buch zugekommen ist, haben sich erst gespalten, nachdem das deutliche Zeichen zu ihnen gekommen ist.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxv{Es wurde ihnen jedoch nur befohlen, Gott zu dienen und dabei Ihm gegenüber aufrichtig in der Religion zu sein, als Anhänger des reinen Glaubens, und das Gebet zu verrichten und die Abgabe zu entrichten. Das ist die richtige Religion.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxvi{Diejenigen von den Leuten des Buches und den Polytheisten, die nicht glauben, werden im Feuer der Hölle sein; darin werden sie ewig weilen. Sie sind die schlimmsten unter den Geschöpfen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxvii{Die aber, die glauben und die guten Werke tun, sind die besten unter den Geschöpfen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxviii{Ihr Lohn bei ihrem Herrn sind die Gärten von Eden, unter denen Bäche fließen; darin werden sie auf immer ewig weilen. Gott hat Wohlgefallen an ihnen, und sie haben Wohlgefallen an Ihm. Das ist für den bestimmt, der seinen Herrn fürchtet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxix{\basmalahde Wenn die Erde durch ihr heftiges Beben erschüttert wird,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxl{Und die Erde ihre schweren Lasten hervorbringt,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxli{Und der Mensch sagt: «Was ist mit ihr?»,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlii{An jenem Tag erzählt sie ihre Nachrichten,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxliii{Weil dein Herr (es) ihr offenbart hat.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxliv{An jenem Tag kommen die Menschen in verschiedenen Gruppen hervor, damit ihnen ihre Werke gezeigt werden.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlv{Wer nun Gutes im Gewicht eines Stäubchens tut, wird es sehen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlvi{Und wer Böses im Gewicht eines Stäubchens tut, wird es sehen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlvii{\basmalahde Bei denen, die schnaubend laufen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlviii{Und die Funken stieben lassen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlix{Und die am Morgen stürmen\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcl{Und damit Staub aufwirbeln,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcli{Und dadurch in die Mitte (der Feinde) eindringen!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclii{Wahrlich, der Mensch ist seinem Herrn gegenüber undankbar,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcliii{Und er ist selbst darüber Zeuge.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcliv{Und er ist heftig in seiner Liebe zu den (irdischen) Gütern.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclv{Weiß er es denn nicht? Wenn das, was in den Gräbern ist, aufgewühlt wird\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmclvi{Und das, was im Inneren (der Menschen) ist, im Ergebnis erfaßt wird,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmclvii{An jenem Tag hat ihr Herr Kenntnis von ihnen (allen).\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmclviii{\basmalahde Die Katastrophe!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclix{Was ist die Katastrophe?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclx{Und woher sollst du wissen, was die Katastrophe ist?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxi{Am Tag, da die Menschen wie verstreute Motten sein werden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmclxii{Und die Berge wie zerflockte bunte Wolle.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclxiii{Wer dann schwere Waagschalen hat,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmclxiv{Der wird ein zufriedenes Leben haben.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmclxv{Und wer leichte Waagschalen hat,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclxvi{Der wird zur Mutter einen Abgrund haben.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmclxvii{Und woher sollst du wissen, was das ist?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmclxviii{Ein glühendes Feuer.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmclxix{\basmalahde Der Wettstreit um noch mehr lenkt euch ab,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclxx{Daß ihr (sogar) die Gräber besucht.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxi{Nein, ihr werdet es noch zu wissen bekommen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxii{Noch einmal: Nein, ihr werdet es noch zu wissen bekommen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxiii{Nein, wenn ihr es nur mit Gewißheit wüßtet!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxiv{Ihr werdet bestimmt die Hölle sehen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxv{Noch einmal: Ihr werdet sie mit völliger Gewißheit sehen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxvi{Dann werdet ihr an jenem Tag euer angenehmes Leben zu verantworten haben.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxvii{\basmalahde Beim Nachmittag!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxviii{Der Mensch erleidet bestimmt Verlust,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxix{Außer denjenigen, die glauben und die guten Werke tun, und einander die Wahrheit nahelegen und die Geduld nahelegen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxx{\basmalahde Wehe jedem Stichler und Nörgler,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxi{Der sein Vermögen zusammenbringt und es zählt\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxii{Und dabei meint, sein Vermögen würde ihn unsterblich machen!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxiii{Nein, er wird bestimmt in die Zermalmende geworfen werden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxiv{Woher sollst du wissen, was die Zermalmende ist?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxv{Es ist das angefachte Feuer Gottes,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxvi{Das die Herzen durchdringt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxvii{Es liegt über ihnen zugeschlagen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxviii{In langgestreckten Säulen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxix{\basmalahde Hast du nicht gesehen, wie dein Herr an den Leuten des Elefanten gehandelt hat?\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxc{Hat Er nicht ihre List ins Leere gehen lassen\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxci{Und Vögel in Schwärmen über sie gesandt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcii{Die sie mit Steinen aus übereinandergeschichtetem Ton bewarfen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxciii{Und sie somit gleich abgefressenen Halmen gemacht?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxciv{\basmalahde Dafür, daß Er die Quraysh zusammenbringt,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcv{Daß Er sie zusammenbringt für die Reise des Winters und des Sommers,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcvi{Sollen sie dem Herrn dieses Hauses dienen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcvii{Der ihnen Speise gegen den Hunger gibt und Sicherheit gegen die Furcht gewährt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcviii{\basmalahde Hast du den gesehen, der das Gericht für Lüge erklärt?\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcix{Das ist der, der die Waise zurückstößt\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcc{Und nicht zur Speisung des Bedürftigen anhält.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcci{Wehe den Betenden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccii{Die auf ihr Gebet nicht achtgeben,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcciii{Die nur gesehen werden wollen\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcciv{Und die Hilfeleistung verwehren!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmccv{\basmalahde Wir haben dir die Fülle gegeben.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccvi{So bete zu deinem Herrn und schächte (Opfertiere).\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccvii{Der dich haßt, der soll ohne Anhang sein.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccviii{\basmalahde Sprich: O ihr Ungläubigen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccix{Ich verehre nicht, was ihr verehrt,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccx{Auch ihr verehrt nicht, was ich verehre.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxi{Weder ich werde verehren, was ihr verehrt habt,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxii{Noch werdet ihr verehren, was ich verehre.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxiii{Ihr habt eure Religion, und ich habe meine Religion.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmccxiv{\basmalahde Wenn die Unterstützung Gottes kommt, und auch der Erfolg,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxv{Und du die Menschen in Scharen in die Religion Gottes eintreten siehst,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxvi{Dann sing das Lob deines Herrn und bitte Ihn um Vergebung. Siehe, Er wendet sich gnädig wieder zu.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxvii{\basmalahde Dem Verderben geweiht seien die Hände des Abu Lahab, und dem Verderben geweiht sei er!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxviii{Nicht nützt ihm sein Vermögen und das, was er erworben hat.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxix{Er wird in einem lodernden Feuer brennen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxx{Und auch seine Frau, sie, die Holzträgerin.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxi{An ihrem Hals hängt ein Strick aus Palmenfasern.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxii{\basmalahde Sprich: Er ist Gott, ein Einziger,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxiii{Gott, der Undurchdringliche.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxiv{Er hat nicht gezeugt, und Er ist nicht gezeugt worden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxv{Und niemand ist Ihm ebenbürtig.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxvi{\basmalahde Sprich: Ich suche Zuflucht beim Herrn des Frühlichtes\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxvii{Vor dem Unheil dessen, was Er erschaffen hat,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxviii{Und vor dem Unheil der Finsternis, wenn sie einsetzt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxix{Und vor dem Unheil der (Hexen), die auf Knoten blasen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxx{Und vor dem Unheil eines Neiders, wenn er neidisch ist.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxi{\basmalahde Sprich: Ich suche Zuflucht beim Herrn der Menschen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxii{Dem König der Menschen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxiii{Dem Gott der Menschen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxiv{Vor dem Unheil des Einflüsterers, des Heimtückischen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxv{Der da in die Brust der Menschen einflüstert,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxvi{Sei es einer von den Djinn oder von den Menschen.\qt@no{6}}%
%% End of file `qurantext-deiii.translation.def'.
