\qt@newcmd\qurantrans@de@i{Im Namen Allahs, des Allerbarmers, des Barmherzigen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@ii{Alles) Lob gehört Allah, dem Herrn der Welten,\qt@no{2}}%
\qt@newcmd\qurantrans@de@iii{dem Allerbarmer, dem Barmherzigen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@iv{dem Herrscher am Tag des Gerichts.\qt@no{4}}%
\qt@newcmd\qurantrans@de@v{Dir allein dienen wir, und zu Dir allein flehen wir um Hilfe.\qt@no{5}}%
\qt@newcmd\qurantrans@de@vi{Leite uns den geraden Weg,\qt@no{6}}%
\qt@newcmd\qurantrans@de@vii{den Weg derjenigen, denen Du Gunst erwiesen hast, nicht derjenigen, die (Deinen) Zorn erregt haben, und nicht der Irregehenden!\qt@no{7}}%
\qt@newcmd\qurantrans@de@viii{\basmalahde Alif-Lam-Mim\qt@no{1}}%
\qt@newcmd\qurantrans@de@ix{Dieses Buch, an dem es keinen Zweifel gibt, ist eine Rechtleitung für die Gottesfürchtigen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@x{die an das Verborgene glauben, das Gebet verrichten und von dem, womit Wir sie versorgt haben, ausgeben\qt@no{3}}%
\qt@newcmd\qurantrans@de@xi{und die an das glauben, was zu dir (an Offenbarung) herabgesandt worden ist, und was vor dir herabgesandt wurde, und die vom Jenseits überzeugt sind.\qt@no{4}}%
\qt@newcmd\qurantrans@de@xii{Jene verfahren nach einer Rechtleitung von ihrem Herrn, und das sind diejenigen, denen es wohl ergeht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@xiii{Gleich ist es in Bezug auf diejenigen, die ungläubig sind, ob du sie warnst oder nicht warnst; sie glauben nicht.\qt@no{6}}%
\qt@newcmd\qurantrans@de@xiv{Allah hat ihre Herzen und ihr Gehör versiegelt, über ihrem Augenlicht befindet sich eine Hülle. Für sie wird es gewaltige Strafe geben.\qt@no{7}}%
\qt@newcmd\qurantrans@de@xv{Unter den Menschen gibt es manche, die sagen: ``Wir glauben an Allah und an den Jüngsten Tag", doch sind sie nicht gläubig.\qt@no{8}}%
\qt@newcmd\qurantrans@de@xvi{Sie möchten Allah und diejenigen, die glauben, betrügen. Aber sie betrügen nur sich selbst, ohne zu merken.\qt@no{9}}%
\qt@newcmd\qurantrans@de@xvii{In ihren Herzen ist Krankheit, und da hat Allah ihnen die Krankheit noch gemehrt. Für sie wird es schmerzhafte Strafe dafür geben, daß sie zu lügen pflegten.\qt@no{10}}%
\qt@newcmd\qurantrans@de@xviii{Und wenn man zu ihnen sagt: ``Stiftet nicht Unheil auf der Erde!" sagen sie: ``Wir sind ja nur Heilstifter".\qt@no{11}}%
\qt@newcmd\qurantrans@de@xix{Dabei sind doch eben sie die Unheilstifter, nur merken sie nicht.\qt@no{12}}%
\qt@newcmd\qurantrans@de@xx{Und wenn man zu ihnen sagt: ``Glaubt, wie die Menschen glauben!", so sagen sie: ``Sollen wir glauben, wie die Toren glauben?" Dabei sind doch eben sie die Toren. Aber sie wissen nicht.\qt@no{13}}%
\qt@newcmd\qurantrans@de@xxi{Und wenn sie diejenigen treffen, die glauben, sagen sie: ``Wir glauben." Wenn sie jedoch mit ihren Teufeln allein sind, so sagen sie: ``Wir stehen zu euch. Wir machen uns ja nur lustig".\qt@no{14}}%
\qt@newcmd\qurantrans@de@xxii{Allah ist es, Der Sich über sie lustig macht. Und Er läßt sie in ihrer Auflehnung umherirren.\qt@no{15}}%
\qt@newcmd\qurantrans@de@xxiii{Das sind diejenigen, die das Irregehen um die Rechtleitung erkauft haben, doch hat ihr Handel keinen Gewinn gebracht, und sie sind nicht rechtgeleitet.\qt@no{16}}%
\qt@newcmd\qurantrans@de@xxiv{Ihr Gleichnis ist das jemandes, der ein Feuer anzündet. Nachdem es seine Umgebung erhellt hat, nimmt Allah ihr Licht weg und läßt sie in Finsternis zurück; sie sehen nicht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@xxv{Taub, stumm und blind: So werden sie nicht umkehren.\qt@no{18}}%
\qt@newcmd\qurantrans@de@xxvi{Oder es ist wie ein Gewitterregen, der vom Himmel niedergeht, voller Finsternis, Donner und Blitz. Sie stecken sich die Finger in die Ohren vor den Donnerschlägen, um dem Tod zu entfliehen, doch Allah umfaßt die Ungläubigen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@xxvii{Der Blitz reißt ihnen beinahe das Augenlicht fort. Jedesmal, wenn er ihnen Helligkeit verbreitet, gehen sie darin. Und wenn es finster um sie wird, bleiben sie stehen. Wenn Allah wollte, nähme Er ihnen wahrlich Gehör und Augenlicht. Allah hat zu allem die Macht.\qt@no{20}}%
\qt@newcmd\qurantrans@de@xxviii{O ihr Menschen! Dient eurem Herrn, Der euch und diejenigen vor euch erschaffen hat, auf daß ihr gottesfürchtig werden möget!\qt@no{21}}%
\qt@newcmd\qurantrans@de@xxix{Der euch die Erde zu einem Ruhebett und den Himmel zu einem Gebäude gemacht hat und vom Himmel Wasser herabkommen läßt, durch das Er dann für euch Früchte als Versorgung hervorbringt. So stellt Allah nicht andere als Seinesgleichen zur Seite, wo ihr (es) doch (besser) wißt.\qt@no{22}}%
\qt@newcmd\qurantrans@de@xxx{Und wenn ihr im Zweifel über das seid, was Wir Unserem Diener offenbart haben, dann bringt doch eine Sura gleicher Art bei und ruft eure Zeugen außer Allah an, wenn ihr wahrhaftig seid!\qt@no{23}}%
\qt@newcmd\qurantrans@de@xxxi{Doch wenn ihr es nicht tut - und ihr werdet es nicht tun - dann hütet euch vor dem (Höllen)feuer, dessen Brennstoff Menschen und Steine sind. Es ist für die Ungläubigen bereitet.\qt@no{24}}%
\qt@newcmd\qurantrans@de@xxxii{Und verkünde denen, die glauben und rechtschaffene Werke tun, (die frohe Botschaft,) daß ihnen Gärten zuteil werden, durcheilt von Bächen. Jedesmal, wenn sie mit einer Frucht daraus versorgt werden, sagen sie: ``Das ist ja das, womit wir zuvor versorgt wurden"; doch es ist ihnen eine ihr ähnliche gegeben worden. Und darin haben sie vollkommen gereinigte Gattinnen. Und ewig werden sie darin bleiben.\qt@no{25}}%
\qt@newcmd\qurantrans@de@xxxiii{Allah schämt Sich nicht, ein Gleichnis auch nur mit einer Mücke oder mit etwas darüber (hinaus) zu prägen. Was nun diejenigen angeht, die glauben, so wissen sie, daß es die Wahrheit von ihrem Herrn ist. Was aber diejenigen angeht, die ungläubig sind, so sagen sie: ``Was will denn Allah damit als Gleichnis?" Er läßt damit viele in die Irre gehen und leitet viele damit recht, doch läßt Er damit nur die Frevler in die Irre gehen,\qt@no{26}}%
\qt@newcmd\qurantrans@de@xxxiv{die Allahs Bund nach seiner Abmachung brechen und trennen, was Allah befohlen hat, daß es verbunden werden soll, und auf der Erde Unheil stiften - das sind die Verlierer.\qt@no{27}}%
\qt@newcmd\qurantrans@de@xxxv{Wie könnt ihr Allah verleugnen, wo ihr doch tot wart und Er euch dann lebendig gemacht hat, euch hierauf sterben läßt und darauf wieder lebendig machen wird, worauf ihr zu Ihm zurückgebracht werdet?\qt@no{28}}%
\qt@newcmd\qurantrans@de@xxxvi{Er ist es, Der für euch alles, was auf der Erde ist, erschuf und Sich hierauf dem Himmel zuwandte und ihn dann zu sieben Himmeln formte. Er weiß über alles Bescheid.\qt@no{29}}%
\qt@newcmd\qurantrans@de@xxxvii{Und als dein Herr zu den Engeln sagte: ``Ich bin dabei, auf der Erde einen Statthalter einzusetzen", da sagten sie: ``Willst Du auf ihr etwa jemanden einsetzen, der auf ihr Unheil stiftet und Blut vergießt, wo wir Dich doch lobpreisen und Deiner Heiligkeit lobsingen?" Er sagte: ``Ich weiß, was ihr nicht wißt."\qt@no{30}}%
\qt@newcmd\qurantrans@de@xxxviii{Und Er lehrte Adam die Namen alle. Hierauf legte Er sie den Engeln vor und sagte: ``Teilt Mir deren Namen mit, wenn ihr wahrhaftig seid!"\qt@no{31}}%
\qt@newcmd\qurantrans@de@xxxix{Sie sagten: ``Preis sei Dir! Wir haben kein Wissen außer dem, was Du uns gelehrt hast. Du bist ja der Allwissende und Allweise."\qt@no{32}}%
\qt@newcmd\qurantrans@de@xl{Er sagte: ``O Adam, teile ihnen ihre Namen mit!" Als er ihnen ihre Namen mitgeteilt hatte, sagte Er: ``Habe Ich euch nicht gesagt, Ich kenne das Verborgene der Himmel und der Erde, und Ich weiß auch, was ihr offenlegt und was ihr verborgen zu halten sucht?"\qt@no{33}}%
\qt@newcmd\qurantrans@de@xli{Und als Wir zu den Engeln sagten: ``Werft euch vor Adam nieder!" Da warfen sie sich nieder, außer Iblis. Er weigerte sich und verhielt sich hochmütig und gehörte zu den Ungläubigen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@xlii{Und Wir sagten: ``O Adam, bewohne du und deine Gattin den (Paradies)garten, und eßt von ihm reichlich, wo immer ihr wollt! Aber naht euch nicht diesem Baum, sonst gehört ihr zu den Ungerechten!"\qt@no{35}}%
\qt@newcmd\qurantrans@de@xliii{Doch Satan entfernte sie davon, und da vertrieb er sie aus dem, worin sie (an Glückseligkeit) gewesen waren. Wir sagten: ``Geht fort! Einige von euch seien der anderen Feind. Und auf der Erde sollt ihr Aufenthalt und Nießbrauch auf Zeit haben."\qt@no{36}}%
\qt@newcmd\qurantrans@de@xliv{Da empfing Adam von seinem Herrn Worte, und darauf nahm Er seine Reue an. Er ist ja der Reue-Annehmende und Barmherzige.\qt@no{37}}%
\qt@newcmd\qurantrans@de@xlv{Wir sagten: Geht alle fort von ihm (, dem Paradiesgarten). Wenn nun von Mir Rechtleitung zu euch kommt, dann soll über diejenigen, die Meiner Rechtleitung folgen, keine Furcht kommen, noch sollen sie traurig sein.\qt@no{38}}%
\qt@newcmd\qurantrans@de@xlvi{Diejenigen aber, die ungläubig sind und Unsere Zeichen für Lüge erklären, das sind Insassen des (Höllen)feuers. Ewig werden sie darin bleiben.\qt@no{39}}%
\qt@newcmd\qurantrans@de@xlvii{O Kinder lsra'ils, gedenkt Meiner Gunst, die Ich euch erwiesen habe! Und haltet euren Bund Mir gegenüber, so will Ich Meinen Bund euch gegenüber halten! Und vor Mir (allein) sollt ihr Ehrfurcht haben.\qt@no{40}}%
\qt@newcmd\qurantrans@de@xlviii{Und glaubt an das, was Ich (als Offenbarung) hinabgesandt habe, das zu bestätigen, was euch bereits vorliegt. Und seid nicht die ersten, die es verleugnen. Und verkauft Meine Zeichen nicht für einen geringen Preis. Und Mich allein sollt ihr fürchten.\qt@no{41}}%
\qt@newcmd\qurantrans@de@xlix{Und verdeckt nicht das Wahre durch das Falsche, und verschweigt nicht die Wahrheit, wo ihr doch wißt!\qt@no{42}}%
\qt@newcmd\qurantrans@de@l{Und verrichtet das Gebet, entrichtet die Abgabe und verbeugt euch (im Gebet) mit den sich Verbeugenden!\qt@no{43}}%
\qt@newcmd\qurantrans@de@li{Befehlt ihr denn den Menschen Güte, während ihr euch selbst vergeßt, wo ihr doch die Schrift lest? Begreift ihr denn nicht?\qt@no{44}}%
\qt@newcmd\qurantrans@de@lii{Und sucht Hilfe in der Standhaftigkeit und im Gebet! Es ist freilich schwer, nur nicht für die Demütigen,\qt@no{45}}%
\qt@newcmd\qurantrans@de@liii{die daran glauben, daß sie ihrem Herrn begegnen werden, und daß sie zu Ihm zurückkehren.\qt@no{46}}%
\qt@newcmd\qurantrans@de@liv{O Kinder Israils, gedenkt Meiner Gunst, die Ich euch erwiesen habe, und daß Ich euch vor den (anderen) Weltenbewohnern bevorzugt habe.\qt@no{47}}%
\qt@newcmd\qurantrans@de@lv{Und hütet euch vor einem Tag, an dem keine Seele etwas anstelle einer anderen übernehmen kann und von keiner Fürsprache noch Ersatz(leistung) angenommen wird und ihnen keine Hilfe zuteil wird!\qt@no{48}}%
\qt@newcmd\qurantrans@de@lvi{Und (gedenkt,) als Wir euch von den Leuten Fir'auns erretteten, die euch eine böse Qual auferlegten, indem sie eure Söhne allesamt abschlachteten und (nur) eure Frauen am Leben ließen. Darin war für euch eine gewaltige Prüfung von eurem Herrn.\qt@no{49}}%
\qt@newcmd\qurantrans@de@lvii{Und als Wir für euch das Meer teilten und euch so retteten und die Leute Fir'auns ertrinken ließen, während ihr zuschautet!\qt@no{50}}%
\qt@newcmd\qurantrans@de@lviii{Und als Wir Uns mit Musa auf vierzig Nächte verabredeten, da nahmt ihr dann nach ihm das Kalb an, womit ihr Unrecht tatet.\qt@no{51}}%
\qt@newcmd\qurantrans@de@lix{Hierauf, nach alledem, verziehen Wir euch, auf daß ihr dankbar wäret.\qt@no{52}}%
\qt@newcmd\qurantrans@de@lx{Und als Wir Musa die Schrift und die Unterscheidung gaben, auf daß ihr rechtgeleitet werdet.\qt@no{53}}%
\qt@newcmd\qurantrans@de@lxi{Und als Musa zu seinem Volk sagte: ``O mein Volk, ihr habt euch selbst Unrecht zugefügt, indem ihr das Kalb (zum Gegenstand der Anbetung) genommen habt. Bereut nun vor eurem Erschaffer und tötet dann (die Schuldigen unter) euch selbst! Dies ist besser für euch vor eurem Erschaffer!" Und da nahm Er eure Reue an; Er ist ja der Reue-Annehmende und Barmherzige.\qt@no{54}}%
\qt@newcmd\qurantrans@de@lxii{Und als ihr sagtet: ``O Musa, wir werden dir nicht eher glauben, bis wir Allah unverhüllt sehen!" Da überkam euch der Donnerschlag, während ihr zuschautet.\qt@no{55}}%
\qt@newcmd\qurantrans@de@lxiii{Hierauf erweckten Wir euch nach eurem Tod, auf daß ihr dankbar wäret.\qt@no{56}}%
\qt@newcmd\qurantrans@de@lxiv{Und Wir ließen die Wolken euch überschatten und sandten das Manna und die Wachteln auf euch hinab: ``Eßt von den guten Dingen, mit denen Wir euch versorgt haben." Und sie fügten nicht Uns Unrecht zu, sondern sich selbst.\qt@no{57}}%
\qt@newcmd\qurantrans@de@lxv{Und als Wir sagten: ``Tretet ein in diese Stadt und dann eßt, wo immer ihr wollt, reichlich von (dem, was in) ihr (ist). Und tretet, euch niederwerfend, durch das Tor ein und sagt: ,Vergebung, so vergeben Wir euch eure Verfehlungen. Und Wir werden den Gutes Tuenden noch mehr erweisen."\qt@no{58}}%
\qt@newcmd\qurantrans@de@lxvi{Doch da tauschten diejenigen, die Unrecht taten, das Wort gegen ein anderes aus, das ihnen nicht gesagt worden war. Und da sandten Wir auf diejenigen, die Unrecht taten, eine unheilvolle Strafe vom Himmel hinab dafür, daß sie gefrevelt hatten.\qt@no{59}}%
\qt@newcmd\qurantrans@de@lxvii{Und als Musa für sein Volk um Wasser bat, da sagten Wir: ``Schlag mit deinem Stock auf den Felsen!" Da entsprangen ihm zwölf Quellen. Nun wußte jedermann, wo sein Platz zum Trinken war: ``Eßt und trinkt von Allahs Versorgung und richtet auf der Erde nicht unheilstiftend Verderben an!"\qt@no{60}}%
\qt@newcmd\qurantrans@de@lxviii{Und als ihr sagtet: ``O Musa, wir halten eine Speise allein nicht aus. Bitte doch für uns deinen Herrn, Er soll für uns etwas hervorbringen von dem, was die Erde wachsen läßt an Grünzeug, Gurken, Getreide, Linsen und Zwiebeln!" Er sagte: ``Wollt ihr das, was besser ist, eintauschen gegen das, was geringer (an Wert) ist? Geht fort in (irgendeine) Stadt! Dann werdet ihr bekommen, was ihr verlangt habt." Und es wurde ihnen Erniedrigung und Elend auferlegt, und sie zogen sich den Zorn von Allah zu. Dies, weil sie stets Allahs Zeichen verleugneten und die Propheten ohne Recht töteten; dies, weil sie sich widersetzten und stets übertraten.\qt@no{61}}%
\qt@newcmd\qurantrans@de@lxix{Gewiß, diejenigen, die glauben, und diejenigen, die dem Judentum angehören, und die Christen und die Säbier - wer immer an Allah und den Jüngsten Tag glaubt und rechtschaffen handelt, - die haben ihren Lohn bei ihrem Herrn, und keine Furcht soll sie überkommen, noch werden sie traurig sein.\qt@no{62}}%
\qt@newcmd\qurantrans@de@lxx{Und als Wir mit euch ein Abkommen trafen und den Berg über euch emporhoben (und zu euch sagten): ``Haltet fest an dem, was Wir euch gegeben haben, und gedenkt dessen, was es enthält, auf daß ihr gottesfürchtig werden möget!"\qt@no{63}}%
\qt@newcmd\qurantrans@de@lxxi{Dann kehrtet ihr euch nach alledem ab. Aber wenn nicht Allahs Huld gegen euch und Sein Erbarmen gewesen wären, gehörtet ihr wahrlich zu den Verlierern.\qt@no{64}}%
\qt@newcmd\qurantrans@de@lxxii{Und ihr kennt doch diejenigen von euch, die den Sabbat übertraten. Da sagten Wir zu ihnen: ``Werdet verstoßene Affen!"\qt@no{65}}%
\qt@newcmd\qurantrans@de@lxxiii{Und so machten Wir dies für alle mit und nach euch zu einem warnenden Beispiel und zu einer Ermahnung für die Gottesfürchtigen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@lxxiv{Und als Musa zu seinem Volk sagte: ``Allah befiehlt euch, daß ihr eine Kuh schlachten sollt!" Sie sagten: ``Machst du dich über uns lustig?" Er sagte: ``Ich nehme Zuflucht bei Allah (davor), daß ich zu den Toren gehöre!"\qt@no{67}}%
\qt@newcmd\qurantrans@de@lxxv{Sie sagten: ``Bitte für uns deinen Herrn, uns Klarheit zu geben, wie sie sein soll!" Er sagte: ``Er (Allah) sagt, es soll eine Kuh sein, die weder zu alt noch zu jung zum Kalben ist, sondern dazwischen, in mittlerem Alter. So tut nun, was euch befohlen wird!"\qt@no{68}}%
\qt@newcmd\qurantrans@de@lxxvi{Sie sagten. ``Bitte für uns deinen Herrn, uns Klarheit zu geben, welche Farbe sie haben soll!" Er sagte: ``Er (Allah) sagt, es soll eine gelbe Kuh sein, von lebhafter Farbe, die die Betrachter erfreut."\qt@no{69}}%
\qt@newcmd\qurantrans@de@lxxvii{Sie sagten: ``Bitte für uns deinen Herrn, uns Klarheit zu geben, wie sie sein soll! Die Kühe erscheinen uns (alle) ähnlich. Doch wenn Allah will, so werden wir wahrlich rechtgeleitet sein."\qt@no{70}}%
\qt@newcmd\qurantrans@de@lxxviii{Er sagte: ``Er (Allah) sagt, es soll eine Kuh sein, nicht fügsam gemacht durch Pflügen der Erde noch durch Bewässern von Saatfeld, fehlerfrei und makellos!" Sie sagten: ``Jetzt bist du mit der Wahrheit gekommen." So schlachteten sie sie, doch beinahe hätten sie es nicht getan.\qt@no{71}}%
\qt@newcmd\qurantrans@de@lxxix{Und (gedenkt,) als ihr jemanden getötet hattet und euch darüber strittet; und Allah wollte herausbringen, was ihr verborgen hieltet.\qt@no{72}}%
\qt@newcmd\qurantrans@de@lxxx{Da sagten Wir: ``Schlagt ihn mit einem Stück von ihr!" So macht Allah die Toten wieder lebendig und zeigt euch Seine Zeichen, auf daß ihr begreifen möget.\qt@no{73}}%
\qt@newcmd\qurantrans@de@lxxxi{Hierauf, nach alledem, verhärteten sich dann eure Herzen, so daß sie wie Steine waren oder noch härter. Und unter den Steinen gibt es wahrlich manche, denen Flüsse entspringen, und andere, die bersten, worauf Wasser aus ihnen herausfließt, und wieder andere, die herabstürzen aus Furcht vor Allah. Und Allah ist nicht unachtsam dessen, was ihr tut.\qt@no{74}}%
\qt@newcmd\qurantrans@de@lxxxii{Begehrt ihr (Muslime) denn, daß sie (die Juden) euch glauben, wo doch eine Gruppe von ihnen das Wort Allahs gehört und es dann, nachdem er es begriffen hatte, wissentlich verfälscht hat?\qt@no{75}}%
\qt@newcmd\qurantrans@de@lxxxiii{Und wenn sie diejenigen treffen, die glauben, sagen sie: ``Wir glauben." Wenn sie aber untereinander allein sind, sagen sie: ``Wollt ihr ihnen erzählen, was Allah euch (Juden) enthüllt hat, damit sie es vor eurem Herrn als Beweis vorlegen? Begreift ihr denn nicht?"\qt@no{76}}%
\qt@newcmd\qurantrans@de@lxxxiv{Wissen sie denn nicht, daß Allah weiß, was sie geheimhalten und was sie offen legen?\qt@no{77}}%
\qt@newcmd\qurantrans@de@lxxxv{Unter ihnen gibt es auch Schriftunkundige, die die Schrift nicht kennen, sondern nur Wunschvorstellungen hegen, und die doch nur Mutmaßungen anstellen.\qt@no{78}}%
\qt@newcmd\qurantrans@de@lxxxvi{Doch wehe denjenigen, die die Schrift mit ihren (eigenen) Händen schreiben und hierauf sagen: ``Das ist von Allah", um sie für einen geringen Preis zu verkaufen! Wehe ihnen wegen dessen, was ihre Hände geschrieben haben, und wehe ihnen wegen dessen, was sie verdienen.\qt@no{79}}%
\qt@newcmd\qurantrans@de@lxxxvii{Und sie sagen: ``Das (Höllen)feuer wird uns nur für gezählte Tage berühren." Sag: Habt ihr mit Allah einen Bund geschlossen? Dann wird Allah Seinen Bund nicht brechen. Oder wollt ihr von Allah etwas sagen, was ihr nicht wißt?\qt@no{80}}%
\qt@newcmd\qurantrans@de@lxxxviii{Aber nein! Wer eine böse Tat begangen hat, und wen seine Verfehlung umfangen hält, das sind Insassen des (Höllen)feuers. Ewig werden sie darin bleiben.\qt@no{81}}%
\qt@newcmd\qurantrans@de@lxxxix{Diejenigen aber, die glauben und rechtschaffene Werke tun, das sind Insassen des (Paradies)gartens. Ewig werden sie darin bleiben.\qt@no{82}}%
\qt@newcmd\qurantrans@de@xc{Und als Wir mit den Kindern Isra'ils ein Abkommen trafen: Dient keinem außer Allah! Und zu den Eltern sollt ihr gütig sein und zu den Verwandten, den Waisen und den Armen! Und sagt Gutes zu den Menschen, verrichtet das Gebet und entrichtet die Abgabe. Danach kehrtet ihr euch - bis auf wenige von euch - ab; ihr seid ja Widerstrebende.\qt@no{83}}%
\qt@newcmd\qurantrans@de@xci{Und als Wir mit euch ein Abkommen trafen: Vergießt nicht (gegenseitig) euer Blut und vertreibt euch nicht selbst aus euren Wohnstätten! Hierauf habt ihr euch dazu bekannt und ihr seid dafür Zeugen.\qt@no{84}}%
\qt@newcmd\qurantrans@de@xcii{Dennoch seid (gerade) ihr es, die ihr euch selbst (gegenseitig) tötet und eine Gruppe von euch aus seinen Wohnstätten vertreibt, indem ihr einander in Sünde und feindseligem Vorgehen gegen sie beisteht. Und wenn sie als Gefangene zu euch kommen, so kauft ihr sie los, wo euch doch ihre Vertreibung verboten worden ist. Glaubt ihr denn an einen Teil der Schrift und verleugnet einen anderen? Wer von euch aber solches tut, dessen Lohn ist nur Schande im diesseitigen Leben. Und am Tag der Auferstehung werden sie der schwersten Strafe zugeführt werden. Und Allah ist nicht unachtsam dessen, was ihr tut.\qt@no{85}}%
\qt@newcmd\qurantrans@de@xciii{Das sind diejenigen, die das diesseitige Leben für das Jenseits erkauft haben. Deshalb wird ihre Strafe nicht erleichtert werden, und ihnen wird keine Hilfe zuteil.\qt@no{86}}%
\qt@newcmd\qurantrans@de@xciv{Und Wir gaben bereits Musa die Schrift und ließen nach ihm die Gesandten folgen. Und Wir gaben 'Isa, dem Sohn Maryams, die klaren Beweise und stärkten ihn mit dem Heiligen Geist. War es nicht (so), daß jedesmal, wenn euch (Juden) ein Gesandter etwas überbrachte, was euren Neigungen nicht entsprach, ihr euch hochmütig verhieltet, indem ihr eine Gruppe (der Gesandten) der Lüge bezichtigtet und eine (andere) Gruppe tötetet?\qt@no{87}}%
\qt@newcmd\qurantrans@de@xcv{Und sie sagen: ``Unsere Herzen sind verhüllt." Nein! Vielmehr hat Allah sie für ihren Unglauben verflucht. Wie wenig sie glauben!\qt@no{88}}%
\qt@newcmd\qurantrans@de@xcvi{Und als von Allah ein Buch zu ihnen kam, das bestätigend, was ihnen bereits vorlag - und zuvor hatten sie um einen entscheidenden Sieg über diejenigen, die ungläubig waren, angerufen -, als nun das zu ihnen kam, was sie kannten, verleugneten sie es. Darum Allahs Fluch über die Ungläubigen!\qt@no{89}}%
\qt@newcmd\qurantrans@de@xcvii{Wie schlimm ist das, wofür sie ihre Seelen verkauft haben: daß sie das verleugnen, was Allah herabgesandt hat, aus Mißgunst (darüber), daß Allah etwas von Seiner Huld offenbart, wem von Seinen Dienern Er will! So haben sie sich Zorn über Zorn zugezogen. Und für die Ungläubigen gibt es schmachvolle Strafe.\qt@no{90}}%
\qt@newcmd\qurantrans@de@xcviii{Und wenn man zu ihnen sagt: ``Glaubt an das, was Allah (als Offenbarung) herabgesandt hat", sagen sie: ``Wir glauben an das, was zu uns (schon zuvor) herabgesandt worden ist", verleugnen aber das, was später (offenbart worden) ist, wo es doch die Wahrheit ist, das zu bestätigen, was ihnen (bereits) vorliegt. Sag: Warum habt ihr denn zuvor Allahs Propheten getötet, wenn ihr gläubig seid?\qt@no{91}}%
\qt@newcmd\qurantrans@de@xcix{Und Musa war doch zu euch gekommen mit den klaren Beweisen. Aber dann nahmt ihr das Kalb an nach ihm, womit ihr Unrecht tatet.\qt@no{92}}%
\qt@newcmd\qurantrans@de@c{Und als Wir mit euch ein Abkommen trafen und den Berg über euch emporhoben (und zu euch sagten): ``Haltet fest an dem, was Wir euch gegeben haben und hört." Sie sagten: ``Wir hören, doch wir widersetzen uns." Und es geschah ihnen durch ihren Unglauben, daß (die Liebe für) das Kalb in ihre Herzen eindrang. Sag: Wie schlimm ist das, was euch euer Glaube gebietet, wenn ihr gläubig seid!\qt@no{93}}%
\qt@newcmd\qurantrans@de@ci{Sag: Wenn die jenseitige Wohnstätte bei Allah euch (Juden) vorbehalten ist unter Ausschluß der (anderen) Menschen, dann wünscht (euch) den Tod, wenn ihr wahrhaftig seid!\qt@no{94}}%
\qt@newcmd\qurantrans@de@cii{Aber sie werden ihn sich niemals wünschen wegen dessen, was ihre Hände (an Taten) vorausgeschickt haben. Und Allah weiß über die Ungerechten Bescheid.\qt@no{95}}%
\qt@newcmd\qurantrans@de@ciii{Und ganz gewiß wirst du sie als die gierigsten Menschen nach Leben finden, sogar mehr (noch) als diejenigen, die (Allah etwas) beigesellen. (Manch) einer von ihnen möchte gern tausend Jahre am Leben bleiben, aber selbst daß er am Leben bliebe, würde ihn nicht das der Strafe entrücken. Allah sieht wohl, was sie tun.\qt@no{96}}%
\qt@newcmd\qurantrans@de@civ{Sag: Wer (auch immer) Gibril feind ist, so hat er ihn doch (den Qur'an) mit Allahs Erlaubnis in dein Herz offenbart, das zu bestätigen, was vor ihm (offenbart) war, und als Rechtleitung und frohe Botschaft für die Gläubigen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@cv{Wer Allah und Seinen Engeln und Seinen Gesandten und Gibril und Mikal feind ist, so ist Allah den Ungläubigen feind.\qt@no{98}}%
\qt@newcmd\qurantrans@de@cvi{Und Wir haben zu dir ja (im Qur'an) klare Zeichen hinabgesandt. Doch nur die Frevler verleugnen sie.\qt@no{99}}%
\qt@newcmd\qurantrans@de@cvii{Ist es nicht (so), daß jedesmal, wenn sie eine Verpflichtung eingingen, sie eine Gruppe von ihnen verworfen hat? - Nein! Vielmehr glauben die meisten von ihnen (doch) nicht.\qt@no{100}}%
\qt@newcmd\qurantrans@de@cviii{Und als (nun) zu ihnen ein Gesandter von Allah gekommen ist, das bestätigend, was ihnen (an Offenbarungen) bereits vorlag, da hat eine Gruppe von denjenigen, denen die Schrift gegeben wurde, das Buch Allahs hinter ihren Rücken geworfen, als ob sie nicht Bescheid wüßten.\qt@no{101}}%
\qt@newcmd\qurantrans@de@cix{Und sie folgten dem, was die Teufel unter der Herrschaft Sulaimans (den Menschen) verlasen. Nicht Sulaiman war ungläubig, sondern die Teufel waren es, indem sie die Menschen in der Zauberei unterwiesen und in dem, was auf die (beiden) engelhaften Männer) in Babel, Harut und Marut, herabgesandt worden war. Und sie (beide) unterwiesen niemanden (in der Zauberei), ohne zu sagen: ``Wir sind nur eine Versuchung; so werde (darum) nicht ungläubig." Und so lernten sie von ihnen (beiden) das (Zaubermittel), womit man Zwietracht zwischen den Ehegatten stiftet. Doch können sie damit niemandem schaden, außer mit Allahs Erlaubnis. Und sie erlernten, was ihnen schadet und nicht nützt. Und sie wußten doch, daß, wer es erkaufte, am Jenseits wahrlich keinen Anteil hätte. Fürwahr, wie schlimm ist das, wofür sie ihre Seelen verkauft haben, wenn sie (es) nur wußten!\qt@no{102}}%
\qt@newcmd\qurantrans@de@cx{Und wenn sie geglaubt hätten und gottesfürchtig gewesen wären, dann wäre eine Belohnung von Allah dafür wahrlich besser (als das, was sie erkauft haben), wenn sie (es) nur wüßten.\qt@no{103}}%
\qt@newcmd\qurantrans@de@cxi{O die ihr glaubt, sagt nicht (zum Propheten): ``ra'ina" sondern sagt: ``unzurna" Und hört (auf den Propheten)! Und für die Ungläubigen wird es schmerzhafte Strafe geben.\qt@no{104}}%
\qt@newcmd\qurantrans@de@cxii{Weder diejenigen unter den Leuten der Schrift, die ungläubig sind, noch die Götzendiener möchten, daß euch etwas Gutes von eurem Herrn offenbart wird. Aber Allah zeichnet mit Seinem Erbarmen aus, wen Er will, denn Allah besitzt große Huld.\qt@no{105}}%
\qt@newcmd\qurantrans@de@cxiii{Was Wir an Versen aufheben oder in Vergessenheit geraten lassen - Wir bringen bessere oder gleichwertige dafür. Weißt du denn nicht, daß Allah zu allem die Macht hat?\qt@no{106}}%
\qt@newcmd\qurantrans@de@cxiv{Weißt du denn nicht, daß Allah es ist, Der die Herrschaft über die Himmel und die Erde hat, und daß ihr außer Allah weder Schutzherrn noch Helfer habt?\qt@no{107}}%
\qt@newcmd\qurantrans@de@cxv{Oder wollt ihr vielmehr euren Gesandten bitten, so wie zuvor schon Musa gebeten worden war. Wer den Glauben mit dem Unglauben vertauscht, der ist fürwahr vom rechten Weg abgeirrt.\qt@no{108}}%
\qt@newcmd\qurantrans@de@cxvi{Viele von den Leuten der Schrift möchten euch, nachdem ihr den Glauben (angenommen) habt, wieder zu Ungläubigen machen, aus Mißgunst von sich selbst aus, nachdem ihnen die Wahrheit klargeworden ist. Doch verzeiht und seid nachsichtig, bis Allah mit Seiner Anordnung kommt! Allah hat zu allem die Macht.\qt@no{109}}%
\qt@newcmd\qurantrans@de@cxvii{Und verrichtet das Gebet und entrichtet die Abgabe. Und was ihr für euch selbst an Gutem vorausschickt, werdet ihr bei Allah finden. Was ihr tut, sieht Allah wohl.\qt@no{110}}%
\qt@newcmd\qurantrans@de@cxviii{Und sie sagen: ``Niemand wird in den (Paradies)garten eingehen außer, wer Jude oder Christ ist." Das sind ihre Wünsche. Sag: ``Bringt euren Beweis vor, wenn ihr wahrhaftig seid!"\qt@no{111}}%
\qt@newcmd\qurantrans@de@cxix{Aber nein! Wer sich Allah völlig hingibt und dabei Gutes tut, dessen Lohn steht für ihn bei seinem Herrn. Und sie soll keine Furcht überkommen, noch sollen sie traurig sein.\qt@no{112}}%
\qt@newcmd\qurantrans@de@cxx{Die Juden sagen: ``Auf nichts fußen die Christen"; und die Christen sagen: ``Auf nichts fußen die Juden", obwohl sie doch (beide) die Schrift lesen. Dergleichen Worte führten schon diejenigen, die nicht Bescheid wissen. Aber Allah wird zwischen ihnen am Tag der Auferstehung über das richten, worüber sie uneinig sind.\qt@no{113}}%
\qt@newcmd\qurantrans@de@cxxi{Und wer ist ungerechter, als wer verhindert, daß an Allahs Gebetsstätten Sein Name genannt wird, und sich bemüht, sie zu zerstören? Jene sollen sie nur in Furcht betreten. Für sie gibt es im Diesseits Schande und im Jenseits gewaltige Strafe.\qt@no{114}}%
\qt@newcmd\qurantrans@de@cxxii{Allah gehört der Osten und der Westen; wohin ihr euch auch immer wendet, dort ist Allahs Angesicht. Allah ist Allumfassend und Allwissend.\qt@no{115}}%
\qt@newcmd\qurantrans@de@cxxiii{Und sie sagen: ``Allah hat Sich Kinder genommen." Preis sei Ihm! Nein! Vielmehr gehört Ihm (alles), was in den Himmeln und auf der Erde ist. Alle sind Ihm demütig ergeben.\qt@no{116}}%
\qt@newcmd\qurantrans@de@cxxiv{Er ist) der Schöpfer der Himmel und der Erde in ihrer schönsten Form. Und wenn Er eine Angelegenheit bestimmt, so sagt Er zu ihr nur: ``Sei!" und so ist sie.\qt@no{117}}%
\qt@newcmd\qurantrans@de@cxxv{Und diejenigen, die nicht Bescheid wissen, sagen: ``O würde Allah doch zu uns sprechen oder käme zu uns ein Zeichen!" Dergleichen Worte führten schon diejenigen, die vor ihnen waren. Ihre Herzen sind einander ähnlich. Wir haben die Zeichen klargemacht für Leute, die überzeugt sind.\qt@no{118}}%
\qt@newcmd\qurantrans@de@cxxvi{Gewiß, Wir haben dich mit der Wahrheit gesandt als Frohboten und als Warner. Und du wirst nicht nach den (Taten der) Insassen des Höllenbrandes gefragt werden.\qt@no{119}}%
\qt@newcmd\qurantrans@de@cxxvii{Weder die Juden noch die Christen werden mit dir zufrieden sein, bis du ihrem Glaubensbekenntnis folgst. Sag: Gewiß, Allahs Rechtleitung ist die (wahre) Rechtleitung. Wenn du jedoch ihren Neigungen folgst nach dem, was dir an Wissen zugekommen ist, so wirst du vor Allah weder Schutzherrn noch Helfer haben.\qt@no{120}}%
\qt@newcmd\qurantrans@de@cxxviii{Diejenigen, denen Wir die Schrift gegeben haben, lesen sie, wie es ihr zusteht; sie glauben daran. Wer sie jedoch verleugnet, das sind die Verlierer.\qt@no{121}}%
\qt@newcmd\qurantrans@de@cxxix{O Kinder Isra'ils, gedenkt Meiner Gunst, die Ich euch erwiesen, und daß Ich euch vor den (anderen) Weltenbewohnern bevorzugt habe.\qt@no{122}}%
\qt@newcmd\qurantrans@de@cxxx{Und hütet euch vor einem Tag, an dem keine Seele etwas anstelle einer anderen leisten kann und von ihr keine Ersatz(leistung) angenommen wird, noch Fürsprache ihr nützt; und (an dem) ihnen keine Hilfe zuteil wird.\qt@no{123}}%
\qt@newcmd\qurantrans@de@cxxxi{Und (gedenkt,) als Ibrahim von seinem Herrn mit Worten geprüft wurde, da befolgte er sie. Er (Allah) sagte: ``Ich will dich zu einem Vorbild für die Menschen machen." Er (Ibrahim) sagte: ``Und von meiner Nachkommenschaft?" Er sagte: ``Mein Bund erstreckt sich nicht auf die Ungerechten."\qt@no{124}}%
\qt@newcmd\qurantrans@de@cxxxii{Und als Wir das Haus zu einem Ort der Einkehr für die Menschen und zu einer Stätte der Sicherheit machten und (sagten): ``Nehmt Ibrahims Standort als Gebetsplatz!" Und Wir verpflichteten Ibrahim und Isma'il: ``Reinigt Mein Haus für diejenigen, die den Umlauf vollziehen und die sich (dort) zur Andacht zurückziehen und die sich (vor Allah) verbeugen und niederwerfen."\qt@no{125}}%
\qt@newcmd\qurantrans@de@cxxxiii{Und (gedenkt,) als Ibrahim sagte: ``Mein Herr, mache dies zu einer sicheren Ortschaft und versorge ihre Bewohner mit Früchten, wer von ihnen an Allah und den Jüngsten Tag glaubt!" Er (Allah) sagte: ``Wer aber ungläubig ist, den lasse Ich ein wenig genießen, hierauf zwinge Ich ihn in die Strafe des (Höllen)feuers - ein schlimmer Ausgang!"\qt@no{126}}%
\qt@newcmd\qurantrans@de@cxxxiv{Und (gedenkt,) als Ibrahim die Grundmauern des Hauses errichtete, zusammen mit Isma'il, (da beteten sie): ``Unser Herr, nimm (es) von uns an. Du bist ja der Allhörende und Allwissende.\qt@no{127}}%
\qt@newcmd\qurantrans@de@cxxxv{Unser Herr, mache uns Dir ergeben und von unserer Nachkommenschaft eine Dir ergebene Gemeinschaft. Und zeige uns unsere Riten, und nimm unsere Reue an. Du bist ja der Reue-Annehmende und Barmherzige.\qt@no{128}}%
\qt@newcmd\qurantrans@de@cxxxvi{Unser Herr, schicke zu ihnen einen Gesandten von ihnen, der ihnen Deine Worte verliest und sie das Buch und die Weisheit lehrt und sie läutert. Du bist ja der Allmächtige und Allweise."\qt@no{129}}%
\qt@newcmd\qurantrans@de@cxxxvii{Wer wird das Glaubensbekenntnis Ibrahims wohl verschmähen außer dem, der selbst betört ist? Wir haben ihn (Ibrahim) ja bereits im Diesseits auserwählt, und im Jenseits gehört er wahrlich zu den Rechtschaffenen.\qt@no{130}}%
\qt@newcmd\qurantrans@de@cxxxviii{Damals,) als sein Herr zu ihm sagte: ``Werde Muslim!" Ich habe mich dem Herrn der Weltenbewohner ergeben."\qt@no{131}}%
\qt@newcmd\qurantrans@de@cxxxix{Und Ibrahim befahl es seinen Söhnen an - (er) und Ya'qub: ``O meine Kinder, Allah hat euch die Religion auserwählt; so sterbt denn nicht, außer (Ihm) ergeben zu sein!"\qt@no{132}}%
\qt@newcmd\qurantrans@de@cxl{Oder wart ihr etwa Zeugen, als Ya'qub der Tod nahte? Als er zu seinen Söhnen sagte: ``Wem werdet ihr nach mir dienen?" Sie sagten: ``Wir werden deinem Gott und dem Gott deiner Vorväter Ibrahim, lsma'il und Ishaq dienen, als dem Einen Gott, und Ihm sind wir ergeben.\qt@no{133}}%
\qt@newcmd\qurantrans@de@cxli{Das ist eine Gemeinschaft, die schon vergangen ist; ihr kommt zu, was sie verdient hat, und euch, was ihr verdient habt. Und ihr werdet nicht danach befragt werden, was jene zu tun pflegten.\qt@no{134}}%
\qt@newcmd\qurantrans@de@cxlii{Sie sagen: ``Werdet Juden oder Christen, so seid ihr rechtgeleitet." Sag: Nein! Vielmehr das Glaubensbekenntnis Ibrahims, Anhängers des rechten Glaubens, und er gehörte nicht zu den Götzendienern.\qt@no{135}}%
\qt@newcmd\qurantrans@de@cxliii{Sagt: Wir glauben an Allah und an das, was zu uns (als Offenbarung) herabgesandt worden ist, und an das, was zu Ibrahim, Isma'il, lshaq, Ya`qub und den Stämmen herabgesandt wurde, und (an das,) was Musa und 'Isa gegeben wurde, und (an das,) was den Propheten von ihrem Herrn gegeben wurde. Wir machen keinen Unterschied bei jemandem von ihnen, und wir sind Ihm ergeben.\qt@no{136}}%
\qt@newcmd\qurantrans@de@cxliv{Wenn sie an das gleiche glauben, woran ihr glaubt, dann sind sie somit rechtgeleitet. Wenn sie sich jedoch abkehren, dann befinden sie sich in Widerstreit. Aber gegen sie wird Allah dir genügen. Er ist der Allhörende und Allwissende.\qt@no{137}}%
\qt@newcmd\qurantrans@de@cxlv{Allahs Farbgebung - und wessen Farbgebung ist besser als diejenige Allahs! Und Ihm dienen wir.\qt@no{138}}%
\qt@newcmd\qurantrans@de@cxlvi{Sag: Wollt ihr mit uns über Allah streiten, wo Er doch unser und euer Herr ist? Für uns sind unsere Werke und für euch eure Werke. Und wir sind Ihm aufrichtig zugetan.\qt@no{139}}%
\qt@newcmd\qurantrans@de@cxlvii{Oder wollt ihr etwa sagen, Ibrahim, Isma'il, lshaq, Ya'qub und die Stämme seien Juden oder Christen gewesen? - Sag: Wißt ihr es besser oder Allah? - Wer ist ungerechter, als wer ein Zeugnis von Allah bei sich verheimlicht? Und Allah ist nicht unachtsam dessen, was ihr tut.\qt@no{140}}%
\qt@newcmd\qurantrans@de@cxlviii{Das ist eine Gemeinschaft, die schon vergangen ist; ihr kommt zu, was sie verdient hat und euch, was ihr verdient habt. Und ihr werdet nicht danach befragt werden, was jene zu tun pflegten.\qt@no{141}}%
\qt@newcmd\qurantrans@de@cxlix{Die Toren unter den Menschen werden sagen: ``Was hat sie von der Gebetsrichtung, die sie (bisher) einhielten, abgebracht?" Sag: Allah gehört der Osten und der Westen. Er leitet, wen Er will, auf einen geraden Weg.\qt@no{142}}%
\qt@newcmd\qurantrans@de@cl{Und so haben Wir euch zu einer Gemeinschaft der Mitte gemacht, damit ihr Zeugen über die (anderen) Menschen seiet und damit der Gesandte über euch Zeuge sei. Wir hatten die Gebetsrichtung, die du einhieltest, nur bestimmt, um zu wissen, wer dem Gesandten folgt und wer sich auf den Fersen umkehrt. Und es ist wahrlich schwer außer für diejenigen, die Allah rechtgeleitet hat. Aber Allah läßt nicht zu, daß euer Glaube verloren geht. Allah ist zu den Menschen wahrlich Gnädig, Barmherzig.\qt@no{143}}%
\qt@newcmd\qurantrans@de@cli{Wir sehen ja dein Gesicht sich (suchend) zum Himmel wenden. Nun wollen Wir dir ganz gewiß eine Gebetsrichtung zuweisen, mit der du zufrieden bist. So wende dein Gesicht in Richtung der geschützten Gebetsstätte! Und wo immer ihr seid, wendet eure Gesichter in ihrer Richtung! Diejenigen, denen die Schrift gegeben wurde, wissen sehr wohl, daß dies die Wahrheit von ihrem Herrn ist. Und Allah ist nicht unachtsam dessen, was sie tun.\qt@no{144}}%
\qt@newcmd\qurantrans@de@clii{Selbst wenn du zu denjenigen, denen die Schrift gegeben wurde, mit jeglichen Zeichen kämest, würden sie doch nicht deiner Gebetsrichtung folgen; noch folgst du ihrer Gebetsrichtung. Und auch untereinander folgen sie nicht der Gebetsrichtung der anderen. Würdest du aber ihren Neigungen folgen, nach all dem, was dir an Wissen zugekommen ist, dann gehörtest du wahrlich zu den Ungerechten.\qt@no{145}}%
\qt@newcmd\qurantrans@de@cliii{Diejenigen, denen Wir die Schrift gegeben haben, kennen es, wie sie ihre Söhne kennen. Aber ein Teil von ihnen verheimlicht wahrlich die Wahrheit, obwohl sie (sie) wissen.\qt@no{146}}%
\qt@newcmd\qurantrans@de@cliv{Es ist) die Wahrheit von deinem Herrn, gehöre daher nicht zu den Zweiflern!\qt@no{147}}%
\qt@newcmd\qurantrans@de@clv{Jeder hat eine Zielrichtung, zu der er sich hinwendet. So wetteifert nach den guten Dingen! Wo immer ihr auch sein werdet, Allah wird euch alle herbeibringen. Allah hat zu allem die Macht.\qt@no{148}}%
\qt@newcmd\qurantrans@de@clvi{Und woher du immer heraustrittst, da wende dein Gesicht in Richtung der geschützten Gebetsstätte. Es ist wirklich die Wahrheit von deinem Herrn. Und Allah ist nicht unachtsam dessen, was ihr tut.\qt@no{149}}%
\qt@newcmd\qurantrans@de@clvii{Und woher du immer heraustrittst, da wende dein Gesicht in Richtung der geschützten Gebetsstätte. Und wo immer ihr seid, da wendet eure Gesichter in ihrer Richtung, damit die Menschen kein Beweismittel gegen euch haben, außer denjenigen von ihnen, die Unrecht tun. - So fürchtet nicht sie, sondern fürchtet Mich! - Und damit Ich Meine Gunst an euch vollende, auf daß ihr rechtgeleitet werden möget.\qt@no{150}}%
\qt@newcmd\qurantrans@de@clviii{So, wie Wir zu euch einen Gesandten von euch geschickt haben, der euch Unsere Worte verliest und euch läutert und euch die Schrift und die Wahrheit lehrt und euch lehrt, was ihr nicht wußtet.\qt@no{151}}%
\qt@newcmd\qurantrans@de@clix{Gedenkt Meiner, so gedenke Ich eurer. Seid Mir dankbar und seid nicht undankbar gegen Mich.\qt@no{152}}%
\qt@newcmd\qurantrans@de@clx{O die ihr glaubt, sucht Hilfe in der Standhaftigkeit und im Gebet! Allah ist mit den Standhaften.\qt@no{153}}%
\qt@newcmd\qurantrans@de@clxi{Und sagt nicht von denen, die auf Allahs Weg getötet werden, sie seien tot! Nein! Vielmehr sind sie lebendig; aber ihr nehmt es nicht wahr.\qt@no{154}}%
\qt@newcmd\qurantrans@de@clxii{Und Wir werden euch ganz gewiß mit ein wenig Furcht und Hunger und Mangel an Besitz, Seelen und Früchten prüfen. Doch verkünde frohe Botschaft den Standhaften,\qt@no{155}}%
\qt@newcmd\qurantrans@de@clxiii{die, wenn sie ein Unglück trifft, sagen: ``Wir gehören Allah, und zu Ihm kehren wir zurück."\qt@no{156}}%
\qt@newcmd\qurantrans@de@clxiv{Sie sind es, denen Segnungen von ihrem Herrn und Erbarmen zuteil werden, und sie sind die Rechtgeleiteten.\qt@no{157}}%
\qt@newcmd\qurantrans@de@clxv{Gewiß, as-Safa und al-Marwa gehören zu den (Orten der) Kulthandlungen Allahs. Wenn einer die Pilgerfahrt zum Hause oder die Besuchsfahrt vollzieht, so ist es keine Sünde für ihn, wenn er zwischen ihnen (beiden) den Umgang macht. Und wer (von sich aus) freiwillig Gutes tut, so ist Allah Dankbar und Allwissend.\qt@no{158}}%
\qt@newcmd\qurantrans@de@clxvi{Diejenigen, die verheimlichen, was Wir an klaren Beweisen und Rechtleitung hinabgesandt haben, nachdem Wir es den Menschen in der Schrift klar gemacht haben, sie werden von Allah verflucht und auch von den Fluchenden,\qt@no{159}}%
\qt@newcmd\qurantrans@de@clxvii{außer denjenigen, die bereuen und verbessern und klar machen. Ihre Reue nehme Ich an, Ich bin ja der Reue-Annehmende und Barmherzige.\qt@no{160}}%
\qt@newcmd\qurantrans@de@clxviii{Diejenigen aber, die ungläubig sind und als Ungläubige sterben, auf ihnen liegt der Fluch Allahs und der Engel und der Menschen allesamt,\qt@no{161}}%
\qt@newcmd\qurantrans@de@clxix{ewig darin zu bleiben. Die Strafe soll ihnen nicht erleichtert noch soll ihnen Aufschub gewährt werden.\qt@no{162}}%
\qt@newcmd\qurantrans@de@clxx{Euer Gott ist ein Einziger Gott. Es gibt keinen Gott außer Ihm, dem Allerbarmer, dem Barmherzigen.\qt@no{163}}%
\qt@newcmd\qurantrans@de@clxxi{In der Schöpfung der Himmel und der Erde; im Unterschied von Nacht und Tag; in den Schiffen, die das Meer befahren mit dem, was den Menschen nützt; darin, daß Allah Wasser vom Himmel herabkommen läßt, und damit dann die Erde nach ihrem Tod wieder lebendig macht und auf ihr allerlei Tiere sich ausbreiten läßt; und im Wechsel der Winde und der Wolken, die zwischen Himmel und Erde dienstbar gemacht sind, sind wahrlich Zeichen für Leute, die begreifen.\qt@no{164}}%
\qt@newcmd\qurantrans@de@clxxii{Und doch gibt es unter den Menschen manche, die außer Allah andere als Seinesgleichen annehmen und ihnen dieselbe Liebe schenken wie Allah. Aber die Gläubigen sind stärker in ihrer Liebe zu Allah. Und wenn doch diejenigen, die Unrecht tun, wenn sie der Strafe angesichtig werden, erkennen würden, daß alle Stärke Allah gehört und daß Allah streng im Strafen ist.\qt@no{165}}%
\qt@newcmd\qurantrans@de@clxxiii{Wenn diejenigen, denen Gefolgschaft geleistet worden ist, sich von denen lossagen, die Gefolgschaft geleistet haben, und sie die Strafe sehen und die Verbindungsstricke zu ihnen abgeschnitten sind!\qt@no{166}}%
\qt@newcmd\qurantrans@de@clxxiv{Und diejenigen, die Gefolgschaft geleistet haben, sagen: ``Hätten wir doch eine (Möglichkeit zur) Wiederholung, dann würden wir uns von ihnen lossagen, so wie sie sich von uns losgesagt haben." So wird Allah ihnen ihre Werke zeigen, als gramvolle Reue für sie. Und sie werden aus dem (Höllen)feuer nicht herauskommen.\qt@no{167}}%
\qt@newcmd\qurantrans@de@clxxv{O ihr Menschen! Eßt von dem, was es auf der Erde gibt, als etwas Erlaubtem und Gutem, und folgt nicht den Fußstapfen des Satans! Er ist euch ein deutlicher Feind.\qt@no{168}}%
\qt@newcmd\qurantrans@de@clxxvi{Er befiehlt euch nur Böses und Schändliches, und daß ihr gegen Allah aussagen sollt, was ihr nicht wißt.\qt@no{169}}%
\qt@newcmd\qurantrans@de@clxxvii{Und wenn man zu ihnen sagt: ``Folgt dem, was Allah herabgesandt hat", sagen sie: ``Nein! Vielmehr folgen wir dem, worin wir unsere Väter vorgefunden haben." Was denn, auch wenn ihre Väter nichts begriffen und nicht rechtgeleitet waren?\qt@no{170}}%
\qt@newcmd\qurantrans@de@clxxviii{Das Gleichnis derjenigen, die ungläubig sind, ist wie jemand, der etwas nachschreit, was nichts hört außer Stimmen und Zurufen. Taub, stumm, blind; so begreifen sie nicht.\qt@no{171}}%
\qt@newcmd\qurantrans@de@clxxix{O die ihr glaubt, eßt von den guten Dingen, mit denen Wir euch versorgt haben, und seid Allah dankbar, wenn ihr Ihm (allein) dient!\qt@no{172}}%
\qt@newcmd\qurantrans@de@clxxx{Verboten hat Er euch nur (den Genuß von) Verendetem, Blut, Schweinefleisch und dem, worüber ein anderer (Name) als Allah(s) angerufen worden ist. Wer sich aber in einer Zwangslage befindet, ohne zu begehren oder das Maß zu überschreiten, für den ist es keine Sünde. Allah ist Allvergebend und Barmherzig.\qt@no{173}}%
\qt@newcmd\qurantrans@de@clxxxi{Diejenigen, die verheimlichen, was Allah von der Schrift herabgesandt hat, und es für einen geringen Preis verkaufen, sie verzehren in ihren Bäuchen nichts als Feuer. Und Allah wird zu ihnen am Tag der Auferstehung weder sprechen noch sie läutern. Für sie wird es schmerzhafte Strafe geben.\qt@no{174}}%
\qt@newcmd\qurantrans@de@clxxxii{Das sind diejenigen, die das Irregehen für die Rechtleitung erkauft haben und die Strafe für die Vergebung. Wie beharrlich sind sie gegenüber dem (Höllen)feuer!\qt@no{175}}%
\qt@newcmd\qurantrans@de@clxxxiii{Dies, weil Allah die Schrift mit der Wahrheit offenbart hat. Und diejenigen, die über die Schrift uneinig sind, befinden sich wahrlich in tiefem Widerstreit.\qt@no{176}}%
\qt@newcmd\qurantrans@de@clxxxiv{Nicht darin besteht die Güte, daß ihr eure Gesichter gegen Osten oder Westen wendet. Güte ist vielmehr, daß man an Allah, den Jüngsten Tag, die Engel, die Bücher und die Propheten glaubt und vom Besitz - obwohl man ihn liebt - der Verwandtschaft, den Waisen, den Armen, dem Sohn des Weges, den Bettlern und für (den Loskauf von) Sklaven hergibt, das Gebet verrichtet und die Abgabe entrichtet; und diejenigen, die ihre Verpflichtung einhalten, wenn sie eine eingegangen sind, und diejenigen, die standhaft bleiben in Not, Leid und in Kriegszeiten, das sind diejenigen, die wahrhaftig sind, und das sind die Gottesfürchtigen.\qt@no{177}}%
\qt@newcmd\qurantrans@de@clxxxv{O die ihr glaubt, vorgeschrieben ist euch Wiedervergeltung für die Getöteten: der Freie für den Freien, der Sklave für den Sklaven und das Weib für das Weib. Doch wenn einem von seinem Bruder etwas erlassen wird, so soll die Verfolgung (der Ansprüche) in rechtlicher Weise und die Zahlungsleistung an ihn auf ordentliche Weise geschehen. Das ist eine Erleichterung von eurem Herrn und Erbarmung. Wer aber nach diesem eine Übertretung begeht, für den gibt es schmerzhafte Strafe.\qt@no{178}}%
\qt@newcmd\qurantrans@de@clxxxvi{In der Wiedervergeltung liegt Leben für euch, o die ihr Verstand besitzt, auf daß ihr gottesfürchtig werden möget!\qt@no{179}}%
\qt@newcmd\qurantrans@de@clxxxvii{Vorgeschrieben ist euch, wenn sich einem von euch der Tod naht, sofern er Gut hinterläßt, ein Vermächtnis zugunsten der Eltern und nächsten Verwandten in rechtlicher Weise zu treffen, als eine Pflicht für die Gottesfürchtigen.\qt@no{180}}%
\qt@newcmd\qurantrans@de@clxxxviii{Wer es aber abändert - nachdem er es gehört hat-, so liegt dessen Sünde nur bei demjenigen, der es abändert. Allah ist Allhörend und Allwissend.\qt@no{181}}%
\qt@newcmd\qurantrans@de@clxxxix{Wer aber von Seiten eines Erblassers Abweichung oder Sünde befürchtet und zwischen ihnen schlichtet, den trifft keine Sünde. Allah ist Allvergebend und Barmherzig.\qt@no{182}}%
\qt@newcmd\qurantrans@de@cxc{O die ihr glaubt, vorgeschrieben ist euch das Fasten, so wie es denjenigen vor euch vorgeschrieben war, auf daß ihr gottesfürchtig werden möget.\qt@no{183}}%
\qt@newcmd\qurantrans@de@cxci{Vorgeschrieben ist es euch) an bestimmten Tagen. Wer von euch jedoch krank ist oder sich auf einer Reise befindet, der soll eine (gleiche) Anzahl von anderen Tagen (fasten). Und denjenigen, die es zu leisten vermögen, ist als Ersatz die Speisung eines Armen auferlegt. Wer aber freiwillig Gutes tut, für den ist es besser. Und daß ihr fastet, ist besser für euch, wenn ihr (es) nur wißt!\qt@no{184}}%
\qt@newcmd\qurantrans@de@cxcii{Der Monat Ramadan (ist es), in dem der Qur'an als Rechtleitung für die Menschen herabgesandt worden ist und als klare Beweise der Rechtleitung und der Unterscheidung. Wer also von euch während dieses Monats anwesend ist, der soll ihn fasten, wer jedoch krank ist oder sich auf einer Reise befindet, eine (gleiche) Anzahl von anderen Tagen (fasten). Allah will für euch Erleichterung; Er will für euch nicht Erschwernis, - damit ihr die Anzahl vollendet und Allah als den Größten preist, dafür, daß Er euch rechtgeleitet hat, auf daß ihr dankbar sein möget.\qt@no{185}}%
\qt@newcmd\qurantrans@de@cxciii{Und wenn dich Meine Diener nach Mir fragen, so bin Ich nahe; Ich erhöre den Ruf des Bittenden, wenn er Mich anruft. So sollen sie nun auf Mich hören und an Mich glauben, auf daß sie besonnen handeln mögen.\qt@no{186}}%
\qt@newcmd\qurantrans@de@cxciv{Erlaubt ist euch, in der Nacht des Fastens mit euren Frauen Beischlaf auszuüben; sie sind euch ein Kleid, und ihr seid ihnen ein Kleid. Allah weiß, daß ihr euch selbst (immer wieder) betrogt, und da hat Er eure Reue angenommen und euch verziehen. Von jetzt an verkehrt mit ihnen und trachtet nach dem, was Allah für euch bestimmt hat, und eßt und trinkt, bis sich für euch der weiße vom schwarzen Faden der Morgendämmerung klar unterscheidet! Hierauf vollzieht das Fasten bis zur Nacht! Und verkehrt nicht mit ihnen, während ihr euch (zur Andacht) in die Gebetsstätten zurückgezogen habt! Dies sind Allahs Grenzen, so kommt ihnen nicht zu nahe! So macht Allah den Menschen Seine Zeichen klar, auf daß sie gottesfürchtig werden mögen.\qt@no{187}}%
\qt@newcmd\qurantrans@de@cxcv{Und zehrt nicht euren Besitz untereinander auf nichtige Weise auf und bestecht nicht damit die Richter, um einen Teil des Besitzes der Menschen in sündhafter Weise zu verzehren, wo ihr (es) doch wißt.\qt@no{188}}%
\qt@newcmd\qurantrans@de@cxcvi{Sie fragen dich nach den Jungmonden. Sag: Sie sind festgesetzte Zeiten für die Menschen und für die Pilgerfahrt. Und nicht darin besteht die Frömmigkeit, daß ihr von der Rückseite in die Häuser kommt. Frömmigkeit besteht vielmehr darin, daß man gottesfürchtig ist. So kommt durch die Türen in die Häuser und fürchtet Allah, auf daß es euch wohl ergehen möge!\qt@no{189}}%
\qt@newcmd\qurantrans@de@cxcvii{Und kämpft auf Allahs Weg gegen diejenigen, die gegen euch kämpfen, doch übertretet nicht! Allah liebt nicht die Übertreter.\qt@no{190}}%
\qt@newcmd\qurantrans@de@cxcviii{Und tötet sie, wo immer ihr auf sie trefft, und vertreibt sie, von wo sie euch vertrieben haben, denn Verfolgung ist schlimmer als Töten! Kämpft jedoch nicht gegen sie bei der geschützten Gebetsstätte, bis sie dort (zuerst) gegen euch kämpfen. Wenn sie aber (dort) gegen euch kämpfen, dann tötet sie. Solcherart ist der Lohn der Ungläubigen.\qt@no{191}}%
\qt@newcmd\qurantrans@de@cxcix{Wenn sie jedoch aufhören, so ist Allah Allvergebend und Barmherzig.\qt@no{192}}%
\qt@newcmd\qurantrans@de@cc{Und kämpft gegen sie, bis es keine Verfolgung mehr gibt und die Religion (allein) Allahs ist. Wenn sie jedoch aufhören, dann darf es kein feindseliges Vorgehen geben außer gegen die Ungerechten.\qt@no{193}}%
\qt@newcmd\qurantrans@de@cci{Der Schutzmonat ist für den Schutzmonat, und (für) die unantastbaren Dinge ist Wiedervergeltung. Wenn jemand gegen euch übertritt, dann geht (auch ihr) in gleichem Maß gegen ihn vor, wie er gegen euch übertreten hat. Und fürchtet Allah und wißt, daß Allah mit den Gottesfürchtigen ist.\qt@no{194}}%
\qt@newcmd\qurantrans@de@ccii{Und gebt auf Allahs Weg aus und stürzt euch nicht mit eigener Hand ins Verderben. Und tut Gutes. Allah liebt die Gutes Tuenden.\qt@no{195}}%
\qt@newcmd\qurantrans@de@cciii{Vollzieht die Pilgerfahrt und die Besuchsfahrt für Allah. Wenn ihr jedoch (daran) gehindert werdet, dann (bringt) an Opfertieren (dar), was euch leichtfällt. Und schert euch nicht die Köpfe, bevor die Opfertiere ihren Schlachtort erreicht haben! Wer von euch krank ist oder ein Leiden an seinem Kopf hat, der soll Ersatz leisten mit Fasten, Almosen oder Opferung eines Schlachttieres. - Wenn ihr aber in Sicherheit seid, dann soll derjenige, der die Besuchsfahrt mit der Pilgerfahrt durchführen möchte, an Opfertieren (darbringen), was ihm leichtfällt. Wer jedoch nicht(s) finden kann, der soll drei Tage während der Pilgerfahrt fasten und sieben, wenn ihr zurückgekehrt seid; das sind im ganzen zehn. Dies (gilt nur) für den, dessen Angehörige nicht in der geschützten Gebetsstätte wohnhaft sind. Und fürchtet Allah und wißt, daß Allah streng im Bestrafen ist!\qt@no{196}}%
\qt@newcmd\qurantrans@de@cciv{Die (Zeit der) Pilgerfahrt (sind) bekannte Monate. Wer in ihnen die (Durchführung der) Pilgerfahrt beschlossen hat, der darf keinen Beischlaf ausüben, keinen Frevel begehen und nicht Streit führen während der Pilgerfahrt. Und was ihr an Gutem tut, Allah weiß es. Und versorgt euch mit Reisevorrat, doch der beste Vorrat ist die Gottesfurcht. Und fürchtet Mich, o die ihr Verstand besitzt!\qt@no{197}}%
\qt@newcmd\qurantrans@de@ccv{Es ist keine Sünde für euch, daß ihr nach Huld von eurem Herrn trachtet. Doch wenn ihr von 'Arafat hergeströmt seid, dann gedenkt Allahs bei der geschützten Kultstätte. Und gedenkt Seiner, wie Er euch rechtgeleitet hat, obwohl ihr vordem wahrlich zu den Irregehenden gehörtet.\qt@no{198}}%
\qt@newcmd\qurantrans@de@ccvi{Hierauf strömt weiter, woher die (anderen) Menschen weiterströmen, und bittet Allah um Vergebung. Allah ist Allvergebend und Barmherzig.\qt@no{199}}%
\qt@newcmd\qurantrans@de@ccvii{Wenn ihr dann eure Riten vollzogen habt, dann gedenkt Allahs, wie ihr eurer Väter gedenkt, oder mit noch innigerem Gedenken. Unter den Menschen gibt es manch einen, der sagt: ``Unser Herr, gib uns im Diesseits!" Doch hat er am Jenseits keinen Anteil.\qt@no{200}}%
\qt@newcmd\qurantrans@de@ccviii{Unter ihnen gibt es aber auch solche, die sagen: ``Unser Herr, gib uns im Diesseits Gutes und im Jenseits Gutes, und bewahre uns vor der Strafe des (Höllen)feuers!\qt@no{201}}%
\qt@newcmd\qurantrans@de@ccix{Für jene gibt es einen Anteil an dem, was sie verdient haben. Und Allah ist schnell im Abrechnen.\qt@no{202}}%
\qt@newcmd\qurantrans@de@ccx{Und gedenkt Allahs während einer bestimmten Anzahl von Tagen. Wer sich jedoch in zwei Tagen (mit dem Aufbruch) beeilt, den trifft keine Sünde, und wer länger bleibt, den trifft keine Sünde; (das gilt) für den, der gottesfürchtig ist. Fürchtet Allah und wißt, daß ihr zu Ihm versammelt werdet!\qt@no{203}}%
\qt@newcmd\qurantrans@de@ccxi{Unter den Menschen gibt es manch einen, dessen Worte über das diesseitige Leben dir gefallen, und der Allah zum Zeugen nimmt für das, was in seinem Herzen ist. Dabei ist er der hartnäckigste Widersacher.\qt@no{204}}%
\qt@newcmd\qurantrans@de@ccxii{Wenn er sich abkehrt, bemüht er sich eifrig darum, auf der Erde Unheil zu stiften und Saatfelder und Nachkommenschaft zu vernichten. Aber Allah liebt nicht das Unheil.\qt@no{205}}%
\qt@newcmd\qurantrans@de@ccxiii{Und wenn man zu ihm sagt: ``Fürchte Allah", reißt ihn der Stolz zur Sünde hin. Die Hölle soll seine Genüge sein - wahrlich eine schlimme Lagerstatt!\qt@no{206}}%
\qt@newcmd\qurantrans@de@ccxiv{Unter den Menschen gibt es (auch) manchen, der sich selbst im Trachten nach Allahs Zufriedenheit verkauft. Und Allah ist zu den Menschen Gnädig.\qt@no{207}}%
\qt@newcmd\qurantrans@de@ccxv{O die ihr glaubt, tretet allesamt in den Islam ein und folgt nicht den Fußstapfen des Satans! Er ist euch ja ein deutlicher Feind.\qt@no{208}}%
\qt@newcmd\qurantrans@de@ccxvi{Wenn ihr aber strauchelt, nachdem die klaren Beweise zu euch gekommen sind, so wißt, daß Allah Allmächtig und Allweise ist.\qt@no{209}}%
\qt@newcmd\qurantrans@de@ccxvii{Erwarten sie (etwas anderes), als daß Allah in Schattendächern aus Wolken zu ihnen kommt und die Engel? Doch dann wird die Angelegenheit entschieden sein. Und zu Allah werden (all) die Angelegenheiten zurückgebracht.\qt@no{210}}%
\qt@newcmd\qurantrans@de@ccxviii{Frage die Kinder Isra'ils, wie viele klare Beweise Wir ihnen gegeben haben. Wenn aber einer die Gunst Allahs abändert, nachdem sie ihm zuteil geworden ist -, dann ist Allah streng im Bestrafen.\qt@no{211}}%
\qt@newcmd\qurantrans@de@ccxix{Ausgeschmückt ist denen, die ungläubig sind, das diesseitige Leben, und sie spotten über diejenigen, die glauben. Aber diejenigen, die gottesfürchtig sind, werden am Tag der Auferstehung über ihnen stehen. Und Allah versorgt, wen Er will, ohne zu berechnen.\qt@no{212}}%
\qt@newcmd\qurantrans@de@ccxx{Die Menschen waren eine einzige Gemeinschaft. Dann schickte Allah die Propheten als Verkünder froher Botschaft und als Überbringer von Warnungen und sandte mit ihnen die Bücher mit der Wahrheit herab, um zwischen den Menschen über das zu richten, worüber sie uneinig waren. Doch nur diejenigen waren - aus Mißgunst untereinander - darüber uneinig, denen sie gegeben wurden, nachdem die klaren Beweise zu ihnen gekommen waren. Und so hat Allah mit Seiner Erlaubnis diejenigen, die glauben, zu der Wahrheit geleitet, über die sie uneinig waren. Und Allah leitet, wen Er will, auf einen geraden Weg.\qt@no{213}}%
\qt@newcmd\qurantrans@de@ccxxi{Oder meint ihr etwa, daß ihr in den (Paradies)garten eingehen werdet, noch bevor Gleiches über euch gekommen ist, wie über diejenigen, die vor euch dahingegangen sind? Not und Leid widerfuhr ihnen, und sie wurden erschüttert, bis daß der Gesandte und diejenigen, die mit ihm glaubten, sagten: ``Wann kommt Allahs Sieg?" Aber wahrlich, Allahs Sieg ist nahe.\qt@no{214}}%
\qt@newcmd\qurantrans@de@ccxxii{Sie fragen dich, was sie ausgeben sollen. Sag: Was immer ihr an Gutem ausgebt, soll den Eltern, den nächsten Verwandten, den Waisen, den Armen und dem Sohn des Weges zukommen. Und was immer ihr an Gutem tut, so weiß Allah darüber Bescheid.\qt@no{215}}%
\qt@newcmd\qurantrans@de@ccxxiii{Vorgeschrieben ist euch zu kämpfen, obwohl es euch zuwider ist. Aber vielleicht ist euch etwas zuwider, während es gut für euch ist, und vielleicht ist euch etwas lieb, während es schlecht für euch ist. Allah weiß, ihr aber wißt nicht.\qt@no{216}}%
\qt@newcmd\qurantrans@de@ccxxiv{Sie fragen dich nach dem Schutzmonat, danach, in ihm zu kämpfen. Sag: In ihm zu kämpfen ist schwerwiegend. Aber von Allahs Weg abzuhalten - und Ihn zu verleugnen -, und von der geschützten Gebetsstätte (abzuhalten) und deren Anwohner von ihr vertreiben, ist (noch) schwerwiegender bei Allah. Und Verfolgung ist schwerwiegender als Töten. Und sie werden nicht eher aufhören, gegen euch zu kämpfen, bis sie euch von eurer Religion abgekehrt haben - wenn sie (es) können. Wer aber unter euch sich von seiner Religion abkehrt und dann als Ungläubiger stirbt -, das sind diejenigen, deren Werke im Diesseits und im Jenseits hinfällig werden. Das sind Insassen des (Höllen)feuers. Ewig werden sie darin bleiben.\qt@no{217}}%
\qt@newcmd\qurantrans@de@ccxxv{Diejenigen (aber), die glauben, und diejenigen, die auswandern und sich auf Allahs Weg abmühen, sie hoffen auf Allahs Erbarmen. Allah ist Allvergebend und Barmherzig.\qt@no{218}}%
\qt@newcmd\qurantrans@de@ccxxvi{Sie fragen dich nach berauschendem Trunk und Glücksspiel. Sag: In ihnen (beiden) liegt große Sünde und Nutzen für die Menschen. Aber die Sünde in ihnen (beiden) ist größer als ihr Nutzen. Und sie fragen dich, was sie ausgeben sollen. Sag: Den Überschuss. So macht Allah euch die Zeichen klar, auf daß ihr nachdenken möget,\qt@no{219}}%
\qt@newcmd\qurantrans@de@ccxxvii{über das Diesseits und das Jenseits. Und sie fragen dich nach den Waisen. Sag: Ihre Lage zu verbessern ist besser. Und wenn ihr euch mit ihnen zusammentut, so sind sie eure Brüder. Allah weiß den Unheilstifter vom Heilstifter (zu unterscheiden). Und wenn Allah gewollt hätte, hätte Er euch wahrlich in Bedrängnis gebracht. Allah ist Allmächtig und Allweise.\qt@no{220}}%
\qt@newcmd\qurantrans@de@ccxxviii{Und heiratet Götzendienerinnen nicht, bevor sie glauben. Und eine gläubige Sklavin ist fürwahr besser als eine Götzendienerin, auch wenn diese euch gefallen sollte. Und verheiratet nicht (gläubige Frauen) mit Götzendienern, bevor sie glauben. Und ein gläubiger Sklave ist fürwahr besser als ein Götzendiener, auch wenn dieser euch gefallen sollte. Jene laden zum (Höllen)feuer ein. Allah aber lädt zum (Paradies)garten und zur Vergebung ein, mit Seiner Erlaubnis, und macht den Menschen Seine Zeichen klar, auf daß sie bedenken mögen.\qt@no{221}}%
\qt@newcmd\qurantrans@de@ccxxix{Sie fragen dich nach der Monatsblutung. Sag: Sie ist ein Leiden. So haltet euch von den Frauen während der Monatsblutung fern, und kommt ihnen nicht nahe, bis sie rein sind. Wenn sie sich dann gereinigt haben, so kommt zu ihnen, wie Allah es euch geboten hat. Allah liebt die Reumütigen, und Er liebt die, die sich rein halten.\qt@no{222}}%
\qt@newcmd\qurantrans@de@ccxxx{Eure Frauen sind euch ein Saatfeld. So kommt zu eurem Saatfeld, wann und wie ihr wollt. Doch schickt (Gutes) für euch selbst voraus. Und fürchtet Allah und wißt, daß ihr Ihm begegnen werdet. Und verkünde den Gläubigen frohe Botschaft.\qt@no{223}}%
\qt@newcmd\qurantrans@de@ccxxxi{Und macht nicht Allah mit euren Eiden zu einem Hinderungsgrund, gütig und gottesfürchtig zu sein und zwischen den Menschen Frieden zu stiften. Allah ist Allhörend und Allwissend.\qt@no{224}}%
\qt@newcmd\qurantrans@de@ccxxxii{Allah wird euch nicht für etwas Unbedachtes in euren Eiden belangen. Jedoch wird Er euch für das belangen, was eure Herzen erworben haben. Allah ist Allvergebend und Nachsichtig.\qt@no{225}}%
\qt@newcmd\qurantrans@de@ccxxxiii{Diejenigen, die schwören, sich ihrer Frauen zu enthalten, haben eine Wartezeit von vier Monaten. Wenn sie dann (von ihrem Schwur) zurücktreten, so ist Allah Allvergebend und Barmherzig.\qt@no{226}}%
\qt@newcmd\qurantrans@de@ccxxxiv{Wenn sie aber zur Scheidung entschlossen sind, so ist Allah Allhörend und Allwissend.\qt@no{227}}%
\qt@newcmd\qurantrans@de@ccxxxv{Geschiedene Frauen sollen (mit sich) selbst drei Zeitabschnitte abwarten. Und es ist ihnen nicht erlaubt, zu verheimlichen, was Allah in ihrem Mutterleib erschaffen hat, wenn sie an Allah und den Jüngsten Tag glauben. Und ihre Ehemänner haben ein größeres Anrecht, sie zurückzunehmen, wenn sie eine Aussöhnung wollen. Und ihnen (den Frauen) steht in rechtlicher Weise (gegenüber den Männern) das gleiche zu, wie (den Männern) gegenüber ihnen. Doch die Männer haben ihnen gegenüber einen gewissen Vorzug. Und Allah ist Allmächtig und Allweise.\qt@no{228}}%
\qt@newcmd\qurantrans@de@ccxxxvi{Die (widerrufliche) Scheidung ist zweimal (erlaubt). Dann (sollen die Frauen) in rechtlicher Weise behalten oder in ordentlicher Weise freigegeben (werden). Und es ist euch nicht erlaubt, etwas von dem, was ihr ihnen gegeben habt, (wieder) zu nehmen, außer wenn die beiden fürchten, daß sie Allahs Grenzen nicht einhalten werden. Wenn ihr aber befürchtet, daß die beiden Allahs Grenzen nicht einhalten werden, dann ist für die beiden keine Sünde in dem, womit (an Geld) sie sich löst. Dies sind Allahs Grenzen, so übertretet sie nicht! Wer aber Allahs Grenzen übertritt, diejenigen sind die Ungerechten.\qt@no{229}}%
\qt@newcmd\qurantrans@de@ccxxxvii{Wenn er sich (ein drittes, unwiderrufliches Mal) von ihr scheidet, dann ist sie ihm nicht mehr (als Gattin) erlaubt, bevor sie nicht einen anderen Mann geheiratet hat. Wenn dieser sich von ihr scheidet, so ist es keine Sünde für die beiden, zu einander zurückzukehren, wenn sie (dabei) glauben, Allahs Grenzen einhalten zu können. Dies sind Allahs Grenzen, die Er Leuten klar macht, die Bescheid wissen.\qt@no{230}}%
\qt@newcmd\qurantrans@de@ccxxxviii{Und wenn ihr euch von Frauen scheidet und sie dann ihre festgesetzte Zeit erreichen, so behaltet sie in rechtlicher Weise oder gebt sie in rechtlicher Weise frei. Doch behaltet sie nicht mit der Absicht der Schädigung, so daß ihr übertretet. Wer dies tut, der fügt sich ja selbst Unrecht zu. Und macht euch nicht über Allahs Zeichen lustig. Und gedenkt Allahs Gunst an euch und dessen, was Er von dem Buch und an Weisheit auf euch herabgesandt hat, um euch damit zu ermahnen. Und fürchtet Allah und wißt, daß Allah über alles Bescheid weiß.\qt@no{231}}%
\qt@newcmd\qurantrans@de@ccxxxix{Und wenn ihr euch von Frauen scheidet und sie dann ihre festgesetzte Zeit erreicht haben, so haltet sie nicht davon ab, ihre Gatten zu heiraten, wenn sie sich in rechtlicher Weise miteinander geeinigt haben. Damit wird von euch ermahnt, wer an Allah und den Jüngsten Tag glaubt. Das ist lauterer für euch und reiner. Allah weiß, ihr aber wißt nicht.\qt@no{232}}%
\qt@newcmd\qurantrans@de@ccxl{Und die Mütter stillen ihre Kinder zwei volle Jahre. (Das gilt) für jemanden, der das Stillen zu Ende führen will. Und demjenigen, dem das Kind geboren wurde, obliegt es, für ihre Versorgung und Kleidung in rechtlicher Weise aufzukommen. Keiner Seele wird mehr auferlegt, als sie zu leisten vermag. Keine Mutter soll wegen ihres Kindes zu Schaden kommen, noch einer, dem das Kind geboren wurde, wegen seines Kindes. Und dem Erben obliegt das gleiche. Wenn sie beide jedoch in gegenseitigem Einvernehmen und gemeinsamer Beratung (das Kind vorzeitig) entwöhnen wollen, so ist darin keine Sünde für sie (beide). Und wenn ihr eure Kinder (von einer Amme) stillen lassen wollt, so ist darin keine Sünde für euch, sofern ihr das, was ihr geben wollt, in rechtlicher Weise aushändigt. Und fürchtet Allah und wißt, daß Allah das, was ihr tut, wohl sieht!\qt@no{233}}%
\qt@newcmd\qurantrans@de@ccxli{Und diejenigen von euch, die abberufen werden und Gattinnen hinterlassen - so sollen diese (mit sich) selbst vier Monate und zehn (Tage) abwarten. Wenn sie dann ihre festgesetzte Zeit erreicht haben, so ist für euch keine Sünde in dem, was sie in rechtlicher Weise mit sich selbst unternehmen. Allah ist dessen, was ihr tut, Kundig.\qt@no{234}}%
\qt@newcmd\qurantrans@de@ccxlii{Und es ist für euch keine Sünde darin, daß ihr den Frauen Andeutungen auf einen Heiratsantrag macht, oder daß ihr etwas (derartiges) in euch hegt. Allah weiß, daß ihr an sie denken werdet. Aber trefft nicht heimlich eine Abmachung mit ihnen, außer ihr sagt geziemende Worte. Und schließt nicht den Ehebund, bevor die vorgeschriebene Frist ihre festgesetzte Zeit erreicht hat. Wißt, daß Allah weiß, was in eurem Innersten ist. So seht euch vor Ihm vor! Und wißt, daß Allah Allvergebend und Nachsichtig ist.\qt@no{235}}%
\qt@newcmd\qurantrans@de@ccxliii{Es ist für euch keine Sünde darin, wenn ihr euch von Frauen scheidet, solange ihr sie noch nicht berührt oder euch ihnen gegenüber (zu einer Morgengabe) verpflichtet habt. Doch gewährt ihnen eine Abfindung - der Wohlhabende entsprechend seinen Verhältnissen und der Geringbemittelte entsprechend seinen Verhältnissen -, eine Abfindung in rechtlicher Weise. (Dies ist) eine Pflicht für die Gutes Tuenden.\qt@no{236}}%
\qt@newcmd\qurantrans@de@ccxliv{Aber wenn ihr euch von ihnen scheidet, bevor ihr sie berührt und euch ihnen gegenüber schon (zu einer Morgengabe) verpflichtet habt, dann (händigt) die Hälfte dessen (aus), wozu ihr euch verpflichtet habt, es sei denn, daß sie (es) erlassen oder der, in dessen Hand der Ehebund ist. Und wenn ihr (es) erlaßt, kommt das der Gottesfurcht näher. Und versäumt es nicht, gut zueinander zu sein . Was ihr tut, sieht Allah wohl.\qt@no{237}}%
\qt@newcmd\qurantrans@de@ccxlv{Haltet die Gebete ein, und (besonders) das mittlere Gebet, und steht demütig ergeben vor Allah.\qt@no{238}}%
\qt@newcmd\qurantrans@de@ccxlvi{Wenn ihr in Furcht seid, dann (verrichtet das Gebet) zu Fuß oder im Reiten. Wenn ihr aber (wieder) in Sicherheit seid, dann gedenkt Allahs, wie Er euch gelehrt hat, was ihr nicht wußtet.\qt@no{239}}%
\qt@newcmd\qurantrans@de@ccxlvii{Diejenigen von euch, die abberufen werden und Gattinnen hinterlassen, sollen ihren Gattinnen eine Abfindung für ein Jahr vermachen, ohne daß sie aus (dem Haus) gewiesen werden. Wenn sie aber ausziehen, so liegt für euch keine Sünde in dem, was sie mit sich selbst an Geziemendem unternehmen. Allah ist Allmächtig und Allweise.\qt@no{240}}%
\qt@newcmd\qurantrans@de@ccxlviii{Und den geschiedenen Frauen steht eine Abfindung in rechtlicher Weise zu - eine Pflicht für die Gottesfürchtigen.\qt@no{241}}%
\qt@newcmd\qurantrans@de@ccxlix{So macht Allah euch Seine Zeichen klar, auf daß ihr begreifen möget.\qt@no{242}}%
\qt@newcmd\qurantrans@de@ccl{Siehst du nicht jene, die, um dem Tod zu entfliehen, zu Tausenden aus ihren Wohnstätten hinauszogen? Da sagte Allah zu ihnen: ``Sterbt!" Hierauf machte Er sie wieder lebendig. Allah ist wahrlich voll Huld gegen die Menschen. Aber die meisten Menschen sind nicht dankbar.\qt@no{243}}%
\qt@newcmd\qurantrans@de@ccli{Und kämpft auf Allahs Weg und wißt, daß Allah Allhörend und Allwissend ist.\qt@no{244}}%
\qt@newcmd\qurantrans@de@cclii{Wer ist es denn, der Allah ein schönes Darlehen gibt? So vermehrt Er es ihm um ein Vielfaches. Allah hält zurück und gewährt, und zu Ihm werdet ihr zurückgebracht.\qt@no{245}}%
\qt@newcmd\qurantrans@de@ccliii{Siehst du nicht die führende Schar von den Kindern Isra'ils nach Musa, als sie zu einem ihrer Propheten sagten: ``Setze einen König für uns ein, damit wir auf Allahs Weg kämpfen."? Er sagte: ``Werdet ihr vielleicht, wenn euch zu kämpfen vorgeschrieben ist, doch nicht kämpfen?" Sie sagten: ``Warum sollten wir nicht auf Allahs Weg kämpfen, wo wir doch aus unseren Wohnstätten und von unseren Söhnen vertrieben worden sind?" Doch als ihnen vorgeschrieben wurde zu kämpfen, kehrten sie sich - bis auf wenige von ihnen - ab. Und Allah weiß über die Ungerechten Bescheid.\qt@no{246}}%
\qt@newcmd\qurantrans@de@ccliv{Und ihr Prophet sagte zu ihnen: ``Allah hat euch (hiermit) Talut als König geschickt." Sie sagten: ``Wie sollte er die Herrschaft über uns haben, wo wir doch ein größeres Anrecht auf die Herrschaft haben, und ihm nicht Wohlstand gegeben ist?" Er sagte: ``Allah hat ihn vor euch auserwählt und ihm ein Übermaß an Wissen und körperlichen Vorzügen verliehen. Und Allah gibt Seine Herrschaft, wem Er will. Allah ist Allumfassend und Allwissend.."\qt@no{247}}%
\qt@newcmd\qurantrans@de@cclv{Und ihr Prophet sagte zu ihnen: ``Das Zeichen seiner Herrschaft ist, daß die Bundeslade zu euch kommen wird; in ihr ist innere Ruhe von eurem Herrn und ein Rest von dem, was die Sippe Musas und die Sippe Haruns hinterließen, getragen von Engeln. Darin soll wahrlich ein Zeichen für euch sein, wenn ihr gläubig seid."\qt@no{248}}%
\qt@newcmd\qurantrans@de@cclvi{Und als nun Talut mit den Heerscharen aufgebrochen war, sagte er: ``Allah wird euch mit einem Fluß prüfen. Wer davon trinkt, gehört nicht zu mir. Und wer nicht davon kostet, der gehört zu mir, außer demjenigen, der (nur) eine Handvoll schöpft." Da tranken sie davon - bis auf wenige von ihnen. Und als er ihn überschritten hatte, er und diejenigen, mit ihm glaubten, sagten sie: ``Wir haben heute keine Kraft gegen Galut und seine Heerscharen." Diejenigen aber, die glaubten, daß sie Allah begegnen würden, sagten: ``Wie so manch eine geringe Schar hat schon mit Allahs Erlaubnis eine große Schar besiegt! Allah ist mit den Standhaften."\qt@no{249}}%
\qt@newcmd\qurantrans@de@cclvii{Und als sie gegen Galut und seine Heerscharen auf dem Plan erschienen, sagten sie: ``Unser Herr, überschütte uns mit Standhaftigkeit, festige unsere Füße und verhilf uns zum Sieg über das ungläubige Volk!"\qt@no{250}}%
\qt@newcmd\qurantrans@de@cclviii{Und so schlugen sie sie mit Allahs Erlaubnis, und Dawud tötete Galut. Und Allah gab ihm die Herrschaft und die Weisheit und lehrte ihn von dem, was Er wollte. Und wenn nicht Allah die einen Menschen durch die anderen zurückweisen würde, geriete die Erde wahrlich ins Verderben. Aber Allah ist voll Huld gegen die Weltenbewohner.\qt@no{251}}%
\qt@newcmd\qurantrans@de@cclix{Dies sind Allahs Zeichen, die Wir dir der Wahrheit entsprechend verlesen. Und du bist wahrlich einer der Gesandten.\qt@no{252}}%
\qt@newcmd\qurantrans@de@cclx{Dies sind die Gesandten; einige von ihnen haben Wir vor anderen bevorzugt. Unter ihnen gibt es manche, zu denen Allah gesprochen hat, und einige, die Er um Rangstufen erhöht hat. Und 'Isa, dem Sohn Maryams, gaben Wir die klaren Beweise und stärkten ihn mit dem Heiligen Geist. Und wenn Allah gewollt hätte, hätten diejenigen nach ihnen nicht miteinander gekämpft, nachdem die klaren Beweise zu ihnen gekommen waren. Aber sie waren uneinig: Unter ihnen gab es manche, die glaubten und andere, die ungläubig waren. Und wenn Allah gewollt hätte, hätten sie nicht miteinander gekämpft. Doch Allah tut, was Er will.\qt@no{253}}%
\qt@newcmd\qurantrans@de@cclxi{O die ihr glaubt, gebt aus von dem, womit Wir euch versorgt haben, bevor ein Tag kommt, an dem es keinen Verkauf, keine Freundschaft und keine Fürsprache gibt! Die Ungläubigen sind die Ungerechten.\qt@no{254}}%
\qt@newcmd\qurantrans@de@cclxii{Allah - es gibt keinen Gott außer Ihm, dem Lebendigen und Beständigen. Ihn überkommt weder Schlummer noch Schlaf. Ihm gehört (alles), was in den Himmeln und was auf der Erde ist. Wer ist es denn, der bei Ihm Fürsprache einlegen könnte - außer mit Seiner Erlaubnis? Er weiß, was vor ihnen und was hinter ihnen liegt, sie aber umfassen nichts von Seinem Wissen - außer, was Er will. Sein Thronschemel umfaßt die Himmel und die Erde, und ihre Behütung beschwert Ihn nicht. Er ist der Erhabene und Allgewaltige.\qt@no{255}}%
\qt@newcmd\qurantrans@de@cclxiii{Es gibt keinen Zwang im Glauben. (Der Weg der) Besonnenheit ist nunmehr klar unterschieden von (dem der) Verirrung. Wer also falsche Götter verleugnet, jedoch an Allah glaubt, der hält sich an der festesten Handhabe, bei der es kein Zerreißen gibt. Und Allah ist Allhörend und Allwissend.\qt@no{256}}%
\qt@newcmd\qurantrans@de@cclxiv{Allah ist der Schutzherr derjenigen, die glauben. Er bringt sie aus den Finsternissen heraus ins Licht. Diejenigen aber, die ungläubig sind, deren Schutzherren sind die falschen Götter. Sie bringen sie aus dem Licht hinaus in die Finsternisse. Das sind Insassen des (Höllen)feuers. Ewig werden sie darin bleiben.\qt@no{257}}%
\qt@newcmd\qurantrans@de@cclxv{Siehst du nicht jenen, der mit Ibrahim über seinen Herrn stritt, weil Allah ihm die Herrschaft gegeben hatte? (Damals) als Ibrahim sagte: ``Mein Herr ist Derjenige, Der lebendig macht und sterben läßt." Er sagte: ``Ich mache lebendig und lasse sterben." Ibrahim sagte: ``Allah bringt ja die Sonne vom Osten her; so bringe du sie vom Westen her!" Da war derjenige, der ungläubig war, verblüfft. Und Allah leitet nicht das ungerechte Volk recht.\qt@no{258}}%
\qt@newcmd\qurantrans@de@cclxvi{Oder (kennst du nicht) einen ähnlichen, denjenigen, der an einer Stadt vorbeikam, die wüst in Trümmern lag? Er sagte: ``Wie sollte Allah diese (Stadt) wieder lebendig machen, nachdem sie ausgestorben ist?" Da ließ Allah ihn (für) hundert Jahre tot sein. Hierauf erweckte Er ihn und sagte: ``Wie lange hast du verweilt?" Er sagte: ``Ich verweilte einen Tag oder den Teil eines Tages." Er sagte: ``Nein! Vielmehr verweiltest du hundert Jahre. Nun schau deine Speise und dein Getränk an! Sie sind nicht verfault. Und schau deinen Esel an! - Und damit Wir dich zu einem Zeichen machen für die Menschen. Und schau die Knochen an, wie Wir sie zusammensetzen und sie hierauf mit Fleisch bekleiden!" Nachdem es ihm klar geworden war, sagte er: ``Ich weiß (jetzt), daß Allah zu allem die Macht hat."\qt@no{259}}%
\qt@newcmd\qurantrans@de@cclxvii{Und als Ibrahim sagte: ``Mein Herr, zeige mir, wie Du die Toten lebendig machst!" Er sagte: ``Glaubst du immer noch nicht?" Er sagte: ``Doch, aber (ich frage,) damit mein Herz Ruhe findet." Er (Allah) sagte: ``So nimm vier von den Vögeln und zieh sie dann her zu dir. Hierauf setze auf jeden Berg einen Teil von ihnen. Hierauf rufe sie, so werden sie zu dir herbeigeeilt kommen. Und wisse, daß Allah Allmächtig und Allweise ist."\qt@no{260}}%
\qt@newcmd\qurantrans@de@cclxviii{Das Gleichnis derjenigen, die ihren Besitz auf Allahs Weg ausgeben, ist das eines Saatkorns, das sieben Ähren wachsen läßt, (und) in jeder Ähre hundert Körner. Allah vervielfacht, wem Er will. Und Allah ist Allumfassend und Allwissend.\qt@no{261}}%
\qt@newcmd\qurantrans@de@cclxix{Diejenigen, die ihren Besitz auf Allahs Weg ausgeben und hierauf dem, was sie ausgegeben haben, weder Vorhaltungen noch Beleidigungen nachfolgen lassen, die haben ihren Lohn bei ihrem Herrn, und keine Furcht soll über sie kommen, noch sollen sie traurig sein.\qt@no{262}}%
\qt@newcmd\qurantrans@de@cclxx{Freundliche Worte und Vergebung sind besser als ein Almosen, dem Beleidigungen nachfolgen. Allah ist Unbedürftig und Nachsichtig.\qt@no{263}}%
\qt@newcmd\qurantrans@de@cclxxi{O die ihr glaubt, macht nicht eure Almosen durch Vorhaltungen und Beleidigungen zunichte, wie derjenige, der seinen Besitz aus Augendienerei vor den Menschen ausgibt und nicht an Allah und den Jüngsten Tag glaubt! So ist sein Gleichnis das eines glatten Steins mit Erdreich darüber: Ein heftiger Regenguß trifft ihn und läßt ihn nackt. Sie haben keine Macht über etwas von dem, was sie erworben haben. Allah leitet das ungläubige Volk nicht recht.\qt@no{264}}%
\qt@newcmd\qurantrans@de@cclxxii{Das Gleichnis jener aber, die ihren Besitz im Trachten nach Allahs Zufriedenheit und aus ihrer Gewißheit ausgeben, ist das eines Gartens auf einer Anhöhe, den ein heftiger Regenguß trifft, und da bringt er seinen Ernteertrag zweifach (hervor). Und wenn ihn kein heftiger Regenguß trifft, so doch Sprühregen. Und was ihr tut, sieht Allah wohl.\qt@no{265}}%
\qt@newcmd\qurantrans@de@cclxxiii{Möchte einer von euch gern, daß er einen Garten habe mit Palmen und Rebstöcken, durcheilt von Bächen, in dem er von allen Früchten hat, während ihn (schon) hohes Alter getroffen hat und er (noch) schwache Nachkommenschaft hat, und daß ihn (den Garten) dann ein glühend heißer Wirbelwind treffe und er hierauf verbrenne? So macht Allah euch die Zeichen klar, auf daß ihr nachdenken möget!\qt@no{266}}%
\qt@newcmd\qurantrans@de@cclxxiv{O die ihr glaubt, gebt aus von den guten Dingen aus eurem Erworbenen und von dem, was Wir für euch aus der Erde hervorgebracht haben. Und sucht nicht zum Ausgeben das Schlechte davon aus, während ihr (selbst) es nicht nehmen würdet, ohne dabei ein Auge zuzudrücken. Und wißt, daß Allah Unbedürftig und Lobenswürdig ist.\qt@no{267}}%
\qt@newcmd\qurantrans@de@cclxxv{Der Satan verspricht euch Armut und befiehlt euch Schändliches. Allah aber verspricht euch Vergebung von Sich aus und Huld. Allah ist Allumfassend und Allwissend.\qt@no{268}}%
\qt@newcmd\qurantrans@de@cclxxvi{Er gibt Weisheit, wem Er will; und wem Weisheit gegeben wurde, dem wurde da viel Gutes gegeben. Aber nur diejenigen bedenken, die Verstand besitzen.\qt@no{269}}%
\qt@newcmd\qurantrans@de@cclxxvii{Und was immer ihr an Spenden ausgebt oder als Gelübde gelobt, Allah weiß es. Und die Ungerechten werden keine Helfer haben.\qt@no{270}}%
\qt@newcmd\qurantrans@de@cclxxviii{Wenn ihr Almosen offen zeigt, so ist es trefflich. Wenn ihr sie aber verbergt und den Armen gebt, so ist es besser für euch, und Er (Allah) wird etwas von euren bösen Taten tilgen. Allah ist dessen, was ihr tut, Kundig.\qt@no{271}}%
\qt@newcmd\qurantrans@de@cclxxix{Nicht dir obliegt ihre Rechtleitung, sondern Allah leitet recht, wen Er will. Was immer ihr an Gutem ausgebt, (das) ist für euch selbst. Und ihr gebt nur im Trachten nach Allahs Angesicht aus. Und was immer ihr an Gutem ausgebt, wird euch in vollem Maß zukommen, und es wird euch kein Unrecht zugefügt.\qt@no{272}}%
\qt@newcmd\qurantrans@de@cclxxx{Gebt am besten aus) für die Armen, die auf Allahs Weg daran gehindert werden, im Lande umherreisen zu können. Der Unwissende hält sie wegen ihrer Zurückhaltung für unbedürftig. Du erkennst sie an ihrem Merkmal: Sie betteln die Menschen nicht aufdringlich an. Und was immer ihr an Gutem ausgebt, so weiß Allah darüber Bescheid.\qt@no{273}}%
\qt@newcmd\qurantrans@de@cclxxxi{Diejenigen, die ihren Besitz bei Nacht und Tag, heimlich oder öffentlich ausgeben, haben ihren Lohn bei ihrem Herrn, und keine Furcht soll sie überkommen, noch werden sie traurig sein.\qt@no{274}}%
\qt@newcmd\qurantrans@de@cclxxxii{Diejenigen, die Zins verschlingen, werden nicht anders aufstehen als jemand, den der Satan durch Wahnsinn hin und her schlägt. Dies (wird sein), weil sie sagten: ``Verkaufen ist das gleiche wie Zinsnehmen." Doch hat Allah Verkaufen erlaubt und Zinsnehmen verboten. Zu wem nun eine Ermahnung von seinem Herrn kommt, und der dann aufhört, dem soll gehören, was vergangen ist, und seine Angelegenheit steht bei Allah. Wer aber rückfällig wird, jene sind Insassen des (Höllen)feuers. Ewig werden sie darin bleiben.\qt@no{275}}%
\qt@newcmd\qurantrans@de@cclxxxiii{Dahinschwinden lassen wird Allah den Zins und vermehren die Almosen. Allah liebt niemanden, der ein beharrlicher Ungläubiger und Sünder ist.\qt@no{276}}%
\qt@newcmd\qurantrans@de@cclxxxiv{Gewiß, diejenigen, die glauben und rechtschaffene Werke tun, das Gebet verrichten und die Abgabe entrichten, die haben ihren Lohn bei ihrem Herrn, und keine Furcht soll sie überkommen, noch werden sie traurig sein.\qt@no{277}}%
\qt@newcmd\qurantrans@de@cclxxxv{O die ihr glaubt, fürchtet Allah und laßt das sein, was an Zins(geschäften) noch übrig ist, wenn ihr gläubig seid.\qt@no{278}}%
\qt@newcmd\qurantrans@de@cclxxxvi{Wenn ihr es aber nicht tut, dann laßt euch Krieg von Allah und Seinem Gesandten ansagen! Doch wenn ihr bereut, dann steht euch euer (ausgeliehenes) Grundvermögen zu; (so) tut weder ihr Unrecht, noch wird euch Unrecht zugefügt.\qt@no{279}}%
\qt@newcmd\qurantrans@de@cclxxxvii{Und wenn er (der Schuldner) in Schwierigkeiten ist, dann sei (ihm) Aufschub (gewährt,) bis eine Erleichterung (eintritt). Und daß ihr (es) als Almosen erlaßt, ist besser für euch, wenn ihr (es) nur wißt.\qt@no{280}}%
\qt@newcmd\qurantrans@de@cclxxxviii{Und hütet euch vor einem Tag, an dem ihr zu Allah zurückgebracht werdet. Dann wird jeder Seele in vollem Maß zukommen, was sie verdient hat, und es wird ihnen kein Unrecht zugefügt.\qt@no{281}}%
\qt@newcmd\qurantrans@de@cclxxxix{O die ihr glaubt, wenn ihr auf eine festgesetzte Frist, einer vom anderen, eine Geldschuld aufnehmt, dann schreibt es auf. Und ein Schreiber soll (es) für euch gerecht aufschreiben. Und kein Schreiber soll sich weigern zu schreiben, so wie Allah (es) ihn gelehrt hat. So soll er denn schreiben, und diktieren soll der Schuldner, und er soll Allah, seinen Herrn, fürchten und nichts davon schmälem. Wenn aber der Schuldner töricht oder schwach ist oder unfähig, selbst zu diktieren, so soll sein Sachwalter (es) gerecht diktieren. Und bringt zwei Männer von euch als Zeugen. Wenn es keine zwei Männer sein (können), dann sollen es ein Mann und zwei Frauen sein, mit denen als Zeugen ihr zufrieden seid, - damit, wenn eine von beiden sich irrt, eine die andere erinnere. Und die Zeugen sollen sich nicht weigern, wenn sie aufgefordert werden. Und seid nicht abgeneigt, es - (seien es) klein(e) oder groß(e Beträge) - mit seiner (vereinbarten) Frist aufzuschreiben! Das ist gerechter vor Allah und richtiger für das Zeugnis und eher geeignet, daß ihr nicht zweifelt; es sei denn, es ist ein sofortiger Handel, den ihr unter euch tätigt. Dann ist es keine Sünde für euch, wenn ihr es nicht aufschreibt. Und nehmt Zeugen, wenn ihr untereinander einen Verkauf abschließt. Und kein Schreiber oder Zeuge soll zu Schaden kommen. Wenn ihr (es) aber (dennoch) tut, so ist es ein Frevel von euch. Und fürchtet Allah! Und Allah lehrt euch. Allah weiß über alles Bescheid.\qt@no{282}}%
\qt@newcmd\qurantrans@de@ccxc{Und wenn ihr auf einer Reise seid und keinen Schreiber findet, dann sollen Pfänder in Empfang genommen werden. Und wenn dann einer von euch dem anderen (etwas) anvertraut, so soll derjenige, dem (es) anvertraut wurde, das ihm anvertraute Pfand (wieder) aushändigen, und er soll Allah, seinen Herrn, fürchten. Und verheimlicht kein Zeugnis. Wer es aber verheimlicht, dessen Herz ist gewiß sündhaft. Und Allah weiß über das, was ihr tut, Bescheid.\qt@no{283}}%
\qt@newcmd\qurantrans@de@ccxci{Allah gehört (alles), was in den Himmeln und was auf der Erde ist. Und ob ihr offenlegt, was in euch selbst ist, oder es verbergt, Allah wird euch dafür zur Rechenschaft ziehen. Dann vergibt Er, wem Er will, und straft, wen Er will. Und Allah hat zu allem die Macht.\qt@no{284}}%
\qt@newcmd\qurantrans@de@ccxcii{Der Gesandte (Allahs) glaubt an das, was zu ihm von seinem Herrn (als Offenbarung) herabgesandt worden ist, und ebenso die Gläubigen; alle glauben an Allah, Seine Engel, Seine Bücher und Seine Gesandten - Wir machen keinen Unterschied bei jemandem von Seinen Gesandten. Und sie sagen: ``Wir hören und gehorchen. (Gewähre uns) Deine Vergebung, unser Herr! Und zu Dir ist der Ausgang."\qt@no{285}}%
\qt@newcmd\qurantrans@de@ccxciii{Allah erlegt keiner Seele mehr auf, als sie zu leisten vermag. Ihr kommt (nur) zu, was sie verdient hat, und angelastet wird ihr (nur), was sie verdient hat. ``Unser Herr, belange uns nicht, wenn wir (etwas) vergessen oder einen Fehler begehen. Unser Herr, lege uns keine Bürde auf, wie Du sie denjenigen vor uns auferlegt hast. Unser Herr, bürde uns nichts auf, wozu wir keine Kraft haben. Verzeihe uns, vergib uns und erbarme Dich unser! Du bist unser Schutzherr. So verhilf uns zum Sieg über das ungläubige Volk!\qt@no{286}}%
\qt@newcmd\qurantrans@de@ccxciv{\basmalahde Alif-Lam-Mim\qt@no{1}}%
\qt@newcmd\qurantrans@de@ccxcv{Allah - es gibt keinen Gott außer Ihm, dem Lebendigen und Beständigen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@ccxcvi{Er hat dir das Buch mit der Wahrheit offenbart, das zu bestätigen, was vor ihm (offenbart) war. Und Er hat (auch) die Tora und das Evangelium (als Offenbarung) herabgesandt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@ccxcvii{zuvor, als Rechtleitung für die Menschen. Und Er hat die Unterscheidung herabgesandt. Gewiß, diejenigen, die Allahs Zeichen verleugnen, für sie wird es strenge Strafe geben. Allah ist Allmächtig und Besitzer von Vergeltungsgewalt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@ccxcviii{Vor Allah ist nichts verborgen, weder auf der Erde noch im Himmel.\qt@no{5}}%
\qt@newcmd\qurantrans@de@ccxcix{Er ist es, Der euch im Mutterleib gestaltet, wie Er will. Es gibt keinen Gott außer Ihm, dem Allmächtigen und Allweisen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@ccc{Er ist es, Der das Buch (als Offenbarung) auf dich herabgesandt hat. Dazu gehören eindeutige Verse - sie sind der Kern des Buches - und andere, mehrdeutige. Was aber diejenigen angeht, in deren Herzen (Neigung zum) Abschweifen ist, so folgen sie dem, was davon mehrdeutig ist, im Trachten nach Irreführung und im Trachten nach ihrer Mißdeutung. Aber niemand weiß ihre Deutung außer Allah. Und diejenigen, die im Wissen fest gegründet sind, sagen: ``Wir glauben daran; alles ist von unserem Herrn." Aber nur diejenigen bedenken, die Verstand besitzen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@ccci{Unser Herr, lasse unsere Herzen nicht abschweifen, nachdem Du uns rechtgeleitet hast, und schenke uns Erbarmen von Dir aus. Du bist ja der unablässig Schenkende.\qt@no{8}}%
\qt@newcmd\qurantrans@de@cccii{Unser Herr, Du wirst die Menschen gewiß zu einem Tag versammeln, an dem es keinen Zweifel gibt. Allah bricht nicht, was Er versprochen hat.\qt@no{9}}%
\qt@newcmd\qurantrans@de@ccciii{Gewiß, denjenigen, die ungläubig sind, werden weder ihr Besitz noch ihre Kinder vor Allah etwas nützen. Sie sind es, die Brennstoff des (Höllen)feuers sein werden.\qt@no{10}}%
\qt@newcmd\qurantrans@de@ccciv{Nach der Art der Leute Fir'auns und derjenigen vor ihnen: Sie erklärten Unsere Zeichen für Lüge, und da ergriff sie Allah für ihre Sünden. Und Allah ist streng im Bestrafen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@cccv{Sag zu denjenigen, die ungläubig sind: Ihr werdet besiegt und zur Hölle zusammengetrieben werden - eine schlimme Lagerstatt!\qt@no{12}}%
\qt@newcmd\qurantrans@de@cccvi{Ihr hattet ja ein Zeichen in zwei Scharen, die aufeinandertrafen: Die eine kämpfte auf Allahs Weg und eine andere, ungläubige. Sie sahen sie dem Augenschein nach zweimal so viel, wie sie (selbst waren). Und Allah stärkt mit Seiner Hilfe, wen Er will. Darin ist wahrlich eine Lehre für diejenigen, die Einsicht besitzen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@cccvii{Ausgeschmückt ist den Menschen die Liebe zu den Begierden, nach Frauen, Söhnen, aufgehäuften Mengen von Gold und Silber, Rassepferden, Vieh und Saatfeldern. Das ist der Genuß im diesseitigen Leben. Doch bei Allah ist die schöne Heimstatt.\qt@no{14}}%
\qt@newcmd\qurantrans@de@cccviii{Sag: Soll ich euch von etwas Besserem als diesem Kunde geben? Für diejenigen, die gottesfürchtig sind, werden bei ihrem Herrn Gärten sein, durcheilt von Bächen, ewig darin zu bleiben, und vollkommen gereinigte Gattinnen und Wohlgefallen von Allah. Allah sieht die Menschen wohl,\qt@no{15}}%
\qt@newcmd\qurantrans@de@cccix{die sagen: ``Unser Herr, gewiß, wir glauben. Darum vergib uns unsere Sünden und bewahre uns vor der Strafe des (Höllen)feuers."\qt@no{16}}%
\qt@newcmd\qurantrans@de@cccx{Die Standhaften und die Wahrhaftigen, die demütig Ergebenen und diejenigen, die ausgeben, und die im letzten Teil der Nacht um Vergebung Bittenden.\qt@no{17}}%
\qt@newcmd\qurantrans@de@cccxi{Allah bezeugt, daß es keinen Gott gibt außer Ihm; und (ebenso bezeugen) die Engel und diejenigen, die Wissen besitzen; der Wahrer der Gerechtigkeit. Es gibt keinen Gott außer Ihm, dem Allmächtigen und Allweisen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@cccxii{Gewiß, die Religion ist bei Allah der Islam. Doch diejenigen, denen die Schrift gegeben wurde, wurden erst uneinig, nachdem das Wissen zu ihnen gekommen war - aus Mißgunst untereinander, Doch wer Allahs Zeichen verleugnet -, so ist Allah schnell im Abrechnen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@cccxiii{Und wenn sie mit dir streiten (wollen), dann sag: ``Ich habe mein Gesicht Allah ergeben, und (ebenso,) wer mir folgt!" Und sag zu jenen, denen die Schrift gegeben wurde, und den Schriftunkundigen: ``Seid ihr (Allah) ergeben?" Wenn sie (Ihm) ergeben sind, dann sind sie rechtgeleitet. Kehren sie sich aber ab, so obliegt dir nur die Übermittelung (der Botschaft). Allah sieht die Menschen wohl.\qt@no{20}}%
\qt@newcmd\qurantrans@de@cccxiv{Diejenigen, die Allahs Zeichen verleugnen, die Propheten zu Unrecht töten und diejenigen unter den Menschen töten, die Gerechtigkeit befehlen, denen verkünde schmerzhafte Strafe.\qt@no{21}}%
\qt@newcmd\qurantrans@de@cccxv{Das sind diejenigen, deren Werke im Diesseits und Jenseits hinfällig werden. Und sie haben keine Helfer.\qt@no{22}}%
\qt@newcmd\qurantrans@de@cccxvi{Siehst du nicht jene, denen ein Teil der Schrift gegeben wurde? Sie werden zum Buch Allahs aufgefordert, damit es zwischen ihnen richte. Hierauf kehrt sich eine Gruppe von ihnen ab, und sie sind Widerstrebende.\qt@no{23}}%
\qt@newcmd\qurantrans@de@cccxvii{Dies, weil sie sagen: ``Das (Höllen)feuer wird uns nur für eine bestimmte Anzahl von Tagen berühren." Was sie (an Lügen) zu ersinnen pflegten, hat sie über ihre Religion getäuscht.\qt@no{24}}%
\qt@newcmd\qurantrans@de@cccxviii{Aber wie (wird es sein), wenn Wir sie zu einem Tag versammeln, an dem es keinen Zweifel gibt, und jeder Seele in vollem Maß zukommt, was sie verdient hat, und ihnen kein Unrecht zugefügt wird?\qt@no{25}}%
\qt@newcmd\qurantrans@de@cccxix{Sag: O Allah, Herr der Herrschaft, Du gibst die Herrschaft, wem Du willst, und Du entziehst die Herrschaft, wem Du willst. Du machst mächtig, wen Du willst, und Du erniedrigst, wen Du willst. In Deiner Hand ist (all) das Gute. Gewiß, Du hast zu allem die Macht.\qt@no{26}}%
\qt@newcmd\qurantrans@de@cccxx{Du läßt die Nacht in den Tag eindringen und läßt den Tag eindringen in die Nacht. Und Du läßt das Lebendige aus dem Toten hervorgehen und läßt hervorgehen das Tote aus dem Lebendigen. Und Du versorgst, wen Du willst, ohne zu berechnen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@cccxxi{Die Gläubigen sollen nicht die Ungläubigen anstatt der Gläubigen zu Schutzherren nehmen. Wer das tut, hat überhaupt nichts mit Allah (gemeinsam) -, es sei denn, daß ihr euch (durch dieses Verhalten) vor ihnen wirklich schützt. Allah mahnt euch zur Vorsicht vor Sich selbst. Und zu Allah ist der Ausgang.\qt@no{28}}%
\qt@newcmd\qurantrans@de@cccxxii{Sag: Ob ihr verbergt, was in euren Brüsten ist oder es offenlegt, Allah weiß es; Er weiß, was in den Himmeln und was auf der Erde ist. Und Allah hat zu allem die Macht.\qt@no{29}}%
\qt@newcmd\qurantrans@de@cccxxiii{An dem Tag wird jede Seele das, was sie an Gutem getan hat, bereit finden. Und von dem, was sie an Bösem getan hat, hätte sie gern, wenn zwischen ihr und ihm ein weiter Abstand wäre. Und Allah mahnt euch zur Vorsicht vor Sich selbst. Allah ist Gnädig zu den Menschen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@cccxxiv{Sag: Wenn ihr Allah liebt, dann folgt mir. So liebt euch Allah und vergibt euch eure Sünden. Allah ist Allvergebend und Barmherzig.\qt@no{31}}%
\qt@newcmd\qurantrans@de@cccxxv{Sag: Gehorcht Allah und dem Gesandten. Doch wenn sie sich abkehren, so liebt Allah die Ungläubigen nicht.\qt@no{32}}%
\qt@newcmd\qurantrans@de@cccxxvi{Gewiß, Allah hat Adam und Nuh und die Sippe Ibrahims und die Sippe 'Imrans vor den (anderen) Weltenbewohnern auserwählt,\qt@no{33}}%
\qt@newcmd\qurantrans@de@cccxxvii{eine der anderen Nachkommenschaft. Und Allah ist Allhörend und Allwissend.\qt@no{34}}%
\qt@newcmd\qurantrans@de@cccxxviii{Als 'Imrans Frau sagte: ``Mein Herr, ich gelobe Dir, was in meinem Mutterleib ist, für Deinen Dienst freigestellt. So nimm (es) von mir an! Du bist ja der Allhörende und Allwissende."\qt@no{35}}%
\qt@newcmd\qurantrans@de@cccxxix{Als sie sie dann zur Welt gebracht hatte, sagte sie: ``Mein Herr, ich habe ein Mädchen zur Welt gebracht." Und Allah wußte sehr wohl, was sie zur Welt gebracht hatte, und der Knabe ist nicht wie das Mädchen. ``Ich habe sie Maryam genannt, und ich stelle sie und ihre Nachkommenschaft unter Deinen Schutz vor dem gesteinigten Satan."\qt@no{36}}%
\qt@newcmd\qurantrans@de@cccxxx{Da nahm ihr Herr sie auf gütigste Art an und ließ sie auf schöne Weise heranwachsen und gab sie Zakariyya zur Betreuung. Jedesmal, wenn Zakariyya zu ihr in die Zelle trat, fand er bei ihr Versorgung. Er sagte: ``O Maryam, woher hast du das?" Sie sagte: ``Es kommt von Allah; Allah versorgt, wen Er will, ohne zu berechnen."\qt@no{37}}%
\qt@newcmd\qurantrans@de@cccxxxi{Da rief Zakariyya seinen Herrn an und sagte: ``Mein Herr, schenke mir von Dir aus gute Nachkommenschaft! Du bist ja der Gebetserhörer.\qt@no{38}}%
\qt@newcmd\qurantrans@de@cccxxxii{Und da riefen ihm die Engel zu, während er betend in der Zelle stand: ``Allah verkündet dir Yahya, ein Wort von Allah zu bestätigen, einen Herrn, einen Keuschen und Propheten von den Rechtschaffenen."\qt@no{39}}%
\qt@newcmd\qurantrans@de@cccxxxiii{Er sagte: ``Mein Herr, wie soll ich einen Jungen bekommen, wo mich schon hohes Alter überkommen hat und meine Frau unfruchtbar ist?" Er (, der Engel,) sagte: ``So (wird es sein); Allah tut, was Er will."\qt@no{40}}%
\qt@newcmd\qurantrans@de@cccxxxiv{Er (Zakariyya) sagte: ``Mein Herr, setze mir ein Zeichen." Er sagte: ``Dein Zeichen ist, daß du drei Tage lang nicht zu den Menschen sprechen wirst außer durch Gebärden. Und gedenke deines Herrn häufig und preise (Ihn) am Abend und am frühen Morgen!"\qt@no{41}}%
\qt@newcmd\qurantrans@de@cccxxxv{Und als die Engel sagten: ``O Maryam, Allah hat dich auserwählt und dich rein gemacht und dich auserwählt vor den Frauen der (anderen) Weltenbewohner!\qt@no{42}}%
\qt@newcmd\qurantrans@de@cccxxxvi{O Maryam, sei deinem Herrn demütig ergeben, wirf dich nieder und verbeuge dich zusammen mit den sich Verbeugenden."\qt@no{43}}%
\qt@newcmd\qurantrans@de@cccxxxvii{Dies gehört zu den Nachrichten vom Verborgenen, das Wir dir (als Offenbarung) eingeben. Denn du warst nicht bei ihnen, als sie ihre Rohre warfen (, um durch das Los zu bestimmen), wer von ihnen Maryam betreuen sollte. Und du warst nicht bei ihnen, als sie miteinander stritten.\qt@no{44}}%
\qt@newcmd\qurantrans@de@cccxxxviii{Als die Engel sagten: ``O Maryam, Allah verkündet dir ein Wort von Ihm, dessen Name al-Masih 'Isa, der Sohn Maryams ist, angesehen im Diesseits und Jenseits und einer der (Allah) Nahegestellten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@cccxxxix{Und er wird in der Wiege zu den Menschen sprechen und im Mannesalter und einer der Rechtschaffenen sein."\qt@no{46}}%
\qt@newcmd\qurantrans@de@cccxl{Sie sagte: ``Mein Herr, wie sollte ich ein Kind haben, wo mich (doch) kein menschliches Wesen berührt hat?" Er (, der Engel,) sagte: ``So (wird es sein); Allah erschafft, was Er will. Wenn Er eine Angelegenheit bestimmt, so sagt Er zu ihr nur: ,Sei! und so ist sie.\qt@no{47}}%
\qt@newcmd\qurantrans@de@cccxli{Und Er wird ihn die Schrift, die Weisheit, die Tora und das Evangelium lehren.\qt@no{48}}%
\qt@newcmd\qurantrans@de@cccxlii{Und (Er wird ihn schicken) als einen Gesandten zu den Kindern Isra'ils (, zu denen er sagen wird): ,Gewiß, ich bin ja mit einem Zeichen von eurem Herrn zu euch gekommen: daß ich euch aus Lehm (etwas) schaffe, (was so aussieht) wie die Gestalt eines Vogels, und dann werde ich ihm einhauchen, und da wird es ein (wirklicher) Vogel sein. Und ich werde mit Allahs Erlaubnis den Blindgeborenen und den Weißgefleckten heilen und werde Tote mit Allahs Erlaubnis wieder lebendig machen. Und ich werde euch kundtun, was ihr eßt und was ihr in euren Häusern aufspeichert. Darin ist wahrlich ein Zeichen für euch, wenn ihr gläubig seid.\qt@no{49}}%
\qt@newcmd\qurantrans@de@cccxliii{Und das zu bestätigen, was von der Tora vor mir (offenbart) war, und um euch einiges von dem zu erlauben, was euch verboten war. Und ich bin mit einem Zeichen von eurem Herrn zu euch gekommen; so fürchtet Allah und gehorcht mir!\qt@no{50}}%
\qt@newcmd\qurantrans@de@cccxliv{Gewiß, Allah ist mein Herr und euer Herr; so dient Ihm. Das ist ein gerader Weg."\qt@no{51}}%
\qt@newcmd\qurantrans@de@cccxlv{Als 'Isa aber ihren Unglauben wahrnahm, sagte er: ``Wer sind meine Helfer (auf dem Weg hin) zu Allah?" Die Jünger sagten: ``Wir sind Allahs Helfer. Wir glauben an Allah, und bezeuge, daß wir (Ihm) ergeben sind!\qt@no{52}}%
\qt@newcmd\qurantrans@de@cccxlvi{Unser Herr, wir glauben an das, was Du (als Offenbarung) herabgesandt hast, und folgen dem Gesandten. So schreibe uns auf unter die Zeugnis Ablegenden!"\qt@no{53}}%
\qt@newcmd\qurantrans@de@cccxlvii{Und sie schmiedeten Ränke, und (auch) Allah schmiedete Ränke; und Allah ist der beste Ränkeschmied.\qt@no{54}}%
\qt@newcmd\qurantrans@de@cccxlviii{Als Allah sagte: ``O 'Isa, Ich werde dich (nunmehr) abberufen und dich zu mir emporheben und dich von denen, die ungläubig sind, reinigen und diejenigen, die dir folgen, bis zum Tag der Auferstehung über diejenigen stellen, die ungläubig sind. Hierauf wird eure Rückkehr zu Mir sein, und dann werde Ich zwischen euch richten über das, worüber ihr uneinig zu sein pflegtet.\qt@no{55}}%
\qt@newcmd\qurantrans@de@cccxlix{Was nun diejenigen angeht, die ungläubig sind, so werde Ich sie im Diesseits und im Jenseits mit strenger Strafe strafen; und sie werden keine Helfer haben.\qt@no{56}}%
\qt@newcmd\qurantrans@de@cccl{Was aber jene angeht, die glauben und rechtschaffene Werke tun, so wird Er ihnen ihren Lohn in vollem Maß zukommen lassen. Und Allah liebt nicht die Ungerechten."\qt@no{57}}%
\qt@newcmd\qurantrans@de@cccli{Dies verlesen Wir dir von den Zeichen und den weisen Sprüchen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@ccclii{Gewiß, das Gleichnis 'Isas ist bei Allah wie das Gleichnis Adams. Er erschuf ihn aus Erde. Hierauf sagte Er zu ihm: ``Sei!" und da war er.\qt@no{59}}%
\qt@newcmd\qurantrans@de@cccliii{Das ist) die Wahrheit von deinem Herrn; gehöre daher nicht zu den Zweiflern!\qt@no{60}}%
\qt@newcmd\qurantrans@de@cccliv{Wer nun mit dir darüber streitet, nach dem, was dir an Wissen zugekommen ist, so sag (zu denen): ``Kommt her! Laßt uns unsere Söhne und eure Söhne, unsere Frauen und eure Frauen, uns selbst und euch selbst zusammenrufen und hierauf flehen und so den Fluch Allahs über die Lügner kommen lassen!"\qt@no{61}}%
\qt@newcmd\qurantrans@de@ccclv{Gewiß, dies sind die wahren Geschichten. Und keinen Gott gibt es außer Allah. Allah - Er ist wahrlich der Allmächtige und Allweise.\qt@no{62}}%
\qt@newcmd\qurantrans@de@ccclvi{Doch wenn sie sich abkehren, so weiß Allah sehr wohl über die Unheilstifter Bescheid.\qt@no{63}}%
\qt@newcmd\qurantrans@de@ccclvii{Sag: O Leute der Schrift, kommt her zu einem zwischen uns und euch gleichen Wort: daß wir niemandem dienen außer Allah und Ihm nichts beigesellen und sich nicht die einen von uns die anderen zu Herren außer Allah nehmen. Doch wenn sie sich abkehren, dann sagt: Bezeugt, daß wir (Allah) ergeben sind.\qt@no{64}}%
\qt@newcmd\qurantrans@de@ccclviii{O Leute der Schrift, warum streitet ihr über Ibrahim, wo die Tora und das Evangelium erst nach ihm (als Offenbarung) herabgesandt worden sind? Begreift ihr denn nicht?\qt@no{65}}%
\qt@newcmd\qurantrans@de@ccclix{Ihr da seid es doch, die ihr über etwas gestritten habt, wovon ihr Wissen habt; warum streitet ihr nun aber über etwas, wovon ihr kein Wissen habt? Allah weiß, ihr aber wißt nicht.\qt@no{66}}%
\qt@newcmd\qurantrans@de@ccclx{Ibrahim war weder ein Jude noch ein Christ, sondern er war Anhänger des rechten Glaubens, einer, der sich Allah ergeben hat, und er gehörte nicht zu den Götzendienern.\qt@no{67}}%
\qt@newcmd\qurantrans@de@ccclxi{Die Menschen, die Ibrahim am nächsten stehen, sind wahrlich diejenigen, die ihm folgten, sowie dieser Prophet und die, die (mit ihm) glauben. Und Allah ist der Schutzherr der Gläubigen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@ccclxii{Gern möchte euch ein Teil von den Leuten der Schrift in die Irre führen. Aber sie führen nur sich selbst in die Irre, ohne (es) zu merken.\qt@no{69}}%
\qt@newcmd\qurantrans@de@ccclxiii{O Leute der Schrift! Warum verleugnet ihr Allahs Zeichen, wo ihr doch (selbst) Zeugen seid?\qt@no{70}}%
\qt@newcmd\qurantrans@de@ccclxiv{O Leute der Schrift, warum verdeckt ihr das Wahre durch das Falsche und verbergt wissentlich die Wahrheit?\qt@no{71}}%
\qt@newcmd\qurantrans@de@ccclxv{Und ein Teil von den Leuten der Schrift sagt: ``Glaubt an das, was auf diejenigen, die glauben, (als Offenbarung) herabgesandt worden ist, am Anfang des Tages und verleugnet es (wieder) an seinem Ende, auf daß sie umkehren mögen!\qt@no{72}}%
\qt@newcmd\qurantrans@de@ccclxvi{Und glaubt nur jemandem, der eurer Religion folgt. Sag: Gewiß, die (wahre) Rechtleitung ist Allahs Rechtleitung. (Und glaubt nicht,) daß (auch) jemandem (anderen) das gleiche gegeben werde, was euch gegeben wurde, oder daß man mit euch (zu Recht) vor eurem Herrn streite. Sag: Gewiß, die Huld liegt in Allahs Hand. Er gewährt sie, wem Er will. Und Allah ist Allumfassend und Allwissend.\qt@no{73}}%
\qt@newcmd\qurantrans@de@ccclxvii{Er zeichnet mit Seinem Erbarmen aus, wen Er will, und Allah besitzt große Huld.\qt@no{74}}%
\qt@newcmd\qurantrans@de@ccclxviii{Unter den Leuten der Schrift gibt es manche, die, wenn du ihnen einen Qintar anvertraust, ihn dir (wieder) aushändigen. Es gibt unter ihnen aber auch manche, die, wenn du ihnen (nur) einen Dinar anvertraust, ihn dir nicht (wieder) aushändigen, es sei denn, du bist ständig hinter ihnen her. Dies, weil sie sagen: ``Gegen uns kann man der Schriftunkundigen wegen nicht vorgehen". Und sie sprechen (damit) wissentlich eine Lüge gegen Allah aus.\qt@no{75}}%
\qt@newcmd\qurantrans@de@ccclxix{Aber nein! Wer seinen Bund hält und gottesfürchtig ist, - gewiß, Allah liebt die Gottes fürchtigen.\qt@no{76}}%
\qt@newcmd\qurantrans@de@ccclxx{Diejenigen, die ihren Bund mit Allah und ihre Eide für einen geringen Preis verkaufen, haben am Jenseits keinen Anteil. Und Allah wird am Tag der Auferstehung weder zu ihnen sprechen noch sie anschauen noch sie läutern. Für sie wird es schmerzhafte Strafe geben.\qt@no{77}}%
\qt@newcmd\qurantrans@de@ccclxxi{Und wahrlich, eine Gruppe von ihnen verdreht mit seinen Zungen die Schrift, damit ihr es für zur Schrift gehörig haltet, während es nicht zur Schrift gehört. Und sie sagen: ``Es ist von Allah", während es nicht von Allah ist. Und sie sprechen (damit) wissentlich eine Lüge gegen Allah aus.\qt@no{78}}%
\qt@newcmd\qurantrans@de@ccclxxii{Es steht einem menschlichen Wesen nicht zu, daß ihm Allah die Schrift, das Urteil und das Prophetentum gibt, und er hierauf zu den Menschen sagt: ``Seid Diener von mir anstatt Allahs!", sondern: ``Seid Leute des Herrn, da ihr das Buch zu lehren und da ihr (es) zu erlernen pflegtet."\qt@no{79}}%
\qt@newcmd\qurantrans@de@ccclxxiii{Und (es steht ihm nicht zu,) euch zu befehlen, die Engel und die Propheten zu Herren nehmen. Sollte er euch den Unglauben befehlen, nachdem ihr (Allah) Ergebene seid?\qt@no{80}}%
\qt@newcmd\qurantrans@de@ccclxxiv{Und als Allah mit den Propheten ein Abkommen traf: Was immer Ich euch an Büchern und Weisheit gebracht habe -, und danach ist zu euch ein Gesandter gekommen, das bestätigend, was euch (bereits) vorliegt, an den müßt ihr ganz gewiß glauben und dem müßt ihr ganz gewiß helfen. Er sagte: ``Erklärt ihr euch einverstanden und nehmt ihr unter dieser (Bedingung) Meine Bürde an?" Sie sagten: ``Wir erklären uns einverstanden." Er sagte: ``So bezeugt es, und Ich gehöre mit euch zu den Zeugnis Ablegenden."\qt@no{81}}%
\qt@newcmd\qurantrans@de@ccclxxv{Wer sich aber nach diesem abkehrt, so sind jene die Frevler.\qt@no{82}}%
\qt@newcmd\qurantrans@de@ccclxxvi{Begehren sie denn eine andere als Allahs Religion, wo sich Ihm doch (jeder) ergeben hat, der in den Himmeln und auf der Erde ist, freiwillig oder widerwillig? Und zu Ihm werden sie zurückgebracht.\qt@no{83}}%
\qt@newcmd\qurantrans@de@ccclxxvii{Sag: Wir glauben an Allah und (an das,) was auf uns und was auf Ibrahim, Isma'il, Ishaq, Ya'qub und die Stämme (als Offenbarung) herabgesandt wurde und was Musa, ``Isa und den Propheten von ihrem Herrn gegeben wurde. Wir machen keinen Unterschied bei jemandem von Ihnen, und wir sind Ihm ergeben.\qt@no{84}}%
\qt@newcmd\qurantrans@de@ccclxxviii{Wer aber als Religion etwas anderes als den Islam begehrt, so wird es von ihm nicht angenommen werden, und im Jenseits wird er zu den Verlierern gehören.\qt@no{85}}%
\qt@newcmd\qurantrans@de@ccclxxix{Wie sollte Allah ein Volk rechtleiten, das (wieder) ungläubig wurde, nachdem es den Glauben (angenommen) hatte und (nachdem) es bezeugt hatte, daß der Gesandte wahrhaft ist, und (nachdem) zu ihm die klaren Beweise gekommen waren! Und Allah leitet nicht das ungerechte Volk recht.\qt@no{86}}%
\qt@newcmd\qurantrans@de@ccclxxx{Der Lohn jener ist, daß auf ihnen der Fluch Allahs und der Engel und der Menschen allesamt liegt,\qt@no{87}}%
\qt@newcmd\qurantrans@de@ccclxxxi{ewig darin zu bleiben. Die Strafe soll ihnen nicht erleichtert noch soll ihnen Aufschub gewährt werden,\qt@no{88}}%
\qt@newcmd\qurantrans@de@ccclxxxii{außer denjenigen, die nach alledem bereuen und verbessern, so ist Allah Allvergebend und Barmherzig.\qt@no{89}}%
\qt@newcmd\qurantrans@de@ccclxxxiii{Jene (aber), die ungläubig werden, nachdem sie den Glauben (angenommen) haben, und hierauf an Unglauben zunehmen, deren Reue wird nicht angenommen werden, und jene sind die Irregehenden.\qt@no{90}}%
\qt@newcmd\qurantrans@de@ccclxxxiv{Gewiß, diejenigen, die ungläubig geworden sind und als Ungläubige sterben, - von keinem von ihnen würde die (ganze) Erde voll Gold angenommen werden, auch wenn er sich damit loskaufen wollte. Für jene wird es schmerzhafte Strafe geben; und sie werden keine Helfer haben.\qt@no{91}}%
\qt@newcmd\qurantrans@de@ccclxxxv{Ihr werdet die Güte nicht erreichen, bevor ihr nicht von dem ausgebt, was euch lieb ist. Und was immer ihr ausgebt, so weiß Allah darüber Bescheid.\qt@no{92}}%
\qt@newcmd\qurantrans@de@ccclxxxvi{Alle Speisen waren den Kindern Isra'ils erlaubt außer dem, was Isra'il sich selbst verbot, bevor die Tora offenbart wurde. Sag: Bringt doch die Tora bei und verlest sie dann, wenn ihr wahrhaftig seid.\qt@no{93}}%
\qt@newcmd\qurantrans@de@ccclxxxvii{Wer also nach alledem gegen Allah (noch) Lügen ersinnt, jene sind die Ungerechten.\qt@no{94}}%
\qt@newcmd\qurantrans@de@ccclxxxviii{Sag: Allah hat die Wahrheit gesprochen. So folgt dem Glaubensbekenntnis Ibrahims, (als) Anhänger des rechten Glaubens, und er gehörte nicht zu den Götzendienern.\qt@no{95}}%
\qt@newcmd\qurantrans@de@ccclxxxix{Das erste (Gottes)haus, das für die Menschen gegründet wurde, ist wahrlich dasjenige in Bakka, als ein gesegnetes (Haus) und eine Rechtleitung für die Weltenbewohner.\qt@no{96}}%
\qt@newcmd\qurantrans@de@cccxc{Darin liegen klare Zeichen. (Es ist) der Standort Ibrahims. Und wer es betritt, ist sicher. Und Allah steht es den Menschen gegenüber zu, daß sie die Pilgerfahrt zum Hause unternehmen - (diejenigen,) die dazu die Möglichkeit haben. Wer aber ungläubig ist, so ist Allah der Weltenbewohner Unbedürftig.\qt@no{97}}%
\qt@newcmd\qurantrans@de@cccxci{Sag: O Leute der Schrift, warum verleugnet ihr Allahs Zeichen, wo doch Allah Zeuge dessen ist, was ihr tut?\qt@no{98}}%
\qt@newcmd\qurantrans@de@cccxcii{Sag: O Leute der Schrift, warum haltet ihr die, die glauben, von Allahs Weg ab, indem ihr danach trachtet, ihn krumm zu machen, wo ihr doch Zeugen seid? Doch Allah ist nicht unachtsam dessen, was ihr tut.\qt@no{99}}%
\qt@newcmd\qurantrans@de@cccxciii{O die ihr glaubt, wenn ihr einer Gruppe von denen gehorcht, denen die Schrift gegeben wurde, werden sie euch, nachdem ihr den Glauben (angenommen) habt, wieder zu Ungläubigen machen.\qt@no{100}}%
\qt@newcmd\qurantrans@de@cccxciv{Wie aber könnt ihr (wieder) ungläubig werden, wo euch doch Allahs Zeichen verlesen werden und unter euch Sein Gesandter ist? Doch wer an Allah festhält, der wird gewiß auf einen geraden Weg geleitet.\qt@no{101}}%
\qt@newcmd\qurantrans@de@cccxcv{O die ihr glaubt, fürchtet Allah in gebührender Furcht und sterbt ja nicht anders denn als (Allah) Ergebene!\qt@no{102}}%
\qt@newcmd\qurantrans@de@cccxcvi{Und haltet alle fest am Seil Allahs und geht nicht auseinander! Und gedenkt Allahs Gunst an euch, als ihr Feinde wart und Er dann eure Herzen zusammenführte, worauf ihr durch Seine Gunst Brüder wurdet. Und (als) ihr am Rand einer Feuergrube wart und Er euch dann davor errettete. So macht Allah euch Seine Zeichen klar, auf daß ihr rechtgeleitet werden möget!\qt@no{103}}%
\qt@newcmd\qurantrans@de@cccxcvii{Und es soll aus euch eine Gemeinschaft werden, die zum Guten aufruft, das Rechte gebietet und das Verwerfliche verbietet. Jene sind es, denen es wohl ergeht.\qt@no{104}}%
\qt@newcmd\qurantrans@de@cccxcviii{Und seid nicht wie jene, die auseinander gingen und uneinig wurden, nachdem die klaren Beweise zu ihnen gekommen waren. Für jene gibt es gewaltige Strafe.\qt@no{105}}%
\qt@newcmd\qurantrans@de@cccxcix{An dem Tag, da (die einen) Gesichter weiß und (die anderen) Gesichter schwarz sein werden. Was nun diejenigen angeht, deren Gesichter schwarz sein werden (, so wird zu ihnen gesagt werden): ``Seid ihr ungläubig geworden, nachdem ihr den Glauben (angenommen) hattet? So kostet die Strafe dafür, daß ihr wieder ungläubig geworden seid."\qt@no{106}}%
\qt@newcmd\qurantrans@de@cd{Was aber diejenigen angeht, deren Gesichter weiß sein werden, so werden sie sich in Allahs Erbarmen befinden. Ewig werden sie darin bleiben.\qt@no{107}}%
\qt@newcmd\qurantrans@de@cdi{Dies sind die Zeichen Allahs, die Wir der Wahrheit entsprechend verlesen. Und Allah will keine Ungerechtigkeit für die Weltenbewohner\qt@no{108}}%
\qt@newcmd\qurantrans@de@cdii{Allah gehört (alles), was in den Himmeln und was auf der Erde ist, und zu Allah werden (all) die Angelegenheiten zurückgebracht.\qt@no{109}}%
\qt@newcmd\qurantrans@de@cdiii{Ihr seid die beste Gemeinschaft, die für die Menschen hervorgebracht worden ist. Ihr gebietet das Rechte und verbietet das Verwerfliche und glaubt an Allah. Und wenn die Leute der Schrift glauben würden, wäre es wahrlich besser für sie. Unter ihnen gibt es Gläubige, aber die meisten von ihnen sind Frevler.\qt@no{110}}%
\qt@newcmd\qurantrans@de@cdiv{Sie werden euch keinen Schaden zufügen, außer Beleidigungen. Und wenn sie gegen euch kämpfen, werden sie euch den Rücken kehren, und hierauf wird ihnen keine Hilfe zuteil werden.\qt@no{111}}%
\qt@newcmd\qurantrans@de@cdv{Auferlegt ist ihnen Erniedrigung, wo immer sie angetroffen werden-, außer mit einem Seil von Allah und einem Seil von den Menschen -, und sie haben sich Zorn von Allah zugezogen, und Elend ist ihnen auferlegt. Dies, weil sie stets Allahs Zeichen verleugneten und die Propheten zu Unrecht töteten, weil sie sich widersetzten und stets übertraten.\qt@no{112}}%
\qt@newcmd\qurantrans@de@cdvi{Sie sind nicht (alle) gleich. Unter den Leuten der Schrift ist eine standhafte Gemeinschaft, die Allahs Zeichen zu Stunden der Nacht verliest und sich (im Gebet) niederwirft.\qt@no{113}}%
\qt@newcmd\qurantrans@de@cdvii{Sie glauben an Allah und den Jüngsten Tag und gebieten das Rechte und verbieten das Verwerfliche und beeilen sich mit den guten Dingen. Jene gehören zu den Rechtschaffenen.\qt@no{114}}%
\qt@newcmd\qurantrans@de@cdviii{Und was sie an Gutem tun, das wird ihnen nicht ungedankt bleiben. Und Allah weiß über die Gottesfürchtigen Bescheid.\qt@no{115}}%
\qt@newcmd\qurantrans@de@cdix{Gewiß, denjenigen, die ungläubig sind, werden weder ihr Besitz noch ihre Kinder vor Allah etwas nützen. Das sind Insassen des (Höllen)feuers. Ewig werden sie darin bleiben.\qt@no{116}}%
\qt@newcmd\qurantrans@de@cdx{Das Gleichnis dessen, was sie in diesem irdischen Leben ausgeben, ist das eines eisigen Windes, der die Saatfelder von Leuten trifft, die sich selbst Unrecht zugefügt haben; und so vernichtet er sie. Allah hat ihnen kein Unrecht zugefügt, sondern sie selbst fügen sich Unrecht zu.\qt@no{117}}%
\qt@newcmd\qurantrans@de@cdxi{O die ihr glaubt, nehmt keine Vertrauten außer von euch. Sie scheuen keine Mühe, euch zu verwirren, und möchten gern, daß ihr in Bedrängnis geratet. Schon wurde aus ihren Mündern Haß offenkundig, aber was ihre Brüste verborgen halten, ist (noch) schwerwiegender. Wir haben euch die Zeichen bereits klargemacht, wenn ihr begreifen wollt.\qt@no{118}}%
\qt@newcmd\qurantrans@de@cdxii{Da habt ihr sie doch geliebt, während sie euch nicht lieben, und ihr glaubt an das gesamte Buch. Wenn sie euch treffen, sagen sie: ``Wir glauben." Wenn sie jedoch allein sind, beißen sie sich in die Fingerspitzen vor Grimm gegen euch. Sag: Sterbt an eurem Grimm! Gewiß, Allah weiß über das Innerste der Brüste Bescheid.\qt@no{119}}%
\qt@newcmd\qurantrans@de@cdxiii{Wenn euch etwas Gutes widerfährt, tut es ihnen leid. Wenn euch jedoch etwas Böses trifft, sind sie froh darüber. Wenn ihr aber geduldig seid und gottesfürchtig, wird euch ihre Arglist keinen Schaden zufügen. Allah umfaßt, was sie tun.\qt@no{120}}%
\qt@newcmd\qurantrans@de@cdxiv{Und als du von deinen Angehörigen frühmorgens weggingst, um den Gläubigen Stellungen für den Kampf zuzuweisen. Und Allah ist Allhörend und Allwissend.\qt@no{121}}%
\qt@newcmd\qurantrans@de@cdxv{Als zwei Teil(gruppen) von euch vorhatten, aufzugeben, wo doch Allah ihr Schutzherr ist. Und auf Allah sollen sich die Gläubigen verlassen.\qt@no{122}}%
\qt@newcmd\qurantrans@de@cdxvi{Allah hat euch doch schon in Badr geholfen, als ihr verächtlich erschient. Darum fürchtet Allah, auf daß ihr dankbar sein möget!\qt@no{123}}%
\qt@newcmd\qurantrans@de@cdxvii{Als du zu den Gläubigen sagtest: ``Genügt es euch denn nicht, daß euch euer Herr mit dreitausend herabgesandten Engeln unterstützt?\qt@no{124}}%
\qt@newcmd\qurantrans@de@cdxviii{Ja doch! Wenn ihr standhaft seid und gottesfürchtig und sie unverzüglich über euch kommen, unterstützt euch euer Herr mit fünftausend gekennzeichneten Engeln."\qt@no{125}}%
\qt@newcmd\qurantrans@de@cdxix{Allah machte es nur zu einer frohen Botschaft für euch, und damit eure Herzen dadurch Ruhe finden -, denn der Sieg kommt nur von Allah, dem Allmächtigen, dem Allweisen -,\qt@no{126}}%
\qt@newcmd\qurantrans@de@cdxx{damit Er einen Teil derjenigen, die ungläubig waren, abtrennte oder sie niederwarf, so daß sie enttäuscht umkehrten.\qt@no{127}}%
\qt@newcmd\qurantrans@de@cdxxi{Es ist gar nicht deine Angelegenheit, ob Er ihre Reue annimmt oder sie straft, denn sie sind ja Ungerechte.\qt@no{128}}%
\qt@newcmd\qurantrans@de@cdxxii{Und Allah gehört (alles), was in den Himmeln und was auf der Erde ist. Er vergibt, wem Er will, und Er straft, wen Er will. Und Allah ist Allvergebend und Barmherzig.\qt@no{129}}%
\qt@newcmd\qurantrans@de@cdxxiii{O die ihr glaubt, verschlingt nicht den Zins um ein Vielfaches vermehrt, sondern fürchtet Allah, auf daß es euch wohl ergehen möge!\qt@no{130}}%
\qt@newcmd\qurantrans@de@cdxxiv{Und hütet euch vor dem (Höllen)feuer, das für die Ungläubigen bereitet ist!\qt@no{131}}%
\qt@newcmd\qurantrans@de@cdxxv{Und gehorcht Allah und dem Gesandten, auf daß ihr Erbarmen finden möget!\qt@no{132}}%
\qt@newcmd\qurantrans@de@cdxxvi{Und beeilt euch um Vergebung von eurem Herrn und (um) einen (Paradies)garten, dessen Breite (wie) die Himmel und die Erde ist. Er ist für die Gottesfürchtigen bereitet,\qt@no{133}}%
\qt@newcmd\qurantrans@de@cdxxvii{die in Freude und Leid ausgeben und ihren Grimm zurückhalten und den Menschen verzeihen. Und Allah liebt die Gutes Tuenden\qt@no{134}}%
\qt@newcmd\qurantrans@de@cdxxviii{und diejenigen, die, wenn sie eine Abscheulichkeit begangen oder sich selbst Unrecht zugefügt haben, Allahs gedenken und dann für ihre Sünden um Vergebung bitten - und wer sollte die Sünden vergeben außer Allah? - und (die) nicht auf dem beharren, was sie getan haben, wo sie doch wissen.\qt@no{135}}%
\qt@newcmd\qurantrans@de@cdxxix{Der Lohn jener ist Vergebung von ihrem Herrn und Gärten, durcheilt von Bächen, ewig darin zu bleiben. Und wie trefflich ist der Lohn derjenigen, die (gut) handeln!\qt@no{136}}%
\qt@newcmd\qurantrans@de@cdxxx{Schon vor euch sind Gesetzmäßigkeiten ergangen. So reist doch auf der Erde umher und schaut, wie das Ende der Leugner war.\qt@no{137}}%
\qt@newcmd\qurantrans@de@cdxxxi{Dies ist eine klare Darlegung für die Menschen und eine Rechtleitung und Ermahnung für die Gottesfürchtigen.\qt@no{138}}%
\qt@newcmd\qurantrans@de@cdxxxii{Und werdet nicht schwach noch seid traurig, wo ihr doch die Oberhand haben werdet, wenn ihr gläubig seid.\qt@no{139}}%
\qt@newcmd\qurantrans@de@cdxxxiii{Wenn euch eine Wunde zugefügt worden ist, so ist dem (ungläubigen) Volk schon eine gleiche Wunde zugefügt worden. Und diese Tage (des Kriegsglücks) lassen Wir unter den Menschen wechseln, - damit Allah diejenigen, die glauben, kennt und (damit) Er Sich von euch (Blut)zeugen nimmt. Und Allah liebt nicht die Ungerechten.\qt@no{140}}%
\qt@newcmd\qurantrans@de@cdxxxiv{Und damit Allah diejenigen, die glauben, herausstellt und die Ungläubigen dahinschwinden läßt.\qt@no{141}}%
\qt@newcmd\qurantrans@de@cdxxxv{Oder meint ihr etwa, daß ihr in den (Paradies)garten eingehen werdet, noch ehe Allah diejenigen von euch kennt, die unter euch sich (für Seine Sache) abmühen, und die Standhaften kennt?\qt@no{142}}%
\qt@newcmd\qurantrans@de@cdxxxvi{Und ihr habt euch ja wiederholt den Tod gewünscht, bevor ihr ihm begegnet seid. Nun habt ihr ihn mit eigenen Augen geschaut.\qt@no{143}}%
\qt@newcmd\qurantrans@de@cdxxxvii{Und Muhammad ist doch nur ein Gesandter, vor dem schon Gesandte vorübergegangen sind. Wenn er nun stirbt oder getötet wird, werdet ihr euch (dann) auf den Fersen umkehren? Und wer sich auf den Fersen umkehrt, wird Allah keinerlei Schaden zufügen. Aber Allah wird (es) den Dankbaren vergelten.\qt@no{144}}%
\qt@newcmd\qurantrans@de@cdxxxviii{Keiner Seele ist es möglich zu sterben, außer mit Allahs Erlaubnis (und) nach einer befristeten Vorbestimmung. Wer die Belohnung des Diesseits will, dem geben Wir (etwas) von ihm; und wer die Belohnung des Jenseits will, dem geben Wir (etwas) von ihm. Und Wir werden (es) den Dankbaren vergelten.\qt@no{145}}%
\qt@newcmd\qurantrans@de@cdxxxix{Und mit wie vielen Propheten zusammen kämpften zahlreiche Bekenner des Herrn! Doch sie gaben nicht auf ob dessen, was sie auf Allahs Weg traf, noch wurden sie schwach, noch unterwarfen sie sich. Und Allah liebt die Standhaften.\qt@no{146}}%
\qt@newcmd\qurantrans@de@cdxl{Ihre Worte waren nichts anderes, als daß sie sagten: ``Unser Herr, vergib uns unsere Sünden und die Maßlosigkeit in unserer Angelegenheit, festige unsere Füße und hilf uns gegen das ungläubige Volk!"\qt@no{147}}%
\qt@newcmd\qurantrans@de@cdxli{Da gab Allah ihnen die Belohnung des Diesseits und die schöne Belohnung des Jenseits. Und Allah liebt die Gutes Tuenden.\qt@no{148}}%
\qt@newcmd\qurantrans@de@cdxlii{O die ihr glaubt, wenn ihr denen, die ungläubig sind, gehorcht, lassen sie euch auf den Fersen kehrtmachen, und dann werdet ihr als Verlierer zurückkehren.\qt@no{149}}%
\qt@newcmd\qurantrans@de@cdxliii{Nein! Vielmehr ist Allah euer Schutzherr; und Er ist der beste Helfer.\qt@no{150}}%
\qt@newcmd\qurantrans@de@cdxliv{Wir werden in die Herzen derjenigen, die ungläubig sind, Schrecken einjagen dafür, daß sie Allah (andere Götter) beigesellt haben, wozu Er keine Ermächtigung offenbart hat. Ihr Zufluchtsort wird das (Höllen)feuer sein - ein schlimmer Aufenthaltsort für die Ungerechten!\qt@no{151}}%
\qt@newcmd\qurantrans@de@cdxlv{Allah hat ja Sein Versprechen euch gegenüber gehalten, als ihr sie mit Seiner Erlaubnis vernichtetet, bis daß ihr den Mut verlort und über die Angelegenheit miteinander strittet und euch widersetztet, nachdem Er euch gezeigt hatte, was euch lieb ist. - Unter euch gibt es manche, die das Diesseits wollen; unter euch gibt es aber auch manche, die das Jenseits wollen. - Hierauf wandte Er euch von ihnen ab, um euch zu prüfen. Nun hat Er euch wahrlich schon verziehen, denn Allah ist voll Huld gegen die Gläubigen.\qt@no{152}}%
\qt@newcmd\qurantrans@de@cdxlvi{Als ihr fortlieft und euch nach niemandem umdrehtet, während der Gesandte euch von hinten her rief. Da belohnte Er euch mit Kummer für (den) Kummer (des Propheten), damit ihr weder traurig wäret über das, was euch entgangen war, noch über das, was euch getroffen hatte. Und Allah ist Kundig dessen, was ihr tut.\qt@no{153}}%
\qt@newcmd\qurantrans@de@cdxlvii{Dann sandte Er nach dem Kummer Sicherheit auf euch herab, Schläfrigkeit, die einen Teil von euch überkam, während ein (anderer) Teil um sich selbst besorgt war, indem sie von Allah in Gedanken der Unwissenheit dachte, was dem Wahren nicht entspricht. Sie sagten: ``Haben wir denn irgend etwas in der Angelegenheit (zu entscheiden gehabt)?" Sag: Die Angelegenheit ist ganz und gar (in) Allahs (Entscheidung). Sie halten in ihrem Innersten verborgen, was sie dir nicht offenlegen; sie sagen: ``Wenn wir etwas von der Angelegenheit (zu entscheiden) hätten, würden wir hier nicht getötet." Sag: Wenn ihr auch in euren Häusern gewesen wäret, wären diejenigen, denen der Tod vorgezeichnet war, an den Stellen erschienen, wo sie (als Gefallene) liegen sollten, - damit Allah prüfe, was in euren Brüsten und herausstellt, was in euren Herzen ist. Und Allah weiß über das Innerste der Brüste Bescheid.\qt@no{154}}%
\qt@newcmd\qurantrans@de@cdxlviii{Gewiß, diejenigen von euch, die sich an dem Tag, da die beiden Heere aufeinandertrafen, abkehrten, ließ nur der Satan straucheln durch etwas von dem, was sie begangen hatten. Aber wahrlich, nun hat Allah ihnen verziehen. Allah ist Allvergebend und Nachsichtig.\qt@no{155}}%
\qt@newcmd\qurantrans@de@cdxlix{O die ihr glaubt, seid nicht wie diejenigen, die ungläubig sind und von ihren Brüdern, wenn sie im Lande umhergereist sind oder sich auf einem Kriegszug befunden haben, sagen: ``Wenn sie bei uns geblieben wären, wären sie nicht gestorben und nicht getötet worden", damit Allah dies zu einer gramvollen Reue in ihren Herzen mache. Allah macht lebendig und läßt sterben. Und was ihr tut, sieht Allah wohl.\qt@no{156}}%
\qt@newcmd\qurantrans@de@cdl{Und wenn ihr auf Allahs Weg getötet werdet oder sterbt, so sind Vergebung von Allah und Erbarmen fürwahr besser als (all) das, was sie zusammentragen.\qt@no{157}}%
\qt@newcmd\qurantrans@de@cdli{Und wenn ihr sterbt oder getötet werdet, so werdet ihr fürwahr zu Allah versammelt werden.\qt@no{158}}%
\qt@newcmd\qurantrans@de@cdlii{Durch Erbarmen von Allah bist du mild zu ihnen gewesen; wärst du aber schroff und hartherzig, so würden sie wahrlich rings um dich auseinandergelaufen. So verzeihe ihnen, bitte für sie um Vergebung und ziehe sie in den Angelegenheiten zu Rate. Und wenn du dich entschlossen hast, dann verlasse dich auf Allah! Gewiß, Allah liebt die sich (auf Ihn) Verlassenden.\qt@no{159}}%
\qt@newcmd\qurantrans@de@cdliii{Wenn Allah euch zum Sieg verhilft, so kann euch keiner besiegen. Doch wenn Er euch im Stich läßt, wer ist es denn, der euch dann, nach Ihm, noch helfen könnte? Und auf Allah sollen sich die Gläubigen verlassen.\qt@no{160}}%
\qt@newcmd\qurantrans@de@cdliv{Es steht keinem Propheten zu, (etwas) zu veruntreuen. Und wer (etwas) veruntreut, wird das, was er veruntreut hat, am Tag der Auferstehung bringen. Dann wird jeder Seele in vollem Maß zukommen, was sie verdient hat, und es wird ihnen kein Unrecht zugefügt.\qt@no{161}}%
\qt@newcmd\qurantrans@de@cdlv{Ist denn jemand, der Allahs Wohlgefallen folgt, wie einer, der sich Allahs Mißfallen zuzieht und dessen Zufluchtsort die Hölle ist? - Ein schlimmer Ausgang!\qt@no{162}}%
\qt@newcmd\qurantrans@de@cdlvi{Sie nehmen (unterschiedliche) Rangstufen bei Allah ein. Und Allah sieht wohl, was sie tun.\qt@no{163}}%
\qt@newcmd\qurantrans@de@cdlvii{Allah hat den Gläubigen wirklich eine Wohltat erwiesen, als Er unter ihnen einen Gesandten von ihnen selbst geschickt hat, der ihnen Seine Zeichen verliest, und sie läutert und sie das Buch und die Weisheit lehrt, obgleich sie sich zuvor wahrlich in deutlichem Irrtum befanden.\qt@no{164}}%
\qt@newcmd\qurantrans@de@cdlviii{Ist es nicht (so), daß, als euch ein Unglück traf, obwohl ihr (den Feind) mit einem zweimal so großen getroffen hattet, ihr sagtet: ``Woher kommt das?" Sag: Es kommt von euch selbst. Gewiß, Allah hat zu allem die Macht.\qt@no{165}}%
\qt@newcmd\qurantrans@de@cdlix{Und was euch an dem Tag traf, da die beiden Heere aufeinandertrafen, so (geschah) das mit Allahs Erlaubnis, - damit Er die Gläubigen kennt\qt@no{166}}%
\qt@newcmd\qurantrans@de@cdlx{und damit Er diejenigen kennt, die heucheln, zu denen gesagt wurde: ``Kommt her und kämpft auf Allahs Weg oder wehrt ab", worauf sie sagten: ``Wenn wir zu kämpfen wüßten, würden wir euch wahrlich folgen." Sie waren an jenem Tag dem Unglauben näher als dem Glauben. Sie sagen mit ihren Mündern, was nicht in ihren Herzen ist. Doch Allah weiß sehr wohl, was sie verbergen.\qt@no{167}}%
\qt@newcmd\qurantrans@de@cdlxi{Diejenigen, die von ihren Brüdern sagten, während sie (selbst daheim) sitzen blieben: ``Wenn sie uns gehorcht hätten, wären sie nicht getötet worden". Sag: So wehrt den Tod von euch selbst ab, wenn ihr wahrhaftig seid!\qt@no{168}}%
\qt@newcmd\qurantrans@de@cdlxii{Und meine ja nicht, diejenigen, die auf Allahs Weg getötet worden sind, seien (wirklich) tot. Nein! Vielmehr sind sie lebendig bei ihrem Herrn und werden versorgt\qt@no{169}}%
\qt@newcmd\qurantrans@de@cdlxiii{und sind froh über das, was Allah ihnen von Seiner Huld gewährt hat, und sind glückselig über diejenigen, die sich nach ihnen noch nicht angeschlossen haben, daß keine Furcht über sie kommen soll, noch sie traurig sein sollen.\qt@no{170}}%
\qt@newcmd\qurantrans@de@cdlxiv{Sie sind glückselig über eine Gunst von Allah und eine Huld und (darüber), daß Allah den Lohn der Gläubigen nicht verlorengehen läßt.\qt@no{171}}%
\qt@newcmd\qurantrans@de@cdlxv{Diejenigen, die auf Allah und den Gesandten hörten, nachdem ihnen eine Wunde zugefügt worden war - für diejenigen unter ihnen, die Gutes tun und gottesfürchtig sind, wird es großartigen Lohn geben.\qt@no{172}}%
\qt@newcmd\qurantrans@de@cdlxvi{Diejenigen, zu denen die Menschen sagten: ``Die Menschen haben (sich) bereits gegen euch versammelt; darum fürchtet sie!" - Doch da mehrte das (nur) ihren Glauben, und sie sagten: ``Unsere Genüge ist Allah, und wie trefflich ist der Sachwalter!"\qt@no{173}}%
\qt@newcmd\qurantrans@de@cdlxvii{So kehrten sie mit einer Gunst von Allah und einer Huld zurück, ohne daß ihnen etwas Böses widerfahren war. Sie folgten Allahs Wohlgefallen, und Allah ist voll großer Huld.\qt@no{174}}%
\qt@newcmd\qurantrans@de@cdlxviii{Dies ist nur der Satan, der (euch) mit seinen Gefolgsleuten Furcht einzuflößen sucht. Fürchtet sie aber nicht, sondern fürchtet Mich, wenn ihr gläubig seid!\qt@no{175}}%
\qt@newcmd\qurantrans@de@cdlxix{Und lasse dich nicht durch jene traurig machen, die im Unglauben dahineilen! Sie werden Allah gewiß keinerlei Schaden zufügen. Allah will, daß ihnen im Jenseits kein Anteil zugewiesen wird. Für sie wird es gewaltige Strafe geben.\qt@no{176}}%
\qt@newcmd\qurantrans@de@cdlxx{Diejenigen, die den Unglauben für den Glauben erkauft haben, werden Allah keinerlei Schaden zufügen, und für sie wird es schmerzhafte Strafe geben.\qt@no{177}}%
\qt@newcmd\qurantrans@de@cdlxxi{Und diejenigen, die ungläubig sind, sollen ja nicht meinen, daß das, was Wir ihnen an Aufschub gewähren, gut für sich selbst sei. Wir gewähren ihnen nur Aufschub, damit sie an Sünde zunehmen, und für sie gibt es schmachvolle Strafe.\qt@no{178}}%
\qt@newcmd\qurantrans@de@cdlxxii{Nimmer wird Allah die Gläubigen in dem (Zustand) belassen, in dem ihr euch befindet, bis Er das Schlechte vom Guten gesondert hat. Und nimmer wird Allah euch Einblick in das Verborgene gewähren, doch Allah erwählt von Seinen Gesandten, wen Er will. So glaubt an Allah und Seine Gesandten! Und wenn ihr glaubt und gottesfürchtig seid, dann wird es für euch großartigen Lohn geben.\qt@no{179}}%
\qt@newcmd\qurantrans@de@cdlxxiii{Und diejenigen, die mit dem geizen, was Allah ihnen von Seiner Huld gewährt hat, sollen ja nicht meinen, das sei (so) besser für sie. Nein! Vielmehr ist es schlecht für sie. Mit dem, womit sie gegeizt haben, werden sie am Tag der Auferstehung umschlungen werden. Und Allah gehört das Erbe der Himmel und der Erde, und Allah ist dessen, was ihr tut, Kundig.\qt@no{180}}%
\qt@newcmd\qurantrans@de@cdlxxiv{Allah hat ja die Worte derjenigen gehört, die sagten: ``Allah ist gewiß arm, und wir sind reich." Wir werden aufschreiben, was sie sagten, und daß sie die Propheten zu Unrecht töteten; und Wir werden zu ihnen sagen ``Kostet die Strafe des Brennens!\qt@no{181}}%
\qt@newcmd\qurantrans@de@cdlxxv{Dies (geschieht) wegen dessen, was eure Hände vorausgeschickt haben, und deshalb, weil Allah keiner ist, der den Menschen Unrecht zufügt."\qt@no{182}}%
\qt@newcmd\qurantrans@de@cdlxxvi{Es sind) diejenigen, die sagten: ``Allah hat uns verpflichtet, keinem Gesandten zu glauben, bis er uns ein Opfer bringt, das vom Feuer verzehrt wird." Sag: Schon vor mir sind Gesandte mit klaren Beweisen zu euch gekommen und mit dem, wovon ihr spracht. Warum habt ihr sie dann getötet, wenn ihr wahrhaftig seid?"\qt@no{183}}%
\qt@newcmd\qurantrans@de@cdlxxvii{Wenn sie dich der Lüge bezichtigen, so sind bereits Gesandte vor dir der Lüge bezichtigt worden, die mit den klaren Beweisen, den Büchern der Weisheit und den erleuchtenden Büchern kamen.\qt@no{184}}%
\qt@newcmd\qurantrans@de@cdlxxviii{Jede Seele wird den Tod kosten. Und erst am Tag der Auferstehung wird euch euer Lohn in vollem Maß zukommen. Wer dann dem (Höllen)feuer entrückt und in den (Paradies)garten eingelassen wird, der hat fürwahr einen Erfolg erzielt. Und das diesseitige Leben ist nur trügerischer Genuß.\qt@no{185}}%
\qt@newcmd\qurantrans@de@cdlxxix{Ihr werdet ganz gewiß in eurem Besitz und in eurer eigenen Person geprüft werden, und ihr werdet ganz gewiß von denjenigen, denen die Schrift vor euch gegeben wurde, und denen, die (Allah etwas) beigesellen, viel Beleidigendes zu hören bekommen. Doch wenn ihr standhaft und gottesfürchtig seid, so gehört dies gewiß zur Entschlossenheit (in der Handhabung) der Angelegenheiten.\qt@no{186}}%
\qt@newcmd\qurantrans@de@cdlxxx{Und (gedenkt,) als Allah mit denjenigen, denen die Schrift gegeben worden war, ein Abkommen traf: ``Ihr sollt sie den Menschen ganz gewiß klar machen und sie nicht verborgen halten!" Da warfen sie sie hinter ihren Rücken und verkauften sie für einen geringen Preis; wie schlimm ist das, was sie erkaufen!\qt@no{187}}%
\qt@newcmd\qurantrans@de@cdlxxxi{Und meine ja nicht, daß diejenigen, die froh sind über das, was sie begangen haben, und denen es lieb ist, für etwas gelobt zu werden, was sie nicht getan haben, - meine ja nicht, sie seien der Strafe entronnen! Für sie wird es schmerzhafte Strafe geben.\qt@no{188}}%
\qt@newcmd\qurantrans@de@cdlxxxii{Allah gehört die Herrschaft der Himmel und der Erde. Allah hat zu allem die Macht.\qt@no{189}}%
\qt@newcmd\qurantrans@de@cdlxxxiii{In der Schöpfung der Himmel und der Erde und in dem Unterschied von Nacht und Tag liegen wahrlich Zeichen für diejenigen, die Verstand besitzen,\qt@no{190}}%
\qt@newcmd\qurantrans@de@cdlxxxiv{die Allahs stehend, sitzend und auf der Seite (liegend) gedenken und über die Schöpfung der Himmel und der Erde nachdenken: ``Unser Herr, Du hast (all) dies nicht umsonst erschaffen. Preis sei Dir! Bewahre uns vor der Strafe des (Höllen)feuers.\qt@no{191}}%
\qt@newcmd\qurantrans@de@cdlxxxv{Unser Herr, gewiß, wen Du ins (Höllen)feuer eingehen läßt, den hast Du (damit) in Schande gestürzt. Und die Ungerechten werden keine Helfer haben.\qt@no{192}}%
\qt@newcmd\qurantrans@de@cdlxxxvi{Unser Herr, gewiß, wir hörten einen Rufer, der zum Glauben aufrief: .Glaubt an euren Herrn. Da glaubten wir. Unser Herr, vergib uns unsere Sünden, tilge unsere bösen Taten und berufe uns ab unter den Gütigen.\qt@no{193}}%
\qt@newcmd\qurantrans@de@cdlxxxvii{Unser Herr, und gib uns, was Du uns durch Deine Gesandten versprochen hast, und stürze uns nicht in Schande am Tag der Auferstehung. Gewiß, Du brichst nicht, was Du versprochen hast."\qt@no{194}}%
\qt@newcmd\qurantrans@de@cdlxxxviii{Da erhörte sie ihr Herr: ``Ich lasse kein Werk eines (Gutes) Tuenden von euch verlorengehen, sei es von Mann oder Frau; die einen von euch sind von den anderen. Denen also, die ausgewandert und aus ihren Wohnstätten vertrieben worden sind und denen auf Meinem Weg Leid zugefügt worden ist, und die gekämpft haben und getötet worden sind, werde Ich ganz gewiß ihre bösen Taten tilgen und sie ganz gewiß in Gärten eingehen lassen, durcheilt von Bächen, als Belohnung von Allah." Und Allah - bei Ihm ist die schöne Belohnung.\qt@no{195}}%
\qt@newcmd\qurantrans@de@cdlxxxix{Lasse dich ja nicht durch den Wandel derer in den Landstrichen täuschen, die ungläubig sind.\qt@no{196}}%
\qt@newcmd\qurantrans@de@cdxc{Es ist nur) ein geringer Genuß, daraufhin wird die Hölle ihr Zufluchtsort sein. Und wie schlimm ist die Lagerstatt!\qt@no{197}}%
\qt@newcmd\qurantrans@de@cdxci{Aber für diejenigen, die ihren Herrn fürchten, sind Gärten, durcheilt von Bächen, ewig darin zu bleiben; eine gastliche Aufnahme von Allah. Was bei Allah ist, ist besser für die Gütigen.\qt@no{198}}%
\qt@newcmd\qurantrans@de@cdxcii{Und unter den Leuten der Schrift gibt es wahrlich manche, die an Allah glauben und (an) das, was zu euch (als Offenbarung) herabgesandt worden ist, und was zu ihnen (selbst) herabgesandt worden ist. Dabei sind sie demütig vor Allah und verkaufen nicht die Zeichen Allahs für einen geringen Preis. Jene haben ihren Lohn bei ihrem Herrn. Gewiß, Allah ist schnell im Abrechnen.\qt@no{199}}%
\qt@newcmd\qurantrans@de@cdxciii{O die ihr glaubt, geduldet euch, haltet standhaft aus, seid kampfbereit und fürchtet Allah, auf daß es euch wohl ergehen möge!\qt@no{200}}%
\qt@newcmd\qurantrans@de@cdxciv{\basmalahde O ihr Menschen, fürchtet euren Herrn, Der euch aus einem einzigen Wesen schuf, und aus ihm schuf Er seine Gattin und ließ aus beiden viele Männer und Frauen sich ausbreiten. Und fürchtet Allah, in Dessen (Namen) ihr einander bittet, und die Verwandtschaftsbande. Gewiß, Allah ist Wächter über euch.\qt@no{1}}%
\qt@newcmd\qurantrans@de@cdxcv{Und gebt den Waisen ihren Besitz und tauscht nicht Schlechtes mit Gutem aus und zehrt nicht ihren Besitz zu eurem Besitz hinzu. Das ist gewiß ein schweres Vergehen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@cdxcvi{Und wenn ihr befürchtet, nicht gerecht hinsichtlich der Waisen zu handeln, dann heiratet, was euch an Frauen gut scheint, zwei, drei oder vier. Wenn ihr aber befürchtet, nicht gerecht zu handeln, dann (nur) eine oder was eure rechte Hand besitzt. Das ist eher geeignet, daß ihr nicht ungerecht seid.\qt@no{3}}%
\qt@newcmd\qurantrans@de@cdxcvii{Und gebt den Frauen ihre Morgengabe als Geschenk. Wenn sie für euch aber freiwillig auf etwas davon verzichten, dann verzehrt es als wohlbekömmlich und zuträglich.\qt@no{4}}%
\qt@newcmd\qurantrans@de@cdxcviii{Und gebt nicht den Toren euren Besitz, den Allah euch zum Unterhalt bestimmt hat, sondern versorgt sie davon und kleidet sie und sagt zu ihnen geziemende Worte.\qt@no{5}}%
\qt@newcmd\qurantrans@de@cdxcix{Und prüft die Waisen, bis daß sie das Heiratsalter erreicht haben. Und wenn ihr dann an ihnen Besonnenheit feststellt, so händigt ihnen ihren Besitz aus. Und verzehrt ihn nicht maßlos und ihrem Erwachsenwerden zuvorkommend. Wer reich ist, der soll sich enthalten; und wer arm ist, der soll in rechtlicher Weise (davon) zehren. Und wenn ihr ihnen dann ihren Besitz aushändigt, so nehmt Zeugen vor ihnen. Doch Allah genügt als Abrechner.\qt@no{6}}%
\qt@newcmd\qurantrans@de@d{Den Männern steht ein Anteil von dem zu, was die Eltern und nächsten Verwandten hinterlassen, und den Frauen steht ein Anteil von dem zu, was die Eltern und nächsten Verwandten hinterlassen, sei es wenig oder viel - ein festgesetzter Anteil.\qt@no{7}}%
\qt@newcmd\qurantrans@de@di{Und wenn bei der Verteilung Verwandte, Waisen und Arme zugegen sind, dann versorgt sie davon und sagt zu ihnen geziemende Worte.\qt@no{8}}%
\qt@newcmd\qurantrans@de@dii{Und diejenigen sollen sich fürchten, die, wenn sie schwache Nachkommenschaft hinterließen, um sie bangen würden; darum sollen sie sich vor Allah hüten und treffende Worte sagen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@diii{Diejenigen, die den Besitz der Waisen ungerechterweise verschlingen, verzehren in ihren Bäuchen nur Feuer; und sie werden der Feuerglut ausgesetzt sein.\qt@no{10}}%
\qt@newcmd\qurantrans@de@div{Allah empfiehlt euch hinsichtlich eurer Kinder: Einem männlichen Geschlechts kommt ebensoviel zu wie der Anteil von zwei weiblichen Geschlechts. Wenn es aber (ausschließlich) Frauen sind, mehr als zwei, dann stehen ihnen zwei Drittel dessen zu, was er hinterläßt; wenn es (nur) eine ist, dann die Hälfte. Und den Eltern steht jedem ein Sechstel von dem zu, was er hinterläßt, wenn er Kinder hat. Wenn er jedoch keine Kinder hat und seine Eltern ihn beerben, dann steht seiner Mutter ein Drittel zu. Wenn er Brüder hat, dann steht seiner Mutter (in diesem Fall) ein Sechstel zu. (Das alles) nach (Abzug) eines (etwaigen) Vermächtnisses, das er festgesetzt hat, oder einer Schuld. Eure Väter und eure Söhne - ihr wißt nicht, wer von ihnen euch an Nutzen näher steht. (Das alles gilt für euch) als Verpflichtung von Allah. Gewiß, Allah ist Allwissend und Allweise.\qt@no{11}}%
\qt@newcmd\qurantrans@de@dv{Und euch steht die Hälfte vom dem zu, was eure Gattinnen hinterlassen, wenn sie keine Kinder haben. Wenn sie jedoch Kinder haben, dann steht euch ein Viertel von dem zu, was sie hinterlassen. (Das alles) nach (Abzug) eines (etwaigen) Vermächtnisses, das sie festgesetzt haben, oder einer Schuld. Und ihnen steht ein Viertel von dem zu, was ihr hinterlaßt, wenn ihr keine Kinder habt. Wenn ihr jedoch Kinder habt, dann steht ihnen ein Achtel von dem zu, was ihr hinterlaßt. (Das alles) nach (Abzug) eines (etwaigen) Vermächtnisses, das ihr festgesetzt habt, oder einer Schuld. Und wenn ein Mann oder eine Frau ohne Eltern oder Kinder beerbt wird und er (bzw. sie) einen (Halb)bruder oder eine (Halb)schwester (mütterlicherseits) hat, dann steht jedem von beiden ein Sechstel zu. Wenn es jedoch mehr als dies sind, dann sollen sie Teilhaber an einem Drittel sein. (Das alles) nach (Abzug) eines (etwaigen) Vermächtnisses, das festgesetzt worden ist, oder einer Schuld, ohne Schädigung. (Das alles ist euch) anbefohlen von Allah. Allah ist Allwissend und Nachsichtig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@dvi{Dies sind Allahs Grenzen. Wer nun Allah und Seinem Gesandten gehorcht, den wird Er in Gärten eingehen lassen, durcheilt von Bächen, ewig darin zu bleiben; und das ist ein großartiger Erfolg.\qt@no{13}}%
\qt@newcmd\qurantrans@de@dvii{Wer sich aber Allah und Seinem Gesandten widersetzt und Seine Grenzen überschreitet, den läßt Er in ein Feuer eingehen, ewig darin zu bleiben; und für ihn gibt es schmachvolle Strafe.\qt@no{14}}%
\qt@newcmd\qurantrans@de@dviii{Und diejenigen von euren Frauen, die das Abscheuliche begehen, - bringt vier Zeugen von euch gegen sie. Wenn sie (es) bezeugen, dann haltet sie im Haus fest, bis der Tod sie abberuft oder Allah ihnen einen (Aus)weg schafft.\qt@no{15}}%
\qt@newcmd\qurantrans@de@dix{Und die beiden von euch, die es begehen, - züchtigt sie. Wenn sie dann bereuen und sich bessern, so laßt von ihnen ab. Gewiß, Allah ist Reue-Annehmend und Barmherzig.\qt@no{16}}%
\qt@newcmd\qurantrans@de@dx{Die Annahme der Reue obliegt Allah nur für diejenigen, die in Unwissenheit Böses tun und hierauf beizeiten bereuen. Deren Reue nimmt Allah an. Und Allah ist Allwissend und Allweise.\qt@no{17}}%
\qt@newcmd\qurantrans@de@dxi{Nicht aber ist die Annahme der Reue für diejenigen, die böse Taten begehen, bis daß, wenn sich bei einem von ihnen der Tod einstellt, er sagt: ``Jetzt bereue ich", und auch nicht für diejenigen, die als Ungläubige sterben. Für jene haben Wir schmerzhafte Strafe bereitet.\qt@no{18}}%
\qt@newcmd\qurantrans@de@dxii{O die ihr glaubt, es ist euch nicht erlaubt, Frauen wider (ihren) Willen zu erben. Und drangsaliert sie nicht, um (ihnen) einen Teil von dem, was ihr ihnen gegeben habt, zu nehmen, außer sie begehen etwas klar Abscheuliches. Und geht in rechtlicher Weise mit ihnen um. Wenn sie euch zuwider sind, so ist euch vielleicht etwas zuwider, während Allah viel Gutes in es hineinlegt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@dxiii{Und wenn ihr eine Gattin anstelle einer anderen eintauschen wollt und ihr der einen von ihnen einen Qintar gegeben habt, dann nehmt nichts davon (zurück). Wollt ihres (etwa) durch Verleumdung und deutliche Sünde (zurück)nehmen?\qt@no{20}}%
\qt@newcmd\qurantrans@de@dxiv{Wie könnt ihr es (zurück)nehmen, wo ihr doch zueinander eingegangen seid und sie mit euch ein festes Abkommen getroffen haben?\qt@no{21}}%
\qt@newcmd\qurantrans@de@dxv{Und heiratet nicht Frauen, die (vorher) eure Väter geheiratet haben, außer dem, was bereits geschehen ist. Gewiß, das ist eine Abscheulichkeit und etwas Hassenswertes und ein böser Weg.\qt@no{22}}%
\qt@newcmd\qurantrans@de@dxvi{Verboten (zu heiraten) sind euch eure Mütter, eure Töchter, eure Schwestern, eure Tanten väterlicherseits, eure Tanten mütterlicherseits, die Nichten, eure Nährmütter, die euch gestillt haben, eure Milchschwestem, die Mütter eurer Frauen, eure Stieftöchter, die sich im Schoß eurer Familie befinden von euren Frauen, zu denen ihr eingegangen seid, - wenn ihr jedoch nicht zu ihnen eingegangen seid, so ist es keine Sünde für euch (, deren Töchter zu heiraten) - und (verboten zu heiraten sind euch) die Ehefrauen eurer Söhne, die aus euren Lenden (hervorgegangen) sind, und (verboten ist es euch,) daß ihr zwei Schwestern zusammen (zur Frau) nehmt, außer dem, was bereits geschehen ist. Gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{23}}%
\qt@newcmd\qurantrans@de@dxvii{Und (verboten sind euch) von den Frauen die verheirateten, außer denjenigen, die eure rechte Hand besitzt. (Dies gilt) als Allahs Vorschrift für euch. Erlaubt ist euch, was darüber hinausgeht, (nämlich) daß ihr mit eurem Besitz (Frauen) begehrt zur Ehe und nicht zur Hurerei (Frauen). Welche von ihnen ihr dann genossen habt, denen gebt ihren Lohn als Pflichtteil. Es liegt aber keine Sünde für euch darin, daß ihr, nachdem der Pflichtteil (festgelegt) ist, (darüberhinausgehend) euch miteinander einigt. Gewiß, Allah ist Allwissend und Allweise.\qt@no{24}}%
\qt@newcmd\qurantrans@de@dxviii{Und wer von euch nicht so bemittelt ist, daß er ehrbare, gläubige Frauen zu heiraten vermag, der (soll) von den gläubigen Mädchen (heiraten), die eure rechte Hand besitzt. Und Allah weiß sehr wohl über euren Glauben Bescheid; die einen von euch sind von den anderen. So heiratet sie mit der Erlaubnis ihrer Angehörigen und gebt ihnen ihren Lohn in rechtlicher Weise, wenn sie ehrbar sind, nicht solche, die Hurerei treiben und sich Liebhaber halten! Und wenn sie (durch Heirat) ehrbare Frauen geworden sind und dann eine Abscheulichkeit begehen, soll ihnen (nur) die Hälfte der Strafe zukommen, die ehrbaren Frauen zukommt. Dies gilt für denjenigen von euch, der (in) Bedrängnis (zu kommen) fürchtet. Doch daß ihr Geduld übt, ist besser für euch. Allah ist Allvergebend und Barmherzig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@dxix{Allah will euch Klarheit geben und euch rechtleiten nach den Gesetzmäßigkeiten derer, die vor euch waren, und eure Reue annehmen. Allah ist Allwissend und Allweise.\qt@no{26}}%
\qt@newcmd\qurantrans@de@dxx{Und Allah will eure Reue annehmen; diejenigen aber, die den Begierden folgen, wollen, daß ihr (vom rechten Weg) völlig abweicht.\qt@no{27}}%
\qt@newcmd\qurantrans@de@dxxi{Allah will es euch leicht machen, denn der Mensch ist (ja) schwach erschaffen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@dxxii{O die ihr glaubt, zehrt nicht euren Besitz untereinander auf nichtige Weise auf, es sei denn, daß es sich um einen Handel in gegenseitigem Einvernehmen handelt. Und tötet euch nicht selbst (gegenseitig). Allah ist gewiß Barmherzig gegen euch.\qt@no{29}}%
\qt@newcmd\qurantrans@de@dxxiii{Und wer dies in feindseligem Vorgehen und ungerechterweise tut, den werden Wir einem Feuer aussetzen. Und dies ist Allah ein leichtes.\qt@no{30}}%
\qt@newcmd\qurantrans@de@dxxiv{Wenn ihr die schwerwiegenden (Dinge) meidet, die euch verboten sind, tilgen Wir euch eure bösen Taten und gewähren euch auf eine ehrenvolle Weise Eingang (in den Paradiesgarten).\qt@no{31}}%
\qt@newcmd\qurantrans@de@dxxv{Und wünscht euch nicht das, womit Allah die einen von euch vor den anderen bevorzugt hat. Den Männern kommt ein Anteil von dem zu, was sie verdient haben, und den Frauen kommt ein Anteil von dem zu, was sie verdient haben. Und bittet Allah (um etwas) von Seiner Huld. Allah weiß über alles Bescheid.\qt@no{32}}%
\qt@newcmd\qurantrans@de@dxxvi{Einem jeden haben Wir Erbberechtigte bestimmt für das, was die Eltern und nächsten Verwandten hinterlassen. Denjenigen, mit denen eure rechte Hand eine Abmachung getroffen hat, gebt ihnen ihren Anteil. Allah ist über alles Zeuge.\qt@no{33}}%
\qt@newcmd\qurantrans@de@dxxvii{Die Männer stehen in Verantwortung für die Frauen wegen dessen, womit Allah die einen von ihnen vor den anderen ausgezeichnet hat und weil sie von ihrem Besitz (für sie) ausgeben. Darum sind die rechtschaffenen Frauen (Allah) demütig ergeben und hüten das zu Verbergende, weil Allah (es) hütet. Und diejenigen, deren Widersetzlichkeit ihr befürchtet, - ermahnt sie, meidet sie im Ehebett und schlagt sie. Wenn sie euch aber gehorchen, dann sucht kein Mittel gegen sie. Allah ist Erhaben und Groß.\qt@no{34}}%
\qt@newcmd\qurantrans@de@dxxviii{Und wenn ihr Widerstreit zwischen den beiden befürchtet, dann setzt einen Schiedsrichter aus seiner Familie und einen Schiedsrichter aus ihrer Familie ein. Wenn sie (beide) eine Aussöhnung wollen, wird Allah sie (beide) in Einklang bringen. Gewiß, Allah ist Allwissend und Allkundig.\qt@no{35}}%
\qt@newcmd\qurantrans@de@dxxix{Und dient Allah und gesellt Ihm nichts bei. Und zu den Eltern sollt ihr gütig sein und zu den Verwandten, den Waisen, den Armen, dem verwandten Nachbarn, dem fremden Nachbarn, dem Gefährten zur Seite, dem Sohn des Weges und denen, die eure rechte Hand besitzt. Allah liebt nicht, wer eingebildet und prahlerisch ist,\qt@no{36}}%
\qt@newcmd\qurantrans@de@dxxx{diejenigen, die geizen und den Menschen befehlen, geizig zu sein, und verbergen, was Allah ihnen von Seiner Huld gewährt hat, - für die Ungläubigen haben Wir schmachvolle Strafe bereitet -\qt@no{37}}%
\qt@newcmd\qurantrans@de@dxxxi{und diejenigen, die ihren Besitz ausgeben, um von den Menschen gesehen zu werden, und nicht an Allah und (auch) nicht an den Jüngsten Tag glauben; wer den Satan zum Gesellen hat, der hat da einen bösen Gesellen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@dxxxii{Was würde es ihnen denn ausmachen, wenn sie an Allah und den Jüngsten Tag glaubten und von dem, womit Allah sie versorgt hat, ausgäben? Aber Allah weiß über sie wohl Bescheid.\qt@no{39}}%
\qt@newcmd\qurantrans@de@dxxxiii{Allah tut gewiß nicht im Gewicht eines Stäubchens Unrecht. Und wenn es eine gute Tat ist, vervielfacht Er sie und gibt von Sich aus großartigen Lohn.\qt@no{40}}%
\qt@newcmd\qurantrans@de@dxxxiv{Aber wie (wird es sein), wenn Wir von jeder Gemeinschaft einen Zeugen bringen und dich über diese da als Zeugen bringen?\qt@no{41}}%
\qt@newcmd\qurantrans@de@dxxxv{An jenem Tag hätten es diejenigen gern, die ungläubig waren und sich dem Gesandten widersetzten, wenn sie gleich der Erde wären. Und sie werden vor Allah keine Aussage verheimlichen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@dxxxvi{O die ihr glaubt, nähert euch nicht dem Gebet, während ihr trunken seid, bis ihr wißt, was ihr sagt, noch im Zustand der Unreinheit - es sei denn, ihr geht bloß vorbei -, bis ihr den ganzen (Körper) gewaschen habt. Und wenn ihr krank seid oder auf einer Reise oder jemand von euch vom Abort kommt oder ihr Frauen berührt habt und dann kein Wasser findet, so wendet euch dem guten Erdboden zu und streicht euch über das Gesicht und die Hände. Allah ist Allverzeihend und Allvergebend.\qt@no{43}}%
\qt@newcmd\qurantrans@de@dxxxvii{Siehst du nicht jene, denen ein Teil der Schrift gegeben wurde? Sie erkaufen das Irregehen und wollen, daß (auch) ihr vom Weg abirrt.\qt@no{44}}%
\qt@newcmd\qurantrans@de@dxxxviii{Allah weiß sehr wohl über eure Feinde Bescheid, und Allah genügt als Schutzherr, und Allah genügt als Helfer.\qt@no{45}}%
\qt@newcmd\qurantrans@de@dxxxix{Unter denjenigen, die dem Judentum angehören, verdrehen manche den Sinn der Worte und sagen: ``Wir hören, doch wir widersetzen uns" und: ``Höre!", als ob du nicht hörtest ``raina", wobei sie mit ihren Zungen verdrehen und die Religion schmähen. Wenn sie gesagt hätten: ``Wir hören und gehorchen" und: ``Höre!" und: ``unzurna", wäre es wahrlich besser und richtiger für sie. Aber Allah hat sie für ihren Unglauben verflucht. Darum glauben sie nur wenig.\qt@no{46}}%
\qt@newcmd\qurantrans@de@dxl{O ihr, denen die Schrift gegeben wurde, glaubt an das, was Wir offenbart haben, das zu bestätigen, was euch (bereits) vorliegt, bevor Wir Gesichter auslöschen und nach hinten versetzen oder sie verfluchen, wie Wir die Sabbatleute verfluchten. Allahs Anordnung wird (stets) ausgeführt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@dxli{Allah vergibt gewiß nicht, daß man Ihm (etwas) beigesellt. Doch was außer diesem ist, vergibt Er, wem Er will. Wer Allah (etwas) beigesellt, der hat fürwahr eine gewaltige Sünde ersonnen.\qt@no{48}}%
\qt@newcmd\qurantrans@de@dxlii{Siehst du nicht jene, die sich selbst für lauter erklären? Nein! Vielmehr läutert Allah, wen Er will. Und ihnen wird nicht um ein Fädchen Unrecht zugefügt.\qt@no{49}}%
\qt@newcmd\qurantrans@de@dxliii{Schau, wie sie gegen Allah Lügen ersinnen! Das (allein schon) genügt als deutliche Sünde.\qt@no{50}}%
\qt@newcmd\qurantrans@de@dxliv{Siehst du nicht jene, denen ein Teil der Schrift gegeben wurde? Sie glauben an Zauberei und falsche Götter und sagen von denen, die ungläubig sind: ``Diese da sind eher auf dem rechten Weg geleitet als die Gläubigen."\qt@no{51}}%
\qt@newcmd\qurantrans@de@dxlv{Das sind diejenigen, die Allah verflucht hat; und wen Allah verflucht, für den wirst du keinen Helfer finden.\qt@no{52}}%
\qt@newcmd\qurantrans@de@dxlvi{Oder haben sie (etwa) einen Anteil an der Herrschaft? Dann würden sie den Menschen nicht ein Dattelkerngrübchen (ab)geben.\qt@no{53}}%
\qt@newcmd\qurantrans@de@dxlvii{Oder beneiden sie die Menschen um das, was Allah ihnen von Seiner Huld gegeben hat? Nun, Wir gaben der Sippe Ibrahims die Schrift und die Weisheit und gaben ihnen gewaltige Herrschaft.\qt@no{54}}%
\qt@newcmd\qurantrans@de@dxlviii{Unter ihnen gab es manche, die daran glaubten, und manche, die davon abhielten. Und die Hölle genügt als Feuerglut.\qt@no{55}}%
\qt@newcmd\qurantrans@de@dxlix{Diejenigen, die Unsere Zeichen verleugnen, werden Wir gewiß einem Feuer aussetzen. Jedesmal, wenn ihre Haut verbrannt ist, tauschen Wir sie ihnen gegen eine andere Haut aus, damit sie die Strafe kosten. Allah ist Allmächtig und Allweise.\qt@no{56}}%
\qt@newcmd\qurantrans@de@dl{Diejenigen aber, die glauben und rechtschaffene Werke tun, werden Wir in Gärten eingehen lassen, durcheilt von Bächen, ewig und auf immer darin zu bleiben. Darin haben sie vollkommen gereinigte Gattinnen; und Wir lassen sie in vollkommenen Schatten eingehen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@dli{Allah befiehlt euch, anvertraute Güter ihren Eigentümern (wieder) auszuhändigen und, wenn ihr zwischen den Menschen richtet, in Gerechtigkeit zu richten. Wie trefflich ist das, womit Allah euch ermahnt! Gewiß, Allah ist Allhörend und Allsehend.\qt@no{58}}%
\qt@newcmd\qurantrans@de@dlii{O die ihr glaubt, gehorcht Allah und gehorcht dem Gesandten und den Befehlshabern unter euch! Wenn ihr miteinander über etwas streitet, dann bringt es vor Allah und den Gesandten, wenn ihr wirklich an Allah und den Jüngsten Tag glaubt. Das ist am besten und am ehesten ein guter Ausgang.\qt@no{59}}%
\qt@newcmd\qurantrans@de@dliii{Siehst du nicht jene, die behaupten, an das zu glauben, was zu dir (als Offenbarung) herabgesandt worden ist, und was vor dir herabgesandt wurde, während sie sich in Entscheidungsfragen an falsche Götter wenden wollen, wo ihnen doch befohlen worden ist, es zu verleugnen'? Aber der Satan will sie weit in die Irre führen.\qt@no{60}}%
\qt@newcmd\qurantrans@de@dliv{Und wenn man zu ihnen sagt: ``Kommt her zu dem, was Allah (als Offenbarung) herabgesandt hat, und zum Gesandten", siehst du die Heuchler sich nachdrücklich abwenden.\qt@no{61}}%
\qt@newcmd\qurantrans@de@dlv{Aber wie (ist es), wenn sie ein Unglück trifft für das, was ihre Hände vorausgeschickt haben, und sie hierauf zu dir kommen und bei Allah schwören: ``Wir wollten (es) ja nur gut machen und Einklang herstellen"?\qt@no{62}}%
\qt@newcmd\qurantrans@de@dlvi{Das sind diejenigen, von denen Allah weiß, was in ihren Herzen ist. So wende dich von ihnen ab, ermahne sie und sag zu ihnen über sie selbst eindringliche Worte.\qt@no{63}}%
\qt@newcmd\qurantrans@de@dlvii{Und Wir haben keinen Gesandten gesandt, ohne daß ihm - mit Allahs Erlaubnis gehorcht werde. Und wenn sie, wo sie sich selbst Unrecht zugefügt haben, zu dir kämen und dann Allah um Vergebung bäten, und der Gesandte für sie um Vergebung bäte, würden sie wahrlich Allah Reue-Annehmend und Barmherzig finden.\qt@no{64}}%
\qt@newcmd\qurantrans@de@dlviii{Aber nein, bei deinem Herrn! Sie glauben nicht eher, bis sie dich über das richten lassen, was zwischen ihnen umstritten ist, und hierauf in sich selbst keine Bedrängnis finden durch das, was du entschieden hast, und sich in voller Ergebung fügen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@dlix{Wenn Wir ihnen vorschreiben würden: ``Tötet euch selbst oder zieht aus euren Wohnstätten aus", so täten sie es nicht, außer wenigen von ihnen. Wenn sie aber tun würden, wozu sie ermahnt werden, wäre es wahrlich besser für sie und eine größere Stärkung (im Glauben).\qt@no{66}}%
\qt@newcmd\qurantrans@de@dlx{Dann würden Wir ihnen wahrlich von Uns aus großartigen Lohn geben\qt@no{67}}%
\qt@newcmd\qurantrans@de@dlxi{und würden sie wahrlich (auf) einen geraden Weg leiten.\qt@no{68}}%
\qt@newcmd\qurantrans@de@dlxii{Wer Allah und dem Gesandten gehorcht, die werden mit denjenigen von den Propheten, den Wahrheitsliebenden, den Märtyrern und den Rechtschaffenen Zusammensein, denen Allah Gunst erwiesen hat. Welch gute Gefährten jene sind!\qt@no{69}}%
\qt@newcmd\qurantrans@de@dlxiii{Das ist die Huld von Allah, und Allah genügt als Allwissender.\qt@no{70}}%
\qt@newcmd\qurantrans@de@dlxiv{O die ihr glaubt, seid auf eurer Hut. Und rückt dann in Trupps aus oder rückt alle aus.\qt@no{71}}%
\qt@newcmd\qurantrans@de@dlxv{Unter euch gibt es sehr wohl manch einen, der ganz langsam tut. Wenn euch dann ein Unglück trifft, sagt er: ``Allah hat mir fürwahr eine Gunst erwiesen, da ich nicht mit ihnen zugegen war."\qt@no{72}}%
\qt@newcmd\qurantrans@de@dlxvi{Wenn euch aber Huld von Allah zuteil wird, sagt er ganz gewiß, als ob zwischen euch und ihm keine Freundschaft bestanden hätte: ``O wäre ich doch mit ihnen zugegen gewesen! Dann hätte ich einen großartigen Erfolg erzielt."\qt@no{73}}%
\qt@newcmd\qurantrans@de@dlxvii{So sollen denn diejenigen auf Allahs Weg kämpfen, die das diesseitige Leben für das Jenseits verkaufen. Und wer auf Allahs Weg kämpft und dann getötet wird oder siegt, dem werden Wir großartigen Lohn geben.\qt@no{74}}%
\qt@newcmd\qurantrans@de@dlxviii{Was ist mit euch, daß ihr nicht auf Allahs Weg, und (zwar) für die Unterdrückten unter den Männern, Frauen und Kindern kämpft, die sagen: ``Unser Herr, bringe uns aus dieser Stadt heraus, deren Bewohner ungerecht sind, und schaffe uns von Dir aus einen Schutzherrn, und schaffe uns von Dir aus einen Helfer."\qt@no{75}}%
\qt@newcmd\qurantrans@de@dlxix{Diejenigen, die glauben, kämpfen auf Allahs Weg, und diejenigen, die ungläubig sind, kämpfen auf dem Weg der falschen Götter. So kämpft gegen die Gefolgsleute des Satans! Gewiß, die List des Satans ist schwach.\qt@no{76}}%
\qt@newcmd\qurantrans@de@dlxx{Siehst du nicht jene, zu denen gesagt wurde: ``Haltet eure Hände zurück und verrichtet das Gebet und entrichtet die Abgabe." Als ihnen dann aber vorgeschrieben wurde zu kämpfen, fürchtete auf einmal eine Gruppe von ihnen die Menschen wie (sie) Furcht vor Allah (haben) oder mit noch größerer Furcht, und sagten: ``Unser Herr, warum hast Du uns vorgeschrieben zu kämpfen? O würdest Du uns doch auf eine kurze Frist zurückstellen!" Sag: Der Genuß des Diesseits ist gering. Und das Jenseits ist besser für jemanden, der gottesfürchtig ist. Und euch wird nicht um ein Fädchen Unrecht zugefügt.\qt@no{77}}%
\qt@newcmd\qurantrans@de@dlxxi{Wo immer ihr auch seid, wird euch der Tod erfassen, und wäret ihr in hochgebauten Türmen. Und wenn sie etwas Gutes trifft, sagen sie: ``Das ist von Allah." Und wenn sie etwas Böses trifft, sagen sie: ``Das ist von dir." Sag: Alles ist von Allah. Was ist mit diesem Volk, daß sie beinahe keine Aussage verstehen?\qt@no{78}}%
\qt@newcmd\qurantrans@de@dlxxii{Was dich an Gutem trifft, ist von Allah, und was dich an Bösem trifft, ist von dir selbst. Und Wir haben dich als Gesandten für die Menschen gesandt. Und Allah genügt als Zeuge.\qt@no{79}}%
\qt@newcmd\qurantrans@de@dlxxiii{Wer dem Gesandten gehorcht, der gehorcht Allah, und wer sich abkehrt, - so haben Wir dich nicht als Hüter über sie entsandt.\qt@no{80}}%
\qt@newcmd\qurantrans@de@dlxxiv{Und sie sagen: ``Gehorsam." Wenn sie aber von dir weggehen, heckt ein Teil von ihnen etwas anderes aus, als was du sagst. Aber Allah schreibt auf, was sie aushecken. So wende dich von ihnen ab und verlasse dich auf Allah. Allah genügt als Sachwalter.\qt@no{81}}%
\qt@newcmd\qurantrans@de@dlxxv{Denken sie denn nicht sorgfältig über den Qur'an nach? Wenn er von jemand anderem wäre als von Allah, würden sie in ihm wahrlich viel Widerspruch finden.\qt@no{82}}%
\qt@newcmd\qurantrans@de@dlxxvi{Und wenn ihnen eine Angelegenheit zu (Ohren) kommt, die Sicherheit oder Furcht betrifft, machen sie es bekannt. Wenn sie es jedoch vor den Gesandten und den Befehlshabern unter ihnen brächten, würden es wahrlich diejenigen unter ihnen wissen, die es herausfinden können. Und wenn nicht Allahs Huld und Erbarmen gewesen wären, wäret ihr fürwahr außer wenigen dem Satan gefolgt.\qt@no{83}}%
\qt@newcmd\qurantrans@de@dlxxvii{So kämpfe auf Allahs Weg - du wirst nur für dich selbst verantwortlich gemacht und sporne die Gläubigen an. Vielleicht wird Allah die Gewalt derjenigen, die ungläubig sind, zurückhalten. Allah ist stärker an Gewalt und an Bestrafung.\qt@no{84}}%
\qt@newcmd\qurantrans@de@dlxxviii{Wer (in) eine(r) gute(n Sache) Fürsprache einlegt, für den wird es einen Anteil daran geben, und wer (in) eine(r) schlechte(n Sache) Fürsprache einlegt, für den wird es ein gleiches davon geben. Und Allah ist zu allem Fähig.\qt@no{85}}%
\qt@newcmd\qurantrans@de@dlxxix{Und wenn euch ein Gruß entboten wird, dann grüßt mit einem schöneren (zurück) oder erwidert ihn. Gewiß, Allah ist über alles ein Abrechner.\qt@no{86}}%
\qt@newcmd\qurantrans@de@dlxxx{Allah - es gibt keinen Gott außer Ihm. Er wird euch ganz gewiß zum Tag der Auferstehung versammeln, an dem es keinen Zweifel gibt. Und wer ist wahrhaftiger als Allah in der Aussage?\qt@no{87}}%
\qt@newcmd\qurantrans@de@dlxxxi{Was ist mit euch, daß ihr hinsichtlich der Heuchler (in) zwei Scharen (gespalten) seid, wo doch Allah sie wegen dessen, was sie verdient haben, umgekehrt hat? Wollt ihr denn rechtleiten, wen Allah in die Irre gehen läßt? Wen aber Allah in die Irre gehen läßt, für den wirst du keinen Weg finden.\qt@no{88}}%
\qt@newcmd\qurantrans@de@dlxxxii{Sie möchten gern, daß ihr ungläubig werdet, wie sie ungläubig sind, so daß ihr (alle) gleich seiet. Nehmt euch daher von ihnen keine Vertrauten, bevor sie nicht auf Allahs Weg auswandern! Kehren sie sich jedoch ab, dann ergreift sie und tötet sie, wo immer ihr sie findet, und nehmt euch von ihnen weder Schutzherrn noch Helfer,\qt@no{89}}%
\qt@newcmd\qurantrans@de@dlxxxiii{außer denjenigen, die sich einem Volk anschließen, zwischen dem und euch ein Abkommen besteht, oder die zu euch gekommen sind, weil ihre Brüste beklommen sind, gegen euch zu kämpfen oder gegen ihr (eigenes) Volk zu kämpfen. Und wenn Allah gewollt hätte, hätte Er ihnen wahrlich Gewalt über euch gegeben, und dann hätten sie gegen euch wahrlich gekämpft. Wenn sie sich jedoch von euch fernhalten und dann nicht gegen euch kämpfen, sondern Frieden anbieten, so hat euch Allah keine Veranlassung gegeben, gegen sie (vorzugehen).\qt@no{90}}%
\qt@newcmd\qurantrans@de@dlxxxiv{Ihr werdet andere finden, die vor euch Sicherheit und vor ihrem (eigenen) Volk Sicherheit wollen. Jedesmal, wenn sie wieder der Versuchung ausgesetzt sind, werden sie in ihr zu Fall gebracht. Wenn sie sich nicht von euch fernhalten und euch nicht Frieden anbieten und nicht ihre Hände zurückhalten, dann ergreift sie und tötet sie, wo immer ihr auf sie trefft. Über jene haben Wir euch deutliche Gewalt verliehen.\qt@no{91}}%
\qt@newcmd\qurantrans@de@dlxxxv{Es steht keinem Gläubigen zu, einen (anderen) Gläubigen zu töten, es sei denn aus Versehen'. Und wer einen Gläubigen aus Versehen tötet, (der hat) einen gläubigen Sklaven (zu) befreien und ein Blutgeld an seine Angehörigen aus(zu)händigen, es sei denn, sie erlassen (es ihm) als Almosen. Wenn er (der Getötete) zu einem euch feindlichen Volk gehörte und gläubig war, dann (gilt es,) einen gläubigen Sklaven (zu) befreien. Und wenn er zu einem Volk gehörte, zwischen dem und euch ein Abkommen besteht, dann (gilt es,) ein Blutgeld an seine Angehörigen aus(zu)händigen und einen gläubigen Sklaven (zu) befreien. Wer aber keine (Möglichkeit) findet, (der hat) zwei aufeinanderfolgende Monate (zu) fasten - als eine Reueannahme von Allah. Und Allah ist Allwissend und Allweise.\qt@no{92}}%
\qt@newcmd\qurantrans@de@dlxxxvi{Und wer einen Gläubigen vorsätzlich tötet, dessen Lohn ist die Hölle, ewig darin zu bleiben. Und Allah zürnt ihm und verflucht ihn und bereitet ihm gewaltige Strafe.\qt@no{93}}%
\qt@newcmd\qurantrans@de@dlxxxvii{O die ihr glaubt, wenn ihr auf Allahs Weg umherreist, dann unterscheidet klar und sagt nicht zu einem, der euch Frieden anbietet: ``Du bist nicht gläubig", wobei ihr nach den Glücksgütern des diesseitigen Lebens trachtet. Doch bei Allah ist Gutes in Fülle. So wart ihr zuvor. Aber dann hat Allah euch eine Wohltat erwiesen. Unterscheidet also klar. Gewiß, Allah ist dessen, was ihr tut, Kundig.\qt@no{94}}%
\qt@newcmd\qurantrans@de@dlxxxviii{Nicht gleich sind diejenigen unter den Gläubigen, die sitzen bleiben - außer denjenigen, die eine Schädigung haben -, den sich auf Allahs Weg mit ihrem Besitz und ihrer eigenen Person Abmühenden. Allah hat den sich mit ihrem Besitz und ihrer eigenen Person Abmühenden einen Vorzug gegeben vor denen, die sitzen bleiben. Aber allen hat Allah das Beste versprochen. Doch hat Allah die sich Abmühenden vor denen, die sitzen bleiben, mit großartigem Lohn bevorzugt,\qt@no{95}}%
\qt@newcmd\qurantrans@de@dlxxxix{mit Rangstufen von Ihm und Vergebung und Erbarmen. Allah ist Allvergebend und Barmherzig.\qt@no{96}}%
\qt@newcmd\qurantrans@de@dxc{Diejenigen, die die Engel abberufen, während sie sich selbst Unrecht tun, (zu jenen) sagen sie: ``Worin habt ihr euch befunden?" Sie sagen: ``Wir waren Unterdrückte im Lande." Sie (die Engel) sagen: ``War Allahs Erde nicht weit, so daß ihr darauf hättet auswandern können?" Jene aber, - ihr Zufluchtsort wird die Hölle sein, und (wie) böse ist der Ausgang!\qt@no{97}}%
\qt@newcmd\qurantrans@de@dxci{Ausgenommen die Unterdrückten unter den Männern, Frauen und Kindern, die keine Möglichkeit haben auszuwandern und auf dem Weg nicht rechtgeleitet sind.\qt@no{98}}%
\qt@newcmd\qurantrans@de@dxcii{Jenen wird Allah vielleicht verzeihen. Allah ist Allverzeihend und Allvergebend.\qt@no{99}}%
\qt@newcmd\qurantrans@de@dxciii{Wer auf Allahs Weg auswandert, wird auf der Erde viele Zufluchtsstätten und Wohlstand finden. Und wer sein Haus auswandernd zu Allah und Seinem Gesandten verläßt, und den hierauf der Tod erfaßt, so fällt es Allah zu, ihm seinen Lohn (zu geben). Allah ist Allvergebend und Barmherzig.\qt@no{100}}%
\qt@newcmd\qurantrans@de@dxciv{Und wenn ihr im Land umherreist, so ist es keine Sünde für euch, das Gebet abzukürzen, wenn ihr befürchtet, diejenigen, die ungläubig sind, könnten euch überfallen. Die Ungläubigen sind euch ja ein deutlicher Feind.\qt@no{101}}%
\qt@newcmd\qurantrans@de@dxcv{Und wenn du unter ihnen bist und nun für sie das Gebet anfuhrst, dann soll sich ein Teil von ihnen mit dir aufstellen, doch sollen sie ihre Waffen nehmen. Wenn sie (die Betenden) sich dann niedergeworfen haben, sollen sie hinter euch treten', und ein anderer Teil, die noch nicht gebetet hat, soll (nach vorne) kommen. Und sie sollen dann (auch) mit dir beten; doch sollen sie auf ihrer Hut sein und ihre Waffen nehmen. Diejenigen, die ungläubig sind, möchten, daß ihr eure Waffen und eure Sachen außer acht laßt, so daß sie auf einmal über euch herfallen (können). Es ist keine Sünde für euch, wenn ihr unter Regen zu leiden habt oder krank seid, eure Waffen abzulegen. Aber seid auf eurer Hut. Gewiß, Allah bereitet für die Ungläubigen schmachvolle Strafe.\qt@no{102}}%
\qt@newcmd\qurantrans@de@dxcvi{Wenn ihr das Gebet vollzogen habt, dann gedenkt Allahs stehend, sitzend und auf der Seite (liegend). Wenn ihr in Ruhe seid, dann verrichtet das Gebet (wie üblich). Das Gebet ist den Gläubigen zu bestimmten Zeiten vorgeschrieben.\qt@no{103}}%
\qt@newcmd\qurantrans@de@dxcvii{Und laßt nicht nach, das Volk (der feindlichen Ungläubigen) zu suchen. Wenn ihr zu leiden habt, so haben sie zu leiden wie ihr. Ihr aber habt von Allah zu erhoffen, was sie nicht zu erhoffen haben. Und Allah ist Allwissend und Allweise.\qt@no{104}}%
\qt@newcmd\qurantrans@de@dxcviii{Gewiß, Wir haben dir das Buch mit der Wahrheit hinabgesandt, damit du zwischen den Menschen richtest auf Grund dessen, was Allah dir gezeigt hat. Sei kein Verfechter für die Verräter!\qt@no{105}}%
\qt@newcmd\qurantrans@de@dxcix{Und bitte Allah um Vergebung. Gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{106}}%
\qt@newcmd\qurantrans@de@dc{Und streite nicht zur Verteidigung derer, die sich selbst betrügen. Allah liebt nicht, wer ein Verräter und Sünder ist.\qt@no{107}}%
\qt@newcmd\qurantrans@de@dci{Sie verbergen sich vor den Menschen; verbergen sich jedoch nicht vor Allah, wo Er doch bei ihnen ist, wenn sie an Worten aushecken, womit Er nicht zufrieden ist. Allah umfaßt, was sie tun.\qt@no{108}}%
\qt@newcmd\qurantrans@de@dcii{Ihr da seid es doch, die ihr im diesseitigen Leben zu ihrer Verteidigung gestritten habt. Aber wer wird am Tag der Auferstehung mit Allah zu ihrer Verteidigung streiten, oder wer wird (dann) ihr Sachwalter sein?\qt@no{109}}%
\qt@newcmd\qurantrans@de@dciii{Und wer etwas Böses tut oder sich selbst Unrecht zufügt und hierauf Allah um Vergebung bittet, wird Allah Allvergebend und Barmherzig finden.\qt@no{110}}%
\qt@newcmd\qurantrans@de@dciv{Und wer eine Sünde begeht', begeht sie nur gegen sich selbst. Allah ist Allwissend und Allweise.\qt@no{111}}%
\qt@newcmd\qurantrans@de@dcv{Und wer eine Verfehlung oder eine Sünde begeht und sie hierauf einem Unschuldigen vorwirft, der lädt damit Verleumdung und deutliche Sünde auf sich.\qt@no{112}}%
\qt@newcmd\qurantrans@de@dcvi{Und wenn nicht Allahs Huld gegen dich und Sein Erbarmen gewesen wären, hätte ein Teil von ihnen wahrlich vorgehabt, dich in die Irre zu führen. Aber sie führen nur sich selbst in die Irre und fügen dir keinerlei Schaden zu. Allah hat auf dich das Buch und die Weisheit herabgesandt und dich gelehrt, was du (vorher) nicht wußtest. Und Allahs Huld gegen dich ist groß.\qt@no{113}}%
\qt@newcmd\qurantrans@de@dcvii{Nichts Gutes ist in vielen ihrer vertraulichen Gespräche, außer derer, die Almosen, Rechtes oder Aussöhnung unter den Menschen befehlen. Und wer dies im Trachten nach Allahs Zufriedenheit tut, dem werden Wir großartigen Lohn geben.\qt@no{114}}%
\qt@newcmd\qurantrans@de@dcviii{Wer aber dem Gesandten entgegenwirkt, nachdem ihm die Rechtleitung klargeworden ist, und einem, anderen Weg als dem der Gläubigen folgt, werden Wir dem zukehren, dem er sich zugekehrt hat, und ihn der Hölle aussetzen, und (wie) böse ist der Ausgang!\qt@no{115}}%
\qt@newcmd\qurantrans@de@dcix{Allah vergibt gewiß nicht, daß man Ihm (etwas) beigesellt. Doch was außer diesem ist, vergibt Er, wem Er will. Wer Allah (etwas) beigesellt, der ist fürwahr weit abgeirrt.\qt@no{116}}%
\qt@newcmd\qurantrans@de@dcx{Sie rufen außer Ihm nur weibliche (Gottheiten) an, und sie rufen nur einen rebellischen Satan an.\qt@no{117}}%
\qt@newcmd\qurantrans@de@dcxi{Allah verfluchte ihn. Und er sagte: ``Ich werde von Deinen Dienern ganz gewiß einen festgesetzten Anteil nehmen,\qt@no{118}}%
\qt@newcmd\qurantrans@de@dcxii{und ich werde sie ganz gewiß in die Irre führen und ganz gewiß in ihnen Wünsche erwecken und ihnen ganz gewiß befehlen, und da werden sie ganz gewiß die Ohren des Viehs abschneiden; wahrlich, ich werde ihnen befehlen, und da werden sie ganz gewiß Allahs Schöpfung ändern." Wer sich den Satan außer Allah zum Schutzherrn nimmt, der hat fürwahr einen offenkundigen Verlust erlitten.\qt@no{119}}%
\qt@newcmd\qurantrans@de@dcxiii{Er macht ihnen Versprechungen und erweckt in ihnen Wünsche; aber der Satan macht ihnen nur Versprechungen in Trug.\qt@no{120}}%
\qt@newcmd\qurantrans@de@dcxiv{Jene - ihr Zufluchtsort wird die Hölle sein, und sie werden aus ihr kein Entrinnen finden.\qt@no{121}}%
\qt@newcmd\qurantrans@de@dcxv{Diejenigen aber, die glauben und rechtschaffene Werke tun, die werden Wir in Gärten eingehen lassen, durcheilt von Bächen, ewig und auf immer darin zu bleiben - Allahs Versprechen in Wahrheit. Und wer ist wahrhaftiger als Allah in Seinen Worten?\qt@no{122}}%
\qt@newcmd\qurantrans@de@dcxvi{Es geht weder nach euren Wünschen noch nach den Wünschen der Leute der Schrift. Wer Böses tut, dem wird es vergolten, und der findet für sich außer Allah weder Schutzherrn noch Helfer.\qt@no{123}}%
\qt@newcmd\qurantrans@de@dcxvii{Wer aber, sei es Mann oder Frau, etwas an rechtschaffenen Werken tut, und dabei gläubig ist, jene werden in den (Paradies)garten eingehen, und es wird ihnen nicht ein Dattelkerngrübchen Unrecht zugefügt.\qt@no{124}}%
\qt@newcmd\qurantrans@de@dcxviii{Wer hätte eine bessere Religion, als wer sein Gesicht Allah hingibt und dabei Gutes tut und dem Glaubensbekenntnis Ibrahims folgt, (als) Anhänger des rechten Glaubens? Und Allah nahm sich Ibrahim zum Freund.\qt@no{125}}%
\qt@newcmd\qurantrans@de@dcxix{Und Allah gehört (alles), was in den Himmeln und was auf der Erde ist; Allah umfaßt alles.\qt@no{126}}%
\qt@newcmd\qurantrans@de@dcxx{Sie fragen dich um Belehrung über die Frauen. Sag: ``Allah belehrt euch über sie, und (weiter lehrt,) was euch im Buch verlesen wird über die weiblichen Waisen, denen ihr nicht gebt, was ihnen vorgeschrieben ist, und die ihr zu heiraten begehrt, und die schwachen unter den Kindern, und daß ihr gegenüber den Waisen Gerechtigkeit wahrt. Und was immer ihr an Gutem tut, so weiß Allah darüber Bescheid.\qt@no{127}}%
\qt@newcmd\qurantrans@de@dcxxi{Und wenn eine Frau von ihrem Ehemann Widersetzlichkeit oder Meidung befürchtet, so ist es keine Sünde für sie (beide), sich in Frieden zu einigen; denn friedliche Einigung ist besser. Und die Seelen sind der Habsucht zugänglich. Doch wenn ihr wohltätig seid und gottesfürchtig, so ist Allah dessen, was ihr tut, gewiß Kundig.\qt@no{128}}%
\qt@newcmd\qurantrans@de@dcxxii{Und ihr werdet zwischen den Frauen nicht gerecht handeln können, auch wenn ihr danach trachtet. Aber neigt nicht gänzlich (von einer weg zu der anderen), so daß ihr sie gleichsam in der Schwebe laßt. Und wenn ihr (es) wiedergutmacht und gottesfürchtig seid, gewiß, so ist Allah Allvergebend und Barmherzig.\qt@no{129}}%
\qt@newcmd\qurantrans@de@dcxxiii{Und wenn die beiden sich trennen, wird Allah jeden aus Seiner Fülle bereichern. Allah ist Allumfassend und Allweise.\qt@no{130}}%
\qt@newcmd\qurantrans@de@dcxxiv{Allah gehört (alles), was in den Himmeln und was auf der Erde ist. Und Wir haben bereits denjenigen, denen vor euch die Schrift gegeben wurde, und euch anbefohlen: Fürchtet Allah! Wenn ihr aber ungläubig seid, gewiß, so gehört Allah (alles), was in den Himmeln und was auf der Erde ist. Allah ist Unbedürftig und Lobens würdig.\qt@no{131}}%
\qt@newcmd\qurantrans@de@dcxxv{Allah gehört (alles), was in den Himmeln und was auf der Erde ist, und Allah genügt als Sachwalter.\qt@no{132}}%
\qt@newcmd\qurantrans@de@dcxxvi{Wenn Er will, läßt Er euch vergehen, ihr Menschen, und bringt andere. Allah hat dazu die Macht.\qt@no{133}}%
\qt@newcmd\qurantrans@de@dcxxvii{Wer immer die Belohnung des Diesseits will, so ist bei Allah die Belohnung des Diesseits und des Jenseits. Allah ist Allhörend und Allsehend.\qt@no{134}}%
\qt@newcmd\qurantrans@de@dcxxviii{O die ihr glaubt, seid Wahrer der Gerechtigkeit, Zeugen für Allah, auch wenn es gegen euch selbst oder die Eltern und nächsten Verwandten sein sollte! Ob er (der Betreffende) reich oder arm ist, so steht Allah beiden näher. Darum folgt nicht der Neigung, daß ihr nicht gerecht handelt! Wenn ihr (die Wahrheit) verdreht oder euch (davon) abwendet, gewiß, so ist Allah dessen, was ihr tut, Kundig.\qt@no{135}}%
\qt@newcmd\qurantrans@de@dcxxix{O die ihr glaubt, glaubt an Allah und Seinen Gesandten und das Buch, das Er Seinem Gesandten offenbart und die Schrift, die Er zuvor herabgesandt hat. Wer Allah, Seine Engel, Seine Schriften, Seine Gesandten und den Jüngsten Tag verleugnet, der ist fürwahr weit abgeirrt.\qt@no{136}}%
\qt@newcmd\qurantrans@de@dcxxx{Gewiß, diejenigen, die gläubig sind, hierauf ungläubig werden, hierauf (wieder) gläubig werden, hierauf (wieder) ungläubig werden und dann an Unglauben zunehmen - es ist nicht Allahs (Wille), ihnen zu vergeben noch sie einen (rechten) Weg zu leiten.\qt@no{137}}%
\qt@newcmd\qurantrans@de@dcxxxi{Verkünde den Heuchlern, daß es für sie schmerzhafte Strafe geben wird,\qt@no{138}}%
\qt@newcmd\qurantrans@de@dcxxxii{sie,) die die Ungläubigen anstatt der Gläubigen zu Schutzherren nehmen. Begehren sie (etwa) Macht bei ihnen (zu finden)? Gewiß, alle Macht gehört Allah.\qt@no{139}}%
\qt@newcmd\qurantrans@de@dcxxxiii{Er hat euch (doch) bereits im Buch offenbart: Wenn ihr hört, daß man Allahs Zeichen verleugnet und sich über sie lustig macht, dann sitzt nicht mit ihnen (zusammen), bis sie auf ein anderes Gespräch eingehen. Sonst seid ihr ihnen gleich. Gewiß, Allah wird die Heuchler und die Ungläubigen alle in der Hölle versammeln,\qt@no{140}}%
\qt@newcmd\qurantrans@de@dcxxxiv{die euch gegenüber abwarten. Wenn euch ein entscheidender Sieg von Allah beschieden ist, sagen sie: ``Sind wir nicht mit euch gewesen?" Und wenn den Ungläubigen ein Sieg beschieden ist, sagen sie: ``Haben wir uns nicht eurer (Angelegenheiten) bemächtigt gehabt und die Gläubigen von euch abgehalten?" Aber Allah wird zwischen euch am Tag der Auferstehung richten. Und Allah wird den Ungläubigen keine Möglichkeit geben(,) gegen die Gläubigen (vorzugehen).\qt@no{141}}%
\qt@newcmd\qurantrans@de@dcxxxv{Gewiß, die Heuchler möchten Allah betrügen, doch ist Er es, der sie betrügt. Und wenn sie sich zum Gebet hinstellen, stellen sie sich schwerfällig hin, wobei sie von den Menschen gesehen werden wollen, und gedenken Allahs nur wenig;\qt@no{142}}%
\qt@newcmd\qurantrans@de@dcxxxvi{dazwischen hin und her schwankend - weder zu diesen noch zu jenen (gehörend). Und wen Allah in die Irre gehen läßt, für den wirst du keinen Weg finden.\qt@no{143}}%
\qt@newcmd\qurantrans@de@dcxxxvii{O die ihr glaubt, nehmt nicht die Ungläubigen anstatt der Gläubigen zu Schutzherren! Wollt ihr denn Allah eine offenkundige Handhabe gegen euch liefern?\qt@no{144}}%
\qt@newcmd\qurantrans@de@dcxxxviii{Gewiß, die Heuchler werden sich auf dem untersten Grund des (Höllen)feuers befinden, und du wirst für sie keinen Helfer finden,\qt@no{145}}%
\qt@newcmd\qurantrans@de@dcxxxix{- ausgenommen diejenigen, die bereuen und verbessern und an Allah festhalten und ihre Religion aufrichtig für Allah ausüben; jene werden mit den Gläubigen sein. Und Allah wird den Gläubigen großartigen Lohn geben.\qt@no{146}}%
\qt@newcmd\qurantrans@de@dcxl{Warum sollte Allah euch strafen, wenn ihr dankbar und gläubig seid? Allah ist Dankbar und Allwissend.\qt@no{147}}%
\qt@newcmd\qurantrans@de@dcxli{Allah liebt nicht den laut vernehmbaren Gebrauch von bösen Worten, außer durch jemanden, dem Unrecht zugefügt worden ist. Allah ist Allhörend und Allwissend.\qt@no{148}}%
\qt@newcmd\qurantrans@de@dcxlii{Ob ihr etwas Gutes offenlegt, oder es verbergt, oder etwas Böses verzeiht - gewiß, Allah ist Allverzeihend und Allmächtig.\qt@no{149}}%
\qt@newcmd\qurantrans@de@dcxliii{Diejenigen, die Allah und Seine Gesandten verleugnen und zwischen Allah und Seinen Gesandten einen Unterschied machen wollen und sagen: ``Wir glauben an die einen und verleugnen die anderen", und einen Weg dazwischen einschlagen wollen,\qt@no{150}}%
\qt@newcmd\qurantrans@de@dcxliv{das sind die wahren Ungläubigen. Und für die Ungläubigen haben Wir schmachvolle Strafe bereitet.\qt@no{151}}%
\qt@newcmd\qurantrans@de@dcxlv{Denjenigen aber, die an Allah und Seine Gesandten glauben und keinen Unterschied zwischen jemandem von ihnen machen, denen wird Er ihren Lohn geben. Allah ist Allvergebend und Barmherzig.\qt@no{152}}%
\qt@newcmd\qurantrans@de@dcxlvi{Die Leute der Schrift verlangen von dir, daß du ihnen vom Himmel ein Buch offenbaren läßt. Sie haben von Musa bereits etwas noch Größeres als dies verlangt, denn sie sagten: ``Zeige uns Allah unverhüllt!" Da ergriff sie der Donnerschlag wegen ihrer Ungerechtigkeit. Hierauf nahmen sie das Kalb an, nachdem die klaren Beweise zu ihnen gekommen waren. Aber Wir verziehen es und gaben Musa offen kundige Gewalt.\qt@no{153}}%
\qt@newcmd\qurantrans@de@dcxlvii{Und Wir hoben den Berg über sie bei (der Entgegennahme von) ihrem Abkommen empor. Und Wir sagten zu ihnen: ``Tretet, euch niederwerfend, durch das Tor ein!" Und Wir sagten zu ihnen: ``Übertretet nicht den Sabbat!" Und Wir trafen mit ihnen ein festes Abkommen.\qt@no{154}}%
\qt@newcmd\qurantrans@de@dcxlviii{Verflucht sind sie) dafür, daß sie ihr Abkommen brachen und Allahs Zeichen verleugneten und (daß sie) die Propheten zu Unrecht töteten und (daß sie) sagten: ``Unsere Herzen sind verhüllt." - Nein! Vielmehr hat Allah sie für ihren Unglauben versiegelt; darum glauben sie nur wenig,\qt@no{155}}%
\qt@newcmd\qurantrans@de@dcxlix{und daß sie ungläubig waren und gegen Maryam gewaltige Verleumdung aussprachen,\qt@no{156}}%
\qt@newcmd\qurantrans@de@dcl{und dafür, daß sie sagten: ``Gewiß, wir haben al-Masih 'Isa, den Sohn Maryams, den Gesandten Allahs getötet." - Aber sie haben ihn weder getötet noch gekreuzigt, sondern es erschien ihnen so. Und diejenigen, die sich darüber uneinig sind, befinden sich wahrlich im Zweifel darüber. Sie haben kein Wissen darüber, außer daß sie Mutmaßungen folgen. Und sie haben ihn mit Gewißheit nicht getötet.\qt@no{157}}%
\qt@newcmd\qurantrans@de@dcli{Nein! Vielmehr hat Allah ihn zu Sich erhoben. Allah ist Allmächtig und Allweise.\qt@no{158}}%
\qt@newcmd\qurantrans@de@dclii{Es gibt keinen unter den Leuten der Schrift, der nicht noch vor dessen Tod ganz gewiß an ihn glauben wird. Und am Tag der Auferstehung wird er über sie Zeuge sein.\qt@no{159}}%
\qt@newcmd\qurantrans@de@dcliii{Wegen Ungerechtigkeit derer, die dem Judentum angehören, hatten Wir ihnen gute Dinge verboten, die ihnen erlaubt gewesen waren, und weil sie viel von Allahs Weg abhielten,\qt@no{160}}%
\qt@newcmd\qurantrans@de@dcliv{und (weil sie) Zins nahmen, wo es ihnen doch verboten worden war, und den Besitz der Menschen in unrechter Weise aufzehrten. Und Wir haben den Ungläubigen unter ihnen schmerzhafte Strafe bereitet.\qt@no{161}}%
\qt@newcmd\qurantrans@de@dclv{Aber diejenigen unter ihnen, die im Wissen fest gegründet sind, und die Gläubigen glauben an das, was zu dir (an Offenbarung) herabgesandt worden ist, und was vor dir herabgesandt wurde, und diejenigen, die das Gebet verrichten und die Abgabe entrichten und an Allah und den Jüngsten Tag glauben. Ihnen werden Wir großartigen Lohn geben.\qt@no{162}}%
\qt@newcmd\qurantrans@de@dclvi{Gewiß, Wir haben dir (Offenbarung) eingegeben, wie Wir Nuh und den Propheten nach ihm (Offenbarung) eingegeben haben. Und Wir haben Ibrahim, Isma'il, Ishaq, Ya'qub, den Stämmen, ``Isa, Ayyub, Yunus, Harun und Sulaiman (Offenbarung) eingegeben, und Dawud haben Wir ein Buch der Weisheit gegeben.\qt@no{163}}%
\qt@newcmd\qurantrans@de@dclvii{Und (Wir sandten) Gesandte, über die Wir dir schon zuvor berichtet haben, und andere, über die Wir dir nicht berichtet haben, und zu Musa hat Allah unmittelbar gesprochen -,\qt@no{164}}%
\qt@newcmd\qurantrans@de@dclviii{Gesandte als Verkünder froher Botschaft und als Überbringer von Warnungen, damit die Menschen nach den Gesandten kein Beweismittel gegen Allah haben. Und Allah ist Allmächtig und Allweise.\qt@no{165}}%
\qt@newcmd\qurantrans@de@dclix{Aber Allah bezeugt, was Er zu dir (als Offenbarung) herabgesandt hat; Er hat es mit Seinem Wissen herabgesandt. Und (auch) die Engel bezeugen es. Und Allah genügt als Zeuge.\qt@no{166}}%
\qt@newcmd\qurantrans@de@dclx{Diejenigen, die ungläubig sind und von Allahs Weg abhalten, sind fürwahr weit abgeirrt.\qt@no{167}}%
\qt@newcmd\qurantrans@de@dclxi{Für diejenigen, die ungläubig sind und Unrecht tun, ist es nicht Allahs (Wille), ihnen zu vergeben noch sie einen (rechten) Weg zu leiten,\qt@no{168}}%
\qt@newcmd\qurantrans@de@dclxii{außer den Weg zur Hölle, ewig und auf immer darin zu bleiben. Und dies ist Allah ein leichtes.\qt@no{169}}%
\qt@newcmd\qurantrans@de@dclxiii{O ihr Menschen, der Gesandte ist nunmehr mit der Wahrheit von eurem Herrn zu euch gekommen, darum glaubt (an ihn), das ist besser für euch. Wenn ihr aber ungläubig seid, gewiß, so gehört Allah (alles), was in den Himmeln und was auf der Erde ist. Und Allah ist Allwissend und Allweise.\qt@no{170}}%
\qt@newcmd\qurantrans@de@dclxiv{O Leute der Schrift, übertreibt nicht in eurer Religion und sagt gegen Allah nur die Wahrheit aus! al-Masih 'Isa, der Sohn Maryams, ist nur Allahs Gesandter und Sein Wort, das Er Maryam entbot, und Geist von Ihm. Darum glaubt an Allah und Seine Gesandten und sagt nicht ``Drei". Hört auf (damit), das ist besser für euch! Allah ist nur ein Einziger Gott. Preis sei Ihm (, und Erhaben ist Er darüber), daß Er ein Kind haben sollte! Ihm gehört (alles), was in den Himmeln und was auf der Erde ist, und Allah genügt als Sachwalter.\qt@no{171}}%
\qt@newcmd\qurantrans@de@dclxv{Al-Masih wird es nicht verschmähen, ein Diener Allahs zu sein, auch nicht die (Allah) nahegestellten Engel. Wer es aber verschmäht, Ihm zu dienen, und sich hochmütig verhält -, so wird Er sie alle zu Sich versammeln.\qt@no{172}}%
\qt@newcmd\qurantrans@de@dclxvi{Was nun diejenigen angeht, die glauben und rechtschaffene Werke tun, so wird Er ihnen ihren Lohn in vollem Maß zukommen lassen und ihnen von Seiner Huld noch mehr erweisen. Was aber diejenigen angeht, die es verschmähen und sich hochmütig verhalten, so wird Er sie mit schmerzhafter Strafe strafen. Und sie werden für sich außer Allah weder Schutzherrn noch Helfer finden.\qt@no{173}}%
\qt@newcmd\qurantrans@de@dclxvii{O ihr Menschen, zu euch ist nunmehr ein Beweis von eurem Herrn gekommen, und Wir haben zu euch ein deutliches Licht hinabgesandt.\qt@no{174}}%
\qt@newcmd\qurantrans@de@dclxviii{Was nun diejenigen angeht, die an Allah glauben und an Ihm festhalten, so wird Er sie in Barmherzigkeit von Ihm und Huld eingehen lassen und sie zu Sich auf einem geraden Weg leiten.\qt@no{175}}%
\qt@newcmd\qurantrans@de@dclxix{Sie fragen dich um Belehrung. Sag: ``Allah belehrt euch über den Erbanteil seitlicher Verwandtschaft. Wenn ein Mann umkommt, der keine Kinder hat, aber eine Schwester, dann steht ihr die Hälfte dessen zu, was er hinterläßt. Und er beerbt sie, wenn sie keine Kinder hat. Und wenn es zwei (Schwestern) sind, stehen ihnen (beiden) zwei Drittel dessen zu, was er hinterläßt. Und wenn es Geschwister sind, Männer und Frauen, dann kommt einem männlichen Geschlechts ebensoviel zu wie der Anteil von zwei weiblichen Geschlechts. Allah gibt euch Klarheit, damit ihr (nicht) in die Irre geht. Allah weiß über alles Bescheid.\qt@no{176}}%
\qt@newcmd\qurantrans@de@dclxx{\basmalahde O die ihr glaubt, haltet die Abmachungen! Erlaubt (zu essen) sind euch die Vierfüßler unter dem Vieh, außer dem, was euch verlesen wird, ohne daß ihr jedoch das Jagdwild als erlaubt betrachtet, während ihr im Zustand der Pilgerweihe seid. Allah ordnet an, was Er will.\qt@no{1}}%
\qt@newcmd\qurantrans@de@dclxxi{O die ihr glaubt, entweiht nicht die Kultzeichen Allahs, noch den Schutzmonat, noch die Opfertiere, noch die Halsgehänge, noch die, die das geschützte Haus aufsuchen, indem sie nach Huld von ihrem Herrn trachten und nach Wohlgefallen. - Wenn ihr den Weihezustand abgelegt habt, dann dürft ihr jagen. - Und der Haß, den ihr gegen (bestimmte) Leute hegt, weil sie euch von der geschützten Gebetsstätte abgehalten haben, soll euch ja nicht dazu bringen zu übertreten. Helft einander zur Güte und Gottesfurcht, aber helft einander nicht zur Sünde und feindseligem Vorgehen, und fürchtet Allah! Allah ist streng im Bestrafen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@dclxxii{Verboten ist euch (der Genuß von) Verendetem, Blut, Schweinefleisch und dem, worüber ein anderer (Name) als Allah(s) angerufen worden ist, und (der Genuß von) Ersticktem, Erschlagenem, zu Tode Gestürztem oder Gestoßenem, und was von einem wilden Tier gerissen worden ist - außer dem, was ihr schlachtet - und (verboten ist euch,) was auf einem Opferstein geschlachtet worden ist, und mit Pfeilen zu losen. Das ist Frevel. - Heute haben diejenigen, die ungläubig sind, hinsichtlich eurer Religion die Hoffnung aufgegeben. So fürchtet nicht sie, sondern fürchtet Mich! Heute habe Ich euch eure Religion vervollkommnet und Meine Gunst an euch vollendet, und Ich bin mit dem Islam als Religion für euch zufrieden. - Und wer sich aus Hunger in einer Zwangslage befindet, ohne zu einer Sünde hinzuneigen, so ist Allah Allvergebend und Barmherzig.\qt@no{3}}%
\qt@newcmd\qurantrans@de@dclxxiii{Sie fragen dich, was ihnen erlaubt ist. Sag: Erlaubt sind euch die guten Dinge. Und wenn ihr beutegreifende Tiere durch Abrichtung von dem gelehrt habt, was Allah euch gelehrt hat, dann eßt von dem, was sie für euch fassen, und sprecht den Namen Allahs darüber aus. Und fürchtet Allah. Allah ist schnell im Abrechnen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@dclxxiv{Heute sind euch die guten Dinge erlaubt. Und die Speise derjenigen, denen die Schrift gegeben wurde, ist euch erlaubt, und eure Speise ist ihnen erlaubt. Und die Ehrbaren von den gläubigen Frauen und die ehrbaren Frauen von denjenigen, denen vor euch die Schrift gegeben wurde, wenn ihr ihnen ihren Lohn gebt, als ehrbare Ehemänner, nicht als solche, die Hurerei treiben und sich Liebschaften halten. Wer den Glauben verleugnet, dessen Werk wird hinfällig, und im Jenseits gehört er zu den Verlierern.\qt@no{5}}%
\qt@newcmd\qurantrans@de@dclxxv{O die ihr glaubt, wenn ihr euch zum Gebet aufstellt, dann wascht euch das Gesicht und die Hände bis zu den Ellbogen und streicht euch über den Kopf und (wascht euch) die Füße bis zu den Knöcheln. Und wenn ihr im Zustand der Unreinheit seid, dann reinigt euch. Und wenn ihr krank seid oder auf einer Reise oder jemand von euch vom Abort kommt oder ihr Frauen berührt habt und dann kein Wasser findet, so wendet euch dem guten Erdboden zu und streicht euch damit über das Gesicht und die Hände. Allah will euch keine Bedrängnis auferlegen, sondern Er will euch reinigen und Seine Gunst an euch vollenden, auf daß ihr dankbar sein möget.\qt@no{6}}%
\qt@newcmd\qurantrans@de@dclxxvi{Und gedenkt Allahs Gunst an euch und des Abkommens, das Er mit euch getroffen hat, als ihr sagtet: ``Wir hören und gehorchen!" Und fürchtet Allah. Gewiß, Allah weiß über das Innerste der Brüste Bescheid.\qt@no{7}}%
\qt@newcmd\qurantrans@de@dclxxvii{O die ihr glaubt, seid Wahrer (der Sache) Allahs als Zeugen für die Gerechtigkeit. Und der Haß, den ihr gegen (bestimmte) Leute hegt, soll euch ja nicht dazu bringen, daß ihr nicht gerecht handelt. Handelt gerecht. Das kommt der Gottesfurcht näher. Und fürchtet Allah. Gewiß, Allah ist Kundig dessen, was ihr tut.\qt@no{8}}%
\qt@newcmd\qurantrans@de@dclxxviii{Allah hat denen, die glauben und rechtschaffene Werke tun, versprochen, daß es für sie Vergebung und großartigen Lohn geben wird.\qt@no{9}}%
\qt@newcmd\qurantrans@de@dclxxix{Diejenigen aber, die ungläubig sind und Unsere Zeichen für Lüge erklären, das sind Insassen des Höllenbrandes.\qt@no{10}}%
\qt@newcmd\qurantrans@de@dclxxx{O ihr, die ihr glaubt, gedenkt Allahs Gunst an euch, (damals) als (gewisse) Leute vorhatten, ihre Hände nach euch auszustrecken, worauf Er ihre Hände von euch zurückhielt. Und fürchtet Allah. Und auf Allah sollen sich die Gläubigen verlassen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@dclxxxi{Allah hatte ja mit den Kinder Isra'ils ein Abkommen getroffen. Und Wir beriefen von ihnen zwölf Obmänner. Und Allah sagte: ``Ich bin mit euch. Wenn ihr das Gebet verrichtet, die Abgabe entrichtet, an Meine Gesandten glaubt und ihnen beisteht und Allah ein gutes Darlehen gebt, werde Ich euch ganz gewiß eure bösen Taten tilgen und euch ganz gewiß in Gärten eingehen lassen, durcheilt von Bächen. Wer aber von euch danach ungläubig wird, der ist wirklich vom rechten Weg abgeirrt."\qt@no{12}}%
\qt@newcmd\qurantrans@de@dclxxxii{Dafür, daß sie ihr Abkommen brachen, haben Wir sie verflucht und ihre Herzen hart gemacht. Sie verdrehen den Sinn der Worte, und sie haben einen Teil von dem vergessen, womit sie ermahnt worden waren. Und du wirst immer wieder Verrat von ihnen erfahren - bis auf wenige von ihnen. Aber verzeihe ihnen und übe Nachsicht. Gewiß, Allah liebt die Gutes Tuenden.\qt@no{13}}%
\qt@newcmd\qurantrans@de@dclxxxiii{Und (auch) mit denen, die sagen: ``Wir sind Christen'" haben Wir ihr Abkommen getroffen. Aber dann vergaßen sie einen Teil von dem, womit sie ermahnt worden waren. So erregten Wir unter ihnen Feindschaft und Haß bis zum Tag der Auferstehung. Und Allah wird ihnen kundtun, was sie zu machen pflegten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@dclxxxiv{O Leute der Schrift, Unser Gesandter ist nunmehr zu euch gekommen, um euch vieles von dem klarzumachen, was ihr von der Schrift verborgengehalten habt, und er vieles verzeiht. Gekommen ist, nunmehr zu euch von Allah ein Licht und ein deutliches Buch,\qt@no{15}}%
\qt@newcmd\qurantrans@de@dclxxxv{mit dem Allah diejenigen, die Seinem Wohlgefallen folgen, die Wege des Heils leitet und sie aus den Finsternissen hinaus mit Seiner Erlaubnis ins Licht bringt und sie zu einem geraden Weg leitet.\qt@no{16}}%
\qt@newcmd\qurantrans@de@dclxxxvi{Ungläubig sind ja diejenigen, die sagen: ``Allah ist ja al-Masih, der Sohn Maryams". Sag: Wer vermag denn gegen Allah etwas (auszurichten), wenn Er alMasih, den Sohn Maryams, seine Mutter und all diejenigen, die auf der Erde sind, vernichten will? Allah gehört die Herrschaft der Himmel und der Erde und dessen, was dazwischen ist. Er erschafft, was Er will. Und Allah hat zu allem die Macht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@dclxxxvii{Die Juden und Christen sagen: ``Wir sind Allahs Söhne und Seine Lieblinge." Sag: Warum bestraft Er euch dann für eure Sünden? Nein! Vielmehr seid ihr menschliche Wesen von denen, die Er erschaffen hat. Er vergibt, wem Er will, und Er straft, wen Er will. Und Allah gehört die Herrschaft der Himmel und der Erde und dessen, was dazwischen ist, und zu Ihm ist der Ausgang.\qt@no{18}}%
\qt@newcmd\qurantrans@de@dclxxxviii{O Leute der Schrift, Unser Gesandter ist nunmehr zu euch gekommen, um euch nach einer Unterbrechungszeit (in der Reihe) der Gesandten Klarheit zu geben, damit ihr (nicht etwa) sagt: ``Zu uns ist kein Frohbote und kein Warner gekommen." Nun ist ja ein Frohbote und ein Warner zu euch gekommen. Und Allah hat zu allem die Macht.\qt@no{19}}%
\qt@newcmd\qurantrans@de@dclxxxix{Und als Musa zu seinem Volk sagte: ``O mein Volk, gedenkt der Gunst Allahs an euch, als Er unter euch Propheten einsetzte und euch zu Königen machte und euch gab, was Er niemandem (anderen) der Weltenbewohner gegeben hat.\qt@no{20}}%
\qt@newcmd\qurantrans@de@dcxc{O mein Volk, tretet in das geheiligte Land ein, das Allah für euch bestimmt hat, und kehrt nicht den Rücken, denn dann werdet ihr als Verlierer zurückkehren."\qt@no{21}}%
\qt@newcmd\qurantrans@de@dcxci{Sie sagten: ``O Musa, in ihm ist ein übermächtiges Volk. Wir werden es gewiß nicht betreten, bis sie aus ihm herausgehen. Wenn sie aus ihm herausgehen, dann wollen wir (es) gewiß betreten."\qt@no{22}}%
\qt@newcmd\qurantrans@de@dcxcii{Zwei Männer von denen, die (Allah) fürchteten und denen Allah Gunst erwiesen hatte, sagten: ``Tretet gegen sie durch das Tor ein; wenn ihr dadurch eintretet, dann werdet ihr Sieger sein. Und verlaßt euch auf Allah, wenn ihr gläubig seid."\qt@no{23}}%
\qt@newcmd\qurantrans@de@dcxciii{Sie (, die anderen,) sagten: ``O Musa, gewiß werden wir es niemals betreten, solange sie darin sind. Geh doch du und dein Herr hin und kämpft! Wir werden hier sitzen bleiben."\qt@no{24}}%
\qt@newcmd\qurantrans@de@dcxciv{Er sagte: ``Mein Herr, ich habe ja nur Macht über mich selbst und meinen Bruder. Darum (unter)scheide uns von dem Volk der Frevler."\qt@no{25}}%
\qt@newcmd\qurantrans@de@dcxcv{Er (Allah) sagte: ``Gewiß, so soll es ihnen denn für vierzig Jahre verwehrt sein, (während derer) sie auf der Erde umherirren. So sei nicht betrübt über das Volk der Frevler!"\qt@no{26}}%
\qt@newcmd\qurantrans@de@dcxcvi{Und verlies ihnen die Kunde von den beiden Söhnen Adams, der Wahrheit entsprechend, als sie ein Opfer darbrachten. Da wurde es von dem einen von ihnen angenommen, während es vom anderen nicht angenommen wurde. Der sagte: ``Ich werde dich ganz gewiß töten." Der andere sagte: ``Allah nimmt nur von den Gottesfürchtigen an.\qt@no{27}}%
\qt@newcmd\qurantrans@de@dcxcvii{Wenn du deine Hand nach mir ausstreckst, um mich zu töten, so werde ich meine Hand nicht nach dir ausstrecken, um dich zu töten. Ich fürchte Allah, den Herrn der Weltenbwohner.\qt@no{28}}%
\qt@newcmd\qurantrans@de@dcxcviii{Ich will, daß du meine und deine Sünde auf dich lädst und so einer von den Insassen des (Höllen)feuers sein wirst. Das ist der Lohn der Ungerechten."\qt@no{29}}%
\qt@newcmd\qurantrans@de@dcxcix{Doch machte ihn seine Seele willig, seinen Bruder zu töten. Und so tötete er ihn. Und er wurde einer von den Verlierern.\qt@no{30}}%
\qt@newcmd\qurantrans@de@dcc{Da schickte Allah einen Raben, der in der Erde scharrte, um ihm zu zeigen, wie er die böse Tat an seinem Bruder verbergen könne. Er sagte: ``O wehe mir! War ich unfähig, zu sein wie dieser Rabe und die böse Tat an meinem Bruder zu verbergen?" So wurde er von denjenigen, die bereuen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@dcci{Aus diesem Grunde haben Wir den Kindern Isra'ils vorgeschrieben: Wer ein menschliches Wesen tötet, ohne (daß es) einen Mord (begangen) oder auf der Erde Unheil gestiftet (hat), so ist es, als ob er alle Menschen getötet hätte. Und wer es am Leben erhält, so ist es', als ob er alle Menschen am Leben erhält. Unsere Gesandten sind bereits mit klaren Beweisen zu ihnen gekommen. Danach aber sind viele von ihnen wahrlich maßlos auf der Erde geblieben.\qt@no{32}}%
\qt@newcmd\qurantrans@de@dccii{Der Lohn derjenigen, die Krieg führen gegen Allah und Seinen Gesandten und sich bemühen, auf der Erde Unheil zu stiften, ist indessen (der), daß sie allesamt getötet oder gekreuzigt werden, oder daß ihnen Hände und Füße wechselseitig abgehackt werden, oder daß sie aus dem Land verbannt werden. Das ist für sie eine Schande im Diesseits, und im Jenseits gibt es für sie gewaltige Strafe,\qt@no{33}}%
\qt@newcmd\qurantrans@de@dcciii{- außer denjenigen, die bereuen, bevor ihr Macht über sie habt. So wisset, daß Allah Allvergebend und Barmherzig ist.\qt@no{34}}%
\qt@newcmd\qurantrans@de@dcciv{O die ihr glaubt, fürchtet Allah und trachtet nach einem Mittel zu Ihm und müht euch auf Seinem Weg ab, auf daß es euch wohl ergehen möge!\qt@no{35}}%
\qt@newcmd\qurantrans@de@dccv{Gewiß, wenn diejenigen, die ungläubig sind, alles hätten, was auf der Erde ist, und noch einmal das Gleiche dazu, um sich damit von der Strafe des Tages der Auferstehung loszukaufen, würde es nicht von ihnen angenommen werden, und für sie gibt es schmerzhafte Strafe.\qt@no{36}}%
\qt@newcmd\qurantrans@de@dccvi{Sie wollen aus dem (Höllen)feuer herauskommen, aber sie werden nicht aus ihm herauskommen, und für sie gibt es beständige Strafe.\qt@no{37}}%
\qt@newcmd\qurantrans@de@dccvii{Der Dieb und die Diebin: trennt ihnen ihre Hände ab als Lohn für das, was sie begangen haben, und als ein warnendes Beispiel von Allah. Allah ist Allmächtig und Allweise.\qt@no{38}}%
\qt@newcmd\qurantrans@de@dccviii{Wer aber bereut, nachdem er Unrecht getan hat, und es wieder gutmacht, so nimmt Allah seine Reue gewiß an. Allah ist Allvergebend und Barmherzig.\qt@no{39}}%
\qt@newcmd\qurantrans@de@dccix{Weißt du nicht, daß Allah die Herrschaft über die Himmel und die Erde gehört? Er straft, wen Er will, und vergibt, wem Er will. Und Allah hat zu allem die Macht.\qt@no{40}}%
\qt@newcmd\qurantrans@de@dccx{O du Gesandter, lasse dich nicht durch jene traurig machen, die im Unglauben dahineilen, unter denen, die mit ihren Mündern sagen: ``Wir glauben", während ihre Herzen nicht glauben, Und unter denjenigen, die dem Judentum angehören, unter ihnen gibt es manche, die auf Lügen horchen, die auf andere Leute horchen, die nicht zu dir gekommen sind. Sie verdrehen den Sinn der Worte, nach(dem sie an) ihrer (richtigen) Stelle (waren), und sagen: ``Wenn euch dies gegeben wird, dann nehmt es an. Wenn euch dies aber nicht gegeben wird, dann seht euch vor." Wen Allah der Versuchung aussetzen will, für den wirst du gegen Allah nichts (auszurichten) vermögen. Das sind diejenigen, deren Herzen Allah nicht rein machen wollte. Schande gibt es für sie im Diesseits, und im Jenseits gibt es für sie gewaltige Strafe;\qt@no{41}}%
\qt@newcmd\qurantrans@de@dccxi{sie), die auf Lügen horchen, und die darauf aus sind, unrechtmäßig Erworbenes zu verschlingen. Wenn sie zu dir kommen, so richte zwischen ihnen oder wende dich von ihnen ab! Wenn du dich von ihnen abwendest, werden sie dir keinen Schaden zufügen, wenn du aber richtest, dann richte zwischen ihnen in Gerechtigkeit. Gewiß, Allah liebt die Gerechten.\qt@no{42}}%
\qt@newcmd\qurantrans@de@dccxii{Wie aber können sie dich richten lassen, während sie doch die Tora haben, in der das Urteil Allahs (enthalten) ist, und sich hierauf, nach alledem, abkehren? Diese sind doch keine Gläubigen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@dccxiii{Gewiß, Wir haben die Tora hinabgesandt, in der Rechtleitung und Licht sind, womit die Propheten, die sich (Allah) ergeben hatten, für diejenigen, die dem Judentum angehören, walten, und so auch die Leute des Herrn und die Gelehrten, nach dem, was ihnen von der Schrift Allahs anvertraut worden war und worüber sie Zeugen waren. So fürchtet nicht die Menschen, sondern fürchtet Mich. Und verkauft Meine Zeichen nicht für einen geringen Preis! Wer nicht nach dem waltet, was Allah (als Offenbarung) herabgesandt hat, das sind die Ungläubigen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@dccxiv{Und Wir haben ihnen darin vorgeschrieben: Leben um Leben, Auge um Auge, Nase um Nase, Ohr um Ohr, Zahn um Zahn; und (auch) für Verwundungen Wiedervergeltung. Wer es aber als Almosen erläßt, für den ist es eine Sühne. Wer nicht nach dem waltet, was Allah (als Offenbarung) herabgesandt hat, das sind die Ungerechten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@dccxv{Und Wir ließen auf ihren Spuren 'Isa, den Sohn Maryams, folgen, das zu bestätigen, was von der Tora vor ihm (offenbart) war; und Wir gaben ihm das Evangelium, in dem Rechtleitung und Licht sind, und das zu bestätigen, was von der Tora vor ihm (offenbart) war, und als Rechtleitung und Ermahnung für die Gottesfürchtigen.\qt@no{46}}%
\qt@newcmd\qurantrans@de@dccxvi{Und so sollen die Leute des Evangeliums nach dem walten, was Allah darin herabgesandt hat. Wer nicht nach dem waltet, was Allah (als Offenbarung) herabgesandt hat, das sind die Frevler.\qt@no{47}}%
\qt@newcmd\qurantrans@de@dccxvii{Und Wir haben zu dir das Buch mit der Wahrheit hinabgesandt, das zu bestätigen, was von dem Buch vor ihm (offenbart) war, und als Wächter darüber. So richte zwischen ihnen nach dem, was Allah (als Offenbarung) herabgesandt hat, und folge nicht ihren Neigungen entgegen dem, was dir von der Wahrheit zugekommen ist. Für jeden von euch haben Wir ein Gesetz und einen deutlichen Weg festgelegt. Und wenn Allah wollte, hätte Er euch wahrlich zu einer einzigen Gemeinschaft gemacht. Aber (es ist so,) damit Er euch in dem, was Er euch gegeben hat? prüfe. So wetteifert nach den guten Dingen! Zu Allah wird euer aller Rückkehr sein, und dann wird Er euch kundtun, worüber ihr uneinig zu sein pflegtet.\qt@no{48}}%
\qt@newcmd\qurantrans@de@dccxviii{Und so richte zwischen ihnen nach dem, was Allah (als Offenbarung) herabgesandt hat, und folge nicht ihren Neigungen, sondern sieh dich vor ihnen vor, daß sie dich nicht der Versuchung aussetzen (abzuweichen) von einem Teil dessen, was Allah zu dir (als Offenbarung) herabgesandt hat! Doch wenn sie sich abkehren, so wisse, daß Allah sie für einen Teil ihrer Sünden treffen will. Viele von den Menschen sind fürwahr Frevler.\qt@no{49}}%
\qt@newcmd\qurantrans@de@dccxix{Begehren sie etwa das Urteil der Unwissenheit? Wer kann denn besser walten als Allah für Leute, die (in ihrem Glauben) überzeugt sind?\qt@no{50}}%
\qt@newcmd\qurantrans@de@dccxx{O die ihr glaubt, nehmt nicht die Juden und die Christen zu Schutzherren! Sie sind einer des anderen Schutzherren. Und wer von euch sie zu Schutzherren nimmt, der gehört zu ihnen. Gewiß, Allah leitet das ungerechte Volk nicht recht.\qt@no{51}}%
\qt@newcmd\qurantrans@de@dccxxi{Und so siehst du diejenigen, in deren Herzen Krankheit ist, sich ihretwegen beeilen; sie sagen: ``Wir fürchten, daß uns eine Schicksalswendung treffen wird." Aber vielleicht wird Allah den entscheidenden Sieg oder eine Anordnung von Ihm herbeibringen. Dann werden sie über das, was sie in ihrem Innersten geheimgehalten haben, Reue empfinden.\qt@no{52}}%
\qt@newcmd\qurantrans@de@dccxxii{Und diejenigen, die glauben, werden sagen: ``Sind dies diejenigen, die bei Allah ihren kräftigsten Eid schworen, daß sie wahrlich mit euch seien?" Ihre Werke sind hinfallig, so daß sie (nun) Verlierer geworden sind.\qt@no{53}}%
\qt@newcmd\qurantrans@de@dccxxiii{O die ihr glaubt, wer von euch sich von seiner Religion abkehrt -, so wird Allah Leute bringen, die Er liebt und die Ihn lieben, bescheiden gegenüber den Gläubigen, mächtig (auftretend) gegenüber den Ungläubigen, und die sich auf Allahs Weg abmühen und nicht den Tadel des Tadlers fürchten. Das ist Allahs Huld, die Er gewährt, wem Er will. Allah ist Allumfassend und Allwissend.\qt@no{54}}%
\qt@newcmd\qurantrans@de@dccxxiv{Euer Schutzherr ist (allein) Allah und (auch) Sein Gesandter und diejenigen, die glauben, die das Gebet verrichten, die Abgabe entrichten und zu den sich Verbeugenden gehören.\qt@no{55}}%
\qt@newcmd\qurantrans@de@dccxxv{Wer sich Allah zum Schutzherrn nimmt und (auch) Seinen Gesandten und diejenigen, die glauben, - gewiß, die Anhängerschaft Allahs wird der Sieger sein.\qt@no{56}}%
\qt@newcmd\qurantrans@de@dccxxvi{O die ihr glaubt, nehmt nicht diejenigen, die sich über eure Religion lustig machen und sie zum Gegenstand des Spiels nehmen, von denjenigen, denen vor euch die Schrift gegeben wurde, und die Ungläubigen zu Schutzherren. Und fürchtet Allah, wenn ihr gläubig seid!\qt@no{57}}%
\qt@newcmd\qurantrans@de@dccxxvii{Wenn ihr zum Gebet ruft, machen sie sich darüber lustig und nehmen es zum Gegenstand des Spiels. Dies, weil sie Leute sind, die nicht begreifen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@dccxxviii{Sag: O Leute der Schrift, grollt ihr uns nur (darum), daß wir an Allah glauben und an das, was zu uns (als Offenbarung) herabgesandt worden ist und was zuvor herabgesandt wurde, und daß die meisten von euch Frevler sind?\qt@no{59}}%
\qt@newcmd\qurantrans@de@dccxxix{Sag: Soll ich euch kundtun, was als Belohnung bei Allah (noch) schlechter ist? Diejenigen, die Allah verflucht hat und denen Er zürnt und aus denen Er Affen und Schweine gemacht hat und die falschen Göttern dienen. Diese befinden sich in einer (noch) schlechteren Lage und sind (noch) weiter vom rechten Weg abgeirrt.\qt@no{60}}%
\qt@newcmd\qurantrans@de@dccxxx{Wenn sie zu euch kommen, sagen sie: ``Wir glauben", wo sie schon mit Unglauben (behaftet) eintreten und auch noch mit ihm (behaftet wieder) herausgehen. Doch Allah weiß sehr wohl, was sie verbergen.\qt@no{61}}%
\qt@newcmd\qurantrans@de@dccxxxi{Und du siehst viele von ihnen in Sünde, feindseligem Vorgehen und dem Verschlingen von unrechtmäßig Erworbenem dahineilen. Fürwahr, wie schlimm ist, was sie tun!\qt@no{62}}%
\qt@newcmd\qurantrans@de@dccxxxii{Warum verbieten ihnen nicht die Leute des Herrn und die Gelehrten die sündhaften Worte und das Verschlingen von unrechtmäßig Erworbenem? Fürwahr, wie schlimm ist, was sie machen!\qt@no{63}}%
\qt@newcmd\qurantrans@de@dccxxxiii{Und die Juden sagen: ``Allahs Hand ist gefesselt." Ihre (eigenen) Hände seien gefesselt und sie seien verflucht für das, was sie sagen. Nein! Vielmehr sind Seine Hände (weit) ausgestreckt; Er gibt aus, wie Er will. Was zu dir (als Offenbarung) von deinem Herrn herabgesandt worden ist, wird sicherlich bei vielen von ihnen die Auflehnung und den Unglauben noch mehren. Und Wir haben unter ihnen Feindschaft und Haß erregt bis zum Tag der Auferstehung. Jedesmal, wenn sie ein Feuer zum Krieg anzünden, löscht Allah es aus. Und sie bemühen sich, auf der Erde Unheil zu stiften. Aber Allah liebt nicht die Unheilstifter.\qt@no{64}}%
\qt@newcmd\qurantrans@de@dccxxxiv{Wenn die Leute der Schrift nur glaubten und gottesfürchtig wären, würden Wir ihnen wahrlich ihre bösen Taten tilgen und sie wahrlich in die Gärten der Wonne eingehen lassen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@dccxxxv{Wenn sie nur die Tora und das Evangelium und das befolgten, was zu ihnen (als Offenbarung) von ihrem Herrn herabgesandt wurde, würden sie fürwahr von (den guten Dingen) über ihnen und unter ihren Füßen essen. Unter ihnen ist eine gemäßigte Gemeinschaft; aber wie böse ist bei vielen von ihnen, was sie tun.\qt@no{66}}%
\qt@newcmd\qurantrans@de@dccxxxvi{O du Gesandter, übermittele, was zu dir (als Offenbarung) von deinem Herrn herabgesandt worden ist! Wenn du es nicht tust, so hast du Seine Botschaft nicht übermittelt. Allah wird dich vor den Menschen schützen. Gewiß, Allah leitet das ungläubige Volk nicht recht.\qt@no{67}}%
\qt@newcmd\qurantrans@de@dccxxxvii{Sag: O Leute der Schrift, ihr fußt auf nichts, bis ihr die Tora und das Evangelium und das befolgt, was zu euch (als Offenbarung) von eurem Herrn herabgesandt worden ist. Was zu dir (als Offenbarung) von deinem Herrn herabgesandt worden ist, wird ganz gewiß bei vielen von ihnen die Auflehnung und den Unglauben noch mehren. So sei nicht betrübt über das ungläubige Volk!\qt@no{68}}%
\qt@newcmd\qurantrans@de@dccxxxviii{Gewiß, diejenigen, die glauben', und diejenigen, die dem Judentum angehören, und die Säbier und die Christen, - wer (immer) an Allah und den Jüngsten Tag glaubt und rechtschaffen handelt, - über die soll keine Furcht kommen, noch sollen sie traurig sein.\qt@no{69}}%
\qt@newcmd\qurantrans@de@dccxxxix{Und Wir trafen doch schon mit den Kindern Isra'ils ein Abkommen und sandten zu ihnen Gesandte. Jedesmal, wenn ein Gesandter zu ihnen kam mit etwas, wozu sie selbst nicht neigten, bezichtigten sie einen Teil (der Gesandten) der Lüge und töteten einen (anderen) Teil.\qt@no{70}}%
\qt@newcmd\qurantrans@de@dccxl{Und sie meinten, daß es keine Versuchung gäbe. Sie waren blind und taub. Hierauf nahm Allah ihre Reue an. Dann aber wurden (wieder) viele von ihnen blind und taub. Und Allah sieht wohl, was sie tun.\qt@no{71}}%
\qt@newcmd\qurantrans@de@dccxli{Fürwahr, ungläubig sind diejenigen, die sagen: ``Gewiß, Allah ist al-Masih, der Sohn Maryams", wo doch al-Masih (selbst) gesagt hat: ``O Kinder Isra'ils, dient Allah, meinem Herrn und eurem Herrn!" Wer Allah (etwas) beigesellt, dem verbietet fürwahr Allah das Paradies, und dessen Zufluchtsort wird das (Höllen)feuer sein. Die Ungerechten werden keine Helfer haben.\qt@no{72}}%
\qt@newcmd\qurantrans@de@dccxlii{Fürwahr, ungläubig sind diejenigen, die sagen: ``Gewiß, Allah ist einer von dreien." Es gibt aber keinen Gott außer dem Einen Einzigen. Wenn sie mit dem, was sie sagen, nicht aufhören, so wird denjenigen von ihnen, die ungläubig sind, ganz gewiß schmerzhafte Strafe widerfahren.\qt@no{73}}%
\qt@newcmd\qurantrans@de@dccxliii{Wenden sie sich denn nicht in Reue zu Allah und bitten Ihn um Vergebung? Allah ist All vergebend und Barmherzig.\qt@no{74}}%
\qt@newcmd\qurantrans@de@dccxliv{Al-Masih, der Sohn Maryams, war doch nur ein Gesandter, vor dem bereits Gesandte vorübergegangen waren. Und seine Mutter war sehr wahrheitsliebend; sie (beide) pflegten Speise zu essen. Schau, wie Wir ihnen die Zeichen klar machen, und schau, wie sie sich abwendig machen lassen!\qt@no{75}}%
\qt@newcmd\qurantrans@de@dccxlv{Sag: Wollt ihr außer Allah dem dienen, was euch weder Schaden noch Nutzen zu bringen vermag? Und Allah ist es (doch), der Allhörend und Allwissend ist.\qt@no{76}}%
\qt@newcmd\qurantrans@de@dccxlvi{Sag: O Leute der Schrift, übertreibt nicht in eurer Religion außer in (dem Rahmen) der Wahrheit und folgt nicht den Neigungen von Leuten, die schon zuvor irregegangen sind und viele (andere mit ihnen) in die Irre geführt haben und vom rechten Weg abgeirrt sind.\qt@no{77}}%
\qt@newcmd\qurantrans@de@dccxlvii{Verflucht wurden diejenigen von den Kindern Isra'ils, die ungläubig waren, durch den Mund Dawuds und 'Isas, des Sohnes Maryams. Dies dafür, daß sie sich widersetzten und stets übertraten.\qt@no{78}}%
\qt@newcmd\qurantrans@de@dccxlviii{Sie pflegten einander nichts Verwerfliches, das sie taten, zu verbieten. Fürwahr, wie schlimm ist, was sie zu tun pflegten!\qt@no{79}}%
\qt@newcmd\qurantrans@de@dccxlix{Du siehst viele von ihnen diejenigen, die ungläubig sind, zu Vertrauten nehmen. Fürwahr, wie schlimm ist, was sie sich selbst vorausgeschickt haben; (es ist,) daß Allah ihnen gegenüber Sein Mißfallen zeigt; und ewig werden sie in der Strafe bleiben.\qt@no{80}}%
\qt@newcmd\qurantrans@de@dccl{Wenn sie an Allah und den Propheten und das, was zu ihm (als Offenbarung) herabgesandt worden ist, glauben würden, hätten sie sie nicht zu Schutzherren genommen. Aber viele von ihnen sind Frevler.\qt@no{81}}%
\qt@newcmd\qurantrans@de@dccli{Du wirst ganz gewiß finden, daß diejenigen Menschen, die den Gläubigen am heftigsten Feindschaft zeigen, die Juden und diejenigen sind, die (Allah etwas) beigesellen. Und du wirst ganz gewiß finden, daß diejenigen, die den Gläubigen in Freundschaft am nächsten stehen, die sind, die sagen: ``Wir sind Christen." Dies, weil es unter ihnen Priester und Mönche gibt und weil sie sich nicht hochmütig verhalten.\qt@no{82}}%
\qt@newcmd\qurantrans@de@dcclii{Wenn sie hören, was zum Gesandten (als Offenbarung) herabgesandt worden ist, siehst du ihre Augen von Tränen überfließen wegen dessen, was sie (darin) als Wahrheit erkannt haben. Sie sagen: ``Unser Herr, wir glauben. Schreibe uns unter den Zeugnis Ablegenden auf.\qt@no{83}}%
\qt@newcmd\qurantrans@de@dccliii{Warum sollten wir nicht an Allah glauben und an das, was von der Wahrheit zu uns gekommen ist, und begehren, daß uns unser Herr zusammen mit dem rechtschaffenen Volk (in den Paradiesgarten) eingehen lasse?"\qt@no{84}}%
\qt@newcmd\qurantrans@de@dccliv{So wird Allah sie für das, was sie gesagt haben, mit Gärten belohnen, durcheilt von Bächen, ewig darin zu bleiben. Das ist der Lohn der Gutes Tuenden.\qt@no{85}}%
\qt@newcmd\qurantrans@de@dcclv{Diejenigen aber, die ungläubig sind und Unsere Zeichen für Lüge erklären, das sind die Insassen des Höllenbrandes.\qt@no{86}}%
\qt@newcmd\qurantrans@de@dcclvi{O die ihr glaubt, verbietet nicht die guten Dinge, die Allah euch erlaubt hat. und übertretet nicht! Allah liebt nicht die Übertreter.\qt@no{87}}%
\qt@newcmd\qurantrans@de@dcclvii{Und eßt von dem, womit Allah euch versorgt hat, als etwas Erlaubtem und Gutem, und fürchtet Allah, an Den ihr glaubt!\qt@no{88}}%
\qt@newcmd\qurantrans@de@dcclviii{Allah wird euch nicht für etwas Unbedachtes in euren Eiden belangen. Jedoch wird Er euch für das belangen, was ihr mit euren Eiden fest abmacht (und dieses dann nicht einhaltet). Die Sühne dafür besteht in der Speisung von zehn Armen in dem Maß, wie ihr eure Angehörigen im Durchschnitt speist, oder ihrer Bekleidung oder der Befreiung eines Sklaven. Wer aber keine (Möglichkeit) findet, (der hat) drei Tage (zu) fasten. Das ist die Sühne für eure Eide, wenn ihr schwört. Und erfüllt eure Eide. So macht Allah euch Seine Zeichen klar, auf daß ihr dankbar sein möget!\qt@no{89}}%
\qt@newcmd\qurantrans@de@dcclix{O die ihr glaubt, berauschender Trank, Glücksspiel, Opfersteine und Lospfeile sind nur ein Greuel vom Werk des Satans. So meidet ihn, auf daß es euch wohl ergehen möge!\qt@no{90}}%
\qt@newcmd\qurantrans@de@dcclx{Der Satan will (ja) zwischen euch nur Feindschaft und Haß säen durch berauschenden Trank und Glücksspiel und euch vom Gedenken Allahs und vom Gebet abhalten. Werdet ihr (damit) nun wohl aufhören?\qt@no{91}}%
\qt@newcmd\qurantrans@de@dcclxi{Und gehorcht Allah und gehorcht dem Gesandten und seht euch vor! Doch wenn ihr euch abkehrt, so wißt, daß Unserem Gesandten nur die deutliche Übermittelung (der Botschaft) obliegt.\qt@no{92}}%
\qt@newcmd\qurantrans@de@dcclxii{Es lastet keine Sünde auf denjenigen, die glauben und rechtschaffene Werke tun, hinsichtlich dessen, was sie (bisher) verzehrt haben, wenn sie (fortan) gottesfürchtig sind und glauben und rechtschaffene Werke tun und wiederum gottesfürchtig sind und glauben und wiederum gottesfürchtig sind und Gutes tun. Allah liebt die Gutes Tuenden.\qt@no{93}}%
\qt@newcmd\qurantrans@de@dcclxiii{O die ihr glaubt, Allah wird euch ganz gewiß mit etwas von dem Jagdwild prüfen, das eure Hände und Lanzen erreichen (können), damit Allah denjenigen kenne, der Ihn im Verborgenen fürchtet. Wer nun danach eine Übertretung begeht, für den gibt es schmerzhafte Strafe.\qt@no{94}}%
\qt@newcmd\qurantrans@de@dcclxiv{O die ihr glaubt, tötet nicht das Jagdwild, während ihr im Zustand der Pilgerweihe seid! Wer von euch es vorsätzlich tötet, (für den gilt es,) eine Ersatzleistung (zu zahlen), ein Gleiches, wie das, was er getötet hat, an Stück Vieh - darüber sollen zwei gerechte Personen von euch richten - (eine Ersatzleistung, die) als Opfertier die Ka'ba erreichen soll. Oder (er leistet als) eine Sühne die Speisung von Armen oder die (entsprechende) Ersatzleistung an Fasten, damit er die schlimmen Folgen seines Verhaltens koste. Allah verzeiht, was (vorher) geschehen ist. Wer aber rückfällig wird, den wird Allah der Vergeltung aussetzen. Allah ist Allmächtig und Besitzer von Vergeltungsgewalt.\qt@no{95}}%
\qt@newcmd\qurantrans@de@dcclxv{Erlaubt sind euch die Jagdtiere des Meeres und (all) das Eßbare aus ihm als Nießbrauch für euch und für die Reisenden; doch verboten ist euch die Jagd auf die Landtiere, solange ihr im Zustand der Pilgerweihe seid. Und fürchtet Allah, zu Dem ihr versammelt werdet.\qt@no{96}}%
\qt@newcmd\qurantrans@de@dcclxvi{Allah hat die Ka'ba, das geschützte Haus, zu einer Stätte des Gottesdienstes für die Menschen gemacht, und (ebenso) den Schutzmonat, die Opfertiere und die (Opfertiere mit den) Halsgehänge(n). Dies, damit ihr wisset, daß Allah weiß, was in den Himmeln und was auf der Erde ist, und daß Allah über alles Bescheid weiß.\qt@no{97}}%
\qt@newcmd\qurantrans@de@dcclxvii{Wisset, daß Allah streng im Bestrafen, daß Allah (aber auch) Allvergebend und Barmherzig ist.\qt@no{98}}%
\qt@newcmd\qurantrans@de@dcclxviii{Dem Gesandten obliegt nur die Übermittelung (der Botschaft). Und Allah weiß, was ihr offenlegt und was ihr verbergt.\qt@no{99}}%
\qt@newcmd\qurantrans@de@dcclxix{Sag: Nicht gleich sind das Schlechte und das Gute, auch wenn die Menge des Schlechten dir gefallen sollte. Darum fürchtet Allah, o die ihr Verstand besitzt, auf daß es euch wohl ergehen möge!\qt@no{100}}%
\qt@newcmd\qurantrans@de@dcclxx{O die ihr glaubt, fragt nicht nach Dingen, die, wenn sie euch offengelegt werden, euch leid tun, wenn ihr nach ihnen fragt zu der Zeit, da der Qur'an offenbart wird, sie euch (gewiß) offengelegt werden, wo Allah sie übergangen hat. Und Allah ist Allvergebend und Nachsichtig.\qt@no{101}}%
\qt@newcmd\qurantrans@de@dcclxxi{Leute vor euch haben schon danach gefragt, doch dann wurden sie ihretwegen ungläubig.\qt@no{102}}%
\qt@newcmd\qurantrans@de@dcclxxii{Allah hat keine Bahira, keine Sa'iba, keine Wasila und keine Ham bestimmt, sondern diejenigen, die ungläubig sind, ersinnen gegen Allah Lügen, und die meisten von ihnen begreifen nicht.\qt@no{103}}%
\qt@newcmd\qurantrans@de@dcclxxiii{Und wenn man zu ihnen sagt: ``Kommt her zu dem, was Allah (als Offenbarung) herabgesandt hat, und zum Gesandten", sagen sie: ``Unsere Genüge ist das, worin wir unsere Väter (vor)gefunden haben." Was denn, auch wenn ihre Väter nichts begriffen und nicht rechtgeleitet waren?\qt@no{104}}%
\qt@newcmd\qurantrans@de@dcclxxiv{O die ihr glaubt, wacht über euch selbst! Wer abirrt, kann euch keinen Schaden zufügen, wenn ihr rechtgeleitet seid. Zu Allah wird eure aller Rückkehr sein, und dann wird Er kundtun, was ihr zu tun pflegtet.\qt@no{105}}%
\qt@newcmd\qurantrans@de@dcclxxv{O die ihr glaubt, wenn einem von euch der Tod naht zu der Zeit, da (er sein) Vermächtnis (macht), (soll) das Zeugnis unter euch (erfolgen) durch zwei gerechte Personen von euch, oder durch zwei andere, (die) nicht von euch (sind), wenn ihr im Land umherreist und euch dann das Unglück des Todes trifft. Ihr sollt sie nach dem Gebet festhalten, und sie sollen dann, wenn ihr zweifelt, bei Allah schwören: ``Wir verkaufen es für keinen Preis, auch wenn es sich um einen Verwandten handelt, und verheimlichen das Zeugnis Allahs nicht; wir gehörten sonst wahrlich zu den Sündhaften."\qt@no{106}}%
\qt@newcmd\qurantrans@de@dcclxxvi{Wenn man danach aber entdeckt, daß sie (beide) sich einer Sünde schuldig gemacht haben, dann (sollen) zwei andere an ihre Stelle treten von denen, zu deren Nachteil die beiden sich (einer Sünde) schuldig gemacht haben, und (sie) (beide sollen) bei Allah schwören: ``Unser Zeugnis ist wahrlich berechtigter als deren Zeugnis. Und wir haben nicht übertreten; wir gehörten sonst wahrlich zu den Ungerechten."\qt@no{107}}%
\qt@newcmd\qurantrans@de@dcclxxvii{Das ist eher geeignet, daß sie das Zeugnis, wie es sein soll, ablegen oder sie befürchten (müssen), daß nach ihren Eiden wieder (andere) Eide (geschworen) werden. Und fürchtet Allah und hört! Allah leitet das Volk der Frevler nicht recht.\qt@no{108}}%
\qt@newcmd\qurantrans@de@dcclxxviii{An dem Tag, da Allah die Gesandten versammelt und dann sagt: ``Was wurde euch geantwortet?'", werden sie sagen: ``Wir haben kein Wissen (darüber). Du bist ja der Allwisser der verborgenen Dinge."\qt@no{109}}%
\qt@newcmd\qurantrans@de@dcclxxix{Wenn Allah sagt: ``O 'Isa, Sohn Maryams, gedenke Meiner Gunst an dir und an deiner Mutter, als Ich dich mit dem Heiligen Geist stärkte, so daß du in der Wiege zu den Menschen sprachst und im Mannesalter; und als Ich dich die Schrift, die Weisheit, die Tora und das Evangelium lehrte; und als du aus Lehm mit Meiner Erlaubnis (etwas) schufst, (was so aussah) wie die Gestalt eines Vogels, und ihr dann einhauchtest, und sie da ein (wirklicher) Vogel wurde mit Meiner Erlaubnis; und (als) du den Blindgeborenen und den Weißgefleckten mit Meiner Erlaubnis heiltest und Tote mit Meiner Erlaubnis (aus den Gräbern) herauskommen ließest; und als Ich die Kinder Isra'ils von dir zurückhielt, als du mit den klaren Beweisen zu ihnen kamst, worauf diejenigen von ihnen, die ungläubig waren, sagten: ,Das ist nichts als deutliche Zauberei'."\qt@no{110}}%
\qt@newcmd\qurantrans@de@dcclxxx{Und als Ich den Jüngern eingab: ``Glaubt an Mich und an Meinen Gesandten!" Sie sagten: ``Wir glauben. Bezeuge, daß wir Dir ergeben sind!"\qt@no{111}}%
\qt@newcmd\qurantrans@de@dcclxxxi{Als die Jünger sagten: ``O 'Isa, Sohn Maryams, kann dein Herr zu uns einen Tisch (mit Speisen) vom Himmel herabsenden?" Er sagte: ``Fürchtet Allah, wenn ihr gläubig seid!"\qt@no{112}}%
\qt@newcmd\qurantrans@de@dcclxxxii{Sie sagten: ``Wir wollen von ihm essen und, daß unsere Herzen Ruhe finden und, daß wir wissen, daß du zu uns tatsächlich die Wahrheit gesprochen hast und, daß wir zu denen gehören, die darüber Zeugnis ablegen (können)."\qt@no{113}}%
\qt@newcmd\qurantrans@de@dcclxxxiii{'Isa, der Sohn Maryams, sagte: ``O Allah, unser Herr, sende zu uns einen Tisch (mit Speisen) vom Himmel herab, der für uns, für den ersten von uns und den letzten von uns, ein Fest sein soll und ein Zeichen von Dir. Und versorge uns. Du bist ja der beste Versorger."\qt@no{114}}%
\qt@newcmd\qurantrans@de@dcclxxxiv{Allah sagte: ``Ich werde ihn gewiß zu euch hinabsenden. Wer von euch aber danach ungläubig ist, den werde Ich mit einer Strafe strafen, mit der Ich (sonst) niemanden (anderen) der Weltenbewohner strafe."\qt@no{115}}%
\qt@newcmd\qurantrans@de@dcclxxxv{Und wenn Allah sagt: ``O 'Isa, Sohn Maryams, bist du es, der zu den Menschen gesagt hat: ,Nehmt mich und meine Mutter außer Allah zu Göttern!'?", wird er sagen: ``Preis sei Dir! Es steht mir nicht zu, etwas zu sagen, wozu ich kein Recht habe. Wenn ich es (tatsächlich doch) gesagt hätte, dann wüßtest Du es bestimmt. Du weißt, was in mir vorgeht, aber ich weiß nicht, was in Dir vorgeht. Du bist ja der Allwisser der verborgenen Dinge.\qt@no{116}}%
\qt@newcmd\qurantrans@de@dcclxxxvi{Ich habe ihnen nur gesagt, was Du mir befohlen hast (, nämlich): ,Dient Allah, meinem und eurem Herrn! Und ich war über sie Zeuge, solange ich unter ihnen weilte. Seitdem Du mich abberufen hast, bist Du der Wächter über sie. Du bist über alles Zeuge.\qt@no{117}}%
\qt@newcmd\qurantrans@de@dcclxxxvii{Wenn Du sie strafst, so sind sie Deine Diener, und wenn Du ihnen vergibst, so bist Du ja der Allmächtige und Allweise."\qt@no{118}}%
\qt@newcmd\qurantrans@de@dcclxxxviii{Allah wird sagen: ``Dies ist der Tag, an dem den Wahrhaftigen ihre Wahrhaftigkeit nützt." Für sie sind Gärten, durcheilt von Bächen, ewig und auf immer darin zu bleiben. Allah hat Wohlgefallen an ihnen, und sie haben Wohlgefallen an Ihm. Das ist der großartige Erfolg.\qt@no{119}}%
\qt@newcmd\qurantrans@de@dcclxxxix{Allah gehört die Herrschaft über die Himmel und die Erde und das, was in ihnen ist. Und Er hat zu allem die Macht.\qt@no{120}}%
\qt@newcmd\qurantrans@de@dccxc{\basmalahde (Alles) Lob gehört Allah, Der die Himmel und die Erde erschaffen und die Finsternisse und das Licht gemacht hat; dennoch setzen diejenigen, die ungläubig sind, ihrem Herrn (andere) gleich.\qt@no{1}}%
\qt@newcmd\qurantrans@de@dccxci{Er ist es, Der euch aus Lehm erschaffen und hierauf eine Frist bestimmt hat. Und (es gibt) eine (andere) festgesetzte Frist bei Ihm; dennoch zweifelt ihr.\qt@no{2}}%
\qt@newcmd\qurantrans@de@dccxcii{Er ist Allah in den Himmeln und auf der Erde. Er kennt euer Geheimes und euer Verlautbartes und weiß, was ihr verdient.\qt@no{3}}%
\qt@newcmd\qurantrans@de@dccxciii{Kein Zeichen von den Zeichen ihres Herrn kam zu ihnen, ohne daß sie sich von ihm abzuwenden pflegten.\qt@no{4}}%
\qt@newcmd\qurantrans@de@dccxciv{So erklärten sie die Wahrheit für Lüge, als sie zu ihnen kam. Aber zu ihnen werden die Nachrichten kommen von dem, worüber sie sich lustig zu machen pflegten.\qt@no{5}}%
\qt@newcmd\qurantrans@de@dccxcv{Haben sie nicht gesehen, wie viele Geschlechter Wir vor ihnen vernichteten, denen Wir auf der Erde eine feste Stellung verliehen hatten, wie Wir sie euch nicht verliehen haben, und auf die Wir den Regen ergiebig (hinab)sandten, und unterhalb derer Wir Flüsse strömen ließen? Und da haben Wir sie für ihre Sünden vernichtet und nach ihnen ein anderes Geschlecht entstehen lassen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@dccxcvi{Wenn Wir auf dich ein Buch aus (beschriebenen) Blättern hinabgesandt hätten, so daß sie es mit ihren Händen befühlen (könnten), würden diejenigen, die ungläubig sind, dennoch sagen: ``Das ist ja nur deutliche Zauberei."\qt@no{7}}%
\qt@newcmd\qurantrans@de@dccxcvii{Und sie sagen: ``Warum ist kein Engel zu ihm herabgesandt worden?" Wenn Wir aber einen Engel herabsenden würden, dann wäre die Angelegenheit wahrlich entschieden, und ihnen würde dann kein Aufschub gewährt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@dccxcviii{Und wenn Wir ihn (den Gesandten auch) zu einem Engel gemacht hätten, so hätten Wir ihn (doch) wahrlich zu einem Mann gemacht, und Wir hätten ihnen wahrlich verdeckt, was sie zu verdecken suchten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@dccxcix{Man machte sich ja schon vor dir über Gesandte lustig. Da umschloß diejenigen, die über sie spotteten, das, worüber sie sich lustig zu machen pflegten.\qt@no{10}}%
\qt@newcmd\qurantrans@de@dccc{Sag: Reist auf der Erde umher und schaut, wie das Ende der Leugner war.\qt@no{11}}%
\qt@newcmd\qurantrans@de@dccci{Sag: Wem gehört, was in den Himmeln und auf der Erde ist? Sag: Allah. Er hat Sich Selbst Barmherzigkeit vorgeschrieben. Er wird euch ganz gewiß zum Tag der Auferstehung versammeln, an dem es keinen Zweifel gibt. (Schaut; wie das Ende) derjenigen (war), die ihre Seelen verloren haben, denn sie glaubten nicht.\qt@no{12}}%
\qt@newcmd\qurantrans@de@dcccii{Ihm gehört (alles), was in der Nacht und am Tag ruht. Und Er ist der Allhörende und Allwissende.\qt@no{13}}%
\qt@newcmd\qurantrans@de@dccciii{Sag: Sollte ich einen anderen zum Schutzherren nehmen als Allah, den Erschaffer der Himmel und der Erde, Der Speise gibt, während Ihm (selbst) keine Speise gegeben wird? Sag: Mir ist befohlen worden, der erste zu sein, der sich (Allah) ergibt, und: ``Gehöre ja nicht zu den Götzendienern!"\qt@no{14}}%
\qt@newcmd\qurantrans@de@dccciv{Sag: Gewiß, ich fürchte, wenn ich mich meinem Herrn widersetze, die Strafe eines gewaltigen Tages.\qt@no{15}}%
\qt@newcmd\qurantrans@de@dcccv{Von wem sie an jenem Tag abgewendet wird, dessen hat Er sich wirklich erbarmt; und das ist der deutliche Erfolg.\qt@no{16}}%
\qt@newcmd\qurantrans@de@dcccvi{Wenn Allah dir Unheil widerfahren läßt, so kann es keiner hinwegnehmen außer Ihm. Und wenn Er dir (etwas) Gutes widerfahren läßt, - so hat Er zu allem die Macht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@dcccvii{Er ist der Bezwinger über Seinen Dienern, und Er ist der Allweise und Allkundige.\qt@no{18}}%
\qt@newcmd\qurantrans@de@dcccviii{Sag: Welches ist das größte Zeugnis? Sag: Allah (, Er) ist Zeuge zwischen mir und euch. Und dieser Qur'an ist mir eingegeben worden, damit ich euch und (jeden), den er erreicht, mit ihm warne. Wollt ihr denn wahrlich bezeugen, daß es neben Allah andere Götter gibt? Sag: Ich bezeuge (es) nicht. Sag: Er ist nur ein Einziger Gott, und ich sage mich von dem los, was ihr (Ihm) beigesellt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@dcccix{Diejenigen, denen Wir die Schrift gegeben haben, kennen ihn, wie sie ihre Söhne kennen. (Es sind) diejenigen, die sich selbst verloren haben, denn sie glauben nicht.\qt@no{20}}%
\qt@newcmd\qurantrans@de@dcccx{Und wer ist ungerechter, als wer gegen Allah eine Lüge ersinnt oder Seine Zeichen für Lüge erklärt? Gewiß, den Ungerechten wird es nicht wohl ergehen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@dcccxi{Und an dem Tag, da Wir sie alle versammeln, und dann werden Wir zu denen, die (Allah etwas) beigesellt hatten, sagen: ``Wo sind (nun) eure Teilhaber, die ihr stets angegeben habt?"\qt@no{22}}%
\qt@newcmd\qurantrans@de@dcccxii{Dann wird ihre Ausrede nur sein, daß sie sagen: ``Bei Allah, unserem Herrn! Wir waren keine Götzendiener."\qt@no{23}}%
\qt@newcmd\qurantrans@de@dcccxiii{Schau, wie sie gegen sich selbst lügen und (wie) ihnen das entschwunden ist, was sie zu ersinnen pflegten!\qt@no{24}}%
\qt@newcmd\qurantrans@de@dcccxiv{Und unter ihnen gibt es manche, die dir zuhören. Aber Wir haben auf ihre Herzen Hüllen gelegt, so daß sie ihn nicht verstehen, und in ihre Ohren Schwerhörigkeit. Auch wenn sie jedes Zeichen sähen, glaubten sie nicht daran. Wenn sie zu dir kommen, um mit dir zu streiten, sagen diejenigen, die ungläubig sind: ``Das sind nur Fabeln der Früheren."\qt@no{25}}%
\qt@newcmd\qurantrans@de@dcccxv{Sie verbieten es (anderen) und halten sich (selbst) davon fern. Sie vernichten nur sich selbst, ohne zu merken.\qt@no{26}}%
\qt@newcmd\qurantrans@de@dcccxvi{Und wenn du sehen würdest, wenn sie vor das (Höllen)feuer gestellt werden! Dann werden sie sagen: ``O würden wir doch zurückgebracht werden! Wir würden (dann) nicht die Zeichen unseres Herrn für Lüge erklären, sondern würden zu den Gläubigen gehören."\qt@no{27}}%
\qt@newcmd\qurantrans@de@dcccxvii{Nein! Vielmehr hat sich ihnen offenkundig gezeigt, was sie zuvor zu verbergen pflegten. Und wenn sie zurückgebracht würden, würden sie doch wieder zu dem zurückkehren, was ihnen verboten wurde. Sie sind wahrlich Lügner.\qt@no{28}}%
\qt@newcmd\qurantrans@de@dcccxviii{Und sie sagen: ``Es gibt nur unser diesseitiges Leben, und wir werden nicht auferweckt."\qt@no{29}}%
\qt@newcmd\qurantrans@de@dcccxix{Und wenn du sehen würdest, wenn sie vor ihren Herrn gestellt werden! Er wird sagen: ``Ist dies nicht die Wahrheit?" Sie sagen: ``Ja doch, bei unserem Herrn!" Er wird sagen: ``Kostet nun die Strafe dafür, daß ihr stets ungläubig wart."\qt@no{30}}%
\qt@newcmd\qurantrans@de@dcccxx{Verloren sind diejenigen, die die Begegnung mit Allah für Lüge erklären, so daß, wenn die Stunde plötzlich über sie kommt, sie sagen: ``O welch gramvolle Reue für uns, wegen dessen, was wir hinsichtlich ihrer vernachlässigt haben!" Sie tragen ihre Lasten auf ihren Rücken. Wie übel ist das, was sie an Last auf sich nehmen!\qt@no{31}}%
\qt@newcmd\qurantrans@de@dcccxxi{Das diesseitige Leben ist nur Spiel und Zerstreuung. Die jenseitige Wohnstätte ist für diejenigen, die gottesfürchtig sind, wahrlich besser. Begreift ihr denn nicht?\qt@no{32}}%
\qt@newcmd\qurantrans@de@dcccxxii{Wir wissen wohl, daß dich das, was sie sagen, in der Tat traurig macht. Aber nicht dich bezichtigen sie (in Wirklichkeit) der Lüge, sondern die Zeichen Allahs verleugnen die(se) Ungerechten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@dcccxxiii{Ja, bereits vor dir wurden Gesandte der Lüge bezichtigt. Und da ertrugen sie standhaft, daß sie der Lüge bezichtigt wurden und daß ihnen Leid zugefügt wurde, bis Unsere Hilfe zu ihnen kam. Es gibt nichts, was Allahs Worte abändern könnte. Dir ist ja bereits Kunde von den Gesandten zugekommen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@dcccxxiv{Und wenn dir ihre Ablehnung schwer zusetzt, dann, (selbst) wenn du einen Gang in die Erde oder eine Leiter in den Himmel ausfindig machen kannst, und ihnen dann ein Zeichen bringen kannst, ... Und wenn Allah gewollt hätte, hätte Er sie wahrlich unter der Rechtleitung zusammengeführt. Gehöre also ja nicht zu den Toren!\qt@no{35}}%
\qt@newcmd\qurantrans@de@dcccxxv{Nur diejenigen können Folge leisten, die (zu)hören. Die Toten aber wird Allah auferwecken. Hierauf werden sie zu Ihm zurückgebracht.\qt@no{36}}%
\qt@newcmd\qurantrans@de@dcccxxvi{Und sie sagen: ``Wenn ihm doch ein Zeichen von seinem Herrn offenbart worden wäre!" Sag: Gewiß, Allah hat die Macht, ein Zeichen zu offenbaren. Aber die meisten von ihnen wissen nicht.\qt@no{37}}%
\qt@newcmd\qurantrans@de@dcccxxvii{Es gibt kein Tier auf der Erde und keinen Vogel, der mit seinen Flügeln fliegt, die nicht Gemeinschaften wären gleich euch. Wir haben im Buch nichts vernachlässigt. Hierauf werden sie zu ihrem Herrn versammelt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@dcccxxviii{Und diejenigen, die Unsere Zeichen für Lüge erklären, sind taub und stumm in Finsternissen. Wen Allah will, den läßt Er in die Irre gehen, und wen Er will, den bringt Er auf einen geraden Weg.\qt@no{39}}%
\qt@newcmd\qurantrans@de@dcccxxix{Sag: Was meint ihr wohl? Wenn die Strafe Allahs (über) euch kommt oder die Stunde (über) euch kommt, werdet ihr (dann) einen anderen anrufen als Allah? (So antwortet,) wenn ihr wahrhaftig seid.\qt@no{40}}%
\qt@newcmd\qurantrans@de@dcccxxx{Nein! Vielmehr Ihn werdet ihr anrufen. Dann wird Er das hinwegnehmen, weswegen ihr (Ihn) anruft, wenn Er will, und ihr werdet vergessen, was ihr (Ihm) beigesellt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@dcccxxxi{Wir haben ja schon zu Gemeinschaften vor dir (Gesandte) gesandt und über sie Not und Leid kommen lassen, auf daß sie unterwürfig flehen mögen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@dcccxxxii{Wenn sie doch nur, als Unsere Gewalt über sie kam, unterwürfig gefleht hätten! Aber ihre Herzen verhärteten sich, und der Satan schmückte ihnen aus, was sie zu tun pflegten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@dcccxxxiii{Als sie dann vergaßen, womit sie ermahnt worden waren, öffneten Wir ihnen die Tore zu allen Dingen. Als sie dann froh waren über das, was ihnen gegeben worden war, ergriffen Wir sie plötzlich. Da waren sie sogleich verzweifelt.\qt@no{44}}%
\qt@newcmd\qurantrans@de@dcccxxxiv{So soll die Rückkehr der Leute, die Unrecht taten, abgeschnitten sein. (Alles) Lob gehört Allah, dem Herrn der Weltenbewohner!\qt@no{45}}%
\qt@newcmd\qurantrans@de@dcccxxxv{Sag: Was meint ihr, wenn Allah euer Gehör und euer Augenlicht (weg)nähme und eure Herzen versiegelte, wer ist der Gott außer Allah, der es euch zurückgeben könnte? Schau, wie Wir die Zeichen verschiedenartig darlegen! Dennoch wenden sie sich hierauf ab.\qt@no{46}}%
\qt@newcmd\qurantrans@de@dcccxxxvi{Sag: Was meint ihr wohl? Wenn Allahs Strafe plötzlich oder offen (über) euch kommt, wer wird (dann) vernichtet außer dem ungerechten Volk?\qt@no{47}}%
\qt@newcmd\qurantrans@de@dcccxxxvii{Wir senden die Gesandten nur als Verkünder froher Botschaft und als Überbringer von Warnungen. Wer also glaubt und Besserung bringt, über die soll keine Furcht kommen, noch sollen sie traurig sein.\qt@no{48}}%
\qt@newcmd\qurantrans@de@dcccxxxviii{Denjenigen aber, die Unsere Zeichen für Lüge erklären, wird die Strafe widerfahren dafür, daß sie zu freveln pflegten.\qt@no{49}}%
\qt@newcmd\qurantrans@de@dcccxxxix{Sag: Ich sage nicht zu euch, ich besäße die Schatzkammern Allahs, und ich weiß auch nicht das Verborgene; und ich sage nicht zu euch, ich sei ein Engel. Ich folge nur dem, was mir eingegeben wird. Sag: Sind (etwa) der Blinde und der Sehende gleich? Denkt ihr denn nicht nach?\qt@no{50}}%
\qt@newcmd\qurantrans@de@dcccxl{Und warne damit diejenigen, die befürchten, daß sie zu ihrem Herrn versammelt werden - sie haben außer Ihm weder Schutzherrn noch Fürsprecher -, auf daß sie gottesfürchtig werden mögen!\qt@no{51}}%
\qt@newcmd\qurantrans@de@dcccxli{Und weise nicht diejenigen ab, die morgens und abends ihren Herrn anrufen im Begehren nach Seinem Angesicht! Dir obliegt in keiner Weise, sie zur Rechenschaft zu ziehen, und ihnen obliegt in keiner Weise, dich zur Rechenschaft zu ziehen. Wenn du sie abwiesest, dann würdest du zu den Ungerechten gehören.\qt@no{52}}%
\qt@newcmd\qurantrans@de@dcccxlii{Und so haben Wir die einen von ihnen durch die anderen einer Prüfung unterzogen, so daß sie sagen: ``Hat Allah diesen da aus unserer Mitte eine Wohltat erwiesen?" Weiß nicht Allah am besten über die Dankbaren Bescheid?\qt@no{53}}%
\qt@newcmd\qurantrans@de@dcccxliii{Und wenn diejenigen, die an Unsere Zeichen glauben, zu dir kommen, dann sag: Friede sei auf euch! Euer Herr hat Sich Selbst Barmherzigkeit vorgeschrieben: Wer von euch in Unwissenheit Böses tut, aber danach dann bereut und (es) wieder gutmacht, so ist Er Allvergebend und Barmherzig.\qt@no{54}}%
\qt@newcmd\qurantrans@de@dcccxliv{So legen Wir die Zeichen ausführlich dar, - damit sich der Weg der Übeltäter klar zeigt.\qt@no{55}}%
\qt@newcmd\qurantrans@de@dcccxlv{Sag: Mir ist verboten worden, denjenigen zu dienen, die ihr anstatt Allahs anruft. Sag: Ich folge nicht euren Neigungen; ich würde sonst nämlich fürwahr abirren und würde nicht zu den Rechtgeleiteten gehören.\qt@no{56}}%
\qt@newcmd\qurantrans@de@dcccxlvi{Sag: Ich halte mich an einen klaren Beweis von meinem Herrn, während ihr Ihn der Lüge bezichtigt. Ich verfüge nicht über das, was ihr zu beschleunigen wünscht. Das Urteil gehört allein Allah. Er berichtet die Wahrheit, und Er ist der Beste derer, die entscheiden.\qt@no{57}}%
\qt@newcmd\qurantrans@de@dcccxlvii{Sag: Wenn ich über das verfügen würde, was ihr zu beschleunigen wünscht, wäre die Angelegenheit zwischen mir und euch wahrlich schon entschieden. Allah weiß am besten über die Ungerechten Bescheid.\qt@no{58}}%
\qt@newcmd\qurantrans@de@dcccxlviii{Er verfugt über die Schlüssel des Verborgenen; niemand kennt sie außer Ihm. Und Er weiß, was auf dem Festland und im Meer ist. Kein Blatt fällt, ohne daß Er es weiß; und (es gibt) kein Korn in den Finsternissen der Erde und nichts Feuchtes und nichts Trockenes, das nicht in einem deutlichen Buch (verzeichnet) wäre.\qt@no{59}}%
\qt@newcmd\qurantrans@de@dcccxlix{Er ist es, Der euch bei Nacht abberuft und weiß, was ihr bei Tag begangen habt, und euch hierauf an ihm auferweckt, damit eine festgesetzte Frist erfüllt wird. Hierauf wird eure Rückkehr zu Ihm sein, und hierauf wird Er euch kundtun, was ihr zu tun pflegtet.\qt@no{60}}%
\qt@newcmd\qurantrans@de@dcccl{Er ist der Bezwinger über Seinen Dienern. Und Er sendet Hüter über euch, bis, wenn dann zu einem von euch der Tod kommt, ihn Unsere Gesandten abberufen, und sie vernachlässigen nichts.\qt@no{61}}%
\qt@newcmd\qurantrans@de@dcccli{Hierauf werden sie zu Allah, ihrem wahren Schutzherrn, zurückgebracht. Sicherlich, Sein ist das Urteil, und Er ist der schnellste Berechner.\qt@no{62}}%
\qt@newcmd\qurantrans@de@dccclii{Sag: Wer errettet euch von den Finsternissen des Festlandes und des Meeres, (wenn) ihr Ihn in Unterwürfigkeit und im Verborgenen flehend anruft: ``Wenn Er uns nur hieraus rettet, werden wir ganz gewiß zu den Dankbaren gehören"?\qt@no{63}}%
\qt@newcmd\qurantrans@de@dcccliii{Sag: Allah errettet euch davon und von jeder Trübsal, hierauf gesellt ihr (Ihm) dennoch (andere Götter) bei.\qt@no{64}}%
\qt@newcmd\qurantrans@de@dcccliv{Sag: Er hat die Macht dazu, euch eine Strafe von oben oder unter euren Füßen zu schicken oder euch in (gespaltene) Lager durcheinanderzubringen und die einen von euch die Gewalt der anderen kosten zu lassen! Schau, wie Wir die Zeichen verschiedenartig darlegen, auf daß sie verstehen mögen!\qt@no{65}}%
\qt@newcmd\qurantrans@de@dccclv{Aber dein Volk erklärt es für Lüge, obwohl es die Wahrheit ist. Sag: Ich bin nicht als Sachwalter über euch (eingesetzt).\qt@no{66}}%
\qt@newcmd\qurantrans@de@dccclvi{Jede Vorankündigung hat ihre festgesetzte Zeit, und ihr werdet (es noch) erfahren.\qt@no{67}}%
\qt@newcmd\qurantrans@de@dccclvii{Und wenn du diejenigen siehst, die auf Unsere Zeichen (spottend) eingehen, so wende dich von ihnen ab, bis sie auf ein anderes Gespräch eingehen. Und wenn dich der Satan nun vergessen läßt, dann sitze nicht, nachdem du dich (daran) erinnert hast, mit dem ungerechten Volk zusammen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@dccclviii{Denjenigen, die gottesfürchtig sind, obliegt in keiner Weise, sie zur Rechenschaft zu ziehen, sondern nur die Ermahnung, auf daß sie gottesfürchtig werden mögen.\qt@no{69}}%
\qt@newcmd\qurantrans@de@dccclix{Und lasse diejenigen allein, die ihre Religion zum Gegenstand des Spiels und der Zerstreuung nehmen und die das diesseitige Leben täuscht! Und ermahne durch ihn, damit sich keine Seele für das, was sie verdient hat, dem Verderben ausliefert. Sie hat (dann) außer Allah weder Schutzherrn noch Fürsprecher. Und wenn sie auch (als Lösegeld) jede Ersatzleistung anbietet, wird sie von ihr doch nicht angenommen. Das sind diejenigen, die dem Verderben ausgeliefert werden für das, was sie verdient haben. Für sie wird es ein Getränk aus heißem Wasser und schmerzhafte Strafe geben dafür, daß sie ungläubig gewesen sind.\qt@no{70}}%
\qt@newcmd\qurantrans@de@dccclx{Sag: Sollen wir anstatt Allahs anrufen, was uns nicht nützt und nicht schadet, und auf unseren Fersen kehrtmachen, nachdem Allah uns rechtgeleitet hat, wie derjenige, den die Teufel auf der Erde verführt haben, so daß er verwirrt ist? Er hat Gefährten, die ihn zur Rechtleitung einladen: ``Komm zu uns!" Sag: Gewiß, Allahs Rechtleitung ist die (wahre) Rechtleitung. Und uns wurde befohlen, uns dem Herrn der Weltenbewohner zu ergeben,\qt@no{71}}%
\qt@newcmd\qurantrans@de@dccclxi{und verrichtet das Gebet und fürchtet Ihn! Er ist es, zu Dem ihr versammelt werdet.\qt@no{72}}%
\qt@newcmd\qurantrans@de@dccclxii{Und Er ist es, Der die Himmel und die Erde in Wahrheit erschaffen hat. Und an dem Tag, da Er sagt: ``Sei!" da wird es sein. Sein Wort ist die Wahrheit. Und Er hat die Herrschaft an dem Tag, da ins Horn geblasen wird. (Er ist) der Kenner des Verborgenen und des Offenbaren, und Er ist der Allweise und Allkundige.\qt@no{73}}%
\qt@newcmd\qurantrans@de@dccclxiii{Und als Ibrahim zu seinem Vater Azar sagte: ``Nimmst du (denn) Götzenbilder zu Göttern? Gewiß, ich sehe dich und dein Volk in deutlichem Irrtum."\qt@no{74}}%
\qt@newcmd\qurantrans@de@dccclxiv{Und so zeigten Wir Ibrahim das Reich der Himmel und der Erde, - und damit er zu den Überzeugten gehöre.\qt@no{75}}%
\qt@newcmd\qurantrans@de@dccclxv{Als die Nacht über ihn hereinbrach, sah er einen Himmelskörper. Er sagte: ``Das ist mein Herr." Als er aber unterging, sagte er: ``Ich liebe nicht diejenigen, die untergehen'."\qt@no{76}}%
\qt@newcmd\qurantrans@de@dccclxvi{Als er dann den Mond aufgehen sah, sagte er: ``Das ist mein Herr." Als er aber unterging, sagte er: ``Wenn mein Herr mich nicht rechtleitet, werde ich ganz gewiß zum irregehenden Volk gehören."\qt@no{77}}%
\qt@newcmd\qurantrans@de@dccclxvii{Als er dann die Sonne aufgehen sah, sagte er: ``Das ist mein Herr. Das ist größer." Als sie aber unterging, sagte er: ``O mein Volk, ich sage mich ja von dem los, was ihr (Ihm) beigesellt.\qt@no{78}}%
\qt@newcmd\qurantrans@de@dccclxviii{Ich wende mein Gesicht Dem zu, Der die Himmel und die Erde erschaffen hat, als Anhänger des rechten Glaubens, und ich gehöre nicht zu den Götzendienern.\qt@no{79}}%
\qt@newcmd\qurantrans@de@dccclxix{Sein Volk stritt mit ihm. Er sagte: ``Wollt ihr mit mir über Allah streiten, wo Er mich doch schon rechtgeleitet hat? Ich fürchte nicht, was ihr Ihm beigesellt, außer daß mein Herr will (, daß ich) es (fürchte). Mein Herr umfaßt alles mit (Seinem) Wissen. Bedenkt ihr denn nicht?\qt@no{80}}%
\qt@newcmd\qurantrans@de@dccclxx{Wie sollte ich fürchten, was ihr (Ihm) beigesellt, wo ihr euch nicht fürchtet, Allah beizugesellen, wozu Er euch keine Ermächtigung offenbart hat? Welche der beiden Gruppen hat also ein größeres Anrecht auf Sicherheit, wenn ihr wißt?"\qt@no{81}}%
\qt@newcmd\qurantrans@de@dccclxxi{Diejenigen, die glauben und ihren Glauben nicht mit Ungerechtigkeit verdecken, die haben (das Recht auf) Sicherheit, und sie sind rechtgeleitet.\qt@no{82}}%
\qt@newcmd\qurantrans@de@dccclxxii{Das ist unser Beweismittel, das Wir Ibrahim gegen sein Volk gaben. Wir erhöhen, wen Wir wollen, um Rangstufen. Gewiß, dein Herr ist Allweise und Allwissend.\qt@no{83}}%
\qt@newcmd\qurantrans@de@dccclxxiii{Und Wir schenkten ihm Ishaq und Ya'qub; jeden (von ihnen) haben Wir rechtgeleitet. Und (auch) Nuh haben Wir zuvor rechtgeleitet, und aus seiner Nachkommenschaft Dawud, Sulaiman, Ayyub, Yusuf, Musa und Harun - so vergelten Wir (es) den Gutes Tuenden -;\qt@no{84}}%
\qt@newcmd\qurantrans@de@dccclxxiv{und Zakariyya, Yahya, 'Isa und Ilyas: jeder (von ihnen) gehört zu den Rechtschaffenen;\qt@no{85}}%
\qt@newcmd\qurantrans@de@dccclxxv{und lsma'il, Alyasa', Yunus und Lut: jeden (von ihnen) haben Wir vor den (anderen) Weltenbewohnern bevorzugt;\qt@no{86}}%
\qt@newcmd\qurantrans@de@dccclxxvi{und (auch manche) von ihren (Vor)Vätern, ihren Nachkommen und ihren Brüdern; Wir haben sie erwählt und zu einem geraden Weg geleitet.\qt@no{87}}%
\qt@newcmd\qurantrans@de@dccclxxvii{Das ist Allahs Rechtleitung. Er leitet damit recht, wen von Seinen Dienern Er will. Wenn sie (Ihm) aber (andere) beigesellt hätten, wäre für sie wahrlich hinfällig geworden, was sie zu tun pflegten.\qt@no{88}}%
\qt@newcmd\qurantrans@de@dccclxxviii{Das sind diejenigen, denen Wir die Schrift, das Urteil und das Prophetentum gegeben haben. Wenn aber diese es verleugnen, so haben Wir damit schon (andere) Leute betraut, die dem gegenüber nicht ungläubig sind.\qt@no{89}}%
\qt@newcmd\qurantrans@de@dccclxxix{Das sind diejenigen, die Allah rechtgeleitet hat. So nimm ihre Rechtleitung zum Vorbild! Sag: Ich verlange von euch keinen Lohn dafür. Es ist nur eine Ermahnung für die Weltenbewohner.\qt@no{90}}%
\qt@newcmd\qurantrans@de@dccclxxx{Sie schätzen Allah nicht ein, wie es Ihm gebührt', wenn sie sagen: ``Allah hat nichts auf ein menschliches Wesen (als Offenbarung) herabgesandt." Sag: Wer sandte (denn) die Schrift herab, die Musa als Licht und Rechtleitung für die Menschen brachte, die ihr zu (beschriebenen) Blättern macht, die ihr offen (vor)zeigt, während ihr vieles verbergt, und (sag: von wem) wurde euch gelehrt, was ihr nicht wußtet, weder ihr noch eure Väter? Sag: (Von) Allah. Sodann lasse sie mit ihren schweifenden Gesprächen ihr Spiel treiben.\qt@no{91}}%
\qt@newcmd\qurantrans@de@dccclxxxi{Und dies ist ein Buch, das Wir hinabgesandt haben, ein gesegnetes, das bestätigend, was vor ihm war, und damit du die Mutter der Städte und diejenigen rings umher (Wohnenden) warnst. Diejenigen, die an das Jenseits glauben, glauben (auch) daran, und sie halten ihr Gebet ein.\qt@no{92}}%
\qt@newcmd\qurantrans@de@dccclxxxii{Und wer ist ungerechter, als wer gegen Allah eine Lüge ersinnt oder sagt: ``Mir ist (Offenbarung) eingegeben worden", während ihm überhaupt nichts eingegeben worden ist, und wer sagt: ``Ich werde hinabsenden, gleich dem, was Allah hinabgesandt hat"? Und wenn du sehen würdest, wie sich die Ungerechten in den Fluten des Todes befinden und die Engel ihre Hände ausstrecken: ``Gebt eure Seelen heraus! Heute wird euch mit der schmählichen Strafe vergolten, daß ihr stets über Allah die Unwahrheit gesagt habt und euch gegenüber Seinen Zeichen hochmütig zu verhalten pflegtet".\qt@no{93}}%
\qt@newcmd\qurantrans@de@dccclxxxiii{Nun seid ihr einzeln zu Uns gekommen, so wie Wir euch das erste Mal erschaffen haben, und ihr habt hinter euch zurückgelassen, was Wir euch übertragen hatten. Und Wir sehen eure Fürsprecher nicht bei euch, von denen ihr behauptet habt, daß sie Teilhaber an euch waren. Nun ist (das Band) zwischen euch abgeschnitten, und entschwunden ist euch, was ihr zu behaupten pflegtet.\qt@no{94}}%
\qt@newcmd\qurantrans@de@dccclxxxiv{Allah ist es, Der die Körner und die Kerne spaltet und das Lebendige aus dem Toten hervorbringt. Und (Er ist es,) Der das Tote aus dem Lebendigen hervorbringt. Dies ist doch Allah - wie laßt ihr euch also abwendig machen? -,\qt@no{95}}%
\qt@newcmd\qurantrans@de@dccclxxxv{Er, Der den Morgen anbrechen läßt'. Er hat die Nacht zur Ruhe(zeit) und die Sonne und den Mond als (Mittel der) Berechnung gemacht. Das ist die Anordnung des Allmächtigen und Allwissenden.\qt@no{96}}%
\qt@newcmd\qurantrans@de@dccclxxxvi{Und Er ist es, Der euch die Sterne gemacht hat, damit ihr euch durch sie rechtleiten laßt in den Finsternissen des Festlandes und des Meeres. Wir haben ja die Zeichen ausführlich dargelegt für Leute, die Bescheid wissen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@dccclxxxvii{Und Er ist es, Der euch aus einem einzigen Wesen hat entstehen lassen. Dann gibt es einen Aufenthaltsort und einen Aufbewahrungsort. Wir haben die Zeichen ausführlich dargelegt für Leute, die verstehen.\qt@no{98}}%
\qt@newcmd\qurantrans@de@dccclxxxviii{Und Er ist es, Der vom Himmel Wasser herabkommen läßt. Damit bringen Wir den Wuchs aller Arten hervor; aus ihnen bringen Wir dann Grün hervor, aus dem Wir übereinandergeschichtete Körner hervorbringen - und aus den Palmen, aus ihren Blütenscheiden (entstehen) herabhängende Dattelbüschel -, und (auch) Gärten mit Rebstöcken und die Öl- und die Granatapfelbäume, die einander ähnlich und unähnlich sind. Schaut ihre Früchte an, wenn sie Früchte tragen, und (schaut) auf deren Reife! Seht, darin sind wahrlich Zeichen für Leute, die glauben.\qt@no{99}}%
\qt@newcmd\qurantrans@de@dccclxxxix{Und sie haben Teilhaber Allah gegeben: die Ginn, wo Er sie doch erschaffen hat. Und sie haben Ihm Söhne und Töchter angedichtet, ohne Wissen. Preis sei Ihm! Erhaben ist Er über das, was sie (Ihm) zuschreiben.\qt@no{100}}%
\qt@newcmd\qurantrans@de@dcccxc{Er ist) der Schöpfer der Himmel und der Erde in ihrer schönsten Form. Wie soll Er Kinder haben, wo Er doch keine Gefährtin hat und Er (sonst) alles erschaffen hat? Und Er weiß über alles Bescheid.\qt@no{101}}%
\qt@newcmd\qurantrans@de@dcccxci{Dies ist doch Allah, euer Herr. Es gibt keinen Gott außer Ihm, dem Schöpfer von allem. So dient Ihm! Er ist Sachwalter über alles.\qt@no{102}}%
\qt@newcmd\qurantrans@de@dcccxcii{Die Blicke erfassen Ihn nicht, Er aber erfaßt die Blicke. Und Er ist der Feinfühlige und Allkundige.\qt@no{103}}%
\qt@newcmd\qurantrans@de@dcccxciii{Zu euch sind nunmehr einsicht bringende Zeichen von eurem Herrn gekommen. Wer einsichtig wird, der ist es zu seinem eigenen Vorteil, und wer blind ist, der ist es zu seinem eigenen Nachteil. Und ich bin nicht Hüter über euch.\qt@no{104}}%
\qt@newcmd\qurantrans@de@dcccxciv{So legen Wir die Zeichen verschiedenartig dar, - und damit sie sagen können: ``Du hast (es) erlernt" und damit Wir ihn Leuten klar machen, die Bescheid wissen.\qt@no{105}}%
\qt@newcmd\qurantrans@de@dcccxcv{Folge dem, was dir von deinem Herrn (als Offenbarung) eingegeben worden ist! Es gibt keinen Gott außer Ihm. Und wende dich von den Götzendienern ab!\qt@no{106}}%
\qt@newcmd\qurantrans@de@dcccxcvi{Wenn Allah gewollt hätte, hätten sie (Ihm) nicht(s) beigesellt. Und Wir haben dich nicht zum Hüter über sie gemacht, noch bist du (als) Sachwalter über sie (eingesetzt).\qt@no{107}}%
\qt@newcmd\qurantrans@de@dcccxcvii{Und schmäht nicht diejenigen, die sie außer Allah anrufen, damit sie nicht in Übertretung ohne Wissen Allah schmähen! So haben Wir jeder Gemeinschaft ihr Tun ausgeschmückt erscheinen lassen. Hierauf wird ihre Rückkehr zu ihrem Herrn sein, und Er wird ihnen kundtun, was sie zu tun pflegten.\qt@no{108}}%
\qt@newcmd\qurantrans@de@dcccxcviii{Und sie haben bei Allah ihren kräftigsten Eid geschworen, sie würden, wenn nur ein Zeichen zu ihnen käme, ganz gewiß daran glauben. Sag: Nur bei Allah sind die Zeichen. Was sollte euch auch merken lassen, daß, wenn es tatsächlich käme, sie doch nicht glauben?\qt@no{109}}%
\qt@newcmd\qurantrans@de@dcccxcix{Und Wir kehren ihre Herzen und ihr Augenlicht um, so wie sie das erste Mal nicht daran geglaubt haben. Und Wir lassen sie in ihrer Auflehnung umherirren.\qt@no{110}}%
\qt@newcmd\qurantrans@de@cm{Wenn Wir auch zu ihnen die Engel hinabsendeten, die Toten zu ihnen sprächen und Wir alle Dinge vor ihren Augen versammelten, sie würden unmöglich glauben, es sei denn, Allah wollte es. Aber die meisten von ihnen wissen nicht.\qt@no{111}}%
\qt@newcmd\qurantrans@de@cmi{Und so haben Wir jedem Propheten Feinde bestimmt: die Satane der Menschen und der Ginn, von denen die einen den anderen prunkende Worte eingeben in Trug und wenn dein Herr gewollt hätte, hätten sie es nicht getan; so lasse sie (stehen) mit dem, was sie an Lügen ersinnen -,\qt@no{112}}%
\qt@newcmd\qurantrans@de@cmii{und damit die Herzen derjenigen, die an das Jenseits nicht glauben, dem zuneigen, und damit sie damit zufrieden sind und begehen, was sie begehen.\qt@no{113}}%
\qt@newcmd\qurantrans@de@cmiii{Soll ich denn einen anderen Schiedsrichter als Allah begehren, wo Er es doch ist, der das Buch, ausführlich dargelegt, zu euch herabgesandt hat? Diejenigen, denen Wir die Schrift gaben, wissen, daß es von deinem Herrn mit der Wahrheit herabgesandt wurde. So gehöre ja nicht zu den Zweiflern.\qt@no{114}}%
\qt@newcmd\qurantrans@de@cmiv{Vollkommen ist das Wort deines Herrn in Wahrhaftigkeit und Gerechtigkeit. Es gibt niemanden, der Seine Worte abändern könnte. Und Er ist der Allhörende und Allwissende.\qt@no{115}}%
\qt@newcmd\qurantrans@de@cmv{Wenn du den meisten von denen, die auf der Erde sind, gehorchst, werden sie dich von Allahs Weg ab in die Irre führen. Sie folgen nur Mutmaßungen, und sie stellen nur Schätzungen an.\qt@no{116}}%
\qt@newcmd\qurantrans@de@cmvi{Gewiß, dein Herr kennt sehr wohl, wer von Seinem Weg abirrt, und Er kennt sehr wohl die Rechtgeleiteten.\qt@no{117}}%
\qt@newcmd\qurantrans@de@cmvii{Eßt von dem, worüber Allahs Name ausgesprochen worden ist, wenn ihr an Seine Zeichen glaubt.\qt@no{118}}%
\qt@newcmd\qurantrans@de@cmviii{Was ist mit euch, daß ihr nicht von dem eßt, worüber Allahs Name ausgesprochen worden ist, wo Er euch doch ausführlich dargelegt hat, was Er euch verboten hat, außer dem, wozu ihr gezwungen werdet? Viele führen wahrlich (andere) durch ihre Neigungen ohne Wissen in die Irre. Aber dein Herr weiß sehr wohl über die Übertreter Bescheid.\qt@no{119}}%
\qt@newcmd\qurantrans@de@cmix{Unterlaßt das Offenkundige an Sünde und das Verborgene! Gewiß, denen, die Sünde erwerben, wird das vergolten, was sie zu begehen pflegten.\qt@no{120}}%
\qt@newcmd\qurantrans@de@cmx{Und eßt nicht von dem, worüber der Name Allahs nicht ausgesprochen worden ist. Das ist wahrlich Frevel. Die Satane geben ihren Schützlingen in der Tat ein, mit euch zu streiten. Wenn ihr ihnen gehorcht, seid ihr fürwahr Götzendiener.\qt@no{121}}%
\qt@newcmd\qurantrans@de@cmxi{Ist denn der, der tot war, und den Wir dann lebendig gemacht und dem Wir ein Licht gegeben haben, worin er unter den Menschen geht, wie einer, dessen Gleichnis das jemandes ist, der sich in Finsternissen befindet, aus denen er nicht herauskommen kann? So ist den Ungläubigen ausgeschmückt, was sie zu tun pflegten.\qt@no{122}}%
\qt@newcmd\qurantrans@de@cmxii{Und so haben Wir in jeder Stadt ihre größten Übeltäter bestimmt, damit sie darin Ränke schmieden. Sie schmieden aber Ränke nur gegen sich selbst, ohne zu merken.\qt@no{123}}%
\qt@newcmd\qurantrans@de@cmxiii{Und wenn ein Zeichen zu ihnen kommt, sagen sie: ``Wir werden nicht glauben, bevor uns nicht gegeben wird, was den Gesandten Allahs gegeben wurde." Allah weiß sehr wohl, wo Er Seine Botschaft anbringt. Diejenigen, die Übeltaten begehen, werden bei Allah Erniedrigung und strenge Strafe treffen dafür, daß sie Ränke zu schmieden pflegten.\qt@no{124}}%
\qt@newcmd\qurantrans@de@cmxiv{Wen Allah rechtleiten will, dem tut Er die Brust auf für den Islam. Und wen Er in die Irre gehen lassen will, dem macht Er die Brust eng und bedrängt, so als ob er in den Himmel hochsteigen sollte. So legt Allah den Greuel auf diejenigen, die nicht glauben.\qt@no{125}}%
\qt@newcmd\qurantrans@de@cmxv{Und dies ist der Weg deines Herrn, ein gerader. Nunmehr haben Wir die Zeichen ausführlich dargelegt für Leute, die bedenken.\qt@no{126}}%
\qt@newcmd\qurantrans@de@cmxvi{Für sie gibt es die Wohnstätte des Friedens bei ihrem Herrn. Er ist ihr Schutzherr, für das, was sie zu tun pflegten.\qt@no{127}}%
\qt@newcmd\qurantrans@de@cmxvii{Und an dem Tag, da Er sie alle versammelt (wird Er sagen): ``O Gesellschaft der Ginn! Viele Menschen habt ihr (verführt)." Und ihre Schützlinge unter den Menschen sagen: ``Unser Herr, wir haben voneinander (den Vorteil) genossen und haben (nun) unsere Frist erreicht, die Du uns gesetzt hast." Er wird sagen: ``Das (Höllen)feuer ist euer Aufenthalt, ewig darin zu bleiben, außer Allah will es (anders)." Gewiß, dein Herr ist Allweise und Allwissend.\qt@no{128}}%
\qt@newcmd\qurantrans@de@cmxviii{So machen Wir die einen Ungerechten zu Beschützern der anderen für das, was sie erworben haben.\qt@no{129}}%
\qt@newcmd\qurantrans@de@cmxix{``O Gesellschaft der Ginn und der Menschen, sind nicht zu euch Gesandte von euch (selbst) gekommen, die euch Meine Zeichen berichtet und euch vor dem Eintreffen dieses euren Tages gewarnt haben?" Sie werden sagen: ``Wir zeugen gegen uns selbst." Das diesseitige Leben hat sie getäuscht, und sie zeugen gegen sich selbst, daß sie ungläubig waren.\qt@no{130}}%
\qt@newcmd\qurantrans@de@cmxx{Das (ist so), weil dein Herr die Städte nicht zu Unrecht vernichtet', während ihre Bewohner unachtsam sind.\qt@no{131}}%
\qt@newcmd\qurantrans@de@cmxxi{Für alle wird es Rangstufen geben je nach dem, was sie getan haben. Und dein Herr ist nicht unachtsam dessen, was sie tun.\qt@no{132}}%
\qt@newcmd\qurantrans@de@cmxxii{Und dein Herr ist der Unbedürftige, voll der Barmherzigkeit. Wenn Er will, läßt Er euch vergehen und nach euch folgen, was Er will, so wie Er euch aus der Nachkommenschaft anderer Leute entstehen ließ.\qt@no{133}}%
\qt@newcmd\qurantrans@de@cmxxiii{Gewiß, was euch versprochen wird, wird sicher eintreffen. Und ihr werdet euch (Ihm) nicht entziehen können.\qt@no{134}}%
\qt@newcmd\qurantrans@de@cmxxiv{Sag: O mein Volk, handelt nach eurer Stellung! Ich werde ebenfalls (so) handeln. Dann werdet ihr (noch) erfahren, wem die letztendliche Wohnstätte gehören wird. Gewiß, den Ungerechten wird es nicht wohl ergehen.\qt@no{135}}%
\qt@newcmd\qurantrans@de@cmxxv{Und sie haben Allah mit dem, was Er an (Früchten auf den) Saatfeldern und Vieh wachsen läßt, einen Anteil bestimmt. Und sie sagen: ``Dies ist für Allah" - so behaupten sie -, ``Und dies ist für unsere Teilhaber." Was nun für ihre Teilhaber (bestimmt) ist, gelangt nicht zu Allah, was aber für Allah (bestimmt) ist, gelangt zu ihren Teilhabern. Wie böse ist, was sie urteilen!\qt@no{136}}%
\qt@newcmd\qurantrans@de@cmxxvi{Und ebenso haben vielen der Götzendiener ihre Teilhaber wohlgefällig erscheinen lassen, ihre Kinder zu töten, um sie ins Verderben zu stürzen und um ihnen ihre Religion zu verdecken. Wenn Allah wollte, hätten sie es nicht getan. So lasse sie (allein) mit dem, was sie an Lügen ersinnen!\qt@no{137}}%
\qt@newcmd\qurantrans@de@cmxxvii{Und sie sagen: ``Dies sind verwehrtes Vieh und verwehrte (Früchte auf den) Saatfelder(n). Niemand kostet davon außer, wen wir (essen lassen) wollen." - So behaupten sie. - Und (es gibt nach ihrer Behauptung) Vieh, dessen Rücken (zum Tragen von Lasten) verboten ist, und Vieh, über das sie den Namen Allahs nicht aussprechen, womit sie Lügen gegen Ihn ersinnen. Er wird ihnen vergelten, was sie stets an Lügen ersinnen.\qt@no{138}}%
\qt@newcmd\qurantrans@de@cmxxviii{Und sie sagen: ``Was in den Leibern dieses Viehs ist, ist unseren Männern vorbehalten, unseren Gattinnen aber verboten." Wenn es aber tot (geboren) ist, so haben sie (alle) daran teil. Er wird ihnen ihre (ersonnene) Zuschreibung vergelten. Er ist ja Allweise und Allwissend.\qt@no{139}}%
\qt@newcmd\qurantrans@de@cmxxix{Verloren sind diejenigen, die ihre Kinder in Torheit und ohne Wissen töten und verbieten, womit Allah sie versorgt hat, indem sie Lügen gegen Allah ersinnen. Sie sind fürwahr irregegangen und nicht rechtgeleitet gewesen.\qt@no{140}}%
\qt@newcmd\qurantrans@de@cmxxx{Und Er ist es, Der Gärten mit Spalieren und ohne Spaliere entstehen läßt, sowie Palmen und (sonstige) Pflanzen mit unterschiedlichen Erntesorten und Öl- und Granatapfelbäume, die einander ähnlich und unähnlich sind. Eßt von ihren Früchten', wenn sie Früchte tragen, und entrichtet am Tag ihrer Ernte ihre(n) Pflicht(anteil, der darauf steht), doch seid nicht maßlos! - Er liebt nicht die Maßlosen.\qt@no{141}}%
\qt@newcmd\qurantrans@de@cmxxxi{Und an Vieh (läßt Allah entstehen) Lasttiere und Kleintiere. Eßt von dem, womit Allah euch versorgt hat, und folgt nicht den Fußstapfen des Satans! Er ist euch ein deutlicher Feind.\qt@no{142}}%
\qt@newcmd\qurantrans@de@cmxxxii{Acht (Tiere) zu Paaren (hat Er euch erschaffen): von den Schafen zwei und von den Ziegen zwei. - Sag: Sind es die (beiden) Männchen, die Er verboten hat, oder die (beiden) Weibchen, oder was der Mutterleib der (beiden) Weibchen umschließt? Tut (es) mir kund auf Grund von (wirklichem) Wissen, wenn ihr wahrhaftig seid!\qt@no{143}}%
\qt@newcmd\qurantrans@de@cmxxxiii{Und (auch) von den Kamelen zwei und von den Rindern zwei. - Sag: Sind es die (beiden) Männchen, die Er verboten hat, oder die (beiden) Weibchen, oder was der Mutterleib der (beiden) Weibchen umschließt? Oder wart ihr Zeugen, als Allah euch dies anbefahl? Wer ist denn ungerechter, als wer gegen Allah eine Lüge ersinnt, um die Menschen ohne Wissen in die Irre zu führen? Gewiß, Allah leitet das ungerechte Volk nicht recht.\qt@no{144}}%
\qt@newcmd\qurantrans@de@cmxxxiv{Sag: Ich finde in dem, was mir (als Offenbarung) eingegeben wurde, nichts, das für den Essenden zu essen verboten wäre, außer es ist Verendetes oder ausgeflossenes Blut oder Schweinefleisch - denn das ist ein Greuel - oder ein Frevel, worüber ein anderer (Name) als Allah(s) angerufen worden ist. Wer sich aber in einer Zwangslage befindet, ohne zu begehren oder das Maß zu überschreiten, - so ist dein Herr Allvergebend und Barmherzig.\qt@no{145}}%
\qt@newcmd\qurantrans@de@cmxxxv{Und denen, die dem Judentum angehören, haben Wir alles verboten, was Klauen besitzt; und von den Rindern und dem Kleinvieh haben Wir ihnen das Fett verboten, außer dem, was ihre Rücken und ihre Eingeweide tragen, oder was mit Knochen verwachsen ist. Damit haben Wir ihnen ihre Übertretung vergolten. Und Wir sagen ganz gewiß die Wahrheit.\qt@no{146}}%
\qt@newcmd\qurantrans@de@cmxxxvi{Wenn sie dich nun der Lüge bezichtigen, dann sag: Euer Herr ist voll umfassender Barmherzigkeit, aber vom übeltätigen Volk wird Seine Gewalt nicht abgewandt.\qt@no{147}}%
\qt@newcmd\qurantrans@de@cmxxxvii{Diejenigen, die (Ihm) beigesellen, werden sagen: ``Wenn Allah gewollt hätte, hätten wir (Ihm) nichts beigesellt, und (auch) nicht unsere Väter, und wir hätten nichts verboten." Ebenso haben diejenigen vor ihnen (ihre Gesandten) der Lüge bezichtigt, bis sie Unsere Gewalt kosteten. Sag: Habt ihr (irgendein) Wissen, das ihr uns vorbringen könnt? Ihr folgt ja nur Mutmaßungen, und ihr stellt nur Schätzungen an.\qt@no{148}}%
\qt@newcmd\qurantrans@de@cmxxxviii{Sag: Allah hat das überzeugende Beweismittel. Wenn Er gewollt hätte, hätte Er euch fürwahr allesamt rechtgeleitet.\qt@no{149}}%
\qt@newcmd\qurantrans@de@cmxxxix{Sag: Bringt eure Zeugen her, die bezeugen, daß Allah dies verboten hat! Wenn sie (es) dann bezeugen, so bezeuge (es) nicht mit ihnen. Und folge nicht den Neigungen derjenigen, die Unsere Zeichen für Lüge erklären und die nicht an das Jenseits glauben und ihrem Herrn (andere) gleichsetzen.\qt@no{150}}%
\qt@newcmd\qurantrans@de@cmxl{Sag: Kommt her! Ich will euch verlesen, was euer Herr euch verboten hat: Ihr sollt Ihm nichts beigesellen, und zu den Eltern gütig sein; und tötet nicht eure Kinder aus Armut - Wir versorgen euch und auch sie; und nähert euch nicht den Abscheulichkeiten, was von ihnen offen und was verborgen ist; und tötet nicht die Seele, die Allah verboten hat (zu töten), außer aus einem rechtmäßigen Grund! Dies hat Er euch anbefohlen, auf daß ihr begreifen möget.\qt@no{151}}%
\qt@newcmd\qurantrans@de@cmxli{Und nähert euch nicht dem Besitz des Waisenkindes, außer auf die beste Art, bis es seine Vollreife erlangt hat. Und gebt volles Maß und Gewicht in Gerechtigkeit. Wir erlegen keiner Seele mehr auf, als sie zu leisten vermag. Und wenn ihr euer Wort gebt, dann seid gerecht, auch wenn es um einen Verwandten geht. Und haltet euren Bund gegenüber Allah. Dies hat Er euch anbefohlen, auf daß ihr (es) bedenken möget!\qt@no{152}}%
\qt@newcmd\qurantrans@de@cmxlii{Und (Er hat euch anbefohlen:) Dies ist Mein Weg, ein gerader. So folgt ihm! Und folgt nicht den (anderen) Wegen, damit sie euch nicht von Seinem Weg auseinanderführen! Dies hat Er euch anbefohlen, auf daß ihr gottesfürchtig werden möget!\qt@no{153}}%
\qt@newcmd\qurantrans@de@cmxliii{Hierauf gaben Wir Musa die Schrift als Vollendung (und Belohnung) für das, was er an Gutem getan hatte', als eine ausführliche Darlegung für alles und als Rechtleitung und Barmherzigkeit, auf daß sie an die Begegnung mit ihrem Herrn glauben mögen!\qt@no{154}}%
\qt@newcmd\qurantrans@de@cmxliv{Und dies ist ein Buch, das Wir (als Offenbarung) hinabgesandt haben, ein gesegnetes (Buch). So folgt ihm und seid gottesfürchtig, auf daß ihr Erbarmen finden möget!\qt@no{155}}%
\qt@newcmd\qurantrans@de@cmxlv{Damit ihr (nicht) sagt: ``Die Schrift wurde nur auf zwei Gruppen vor uns herabgesandt, und wir waren gegenüber dem, was sie erlernt haben, wahrlich unachtsam."\qt@no{156}}%
\qt@newcmd\qurantrans@de@cmxlvi{Oder damit ihr (nicht) sagt: ``Wenn die Schrift auf uns herabgesandt worden wäre, wären wir wahrlich eher rechtgeleitet als sie." Aber nun ist ein klarer Beweis von eurem Herrn und Rechtleitung und Barmherzigkeit zu euch gekommen. Wer ist denn ungerechter, als wer Allahs Zeichen für Lüge erklärt und sich von ihnen abwendet? Wir werden denjenigen, die sich von Unseren Zeichen abwenden, mit einer bösen Strafe vergelten, daß sie sich stets abwandten.\qt@no{157}}%
\qt@newcmd\qurantrans@de@cmxlvii{Erwarten sie (etwas anderes), als daß die Engel zu ihnen kommen oder daß dein Herr kommt oder daß etwas von den Zeichen deines Herrn kommt? An dem Tag, da etwas von den Zeichen deines Herrn kommt, nützt einer Seele ihr Glaube nicht, wenn sie nicht schon zuvor geglaubt oder in ihrem Glauben Gutes erworben hat. Sag: Wartet ab! Wir warten ebenfalls ab.\qt@no{158}}%
\qt@newcmd\qurantrans@de@cmxlviii{Gewiß, mit denjenigen, die ihre Religion spalteten und zu Lagern geworden sind, hast du nichts gemein. Ihre Angelegenheit steht (allein) bei Allah. Hierauf wird Er ihnen kundtun, was sie zu tun pflegten.\qt@no{159}}%
\qt@newcmd\qurantrans@de@cmxlix{Wer mit (etwas) Gutem kommt, erhält zehnmal soviel. Und Wer mit einer bösen Tat kommt, dem wird nur gleichviel vergolten, und es wird ihnen kein Unrecht zugefügt.\qt@no{160}}%
\qt@newcmd\qurantrans@de@cml{Sag: Gewiß, mich hat mein Herr zu einem geraden Weg geleitet, einer richtigen Religion, dem Glaubensbekenntnis Ibrahims, als Anhänger des rechten Glaubens, und er war keiner der Götzendiener.\qt@no{161}}%
\qt@newcmd\qurantrans@de@cmli{Sag: Gewiß, mein Gebet und mein (Schlacht)opfer, mein Leben und mein Sterben gehören Allah, dem Herrn der Weltenbewohner.\qt@no{162}}%
\qt@newcmd\qurantrans@de@cmlii{Er hat keinen Teilhaber. Dies ist mir befohlen worden, und ich bin der erste der (Ihm) Ergebenen.\qt@no{163}}%
\qt@newcmd\qurantrans@de@cmliii{Sag: Soll ich einen anderen Herrn als Allah begehren, wo Er doch der Herr von allem ist? Jede Seele erwirbt nur gegen sich selbst. Und keine lasttragende (Seele) nimmt die Last einer anderen auf sich. Hierauf wird eure Rückkehr zu eurem Herrn sein, und Er wird kundtun, worüber ihr uneinig zu sein pflegtet.\qt@no{164}}%
\qt@newcmd\qurantrans@de@cmliv{Er ist es, Der euch zu Nachfolgern (auf) der Erde gemacht und die einen von euch über die anderen um Rangstufen erhöht hat, damit Er euch mit dem, was Er euch gegeben hat, prüfe. Gewiß, dein Herr ist schnell im Bestrafen, aber Er ist auch wahrlich Allvergebend und Barmherzig.\qt@no{165}}%
\qt@newcmd\qurantrans@de@cmlv{\basmalahde Alif-Lam-Mim-Sad\qt@no{1}}%
\qt@newcmd\qurantrans@de@cmlvi{Dies ist) ein Buch, das zu dir (als Offenbarung) herabgesandt worden ist - es soll seinetwegen in deiner Brust keine Bedrängnis sein -, damit du mit ihm warnst, und als Ermahnung für die Gläubigen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@cmlvii{Folgt dem, was zu euch von eurem Herrn herabgesandt worden ist, und folgt außer Ihm keinen (anderen) Schutzherren! Wie wenig ihr bedenkt!\qt@no{3}}%
\qt@newcmd\qurantrans@de@cmlviii{Wie viele Städte haben Wir vernichtet! Da kam Unsere Gewalt über sie bei Nacht, oder während sie zu Mittag ruhten.\qt@no{4}}%
\qt@newcmd\qurantrans@de@cmlix{So war ihr Ausruf, als Unsere Gewalt über sie kam, nur, daß sie sagten: ``Wir haben gewiß Unrecht getan."\qt@no{5}}%
\qt@newcmd\qurantrans@de@cmlx{Ganz gewiß werden Wir diejenigen, zu denen Gesandte gesandt worden sind, fragen, und ganz gewiß werden Wir die Gesandten fragen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@cmlxi{Dann werden Wir ihnen ganz gewiß aus (vollem) Wissen berichten, denn Wir waren ja nicht abwesend.\qt@no{7}}%
\qt@newcmd\qurantrans@de@cmlxii{Das Wägen erfolgt an jenem Tag der Wahrheit (entsprechend). Wessen Waagschalen schwer sein werden, jene sind es, denen es wohl ergeht.\qt@no{8}}%
\qt@newcmd\qurantrans@de@cmlxiii{Wessen Waagschalen aber leicht sein werden, das sind diejenigen, die ihre Seelen verloren haben dafür, daß sie an Unseren Zeichen unrecht zu handeln pflegten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@cmlxiv{Und Wir haben euch ja auf der Erde eine feste Stellung verliehen, und Wir haben für euch auf ihr (Möglichkeiten für den) Lebensunterhalt geschaffen. Wie wenig ihr dankbar seid!\qt@no{10}}%
\qt@newcmd\qurantrans@de@cmlxv{Und Wir haben euch ja erschaffen. Hierauf haben Wir euch gestaltet. Hierauf haben Wir zu den Engeln gesagt: ``Werft euch vor Adam nieder!" Da warfen sie sich nieder, außer Iblis. Er gehörte nicht zu denjenigen, die sich niederwerfen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@cmlxvi{Er (Allah) sagte: ``Was hat dich davon abgehalten, dich niederzuwerfen, als Ich (es) dir befahl?" Er sagte: ``Ich bin besser als er. Du hast mich aus Feuer erschaffen, ihn aber hast Du aus Lehm erschaffen."\qt@no{12}}%
\qt@newcmd\qurantrans@de@cmlxvii{Er (Allah) sagte: ``So geh fort von ihm! Es steht dir nicht zu, darin hochmütig zu sein. So geh hinaus! Gewiß, du gehörst zu den Geringgeachteten."\qt@no{13}}%
\qt@newcmd\qurantrans@de@cmlxviii{Er sagte: ``Gewähre mir Aufschub bis zu dem Tag, da sie auferweckt werden."\qt@no{14}}%
\qt@newcmd\qurantrans@de@cmlxix{Er (Allah) sagte: ``Du sollst gewiß zu denjenigen gehören, denen Aufschub gewährt wird."\qt@no{15}}%
\qt@newcmd\qurantrans@de@cmlxx{Er sagte: ``Darum, daß Du mich in Verirrung hast fallen lassen, werde ich ihnen ganz gewiß auf Deinem geraden Weg auflauern.\qt@no{16}}%
\qt@newcmd\qurantrans@de@cmlxxi{Hierauf werde ich ganz gewiß von vorn und von hinten, von ihrer Rechten und von ihrer Linken über sie kommen. Und Du wirst die meisten von ihnen nicht dankbar finden."\qt@no{17}}%
\qt@newcmd\qurantrans@de@cmlxxii{Er (Allah) sagte: ``Geh hinaus aus ihm, verachtet und verstoßen! Wer auch immer von ihnen dir folgt, Ich werde die Hölle ganz gewiß mit euch allesamt füllen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@cmlxxiii{- Und (du,) o Adam, bewohne du und deine Gattin den (Paradies)garten, und dann eßt, wo immer ihr wollt. Aber naht euch nicht diesem Baum, sonst gehört ihr zu den Ungerechten!"\qt@no{19}}%
\qt@newcmd\qurantrans@de@cmlxxiv{Da flüsterte ihnen der Satan ein, um ihnen offen zu zeigen, was ihnen von ihrer Blöße verborgen war. Und er sagte: ``Unser Herr hat euch diesen Baum nur verboten, damit ihr nicht Engel werdet oder zu den Ewiglebenden gehört."\qt@no{20}}%
\qt@newcmd\qurantrans@de@cmlxxv{Und er schwor ihnen: ``Ich gehöre wahrlich zu denjenigen, die euch guten Rat geben."\qt@no{21}}%
\qt@newcmd\qurantrans@de@cmlxxvi{So verführte er sie trügerisch. Als sie dann von dem Baum gekostet hatten, zeigte sich ihnen ihre Blöße offenkundig, und sie begannen, Blätter des (Paradies)gartens auf sich zusammenzuheften. Und ihr Herr rief ihnen zu: ``Habe Ich euch nicht jenen Baum verboten und euch gesagt: Der Satan ist euch ein deutlicher Feind?"\qt@no{22}}%
\qt@newcmd\qurantrans@de@cmlxxvii{Sie sagten: ``Unser Herr, wir haben uns selbst Unrecht zugefügt. Wenn Du uns nicht vergibst und Dich unser erbarmst, werden wir ganz gewiß zu den Verlorenen gehören."\qt@no{23}}%
\qt@newcmd\qurantrans@de@cmlxxviii{Er (Allah) sagte: ``Geht fort! Einer sei des anderen Feind. Und auf der Erde sollt ihr Aufenthalt und Nießbrauch auf Zeit haben."\qt@no{24}}%
\qt@newcmd\qurantrans@de@cmlxxix{Er (Allah) sagte: ``Auf ihr werdet ihr leben, und auf ihr werdet ihr sterben, und aus ihr werdet ihr hervorgebracht werden."\qt@no{25}}%
\qt@newcmd\qurantrans@de@cmlxxx{O Kinder Adams, Wir haben auf euch Kleidung hinabgesandt, die eure Blöße verbirgt, und Gefieder. Aber die Kleidung der Gottesfurcht, die ist besser. Das ist (eines) von Allahs Zeichen, auf daß sie bedenken mögen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@cmlxxxi{O Kinder Adams, der Satan soll euch ja nicht der Versuchung aussetzen, wie er eure (Stamm)eltern aus dem (Paradies)garten vertrieben hat, indem er ihnen ihre Kleidung wegnahm, um ihnen ihre Blöße zu zeigen. Gewiß, er sieht euch, er und sein Stamm, von wo ihr sie nicht seht. Gewiß, Wir haben die Satane zu Schutzherren für diejenigen gemacht, die nicht glauben.\qt@no{27}}%
\qt@newcmd\qurantrans@de@cmlxxxii{Und wenn sie eine Abscheulichkeit begehen, sagen sie: ``Wir haben unsere Väter darin (vorgefunden, und Allah hat es uns geboten." Sag: Allah gebietet nicht Schändliches. Wollt ihr (denn) über Allah sagen, was ihr nicht wißt?\qt@no{28}}%
\qt@newcmd\qurantrans@de@cmlxxxiii{Sag: Mein Herr hat Gerechtigkeit geboten, und ihr sollt euer Gesicht bei jeder Gebetsstätte aufrichten und Ihn anrufen, (wobei ihr) Ihm gegenüber aufrichtig in der Religion (sein sollt). So wie Er euch anfangs gebracht hat, werdet ihr zurückkehren.\qt@no{29}}%
\qt@newcmd\qurantrans@de@cmlxxxiv{Einen Teil hat Er rechtgeleitet, an einem (anderen) Teil aber hat sich das Irregehen bewahrheitet, denn sie haben sich die Satane anstatt Allahs zu Schutzherren genommen und meinen, sie seien rechtgeleitet.\qt@no{30}}%
\qt@newcmd\qurantrans@de@cmlxxxv{O Kinder Adams, legt euren Schmuck bei jeder Gebetsstätte an und eßt und trinkt, aber seid nicht maßlos! - Er (Allah) liebt nicht die Maßlosen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@cmlxxxvi{Sag: Wer hat den Schmuck Allahs verboten, den Er für Seine Diener hervorgebracht hat, und (auch) die guten Dinge (aus) der Versorgung (Allahs)? Sag: Sie sind im diesseitigen Leben für diejenigen (bestimmt), die glauben, und am Tag der Auferstehung (ihnen) vorbehalten. So legen Wir die Zeichen ausführlich dar für Leute, die Bescheid wissen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@cmlxxxvii{Sag: Mein Herr hat nur die Abscheulichkeiten verboten, was von ihnen offen und was verborgen ist; und (auch) die Sünde und die Gewalttätigkeit ohne Recht, und, daß ihr Allah (etwas) beigesellt, wofür Er keine Ermächtigung herabgesandt hat, und daß ihr über Allah (etwas) sagt, was ihr nicht wißt.\qt@no{33}}%
\qt@newcmd\qurantrans@de@cmlxxxviii{Jede Gemeinschaft hat eine (festgesetzte) Frist. Und wenn nun ihre Frist kommt, können sie (sie) weder um eine Stunde hinausschieben noch sie vorverlegen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@cmlxxxix{O Kinder Adams, wenn immer Gesandte von euch (selbst) zu euch kommen, die euch Meine Zeichen berichten - wer dann gottesfürchtig ist und Besserung bringt, über die soll keine Furcht kommen, noch sollen sie traurig sein.\qt@no{35}}%
\qt@newcmd\qurantrans@de@cmxc{Diejenigen aber, die Unsere Zeichen für Lüge erklären und sich ihnen gegenüber hochmütig verhalten, das sind Insassen des (Höllen)feuers. Ewig werden sie darin bleiben.\qt@no{36}}%
\qt@newcmd\qurantrans@de@cmxci{Wer ist denn ungerechter, als wer gegen Allah eine Lüge ersinnt oder Seine Zeichen für Lüge erklärt? Jene wird ihr Anteil vom Buch erreichen, bis daß, wenn Unsere Gesandten dann zu ihnen kommen, um sie abzuberufen, sie sagen: ``Wo ist das, was ihr außer Allah anzurufen pflegtet?" Sie werden sagen: ``Sie sind uns entschwunden." Und sie zeugen gegen sich selbst, daß sie ungläubig waren.\qt@no{37}}%
\qt@newcmd\qurantrans@de@cmxcii{Er (Allah) sagt: ``Tretet ein unter Gemeinschaften von den Ginn und den Menschen, die schon vor euch dahingegangen sind, ins (Höllen)feuer." Jedesmal, wenn eine Gemeinschaft eintritt, verflucht sie die vorhergehende'. Wenn sie dann alle darin eingeholt werden, sagt die letzte von ihnen in Bezug auf die erste: ``Unser Herr, diese da haben uns in die Irre geführt. So gib ihnen eine mehrfache Strafe vom (Höllen)feuer!" Er sagt: ``Jeder bekommt das Mehrfache, aber ihr wißt (es) nicht."\qt@no{38}}%
\qt@newcmd\qurantrans@de@cmxciii{Und die erste von ihnen sagt zu der letzten: ``Ihr habt doch keinen Vorzug gegenüber uns. So kostet die Strafe für das, was ihr verdient habt."\qt@no{39}}%
\qt@newcmd\qurantrans@de@cmxciv{Gewiß, denjenigen, die Unsere Zeichen für Lüge erklären und sich ihnen gegenüber hochmütig verhalten, werden die Tore des Himmels nicht geöffnet, und sie werden nicht in den (Paradies)garten eingehen, bis ein Kamel durch ein Nadelöhr geht. So vergelten Wir den Übeltätern.\qt@no{40}}%
\qt@newcmd\qurantrans@de@cmxcv{Für sie wird die Hölle eine Lagerstatt sein, und über ihnen werden Decken (aus Höllenfeuer) sein. So vergelten Wir den Ungerechten.\qt@no{41}}%
\qt@newcmd\qurantrans@de@cmxcvi{Diejenigen aber, die glauben und rechtschaffene Werke tun - Wir erlegen keiner Seele mehr auf, als sie zu leisten vermag -, jene sind Insassen des (Paradies)gartens. Ewig werden sie darin bleiben.\qt@no{42}}%
\qt@newcmd\qurantrans@de@cmxcvii{Und Wir nehmen weg, was in ihren Brüsten an Groll ist. Unter ihnen strömen Flüsse. Und sie sagen: ``(Alles) Lob gehört Allah, Der uns hierher geleitet hat! Wir hätten unmöglich die Rechtleitung gefunden, wenn uns Allah nicht rechtgeleitet hätte. Die Gesandten unseres Herrn sind wirklich mit der Wahrheit gekommen." Und es wird ihnen zugerufen: ``Siehe, das ist der (Paradies)garten. Er ist euch zum Erbe gegeben worden für das, was ihr zu tun pflegtet."\qt@no{43}}%
\qt@newcmd\qurantrans@de@cmxcviii{Die Insassen des (Paradies)gartens rufen den Insassen des (Höllen)feuers zu: ``Wir haben gefunden, daß das, was uns unser Herr versprochen hat, wahr ist. Habt (auch) ihr gefunden, daß das, was euer Herr versprochen hat, wahr ist?" Sie sagen: ``Ja!" Und dann ruft ein Rufer unter ihnen aus: ``Allahs Fluch (kommt) über die Ungerechten,\qt@no{44}}%
\qt@newcmd\qurantrans@de@cmxcix{die von Allahs Weg abhalten und danach trachten, ihn krumm zu machen, und die das Jenseits verleugnen."\qt@no{45}}%
\qt@newcmd\qurantrans@de@m{Und zwischen ihnen ist ein Vorhang. Und auf den Höhen sind Männer, die jeden an seinem Merkmal erkennen. Sie rufen den Insassen des (Paradies)gartens zu: ``Friede sei auf euch!" Sie sind aber in ihn nicht eingetreten, und sie begehren (es) doch.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mi{Und wenn ihre Blicke den Insassen des (Höllen)feuers zugewendet werden, sagen sie: ``Unser Herr, lasse uns nicht mit dem ungerechten Volk sein!"\qt@no{47}}%
\qt@newcmd\qurantrans@de@mii{Und die Leute der Höhen rufen Männern, die sie an ihren Merkmalen erkennen, zu. Sie sagen: ``Es hat euch eure Ansammlung, und daß ihr euch hochmütig zu verhalten pflegtet, nicht genützt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@miii{Sind das diejenigen, von denen ihr geschworen habt, Allah werde ihnen keine Barmherzigkeit erweisen? - Geht in den (Paradies)garten ein! Über euch soll keine Furcht kommen, noch sollt ihr traurig sein."\qt@no{49}}%
\qt@newcmd\qurantrans@de@miv{Und die Insassen des (Höllen)feuers rufen den Insassen des (Paradies)gartens zu: ``Schüttet auf uns etwas Wasser aus oder etwas von dem, womit Allah euch versorgt hat!" Sie sagen: ``Gewiß, Allah hat beides für die Ungläubigen verboten,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mv{die ihre Religion zum Gegenstand der Zerstreuung und des Spiels genommen haben und die das diesseitige Leben getäuscht hat." Heute werden Wir sie vergessen, so wie sie die Begegnung mit diesem ihrem Tag vergaßen und wie sie Unsere Zeichen zu verleugnen pflegten.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mvi{Wir haben ihnen ja ein Buch gebracht, das Wir mit Wissen ausführlich dargelegt haben, als Rechtleitung und Barmherzigkeit für Leute, die glauben.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mvii{Erwarten sie (etwas anderes) als seine Deutung? An dem Tag, da seine Deutung eintrifft, werden diejenigen, die es zuvor vergessen haben, sagen: ``Die Gesandten unseres Herrn sind wirklich mit der Wahrheit gekommen! Haben wir denn Fürsprecher, die für uns Fürsprache einlegen? Oder können wir zurückgebracht werden, daß wir anders handeln, als wir zu handeln pflegten?" Sie haben wirklich ihre Seelen verloren, und ihnen ist entschwunden, was sie zu ersinnen pflegten.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mviii{Gewiß, euer Herr ist Allah, Der die Himmel und die Erde in sechs Tagen erschuf und Sich hierauf über den Thron erhob. Er läßt die Nacht den Tag überdecken, wobei sie ihn eilig einzuholen sucht. Und (Er schuf auch) die Sonne, den Mond und die Sterne, durch Seinen Befehl dienstbar gemacht. Sicherlich, Sein ist die Schöpfung und der Befehl. Segensreich ist Allah, der Herr der Weltenbewohner.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mix{Ruft euren Herrn in Unterwürfigkeit flehend und im Verborgenen an. Gewiß, Er liebt nicht die Übertreter.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mx{Und stiftet auf der Erde nicht Unheil, nachdem sie in Ordnung gebracht worden ist! Und ruft Ihn in Furcht und Begehren an. Gewiß, die Barmherzigkeit Allahs ist den Gutes Tuenden nahe.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mxi{Und Er ist es, Der die Winde als Frohboten Seiner Barmherzigkeit (voraus)sendet, bis daß, wenn sie dann schwere Wolken herbeitragen, Wir sie zu einem toten Land treiben, dadurch Wasser hinabsenden und dann dadurch bedenken möget! alle Früchte hervorbringen. Ebenso bringen Wir (dereinst) die Toten hervor, auf daß ihr bedenken möget!\qt@no{57}}%
\qt@newcmd\qurantrans@de@mxii{Und aus dem guten Land kommt sein Pflanzenwuchs mit Allahs Erlaubnis hervor, aus dem schlechten (Land) jedoch kommt (nur) Kümmerliches hervor. So legen Wir die Zeichen verschiedenartig dar für Leute, die dankbar sind.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mxiii{Wir sandten doch bereits Nuh zu seinem Volk, und da sagte er: ``O mein Volk, dient Allah! Keinen Gott habt ihr außer Ihm. Gewiß, ich fürchte für euch die Strafe eines gewaltigen Tages."\qt@no{59}}%
\qt@newcmd\qurantrans@de@mxiv{Die führende Schar aus seinem Volk sagte: ``Wir sehen dich wahrlich in deutlichem Irrtum."\qt@no{60}}%
\qt@newcmd\qurantrans@de@mxv{Er sagte: ``O mein Volk, bei mir befindet sich kein Irrtum, sondern ich bin ein Gesandter vom Herrn der Weltenbewohner.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mxvi{Ich übermittele euch die Botschaften meines Herrn und rate euch gut. Und ich weiß von Allah her, was ihr nicht wißt.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mxvii{Wundert ihr euch etwa (darüber), daß eine Erinnerung von eurem Herrn zu euch gekommen ist durch einen Mann von euch, damit er euch warne und damit ihr gottesfürchtig werdet, auf daß ihr Erbarmen finden möget?"\qt@no{63}}%
\qt@newcmd\qurantrans@de@mxviii{Sie aber bezichtigten ihn der Lüge. Da retteten Wir ihn und diejenigen, die mit ihm waren, im Schiff und ließen diejenigen ertrinken, die Unsere Zeichen für Lüge erklärten. Sie waren gewiß blinde Leute.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mxix{Und (Wir sandten) zu ``Ad ihren Bruder Hud. Er sagte: ``O mein Volk, dient Allah! Keinen Gott habt ihr außer Ihm. Wollt ihr denn nicht gottesfürchtig sein?"\qt@no{65}}%
\qt@newcmd\qurantrans@de@mxx{Die führende Schar aus seinem Volk, die ungläubig war, sagte: ``Wir sehen dich wahrlich in Torheit (befangen), und wir meinen wahrlich, daß du zu den Lügnern gehörst."\qt@no{66}}%
\qt@newcmd\qurantrans@de@mxxi{Er sagte: ``O mein Volk, bei mir befindet sich keine Torheit, sondern ich bin ein Gesandter vom Herrn der Weltenbewohner.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mxxii{Ich übermittele euch die Botschaften meines Herrn, und ich bin für euch ein vertrauenswürdiger Ratgeber.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mxxiii{Wundert ihr euch etwa (darüber), daß eine Erinnerung von eurem Herrn zu euch gekommen ist durch einen Mann von euch, damit er euch warne? Gedenkt, als Er euch zu Nachfolgern nach dem Volk Nuhs machte und euch ein Übermaß an körperlichen Vorzügen gab! Gedenkt also der Wohltaten Allahs, auf daß es euch wohl ergehen möge!"\qt@no{69}}%
\qt@newcmd\qurantrans@de@mxxiv{Sie sagten: ``Bist du zu uns gekommen, damit wir Allah allein dienen und das verlassen, dem unsere Väter immer gedient haben? Bring uns doch her, was du uns androhst, wenn du zu den Wahrhaftigen gehören solltest!"\qt@no{70}}%
\qt@newcmd\qurantrans@de@mxxv{Er sagte: ``Greuel und Zorn von eurem Herrn überfallen euch nunmehr. (Wollt) ihr denn mit mir über Namen streiten, die ihr genannt habt, ihr und eure Väter, für die Allah (aber) keine Ermächtigung offenbart hat? So wartet ab! Ich gehöre mit euch zu den Abwartenden."\qt@no{71}}%
\qt@newcmd\qurantrans@de@mxxvi{Da retteten Wir ihn und diejenigen, die mit ihm waren, durch Barmherzigkeit von Uns. Aber Wir schnitten die Rückkehr derer, die Unsere Zeichen für Lüge erklärten, ab, und sie waren nicht gläubig."\qt@no{72}}%
\qt@newcmd\qurantrans@de@mxxvii{Und (Wir sandten) zu Tamud ihren Bruder Salih. Er sagte: ``O mein Volk, dient Allah! Keinen Gott habt ihr außer Ihm. Nun ist ein klarer Beweis von eurem Herrn zu euch gekommen: Dies ist die Kamelstute Allahs, euch zum Zeichen. So laßt sie auf Allahs Erde fressen und fügt ihr nichts Böses zu, sonst überkommt euch schmerzhafte Strafe.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mxxviii{Und gedenkt, als Er euch zu Nachfolgern nach den 'Ad machte und euch auf der Erde (Stätten) zuwies, so daß ihr euch in ihrem Flachland Schlösser nahmt und die Berge zu Häusern aushautet. Gedenkt also der Wohltaten Allahs und richtet auf der Erde nicht unheilstiftend Verderben an!"\qt@no{74}}%
\qt@newcmd\qurantrans@de@mxxix{Die führende Schar, die sich hochmütig verhielt, aus seinem Volk, sagte zu denjenigen, die unterdrückt waren, (und zwar) zu denen von ihnen, die gläubig waren: ``Wißt ihr (sicher), daß Salih von seinem Herrn gesandt ist?" Sie sagten: ``Wir glauben gewiß an das, womit er gesandt worden ist."\qt@no{75}}%
\qt@newcmd\qurantrans@de@mxxx{Diejenigen, die sich hochmütig verhielten, sagten: ``Wir verleugnen ja das, woran ihr glaubt."\qt@no{76}}%
\qt@newcmd\qurantrans@de@mxxxi{Dann schnitten sie der Kamelstute die Sehnen durch und lehnten sich gegen den Befehl ihres Herrn auf. Und sie sagten: ``O Salih, bringe uns doch her, was du uns angedroht hast, wenn du zu den Gesandten gehörst!"\qt@no{77}}%
\qt@newcmd\qurantrans@de@mxxxii{Da ergriff sie das Zittern, und am Morgen lagen sie in ihrer Wohnstätte auf den Brüsten da.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mxxxiii{So kehrte er sich von ihnen ab und sagte: ``O mein Volk, ich habe euch doch die Botschaft meines Herrn ausgerichtet und euch gut geraten. Aber ihr liebt nicht die guten Ratgeber."\qt@no{79}}%
\qt@newcmd\qurantrans@de@mxxxiv{Und (Wir sandten) Lut, als er zu seinem Volk sagte: ``Wollt ihr denn das Abscheuliche begehen, wie es vor euch niemand von den Weltenbewohnern begangen hat?\qt@no{80}}%
\qt@newcmd\qurantrans@de@mxxxv{Ihr laßt euch doch wahrlich in Begierde mit den Männern ein anstatt mit den Frauen. Aber nein! Ihr seid maßlose Leute.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mxxxvi{Die Antwort seines Volkes war nur, daß sie sagten: ``Vertreibt sie aus eurer Stadt! Das sind Menschen, die sich rein halten."\qt@no{82}}%
\qt@newcmd\qurantrans@de@mxxxvii{Da retteten Wir ihn und seine Angehörigen, außer seiner Frau; Sie gehörte zu denjenigen, die zurückblieben.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mxxxviii{Und Wir ließen einen Regen auf sie niedergehen. So schau, wie das Ende der Übeltäter war!\qt@no{84}}%
\qt@newcmd\qurantrans@de@mxxxix{Und (Wir sandten) zu Madyan ihren Bruder Su'aib. Er sagte: ``O mein Volk, dient Allah! Keinen Gott habt ihr außer Ihm. Nun ist ein klarer Beweis von eurem Herrn zu euch gekommen; so gebt volles Maß und Gewicht und schmälert den Menschen nicht ihre Sachen und stiftet auf der Erde nicht Unheil, nachdem sie in Ordnung gebracht worden ist! Das ist besser für euch, wenn ihr gläubig seid.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mxl{Und lauert nicht auf jedem Weg, indem ihr droht und von Allahs Weg denjenigen abhaltet, der an Ihn glaubt, und danach trachtet, ihn krumm zu machen. Und gedenkt (der Zeit), als ihr (nur) wenige wart und Er euch dann zu vielen machte. Und schaut, wie das Ende der Unheilstifter war!\qt@no{86}}%
\qt@newcmd\qurantrans@de@mxli{Und wenn ein Teil von euch an das glauben sollte, womit ich gesandt worden bin, ein anderer Teil aber nicht glauben sollte, so geduldet euch, bis Allah zwischen uns richtet. Er ist der Beste derer, die richten."\qt@no{87}}%
\qt@newcmd\qurantrans@de@mxlii{Die führende Schar, die sich hochmütig verhielt, aus seinem Volk, sagte: ``Wir werden dich, o Su'aib, und diejenigen, die mit dir glauben, ganz gewiß aus unserer Stadt vertreiben, oder aber ihr kehrt zu unserem Glaubensbekenntnis zurück!" Er sagte: ``Was denn, auch wenn es uns zuwider ist?\qt@no{88}}%
\qt@newcmd\qurantrans@de@mxliii{Wir würden ja gegen Allah eine Lüge ersinnen, wenn wir zu eurem Glaubensbekenntnis zurückkehrten, nachdem Allah uns vor ihr errettet hat. Es steht uns nicht zu, zu ihr zurückzukehren, außer daß Allah, unser Herr, (es) wollte. Unser Herr umfaßt alles mit (Seinem) Wissen. Auf Allah verlassen wir uns. Unser Herr, entscheide zwischen uns und unserem Volk der Wahrheit entsprechend! Du bist ja der beste Entscheider."\qt@no{89}}%
\qt@newcmd\qurantrans@de@mxliv{Aber die führende Schar aus seinem Volk, die ungläubig war, sagte: ``Wenn ihr Su'aib folgt, dann werdet ihr fürwahr Verlierer sein."\qt@no{90}}%
\qt@newcmd\qurantrans@de@mxlv{Da ergriff sie das Zittern, und am Morgen lagen sie in ihrer Wohnstätte auf den Brüsten da.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mxlvi{Diejenigen, die Su'aib der Lüge bezichtigen, waren, als hätten sie (überhaupt) nicht darin gewohnt. Diejenigen, die Su'aib der Lüge bezichtigten, waren (nun) selbst die Verlierer.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mxlvii{So kehrte er sich von ihnen ab und sagte: ``O mein Volk, ich habe euch doch die Botschaften meines Herrn ausgerichtet und euch gut geraten. Wie sollte ich betrübt sein über ungläubige Leute?"\qt@no{93}}%
\qt@newcmd\qurantrans@de@mxlviii{Und Wir haben in keine Stadt Propheten gesandt, ohne daß Wir über ihre Bewohner Not und Leid hätten kommen lassen, auf daß sie in Unterwürfigkeit flehen mögen.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mxlix{Hierauf tauschten Wir anstelle des Bösen Gutes ein, bis sie (sich und ihren Besitz) vermehrten und sagten: ``Schon unseren Vätern ist Leid und (auch) Freude widerfahren." Da ergriffen Wir sie plötzlich, ohne daß sie merkten.\qt@no{95}}%
\qt@newcmd\qurantrans@de@ml{Hätten aber die Bewohner der Städte geglaubt und wären sie gottesfürchtig gewesen, hätten Wir ihnen bestimmt Segnungen von dem Himmel und der Erde aufgetan. Aber sie erklärten (die Botschaft) für Lüge, und so ergriffen Wir sie für das, was sie erworben hatten.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mli{Glauben denn die Bewohner der Städte, sicher davor zu sein, daß Unsere Gewalt bei Nacht über sie kommt, während sie schlafen?\qt@no{97}}%
\qt@newcmd\qurantrans@de@mlii{Oder glauben die Bewohner der Städte, sicher davor zu sein, daß Unsere Gewalt am hellen Morgen über sie kommt, während sie spielen?\qt@no{98}}%
\qt@newcmd\qurantrans@de@mliii{Glauben sie denn, sicher vor Allahs Ränken zu sein? Aber vor Allahs Ränken sicher zu sein, glaubt nur das Volk derjenigen, die Verlierer sind.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mliv{Ist denjenigen, die die Erde nach ihren (vorherigen) Bewohnern erben, nicht deutlich geworden, daß, wenn Wir wollten, Wir sie für ihre Sünden treffen würden? Und Wir versiegeln ihre Herzen, so daß sie nicht hören.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mlv{Das sind die Städte. Über sie erzählen Wir dir (einiges) von ihren Berichten. Ihre Gesandten kamen ja zu ihnen mit den klaren Beweisen, aber sie konnten nicht an das glauben, was sie zuvor für Lüge erklärt hatten. So versiegelt Allah die Herzen der Ungläubigen.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mlvi{Und Wir fanden bei den meisten von ihnen keine Vertragstreue, sondern Wir fanden (, daß) die meisten von ihnen fürwahr Frevler (waren).\qt@no{102}}%
\qt@newcmd\qurantrans@de@mlvii{Hierauf schickten Wir nach ihnen Musa mit Unseren Zeichen zu Fir'aun und seiner führenden Schar, doch sie handelten unrecht an ihnen. Schau nur, wie das Ende der Unheilstifter war!\qt@no{103}}%
\qt@newcmd\qurantrans@de@mlviii{Musa sagte: ``O Fir'aun, gewiß, ich bin ein Gesandter vom Herrn der Weltenbewohner,\qt@no{104}}%
\qt@newcmd\qurantrans@de@mlix{bestrebt', über Allah nur die Wahrheit zu sagen. Ich bin doch mit einem klaren Beweis von eurem Herrn zu euch gekommen. So lasse die Kinder Isra'ils mit mir gehen!"\qt@no{105}}%
\qt@newcmd\qurantrans@de@mlx{Er sagte: ``Wenn du mit einem Zeichen gekommen bist, dann bringe es her, wenn du zu den Wahrhaftigen gehörst."\qt@no{106}}%
\qt@newcmd\qurantrans@de@mlxi{Er warf seinen Stock hin, und sogleich war er eine deutliche Schlange.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mlxii{Und er zog seine Hand heraus, da war sie weiß für die Betrachter.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mlxiii{Die führende Schar aus dem Volk Fir'auns sagte: ``Dieser ist fürwahr ein kenntnisreicher Zauberer,\qt@no{109}}%
\qt@newcmd\qurantrans@de@mlxiv{der euch aus eurem Land vertreiben will. Was befehlt ihr nun?"\qt@no{110}}%
\qt@newcmd\qurantrans@de@mlxv{Sie sagten: ``Stelle ihn und seinen Bruder zurück, und sende in die Städte (Boten), die (die Leute) versammeln,\qt@no{111}}%
\qt@newcmd\qurantrans@de@mlxvi{damit sie dir jeden kenntnisreichen Zauberer herbeibringen."\qt@no{112}}%
\qt@newcmd\qurantrans@de@mlxvii{Und die Zauberer kamen zu Fir'aun. Sie sagten: ``Wir bekommen doch sicher Lohn, wenn wir es sind, die siegen?"\qt@no{113}}%
\qt@newcmd\qurantrans@de@mlxviii{Er sagte: ``Ja. Und ihr werdet fürwahr zu den Nahegestellten gehören."\qt@no{114}}%
\qt@newcmd\qurantrans@de@mlxix{Sie sagten: ``O Musa, entweder wirfst du oder wir sind es, die (zuerst) werfen."\qt@no{115}}%
\qt@newcmd\qurantrans@de@mlxx{Er sagte: ``Werft (ihr zuerst)!" Als sie nun warfen, bezauberten sie die Augen der Menschen und flößten ihnen Furcht ein, und sie brachten einen gewaltigen Zauber vor.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mlxxi{Und Wir gaben Musa ein: ``Wirf deinen Stock hin!" Da verschlang er sogleich, was sie vortäuschten.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mlxxii{So bestätigte sich die Wahrheit, und zunichte wurde das, was sie taten.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mlxxiii{Sie wurden dort besiegt und kehrten gering geachtet zurück.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mlxxiv{Und die Zauberer warfen sich ehrerbietig nieder.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mlxxv{Sie sagten: ``Wir glauben an den Herrn der Weltenbewohner,\qt@no{121}}%
\qt@newcmd\qurantrans@de@mlxxvi{den Herrn von Musa und Harun."\qt@no{122}}%
\qt@newcmd\qurantrans@de@mlxxvii{Fir'aun sagte: ``Ihr glaubt an Ihn, bevor ich es euch erlaube? Das sind wahrlich Ränke, die ihr in der Stadt geschmiedet habt, um ihre Bewohner daraus zu vertreiben. Aber ihr werdet (es noch) erfahren.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mlxxviii{Ich werde ganz gewiß eure Hände und eure Füße wechselseitig abhacken und euch hierauf ganz gewiß allesamt kreuzigen (lassen)."\qt@no{124}}%
\qt@newcmd\qurantrans@de@mlxxix{Sie sagten: ``Wir werden gewiß zu unserem Herrn zurückkehren!\qt@no{125}}%
\qt@newcmd\qurantrans@de@mlxxx{Du grollst uns ja nur (darum), daß wir an die Zeichen unseres Herrn glaubten, als sie zu uns kamen. Unser Herr, überschütte uns mit Standhaftigkeit und berufe uns ab als (Dir) Ergebene!"\qt@no{126}}%
\qt@newcmd\qurantrans@de@mlxxxi{Die führende Schar aus dem Volk Fir'auns sagte: ``Willst du zulassen, daß Musa und sein Volk auf der Erde Unheil stiften und daß er dich und deine Götter verläßt?" Er sagte: ``Wir werden ihre Söhne allesamt töten und (nur) ihre Frauen am Leben lassen. Wir sind ja Bezwinger über sie."\qt@no{127}}%
\qt@newcmd\qurantrans@de@mlxxxii{Musa sagte zu seinem Volk: ``Sucht Hilfe bei Allah, und seid standhaft! Gewiß, die Erde gehört Allah; Er gibt sie zum Erbe, wem von Seinen Dienern Er will. Und das (gute) Ende gehört den Gottesfürchtigen."\qt@no{128}}%
\qt@newcmd\qurantrans@de@mlxxxiii{Sie sagten: ``Uns ist Leid zugefügt worden, bevor du zu uns kamst und nachdem du zu uns gekommen bist." Er sagte: ``Vielleicht wird euer Herr euren Feind vernichten und euch zu Nachfolgern auf der Erde machen und dann schauen, wie ihr handelt."\qt@no{129}}%
\qt@newcmd\qurantrans@de@mlxxxiv{Und Wir ergriffen ja schon die Leute Fir'auns mit Dürrejahren und Mangel an Früchten, auf daß sie bedenken mögen.\qt@no{130}}%
\qt@newcmd\qurantrans@de@mlxxxv{Aber wenn dann etwas Gutes zu ihnen kam, sagten sie: ``Dies gehört uns." und wenn sie etwas Böses traf, sahen sie ein (böses) Vorzeichen in Musa und denjenigen, die mit ihm waren. Aber ihr Vorzeichen ist sicherlich bei Allah, jedoch wissen die meisten von ihnen nicht.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mlxxxvi{Und sie sagten: ``Welches Zeichen du uns auch vorbringen magst, um uns damit zu bezaubern, so werden wir dir nicht glauben."\qt@no{132}}%
\qt@newcmd\qurantrans@de@mlxxxvii{Da sandten Wir über sie die Überschwemmung, die Heuschrecken, die Zecken, die Frösche und das Blut als einzeln vorgebrachte Zeichen. Aber sie verhielten sich hochmütig und waren ein Volk von Übeltätern.\qt@no{133}}%
\qt@newcmd\qurantrans@de@mlxxxviii{Als nun die unheilvolle Strafe sie überfiel, sagten sie: ``O Musa, rufe für uns deinen Herrn an auf Grund Seines Bundes mit dir. Wenn du die unheilvolle Strafe hinwegnimmst, werden wir dir ganz gewiß glauben und die Kinder Isra'ils ganz gewiß mit dir wegschicken."\qt@no{134}}%
\qt@newcmd\qurantrans@de@mlxxxix{Als Wir dann die unheilvolle Strafe von ihnen hinwegnahmen auf eine Frist, die sie erreichen sollten, brachen sie sogleich (ihr Wort).\qt@no{135}}%
\qt@newcmd\qurantrans@de@mxc{Da übten Wir an ihnen Vergeltung und ließen sie im großen Gewässer ertrinken, dafür, daß sie Unsere Zeichen für Lüge erklärten und ihnen gegenüber unachtsam waren.\qt@no{136}}%
\qt@newcmd\qurantrans@de@mxci{Und Wir gaben dem Volk, das unterdrückt worden war, zum Erbe die östlichen und die westlichen (Gegenden) des Landes, das Wir gesegnet haben. Und das schönste Wort deines Herrn erfüllte sich an den Kindern Isra'ils dafür, daß sie standhaft waren. Und Wir zerstörten, was Fir'aun und sein Volk zu machen und was sie zu errichten pflegten.\qt@no{137}}%
\qt@newcmd\qurantrans@de@mxcii{Und Wir ließen die Kinder lsra'ils das Meer durchschreiten. Sie kamen zu Leuten, die sich zur Andacht an ihren Götzen zurückzogen. Sie sagten: ``O Musa, mache uns einen Gott, so wie sie Götter haben!" Er sagte: ``Ihr seid ja Leute, die töricht sind.\qt@no{138}}%
\qt@newcmd\qurantrans@de@mxciii{Gewiß, was diese da betreiben, wird zerstört, und zunichte wird, was sie zu tun pflegten."\qt@no{139}}%
\qt@newcmd\qurantrans@de@mxciv{Er sagte: ``Sollte ich für euch einen anderen Gott begehren als Allah, wo Er euch vor den (anderen) Weltenbewohnern bevorzugt hat?"\qt@no{140}}%
\qt@newcmd\qurantrans@de@mxcv{Und (gedenkt,) als Wir euch vor den Leuten Fir'auns retteten, als sie euch eine böse Qual auferlegten, indem sie eure Söhne allesamt töteten und (nur) eure Frauen am Leben ließen. Darin war für euch eine gewaltige Prüfung von eurem Herrn.\qt@no{141}}%
\qt@newcmd\qurantrans@de@mxcvi{Und Wir gaben Musa eine Verabredung auf dreißig Nächte und machten sie mit (weiteren) zehn voll. So vervollständigte sich die festgesetzte Zahl seines Herrn auf vierzig Nächte. Und Musa sagte zu seinem Bruder Harun: ``Nimm meine Stelle bei meinem Volk ein, sorge für Ordnung und folge nicht dem Weg der Unheilstifter!"\qt@no{142}}%
\qt@newcmd\qurantrans@de@mxcvii{Als nun Musa zu Unserer festgesetzten Zeit kam und sein Herr zu ihm sprach, sagte er: ``Mein Herr, zeige (Dich) mir, daß ich Dich anschaue!" Er sagte: ``Du wirst Mich nicht sehen. Aber schau den Berg an! Wenn er fest an seiner Stelle bleibt, dann wirst du Mich sehen." Als nun sein Herr dem Berg offenbar erschien, ließ Er ihn in sich zusammensinken, und Musa fiel bewußtlos nieder'. Als er dann (wieder) zu sich kam, sagte er: ``Preis sei Dir! Ich wende mich in Reue Dir zu, und ich bin der erste der Gläubigen."\qt@no{143}}%
\qt@newcmd\qurantrans@de@mxcviii{Er sagte: ``O Musa, Ich habe dich durch Meine Botschaften und Mein Gespräch (mit dir) vor den Menschen auserwählt! So nimm, was Ich dir gegeben habe, und gehöre zu den Dankbaren!"\qt@no{144}}%
\qt@newcmd\qurantrans@de@mxcix{Und Wir schrieben ihm auf den Tafeln von allem eine Ermahnung und eine ausführliche Darlegung von allem. ``So halte sie fest und befiel deinem Volk, sich an das Schönste in ihnen zu halten! Ich werde euch die Wohnstätte der Frevler zeigen."\qt@no{145}}%
\qt@newcmd\qurantrans@de@mc{Ich werde von meinen Zeichen diejenigen abwenden, die auf der Erde ohne Recht hochmütig sind. Wenn sie auch jedes Zeichen sehen, glauben sie nicht daran. Und wenn sie den Weg der Besonnenheit sehen, nehmen sie ihn sich nicht zum Weg. Wenn sie aber den Weg der Verirrung sehen, nehmen sie ihn sich zum Weg. Dies, weil sie Unsere Zeichen für Lüge erklären und ihnen gegenüber unachtsam sind.\qt@no{146}}%
\qt@newcmd\qurantrans@de@mci{Diejenigen, die Unsere Zeichen und die Begegnung mit dem Jenseits für Lüge erklären, deren Werke werden hinfällig. Wird ihnen (denn) etwas anderes vergolten als das, was sie zu tun pflegten?\qt@no{147}}%
\qt@newcmd\qurantrans@de@mcii{Und das Volk Musas nahm sich, nachdem er (weggegangen) war, aus ihren Schmucksachen (verfertigt) ein Kalb als Leib, der blökte. Sahen sie denn nicht, daß es nicht zu ihnen spricht und sie nicht den rechten Weg leitet? Sie nahmen es sich (als Götzen) und waren ungerecht.\qt@no{148}}%
\qt@newcmd\qurantrans@de@mciii{Und als sich bei ihnen die Reue einstellte und sie sahen, daß sie bereits irregegangen waren, sagten sie: ``Wenn sich unser Herr nicht unser erbarmt und uns vergibt, werden wir ganz gewiß zu den Verlierern gehören!"\qt@no{149}}%
\qt@newcmd\qurantrans@de@mciv{Als Musa zu seinem Volk zornig und bekümmert zurückkam, sagte er: ``Wie schlimm ist das, was ihr nach mir an meiner Stelle begangen habt! Wollt ihr den Befehl eures Herrn beschleunigen?" Er warf die Tafeln hin und ergriff seinen Bruder beim Kopf, indem er ihn an sich zog. (Dieser) sagte: ``Sohn meiner Mutter, das Volk unterdrückte mich und hätte mich beinahe getötet! So lasse nicht die Feinde über mich Schadenfreude empfinden und stelle mich nicht zum ungerechten Volk!"\qt@no{150}}%
\qt@newcmd\qurantrans@de@mcv{Er (Musa) sagte: ``Mein Herr, vergib mir und meinem Bruder, und lasse uns in Deine Barmherzigkeit eingehen. Du bist ja der Barmherzigste der Barmherzigen."\qt@no{151}}%
\qt@newcmd\qurantrans@de@mcvi{Gewiß, diejenigen, die sich das Kalb nahmen, wird Zorn von ihrem Herrn und Erniedrigung im diesseitigen Leben ereilen. So vergelten Wir denen, die Lügen ersinnen.\qt@no{152}}%
\qt@newcmd\qurantrans@de@mcvii{Aber diejenigen, die böse Taten begehen und danach bereuen und glauben-, gewiß, dein Herr ist danach wahrlich Allvergebend und Barmherzig.\qt@no{153}}%
\qt@newcmd\qurantrans@de@mcviii{Und als sich in Musa der Zorn beruhigt hatte, nahm er die Tafeln. In ihrer Schrift ist Rechtleitung und Barmherzigkeit für die, die vor ihrem Herrn Ehrfurcht haben.\qt@no{154}}%
\qt@newcmd\qurantrans@de@mcix{Und Musa wählte aus seinem Volk siebzig Männer zu Unserer festgesetzten Zeit. Als sie nun das Zittern ergriff, sagte er: ``Mein Herr, wenn Du gewollt hättest, hättest Du sie schon zuvor vernichtet, und auch mich. Willst Du uns vernichten für das, was die Toren von uns getan haben? Es ist doch nur Deine Versuchung, mit der Du in die Irre gehen läßt, wen Du willst, und rechtleitest, wen Du willst. Du bist unser Schutzherr, so vergib uns und erbarme Dich unser! Du bist der Beste derer, die vergeben.\qt@no{155}}%
\qt@newcmd\qurantrans@de@mcx{Und bestimme für uns in diesem Diesseits Gutes und auch im Jenseits! Gewiß, wir haben zu Dir zurückgefunden." Er sagte: ``Mit Meiner Strafe treffe Ich, wen Ich will, aber Meine Barmherzigkeit umfaßt alles. Ich werde sie für die bestimmen, die gottesfürchtig sind und die Abgabe entrichten und die an Unsere Zeichen glauben,\qt@no{156}}%
\qt@newcmd\qurantrans@de@mcxi{die dem Gesandten, dem schriftunkundigen Propheten, folgen, den sie bei sich in der Tora und im Evangelium aufgeschrieben finden. Er gebietet ihnen das Rechte und verbietet ihnen das Verwerfliche, er erlaubt ihnen die guten Dinge und verbietet ihnen die schlechten, und er nimmt ihnen ihre Bürde und die Fesseln ab, die auf ihnen lagen. Diejenigen nun, die an ihn glauben, ihm beistehen, ihm helfen und dem Licht, das mit ihm herabgesandt worden ist, folgen, das sind diejenigen, denen es wohl ergeht."\qt@no{157}}%
\qt@newcmd\qurantrans@de@mcxii{Sag: O ihr Menschen, ich bin der Gesandte Allahs an euch alle, Dem die Herrschaft der Himmel und der Erde gehört. Es gibt keinen Gott außer Ihm. Er macht lebendig und läßt sterben. So glaubt an Allah und Seinen Gesandten, den schriftunkundigen Propheten, der an Allah und Seine Worte glaubt, und folgt ihm, auf daß ihr rechtgeleitet sein möget!\qt@no{158}}%
\qt@newcmd\qurantrans@de@mcxiii{Und unter dem Volk Musas ist eine Gemeinschaft, die mit der Wahrheit rechtleitet und nach ihr gerecht handelt.\qt@no{159}}%
\qt@newcmd\qurantrans@de@mcxiv{Und Wir zerteilten sie in zwölf Stämme, in Gemeinschaften. Und Wir gaben Musa ein, als sein Volk ihn um Wasser bat: ``Schlage mit deinem Stock auf den Felsen!" Da brachen aus ihm zwölf Quellen hervor. Nun wußte jedermann, wo sein Platz zum Trinken war. Und Wir ließen die Wolken sie überschatten und sandten das Manna und die Wachteln auf sie herab: ``Eßt von den guten Dingen, mit denen Wir euch versorgt haben!" Und sie fügten nicht Uns Unrecht zu, sondern sich selbst fügten sie Unrecht zu.\qt@no{160}}%
\qt@newcmd\qurantrans@de@mcxv{Und als zu ihnen gesagt wurde: ``Bewohnt diese Stadt und eßt von (dem, was in) ihr (ist), wo immer ihr wollt! und sagt: ``Vergebung!" und tretet, euch niederwerfend, durch das Tor ein, so vergeben Wir euch eure Verfehlungen. Und Wir werden den Gutes Tuenden noch mehr erweisen.\qt@no{161}}%
\qt@newcmd\qurantrans@de@mcxvi{Doch da tauschten diejenigen von ihnen, die Unrecht taten, das Wort gegen ein anderes aus, das ihnen nicht gesagt worden war. Und da sandten Wir auf sie eine unheilvolle Strafe vom Himmel hinab dafür, daß sie Unrecht zu tun pflegten.\qt@no{162}}%
\qt@newcmd\qurantrans@de@mcxvii{Und frag sie nach der Stadt, die am Meer lag, als sie den Sabbat übertraten, als ihre Fische zu ihnen an ihrem Sabbat sichtbar (geschwommen) kamen, aber an dem Tag, an dem sie nicht Sabbat hielten, nicht zu ihnen kamen. So prüfen Wir sie dafür, daß sie zu freveln pflegten.\qt@no{163}}%
\qt@newcmd\qurantrans@de@mcxviii{Und als eine Gemeinschaft von ihnen sagte: ``Warum ermahnt ihr Leute, die Allah vernichten oder mit strenger Strafe strafen wird?", sagten sie: ``Als eine Entschuldigung bei eurem Herrn und auf daß sie gottesfürchtig werden mögen."\qt@no{164}}%
\qt@newcmd\qurantrans@de@mcxix{Und als sie vergessen hatten, womit sie ermahnt worden waren, retteten Wir diejenigen, die das Böse verboten, und ergriffen diejenigen, die Unrecht taten, mit schlimmer Strafe dafür, daß sie zu freveln pflegten.\qt@no{165}}%
\qt@newcmd\qurantrans@de@mcxx{Als sie dann das mißachteten, was ihnen verboten war, sagten Wir zu ihnen: ``Werdet verstoßene Affen!"\qt@no{166}}%
\qt@newcmd\qurantrans@de@mcxxi{Und als dein Herr ankündigte, Er werde ganz gewiß gegen sie bis zum Tag der Auferstehung jemanden schicken, der ihnen eine böse Qual auferlegt. Dein Herr ist fürwahr schnell im Bestrafen, aber Er ist auch wahrlich Allvergebend und Barmherzig.\qt@no{167}}%
\qt@newcmd\qurantrans@de@mcxxii{Und Wir zerteilten sie auf der Erde in Gemeinschaften. Unter ihnen gab es Rechtschaffene und solche, die dies nicht waren. Und Wir prüften sie mit Gutem und Bösem, auf daß sie umkehren mögen.\qt@no{168}}%
\qt@newcmd\qurantrans@de@mcxxiii{Dann folgten nach ihnen Nachfolger, die die Schrift erbten. Sie greifen nach den Glücksgütern des Diesseitigen und sagen: ``Es wird uns vergeben werden." Und wenn sich ihnen gleiche Glücksgüter bieten, greifen sie danach. Wurde nicht mit ihnen das Abkommen der Schrift getroffen, sie sollen über Allah nur die Wahrheit sagen? Sie haben doch das, was darin steht, erlernt. Und die jenseitige Wohnstätte ist besser für diejenigen, die gottesfürchtig sind. Begreift ihr denn nicht?\qt@no{169}}%
\qt@newcmd\qurantrans@de@mcxxiv{Und diejenigen, die an der Schrift festhalten und das Gebet verrichten - gewiß, Wir lassen den Lohn der Heilstifter nicht verlorengehen.\qt@no{170}}%
\qt@newcmd\qurantrans@de@mcxxv{Und als Wir den Berg über sie heraushoben, als wäre er ein Schattendach, und sie glaubten, er würde auf sie fallen: ``Haltet fest an dem, was Wir euch gegeben haben, und gedenkt dessen, was es enthält, auf daß ihr gottesfürchtig werden möget!"\qt@no{171}}%
\qt@newcmd\qurantrans@de@mcxxvi{Und als dein Herr aus den Kindern Adams, aus ihren Rücken, ihre Nachkommenschaft nahm und sie gegen sich selbst zeugen ließ: ``Bin Ich nicht euer Herr?" Sie sagten: ``Doch, wir bezeugen (es)!" (Dies,) damit ihr nicht am Tag der Auferstehung sagt: ``Wir waren dessen unachtsam",\qt@no{172}}%
\qt@newcmd\qurantrans@de@mcxxvii{oder (auch) nicht sagt: ``Unsere Väter gesellten doch zuvor (Allah) Teilhaber zu, und wir sind (nur) eine Nachkommenschaft nach ihnen. Willst du uns vernichten für das, was diejenigen getan haben, die Nichtiges tun?"\qt@no{173}}%
\qt@newcmd\qurantrans@de@mcxxviii{So legen Wir die Zeichen ausführlich dar, - auf daß sie umkehren mögen.\qt@no{174}}%
\qt@newcmd\qurantrans@de@mcxxix{Und verlies ihnen die Kunde von dem, dem Wir Unsere Zeichen gaben und der sich dann ihrer entledigte. Da verfolgte ihn der Satan, und so wurde er einer der Verirrten.\qt@no{175}}%
\qt@newcmd\qurantrans@de@mcxxx{Und wenn Wir gewollt hätten, hätten Wir ihn durch sie fürwahr erhöht. Aber er neigte zur Erde hin und folgte seiner Neigung. So ist sein Gleichnis das eines Hundes: Wenn du auf ihn losgehst, hechelt er; läßt du ihn (in Ruhe), hechelt er (auch). So ist das Gleichnis der Leute, die Unsere Zeichen für Lüge erklärten. Berichte die Geschichte, auf daß sie nachdenken mögen.\qt@no{176}}%
\qt@newcmd\qurantrans@de@mcxxxi{Wie böse ist das Gleichnis der Leute, die Unsere Zeichen für Lüge erklärten und sich selbst Unrecht zu tun pflegten!\qt@no{177}}%
\qt@newcmd\qurantrans@de@mcxxxii{Wen Allah rechtleitet, der ist (in Wahrheit) rechtgeleitet. Wen Er aber in die Irre gehen läßt, das sind die Verlierer.\qt@no{178}}%
\qt@newcmd\qurantrans@de@mcxxxiii{Wir haben ja schon viele von den Ginn und den Menschen für die Hölle erschaffen. Sie haben Herzen, mit denen sie nicht verstehen; sie haben Augen, mit denen sie nicht sehen; und sie haben Ohren, mit denen sie nicht hören. Jene sind wie das Vieh. Aber nein! Sie irren noch weiter ab. Jene sind (überhaupt) die Unachtsamen.\qt@no{179}}%
\qt@newcmd\qurantrans@de@mcxxxiv{Allahs sind die schönsten Namen; so ruft Ihn damit an und laßt diejenigen, die mit Seinen Namen abwegig umgehen. Ihnen wird das vergolten, was sie zu tun pflegten.\qt@no{180}}%
\qt@newcmd\qurantrans@de@mcxxxv{Und unter denjenigen, die Wir erschaffen haben, gibt es eine Gemeinschaft, die mit der Wahrheit rechtleitet und nach ihr gerecht handelt.\qt@no{181}}%
\qt@newcmd\qurantrans@de@mcxxxvi{Diejenigen aber, die Unsere Zeichen für Lüge erklären, werden Wir stufenweise (dem Verderben) näherbringen, von wo sie nicht wissen.\qt@no{182}}%
\qt@newcmd\qurantrans@de@mcxxxvii{Und Ich gewähre ihnen Aufschub. Gewiß, Meine List ist fest.\qt@no{183}}%
\qt@newcmd\qurantrans@de@mcxxxviii{Denken sie denn nicht (darüber) nach? Ihr Gefährte leidet nicht an Besessenheit; er ist nur ein deutlicher Warner.\qt@no{184}}%
\qt@newcmd\qurantrans@de@mcxxxix{Haben sie sich denn nicht im Reich der Himmel und der Erde umgeschaut und was Allah an Dingen erschaffen hat und, daß ihre Frist vielleicht nahegerückt ist? An welche Aussage nach dieser wollen sie denn (sonst) glauben?\qt@no{185}}%
\qt@newcmd\qurantrans@de@mcxl{Wen Allah in die Irre gehen läßt, der hat keinen, der ihn rechtleiten könnte; Er läßt sie in ihrer Auflehnung umherirren.\qt@no{186}}%
\qt@newcmd\qurantrans@de@mcxli{Sie fragen dich nach der Stunde, für wann sie feststeht. Sag: Das Wissen darüber ist nur bei meinem Herrn. Nur Er wird sie zu ihrer Zeit erscheinen lassen. Schwer lastet sie in den Himmeln und auf der Erde. Sie wird nur plötzlich über euch kommen. Sie fragen dich, als ob du gut unterrichtet über sie seiest. Sag: Das Wissen darüber ist nur bei Allah. Aber die meisten Menschen wissen nicht.\qt@no{187}}%
\qt@newcmd\qurantrans@de@mcxlii{Sag: Ich vermag mir selbst weder Nutzen noch Schaden (zu bringen), außer was Allah will. Wenn ich das Verborgene wüßte, würde ich mir wahrlich viel Gutes verschaffen, und Böses würde mir nicht widerfahren. Ich bin nur ein Warner und ein Frohbote für Leute, die glauben.\qt@no{188}}%
\qt@newcmd\qurantrans@de@mcxliii{Er ist es, Der euch aus einem einzigen Wesen schuf, und Er hat aus ihm seine Gattin gemacht, damit er (der Mann) bei ihr Ruhe finde. Und als er über sie kam', trug sie an einer leichten Schwangerschaft und verbrachte damit (eine Zeit). Als sie dann schwer zu tragen hatte, riefen die beiden Allah, ihren Herrn, an: ``Wenn Du uns einen Rechtschaffenen gibst, werden wir ganz gewiß zu den Dankbaren gehören."\qt@no{189}}%
\qt@newcmd\qurantrans@de@mcxliv{Als Er ihnen dann einen Rechtschaffenen gegeben hatte, gaben sie Ihm Teilhaber an dem, was Er ihnen gegeben hatte. Aber Allah ist Erhaben über das, was sie (Ihm) beigesellen.\qt@no{190}}%
\qt@newcmd\qurantrans@de@mcxlv{Wollen sie (Ihm etwas) beigesellen, was nichts erschafft, während sie selbst erschaffen wurden,\qt@no{191}}%
\qt@newcmd\qurantrans@de@mcxlvi{und die ihnen keine Hilfe leisten, noch sich selbst helfen können?\qt@no{192}}%
\qt@newcmd\qurantrans@de@mcxlvii{Und wenn ihr sie zur Rechtleitung aufruft, folgen sie euch nicht. Gleich ist es in Bezug auf euch, ob ihr sie aufruft oder ob ihr schweigt.\qt@no{193}}%
\qt@newcmd\qurantrans@de@mcxlviii{Gewiß, diejenigen, die ihr anstatt Allahs anruft, sind (nur) Diener gleich euch. So ruft sie (doch) an, und so sollen sie euch doch erhören, wenn ihr wahrhaftig seid!\qt@no{194}}%
\qt@newcmd\qurantrans@de@mcxlix{Haben sie (etwa) Füße, mit denen sie gehen, oder haben sie Hände, mit denen sie zupacken, oder haben sie Augen, mit denen sie sehen, oder haben sie Ohren, mit denen sie hören? Sag: Ruft eure Teilhaber an, und hierauf schmiedet eure List gegen mich an und gewährt mir keinen Aufschub.\qt@no{195}}%
\qt@newcmd\qurantrans@de@mcl{Mein Schutzherr ist Allah, Der das Buch offenbart hat, und Er macht Sich zum Schutzherrn der Rechtschaffenen.\qt@no{196}}%
\qt@newcmd\qurantrans@de@mcli{Diejenigen, die ihr außer Ihm anruft, können euch keine Hilfe leisten, noch sich selbst helfen.\qt@no{197}}%
\qt@newcmd\qurantrans@de@mclii{Und wenn ihr sie zur Rechtleitung aufruft, hören sie nicht; du siehst sie dich anschauen, doch sie sehen nicht.\qt@no{198}}%
\qt@newcmd\qurantrans@de@mcliii{Nimm den Überschuß, gebiete das allgemein Gute und wende dich von den Toren ab!\qt@no{199}}%
\qt@newcmd\qurantrans@de@mcliv{Und wenn dich vom Satan eine Eingebung aufstachelt, dann suche Schutz bei Allah, denn Er ist ja Allhörend und Allwissend.\qt@no{200}}%
\qt@newcmd\qurantrans@de@mclv{Diejenigen, die gottesfürchtig sind, - wenn ihnen eine Anwandlung vom Satan widerfährt, bedenken sie, und da werden sie sogleich einsichtig.\qt@no{201}}%
\qt@newcmd\qurantrans@de@mclvi{Aber ihre Brüder lassen sie in ihrer Verirrung gewähren, und dann lassen sie (daran) nicht nach.\qt@no{202}}%
\qt@newcmd\qurantrans@de@mclvii{Und wenn du ihnen kein Zeichen bringst, sagen sie: ``Hättest du es dir doch (selbst) ausgesucht!" Sag: Ich folge nur dem, was mir von meinem Herrn (als Offenbarung) eingegeben wird. Dies sind einsichtbringende Zeichen von eurem Herrn und Rechtleitung und Barmherzigkeit für Leute, die glauben.\qt@no{203}}%
\qt@newcmd\qurantrans@de@mclviii{Und wenn der Qur'an vorgetragen wird, dann hört ihm zu und horcht hin, auf daß ihr Erbarmen finden möget!\qt@no{204}}%
\qt@newcmd\qurantrans@de@mclix{Und gedenke deines Herrn in deiner Seele in Unterwürfigkeit flehend und in Furcht und mit leiser Stimme, am Morgen und am Abend. Und gehöre nicht zu den Unachtsamen!\qt@no{205}}%
\qt@newcmd\qurantrans@de@mclx{Diejenigen, die bei deinem Herrn sind, sind nicht zu hochmütig dazu, Ihm zu dienen; sie preisen Ihn und werfen sich vor Ihm nieder.\qt@no{206}}%
\qt@newcmd\qurantrans@de@mclxi{\basmalahde Sie fragen dich nach der (zugedachten) Beute. Sag: Die (zugedachte) Beute gehört Allah und dem Gesandten. So fürchtet Allah und stiftet Frieden untereinander, und gehorcht Allah und Seinem Gesandten, wenn ihr gläubig seid!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mclxii{Die (wahren) Gläubigen sind ja diejenigen, deren Herzen sich vor Ehrfurcht regen, wenn Allahs gedacht wird, und die, wenn ihnen Seine Zeichen verlesen werden, es ihren Glauben mehrt, und die sich auf ihren Herrn verlassen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mclxiii{die das Gebet verrichten und von dem, womit Wir sie versorgt haben, ausgeben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mclxiv{Das sind die wahren Gläubigen. Für sie gibt es bei ihrem Herrn Rangstufen und Vergebung und ehrenvolle Versorgung.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mclxv{So wie dein Herr dich aus deinem Haus hinausziehen ließ mit der Wahrheit, während es einer Gruppe der Gläubigen wahrlich zuwider war,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mclxvi{und sie mit dir über die Wahrheit stritten, nachdem (es) klargeworden war, als ob sie in den Tod getrieben würden, während sie zuschauten.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mclxvii{Und als Allah euch versprach, daß die eine der beiden Gruppen euch gehören sollte, und ihr (es) lieber gehabt hättet, daß diejenige ohne Kampfkraft euer sein sollte! Aber Allah will mit Seinen Worten die Wahrheit bestätigen und die Rückkehr der Ungläubigen abschneiden,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mclxviii{um die Wahrheit zu bestätigen und das Falsche zunichte zu machen, auch wenn es den Übeltätern zuwider ist.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mclxix{Gedenke) als ihr euren Herrn um Hilfe anrieft. Da erhörte Er euch: ``Ich werde euch mit Tausend von den Engeln unterstützen, hintereinander reitend."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mclxx{Allah machte es nur zu einer frohen Botschaft und damit eure Herzen dadurch Ruhe fänden. Der Sieg kommt ja nur von Allah. Gewiß, Allah ist Allmächtig und Allweise.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mclxxi{Als Er Schläfrigkeit euch überkommen ließ als (Gefühl der) Sicherheit von Ihm und Wasser vom Himmel auf euch herabsandte, um euch damit zu reinigen und das Unheil des Satans von euch zu entfernen und um eure Herzen zu stärken und (eure) Füße damit zu festigen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mclxxii{Als dein Herr den Engeln eingab: ``Gewiß, Ich bin mit euch. So festigt diejenigen, die glauben! Ich werde in die Herzen derer, die ungläubig sind, Schrecken einjagen. So schlagt oberhalb der Nacken und schlagt von ihnen jeden Finger!"\qt@no{12}}%
\qt@newcmd\qurantrans@de@mclxxiii{Dies dafür, daß sie Allah und Seinem Gesandten entgegenwirkten. Wer Allah und Seinem Gesandten entgegenwirkt, - gewiß, Allah ist streng im Bestrafen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mclxxiv{Das (ist eure Strafe dafür), so kostet sie! Und (wisset,) daß es für die Ungläubigen die Strafe des (Höllen)feuers geben wird.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mclxxv{O die ihr glaubt, wenn ihr auf die, die ungläubig sind, trefft, während sie (zur Schlacht) anrücken, dann kehrt ihnen nicht den Rücken.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mclxxvi{Wer ihnen an jenem Tag den Rücken kehrt - außer, er setzt sich ab zum Kampf, oder er schließt sich einer (anderen) Schar an -, zieht sich fürwahr Zorn von Allah zu, und sein Zufluchtsort ist die Hölle - ein schlimmer Ausgang!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mclxxvii{Nicht ihr habt sie getötet, sondern Allah hat sie getötet. Und nicht du hast geworfen, als du geworfen hast, sondern Allah hat geworfen, und damit Er die Gläubigen einer schönen Prüfung von Ihm unterziehe. Gewiß, Allah ist Allhörend und Allwissend.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mclxxviii{Das (ist eure Prüfung), und (wisset), daß Allah die List der Ungläubigen abschwächt!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mclxxix{Wenn ihr nach einer Entscheidung sucht, so ist nunmehr die Entscheidung zu euch gekommen. Und wenn ihr aufhört, so ist es besser für euch. Aber wenn ihr (dazu) zurückkehrt, kehren Wir (auch) zurück. Und eure Schar wird euch nichts nützen, auch wenn sie zahlreich sein sollte. Und (wisset,) daß Allah mit den Gläubigen ist!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mclxxx{O die ihr glaubt, gehorcht Allah und Seinem Gesandten, und kehrt euch nicht von ihm ab, wo ihr doch hört!\qt@no{20}}%
\qt@newcmd\qurantrans@de@mclxxxi{Und seid nicht wie diejenigen, die sagen: ``Wir hören", wo sie doch nicht hören!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mclxxxii{Gewiß, die schlimmsten Tiere bei Allah sind die tauben und stummen, die nicht begreifen'.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mclxxxiii{Wenn Allah von ihnen etwas Gutes gewußt hätte, hätte Er sie wahrlich hören lassen. Und wenn Er sie hätte hören lassen, so hätten sie sich dennoch widerstrebend abgekehrt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mclxxxiv{O die ihr glaubt, leistet Allah und dem Gesandten Folge, wenn er euch zu dem aufruft, was euch Leben gibt. Und wisset, daß Allah zwischen dem Menschen und seinem Herzen trennt und daß ihr zu Ihm versammelt werdet!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mclxxxv{Und hütet euch vor einer Versuchung, die nicht nur besonders diejenigen von euch treffen wird, die Unrecht taten. Und wisset, daß Allah streng im Bestrafen ist!\qt@no{25}}%
\qt@newcmd\qurantrans@de@mclxxxvi{Und gedenkt, als ihr wenige wart und auf der Erde unterdrückt wurdet und fürchtetet, daß euch die Menschen wegschnappen würden! Da hat Er euch Zuflucht gewährt, euch mit Seiner Hilfe gestärkt und euch mit (einigen von) den guten Dingen versorgt, auf daß ihr dankbar sein möget.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mclxxxvii{O die ihr glaubt, verratet nicht Allah und den Gesandten, und handelt nicht verräterisch in Bezug auf die euch anvertrauten Güter, wo ihr wisset!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mclxxxviii{Und wißt, daß euer Besitz und eure Kinder eine Versuchung sind und daß es bei Allah großartigen Lohn gibt!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mclxxxix{O die ihr glaubt, wenn ihr Allah fürchtet, bestimmt Er euch eine Unterscheidung(snorm), tilgt euch eure bösen Taten und vergibt euch. Und Allah besitzt große Huld.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcxc{Und als diejenigen, die ungläubig sind, gegen dich Ränke schmiedeten, um dich festzusetzen oder zu töten oder zu vertreiben. Sie schmiedeten Ränke, und (auch) Allah schmiedete Ränke. Aber Allah ist der beste Ränkeschmied.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcxci{Und wenn ihnen Unsere Zeichen verlesen werden, sagen sie: ``Wir haben es bereits gehört. Wenn wir wollten, könnten wir fürwahr etwas Gleichartiges sagen. Das sind nur Fabeln der Früheren."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcxcii{Und als sie sagten: ``O Allah, wenn dies tatsächlich die Wahrheit von Dir ist, dann lasse auf uns Steine vom Himmel regnen, oder bringe schmerzhafte Strafe über uns!"\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcxciii{Aber Allah würde sie nimmer strafen, solange du unter ihnen bist; und Allah würde sie nimmer strafen, solange sie um Vergebung bitten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcxciv{Und was haben sie nun, daß Allah sie nicht strafen sollte, wo sie (euch) von der geschützten Gebetsstätte abhalten? Und sie waren nicht ihre Beschützer. Ihre Beschützer sind ja nur die Gottesfürchtigen'! Aber die meisten von ihnen wissen nicht.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcxcv{Und ihr Gebet beim Haus ist nur Pfeifen und Klatschen. Kostet nun die Strafe dafür, daß ihr stets ungläubig wart.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcxcvi{Diejenigen, die ungläubig sind, geben ihren Besitz aus, um von Allahs Weg abzuhalten. Sie werden ihn ausgeben, und hierauf wird es eine gramvolle Reue für sie sein. Hierauf werden sie besiegt werden. Und diejenigen, die ungläubig sind, werden zur Hölle versammelt werden,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcxcvii{damit Allah das Schlechte vom Guten sondere und das Schlechte übereinander tue, es alles zusammenhäufe und in die Hölle bringe. Das sind die Verlierer.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcxcviii{Sag zu denen, die ungläubig sind: Wenn sie aufhören, wird ihnen vergeben, was bereits vergangen ist. Wenn sie aber (dazu) zurückkehren, - so hat sich schon die Gesetzmäßigkeit an den Früheren vollzogen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcxcix{Und kämpft gegen sie, bis es keine Verfolgung mehr gibt und (bis) die Religion gänzlich Allahs ist. Wenn sie jedoch aufhören, so sieht Allah wohl, was sie tun.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcc{Und wenn sie sich abkehren, so wisset, daß Allah euer Schutzherr ist. Wie trefflich ist der Schutzherr, und wie trefflich ist der Helfer!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcci{Und wisset: Was immer ihr erbeutet, so gehört Allah ein Fünftel davon und dem Gesandten, und den Verwandten, den Waisen, den Armen und dem Sohn des Weges, wenn ihr an Allah glaubt und an das, was Wir auf Unseren Diener am Tag der Unterscheidung (als Offenbarung) hinabgesandt haben, an dem Tag, da die beiden Heere aufeinandertrafen. Und Allah hat zu allem die Macht.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mccii{Als ihr auf der näheren Talseite wart, sie auf der ferneren Talseite und die Karawane unterhalb von euch. Und wenn ihr euch verabredet hättet, wäret ihr über die Verabredung fürwahr uneinig gewesen, aber (es geschah so), damit Allah eine Angelegenheit entscheide, die ausgeführt werden sollte, damit, wer umkam, auf Grund eines klaren Beweises umkäme, und wer am Leben bliebe, auf Grund eines klaren Beweises am Leben bliebe. Allah ist wahrlich Allhörend und Allwissend.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcciii{Als Allah sie dir in deinem Schlaf als wenige zeigte -, und wenn Er sie dir als viele gezeigt hätte, hättet ihr wahrlich den Mut verloren und über die Angelegenheit miteinander gestritten. Aber Allah hat (euch) bewahrt. Gewiß, Er weiß über das Innerste der Brüste Bescheid.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcciv{Und als Er sie euch, als ihr aufeinandertraft, in euren Augen als wenige erscheinen ließ, und (auch) euch in ihren Augen weniger machte, damit Allah eine Angelegenheit entscheide, die ausgeführt werden sollte. Und zu Allah werden (all) die Angelegenheiten zurückgebracht.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mccv{O die ihr glaubt, wenn ihr auf eine Schar trefft, so steht fest und gedenkt Allahs häufig, auf daß es euch wohl ergehen möge!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mccvi{Und gehorcht Allah und Seinem Gesandten, und streitet nicht miteinander, sonst würdet ihr den Mut verlieren, und eure Kraft würde vergehen! Und seid standhaft! Gewiß, Allah ist mit den Standhaften.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mccvii{Und seid nicht wie diejenigen, die aus ihren Wohnstätten hinauszogen, in Übermut und aus Augendienerei vor den Menschen, und die von Allahs Weg abhalten. Allah umfaßt, was sie tun.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mccviii{Und als der Satan ihnen ihre Werke ausschmückte und sagte: ``Es gibt heute keinen unter den Menschen, der euch besiegen könnte. Und ich bin euch ein Beschützer. Als aber die beiden Scharen einander sahen, machte er auf seinen Fersen kehrt und sagte: ``Gewiß, ich sage mich von euch los. Ich sehe, was ihr nicht seht. Ich fürchte Allah. Und Allah ist streng im Bestrafen."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mccix{Als die Heuchler und diejenigen, in deren Herzen Krankheit ist, sagten: ``Getäuscht hat diese da ihre Religion!" Wer sich aber auf Allah verläßt, - so ist Allah Allmächtig und Allwissend.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mccx{Und wenn du sehen würdest, wenn die Engel diejenigen abberufen, die ungläubig sind, wobei sie sie ins Gesicht und auf den Rücken schlagen und (sagen): ``Kostet die Strafe des Brennens!\qt@no{50}}%
\qt@newcmd\qurantrans@de@mccxi{Dies (geschieht) wegen dessen, was eure Hände vorausgeschickt haben, und deshalb, weil Allah keiner ist, der den Menschen Unrecht zufügt."\qt@no{51}}%
\qt@newcmd\qurantrans@de@mccxii{Nach der Art der Leute Fir'auns und derjenigen vor ihnen: Sie verleugneten Allahs Zeichen, und da ergriff Allah sie für ihre Sünden. Gewiß, Allah ist Stark und streng im Bestrafen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mccxiii{Dies, weil Allah nimmer eine Gunst, die Er einem Volk erwiesen hat, ändert, bis sie das ändern, was in ihnen selbst ist, und weil Allah Allhörend und Allwissend ist.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mccxiv{Nach der Art der Leute Fir'auns und derjenigen vor ihnen: Sie erklärten die Zeichen ihres Herrn für Lüge, und da vernichteten Wir sie für ihre Sünden, und Wir ließen die Leute Fir'auns ertrinken. Sie alle waren ungerecht.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mccxv{Gewiß, die schlimmsten Tiere bei Allah sind die, die ungläubig sind und (auch) weiterhin nicht glauben,\qt@no{55}}%
\qt@newcmd\qurantrans@de@mccxvi{mit denen du eine Verpflichtung eingegangen bist, die aber dann ihre Verpflichtung jedesmal brechen und nicht gottesfürchtig sind.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mccxvii{Wenn du nun auf sie im Krieg triffst, dann verscheuche mit ihnen diejenigen, die hinter ihnen stehen, auf daß sie bedenken mögen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mccxviii{Und wenn du dann von irgendwelchen Leuten Verrat befürchtest, so verwirf ihnen (die Verpflichtung) in gleicher Weise. Gewiß, Allah liebt nicht die Verräter.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mccxix{Und diejenigen, die ungläubig sind, sollen ja nicht meinen, sie könnten zuvorkommen. Sie werden sich (Mir) gewiß nicht entziehen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mccxx{Und haltet für sie bereit, was ihr an Kraft und an kampfbereiten Pferden (haben) könnt, um damit den Feinden Allahs und euren Feinden Angst zu machen, sowie anderen außer ihnen, die ihr nicht kennt; Allah aber kennt sie! Und was immer ihr auf Allahs Weg ausgebt, wird euch in vollem Maß zukommen, und es wird euch kein Unrecht zugefügt.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mccxxi{Und wenn sie sich dem Frieden zuneigen, dann neige auch du dich ihm zu und verlasse dich auf Allah! Gewiß, Er ist ja der Allhörende und Allwissende.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mccxxii{Und wenn sie dich betrügen wollen - gewiß, so ist deine Genüge Allah. Er ist es, Der dich mit Seiner Hilfe und mit den Gläubigen gestärkt hat.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mccxxiii{Und Er hat ihre Herzen zusammengefügt. Wenn du alles, was auf der Erde ist, (dafür) ausgegeben hättest, hättest du ihre Herzen nicht zusammenfügen können. Aber Allah hat sie zusammengefügt. Gewiß, Er ist Allmächtig und Allweise.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mccxxiv{O Prophet, deine Genüge ist Allah und (auch) derer von den Gläubigen, die dir folgen.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mccxxv{O Prophet, sporne die Gläubigen zum Kampf an! Wenn es unter euch zwanzig Standhafte gibt, werden sie zweihundert besiegen. Und wenn es unter euch hundert gibt, werden sie Tausend von denen, die ungläubig sind, besiegen, weil sie Leute sind, die nicht verstehen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mccxxvi{Jetzt (aber) hat Allah es euch leicht gemacht. Er weiß ja, daß in euch Schwaches (angelegt) ist. Wenn es nun unter euch hundert Standhafte gibt, werden sie zweihundert besiegen, und wenn es unter euch Tausend gibt, werden sie Zweitausend besiegen, mit Allahs Erlaubnis. Allah ist mit den Standhaften.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mccxxvii{Es steht keinem Propheten zu, Gefangene zu haben, bis er (den Feind überall) im Land schwer niedergekämpft hat. Ihr wollt Glücksgüter des Diesseitigen, aber Allah will das Jenseits. Allah ist Allmächtig und Allweise.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mccxxviii{Wenn nicht von Allah eine früher ergangene Bestimmung wäre, würde euch für das, was ihr genommen habt, wahrlich gewaltige Strafe widerfahren.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mccxxix{Eßt nun von dem, was ihr erbeutet habt, als etwas Erlaubtes und Gutes, und fürchtet Allah! Gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mccxxx{O Prophet, sag zu denen von den Gefangenen (, die sich) in euren Händen (befinden): ``Wenn Allah in euren Herzen etwas Gutes weiß, wird Er euch etwas Besseres geben als das, was euch genommen worden ist, und wird euch vergeben. Allah ist Allvergebend und Barmherzig."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mccxxxi{Und wenn sie dich verraten wollen, so haben sie schon zuvor Allah verraten, und dann hat Er (euch) Macht über sie gegeben. Allah ist Allwissend und Allweise.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mccxxxii{Gewiß, diejenigen, die glauben und ausgewandert sind und sich mit ihrem Besitz und ihrer eigenen Person auf Allahs Weg abgemüht haben, und diejenigen, die (jenen) Zuflucht gewährt und (ihnen) geholfen haben, sie sind einer des anderen Schutzherren. Zu denjenigen aber, die glauben und nicht ausgewandert sind, habt ihr kein Schutzverhältnis, bis sie auswandern. Wenn sie euch jedoch um der (euch gemeinsamen) Religion willen um Hilfe bitten, dann obliegt euch die Hilfe, außer gegen Leute, zwischen euch und denen ein Abkommen besteht. Und was ihr tut, sieht Allah wohl.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mccxxxiii{Und diejenigen, die ungläubig sind, sind einer des anderen Schutzherren. - Wenn ihr es nicht tut, wird es im Land Aufruhr und großes Unheil geben.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mccxxxiv{Und diejenigen, die glauben und ausgewandert sind und sich auf Allahs Weg abgemüht haben, und diejenigen, die (jenen) Zuflucht gewährt und geholfen haben, das sind die wahren Gläubigen. Für sie gibt es Vergebung und ehrenvolle Versorgung.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mccxxxv{Und diejenigen, die danach geglaubt haben und ausgewandert sind und sich mit euch abgemüht haben, sie gehören zu euch. Aber die Blutsverwandten stehen einander am nächsten; (dies steht) im Buch Allahs. Gewiß, Allah weiß über alles Bescheid.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mccxxxvi{\basmalahde Eine Lossagung von Seiten Allahs und Seines Gesandten an diejenigen unter den Götzendienern, mit denen ihr einen Vertrag abgeschlossen habt.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mccxxxvii{Nun zieht im Land vier Monate umher und wißt, daß ihr euch Allah nicht entziehen könnt, und daß Allah die Ungläubigen in Schande stürzt!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mccxxxviii{Und eine Bekanntmachung von Seiten Allahs und Seines Gesandten an die Menschen am Tag der großen Pilgerfahrt, daß Allah der (Verpflichtung gegenüber den) Götzendiener(n) ledig ist, und auch Sein Gesandter. Wenn ihr nun bereut, so ist es besser für euch. Wenn ihr euch jedoch abkehrt, so wißt, daß ihr euch Allah nicht entziehen könnt! Und verkünde denen, die ungläubig sind, schmerzhafte Strafe!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mccxxxix{Mit Ausnahme derer von den Götzendienern, mit denen ihr einen Vertrag abgeschlossen habt und die (es) euch (in) nichts haben fehlen lassen und niemandem gegen euch beigestanden haben. So erfüllt ihnen gegenüber ihren Vertrag bis zu der ihnen eingeräumten Frist! Gottesfürchtigen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mccxl{Wenn nun die Schutzmonate abgelaufen sind, dann tötet die Götzendiener, wo immer ihr sie findet, ergreift sie, belagert sie und lauert ihnen aus jedem Hinterhalt auf! Wenn sie aber bereuen, das Gebet verrichten und die Abgabe entrichten, dann laßt sie ihres Weges ziehen! Gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mccxli{Und wenn jemand von den Götzendienern dich um Schutz bittet, dann gewähre ihm Schutz, bis er das Wort Allahs hört. Hierauf lasse ihn den Ort erreichen, wo er in Sicherheit ist. Dies, weil sie Leute sind, die nicht Bescheid wissen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mccxlii{Wie sollte es denn für die Götzendiener bei Allah und bei Seinem Gesandten einen Vertrag geben, außer (für) diejenigen, mit denen ihr bei der geschützten Gebetsstätte einen Vertrag abgeschlossen habt? Solange sie sich euch gegenüber recht verhalten, verhaltet (auch) ihr euch ihnen gegenüber recht. Gewiß, Allah liebt die Gottesfürchtigen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mccxliii{Wie (sollte es einen Vertrag geben), wo sie doch, wenn sie die Oberhand über euch bekommen, euch gegenüber weder Verwandtschaftsbande noch (Schutz)vertrag beachten? Sie stellen euch mit ihren Mündern zufrieden, aber ihre Herzen weigern sich. Und die meisten von ihnen sind Frevler.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mccxliv{Sie haben die Zeichen Allahs für einen geringen Preis verkauft und damit von Seinem Weg abgehalten. Gewiß, wie böse ist, was sie zu tun pflegten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mccxlv{Sie beachten gegenüber einem Gläubigen weder Verwandtschaftsbande noch (Schutz)vertrag. Das sind die Übertreter.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mccxlvi{Wenn sie aber bereuen, das Gebet verrichten und die Abgabe entrichten, dann sind sie eure Brüder in der Religion. Wir legen die Zeichen ausführlich dar für Leute, die Bescheid wissen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mccxlvii{Wenn sie ihre Eide aber nach Vertragsabschluß brechen und eure Religion schmähen, dann kämpft gegen die Anführer des Unglaubens - für sie gibt es ja keine Eide -, auf daß sie aufhören mögen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mccxlviii{Wollt ihr nicht gegen Leute kämpfen, die ihre Eide gebrochen haben und vorhatten, den Gesandten zu vertreiben, wobei sie zuerst gegen euch (mit Feindseligkeiten) anfingen? Fürchtet ihr sie? Aber Allah hat ein größeres Anrecht darauf, daß ihr Ihn fürchtet, wenn ihr gläubig seid.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mccxlix{Kämpft gegen sie! Allah wird sie durch eure Hände strafen, sie in Schande stürzen, euch zum Sieg über sie verhelfen und die Brüste von gläubigen Leuten heilen'\qt@no{14}}%
\qt@newcmd\qurantrans@de@mccl{und den Grimm in ihren Herzen vergehen lassen. Allah nimmt die Reue an, von wem Er will. Allah ist Allwissend und Allweise.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mccli{Oder meint ihr etwa, daß ihr (in Ruhe) gelassen werdet, ohne daß Allah zuvor diejenigen kenne, die von euch sich abgemüht und außer Allah, Seinem Gesandten und den Gläubigen keinen Vertrauten genommen haben? Und Allah ist Kundig dessen, was ihr tut.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mcclii{Es steht den Götzendienern nicht zu, Allahs Gebetsstätten zu bevölkern, wo sie gegen sich selbst Zeugnis ablegen durch den Unglauben. Deren Werke werden hinfällig, und im (Höllen)feuer werden sie ewig bleiben.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mccliii{Gewiß, Allahs Gebetsstätten bevölkert nur, wer an Allah und den Jüngsten Tag glaubt, das Gebet verrichtet, die Abgabe entrichtet und niemanden außer Allah fürchtet. Diese aber werden vielleicht zu den Rechtgeleiteten gehören.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mccliv{Stellt ihr etwa die Tränkung der Pilger und das Bevölkern der geschützten Gebetsstätte (den Werken) dessen gleich, der an Allah und den Jüngsten Tag glaubt und sich auf Allahs Weg abmüht? Sie sind nicht gleich bei Allah. Und Allah leitet das ungerechte Volk nicht recht.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mcclv{Diejenigen, die glauben und ausgewandert sind und sich auf Allahs Weg mit ihrem Besitz und ihrer eigenen Person abgemüht haben, haben einen größeren Vorzug bei Allah. Das sind die Erfolgreichen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mcclvi{Ihr Herr verkündet ihnen Barmherzigkeit von sich und Wohlgefallen und Gärten, in denen sie beständige Wonne haben,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mcclvii{ewig und auf immer darin zu bleiben. Gewiß, bei Allah gibt es großartigen Lohn.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mcclviii{O die ihr glaubt, nehmt nicht eure Väter und eure Brüder zu Schutzherren', wenn sie den Unglauben mehr lieben als den Glauben! Wer von euch sie zu Vertrauten nimmt, das sind die Ungerechten.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mcclix{Sag: Wenn eure Väter, eure Söhne, eure Brüder, eure Gattinnen und eure Sippemitglieder, Besitz, den ihr erworben habt, Handel, dessen Niedergang ihr fürchtet, und Wohnungen, an denen ihr Gefallen findet, euch lieber sind als Allah und Sein Gesandter und das Abmühen auf Seinem Weg, dann wartet ab, bis Allah mit Seiner Anordnung kommt! Allah leitet das Volk der Frevler nicht recht.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mcclx{Allah hat euch doch an vielen Orten zum Sieg verholfen, und auch am Tag von Hunain, als eure große Zahl euch gefiel, euch aber nichts nutzte. Die Erde wurde euch eng bei all ihrer Weite. Hierauf kehrtet ihr den Rücken (zur Flucht).\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcclxi{Daraufhin sandte Allah Seine innere Ruhe auf Seinen Gesandten und auf die Gläubigen herab, und Er sandte Heerscharen, die ihr nicht saht, herab und strafte diejenigen, die ungläubig waren. Das war der Lohn der Ungläubigen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mcclxii{Hierauf nimmt Allah nach alledem die Reue an, von wem Er will. Und Allah ist Allvergebend und Barmherzig.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mcclxiii{O die ihr glaubt, die Götzendiener sind fürwahr unrein, so sollen sie sich der geschützten Gebetsstätte nach diesem, ihrem Jahr nicht mehr nähern! Und wenn ihr (deshalb) Armut befürchtet, so wird Allah euch durch Seine Huld reich machen, wenn Er will. Gewiß, Allah ist Allwissend und Allweise.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mcclxiv{Kämpft gegen diejenigen, die nicht an Allah und nicht an den Jüngsten Tag glauben und nicht verbieten, was Allah und Sein Gesandter verboten haben, und nicht die Religion der Wahrheit befolgen - von denjenigen, denen die Schrift gegeben wurde -, bis sie den Tribut aus der Hand entrichten und gefügig sind!\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcclxv{Die Juden sagen: ``'Uzair ist Allahs Sohn", und die Christen sagen: ``Al-Masih ist Allahs Sohn." Das sind ihre Worte aus ihren (eigenen) Mündern. Sie führen ähnliche Worte wie diejenigen, die zuvor ungläubig waren. Allah bekämpfe sie! Wie sie sich (doch) abwendig machen lassen!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcclxvi{Sie haben ihre Gelehrten und ihre Mönche zu Herren genommen außer Allah, sowie al-Masih ibn Maryam, wo ihnen doch nur befohlen worden ist, einem einzigen Gott zu dienen. Es gibt keinen Gott außer Ihm. Preis sei Ihm! (Erhaben ist Er) über das, was sie (Ihm) beigesellen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcclxvii{Sie wollen Allahs Licht mit ihren Mündern auslöschen. Aber Allah besteht darauf, Sein Licht doch zu vollenden, auch wenn es den Ungläubigen zuwider ist.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcclxviii{Er ist es, Der Seinen Gesandten mit der Rechtleitung und der Religion der Wahrheit gesandt hat, um ihr die Oberhand über alle Religion zu geben', auch wenn es den Götzendienern zuwider ist.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcclxix{O die ihr glaubt, viele von den Gelehrten und den Mönchen verschlingen fürwahr den Besitz der Menschen auf nichtige Weise und halten von Allahs Weg ab. Diejenigen, die Gold und Silber horten und es nicht auf Allahs Weg ausgeben, denen verkünde schmerzhafte Strafe,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcclxx{am Tag, da im Feuer der Hölle darüber heiß gemacht wird und damit ihre Stirnen, ihre Seiten und ihre Rücken gebrandmarkt werden: Dies ist, was ihr für euch selbst gehortet habt. Nun kostet, was ihr zu horten pflegtet!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcclxxi{Gewiß, die Anzahl der Monate bei Allah ist zwölf Monate, im Buch Allahs (festgelegt) am Tag, da Er die Himmel und die Erde schuf. Davon sind vier geschützt, Das ist die richtige Religion. So fügt euch selbst in ihnen kein Unrecht zu. Und kämpft gegen die Götzendiener allesamt wie sie gegen euch allesamt kämpfen! Und wißt, daß Allah mit den Gottesfürchtigen ist!\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcclxxii{Das Verschieben eines Monats ist nur eine Mehrung des Unglaubens. Damit werden diejenigen, die ungläubig sind, in die Irre geführt; sie erklären ihn in einem Jahr für ungeschützt und in einem (anderen) Jahr für geschützt, um der Anzahl dessen gleichzukommen, was Allah für geschützt erklärt hat; so erklären sie für ungeschützt, was Allah für geschützt erklärt hat. Ihre bösen Taten sind ihnen ausgeschmückt worden. Allah leitet das ungläubige Volk nicht recht.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcclxxiii{O die ihr glaubt, was ist mit euch, daß, wenn zu euch gesagt wird: ``Rückt aus auf Allahs Weg!", ihr euch schwer zur Erde sinken laßt? Seid ihr mit dem diesseitigen Leben mehr zufrieden als mit dem Jenseits? Aber der Genuß des diesseitigen Lebens wird im Jenseits nur gering (erscheinen).\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcclxxiv{Wenn ihr nicht ausrückt, wird Er euch mit schmerzhafter Strafe strafen und euch durch ein anderes Volk ersetzen, und ihr (könnt) Ihm keinerlei Schaden zufügen. Allah hat zu allem die Macht.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcclxxv{Wenn ihr ihm nicht helft, so hat Allah ihm (schon damals) geholfen, als diejenigen, die ungläubig waren, ihn als einen von Zweien vertrieben; als sie beide in der Höhle waren und als er zu seinem Gefährten sagte: ``Sei nicht traurig! Gewiß, Allah ist mit uns!" Da sandte Allah Seine innere Ruhe auf ihn herab und stärkte ihn mit Heerscharen, die ihr nicht saht, und erniedrigte das Wort derjenigen, die ungläubig waren, während Allahs Wort (doch) das hohe ist. Allah ist Allmächtig und Allweise.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcclxxvi{Rückt aus, leicht oder schwer, und müht euch mit eurem Besitz und eurer eigenen Person auf Allahs Weg ab. Das ist besser für euch, wenn ihr (es) nur wißt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mcclxxvii{Wenn es um nahe Glücksgüter und eine mäßige Reise ginge, würden sie dir wahrlich folgen. Aber die Entfernung ist ihnen zu weit. Und dennoch werden sie bei Allah schwören: ``Wenn wir (es) könnten, würden wir fürwahr mit euch hinausziehen." Dabei vernichteten sie sich selbst. Doch Allah weiß, daß sie wahrlich lügen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcclxxviii{Allah verzeihe dir! Warum hast du ihnen erlaubt (, zurückzubleiben), bevor sich dir diejenigen klar gezeigt haben, die wahrhaftig sind, und du die Lügner kennst?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcclxxix{Diejenigen, die an Allah und den Jüngsten Tag glauben, bitten dich nicht um Erlaubnis, sich mit ihrem Besitz und ihrer eigenen Person nicht abzumühen. Und Allah weiß über die Gottesfürchtigen Bescheid.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mcclxxx{Um Erlaubnis bitten dich nur diejenigen, die an Allah und den Jüngsten Tag nicht glauben und deren Herzen zweifeln; so zaudern sie in ihrem Zweifel.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mcclxxxi{Wenn sie (wirklich) hätten hinausziehen wollen, hätten sie fürwahr Vorbereitungen dazu getroffen. Aber Allah war ihr Ausziehen zuwider, und so hielt Er sie zurück. Und es wurde gesagt: ``So bleibt (daheim) mit denjenigen, die (daheim) sitzen bleiben!"\qt@no{46}}%
\qt@newcmd\qurantrans@de@mcclxxxii{Wenn sie mit euch hinausgezogen wären, hätten sie euch nur Verwirrung gebracht und wären unter euch fürwahr umhergelaufen im Trachten danach, euch der Versuchung auszusetzen. Und unter euch gibt es manche, die immer (wieder) auf sie horchen. Und Allah weiß über die Ungerechten Bescheid.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mcclxxxiii{Sie haben bereits zuvor danach getrachtet, euch der Versuchung auszusetzen, und haben dir die Angelegenheiten umgekehrt', bis die Wahrheit kam und die Anordnung Allahs erschien, obwohl es ihnen zuwider war.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mcclxxxiv{Unter ihnen gibt es manche, die sagen: ``Erlaube mir (, zurückzubleiben) und setze mich nicht der Versuchung aus!" Dabei sind sie doch in Versuchung gefallen. Und wahrlich, die Hölle umfaßt die Ungläubigen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mcclxxxv{Wenn dich etwas Gutes trifft, tut es ihnen leid. Wenn dich jedoch ein Unglück trifft, sagen sie: ``Wir haben unsere Angelegenheit schon zuvor selbst übernommen" und kehren sich froh ab.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mcclxxxvi{Sag: Uns wird nur das treffen, was Allah für uns bestimmt hat. Er ist unser Schutzherr. Auf Allah sollen sich die Gläubigen verlassen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mcclxxxvii{Sag: Erwartet ihr für uns etwas (anderes) als eines der beiden schönsten Dinge? Wir erwarten für euch, daß Allah euch mit einer Strafe von Ihm oder durch unsere Hände trifft. So wartet nur ab! Gewiß, Wir warten mit euch ab.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mcclxxxviii{Sag: Gebt freiwillig oder widerwillig aus, es wird von euch (doch) nicht angenommen werden! Ihr seid ja frevlerische Leute.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mcclxxxix{Und nichts (anderes) verhindert, daß ihre Ausgaben von ihnen angenommen werden, als daß sie Allah und Seinen Gesandten verleugnen, sich nur schwerfällig zum Gebet begeben und nur widerwillig ausgeben.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mccxc{So sollen dir weder ihr Besitz noch ihre Kinder gefallen. Allah will sie damit ja nur im diesseitigen Leben strafen, und (daß) ihre Seelen (im Tod) dahingehen, während sie ungläubig sind.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mccxci{Und sie schwören bei Allah, daß sie fürwahr zu euch gehören, während sie nicht zu euch gehören, sondern sie sind Leute, die ängstlich sind.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mccxcii{Wenn sie einen Zufluchtsort oder Höhlen oder (sonst) ein Schlupfloch fänden, würden sie sich wahrlich dorthin wenden, und zwar fluchtartig.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mccxciii{Und unter ihnen gibt es manche, die gegen dich wegen der Almosen nörgeln. Wenn ihnen davon gegeben wird, zeigen sie Wohlgefallen. Wenn ihnen davon aber nichts gegeben wird, zeigen sie sogleich Mißfallen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mccxciv{Wenn sie doch mit dem zufrieden wären, was Allah und Sein Gesandter ihnen gewährten, und sagten: ``Unsere Genüge ist Allah. Allah wird uns (etwas) von Seiner Huld gewähren und auch Sein Gesandter. Gewiß, nach Allah steht unser Begehren".\qt@no{59}}%
\qt@newcmd\qurantrans@de@mccxcv{Die Almosen sind nur für die Armen, die Bedürftigen, diejenigen, die damit beschäftigt sind, diejenigen, deren Herzen vertraut gemacht werden sollen, (den Loskauf von) Sklaven, die Verschuldeten, auf Allahs Weg und (für) den Sohn des Weges, als Verpflichtung von Allah. Allah ist Allwissend und Allweise.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mccxcvi{Unter ihnen gibt es diejenigen, die dem Propheten Leid zufügen und sagen: ``Er ist ein Ohr." Sag: (Er ist) ein Ohr des Guten für euch. Er glaubt an Allah und glaubt den Gläubigen, und (er ist) eine Barmherzigkeit für diejenigen von euch, die glauben. Für diejenigen aber, die Allahs Gesandtem Leid zufügen, wird es schmerzhafte Strafe geben.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mccxcvii{Sie schwören euch bei Allah, um euch zufriedenzustellen. Aber Allah - und (auch) Sein Gesandter - hat ein größeres Anrecht darauf, daß sie Ihn zufriedenstellen, wenn sie gläubig sind.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mccxcviii{Wissen sie denn nicht, daß es für denjenigen, der Allah und Seinem Gesandten zuwiderhandelt -, daß es für ihn das Feuer der Hölle gibt, ewig darin zu bleiben? Das ist die gewaltige Schande.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mccxcix{Die Heuchler fürchten, daß eine Sura über sie offenbart werden könnte', die ihnen kundtut, was in ihren Herzen ist. Sag: Macht euch (nur) lustig! Allah wird herausbringen, was ihr fürchtet.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mccc{Und wenn du sie fragst, werden sie ganz gewiß sagen: ``Wir haben nur (schweifende) Gespräche geführt und gescherzt." Sag: Habt ihr euch denn über Allah und Seine Zeichen und Seinen Gesandten lustig gemacht?\qt@no{65}}%
\qt@newcmd\qurantrans@de@mccci{Entschuldigt euch nicht! Ihr seid ja ungläubig geworden, nachdem ihr den Glauben (angenommen) hattet. Wenn Wir (auch) einem Teil von euch verzeihen, so strafen Wir einen (anderen) Teil (dafür), daß sie Übeltäter waren.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mcccii{Die Heuchler und die Heuchlerinnen stammen voneinander. Sie gebieten das Verwerfliche und verbieten das Rechte und halten ihre Hände geschlossen. Sie haben Allah vergessen, und so hat Er sie vergessen. Gewiß, die Heuchler sind die (wahren) Frevler.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mccciii{Allah hat den Heuchlern und den Heuchlerinnen und den Ungläubigen das Feuer der Hölle versprochen, ewig darin zu bleiben. Es ist ihre Genüge. Und Allah hat sie verflucht, und für sie gibt es beständige Strafe.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mccciv{Es ist) wie mit denen, die vor euch waren. Sie hatten stärkere Kraft als ihr und mehr Besitz und Kinder. Sie genossen ihren Anteil, und dann habt ihr euren Anteil genossen, wie diejenigen, die vor euch waren, ihren Anteil genossen haben. Und ihr habt (schweifende) Gespräche geführt wie die Gespräche, die sie führten. Deren Werke werden im Diesseits und im Jenseits hinfällig. Das sind die Verlierer.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mcccv{Ist ihnen nicht die Kunde von denjenigen zugekommen, die vor ihnen waren, des Volkes Nuhs, der 'Ad und der Tamud, des Volkes Ibrahims, der Bewohner von Madyan und der umgestürzten Städte? Ihre Gesandten kamen zu ihnen mit den klaren Beweisen. Nimmer ist es Allah, der ihnen Unrecht getan hat, sondern sie selbst haben sich Unrecht zugefügt.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mcccvi{Die gläubigen Männer und Frauen sind einer des anderen Beschützer. Sie gebieten das Rechte und verbieten das Verwerfliche, verrichten das Gebet und entrichten die Abgabe und gehorchen Allah und Seinem Gesandten. Sie sind es, derer Allah Sich erbarmen wird. Gewiß, Allah ist Allmächtig und Allweise.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mcccvii{Allah hat den gläubigen Männern und Frauen Gärten versprochen, durcheilt von Bächen, ewig darin zu bleiben, und gute Wohnungen in den Gärten Edens. Wohlgefallen von Allah ist aber (noch) größer. Das ist der großartige Erfolg.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mcccviii{O Prophet, mühe dich gegen die Ungläubigen und Heuchler ab und sei hart gegen sie! Ihr Zufluchtsort wird die Hölle sein - ein schlimmer Ausgang!\qt@no{73}}%
\qt@newcmd\qurantrans@de@mcccix{Sie schwören bei Allah, sie hätten (es) nicht gesagt. Aber sie haben ja das Wort des Unglaubens gesagt und sind, nachdem sie den Islam (angenommen) hatten, ungläubig geworden. Sie hatten vor (, das auszuführen), was sie (doch) nicht erreicht haben'. Und sie grollten darüber nur, daß Allah - und (auch) Sein Gesandter - sie von Seiner Huld reich gemacht hat. Wenn sie nun bereuen, ist es besser für sie. Wenn sie sich aber abkehren, wird Allah sie mit einer schmerzhaften Strafe im Diesseits und Jenseits strafen, und sie werden auf der Erde weder Schutzherrn noch Helfer haben.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mcccx{Unter ihnen gibt es manche, die gegenüber Allah eine Verpflichtung eingegangen sind: Wenn Er uns etwas von Seiner Huld gewährt, werden wir ganz gewiß Almosen geben und ganz gewiß zu den Rechtschaffenen gehören.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mcccxi{Als Er ihnen aber von Seiner Huld gewährt hatte, geizten sie damit, und sie kehrten sich widerstrebend ab.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mcccxii{So setzte Er als Folge davon in ihre Herzen Heuchelei bis zum Tag, an dem sie Ihm begegnen werden, dafür, daß sie gegenüber Allah brachen, was sie Ihm versprochen hatten, und daß sie zu lügen pflegten.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mcccxiii{Wissen sie denn nicht, daß Allah ihr Geheimes und ihre vertraulichen Gespräche kennt und daß Allah der Allwisser der verborgenen Dinge ist?\qt@no{78}}%
\qt@newcmd\qurantrans@de@mcccxiv{Diejenigen, die gegen die Freiwilligen unter den Gläubigen wegen der Almosen verhöhnen und (auch) gegen diejenigen, die nichts als ihre Mühe (als Leistung zu erbringen) finden, und dann über sie spotten - Allah spottet über sie, und für sie wird es schmerzhafte Strafe geben.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mcccxv{Bitte um Vergebung für sie, oder bitte nicht um Vergebung für sie; wenn du (auch) siebzigmal um Vergebung für sie bittest, so wird Allah ihnen doch nicht vergeben. Dies (geschieht deshalb), weil sie Allah und Seinen Gesandten verleugnet haben. Allah leitet das Volk der Frevler nicht recht.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mcccxvi{Die Zurückgelassenen waren froh darüber, daß sie hinter Allahs Gesandtem (daheim) sitzen geblieben sind, und es war ihnen zuwider, sich mit ihrem Besitz und mit ihrer eigenen Person auf Allahs Weg abzumühen. Und sie sagten: ``Rückt nicht in der Hitze aus!" Sag: Das Feuer der Hölle ist noch heißer; wenn sie (es doch) verstehen würden!\qt@no{81}}%
\qt@newcmd\qurantrans@de@mcccxvii{So sollen sie ein wenig lachen und sollen häufig weinen als Lohn für das, was sie erworben haben.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mcccxviii{Wenn Allah dich zu einem Teil von ihnen zurückkehren läßt und sie dich um Erlaubnis bitten, hinauszuziehen, dann sag: Ihr werdet niemals mit mir hinausziehen, und ihr werdet mit mir gegen keinen Feind kämpfen. Ihr seid (ja) das erste Mal damit zufrieden gewesen, (daheim) sitzen zu bleiben. Bleibt nun (daheim) sitzen mit denen, die zurückbleiben!\qt@no{83}}%
\qt@newcmd\qurantrans@de@mcccxix{Und bete niemals über jemandem von ihnen, der gestorben ist, und stehe nicht an seinem Grab! Sie verleugneten ja Allah und Seinen Gesandten, und sie starben als Frevler.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mcccxx{Ihr Besitz und ihre Kinder sollen dir nicht gefallen. Allah will sie ja nur im Diesseits damit strafen, und (daß) ihre Seelen (im Tod) dahingehen, während sie ungläubig sind.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mcccxxi{Und wenn eine Sura (als Offenbarung) hinabgesandt wird: ``Glaubt an Allah und müht euch (zusammen) mit Seinem Gesandten ab", (dann) bitten dich die Bemittelten unter ihnen um Erlaubnis und sagen: ``Lasse uns mit denen (zusammen) sein, die (daheim) sitzen bleiben!"\qt@no{86}}%
\qt@newcmd\qurantrans@de@mcccxxii{Sie waren damit zufrieden, (zusammen) mit den zurückbleibenden Frauen zu sein, und ihre Herzen wurden versiegelt; so verstehen sie nicht.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mcccxxiii{Aber der Gesandte und diejenigen, die mit ihm glaubten, mühten sich mit ihrem Besitz und ihrer eigenen Person ab. Das sind die, für die es die guten Dinge geben wird, und das sind diejenigen, denen es wohl ergeht.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mcccxxiv{Allah hat für sie Gärten bereitet, durcheilt von Bächen', ewig darin zu bleiben; das ist der großartige Erfolg.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mcccxxv{Und diejenigen von den Wüstenarabern, die sich entschuldigen wollten, kamen (her), um Erlaubnis zu erhalten. Und (daheim) blieben diejenigen sitzen, die Allah und Seinen Gesandten belogen. Treffen wird diejenigen von ihnen, die ungläubig sind, schmerzhafte Strafe.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mcccxxvi{Kein Grund zur Bedrängnis (wegen des Daheimbleibens) ist es für die Schwachen, für die Kranken und für diejenigen, die nichts finden, was sie ausgeben (können), wenn sie sich gegenüber Allah und Seinem Gesandten aufrichtig verhalten. Gegen die Gutes Tuenden gibt es keine Möglichkeit (, sie zu belangen) - Allah ist Allvergebend und Barmherzig -,\qt@no{91}}%
\qt@newcmd\qurantrans@de@mcccxxvii{und (auch) nicht gegen diejenigen, die, als sie zu dir kamen, damit du ihnen gäbest, (worauf) sie mitreiten (könnten), und du sagtest: ``Ich finde nichts, worauf ich euch mitreiten lassen kann", sich abkehrten, während ihre Augen von Tränen überflossen aus Traurigkeit darüber, daß sie nichts fanden, was sie ausgeben (konnten).\qt@no{92}}%
\qt@newcmd\qurantrans@de@mcccxxviii{Eine Möglichkeit (zu belangen) gibt es nur gegen diejenigen, die dich um Erlaubnis bitten, obwohl sie reich sind. Sie waren damit zufrieden, (zusammen) mit den zurückbleibenden Frauen zu sein. Allah hat ihre Herzen versiegelt, so daß sie nicht Bescheid wissen.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mcccxxix{Sie werden sich bei euch entschuldigen, wenn ihr zu ihnen zurückkehrt. Sag: Entschuldigt euch nicht! Wir werden euch (doch) nicht glauben. Allah hat uns ja von den Nachrichten über euch kundgetan. Allah wird euer Tun sehen und (auch) Sein Gesandter. Hierauf werdet ihr zum Kenner des Verborgenen und des Offenbaren zurückgebracht werden, und dann wird Er euch kundtun, was ihr zu tun pflegtet.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mcccxxx{Sie werden euch bei Allah schwören, wenn ihr zu ihnen zurückgekehrt seid, damit ihr von ihnen ablaßt. So laßt von ihnen ab, denn sie sind ein Greuel! Ihr Zufluchtsort ist die Hölle als Lohn für das, was sie erworben haben.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mcccxxxi{Sie schwören euch, damit ihr mit ihnen zufrieden seid. Und wenn ihr auch mit ihnen zufrieden seid, so ist Allah doch nicht zufrieden mit dem Volk der Frevler.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mcccxxxii{Die Wüstenaraber sind mehr ungläubig und heuchlerischer, und es paßt eher zu ihnen, daß sie die Grenzen dessen, was Allah auf Seinen Gesandten (als Offenbarung) herabgesandt hat, nicht kennen. Und Allah ist Allwissend und Allweise.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcccxxxiii{Unter den Wüstenarabern gibt es (auch) manche, die das, was sie (als Spende) ausgeben, als erzwungene Zahlung ansehen und für euch die Schicksalswendungen abwarten. Gegen sie wird die böse Schicksalswendung sein. Allah ist Allhörend und Allwissend.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mcccxxxiv{Und unter den Wüstenarabern gibt es (aber auch) manche, die an Allah und den Jüngsten Tag glauben und das, was sie (als Spende) ausgeben, als Mittel der Annäherung zu Allah ansehen, und (auch) die Gebete des Gesandten. Es ist sicherlich für sie ein Mittel der Annäherung (zu Allah). Allah wird sie in Seine Barmherzigkeit eingehen lassen. Gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mcccxxxv{Die vorausgeeilten Ersten von den Auswanderern und den Helfern und diejenigen, die ihnen auf beste Weise gefolgt sind - Allah hat Wohlgefallen an ihnen, und sie haben Wohlgefallen an Ihm. Und Er hat für sie Gärten bereitet, durcheilt von Bächen, ewig und auf immer darin zu bleiben; das ist der großartige Erfolg.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mcccxxxvi{Unter den Wüstenarabern in eurer Umgebung gibt es Heuchler und (auch) unter den Bewohnern von al-Madina, die in der Heuchelei geübt sind. Du kennst sie nicht, aber Wir kennen sie. Wir werden sie zweimal strafen, und hierauf werden sie einer gewaltigen Strafe zugeführt werden.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mcccxxxvii{Und (es gibt) andere, die ihre Sünden bekennen. Sie haben eine rechtschaffene Tat mit einer anderen bösen vermischt. Vielleicht wird Allah ihre Reue annehmen. Gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mcccxxxviii{Nimm von ihrem Besitz ein Almosen, mit dem du sie rein machst und läuterst, und bete für sie, denn dein Gebet ist für sie eine Beruhigung! Allah ist Allhörend und Allwissend.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mcccxxxix{Wissen sie denn nicht, daß Allah es ist, Der die Reue von Seinen Dienern annimmt und die Almosen nimmt, und daß Allah der Reue-Annehmende und Barmherzige ist?\qt@no{104}}%
\qt@newcmd\qurantrans@de@mcccxl{Und sag: Wirkt! Allah wird euer Tun sehen, und (auch) Sein Gesandter und die Gläubigen. Und ihr werdet zum Kenner des Verborgenen und des Offenbaren zurückgebracht werden, und dann wird Er euch kundtun, was ihr zu tun pflegtet.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mcccxli{Und (wieder) andere werden zurückgestellt, bis die Anordnung Allahs (ergeht). Entweder straft Er sie, oder Er nimmt ihre Reue an. Allah ist Allwissend und Allweise.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mcccxlii{Und (es) gibt (auch) diejenigen, die sich eine Gebetsstätte genommen haben in der Absicht der Schädigung und aus Unglauben und zur Spaltung der Gläubigen und zur Beobachtung für denjenigen, der zuvor gegen Allah und Seinen Gesandten Krieg geführt hat. Sie werden ganz gewiß schwören: ``Wir haben nur das Beste gewollt." Doch Allah bezeugt, daß sie fürwahr Lügner sind.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mcccxliii{Stell dich niemals in ihr (zum Gebet) hin! Eine Gebetsstätte, die vom ersten Tag an auf die Gottesfurcht gegründet worden ist, hat wahrlich ein größeres Anrecht darauf, daß du dich in ihr hinstellst. In ihr sind Männer, die es lieben, sich zu reinigen. Und Allah liebt die sich Reinigenden.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mcccxliv{Ist derjenige, der seinen Bau auf Furcht vor Allah und (Sein) Wohlgefallen gegründet hat, besser oder der, der seinen Bau auf den Rand eines abstürzenden Hanges gegründet hat, so daß er mit ihm ins Feuer der Hölle abstürzt? Und Allah leitet das ungerechte Volk nicht recht.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mcccxlv{Ihr Bau, den sie errichtet haben, wird nicht aufhören, Zweifel (hervorzurufen), bis ihre Herzen in Stücke gerissen sind. Allah ist Allwissend und Allweise.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mcccxlvi{Allah hat von den Gläubigen ihre eigene Person und ihren Besitz dafür erkauft, daß ihnen der (Paradies)garten gehört: Sie kämpfen auf Allahs Weg, und so töten sie und werden getötet. (Das ist) ein für Ihn bindendes Versprechen in Wahrheit in der Tora, dem Evangelium und dem Qur'an. Und wer ist treuer in (der Einhaltung) seiner Abmachung als Allah? So freut euch über das Kaufgeschäft, das ihr abgeschlossen habt, denn das ist der großartige Erfolg!\qt@no{111}}%
\qt@newcmd\qurantrans@de@mcccxlvii{... für die Gläubigen,) die in Reue Umkehrenden, die (Allah) Dienenden, die Lobenden, die (auf Allahs Weg) Umherziehenden, die sich Verbeugenden, die sich Niederwerfenden, die das Rechte Gebietenden und das Verwerfliche Verbietenden, die Allahs Grenzen Beachtenden ... und verkünde den Gläubigen frohe Botschaft.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mcccxlviii{Dem Propheten und denjenigen, die glauben, steht es nicht zu, für die Götzendiener um Vergebung zu bitten, auch wenn es Verwandte wären, nachdem es ihnen klargeworden ist, daß sie Insassen des Höllenbrandes sein werden.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mcccxlix{Und die Bitte Ibrahims um Vergebung für seinen Vater war nur wegen eines Versprechens, das er ihm gegeben hatte. Als es ihm aber klarwurde, daß er ein Feind Allahs war, sagte er sich von ihm los. Ibrahim war fürwahr weichherzig und nachsichtig.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mcccl{Nimmer wird Allah Leute in die Irre gehen lassen, nachdem Er sie rechtgeleitet hat, bis Er ihnen darüber Klarheit gegeben hat, wovor sie sich hüten sollen. Gewiß, Allah weiß über alles Bescheid.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mcccli{Allah hat die Herrschaft über die Himmel und die Erde. Er macht lebendig und läßt sterben. Ihr habt außer Allah weder Schutzherrn noch Helfer.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mccclii{Allah hat die Reue des Propheten, der Auswanderer und der Helfer angenommen, die ihm in der Stunde der Bedrängnis folgten, nachdem die Herzen einer Gruppe von ihnen beinahe abgeschweift wären. Hierauf hat Er ihre Reue angenommen gewiß, Er ist zu ihnen Gnädig und Barmherzig -,\qt@no{117}}%
\qt@newcmd\qurantrans@de@mcccliii{und (die Reue) der Dreien, die zurückgelassen wurden, bis die Erde ihnen eng wurde bei all ihrer Weite und ihre Seelen ihnen eng wurden und sie wußten, daß es vor Allah keine (andere) Zuflucht gibt als zu Ihm. Hierauf wandte Er Sich ihnen verzeihend zu, damit sie bereuen. Gewiß, Allah ist der Reue-Annehmende und Barmherzige.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mcccliv{O ihr, die ihr glaubt, fürchtet Allah und seid mit den Wahrhaftigen!\qt@no{119}}%
\qt@newcmd\qurantrans@de@mccclv{Es steht den Bewohnern von al-Madina und den Wüstenarabern in ihrer Umgebung nicht zu, hinter dem Gesandten Allahs zurückzubleiben und sich selbst (und ihr Leben) ihm vorzuziehen. Dies (ist), weil sie weder Durst noch Mühsal noch Hunger auf Allahs Weg trifft, noch sie einen Schritt unternehmen, der die Ungläubigen ergrimmen läßt, noch einem Feind etwas Schlimmes zufügen, ohne daß ihnen dafür eine rechtschaffene Tat aufgeschrieben würde. Gewiß, Allah läßt den Lohn der Gutes Tuenden nicht verlorengehen.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mccclvi{Auch geben sie keine Spende aus, ob klein oder groß, noch durchqueren sie ein Tal, ohne daß es ihnen (als gute Tat) aufgeschrieben würde, damit Allah ihnen das Beste vergelte von dem, was sie getan haben.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mccclvii{Es steht den Gläubigen nicht zu, allesamt auszurücken. Wenn doch von jeder Gruppe von ihnen ein Teil ausrücken würde, um (mehr) von der Religion zu erlernen und um ihre Leute zu warnen, wenn sie zu ihnen zurückkehren, auf daß sie sich vorsehen mögen.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mccclviii{O die ihr glaubt, kämpft gegen diejenigen, die in eurer Nähe sind von den Ungläubigen! Sie sollen in euch Härte vorfinden. Und wisset, daß Allah mit den Gottesfürchtigen ist!\qt@no{123}}%
\qt@newcmd\qurantrans@de@mccclix{Wenn eine Sura (als Offenbarung) herabgesandt wird, dann gibt es unter ihnen manche, die sagen: ``Wem von euch hat dies(e Sura) seinen Glauben vermehrt?" Was nun diejenigen angeht, die glauben, so hat sie ihren Glauben vermehrt, und sie freuen sich über die frohe Botschaft.\qt@no{124}}%
\qt@newcmd\qurantrans@de@mccclx{Was aber diejenigen angeht, in deren Herzen Krankheit ist, so fügt sie ihrem Greuel noch (weiteren) Greuel hinzu, und sie sterben als Ungläubige.\qt@no{125}}%
\qt@newcmd\qurantrans@de@mccclxi{Sehen sie denn nicht, daß sie in jedem Jahr einmal oder zweimal einer Prüfung unterzogen werden? Aber hierauf bereuen sie nicht, und sie bedenken nicht.\qt@no{126}}%
\qt@newcmd\qurantrans@de@mccclxii{Wenn eine Sura (als Offenbarung) herabgesandt wird, schauen sie einander an: ``Sieht euch irgend jemand?" Hierauf wenden sie sich ab. Allah hat ihre Herzen abgewendet, weil sie Leute sind, die nicht verstehen.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mccclxiii{Zu euch ist nunmehr ein Gesandter aus euren eigenen Reihen gekommen. Bedrückend ist es für ihn, wenn ihr in Bedrängnis seid, (er ist) eifrig um euch bestrebt, zu den Gläubigen gnadenvoll und barmherzig.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mccclxiv{Wenn sie sich aber abkehren, dann sag: Meine Genüge ist Allah. Es gibt keinen Gott außer Ihm. Auf Ihn verlasse ich mich, und Er ist der Herr des gewaltigen Thrones.\qt@no{129}}%
\qt@newcmd\qurantrans@de@mccclxv{\basmalahde Alif-Lam-Ra. Dies sind die Zeichen des weisen Buches.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mccclxvi{Ist es denn für die Menschen verwunderlich, daß Wir einem Mann von ihnen (als Offenbarung) eingegeben haben: ``Warne die Menschen und verkünde denen, die glauben, die frohe Botschaft, daß sie bei ihrem Herrn einen wahrhaftigen Stand haben"? Die Ungläubigen sagen: ``Das ist wahrlich ein deutlicher Zauberer.3"\qt@no{2}}%
\qt@newcmd\qurantrans@de@mccclxvii{Gewiß, euer Herr ist Allah, Der die Himmel und die Erde in sechs Tagen erschuf und Sich hierauf über den Thron erhob. Er regelt die Angelegenheit. Es gibt keinen Fürsprecher außer nach Seiner Erlaubnis. Dies ist doch Allah, euer Herr, so dient Ihm! Bedenkt ihr denn nicht?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mccclxviii{Zu Ihm wird eure aller Rückkehr sein. (Das ist) Allahs Versprechen in Wahrheit. Er macht die Schöpfung am Anfang und wiederholt sie hierauf, um (es) denjenigen, die glauben und rechtschaffene Werke tun, in Gerechtigkeit zu vergelten. Für diejenigen, die ungläubig sind, wird es ein Getränk aus heißem Wasser und schmerzhafte Strafe geben dafür, daß sie ungläubig gewesen sind.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mccclxix{Er ist es, Der die Sonne zu einer Leuchte und den Mond zu einem Licht gemacht und ihm Himmelspunkte zugemessen hat, damit ihr die Zahl der Jahre und die (Zeit)rechnung wißt. Allah hat dies ja nur in Wahrheit erschaffen. Er legt die Zeichen ausführlich dar für Leute, die Bescheid wissen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mccclxx{Gewiß, in dem Unterschied von Nacht und Tag und in dem, was Allah in den Himmeln und auf der Erde erschaffen hat, sind wahrlich Zeichen für Leute, die gottesfürchtig sind.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mccclxxi{Diejenigen nun, die nicht die Begegnung mit Uns erwarten und mit dem diesseitigen Leben zufrieden sind und darin Ruhe finden, und die gegenüber Unseren Zeichen unachtsam sind,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mccclxxii{deren Zufluchtsort wird das (Höllen)feuer sein für das, was sie erworben haben.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mccclxxiii{Diejenigen aber, die glauben und rechtschaffene Werke tun, leitet ihr Herr wegen ihres Glaubens recht. Unter ihnen werden Flüsse strömen in den Gärten der Wonne.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mccclxxiv{Ihr Ausruf darin wird sein: ``Preis sei Dir, o Allah!" und ihr Gruß darin: ``Friede!" und ihr abschließender Ausruf: ``(Alles) Lob gehört Allah, dem Herrn der Weltenbewohner!"\qt@no{10}}%
\qt@newcmd\qurantrans@de@mccclxxv{Wenn Allah den Menschen das Schlechte so beschleunigen würde, wie sie das Gute zu beschleunigen wünschen, wäre ihre Frist für sie bereits bestimmt. Aber Wir lassen diejenigen, die nicht die Begegnung mit Uns erwarten, in ihrer Auflehnung umherirren.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mccclxxvi{Und wenn dem Menschen Unheil widerfährt, ruft er Uns auf der Seite (liegend), im Sitzen oder Stehen an. Wenn Wir ihm aber sein Unheil hinweggenommen haben, geht er vorbei, als hätte er Uns nicht wegen eines Unheils, das ihm widerfahren ist, angerufen. So ist den Maßlosen ausgeschmückt, was sie tun.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mccclxxvii{Wir haben bereits die Geschlechter vor euch vernichtet, als sie Unrecht taten, als ihre Gesandten mit den klaren Beweisen zu ihnen kamen, und sie nicht glauben mochten. So vergelten Wir dem übeltätigen Volk.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mccclxxviii{Hierauf haben Wir euch nach ihnen zu Nachfolgern (auf) der Erde gemacht, um zu schauen, wie ihr handeln würdet.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mccclxxix{Wenn ihnen Unsere Zeichen als klare Beweise verlesen werden, sagen diejenigen, die nicht die Begegnung mit Uns erwarten: ``Bringe einen anderen Qur'an als diesen oder ändere ihn ab." Sag: Es steht mir nicht zu, ihn von mir selbst aus abzuändern. Ich folge nur dem, was mir (als Offenbarung) eingegeben wird. Gewiß, ich fürchte, wenn ich mich meinem Herrn widersetze, die Strafe eines gewaltigen Tages.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mccclxxx{Sag: Wenn Allah wollte, würde ich ihn euch nicht verlesen, und Er würde euch nicht davon Kenntnis geben. Ich habe doch vordem ein Leben lang unter euch verweilt. Begreift ihr denn nicht?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mccclxxxi{Wer ist ungerechter, als wer gegen Allah eine Lüge ersinnt oder Seine Zeichen für Lüge erklärt? Gewiß, den Übeltätern wird es nicht wohl ergehen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mccclxxxii{Sie dienen anstatt Allahs, was ihnen weder schadet noch nützt, und sagen: ``Das sind unsere Fürsprecher bei Allah." Sag: Wollt ihr denn Allah kundtun, was Er nicht kennt, weder in den Himmeln noch auf der Erde? Preis sei Ihm! Erhaben ist Er über das, was sie (ihm) beigesellen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mccclxxxiii{Die Menschen waren nur eine einzige Gemeinschaft. Dann wurden sie uneinig. Wenn es nicht ein früher ergangenes Wort von deinem Herrn gäbe, so wäre zwischen ihnen wahrlich entschieden worden über das, worüber sie uneinig sind.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mccclxxxiv{Und sie sagen: ``Wenn doch ein Zeichen von seinem Herrn auf ihn herabgesandt würde!" Sag: Das Verborgene ist nur Allahs. So wartet ab! Ich gehöre mit euch zu denjenigen, die abwarten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mccclxxxv{Wenn Wir die Menschen Barmherzigkeit kosten lassen nach Leid, das ihnen widerfuhr, schmieden sie gleich Ränke gegen Unsere Zeichen. Sag: Allah schmiedet noch schneller Ränke. Gewiß, Unsere Gesandten schreiben auf, was ihr an Ränken schmiedet.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mccclxxxvi{Er ist es, Der euch auf dem Festland und dem Meer reisen läßt. Wenn ihr dann auf den Schiffen seid und diese mit ihnen bei einem guten Wind dahinfahren und sie froh darüber sind, ein stürmischer Wind darüber kommt, die Wogen von überall über sie kommen, und sie meinen, daß sie rings umgeben werden, (dann) rufen sie Allah an, (wobei sie) Ihm gegenüber aufrichtig in der Religion (sind): ``Wenn Du uns nur hieraus rettest, werden wir ganz gewiß zu den Dankbaren gehören!"\qt@no{22}}%
\qt@newcmd\qurantrans@de@mccclxxxvii{Aber wenn Er sie gerettet hat, fangen sie sogleich an, ohne Recht auf der Erde gewalttätig zu handeln. O ihr Menschen, eure Gewalttätigkeit richtet sich doch nur gegen euch selbst. (Es ist doch nur) der Genuß des diesseitigen Lebens. Hierauf wird eure Rückkehr zu Uns sein, und Wir werden euch kundtun, was ihr zu tun pflegtet.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mccclxxxviii{Das Gleichnis des diesseitigen Lebens ist nur wie Wasser, das Wir vom Himmel hinabsenden, worauf das Gewächs der Erde, von dem die Menschen und das Vieh verzehren, sich damit vermischt, bis dann, wenn die Erde ihren Prunk angenommen hat und sich geschmückt hat und ihre Bewohner meinen, daß sie Macht über sie hätten, kommt Unser Befehl über sie" bei Nacht oder bei Tag, und da lassen Wir sie abgemäht sein, als ob sie am Tag zuvor nicht in Blüte gestanden hätte. So legen Wir die Zeichen ausführlich dar für Leute, die nachdenken.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mccclxxxix{Allah lädt zur Wohnstätte des Friedens ein und leitet, wen Er will, zu einem geraden Weg.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcccxc{Für diejenigen, die Gutes tun, gibt es das Beste (an Lohn) und noch mehr. Ihre Gesichter werden weder von Dunkelheit noch Erniedrigung bedeckt. Das sind die Insassen des (Paradies)gartens; ewig werden sie darin bleiben.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mcccxci{Für diejenigen aber, die böse Taten erwerben, ist der Lohn einer bösen Tat ein Gleiches, und Erniedrigung wird sie bedecken - sie haben vor Allah nichts, das sie schützen könnte -, als ob ihre Gesichter von Stücken finsterer Nacht überdeckt wären. Das sind die Insassen des (Höllen)feuers; ewig werden sie darin bleiben.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mcccxcii{Und am Tag, da Wir sie alle versammeln, und dann werden Wir zu denen, die (Allah etwas) beigesellt haben, sagen: ``(Bleibt) an eurem Platz, ihr und eure Teilhaber!" Wir trennen sie dann voneinander. Ihre Teilhaber sagen: ``Nicht uns habt ihr gedient.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mcccxciii{Allah genügt als Zeuge zwischen uns und euch. Wir waren gegenüber eurem Dienst (an uns) fürwahr unachtsam."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcccxciv{Dort wird jede Seele über das, was sie früher getan hat, nachgeprüft. Und sie werden zu Allah, ihrem wahren Schutzherrn, zurückgebracht, und entschwunden ist ihnen (dann), was sie zu ersinnen pflegten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcccxcv{Sag: Wer versorgt euch vom Himmel und von der Erde, oder wer verfügt über Gehör und Augenlicht? Und wer bringt das Lebendige aus dem Toten und bringt das Tote aus dem Lebendigen hervor? Und wer regelt die Angelegenheit? Sie werden sagen: ``Allah." Sag: Wollt ihr denn nicht gottesfürchtig sein?\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcccxcvi{Dies ist doch Allah, euer wahrer Herr. Was gibt es denn über die Wahrheit hinaus außer dem Irrtum? Wie laßt ihr euch also abwenden?\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcccxcvii{So hat sich das Wort deines Herrn bewahrheitet gegen die, die freveln, weil sie nicht glauben.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcccxcviii{Sag: Ist unter euren Teilhabern jemand, der die Schöpfung am Anfang macht und sie hierauf wiederholt? Sag: Allah macht die Schöpfung am Anfang und wiederholt sie hierauf. Wie laßt ihr euch also abwendig machen?\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcccxcix{Sag: Ist unter euren Teilhabern jemand, der zur Wahrheit leitet? Sag: Allah leitet zur Wahrheit. Hat jemand, der zur Wahrheit leitet, ein größeres Anrecht darauf, daß man ihm folgt, oder jemand, der nur (dann) die Rechtleitung findet, wenn er (selbst) rechtgeleitet wird? Was ist denn mit euch? Wie urteilt ihr?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcd{Und die meisten von ihnen folgen nur Mutmaßungen. Aber Mutmaßungen nützen nichts gegenüber der Wahrheit. Gewiß, Allah weiß Bescheid über das, was sie tun.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcdi{Dieser Qur'an kann unmöglich ohne Allah ersonnen werden. Sondern (er ist) die Bestätigung dessen, was vor ihm war, und die ausführliche Darlegung des Buches, an dem es keinen Zweifel gibt, vom Herrn der Weltenbewohner.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcdii{Oder sagen sie: ``Er hat ihn ersonnen" Sag: Dann bringt eine Sura bei, die ihm gleich ist, und ruft an, wen ihr könnt, anstatt Allahs, wenn ihr wahrhaftig seid.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcdiii{``Nein! Vielmehr erklären sie das für Lüge, wovon sie kein umfassendes Wissen haben, und schon bevor seine Deutung zu ihnen gekommen ist. So haben es auch diejenigen, die vor ihnen waren, für Lüge erklärt. Schau, wie das Ende der Ungerechten war!\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcdiv{Und unter ihnen gibt es manche, die an ihn glauben, und unter ihnen gibt es manche, die nicht an ihn glauben. Dein Herr weiß sehr wohl über die Unheilstifter Bescheid.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcdv{Und wenn sie dich der Lüge bezichtigen, dann sag: Für mich ist mein Tun, und für euch ist euer Tun. Ihr seid unschuldig an dem, was ich tue; und ich bin unschuldig an dem, was ihr tut.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mcdvi{Und unter ihnen gibt manche, die dir zuhören. Kannst du aber die Tauben hören lassen, auch wenn sie nicht begreifen (wollen)?\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcdvii{Und unter ihnen gibt es manche, die auf dich schauen. Kannst du aber die Blinden rechtleiten, auch wenn sie nicht sehen?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcdviii{Gewiß, Allah fügt den Menschen kein Unrecht zu, sondern die Menschen fügen sich selbst Unrecht zu.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mcdix{Und an dem Tag, da Er sie versammelt', wird (ihnen) sein, als ob sie nur eine Stunde vom Tag verweilt hätten, und sie erkennen einander wieder. Verloren sind dann diejenigen, die die Begegnung mit Allah für Lüge erklärten und nicht rechtgeleitet waren.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mcdx{Ob Wir dich nun einen Teil dessen, was Wir ihnen androhen, sehen lassen oder dich (vorher) abberufen, zu Uns wird (auf jeden Fall) ihre Rückkehr sein. Hierauf ist Allah Zeuge über das, was sie tun.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mcdxi{Jede Gemeinschaft hat einen Gesandten. Wenn nun ihr Gesandter kommt, wird zwischen ihnen in Gerechtigkeit entschieden, und es wird ihnen kein Unrecht zugefügt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mcdxii{Und sie sagen: ``Wann wird dieses Versprechen eintreten, wenn ihr wahrhaftig seid?"\qt@no{48}}%
\qt@newcmd\qurantrans@de@mcdxiii{Sag: Ich vermag mir selbst weder Schaden noch Nutzen (zu bringen), außer was Allah will. Jede Gemeinschaft hat eine (festgesetzte) Frist. Und wenn ihre Frist kommt, können sie (sie) weder um eine Stunde hinausschieben noch vorverlegen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mcdxiv{Sag: Was meint ihr, wenn Seine Strafe bei Nacht oder bei Tag (über) euch kommt, was werden die Übeltäter davon vorzuverlegen wünschen?\qt@no{50}}%
\qt@newcmd\qurantrans@de@mcdxv{Werdet ihr dann, wenn sie hereinbricht, daran glauben? Wie, erst jetzt, und dabei habt ihr sie doch vorzuverlegen gewünscht?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mcdxvi{Dann wird zu denen, die Unrecht getan haben, gesagt werden: ``Kostet die ewige Strafe! Wird euch denn etwas anderes vergolten als das, was ihr verdient habt?"\qt@no{52}}%
\qt@newcmd\qurantrans@de@mcdxvii{Und sie erkundigen sich bei dir: ``Ist es wahr?" Sag: Ja, bei meinem Herrn! Es ist bestimmt wahr, und ihr werdet euch (Ihm) nicht entziehen können.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mcdxviii{Und wenn jede Seele, die Unrecht getan hat, das besäße, was auf der Erde ist, würde sie sich wahrlich damit loskaufen. Sie halten Reue geheim, wenn sie die Strafe sehen. Und es wird in Gerechtigkeit zwischen ihnen entschieden, und es wird ihnen kein Unrecht zugefügt.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mcdxix{Sicherlich, Allah gehört (alles), was in den Himmeln und auf der Erde ist. Sicherlich, Allahs Versprechen ist wahr. Aber die meisten von ihnen wissen nicht.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mcdxx{Er macht lebendig und läßt sterben. Und zu Ihm werdet ihr zurückgebracht.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mcdxxi{O ihr Menschen, zu euch ist nunmehr eine Ermahnung von eurem Herrn gekommen und eine Heilung für das, was in den Brüsten ist, eine Rechtleitung und Barmherzigkeit für die Gläubigen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mcdxxii{Sag: Über die Huld Allahs und über Seine Barmherzigkeit, ja darüber sollen sie froh sein. Das ist besser als das, was sie zusammentragen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mcdxxiii{Sag: Was meint ihr zu dem, was Allah für euch an Versorgung herabgesandt hat und was ihr dann als Verbotenes und Erlaubtes festgelegt habt, - sag: Hat Allah es euch tatsächlich erlaubt, oder ersinnt ihr etwas gegen Allah?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mcdxxiv{Was wird die Meinung derer, die gegen Allah Lügen ersinnen, am Tag der Auferstehung sein? Allah ist wahrlich voll Huld gegen die Menschen. Aber die meisten von ihnen sind nicht dankbar.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mcdxxv{Und mit keiner Angelegenheit befaßt Du dich, und nichts verliest du davon an Qur'an, und keine Tat vollbringt ihr, ohne daß Wir über euch Zeugen sind, wenn ihr euch (ausgiebig) darüber auslaßt. Und es entgeht deinem Herrn nicht das Gewicht eines Stäubchens, weder auf der Erde noch im Himmel, und nichts Kleineres als dies oder Größeres; (es gibt) nichts, das nicht in einem deutlichen Buch (verzeichnet) wäre.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mcdxxvi{Sicherlich, über Allahs Gefolgsleute soll keine Furcht kommen, noch sollen sie traurig sein,\qt@no{62}}%
\qt@newcmd\qurantrans@de@mcdxxvii{diejenigen, die glauben und gottesfürchtig sind.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mcdxxviii{Für sie ist die frohe Botschaft im diesseitigen Leben und im Jenseits. Keine Abänderung gibt es für die Worte Allahs. Das ist der großartige Erfolg.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mcdxxix{Ihre Worte sollen dich nicht traurig machen. Gewiß, alle Macht gehört Allah. Er ist der Allhörende und Allwissende.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mcdxxx{Sicherlich, Allah gehört, wer in den Himmeln und wer auf der Erde ist. Es folgen diejenigen, die anstatt Allahs Teilhaber anrufen (ihren falschen Göttern); sie folgen nur Mutmaßungen, und sie stellen nur Schätzungen an.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mcdxxxi{Er ist es, Der euch die Nacht gemacht hat, damit ihr in ihr ruht, und den Tag hell'. Darin sind wahrlich Zeichen für Leute, die hören.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mcdxxxii{Sie sagen: ``Allah hat Sich Kinder genommen." Preis sei Ihm! Er ist der Unbedürftige. Ihm gehört, was in den Himmeln und was auf der Erde ist. Keine Ermächtigung habt ihr dafür. Wollt ihr über Allah etwas sagen, was ihr nicht wißt?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mcdxxxiii{Sag: Gewiß, denjenigen, die gegen Allah Lügen ersinnen, wird es nicht wohl ergehen.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mcdxxxiv{Für sie ist) im Diesseits Nießbrauch, hierauf wird ihre Rückkehr zu Uns sein. Hierauf lassen Wir sie die strenge Strafe kosten dafür, daß sie ungläubig waren.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mcdxxxv{Und verlies ihnen die Kunde von Nuh, als er zu seinem Volk sagte: ``O mein Volk, wenn euch mein Stand (unter euch) und meine Ermahnung mit Allahs Zeichen schwer ankommt, so verlasse ich mich auf Allah. So einigt euch über eure Angelegenheit, ihr und eure Teilhaber, und (der Entschluß in) eure(r) Angelegenheit soll für euch nicht unklar sein; hierauf führt (ihn) an mir aus und gewährt mir keinen Aufschub.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mcdxxxvi{Doch wenn ihr euch abkehrt, so habe ich von euch keinen Lohn verlangt. Mein Lohn obliegt nur Allah. Und mir ist befohlen worden, einer der (Allah) Ergebenen zu sein ."\qt@no{72}}%
\qt@newcmd\qurantrans@de@mcdxxxvii{Sie bezichtigten ihn aber der Lüge. Da erretteten Wir ihn und diejenigen, die mit ihm waren, im Schiff. Und Wir machten sie zu Nachfolgern und ließen diejenigen ertrinken, die Unsere Zeichen für Lüge erklärten. So schau, wie das Ende der Gewarnten war!\qt@no{73}}%
\qt@newcmd\qurantrans@de@mcdxxxviii{Hierauf schickten Wir nach ihm Gesandte zu ihrem (jeweiligen) Volk; sie kamen mit den klaren Beweisen zu ihnen. Aber sie waren nicht bereit, an das zu glauben, was sie zuvor für Lüge erklärt hatten. So versiegeln Wir die Herzen der Übertreter.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mcdxxxix{Hierauf schickten Wir nach ihnen Musa und Harun mit Unseren Zeichen zu Fir'aun und seiner führenden Schar, aber sie verhielten sich hochmütig und waren ein Volk von Übeltätern.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mcdxl{Als nun die Wahrheit von Uns zu ihnen kam, sagten sie: ``Das ist wahrlich deutliche Zauberei."\qt@no{76}}%
\qt@newcmd\qurantrans@de@mcdxli{Musa sagte: ``Sagt ihr denn von der Wahrheit, nachdem sie zu euch gekommen ist: ,Ist das etwa Zauberei? Aber den Zauberern wird es nicht wohl ergehen."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mcdxlii{Sie sagten: ``Bist du zu uns gekommen, um uns von dem abzubringen, worin wir unsere Väter (vor)gefunden haben, und damit euch beiden die Oberhoheit im Land werde? Wir werden euretwegen nicht gläubig werden.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mcdxliii{Und Fir'aun sagte: ``Bringt mir jeden kenntnisreichen Zauberer herbei!"\qt@no{79}}%
\qt@newcmd\qurantrans@de@mcdxliv{Als die Zauberer kamen, sagte Musa zu ihnen: ``Werft hin, was ihr zu werfen habt."\qt@no{80}}%
\qt@newcmd\qurantrans@de@mcdxlv{Als sie geworfen hatten, sagte Musa: ``Was ihr vorgebracht habt, ist Zauberei. Allah wird sie zunichte machen. Gewiß, Allah läßt das Tun der Unheilstifter nicht als gut gelten.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mcdxlvi{Allah bestätigt die Wahrheit mit Seinen Worten, auch wenn es den Übeltätern zuwider ist."\qt@no{82}}%
\qt@newcmd\qurantrans@de@mcdxlvii{Aber es glaubten Musa nur junge Leute aus seinem Volk, trotz ihrer Furcht vor Fir'aun und ihrer führenden Schar, davor, daß er sie verfolgen würde. Fir'aun war ja überheblich im Land und gehörte wahrlich zu den Maßlosen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mcdxlviii{Und Musa sagte: ``O mein Volk, wenn ihr wirklich an Allah glaubt, dann verlaßt euch auf Ihn, wenn ihr (Ihm) Ergebene seid."\qt@no{84}}%
\qt@newcmd\qurantrans@de@mcdxlix{Sie sagten: ``Auf Allah verlassen wir uns. Unser Herr, mache uns nicht zu einer Versuchung für das ungerechte Volk,\qt@no{85}}%
\qt@newcmd\qurantrans@de@mcdl{und errette uns durch Deine Barmherzigkeit von dem ungläubigen Volk!"\qt@no{86}}%
\qt@newcmd\qurantrans@de@mcdli{Und Wir gaben Musa und seinem Bruder ein: ``Weiset eurem Volk in Ägypten Häuser zu und macht eure Häuser zu Gebetsstätten und verrichtet das Gebet. Und verkünde den Gläubigen frohe Botschaft."\qt@no{87}}%
\qt@newcmd\qurantrans@de@mcdlii{Und Musa sagte: ``Unser Herr, Du hast ja Fir'aun und seiner führenden Schar im diesseitigen Leben Pracht und Besitz gegeben, unser Herr, damit sie (andere) von Deinem Weg in die Irre fuhren. Unser Herr, lösche ihren Besitz aus und schnüre ihre Herzen fest, so daß sie nicht glauben, bis sie die schmerzhafte Strafe sehen!"\qt@no{88}}%
\qt@newcmd\qurantrans@de@mcdliii{Er sagte: ``Eure beider Anrufung ist erhört. So verhaltet euch recht und folgt ja nicht dem Weg derjenigen, die nicht Bescheid wissen."\qt@no{89}}%
\qt@newcmd\qurantrans@de@mcdliv{Und Wir ließen die Kinder Isra'ils das Meer durchschreiten. Da verfolgten sie Fir'aun und seine Heerscharen in Auflehnung und Übertretung, bis daß, als er vom Ertrinken erfaßt wurde, er sagte: ``Ich glaube, daß es keinen Gott gibt außer dem, an den die Kinder Isra'ils glauben. Und ich gehöre (nun) zu den (Allah) Ergebenen."\qt@no{90}}%
\qt@newcmd\qurantrans@de@mcdlv{``Aber jetzt erst! Wo du dich doch zuvor widersetztest und zu den Unheilstiftern gehörtest?\qt@no{91}}%
\qt@newcmd\qurantrans@de@mcdlvi{Heute wollen Wir dich mit deinem Leib erretten', damit du für diejenigen, die nach dir kommen, ein Zeichen seiest." Und viele von den Menschen sind gegenüber Unseren Zeichen wahrlich unachtsam.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mcdlvii{Und Wir wiesen den Kindern Isra'ils einen wahrhaftigen Aufenthaltsort zu und versorgten sie von den guten Dingen. Sie wurden aber nicht (eher) uneinig, bis das Wissen zu ihnen gekommen war. Gewiß, dein Herr wird am Tag der Auferstehung zwischen ihnen über das entscheiden, worüber sie uneinig zu sein pflegten.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mcdlviii{Wenn du über das, was Wir zu dir (als Offenbarung) hinabgesandt haben, im Zweifel bist, dann frag diejenigen, die vor dir die Schrift lesen. Dir ist ja die Wahrheit von deinem Herrn zugekommen, so gehöre nun nicht zu den Zweiflern,\qt@no{94}}%
\qt@newcmd\qurantrans@de@mcdlix{und gehöre auch nicht zu denen, die Allahs Zeichen für Lüge erklären, sonst wirst du zu den Verlierern gehören.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mcdlx{Diejenigen,gegen die sich das Wort deines Herrn bewahrheitet hat, glauben nicht -,\qt@no{96}}%
\qt@newcmd\qurantrans@de@mcdlxi{auch wenn jedes Zeichen zu ihnen käme -, bis sie die schmerzhafte Strafe sehen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcdlxii{Wenn doch (irgend)eine Stadt geglaubt hätte, so daß ihr Glaube ihr genützt hätte! (Keine tat es), außer dem Volk des Yunus. Als diese glaubten, hoben Wir die schändliche Strafe im diesseitigen Leben von ihnen auf und gewährten ihnen Nießbrauch auf Zeit.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mcdlxiii{Und wenn dein Herr wollte, würden fürwahr alle auf der Erde zusammen gläubig werden. Willst du etwa die Menschen dazu zwingen, gläubig zu werden?\qt@no{99}}%
\qt@newcmd\qurantrans@de@mcdlxiv{Keiner Seele ist es möglich zu glauben, außer mit Allahs Erlaubnis. Und Er legt den Greuel auf diejenigen, die nicht begreifen.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mcdlxv{Sag: Schaut, was in den Himmeln und auf der Erde ist! Aber die Zeichen und die Warnungen werden den Leuten, die nicht glauben, nicht nützen.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mcdlxvi{Haben sie denn etwas anderes zu erwarten, als was den Tagen derer gleicht, die vor ihnen dahingegangen sind? Sag: So wartet ab! Ich bin mit euch einer von denen, die abwarten.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mcdlxvii{Dann werden Wir Unsere Gesandten und diejenigen, die glauben, erretten. So ist es Uns eine Pflicht: Wir retten die Gläubigen.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mcdlxviii{Sag: O ihr Menschen, wenn ihr über meine Religion im Zweifel seid, so diene ich nicht denjenigen, denen ihr anstatt Allahs dient, sondern ich diene Allah, Der euch abberuft. Und mir ist befohlen worden, einer der Gläubigen zu sein.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mcdlxix{Und: Richte dein Gesicht aufrichtig zur Religion hin als Anhänger des rechten Glaubens, und gehöre ja nicht zu den Götzendienern!\qt@no{105}}%
\qt@newcmd\qurantrans@de@mcdlxx{Und rufe nicht außer Allah an, was dir weder nützt noch schadet! Wenn du es tust, dann gehörst du folglich zu den Ungerechten.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mcdlxxi{Wenn Allah dir Unheil widerfahren läßt, so kann es keiner hinwegnehmen außer Ihm. Und wenn Er für dich etwas Gutes will, so kann keiner Seine Huld zurückweisen. Er trifft damit, wen Er will von Seinen Dienern. Er ist der Allvergebende und Barmherzige.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mcdlxxii{Sag: O ihr Menschen! Zu euch ist nunmehr die Wahrheit von eurem Herrn gekommen. Wer sich rechtleiten läßt, der ist nur zu seinem eigenen Vorteil rechtgeleitet. Und wer irregeht, der geht nur zu seinem Nachteil irre. Und ich bin nicht euer Sachwalter.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mcdlxxiii{Und folge dem, was dir (als Offenbarung) eingegeben wird, und gedulde dich, bis Allah richtet! Er ist der Beste derer, die richten.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mcdlxxiv{\basmalahde Alif-Lam-Ra. (Dies ist) ein Buch, dessen Zeichen eindeutig festgefügt und hierauf ausführlich dargelegt sind von Seiten eines Allweisen und Allkundigen:\qt@no{1}}%
\qt@newcmd\qurantrans@de@mcdlxxv{``Ihr sollt nur Allah dienen". Gewiß, ich bin euch von Ihm (aus) ein Warner und Frohbote -,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mcdlxxvi{und: ``Bittet euren Herrn um Vergebung, hierauf wendet euch Ihm in Reue zu, so wird Er euch einen schönen Nießbrauch auf eine festgesetzte Frist gewähren, und Er wird jedem, der voll Huld ist, Seine Huld gewähren." Wenn sie sich jedoch abkehren, (so sag:) Gewiß, ich fürchte für euch die Strafe eines schweren Tages.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mcdlxxvii{Zu Allah ist eure Rückkehr, und Er hat zu allem die Macht.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mcdlxxviii{Sicherlich, sie krümmen ihre Brüste zusammen, um sich vor Ihm zu verbergen. Sicherlich, wenn sie sich (auch) mit ihren Gewändern überdecken, Er weiß doch, was sie geheimhalten und was sie offenlegen. Gewiß, Er weiß über das Innerste der Brüste Bescheid.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mcdlxxix{Und es gibt kein Tier auf der Erde, ohne daß Allah sein Unterhalt obläge'. Und Er kennt seinen Aufenthaltsort und seinen Aufbewahrungsort. Alles ist in einem deutlichen Buch (verzeichnet).\qt@no{6}}%
\qt@newcmd\qurantrans@de@mcdlxxx{Er ist es, Der die Himmel und die Erde in sechs Tagen erschuf, und Sein Thron war auf dem Wasser, damit Er euch prüfe (und feststelle), wer von euch die besten Taten begeht. Und wenn du sagst: ``Ihr werdet nach dem Tod auferweckt werden", sagen diejenigen, die ungläubig sind, ganz gewiß: ``Das ist ja nur deutliche Zauberei."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mcdlxxxi{Und wenn Wir ihnen die Strafe für einen bestimmten Zeitraum zurückstellen, sagen sie ganz gewiß: ``Was hält sie zurück?" Wahrlich, am Tag, an dem sie über sie kommt, kann sie nicht von ihnen abgewendet werden, und es wird sie das umschließen, worüber sie sich lustig zu machen pflegten.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mcdlxxxii{Und wenn Wir den Menschen von Uns (aus) Barmherzigkeit kosten lassen und sie ihm hierauf entziehen, ist er fürwahr sehr verzweifelt und sehr undankbar.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mcdlxxxiii{Und wenn Wir ihn Angenehmes kosten lassen nach Leid, das ihm widerfuhr, sagt er ganz gewiß: ``Das Übel ist von mir gewichen." Und er ist fürwahr froh und prahlerisch,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mcdlxxxiv{außer denjenigen, die standhaft sind und rechtschaffene Werke tun; für sie gibt es Vergebung und großen Lohn.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mcdlxxxv{Vielleicht möchtest du einen Teil von dem, was dir (als Offenbarung) eingegeben wird, auslassen und deine Brust ist dadurch beklommen, und dies, weil sie sagen: ``Wäre doch ein Schatz auf ihn herabgesandt worden oder ein Engel mit ihm gekommen!" Du aber bist nur ein Warner. Und Allah ist Sachwalter über alles.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mcdlxxxvi{Oder sagen sie: ``Er hat ihn ersonnen" Sag: Dann bringt doch zehn ersonnene Suren bei, die ihm gleich sind, und ruft an, wen ihr könnt, außer Allah, wenn ihr wahrhaftig seid.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mcdlxxxvii{Und wenn sie euch nicht erhören, dann wisset, daß er nur mit Allahs Wissen herabgesandt worden ist und daß es keinen Gott gibt außer Ihm? Werdet ihr nun (Allah) ergeben sein?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mcdlxxxviii{Wer immer das diesseitige Leben und seinen Schmuck will, dem lassen wir in ihm (den Lohn für) seine Werke in vollem Maß zukommen, und ihm wird darin nichts geschmälert.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mcdlxxxix{Das sind diejenigen, für die es im Jenseits nur das (Höllen)feuer gibt. Nutzlos ist, was sie in ihm gemacht haben, und hinfallig wird, was sie zu tun pflegten.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mcdxc{Ist denn einer (den anderen gleich), der sich auf einen klaren Beweis von seinem Herrn stützt, und auf den ein Zeuge von Ihm folgt, dem die Schrift Musas vorausging als Vorbild und Barmherzigkeit? Diese glauben daran. Wer aber von den Gruppierungen ihn verleugnet, dessen Verabredung(sort) ist das (Höllen)feuer. So sei nicht über ihn im Zweifel, er ist die Wahrheit von deinem Herrn. Aber die meisten Menschen glauben nicht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mcdxci{Und wer ist ungerechter, als wer gegen Allah eine Lüge ersinnt? Jene werden ihrem Herrn vorgeführt, und die Zeugen werden sagen: ``Das sind diejenigen, die gegen ihren Herrn gelogen haben." Aber wahrlich, Allahs Fluch (kommt) über die Ungerechten,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mcdxcii{die von Allahs Weg abhalten und danach trachten, ihn krumm zu machen, und sie, die sie das Jenseits verleugnen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mcdxciii{Jene werden sich (Allah) auf der Erde nicht entziehen können, und sie werden außer Allah keine Schutzherren haben. Die Strafe wird ihnen vervielfacht. Sie konnten nicht hören, und sie pflegten nicht zu sehen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mcdxciv{Das sind diejenigen, die ihre Seelen verloren haben, und es ist ihnen entschwunden, was sie zu ersinnen pflegten.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mcdxcv{Zweifellos sind sie im Jenseits die größten Verlierer.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mcdxcvi{Gewiß, diejenigen, die glauben und rechtschaffene Werke tun und sich vor ihrem Herrn demütigen, das sind die Insassen des (Paradies)gartens. Ewig werden sie darin bleiben.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mcdxcvii{Das Gleichnis der beiden Gruppen ist wie der Blinde und der Taube und der Sehende und der Hörende. Sind die beiden (etwa) einander gleich? Bedenkt ihr denn nicht?\qt@no{24}}%
\qt@newcmd\qurantrans@de@mcdxcviii{Und Wir sandten ja bereits Nuh zu seinem Volk: ``Ich bin euch ja ein deutlicher Warner:\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcdxcix{Ihr sollt keinem außer Allah dienen. Gewiß, ich fürchte für euch die Strafe eines schmerzhaften Tages."\qt@no{26}}%
\qt@newcmd\qurantrans@de@md{Da sagte die führende Schar aus seinem Volk, die ungläubig war: ``Wir sehen in dir nur einen Menschen unseresgleichen. Und wir sehen, daß nur die dir folgen, die unsere Niedrigsten sind, und zwar ohne reifliche Überlegung. Und wir sehen bei euch keinen Vorzug gegenüber uns. Vielmehr meinen wir, daß ihr Lügner seid."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdi{Er sagte: ``O mein Volk, was meint ihr, wenn ich mich auf einen klaren Beweis von meinem Herrn stütze und Er mir eine Barmherzigkeit von Sich hat zukommen lassen, die aber eurem Blick entzogen wurde, sollen wir sie euch da aufzwingen, wo sie euch zuwider ist?\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdii{Und, o mein Volk, ich verlange von euch keinen Besitz dafür. Mein Lohn obliegt nur Allah. Und ich werde nicht diejenigen, die glauben, abweisen; sie werden ihrem Herrn begegnen. Aber ich sehe, ihr seid Leute, die töricht sind.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdiii{Und, o mein Volk, wer wird mir gegen Allah helfen, wenn ich sie abweise? Bedenkt ihr denn nicht?\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdiv{Und ich sage nicht zu euch, ich besäße die Schatzkammern Allahs, und ich weiß auch nicht das Verborgene; und ich sage nicht, ich sei ein Engel. Noch sage ich von denen, die eure Augen verachten, Allah werde ihnen niemals etwas Gutes zukommen lassen - Allah weiß am besten, was in ihrem Innersten ist -, sonst gehörte ich wahrlich zu den Ungerechten."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdv{Sie sagten: ``O Nuh, du hast bereits mit uns gestritten und dabei den Streit mit uns übertrieben'. Bring uns doch her, was du uns androhst, wenn du zu den Wahrhaftigen gehörst!"\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdvi{Er sagte: ``Nur Allah (allein) wird es euch bringen, wenn Er will, und ihr werdet euch (Ihm) nicht entziehen können.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdvii{Und mein guter Rat wird, wenn ich euch gut raten will, euch nicht nützen, wenn Allah euch in Verirrung fallen lassen will. Er ist euer Herr, und zu Ihm werdet ihr zurückgebracht."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdviii{Oder sagen sie: ``Er hat ihn ersonnen" Sag: Wenn ich ihn ersonnen habe, so lastet meine Übeltat auf mir, und ich bin unschuldig an dem, was ihr an Übeltaten begeht.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdix{Und Nuh wurde (als Offenbarung) eingegeben: ``Niemand aus deinem Volk wird glauben außer denen, die bereits geglaubt haben. So sei nicht bekümmert wegen dessen, was sie getan haben.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdx{Und verfertige das Schiff vor Unseren Augen und nach Unserer Eingebung. Und sprich Mich nicht an zugunsten derer, die Unrecht getan haben; sie werden ertränkt werden."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdxi{Er verfertigte das Schiff, wobei jedesmal, wenn eine führende Schar aus seinem Volk an ihm vorbeikam, sie über ihn spotteten. Er sagte: ``Wenn ihr über uns spottet, werden auch wir über euch spotten, so wie ihr (jetzt über uns) spottet.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdxii{Dann werdet ihr (noch) erfahren, über wen eine Strafe kommen, die ihn in Schande stürzt, und (über wen) eine beständige Strafe hereinbrechen wird."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdxiii{So ging es,) bis nun Unser Befehl kam und der Ofen brodelte; Wir sagten: ``Lade darin von jeder (Art) zwei, ein Paar, und deine Angehörigen außer demjenigen, gegen den das Wort vorher ergangen ist, und diejenigen, die glauben!" Mit ihm glaubten aber nur wenige.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdxiv{Und er sagte: ``Steigt hinein! Im Namen Allahs erfolge seine Fahrt und seine Landung. Mein Herr ist fürwahr Allvergebend und Barmherzig."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdxv{Es fuhr mit ihnen dahin inmitten von Wogen wie Berge. Nuh rief seinem Sohn zu, der sich abseits hielt: ``O mein lieber Sohn, steig mit uns ein und sei nicht mit den Ungläubigen!"\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdxvi{Er sagte: ``Ich werde auf einem Berg Zuflucht suchen, der mich vor dem Wasser schützt." Er sagte: ``Es gibt heute nichts, das vor dem Befehl Allahs schützen könnte, außer für den, dessen Er Sich erbarmt." Die Wogen trennten sie beide, und so gehörte er zu denjenigen, die ertränkt wurden.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdxvii{Und es wurde gesagt: ``O Erde, schlucke dein Wasser! O Himmel, halt ein!" Das Wasser nahm ab, und die Angelegenheit war entschieden. Es saß auf dem (Berg) Gudi auf. Und es wurde gesagt: ``Weg mit dem ungerechten Volk!"\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdxviii{Und Nuh rief zu seinem Herrn. Er sagte: ``Mein Herr, mein Sohn gehört zu meinen Angehörigen, und Dein Versprechen ist die Wahrheit, und Du bist der Weiseste derer, die richten."\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdxix{Er sagte: ``O Nuh, er gehört nicht zu deinen Angehörigen. Er ist eine Tat, die nicht rechtschaffen ist. So bitte Mich nicht um etwas, wovon du kein Wissen hast! Ich ermahne dich, nicht zu den Toren zu gehören."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdxx{Er sagte: ``Mein Herr, ich suche Schutz bei Dir (davor), daß ich Dich um etwas bitte, wovon ich kein Wissen habe! Wenn Du mir nicht vergibst und Dich meiner nicht erbarmst, werde ich zu den Verlierern gehören."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdxxi{Es wurde gesagt: ``O Nuh, steige hinunter in Frieden von Uns und mit Segnungen über dich und über Gemeinschaften von denen, die mit dir sind. Es gibt (aber auch) Gemeinschaften, denen Wir einen Nießbrauch geben werden, hierauf wird ihnen schmerzhafte Strafe von Uns widerfahren."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdxxii{Diese (Nachrichten) gehören zu den Nachrichten vom Verborgenen, die Wir dir (als Offenbarung) eingeben. Du wußtest sie zuvor nicht, weder du noch dein Volk. Sei nun standhaft! Das (gute) Ende gehört den Gottesfürchtigen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdxxiii{Und (Wir sandten) zu 'Ad ihren Bruder Hud. Er sagte: ``O mein Volk, dient Allah! Keinen Gott habt ihr außer Ihm. Ihr ersinnt ja nur Lügen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdxxiv{O mein Volk, ich verlange von euch keinen Lohn dafür. Mein Lohn obliegt nur demjenigen, Der mich erschaffen hat. Begreift ihr denn nicht?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdxxv{Und, o mein Volk, bittet euren Herrn um Vergebung, hierauf bereut vor Ihm, so wird Er den Regen auf euch ergiebig (herab)senden und euch (noch mehr) Kraft zu eurer Kraft hinzufügen. Und kehrt euch nicht als Übeltäter ab!"\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdxxvi{Sie sagten: ``O Hud, du hast uns keinen klaren Beweis gebracht. Wir wollen nicht unsere Götter auf dein Wort hin verlassen, und wir werden deinetwegen nicht gläubig werden.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mdxxvii{Wir können nur sagen, daß einige unserer Götter dich mit etwas Bösem heimgesucht haben." Er sagte: ``Ich nehme Allah zum Zeugen, und bezeugt auch ihr, daß ich mich lossage von dem, was ihr (Ihm) beigesellt\qt@no{54}}%
\qt@newcmd\qurantrans@de@mdxxviii{außer Ihm. So schmiedet alle gegen mich eure List und gewährt mir hierauf keinen Aufschub!\qt@no{55}}%
\qt@newcmd\qurantrans@de@mdxxix{Ich verlasse mich auf Allah, meinen Herrn und euren Herrn. Es gibt kein Tier, das Er nicht an seiner Stirnlocke hielte. Gewiß, mein Herr befindet sich auf einem geraden Weg.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mdxxx{Wenn ihr euch abkehrt, so habe ich euch ja das (von der Botschaft) ausgerichtet, womit ich zu euch gesandt wurde. Mein Herr wird ein anderes Volk nach euch folgen lassen, und ihr könnt Ihm keinerlei Schaden zufügen. Gewiß, mein Herr ist Hüter über alles."\qt@no{57}}%
\qt@newcmd\qurantrans@de@mdxxxi{Als nun Unser Befehl kam, erretteten Wir Hud und diejenigen, die mit ihm glaubten, durch Barmherzigkeit von Uns; und Wir erretteten sie vor harter Strafe.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mdxxxii{Das waren die 'Ad. Sie verleugneten die Zeichen ihres Herrn und widersetzten sich Seinen Gesandten und folgten dem Befehl eines jeden trotzigen Gewalthabers.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mdxxxiii{Aber ein Fluch folgte ihnen im Diesseits nach und (wird ihnen) am Tag der Auferstehung (nachfolgen). Sicherlich, die l 'Ad verleugneten ihren Herrn. Aber ja, weg mit 'Ad, dem Volk von Hud!\qt@no{60}}%
\qt@newcmd\qurantrans@de@mdxxxiv{Und (Wir sandten) zu Tamud ihren Bruder Salih. Er sagte: ``O mein Volk, dient Allah! Keinen Gott habt ihr außer Ihm. Er hat euch aus der Erde entstehen lassen und sie euch zu besiedeln gegeben. So bittet Ihn um Vergebung, hierauf bereut vor Ihm. Mein Herr ist nahe und erhört (die Gebete)."\qt@no{61}}%
\qt@newcmd\qurantrans@de@mdxxxv{Sie sagten: ``O Salih, du warst zuvor unter uns einer, auf den man Hoffnung setzte. Willst du uns denn verbieten, dem zu dienen, dem unsere Väter dienen? Wir sind fürwahr über das, wozu du uns aufrufst, in einem starken Zweifel."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mdxxxvi{Er sagte: ``O mein Volk, was meint ihr, wenn ich mich auf einen klaren Beweis von meinem Herrn stütze und Er mir Barmherzigkeit von Sich hat zukommen lassen, wer wird mir gegen Allah helfen, wenn ich mich Ihm widersetze? Ihr würdet mir nur (meinen) Verlust mehren.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mdxxxvii{Und, o mein Volk, dies ist die Kamelstute Allahs, euch zum Zeichen. Laßt sie auf Allahs Erde weiden und fügt ihr nichts Böses zu, sonst überkommt euch eine baldige Strafe."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mdxxxviii{Aber sie schnitten ihr die Sehnen durch. Da sagte er: ``Genießt (euer Leben) in eurer Wohnstätte noch drei Tage lang. Dies ist ein Versprechen, das nicht erlogen ist."\qt@no{65}}%
\qt@newcmd\qurantrans@de@mdxxxix{Als nun Unser Befehl kam, erretteten Wir Salih und diejenigen, die mit ihm glaubten, durch Barmherzigkeit von Uns, auch vor der Schande jenes Tages. Gewiß, dein Herr ist ja der Starke und Allmächtige.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mdxl{Und es ergriff diejenigen, die Unrecht taten, der Schrei, und so lagen sie morgens in ihren Wohnstätten auf den Brüsten da,\qt@no{67}}%
\qt@newcmd\qurantrans@de@mdxli{als hätten sie (überhaupt) nicht darin gewohnt. Sicherlich, die Tamud verleugneten ihren Herrn. Aber ja, weg mit Tamud!\qt@no{68}}%
\qt@newcmd\qurantrans@de@mdxlii{Unsere Gesandten kamen bereits zu Ibrahim mit der frohen Botschaft. Sie sagten: ``Frieden!" Er sagte: ``Friede!" Er verweilte nicht lange, da brachte er ein gebratenes Kalb herbei.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mdxliii{Als er aber sah, daß ihre Hände nicht danach hinlangten, fand er es von ihnen befremdlich, und er empfand Furcht vor ihnen. Sie sagten: ``Fürchte dich nicht! Wir sind zu dem Volk Luts gesandt."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mdxliv{Seine Frau stand dabei. Sie lachte, und da verkündeten Wir ihr Ishaq, und nach Ishaq Ya'qub.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mdxlv{Sie sagte: ``O wehe mir, soll ich noch gebären, wo ich doch alt bin, und dies ist doch mein Ehemann, schon ein Greis? Das ist fürwahr eine verwunderliche Sache."\qt@no{72}}%
\qt@newcmd\qurantrans@de@mdxlvi{Sie sagten: ``Wunderst du dich über den Befehl Allahs? Die Barmherzigkeit Allahs und Seine Segnungen seien auf euch, Angehörige des Hauses! Er ist Lobenswürdig und Ruhmvoll."\qt@no{73}}%
\qt@newcmd\qurantrans@de@mdxlvii{Als die Angst von Ibrahim gewichen und die frohe Botschaft zu ihm gekommen war, begann er mit Uns über das Volk Luts zu streiten.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mdxlviii{Ibrahim war wahrlich nachsichtig, weichherzig und reuig.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mdxlix{``O Ibrahim, lasse davon ab! Der Befehl deines Herrn ist nun gekommen. Über sie kommt eine Strafe, die unwiderruflich ist."\qt@no{76}}%
\qt@newcmd\qurantrans@de@mdl{Und als Unsere Gesandten zu Lut kamen, geriet er ihretwegen in eine böse Lage und war durch ihre Anwesenheit beklommen. Er sagte: ``Das ist ein drangsalvoller Tag."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mdli{Seine Leute kamen eilig zu ihm getrieben. Zuvor pflegten sie böse Taten zu begehen. Er sagte: ``O mein Volk, dies hier sind meine Töchter, sie sind reiner für euch. So fürchtet Allah und stürzt mich nicht um meiner Gäste willen in Schande! Gibt es denn unter euch keinen besonnenen Mann?"\qt@no{78}}%
\qt@newcmd\qurantrans@de@mdlii{Sie sagten: ``Du weißt sehr wohl, daß wir kein Recht auf deine Töchter haben, und du weißt fürwahr, was wir wollen."\qt@no{79}}%
\qt@newcmd\qurantrans@de@mdliii{Er sagte: ``Hätte ich doch Kraft (genug), um euch zu widerstehen, oder könnte ich nur bei einer starken Stütze Zuflucht finden!"\qt@no{80}}%
\qt@newcmd\qurantrans@de@mdliv{Sie sagten: ``O Lut, wir sind die Gesandten deines Herrn. Sie werden nicht zu dir gelangen. So zieh mit deinen Angehörigen in einem Teil der Nacht fort, und niemand von euch soll sich umwenden - außer deiner Frau! Gewiß, es wird sie treffen, was jene trifft. Der ihnen versprochene Zeitpunkt ist der Tagesanbruch. Ist nicht der Tagesanbruch schon nahe?"\qt@no{81}}%
\qt@newcmd\qurantrans@de@mdlv{Als nun Unser Befehl kam, kehrten Wir das Oberste von ihr zuunterst und ließen auf sie Steine aus vorbereitetem gebrannten Lehm regnen,\qt@no{82}}%
\qt@newcmd\qurantrans@de@mdlvi{bei deinem Herrn gekennzeichnete (Steine). Und sie liegt den Ungerechten nicht fern.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mdlvii{Und (Wir sandten) zu Madyan ihren Bruder Su'aib. Er sagte: ``O mein Volk, dient Allah! Keinen Gott habt ihr außer Ihm. Laßt an Maß und Gewicht nichts fehlen! Ich sehe, es geht euch gut. Aber ich fürchte für euch die Strafe eines umfassenden Tages.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mdlviii{Und, o mein Volk, gebt volles Maß und Gewicht in Gerechtigkeit und schmälert den Menschen nicht ihre Sachen und richtet auf der Erde nicht unheilstiftend Verderben an!\qt@no{85}}%
\qt@newcmd\qurantrans@de@mdlix{Das Bleibende (an Lohn) von Allah ist besser für euch, wenn ihr gläubig seid. Und ich bin nicht Hüter über euch."\qt@no{86}}%
\qt@newcmd\qurantrans@de@mdlx{Sie sagten: ``O Su'aib, befiehlt dir denn dein Gebet, daß wir das verlassen, dem unsere Väter dienen, oder (davon absehen), mit unserem Besitz zu tun, was wir wollen? Du bist fürwahr der Nachsichtige und Besonnene!"\qt@no{87}}%
\qt@newcmd\qurantrans@de@mdlxi{Er sagte: ``O mein Volk, was meint ihr, wenn ich mich auf einen klaren Beweis von meinem Herrn stütze und Er mir eine schöne Versorgung gewährt hat? Und ich will mich (dann in meinem Vorhaben) von euch nicht unterscheiden, indem ich das tue, was ich euch verbiete. Ich will nur Besserung, soweit ich (sie erreichen) kann. Das Gelingen wird mir nur durch Allah (allein) beschieden. Auf Ihn verlasse ich mich, und Ihm wende ich mich reuig zu.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mdlxii{Und, o mein Volk, meine Feindseligkeit soll euch ja nicht dazu bringen, daß euch das Gleiche trifft, was das Volk Nuhs oder das Volk Huds oder das Volk Salihs traf. Und das Volk Luts liegt euch nicht fern.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mdlxiii{Und bittet euren Herrn um Vergebung und hierauf bereut vor Ihm! Gewiß, mein Herr ist Barmherzig und Liebevoll."\qt@no{90}}%
\qt@newcmd\qurantrans@de@mdlxiv{Sie sagten: ``O Su'aib, wir verstehen nicht viel von dem, was du sagst. Und wir sehen wahrlich, daß du unter uns nicht wehrhaft bist. Wenn deine Sippschaft nicht wäre, hätten wir dich fürwahr gesteinigt. Du hast ja bei uns kein Ansehen."\qt@no{91}}%
\qt@newcmd\qurantrans@de@mdlxv{Er sagte: ``O mein Volk, hat meine Sippschaft bei euch mehr Ansehen als Allah, und habt ihr Ihn hinter eurem Rücken gelassen? Gewiß, mein Herr umfaßt, was ihr tut.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mdlxvi{Und, o mein Volk, handelt nach eurer Stellung! Ich werde (ebenfalls so) handeln. Ihr werdet (noch) erfahren, über wen eine Strafe kommen wird, die ihn in Schande stürzt, und wer ein Lügner ist. Und wartet (es) ab! Ich bin mit euch ein Beobachter."\qt@no{93}}%
\qt@newcmd\qurantrans@de@mdlxvii{Als nun Unser Befehl kam, erretteten Wir Su'aib und diejenigen, die mit ihm glaubten, durch Barmherzigkeit von Uns. Da ergriff diejenigen, die Unrecht taten, der Schrei, und so lagen sie morgens in ihren Wohnstätten auf den Brüsten da,\qt@no{94}}%
\qt@newcmd\qurantrans@de@mdlxviii{als hätten sie (überhaupt) nicht darin gewohnt. Aber ja, weg mit Madyan, wie auch die Tamud entfernt wurden!\qt@no{95}}%
\qt@newcmd\qurantrans@de@mdlxix{Und Wir sandten ja bereits Musa mit Unseren Zeichen und deutlicher Gewalt\qt@no{96}}%
\qt@newcmd\qurantrans@de@mdlxx{zu Fir'aun und seiner führenden Schar. Diese folgten dem Befehl Fir'auns; der Befehl Fir'auns aber war nicht rechtweisend.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mdlxxi{Er wird seinem Volk am Tag der Auferstehung vorangehen. Er führt sie wie zur Tränke ins (Höllen)feuer hinab - eine schlimme Tränke, zu der sie hinabgeführt werden!\qt@no{98}}%
\qt@newcmd\qurantrans@de@mdlxxii{Und ein Fluch folgte ihnen im Diesseits nach und (wird ihnen) am Tag der Auferstehung (nachfolgen) - eine schlimme Beigabe, die (ihnen) gegeben wird!\qt@no{99}}%
\qt@newcmd\qurantrans@de@mdlxxiii{Dies gehört zu den Nachrichten von den Städten; Wir erzählen es dir. Einige von ihnen stehen noch, andere sind abgemäht.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mdlxxiv{Nicht Wir haben ihnen Unrecht getan, sondern sie selbst haben sich Unrecht zugefügt. Ihre Götter, die sie anstatt Allahs anrufen, haben ihnen nichts genützt, als der Befehl deines Herrn kam, und sie richteten sie nur noch mehr zugrunde.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mdlxxv{So ist der Griff deines Herrn, wenn Er die Städte ergreift, während sie Unrecht tun. Gewiß, Sein Griff ist schmerzhaft und hart.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mdlxxvi{Darin ist wahrlich ein Zeichen für jemanden, der die Strafe des Jenseits fürchtet. Das ist ein Tag, zu dem die Menschen versammelt werden, und das ist ein Tag, an dem sie (alle) anwesend sein werden'.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mdlxxvii{Und Wir stellen ihn nur auf eine bestimmte Frist zurück.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mdlxxviii{An dem Tag, da er eintrifft, wird keine Seele sprechen, außer mit Seiner Erlaubnis. Dann werden einige von ihnen unglücklich und andere glückselig sein.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mdlxxix{Was nun diejenigen angeht, die unglücklich sind, so werden sie dann im (Höllen)feuer sein; darin werden sie seufzen und schluchzen,\qt@no{106}}%
\qt@newcmd\qurantrans@de@mdlxxx{ewig darin zu bleiben, solange die Himmel und die Erde währen, außer was dein Herr will. Dein Herr tut immer, was Er will.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mdlxxxi{Was aber diejenigen angeht, die glückselig sind, so werden sie im (Paradies)garten sein, ewig darin zu bleiben, solange die Himmel und die Erde währen, außer was dein Herr will, als eine unverkürzte Gabe.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mdlxxxii{So sei nicht im Zweifel über das, dem diese dienen. Sie dienen nur, wie ihre Väter zuvor gedient haben. Und Wir werden ihnen ihren Anteil fürwahr unverringert zukommen lassen.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mdlxxxiii{Und Wir gaben ja bereits Musa die Schrift, doch wurde man darüber uneinig. Und wenn es nicht ein früher ergangenes Wort von deinem Herrn gäbe, so wäre zwischen ihnen wahrlich entschieden worden. Und sie sind darüber fürwahr in einem starken Zweifel.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mdlxxxiv{Und allen wird dein Herr ihre Werke in vollem Maß ganz sicher erstatten. Gewiß, Er ist dessen, was sie tun, Kundig.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mdlxxxv{So verhalte dich recht, wie dir befohlen wurde, (du) und diejenigen, die mit dir bereuen, und lehnt euch nicht auf. Gewiß, was ihr tut, sieht Er wohl.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mdlxxxvi{Und sucht nicht eine Stütze bei denen, die Unrecht tun, sonst berührt euch das (Höllen)feuer; ihr habt außer Allah keine Schutzherren. Dann wird euch keine Hilfe zuteil werden.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mdlxxxvii{Und verrichte das Gebet an beiden Enden des Tages und in Stunden der Nacht. Die guten Taten lassen die bösen Taten vergehen. Das ist eine Ermahnung für diejenigen, die (Allahs) gedenken.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mdlxxxviii{Und sei standhaft; denn Allah läßt den Lohn der Gutes Tuenden nicht verlorengehen.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mdlxxxix{Wenn es unter den Geschlechtern vor euch doch nur Leute mit einem Rest (von Tugend) gegeben hätte, die verbieten, auf der Erde Unheil zu stiften - bis auf wenige unter denen, die Wir von ihnen retteten! Diejenigen, die Unrecht taten, folgten dem ihnen verliehenen üppigen Leben und wurden Übeltäter.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mdxc{Und dein Herr hätte die Städte nimmer zu Unrecht vernichtet, während ihre Bewohner Heilstifter waren.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mdxci{Und wenn dein Herr wollte, hätte Er die Menschen wahrlich zu einer einzigen Gemeinschaft gemacht. Aber sie bleiben doch uneinig,\qt@no{118}}%
\qt@newcmd\qurantrans@de@mdxcii{außer denen, derer Sich dein Herr erbarmt hat. Dazu hat Er sie erschaffen. Und so erfüllt sich das Wort deines Herrn: ``Ich werde die Hölle ganz gewiß mit den Ginn und den Menschen allesamt füllen."\qt@no{119}}%
\qt@newcmd\qurantrans@de@mdxciii{Alles berichten Wir dir von den Nachrichten über die Gesandten, womit Wir dein Herz festigen. Darin ist dir die Wahrheit zugekommen, und eine Ermahnung und Erinnerung für die Gläubigen.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mdxciv{Und sag zu denen, die nicht glauben: Handelt nach eurer Stellung, wir werden (ebenfalls so) handeln.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mdxcv{Und wartet ab, wir warten ebenfalls ab.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mdxcvi{Und Allahs ist das Verborgene der Himmel und der Erde, und zu Ihm wird die ganze Angelegenheit zurückgebracht. So diene Ihm und verlasse dich auf Ihn. Und dein Herr ist nicht unachtsam dessen, was ihr tut.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mdxcvii{\basmalahde Alif-Lam-Ra. Dies sind die Zeichen des deutlichen Buches.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdxcviii{Wir haben es als einen arabischen Qur'an hinabgesandt, auf daß ihr begreifen möget.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdxcix{Wir berichten dir die schönsten Geschichten dadurch, daß Wir dir diesen Qur'an (als Offenbarung) eingegeben haben, obgleich du zuvor wahrlich zu den Unachtsamen gehörtest.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdc{Als Yusuf zu seinem Vater sagte: ``O mein Vater, ich sah elf Sterne und die Sonne und den Mond, ich sah sie sich vor mir niederwerfen."\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdci{Er sagt: ``O mein lieber Sohn, erzähle dein (Traum)gesicht nicht deinen Brüdern, sonst werden sie eine List gegen dich schmieden. Gewiß, der Satan ist dem Menschen ein deutlicher Feind.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdcii{Und so wird dein Herr dich erwählen und dich etwas von der Deutung der Geschichten lehren und Seine Gunst an dir und an der Sippe Ya'qubs vollenden, wie Er sie zuvor an deinen beiden Vätern Ibrahim und lshaq vollendet hat. Gewiß, dein Herr ist Allwissend und Allweise."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdciii{Wahrlich, in Yusuf und seinen Brüdern sind Zeichen für diejenigen, die (nach der Wahrheit) fragen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdciv{Als sie sagten: ``Wahrlich, Yusuf und sein Bruder sind unserem Vater lieber als wir, obwohl wir eine (ansehnliche) Schar sind. Unser Vater befindet sich fürwahr in deutlichem Irrtum.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdcv{Tötet Yusuf oder werft ihn ins Land hinaus, so wird das Gesicht eures Vaters sich nur noch euch zuwenden, und danach werdet ihr rechtschaffene Leute sein."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdcvi{Einer von ihnen sagte: ``Tötet Yusuf nicht, sondern werft ihn in die verborgene Tiefe des Brunnenlochs, so werden ihn schon einige der Reisenden auflesen, wenn ihr doch etwas tun wollt."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdcvii{Sie sagten: ``O unser Vater, warum vertraust du uns Yusuf nicht an? Wir werden ihm wahrlich aufrichtig zugetan sein.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdcviii{Sende ihn morgen mit uns, daß er Spaß und Spiel finde; wir werden ihn fürwahr behüten."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdcix{Er sagte: ``Es macht mich fürwahr traurig, daß ihr ihn mitnehmt. Und ich fürchte, daß ihn der Wolf frißt, während ihr seiner unachtsam seid."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdcx{Sie sagten: ``Wenn ihn der Wolf fressen sollte, wo wir doch eine (ansehnliche) Schar sind, dann wären wir fürwahr Verlierer."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdcxi{Als sie ihn mitnahmen und sich geeinigt hatten, ihn in die verborgene Tiefe des Brunnenlochs zu stecken, gaben Wir ihm ein: ``Du wirst ihnen ganz gewiß noch diese ihre Tat kundtun, ohne daß sie merken."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdcxii{Und am Abend kamen sie weinend zu ihrem Vater.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdcxiii{Sie sagten: ``O unser Vater, wir gingen, um einen Wettlauf zu machen, und ließen Yusuf bei unseren Sachen zurück. Da fraß ihn der Wolf. Aber du glaubst uns wohl nicht, auch wenn wir die Wahrheit sagen."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdcxiv{Sie brachten falsches Blut auf seinem Hemd. Er sagte: ``Nein! Vielmehr habt ihr selbst euch etwas eingeredet. (So gilt es,) schöne Geduld (zu üben). Allah ist Derjenige, bei Dem Hilfe zu suchen ist gegen das, was ihr beschreibt."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdcxv{Reisende kamen vorbei. Sie sandten ihren Wasserschöpfer, und er ließ seinen Eimer hinunter. Er sagte: ``O frohe Botschaft! Da ist ein Junge." Sie verbargen ihn als Ware. Und Allah wußte wohl, was sie taten.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdcxvi{Und sie verkauften ihn für einen zu niedrigen Preis, einige gezählte Dirhams. Und sie übten Verzicht in Bezug auf ihn.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdcxvii{Und derjenige aus Ägypten, der ihn gekauft hatte, sagte zu seiner Frau: ``Bereite ihm einen gastfreundlichen Aufenthalt. Vielleicht wird er uns nützlich sein, oder nehmen wir ihn als Kind an." So verliehen Wir Yusuf eine feste Stellung im Land. Und Wir wollten ihn etwas von der Deutung der Geschichten lehren. Und Allah ist in Seiner Angelegenheit überlegen. Aber die meisten Menschen wissen nicht.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdcxviii{Als er seine Vollreife erlangt hatte, gaben Wir ihm Urteil(skraft) und Wissen. So vergelten Wir den Gutes Tuenden.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdcxix{Und diejenige, in deren Haus er war, versuchte, ihn zu verführen'. Sie schloß die Türen ab und sagte: ``Da bin ich für dich!" Er sagte: ``Allah schütze mich (davor)! Er, mein Herr, hat mir einen schönen Aufenthalt bereitet. Gewiß, den Ungerechten wird es nicht wohl ergehen."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdcxx{Es verlangte sie nach ihm, und es hätte ihn nach ihr verlangt, wenn er nicht den Beweis seines Herrn gesehen hätte. Dies (geschah), damit Wir das Böse und das Schändliche von ihm abwendeten. Er gehört ja zu Unseren auserlesenen Dienern.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdcxxi{Sie versuchten beide als erster zur Tür zu gelangen. Sie zerriß ihm von hinten das Hemd. Und sie fanden ihren Herrn bei der Tür vor. Sie sagte: ``Der Lohn dessen, der deiner Familie Böses (antun) wollte, ist nur, daß er ins Gefängnis gesteckt wird, oder schmerzhafte Strafe."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdcxxii{Er sagte: ``Sie war es, die versucht hat, mich zu verführen." Und ein Zeuge aus ihrer Familie bezeugte: ``Wenn sein Hemd vorn zerrissen ist, dann hat sie die Wahrheit gesprochen, und er gehört zu den Lügnern.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdcxxiii{Wenn sein Hemd aber hinten zerrissen ist, dann hat sie gelogen, und er gehört zu den Wahrhaftigen."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdcxxiv{Als er nun sah, daß sein Hemd hinten zerrissen war, sagte er: ``Das gehört zu euren Listen. Eure List ist gewaltig.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdcxxv{Yusuf, lasse davon ab! Und (du, jene,) bitte um Vergebung für deine Sünde! Du gehörst ja zu denjenigen, die Verfehlungen begangen haben."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdcxxvi{Einige) Frauen in der Stadt sagten: ``Die Frau des hohen Herrn versucht, ihren Burschen zu verführen. Er hat sie in leidenschaftliche Liebe versetzt. Wir sehen wahrlich, sie befindet sich in deutlichem Irrtum."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdcxxvii{Als sie nun von ihren Ränken hörte, sandte sie zu ihnen und bereitete ihnen ein Gastmahl'. Sie gab einer jeden von ihnen ein Messer und sagte (zu Yusuf): ``Komm zu ihnen heraus." Als sie ihn sahen, fanden sie ihn groß(artig), und sie zerschnitten sich ihre Hände und sagten: ``Allah behüte! Das ist kein Mensch, das ist nur ein ehrenvoller Engel."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdcxxviii{Sie sagte: ``Seht, das ist der, dessentwegen ihr mich getadelt habt. Ich habe allerdings versucht, ihn zu verführen, doch er widerstand. Und wenn er nicht tut, was ich ihm befehle, wird er ganz gewiß ins Gefängnis gesteckt werden, und er wird gewiß zu den Geringgeachteten gehören."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdcxxix{Er sagte: ``Mein Herr, das Gefängnis ist mir lieber als das, wozu sie mich auffordern. Und wenn Du ihre List von mir nicht abwendest, werde ich mich zu ihnen hingezogen fühlen und zu den Toren gehören."\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdcxxx{Da erhörte ihn sein Herr und wendete ihre List von ihm ab. Er ist ja der Allhörende und Allwissende.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdcxxxi{Hierauf schien es ihnen angebracht, nachdem sie die Zeichen (seiner Unschuld) gesehen hatten, ihn für eine gewisse Zeit ins Gefängnis zu stecken.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdcxxxii{Mit ihm kamen zwei Burschen ins Gefängnis. Der eine von ihnen sagte: ``Ich sah mich Wein auspressen." Der andere sagte: ``Ich sah mich auf dem Kopf Brot tragen, von dem die Vögel fraßen. Tue uns die Deutung hiervon kund. Wir sehen, daß du zu den Gutes Tuenden gehörst."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdcxxxiii{Er sagte: ``Es wird euch kein Essen gebracht, mit dem ihr versorgt werdet, ohne daß ich euch die Deutung davon kundgetan habe, bevor es euch gebracht wird. Seht, das ist etwas von dem, was mich mein Herr gelehrt hat. Verlassen habe ich das Glaubensbekenntnis von Leuten, die nicht an Allah glauben, und (verlassen habe, ich) sie, die sie das Jenseits verleugnen,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdcxxxiv{und ich bin dem Glaubensbekenntnis meiner Väter Ibrahim, Ishaq und Ya'qub gefolgt. Es steht uns nicht zu, Allah etwas beizugesellen. Das ist etwas von Allahs Huld gegen uns und gegen die Menschen. Aber die meisten Menschen sind nicht dankbar.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdcxxxv{O meine beiden Gefängnisgefährten! Sind verschiedene Herren besser oder Allah, der Eine, der Allbezwinger?\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdcxxxvi{Ihr dient außer Ihm nur Namen, die ihr genannt habt, ihr und eure Väter, für die Allah (jedoch) keine Ermächtigung herabgesandt hat. Das Urteil ist allein Allahs. Er hat befohlen, daß ihr nur Ihm dienen sollt. Das ist die richtige Religion. Aber die meisten Menschen wissen nicht.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdcxxxvii{O meine beiden Gefangnisgefährten! Was den einen von euch angeht, so wird er seinem Herrn Wein zu trinken geben. Was aber den anderen angeht, so wird er gekreuzigt, und die Vögel werden von seinem Kopf fressen. Entschieden ist die Angelegenheit, über die ihr um Auskunft fragt."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdcxxxviii{Und er sagte zu dem von beiden, von dem er glaubte, daß er entkommen werde: ``Erwähne mich bei deinem Herrn." Aber der Satan ließ ihn vergessen, ihn bei seinem Herrn zu erwähnen, (und) so blieb er noch einige Jahre im Gefängnis.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdcxxxix{Und der König sagte: ``Ich sah sieben fette Kühe, die von sieben mageren gefressen wurden, und sieben grüne Ähren und (sieben) andere dürre. O ihr führende Schar, gebt mir Auskunft über mein (Traum)gesicht, wenn ihr ein (Traum)gesicht auslegen könnt."\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdcxl{Sie sagten: ``(Das ist) ein Bündel von wirren Träumen. Wir wissen über die Deutung der Träume nicht Bescheid."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdcxli{Derjenige von beiden, der entkommen war und sich nach einiger Zeit erinnerte, sagte: ``Ich werde euch seine Deutung kundtun. Darum entsendet mich."\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdcxlii{``Yusuf, du Wahrhaftiger, gib uns Auskunft über sieben fette Kühe, die von sieben mageren gefressen werden, und über sieben grüne Ähren und (sieben) andere dürre, auf daß ich zu den Leuten zurückkehre, auf daß sie wissen mögen."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdcxliii{Er sagte: ``Ihr werdet unablässig sieben Jahre wie gewohnt säen. Was ihr erntet, das laßt in seinen Ähren, bis auf ein weniges, wovon ihr eßt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdcxliv{Hierauf werden nach alledem sieben harte (Jahre) kommen, die das aufzehren werden, was ihr für sie vorbereitet habt, bis auf ein weniges von dem, was ihr aufbewahrt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdcxlv{Hierauf wird nach alledem ein Jahr kommen, in dem die Menschen Regen haben und in dem sie (Früchte) auspressen werden."\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdcxlvi{Der König sagte: ``Bringt ihn zu mir." Als der Bote zu ihm kam, sagte er: ``Kehr zu deinem Herrn zurück und frag ihn, wie es mit den Frauen steht, die sich ihre Hände zerschnitten. Mein Herr weiß doch über ihre List Bescheid."\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdcxlvii{Er sagte: ``Was war da mit euch, als ihr versuchtet, Yusuf zu verführen?" Sie sagten: ``Allah behüte! Wir wissen nichts Böses gegen ihn (anzugeben)." Die Frau des hohen Herrn sagte: ``Jetzt ist die Wahrheil ans Licht gekommen. Ich habe versucht, ihn zu verführen. Und er gehört fürwahr zu den Wahrhaftigen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdcxlviii{Dies ist, damit er weiß, daß ich ihn nicht in seiner Abwesenheit verraten habe und daß Allah die List der Verräter nicht gelingen läßt.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdcxlix{Und ich spreche mich nicht selbst frei. Die Seele gebietet fürwahr mit Nachdruck das Böse, außer daß mein Herr Sich erbarmt. Mein Herr ist Allvergebend und Barmherzig."\qt@no{53}}%
\qt@newcmd\qurantrans@de@mdcl{Und der König sagte: ``Bringt ihn zu mir. Ich will ihn mir selbst vorbehalten." Als er mit ihm sprach, sagte er: ``Du bist (von) heute (an) bei uns in fester Stellung und vertrauenswürdig."\qt@no{54}}%
\qt@newcmd\qurantrans@de@mdcli{Er sagte: ``Setze mich über die Vorratskammern des Landes ein; ich bin ein kenntnisreicher Hüter."\qt@no{55}}%
\qt@newcmd\qurantrans@de@mdclii{So verliehen Wir Yusuf eine feste Stellung im Land, so daß er sich darin aufhalten konnte, wo immer er wollte. Wir treffen mit Unserer Barmherzigkeit, wen Wir wollen, und Wir lassen den Lohn der Gutes Tuenden nicht verlorengehen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mdcliii{Aber der Lohn des Jenseits ist wahrlich besser für diejenigen, die glauben und gottesfürchtig sind.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mdcliv{Und da kamen Yusufs Brüder und traten bei ihm ein. Er erkannte sie, während sie ihn nicht erkannten.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mdclv{Als er sie nun mit ihrem Bedarf ausgestattet hatte, sagte er: ``Bringt mir einen (Halb)bruder von euch väterlicherseits. Seht ihr denn nicht, daß ich volles Maß gebe und daß ich der beste Obdachgeber bin?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mdclvi{Wenn ihr ihn mir nicht bringt, so bekommt ihr bei mir kein Maß mehr, und ihr sollt nicht in meine Nähe kommen."\qt@no{60}}%
\qt@newcmd\qurantrans@de@mdclvii{Sie sagten: ``Wir werden versuchen, seinen Vater in Bezug auf ihn zu bewegen, und wir werden es bestimmt tun."\qt@no{61}}%
\qt@newcmd\qurantrans@de@mdclviii{Und er sagte zu seinen Burschen: ``Steckt ihre (Tausch)ware (zurück) in ihr Gepäck, so daß sie sie (wieder)erkennen, wenn sie zu ihren Angehörigen zurückgekehrt sind, auf daß sie wiederkommen mögen."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mdclix{Als sie zu ihrem Vater zurückkamen, sagten sie: ``O unser Vater, ein (weiteres) Maß ist uns verweigert worden. Darum lasse unseren Bruder mit uns gehen, so bekommen wir ein (weiteres) Maß zugeteilt; wir werden ihn fürwahr behüten."\qt@no{63}}%
\qt@newcmd\qurantrans@de@mdclx{Er sagte: ``Kann ich ihn euch etwa anders anvertrauen, als ich euch zuvor seinen Bruder anvertraut habe? Allah ist besser als Behütender, und Er ist der Barmherzigste der Barmherzigen."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mdclxi{Und als sie ihre Sachen öffneten, fanden sie, daß ihre Ware ihnen zurückgegeben worden war. Sie sagten: ``O unser Vater, was begehren wir (mehr)? Das ist unsere Ware, sie ist uns zurückgegeben worden. Wir werden Vorrat für unsere Angehörigen bringen, unseren Bruder behüten und das Maß (der Last) eines Kamels mehr haben. Das ist ein leicht(erhältlich)es Maß."\qt@no{65}}%
\qt@newcmd\qurantrans@de@mdclxii{Er sagte: ``Ich werde ihn nicht mit euch senden, bis ihr mir ein verbindliches Versprechen vor Allah gebt, daß ihr ihn mir ganz gewiß wiederbringt, es sei denn, ihr werdet umringt." Als sie ihm ihr verbindliches Versprechen gegeben hatten, sagte er: ``Allah ist Sachwalter über das, was wir (hier) sagen."\qt@no{66}}%
\qt@newcmd\qurantrans@de@mdclxiii{Und er sagte: ``O meine Kinder, geht nicht (alle) durch ein einziges Tor hinein, sondern geht durch verschiedene Tore hinein. Ich kann euch gegen Allah nichts nützen. Das Urteil ist allein Allahs. Auf Ihn verlasse ich mich; und auf Ihn sollen sich diejenigen verlassen, die sich (überhaupt auf jemanden) verlassen (wollen)."\qt@no{67}}%
\qt@newcmd\qurantrans@de@mdclxiv{Als sie hineingingen, wie ihr Vater ihnen befohlen hatte, konnte es ihnen vor Allah nichts nützen. (Es war) nur ein Bedürfnis in der Seele Ya'qubs, das er (damit) erfüllte. Und er besaß fürwahr Wissen, weil Wir es ihn gelehrt hatten. Aber die meisten Menschen wissen nicht.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mdclxv{Als sie bei Yusuf eintraten, zog er seinen Bruder zu sich. Er sagte: ``Gewiß, ich bin dein Bruder. So sei nicht bekümmert wegen dessen, was sie getan haben."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mdclxvi{Als er sie nun mit ihrem Bedarf ausgestattet hatte, steckte er das Trinkgefäß in das Gepäck seines Bruders. Hierauf rief ein Rufer aus: ``Ihr (da, von der) Karawane, ihr seid fürwahr Diebe."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mdclxvii{Sie sagten, indem sie sich ihnen zuwandten: ``Was vermißt ihr?"\qt@no{71}}%
\qt@newcmd\qurantrans@de@mdclxviii{Sie sagten: ``Wir vermissen den Kelch des Königs. Wer ihn wiederbringt, erhält die Last eines Kamels, und dafür bin ich Bürge."\qt@no{72}}%
\qt@newcmd\qurantrans@de@mdclxix{Sie sagten: ``Bei Allah, ihr wißt doch, wir sind nicht gekommen, um im Land Unheil zu stiften, und wir sind keine Diebe."\qt@no{73}}%
\qt@newcmd\qurantrans@de@mdclxx{Sie sagten: ``Was soll dann die Vergeltung dafür sein, wenn ihr Lügner seid?"\qt@no{74}}%
\qt@newcmd\qurantrans@de@mdclxxi{Sie (die Brüder) sagten: ``Die Vergeltung dafür soll sein, daß derjenige, in dessen Gepäck er gefunden wird, selbst das Entgelt dafür sein soll. So vergelten wir den Ungerechten."\qt@no{75}}%
\qt@newcmd\qurantrans@de@mdclxxii{Er begann (zu suchen) in ihren Behältern vor dem Behälter seines Bruders. Hierauf holte er es aus dem Behälter seines Bruders hervor. So führten Wir für Yusuf eine List aus. Nach dem Gesetz des Königs hätte es ihm nicht zugestanden, seinen Bruder (als Sklaven) zu nehmen, außer daß Allah es wollte. Wir erhöhen, wen Wir wollen, um Rangstufen. Und über jedem, der Wissen besitzt, steht einer, der (noch mehr) weiß.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mdclxxiii{Sie sagten: ``Wenn er stiehlt, so hat ein Bruder von ihm schon zuvor gestohlen." Aber Yusuf hielt es in seinem Innersten geheim und legte es ihnen nicht offen. Er sagte: ``Ihr befindet euch in einer (noch) schlechteren Lage. Und Allah weiß sehr wohl, was ihr beschreibt."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mdclxxiv{Sie sagten: ``O hoher Herr, er hat einen Vater, einen hochbetagten Greis. So nimm einen von uns an seiner Stelle. Wir sehen, daß du zu den Gutes Tuenden gehörst."\qt@no{78}}%
\qt@newcmd\qurantrans@de@mdclxxv{Er sagte: ``Allah schütze uns (davor), daß wir einen anderen nehmen als denjenigen, bei dem wir unsere Sachen gefunden haben! Wir würden sonst wahrlich zu den Ungerechten gehören."\qt@no{79}}%
\qt@newcmd\qurantrans@de@mdclxxvi{Als sie die Hoffnung an ihm aufgegeben hatten, zogen sie sich zurück zu einem vertraulichen Gespräch. Der Älteste von ihnen sagte: ``Wißt ihr (denn) nicht, daß euer Vater von euch ein verbindliches Versprechen vor Allah entgegengenommen hat und daß ihr zuvor (eure Pflicht) gegenüber Yusuf vernachlässigt habt? Ich werde das Land nicht verlassen, bis mein Vater es mir erlaubt oder Allah ein Urteil für mich fällt. Er ist der Beste derer, die Urteile fällen.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mdclxxvii{Kehrt zu eurem Vater zurück und sagt: ,O unser Vater, dein Sohn hat gestohlen, und wir bezeugen nur das, was wir wissen, und wir sind nicht Hüter über das Verborgene.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mdclxxviii{Frag die Stadt, in der wir waren, und die Karawane, mit der wir angekommen sind. Wir sagen gewiß die Wahrheit'."\qt@no{82}}%
\qt@newcmd\qurantrans@de@mdclxxix{Er sagte: ``Nein! Vielmehr habt ihr selbst euch etwas eingeredet. (Es gilt) schöne Geduld (zu üben). Aber vielleicht wird Allah sie mir alle wiederbringen. Er ist ja der Allwissende und Allweise."\qt@no{83}}%
\qt@newcmd\qurantrans@de@mdclxxx{Und er kehrte sich von ihnen ab und sagte: ``O mein Kummer um Yusuf!" Und seine Augen wurden weiß vor Trauer, und so hielt er (seinen Kummer) zurück.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mdclxxxi{Sie sagten: ``Bei Allah, du hörst nicht auf, Yusufs zu gedenken, bis du hinfällig geworden bist oder zu denen gehörst, die umkommen."\qt@no{85}}%
\qt@newcmd\qurantrans@de@mdclxxxii{Er sagte: ``Ich klage meinen unerträglichen Kummer und meine Trauer nur Allah (allein), und ich weiß von Allah her, was ihr nicht wißt.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mdclxxxiii{O meine Kinder, geht und erkundigt euch über Yusuf und seinen Bruder. Und gebt nicht die Hoffnung auf das Erbarmen Allahs auf. Es gibt die Hoffnung auf das Erbarmen Allahs nur das ungläubige Volk auf."\qt@no{87}}%
\qt@newcmd\qurantrans@de@mdclxxxiv{Als sie (wieder) bei ihm eintraten, sagten sie: ``O hoher Herr, Unheil ist uns und unseren Angehörigen widerfahren. Und wir haben (nur) Ware von geringem Wert gebracht. So gib uns (dennoch) volles Maß und gib (es) uns als Almosen. Allah vergilt denjenigen, die Almosen geben."\qt@no{88}}%
\qt@newcmd\qurantrans@de@mdclxxxv{Er sagte: ``Wißt ihr (noch), was ihr Yusuf und seinem Bruder damals in eurer Torheit antatet?"\qt@no{89}}%
\qt@newcmd\qurantrans@de@mdclxxxvi{Sie sagten: ``Bist du denn wirklich Yusuf?" Er sagte: ``Ich bin Yusuf, und das ist mein Bruder. Allah hat uns eine Wohltat erwiesen. Gewiß, wer gottesfürchtig und standhaft ist -- gewiß, Allah läßt den Lohn der Gutes Tuenden nicht verlorenge hen."\qt@no{90}}%
\qt@newcmd\qurantrans@de@mdclxxxvii{Sie sagten: ``Bei Allah, Allah hat dich uns vorgezogen. Und wir haben wahrlich Verfehlungen begangen."\qt@no{91}}%
\qt@newcmd\qurantrans@de@mdclxxxviii{Er sagte: ``Keine Schelte soll heute über euch kommen. Allah vergibt euch, Er ist ja der Barmherzigste der Barmherzigen.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mdclxxxix{Geht mit diesem meinem Hemd und legt es auf das Gesicht meines Vaters, so wird er sein Augenlicht (wieder)erlangen. Und bringt eure Angehörigen allesamt zu mir."\qt@no{93}}%
\qt@newcmd\qurantrans@de@mdcxc{Als nun die Karawane aufgebrochen war, sagte ihr Vater: ``Wahrlich, ich nehme Yusufs Geruch war. Wenn ihr mich nur nicht bezichtigen würdet, Unsinn zu reden!"\qt@no{94}}%
\qt@newcmd\qurantrans@de@mdcxci{Sie sagten: ``Bei Allah, du befindest dich fürwahr in deinem alten Irrtum."\qt@no{95}}%
\qt@newcmd\qurantrans@de@mdcxcii{Als nun der Frohbote kam, legte er es auf sein Gesicht, und da hatte er sein Augenlicht wiedererlangt. Er sagte: ``Habe ich euch nicht gesagt, daß ich von Allah her weiß, was ihr nicht wißt?"\qt@no{96}}%
\qt@newcmd\qurantrans@de@mdcxciii{Sie sagten: ``O unser Vater, bitte für uns um Vergebung unserer Sünden, denn wir haben gewiß Verfehlungen begangen."\qt@no{97}}%
\qt@newcmd\qurantrans@de@mdcxciv{Er sagte: ``Ich werde meinen Herrn um Vergebung für euch bitten. Er ist ja der Allvergebende und Barmherzige."\qt@no{98}}%
\qt@newcmd\qurantrans@de@mdcxcv{Als sie nun bei Yusuf eintraten, zog er seine Eltern an sich und sagte: ``Betretet Ägypten, wenn Allah will, in Sicherheit."\qt@no{99}}%
\qt@newcmd\qurantrans@de@mdcxcvi{Und er hob seine Eltern auf den Thron empor. Und sie fielen vor ihm ehrerbietig nieder. Er sagte: ``O mein lieber Vater, das ist die Deutung meines (Traum)gesichts von zuvor. Mein Herr hat es wahrgemacht. Und Er hat mir Gutes erwiesen, als Er mich aus dem Gefängnis herauskommen ließ und euch aus dem nomadischen Leben hierherbrachte, nachdem der Satan zwischen mir und meinen Brüdern (zu Zwietracht) aufgestachelt hatte. Gewiß, mein Herr ist feinfühlig (in der Durchführung dessen), was Er will. Er ist ja der Allwissende und Allweise.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mdcxcvii{Mein Herr, Du hast mir etwas von der Herrschaft gegeben und mich etwas von der Deutung der Sprüche gelehrt. (O Du) Erschaffer der Himmel und der Erde, Du bist mein Schutzherr im Diesseits und Jenseits. Berufe mich als (Dir) ergeben ab und nimm mich unter die Rechtschaffenen auf."\qt@no{101}}%
\qt@newcmd\qurantrans@de@mdcxcviii{Dies gehört zu den Nachrichten vom Verborgenen, das Wir dir (als Offenbarung) eingeben. Denn du warst nicht bei ihnen, als sie sich einigten, indem sie Ränke schmiedeten.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mdcxcix{Aber die meisten Menschen werden, auch wenn du noch so sehr (danach) trachtest, nicht gläubig sein.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mdcc{Und du verlangst von ihnen keinen Lohn dafür. Es ist nur eine Ermahnung für die Weltenbewohner.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mdcci{Wie viele Zeichen gibt es in den Himmeln und auf der Erde, an denen sie vorbeigehen, wobei sie sie unbeachtet lassen!\qt@no{105}}%
\qt@newcmd\qurantrans@de@mdccii{Und die meisten von ihnen glauben nicht an Allah, ohne (Ihm andere) beizugesellen.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mdcciii{Glauben sie denn, sicher zu sein davor, daß eine überdeckende Strafe von Allah über sie kommt oder daß plötzlich die Stunde über sie kommt, ohne daß sie merken?\qt@no{107}}%
\qt@newcmd\qurantrans@de@mdcciv{Sag: Das ist mein Weg: Ich rufe zu Allah aufgrund eines sichtbaren Hinweises, ich und diejenigen, die mir folgen. Preis sei Allah! Und ich gehöre nicht zu den Götzendienern.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mdccv{Und Wir haben vor dir nur Männer gesandt von den Bewohnern der Städte, denen Wir (Offenbarungen) eingaben. Sind sie denn nicht auf der Erde umhergereist, so daß sie schauen (konnten), wie das Ende derjenigen war, die vor ihnen waren? Die Wohnstätte des Jenseits ist wahrlich besser für diejenigen, die gottesfürchtig sind. Begreift ihr denn nicht?\qt@no{109}}%
\qt@newcmd\qurantrans@de@mdccvi{Erst dann, als die Gesandten die Hoffnung aufgegeben hatten und sie meinten, daß sie belogen worden seien, kam Unsere Hilfe zu ihnen. Und so wird errettet, wen Wir wollen. Aber vom übeltätigen Volk wird Unsere Gewalt nicht abgewandt.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mdccvii{In ihren Geschichten ist wahrlich eine Lehre für diejenigen, die Verstand besitzen. Es ist keine Aussage, die ersonnen wird, sondern die Bestätigung dessen, was vor ihm war, und die ausführliche Darlegung aller Dinge und eine Rechtleitung und Barmherzigkeit für Leute, die glauben.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mdccviii{\basmalahde Alif-Lam-Mim-Ra. Dies sind die Zeichen des Buches. Was zu dir von deinem Herrn herabgesandt worden ist, ist die Wahrheit. Aber die meisten Menschen glauben nicht.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdccix{Allah ist es, Der die Himmel ohne Stützen, die ihr sehen könnt, emporgehoben und Sich hierauf über den Thron erhoben hat. Er hat die Sonne und den Mond dienstbar gemacht - jedes läuft auf eine festgesetzte Frist zu. Er regelt die Angelegenheit, er legt die Zeichen ausführlich dar, auf daß ihr von der Begegnung mit eurem Herrn überzeugt seiet.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdccx{Und Er ist es, Der die Erde gedehnt und auf ihr festgegründete Berge und Flüsse gemacht hat. Und von allen Früchten hat Er auf ihr zwei, ein Paar, gemacht. Er läßt die Nacht den Tag überdecken. Darin sind wahrlich Zeichen für Leute, die nachdenken.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdccxi{Und auf der Erde sind nebeneinanderliegende Landstriche und Gärten mit Rebstöcken und (sonstige) Pflanzen und Palmen, mehrstämmig und einzelstämmig, die (alle) mit demselben Wasser bewässert werden. Wir zeichnen die einen von ihnen vor den anderen im Ernteertrag aus. Darin sind wahrlich Zei chen für Leute, die begreifen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdccxii{Und wenn du dich (schon) wunderst, so sind ihre Worte wunderlich: ``Sollen wir etwa, wenn wir zu Erde geworden sind, denn wirklich in neuer Schöpfung (erstehen)?" Das sind diejenigen, die ihren Herrn verleugnen; das sind diejenigen, die Fesseln um ihre Hälse haben; das sind die Insassen des (Höllen)feuers. Ewig werden sie darin bleiben.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdccxiii{Sie wünschen von dir, ihnen das Schlechte vor dem Guten zu beschleunigen, obwohl doch vor ihnen beispielgebende Strafen ergangen sind. Dein Herr ist gegen die Menschen wahrlich voll der Vergebung, trotz ihres Unrechts. Aber dein Herr ist wahrlich (auch) streng im Bestrafen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdccxiv{Diejenigen, die ungläubig sind, sagen: ``Wenn doch ein Zeichen von seinem Herrn auf ihn herabgesandt würde!" Du bist aber nur ein Überbringer von Warnungen. Und jedes Volk hat einen, der es rechtleitet.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdccxv{Allah weiß, womit jedes weibliche Wesen schwanger ist und wie der Mutterleib abnimmt und wie er zunimmt. Und alles hat bei Ihm ein Maß.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdccxvi{Er ist) der Kenner des Verborgenen und des Offenbaren, der Große und hoch Erhabene.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdccxvii{Es ist gleich, ob jemand von euch seine Worte geheimhält oder sie laut vernehmbar äußert und ob jemand sich bei Nacht verbirgt oder bei Tag offen hervortritt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdccxviii{Er hat vor sich und hinter sich Begleiter, die ihn auf Allahs Befehl beschützen. Allah ändert nicht den Zustand eines Volkes, bis sie das ändern, was in ihnen selbst ist. Und wenn Allah einem Volk Böses will, so kann es nicht zurückgewiesen werden. Und sie haben außer Ihm keinen Schutzherrn.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdccxix{Er ist es, Der euch den Blitz (als Grund) zur Angst und zum Begehren sehen und die schweren Wolken entstehen läßt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdccxx{Und der Donner lobpreist Ihn und (desgleichen) die Engel aus Furcht vor Ihm. Und Er sendet die Donnerschläge und trifft damit, wen Er will. Dabei streiten sie über Allah, wo Er doch stark im Streiten ist.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdccxxi{Ihm gebührt die wahre Anrufung. Diejenigen, die sie außer Ihm anrufen, erhören sie in nichts. Sie sind nur wie einer, der seine Hände nach Wasser ausstreckt, damit es seinen Mund erreiche, aber es erreicht ihn nicht. Aber das Bittgebet der Ungläubigen geht nur ins Leere.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdccxxii{Und vor Allah wirft sich nieder, wer in den Himmeln und auf der Erde ist, ob freiwillig oder widerwillig, und (auch) ihre Schatten, am Morgen und am Abend.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdccxxiii{Sag: Wer ist der Herr der Himmel und der Erde? Sag: Allah. Sag: Nehmt ihr euch denn außer Ihm Schutzherren, die sich selbst weder Nutzen noch Schaden zu bringen vermögen? Sag: Sind (etwa) der Blinde und der Sehende gleich? Oder sind (etwa) die Finsternisse und das Licht gleich? Oder haben sie Allah (solche) Teilhaber gegeben, die eine Schöpfung wie die Seinige schufen, so daß ihnen die Schöpfung ähnlich erscheint? Sag: Allah ist der Schöpfer von allem, und Er ist der Eine, der Allbezwinger.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdccxxiv{Er läßt vom Himmel Wasser herabkommen, und dann fließen Täler entsprechend ihrem Maß, daraufhin trägt die Flut aufschwellenden Schaum. Und aus dem, worüber man das Feuer anzündet, im Trachten (da)nach(,) Schmuck oder Gerät (anzufertigen, entsteht) ein ähnlicher Schaum. So prägt Allah (im Gleichnis) das Wahre und das Falsche. Was nun den Schaum angeht, so vergeht er nutzlos. Was aber den Menschen nützt, das bleibt in der Erde. So prägt Allah die Gleichnisse.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdccxxv{Für diejenigen, die auf ihren Herrn hören, ist das Beste. Diejenigen aber, die nicht auf Ihn hören, wenn sie alles hätten, was auf der Erde ist, und noch einmal das Gleiche dazu, würden sie sich sicherlich damit loskaufen. Für sie wird es eine böse Abrechnung geben, und ihr Zufluchtsort ist die Hölle - eine schlimme Lagerstatt!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdccxxvi{Ist etwa jemand, der weiß, daß das, was zu dir von deinem Herrn (als Offenbarung) herabgesandt worden ist, die Wahrheit ist, wie jemand, der blind ist? Jedoch bedenken nur diejenigen, die Verstand besitzen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdccxxvii{Diejenigen, die Allahs Bund halten und das Abkommen nicht brechen\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdccxxviii{und die verbinden, was Allah befohlen hat zu verbinden, ihren Herrn fürchten und Angst vor einer bösen Abrechnung haben\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdccxxix{und die geduldig sind im Trachten nach dem Angesicht ihres Herrn, das Gebet verrichten und von dem, womit Wir sie versorgt haben, heimlich und öffentlich ausgeben und mit dem Guten das Böse abwehren, für sie gibt es letztendlich die Wohnstätte,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdccxxx{die Gärten Edens, in die sie eingehen werden, (sie) und diejenigen, die rechtschaffen waren von ihren Vätern, ihren Gattinnen und ihren Nachkommenschaften. Und die Engel treten zu ihnen ein durch alle Tore:\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdccxxxi{``Friede sei auf euch dafür, daß ihr geduldig wart!" Wie trefflich ist die endgültige Wohnstätte!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdccxxxii{Diejenigen, die den Bund Allahs nach seiner Abmachung brechen und das trennen, was Allah befohlen hat, daß (es) verbunden werden soll, und auf der Erde Unheil stiften, für sie gibt es den Fluch und eine schlimme Wohnstätte.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdccxxxiii{Allah gewährt die Versorgung großzügig, wem Er will, und bemißt auch. Und sie sind froh über das diesseitige Leben; aber das diesseitige Leben (zählt) im Jenseits nur als (vergänglicher) Nießbrauch.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdccxxxiv{Und diejenigen, die ungläubig sind, sagen: ``Wenn doch ein Zeichen von seinem Herrn auf ihn herabgesandt würde!" Sag: Allah läßt in die Irre gehen, wen Er will, und leitet zu Sich, wer sich Ihm reuig zuwendet.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdccxxxv{Es sind) diejenigen, die glauben und deren Herzen im Gedenken Allahs Ruhe finden. Sicherlich, im Gedenken Allahs finden die Herzen Ruhe!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdccxxxvi{Diejenigen, die glauben und rechtschaffene Werke tun - für sie wird es Tuba geben, und für sie wird eine schöne Heimstatt (da)sein.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdccxxxvii{So haben Wir dich in eine Gemeinschaft gesandt, vor der Gemeinschaften dahingegangen sind, damit du ihnen verliest, was Wir dir (als Offenbarung) eingegeben haben; und doch verleugnen sie den Allerbarmer. Sag: Er ist mein Herr. Es gibt keinen Gott außer Ihm. Auf Ihn verlasse ich mich, und zu Ihm bereue ich.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdccxxxviii{Wenn es auch einen Qur'an gäbe, mit dem die Berge versetzt oder die Erde in Stücke gerissen oder zu den Toten gesprochen werden könnte ... (würden sie doch nicht glauben). Nein! Vielmehr steht die Angelegenheit ganz bei Allah. Wissen denn nicht diejenigen, die glauben, daß wenn Allah wollte, Er die Menschen wahrlich alle rechtleiten würde? Diejenigen, die ungläubig sind, wird immer wieder ein Verhängnis treffen für das, was sie gemacht haben, oder es wird in der Nähe ihrer Wohnstätten geschehen, bis Allahs Versprechen eintrifft. Gewiß, Allah bricht nicht, was Er versprochen hat.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdccxxxix{Man machte sich ja bereits über Gesandte vor dir lustig. Da gewährte Ich denen, die ungläubig waren, Aufschub; hierauf ergriff Ich sie. Wie war da Meine Bestrafung!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdccxl{Ist denn Derjenige, Der über jeder Seele steht, (um ihr zu vergelten) für das, was sie verdient hat, (den Götzen gleich)? Und dennoch geben sie Allah Teilhaber. Sag: Nennt sie. Oder wollt ihr Ihm etwas kundtun, das Er auf der Erde nicht kennt, oder über offenkundige Worte? Nein! Vielmehr sind denjenigen, die ungläubig sind, ihre Ränke ausgeschmückt worden, und sie sind vom (rechten) Weg abgehalten worden. Wen Allah in die Irre gehen läßt, der hat niemanden, der ihn rechtleitet.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdccxli{Für sie gibt es Strafe im diesseitigen Leben; aber die Strafe des Jenseits ist wahrlich härter. Und sie werden vor Allah keinen Behüter haben.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdccxlii{Dies ist) das Gleichnis des (Paradies)gartens, der den Gottesfürchtigen versprochen ist: Er ist durcheilt von Bächen'; sein Ernteertrag ist immerwährend und (auch) sein Schatten. Das ist das ist die letztendliche (Taten)folge derjenigen, die gottesfürchtig sind, während das endgültige Ziel der Ungläubigen das (Höllen)feuer ist.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdccxliii{Diejenigen, denen Wir die Schrift gegeben haben, sind froh über das, was (als Offenbarung) zu dir herabgesandt worden ist. Unter den Gruppierungen gibt es manche, die einen Teil davon nicht anerkennen. Sag: Mir ist befohlen worden, Allah zu dienen und Ihm nicht(s) beizugesellen. Zu Ihm rufe ich, und zu Ihm ist meine Heimkehr.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdccxliv{Und so haben Wir ihn als ein Urteil auf arabisch hinabgesandt. Wenn du ihren Neigungen folgst nach dem, was dir an Wissen zugekommen ist, wirst du vor Allah weder Schutzherrn noch Behüter haben.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdccxlv{Und Wir haben ja bereits Gesandte vor dir gesandt und ihnen Gattinnen und Nachkommenschaft gegeben. Kein Gesandter kann ein Zeichen bringen außer mit Allahs Erlaubnis. Jede Frist hat eine Vorbestimmung.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdccxlvi{Allah löscht aus, was Er will, und läßt bestehen; und bei Ihm ist der Kern des Buches.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdccxlvii{Ob Wir dich nun einen Teil dessen, was Wir ihnen androhen, sehen lassen oder dich nun (zuvor) abberufen, so obliegt dir nur die Übermittlung (der Botschaft), und Uns obliegt die Abrechnung.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdccxlviii{Sehen sie denn nicht, daß Wir über das Land kommen und es an seinen Enden kürzen? Allah (allein) richtet, und es gibt niemanden, der letztendlich Sein Urteil rückgängig machen kann. Und Er ist schnell im Abrechnen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdccxlix{Ränke haben bereits diejenigen vor ihnen geschmiedet. Aber alles an Ränken ist Allahs. Er weiß, was jede Seele erwirbt. Und die Ungläubigen werden zu wissen bekommen, für wen die letztendliche Wohnstätte sein wird.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdccl{Diejenigen, die ungläubig sind, sagen: ``Du bist nicht gesandt." Sag: Allah genügt als Zeuge zwischen mir und euch und derjenige, der das Wissen der Schrift hat'.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdccli{\basmalahde Alif-Lam-Ra. Dies ist ein Buch, das Wir zu dir hinabgesandt haben, damit du die Menschen mit der Erlaubnis ihres Herrn aus den Finsternissen hinaus ins Licht bringst, auf den Weg des Allmächtigen und Lobenswürdigen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdcclii{den Weg) Allahs, Dessen ist, was in den Himmeln und was auf der Erde. Und wehe den Ungläubigen vor einer strengen Strafe!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdccliii{Diejenigen, die das diesseitige Leben mehr lieben als das Jenseits und von Allahs Weg abhalten und danach trachten, ihn krumm zu machen; sie befinden sich in weit(reichend)em Irrtum.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdccliv{Und Wir haben keinen Gesandten gesandt, außer in der Sprache seines Volkes, damit er ihnen (die Botschaft) klar macht. Allah läßt dann in die Irre gehen, wen Er will, und leitet recht, wen Er will. Und Er ist der Allmächtige und Allweise.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdcclv{Und Wir sandten ja bereits Musa mit Unseren Zeichen: ``Bringe dein Volk aus den Finsternissen hinaus ins Licht, und erinnere sie an die Tage Allahs." Darin sind wahrlich Zeichen für jeden sehr Standhaften und sehr Dankbaren.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdcclvi{Und als Musa zu seinem Volk sagte: ``Gedenkt der Gunst Allahs an euch, als Er euch vor den Leuten Fir'auns rettete, die euch eine böse Qual auferlegten, eure Söhne abschlachteten und (nur) eure Frauen am Leben ließen. Seht, darin war für euch eine gewaltige Prüfung von eurem Herrn.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdcclvii{Und als euer Herr ankündigte: ,Wenn ihr dankbar seid, werde Ich euch ganz gewiß noch mehr (Gunst) erweisen. Wenn ihr jedoch undankbar seid, dann ist meine Strafe fürwahr streng'."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdcclviii{Und Musa sagte: ``Wenn ihr undankbar seid, ihr und alle, die auf der Erde sind, so ist Allah wahrlich Unbedürftig und Lobenswürdig."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdcclix{Ist zu euch nicht die Kunde von denjenigen vor euch gekommen, des Volkes Nuhs, der 'Ad und der Tamud und derjenigen nach ihnen? Nur Allah kennt sie. Ihre Gesandten kamen zu ihnen mit den klaren Beweisen; sie aber steckten (vor Grimm) ihre Hände in den Mund und sagten: ``Wir verleugnen das, womit ihr gesandt seid, und wir sind über das, wozu ihr uns aufruft, fürwahr in einem starken Zweifel."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdcclx{Ihre Gesandten sagten: ``Gibt es denn einen Zweifel über Allah, den Erschaffer der Himmel und der Erde, Der euch ruft, um euch (etwas) von euren Sünden zu vergeben und euch auf eine festgesetzte Frist zurückzustellen?" Sie sagten: ``Ihr seid nur menschliche Wesen wie wir. Ihr wollt uns von dem abhalten, dem unsere Väter dienten. So bringt uns eine deutliche Ermächtigung."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdcclxi{Ihre Gesandten sagten zu ihnen: ``Wir sind (zwar) nur menschliche Wesen wie ihr, aber Allah erweist Wohltaten, wem von Seinen Dienern Er will. Es steht uns nicht zu, eine Ermächtigung zu bringen - außer mit der Erlaubnis Allahs. Und auf Allah sollen sich die Gläubigen verlassen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdcclxii{Warum sollten wir uns nicht auf Allah verlassen, wo Er uns doch unsere Wege geleitet hat? Wir werden das, was ihr uns an Leid zufügt, ganz gewiß geduldig ertragen, und auf Allah sollen sich diejenigen verlassen, die sich (überhaupt auf jemanden) verlassen (wollen)."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdcclxiii{Diejenigen, die ungläubig waren, sagten zu ihren Gesandten: ``Wir werden euch ganz gewiß aus unserem Land vertreiben, oder aber ihr kehrt zu unserem Glaubensbekenntnis zurück!" Da gab ihnen ihr Herr (als Offenbarung) ein: ``Ganz gewiß werden Wir die Ungerechten vernichten.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdcclxiv{Und Wir werden euch ganz gewiß nach ihnen das Land bewohnen lassen. Dies (gilt) für denjenigen, der Meinen Stand fürchtet und Meine Androhung fürchtet."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdcclxv{Und sie riefen (Allah) um Sieg (über die Ungläubigen) an. Und enttäuscht wurde jeder widerspenstige Gewalthaber.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdcclxvi{Hinterdrein kommt für ihn die Hölle, und er bekommt eitriges Wasser zu trinken,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdcclxvii{das er schluckt, aber beinahe nicht hinunterbringt. Und der Tod kommt zu ihm von überall her, doch kann er nicht sterben. Und hinterdrein kommt harte Strafe.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdcclxviii{Dies ist) das Gleichnis derjenigen, die ihren Herrn verleugnen: Ihre Werke sind wie Asche, auf die der Wind an einem stürmischen Tag heftig bläst. Sie haben keine Macht über etwas von dem, was sie erworben haben. Das ist wirklich der tiefe Irrtum.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdcclxix{Siehst du nicht, daß Allah die Himmel und die Erde in Wahrheit erschaffen hat? Wenn Er will, läßt Er euch vergehen und bringt eine neue Schöpfung.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdcclxx{Und dies ist für Allah keineswegs schwer.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdcclxxi{Und sie erscheinen alle vor Allah. Dann sagen die Schwachen zu denjenigen, die sich hochmütig verhielten: ``Wir waren doch eure Gefolgsleute; könnt ihr uns nun vor Allahs Strafe etwas nützen?" Sie sagen: ``Wenn Allah uns rechtgeleitet hätte, fürwahr, hätten (auch) wir euch rechtgeleitet. Gleich ist es in Bezug auf uns, ob wir ängstlich sind oder standhaft bleiben; es gibt für uns kein Entrinnen."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdcclxxii{Und der Satan sagt, nachdem die Angelegenheit entschieden ist: ``Gewiß, Allah hat euch ein wahres Versprechen gegeben, und ich habe euch (etwas) versprochen, es aber dann gebrochen. Und ich hatte keine Macht über euch, außer daß ich euch gerufen habe und ihr auf mich gehört habt. So tadelt mich nicht, sondern tadelt euch selbst. Ich kann euch nicht zu Hilfe kommen, und ihr könnt mir nicht zu Hilfe kommen. Ich weise es ja von mir, daß ihr mich zuvor (Allah) beigesellt habt." Gewiß, für die Ungerechten gibt es schmerzhafte Strafe.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdcclxxiii{Aber diejenigen, die glauben und rechtschaffene Werke tun, werden in Gärten eingelassen, durcheilt von Bächen, ewig darin zu bleiben, mit der Erlaubnis ihres Herrn. Ihr Gruß darin wird sein: ``Friede!"\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdcclxxiv{Siehst du nicht, wie Allah ein Gleichnis von einem guten Wort geprägt hat? (Es ist) wie ein guter Baum, dessen Wurzeln fest sitzen und dessen Zweige in den Himmel (reichen).\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdcclxxv{Er bringt seinen Ernteertrag zu jeder Zeit (hervor) - mit der Erlaubnis seines Herrn. Und Allah prägt für die Menschen Gleichnisse, auf daß sie bedenken mögen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdcclxxvi{Und das Gleichnis eines schlechten Wortes ist wie ein schlechter Baum, der aus der Erde herausgerissen worden ist und keinen festen Grund (mehr) hat.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdcclxxvii{Allah festigt diejenigen, die glauben, durch das beständige Wort im diesseitigen Leben und im Jenseits. Doch Allah läßt die Ungerechten in die Irre gehen. Allah tut, was Er will.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdcclxxviii{Siehst du nicht jene, die Allahs Gunst gegen Undankbarkeit eingetauscht und ihr Volk in die Wohnstätte des Niedergangs versetzt haben,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdcclxxix{in die Hölle, der sie ausgesetzt sind? - Ein schlimmer Aufenthalt!\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdcclxxx{Und sie haben Allah andere als Seinesgleichen zur Seite gestellt, um (die Menschen) von Seinem Weg abirren zu lassen. Sag: Genießt nur, euer Ausgang wird ja in das (Höllen)feuer sein.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdcclxxxi{Sag zu Meinen Dienern, die glauben, sie sollen das Gebet verrichten und von dem, womit Wir sie versorgt haben, heimlich und öffentlich (als Spende) ausgeben, bevor ein Tag kommt, an dem es weder Verkaufen noch Freundschaften gibt.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdcclxxxii{Allah ist es, Der die Himmel und die Erde erschaffen hat und vom Himmel Wasser herabkommen läßt, durch das Er dann für euch Früchte als Versorgung hervorbringt. Und Er hat euch die Schiffe dienstbar gemacht, damit sie auf dem Meer auf Seinen Befehl fahren, und Er hat euch die Flüsse dienstbar gemacht.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdcclxxxiii{Er hat euch die Sonne und den Mond in ihrem unablässigen Lauf dienstbar gemacht, und Er hat euch die Nacht und den Tag dienstbar gemacht.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdcclxxxiv{Und Er gewährte euch von allem, worum ihr batet. Wenn ihr die Gunst(erweise) Allahs aufzählen wolltet, könntet ihr sie nicht erfassen. Gewiß, der Mensch ist wahrlich sehr oft ungerecht und sehr oft undankbar.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdcclxxxv{Und als Ibrahim sagte: ``Mein Herr, mache diese Ortschaft sicher, und lasse mich und meine Kinder es meiden, Götzen zu dienen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdcclxxxvi{Mein Herr, sie haben viele von den Menschen in die Irre geführt. Wer mir nun folgt, der gehört zu mir; und wer sich mir widersetzt, so bist Du ja Allvergebend und Barmherzig.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdcclxxxvii{Unser Herr, ich habe (einige) aus meiner Nachkommenschaft in einem Tal ohne Pflanzungen bei Deinem geschützten Haus wohnen lassen, unser Herr, damit sie das Gebet verrichten. So lasse die Herzen einiger Menschen sich ihnen zuneigen und versorge sie mit Früchten, auf daß sie dankbar sein mögen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdcclxxxviii{Unser Herr, Du weißt ja, was wir verbergen und was wir offenlegen; vor Allah ist nichts verborgen, weder auf der Erde noch im Himmel.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdcclxxxix{Alles) Lob gehört Allah, Der mir trotz meines hohen Alters Ismail und Ishaq geschenkt hat! Gewiß, mein Herr ist wahrlich der Erhörer des Gebets.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdccxc{Mein Herr, mach, daß ich das Gebet verrichte, (ich) und (auch einige) aus meiner Nachkommenschaft. Unser Herr, und nimm mein Gebet an.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdccxci{Unser Herr, vergib mir und meinen Eltern und den Gläubigen an dem Tag, da die Abrechnung stattfinden wird."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdccxcii{Und meine ja nicht, Allah sei unachtsam dessen, was die Ungerechten tun. Er stellt sie nur zurück bis zu einem Tag, an dem die Blicke starr werden,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdccxciii{sie kommen) hastend, die Köpfe hochhebend; ihr Blick kehrt nicht zu ihnen zurück, und ihre Herzen sind leer.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdccxciv{Und warne die Menschen vor dem Tag, an dem die Strafe über sie kommt. Da werden diejenigen sagen, die Unrecht taten: ``Unser Herr, stelle uns auf eine kurze Frist zurück, so werden wir Deinen Ruf erhören und den Gesandten folgen." ``Hattet ihr denn nicht zuvor geschworen, es würde für euch keinen Untergang geben?\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdccxcv{Ihr habt noch in den Wohnorten derer gewohnt, die sich selbst Unrecht zugefügt haben, und es ist euch klargeworden, wie Wir an ihnen gehandelt haben. Wir haben euch doch Beispiele geprägt."\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdccxcvi{Und sie haben bereits ihre Ränke geschmiedet, aber ihre Ränke sind bei Allah, auch wenn ihre Ränke derart sind, daß davor die Berge vergehen.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdccxcvii{So meine ja nicht, Allah würde Sein Versprechen gegenüber Seinen Gesandten brechen. Gewiß, Allah ist Allmächtig und Besitzer von Vergeltungsgewalt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdccxcviii{An dem Tag, da die Erde zu einer anderen Erde verändert werden wird, und (ebenso) die Himmel, und da sie vor Allah erscheinen werden, dem Einen, dem Allbezwinger.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdccxcix{An jenem Tag wirst du die Übeltäter in Ketten zusammengebunden sehen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdccc{Ihre Kleider werden aus Pech sein, und das Feuer wird ihre Gesichter überdecken,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdccci{damit Allah jeder Seele vergelte, was sie erworben hat. Gewiß, Allah ist schnell im Abrechnen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdcccii{Dies ist eine Botschaft an die Menschen, damit sie dadurch gewarnt werden und damit sie wissen, daß Er nur ein Einziger Gott ist, und damit diejenigen bedenken, die Verstand besitzen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdccciii{\basmalahde Alif-Lam-Ra. Dies sind die Zeichen des Buches und eines deutlichen Qur'ans.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdccciv{Vielleicht werden diejenigen, die ungläubig sind, wünschen, Muslime gewesen zu sein.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdcccv{Lasse sie nur essen und genießen und sich durch (falsche) Hoffnung ablenken lassen. Sie werden (es noch) erfahren.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdcccvi{Und wir haben keine Stadt vernichtet, ohne daß sie eine festgelegte Frist gehabt hätte.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdcccvii{Keine Gemeinschaft kann ihrer Frist vorausgehen, noch sie hinausschieben.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdcccviii{Und sie sagen: ``O du, dem (angeblich) die Ermahnung offenbart worden ist, du bist ja fürwahr besessen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdcccix{Warum bringst du uns nicht die Engel, wenn du zu den Wahrhaftigen gehörst?"\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdcccx{Wir senden die Engel nur mit der Wahrheit hinab. Dann wird ihnen kein Aufschub gewährt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdcccxi{Gewiß, Wir sind es, die Wir die Ermahnung offenbart haben, und Wir werden wahrlich ihr Hüter sein.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdcccxii{Wir haben ja bereits vor dir unter den Lagern der Früheren entsandt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdcccxiii{Und kein Gesandter kam zu ihnen, ohne daß sie sich über ihn lustig gemacht hätten.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdcccxiv{So lassen Wir ihn in die Herzen der Übeltäter einziehen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdcccxv{Sie glauben nicht daran, obwohl bereits die Gesetzmäßigkeit an den Früheren ergangen ist.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdcccxvi{Selbst wenn Wir ihnen ein Tor vom Himmel öffneten und sie dauernd dadurch hinaufstiegen,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdcccxvii{würden sie dennoch sagen: ``Unsere Blicke sind verschlossen. Nein! Vielmehr sind wir Leute, die einem Zauber verfallen sind."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdcccxviii{Und Wir haben ja im Himmel Türme gesetzt und ihn für die Betrachter ausgeschmückt,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdcccxix{und Wir haben ihn vor jedem gesteinigten Satan behütet,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdcccxx{außer demjenigen, der verstohlen zuhört, worauf ihn ein deutlich erkennbarer Leuchtkörper verfolgt.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdcccxxi{Und die Erde haben Wir gedehnt und darauf festgegründete Berge gesetzt und auf ihr von allen zu wiegenden Dingen wachsen lassen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdcccxxii{Und Wir haben auf ihr für euch (Möglichkeiten für den) Lebensunterhalt geschaffen und (auch) für diejenigen, die ihr nicht versorgt.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdcccxxiii{Und es gibt nichts, dessen Schatzkammern nicht bei Uns wären. Und Wir senden es nur in bestimmtem Maß hinab.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdcccxxiv{Und Wir senden die Winde zur Befruchtung. Und Wir lassen dann vom Himmel Wasser hinabkommen und geben es euch zu trinken, doch ihr könnt es nicht (alles davon) als Vorrat lagern.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdcccxxv{Und fürwahr, Wir sind es, die lebendig machen und sterben lassen, und Wir sind die (alles) Erbenden.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdcccxxvi{Und Wir kennen ja diejenigen unter euch, die vorausgehen, und Wir kennen ja (auch) diejenigen, die zurückbleiben.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdcccxxvii{Und gewiß, dein Herr wird sie versammeln. Er ist Allweise und Allwissend.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdcccxxviii{Und Wir haben ja den Menschen aus trockenem Ton, aus fauligem schwarzen Schlamm erschaffen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdcccxxix{Und die Ginn haben Wir zuvor aus dem Feuer des Glutwindes erschaffen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdcccxxx{Und als dein Herr zu den Engeln sagte: ``Ich bin dabei, ein menschliches Wesen aus trockenem Ton, aus fauligem schwarzen Schlamm zu erschaffen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdcccxxxi{Wenn Ich es zurechtgeformt und ihm von Meinem Geist eingehaucht habe, dann fallt und werft euch vor ihm nieder."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdcccxxxii{Da warfen sich die Engel alle zusammen nieder,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdcccxxxiii{außer Iblis; er weigerte sich, mit denen zu sein, die sich niederwerfen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdcccxxxiv{Er sagte: ``O lblis, was ist mit dir, daß du nicht mit denen bist, die sich niederwerfen?"\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdcccxxxv{Er sagte: ``Ich kann mich unmöglich vor einem menschlichen Wesen niederwerfen, das Du aus trockenem Ton, aus fauligem schwarzen Schlamm erschaffen hast."\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdcccxxxvi{Er sagte: ``Dann geh aus ihm hinaus, denn du bist der Steinigung würdig.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdcccxxxvii{Und auf dir liegt der Fluch bis zum Tag des Gerichts."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdcccxxxviii{Er sagte: ``Mein Herr, so gewähre mir Aufschub bis zu dem Tag, da sie auferweckt werden."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdcccxxxix{Er sagte: ``Gewiß, du gehörst zu denjenigen, denen Aufschub gewährt wird\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdcccxl{bis zum Tag der (wohl)bekannten Zeit."\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdcccxli{Er sagte: ``Mein Herr, darum, daß Du mich in Verirrung hast fallen lassen, werde ich ihnen ganz gewiß auf der Erde (das Böse) ausschmücken und sie ganz gewiß allesamt in Verirrung fallen lassen,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdcccxlii{außer Deinen Dienern, den auserlesenen unter ihnen."\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdcccxliii{Er sagte: ``Das ist ein gerader Weg, der (einzuhalten) Mir obliegt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdcccxliv{Gewiß, über Meine Diener hast du keine Macht, außer wer dir von den Verirrten folgt."\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdcccxlv{Und die Hölle ist wahrlich ihrer aller Verabredung(sort).\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdcccxlvi{Sie hat sieben Tore, und jedem Tor wird ein Teil von ihnen zugewiesen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdcccxlvii{Die Gottesfürchtigen aber werden in Gärten und an Quellen sein:\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdcccxlviii{``Betretet sie in Frieden und in Sicherheit."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdcccxlix{Und Wir nehmen weg, was in ihren Brüsten an Groll ist, als Brüder auf Liegen (ruhend), einander gegenüber.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdcccl{Darin widerfährt ihnen weder Mühsal, noch werden sie daraus vertrieben.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdcccli{Tue Meinen Dienern kund, daß Ich es bin, der Allvergebend und Barmherzig ist,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdccclii{und daß Meine Strafe die schmerzhafte Strafe ist.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdcccliii{Und gib ihnen Kunde über die Gäste Ibrahims.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdcccliv{Als sie bei ihm eintraten und sagten: ``Frieden!", sagte er: ``Wir ängstigen uns vor euch."\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdccclv{Sie sagten: ``Ängstige dich nicht. Wir verkünden dir einen kenntnisreichen Jungen."\qt@no{53}}%
\qt@newcmd\qurantrans@de@mdccclvi{Er sagte: ``Ihr verkündet (es) mir, obwohl mir das hohe Alter widerfahren ist! Was verkündet ihr mir denn da?"\qt@no{54}}%
\qt@newcmd\qurantrans@de@mdccclvii{Sie sagten: ``Wir verkünden (es) dir der Wahrheit entsprechend. So gehöre nicht zu denen, die die Hoffnung verlieren."\qt@no{55}}%
\qt@newcmd\qurantrans@de@mdccclviii{Er sagte: ``Wer verliert die Hoffnung auf die Barmherzigkeit seines Herrn außer den Irregehenden?"\qt@no{56}}%
\qt@newcmd\qurantrans@de@mdccclix{Er sagte: ``Was ist nun euer Auftrag, ihr Boten?"\qt@no{57}}%
\qt@newcmd\qurantrans@de@mdccclx{Sie sagten: ``Wir sind zu einem Volk von Übeltätern gesandt,\qt@no{58}}%
\qt@newcmd\qurantrans@de@mdccclxi{ausgenommen die Sippe Luts. Diese werden Wir wahrlich allesamt erretten,\qt@no{59}}%
\qt@newcmd\qurantrans@de@mdccclxii{außer seiner Frau." Wir haben (es so) bestimmt, sie gehörte fürwahr zu denen, die zurückbleiben.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mdccclxiii{Als nun die Gesandten zu der Sippe Luts kamen,\qt@no{61}}%
\qt@newcmd\qurantrans@de@mdccclxiv{sagte er: ``Ihr seid ja fremde Leute."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mdccclxv{Sie sagten: ``Nein! Vielmehr kommen wir zu dir mit dem, woran sie zu zweifeln pflegten.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mdccclxvi{Und wir kommen zu dir mit der Gewißheit, und wir sagen gewiß Wahrheit.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mdccclxvii{So zieh mit deinen Angehörigen in einem Teil der Nacht fort, und folge du hinterdrein. Und niemand von euch soll sich umwenden, geht weiter, wohin euch befohlen wird."\qt@no{65}}%
\qt@newcmd\qurantrans@de@mdccclxviii{Und Wir haben ihm diese Angelegenheit mitgeteilt, daß der letzte Rest dieser Leute bei Tagesanbruch ausgetilgt werde.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mdccclxix{Und die Bewohner der Stadt kamen voller Freude.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mdccclxx{Er sagte: ``Das sind meine Gäste, so stellt mich nicht bloß.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mdccclxxi{Und fürchtet Allah und stürzt mich nicht in Schande."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mdccclxxii{Sie sagten: ``Haben wir dir nicht die Weltenbewohner verboten (als Gäste aufzunehmen)?4"\qt@no{70}}%
\qt@newcmd\qurantrans@de@mdccclxxiii{Er sagte: ``Hier sind meine Töchter, wenn ihr etwas tun wollt."\qt@no{71}}%
\qt@newcmd\qurantrans@de@mdccclxxiv{Bei deinem Leben, sie irrten wahrlich in ihrer Trunkenheit umher.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mdccclxxv{Da ergriff sie der Schrei bei Sonnenaufgang,\qt@no{73}}%
\qt@newcmd\qurantrans@de@mdccclxxvi{und Wir kehrten das Oberste von ihr zuunterst und ließen auf sie Steine aus gebranntem Lehm regnen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mdccclxxvii{Darin sind wahrlich Zeichen für die Betrachtenden.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mdccclxxviii{Und sie liegt fürwahr an einem noch bestehenden Weg.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mdccclxxix{Darin ist wahrlich ein Zeichen für die Gläubigen.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mdccclxxx{Und die Bewohner des Dickichts taten fürwahr Unrecht,\qt@no{78}}%
\qt@newcmd\qurantrans@de@mdccclxxxi{so übten Wir an ihnen Vergeltung. Beide liegen fürwahr an einem deutlichen Weg.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mdccclxxxii{Und die Bewohner von al-Higr bezichtigten die Gesandten der Lüge.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mdccclxxxiii{Wir ließen ihnen Unsere Zeichen zukommen, aber sie wandten sich von ihnen ab.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mdccclxxxiv{Und sie hauten aus den Bergen Häuser aus, im Trachten nach Sicherheit.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mdccclxxxv{Da ergriff sie der Schrei bei Tagesanbruch;\qt@no{83}}%
\qt@newcmd\qurantrans@de@mdccclxxxvi{so nützte ihnen nicht, was sie erworben hatten.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mdccclxxxvii{Wir haben die Himmel und die Erde und was dazwischen ist nur in Wahrheit erschaffen. Gewiß, die Stunde wird sicher eintreffen. So übe schöne Nachsicht.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mdccclxxxviii{Dein Herr ist der Allerschaffer und der Allwissende.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mdccclxxxix{Und Wir haben dir doch sieben von den sich wiederholenden Versen gegeben, und (auch) den großartigen Qur'an.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mdcccxc{Richte ja nicht deine Augen auf das, was Wir manchen von ihnen paarweise als Nießbrauch gewähren. Und sei nicht traurig über sie. Und senke deinen Flügel für die Gläubigen,\qt@no{88}}%
\qt@newcmd\qurantrans@de@mdcccxci{und sag: Ich bin ja der deutliche Warner.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mdcccxcii{Wie Wir (die Strafe) auf diejenigen hinabgesandt haben, die aufteilten,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mdcccxciii{die den Qur'an (in einzelne Teile) zergliedert haben.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mdcccxciv{Ja, bei deinem Herrn! Wir werden sie allesamt ganz gewiß befragen\qt@no{92}}%
\qt@newcmd\qurantrans@de@mdcccxcv{über das, was sie zu tun pflegten.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mdcccxcvi{So verkünde denn laut, was dir befohlen wird, und wende dich von den Götzendienern ab.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mdcccxcvii{Wir genügen dir (als Schutz) vor den Spöttern,\qt@no{95}}%
\qt@newcmd\qurantrans@de@mdcccxcviii{die neben Allah einen anderen Gott setzen. Aber sie werden (es noch) erfahren.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mdcccxcix{Wir wissen ja, daß deine Brust beklommen ist wegen dessen, was sie sagen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcm{Aber lobpreise deinen Herrn und gehöre zu denen, die sich niederwerfen,\qt@no{98}}%
\qt@newcmd\qurantrans@de@mcmi{und diene deinem Herrn, bis die Gewißheit zu dir kommt.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mcmii{\basmalahde Der Befehl Allahs ist (so gut wie) eingetroffen, so wünscht nicht, ihn zu beschleunigen. Preis sei Ihm! Erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mcmiii{Er sendet die Engel mit dem Geist von Seinem Befehl herab, auf wen von Seinen Dienern Er will: ``Warnt (und verkündet), daß es keinen Gott gibt außer Mir; darum fürchtet Mich (allein)."\qt@no{2}}%
\qt@newcmd\qurantrans@de@mcmiv{Er hat die Himmel und die Erde in Wahrheit erschaffen. Erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mcmv{Er hat den Menschen aus einem Samentropfen erschaffen, und doch ist er sogleich ein deutlicher Widersacher.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mcmvi{Und (auch) das Vieh hat Er erschaffen. An ihm habt ihr Wärme und (allerlei anderen) Nutzen; und davon eßt ihr.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mcmvii{Und ihr habt an ihnen Schönes, wenn ihr (sie abends) eintreibt und wenn ihr (sie) morgens austreibt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mcmviii{Und sie tragen eure Lasten in ein Land, das ihr (sonst) nur mit größter Mühe hättet erreichen können. Euer Herr ist wahrlich Gnädig und Barmherzig.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mcmix{Und (erschaffen hat Er) die Pferde, die Maultiere und die Esel, damit ihr auf ihnen reitet, und (auch) als Schmuck. Und Er erschafft, was ihr nicht wißt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mcmx{Allah obliegt es, (euch) auf dem richtigen Weg zu halten. Es gibt ja (auch) manche, die abweichen. Wenn Er gewollt hätte, hätte Er euch fürwahr allesamt rechtgeleitet.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mcmxi{Er ist es, Der vom Himmel Wasser herabkommen läßt; davon habt ihr zu trinken, und davon (wachsen) Bäume, unter denen ihr (euer Vieh) frei weiden laßt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mcmxii{Er läßt euch damit Getreide wachsen, und Ölbäume, Palmen, Rebstöcke und von allen Früchten. Darin ist wahrlich ein Zeichen für Leute, die nachdenken.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mcmxiii{Und Er hat euch die Nacht und den Tag, die Sonne und den Mond dienstbar gemacht; und (auch) die Sterne sind durch Seinen Befehl dienstbar gemacht worden. Darin sind wahrlich Zeichen für Leute, die begreifen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mcmxiv{Und (dienstbar gemacht ist auch,) was Er euch auf der Erde in unterschiedlichen Farben hat wachsen lassen. Darin ist wahrlich ein Zeichen für Leute, die bedenken.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mcmxv{Und Er ist es, Der euch das Meer dienstbar gemacht hat, damit ihr frisches Fleisch daraus eßt und Schmuck aus ihm hervorholt, den ihr anlegt. Und du siehst die Schiffe es durchpflügen, damit ihr nach etwas von Seiner Huld trachtet und auf daß ihr dankbar sein möget!\qt@no{14}}%
\qt@newcmd\qurantrans@de@mcmxvi{Und Er hat auf der Erde festgegründete Berge gesetzt, daß sie nicht mit euch wanke, und Flüsse und Wege (geschaffen) - auf daß ihr rechtgeleitet werden möget\qt@no{15}}%
\qt@newcmd\qurantrans@de@mcmxvii{und als (Weg)zeichen. Und mit Hilfe der Sterne werden sie geleitet.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mcmxviii{Ist denn Derjenige, Der erschafft, wie derjenige, der nicht erschafft? Bedenkt ihr denn nicht?\qt@no{17}}%
\qt@newcmd\qurantrans@de@mcmxix{Und wenn ihr die Gunst(erweise) Allahs aufzählen wolltet, könntet ihr sie nicht erfassen. Allah ist wahrlich Allvergebend und Barmherzig.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mcmxx{Und Allah weiß, was ihr geheimhaltet und was ihr offenlegt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mcmxxi{Und diejenigen, die sie anstatt Allahs anrufen, erschaffen nichts, während sie selbst erschaffen werden.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mcmxxii{Tot (sind sie), nicht lebendig; und sie merken nicht, wann sie auferweckt werden.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mcmxxiii{Euer Gott ist ein Einziger Gott. Diejenigen aber, die nicht an das Jenseits glauben, deren Herzen weisen (es) ab, und sie verhalten sich hochmütig.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mcmxxiv{Zweifellos weiß Allah, was sie geheimhalten und was sie offenlegen. Gewiß, Er liebt nicht die Hochmütigen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mcmxxv{Und wenn zu ihnen gesagt wird: ``Was hat euer Herr (als Offenbarung) herabgesandt?", sagen sie: ``(Es sind) Fabeln der Früheren."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mcmxxvi{Deshalb sollen sie am Tag der Auferstehung ihre Lasten vollständig tragen, und (auch etwas) von den Lasten derjenigen, die sie ohne (richtiges) Wissen in die Irre führten. Wie böse ist das, was sie an Last auf sich nehmen!\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcmxxvii{Ränke schmiedeten bereits diejenigen, die vor ihnen waren. Da ging Allah ihren Bau an den Grundmauern an, so daß das Dach über ihnen auf sie herabfiel, und die Strafe über sie kam, von wo sie nicht merkten.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mcmxxviii{Hierauf, am Tag der Auferstehung, wird Er sie in Schande stürzen und sagen: ``Wo sind Meine Teilhaber, derentwegen ihr (Mir) entgegenwirktet?" Diejenigen, denen das Wissen gegeben wurde, sagen: ``Schande kommt heute und Böses über die Ungläubigen",\qt@no{27}}%
\qt@newcmd\qurantrans@de@mcmxxix{die die Engel abberufen, während sie sich selbst Unrecht zugefügt haben. Sie werden Frieden anbieten: ``Wir pflegten nichts Böses zu tun." - ``Doch, Allah weiß Bescheid über das, was ihr zu tun pflegtet.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mcmxxx{So betretet die Tore der Hölle, ewig darin zu bleiben. Schlimm ist fürwahr der Aufenthaltsort der Hochmütigen."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcmxxxi{Zu denjenigen, die gottesfürchtig sind, wird gesagt: ``Was hat euer Herr herabgesandt?" Sie sagen: ``Gutes." Für diejenigen, die Gutes tun, gibt es hier im Diesseits Gutes; aber die Wohnstätte des Jenseits ist wahrlich besser. Wie trefflich ist fürwahr die Wohnstätte der Gottesfürchtigen:\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcmxxxii{die Gärten Edens', in die sie eingehen, durcheilt von Bächen, und in denen sie (alles) haben, was sie wollen. So vergilt (es) Allah den Gottesfürchtigen,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcmxxxiii{die die Engel abberufen, während sie gute (Menschen) gewesen sind. Sie sagen: ``Friede sei auf euch! Geht in den (Paradies)garten ein für das, was ihr zu tun pflegtet."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcmxxxiv{Erwarten sie denn (etwas anderes), als daß die Engel zu ihnen kommen oder daß der Befehl deines Herrn eintrifft? So handelten (auch) diejenigen, die vor ihnen waren. Allah fügte ihnen kein Unrecht zu, sondern sie selbst fügten sich Unrecht zu.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcmxxxv{So trafen sie (schließlich) die bösen Taten, die sie begangen hatten, und es umschloß sie das, worüber sie sich lustig zu machen pflegten.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcmxxxvi{Diejenigen, die (Allah etwas) beigesellen, sagen: ``Wenn Allah gewollt hätte, hätten wir nichts anstatt Seiner gedient, weder wir noch unsere Vater, und wir (selbst) hätten anstatt Seiner nichts verboten." So handelten (auch) diejenigen, die vor ihnen waren. Obliegt denn den Gesandten etwas anderes als die deutliche Übermittlung (der Botschaft)?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcmxxxvii{Und Wir haben ja bereits in jeder Gemeinschaft einen Gesandten erweckt: ``Dient Allah und meidet die falschen Götter." Unter ihnen gibt es manche, die Allah rechtgeleitet hat, und unter ihnen gibt es manche, an denen sich das Irregehen bewahrheitet hat. So reist auf der Erde umher und schaut, wie das Ende der Leugner war.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcmxxxviii{Selbst wenn du noch so (danach) trachtest, sie rechtzuleiten, so leitet Allah nicht recht, wen Er (nun einmal) in die Irre gehen läßt; und sie werden keine Helfer haben.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcmxxxix{Und sie schwören bei Allah ihren kräftigsten Eid, Allah werde denjenigen nicht auferwecken, der stirbt. Doch (das ist) ein für Ihn bindendes Versprechen in Wahrheit - aber die meisten Menschen wissen nicht -,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcmxl{damit Er ihnen das klar macht, worüber sie uneinig sind, und damit diejenigen, die ungläubig sind, wissen, daß sie Lügner waren.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcmxli{Unser Wort zu etwas, wenn Wir es wollen, ist, dazu nur zu sagen: ``Sei!", und so ist es.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcmxlii{Und diejenigen, die um Allahs willen ausgewandert sind, nachdem ihnen Unrecht zugefügt wurde, denen werden Wir ganz gewiß im Diesseits Gutes zuweisen. Aber der Lohn des Jenseits ist wahrlich (noch) größer, wenn sie (es) nur wüßten!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mcmxliii{Sie), die standhaft sind und sich auf ihren Herrn verlassen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcmxliv{Und Wir haben vor dir nur Männer gesandt, denen Wir (Offenbarungen) eingegeben haben. So fragt die Leute der Ermahnung, wenn ihr (etwas) nicht wißt.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcmxlv{Wir haben sie gesandt) mit den klaren Beweisen und den Büchern der Weisheit. Und Wir haben zu dir die Ermahnung hinabgesandt, damit du den Menschen klar machst, was ihnen offenbart worden ist, und auf daß sie nachdenken mögen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mcmxlvi{Glauben denn diejenigen, die böse Ränke schmieden, sicher davor zu sein, daß Allah mit ihnen die Erde versinken läßt oder (daß) die Strafe über sie kommt, von wo sie nicht merken,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mcmxlvii{oder daß Er sie in ihrem Umherziehen ergreift, ohne daß sie sich (Ihm) entziehen können,\qt@no{46}}%
\qt@newcmd\qurantrans@de@mcmxlviii{oder daß Er sie in Verängstigung ergreift? Euer Herr ist wahrlich Gnädig und Barmherzig.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mcmxlix{Haben sie nicht auf (all) die Dinge gesehen, die Allah erschaffen hat, wie sich ihre Schatten nach rechts und links wenden, indem sie sich demütigend vor Allah niederwerfen?\qt@no{48}}%
\qt@newcmd\qurantrans@de@mcml{Vor Allah wirft sich (alles) nieder, was in den Himmeln und was auf der Erde an Tieren ist, und (auch) die Engel, und sie verhalten sich nicht hochmütig.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mcmli{Sie fürchten ihren Herrn über sich und tun, was ihnen befohlen wird.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mcmlii{Allah hat gesagt: Nehmt euch nicht zwei Götter. Er ist nur ein Einziger Gott. Vor Mir (allein) sollt ihr Ehrfurcht haben.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mcmliii{Ihm gehört (alles), was in den Himmeln und auf der Erde ist, und Sein ist immerwährend die Religion. Wollt ihr denn einen anderen als Allah fürchten?\qt@no{52}}%
\qt@newcmd\qurantrans@de@mcmliv{Was ihr an Gunst erfahrt, ist von Allah. Wenn euch hierauf ein Unheil widerfährt, so fleht ihr Ihn laut um Hilfe an.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mcmlv{Doch wenn Er hierauf das Unheil von euch hinweggenommen hat, ist da gleich eine Gruppe von euch dabei, ihrem Herrn (andere) beizugesellen,\qt@no{54}}%
\qt@newcmd\qurantrans@de@mcmlvi{um undankbar zu sein für das, was Wir ihnen gaben. So genießt nur; ihr werdet (es noch) erfahren.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mcmlvii{Und sie bestimmen für das, was sie nicht kennen', einen Anteil von dem, womit Wir sie versorgt haben. Bei Allah, ihr werdet ganz gewiß danach befragt werden, was ihr zu ersinnen pflegtet.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mcmlviii{Und sie bestimmen für Allah die Töchter - Preis sei Ihm! - und für sich (selbst), was sie begehren.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mcmlix{Wenn einem von ihnen die frohe Botschaft (von der Geburt) eines Mädchens verkündet wird, bleibt sein Gesicht finster, und er hält (seinen Grimm) zurück.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mcmlx{Er verbirgt sich vor den Leuten wegen der schlimmen Nachricht, die ihm verkündet worden ist. Soll er es trotz der Schmach behalten oder es in die Erde stecken? Wie böse ist, was sie urteilen!\qt@no{59}}%
\qt@newcmd\qurantrans@de@mcmlxi{Diejenigen, die an das Jenseits nicht glauben, haben die Eigenschaft des Bösen. Allah aber hat die höchste Eigenschaft, und Er ist der Allmächtige und Allweise.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mcmlxii{Wenn Allah die Menschen für ihre Ungerechtigkeit belangen wollte, würde Er auf ihr kein Tier übriglassen. Aber Er stellt sie auf eine festgesetzte Frist zurück. Und wenn dann ihre Frist kommt, können sie (sie) weder um eine Stunde hinausschieben noch vorverlegen.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mcmlxiii{Und sie geben Allah, was ihnen (selbst) zuwider ist, und ihre Zungen behaupten die Lüge, daß für sie (nur) das Beste ist. Zweifellos ist für sie das (Höllen)feuer und daß sie vernachlässigt werden.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mcmlxiv{Bei Allah, Wir haben bereits (Propheten) zu Gemeinschaften vor dir gesandt. Da schmückte ihnen der Satan ihre Taten aus. So ist er heute ihr Schutzherr, und für sie wird es schmerzhafte Strafe geben.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mcmlxv{Und Wir haben auf dich das Buch nur hinabgesandt, damit du ihnen das klar machst, worüber sie uneinig gewesen sind, und als Rechtleitung und Barmherzigkeit für Leute, die glauben.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mcmlxvi{Und Allah läßt vom Himmel Wasser herabkommen und macht dann damit die Erde nach ihrem Tod wieder lebendig. Darin ist wahrlich ein Zeichen für Leute, die hören (können).\qt@no{65}}%
\qt@newcmd\qurantrans@de@mcmlxvii{Gewiß, auch im Vieh habt ihr wahrlich eine Lehre. Wir geben euch von dem, was in ihren Leibern zwischen Kot und Blut ist, zu trinken, reine Milch, angenehm für diejenigen, die (sie) trinken.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mcmlxviii{Und (Wir geben euch) von den Früchten der Palmen und der Rebstöcke (zu trinken), woraus ihr euch Rauschgetränk und eine schöne Versorgung nehmt. Darin ist wahrlich ein Zeichen für Leute, die begreifen.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mcmlxix{Und dein Herr hat der Biene eingegeben: ``Nimm dir in den Bergen Häuser, in den Bäumen und in dem, was sie an Spalieren errichten.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mcmlxx{Hierauf iß von allen Früchten, ziehe auf den Wegen deines Herrn dahin, die (dir) geebnet sind." Aus ihren Leibern kommt ein Getränk von unterschiedlichen Farben, in dem Heilung für die Menschen ist. Darin ist wahrlich ein Zeichen für Leute, die nachdenken.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mcmlxxi{Und Allah hat euch erschaffen, hierauf beruft Er euch ab. Und manch einer von euch wird in das niedrigste (Greisen)alter zurückgebracht, so daß er nach (vorherigem) Wissen nichts (mehr) weiß. Gewiß, Allah ist Allwissend und hat Macht.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mcmlxxii{Und Allah hat die einen von euch vor den anderen in der Versorgung bevorzugt. Doch geben diejenigen, die bevorzugt werden, ihre Versorgung nicht an diejenigen zurück, die ihre rechte Hand besitzt, so daß sie darin gleich wären. Wollen sie denn die Gunst Allahs verleugnen?\qt@no{71}}%
\qt@newcmd\qurantrans@de@mcmlxxiii{Und Allah hat euch aus euch selbst Gattinnen gemacht und von euren Gattinnen Söhne und Enkel gemacht und euch von den guten Dingen versorgt. Wollen sie denn an das Falsche glauben und Allahs Gunst verleugnen,\qt@no{72}}%
\qt@newcmd\qurantrans@de@mcmlxxiv{und (wollen sie) anstatt Allahs dienen, was ihnen keine Versorgung in den Himmeln und auf der Erde (zu geben) vermag und (auch) nicht (dazu) in der Lage sein wird?\qt@no{73}}%
\qt@newcmd\qurantrans@de@mcmlxxv{So prägt Allah keine Gleichnisse! Allah weiß, ihr aber wißt nicht.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mcmlxxvi{Allah prägt das Gleichnis eines leibeigenen Dieners, der über nichts Macht hat, und jemandes, dem Wir von Uns her eine schöne Versorgung gewährt haben, so daß er davon heimlich und offen ausgibt. Sind sie (etwa) gleich? (Alles) Lob gehört Allah! Nein! Vielmehr wissen die meisten von ihnen nicht.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mcmlxxvii{Und Allah prägt das Gleichnis zweier Männer. Der eine von beiden ist stumm und hat über nichts Macht; er ist seinem Herrn eine Last; wo auch immer er ihn hinschickt, bringt er nichts Gutes. Ist er etwa jemandem gleich, der Gerechtigkeit gebietet und sich auf einem geraden Weg befindet?\qt@no{76}}%
\qt@newcmd\qurantrans@de@mcmlxxviii{Und Allahs ist das Verborgene der Himmel und der Erde. Und die Angelegenheit der Stunde ist nur wie ein Augenblick, oder sie ist noch näher. Gewiß, Allah hat zu allem die Macht.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mcmlxxix{Und Allah hat euch aus den Leibern eurer Mütter hervorgebracht, während ihr nichts wußtet. Und Er hat euch Gehör, Augenlicht und Herzen gegeben, auf daß ihr dankbar sein möget.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mcmlxxx{Haben sie nicht auf die Vögel gesehen, die im Luftraum des Himmels dienstbar gemacht worden sind? Nur Allah hält sie (oben). Darin sind wahrlich Zeichen für Leute, die glauben.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mcmlxxxi{Und Allah hat euch aus euren Häusern eine Ruhestätte gemacht, und Er hat euch aus den Häuten des Viehs Behausungen gemacht, die ihr am Tag eures Aufbrechens und am Tag eures Aufenthaltes leicht benutzen könnt, und aus ihrer Wolle, ihren Fellhärchen und ihrem Haar Ausstattung und Nießbrauch auf Zeit.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mcmlxxxii{Und Allah hat euch aus dem, was Er erschaffen hat, Schattenspender gemacht. Und Er hat euch in den Bergen Deckung gegeben. Und Er hat euch Kleider gemacht, die euch vor der Hitze schützen, und Kleider, die euch vor eurer Gewalt (gegeneinander) schützen. So vollendet Er Seine Gunst an euch, auf daß ihr (Ihm) ergeben sein möget.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mcmlxxxiii{Wenn sie sich abkehren, so obliegt dir nur die deutliche Übermittlung (der Botschaft).\qt@no{82}}%
\qt@newcmd\qurantrans@de@mcmlxxxiv{Sie erkennen die Gunst Allahs, und sie erkennen sie hierauf nicht an. Die meisten von ihnen sind undankbar.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mcmlxxxv{Und (gedenke) des Tages, da Wir von jeder Gemeinschaft einen Zeugen erwecken. Dann wird denjenigen, die ungläubig sind, keine Erlaubnis gegeben (, sich zu entschuldigen), und es wird ihnen keine Gnade gewährt.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mcmlxxxvi{Und wenn diejenigen, die Unrecht taten, die Strafe sehen, dann wird sie ihnen weder erleichtert noch wird ihnen Aufschub gewährt.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mcmlxxxvii{Und wenn diejenigen, die (Allah andere) beigesellten, ihre Teilhaber sehen, sagen sie: ``Unser Herr, das sind unsere Teilhaber, die wir anstatt Deiner anzurufen pflegten." Sie aber richteten an sie das Wort: ``Ihr seid fürwahr Lügner."\qt@no{86}}%
\qt@newcmd\qurantrans@de@mcmlxxxviii{Sie bieten Allah an jenem Tag Ergebenheit an, und es ist ihnen entschwunden, was sie zu ersinnen pflegten.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mcmlxxxix{Denjenigen, die ungläubig sind und von Allahs Weg abhalten, fügen Wir noch Strafe zu über ihre Strafe hinaus dafür, daß sie Unheil zu stiften pflegten.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mcmxc{Und (gedenke) des Tages, da Wir in jeder Gemeinschaft einen Zeugen über sie aus ihren eigenen Reihen erwecken. Dich bringen Wir als Zeugen über diese da herbei. Und Wir haben dir das Buch offenbart als klare Darlegung von allem und als Rechtleitung, Barmherzigkeit und frohe Botschaft für die (Allah) Ergebenen'.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mcmxci{Allah gebietet Gerechtigkeit, gütig zu sein und den Verwandten zu geben; Er verbietet das Schändliche, das Verwerfliche und die Gewalttätigkeit. Er ermahnt euch, auf daß ihr bedenken möget.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mcmxcii{Und haltet den Bund Allahs, wenn ihr einen Bund geschlossen habt, und brecht nicht die Eide nach ihrer Bekräftigung, wo ihr doch Allah zum Bürgen über euch gemacht habt. Gewiß, Allah weiß, was ihr tut.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mcmxciii{Und seid nicht wie jene, die ihr Garn, nachdem es fest gesponnen war, wieder in aufgelöste Strähnen bricht, indem ihr eure Eide untereinander als Mittel des Betrugs nehmt, weil eine Gemeinschaft zahlreicher ist als eine andere Gemeinschaft. Allah prüft euch damit nur, und Er wird euch am Tag der Auferstehung ganz gewiß über das Klarheit geben, worüber ihr uneinig zu sein pflegtet.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mcmxciv{Und wenn Allah gewollt hätte, hätte Er euch wahrlich zu einer einzigen Gemeinschaft gemacht. Aber Er läßt in die Irre gehen, wen Er will, und Er leitet recht, wen Er will, und ihr werdet ganz gewiß danach befragt werden, was ihr zu tun pflegtet.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mcmxcv{Und nehmt euch nicht eure Eide als Mittel des Betrugs untereinander; sonst würde ein Fuß straucheln, nachdem er fest gestanden hat, und ihr würdet Böses erleiden dafür, daß ihr von Allahs Weg abgehalten habt, und für euch wird es gewaltige Strafe geben.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mcmxcvi{Und verkauft den Bund Allahs nicht für einen geringen Preis. Gewiß, was bei Allah ist, ist besser für euch, wenn ihr (es) nur wißt.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mcmxcvii{Was bei euch ist, geht zu Ende; was aber bei Allah ist, ist bleibend. Wir werden denjenigen, die standhaft sind, ganz gewiß mit ihrem Lohn das Beste vergelten von dem, was sie taten.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mcmxcviii{Wer rechtschaffen handelt, sei es Mann oder Frau, und dabei gläubig ist, den werden Wir ganz gewiß ein gutes Leben leben lassen. Und Wir werden ihnen ganz gewiß mit ihrem Lohn das Beste von dem vergelten, was sie taten.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcmxcix{Wenn du nun den Qur'an vorträgst, so suche Schutz bei Allah vor dem gesteinigten Satan.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mm{Er hat gewiß keine Macht über diejenigen, die glauben und sich auf ihren Herrn verlassen.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmi{Seine Macht erstreckt sich nur über diejenigen, die ihn zum Schutzherrn nehmen und die ihn (Allah) beigesellen.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmii{Und wenn Wir einen Vers anstelle eines (anderen) Verses austauschen - und Allah weiß sehr wohl, was Er offenbart -, sagen sie: ``Du ersinnst nur Lügen." Aber nein! Die meisten von ihnen wissen nicht.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmiii{Sag: Offenbart hat ihn der Heilige Geist von deinem Herrn mit der Wahrheit, um diejenigen, die glauben, zu festigen, und als Rechtleitung und frohe Botschaft für die (Allah) Ergebenen.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmiv{Und Wir wissen sehr wohl, daß sie sagen: ``Es lehrt ihn nur ein menschliches Wesen." Die Sprache dessen, auf den sie hinweisen, ist eine fremde, während dies hier deutliche arabische Sprache ist.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmv{Diejenigen, die nicht an Allahs Zeichen glauben, leitet Allah nicht recht, und für sie wird es schmerzhafte Strafe geben.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmvi{Lügen ersinnen ja nur diejenigen, die nicht an Allahs Zeichen glauben. Das sind die (wahren) Lügner.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmvii{Wer Allah verleugnet, nachdem er den Glauben (angenommen) hatte - außer demjenigen, der gezwungen wird, während sein Herz im Glauben Ruhe gefunden hat -, doch wer aber seine Brust dem Unglauben auftut, über diejenigen kommt Zorn von Allah, und für sie wird es gewaltige Strafe geben.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmviii{Dies (wird sein), weil sie das diesseitige Leben mehr lieben als das Jenseits und weil Allah das ungläubige Volk nicht rechtleitet.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmix{Das sind diejenigen, deren Herzen, Gehör und Augenlicht Allah versiegelt hat; und das sind (überhaupt) die Unachtsamen.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmx{Zweifellos sind sie im Jenseits die Verlierer.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmxi{Hierauf wird dein Herr zu denjenigen, die ausgewandert sind, nachdem sie der Versuchung ausgesetzt wurden, und sich hierauf abgemüht haben und standhaft gewesen sind - gewiß, dein Herr wird nach (all) diesem (zu ihnen) wahrlich Allvergebend und Barmherzig sein.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmxii{Am Tag, da jede Seele kommt und für sich selbst streitet, und jeder Seele in vollem Maß zukommen wird, was sie getan hat. Und es wird ihnen kein Unrecht zugefügt.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmxiii{Allah prägt das Gleichnis einer Stadt, die Sicherheit und Ruhe genoß; ihre Versorgung kam zu ihr reichlich von überall her. Da wurde sie gegenüber den Gnaden Allahs undankbar. So ließ sie Allah das Kleid des Hungers und der Angst erleiden für das, was sie machten.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmxiv{Ein Gesandter von ihnen kam doch zu ihnen, sie bezichtigten ihn aber der Lüge; so ergriff sie die Strafe, während sie Unrecht taten.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmxv{So eßt von dem, womit Allah euch versorgt hat, als etwas Erlaubtes und Gutes, und seid dankbar für die Gunst Allahs, wenn ihr Ihm allein dient.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmxvi{Verboten hat Er euch nur (den Genuß von) Verendetem, Blut, Schweinefleisch und dem, worüber ein anderer (Name) als Allah(s) angerufen worden ist. Wer sich aber in einer Zwangslage befindet, ohne zu begehren oder das Maß zu überschreiten, so ist Allah Allvergebend und Barmherzig.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmxvii{Und sagt nicht von dem, was eure Zungen als Lüge behaupten: ``Das ist erlaubt, und das ist verboten", um gegen Allah eine Lüge zu ersinnen. Gewiß, denjenigen, die gegen Allah eine Lüge ersinnen, wird es nicht wohl ergehen.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmxviii{Es ist nur) ein geringer Genuß, und für sie wird es schmerzhafte Strafe geben.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmxix{Und denjenigen, die dem Judentum angehören, haben Wir das verboten, worüber Wir dir zuvor berichtet haben. Und nicht Wir haben ihnen Unrecht getan, sondern sie haben sich selbst Unrecht zugefügt.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmxx{Hierauf wird dein Herr zu denjenigen, die in Unwissenheit Böses tun, aber nach alledem bereuen und (es) wieder gutmachen - gewiß, dein Herr wird danach (zu ihnen) wahrlich Allvergebend und Barmherzig sein.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmxxi{Ibrahim war eine Gemeinschaft, Allah demütig ergeben und einer, der Anhänger des rechten Glaubens war, und er gehörte nicht zu den Götzendienern;\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmxxii{dankbar (war er) für Seine Gnaden. Er hatte ihn erwählt und zu einem geraden Weg geleitet.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmxxiii{Und Wir hatten ihm im Diesseits Gutes gegeben, und im Jenseits gehört er fürwahr zu den Rechtschaffenen.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmxxiv{Und hierauf haben Wir dir (als Offenbarung) eingegeben: ``Folge dem Glaubensbekenntnis Ibrahims, (als) Anhänger des rechten Glaubens, und er gehörte nicht zu den Götzendienern."\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmxxv{Der Sabbat ist nur denjenigen auferlegt worden, die über ihn uneinig waren. Und dein Herr wird wahrlich am Tag der Auferstehung zwischen ihnen über das richten, worüber sie uneinig zu sein pflegten.\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmxxvi{Rufe zum Weg deines Herrn mit Weisheit und schöner Ermahnung, und streite mit ihnen in bester Weise. Gewiß, dein Herr kennt sehr wohl, wer von Seinem Weg abirrt, und Er kennt sehr wohl die Rechtgeleiteten.\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmxxvii{Und wenn ihr bestraft, so bestraft im gleichen Maß, wie ihr bestraft wurdet. Wenn ihr aber geduldig seid, so ist das wahrlich besser für die Geduldigen.\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmxxviii{Sei standhaft; deine Standhaftigkeit ist nur durch Allah (möglich). Sei nicht traurig über sie, und sei nicht in Beklommenheit wegen der Ränke, die sie schmieden.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmxxix{Gewiß, Allah ist mit denjenigen, die gottesfürchtig sind und Gutes tun.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmxxx{\basmalahde Preis sei Dem, Der Seinen Diener bei Nacht von der geschützten Gebetsstätte zur fernsten Gebetsstätte, deren Umgebung Wir gesegnet haben, reisen ließ, damit Wir ihm (etwas) von Unseren Zeichen zeigen. Er ist ja der Allhörende, der Allsehende.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmxxxi{Und Wir gaben Musa die Schrift und machten sie zu einer Rechtleitung für die Kinder Isra'ils: ``Nehmt euch außer Mir keinen Sachwalter,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmxxxii{o ihr,) die Nachkommenschaft derer, die Wir mit Nuh trugen. Gewiß, er war ein dankbarer Diener."\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmxxxiii{Und Wir haben für die Kinder Isra'ils im Buch entschieden: ``Ihr werdet ganz gewiß zweimal auf der Erde Unheil stiften, und ihr werdet ganz gewiß mächtige Überheblichkeit erlangen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmxxxiv{Wenn nun das Versprechen vom ersten der beiden (Male) eintrifft, schicken Wir gegen euch Diener von Uns, die eine starke Gewalt besitzen. Sie dringen zwischen den Wohnstätten hindurch ein, und das ist ein Versprechen, das sicher ausgeführt wird.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmxxxv{Hierauf geben Wir euch wiederum die Oberhand über sie, und Wir unterstützen euch mit Besitz und Söhnen und machen euch zahlreicher.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmxxxvi{``Wenn ihr Gutes tut, tut ihr Gutes für euch selbst; und wenn ihr Böses tut, ist es (auch) für euch selbst." - Wenn nun das Versprechen vom letzten (Mal) eintrifft, so sollen sie eure Gesichter entstellen und die Gebetsstätte betreten, wie sie diese das erste Mal betraten, und das, worüber sie Macht erlangt haben, völlig zerstören.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmxxxvii{Vielleicht erbarmt euer Herr Sich eurer; doch wenn ihr (dazu) zurückkehrt, kehren (auch) Wir zurück. Und Wir haben die Hölle für die Ungläubigen zum Gefängnis gemacht."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmxxxviii{Gewiß, dieser Qur'an leitet zu dem, was richtiger ist, und verkündet den Gläubigen, die rechtschaffene Werke tun, daß es für sie großen Lohn geben wird\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmxxxix{und daß Wir denjenigen, die an das Jenseits nicht glauben, schmerzhafte Strafe bereitet haben.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmxl{Der Mensch ruft (zu Allah) um das Schlechte, wie er um das Gute ruft; der Mensch ist ja stets voreilig.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmxli{Und Wir haben die Nacht und den Tag zu zwei Zeichen gemacht. Dann haben Wir das Zeichen der Nacht ausgelöscht und das Zeichen des Tages hell gemacht, damit ihr nach Huld von eurem Herrn trachtet und damit ihr die Zahl der Jahre und die (Zeit)rechnung wißt. Und alles haben Wir ganz ausführlich dargelegt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmxlii{Jedem Menschen haben Wir sein Vorzeichen an seinem Hals befestigt. Und am Tag der Auferstehung bringen Wir ihm ein Buch heraus, das er aufgeschlagen vorfinden wird:\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmxliii{``Lies dein Buch! Du selbst genügst heute als Abrechner über dich."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmxliv{Wer der Rechtleitung folgt, der ist nur zu seinem eigenen Vorteil rechtgeleitet. Und wer irregeht, der geht nur zu seinem Nachteil irre. Und keine lasttragende (Seele) nimmt die Last einer anderen auf sich. Wir strafen nicht eher, bis Wir einen Gesandten geschickt haben.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmxlv{Und wenn Wir eine Stadt vernichten wollen, befehlen Wir denjenigen, die in ihr üppig leben (, zu freveln), und dann freveln sie in ihr. So bewahrheitet sich das Wort gegen sie, und dann zerstören Wir sie vollständig.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmxlvi{Wie viele Geschlechter nach Nuh haben Wir vernichtet! Und es genügt, daß dein Herr die Sünden Seiner Diener wohl kennt und sieht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmxlvii{Wer immer das schnell Eintreffende will, dem gewähren Wir darin schnell, was Wir wollen - demjenigen, den Wir wollen; hierauf haben Wir für ihn die Hölle bestimmt, der er ausgesetzt sein wird, mit Vorwürfen behaftet und verstoßen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmxlviii{Wer das Jenseits will und sich darum bemüht, wie es ihm zusteht, wobei er gläubig ist, - denen wird für ihr Bemühen gedankt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmxlix{Sie alle, diese und jene, unterstützen Wir mit etwas von der Gabe deines Herrn. Und die Gabe deines Herrn wird nicht verwehrt.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mml{Schau, wie Wir die einen von ihnen vor den anderen auszeichnen; aber das Jenseits ist wahrlich größer an Rangstufen und größer an Auszeichnungen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmli{Setze neben Allah keinen anderen Gott, sonst wirst du gescholten und im Stich gelassen dasitzen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmlii{Und dein Herr hat bestimmt, daß ihr nur Ihm dienen und zu den Eltern gütig sein sollt. Wenn nun einer von ihnen oder beide bei dir ein hohes Alter erreichen, so sag nicht zu ihnen: ``Pfui!" und fahre sie nicht an, sondern sag zu ihnen ehrerbietige Worte.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmliii{Und senke für sie aus Barmherzigkeit den Flügel der Demut und sag: ``Mein Herr, erbarme Dich ihrer, wie sie mich aufgezogen haben, als ich klein war."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmliv{Euer Herr weiß sehr wohl, was in eurem Innersten ist. Wenn ihr rechtschaffen seid, so ist Er gewiß für die sich (zu Ihm) stets Bekehrenden Allvergebend.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmlv{Und gib dem Verwandten sein Recht, ebenso dem Armen und dem Sohn des Weges. Und handle nicht ganz verschwenderisch.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmlvi{Gewiß, die Verschwender sind die Brüder der Satane; und der Satan ist gegenüber seinem Herrn sehr undankbar.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmlvii{Doch wenn du dich nun von ihnen abwendest - im Trachten nach einer Barmherzigkeit von deinem Herrn, die du dir erhoffst -, so sag zu ihnen milde Worte.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmlviii{Und lasse deine Hand nicht an deinem Hals gefesselt sein, strecke sie aber auch nicht vollständig aus, sonst würdest du getadelt und (aller Mittel) entblößt dasitzen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmlix{Gewiß, dein Herr gewährt die Versorgung großzügig, wem Er will, und bemißt auch. Gewiß, Er kennt und sieht Seine Diener wohl.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmlx{Und tötet nicht eure Kinder aus Furcht vor Verarmung; Wir versorgen sie und auch euch. Gewiß, sie zu töten ist ein großes Vergehen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmlxi{Und nähert euch nicht der Unzucht. Gewiß, sie ist etwas Abscheuliches - und wie böse ist der Weg.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmlxii{Und tötet nicht die Seele, die Allah verboten hat (zu töten), außer aus einem rechtmäßigen Grund. Wer ungerechterweise getötet wird, dessen nächstem Verwandten haben Wir Ermächtigung erteilt (, Recht einzufordern); doch soll er nicht maßlos im Töten sein, denn ihm wird gewiß geholfen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmlxiii{Und nähert euch nicht dem Besitz des Waisenkindes, außer auf die beste Art, bis es seine Vollreife erlangt hat. Und erfüllt die (eingegangene) Verpflichtung. Gewiß, nach der (Erfüllung der) Verpflichtung wird gefragt werden.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmlxiv{Und gebt volles Maß, wenn ihr meßt, und wägt mit der richtigen Waage; das ist besser und eher ein guter Ausgang.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmlxv{Und verfolge nicht das, wovon du kein Wissen hast. Gewiß, Gehör, Augenlicht und Herz, - all diese -, danach wird gefragt werden.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmlxvi{Und gehe nicht übermütig auf der Erde einher. Du wirst ja die Erde nicht aufreißen noch die Berge an Höhe erreichen (können).\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmlxvii{Das schlechte (Verhalten) in alledem ist bei deinem Herrn verabscheut.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmlxviii{Das ist etwas von dem, was dir dein Herr an Weisheit (als Offenbarung) eingegeben hat. Und setze neben Allah keinen anderen Gott, sonst wirst du in die Hölle geworfen, getadelt und verstoßen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmlxix{Hat denn euer Herr für euch die Söhne erwählt und Sich selbst unter den Engeln Töchter genommen? Ihr sagt da fürwahr ein gewaltiges Wort.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmlxx{Wir haben doch in diesem Qur'an (die Botschaft) verschiedenartig dargelegt, damit sie bedenken, doch mehrt dies ihnen nur die Abneigung.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmlxxi{Sag: Wenn es neben Ihm noch (andere) Götter gäbe, wie sie sagen, dann würden sie wahrlich nach einem Weg zum Besitzer des Thrones trachten.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmlxxii{Preis sei Ihm! Überaus hoch Erhaben ist Er über das, was sie sagen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmlxxiii{Ihn preisen die sieben Himmel und die Erde, und wer in ihnen ist. Es gibt nichts, was Ihn nicht lobpreist; ihr aber versteht ihr Preisen nicht. Gewiß, Er ist Nachsichtig und Allvergebend.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmlxxiv{Wenn du den Qur'an vorträgst, machen Wir zwischen dir und denjenigen, die an das Jenseits nicht glauben, einen unsichtbaren Vorhang.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmlxxv{Und Wir legen auf ihre Herzen Hüllen, so daß sie ihn nicht verstehen, und in ihre Ohren Schwerhörigkeit. Und wenn du im Qur'an deinen Herrn allein (ohne ihre Götter) erwähnst, kehren sie aus Abneigung den Rücken.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmlxxvi{Wir wissen sehr wohl, worauf sie hören, wenn sie dir zuhören und (auch) wenn sie sich in vertraulichen Gesprächen befinden, wenn die Ungerechten sagen: ``Ihr folgt doch nur einem Mann, der einem Zauber verfallen ist."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmlxxvii{Schau, wie sie dir Gleichnisse prägen, und so sind sie abgeirrt, daß sie keinen Weg (mehr) finden können.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmlxxviii{Und sie sagen: ``Sollen wir etwa, wenn wir (bereits) Knochen und Überreste geworden sind, denn wirklich als neue Schöpfung auferweckt werden?"\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmlxxix{Sag: ``Seid Steine oder Eisen\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmlxxx{oder etwas Erschaffenes von der Art, die in eurer Vorstellung noch schwerwiegender wäre." Dann werden sie sagen: ``Wer wird uns (ins Leben) zurückbringen?" Sag: ``Derjenige, Der euch das erste Mal erschaffen hat." Dann werden sie vor dir die Köpfe schütteln und sagen: ``Wann wird das sein?" Sag: ``Vielleicht wird es bald sein.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmlxxxi{Am Tag, da Er euch rufen wird und da ihr mit Seinem Lob antworten und meinen werdet, ihr hättet nur ein wenig verweilt."\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmlxxxii{Und sag Meinen Dienern, sie sollen das, was am besten ist, sagen. Gewiß, der Satan stachelt zwischen ihnen (zu Zwietracht) auf. Der Satan ist ja dem Menschen ein deutlicher Feind.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmlxxxiii{Euer Herr kennt euch sehr wohl; wenn Er will, erbarmt Er Sich eurer, oder wenn Er will, straft Er euch. Und Wir haben dich nicht als Sachwalter über sie gesandt.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmlxxxiv{Und dein Herr kennt diejenigen sehr wohl, die in den Himmeln und auf der Erde sind. Und Wir haben ja einige der Propheten vor anderen bevorzugt. Und Dawud haben Wir ein Buch der Weisheit gegeben.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmlxxxv{Sag: Ruft diejenigen an, die ihr außer Ihm angebt; sie vermögen doch von euch das Unheil weder hinwegzunehmen noch abzuwenden.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmlxxxvi{Diejenigen, die sie anrufen, trachten nach einem Mittel zu ihrem Herrn (und wetteifern), wer von ihnen (Ihm) am nächsten sei, und hoffen auf Seine Barmherzigkeit und fürchten Seine Strafe. Gewiß, die Strafe deines Herrn ist furchtbar.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmlxxxvii{Und es gibt keine Stadt, die Wir nicht vor dem Tag der Auferstehung vernichten oder mit strenger Strafe strafen werden. Dies steht im Buch (der Vorsehung) verzeichnet.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmlxxxviii{Und nichts (anderes) hat Uns davon abgehalten, die Zeichen (mit den Propheten) zu senden, als daß die Früheren sie für Lüge erklärten. Und Wir gaben den Tamud die deutlich sichtbare Kamelstute, sie aber taten an ihr Unrecht. Und Wir senden die Zeichen (mit den Propheten) nur, um Furcht einzuflößen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmlxxxix{Und als Wir zu dir sagten: ``Gewiß, dein Herr umfaßt die Menschen." Und Wir haben das Gesicht, das Wir dich sehen ließen, nur zu einer Versuchung für die Menschen gemacht, und (ebenso) den verfluchten Baum im Qur'an. Wir flößen ihnen Furcht ein, doch mehrt ihnen dies nur das Übermaß an Auflehnung.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmxc{Und als Wir zu den Engeln sagten: ``Werft euch vor Adam nieder." Da warfen sie sich nieder, außer Iblis. Er sagte: ``Soll ich mich vor jemandem niederwerfen, den Du aus Lehm erschaffen hast?"\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmxci{Er sagte: ``Was meinst Du wohl von diesem, den Du höher geehrt hast als mich? Wenn Du mich bis zum Tag der Auferstehung zurückstellst, werde ich seiner Nachkommenschaft bis auf wenige ganz gewiß die Zügel anlegen."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmxcii{Er sagte: ``Geh (deines Weges)! Wer von ihnen dir folgt, gewiß, so ist die Hölle euer Lohn, ein reichlicher Lohn.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmxciii{Und errege, wen von ihnen du (erregen) kannst, mit deiner Stimme, und biete gegen sie deine Reiterei und dein Fußvolk auf, und habe Anteil an ihrem Besitz und ihren Kindern, und mache ihnen Versprechungen." - Aber der Satan macht ihnen nur Versprechungen in Trug.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmxciv{``Gewiß, über Meine Diener hast du keine Macht." Und dein Herr genügt als Sachwalter.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmxcv{Euer Herr ist es, der für euch die Schiffe auf dem Meer sanft bewegt, damit ihr nach etwas von Seiner Huld trachtet. Gewiß, Er ist zu euch Barmherzig.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmxcvi{Und wenn euch auf dem Meer ein Unheil widerfährt, entschwinden (euch) diejenigen, die ihr außer Ihm anruft. Aber nachdem Er euch ans (Fest)land errettet hat, wendet ihr euch (von Ihm) ab; der Mensch ist eben sehr undankbar.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmxcvii{Glaubt ihr denn in Sicherheit davor zu sein, daß Er die Seiten des Festlandes mit euch versinken läßt oder einen Sturm von Steinchen gegen euch sendet und (daß) ihr dann für euch keinen Sachwalter findet?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmxcviii{Oder glaubt ihr in Sicherheit davor zu sein, daß Er euch ein anderes Mal aufs Meer zurückbringt und einen verheerenden Wind gegen euch sendet und euch ertrinken läßt, weil ihr ungläubig seid, und (daß) ihr dann für euch keinen Gefolgsmann findet, der Uns deswegen belangen könnte?\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmxcix{Und Wir haben ja die Kinder Adams geehrt; Wir haben sie auf dem Festland und auf dem Meer getragen und sie von den guten Dingen versorgt, und Wir haben sie vor vielen von denen, die Wir erschaffen haben, eindeutig bevorzugt.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmc{Gedenke) des Tages, da Wir alle Menschen mit ihrem (jeweiligen) Anführer rufen werden. Wem dann sein Buch in seine rechte Hand gegeben wird, jene wer den ihr Buch (ohne weiteres) lesen, und ihnen wird nicht um ein Fädchen Unrecht zugefügt.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmci{Und wer in diesem (Leben) blind ist, der wird (auch) im Jenseits blind und noch weiter vom Weg abgeirrt sein.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcii{Beinahe hätten sie dich fürwahr verfuhrt (und) von dem (abgebracht), was Wir dir (als Offenbarung) eingegeben haben, damit du gegen Uns etwas anderes ersinnst, und dann hätten sie dich wahrlich zum Freund genommen.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmciii{Wenn Wir dich nicht gefestigt hätten, hättest du wohl beinahe bei ihnen ein wenig Stütze gesucht.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmciv{Dann hätten Wir dich fürwahr ein Mehrfaches (an Strafe) im Leben und ein Mehrfaches (an Strafe) im Tod kosten lassen, und hierauf würdest du für dich keinen Helfer gegen Uns finden.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcv{Und beinahe hätten sie dich fürwahr aus dem Land aufgestört, um dich daraus zu vertreiben; aber dann würden sie nach dir nur ein wenig verweilen.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcvi{Das ist) die Gesetzmäßigkeit (, mit der) an denjenigen von Unseren Gesandten (verfahren wurde), die Wir bereits vor dir sandten. Und du wirst bei Unserer Gesetzmäßigkeit keine Abwandlung finden.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcvii{Verrichte das Gebet beim Neigen der Sonne bis zum Dunkel der Nacht, und (auch) die (Qur'an-)Lesung (in) der Morgendämmerung. Gewiß, die (Qur'an-)Lesung (in) der Morgendämmerung wird (von den Engeln) bezeugt.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmcviii{Und (einen Teil) der Nacht, verbringe ihn damit, zusätzlich für dich. Vielleicht wird dich dein Herr zu einer lobenswerten (Rang)stellung erwecken.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmcix{Und sag: Mein Herr, gewähre mir einen wahrhaftigen Eingang, und gewähre mir einen wahrhaftigen Ausgang, und schaffe mir von Dir aus eine hilfreiche Macht.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmcx{Und sag: Die Wahrheit ist gekommen, und das Falsche geht dahin; das Falsche ist ja dazu bestimmt, dahinzugehen.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmcxi{Und Wir offenbaren vom Qur'an, was für die Gläubigen Heilung und Barmherzigkeit ist; den Ungerechten aber mehrt es nur den Verlust.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmcxii{Wenn Wir dem Menschen Gunst erweisen, wendet er sich ab und entfernt sich zur Seite. Wenn ihm aber Schlechtes widerfährt, ist er sehr verzweifelt.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmcxiii{Sag: Jeder handelt nach seiner Weise. Euer Herr weiß sehr wohl, wessen Weg der Rechtleitung eher entspricht.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmcxiv{Sie fragen dich nach dem Geist. Sag: Der Geist ist vom Befehl meines Herrn, euch aber ist vom Wissen gewiß nur wenig gegeben.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmcxv{Und wenn Wir wollten, würden Wir ganz gewiß wegnehmen, was Wir dir (als Offenbarung) eingegeben haben. Hierauf würdest du für dich in dieser (Sache) keinen Sachwalter gegen Uns finden,\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmcxvi{außer Barmherzigkeit von deinem Herrn. Gewiß, Seine Huld zu dir ist ja groß.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmcxvii{Sag: Wenn sich die Menschen und die Ginn zusammentäten, um etwas beizubringen, was diesem Qur'an gleich wäre, sie brächten nicht seinesgleichen bei, auch wenn sie einander Beistand leisten würden.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmcxviii{Wir haben ja den Menschen in diesem Qur'an ein jedes Gleichnis verschiedenartig dargelegt, doch die meisten Menschen weigern sich (und wollen nichts) außer dem Unglauben.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmcxix{Und sie sagen: ``Wir werden dir nicht glauben, bis du uns aus der Erde eine Quelle hervorströmen läßt\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmcxx{oder (bis) du einen Garten mit Palmen und Rebstöcken hast, in dem du dazwischen Flüsse ausgiebig hervorströmen läßt\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmcxxi{oder (bis) du den Himmel, wie du behauptet hast, auf uns in Stücken herabfallen läßt oder Allah und die Engel vor unsere Augen bringst\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmcxxii{oder (bis) du ein Haus aus Gold hast oder in den Himmel aufsteigst. Und wir werden nicht an deinen Aufstieg glauben, bis du auf uns ein Buch herabsendest, das wir (selbst) lesen (können).." Sag: Preis sei meinem Herrn! Bin ich etwas anderes als ein menschliches Wesen und ein Gesandter?\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmcxxiii{Und nichts anderes hielt die Menschen davon ab zu glauben, als die Rechtleitung zu ihnen kam, außer daß sie sagten: ``Hat denn Allah ein menschliches Wesen als Gesandten geschickt?"\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmcxxiv{Sag: Wenn es auf der Erde Engel gäbe, die (da) in Ruhe umhergingen, hätten Wir ihnen vom Himmel wahrlich einen Engel als Gesandten hinabgesandt.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmcxxv{Sag: Allah genügt als Zeuge zwischen mir und euch. Gewiß, Er kennt und sieht Seine Diener wohl.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmcxxvi{Wen Allah rechtleitet, der ist (in Wahrheit) rechtgeleitet; wen Er aber in die Irre gehen läßt, für diejenigen wirst du außer Ihm keine Schutzherren finden. Und Wir werden sie am Tag der Auferstehung auf ihren Gesichtern versammeln, (sie wandeln) vor sich hin, blind, stumm, taub. Ihr Zufluchtsort ist die Hölle. Jedesmal, wenn sie schwächer wird, mehren Wir ihnen die Feuerglut.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmcxxvii{Das ist ihr Lohn dafür, daß sie Unsere Zeichen verleugnet und gesagt haben: ``Sollen wir denn, wenn wir (bereits) Knochen und Überreste geworden sind, fürwahr wieder als neue Schöpfung auferweckt werden?"\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmcxxviii{Sehen sie denn nicht, daß Allah, Der die Himmel und die Erde erschaffen hat, (auch) die Macht hat, ihresgleichen zu erschaffen? Und Er hat ihnen eine Frist gesetzt, an der es keinen Zweifel gibt, doch die Ungerechten weigern sich (und wollen nichts) außer dem Unglauben.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmcxxix{Sag: Wenn ihr über die Schatzkammern der Barmherzigkeit meines Herrn verfügtet, dann würdet ihr aus Furcht vor dem Ausgeben wahrlich zurückhaltend sein, denn der Mensch ist knauserig.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmcxxx{Und Wir gaben Musa ja neun klare Zeichen. So frage die Kinder Isra'ils (danach). Als er zu ihnen kam, da sagte Fir'aun zu ihm: ``Ich glaube fürwahr, o Musa, daß du einem Zauber verfallen bist."\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmcxxxi{Er sagte: ``Du weißt ja, niemand außer dem Herrn der Himmel und der Erde hat diese als einsichtbringende Zeichen herabgesandt. Und ich glaube fürwahr, o Fir'aun, daß du der Vernichtung verfallen bist."\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmcxxxii{Da wollte er sie aus dem Land aufstören. Wir aber ließen ihn ertrinken, und (auch) alle, die mit ihm waren.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmcxxxiii{Und Wir sagten nach seiner Vernichtung zu den Kindern Isra'ils: ``Bewohnt das Land. Wenn dann das Versprechen vom letzten Mal eintrifft, bringen Wir euch in buntgemischten Gruppen herbei.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmcxxxiv{Mit der Wahrheit haben Wir ihn (als Offenbarung) hinabgesandt, und mit der Wahrheit ist er hinabgekommen. Und Wir haben dich nur als Verkünder froher Botschaft und Warner gesandt.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmcxxxv{Einen Qur'an haben Wir (offenbart, den Wir in Abschnitte) unterteilt (haben), damit du ihn den Menschen in Abständen vorträgst; und Wir haben ihn wahrlich nach und nach offenbart.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmcxxxvi{Sag: Glaubt daran oder glaubt (eben) nicht. Diejenigen, denen vor ihm das Wissen gegeben wurde, fallen, wenn er ihnen verlesen wird, ehrerbietig auf das Kinn nieder\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmcxxxvii{und sagen: ``Preis sei unserem Herrn! Das Versprechen unseres Herrn ist wahrlich ausgeführt."\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmcxxxviii{Und sie fallen auf das Kinn weinend nieder, und es mehrt ihnen die Demut.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmcxxxix{Sag: Ruft Allah oder ruft den Allerbarmer an; welchen ihr auch ruft, Sein sind die schönsten Namen. Und sei nicht zu laut beim Gebet, und sie auch nicht zu leise da bei, sondern suche einen Weg dazwischen.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmcxl{Und sag: (Alles) Lob gehört Allah, Der Sich keine Kinder genommen hat, und es gibt weder einen Teilhaber an Seiner Herrschaft, noch benötigt Er einen Beschützer vor Demütigung. Und verherrliche Ihn doch als den Größten!\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmcxli{\basmalahde (Alles) Lob gehört Allah, Der das Buch (als Offenbarung) auf Seinen Diener herabgesandt und daran nichts Krummes gemacht hat,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcxlii{ein Buch,) das richtig ist, damit er vor harter Gewalt von Ihm aus warne und den Gläubigen, die rechtschaffene Werke tun, verkünde, daß es für sie schönen Lohn geben wird -,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcxliii{auf immer darin zu verbleiben -,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmcxliv{und (damit er) diejenigen warne, die sagen: ``Allah hat Sich Kinder genommen."\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcxlv{Sie haben kein Wissen davon, und auch nicht ihre Väter. Welch schwerwiegendes Wort kommt aus ihren Mündern heraus. Sie sagen nichts als Lüge.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcxlvi{Vielleicht magst du (aus Gram) noch dich selbst umbringen, wenn sie an diese Botschaft nicht glauben, nachdem sie sich abgewandt haben.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcxlvii{Gewiß, Wir haben (alles), was auf der Erde ist, zu einem Schmuck für sie gemacht, um sie zu prüfen (und festzustellen), wer von ihnen die besten Taten begeht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcxlviii{Und Wir werden das, was auf ihr ist, wahrlich zu unfruchtbarem Erdboden machen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcxlix{Oder meinst du etwa, daß die Leute der Höhle und der Inschrift ein (besonders) verwunderliches unter Unseren Zeichen sind?\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcl{Als die Jünglinge in der Höhle Zuflucht suchten und sagten: ``Unser Herr, gib uns Barmherzigkeit von Dir aus, und bereite uns in unserer Angelegenheit einen rechten (Aus)weg."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcli{Da ließen Wir sie in der Höhle für eine Anzahl von Jahren in Dauerschlaf fallen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmclii{Hierauf erweckten Wir sie auf, um zu wissen, welche der beiden Gruppierungen am richtigsten die Dauer ihres Verweilens erfaßt hat.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcliii{Wir berichten dir ihre Geschichte der Wahrheit entsprechend. Sie waren Jünglinge, die an ihren Herrn glaubten und denen Wir ihre Rechtleitung mehrten.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcliv{Und Wir stärkten ihre Herzen, als sie aufstanden und sagten: ``Unser Herr ist der Herr der Himmel und der Erde. Wir werden außer Ihm keinen (anderen) Gott anrufen, sonst würden wir ja etwas Unrechtes sagen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmclv{Dieses, unser Volk hat sich außer Ihm (andere) Götter genommen. Wenn sie doch für sie eine deutliche Ermächtigung bringen würden! Wer ist denn ungerechter, als wer gegen Allah eine Lüge ersinnt?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmclvi{Und da ihr euch nun von ihnen und von demjenigen, dem sie außer Allah dienen, fernhaltet, so sucht Zuflucht in der Höhle; euer Herr wird über euch (einiges) von Seiner Barmherzigkeit ausbreiten und euch in eurer Angelegenheit eine milde Behandlung bereiten."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmclvii{Und du siehst die Sonne, wenn sie aufgeht, sich von ihrer Höhle zur Rechten wegneigen, und wenn sie untergeht, an ihnen zur Linken vorbeigehen, während sie sich darin in einem Raum befinden. Das gehört zu Allahs Zeichen. Wen Allah rechtleitet, der ist (in Wahrheit) rechtgeleitet; wen Er aber in die Irre gehen läßt, für den wirst du keinen Schutzherrn finden, der ihn den rechten Weg führt.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmclviii{Du meinst, sie seien wach, obwohl sie schlafen. Und Wir drehen sie nach rechts und nach links um, während ihr Hund seine Vorderbeine im Vorraum ausstreckt. Wenn du sie erblicktest, würdest du dich vor ihnen fürwahr zur Flucht kehren und vor ihnen fürwahr mit Schrecken erfüllt sein.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmclix{Und so erweckten Wir sie auf, damit sie sich gegenseitig fragten. Einer von ihnen sagte: ``Wie lange habt ihr verweilt?" Sie sagten: ``Verweilt haben wir einen Tag oder den Teil eines Tages." Sie sagten: ``Euer Herr weiß am besten, wie (lange) ihr verweilt habt. So schickt einen von euch mit diesen euren Silbermünzen in die Stadt; er soll sehen, welche ihre reinste Speise ist, und euch davon eine Versorgung bringen. Er soll behutsam sein und ja niemanden etwas von euch merken lassen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmclx{Denn wenn sie von euch erfahren', werden sie euch steinigen oder euch (zwangsweise) zu ihrer Glaubensrichtung zurückbringen; dann wird es euch niemals mehr wohl ergehen."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmclxi{So ließen Wir (die Menschen) sie doch entdecken, damit sie wissen, daß Allahs Versprechen wahr ist und daß es an der Stunde keinen Zweifel gibt. Als sie untereinander über ihre Angelegenheit stritten, da sagten sie: ``Errichtet über ihnen einen Bau. Ihr Herr weiß am besten über sie Bescheid." Diejenigen, die in ihrer Angelegenheit siegten, sagten: ``Wir werden uns über ihnen ganz gewiß eine Gebetsstätte einrichten."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmclxii{Manche) werden sagen: ``(Es waren ihrer) drei, ihr Hund war der vierte von ihnen." Und (manche) sagen: ``(Es waren ihrer) fünf, der sechste von ihnen war ihr Hund." - Ein Herumraten über das Verborgene. Und (manche) sagen: ``(Es waren ihrer) sieben, und der achte von ihnen war ihr Hund." Sag: Mein Herr kennt ihre Zahl am besten; nur wenige kennen sie. Darum streite über sie nur in offensichtlichem Streit, und frage niemanden von ihnen um Auskunft über sie.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmclxiii{Und sag ja nur nicht von einer Sache: ``Ich werde dies morgen tun",\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmclxiv{außer (du fügst hinzu): ``Wenn Allah will." Und gedenke deines Herrn, wenn du (etwas) vergessen hast, und sag: ``Vielleicht leitet mich mein Herr zu etwas, was dem rechten Ausweg näher kommt als dies."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmclxv{Und sie verweilten in ihrer Höhle dreihundert Jahre und noch neun dazu.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmclxvi{Sag: Allah weiß am besten, wie (lange) sie verweilten. Sein ist das Verborgene der Himmel und der Erde. Wie vorzüglich ist Er als Allsehender, und wie vorzüglich ist Er als Allhörender! Sie haben außer Ihm keinen Schutzherrn, und Er beteiligt an Seiner Urteilsgewalt niemanden.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmclxvii{Und verlies, was dir vom Buch deines Herrn (als Offenbarung) eingegeben worden ist. Es gibt niemanden, der Seine Worte abändern könnte, und du wirst außer (bei) Ihm keine Zuflucht finden.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmclxviii{Halte dich geduldig zurück zusammen mit denen, die ihren Herrn morgens und abends anrufen, im Begehren nach Seinem Angesicht. Und deine Augen sollen nicht über sie hinwegsehen, indem du den Schmuck des diesseitigen Lebens begehrst'. Und gehorche nicht jemandem, dessen Herz Wir Unserem Gedenken gegenüber unachtsam gemacht haben, der seiner Neigung folgt und dessen Angelegenheit (durch) Maßlosigkeit (ausgezeichnet) ist.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmclxix{Und sag: (Es ist) die Wahrheit von eurem Herrn. Wer nun will, der soll glauben, und wer will, der soll ungläubig sein. Gewiß, Wir haben den Ungerechten ein Feuer bereitet, dessen Zeltdecke sie umfangen hält. Und wenn sie um Hilfe rufen, wird ihnen mit Wasser wie geschmolzenem Erz geholfen, das die Gesichter versengt - ein schlimmes Getränk und ein böser Rastplatz!\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmclxx{Diejenigen, die glauben und rechtschaffene Werke tun - gewiß, Wir lassen den Lohn derer nicht verlorengehen, die die besten Taten begehen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmclxxi{Für jene wird es die Gärten Edens geben, wo unterhalb von) ihnen Flüsse strömen. Geschmückt sind sie darin mit Armreifen aus Gold, und sie tragen grüne Gewänder aus Seidenbrokat und schwerem Brokat, indem sie sich darin auf überdachten Liegen lehnen. Wie trefflich ist die Belohnung und wie schön der Rastplatz!\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmclxxii{Und präge ihnen das Gleichnis von zwei Männern. Dem einen der beiden gaben Wir zwei Gärten mit Rebstöcken, und Wir umgaben sie mit Palmen und legten dazwischen (sonstige) Pflanzungen an.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmclxxiii{Beide Gärten brachten ihren Ernteertrag hervor und ließen es in nichts davon fehlen. Und Wir ließen dazwischen einen Fluß hervorströmen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmclxxiv{Er hatte (daraus reichlich) Früchte. Da sagte er zu seinem Gefährten, während er mit ihm einen Wortwechsel führte: ``Ich habe mehr Besitz als du und auch eine mächtigere Schar."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmclxxv{Und er betrat seinen Garten, während er sich selbst Unrecht tat. Er sagte: ``Ich glaube nicht, daß dieser (Garten) jemals zugrunde gehen wird,\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmclxxvi{und ich glaube nicht, daß die Stunde (des Gerichts) sich einstellen wird. Und wenn ich zu meinem Herrn zurückgebracht werde, werde ich ganz gewiß etwas Besseres als ihn als Rückzugsort finden."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmclxxvii{Sein Gefährte sagte zu ihm, während er sich mit ihm unterhielt: ``Verleugnest du denn Denjenigen, Der dich aus Erde, hierauf aus einem Samentropfen erschaffen und hierauf dich zu einem Mann geformt hat?\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmclxxviii{Aber, was mich betrifft: Er, Allah, ist mein Herr, und ich geselle meinem Herrn niemanden bei.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmclxxix{Würdest du doch, wenn du deinen Garten betrittst, sagen: ,(Es sei,) was Allah will; es gibt keine Kraft außer durch Allah'! Wenn du auch siehst, daß ich weniger Besitz und Kinder habe als du,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmclxxx{so wird mein Herr mir vielleicht etwas Besseres als deinen Garten geben und über ihn aufeinanderfolgende Strafe vom Himmel senden, so daß er zu schlüpfrigem Erdboden wird,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmclxxxi{oder daß sein Wasser versickert sein wird, so daß du es nicht mehr wirst ausfindig machen können."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmclxxxii{Seine Früchte wurden ringsum erfaßt. Da begann er, seine Handflächen umzudrehen wegen dessen, was er für ihn ausgegeben hatte, während er wüst in Trümmern lag, und zu sagen: ``O hätte ich doch meinem Herrn niemanden beigesellt!"\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmclxxxiii{Und er hatte weder eine Schar, die ihm anstatt Allahs half, noch konnte er sich selbst helfen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmclxxxiv{In dem Fall gehört (all) die Schutzherrschaft Allah, dem Wahren. Er ist der Beste im Belohnen, und Er verschafft den besten Ausgang.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmclxxxv{Und präge ihnen das Gleichnis vom diesseitigen Leben. (Es ist) wie Wasser, das Wir vom Himmel hinabkommen lassen, worauf sich damit das Gewächs der Erde vermischt. Dann wird es zu vertrocknetem Zeug, das die Winde verwehen. Und Allah hat ja zu allem völlig die Macht.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmclxxxvi{Der Besitz und die Söhne sind der Schmuck des diesseitigen Lebens. Das Bleibende aber, die rechtschaffenen Werke -, sie sind bei deinem Herrn besser hinsichtlich der Belohnung und besser hinsichtlich der Hoffnung.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmclxxxvii{Und (gedenke) des Tages, da Wir die Berge versetzen und du die Erde (kahl) hervortreten siehst und Wir sie versammeln, ohne jemanden von ihnen auszulassen,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmclxxxviii{und (da) sie deinem Herrn in Reihen vorgeführt werden: ``Nun seid ihr zu Uns gekommen, so wie Wir euch das erste Mal erschaffen haben. Ihr aber habt behauptet, Wir würden für euch keine (letzte) Verabredung festlegen."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmclxxxix{Und das Buch wird hingelegt. Dann siehst du die Übeltäter besorgt wegen dessen, was darin steht. Sie sagen: ``O wehe uns! Was ist mit diesem Buch? Es läßt nichts aus, weder klein noch groß, ohne es zu erfassen." Sie finden (alles), was sie taten, gegenwärtig, und dein Herr tut niemandem Unrecht.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcxc{Und als Wir zu den Engeln sagten: ``Werft euch vor Adam nieder." Da warfen sie sich nieder, außer Iblis; er gehörte zu den Ginn. So frevelte er gegen den Befehl seines Herrn. Wollt ihr euch denn ihn und seine Nachkommenschaft zu Schutzherren anstatt Meiner nehmen, wo sie euch doch feind sind? - Ein schlimmer Tausch für die Ungerechten!\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcxci{Ich habe sie weder bei der Erschaffung der Himmel und der Erde noch bei ihrer eigenen Erschaffung zu Zeugen genommen. Ich nehme Mir niemals die Irre führenden als Beistand.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcxcii{Und an dem Tag, da Er sagen wird: ``Ruft Meine Teilhaber, die ihr angegeben habt!", werden sie sie anrufen, aber sie werden ihnen nicht antworten. Und Wir werden zwischen ihnen einen Ort der Vernichtung einrichten'.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcxciii{Und die Übeltäter werden das (Höllen)feuer sehen und überzeugt sein, daß sie hineinfallen, und kein Mittel finden, es abzuwenden.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcxciv{Und Wir haben ja in diesem Qur'an den Menschen ein jedes Gleichnis verschiedenartig dargelegt; aber der Mensch ist von allen Wesen am streitsüchtigsten.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcxcv{Und nichts anderes hielt die Menschen davon ab, zu glauben, als die Rechtleitung zu ihnen kam, und ihren Herrn um Vergebung zu bitten, außer (ihrer Forderung), daß an ihnen nach der Gesetzmäßigkeit der Früheren verfahren werde oder daß die Strafe vor ihren Augen über sie komme.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcxcvi{Wir senden die Gesandten nur als Verkünder froher Botschaft und als Überbringer von Warnungen. Aber diejenigen, die ungläubig sind, streiten mit dem Falschen, um damit die Wahrheit zu widerlegen. Und sie machen sich über Meine Zeichen und das, womit sie gewarnt wurden, lustig.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcxcvii{Wer ist denn ungerechter als jemand, der mit den Zeichen seines Herrn ermahnt wird und sich dann von ihnen abwendet und vergißt, was seine Hände vorausgeschickt haben. Gewiß, Wir haben auf ihre Herzen Hüllen gelegt, so daß sie ihn nicht verstehen, und in ihre Ohren Schwerhörigkeit; und wenn du sie zur Rechtleitung rufst, dann werden sie sich also niemals rechtleiten lassen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcxcviii{Und dein Herr ist der Allvergebende, voll der Barmherzigkeit. Wenn Er sie für das belangen würde, was sie erworben haben, würde Er für sie die Strafe wahrlich beschleunigen. Aber sie haben eine Verabredung, vor der sie keine Zuflucht finden werden.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcxcix{Und diese Städte da vernichteten Wir, als sie Unrecht taten. Und Wir haben für ihre Vernichtung eine Verabredung festgelegt.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcc{Und als Musa zu seinem Burschen sagte: ``Ich lasse nicht ab, bis ich die Stelle erreicht habe, an der die Meere zusammenkommen, und sollte ich lange Zeit weiter gehen."\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcci{Als sie die Stelle erreicht hatten, an der sie zusammenkommen, vergaßen sie ihren Fisch, so nahm er seinen Weg im Meer, (auf und davon) schwimmend.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmccii{Als sie vorbeigegangen waren, sagte er zu seinem Burschen: ``Bringe uns unser Mittagessen. Wir haben ja durch diese unsere Reise (viel) Mühsal erlitten."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcciii{Er sagte: ``Siehst du! Als wir beim Felsen Rast gemacht haben, gewiß, da habe ich den Fisch vergessen. Und es ließ mich ihn nur der Satan vergessen, ihn (dir) zu erwähnen. Und er nahm seinen Weg im Meer auf wunderliche Weise."\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcciv{Er sagte: ``Das ist es, was wir suchten." Da kehrten sie beide zurück, indem sie ihren eigenen Spuren folgten.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmccv{Sie trafen einen von Unseren Dienern, dem Wir Barmherzigkeit von Uns aus hatten zukommen lassen und den Wir Wissen von Uns her gelehrt hatten.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmccvi{Musa sagte zu ihm: ``Darf ich dir folgen, auf daß du mich von dem lehrst, was dir an Besonnenheit gelehrt worden ist?"\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmccvii{Er sagte: ``Du wirst (es) bei mir nicht aushalten können.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmccviii{Wie willst du das auch aushalten, wovon du keine umfassende Kenntnis hast?"\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmccix{Er sagte: ``Du wirst mich, wenn Allah will, standhaft finden, und ich werde mich keinem Befehl von dir widersetzen."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmccx{Er sagte: ``Wenn du mir denn folg(en will)st, dann frage mich nach nichts, bis ich selbst es dir gegenüber zuerst erwähne."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmccxi{Da zogen sie beide los, bis, als sie ein Schiff bestiegen, er darin ein Loch machte. Er (Musa) sagte: ``Hast du ein Loch darin gemacht, um seine Besatzung ertrinken zu lassen? Du hast da ja eine grauenhafte Sache begangen."\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmccxii{Er sagte: ``Habe ich nicht gesagt, daß du (es) bei mir nicht wirst aushalten können?"\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmccxiii{Er (Musa) sagte: ``Belange mich nicht dafür, daß ich vergessen habe, und bedrücke mich in meiner Angelegenheit nicht mit einer Erschwernis."\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmccxiv{Da zogen sie beide weiter, bis, als sie dann einen Jungen trafen, er ihn tötete. Er (Musa) sagte: ``Hast du eine unschuldige Seele getötet (und zwar) nicht (als Wiedervergeltung) für eine (andere) Seele? Du hast da ja eine verwerfliche Sache begangen."\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmccxv{Er sagte: ``Habe ich dir nicht gesagt, daß du (es) bei mir nicht wirst aushalten können?"\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmccxvi{Er (Musa) sagte: ``Wenn ich dich danach (noch einmal) nach irgend etwas frage, dann lasse mich dich nicht mehr begleiten. Dich trifft in Bezug auf mich kein Tadel.1"\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmccxvii{Da zogen sie beide weiter, bis, als sie dann zu den Bewohnern einer Stadt kamen, sie ihre Bewohner um etwas zu essen baten; diese aber weigerten sich, sie gastlich aufzunehmen. Da fanden sie in ihr eine Mauer, die einzustürzen drohte, und so richtete er sie auf. Er (Musa) sagte: ``Wenn du wolltest, hättest du dafür wahrlich Lohn nehmen können."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmccxviii{Er sagte: ``Das ist die Trennung zwischen mir und dir. Ich werde dir jetzt die Deutung dessen kundtun, was du nicht aushalten konntest.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmccxix{Was das Schiff angeht, so gehörte es Armen, die auf dem Meer arbeiteten. Ich wollte es schadhaft machen, denn ein König war hinter ihnen her, der jedes Schiff mit Gewalt wegnahm.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmccxx{Was den Jungen angeht, so waren seine Eltern gläubige (Menschen). Da fürchteten wir, daß er sie durch (seine) Auflehnung und durch (seinen) Unglauben bedrücken würde.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmccxxi{So wollten wir, daß ihr Herr ihnen zum Tausch einen gebe, - besser als er an Lauterkeit und näher kommend an Güte.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmccxxii{Was aber die Mauer angeht, so gehörte sie zwei Waisenjungen in der Stadt, und unter ihr befand sich ein für sie bestimmter Schatz. Ihr Vater war rechtschaffen, und da wollte dein Herr, daß sie (erst) ihre Vollreife erlangen und (dann) ihren Schatz hervorholen - aus Barmherzigkeit von deinem Herrn. Ich tat es ja nicht aus eigenem Ermessen. Das ist die Deutung dessen, was du nicht aushalten konntest."\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmccxxiii{Und sie fragen dich nach Du 'l-Qarnain. Sag: Ich werde euch über ihn eine Geschichte verlesen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmccxxiv{Wir verliehen ihm auf der Erde eine feste Stellung und eröffneten ihm zu allem einen Weg.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmccxxv{Da verfolgte er einen Weg,\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmccxxvi{bis, als er den Ort des Sonnenuntergangs erreichte, er fand, daß sie in einer schlammigen Quelle unterging, und er fand bei ihr ein Volk. Wir sagten: ``O Du '1Qarnain, entweder strafst du (sie), oder du behandelst sie mit Güte."\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmccxxvii{Er sagte: ``Was nun jemanden angeht, der Unrecht tut, so werden wir ihn strafen; hierauf wird er zu seinem Herrn zurückgebracht, und Er straft ihn dann mit entsetzlicher Strafe.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmccxxviii{Was aber jemanden angeht, der glaubt und rechtschaffen handelt, für den wird es als Lohn das Beste geben, und Wir werden ihm von unserem Befehl etwas sagen, was Erleichterung bringt."\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmccxxix{Hierauf verfolgte er einen Weg,\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmccxxx{bis, als er den Ort des Sonnenaufgangs erreichte, er fand, daß sie über einem Volk aufgeht, denen Wir keine Deckung vor ihr gegeben hatten.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmccxxxi{So war es. Und Wir haben ja umfassende Kenntnis von dem, was ihn betrifft.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmccxxxii{Hierauf verfolgte er einen Weg,\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmccxxxiii{bis, als er den Ort zwischen den beiden Bergen erreichte, er diesseits von ihnen ein Volk fand, das beinahe kein Wort verstand.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmccxxxiv{Sie sagten: ``O Du '1-Qarnain, Ya'gug und Ma'gug stiften Unheil auf der Erde. Sollen wir dir eine Gebühr dafür aussetzen, daß du zwischen uns und ihnen eine Sperrmauer errichtest?"\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmccxxxv{Er sagte: ``Was mir mein Herr an fester Stellung verliehen hat, ist besser (als eure Gebühr). Doch helft mir mit (eurer Arbeitskraft, damit ich zwischen euch und ihnen einen aufgeschütteten Wall errichte.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmccxxxvi{Bringt mir die Eisenstücke." Als er nun zwischen den beiden Berghängen gleich hoch (aufgeschüttet) hatte, sagte er: ``Blast (jetzt)." Als er es zum Glühen gebracht hatte, sagte er: ``Bringt mir, damit ich (es) darüber gieße, geschmolzenes Kupfer."\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmccxxxvii{So konnten sie ihn weder überwinden, noch konnten sie ihn durchbrechen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmccxxxviii{Er sagte: ``Das ist eine Barmherzigkeit von meinem Herrn. Wenn dann das Versprechen meines Herrn eintrifft, läßt Er ihn in sich zusammensinken; und das Versprechen meines Herrn ist wahr."\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmccxxxix{Wir lassen die einen von ihnen an jenem Tag wie Wogen unter die anderen geraten, und es wird ins Horn geblasen. Dann versammeln Wir sie vollständig.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmccxl{Und Wir führen die Hölle den Ungläubigen an jenem Tag in aller Deutlichkeit vor,\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmccxli{ihnen,) deren Augen vor Meiner Mahnung (wie) unter einer Decke lagen und die (auch) nicht hören konnten.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmccxlii{Meinen denn diejenigen, die ungläubig sind, daß sie sich Meine Diener anstatt Meiner zu Schutzherren nehmen (können)? Gewiß, Wir haben die Hölle den Ungläubigen zur gastlichen Aufnahme bereitet.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmccxliii{Sag: Sollen Wir euch Kunde geben von denjenigen, die an ihren Werken am meisten verlieren,\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmccxliv{deren Bemühen im diesseitigen Leben fehlgeht, während sie meinen, daß sie gut handeln würden?\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmccxlv{Das sind diejenigen, die die Zeichen ihres Herrn und die Begegnung mit Ihm leugnen. So werden ihre Werke hinfällig, und so werden Wir ihnen am Tag der Auferstehung kein Gewicht beimessen.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmccxlvi{Das ist ihr Lohn - die Hölle - dafür, daß sie ungläubig waren und sich über Meine Zeichen und Meine Gesandten lustig machten.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmccxlvii{Gewiß, für diejenigen, die glauben und rechtschaffene Werke tun, wird es die Gärten des Paradieses zur gastlichen Aufnahme geben,\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmccxlviii{ewig darin zu bleiben, und sie werden nicht begehren, davon wegzugehen'.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmccxlix{Sag: Wenn das Meer Tinte für die Worte meines Herrn wäre, würde das Meer wahrlich zu Ende gehen, bevor die Worte meines Herrn zu Ende gingen, auch wenn Wir als Nachschub noch einmal seinesgleichen hinzubrächten.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmccl{Sag: Gewiß, ich bin ja nur ein menschliches Wesen gleich euch; mir wird (als Offenbarung) eingegeben, daß euer Gott ein Einziger Gott ist. Wer nun auf die Begegnung mit seinem Herrn hofft, der soll rechtschaffen handeln und beim Dienst an seinem Herrn (Ihm) niemanden beigesellen.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmccli{\basmalahde Kaf-Ha-Ya-'Ayn-Sad.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcclii{Diese Verse sind) zum Gedenken an die Barmherzigkeit deines Herrn zu Seinem Diener Zakariyya.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmccliii{Als er zu seinem Herrn im Verborgenen rief\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmccliv{und) sagte: ``Mein Herr, schwach sind mir die Knochen geworden, und in Altersgrauheit entfacht ist der Kopf. Und ich war im Bittgebet zu Dir, mein Herr, (noch) nie unglücklich.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcclv{Gewiß, ich fürchte die Verwandten nach mir, und meine Frau ist unfruchtbar. So schenke mir von Dir aus einen nahen Verwandten,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcclvi{der mich beerbt und von der Sippe Ya'qubs erbt, und mach ihn, mein Herr, (Dir) wohlgefällig."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcclvii{``O Zakariyya, Wir verkünden dir einen Jungen, dessen Name Yahya ist, wie Wir zuvor noch niemandem Kennzeichen gleich den seinen gegeben haben."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcclviii{Er sagte: ``Mein Herr, wie soll ich einen Jungen haben, wo meine Frau unfruchtbar ist und ich vom hohen Alter bereits einen Dürrezustand erreicht habe?"\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcclix{Er sagte: ``So wird es sein. Dein Herr sagt: ,Das ist Mir ein leichtes; und auch dich habe Ich ja zuvor erschaffen, als du noch nichts warst'."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcclx{Er sagte: ``Mein Herr, setze mir ein Zeichen." Er sagte: ``Dein Zeichen ist, daß du drei Nächte lang, obwohl gesund, nicht zu den Menschen sprechen wirst."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcclxi{So kam er zu seinem Volk aus dem Gebetsraum heraus und gab ihnen dann zu verstehen: ``Preist morgens und abends."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmcclxii{``O Yahya, nimm die Schrift mit (aller) Kraft." Und Wir gaben ihm schon als Kind die Urteilskraft\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcclxiii{und Mitgefühl von Uns aus und Lauterkeit. Er war gottesfürchtig\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcclxiv{und gütig gegen seine Eltern, und er war weder gewalttätig noch widerspenstig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmcclxv{Und Friede sei auf ihm am Tag, da er geboren wurde, und am Tag, da er stirbt, und am Tag, da er wieder zum Leben auferweckt wird.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmcclxvi{Und gedenke im Buch Maryams, als sie sich von ihren Angehörigen an einen östlichen Ort zurückzog.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmcclxvii{Sie nahm sich einen Vorhang vor ihnen. Da sandten Wir Unseren Geist zu ihr. Er stellte sich ihr als wohlgestaltetes menschliches Wesen dar.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmcclxviii{Sie sagte: ``Ich suche beim Allerbarmer Schutz vor dir, wenn du gottesfürchtig bist."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmcclxix{Er sagte: ``Ich bin nur der Gesandte deines Herrn, um dir einen lauteren Jungen zu schenken."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmcclxx{Sie sagte: ``Wie soll mir ein Junge gegeben werden, wo mich doch kein menschliches Wesen berührt hat und ich keine Hure bin."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmcclxxi{Er sagte: ``So wird es sein. Dein Herr sagt: ,Das ist Mir ein leichtes, und damit Wir ihn zu einem Zeichen für die Menschen und zu einer Barmherzigkeit von Uns machen'. Und es ist eine beschlossene Angelegenheit."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmcclxxii{So empfing sie ihn und zog sich mit ihm zu einem fernen Ort zurück.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmcclxxiii{Die Wehen ließen sie zum Palmenstamm gehen. Sie sagte: ``O wäre ich doch zuvor gestorben und ganz und gar in Vergessenheit geraten!"\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmcclxxiv{Da rief er ihr von unten her zu: ``Sei nicht traurig; dein Herr hat ja unter dir ein Bächlein geschaffen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmcclxxv{Und schüttle zu dir den Palmenstamm, so läßt er frische, reife Datteln auf dich herabfallen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmcclxxvi{So iß und trink und sei frohen Mutes'. Und wenn du nun jemanden von den Menschen sehen solltest, dann sag: Ich habe dem Allerbarmer Fasten gelobt, so werde ich heute mit keinem Menschenwesen sprechen."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmcclxxvii{Dann kam sie mit ihm zu ihrem Volk, ihn (mit sich) tragend. Sie sagten: ``O Maryam, du hast da ja etwas Unerhörtes begangen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmcclxxviii{O Schwester Haruns, dein Vater war doch kein sündiger Mann, noch war deine Mutter eine Hure."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmcclxxix{Da zeigte sie auf ihn. Sie sagten: ``Wie können wir mit jemandem sprechen, der noch ein Kind in der Wiege ist?"\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmcclxxx{Er sagte: ``Ich bin wahrlich Allahs Diener; Er hat mir die Schrift gegeben und mich zu einem Propheten gemacht.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmcclxxxi{Und gesegnet hat Er mich gemacht, wo immer ich bin, und angeordnet hat Er mir, das Gebet (zu verrichten) und die Abgabe (zu entrichten), solange ich lebe,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmcclxxxii{und gütig gegen meine Mutter zu sein. Und Er hat mich weder gewalttätig noch unglücklich gemacht.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmcclxxxiii{Und der Friede sei auf mir am Tag, da ich geboren wurde, und am Tag, da ich sterbe, und am Tag da ich wieder zum Leben auferweckt werde."\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmcclxxxiv{Das ist 'Isa, der Sohn Maryams: (Es ist) das Wort der Wahrheit, woran sie zweifeln.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmcclxxxv{Es steht Allah nicht an, Sich ein Kind zu nehmen. Preis sei Ihm! Wenn Er eine Angelegenheit bestimmt, so sagt Er dazu nur: ,Sei!', und so ist es.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmcclxxxvi{'Isa sagte:) ``Und gewiß, Allah ist mein Herr und euer Herr; so dient Ihm. Das ist ein gerader Weg."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmcclxxxvii{Doch die Gruppierungen wurden uneinig untereinander; dann wehe denjenigen, die ungläubig sind, vor dem Erleben eines gewaltigen Tages!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmcclxxxviii{Wie vorzüglich werden sie hören und sehen am Tag, da sie zu Uns kommen werden! Aber die Ungerechten befinden sich heute in einem deutlichen Irrtum.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmcclxxxix{Warne sie vor dem Tag der gramvollen Reue, wenn die Angelegenheit entschieden sein wird, während sie (all dessen) unachtsam sind, und während sie (noch) nicht glauben.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmccxc{Gewiß, Wir sind es, die Wir die Erde und (all die,) die auf ihr sind, erben werden. Und zu Uns werden sie zurückgebracht.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmccxci{Und gedenke im Buch Ibrahims. Er war ein Wahrhaftiger und Prophet.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmccxcii{Als er zu seinem Vater sagte: ``O mein lieber Vater, warum dienst du dem, was nicht hört und nicht sieht und dir nichts nützt?\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmccxciii{O mein lieber Vater, gewiß, zu mir ist vom Wissen gekommen, was nicht zu dir gekommen ist. So folge mir, dann leite ich dich einen ebenen Weg.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmccxciv{O mein lieber Vater, diene nicht dem Satan. Gewiß, der Satan ist gegen den Allerbarmer widerspenstig.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmccxcv{O mein lieber Vater, ich fürchte, daß dir Strafe vom Allerbarmer widerfährt, so daß du zu einem Gefolgsmann des Satans wirst."\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmccxcvi{Er sagte: ``Verschmähst du meine Götter, o Ibrahim? Wenn du nicht aufhörst, werde ich dich ganz gewiß steinigen. Und meide mich eine Zeitlang."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmccxcvii{Er sagte: ``Friede sei auf dir! Ich werde meinen Herrn für dich um Vergebung bitten; gewiß, Er ist zu mir sehr entgegenkommend.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmccxcviii{Ich werde mich von euch und von dem, was ihr anstatt Allahs anruft, fernhalten und nur meinen Herrn anrufen. Vielleicht werde ich im Bittgebet zu meinem Herrn nicht unglücklich sein."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmccxcix{Als er sich nun von ihnen und von dem, dem sie anstatt Allahs dienten, fernhielt, schenkten Wir ihm Ishaq und Ya'qub; und alle (beide) machten Wir zu Propheten.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmccc{Und Wir schenkten ihnen (etwas) von Unserer Barmherzigkeit, und Wir verliehen ihnen einen hohen Ruf an Wahrhaftigkeit.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmccci{Und gedenke im Buch Musas. Gewiß, er war auserlesen, und er war ein Gesandter und Prophet.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcccii{Wir riefen ihn von der rechten Seite des Berges und ließen ihn zu vertraulichem Gespräch näherkommen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmccciii{Und Wir schenkten ihm aus Unserer Barmherzigkeit seinen Bruder Harun als Propheten.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmccciv{Und gedenke im Buch Isma'ils. Gewiß, er war wahrhaftig in seinem Versprechen, und er war ein Gesandter und Prophet.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcccv{Und er pflegte seinen Angehörigen das Gebet und die Abgabe zu befehlen, und er war seinem Herrn wohlgefällig.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcccvi{Und gedenke im Buch Idris'. Er war ein Wahrhaftiger und Prophet.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcccvii{Und Wir hoben ihn zu einem hohen Ort empor.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcccviii{Das sind diejenigen, denen Allah Gunst erwiesen hat, unter den Propheten aus der Nachkommenschaft Adams und von denjenigen, die Wir mit Nuh trugen, und aus der Nachkommenschaft Ibrahims und Isra'ils, und von denjenigen, die Wir rechtgeleitet und erwählt haben. Als ihnen die Zeichen des Allerbarmers verlesen wurden, fielen sie ehrerbietig und weinend nieder.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcccix{Dann folgten nach ihnen Nachfolger, die das Gebet vernachlässigten und den Begierden folgten. So werden sie (den Lohn für ihre) Verirrung vorfinden,\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcccx{außer demjenigen, der bereut und glaubt und rechtschaffen handelt. Jene werden in den (Paradies)garten eingehen und ihnen wird in nichts Unrecht zugefügt\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcccxi{in die Gärten Edens, die der Allerbarmer Seinen Dienern im Verborgenen versprochen hat. Sein Versprechen wird bestimmt erfüllt.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcccxii{Sie hören darin keine unbedachte Rede, sondern nur: ``Frieden!" Und sie haben darin ihre Versorgung morgens und abends.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcccxiii{Das ist der (Paradies)garten, den Wir denjenigen von Unseren Dienern zum Erbe geben, die gottesfürchtig sind.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcccxiv{Und die Engel sagen:) ``Wir kommen nur auf Befehl deines Herrn herab. Ihm gehört (alles), was vor uns und was hinter uns und was dazwischen ist; und dein Herr ist gewiß nicht vergeßlich,'\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcccxv{Er), der Herr der Himmel und der Erde und dessen, was dazwischen ist. So diene Ihm und sei beharrlich in Seinem Dienst. Weißt du (etwa) einen, der Kennzeichen gleich den Seinen besäße?1"\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcccxvi{Der Mensch sagt: ``Werde ich, wenn ich erst einmal gestorben bin, wirklich wieder lebendig hervorgebracht werden?"\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcccxvii{Gedenkt der Mensch denn nicht, daß Wir ihn zuvor erschaffen haben, da er (noch) nichts war?\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmcccxviii{Bei deinem Herrn, Wir werden sie ganz gewiß versammeln, (sie) und die Satane. Hierauf werden Wir sie ganz gewiß rings um die Hölle herum auf den Knien herbeibringen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmcccxix{Hierauf werden Wir aus jedem Lager ganz gewiß denjenigen herausnehmen, der sich dem Allerbarmer am heftigsten widersetzt hat.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmcccxx{Dann wissen Wir wahrlich am besten über diejenigen Bescheid, die es am ehesten verdienen, ihr ausgesetzt zu sein.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmcccxxi{Und es gibt keinen unter euch, der nicht daran vorbeigehen würde. Dies obliegt deinem Herrn unabänderlich beschlossen.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmcccxxii{Hierauf erretten Wir diejenigen, die gottesfürchtig waren, und lassen die Ungerechten in ihr auf den Knien zurück.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcccxxiii{Und wenn ihnen Unsere Zeichen als klare Beweise verlesen werden, sagen diejenigen, die ungläubig sind, zu denjenigen, die glauben: ``Welcher der beiden Gruppen hat nun eine bessere Stellung und eine schönere Gesellschaft?"\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmcccxxiv{Aber wie viele Geschlechter vernichteten Wir vor ihnen, die besser an Ausstattung und Aussehen waren!\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmcccxxv{Sag: Wer sich im Irrtum befindet, den möge der Allerbarmer lange darin gewähren lassen. Wenn sie dann sehen, was ihnen angedroht ist, entweder die Strafe oder die Stunde, da werden sie wissen, wer sich in einer (noch) schlechteren Lage befindet und die schwächere Heerschar hat.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcccxxvi{Und Allah mehrt denjenigen, die rechtgeleitet sind, ihre Rechtleitung. Das Bleibende aber, die rechtschaffenen Werke - sie sind bei deinem Herrn besser in der Belohnung und besser als Rückkehr.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcccxxvii{Was meinst du wohl zu demjenigen, der Unsere Zeichen verleugnet und sagt: ``Mir werden ganz gewiß Besitz und Kinder gegeben"?\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcccxxviii{Hat er etwa Einblick in das Verborgene gewonnen, oder hat er vom Allerbarmer eine Verpflichtung entgegengenommen?\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmcccxxix{Keineswegs! Wir werden aufschreiben, was er sagt, und ihm die Strafe noch verlängern.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmcccxxx{Und Wir erben von ihm das, was er sagt, während er einzeln zu Uns kommt.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmcccxxxi{Und sie haben sich anstatt Allahs Götter genommen, damit sie ihnen zu Macht gereichen.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmcccxxxii{Keineswegs! Sie werden den von ihnen (empfangenen) Dienst verleugnen und werden ihnen Gegner sein.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmcccxxxiii{Siehst du nicht, daß Wir die Satane gegen die Ungläubigen gesandt haben, damit sie sie heftig aufreizen?\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmcccxxxiv{So wünsche nichts gegen sie zu beschleunigen; Wir zählen ihnen schon (die Tage) ab.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmcccxxxv{An einem Tag, da Wir die Gottesfürchtigen als eine (geehrte) Abordnung zum Allerbarmer versammeln\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmcccxxxvi{und die Übeltäter, wie eine durstige Herde zur Tränke zur Hölle treiben,\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmcccxxxvii{verfugen sie nicht über die Fürsprache, außer demjenigen, der vom Allerbarmer eine (diesbezügliche) Verpflichtung entgegengenommen hat.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmcccxxxviii{Und sie sagen: ``Der Allerbarmer hat Sich Kinder genommen."\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmcccxxxix{Ihr habt ja eine abscheuliche Sache begangen.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmcccxl{Beinahe brechen davon die Himmel auseinander, und (beinahe) spaltet sich die Erde, und (beinahe) stürzen die Berge in Trümmern zusammen,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmcccxli{daß sie dem Allerbarmer Kinder zuschreiben.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmcccxlii{Es ziemt dem Allerbarmer nicht, Sich Kinder zu nehmen.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmcccxliii{Niemand in den Himmeln und auf der Erde wird zum Allerbarmer anders denn als Diener kommen (können).\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmcccxliv{Er hat sie erfaßt und sie genau gezählt.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmcccxlv{Und sie alle werden zu Ihm am Tag der Auferstehung einzeln kommen.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmcccxlvi{Gewiß, denjenigen, die glauben und rechtschaffene Werke tun, wird der Allerbarmer Liebe bereiten.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmcccxlvii{Wir haben ihn eigens leicht gemacht in deiner Sprache, damit du durch ihn den Gottesfürchtigen frohe Botschaft verkündest und durch ihn hartnäckige Leute warnst.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmcccxlviii{Und wie viele Geschlechter haben Wir vor ihnen vernichtet! Nimmst du noch irgend jemanden von ihnen wahr, oder hörst du von ihnen noch den geringsten Laut?\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmcccxlix{\basmalahde Ta-Ha.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcccl{Wir haben den Qur'an nicht auf dich (als Offenbarung) hinabgesandt, damit du unglücklich bist,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcccli{sondern als Erinnerung für denjenigen, der gottesfürchtig ist,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmccclii{eine Offenbarung von Demjenigen, Der die Erde und die hohen Himmel erschaffen hat.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcccliii{Der Allerbarmer ist über dem Thron erhaben.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcccliv{Ihm gehört (alles), was in den Himmeln und was auf der Erde ist und was dazwischen und was unter dem (feuchten) Erdreich ist.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmccclv{Und wenn du deine Worte laut vernehmbar äußerst, gewiß so weiß Er ja das Geheime und was noch verborgener ist.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmccclvi{Allah - es gibt keinen Gott außer Ihm. Sein sind die schönsten Namen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmccclvii{Ist zu dir die Geschichte Musas gekommen?\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmccclviii{Als er ein Feuer sah und zu seinen Angehörigen sagte: ``Bleibt hier. Ich habe ein Feuer wahrgenommen; vielleicht kann ich euch davon ein brennendes Stück Holz bringen oder am Feuer eine Wegweisung finden."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmccclix{Als er dort hinkam, wurde ihm zugerufen: ``O Musa!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmccclx{Gewiß, Ich bin dein Herr, so ziehe deine Schuhe aus. Du befindest dich im geheiligten Tal Tuwa.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmccclxi{Und Ich habe dich erwählt. So höre auf das, was (als Offenbarung) eingegeben wird.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmccclxii{Gewiß, Ich bin Allah. Es gibt keinen Gott außer Mir. So diene Mir und verrichte das Gebet zu Meinem Gedenken.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmccclxiii{Gewiß, die Stunde kommt - Ich hielte sie beinahe (ganz) verborgen -, damit jeder Seele das vergolten wird, worum sie sich bemüht.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmccclxiv{So lasse denn nicht denjenigen dich von ihr abhalten, der nicht an sie glaubt und seiner Neigung folgt, sonst würdest du dich ins Verderben stürzen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmccclxv{Und was ist das da in deiner Rechten, o Musa?"\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmccclxvi{Er sagte: ``Es ist mein Stock, auf den ich mich stütze und mit dem ich für meine Schafe Blätter abschlage; und ich gebrauche ihn auch noch zu anderen Zwecken."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmccclxvii{Er sagte: ``Wirf ihn hin, o Musa!"\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmccclxviii{Er warf ihn hin, und sogleich war er eine Schlange, die sich rasch bewegte.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmccclxix{Er sagte: ``Nimm sie, und fürchte dich nicht. Wir werden sie in ihren früheren Zustand zurückbringen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmccclxx{Und lege deine Hand dicht an deine Seite (, unter dem Oberarm), so kommt sie weiß heraus, jedoch nicht von Übel befallen. (Nimm dies) als weiteres Zeichen,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmccclxxi{auf daß Wir dich etwas von Unseren größten Zeichen sehen lassen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmccclxxii{Geh zu Fir'aun, denn gewiß, er überschreitet das Maß (an Frevel)."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmccclxxiii{Er sagte: ``Mein Herr, weite mir meine Brust,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmccclxxiv{und mache mir meine Angelegenheit leicht.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmccclxxv{Und löse den Knoten in meiner Zunge,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmccclxxvi{so daß sie meine Worte verstehen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmccclxxvii{Und bestelle mir von meinen Angehörigen einen, der (die Last) mitträgt,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmccclxxviii{Harun, meinen Bruder.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmccclxxix{Festige durch ihn meine Stärke,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmccclxxx{und lasse ihn an meiner Angelegenheit teilhaben,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmccclxxxi{damit wir Dich häufig preisen\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmccclxxxii{und Deiner häufig gedenken.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmccclxxxiii{Gewiß, Du siehst uns wohl."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmccclxxxiv{Er sagte: ``Deine Bitte ist dir ja gewährt, o Musa!\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmccclxxxv{Und Wir erwiesen dir bereits ein anderes Mal eine Wohltat,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmccclxxxvi{als Wir deiner Mutter eingaben, was (als Weisung) eingegeben werden sollte:\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmccclxxxvii{,Wirf ihn in den Kasten und wirf ihn ins Wasser', und das Wasser soll ihn ans Ufer setzen, so daß ihn ein Feind von Mir und ein Feind von ihm aufnimmt. Und Ich habe auf dich Liebe von Mir gelegt - und damit du vor Meinem Auge aufgezogen würdest.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmccclxxxviii{Als deine Schwester hinging und sagte: .Soll ich euch auf jemanden hinweisen, der ihn betreuen würde? So gaben Wir dich deiner Mutter wieder, damit sie frohen Mutes und nicht traurig sei. Und du tötetest eine Seele, und da erretteten Wir dich aus dem Kummer, und Wir unterzogen dich einer harten Prüfung. So verweiltest du jahrelang unter den Leuten von Madyan. Hierauf kamst du zu einer vorausbe stimmten Zeit, o Musa.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmccclxxxix{Und Ich habe dich für Mich auserwählt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmcccxc{Geh, du und dein Bruder, mit Meinen Zeichen, und laßt nicht nach in Meinem Gedenken.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmcccxci{Geht zu Fir'aun, denn er lehnt sich auf.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmcccxcii{Und so redet mit ihm in sanften Worten, auf daß er bedenken oder sich fürchten möge."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmcccxciii{Sie sagten: ``Unser Herr, gewiß, wir fürchten, daß er übereilig gegen uns vorgeht oder daß er das Maß überschreitet."\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmcccxciv{Er sagte: ``Fürchtet euch nicht. Ich bin gewiß mit euch, Ich höre und Ich sehe (, was geschieht).\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmcccxcv{So kommt denn zu ihm und sagt: ,Wir beide sind Gesandte deines Herrn. Lasse die Kinder Isra'ils mit uns gehen, und strafe sie nicht. Wir sind ja mit einem Zeichen von deinem Herrn zu dir gekommen. Und Friede sei auf demjenigen, der der Rechtleitung folgt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmcccxcvi{Uns ist ja (als Offenbarung) eingegeben worden, daß die Strafe denjenigen überkommt, der (die Botschaft) für Lüge erklärt und sich abkehrt'."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmcccxcvii{Er sagte: ``Wer ist denn euer beider Herr, o Musa?"\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcccxcviii{Er sagte: ``Unser Herr ist derjenige, der allem seine Natur gegeben und (es) hierauf rechtgeleitet hat."\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcccxcix{Er sagte: ``Wie steht es denn mit den früheren Geschlechtern?"\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcd{Er sagte: ``Das Wissen um sie ist bei meinem Herrn in .einem Buch. Mein Herr im nicht, noch vergißt Er."\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcdi{Er ist es,) Der euch die Erde zu einer Lagerstatt gemacht und für euch auf ihr Wege sich hinziehen und vom Himmel Wasser herabkommen läßt, womit Wir dann Arten verschiedener Gewächse hervorbringen.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcdii{Eßt und weidet euer Vieh. Darin sind wahrlich Zeichen für Leute von Verstand.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcdiii{Aus ihr haben Wir euch erschaffen, und in sie bringen Wir euch zurück, und aus ihr bringen Wir euch ein anderes Mal hervor.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcdiv{Und Wir zeigten ihm ja Unsere Zeichen alle, aber er erklärte (sie) für Lüge und weigerte sich.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcdv{Er sagte: ``Bist du zu uns gekommen, um uns aus unserem Land mit deiner Zauberei zu vertreiben, o Musa?\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcdvi{Aber wir werden dir ganz gewiß Zauberei gleicher Art bringen. So setze zwischen uns und dir eine Verabredung(szeit) fest, die weder wir noch du verfehlen werden, an einem Ort, der (uns allen) gleichermaßen recht ist."\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcdvii{Er sagte: ``Eure Verabredung(szeit) soll der Tag des Schmuckfestes sein, und die Menschen sollen am hellen Morgen versammelt werden."\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcdviii{Und so kehrte sich Fir'aun ab, und dann nahm er seine ganze List zusammen. Hierauf kam er.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcdix{Musa sagte zu ihnen: ``Wehe euch! Ersinnt gegen Allah keine Lüge, sonst vertilgt Er euch durch eine Strafe. Enttäuscht wird ja, wer Lügen ersinnt."\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcdx{Da stritten sie untereinander über ihre Angelegenheit und führten insgeheim vertrauliche Gespräche.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcdxi{Sie sagten: ``Diese beiden sind wahrlich nur Zauberer, die euch aus eurem Land mit ihrem Zauber vertreiben und eure vorbildliche Lebensweise beseitigen wollen.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcdxii{So einigt euch auf eure List, hierauf kommt in Reihen. Und wohl ergehen wird es ja heute demjenigen, der die Oberhand gewinnt."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcdxiii{Sie sagten: ``O Musa, entweder wirfst du, oder wir werden es sein, die zuerst werfen."\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcdxiv{Er sagte: ``Nein! Vielmehr werft ihr (zuerst)." Und sogleich kamen ihm ihre Stricke und Stöcke durch ihre Zauberei so vor, als ob sie sich rasch bewegten.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcdxv{Und er, Musa, empfand Furcht in seiner Seele.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmcdxvi{Wir sagten: ``Fürchte dich nicht; du, ja gewiß du, wirst die Oberhand gewinnen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmcdxvii{Wirf hin, was in deiner Rechten ist, so verschlingt es das, was sie gemacht haben. Was sie gemacht haben, ist nur die List eines Zauberers, und dem Zauberer wird es nicht wohl ergehen, wohin er auch kommen mag ."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmcdxviii{Da warfen sich die Zauberer ehrerbietig nieder. Sie sagten: ``Wir glauben an den Herrn Haruns und Musas."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmcdxix{Er (Fir'aun) sagte: ``Ihr glaubt an ihn, bevor ich es euch erlaube? Er ist wahrlich euer Ältester, der euch die Zauberei gelehrt hat. So werde ich ganz gewiß eure Hände und eure Füße wechselseitig abhacken und euch ganz gewiß an Palmstämmen kreuzigen (lassen). Und ihr werdet ganz gewiß erfahren, wer von uns strenger im Strafen und beständiger ist."\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmcdxx{Sie sagten: ``Wir werden dich nicht dem vorziehen, was an klaren Beweisen zu uns gekommen ist, und (vor) Demjenigen, Der uns erschaffen hat. So entscheide, was du entscheiden magst; du entscheidest nur über dieses irdische Leben.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcdxxi{Wir glauben an unseren Herrn, damit Er uns unsere Verfehlungen vergebe und (auch) die Zauberei, zu der du uns gezwungen hast. Allah ist besser und be ständiger."\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmcdxxii{Gewiß, wer zu seinem Herrn als Übeltäter kommt, für den gibt es die Hölle; darin wird er weder sterben noch leben.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmcdxxiii{Wer zu Ihm als Gläubiger kommt, der rechtschaffene Werke getan hat, für jene gibt es die höchsten Rangstufen,\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcdxxiv{die Gärten Edens, durcheilt von Bächen; ewig darin zu bleiben. Das ist der Lohn desjenigen, der sich läutert.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcdxxv{Und Wir gaben Musa ja (als Offenbarung) ein: ``Zieh bei Nacht mit Meinen Dienern fort und schlage ihnen einen trockenen Weg durch das Meer; du befürch test weder, eingeholt zu werden, noch hast du Angst'."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcdxxvi{Da verfolgte sie Fir'aun mit seinen Heerscharen, und es überdeckte sie vom Meer, was sie überdeckte.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmcdxxvii{Fir'aun hatte sein Volk in die Irre geführt und nicht rechtgeleitet.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmcdxxviii{O Kinder Isra'ils, Wir retteten euch ja vor eurem Feind, verabredeten Uns mit euch auf der rechten Seite des Berges und sandten das Manna und die Wachteln auf euch hinab:\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmcdxxix{``Eßt von den guten Dingen, mit denen Wir euch versorgt haben, und lehnt euch dabei nicht (durch Undankbarkeit) auf, sonst bricht Mein Zorn über euch herein; denn derjenige, über den Mein Zorn hereinbricht, wird sicherlich stürzen.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmcdxxx{Und Ich bin wahrlich Allvergebend für denjenigen, der bereut und glaubt und rechtschaffen handelt und sich hierauf rechtleiten läßt."\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmcdxxxi{``Und was hat dich veranlaßt, von deinem Volk fortzueilen, o Musa?"\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmcdxxxii{Er sagte: ``Sie folgen mir doch auf der Spur. Und ich bin zu Dir geeilt, mein Herr, damit Du (mit mir) zufrieden seiest."\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmcdxxxiii{Er sagte: ``Wir haben dein Volk der Versuchung ausgesetzt, nachdem du (weggegangen) warst, und der Samiri hat sie in die Irre geführt."\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmcdxxxiv{Da kam Musa zu seinem Volk zornig und bekümmert zurück. Er sagte: ``O mein Volk, hat euch euer Herr nicht ein schönes Versprechen gegeben? Dauerte es euch mit dem Bund zu lange, oder wolltet ihr, daß Zorn von eurem Herrn über euch hereinbricht, so daß ihr die Vereinbarung mit mir gebrochen habt?"\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmcdxxxv{Sie sagten: ``Wir haben die Vereinbarung mit dir nicht aus unserem (eigenen) Willen gebrochen, sondern wir trugen (ganze) Lasten von den Schmucksachen des Volkes, und dann haben wir sie geworfen, und ebenso hat der Samiri (welche) hineingelegt.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmcdxxxvi{So brachte er ihnen ein Kalb hervor als Leib, das blökte. Sie sagten: ,Das ist euer Gott und der Gott Musas, aber er hat (es) vergessen'."\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmcdxxxvii{Sehen sie denn nicht, daß er ihnen kein Wort erwidert und ihnen weder Schaden noch Nutzen zu bringen vermag?\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmcdxxxviii{Harun hatte ihnen ja bereits zuvor gesagt: ``O mein Volk, ihr seid damit nur der Versuchung ausgesetzt worden. Gewiß, euer Herr ist der Allerbarmer; so folgt mir und gehorcht meinem Befehl."\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmcdxxxix{Sie sagten: ``Wir werden nicht davon ablassen, uns seiner Andacht hinzugeben, bis Musa zu uns zurückkehrt."\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmcdxl{Er (Musa) sagte: ``O Harun, was hat dich, als du sie irregehen sahst, davon abgehalten,\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmcdxli{mir zu folgen? Hast du dich denn meinem Befehl widersetzt? ``\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmcdxlii{Er sagte: ``O Sohn meiner Mutter, pack mich nicht am Bart und nicht am Kopf. Ich fürchtete, du würdest sagen: ,Du hast unter den Kindern Isra'ils Zwietracht gestiftet und mein Wort nicht beachtet'."\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmcdxliii{Er sagte: ``Doch was ist mit dir, o Samiri?"\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmcdxliv{Er sagte: ``Ich gewahrte, was sie nicht gewahrten, und so faßte ich eine Handvoll (Erde) von der Spur des Gesandten und warf sie dann hin (ins Feuer). So habe ich es mir selbst eingeredet."\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmcdxlv{Er sagte: ``Geh weg! Es ist dir im Leben beschieden, zu sagen: .Berührt mich nicht! Und du hast eine Verabredung, die dir nicht gebrochen wird. Und schau auf deinen Gott, dessen Andacht du dich dauernd hingegeben hast. Wir werden ihn ganz gewiß verbrennen, und hierauf werden wir ihn ganz gewiß in das große Gewässer streuen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmcdxlvi{Euer Gott ist allein Allah, außer Dem es keinen Gott gibt. Er umfaßt alles mit (Seinem) Wissen."\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmcdxlvii{Auf diese Weise erzählen Wir dir (einiges) von den Berichten dessen, was früher geschah. Und Wir haben dir ja von Uns aus eine Ermahnung gegeben.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmcdxlviii{Wer sich davon abwendet, der wird am Tag der Auferstehung eine (drückende) Last tragen,\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmcdxlix{ewig darin zu bleiben. Wie böse wird (dies) für sie am Tag der Auferstehung als Last sein!\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmcdl{Am Tag, da ins Horn geblasen wird und Wir die Übeltäter als Leute mit blauen Augen versammeln.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmcdli{Sie flüstern einander zu: ``Ihr habt nur zehn (Nächte) verweilt."\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmcdlii{Wir wissen sehr wohl, was sie sagen, wenn der Vorbildlichste von ihnen in seinem Verhalten sagen wird: ``Ihr habt nur einen Tag verweilt."\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmcdliii{Sie fragen dich nach den Bergen. Sag: Mein Herr wird sie wie Staub zerstreuen\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmcdliv{und sie dann als leere Ebene zurücklassen,\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmcdlv{worauf du weder eine Vertiefung noch Erhebung siehst.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmcdlvi{An jenem Tag folgen sie dem Rufer, bei dem es nichts Krummes gibt; die Stimmen senken sich in Demut vor dem Allerbarmer, so daß du nichts hörst außer Flüstern.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmcdlvii{An jenem Tag nützt die Fürsprache nicht, außer durch denjenigen, dem es der Allerbarmer erlaubt und mit dessen Worten Er zufrieden ist.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmcdlviii{Er weiß, was vor ihnen und was hinter ihnen liegt, sie aber umfassen es nicht mit (ihrem) Wissen.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmcdlix{Demütig werden sich die Gesichter vor dem Lebendigen und Beständigen senken. Und wirklich enttäuscht wird, wer (als Last begangenes) Unrecht trägt.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmcdlx{Wer aber etwas an rechtschaffenen Werken tut und dabei gläubig ist, der wird kein Unrecht und keine (Lohn)minderung befürchten.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmcdlxi{Und so haben Wir ihn als einen arabischen Qur'an hinabgesandt. Und Wir haben darin verschiedene Androhungen dargelegt, auf daß sie gottesfürchtig werden mögen oder er bei ihnen Gedenken (Allahs) hervorrufe.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmcdlxii{Erhaben ist Allah, der König, der Wahre! Und übereile dich nicht mit dem Qur'an, bevor dir seine Offenbarung vollständig eingegeben worden ist. Und sag: Mein Herr, lasse mich an Wissen zunehmen.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmcdlxiii{Und Wir hatten bereits zuvor Adam eine Verpflichtung auferlegt. Aber er vergaß (sie), und Wir fanden bei ihm keine Entschlossenheit.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmcdlxiv{Und als Wir zu den Engeln sagten: ``Werft euch vor Adam nieder." Da warfen sie sich nieder, außer Iblis. Er weigerte sich.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmcdlxv{Da sagten Wir: ``O Adam, dieser (da) ist dir und deiner Gattin gewiß ein Feind. Daß er euch beide ja nicht aus dem (Paradies)garten vertreibt! Sonst wirst du unglücklich sein.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmcdlxvi{Gewiß, es ist dir gewährt, daß du darin weder hungerst noch nackt bist,\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmcdlxvii{und daß du darin weder dürstest noch Sonnenhitze erleidest."\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmcdlxviii{Aber da flüsterte ihm der Satan ein und sagte: ``O Adam, soll ich dich auf den Baum der Ewigkeit hinweisen und auf eine Herrschaft, die nicht vergeht?"\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmcdlxix{So aßen sie beide davon, und da zeigte sich ihnen ihre Blöße offenkundig, und sie begannen, Blätter des (Paradies)gartens auf sich zusammenzuheften. So widersetzte Adam sich seinem Herrn, und da fiel er in Verirrung.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmcdlxx{Hierauf erwählte ihn sein Herr, und so wandte Er Sich ihm Reue-Annehmend zu und leitete (ihn) recht.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmcdlxxi{Er sagte: ``Geht alle fort von ihm. Einige von euch seien des anderen Feind. Doch wenn dann von Mir Rechtleitung zu euch kommt, dann wird derjenige, der Meiner Rechtleitung folgt, nicht irregehen und nicht unglücklich sein.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmcdlxxii{Wer sich aber von Meiner Ermahnung abwendet, der wird ein beengtes Leben fuhren, und Wir werden ihn am Tag der Auferstehung blind (zu den anderen) versammeln.\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmcdlxxiii{Er wird sagen: ``Mein Herr, warum hast Du mich blind (zu den anderen) versammelt, wo ich doch sehen konnte?"\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmcdlxxiv{Er sagt: ``So sind auch zu dir Unsere Zeichen gekommen, und doch hast du sie vergessen. Ebenso wirst du heute vergessen."\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmcdlxxv{So vergelten Wir demjenigen, der maßlos ist und nicht an die Zeichen seines Herrn glaubt. Die Strafe des Jenseits ist wahrlich strenger und nachhaltiger.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmcdlxxvi{Ist ihnen denn nicht deutlich geworden, wie viele Geschlechter Wir vor ihnen vernichtet haben, in deren Wohnorten sie (nun) umhergehen? Darin sind wahrlich Zeichen für Leute von Verstand.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmcdlxxvii{Wenn es nicht ein schon früher ergangenes Wort von deinem Herrn und eine festgesetzte Frist gäbe, wäre (die sofortige Strafe) fürwahr unabwendbar.\qt@no{129}}%
\qt@newcmd\qurantrans@de@mmcdlxxviii{So ertrage standhaft, was sie sagen, und lobpreise deinen Herrn vor dem Aufgang der Sonne und vor ihrem Untergang, und zu (verschiedenen) Stunden der Nacht preise (Ihn) und (ebenso) an den Enden des Tages, auf daß du zufrieden sein mögest.\qt@no{130}}%
\qt@newcmd\qurantrans@de@mmcdlxxix{Und richte nur nicht deine Augen auf das, was Wir manchen von ihnen paarweise als Nießbrauch gewähren - den Glanz des diesseitigen Lebens -, um sie darin der Versuchung auszusetzen. Die Versorgung deines Herrn ist besser und beständiger.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mmcdlxxx{Und befiehl deinen Angehörigen, das Gebet (zu verrichten), und sei beharrlich darin. Wir fordern keine Versorgung von dir; Wir versorgen dich. Und das (gute) Ende gehört der Gottesfurcht.\qt@no{132}}%
\qt@newcmd\qurantrans@de@mmcdlxxxi{Und sie sagen: ``Wenn er uns doch ein Zeichen von seinem Herrn bringen würde!" Ist nicht zu ihnen der klare Beweis dessen gekommen, was auf den früheren Blättern steht?\qt@no{133}}%
\qt@newcmd\qurantrans@de@mmcdlxxxii{Hätten Wir sie vor ihm durch eine Strafe vernichtet, hätten sie fürwahr gesagt: ``Unser Herr, hättest Du doch einen Gesandten zu uns gesandt, so daß wir Deinen Zeichen hätten folgen können, bevor wir erniedrigt und in Schande gestürzt würden!"\qt@no{134}}%
\qt@newcmd\qurantrans@de@mmcdlxxxiii{Sag: Jeder wartet ab; so wartet auch (ihr) ab. Dann werdet ihr (noch) erfahren, wer die Leute des ebenen Weges sind und wer rechtgeleitet ist.\qt@no{135}}%
\qt@newcmd\qurantrans@de@mmcdlxxxiv{\basmalahde Nahegerückt ist den Menschen ihre Abrechnung, während sie sich in Unachtsamkeit abwenden.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcdlxxxv{Keine neuerlich offenbarte Ermahnung kommt von ihrem Herrn zu ihnen, ohne daß sie sie anhören, während sie spielen'\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcdlxxxvi{und ihre Herzen zerstreut sind. Und sie führen insgeheim vertrauliche Gespräche sie, die Unrecht tun: ``Ist dieser etwas anderes als ein menschliches Wesen wie ihr? Wollt ihr denn sehend(en Auges) Zauberei begehen?"\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmcdlxxxvii{Sag: Mein Herr weiß (alles), was im Himmel und auf der Erde gesagt wird. Er ist der Allhörende und Allwissende.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcdlxxxviii{Aber nein! Vielmehr sagen sie: ``(Das ist) ein Bündel von wirren Träumen. Nein! Vielmehr hat er ihn ersonnen. Nein! Vielmehr ist er ein Dichter. So soll er uns ein Zeichen bringen, so wie die Früheren gesandt wurden."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcdlxxxix{Vor ihnen hat keine Stadt geglaubt, die Wir vernichteten. Sollten gerade sie nun glauben?\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcdxc{Und Wir haben vor dir nur Männer gesandt, denen Wir (Offenbarungen) eingaben. So fragt die Leute der Ermahnung, wenn ihr (es) nicht wißt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcdxci{Wir machten sie nicht zu Körpern, die keine Speise essen, und sie waren nicht unsterblich.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcdxcii{Hierauf hielten Wir ihnen das Versprechen. Wir retteten sie und diejenigen, die Wir (erretten) wollen, und vernichteten die Maßlosen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcdxciii{Wir haben ja ein Buch zu euch hinabgesandt, in dem eure Ehre liegt . Begreift ihr denn nicht?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcdxciv{Wie viele Städte haben Wir zerschmettert, die Unrecht taten, und haben nach ihnen ein anderes Volk entstehen lassen!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmcdxcv{Als sie dann Unsere Gewalt spürten, liefen sie sogleich davor weg.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcdxcvi{``Lauft nicht weg, sondern kehrt zu dem euch verliehenen üppigen Leben und zu euren Wohnungen zurück, auf daß ihr befragt werden möget."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcdxcvii{Sie sagten: ``O Wehe uns! Wir pflegten ja Unrecht zu tun."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmcdxcviii{So war dies stets ihr Ausruf, bis Wir sie (wie) abgemäht und ausgelöscht sein ließen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmcdxcix{Und Wir haben den Himmel und die Erde und was dazwischen ist, nicht zum Spiel erschaffen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmd{Wenn Wir Uns eine Zerstreuung hätten nehmen wollen, dann hätten Wir sie wahrlich von Uns aus genommen - wenn Wir das (überhaupt) hätten tun wollen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdi{Aber nein! Wir schleudern die Wahrheit gegen das Falsche, und da zerschmettert sie ihm das Haupt, und sogleich geht es dahin. Und wehe euch wegen dessen, was ihr (da fälschlich) zuschreibt!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdii{Ihm gehört, wer in den Himmeln und auf der Erde ist. Und diejenigen, die bei Ihm sind, sind weder zu hochmütig, Ihm zu dienen, noch werden sie (dessen) müde.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdiii{Sie preisen (Ihn) Nacht und Tag und lassen (darin) nicht nach.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdiv{Oder haben sie sich Götter aus der Erde genommen, die (die Toten) auferstehen lassen (können)?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdv{Wenn es in ihnen beiden andere Götter als Allah gäbe, gerieten sie (beide) wahrlich ins Verderben. Preis sei Allah, dem Herrn des Thrones! (Erhaben ist Er) über das, was sie (Ihm) zuschreiben.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdvi{Er wird nicht befragt nach dem, was Er tut; sie aber werden befragt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdvii{Oder haben sie sich außer Ihm Götter genommen? Sag: Bringt euren Beweis vor! Das ist die Ermahnung derjenigen, die mit mir sind, und die Ermahnung derer, die vor mir waren. Aber nein! Die meisten von ihnen kennen die Wahrheit nicht; so wenden sie sich ab.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdviii{Und Wir haben vor dir keinen Gesandten gesandt, dem Wir nicht (die Weisung) eingegeben hätten: ``Es gibt keinen Gott außer Mir, so dient Mir!"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdix{Und sie sagen: ``Der Allerbarmer hat Sich Kinder genommen." Preis sei Ihm! Nein! Vielmehr sind es geehrte Diener.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdx{Sie kommen Ihm im Reden nicht zuvor, und nur nach Seinem Befehl handeln sie.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdxi{Er weiß, was vor ihnen und was hinter ihnen liegt, und sie legen Fürsprache nur für denjenigen ein, dem Er zustimmt. Und sie sind aus Furcht vor Ihm besorgt.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdxii{Und wer von ihnen sagen sollte: ``Ich bin ein Gott außer Ihm", dem vergelten Wir mit der Hölle. So vergelten Wir den Ungerechten.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdxiii{Sehen denn diejenigen, die ungläubig sind, nicht, daß die Himmel und die Erde eine zusammenhängende Masse waren? Da haben Wir sie getrennt und aus dem Wasser alles Lebendige gemacht. Wollen sie denn nicht glauben?\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdxiv{Und Wir haben auf der Erde festgegründete Berge gemacht, daß sie nicht mit ihnen wanke. Und Wir haben auf ihr breite Durchgänge als Wege gemacht, auf daß sie rechtgeleitet werden mögen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdxv{Und Wir haben den Himmel zu einem wohlbehüteten Dach gemacht. Dennoch wenden sie sich von seinen Zeichen ab.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdxvi{Und Er ist es, Der die Nacht und den Tag, die Sonne und den Mond erschaffen hat; alles läuft in einer (jeweils eigenen) Umlaufbahn.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdxvii{Und Wir haben für kein menschliches Wesen vor dir ewiges Leben bestimmt. Wenn du nun stirbst, werden sie dann ewig leben?\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdxviii{Jede Seele wird den Tod kosten. Und Wir prüfen euch mit Schlechtem und Gutem als Versuchung. Und zu Uns werdet ihr zurückgebracht.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdxix{Und wenn diejenigen, die ungläubig sind, dich sehen, machen sie sich über dich nur lustig: ``Ist das derjenige, der eure Götter (in abfälliger Weise) erwähnt?", während sie es sind, die die Ermahnung des Allerbarmers verleugnen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdxx{Der Mensch ist (seiner Natur nach) aus Voreiligkeit erschaffen worden. Ich werde euch (schon) Meine Zeichen zeigen; so wünscht nicht von Mir, (sie) zu beschleunigen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdxxi{Und sie sagen: ``Wann wird dieses Versprechen eintreten, wenn ihr wahrhaftig seid?"\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdxxii{Wenn diejenigen, die ungläubig sind, (es) nur wüßten, (wie es ist,) wenn sie das (Höllen)feuer weder von ihren Gesichtern noch von ihren Rücken zurückhalten können und ihnen auch keine Hilfe zuteil wird!\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdxxiii{Aber nein! Es wird plötzlich über sie kommen und sie verblüffen. Dann werden sie es nicht abwenden können, und es wird ihnen kein Aufschub gewährt.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdxxiv{Man machte sich ja schon vor dir über Gesandte lustig. Da umschloß diejenigen, die über sie spotteten, das, worüber sie sich lustig zu machen pflegten.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdxxv{Sag: Wer beschützt euch bei Nacht und Tag vor dem Allerbarmer? Aber nein! Sie wenden sich von der Ermahnung ihres Herrn ab.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdxxvi{Oder haben sie (etwa) Götter, die sie von Uns abhalten (sollten)? Sie können doch weder sich selbst helfen, noch werden sie einen Beschützer vor Uns erhalten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdxxvii{Nein! Vielmehr haben Wir diesen da und ihren Vätern Nießbrauch gewährt, bis sie ein langes Leben geführt haben. Sehen sie denn nicht, daß Wir über das Land kommen und es an seinen Enden vermindern? Werden denn sie die Sieger sein?\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdxxviii{Sag: Ich warne euch nur mit der Eingebung. Aber die Tauben hören nicht den Ruf, wenn sie gewarnt werden.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdxxix{Aber wenn ein Hauch von der Strafe deines Herrn sie berührt, sagen sie ganz gewiß: ``O wehe uns! Wir pflegten ja Unrecht zu tun."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdxxx{Und Wir stellen die gerechten Waagen für den Tag der Auferstehung auf. So wird keiner Seele um irgend etwas Unrecht zugefügt; und wäre es auch das Gewicht eines Senfkorns, Wir bringen es bei. Und Wir genügen als Berechner.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdxxxi{Und Wir gaben ja Musa und Harun die Unterscheidung(snorm) und eine Erleuchtung und eine Ermahnung für die Gottesfürchtigen,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdxxxii{die ihren Herrn im Verborgenen fürchten und die wegen der Stunde besorgt sind.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdxxxiii{Und dies ist eine gesegnete Ermahnung, die Wir hinabgesandt haben. Wollt ihr sie denn verwerfen?\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdxxxiv{Wir verliehen bereits zuvor Ibrahim seine Besonnenheit; und Wir wußten über ihn Bescheid.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdxxxv{Als er zu seinem Vater und seinem Volk sagte: ``Was sind das für Bildwerke, deren Andacht ihr euch hingebt?"\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdxxxvi{Sie sagten: ``Wir haben (schon) unsere Väter (vor)gefunden, wie sie ihnen dienten."\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdxxxvii{Er sagte: ``Ihr und eure Väter befindet euch ja in einem deutlichen Irrtum."\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdxxxviii{Sie sagten: ``Bringst du uns die Wahrheit, oder gehörst du zu denjenigen, die ihr Spiel treiben?"\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdxxxix{Er sagte: ``Nein! Vielmehr ist euer Herr der Herr der Himmel und der Erde, Der sie erschaffen hat. Und ich gehöre zu denjenigen, die euch darüber Zeugnis ablegen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdxl{Und bei Allah, ich werde ganz gewiß gegen eure Götzen eine List anwenden, nachdem ihr den Rücken gekehrt habt."\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdxli{Da schlug er sie in Stücke, außer einem großen von ihnen, auf daß sie sich an ihn wenden möchten.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdxlii{Sie sagten: ``Wer hat dies mit unseren Göttern getan? Er gehört wahrlich zu den Ungerechten."\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdxliii{Sie sagten: ``Wir hörten einen Jüngling sie (in abfälliger Weise) erwähnen; man nennt ihn Ibrahim."\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdxliv{Sie sagten: ``So bringt ihn her vor die Augen der Menschen, auf daß sie es bezeugen mögen."\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdxlv{Sie sagten: ``Hast du dies mit unseren Göttern getan, o Ibrahim?"\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdxlvi{Er sagte: ``Nein! Vielmehr hat das dieser Große unter ihnen getan. Fragt sie doch, wenn sie reden können."\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdxlvii{Da kamen sie wieder zu sich und sagten: ``Ihr seid ja die Ungerechten."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdxlviii{Hierauf machten sie eine Kehrtwendung': ``Du weißt doch, daß diese nicht reden können."\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmdxlix{Er sagte: ``Dient ihr denn anstatt Allahs dem, was euch nichts nützt und nicht schadet?\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmdl{Pfui über euch und über das, dem ihr anstatt Allahs dient! Begreift ihr denn nicht?"\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmdli{Sie sagten: ``Verbrennt ihn und helft euren Göttern, wenn ihr etwas tun wollt."\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmdlii{Wir sagten: ``O Feuer, sei Kühlung und Unversehrtheit für Ibrahim."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmdliii{Sie wollten gegen ihn mit einer List vorgehen. Da machten Wir sie zu den größten Verlierern.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmdliv{Und Wir erretteten ihn und Lut in das Land, das Wir für (all) die Weltenbewohner gesegnet haben.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmdlv{Und Wir schenkten ihm lshaq und Ya'qub zusätzlich; und alle machten Wir rechtschaffen.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmdlvi{Und Wir machten sie zu Vorbildern, die nach Unserem Befehl leiteten. Und Wir gaben ihnen ein, gute Werke zu tun, das Gebet zu verrichten und die Abgabe zu entrichten. Und sie pflegten Uns zu dienen.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmdlvii{Und Lut gaben Wir Urteil(skraft) und Wissen und erretteten ihn aus der Stadt, die schlechte Taten zu begehen pflegte. Sie waren ja böse Leute, sie waren Frevler.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmdlviii{Und Wir ließen ihn in Unsere Barmherzigkeit eingehen. Gewiß, er gehört zu den Rechtschaffenen.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmdlix{Und (auch) Nuh, als er zuvor rief. Da erhörten Wir ihn und erretteten darauf ihn und seine Angehörigen aus der großen Trübsal.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmdlx{Und Wir halfen ihm vor den Leuten, die Unsere Zeichen für Lüge erklärten. Sie waren ja böse Leute, und so ließen Wir sie allesamt ertrinken.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmdlxi{Und (auch) Dawud und Sulaiman, als sie über das Saatfeld urteilten. Darin hatte Kleinvieh von (fremden) Leuten nachts geweidet. Und Wir waren für ihr Urteil Zeugen.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmdlxii{Und Wir ließen Sulaiman es begreifen. Und allen gaben Wir Urteil(skraft) und Wissen. Und Wir machten die Berge zusammen mit Dawud dienstbar, daß sie (Uns) preisen, und (auch) die Vögel. Wir taten (es wirklich).\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmdlxiii{Und Wir lehrten ihn die Verfertigung von (Panzer)kleidung für euch, damit sie euch vor eurer (gegenseitigen) Gewalt schütze. Wollt ihr nun dankbar sein?\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmdlxiv{Und Sulaiman (machten Wir) den Sturmwind (dienstbar), daß er auf seinen Befehl zum Land wehe, das Wir gesegnet haben - und Wir wußten über alles Bescheid -,\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmdlxv{und auch unter den Satanen manche, die für ihn tauchten und (auch noch andere) Arbeiten außer dieser verrichteten; und Wir bewachten sie.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmdlxvi{Und (auch) Ayyub, als er zu seinem Herrn rief: ``Mir ist gewiß Unheil widerfahren, doch Du bist der Barmherzigste der Barmherzigen."\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmdlxvii{Da erhörten Wir ihn und nahmen das Unheil, das auf ihm war, von ihm hinweg, und gaben ihm seine Angehörigen und noch einmal die gleiche Zahl dazu, aus Barmherzigkeit von Uns und als Ermahnung für diejenigen, die (Uns) dienen.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmdlxviii{Und (auch) Isma'il und Idris und Du'1-Kifl. Jeder gehörte zu den Standhaften.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmdlxix{Und Wir ließen sie in Unsere Barmherzigkeit eingehen. Gewiß, sie gehören zu den Rechtschaffenen.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmdlxx{Und (auch) dem Mann mit dem Fisch', als er erzürnt wegging. Da meinte er, Wir würden ihm nicht (den Lebensunterhalt) bemessen. Dann rief er in den Finsternissen: ``Es gibt keinen Gott außer Dir! Preis sei Dir! Gewiß, ich gehöre zu den Ungerechten."\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmdlxxi{Da erhörten Wir ihn und erretteten ihn aus dem Kummer. So retten Wir die Gläubigen.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmdlxxii{Und (auch) Zakariyya, als er zu seinem Herrn rief: ``Mein Herr, lasse mich nicht kinderlos bleiben, und Du bist der beste Erbe."\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmdlxxiii{Da erhörten Wir ihn und schenkten ihm Yahya und besserten ihm seine Gattin. Sie pflegten sich ja mit den guten Dingen zu beeilen und Uns in Begehren und Ehrfurcht anzurufen, und sie pflegten vor Uns demütig zu sein.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmdlxxiv{Und (auch) diejenige, die ihre Scham unter Schutz stellte'. Da hauchten Wir ihr von Unserem Geist ein und machten sie und ihren Sohn zu einem Zeichen für die Weltenbewohner.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmdlxxv{``Gewiß, diese ist eure Gemeinschaft, eine einzige Gemeinschaft, und Ich bin euer Herr; so dient Mir!"\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmdlxxvi{Aber sie spalteten sich in ihrer Angelegenheit untereinander; doch sie werden alle zu Uns zurückkehren.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmdlxxvii{Wer etwas an rechtschaffenen Werken tut und dabei gläubig ist, der wird für sein Bemühen nicht Undank ernten; Wir schreiben es ihm gut.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmdlxxviii{Und verboten ist es für (die Bewohner) eine(r jeden) Stadt, die Wir vernichteten, daß sie (ins Diesseits) zurückkehren,\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmdlxxix{bis die (Sperrmauer von) Ya'gug und Ma'güg geöffnet wird und sie von jeder Anhöhe schnell herbeilaufen\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmdlxxx{und das wahre Versprechen nahegerückt ist, dann werden sogleich die Blicke derjenigen, die ungläubig sind, starr werden: ``O wehe uns! Wir waren dessen ja unachtsam. Nein! Vielmehr pflegten wir Unrecht zu tun."\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmdlxxxi{``Gewiß, ihr und das, dem ihr anstatt Allahs dient, seid Brennstoff der Hölle. Ihr werdet dahin hinabgehen."\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmdlxxxii{Wenn diese da Götter wären, gingen sie nicht zu ihr hinab. Und alle werden ewig darin bleiben.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmdlxxxiii{Sie werden darin seufzen und selbst darin nicht hören.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmdlxxxiv{Gewiß, diejenigen, an die von Uns (das Versprechen für) die beste Behandlung vorausgegangen ist, sie werden von ihr ferngehalten,\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmdlxxxv{sie hören von ihr nicht (einmal) das leiseste Geräusch. Und sie werden in dem, was ihre Seelen begehrt haben, ewig bleiben.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmdlxxxvi{Der größte Schrecken macht sie nicht traurig, und die Engel empfangen sie: ``Das ist euer Tag, der euch versprochen wurde."\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmdlxxxvii{An dem Tag, da Wir den Himmel zusammenfalten, wie der Urkundenschreiber die Schriftstücke zusammenfaltet. Wie Wir eine erste Schöpfung am Anfang gemacht haben, wiederholen Wir sie; (das ist) ein für Uns bindendes Versprechen. Wir wer den es bestimmt tun.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmdlxxxviii{Und Wir haben bereits im Buch der Weisheit nach der Ermahnung geschrieben, daß Meine rechtschaffenen Diener das Land erben werden.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmdlxxxix{In diesem ist fürwahr eine Botschaft an Leute, die (Uns) dienen.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmdxc{Und Wir haben dich nur als Barmherzigkeit für die Weltenbewohner gesandt.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmdxci{Sag: Mir wird (als Offenbarung) nur eingegeben, daß euer Gott nur ein einziger Gott ist. Werdet ihr nun (Allah) ergeben sein?\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmdxcii{Wenn sie sich abkehren, dann sag: Ich habe es euch (allen) gleichermaßen angekündigt, und ich weiß nicht, ob das nahe ist oder fernliegt, was euch versprochen wird.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmdxciii{Gewiß, Er weiß, was an Worten laut vernehmbar geäußert wird, und Er weiß, was ihr verheimlicht.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmdxciv{Und ich weiß (auch) nicht, ob es vielleicht nur eine Versuchung für euch ist und ein Nießbrauch auf Zeit.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmdxcv{Er sagte: Mein Herr, richte der Wahrheit entsprechend. Und unser Herr ist der Allerbanner, bei Dem Hilfe zu suchen ist gegen das, was ihr beschreibt.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmdxcvi{\basmalahde O ihr Menschen, fürchtet euren Herrn. Gewiß, das Beben der Stunde ist eine gewaltige Sache.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdxcvii{An dem Tag, da ihr es seht, wird jede Stillende (aus Entsetzen) übersehen, was sie (soeben) stillt, und jede Schwangere wird mit dem niederkommen, was sie trägt. Und du siehst die Menschen trunken, obwohl sie nicht betrunken sind; aber die Strafe Allahs ist streng.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdxcviii{Und unter den Menschen gibt es manch einen, der über Allah ohne Wissen streitet und jedem rebellischen Satan folgt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdxcix{gegen den vorgezeichnet ist, daß er denjenigen, der ihn zum Schutzherr nimmt, daß er ihn in die Irre führen und zur Strafe der Feuerglut leiten wird.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdc{O ihr Menschen, wenn ihr über die Auferweckung im Zweifel seid, so haben Wir euch aus Erde erschaffen, hierauf aus einem Samentropfen, hierauf aus einem Anhängsel, hierauf aus einem kleinen Klumpen, gestaltet und ungestaltet, um (es) euch klarzumachen. Und Wir lassen, was Wir wollen, im Mutterleib auf eine festgesetzte Frist untergebracht. Danach lassen Wir euch als kleine Kinder hervorkommen. Hierauf (lassen Wir euch heranwachsen), damit ihr eure Vollreife erlangt. Und mancher von euch wird (frühzeitig) abberufen, und manch einer von euch wird in das niedrigste (Greisen)alter gebracht, so daß er nach (dem vorherigen) Wissen nichts (mehr) weiß. Und du siehst die Erde regungslos, doch wenn Wir Wasser auf sie herabkommen lassen, regt sie sich, schwillt und läßt von jeder entzückenden (Pflanzen)art wachsen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdci{Dies, weil Allah die Wahrheit ist und weil Er die Toten wieder lebendig macht und weil Er zu allem die Macht hat\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdcii{und weil die Stunde kommt, an der es keinen Zweifel gibt, und weil Allah (all) diejenigen auferwecken wird, die in den Gräbern sind.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdciii{Doch gibt es unter den Menschen manchen, der über Allah ohne Wissen, ohne Rechtleitung und ohne erleuchtendes Buch streitet,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdciv{indem er sich (hochmütig) zur Seite wendet, um von Allahs Weg in die Irre zu führen. Schande gibt es für ihn im Diesseits, und Wir lassen ihn am Tag der Auferstehung die Strafe des Brennens kosten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdcv{``Dies (geschieht) wegen dessen, was deine Hände vorausgeschickt haben, und deshalb, weil Allah keiner ist, der den Menschen Unrecht zufügt."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdcvi{Und unter den Menschen gibt es manchen, der Allah nur am Rande dient. Wenn ihn etwas Gutes trifft, ist er damit beruhigt, doch wenn ihn eine Versuchung trifft, macht er eine Kehrtwende. Er verliert das Diesseits und das Jenseits. Das ist der deutliche Verlust.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdcvii{Er ruft anstatt Allahs das an, was ihm weder schadet noch nützt. Das ist wirklich der tiefe Irrtum.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdcviii{Er ruft denjenigen an, von dem eher Schaden (zu erwarten) ist als Nutzen - ein wahrlich schlimmer Schutzherr und ein wahrlich schlimmer Gefährte!\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdcix{Gewiß, Allah läßt diejenigen, die glauben und rechtschaffene Werke tun, in Gärten eingehen, durcheilt von Bächen. Gewiß, Allah tut, was Er will.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdcx{Wer meint, daß Allah ihm im Diesseits und Jenseits nicht helfen werde, der strecke doch ein Seil zum Himmel, dann schneide er (es) ab. So schaue er, ob nun seine List das wegnimmt, was (ihn) ergrimmen läßt.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdcxi{So haben Wir ihn als klare Zeichen hinabgesandt, und (es ist so,) daß Allah rechtleitet, wen Er will.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdcxii{Gewiß, diejenigen, die glauben, und diejenigen, die dem Judentum angehören, und die Säbier und die Christen und die Magus und diejenigen, die Götzendiener sind gewiß, Allah wird am Tag der Auferstehung zwischen ihnen entscheiden; Allah ist ja über alles Zeuge.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdcxiii{Siehst du nicht, daß sich vor Allah (jeder) niederwirft, wer in den Himmeln und wer auf der Erde ist, und (auch) die Sonne, der Mond und die Sterne, die Berge, die Bäume und die Tiere und viele von den Menschen? Und gegen viele ist die Strafe unvermeidlich geworden. Und wen Allah der Schmach aussetzt, der hat niemanden, der ihm Ehre erweist. Gewiß, Allah tut, was Er will.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdcxiv{Das sind zwei Widersacher, die miteinander über ihren Herrn streiten. Für diejenigen nun, die ungläubig sind, werden Gewänder aus Feuer zugeschnitten; über ihre Köpfe wird heißes Wasser gegossen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdcxv{Dadurch wird zum Schmelzen gebracht, was sie in ihrem Bauch haben, und ebenso die Haut.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdcxvi{Und für sie gibt es Keulen aus Eisen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdcxvii{Jedesmal, wenn sie vor Kummer aus ihm herauskommen wollen, werden sie dahin zurückgebracht, und (es wird zu ihnen gesagt): ``Kostet die Strafe des Brennens!"\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdcxviii{Gewiß, Allah läßt diejenigen, die glauben und rechtschaffene Werke tun, in Gärten eingehen, durcheilt von Bächen, worin sie mit Armreifen aus Gold und mit Perlen geschmückt sein werden und worin ihre Kleidung aus Seide sein wird.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdcxix{Rechtgeleitet sind sie zu dem Wort, das gut ist; und rechtgeleitet sind sie zum Weg des Lobenswürdigen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdcxx{Gewiß, diejenigen, die ungläubig sind und von Allahs Weg abhalten und (auch) von der geschützten Gebetsstätte, die Wir für die Menschen bestimmt haben, gleich ob sie dort ansässig oder nur vorübergehend anwesend sind (, werden ihre Strafe bekommen). Wer aber beabsichtigt, dort etwas Abwegiges zu Unrecht (zu begehen), den werden Wir von einer schmerzhaften Strafe kosten lassen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdcxxi{Und als Wir Ibrahim die Stelle des Hauses zuwiesen: ``Geselle Mir nichts bei und reinige Mein Haus für die den Umlauf Vollziehenden, die aufrecht Stehenden, sich Verbeugenden und die sich Niederwerfenden.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdcxxii{Und rufe unter den Menschen die Pilgerfahrt aus, so werden sie zu dir kommen zu Fuß und auf vielen hageren (Reittieren), die aus jedem tiefen Paßweg da herkommen,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdcxxiii{damit sie (allerlei) Nutzen für sich erfahren und den Namen Allahs an wohlbekannten Tagen über den aussprechen, womit Er sie an den Vierfüßlern unter dem Vieh versorgt hat. - Eßt (selbst) davon und gebt dem Elenden, dem Armen zu essen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdcxxiv{Hierauf sollen sie ihre Ungepflegtheit beenden, ihre Gelübde erfüllen und den Umlauf um das alt(ehrwürdig)e Haus vollziehen."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdcxxv{So (ist es). Und wenn einer die unantastbaren Dinge Allahs hoch ehrt, so ist es besser für ihn bei seinem Herrn. Erlaubt ist euch das Vieh, außer dem, was euch verlesen wird. So meidet den Greuel der Götzenbilder, und meidet die falsche Aussage,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdcxxvi{als Anhänger des rechten Glaubens gegenüber Allah, die Ihm nichts beigesellen. Und wenn einer Allah (etwas) beigesellt, so ist es, als ob er vom Himmel herunterfiele und er dann von den Vögeln fortgerissen oder vom Wind an einen fernen Ort hinabgeweht würde.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdcxxvii{So ist es. Und wenn einer die Kulthandlungen Allahs hoch ehrt, so ist es (ein Ausdruck, der) von der Gottesfurcht der Herzen (herrührt).\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdcxxviii{An ihnen habt ihr (allerlei) Nutzen auf eine festgesetzte Frist, hierauf liegt ihr Zielort beim alt(ehrwürdig)en Haus.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdcxxix{Und für jede Gemeinschaft haben Wir einen Ritus festgelegt, damit sie den Namen Allahs über den aussprechen, womit Er sie an den Vierfüßlern unter dem Vieh versorgt hat. Euer Gott ist ein Einziger Gott, so seid Ihm ergeben. Und verkünde frohe Botschaft denjenigen, die sich demütigen,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdcxxx{denjenigen, deren Herzen sich vor Ehrfurcht regen, wenn Allahs gedacht wird, die das standhaft ertragen, was sie trifft, das Gebet verrichten und von dem, womit Wir sie versorgt haben, ausgeben.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdcxxxi{Und die Opferkamele haben Wir euch zu Kultzeichen Allahs gemacht. An ihnen habt ihr (etwas) Gutes. So sprecht den Namen Allahs über sie aus, wenn sie mit gebundenen Beinen dastehen. Wenn sie nun auf die Seite umgefallen sind, dann eßt davon und gebt dem bescheidenen und dem fordernden (Armen) zu essen. So haben Wir sie euch dienstbar gemacht, auf daß ihr dankbar sein möget.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdcxxxii{Weder ihr Fleisch noch ihr Blut werden Allah erreichen, aber Ihn erreicht die Gottesfurcht von euch. So hat Er sie euch dienstbar gemacht, damit ihr Allah als den Größten preist, daß Er euch rechtgeleitet hat. Und verkünde frohe Botschaft den Gutes Tuenden.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdcxxxiii{Gewiß, Allah verteidigt diejenigen, die glauben. Gewiß, Allah liebt keinen undankbaren Verräter.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdcxxxiv{Erlaubnis (zum Kampf) ist denjenigen gegeben, die bekämpft werden, weil ihnen ja Unrecht zugefügt wurde - und Allah hat wahrlich die Macht, ihnen zu helfen -,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdcxxxv{ihnen), die zu Unrecht aus ihren Wohnstätten vertrieben wurden, nur weil sie sagen: Unser Herr ist Allah. Und wenn Allah nicht die einen Menschen durch die anderen abgewehrt hätte, so wären fürwahr Mönchsklausen, Kirchen, Bethäuser und Gebetsstätten zerstört worden, in denen Allahs Name häufig genannt wird. Und Allah wird ganz gewiß denjenigen helfen, die Ihm helfen. Allah ist wahrlich Stark und Allmächtig.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdcxxxvi{- (Ihnen), die, wenn Wir ihnen eine feste Stellung auf der Erde verleihen, das Gebet verrichten und die Abgabe entrichten, das Rechte gebieten und das Verwerfliche verbieten. Und Allah gehört das Ende der Angelegenheiten.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdcxxxvii{Wenn sie dich der Lüge bezichtigen, so haben auch schon vor ihnen das Volk Nuhs, die (Ad und die Tamud (ihre Gesandten) der Lüge bezichtigt,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdcxxxviii{und (auch) das Volk Ibrahims und das Volk Luts\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdcxxxix{und die Bewohner von Madyan. Auch Musa wurde der Lüge bezichtigt. Da gewährte Ich den Ungläubigen Aufschub, hierauf ergriff Ich sie. Wie war da Meine Mißbilligung!\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdcxl{Wie viele Städte, die Unrecht taten, vernichteten Wir, so daß sie wüst in Trümmern lagen, und (wie viele nun) verlassene Brunnen und hochragende Schlösser!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdcxli{Reisen sie denn nicht auf der Erde umher, so daß sie Herzen bekommen, mit denen sie begreifen, oder Ohren, mit denen sie hören? Denn nicht die Blicke sind blind, sondern blind sind die Herzen, die in den Brüsten sind.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdcxlii{Und sie wünschen von dir, die Strafe zu beschleunigen. Allah wird Sein Versprechen nicht brechen. Und gewiß, ein Tag bei deinem Herrn ist wie tausend Jahre nach eurer Berechnung.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdcxliii{Und wie vielen Städten, die Unrecht taten, gewährte Ich Aufschub! Hierauf ergriff Ich sie, und zu Mir ist der Ausgang.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdcxliv{Sag: O ihr Menschen, ich bin euch nur ein deutlicher Warner.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdcxlv{Diejenigen nun, die glauben und rechtschaffene Werke tun, für sie wird es Vergebung und ehrenvolle Versorgung geben.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdcxlvi{Diejenigen hingegen, die ständig gegen Unsere Zeichen eifern und ihnen Machtlosigkeit nachweisen wollen, das sind Insassen des Höllenbrandes.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdcxlvii{Und Wir sandten vor dir keinen Gesandten oder Propheten, ohne daß ihm, wenn er etwas wünschte, der Satan in seinen Wunsch etwas dazwischen geworfen hätte. Aber Allah hebt auf, was der Satan dazwischenwirft. Hierauf legt Allah Seine Zeichen eindeutig fest. Und Allah ist Allwissend und Allweise.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdcxlviii{Das ist so,) damit Er das, was der Satan dazwischenwirft, zu einer Versuchung für diejenigen macht, in deren Herzen Krankheit ist und deren Herzen verhärtet sind die Ungerechten befinden sich wahrlich in tiefem Widerstreit -,\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdcxlix{und damit diejenigen, denen das Wissen gegeben wurde, wissen, daß dies die Wahrheit von deinem Herrn ist, so daß sie daran glauben und ihre Herzen sich dann davor demütigen. Und Allah leitet wahrlich diejenigen, die glauben, zu einem geraden Weg.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdcl{Und diejenigen, die ungläubig sind, werden nicht aufhören, daran zu zweifeln, bis plötzlich die Stunde über sie kommt oder die Strafe eines unheilvollen Tages über sie kommt.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdcli{Die Herrschaft wird an jenem Tag Allah (allein) gehören. Er wird zwischen ihnen richten. Dann werden diejenigen, die glauben und rechtschaffene Werke tun, in den Gärten der Wonne sein.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdclii{Diejenigen aber, die ungläubig sind und Unsere Zeichen für Lüge erklären, für sie wird es schmachvolle Strafe geben.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdcliii{Denjenigen, die auf Allahs Weg auswandern, hierauf getötet werden oder sterben, wird Allah ganz gewiß eine schöne Versorgung gewähren. Allah ist wahrlich der beste Versorger.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdcliv{Er wird ihnen ganz gewiß einen Eingang gewähren, mit dem sie zufrieden sind. Und Allah ist wahrlich Allwissend und Nachsichtig.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdclv{Dies, und wer im gleichen Maß bestraft, wie er bestraft wurde, hierauf aber (wieder zu Unrecht) angegriffen wird, dem wird Allah ganz gewiß helfen. Allah ist wahrlich Allverzeihend und Allvergebend.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdclvi{Dies, weil Allah die Nacht in den Tag eindringen und den Tag in die Nacht eindringen läßt und weil Allah Allhörend und Allsehend ist.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdclvii{Dies, weil Allah die Wahrheit ist und weil das, was sie anstatt Seiner anrufen, das Falsche ist, und weil Allah der Erhabene, der Große ist.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdclviii{Siehst du nicht, daß Allah vom Himmel Wasser herabkommen läßt, so daß die Erde grün wird? Gewiß, Allah ist Feinfühlig und Allkundig.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdclix{Ihm gehört (alles), was in den Himmeln und was auf der Erde ist; Allah ist wahrlich der Unbedürftige und Lobenswürdige.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdclx{Siehst du nicht, daß Allah euch (all) das, was auf der Erde ist, dienstbar gemacht hat, und (ebenso) die Schiffe, die nach Seinem Befehl auf dem Meer fahren? Und Er hält den Himmel, daß er nicht auf die Erde fällt, außer mit Seiner Erlaubnis. Gewiß, Allah ist zu den Menschen wahrlich Gnädig und Barmherzig.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmdclxi{Und Er ist es, Der euch lebendig gemacht hat, euch hierauf sterben läßt, euch hierauf wieder lebendig macht. Der Mensch ist wahrlich sehr undankbar.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmdclxii{Für jede Gemeinschaft haben Wir einen Ritus festgelegt, den sie (zu) vollziehen (haben); so sollen sie mit dir nicht über die Angelegenheit streiten. Und rufe zu deinem Herrn. Du verfährst fürwahr nach einer geraden Rechtleitung.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmdclxiii{Und wenn sie (doch) mit dir streiten, dann sag: Allah weiß sehr wohl, was ihr tut.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmdclxiv{Allah wird zwischen euch am Tag der Auferstehung über das richten, worüber ihr uneinig zu sein pflegtet.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmdclxv{Weißt du denn nicht, daß Allah weiß, was im Himmel und auf der Erde ist? Gewiß, das steht in einem Buch; gewiß, das ist Allah ein leichtes.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmdclxvi{Und sie dienen anstatt Allahs dem, wofür Er keine Ermächtigung herabgesandt hat und wovon sie kein Wissen haben. Und es wird für die Ungerechten keinen Helfer geben.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmdclxvii{Und wenn ihnen Unsere Zeichen als klare Beweise verlesen werden, erkennst du in den Gesichtern derjenigen, die ungläubig sind, Mißbilligung. Beinahe würden sie über diejenigen herfallen, die ihnen Unsere Zeichen verlesen. Sag: Soll ich euch denn kundtun, was schlechter ist als dies? (Es ist) das (Höllen)feuer; Allah hat es denjenigen angedroht hat, die ungläubig sind - ein schlimmer Ausgang!\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmdclxviii{O ihr Menschen, ein Gleichnis wird (euch) angeführt', so hört darauf. Gewiß, diejenigen, die ihr anstatt Allahs anruft, werden nicht (einmal) eine Fliege erschaffen (können), auch wenn sie sich dafür zusammentäten. Und wenn die Fliege ihnen etwas raubte, könnten sie es ihr nicht entreißen. Schwach ist (hier) derjenige, der ersucht, und derjenige, der ersucht wird.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmdclxix{Sie schätzen Allah nicht ein, wie es Ihm gebührt. Allah ist wahrlich Stark und Allmächtig.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmdclxx{Allah erwählt Sich aus den Engeln Gesandte, und (auch) aus den Menschen. Gewiß, Allah ist Allhörend und Allsehend.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmdclxxi{Er weiß, was vor ihnen und was hinter ihnen liegt. Und zu Allah werden (all) die Angelegenheiten zurückgebracht.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmdclxxii{O die ihr glaubt, verbeugt euch, werft euch nieder und dient eurem Herrn und tut das Gute, auf daß es euch wohl ergehen möge!\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmdclxxiii{Und müht euch für Allah ab, wie der wahre Einsatz für Ihn sein soll. Er hat euch erwählt und euch in der Religion keine Bedrängnis auferlegt, dem Glau bensbekenntnis eures Vaters Ibrahim: Er hat euch Muslime genannt, zuvor und (nunmehr) in diesem (Qur'an), damit der Gesandte Zeuge über euch sei und ihr Zeugen über die Menschen seid. So verrichtet das Gebet, entrichtet die Abgabe und haltet an Allah fest. Er ist euer Schutzherr. Wie trefflich ist doch der Schutzherr, und wie trefflich ist der Helfer!\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmdclxxiv{\basmalahde Den Gläubigen wird es ja wohl ergehen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdclxxv{denjenigen, die in ihrem Gebet demütig sind,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdclxxvi{und denjenigen, die sich von unbedachter Rede abwenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdclxxvii{und denjenigen, die die (Zahlung der) Abgabe anwenden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdclxxviii{und denjenigen, die ihre Scham hüten,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdclxxix{außer gegenüber ihren Gattinnen oder was ihre rechte Hand (an Sklavinnen) besitzt, denn sie sind (hierin) nicht zu tadeln,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdclxxx{- wer aber darüber hinaus (etwas) begehrt, das sind die Übertreter -,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdclxxxi{und denjenigen, die auf die ihnen anvertrauten Güter und ihre Verpflichtung achtgeben,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdclxxxii{und denjenigen, die ihre Gebete einhalten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdclxxxiii{Das sind die Erben,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdclxxxiv{die das Paradies erben werden; ewig werden sie darin bleiben.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdclxxxv{Wir schufen den Menschen ja aus einem Auszug aus Lehm.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdclxxxvi{Hierauf machten Wir ihn zu einem Samentropfen in einem festen Aufenthaltsort.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdclxxxvii{Hierauf schufen Wir den Samentropfen zu einem Anhängsel, dann schufen Wir das Anhängsel zu einem kleinen Klumpen, dann schufen Wir den kleinen Klumpen zu Knochen, dann bekleideten Wir die Knochen mit Fleisch. Hierauf ließen Wir ihn als eine weitere Schöpfung entstehen. Segensreich ist Allah, der beste Schöpfer.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdclxxxviii{Hierauf werdet ihr gewiß nach (all) diesem sterben.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdclxxxix{Hierauf werdet ihr gewiß am Tag der Auferstehung auferweckt werden.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdcxc{Und Wir schufen ja über euch sieben Lagen, und Wir sind der Schöpfung nicht unachtsam.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdcxci{Und Wir lassen vom Himmel Wasser in (bestimmtem) Maß herabkommen und lassen es sich dann in der Erde aufhalten. Und Wir habe'n fürwahr die Macht, es (wieder) wegzunehmen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdcxcii{Dann lassen Wir euch dadurch Gärten mit Palmen und Rebstöcken entstehen, in denen ihr vielerlei Früchte habt und von denen ihr eßt,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdcxciii{und einen Baum, der aus dem Berg Saina herauskommt, der Öl hervorbringt und auch Tunke für diejenigen, die essen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdcxciv{Und (auch) im Vieh habt ihr wahrlich eine Lehre. Wir geben euch von dem, was in ihren Leibern ist, zu trinken. An ihnen habt ihr vielerlei Nutzen, und davon eßt ihr.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdcxcv{Und auf ihnen und auf den Schiffen werdet ihr getragen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdcxcvi{Und Wir sandten bereits Nuh zu seinem Volk. Da sagte er: ``O mein Volk, dient Allah. Keinen Gott habt ihr außer Ihm. Wollt ihr denn nicht gottesfürchtig sein?"\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdcxcvii{Da sagte die aus seinem Volk führende Schar, die ungläubig war: ``Dieser ist nur ein menschliches Wesen wie ihr, das einen Vorzug euch gegenüber haben will. Und wenn Allah gewollt hätte, hätte Er wahrlich Engel herabgesandt. Wir haben so etwas bei unseren Vorvätern nicht gehört.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdcxcviii{Er ist nur ein Mann, der an Besessenheit leidet; so wartet mit ihm für eine gewisse Zeit ab."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdcxcix{Er sagte: ``Mein Herr, hilf mir, wo sie mich der Lüge bezichtigen."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdcc{Da gaben Wir ihm ein: ``Verfertige das Schiff vor Unseren Augen und nach Unserer Eingebung. Wenn dann Unser Befehl kommt und der Ofen brodelt, so führe von jeder (Art) ein Paar und deine Angehörigen hinein, außer gegen wen das Wort vorher ergangen ist. Und sprich Mich nicht an zugunsten derjenigen, die Unrecht getan haben; sie werden ertränkt werden.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdcci{Und wenn du dich nun auf dem Schiff eingerichtet hast, du und diejenigen, die mit dir sind, dann sag: (Alles) Lob gehört Allah, Der uns von dem ungerechten Volk errettet hat!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdccii{Und sag: Mein Herr, gewähre mir einen gesegneten Abstieg', denn Du bist der Beste derjenigen, die Abstieg gewähren."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdcciii{Darin sind wahrlich Zeichen. Wir haben (sie damit) fürwahr geprüft.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdcciv{Hierauf ließen Wir nach ihnen ein anderes Geschlecht entstehen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdccv{Dann schickten Wir zu ihnen einen Gesandten von ihnen: ``Dient Allah! Keinen Gott habt ihr außer Ihm. Wollt ihr denn nicht gottesfürchtig sein?"\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdccvi{Und die führende Schar aus seinem Volk, die ungläubig war und die Begegnung mit dem Jenseits für Lüge erklärte und der Wir im diesseitigen Leben üppigen Wohlstand verliehen hatten, sagte: ``Dieser ist nur ein menschliches Wesen wie ihr, das von dem ißt, was ihr eßt, und von dem trinkt, was ihr trinkt.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdccvii{Wenn ihr einem menschlichen Wesen euresgleichen gehorcht, dann werdet ihr fürwahr Verlierer sein.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdccviii{Verspricht er euch etwa, daß ihr, wenn ihr gestorben und zu Erde und Knochen geworden seid, (wieder) hervorgebracht werdet?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdccix{Weit, weit entfernt ist das, was euch versprochen wird.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdccx{Es gibt nur unser diesseitiges Leben: Wir sterben und wir leben, und wir werden nicht auferweckt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdccxi{Er ist nur ein Mann, der gegen Allah eine Lüge ersonnen hat; und wir wollen ihm nicht glauben."\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdccxii{Er sagte: ``Mein Herr, hilf mir, wo sie mich der Lüge bezichtigen."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdccxiii{Er sagte: ``Über ein kleines werden sie wahrlich Reue empfinden."\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdccxiv{Da ergriff sie zu Recht der Schrei, und so machten Wir sie zu Abschaum. Weg darum mit dem ungerechten Volk!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdccxv{Hierauf ließen Wir nach ihnen andere Geschlechter entstehen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdccxvi{Keine Gemeinschaft kann ihrer Frist vorausgehen, noch sie hinausschieben.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdccxvii{Hierauf sandten Wir Unsere Gesandten, einen nach dem anderen. Jedesmal, wenn zu einer Gemeinschaft ihr Gesandter kam, bezichtigten sie ihn der Lüge. So ließen Wir die einen von ihnen auf die anderen folgen und machten sie zu(m Gegenstand von) Geschichten. Weg darum mit Leuten, die nicht glauben!\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdccxviii{Hierauf sandten Wir Musa und seinen Bruder Harun mit Unseren Zeichen und einer deutlichen Ermächtigung\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdccxix{zu Fir'aun und seiner führenden Schar. Sie aber verhielten sich hochmütig und waren überhebliche Leute.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdccxx{Sie sagten: ``Sollen wir denn zwei menschlichen Wesen unseresgleichen glauben, wo doch ihr Volk in unserem Dienst steht?"\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdccxxi{Sie bezichtigten sie der Lüge, und so gehörten sie zu denen, die vernichtet wurden.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdccxxii{Und Wir gaben bereits Musa die Schrift, auf daß sie rechtgeleitet werden mögen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdccxxiii{Und Wir machten den Sohn Maryams und seine Mutter zu einem Zeichen und gewährten ihnen Zuflucht auf einer Anhöhe mit festem Grund und Quellwasser.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdccxxiv{``O ihr Gesandten, eßt von den guten Dingen und handelt rechtschaffen; gewiß, Ich weiß über das, was ihr tut, Bescheid.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdccxxv{Gewiß, diese ist eure Gemeinschaft, eine einzige Gemeinschaft, und Ich bin euer Herr; so fürchtet Mich!"\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdccxxvi{Aber sie spalteten sich in ihrer Angelegenheit untereinander nach (verschiedenen) Büchern'; und jede Gruppierung ist froh über das, was sie bei sich hat.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdccxxvii{So lasse sie in ihrer Verwirrung für eine gewisse Zeit\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdccxxviii{Meinen sie etwa, daß Wir, wenn Wir sie mit Besitz und Söhnen unterstützen,\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdccxxix{Uns beeilen, ihnen gute Dinge (zu gewähren)? Aber nein! Sie merken nicht.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdccxxx{Gewiß, diejenigen, die aus Furcht vor ihrem Herrn besorgt sind\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdccxxxi{und die an die Zeichen ihres Herrn glauben\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdccxxxii{und die ihrem Herrn nicht(s) beigesellen\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdccxxxiii{und die geben, was sie geben, während ihre Herzen sich (davor) ängstigen, weil sie zu ihrem Herrn zurückkehren werden,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdccxxxiv{diese beeilen sich mit den guten Dingen, und sie werden sie erreichen.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdccxxxv{Wir erlegen keiner Seele mehr auf, als sie zu leisten vermag. Und bei Uns ist ein Buch, das die Wahrheit redet, und es wird ihnen kein Unrecht zugefügt.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdccxxxvi{Aber nein! Ihre Herzen befinden sich in tiefer Achtlosigkeit davor'. Und es gibt bei ihnen (andere) Taten außer diesen, die sie ausführen.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdccxxxvii{Wenn Wir dann diejenigen von ihnen, die üppig leben, mit der Strafe ergreifen, flehen sie sogleich laut um Hilfe.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdccxxxviii{``Fleht nicht um Hilfe heute, denn euch wird von Uns nicht geholfen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmdccxxxix{Meine Zeichen wurden euch bereits verlesen. Aber ihr pflegtet auf euren Fersen kehrtzumachen,\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmdccxl{indem ihr euch davon hochmütig in eurem Nachtgeplauder abwandtet."\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmdccxli{Haben sie denn nicht über das Wort nachgedacht, oder ist zu ihnen gekommen, was nicht zu ihren Vorvätern kam?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmdccxlii{Oder haben sie ihren Gesandten nicht erkannt, so daß sie ihn (nun) verwerfen?\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmdccxliii{Oder sagen sie: ``Er leidet an Besessenheit?" Nein! Vielmehr kam er zu ihnen mit der Wahrheit, aber den meisten von ihnen ist die Wahrheit zuwider.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmdccxliv{Wenn die Wahrheit ihren Neigungen gefolgt wäre, gerieten die Himmel und die Erde und wer in ihnen ist wahrlich ins Verderben. Aber nein! Wir kamen zu ihnen mit ihrer Ermahnung, sie aber wenden sich von ihrer Ermahnung ab.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmdccxlv{Oder verlangst du von ihnen eine Entlohnung? Aber die Entlohnung deines Herrn ist besser, und Er ist der beste Versorger.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmdccxlvi{Und du lädst sie wahrlich zu einem geraden Weg ein.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmdccxlvii{Jene, die nicht an das Jenseits glauben, weichen fürwahr vom Weg ab.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmdccxlviii{Wenn Wir Uns ihrer erbarmten und das Unheil, das auf ihnen ist, hinwegnähmen, würden sie wahrlich in ihrer Auflehnung verharren und umherirren.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmdccxlix{Wir haben sie ja mit der Strafe ergriffen. Aber weder unterwarfen sie sich ihrem Herrn noch flehten sie demütig.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmdccl{Wenn Wir ihnen dann ein Tor zu strenger Strafe öffnen, sind sie darüber sogleich ganz verzweifelt.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmdccli{Er ist es, Der euch Gehör, Augenlicht und Herzen hat entstehen lassen. Wie wenig dankbar ihr seid!\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmdcclii{Und Er ist es, Der euch auf der Erde (an Zahl) hat anwachsen lassen, und zu Ihm werdet ihr versammelt werden.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmdccliii{Und Er ist es, Der lebendig macht und sterben läßt; und auf Ihn geht der Unterschied von Nacht und Tag zurück. Begreift ihr denn nicht?\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmdccliv{Aber nein! Sie sagen das gleiche, was die Früheren gesagt haben.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmdcclv{Sie sagen: ``Wenn wir gestorben und zu Erde und Knochen geworden sind, sollen wir denn dann wirklich auferweckt werden?\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmdcclvi{Dies ist bereits zuvor uns und unseren Vätern versprochen worden. Das sind nur Fabeln der Früheren."\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmdcclvii{Sag: Wem gehört die Erde und wer auf ihr ist, wenn ihr (es) wißt?\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmdcclviii{Sie werden sagen: ``(Alles) gehört Allah." Bedenkt ihr denn nicht?\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmdcclix{Sag: Wer ist der Herr der sieben Himmel und der Herr des gewaltigen Thrones?\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmdcclx{Sie werden sagen: ``(Alles) gehört Allah." Sag: Wollt ihr denn nicht gottesfürchtig sein?\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmdcclxi{Sag: In wessen Hand ist die Herrschaftsgewalt über alles, Der Schutz gewährt und gegen Den kein Schutz gewährt werden kann, wenn ihr wißt?\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmdcclxii{Sie werden sagen: ``(Alles) gehört Allah." Wieso seid ihr denn einem Zauber verfallen?\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmdcclxiii{Nein! Vielmehr sind Wir zu ihnen mit der Wahrheit gekommen, aber sie sind wahrlich Lügner.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmdcclxiv{Allah hat Sich keine Kinder genommen, und keinen Gott gibt es neben Ihm, sonst würde fürwahr jeder Gott das wegnehmen, was er erschaffen hat, und die einen von ihnen würden sich den anderen gegenüber wahrlich überheblich zeigen. Preis sei Allah, (und Erhaben ist Er) über das, was sie (Ihm) zuschreiben,\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmdcclxv{Dem Kenner des Verborgenen und des Offenbaren! Erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmdcclxvi{Sag: Mein Herr, wenn immer Du mir zeigst, was ihnen angedroht wird,\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmdcclxvii{mein Herr, dann stelle mich nicht unter das ungerechte Volk.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmdcclxviii{Wir haben fürwahr die Macht dazu, dir zu zeigen, was Wir ihnen androhen.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmdcclxix{Wehre mit dem, was besser ist, das Böse ab. Wir wissen sehr wohl, was sie da zuschreiben.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmdcclxx{Und sag: Mein Herr, ich nehme Zuflucht bei Dir vor den Aufstachelungen der Satane.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmdcclxxi{Und ich nehme Zuflucht bei Dir davor, mein Herr, daß sie mich aufsuchen.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmdcclxxii{Wenn dann der Tod zu einem von ihnen kommt, sagt er: ``Mein Herr, bringt mich zurück,\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmdcclxxiii{auf daß ich rechtschaffen handele in dem, was ich hinterlassen habe." Keineswegs! Es ist nur ein Wort, das er (so) sagt; hinter ihnen wird ein trennendes Hindernis sein bis zu dem Tag, da sie auferweckt werden.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmdcclxxiv{Wenn dann ins Horn geblasen wird, dann wird es zwischen ihnen keine Verwandt schaft mehr geben an jenem Tag, und sie fragen sich nicht mehr gegenseitig.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmdcclxxv{Wessen Waagschalen schwer sein werden, das sind diejenigen, denen es wohl ergeht.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmdcclxxvi{Wessen Waagschalen aber leicht sein werden, das sind diejenigen, die ihre Seelen verloren haben; in der Hölle werden sie ewig bleiben.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmdcclxxvii{Das Feuer schlägt in ihre Gesichter, und sie werden darin erblaßt sein..\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmdcclxxviii{``Wurden euch nicht Meine Zeichen verlesen, ihr aber pflegtet sie für Lüge zu erklären?"\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmdcclxxix{Sie werden sagen: ``Unser Herr, unser Unglück hat uns besiegt, und wir waren irregehende Leute.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmdcclxxx{Unser Herr, bringe uns aus ihr heraus. Wenn wir rückfällig werden, dann sind wir (wirklich) ungerecht."\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmdcclxxxi{Er sagt: ``Seid darin verstoßen, und sprecht nicht mit Mir.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmdcclxxxii{Gewiß, eine Gruppe von Meinen Dienern pflegte zu sagen: .Unser Herr, wir glauben, so vergib uns und erbarme Dich unser, denn Du bist der Beste der Barmherzigen.'\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmdcclxxxiii{Aber ihr nahmt sie zum Gegenstand des Spottes, bis ihr ihretwegen Meine Ermahnung vergaßt, und ihr pflegtet über sie zu lachen.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmdcclxxxiv{Ich habe ihnen heute, daß sie standhaft waren, damit vergolten, daß sie die Erfolgreichen sind."\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmdcclxxxv{Er wird sagen: ``Wie viele Jahre habt ihr in der Erde verweilt?"\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmdcclxxxvi{Sie werden sagen: ``Verweilt haben wir einen Tag oder den Teil von einem Tag; doch frage diejenigen, die zählen (können)."\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmdcclxxxvii{Er wird sagen: ``Ihr habt nur ein wenig verweilt, wenn ihr doch nur wüßtet!\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmdcclxxxviii{Meint ihr denn, daß Wir euch zum sinnlosen Spiel erschaffen hätten und daß ihr nicht zu Uns zurückgebracht würdet?"\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmdcclxxxix{Doch Erhaben ist Allah, der König, der Wahre. Es gibt keinen Gott außer Ihm, dem Herrn des ehrwürdigen Thrones.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmdccxc{Und wer neben Allah einen anderen Gott anruft, für den er keinen Beweis hat, dessen Abrechnung liegt nur bei seinem Herrn. Gewiß, den Ungläubigen wird es nicht wohl ergehen.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmdccxci{Und sag: Mein Herr, vergib und erbarme Dich, denn Du bist der Beste der Barmherzigen.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmdccxcii{\basmalahde (Dies ist) eine Sura, die Wir (als Offenbarung) hinabgesandt und verpflichtend gemacht und in der Wir klare Zeichen hinabgesandt haben, auf daß ihr bedenken möget.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdccxciii{Eine Frau und ein Mann, die Unzucht begehen, geißelt jeden von ihnen mit hundert Hieben. Laßt euch nicht von Mitleid mit ihnen beiden angesichts (der Rechtsbestimmungen) der Religion Allahs ergreifen, wenn ihr an Allah und den Jüngsten Tag glaubt. Und es soll bei (der Vollstreckung) der Strafe an ihnen ein Teil von den Gläubigen zugegen sein.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdccxciv{Ein Unzuchttreiber heiratet keine andere als eine Frau, die Unzucht begeht oder eine Götzendienerin. Und eine Unzuchttreiberin heiratet kein anderer als ein Mann, der Unzucht begeht oder ein Götzendiener. Den Gläubigen ist dies verboten.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdccxcv{Diejenigen, die den ehrbaren Ehefrauen (Untreue) vorwerfen und hierauf nicht vier Zeugen beibringen, die geißelt mit achtzig Hieben und nehmt von ihnen niemals mehr eine Zeugenaussage an - das sind die (wahren) Frevler -,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdccxcvi{außer denjenigen, die nach alledem bereuen und rechtschaffen werden, denn Allah ist Allvergebend und Barmherzig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdccxcvii{Für diejenigen, die ihren Gattinnen (Untreue) vorwerfen, aber keine Zeugen haben außer sich selbst, besteht die Zeugenaussage eines (solchen) von ihnen darin, daß er viermal bei Allah bezeugt, er gehöre wahrlich zu denen, die die Wahrheit sagen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdccxcviii{und zum fünften Mal (bezeugt), der Fluch Allahs komme auf ihn, wenn er zu den Lügnern gehören sollte.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdccxcix{Und es wehrt von ihr die Strafe ab, daß sie viermal bei Allah bezeugt, er gehöre wahrlich zu den Lügnern,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdccc{und das fünfte Mal (bezeugt sie), der Zorn Allahs komme über sie, wenn er zu denjenigen gehören sollte, die die Wahrheit sagen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdccci{Und ohne die Huld Allahs gegen euch und Seine Barmherzigkeit, und daß Allah Reue-Annehmend und Allweise wäre,...\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdcccii{Diejenigen, die die ungeheuerliche Lüge vorgebracht haben, sind eine (gewisse) Schar von euch. Meint nicht, es sei schlecht für euch; nein! Vielmehr ist es gut für euch. Jedermann von ihnen wird zuteil, was er an Sünde erworben hat. Und für denjenigen unter ihnen, der den Hauptanteil daran auf sich genommen hat, wird es gewaltige Strafe geben.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdccciii{Hätten doch, als ihr es hörtet, die gläubigen Männer und Frauen eine gute Meinung voneinander gehabt und gesagt: ``Das ist deutlich eine ungeheuerliche Lüge!"\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdccciv{Hätten sie doch darüber vier Zeugen beigebracht! Da sie aber die Zeugen nicht beigebracht haben, so sind diese bei Allah die Lügner.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdcccv{Und ohne Allahs Huld gegen euch und Seine Barmherzigkeit im Diesseits und Jenseits würde euch für das, worin ihr euch (ausgiebig) ausgelassen habt, gewaltige Strafe widerfahren,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdcccvi{als ihr es mit euren Zungen aufgegriffen und mit euren Mündern das gesagt habt, wovon ihr kein Wissen hattet, und es für eine leichte Sache gehalten habt, während es bei Allah eine ungeheuerliche Sache ist.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdcccvii{Und hättet ihr doch, als ihr es hörtet, gesagt: ``Es steht uns nicht zu, darüber zu sprechen. Preis sei Dir! Das ist eine gewaltige Verleumdung"!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdcccviii{Allah ermahnt euch, niemals wieder dergleichen zu tun, wenn ihr gläubig seid.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdcccix{Und Allah macht euch die Zeichen klar, und Allah ist Allwissend und Allweise.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdcccx{Diejenigen, die es lieben, daß sich das Abscheuliche unter denjenigen, die glauben, verbreitet, für sie wird es schmerzhafte Strafe geben im Diesseits und Jenseits. Allah weiß, ihr aber wißt nicht.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdcccxi{Und ohne die Huld Allahs gegen euch und Seine Barmherzigkeit, und daß Allah nicht Gnädig und Barmherzig wäre ...\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdcccxii{O die ihr glaubt, folgt nicht den Fußstapfen des Satans, denn wer den Fußstapfen des Satans folgt, der gebietet das Schändliche und Verwerfliche. Und ohne die Huld Allahs gegen euch und Seine Barmherzigkeit würde niemand von euch jemals lauter sein. Allah aber läutert, wen Er will. Und Allah ist Allhörend und Allwissend.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdcccxiii{Und es sollen diejenigen von euch, die Überfluß und Wohlstand besitzen, nicht schwören, sie würden den Verwandten, den Armen und denjenigen, die auf Allahs Weg ausgewandert sind, nichts mehr geben, sondern sie sollen verzeihen und nachsichtig sein. Liebt ihr es (selbst) nicht, daß Allah euch vergibt? Allah ist Allvergebend und Barmherzig.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdcccxiv{Gewiß, diejenigen, die den ehrbaren, unachtsamen gläubigen Frauen (Untreue) vorwerfen, sind verflucht im Diesseits und Jenseits. Für sie wird es gewaltige Strafe geben,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdcccxv{am Tag, da ihre Zungen und ihre Hände und ihre Füße gegen sie Zeugnis ablegen werden über das, was sie zu tun pflegten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdcccxvi{An jenem Tag wird Allah ihr wahres Gericht(surteil) in vollem Maß zukommen lassen. Und sie werden erfahren, daß Allah die offenkundige Wahrheit ist.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdcccxvii{Schlechte Frauen gehören zu schlechten Männern, und schlechte Männer gehören zu schlechten Frauen. Gute Frauen gehören zu guten Männern, und gute Männer gehören zu guten Frauen. Freigesprochen sind diese von dem, was man (über sie) redet. Für sie wird es Vergebung und ehrenvolle Versorgung geben.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdcccxviii{O die ihr glaubt, betretet nicht andere Häuser, die nicht eure (eigenen) Häuser sind, bis ihr euch bemerkbar gemacht und ihre Bewohner begrüßt habt. Das ist besser für euch, auf daß ihr bedenken möget!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdcccxix{Wenn ihr niemanden darin findet, dann tretet nicht ein, bis man (es) euch erlaubt. Und wenn man zu euch sagt: ``Kehrt um", dann kehrt um. Das ist lauterer für euch. Und Allah weiß über das, was ihr tut, Bescheid.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdcccxx{Es ist für euch keine Sünde, unbewohnte Häuser zu betreten, in denen sich ein Nießbrauch für euch befindet. Allah weiß, was ihr offen tut und was ihr heimlich tut.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdcccxxi{Sag zu den gläubigen Männern, sie sollen ihre Blicke senken und ihre Scham hüten. Das ist lauterer für sie. Gewiß, Allah ist Kundig dessen, was sie machen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdcccxxii{Und sag zu den gläubigen Frauen, sie sollen ihre Blicke senken und ihre Scham hüten, ihren Schmuck nicht offen zeigen, außer dem, was (sonst) sichtbar ist. Und sie sollen ihre Kopftücher auf den Brustschlitz ihres Gewandes schlagen und ihren Schmuck nicht offen zeigen, außer ihren Ehegatten, ihren Vätern, den Vätern ihrer Ehegatten, ihren Söhnen, den Söhnen ihrer Ehegatten, ihren Brüdern, den Söhnen ihrer Brüder und den Söhnen ihrer Schwestern, ihren Frauen, denen, die ihre rechte Hand besitzt, den männlichen Gefolgsleuten, die keinen (Geschlechts)trieb (mehr) haben, den Kindern, die auf die Blöße der Frauen (noch) nicht aufmerksam geworden sind. Und sie sollen ihre Füße nicht aneinanderschlagen, damit (nicht) bekannt wird, was sie von ihrem Schmuck verborgen tragen. Wendet euch alle reumütig Allah zu, ihr Gläubigen, auf daß es euch wohl ergehen möge!\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdcccxxiii{Und verheiratet die noch ledigen (Männer und Frauen) unter euch und die Rechtschaffenen von euren Sklaven und euren Sklavinnen. Wenn sie arm sind, wird Allah sie durch Seine Huld reich machen. Allah ist Allumfassend und Allwissend.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdcccxxiv{Diejenigen, die keine (Möglichkeit zum) Heirat(en) finden, sollen keusch bleiben, bis Allah sie durch Seine Huld reich macht. Und denjenigen von denen, die eure rechte Hand besitzt, die einen Freibrief begehren, sollt ihr einen Freibrief ausstellen, wenn ihr an ihnen etwas Gutes wißt. Und gebt ihnen (etwas) vom Besitz Allahs, den Er euch gegeben hat. Und zwingt nicht eure Sklavinnen, wo sie ehrbar sein wollen, zur Hurerei im Trachten nach den Glücksgütern des diesseitigen Le bens. Wenn aber einer sie dazu zwingt, so ist Allah, nachdem sie gezwungen wor den sind, Allvergebend und Barmherzig.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdcccxxv{Wir haben ja zu euch Zeichen hinabgesandt, die (alles) klar machen, und ein Beispiel (aus den Berichten) von denjenigen, die vor euch dahingegangen sind, und eine Ermahnung für die Gottesfürchtigen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdcccxxvi{Allah ist das Licht der Himmel und der Erde. Das Gleichnis seines Lichtes ist das einer Nische, in der eine Lampe ist. Die Lampe ist in einem Glas. Das Glas ist, als wäre es ein funkelnder Stern. Ihr Brennstoff kommt von einem gesegneten Baum, einem Ölbaum, weder östlich noch westlich, dessen Öl beinahe schon Helligkeit verbreitete, auch wenn das Feuer es nicht berührte. Licht über Licht. Allah führt zu Seinem Licht, wen Er will. Allah prägt den Menschen die Gleichnisse, und Allah weiß über alles Bescheid.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdcccxxvii{Solche Lampen gibt es) in Häusern, für die Allah erlaubt hat, daß sie errichtet werden und daß darin Sein Name genannt wird. Ihn preisen darin, am Morgen und am Abend,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdcccxxviii{Männer, die weder Handel noch Kaufgeschäft ablenken vom Gedenken Allahs, von der Verrichtung des Gebets und der Entrichtung der Abgabe', die einen Tag fürch ten, an dem Herzen und Augenlicht umgekehrt werden,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdcccxxix{damit Allah ihnen das Beste von dem vergelte, was sie getan haben, und ihnen von Seiner Huld noch mehr gebe. Und Allah versorgt, wen Er will, ohne zu rechnen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdcccxxx{Die Werke derjenigen aber, die ungläubig sind, sind wie eine Luftspiegelung in einer Ebene, die der Durstige für Wasser hält. Wenn er dann dorthin kommt, findet er, daß es nichts ist; aber er findet Allah da, Der ihm dann seine Abrechnung in vollem Maß zukommen läßt. Allah ist schnell im Abrechnen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdcccxxxi{Oder (sie sind) wie Finsternisse in einem abgrundtiefen Meer, das von Wogen überdeckt ist, über denen (nochmals) Wogen sind, über denen (wiederum) Wolken sind: Finsternisse, eine über der anderen. Wenn er seine Hand ausstreckt, kann er sie kaum sehen. Und wem Allah kein Licht schafft, für den gibt es kein Licht.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdcccxxxii{Siehst du nicht, daß (alle) Allah preisen, die in den Himmeln und auf der Erde sind, und (auch) die Vögel mit ausgebreiteten Flügeln? Jeder kennt ja sein Gebet und sein Preisen. Und Allah weiß Bescheid über das, was sie tun.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdcccxxxiii{Allah gehört die Herrschaft der Himmel und der Erde. Und zu Allah ist der Ausgang.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdcccxxxiv{Siehst du nicht, daß Allah die Wolken sanft bewegt, sie hierauf zusammenfügt und hierauf zu einem Haufen macht? Dann siehst du den (Platz)regen dazwischen herauskommen. Und Er sendet vom Himmel (Wolken)berge herab, mit Hagel darin, dann trifft Er damit, wen Er will, und wendet ihn ab, von wem Er will. Das Aufleuchten Seines Blitzes nimmt beinahe das Augenlicht.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdcccxxxv{Allah läßt Nacht und Tag sich abwechseln. Darin ist wahrlich eine Lehre für diejenigen, die Einsicht besitzen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdcccxxxvi{Und Allah hat jedes Tier aus Wasser erschaffen. So gibt es unter ihnen solche, die auf ihrem Bauch kriechen, und solche, die auf zwei Beinen gehen, und (wieder) solche, die auf Vieren gehen. Allah erschafft, was Er will. Gewiß, Allah hat zu allem die Macht.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdcccxxxvii{Wir haben ja Zeichen hinabgesandt, die (alles) klar machen. Und Allah leitet, wen Er will, zu einem geraden Weg.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdcccxxxviii{Und sie sagen: ``Wir glauben an Allah und an den Gesandten, und wir gehorchen." Hierauf, nach diesem (Wort), kehrt sich eine Gruppe von ihnen ab. Diese da sind nicht die Gläubigen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdcccxxxix{Und wenn sie zu Allah und Seinem Gesandten gerufen werden, damit er zwischen ihnen richte, wendet sich sogleich eine Gruppe von ihnen ab.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdcccxl{Wenn aber das Recht auf ihrer Seite ist, dann kommen sie zu ihm, bereit, sich zu unterwerfen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdcccxli{Ist denn in ihren Herzen Krankheit, oder haben sie etwa Zweifel, oder befürchten sie, daß Allah gegen sie ungerecht sein könnte, und (auch) Sein Gesandter? Nein! Vielmehr sind eben sie die Ungerechten.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdcccxlii{Die Rede der Gläubigen, wenn sie zu Allah und Seinem Gesandten gerufen werden, damit er zwischen ihnen richte, besteht nur darin, daß sie sagen: ``Wir hören und gehorchen." Das sind diejenigen, denen es wohl ergeht.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdcccxliii{Wer Allah und Seinem Gesandten gehorcht, Allah fürchtet und sich vor Ihm hütet, das sind die Erfolgreichen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdcccxliv{Und sie haben bei Allah ihren kräftigsten Eid geschworen, sie würden, wenn du ihnen befiehlst, ganz gewiß hinausziehen. Sag: Schwört nicht, geziemender Gehorsam (ist gewiß besser). Gewiß, Allah ist Kundig dessen, was ihr tut.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdcccxlv{Sag: Gehorcht Allah und gehorcht dem Gesandten. Doch wenn ihr euch abkehrt, dann obliegt ihm nur das, was ihm auferlegt ist, und euch obliegt, was euch auferlegt ist. Wenn ihr ihm (aber) gehorcht, seid ihr rechtgeleitet. Und dem Gesandten obliegt nur die deutliche Übermittlung (der Botschaft).\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdcccxlvi{Allah hat denjenigen von euch, die glauben und rechtschaffene Werke tun, versprochen, daß Er sie ganz gewiß als Statthalter auf der Erde einsetzen wird, so wie Er diejenigen, die vor ihnen waren, als Statthalter einsetzte, daß Er für sie ihrer Religion, der Er für sie zugestimmt hat, ganz gewiß eine feste Stellung verleihen wird, und daß Er ihnen nach ihrer Angst (, in der sie gelebt haben,) statt dessen ganz gewiß Sicherheit gewähren wird. Sie dienen Mir und gesellen Mir nichts bei. Wer aber danach ungläubig ist, jene sind die (wahren) Frevler.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdcccxlvii{Und verrichtet das Gebet und entrichtet die Abgabe und gehorcht dem Gesandten, auf daß ihr Erbarmen finden möget!\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdcccxlviii{Meine ja nicht, daß diejenigen, die ungläubig sind, sich (Allah) auf der Erde entziehen können. Ihr Zufluchtsort wird das (Höllen)feuer sein - ein wahrlich schlimmer Ausgang!\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdcccxlix{O die ihr glaubt, diejenigen, die eure rechte Hand (an Sklaven) besitzt, und diejenigen von euch, die noch nicht die Geschlechtsreife erreicht haben, sollen euch zu drei Zeiten um Erlaubnis bitten: vor dem Gebet (in) der Morgendäm merung, wenn ihr zur Zeit der Mittagshitze eure Gewänder ablegt, und nach dem Abendgebet. Das sind drei (Zeiten, in denen die) Blößen von euch (sichtbar sein könnten). Es ist außerhalb dieser (Zeiten) weder für euch noch für sie eine Sünde. Sie gehen oft unter euch umher, und das tut ihr untereinander. So macht Allah euch die Zeichen klar. Und Allah ist Allwissend und Allweise.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdcccl{Und wenn die Kinder unter euch die Geschlechtsreife erreicht haben, dann sollen sie um Erlaubnis bitten, wie diejenigen um Erlaubnis gebeten haben, die vor ihnen waren. So macht Allah euch Seine Zeichen klar. Und Allah ist Allwissend und All weise.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdcccli{Und für diejenigen unter den Frauen, die sich zur Ruhe gesetzt haben und nicht mehr zu heiraten hoffen, ist es keine Sünde, wenn sie ihre Gewänder ablegen, ohne jedoch ihren Schmuck zur Schau zu stellen. Doch sich (dessen) zu enthalten, ist besser für sie. Und Allah ist Allhörend und Allwissend.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdccclii{Es gibt für den Blinden keinen Grund zur Bedrängnis, es gibt für den Hinkenden keinen Grund zur Bedrängnis, es gibt für den Kranken keinen Grund zur Bedrängnis, sowie auch für euch nicht, in euren (eigenen) Häusern zu essen oder in den Häusern eurer Väter, den Häusern eurer Mütter, den Häusern eurer Brüder, den Häusern eurer Schwestern, den Häusern eurer Onkel väterlicherseits, den Häusern eurer Tanten väterlicherseits, den Häusern eurer Onkel mütterlicherseits, den Häusern eurer Tanten mütterlicherseits, (in einem Haus,) dessen Schlüssel ihr besitzt, oder (im Haus) eures Freundes. Es ist für euch keine Sünde, gemeinsam oder getrennt zu essen. Wenn ihr nun Häuser betretet, so grüßt einander mit einem Gruß, der von Allah kommt, (mit) einem gesegneten, guten (Gruß). So macht Allah euch die Zeichen klar, auf daß ihr begreifen möget.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdcccliii{Die (wahren) Gläubigen sind ja diejenigen, die an Allah und Seinen Gesandten glauben und, wenn sie bei ihm in einer für alle wichtigen Angelegenheit ver sammelt sind, nicht eher weggehen, als bis sie ihn um Erlaubnis gebeten haben. Diejenigen, die dich um Erlaubnis bitten, das sind diejenigen, die an Allah und Seinen Gesandten glauben. Wenn sie dich nun wegen manch eines eigenen Anliegens um Erlaubnis bitten, so gib Erlaubnis, wem von ihnen du willst. Und bitte Allah um Vergebung für sie, denn gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdcccliv{Erachtet nicht den Aufruf des Gesandten unter euch wie den Aufruf eines von euch an die anderen. Allah kennt ja diejenigen von euch, die sich (unbemerkt) davonstehlen, indem sie sich hinter anderen verstecken. So sollen diejenigen, die Seinem Befehl zuwiderhandeln, sich vorsehen, daß nicht eine Versuchung sie trifft oder schmerzhafte Strafe sie trifft.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdccclv{Sicherlich, Allah gehört, was in den Himmeln und auf der Erde ist. Er weiß ja, in welchem Zustand ihr euch befindet und den Tag, da sie zu Ihm zurückgebracht werden. Dann wird Er ihnen kundtun, was sie getan haben. Und Allah weiß über alles Bescheid.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdccclvi{\basmalahde Segensreich ist Derjenige, Der Seinem Diener die Unterscheidung offenbart hat, damit er für die Weltenbewohner ein Warner sei,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdccclvii{Er, Dem die Herrschaft der Himmel und der Erde gehört, Der Sich kein Kind genommen hat und Der keinen Teilhaber an der Herrschaft hat und alles erschaffen und ihm dabei sein rechtes Maß gegeben hat.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdccclviii{Und sie haben sich außer Ihm Götter genommen, die nichts erschaffen, während sie (selbst) erschaffen werden, und die sich selbst weder Schaden noch Nutzen zu bringen vermögen und die weder über Tod noch über Leben noch über Auferstehung verfugen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdccclix{Diejenigen, die ungläubig sind, sagen: ``Das ist nur eine ungeheuerliche Lüge, die er ersonnen hat und bei der andere Leute ihm geholfen haben." Sie begehen da ja Ungerechtigkeit und Falschaussage.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdccclx{Und sie sagen: ``(Es sind) Fabeln der Früheren, die er sich aufgeschrieben hat'. So werden sie ihm morgens und abends vorgesagt."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdccclxi{Sag: Herabgesandt hat ihn Derjenige, Der das Geheime in den Himmeln und auf der Erde kennt. Gewiß, Er ist Allvergebend und Barmherzig.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdccclxii{Und sie sagen: ``Was ist mit diesem Gesandten, daß er Speise ißt und auf den Märkten umhergeht? O würde doch zu ihm ein Engel herabgesandt, um mit ihm ein Warner zu sein!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdccclxiii{Oder würde doch zu ihm ein Schatz herabgeworfen, oder hätte er doch einen Garten, von dem er essen könnte!" Und die Ungerechten sagen: ``Ihr folgt ja nur ei nem Mann, der einem Zauber verfallen ist."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdccclxiv{Schau, wie sie dir Gleichnisse prägen! Dabei sind sie abgeirrt, und so können sie keinen (Aus)weg mehr (finden).\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdccclxv{Segensreich ist Derjenige, Der dir, wenn Er will, etwas Besseres als dies zuteilen kann: Gärten, durcheilt von Bächen, und Der dir Schlösser zuteilen kann.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdccclxvi{Aber nein! Sie erklären die Stunde für Lüge. Doch haben Wir für diejenigen, die die Stunde für Lüge erklären, eine Feuerglut bereitet.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdccclxvii{Wenn sie sie aus der Ferne sieht, hören sie (bereits) von ihr Grollen und Fauchen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdccclxviii{Und wenn sie da in einen engen Ort zusammengebunden geworfen werden, rufen sie dort nach Vernichtung.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdccclxix{``Ruft heute nicht nach (nur) einer einzigen Vernichtung, sondern ruft nach vielen Vernichtungen!1"\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdccclxx{Sag: Ist das besser oder der Garten der Ewigkeit, der den Gottesfürchtigen versprochen ist? Er ist für sie Lohn und Reiseziel.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdccclxxi{Sie haben darin, was sie wollen, und werden ewig darin bleiben. Das ist ein für deinen Herrn bindendes Versprechen, nach dem gefragt werden wird.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdccclxxii{Und am Tag, da Er sie und das, dem sie anstatt Allahs dienen, versammeln wird, sagt Er: ``Wart ihr es, die diese Meine Diener in die Irre geführt haben, oder sind sie selbst vom Weg abgeirrt?"\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdccclxxiii{Sie werden sagen: ``Preis sei Dir! Es ziemte uns nicht, uns anstatt Deiner (andere) Sc,hutzherren zu nehmen. Aber Du ließt sie und ihre Väter genießen, bis sie die Ermahnung vergaßen und zu einem Volk des Niedergangs wurden."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdccclxxiv{- ``Nun haben sie euch in dem, was ihr sagt, der Lüge bezichtigt. Da könnt ihr weder (etwas) abwenden noch Hilfe erfahren. Und wer von euch Unrecht tut, den lassen Wir große Strafe kosten ``\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdccclxxv{Und Wir sandten vor dir keine Gesandten, die nicht Speise aßen und auf den Märkten umhergingen. Und Wir haben die einen von euch zur Versuchung für die anderen gemacht, (um festzustellen,) ob ihr standhaft seid. Und dein Herr sieht sehr wohl (alles).\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdccclxxvi{Und diejenigen, die nicht die Begegnung mit Uns erwarten, sagen: ``O würden doch die Engel auf uns herabgesandt, oder könnten wir doch unseren Herrn sehen!" Sie verhalten sich ja hochmütig in Bezug auf sich selbst, und sie lehnen sich in großer Mißachtung auf.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdccclxxvii{Am Tag, da sie die Engel sehen, an dem Tag wird es für die Übeltäter keine frohe Botschaft geben, und sie werden sagen: ``Das sei (euch) verwehrt und versperrt!"\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdccclxxviii{Und Wir werden Uns den Werken, die sie getan haben, zuwenden und sie zu verwehtem Staub machen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdccclxxix{Die Insassen des (Paradies)gartens werden an jenem Tag einen besseren Aufenthalt und einen schöneren Ruheplatz haben.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdccclxxx{Und am Tag, da sich der Himmel, Wolken hervorbringend, aufspaltet und die Engel eindrucksvoll herabgesandt werden,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdccclxxxi{an jenem Tag gehört die Herrschaft, die wahre (Herrschaft) dem Allerbanner, und es wird für die Ungläubigen ein schwerer Tag sein.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdccclxxxii{Und an dem Tag wird der Ungerechte sich in die Hände beißen und sagen: ``O hätte ich doch mit dem Gesandten einen Weg eingeschlagen!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdccclxxxiii{O wehe mir! Hätte ich doch nicht den Soundso zum Freund genommen!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdccclxxxiv{Er hat mich ja von der Ermahnung abirren lassen, nachdem sie zu mir gekommen war." Der Satan pflegt den Menschen stets im Stich zu lassen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdccclxxxv{Und der Gesandte sagt: ``O mein Herr, mein Volk mied diesen Qur'an unter Mißachtung."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdccclxxxvi{So haben Wir für jeden Propheten einen Feind aus den Reihen der Übeltäter bestellt. Und dein Herr genügt als Führer und Helfer.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdccclxxxvii{Und diejenigen, die ungläubig sind, sagen: ``O würde ihm doch der Qur'an als Ganzes offenbart!" So (geschieht es aber), damit Wir dein Herz mit ihm festigen. Und Wir haben ihn wohlgeordnet vorgetragen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdccclxxxviii{Und sie bringen dir kein Beispiel, ohne daß Wir dir die Wahrheit und eine bessere Erklärung brächten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdccclxxxix{Diejenigen, die auf ihren Gesichtern zur Hölle versammelt werden, jene sind in der schlechtesten Lage und sind am weitesten vom (rechten) Weg abgeirrt.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdcccxc{Und Wir gaben bereits Musa die Schrift und bestellten ihm seinen Bruder Harun, (die Last) mitzutragen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdcccxci{Da sagten Wir: ``Geht zu dem Volk, das Unsere Zeichen für Lüge erklärt hat." Dann zerstörten Wir sie vollständig.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdcccxcii{Und auch das Volk Nuhs, als sie die Gesandten der Lüge bezichtigten, ließen Wir ertrinken und machten sie für die Menschen zu einem Zeichen. Und Wir haben für die Ungerechten schmerzhafte Strafe bereitet.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdcccxciii{Und (desgleichen auch) die 'Ad, die Tamud, die Leute von ar-Rass und viele Geschlechter dazwischen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdcccxciv{Ihnen) allen prägten Wir Gleichnisse, und (sie) alle zerstörten Wir vollständig.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdcccxcv{Sie kamen doch an der Stadt vorbei, auf die der Unheilsregen niederging. Haben sie sie denn nicht gesehen? Nein! Vielmehr erwarten sie keine Auferstehung.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdcccxcvi{Und wenn sie dich sehen, machen sie sich über dich nur lustig: ``Ist das derjenige, den Allah als Gesandten geschickt hat?\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdcccxcvii{Beinahe hätte er uns fürwahr von unseren Göttern abirren lassen, wenn wir nicht (beharrlich) an ihnen festgehalten hätten." Sie werden, wenn sie die Strafe sehen, erfahren, wer weiter vom Weg abgeirrt ist.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdcccxcviii{Was meinst du wohl zu einem, der sich seine Neigung zu seinem Gott nimmt? Würdest du denn Sachwalter über ihn sein können?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdcccxcix{Oder meinst du, daß die meisten von ihnen hören oder begreifen? Sie sind doch nur wie das Vieh. Aber nein! Sie irren noch weiter vom Weg ab.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmcm{Siehst du nicht deinen Herrn, wie Er den Schatten lang werden läßt? Wenn Er wollte, würde Er ihn fürwahr stillstehen lassen. Hierauf machen Wir die Sonne zu einem Hinweis auf ihn.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmcmi{Hierauf ziehen Wir ihn leicht zu Uns ein.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmcmii{Und Er ist es, Der euch die Nacht zum Kleid und den Schlaf zum Ausruhen macht; und Er macht den Tag zum Aufstehen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmcmiii{Und Er ist es, Der die Winde als Frohboten Seiner Barmherzigkeit (voraus)sendet. Und Wir lassen vom Himmel reines Wasser herabkommen,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmcmiv{um damit (manch) totes Land wieder lebendig zu machen und um es Vieh und Menschen in großer Zahl, die Wir erschaffen haben, zu trinken zu geben.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcmv{Und Wir verteilen es ja unter ihnen auf verschiedene Weise, damit sie bedenken. Aber die meisten Menschen weisen (alles) zurück außer dem Unglauben.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcmvi{Wenn Wir wollten, würden Wir wahrlich in jeder Stadt einen Warner erstehen lassen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcmvii{So gehorche nicht den Ungläubigen und mühe dich damit gegen sie ab mit großem Einsatz.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcmviii{Und Er ist es, Der den beiden Meeren freien Lauf läßt: Das eine ist süß und erfrischend, das andere salzig und (auf der Zunge) brennend. Und Er hat zwischen ihnen beiden ein trennendes Hindernis und eine verwehrte Absperrung errichtet.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcmix{Und Er ist es, Der aus Wasser menschliche Wesen erschafft und sie dann zu (Bluts)verwandschaft und Schwägerschaft bestimmt; dein Herr ist Allmächtig.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcmx{Aber sie dienen anstatt Allahs, was ihnen weder nützt noch schadet. Und der Ungläubige leistet stets Beistand gegen seinen Herrn.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcmxi{Und Wir haben dich nur als Verkünder froher Botschaft und Warner gesandt.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcmxii{Sag: Ich verlange von euch (für mich selbst) keinen Lohn dafür. Wer aber einen Weg zu seinem Herrn einschlagen will (, bekommt selbst seinen Lohn).\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcmxiii{Und verlasse dich auf den Lebendigen, Der nicht stirbt, und lobpreise Ihn. Er genügt als Kenner der Sünden Seiner Diener,\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcmxiv{Der die Himmel und die Erde und was dazwischen ist, in sechs Tagen erschuf und Sich hierauf über den Thron erhob', (Er ist) der Allerbarmer. So frag einen, der von Ihm Kenntnis hat.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcmxv{Und wenn zu ihnen gesagt wird: ``Werft euch vor dem Allerbarmer nieder" sagen sie: ``Was ist denn der Allerbarmer? Sollen wir uns (einfach) vor dem niederwerfen, was du uns befiehlst?" Und das mehrt ihnen (nur) die Abneigung.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcmxvi{Segensreich ist Derjenige, Der im Himmel Türme gesetzt und darin einen Lichtkörper und einen leuchtenden Mond gesetzt hat!\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcmxvii{Und Er ist es, Der die Nacht und den Tag so gemacht hat, daß sie sich voneinander für jemanden unterscheiden, der bedenken oder Dankbarkeit zeigen will.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcmxviii{Die Diener des Allerbarmers sind diejenigen, die maßvoll auf der Erde umhergehen und die, wenn die Toren sie ansprechen, sagen: ``Frieden!"\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcmxix{Und diejenigen, die die Nacht verbringen, indem sie sich (im Gebet) vor ihrem Herrn niederwerfen und aufrecht stehen.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcmxx{Und diejenigen, die sagen: ``Unser Herr, wende von uns die Strafe der Hölle ab." Ihre Strafe ist ja bedrängend;\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcmxxi{gewiß, sie ist böse als Aufenthaltsort und Bleibe.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcmxxii{Und diejenigen, die, wenn sie ausgeben, weder maßlos noch knauserig sind, sondern den Mittelweg dazwischen (einhalten).\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmcmxxiii{Und diejenigen, die neben Allah keinen anderen Gott anrufen und nicht die Seele töten, die Allah (zu töten) verboten hat, außer aus einem rechtmäßigen Grund, und die keine Unzucht begehen. - Wer das tut, hat die Folge der Sünde zu erleiden;\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmcmxxiv{die Strafe wird ihm am Tag der Auferstehung vervielfacht', und ewig wird er darin in Schmach bleiben,\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmcmxxv{außer demjenigen, der bereut, glaubt und rechtschaffene Werke tut; jenen wird Allah ihre bösen Taten gegen gute eintauschen; und Allah ist stets Allvergebend und Barmherzig.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmcmxxvi{Und wer bereut und rechtschaffen handelt, der wendet sich in wahrhaftiger Reue Allah zu.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmcmxxvii{Und (auch) diejenigen, die keine Falschaussage bezeugen und, wenn sie im Vorbeigehen unbedachte Rede (hören), würdevoll weitergehen.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcmxxviii{Und diejenigen, die, wenn sie mit den Zeichen ihres Herrn ermahnt werden, ihnen gegenüber nicht taub und blind niederfallen .\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmcmxxix{Und diejenigen, die sagen: ``Unser Herr, schenke uns an unseren Gattinnen und unseren Nachkommenschaften Grund zur Freude, und mache uns für die Rechtschaffenen zu einem Vorbild."\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmcmxxx{All) diesen wird mit dem Obergemach vergolten werden, daß sie standhaft waren; und ihnen wird Gruß und Friede" entgegengebracht,\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcmxxxi{ewig darin zu bleiben. Schön ist es als Aufenthaltsort und Bleibe.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcmxxxii{Sag: Mein Herr kümmerte Sich nicht um euch, wenn nicht euer Bittgebet (zu Ihm) wäre. Ihr habt doch (alles) für Lüge erklärt, und so wird (eure Strafe hierfür) un abwendbar sein.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcmxxxiii{\basmalahde Ta-Sin-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcmxxxiv{Dies sind die Zeichen des deutlichen Buches.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcmxxxv{Vielleicht magst du dich noch selbst umbringen aus Gram (darüber), daß sie nicht gläubig sind.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmcmxxxvi{Wenn Wir woll(t)en, könn(t)en Wir vom Himmel ein Zeichen auf sie hinabsenden, so daß sich ihre Nacken dauernd davor unterwerfen (würden).\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcmxxxvii{Keine neuerlich offenbarte Ermahnung kommt zu ihnen vom Allerbarmer, ohne daß sie sich davon abwenden.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcmxxxviii{Sie haben (sie) ja für Lüge erklärt. So werden zu ihnen die Nachrichten kommen von dem, worüber sie sich lustig zu machen pflegten.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcmxxxix{Haben sie nicht auf die Erde gesehen, wie viele edle Arten Wir auf ihr haben wachsen lassen?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcmxl{Darin ist wahrlich ein Zeichen, doch sind die meisten von ihnen nicht gläubig.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcmxli{Und dein Herr ist fürwahr der Allmächtige und Barmherzige.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcmxlii{Als dein Herr Musa zurief: ``Begib dich zum ungerechten Volk,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcmxliii{zum Volk Fir'auns, ob sie nicht gottesfürchtig sein wollen."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmcmxliv{Er sagte: ``Mein Herr, ich fürchte, daß sie mich der Lüge bezichtigen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcmxlv{Auch ist meine Brust beklommen, und meine Zunge ist nicht gelöst. Darum entsende (auch) Harun.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcmxlvi{Und sie haben gegen mich eine Sünde geltend zu machen; so fürchte ich, daß sie mich töten."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmcmxlvii{Er sagte: ``Keineswegs! So geht denn beide hin mit Unseren Zeichen. Wir sind mit euch und hören zu.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmcmxlviii{Begebt euch zu Fir'aun und sagt: ,Wir sind der Gesandte des Herrn der Weltenbewohner:\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmcmxlix{Lasse die Kinder Isra'ils mit uns gehen'."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmcml{Er (Fir'aun) sagte: ``Haben wir dich nicht als kleines Kind unter uns aufgezogen, und hast du dich nicht (viele) Jahre deines Lebens unter uns aufgehalten?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmcmli{Und du hast deine Tat, die du (damals) getan hast, verübt und gehörst zu den Undankbaren."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmcmlii{Er sagte: ``Ich habe sie da(mals) verübt, als ich (noch) zu den Irregehenden gehörte.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmcmliii{Und so floh ich vor euch, als ich Angst vor euch bekommen hatte. (Doch) dann schenkte mir mein Herr Urteil(skraft) und machte mich zu einem der Gesandten.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmcmliv{Ist das (etwa) eine Gunst, die du mir vorhältst, daß du die Kinder lsra'ils geknechtet hast?"\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmcmlv{Fir'aun sagte: ``Was ist denn der Herr der Weltenbewohner?"\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmcmlvi{Er (Musa) sagte: ``Der Herr der Himmel und der Erde und dessen, was dazwischen ist, wenn ihr überzeugt seid."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmcmlvii{Er (Fir'aun) sagte zu denjenigen in seiner Umgebung: ``Hört ihr nicht zu?"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmcmlviii{Er (Musa) sagte: ``(Er ist) euer Herr und der Herr eurer Vorväter."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmcmlix{Er (Fir'aun) sagte: ``Euer Gesandter, der zu euch gesandt wurde, ist fürwahr besessen."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmcmlx{Er (Musa) sagte: ``(Er ist) der Herr des Ostens und des Westens und dessen, was dazwischen ist, wenn ihr (nur) begreifen würdet."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmcmlxi{Er (Fir'aun) sagte: ``Wenn du dir einen anderen Gott als mich nimmst, werde ich dich ganz gewiß zu einem der Gefangenen machen."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmcmlxii{Er (Musa) sagte: `` Etwa auch, wenn ich dir etwas Deutliches bringe?"\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmcmlxiii{Er (Fir'aun) sagte: ``Dann bringe es her, wenn du zu den Wahrhaftigen gehörst."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmcmlxiv{Da warf er seinen Stock hin, und sogleich war er eine deutliche Schlange.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmcmlxv{Und er zog seine Hand heraus, da war sie weiß für die Betrachter.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmcmlxvi{Er (Fir'aun) sagte zu der führenden Schar in seiner Umgebung: ``Dieser ist fürwahr ein kenntnisreicher Zauberer,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmcmlxvii{der euch mit seiner Zauberei aus eurem Land vertreiben will. Was befehlt ihr nun?"\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmcmlxviii{Sie sagten: ``Stelle ihn und seinen Bruder zurück und schicke in die Städte (Boten), die (die Leute) versammeln,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmcmlxix{damit sie dir jeden kenntnisreichen Zauberer herbringen."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmcmlxx{So wurden die Zauberer auf die festgesetzte Zeit eines (wohl)bekannten Tages versammelt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmcmlxxi{Und es wurde zu den Menschen gesagt: ``Werdet (auch) ihr euch nun wohl versammeln?\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmcmlxxii{Vielleicht werden wir den Zauberern folgen, wenn sie es sind, die siegen."\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmcmlxxiii{Als die Zauberer kamen, sagten sie zu Fir'aun: ``Wir bekommen doch sicher einen Lohn, wenn wir es sind, die siegen?"\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmcmlxxiv{Er sagte: ``Ja. Und ihr werdet dann also fürwahr zu den Nahegestellten gehören."\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmcmlxxv{Musa sagte zu ihnen: ``Werft hin, was ihr zu werfen habt!"\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmcmlxxvi{Da warfen sie ihre Stricke und Stöcke hin und sagten: ``Bei der Macht Fir'auns, wir werden ganz gewiß Sieger sein."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmcmlxxvii{So warf Musa seinen Stock hin, und da verschlang er sogleich, was sie vortäuschten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmcmlxxviii{Da warfen sich die Zauberer ehrerbietig nieder.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmcmlxxix{Sie sagten: ``Wir glauben an den Herrn der Weltenbewohner,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmcmlxxx{den Herrn von Musa und Harun."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmcmlxxxi{Er (Fir'aun) sagte: ``Ihr glaubt ihm, bevor ich es euch erlaube? Er ist wahrlich euer Ältester, der euch die Zauberei gelehrt hat. Ihr werdet (es) wahrlich erfahren. Ganz gewiß werde ich eure Hände und eure Füße wechselseitig abhacken und euch al lesamt ganz gewiß kreuzigen (lassen)."\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcmlxxxii{Sie sagten: ``Kein Schaden! Wir werden ganz gewiß zu unserem Herrn zurückkehren.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcmlxxxiii{Wir erhoffen ja, daß unser Herr uns unsere Verfehlungen vergebe dafür, daß wir die ersten (der) Gläubigen sind."\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcmlxxxiv{Und Wir gaben Musa (als Offenbarung) ein: ``Zieh bei Nacht mit Meinen Dienern fort; ihr werdet ja verfolgt werden."\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcmlxxxv{Da sandte Fir'aun in die Städte (Boten), die (die Leute) versammelten:\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcmlxxxvi{``Diese sind fürwahr (nur) eine kleine Schar;\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcmlxxxvii{dennoch lassen sie uns fürwahr ergrimmen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcmlxxxviii{Aber wir sind alle wahrlich auf der Hut."\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcmlxxxix{So ließen Wir sie fortziehen von Gärten und Quellen,\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcmxc{von Schätzen und von trefflicher Stätte.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcmxci{So war es. Und Wir gaben sie den Kindern lsra'ils zum Erbe.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcmxcii{Dann folgten sie ihnen bei Sonnenaufgang.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcmxciii{Als die beiden Heere einander sahen, sagten die Gefährten Musas: ``Wir werden fürwahr eingeholt."\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcmxciv{Er sagte: ``Keineswegs! Denn mit mir ist mein Herr; Er wird mich leiten."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcmxcv{Da gaben Wir Musa ein: ``Schlag mit deinem Stock auf das Meer."\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcmxcvi{Und Wir ließen die anderen dort nahe herankommen.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcmxcvii{Und Wir retteten Musa und diejenigen, die mit ihm waren, allesamt.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcmxcviii{Hierauf ließen Wir die anderen ertrinken.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcmxcix{Darin ist wahrlich ein Zeichen, doch die meisten von ihnen sind nicht gläubig.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmm{Und dein Herr ist fürwahr der Allmächtige und Barmherzige.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmi{Und verlies ihnen die Kunde von Ibrahim.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmii{Als er zu seinem Vater und seinem Volk sagte: ``Wem dient ihr?"\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmiii{Sie sagten: ``Wir dienen Götzen, und so geben wir uns dauernd ihrer Andacht hin."\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmiv{Er sagte: ``Hören sie euch, wenn ihr sie anruft?\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmv{Oder nützen sie euch, oder schaden?"\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmvi{Sie sagten: ``Nein! Vielmehr fanden wir (bereits) unsere Väter desgleichen tun."\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmvii{Er sagte: ``Was meint ihr wohl zu dem, was ihr zu verehren pflegt,\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmviii{ihr und eure Vorväter?\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmix{Gewiß, sie sind mir (alle) Feinde, außer dem Herrn der Weltenbewohner,\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmx{Der mich erschaffen hat und mich (nun) rechtleitet,\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmxi{und Der mir zu essen und zu trinken gibt\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmxii{und Der, wenn ich krank bin, mich heilt,\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmxiii{und Der mich sterben läßt und hierauf wieder lebendig macht,\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmxiv{und von Dem ich erhoffe, daß Er mir am Tag des Gerichts meine Verfehlung vergeben wird.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmxv{Mein Herr, schenke mir Urteil(skraft), und nimm mich unter die Rechtschaffenen auf\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmxvi{Und verleihe mir einen Ruf an Wahrhaftigkeit unter den späteren (Geschlechtern).\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmxvii{Und mach mich zu einem (der) Erben des Gartens der Wonne.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmxviii{Und vergib meinem Vater, denn er gehört zu den Irregehenden.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmxix{Und stürze mich nicht in Schande am Tag, da sie auferweckt werden,\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmxx{an dem Tag, da weder Besitz noch Söhne (jemandem) nützen,\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmxxi{außer, wer zu Allah mit heilem Herzen kommt."\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmxxii{Und der (Paradies)garten wird an die Gottesfürchtigen nahe herangebracht.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmxxiii{Und zum Erscheinen gebracht wird der Höllenbrand den Verirrten.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmxxiv{Und es wird zu ihnen gesagt: ``Wo ist denn das, dem ihr zu dienen pflegtet\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmxxv{anstatt Allahs? Können sie euch helfen oder sich selbst helfen?"\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmxxvi{Dann werden sie kopfüber hineingestürzt, sie und die Verirrten,\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmmxxvii{und die Heerscharen Iblis allesamt.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmmxxviii{Sie sagen, während sie darin miteinander streiten:\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmmxxix{``Bei Allah, wir befanden uns wahrlich in deutlichem Irrtum,\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmmxxx{als wir euch dem Herrn der Weltenbewohner gleichsetzten.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmmxxxi{Es waren nur die Übeltäter, die uns in die Irre geführt haben.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmmxxxii{So haben wir nun niemanden, der Fürsprache einlegt,\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmmxxxiii{und auch keinen warmherzigen Freund.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmmxxxiv{Hätten wir doch eine (Möglichkeit zur) Wiederholung, damit wir zu den Gläubigen gehörten!"\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmmxxxv{Darin ist wahrlich ein Zeichen, doch sind die meisten von ihnen nicht gläubig.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmmxxxvi{Und dein Herr ist fürwahr der Allmächtige und Barmherzige.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmmxxxvii{Das Volk Nuhs bezichtigte die Gesandten der Lüge.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmmxxxviii{Als ihr Bruder Nuh zu ihnen sagte: ``Wollt ihr nicht gottesfürchtig sein?\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmmxxxix{Gewiß, ich bin für euch ein vertrauenswürdiger Gesandter.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmmxl{So furchtet Allah und gehorcht mir.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmmxli{Ich verlange von euch keinen Lohn dafür. Mein Lohn obliegt nur dem Herrn der Weltenbewohner.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmmxlii{So fürchtet Allah und gehorcht mir."\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmmxliii{Sie sagten: ``Sollen wir dir glauben, wo dir (nur) die Niedrigsten folgen?"\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmmxliv{Er sagte: ``Welches Wissen sollte ich darüber haben, was sie zu tun pflegten?\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmmxlv{Ihre Abrechnung obliegt nur meinem Herrn, wenn ihr nur merken würdet!\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmmxlvi{Und ich werde die Gläubigen nicht abweisen.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmmxlvii{Ich bin nur ein deutlicher Warner."\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmmxlviii{Sie sagten: ``Wenn du nicht aufhörst, o Nuh, wirst du ganz gewiß zu den Gesteinigten gehören."\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmmxlix{Er sagte: ``Mein Herr, gewiß mein Volk hat mich der Lüge bezichtigt.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmml{So triff zwischen mir und ihnen eine klare Entscheidung und errette mich und wer von den Gläubigen mit mir ist."\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmmli{Da retteten Wir ihn und wer mit ihm war im vollbeladenen Schiff.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmmlii{Hierauf ließen Wir alsdann die übrigen ertrinken.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmmliii{Darin ist wahrlich ein Zeichen, doch sind die meisten von ihnen nicht gläubig.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmmliv{Und dein Herr ist fürwahr der Allmächtige und Barmherzige.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmmlv{Die 'Ad bezichtigten die Gesandten der Lüge.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmmlvi{Als ihr Bruder Hud zu ihnen sagte: ``Wollt ihr nicht gottesfürchtig sein?\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmmlvii{Gewiß, ich bin für euch ein vertrauenswürdiger Gesandter.\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmmlviii{So fürchtet Allah und gehorcht mir.\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmmlix{Ich verlange von euch keinen Lohn dafür. Mein Lohn obliegt nur dem Herrn der Weltenbewohner.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmmlx{Baut ihr denn auf jeder Anhöhe ein Wahrzeichen und treibt ein sinnloses Spiel\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmmlxi{und nehmt euch Bauwerke (in der Hoffnung), auf daß ihr ewig leben würdet?\qt@no{129}}%
\qt@newcmd\qurantrans@de@mmmlxii{Und, wenn ihr zupackt, packt ihr gewalttätig zu.\qt@no{130}}%
\qt@newcmd\qurantrans@de@mmmlxiii{So fürchtet Allah und gehorcht mir.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mmmlxiv{Und fürchtet Denjenigen, Der euch unterstützt hat mit dem, was ihr wißt,\qt@no{132}}%
\qt@newcmd\qurantrans@de@mmmlxv{Der) euch unterstützt hat mit Vieh und Söhnen\qt@no{133}}%
\qt@newcmd\qurantrans@de@mmmlxvi{und Gärten und Quellen.\qt@no{134}}%
\qt@newcmd\qurantrans@de@mmmlxvii{Gewiß, ich fürchte für euch die Strafe eines gewaltigen Tages."\qt@no{135}}%
\qt@newcmd\qurantrans@de@mmmlxviii{Sie sagten: ``Gleich ist es in Bezug auf uns, ob du ermahnst oder ob du nicht zu denjenigen gehörst, die ermahnen.\qt@no{136}}%
\qt@newcmd\qurantrans@de@mmmlxix{Das hier ist nur die Sitte der Früheren.\qt@no{137}}%
\qt@newcmd\qurantrans@de@mmmlxx{Und wir werden nicht gestraft werden."\qt@no{138}}%
\qt@newcmd\qurantrans@de@mmmlxxi{So bezichtigten sie ihn der Lüge, und da vernichteten Wir sie. Darin ist wahrlich ein Zeichen, doch sind die meisten von ihnen nicht gläubig.\qt@no{139}}%
\qt@newcmd\qurantrans@de@mmmlxxii{Und dein Herr ist fürwahr der Allmächtige und Barmherzige.\qt@no{140}}%
\qt@newcmd\qurantrans@de@mmmlxxiii{Die Tamud bezichtigten die Gesandten der Lüge.\qt@no{141}}%
\qt@newcmd\qurantrans@de@mmmlxxiv{Als ihr Bruder Salih zu ihnen sagte: ``Wollt ihr nicht gottesfürchtig sein?\qt@no{142}}%
\qt@newcmd\qurantrans@de@mmmlxxv{Gewiß, ich bin für euch ein vertrauenswürdiger Gesandter.\qt@no{143}}%
\qt@newcmd\qurantrans@de@mmmlxxvi{So fürchtet Allah und gehorcht mir.\qt@no{144}}%
\qt@newcmd\qurantrans@de@mmmlxxvii{Ich verlange von euch keinen Lohn dafür. Mein Lohn obliegt nur dem Herrn der Weltenbewohner.\qt@no{145}}%
\qt@newcmd\qurantrans@de@mmmlxxviii{Werdet ihr etwa in Sicherheit belassen in dem, was hier ist,\qt@no{146}}%
\qt@newcmd\qurantrans@de@mmmlxxix{in Gärten und an Quellen,\qt@no{147}}%
\qt@newcmd\qurantrans@de@mmmlxxx{in Getreidefeldern und unter Palmen, deren Blütenscheiden zart sind?\qt@no{148}}%
\qt@newcmd\qurantrans@de@mmmlxxxi{Und werdet ihr (weiter) voller Stolz aus den Bergen Häuser aushauen?\qt@no{149}}%
\qt@newcmd\qurantrans@de@mmmlxxxii{So fürchtet Allah und gehorcht mir.\qt@no{150}}%
\qt@newcmd\qurantrans@de@mmmlxxxiii{Und gehorcht nicht dem Befehl der Maßlosen,\qt@no{151}}%
\qt@newcmd\qurantrans@de@mmmlxxxiv{die auf der Erde Unheil stiften und keine Besserung bringen."\qt@no{152}}%
\qt@newcmd\qurantrans@de@mmmlxxxv{Sie sagten: ``Du gehörst ja nur zu denjenigen, die einem Zauber verfallen sind.\qt@no{153}}%
\qt@newcmd\qurantrans@de@mmmlxxxvi{Du bist nur ein menschliches Wesen wie wir. So bringe doch ein Zeichen her, wenn du zu den Wahrhaftigen gehörst."\qt@no{154}}%
\qt@newcmd\qurantrans@de@mmmlxxxvii{Er sagte: ``Dies ist eine Kamelstute; sie hat eine Trinkzeit, und ihr habt eine Trinkzeit' an einem bestimmten Tag.\qt@no{155}}%
\qt@newcmd\qurantrans@de@mmmlxxxviii{Fügt ihr nichts Böses zu, sonst überkommt euch die Strafe eines gewaltigen Tages."\qt@no{156}}%
\qt@newcmd\qurantrans@de@mmmlxxxix{Aber sie schnitten ihr die Sehnen durch . So wurden sie zu Leuten, die Reue empfinden.\qt@no{157}}%
\qt@newcmd\qurantrans@de@mmmxc{Da ergriff sie die Strafe. Darin ist wahrlich ein Zeichen, doch sind die meisten von ihnen nicht gläubig.\qt@no{158}}%
\qt@newcmd\qurantrans@de@mmmxci{Und dein Herr ist fürwahr der Allmächtige und Barmherzige.\qt@no{159}}%
\qt@newcmd\qurantrans@de@mmmxcii{Das Volk Luts bezichtigte die Gesandten der Lüge.\qt@no{160}}%
\qt@newcmd\qurantrans@de@mmmxciii{Als ihr Bruder Lut zu ihnen sagte: ``Wollt ihr nicht gottesfürchtig sein?\qt@no{161}}%
\qt@newcmd\qurantrans@de@mmmxciv{Gewiß, ich bin für euch ein vertrauenswürdiger Gesandter.\qt@no{162}}%
\qt@newcmd\qurantrans@de@mmmxcv{So fürchtet Allah und gehorcht mir.\qt@no{163}}%
\qt@newcmd\qurantrans@de@mmmxcvi{Ich verlange von euch keinen Lohn dafür. Mein Lohn obliegt nur dem Herrn der Weltenbewohner'.\qt@no{164}}%
\qt@newcmd\qurantrans@de@mmmxcvii{Laßt ihr euch etwa mit Männern unter den Weltenbewohnern ein\qt@no{165}}%
\qt@newcmd\qurantrans@de@mmmxcviii{und laßt, was euer Herr für euch an Gattinnen erschaffen hat, stehen? Aber nein! Ihr seid Leute, die Übertretungen begehen."\qt@no{166}}%
\qt@newcmd\qurantrans@de@mmmxcix{Sie sagten: ``Wenn du nicht aufhörst, o Lut, wirst du ganz gewiß zu den Vertriebenen gehören."\qt@no{167}}%
\qt@newcmd\qurantrans@de@mmmc{Er sagte: ``Ich gehöre gewiß zu denjenigen, die euer Tun verabscheuen.\qt@no{168}}%
\qt@newcmd\qurantrans@de@mmmci{Mein Herr, errette mich und meine Angehörigen von dem, was sie tun."\qt@no{169}}%
\qt@newcmd\qurantrans@de@mmmcii{Da erretteten Wir ihn und seine Angehörigen allesamt,\qt@no{170}}%
\qt@newcmd\qurantrans@de@mmmciii{außer einer alten Frau unter denjenigen, die zurückblieben.\qt@no{171}}%
\qt@newcmd\qurantrans@de@mmmciv{Hierauf vernichteten Wir die anderen.\qt@no{172}}%
\qt@newcmd\qurantrans@de@mmmcv{Und Wir ließen einen Regen auf sie niedergehen; wie böse war der Regen für diejenigen, die gewarnt worden waren!\qt@no{173}}%
\qt@newcmd\qurantrans@de@mmmcvi{Darin ist wahrlich ein Zeichen, doch sind die meisten von ihnen nicht gläubig.\qt@no{174}}%
\qt@newcmd\qurantrans@de@mmmcvii{Und dein Herr ist fürwahr der Allmächtige und Barmherzige.\qt@no{175}}%
\qt@newcmd\qurantrans@de@mmmcviii{Die Bewohner des Dickichts bezichtigte die Gesandten der Lüge.\qt@no{176}}%
\qt@newcmd\qurantrans@de@mmmcix{Als Su'aib zu ihnen sagte: ``Wollt ihr nicht gottesfürchtig sein?\qt@no{177}}%
\qt@newcmd\qurantrans@de@mmmcx{Gewiß, ich bin für euch ein vertrauenswürdiger Gesandter.\qt@no{178}}%
\qt@newcmd\qurantrans@de@mmmcxi{So fürchtet Allah und gehorcht mir.\qt@no{179}}%
\qt@newcmd\qurantrans@de@mmmcxii{Ich verlange von euch keinen Lohn dafür. Mein Lohn obliegt nur dem Herrn der Weltenbewohner.\qt@no{180}}%
\qt@newcmd\qurantrans@de@mmmcxiii{Gebt volles Maß und gehört nicht zu denjenigen, die (anderen) Verlust verursachen.\qt@no{181}}%
\qt@newcmd\qurantrans@de@mmmcxiv{Und wägt mit der richtigen Waage.\qt@no{182}}%
\qt@newcmd\qurantrans@de@mmmcxv{Und schmälert den Menschen nicht ihre Sachen und richtet auf der Erde nicht unheilstiftend Verderben an.\qt@no{183}}%
\qt@newcmd\qurantrans@de@mmmcxvi{Und furchtet Denjenigen, Der euch und die früheren Geschöpfe erschaffen hat."\qt@no{184}}%
\qt@newcmd\qurantrans@de@mmmcxvii{Sie sagten: ``Du gehörst ja nur zu denjenigen, die einem starken Zauber verfallen sind.\qt@no{185}}%
\qt@newcmd\qurantrans@de@mmmcxviii{Du bist nur ein menschliches Wesen wie wir. Wir meinen wahrlich, daß du zu den Lügnern gehörst.\qt@no{186}}%
\qt@newcmd\qurantrans@de@mmmcxix{Lasse doch Stücke vom Himmel auf uns herabfallen, wenn du zu den Wahrhaftigen gehörst."\qt@no{187}}%
\qt@newcmd\qurantrans@de@mmmcxx{Er sagte: ``Mein Herr weiß am besten, was ihr tut."\qt@no{188}}%
\qt@newcmd\qurantrans@de@mmmcxxi{Aber sie bezichtigten ihn der Lüge. Da ergriff sie die Strafe des Tages der schattenspendenden Wolke; gewiß es war die Strafe eines gewaltigen Tages.\qt@no{189}}%
\qt@newcmd\qurantrans@de@mmmcxxii{Darin ist wahrlich ein Zeichen, doch sind die meisten von ihnen nicht gläubig.\qt@no{190}}%
\qt@newcmd\qurantrans@de@mmmcxxiii{Und dein Herr ist fürwahr der Allmächtige und Barmherzige.\qt@no{191}}%
\qt@newcmd\qurantrans@de@mmmcxxiv{Und er ist ganz sicher eine Offenbarung des Herrn der Weltenbewohner;\qt@no{192}}%
\qt@newcmd\qurantrans@de@mmmcxxv{mit dem der vertrauenswürdige Geist herabgekommen ist\qt@no{193}}%
\qt@newcmd\qurantrans@de@mmmcxxvi{auf dein Herz, damit du zu den Überbringern von Warnung gehörst,\qt@no{194}}%
\qt@newcmd\qurantrans@de@mmmcxxvii{in deutlicher arabischer Sprache.\qt@no{195}}%
\qt@newcmd\qurantrans@de@mmmcxxviii{Und er ist wahrlich in den Schriften der Früheren (erwähnt).\qt@no{196}}%
\qt@newcmd\qurantrans@de@mmmcxxix{War es denn kein Zeichen für sie, daß die Gelehrten der Kinder Isra'ils darüber Bescheid wissen?\qt@no{197}}%
\qt@newcmd\qurantrans@de@mmmcxxx{Wenn Wir ihn einem der Nichtaraber offenbart hätten\qt@no{198}}%
\qt@newcmd\qurantrans@de@mmmcxxxi{und er ihn ihnen vorgelesen hätte, hätten sie nicht daran geglaubt.\qt@no{199}}%
\qt@newcmd\qurantrans@de@mmmcxxxii{So haben Wir ihn in die Herzen der Übeltäter einziehen lassen.\qt@no{200}}%
\qt@newcmd\qurantrans@de@mmmcxxxiii{Sie glauben nicht eher daran, bis sie die schmerzhafte Strafe sehen\qt@no{201}}%
\qt@newcmd\qurantrans@de@mmmcxxxiv{und diese plötzlich über sie kommt, ohne daß sie merken,\qt@no{202}}%
\qt@newcmd\qurantrans@de@mmmcxxxv{bis sie dann sagen: ``Wird uns Aufschub gewährt?"\qt@no{203}}%
\qt@newcmd\qurantrans@de@mmmcxxxvi{Wünschen sie denn Unsere Strafe zu beschleunigen?\qt@no{204}}%
\qt@newcmd\qurantrans@de@mmmcxxxvii{Was meinst du wohl? Wenn Wir sie über Jahre hinweg genießen lassen,\qt@no{205}}%
\qt@newcmd\qurantrans@de@mmmcxxxviii{und hierauf über sie kommt, was ihnen stets angedroht wurde,\qt@no{206}}%
\qt@newcmd\qurantrans@de@mmmcxxxix{nicht nützen wird ihnen (dann)', was ihnen an Nießbrauch gewährt wurde.\qt@no{207}}%
\qt@newcmd\qurantrans@de@mmmcxl{Und Wir vernichteten keine Stadt, ohne daß sie Überbringer von Warnungen gehabt hätte,\qt@no{208}}%
\qt@newcmd\qurantrans@de@mmmcxli{dies) als Ermahnung. Und nie sind Wir ungerecht.\qt@no{209}}%
\qt@newcmd\qurantrans@de@mmmcxlii{Nicht die Satane sind mit ihm herabgekommen;\qt@no{210}}%
\qt@newcmd\qurantrans@de@mmmcxliii{es ziemt ihnen nicht, und sie können es (auch) nicht.\qt@no{211}}%
\qt@newcmd\qurantrans@de@mmmcxliv{Sie sind vom Hören fürwahr ausgeschlossen.\qt@no{212}}%
\qt@newcmd\qurantrans@de@mmmcxlv{So rufe neben Allah keinen anderen Gott an, sonst wirst du zu den Gestraften gehören.\qt@no{213}}%
\qt@newcmd\qurantrans@de@mmmcxlvi{Und warne die Nächsten deiner Sippe.\qt@no{214}}%
\qt@newcmd\qurantrans@de@mmmcxlvii{Und senke deinen Flügel für diejenigen von den Gläubigen, die dir folgen.\qt@no{215}}%
\qt@newcmd\qurantrans@de@mmmcxlviii{Wenn sie sich dir widersetzen, dann sag: ``Ich sage mich los von dem, was ihr tut."\qt@no{216}}%
\qt@newcmd\qurantrans@de@mmmcxlix{Und verlasse dich auf den Allmächtigen und Barmherzigen,\qt@no{217}}%
\qt@newcmd\qurantrans@de@mmmcl{Der dich sieht, wenn du aufrecht stehst,\qt@no{218}}%
\qt@newcmd\qurantrans@de@mmmcli{und deine Stellungswechsel unter denjenigen, die sich niederwerfen.\qt@no{219}}%
\qt@newcmd\qurantrans@de@mmmclii{Er ist ja der Allhörende und Allwissende.\qt@no{220}}%
\qt@newcmd\qurantrans@de@mmmcliii{Soll ich euch kundtun, auf wen die Satane herabkommen?\qt@no{221}}%
\qt@newcmd\qurantrans@de@mmmcliv{Sie kommen auf jeden ungeheuerlichen Lügner und Sünder herab.\qt@no{222}}%
\qt@newcmd\qurantrans@de@mmmclv{Sie hören hin, und die meisten von ihnen sind Lügner.\qt@no{223}}%
\qt@newcmd\qurantrans@de@mmmclvi{Und die Dichter - es folgen ihnen die Verirrten.\qt@no{224}}%
\qt@newcmd\qurantrans@de@mmmclvii{Siehst du nicht, daß sie in jedem Tal ziellos umherwandern\qt@no{225}}%
\qt@newcmd\qurantrans@de@mmmclviii{und daß sie sagen, was sie nicht tun?\qt@no{226}}%
\qt@newcmd\qurantrans@de@mmmclix{Außer denjenigen, die glauben, rechtschaffene Werke tun und Allahs häufig gedenken und sich (erst) selbst helfen, nachdem ihnen ja Unrecht zugefügt wurde. Und diejenigen, die Unrecht tun, werden erfahren, was für eine Rückkehr sie haben werden.\qt@no{227}}%
\qt@newcmd\qurantrans@de@mmmclx{\basmalahde Ta-Sin. Dies sind die Zeichen des Qur'ans und eines deutlichen Buches,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmclxi{als Rechtleitung und frohe Botschaft für die Gläubigen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmclxii{die das Gebet verrichten und die Abgabe entrichten, und sie, die sie vom Jenseits überzeugt sind.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmclxiii{Gewiß, denjenigen, die nicht an das Jenseits glauben, haben Wir ihre Taten ausgeschmückt, so daß sie blind umherirren.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmclxiv{Das sind diejenigen, für die es böse Strafe geben wird, und im Jenseits sind sie die größten Verlierer.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmclxv{Dir wird der Qur'an wahrlich von Seiten eines Allweisen und Allwissenden zum Empfang überbracht.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmclxvi{Als Musa zu seinen Angehörigen sagte: ``Ich habe ein Feuer wahrgenommen. Ich werde euch davon eine Nachricht bringen, oder ich bringe euch einen Leuchtkörper, ein brennendes Stück Holz, auf daß ihr euch (daran) wärmen könnt."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmclxvii{Als er dort ankam, wurde ihm zugerufen: ``Gesegnet ist wer im Feuer und wer in dessen Umgebung ist, und Preis sei Allah, dem Herrn der Weltenbewohner!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmclxviii{O Musa, Ich bin es, gewiß, Ich bin Allah, der Allmächtige und Allweise."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmclxix{Und: ``Wirf deinen Stock hin." Doch als er sah, daß er sich hin und her bewegte, als wäre er eine flinke Schlange, kehrte er den Rücken und wandte sich nicht mehr um. ``O Musa, fürchte dich nicht, denn bei Mir brauchen sich die Gesandten nicht zu fürchten.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmclxx{Wer aber Unrecht getan und hierauf nach Bösem Gutes eingetauscht hat, so bin Ich Allvergebend und Barmherzig.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmclxxi{Und stecke deine Hand in deinen Brustschlitz, so kommt sie weiß, jedoch nicht von Übel befallen, heraus. (Dies zählt) zu neun Zeichen, (gerichtet) an Fir'aun und sein Volk. Gewiß, sie sind ein Volk von Frevlern."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmclxxii{Als dann Unsere Zeichen offenkundig sichtbar zu ihnen kamen, sagten sie: ``Das ist deutliche Zauberei."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmclxxiii{Und sie verleugneten sie, obwohl sie selbst davon überzeugt waren, aus Ungerechtigkeit und Überheblichkeit. So schau, wie das Ende der Unheilstifter war.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmclxxiv{Und Wir gaben bereits Dawud und Sulaiman Wissen. Und sie sagten: ``(Alles) Lob gehört Allah, der uns vor vielen Seiner gläubigen Diener bevorzugt hat!"\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmclxxv{Und Sulaiman beerbte Dawud und sagte: ``O ihr Menschen, uns ist die Sprache der Vögel gelehrt worden, und uns wurde von allem gegeben. Das ist wahrlich die deutliche Huld."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmclxxvi{Und versammelt wurden für Sulaiman seine Heerscharen - unter den Ginn, Menschen und Vögeln -, und so wurden sie in Reihen geordnet.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmclxxvii{Als sie dann zum Ameisental kamen, sagte eine Ameise: ``O ihr Ameisen, geht in eure Wohnungen hinein, damit euch ja nicht Sulaiman und seine Heerscharen niederwalzen, ohne daß sie merken."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmclxxviii{Da lächelte er erheitert über ihre Worte und sagte: ``Mein Herr, veranlasse mich, für Deine Gunst zu danken, die Du mir und meinen Eltern erwiesen hast, und rechtschaffen zu handeln, womit Du zufrieden bist. Und lasse mich durch Deine Barmherzigkeit eingehen in die Reihen Deiner rechtschaffenen Diener."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmclxxix{Und er schaute bei den Vögeln nach. Da sagte er: ``Wie kommt es, daß ich den Wiedehopf nicht sehe? Befindet er sich etwa unter den Abwesenden?\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmclxxx{Ich werde ihn ganz gewiß strengstens strafen, oder ich werde ihn ganz gewiß hinrichten (lassen)', es sei denn, er bringt mir fürwahr eine deutliche Ermächtigung."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmclxxxi{Aber er blieb nicht lange aus. Er sagte: ``Ich habe (an Wissen) erfaßt, was du nicht erfaßt hast. Und ich bringe dir aus Saba sichere Kunde.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmclxxxii{Gewiß, ich habe herausgefunden, daß eine Frau über sie herrscht, daß ihr von allem gegeben worden ist und daß sie einen gewaltigen Thron hat.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmclxxxiii{Ich habe herausgefunden, daß sie und ihr Volk sich vor der Sonne niederwerfen, anstatt vor Allah. Und der Satan hat ihnen ihre Taten ausgeschmückt und sie dann vom Weg abgehalten, so daß sie nicht rechtgeleitet sind,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmclxxxiv{dies), damit sie sich nicht vor Allah niederwerfen, Der das Versteckte in den Himmeln und auf der Erde herausbringt und weiß, was ihr verbergt, und was ihr offenlegt.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmclxxxv{Allah - es gibt keinen Gott außer Ihm, dem Herrn des gewaltigen Thrones."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmclxxxvi{Er sagte: ``Wir werden schauen, ob du die Wahrheit sagst oder ob du zu den Lügnern gehörst.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmclxxxvii{Geh mit diesem meinem Schreiben, überbringe es ihnen und kehre dich hierauf von ihnen ab. Dann schau, was sie erwidern."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmclxxxviii{Sie sagte: ``O ihr führende Schar, mir ist ein edles Schreiben zugeworfen worden.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmclxxxix{Gewiß, es ist von Sulaiman, und es lautet: ,Im Namen Allahs, des Allerbarmers, des Barmherzigen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmcxc{Seid mir gegenüber nicht überheblich und kommt als (Allah) Ergebene zu mir'."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmcxci{Sie sagte: ``O ihr führende Schar, gebt mir eure Meinung über meine Angelegenheit bekannt; ich pflege ja keine Angelegenheit zu entscheiden, solange ihr nicht bei mir anwesend seid."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmcxcii{Sie sagten: ``Wir besitzen eine Streitmacht und besitzen eine starke Gewalt, doch es steht dir zu, zu befehlen. So sieh zu, was du befehlen willst."\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmcxciii{Sie sagte: ``Gewiß, wenn Könige eine Stadt betreten, verderben sie sie und machen die Mächtigen ihrer Bewohner zu Erniedrigten, und so handeln sie.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmcxciv{Gewiß, ich werde (Boten) zu ihnen senden mit einem Geschenk und dann abwarten, was (für eine Antwort) die Gesandten zurückbringen."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmcxcv{Als er zu Sulaiman kam, sagte dieser : ``Wollt ihr mich mit Besitz unterstützen? Aber das, was mir Allah gegeben hat, ist besser als das, was Er euch gegeben hat. Nein! Vielmehr seid ihr es, die ihr über euer Geschenk froh seid.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmcxcvi{Kehr zu ihnen zurück. Wir werden ganz gewiß mit Heerscharen über sie kommen, denen sie nichts entgegenzusetzen haben. Und wir werden sie ganz gewiß erniedrigt daraus vertreiben, als Geringgeachtete."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmcxcvii{Er sagte: ``O ihr führende Schar, wer von euch bringt mir ihren Thron, bevor sie als (Allah) Ergebene zu mir kommen?"\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmcxcviii{Ein unüberwindlicher von den Ginn sagte: ``Ich bringe ihn dir, bevor du dich von deiner Stelle erhebst. Und ich bin wahrlich stark (genug) dazu und vertrauenswürdig."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmcxcix{Derjenige, der Wissen aus der Schrift hatte, sagte: ``Ich bringe ihn dir, bevor dein Blick zu dir zurückkehrt." Als er ihn unbeweglich bei sich stehen sah, sagte er: ``Dies ist von der Huld meines Herrn, damit Er mich prüft, ob ich dankbar oder undankbar bin. Wer dankbar ist, der ist nur zu seinem eigenen Vorteil dankbar; wer aber undankbar ist - so ist mein Herr unbedürftig und freigebig."\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmcc{Er sagte: ``Macht ihr ihren Thron unkenntlich, wir wollen schauen, ob sie rechtgeleitet wird oder ob sie zu denjenigen gehört, die nicht rechtgeleitet werden."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmcci{Als sie kam, wurde (zu ihr) gesagt: ``Ist dein Thron so (wie dieser hier)?" Sie sagte: ``Es ist so, als ob er es sei.8" (Sulaiman sagte:) ``Und uns wurde schon davor das Wissen gegeben, und wir waren (Allah) ergeben."\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmccii{Aber abgehalten (vom Glauben) hat sie das, dem sie anstatt Allahs diente, denn sie gehörte zu ungläubigen Leuten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmcciii{Es wurde zu ihr gesagt: ``Tritt in den Prachtbau ein." Als sie ihn sah, hielt sie ihn für ein tiefes Wasser und entblößte ihre Unterschenkel. Er sagte: ``Es ist ein mit Glas ausgelegter Prachtbau." Sie sagte: ``Mein Herr, ich habe mir selbst Unrecht zugefügt, aber ich ergebe mich (nun), zusammen mit Sulaiman, Allah, dem Herrn der Weltenbewohner."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmcciv{Und Wir sandten bereits zu den Tamud ihren Bruder Salih: ``Dient Allah." Da waren es sogleich zwei Gruppen, die miteinander stritten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmccv{Er sagte: ``O mein Volk, warum wünscht ihr, das Schlechte vor dem Guten zu beschleunigen? Würdet ihr doch Allah um Vergebung bitten, auf daß ihr Erbarmen finden möget!"\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmccvi{Sie sagten: ``Wir sehen ein böses Vorzeichen in dir und in denjenigen, die mit dir sind." Er sagte: ``Euer Vorzeichen ist bei Allah. Nein! Vielmehr seid ihr Leute, die der Versuchung ausgesetzt werden."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmccvii{Nun befanden sich in der Stadt neun Mitglieder einer Sippschaft, die auf der Erde Unheil stifteten und keine Besserung brachten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmccviii{Sie sagten: ``Schwört einander bei Allah: Wir werden ganz gewiß ihn und seine Angehörigen nachts überfallen, und hierauf werden wir zu seinem nächsten Verwandten sagen: ,Wir waren bei der Vernichtung seiner Angehörigen nicht zugegen, und wir sagen ganz gewiß die Wahrheit'."\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmccix{Sie schmiedeten Ränke, und Wir schmiedeten Ränke, ohne daß sie merkten.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmccx{Schau nur, wie die Folge ihrer Ränke war: Wir zerstörten sie und ihr Volk allesamt.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmccxi{Dies sind nun ihre Häuser, wüst dafür, daß sie Unrecht taten. Darin ist wahrlich ein Zeichen für Leute, die Bescheid wissen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmccxii{Und Wir retteten diejenigen, die glaubten und gottesfürchtig waren.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmccxiii{Und (Wir sandten) Lut. Als er zu seinem Volk sagte: ``Wollt ihr denn sehend(en Auges) das Abscheuliche begehen?\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmccxiv{Laßt ihr euch denn wahrlich in Begierde mit den Männern ein anstatt mit den Frauen? Aber nein! Ihr seid Leute, die töricht sind."\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmccxv{Doch die Antwort seines Volkes war nur, daß sie sagten: ``Vertreibt die Sippe Luts aus eurer Stadt! Das sind Menschen, die sich rein halten."\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmccxvi{Da retteten Wir ihn und seine Angehörigen, außer seiner Frau; Wir bestimmten, daß sie zu denjenigen gehörte, die zurückblieben.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmccxvii{Und Wir ließen einen Regen auf sie niedergehen; wie böse war der Regen der Ge warnten!\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmccxviii{Sag: (Alles) Lob gehört Allah, und Friede sei auf Seinen Dienern, die Er Sich auserwählt hat! Ist denn Allah besser oder das, was sie (Ihm) beigesellen?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmccxix{Oder (ist besser) Wer die Himmel und die Erde erschaffen hat und für euch vom Himmel Wasser herabkommen läßt, durch das Wir in Entzücken versetzende umfriedete Gärten wachsen lassen, deren Bäume unmöglich ihr wachsen lassen könntet. Gibt es denn einen (anderen) Gott neben Allah? Nein! Vielmehr sind sie Leute, die (Allah andere) gleichsetzen.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmccxx{Oder (ist besser) Wer die Erde zu einem festen Grund gemacht und Flüsse durch sie gemacht und festgegründete Berge (auf) ihr gemacht und zwischen den beiden Meeren eine Schranke gemacht hat? Gibt es denn einen (anderen) Gott neben Allah? Aber nein! Die meisten von ihnen wissen nicht.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmccxxi{Oder (ist besser) Wer den in einer Notlage Befindlichen erhört, wenn er Ihn anruft, und das Böse hinwegnimmt und euch zu Nachfolgern (auf) der Erde macht? Gibt es denn einen (anderen) Gott neben Allah? Wie wenig ihr bedenkt!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmccxxii{Oder (ist besser) Wer euch in den Finsternissen des Festlandes und des Meeres (den rechten Weg) leitet und Wer die Winde als Frohboten Seiner Barmherzigkeit (voraus)sendet? Gibt es denn einen (anderen) Gott neben Allah? Erhaben ist Allah über das, was sie (Ihm) beigesellen.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmccxxiii{Oder (ist besser) Wer die Schöpfung am Anfang macht und sie hierauf wiederholt und Wer euch vom Himmel und von der Erde versorgt? Gibt es denn einen (anderen) Gott neben Allah? Sag: Bringt euren Beweis vor, wenn ihr wahrhaftig seid.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmccxxiv{Sag: Über das Verborgene weiß nicht Bescheid wer in den Himmeln und auf der Erde ist, außer Allah; und sie merken nicht, wann sie auferweckt werden.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmccxxv{Nein! Vielmehr hat ihr Wissen über das Jenseits versagt. Nein! Vielmehr sind sie darüber im Zweifel. Nein! Vielmehr sind sie ihm gegenüber blind.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmccxxvi{Und diejenigen, die ungläubig sind, sagen: ``Wenn wir zu Erde geworden sind, und (auch) unsere Väter, sollen wir denn dann wirklich wieder hervorgebracht werden?\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmccxxvii{Dies ist zuvor bereits uns und unseren Vätern versprochen worden. Das sind nur Fabeln der Früheren."\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmccxxviii{Sag: Reist auf der Erde umher und schaut, wie das Ende der Übeltäter war.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmccxxix{Sei nicht traurig über sie, und sei nicht in Beklommenheit wegen der Ränke, die sie schmieden.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmccxxx{Und sie sagen: ``Wann wird dieses Versprechen eintreten, wenn ihr wahrhaftig seid?"\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmccxxxi{Sag: Vielleicht ist dicht hinter euch einiges von dem, was ihr zu beschleunigen wünscht.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmccxxxii{Dein Herr ist wahrlich voll Huld gegen die Menschen. Aber die meisten von ihnen sind nicht dankbar.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmccxxxiii{Und dein Herr weiß fürwahr, was ihre Brüste verhehlen und was sie offenlegen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmccxxxiv{Und es gibt nichts Verborgenes im Himmel und auf der Erde, das nicht in einem deutlichen Buch (verzeichnet) wäre.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmccxxxv{Gewiß, dieser Qur'an erzählt den Kindern Isra'ils das meiste von dem, worüber sie uneinig sind.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmccxxxvi{Und es ist wahrlich eine Rechtleitung und Barmherzigkeit für die Gläubigen.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmccxxxvii{Gewiß, dein Herr wird zwischen ihnen durch Sein Urteil entscheiden. Und Er ist der Allmächtige und Allwissende.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmccxxxviii{So verlasse dich auf Allah, denn du verfährst nach der offenkundigen Wahrheit.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmccxxxix{Du kannst weder die Toten hören lassen noch die Tauben den Ruf hören lassen, wenn sie den Rücken kehren.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmccxl{Noch kannst du die Blinden aus ihrem Irrtum heraus rechtleiten. Hören lassen kannst du nur, wer an Unsere Zeichen glaubt und die (Uns) somit ergeben sind.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmccxli{Und wenn das Wort über sie fällig wird, bringen Wir ihnen ein Tier aus der Erde hervor, das zu ihnen spricht; (dies), weil die Menschen von Unseren Zeichen nicht überzeugt sind.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmccxlii{Und (denke an) den Tag, da Wir aus jeder Gemeinschaft eine Schar von denjenigen versammeln werden, die Unsere Zeichen für Lüge erklären, und sie dann in Reihen geordnet werden.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmccxliii{Wenn sie dann kommen, sagt Er: ``Habt ihr denn Meine Zeichen für Lüge erklärt und davon kein umfassendes Wissen gehabt, oder was habt ihr denn sonst getan?"\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmccxliv{Und das Wort wird über sie fällig werden dafür, daß sie Unrecht taten, und da werden sie nicht reden.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmccxlv{Sehen sie nicht, daß Wir die Nacht gemacht haben, damit sie in ihr ruhen, und den Tag hell? Darin sind wahrlich Zeichen für Leute, die glauben.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmccxlvi{Und (denke an) den Tag, da ins Horn geblasen wird und da (all) diejenigen erschrecken, die in den Himmeln und die auf der Erde sind, außer wer Allah will. Und alle kommen in Demut zu Ihm.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmccxlvii{Und du siehst die Berge, von denen du meinst, sie seien unbeweglich, während sie wie Wolken vorbeiziehen. - (Es ist) das Werk Allahs, Der alles sehr geschickt gemacht hat. Gewiß, Er ist Kundig dessen, was ihr tut.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmccxlviii{Wer mit einer guten Tat kommt, wird etwas Besseres als sie erhalten. Und sie sind vor (jedem) Schrecken an jenem Tag in Sicherheit'.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmccxlix{Wer aber mit einer schlechten Tat kommt, - diese werden mit ihren Gesichtern (voran) ins (Höllen)feuer gestürzt: ``Wird euch denn etwas anderes vergolten als das, was ihr zu tun pflegtet?"\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmccl{Mir ist nur befohlen worden, dem Herrn dieser Ortschaft zu dienen, Der sie geschützt hat und Dem alles gehört. Und mir ist befohlen worden, einer der (Ihm) Ergebenen zu sein\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmccli{und den Qur'an zu verlesen. Wer sich nun rechtleiten läßt, der ist nur zu seinem eigenen Vorteil rechtgeleitet. Und wenn einer irregeht, dann sag: Ich gehöre ja nur zu den Überbringern von Warnungen.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmcclii{Und sag: (Alles) Lob gehört Allah! Er wird euch Seine Zeichen zeigen, und dann werdet ihr sie erkennen. Allah ist nicht unachtsam dessen, was ihr tut.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmccliii{\basmalahde Ta-Sin-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmccliv{Dies sind die Zeichen des deutlichen Buches.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcclv{Wir verlesen dir von der Kunde über Musa und Fir'aun der Wahrheit entsprechend, für Leute, die glauben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcclvi{Gewiß, Fir'aun zeigte sich überheblich im Land und machte seine Bewohner zu Lagern, von denen er einen Teil unterdrückte, indem er ihre Söhne abschlachtete und (nur) ihre Frauen am Leben ließ. Gewiß, er gehörte zu den Unheilstiftern.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcclvii{Wir aber wollten denjenigen, die im Land unterdrückt wurden, eine Wohltat erweisen und sie zu Vorbildern machen und zu Erben machen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcclviii{ihnen eine feste Stellung im Land verleihen und Fir'aun, Haman und deren Heerscharen durch sie das erfahren lassen, wovor sie sich immer vorzusehen suchten.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcclix{Und Wir gaben der Mutter Musas ein: ``Stille ihn. Und wenn du um ihn fürchtest, dann setze ihn in das Wasser und fürchte dich nicht und sei nicht traurig. Wir werden ihn dir zurückbringen und ihn zu einem der Gesandten machen."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcclx{Da lasen ihn die Angehörigen Fir'auns auf, damit er ihnen zum Feind und (Grund zum) Kummer werde. Gewiß, Fir'aun, Haman und deren Heerscharen pflegten verfehlt zu handeln.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcclxi{Die Frau Fir'auns sagte: ``(Er wird) für mich und dich ein Grund zur Freude sein. Tötet ihn nicht. Vielleicht nützt er uns oder wir nehmen ihn als Kind an." Dabei merkten sie nicht (, was sie taten).\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcclxii{Und das Herz der Mutter Musas wurde leer. Beinahe hätte sie ihn fürwahr offen bekanntgegeben, wenn Wir nicht ihr Herz gestärkt hätten, damit sie zu den Gläubigen gehöre.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcclxiii{Und sie sagte zu seiner Schwester: ``Folge seiner Spur." Sie beobachtete ihn von fern, ohne daß sie merkten.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcclxiv{Nun hatten Wir ihm zuvor die Ammenbrüste verwehrt. Da sagte sie: ``Soll ich euch auf Hausleute hinweisen, die ihn für euch betreuen und ihm aufrichtig zugetan sein würden?"\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcclxv{So brachten Wir ihn zu seiner Mutter zurück, damit sie frohen Mutes und nicht mehr traurig sei und damit sie wissen sollte, daß Allahs Versprechen wahr ist. Aber die meisten von ihnen wissen nicht.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcclxvi{Und als er seine Vollreife und sein Ebenmaß erlangt hatte, gaben Wir ihm Urteil(skraft) und Wissen. So vergelten Wir den Gutes Tuenden.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcclxvii{Und er betrat die Stadt zu einer Zeit, als ihre Bewohner unachtsam waren. Da fand er darin zwei Männer, die miteinander kämpften, der eine war von seinem (eigenen) Lager, der andere von seinen Feinden. Da rief ihn derjenige, der von seinem Lager war, zu Hilfe gegen denjenigen, der von den Feinden war. Dann schlug ihn Musa mit der Faust und brachte ihn so um. Er sagte: ``Das gehört zum Werk des Satans. Gewiß, er ist ein deutlicher Feind, der in die Irre führt."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcclxviii{Er sagte: ``Mein Herr, ich habe mir selbst Unrecht zugefügt; so vergib mir." Da vergab Er ihm, denn Er ist ja der Allvergebende und Barmherzige.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcclxix{Er sagte: ``Mein Herr, darum, daß Du mir Gunst erwiesen hast, werde ich den Übeltätern nicht mehr Beistand leisten."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcclxx{Am Morgen war er in der Stadt furchtsam und hielt (immer wieder) Ausschau. Auf einmal schrie derjenige, der ihn am Tag zuvor um Hilfe gebeten hatte, zu ihm um Beistand. Musa sagte zu ihm: ``Du bist offenkundig einer, der wahrlich (zu Gewalttaten) verleitet."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcclxxi{Als er nun mit Gewalt denjenigen packen wollte, der ihrer beider Feind war, sagte dieser: ``O Musa, willst du denn mich töten, wie du gestern eine (Menschen)seele getötet hast? Du willst ja nur ein Gewalttäter im Land sein, und du willst nicht zu den Heilstiftern gehören."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcclxxii{Und es kam ein Mann vom äußersten Ende der Stadt gelaufen. Er sagte: ``O Musa, die führende Schar berät über dich, um dich zu töten. So geh fort, gewiß, ich gehöre zu denjenigen, die dir guten Rat geben."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcclxxiii{So ging er furchtsam aus ihr fort und hielt (immer wieder) Ausschau. Er sagte: ``Mein Herr, errette mich von dem ungerechten Volk."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcclxxiv{Und als er sich Madyan zuwandte, sagte er: ``Vielleicht wird mein Herr mich den rechten Weg leiten."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcclxxv{Als er nun zum Wasser von Madyan kam, fand er dort eine (ganze) Gemeinschaft von Menschen, die (ihr Vieh) tränkten. Und er fand außer ihnen zwei Frauen, die (ihre Tiere) fernhielten. Er sagte: ``Was ist mit euch beiden?" Sie sagten: ``Wir tränken (unsere Tiere) nicht, bis die Hirten (ihr Vieh) weggetrieben haben. Und unser Vater ist ein hochbetagter Greis."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcclxxvi{Da tränkte er ihnen (ihre Tiere). Hierauf zog er sich zurück in den Schatten und sagte: ``Mein Herr, ich bin dessen bedürftig, was Du auch immer an Gutem zu mir herabsendest."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcclxxvii{Da kam die eine von den beiden zu ihm, indem sie verschämt einherging. Sie sagte: ``Mein Vater ruft dich, um dir den Lohn dafür zu entrichten, daß du uns (die Tiere) getränkt hast." Nachdem er zu ihm gekommen war und ihm die Geschichte berichtet hatte, sagte er: ``Fürchte dich nicht; du bist dem ungerechten Volk entkommen."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcclxxviii{Die eine von den beiden sagte: ``O mein lieber Vater, nimm ihn in Dienst, denn der Beste, den du in Dienst nehmen kannst, ist der Starke und Vertrauenswürdige."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcclxxix{Er sagte: ``Ich will dich mit einer dieser meiner beiden Töchter verheiraten unter der Bedingung, daß du acht Jahre in meinen Dienst trittst. Wenn du sie aber auf zehn vollmachst, so steht es bei dir. Ich will dir keine Härte auferlegen. Du wirst mich, wenn Allah will, als einen der Rechtschaffenen finden."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcclxxx{Er sagte: ``Dies sei zwischen mir und dir (abgemacht). Welche der beiden Fristen ich auch erfülle, so darf es keine Bedrängnis gegen mich geben. Und Allah ist Sachwalter über das, was wir (hier) sagen."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcclxxxi{Als Musa die Frist erfüllt hatte und mit seinen Angehörigen fortzog, nahm er auf der Seite des Berges ein Feuer wahr. Er sagte zu seinen Angehörigen: ``Bleibt hier. Ich habe ein Feuer wahrgenommen; vielleicht kann ich euch davon eine Nachricht oder ein Stück brennendes Holz aus dem Feuer bringen, auf daß ihr euch (daran) wärmen könnt."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmcclxxxii{Als er dort ankam, wurde ihm vom rechten Ufer des Tales im gesegneten Fleck (Land) aus dem Baum zugerufen: ``O Musa, gewiß, Ich bin Allah, der Herr der Weltenbewohner."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmcclxxxiii{Und: ``Wirf deinen Stock hin." Doch als er sah, daß er sich hin und her bewegte, als wäre er eine flinke Schlange, kehrte er den Rücken und wandte sich nicht mehr um. - ``O Musa, komm her und fürchte dich nicht, denn du sollst zu denjenigen gehören, die in Sicherheit sind.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmcclxxxiv{Und) stecke die Hand in deinen Brustschlitz, so kommt sie weiß, jedoch nicht von Übel befallen, heraus. - Und zieh deinen Arm an dich gegen den Schreck. - Dies sind zwei Beweise von deinem Herrn, (gerichtet) an Fir'aun und seine führende Schar. Gewiß, sie sind ein Volk von Frevlern."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmcclxxxv{Er sagte: ``Mein Herr, ich habe eine (Menschen)seele von ihnen getötet, und so fürchte ich, daß sie mich töten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmcclxxxvi{Auch hat mein Bruder Harun eine redegewandtere Zunge. So sende ihn mit mir zur Unterstützung, mich zu bestätigen, denn ich furchte, daß sie mich der Lüge bezichtigen."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmcclxxxvii{Er sagte: ``Wir werden deinen Arm durch deinen Bruder festigen und euch beiden eine Ermächtigung erteilen, so daß sie nicht zu euch gelangen (können). Mit Unseren Zeichen werdet ihr und diejenigen, die euch folgen, Sieger sein."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmcclxxxviii{Als nun Musa mit Unseren Zeichen als klare Beweise kam, sagten sie: ``Das ist nur ersonnene Zauberei. Und wir haben so etwas bei unseren Vorvätern nicht gehört."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmcclxxxix{Musa sagte: ``Mein Herr weiß besser, wer von Ihm her die Rechtleitung bringt und wem die letztendliche Wohnstätte gehören wird. Gewiß, den Ungerechten wird es nicht wohl ergehen."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmccxc{Und Fir'aun sagte: ``O ihr führende Schar, keinen anderen Gott weiß ich für euch als mich (selbst). So entfache mir, o Haman, einen Brand auf Lehm, und mache mir einen Hochbau, auf daß ich zum Gott Musas emporsteige. Ich glaube fürwahr, daß er zu den Lügnern gehört."\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmccxci{Er und seine Heerscharen verhielten sich ohne Recht hochmütig auf der Erde und glaubten, daß sie (dereinst) nicht zu Uns zurückgebracht würden.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmccxcii{Da ergriffen Wir ihn und seine Heerscharen und warfen sie dann in das große Gewässer. So schau, wie das Ende der Ungerechten war.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmccxciii{Und Wir machten sie zu Anführern, die zum (Höllen)feuer einladen. Und am Tag der Auferstehung wird ihnen keine Hilfe zuteil werden.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmccxciv{Und Wir ließen ihnen einen Fluch im Diesseits nachfolgen, und am Tag der Auferstehung werden sie zu den Verabscheuten gehören.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmccxcv{Und Wir gaben ja Musa die Schrift -, nachdem Wir die früheren Geschlechter vernichteten, - als einsichtbringende Zeichen für die Menschen und als Rechtleitung und Barmherzigkeit, auf daß sie bedenken mögen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmccxcvi{Du warst nicht auf der westlichen Seite', als Wir Musa die Angelegenheit auftrugen, und du warst nicht zugegen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmccxcvii{Wir aber ließen Geschlechter entstehen, und es war ihnen ein langes Leben beschieden. Und du hattest dich nicht unter den Leuten von Madyan niederge lassen, um ihnen Unsere Zeichen zu verlesen. Wir aber entsandten immer wieder (Propheten).\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmccxcviii{Und du warst nicht auf der Seite des Berges, als Wir zuriefen. Aber (die Offenbarung ist) als Barmherzigkeit von deinem Herrn (zu dir gekommen), damit du Leute warnst, zu denen noch kein Warner vor dir gekommen ist, auf daß sie be denken mögen.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmccxcix{Hätten Wir dich nicht entsandt), würden sie sagen, wenn sie ein Unglück trifft wegen dessen, was ihre Hände vorausgeschickt haben: ``Unser Herr, hättest Du doch einen Gesandten zu uns gesandt, so daß wir Deinen Zeichen hätten folgen können und zu den Gläubigen'hätten gehören können!"\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmccc{Als nun die Wahrheit von Uns her zu ihnen kam, sagten sie: ``Wäre ihm doch das gleiche gegeben worden, was Musa gegeben wurde!" Haben sie denn nicht das verleugnet, was zuvor Musa gegeben wurde? Sie sagen: ``Zwei Zauberwerke, die einander beistehen." Und sie sagen: ``Wir verleugnen sie alle (beide)."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmccci{Sag: Bringt doch eine Schrift von Allah bei, die eine bessere Rechtleitung enthält als diese beiden, so will ich ihr folgen, wenn ihr wahrhaftig seid.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmcccii{Wenn sie nicht auf dich hören, so wisse, daß sie nur ihren Neigungen folgen. Und wer ist weiter abgeirrt als jemand, der seiner Neigung folgt ohne Rechtleitung von Allah. Gewiß, Allah leitet das ungerechte Volk nicht recht.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmccciii{Und Wir haben ihnen ja das Wort aufeinanderfolgend übermittelt, auf daß sie bedenken mögen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmccciv{Diejenigen, denen Wir vor ihm die Schrift gaben, glauben an ihn.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmcccv{Und wenn er ihnen verlesen wird, sagen sie: ``Wir glauben an ihn. Gewiß, es ist die Wahrheit von unserem Herrn. Wir waren ja schon vor ihm (Allah) ergeben."\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmcccvi{Diese erhalten ihren Lohn zweimal dafür, daß sie standhaft waren. Und sie wehren mit dem Guten das Böse ab und geben von dem aus, womit Wir sie versorgt haben.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmcccvii{Und wenn sie unbedachte Rede hören, wenden sie sich davon ab und sagen: ``Wir haben unsere Taten und ihr habt eure Taten (zu verantworten). Friede sei auf euch! Wir trachten nicht nach (dem Umgang mit) den Toren."\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmcccviii{Gewiß, du kannst nicht rechtleiten, wen du gern (rechtgeleitet sehen) möchtest. Allah aber leitet recht, wen Er will. Er kennt sehr wohl die Rechtgeleiteten.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmcccix{Und sie sagen: ``Wenn wir der Rechtleitung mit dir folgen, werden wir von unserem Land fortgerissen." Haben Wir ihnen denn nicht eine feste Stellung in einem sicheren geschützten Bezirk verliehen, zu dem die Früchte jeder Art zusammengetragen werden als Versorgung von Uns aus? Aber die meisten von ihnen wissen nicht.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmcccx{Und wie (so) manche Stadt, die sich übermütig ihres Lebensunterhaltes erfreute, haben Wir vernichtet! Da sind nun ihre Wohnorte, die nach ihnen nicht mehr bewohnt wurden, bis auf wenige. Und Wir sind es, die Erben (all dessen) geworden sind.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmcccxi{Und dein Herr hätte nimmer die Städte vernichtet, bevor Er nicht in ihrem Mittelpunkt einen Gesandten hätte erstehen lassen, der ihnen Unsere Zeichen verliest. Und Wir hätten die Städte nimmer vernichtet, wenn ihre Bewohner nicht ungerecht gewesen wären.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmcccxii{Und was immer euch gegeben geworden ist, ist Genuß und Schmuck des diesseitigen Lebens. Was aber bei Allah ist, ist besser und beständiger. Begreift ihr denn nicht?\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmcccxiii{Ist denn einer, dem Wir ein schönes Versprechen gegeben haben und der es auch vorfinden wird, einem gleich, den Wir den Genuß des diesseitigen Lebens genießen lassen, der hierauf aber am Tag der Auferstehung zu den Vorgeführten gehören wird?\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmcccxiv{Und am Tag, da Er ihnen zuruft und sagt: ``Wo sind (nun) Meine Teilhaber, die ihr stets angegeben habt?"\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmcccxv{Diejenigen, gegen die sich das Wort bewahrheitet hat, sagen: ``Unser Herr, diese, die wir in Verirrung haben fallen lassen, haben wir (nur) in Verirrung fallen lassen, wie wir selbst in Verirrung gefallen waren. (Von ihnen) sagen wir uns vor Dir los. Nicht uns dienten sie ja."\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmcccxvi{Und es wird gesagt werden: ``Ruft eure Teilhaber." Sie werden sie anrufen, aber sie werden ihnen nicht antworten. Und sie werden die Strafe sehen. Wenn sie sich hätten rechtleiten lassen (, wären sie gerettet gewesen).\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmcccxvii{Und am Tag, da Er ihnen zurufen wird und sagt: ``Was habt ihr den Gesandten geantwortet?"\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmcccxviii{Ihren Blicken werden an jenem Tag die Beweise entzogen sein, und so fragen sie sich nicht mehr gegenseitig.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmcccxix{Was aber jemanden angeht, der bereut und glaubt und rechtschaffen handelt, der wird vielleicht zu denjenigen gehören, denen es wohl ergeht.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmcccxx{Und dein Herr erschafft, was Er will, und wählt. Ihnen aber steht es nicht zu, zu wählen. Preis sei Allah! Erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmcccxxi{Und dein Herr weiß, was ihre Brüste verhehlen und was sie offenlegen.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmcccxxii{Und Er ist Allah. Es gibt keinen Gott außer Ihm. (Alles) Lob gehört Ihm in der diesseitigen und in der jenseitigen Welt! Ihm gehört das Urteil, und zu Ihm werdet ihr zurückgebracht.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmcccxxiii{Sag: Was meint ihr, wenn Allah auf euch die Nacht als Dauerzustand legen würde bis zum Tag der Auferstehung, wer wäre (dann) Gott außer Allah, Der euch Licht bringen würde? Wollt ihr denn nicht hören?\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmcccxxiv{Sag: Was meint ihr, wenn Allah auf euch den Tag als Dauerzustand legen würde bis zum Tag der Auferstehung, wer wäre (dann) Gott außer Allah, Der euch eine Nacht bringen würde, in der ihr ruht? Wollt ihr denn nicht einsichtig sein?\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmcccxxv{In Seiner Barmherzigkeit hat Er euch die Nacht und den Tag gemacht, damit ihr darin ruht und damit ihr (auch) nach etwas von Seiner Huld trachtet, und auf daß ihr dankbar sein möget.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmcccxxvi{Und am Tag, da Er ihnen zurufen wird und dann sagt: ``Wo sind denn Meine Teilhaber, die ihr stets angegeben habt?"\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmcccxxvii{Und Wir nehmen aus jeder Gemeinschaft einen Zeugen heraus, und da sagen Wir: ``Bringt euren Beweis vor." Dann werden sie wissen, daß die Wahrheit Allah gehört. Und entschwunden wird ihnen sein, was sie zu ersinnen pflegten.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmcccxxviii{Gewiß, Qarun gehörte zum Volk Musas, doch unterdrückte er sie. Und Wir gaben ihm solche Schätze, daß deren Schlüssel wahrlich eine schwere Last für eine (ganze) Schar kräftiger Männer gewesen wären. Als sein Volk zu ihm sagte: ``Sei nicht (übermütig) froh, denn Allah liebt nicht diejenigen (Unterdrücker), die (zu übermütig) froh sind,\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmcccxxix{sondern trachte mit dem, was Allah dir gegeben hat, nach der jenseitigen Wohnstätte, vergiß aber auch nicht deinen Anteil am Diesseits. Und tu Gutes, so wie Allah dir Gutes getan hat. Und trachte nicht nach Unheil auf der Erde, denn Allah liebt nicht die Unheilstifter."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmcccxxx{Er sagte: ``Es ist mir nur gegeben worden aufgrund von Wissen, das ich besitze." Wußte er denn nicht, daß Allah bereits vor ihm solche Geschlechter vernichtet hatte, die eine stärkere Kraft als er besaßen und eine größere Ansammlung (an Helfern) hatten? Und die Übeltäter werden nicht nach ihren Sünden befragt.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmcccxxxi{Und so trat er zu seinem Volk in seinem Schmuck heraus. Diejenigen, die das diesseitige Leben begehrten, sagten: ``O hätten doch (auch) wir das gleiche wie das, was Qarun gegeben worden ist! Er hat wahrlich gewaltiges Glück."\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmcccxxxii{Aber diejenigen, denen das Wissen gegeben worden war, sagten: ``Wehe euch! Allahs Belohnung ist besser für jemanden, der glaubt und rechtschaffen handelt." Aber es wird nur den Standhaften dargeboten.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmcccxxxiii{Da ließen Wir mit ihm und mit seiner Wohnstätte die Erde versinken. Und da hatte er weder eine Schar, die ihm vor Allah half, noch konnte er sich selbst helfen.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmcccxxxiv{Und diejenigen, die sich am Tag zuvor (an) seine(r) Stelle (zu sein) gewünscht hatten, begannen zu sagen: ``Ah sieh! Allah gewährt die Versorgung großzügig, wem von Seinen Dienern Er will, und bemißt auch. Wenn Allah uns nicht eine Wohltat erwiesen hätte, hätte Er uns wahrlich (auch) versinken lassen. Ah sieh! Den Ungläubigen wird es nicht wohl ergehen."\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmcccxxxv{Das ist die jenseitige Wohnstätte. Wir bestimmen sie für diejenigen, die weder Überheblichkeit auf der Erde noch Unheil begehren. Und das (gute) Ende gehört den Gottesfürchtigen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmcccxxxvi{Wer mit einer guten Tat kommt, für den wird es etwas Besseres als sie geben. Wer aber mit einer bösen Tat kommt, so wird denjenigen, die böse Taten begehen, nur das vergolten, was sie zu tun pflegten.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmcccxxxvii{Derjenige, Der dir den Qur'an verpflichtend gemacht hat, wird dich sicher zu einem Ort der Wiederkehr zurückkehren lassen. Sag: Mein Herr weiß besser, wer die Rechtleitung bringt und wer sich in deutlichem Irrtum befindet.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmcccxxxviii{Und du hattest nicht erwartet, daß das Buch an dich gerichtet würde. Es ist nur aus Barmherzigkeit von deinem Herrn. Darum leiste den Ungläubigen ja keinen Beistand.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmcccxxxix{Und lasse sie dich nur nicht von den Zeichen Allahs abhalten, nachdem sie nun zu dir herabgesandt worden sind. Und rufe zu deinem Herrn, und gehöre ja nicht zu den Götzendienern.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmcccxl{Und rufe neben Allah keinen anderen Gott an. Es gibt keinen Gott außer Ihm. Alles wird untergehen - außer Seinem Angesicht. Ihm gehört das Urteil, und zu Ihm werdet ihr zurückgebracht.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmcccxli{\basmalahde Alif-Lam-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcccxlii{Meinen die Menschen, daß sie in Ruhe gelassen werden, (nur) weil sie sagen: ``Wir glauben", ohne daß sie geprüft werden?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcccxliii{Wir haben bereits diejenigen vor ihnen geprüft. Allah wird ganz gewiß diejenigen kennen, die die Wahrheit sprechen, und Er wird ganz gewiß die Lügner kennen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcccxliv{Oder meinen diejenigen, die böse Taten begehen, daß sie Uns entkommen (können)? Wie böse ist, was sie urteilen!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcccxlv{Wer die Begegnung mit Allah erwartet-, so wird Allahs Frist sicher kommen. Und Er ist der Allhörende und Allwissende.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcccxlvi{Und wer sich abmüht, der müht sich nur zu seinem eigenen Vorteil ab, denn Allah ist der Weltenbewohner fürwahr unbedürftig.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcccxlvii{Denjenigen, die glauben und rechtschaffene Werke tun, werden Wir ganz gewiß ihre bösen Taten tilgen und ihnen ganz gewiß das Beste vergelten von dem, was sie taten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcccxlviii{Und Wir haben dem Menschen anbefohlen, seine Eltern mit Güte zu behandeln. Wenn sie sich aber darum bemühen, daß Du Mir das beigesellst, wovon du kein Wissen hast, dann gehorche ihnen nicht. Zu Mir wird eure Rückkehr sein, da werde Ich euch kundtun, was ihr zu tun pflegtet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcccxlix{Diejenigen, die glauben und rechtschaffene Werke tun, lassen Wir in die Reihen der Rechtschaffenen eingehen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcccl{Und unter den Menschen gibt es manchen, der sagt: ``Wir glauben an Allah." Wenn ihm aber um Allahs willen Leid zugefügt wird, setzt er die Anfechtung durch die Menschen der Strafe Allahs gleich. Wenn jedoch Hilfe von deinem Herrn kommt, sagen sie ganz gewiß: ``Wir sind ja mit euch gewesen." Weiß Allah denn nicht besser Bescheid über das, was'in den Brüsten der Weltenbewohner steckt?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcccli{Und Allah wird ganz gewiß diejenigen kennen, die glauben, und Er wird ganz gewiß die Heuchler kennen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmccclii{Und diejenigen, die ungläubig sind, sagen zu denjenigen, die glauben: ``Folgt unserem Weg. Laßt uns eure Verfehlungen tragen." Tragen werden sie aber nichts von ihren Verfehlungen, denn sie sind fürwahr Lügner.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcccliii{Ganz gewiß werden sie ihre (eigenen) Lasten tragen, und auch (weitere) Lasten zu ihren (eigenen) Lasten hinzu. Und sie werden am Tag der Auferstehung ganz gewiß befragt werden nach dem, was sie zu ersinnen pflegten.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcccliv{Und Wir sandten bereits Nuh zu seinem Volk. Er verweilte unter ihnen tausend Jahre weniger fünfzig Jahre. Da ergriff sie die Überschwemmung, während sie Unrecht taten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmccclv{Da retteten Wir ihn und die Insassen des Schiffes und machten es zu einem Zeichen für die Weltenbewohner.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmccclvi{Und (Wir sandten) Ibrahim. Als er zu seinem Volk sagte: ``Dient Allah und fürchtet Ihn. Das ist besser für euch, wenn ihr (es) nur wißt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmccclvii{Ihr dient anstatt Allahs nur Götzen und schafft (dabei nur) ungeheuerliche Lüge. Gewiß, diejenigen, denen ihr anstatt Allahs dient, vermögen euch nicht zu versor gen. Sucht darum bei Allah die Versorgung und dient Ihm und dankt Ihm. Zu Ihm werdet ihr zurückgebracht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmccclviii{Und wenn ihr (die Botschaft) für Lüge erklärt, so haben bereits vor euch (andere) Gemeinschaften (sie) für Lüge erklärt. Und dem Gesandten obliegt nur die deutliche Übermittlung (der Botschaft)."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmccclix{Haben sie denn nicht gesehen, wie Allah die Schöpfung am Anfang macht und sie hierauf wiederholt? Gewiß, das ist für Allah ein leichtes.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmccclx{Sag: Reist auf der Erde umher und schaut, wie Er die Schöpfung am Anfang gemacht hat. Hierauf läßt Allah die letzte Schöpfung entstehen. Gewiß, Allah hat zu allem die Macht.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmccclxi{Er straft, wen Er will, und Er erbarmt Sich, wessen Er will. Und zu Ihm werdet ihr zurückgebracht.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmccclxii{Und ihr werdet euch (Ihm) weder auf der Erde noch im Himmel entziehen können, und ihr habt außer Allah weder Schutzherrn noch Helfer.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmccclxiii{Diejenigen, die Allahs Zeichen und die Begegnung mit Ihm ableugnen, diese haben die Hoffnung auf Meine Barmherzigkeit aufgegeben, und für diese wird es schmerzhafte Strafe geben.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmccclxiv{Aber die Antwort seines Volkes war nur, daß sie sagten: ``Tötet ihn oder verbrennt ihn." Da rettete Allah ihn aus dem Feuer. Darin sind wahrlich Zeichen für Leute, die glauben.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmccclxv{Und er sagte: ``Ihr habt euch ja anstatt Allahs nur Götzen genommen aus Freundschaft zueinander im diesseitigen Leben. Aber dereinst, am Tag der Auferstehung, werdet ihr einander verleugnen und einander verfluchen. Euer Zufluchtsort ist das (Höllen)feuer, und ihr werdet keine Helfer haben."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmccclxvi{Da glaubte Lut ihm und sagte: ``Ich werde zu meinem Herrn auswandern. Er ist ja der Allmächtige und Allweise."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmccclxvii{Und Wir schenkten ihm Ishaq und Ya'qub und richteten unter seiner Nachkommenschaft das Prophetentum und die Schrift ein und gaben ihm seinen Lohn im Diesseits. Und im Jenseits gehört er fürwahr zu den Rechtschaffenen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmccclxviii{Und (Wir sandten) Lut. Als er zu seinem Volk sagte: ``Ihr begeht fürwahr das Abscheuliche, wie es vor euch niemand von den Weltenbewohnern getan hat.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmccclxix{Laßt ihr euch denn wahrlich (in Begierde) mit den Männern ein, und schneidet den (natürlichen) Weg ab und begeht in euren Zusammenkünften das Verwerfliche?" Aber die Antwort seines Volkes war nur, daß sie sagten: ``Bringe uns doch die Strafe Allahs her, wenn du zu den Wahrhaftigen gehörst."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmccclxx{Er sagte: ``Mein Herr, hilf mir gegen das Volk, das Unheil stiftet."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmccclxxi{Und als Unsere Gesandten zu Ibrahim mit der frohen Botschaft kamen, sagten sie: ``Wir werden die Bewohner dieser Stadt vernichten, denn ihre Bewohner sind ungerecht."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmccclxxii{Er sagte: ``Aber Lut befindet sich in ihr." Sie sagten: ``Wir wissen sehr wohl, wer sich in ihr befindet. Wir werden ihn ganz gewiß erretten, (ihn) und seine Angehörigen, außer seiner Frau; sie gehört zu denjenigen, die zurückbleiben."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmccclxxiii{Und als nun Unsere Gesandten zu Lut kamen, geriet er ihretwegen in eine böse Lage und war durch ihre Anwesenheit beklommen. Sie sagten: ``Fürchte dich nicht und sei nicht traurig, denn wir werden dich erretten, (dich) und deine Angehörigen, außer deiner Frau; sie gehört zu denjenigen, die zurückbleiben.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmccclxxiv{Wir werden auf die Bewohner dieser Stadt eine unheilvolle Strafe vom Himmel herabkommen lassen dafür, daß sie gefrevelt haben,"\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmccclxxv{Und Wir ließen ja von ihr ein klares Zeichen zurück für Leute, die begreifen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmccclxxvi{Und (Wir sandten) zu Madyan ihren Bruder Su'aib. Da sagte er: ``O mein Volk, dient Allah, erwartet den Jüngsten Tag und richtet auf der Erde nicht unheilstiftend Verderben an."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmccclxxvii{Aber sie bezichtigten ihn der Lüge. Da ergriff sie das Zittern, und am Morgen lagen sie in ihrer Wohnstätte auf den Brüsten da.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmccclxxviii{Und (erwähne auch) die 'Ad und die Tamud. (Ihr Untergang) ist euch ja deutlich geworden an ihren Wohnorten. Der Satan schmückte ihnen ihre Taten aus und hielt sie so vom Weg ab, obwohl sie Einsicht besaßen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmccclxxix{Und (erwähne weiterhin) Qarun und Fir'aun und Haman. Musa kam ja zu ihnen mit den klaren Beweisen. Aber sie verhielten sich hochmütig auf der Erde; doch konnten sie (Uns) nicht zuvorkommen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmccclxxx{Einen jeden ergriffen Wir für seine Sünde; so sandten Wir gegen einige von ihnen einen Sturm von Steinchen, andere ergriff der Schrei, mit anderen ließen Wir die Erde versinken, andere ließen Wir ertrinken. Und nimmer ist es Allah, der ihnen Unrecht getan hat, sondern sie selbst haben sich Unrecht zugefügt.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmccclxxxi{Das Gleichnis derjenigen, die sich anstatt Allahs Schutzherren nehmen, ist das der Spinne, die sich ein Haus genommen hat; das schwächste Haus ist fürwahr das Haus der Spinne, wenn sie (es) nur wüßten!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmccclxxxii{Gewiß, Allah weiß all das, was sie anstatt Seiner anrufen. Und Er ist der Allmächtige und Allweise.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmccclxxxiii{Diese Gleichnisse prägen Wir für die Menschen. Aber nur diejenigen verstehen sie, die Wissen besitzen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmccclxxxiv{Allah hat die Himmel und die Erde in Wahrheit erschaffen. Darin ist wahrlich ein Zeichen für die Gläubigen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmccclxxxv{Verlies, was dir vom Buch (als Offenbarung) eingegeben wird, und verrichte das Gebet. Gewiß, das Gebet hält davon ab, das Schändliche und das Verwerfliche (zu tun). Und das Gedenken Allahs ist wahrlich größer. Und Allah weiß, was ihr macht.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmccclxxxvi{Und streitet mit den Leuten der Schrift nur in bester Weise, außer denjenigen von ihnen, die Unrecht tun. Und sagt: ``Wir glauben an das, was (als Offenbarung) zu uns herabgesandt worden ist und zu euch herabgesandt worden ist; unser Gott und euer Gott ist Einer, und wir sind Ihm ergeben'."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmccclxxxvii{Und so haben Wir das Buch zu dir hinabgesandt. Diejenigen, denen Wir die Schrift gaben, glauben daran. Und auch unter diesen da gibt es manche, die daran glauben. Nur die Ungläubigen verleugnen Unsere Zeichen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmccclxxxviii{Und du hast vordem kein Buch verlesen und es auch nicht mit deiner rechten Hand niedergeschrieben. Sonst würden wahrlich diejenigen zweifeln, die (es) für falsch erklären.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmccclxxxix{Nein! Vielmehr sind es klare Zeichen in den Brüsten derjenigen, denen das Wissen gegeben worden ist. Und nur die Ungerechten verleugnen Unsere Zeichen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmcccxc{Und sie sagen: ``Wenn doch Zeichen von seinem Herrn auf ihn herabgesandt würden!" Sag: Über die Zeichen verfügt (allein) Allah . Und ich bin nur ein deutlicher Warner.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmcccxci{Genügt es ihnen denn nicht, daß Wir das Buch auf dich hinabgesandt haben, das ihnen verlesen wird? Darin ist wahrlich eine Barmherzigkeit und eine Ermahnung für Leute, die glauben.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmcccxcii{Sag: Allah genügt zwischen mir und euch als Zeuge. Er weiß, was in den Himmeln und auf der Erde ist. Und diejenigen, die an das Falsche glauben und Allah verleugnen, das sind die Verlierer.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmcccxciii{Und sie wünschen von dir, die Strafe zu beschleunigen. Wenn es nicht eine festgesetzte Frist gäbe, wäre die Strafe wahrlich zu ihnen gekommen. Aber ganz gewiß wird sie plötzlich über sie kommen, ohne daß sie merken.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmcccxciv{Sie wünschen von dir, die Strafe zu beschleunigen, doch wahrlich, die Hölle umfaßt die Ungläubigen,\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmcccxcv{am Tag, da die Strafe sie von oben und von unterhalb ihrer Füße her überdeckt, und Er sagt': ``Kostet, was ihr zu tun pflegtet."\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmcccxcvi{O Meine Diener, die ihr glaubt, gewiß, Meine Erde ist weit. So dient Mir, ja, allein Mir.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmcccxcvii{Jede Seele wird den Tod kosten. Hierauf werdet ihr zu Uns zurückgebracht.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmcccxcviii{Denjenigen, die glauben und rechtschaffene Werke tun, werden Wir im (Paradies)garten ganz gewiß Obergemächer zuweisen, durcheilt von Bächen; ewig darin zu bleiben. Wie trefflich ist der Lohn derjenigen, die (gut) handeln,\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmcccxcix{die standhaft sind und sich auf ihren Herrn verlassen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmcd{Und wie viele Tiere tragen ihre (eigene) Versorgung nicht herbei! Allah versorgt sie und euch. Und Er ist der Allhörende und Allwissende.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmcdi{Und wenn du sie fragst, wer die Himmel und die Erde erschaffen und die Sonne und den Mond dienstbar gemacht hat, sagen sie ganz gewiß: ``Allah." Wie lassen sie sich also abwendig machen?\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmcdii{Allah gewährt die Versorgung großzügig, wem von Seinen Dienern Er will, und bemißt ihm auch. Gewiß, Allah weiß über alles Bescheid.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmcdiii{Und wenn du sie fragst, wer vom Himmel Wasser herabkommen läßt und dann damit die Erde nach ihrem Tod wieder lebendig macht, sagen sie ganz gewiß: ``Allah." Sag: (Alles) Lob gehört Allah! Aber nein! Die meisten von ihnen be greifen nicht.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmcdiv{Dieses irdische Leben ist nur Zerstreuung und Spiel. Die jenseitige Wohnstätte aber ist wahrlich das eigentliche Leben, wenn sie (es) nur wüßten!\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmcdv{Wenn sie ein Schiff besteigen, rufen sie Allah an, (wobei sie) Ihm gegenüber aufrichtig in der Religion (sind). Wenn Er sie nun ans Land errettet, gesellen sie sogleich (Ihm wieder andere) bei,\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmcdvi{um undankbar zu sein für das, was Wir ihnen gegeben haben, und um zu genießen; sie werden (es noch) erfahren.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmcdvii{Sehen sie denn nicht, daß Wir einen sicheren geschützten Bezirk gemacht haben, während die Menschen in ihrer Umgebung fortgerissen werden? Wollen sie denn an das Falsche glauben und Allahs Gunst verleugnen?\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmcdviii{Und wer ist ungerechter als jemand, der gegen Allah eine Lüge ersinnt oder die Wahrheit, nachdem sie zu ihm gekommen ist, für Lüge erklärt? Ist nicht in der Hölle ein Aufenthaltsort für die Ungläubigen?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmcdix{Diejenigen aber, die sich um Unsertwillen abmühen, werden Wir ganz gewiß Unsere Wege leiten. Und Allah ist wahrlich mit den Gutes Tuenden.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmcdx{\basmalahde Alif-Lam-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcdxi{Die Römer sind besiegt worden\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcdxii{im nächstliegenden Land. Aber sie werden nach ihrer Niederlage (selbst) siegen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcdxiii{in einigen Jahren. Allah gehört der Befehl vorher und nachher. An jenem Tag werden die Gläubigen froh sein\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcdxiv{über Allahs Hilfe. Er hilft, wem Er will, und Er ist der Allmächtige und Barmherzige.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcdxv{Das (ist) Allahs Versprechen. Allah bricht Sein Versprechen nicht, aber die meisten Menschen wissen nicht.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcdxvi{Sie kennen nur das Äußerliche vom diesseitigen Leben, während sie des Jenseits unachtsam sind.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcdxvii{Denken sie denn nicht in ihrem Inneren (darüber) nach? Allah hat die Himmel und die Erde und was dazwischen ist nur in Wahrheit und (auf) eine festgesetzte Frist erschaffen. Aber viele von den Menschen verleugnen fürwahr die Begegnung mit ihrem Herrn.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcdxviii{Sind sie denn nicht auf der Erde umhergereist, so daß sie schauen (konnten), wie das Ende derjenigen war, die vor ihnen waren? Sie hatten eine stärkere Kraft als sie, pflügten und bevölkerten das Land noch mehr, als sie es bevölkerten. Und ihre Gesandten kamen zu ihnen mit den klaren Beweisen. Aber nimmer ist es Allah, der ihnen Unrecht getan hat, sondern sie selbst haben sich Unrecht zugefügt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcdxix{Hierauf war das Ende derjenigen, die Böses taten, das Schlechteste dafür, daß sie Allahs Zeichen für Lüge erklärt hatten und sich über sie lustig zu machen pflegten.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcdxx{Allah macht die Schöpfung am Anfang und wiederholt sie hierauf. Dann werdet ihr zu Ihm zurückgebracht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcdxxi{Und am Tag, da sich die Stunde erhebt, werden die Übeltäter ganz verzweifelt sein.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcdxxii{Sie haben dann an ihren Teilhabern keine Fürsprecher und werden ihre Teilhaber verleugnen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcdxxiii{Am Tag, da sich die Stunde erhebt, an jenem Tag werden sie sich in (Gruppen) teilen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcdxxiv{Was nun diejenigen angeht, die geglaubt und rechtschaffene Werke getan haben, so wird ihnen in einem Lustgarten Freude bereitet.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcdxxv{Was aber diejenigen angeht, die ungläubig waren und Unsere Zeichen und die Begegnung mit dem Jenseits für Lüge erklärt haben, sie werden zur Strafe vorge führt werden.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcdxxvi{Preis sei daher Allah, wenn ihr den Abend erreicht und auch wenn ihr den Morgen erreicht!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcdxxvii{Und Ihm gehört (alles) Lob in den Himmeln und auf der Erde, und abends, und wenn ihr den Mittag erreicht!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcdxxviii{Er bringt das Lebendige aus dem Toten und bringt das Tote aus dem Lebendigen hervor und macht die Erde nach ihrem Tod wieder lebendig. Und so werdet auch ihr hervorgebracht werden.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcdxxix{Es gehört zu Seinen Zeichen, daß Er euch aus Erde erschaffen hat, hierauf wart ihr auf einmal menschliche Wesen, die sich ausbreiten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcdxxx{Und es gehört zu Seinen Zeichen, daß Er euch aus euch selbst Gattinnen erschaffen hat, damit ihr bei ihnen Ruhe findet; und Er hat Zuneigung und Barmherzigkeit zwischen euch gesetzt. Darin sind wahrlich Zeichen für Leute, die nachdenken.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcdxxxi{Und zu Seinen Zeichen gehört die Erschaffung der Himmel und der Erde und (auch) die Verschiedenheit eurer Sprachen und Farben. Darin sind wahrlich Zei chen für die Wissenden.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcdxxxii{Und zu Seinen Zeichen gehört euer Schlaf bei Nacht und Tag und auch euer Trachten nach etwas von Seiner Huld. Darin sind wahrlich Zeichen für Leute, die hören.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcdxxxiii{Und es gehört zu Seinen Zeichen, daß Er euch den Blitz (als Grund) zur Furcht und zum Begehren sehen läßt und vom Himmel Wasser herabkommen läßt und mit ihm dann die Erde nach ihrem Tod wieder lebendig macht. Darin sind wahrlich Zeichen für Leute, die begreifen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcdxxxiv{Und es gehört zu Seinen Zeichen, daß der Himmel und die Erde durch Seinen Befehl bestehen. Wenn Er euch hierauf ein (einziges Mal) ruft, da kommt ihr sogleich aus der Erde hervor.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcdxxxv{Ihm gehört, wer in den Himmeln und auf der Erde ist. Alle sind Ihm demütig ergeben.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcdxxxvi{Und Er ist es, Der die Schöpfung am Anfang macht und sie hierauf wiederholt; das ist für Ihn noch leichter. Er hat die höchste Eigenschaft in den Himmeln und auf der Erde, und Er ist der Allmächtige und Allweise.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcdxxxvii{Er prägt euch aus eurem eigenen (Lebens)bereich ein Gleichnis: Habt ihr denn (aus der Schar) unter denjenigen, die eure rechte Hand (an Sklaven) besitzt, Teilhaber an dem, womit Wir euch versorgt haben, so daß ihr darin gleich wäret und ihr sie fürchten müßtet, wie ihr einander fürchtet? So legen Wir die Zeichen ausführlich dar für Leute, die begreifen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcdxxxviii{Aber nein! Diejenigen, die Unrecht tun, folgen ihren Neigungen ohne (richtiges) Wissen. Wer sollte rechtleiten, wen Allah in die Irre gehen läßt? Und sie haben keine Helfer.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmcdxxxix{So richte dein Gesicht aufrichtig zur Religion hin als Anhänger des rechten Glaubens, - (gemäß) der natürlichen Anlage Allahs, in der Er die Menschen er schaffen hat. Keine Abänderung gibt es für die Schöpfung Allahs. Das ist die richtige Religion. Aber die meisten Menschen wissen nicht.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmcdxl{Haltet daran fest), indem ihr euch Ihm reuig zuwendet, und fürchtet Ihn und verrichtet das Gebet und gehört nicht zu den Götzendienern,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmcdxli{zu denjenigen, die ihre Religion spalteten und zu Lagern geworden sind, wobei jede Gruppierung froh ist über das, was sie bei sich hat.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmcdxlii{Und wenn den Menschen Unheil widerfahrt, rufen sie ihren Herrn an, indem sie sich Ihm reuig zuwenden. Wenn Er sie hierauf Barmherzigkeit von Sich kosten läßt, gesellt sogleich eine Gruppe von ihnen ihrem Herrn (andere) bei,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmcdxliii{um undankbar zu sein für das, was Wir ihnen gegeben haben. - Genießt nur; ihr werdet (es noch) erfahren.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmcdxliv{Oder haben Wir ihnen etwa eine Ermächtigung offenbart, die dann von dem spricht, was sie Ihm beigesellen?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmcdxlv{Und wenn Wir die Menschen Barmherzigkeit kosten lassen, sind sie froh darüber. Wenn sie aber etwas Böses trifft für das, was ihre Hände vorausgeschickt haben, verlieren sie sogleich die Hoffnung.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmcdxlvi{Sehen sie denn nicht, daß Allah die Versorgung großzügig gewährt, wem Er will, und auch bemißt? Darin sind wahrlich Zeichen für Leute, die glauben.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmcdxlvii{So gib dem Verwandten sein Recht, ebenso dem Armen und dem Sohn des Weges'. Das ist besser für diejenigen, die Allahs Angesicht begehren. Und jene sind es, denen es wohl ergeht.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmcdxlviii{Und was ihr an Ausgaben aufwendet, damit sie sich aus dem Besitz der Menschen vermehren, sie vermehren sich nicht bei Allah. Was ihr aber an Abgaben entrichtet im Begehren nach Allahs Angesicht ... - das sind diejenigen, die das Vielfache erzielen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmcdxlix{Allah ist es, Der euch erschaffen und dann versorgt hat. Hierauf läßt Er euch sterben, hierauf macht Er euch wieder lebendig. Gibt es unter euren Teilhabern einen, der irgend etwas von alledem tut? Preis sei Ihm! Erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmcdl{Unheil ist auf dem Festland und im Meer erschienen wegen dessen, was die Hände der Menschen erworben haben, damit Er sie einiges von dem kosten läßt, was sie getan haben, auf daß sie umkehren mögen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmcdli{Sag: Reist auf der Erde umher und schaut, wie das Ende derjenigen zuvor war. Die meisten von ihnen waren Götzendiener.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmcdlii{So richte dein Gesicht aufrichtig zur richtigen Religion hin, bevor von Allah ein Tag kommt, der nicht zurückgewiesen wird\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmcdliii{Wer ungläubig ist, dessen Unglaube geht zu seinem (eigenen) Nachteil. Und wer rechtschaffen handelt, bereitet sich selbst die Lagerstatt vor,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmcdliv{damit Er denjenigen, die glauben und rechtschaffene Werke tun, aus Seiner Huld vergelte. Gewiß, Er liebt nicht die Ungläubigen.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmcdlv{Und es gehört zu Seinen Zeichen, daß Er die Winde als Verkünder froher Botschaft (voraus)sendet, und damit Er euch etwas von Seiner Barmherzigkeit kosten läßt und damit die Schiffe auf Seinen Befehl fahren und damit ihr nach etwas von Seiner Huld trachtet, und auf daß ihr dankbar sein möget.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmcdlvi{Und Wir haben bereits vor dir Gesandte zu ihren (jeweiligen) Völkern gesandt. Sie kamen zu ihnen mit den klaren Beweisen. Da übten Wir Vergeltung an denjenigen, die Übeltaten begingen; und es war für Uns eine Pflicht, den Gläubigen zu helfen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmcdlvii{Allah ist es, der die Winde sendet, und da wühlen sie die Wolken auf. Dann breitet Er sie im Himmel aus, wie Er will, und macht sie zu Stücken. Da siehst du den (Platz)regen dazwischen herauskommen. Wenn Er damit von Seinen Dienern, wen Er will, trifft, freuen sie sich sogleich,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmcdlviii{obwohl sie vorher, bevor (er) auf sie herabgesandt wurde, wahrlich ganz verzweifelt waren.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmcdlix{Schau doch auf die Spuren von Allahs Barmherzigkeit, wie Er die Erde nach ihrem Tod wieder lebendig macht. Er ist wahrlich Derjenige, Der (auch) die Toten wieder lebendig macht, und Er hat zu allem die Macht.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmcdlx{Und wenn Wir einen Wind sendeten und sie dann alles gelb werden sähen, würden sie (trotzdem) danach fortfahren, ungläubig zu sein.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmcdlxi{Sei nicht traurig,) denn du kannst nicht die Toten hören lassen noch die Tauben den Ruf hören lassen, wenn sie den Rücken kehren.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmcdlxii{Noch kannst du die Blinden aus ihrem Irrtum heraus rechtleiten. Hören lassen kannst du nur, wer an Unsere Zeichen glaubt und die somit (Allah) ergeben sind.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmcdlxiii{Allah ist es, der euch (zuerst) in Schwäche erschafft. Hierauf verleiht Er nach der Schwäche Kraft. Hierauf verleiht Er nach der Kraft Schwäche und weißes Haar. Er erschafft, was Er will, und Er ist der Allwissende und Allmächtige.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmcdlxiv{Und am Tag, da sich die Stunde erhebt, schwören die Übeltäter, sie hätten nicht länger als eine Stunde verweilt. So pflegten sie sich (von der Wahrheit) abwendig machen zu lassen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmcdlxv{Und diejenigen, denen Wissen und Glauben gegeben worden ist, sagen: ``Nach Allahs Buch habt ihr bis zum Tag der Auferweckung verweilt. Das ist nun der Tag der Auferweckung, aber ihr wußtet (es) nicht."\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmcdlxvi{So wird an jenem Tag denjenigen, die Unrecht tun, ihre Entschuldigung nicht nützen, und es wird ihnen keine Gnade erwiesen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmcdlxvii{Wir haben ja den Menschen in diesem Qur'an allerlei Gleichnisse geprägt. Und wenn du zu ihnen mit einem Zeichen kommst, sagen ganz gewiß diejenigen, die ungläubig sind: ``Ihr bringt nur Falsches."\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmcdlxviii{So versiegelt Allah die Herzen derjenigen, die nicht Bescheid wissen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmcdlxix{So sei standhaft. Gewiß, Allahs Versprechen ist wahr. Und diejenigen, die nicht überzeugt sind, sollen dich ja nicht ins Wanken bringen.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmcdlxx{\basmalahde Alif-Lam-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcdlxxi{Dies sind die Zeichen des weisen Buches,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcdlxxii{eine Rechtleitung und Barmherzigkeit für die Gutes Tuenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcdlxxiii{die das Gebet verrichten und die Abgabe entrichten, und sie, die sie vom Jenseits überzeugt sind.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcdlxxiv{Jene verfahren nach einer Rechtleitung von ihrem Herrn, und das sind diejenigen, denen es wohl ergeht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcdlxxv{Unter den Menschen gibt es manchen, der zerstreuende Unterhaltung erkauft, um (die Menschen) von Allahs Weg ohne (richtiges) Wissen in die Irre zu führen und sich über ihn lustig zu machen. Für solche wird es schmachvolle Strafe geben.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcdlxxvi{Und wenn ihm Unsere Zeichen verlesen werden, kehrt er sich hochmütig ab, als ob er sie nicht gehört hätte, als ob in seinen Ohren Schwerhörigkeit wäre. So verkünde ihm schmerzhafte Strafe.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcdlxxvii{Gewiß, diejenigen, die glauben und rechtschaffene Werke tun, für sie wird es die Gärten der Wonne geben;\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcdlxxviii{ewig darin zu bleiben. (Das ist) Allahs Versprechen in Wahrheit. Und Er ist der Allmächtige und Allweise.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcdlxxix{Er hat die Himmel ohne Stützen, die ihr sehen könnt, erschaffen und auf der Erde festgegründete Berge gesetzt, daß sie nicht mit euch wanke, und auf ihr allerlei Tiere sich ausbreiten lassen. Und Wir lassen Wasser vom Himmel herabkommen und dann viele edle Arten auf ihr wachsen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcdlxxx{Das ist Allahs Schöpfung. Zeigt mir nun, was diejenigen außer Ihm erschaffen haben. Aber nein! Die Ungerechten befinden sich in deutlichem Irrtum.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxi{Und Wir gaben ja Luqman Weisheit: ``Sei Allah dankbar." Und wer dankbar ist, der ist nur zu seinem eigenen Vorteil dankbar. Und wer undankbar ist, - so ist Allah Unbedürftig und Lobenswürdig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxii{Und (gedenke,) als Luqman zu seinem Sohn sagte, indem er ihn ermahnte: ``O mein lieber Sohn, geselle Allah nicht(s) bei, denn Götzendienst ist fürwahr ein gewal tiges Unrecht."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxiii{Und Wir haben dem Menschen seine Eltern anbefohlen - seine Mutter hat ihn unter wiederholter Schwäche getragen, und seine Entwöhnung (erfolgt) innerhalb von zwei Jahren -: ``Sei Mir und deinen Eltern dankbar. Zu Mir ist der Ausgang."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxiv{Wenn sie sich aber darum bemühen, daß du Mir das beigesellst, wovon du kein Wissen hast, dann gehorche ihnen nicht, doch geh mit ihnen im Diesseits in rechtli cher Weise um. Und folge dem Weg dessen, der sich Mir reuig zuwendet. Zu Mir wird hierauf eure Rückkehr sein, da werde Ich euch kundtun, was ihr zu tun pflegtet.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxv{O mein lieber Sohn, gewiß, wäre es auch das Gewicht eines Senfkorns und befände es sich in einem Felsen oder in den Himmeln oder in der Erde, bringt es Allah bei. Gewiß, Allah ist Feinfühlig und Allkundig.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxvi{O mein lieber Sohn, verrichte das Gebet, gebiete das Rechte und verbiete das Verwerfliche und ertrage standhaft, was dich trifft. Gewiß, dies gehört zur Ent schlossenheit (in der Handhabung) der Angelegenheiten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxvii{Und zeige den Menschen nicht geringschätzig die Wange und gehe nicht übermütig auf der Erde einher, denn Allah liebt niemanden, der eingebildet und prahlerisch ist.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxviii{Halte das rechte Maß in deinem Gang und dämpfe deine Stimme, denn die widerwärtigste der Stimmen ist wahrlich die Stimme der Esel."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxix{Seht ihr nicht, daß Allah euch das, was in den Himmeln und was auf der Erde ist, dienstbar gemacht hat, und euch mit Seinen Gunsterweisen überhäuft hat äußerlich und innerlich? Doch gibt es unter den Menschen manchen, der über Allah ohne (richtiges) Wissen, ohne Rechtleitung und ohne erleuchtendes Buch streitet.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcdxc{Und wenn zu ihnen gesagt wird: ``Folgt dem, was Allah herabgesandt hat", sagen sie: ``Nein! Vielmehr folgen wir dem, worin wir unsere Väter vorgefunden haben." Was denn, auch wenn der Satan sie zur Strafe der Feuerglut einlädt?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcdxci{Wer sich Allah völlig hingibt und dabei Gutes tut, der hält sich an die festeste Handhabe. Und zu Allah (führt) das Ende der Angelegenheiten.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcdxcii{Und wer ungläubig ist, dessen Unglaube soll dich nicht traurig machen. Zu Uns wird ihre Rückkehr sein, dann werden Wir ihnen kundtun, was sie taten. Gewiß, Allah weiß über das Innerste der Brüste Bescheid.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcdxciii{Wir lassen sie ein wenig genießen, dann zwingen Wir sie in eine harte Strafe.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcdxciv{Und wenn du sie fragst, wer die Himmel und die Erde erschaffen hat, sagen sie ganz gewiß: ``Allah." Sag: (Alles) Lob gehört Allah! Aber nein! Die meisten von ihnen wissen nicht.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcdxcv{Allah gehört, was in den Himmeln und auf der Erde ist. Gewiß, Allah ist der Unbedürftige und Lobenswürdige.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcdxcvi{Und wenn auch das, was es auf der Erde an Bäumen gibt, Schreibrohre wären und das (gesamte) Meer und danach sieben weitere Meere als Nachschub (Tinte wären), würden die Worte Allahs nicht zu Ende gehen, denn Allah ist Allmächtig und Allweise.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcdxcvii{Die Erschaffung und die Auferweckung von euch (allen) ist nur (so) wie bei einer einzigen Seele. Gewiß, Allah ist Allhörend und Allsehend.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcdxcviii{Siehst du denn nicht, daß Allah die Nacht in den Tag eindringen und den Tag in die Nacht eindringen läßt und die Sonne und den Mond dienstbar gemacht hat - ein jedes läuft auf eine festgesetzte Frist zu -, und daß Allah dessen Kundig ist, was ihr tut?\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmcdxcix{Dies, weil Allah die Wahrheit ist und weil das, was sie anstatt Seiner anrufen, das Falsche ist und weil Allah der Erhabene, der Große ist.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmd{Siehst du nicht, daß die Schiffe durch die Gunst Allahs auf dem Meer fahren, damit Er euch etwas von Seinen Zeichen zeigt? Darin sind wahrlich Zeichen für jeden sehr Standhaften und sehr Dankbaren.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdi{Und wenn Wogen wie Schattendächer sie überdecken, rufen sie Allah an, (wobei sie) Ihm gegenüber aufrichtig in der Religion (sind). Wenn Er sie nun ans Land errettet, zeigen einige von ihnen ein gemäßigtes Verhalten. Und Unsere Zeichen verleugnet nur jeder sehr Treulose und sehr Undankbare.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdii{O ihr Menschen, fürchtet euren Herrn und habt Angst vor einem Tag, an dem weder ein Vater etwas für sein Kind begleichen kann, noch ein Kind für seinen Vater etwas wird begleichen können. Gewiß, Allahs Versprechen ist wahr. So soll euch das diesseitige Leben nicht täuschen, und nicht täuschen soll euch hinsichtlich Allahs der Täuscher.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdiii{Gewiß, Allah (allein) besitzt das Wissen über die Stunde, läßt den Regen herabkommen und weiß, was im Mutterleib ist. Niemand weiß, was er morgen erwerben wird, und niemand weiß, in welchem Land er sterben wird. Gewiß, Allah ist Allwissend und Allkundig.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdiv{\basmalahde Alif-Lam-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdv{Die Offenbarung des Buches, an dem es keinen Zweifel gibt, ist vom Herrn der Weltenbewohner.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdvi{Oder sagen sie: ``Er hat es ersonnen"? Nein! Vielmehr ist es die Wahrheit von deinem Herrn, damit du ein Volk warnst, zu denen noch kein Warner vor dir ge kommen ist, auf daß sie rechtgeleitet werden mögen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdvii{Allah ist es, Der die Himmel und die Erde und was dazwischen ist in sechs Tagen erschuf und Sich hierauf über den Thron erhob. Ihr habt außer Ihm weder Schutzherrn noch Fürsprecher. Bedenkt ihr denn nicht?\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdviii{Er regelt die Angelegenheit vom Himmel bis zur Erde. Hierauf steigt sie zu Ihm auf an einem Tag, dessen Maß tausend Jahre nach eurer Berechnung sind.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdix{Jener ist der Kenner des Verborgenen und des Offenbaren, der Allmächtige und Barmherzige,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdx{Der alles gut macht, was Er erschafft. Und Er machte die Schöpfung des Menschen am Anfang aus Lehm,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdxi{hierauf machte Er seine Nachkommenschaft aus einem Auszug aus verächtlichem Wasser.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdxii{Hierauf formte Er ihn zurecht und hauchte ihm von Seinem Geist ein, und Er hat euch Gehör, Augenlicht und Herzen gemacht. Wie wenig ihr dankbar seid!\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdxiii{Und sie sagen: ``Sollen wir etwa, wenn wir uns in der Erde verloren haben, denn wirklich wieder in neuer Schöpfung (erstehen)?" Aber nein! Sie verleugnen die Begegnung mit ihrem Herrn.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdxiv{Sag: Abberufen wird euch der Engel des Todes, der mit euch betraut ist, hierauf werdet ihr zu eurem Herrn zurückgebracht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdxv{Könntest du nur sehen, wenn die Übeltäter vor ihrem Herrn die Köpfe hängenlassen: ``Unser Herr, jetzt haben wir gesehen und gehört. Bringe uns zurück, so wollen wir rechtschaffen handeln. Gewiß, wir sind nun überzeugt"!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdxvi{Und wenn Wir gewollt hätten, hätten Wir jeder Seele ihre Rechtleitung gegeben. Aber (nun) ist das Wort von Mir unvermeidlich fällig geworden: ``Ganz gewiß werde Ich die Hölle mit den Ginn und den Menschen allesamt füllen."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdxvii{So kostet (es) dafür, daß ihr die Begegnung mit diesem eurem Tag vergessen habt. Gewiß, Wir haben euch (auch) vergessen'. Kostet die ewige Strafe für das, was ihr zu tun pflegtet.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdxviii{Nur diejenigen glauben an Unsere Zeichen, die, wenn sie damit ermahnt werden, ehrerbietig niederfallen und ihren Herrn lobpreisen und sich nicht hochmütig verhalten.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdxix{Ihre Seiten weichen vor den Schlafstätten zurück; sie rufen ihren Herrn in Furcht und Begehren an und geben von dem, womit Wir sie versorgt haben, aus.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdxx{Keine Seele weiß, welche Freuden im Verborgenen für sie bereitgehalten werden als Lohn für das, was sie zu tun pflegten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdxxi{Ist denn jemand, der gläubig ist, wie jemand, der ein Frevler ist? Sie sind nicht gleich(zustellen).\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdxxii{Was nun diejenigen angeht, die glauben und rechtschaffene Werke tun, so wird es für sie die Gärten der Zuflucht zur gastlichen Aufnahme geben für das, was sie zu tun pflegten.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdxxiii{Was aber diejenigen angeht, die freveln, so wird das (Höllen)feuer ihr Zufluchtsort sein. Jedesmal, wenn sie aus ihm herauskommen wollen, werden sie dahinein zu rückgebracht, und es wird zu ihnen gesagt: ``Kostet die Strafe des (Höllen)feuers, die ihr für Lüge zu erklären pflegtet."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdxxiv{Wir werden sie ganz gewiß etwas von der diesseitigen Strafe vor der größeren Strafe kosten lassen, auf daß sie umkehren mögen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdxxv{Und wer ist ungerechter als jemand, der mit den Zeichen seines Herrn ermahnt wird und sich hierauf von ihnen abwendet? Gewiß, Wir werden an den Übeltätern Vergeltung üben.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdxxvi{Wir gaben bereits Musa die Schrift - so sei nicht im Zweifel über die Begegnung mit Ihm - und machten sie zu einer Rechtleitung für die Kinder Isra'ils.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdxxvii{Und Wir bestellten unter ihnen Vorbilder, die (sie) nach Unserem Befehl leiteten, als sie sich standhaft gezeigt halten und von Unseren Zeichen überzeugt waren.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdxxviii{Gewiß, dein Herr wird zwischen ihnen am Tag der Auferstehung über das entscheiden, worüber sie uneinig waren.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdxxix{Ist ihnen nicht deutlich geworden, wie viele Geschlechter Wir vor ihnen vernichtet haben, in deren Wohnorten sie (nun) umhergehen? Darin sind wahrlich Zeichen. Wollen sie denn nicht hören?\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdxxx{Sehen sie denn nicht, daß Wir das Wasser zum dürren Land treiben und dann dadurch Pflanzen hervorbringen, von denen ihr Vieh und sie selbst essen? Wollen sie denn nicht einsichtig sein?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdxxxi{Und sie sagen: ``Wann wird diese Entscheidung eintreten, wenn ihr wahrhaftig seid?"\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdxxxii{Sag: Am Tag der Entscheidung wird denjenigen, die ungläubig waren, ihr Glaube nicht (mehr) nützen, noch wird ihnen Aufschub gewährt.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdxxxiii{So wende dich ab von ihnen und warte ab; sie warten ebenfalls ab.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdxxxiv{\basmalahde O Prophet, furchte Allah und gehorche nicht den Ungläubigen und den Heuchlern. Gewiß, Allah ist Allwissend und Allweise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdxxxv{Und folge dem, was dir (als Offenbarung) von deinem Herrn eingegeben wird. Gewiß, Allah ist dessen, was ihr tut, Kundig.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdxxxvi{Und verlasse dich auf Allah. Allah genügt als Sachwalter.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdxxxvii{Allah hat keinem Mann zwei Herzen in seinem Inneren gemacht. Und Er hat eure Gattinnen, von denen ihr euch durch den Rückenschwur trennt, nicht (wirklich) zu euren Müttern gemacht. Und Er hat eure angenommenen Söhne nicht (wirklich) zu euren Söhnen gemacht. Das sind eure Worte aus eurem (eigenen) Mund. Aber Allah sagt die Wahrheit, und Er leitet den (rechten) Weg.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdxxxviii{Nennt sie nach ihren Vätern; das ist gerechter vor Allah. Wenn ihr ihre Väter nicht kennt, dann sind sie eure Brüder in der Religion und eure Schützlinge. Es ist für euch keine Sünde in dem, was ihr an Fehlern begeht, sondern was eure Herzen vorsätzlich anstreben. Und Allah ist Allvergebend und Barmherzig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdxxxix{Der Prophet steht den Gläubigen näher als sie sich selbst, und seine Gattinnen sind ihre Mütter. Und die Blutsverwandten stehen nach dem Buch Allahs einander näher als die Gläubigen und Auswanderer, außer, daß ihr euren Schützlingen Gutes tun solltet. Dies steht im Buch verzeichnet.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdxl{Und (gedenke,) als Wir von den Propheten ihr Versprechen abnahmen, und auch von dir und von Nuh, Ibrahim, Musa und 'Isa, dem Sohn Maryams; Wir nahmen ihnen ein festes Versprechen ab,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdxli{damit Er die Wahrhaftigen nach ihrer Wahrhaftigkeit frage. Und Er hat für die Ungläubigen schmerzhafte Strafe bereitet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdxlii{O die ihr glaubt, gedenkt der Gunst Allahs an euch, als Heerscharen zu euch kamen'. Da sandten Wir gegen sie einen Wind und Heerscharen, die ihr nicht saht. Und was ihr tut, sieht Allah wohl.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdxliii{Als sie von oben zu euch kamen und von unten her, und als die Blicke unsicher wurden und die Herzen die Kehle erreichten und ihr von Allah verschiedene Meinungen hattet.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdxliv{Dort wurden die Gläubigen geprüft und heftig erschüttert.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdxlv{Als die Heuchler und diejenigen, in deren Herzen Krankheit ist, sagten: ``Allah und Sein Gesandter haben uns nur Trügerisches versprochen."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdxlvi{Und als eine Gruppe von ihnen sagte: ``O ihr Leute von Yatrib, ihr könnt euch (hier) nicht aufhalten. Kehrt zurück." Und ein Teil von ihnen bat den Propheten um Erlaubnis, (heimzukehren,) indem sie sagten: ``Unsere Häuser sind ohne Schutz." Dabei waren sie nicht ohne Schutz, sie wollten nur fliehen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdxlvii{Wäre man von ihren (verschiedenen) Bezirken her zu ihnen eingedrungen und wären sie dann aufgefordert worden, der Versuchung nachzugeben, sie hätten es begangen, und sie hätten dabei nur kurze Zeit gezögert.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdxlviii{Dabei hatten sie sich zuvor gegenüber Allah verpflichtet, sie würden nicht den Rücken kehren. Nach der (Erfüllung der) Verpflichtung gegenüber Allah wird ge fragt werden.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdxlix{Sag: Die Flucht wird euch nicht nützen, wenn ihr davor flieht, zu sterben oder getötet zu werden. Und dann wird euch nur noch ein wenig Nießbrauch gewährt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdl{Sag: Wer ist es denn, der euch vor Allah schützen könnte, wenn Er euch Böses will oder wenn Er euch Barmherzigkeit (erweisen) will? Und sie werden für sich vor Allah weder Schutzherrn noch Helfer finden.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdli{Allah kennt wohl diejenigen von euch, die (die anderen) behindern und die zu ihren Brüdern sagen: ``Kommt her zu uns." Und sie lassen sich nur wenig auf den Kampf ein\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdlii{und sind dabei habsüchtig euch gegenüber. Wenn aber die Angst sich einstellt, siehst du, wie sie dich mit kreisenden Augen anschauen wie einer, der vor dem Tod ohnmächtig wird. Wenn dann die Angst gewichen ist, verletzen sie euch mit scharfen Zungen aus Habsucht nach dem (erbeuteten) Gut. Jene haben nicht (wirklich) geglaubt; darum läßt Allah ihre Werke hinfällig werden; und dies ist Allah ein leichtes.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdliii{Sie meinen, die Gruppierungen seien nicht weggegangen. Und wenn die Gruppierungen (wieder)kommen sollten, möchten sie gern, sie wären in der Wüste unter den Wüstenarabern und würden sich (nur) über euch erkundigen. Und wenn sie unter euch wären, würden sie nur wenig kämpfen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdliv{Ihr habt ja im Gesandten Allahs ein schönes Vorbild, (und zwar) für einen jeden, der auf Allah und den Jüngsten Tag hofft und Allahs viel gedenkt.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdlv{Und als die Gläubigen die Gruppierungen sahen, sagten sie: ``Das ist, was Allah und Sein Gesandter uns versprochen haben; Allah und Sein Gesandter sagen die Wahrheit." Und es mehrte ihnen nur den Glauben und die Ergebung.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdlvi{Unter den Gläubigen gibt es Männer, die das wahr gemacht haben, wozu sie sich Allah gegenüber verpflichteten. Unter ihnen gibt es manche, die ihr Gelübde er füllt haben; und unter ihnen gibt es manche, die noch warten (müssen). Und sie haben keine Änderung vorgenommen;\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdlvii{damit Allah den Wahrhaftigen ihre Wahrhaftigkeit vergelte und die Heuchler strafe, wenn Er will, oder sich ihnen Reue-Annehmend zuwende. Gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdlviii{Und Allah wies diejenigen, die ungläubig waren, mit ihrem Grimm zurück, ohne daß sie etwas Gutes erlangt hätten. Und Allah ersparte den Gläubigen den Kampf. Allah ist Stark und Allmächtig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdlix{Und Er ließ diejenigen von den Leuten der Schrift, die ihnen beigestanden hatten, aus ihren Burgen heruntersteigen. Und Er jagte in ihre Herzen Schrecken; eine Gruppe (von ihnen) habt ihr getötet und eine Gruppe gefangengenommen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdlx{Und Er gab euch zum Erbe ihr Land, ihre Wohnstätten und ihren Besitz und auch ein Land, das ihr (vorher) nicht betreten hattet. Und Allah hat zu allem die Macht.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdlxi{O Prophet, sag zu deinen Gattinnen: Wenn ihr das diesseitige Leben und seinen Schmuck haben wollt, dann kommt her, ich werde euch eine Abfindung gewähren und euch auf schöne Weise freigeben.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdlxii{Wenn ihr aber Allah und Seinen Gesandten und die jenseitige Wohnstätte haben wollt, so hat Allah für diejenigen von euch, die Gutes tun, großartigen Lohn berei tet.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdlxiii{O Frauen des Propheten, wer von euch etwas klar Abscheuliches begeht, derjenigen wird die Strafe verzweifacht. Und das ist Allah ein leichtes.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdlxiv{Und wer von euch Allah und Seinem Gesandten demütig ergeben ist und rechtschaffen handelt, der geben Wir ihren Lohn zweimal. Und Wir haben für sie ehrenvolle Versorgung bereitet.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdlxv{O Frauen des Propheten, ihr seid nicht wie irgendeine von den (übrigen) Frauen. Wenn ihr gottesfürchtig seid, dann seid nicht unterwürfig im Reden, damit nicht derjenige, in dessen Herzen Krankheit ist, begehrlich wird, sondern sagt geziemende Worte.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdlxvi{Haltet euch in euren Häusern auf; und stellt euch nicht zur Schau wie in der Zeit der früheren Unwissenheit. Verrichtet das Gebet und entrichtet die Abgabe und gehorcht Allah und Seinem Gesandten. Allah will gewiß nur den Makel von euch entfernen, ihr Angehörigen des Hauses, und euch völlig rein machen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdlxvii{Und gedenkt dessen, was in euren Häusern von den Zeichen Allahs und von der Weisheit verlesen wird. Gewiß, Allah ist Feinfühlig und Allkundig.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdlxviii{Gewiß, muslimische Männer und muslimische Frauen, gläubige Männer und gläubige Frauen, ergebene Männer und ergebene Frauen, wahrhaftige Männer und wahrhaftige Frauen, standhafte Männer und standhafte Frauen, demütige Männer und demütige Frauen, Almosen gebende Männer und Almosen gebende Frauen, fa stende Männer und fastende Frauen, Männer, die ihre Scham hüten und Frauen, die (ihre Scham) hüten, und Allahs viel gedenkende Männer und gedenkende Frauen für (all) sie hat Allah Vergebung und großartigen Lohn bereitet.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdlxix{Weder für einen gläubigen Mann noch für eine gläubige Frau gibt es, wenn Allah und Sein Gesandter eine Angelegenheit entschieden haben, die Möglichkeit, in ihrer Angelegenheit zu wählen. Und wer sich Allah und Seinem Gesandten widersetzt, der befindet sich ja in deutlichem Irrtum.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdlxx{Und als du zu demjenigen sagtest, dem Allah Gunst erwiesen hatte und dem auch du Gunst erwiesen hattest: ``Behalte deine Gattin für dich und fürchte Allah", und in deinem Inneren verborgen hieltest, was Allah doch offenlegen wird, und die Menschen fürchtetest, während Allah ein größeres Anrecht darauf hat, daß du Ihn fürchtest. Als dann Zaid keinen Wunsch mehr an ihr hatte, gaben Wir sie dir zur Gattin, damit für die Gläubigen kein Grund zur Bedrängnis bestehe hinsichtlich der Gattinnen ihrer angenommenen Söhne, wenn diese keinen Wunsch mehr an ihnen haben. Und Allahs Anordnung wird (stets) ausgeführt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdlxxi{Es besteht für den Propheten kein Grund zur Bedrängnis in dem, was Allah für ihn verpflichtend gemacht hat. (So war) Allahs Gesetzmäßigkeit mit denjenigen, die zuvor dahingegangen sind - und Allahs Anordnung ist ein fest gefaßter Beschluß\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdlxxii{die Allahs Botschaften übermitteln, Ihn fürchten und niemanden fürchten außer Allah. Und Allah genügt als Abrechner.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdlxxiii{Muhammad ist nicht der Vater irgend jemandes von euren Männern, sondern Allahs Gesandter und das Siegel der Propheten. Und Allah weiß über alles Bescheid.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdlxxiv{O die ihr glaubt, gedenkt Allahs in häufigem Gedenken\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdlxxv{und preist Ihn morgens und abends.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdlxxvi{Er ist es, Der über euch den Segen spricht - und auch Seine Engel -, damit Er euch aus den Finsternissen ins Licht hinausbringt; und Er ist zu den Gläubigen Barmherzig.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdlxxvii{Ihr Gruß am Tag, da sie Ihm begegnen, wird sein: ``Friede !" Und Er hat für sie trefflichen Lohn bereitet.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdlxxviii{O Prophet, Wir haben dich gesandt als Zeugen, als Verkünder froher Botschaft und als Warner\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdlxxix{und als einen, der zu Allah mit Seiner Erlaubnis ruft und als eine lichtspendende Leuchte.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdlxxx{Und verkünde den Gläubigen, daß es für sie von Allah große Huld geben wird.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdlxxxi{Und gehorche nicht den Ungläubigen und den Heuchlern; beachte nicht die Beleidigungen, die sie dir zufügen, und verlasse dich auf Allah. Und Allah genügt als Sachwalter.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdlxxxii{O die ihr glaubt, wenn ihr gläubige Frauen heiratet und euch hierauf von ihnen scheidet, bevor ihr sie berührt habt, dann dürft ihr für sie keine von euch berechnete Wartezeit aussetzen; gewährt ihnen eine Abfindung und gebt sie auf schöne Weise frei.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdlxxxiii{O Prophet, Wir haben dir (zu heiraten) erlaubt: deine Gattinnen, denen du ihren Lohn gegeben hast, das, was deine rechte Hand (an Sklavinnen) besitzt von dem, was Allah dir als Beute zugeteilt hat, die Töchter deiner Onkel väterlicherseits und die Töchter deiner Tanten väterlicherseits, die Töchter deiner Onkel müt terlicherseits und die Töchter deiner Tanten mütterlicherseits, die mit dir ausgewandert sind; auch eine (jede) gläubige Frau, wenn sie sich dem Propheten (ohne Gegenforderung) schenkt und falls der Prophet sie heiraten will: Dies ist dir vorbehalten unter Ausschluß der (übrigen) Gläubigen - Wir wissen wohl, was Wir ihnen hinsichtlich ihrer Gattinnen und dessen, was ihre rechte Hand (an Sklavinnen) besitzt, verpflichtend gemacht haben -, damit für dich kein Grund zur Bedrängnis bestehe. Und Allah ist Allvergebend und Barmherzig.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdlxxxiv{Du darfst zurückstellen, wen von ihnen du willst, und du darfst bei dir aufnehmen, wen du willst. Und wenn du doch eine von denjenigen begehrst, die du abgewiesen hast, dann ist das für dich keine Sünde. Das ist eher geeignet, daß sie frohen Mutes, nicht traurig und daß sie alle mit dem zufrieden sind, was du ihnen gibst. Allah weiß, was in euren Herzen ist. Allah ist Allwissend und Nachsichtig.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdlxxxv{Darüber hinaus ist dir weder erlaubt, Frauen zu heiraten noch sie gegen (andere) Gattinnen einzutauschen, auch wenn ihre Schönheit dir gefallen sollte, mit Aus nahme dessen, was deine rechte Hand (an Sklavinnen) besitzt. Und Allah ist Wächter über alles.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdlxxxvi{O die ihr glaubt, tretet nicht in die Häuser des Propheten ein - außer es wird euch erlaubt - zu(r Teilnahme an) einem Essen, ohne auf die rechte Zeit zu warten. Sondern wenn ihr (herein)gerufen werdet, dann tretet ein, und wenn ihr gegessen habt, dann geht auseinander, und (tut dies,) ohne euch mit geselliger Unterhaltung aufzuhalten. Solches fügt dem Propheten Leid zu, aber er schämt sich vor euch. Allah aber schämt sich nicht vor der Wahrheit. Und wenn ihr sie um einen Gegenstand bittet, so bittet sie hinter einem Vorhang. Das ist reiner für eure Herzen und ihre Herzen. Und es steht euch nicht zu, dem Gesandten Allahs Leid zuzufügen, und auch nicht, jemals seine Gattinnen nach ihm zu heiraten. Gewiß, das wäre bei Allah etwas Ungeheuerliches.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdlxxxvii{Ob ihr etwas offenlegt oder verbergt, gewiß, Allah weiß über alles Bescheid.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdlxxxviii{Es besteht für sie keine Sünde weder hinsichtlich ihrer Väter, noch ihrer Söhne, noch ihrer Brüder, der Söhne ihrer Brüder, noch der Söhne ihrer Schwestern, noch ihrer Frauen noch derjenigen, die ihre rechte Hand besitzt. Und fürchtet Allah. Gewiß, Allah ist über alles Zeuge.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmdlxxxix{Gewiß, Allah und Seine Engel sprechen den Segen über den Propheten. O die ihr glaubt, sprecht den Segen über ihn und grüßt ihn mit gehörigem Gruß.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmdxc{Diejenigen, die Allah und Seinem Gesandten Leid zufügen, verflucht Allah im Diesseits und Jenseits, und für sie hat Er schmachvolle Strafe bereitet.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmdxci{Und diejenigen, die den gläubigen Männern und den gläubigen Frauen Leid zufügen für etwas, was sie nicht begangen haben, laden damit Verleumdung und offenkundige Sünde auf sich.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmdxcii{O Prophet, sag deinen Gattinnen und deinen Töchtern und den Frauen der Gläubigen, sie sollen etwas von ihrem Überwurf über sich herunterziehen. Das ist eher geeignet, daß sie erkannt und so nicht belästigt werden. Und Allah ist Allver gebend und Barmherzig.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmdxciii{Wenn nicht die Heuchler und diejenigen, in deren Herzen Krankheit ist, und diejenigen, die beunruhigende Gerüchte in der Stadt verbreiten, (damit) aufhören, werden Wir dich ganz gewiß gegen sie antreiben. Hierauf werden sie nur noch kurze Zeit darin deine Nachbarn sein.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmdxciv{Sie sind) verflucht. Wo immer sie (im Kampf) angetroffen werden, werden sie ergriffen und allesamt getötet.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmdxcv{So war) Allahs Gesetzmäßigkeit mit denjenigen, die zuvor dahingegangen sind. Und du wirst in Allahs Gesetzmäßigkeit keine Änderung finden.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmdxcvi{Die Menschen fragen dich nach der Stunde. Sag: Das Wissen darüber ist nur bei Allah. Was läßt dich wissen? Vielleicht ist die Stunde nahe.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmdxcvii{Gewiß, Allah hat die Ungläubigen verflucht und für sie eine Feuerglut bereitet,\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmdxcviii{ewig und auf immer darin zu bleiben, ohne einen Schutzherrn noch Helfer zu finden.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmdxcix{Am Tag, da ihre Gesichter im (Höllen)feuer hin und her gedreht werden, sagen sie: ``O hätten wir doch Allah gehorcht und hätten wir dem Gesandten gehorcht!"\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmdc{Und sie sagen: ``Unser Herr, gewiß, wir haben unseren Herrschern und unseren Großen gehorcht, und da haben sie uns vom Weg abirren lassen.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmdci{Unser Herr, lasse ihnen zweifache Strafe zukommen, und verfluche sie mit einem großen Fluch'."\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmdcii{O die ihr glaubt, seid nicht wie diejenigen, die Musa Leid zufügten, worauf Allah ihn freisprach von dem, was sie (über ihn) sagten; und er war bei Allah angesehen.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmdciii{O die ihr glaubt, fürchtet Allah und sagt treffende Worte,\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmdciv{so läßt Er eure Werke als gut gelten und vergibt euch eure Sünden. Und wer Allah und Seinem Gesandten gehorcht, der erzielt ja einen großartigen Erfolg,\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmdcv{Wir haben das anvertraute Gut den Himmeln und der Erde und den Bergen angeboten, aber sie weigerten sich, es zu tragen, sie scheuten sich davor. Der Mensch trug es - gewiß, er ist sehr oft ungerecht und sehr oft töricht.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmdcvi{Das ist so,) damit Allah die Heuchler, Männer und Frauen, und die Götzendiener, Männer und Frauen, straft und sich den Gläubigen, Männern und Frauen, ReueAnnehmend zuwendet. Allah ist Allvergebend und Barmherzig.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmdcvii{\basmalahde (Alles) Lob gehört Allah, Dem gehört, was in den Himmeln und was auf der Erde ist, und (alles) Lob gehört Ihm im Jenseits. Er ist der Allweise und Allkundige.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdcviii{Er weiß, was in die Erde eindringt und was aus ihr herauskommt, was vom Himmel herabkommt und was dorthin aufsteigt. Und Er ist der Barmherzige und Allvergebende.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdcix{Diejenigen, die ungläubig sind, sagen: ``Die Stunde kommt nicht über uns." Sag: Doch bei meinem Herrn - sie kommt ganz gewiß über euch -, (bei Ihm,) dem Kenner des Verborgenen! Es entgeht Ihm nicht das Gewicht eines Stäubchens, weder in den Himmeln noch auf der Erde. Und es gibt nichts, was kleiner ist als dies oder größer, das nicht in einem deutlichen Buch (verzeichnet) wäre,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdcx{damit Er (es) denjenigen, die glauben und rechtschaffene Werke tun, vergelte. Für diese wird es Vergebung und ehrenvolle Versorgung geben.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdcxi{Und diejenigen, die ständig gegen Unsere Zeichen eifern und ihnen Machtlosigkeit nachweisen wollen, für sie wird es schmerzhafte Strafe von Züchtigung geben.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdcxii{Und diejenigen, denen das Wissen gegeben worden ist, sehen, (daß) das, was zu dir von deinem Herrn (als Offenbarung) herabgesandt worden ist, die Wahrheit ist und zum Weg des Allmächtigen und Lobenswürdigen führt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdcxiii{Und diejenigen, die ungläubig sind, sagen: ``Sollen wir euch auf einen Mann hinweisen, der euch kundtut, daß ihr, wenn ihr vollständig in Stücke gerissen worden seid, doch wirklich in neuer Schöpfung (erstehen) werdet?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdcxiv{Hat er gegen Allah eine Lüge ersonnen, oder leidet er an Besessenheit?" Nein! Vielmehr stecken diejenigen, die an das Jenseits nicht glauben, in der Strafe und befinden sich im tiefen Irrtum.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdcxv{Sehen sie denn nicht auf das, was vom Himmel und von der Erde vor ihnen und was hinter ihnen ist? Wenn Wir wollen, lassen Wir die Erde mit ihnen versinken oder Stücke vom Himmel auf sie herabfallen. Darin ist wahrlich ein Zeichen für jeden Diener, der sich (Allah) reuig zuwendet.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdcxvi{Und Wir gaben ja Dawud eine Huld von Uns. - ``Ihr Berge, preist (Allah) im Widerhall mit ihm, und auch ihr Vögel." Und Wir machten für ihn das Eisen ge schmeidig:\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdcxvii{``Fertige Panzergewänder an und füge im richtigen Maß die Panzermaschen aneinander. Und handelt rechtschaffen, denn Ich sehe wohl, was ihr tut."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdcxviii{Und Sulaiman (machten Wir) den Wind (dienstbar), dessen Morgenlauf einen Monat und dessen Abendlauf einen Monat beträgt'. Und Wir ließen die Quelle des geschmolzenen Kupfers für ihn fließen. Und unter den Ginn gab es manche, die mit der Erlaubnis seines Herrn vor ihm tätig waren. Wer von ihnen von Unserem Befehl abweicht, den lassen Wir von der Strafe der Feuerglut kosten.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdcxix{Sie machten ihm, was er wollte, an Gebetsräumen, Bildwerken, Schüsseln wie Wasserbecken und feststehenden Kesseln. - ``Verrichtet, ihr Sippe Dawuds, eure Arbeit in Dankbarkeit", denn (nur) wenige von Meinen Dienern sind wirklich dankbar.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdcxx{Und als Wir für ihn den Tod bestimmt hatten, wies sie auf sein Ableben nur das Tier der Erde hin, das seinen Stab fraß. Als er dann niederstürzte, wurde den Ginn klar, daß sie, wenn sie das Verborgene gewußt, nicht (weiter) in der schmachvollen Strafe verweilt hätten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdcxxi{Die Sabaer hatten ja in ihrem Wohnort ein Zeichen: zwei Gärten zur Rechten und zur Linken. - ``Eßt von der Versorgung eures Herrn und dankt Ihm. Eine gute Ort schaft (ist es) und ein Allvergebender Herr."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdcxxii{Aber sie wandten sich ab. Da sandten Wir gegen sie die Flut des Staudammes, und Wir tauschten ihnen ihre zwei Gärten aus gegen zwei Gärten mit bitterem Ern teertrag und Tamarisken und einigen wenigen Zizyphusbäumen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdcxxiii{Das vergalten Wir ihnen, daß sie undankbar waren. Vergelten Wir denn sonst jemand anderem als dem Undankbaren?\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdcxxiv{Und Wir legten zwischen ihnen und den Städten, die Wir gesegnet haben, sichtbare Städte an. Und Wir setzten das rechte Maß der Reise zwischen ihnen fest: ``Reist in ihnen - Nächte und Tage - in Sicherheit umher."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdcxxv{Sie aber sagten: ``Unser Herr, vergrößere die Entfernungen zwischen unseren Reise(abschnitte)n." Sie taten sich selbst Unrecht. So machten Wir sie zu(m Gegenstand von) Geschichten und rissen sie vollständig in Stücke. Darin sind wahrlich Zeichen für jeden sehr Standhaften und sehr Dankbaren.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdcxxvi{Wahrlich, Iblis fand seine Meinung von ihnen bestätigt. Sie folgten ihm, außer einer Gruppe der Gläubigen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdcxxvii{Doch hatte er keine Macht über sie; (es geschah) nur, damit Wir in Erfahrung bringen, wer an das Jenseits glaubt, und (ihn unterscheiden) von demjenigen, der darüber im Zweifel ist. Dein Herr ist Hüter über alles.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdcxxviii{Sag: Ruft diejenigen an, die ihr anstatt Allahs angebt. Sie verfügen nicht (einmal) über das Gewicht eines Stäubchens weder in den Himmeln noch auf der Erde. Und sie haben an ihnen beiden keinen Anteil, noch nahm Er sich unter ihnen jemanden, der Ihm Beistand leisten sollte.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdcxxix{Auch nützt bei Ihm die Fürsprache nicht - außer, wem Er es erlaubt. Wenn dann der Schrecken von ihren Herzen genommen ist, sagen sie: ``Was hat euer Herr ge sagt?" Sie sagen: ``Die Wahrheit", und Er ist der Erhabene, der Große.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdcxxx{Sag: Wer versorgt euch von den Himmeln und der Erde? Sag: Allah. Fürwahr, entweder wir oder ihr (die einen von uns) befinden sich in Rechtleitung oder in deutlichem Irrtum.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdcxxxi{Sag: Ihr werdet nicht danach befragt werden, was wir an Übeltaten begangen haben, noch werden wir danach befragt werden, was ihr tut.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdcxxxii{Sag: Zusammenbringen wird uns unser Herr, hierauf wird Er zwischen uns der Wahrheit entsprechend entscheiden. Er ist der wahrhaft gerechte Entscheider, der Allwissende.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdcxxxiii{Sag: Zeigt mir diejenigen, die ihr Ihm als Teilhaber beigegeben habt. Keineswegs! Vielmehr ist Er Allah, der Allmächtige und Allweise.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdcxxxiv{Und Wir haben dich für die Menschen allesamt nur als Frohboten und Warner gesandt. Aber die meisten Menschen wissen nicht.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdcxxxv{Und sie sagen: ``Wann wird dieses Versprechen eintreten, wenn ihr wahrhaftig seid?"\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdcxxxvi{Sag: Für euch ist eine Verabredung auf einen Tag (festgelegt), von dem ihr (euer Schicksal) weder um eine Stunde hinausschieben noch vorverlegen könnt.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdcxxxvii{Und diejenigen, die ungläubig sind, sagen: ``Wir werden nicht an diesen Qur'an glauben und auch nicht an das, was (an Offenbarungen) vor ihm da war." Könntest du nur sehen, wenn die Ungerechten vor ihren Herrn gestellt werden und untereinander die Worte wechseln! Diejenigen, die unterdrückt wurden, sagen zu denjenigen, die sich hochmütig verhielten: ``Wenn ihr nicht gewesen wäret, so wären wir wahrlich gläubig gewesen."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdcxxxviii{Diejenigen, die sich hochmütig verhielten, sagen zu denjenigen, die unterdrückt wurden: ``Sind wir es gewesen, die euch von der Rechtleitung abgehalten haben, nachdem sie zu euch gekommen war? Nein! Vielmehr wart ihr Übeltäter."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdcxxxix{Und diejenigen, die unterdrückt wurden, sagen zu denjenigen, die sich hochmütig verhielten: ``Nein! Vielmehr waren es (eure) Ränke bei Nacht und bei Tag, da ihr uns befohlen hattet, Allah zu verleugnen und Ihm andere als Seinesgleichen zur Seite zu stellen." Und sie werden insgeheim Reue hegen, wenn sie die Strafe sehen. Und Wir werden die Fesseln um die Hälse derjenigen legen, die ungläubig waren. Wird ihnen (denn) etwas anderes vergolten als das, was sie zu tun pflegten?\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdcxl{Wir haben in keine Stadt einen Warner gesandt, ohne daß diejenigen, die in ihr üppig lebten, gesagt hätten: ``Gewiß, das, womit ihr gesandt seid, verleugnen wir."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdcxli{Und sie sagen: ``Wir haben mehr Besitz und Kinder, und wir werden nicht gestraft werden."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdcxlii{Sag: Gewiß, mein Herr gewährt die Versorgung großzügig, wem Er will, und bemißt auch. Aber die meisten Menschen wissen nicht.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdcxliii{Nicht euer Besitz ist es, und auch nicht eure Kinder sind es, die euch Zutritt in Unsere Nähe verschaffen, außer jemandem, der glaubt und rechtschaffen handelt. Diese erhalten mehrfachen Lohn für das, was sie getan haben, und sie werden in den Obergemächern in Sicherheit sein.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdcxliv{Diejenigen aber, die ständig gegen Unsere Zeichen eifern und ihnen Machtlosigkeit nachweisen wollen, sie werden zur Strafe vorgeführt werden.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdcxlv{Sag: Gewiß, mein Herr gewährt die Versorgung großzügig, wem von Seinen Dienern Er will, und bemißt auch. Und was immer ihr auch ausgebt, so wird Er es euch ersetzen, und Er ist der Beste der Versorger.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdcxlvi{Und am Tag, da Er sie alle versammelt, hierauf Er zu den Engeln sagen wird: ``Diese da, pflegten sie euch zu dienen?",\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdcxlvii{werden sie sagen: ``Preis sei Dir! Du bist unser Schutzherr, nicht sie. Nein! Vielmehr dienten sie den Ginn; die meisten von ihnen glauben an sie."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdcxlviii{``Heute vermögt ihr einer dem anderen weder Nutzen noch Schaden zu bringen." Und Wir sagen zu denjenigen, die Unrecht getan haben: ``Kostet die Strafe des (Höllen)feuers, das ihr für Lüge zu erklären pflegtet."\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdcxlix{Und wenn ihnen Unsere Zeichen als klare Beweise verlesen werden, sagen sie: ``Dieser ist nur ein Mann, der euch von dem abhalten will, dem eure Väter dienten." Und sie sagen: ``Das ist nur eine ersonnene ungeheuerliche Lüge." Und diejenigen, die ungläubig sind, sagen von der Wahrheit, nachdem sie zu ihnen gekommen ist: ``Das ist nur deutliche Zauberei."\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdcl{Und wir haben ihnen weder Schriften gegeben, die sie (hätten) eingehend lesen (können), noch haben Wir vor dir einen Warner zu ihnen gesandt.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdcli{Für Lüge erklärt haben (es auch) diejenigen, die vor ihnen waren, wobei diese nicht einmal das Zehntel dessen erreichten, was Wir jenen gegeben hatten. Sie bezichtigten Meine Gesandten der Lüge. Wie war da Meine Mißbilligung!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdclii{Sag: Ich ermahne euch nur zu einem: daß ihr euch zu zweit und einzeln um Allahs willen hinstellt und hierauf nachdenkt. Euer Gefährte leidet nicht an Besessenheit. Er ist euch nur ein Warner vor einer (bevorstehenden) strengen Strafe.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdcliii{Sag: Was auch immer an Lohn ich von euch hätte verlangen können, das gehört euch. Mein Lohn obliegt nur Allah. Und Er ist über alles Zeuge.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdcliv{Sag: Mein Herr kommt mit der Wahrheit, (Er,) der Allwisser der verborgenen Dinge.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdclv{Sag: Die Wahrheit ist gekommen. Das Falsche kann weder (etwas) neu von Anfang an machen noch es wiederholen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdclvi{Sag: Wenn ich irregehe, gehe ich nur zu meinem eigenen Nachteil irre, und wenn ich rechtgeleitet bin, so ist es durch das, was mir mein Herr (als Offenbarung) eingibt. Gewiß, Er ist Allhörend und nahe.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdclvii{Könntest du nur sehen, wenn sie erschrecken! - Da gibt es kein Entrinnen -, und sie werden von einem nahen Ort aus weggenommen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdclviii{Und sie sagen: ``Wir glauben daran." Aber wie könnten sie (den Glauben) von einem fernen Ort aus erlangen,\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdclix{wo sie ihn doch zuvor verleugnet haben? Und sie schleudern (Vermutungen) auf das Verborgene von einem fernen Ort aus.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdclx{Und es wird eine Trennung vorgenommen zwischen ihnen und dem, was sie begehren, so wie es zuvor mit ihresgleichen gemacht wurde; sie waren ja in starkem Zweifel.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdclxi{\basmalahde (Alles) Lob gehört Allah, dem Erschaffer der Himmel und der Erde, Der die Engel zu Gesandten gemacht hat mit Flügeln, (je) zwei, drei und vier! Er fügt der Schöpfung hinzu, was Er will. Gewiß, Allah hat zu allem die Macht.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdclxii{Was Allah den Menschen an Barmherzigkeit auftut, das kann keiner zurückhalten. Und was Er zurückhält, das kann keiner nach Ihm freigeben. Und Er ist der Allmächtige und Allweise.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdclxiii{O ihr Menschen, gedenkt der Gunst Allahs an euch. Gibt es einen anderen Schöpfer als Allah, Der euch vom Himmel und von der Erde versorgt? Es gibt keinen Gott außer Ihm. Wie laßt ihr euch also abwendig machen?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdclxiv{Wenn sie dich der Lüge bezichtigen, so wurden bereits vor dir Gesandte der Lüge bezichtigt. Und zu Allah werden (all) die Angelegenheiten zurückgebracht.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdclxv{O ihr Menschen, gewiß, Allahs Versprechen ist wahr. So soll euch das diesseitige Leben nicht täuschen, und nicht täuschen soll euch hinsichtlich Allahs der Täuscher.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdclxvi{Gewiß, der Satan ist euch ein Feind; so nehmt ihn euch zum Feind. Er ruft ja seine Anhängerschaft nur dazu auf, zu den Insassen der Feuerglut zu gehören.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdclxvii{Diejenigen, die ungläubig sind, für sie wird es strenge Strafe geben; und diejenigen, die glauben und rechtschaffene Werke tun, für sie wird es Vergebung und großen Lohn geben.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdclxviii{Soll denn derjenige, dem sein böses Tun ausgeschmückt wird und der es dann für schön befindet, (wie jemand sein, der rechtgeleitet ist)? Allah läßt in die Irre gehen, wen Er will, und Er leitet recht, wen Er will. Darum soll deine Seele nicht über sie in Bedauern zergehen, denn Allah weiß Bescheid über das, was sie machen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdclxix{Und Allah ist es, Der die Winde sendet, und da wühlen sie die Wolken auf. Dann treiben Wir sie zu einem toten Land und machen damit dann die Erde nach ihrem Tod wieder lebendig. Ebenso wird es auch mit der Auferstehung sein.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdclxx{Wenn immer einer die Macht anstrebt, so gehört alle Macht Allah. Zu Ihm steigt das gute Wort hinauf, und die rechtschaffene Tat hebt Er (zu sich) empor. Und für diejenigen, die Ränke böser Taten schmieden, wird es strenge Strafe geben, und die Ränke jener werden zu Fall kommen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdclxxi{Und Allah hat euch aus Erde, hierauf aus einem Samentropfen erschaffen und euch hierauf zu Paaren gemacht. Nicht schwanger wird ein weibliches Wesen noch kommt es nieder, außer mit Seinem Wissen. Und keinem, dem ein langes Leben gewährt wird, wird das Altwerden oder eine Verkürzung seiner Lebenszeit zuteil, ohne daß es in einem Buch (verzeichnet) wäre. Gewiß, dies ist Allah ein leichtes.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdclxxii{Nicht gleich sind die beiden Meere : dies ist süß, erfrischend und angenehm zu trinken; jenes ist salzig und (auf der Zunge) brennend. Und aus allen (beiden) eßt ihr frisches Fleisch und holt Schmuck heraus, den ihr anlegt. Und du siehst die Schiffe, wie sie es durchpflügen, damit ihr nach etwas von Seiner Huld trachtet, und auf daß ihr dankbar sein möget!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdclxxiii{Er läßt die Nacht in den Tag eindringen, und Er läßt den Tag in die Nacht eindringen. Und Er hat die Sonne und den Mond dienstbar gemacht - ein jedes läuft auf eine festgesetzte Frist zu. Dies ist doch Allah, euer Herr; Ihm gehört die Herrschaft. Diejenigen aber, die ihr anstatt Seiner anruft, verfügen nicht einmal über das Häutchen eines Dattelkerns.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdclxxiv{Wenn ihr sie anruft, hören sie euer Bittgebet nicht, und wenn sie (es) auch hörten, würden sie euch doch nicht erhören. Und am Tag der Auferstehung verleugnen sie, daß ihr (sie Allah) beigesellt habt. Keiner kann dir kundtun wie Einer, der Kenntnis von allem hat.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdclxxv{O ihr Menschen, ihr seid es, die Allahs bedürftig sind; Allah aber ist der Unbedürftige und Lobenswürdige.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdclxxvi{Wenn Er will, läßt Er euch vergehen und bringt eine neue Schöpfung.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdclxxvii{Und dies ist für Allah keineswegs schwer.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdclxxviii{Keine lasttragende (Seele) nimmt die Last einer anderen auf sich. Und wenn eine Schwerbeladene (zum Mittragen) ihrer Last aufruft, wird nichts davon (für sie) getragen, und handelte es sich dabei um einen Verwandten . Du kannst nur diejenigen warnen, die ihren Herrn im Verborgenen fürchten und das Gebet verrichten. Und wer sich läutert, läutert sich nur zu seinem eigenen Vorteil. Und zu Allah ist der Ausgang.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdclxxix{Nicht gleich sind der Blinde und der Sehende,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdclxxx{noch die Finsternisse und das Licht,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdclxxxi{noch der Schatten und die (Sonnen)hitze.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdclxxxii{Und nicht gleich sind auch die Lebenden und die Toten. Allah läßt hören, wen Er will. Und du vermagst nicht, diejenigen hören zu lassen, die in den Gräbern sind.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdclxxxiii{Du bist nur ein Warner.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdclxxxiv{Wir haben dich ja mit der Wahrheit gesandt als Frohboten und als Warner. Und es gibt keine Gemeinschaft, in der nicht ein Warner vorangegangen wäre.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdclxxxv{Und wenn sie dich der Lüge bezichtigen, so haben bereits diejenigen, die vor ihnen waren, (die Gesandten) der Lüge bezichtigt. Ihre Gesandten kamen zu ihnen mit den klaren Beweisen und mit den Schriften und den erleuchtenden Büchern.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdclxxxvi{Hierauf ergriff Ich diejenigen, die ungläubig waren. Wie war da Meine Mißbilligung!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdclxxxvii{Siehst du nicht, daß Allah vom Himmel Wasser herabkommen läßt? Damit bringen Wir dann Früchte von unterschiedlichen Farben hervor. Und von den Bergen gibt es Schichten, weiße und rote - von unterschiedlichen Farben - und rabenschwarze.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdclxxxviii{Und unter den Menschen und den Tieren und dem Vieh gibt es (auf ähnliche Weise) unterschiedliche Farben. So ist es. Allah fürchten von Seinen Dienern eben nur die Gelehrten. Gewiß, Allah ist Allmächtig und Allvergebend.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdclxxxix{Gewiß, diejenigen, die Allahs Buch verlesen, das Gebet verrichten und von dem, womit Wir sie versorgt haben, heimlich und öffentlich ausgeben, hoffen auf einen Handel, der nicht zu Fall kommen wird,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdcxc{damit Er ihnen ihren Lohn in vollem Maß zukommen lasse und ihnen von Seiner Huld noch mehr gebe. Gewiß, Er ist Allvergebend und stets zu Dank bereit.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdcxci{Und was Wir dir vom Buch (als Offenbarung) eingegeben haben, ist die Wahrheit, das zu bestätigen, was vor ihm war. Allah hat wahrlich Kenntnis von Seinen Dienern und sieht sie wohl.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdcxcii{Hierauf gaben Wir das Buch denjenigen von Unseren Dienern, die Wir auserwählten, zum Erbe. Mancher von ihnen tut sich selbst Unrecht, mancher von ihnen zeigt ein gemäßigtes Verhalten, und mancher von ihnen geht mit den guten Dingen mit Allahs Erlaubnis voran. Das ist die große Huld.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdcxciii{Die Gärten Edens (sind es), in die sie eingehen, worin sie mit Armreifen aus Gold und mit Perlen geschmückt sein werden, und ihre Kleidung wird darin aus Seide sein.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdcxciv{Und sie werden sagen: ``(Alles) Lob gehört Allah, Der den Kummer von uns hinweggenommen hat! Unser Herr ist wahrlich Allvergebend und stets zu Dank bereit.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdcxcv{Er), Der uns durch Seine Huld in die (ewig) bleibende Wohnstätte versetzt hat, in der uns keine Mühsal widerfährt und in der uns keine Ermüdung befällt."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdcxcvi{Diejenigen aber, die ungläubig sind, für sie wird es das Feuer der Hölle geben, wobei mit ihnen kein Ende gemacht wird, so daß sie sterben, und ihnen auch seine Strafe nicht erleichtert wird. So vergelten Wir jedem sehr Ungläubigen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdcxcvii{Und sie schreien darin laut: ``Unser Herr, bringe uns heraus, so werden wir rechtschaffen handeln, anders als das, was wir zu tun pflegten." - ``Haben Wir euch nicht ein so langes Leben gewährt, daß jeder, der hätte bedenken wollen, darin hätte bedenken können? Und ist nicht der Warner zu euch gekommen? So kostet nun, denn die Ungerechten haben keinen Helfer."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdcxcviii{Gewiß, Allah ist der Kenner des Verborgenen der Himmel und der Erde; gewiß, Er weiß über das Innerste der Brüste Bescheid.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdcxcix{Er ist es, Der euch zu Nachfolgern auf der Erde gemacht hat. Wer nun ungläubig ist, dessen Unglaube lastet auf ihm. Und den Ungläubigen mehrt ihr Unglaube nur noch die Abscheu bei ihrem Herrn; und den Ungläubigen mehrt ihr Unglaube nur noch den Verlust.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdcc{Sag: Was meint ihr zu euren Teilhabern, die ihr anstatt Allahs anruft? Zeigt mir, was sie von der Erde erschaffen haben. Oder haben sie etwa einen Anteil an den Himmeln? Oder haben Wir ihnen (sonst) eine Schrift gegeben, so daß sie sich von daher auf einen klaren Beweis stützten? Nein! Vielmehr versprechen die Ungerechten einander nur Trügerisches.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdcci{Allah hält die Himmel und die Erde, daß sie nicht vergehen. Und wenn sie vergehen würden, so könnte niemand nach Ihm sie halten. Gewiß, Er ist Nachsichtig und Allvergebend.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdccii{Und sie haben bei Allah ihren kräftigsten Eid geschworen, sie würden, wenn ein Warner zu ihnen käme, wahrlich eher rechtgeleitet sein als irgendeine der Ge meinschaften. Als dann aber ein Warner zu ihnen kam, mehrte es in ihnen nur (noch) die Abneigung,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdcciii{indem sie sich auf der Erde hochmütig verhielten und Ränke böser Taten schmiedeten. Aber die bösen Ränke umschließen nur ihre Urheber. Erwarten sie denn (für sich) etwas anderes als die Gesetzmäßigkeit (, nach) der (an den) Früheren (verfahren wurde)? Du wirst in Allahs Gesetzmäßigkeit keine Änderung finden, und du wirst in Allahs Gesetzmäßigkeit keine Abwandlung finden.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdcciv{Sind sie denn nicht auf der Erde umhergereist, so daß sie schauen (konnten), wie das Ende derjenigen war, die vor ihnen waren und die eine stärkere Kraft als sie hatten? Und Allah kann sich nichts entziehen - weder in den Himmeln noch auf der Erde, denn Er ist Allwissend und Allmächtig.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdccv{Und wenn Allah die Menschen für das belangen wollte, was sie verdient haben, würde Er auf ihrer Oberfläche kein Tier übriglassen. Aber Er stellt sie auf eine festgesetzte Frist zurück. Und wenn dann ihre Frist kommt (-, so wird Er sie belangen), denn gewiß, Allah sieht Seine Diener wohl.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdccvi{\basmalahde Ya-Sin.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdccvii{Bei dem weisen Qur'an,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdccviii{du bist wahrlich einer der Gesandten\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdccix{auf einem geraden Weg.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdccx{Er ist) die Offenbarung des Allmächtigen und Barmherzigen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdccxi{damit du ein Volk warnst, dessen Väter nicht gewarnt wurden, so daß sie (gegenüber allem) unachtsam sind.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdccxii{Das Wort ist ja gegen die meisten von ihnen unvermeidlich fällig geworden, so glauben sie nicht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdccxiii{Gewiß, Wir haben um ihre Hälse Fesseln gelegt. Sie reichen bis zum Kinn, so daß sie den Kopf hochhalten (müssen).\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdccxiv{Und Wir haben vor ihnen eine Sperrmauer errichtet und hinter ihnen eine Sperrmauer und sie so überdeckt, daß sie nicht sehen (können).\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdccxv{Und gleich ist es in Bezug auf sie, ob du sie warnst oder sie nicht warnst; sie glauben nicht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdccxvi{Du kannst nur jemanden warnen, der der Ermahnung folgt und den Allerbarmer im Verborgenen fürchtet. So verkünde ihm Vergebung und trefflichen Lohn.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdccxvii{Gewiß, Wir sind es, Die Wir die Toten wieder lebendig machen. Und Wir schreiben auf, was sie vorausgeschickt haben und (auch) ihre Spuren. Alles haben Wir in einem deutlichen Verzeichnis erfaßt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdccxviii{Und präge ihnen als Gleichnis die (Geschichte der) Bewohner der Stadt, als die Gesandten zu ihr kamen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdccxix{Als Wir zwei zu ihnen sandten, da bezichtigten sie beide der Lüge, und so brachten Wir durch einen dritten Verstärkung. Sie sagten: ``Gewiß, wir sind zu euch ge sandt."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdccxx{Sie sagten: ``Ihr seid nur menschliche Wesen wie wir. Nichts (als Offenbarung) hat der Allerbarmer herabgesandt. Ihr lügt nur."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdccxxi{Sie sagten: ``Unser Herr weiß (es), wir sind wirklich zu euch gesandt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdccxxii{Und uns obliegt nur die deutliche Übermittelung (der Botschaft)."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdccxxiii{Sie sagten: ``Wir sehen in euch ein böses Vorzeichen. Wenn ihr nicht aufhört, werden wir euch ganz gewiß steinigen, und euch wird ganz gewiß schmerzhafte Strafe von uns widerfahren."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdccxxiv{Sie sagten: ``Euer Vorzeichen ist bei euch (selbst). (Ist es ein böses Vorzeichen,) wenn ihr ermahnt werdet? Aber nein! Ihr seid maßlose Leute."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdccxxv{Und es kam vom äußersten Ende der Stadt ein Mann gelaufen. Er sagte: ``O mein Volk, folgt den Gesandten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdccxxvi{Folgt denjenigen, die von euch keinen Lohn verlangen und (selbst) rechtgeleitet sind.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdccxxvii{Und warum sollte ich nicht Demjenigen dienen, Der mich erschaffen hat und zu Dem ihr zurückgebracht werdet?\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdccxxviii{Soll ich mir etwa anstatt Seiner (andere) Götter nehmen? Wenn der Allerbarmer für mich Unheil will, nützt mir ihre Fürsprache nichts, und sie (können) mich nicht erretten.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdccxxix{Ich würde mich dann wahrlich in deutlichem Irrtum befinden.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdccxxx{Gewiß, ich glaube an euren Herrn, so hört auf mich."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdccxxxi{Es wurde (zu ihm) gesagt: ``Geh in den (Paradies)garten ein." Er sagte: ``O wüßte doch mein Volk davon,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdccxxxii{daß mein Herr mir vergeben und mich zu den Geehrten hat gehören lassen!"\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdccxxxiii{Wir sandten gegen sein Volk nach ihm keine Heerschar vom Himmel herab; Wir brauchten auch sonst nichts (gegen sie) herabzusenden.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdccxxxiv{Es war nur ein einziger Schrei (nötig), da waren sie sogleich ausgelöscht.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdccxxxv{O schade um die Diener! Es kommt kein Gesandter zu ihnen, ohne daß sie sich über ihn lustig machen würden.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdccxxxvi{Sehen sie denn nicht, wie viele Geschlechter Wir vor ihnen vernichtet haben, und daß sie zu ihnen nicht zurückkehren?\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdccxxxvii{Und sie werden alle insgesamt bei Uns vorgeführt werden.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdccxxxviii{Und ein Zeichen ist für sie die tote Erde. Wir machen sie lebendig und bringen aus ihr Körner hervor, so daß sie davon essen (können).\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdccxxxix{Und Wir haben auf ihr Gärten mit Palmen und Rebstöcken geschaffen und auf ihr Quellen hervorströmen lassen,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdccxl{damit sie von seinen Früchten essen (können). Nicht ihre Hände haben sie geschaffen. Wollen sie denn nicht dankbar sein?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdccxli{Preis sei Demjenigen, Der die Paare alle erschaffen hat von dem, was die Erde wachsen läßt, von ihnen selbst und von dem, was sie nicht wissen!\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdccxlii{Und ein Zeichen ist für sie die Nacht. Wir ziehen von ihr den Tag weg, und sogleich befinden sie sich im Finstern.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdccxliii{Und die Sonne läuft zu einem für sie bestimmten Aufenthaltsort. Das ist die Anordnung des Allmächtigen und Allwissenden.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdccxliv{Und dem Mond haben Wir das rechte Maß in Himmelspunkten festgesetzt, bis er wieder wie ein alter Dattelrispenstiel wird.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdccxlv{Weder ziemt es der Sonne, den Mond einzuholen, noch wird die Nacht dem Tag zuvorkommen; alle laufen in einer (jeweils eigenen) Umlaufbahn.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdccxlvi{Und ein Zeichen ist es für sie, daß Wir ihre Nachkommenschaft vollbeladenen Schiff trugen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdccxlvii{Und Wir haben ihnen etwas gleicher Art erschaffen, was sie besteigen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdccxlviii{Und wenn Wir wollen, lassen Wir sie ertrinken; dann gibt es niemanden, zu dem sie um Hilfe schreien könnten, und sie werden nicht gerettet,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdccxlix{außer aus Barmherzigkeit von Uns und als Nießbrauch auf Zeit.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdccl{Und wenn zu ihnen gesagt wird: ``Hütet euch vor dem, was vor euch, und dem, was hinter euch ist, auf daß ihr Erbarmen finden möget!" (..., wenden sie sich ab.)\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdccli{Kein Zeichen von den Zeichen ihres Herrn kommt zu ihnen, ohne daß sie sich davon abwenden.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdcclii{Und wenn zu ihnen gesagt wird: ``Gebt von dem aus, womit Allah euch versorgt hat", sagen diejenigen, die ungläubig sind, zu denjenigen, die glauben: ``Sollen wir jemanden ernähren, den Allah, wenn Er wollte, ernähren würde? Ihr befindet euch nur in deutlichem Irrtum."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdccliii{Und sie sagen: ``Wann wird dieses Versprechen eintreten, wenn ihr wahrhaftig seid?"\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdccliv{Sie haben nur einen einzigen Schrei zu erwarten, der sie ergreift, während sie noch miteinander streiten.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdcclv{Da werden sie kein Vermächtnis (mehr) hinterlassen können und auch nicht zu ihren Angehörigen zurückkehren.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdcclvi{Es wird ins Horn geblasen werden, und da laufen sie sogleich schnell aus den Gräbern zu ihrem Herrn herbei.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdcclvii{Sie sagen: ``O wehe uns! Wer hat uns von unserer Schlafstätte auferweckt? Das ist, was der Allerbarmer versprochen hat, und die Gesandten haben die Wahrheit gesagt."\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdcclviii{Es wird nur ein einziger Schrei (nötig) sein, da werden sie sogleich alle bei Uns vorgeführt.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdcclix{Heute wird keiner Seele irgendein Unrecht zugefügt. Und euch wird nur das vergolten, was ihr zu tun pflegtet.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdcclx{Gewiß, die Insassen des (Paradies)gartens sind heute in Beschäftigung und Vergnügen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmdcclxi{Sie und ihre Gattinnen befinden sich im Schatten und lehnen sich auf überdachte Liegen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmdcclxii{Sie haben darin Früchte, und sie haben, was sie erbeten.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmdcclxiii{``Friede!", als (Gruß)wort von einem barmherzigen Herrn.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmdcclxiv{``Sondert euch heute (von den Gottesfürchtigen) ab, ihr Übeltäter.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmdcclxv{Habe Ich euch, o Kinder Adams, nicht als Verpflichtung auferlegt, daß ihr nicht dem Satan dienen sollt - gewiß er ist euch ein deutlicher Feind -,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmdcclxvi{und daß ihr Mir dienen sollt?, - das ist ein gerader Weg!\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmdcclxvii{Er hat ja doch viele Geschöpfe von euch in die Irre geführt. Hattet ihr denn nicht begriffen?\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmdcclxviii{Das ist die Hölle, die euch stets angedroht wurde.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmdcclxix{Ihr sollt ihr heute ausgesetzt sein dafür, daß ihr ungläubig wart."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmdcclxx{Heute versiegeln Wir ihnen ihre Münder. Ihre Hände werden zu Uns sprechen und ihre Füße Zeugnis ablegen über das, was sie erworben haben.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmdcclxxi{Und wenn Wir wollten, würden Wir ihre Augen auslöschen. Und sie laufen dann zum Weg um die Wette, aber wie können sie da sehen?\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmdcclxxii{Wenn Wir wollten, würden Wir sie auf ihrer Stelle verwandeln, so daß sie nicht mehr vorangehen können und auch nicht zurückkehren.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmdcclxxiii{Wem Wir ein langes Leben gewähren, den lassen Wir in seiner Gestalt eine Kehrtwendung machen. Begreifen sie denn nicht?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmdcclxxiv{Und Wir haben ihn nicht das Dichten gelehrt, und es ziemt ihm (auch) nicht. Das ist doch nur eine Ermahnung und ein deutlicher Qur'an,\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmdcclxxv{damit er warne, wer (da) lebt, und das Wort gegen die Ungläubigen unvermeidlich fällig werde.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmdcclxxvi{Sehen sie denn nicht, daß Wir ihnen unter dem, was Unsere Hände gemacht haben, Vieh erschaffen haben, über das sie verfügen?\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmdcclxxvii{Und Wir haben es ihnen fügsam gemacht, so daß sie davon Reittiere haben und davon essen (können).\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmdcclxxviii{Und sie haben an ihm (allerlei) Nutzen und (etwas) zu trinken. Wollen sie denn nicht dankbar sein?\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmdcclxxix{Und sie haben sich anstatt Allahs Götter genommen, auf daß sie Hilfe finden mögen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmdcclxxx{Sie können ihnen jedoch keine Hilfe gewähren, obwohl sie ihnen als eine (dienstbereite) Heerschar vorgeführt werden.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxi{Ihre Worte sollen dich nicht traurig machen. Wir wissen ja, was sie geheimhalten und was sie offenlegen.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxii{Sieht denn der Mensch nicht, daß Wir ihn aus einem Samentropfen erschaffen haben, und doch ist er sogleich ein deutlicher Widersacher.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxiii{Er führt Uns ein Beispiel an und vergißt seine (eigene) Erschaffung. Er sagt: ``Wer macht die Knochen wieder lebendig, wenn sie zerfallen sind?"\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxiv{Sag: Wieder lebendig macht sie Derjenige, Der sie das erste Mal hat entstehen lassen. Und Er weiß über jede Schöpfung Bescheid,\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxv{Er,) Der euch aus grünen Bäumen Feuer gemacht hat, so daß ihr sogleich damit anzünden könnt.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxvi{Hat nicht Derjenige, Der die Himmel und die Erde erschaffen hat, (auch) die Macht, ihresgleichen zu erschaffen? Ja doch! Und Er ist der Allerschaffer und Allwissende.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxvii{Sein Befehl, wenn Er etwas will, ist, dazu nur zu sagen: ,Sei!', und so ist es.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxviii{So sei Preis Demjenigen, in Dessen Hand die Herrschaftsgewalt über alles ist und zu Dem ihr zurückgebracht werdet!\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxix{\basmalahde Bei den sich reihenweise Reihenden,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdccxc{dann den einen Schelteschrei Ausstoßenden,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdccxci{dann den eine Ermahnung Verlesenden!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdccxcii{Euer Gott ist wahrlich ein Einziger,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdccxciii{der Herr der Himmel und der Erde und dessen, was dazwischen ist, und der Herr der Osten.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdccxciv{Wir haben den untersten Himmel mit einem Schmuck geziert: mit den Himmelskörpern';\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdccxcv{und als Schutz vor jedem rebellischen Satan.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdccxcvi{So können sie der höchsten führenden Schar (der Engel) nicht zuhören, und sie werden von allen Seiten beworfen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdccxcvii{damit sie weggetrieben werden - und für sie wird es immerwährende Strafe geben -,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdccxcviii{außer jemandem, der rasch etwas aufschnappt und den dann ein durchbohrend heller Leuchtkörper verfolgt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdccxcix{Frage sie doch um ihre Meinung: Ist es etwa schwerer, sie zu erschaffen als wen Wir (sonst) erschaffen\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdccc{Aber nein! Du wunderst dich, während sie spotten.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdccci{Und wenn sie ermahnt werden, gedenken sie nicht.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdcccii{Und wenn sie ein Zeichen sehen, spotten sie darüber.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdccciii{Und sie sagen: ``Das ist nur deutliche Zauberei.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdccciv{Wenn wir gestorben und zu Erde und Knochen geworden sind, sollen wir denn dann wirklich auferweckt werden?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdcccv{Und auch unsere Vorväter?"\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdcccvi{Sag: Ja, und ihr werdet euch demütig verhalten.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdcccvii{Es wird nur ein einziger erschreckender Schrei sein, da schauen sie sogleich hin.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdcccviii{Und sie werden sagen: ``O wehe uns! Das ist der Tag des Gerichts!"\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdcccix{- ``Das ist der Tag der Entscheidung, den ihr für Lüge zu erklären pflegtet.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdcccx{- (Zu den Engeln des Gerichts wird gesagt:) Versammelt (nun) diejenigen, die Unrecht getan haben, ihre Gattinnen und das, dem sie dienten\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdcccxi{anstatt Allahs. Dann leitet sie zum Weg des Höllenbrandes\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdcccxii{und stellt sie auf, denn sie werden befragt werden."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdcccxiii{- (Und es wird zu ihnen gesagt:) ``Was ist mit euch, daß ihr einander nicht unterstützt?"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdcccxiv{Nein! Vielmehr ergeben sie sich heute.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdcccxv{Und sie wenden sich einander zu und fragen sich gegenseitig.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdcccxvi{Sie sagen: ``Ihr kamt doch zu uns (stets) von rechts her'."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdcccxvii{Sie sagen: ``Aber nein! Ihr wart ja nicht gläubig.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdcccxviii{Wir aber hatten keine Macht über euch, sondern vielmehr wart ihr Leute, die das Maß (an Frevel) überschreiten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdcccxix{So ist das Wort unseres Herrn gegen uns unvermeidlich fällig geworden. Wir werden bestimmt (die Strafe) kosten.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdcccxx{Und so haben wir euch in Verirrung fallen lassen, denn wir waren ja selbst in Verirrung gefallen."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdcccxxi{Gewiß denn, an jenem Tag werden sie an der(selben) Strafe teilhaben.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdcccxxii{Gewiß, so verfahren Wir mit den Übeltätern,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdcccxxiii{denn sie pflegten, wenn zu ihnen gesagt wurde: ``Es gibt keinen Gott außer Allah", sich hochmütig zu verhalten,\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdcccxxiv{und sagten: ``Sollen wir denn wahrlich unsere Götter verlassen wegen eines besessenen Dichters?"\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdcccxxv{Nein! Vielmehr ist er mit der Wahrheit gekommen und hat die Gesandten bestätigt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdcccxxvi{Ihr werdet bestimmt die schmerzhafte Strafe kosten,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdcccxxvii{und euch wird nur das vergolten, was ihr zu tun pflegtet,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdcccxxviii{außer Allahs auserlesenen Dienern.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdcccxxix{Für diese wird es eine festgesetzte Versorgung geben:\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdcccxxx{Früchte, und sie werden geehrt\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxi{in den Gärten der Wonne\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxii{auf Liegen (ruhend), einander gegenüber,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxiii{wobei ihnen ein Becher (voll) Quellwasser herumgereicht wird,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxiv{weiß, köstlich für diejenigen, die (daraus) trinken.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxv{Darin steckt nichts Beeinträchtigendes, und dadurch werden sie nicht benommen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxvi{Und bei ihnen sind solche (weiblichen Wesen), die ihre Blicke zurückhalten, mit schönen, großen Augen,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxvii{als ob sie wohlverwahrte Eier wären.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxviii{Sie wenden sich einander zu und fragen sich gegenseitig.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxix{Einer von ihnen sagt: ``Gewiß, ich hatte einen Gesellen,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdcccxl{der da sagte: .Gehörst du wirklich zu denjenigen, die es für wahr halten?\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdcccxli{Wenn wir gestorben und zu Erde und Knochen geworden sind, sollen wir denn dann wirklich vor Gericht gestellt werden'?"\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdcccxlii{Er sagt: ``Wollt ihr denn hinschauen?"\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdcccxliii{Er schaut selbst hin und sieht ihn mitten im Höllenbrand.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmdcccxliv{Er sagt: ``Bei Allah, beinahe hättest du mich fürwahr ins Verderben gestürzt.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmdcccxlv{Und ohne die Gnade meines Herrn würde ich nun wahrlich zu den Vorgeführten gehören.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmdcccxlvi{Werden wir tatsächlich nicht mehr sterben,\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmdcccxlvii{außer unserem ersten Tod, und gehören wir tatsächlich nicht zu den Bestraften?"\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmdcccxlviii{Das ist doch wahrlich der großartige Erfolg!\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmdcccxlix{Für ein Solches sollen diejenigen, die handeln, (ihre Werke) tun.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmdcccl{Ist dies als gastliche Aufnahme besser oder der Zaqqum-Baum?\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmdcccli{Den haben Wir zu einer Versuchung für die Ungerechten gemacht.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmdccclii{Er ist ein Baum, der im Grund des Höllenbrandes hervorkommt,\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmdcccliii{dessen Fruchtscheide so ist, als wären es Köpfe von Satanen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmdcccliv{Sie werden wahrlich davon essen und werden (sich) so davon die Bäuche füllen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmdccclv{Hierauf erhalten sie fürwahr eine Mischung von heißem Wasser.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmdccclvi{Hierauf kehren sie fürwahr zum Höllenbrand zurück.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmdccclvii{Sie haben ja ihre Väter im Irrtum vorgefunden,\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmdccclviii{und so sind sie auf ihren Spuren eilig getrieben worden.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmdccclix{Vor ihnen sind bereits die meisten der Früheren abgeirrt,\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmdccclx{und Wir sandten ja zu ihnen Überbringer von Warnungen.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmdccclxi{So schau, wie das Ende der Gewarnten war,\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmdccclxii{außer Allahs auserlesener Diener.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmdccclxiii{Und Nuh rief Uns ja bereits zu - welch trefflicher Erhörer sind Wir fürwahr!\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmdccclxiv{Und Wir erretteten ihn und seine Angehörigen aus der großen Trübsal\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmdccclxv{und machten seine Nachkommenschaft zu denjenigen, die Fortbestand haben.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmdccclxvi{Und Wir ließen für ihn (den Ruf) unter den späteren (Geschlechtern lauten):\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmdccclxvii{``Friede sei auf Nuh unter den Weltenbewohnern!"\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmdccclxviii{Gewiß, so vergelten Wir den Gutes Tuenden.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmdccclxix{Er gehört ja zu Unseren gläubigen Dienern.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmdccclxx{Hierauf ließen Wir die anderen ertrinken.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmdccclxxi{Zu seiner Gemeinde gehörte fürwahr Ibrahim.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmdccclxxii{Als er zu seinem Herrn mit heilem Herzen kam.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmdccclxxiii{Als er zu seinem Vater und seinem Volk sagte: ``Wem dient ihr da?\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmdccclxxiv{Wollt ihr außer Allah falsche Götter haben?\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmdccclxxv{Welche Meinung habt ihr denn vom Herrn der Weltenbewohner?"\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmdccclxxvi{Dann warf er einen Blick zu den Sternen\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmdccclxxvii{und sagte: ``Gewiß, ich bin krank."\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmdccclxxviii{Da kehrten sie ihm den Rücken.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmdccclxxix{Nun schlich er sich zu ihren Göttern und sagte: ``Wollt ihr nicht essen?\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmdccclxxx{Was ist mit euch, daß ihr nicht redet?"\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxi{Und er wandte sich gegen sie und schlug auf sie mit der Rechten ein.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxii{Da kamen sie auf ihn zugeeilt.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxiii{Er sagte: ``Wie könnt ihr denn dem dienen, was ihr selbst aushaut,\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxiv{wo doch Allah euch und das, was ihr tut, erschaffen hat?"\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxv{Sie sagten: ``Errichtet für ihn einen Bau und werft ihn in den Feuersbrand."\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxvi{Sie wollten mit einer List gegen ihn vorgehen, doch da machten Wir sie zu den Unterlegenen.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxvii{Er sagte: ``Gewiß, ich gehe zu meinem Herrn; Er wird mich rechtleiten.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxviii{Mein Herr, schenke mir einen von den Rechtschaffenen."\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxix{Da verkündeten Wir ihm einen nachsichtigen Jungen.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmmdcccxc{Als dieser das Alter erreichte, daß er mit ihm laufen konnte, sagte er: ``O mein lieber Sohn, ich sehe im Schlaf, daß ich dich schlachte. Schau jetzt, was du (dazu) meinst." Er sagte: ``O mein lieber Vater, tu, was dir befohlen wird. Du wirst mich, wenn Allah will, als einen der Standhaften finden."\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmmdcccxci{Als sie sich beide ergeben gezeigt hatten und er ihn auf die Seite der Stirn niedergeworfen hatte,\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmmdcccxcii{riefen Wir ihm zu: ``O Ibrahim,\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmmdcccxciii{du hast das Traumgesicht bereits wahr gemacht." Gewiß, so vergelten Wir den Gutes Tuenden.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmmdcccxciv{Das ist wahrlich die deutliche Prüfung.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmmdcccxcv{Und Wir lösten ihn mit einem großartigen Schlachtopfer aus.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmmdcccxcvi{Und Wir ließen für ihn (den Ruf) unter den späteren (Geschlechtern lauten):\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmmdcccxcvii{``Friede sei auf Ibrahim!"\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmmdcccxcviii{So vergelten Wir den Gutes Tuenden.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmmdcccxcix{Er gehört ja zu Unseren gläubigen Dienern.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmmcm{Und Wir verkündeten ihm Ishaq als einen Propheten von den Rechtschaffenen.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmmcmi{Und Wir segneten ihn und Ishaq. Unter ihrer Nachkommenschaft gibt es manche, die Gutes tun, und manche, die sich selbst offenkundig Unrecht zufügen.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmmcmii{Und Wir erwiesen bereits Musa und Harun eine Wohltat\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmmcmiii{und erretteten sie beide und ihr Volk aus der großen Trübsal.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmmcmiv{Und Wir halfen ihnen, da waren sie es, die Sieger wurden.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmmcmv{Und Wir gaben ihnen beiden die verdeutlichende Schrift\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmmcmvi{und leiteten sie den geraden Weg.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmmcmvii{Und Wir ließen für beide (den Ruf) unter den späteren (Geschlechtern lauten):\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmmcmviii{``Friede sei auf Musa und Harun!"\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmmcmix{Gewiß, so vergelten Wir den Gutes Tuenden.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmmcmx{Sie beide gehören ja zu Unseren gläubigen Dienern.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmmcmxi{Auch Ilyas gehörte wahrlich zu den Gesandten.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmmcmxii{Als er zu seinem Volk sagte: ``Wollt ihr nicht gottesfürchtig sein?\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmmcmxiii{Wie könnt ihr Bacl anrufen und den besten Schöpfer verlassen,\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmmcmxiv{Allah, euren Herrn und den Herrn eurer Vorväter?"\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmmcmxv{Da bezichtigten sie ihn der Lüge. So werden sie bestimmt vorgeführt werden,\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmmcmxvi{außer Allahs auserlesenen Dienern.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmmcmxvii{Und Wir ließen für ihn (den Ruf) unter den späteren (Geschlechtern lauten):\qt@no{129}}%
\qt@newcmd\qurantrans@de@mmmcmxviii{``Friede sei auf Ilyas!"\qt@no{130}}%
\qt@newcmd\qurantrans@de@mmmcmxix{Gewiß, so vergelten Wir den Gutes Tuenden.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mmmcmxx{Er gehört ja zu Unseren gläubigen Dienern.\qt@no{132}}%
\qt@newcmd\qurantrans@de@mmmcmxxi{Auch Lut gehörte wahrlich zu den Gesandten.\qt@no{133}}%
\qt@newcmd\qurantrans@de@mmmcmxxii{Als Wir ihn und seine Angehörigen allesamt erretteten,\qt@no{134}}%
\qt@newcmd\qurantrans@de@mmmcmxxiii{außer einer alten Frau unter denjenigen, die zurückblieben.\qt@no{135}}%
\qt@newcmd\qurantrans@de@mmmcmxxiv{Hierauf vertilgten Wir die anderen.\qt@no{136}}%
\qt@newcmd\qurantrans@de@mmmcmxxv{Ihr kommt ja an ihnen vorbei, bei Tagesanbruch\qt@no{137}}%
\qt@newcmd\qurantrans@de@mmmcmxxvi{und bei Nacht. Begreift ihr denn nicht?'\qt@no{138}}%
\qt@newcmd\qurantrans@de@mmmcmxxvii{Auch Yunus gehörte wahrlich zu den Gesandten.\qt@no{139}}%
\qt@newcmd\qurantrans@de@mmmcmxxviii{Als er zum vollbeladenen Schiff davonlief.\qt@no{140}}%
\qt@newcmd\qurantrans@de@mmmcmxxix{Er warf Lose und wurde einer der Unterlegenen.\qt@no{141}}%
\qt@newcmd\qurantrans@de@mmmcmxxx{Da verschlang ihn der (große) Fisch", während er sich Tadel zugezogen hatte.\qt@no{142}}%
\qt@newcmd\qurantrans@de@mmmcmxxxi{Und hätte er nicht zu denjenigen gehört, die (Allah) preisen,\qt@no{143}}%
\qt@newcmd\qurantrans@de@mmmcmxxxii{hätte er wahrlich in seinem Bauch verweilt bis zu dem Tag, an dem sie auferweckt werden.\qt@no{144}}%
\qt@newcmd\qurantrans@de@mmmcmxxxiii{Da warfen Wir ihn auf das kahle Land -, und dabei war er krank.\qt@no{145}}%
\qt@newcmd\qurantrans@de@mmmcmxxxiv{Und Wir ließen eine Kürbisstaude über ihm wachsen.\qt@no{146}}%
\qt@newcmd\qurantrans@de@mmmcmxxxv{Und Wir sandten ihn zu Hunderttausend oder sogar noch mehr.\qt@no{147}}%
\qt@newcmd\qurantrans@de@mmmcmxxxvi{Da glaubten sie, und so gewährten Wir ihnen Nießbrauch auf Zeit.\qt@no{148}}%
\qt@newcmd\qurantrans@de@mmmcmxxxvii{Frage sie doch nach ihrer Meinung: Gehören deinem Herrn etwa die Töchter und ihnen die Söhne?\qt@no{149}}%
\qt@newcmd\qurantrans@de@mmmcmxxxviii{Oder haben Wir die Engel als weibliche Wesen erschaffen, während sie anwesend waren?\qt@no{150}}%
\qt@newcmd\qurantrans@de@mmmcmxxxix{Dabei ist es fürwahr aus ihrer ungeheuren Lügenhaftigkeit, daß sie sagen:\qt@no{151}}%
\qt@newcmd\qurantrans@de@mmmcmxl{``Allah hat gezeugt." Wahrlich, sie sind Lügner.\qt@no{152}}%
\qt@newcmd\qurantrans@de@mmmcmxli{Hat Er die Töchter vor den Söhnen auserwählt?\qt@no{153}}%
\qt@newcmd\qurantrans@de@mmmcmxlii{Was ist mit euch? Wie urteilt ihr?\qt@no{154}}%
\qt@newcmd\qurantrans@de@mmmcmxliii{Bedenkt ihr denn nicht?\qt@no{155}}%
\qt@newcmd\qurantrans@de@mmmcmxliv{Oder habt ihr eine deutliche Ermächtigung?\qt@no{156}}%
\qt@newcmd\qurantrans@de@mmmcmxlv{Bringt doch eure Schrift bei, wenn ihr wahrhaftig seid.\qt@no{157}}%
\qt@newcmd\qurantrans@de@mmmcmxlvi{Und sie stellen zwischen Ihm und den Ginn eine Verwandtschaft her. Aber die Ginn wissen ja, daß sie bestimmt vorgeführt werden\qt@no{158}}%
\qt@newcmd\qurantrans@de@mmmcmxlvii{Preis sei Allah! (Und Erhaben ist Er) über das, was sie (Ihm) zuschreiben -,\qt@no{159}}%
\qt@newcmd\qurantrans@de@mmmcmxlviii{außer Allahs auserlesenen Dienern.\qt@no{160}}%
\qt@newcmd\qurantrans@de@mmmcmxlix{Ihr aber und das, dem ihr dient,\qt@no{161}}%
\qt@newcmd\qurantrans@de@mmmcml{könnt niemanden gegen Ihn verfuhren,\qt@no{162}}%
\qt@newcmd\qurantrans@de@mmmcmli{außer denjenigen, der dem Höllenbrand ausgesetzt wird.\qt@no{163}}%
\qt@newcmd\qurantrans@de@mmmcmlii{Und es gibt niemanden unter uns, der nicht einen bestimmten Platz hätte.\qt@no{164}}%
\qt@newcmd\qurantrans@de@mmmcmliii{Wahrlich, wir sind es, die sich (reihenweise) reihen,\qt@no{165}}%
\qt@newcmd\qurantrans@de@mmmcmliv{und wahrlich, wir sind es, die preisen.\qt@no{166}}%
\qt@newcmd\qurantrans@de@mmmcmlv{Und sie pflegten zu sagen:\qt@no{167}}%
\qt@newcmd\qurantrans@de@mmmcmlvi{``Wenn wir nur eine Ermahnung (gleich derjenigen) der Früheren hätten,\qt@no{168}}%
\qt@newcmd\qurantrans@de@mmmcmlvii{dann wären wir fürwahr Allahs auserlesene Diener."\qt@no{169}}%
\qt@newcmd\qurantrans@de@mmmcmlviii{Dennoch verleugnen sie Ihn. Aber sie werden (es noch) erfahren.\qt@no{170}}%
\qt@newcmd\qurantrans@de@mmmcmlix{Es erging bereits früher Unser Wort an Unsere gesandten Diener:\qt@no{171}}%
\qt@newcmd\qurantrans@de@mmmcmlx{Wahrlich, sie sind es, denen Hilfe zuteil wird.\qt@no{172}}%
\qt@newcmd\qurantrans@de@mmmcmlxi{Und wahrlich, Unsere Heerschar ist es, die Sieger sein wird.\qt@no{173}}%
\qt@newcmd\qurantrans@de@mmmcmlxii{So kehre dich für eine gewisse Zeit von ihnen ab\qt@no{174}}%
\qt@newcmd\qurantrans@de@mmmcmlxiii{und sieh sie dir an. Auch sie werden sehen.\qt@no{175}}%
\qt@newcmd\qurantrans@de@mmmcmlxiv{Wünschen sie denn Unsere Strafe zu beschleunigen?\qt@no{176}}%
\qt@newcmd\qurantrans@de@mmmcmlxv{Wenn sie sich aber in ihre Höfe niederläßt, wie böse wird dann der Morgen der Gewarnten sein!\qt@no{177}}%
\qt@newcmd\qurantrans@de@mmmcmlxvi{Und kehre dich für eine gewisse Zeit von ihnen ab\qt@no{178}}%
\qt@newcmd\qurantrans@de@mmmcmlxvii{und sieh zu. Auch sie werden sehen.\qt@no{179}}%
\qt@newcmd\qurantrans@de@mmmcmlxviii{Preis sei deinem Herrn, dem Herrn der Macht! (Erhaben ist Er) über das, was sie (Ihm) zuschreiben.\qt@no{180}}%
\qt@newcmd\qurantrans@de@mmmcmlxix{Und Friede sei auf den Gesandten!\qt@no{181}}%
\qt@newcmd\qurantrans@de@mmmcmlxx{Und (alles) Lob gehört Allah, dem Herrn der Weltenbewohner!\qt@no{182}}%
\qt@newcmd\qurantrans@de@mmmcmlxxi{\basmalahde Sad. Beim Qur'an, voll der Ermahnung!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcmlxxii{Aber nein! Diejenigen, die ungläubig sind, befinden sich in (falschem) Stolz und in Widerstreit.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcmlxxiii{Wie viele Geschlechter haben Wir vor ihnen vernichtet! Sie riefen, doch da war keine Zeit mehr zum Entrinnen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcmlxxiv{Sie wundern sich darüber, daß zu ihnen ein Überbringer von Warnungen von ihnen gekommen ist. Die Ungläubigen sagen: ``Dies ist ein verlogener Zauberer.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcmlxxv{Macht er denn die Götter zu einem einzigen Gott? Das ist fürwahr etwas sehr Verwunderliches."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcmlxxvi{Und so ging die führende Schar unter ihnen fort: ``Geht hin und haltet an euren Göttern beharrlich fest. Das ist fürwahr etwas, was (zu eurem Nachteil) gewollt wird.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcmlxxvii{Wir haben hiervon nicht in dem vorherigen Glaubensbekenntnis gehört; dies ist nur eine Erfindung.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcmlxxviii{Soll die Ermahnung (ausgerechnet) auf ihn aus unserer Mitte herabgesandt worden sein?" Nein! Vielmehr sind sie über Meine Ermahnung im Zweifel. Nein! Vielmehr haben sie noch nicht Meine Strafe gekostet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcmlxxix{Oder besitzen sie etwa die Schatzkammern der Barmherzigkeit deines Herrn, des Allmächtigen, des unablässig Schenkenden?\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcmlxxx{Oder gehört (etwa) ihnen die Herrschaft der Himmel und der Erde und dessen, was dazwischen ist? So sollen sie doch an Seilen emporsteigen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxi{Manch eine Heerschar von den Gruppierungen wird da geschlagen stehen!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxii{Der Lüge bezichtigten (ihre Gesandten) schon vor ihnen das Volk Nuhs, die 'Ad und Fir'aun, der Besitzer der Pfähle,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxiii{und die Tamud und das Volk Luts und die Bewohner des Dickichts; das sind die Gruppierungen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxiv{Al]e ausnahmslos bezichtigten die Gesandten der Lüge, so wurde Meine Bestra fung unvermeidlich fällig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxv{Diese erwarten wohl nur einen einzigen Schrei, der keine Unterbrechung hat.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxvi{Und sie sagen: ``Unser Herr, gib uns schnell unser Buch noch vor dem Tag der Abrechnung."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxvii{Ertrage standhaft, was sie sagen. Und gedenke Unseres Dieners Dawud, des Kraftvollen. Gewiß, er war immer wieder umkehrbereit.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxviii{Wir machten ja die Berge dienstbar, daß sie mit ihm zusammen abends und bei Sonnenaufgang (Allah) preisen,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxix{und auch die (in Scharen) versammelten Vögel. Alle waren immer wieder zu ihm umkehrbereit.'\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcmxc{Und Wir festigten seine Herrschaft und gaben ihm die Weisheit und die Entschiedenheit im Spruch.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcmxci{Ist zu dir die Kunde von den Widersachern gekommen? Als sie über die Mauern in die Andachtsräume einstiegen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcmxcii{Als sie bei Dawud eintraten. Da erschrak er vor ihnen. Sie sagten: ``Fürchte dich nicht. (Wir sind) zwei Widersacher, von denen der eine den anderen unterdrückt hat. So urteile zwischen uns der Wahrheit entsprechend, handle nicht ungerecht und führe uns zum rechten Weg.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcmxciii{Dieser da, mein Bruder, hat neunundneunzig weibliche Schafe, ich aber (nur) ein einziges Schaf. Dann sagte er: .Vertraue es mir an', und er überwand mich in der Rede."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcmxciv{Er sagte: ``Er hat dir Unrecht getan, daß er dein Schaf zu seinen Schafen hinzu verlangte. Gewiß, viele von den Teilhabern begehen gegeneinander Übergriffe, außer denjenigen, die glauben und rechtschaffene Werke tun - und das sind nur wenige." Und Dawud verstand, daß Wir ihn nur der Versuchung ausgesetzt hatten. Da bat er seinen Herrn um Vergebung und fiel in Verbeugung nieder und wandte sich (Ihm) reuig zu.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcmxcv{Und so vergaben Wir ihm dies. Für ihn wird es fürwahr (den Zutritt in) Unsere Nähe und eine schöne Heimstatt geben.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcmxcvi{O Dawud, Wir haben dich zu einem Statthalter auf der Erde gemacht. So richte zwischen den Menschen der Wahrheit entsprechend und folge nicht der Neigung, auf daß sie dich nicht von Allahs Weg abirren läßt, denn für diejenigen, die von Allahs Weg abirren, wird es strenge Strafe dafür geben, daß sie den Tag der Abrechnung vergessen haben.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcmxcvii{Und Wir haben den Himmel und die Erde und das, was dazwischen ist, nicht umsonst erschaffen. Das ist die Meinung derjenigen, die ungläubig sind. Doch wehe denjenigen, die ungläubig sind, vor dem (Höllen)feuer!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcmxcviii{Oder sollen Wir etwa diejenigen, die glauben und rechtschaffene Werke tun, den Unheilstiftern auf der Erde gleichstellen oder die Gottesfürchtigen den Lasterhaften?\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcmxcix{Dies ist) ein gesegnetes Buch, das Wir zu dir hinabgesandt haben, damit sie über seine Zeichen nachsinnen und damit diejenigen bedenken, die Verstand besitzen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmm{Und Wir schenkten Dawud Sulaiman. Welch ein trefflicher Diener! Er war immer wieder umkehrbereit.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmi{Als ihm am Abend die edlen, schnellen Pferde vorgeführt wurden,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmii{da sagte er: ``Ich habe mich der Liebe der (irdischen) Güter hingegeben und es darüber unterlassen, meines Herrn zu gedenken, bis sie sich hinter dem Vorhang verbarg.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmiii{Bringt sie mir wieder her." Da begann er, ihnen die Beine und den Hals zu zerhauen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmiv{Und Wir setzten ja Sulaiman der Versuchung aus und setzten eine Gestalt auf seinen Thron. Hierauf wandte er sich (Allah) reuig zu.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmv{Er sagte: ``Mein Herr, vergib mir und schenke mir eine Königsherrschaft, die niemandem nach mir geziemt. Gewiß, Du bist ja der unablässig Schenkende."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmvi{Da machten Wir ihm den Wind dienstbar, daß er nach seinem Befehl sanft wehte, wohin er es für treffend hielt;\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmvii{und auch die Satane, jeden Bauarbeiter und Taucher,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmviii{und andere, in Ketten zusammengebunden.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmix{``Das ist Unsere Gabe. So erweise Wohltaten oder sei zurückhaltend, (und dies,) ohne zu rechnen."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmx{Für ihn wird es fürwahr (den Zutritt in) Unsere Nähe und eine schöne Heimstatt geben.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmxi{Und gedenke Unseres Dieners Ayyub. Als er zu seinem Herrn rief: ``Mich hat der Satan mit Mühsal und Pein heimgesucht."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmxii{- ``Tritt kräftig mit deinem Fuß auf; da ist kühles Wasser zum Waschen und zum Trinken."\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmxiii{Und Wir schenkten ihm seine Angehörigen (wieder) und noch einmal die gleiche Zahl dazu, aus Barmherzigkeit von Uns und als Ermahnung für diejenigen, die Ver stand besitzen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmxiv{Und: ``Nimm in deine Hand ein Bündel (dünner Zweige) und schlag damit zu und sei nicht eidbrüchig." Gewiß, Wir fanden ihn standhaft. Welch ein trefflicher Diener! Er war immer wieder umkehrbereit.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmxv{Und gedenke Unserer Diener Ibrahim, Ishaq und Ya'qub, die Kraft und Einsicht besaßen.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmxvi{Wir erlasen sie durch eine besondere Eigenschaft aus, mit dem Gedenken an die (jenseitige) Wohnstätte.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmxvii{Sie gehören bei Uns wahrlich zu den Auserwählten und Besten.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmxviii{Und gedenke Isma'ils, Alyasa's und Du '1-Kifls. Alle gehören zu den Besten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmxix{Dies ist eine Ermahnung. Und für die Gottesfürchtigen wird es wahrlich eine schöne Heimstatt geben,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmxx{die Gärten Edens, deren Tore ihnen geöffnet stehen,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmxxi{worin sie sich lehnen und worin sie nach vielen Früchten und nach Getränk rufen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmxxii{Und bei ihnen sind gleichaltrige (weibliche Wesen), die ihre Blicke zurückhalten.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmxxiii{Das ist, was euch für den Tag der Abrechnung versprochen wird.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmxxiv{Das ist fürwahr Unsere Versorgung; sie geht nicht zu Ende.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmxxv{Das ist (für die Gottesfürchtigen). Für diejenigen jedoch, die das Maß (an Frevel) überschreiten, wird es wahrlich eine schlechte Heimstatt geben,\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmxxvi{die Hölle, der sie ausgesetzt sein werden - eine schlimme Lagerstatt!\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmxxvii{Das ist - sie sollen es kosten - heißes Wasser und stinkender Eiter\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmxxviii{und anderes dergleichen in verschiedenen Arten.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmxxix{- ``Da ist eine Schar, die sich mit euch hineinstürzt." - ``Sie sind nicht willkommen, denn sie werden dem (Höllen)feuer ausgesetzt sein."\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmxxx{Sie sagen: ``Nein! Vielmehr seid ihr es, die ihr nicht willkommen seid. Das habt ihr uns doch bereitet - ein schlimmer Aufenthalt!"\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmxxxi{Sie sagen: ``Unser Herr, erhöhe demjenigen, der uns dies bereitet hat, das (Strafmaß) als mehrfache Strafe im (Höllen)feuer."\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmxxxii{Und sie sagen: ``Wie kommt das, daß wir (gewisse) Männer nicht sehen, die wir zu den Bösewichten zählten?\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmxxxiii{Haben wir sie (etwa zu Unrecht) zum Gegenstand des Spottes genommen? Oder haben die Blicke sie verfehlt?"\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmxxxiv{Das ist tatsächlich wahr: (so ist) der Streit der Bewohner des (Höllen)feuers miteinander.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmxxxv{Sag: Ich bin nur ein Überbringer von Warnungen, und keinen Gott gibt es außer Allah, dem Einen, dem Allbezwinger,\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmxxxvi{dem Herrn der Himmel und der Erde und dessen, was dazwischen ist, dem Allmächtigen und Allvergebenden.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmxxxvii{Sag: Es ist eine gewaltige Kunde,\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmxxxviii{von der ihr euch abwendet.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmxxxix{Ich hatte kein Wissen über die höchste führende Schar', als sie miteinander stritten.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmxl{Mir wird ja (als Offenbarung) eingegeben, daß ich nur ein deutlicher Warner bin.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmxli{Als dein Herr zu den Engeln sagte: ``Ich werde ein menschliches Wesen aus Lehm erschaffen.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmxlii{Wenn Ich es zurechtgeformt und ihm von Meinem Geist eingehaucht habe, dann fallt und werft euch vor ihm nieder."\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmxliii{Da warfen sich die Engel alle zusammen nieder,\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmxliv{außer Iblis; er verhielt sich hochmütig und gehörte zu den Ungläubigen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmxlv{Er sagte: ``O Iblis, was hat dich davon abgehalten, dich vor dem niederzuwerfen, was Ich mit Meinen Händen erschaffen habe? Verhältst du dich hochmütig, oder gehörst du etwa zu den Überheblichen?"\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmxlvi{Er sagte: ``Ich bin besser als er. Mich hast Du aus Feuer erschaffen, wohingegen Du ihn aus Lehm erschaffen hast."\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmxlvii{Er sagte: ``Dann geh aus ihm hinaus, denn du bist der Steinigung würdig,\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmxlviii{und gewiß, auf dir liegt Mein Fluch bis zum Tag des Gerichts."\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmxlix{Er sagte: ``Mein Herr, gewähre mir Aufschub bis zu dem Tag, da sie auferweckt werden."\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmml{Er sagte: ``Gewiß, du gehörst zu denjenigen, denen Aufschub gewährt wird,\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmli{bis zum Tag der (wohl)bekannten Zeit."\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmlii{Er sagte: ``Nun, bei Deiner Macht, ich werde sie allesamt ganz gewiß in Verirrung fallen lassen,\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmliii{außer Deinen Dienern, den auserlesenen unter ihnen."\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmliv{Er sagte: ``Es ist die Wahrheit - und Ich sage ja die Wahrheit;\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmlv{Ich werde die Hölle ganz gewiß mit dir und mit all denjenigen von ihnen füllen, die dir folgen."\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmlvi{Sag: Ich verlange von euch keinen Lohn dafür. Und ich gehöre nicht zu denjenigen, die Unzumutbares auferlegen.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmmlvii{Es ist nur eine Ermahnung für die Weltenbewohner.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmmlviii{Und ihr werdet die Kunde darüber ganz bestimmt nach einer gewissen Zeit erfahren.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmmlix{\basmalahde (Dies ist) die Offenbarung des Buches von Allah, dem Allmächtigen und Allweisen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmlx{Wir haben das Buch mit der Wahrheit zu dir hinabgesandt. So diene Allah und sei Ihm gegenüber aufrichtig in der Religion.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmlxi{Sicherlich, Allah gehört die aufrichtige Religion. Diejenigen aber, die sich anstatt Seiner Schutzherren nehmen (, sagen:) ``Wir dienen ihnen nur, damit sie uns Zutritt in Allahs Nähe verschaffen" -, gewiß, Allah wird zwischen ihnen richten über das, worüber sie uneinig sind. Gewiß, Allah leitet nicht recht, wer ein Lügner und be harrlicher Ungläubiger ist.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmlxii{Wenn Allah sich hätte Kinder nehmen wollen, hätte Er wahrlich aus dem, was Er erschaffen hat, sich auserwählt, was Er will. Preis sei Ihm! Er ist Allah, der Eine, der Allbezwinger.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmlxiii{Er hat die Himmel und die Erde in Wahrheit erschaffen. Er läßt die Nacht den Tag umschlingen, und Er läßt den Tag die Nacht umschlingen. Und Er hat die Sonne und den Mond dienstbar gemacht - ein jedes läuft auf eine festgesetzte Frist zu. Sicherlich, Er ist der Allmächtige und Allvergebende.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmlxiv{Er schuf euch aus einem einzigen Wesen, hierauf machte Er aus ihm seine Gattin. Und Er hat für euch an Vieh acht (Tiere) in Paaren herabgesandt. Er erschafft euch in den Leibern eurer Mütter, eine Schöpfung nach der anderen in dreifacher Finsternis. Dies ist doch Allah, euer Herr. Ihm gehört die Herrschaft. Es gibt keinen Gott außer Ihm. Wie laßt ihr euch also abwenden?\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmlxv{Wenn ihr ungläubig seid, so ist Allah eurer unbedürftig, obgleich Er mit dem Unglauben für Seine Diener nicht zufrieden ist. Wenn ihr aber dankbar seid, ist Er damit zufrieden für euch. Und keine lasttragende (Seele) nimmt die Last einer anderen auf sich. Hierauf wird eure Rückkehr zu eurem Herrn sein. Dann wird Er euch kundtun, was ihr zu tun pflegtet. Gewiß, Er weiß über das Innerste der Brüste Bescheid.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmlxvi{Und wenn dem Menschen Unheil widerfährt, ruft er seinen Herrn an, indem er sich Ihm reuig zuwendet. Wenn Er ihm hierauf Gunst von Sich gewährt, vergißt er, worum er zuvor angerufen hat, und er stellt Allah andere als Seinesgleichen zur Seite, um (die Menschen) von Seinem Weg ab in die Irre zu führen. Sag: Genieße deinen Unglauben ein wenig; du gehörst ja zu den Insassen des (Höllen)feuers.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmlxvii{Ist etwa einer, der sich zu (verschiedenen) Stunden der Nacht in demütiger Andacht befindet, (ob er) sich niederwirft oder aufrecht steht, der sich vor dem Jenseits vorsieht und auf seines Herrn Barmherzigkeit hofft ...? - Sag: Sind etwa dieje nigen, die wissen, und diejenigen, die nicht wissen, gleich? Doch bedenken nur diejenigen, die Verstand besitzen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmlxviii{Sag: O meine Diener, die ihr gläubig seid, fürchtet euren Herrn. Für diejenigen, die Gutes tun, gibt es hier im Diesseits Gutes. Und Allahs Erde ist weit. Gewiß, den Standhaften wird ihr Lohn ohne Berechnung in vollem Maß zukommen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmlxix{Sag: Mir ist befohlen worden, Allah zu dienen und dabei Ihm gegenüber aufrichtig in der Religion zu sein.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmlxx{Und mir ist befohlen worden, der erste der (Ihm) Ergebenen zu sein.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmlxxi{Sag: Gewiß, ich fürchte, wenn ich mich meinem Herrn widersetze, die Strafe eines gewaltigen Tages.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmlxxii{Sag: Allah (allein) diene ich und bin dabei Ihm gegenüber aufrichtig in meiner Religion.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmlxxiii{So dient nun, wem ihr wollt, anstatt Seiner. Sag: Die (wahren) Verlierer sind diejenigen, die am Tag der Auferstehung sich selbst und ihre Angehörigen verlieren. Sicherlich, das ist der deutliche Verlust.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmlxxiv{Sie haben über sich Schattendächer von Feuer und unter sich (ebensolche) Schattendächer. Damit flößt Allah Seinen Dienern Furcht ein: ``O Meine Diener, fürchtet Mich also."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmlxxv{Diejenigen aber, die die falschen Götter meiden, um ihnen nicht zu dienen, und sich Allah reuig zuwenden, für sie gibt es die frohe Botschaft. So verkünde frohe Botschaft Meinen Dienern,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmlxxvi{die auf das Wort hören und dann dem Besten davon folgen. Das sind diejenigen, die Allah rechtleitet, und das sind diejenigen, die Verstand besitzen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmlxxvii{Kannst du etwa jemanden retten, der im (Höllen)feuer ist... -jemanden, gegen den der Spruch der Strafe unvermeidlich fällig geworden ist?\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmlxxviii{Aber diejenigen, die ihren Herrn fürchten, für sie wird es Obergemächer geben, über denen (andere) Obergemächer gebaut sind, unterhalb derer Flüsse strömen. (Das ist) Allahs Versprechen. Allah bricht nicht, was Er versprochen hat.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmlxxix{Siehst du nicht, daß Allah vom Himmel Wasser herabkommen und es dann als Quellen die Erde durchziehen läßt? Hierauf bringt Er damit Pflanzen von unterschiedlichen Arten hervor. Hierauf läßt Er sie austrocknen, und da siehst du sie gelb werden. Hierauf macht Er sie zu zermalmtem Zeug. Darin ist wahrlich eine Ermahnung für diejenigen, die Verstand besitzen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmlxxx{Ist denn jemand, dessen Brust Allah für den Islam auftut, so daß er sich nach einem Licht von seinem Herrn richtet, (einem beharrlich Ungläubigen gleich)? So wehe denjenigen, deren Herzen gegen Allahs Ermahnung verhärtet sind! Sie befinden sich in deutlichem Irrtum.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmlxxxi{Allah hat die beste Botschaft offenbart, ein Buch mit gleichartigen, sich wiederholenden (Versen), vor dem die Haut derjenigen, die ihren Herrn fürchten, erschauert. Hierauf werden ihre Haut und ihr Herz weich (und neigen sich) zu Allahs Gedenken hin. Das ist Allahs Rechtleitung. Er leitet damit recht, wen Er will. Und wen Allah in die Irre gehen läßt, der hat niemanden, der ihn rechtleitet.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmlxxxii{Ist denn jemand, der sich mit seinem Gesicht vor der bösen Strafe am Tag der Auferstehung hütet, ...? Und es wird zu den Ungerechten gesagt: ``Kostet, was ihr erworben habt."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmlxxxiii{Für Lüge erklärt haben es diejenigen, die vor ihnen waren. Da kam die Strafe über sie, von wo sie nicht merkten.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmlxxxiv{So ließ Allah sie im diesseitigen Leben die Schande kosten; aber die Strafe des Jenseits ist wahrlich größer, wenn sie nur wüßten!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmlxxxv{Und Wir haben ja den Menschen in diesem Qur'an allerlei Gleichnisse geprägt, auf daß sie bedenken mögen -,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmlxxxvi{in diesem) arabischen Qur'an, an dem nichts Krummes ist, - auf daß sie gottesfürchtig werden mögen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmlxxxvii{Allah prägt als Gleichnis dasjenige von einem Mann, in dem sich (mehrere) Herren (als Eigentümer) teilen, die sich miteinander nicht vertragen, und einem Mann, der nur einem Herrn gehört. Sind die beiden im Gleichnis etwa gleich? (Alles) Lob gehört Allah! Aber die meisten von ihnen wissen nicht.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmlxxxviii{Du wirst gewiß sterben, und auch sie werden sterben.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmlxxxix{Hierauf werdet ihr am Tag der Auferstehung bei eurem Herrn miteinander streiten.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmxc{Wer ist denn ungerechter, als wer gegen Allah lügt und die Wahrheit, wenn sie zu ihm kommt, für Lüge erklärt? Ist nicht in der Hölle ein Aufenthaltsort für die Ungläubigen?\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmxci{Derjenige aber, der mit der Wahrheit kommt (,) und (derjenige andere, der) sie für wahr hält, - das sind die Gottesfürchtigen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmxcii{Sie haben, was sie wollen, bei ihrem Herrn; das ist der Lohn der Gutes Tuenden,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmxciii{auf daß Allah ihnen das Schlechteste von dem, was sie getan haben, tilge und ihnen ihren Lohn vergelte für das Beste von dem, was sie taten.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmxciv{Wird Allah nicht Seinem Diener genügen? Dennoch wollen sie dir mit denjenigen, die es außer Ihm geben soll, Furcht einflößen. Und wen Allah in die Irre gehen läßt, der hat niemanden, der ihn rechtleitet.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmxcv{Wen aber Allah rechtleitet, den kann niemand in die Irre führen. Ist nicht Allah Allmächtig und Besitzer von Vergeltungsgewalt?\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmxcvi{Und wenn du sie fragst, wer die Himmel und die Erde erschaffen hat, sagen sie ganz gewiß: ``Allah." Sag: Was meint ihr wohl zu dem, was ihr anstatt Allahs anruft; wenn Allah für mich Unheil will, können sie (dann) Sein Unheil hinwegnehmen? Oder wenn Er für mich Barmherzigkeit will, können sie (dann) Seine Barmherzigkeit zurückhalten? Sag: Meine Genüge ist Allah. Auf Ihn ver lassen sich diejenigen, die sich (überhaupt auf jemanden) verlassen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmxcvii{Sag: O mein Volk, handelt nach eurer Stellung, ich werde (ebenfalls so) handeln. Dann werdet ihr erfahren,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmxcviii{über wen eine Strafe kommen wird, die ihn in Schande stürzt, und (über wen) eine beständige Strafe hereinbrechen wird.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmxcix{Gewiß, Wir haben für die Menschen das Buch mit der Wahrheit auf dich hinabgesandt. Wer sich nun rechtleiten läßt, der (tut das) zu seinem eigenen Vorteil; und wer in die Irre geht, der geht nur zu seinem eigenen Nachteil in die Irre. Und du bist nicht (als) Sachwalter über sie (eingesetzt).\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmc{Allah beruft die Seelen zur Zeit ihres Todes ab und auch diejenigen, die nicht gestorben sind, während ihres Schlafes'. Er hält die eine, für die Er den Tod beschlossen hat, zurück und gibt die andere auf eine festgesetzte Frist frei. Darin sind wahrlich Zeichen für Leute, die nachdenken.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmci{Oder haben sie sich anstatt Allahs Fürsprecher genommen? Sag: Auch wenn sie über nichts verfügen und nicht begreifen?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcii{Sag: Alle Fürsprache gehört Allah (allein). Ihm gehört die Herrschaft der Himmel und der Erde. Hierauf werdet ihr zu Ihm zurückgebracht.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmciii{Und wenn Allah allein erwähnt wird, verkrampfen sich die Herzen derjenigen, die nicht an das Jenseits glauben. Wenn aber diejenigen erwähnt werden, die es außer Ihm geben soll, freuen sie sich sogleich.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmciv{Sag: O Allah, Erschaffer der Himmel und der Erde, Kenner des Verborgenen und des Offenbaren! Du wirst zwischen Deinen Dienern über das richten, worüber sie uneinig zu sein pflegten.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcv{Wenn diejenigen, die Unrecht tun, alles hätten, was auf der Erde ist, und noch einmal das Gleiche dazu, würden sie sich sicherlich damit von der bösen Strafe am Tag der Auferstehung loskaufen. Ihnen wird dann (nämlich) von Allah her klar, womit sie nicht gerechnet haben.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcvi{Und ihnen werden die bösen Taten klar, die sie begangen haben, und es umschließt sie das, worüber sie sich lustig zu machen pflegten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcvii{Wenn dem Menschen Unheil widerfährt, ruft er Uns an. Wenn Wir ihm hierauf Gunst von Uns gewähren, sagt er: ``Es ist mir nur gegeben worden aufgrund von Wissen." Nein! Vielmehr ist es eine Versuchung. Aber die meisten von ihnen wissen nicht.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcviii{Das haben bereits diejenigen, die vor ihnen waren, gesagt. Aber nicht nützte ihnen, was sie zu. erwerben pflegten.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcix{So trafen sie (schließlich) die bösen Taten, die sie begangen hatten. Und (auch) diejenigen unter diesen da, die Unrecht tun, werden die bösen Taten treffen, die sie begangen haben. Und sie können sich (Allah) nicht entziehen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcx{Wissen sie denn nicht, daß Allah die Versorgung großzügig gewährt, wem Er will, und auch bemißt? Darin sind wahrlich Zeichen für Leute, die glauben.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcxi{Sag: O Meine Diener, die ihr gegen euch selbst maßlos gewesen seid, verliert nicht die Hoffnung auf Allahs Barmherzigkeit. Gewiß, Allah vergibt die Sünden alle. Er ist ja der Allvergebende und Barmherzige.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcxii{Und wendet euch eurem Herrn reuig zu und seid Ihm ergeben, bevor die Strafe über euch kommt, worauf euch keine Hilfe zuteil werden wird.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcxiii{Und folgt dem Besten von dem, was zu euch von eurem Herrn (als Offenbarung) herabgesandt worden ist, bevor die Strafe plötzlich über euch kommt, ohne daß ihr merkt,\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcxiv{daß ja keine Seele sage(n muß): ``O welch gramvolle Reue für mich wegen dessen, was ich (an Pflichten) gegenüber Allah vernachlässigte. Ich gehörte ja wirklich zu den Spöttern."\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcxv{Oder daß sie nicht etwa sage(n soll): ``Wenn Allah mich nur rechtgeleitet hätte, würde ich bestimmt zu den Gottesfürchtigen gehören."\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcxvi{Oder daß sie nicht etwa, wenn sie die Strafe sieht, sage(n soll): ``Wenn ich doch nur eine (Möglichkeit zur) Wiederholung hätte, dann würde ich zu den Gutes Tuenden gehören!"\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcxvii{- ``Aber ja! Meine Zeichen sind doch zu dir gekommen. Da hast du sie für Lüge erklärt und dich hochmütig verhalten und hast zu den Ungläubigen gehört."\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcxviii{Und am Tag der Auferstehung siehst du diejenigen, die gegen Allah gelogen haben; ihre Gesichter sind schwarz geworden. Ist nicht in der Hölle ein Aufenthaltsort für die Hochmütigen?\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmcxix{Aber Allah errettet diejenigen, die gottesfürchtig waren, indem Er sie entrinnen läßt. Böses wird ihnen nicht widerfahren, noch werden sie traurig sein.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmcxx{Allah ist der Schöpfer von allem, und Er ist Sachwalter über alles.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmcxxi{Ihm gehören die Schlüssel der Himmel und der Erde. Diejenigen, die Allahs Zeichen verleugnen, das sind die Verlierer.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmcxxii{Sag: Wollt ihr mir denn wirklich befehlen, einem anderen als Allah zu dienen, ihr Toren?\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcxxiii{Dir und denjenigen, die vor dir waren, ist ja (als Offenbarung) eingegeben worden: ``Wenn du (Allah andere) beigesellst, wird dein Werk ganz gewiß hinfällig, und du gehörst ganz gewiß zu den Verlierern.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcxxiv{Nein! Vielmehr Allah (allein) sollst du dienen und zu den Dankbaren sollst du gehören."\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcxxv{Sie haben Allah nicht eingeschätzt, wie es Ihm gebührt', wo die ganze Erde am Tag der Auferstehung in Seiner Hand gehalten wird und (auch) die Himmel in Seiner Rechten zusammengefaltet sein werden. Preis sei Ihm! Erhaben ist Er über das, was sie (Ihm) beigesellen.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcxxvi{Und es wird ins Horn geblasen, und da bricht zusammen, wie vom Donnerschlag getroffen, wer in den Himmeln und wer auf der Erde ist, außer wem Allah will. Hierauf wird ein weiteres Mal hineingeblasen, da.stehen sie sogleich auf und schauen hin.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmcxxvii{Und die Erde wird im Licht ihres Herrn erstrahlen; das Buch wird hingelegt, und die Propheten und die Zeugen werden herbeigebracht; und es wird zwischen ihnen der Wahrheit entsprechend entschieden, und es wird ihnen kein Unrecht zugefügt.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcxxviii{Und jeder Seele wird in vollem Maß zukommen, was sie getan hat. Er weiß sehr wohl Bescheid über das, was sie tun.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcxxix{Zur Hölle getrieben werden diejenigen, die ungläubig waren, in Scharen. Wenn sie dann dort ankommen, werden ihre Tore geöffnet und ihre Wärter sagen zu ihnen: ``Sind nicht zu euch Gesandte von euch (selbst) gekommen, die euch die Zeichen eures Herrn verlesen und euch die Begegnung mit diesem eurem Tag warnend an kündigen?" Sie werden sagen: ``Ja doch! Aber das Wort der Strafe ist gegen die Ungläubigen unvermeidlich fällig geworden."\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmcxxx{Es wird (zu ihnen) gesagt werden: ``Betretet die Tore der Hölle, ewig darin zu bleiben." Schlimm ist der Aufenthaltsort der Hochmütigen!\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmcxxxi{Und in den (Paradies)garten geführt werden diejenigen, die ihren Herrn fürchteten, in Scharen. Wenn sie dann dort ankommen und seine Tore geöffnet werden und seine Wärter zu ihnen sagen: ``Friede sei auf euch! Gut wart ihr, so betretet ihn, ewig (darin) zu bleiben."\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmcxxxii{Und sie werden sagen: ``(Alles) Lob gehört Allah, Der Sein Versprechen an uns gehalten und uns die Erde zum Erbe gegeben hat, so daß wir uns im (Paradies)garten aufhalten können, wo immer wir wollen! Wie trefflich ist der Lohn derjenigen, die (gut) handeln!"\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmcxxxiii{Und du wirst die Engel den Thron umgeben und ihren Herrn lobpreisen sehen. Und es wird zwischen ihnen der Wahrheit entsprechend entschieden. Und es wird gesagt: ``(Alles) Lob gehört Allah, dem Herrn der Weltenbewohner!"\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmcxxxiv{\basmalahde Ha-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcxxxv{Dies ist) die Offenbarung des Buches von Allah, dem Allmächtigen und Allwissenden,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcxxxvi{dem Vergebenden der Sünde und dem Anehmenden der Reue, dem Strengen in der Bestrafung und Dem, Der voller Wohltätigkeit ist. Es gibt keinen Gott außer Ihm. Zu Ihm ist der Ausgang.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcxxxvii{Über Allahs Zeichen streiten nur diejenigen, die ungläubig sind. Lasse dich durch ihren Wechsel in den Landstrichen nicht täuschen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcxxxviii{Schon) vor ihnen haben das Volk Nuhs und nach diesen die Gruppierungen (ihre Gesandten) der Lüge bezichtigt. Jede Gemeinschaft hatte vor, (gegen) ihren Gesandten (vorzugehen, um ihn) zu ergreifen. Und sie stritten mit dem Falschen, um damit die Wahrheit zu widerlegen. Da ergriff Ich sie. Wie war da Meine Bestrafung!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcxxxix{Und so wurde das Wort deines Herrn gegen diejenigen, die ungläubig sind, unvermeidlich fällig, nämlich daß sie Insassen des (Höllen)feuers sind.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcxl{Diejenigen, die den Thron tragen, und diejenigen, die in seiner Umgebung sind, lobpreisen ihren Herrn und glauben an Ihn und bitten um Vergebung für diejenigen, die glauben: ``Unser Herr, Du umfaßt alles in Deiner Barmherzigkeit und Deinem Wissen. So vergib denjenigen, die bereuen und Deinem Weg folgen, und bewahre sie vor der Strafe des Höllenbrandes.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcxli{Unser Herr, lasse sie in die Gärten Edens, die Du ihnen versprochen hast, eingehen, (sie) und wer rechtschaffen ist von ihren Vätern, ihren Gattinnen und ihren Nachkommen, Du bist ja der Allmächtige und Allweise.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcxlii{Und bewahre sie vor den bösen Taten. Wen Du an jenem Tag vor den bösen Taten bewahrst, dessen hast Du dich ja erbarmt, und das ist der großartige Erfolg."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcxliii{Gewiß, denjenigen, die ungläubig waren, wird zugerufen: ``Allahs Abscheu ist wahrlich größer als eure Abscheu vor euch selbst, da ihr zum Glauben aufgerufen worden, aber ungläubig geblieben seid."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcxliv{Sie sagen: ``Unser Herr, Du hast uns zweimal sterben lassen und zweimal lebendig gemacht. So bekennen wir unsere Sünden. Gibt es denn einen Weg, (hier) herauszukommen?"\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcxlv{``Dies ist so, weil ihr, wenn (immer) Allah allein angerufen wurde, ungläubig bliebt, ihr aber, wenn Ihm (andere) beigesellt wurden, glaubtet. Das Urteil gehört Allah, dem Erhabenen und Großen."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcxlvi{Er ist es, Der euch Seine Zeichen zeigt und euch vom Himmel Versorgung herabsendet, doch bedenkt nur, wer sich (Allah) reuig zuwendet.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcxlvii{So ruft Allah an, (wobei ihr) Ihm gegenüber aufrichtig in der Religion (seid), auch wenn es den Ungläubigen zuwider ist.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcxlviii{Der Inhaber der hohen Rangstufen und der Herr des Thrones sendet den Geist von Seinem Befehl, wem von Seinen Dienern Er will, damit er den Tag der Begegnung warnend ankündige,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcxlix{den Tag, an dem sie vortreten, wobei nichts von ihnen vor Allah verborgen bleibt. - ``Wem gehört heute die Herrschaft?" - Allah, dem Einen, dem Allbezwinger.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcl{Heute wird jeder Seele das vergolten, was sie erworben hat. Heute gibt es kein Un recht. Gewiß, Allah ist schnell im Abrechnen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcli{Und warne sie vor dem Tag der immer näher kommenden (Stunde des Gerichts), an dem die Herzen (vor Angst) in der Kehle sitzen und sie unterdrücken (ihren Grimm), an dem die Ungerechten weder einen warmherzigen Freund noch einen Fürsprecher haben, dem man gehorchen würde.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmclii{Er kennt die verräterischen Augen und weiß, was die Brüste verbergen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcliii{Und Allah entscheidet der Wahrheit entsprechend. Diejenigen aber, die sie anstatt Seiner anrufen, haben nichts zu entscheiden. Gewiß, Allah ist der Allhörende und Allsehende.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcliv{Sind sie nicht auf der Erde umhergereist, so daß sie schauen (konnten), wie das Ende derjenigen war, die vor ihnen waren? Sie hatten ja mehr Kraft und Wirkung auf der Erde als sie. Da ergriff sie Allah für ihre Sünden, und sie hatten vor Allah keinen Beschützer.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmclv{Dies, weil ihre Gesandten immer wieder mit den klaren Beweisen zu ihnen kamen, sie aber ungläubig blieben. So ergriff sie Allah, denn Er ist Stark und streng im Bestrafen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmclvi{Und Wir sandten bereits Musa mit Unseren Zeichen und mit einer deutlichen Ermächtigung\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmclvii{zu Fir'aun und Haman und Qarun. Sie aber sagten: ``Ein verlogener Zauberer."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmclviii{Als er ihnen nun die Wahrheit von Uns brachte, sagten sie: ``Tötet die Söhne derjenigen, die mit ihm glauben, und laßt (nur) ihre Frauen am Leben." Aber die List der Ungläubigen geht bestimmt verloren.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmclix{Fir'aun sagte: ``Laßt mich Musa töten; soll er (doch) seinen Herrn anrufen! Ich fürchte, daß er (sonst) eure Religion abändern oder daß er Unheil im Land hervorrufen wird."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmclx{Und Musa sagte: ``Ich nehme Zuflucht zu meinem Herrn und eurem Herrn vor jedem Hochmütigen, der nicht an den Tag der Abrechnung glaubt."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmclxi{Ein gläubiger Mann von den Leuten Fir'auns, der seinen Glauben verborgen hielt, sagte: ``Wollt ihr denn einen Mann töten, weil er sagt: ,Mein Herr ist Allah', wo er doch mit den klaren Beweisen von eurem Herrn zu euch gekommen ist? Wenn er ein Lügner ist, so trägt er (die Last) seiner Lügen. Wenn er aber die Wahrheit sagt, wird euch etwas von dem treffen, was er euch androht. Gewiß, Allah leitet nicht recht, wer maßlos und verlogen ist.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmclxii{O mein Volk, es gehört euch heute die Herrschaft, und ihr habt die Oberhand im Land. Wer wird uns dann gegen Allahs Gewalt helfen, wenn sie über uns kommt?" Fir'aun sagte: ``Ich weise euch nur auf das hin, was ich sehe, und leite euch nur den Weg des besonnenen Handelns."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmclxiii{Derjenige, der glaubte, sagte: ``O mein Volk, gewiß, ich fürchte für euch etwas Gleiches wie den Tag der Gruppierungen,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmclxiv{etwas Gleichartiges wie das, was dem Volk Nuhs, den 'Ad, den Tamud und denjenigen, die nach ihnen waren, (widerfuhr). Und Allah will keine Ungerechtigkeit für die Diener.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmclxv{O mein Volk, gewiß, ich fürchte für euch den Tag des gegenseitigen Zurufens,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmclxvi{den Tag, an dem ihr den Rücken kehren werdet, wobei ihr nichts vor Allah Schützendes haben werdet. Und wen Allah in die Irre gehen läßt, der hat niemanden, der ihn rechtleitet.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmclxvii{Bereits zuvor kam Yusuf zu euch mit den klaren Beweisen. Ihr aber bliebt weiterhin im Zweifel über das, was er euch gebracht hatte. Als er dann gestorben war, sagtet ihr: ,Allah wird nach ihm keinen Gesandten mehr erstehen lassen. So läßt Allah in die Irre gehen, wer maßlos und voller Zweifel ist."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmclxviii{Diejenigen, die über Allahs Zeichen streiten, ohne eine Ermächtigung erhalten zu haben, - welch schwerwiegende Abscheu erregen diese bei Allah und bei denjenigen, die gläubig sind. So versiegelt Allah das Herz eines jeden Hochmütigen und Gewalttätigen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmclxix{Und Fir'aun sagte: ``O Haman, errichte mir einen Hochbau, vielleicht kann ich die Seile erreichen,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmclxx{die Seile der Himmel, daß ich zum Gott Musas emporsteige, und ich halte ihn wahrlich für einen Lügner." So wurde Fir'aun sein böses Tun ausgeschmückt, und er wurde vom (rechten) Weg abgehalten. Aber Fir'auns listige Tat wird nur zugrunde gehen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmclxxi{Derjenige, der gläubig war, sagte: ``O mein Volk, folgt mir, dann leite ich euch den Weg des besonnenen Handelns.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmclxxii{O mein Volk, dieses irdische Leben ist nur Nießbrauch; das Jenseits aber ist die Wohnstätte zum (bleibenden) Aufenthalt.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmclxxiii{Wer etwas Böses tut, dem wird nur gleichviel vergolten. Wer aber rechtschaffen handelt, ob Mann oder Frau, und dabei gläubig ist, jene werden dann in den (Paradies)garten eingehen, wo sie versorgt werden ohne Abrechnung.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmclxxiv{O mein Volk, seht ihr, wie ich euch zur Rettung rufe, ihr mich aber zum (Höllen)feuer ruft?\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmclxxv{Ihr ruft mich dazu auf, Allah zu verleugnen und Ihm das beizugesellen, wovon ich kein Wissen habe, während ich euch zum Allmächtigen und Allvergebenden rufe.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmclxxvi{Es ist zweifellos, daß demjenigen, zu dem ihr mich ruft, keine Anrufung zusteht, weder im Diesseits noch im Jenseits, daß unsere Rückkehr zu Allah sein wird und daß die Maßlosen Insassen des (Höllen)feuers sein werden.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmclxxvii{Dann werdet ihr an das denken, was ich euch sage. Ich überlasse meine Angelegen heit Allah. Gewiß, Allah sieht die Diener wohl."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmclxxviii{So bewahrte Allah ihn vor den Bosheiten dessen, was sie an Ränken geschmiedet hatten, und die Leute Fir'auns umschloß die böse Strafe,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmclxxix{das (Höllen)feuer, dem sie morgens und abends vorgeführt werden. Und am Tag, da sich die Stunde erhebt (, wird es heißen): ``Laßt die Leute Fir'auns in die strengste Strafe eingehen."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmclxxx{Und (denke daran,) wenn sie (dereinst) im (Höllen)feuer miteinander streiten, dann sagen die Schwachen zu denjenigen, die sich hochmütig verhielten: ``Wir waren doch eure Gefolgsleute; könnt ihr uns nun einen Teil des (Höllen)feuers abnehmen?"\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmclxxxi{Diejenigen, die sich hochmütig verhielten, werden sagen: ``Wir befinden uns doch alle darin. Allah hat nun zwischen den Dienern gerichtet."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmclxxxii{Diejenigen, die sich im (Höllen)feuer befinden, werden zu den Wärtern der Hölle sagen: ``Ruft euren Herrn an, daß Er uns einen Tag von der Strafe erlasse."\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmclxxxiii{Sie sagen: ``Pflegten nicht doch eure Gesandten mit den klaren Beweisen zu euch zu kommen?" Sie sagen: ``Ja, doch!" Sie sagen: ``So ruft an." Aber das Anrufen der Ungläubigen geht bestimmt verloren'.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmclxxxiv{Wahrlich, Wir helfen Unseren Gesandten und denjenigen, die glauben, im diesseitigen Leben und am Tag, da die Zeugen auftreten,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmclxxxv{am Tag, da den Ungerechten ihre Entschuldigung nicht nützt und es für sie den Fluch geben und es für sie die böse Wohnstätte geben wird.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmclxxxvi{Wir gaben bereits Musa die Rechtleitung und ließen die Kinder Isra'ils die Schrift erben\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmclxxxvii{als Rechtleitung und Ermahnung für diejenigen, die Verstand besitzen.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmclxxxviii{So sei standhaft. Gewiß, Allahs Versprechen ist wahr. Und bitte um Vergebung für deine Sünde und lobpreise deinen Herrn am Abend und am frühen Morgen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmclxxxix{Gewiß, diejenigen, die über Allahs Zeichen streiten, ohne daß sie eine Ermächtigung erhalten hätten, haben in ihren Brüsten nur Neid (auf das), was sie nicht erreichen werden können. So suche Zuflucht bei Allah, denn Er ist der Allhörende und Allsehende.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcxc{Wahrlich die Erschaffung der Himmel und der Erde ist größer als die Erschaffung der Menschen. Aber die meisten Menschen wissen nicht.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcxci{Nicht gleich sind der Blinde und der Sehende und auch nicht diejenigen, die glauben und rechtschaffene Werke tun, und der Missetäter. Wie wenig ihr bedenkt!\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcxcii{Gewiß, die Stunde kommt bestimmt, an ihr gibt es keinen Zweifel. Aber die meisten Menschen glauben nicht.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcxciii{Euer Herr sagt: ``Ruft Mich an, so erhöre Ich euch. Gewiß, diejenigen, die sich aus Hochmut weigern, Mir zu dienen, werden in die Hölle gedemütigt eingehen."\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmcxciv{Allah ist es, Der euch die Nacht gemacht hat, damit ihr in ihr ruht, und den Tag hell'. Allah ist wahrlich voll Huld gegen die Menschen. Aber die meisten Menschen sind nicht dankbar.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmcxcv{Dies ist doch Allah, euer Herr, der Schöpfer von allem. Es gibt keinen Gott außer Ihm. Wie laßt ihr euch also abwendig machen?\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmcxcvi{So lassen sich diejenigen abwendig machen, die Allahs Zeichen zu verleugnen pflegen.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmcxcvii{Allah ist es, Der euch die Erde zu einem festen Grund und den Himmel zu einem Gebäude gemacht, euch gestaltet und dabei eure Gestalten schön geformt hat und euch von den guten Dingen versorgt. Dies ist doch Allah, euer Herr. Segensreich ist Allah, der Herr der Weltenbewohner!\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcxcviii{Er ist der Lebendige. Es gibt keinen Gott außer Ihm. So ruft Ihn an, (wobei ihr) Ihm gegenüber aufrichtig in der Religion (seid). (Alles) Lob gehört Allah, dem Herrn der Weltenbewohner!\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcxcix{Sag: Mir wurde verboten, als die klaren Zeichen von meinem Herrn zu mir kamen, denjenigen zu dienen, die ihr anstatt Allahs anruft; und mir wurde befohlen, mich dem Herrn der Weltenbewohner zu ergeben.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcc{Er ist es, Der euch aus Erde erschaffen hat, hierauf aus einem Samentropfen, hierauf aus einem Anhängsel'. Hierauf läßt Er euch als kleine Kinder hervorkommen. Hierauf (läßt Er euch heranwachsen), damit ihr eure Vollreife erlangt und damit ihr dann Greise werdet - manch einer von euch wird vorher abberufen - und damit ihr eine festgesetzte Frist erreicht, und auf daß ihr begreifen möget.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcci{Er ist es, Der lebendig macht und sterben läßt. Wenn Er dann eine Angelegenheit bestimmt, sagt Er zu ihr nur: ``Sei!", und so ist sie.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmccii{Siehst du nicht diejenigen, die über Allahs Zeichen streiten? Wie sie sich doch abwenden lassen!\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcciii{Sie,) die die Schrift und das, womit Wir Unsere Gesandten gesandt haben, für Lüge erklären. Aber sie werden (es noch) erfahren,\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcciv{wenn die Fesseln und die Ketten um ihre Hälse (angebracht) sind und sie gezerrt werden\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmccv{ins heiße Wasser und hierauf ins (Höllen)feuer als Brennstoff geworfen werden.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmccvi{Dann wird zu ihnen gesagt werden: ``Wo ist das, was ihr (Allah) beigesellt habt (, und dem ihr gedient habt)\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmccvii{anstatt Allahs?" Sie werden sagen: ``Sie sind uns entschwunden. Aber nein! Wir riefen zuvor (doch gar) nichts an." So läßt Allah die Ungläubigen in die Irre gehen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmccviii{``Dies (geschieht deshalb), weil ihr auf der Erde ohne Recht froh zu sein und (zu sehr) frohsinnig zu leben pflegtet.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmccix{Betretet (nun) die Tore der Hölle, ewig darin zu bleiben." Schlimm ist der Aufent haltsort der Hochmütigen!\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmccx{So sei standhaft. Gewiß, Allahs Versprechen ist wahr. Ob Wir dich einen Teil dessen, was Wir ihnen androhen, sehen lassen oder dich (vorher) abberufen, zu Uns werden sie zurückgebracht.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmccxi{Wir haben doch bereits vor dir Gesandte gesandt. Unter ihnen gibt es manche, von denen Wir dir berichtet haben, und unter ihnen gibt es manche, von denen Wir dir nicht bereichtet haben. Kein Gesandter kann ein Zeichen bringen - außer mit Allahs Erlaubnis. Wenn dann Allahs Befehl kommt, wird der Wahrheit entsprechend entschieden, und verloren haben dann diejenigen, die (die Botschaft) für falsch erklären.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmccxii{Allah ist es, Der für euch das Vieh gemacht hat, damit ihr (teils) auf ihm reiten könnt; und ihr könnt (teils) von ihm essen.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmmccxiii{Und ihr habt an ihm (allerlei) Nutzen. Und damit ihr, auf ihm (reitend,) ein Ziel', das ihr in euren Brüsten (hegt), erreichen könnt. Und auf ihm und auf den Schiffen werdet ihr getragen.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmccxiv{Und Er zeigt euch Seine Zeichen. Welches von Allahs Zeichen wollt ihr nun nicht anerkennen?\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmccxv{Sind sie denn nicht auf der Erde umhergereist, so daß sie schauen (konnten), wie das Ende derjenigen war, die vor ihnen waren? Sie waren zahlreicher als sie und hatten mehr Kraft und Wirkung auf der Erde als sie. Aber nicht nützte ihnen, was sie zu erwerben pflegten.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmccxvi{Als nun ihre Gesandten zu ihnen mit den klaren Beweisen kamen, waren sie froh über das Wissen, das sie besaßen, und es umschloß sie das, worüber sie sich lustig zu machen pflegten.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmccxvii{Als sie dann Unsere Gewalt sahen, sagten sie: ``Wir glauben an Allah allein und verleugnen das, was wir Ihm beizugesellen pflegten."\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmccxviii{Aber nicht mehr nützen konnte ihnen ihr Glaube, als sie Unsere Gewalt sahen - (so war) Allahs Gesetzmäßigkeit, die bereits (in der Vergangenheit) an Seine Diener ergangen war, und verloren waren da die Ungläubigen.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmccxix{\basmalahde Ha-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmccxx{Dies ist) eine Offenbarung vom Allerbarmer, dem Barmherzigen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmccxxi{ein Buch, dessen Zeichen ausführlich dargelegt sind, als ein arabischer Qur'an, für Leute, die Bescheid wissen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmccxxii{als Frohboten und als Warner. Aber die meisten von ihnen wenden sich ab, so daß sie nicht hören.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmccxxiii{Sie sagen: ``Unsere Herzen sind in Hüllen vor dem, wozu du uns aufrufst, und in unseren Ohren ist Schwerhörigkeit, und zwischen uns und dir ist ein Vorhang. Handle also, wir werden ebenfalls handeln."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmccxxiv{Sag: Gewiß, ich bin ja nur ein menschliches Wesen wie ihr; mir wird (als Offenbarung) eingegeben, daß euer Gott nur ein Einziger Gott ist. So richtet euch zu Ihm hin und bittet Ihn um Vergebung. Und wehe den Götzendienern,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmccxxv{die die Abgabe nicht entrichten, und sie, die sie das Jenseits verleugnen!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmccxxvi{Gewiß, diejenigen, die glauben und rechtschaffene Werke tun, für sie wird es einen Lohn geben, der nicht aufhört.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmccxxvii{Sag: Wollt ihr denn wirklich denjenigen verleugnen, Der die Erde in zwei Tagen erschaffen hat, und Ihm andere als Seinesgleichen zur Seite stellen? Das ist der Herr der Weltenbewohner.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmccxxviii{Er hat in ihr festgegründete Berge gemacht, (die) über ihr (aufragen), und hat sie gesegnet und in ihr die Nahrung im rechten Maß in vier Tagen festgelegt, gleichmäßig für diejenigen, die danach fragen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmccxxix{Hierauf wandte Er sich dem Himmel zu, während er noch aus Rauch bestand, und sagte dann zu ihm und zur Erde: ``Kommt beide her, freiwillig oder widerwillig." Sie sagten: ``Wir kommen in Gehorsam."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmccxxx{So führte Er sie als sieben Himmel in zwei Tagen aus und gab jedem Himmel seine Aufgabe ein. Und Wir haben den untersten Himmel mit Lampen geschmückt, und auch als Schutz. Das ist die Anordnung des Allmächtigen und Allwissenden.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmccxxxi{Wenn sie sich also abwenden, dann sag: Ich warne euch vor einem Donnerschlag gleich dem Donnerschlag der 'Ad und der Tamud,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmccxxxii{als die Gesandten von vorn und von hinten zu ihnen kamen: ``Dient nur Allah (allein)". Sie sagten: ``Wenn unser Herr gewollt hätte, hätte Er Engel herabgesandt. So verleugnen wir das, womit ihr gesandt worden seid."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmccxxxiii{Was nun die 'Ad angeht, so verhielten sie sich hochmütig auf der Erde ohne Recht und sagten: ``Wer hat eine stärkere Kraft als wir?" Sahen sie denn nicht, daß Allah, Der sie erschaffen hatte, eine stärkere Kraft hat als sie? Aber sie pflegten Unsere Zeichen zu verleugnen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmccxxxiv{Da sandten Wir gegen sie einen eiskalten Wind an unheilvollen Tagen, um sie die schändliche Strafe im diesseitigen Leben kosten zu lassen. Aber die Strafe des Jenseits ist wahrlich schändlicher, und es wird ihnen keine Hilfe zuteil werden.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmccxxxv{Was aber die Tamud angeht, so wollten Wir sie rechtleiten, sie aber liebten die Blindheit mehr als die Rechtleitung. Da ergriff sie der Donnerschlag der schmählichen Strafe für das, was sie erworben hatten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmccxxxvi{Und Wir erretteten diejenigen, die glaubten und gottesfürchtig waren.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmccxxxvii{Und (denke an) den Tag, da Allahs Feinde zum (Höllen)feuer versammelt und in Reihen gebracht werden.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmccxxxviii{Wenn sie dann dort angekommen sind, legen ihr Gehör, ihre Augen und ihre Häute gegen sie Zeugnis ab über das, was sie zu tun pflegten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmccxxxix{Sie werden zu ihren Häuten sagen: ``Warum habt ihr gegen uns Zeugnis abgelegt?" Sie sagen: ``Allah, Der alles reden läßt, hat uns reden lassen. Er ist es, Der euch das erste Mal erschaffen hat, und zu Ihm werdet ihr zurückgebracht.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmccxl{Und ihr pflegtet euch nicht so zu verstecken, daß euer Gehör, eure Augen und eure Häute nicht Zeugnis gegen euch ablegten. Aber ihr meintet, daß Allah nicht viel wisse von dem, was ihr tut.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmccxli{Und diese eure Meinung, die ihr von eurem Herrn hattet, hat euch ins Verderben gestürzt, so daß ihr jetzt zu den Verlierern gehört."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmccxlii{Auch) wenn sie standhaft sind, so ist (doch) das (Höllen)feuer ein Aufenthaltsort für sie'. Und wenn sie um Erweisung von Gnade bitten, so werden sie doch nicht zu den Begnadigten gehören.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmccxliii{Und Wir hatten ihnen Gesellen verschafft, die ihnen dann ausschmückten, was vor ihnen und was hinter ihnen lag. Und unvermeidlich fällig wurde das Wort gegen sie, unter (anderen) Gemeinschaften von den Ginn und den Menschen, die vor ihnen dahingegangen waren. Gewiß, sie waren ja Verlierer.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmccxliv{Diejenigen, die ungläubig sind, sagen: ``Hört nicht auf diesen Qur'an, und führt dazwischen unbedachte Reden, auf daß ihr siegen möget."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmccxlv{Ganz gewiß werden Wir diejenigen, die ungläubig sind, eine strenge Strafe kosten lassen, und ganz gewiß werden Wir ihnen das Schlechteste von dem vergelten, was sie zu tun pflegten.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmccxlvi{Das ist der Lohn der Feinde Allahs: das (Höllen)feuer, in dem sie die ewige Wohnstätte haben, als Vergeltung dafür, daß sie Unsere Zeichen zu verleugnen pflegten.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmccxlvii{Und diejenigen, die ungläubig sind, werden sagen: ``Unser Herr, zeige uns die beiden von den Ginn und den Menschen, die uns in die Irre geführt haben, daß wir sie unter unsere Füße legen, damit sie zu den Unterlegenen gehören."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmccxlviii{Gewiß, diejenigen, die sagen: ``Unser Herr ist Allah", und sich hierauf recht verhalten, auf sie kommen die Engel herab: ``Fürchtet euch nicht, seid nicht traurig, und vernehmt die frohe Botschaft vom (Paradies)garten', der euch stets versprochen wurde.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmccxlix{Wir sind eure Beschützer im diesseitigen Leben und im Jenseits. Ihr werdet darin haben, was eure Seelen begehren, und ihr werdet darin haben, was ihr erbetet,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmccl{- eine gastliche Aufnahme von einem Allvergebenden, einem Barmherzigen."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmccli{Und wer spricht bessere Worte als wer zu Allah ruft, rechtschaffen handelt und sagt: ``Gewiß doch, ich gehöre zu den (Allah) Ergebenen `` ?\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcclii{Nicht gleich sind die gute Tat und die schlechte Tat. Wehre mit einer Tat, die besser ist, (die schlechte) ab, dann wird derjenige, zwischen dem und dir Feindschaft besteht, so, als wäre er ein warmherziger Freund.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmccliii{Aber dies wird nur denjenigen dargeboten, die standhaft sind, ja es wird nur demjenigen dargeboten, der ein gewaltiges Glück hat.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmccliv{Und wenn dich vom Satan eine Eingebung aufstachelt, dann suche Zuflucht bei Allah, denn Er ist ja der Allhörende und Allwissende.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcclv{Und zu Seinen Zeichen gehören die Nacht und der Tag, die Sonne und der Mond. Werft euch weder vor der Sonne noch vor dem Mond nieder, sondern werft euch vor Allah nieder, Der sie erschaffen hat, wenn ihr (tatsächlich) Ihm allein dient.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcclvi{Wenn sie sich jedoch hochmütig weigern ... (, dann lasse sie,) - diejenigen, die bei deinem Herrn sind, preisen Ihn doch bei Nacht und Tag, ohne Überdruß zu empfinden.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcclvii{Zu Seinen Zeichen gehört es, daß du die Erde demütig siehst. Wenn Wir aber Wasser auf sie herabkommen lassen, regt sie sich und schwillt. Gewiß, Derjenige, Der sie wieder belebt, wird (auch) die Toten wieder lebendig machen, denn gewiß, Er hat zu allem die Macht.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcclviii{Gewiß, diejenigen, die mit Unseren Zeichen abwegig umgehen, sind Uns nicht verborgen. Ist denn einer, der ins (Höllen)feuer geworfen wird, besser oder jemand, der am Tag der Auferstehung in Sicherheit kommt. Tut, was ihr wollt, denn was ihr tut, sieht Er wohl.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcclix{Gewiß, diejenigen, die die Ermahnung verleugnen, nachdem sie zu ihnen gekommen ist ... Es ist fürwahr ein wehrhaftes Buch,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcclx{an das das Falsche weder von vorn noch von hinten herankommt, eine Offenbarung von einem Allweisen und Lobenswürdigen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcclxi{Es wird dir nur das gesagt, was schon den Gesandten vor dir gesagt wurde. Dein Herr ist wahrlich voll der Vergebung und verhängt (auch) schmerzhafte Strafe.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcclxii{Hätten Wir ihn zu einem fremdsprachigen Qur'an gemacht, hätten sie sicherlich gesagt: ``Wären doch seine Zeichen ausführlich dargelegt worden! Wie, ein fremdsprachiger (Qur'an) und ein Araber?" Sag: Er ist für diejenigen, die glauben, eine Rechtleitung und eine Heilung. Und diejenigen, die nicht glauben, haben Schwerhörigkeit in ihren Ohren, und er ist für sie (wie) Blindheit. Diese sind, als würde ihnen von einem fernen Ort aus zugerufen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcclxiii{Und Wir gaben bereits Musa die Schrift, doch wurde man darüber uneinig. Und wenn es nicht ein früher ergangenes Wort von deinem Herrn gegeben hätte, so wäre zwischen ihnen wahrlich entschieden worden. Und sie sind darüber fürwahr in starkem Zweifel.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcclxiv{Wer rechtschaffen handelt, der (tut es) zu seinem eigenen Vorteil, und wer Böses tut, der (tut es) zu seinem eigenen Nachteil. Und dein Herr ist keiner, der den Dienern Unrecht zufügt.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcclxv{Ihm ist das Wissen um die Stunde vorbehalten. Keine Früchte kommen aus ihren Hüllen hervor und kein weibliches Wesen wird schwanger und kommt nieder, außer mit Seinem Wissen. Und am Tag, da Er ihnen zurufen wird: ``Wo sind denn Meine Teilhaber?", werden sie sagen: ``Wir geben Dir zur Antwort': Es gibt unter uns keinen Zeugen (hierfür)."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcclxvi{Und entschwunden wird ihnen sein, was sie zuvor anzurufen pflegten, und sie werden überzeugt sein, daß es für sie kein Entrinnen gibt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcclxvii{Nicht überdrüssig wird der Mensch, um das Gute zu beten, doch wenn ihm Schlechtes widerfährt, dann ist er sehr verzweifelt und verliert alle Hoffnung.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcclxviii{Und wenn Wir ihn Barmherzigkeit von Uns kosten lassen nach Leid, das ihm widerfuhr, sagt er ganz gewiß: ``Das steht mir zu. Und ich glaube nicht, daß die Stunde sich einstellen wird. Und wenn ich zu meinem Herrn zurückgebracht werde, werde ich sicherlich das Beste bei Ihm erhalten." Aber Wir werden denjenigen, die ungläubig sind, ganz gewiß kundtun, was sie getan haben; und Wir werden sie ganz gewiß von harter Strafe kosten lassen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcclxix{Und wenn Wir dem Menschen Gunst erweisen, wendet er sich ab und entfernt sich beiseite. Wenn ihm aber Schlechtes widerfährt, dann verfällt er in ausgedehntes Bittgebet.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcclxx{Sag: Was meint ihr, wenn er doch von Allah stammt und ihr ihn hierauf verleugnet? Wer ist weiter abgeirrt als jemand, der sich in tiefem Widerstreit befindet?\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcclxxi{Wir werden ihnen Unsere Zeichen am Gesichtskreis" und in ihnen selbst zeigen, bis es ihnen klar wird, daß es die Wahrheit ist. Genügt es denn nicht, daß dein Herr über alles Zeuge ist?\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcclxxii{Sicherlich, sie sind im Zweifel über die Begegnung mit ihrem Herrn. Sicherlich, Er umfaßt doch alles.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcclxxiii{\basmalahde Ha-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcclxxiv{'Ayn-Sin-Qaf.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcclxxv{So gibt dir Allah (als Offenbarung) ein und (hat auch zuvor) denjenigen, die vor dir waren, (als Offenbarungen eingegeben), (Er), der Allmächtige und Allweise.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcclxxvi{Ihm gehört, was in den Himmeln und was auf der Erde ist, und Er ist der Erhabene und Allgewaltige.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcclxxvii{Beinahe brechen die Himmel auseinander von oben her. Und die Engel lobpreisen ihren Herrn und bitten (Ihn) um Vergebung für diejenigen, die auf der Erde sind. Ja sicherlich, Allah ist der Allvergebende und Barmherzige.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcclxxviii{Über diejenigen, die sich anstatt Seiner Schutzherren nehmen, ist Allah Hüter, und du bist nicht ihr Sachwalter.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcclxxix{Und so haben Wir dir einen arabischen Qur'an (als Offenbarung) eingegeben, damit du die Mutter der Städte und diejenigen rings umher warnst, und damit du vor dem Tag der Versammlung warnst, an dem es keinen Zweifel gibt. Eine Gruppe wird im (Paradies)garten sein und eine Gruppe in der Feuerglut.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcclxxx{Und wenn Allah gewollt hätte, hätte Er sie wahrlich zu einer einzigen Gemeinschaft gemacht. Aber Er läßt, wen Er will, in Seine Barmherzigkeit eingehen. Und die Ungerechten haben weder Schutzherrn noch Helfer.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxi{Oder haben sie sich anstatt Seiner Schutzherren genommen? Allah (allein) ist doch der Schutzherr. Er macht die Toten wieder lebendig, und Er hat zu allem die Macht.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxii{Und worüber ihr auch immer uneinig seid, das Urteil darüber steht Allah (allein) zu. Dies ist doch Allah, mein Herr. Auf Ihn verlasse ich mich, und Ihm wende ich mich reuig zu.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxiii{Er ist) der Erschaffer der Himmel und der Erde. Er hat euch aus euch selbst Gattinnen gemacht, und auch aus dem Vieh Paare, wodurch Er euch vermehrt. Nichts ist Ihm gleich; und Er ist der Allhörende und Allsehende.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxiv{Ihm gehören die Schlüssel der Himmel und der Erde. Er gewährt die Versorgung großzügig, wem Er will, und bemißt auch. Gewiß, Er weiß über alles Bescheid.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxv{Er hat euch von der Religion festgelegt, was Er Nuh anbefahl und was Wir dir (als Offenbarung) eingegeben haben und was Wir Ibrahim, Musa und 'Isa anbefahlen: Haltet die (Vorschriften der) Religion ein und spaltet euch nicht darin (in Gruppen). Den Götzendienern setzt das schwer zu, wozu du sie aufrufst. Allah erwählt dazu, wen Er will, und leitet dazu, wer sich (Ihm) reuig zuwendet.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxvi{Und sie spalteten sich erst, nachdem das Wissen zu ihnen gekommen war - aus Mißgunst untereinander. Und wenn es nicht ein früher ergangenes Wort von deinem Herrn auf eine festgesetzte Frist gäbe, so wäre wahrlich zwischen ihnen entschieden worden. Gewiß, diejenigen, denen die Schrift nach ihnen zum Erbe gegeben wurde, sind darüber in starkem Zweifel.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxvii{Darum rufe du auf und verhalte dich recht, wie dir befohlen wurde. Und folge nicht ihren Neigungen und sag: Ich glaube an das, was Allah an Büchern herabgesandt hat, und mir ist befohlen worden, unter euch gerecht zu handeln. Allah ist unser Herr und euer Herr. Uns unsere Werke und euch eure Werke. Es gibt keine (gemeinsame) Beweisgrundlage zwischen uns und euch. Allah wird uns zusammenbringen. Und zu Ihm ist der Ausgang.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxviii{Diejenigen, die über Allah streiten, nachdem auf Ihn gehört worden ist, deren Beweisgrundlage wird bei ihrem Herrn widerlegt; auf ihnen liegt Zorn, und für sie wird es strenge Strafe geben.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxix{Allah ist es, Der das Buch mit der Wahrheit herabgesandt hat, und (auch) die Waage. Was läßt dich wissen? Vielleicht ist die Stunde nahe.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmccxc{Diejenigen, die nicht an sie glauben, wünschen, sie zu beschleunigen. Diejenigen aber, die glauben, sind besorgt wegen ihr und wissen, daß sie Wirklichkeit ist. Doch eben diejenigen, die über die Stunde streiten, befinden sich wahrlich in tiefem Irrtum.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmccxci{Allah ist Feinfühlig zu Seinen Dienern. Er versorgt, wen Er will. Und Er ist der Starke und Allmächtige.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmccxcii{Wer immer die (Ernte) des Saatfeldes für das Jenseits haben will, dem mehren Wir noch (die Ernte) in seinem Saatfeld. Und wer die (Ernte) des Saatfeldes für das Diesseits haben will, dem lassen Wir etwas davon zukommen, für den wird es aber im Jenseits keinen Anteil geben.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmccxciii{Oder haben sie (etwa) Teilhaber, die ihnen als Religion festgelegt haben, was Allah nicht erlaubt hat? Wenn es nicht das (bereits ergangene) Wort der Entscheidung (des Aufschubs bis zum Jüngsten Gericht) gäbe, wäre zwischen ihnen wahrlich (schon jetzt) entschieden worden. Und gewiß, für die Ungerechten wird es schmerzhafte Strafe geben.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmccxciv{Du wirst (dann) die Ungerechten besorgt sehen wegen dessen, was sie verdient haben, und es wird über sie hereinbrechen. Diejenigen aber, die glauben und rechtschaffene Werke tun, werden sich auf den Auen des (Paradies)gartens befinden. Sie haben, was sie wollen, bei ihrem Herrn; das ist die große Huld.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmccxcv{Das ist die frohe Botschaft, die Allah Seinen Dienern, die glauben und rechtschaffene Werke tun, verkündet. Sag: Ich verlange von euch keinen Lohn dafür, es sei denn die Liebe wie zu den Verwandten'. Und wer ein gutes Werk tut, dem schenken Wir dafür noch mehr Gutes. Gewiß, Allah ist Allvergebend und stets zu Dank bereit.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmccxcvi{Oder sagen sie etwa: ``Er hat eine Lüge gegen Allah ersonnen"? Wenn nun Allah will, kann Er dir das Herz versiegeln. Und Allah löscht das Falsche aus und setzt die Wahrheit mit Seinen Worten durch. Er weiß über das innere Geheimnis Bescheid.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmccxcvii{Er ist es, Der die Reue von Seinen Dienern annimmt und die Missetaten verzeiht. Und Er weiß, was ihr tut.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmccxcviii{Und Er erhört diejenigen, die glauben und rechtschaffene Werke tun, und Er gibt ihnen noch mehr von Seiner Huld. Für die Ungläubigen aber wird es schmerzhafte Strafe geben.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmccxcix{Und würde Allah Seinen Dienern die Versorgung großzügig zuteilen, würden sie auf der Erde Ungerechtigkeit verüben. Aber Allah läßt im richtigen Maß herabkommen, was Er will. Er hat Kenntnis von Seinen Dienern und sieht sie wohl.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmccc{Und Er ist es, Der den Regen herabkommen läßt, nachdem sie die Hoffnung aufgegeben haben, und Der Seine Barmherzigkeit ausbreitet. Und Er ist der Schutzherr und Lobenswürdige.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmccci{Und zu Seinen Zeichen gehört die Erschaffung der Himmel und der Erde und das, was Er an Tieren in ihnen beiden sich ausbreiten läßt. Und Er hat die Macht, sie zu versammeln, wenn Er will.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmcccii{Und was immer euch an Unglück trifft, es ist für das, was eure Hände erworben haben. Und Er verzeiht vieles.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmccciii{Ihr könnt euch (Ihm) auf der Erde nicht entziehen; und ihr habt außer Allah weder Schutzherrn noch Helfer.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmccciv{Und zu Seinen Zeichen gehören die auf dem Meer fahrenden (Schiffe), wie die Berge.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmcccv{Wenn Er will, läßt Er den Wind sich legen, dann bleiben sie bewegungslos auf seiner Oberfläche stehen. Darin sind wahrlich Zeichen für jeden sehr Standhaften und sehr Dankbaren;\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcccvi{oder Er läßt sie untergehen für das, was sie verdient haben. Und Er verzeiht vieles.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmcccvii{Und diejenigen, die über Unsere Zeichen streiten, sollen wissen, daß es für sie kein Entrinnen gibt.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmcccviii{Was immer euch gegeben worden ist, ist Nießbrauch des diesseitigen Lebens. Was aber bei Allah ist, ist besser und beständiger für diejenigen, die glauben und sich auf ihren Herrn verlassen,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcccix{und diejenigen, die schwerwiegende Sünden und Abscheulichkeiten meiden und, wenn sie zornig sind, (doch) vergeben,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcccx{und diejenigen, die auf ihren Herrn hören und das Gebet verrichten, ihre An gelegenheiten) durch Beratung untereinander (regeln) und von dem ausgeben, womit Wir sie versorgt haben,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcccxi{und diejenigen, die, wenn Gewalttätigkeit gegen sie verübt wird, sich selbst helfen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcccxii{Die Vergeltung für eine böse Tat ist etwas gleich Böses. Wer aber verzeiht und Besserung bringt, dessen Lohn obliegt Allah. Er liebt ja nicht die Ungerechten.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcccxiii{Und wer immer sich selbst hilft, nachdem ihm Unrecht zugefügt wurde, gegen jene gibt es keine Möglichkeit (, sie zu belangen).\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcccxiv{Eine Möglichkeit (zu belangen) gibt es nur gegen diejenigen, die den Menschen Unrecht zufügen und auf der Erde ohne Recht Gewalttätigkeiten begehen. Für sie wird es schmerzhafte Strafe geben.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcccxv{Wahrlich, wenn einer standhaft erträgt und vergibt, so gehört dies zur Entschlossenheit (in der Handhabe) der Angelegenheiten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcccxvi{Wen Allah in die Irre gehen läßt, der hat keinen Schutzherrn nach Ihm. Und du siehst, wie die Ungerechten, wenn sie die Strafe sehen, sagen: ``Gibt es denn einen Weg zur Rückkehr?"\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcccxvii{Und du siehst, wie sie ihm vorgeführt werden, demütig vor Unterwürfigkeit und mit verborgenen Blicken schauend. Und diejenigen, die glauben, sagen: ``Die (wahren) Verlierer sind diejenigen, die sich selbst und ihre Angehörigen am Tag der Auferstehung verlieren." Ja gewiß, die Ungerechten befinden sich in beständiger Strafe.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcccxviii{Und sie werden keine Schutzherren haben, die ihnen anstatt Allahs helfen (könnten). Wen Allah in die Irre gehen läßt, für den gibt es keinen (Aus)weg.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcccxix{Hört auf euren Herrn, bevor von Allah ein Tag kommt, der nicht zurückgewiesen wird. Keinen Zufluchtsort werdet ihr an jenem Tag haben und keine Möglichkeit, (etwas) zu mißbilligen'.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcccxx{Wenn sie sich nun abwenden, so haben Wir dich nicht als Hüter über sie gesandt. Dir obliegt nur die Übermittelung (der Botschaft). Und siehe, wenn Wir den Menschen von Uns Barmherzigkeit kosten lassen, ist er froh darüber. Wenn sie aber etwas Böses trifft für das, was ihre Hände vorausgeschickt haben, gewiß, dann ist der Mensch sehr undankbar.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcccxxi{Allah gehört die Herrschaft der Himmel und der Erde. Er erschafft, was Er will. Er schenkt, wem Er will, (nur) weibliche, und Er schenkt, wem Er will, (nur) männliche (Nachkommen).\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcccxxii{Oder (Er schenkt) beides zusammen, männliche und weibliche (Nachkommen). Und Er macht, wen Er will, unfruchtbar. Gewiß, Er ist Allwissend und Allmächtig.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcccxxiii{Und es steht keinem menschlichen Wesen zu, daß Allah zu ihm spricht, außer durch Eingeben (von Offenbarung) oder hinter einem Vorhang, oder indem Er einen Boten sendet, der (ihm) dann mit Seiner Erlaubnis (als Offenbarung) eingibt, was Er will. Gewiß, Er ist Erhaben und Allweise.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcccxxiv{Und ebenso haben Wir dir Geist von Unserem Befehl (als Offenbarung) eingegeben. Du wußtest (vorher) weder, was das Buch noch was der Glaube ist; doch haben Wir es zu einem Licht gemacht, mit dem Wir rechtleiten, wen Wir wollen von Unseren Dienern. Und du leitest ja wahrlich zu einem geraden Weg,\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcccxxv{Allahs Weg, Dem gehört, was in den Himmeln und was auf der Erde ist. Sicherlich, zu Allah nehmen die Angelegenheiten ihren Ausgang.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcccxxvi{\basmalahde Ha-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcccxxvii{Bei dem deutlichen Buch!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcccxxviii{Wir haben es ja zu einem arabischen Qur'an gemacht, auf daß ihr begreifen möget;\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcccxxix{und gewiß, er ist in der Urschrift des Buches bei Uns wahrlich erhaben und weise.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcccxxx{Sollen Wir denn euch bei der Ermahnung unbeachtet lassen, weil ihr maßlose Leute seid?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxi{Und wie viele Propheten haben Wir zu den früheren Geschlechtern gesandt!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxii{Und kein Gesandter kam zu ihnen, ohne daß sie sich über ihn lustig gemacht hätten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxiii{So haben Wir solche vernichtet, die eine stärkere Gewalt hatten als diese. Und vollzogen hat sich das Beispiel an den Früheren.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxiv{Und wenn du sie fragst, wer die Himmel und die Erde erschaffen hat, sagen sie ganz gewiß: ``Erschaffen hat sie der Allmächtige und Allwissende,"\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxv{Der euch die Erde zu einer Lagerstatt gemacht und euch auf ihr Wege gemacht hat, auf daß ihr rechtgeleitet werden möget,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxvi{und Der Wasser vom Himmel in (bestimmtem) Maß herabkommen läßt. Damit lassen Wir (manch) totes Land auferstehen. So werdet auch ihr hervorgebracht werden -,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxvii{und Der die Paare alle erschaffen und euch an Schiffen und Vieh gemacht hat, was ihr besteigen könnt,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxviii{damit ihr euch auf ihren Rücken zurechtsetzt und hierauf der Gunst eures Herrn gedenkt, wenn ihr euch darauf zurechtgesetzt habt, und sagt: ``Preis sei demjenigen, Der uns dies dienstbar gemacht hat! Wir wären hierzu ja nicht imstande gewesen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxix{Und wir werden ganz gewiß zu unserem Herrn zurückkehren."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcccxl{Und sie stellen Ihm einen Teil von Seinen Dienern (als Seinesgleichen zur Seite). Der Mensch ist ja offenkundig sehr undankbar.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcccxli{Oder hat Er sich etwa unter Seinen Geschöpfen Töchter genommen und für euch die Söhne erwählt?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcccxlii{Wenn einem von ihnen die frohe Botschaft (von der Geburt) dessen verkündet wird, was er dem Allerbarmer zum Gleichnis zuschreibt, bleibt sein Gesicht finster, und er hält (seinen Grimm) zurück.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcccxliii{Wollen sie Allah) denn jemanden (zuschreiben), der im Schmuck aufgezogen wird und im Streit nicht deutlich wird?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcccxliv{Und sie machen die Engel, die sie ja Diener des Allerbarmers sind, zu weiblichen Wesen. Waren sie denn Zeugen ihrer Erschaffung? Ihr Zeugnis wird aufgeschrieben, und sie werden befragt werden.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcccxlv{Und sie sagen: ``Wenn der Allerbarmer gewollt hätte, hätten wir ihnen nicht gedient." Sie haben kein Wissen davon; sie stellen nur Schätzungen an.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcccxlvi{Oder haben Wir ihnen etwa vor ihm eine Schrift gegeben, an der sie sich festhalten könnten?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcccxlvii{Nein! Vielmehr sagen sie: ``Wir haben ja (bereits) unsere Väter in einer (bestimmten) Glaubensrichtung vorgefunden, und auf ihren Spuren sind wir rechtgeleitet."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcccxlviii{So haben Wir (auch) vor dir in eine Stadt keinen Warner gesandt, ohne daß diejenigen, die in ihr üppig lebten, gesagt hätten: ``Wir haben ja (bereits) unsere Väter in einer bestimmten Glaubensrichtung vorgefunden, und auf ihren Spuren folgen wir ihrem Vorbild."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcccxlix{Er sagte: ``Etwa auch, wenn ich euch bringe, was eine bessere Rechtleitung enthält als das, worin ihr eure Väter vorgefunden habt?" Sie sagten: ``Wir verleugnen ja das, womit ihr gesandt worden seid."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcccl{Da übten Wir an ihnen Vergeltung. So schau, wie das Ende der Leugner war.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcccli{Und als Ibrahim zu seinem Vater und seinem Volk sagte: ``Gewiß, ich sage mich los von dem, dem ihr dient,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmccclii{außer Demjenigen, Der mich erschaffen hat; denn Er wird mich gewiß rechtleiten."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmcccliii{Und er machte es zu einem bleibenden Wort unter seinen Nachkommen, auf daß sie umkehren mögen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmcccliv{Aber nein! Ich habe diese da und ihre Väter genießen lassen, bis die Wahrheit und ein deutlicher Gesandter zu ihnen gekommen ist.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmccclv{Als nun die Wahrheit zu ihnen gekommen ist, sagen sie: ``Das ist Zauberei, und wir verleugnen ihn."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmccclvi{Und sie sagen: ``Wäre doch dieser Qur'an einem bedeutenden Mann aus den beiden Städten offenbart worden!"\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmccclvii{Verteilen etwa sie die Barmherzigkeit deines Herrn? Wir verteilen doch unter ihnen ihren Lebensunterhalt im diesseitigen Leben und erhöhen die einen von ihnen über die anderen um Rangstufen, damit die einen von ihnen die anderen in Dienst nehmen. Aber die Barmherzigkeit deines Herrn ist besser als das, was sie zusammentragen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmccclviii{Und damit die Menschen nicht eine einzige Gemeinschaft werden (, sahen Wir davon ab, sonst), hätten Wir wahrlich denjenigen, die den Allerbarmer verleugnen, für ihre Häuser Decken aus Silber gemacht und (auch) Treppen, auf denen sie hätten hochsteigen können,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmccclix{und Türen für ihre Häuser, und Liegen, auf denen sie sich lehnen können,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmccclx{und (weitere) Zierde. All das ist aber nur Nießbrauch des diesseitigen Lebens. Das Jenseits bei deinem Herrn ist jedoch für die Gottesfürchtigen (bestimmt).\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmccclxi{Wer für die Ermahnung des Allerbarmers blind ist, dem verschaffen Wir einen Satan, der ihm dann zum Gesellen wird.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmccclxii{Und sie halten sie wahrlich vom Weg ab, und diese meinen, sie seien rechtgeleitet.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmccclxiii{Wenn er dann (schließlich) zu Uns kommt, sagt er: ``O wäre doch zwischen mir und dir eine Entfernung wie zwischen Osten und Westen !" - ein schlimmer Geselle!\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmccclxiv{Heute wird euch, da ihr Unrecht getan habt, nicht nützen, daß ihr an demselben) Strafe teilhabt.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmccclxv{Bist du es etwa, der die Tauben hören läßt oder die Blinden und diejenigen recht leitet, die sich in deutlichem Irrtum befinden?\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmccclxvi{Sollten Wir dich (vorher) fortnehmen, so werden Wir (doch) an ihnen Vergeltung üben.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmccclxvii{Oder Wir zeigen dir, was Wir ihnen angedroht haben; so haben Wir ja ganz die Macht über sie.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmccclxviii{Halte also fest an dem, was dir (als Offenbarung) eingegeben worden ist; gewiß, du befindest dich auf einem geraden Weg.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmccclxix{Das ist wahrlich eine Ermahnung für dich und dein Volk. Und ihr werdet befragt werden.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmccclxx{Und frage wen von Unseren Gesandten Wir vor dir gesandt haben, ob Wir anstatt des Allerbarmers (andere) Götter eingesetzt haben, denen man dienen soll.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmccclxxi{Und Wir sandten bereits Musa mit Unseren Zeichen zu Fir'aun und seiner führenden Schar. Er sagte: ``Gewiß, ich bin der Gesandte des Herrn der Weltenbewohner."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmccclxxii{Als er nun zu ihnen mit Unseren Zeichen kam, lachten sie sogleich über sie.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmccclxxiii{Und Wir zeigten ihnen kein Zeichen, das nicht größer gewesen wäre als das vorhergehende. Und Wir ergriffen sie mit der Strafe, auf daß sie umkehren mochten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmccclxxiv{Und sie sagten: ``O du Zauberer, rufe für uns deinen Herrn an auf Grund dessen, was Er dir anvertraut hat, dann werden wir uns wahrlich rechtleiten lassen."\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmccclxxv{Als Wir dann die Strafe von ihnen hinwegnahmen, da brachen sie sogleich ihr Wort.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmccclxxvi{Und Fir'aun ließ unter seinem Volk ausrufen; er sagte: ``O mein Volk, gehört mir nicht die Herrschaft über Ägypten und (auch) diese Flüsse, die unter mir strömen? Wollt ihr denn nicht einsichtig sein?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmccclxxvii{Oder bin ich nicht besser als dieser, der verächtlich ist und sich kaum deutlich äußern kann?\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmccclxxviii{Würden doch Armringe aus Gold auf ihn herabgeworfen oder die Engel mit ihm als Begleitung kommen!"\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmccclxxix{Er fand sein Volk leicht (zu beeinflussen), und da gehorchten sie ihm. Gewiß, sie waren ein Volk von Frevlern.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmccclxxx{Als sie Uns Kummer bereiteten, übten Wir an ihnen Vergeltung, und so ließen Wir sie allesamt ertrinken.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxi{Wir machten sie zu Vorgängern und zu einem Beispiel für die späteren (Geschlechter).\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxii{Und als der Sohn Maryams als Beispiel angeführt wurde, da brach dein Volk sogleich in Geschrei aus.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxiii{Und sie sagten: ``Sind unsere Götter besser oder er?" Sie führten ihn dir nur zum Streiten an. Nein! Vielmehr sind sie streitsüchtige Leute.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxiv{Er ist nur ein Diener, dem Wir Gunst erwiesen und den Wir zu einem Beispiel für die Kinder Isra'ils gemacht haben.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxv{Und wenn Wir wollten, könnten Wir euch wahrlich zu Engeln machen, die zu Statthaltern auf der Erde würden.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxvi{Und er ist wahrlich Wissen von der Stunde (des Gerichts)'. So hegt ja keinen Zweifel an ihr und folgt mir. Das ist ein gerader Weg.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxvii{Und der Satan soll euch ja nicht abhalten. Gewiß, er ist euch ein deutlicher Feind.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxviii{Und als 'Isa mit den klaren Beweisen kam, sagte er: ``Ich komme ja zu euch mit der Weisheit, und um euch einiges von dem klarzumachen, worüber ihr uneinig seid. So fürchtet Allah und gehorcht mir.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxix{Gewiß, Allah ist ja mein Herr und euer Herr; so dient Ihm. Das ist ein gerader Weg."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcccxc{Doch wurden die Gruppierungen untereinander uneinig; so wehe denjenigen, die Unrecht tun, vor der Strafe eines schmerzhaften Tages!\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcccxci{Erwarten sie nur die Stunde, daß sie plötzlich über sie kommt, ohne daß sie merken?\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcccxcii{Die Freunde werden an jenem Tag einer des anderen Feind sein, außer den Gottesfürchtigen.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcccxciii{``O meine Diener, keine Furcht soll heute über euch kommen, noch sollt ihr traurig sein,\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmcccxciv{ihr,) die ihr an Unsere Zeichen glaubtet und (Allah) ergeben wart.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcccxcv{Geht in den (Paradies)garten ein, euch und euren Gattinnen wird Freude bereitet."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcccxcvi{Es werden ihnen Schüsseln aus Gold und Trinkschalen herumgereicht; darin gibt es, was die Seelen begehren und köstlich für die Augen ist. ``Und ewig werdet ihr darin bleiben.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmcccxcvii{Das ist der (Paradies)garten, der euch zum Erbe gegeben worden ist für das, was ihr zu tun pflegtet.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmcccxcviii{Darin habt ihr vielerlei Früchte, von denen ihr eßt."\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmcccxcix{Gewiß, die Übeltäter (hingegen) werden in der Strafe der Hölle ewig bleiben.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmcd{Sie wird ihnen nicht herabgesetzt, und sie werden darin ganz verzweifelt sein.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmcdi{Nicht Wir haben ihnen Unrecht getan, sondern sie sind es, die Unrecht getan haben.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmcdii{Und sie rufen: ``O Malik, dein Herr soll unserem Leben ein Ende setzen." Er sagt: ``Gewiß, ihr werdet (hier) bleiben."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmcdiii{Wir sind zu euch mit der Wahrheit gekommen, aber den meisten von euch ist die Wahrheit zuwider.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmcdiv{Oder haben sie sich eine Sache ausgedacht? Auch Wir können Uns (etwas) ausdenken.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmmcdv{Oder meinen sie, daß Wir ihr Geheimes und ihre vertraulichen Gespräche nicht hören? Ja doch, Unsere Boten sind bei ihnen und schreiben (alles) auf.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmcdvi{Sag: Wenn der Allerbarmer Kinder hätte, wäre ich der erste derjenigen, die (ihnen) dienen.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmcdvii{Preis sei dem Herrn der Himmel und der Erde, dem Herrn des Thrones! (Erhaben ist Er) über das, was sie (Ihm) zuschreiben.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmcdviii{So lasse sie schweifende Gespräche führen und ihr Spiel treiben, bis sie ihrem Tag begegnen, der ihnen angedroht ist.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmcdix{Er ist es, Der im Himmel Gott und auf der Erde Gott ist; Er ist der Allweise und Allwissende.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmcdx{Und Segensreich ist Derjenige, Dem die Herrschaft der Himmel und der Erde und dessen, was dazwischen ist, gehört, Der das Wissen über die Stunde besitzt und zu Dem ihr zurückgebracht werdet!\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmcdxi{Und diejenigen, die sie anstatt Seiner anrufen, verfügen nicht über die Fürsprache, außer wer der Wahrheit entsprechend bezeugt, und sie wissen (es).\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmmcdxii{Wenn du sie fragst, wer sie erschaffen hat, sagen sie ganz gewiß: ``Allah". Wie lassen sie sich also abwendig machen?\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmmcdxiii{Und bei seinem Ausspruch: ``O mein Herr, gewiß, das sind Leute, die nicht glauben."\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmmcdxiv{Übe Nachsicht mit ihnen und sag: ``Friede!" Sie werden (es noch) erfahren.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmmcdxv{\basmalahde Ha-Mim\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcdxvi{Bei dem deutlichen Buch!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcdxvii{Wir haben es wahrlich in einer gesegneten Nacht herabgesandt - Wir haben ja (die Menschen) immer wieder gewarnt -,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcdxviii{in der jede weise Angelegenheit einzeln entschieden wird\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcdxix{als eine Angelegenheit von Uns aus - Wir haben ja immer wieder (Warner) gesandt -,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcdxx{als eine Barmherzigkeit von deinem Herrn - gewiß, Er ist der Allhörende und Allwissende -,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcdxxi{dem Herrn der Himmel und der Erde und dessen, was dazwischen ist, wenn ihr überzeugt seid.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcdxxii{Es gibt keinen Gott außer Ihm. Er macht lebendig und läßt sterben, (Er), euer Herr und der Herr eurer Vorväter.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcdxxiii{Aber nein! Sie sind im Zweifel und treiben ihr Spiel.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcdxxiv{So erwarte den Tag, an dem der Himmel deutlichen Rauch hervorbringt,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcdxxv{der die Menschen überdecken wird. Das ist eine schmerzhafte Strafe.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcdxxvi{``Unser Herr, nimm die Strafe von uns hinweg; wir sind (jetzt) ja gläubig."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcdxxvii{Wie soll ihnen (jetzt) die Selbstbesinnung nützen, wo bereits ein deutlicher Gesandter zu ihnen kam,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcdxxviii{sie sich hierauf aber von ihm abkehrten und sagten: ``Einer, dem vorgesagt wird, ein Besessener"?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcdxxix{Wir werden die Strafe ein wenig hinwegnehmen. Ihr werdet sicher zurückkehren.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcdxxx{Am Tag, da Wir mit der größten Gewalt zupacken werden, gewiß, da werden Wir Vergeltung üben.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxi{Und Wir setzten bereits vor ihnen das Volk Fir'auns der Versuchung aus. Zu ihnen kam ein edler Gesandter:\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxii{``Übergebt mir Allahs Diener. Gewiß, ich bin für euch ein vertrauenswürdiger Gesandter.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxiii{Und seid Allah gegenüber nicht überheblich. Gewiß, ich bringe euch eine deutliche Ermächtigung.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxiv{Und gewiß, ich nehme Zuflucht bei meinem Herrn und eurem Herrn davor, daß ihr mich steinigt.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxv{Und wenn ihr mir nicht glaubt, dann haltet euch (wenigstens) fern von mir."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxvi{Da rief er zu seinem Herrn: ``Diese sind ein Volk von Übeltätern."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxvii{- ``Zieh bei Nacht mit Meinen Dienern fort; ihr werdet ja verfolgt werden.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxviii{Und laß das Meer in Ruhe zurück'. Gewiß, sie sind eine Heerschar, die ertränkt werden soll."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxix{Wie viele Gärten und Quellen ließen sie zurück\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcdxl{und Getreidefelder und treffliche Stätte\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmcdxli{und ein angenehmes Leben, in dem sie vergnügt waren!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmcdxlii{So (war es). Und Wir gaben es anderen Leuten zum Erbe.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmcdxliii{Weder Himmel noch Erde weinte über sie, und ihnen wurde kein Aufschub gewährt.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmcdxliv{Und Wir erretteten bereits die Kinder lsra'ils von der schmachvollen Qual,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmcdxlv{von Fir'aun. Er war überheblich und einer der Maßlosen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmcdxlvi{Und Wir erwählten sie ja mit Wissen vor den (anderen) Weltenbewohnern\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmcdxlvii{und ließen ihnen von den Zeichen solche zukommen, die eine deutliche Prüfung enthielten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcdxlviii{Diese sagen fürwahr:\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmcdxlix{``Es gibt nur unseren ersten Tod. Und wir werden nicht auferstehen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmcdl{Bringt doch unsere Väter zurück, wenn ihr wahrhaftig seid."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcdli{Sind sie etwa besser oder das Volk Tubba's und diejenigen vor ihnen? Wir vernichteten sie; sie waren ja Übeltäter.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcdlii{Und Wir haben die Himmel und die Erde und was dazwischen ist, nicht zum Spiel erschaffen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcdliii{Wir haben sie nur in Wahrheit erschaffen. Aber die meisten von ihnen wissen nicht.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcdliv{Gewiß, der Tag der Entscheidung ist ihrer aller festgesetzte Zeit,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcdlv{der Tag, an dem kein Schutzherr seinem Schützling etwas nützen kann und ihnen keine Hilfe zuteil wird,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcdlvi{außer demjenigen, dessen Allah sich erbarmt. Er ist ja der Allmächtige und Barmherzige.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcdlvii{Gewiß, der Zaqqum-Baum\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcdlviii{ist die Speise des Sünders;\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcdlix{wie siedendes Öl kocht er in den Bäuchen,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcdlx{wie das heiße Wasser kocht.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcdlxi{``Nehmt ihn und schleppt ihn mitten in den Höllenbrand hinein.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcdlxii{Hierauf gießt über seinen Kopf von der Strafe des heißen Wassers.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcdlxiii{Koste (doch); du bist ja der Mächtige und Edle!"\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcdlxiv{``Das ist (es) ja, worüber ihr zu zweifeln pflegtet."\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcdlxv{Gewiß, die Gottesfürchtigen befinden sich an sicherer Stätte,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcdlxvi{in Gärten und an Quellen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcdlxvii{Sie tragen (Gewänder) aus Seidenbrokat und schwerem Brokat (,) und (sie liegen) einander gegenüber.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcdlxviii{So ist es. Und Wir geben ihnen als Gattinnen Huris mit schönen, großen Augen.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcdlxix{Sie rufen darin nach Früchten aller Art (und sind dort) in Sicherheit.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcdlxx{Sie kosten darin nicht den Tod, außer dem ersten Tod. Und Er bewahrt sie vor der Strafe des Höllenbrandes\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxi{als Huld von deinem Herrn. Das ist der großartige Erfolg.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxii{Wir haben ihn in deiner Sprache wahrlich leicht gemacht, auf daß sie bedenken mögen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxiii{Warte nun ab; auch sie warten ab.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxiv{\basmalahde Ha-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxv{Dies ist) die Offenbarung des Buches von Allah, dem Allmächtigen und Allweisen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxvi{In den Himmeln und auf der Erde sind wahrlich Zeichen für die Gläubigen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxvii{Und in eurer Erschaffung und in dem, was Er an Tieren sich ausbreiten läßt, sind Zeichen für Leute, die überzeugt sind.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxviii{Und (auch in) dem Unterschied von Nacht und Tag und (in) dem, was Allah an Versorgung vom Himmel herabkommen läßt und dann damit die Erde nach ihrem Tod wieder lebendig macht, und im Wechsel der Winde sind Zeichen für Leute, die begreifen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxix{Dies sind Allahs Zeichen, die Wir dir der Wahrheit entsprechend verlesen. An welche Aussage nach (derjenigen) Allah(s) und Seinen Zeichen wollen sie denn (sonst) glauben?\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxx{Wehe jedem durch und durch sündigen Lügner,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxi{der hört, wie ihm Allahs Zeichen verlesen werden, und hierauf hochmütig (in seinem Unglauben) verharrt, als ob er sie nicht gehört hätte! So verkünde ihm schmerzhafte Strafe.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxii{Und wenn er etwas von Unseren Zeichen kennenlernt, macht er sich darüber lustig. Für solche wird es schmachvolle Strafe geben.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxiii{Hinter ihnen (wartet) die Hölle. Und es nützt ihnen nicht, was sie erworben und (auch) nicht, was sie sich anstatt Allahs zu Schutzherren genommen haben; für sie wird es gewaltige Strafe geben.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxiv{Dies ist eine Rechtleitung. Und diejenigen, die die Zeichen ihres Herrn verleugnen, für sie wird es schmerzhafte Strafe von Züchtigung geben.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxv{Allah ist es, Der euch das Meer dienstbar gemacht hat, damit die Schiffe darauf auf Seinen Befehl fahren und damit ihr nach etwas von Seiner Huld trachtet, und auf daß ihr dankbar sein möget.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxvi{Und Er hat euch alles, was in den Himmeln und was auf der Erde ist, dienstbar gemacht, alles von Sich aus. Darin sind wahrlich Zeichen für Leute, die nachdenken.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxvii{Sag zu denjenigen, die glauben, sie sollen denjenigen vergeben, die nicht Allahs Tage erhoffen, auf daß Er einem (jeden) Volk das vergelte, was sie erworben haben.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxviii{Wer rechtschaffen handelt, der (tut es) zu seinem eigenen Vorteil, und wer Böses tut, der (tut es) zu seinem eigenen Nachteil. Hierauf werdet ihr zu eurem Herrn zurückgebracht.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxix{Wir gaben ja den Kindern Isra'ils die Schrift, das Urteil und das Prophetentum und versorgten sie von den guten Dingen und bevorzugten sie vor den (anderen) Weltenbewohnern.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcdxc{Und Wir gaben ihnen klare Beweise in der Angelegenheit (der Religion). Sie wurden aber erst uneinig, nachdem das Wissen zu ihnen gekommen war - aus Mißgunst untereinander. Gewiß, dein Herr wird am Tag der Auferstehung zwischen ihnen über das entscheiden, worüber sie uneinig zu sein pflegten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcdxci{Hierauf haben Wir dich auf eine Richtung in der Angelegenheit (der Religion) festgelegt. So folge ihr und folge nicht den Neigungen derjenigen, die nicht Bescheid wissen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcdxcii{Gewiß, sie werden dir vor Allah nichts nützen. Aber gewiß, die Ungerechten sind einer des anderen Schutzherren. Allah aber ist der Schutzherr der Gottesfürchtigen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcdxciii{Dies sind einsichtbringende Zeichen für die Menschen und Rechtleitung und Barm herzigkeit für Leute, die überzeugt sind.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcdxciv{Oder meinen diejenigen, die böse Taten verüben, daß Wir sie denjenigen, die glauben und rechtschaffene Werke tun, gleichstellen sowohl in ihrem Leben als in ihrem Tod? Wie böse ist, was sie urteilen!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcdxcv{Und Allah hat die Himmel und die Erde in Wahrheit erschaffen, und damit jeder Seele das vergolten wird, was sie erworben hat, und es wird ihnen kein Unrecht zugefügt.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcdxcvi{Was meinst du wohl zu jemandem, der sich als seinen Gott seine Neigung genommen hat, den Allah trotz (seines) Wissens hat in die Irre gehen lassen und dem Er das Gehör und das Herz versiegelt und auf dessen Augenlicht eine Hülle gelegt hat? Wer könnte ihn nach Allah rechtleiten? Bedenkt ihr denn nicht?\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcdxcvii{Und sie sagen: ``Es gibt nur unser diesseitiges Leben; wir sterben und leben, und nur die Zeiten vernichten uns." Sie haben (aber) kein Wissen davon, sie stellen nur Mutmaßungen an.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcdxcviii{Und wenn ihnen Unsere Zeichen als klare Beweise verlesen werden, ist ihre Beweisgrundlage nur, daß sie sagen: ``Bringt unsere Väter herbei, wenn ihr wahrhaftig seid."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcdxcix{Sag: Allah macht euch lebendig und läßt euch hierauf sterben; hierauf wird Er euch zum Tag der Auferstehung versammeln, an dem es keinen Zweifel gibt. Aber die meisten Menschen wissen nicht.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmd{Allah gehört die Herrschaft der Himmel und der Erde. Und am Tag, da sich die Stunde erhebt, an jenem Tag werden diejenigen verlieren, die Nichtiges tun.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdi{Und du wirst jede Gemeinschaft kniend sehen. Jede Gemeinschaft wird zu ihrem Buch gerufen: ``Heute wird euch das vergolten, was ihr zu tun pflegtet.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdii{Dies ist Unser Buch, das mit der Wahrheit gegen euch spricht. Wir ließen (alles) niederschreiben, was ihr zu tun pflegtet."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdiii{Was nun diejenigen angeht, die geglaubt und rechtschaffene Werke getan haben, so wird ihr Herr sie in Seine Barmherzigkeit eingehen lassen. Das ist der deutliche Erfolg.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdiv{Was aber diejenigen angeht, die ungläubig waren, so (wird zu ihnen gesagt werden): ``Sind euch Meine Zeichen nicht stets verlesen worden? Aber ihr habt euch hochmütig verhalten und seid ein Volk von Übeltätern gewesen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdv{Und wenn gesagt wurde: .Gewiß, Allahs Versprechen ist wahr, und an der Stunde gibt es keinen Zweifel', sagtet ihr: ,Wir wissen nicht, was die Stunde ist. Wir stellen ja nur Mutmaßungen an und sind gar nicht überzeugt'."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdvi{Und es werden ihnen die bösen Taten, die sie vollbracht haben, klar, und es umschließt sie das, worüber sie sich lustig zu machen pflegten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdvii{Und es wird gesagt: ``Heute werden Wir euch vergessen, wie ihr die Begegnung mit diesem eurem Tag vergessen habt. Euer Zufluchtsort ist das (Höllen)feuer, und ihr werdet keine Helfer haben.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdviii{Dies ist dafür, daß ihr euch über Allahs Zeichen lustig machtet und daß euch das diesseitige Leben täuschte." Heute werden sie nicht (mehr) aus ihm herausgebracht, und es wird ihnen keine Gnade (mehr) erwiesen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdix{So gehört (alles) Lob Allah, dem Herrn der Himmel und dem Herrn der Erde, dem Herrn der Weltenbewohner'!\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdx{Ihm gehört die Oberhoheit in den Himmeln und auf der Erde, und Er ist der Allmächtige und Allweise.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdxi{\basmalahde Ha-Mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdxii{Dies ist) die Offenbarung des Buches von Allah, dem Allmächtigen und Allweisen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdxiii{Wir haben die Himmel und die Erde und was dazwischen ist nur in Wahrheit und (auf) eine festgesetzte Frist erschaffen. Aber diejenigen, die ungläubig sind, wenden sich von dem ab, wovor sie gewarnt werden.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdxiv{Sag: Was meint ihr zu dem, was ihr anstatt Allahs anruft? Zeigt mir, was sie von der Erde erschaffen haben. Oder haben sie etwa an den Himmeln teil? Bringt mir doch ein Buch vor diesem herbei oder die (geringste) Spur von Wissen, wenn ihr wahrhaftig seid.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdxv{Und wer ist weiter abgeirrt als jemand, der anstatt Allahs jemanden anruft, der ihn nicht erhört bis zum Tag der Auferstehung? Und sie achten nicht auf ihr Bittgebet.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdxvi{Und wenn die Menschen versammelt werden, werden sie ihnen feind sein, und sie werden den von ihnen (empfangenen) Dienst verleugnen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdxvii{Und wenn ihnen Unsere Zeichen als klare Beweise verlesen werden, sagen diejenigen, die ungläubig sind, über die Wahrheit, da sie zu ihnen gekommen ist: ``Das ist deutliche Zauberei."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdxviii{Oder sagen sie etwa: ``Er hat ihn ersonnen"? Sag: Wenn ich ihn ersonnen haben sollte, so vermögt ihr für mich nichts gegen Allah (auszurichten). Er weiß sehr wohl, worüber ihr euch (ausgiebig) auslaßt. Er genügt als Zeuge zwischen mir und euch. Und Er ist der Allvergebende und Barmherzige.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdxix{Sag: Ich bin kein Neubeginn unter den Gesandten, und ich weiß nicht, was mit mir, und auch nicht, was mit euch geschehen wird. Ich folge nur dem, was mir (als Offenbarung) eingegeben wird, und ich bin nur ein deutlicher Warner.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdxx{Sag: Was meint ihr, wenn er doch von Allah ist und ihr ihn aber verleugnet, während ein Zeuge von den Kindern Isra'ils etwas bezeugt, was ihm gleich ist, und so glaubt er an ihn, während ihr euch hochmütig verhaltet? Gewiß, Allah leitet das Volk der Ungerechten nicht recht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdxxi{Und diejenigen, die ungläubig sind, sagen von denjenigen, die glauben: ``Wenn er etwas Gutes wäre, wären sie uns damit nicht zuvorgekommen." Und da sie sich nicht durch ihn rechtleiten lassen, werden sie sagen: ``Das ist eine alte unge heuerliche Lüge."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdxxii{Und vor ihm (war) die Schrift Musas als Vorbild und Barmherzigkeit. Und dies ist ein bestätigendes Buch in arabischer Sprache, um diejenigen, die Unrecht tun, zu warnen, und als frohe Botschaft für die Gutes Tuenden.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdxxiii{Gewiß, diejenigen, die sagen: ``Unser Herr ist Allah" und sich hierauf recht verhalten, über sie soll keine Furcht kommen, noch sollen sie traurig sein.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdxxiv{Das sind die Insassen des (Paradies)gartens, ewig darin zu bleiben als Lohn für das, was sie zu tun pflegten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdxxv{Und Wir haben dem Menschen anempfohlen, zu seinen Eltern gütig zu sein. Seine Mutter hat ihn unter Widerwillen getragen und unter Widerwillen zur Welt gebracht. Die (Zeit der) Schwangerschaft mit ihm bis zu seiner Entwöhnung (beträgt) dreißig Monate. Wenn er dann seine Vollreife erlangt hat und das Alter von vierzig Jahren erreicht hat, sagt er: ``Mein Herr, veranlasse mich, für Deine Gunst zu danken, die Du mir und meinen Eltern erwiesen hast, und rechtschaffen zu handeln, womit Du zufrieden bist. Und gib mir Rechtschaffenheit in meiner Nachkommenschaft. Ich wende mich Dir ja in Reue zu, und ich gehöre ja zu den (Dir) Ergebenen."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdxxvi{Das sind diejenigen, von denen Wir das Beste von dem, was sie getan haben, annehmen und über deren böse Taten Wir hinwegsehen, unter den Insassen des (Paradies)gartens - (dies ist) das wahrhaftige Versprechen, das ihnen stets gegeben wurde.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdxxvii{Und derjenige, der zu seinen Eltern sagt: ``Pfui über euch! Versprecht ihr mir etwa, ich sollte hervorgebracht werden, wo vor mir bereits Geschlechter dahingegangen sind?" während sie beide Allah um Hilfe anrufen: ``Wehe dir! glaube doch! Gewiß, Allahs Versprechen ist wahr." Da sagt er: ``Das sind nur Fabeln der Früheren."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdxxviii{Das sind diejenigen, gegen die das Wort unvermeidlich fällig geworden war, unter (anderen) Gemeinschaften von den Ginn und den Menschen, die bereits vor ihnen dahingegegangen sind. Gewiß, sie sind ja Verlierer.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdxxix{Für alle wird es Rangstufen geben gemäß dem, was sie getan haben, und damit Er ihnen ihre Taten voll erstatte, und es wird ihnen kein Unrecht zugefügt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdxxx{Und am Tag, da diejenigen, die ungläubig sind, dem (Höllen)feuer vorgeführt werden: ``Ihr habt eure guten Dinge im diesseitigen Leben dahingehen lassen und sie genossen. Heute wird euch mit der schmählichen Strafe vergolten, daß ihr euch auf der Erde ohne Recht hochmütig zu verhalten und daß ihr zu freveln pflegtet."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdxxxi{Und gedenke des Bruders der Ad', als er bei al-Ahqaf sein Volk warnte, wo bereits Warner vor ihm und (auch) nach ihm dahingingen: ``Ihr sollt nur Allah dienen. Gewiß, ich fürchte für euch die Strafe eines gewaltigen Tages."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdxxxii{Sie sagten: ``Bist du zu uns gekommen, um uns von unseren Göttern abwendig zu machen? So bring uns doch her, was du uns androhst, wenn du zu den Wahrhaftigen gehörst."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdxxxiii{Er sagte: ``Das Wissen (darüber) ist nur bei Allah; ich übermittele euch (nur) das, womit ich gesandt worden bin. Aber ich sehe, ihr seid Leute, die töricht sind."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdxxxiv{Als sie es als sich ausbreitende Wolke sahen, die auf ihre Täler zukam, sagten sie: ``Das ist eine sich ausbreitende Wolke, die uns Regen bringt." - ``Nein! Vielmehr ist es das, was ihr zu beschleunigen gewünscht habt: ein Wind, in dem es schmerzhafte Strafe gibt,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdxxxv{der auf den Befehl seines Herrn alles zerstört." So waren sie am Morgen (in einem Zustand), daß nichts anderes (von ihnen) als nur ihre Wohnungen zu sehen war. So vergelten Wir dem Volk der Übeltäter.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdxxxvi{Und Wir hatten ihnen ja eine feste Stellung verliehen darin, worin Wir euch keine feste Stellung verliehen haben. Und Wir hatten ihnen Gehör, Augenlicht und Herzen gegeben. Aber weder ihr Gehör, noch ihr Augenlicht, noch ihre Herzen nützten ihnen etwas, da sie Allahs Zeichen zu verleugnen pflegten, und es umschloß sie das, worüber sie sich lustig zu machen pflegten.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdxxxvii{Wir hatten bereits vernichtet, was an Städten in eurer Umgebung ist. Und Wir hatten die Zeichen verschiedenartig dargelegt, auf daß sie umkehren mochten.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdxxxviii{Wenn ihnen doch diejenigen geholfen hätten, die sie sich anstatt Allahs zu Göttern nahmen (als Vermittler), um sich Zutritt in Seine Nähe zu verschaffen! Aber nein! Sie entschwanden ihnen. Dies war ihre ungeheuerlich Lüge und das, was sie zu ersinnen pflegten.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdxxxix{Und (gedenke,) als Wir eine kleinere Schar Ginn veranlaßten, sich zu dir zu begeben und dem Qur'an zuzuhören. Als sie zu ihm eingefunden hatten, sagten sie: ``Horcht hin!" Als er dann zum Ende kam, kehrten sie zu ihrem Volk zurück, um sie zu warnen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdxl{Sie sagten: ``O unser Volk, wir haben ein Buch gehört, das nach Musa (als Offenbarung) herabgesandt worden ist, das zu bestätigen, was vor ihm war, und das zur Wahrheit und zu einem geraden Weg leitet.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdxli{O unser Volk, erhört Allahs Rufer und glaubt an ihn, so vergibt Er euch (etwas) von euren Sünden und gewährt euch Schutz vor schmerzhafter Strafe.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdxlii{Wer Allahs Rufer nicht erhört, wird sich (Ihm) auf der Erde doch nicht entziehen können, und er hat außer Ihm keine Schutzherren. Jene befinden sich in deutlichem Irrtum."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdxliii{Sehen sie denn nicht, daß Allah, Der die Himmel und die Erde erschaffen hat und bei ihrer Erschaffung nicht ermüdet ist, (auch) die Macht hat, die Toten wieder le bendig zu machen? Ja doch, gewiß, Er hat zu allem die Macht.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdxliv{Und am Tag, da diejenigen, die ungläubig sind, dem (Höllen)feuer vorgeführt werden: ``Ist dies nicht die Wahrheit?" Sie werden sagen: ``Ja, doch, bei unserem Herrn!" Er wird sagen: ``Kostet nun die Strafe dafür, daß ihr stets ungläubig wart."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdxlv{Sei nun standhaft, wie diejenigen der Gesandten, die Entschlossenheit besaßen, standhaft waren; und wünsche nichts gegen sie zu beschleunigen. Am Tag, da sie sehen, was ihnen angedroht wird, glauben sie, nur eine Stunde eines Tages verweilt zu haben. (Dies ist) eine Botschaft. Wird denn jemand vernichtet außer dem Volk der Frevler?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdxlvi{\basmalahde Denjenigen, die ungläubig sind und von Allahs Weg abhalten, läßt Er ihre Werke fehlgehen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdxlvii{Denjenigen aber, die glauben und rechtschaffene Werke tun und an das glauben, was Muhammad offenbart worden ist - und es ist (ja) die Wahrheit von ihrem Herrn -, tilgt Er ihre bösen Taten und bessert ihren Gemütszustand.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdxlviii{Dies, weil diejenigen, die ungläubig sind, dem Falschen folgen, und diejenigen, die glauben, der Wahrheit von ihrem Herrn folgen. So prägt Allah den Menschen ihre Gleichnisse.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdxlix{Wenn ihr auf diejenigen, die ungläubig sind, (im Kampf) trefft, dann schlagt den Hals. Wenn ihr sie schließlich schwer niedergeschlagen habt, dann legt (ihnen) die Fesseln fest an. Danach (laßt sie) als Wohltat frei oder gegen Lösegeld, bis der Krieg seine Lasten ablegt. Dies (soll so sein)! Und wenn Allah wollte, würde Er sie wahrlich (allein) besiegen . Er will aber damit die einen von euch durch die anderen prüfen. Und denjenigen, die auf Allahs Weg getötet werden, wird Er ihre Werke nicht fehlgehen lassen;\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdl{Er wird sie rechtleiten und ihren Gemütszustand bessern\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdli{und sie in den (Paradies)garten eingehen lassen, den Er ihnen kenntlich gemacht hat.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdlii{O die ihr glaubt, wenn ihr Allah(s Sache) helft, hilft Er euch und festigt eure Füße.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdliii{Diejenigen aber, die ungläubig sind, so sollen sie (in Unglück) fallen! Und Er wird ihre Werke fehlgehen lassen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdliv{Dies, weil ihnen zuwider ist, was Allah (als Offenbarung) herabgesandt hat, und so läßt Er ihre Werke hinfällig werden.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdlv{Sind sie denn nicht auf der Erde umhergereist, so daß sie schauen (konnten), wie das Ende derjenigen vor ihnen war? Allah hat über sie Zerstörung gebracht. Für die Ungläubigen wird es etwas dergleichen geben.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdlvi{Dies, weil Allah der Schutzherr derjenigen ist, die glauben, und weil die Ungläubigen keinen Schutzherrn haben.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdlvii{Allah läßt diejenigen, die glauben und rechtschaffene Werke tun, in Gärten eingehen, durcheilt von Bächen'. Diejenigen aber, die ungläubig sind, genießen und essen (im Diesseits), wie das Vieh frißt; das (Höllen)feuer wird ein Aufent haltsort für sie sein.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdlviii{Und wie viele Städte, die eine stärkere Kraft hatten als deine Stadt, die dich vertrieben hat, haben Wir vernichtet; und da hatten sie keine Helfer.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdlix{Ist denn einer, der sich auf einen klaren Beweis von seinem Herrn stützt, wie jemand, dem sein böses Tun ausgeschmückt wird (,) und (wie diejenigen,) die ihren Neigungen folgen?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdlx{Das Gleichnis des (Paradies)gartens, der den Gottesfürchtigen versprochen ist: Darin sind Bäche mit Wasser, das nicht schal wird, und Bäche mit Milch, deren Geschmack sich nicht ändert, und Bäche mit Wein, der köstlich ist für diejenigen, die (davon) trinken, und Bäche mit geklärtem Honig. Und sie haben darin von allen Früchten und Vergebung von ihrem Herrn. (Sind diese denn) jemandem gleich, der im (Höllen)feuer ewig bleibt und dem heißes Wasser zu trinken gegeben wird, das seine Gedärme zerreißt?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdlxi{Und unter ihnen gibt es manche, die dir zuhören. Wenn sie aber von dir hinausgehen, sagen sie zu denjenigen, denen das Wissen gegeben worden ist: ``Was hat er eben gesagt?" Das sind diejenigen, deren Herzen Allah versiegelt hat und die ihren Neigungen folgen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdlxii{Und denjenigen, die rechtgeleitet sind, mehrt Er die Rechtleitung und verleiht ihnen ihre Gottesfurcht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdlxiii{Erwarten sie denn etwas anderes, als daß die Stunde plötzlich über sie kommt? So sind ihre Vorzeichen bereits eingetroffen. Wie soll ihnen dann ihre Selbstbesinnung nützen, wenn sie über sie kommt?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdlxiv{Wisse also, daß es keinen Gott außer Allah gibt. Und bitte um Vergebung für deine Sünde und für die gläubigen Männer und die gläubigen Frauen. Allah kennt euren Wandel und euren Aufenthalt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdlxv{Und diejenigen, die glauben, sagen: ``Wäre doch eine Sura offenbart worden!" Wenn aber eine eindeutige Sura herabgesandt wird und darin der Kampf erwähnt wird, siehst du diejenigen, in deren Herzen Krankheit ist, dich anschauen, wie einer schaut, der im Sterben ohnmächtig wird. Näherliegender wären für sie\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdlxvi{Gehorsam und geziemende Worte. Wenn die Angelegenheit beschlossen ist, dann wäre es wahrlich besser für sie, sie würden Allah gegenüber wahrhaftig sein.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdlxvii{Werdet ihr vielleicht, wenn ihr euch abkehrt, auf der Erde Unheil stiften und eure Verwandtschaftsbande zerreißen?\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdlxviii{Das sind diejenigen, die Allah verflucht; so macht Er sie taub und läßt ihr Augenlicht erblinden.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdlxix{Denken sie denn nicht sorgfältig über den Qur'an nach? Oder sind an (diesen) Herzen deren Verriegelungen (angebracht)?\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdlxx{Gewiß, diejenigen, die den Rücken kehren, nachdem ihnen die Rechtleitung klar geworden ist, denen hat der Satan (etwas) eingeredet und Hoffnung auf Aufschub gemacht.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdlxxi{Dies, weil sie zu denjenigen sagen, denen zuwider ist, was Allah offenbart hat: ``Wir werden euch in einigen Angelegenheiten gehorchen." Aber Allah weiß, was sie im geheimen sagen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdlxxii{Wie (wird es sein), wenn die Engel sie abberufen, wobei sie ihre Gesichter und ihre Rücken schlagen?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdlxxiii{Dies, weil sie dem folgen, was Allah mißfällt, und ihnen Sein Wohlgefallen zuwider ist, und so läßt Er ihre Werke hinfällig werden.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdlxxiv{Oder meinen diejenigen, in deren Herzen Krankheit ist, Allah würde ihren Groll nicht zum Vorschein bringen?\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdlxxv{Und wenn Wir wollten, würden Wir sie dir fürwahr zeigen, und so würdest du sie sicher an ihrem Merkmal erkennen. Und du wirst sie ganz gewiß an ihrer schiefen Sprache erkennen. Und Allah weiß über eure Werke Bescheid.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdlxxvi{Und Wir werden euch ganz gewiß prüfen, bis Wir feststellen, welche sich abmühen von euch und welche standhaft sind, und bis Wir eure Werke prüfen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdlxxvii{Gewiß, diejenigen, die ungläubig sind, von Allahs Weg abhalten und dem Gesandten entgegenwirken, nachdem ihnen die Rechtleitung klar geworden ist, können Allah in nichts schaden. Und Er wird ihre Werke hinfällig werden lassen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdlxxviii{O die ihr glaubt, gehorcht Allah und gehorcht dem Gesandten und macht eure Werke nicht zunichte.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdlxxix{Gewiß, diejenigen, die ungläubig sind und von Allahs Weg abhalten und hierauf als Ungläubige sterben, so wird Allah ihnen nicht vergeben.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdlxxx{So werdet nicht schwach und ruft (nicht) zum Frieden, wo ihr doch die Oberhand haben werdet, denn Allah ist mit euch, und Er wird euch nicht um eure Werke bringen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxi{Das diesseitige Leben ist nur Spiel und Zerstreuung. Wenn ihr glaubt und gottesfürchtig seid, wird Er euch euren Lohn geben und von euch nicht euren (gesamten) Besitz verlangen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxii{Wenn Er ihn von euch verlangte und euch (dadurch) in Bedrängnis brächte, dann würdet ihr damit geizen, und Er würde (dadurch) euren Groll zum Vorschein bringen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxiii{Ihr da seid es doch, die ihr aufgerufen werdet, auf Allahs Weg auszugeben. Jedoch gibt es unter euch welche, die geizen. Wer aber geizt, der geizt nur gegen sich selbst. Allah ist der Unbedürftige, ihr aber seid die Bedürftigen. Wenn ihr euch abkehrt, wird Er euch durch ein anderes Volk ersetzen, und sie werden dann nicht gleich euch sein.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxiv{\basmalahde Gewiß, Wir haben dir einen deutlichen Sieg verliehen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxv{damit dir Allah das von deinen Sünden vergebe, was vorher war und was später sein wird, und damit Er Seine Gunst an dir vollende und dich einen geraden Weg leite\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxvi{und (damit) Allah dir helfe mit mächtiger Hilfe.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxvii{Er ist es, der die innere Ruhe in die Herzen der Gläubigen herabgesandt hat, damit sie in ihrem Glauben noch an Glauben zunehmen. Und Allah gehören die Heerscharen der Himmel und der Erde. Und Allah ist Allwissend und Allweise.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxviii{Dies,) damit Er die gläubigen Männer und die gläubigen Frauen in Gärten eingehen lasse, durcheilt von Bächen - ewig darin zu bleiben -, und ihnen ihre bösen Taten tilge; das ist bei Allah ein großartiger Erfolg.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxix{Und (damit) Er die Heuchler und Heuchlerinnen und die Götzendiener und Götzendienerinnen strafe, die von Allah die böse Erwartung hegen. Gegen sie wird die böse Schicksalswendung sein. Allah zürnt ihnen, verflucht sie und bereitet ihnen die Hölle - (wie) böse ist der Ausgang!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdxc{Und Allah gehören die Heerscharen der Himmel und der Erde. Und Allah ist Allmächtig und Allweise.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdxci{Wir haben dich ja als Zeugen, Verkünder froher Botschaft und Warner gesandt,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdxcii{damit ihr an Allah und Seinen Gesandten glaubt, ihm beisteht und ihn hochachtet und (damit ihr) Ihn preist morgens und abends.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdxciii{Gewiß, diejenigen, die dir den Treueid leisten, leisten (in Wirklichkeit) nur Allah den Treueid; Allahs Hand ist über ihren Händen. Wer nun (sein Wort) bricht, bricht es nur zu seinem eigenen Nachteil; wer aber das einhält, wozu er sich Allah gegenüber verpflichtet hat, dem wird Er großartigen Lohn geben.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdxciv{Die Zurückgelassenen der Wüstenaraber werden zu dir sagen: ``Unser Besitz und unsere Angehörigen haben uns (zu sehr) beschäftigt; so bitte für uns um Vergebung." Sie sagen mit ihren Zungen, was nicht in ihren Herzen ist. Sag: Wer vermag denn für euch bei Allah etwas (zu ändern), wenn Er für euch Schaden will oder wenn Er für euch Nutzen will? Aber nein! Allah ist dessen, was ihr tut, Kundig.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdxcv{Aber nein! Ihr meintet, daß der Gesandte und die Gläubigen niemals mehr zu ihren Angehörigen zurückkehren würden, und das ist euch in euren Herzen ausgeschmückt worden. Und ihr hegtet die böse Erwartung und wart ein Volk des Niedergangs.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdxcvi{Doch wer an Allah und Seinen Gesandten nicht glaubt - gewiß, so haben Wir für die Ungläubigen eine Feuerglut bereitet.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdxcvii{Und Allah gehört die Herrschaft der Himmel und der Erde. Er vergibt, wem Er will, und Er straft, wen Er will. Und Allah ist Allvergebend und Barmherzig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdxcviii{Die Zurückgelassenen werden, wenn ihr loszieht, um Beute zu machen, sagen: ``Laßt uns euch folgen", indem sie Allahs Wort abändern wollen. Sag: ``Ihr werdet uns nicht folgen. So hat Allah (schon) zuvor gesprochen." Dann werden sie sagen: ``Nein! Vielmehr beneidet ihr uns." Aber nein! Sie verstehen ja nur wenig.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdxcix{Sag zu den Zurückgelassenen der Wüstenaraber: ``Ihr werdet zu Leuten gerufen werden, die eine starke Gewalt besitzen, gegen die ihr kämpfen sollt, es sei denn, sie werden vorher Muslime. Wenn ihr gehorcht, gibt Allah euch schönen Lohn; wenn ihr euch jedoch abkehrt, wie ihr euch zuvor abgekehrt habt, wird Er euch mit schmerzhafter Strafe strafen."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdc{Keinen Grund zur Bedrängnis gibt es für den Blinden, und keinen Grund zur Bedrängnis gibt es für den Hinkenden, und keinen Grund zur Bedrängnis gibt es für den Kranken. Wer Allah und Seinem Gesandten gehorcht, den wird Er in Gärten eingehen lassen, durcheilt von Bächen. Wer sich jedoch abkehrt, den straft Er mit schmerzhafter Strafe.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdci{Allah hatte ja Wohlgefallen an den Gläubigen, als sie dir unter dem Baum den Treueid leisteten. Er wußte, was in ihren Herzen war, und da sandte Er die innere Ruhe auf sie herab und belohnte sie mit einem nahen Sieg\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcii{und viel Beute, die sie machen werden. Und Allah ist Allmächtig und Allweise.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdciii{Allah hat euch versprochen, daß ihr viel Beute machen werdet. So hat Er euch diese schnell gewährt und die Hände der Menschen von euch zurückgehalten, und (dies,) damit es ein Zeichen für die Gläubigen sei und Er euch einen geraden Weg leite.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdciv{Auch eine andere (Beute), die ihr (noch) nicht zu erlangen vermochtet, hat Allah bereits umfangen. Allah hat zu allem die Macht.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcv{Und wenn diejenigen, die ungläubig sind, gegen euch gekämpft hätten, hätten sie (euch) wahrlich den Rücken gekehrt. Hierauf finden sie weder Schutzherrn noch Helfer.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcvi{So war) Allahs Gesetzmäßigkeit, die bereits zuvor ergangen war. Und du wirst in Allahs Gesetzmäßigkeit keine Änderung finden.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcvii{Und Er ist es, Der im Talgrund von Makka ihre Hände von euch und eure Hände von ihnen zurückgehalten hat, nachdem Er euch den Sieg über sie verliehen hatte. Und Allah sieht, was ihr tut, wohl.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcviii{Sie sind es, die ungläubig gewesen sind und euch von der geschützten Gebetsstätte abgehalten und die Opfertiere aufgehalten haben, daß sie (nicht) ihren Schlachtort erreichen. Und wenn es (dort) nicht gläubige Männer und gläubige Frauen gegeben hätte, die ihr nicht kanntet, und damit ihr sie nicht niedertretet und damit euch nicht dadurch Schande ohne Wissen trifft, (haben Wir ..?. Dies,) damit Allah in Seine Barmherzigkeit eingehen läßt, wen Er will. Wenn diese sich (von den Un gläubigen) deutlich getrennt hätten, hätten Wir diejenigen von ihnen, die ungläubig sind, wahrlich mit schmerzhafter Strafe gestraft.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcix{Als diejenigen, die ungläubig waren, in ihren Herzen die Hitzigkeit entfachten, die Hitzigkeit der Unwissenheit -, da sandte Allah Seine innere Ruhe auf Seinen Gesandten und auf die Gläubigen herab und ließ sie an dem Wort der Gottesfurcht festhalten. Sie hatten ja ein größeres Anrecht darauf und waren dessen würdig. Und Allah weiß über alles Bescheid.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdcx{Allah hat ja Seinem Gesandten das Traumgesicht der Wahrheit entsprechend wahr gemacht: Ihr werdet ganz gewiß, wenn Allah will, die geschützte Gebetsstätte in Sicherheit betreten, sowohl mit geschorenem Kopf als auch (mit) gekürztem Haar), und ohne euch zu fürchten. Er wußte doch, was ihr nicht wußtet, und so be stimmte Er (für euch) vorher einen nahen Sieg.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcxi{Er ist es, Der Seinen Gesandten mit der Rechtleitung und der Religion der Wahrheit gesandt hat, um ihr die Oberhand über alle Religion zu geben. Und Allah genügt als Zeuge.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcxii{Muhammad ist Allahs Gesandter. Und diejenigen, die mit ihm sind, sind den Ungläubigen gegenüber hart, zueinander aber barmherzig. Du siehst sie sich verbeugen und niederwerfen, indem sie nach Huld von Allah und Wohlgefallen trachten. Ihr Merkmal steht auf ihren Gesichtern durch die Niederwerfung. Das ist ihr Gleichnis in der Tora. Und ihr Gleichnis im Evangelium ist das eines Getreidefeldes, das seine Triebe hervorbringt und dann stärker werden läßt, so daß sie verdicken und ebenmäßig auf ihren Halmen stehen, so daß es den Anbauern gefällt. (Dies,) damit Er die Ungläubigen durch sie ergrimmen lasse. Allah hat denjenigen von ihnen, die glauben und rechtschaffene Werke tun, Vergebung und großartigen Lohn versprochen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdcxiii{\basmalahde O die ihr glaubt, kommt nicht Allah und Seinem Gesandten zuvor und fürchtet Allah. Gewiß, Allah ist Allhörend und Allwissend.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcxiv{O die ihr glaubt, erhebt nicht eure Stimmen über die Stimme des Propheten, und sprecht nicht so laut zu ihm, wie ihr laut zueinander sprecht, auf daß (nicht) eure Werke hinfällig werden, ohne daß ihr merkt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcxv{Gewiß, diejenigen, die ihre Stimmen bei Allahs Gesandtem mäßigen, das sind diejenigen, deren Herzen Allah auf die Gottesfurcht geprüft hat. Für sie wird es Vergebung und großartigen Lohn geben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcxvi{Gewiß, diejenigen, die dich hinter den (Wänden der) Gemächer rufen, die meisten von ihnen haben keinen Verstand.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcxvii{Wenn sie sich gedulden würden, bis du zu ihnen herauskommst, wäre es wahrlich besser für sie. Und Allah ist Allvergebend und Barmherzig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdcxviii{O die ihr glaubt, wenn ein Frevler zu euch mit einer Kunde kommt, dann schafft Klarheit, damit ihr (nicht einige) Leute in Unwissenheit (mit einer Anschuldigung) trefft und dann über das, was ihr getan habt, Reue empfinden werdet.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdcxix{Und wißt, daß Allahs Gesandter unter euch ist. Wenn er euch in vielen Angelegenheiten gehorchte, würdet ihr wahrlich in Bedrängnis kommen. Aber Allah hat euch den Glauben lieb gemacht und in euren Herzen ausgeschmückt, und Er hat euch den Unglauben, den Frevel und den Ungehorsam verabscheuen lassen. Das sind diejenigen, die besonnen handeln.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdcxx{Dies als Huld und Gunst von Allah. Und Allah ist Allwissend und Allweise.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdcxxi{Und wenn zwei Gruppen von den Gläubigen miteinander kämpfen, so stiftet Frieden zwischen ihnen. Wenn die eine von ihnen gegen die andere widerrechtlich vorgeht, dann kämpft gegen diejenige, die widerrechtlich vorgeht, bis sie zu Allahs Befehl zurückkehrt. Wenn sie zurückkehrt, dann stiftet Frieden zwischen ihnen nach Gerechtigkeit und handelt dabei gerecht. Allah liebt ja die Gerechten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdcxxii{Die Gläubigen sind doch Brüder. So stiftet Frieden zwischen euren beiden Brüdern und fürchtet Allah, auf daß ihr Erbarmen finden möget.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdcxxiii{O die ihr glaubt, die einen sollen nicht über die anderen spotten, vielleicht sind eben diese besser als sie. Auch sollen nicht Frauen über andere Frauen (spotten), vielleicht sind eben diese besser als sie. Und beleidigt euch nicht gegenseitig durch Gesten und bewerft euch nicht gegenseitig mit (häßlichen) Beinamen. Wie schlimm ist die Bezeichnung ``Frevel" nach (der Bezeichnung) ``Glaube"! Und wer nicht bereut, das sind die Ungerechten.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdcxxiv{O die ihr glaubt, meidet viel von den Mutmaßungen; gewiß, manche Mutmaßung ist Sünde. Und sucht nicht (andere) auszukundschaften und führt nicht üble Nachrede übereinander. Möchte denn einer von euch gern das Fleisch seines Bruders, wenn er tot sei, essen? Es wäre euch doch zuwider. Fürchtet Allah. Gewiß, Allah ist Reue-Annehmend und Barmherzig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdcxxv{O ihr Menschen, Wir haben euch ja von einem männlichen und einem weiblichen Wesen erschaffen, und Wir haben euch zu Völkern und Stämmen gemacht, damit ihr einander kennenlernt. Gewiß, der Geehrteste von euch bei Allah ist der Gottesfürchtigste von euch. Gewiß, Allah ist Allwissend und Allkundig.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdcxxvi{Die Wüstenaraber sagen: ``Wir glauben." Sag: Ihr glaubt nicht (wirklich), sondern sagt: ,Wir sind Muslime geworden', denn der Glaube ist noch nicht in eure Herzen eingezogen. Wenn ihr aber Allah und Seinem Gesandten gehorcht, verringert Er euch nichts von euren Werken. Gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdcxxvii{Die (wahren) Gläubigen sind ja diejenigen, die an Allah und Seinen Gesandten glauben und hierauf nicht zweifeln und sich mit ihrem Besitz und mit ihrer eigenen Person auf Allahs Weg abmühen. Das sind die Wahrhaftigen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdcxxviii{Sag: Wollt ihr Allah über eure Religion belehren, wo Allah weiß, was in den Himmeln und was auf der Erde ist? Und Allah weiß über alles Bescheid.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcxxix{Sie halten es dir als Wohltat vor, daß sie Muslime geworden sind. Sag: Haltet mir nicht eure (Annahme des) Islams als Wohltat vor. Nein! Vielmehr hält Allah euch die Wohltat vor, daß Er euch zum Glauben geleitet hat, wenn ihr wahrhaftig seid.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcxxx{Gewiß, Allah kennt das Verborgene der Himmel und der Erde. Und Allah sieht wohl, was ihr tut.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxi{\basmalahde Qaf. Bei dem ruhmvollen Qur'an!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxii{Aber nein! Sie wundern sich darüber, daß ein Überbringer von Warnungen von ihnen selbst zu ihnen gekommen ist. Da sagen die Ungläubigen: ``Das ist eine verwunderliche Sache.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxiii{Wenn wir gestorben und zu Erde geworden sind ... (, sollten wir dann wirklich wieder auferstehen)? Das ist doch eine weitentfernte Rückkehr."\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxiv{Wir wissen bestimmt, was die Erde von ihnen verringert. Und bei Uns ist ein Buch, das alles bewahrt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxv{Aber nein! Sie haben die Wahrheit für Lüge erklärt, als sie zu ihnen gekommen ist, und so befinden sie sich (stets) in Unbeständigkeit.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxvi{Schauen sie denn nicht zum Himmel über ihnen, wie Wir ihn aufgebaut und geschmückt haben, und daß er keine Spalten hat?\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxvii{Und die Erde haben Wir gedehnt und darauf festgegründete Berge gesetzt, und Wir haben auf ihr von jeder entzückenden (Pflanzen)art wachsen lassen,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxviii{zur Einsicht und Ermahnung für jeden reuigen Diener.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxix{Und Wir lassen vom Himmel gesegnetes Wasser herabkommen, womit Wir dann Gärten und Korn, das man erntet, wachsen lassen,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdcxl{und Palmen, hochreichend und mit übereinander gereihten Blütenkolben,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdcxli{als Versorgung für die Diener. Und Wir machen damit (manch) totes Land wieder lebendig; so wird auch das Herauskommen (erfolgen).\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdcxlii{Der Lüge bezichtigten vor ihnen das Volk Nuhs und die Leute von ar-Rass und die Tamud\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdcxliii{und die 'Ad und Fir'aun und die Brüder Luts\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdcxliv{und die Bewohner des Dickichts und das Volk Tubba's. Alle bezichtigten die Gesandten der Lüge, so ist Meine Androhung unvermeidlich fällig geworden.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdcxlv{Ermüdeten Wir denn durch die erste Erschaffung? Nein! Vielmehr ist ihnen verdeckt (geblieben), was eine neue Erschaffung betrifft.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdcxlvi{Wir haben ja den Menschen erschaffen und wissen, was (alles ihm) seine Seele einflüstert, und Wir sind ihm doch näher als seine Halsschlagader,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcxlvii{wo die beiden Empfänger (der Taten) empfangen, zur Rechten und zur Linken sitzend.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcxlviii{Kein Wort äußert er, ohne daß bei ihm ein Beobachter bereit wäre.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcxlix{Und es wird die Trunkenheit des Todes mit der Wahrheit kommen: ``Das ist das, wovor du auszuweichen pflegtest."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdcl{Und es wird ins Horn geblasen; das ist der angedrohte Tag.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcli{Und jede Seele wird kommen und mit ihr ein Treiber und ein Zeuge.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdclii{``Du warst dessen ja unachtsam. Nun haben Wir deine Decke von dir hinweggenommen, so daß dein Blick heute scharf ist."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcliii{Und sein Geselle wird sagen: ``Das ist, was bei mir bereit ist."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcliv{- ``Werft, ihr beide, in die Hölle jeden beharrlichen, widerspenstigen Ungläubigen,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdclv{Verweigerer des Guten, der Übertretungen begeht und sich in starkem Zweifel befindet,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdclvi{der neben Allah einen anderen Gott setzt. So werft ihn in die strenge Strafe."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdclvii{Sein Geselle wird sagen: ``Unser Herr, ich habe ihn nicht zu einem Übermaß (an Frevel) verleitet, sondern er (selbst) befand sich in tiefem Irrtum."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdclviii{Er (Allah) wird sagen: ``Streitet nicht miteinander vor Mir, wo Ich euch doch die Androhung vorausgeschickt habe.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdclix{Das Wort wird bei Mir nicht abgeändert, und Ich bin keiner, der den Dienern Unrecht zufügt."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdclx{Am Tag, da Wir zur Hölle sagen werden: ``Bist du voll geworden?", und sie sagt: ``Gibt es denn noch mehr?"\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdclxi{Und nahe herangebracht wird der (Paradies)garten an die Gottesfürchtigen, gar nicht fern:\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdclxii{``Das ist, was euch versprochen worden ist, (versprochen) für jeden immer wieder Umkehrbereiten und (Allahs Gebote) Hütenden,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdclxiii{der den Allerbarmer im verborgenen fürchtet und mit reuigem Herzen (zu Ihm) kommt.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdclxiv{Betretet ihn in Frieden. Das ist der Tag der Ewigkeit."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdclxv{Sie werden, was sie wollen, darin haben. Und bei Uns ist noch mehr.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdclxvi{Wie viele Geschlechter vernichteten Wir vor ihnen, die eine stärkere Gewalt hatten als sie und in den Landstrichen umherzogen! Gibt es denn ein Entrinnen?\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdclxvii{Darin ist wahrlich eine Ermahnung für jemanden, der Herz hat oder hinhört, während er (geistig) anwesend ist.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdclxviii{Und Wir haben ja die Himmel und die Erde und das, was dazwischen ist, in sechs Tagen erschaffen, wobei Uns keine Ermüdung überkommen hat.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdclxix{So ertrage standhaft, was sie sagen, und lobpreise deinen Herrn vor dem Aufgang der Sonne und vor dem Untergang,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdclxx{und (in einem Teil) von der Nacht, da preise Ihn, und am Ende der Niederwerfung.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdclxxi{Und höre zu am Tag, da der Rufer von einem nahen Ort aus ruft,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdclxxii{am Tag, da sie den Schrei in Wahrheit hören werden; das ist der Tag des Herauskommens.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdclxxiii{Gewiß, Wir sind es, die Wir wieder lebendig machen und sterben lassen, und zu Uns ist der Ausgang.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdclxxiv{- Am Tag, da die Erde, sie freilassend, sich ihnen aufspalten wird und sie eilig (hervorkommen). Das ist für Uns ein leichtes Versammeln.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdclxxv{Wir wissen sehr wohl, was sie sagen, und du bist nicht als Gewalthaber über sie (eingesetzt). Darum ermahne (nur) mit dem Qur'an (jeden,) wer Meine Androhung fürchtet.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdclxxvi{\basmalahde Bei den heftig Zerstreuenden,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdclxxvii{dann den eine Last Tragenden,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdclxxviii{dann den leicht Dahinziehenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdclxxix{dann den Angelegenheiten Regelnden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdclxxx{sicherlich, was euch angedroht wird, ist wahr,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxi{und das Gericht wird gewiß hereinbrechen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxii{Beim Himmel in seiner Vollkommenheit;\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxiii{ihr äußert fürwahr unterschiedliche Reden.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxiv{Abwendig machen läßt sich davon, wer sich abwendig machen läßt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxv{Tod den Mutmaßern,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxvi{die in Versenkung zerstreut sind.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxvii{Sie fragen: ``Wann wird denn der Tag des Gerichts sein?"\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxviii{- Am Tag, da sie im Feuer geprüft werden.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxix{``Kostet (nun) eure Prüfung. Das ist, was ihr zu beschleunigen wünschtet."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdcxc{Gewiß, die Gottesfürchtigen werden in Gärten und an Quellen sein,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdcxci{sie nehmen, was ihr Herr ihnen gegeben hat, denn sie pflegten davor Gutes zu tun.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcxcii{Nur ein wenig pflegten sie in der Nacht zu schlafen,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcxciii{und im letzten Teil der Nacht pflegten sie um Vergebung zu bitten,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcxciv{und (sie gestanden) an ihrem Besitz dem Bettler und dem Unbemittelten ein Anrecht (zu).\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdcxcv{Und auf der Erde gibt es Zeichen für die Überzeugten\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcxcvi{und (auch) in euch selbst. Seht ihr denn nicht?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcxcvii{Und im Himmel ist eure Versorgung und das, was euch versprochen wird.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcxcviii{Beim Herrn des Himmels und der Erde, das ist gewiß so wahr, gleichermaßen wie ihr reden könnt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcxcix{Ist zu dir die Geschichte von den geehrten Gästen Ibrahims gekommen?\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcc{Als sie bei ihm eintraten und sagten: ``Frieden!" Er sagte: ``Friede! - Fremde Leute."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcci{Er schlich sich zu seinen Angehörigen und brachte (zum Essen) dann ein ansehnliches Kalb her.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdccii{Er setzte es ihnen vor; er sagte: ``Wollt ihr nicht essen?"\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcciii{Er empfand in sich Furcht vor ihnen. Sie sagten: ``Fürchte dich nicht." Und sie verkündeten ihm einen kenntnisreichen Jungen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcciv{Da kam seine Frau in großem Geschrei herbei. Sie schlug sich ins Gesicht und sagte: ``(Ich bin doch) eine alte, unfruchtbare Frau!"\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdccv{Sie sagten: ``So hat (es) dein Herr gesagt. Gewiß, Er ist der Allweise und Allwissende."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdccvi{Er sagte: ``Was ist nun euer Auftrag, ihr Boten?"\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdccvii{Sie sagten: ``Wir sind zu einem Volk von Übeltätern gesandt,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdccviii{um über sie Steine aus Lehm niederkommen zu lassen,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdccix{gekennzeichnet bei deinem Herrn für die Maßlosen."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdccx{Und so ließen Wir wer in ihr gläubig war, fortziehen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdccxi{Da fanden wir in ihr aber nur ein (einziges) Haus von (Allah) Ergebenen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdccxii{Und Wir hinterließen in ihr ein Zeichen für diejenigen, die die schmerzhafte Strafe fürchten.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdccxiii{Und in Musa, als Wir ihn zu Fir'aun mit einer deutlichen Ermächtigung sandten.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdccxiv{Da aber kehrte er sich ab samt seiner Stütze und sagte: ``Ein Zauberer oder ein Besessener."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdccxv{Da ergriffen Wir ihn und seine Heerscharen und warfen sie dann in das große Gewässer, denn er hatte sich Tadel zugezogen.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdccxvi{Und (auch) in den 'Ad, als Wir gegen sie den unheilvollen Wind sandten.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdccxvii{Nichts von (all) dem, worüber er kam, ließ er zurück, ohne es wie Zerfallenes werden zu lassen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdccxviii{Und (auch) in den Tamud, als zu ihnen gesagt wurde: ``Genießt für eine gewisse Zeit."\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdccxix{Sie lehnten sich gegen den Befehl ihres Herrn auf. Da ergriff sie der Donnerschlag, während sie hinschauten,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdccxx{so konnten sie nicht (mehr) aufstehen und es wurde ihnen keine Hilfe zuteil.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdccxxi{Und (Wir vernichteten) zuvor (auch) das Volk Nuhs; gewiß, sie waren ein Volk von Frevlern.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdccxxii{Und den Himmel haben Wir mit Kraft aufgebaut, und Wir weiten (ihn) wahrlich (noch) aus.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdccxxiii{Und die Erde haben Wir ausgebreitet. Wie trefflich haben Wir (sie) geebnet!\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdccxxiv{Und von allem haben Wir ein Paar erschaffen, auf daß ihr bedenken möget.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdccxxv{So flüchtet (nun) zu Allah. Gewiß, ich bin euch von Ihm ein deutlicher Warner.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmdccxxvi{Und setzt neben Allah keinen anderen Gott. Gewiß, ich bin euch von Ihm ein deutlicher Warner.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmdccxxvii{So kam auch zu denjenigen, die vor ihnen waren, kein Gesandter, ohne daß sie gesagt hätten: ``Ein Zauberer oder ein Besessener."\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmdccxxviii{Haben sie es denn einander eindringlich empfohlen? Nein! Vielmehr sind sie ein das Maß (an Frevel) überschreitendes Volk.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmdccxxix{So kehre dich von ihnen ab; du bist (hierin) nicht zu tadeln.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmdccxxx{Und ermahne, denn die Ermahnung nützt den Gläubigen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxi{Und Ich habe die Ginn und die Menschen nur (dazu) erschaffen, damit sie Mir dienen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxii{Ich will weder von ihnen irgendeine Versorgung, noch will Ich, daß sie Mir zu essen geben.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxiii{Gewiß, Allah ist der Versorger, der Besitzer von Kraft und der Feste .\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxiv{Gewiß, für diejenigen, die Unrecht tun, wird es einen vorgesehenen Anteil geben wie den Anteil ihrer Gefährten; so sollen sie Mich nicht um Beschleunigung (der Strafe) bitten.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxv{So wehe denjenigen, die ungläubig sind, vor ihrem Tag, der ihnen angedroht ist!\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxvi{\basmalahde Bei dem Berg;\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxvii{und bei einem Buch, das in Zeilen niedergeschrieben ist\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxviii{auf ausgebreiteter Tierhaut;\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxix{bei dem vielbesuchten (Gottes)haus\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdccxl{und bei dem erhöhten Dach\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdccxli{und bei dem angefüllten Meer!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdccxlii{Die Strafe deines Herrn wird gewiß hereinbrechen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdccxliii{Es gibt niemanden, der sie abwehren könnte.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdccxliv{Am Tag, da der Himmel sich heftig hin und her bewegen\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdccxlv{und die Berge sich in der Tat bewegen werden,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdccxlvi{wehe an jenem Tag den Leugnern (der Botschaft),\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdccxlvii{die in schweifenden Gesprächen ihr Spiel treiben!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdccxlviii{Am Tag, da sie gewaltsam zum Feuer der Hölle gestoßen werden:\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdccxlix{``Das ist das Feuer, das ihr für Lüge zu erklären pflegtet.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdccl{Ist dies (nun) etwa Zauberei, oder könnt ihr nicht sehen?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdccli{Seid ihm ausgesetzt und ertragt es dann standhaft oder nicht, gleich ist es in Bezug auf euch; euch wird nur das vergolten, was ihr zu tun pflegtet."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcclii{Die Gottesfürchtigen aber werden in Gärten und Wonne sein.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdccliii{Sie lassen es sich wohl sein an dem, was ihnen ihr Herr gibt. Und ihr Herr bewahrt sie vor der Strafe des Höllenbrandes.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdccliv{``Eßt und trinkt als wohlbekömmlich für das, was ihr zu tun pflegtet,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdcclv{indem ihr euch auf (voreinander)gereihten Liegen lehnt." Und Wir geben ihnen als Gattinnen Huris mit schönen, großen Augen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcclvi{Und denjenigen, die glauben und denen ihre Nachkommenschaft im Glauben nachfolgt, lassen Wir ihre Nachkommenschaft sich (ihnen) anschließen. Und Wir verringern ihnen gar nichts von ihren Werken. Jedermann ist an das, was er erworben hat, gebunden.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcclvii{Und Wir versorgen sie reichlich mit Früchten und Fleisch von dem, was sie begehren.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcclviii{Sie greifen darin miteinander nach einem Becher, in dem nichts darin ist, was zu unbedachter Rede verleitet und zu keiner Versündigung.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcclix{Und unter ihnen gehen Jünglinge umher, die zu ihnen gehören, als wären sie wohlverwahrte Perlen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcclx{Und sie wenden sich einander zu und fragen sich gegenseitig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcclxi{Sie sagen: ``Gewiß, früher waren wir inmitten unserer Angehörigen besorgt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdcclxii{Da erwies Allah uns eine Wohltat und bewahrte uns vor der Strafe des Glutwindes.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcclxiii{Gewiß, früher pflegten wir Ihn anzurufen. Gewiß, Er ist ja der Gütige und Barmherzige."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcclxiv{So ermahne (nur); du bist ja durch die Gunst deines Herrn weder ein Wahrsager noch ein Besessener.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdcclxv{Oder sagen sie etwa: ``Ein Dichter, gegen den wir die (Unglücks)fälle der Zeit abwarten"?\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdcclxvi{Sag: Wartet ab! Gewiß, ich gehöre mit euch zu denjenigen, die abwarten.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdcclxvii{Oder befiehlt ihnen etwa ihr Verstand dies, oder sind sie (vielmehr) Leute, die das Maß (an Frevel) überschreiten?\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdcclxviii{Oder sagen sie etwa: ``Er hat ihn sich selbst ausgedacht'"? Nein! Vielmehr glauben sie nicht.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdcclxix{So sollen sie doch eine Aussage gleicher Art beibringen, wenn sie wahrhaftig sind.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdcclxx{Oder sind sie etwa aus dem Nichts erschaffen worden, oder sind sie (gar) selbst die Schöpfer?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxi{Oder haben sie (etwa) die Himmel und die Erde erschaffen? Nein! Vielmehr sind sie nicht überzeugt.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxii{Oder besitzen sie (etwa) die Schatzkammern deines Herrn, oder sind sie es, die die Oberherrschaft ausüben?\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxiii{Oder haben sie eine Leiter, mit der sie (den Himmel) abhören (können)? Dann soll doch ihr Abhörer eine deutliche Ermächtigung beibringen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxiv{Oder hat Er (für Sich selbst) die Töchter und habt ihr (für euch selbst) die Söhne (bestimmt)?\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxv{Oder verlangst du (etwa) von ihnen einen Lohn, so daß sie mit Schulden belastet wären?\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxvi{Oder besitzen sie (das Wissen über) das Verborgene, so daß sie (es) aufschreiben (können)?\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxvii{Oder wollen sie eine List anwenden? Aber diejenigen, die ungläubig sind, sind es, die der List erliegen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxviii{Oder haben sie einen anderen Gott als Allah? Preis sei Allah! (und Erhaben ist Er) über das, was sie (Ihm) beigesellen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxix{Und wenn sie ein Stück vom Himmel herabfallen sähen, würden sie sagen: ``Aufgeschichtete Wolken."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxx{Lasse sie nur, bis sie ihrem Tag begegnen, an dem sie wie vom Donnerschlag getroffen zusammenbrechen werden,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxi{dem Tag, da ihre List ihnen nichts nützen und ihnen keine Hilfe zuteil werden wird.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxii{Gewiß, für diejenigen, die Unrecht tun, wird es eine Strafe außer dieser geben, aber die meisten von ihnen wissen nicht.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxiii{Sei standhaft gegenüber dem Urteil deines Herrn. Gewiß, du bist ja vor Unseren Augen. Und lobpreise deinen Herrn, wenn du aufstehst,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxiv{und (in einem Teil) von der Nacht, da preise Ihn, und beim Schwinden der Sterne.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxv{\basmalahde Bei dem Stern, wenn er sinkt!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxvi{Nicht in die Irre geht euer Gefährte, und auch nicht einem Irrtum ist er erlegen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxvii{und er redet nicht aus (eigener) Neigung.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxviii{Es ist nur eine Offenbarung, die eingegeben wird.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxix{Belehrt hat ihn der Besitzer starker Kräfte,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdccxc{der (geistige) Macht besitzt. So stand Er da\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdccxci{und war am obersten Gesichtskreis.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdccxcii{Hierauf kam er näher und stieg herunter,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdccxciii{so daß er (nur) zwei Bogenlängen entfernt war oder noch näher.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdccxciv{Da gab Er Seinem Diener (als Offenbarung) ein, was Er eingab.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdccxcv{Nicht hat sein Herz erlogen, was es sah.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdccxcvi{Wollt ihr denn mit ihm streiten über das, was er sieht?\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdccxcvii{Und er sah ihn ja ein anderes Mal herabkommen,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdccxcviii{beim Sidr-Baum des Endziels,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdccxcix{bei dem der Garten der Zuflucht ist.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdccc{Als den Sidr-Baum überdeckte, was (ihn) überdeckte,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdccci{da wich der Blick nicht ab, noch überschritt er das Maß.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcccii{Wahrlich, er sah von den Zeichen seines Herrn die größten.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdccciii{Was meint ihr wohl zu al-Lat und al-'Uzza\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdccciv{und auch zu Manat, dieser anderen, der dritten?\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcccv{Ist denn für euch das, was männlich ist, und für Ihn das, was weiblich ist (bestimmt)?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcccvi{Das wäre dann eine ungerechte Verteilung.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcccvii{Das sind nur Namen, die ihr genannt habt, ihr und eure Väter, für die Allah (aber) keine Ermächtigung herabgesandt hat. Sie folgen nur Vermutungen und dem, wozu sie selbst neigen, wo doch bereits von ihrem Herrn die Rechtleitung zu ihnen gekommen ist.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcccviii{Oder soll (etwa) der Mensch haben, was (immer) er (sich) wünscht?\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcccix{So gehört Allah das Jenseits und das Diesseits.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcccx{Und wie viele Engel gibt es in den Himmeln, deren Fürsprache nichts nützt, außer nachdem Allah es erlaubt hat, wem Er will und wer Sein Wohlgefallen findet.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdcccxi{Diejenigen, die an das Jenseits nicht glauben, benennen die Engel fürwahr mit Namen, wie weibliche Wesen benannt werden.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcccxii{Sie haben jedoch kein Wissen hierüber. Sie folgen nur Vermutungen; die Vermutungen nützen aber nichts gegenüber der Wahrheit.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcccxiii{Laß nun ab von jemandem, der sich von Unserer Ermahnung abkehrt und nur das diesseitige Leben will.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdcccxiv{Das ist ihr erreichter Wissensstand. Gewiß, dein Herr kennt sehr wohl wer von Seinem Weg abirrt, und Er kennt sehr wohl die Rechtgeleiteten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdcccxv{Allah gehört (alles), was in den Himmeln und was auf der Erde ist, auf daß Er denjenigen, die Böses tun, das vergelte, was sie tun, und daß Er denjenigen, die Gutes tun, mit dem (Allerbesten vergelte.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdcccxvi{Diejenigen, die schwerwiegende Sünden und Abscheulichkeiten meiden, außer leichten Verfehlungen... Gewiß, dein Herr ist Allumfassend in (Seiner) Vergebung. Er weiß sehr wohl über euch Bescheid, als Er euch aus der Erde hervorgebracht hat und als ihr Keimlinge in den Leibern eurer Mütter gewesen seid. So erklärt nicht euch selbst für lauter. Er weiß sehr wohl, wer gottesfürchtig ist.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdcccxvii{Was meinst du wohl zu demjenigen, der sich abkehrt\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdcccxviii{und (nur) wenig gibt und dann (ganz) aussetzt?\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdcccxix{Besitzt er (etwa) Wissen über das Verborgene, daß er sehen könnte?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdcccxx{Oder wurde ihm nicht kundgetan, was auf den Blättern Musas steht\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxi{und Ibrahims, der (seine Pflichten) erfüllte?\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxii{Nämlich,) daß keine lasttragende (Seele) die Last einer anderen auf sich nehmen wird\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxiii{und daß es für den Menschen nichts anderes geben wird als das, worum er sich (selbst) bemüht,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxiv{daß sein Bemühen gesehen werden wird,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxv{daß ihm hierauf nach vollem Maß vergolten wird\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxvi{und daß zu deinem Herrn das Endziel sein wird\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxvii{und daß Er es ist, Der lachen und weinen läßt,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxviii{und daß Er es ist, Der sterben läßt und wieder lebendig macht,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxix{daß Er beide Teile des Paares, das Männliche und das Weibliche, erschafft\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxx{aus einem Samentropfen, wenn er ausgespritzt wird,\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxi{und daß Ihm die letzte Schöpfung obliegt\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxii{und daß Er es ist, Der reich macht und Der genügsam macht,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxiii{und daß Er es ist, Der der Herr des Hundssterns ist,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxiv{daß Er die früheren 'Ad vernichtete\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxv{und (auch) die Tamud; so ließ Er nichts übrig;\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxvi{und zuvor das Volk Nuhs - gewiß, sie waren ja noch ungerechter und übermäßiger (in ihrem Frevel).\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxvii{Und die umgestürzte Stadt ließ Er stürzen.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxviii{Und so überdeckte sie, was (sie dann) überdeckte.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxix{Welche der Wohltaten deines Herrn willst du nun bestreiten?\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmdcccxl{Das ist ein Warner wie die früheren Warner.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmdcccxli{Es steht bevor die immer näher kommende (Stunde des Gerichts);\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlii{niemand außer Allah kann sie beheben.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmdcccxliii{Wundert ihr euch denn über diese Aussage\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmdcccxliv{und lacht ihr, und weint ihr nicht,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlv{und seid ihr noch belustigt?\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlvi{Werft euch doch vor Allah nieder und dient (Ihm).\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlvii{\basmalahde Näher ist die Stunde (des Gerichts) gekommen, und gespalten hat sich der Mond.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlviii{Und wenn sie ein Zeichen sehen, wenden sie sich ab und sagen: ``Fortdauernde Zauberei."\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlix{Und sie erklären (es) für Lüge und folgen ihren Neigungen. Doch jede Angelegenheit ist festgesetzt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcccl{Zu ihnen ist doch von den Berichten gekommen, was reichliche Verweisung enthält,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcccli{eine durchdringende Weisheit. Aber was nützen da die Warnungen?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdccclii{So kehre dich von ihnen ab. Am Tag, da der Rufer zu etwas Entsetzlichem ruft,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdcccliii{werden sie mit gedemütigten Blicken aus den Gräbern herauskommen wie ausschwärmende Heuschrecken,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdcccliv{zu dem Rufer hastend. Die Ungläubigen werden sagen: ``Das ist ein schwerer Tag."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdccclv{Vor ihnen (schon) bezichtigte das Volk Nuhs (ihren Gesandten) der Lüge. Sie bezichtigten Unseren Diener der Lüge und sagten: ``Ein Besessener." Und er wurde (von ihnen) gescholten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdccclvi{Da rief er seinen Herrn an: ``Ich bin überwältigt, so leiste Du (mir) Hilfe."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdccclvii{Da öffneten Wir die Tore des Himmels mit niederströmendem Wasser\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdccclviii{und ließen aus der Erde Quellen hervorströmen; so traf das Wasser zu einer bereits festgesetzten Angelegenheit zusammen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdccclix{Und Wir trugen ihn auf einem Schiff aus Planken und Nägeln,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdccclx{das vor Unseren Augen dahinfuhr: (Dies) als Lohn für jemanden, der immer verleugnet worden war.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdccclxi{Und Wir ließen es ja als Zeichen zurück. Gibt es aber jemanden, der bedenkt?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdccclxii{Wie waren da Meine Strafe und Meine Warnungen!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdccclxiii{Und Wir haben den Qur'an ja leicht zum Bedenken gemacht. Aber gibt es jemanden, der bedenkt?\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdccclxiv{Die 'Ad bezichtigten (ihren Gesandten) der Lüge. Wie waren da Meine Strafe und Meine Warnungen!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdccclxv{Wir sandten gegen sie ja einen eiskalten Wind an einem Tag fortdauernden Unheils,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdccclxvi{der die Menschen fortnahm, als wären sie Stämme entwurzelter Palmen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdccclxvii{Wie waren da Meine Strafe und Meine Warnungen!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdccclxviii{Und Wir haben den Qur'an ja leicht zum Bedenken gemacht. Aber gibt es jemanden, der bedenkt?\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdccclxix{Die Tamud erklärten die Warnungen für Lüge.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdccclxx{Sie sagten: ``Sollen wir denn einem menschlichen Wesen von uns, einem einzelnen, folgen? Dann befänden wir uns wahrlich im Irrtum und Wahnsinn.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxi{Ist die Ermahnung gerade ihm aus unserer Mitte offenbart worden? Nein! Vielmehr ist er ein selbstgefälliger Lügner."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxii{Sie werden" morgen erfahren, wer der selbstgefällige Lügner ist.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxiii{Wir werden die Kamelstute senden als Versuchung für sie. So warte mit ihnen ab und sei beharrlich.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxiv{Und tu ihnen kund, daß das Wasser zwischen ihnen (und der Kamelstute) zu teilen ist. Jeder Trinkanteil soll (dann abwechselnd) wahrgenommen werden.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxv{Da riefen sie ihren Gefährten her. Er griff zu und schnitt dann (der Kamelstute) die Sehnen durch.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxvi{Wie waren da Meine Strafe und Meine Warnungen!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxvii{Wir sandten ja gegen sie einen einzigen Schrei, da waren sie (sogleich) wie das vertrocknete Zeug von Viehgehegen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxviii{Und Wir haben den Qur'an ja leicht zum Bedenken gemacht. Aber gibt es jemanden, der bedenkt?\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxix{Das Volk Luts erklärte die Warnungen für Lüge.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxx{Wir sandten ja gegen sie einen Sturm von Steinchen, außer den Angehörigen Luts; Wir erretteten sie im letzten Teil der Nacht\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxi{aus Gunst von Uns. So vergelten Wir, wer dankbar ist.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxii{Er hatte sie ja vor Unserem gewaltsamen Zupacken gewarnt, sie aber bestritten die Warnungen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxiii{Und sie versuchten ja, ihn in Bezug auf seine Gäste zu überreden (, sie ihnen auszuliefern). Da löschten Wir ihre Augen aus. ``Kostet nun Meine Strafe und Meine Warnungen."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxiv{Und es ereilte sie ja am frühen Morgen eine beständige Strafe.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxv{``Kostet nun Meine Strafe und Meine Warnungen."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxvi{Und Wir haben den Qur'an ja leicht zum Bedenken gemacht. Aber gibt es jemanden, der bedenkt?\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxvii{Und (auch) zu Fir'auns Leuten kamen ja die Warnungen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxviii{Sie erklärten alle Unsere Zeichen für Lüge, und da ergriffen Wir sie mit dem Griff eines Unüberwindlichen und Allmächtigen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxix{Sind etwa eure Ungläubigen besser als jene da? Oder gibt es für euch einen Freispruch in den Schriften?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdcccxc{Oder sagen sie: ``Wir halten zusammen und werden siegen"?\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdcccxci{Die Ansammlung wird gewiß besiegt werden, und sie werden den Rücken kehren.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcii{Nein! Vielmehr ist die Stunde (des Gerichts) der ihnen versprochene Zeitpunkt. Und die Stunde ist noch schrecklicher und bitterer.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdcccxciii{Gewiß, die Übeltäter befinden sich im Irrtum und Wahnsinn.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdcccxciv{Am Tag, da sie auf ihren Gesichtern ins (Höllen)feuer gezerrt werden: ``Kostet die Berührung der Sengenden."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcv{Gewiß, Wir haben alles in (bestimmtem) Maß erschaffen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcvi{Und Unser Befehl ist nur ein einziges (Wort), wie ein Augenblick.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcvii{Und Wir haben doch bereits manche Lager gleich euch vernichtet. Aber gibt es jemanden, der bedenkt?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcviii{Alles, was sie getan haben, (steht) in den Schriften.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcix{Alles, ob klein oder groß, wird in Zeilen niedergeschrieben.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcm{Gewiß, die Gottesfürchtigen werden in Gärten und an Bächen sein,\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcmi{am Sitz der Wahrhaftigkeit, bei einem allmächtigen Herrscher.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcmii{\basmalahde Der Allerbarmer\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcmiii{hat den Qur'an gelehrt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcmiv{Er hat den Menschen erschaffen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcmv{Er hat ihn die klare Darlegung gelehrt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcmvi{Die Sonne und der Mond (laufen) nach Berechnung.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcmvii{Die stiellosen Pflanzen und die Bäume werfen sich nieder.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcmviii{Den Himmel hat Er emporgehoben und die Waage aufgestellt,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcmix{damit ihr beim Wägen nicht das Maß überschreitet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcmx{Und meßt das Gewicht in Gerechtigkeit und gebt beim Wägen nicht weniger.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcmxi{Und die Erde hat Er für die Geschöpfe (an)gelegt;\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcmxii{auf ihr gibt es Früchte, Palmen mit Fruchthüllen\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcmxiii{und Korn mit Halmen und duftende Pflanzen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcmxiv{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcmxv{Er hat den Menschen aus trockenem Ton wie Töpferware erschaffen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcmxvi{Und Er hat die Ginn aus einer unruhigen Feuerflamme erschaffen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcmxvii{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcmxviii{Er ist) der Herr der beiden Osten und der Herr der beiden Westen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcmxix{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcmxx{Er hat den beiden Meeren freien Lauf gelassen; sie treffen aufeinander,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcmxxi{zwischen ihnen ist (aber) ein trennendes Hindernis, (das)s sie (ihre Grenzen) nicht überschreiten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcmxxii{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcmxxiii{Aus ihnen beiden kommen Perlen und Korallen hervor.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcmxxiv{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcmxxv{Und Sein sind (auch) die (hoch)gebauten auf dem Meer fahrenden (Schiffe), wie Berge.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcmxxvi{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcmxxvii{Alle, die auf ihr sind, werden vergehen;\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmcmxxviii{bleiben wird (nur) das Angesicht deines Herrn, Besitzer der Erhabenheit und Ehre.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmcmxxix{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmcmxxx{Ihn bittet wer in den Himmeln und auf der Erde ist. Jeden Tag befaßt Er sich mit einer Angelegenheit.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxi{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxii{Wir werden Uns euch widmen, ihr beiden Schwerlastenden.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxiii{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxiv{O ihr Gesellschaft der Ginn und der Menschen, wenn ihr aus den Bezirken der Himmel und der Erde hinausdringen könnt, so dringt doch heraus. Ihr werdet nicht herausdringen, außer mit einer Ermächtigung.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxv{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxvi{Gegen euch beide wird eine rauchlose Feuerflamme und Kupfer gesandt werden, so werdet ihr euch selbst nicht helfen (können).\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxvii{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxviii{Wenn der Himmel sich spalten und dann rosig werden wird wie Farböl\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxix{- welche der Wohltaten eures Herrn wollt ihr beide denn leugnen? -,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcmxl{an jenem Tag nun werden nach ihrer Sünde weder Mensch noch Gänn gefragt.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcmxli{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcmxlii{Die Übeltäter werden an ihrem Merkmal erkannt werden und dann an den Stirnlocken und an den Füßen ergriffen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcmxliii{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcmxliv{Das ist die Hölle, die die Übeltäter für Lüge erklären.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcmxlv{Sie werden zwischen ihr und siedend heißem Wasser umhergehen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcmxlvi{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcmxlvii{Für jemanden aber, der den Stand seines Herrn fürchtet, wird es zwei Gärten geben\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcmxlviii{- welche der Wohltaten eures Herrn wollt ihr beide denn leugnen? -,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcmxlix{die beide Zweige haben.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcml{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcmli{Darin sind zwei Quellen, die fließen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcmlii{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcmliii{Darin gibt es von jeder Frucht zwei Arten'.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcmliv{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcmlv{Sie werden sich auf Ruhebetten lehnen, deren Futter aus schwerem Brokat sind. Und die Ernte der beiden Gärten hängt herab.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcmlvi{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcmlvii{Darin sind (weibliche Wesen), die ihre Blicke zurückhalten, die vor ihnen weder Mensch noch Ginn berührt haben\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcmlviii{- welche der Wohltaten eures Herrn wollt ihr beide denn leugnen? -,\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcmlix{als wären sie Rubine und Korallen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcmlx{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcmlxi{Ist der Lohn des Guten nicht ebenfalls das Gute?\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmcmlxii{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmcmlxiii{Unterhalb der beiden gibt es (noch) zwei andere Gärten\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmcmlxiv{- welche der Wohltaten eures Herrn wollt ihr beide denn leugnen? -,\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmcmlxv{dich bewachsene.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcmlxvi{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcmlxvii{Darin sind zwei sprudelnde Quellen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcmlxviii{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcmlxix{Darin sind Früchte und Palmen und Granatäpfel.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmcmlxx{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxi{Darin sind gute schöne (weibliche Wesen)\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxii{- welche der Wohltaten eures Herrn wollt ihr beide denn leugnen? -,\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxiii{Huris, (die) in den Zelten zurückgezogen (leben) -,\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxiv{welche der Wohltaten eures Herrn wollt ihr beide denn leugnen? -,\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxv{die vor ihnen weder Mensch noch Ginn berührt haben.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxvi{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxvii{Sie lehnen sich aufgrünen Decken und schönen Teppichen.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxviii{Welche der Wohltaten eures Herrn wollt ihr beide denn leugnen?\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxix{Segensreich ist der Name deines Herrn, Besitzer der Erhabenheit und Ehre!\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxx{\basmalahde Wenn die (Stunde) eintrifft, die eintreffen wird,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxi{gibt es niemanden, der ihr Eintreffen leugnen wird.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxii{Sie wird (manches) niedrig machen und (manches) erhöhen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxiii{Wenn die Erde heftig hin und her geschüttelt wird\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxiv{und die Berge völlig zermalmt werden\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxv{und dann zu verstreutem Staub werden\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxvi{und ihr (in) drei Arten (aufgeteilt) werdet:\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxvii{Die Gefährten von der rechten Seite - was sind die Gefährten von der rechten Seite?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxviii{Und die Gefährten von der unheilvollen Seite - was sind die Gefährten von der unheilvollen Seite?\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxix{Und die Vorausgeeilten, ja die Vorausgeeilten,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcmxc{das sind diejenigen, die (Allah) nahegestellt sein werden,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcmxci{in den Gärten der Wonne.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcmxcii{Eine Menge von den Früheren\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcmxciii{und wenige von den Späteren,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcmxciv{auf (mit Gold) durchwobenen Liegen\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcmxcv{lehnen sie sich darauf einander gegenüber.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcmxcvi{Unter ihnen gehen ewig junge Knaben umher\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcmxcvii{mit Trinkschalen und Krügen und einem Becher aus einem Quell -,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcmxcviii{von ihm bekommen sie weder Kopfschmerzen noch werden sie dadurch benommen,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcmxcix{und (mit) Früchten von dem, was sie sich auswählen,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmm{und Fleisch von Geflügel von dem, was sie begehren.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmi{Und (darin sind) Huris mit schönen, großen Augen,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmii{gleich wohlverwahrten Perlen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmiii{Dies) als Lohn für das, was sie zu tun pflegten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmiv{Weder hören sie darin unbedachte Rede noch Anklage der Sünde,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmv{sondern nur den Ausspruch: ``Frieden! Frieden!"\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmvi{Und die Gefährten der rechten Seite - was sind die Gefährten von der rechten Seite?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmvii{Sie sind) unter dornenlosen Sidr-Bäumen\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmviii{und dichtgeschichteten Mimosen\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmix{und langgestrecktem Schatten,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmx{an) sich ergießendem Wasser,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmxi{bei) vielen Früchten,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmxii{die weder unterbrochen noch verwehrt sind,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmxiii{und (auf) erhöhten Ruhebetten.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmxiv{Wir haben sie derart entstehen lassen\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmxv{und sie zu Jungfrauen gemacht,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmxvi{liebevoll und gleichaltrig,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmxvii{für die Gefährten der rechten Seite.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmxviii{Eine Menge von den Früheren,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmxix{und eine Menge von den Späteren.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmxx{Und die Gefährten der linken Seite - was sind die Gefährten der linken Seite?\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmxxi{Sie sind) in Glutwind und heißem Wasser\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmxxii{und (in) Schatten aus schwarzem Qualm,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmxxiii{der) weder kühl noch trefflich (ist).\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmxxiv{Sie lebten ja vordem üppig\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmxxv{und verharrten in dem gewaltigen Unglauben\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmxxvi{und pflegten zu sagen: ``Wenn wir gestorben und zu Erde und Knochen geworden sind, sollen wir denn dann wirklich auferweckt werden?\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmxxvii{Und auch unsere Vorväter?"\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmxxviii{Sag: Die Früheren und die Späteren\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmxxix{werden wahrlich versammelt werden zur festgesetzten Zeit eines (wohl)bekannten Tages.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmxxx{Hierauf werdet ihr ja, ihr irregehenden Leugner,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmxxxi{wahrlich von Zaqqum-Bäumen essen\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmxxxii{und euch dann davon die Bäuche füllen\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmmxxxiii{und dann darauf heißes Wasser trinken;\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmmxxxiv{trinken werdet ihr dann, wie ewig Durstige trinken.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmmxxxv{Das ist ihre gastliche Aufnahme am Tag des Gerichts.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmmxxxvi{Wir doch haben euch erschaffen; wenn ihr (es) doch für wahr halten würdet!\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmmxxxvii{Was meint ihr denn zu dem, was ihr als Samen ausspritzt?\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmmxxxviii{Seid ihr es etwa, die ihn erschaffen oder sind nicht doch Wir die Erschaffer?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmmxxxix{Wir doch haben unter euch den Tod festgelegt. Und niemand kann Uns zuvorkommen,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmmxl{daß Wir (nicht) euresgleichen (gegen euch) austauschen und euch (in einem Zustand wieder)entstehen lassen, den ihr nicht kennt.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmmxli{Und ihr kennt doch die erste Schöpfung. Wenn ihr doch bedenken würdet!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmmxlii{Was meint ihr denn zu dem, was ihr an Saatfeldern bestellt?\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmmxliii{Seid ihr es etwa, die es wachsen lassen, oder sind nicht doch Wir es, die wachsen lassen?\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmmxliv{Wenn Wir wollten, könnten Wir es wahrlich zu zermalmtem Zeug machen, und ihr würdet dann dauernd verwundert darüber reden:\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmmxlv{``Wir sind wahrlich mit Schulden belastet.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmmxlvi{Nein! Vielmehr ist uns alles verwehrt worden."\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmmxlvii{Was meint ihr denn zu dem Wasser, das ihr trinkt?\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmmxlviii{Seid ihr es etwa, die es von den Wolken herabkommen lassen, oder sind doch nicht Wir es, die herabkommen lassen?\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmmxlix{Wenn Wir wollten, könnten Wir es (auf der Zunge) brennend machen. Wenn ihr doch dankbar sein würdet!\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmml{Was meint ihr denn zu dem Feuer, das ihr (durch Reiben) zündet?\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmmli{Seid ihr es etwa, die den dazu nötigen Baum entstehen lassen, oder sind nicht doch Wir es, die entstehen lassen?\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmmlii{Wir haben es ja zur Erinnerung (an das Höllenfeuer) und als Nießbrauch für die Wanderer gemacht.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmmliii{Darum preise den Namen deines Allgewaltigen Herrn.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmmliv{Nein! Ich schwöre bei den Standorten der Sterne\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmmlv{- und das ist wahrlich, wenn ihr (nur) wüßtet, ein gewaltiger Schwur:\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmmlvi{Das ist wahrlich ein ehrwürdiger Qur'an\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmmlvii{in einem wohlverwahrten Buch,\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmmlviii{das nur diejenigen berühren (dürfen), die vollkommen gereinigt sind;\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmmmlix{er ist) eine Offenbarung vom Herrn der Weltenbewohner.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmmlx{Wollt ihr denn über diese Aussage nur schön reden\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmmlxi{und (als Dankbarkeit) für eure Versorgung soll euer Ableugnen sein?\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmmlxii{Wenn sie die Kehle erreicht,\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmmlxiii{während ihr dabei zuschaut\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmmlxiv{und Wir ihm näher sind als ihr - aber ihr könnt es nicht sehen -,\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmmlxv{könntet ihr dann, wenn ihr doch nicht vor Gericht gestellt werden sollt (, wie ihr behauptet),\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmmmlxvi{sie zurückbringen, wenn ihr wahrhaftig seid?\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmmmlxvii{Wenn er nun zu den (Allah) Nahegestellten gehört,\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmmmlxviii{dann (wird es für ihn) Ruhe, duftende Pflanzen und einen Garten der Wonne (geben).\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmmmlxix{Und wenn er nun zu den Gefährten der rechten Seite gehört,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmmmlxx{so: Friede sei dir von den Gefährten der rechten Seite.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmmmlxxi{Wenn er aber zu den Leugnern, den Irregehenden gehört,\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmmmlxxii{dann wird ihm eine gastliche Aufnahme aus heißem Wasser (zuteil) werden,\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmmmlxxiii{und das Erleiden des Höllenbrandes.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmmmmlxxiv{Dies ist wahrlich die reine Gewißheit.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmmmmlxxv{Darum preise den Namen deines Allgewaltigen Herrn.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmmmmlxxvi{\basmalahde Allah preist (alles), was in den Himmeln und auf der Erde ist. Und Er ist der Allmächtige und Allweise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmlxxvii{Ihm gehört die Herrschaft der Himmel und der Erde. Er macht lebendig und läßt sterben. Und Er hat zu allem die Macht.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmlxxviii{Er ist der Erste und der Letzte, der Offenbare und der Verborgene. Und Er weiß über alles Bescheid.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmlxxix{Er ist es, der die Himmel und die Erde in sechs Tagen erschuf und Sich hierauf über den Thron erhob'. Er weiß, was in die Erde eindringt und was aus ihr herauskommt, was vom Himmel herabkommt und was dorthin aufsteigt. Und Er ist mit euch, wo immer ihr auch seid. Und was ihr tut, sieht Allah wohl.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmlxxx{Ihm gehört die Herrschaft der Himmel und der Erde. Und zu Allah werden die Angelegenheiten zurückgebracht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxi{Er läßt die Nacht in den Tag eindringen und läßt den Tag in die Nacht eindringen. Und Er weiß über das Innerste der Brüste Bescheid.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxii{Glaubt an Allah und Seinen Gesandten und gebt aus von dem, worüber Er euch zu Statthaltern eingesetzt hat. Diejenigen von euch, die glauben und ausgeben, für sie wird es großen Lohn geben.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxiii{Was ist mit euch, daß ihr nicht an Allah glauben wollt, wo doch der Gesandte euch dazu aufruft, an euren Herrn zu glauben, und wo Er bereits ein Abkommen mit euch getroffen hat, wenn ihr gläubig seid?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxiv{Er ist es, Der Seinem Diener klare Zeichen offenbart, damit Er euch aus den Finsternissen ins Licht hinausbringt. Und Allah ist wahrlich mit euch Gnädig und Barmherzig.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxv{Und was ist mit euch, daß ihr nicht auf Allahs Weg ausgeben sollt, wo doch das Erbe der Himmel und der Erde Allah gehört. Nicht gleich sind diejenigen von euch, die vor dem Sieg ausgegeben und gekämpft haben, ... Diese haben eine höhere Rangstufe als diejenigen, die erst nachher ausgegeben und gekämpft haben. Allen aber hat Allah das Beste versprochen. Und Allah ist dessen, was ihr tut, Kundig.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxvi{Wer ist es denn, der Allah ein gutes Darlehen gibt? So wird Er es ihm vervielfachen; und für ihn wird es trefflichen Lohn geben.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxvii{Am Tag, da du siehst, wie den gläubigen Männern und den gläubigen Frauen ihr Licht vorauseilt und (ebenso) zu ihrer Rechten: ``Die frohe Botschaft für euch heute: Gärten, durcheilt von Bächen, ewig darin zu bleiben. Das ist der großartige Erfolg."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxviii{Am Tag, da die Heuchler und die Heuchlerinnen zu denjenigen, die glauben, sagen: ``Wartet auf uns, daß wir (unser Licht) von eurem Licht nehmen." Es wird (zu ihnen) gesagt werden: ``Geht doch nach hinten zurück und sucht dort nach Licht." Da wird zwischen ihnen eine (Schutz)mauer gesetzt mit einem Tor, zu dessen Innenseite die Barmherzigkeit und zu dessen Außenseite, davor, die Strafe ist.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxix{Sie rufen ihnen zu: ``Waren wir nicht mit euch?" Sie werden sagen: ``Ja doch, aber ihr habt euch selbst der Versuchung ausgesetzt, ihr habt abgewartet und gezweifelt, und die Wünsche haben euch getäuscht, bis Allahs Befehl kam. Und getäuscht hat euch hinsichtlich Allahs der Täuscher.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmxc{So wird heute von euch kein Lösegeld angenommen und auch nicht von denjenigen, die ungläubig sind. Euer Zufluchtsort ist das (Höllen)feuer; es ist euer Schutzherr - ein schlimmer Ausgang!"\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmxci{Ist es denn nicht Zeit für diejenigen, die glauben, daß ihre Herzen demütig werden vor Allahs Ermahnung und vor dem, was von der Wahrheit herabgekommen ist, und daß sie nicht wie diejenigen sind, denen zuvor die Schrift gegeben wurde, es ihnen aber zu lang gedauert hat, und so ihre Herzen sich verhärtet haben? Und viele von ihnen sind Frevler.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmxcii{Wißt, daß Allah die Erde nach ihrem Tod wieder lebendig macht. Wir haben euch die Zeichen bereits klargemacht, auf daß ihr begreifen möget.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmxciii{Gewiß, denjenigen Männern, die Almosen geben und denjenigen Frauen, die Almosen geben und (damit) ein gutes Darlehen geben, wird es vervielfacht werden; und für sie wird es trefflichen Lohn geben.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmxciv{Und diejenigen, die an Allah und Seine Gesandten glauben, das sind die stets Wahrhaftigen und die Zeugen vor ihrem Herrn. Sie erhalten ihren Lohn und ihr Licht. Diejenigen aber, die ungläubig sind und Unsere Zeichen für Lüge erklären, das sind die Insassen des Höllenbrandes.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmxcv{Wißt, daß das diesseitige Leben nur Spiel und Zerstreuung ist, Schmuck und gegenseitige Prahlerei und Wettstreit nach noch mehr Besitz und Kindern. Es ist wie das Gleichnis von Regen, dessen Pflanzenwuchs den Ungläubigen gefällt. Hierauf aber trocknet er aus, und da siehst du ihn gelb werden. Hierauf wird es zu zermalmtem Zeug. Im Jenseits aber gibt es strenge Strafe und (auch) Vergebung von Allah und Wohlgefallen. Und das diesseitige Leben ist nur trügerischer Genuß.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmxcvi{Wetteilt zu Vergebung von eurem Herrn und (zu) einem (Paradies)garten, dessen Breite wie die Breite der Himmel und der Erde ist, bereitet für diejenigen, die an Allah und Seine Gesandten glauben. Das ist Allahs Huld, die Er gewährt, wem Er will. Und Allah besitzt große Huld.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmxcvii{Kein Unglück trifft ein auf der Erde oder bei euch selbst, ohne daß es in einem Buch (verzeichnet) wäre, bevor Wir es erschaffen - gewiß, dies ist Allah ein leichtes -,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmxcviii{damit ihr nicht betrübt seid über das, was euch entgangen ist, und euch nicht (zu sehr) freut über das, was Er euch gegeben hat. Und Allah liebt niemanden, der eingebildet und prahlerisch ist,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmxcix{diejenigen, die geizen und den Menschen befehlen, geizig zu sein. Und wenn einer sich abkehrt, gewiß, so ist Allah der Unbedürftige und Lobenswürdige.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmc{Wir haben ja Unsere Gesandten mit den klaren Beweisen gesandt und mit ihnen die Schrift und die Waage herabkommen lassen, damit die Menschen für die Gerechtigkeit eintreten. Und Wir haben das Eisen herabkommen lassen. In ihm ist starke Gewalt und Nutzen für die Menschen -, damit Allah kennt, wer Ihm und Seinen Gesandten im Verborgenen hilft. Gewiß, Allah ist Stark und Allmächtig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmci{Und Wir sandten bereits Nuh und Ibrahim und richteten in ihrer Nachkommenschaft das Prophetentum und die Schrift ein. Unter ihnen gab es (einige, die) rechtgeleitet (waren), aber viele von ihnen waren Frevler.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcii{Hierauf ließen Wir auf ihren Spuren Unsere Gesandten folgen; und Wir ließen 'Isa, den Sohn Maryams, folgen und gaben ihm das Evangelium. Und Wir setzten in die Herzen derjenigen, die ihm folgten, Mitleid und Barmherzigkeit, und (auch) Mönchtum, das sie erfanden - Wir haben es ihnen nicht vorgeschrieben -, (dies) nur im Trachten nach Allahs Wohlgefallen. Sie beachteten es jedoch nicht, wie es ihm zusteht. Und so gaben Wir denjenigen von ihnen, die glaubten, ihren Lohn. Aber viele von ihnen waren Frevler.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmciii{O die ihr glaubt, fürchtet Allah und glaubt an Seinen Gesandten, dann gibt Er euch einen zweifachen Anteil an Seiner Barmherzigkeit und macht euch ein Licht, in dem ihr gehen könnt, und vergibt euch. Allah ist Allvergebend und Barmherzig.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmciv{Dies ist) damit die Leute der Schrift wissen, daß sie über nichts von Allahs Huld Macht haben, sondern daß die Huld in Allahs Hand liegt. Er gewährt sie, wem Er will. Und Allah besitzt große Huld.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcv{\basmalahde Gehört hat ja Allah die Aussage derjenigen, die mit dir über ihren Gatten streitet und sich bei Allah beklagt, während Allah euren Wortwechsel hört. Gewiß, Allah ist Allhörend und Allsehend.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcvi{Diejenigen von euch, die sich von ihren Frauen durch den Rückenschwur trennen - sie sind doch nicht ihre Mütter. Ihre Mütter sind nur diejenigen, die sie geboren haben. Sie sagen da fürwahr etwas Verwerfliches an Worten und etwas Falsches. Und Allah ist wahrlich All verzeihend und Allvergebend.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcvii{Diejenigen, die sich von ihren Frauen durch den Rückenschwur trennen und hierauf dann doch zu dem zurückkehren, was sie gesagt haben, (sollen) einen Sklaven befreien, bevor sie beide einander berühren. Damit werdet ihr ja ermahnt. Und Allah ist dessen, was ihr tut, Kundig.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcviii{Wer aber keine (Möglichkeit dazu) findet, (der hat) zwei aufeinanderfolgende Monate (zu) fasten, bevor sie beide einander berühren. Wer (es) aber nicht kann, (der hat) sechzig Arme (zu) speisen. Dies, damit ihr an Allah und Seinen Gesandten glaubt. Dies sind Allahs Grenzen\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcix{Und für die Ungläubigen wird es schmerzhafte Strafe geben. Gewiß, diejenigen, die Allah und Seinem Gesandten zuwiderhandeln, werden niedergeworfen, wie diejenigen vor ihnen niedergeworfen worden sind. Wir haben ja doch klare Zeichen hinabgesandt. Und für die Ungläubigen wird es schmachvolle Strafe geben.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcx{Am Tag, da Allah sie alle auferwecken und ihnen kundtun wird, was sie getan haben. Allah hat es erfaßt, sie aber haben es vergessen. Und Allah ist über alles Zeuge.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcxi{Siehst du nicht, daß Allah weiß, was in den Himmeln und was auf der Erde ist? Es gibt kein vertrauliches Gespräch zwischen dreien, ohne daß Er ihr vierter wäre, und keines zwischen fünfen, ohne daß Er ihr sechster wäre, und auch nicht weniger oder mehr als dieser (Zahl), ohne daß Er mit ihnen wäre, wo immer sie sein mögen. Hierauf tut Er ihnen am Tag der Auferstehung kund, was sie getan haben. Gewiß, Allah weiß über alles Bescheid.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcxii{Siehst du nicht jene, denen verboten wurde, vertrauliche Gespräche zu führen, die aber hierauf zurückkehren zu dem, was ihnen verboten wurde und miteinander über Sünde, feindseliges Vorgehen und Ungehorsam gegen den Gesandten vertraulich sprechen? Und wenn sie zu dir kommen, grüßen sie dich, wie nicht einmal Allah dich grüßt, und sagen unter sich: ``Würde Allah uns doch nur strafen für das, was wir sagen!" Ihre Genüge ist die Hölle, der sie ausgesetzt sein werden - ein schlimmer Ausgang!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcxiii{O die ihr glaubt, wenn ihr vertraulich miteinander sprecht, dann sprecht nicht vertraulich miteinander über Sünde, feindseliges Vorgehen und Ungehorsam gegen den Gesandten, sondern sprecht vertraulich miteinander über Frömmigkeit und Gottesfurcht. Und fürchtet Allah, zu Dem ihr versammelt werdet.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcxiv{Vertrauliche Gespräche sind nur vom Satan, damit diejenigen traurig seien, die gläubig sind. Aber er kann ihnen keinen Schaden zufügen, außer mit Allahs Erlaubnis. Und auf Allah sollen sich die Gläubigen verlassen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcxv{O die ihr glaubt, wenn zu euch gesagt wird: ``Macht Platz!" in den Versammlungen!, dann schafft Platz, so schafft auch Allah euch Platz. Und wenn gesagt wird: ``Erhebt euch!", dann erhebt euch eben, so erhöht auch Allah diejenigen von euch, die glauben, und diejenigen, denen das Wissen gegeben worden ist, um Rangstufen. Und Allah ist dessen, was ihr tut, Kundig.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcxvi{O die ihr glaubt, wenn ihr mit dem Gesandten vertraulich sprechen wollt, dann gebt schon vor eurem vertraulichen Gespräch ein Almosen. Das ist besser und reiner für euch. Wenn ihr jedoch nicht(s dafür) findet, gewiß, dann ist Allah Allvergebend und Barmherzig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcxvii{Scheut ihr davor, schon vor eurem vertraulichen Gespräch Almosen zu geben? Da ihr (es) nicht getan habt und Allah sich euch wieder Reue-Annehmend zugewendet hat, so verrichtet das Gebet und entrichtet die Abgabe', und gehorcht Allah und Seinem Gesandten. Und Allah ist Kundig dessen, was ihr tut.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcxviii{Siehst du nicht jene, die sich Leute zu Schutzherren nehmen, denen Allah zürnt? Sie gehören weder zu euch noch zu ihnen, und sie schwören auf (ihre) Lügen, wobei sie (es) wissen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcxix{Allah hat für sie strenge Strafe bereitet. Gewiß, wie böse ist, was sie zu tun pflegen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcxx{Sie haben sich ihre Eide zu einem Schutzschirm genommen und halten so von Allahs Weg ab. Für sie wird es schmachvolle Strafe geben.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcxxi{Weder ihr Besitz noch ihre Kinder werden ihnen vor Allah etwas nützen. Das sind die Insassen des (Höllen)feuers; ewig werden sie darin bleiben.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcxxii{Am Tag, da Allah sie alle auferweckt, da werden sie Ihm schwören, wie sie euch schwören, und meinen, sie hätten eine Grundlage. Aber sicherlich, sie sind ja Lügner.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcxxiii{Der Satan hat sie in seine Gewalt gebracht und sie dann Allahs Ermahnung vergessen lassen. Jene sind die Gruppierung des Satans. Aber sicherlich, die Gruppierung des Satans, das sind ja die Verlierer.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcxxiv{Gewiß, diejenigen, die Allah und Seinem Gesandten zuwiderhandeln, sie gehören zu den Niedrigsten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcxxv{Allah hat (vor)geschrieben: ``Siegen werde Ich ganz gewiß, (Ich) und Meine Gesandten." Gewiß, Allah ist Stark und Allmächtig.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcxxvi{Du findest keine Leute, die an Allah und den Jüngsten Tag glauben und denjenigen Zuneigung bezeigen, die Allah und Seinem Gesandten zuwiderhandeln, auch wenn diese ihre Väter wären oder ihre Söhne oder ihre Brüder oder ihre Sippenmitglieder. Jene - in ihre Herzen hat Er den Glauben geschrieben und sie mit Geist von Sich gestärkt. Er wird sie in Gärten eingehen lassen, durcheilt von Bächen, ewig darin zu bleiben. Allah hat Wohlgefallen an ihnen, und sie haben Wohlgefallen an Ihm. Jene sind Allahs Gruppierung. Sicherlich, Allahs Gruppierung, dies sind diejenigen, denen es wohl ergeht.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcxxvii{\basmalahde Allah preist (alles), was in den Himmeln und was auf der Erde ist. Und Er ist der Allmächtige und All weise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcxxviii{Er ist es, Der diejenigen von den Leuten der Schrift, die ungläubig sind, aus ihren Wohnstätten zur ersten Versammlung vertrieben hat. Ihr habt nicht geglaubt, daß sie fortziehen würden; und sie meinten, daß ihre Festungen sie vor Allah schützten. Da kam Allah über sie, von wo sie nicht (damit) rechneten, und jagte in ihre Herzen Schrecken, so daß sie ihre Häuser mit ihren (eigenen) Händen und den Händen der Gläubigen zerstörten. Darum zieht die Lehre daraus, o die ihr Einsicht besitzt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcxxix{Und wenn Allah ihnen nicht die Verbannung vorgeschrieben hätte, hätte Er sie wahrlich im Diesseits gestraft; und im Jenseits wird es für sie die Strafe des (Höllen)feuers geben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcxxx{Dies dafür, daß sie Allah und Seinem Gesandten entgegengewirkt haben. Und wer Allah entgegenwirkt, gewiß, so ist Allah streng im Bestrafen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxi{Was an Palmen ihr umgehauen habt oder auf ihren Wurzeln habt stehenlassen, so geschah es mit Allahs Erlaubnis, und damit Er die Frevler in Schande stürze.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxii{Und für das, was Allah Seinem Gesandten von ihnen als kampflose Beute zugeteilt hat, habt ihr weder Pferde noch Reitkamele in Trab gebracht, sondern Allah gibt Seinen Gesandten Gewalt, über wen Er will. Und Allah hat zu allem die Macht.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxiii{Was Allah Seinem Gesandten von den Bewohnern der Städte als kampflose Beute zugeteilt hat, das gehört Allah, Seinem Gesandten und den Verwandten, den Waisen, den Armen und dem Sohn des Weges. Dies, damit es nicht nur im Kreis der Reichen von euch bleibt. Was nun der Gesandte euch gibt, das nehmt; und was er euch untersagt, dessen enthaltet euch. Und fürchtet Allah. Gewiß, Allah ist streng im Bestrafen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxiv{Das gehört) den armen Auswanderern, die aus ihren Wohnstätten und von ihrem Besitz vertrieben worden sind, weil sie nach Huld von Allah und Wohlgefallen trachten und Allah und Seinem Gesandten helfen. Das sind die Wahrhaftigen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxv{Und diejenigen, die in der Wohnstätte und im Glauben vor ihnen zu Hause waren, lieben (all die,) wer zu ihnen ausgewandert ist, und empfinden in ihren Brüsten kein Bedürfnis nach dem, was (diesen) gegeben worden ist, und sie ziehen (sie) sich selbst vor, auch wenn sie selbst Mangel erlitten. Und diejenigen, die vor ihrer eigenen Habsucht bewahrt bleiben, das sind diejenigen, denen es wohl ergeht.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxvi{Und diejenigen, die nach ihnen gekommen sind, sagen: ``Unser Herr, vergib uns und unseren Brüdern, die uns im Glauben vorausgegangen sind, und lasse in unseren Herzen keinen Groll sein gegen diejenigen, die glauben. Unser Herr, Du bist ja Gnädig und Barmherzig."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxvii{Siehst du nicht jene, die heucheln? Sie sagen zu ihren Brüdern, die ungläubig sind von den Leuten der Schrift: ``Wenn ihr tatsächlich vertrieben werdet, werden wir ganz gewiß mit euch fortziehen, und wir werden niemals jemandem gegen euch gehorchen. Und wenn gegen euch tatsächlich gekämpft wird, werden wir euch ganz gewiß helfen." Doch Allah bezeugt, daß sie wahrlich lügen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxviii{Wenn sie tatsächlich vertrieben werden, werden sie nicht mit ihnen fortziehen; und wenn tatsächlich gegen sie gekämpft wird, werden sie ihnen nicht helfen. Und wenn sie ihnen (auch) tatsächlich helfen sollten, werden sie ganz gewiß den Rücken kehren, und hierauf wird ihnen (selbst) keine Hilfe zuteil werden.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxix{Vor euch empfinden sie wahrlich größere Angst in ihren Brüsten als vor Allah. Dies (ist so), weil sie Leute sind, die nicht verstehen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcxl{Sie werden nicht alle gegen euch kämpfen, außer in befestigten Städten oder hinter Mauern. Ihre Gewalt ist untereinander stark. Du meinst, sie halten zusammen, doch ihre Herzen sind verschieden. Dies (ist so), weil sie Leute sind, die nicht begreifen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcxli{Es ist wie das Gleichnis derjenigen (, die es) nahe vor ihnen (erlebten). Sie haben die schlimmen Folgen ihres Verhaltens gekostet, und für sie wird es schmerzhafte Strafe geben.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcxlii{Es ist wie das Gleichnis des Satans, wenn er zum Menschen sagt: ``Sei ungläubig." Wenn er dann ungläubig geworden ist, sagt er: ``Gewiß, ich bin unschuldig an dir. Gewiß, ich fürchte Allah, den Herrn der Weltenbewohner."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcxliii{Und so wird das Ende von beiden sein, daß sie im (Höllen)feuer sein werden; ewig werden beide darin bleiben; das ist der Lohn der Ungerechten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcxliv{O die ihr glaubt, fürchtet Allah. Und eine jede Seele schaue, was sie für morgen vorausschickt. Und fürchtet Allah; gewiß, Allah ist Kundig dessen, was ihr tut.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcxlv{Und seid nicht wie diejenigen, die Allah vergessen haben und die Er dann sich selbst hat vergessen lassen. Das sind die Frevler.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcxlvi{Nicht gleich sind die Insassen des (Höllen)feuers und die Insassen des (Paradies)gartens. Die Insassen des (Paradies)gartens sind die Erfolgreichen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcxlvii{Wenn Wir diesen Qur'an (als Offenbarung) auf einen Berg hinabsendeten, würdest du ihn wahrlich aus Furcht vor Allah demütig werden und sich spalten sehen. Diese Gleichnisse prägen Wir den Menschen, auf daß sie nachdenken mögen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcxlviii{Er ist Allah, außer dem es keinen Gott gibt, der Kenner des Verborgenen und des Offenbaren. Er ist der Allerbarmer und Barmherzige.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcxlix{Er ist Allah, außer dem es keinen Gott gibt, der König, der Heilige, der Friede', der Gewährer der Sicherheit, der Wächter, der Allmächtige, der Gewalthaber, der Stolze. Preis sei Allah! (Und Erhaben ist Er) über das, was sie (Ihm) beigesellen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcl{Er ist Allah, der Schöpfer, der Erschaffer, der Gestalter. Sein sind die schönsten Namen. Ihn preist (alles), was in den Himmeln und auf der Erde ist. Und Er ist der Allmächtige und Allweise.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcli{\basmalahde O die ihr glaubt, nehmt nicht Meine Feinde und eure Feinde zu Schutzherren, indem ihr ihnen Zuneigung entgegenbringt, wo sie doch das verleugnen, was von der Wahrheit zu euch gekommen ist, und den Gesandten und euch selbst vertreiben, weil ihr an Allah, euren Herrn, glaubt. (Nehmt sie nicht zu Schutzherren), wenn ihr wirklich ausgezogen seid zum Abmühen auf Meinem Weg und im Trachten nach Meiner Zufriedenheit. (Tut das nicht, indem) ihr ihnen heimlich Zuneigung zeigt, wo Ich doch besser weiß, was ihr verbergt und was ihr offenlegt. Und wer von euch das tut, der ist fürwahr vom rechten Weg abgeirrt.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmclii{Wenn sie auf euch treffen, sind sie euch feind und strecken gegen euch ihre Hände und ihre Zungen zum Bösen aus. Sie möchten gern, ihr würdet ungläubig.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcliii{Nicht nützen werden euch eure Blutsverwandten und auch nicht eure Kinder. Am Tag der Auferstehung wird Er (Allah) zwischen euch entscheiden'. Und was ihr tut, sieht Allah wohl.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcliv{Ihr habt doch ein schönes Vorbild in Ibrahim und denjenigen, die mit ihm waren, als sie zu ihrem Volk sagten: ``Wir sind unschuldig an euch und an dem, dem ihr anstatt Allahs dient. Wir verleugnen euch, und zwischen uns und euch haben sich Feindschaft und Haß auf immer offenkundig gezeigt, bis ihr an Allah allein glaubt." (Dies), außer das Wort Ibrahims zu seinem Vater: ``Ich werde ganz gewiß für dich um Vergebung bitten; doch vermag ich für dich vor Allah gar nichts auszurichten. Unser Herr, auf Dich verlassen wir uns, und Dir wenden wir uns reuig zu. Und zu Dir ist der Ausgang.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmclv{Unser Herr, mache uns nicht zu einer Versuchung für diejenigen, die ungläubig sind. Und vergib uns, unser Herr. Du bist ja der Allmächtige und Allweise."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmclvi{Ihr habt ja in ihnen ein schönes Vorbild, und zwar für einen (jeden), der auf Allah und (auf) den Jüngsten Tag hofft. Wer sich jedoch abkehrt, so ist Allah der Unbedürftige und Lobenswürdige.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmclvii{Vielleicht setzt Allah zwischen euch und denjenigen von ihnen, mit denen ihr verfeindet seid, Zuneigung! Und Allah ist Allmächtig. Und Allah ist Allvergebend und Barmherzig.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmclviii{Allah verbietet euch nicht, gegenüber denjenigen, die nicht gegen euch der Religion wegen gekämpft und euch nicht aus euren Wohnstätten vertrieben haben, gütig zu sein und sie gerecht zu behandeln. Gewiß, Allah liebt die Gerechten.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmclix{Er verbietet euch nur, diejenigen, die gegen euch der Religion wegen gekämpft und euch aus euren Wohnstätten vertrieben und zu eurer Vertreibung Beistand geleistet haben, zu Schutzherren zu nehmen. Diejenigen, die sie zu Schutzherren nehmen, das sind die Ungerechten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmclx{O die ihr glaubt, wenn gläubige Frauen als Auswanderer zu euch kommen, dann prüft sie. Allah weiß besser über ihren Glauben Bescheid. Wenn ihr sie dann als gläubig erkennt, dann schickt sie nicht zu den Ungläubigen zurück. Weder sind sie ihnen (zur Ehe) erlaubt, noch sind sie ihnen (diesen Frauen) erlaubt. Und gebt ihnen (jedoch), was sie (früher als Morgengabe) ausgegeben haben. Es ist für euch kein Vergehen, sie zu heiraten, wenn ihr ihnen ihren Lohn gebt. Und haltet nicht an der Ehe mit den ungläubigen Frauen fest, und fordert, was ihr (als Morgengabe) ausgegeben habt, zurück. Auch sie sollen zurückfordern, was sie ausgegeben haben. Das ist Allahs Urteil; Er richtet zwischen euch. Und Allah ist Allwissend und Allweise.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmclxi{Und wenn euch etwas von euren Gattinnen bei den Ungläubigen verlorengeht und ihr dann (in) eine(r) Strafmaßnahme (Beute) macht, dann gebt denjenigen, deren Gattinnen fortgegangen sind, soviel, wie sie (zuvor für sie) ausgegeben haben. Und fürchtet Allah, an Den ihr glaubt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmclxii{O Prophet, wenn gläubige Frauen zu dir kommen, um dir den Treueid zu leisten, daß sie Allah nichts beigesellen, nicht stehlen, keine Unzucht begehen, ihre Kinder nicht töten, keine Verleumdung vorbringen, die sie vor ihren (eigenen) Händen und Füßen ersinnen, und sich dir nicht widersetzen in dem, was recht ist, dann nimm ihren Treueid an und bitte Allah für sie um Vergebung. Gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmclxiii{O die ihr glaubt, nehmt nicht Leute zu Schutzherren, denen Allah zürnt. Sie haben ja die Hoffnung auf das Jenseits aufgegeben, so wie die Ungläubigen die Hoffnung (auf Wiederauferstehung) der (Toten) in den Gräbern aufgegeben haben.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmclxiv{\basmalahde Allah preist (alles), was in den Himmeln und was auf der Erde ist. Und Er ist der Allmächtige und Allweise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmclxv{O die ihr glaubt, warum sagt ihr, was ihr nicht tut?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmclxvi{Welch schwerwiegende Abscheu erregt es bei Allah, daß ihr sagt, was ihr nicht tut.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmclxvii{Gewiß, Allah liebt diejenigen, die auf Seinem Weg kämpfen in Reihe, als wären sie ein zusammengefügter Bau.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmclxviii{Und als Musa zu seinem Volk sagte: ``O mein Volk, warum fügt ihr mir Leid zu, wo ihr doch sicher wißt, daß ich Allahs Gesandter an euch bin?" Als sie nun abschweiften, ließ Allah ihre Herzen abschweifen. Und Allah leitet das Volk der Frevler nicht recht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmclxix{Und als 'Isa, der Sohn Maryams, sagte: ``O Kinder Isra'ils, gewiß, ich bin Allahs Gesandter an euch, das bestätigend, was von der Tora vor mir (offenbart) war, und einen Gesandten verkündend, der nach mir kommen wird: sein Name ist Ahmad'." Als er nun mit den klaren Beweisen zu ihnen kam, sagten sie: ``Das ist deutliche Zauberei."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmclxx{Und wer ist ungerechter, als wer gegen Allah Lügen ersinnt, während er zum Islam aufgerufen wird? Und Allah leitet das Volk der Ungerechten nicht recht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmclxxi{Sie wollen gern Allahs Licht mit ihren Mündern auslöschen. Aber Allah wird Sein Licht vollenden, auch wenn es den Ungläubigen zuwider ist.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmclxxii{Er ist es, Der Seinen Gesandten mit der Rechtleitung und der Religion der Wahrheit gesandt hat, um ihr die Oberhand über alle Religion zu geben, auch wenn es den Götzendienern zuwider ist.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmclxxiii{O die ihr glaubt, soll ich euch auf einen Handel hinweisen, der euch vor schmerzhafter Strafe rettet?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmclxxiv{Ihr sollt an Allah und Seinen Gesandten glauben und euch auf Allahs Weg mit eurem Besitz und mit eurer eigenen Person abmühen; das ist besser für euch, wenn ihr nur wißt -,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmclxxv{dann wird Er euch eure Sünden vergeben und euch eingehen lassen in Gärten, durcheilt von Bächen, und in angenehme Wohnungen in den Gärten Edens. Das ist der großartige Erfolg.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmclxxvi{Und (noch) eine andere (Huld), die ihr liebt (, wird euch gewährt): Hilfe von Allah und ein naher Sieg. Und (so) verkünde (diese) frohe Botschaft den Gläubigen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmclxxvii{O die ihr glaubt, seid Allahs Helfer, so wie 'Isa, der Sohn Maryams, zu den Jüngern sagte: ``Wer sind meine Helfer (auf dem Weg) zu Allah (hin)?" Die Jünger sagten: ``Wir sind Allahs Helfer." So glaubte ein Teil von den Kindern Isra'ils, während ein (anderer) Teil ungläubig war. Da stärkten Wir diejenigen, die glaubten, gegen ihre Feinde, und so bekamen sie die Oberhand.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmclxxviii{\basmalahde Allah preist (alles), was in den Himmeln und was auf der Erde ist, (Ihn), den König, den Heiligen, den Allmächtigen und Allweisen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmclxxix{Er ist es, Der unter den Schriftunkundigen einen Gesandten von ihnen hat erstehen lassen, der ihnen Seine Zeichen verliest, sie läutert und sie das Buch und die Weisheit lehrt, obgleich sie sich ja zuvor in deutlichem Irrtum befanden -,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmclxxx{und andere vor ihnen, die sich ihnen noch nicht angeschlossen haben. Und Er ist der Allmächtige und Allweise.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxi{Das ist Allahs Huld, die Er gewährt, wem Er will. Und Allah besitzt große Huld.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxii{Das Gleichnis derjenigen, denen die Tora auferlegt wurde, die sie aber hierauf doch nicht getragen haben, ist das eines Esels, der Bücher trägt. Schlimm ist das Gleichnis der Leute, die Allahs Zeichen für Lüge erklären. Und Allah leitet das ungerechte Volk nicht recht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxiii{Sag: O die ihr dem Judentum angehört, wenn ihr behauptet, daß ihr Allahs Schützlinge unter Ausschluß der (anderen) Menschen seid, dann wünscht (euch doch) den Tod, wenn ihr wahrhaftig seid.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxiv{Aber sie wünschen sich ihn niemals wegen dessen, was ihre Hände vorausgeschickt haben. Und Allah weiß über die Ungerechten Bescheid.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxv{Sag: Gewiß, der Tod, vor dem ihr flieht -, gewiß, er wird euch begegnen. Hierauf werdet ihr zu dem Kenner des Verborgenen und des Offenbaren zurückgebracht, und dann wird Er euch kundtun, was ihr zu tun pflegtet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxvi{O die ihr glaubt, wenn zum Gebet gerufen wird am Freitag, dann eilt zu Allahs Gedenken und laßt das Kaufgeschäft. Das ist besser für euch, wenn ihr wißt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxvii{Wenn das Gebet beendet ist, dann breitet euch im Land aus und trachtet nach etwas von Allahs Huld. Und gedenkt Allahs viel, auf daß es euch wohl ergehen möge!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxviii{Und wenn sie einen Handel oder eine Zerstreuung sehen, laufen sie dorthin auseinander und lassen dich stehen. Sag: Was bei Allah ist, ist besser als Zerstreuung und als Handel. Und Allah ist der beste Versorger.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxix{\basmalahde Wenn die Heuchler zu dir kommen, sagen sie: ``Wir bezeugen, daß du wahrlich Allahs Gesandter bist". Allah weiß, daß du fürwahr Sein Gesandter bist; doch Allah bezeugt, daß die Heuchler wahrlich lügen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcxc{Sie haben sich ihre Eide zu einem Schutzschirm genommen und halten so von Allahs Weg ab. Gewiß, wie böse ist, was sie zu tun pflegen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcxci{Dies, weil sie (zuerst) gläubig gewesen, hierauf aber ungläubig geworden sind. Da wurden ihre Herzen versiegelt; so verstehen sie nicht.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcxcii{Und wenn du sie siehst, gefällt dir ihr Äußeres, und wenn sie reden, hörst du ihren Worten zu. Sie sind wie angelehnte Hölzer. Sie meinen, jeder Schrei sei gegen sie. Sie sind die (wahren) Feinde; so sieh dich vor ihnen vor. Allah bekämpfe sie! Wie sie sich (doch) abwendig machen lassen!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcxciii{Und wenn zu ihnen gesagt wird: ``Kommt her, damit Allahs Gesandter für euch um Vergebung bittet", drehen sie ihre Köpfe zur Seite, und du siehst sie sich hochmütig abwenden.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcxciv{Gleich ist es in Bezug auf sie, ob du für sie um Vergebung bittest oder nicht bittest; Allah wird ihnen nicht vergeben. Gewiß, Allah leitet das Volk der Frevler nicht recht.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcxcv{Sie sind es, die sagen: ``Gebt nicht aus für diejenigen, die bei Allahs Gesandtem sind, bis sie (von ihm weg) auseinander (und davon)laufen." Doch Allah gehören die Schatzkammern der Himmel und der Erde. Aber die Heuchler verstehen nicht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcxcvi{Sie sagen: ``Wenn wir nach al-Madina zurückkehren, so wird ganz gewiß der Mächtigere den Niedrigeren aus ihr vertreiben." Doch Allah gehört die Macht, und auch Seinem Gesandten und den Gläubigen. Aber die Heuchler wissen nicht.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcxcvii{O die ihr glaubt, nicht ablenken sollen euch euer Besitz und eure Kinder von Allahs Gedenken. Diejenigen, die dies tun, das sind die Verlierer.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcxcviii{Und gebt aus von dem, womit Wir euch versorgt haben, bevor zu einem von euch der Tod kommt und er dann sagt: ``Mein Herr, würdest Du mich doch auf eine kurze Frist zurückstellen! Dann würde ich Almosen geben und zu den Rechtschaffenen gehören."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcxcix{Allah wird aber keine Seele zurückstellen, wenn ihre Frist kommt. Und Allah ist Kundig dessen, was ihr tut.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcc{\basmalahde Allah preist (alles), was in den Himmeln und (alles) was auf der Erde ist. Ihm gehört die Herrschaft, und Ihm gehört (alles) Lob, und Er hat zu allem die Macht.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcci{Er ist es, Der euch erschaffen hat. So gibt es unter euch Ungläubige und gibt es unter euch Gläubige, und was ihr tut, sieht Allah wohl.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccii{Er hat die Himmel und die Erde in Wahrheit erschaffen und euch gestaltet, und da(bei) eure Gestalten schön gemacht. Und zu Ihm ist der Ausgang.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcciii{Er weiß, was in den Himmeln und auf der Erde ist, und Er weiß, was ihr geheimhaltet und was ihr offenlegt. Und Allah weiß über das Innerste der Brüste Bescheid.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcciv{Ist zu euch nicht die Kunde von denjenigen gekommen, die zuvor ungläubig waren und dann die schlimmen Folgen ihres Verhaltens kosteten? Und für sie wird es schmerzhafte Strafe geben.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccv{Dies, weil ihre Gesandten immer wieder zu ihnen mit den klaren Beweisen kamen, sie aber sagten: ``Sollen (etwa) menschliche Wesen uns rechtleiten?" So wurden sie ungläubig und kehrten sich ab. Doch Allah bedurfte (ihrer) nicht, und Allah ist Unbedürftig und Lobenswürdig.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccvi{Diejenigen, die ungläubig sind, behaupten, daß sie nicht auferweckt werden. Sag: Aber ja doch, bei meinem Herrn, ihr werdet ganz gewiß auferweckt werden; hierauf wird euch ganz gewiß das Kundgetan, was ihr getan habt. Und dies ist Allah ein leichtes.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccvii{Darum glaubt an Allah und Seinen Gesandten und das Licht, das Wir hinabgesandt haben. Und Allah ist dessen, was ihr tut, Kundig.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccviii{Am Tag, da Er euch zum Tag der Versammlung versammeln wird. Das ist der Tag der Übervorteilung'. Wer an Allah glaubt und rechtschaffen handelt, dem tilgt Er seine bösen Taten, und den wird Er in Gärten eingehen lassen, durcheilt von Bächen, ewig und auf immer darin zu bleiben; das ist der großartige Erfolg.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccix{Und diejenigen, die ungläubig sind und Unsere Zeichen für Lüge erklären, das sind Insassen des (Höllen)feuers, ewig darin zu bleiben - ein schlimmer Ausgang!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccx{Kein Unglück trifft (jemanden), außer mit Allahs Erlaubnis. Und wer an Allah glaubt, dessen Herz leitet Er recht. Und Allah weiß über alles Bescheid.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccxi{Und gehorcht Allah und gehorcht dem Gesandten. Wenn ihr euch jedoch abkehrt, so obliegt Unserem Gesandten nur die deutliche Übermittlung (der Botschaft).\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccxii{Allah - es gibt keinen Gott außer Ihm. Und auf Allah sollen sich die Gläubigen verlassen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmccxiii{O die ihr glaubt, unter euren Gattinnen und euren Kindern gibt es welche, die euch feind sind; so seht euch vor ihnen vor. Wenn ihr aber verzeiht, nachsichtig seid und vergebt - gewiß, so ist Allah Allvergebend und Barmherzig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmccxiv{Euer Besitz und eure Kinder sind nur eine Versuchung; Allah aber - bei Ihm gibt es großartigen Lohn.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmccxv{Daher fürchtet Allah, soweit ihr könnt. Und hört zu und gehorcht und gebt Gutes für euch selbst aus'. Und diejenigen, die vor ihrer eigenen Habsucht bewahrt bleiben, das sind diejenigen, denen es wohl ergeht.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmccxvi{Wenn ihr Allah ein gutes Darlehen gebt, wird Er es euch vervielfachen und euch vergeben. Und Allah ist stets zu Dank bereit und Nachsichtig.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmccxvii{Er ist) der Kenner des Verborgenen und des Offenbaren, der Allmächtige und Allweise.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmccxviii{\basmalahde O Prophet, wenn ihr euch von Frauen scheidet, dann scheidet euch von ihnen auf ihre Wartezeit hin, und berechnet die Wartezeit. Und fürchtet Allah, euren Herrn. Weist sie nicht aus ihren Häusern aus; sie sollen auch nicht selbst ausziehen, außer, sie begehen etwas klar Abscheuliches. Dies sind Allahs Grenzen\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccxix{Wer aber Allahs Grenzen übertritt, der fügt sich ja selbst Unrecht zu. Du weißt nicht, vielleicht führt Allah danach eine neue Lage herbei. Wenn sie dann ihre Frist erreichen, so behaltet sie in rechtlicher Weise oder trennt euch von ihnen in rechtlicher Weise. Und nehmt zwei gerechte Personen von euch zu Zeugen, und legt das Zeugnis (in Aufrichtigkeit) um Allahs willen ab. Damit wird ermahnt, wer an Allah und den Jüngsten Tag glaubt. Und wer Allah fürchtet, dem schafft Er einen Ausweg\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccxx{und gewährt ihm Versorgung, von wo (aus) er damit nicht rechnet. Und wer sich auf Allah verläßt, dem ist Er seine Genüge. Allah wird gewiß (die Durchführung) seine(r) Angelegenheit erreichen. Allah legt ja für alles ein Maß fest.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccxxi{Und diejenigen von euren Frauen, die keine Monatsblutung mehr erwarten, wenn ihr im Zweifel seid, so ist ihre Wartezeit drei Monate; und ebenso derjenigen, die (noch) keine Monatsblutung haben. Diejenigen, die schwanger sind - ihre Frist ist (erreicht), wenn sie mit dem niederkommen, was sie (in ihren Leibern) tragen. Und wer Allah fürchtet, dem schafft Er in seiner Angelegenheit Erleichterung.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccxxii{Das ist Allahs Anordnung, die Er zu euch (als Offenbarung) herabgesandt hat. Und wer Allah fürchtet, dem tilgt Er seine bösen Taten und dem gewährt Er großartigen Lohn.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccxxiii{Laßt sie dort wohnen, wo ihr (selbst) wohnt, von dem, was ihr euch leisten könnt. Und fügt ihnen keinen Schaden zu, um sie in die Enge zu treiben. Und wenn sie schwanger sind, dann gebt für sie (das Nötige) aus, bis sie mit dem niederkommen, was sie (in ihren Leibern) tragen. Wenn sie für euch (das Kind) stillen, dann gebt ihnen ihren Lohn, und beratet untereinander (darüber) in rechtlicher Weise. Wenn ihr aber einander Schwierigkeiten bereitet, dann wird (das Kind) für ihn eine andere stillen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccxxiv{Der Wohlhabende soll entsprechend seinem Wohlstand (die Aufwendungen) ausgeben. Und wem seine Versorgung bemessen (zugeteilt) wurde, der soll (eben) von dem ausgeben, was Allah ihm gegeben hat. Allah erlegt keiner Seele mehr auf als das, was Er ihr gegeben hat. Allah wird nach Schwierigkeit Erleichterung schaffen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccxxv{Und wieviele Städte lehnten sich gegen den Befehl ihres Herrn und Seiner Gesandten auf, worauf Wir sie einer strengen Abrechnung unterzogen und mit entsetzlicher Strafe straften.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccxxvi{Da kosteten sie die schlimmen Folgen ihres Verhaltens, und das Ende ihres Verhaltens war (nur) Verlust.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccxxvii{Allah hat für sie strenge Strafe bereitet. So fürchtet Allah, o die ihr Verstand besitzt und die ihr glaubt. Allah hat ja zu euch eine Ermahnung (als Offenbarung) herabgesandt,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccxxviii{einen Gesandten, der euch Allahs Zeichen verliest, die alles klar machen, um diejenigen, die glauben und rechtschaffene Werke tun, aus den Finsternissen ins Licht hinauszubringen. Und wer an Allah glaubt und rechtschaffen handelt, den wird Er in Gärten eingehen lassen, durcheilt von Bächen, ewig und auf immer darin zu bleiben. Allah hat ihm (damit) ja eine schöne Versorgung gewährt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccxxix{Allah ist es, Der sieben Himmel erschaffen hat, und von der Erde gleich (viel). Der Befehl (Allahs) kommt wahrhaftig zwischen ihnen herab, damit ihr wißt, daß Allah zu allem die Macht hat und daß Allah ja alles mit Seinem Wissen umfaßt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccxxx{\basmalahde O Prophet, warum verbietest du, was Allah dir erlaubt hat, indem du da nach trachtest, die Zufriedenheit deiner Gattinnen zu erlangen? Und Allah ist Allvergebend und Barmherzig.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxi{Allah hat ja für euch festgelegt, womit eure Eide zu lösen sind. Und Allah ist euer Schutzherr, und Er ist der Allwissende und Allweise.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxii{Als der Prophet einer seiner Gattinnen eine Mitteilung im geheimen anvertraute. Als sie sie dann kundtat und Allah es ihm offen darlegte, gab er (ihr) einen Teil davon bekannt und überging einen (anderen) Teil. Als er es ihr nun kundtat, sagte sie: ``Wer hat dir das mitgeteilt?" Er sagte: ``Kundgetan hat (es) mir der Allwissende und Allkundige."\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxiii{Wenn ihr beide euch in Reue zu Allah umkehrt - so werden eure Herzen sich ja (der Aufrichtigkeit) zuneigen - ... Und wenn ihr einander gegen ihn beisteht, so ist Allah sein Schutzherr; und Gibril, die Rechtschaffenen von den Gläubigen, und die Engel werden darüber hinaus (ihm) Beistand sein.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxiv{Vielleicht wird sein Herr ihm, wenn er sich von euch (allen) scheidet, zum Tausch (andere) Gattinnen geben, (die) besser (sind) als ihr: muslimische, gläubige, demütig ergebene, in Reue umkehrende, (Allah) dienende und umherziehende, früher verheiratete oder Jungfrauen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxv{O die ihr glaubt, bewahrt euch selbst und eure Angehörigen vor einem Feuer, dessen Brennstoff Menschen und Steine sind, über das hartherzige, strenge Engel (gesetzt) sind, die sich Allah nicht widersetzen in dem, was Er ihnen befiehlt, sondern tun, was ihnen befohlen wird.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxvi{O die ihr ungläubig seid, entschuldigt euch heute nicht. Euch wird nur das vergolten, was ihr zu tun pflegtet.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxvii{O die ihr glaubt, kehrt zu Allah um in aufrichtiger Reue; vielleicht wird euer Herr euch eure bösen Taten tilgen und euch in Gärten eingehen lassen, durcheilt von Bächen, am Tag, da Allah den Propheten und diejenigen, die mit ihm geglaubt haben, nicht in Schande stürzen wird. Ihr Licht eilt ihnen voraus und (ebenso) zu ihrer Rechten. Sie werden sagen: ``Unser Herr, vollende für uns unser Licht und vergib uns. Gewiß, Du hast zu allem die Macht."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxviii{O Prophet, mühe dich gegen die Ungläubigen und die Heuchler ab und sei hart gegen sie. Ihr Zufluchtsort wird die Hölle sein - ein schlimmer Ausgang!\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxix{Allah hat als Gleichnis für diejenigen, die ungläubig sind, dasjenige von Nuhs Frau und Luts Frau geprägt. Sie beide unterstanden zwei rechtschaffenen (Dienern) von Unseren Dienern, doch handelten sie verräterisch an ihnen, und so haben ihnen diese vor Allah nichts genützt. Und es wurde gesagt: ``Geht beide ins (Höllen)feuer ein mit denjenigen, die hineingehen."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccxl{Und Allah hat als Gleichnis für diejenigen, die glauben, dasjenige von Fir'auns Frau geprägt. Als sie sagte: ``Mein Herr, baue mir bei Dir ein Haus im (Paradies)garten, und errette mich von Fir'aun und seinem Werk, und errette mich von dem Volk der Ungerechten."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccxli{Und (auch von) Maryam, 'Imrans Tochter, die ihre Scham unter Schutz stellte, worauf Wir in sie von Unserem Geist einhauchten. Und sie hielt die Worte ihres Herrn und Seine Bücher für wahr und gehörte zu den (Allah) demütig Ergebenen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccxlii{\basmalahde Segensreich ist Derjenige, in Dessen Hand die Herrschaft ist, und Er hat zu allem die Macht.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccxliii{Er,) Der den Tod und das Leben erschaffen hat, damit Er euch prüfe, wer von euch die besten Taten begeht. Und Er ist der Allmächtige und Allvergebende.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccxliv{Er,) Der sieben Himmel in Schichten (übereinander) erschaffen hat. Du kannst in der Schöpfung des Allerbarmers keine Ungesetzmäßigkeit sehen. Wende den Blick zurück: Siehst du irgendwelche Risse?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccxlv{Hierauf wende den Blick wiederholt zurück. Der Blick wird zu dir erfolglos und ermüdet zurückkehren.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccxlvi{Wir haben ja den untersten Himmel mit Lampen geschmückt und haben sie zu Wurfgeschossen gegen die Satane gemacht. Und bereitet haben Wir für sie die Strafe der Feuerglut.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccxlvii{Und für diejenigen, die ihren Herrn verleugnen, wird es die Strafe der Hölle geben - ein schlimmer Ausgang!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccxlviii{Wenn sie hineingeworfen werden, hören sie von ihr stoßweises Aufheulen, während sie brodelt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccxlix{Beinahe würde sie vor Grimm bersten. Jedesmal, wenn eine Schar hineingeworfen wird, werden ihre Wärter sie fragen: ``Ist zu euch denn kein Warner gekommen?"\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccl{Sie werden sagen: ``Ja doch, zu uns kam bereits ein Warner; aber wir haben (ihn) der Lüge bezichtigt und gesagt: , Allah hat nichts offenbart; ihr befindet euch nur in großem Irrtum'."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccli{Und sie werden sagen: ``Hätten wir nur gehört und begriffen, wären wir (nun) nicht unter den Insassen der Feuerglut."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcclii{So werden sie ihre Sünde bekennen. Weit weg darum mit den Insassen der Feuerglut!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccliii{Gewiß, diejenigen, die ihren Herrn im Verborgenen fürchten, für sie wird es Vergebung und großen Lohn geben.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccliv{Haltet eure Worte geheim oder äußert sie laut vernehmbar, gewiß, Er weiß über das Innerste der Brüste Bescheid.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcclv{Sollte denn Derjenige, Der erschaffen hat, nicht Bescheid wissen'? Und Er ist der Feinfühlige und Allkundige.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcclvi{Er ist es, Der euch die Erde fügsam gemacht hat. So geht auf ihrem Rücken einher und eßt von dem, womit Er (euch) versorgt. Und zu Ihm wird die Auferstehung sein.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcclvii{Glaubt ihr in Sicherheit davor zu sein, daß Wer im Himmel ist, die Erde mit euch versinken läßt, so daß sie sich dann hin und her bewegt?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcclviii{Oder glaubt ihr euch in Sicherheit davor, daß Wer im Himmel ist, gegen euch einen Sturm von Steinchen sendet? Dann werdet ihr (noch) erfahren, wie Meine Warnung ist.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcclix{Auch diejenigen, die vor ihnen waren, haben bereits (die Gesandten) der Lüge bezichtigt. Wie war da Meine Mißbilligung!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcclx{Sehen sie denn nicht auf die Vögel über ihnen, wie sie ihre Flügel ausgebreitet halten und einziehen? Nur der Allerbarmer hält sie. Gewiß, Er sieht alles wohl.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcclxi{Oder wer ist denn dieser, der euch eine Heerschar wäre, die euch helfen (sollte) außer dem Allerbarmer? Die Ungläubigen befinden sich nur im Trug.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcclxii{Oder wer ist denn dieser, der euch versorgen (sollte), wenn Er Seine Versorgung zurückhält? Aber nein! Sie fahren weiter in Auflehnung und Abneigung.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcclxiii{Ist denn jemand, der, auf sein Gesicht gestürzt, einhergeht, eher rechtgeleitet, oder jemand, der aufrecht auf einem geraden Weg einhergeht?\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcclxiv{Sag: Er ist es, Der euch hat entstehen lassen und euch Gehör, Augenlicht und Herzen gemacht hat. Wie wenig ihr dankbar seid!\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcclxv{Sag: Er ist es, Der euch auf der Erde (an Zahl) hat anwachsen lassen, und zu Ihm werdet ihr versammelt werden.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcclxvi{Und sie sagen: ``Wann wird dieses Versprechen eintreten, wenn ihr wahrhaftig seid?"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcclxvii{Sag: Nur Allah weiß (darüber) Bescheid. Und ich bin nur ein deutlicher Warner.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcclxviii{Wenn sie es aber nahe (bei sich) sehen, dann werden die Gesichter derjenigen, die ungläubig sind, böse betroffen sein. Und es wird gesagt werden: ``Das ist das, was ihr stets herbeizurufen wünschtet."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcclxix{Sag: Was meint ihr, wenn Allah mich und wer mit mir ist, vernichten oder wenn Er Sich unser erbarmen sollte, wer wird dann den Ungläubigen Schutz vor schmerzhafter Strafe gewähren?\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcclxx{Sag: Er ist der Allerbarmer. Wir glauben an Ihn, und auf Ihn verlassen wir uns. Ihr werdet (noch) erfahren, wer sich in deutlichem Irrtum befindet.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxi{Sag: Was meint ihr, wenn euer Wasser versickert sein sollte, wer sollte euch dann hervorquellendes Wasser bringen?\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxii{\basmalahde Nun. Beim Schreibrohr und dem, was sie in Zeilen niederschreiben.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxiii{Du bist durch die Gunst deines Herrn kein Besessener.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxiv{Es wird für dich wahrlich Lohn geben, der nicht aufhört.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxv{Und du bist wahrlich von großartiger Wesensart.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxvi{Du wirst sehen, und (auch) sie werden sehen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxvii{wer von euch (durch Besessenheit) der Versuchung ausgesetzt ist.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxviii{Gewiß, dein Herr weiß sehr wohl, wer von Seinem Weg abirrt, und Er kennt sehr wohl die Rechtgeleiteten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxix{So gehorche nicht den Leugnern (der Botschaft).\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxx{Sie möchten gern, daß du schmeichelst, so daß (auch) sie schmeicheln (können).\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxi{Und gehorche keinem verächtlichen Schwörer,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxii{Stichler und Verbreiter von Verleumdungen,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxiii{Verweigerer des Guten, der Übertretungen begeht und ein Sünder ist,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxiv{einem von grobem Benehmen und darüber hinaus einem Eindringling\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxv{nur deshalb), weil er Besitz und Söhne hat.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxvi{Wenn ihm Unsere Zeichen verlesen werden, sagt er: ``(Es sind) Fabeln der Früheren."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxvii{Wir werden ihn mit einem Merkmal auf der Nase zeichnen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxviii{Wir haben sie geprüft, wie Wir die Besitzer des Gartens prüften, als sie schworen, sie würden ihn bei Tagesanbruch abpflücken,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxix{und nicht(s) ausnahmen'.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmccxc{Da ging darin Verheerendes von deinem Herrn umher, während sie schliefen,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmccxci{und er war dann am Morgen wie abgepflückt.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmccxcii{Da riefen sie bei Tagesanbruch einander zu:\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmccxciii{``Geht in der Morgenfrühe zu eurer Pflanzung, wenn ihr pflücken wollt."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmccxciv{Da zogen sie los und flüsterten dabei einander zu:\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmccxcv{``In ihn soll heute ja kein Armer zu euch hineingelangen."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmccxcvi{Und sie gingen hin in der Morgenfrühe, zum Verwehren bereit.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmccxcvii{Als sie ihn aber sahen, sagten sie: ``Wir haben uns fürwahr geirrt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmccxcviii{Nein! Vielmehr entbehren wir alles."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmccxcix{Der Mittlere von ihnen sagte: ``Habe ich nicht zu euch gesagt: Würdet ihr doch Allah preisen!"\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmccc{Sie sagten: ``Preis sei unserem Herrn! Wir waren gewiß ungerecht."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmccci{Da wandten sie sich einander zu, sich gegenseitig tadelnd.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcccii{Sie sagten: ``O wehe uns! Wir pflegten ja das Maß (an Frevel) zu überschreiten.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmccciii{Vielleicht wird unser Herr uns zum Tausch dafür einen besseren (Garten) als ihn geben! Gewiß, nach Allah steht unser Begehren."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmccciv{So ist die Strafe; aber die Strafe des Jenseits ist wahrlich größer, wenn sie nur wüßten!\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmcccv{Gewiß, für die Gottesfürchtigen wird es bei ihrem Herrn die Gärten der Wonne geben.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmcccvi{Sollen Wir etwa die Gottergebenen den Übeltätern gleichstellen?\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmcccvii{Was ist mit euch? Wie urteilt ihr?\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmcccviii{Oder habt ihr eine Schrift, in der ihr lernt (und sucht);\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmcccix{ihr sollt darin wahrlich das haben, was ihr euch auswählt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmcccx{Oder habt ihr von Uns verbindliche Eide, die bis zum Tag der Auferstehung reichen; ihr bekommt wahrlich das, was ihr urteilt.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmcccxi{Frage sie, wer von ihnen dafür Bürge ist.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmcccxii{Oder haben sie (etwa) Teilhaber? So sollen sie doch ihre Teilhaber beibringen, wenn sie wahrhaftig sind.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmcccxiii{Am Tag, da (manch) ein Schenkel entblößt wird und sie aufgerufen werden, sich (anbetend) niederzuwerfen, aber da werden sie es nicht können,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmcccxiv{während ihre Blicke demütig sind und Erniedrigung sie bedeckt. Dabei waren sie doch bereits aufgefordert worden, sich (anbetend) niederzuwerfen, als sie (noch) heil waren.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmcccxv{Lasse Mich (allein) mit denjenigen, die diese Aussage für Lüge erklären; Wir werden sie stufenweise (dem Verderben) näherbringen, von wo sie nicht wissen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmcccxvi{Und Ich gewähre ihnen Aufschub. (Aber) gewiß, Meine List ist fest.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmcccxvii{Oder verlangst du (etwa) von ihnen einen Lohn, so daß sie mit Schulden belastet wären?\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmcccxviii{Oder besitzen sie (das Wissen über) das Verborgene, so daß sie (es) aufschreiben (können)?\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmcccxix{So sei standhaft in Bezug auf das Urteil deines Herrn'. Und sei nicht wie der Gefährte des (großen) Fisches, als er voller Gram (zu Allah) rief.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmcccxx{Wenn ihn nicht eine Gunst von seinem Herrn rechtzeitig erreicht hätte, wäre er wahrlich auf das kahle Land geworfen worden und hätte sich dabei Vorwürfe zugezogen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxi{Da erwählte ihn sein Herr und machte ihn zu einem der Rechtschaffenen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxii{Diejenigen, die ungläubig sind, würden dich, wenn sie die Ermahnung hören, mit ihren Blicken wahrlich beinahe ins Straucheln bringen. Und sie sagen: ``Er ist ja fürwahr besessen."\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxiii{Doch es ist nur eine Ermahnung für die Weltenbewohner.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxiv{\basmalahde Die fällig Werdende,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxv{was ist die fällig Werdende?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxvi{Und was läßt dich wissen, was die fällig Werdende ist?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxvii{Die Tamud und die 'Ad erklärten das Verhängnis für Lüge.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxviii{Was nun die Tamud angeht, so wurden sie durch den übermäßigen (Donnerschlag) vernichtet.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxix{Was aber die 'Ad angeht, so wurden sie durch einen heftig wehenden eiskalten Wind vernichtet,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxx{den Er entscheidende sieben Nächte und acht Tage fortgesetzt gegen sie einsetzte. Da hättest du in ihnen die Leute (auf dem Boden) niedergestreckt sehen können, als wären sie Stämme hohler Palmen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxi{Siehst du denn etwas von ihnen (übrig)geblieben?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxii{Und es begingen Fir'aun und wer vor ihm war und die umgestürzten Städte Vergehen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxiii{So widersetzten sie sich den Gesandten ihres Herrn. Da ergriff Er sie mit stärker werdendem Griff.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxiv{Gewiß, als das Wasser das Maß überschritt, trugen ja Wir euch auf dem fahrenden (Schiff),\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxv{um es für euch zu einer Erinnerung zu machen, und damit es von einem jeden aufnahmefähigen Ohr aufgenommen wird.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxvi{Wenn ins Horn ein (einziges) Mal geblasen wird\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxvii{und Erde und Berge gehoben und dann mit einem einzigen Schlag zu Staub eingeebnet werden,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxviii{an jenem Tag dann trifft (die Stunde) ein, die eintreffen wird,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxix{und der Himmel spaltet sich, so daß er an jenem Tag brüchig wird,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcccxl{und die Engel (befinden sich) an seinen Seiten. Und den Thron deines Herrn werden über ihnen an jenem Tag acht tragen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcccxli{An jenem Tag werdet ihr vorgeführt; nichts von euch wird verborgen bleiben.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlii{Was nun jemanden angeht, dem dann sein Buch in seine Rechte gegeben wird, der wird sagen: ``Ihr da, lest mein Buch.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcccxliii{Ich glaubte ja, daß ich meiner Abrechnung begegnen werde."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcccxliv{So wird er in einem zufriedenen Leben sein,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlv{in einem hohen Garten,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlvi{dessen Pflückobst herabhängt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlvii{``Eßt und trinkt als wohlbekömmlich für das, was ihr früher in den vergangenen Tagen getan habt."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlviii{Was aber jemanden angeht, dem sein Buch in seine Linke gegeben wird, der wird sagen: ``O wäre mir doch mein Buch nicht gegeben worden!\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlix{Und wüßte ich nicht, was meine Abrechnung ist!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcccl{O wäre dies doch nur das Ende (meines Lebens)!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcccli{Nicht nützt mir mein Besitz.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmccclii{Vernichtet ist meine Macht(fülle)."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcccliii{- ``Nehmt ihn und fesselt ihn dann.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcccliv{Hierauf setzt ihn dem Höllenbrand aus.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmccclv{Hierauf steckt ihn in eine Kette, deren Länge siebzig Ellen ist.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmccclvi{Er pflegte nämlich nicht an Allah, den Allgewaltigen zu glauben\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmccclvii{und nicht zur Speisung des Armen anzuhalten.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmccclviii{So hat er hier heute keinen warmherzigen Freund\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmccclix{und auch keine Speise außer Jauche,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmccclx{die nur diejenigen essen, die Verfehlungen begangen haben."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmccclxi{Nein! Ich schwöre bei dem, was ihr seht,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmccclxii{und dem, was ihr nicht seht.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmccclxiii{Das sind wahrlich die Worte eines edlen Gesandten,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmccclxiv{das sind nicht die Worte eines Dichters. Wie wenig ihr glaubt!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmccclxv{Und es sind auch nicht die Worte eines Wahrsagers. Wie wenig ihr bedenkt!\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmccclxvi{Es ist) eine Offenbarung vom Herrn der Weltenbewohner.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmccclxvii{Und wenn er sich gegen Uns einige Aussprüche selbst ausgedacht hätte,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmccclxviii{hätten Wir ihn sicherlich an der Rechten gefaßt\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmccclxix{und ihm hierauf sicherlich die Herzader durchschnitten,\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmccclxx{und niemand von euch hätte (Uns) dann von ihm abhalten können.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxi{Er ist wahrlich eine Erinnerung für die Gottesfürchtigen.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxii{Und Wir wissen wahrlich, daß es unter euch Leugner gibt.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxiii{Und er ist wahrlich (ein Grund zum) Gram für die Ungläubigen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxiv{Und er ist wahrlich die reine Gewißheit.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxv{Darum preise den Namen deines Allgewaltigen Herrn.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxvi{\basmalahde Es fragt ein Fragesteller nach einer Strafe, die hereinbrechen wird\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxvii{für die Ungläubigen, (einer Strafe,) die niemand abwehren kann;\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxviii{von Allah, dem Besitzer der Aufstiegswege.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxix{Es steigen die Engel und der Geist zu Ihm auf an einem Tag, dessen (Aus)maß fünfzigtausend Jahre ist.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxx{Darum sei standhaft in schöner Geduld.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxi{Gewiß, sie sehen sie weit entfernt,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxii{Wir aber sehen sie nahe.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxiii{Am Tag, da der Himmel wie siedendes Öl\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxiv{und die Berge wie gefärbte Wolle sein werden\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxv{und kein warmherziger Freund seinen Freund (irgend etwas) fragt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxvi{obwohl sie ihrem Anblick ausgesetzt sein werden. Der Übeltäter hätte es gern, wenn er sich von der Strafe jenes Tages loskaufte mit seinen Söhnen\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxvii{und seiner Gefährtin und seinem Bruder\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxviii{und seiner Familie, die ihn aufgenommen hat,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxix{und allen, die auf der Erde sind. (Er wünscht,) dies möge ihn hierauf retten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcccxc{Keineswegs! Es ist doch ein loderndes Feuer,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcccxci{das die Kopfhaut abzieht',\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcii{das (den) rufen wird, wer den Rücken kehrt und sich abkehrt,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcccxciii{Besitz) zusammenträgt und dann in Behältern hortet.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcccxciv{Gewiß, der Mensch ist als kleinmütig erschaffen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcv{Wenn ihm Schlechtes widerfährt, ist er sehr mutlos;\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcvi{und wenn ihm Gutes widerfährt, (ist er) ein stetiger Verweigerer,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcvii{außer den Betenden,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcviii{diejenigen, die in ihrem Gebet beharrlich sind\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcix{und die ein festgesetztes Recht an ihrem Besitz (zugestehen)\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcd{dem Bettler und dem Unbemittelten\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcdi{und die den Tag des Gerichts für wahr halten\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcdii{und diejenigen, die wegen der Strafe ihres Herrn besorgt sind,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcdiii{- denn gewiß, vor der Strafe ihres Herrn (kann) sich niemand sicher glauben,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcdiv{- und diejenigen, die ihre Scham hüten,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcdv{außer gegenüber ihren Gattinnen oder was ihre rechte Hand (an Sklavinnen) besitzt, denn sie sind (hierin) nicht zu tadeln,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcdvi{- wer aber darüber hinaus (etwas) begehrt, das sind die Übertreter,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmcdvii{- und diejenigen, die auf die ihnen anvertrauten Güter und ihre Verpflichtung achtgeben,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmcdviii{und diejenigen, die ihr Zeugnis in Aufrichtigkeit ablegen,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmcdix{und diejenigen, die ihr Gebet einhalten.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmcdx{Jene werden sich in Gärten befinden und (darin) geehrt.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmcdxi{Was ist mit denjenigen, die ungläubig sind, daß sie hastig auf dich zukommen\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmcdxii{rechts und links, in Gruppen aufgeteilt?\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmcdxiii{Begehrt (etwa) jedermann von ihnen, in einen Garten der Wonne eingelassen zu werden?\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmcdxiv{Keineswegs! Wir haben sie doch aus dem erschaffen, was sie wissen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmcdxv{Nein! Ich schwöre beim Herrn der Osten und der Westen, Wir haben fürwahr die Macht dazu,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmcdxvi{daß Wir sie durch bessere als sie austauschen. Und niemand kann Uns zuvorkommen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmcdxvii{So lasse sie nur schweifende Gespräche führen und ihr Spiel treiben, bis sie ihrem Tag begegnen, der ihnen angedroht ist,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmcdxviii{dem Tag, da sie aus den Gräbern eilig herauskommen werden, als würden sie hastig zu einem aufgerichteten Opferstein laufen,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmcdxix{mit demütigen Blicken, bedeckt mit Erniedrigung. Das ist der Tag, der ihnen immer wieder angedroht wurde.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmcdxx{\basmalahde Wir sandten ja Nuh zu seinem Volk: ``Warne dein Volk, bevor schmerzhafte Strafe über sie kommt."\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxi{Er sagte: ``O mein Volk, ich bin euch ja ein deutlicher Warner,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxii{euch zu mahnen:) Dient Allah und fürchtet Ihn, und gehorcht mir,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxiii{so wird Er euch (etwas) von euren Sünden vergeben und euch auf eine festgesetzte Frist zurückstellen. Gewiß, Allahs Frist, wenn sie kommt, kann nicht zurückgestellt werden, wenn ihr nur wüßtet!"\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxiv{Er sagte: ``Mein Herr, ich habe mein Volk bei Nacht und bei Tag aufgerufen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxv{Aber mein Rufen hat sie nur in ihrer Flucht bestärkt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxvi{Gewiß, jedesmal, wenn ich sie aufrief, damit Du ihnen vergibst, steckten sie ihre Finger in ihre Ohren, überdeckten sich mit ihren Gewändern, verharrten (im Irrtum) und verhielten sich sehr hochmütig.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxvii{Hierauf richtete ich an sie den Ruf dann öffentlich.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxviii{Hierauf sprach ich zu ihnen offen und ganz im geheimen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxix{Ich sagte: .Bittet euren Herrn um Vergebung - Er ist ja Allvergebend -,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxx{so wird Er den Regen auf euch ergiebig (herab)senden\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxi{und euch mit Besitz und Söhnen unterstützen und für euch Gärten machen und für euch Bäche machen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxii{Was ist mit euch, daß ihr nicht glaubt, von Allah würdevoll behandelt zu werden,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxiii{wo Er euch doch in Entwicklungsabschnitten erschaffen hat?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxiv{Seht ihr nicht, wie Allah sieben Himmel in Schichten (übereinander) erschaffen hat,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxv{den Mond darin zu einem Licht gemacht und die Sonne zu einem Leuchtkörper gemacht hat?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxvi{Und Allah hat euch aus der Erde heranwachsen lassen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxvii{Hierauf wird Er euch in sie zurückbringen und gewiß (wieder aus ihr) hervorbringen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxviii{Und Allah hat euch die Erde zu einer ausgebreiteten Unterlage gemacht,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxix{damit ihr auf ihr dahinziehen könnt - auf Wegen als breiten Durchgängen'."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcdxl{Nuh sagte: ``Mein Herr, sie haben sich mir widersetzt und sind jemandem gefolgt, dem sein Besitz und seine Kinder nur den Verlust mehren.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcdxli{Und sie haben ungeheure Ränke geschmiedet.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlii{Und sie haben gesagt: .Verlaßt doch nicht eure Götter; verlaßt doch nicht Wadd, noch Suwa", noch Yagut,Ya'uq und Nasr.'\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcdxliii{Sie haben ja viele in die Irre geführt. So lasse die Ungerechten nur noch tiefer in die Irre gehen."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcdxliv{Wegen ihrer Verfehlungen wurden sie ertränkt und dann in ein (Höllen)feuer hineingebracht. Und da fanden sie für sich außer Allah keine Helfer.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlv{Und Nuh sagte: ``Mein Herr, lasse auf der Erde von den Ungläubigen nicht einen einzigen Bewohner.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlvi{Denn, gewiß, wenn Du sie (leben) läßt, führen sie Deine Diener in die Irre und zeugen nur Sittenlose und sehr beharrliche Ungläubige.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlvii{Mein Herr, vergib mir und meinen Eltern und demjenigen, der als Gläubiger mein Haus betritt, und den gläubigen Männern und den gläubigen Frauen. Und stürze die Ungerechten nur noch tiefer ins Verderben."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlviii{\basmalahde Sag: Mir ist (als Offenbarung) eingegeben worden, daß eine kleinere Schar Ginn zuhörte. Sie sagten: ``Wir haben einen wunderbaren Qur'an gehört,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlix{der zur Besonnenheit leitet; so haben wir an ihn geglaubt, und wir werden unserem Herrn niemanden beigesellen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdl{Und erhaben ist die Größe unseres Herrn. Er hat sich weder eine Gefährtin noch Kinder genommen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdli{Und es pflegte der Tor unter uns etwas Abwegiges gegen Allah zu sagen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcdlii{Und wir meinten, daß weder die Menschen noch die Ginn gegen Allah jemals eine Lüge sagen würden.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcdliii{Und (manche) Männer von den Menschen pflegten Zuflucht zu nehmen bei (einigen) Männern von den Ginn, doch mehrten sie so (bei) ihnen die Drangsal.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcdliv{Und sie meinten -, wie (auch) ihr meint -, daß Allah niemanden mehr (als Gesandten) schicken würde.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcdlv{Und wir haben den Himmel abgetastet, aber festgestellt, daß er mit strengen Wächtern und Leuchtkörpern gefüllt worden ist.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcdlvi{Und wir pflegten ja an Stellen von ihm Sitze zum Abhören einzunehmen; wer aber jetzt abhört, der findet einen auf ihn wartenden Leuchtkörper.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcdlvii{Und wir wissen doch nicht, ob für diejenigen auf der Erde Schlechtes gewollt wird oder ob ihr Herr sie zur Besonnenheit (führen) will.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcdlviii{Und unter uns gibt es Rechtschaffene, und unter uns gibt es solche, die weniger rechtschaffen sind; wir sind getrennte(n) Wege(n gefolgt).\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcdlix{Und wir meinten, daß wir uns auf der Erde Allah niemals entziehen können und daß wir uns Ihm niemals durch Flucht entziehen können.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcdlx{Und als wir die Rechtleitung hörten, glaubten wir daran. Wer an seinen Herrn glaubt, der befürchtet keine (Lohn)schmälerung und keine Drangsal.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxi{Und unter uns sind Gottergebene; und unter uns sind (vom rechten Weg) Abschweifende. Was nun jemanden angeht, der sich (Allah) ergeben hat, so bemüht er sich um (mehr) Besonnenheit.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxii{Was aber die (vom rechten Weg) Abschweifenden angeht, so werden sie Brennholz für die Hölle sein.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxiii{Und wenn sie sich auf dem Weg recht verhielten, würden Wir ihnen wahrlich Wasser im Überfluß zu trinken geben,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxiv{damit Wir sie dadurch der Versuchung aussetzen. Und wer sich von der Ermahnung seines Herrn abwendet, den wird Er in überwältigende Strafe stecken.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxv{Und die Gebetsstätten gehören doch Allah; so ruft neben Allah niemanden an.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxvi{Und als Allahs Diener aufstand, um Ihn anzurufen, hätten sie ihn (alle) beinahe erdrückt."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxvii{Sag: Ich rufe nur meinen Herrn an, und ich geselle Ihm niemanden bei.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxviii{Sag: Ich vermag euch weder Schaden noch Besonnenheit (zu bringen).\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxix{Sag: Niemand kann mir gegen Allah Schutz gewähren, und ich werde außer (bei) Ihm keinen Ort der Sicherheit finden.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxx{Ich vermag) nur etwas zu übermitteln von Allah und Seinen Botschaften. Und wer sich Allah und Seinem Gesandten widersetzt, gewiß, für ihn wird es das Feuer der Hölle geben, ewig und auf immer darin zu bleiben.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxi{Wenn sie dann sehen, was ihnen versprochen worden ist, da werden sie wissen, wer die schwächeren Helfer und die geringere Zahl hat.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxii{Sag: Ich weiß nicht, ob das, was euch versprochen worden ist, nahe ist, oder ob mein Herr ihm eine (längere) Frist setzt,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxiii{Er,) der Kenner des Verborgenen - Er enthüllt niemandem das, was bei Ihm verborgen ist,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxiv{außer dem Gesandten, den Er bewilligt; da läßt Er vor ihm und hinter ihm Wächter einhergehen,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxv{damit er weiß, ob sie wohl die Botschaften ihres Herrn ausgerichtet haben, und daß Er das, was bei ihnen ist, umfaßt, und Er die Zahl von allem erfaßt'.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxvi{\basmalahde O du Eingehüllter,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxvii{steh (zum Gebet) die (ganze) Nacht auf- bis auf einen kleinen Teil,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxviii{ihre Hälfte, oder verringere sie um einen kleinen Teil\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxix{oder füge etwas hinzu. Und trage den Qur'an wohlgeordnet vor.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxx{Gewiß, Wir werden dir gewichtige Worte offenbaren.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxi{Das Aufstehen in der Nacht hat stärkeren Einfluß (auf die Seele) und bringt eher aufklärende Worte.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxii{Du hast ja am Tage lange Zeit für Beschäftigung.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxiii{Und gedenke des Namens deines Herrn und widme dich Ihm ganz allein.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxiv{Er ist) der Herr des Ostens und des Westens. Es gibt keinen Gott außer Ihm. So nimm Ihn dir zum Sachwalter.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxv{Und ertrage standhaft, was sie sagen, und meide sie auf schöne Weise.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxvi{Und lasse Mich (allein) mit den Leugnern (der Botschaft), die ein angenehmes Leben haben, und lasse ihnen noch kurze Zeit.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxvii{Gewiß, bei Uns gibt es schwere Fesseln und einen Höllenbrand\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxviii{und Speise, die im Hals Würgen hervorruft, und schmerzhafte Strafe\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxix{am Tag, da die Erde und die Berge zittern und die Berge ein zerrinnender Sandhügel sein werden.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcdxc{Wir haben zu euch ja einen Gesandten als Zeugen über euch gesandt, so wie Wir zu Fir'aun einen Gesandten sandten.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcdxci{Es widersetzte sich Fir'aun dem Gesandten. Da ergriffen Wir ihn mit hartem Griff.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcii{Wie wollt ihr euch denn, wenn ihr ungläubig seid, vor einem Tag bewahren, der die Kinder weißhaarig (wie Greise) macht?\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcdxciii{Der Himmel wird an ihm zerbrechen. Sein Versprechen wird ausgeführt.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcdxciv{Gewiß, dies ist eine Erinnerung; wer nun will, (der) nimmt (so diesen) einen Weg zu seinem Herrn.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcv{Gewiß, dein Herr weiß, daß du etwas weniger als zwei Drittel der Nacht (zum Gebet) aufstehst, oder die Hälfte oder ein Drittel davon, und (ebenso) ein Teil von denjenigen, die mit dir sind. Und Allah setzt das Maß der Nacht und des Tages fest. Er weiß, daß ihr es nicht erfassen würdet'. Da wandte Er Sich euch zu und erließ es euch. So lest (bei Nacht), was euch vom Qur'an leichtfällt. Er weiß, daß es unter euch Kranke geben wird und andere, die im Land umherreisen, wo sie nach (etwas) von Allahs Huld trachten, und (wieder) andere, die auf Allahs Weg kämpfen. So lest davon, was euch leichtfällt, und verrichtet das Gebet und entrichtet die Abgabe und gebt Allah ein gutes Darlehen. Und was ihr für euch selbst an Gutem vorausschickt, das werdet ihr noch besser und großartiger belohnt bei Allah finden. Und bittet Allah um Vergebung. Gewiß, Allah ist Allvergebend und Barmherzig.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcvi{\basmalahde O du Zugedeckter,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcvii{stehe auf und warne;\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcviii{und deinen Herrn, Den preise als den Größten,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcix{und deine Gewänder, die reinige,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmd{und die (Unreinheit des) Götzen(dienstes), die meide,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdi{und halte nicht (deinen Verdienst) vor, und halte ihn nicht für zu groß,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdii{und ertrage standhaft (alles) für deinen Herrn.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdiii{Wenn dann ins Horn gestoßen wird,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdiv{so wird jener Tag ein schwerer Tag sein\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdv{für die Ungläubigen nicht leicht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdvi{Lasse Mich (allein) mit wem Ich allein erschaffen habe,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdvii{und dem Ich ausgedehnten Besitz gegeben habe\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdviii{und Söhne (als ständige Begleiter) anwesend\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdix{und dem Ich alles schön zurechtgemacht habe;\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdx{doch begehrt er hierauf, daß Ich (ihm) noch mehr gebe.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdxi{Keineswegs! Er ist gegen Unsere Zeichen wiederholt widerspenstig gewesen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdxii{Ich werde ihn mit Beschwerlichem bedrücken.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdxiii{Er hat ja nachgedacht und abgewogen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdxiv{Tod ihm, wie er abgewogen hat!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdxv{Abermals: Tod ihm, wie er abgewogen hat!\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdxvi{Hierauf hat er geschaut,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdxvii{hierauf hat er düster geblickt und ein finsteres Gesicht gemacht,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdxviii{hierauf hat er den Rücken gekehrt und ist hochmütig geblieben.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdxix{Er sagt: ``Das ist nur Zauberei, die überliefert wird.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdxx{Das sind nur die Worte von Menschenwesen."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdxxi{Ich werde ihn der Saqar aussetzen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdxxii{Und was läßt dich wissen, was Saqar ist?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdxxiii{Sie läßt nichts übrig und läßt nichts bestehen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdxxiv{Sie verändert die Menschen ganz.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdxxv{Über ihr gibt es neunzehn (Wächter).\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdxxvi{Wir haben als Wächter des (Höllen)feuers nur Engel eingesetzt, und Wir haben ihre Zahl nur zu einer Versuchung gemacht für diejenigen, die ungläubig sind, damit diejenigen Überzeugung gewinnen, denen die Schrift gegeben wurde, und damit diejenigen, die glauben, an Glauben zunehmen, und damit diejenigen, denen die Schrift gegeben wurde, und (auch) die Gläubigen nicht zweifeln und damit diejenigen, in deren Herzen Krankheit ist, und (auch) die Ungläubigen sagen: ``Was will denn Allah damit als Gleichnis?" So läßt Allah in die Irre gehen, wen Er will, und leitet recht, wen Er will. Aber niemand weiß über die Heerscharen deines Herrn Bescheid außer Ihm. Und es ist nur eine Ermahnung für die Menschenwesen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdxxvii{Keineswegs! Beim Mond\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdxxviii{und (bei) der Nacht, wenn sie den Rücken kehrt,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdxxix{und (bei) dem Morgen, wenn er erstrahlt,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdxxx{sie ist wahrlich eine der größten (Heimsuchungen)\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxi{zur Warnung für die Menschenwesen,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxii{für jemanden von euch, der vorankommen oder zurückbleiben will.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxiii{Jede Seele haftet für das, was sie erworben hat,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxiv{außerdem die Gefährten von der rechten Seite;\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxv{sie werden sich in Gärten befinden, und sie werden einander fragen\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxvi{nach den Übeltätern:\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxvii{``Was hat euch in Saqar geführt?"\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxviii{Sie werden sagen: ``Wir gehörten nicht zu denjenigen, die beteten,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxix{und wir pflegten nicht den Armen zu speisen,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmdxl{und wir pflegten auf schweifende Reden mit denjenigen einzugehen, die solche führten,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmdxli{und wir erklärten stets den Tag des Gerichts für Lüge\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmdxlii{bis die Gewißheit zu uns kam."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmdxliii{Nun nützt ihnen die Fürsprache derjenigen nicht, die Fürsprache einlegen (können).\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmdxliv{Was ist denn mit ihnen, daß sie sich von der Ermahnung abwenden,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmdxlv{als wären sie aufgeschreckte Wildesel,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmdxlvi{die vor einem Löwen fliehen?\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmdxlvii{Aber nein! Jedermann von ihnen will, daß ihm aufgeschlagene Blätter zukommen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmdxlviii{Keineswegs! Vielmehr fürchten sie nicht das Jenseits.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmmdxlix{Keineswegs! Er ist ja eine Erinnerung.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmmdl{Wer nun will, gedenkt seiner.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmmdli{Sie werden aber (seiner) nicht gedenken, außer daß Allah es will. Ihm gebührt die Gottesfurcht und Ihm gebührt die Vergebung.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmmdlii{\basmalahde Nein! Ich schwöre beim Tag der Auferstehung.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdliii{Nein! Ich schwöre bei der Seele, die sich selbst tadelt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdliv{Meint der Mensch (etwa), daß Wir seine Knochen nicht zusammenfügen werden?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdlv{Ja doch! (Wir) haben die Macht dazu, seine Fingerspitzen zurechtzuformen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdlvi{Aber nein! Der Mensch will (fortdauernd) vor sich hin lasterhaft sein.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdlvii{Er fragt: ``Wann wird bloß der Tag der Auferstehung sein?"\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdlviii{Wenn dann der Blick verwirrt ist\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdlix{und der Mond sich verfinstert\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdlx{und Sonne und Mond zusammengebracht werden,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdlxi{wird der Mensch an jenem Tag sagen: ``Wohin soll die Flucht sein?"\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdlxii{Keineswegs! (Es gibt) keine Zuflucht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdlxiii{Zu deinem Herrn wird an jenem Tag der Weg zum Aufenthaltsort (führen).\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdlxiv{Dem Menschen wird an jenem Tag kundgetan, was er vorausgeschickt und zurückgestellt hat.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdlxv{Aber nein! Der Mensch wird bezüglich seiner selbst durchblicken,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdlxvi{auch wenn er seine Entschuldigungen vorbrächte .\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdlxvii{Bewege deine Zunge nicht damit, um ihn übereilt weiterzugeben.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdlxviii{Uns obliegt es, ihn zusammenzustellen und ihn vorlesen zu lassen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdlxix{Und (erst) wenn Wir ihn vorgelesen haben, dann folge du der Art seines Vortragens".\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdlxx{Hierauf obliegt es Uns, ihn klar darzulegen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxi{Keineswegs! Vielmehr liebt ihr das schnell Eintreffende,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxii{und laßt das Jenseits (außer acht).\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxiii{Die einen) Gesichter werden an jenem Tag strahlen,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxiv{zu ihrem Herrn schauen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxv{Und (die anderen) Gesichter werden an jenem Tag finster sein;\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxvi{die meinen, es würde ihnen das Rückgrat gebrochen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxvii{Keineswegs! Wenn sie das Schlüsselbein erreicht\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxviii{und gesagt wird: ``Wer ist ein Zauberer (, der ihn retten kann)?",\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxix{und wenn er meint, daß es (jetzt) die Trennung sei,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxx{und wenn (bei ihm) sich das eine Bein um das andere legt,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxi{zu deinem Herrn wird an jenem Tag das Treiben sein.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxii{Doch hielt er nicht (die Botschaft) für wahr noch betete er,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxiii{sondern er erklärte (sie) für Lüge, und er kehrte sich ab.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxiv{Hierauf ging er zu seinen Angehörigen, stolzierend.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxv{- ``Wehe dir, ja wehe!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxvi{Abermals: Wehe dir, ja wehe!"\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxvii{Meint der Mensch (etwa), daß er außer acht gelassen wird?\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxviii{Ist er nicht ein Tropfen von Samenflüssigkeit, die ausgespritzt wird,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxix{hierauf ein Anhängsel gewesen? Da hat Er erschaffen und zurechtgeformt\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdxc{und dann daraus beide Teile des Paares gemacht, das Männliche und das Weibliche.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdxci{Hat dieser (Gott) nicht die Macht, die Toten wieder lebendig zu machen?\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdxcii{\basmalahde Ist (nicht) über den Menschen eine Spanne der endlosen Zeit gekommen, in der er nichts Nennenswertes ist?\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdxciii{Wir haben den Menschen ja aus einem Samentropfen, einem Gemisch erschaffen, (um) ihn zu prüfen. Und so haben Wir ihn mit Gehör und Augenlicht versehen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdxciv{Wir haben ihn ja den (rechten) Weg geleitet, ob er nun dankbar oder undankbar sein mag.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdxcv{Wir haben ja für die Ungläubigen Ketten und Fesseln und eine Feuerglut bereitet.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdxcvi{Gewiß, die Frommen trinken aus einem Becher, dessen Beimischung Kampfer ist,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdxcvii{aus einer Quelle, aus der Allahs Diener trinken, die sie sprudelnd hervorströmen lassen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdxcviii{Sie erfüllen das Gelübde und fürchten einen Tag, dessen Übel sich wie im Flug ausbreitet,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdxcix{und sie geben - obwohl man sie liebt - Speise zu essen einem Armen, einer Waisen und einem Gefangenen:\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdc{``Wir speisen euch nur um Allahs Angesicht willen. Wir wollen von euch weder Belohnung noch Dank.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdci{Wir fürchten ja von unserem Herrn einen Tag, der düster blicken läßt, einen unheilvollen."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcii{So bewahrt sie Allah vor dem Übel jenes Tages und wird ihnen strahlendes Glück und Freude darbieten\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdciii{und ihnen vergelten, daß sie standhaft gewesen sind, mit einem Garten und mit Seide,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdciv{worin sie sich auf überdachten Liegen lehnen, und worin sie weder (heiße) Sonne noch bittere Kälte wahrnehmen werden.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcv{Nahe über ihnen sind seine Schatten, und seine Früchte sind ihnen (zum Pflücken) sehr gefügig gemacht.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcvi{Herumgereicht werden ihnen Gefäße aus Silber und Trinkschalen, die (wie) aus Glas sind,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcvii{Gläser aus Silber, die sie im rechten Maß bemessen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcviii{Und zu trinken gegeben wird ihnen darin ein Becher, dessen Beimischung Ingwer ist,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcix{aus einer Quelle darin, die Salsabil genannt wird.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcx{Und unter ihnen gehen ewig junge Knaben umher. Wenn du sie siehst, hältst du sie für ausgestreute Perlen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcxi{Und wenn du dort hinsiehst, wirst du Wonne und ein großes Reich sehen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcxii{Auf ihnen werden grüne Gewänder aus Seidenbrokat sein und schwerer Brokat, und geschmückt werden sie sein mit Armreifen aus Silber. Und zu trinken wird ihr Herr ihnen ein reines" Getränk geben.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcxiii{``Dies ist ja der Lohn für euch, und euer Bemühen wird gedankt sein."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcxiv{Gewiß, Wir sind es, die Wir dir den Qur'an offenbart, ja offenbart haben.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcxv{So sei standhaft in Bezug auf das Urteil deines Herrn, und gehorche von ihnen keinem Sündhaften oder sehr Undankbaren.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcxvi{Und gedenke des Namens deines Herrn morgens und abends,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcxvii{und in (einem Teil) der Nacht, da wirf dich vor Ihm nieder und preise Ihn lange Zeit zur Nacht.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcxviii{Diese da lieben ja das schnell Eintreffende und lassen hinter sich einen schwer lastenden Tag.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcxix{Wir doch haben sie erschaffen und ihren Körperbau vollendet. Und wenn Wir wollen, tauschen Wir sie durch andere ihresgleichen aus.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcxx{Gewiß, dies ist eine Erinnerung; wer nun will, nimmt (so diesen) einen Weg zu seinem Herrn.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxi{Und ihr könnt nicht(s) wollen, außer daß Allah (es) will. Gewiß, Allah ist All wissend und All weise.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxii{Er läßt, wen Er will, in Seine Barmherzigkeit eingehen. Und die Ungerechten, für sie hat Er schmerzhafte Strafe bereitet.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxiii{\basmalahde Bei den Entsandten, die wie eine Mähne aufeinanderfolgen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxiv{den einen Sturm Entfesselnden\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxv{und den alles Ausbreitenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxvi{den klar Trennenden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxvii{den Ermahnung Überbringenden\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxviii{zur Pflichterfüllung oder zur Warnung!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxix{Gewiß, was euch versprochen wird, wird sicher hereinbrechen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxx{Wenn dann die Sterne ausgelöscht werden\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxi{und wenn der Himmel gespalten wird\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxii{und wenn die Berge zersprengt werden\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxiii{und wenn für die Gesandten ihre Zeit gesetzt wird.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxiv{Aufweichen Tag ist ihre Frist festgelegt worden?\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxv{Auf den Tag der Entscheidung.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxvi{Und was läßt dich wissen, was der Tag der Entscheidung ist?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxvii{Wehe an jenem Tag den Leugnern!\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxviii{Haben Wir nicht die Früheren vernichtet?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxix{Hierauf lassen Wir ihnen die Späteren folgen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcxl{So verfahren Wir mit den Übeltätern.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcxli{Wehe an jenem Tag den Leugnern!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlii{Haben Wir euch nicht aus verächtlichem Wasser erschaffen,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcxliii{das Wir dann in einem festen Aufenthaltsort haben sein lassen,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcxliv{bis zu einem bekannten Zeitpunkt?\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlv{So haben Wir bemessen. Welch trefflicher Bemesser sind Wir!\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlvi{Wehe an jenem Tag den Leugnern!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlvii{Haben Wir nicht die Erde zu einem Ort der Aufbewahrung gemacht\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlviii{- für die Lebenden und die Toten -\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlix{und auf ihr festgegründete, hoch aufragende Berge gemacht und euch frisches Wasser zu trinken gegeben?\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcl{Wehe an jenem Tag den Leugnern!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcli{- ``Zieht los zu dem, was ihr stets für Lüge erklärt habt,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdclii{zieht los zu einem Schatten mit drei Verzweigungen,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcliii{der kein (wirklicher) Schattenspender ist und nicht (als Schutz) gegen die Flammen nützt."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdcliv{Sie wirft mit Funken wie Schlösser,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdclv{als wären sie gelbe Kamele.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdclvi{Wehe an jenem Tag den Leugnern!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdclvii{Das ist der Tag, an dem sie nicht reden werden\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdclviii{und (es) ihnen nicht erlaubt wird, daß sie sich entschuldigen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdclix{Wehe an jenem Tag den Leugnern!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdclx{- ``Das ist der Tag der Entscheidung; Wir haben euch mit den Früheren versammelt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdclxi{Wenn ihr also eine List habt, so führt sie gegen Mich aus."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdclxii{Wehe an jenem Tag den Leugnern!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdclxiii{Gewiß, die Gottesfürchtigen werden sich in Schatten und an Quellen befinden\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdclxiv{und bei Früchten von dem, was sie begehren.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdclxv{- ``Eßt und trinkt als wohlbekömmlich für das, was ihr zu tun pflegtet."\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmdclxvi{Gewiß, so vergelten Wir den Rechtschaffenen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmdclxvii{Wehe an jenem Tag den Leugnern!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmdclxviii{- ``Eßt und genießt ein wenig, ihr seid ja Übeltäter."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmdclxix{Wehe an jenem Tag den Leugnern!\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmdclxx{Und wenn zu ihnen gesagt wird: ``Verbeugt euch!", verbeugen sie sich nicht.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxi{Wehe an jenem Tag den Leugnern!\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxii{An welche Aussage nach dieser wollen sie denn glauben?\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxiii{\basmalahde Wonach fragen sie sich?\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxiv{Nach der gewaltigen Kunde,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxv{über die sie sich uneinig sind.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxvi{Keineswegs! Sie werden (es noch) erfahren.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxvii{Abermals: Keineswegs! Sie werden (es noch) erfahren.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxviii{Haben Wir nicht die Erde zu einer Lagerstatt gemacht\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxix{und die Berge zu Pfählen?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxx{Und Wir haben euch als Paare erschaffen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxi{Und Wir haben euren Schlaf zum Ausruhen gemacht.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxii{Und Wir haben die Nacht zum Kleid gemacht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxiii{Und Wir haben den Tag zur Lebensführung gemacht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxiv{Und Wir haben über euch sieben feste (Himmel) aufgebaut.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxv{Und Wir haben einen hell glühenden Leuchtkörper gemacht.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxvi{Und Wir haben von den Regenwolken sich ergießendes Wasser herabkommen lassen,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxvii{um damit Korn und Gewächse hervorzubringen\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxviii{und dichtbestandene Gärten.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxix{Gewiß, der Tag der Entscheidung ist eine festgesetzte Zeit,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcxc{der Tag, da ins Horn geblasen wird und ihr in Scharen herkommt\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcxci{und da der Himmel geöffnet und dann zu Toren wird\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcii{und da die Berge versetzt und dann zu einer Luftspiegelung werden.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcxciii{Gewiß, die Hölle ist ein Hinterhalt,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcxciv{für diejenigen, die das Maß (an Frevel) überschreiten, eine Heimstatt,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcv{lange Zeiten darin zu verweilen;\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcvi{sie werden darin weder Kühlung noch Getränk kosten,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcvii{außer heißem Wasser und stinkender Brühe,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcviii{als angemessene Vergeltung.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcix{Sie erwarteten ja niemals eine Abrechnung\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcc{und erklärten Unsere Zeichen hartnäckig für Lüge.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcci{Doch alles haben Wir schriftlich erfaßt.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdccii{So kostet; Wir werden euch nur die Strafe mehren.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcciii{Gewiß, für die Gottesfürchtigen wird es einen Ort des Erfolgs geben,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdcciv{umfriedete Gärten und Rebstöcke\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccv{und prächtige, gleichaltrige (weibliche Wesen)\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccvi{und ein (stets) voller Becher.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccvii{Weder hören sie darin unbedachte Rede noch Lüge.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccviii{Dies) als Belohnung von deinem Herrn, als hinreichende Gabe,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccix{dem Herrn der Himmel und der Erde und dessen, was dazwischen ist, dem Allerbarmer. Sie verfügen nicht darüber, vor Ihm zu reden,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdccx{am Tag, da der Geist und die Engel in Reihe stehen. Sie werden nicht sprechen, außer wem der Allerbarmer es erlaubt und wer Richtiges sagt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdccxi{Das ist der wahrhaftig (eintreffend)e Tag. Wer nun will, nimmt zu seinem Herrn eine Heimkehr.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdccxii{Wir warnen euch ja vor naher Strafe, am Tag, da der Mensch schauen wird, was seine Hände vorausgeschickt haben, und der Ungläubige sagen wird: ``O wäre ich doch Erde!"\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdccxiii{\basmalahde Bei den mit Heftigkeit Entreißenden\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdccxiv{und den leicht Herausziehenden\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdccxv{und den unbeschwert Dahingleitenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccxvi{den allem Vorauseilenden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdccxvii{den eine Angelegenheit Regelnden!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdccxviii{Am Tag, da das Zittern einsetzt\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdccxix{und das nächste hinterherfolgt,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdccxx{gewisse) Herzen werden an jenem Tag beunruhigt klopfen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxi{und ihre Blicke werden demütig sein.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxii{Sie sagen: ``Sollen wir denn wirklich aus den Gräbern zurückgebracht werden?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxiii{Wenn wir zu verrotteten Knochen geworden sind?"\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxiv{Sie sagen: ``Das wäre dann eine verlustreiche Wiederkehr."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxv{Es wird nur ein einziger erschreckender Schrei sein,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxvi{und sogleich sind sie auf der Oberfläche.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxvii{Ist zu dir die Geschichte Musas gekommen?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxviii{Als sein Herr ihn im geheiligten Tal Tuwa rief:\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxix{``Geh zu Fir'aun, denn gewiß, er überschreitet das Maß (an Frevel).\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxx{Dann sag: Hast du (nicht) den Wunsch, dich zu läutern,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxi{und daß ich dich zu deinem Herrn rechtleite, so daß du gottesfürchtig wirst?"\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxii{Da zeigte er ihm das größte Zeichen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxiii{Er aber erklärte (es) für Lüge und widersetzte sich.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxiv{Hierauf kehrte er den Rücken und ging eilig weg'.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxv{Da versammelte er (das Volk) und rief dann aus.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxvi{Er sagte: ``Ich bin euer höchster Herr."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxvii{Da ergriff Allah ihn als warnendes Beispiel für das Jenseits und das Diesseits.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxviii{Darin ist wahrlich eine Lehre für jemanden, der gottesfürchtig ist.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxix{Seid etwa ihr schwerer zu erschaffen oder der Himmel? - Er hat ihn aufgebaut.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdccxl{Er hat seinen höchsten Teil emporgehoben und ihn dann zurechtgeformt.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdccxli{Und Er hat seine Nacht finster gemacht und seine Morgenhelle hervorkommen lassen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlii{Und die Erde, Er hat sie danach hingebreitet.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdccxliii{Er hat aus ihr ihr Wasser und ihre Weide hervorkommen lassen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdccxliv{Und die Berge, Er hat sie fest gegründet.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlv{Dies) als Nießbrauch für euch und für euer Vieh.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlvi{Wenn dann der größte, überwältigende Umsturz kommt,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlvii{am Tag, da der Mensch das bedenkt, worum er sich bemüht hat,\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlviii{und zum Erscheinen gebracht wird der Höllenbrand für (je)den, der sieht;\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlix{was den (denjenigen) angeht, wer das Maß (an Frevel) überschritten\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdccl{und das diesseitige Leben vorgezogen hat,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdccli{gewiß, so wird der Höllenbrand (ihm) Zufluchtsort sein.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdcclii{Was aber jemanden angeht, der den Stand seines Herrn gefürchtet und seiner Seele die (bösen) Neigungen untersagt hat,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdccliii{so wird der (Paradies)garten (ihm) Zufluchtsort sein.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdccliv{Sie fragen dich nach der Stunde, wann sie bloß feststehen wird.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdcclv{Was hast du über sie zu erwähnen?\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmdcclvi{Zu deinem Herrn ist ihr Endziel.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmdcclvii{Du bist nur ein Überbringer von Warnungen für jemanden, der sie fürchtet.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmdcclviii{Am Tag, da sie sie sehen, wird ihnen sein, als hätten sie nur einen Nachmittag verweilt oder seinen (dazugehörigen) Vormittag\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmdcclix{\basmalahde Er blickte düster und kehrte sich ab,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcclx{weil der Blinde zu ihm kam.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxi{Was läßt dich wissen, vielleicht läutert er sich\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxii{oder bedenkt, so daß ihm die Ermahnung nützt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxiii{Was nun jemanden angeht, der sich für unbedürftig hält,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxiv{so widmest du dich ihm,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxv{obgleich es dich nicht zu kümmern hat, daß er sich nicht läutern will.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxvi{Was aber jemanden angeht, der zu dir geeilt kommt\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxvii{und dabei gottesfürchtig ist,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxviii{von dem läßt du dich ablenken.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxix{Keineswegs! Gewiß, es ist eine Erinnerung.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxx{Wer nun will, gedenkt seiner.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxi{Er steht) auf in Ehren gehaltenen Blättern,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxii{erhöhten und rein gehaltenen,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxiii{durch die Hände von Entsandten,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxiv{edlen, frommen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxv{Tod dem Menschen, wie undankbar er ist!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxvi{Woraus hat Er ihn erschaffen?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxvii{Aus einem Samentropfen hat Er ihn erschaffen und ihm dabei sein Maß festgesetzt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxviii{Den Weg hierauf macht Er ihm leicht.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxix{Hierauf läßt Er ihn sterben und bringt ihn dann ins Grab.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxx{Hierauf, wenn Er will, läßt Er ihn auferstehen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxi{Keineswegs! Er hat noch nicht ausgeführt, was Er ihm befohlen hat.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxii{So schaue der Mensch doch auf seine Nahrung:\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxiii{Wir gießen ja Güsse von Wasser,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxiv{hierauf spalten Wir die Erde in Spalten auf\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxv{und lassen dann auf ihr Korn wachsen\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxvi{und Rebstöcke und Grünzeug\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxvii{und Ölbäume und Palmen\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxviii{und Gärten mit dicken Bäumen\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxix{und Früchte und Futter,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdccxc{als Nießbrauch für euch und für euer Vieh.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccxci{Wenn dann der betäubende (Schrei) kommt,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcii{am Tag, da der Mensch flieht vor seinem Bruder\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccxciii{und seiner Mutter und seinem Vater\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccxciv{und seiner Gefährtin und seinen Söhnen\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcv{jedermann von ihnen wird an jenem Tag eine Angelegenheit haben, die ihn beschäftigt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcvi{Die einen) Gesichter werden an jenem Tag erstrahlen,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcvii{lachen und sich freuen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcviii{Und auf (den anderen) Gesichtern wird an jenem Tag Staub sein.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcix{und sie werden von Dunkelheit bedeckt sein.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdccc{Das sind die Ungläubigen und Sittenlosen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdccci{\basmalahde Wenn die Sonne umschlungen wird'\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcccii{und wenn die Sterne verstreut werden\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdccciii{und wenn die Berge versetzt werden\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccciv{und wenn die trächtigen Kamelstuten vernachlässigt werden\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcccv{und wenn die wilden Tiere versammelt werden\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccvi{und wenn die Meere zum Überfließen gebracht werden\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcccvii{und wenn die Seelen gepaart werden\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcccviii{und wenn das lebendig begrabene Mädchen gefragt wird,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcccix{wegen welcher Sünde es getötet wurde,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcccx{und wenn die Blätter aufgeschlagen werden\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxi{und wenn der Himmel abgezogen wird\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxii{und wenn die Hölle angefacht wird\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxiii{und wenn der (Paradies)garten nahe herangebracht wird,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxiv{wird eine (jede) Seele erfahren, was sie vorgebracht hat.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxv{Nein! Ich schwöre bei den sich Verbergenden,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxvi{den Dahinziehenden und sich wieder Zeigenden\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxvii{und bei der Nacht, wenn sie anbricht,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxviii{und bei dem Morgen, wenn er Atem schöpft:\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxix{Dies sind wahrlich die Worte eines edlen Gesandten,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxx{Besitzer von Kraft und beim Herrn des Thrones in Ansehen,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxi{dem man dort gehorcht und (der) vertrauenswürdig (ist).\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxii{Und euer Gefährte ist kein Besessener.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxiii{Er hat ihn ja am deutlichen Gesichtskreis gesehen,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxiv{und er hält nicht aus Geiz das Verborgene zurück.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxv{Und es sind nicht die Worte eines gesteinigten Satans.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxvi{Wo geht ihr denn hin?\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxvii{Es ist nur eine Ermahnung für die Weltenbewohner,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxviii{für jemanden von euch, der sich recht verhalten will.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxix{Und ihr könnt nicht wollen, außer daß Allah will, (Er), der Herr der Weltenbewohner.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxx{\basmalahde Wenn der Himmel zerbricht\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxi{und wenn die Sterne sich zerstreuen\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxii{und wenn die Meere gesprengt werden\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxiii{und wenn die Gräber durchwühlt werden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxiv{wird eine (jede) Seele erfahren, was sie vorausgeschickt und zurückgestellt hat.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxv{O Mensch, was hat dich hinsichtlich deines edelmütigen Herrn getäuscht,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxvi{Der dich erschaffen und da(bei) zurechtgeformt und wohlgebildet gemacht hat.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxvii{und dich, in welcher Gestalt Er wollte, zusammengefügt hat?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxviii{Keineswegs! Vielmehr erklärt ihr das Gericht für Lüge.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxix{Über euch sind wahrlich Hüter (eingesetzt),\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxl{edle, die (alles) aufschreiben\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxli{und die wissen, was ihr tut.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlii{Die Frommen werden wahrlich in Wonne sein.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxliii{Und die Sittenlosen werden wahrlich in einem Höllenbrand sein,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxliv{dem sie am Tag des Gerichts ausgesetzt werden\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlv{und dem sie nicht abwesend sein werden.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlvi{Und was läßt dich wissen, was der Tag des Gerichts ist?\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlvii{Abermals: Was läßt dich wissen, was der Tag des Gerichts ist?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlviii{Am Tag, da keine Seele für eine (andere) Seele etwas (auszurichten) vermag; und der Befehl wird an jenem Tag Allah (allein) zustehen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlix{\basmalahde Wehe den das Maß Kürzenden,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcccl{die, wenn sie sich von den Menschen zumessen lassen, sich volles Maß geben lassen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcccli{wenn sie ihnen aber zumessen oder wägen, Verlust zufügen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccclii{Glauben jene nicht, daß sie auferweckt werden\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcccliii{zu einem gewaltigen Tag,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccliv{am Tag, da die Menschen sich um des Herrn der Weltenbewohner willen aufstellen werden?\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdccclv{Keineswegs! Das Buch der Sittenlosen ist wahrlich in Siggin.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdccclvi{Und was läßt dich wissen, was Siggin ist?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdccclvii{Es ist) ein Buch mit festen Eintragungen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdccclviii{Wehe an jenem Tag den Leugnern,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdccclix{die den Tag des Gerichts für Lüge erklären!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdccclx{Und nur derjenige erklärt ihn für Lüge, der Übertretungen begeht und ein Sünder ist.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxi{Wenn ihm Unsere Zeichen verlesen werden, sagt er: ``(Es sind) Fabeln der Früheren."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxii{Keineswegs! Vielmehr hat sich das, was sie zu erwerben pflegten, über ihren Herzen angesetzt.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxiii{Keineswegs! Sie werden von ihrem Herrn an jenem Tag bestimmt abgeschirmt sein.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxiv{Hierauf werden sie bestimmt dem Höllenbrand ausgesetzt sein.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxv{Hierauf wird gesagt werden: ``Das ist das, was ihr für Lüge zu erklären pflegtet."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxvi{Keineswegs! Das Buch der Frommen ist wahrlich in 'Illiyyin.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxvii{Und was läßt dich wissen, was 'Illiyyun ist?\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxviii{Es ist) ein Buch mit festen Eintragungen,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxix{in das die (Allah) Nahegestellten Einsicht nehmen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxx{Die Frommen werden wahrlich in Wonne sein,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxi{auf überdachten Liegen (gelehnt), und blicken um sich.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxii{Du erkennst in ihren Gesichtern das Strahlen der Wonne.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxiii{Ihnen wird von versiegeltem Nektar zu trinken gegeben,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxiv{dessen Siegel Moschus ist -, und darum sollen die Wettbewerber wettkämpfen -,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxv{und dessen Beimischung Tasnim ist,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxvi{aus einer Quelle, aus der die (Allah) Nahegestellten trinken.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxvii{Gewiß, diejenigen, die Übeltaten begingen, pflegten über diejenigen zu lachen, die glauben,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxviii{und, wenn sie an ihnen vorbeikamen, einander zuzuzwinkern.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxix{Und, wenn sie zu ihren Angehörigen zurückkehrten, kehrten sie zurück, indem sie es sich wohl sein ließen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxx{Und, wenn sie sie sahen, sagten sie: ``Diese gehen fürwahr in die Irre."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxi{Dabei waren sie doch nicht als Hüter über sie gesandt worden.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxii{Heute aber lachen diejenigen, die glauben, über die Ungläubigen,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxiii{auf überdachten Liegen (gelehnt), und blicken um sich.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxiv{Sind die Ungläubigen nicht doch belohnt worden für das, was sie zu tun pflegten?\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxv{\basmalahde Wenn der Himmel sich spaltet\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxvi{und auf seinen Herrn hört - und es ist ja rechtens für ihn -,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxvii{und wenn die Erde ausgedehnt wird\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxviii{und herauswirft, was in ihr ist, und sich entleert\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxix{und auf ihren Herrn hört - und es ist ja rechtens für sie ...\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxc{O du Mensch, du mühst dich hart zu deinem Herrn hin, und so wirst du Ihm begegnen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxci{Was nun jemanden angeht, dem dann sein Buch in seine Rechte gegeben wird.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcii{der wird einer leichten Abrechnung unterzogen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxciii{und er wird erfreut zu seinen Angehörigen zurückkehren.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxciv{Was aber jemanden angeht, dem sein Buch hinter seinem Rücken gegeben wird,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcv{der wird nach Vernichtung rufen\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcvi{und der Feuerglut ausgesetzt sein.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcvii{Er war ja froh inmitten seiner Angehörigen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcviii{Er meinte ja, daß er nicht zurückkehren würde.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcix{Ja doch! Gewiß, sein Herr sieht ihn wohl.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcm{Nein! Ich schwöre beim Abendrot\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmi{und der Nacht und dem, was sie zusammentreibt,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmii{und dem Mond, wenn er voll geworden ist.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmiii{Ihr werdet eine Schicht nach der anderen besteigen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmiv{Was ist denn mit ihnen, daß sie nicht glauben\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcmv{und, wenn ihnen der Qur'an vorgelesen wird, sich nicht niederwerfen?\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcmvi{Aber nein! Diejenigen, die ungläubig sind, erklären (ihn) für Lüge.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcmvii{Doch Allah weiß sehr wohl, was sie (in ihren Herzen) für sich behalten.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcmviii{So verkünde ihnen schmerzhafte Strafe,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcmix{außer denjenigen, die glauben und rechtschaffene Werke tun; für sie wird es einen Lohn geben, der nicht aufhört.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcmx{\basmalahde Beim Himmel mit den Türmen\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmxi{und dem versprochenen Tag\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmxii{und dem Zeugen und dem Bezeugten!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmxiii{Tod (sei) geweiht den Leuten des Grabens,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmxiv{des Feuers mit dem (vielen) Brennstoff,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmxv{als sie daran saßen\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmxvi{und Zeugen dessen waren, was sie den Gläubigen antaten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmxvii{Und sie grollten ihnen nur, daß sie an Allah glaubten, den Allmächtigen und Lobenswürdigen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmxviii{Dem die Herrschaft der Himmel und der Erde gehört. Und Allah ist über alles Zeuge.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmxix{Gewiß, diejenigen, die die gläubigen Männer und die gläubigen Frauen in Versuchung bringen und hierauf nicht in Reue umkehren, für sie wird es die Strafe der Hölle geben, und für sie wird es die Strafe des Brennens geben.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmxx{Gewiß, diejenigen, die glauben und rechtschaffene Werke tun, für sie wird es Gärten geben, durcheilt von Bächen; das ist der große Erfolg.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxi{Das Zupacken deines Herrn ist wahrlich hart.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxii{Gewiß, Er macht den Anfang und läßt wiederkehren.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxiii{Und Er ist der Allvergebende und Liebevolle,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxiv{der Herr des Thrones, der Ruhmvolle.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxv{Er) tut alles, was Er will.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxvi{Ist zu dir die Geschichte der Heerscharen gekommen,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxvii{Fir'auns und der Tamud?\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxviii{Aber nein! Diejenigen, die ungläubig sind, sind dem Leugnen (der Botschaft) verhaftet.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxix{Doch Allah ist hinter ihnen her, (sie) umfassend.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxx{Nein! Vielmehr ist es ein ruhmvoller Qur'an\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxi{auf einer wohlbehüteten Tafel.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxii{\basmalahde Beim Himmel und dem Pochenden!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxiii{Und was läßt dich wissen, was der Pochende ist?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxiv{Es ist) der durchbohrend helle Stem.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxv{Es gibt keine Seele, über der nicht ein Hüter (eingesetzt) ist.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxvi{So soll der Mensch doch hinsehen, woraus er erschaffen ist.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxvii{Er ist aus sich ergießendem Wasser erschaffen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxviii{das zwischen der Lende und der (weiblichen) Brust hervorkommt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxix{Er hat die Macht dazu, ihn zurückkehren zu lassen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmxl{am Tag, da die geheimsten Gedanken geprüft werden.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmxli{Dann hat er weder Kraft noch einen Helfer.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlii{Beim Himmel mit der Wiederkehr\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmxliii{und der Erde mit ihren Sprüngen!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmxliv{Er ist wahrlich ein entscheidendes Wort;\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlv{er ist kein Scherz.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlvi{Sie wenden eine List an,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlvii{Und Ich wende eine List an.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlviii{So lasse den Ungläubigen noch Zeit; lasse ihnen nur eine Weile Zeit.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlix{\basmalahde Preise den Namen deines höchsten Herrn,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcml{Der erschafft und dann zurechtformt\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmli{und Der das Maß festsetzt und dann rechtleitet\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmlii{und Der die Weide hervorbringt\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmliii{und sie dann zu dunkelbrauner Spreu macht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmliv{Wir werden dich lesen lassen, und dann wirst du nichts vergessen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmlv{außer dem, was Allah will. Er weiß ja, was laut vernehmbar geäußert wird und was verborgen bleibt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmlvi{Und Wir werden dir den Weg zum Leichteren leicht machen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmlvii{So ermahne -, wenn die Ermahnung nützt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmlviii{Bedenken wird jemand, der gottesfürchtig ist.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmlix{Meiden aber wird es der Unseligste,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmlx{der dem größten (Höllen)feuer ausgesetzt sein wird;\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxi{darin wird er hierauf weder sterben noch leben.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxii{Wohl ergehen wird es ja jemandem, der sich läutert,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxiii{und des Namens seines Herrn gedenkt; so betet er.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxiv{Nein! Vielmehr zieht ihr das diesseitige Leben vor,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxv{während das Jenseits besser und beständiger ist.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxvi{Dies ist wahrlich in den früheren Blättern (enthalten),\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxvii{den Blättern Ibrahims und Musas.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxviii{\basmalahde Ist zu dir die Geschichte der Überdeckenden gekommen?\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxix{Die einen) Gesichter werden an jenem Tag demütig sein,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxx{sich abarbeitend und von Mühsal erfüllt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxi{einem sehr heißen Feuer ausgesetzt,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxii{und aus einer siedenden Quelle zu trinken bekommen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxiii{Es wird für sie keine Speise geben außer aus trockenen Dornen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxiv{die weder fett machen noch gegen den Hunger nützen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxv{Die anderen) Gesichter werden an jenem Tag wonnig sein,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxvi{mit ihrem Bemühen zufrieden\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxvii{und in einem hohen Garten,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxviii{worin sie keine unbedachte Rede hören.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxix{Darin gibt es eine fließende Quelle,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxx{darin gibt es erhöhte Liegen\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxi{und hingestellte Trinkschalen\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxii{und aufgereihte Kissen\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxiii{und ausgebreitete Teppiche.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxiv{Schauen sie denn nicht zu den Kamelen, wie sie erschaffen worden sind,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxv{und zum Himmel, wie er emporgehoben worden ist,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxvi{und zu den Bergen, wie sie aufgerichtet worden sind,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxvii{und zur Erde, wie sie flach gemacht worden ist?\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxviii{So ermahne; du bist nur ein Ermahner.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxix{Du übst nicht die Oberherrschaft über sie aus.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcmxc{Wer sich aber abkehrt und ungläubig ist,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcmxci{den straft dann Allah mit der größten Strafe.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcii{Gewiß, zu Uns ist ihre Rückkehr\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcmxciii{und gewiß, hierauf obliegt Uns ihre Abrechnung.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcmxciv{\basmalahde Bei der Morgendämmerung\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcv{und den zehn Nächten\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcvi{und dem geraden und dem ungeraden (Tag)\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcvii{und der Nacht, wenn sie fortzieht!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcviii{Ist darin ein (ausreichender) Schwur für jemanden, der Klugheit besitzt?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcix{Siehst du nicht, wie dein Herr mit den 'Ad verfuhr,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmm{mit) Iram mit den Stützen,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmi{dergleichen nicht erschaffen wurden in den Landstrichen?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmii{Und (mit) den Tamud, die im Tal die Felsen aushöhlten?\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmiii{Und (mit) Fir'aun, dem Besitzer der Pfähle,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmiv{die (sie alle) das Maß (an Frevel) in den Landstrichen überschritten\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmv{und darin viel Unheil stifteten?\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmvi{Da schüttete dein Herr auf sie eine Geißel von Strafe aus.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmvii{Gewiß, dein Herr wartet ja (auf sie).\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmviii{Was nun den Menschen angeht, wenn sein Herr ihn prüft und ihn da(bei) freigebig behandelt und ihm ein wonnevolles Leben verschafft, so sagt er: ``Mein Herr behandelt mich freigebig."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmix{Was (ihn) aber angeht, wenn Er ihn prüft und ihm da(bei) seine Versorgung bemißt, so sagt er: ``Mein Herr setzt mich der Schmach aus."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmx{Keineswegs! Vielmehr behandelt ihr die Waise nicht freigebig\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmxi{und haltet euch nicht gegenseitig zur Speisung des Armen an.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmxii{Und ihr verzehrt das Erbe, ja ihr verzehrt es ganz und gar.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmxiii{Und ihr liebt den Besitz, ja ihr liebt ihn voll und ganz.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmmxiv{Keineswegs! Wenn die Erde eingeebnet, ja platt eingeebnet wird\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmmxv{und dein Herr kommt und die Engel, Reihe um Reihe,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmmxvi{und herbeigebracht wird an jenem Tag die Hölle, an jenem Tag wird der Mensch bedenken. Wie soll ihm dann die Selbstbesinnung (nützen)?\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmmxvii{Er wird sagen: ``O hätte ich doch für mein (jenseitiges) Leben (etwas) vorausgeschickt!"\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmmxviii{An jenem Tag wird niemand strafen, so wie Er straft,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmmxix{und niemand wird fesseln, so wie Er fesselt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmmxx{O du Seele, die du Ruhe gefunden hast,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmmxxi{kehre zu deinem Herrn zufrieden und mit Wohlgefallen zurück.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmmxxii{Tritt ein unter Meine Diener,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmmxxiii{und tritt ein in Meinen (Paradies)garten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmmxxiv{\basmalahde Nein, Ich schwöre bei dieser Ortschaft\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmxxv{-, wo du in dieser Ortschaft deinen Aufenthalt hast,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmxxvi{- und bei einem (jeden) Erzeuger und dem, was er zeugt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmxxvii{Wir haben den Menschen ja (zu einem Leben) in Mühsal erschaffen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmxxviii{Meint er etwa, daß überhaupt niemand Macht über ihn hat?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmxxix{Er sagt: ``Ich habe Besitz in Mengen verbraucht."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmxxx{Meint er etwa, daß niemand ihn gesehen hat?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxi{Haben Wir ihm nicht zwei Augen gemacht,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxii{eine Zunge und zwei Lippen\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxiii{und ihn beide Hochebenen geleitet?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxiv{Aber er ist nicht den steilen Paßweg hinaufgestürmt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxv{Und was läßt dich wissen, was der steile Paßweg ist?\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxvi{Es ist) die Freilassung eines Sklaven10\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxvii{oder zu speisen am Tag der Hungersnot\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxviii{eine Waise, die einem nahe ist,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxix{oder einen Armen, der dem Boden nahe ist.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmxl{Und daß man hierauf zu denjenigen gehört, die glauben, einander die Standhaftigkeit eindringlich empfehlen und einander die Barmherzigkeit eindringlich empfehlen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmxli{Das sind die Gefährten der rechten Seite.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmxlii{Diejenigen aber, die Unsere Zeichen verleugnen, sie sind die Gefährten der unglückseligen Seite.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmxliii{Über ihnen liegt ein (sie) einschließendes Feuer.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmmxliv{\basmalahde Bei der Sonne und ihrer Morgenhelle\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmxlv{und dem Mond, wenn er ihr folgt,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmxlvi{und dem Tag, wenn er sie erscheinen läßt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmxlvii{und der Nacht, wenn sie sie überdeckt,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmxlviii{und dem Himmel und Dem, Der ihn aufgebaut hat,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmxlix{und der Erde und Dem, Der sie ausgebreitet hat,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmml{und einer (jeden) Seele und Dem, Der sie zurechtgeformt hat\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmli{und ihr dann ihre Sittenlosigkeit und ihre Gottesfurcht eingegeben hat!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmlii{Wohl ergehen wird es ja jemandem, der sie läutert,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmliii{und enttäuscht sein wird ja, wer sie verkümmern läßt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmliv{Die Tamud erklärten in ihrer Auflehnung (die Botschaft) für Lüge,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmlv{als der Unseligste von ihnen sich erhob.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmlvi{Allahs Gesandter sagte zu ihnen: ``(Achtet auf) Allahs Kamelstute und ihre Trinkzeit."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmlvii{Sie aber bezichtigten ihn der Lüge, und so schnitten sie ihr die Sehnen durch. Da schmetterte ihr Herr sie für ihre Sünde nieder, und so ebnete Er über ihnen die Erde ein.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmlviii{Und Er (Allah) fürchtet nicht die Folge davon.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmlix{\basmalahde Bei der Nacht, wenn sie (alles) überdeckt,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmlx{und dem Tag, wenn er (in seiner Helligkeit) erscheint,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmlxi{und Dem, Der das Männliche und das Weibliche erschaffen hat!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmlxii{Euer Bemühen ist wahrlich verschieden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmlxiii{Was nun jemanden angeht, der gibt und gottesfürchtig ist\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmlxiv{und das Beste für wahr hält,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmlxv{so werden Wir ihm den Weg zum Leichteren leicht machen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmlxvi{Was aber jemanden angeht, der geizt und sich für unbedürftig hält\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmlxvii{und das Beste für Lüge erklärt,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmlxviii{so werden Wir ihm den Weg zum Schwereren leicht machen;\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmlxix{und was soll sein Besitz ihm nützen, wenn er sich ins Verderben stürzt?\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmlxx{Uns obliegt wahrlich die Rechtleitung.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxi{Und Uns gehört wahrlich das Jenseits und das Diesseits.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxii{So habe Ich euch gewarnt vor einem Feuer, das lodert,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxiii{dem nur der Unseligste ausgesetzt sein wird,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxiv{der (die Botschaft) für Lüge erklärt und sich abkehrt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxv{Doch davon ferngehalten wird der Gottesfürchtigste werden,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxvi{der seinen Besitz hingibt, um sich zu läutern,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxvii{und niemand hat bei ihm eine Gunst (anzurechnen), die vergolten werden müßte,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxviii{sondern (er handelt) im Trachten nach dem Angesicht seines höchsten Herrn.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxix{Und er wird wahrlich zufrieden sein.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxx{\basmalahde Bei der Morgenhelle\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxi{und der Nacht, wenn sie (alles) umhüllt!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxii{Dein Herr hat sich weder von dir verabschiedet noch haßt Er (dich).\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxiii{Und das Jenseits ist wahrlich besser für dich als das Diesseits.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxiv{Und dein Herr wird dir wahrlich geben, und dann wirst du zufrieden sein.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxv{Hat Er dich nicht als Waise gefunden und (dir) dann Zuflucht verschafft\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxvi{und dich irregehend gefunden und dann rechtgeleitet\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxvii{und dich arm gefunden und dann reich gemacht?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxviii{Was nun die Waise angeht, so unterjoche (sie) nicht,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxix{und was den Bettler angeht, so fahre (ihn) nicht an,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmxc{und was die Gunst deines Herrn angeht, so erzähle (davon).\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmxci{\basmalahde Haben Wir dir nicht deine Brust aufgetan\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmxcii{und dir deine Last abgenommen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmxciii{die deinen Rücken niederdrückte,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmxciv{und dir dein Ansehen erhöht?\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmxcv{Also gewiß, mit der Erschwernis ist Erleichterung,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmxcvi{gewiß, mit der Erschwernis ist Erleichterung.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmxcvii{Wenn du nun fertig bist, dann strenge dich an\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmxcviii{und nach deinem Herrn richte dein Begehren aus.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmxcix{\basmalahde Bei der Feige und der Olive\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmc{und dem Berg Sinin\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmci{und dieser sicheren Ortschaft!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcii{Wir haben den Menschen ja in schönster Gestaltung erschaffen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmciii{hierauf haben Wir ihn zu den Niedrigsten der Niedrigen werden lassen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmciv{außer denjenigen, die glauben und rechtschaffene Werke tun; für sie wird es einen Lohn geben, der nicht aufhört.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcv{Was läßt dich da weiterhin das Gericht für Lüge erklären?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcvi{Ist nicht Allah der Weiseste derjenigen, die richten?\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcvii{\basmalahde Lies im Namen deines Herrn, Der erschaffen hat,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcviii{den Menschen erschaffen hat aus einem Anhängsel.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcix{Lies, und dein Herr ist der Edelste,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcx{Der (das Schreiben) mit dem Schreibrohr gelehrt hat,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxi{den Menschen gelehrt hat, was er nicht wußte.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxii{Keineswegs! Der Mensch lehnt sich wahrlich auf,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcxiii{daß er von sich meint, unbedürftig zu sein.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcxiv{Gewiß, zu deinem Herrn wird die Rückkehr sein.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcxv{Siehst du denjenigen, der abhält\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmcxvi{einen Diener, wenn er betet?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmcxvii{Siehst du! Obwohl er nach der Rechtleitung verfährt,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmcxviii{oder die Gottesfurcht gebietet?\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmcxix{Siehst du! Wie (wäre es) wenn er (die Botschaft) für Lüge erklärt und sich abkehrt?\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmcxx{Weiß er denn nicht, daß Allah sieht?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxi{Keineswegs! Wenn er nicht aufhört, werden Wir ihn ganz gewiß an der Stirnlocke packen und ziehen,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxii{einer Stirnlocke, einer lügnerischen, einer verfehlt handelnden.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxiii{So soll er doch seine Genossen rufen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxiv{Wir werden die (Höllen)wache rufen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxv{Keineswegs! Gehorche ihm nicht, sondern wirf dich nieder und sei (Allah) nah!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxvi{\basmalahde Wir haben ihn ja in der Nacht der Bestimmung hinabgesandt.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxvii{Und was läßt dich wissen, was die Nacht der Bestimmung ist?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxviii{Die Nacht der Bestimmung ist besser als tausend Monate.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxix{Es kommen die Engel und der Geist in ihr mit der Erlaubnis ihres Herrn mit jeder Angelegenheit herab.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxx{Frieden ist sie bis zum Anbruch der Morgendämmerung.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxi{\basmalahde Diejenigen von den Leuten der Schrift und den Götzendienern, die ungläubig sind, werden sich nicht (eher von ihrem Unglauben) lösen, bis (daß) der klare Beweis zu ihnen kommt,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxii{ein Gesandter von Allah, der gereinigte Blätter verliest,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxiii{in denen rechte Schriften sind.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxiv{Und diejenigen, denen die Schrift gegeben wurde, haben sich nicht eher gespalten, als bis der klare Beweis zu ihnen gekommen ist.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxv{Und nichts anderes wurde ihnen befohlen, als nur Allah zu dienen und (dabei) Ihm gegenüber aufrichtig in der Religion (zu sein), als Anhänger des rechten Glaubens, und das Gebet zu verrichten und die Abgabe zu entrichten; das ist die Religion des rechten Verhaltens.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxvi{Gewiß, diejenigen unter den Leuten der Schrift und den Götzendienern, die ungläubig sind, werden im Feuer der Hölle sein, ewig darin zu bleiben. Das sind die schlechtesten Geschöpfe.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxvii{Gewiß, diejenigen aber, die glauben und rechtschaffene Werke tun, das sind die besten Geschöpfe.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxviii{Ihr Lohn bei ihrem Herrn sind die Gärten Edens, durcheilt von Bächen, ewig und auf immer darin zu bleiben. Allah hat Wohlgefallen an ihnen, und sie haben Wohlgefallen an Ihm; das ist für jemanden, der seinen Herrn fürchtet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxix{\basmalahde Wenn die Erde erschüttert wird durch ihr heftiges Beben\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxl{und die Erde hervorbringt ihre Lasten\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxli{und der Mensch sagt: ``Was ist mit ihr?",\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlii{an jenem Tag wird sie die Nachrichten über sich erzählen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxliii{weil dein Herr (es) ihr eingegeben hat.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxliv{An jenem Tag werden die Menschen (in Gruppen) getrennt herauskommen, damit ihnen ihre Werke gezeigt werden.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlv{Wer nun im Gewicht eines Stäubchens Gutes tut, wird es sehen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlvi{Und wer im Gewicht eines Stäubchens Böses tut, wird es sehen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlvii{\basmalahde Bei den schnaubend Rennenden,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlviii{den (mit ihren Hufen) Funken Schlagenden,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlix{den am Morgen Angreifenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcl{die darin Staub aufwirbeln,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcli{die dann mitten in die Ansammlung eindringen!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclii{Der Mensch ist seinem Herrn gegenüber wahrlich undankbar,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcliii{und er (selbst) ist darüber wahrlich Zeuge.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcliv{Und er ist in seiner Liebe zum (eigenen) Besten wahrlich heftig.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclv{Weiß er denn nicht? Wenn durchwühlt wird, was in den Gräbern ist,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmclvi{und herausgeholt wird, was in den Brüsten ist,...\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmclvii{ihr Herr wird an jenem Tag ihrer wahrlich Kundig sein.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmclviii{\basmalahde Das Verhängnis!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclix{Was ist das Verhängnis?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclx{Und was läßt dich wissen, was das Verhängnis ist?\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxi{Am Tag, da die Menschen wie flatternde Motten sein werden\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmclxii{und die Berge wie zerflockte gefärbte Wolle sein werden.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclxiii{Was nun jemanden angeht, dessen Waagschalen schwer sind,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmclxiv{so wird er in einem zufriedenen Leben sein.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmclxv{Was aber jemanden angeht, dessen Waagschalen leicht sind,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclxvi{dessen Mutter wird ein Abgrund sein.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmclxvii{Und was läßt dich wissen, was das ist?\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmclxviii{Ein sehr heißes Feuer.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmclxix{\basmalahde Die Vermehrung lenkt euch ab,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclxx{bis ihr die Friedhöfe besucht.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxi{Keineswegs! Ihr werdet (es noch) erfahren.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxii{Abermals: Keineswegs! Ihr werdet (es noch) erfahren.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxiii{Keineswegs! Wenn ihr es nur mit dem Wissen der Gewißheit wüßtet!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxiv{Ihr werdet ganz gewiß den Höllenbrand sehen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxv{Abermals: Ihr werdet ihn mit dem Auge der Gewißheit sehen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxvi{Hierauf werdet ihr an jenem Tag ganz gewiß nach der Wonne gefragt werden.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxvii{\basmalahde Beim Zeitalter!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxviii{Der Mensch befindet sich wahrlich in Verlust,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxix{außer denjenigen, die glauben und rechtschaffene Werke tun und einander die Wahrheit eindringlich empfehlen und einander die Standhaftigkeit eindringlich empfehlen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxx{\basmalahde Wehe jedem Stichler und Nörgler,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxi{der Besitz zusammenträgt und ihn zählt und immer wieder zählt,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxii{wobei er meint, daß sein Besitz ihn ewig leben ließe!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxiii{Keineswegs! Er wird ganz gewiß in al-Hutama geworfen werden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxiv{Was läßt dich wissen, was al-Hutama ist?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxv{Sie ist) Allahs entfachtes Feuer,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxvi{das Einblick in die Herzen gewinnt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxvii{Gewiß, es wird sie einschließen\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxviii{in langgestreckten Säulen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxix{\basmalahde Siehst du nicht, wie dein Herr mit den Leuten des Elefanten verfuhr?\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxc{Ließ Er nicht ihre List verlorengehen\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxci{und sandte gegen sie Vögel in aufeinanderfolgenden Schwärmen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcii{die sie mit Steinen aus gebranntem Lehm bewarfen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxciii{und sie so wie abgefressene Halme machte?\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxciv{\basmalahde Für die Vereinigung der Qurais\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcv{ihre Vereinigung während der Reise des Winters und des Sommers.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcvi{So sollen sie dem Herrn dieses Hauses dienen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcvii{Der ihnen Speise nach ihrem Hunger gegeben und ihnen Sicherheit nach ihrer Furcht gewährt hat.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcviii{\basmalahde Siehst du (nicht) denjenigen, der das Gericht für Lüge erklärt?\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcix{Das ist derjenige, der die Waise zurückstößt\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcc{und nicht zur Speisung des Armen anhält.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcci{Wehe nun den Betenden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccii{denjenigen, die auf ihre Gebete nicht achten,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcciii{denjenigen, die dabei (nur) gesehen werden wollen;\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcciv{und die Hilfeleistung verweigern!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmccv{\basmalahde Wir haben dir ja al-Kautar gegeben.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccvi{So bete zu deinem Herrn und opfere.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccvii{Gewiß, derjenige, der dich haßt, - er ist vom Guten abgetrennt .\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccviii{\basmalahde Sag: O ihr Ungläubigen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccix{ich diene nicht dem, dem ihr dient,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccx{und ihr dient nicht Dem, Dem ich diene.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxi{Und ich werde (auch) nicht dem dienen, dem ihr gedient habt,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxii{Und ihr werdet nicht Dem dienen, Dem ich diene.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxiii{Euch eure Religion und mir meine Religion.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmccxiv{\basmalahde Wenn Allahs Hilfe kommt und der Sieg\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxv{und du die Menschen in Allahs Religion in Scharen eintreten siehst,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxvi{dann lobpreise deinen Herrn und bitte Ihn um Vergebung; gewiß, Er ist Reueannehmend.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxvii{\basmalahde Zugrunde gehen sollen die Hände Abu Lahabs, und zugrunde gehen soll er (selbst)!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxviii{Was nützt ihm sein Besitz und das, was er erworben hat?\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxix{Er wird einem Feuer voller Flammen ausgesetzt sein\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxx{und (auch) seine Frau, die Brennholzträgerin.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxi{Um ihrem Hals ist ein Strick aus Palmfasern.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxii{\basmalahde Sag: Er ist Allah, ein Einer,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxiii{Allah, der Überlegene.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxiv{Er hat nicht gezeugt und ist nicht gezeugt worden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxv{und niemand ist Ihm jemals gleich.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxvi{\basmalahde Sag: Ich nehme Zuflucht beim Herrn des Tagesanbruchs\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxvii{vor dem Übel dessen, was Er erschaffen hat,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxviii{und vor dem Übel der Dunkelheit, wenn sie zunimmt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxix{und vor dem Übel der Knotenanbläserinnen\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxx{und vor dem Übel eines (jeden) Neidenden, wenn er neidet.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxi{\basmalahde Sag: Ich nehme Zuflucht beim Herrn der Menschen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxii{dem König der Menschen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxiii{dem Gott der Menschen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxiv{vor dem Übel des Einflüsterers, des Davonschleichers,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxv{der in die Brüste der Menschen einflüstert,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxvi{von den Ginn und den Menschen.\qt@no{6}}%
%% End of file `qurantext-deii.translation.def'.
