\qt@newcmd\qurantrans@de@i{Bismil-lahir-rahmanir-rahim: Mit dem Namen ALLAHs, Des Allgnade Erweisenden, Des Allgnädigen, (rezitiere ich).\qt@no{1}}%
\qt@newcmd\qurantrans@de@ii{Alhamdulillah: Alles Lob gebührt ALLAH, Dem HERRN aller Geschöpfe,\qt@no{2}}%
\qt@newcmd\qurantrans@de@iii{Dem Allgnade Erweisenden, Dem Allgnädigen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@iv{Dem Herrscher am Tage des Din.\qt@no{4}}%
\qt@newcmd\qurantrans@de@v{Dir allein dienen wir, und Dich allein bitten wir um Hilfe!\qt@no{5}}%
\qt@newcmd\qurantrans@de@vi{Leite uns recht auf den geradlinigen Weg,\qt@no{6}}%
\qt@newcmd\qurantrans@de@vii{den Weg derer, denen DU Wohlergehen hast zuteil werden lassen, die weder vom Zorn geplagt werden noch abgeirrt sind!\qt@no{7}}%
\qt@newcmd\qurantrans@de@viii{\basmalahde Alif-lam-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@ix{Diese ist die eigentliche Schrift, keinerlei Zweifel ist darin, (sie ist) eine Rechtleitung für die Muttaqi.\qt@no{2}}%
\qt@newcmd\qurantrans@de@x{Es sind diejenigen, die den Iman an das Verborgene verinnerlichen, das rituelle Gebet ordnungsgemäß verrichten und von dem, was WIR ihnen vom Rizq gewährten, geben,\qt@no{3}}%
\qt@newcmd\qurantrans@de@xi{sowie diejenigen, die den Iman verinnerlichen an das, was dir hinabgesandt wurde, und an das, was vor dir hinabgesandt wurde, und die über das Jenseits Gewißheit haben.\qt@no{4}}%
\qt@newcmd\qurantrans@de@xii{Diese verfügen über Rechtleitung von ihrem HERRN und diese sind die eigentlichen Erfolgreichen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@xiii{Gewiß, diejenigen, die (beharrlich) Kufr betrieben haben - gleich ist es für sie, ob du sie warnst oder nicht warnst, sie verinnerlichen den Iman nicht.\qt@no{6}}%
\qt@newcmd\qurantrans@de@xiv{ALLAH versiegelte ihre Herzen und ihr Gehör, und über ihren Augen ist eine (Sicht-) Blende. Und für sie ist überharte Peinigung bestimmt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@xv{Und unter den Menschen sind manche, die sagen: ``Wir haben den Iman an ALLAH und an den Jüngsten Tag verinnerlicht." Doch sie sind keine Mumin.\qt@no{8}}%
\qt@newcmd\qurantrans@de@xvi{Täuschen wollen sie ALLAH und diejenigen, die den Iman verinnerlicht haben. Doch sie täuschen niemanden außer sich selbst, nur sie merken es nicht.\qt@no{9}}%
\qt@newcmd\qurantrans@de@xvii{In ihren Herzen ist Krankheit, so mehrte ALLAH sie mit Krankheit, und für sie ist qualvolle Peinigung bestimmt für das, was sie zu lügen pflegten.\qt@no{10}}%
\qt@newcmd\qurantrans@de@xviii{Und als ihnen gesagt wurde: ``Richtet kein Verderben an auf Erden!", sagten sie: ``Wir sind doch nur gottgefällig Guttuende."\qt@no{11}}%
\qt@newcmd\qurantrans@de@xix{Aber sicher, sie sind die Verderben-Anrichtenden, doch sie merken es nicht.\qt@no{12}}%
\qt@newcmd\qurantrans@de@xx{Und als ihnen gesagt wurde: ``Verinnerlicht den Iman, wie die (anderen) Menschen den Iman verinnerlicht haben", sagten sie: ``Sollen wir etwa den Iman verinnerlichen, wie die Beschränkten den Iman verinnerlicht haben?" Jedenfalls sind sie die Beschränkten, doch sie wissen es nicht.\qt@no{13}}%
\qt@newcmd\qurantrans@de@xxi{Und als sie denjenigen begegnet sind, die den Iman verinnerlicht haben, sagten sie: ``Wir haben den Iman verinnerlicht." Jedoch als sie sich alleine unter ihren Satanen befanden, sagten sie: ``Wir sind mit euch, wir sind lediglich Spötter!"\qt@no{14}}%
\qt@newcmd\qurantrans@de@xxii{ALLAH vergilt ihnen ihren Spott und läßt sie in ihrer Maßlosigkeit bestärken - sie bleiben verblendet.\qt@no{15}}%
\qt@newcmd\qurantrans@de@xxiii{Diese sind diejenigen, die das Abirren gegen die Rechtleitung eintauschten, so war weder ihr Handel gewinnbringend, noch waren sie rechtgeleitet.\qt@no{16}}%
\qt@newcmd\qurantrans@de@xxiv{Ihr Gleichnis ähnelt dem Gleichnis desjenigen, der Feuer anzumachen suchte, und nachdem es seine Umgebung erhellt hatte, nahm ALLAH ihnen ihr Licht weg und ließ sie in Finsternissen zurück - sie können nicht sehen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@xxv{Sie sind taub, stumm und blind, so kehren sie nicht um.\qt@no{18}}%
\qt@newcmd\qurantrans@de@xxvi{Oder (sie ähneln manchen unter) einem Wolkenbruch aus dem Himmel, welcher von Finsternissen, Donner und Blitz begleitet wird. Sie stecken ihre Finger in ihre Ohren wegen der Donnerschläge aus Angst vor dem Tod  doch ALLAH ist umgebend den Kafir gegenüber.\qt@no{19}}%
\qt@newcmd\qurantrans@de@xxvii{Beinahe raubte ihnen der Blitzstrahl ihr Augenlicht. Immer wieder, wenn er ihnen aufleuchtete, gingen sie darin, und wenn er über ihnen finster wurde, erstarrten sie. Und wenn ALLAH es wollte, hätte ER ihnen ihr Gehör und ihr Augenlicht weggenommen. Gewiß, ALLAH ist über alles allmächtig.\qt@no{20}}%
\qt@newcmd\qurantrans@de@xxviii{Ihr Menschen! Dient eurem HERRN, Der euch erschuf und diejenigen vor euch, damit ihr Taqwa gemäß handelt.\qt@no{21}}%
\qt@newcmd\qurantrans@de@xxix{Dient Dem), Der euch die Erde als Unterlage und den Himmel als Überdachung machte und vom Himmel Wasser fallen ließ, dann damit Früchte hervorbrachte als Rizq für euch. So setzt ALLAH nichts als Ebenbürtige bei, während ihr wisst.\qt@no{22}}%
\qt@newcmd\qurantrans@de@xxx{Und solltet ihr Zweifel haben über das, was WIR Unserem Diener nach und nach hinabsandten, dann bringt (nur) eine Sura seinesgleichen her und ruft eure Zeugen anstelle von ALLAH auf, solltet ihr wahrhaftig sein.\qt@no{23}}%
\qt@newcmd\qurantrans@de@xxxi{Und wenn ihr es nicht tut, und ihr werdet es gewiß niemals tun, so sucht Taqwa vor dem Feuer, dessen Brennstoff Menschen und Steine sind, das für die Kafir vorbereitet wurde.\qt@no{24}}%
\qt@newcmd\qurantrans@de@xxxii{Und überbringe frohe Botschaft denjenigen, die den Iman verinnerlicht und das gottgefällig Gute getan haben, daß für sie Dschannat bestimmt sind, die von Flüssen durchflossen werden. Immer wieder, wenn ihnen daraus eine Frucht als Rizq gewährt wird, sagen sie: ``Das ist es, das uns vorher als Rizq gewährt wurde." Und ihnen schien es ähnlich. Und für sie sind darin gereinigte Partnerwesen bestimmt, und sie sind darin ewig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@xxxiii{Gewiß, ALLAH findet nichts Beschämendes dabei, daß ER ein Gleichnis einer Mücke oder dessen, was größer ist, prägt. Also hinsichtlich derjenigen, die den Iman verinnerlicht haben, so wissen sie, daß dies sicher die Wahrheit von ihrem HERRN ist. Und hinsichtlich derjenigen, die Kufr betrieben haben, so sagen sie: ``Was wollte ALLAH damit als Gleichnis?" Damit läßt ALLAH viele abirren und viele rechtleiten. Doch ER läßt damit niemanden außer den Fasiq abirren.\qt@no{26}}%
\qt@newcmd\qurantrans@de@xxxiv{Es sind diejenigen, die das von ALLAH Auferlegte nach dessen verbindlichem Abschluß verletzen, das abbrechen, dessen Verbindung ALLAH gebot, und auf der Erde Verderben anrichten. Diese sind die eigentlichen Verlierer.\qt@no{27}}%
\qt@newcmd\qurantrans@de@xxxv{Wie betreibt ihr nur Kufr ALLAH gegenüber, während ihr tot wart, dann ER euch belebte?! Dann läßt ER euch sterben, dann belebt ER euch (wieder), dann werdet ihr zu Ihm zurückgebracht.\qt@no{28}}%
\qt@newcmd\qurantrans@de@xxxvi{ER ist Derjenige, Der für euch alles auf der Erde erschuf, dann wandte ER sich dem Himmel zu, sogleich machte ER sie sieben Himmel. Und ER ist über alles allwissend.\qt@no{29}}%
\qt@newcmd\qurantrans@de@xxxvii{Und (erinnere daran), als dein HERR zu den Engeln sagte: ``Gewiß, ICH setze auf der Erde einen Khalifa ein." Sie sagten: ``Setzt DU etwa darauf ein, wer darauf Verderben anrichtet und Blut vergießt, während wir Dich mit Deinem Lob rühmen und uns für Dich rein halten?" ER sagte: ``Gewiß, ICH weiß, was ihr nicht wißt."\qt@no{30}}%
\qt@newcmd\qurantrans@de@xxxviii{Und ER lehrte Adam alle Namen, dann führte ER sie (die Dinge) den Engeln vor und sagte: ``TeiltMir deren Namen mit, solltet ihr wahrhaftig sein."\qt@no{31}}%
\qt@newcmd\qurantrans@de@xxxix{Sie sagten: ``Gepriesen-erhaben seist DU! Wir haben kein Wissen außer dem, das DU uns gelehrt hast. Gewiß, DU bist Der Allwissende, Der Allweise."\qt@no{32}}%
\qt@newcmd\qurantrans@de@xl{ER sagte: ``Adam! Teile ihnen deren Namen mit!" Und nachdem er ihnen deren Namen mitgeteilt hatte, sagte ER: ``Sagte ICH euch etwa nicht, daß ICH das Verborgene der Himmel und der Erde kenne, und daß ICH das kenne, was ihr offenlegt und was ihr zu verheimlichen pflegtet?!"\qt@no{33}}%
\qt@newcmd\qurantrans@de@xli{Und (erinnere daran), als WIR zu den Engeln sagten: ``Vollzieht Sudschud vor Adam!" Sogleich vollzogen sie Sudschud, außer Iblis lehnte es ab, erhob sich in erheblicher Arroganz und war von den Kafir.\qt@no{34}}%
\qt@newcmd\qurantrans@de@xlii{Und WIR sagten: ``Adam, bewohne mit deiner Gattin die Dschanna und esst darin reichlich, von wo ihr beide wollt, doch nähert euch nicht diesem Baum, sonst werdet ihr von den Verlierern sein."\qt@no{35}}%
\qt@newcmd\qurantrans@de@xliii{So ließ der Satan sie diesbezüglich Fehler begehen, so brachte er sie hinaus von dem, wo sie sich befanden. Und WIR sagten: ``Steigt ab! Die einen von euch (Menschen) sind Feind für die anderen (Satane), und für euch gibt es auf Erden Aufenthaltsort und Verbrauchsgut bis zu einer bestimmten Zeit."\qt@no{36}}%
\qt@newcmd\qurantrans@de@xliv{Dann empfing Adam Worte von seinem HERRN, so vergab ER ihm. Gewiß, ER ist Der Allvergebende, Der Allgnädige.\qt@no{37}}%
\qt@newcmd\qurantrans@de@xlv{WIR sagten: ``Steigt davon allesamt ab! Und wenn euch von Mir Rechtleitung erreicht, so wird es um diejenigen, die Meiner Rechtleitung folgen, weder Angst geben, noch werden sie traurig sein."\qt@no{38}}%
\qt@newcmd\qurantrans@de@xlvi{Und diejenigen, die Kufr betrieben und Unsere Ayat ableugnet haben, diese sind die Weggenossen des Feuers. Darin bleiben sie ewig.\qt@no{39}}%
\qt@newcmd\qurantrans@de@xlvii{Kinder Israils! Entsinnt euch Meiner Gaben, die ICH euch gewährt habe, und erfüllt das von Mir Auferlegte, dann erfülle ICH das euch von Mir Versprochene, und habt Ehrfurcht Mir gegenüber!\qt@no{40}}%
\qt@newcmd\qurantrans@de@xlviii{Und verinnerlicht den Iman an das, was ICH hinabsandte als Bestätigung dessen, worüber ihr verfügt, und seid nicht die ersten Kufr-Betreibenden ihm gegenüber. Und erkauft euch nicht mit Meinen Ayat einen minderwertigen Gegenwert, und handelt Taqwa gemäßMir gegenüber!\qt@no{41}}%
\qt@newcmd\qurantrans@de@xlix{Auch vermengt nicht das Wahre mit dem für nichtig Erklärten, und verbergt nicht dasWahre, während ihr wisst.\qt@no{42}}%
\qt@newcmd\qurantrans@de@l{Und verrichtet ordnungsgemäß das rituelle Gebet, entrichtet die Zakat und vollzieht Ruku' mit den Ruku'-Vollziehenden!\qt@no{43}}%
\qt@newcmd\qurantrans@de@li{Wollt ihr etwa den Menschen das gottgefällige Handeln gebieten und euch selbst (dabei) vergessen, während ihr die Schrift vortragt?! Begreift ihr nicht?!\qt@no{44}}%
\qt@newcmd\qurantrans@de@lii{Also helft euch mit Geduld und dem rituellen Gebet! Und gewiß, dies ist sicherlich schwer außer für die Ehrfürchtigen,\qt@no{45}}%
\qt@newcmd\qurantrans@de@liii{die sich sicher sind, daß sie ihrem HERRN begegnen und daß sie zu Ihm zurückkehren werden.\qt@no{46}}%
\qt@newcmd\qurantrans@de@liv{Kinder Israils! Entsinnt euch Meiner Gaben, die ICH euch gewährt habe, und daß ICH euch den Menschen gegenüber auszeichnete.\qt@no{47}}%
\qt@newcmd\qurantrans@de@lv{Und sucht Taqwa einem Tag gegenüber, an dem weder eine Seele einer (anderen) Seele etwas nützt, noch von ihr Fürbitte angenommen wird, noch von ihr Ausgleich genommen wird, noch wird ihnen beigestanden.\qt@no{48}}%
\qt@newcmd\qurantrans@de@lvi{Und (erinnere daran), als WIR euch vor Pharaos Leute errettet haben. Sie fügten euch das Übelste an Mißhandlung zu. Sie schlachteten eure Söhne ab und ließen (nur) eure Frauen am Leben. Und darin war eine Prüfung von eurem HERRN, die überhart war.\qt@no{49}}%
\qt@newcmd\qurantrans@de@lvii{Und (erinnere daran), als WIR für euch das Meer gespaltet haben, dann erretteten WIR euch und ließen Pharaos Leute ertrinken, während ihr zugeschaut habt.\qt@no{50}}%
\qt@newcmd\qurantrans@de@lviii{Und (erinnere daran), als WIR Musa ein Versprechen für vierzig Tage gaben, dann nahmt ihr euch das Kalb (als Götzen) in dessen (Abwesenheit), während ihr Unrecht-Begehende wart.\qt@no{51}}%
\qt@newcmd\qurantrans@de@lix{Dann vergaben WIR es euch danach, damit ihr euch dankbar erweist.\qt@no{52}}%
\qt@newcmd\qurantrans@de@lx{Und (erinnere daran), als WIR Musa die Schrift und Al-furqan zuteil werden ließen, damit ihr Rechtleitung findet,\qt@no{53}}%
\qt@newcmd\qurantrans@de@lxi{und als Musa seinen Leuten sagte: ``Meine Leute! Zweifelsohne habt ihr euch selbst durch eure Annahme des Kalbes Unrecht angetan, so bereut vor eurem Schöpfer und tötet euch selbst. Dies ist besser für euch bei eurem Schöpfer." Dann vergab ER euch. Gewiß, ER ist Der äußerst Reue-Annehmende, Der Allgnädige.\qt@no{54}}%
\qt@newcmd\qurantrans@de@lxii{Und (erinnere daran), als ihr zu Musa gesagt habt: ``Wir werden dir nicht glauben, bis wir ALLAH offenkundig sehen." Dann traf euch der Blitz, während ihr zugeschaut habt.\qt@no{55}}%
\qt@newcmd\qurantrans@de@lxiii{Dann erweckten WIR euch nach eurem Tod, damit ihr euch dankbar erweist,\qt@no{56}}%
\qt@newcmd\qurantrans@de@lxiv{und beschatteten euch mit Wolken und schickten euch Al-mann und As-salwa. Esst von den Tay-yibat dessen, was WIR euch als Rizq gewährten! Doch nicht gegen Uns begingen sie Unrecht, sondern sie pflegten gegen sich selbst Unrecht zu begehen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@lxv{Und (erinnere daran), als WIR sagten: ``Tretet ein in diese Ortschaft und esst darin allerorts, wo ihr wollt, reichlich. Und tretet ein durch das Tor Sudschudvollziehend und sagt: ``Entlastung!" Dann vergeben WIR euch eure Verfehlungen und WIR werden die Muhsin noch mehr gewinnen lassen."\qt@no{58}}%
\qt@newcmd\qurantrans@de@lxvi{Dennoch tauschten diejenigen, die Unrecht begingen, ein Wort gegen das aus, das ihnen nicht gesagt wurde, dann schickten WIR über diejenigen, die Unrecht begingen, Peinigung vom Himmel für das, was sie an Fisq zu betreiben pflegten.\qt@no{59}}%
\qt@newcmd\qurantrans@de@lxvii{Und (erinnere daran), als Musa für seine Leute um Wasser bat, dann sagten WIR: ``Schlage mit deinem Stock an den Felsen!" Dann entsprangen daraus zwölf Quellen, bereits kennt jede Gruppe ihre Trinkstelle. Esst und trinkt von ALLAHs Rizq und übertreibt nicht auf Erden als Verderben-Anrichtende!\qt@no{60}}%
\qt@newcmd\qurantrans@de@lxviii{Und (erinnere daran), als ihr gesagt habt: ``Musa! Wir werden (nur) eine einzige Speise nicht länger ertragen, so richte Bittgebet für uns an deinen HERRN, uns von dem hervorzubringen, was die Erde an Grünzeug, Gurken, Knoblauch, Linsen und Zwiebeln hervorsprießt." Er sagte: Wollt ihr etwa das Geringere gegen das Bessere tauschen?! Geht zu einem Ort herunter, denn dort gibt es sicher für euch, worum ihr gebeten habt." Und sie wurden von Demütigung und Elend erfaßt und kehrten mit Erzürnen von ALLAH zurück. Dies, weil sie ALLAHs Ayat gegenüber Kufr zu betreiben und die Propheten zu Unrecht zu töten pflegten. Dies war für das, wie sie sich widersetzten und zu übertreten pflegten.\qt@no{61}}%
\qt@newcmd\qurantrans@de@lxix{Gewiß, diejenigen, die den Iman bekundet haben, diejenigen, die Juden wurden, die Nazarener und die Sabia, wer (von ihnen) den Iman an ALLAH und den Jüngsten Tag verinnerlicht und gottgefällig Gutes getan hat, diese haben ihre Belohnung bei ihrem HERRN und weder Angst gibt es um sie, noch werden sie traurig sein.\qt@no{62}}%
\qt@newcmd\qurantrans@de@lxx{Und (erinnere daran), als WIR euer Gelöbnis (zum Gehorsam) entgegennahmen und über euch den Tur-Berg hochhoben: ``Nehmt, was WIR euch gaben, mit Nachdruck an, und entsinnt euch dessen, damit ihr Taqwa gemäß handelt."\qt@no{63}}%
\qt@newcmd\qurantrans@de@lxxi{Dann habt ihr euch danach abgewandt. Und gäbe es keine Gunst von ALLAH und keine Gnade von Ihm, wäret ihr unweigerlich von den Verlierern.\qt@no{64}}%
\qt@newcmd\qurantrans@de@lxxii{Und gewiß, bereits habt ihr diejenigen von euch gekannt, die am Sabat Übertretung begangen haben, dann sagten WIR ihnen: ``Werdet zu ausgestoßenen Affen."\qt@no{65}}%
\qt@newcmd\qurantrans@de@lxxiii{Und Wir machten sie zu einem Strafexempel für die Zeitgenossen, für die Nachwelt und zur Ermahnung für die Muttaqi.\qt@no{66}}%
\qt@newcmd\qurantrans@de@lxxiv{Und (erinnere daran), als Musa zu seinen Leuten sagte: ``Gewiß, ALLAH gebietet euch, eine Kuh zu schlachten." Sie sagten: ``Nimmst du uns etwa zum Spott?" Er sagte: ``Ich suche Schutz bei ALLAH davor, einer der Unwissenden zu sein."\qt@no{67}}%
\qt@newcmd\qurantrans@de@lxxv{Sie sagten: ``Richte Bittgebet für uns an deinen HERRN, damit ER uns verdeutlicht, wie sie ist." Er sagte: ``Ja! ER sagt: ``Gewiß, sie ist eine Kuh, die weder alt noch zu jung ist - im mittlerem Alter dazwischen." So macht, was euch geboten wurde!"\qt@no{68}}%
\qt@newcmd\qurantrans@de@lxxvi{Sie sagten: ``Richte Bittgebet für uns an deinen HERRN, damit ER uns verdeutlicht, was ihre Farbe ist." Er sagte: ``Ja! ER sagt: ``Gewiß, sie ist eine gelbe Kuh mit leuchtend reiner Farbe, sie erfreut die Betrachter."\qt@no{69}}%
\qt@newcmd\qurantrans@de@lxxvii{Sie sagten: ``Richte Bittgebet für uns an deinen HERRN, damit ER uns verdeutlicht, wie sie ist, denn die Kühe scheinen uns ja ähnlich zu sein. Und gewiß, wir werden, wenn ALLAH will, sicherlich fündig sein."\qt@no{70}}%
\qt@newcmd\qurantrans@de@lxxviii{Er sagte: ``Ja! ER sagt: ``Gewiß, sie ist eine Kuh, die weder fügsam die Erde pflügt, noch den Acker bewässert, sie ist gesund und makellos." Sie sagten: ``Nun kamst du mit der Wahrheit." Dann schlachteten sie sie, und beinahe machten sie es nicht.\qt@no{71}}%
\qt@newcmd\qurantrans@de@lxxix{Und (erinnere daran), als ihr einen Menschen getötet und euch darüber gestritten habt. Und ALLAH ist Offenleger dessen, was ihr zu verbergen pflegtet.\qt@no{72}}%
\qt@newcmd\qurantrans@de@lxxx{Dann sagten WIR: ``Beklopft ihn mit einem Teil davon!" So belebt ALLAH die Toten und zeigt euch Seine Ayat, damit ihr verständig werdet.\qt@no{73}}%
\qt@newcmd\qurantrans@de@lxxxi{Dann verhärteten sich eure Herzen danach, so wurden sie wie Steine oder noch härter. Und gewiß, von Steinen gibt es solche, aus denen Flüsse entspringen, auch davon sind sicher solche, die aufbrechen, dann sprudelt Wasser daraus, ebenso sind davon doch solche, die sich aus Ehrfurcht vor ALLAH absenken. Und ALLAH ist sicherlich nicht achtlos dem gegenüber, was ihr tut.\qt@no{74}}%
\qt@newcmd\qurantrans@de@lxxxii{Hofft ihr etwa, daß sie euch Glauben schenken, wo eine Gruppe von ihnen ALLAHs Wort zu hören und es dann (bewußt) zu verändern pflegte, nachdem sie es verstanden hatte, während sie es kannte?!\qt@no{75}}%
\qt@newcmd\qurantrans@de@lxxxiii{Und wenn sie denjenigen begegneten, die den Iman verinnerlicht haben, sagten sie: ``Wir haben den Iman verinnerlicht." Doch wenn die einen von ihnen sich mit den anderen zurückzogen, sagten sie: ``Erzählt ihr ihnen von dem, was ALLAH euch erläuterte, damit sie mit diesem gegen euch vor eurem HERRN argumentieren? Seid ihr etwa nicht verständig?!"\qt@no{76}}%
\qt@newcmd\qurantrans@de@lxxxiv{Wissen sie etwa nicht, daß ALLAH ja das kennt, was sie verheimlichen und was sie offenlegen?!\qt@no{77}}%
\qt@newcmd\qurantrans@de@lxxxv{Und unter ihnen sind Analphabeten, welche die Schrift nicht kennen, nur Wunschträume (haben sie). Und gewiß spekulieren sie ja nur.\qt@no{78}}%
\qt@newcmd\qurantrans@de@lxxxvi{Und Niedergang sei denjenigen, welche die Schrift mit eigenen Händen schreiben, dann sagen: ``Dies ist von ALLAH", um sich damit einen minderwertigen Gegenwert zu erkaufen. Niedergang sei ihnen für das, was ihre Hände geschrieben haben, und Niedergang sei ihnen für das, was sie erwarben.\qt@no{79}}%
\qt@newcmd\qurantrans@de@lxxxvii{Und sie sagten: ``Das Feuer wird uns sicher nicht berühren außer für abgezählte Tage." Sag: ``Habt ihr etwa ein Versprechen von ALLAH entgegengenommen? So bricht ALLAH gewiß Sein Versprechen nicht. Oder sagt ihr über ALLAH etwas, worüber ihr nichts wisst?"\qt@no{80}}%
\qt@newcmd\qurantrans@de@lxxxviii{Doch, mit Sicherheit! Wer eine gottmißfällige Tat erwirbt und dessen Verfehlungen ihn umgeben, so sind diese die Weggenossen des Feuers. Darin bleiben sie ewig.\qt@no{81}}%
\qt@newcmd\qurantrans@de@lxxxix{Und diejenigen, die den Iman verinnerlicht haben und gottgefällig Gutes taten, diese sind die Weggenossen der Dschanna. Darin bleiben sie ewig.\qt@no{82}}%
\qt@newcmd\qurantrans@de@xc{Und (erinnere daran), als WIR das Gelöbnis der Kinder Israils entgegennahmen: ``Ihr dient nichts außer ALLAH, erweist den Eltern Ihsan, sowie den Verwandten, den Waisen und den Bedürftigen, und sagt den Menschen Schönes, verrichtet ordnungsgemäß das rituelle Gebet und entrichtet die Zakat." Dann habt ihr euch abgewandt außer wenigen von euch, während ihr widerstrebend wart.\qt@no{83}}%
\qt@newcmd\qurantrans@de@xci{Und (erinnere daran), als WIR euer Gelöbnis entgegennahmen: ``Ihr vergießt euer Blut nicht und vertreibt euch selbst nicht aus euren Wohnstätten." Dann habt ihr es gebilligt, während ihr es bezeugt habt.\qt@no{84}}%
\qt@newcmd\qurantrans@de@xcii{Nun aber tötet ihr euch selbst und vertreibt eine Gruppe von euch aus ihren Wohnstätten. Ihr unterstützt gegen sie mit Verfehlung und Aggression. Und wenn sie in Kriegsgefangenschaft gerieten, würdet ihr für sie Lösegeld zahlen, obwohl euch ihr Vertreiben für haram erklärt wurde. Bekundet ihr etwa Iman an einen Teil der Schrift und betreibt Kufr einem (anderen) Teil gegenüber?! Und was wäre die Vergeltung für denjenigen von euch, der dies tut, außer Erniedrigung im diesseitigen Leben?! Und am Tag der Auferstehung werden sie zum Härtesten der Peinigung geführt. Und ALLAH ist gewiß nicht achtlos dem gegenüber, was ihr tut.\qt@no{85}}%
\qt@newcmd\qurantrans@de@xciii{Diese sind diejenigen, die das diesseitige Leben für das Jenseits erkauft haben. So wird ihnen weder die Peinigung erleichtert, noch wird ihnen beigestanden.\qt@no{86}}%
\qt@newcmd\qurantrans@de@xciv{Und gewiß, bereits ließen WIR Musa die Schrift zuteil werden, und ließen nach ihm Gesandte nach und nach folgen. Und WIR gaben 'Isa Ibnu-Maryam, die Wunderzeichen und unterstützten ihn mit Ruhhul-qudus. (Bleibt es dabei), daß immer wieder, wenn zu euch ein Gesandter mit dem kam, was eurem Ego nicht lieb war, ihr euch in Arroganz erhoben habt, dann eine Gruppe (der Gesandten) der Lüge bezichtigt und eine (andere) Gruppe getötet habt?!\qt@no{87}}%
\qt@newcmd\qurantrans@de@xcv{Und sie sagten: ``Unsere Herzen sind bedeckt." Nein, sondern ALLAH verfluchte sie wegen ihres Kufr, also nur ein wenig Iman verinnerlichen sie.\qt@no{88}}%
\qt@newcmd\qurantrans@de@xcvi{Und als zu ihnen eine Schrift von ALLAH kam, die bestätigend für das war, was sie schon haben, (leugneten sie ab). Doch sie pflegten vorher (damit) um Unterstützung gegen die Kafir zu bitten. Aber als zu ihnen kam, was sie kannten, haben sie dem gegenüber Kufr betrieben. ALLAHs Fluch soll auf den Kafir lasten.\qt@no{89}}%
\qt@newcmd\qurantrans@de@xcvii{Erbärmlich ist das, womit sie sich selbst verkauften, daß sie dem gegenüber Kufr betreiben, was ALLAH hinabsandte - aus Übertretung, daß ALLAH von Seiner Gunst auf denjenigen von Seinen Dienern, den ER will, nach und nach hinabsendet. So kehrten sie mit Zorn über Zorn um, und für die Kafir ist erniedrigende Peinigung bestimmt.\qt@no{90}}%
\qt@newcmd\qurantrans@de@xcviii{Und als ihnen gesagt wurde: ``Verinnerlicht den Iman an das, was ALLAH hinabsandte.", sagten sie: ``Wir verinnerlichen den Iman (nur) an das, was uns hinabgesandt wurde." Doch sie betreiben Kufr dem gegenüber, das nachfolgte, obwohl dies die Wahrheit ist, die das bestätigt, was sie bereits haben. Sag: ``Weshalb habt ihr ALLAHs Propheten vorher getötet, solltet ihr Mumin sein?!"\qt@no{91}}%
\qt@newcmd\qurantrans@de@xcix{Und gewiß, bereits kam Musa zu euch mit den Wunderzeichen, dann nahmt ihr euch in dessen (Abwesenheit) das Kalb (als Götzen), während ihr Unrecht-Begehende wart.\qt@no{92}}%
\qt@newcmd\qurantrans@de@c{Und (erinnere daran), als WIR euer Gelöbnis entgegennahmen und über euch den Tur-Berg hochhoben: ``Nehmt, was WIR euch gaben, mit Nachdruck an und hört zu!" Sie sagten: ``Wir haben zugehört und uns widersetzt." So wurde in ihren Herzen (die Zuneigung) zum Kalb wegen ihres Kufr eingeprägt. Sag: ``Erbärmlich ist das, was euch euer Iman gebietet, solltet ihr Mumin sein."\qt@no{93}}%
\qt@newcmd\qurantrans@de@ci{Sag: ``Sollte euch die jenseitige Wohnstätte bei ALLAH alleine - unter Ausschluß aller (anderen) Menschen gehören, so wünscht euch den Tod, solltet ihr wahrhaftig sein."\qt@no{94}}%
\qt@newcmd\qurantrans@de@cii{Doch sie werden sich ihn niemals wünschen wegen dem, was sie bereits angerichtet haben. Und ALLAH ist allwissend über die Unrecht-Begehenden.\qt@no{95}}%
\qt@newcmd\qurantrans@de@ciii{Und mit Sicherheit wirst du sie mehr am Leben hängend als alleMenschen finden, und ebenso diejenigen, die Schirk betrieben haben. Jeder von ihnen wünscht sich, er würde tausend Jahre leben. Doch es wird ihn von der Peinigung keineswegs verschonen, daß er lange lebt. Und ALLAH ist allsehend dessen, was sie tun.\qt@no{96}}%
\qt@newcmd\qurantrans@de@civ{Sag: ``Wer ein Feind von Dschibril ist, (der soll es lassen), denn er war es zweifelsohne, der ihn (den Quran) auf dein Herz mit Zustimmung von ALLAH nach und nach hinabsandte als Bestätigung dessen, was vor ihm war, als Rechtleitung und als frohe Botschaft für die Mumin."\qt@no{97}}%
\qt@newcmd\qurantrans@de@cv{Wer ein Feind für ALLAH, Seine Engel, Seine Gesandten, Dschibril und Mikail ist, (der soll es lassen), denn zweifelsohne, ALLAH ist ein Feind für die Kafir.\qt@no{98}}%
\qt@newcmd\qurantrans@de@cvi{Und gewiß, bereits sandten WIR dir deutliche Ayat hinab. Und Kufr denen gegenüber betreibt niemand außer den Fasiq.\qt@no{99}}%
\qt@newcmd\qurantrans@de@cvii{Ist es nicht so, daß immer wieder, wenn sie einen Vertrag abschließen, eine Gruppe von ihnen ihn verletzt?! Nein, sondern die meisten von ihnen verinnerlichen den Iman nicht.\qt@no{100}}%
\qt@newcmd\qurantrans@de@cviii{Und als zu ihnen ein Gesandter von ALLAH kam bestätigend dessen, was bei ihnen ist, warf eine Gruppe von denjenigen, denen die Schrift zuteil wurde, ALLAHs Schrift hinter ihren Rücken, als wüßten sie nichts.\qt@no{101}}%
\qt@newcmd\qurantrans@de@cix{Und sie folgten dem, was die Satane über Sulaimans Herrschaft verbreitet haben. Und nie hat Sulaiman Kufr betrieben, doch die Satane haben Kufr betrieben. Sie lehrten die Menschen die Magie und das, was auf die beiden Engel in Babylon Harut und Marut hinabgesandt wurde. Und beide lehrten niemanden, bis sie sagten: ``Wir sind doch nur eine Fitna, betreibe also keinen Kufr!" So lernten sie von ihnen das, womit sie zwischen dem Mann und seiner Ehepartnerin scheiden. Doch sie schaden damit niemandem, es sei denn mit Zustimmung von ALLAH. Und sie lernen, was ihnen schadet und nicht nützt. Und gewiß, bereits wußten sie, daß jeder, der dies sich aneignet, am Jenseits keinerlei Anteil hat. Gewiß, erbärmlich ist das, womit sie sich verkauften, würden sie es nur wissen.\qt@no{102}}%
\qt@newcmd\qurantrans@de@cx{Und hätten sie doch Iman verinnerlicht und Taqwa gemäß gehandelt, gewiß wäre eine Belohnung von ALLAH besser, würden sie es nur wissen.\qt@no{103}}%
\qt@newcmd\qurantrans@de@cxi{Ihr, die den Iman verinnerlicht habt! Sagt nicht ra'ina, sondern sagt unzurna und hört zu! Und für die Kafir ist eine qualvolle Peinigung bestimmt.\qt@no{104}}%
\qt@newcmd\qurantrans@de@cxii{Weder diejenigen unter den Schriftbesitzern, die Kufr betrieben haben, noch die Muschrik wünschen sich, daß euch etwas Gutes von eurem HERRN nach und nach hinabgesandt wird. Doch ALLAH gewährt Gnade, wem er will. Und ALLAH ist von unermeßlicher Gunst.\qt@no{105}}%
\qt@newcmd\qurantrans@de@cxiii{Jede Aya, die WIR Naskh unterziehen oder vergessen lassen, ersetzen WIR durch etwas Besseres als sie oder durch Gleichwertiges. Weißt du etwa nicht, daß ALLAH ja über alles allmächtig ist?!\qt@no{106}}%
\qt@newcmd\qurantrans@de@cxiv{Weißt du etwa nicht, daß ALLAH doch die Herrschaft der Himmel und der Erde gehört, und daß es für euch anstelle von ALLAH weder Wali noch Beistehenden gibt?!\qt@no{107}}%
\qt@newcmd\qurantrans@de@cxv{Oder wollt ihr etwa von eurem Gesandten Gleiches verlangen, wie vorher von Musa verlangt wurde?! Und wer den Kufr gegen den Iman eintauscht, der verfehlte bereits den geradlinigen Weg.\qt@no{108}}%
\qt@newcmd\qurantrans@de@cxvi{Viele von den Schriftbesitzern wünschten sich, sie könnten euch nach eurem Iman-Bekenntnis wieder zu Kafir umwandeln aus Neid in ihren Seelen, nachdem ihnen die Wahrheit sichtbar wurde. So erlaßt und seht nach, bis ALLAH Seine Anweisung kommen läßt. Gewiß, ALLAH ist über alles allmächtig.\qt@no{109}}%
\qt@newcmd\qurantrans@de@cxvii{Und verrichtet ordnungsgemäß das rituelle Gebet und entrichtet die Zakat. Und was ihr für euch an Gutem vorlegt, werdet ihr bei ALLAH vorfinden. Gewiß, ALLAH ist dessen, was ihr tut, allsehend.\qt@no{110}}%
\qt@newcmd\qurantrans@de@cxviii{Und schon sagten sie: ``Die Dschanna wird keiner betreten außer demjenigen, der Jude oder Nazarener war." - diese sind ihre Wunschvorstellungen - Sag: ``Bringt euren Beweis, solltet ihr wahrhaftig sein."\qt@no{111}}%
\qt@newcmd\qurantrans@de@cxix{Doch, mit Sicherheit! Wer sich ALLAH ergibt, während er Muhsin ist, dieser hat seine Belohnung bei seinem HERRN, und um sie gibt es weder Angst, noch werden sie traurig sein.\qt@no{112}}%
\qt@newcmd\qurantrans@de@cxx{Und die Juden sagten: ``Die Nazarener verfügen über nichts Nennenswertes.", und die Nazarener sagten: ``Die Juden verfügen über nichts Nennenswertes.", obwohl beide die Schrift vortragen. Solcherart sprachen diejenigen, die nicht wissen, Gleiches wie ihre Worte. So wird ALLAH am Tag der Auferstehung zwischen ihnen richten, worüber sie uneins zu sein pflegten.\qt@no{113}}%
\qt@newcmd\qurantrans@de@cxxi{Und wer ist unrecht-begehender als diejenigen, die in ALLAHs Moschee verbieten, daß Seines Namens darin gedacht wird, und sich um ihre Zerstörung bemühen?! Diesen gebührt es nicht, sie zu betreten, außer angsterfüllt. Für sie gibt es im Diesseits Erniedrigung, und im Jenseits ist für sie überharte Peinigung bestimmt.\qt@no{114}}%
\qt@newcmd\qurantrans@de@cxxii{Und ALLAH gehört der Westen und der Osten. Allerorts, von wo ihr euch hinwendet, ist die von ALLAH gebotene Richtung. Gewiß, ALLAH ist allumfassend, allwissend.\qt@no{115}}%
\qt@newcmd\qurantrans@de@cxxiii{Und schon sagten sie: ``ALLAH nahm sich Kinder". Gepriesen-erhaben ist ER! Nein, sondern Ihm gehört, was in Himmeln und was auf Erden ist. Alle sind Ihm unterwürfig.\qt@no{116}}%
\qt@newcmd\qurantrans@de@cxxiv{ER ist Der Erschaffer der Himmel und der Erde. Und wenn ER eine Angelegenheit bestimmte, so sagt ER ihr nur: ``Sei!", sogleich ist sie.\qt@no{117}}%
\qt@newcmd\qurantrans@de@cxxv{Und diejenigen, die nicht wissen, sagten: ``Würde ALLAH zu uns doch nur sprechen oder käme zu uns doch nur eine Aya!" Solcherart sprachen diejenigen vor ihnen Gleiches wie ihre Worte, ihre Herzen sind sich ähnlich. Bereits erläuterten WIR die Ayat für Leute, die Gewißheit haben.\qt@no{118}}%
\qt@newcmd\qurantrans@de@cxxvi{Gewiß, WIR entsandten dich mit der Wahrheit als Überbringer froher Botschaft und als Ermahner. Und du wirst nicht für die Weggenossen der Hölle zur Rechenschaft gezogen.\qt@no{119}}%
\qt@newcmd\qurantrans@de@cxxvii{Und weder die Juden werden Wohlgefallen an dir finden noch die Nazarener, bis du ihrer Gemeinschaft folgst. Sag: ``ALLAHs Rechtleitung ist die wahre Rechtleitung". Und solltest du ihren Neigungen folgen nach dem Wissen, das zu dir kam, vor ALLAH hast du weder Wali noch Beistehenden.\qt@no{120}}%
\qt@newcmd\qurantrans@de@cxxviii{Diejenigen, denen WIR die Schrift zuteil werden ließen, tragen sie vor, wie es ihr gebührt. Diese verinnerlichen den Iman daran. Und wer ihr gegenüber Kufr betreibt, so sind diese die wirklichen Verlierer.\qt@no{121}}%
\qt@newcmd\qurantrans@de@cxxix{Kinder Israils! Entsinnt euch Meiner Gaben, die ICH euch gewährte, und daß ICH euch den Menschen gegenüber auszeichnete.\qt@no{122}}%
\qt@newcmd\qurantrans@de@cxxx{Und sucht Taqwa einem Tag gegenüber, an dem weder eine Seele einer Seele etwas nützt, noch von ihr Ausgleich angenommen wird, noch ihr Fürbitte nützt, noch wird ihnen beigestanden.\qt@no{123}}%
\qt@newcmd\qurantrans@de@cxxxi{Und (erinnere daran), als Ibrahim von seinem HERRN mit Worten geprüft wurde, und er sie dann einhielt. ER sagte: ``Gewiß, ICH mache dich zum Imam für die Menschen." Er sagte: ``Auch von meiner Nachkommenschaft?" ER sagte: ``Das von Mir Versprochene umfaßt nicht die Unrecht-Begehenden."\qt@no{124}}%
\qt@newcmd\qurantrans@de@cxxxii{Und (erinnere daran), als WIR die Ka'ba zu einem immer wieder aufgesuchten Ort für die Menschen machten, und zu einem sicheren Ort. Und nehmt euch Maqamu-ibrahim als Gebetsstätte! Und WIR erlegten Ibrahim und Isma'il auf: ``Reinigt Mein Haus für die Tawaf -, die I'tikaf - und die Ruku'-Sudschud- Vollziehenden!"\qt@no{125}}%
\qt@newcmd\qurantrans@de@cxxxiii{Und (erinnere daran), als Ibrahim sagte: ``Mein HERR! Mache diesen zu einem sicheren Ort und gewähre seinen Bewohnern Rizq von den Erzeugnissen, wem von ihnen den Iman an ALLAH und an den Jüngsten Tag verinnerlicht hat." ER sagte: ``Und wer Kufr betreibt, den werde ICH ein wenig vergnügen lassen, dann zwinge ICH ihn zur Peinigung des Feuers." Und erbärmlich ist das Werden.\qt@no{126}}%
\qt@newcmd\qurantrans@de@cxxxiv{Und (erinnere daran), als Ibrahim die Grundmauer des Hauses errichtete und Isma'il: ``Unser HERR! Nimm von uns an. Gewiß, DU bist ja Der Allhörende, Der Allwissende.\qt@no{127}}%
\qt@newcmd\qurantrans@de@cxxxv{HERR! Mache uns zu zwei Muslimen, lasse aus unserer Nachkommenschaft eine Dir gegenüber islam-praktizierende Umma entstehen, zeige uns unsere Riten und vergib uns! Gewiß, DU bist ja Der Allvergebende, Der Allgnädige.\qt@no{128}}%
\qt@newcmd\qurantrans@de@cxxxvi{Unser HERR! Und lasse unter ihnen einen Gesandten von ihnen entstehen, der ihnen Deine Ayat vorträgt, sie die Schrift und die Weisheit lehrt und sie läutert! Gewiß, DU bist ja Der Allwürdige, Der Allweise."\qt@no{129}}%
\qt@newcmd\qurantrans@de@cxxxvii{Und wer kann der Gemeinschaft Ibrahims gegenüber abgeneigt sein außer demjenigen, der sich selbst geringschätzt?! Und gewiß, bereits erwählten WIR ihn im Diesseits aus. Und gewiß, er ist im Jenseits zweifelsohne unter den gottgefällig Guttuenden.\qt@no{130}}%
\qt@newcmd\qurantrans@de@cxxxviii{Und (erinnere daran), als sein HERR ihm sagte: ``Praktiziere den Islam!" Er sagte: ``Ich praktiziere den Islam Dem HERRN aller Geschöpfe."\qt@no{131}}%
\qt@newcmd\qurantrans@de@cxxxix{Und dies hinterließ Ibrahim seinen Kindern als Vermächtnis, sowie Ya'qub: ``Meine Kinder! ALLAH hat euch doch den Din auserwählt, so sterbt nur als Muslime!"\qt@no{132}}%
\qt@newcmd\qurantrans@de@cxl{Oder wart ihr etwa Zeugen, als der Tod bei Ya'qub zugegen wurde, als er zu seinen Söhnen sagte: ``Wem werdet ihr nach mir dienen?" Sie sagten: ``Wir werden weiterhin deinem Gott und dem Gott deiner Väter Ibrahim, Isma'il und Ishaq dienen, einem einzigen Gott, und wir sind Ihm gegenüber Muslime."\qt@no{133}}%
\qt@newcmd\qurantrans@de@cxli{Diese war eine Umma, die bereits verging. Ihr gebührt, was sie erworben hat, und euch gebührt, was ihr erworben habt, und ihr werdet nicht zur Rechenschaft gezogen für das, was sie zu tun pflegten.\qt@no{134}}%
\qt@newcmd\qurantrans@de@cxlii{Und sie sagten: ``Werdet Juden oder Nazarener, werdet ihr Rechtleitung finden." Sag: ``Nein, sondern wir folgen der Gemeinschaft Ibrahims, der hanif war, und er war nicht von den Muschrik."\qt@no{135}}%
\qt@newcmd\qurantrans@de@cxliii{Sagt: ``Wir verinnerlichen den Iman an ALLAH, an das, was uns hinabgesandt wurde, an das, was Ibrahim, Isma'il, Ishaq, Ya'qub und den Israil-Stämmen hinabgesandt wurde, an das, was Musa zuteil wurde sowie 'Isa, und an das, was den Propheten von ihrem HERRN zuteil wurde. Wir machen keinen Unterschied zwischen irgendeinem von ihnen und wir sind Ihm gegenüber Muslime. -\qt@no{136}}%
\qt@newcmd\qurantrans@de@cxliv{Und wenn sie den Iman an das Gleiche verinnerlichen, woran ihr den Iman verinnerlicht habt, so haben sie bereits Rechtleitung gefunden. Und wenn sie sich abwenden, so sind sie doch nur im Widerspruch, und ALLAH wird dich vor ihnen schützen. Und ER ist Der Allhörende, Der Allwissende. -\qt@no{137}}%
\qt@newcmd\qurantrans@de@cxlv{Dies ist ALLAHs Prägung. Und wer verfügt über eine schönere Prägung als ALLAH?! Und wir sind Ihm gegenüber Dienende."\qt@no{138}}%
\qt@newcmd\qurantrans@de@cxlvi{Sagt: ``Wollt ihr mit uns über ALLAH disputieren, wo ER unser und euer HERR ist, und wo uns unsere Taten und euch eure Taten gehören, und wo wir Ihm gegenüber aufrichtig sind?!"\qt@no{139}}%
\qt@newcmd\qurantrans@de@cxlvii{Oder sagt ihr etwa, daß Ibrahim, Isma'il, Ya'qub und die Israil-Stämme Juden oder Nazarener waren? Sag: ``Wisst ihr besser oder ALLAH?" Und wer ist unrecht-begehender als derjenige, der ein Zeugnis bei sich von ALLAH verschweigt?! Und ALLAH ist nicht achtlos dem gegenüber, was ihr tut.\qt@no{140}}%
\qt@newcmd\qurantrans@de@cxlviii{Diese war eine Umma, die bereits verging. Ihr gebührt, was sie erworben hat, und euch gebührt, was ihr erworben habt, und ihr werdet nicht zur Rechenschaft gezogen für das, was sie zu tun pflegten.\qt@no{141}}%
\qt@newcmd\qurantrans@de@cxlix{Die Beschränkten unter den Menschen werden sagen: ``Was brachte sie von ihrer Gebetsrichtung ab, die sie vorher hatten?" Sag: ``ALLAH gehört der Osten und der Westen. ER leitet recht, wen ER will, auf einen geradlinigen Weg."\qt@no{142}}%
\qt@newcmd\qurantrans@de@cl{Und solcherart machten WIR euch zu einer gemäßigten Umma, damit ihr Zeugen über die Menschen seid, und der Gesandte über euch Zeuge ist. Und nicht machten WIR die Gebetsrichtung, die du vorher hattest, außer damit WIR kenntlich machen denjenigen, der dem Gesandten folgt, von demjenigen, der sich um seine Fersen wendet. Und es ist gewiß etwas Schwerwiegendes außer für diejenigen, die ALLAH rechtleitete. Und es gebührt ALLAH nicht, daß ER euren Iman (euer rituelles Gebet) verloren gehen läßt. Gewiß, ALLAH ist den Menschen gegenüber zweifelsohne allgütig, allgnädig.\qt@no{143}}%
\qt@newcmd\qurantrans@de@cli{Bereits sehen WIR das Schweifen deines Gesichts himmelwärts. So werden WIR dich gewiß eine Gebetsrichtung einnehmen lassen, an der du Wohlgefallen findest. So wende dich mit dem Gesicht in Richtung von Almasdschidil-haram, und allerorts, wo ihr seid, wendet euch mit dem Gesicht in seine Richtung! Auch sicher, diejenigen, denen die Schrift zuteil wurde, wissen zweifellos, daß es doch die Wahrheit von ihrem HERRN ist. Und ALLAH ist gewiß nicht achtlos dem gegenüber, was ihr tut.\qt@no{144}}%
\qt@newcmd\qurantrans@de@clii{Und würdest du denjenigen, denen die Schrift zuteil wurde, alle Ayat bringen, so werden sie keineswegs deine Gebetsrichtung befolgen. Auch du bist kein Befolger ihrer Gebetsrichtung. Auch die einen von ihnen befolgen nicht die Gebetsrichtung der anderen. Und würdest du ihren Neigungen folgen nach dem, was zu dir an Wissen kam, gewiß würdest du dann unweigerlich von den Unrecht- Begehenden sein.\qt@no{145}}%
\qt@newcmd\qurantrans@de@cliii{Diejenigen, denen WIR die Schrift zuteil werden ließen, kennen es/ihn, wie sie ihre Kinder kennen. Doch eine Gruppe von ihnen verschweigt die Wahrheit, während sie weiß.\qt@no{146}}%
\qt@newcmd\qurantrans@de@cliv{Das ist die Wahrheit von deinem HERRN. Und sei nicht von den Zweifel-Hegenden!\qt@no{147}}%
\qt@newcmd\qurantrans@de@clv{Und für jeden gibt es eine Richtung, die er einnimmt. So wetteifert um die gottgefällig guten Taten! Allerorts, wo auch immer ihr euch befindet, wird ALLAH euch zusammenbringen, allesamt. Gewiß, ALLAH ist über alles allmächtig.\qt@no{148}}%
\qt@newcmd\qurantrans@de@clvi{Und wohin auch immer du gehst, wende dich mit dem Gesicht in Richtung von Almasdschidil-haram. Und gewiß, dies ist zweifelsohne die Wahrheit von deinem HERRN. Und ALLAH ist nicht achtlos dem gegenüber, was ihr tut.\qt@no{149}}%
\qt@newcmd\qurantrans@de@clvii{Und wohin auch immer du gehst, wende dich mit dem Gesicht in Richtung von Almasdschidil-haram, und allerorts, wo auch immer ihr euch befindet, wendet euch mit dem Gesicht in Richtung von Almasdschidil-haram, damit die Menschen gegen euch kein Argument haben, außer denjenigen von ihnen, die Unrecht begehen, so habt keine Ehrfurcht vor ihnen, und habt Ehrfurcht vor Mir! Zweifelsohne werde ICH euch dann Meine Gabe vervollständigen - damit ihr Rechtleitung findet -\qt@no{150}}%
\qt@newcmd\qurantrans@de@clviii{wie WIR zu euch einen Gesandten von euch entsandten, der euch Unsere Ayat vorträgt, euch läutert, euch die Schrift und die Weisheit lehrt und euch das lehrt, was ihr vorher nicht wußtet.\qt@no{151}}%
\qt@newcmd\qurantrans@de@clix{Also gedenkt Meiner, bedenke ICH euch, erweist euch Mir gegenüber dankbar und betreibt Mir gegenüber keinen Kufr.\qt@no{152}}%
\qt@newcmd\qurantrans@de@clx{Ihr, die den Iman verinnerlicht habt! Helft euch mit Geduld und mit dem rituellen Gebet. Gewiß, ALLAH ist mit den Duldsamen.\qt@no{153}}%
\qt@newcmd\qurantrans@de@clxi{Und sagt nicht über diejenigen, die fi-sabilillah getötet werden: ``Sie seien Verstorbene." Nein, sondern sie sind Lebendige. Doch ihr merkt es nicht.\qt@no{154}}%
\qt@newcmd\qurantrans@de@clxii{Und unweigerlich werden WIR euch mit etwas Angst, Hunger und Rückgang von Vermögen, Bevölkerung und Erzeugnissen prüfen. Und überbringe den Duldsamen frohe Botschaft -\qt@no{155}}%
\qt@newcmd\qurantrans@de@clxiii{die, wenn sie Unglück trifft, sagen: ``inna lilahi wa inna ilaihi radschi'un."\qt@no{156}}%
\qt@newcmd\qurantrans@de@clxiv{Für diese gibt es von ihrem HERRN Vergebungen und Gnade, und diese sind die wirklichen Geleiteten.\qt@no{157}}%
\qt@newcmd\qurantrans@de@clxv{Gewiß, As-safa und Al-marwa gehören zu den von ALLAH (gebotenen) Riten. Wer zum Al-bait mit Hadsch oder 'Umra pilgert, für den ist es keine Verfehlung, wenn er zwischen beiden hin- und herläuft. Und wer Freiwilliges tut, so ist ALLAH gewiß reichlich belohnend, allwissend.\qt@no{158}}%
\qt@newcmd\qurantrans@de@clxvi{Gewiß, diejenigen, die das verschweigen, das WIR von den klaren Zeichen und der Rechtleitung hinabsandten, nachdem WIR es den Menschen in der Schrift verdeutlicht haben, diese verflucht ALLAH und verfluchen die Verfluchenden.\qt@no{159}}%
\qt@newcmd\qurantrans@de@clxvii{Ausgenommen sind diejenigen, die bereut, Wiedergutmachung geleistet und es verdeutlicht haben, denn diesen vergebe ICH. Und ICH bin Der äußerst Reue- Annehmende, Der Allgnädige.\qt@no{160}}%
\qt@newcmd\qurantrans@de@clxviii{Gewiß, diejenigen, die Kufr betrieben haben und als Kafir gestorben sind, auf diesen lastet der Fluch von ALLAH, den Engeln und den Menschen, allesamt.\qt@no{161}}%
\qt@newcmd\qurantrans@de@clxix{Ewig werden sie darin sein, weder wird ihnen die Peinigung erleichtert, noch wird ihnen Verschnaufpause gewährt.\qt@no{162}}%
\qt@newcmd\qurantrans@de@clxx{Und euer Gott ist ein einziger Gott, es gibt keinen Gott außer Ihm, Dem Allgnade Erweisenden, Dem Allgnädigen.\qt@no{163}}%
\qt@newcmd\qurantrans@de@clxxi{Gewiß, in der Erschaffung der Himmel und der Erde, in der Aufeinanderfolge der Nacht und des Tages, in den Schiffen, die über das Meer mit Nützlichem für die Menschen ziehen, in dem, was ALLAH vom Himmel an Wasser fallen ließ, mit dem ER dann die Landschaft nach ihrem Tod wieder belebte und dann auf ihr jedes sich bewegendes Lebewesen ausbreiten ließ, in der Strömung der Winde und in den gratis fügsam gemachten Wolken zwischen dem Himmel und der Erde, sind zweifelsohne Ayat für die Leute, die begreifen.\qt@no{164}}%
\qt@newcmd\qurantrans@de@clxxii{Und unter den Menschen sind einige, die sich anstelle von ALLAH etwas als Ebenbürtiges nehmen. Sie empfinden für sie Liebe wie die Liebe zu ALLAH. Doch diejenigen, die den Iman verinnerlicht haben, lieben ALLAH noch viel mehr. Und würden diejenigen, die Unrecht begehen, nur einsehen - wenn sie die Peinigung erleben, daß gewiß die Macht allesamt ALLAH gehört und daß ALLAH doch hart im Peinigen ist, (werden sie bereuen),\qt@no{165}}%
\qt@newcmd\qurantrans@de@clxxiii{wenn diejenigen, denen gefolgt wurde, sich von denjenigen, die ihnen folgten, lossagen. Und sie werden die Peinigung erleben und die Verbindungen zwischen ihnen werden abbrechen.\qt@no{166}}%
\qt@newcmd\qurantrans@de@clxxiv{Und diejenigen, die folgten, sagten: ``Hätten wir doch nur eine Rückkehr gehabt, damit wir uns von ihnen lossagen, wie sie sich von uns los- sagten." Solcherart läßt ALLAH sie ihre Taten als Selbstanklagen wahrnehmen. Und sie werden das Feuer nie verlassen.\qt@no{167}}%
\qt@newcmd\qurantrans@de@clxxv{Ihr Menschen! Esst von dem, was auf Erden ist, an genießbaren Halalmitteln, und folgt nicht den Fußstapfen des Satans! Gewiß, er ist für euch ein entschiedener Feind.\qt@no{168}}%
\qt@newcmd\qurantrans@de@clxxvi{Er gebietet euch doch nur Schlechtes und Abscheuliches und daß ihr über ALLAH sagt, was ihr nicht wisst.\qt@no{169}}%
\qt@newcmd\qurantrans@de@clxxvii{Und wenn ihnen gesagt wurde: ``Folgt dem, was ALLAH hinabsandte." sagen sie: ``Nein, sondern wir folgen dem, was wir bereits bei unseren Ahnen fanden." Auch dann, wenn ihre Ahnen sich nichts besannen und keine Rechtleitung fanden?!\qt@no{170}}%
\qt@newcmd\qurantrans@de@clxxviii{So ist das Gleichnis derjenigen, die Kufr betrieben haben, wie das Gleichnis desjenigen, der etwas anschreit, das nichts außer Rufen und Schreien vernimmt. Sie sind taub, stumm und blind, also sie begreifen nicht.\qt@no{171}}%
\qt@newcmd\qurantrans@de@clxxix{Ihr, die den Iman verinnerlicht habt! Verzehrt von den Tay-yibat, dieWIR euch als Rizq gewährt haben, und erweist euch ALLAH gegenüber dankbar, solltet ihr Ihm zu dienen pflegen.\qt@no{172}}%
\qt@newcmd\qurantrans@de@clxxx{Für haram erklärte ER euch doch nur das Verendete, das (vergossene) Blut, das Schweinefleisch und das, worauf (bei der Schächtung) andere als ALLAH angerufen wurden. Wer jedoch dazu gezwungen wird  ohne dabei erstrebend oder übertretend zu sein, für den ist es keine Verfehlung. Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{173}}%
\qt@newcmd\qurantrans@de@clxxxi{Gewiß, diejenigen, die das verschweigen, was ALLAH von der Schrift hinabsandte, und damit einen minderwertigen Gegenwert erkaufen, diese verzehren in ihren Bäuchen nichts anderes außer dem Feuer. Und ALLAH wird weder zu ihnen am Tag der Auferstehung sprechen, noch sie läutern. Und für sie ist qualvolle Peinigung bestimmt.\qt@no{174}}%
\qt@newcmd\qurantrans@de@clxxxii{Diese sind diejenigen, die das Abirren gegen die Rechtleitung eingetauscht haben, sowie die Peinigung gegen die Vergebung. Also erstaunlich wird ihre Geduld mit dem Feuer sein.\qt@no{175}}%
\qt@newcmd\qurantrans@de@clxxxiii{Dies, weil ALLAH die Schrift mit der Wahrheit nach und nach hinabsandte. Doch diejenigen, die über die Schrift uneins wurden, befinden sich zweifelsohne in tiefer Zerstrittenheit.\qt@no{176}}%
\qt@newcmd\qurantrans@de@clxxxiv{Das gottgefällige Handeln liegt nicht darin, daß ihr euch mit dem Gesicht in Richtung des Ostens und des Westens wendet, sondern das gottgefällige Handeln ist, daß man den Iman an ALLAH, an den Jüngsten Tag, an die Engel, an die Schrift und an die Propheten verinnerlicht sowie das Vermögen - trotz der Liebe dazu - den Verwandten, den Waisen, den Bedürftigen, dem in Not geratenen Reisenden, den Bittenden und für die Unfreien gibt und das rituelle Gebet ordnungsgemäß verrichtet und die Zakat entrichtet, (dazu gehören) auch die Einhaltenden ihrer Verträge, wenn sie solche abschließen, und die Duldsamen in bitterer Armut, in Krankheit und beim Gewalt-Antun, denn diese sind diejenigen, die wahrhaftig sind. Und diese sind die wirklichen Muttaqi.\qt@no{177}}%
\qt@newcmd\qurantrans@de@clxxxv{Ihr, die den Iman verinnerlicht habt! Euch wurde Qisas für die Getöteten geboten: ``Der Freie für den Freien, der Sklave für den Sklaven und das Weibliche für das Weibliche." Und wem von den (Konsequenzen für die Tötung) seines Bruders etwas erlassen wird, dann gilt die Forderung (der Entschädigungszahlung) nach dem Gebilligten und die Bezahlung an ihn Ihsan gemäß. Dies ist Erleichterung von eurem HERRN und Gnade. Also wer danach übertritt, für den ist qualvolle Peinigung bestimmt.\qt@no{178}}%
\qt@newcmd\qurantrans@de@clxxxvi{Und für euch gibt es durch Qisas Leben, ihr mit Verstand!, damit ihr Taqwa gemäß handelt.\qt@no{179}}%
\qt@newcmd\qurantrans@de@clxxxvii{Euch wurde geboten - wenn der Tod bei einem von euch zugegen wird, wenn er Gut hinterläßt, das Vermächtnis für die Eltern und die Nahverwandten nach dem Gebilligten, es ist eine Pflicht für die Muttaqi.\qt@no{180}}%
\qt@newcmd\qurantrans@de@clxxxviii{Wer dieses (Vermächtnis) verändert, nachdem er es vernommen hat, so lastet deren Verfehlung doch nur auf denjenigen, die es verändern. Gewiß, ALLAH ist allhörend, allwissend.\qt@no{181}}%
\qt@newcmd\qurantrans@de@clxxxix{Also wer vom Testator fahrlässige oder vorsätzliche Verfehlung fürchtete, dann unter ihnen (den Erben) dies verbessert, so trifft ihn keine Verfehlung. Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{182}}%
\qt@newcmd\qurantrans@de@cxc{Ihr, die den Iman verinnerlicht habt! Das Siyam (rituelle Fasten) wurde euch geboten, wie es denjenigen vor euch geboten wurde, damit ihr Taqwa gemäß handelt.\qt@no{183}}%
\qt@newcmd\qurantrans@de@cxci{Es sind abgezählte Tage. Wer jedoch von euch krank oder auf Reisen war, so (holt er nach die) gleiche Anzahl an anderen Tagen. Und für diejenigen, denen es schwer fällt, ist dessen Ersatz die Speisung eines Bedürftigen. Und wer mehr Gutes freiwillig gibt, so ist dies besser für ihn. Und wenn ihr fastet, ist es besser für euch, würdet ihr es nur wissen.\qt@no{184}}%
\qt@newcmd\qurantrans@de@cxcii{Es ist der Monat Ramadan, in dem der Quran hinabgesandt wurde als Rechtleitung für die Menschen und als deutliches Zeichen von der Rechtleitung und Alfurqan. Wer von euch diesen Monat erlebt, so soll er darin Siyam vollziehen! Und wer krank oder auf Reisen war, (fastet) an anderen Tagen die gleiche Anzahl. ALLAH will für euch Erleichterung, doch ER will für euch keine Erschwernis. Und (ER gebot es euch), damit ihr die Anzahl (der Siyam-Tage) vervollständigt, ALLAH mit ``allahu-akbar" für das rühmt, was ER euch an Rechtleitung zukommen ließ, und ihr euch dankbar erweist.\qt@no{185}}%
\qt@newcmd\qurantrans@de@cxciii{Und wenn dich Meine Diener nach Mir fragen: ``ICH bin gewiß nahe, erhöre das Bittgebet des Bittenden, wenn er an Mich Bittgebete richtet, so sollen Sie Mir folgen und den Iman an Mich verinnerlichen, damit sie das Wahre treffen."\qt@no{186}}%
\qt@newcmd\qurantrans@de@cxciv{Für halal wurde euch erklärt, in der Nacht (während) der Siyam-Zeit mit euren Ehefrauen intim zu sein. Sie sind (wie) eine Bekleidung für euch und ihr seid (wie) eine Bekleidung für sie. ALLAH wußte, daß ihr euch selbst gegenüber untreu wart, so nahm ER eure Reue an und erließ es euch. Also jetzt verkehrt mit ihnen und erstrebt das, was ALLAH euch geboten hat. Und esst und trinkt, bis für euch der weiße Faden (der Dämmerung) vom schwarzen Faden (der Nacht) unterscheidbar wird, dann vollendet das Siyam bis zur Nacht! Und werdet nicht intim mit ihnen, während ihr I'tikaf in den Moscheen vollzieht. Dies sind ALLAHs Richtlinien, so kommt ihnen nicht nahe! Solcherart verdeutlicht ALLAH den Menschen Seine Ayat, damit sie Taqwa gemäß handeln.\qt@no{187}}%
\qt@newcmd\qurantrans@de@cxcv{Und eignet euch euer zwischen einander (vorhandenes) Vermögen nicht durch das als nichtig Erklärte an und bringt dieses nicht vor den Richtenden, um euch damit einen Teil des Vermögens von Leuten durch Sündhaftes anzueignen, während ihr es wisst.\qt@no{188}}%
\qt@newcmd\qurantrans@de@cxcvi{Sie fragen dich nach den Neumonden. Sag: ``Diese sind Zeitmarkierungen für die Menschen und für die Hadsch." Und das gottgefällige Handeln liegt nicht darin, daß ihr die Häuser von der Rückseite betretet, sondern das gottgefällige Handeln ist (das gottgefällige Handeln dessen), der Taqwa gemäß handelt. Und betretet die Häuser durch ihre Eingangstüren! Und handelt Taqwa gemäß ALLAH gegenüber, damit ihr erfolgreich werdet.\qt@no{189}}%
\qt@newcmd\qurantrans@de@cxcvii{Und führt den bewaffneten Kampf fi-sabilillah gegen diejenigen, die gegen euch den bewaffneten Kampf führen, doch begeht keine Aggression! Denn gewiß, ALLAH liebt nicht die Aggressoren.\qt@no{190}}%
\qt@newcmd\qurantrans@de@cxcviii{Und tötet sie überall, wo ihr sie ausfindig macht, und vertreibt sie, von wo sie euch vertrieben haben. Und die Fitna ist härter als das Töten. Und führt gegen sie den bewaffneten Kampf nicht bei Almasdschidil- haram, bis sie gegen euch darin den bewaffneten Kampf aufnehmen. Und wenn sie den bewaffneten Kampf gegen euch aufnahmen, dann tötet sie. Solcherart ist die Vergeltung für die Kafir.\qt@no{191}}%
\qt@newcmd\qurantrans@de@cxcix{Und wenn sie sich (der Aggression) enthalten, also gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{192}}%
\qt@newcmd\qurantrans@de@cc{Und führt gegen sie den bewaffneten Kampf, damit es keine Fitna gibt und der Din (nur) für ALLAH (praktiziert wird). Und wenn sie sich (der Aggression) enthalten, dann gibt es keine Aggression außer gegen die Unrecht-Begehenden.\qt@no{193}}%
\qt@newcmd\qurantrans@de@cci{Nichtbeachtung) des Haram-Monats ist gegen (Nichtbeachtung) eines Haram- Monats, denn bei den Hurumat gilt Qisas. Also wer gegen euch Aggression begeht, so seid ihm gegenüber genauso aggressiv, wie er gegen euch Aggression beging. Und handelt Taqwa gemäß ALLAH gegenüber und wisst, daß ALLAH gewiß mit den Muttaqi ist.\qt@no{194}}%
\qt@newcmd\qurantrans@de@ccii{Und gebt fi-sabilillah, liefert (euer Selbst) nicht eigenhändig dem Niedergang aus und handelt Ihsan gemäß! Gewiß, ALLAH liebt die Muhsin.\qt@no{195}}%
\qt@newcmd\qurantrans@de@cciii{Und vervollständigt die Hadsch und die 'Umra für ALLAH! Und solltet ihr daran gehindert werden, dann (opfert) das, was an Opfertieren verfügbar ist, und rasiert nicht eure Köpfe, bis das Opfertier seinen Opferplatz erreicht hat. Und wer von euch krank oder am Kopf verletzt ist, so (leistet er dafür) Ersatz mit Siyam, Sadaqa oder Opferung. Wenn ihr dann wieder sicher seid, dann (opfert) derjenige, der die 'Umra mit der Hadsch vollzieht, das, was an Opfertieren verfügbar ist. Doch für den, der nichts findet, (gilt) Siyam an drei Tagen während der Hadsch und sieben nach eurer Heimkehr. Diese sind ganze zehn (Tage). Dies (gilt) für denjenigen, dessen Familie bei Almasdschidil-haram nicht zugegen ist. Und handelt Taqwa gemäß ALLAH gegenüber und wisst, daß ALLAH gewiß hart im Bestrafen ist!\qt@no{196}}%
\qt@newcmd\qurantrans@de@cciv{Die Hadsch findet in bekannten Monaten statt. Wer sich in ihnen zur Hadsch entschließt, so gibt es weder Intimitäten noch Fisq noch Streitigkeit während der Hadsch. Und was ihr an Gutem tut, das kennt ALLAH. Und deckt euch mit Reiseproviant ein, denn gewiß, der beste Reiseproviant ist die Taqwa. Und handelt Taqwa gemäß Mir gegenüber, ihr von Verstand!\qt@no{197}}%
\qt@newcmd\qurantrans@de@ccv{Für euch ist es keine Verfehlung, daß ihr nach Gunst von eurem HERRN strebt. Und wenn ihr Ifada von 'Arafat macht, dann gedenkt ALLAHs bei Almasch'arilharam. Und gedenkt Seiner, wie ER euch rechtleitete, und ihr wart vorher gewiß von den Abirrenden.\qt@no{198}}%
\qt@newcmd\qurantrans@de@ccvi{Dann vollzieht Ifada von da, wovon die (anderen) Menschen Ifada vollziehen, und bittet ALLAH um Vergebung! Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{199}}%
\qt@newcmd\qurantrans@de@ccvii{Und wenn ihr eure Riten vollendet habt, dann gedenkt ALLAHs, wie ihr eurer Ahnen gedenkt, oder noch mehr. Und unter den Menschen gibt es manche, die sagen: ``Unser HERR, gib uns im Diesseits!", während es für sie am Jenseits keinerlei Anteil gibt.\qt@no{200}}%
\qt@newcmd\qurantrans@de@ccviii{Und unter ihnen gibt es manche, die sagen: ``Unser HERR, gewähre uns im Diesseits gottgefällig Schönes und im Jenseits gottgefällig Schönes und bewahre uns vor der Peinigung des Feuers!"\qt@no{201}}%
\qt@newcmd\qurantrans@de@ccix{Für diese gibt es einen Anteil von dem, was sie erworben haben, und ALLAH ist schnell im Zur-Rechenschaft-Ziehen.\qt@no{202}}%
\qt@newcmd\qurantrans@de@ccx{Und gedenkt ALLAHs an abgezählten Tagen. Und wer in zwei Tagen beeilt aufbricht, auf diese kommt keine Verfehlung zu, und wer später aufbricht, für diesen ist es (ebenfalls) keine Verfehlung, (dies) für denjenigen, der Taqwa gemäß handelt. Und handelt Taqwa gemäß ALLAH gegenüber und wisst, daß ihr doch vor Ihm versammelt werdet.\qt@no{203}}%
\qt@newcmd\qurantrans@de@ccxi{Und unter den Menschen gibt es manch einen, dessen Worte über das diesseitige Leben dir gefallen, auch läßt er ALLAH Zeuge sein für das, was sein Herz empfindet, während er der heftigste Streitsüchtige ist.\qt@no{204}}%
\qt@newcmd\qurantrans@de@ccxii{Und wenn er fortgeht, zieht er im Lande umher, um darin Verderben anzurichten und die Frauen und die Nachkommenschaft zugrunde zu richten. Und ALLAH liebt nicht das Verderben- Anrichten.\qt@no{205}}%
\qt@newcmd\qurantrans@de@ccxiii{Und wenn ihm gesagt wird: ``Handle Taqwa gemäß ALLAH gegenüber!", ermutigt ihn der Stolz zur Verfehlung. So genügt Ihm Dschahannam. Und gewiß erbärmlich ist der Aufenthaltsort.\qt@no{206}}%
\qt@newcmd\qurantrans@de@ccxiv{Und unter den Menschen sind manche, die sich selbst im Streben nach Wohlgefallen von ALLAH aufgeben. Und ALLAH ist allgütig den Dienern gegenüber.\qt@no{207}}%
\qt@newcmd\qurantrans@de@ccxv{Ihr, die den Iman verinnerlicht habt! Tretet in die Ergebenheit ganzheitlich ein und folgt nicht den Fußstapfen des Satans! Gewiß, er ist für euch ein entschiedener Feind.\qt@no{208}}%
\qt@newcmd\qurantrans@de@ccxvi{Und solltet ihr irregehen, nachdem zu euch die deutlichen Zeichen kamen, so wisst, daß ALLAH gewiß allwürdig, allweise ist!\qt@no{209}}%
\qt@newcmd\qurantrans@de@ccxvii{Warten sie auf etwas anderes außer darauf, daß zu ihnen ALLAHs (Anweisung) im Schatten von dicken Wolken und die Engel kommen?! Doch die Angelegenheit wurde bereits bestimmt. Und an ALLAH werden alle Angelegenheiten gewandt.\qt@no{210}}%
\qt@newcmd\qurantrans@de@ccxviii{Frage Israils Kinder, wie viele deutliche Aya WIR ihnen zuteil werden ließen! Und wer ALLAHs Gabe (Schrift) verändert, nachdem sie zu ihm kam, gewiß ist ALLAH dann hart im Bestrafen.\qt@no{211}}%
\qt@newcmd\qurantrans@de@ccxix{Denjenigen, die Kufr betrieben haben, wurde das diesseitige Leben verschönert, und sie machen sich lustig über diejenigen, die den Iman verinnerlicht haben. Doch diejenigen, die Taqwa gemäß handelten, stehen über ihnen am Tag der Auferstehung. Und ALLAH gewährt Rizq, wem er will, ohne Berechnung.\qt@no{212}}%
\qt@newcmd\qurantrans@de@ccxx{Die Menschen waren eine einzige Umma. Dann entsandte ALLAH die Propheten als Überbringer froher Botschaft und als Ermahner und sandte ihnen die Schrift mit der Wahrheit hinab, damit sie (die Schrift) zwischen den Menschen über das richtet, worüber sie uneins waren. Und uneins darüber wurden nur diejenigen, denen sie zuteil wurde, nachdem zu ihnen die deutlichen Zeichen kamen, aus Übertretung von ihnen. Dann leitete ALLAH diejenigen recht, die den Iman verinnerlichten, zu dem vom Wahren, worüber sie uneins waren - durch Erleichterung von Ihm. Und ALLAH leitet recht, wen ER will, zu einem geradlinigen Weg.\qt@no{213}}%
\qt@newcmd\qurantrans@de@ccxxi{Oder habt ihr etwa gedacht, daß ihr in die Dschanna eintretet, ohne daß euch Ähnliches trifft, wie es diejenigen traf, die vor euch vergingen?! Die bittere Armut und die Krankheit suchten sie heim und sie wurden erschüttert, bis der Gesandte und diejenigen mit ihm, die den Iman verinnerlicht haben, sagten: ``Wann kommt ALLAHs Beistand?" Ja! Gewiß, ALLAHs Beistand ist nahe!\qt@no{214}}%
\qt@newcmd\qurantrans@de@ccxxii{Sie fragen dich, was sie geben sollen. Sag: ``Was ihr an Gut gebt, dann für die Eltern, für die Nahverwandten, für die Waisen, für die Bedürftigen und für den Reisenden." Und was ihr an Gutem macht, so ist ALLAH gewiß darüber allwissend.\qt@no{215}}%
\qt@newcmd\qurantrans@de@ccxxiii{Euch wurde der bewaffnete Kampf geboten, während er euch zuwider ist. Doch vielleicht hegt ihr Abneigung gegen etwas, während es gut für euch ist, und vielleicht liebt ihr etwas, während es schlecht für euch ist. Und ALLAH weiß und ihr wisst nicht.\qt@no{216}}%
\qt@newcmd\qurantrans@de@ccxxiv{Sie fragen dich nach dem Haram-Monat: ``Ist das Führen des bewaffneten Kampfs darin erlaubt?" Sag: ``Das Führen des bewaffneten Kampfs darin ist etwas Schwerwiegendes. Doch das Abhalten vom ALLAHs Weg, Kufr Ihm gegenüber, (das Abhalten) von Almasdschidil-haram und die Vertreibung seiner Bewohner daraus ist bei ALLAH noch schwerwiegender. Und die Fitna ist schwerwiegender als das Töten. Und sie werden gegen euch immer wieder den bewaffneten Kampf führen, bis sie euch von eurem Din abbrachten, wenn sie es könnten. Und wer von euch sich von seinem Din abkehrt und stirbt, während er Kafir ist, so sind diese diejenigen, deren Taten im Diesseits und im Jenseits zunichte wurden, und diese sind die Weggenossen des Feuers. Darin werden sie ewig sein.\qt@no{217}}%
\qt@newcmd\qurantrans@de@ccxxv{Gewiß, diejenigen, die den Iman verinnerlicht haben, und diejenigen, die Hidschra unternommen und Dschihad fi-sabilillah geleistet haben, diese hoffen auf ALLAHs Gnade. Und ALLAH ist allvergebend, allgnädig.\qt@no{218}}%
\qt@newcmd\qurantrans@de@ccxxvi{Und sie fragen dich nach dem Khamr und dem Glücksspiel. Sag: ``In beiden sind große Verfehlungen und Nutzen für die Menschen, doch ihre Verfehlungen überwiegen ihrem Nutzen." Und sie fragen dich, wieviel sie geben sollen. Sag: ``Den Überschuß." Solcherart verdeutlicht ALLAH euch die Ayat, damit ihr nachdenkt\qt@no{219}}%
\qt@newcmd\qurantrans@de@ccxxvii{über das Diesseits und das Jenseits. Und sie fragen dich nach den Waisen. Sag: ``Gottgefällig Gutes ihnen erweisen, ist gut. Und solltet ihr mit ihnen Beteiligung eingehen, so sind sie eure Geschwister. Und ALLAH kennt den Verderben-Anrichtenden dem gottgefällig Guttuenden gegenüber." Und wollte ALLAH es, hätte ER es euch erschwert. Gewiß, ALLAH ist allwürdig, allweise.\qt@no{220}}%
\qt@newcmd\qurantrans@de@ccxxviii{Und heiratet die Muschrik-Frauen nicht, bis sie den Iman annehmen. Und eineMumin-Dienerin ist gewiß besser als eineMuschrika, selbst dann, sollte diese euch gefallen. Und verheiratet nicht an Muschrik-Männer, bis sie den Iman angenommen haben. Und gewiß ein Mumin-Diener ist besser als ein Muschrik, selbst dann, sollte dieser euch gefallen. Diese laden zum Feuer ein, und ALLAH lädt ein zur Dschanna und zur Vergebung mit Erleichterung von ihm. Und ER verdeutlicht Seine Ayat den Menschen, damit sie sich besinnen.\qt@no{221}}%
\qt@newcmd\qurantrans@de@ccxxix{Und sie fragen dich nach der Menstruation. Sag: ``Sie ist eine Beschwerlichkeit, so enthaltet euch (des Geschlechtsverkehrs mit den) Ehefrauen während der Menstruation, und vollzieht mit ihnen keinen Geschlechtsverkehr, bis sie sich rituell reinigen. Und wenn sie sich rituell gereinigt haben, dann verkehrt mit ihnen da, wo ALLAH es euch gebot." Gewiß, ALLAH liebt die stets Reumütigen und die sich rituell Reinigenden.\qt@no{222}}%
\qt@newcmd\qurantrans@de@ccxxx{Eure Ehefrauen sind (wie) ein Einpflanzungsort für euch, so kommt zu eurem Einpflanzungsort, wie ihr wollt, und legt euch etwas vor! Und handelt Taqwa gemäß ALLAH gegenüber und wisst, daß ihr gewiß Ihm begegnen werdet. Und überbringe den Mumin frohe Botschaft!\qt@no{223}}%
\qt@newcmd\qurantrans@de@ccxxxi{Und nehmt ALLAH nicht als Vorwand für eure Eide, damit ihr (sie) nicht erfüllt, nicht Taqwa gemäß handelt und keine Aussöhnung zwischen den Menschen zulaßt. Und ALLAH ist allhörend, allwissend.\qt@no{224}}%
\qt@newcmd\qurantrans@de@ccxxxii{ALLAH belangt euch nicht für etwas Unbedachtes bei euren Eiden, doch ER belangt euch für das, was eure Herzen beabsichtigten. Und ALLAH ist allvergebend, allnachsichtig.\qt@no{225}}%
\qt@newcmd\qurantrans@de@ccxxxiii{Diejenigen, die einen Ilaa-Eid ihren Ehefrauen gegenüber leisten, warten vier Monate. Und wenn sie (vom Eid) zurücktreten, so ist ALLAH gewiß allvergebend, allgnädig!\qt@no{226}}%
\qt@newcmd\qurantrans@de@ccxxxiv{Und sollten sie sich zur Talaq-Scheidung entschließen, so ist ALLAH gewiß allhörend, allwissend.\qt@no{227}}%
\qt@newcmd\qurantrans@de@ccxxxv{Und die talaq-geschiedenen Frauen warten mit sich drei Quru ab. Und es ist nicht halal für sie zu verschweigen, was ALLAH in ihren Gebärmüttern erschuf, sollten sie den Iman an ALLAH und den Jüngsten Tag verinnerlicht haben. Und ihre Ehemänner haben das Vorrecht, die Talaq-Scheidung von ihnen (während der Wartezeit) rückgängig zu machen, wenn sie Aussöhnung wünschen. Und ihnen (den Ehefrauen) steht (den Ehemännern gegenüber) Gleiches zu, wie es ihnen obliegt nach dem Gebilligten, und den Ehemännern steht ihnen gegenüber eine Zusätzlichkeit zu. Und ALLAH ist allwürdig, allweise.\qt@no{228}}%
\qt@newcmd\qurantrans@de@ccxxxvi{Die Talaq-Scheidung ist zweimal (widerrufbar), dann gilt entweder Behalten nach dem Gebilligten oder Entlassen Ihsan gemäß. Und es gilt für euch (Ehemänner) nicht als halal, irgend etwas von dem zu nehmen, was ihr ihnen gegeben habt, außer wenn beide befürchten, daß sie ALLAHs Richtlinien nicht einhalten könnten. Also solltet ihr es befürchten, daß beide ALLAHs Richtlinien nicht einhalten könnten, dann ist es für beide keine Verfehlung, wenn sie eine Ent- schädigung zahlt, (um Talaq-Scheidung zu bewirken). Diese sind ALLAHs Richtlinien, so überschreitet sie nicht! Und wer ALLAHs Richtlinien überschreitet, diese sind die wirklichen Unrecht-Begehenden.\qt@no{229}}%
\qt@newcmd\qurantrans@de@ccxxxvii{Sollte er von ihr dann (zum dritten Mal) die Talaq-Scheidung vollzogen haben, so wird sie für ihn danach nicht mehr halal, bis sie einen anderen Ehemann heiratet. Und sollte dieser (danach) von ihr die Talaq-Scheidung vollzogen haben, dann ist es für beide keine Verfehlung, wenn beide wieder heiraten, wenn beide glauben, sie könnten (nun) ALLAHs Richtlinien einhalten. Und diese sind ALLAHs Richtlinien, ER erläutert sie für Leute, die wissen.\qt@no{230}}%
\qt@newcmd\qurantrans@de@ccxxxviii{Und wenn ihr von den Ehefrauen die Talaq-Scheidung vollzogen habt und sie sich dem Ende ihrer Wartezeit näherten, dann behaltet sie gemäß dem Gebilligten oder entlasst sie gemäß dem Gebilligten. Und behaltet sie nicht zum Schädigen, um Übertretung zu begehen. Und wer solches macht, der beging bereits Unrecht gegen sich selbst. Und nehmt euch ALLAHs Ayat nicht zum Spott! Und gedenkt der Gabe ALLAHs euch gegenüber und dessen, was ER euch von der Schrift und der Weisheit hinabsandte, um euch damit zu ermahnen. Und handelt Taqwa gemäß ALLAH gegenüber und wisst, daß ALLAH gewiß über alles allwissend ist.\qt@no{231}}%
\qt@newcmd\qurantrans@de@ccxxxix{Und wenn ihr von den Ehefrauen die Talaq-Scheidung vollzogen habt und sie sich dem Ende ihrerWartezeit näherten, dann hindert sie (die Frauen) nicht daran, ihre Männer wieder zu heiraten, wenn sie sich nach dem Gebilligten aussöhnten. Damit wird von euch ermahnt, wer den Iman an ALLAH und an den Jüngsten Tag zu verinnerlichen pflegte. Dies ist für euch nutzbringender und reiner. Und ALLAH weiß und ihr wisst nicht.\qt@no{232}}%
\qt@newcmd\qurantrans@de@ccxl{Und die Gebärenden stillen ihre Geborenen zwei volle Jahre für denjenigen, der die Stillzeit vollständig durchführen will. Und demjenigen, dem geboren wurde, obliegt ihr Rizq und ihr Bekleiden nach dem Gebilligten. Einer Seele wird nicht auferlegt, außer was sie vermag. Weder einer Gebärenden darf wegen ihres Geborenen Schaden zugefügt werden, noch demjenigen, dem geboren wurde, wegen seines Geborenen, und dem Erben obliegt Gleiches wie dies. Und sollten beide sich zum Abstillen im gegenseitigen Einvernehmen und nach Beratung entschließen, dann ist es für beide keine Verfehlung. Und wenn ihr eure Kinder (durch andere) stillen lassen wollt, dann ist es keine Verfehlung für euch, wenn ihr das gebt, was ihr vereinbart habt, nach dem Gebilligten. Und handelt Taqwa gemäß ALLAH gegenüber und wisst, daß ALLAH gewiß dessen, was ihr tut, allsehend ist.\qt@no{233}}%
\qt@newcmd\qurantrans@de@ccxli{Und diejenigen von euch, die sterben und Witwen hinterlassen, diese warten mit sich vier Monate und zehn Tage ab. Und wenn sie ihre Wartezeit beendet haben, dann ist es für euch keine Verfehlung in dem, was sie mit sich selbst machen, nach dem Gebilligten. Und ALLAH ist dessen, was ihr tut, allkundig.\qt@no{234}}%
\qt@newcmd\qurantrans@de@ccxlii{Und es ist für euch keine Verfehlung in dem, was ihr von der Verlobung der (verwitweten) Frauen angedeutet oder in eurem Sinne gehabt habt. ALLAH wußte, daß ihr an sie denken werdet, doch gebt ihnen kein Heiratsversprechen, außer daß ihr gebilligtes Wort sagt! Und entschließt euch nicht zum Heiratsvertrag, bis die Vorgeschriebene (Idda-Zeit) ihr Ende hat. Und wisst, daß ALLAH gewiß das kennt, was in eurem Innern ist, so seid achtsam Ihm gegenüber! Und wisst, daß ALLAH gewiß allvergebend, allnachsichtig ist.\qt@no{235}}%
\qt@newcmd\qurantrans@de@ccxliii{Es ist für euch keine Verfehlung, wenn ihr von den Frauen die Talaq- Scheidung vollzogen habt, bevor ihr sie intim berührt und ihnen eine Pflichtgabe (Mahr) zugesprochen habt. Und entschädigt sie, der Begüterte nach seinen Mitteln und der Unvermögende nach seinen Mitteln - eine Entschädigung nach dem Gebilligten. Es ist eine Verpflichtung, die den Muhsin obliegt.\qt@no{236}}%
\qt@newcmd\qurantrans@de@ccxliv{Und wenn ihr von ihnen die Talaq-Scheidung vollzogen habt, bevor ihr sie intim berührt habt, und während ihr ihnen bereits eine Pflichtgabe (Mahr) zugesprochen habt, dann erhalten sie die Hälfte dessen, was ihr zugesprochen habt, es sei denn, sie verzichten oder derjenige verzichtet, der über den Heiratsvertrag verfügt. Und wenn ihr verzichtet, dann ist dies näher zur Taqwa. Und vergesst nicht die Zuvorkommenheit zwischen euch! Gewiß, ALLAH ist dessen, was ihr tut, allsehend.\qt@no{237}}%
\qt@newcmd\qurantrans@de@ccxlv{Haltet die rituellen Gebete und (besonders) das mittlere rituelle Gebet ein, und vollzieht Qiyam für ALLAH in demütiger Ergebenheit!\qt@no{238}}%
\qt@newcmd\qurantrans@de@ccxlvi{Und solltet ihr euch fürchten, dann (verrichtet das rituelle Gebet) beim Gehen oder beim Reiten/ Fahren. Und wenn ihr in Sicherheit seid, dann gedenkt ALLAHs, wie ER euch das lehrte, was ihr nicht zu wissen pflegtet.\qt@no{239}}%
\qt@newcmd\qurantrans@de@ccxlvii{Und denjenigen von euch, die sterben und Witwen hinterlassen, (ist auferlegt) für ihre Frauen ein Vermächtnis mit einjähriger Versorgung undWohnrecht. Und wenn diese (die Wohnung) verlassen, dann ist es für euch keine Verfehlung in dem, was sie mit sich selbst vom Gebilligten machen. Und ALLAH ist allwürdig, allweise.\qt@no{240}}%
\qt@newcmd\qurantrans@de@ccxlviii{Und für die Talaq-Geschiedenen gibt es Versorgung nach dem Gebilligten, es ist eine Verpflichtung, die den Muttaqi obliegt.\qt@no{241}}%
\qt@newcmd\qurantrans@de@ccxlix{Solcherart verdeutlicht ALLAH euch Seine Ayat, damit ihr euch besinnt.\qt@no{242}}%
\qt@newcmd\qurantrans@de@ccl{Gewahrtest du etwa nicht diejenigen, die ihre Wohngebiete zu Tausenden aus Angst vor dem Tod verließen, dann sagte ALLAH ihnen: ``Sterbt!", dann belebte ER sie wieder?! Gewiß, ALLAH ist doch Gunst erweisend den Menschen gegenüber, doch die meisten Menschen erweisen sich undankbar.\qt@no{243}}%
\qt@newcmd\qurantrans@de@ccli{Und führt den bewaffneten Kampf fi-sabilillah und wisst, daß ALLAH gewiß allhörend, allwissend ist.\qt@no{244}}%
\qt@newcmd\qurantrans@de@cclii{Wer ist dieser, der um ALLAHs willen eine Hasan-Anleihe macht, damit ALLAH sie für ihn um ein Vielfaches vermehrt?! Und ALLAH läßt beengen und ausdehnen, und zu Ihm werdet ihr zurückgekehrt.\qt@no{245}}%
\qt@newcmd\qurantrans@de@ccliii{Gewahrtest du etwa nicht die Entscheidungsträger von den Kindern Israils nach Musa, als sie zu einem Propheten von ihnen sagten: ``Schicke uns einen König, dann führen wir den bewaffneten Kampf fi-sabilillah"?! Er sagte: ``Könnte es sein, daß wenn euch der bewaffnete Kampf geboten wird, daß ihr nicht kämpft?" Sie sagten: ``Weshalb sollten wir nicht fi-sabilillah kämpfen, nachdem wir bereits von unseren Wohngebieten vertrieben wurden, sowie von unseren Kindern?" Doch als ihnen der bewaffnete Kampf geboten wurde, wandten sie sich ab außer wenigen von ihnen. Und ALLAH ist allwissend über die Unrecht- Begehenden.\qt@no{246}}%
\qt@newcmd\qurantrans@de@ccliv{Ihr Prophet sagte zu ihnen dann: ``Gewiß, ALLAH hat euch bereits Talut als König geschickt." Sie sagten: ``Woher gebührt ihm die Herrschaft über uns, während wir doch mehr Berechtigung auf die Herrschaft haben, und ihm ohnehin nicht viel an Vermögen gewährt wurde?" Er sagte: ``Gewiß, ALLAH erwählte ihn vor euch aus und mehrte ihn an Wissen und an Körpermasse." Und ALLAH gewährt Seine Herrschaft, wem ER will. Und ALLAH ist allumfassend, allwissend.\qt@no{247}}%
\qt@newcmd\qurantrans@de@cclv{Und ihr Prophet sagte zu ihnen: ``Gewiß, die Aya seiner Herrschaft ist, daß zu euch der Schrein kommt, in dem sich Herzensruhe von eurem HERRN befindet und ein Rest von dem, was die Familie Musas und die Familie Haruns hinterließen - von Engeln getragen. Gewiß, darin ist zweifelsohne eine Aya für euch, solltet ihr Mumin sein."\qt@no{248}}%
\qt@newcmd\qurantrans@de@cclvi{Und als sich Talut mit den Soldaten entfernte, sagte er: ``Gewiß, ALLAH wird euch mit einem Fluß prüfen. Wer davon trinkt, der gehört nicht zu mir, und wer es nicht kostet, dieser gehört zweifelsohne zu mir - ausgenommen derjenige, der nur eine Handvoll schöpft." Dann tranken sie davon außer wenigen von ihnen. Und als er ihn mit denjenigen mit ihm, die den Iman verinnerlicht haben, überquerte, sagten sie: ``Keine Kampfkraft haben wir heute gegen Dschalut und seine Soldaten." Diejenigen, die glaubten, daß sie ALLAH (bald) begegnen werden, sagten: ``Wie oft bewältigte eine kleine Gruppe eine große Gruppe mit ALLAHs Zustimmung! Und ALLAH ist mit den Duldsamen."\qt@no{249}}%
\qt@newcmd\qurantrans@de@cclvii{Und als sie vor Dschalut und seinen Soldaten hervortraten, sagten sie: ``Unser HERR! lasse Geduld uns überkommen, mache uns standhaft und gewähre uns den Sieg über die kufr-betreibenden Leute!"\qt@no{250}}%
\qt@newcmd\qurantrans@de@cclviii{Dann haben sie sie mit ALLAHs Zustimmung in die Flucht getrieben. Und Dawud tötete Dschalut, und ALLAH gewährte ihm die Herrschaft und die Weisheit und lehrte ihn von dem, was ER wollte. Und würde ALLAH die Menschen, die einen durch die anderen nicht zurückhalten, wäre die Erde gewiß verdorben. Doch ALLAH ist Gunst erweisend den Geschöpfen gegenüber.\qt@no{251}}%
\qt@newcmd\qurantrans@de@cclix{Diese sind ALLAHs Ayat, WIR lassen sie dir mit der Wahrheit vortragen. Und gewiß, du bist doch von den Gesandten.\qt@no{252}}%
\qt@newcmd\qurantrans@de@cclx{Diese Gesandten, den einen von ihnen gewährten WIR Gunst den anderen gegenüber. Zu einigen von ihnen sprach ALLAH und andere erhöhte ER um Stellungen. Und WIR ließen 'Isa Ibnu- Maryam, dieWunderzeichen zukommen und unterstützten ihn mit Ruhhul-qudus. Und hätte ALLAH es gewollt, hätten diejenigen nach ihnen den bewaffneten Kampf gegenseitig nicht geführt, nachdem zu ihnen die Zeichen gekommen waren. Doch sie wurden uneins. Also manche von ihnen haben den Iman verinnerlicht und manche von ihnen haben Kufr betrieben. Und hätte ALLAH es gewollt, hätten sie den bewaffneten Kampf gegenseitig nicht geführt, doch ALLAH macht, was ER will.\qt@no{253}}%
\qt@newcmd\qurantrans@de@cclxi{Ihr, die den Iman verinnerlicht habt! Gebt vom Rizq, das WIR euch gewährt haben, bevor ein Tag kommt, an dem weder Kaufen noch Freundschaft noch Fürbitte nützt. Und die Kafir sind die wirklichen Unrecht-Begehenden.\qt@no{254}}%
\qt@newcmd\qurantrans@de@cclxii{ALLAH ist ER, es gibt keine Gottheit außer Ihm, Der Lebendige (ohne Ende), Der Allverantwortliche. Ihm kommt weder Schläfrigkeit noch Schlaf bei. Ihm gehört, was in den Himmeln und was auf Erden ist. Wer ist dieser, der bei Ihm Fürsprache einlegen könnte außer mit Seiner Zustimmung?! ER weiß, was vor ihnen und was hinter ihnen ist, doch sie umfassen nichts von Seinem Wissen außer dem, was ER will. Das Kursi von Ihm umfaßt die Himmel und die Erde, und Ihm fällt es nicht schwer, sie zu bewahren. Und ER ist Der Allhöchste, Der Allerhabene.\qt@no{255}}%
\qt@newcmd\qurantrans@de@cclxiii{Kein Zwang gilt im Din! Bereits ist das Richtige dem Irren gegenüber deutlich geworden. Also wer dem Taghut gegenüber Kufr betreibt und den Iman an ALLAH verinnerlicht, der hielt sich bereits am sichersten Griff, bei dem es kein Abreißen gibt. Und ALLAH ist allhörend, allwissend.\qt@no{256}}%
\qt@newcmd\qurantrans@de@cclxiv{ALLAH ist der Wali derjenigen, die den Iman verinnerlicht haben. ER führt sie von den Finsternissen ins Licht. Und diejenigen, die Kufr betrieben haben, ihre Wali sind die Taghut. Sie führen sie vom Licht in die Finsternisse. Diese sind die Weggenossen des Feuers. Darin werden sie ewig sein.\qt@no{257}}%
\qt@newcmd\qurantrans@de@cclxv{Gewahrtest du etwa nicht denjenigen, der mit Ibrahim über seinen HERRN disputierte, da ALLAH ihm die Herrschaft gewährte?! Als Ibrahim sagte: ``Mein HERR belebt und läßt sterben." Er sagte: ``Ich belebe ebenfalls und lasse sterben." Ibrahim sagte: ``Also ALLAH läßt die Sonne vom Osten aufgehen, so lasse sie vom Westen aufgehen!" Dann war derjenige, der Kufr betrieben hat, sprachlos verwundert. Gewiß, ALLAH leitet die unrecht-begehenden Leute nicht recht.\qt@no{258}}%
\qt@newcmd\qurantrans@de@cclxvi{Oder (gewahrtest du etwa nicht) denjenigen, der an einer Ortschaft vorbeiging, während die Stützen ihrer (Häuser) dem Erdboden gleichgemacht wurden?! Er sagte: ``Wie belebt ALLAH diese nach ihrem Tod?" Dann ließ ALLAH ihn einhundert Jahre sterben, dann erweckte ER ihn auf. ER sagte: ``Wie lange hast du verweilt?" Er sagte: ``Ich verweilte einen Tag oder den Teil eines Tages." ER sagte: ``Nein, sondern du hast einhundert Jahre verweilt. So schau zu deinem Essen und deinem Getränk, sie verdarben nicht. Und schau zu deinem Esel! Und WIR machen dich zweifelsohne zu einer Aya für dieMenschen. Und schau zu den Knochen, wie WIR sie zusammenfügen und dann mit Fleisch überziehen." Und als es ihm deutlich wurde, sagte er: ``Ich weiß, daß ALLAH gewiß über alles allmächtig ist."\qt@no{259}}%
\qt@newcmd\qurantrans@de@cclxvii{Und erinnere daran, als Ibrahim sagte: ``Mein HERR! Zeige mir, wie DU die Toten belebst." ER sagte: ``Hast du etwa den Iman noch nicht verinnerlicht?!" Er sagte: ``Doch, mit Sicherheit! Aber damit mein Herz Ruhe findet." ER sagte: ``Nimm dir vier Vögel und zerstückle sie, dann lege auf jeden Berg einen Teil von ihnen, dann rufe sie, kommen sie eilends zu dir. Und wisse, daß ALLAH gewiß allwürdig, allweise ist."\qt@no{260}}%
\qt@newcmd\qurantrans@de@cclxviii{Das Gleichnis derjenigen, die ihr Vermögen fi-sabilillah ausgeben, gleicht dem Gleichnis eines Saatkorns, das sieben Ähren hervorsprießen ließ, jede Ähre trägt hundert Körner. Und ALLAH vervielfacht, wem ER will. Und ALLAH ist allumfassend, allwissend.\qt@no{261}}%
\qt@newcmd\qurantrans@de@cclxix{Diejenigen, die ihr Vermögen fi-sabilillah ausgeben, dann dem, was sie gaben, weder Vorhaltung noch Belästigung folgen lassen, haben ihre Belohnung bei ihrem HERRN und um sie wird es weder Angst geben, noch werden sie traurig sein.\qt@no{262}}%
\qt@newcmd\qurantrans@de@cclxx{Gebilligtes Wort und Vergebung sind besser als eine Sadaqa, der Belästigung folgt. Und ALLAH ist autark, allnachsichtig.\qt@no{263}}%
\qt@newcmd\qurantrans@de@cclxxi{Ihr, die den Iman verinnerlicht habt! Annulliert eure Sadaqa nicht durch Vorhaltung und Belästigung wie derjenige, der sein Vermögen aus Riyaa den Menschen gegenüber gibt und den Iman an ALLAH und den Jüngsten Tag nicht verinnerlicht. Sein Gleichnis ähnelt einem mit Erde bedeckten Felsgestein, das vom heftigen Regen getroffen wurde, und es dann glatt werden ließ. Sie finden nichts von dem, was sie erworben haben. Und ALLAH leitet die kufr-betreibenden Leute nicht recht.\qt@no{264}}%
\qt@newcmd\qurantrans@de@cclxxii{Und das Gleichnis derjenigen, die ihr Vermögen strebend nach ALLAHsWohlgefallen und zur Festigung ihrer Seelen ausgeben, ähnelt einer Dschanna auf einem Hügel, auf die Wolkenbruch niederfiel, dann sie ihre Früchte zweifach trug. Und selbst wenn kein Wolkenbruch sie trifft, dann Sprühregen. Und ALLAH ist dessen, was ihr tut, allsehend.\qt@no{265}}%
\qt@newcmd\qurantrans@de@cclxxiii{Wünscht sich einer von euch etwa, daß ihm eine Dschanna mit Dattelpalmen und Rebstöcken gehört, die von Flüssen durchflossen ist und darin er über Früchte aller Art verfügt - und ihn das Greisenalter erreichte, während er schwache Kleinkinder hat - dann traf sie ein Sturm, in dem Feuer ist, und dann verbrannte sie?! Solcherart verdeutlicht ALLAH euch die Ayat, damit ihr nachdenkt.\qt@no{266}}%
\qt@newcmd\qurantrans@de@cclxxiv{Ihr, die den Iman verinnerlicht habt! Gebt von den Tay-yibat dessen, was ihr erworben habt, und von dem, was WIR für euch aus der Erde hervorsprießen ließen. Und wählt nicht das Schlechte aus, um davon zu geben, und ihr werdet selbst es nicht annehmen, es sei denn, ihr drückt beide Augen zu. Und wisst, daß ALLAH gewiß autark, überreichlich belohnend ist.\qt@no{267}}%
\qt@newcmd\qurantrans@de@cclxxv{Der Satan verspricht euch Armut und gebietet euch das Verwerfliche. Und ALLAH verspricht euch Vergebung von Ihm und Gunst. Und ALLAH ist allumfassend, allwissend.\qt@no{268}}%
\qt@newcmd\qurantrans@de@cclxxvi{ER gewährt Weisheit, wem ER will. Und wem Weisheit gewährt wird, diesem wurde bereits viel Gutes gewährt. Doch nicht erinnern lassen sich außer diejenigen mit Verstand.\qt@no{269}}%
\qt@newcmd\qurantrans@de@cclxxvii{Und was ihr an Spende gebt oder euch an Gelübde auferlegt, gewiß ALLAH kennt es. Und für die Unrecht-Begehenden gibt es keine Beistehenden.\qt@no{270}}%
\qt@newcmd\qurantrans@de@cclxxviii{Solltet ihr die Sadaqa sichtbar machen, so ist dies gut. Doch wenn ihr sie verbergt und den Bedürftigen zukommen lasst, dann ist dies besser für euch. Und ER vergibt euch von euren gottmißfälligen Taten. Und ALLAH ist dessen, was ihr tut, allkundig.\qt@no{271}}%
\qt@newcmd\qurantrans@de@cclxxix{Dir obliegt nicht ihre Rechtleitung. Doch ALLAH leitet recht, wen ER will. Und was ihr an Gutem gebt, so ist dies für euch selbst. Und nicht gebt ihr außer im Streben nach ALLAHs Wohlgefallen. Und was ihr an Gutem gebt, wird euch vergütet und euch wird kein Unrecht angetan.\qt@no{272}}%
\qt@newcmd\qurantrans@de@cclxxx{Die Sadaqa sind) für die Bedürftigen, die fi-sabilillah gehindert wurden, sie können nicht im Lande (für den Lebensunterhalt) umherreisen. Derjenige, der (sie nicht) kennt, denkt, sie seien vermögend wegen ihres Nichtbittens. Du erkennst sie durch ihre Merkmale, sie tragen den Menschen ihre Bitte nicht aufdringlich vor. Und was immer ihr an Gutem gebt, so ist ALLAH gewiß darüber allwissend.\qt@no{273}}%
\qt@newcmd\qurantrans@de@cclxxxi{Diejenigen, die von ihrem Vermögen nachts und am Tage, insgeheim und öffentlich geben, diese haben ihre Belohnung bei ihrem HERRN und um sie gibt es weder Angst, noch werden sie traurig sein.\qt@no{274}}%
\qt@newcmd\qurantrans@de@cclxxxii{Diejenigen, die Riba einnehmen, stehen nicht auf außer wie derjenige aufsteht, auf den der Satan schlägt - vor der Besessenheit. Dies weil sie sagten: ``Das Verkaufen ist doch nur genauso wie Riba." Doch ALLAH erklärte das Verkaufen für halal und Riba für haram. Also wem eine Ermahnung von seinem HERRN zukam, dann es unterließ, dem gehört, was er bereits an Riba nahm, und seine Angelegenheit unterliegt ALLAH. Und wer es wiederholt, diese sind die Weggenossen des Feuers. Darin werden sie ewig sein.\qt@no{275}}%
\qt@newcmd\qurantrans@de@cclxxxiii{ALLAH läßt Riba schwinden und die Sadaqa anwachsen. Und ALLAH liebt nicht jeden äußerst verfehlenden Kufr-Betreibenden.\qt@no{276}}%
\qt@newcmd\qurantrans@de@cclxxxiv{Gewiß, diejenigen, die den Iman verinnerlicht, gottgefällig Gutes getan, das rituelle Gebet ordnungsgemäß verrichtet und die Zakat entrichtet haben, diese haben ihre Belohnung bei ihrem HERRN und um sie gibt es weder Angst, noch werden sie traurig sein.\qt@no{277}}%
\qt@newcmd\qurantrans@de@cclxxxv{Ihr, die den Iman bekundet habt! Handelt Taqwa gemäß ALLAH gegenüber und verzichtet auf das Ausstehende an Riba, solltet ihr Mumin sein.\qt@no{278}}%
\qt@newcmd\qurantrans@de@cclxxxvi{Und solltet ihr dies nicht tun, dann wisst Bescheid über einen Krieg von ALLAH und Seinem Gesandten! Und wenn ihr bereut, so gehören euch (nur) eure Ausgangskapitale, ihr begeht kein Unrecht und euch wird kein Unrecht angetan.\qt@no{279}}%
\qt@newcmd\qurantrans@de@cclxxxvii{Und sollte er in Schwierigkeit sein, dann gilt Nachsehen bis zur Erleichterung. Und wenn ihr sie (die Schulden) als Sadaqa lasst, ist es besser für euch, wenn ihr nur wüßtet.\qt@no{280}}%
\qt@newcmd\qurantrans@de@cclxxxviii{Und sucht Taqwa einem Tag gegenüber, an dem ihr zu ALLAH zurückgebracht werdet, dann wird jeder Seele das vergütet, was sie erworben hat. Und ihnen wird gewiß kein Unrecht angetan.\qt@no{281}}%
\qt@newcmd\qurantrans@de@cclxxxix{Ihr, die den Iman verinnerlicht habt! Wenn ihr einander eine Schuld für eine festgesetzte Frist gewährt, dann schreibt sie nieder! Und ein Schriftführer soll für euch gerecht schreiben. Und ein Schriftführer darf nicht ablehnen, zu schreiben, wie ALLAH es ihm lehrte. So soll er schreiben! Und der Schuldner soll diktieren, ALLAH gegenüber, seinem HERRN, Taqwa gemäß handeln und nichts davon geringer angeben. Und sollte der Schuldner beschränkt, schwach oder unfähig sein, selbst zu diktieren, dann soll sein Wali gerecht diktieren. Und lasst zwei Zeugen von euren Männern ihn (den Vertrag) bezeugen. Und sollten beide keine Männer sein, dann sollen sie ein Mann und zwei Frauen von den Zeugen sein, die ihr akzeptiert, denn sollte die eine von beiden (Frauen) sich irren, dann kann die eine die andere erinnern. Und die Zeugen dürfen das Zeugnis nicht verweigern, wenn sie geladen werden. Und seid nicht lustlos, sie (die Schuld) niederzuschreiben, egal ob sie klein oder groß ist, bis zu ihrer Frist. Dies ist gerechter vor ALLAH, glaubwürdiger für das Zeugnis und näher dazu, keine Zweifel zu hegen. Ausgenommen davon - wenn es sich um einen gegenwärtigen Handel dreht, den ihr untereinander zum Abschluß bringt, dann trifft euch keine Verfehlung, wenn ihr ihn (diesen Handel) nicht niederschreibt. Und lasst es bezeugen, wenn ihr miteinander Handel betreibt. Und weder ein Schriftführer noch ein Zeuge dürfen geschädigt werden. Und solltet ihr es dennoch tun, so ist dies zweifelsohne Fisq von euch. Und handelt Taqwa gemäß ALLAH gegenüber! Und ALLAH lehrt euch. Und ALLAH ist über alles allwissend.\qt@no{282}}%
\qt@newcmd\qurantrans@de@ccxc{Und wenn ihr auf Reisen seid und keinen Schriftführer findet, dann nehmt ein Pfand! Und solltet ihr einander vertrauen, so soll derjenige, dem (die Schuld) anvertraut wurde, das ihm Anvertraute zurückgeben und Taqwa gemäß ALLAH gegenüber, seinem HERRN, handeln. Und verschweigt nicht das Zeugnis! Und wer es verschweigt, so ist dessen Herz gewiß voller Verfehlung. Und ALLAH ist über das, was ihr tut, allwissend.\qt@no{283}}%
\qt@newcmd\qurantrans@de@ccxci{ALLAH gehört, was in den Himmeln und was in der Erde ist. Und wenn ihr das, was in eurem Innern ist, offenbart oder verheimlicht, dafür wird ALLAH euch zur Rechenschaft ziehen, dann vergibt ER, wem ER will, und peinigt, wen ER will. Und ALLAH ist über alles allmächtig.\qt@no{284}}%
\qt@newcmd\qurantrans@de@ccxcii{Der Gesandte hat den Iman an das verinnerlicht, was ihm von seinem HERRN hinabgesandt wurde, ebenso die Mumin. Sie alle haben den Iman verinnerlicht an ALLAH, an Seine Engel, an Seine Schriften und an Seine Gesandten: ``Wir machen keinen Unterschied bei keinem Seiner Gesandten." Und sie sagten: ``Wir haben gehört und gehorcht, (gewähre uns) Deine Vergebung, unser HERR! Und zu Dir ist das Werden."\qt@no{285}}%
\qt@newcmd\qurantrans@de@ccxciii{ALLAH erlegt einer Seele nichts auf, außer was sie vermag. Ihr gebührt, was sie erworben, und ihr obliegt, was sie sich angeeignet hat. ``Unser HERR, belange uns nicht, wenn wir vergessen oder Fehler begangen haben! Unser HERR, erlege uns keine schweren Verpflichtungen auf, wie DU es denjenigen vor uns auferlegtest! Unser HERR, und gebiete uns nicht, was wir nicht vermögen, erlasse uns, vergib uns und erbarme Dich unser! DU bist Unser Maula, so leiste uns Beistand gegen die kufr-betreibenden Leute!"\qt@no{286}}%
\qt@newcmd\qurantrans@de@ccxciv{\basmalahde Alif-lam-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@ccxcv{ALLAH ist ER, es gibt keine Gottheit außer Ihm, Der Lebendige (ohne Ende), Der Allverantwortliche.\qt@no{2}}%
\qt@newcmd\qurantrans@de@ccxcvi{ER sandte dir die Schrift nach und nach mit der Wahrheit hinab als Bestätigung dessen, was vorher war. Ebenfalls sandte ER At-taurat und Alindschil hinab\qt@no{3}}%
\qt@newcmd\qurantrans@de@ccxcvii{vorher als Rechtleitung für die Menschen und ER hat Al-furqan hinabgesandt. Für diejenigen, die Kufr ALLAHs Ayat gegenüber betreiben, ist harte Peinigung bestimmt. Und ALLAH ist allwürdig, vergeltung-übend.\qt@no{4}}%
\qt@newcmd\qurantrans@de@ccxcviii{Gewiß, ALLAH bleibt nichts verborgen, weder auf Erden noch im Himmel.\qt@no{5}}%
\qt@newcmd\qurantrans@de@ccxcix{ER ist Derjenige, Der euch in den Mütterleibern formt, wie ER will. Es ist kein Gott außer Ihm, Dem Allwürdigen, Dem Allweisen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@ccc{ER ist Derjenige, Der dir die Schrift hinabgesandt hat, in der eindeutige Ayat sind, die das Elementare der Schrift ausmachen, und andere mehrdeutige Ayat. Hinsichtlich derjenigen, in deren Herzen Falschheit ist, so folgen sie dem, was darin mehrdeutig ist  in Bestrebung nach Fitna und in Bestrebung nach (irriger) Auslegung. Jedoch ihre (genaue) Auslegung kennt nur ALLAH und die im Wissen fest Verankerten, sie sagen: ``Wir haben den Iman daran verinnerlicht, alles ist von unserem HERRN." Doch nicht erinnern lassen sich außer diejenigen mit Verstand.\qt@no{7}}%
\qt@newcmd\qurantrans@de@ccci{Unser HERR! Lasse in unseren Herzen keine Falschheit entstehen, nachdem DU uns rechtgeleitet hast, und schenke uns von Dir aus Gnade! Gewiß, DU bist zweifellos Der großzügigste Schenkende.\qt@no{8}}%
\qt@newcmd\qurantrans@de@cccii{Unser HERR! DU wirst zweifellos die Menschen zu einem Tag versammeln, über den es keinerlei Zweifel gibt. Gewiß, ALLAH mißachtet niemals Sein Versprechen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@ccciii{Gewiß, denjenigen, die Kufr betrieben haben, werden weder ihre Vermögenswerte noch ihre Kinder vor ALLAH etwas nützen. Und diese sind der Brennstoff des Feuers.\qt@no{10}}%
\qt@newcmd\qurantrans@de@ccciv{Genauso wie im Falle der Pharao-Familie und derjenigen vor ihnen, sie haben Unsere Ayat verleugnet, dann bestrafte ALLAH sie umgehend wegen ihrer Verfehlungen. Und ALLAH ist hart im Strafen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@cccv{Sag zu den Kafir: ``Ihr werdet unterlegen sein und für Dschahannam versammelt." Und was für ein erbärmlicher Aufenthaltsort.\qt@no{12}}%
\qt@newcmd\qurantrans@de@cccvi{Bereits war es eine Aya für euch in zwei Gruppierungen, die aufeinander trafen, eine Gruppe kämpfte fi-sabilillah, und eine andere war kufr-betreibend. (Die Mumin) sahen sie augenscheinlich in doppelter Anzahl. Aber ALLAH läßt mit Seinem Sieg mächtiger werden, wen ER will. Gewiß, darin ist zweifelsohne eine Lehre für die sich Besinnenden.\qt@no{13}}%
\qt@newcmd\qurantrans@de@cccvii{Den Menschen wurde als etwas Schönes dargestellt die Neigung zur Erfüllung der Sehnsüchte nach Frauen, Kindern, angehäuften Vermögenswerten - aus Gold, Silber, schönen Pferden, An'am und Ländereien. Diese sind die Gebrauchsgüter des diesseitigen Lebens. Und bei ALLAH ist die schöne Heimkehr.\qt@no{14}}%
\qt@newcmd\qurantrans@de@cccviii{Sag: ``Soll ich euch von Besserem als dies Mitteilung machen? Für diejenigen, die Taqwa gemäß ihrem HERRN gegenüber handeln, sind Dschannat, die von Flüssen durchflossen sind, gereinigte Partnerwesen und Wohlgefallen von ALLAH bestimmt." Und ALLAH kennt zweifellos die Dienenden.\qt@no{15}}%
\qt@newcmd\qurantrans@de@cccix{Es sind diejenigen, die sagen: ``Unser HERR! Wir haben ja den Iman verinnerlicht, so vergib uns unsere Verfehlungen und bewahre uns vor der Peinigung des Feuers!\qt@no{16}}%
\qt@newcmd\qurantrans@de@cccx{Diese sind) die Duldsamen, die Wahrhaftigen, die Demütigen, die Spendenden und die gegen Ende der Nacht (ALLAH) um Vergebung Bittenden.\qt@no{17}}%
\qt@newcmd\qurantrans@de@cccxi{ALLAH hat deutlich gemacht, daß es zweifelsohne keine Gottheit außer Ihm gibt - (dies bezeugen) ebenfalls die Engel und die Wissenden - und daß ER mit Gerechtigkeit waltet. Es gibt keine Gottheit außer Ihm, Dem Allwürdigen, Dem Allweisen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@cccxii{Gewiß, der (annehmbare) Din bei ALLAH ist der Islam. Und diejenigen, denen die Schrift zuteil wurde, sind erst uneins geworden, nachdem dasWissen zu ihnen kam - aus gegenseitiger Übertretung. Und wer Kufr ALLAHs Ayat gegenüber betreibt, so ist ALLAH gewiß schnell im Zur-Rechenschaft-Ziehen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@cccxiii{Und sollten sie mit dir disputieren, dann sag: ``Ich habe den Islam angenommen und (mit mir) alle, die mir folgen." Und sag zu denjenigen, denen die Schrift zuteil wurde, und zu den Lese- und Schreibunkundigen: ``Habt ihr den Islam angenommen?" Und wenn sie den Islam angenommen haben, so sind sie bereits rechtgeleitet. Doch sollten sie sich abwenden, dann obliegt dir doch nur das Verkünden. Und ALLAH kennt die Dienenden.\qt@no{20}}%
\qt@newcmd\qurantrans@de@cccxiv{Gewiß, denjenigen unter den Menschen, die ALLAHs Ayat gegenüber Kufr betreiben, die Propheten unrechtmäßig töten und diejenigen von den Menschen töten, welche Gerechtigkeit gebieten, diesen überbringe die ``frohe Botschaft" über qualvolle Peinigung!\qt@no{21}}%
\qt@newcmd\qurantrans@de@cccxv{Diese sind diejenigen, deren Taten im Diesseits und im Jenseits verloren gehen, und für sie gibt es keine Beistehenden.\qt@no{22}}%
\qt@newcmd\qurantrans@de@cccxvi{Hast du etwa nicht diejenigen gesehen, denen ein Teil der Schrift zuteil wurde, welche zu ALLAHs Schrift gerufen werden, damit diese zwischen ihnen richtet, dann ein Teil von ihnen fortgeht, während er abwendend den Rücken kehrt?!\qt@no{23}}%
\qt@newcmd\qurantrans@de@cccxvii{Dies war so, weil sie sagten: ``Das Feuer wird uns nie berühren außer für abgezählte Tage.  Und täuschte sie in ihrem Din das, was sie zu erlügen pflegten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@cccxviii{Aber wie (wird es ihnen gehen), wenn WIR sie versammeln zu einem Tag, über den es keinen Zweifel gibt, und jeder Seele vergolten wird, was sie erworben hat, und sie kein Unrecht erleiden?!\qt@no{25}}%
\qt@newcmd\qurantrans@de@cccxix{Sag: ``Oh ALLAH, Besitzer aller Herrschaft! DU verleihst die Herrschaft, wem DU willst, entziehst die Herrschaft, wem DU willst, erhöhst, wen DU willst, und erniedrigst, wen DU willst - bei Dir liegt das (gottgefällig) Gute. Gewiß, DU bist über alles allmächtig.\qt@no{26}}%
\qt@newcmd\qurantrans@de@cccxx{Du läßt die Nacht in den Tag einfließen und den Tag in die Nacht einfließen, DU läßt das Lebendige aus dem Toten entstehen und das Tote aus dem Lebendigen entstehen und DU gewährst Rizq, wem DU willst, ohne Berechnung.\qt@no{27}}%
\qt@newcmd\qurantrans@de@cccxxi{Die Mumin sollen sich die Kafir nicht als Wali anstelle der Mumin nehmen. - und wer dies tut, dieser hat nichts mit ALLAHs (Din) gemeinsam, außer wenn ihr euch vor ihnen wirklich schützen müßt. Und ALLAH warnt euch vor Seiner (Peinigung). Und zu ALLAH ist das Werden.\qt@no{28}}%
\qt@newcmd\qurantrans@de@cccxxii{Sag: ``Egal ob ihr das, was ihr in euren Brüsten tragt, verheimlicht oder offenbart, ALLAH weiß darüber Bescheid, und ER weiß, was in den Himmeln und was auf Erden ist. Und ALLAH ist über alles allmächtig.\qt@no{29}}%
\qt@newcmd\qurantrans@de@cccxxiii{An jenem Tag, wenn jede Seele das, was sie an (gottgefällig) Gutem tat, vorfindet, ebenso was sie an Schlechtem tat, da wünscht sie sich: Läge doch zwischen ihr und diesem ein weit entfernter Zeitraum! Und ALLAH warnt euch vor Seiner (Peinigung). Und ALLAH ist den Dienenden allerbarmend.\qt@no{30}}%
\qt@newcmd\qurantrans@de@cccxxiv{Sag: ``Wenn ihr (wirklich) ALLAH liebt, dann folgt mir, damit ALLAH euch liebt und euch eure Verfehlungen vergibt." Und ALLAH ist allvergebend, allgnädig.\qt@no{31}}%
\qt@newcmd\qurantrans@de@cccxxv{Sag: ``Gehorcht ALLAH und dem Gesandten!" Und sollten sie sich abwenden, so liebt ALLAH doch nicht die Kafir.\qt@no{32}}%
\qt@newcmd\qurantrans@de@cccxxvi{Gewiß, ALLAH erwählte Adam, Nuh, die Familie Ibrahims und die Familie Imrans vor allen Menschen aus.\qt@no{33}}%
\qt@newcmd\qurantrans@de@cccxxvii{Sie waren eine Nachkommenschaft, die einen von ihnen sind von den anderen. Und ALLAH ist allhörend, allwissend.\qt@no{34}}%
\qt@newcmd\qurantrans@de@cccxxviii{Erinnere daran), als die Frau von Imran sagte: ``HERR! Ich habe Dir das gelobt, was ich in meinem Bauch trage - nur für den Gottesdienst (zu weihen), so nimm es von mir an! Gewiß, DU bist Der Allhörende, Der Allwissende.\qt@no{35}}%
\qt@newcmd\qurantrans@de@cccxxix{Und als sie es (zur Welt) gebracht hatte, sagte sie: ``HERR! Ich habe es als Mädchen (zur Welt) gebracht - und ALLAH weiß besser, was (sie zur Welt) brachte - und der Junge ist nicht gleich dem Mädchen, ich habe es Maryam genannt und ich suche ja für sie und ihre Nachkommenschaft bei Dir Schutz vor dem verfluchten Satan!"\qt@no{36}}%
\qt@newcmd\qurantrans@de@cccxxx{So nahm sie ihr HERR in angenehmer Weise an, ließ sie in angenehmer Weise heranwachsen und ließ Zakaria sie in Obhut nehmen. Jedesmal wenn Zakaria zu ihr ins Mihrab eintrat, fand er bei ihr Rizq. Er sagte: ``Maryam! Woher hast du dies?" Sie sagte: ``Es ist von ALLAH. Gewiß, ALLAH gewährt Rizq, wem ER will, ohne Berechnung."\qt@no{37}}%
\qt@newcmd\qurantrans@de@cccxxxi{Da richtete Zakaria an seinen HERRN Bittgebete, er sagte: ``HERR! Schenke mir von Dir aus eine (gottgefällig) gute Nachkommenschaft. Gewiß, DU bist Der Erhörer des Bittgebets."\qt@no{38}}%
\qt@newcmd\qurantrans@de@cccxxxii{Dann haben ihn die Engel - während er im Mihrab stand und betete, gerufen: ``Ja! ALLAH schickt dir die frohe Botschaft von Yahya, er wird sein ein Bestätiger eines Wortes von ALLAH, ein Gebieter, ein Asket und ein Prophet von den gottgefällig Guttuenden."\qt@no{39}}%
\qt@newcmd\qurantrans@de@cccxxxiii{Er sagte: ``HERR! Wie kann ich einen Sohn haben, während das Alter mich bereits ereilte und meine Frau unfruchtbar ist?!" ER sagte: ``Solcherart tut ALLAH, was ER will."\qt@no{40}}%
\qt@newcmd\qurantrans@de@cccxxxiv{Er sagte: ``HERR! Bestimme für mich eine Aya!" ER sagte: ``Deine Aya ist, daß du mit den Menschen drei Tage lang nicht sprechen kannst, es sei denn durch Gesten. Und gedenke deines HERRN viel und lobpreise Ihn zu Sonnenuntergang und zu Sonnenaufgang!"\qt@no{41}}%
\qt@newcmd\qurantrans@de@cccxxxv{Und (erinnere daran), als die Engel sagten: ``Maryam! Gewiß, ALLAH hat dich auserwählt, gereinigt und vor den Frauen in aller Welt auserwählt."\qt@no{42}}%
\qt@newcmd\qurantrans@de@cccxxxvi{Maryam! Zolle deinem HERRN Gehorsam, vollziehe Sudschud und vollziehe Ruku' mit den Ruku'-Vollziehenden."\qt@no{43}}%
\qt@newcmd\qurantrans@de@cccxxxvii{Dies ist eine Mitteilung vom Verborgenen, die WIR dir durch Wahy zuteil werden lassen. Denn du warst nicht bei ihnen, als sie auslosten, wer von ihnen Maryam in seine Obhut nehmen solle. Und du warst nicht bei ihnen, als sie sich untereinander gestritten haben.\qt@no{44}}%
\qt@newcmd\qurantrans@de@cccxxxviii{Erinnere daran), als die Engel sagten: ``Maryam! Gewiß, ALLAH läßt dir frohe Botschaft über ein Wort von Ihm überbringen, sein Name ist Almasih 'Isa Ibnu-Maryam, hochgeachtet im Diesseits und im Jenseits und einer von denjenigen, die (ALLAH) nahe stehen.\qt@no{45}}%
\qt@newcmd\qurantrans@de@cccxxxix{Und er wird zu den Menschen in der Wiege sprechen und als Mann und einer der gottgefällig Guttuenden sein."\qt@no{46}}%
\qt@newcmd\qurantrans@de@cccxl{Sie sagte: ``HERR! Wie kann ich einen Sohn haben, wo kein Mann mich jemals intim berührt hat." ER sagte: ``Solcherart erschafft ALLAH, was ER will. Wenn ER eine Angelegenheit bestimmt hat, sagt ER zu ihr doch nur: ``Sei!", und sie ist."\qt@no{47}}%
\qt@newcmd\qurantrans@de@cccxli{Und ER lehrt ihn die Schrift, die Weisheit, At-taurat und Alindschil.\qt@no{48}}%
\qt@newcmd\qurantrans@de@cccxlii{Und (er ist) ein Gesandter zu den Kindern Israils: ``Gewiß, ich kam bereits zu euch durch eine Aya von eurem HERRN: ich forme für euch aus Lehm wie die Gestalt eines Vogels, dann hauche ich sie an, und sie wird zum Vogel mit ALLAHs Zustimmung. Und ich heile den Blindgeborenen und den Leprakranken und mache die Toten mit ALLAHs Zustimmung lebendig und teile euch mit, was ihr esst und was ihr in euren Häusern speichert. Gewiß, darin ist zweifelsohne eine Aya für euch, wenn ihr Mumin seid.\qt@no{49}}%
\qt@newcmd\qurantrans@de@cccxliii{Und (ich bin) Bestätiger dessen, was vor mir in At-taurat war, und damit ich euch manches von dem für halal erkläre, was für euch als haram galt. Auch bin ich zu euch durch eine Aya gekommen, so handelt Taqwa gemäß ALLAH gegenüber und gehorcht mir!\qt@no{50}}%
\qt@newcmd\qurantrans@de@cccxliv{Gewiß, ALLAH ist mein HERR und euer HERR. Also dient Ihm, dies ist ein gerader Weg!"\qt@no{51}}%
\qt@newcmd\qurantrans@de@cccxlv{Und als 'Isa den Kufr bei ihnen wahrnahm, sprach er: ``Wer sind meine Helfer für ALLAHs (Din)?" Die Jünger sprachen: ``Wir sind die Helfer für ALLAHs (Din), wir haben den Iman an ALLAH verinnerlicht, und bezeuge, daß wir doch Muslime sind.\qt@no{52}}%
\qt@newcmd\qurantrans@de@cccxlvi{Unser HERR! Wir haben den Iman an das verinnerlicht, was DU hinabgesandt hast, und wir folgten dem Gesandten, so registriere uns mit den Bezeugenden!"\qt@no{53}}%
\qt@newcmd\qurantrans@de@cccxlvii{Und sie (die Kafir) haben intrigiert und ALLAH entgegnete ihre Intrigen und ALLAH ist der Beste der Intrigen entgegnet.\qt@no{54}}%
\qt@newcmd\qurantrans@de@cccxlviii{Erinnere daran), als ALLAH sagte: ``'Isa! Gewiß, ICH werde deine Frist ablaufen lassen, dich zu Meinem (Himmel) erheben, dich freimachen von denjenigen, die Kufr betrieben haben, und diejenigen, die dir gefolgt sind, bis zum Tag der Auferstehung über diejenigen stellen, die Kufr betrieben haben. Dann ist eure Rückkehr zu Mir und dann werde ICH zwischen euch richten über das, worüber ihr uneins zu sein pflegtet.\qt@no{55}}%
\qt@newcmd\qurantrans@de@cccxlix{Also hinsichtlich derjenigen, die Kufr betrieben haben, diese werde ICH einer harten Peinigung sowohl im Diesseits, als auch im Jenseits unterziehen. Und sie haben keine Beistehenden.\qt@no{56}}%
\qt@newcmd\qurantrans@de@cccl{Und hinsichtlich derjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, diesen gewährt ER ihre Belohnungen voll. Und ALLAH liebt nicht die Unrecht-Begehenden.\qt@no{57}}%
\qt@newcmd\qurantrans@de@cccli{Dies tragen WIR dir vor aus den Ayat und aus der weisen Ermahnung.\qt@no{58}}%
\qt@newcmd\qurantrans@de@ccclii{Gewiß, das Gleichnis von 'Isa bei ALLAH ist wie das Gleichnis von Adam, ER erschuf ihn aus Erde, dann sagte ER zu ihm: ``Sei!", und er ist.\qt@no{59}}%
\qt@newcmd\qurantrans@de@cccliii{Dies ist die Wahrheit von deinem HERRN, so sei nicht unter den Zweifel-Hegenden!\qt@no{60}}%
\qt@newcmd\qurantrans@de@cccliv{Wer nun mit dir über ihn ('Isa) disputiert, nach dem Wissen, das zu dir kam, so sag: ``Laßt uns unsere Kinder und eure Kinder, unsere Frauen und eure Frauen und uns selbst und euch selbst zusammenbringen, dann (ALLAH) anflehen, den Fluch ALLAHs auf die Lügner zu legen."\qt@no{61}}%
\qt@newcmd\qurantrans@de@ccclv{Gewiß, dies ist zweifelsohne die wahre Begebenheit. Und es gibt keine Gottheit außer ALLAH. Und gewiß, ALLAH ist doch Der Allwürdige, Der Allweise.\qt@no{62}}%
\qt@newcmd\qurantrans@de@ccclvi{Und sollten sie sich abwenden, so ist ALLAH allwissend über die Verderben-Anrichtenden.\qt@no{63}}%
\qt@newcmd\qurantrans@de@ccclvii{Sag: ``Ihr Schriftbesitzer! Kommt zu einem Wort, das gleichermaßen für euch und uns gilt, daß wir nur ALLAH dienen und Ihm gegenüber mit Nichts Schirk praktizieren, und daß die einen von uns sich die anderen nicht als Herren anstelle von ALLAH nehmen." Und sollten sie sich abwenden, so sagt: ``Seid Zeugen, daß wir doch Muslime sind."\qt@no{64}}%
\qt@newcmd\qurantrans@de@ccclviii{Ihr Schriftbesitzer! Weshalb disputiert ihr über Ibrahim, wo At-taurat und Alindschil erst nach ihm hinabgesandt wurden, denkt ihr denn nicht nach.\qt@no{65}}%
\qt@newcmd\qurantrans@de@ccclix{Ihr habt bereits über das disputiert, worüber ihr Wissen habt. Wieso disputiert ihr denn über das, worüber ihr kein Wissen habt. Und ALLAH weiß, während ihr nicht wißt.\qt@no{66}}%
\qt@newcmd\qurantrans@de@ccclx{Ibrahim war weder Jude noch Nazarener, sondern er war hanif, islam-praktizierend, und er war nie von den Muschrik.\qt@no{67}}%
\qt@newcmd\qurantrans@de@ccclxi{Gewiß, diejenigen unter den Menschen, die das Anrecht darauf haben, Ibrahim als einen der ihren zu bezeichnen, sind diejenigen, die ihm gefolgt sind, dieser Gesandte und diejenigen, die den Iman verinnerlicht haben. Und ALLAH ist Der Wali der Mumin.\qt@no{68}}%
\qt@newcmd\qurantrans@de@ccclxii{Eine Gruppe von den Schriftbesitzern wünschte sich, sie könnte euch in die Irre führen, doch sie führen nur sich selbst irre, ohne daß sie es bemerken.\qt@no{69}}%
\qt@newcmd\qurantrans@de@ccclxiii{Ihr Schriftbesitzer! Weshalb betreibt ihr Kufr ALLAHs Ayat gegenüber, während ihr es wißt?!\qt@no{70}}%
\qt@newcmd\qurantrans@de@ccclxiv{Ihr Schriftbesitzer! Weshalb vermengt ihr die Wahrheit mit dem für nichtig Erklärten und verschweigt die Wahrheit, während ihr wißt?!\qt@no{71}}%
\qt@newcmd\qurantrans@de@ccclxv{Und eine Gruppe von den Schriftbesitzern sagte: ``Bekundet den Iman an das, was den Mumin hinabgesandt wurde zu Beginn des Tages, und betreibt demgegenüber Kufr am Ende des Tages, vielleicht wenden sie sich ab.\qt@no{72}}%
\qt@newcmd\qurantrans@de@ccclxvi{Und glaubt niemandem außer dem, der eurem Din gefolgt ist - sag: ``Gewiß, die (wahre) Rechtleitung ist die Rechtleitung ALLAHs." - damit keinem etwas Ähnliches zuteil wird, was euch zuteil wurde und damit sie mit euch vor eurem HERRN nicht disputieren können." Sag: ``Gewiß, die Gunst liegt bei ALLAH. ER läßt sie zuteil werden, wem ER will." Und ALLAH ist allumfassend, allwissend.\qt@no{73}}%
\qt@newcmd\qurantrans@de@ccclxvii{ER bevorzugt mit Seiner Gnade, wen ER will. Und ALLAH ist von unermeßlicher Gunst.\qt@no{74}}%
\qt@newcmd\qurantrans@de@ccclxviii{Und unter den Schriftbesitzern gibt es manch einen, der, wenn du ihm auch viel (Vermögen) anvertraust, es dir zurückgibt. Und unter ihnen gibt es manch einen, der, wenn du ihm einen einzigen Dinar anvertraust, ihn dir nicht zurückgibt, es sei denn du drängst ihn dazu. Dieses ist so, weil sie doch sagten: ``Was die Nichtjuden anbetrifft, da begehen wir keine Verfehlung." Und sie verbreiten im Namen ALLAHs Lügen, während sie es wissen.\qt@no{75}}%
\qt@newcmd\qurantrans@de@ccclxix{Doch, mit Sicherheit! Wer sein Versprechen einhält und Taqwa gemäß handelt, also gewiß, ALLAH liebt die Muttaqi.\qt@no{76}}%
\qt@newcmd\qurantrans@de@ccclxx{Gewiß, diejenigen, die gegen das von ALLAH Auferlegte und gegen ihre Eide Minderwertiges eintauschen, diese haben keinen Anteil am Jenseits und ALLAH wird zu ihnen nicht sprechen, ihnen am Tag der Auferstehung keine Gnade erweisen und sie nicht läutern. Und für sie ist qualvolle Peinigung bestimmt.\qt@no{77}}%
\qt@newcmd\qurantrans@de@ccclxxi{Und unter ihnen ist eine Gruppe, die mit ihren Zungen die Schrift verdrehen, damit ihr denkt, dies gehöre zur Schrift, obwohl es nicht zur Schrift gehört, noch dazu sagen sie: ``Es sei von ALLAH", doch es ist nicht von ALLAH. Und sie verbreiten Lügen im Namen ALLAHs, während sie es wissen.\qt@no{78}}%
\qt@newcmd\qurantrans@de@ccclxxii{Es gebührt nicht einem Menschen, daß ALLAH ihm die Schrift, die Herrschaft und die Prophetenschaft zuteil werden läßt, und er dann den Menschen sagt: ``Dient mir anstelle von ALLAH.", (Statt dessen wird er sagen:) ``Seid Gottzugehörige aufgrund dessen, was ihr von der Schrift gelehrt habt, und aufgrund dessen, was ihr von der Schrift studiert habt.\qt@no{79}}%
\qt@newcmd\qurantrans@de@ccclxxiii{Auch wird er euch nicht gebieten, daß ihr euch die Engel und die Propheten als Herren annehmt. Würde er euch Kufr gebieten, nachdem ihr Muslime geworden seid?!\qt@no{80}}%
\qt@newcmd\qurantrans@de@ccclxxiv{Und (erinnere daran), als ALLAH das Gelöbnis der Propheten entgegennahm (darüber) - daß unabhängig davon wieviel ICH euch an Schrift undWeisheit zuteil werden lasse, und dann zu euch ein Gesandter kommt als Bestätiger dessen, was ihr habt - daß ihr den Iman an ihn doch verinnerlicht und ihm doch beisteht. ER sagte: ``Bestätigt ihr dieses und stimmt ihr auch dem von Mir Auferlegten zu?" Sie sagten: ``Wir haben es bestätigt." ER sagte: ``Dann bezeugt es, und ICH bin mit euch von den Bezeugenden."\qt@no{81}}%
\qt@newcmd\qurantrans@de@ccclxxv{Und wer sich danach abwendet, diese sind die wirklichen Fasiq.\qt@no{82}}%
\qt@newcmd\qurantrans@de@ccclxxvi{Erstreben sie etwa einen anderen als ALLAHs Din, und für Ihn praktizieren den Islam alle, die in den Himmeln und auf Erden ist - gehorsam oder wider Willen?! Und zu Ihm werden sie zurückgebracht.\qt@no{83}}%
\qt@newcmd\qurantrans@de@ccclxxvii{Sag: ``Wir haben den Iman verinnerlicht an ALLAH, an das was uns hinabgesandt wurde, an das was Ibrahim, Isma'il, Ishaq, Ya'qub und den israilitischen Stämmen hinabgesandt wurde und an das, was Musa, 'Isa und den Propheten von Ihrem HERRN zuteil wurde. Wir machen keinen Unterschied mit keinem von ihnen und wir sind Ihm gegenüber Muslime."\qt@no{84}}%
\qt@newcmd\qurantrans@de@ccclxxviii{Und wer einen anderen Din außer dem Islam erstrebt, dies wird sicher von ihm nicht angenommen, und er gehört im Jenseits zu den Verlierern.\qt@no{85}}%
\qt@newcmd\qurantrans@de@ccclxxix{Wie leitet ALLAH denn Leute recht, die Kufr nach ihrem Iman betrieben, (nachdem) sie bezeugt hatten, daß der Gesandte wahrhaftig ist, und (nachdem) zu ihnen die klaren Zeichen gekommen waren?! Und ALLAH leitet die unrecht-begehenden Leute nicht recht.\qt@no{86}}%
\qt@newcmd\qurantrans@de@ccclxxx{Für diese ist ihre Vergeltung, daß auf ihnen der Fluch von ALLAH, von den Engeln und von den gesamten Menschen lastet.\qt@no{87}}%
\qt@newcmd\qurantrans@de@ccclxxxi{Ewig bleiben sie darin - und ihnen wird weder die Peinigung erleichtert, noch wird ihnen Schonzeit gewährt -\qt@no{88}}%
\qt@newcmd\qurantrans@de@ccclxxxii{außer denjenigen, die danach bereut und gottgefällig Gutes getan haben, so ist ALLAH gewiß allvergebend, allgnädig.\qt@no{89}}%
\qt@newcmd\qurantrans@de@ccclxxxiii{Gewiß, von denjenigen, die Kufr nach ihrem Iman betrieben und dann ihr Kufr noch intensiviert haben, wird keine Reue angenommen. Und diese sind die wirklichen Abirrenden.\qt@no{90}}%
\qt@newcmd\qurantrans@de@ccclxxxiv{Gewiß, von denjenigen, die Kufr betrieben haben und als Kafir starben, von keinem von ihnen wird selbst erden-voll Gold angenommen, auch dann nicht, würde er sich damit freikaufen (wollen). Für diese ist qualvolle Peinigung bestimmt und sie haben keine Beistehenden.\qt@no{91}}%
\qt@newcmd\qurantrans@de@ccclxxxv{Ihr werdet (die Belohnung für) das gottgefällige Handeln nicht erhalten, bis ihr von dem spendet, was ihr liebt. Und egal was ihr spendet, darüber ist ALLAH gewiß allwissend.\qt@no{92}}%
\qt@newcmd\qurantrans@de@ccclxxxvi{Alle Speisen waren für die Kinder Israils halal, ausgenommen dem, was Israil sich selbst für haram erklärte, bevor At-taurat nach und nach hinabgesandt wurde. Sag: ``Holt At-taurat und tragt sie vor, wenn ihr wahrhaftig seid."\qt@no{93}}%
\qt@newcmd\qurantrans@de@ccclxxxvii{Und wer danach im Namen ALLAHs Lügen erfindet, so sind diese die wirklichen Unrecht- Begehenden.\qt@no{94}}%
\qt@newcmd\qurantrans@de@ccclxxxviii{Sag: ``ALLAH hat dieWahrheit gesprochen, so folgt der Gemeinschaft Ibrahims, der hanif war und nicht zu den Muschrik gehörte."\qt@no{95}}%
\qt@newcmd\qurantrans@de@ccclxxxix{Gewiß, das erste Haus, das für dieMenschen (zum Gottesdienst) errichtet wurde, ist sicherlich das in Makka - ein mit Baraka erfülltes Haus und eine Rechtleitung für alle Menschen.\qt@no{96}}%
\qt@newcmd\qurantrans@de@cccxc{In ihm sind deutliche Ayat, (davon) Maqamu-ibrahim. Und wer es betritt, ist in Sicherheit. Und den Menschen ist es ALLAH gegenüber auferlegt, zum Haus zu pilgern - wer von ihnen dazu die Möglichkeit findet. Und wer Kufr betreibt, so ist ALLAH gewiß autark den Geschöpfen gegenüber.\qt@no{97}}%
\qt@newcmd\qurantrans@de@cccxci{Sag: ``Ihr Schriftbesitzer! Weshalb betreibt ihr Kufr ALLAHs Ayat gegenüber?! Und ALLAH ist Zeuge auf das, was ihr tut."\qt@no{98}}%
\qt@newcmd\qurantrans@de@cccxcii{Sag: ``Ihr Schriftbesitzer! Weshalb haltet ihr diejenigen, die den Iman verinnerlicht haben, von dem Gebotenen von ALLAH ab, im Streben danach, es zu verfälschen, während ihr wißt?" Und ALLAH ist nicht achtlos dem gegenüber, was ihr tut.\qt@no{99}}%
\qt@newcmd\qurantrans@de@cccxciii{Ihr, die den Iman verinnerlicht habt! Wenn ihr einer Gruppe von denjenigen, denen die Schrift zuteil wurde, gehorcht, führen sie euch, nachdem ihr den Iman bekundet habt, zum Kufr zurück.\qt@no{100}}%
\qt@newcmd\qurantrans@de@cccxciv{Und wie könnt ihr Kufr betreiben, während euch die Ayat ALLAHs vorgetragen werden, und unter euch der Gesandte ist?! Und wer sich an ALLAH festhält, dieser wurde bereits den geradlinigen Weg rechtgeleitet.\qt@no{101}}%
\qt@newcmd\qurantrans@de@cccxcv{Ihr, die den Iman verinnerlicht habt! Handelt Taqwa gemäß ALLAH gegenüber, wie Ihm gegenüber Taqwa gemäß gehandelt werden soll, und sterbt nur als Muslime.\qt@no{102}}%
\qt@newcmd\qurantrans@de@cccxcvi{Und haltet fest an ALLAHs Din - allesamt, und zersplittert euch nicht! Und entsinnt euch der Gaben ALLAHs euch gegenüber, als ihr verfeindet wart und ER dann eure Herzen versöhnte, und als ihr am Rande der Feuergrube wart und ER euch davor errettet hat. Solcherart verdeutlicht ALLAH euch Seine Ayat, damit ihr Rechtleitung findet.\qt@no{103}}%
\qt@newcmd\qurantrans@de@cccxcvii{Und unter euch soll eine Umma sein, die zum Guten einlädt, zum Gebilligten aufruft und vom Mißbilligten abrät. Und diese sind die wirklichen Erfolgreichen.\qt@no{104}}%
\qt@newcmd\qurantrans@de@cccxcviii{Und seid nicht wie diejenigen, die sich zersplittert haben und uneins wurden, nachdem zu ihnen die klaren Zeichen kamen. Und für diese ist überharte Peinigung bestimmt,\qt@no{105}}%
\qt@newcmd\qurantrans@de@cccxcix{an jenem Tag, an dem manche Gesichter sich erhellen werden und manche Gesichter sich verfinstern werden. Also hinsichtlich derjenigen, deren Gesichter sich verfinstert haben: ``Habt ihr Kufr nach eurem Iman betrieben?! So erfahrt die Peinigung für das, was ihr an Kufr zu betreiben pflegtet."\qt@no{106}}%
\qt@newcmd\qurantrans@de@cd{Doch hinsichtlich derjenigen, deren Gesichter sich erhellt haben, diese befinden sich in ALLAHs Gnade. Darin werden sie ewig bleiben.\qt@no{107}}%
\qt@newcmd\qurantrans@de@cdi{Diese sind ALLAHs Ayat, die WIR dir wahrheitsgemäß vortragen und ALLAH will kein Unrecht für die Geschöpfe.\qt@no{108}}%
\qt@newcmd\qurantrans@de@cdii{Und ALLAH gehört, was in den Himmeln und was auf Erden ist, und an ALLAH werden alle Angelegenheiten gewandt.\qt@no{109}}%
\qt@newcmd\qurantrans@de@cdiii{Ihr wart die beste Umma, die unter den Menschen hervorgebracht wurde, ihr ruft zum Gebilligten auf und ratet vomMißbilligten ab und verinnerlicht den Iman an ALLAH. Und hätten die Schriftbesitzer den Iman verinnerlicht, wäre dies gewiß besser für sie. Unter ihnen sind mancheMumin, aber die meisten von ihnen sind die Fasiq.\qt@no{110}}%
\qt@newcmd\qurantrans@de@cdiv{Sie werden euch außer Belästigung keinen Schaden zufügen. Und sollten sie gegen euch den bewaffneten Kampf aufnehmen, werden sie euch den Rücken flüchtend kehren, dann wird ihnen nicht beigestanden.\qt@no{111}}%
\qt@newcmd\qurantrans@de@cdv{Begleiten wird sie die Erniedrigung, überall wo sie hingehen - es sei denn, (sie halten sich) an das von ALLAH Gebotene und an Verträge mit den Menschen - und sie kehrten mit Erzürnen von ALLAH ab und die Demütigung wurde über sie verhängt, weil sie den Ayat ALLAHs gegenüber Kufr betrieben und die Propheten unrechtmäßig getötet haben. Dies ist für das, was sie an Verfehlungen verübt haben, und für das, was sie an Übertretungen zu begehen pflegten.\qt@no{112}}%
\qt@newcmd\qurantrans@de@cdvi{Sie sind nicht gleich! Unter den Schriftbesitzern gibt es eine standhafte Umma, sie tragen ALLAHs Ayat während der Nacht vor, und während sie Sudschud vollziehen.\qt@no{113}}%
\qt@newcmd\qurantrans@de@cdvii{Sie verinnerlichen den Iman an ALLAH und an den Jüngsten Tag, sie rufen zum Gebilligten auf, raten vom Mißbilligten ab und sind beim (gottgefällig) Guten zuvorkommend. Und diese sind von den gottgefällig Guttuenden.\qt@no{114}}%
\qt@newcmd\qurantrans@de@cdviii{Und was sie an gottgefällig Gutem tun, wird ihnen sicherlich nicht abgeleugnet. Und ALLAH ist allwissend über die Muttaqi.\qt@no{115}}%
\qt@newcmd\qurantrans@de@cdix{Gewiß, denjenigen, die Kufr betrieben haben, werden weder ihre Vermögenswerte noch ihre Kinder vor ALLAH etwas nutzen. Und diese sind die Weggenossen des Feuers. Darin werden sie ewig bleiben.\qt@no{116}}%
\qt@newcmd\qurantrans@de@cdx{Das Gleichnis dessen, was sie in diesem diesseitigen Leben spenden, ist wie das Gleichnis vom Wind, der bittere Kälte bringt und die Äcker von Leuten, die Unrecht begangen haben, heimsucht und dann zerstört. Und ALLAH fügte ihnen kein Unrecht zu, sondern sie fügten sich selbst Unrecht zu.\qt@no{117}}%
\qt@newcmd\qurantrans@de@cdxi{Ihr, die den Iman verinnerlicht habt! Nehmt euch keine Vertrauten außerhalb eurer (eigenen Gruppe), weil sie keine Mühe scheuen, euch ins Verderben zu stürzen, und sich das wünschen, was euch großen Schaden bringt. Ihre Feindschaft haben sie bereits bekundet, doch was ihre Brüste verbergen, ist noch weit schlimmer. Bereits haben WIR euch die Ayat verdeutlicht. Solltet ihr nachdenken, (dann vertraut ihnen nicht!).\qt@no{118}}%
\qt@newcmd\qurantrans@de@cdxii{Da seid ihr, ihr liebt sie, während sie euch nicht lieben, und ihr habt den Iman an die ganze Schrift verinnerlicht. Und wenn sie euch begegnen, sagen sie: Wir haben den Iman verinnerlicht." Doch wenn sie unter sich sind, beißen sie sich in die Fingerspitzen vor Wut gegen euch. Sag: ``Sterbt mit eurer Wut." Gewiß, ALLAH ist allwissend über das, was in den Brüsten ist.\qt@no{119}}%
\qt@newcmd\qurantrans@de@cdxiii{Wenn euch Gutes trifft, werden sie traurig. Wenn euch jedoch Übel überkommt, freuen sie sich darüber. Doch wenn ihr euch in Geduld übt und Taqwa gemäß handelt, wird ihre Heimtücke euch nicht im Geringsten schaden. Gewiß, ALLAH ist dessen, was sie tun, (mit Seinem Wissen) allumfassend.\qt@no{120}}%
\qt@newcmd\qurantrans@de@cdxiv{Und (erinnere daran), als du dich bei Tagesanbruch von deinen Angehörigen aufgemacht hast, um die Mumin in den Kampfstellungen zu postieren. Und ALLAH ist allhörend, allwissend.\qt@no{121}}%
\qt@newcmd\qurantrans@de@cdxv{Und (erinnere daran), als zwei Gruppen unter euch beinahe Rückzieher machten, aber ALLAH war ihr Wali. Und ALLAH gegenüber sollen die Mumin Tawakkul üben.\qt@no{122}}%
\qt@newcmd\qurantrans@de@cdxvi{Und gewiß, bereits hat ALLAH euch in Badr zum Sieg verholfen, während ihr noch schwach wart. So handelt Taqwa gemäß ALLAH gegenüber, damit ihr euch dankbar erweist.\qt@no{123}}%
\qt@newcmd\qurantrans@de@cdxvii{Erinnere daran), als du zu den Mumin gesagt hast: ``Reicht es euch etwa nicht aus, daß euer HERR euch mit dreitausend nach und nach hinabgesandten Engeln Nachschub leistet?!"\qt@no{124}}%
\qt@newcmd\qurantrans@de@cdxviii{Doch mit Sicherheit! Wenn ihr euch in Geduld übt, Taqwa gemäß handelt und sie (die Feinde) euch von nun an angreifen, wird euer HERR euch mit fünftausend kampferprobten Engeln Nachschub leisten.\qt@no{125}}%
\qt@newcmd\qurantrans@de@cdxix{Und ALLAH hat ihn (den Nachschub) nur als frohe Botschaft für euch geschickt und damit eure Herzen dadurch Ruhe finden. Und den Sieg verleiht nur ALLAH, Der Allwürdige, Der Allweise.\qt@no{126}}%
\qt@newcmd\qurantrans@de@cdxx{Damit ER einen Teil von denjenigen, die Kufr betrieben haben, eliminiert oder sie erniedrigt - so werden sie als Versager zurückkehren,\qt@no{127}}%
\qt@newcmd\qurantrans@de@cdxxi{dir unterliegt von der Angelegenheit nichts - oder ihnen vergibt oder sie peinigt. Denn sie sind gewiß Unrecht-Begehende.\qt@no{128}}%
\qt@newcmd\qurantrans@de@cdxxii{Und ALLAH gehört, was in den Himmeln und was auf Erden ist. ER vergibt, wem ER will, und ER peinigt, wen ER will. Und ALLAH ist allvergebend, allgnädig.\qt@no{129}}%
\qt@newcmd\qurantrans@de@cdxxiii{Ihr, die den Iman verinnerlicht habt! Nehmt kein Riba in vervielfacht verdoppelter Höhe ein, und handelt Taqwa gemäß ALLAH gegenüber, damit ihr erfolgreich seid.\qt@no{130}}%
\qt@newcmd\qurantrans@de@cdxxiv{Und sucht Taqwa dem Feuer gegenüber, das für die Kafir vorbereitet ist.\qt@no{131}}%
\qt@newcmd\qurantrans@de@cdxxv{Und gehorcht ALLAH und dem Gesandten, damit euch Gnade erwiesen wird.\qt@no{132}}%
\qt@newcmd\qurantrans@de@cdxxvi{Und eilt zur Vergebung von eurem HERRN und zu einer Dschanna, deren Breite die Himmel und die Erde umfaßt, die für die Muttaqi vorbereitet wurde.\qt@no{133}}%
\qt@newcmd\qurantrans@de@cdxxvii{Es sind diejenigen, die sowohl im Wohlstand als auch in der Not spenden, ihren Zorn unterdrücken und den Menschen verzeihen - und ALLAH liebt die Muhsin -\qt@no{134}}%
\qt@newcmd\qurantrans@de@cdxxviii{sowie diejenigen, die, wenn sie eine Verfehlung verübt oder Unrecht begangen haben, ALLAHs gedenken, dann für ihre Verfehlungen um Vergebung bitten - und wer vergibt die Verfehlungen außer ALLAH?! - und die nicht auf dem beharren, was sie getan haben, während sie wissen.\qt@no{135}}%
\qt@newcmd\qurantrans@de@cdxxix{Für diese ist ihre Belohnung Vergebung von Ihrem HERRN und Dschannat, die von Flüssen durchflossen sind, in denen sie ewig bleiben. Und was für eine schöne Belohnung für die gottgefällig Handelnden!\qt@no{136}}%
\qt@newcmd\qurantrans@de@cdxxx{Bereits vergingen vor euch Vorbilder, so geht auf der Erde umher und betrachtet euch, wie das Anschließende von den Lügnern war.\qt@no{137}}%
\qt@newcmd\qurantrans@de@cdxxxi{Dieser (Quran) ist eine Erklärung für die Menschen, eine Rechtleitung und eine Lehre für die Muttaqi.\qt@no{138}}%
\qt@newcmd\qurantrans@de@cdxxxii{Und werdet nicht müde und grämt euch nicht! Und ihr seid die Sieger, wenn ihr Mumin seid.\qt@no{139}}%
\qt@newcmd\qurantrans@de@cdxxxiii{Sollte euch Verwundung leiden lassen, (so seid getröstet), denn gleiche Verwundung ließ ebenfalls die (gegnerischen) Leute leiden. Und diese Tage des Triumphs lassen wir zwischen den Menschen rotieren. Und damit ALLAH diejenigen, die den Iman verinnerlicht haben, kenntlich macht und manche von euch als Schahid annimmt. Und ALLAH liebt nicht die Unrecht-Begehenden.\qt@no{140}}%
\qt@newcmd\qurantrans@de@cdxxxiv{Und damit ALLAH diejenigen, die den Iman verinnerlicht haben, prüft und die Kafir untergehen läßt.\qt@no{141}}%
\qt@newcmd\qurantrans@de@cdxxxv{Oder habt ihr etwa gedacht, daß ihr in die Dschanna eintretet, ohne daß ALLAH diejenigen kenntlich macht, die von euch Dschihad leisteten, und kenntlich macht die sich in Geduld Übenden?!\qt@no{142}}%
\qt@newcmd\qurantrans@de@cdxxxvi{Und gewiß, bereits habt ihr euch den Tod herbeigesehnt, bevor ihr ihm begegnet seid. So seid ihr bereits ihm begegnet, während ihr (die Gefallenen) anschaut.\qt@no{143}}%
\qt@newcmd\qurantrans@de@cdxxxvii{Und Muhammad ist nichts anderes außer ein Gesandter, vor dem bereits (andere) Gesandte vergangen sind. Kann es sein, daß wenn er stirbt oder getötet wird, daß ihr auf euren Fersen (zum Kufr) umkehrt? Und wer von euch auf seinen Fersen umkehrt, der schadet ALLAH keineswegs. Und ALLAH wird die Dankbaren belohnen.\qt@no{144}}%
\qt@newcmd\qurantrans@de@cdxxxviii{Und kein Mensch wird sterben außer mit ALLAHs Zustimmung - nach einer festgelegten Zeit. Und wer Belohnung für das Diesseits anstrebt, dem geben WIR davon Anteil, und wer Belohnung für das Jenseits anstrebt, dem geben WIR davon Anteil. Und WIR werden die Dankbaren belohnen.\qt@no{145}}%
\qt@newcmd\qurantrans@de@cdxxxix{Und wie viele an Propheten gab es, mit denen viele gottzugewandte Gelehrte gekämpft haben, die weder - aufgrund dessen was sie fi-sabilillah erlitten haben - verzagten, noch schwach wurden, noch sich ihren Feinden unterwarfen. Und ALLAH liebt die sich in Geduld Übenden.\qt@no{146}}%
\qt@newcmd\qurantrans@de@cdxl{Und ihre Worte waren (dabei) lediglich: ``Unser HERR! Vergib uns unsere Verfehlungen und unsere Übertretungen in unseren Angelegenheiten, mache uns standhaft und stehe uns bei gegen die kufr-betreibenden Leute!"\qt@no{147}}%
\qt@newcmd\qurantrans@de@cdxli{So hat ALLAH ihnen die Belohnung des Diesseits und das Schöne an der Belohnung des Jenseits zuteil werden lassen. Und ALLAH liebt die Muhsin.\qt@no{148}}%
\qt@newcmd\qurantrans@de@cdxlii{Ihr, die den Iman verinnerlicht habt! Wenn ihr denjenigen, die Kufr betrieben haben, gehorcht, werden sie euch auf euren Fersen (zum Kufr) umkehren, dann werdet ihr als Verlierer zurückkehren.\qt@no{149}}%
\qt@newcmd\qurantrans@de@cdxliii{Nein, sondern nur ALLAH ist eurer Maula, und ER ist der Beste der Beistehenden.\qt@no{150}}%
\qt@newcmd\qurantrans@de@cdxliv{WIR werden Furcht in die Herzen derjenigen, die Kufr betrieben haben, einwerfen für das, was sie ALLAH gegenüber an Schirk betrieben haben, wozu ER niemals eine Bestätigung hinabsandte. Und ihr Aufenthaltsort ist das Feuer. Und erbärmlich ist der Aufenthaltsort der Unrecht-Begehenden.\qt@no{151}}%
\qt@newcmd\qurantrans@de@cdxlv{Und gewiß, bereits hielt ALLAH euch gegenüber sein Versprechen ein, als ihr sie mit Seiner Zustimmung getötet habt, bis ihr Rückzieher gemacht, euch über die Angelegenheit gestritten und euch widersetzt habt, nachdem ER euch von dem gezeigt hat, was ihr liebt - manche von euch wollen das Diesseits und andere wollen das Jenseits - dann hat ER euch von ihnen (den Kafir) abgelenkt, damit ER euch prüft. Und gewiß, bereits hat ER euch vergeben. Und ALLAH ist den Mumin gegenüber Gunst erweisend.\qt@no{152}}%
\qt@newcmd\qurantrans@de@cdxlvi{Und (erinnere daran), als ihr euch davon gemacht habt, ohne euch nach jemanden umzuschauen, während der Gesandte hinter euch herrief. Dann hat ER es euch mit Kümmernis über Kümmernis vergolten, damit ihr weder dem nachtrauert, was euch (an Beutegütern) entging, noch dem, was euch an Verlusten traf. Und ALLAH ist dessen allkundig, was ihr tut.\qt@no{153}}%
\qt@newcmd\qurantrans@de@cdxlvii{Dann ließ ER euch nach der Kümmernis Sicherheit zukommen, Schlummer, der nur einen Teil von euch überkam. Ein anderer Teil sorgte sich jedoch nur um sich selbst, sie dachten von ALLAH alles andere als das Wahre, eben das Denken der unwissenden Gemeinschaft, sie sagten: ``Haben wir in dieser Angelegenheit irgendwelchen Einfluß?!" Sag: ``Gewiß, alle Angelegenheiten unterliegen ALLAH." Sie verbergen in sich, was sie dir nicht offenbaren, sie sagen: ``Hätten wir in dieser Angelegenheit irgendwelchen Einfluß, würden wir hier nicht getötet." Sag: ``Selbst dann, solltet ihr in euren Häusern sein, würden sich diejenigen, denen das Getötet-Werden bestimmt war, an den Ort ihres Todes begeben." Und (ALLAH ließ es zu), damit ALLAH prüft, was in euren Brüsten ist, und damit ER reinigt, was in euren Herzen ist. Und ALLAH ist allwissend über das, was in den Brüsten ist.\qt@no{154}}%
\qt@newcmd\qurantrans@de@cdxlviii{Gewiß, diejenigen von euch, die geflüchtet sind, als die beiden Kampftruppen aufeinander trafen, diese verleitete der Satan mit etwas von dem, was sie sich (an Verfehlungen) erwarben. Und gewiß, bereits hat ALLAH ihnen vergeben. Gewiß, ALLAH ist allvergebend, allnachsichtig.\qt@no{155}}%
\qt@newcmd\qurantrans@de@cdxlix{Ihr, die den Iman verinnerlicht habt! Seid nicht wie diejenigen, die Kufr betrieben haben und zu ihren Brüdern sagten, als diese sich auf Handelsreisen oder Feldzügen befanden: ``Wären sie bei uns geblieben, wären sie weder gestorben, noch getötet worden." (Sagt es nicht), damit ALLAH dies als Bitterkeit (nur) in ihren Herzen bleiben läßt. Und ALLAH (alleine) macht lebendig und läßt sterben. Und ALLAH ist dessen, was ihr tut, allsehend.\qt@no{156}}%
\qt@newcmd\qurantrans@de@cdl{Und solltet ihr fisabilillah getötet werdet oder (eines natürlichen Todes) sterben, so ist Vergebung von ALLAH und Gnade mit Sicherheit besser als das, was sie sich (an Vermögenswerten) anhäufen.\qt@no{157}}%
\qt@newcmd\qurantrans@de@cdli{Und solltet ihr (eines natürlichen Todes) sterben oder getötet werden, so werdet ihr sicherlich vor ALLAH versammelt werden.\qt@no{158}}%
\qt@newcmd\qurantrans@de@cdlii{Und (nur) auf Grund der Gnade von ALLAH warst du ihnen gegenüber sanftmütig. Doch wärest du barsch, hartherzig gewesen, hätten sie dich gewiß verlassen. So verzeih ihnen, bitte um Vergebung für sie und berate dich mit ihnen über die ganze Angelegenheit. Und solltest du dich dann entschieden haben, so übe Tawakkul ALLAH gegenüber. Gewiß, ALLAH liebt die Tawakkul-Übenden.\qt@no{159}}%
\qt@newcmd\qurantrans@de@cdliii{Wenn ALLAH euch zum Sieg verhilft, so gibt es keinen, der euch besiegen kann. Doch sollte ER euch die Unterstützung verweigern, wer ist dieser, der euch nach Ihm zum Sieg verhelfen kann. Und ALLAH gegenüber sollen sich die Mumin in Tawakkul üben!\qt@no{160}}%
\qt@newcmd\qurantrans@de@cdliv{Es gebührt keinem Propheten, daß er jemals etwas (von den Beutegütern) unterschlägt. Und wer etwas unterschlägt, der kommt am Tag der Auferstehung mit dem, was er unterschlagen hat, dann wird jeder Seele das vergolten, was sie sich erworben hat, und ihnen wird kein Unrecht zugefügt.\qt@no{161}}%
\qt@newcmd\qurantrans@de@cdlv{Ist denn derjenige, der nach dem Wohlgefallen von ALLAH strebt, gleich demjenigen, der mit Erzürnen von ALLAH zurückkehrt und sein Aufenthaltsort Dschahannam ist?! Und erbärmlich ist das Werden.\qt@no{162}}%
\qt@newcmd\qurantrans@de@cdlvi{Diese werden bei ALLAH verschieden eingestuft. Und ALLAH ist dessen allsehend, was sie tun.\qt@no{163}}%
\qt@newcmd\qurantrans@de@cdlvii{Gewiß, bereits erwies ALLAH den Mumin Gutes, als ER ihnen einen Gesandten schickte, der zu ihnen gehört, der ihnen Seine Ayat vorträgt, sie läutert und ihnen die Schrift und die Weisheit lehrt. Und sie waren vor ihm zweifelsohne im offenkundigen Irregehen.\qt@no{164}}%
\qt@newcmd\qurantrans@de@cdlviii{War es nicht so, daß erst dann, als euch ein Unglück traf, nachdem ihr bereits das Zweifache gleicher Art (den Feinden) zugefügt habt, ihr gesagt habt: ``Woher kommt das?" Sag: ``Es wurde von euch selbst verschuldet." Gewiß, ALLAH ist über alles allmächtig.\qt@no{165}}%
\qt@newcmd\qurantrans@de@cdlix{Und das, was euch traf, als die beiden Kampftruppen aufeinandertrafen, geschah nur mit ALLAHs Zustimmung, damit ER die Mumin kenntlich macht,\qt@no{166}}%
\qt@newcmd\qurantrans@de@cdlx{und damit ER diejenigen, die Nifaq betrieben haben, kenntlich macht. Und es wurde zu ihnen gesagt: ``Kommt her und kämpft fisabilillah oder haltet (die Feinde zumindest) zurück!" Sie sagten: ``Wenn wir wüßten, daß es zum Kampf kommen würde, würden wir euch sicherlich folgen." Damals waren sie dem Kufr näher als dem Iman. Sie sagen mit ihren Mündern, was nicht in ihren Herzen ist. Doch ALLAH weiß besser, was sie verschweigen.\qt@no{167}}%
\qt@newcmd\qurantrans@de@cdlxi{Diejenigen, die von ihren Brüdern sagten, während sie selbst zurückblieben: ``Hätten sie auf uns gehört, wären sie nicht getötet worden." Sag: ``Dann wendet von euch selbst den Tod ab, so ihr wahrhaftig seid."\qt@no{168}}%
\qt@newcmd\qurantrans@de@cdlxii{Und denkt nicht, daß diejenigen, die fi-sabilillah getötet wurden, Tote seien. Nein, sondern Lebendige sind sie! Bei ihrem HERRN wird ihnen Rizq gewährt.\qt@no{169}}%
\qt@newcmd\qurantrans@de@cdlxiii{Voller Freude sind sie über das, was ALLAH ihnen von Seiner Gunst zuteil werden ließ, auch freuen sie sich über die frohe Botschaft von denjenigen, die ihnen noch nicht gefolgt und zurückgeblieben sind, daß es um diese weder Angst gibt, noch werden sie traurig sein.\qt@no{170}}%
\qt@newcmd\qurantrans@de@cdlxiv{Sie freuen sich über die frohe Botschaft von einer Gabe von ALLAH und Gunst und darüber, daß ALLAH gewiß keine Belohnung für die Mumin verlorengehen läßt,\qt@no{171}}%
\qt@newcmd\qurantrans@de@cdlxv{die (der Aufforderung) ALLAHs und des Gesandten gefolgt sind, nachdem die Verwundung sie leiden ließ. Für diejenigen unter ihnen, die Ihsan und Taqwa gemäß gehandelt haben, ist übergroße Belohnung bestimmt.\qt@no{172}}%
\qt@newcmd\qurantrans@de@cdlxvi{Diejenigen, denen die Menschen sagten: ``Gewiß, die Menschen zogen bereits gegen euch Kampftruppen zusammen, so fürchtet euch vor ihnen!" Dies aber hat ihren Iman verstärkt, und sie sagten: ``Uns genügt ALLAH, Der besteWakil."\qt@no{173}}%
\qt@newcmd\qurantrans@de@cdlxvii{So kehrten sie mit Gabe von ALLAH und Gunst um, ohne daß auch nur (ein Hauch von) Übel sie berührte. Und sie strebten nach dem Wohlgefallen ALLAHs. Und ALLAH ist von unermeßlicher Gunst.\qt@no{174}}%
\qt@newcmd\qurantrans@de@cdlxviii{Dies war nichts anderes als der Satan, er macht seinen Wali Angst. So fürchtet sie nicht! Doch fürchtet (nur) Mich, solltet ihr Mumin sein.\qt@no{175}}%
\qt@newcmd\qurantrans@de@cdlxix{Und sei nicht traurig wegen denjenigen, die zum Kufr eilen. Gewiß, diese schaden ALLAH in keiner Weise. ALLAH will ihnen keinen Anteil am Jenseits gewähren. Und für sie ist überharte Peinigung bestimmt.\qt@no{176}}%
\qt@newcmd\qurantrans@de@cdlxx{Gewiß, diejenigen, die Kufr gegen Iman eingetauscht haben, schaden ALLAH in keiner Weise und für sie ist eine qualvolle Peinigung bestimmt.\qt@no{177}}%
\qt@newcmd\qurantrans@de@cdlxxi{Und diejenigen, die Kufr betrieben haben, sollen nicht denken, daß wenn WIR ihnen Zeit gewähren, daß dies für sie besser sei. WIR gewähren ihnen Zeit nur, damit sie noch mehr Verfehlungen begehen. Und für sie ist eine erniedrigende Peinigung bestimmt.\qt@no{178}}%
\qt@newcmd\qurantrans@de@cdlxxii{ALLAH gebührt es niemals, die Mumin so zu lassen, wie ihr gegenwärtig seid, bis ER die Schlechten von den Guten auseinander trennt. Und ALLAH gebührt es niemals, euch einen Einblick in das Verborgene zu gewähren. Doch ALLAH erwählt von Seinen Gesandten aus, wen ER will. So verinnerlicht den Iman an ALLAH und an Seine Gesandten! Und wenn ihr den Iman verinnerlicht und Taqwa gemäß handelt, so ist für euch übergroße Belohnung bestimmt.\qt@no{179}}%
\qt@newcmd\qurantrans@de@cdlxxiii{Diejenigen, die geizen mit dem, was ALLAH ihnen von Seiner Gunst zuteil werden ließ, sollen nicht denken, daß dies besser für sie wäre. Nein, sondern das ist übel für sie. Und das, womit sie geizten, wird ihnen am Tag der Auferstehung um den Hals geschlungen werden. Und ALLAH gehört die Hinterlassenschaft der Himmel und der Erde. Und ALLAH ist dessen, was ihr tut, allkundig.\qt@no{180}}%
\qt@newcmd\qurantrans@de@cdlxxiv{Gewiß, bereits hat ALLAH das Gerede derjenigen gehört, die gesagt haben: ``Gewiß, ALLAH ist arm und wir sind reich." WIR werden das niederschreiben lassen, was sie gesagt haben, sowie ihr Töten von Propheten zu Unrecht, undWIR werden sagen: ``Kostet die Peinigung des Feuers!"\qt@no{181}}%
\qt@newcmd\qurantrans@de@cdlxxv{Dies ist so wegen dem, was ihr euch eigenhändig erworben habt. Und gewiß, ALLAH ist in keiner Weise den Dienern gegenüber ein Ungerechter.\qt@no{182}}%
\qt@newcmd\qurantrans@de@cdlxxvi{Diejenigen, die sagten: ``ALLAH hat uns doch auferlegt, daß wir keinen Iman einem Gesandten schenken, bis er uns eine Opfergabe bringt, die das Feuer verbrennt." Sag: ``Zu euch kamen bereits vor mir Gesandte mit deutlichen Zeichen und mit dem, was ihr gesagt habt. Weshalb habt ihr sie denn getötet, wenn ihr wahrhaftig seid?"\qt@no{183}}%
\qt@newcmd\qurantrans@de@cdlxxvii{Und sollten sie dich der Lüge bezichtigen, so wurden bereits Gesandte vor dir der Lüge bezichtigt, sie kamen mit den deutlichen Zeichen, mit Az-zubur und mit der erleuchtenden Schrift.\qt@no{184}}%
\qt@newcmd\qurantrans@de@cdlxxviii{Jedes Lebewesen wird den Tod erfahren. Und ihr werdet eure Belohnungen doch nur am Tag der Auferstehung erhalten. Derjenige, der dann vom Feuer ferngehalten wird und in die Dschanna hineingelassen wird, war bereits erfolgreich. Und das diesseitige Leben ist nichts anderes außer Verbrauchsgut der Täuschung.\qt@no{185}}%
\qt@newcmd\qurantrans@de@cdlxxix{Und zweifelsohne werdet ihr mit eurem Vermögen und mit euch selbst geprüft. Und zweifelsohne werdet ihr von denjenigen, denen die Schrift vor euch zuteil wurde, und von denjenigen, die Schirk betrieben, viele Kränkungen hören. Und wenn ihr euch in Geduld übt und Taqwa gemäß handelt, dann gehört dies gewiß zu den von ALLAH gebotenen Angelegenheiten.\qt@no{186}}%
\qt@newcmd\qurantrans@de@cdlxxx{Und (erinnere daran), als ALLAH das Gelöbnis derjenigen entgegennahm, denen die Schrift zuteil wurde: ``Ihr werdet sie doch den Menschen erläutern und sie nicht verbergen." Dann haben sie ihr (der Schrift) jedoch den Rücken gekehrt und sie gegen Minderwertiges eingetauscht. Und erbärmlich ist das, was sie eingetauscht haben.\qt@no{187}}%
\qt@newcmd\qurantrans@de@cdlxxxi{Denkt nicht, daß diejenigen, die sich über das freuen, was ihnen zuteil wurde, und es lieben, gelobt zu werden für das, was sie nicht getan haben, denkt nicht, daß diese von der Peinigung gerettet werden. Und für sie ist qualvolle Peinigung bestimmt.\qt@no{188}}%
\qt@newcmd\qurantrans@de@cdlxxxii{Und ALLAH gehört, was in den Himmeln und auf Erden ist. Und ALLAH ist über alles allmächtig.\qt@no{189}}%
\qt@newcmd\qurantrans@de@cdlxxxiii{Gewiß, in der Schöpfung der Himmel und der Erde und in der Verschiedenheit von Nacht und Tag sind zweifelsohne Ayat für die Verständigen.\qt@no{190}}%
\qt@newcmd\qurantrans@de@cdlxxxiv{Diejenigen, die ALLAHs im Stehen, im Sitzen und liegend auf ihren Seiten gedenken und über die Schöpfung der Himmel und der Erde nachdenken: ``Unser HERR, DU hast dies nicht umsonst erschaffen, gepriesen-erhaben bist Du, so bewahre uns vor der Peinigung des Feuers!\qt@no{191}}%
\qt@newcmd\qurantrans@de@cdlxxxv{Unser HERR! Gewiß, wen DU ins Feuer eintreten läßt, den hast DU bereits mit Sicherheit erniedrigt. Und für die Unrecht-Begehenden gibt es keine Unterstützer.\qt@no{192}}%
\qt@newcmd\qurantrans@de@cdlxxxvi{Unser HERR! Wir haben einen Rufenden gehört, der zum Iman ruft: ``Verinnerlicht den Iman an euren HERRN", so haben wir den Iman verinnerlicht. Unser HERR! So vergib uns unsere Verfehlungen, tilge für uns unsere gottmißfälligen Taten und lasse uns unter den gottgefällig Gütigen sterben!\qt@no{193}}%
\qt@newcmd\qurantrans@de@cdlxxxvii{Unser HERR! Und lasse uns das zuteil werden, was DU uns über Deine Gesandten versprochen hast und erniedrige uns nicht am Tag der Auferstehung! Gewiß, DU brichst nie Dein Versprechen."\qt@no{194}}%
\qt@newcmd\qurantrans@de@cdlxxxviii{Dann erhörte sie ihr HERR: ``Gewiß, ICH lasse keine Tat eines Tuenden von euch, ob männlich oder weiblich, verlorengehen, die einen von euch sind wie die anderen. Denjenigen, die Hidschra unternommen haben, von ihren Häusern vertrieben wurden, für das von Mir Gebotene Schaden erlitten haben, gekämpft haben und getötet wurden, diesen werde ICH zweifelsohne ihre gottmißfälligen Taten vergeben, und ICH werde sie doch eintreten lassen in Dschannat, die von Flüssen durchflossen sind, als Belohnung von ALLAH." Und bei ALLAH befindet sich das Schöne an Belohnung.\qt@no{195}}%
\qt@newcmd\qurantrans@de@cdlxxxix{Dich soll nicht täuschen das Umherreisen der Kafir in den Ländern.\qt@no{196}}%
\qt@newcmd\qurantrans@de@cdxc{Das ist ein kurzweiliges Vergnügen, dann wird ihr Aufenthaltsort Dschahannam sein. Und erbärmlich ist die Bleibe.\qt@no{197}}%
\qt@newcmd\qurantrans@de@cdxci{Doch für diejenigen, die ihrem HERRN gegenüber Taqwa gemäß handelten, sind Dschannat bestimmt, die von Flüssen durchflossen sind, dort werden sie ewig bleiben - dies als Wohnstätte (für sie) von ALLAH. Und was sich bei ALLAH befindet, ist besser für die gottgefällig Handelnden.\qt@no{198}}%
\qt@newcmd\qurantrans@de@cdxcii{Und unter den Schriftbesitzern gibt es gewiß manche, die den Iman an ALLAH verinnerlichen, an das, was euch hinabgesandt wurde, und an das, was ihnen hinabgesandt wurde. Sie sind ALLAH gegenüber ergeben und erkaufen sich mit ALLAHs Ayat nichts Minderwertiges. Diese haben ihre Belohnung bei ihrem HERRN. Gewiß, ALLAH ist schnell im Zur-Rechenschaft-Ziehen.\qt@no{199}}%
\qt@newcmd\qurantrans@de@cdxciii{Ihr, die den Iman verinnerlicht habt! Übt euch in Geduld, seid standhaft nach Kräften, haltet euch in Bereitschaft und handelt Taqwa gemäß ALLAH gegenüber, damit ihr erfolgreich werdet.\qt@no{200}}%
\qt@newcmd\qurantrans@de@cdxciv{\basmalahde Ihr Menschen! Handelt Taqwa gemäß eurem HERRN gegenüber, Der euch aus einem einzigen Wesen geschaffen hat, aus ihm sein Partnerwesen geschaffen hat und aus beiden viele Männer und Frauen vermehren ließ. Und handelt Taqwa gemäß ALLAH gegenüber, Dessenthalben ihr euch gegenseitig bittet, sowie gegenüber (dem Brechen) der Verwandtschaftsbande. Gewiß, ALLAH bleibt euch gegenüber immer aufmerksam.\qt@no{1}}%
\qt@newcmd\qurantrans@de@cdxcv{Und gebt den Waisen ihr Vermögen, tauscht nicht das Schlechte gegen das Gute ein, und nehmt nicht ihr Vermögen zu eurem Vermögen! Gewiß, dies wäre eine schwere Verfehlung.\qt@no{2}}%
\qt@newcmd\qurantrans@de@cdxcvi{Und solltet ihr es fürchten, den Waisen (als Ehefrauen) gegenüber nicht gerecht zu sein, so heiratet diejenigen, die euch lieb sind von den (sonstigen) Frauen, zwei, drei oder vier. Und solltet ihr fürchten, nicht gerecht zu sein, so (heiratet) nur eine, oder von denjenigen, die euch gehören. Das ist näher dazu, daß ihr nicht übertretet.\qt@no{3}}%
\qt@newcmd\qurantrans@de@cdxcvii{Und gebt den Frauen ihreMorgengabe als Nihla. Und sollten sie für euch auf etwas davon freiwillig verzichten, dann nehmt es als Gutes Unschädliches an.\qt@no{4}}%
\qt@newcmd\qurantrans@de@cdxcviii{Und gebt den Unerfahrenen (in Vermögensangelegenheiten) nicht euer Vermögen, das ALLAH euch als Lebensgrundlage zur Verfügung gestellt hat. Jedoch versorgt sie dadurch, bekleidet sie und sagt ihnen ein gütiges Wort!\qt@no{5}}%
\qt@newcmd\qurantrans@de@cdxcix{Und prüft die Waisen, bis sie das Heiratsalter erreicht haben. Und wenn ihr bei ihnen vernünftiges Verhalten feststellt, dann übergebt ihnen ihr Vermögen, und verbraucht es nicht auf verschwenderische Art und nicht übereilt, bevor sie das Alter erreichen. Und wer reich ist, so soll er verzichten, und wer arm ist, so soll er davon nach dem Gebilligten nehmen. Und wenn ihr ihnen ihr Vermögen übergebt, dann zieht Zeugen dafür hinzu. Und ALLAH genügt als Abrechnender.\qt@no{6}}%
\qt@newcmd\qurantrans@de@d{Den Männern gehört ein Teil dessen, was die Eltern und Verwandten hinterlassen. Und den Frauen gehört ein Teil dessen, was die Eltern und Verwandten hinterlassen haben - ob wenig oder viel - ein Pflichtanteil.\qt@no{7}}%
\qt@newcmd\qurantrans@de@di{Und sollten Verwandte, Bedürftige und Arme bei der Aufteilung (des Nachlasses) anwesend sein, so gebt ihnen davon und sagt ihnen ein gütiges Wort!\qt@no{8}}%
\qt@newcmd\qurantrans@de@dii{Und Ehrfurcht sollen diejenigen haben, die, wenn sie nach ihrem Verscheiden minderjährige Kinder hinterlassen würden, um diese besorgt wären. So sollen sie Taqwa gemäß ALLAH gegenüber handeln und ein angebrachtes Wort sprechen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@diii{Gewiß, diejenigen, die das Vermögen der Waisen zu Unrecht aufbrauchen, verzehren in ihren Bäuchen nur Feuer. Und sie werden in die Gluthitze hineingeworfen werden.\qt@no{10}}%
\qt@newcmd\qurantrans@de@div{ALLAH gebietet euch hinsichtlich eurer Kinder, dem Kind männlichen Geschlechts das Gleiche (an Erbteilen) zu geben wie zwei Kindern weiblichen Geschlechts. Und sollten sie (die Kinder) nur Frauen und mehr als zwei sein, so bekommen sie Zweidrittel dessen, was man hinterlassen hat. Und sollte es sich nur um eine einzige Frau handeln, so bekommt sie die Hälfte. Und für seine Eltern, jedes Elternteil bekommt ein Sechstel dessen, was man hinterlassen hat, wenn man Kinder hat. Wenn man jedoch keine Kinder hat und (nur) seine Eltern ihn beerben, dann bekommt seine Mutter das Drittel. Und wenn er Geschwister hat, dann bekommt seine Mutter das Sechstel nach (der Vollstreckung) des Testaments, das man hinterlegt hat, und nach (der Begleichung) der Schulden. - Eure Eltern und eure Kinder, ihr wißt nicht, welche von ihnen euch am ehesten nützen. - Dies ist ein Gebot von ALLAH. Gewiß, ALLAH bleibt allwissend, allweise.\qt@no{11}}%
\qt@newcmd\qurantrans@de@dv{Und ihr bekommt die Hälfte dessen, was eure Ehefrauen hinterlassen haben, wenn sie keine Kinder haben. Sollten sie jedoch Kinder haben, dann bekommt ihr das Viertel von dem, was sie hinterlassen haben, nach (der Vollstreckung) des Testaments, das sie hinterlegt haben, und nach (der Begleichung) der Schulden. Und sie (die Ehefrauen) bekommen das Viertel von dem, was ihr hinterlassen habt, wenn ihr keine Kinder habt. Solltet ihr aber Kinder haben, dann bekommen sie das Achtel von dem, was ihr hinterlassen habt, nach (der Vollstreckung) des Testaments, das ihr hinterlegt habt, und nach (der Begleichung) der Schulden. Und sollte (der Verstorbene) ein Mann gewesen sein, der als Kalala beerbt wird - oder eine Frau, und er (oder sie) (mütterlicherseits) einen Bruder oder eine Schwester haben, dann bekommt jeder von ihnen das Sechstel. Sollten sie jedoch mehr als das vorher Erwähnte sein, dann sind sie Partner im Drittel. (Dieses erfolgt erst) nach (der Vollstreckung) des Testaments, das ihr hinterlegt habt, und nach (der Begleichung) der Schulden, ohne (die Erbberechtigten) zu schädigen. Dies ist ein Gebot von ALLAH. Und ALLAH ist allwissend, allnachsichtig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@dvi{Diese sind ALLAHs Richtlinien. Und wer ALLAH und seinem Gesandten gehorcht, den wird ER in Dschannat eintreten lassen, die von Flüssen durchflossen sind, dort werden sie ewig bleiben. Und dies ist der übergroße Erfolg.\qt@no{13}}%
\qt@newcmd\qurantrans@de@dvii{Doch wer ALLAH und Seinem Gesandten widerspricht und ALLAHs Richtlinien überschreitet, den wird ER ins Feuer eintreten lassen, darin wird er ewig bleiben. Und für ihn ist eine erniedrigende Peinigung bestimmt.\qt@no{14}}%
\qt@newcmd\qurantrans@de@dviii{Und diejenigen von euren Frauen, die das Abscheuliche betreiben, so bringt gegen sie vier Augen- zeugen von euch. Und wenn diese ein Zeugnis abgelegt haben, dann sperrt sie (die Frauen) in den Häusern ein, bis sie sterben oder ALLAH ihnen einen Ausweg macht.\qt@no{15}}%
\qt@newcmd\qurantrans@de@dix{Und diejenigen von euch, die sie (die Unzucht) begehen, sollt ihr anprangern. Und wenn sie danach bereuen und gottgefällig Gutes tun, dann laßt von ihnen ab! Gewiß, ALLAH bleibt reue-annehmend, allgnädig.\qt@no{16}}%
\qt@newcmd\qurantrans@de@dx{ALLAH nimmt die Reue nur von denjenigen an, die das Schlechte aus Unwissenheit tun und danach umgehend bereuen. Von diesen nimmt ALLAH die Reue an. Und ALLAH bleibt immer allwissend, allweise.\qt@no{17}}%
\qt@newcmd\qurantrans@de@dxi{Und die Reue wird nicht von denjenigen angenommen, die so lange die gottmißfälligen Taten begehen, bis der Eine von ihnen im Sterben liegt und erst dann sagt: ``Ich bereue es jetzt.", und nicht von denjenigen, die sterben, während sie Kafir sind. Für diese bereiteten WIR qualvolle Peinigung vor.\qt@no{18}}%
\qt@newcmd\qurantrans@de@dxii{Ihr, die den Iman verinnerlicht habt! Für euch gilt nicht als halal, die Frauen durch Zwang zu beerben und ihnen (die Wiederheirat) zu verbieten, um ihnen einen Teil von dem wegzunehmen, was ihr ihnen habt zuteil werden lassen, es sei denn, sie begehen nachgewiesene unzüchtige Handlung. Und verkehrt mit ihnen nach dem Gebilligten! Und solltet ihr gegen sie Abneigung empfinden, dann kann es sein, daß ihr einer Sache gegenüber Abneigung empfindet, in die ALLAH jedoch viel Gutes für euch gelegt hat.\qt@no{19}}%
\qt@newcmd\qurantrans@de@dxiii{Und wenn ihr eine Ehefrau anstelle einer (anderen) Ehefrau heiraten wollt und ihr einer von ihnen eine großzügige (Brautgabe) gegeben habt, dann nehmt ihnen davon nichts weg! Wollt ihr es (ihnen) etwa in betrügerischer und eindeutig verfehlter Weise wegnehmen?!\qt@no{20}}%
\qt@newcmd\qurantrans@de@dxiv{Und wie könnt ihr es ihnen wegnehmen, nachdem ihr bereits miteinander intim wart und sie mit euch einen verbindlichen Vertrag eingegangen sind?!\qt@no{21}}%
\qt@newcmd\qurantrans@de@dxv{Und heiratet nicht diejenigen Frauen, die eure Väter geheiratet haben, es sei denn, was bereits geschehen ist. Dies ist gewiß eine Abscheulichkeit und eine Ekligkeit, und erbärmlich ist dieser Weg.\qt@no{22}}%
\qt@newcmd\qurantrans@de@dxvi{Euch wurden für haram (zum Heiraten) erklärt: Eure Mütter, eure Töchter, eure Schwestern, eure Tanten väterlicherseits, eure Tanten mütterlicherseits, die Töchter des Bruders, die Töchter der Schwester, eure Ammen, die euch gestillt haben, eure Milchschwestern, die Mütter eurer Ehefrauen, eure Stieftöchter, die in eurem Haushalt leben, die zu den Ehefrauen gehören, mit denen ihr bereits intim wart, und wenn ihr mit ihnen noch nicht intim wart, dann ist es für euch keine Verfehlung, (diese Stieftöchter zu heiraten), und die Ehefrauen eurer leiblichen Söhne, und daß ihr zwei Schwestern gleichzeitig heiratet, es sei denn, was bereits geschehen ist. Gewiß, ALLAH bleibt immer allvergebend, allgnädig.\qt@no{23}}%
\qt@newcmd\qurantrans@de@dxvii{Haram für die Heirat sind ebenso) die bereits verheirateten Frauen, es sei denn, sie gehören euch. Dies ist das euch von ALLAH Gebotene. Und euch wurden alle anderen (Frauengruppen) für halal (zum Heiraten) erklärt, damit ihr mit eurem Vermögen (das Heiraten) anstrebt als Ehemänner und nicht als Unzucht-Treibende. Und wen ihr von ihnen heiratet, so gebt ihnen ihre Morgengabe - eine Pflichtgabe. Und es ist für euch keine Verfehlung wegen dem, worüber ihr euch einigt außerhalb der Pflichtgabe. Gewiß, ALLAH bleibt immer allwissend, allweise.\qt@no{24}}%
\qt@newcmd\qurantrans@de@dxviii{Und wer von euch es sich nicht leisten kann, freie Mumin-Frauen zu heiraten, dann (heiratet er) von den Mumin-Dienerinnen, die euch gehören. Und ALLAH kennt euren Iman besser - die einen von euch sind wie die anderen. So heiratet sie mit der Zustimmung ihrer Angehörigen und gebt ihnen ihre Morgengabe nach dem Gebilligten als Ehefrauen und nicht als Unzucht-Treibende und nicht als diejenigen, die sich Geliebte nehmen. Und wenn sie (die Mumin-Dienerinnen) verheiratet sind und dann Unzucht begehen, dann ist ihnen die Hälfte des Strafmaßes auferlegt, das für freie Frauen vorgesehen ist. Dies ist für denjenigen von euch, der fürchtet, ansonsten Schädlichkeit (Unzucht) zu begehen. Und wenn ihr euch in Geduld übt, ist es besser für euch. Und ALLAH ist allvergebend, allgnädig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@dxix{ALLAH will es euch verdeutlichen, euch zu Vorgehensweisen derjenigen, die vor euch waren, rechtleiten und eure Reue annehmen. Und ALLAH ist allwissend, allweise.\qt@no{26}}%
\qt@newcmd\qurantrans@de@dxx{Und ALLAH will eure Reue annehmen. Doch diejenigen, die ihren Neigungen folgen, wollen, daß ihr massiv (vom rechten Weg) abweicht.\qt@no{27}}%
\qt@newcmd\qurantrans@de@dxxi{ALLAH will es euch erleichtern. Und der Mensch wurde als schwaches (Geschöpf) geschaffen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@dxxii{Ihr, die den Iman verinnerlicht habt! Eignet euch euer Vermögen untereinander nicht durch das für nichtig Erklärte an! Nicht verboten ist es, (es euch anzueignen) durch einen Handel, den ihr in gegenseitigem Einvernehmen abschließt. Auch begeht keinen Selbstmord! Gewiß, ALLAH bleibt immer euch gegenüber allgnädig.\qt@no{29}}%
\qt@newcmd\qurantrans@de@dxxiii{Und wer dies aus Übertretung und zu Unrecht tut, den werden WIR ins Feuer hineinwerfen lassen. Und dies ist für ALLAH etwas Leichtes.\qt@no{30}}%
\qt@newcmd\qurantrans@de@dxxiv{Wenn ihr die Kabiras dessen meidet, das euch verboten wurde, werden WIR euch eure gottmißfälligen Taten tilgen und euch in einen ehrenwerten Eingang eintreten lassen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@dxxv{Und wünscht euch nicht das, womit ALLAH den einen von euch vor den anderen ausgezeichnet hat! Die Männer haben Anteil an dem, was sie sich erworben haben, und die Frauen haben Anteil an dem, was sie sich erworben haben. Und bittet ALLAH um etwas von Seiner Gunst! Gewiß, ALLAH bleibt immer über alles allwissend.\qt@no{32}}%
\qt@newcmd\qurantrans@de@dxxvi{Und für jeden (von euch) haben WIR Erbberechtigte bestimmt für das, was die Eltern und die Verwandten hinterlassen haben. Und gebt denjenigen, mit denen ihr einen Vertrag geschlossen habt, ihren Anteil! Gewiß, ALLAH bleibt immer über alles Zeuge.\qt@no{33}}%
\qt@newcmd\qurantrans@de@dxxvii{Die Ehemänner tragen Verantwortung den Ehefrauen gegenüber wegen dem, womit ALLAH die einen vor den anderen ausgezeichnet hat, und wegen dem, was sie von ihrem Vermögen ausgegeben haben. Die gottgefällig guttuenden Frauen sind (ALLAH gegenüber) ergeben und bewahren das vom Verborgenen (zwischen ihnen und ihren Ehemännern), was ALLAH zu bewahren auferlegt hat. Und diejenigen Ehefrauen, deren böswillige trotzige Auflehnung ihr fürchtet, diese sollt ihr (zunächst) ermahnen, dann in den Ehebetten meiden und (erst danach) einen (leichten) Klaps geben! Und sollten sie wieder auf euch hören, dann unternehmt nichts mehr gegen sie! Gewiß, ALLAH bleibt immer allhöchst, allgrößt.\qt@no{34}}%
\qt@newcmd\qurantrans@de@dxxviii{Und wenn ihr Streitigkeit zwischen ihnen (den Eheleuten) fürchtet, dann schaltet einen Schiedsmann von seinen Angehörigen und einen Schiedsmann von ihren Angehörigen ein. Und wenn beide Versöhnung wollen, wird ALLAH beide erfolgreich sein lassen. Gewiß, ALLAH bleibt immer allwissend, allkundig.\qt@no{35}}%
\qt@newcmd\qurantrans@de@dxxix{Und dient ALLAH und betreibt Ihm gegenüber keinerlei Schirk! Und die Eltern behandelt Ihsan gemäß und die Verwandten, Waisen, Bedürftigen, den verwandten Nachbarn, den nicht-verwandten Nachbarn, den nächsten Gefährten, den Reisenden und diejenigen, die euch gehören. Gewiß, ALLAH liebt nicht denjenigen, der eingebildet, arrogant ist.\qt@no{36}}%
\qt@newcmd\qurantrans@de@dxxx{Es sind diejenigen, die geizen, den Menschen Geiz gebieten und verschweigen, was ALLAH ihnen von Seiner Gunst zuteil werden ließ, - und WIR haben für die Kafir erniedrigende Peinigung vorbereitet. -\qt@no{37}}%
\qt@newcmd\qurantrans@de@dxxxi{so wie diejenigen, die ihr Vermögen aus Riyaa den Menschen gegenüber spenden, und weder den Iman an ALLAH noch an den Jüngsten Tag verinnerlichen. Und wer den Satan als ständigen Begleiter hat, dann erbärmlich ist sein ständiger Begleiter.\qt@no{38}}%
\qt@newcmd\qurantrans@de@dxxxii{Und was wäre dabei, wenn sie den Iman an ALLAH und an den Jüngsten Tag verinnerlicht und vom Rizq gegeben hätten, das ALLAH ihnen gewährte?! Und ALLAH ist immer über sie allwissend.\qt@no{39}}%
\qt@newcmd\qurantrans@de@dxxxiii{Gewiß, ALLAH fügt kein Unrecht zu, auch nicht das Gewicht eines Stäubchens. Und wäre es (nur) eine (einzige) gute Tat, diese vervielfacht ER, und gibt von Sich aus eine übergroße Belohnung.\qt@no{40}}%
\qt@newcmd\qurantrans@de@dxxxiv{Und wie würde es sein, wenn WIR von jeder Umma einen Zeugen bringen würden und dich (Muhammad) als einen Zeugen gegen diese bringen würden?!\qt@no{41}}%
\qt@newcmd\qurantrans@de@dxxxv{An diesem Tag werden diejenigen, die Kufr betrieben und dem Gesandten widersprochen haben, sich wünschen, sie wären mit der Erde gleich. Und sie werden vor ALLAH kein Wort verheimlichen können.\qt@no{42}}%
\qt@newcmd\qurantrans@de@dxxxvi{Ihr, die den Iman verinnerlicht habt! Nähert euch nicht dem rituellen Gebet, wenn ihr betrunken seid - damit ihr wißt, was ihr sagt -, auch nicht, wenn ihr dschunub seid - mit Ausnahme von Vorbeigehenden (bzw. Reisenden), bis ihr Ghusl durchgeführt habt. Und wenn ihr krank oder auf Reisen seid oder der eine von euch von der Notdurft kommt oder ihr die Frauen (intim) berührt habt und kein Wasser finden könnt, dann sucht reine Erdoberfläche und streicht euch über eure Gesichter und Hände (bis zu den Ellbogen). Gewiß, ALLAH bleibt immer reue-annehmend, allvergebend.\qt@no{43}}%
\qt@newcmd\qurantrans@de@dxxxvii{Hast du etwa nicht diejenigen gesehen, denen ein Teil der Schrift gegeben wurde, sie erkaufen sich das Irregehen und wollen, daß auch ihr irregeht?!\qt@no{44}}%
\qt@newcmd\qurantrans@de@dxxxviii{Und ALLAH weiß besser Bescheid über eure Feinde. Und es genügt ALLAH als Wali, und es genügt ALLAH als Verhelfer zum Sieg.\qt@no{45}}%
\qt@newcmd\qurantrans@de@dxxxix{Unter den Juden gibt es einige, welche die Worte (der Schrift) verdrehen und sagen: ``Wir haben gehört und widersprochen, höre, hättest du doch nie gehört und ra'ina ``, als Verdrehung mit ihren Zungen und als Beschimpfung des Din. Und hätten sie doch gesagt: ``Wir haben gehört und gehorcht, höre uns zu und unzurna!", wäre das besser für sie und verantwortungsvoller. Aber ALLAH hat sie wegen ihres Kufr verflucht, so verinnerlichen sie den Iman nur ein wenig.\qt@no{46}}%
\qt@newcmd\qurantrans@de@dxl{Ihr, denen die Schrift zuteil wurde! Verinnerlicht den Iman an das, was WIR hinabgesandt haben als Bestätigung dessen, was ihr habt, bevor WIR (Konturen) von Gesichtern verwischen und diese ihren Hinterköpfen gleich machen oder sie verfluchen, wie WIR diejenigen vom Sabbat verflucht haben. Und ALLAHs Entscheidung wird immer umgesetzt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@dxli{Gewiß, ALLAH vergibt nie, daß Ihm gegenüber Schirk betrieben wird! Doch ER vergibt, was (an Verfehlungen) geringer ist als dies, wem ER will. Und wer Schirk ALLAH gegenüber betreibt, der hat bereits eine übergroße Verfehlung erlogen.\qt@no{48}}%
\qt@newcmd\qurantrans@de@dxlii{Hast du etwa nicht diejenigen wahrgenommen, die von sich selbst (behaupten), geläutert zu sein?! Nein, sondern ALLAH erklärt als geläutert, wen ER will. Und ihnen wird nicht im Geringsten Unrecht angetan.\qt@no{49}}%
\qt@newcmd\qurantrans@de@dxliii{Siehe, wie sie Lügen im Namen ALLAHs erfinden. Und dies genügt als eine eindeutige Verfehlung.\qt@no{50}}%
\qt@newcmd\qurantrans@de@dxliv{Hast du etwa nicht diejenigen gesehen, denen ein Teil der Schrift gegeben wurde, wie sie den Iman an Adsch-dschibt und At-taghut bekunden und zu denjenigen, die Kufr betrieben haben, sagen: ``Diese sind rechtgeleiteter in der Lebensweise als diejenigen, die den Iman verinnerlicht haben."?!\qt@no{51}}%
\qt@newcmd\qurantrans@de@dxlv{Diese sind diejenigen, die ALLAH verflucht hat. Und wen ALLAH verflucht, für den wirst du keinen Beistehenden finden.\qt@no{52}}%
\qt@newcmd\qurantrans@de@dxlvi{Oder würden sie vielleicht Anteil an der Herrschaft (ALLAHs) besitzen?! Dann würden sie den Menschen nicht das Geringste davon geben.\qt@no{53}}%
\qt@newcmd\qurantrans@de@dxlvii{Oder beneiden sie etwa die Menschen für das, was ALLAH ihnen von Seiner Gunst zuteil werden ließ?! So ließen WIR bereits der Familie Ibrahims die Schrift und die Weisheit zuteil werden. Auch ließen WIR ihnen ein mächtiges Königtum zuteil werden.\qt@no{54}}%
\qt@newcmd\qurantrans@de@dxlviii{Dann haben einige von ihnen den Iman an ihn (Muhammad) verinnerlicht, und einige von ihnen haben sich von ihm abgewendet. Und Dschahannam genügt als Gluthitze.\qt@no{55}}%
\qt@newcmd\qurantrans@de@dxlix{Gewiß, diejenigen, die Kufr Unseren Ayat gegenüber betreiben, werden WIR in ein Feuer hineinwerfen lassen. Immer wieder, wenn ihre Häute gargekocht sein werden, werden WIR sie ihnen gegen andere Häute austauschen, damit sie die Peinigung (richtig) erfahren. Gewiß, ALLAH bleibt immer allwürdig, allweise.\qt@no{56}}%
\qt@newcmd\qurantrans@de@dl{Und diejenigen, die den Iman verinnerlicht haben und gottgefällig Gutes taten, werden WIR in Dschannat eintreten lassen, die von Flüssen durchflossen sind, dort werden sie für immer und ewig bleiben. Dort haben sie gereinigte Partnerwesen, und WIR lassen sie in nicht vergehenden Schatten eintreten.\qt@no{57}}%
\qt@newcmd\qurantrans@de@dli{Gewiß, ALLAH gebietet euch, daß ihr die euch anvertrauten Dinge ihren Besitzern zurückgebt, und wenn ihr unter den Menschen richtet, daß ihr mit Gerechtigkeit richtet. Und sicherlich gut ist das, wozu ALLAH euch ermahnt. Gewiß, ALLAH bleibt immer allhörend, allsehend.\qt@no{58}}%
\qt@newcmd\qurantrans@de@dlii{Ihr, die den Iman verinnerlicht habt! Gehorcht ALLAH und gehorcht dem Gesandten und den Verantwortlichen unter euch. Und wenn ihr euch über eine Angelegenheit streitet, dann legt sie ALLAH und seinem Gesandten vor, solltet ihr den Iman an ALLAH und an den Jüngsten Tag verinnerlicht haben. Dies ist besser und hat einen besseren Abschluß.\qt@no{59}}%
\qt@newcmd\qurantrans@de@dliii{Hast du etwa nicht diejenigen wahrgenommen, die behaupten, sie hätten den Iman verinnerlicht an das, was dir hinabgesandt wurde, und was vor dir hinabgesandt wurde, wie sie sich At-taghut widmen wollen, um zwischen ihnen zu richten, obgleich ihnen bereits geboten wurde, ihm (At-taghut ) gegenüber Kufr zu betreiben?! Doch der Satan will sie in weite Irre gehen lassen.\qt@no{60}}%
\qt@newcmd\qurantrans@de@dliv{Und wenn zu ihnen gesagt wird: ``Kommt zu dem, was ALLAH hinabgesandt hat und zum Gesandten", hast du gesehen, wie die Munafiq sich von dir mit Entschiedenheit abwenden.\qt@no{61}}%
\qt@newcmd\qurantrans@de@dlv{Und wie wird es sein, wenn ein Unglück sie heimsucht aufgrund dessen, was sie eigenhändig getan haben, dann sie zu dir kommen und schwören im Namen ALLAHs: ``Wir wollten nichts anderes außer Gutem und Versöhnlichem"?!\qt@no{62}}%
\qt@newcmd\qurantrans@de@dlvi{Diese sind diejenigen, über deren Herzen ALLAH Bescheid weiß, so wende dich von ihnen ab, ermahne sie und sage ihnen bezogen auf sie selbst aufrüttelnde Worte!\qt@no{63}}%
\qt@newcmd\qurantrans@de@dlvii{Und WIR haben jeden Gesandten nur deshalb entsandt, damit ihm mit der Zustimmung ALLAHs gehorcht wird. Und wären sie doch, nachdem sie Unrecht begangen hatten, zu dir gekommen und hätten ALLAH um Vergebung gebeten und hätte dann der Gesandte für sie um Vergebung gebeten, würden sie ALLAH als reue-annehmend, allgnädig finden.\qt@no{64}}%
\qt@newcmd\qurantrans@de@dlviii{Nein, bei deinem HERRN! Sie werden den Iman nicht verinnerlichen, bis sie dich über das richten lassen, was zwischen ihnen strittig ist, und dann von sich aus keine Abneigung dem gegenüber empfinden, was du entschieden hast, und sich deiner (Entscheidung) widerspruchslos fügen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@dlix{Und hätten WIR ihnen geboten: ``ihr sollt euch selbst töten oder eure Wohnstätten verlassen, würden dies nur wenige von ihnen tun." Und hätten sie doch das getan, womit sie ermahnt werden, wäre es besser für sie und würde sie noch mehr (im Iman) bestärken.\qt@no{66}}%
\qt@newcmd\qurantrans@de@dlx{Auch dann hätten WIR ihnen gewiß von Uns übergroße Belohnung zukommen lassen,\qt@no{67}}%
\qt@newcmd\qurantrans@de@dlxi{ebenfalls hätten WIR sie gewiß zu einem geraden Weg rechtgeleitet.\qt@no{68}}%
\qt@newcmd\qurantrans@de@dlxii{Und wer ALLAH und dem Gesandten gehorcht, diese sind unter denjenigen, denen ALLAH gute Gaben erwiesen hat, von den Propheten, Wahrhaftigen, Schahid und den gottgefällig Guttuenden. Und diese wurden vom guten Begleiter begleitet.\qt@no{69}}%
\qt@newcmd\qurantrans@de@dlxiii{Diese Gunst erweist ALLAH. Und ALLAH genügt als Allwissender.\qt@no{70}}%
\qt@newcmd\qurantrans@de@dlxiv{Ihr, die den Iman verinnerlicht habt! Nehmt euch in Acht, so folgt dem Aufruf der Mobilmachung in Gruppen oder alle gemeinsam!\qt@no{71}}%
\qt@newcmd\qurantrans@de@dlxv{Und gewiß, manch einer von euch zögert. Und wenn ein Unglück euch heimsucht, sagt er: ``ALLAH hat mir bereits eine Wohltat erwiesen, daß ich unter ihnen nicht anwesend war."\qt@no{72}}%
\qt@newcmd\qurantrans@de@dlxvi{Doch wenn euch Gunst von ALLAH zukommt, wird er gewiß sagen, als ob zwischen euch und ihm keine Verbindung bestanden hätte: ``Wäre ich doch mit ihnen gewesen, damit ich ebenso einen übergroßen Erfolg hätte."\qt@no{73}}%
\qt@newcmd\qurantrans@de@dlxvii{So sollen fi-sabilillah diejenigen den bewaffneten Kampf führen, die das diesseitige Leben gegen das Jenseits eintauschen. Und wer fi-sabilillah an dem bewaffneten Kampf teilnimmt, dann getötet wird oder siegt, dem werden WIR eine übergroße Belohnung zuteil werden lassen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@dlxviii{Und weshalb führt ihr nicht den bewaffneten Kampf fi-sabilillah und für die Unterdrückten von den Männern, Frauen und Kindern, die sagen: ``Unser HERR! Bringe uns heraus aus diesem Ort, dessen Leute Übertretende sind, und bestimme uns von Dir aus einen Wali, und bestimme uns von Dir aus einen Beistehenden"?!\qt@no{75}}%
\qt@newcmd\qurantrans@de@dlxix{Diejenigen, die den Iman verinnerlicht haben, führen den bewaffneten Kampf fi-sabilillah, und diejenigen, die Kufr betrieben haben, führen den bewaffneten Kampf für At-taghut. So führt den bewaffneten Kampf gegen die Wali des Satan! Gewiß, das Intrigieren des Satans bleibt ohnehin immer schwach.\qt@no{76}}%
\qt@newcmd\qurantrans@de@dlxx{Hast du etwa nicht diejenigen wahrgenommen, denen gesagt wurde: ``Haltet euch zurück, verrichtet ordnungsgemäß das rituelle Gebet und entrichtet die Zakat"?! Doch als ihnen der bewaffnete Kampf geboten wurde, fürchtete ein Teil von ihnen sich so sehr vor den Menschen, wie sie sich nur vor ALLAHs (Peinigung) fürchten sollten oder gar noch mehr. Und sie sagten: ``Unser HERR! Weshalb hast DU uns den bewaffneten Kampf geboten? Hättest DU uns doch zurückgestellt, bis zum ohnehin nahenden Tod!" Sag: ``Die Gebrauchsgüter im diesseitigen Leben sind wenig und das Jenseits ist besser für jeden, der Taqwa gemäß handelt. Und ihr werdet nicht das Geringste an Unrecht erleiden."\qt@no{77}}%
\qt@newcmd\qurantrans@de@dlxxi{Wo auch immer ihr zu sein pflegtet, der Tod wird euch doch ereilen, auch dann wenn ihr in hohen Burgen wäret. Und wenn sie Gutes trifft, sagen sie: ``Dies ist von ALLAH." Und wenn sie ein Unglück trifft, sagen sie: ``Dies ist von dir." Sag: ``Alles ist von ALLAH." Weshalb verstehen diese Leute kaum das (zu ihnen) Gesprochene?!\qt@no{78}}%
\qt@newcmd\qurantrans@de@dlxxii{Was dich an Gutem traf, dies ist von ALLAH. Und was dich an Unglück traf, dies ist aufgrund eigener (Verfehlungen). Und wir entsandten dich für die Menschen als Gesandten. Und ALLAH genügt als Bezeugender.\qt@no{79}}%
\qt@newcmd\qurantrans@de@dlxxiii{Wer dem Gesandten gehorcht, der hat bereits ALLAH gehorcht. Doch wer sich abwendet, (siehe) WIR haben dich zu ihnen nicht als Aufseher entsandt.\qt@no{80}}%
\qt@newcmd\qurantrans@de@dlxxiv{Und sie sagen: ``(Wir geloben dir) Gehorsam." Doch wenn sie von dir weggegangen sind, denkt sich nachts ein Teil von ihnen etwas anderes aus als das, was sie sagen. Und ALLAH läßt das niederschreiben, was sie sich nachts ausdenken. So wende dich von ihnen ab und übe Tawakkul ALLAH gegenüber! Und ALLAH genügt als Wakil.\qt@no{81}}%
\qt@newcmd\qurantrans@de@dlxxv{Denken sie etwa nicht über den Quran nach?! Und wäre er von einem anderen als ALLAH, gewiß hätten sie darin viele Widersprüche gefunden.\qt@no{82}}%
\qt@newcmd\qurantrans@de@dlxxvi{Und wenn zu ihnen eine Nachricht über etwas Sicherheitsgewährendes oder Angsteinflössendes kommt, verbreiten sie dieses. Und hätten sie es dem Gesandten und den Verantwortlichen unter ihnen überlassen, hätten davon Kenntnis diejenigen unter ihnen, die dieses analysieren können. Und gäbe es ALLAHs Gunst euch gegenüber nicht sowie Seine Gnade, wäret ihr dem Satan gefolgt, außer wenigen von euch.\qt@no{83}}%
\qt@newcmd\qurantrans@de@dlxxvii{Also führe den bewaffneten Kampf fi-sabilillah! Dir ist nur auferlegt, dich selbst dazu (zu verpflichten). Doch mobilisiere die Mumin, damit ALLAH die Gewalttätigkeit derjenigen, die Kufr betrieben haben, zurückhält. Und ALLAH ist noch fähiger im Gewalt-Antun und noch härter im Peinigen.\qt@no{84}}%
\qt@newcmd\qurantrans@de@dlxxviii{Wer eine gottgefällig gute Fürbitte einlegt, hat daran Anteil. Und wer für etwas Verwerfliches Fürbitte einlegt, hat daran Anteil. Und ALLAH bleibt immer über alles überwachend.\qt@no{85}}%
\qt@newcmd\qurantrans@de@dlxxix{Und wenn ihr mit einem Gruß gegrüßt werdet, dann grüßt zurück mit einem Besseren als ihm oder gebt ihn in gleicher Weise zurück. Gewiß, ALLAH bleibt immer über alles Zur-Rechenschaft-Ziehender.\qt@no{86}}%
\qt@newcmd\qurantrans@de@dlxxx{Er ist ALLAH, es gibt keine Gottheit außer Ihm, ER wird euch gewiß versammeln zum Tag der Auferstehung - daran gibt es keinerlei Zweifel! Und wer ist dieser, der wahrhaftiger als ALLAH spricht?!\qt@no{87}}%
\qt@newcmd\qurantrans@de@dlxxxi{Und weshalb wurdet ihr hinsichtlich der Munafiq zwei Gruppen, während ALLAH sie, wegen, dem was sie sich erworben haben, (zum Kufr) hat umkehren lassen?! Wollt ihr etwa diejenigen rechtleiten, die ALLAH abirren ließ?! Und wen ALLAH abirren läßt, für den findest du keinen Weg (zur Rechtleitung).\qt@no{88}}%
\qt@newcmd\qurantrans@de@dlxxxii{Sie wünschten sich, daß ihr genauso Kufr betreibt, wie sie Kufr betrieben haben, damit ihr gleich werdet. So nehmt euch von ihnen keine alsWali, bis sie fisabilillah Hidschra unternehmen. Und sollten sie sich dennoch abwenden, dann nehmt sie gefangen und tötet sie überall da, wo ihr sie findet, - und nehmt euch von ihnen weder Wali noch Verhelfer zum Sieg. -\qt@no{89}}%
\qt@newcmd\qurantrans@de@dlxxxiii{außer denjenigen, die zu Leuten flüchten, mit denen ihr einen Vertrag habt, oder wenn sie zu euch kommen, während sie darüber bekümmert sind, daß sie gegen euch oder gegen ihre Leute kämpfen. Und hätte ALLAH es gewollt, hätte ER sie gegen euch aufgebracht, damit sie euch bekämpfen. Und wenn sie euch meiden, nicht gegen euch kämpfen und euch den Salam anbieten, so hat ALLAH euch gegen sie keine (weitere) Möglichkeit eingeräumt.\qt@no{90}}%
\qt@newcmd\qurantrans@de@dlxxxiv{Ihr werdet auch andere finden, die vor euch sicher sein wollen und vor ihren Leuten sicher sein wollen. Immer wieder, wenn sie zur Fitna aufgefordert wurden, sind sie auch rückfällig geworden. Und wenn sie euch nicht meiden, euch den Salam nicht anbieten und sich nicht zurückhalten, so nehmt sie gefangen und tötet sie überall dort, wo ihr sie ergreifen könnt. Und gegen diese haben WIR euch eindeutige Verfügung gegeben.\qt@no{91}}%
\qt@newcmd\qurantrans@de@dlxxxv{Und es gebührt nicht einem Mumin, einen anderen Mumin zu töten, es sei denn versehentlich. Und wer einen Mumin versehentlich tötet, (für den gilt) die Befreiung eines Mumin-Sklaven und eine an seine Angehörigen zu zahlende Diya, es sei denn, sie erlassen sie. Und wenn er (der Getötete) Leuten angehörte, die euch gegenüber feindselig eingestellt sind, aber er selbst Mumin war, dann (gilt) die Befreiung eines Mumin-Sklaven. Und wenn er (der Getötete) Leuten angehörte, mit denen ihr einen Vertrag habt, dann (gilt) eine an seine Angehörigen zu zahlende Diya und die Befreiung eines Mumin-Sklaven. Und wer (dies) nicht aufbringen kann, fastet an zwei aufeinanderfolgenden Monaten als Reue vor ALLAH. Und ALLAH bleibt immer allwissend, allweise.\qt@no{92}}%
\qt@newcmd\qurantrans@de@dlxxxvi{Und wer einen Mumin vorsätzlich tötet, seine Vergeltung ist Dschahannam, darin wird er ewig bleiben. ALLAH zürnt über ihn, verflucht ihn und hat für ihn eine qualvolle Peinigung vorbereitet.\qt@no{93}}%
\qt@newcmd\qurantrans@de@dlxxxvii{Ihr, die den Iman verinnerlicht habt! Wenn ihr euch fi-sabilillah erhebt, prüft es nach und sagt nicht demjenigen, der euch mit Salam-Gruß grüßt: ``Du bist kein Mumin." - im Streben nach dem Vergänglichen des diesseitigen Lebens. ALLAH hat aber Vieles, woran ihr gewinnen könnt. Gleichermaßen wart ihr vorher, ALLAH aber hat euch Güte zuteil werden lassen, so prüft es nach. Gewiß, ALLAH bleibt immer dem gegenüber, was ihr tut, allkundig.\qt@no{94}}%
\qt@newcmd\qurantrans@de@dlxxxviii{Die (dem Dschihad) Fernbleibenden unter den Mumin - außer den Behinderten, sind nicht gleichzusetzen mit den fi-sabilillah Dschihad-Leistenden mit ihrem Vermögen und eigenem Leib. ALLAH gab den Dschihad-Leistenden mit ihrem Vermögen und eigenem Leib eine Stellung mehr als den (dem Dschihad) Fern-Bleibenden. Und beiden Gruppen hat ALLAH Gutes versprochen. Doch ALLAH begünstigte die Dschihad-Leistenden vor den (dem Dschihad) Fern- Bleibenden mit übergroßer Belohnung -\qt@no{95}}%
\qt@newcmd\qurantrans@de@dlxxxix{höheren Stellungen vom Ihm, Vergebung und Gnade. Und ALLAH bleibt immer allvergebend, allgnädig.\qt@no{96}}%
\qt@newcmd\qurantrans@de@dxc{Gewiß, denjenigen, deren Ableben die Engel bewirkt haben, während sie sich selbst noch Unrecht antun, zu denen sagten (die Engel): ``Woran wart ihr?" Sie sagten: ``Wir waren im Lande unterdrückt." Sie sagten: ``War denn ALLAHs Erde nicht groß genug, damit ihr darauf auswandert?!" Für diese ist Dschahannam als Unterkunft bestimmt. Und erbärmlich ist sie als (ihr) Werden.\qt@no{97}}%
\qt@newcmd\qurantrans@de@dxci{Ausgenommen sind die Unterdrückten von den Männern, Frauen und Kindern, die keinen Ausweg haben und keine Möglichkeit (zur Hidschra) finden.\qt@no{98}}%
\qt@newcmd\qurantrans@de@dxcii{Diesen möge ALLAH vergeben. Und ALLAH bleibt immer reue-annehmend, allvergebend.\qt@no{99}}%
\qt@newcmd\qurantrans@de@dxciii{Und wer Hidschra fi-sabilillah unternimmt, wird auf Erden viele Zufluchtmöglichkeiten und Erleichterung finden. Und wer seine Wohnung als Hidschra- Unternehmer zu ALLAH und Seinem Gesandten verläßt und unterwegs vom Tod ereilt wird, dessen Belohnung ist bereits bei ALLAH sicher. Und ALLAH bleibt immer allvergebend, allgnädig.\qt@no{100}}%
\qt@newcmd\qurantrans@de@dxciv{Und wenn ihr durch das Land umherzieht, ist es für euch keine Verfehlung, wenn ihr das rituelle Gebet verkürzt, wenn ihr fürchtet, daß diejenigen, die Kufr betrieben haben, euch der Fitna aussetzen. Gewiß, die Kafir bleiben euch immer entschiedene Feinde.\qt@no{101}}%
\qt@newcmd\qurantrans@de@dxcv{Und wenn du unter ihnen bist und sie dann mit Iqama zum rituellen Gebet rufst, dann soll eine Gruppe von ihnen mit dir Qiyam vollziehen und ihre Waffen bei sich behalten. Und wenn diese Sudschud vollziehen, dann sollen die anderen hinter ihnen stehen, und dann soll eine andere Gruppe, die das rituelle Gebet noch nicht verrichtet hat, kommen und das rituelle Gebet mit dir verrichten. Und sie sollen sich in Acht nehmen und ihre Waffen bei sich behalten. Diejenigen, die Kufr betrieben haben, wünschen sich: ``Würdet ihr doch euren Waffen und Sachen gegenüber achtlos werden", dann würden sie euch durch einen überraschenden Angriff überrumpeln. Und es ist für euch keine Verfehlung, wenn ihr Schaden durch Regen habt oder ihr krank wart, daß ihr eure Waffen liegen laßt. Doch nehmt euch in Acht! Gewiß, ALLAH hat für die Kafir eine erniedrigende Peinigung vorbereitet.\qt@no{102}}%
\qt@newcmd\qurantrans@de@dxcvi{Und wenn ihr dann das rituelle Gebet vollendet habt, dann gedenkt ALLAHs im Stehen, im Sitzen und im Liegen (auf der Seite). Und wenn ihr euch sicher fühlt, dann verrichtet das rituelle Gebet ordnungsgemäß! Gewiß, das (Verrichten des) Gebets bleibt den Mumin immer eine durch Zeitpunkte bestimmte Verpflichtung.\qt@no{103}}%
\qt@newcmd\qurantrans@de@dxcvii{Und werdet nicht schwach im Ergreifen der (feindlichen) Leute. Solltet ihr leiden, so leiden sie gewiß ebenso wie ihr leidet. Doch ihr erhofft euch von ALLAH, was sie sich nicht erhoffen. Und ALLAH bleibt immer allwissend, allweise.\qt@no{104}}%
\qt@newcmd\qurantrans@de@dxcviii{Gewiß, WIR haben dir die Schrift mit der Wahrheit hinabgesandt, damit du zwischen den Menschen richtest mit dem, was ALLAH dir erklärt hat. Und sei den Untreuen kein Mitstreiter!\qt@no{105}}%
\qt@newcmd\qurantrans@de@dxcix{Und bitte ALLAH um Vergebung! Gewiß, ALLAH bleibt immer allvergebend, allgnädig.\qt@no{106}}%
\qt@newcmd\qurantrans@de@dc{Und führe keine Streitgespräche stellvertretend für diejenigen, die sich selbst untreu verhalten. Gewiß, ALLAH liebt keinen, der als Untreuer, Verfehlender bleibt.\qt@no{107}}%
\qt@newcmd\qurantrans@de@dci{Sie verstecken sich vor den Menschen, und nicht verstecken sie sich vor ALLAH, obwohl ER mit ihnen ist, wenn sie sich nachts das ausdenken, was Ihm an Reden nicht gefällt. Und ALLAH bleibt alledem, was sie tun, allumfassend.\qt@no{108}}%
\qt@newcmd\qurantrans@de@dcii{Da habt ihr selbst stellvertretend für sie im diesseitigen Leben disputiert, wer wird denn stellvertretend für sie vor ALLAH am Tag der Auferstehung disputieren, oder wer wird für sie ein Wakil sein?!\qt@no{109}}%
\qt@newcmd\qurantrans@de@dciii{Und wer Verwerfliches tut oder sich selbst Unrecht antut, dann ALLAH um Vergebung bittet, der wird ALLAH als allvergebend, allgnädig finden.\qt@no{110}}%
\qt@newcmd\qurantrans@de@dciv{Und wer sich eine Verfehlung erwirbt, der belastet damit nur sich selbst. Und ALLAH bleibt immer allwissend, allweise.\qt@no{111}}%
\qt@newcmd\qurantrans@de@dcv{Und wer sich einen Fehler oder eine Verfehlung erwirbt und dann einen Unschuldigen damit belastet, der hat bereits eine große Unterstellung und eine klare Verfehlung begangen.\qt@no{112}}%
\qt@newcmd\qurantrans@de@dcvi{Und gäbe es nicht ALLAHs Gunst für dich und Seine Gnade, hätte eine Gruppe von ihnen dich beinahe in die Irre geführt. Doch sie führen nur sich selbst in die Irre und sie schaden dir dabei in keiner Weise. Und ALLAH hat dir die Schrift hinabgesandt und die Weisheit und hat dich gelehrt, was du nicht wußtest. Und ALLAHs Gunst dir gegenüber ist gewiß unermeßlich.\qt@no{113}}%
\qt@newcmd\qurantrans@de@dcvii{Nichts Nützliches liegt in den meisten ihrer heimlichen Unterhaltungen, außer wer aufruft zu einer Spende, zum Gebilligten oder zur Versöhnung zwischen den Menschen. Und wer dies im Trachten nach ALLAHs Wohlgefallen tut, dem werden WIR eine übergroße Belohnung zuteil werden lassen.\qt@no{114}}%
\qt@newcmd\qurantrans@de@dcviii{Und wer dem Gesandten widerspricht - nachdem ihm die Rechtleitung deutlich wurde, und einem anderen als dem Weg der Mumin folgt, den werden WIR das verantworten lassen, wofür er verantwortlich war, und dann werden WIR ihn in Dschahannam hineinwerfen lassen. Und erbärmlich ist dieses Werden.\qt@no{115}}%
\qt@newcmd\qurantrans@de@dcix{Gewiß, ALLAH vergibt nicht, daß Ihm gegenüber Schirk betrieben wird. Doch ER vergibt, was geringer ist als dies, wem ER will. Und wer Schirk ALLAH gegenüber betreibt, ist bereits weit in die Irre gegangen.\qt@no{116}}%
\qt@newcmd\qurantrans@de@dcx{Sie richten Bittgebete statt an Ihn doch nur an weibliche (Götzen). Und sie richten Bittgebete nur an einen auflehnerischen Satan.\qt@no{117}}%
\qt@newcmd\qurantrans@de@dcxi{Den hat ALLAH verflucht. Und er sagte: ``Ich werde doch von Deinen Dienern einen bestimmten Anteil unterwerfen,\qt@no{118}}%
\qt@newcmd\qurantrans@de@dcxii{sie zweifelsohne in die Irre führen, in ihnen Illusionen wecken, ihnen gebieten, die Ohren von An'am einzuschneiden und ihnen gebieten, die Schöpfung ALLAHs zu verändern." Und wer sich den Satan als Wali anstelle von ALLAH nimmt, hat bereits einen klaren Verlust erlitten.\qt@no{119}}%
\qt@newcmd\qurantrans@de@dcxiii{Er (Satan) macht ihnen Versprechungen und weckt in ihnen Illusionen. Und der Satan verspricht ihnen nur betrügerischen Schein.\qt@no{120}}%
\qt@newcmd\qurantrans@de@dcxiv{Für diese ist Dschahannam als Unterkunft bestimmt, und davon finden sie kein Entkommen.\qt@no{121}}%
\qt@newcmd\qurantrans@de@dcxv{Und diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, werden WIR in Dschannat eintreten lassen, die von Flüssen durchflossen sind, darin werden sie für immer und ewig bleiben. Dies ist ALLAHs Versprechen, ein Wahrhaftiges. Und wer ist dieser, der wahrhaftiger als ALLAH spricht?!\qt@no{122}}%
\qt@newcmd\qurantrans@de@dcxvi{Dieses ist unabhängig) sowohl von euren Wünschen, als auch von den Wünschen der Schriftbesitzer. Wer Verwerfliches tut, dem wird dementsprechend vergolten, und er wird anstelle von ALLAH weder einen Wali noch einen Beistehenden finden.\qt@no{123}}%
\qt@newcmd\qurantrans@de@dcxvii{Und wer vom gottgefällig Guten tut, ob Mann oder Frau, und Mumin ist, diese werden in die Dschanna eintreten und nicht das Geringste an Unrecht erleiden.\qt@no{124}}%
\qt@newcmd\qurantrans@de@dcxviii{Und wer folgt einem besseren Din als derjenige, der sich ALLAH hingab, während er gottgefällig Gutes tut und der Gemeinschaft Ibrahims folgt, der hanif war. Und ALLAH nahm Ibrahim als Auserwählten an.\qt@no{125}}%
\qt@newcmd\qurantrans@de@dcxix{Und ALLAH gehört, was in den Himmeln und auf Erden ist. Und ALLAH bleibt immer über alles allumfassend.\qt@no{126}}%
\qt@newcmd\qurantrans@de@dcxx{Und sie fragen dich nach einer Fatwa die Frauen betreffend. Sag: ``ALLAH gibt euch Fatwa sie betreffend, ebenfalls (gibt euch Fatwa) das, was euch vorgetragen wird von der Schrift über die weiblichen Waisen - denen ihr nicht gebt, was für sie vorgeschrieben wurde, und die ihr heiraten (bzw. nicht heiraten) wollt - sowie über die hilflosen kleinen Kinder. Und (ER gebietet euch), daß ihr eure Verantwortung den Waisen gegenüber in gerechter Weise erfüllt." Und was ihr an gottgefällig Gutem tut, darüber bleibt ALLAH gewiß immer allwissend.\qt@no{127}}%
\qt@newcmd\qurantrans@de@dcxxi{Sollte eine Frau von ihrem Ehemann Auflehnung oder Vernachlässigung fürchten, dann ist es für beide keine Verfehlung, wenn sie sich durch eine Vereinbarung aussöhnen. Und die Aussöhnung ist besser. Doch die Seelen sind der Habsucht zugeneigt. Und wenn ihr gottgefällig Gutes tut und Taqwa gemäß handelt, so bleibt ALLAH immer dem gegenüber, was ihr tut, gewiß allkundig.\qt@no{128}}%
\qt@newcmd\qurantrans@de@dcxxii{Ihr werdet den Ehefrauen gegenüber nicht gerecht sein können, möget ihr auch noch so sehr darauf bedacht sein. So neigt euch nicht (einer Ehefrau) voll zu (bzw. ab), damit ihr sie nicht wie eine laßt, die in der Schwebe ist. Doch wenn ihr dies korrigiert und Taqwa gemäß handelt, so bleibt ALLAH gewiß immer allvergebend, allgnädig.\qt@no{129}}%
\qt@newcmd\qurantrans@de@dcxxiii{Und sollten beide sich trennen, wird ALLAH jedem von ihnen reichlich von Seinem Reichtum gewähren. Und ALLAH bleibt immer allumfassend, allweise.\qt@no{130}}%
\qt@newcmd\qurantrans@de@dcxxiv{ALLAH gehört alles, was in den Himmeln und was auf Erden ist. Und gewiß, bereits haben WIR denjenigen, denen die Schrift vor euch zuteil wurde, sowie euch angewiesen: ``Handelt Taqwa gemäß ALLAH gegenüber! Und solltet ihr Kufr betreiben, so gehört ALLAH gewiß alles, was in den Himmeln und was auf Erden ist." Und ALLAH bleibt immer absolut autark, alllobenswürdig.\qt@no{131}}%
\qt@newcmd\qurantrans@de@dcxxv{Und ALLAH gehört alles, was in den Himmeln und was auf Erden ist. Und ALLAH genügt als Wakil.\qt@no{132}}%
\qt@newcmd\qurantrans@de@dcxxvi{Wenn ER will, läßt ER euch - ihr Menschen! - vergehen und bringt andere. Und ALLAH bleibt immer über dies allmächtig.\qt@no{133}}%
\qt@newcmd\qurantrans@de@dcxxvii{Wer (nur) die Belohnung des Diesseits erstrebt, so hat ALLAH die Belohnung des Diesseits und des Jenseits. Und ALLAH bleibt immer allhörend, allsehend.\qt@no{134}}%
\qt@newcmd\qurantrans@de@dcxxviii{Ihr, die den Iman verinnerlicht habt! Bemüht euch, ständig eurer Verantwortung auf gerechte Weise nachzukommen, und seid Zeugen für ALLAH, selbst dann, wenn es gegen euch selbst oder gegen die Eltern und die Verwandten sein sollte. Wenn er reich oder arm ist, so ist ALLAH verantwortlich für beide. Also folgt nicht der (eigenen) Zuneigung, um nicht (von der Wahrheit) abzuweichen. Und solltet ihr (das Zeugnis) verändern oder euch es verweigern, so bleibt ALLAH gewiß immer dem gegenüber, was ihr tut, allkundig.\qt@no{135}}%
\qt@newcmd\qurantrans@de@dcxxix{Ihr, die den Iman bekundet habt! Verinnerlicht den Iman an ALLAH, an Seinen Gesandten, an die Schrift, die ER Seinem Gesandten nach und nach hinabsandte und an die Schrift, die ER vorher hinabsandte. Und wer Kufr betreibt ALLAH, Seinen Engeln, Seinen Schriften, Seinen Gesandten und dem Jüngsten Tag gegenüber, der ist bereits weit in die Irre gegangen.\qt@no{136}}%
\qt@newcmd\qurantrans@de@dcxxx{Gewiß, diejenigen, die den Iman bekundet, dann Kufr betrieben haben, dann den Iman bekundet, dann Kufr betrieben haben, dann ihren Kufr vermehrt haben, denen wird ALLAH doch weder vergeben noch auf dem (rechten) Weg rechtleiten.\qt@no{137}}%
\qt@newcmd\qurantrans@de@dcxxxi{Überbringe den Munafiq die ``frohe Botschaft", daß für sie zweifelsohne qualvolle Peinigung bestimmt ist.\qt@no{138}}%
\qt@newcmd\qurantrans@de@dcxxxii{Es sind diejenigen, die sich die Kafir als Wali anstelle der Mumin nehmen. Erstreben sie bei ihnen etwa Überlegenheit?! So gehört gewiß die gesamte Überlegenheit (nur) ALLAH.\qt@no{139}}%
\qt@newcmd\qurantrans@de@dcxxxiii{Bereits sandte ER euch nach und nach in der Schrift hinab: ``Wenn ihr hört, daß ALLAHs Ayat gegenüber Kufr betrieben wird und sie verspottet werden, so sitzt nicht bei ihnen, bis sie zu einem anderen Thema übergehen." Gewiß wäret ihr sonst wie sie. Gewiß, ALLAH wird die Nifaq- und die Kufr-Betreibenden in Dschahannam versammeln, allesamt.\qt@no{140}}%
\qt@newcmd\qurantrans@de@dcxxxiv{Es sind diejenigen, die euch auflauern, wenn euch von ALLAH Sieg verliehen wurde, sagten sie: ``Waren wir etwa nicht mit euch?!" Und wenn den Kafir etwas zuteil wurde, sagten sie: ``Haben wir uns nicht für euch eingesetzt und euch vor den Mumin geschützt?!" ALLAH wird zwischen euch am Tag der Auferstehung richten. Und ALLAH wird den Kafir gegen die Mumin keine (wirkliche) Möglichkeit geben.\qt@no{141}}%
\qt@newcmd\qurantrans@de@dcxxxv{Gewiß, die Munafiq wollen ALLAH täuschen, doch ER täuscht sie. Und wenn sie sich zum rituellen Gebet erheben, erheben sie sich nur träge, damit wollen sie sich nur den Menschen zeigen, und sie gedenken ALLAHs nur ein wenig.\qt@no{142}}%
\qt@newcmd\qurantrans@de@dcxxxvi{Sie schwanken dazwischen, weder (gehören sie) zu diesen noch zu den anderen. Und wen ALLAH irre gehen läßt, für den wirst du keinen Weg (zum Iman) finden.\qt@no{143}}%
\qt@newcmd\qurantrans@de@dcxxxvii{Ihr, die den Iman verinnerlicht habt! Nehmt euch die Kafir nicht als Wali anstelle der Mumin, wollt ihr ALLAH gegen euch eine eindeutige Verfügung geben?!\qt@no{144}}%
\qt@newcmd\qurantrans@de@dcxxxviii{Gewiß, die Munafiq sind in der untersten Stufe des Feuers, und du wirst für sie keinen Beistehenden finden.\qt@no{145}}%
\qt@newcmd\qurantrans@de@dcxxxix{Ausgenommen sind diejenigen, die bereut, es korrigiert, sich an ALLAH fest gehalten und ihren Din für ALLAH aufrichtig praktiziert haben, diese sind mit den Mumin. ALLAH wird den Mumin eine übergroße Belohnung zuteil werden lassen.\qt@no{146}}%
\qt@newcmd\qurantrans@de@dcxl{Was hat ALLAH an eurer Peinigung, wenn ihr euch dankbar erweist und den Iman verinnerlicht habt?! Und ALLAH bleibt immer allbelohnend, allwissend.\qt@no{147}}%
\qt@newcmd\qurantrans@de@dcxli{ALLAH liebt nicht das öffentliche Vortragen von negativen Aussagen, es sei denn für denjenigen, dem Unrecht widerfahren ist. Und ALLAH bleibt immer allhörend, allwissend.\qt@no{148}}%
\qt@newcmd\qurantrans@de@dcxlii{Wenn ihr das Gute offenlegt oder verbergt, oder das Negative verzeiht, so bleibt ALLAH gewiß immer allvergebend, allmächtig.\qt@no{149}}%
\qt@newcmd\qurantrans@de@dcxliii{Gewiß, diejenigen, die ALLAH und Seinen Gesandten gegenüber Kufr betreiben, zwischen ALLAH und Seinen Gesandten trennen wollen und sagen: ``Wir verinnerlichen den Iman an einen Teil, und betreiben Kufr einem anderen Teil gegenüber", und streben dazwischen, einen Weg zu finden,\qt@no{150}}%
\qt@newcmd\qurantrans@de@dcxliv{diese sind die wirklichen Kafir. Und WIR haben für die Kafir eine erniedrigende Peinigung vorbereitet.\qt@no{151}}%
\qt@newcmd\qurantrans@de@dcxlv{Diejenigen, die den Iman an ALLAH und an Seine Gesandten verinnerlicht und keinen Unterschied zwischen keinen von ihnen gemacht haben, diesen wird ER ihre Belohnung zuteil werden lassen. Und ALLAH bleibt immer allvergebend, allgnädig.\qt@no{152}}%
\qt@newcmd\qurantrans@de@dcxlvi{Die Schriftbesitzer bitten dich darum, daß du ihnen eine Schrift vom Himmel nach und nach hinabsenden läßt. Bereits haben sie doch Musa um Größeres als dieses gebeten, sie sagten: ``Zeige uns ALLAH offenkundig!" Dann erschlug sie der Blitz wegen ihrer Übertretung, dann nahmen sie sich das Kalb (als Götzen), nachdem zu ihnen die deutlichen Zeichen gekommen waren, dann haben WIR ihnen dies vergeben und ließen Musa einen eindeutigen Beweis zuteil werden.\qt@no{153}}%
\qt@newcmd\qurantrans@de@dcxlvii{Und WIR hoben über sie den Tur-Berg wegen ihres Gelöbnisses empor und sagten ihnen: ``Tretet durch das Tor in Sudschud-Haltung ein!" Auch sagten WIR ihnen: ``Übertretet nicht am Sabbat!" Und WIR nahmen von ihnen ein unwiderrufliches Gelöbnis entgegen.\qt@no{154}}%
\qt@newcmd\qurantrans@de@dcxlviii{Sie wurden bestraft) infolge ihrer Verletzung ihres Gelöbnisses, ihres Kufrs ALLAHs Ayat gegenüber, ihres Tötens der Propheten zu Unrecht und ihrer Aussage: ``Unsere Herzen sind verhüllt." - Nein, sondern ALLAH hat sie versiegelt, wegen ihres Kufrs, so verinnerlichen sie den Iman nicht, außer wenigen (von ihnen). -\qt@no{155}}%
\qt@newcmd\qurantrans@de@dcxlix{Ebenso infolge ihres Kufrs, ihrer erfundenen ungeheuerlichen Äußerung über Maryam und\qt@no{156}}%
\qt@newcmd\qurantrans@de@dcl{ihrer Äußerung: ``Gewiß, wir haben Almasih, 'Isa Ibnu-Maryam, den Gesandten ALLAHs getötet." Doch weder haben sie ihn getötet, noch gekreuzigt, sondern es erschien ihnen nur etwas Ähnliches. Und gewiß, diejenigen, die über ihn uneins sind, sind doch im Zweifel über ihn. Sie haben kein Wissen davon, sie folgen nur einer Spekulation. Und sie haben ihn gewiß nicht getötet!\qt@no{157}}%
\qt@newcmd\qurantrans@de@dcli{Nein, sondern ALLAH hat ihn zu Seinem (zweiten Himmel ) emporgehoben. Und ALLAH bleibt immer allwürdig, allweise.\qt@no{158}}%
\qt@newcmd\qurantrans@de@dclii{Und es gibt keinen von den Schriftbesitzern, der nicht vor seinem Tod doch den Iman an ihn verinnerlichen wird. Und am Tag der Auferstehung wird er gegen sie Zeuge sein.\qt@no{159}}%
\qt@newcmd\qurantrans@de@dcliii{Und infolge des Unrecht-Begehens derjenigen, die Juden wurden, erklärten WIR ihnen für haram einige der Tay-yibat, die für sie als halal galten, (dies) ebenfalls infolge ihres häufigen Abbringens vom Wege ALLAHs,\qt@no{160}}%
\qt@newcmd\qurantrans@de@dcliv{ihres Nehmens von Riba - obwohl dies ihnen bereits verboten wurde - und ihres Aneignens der Vermögen von Menschen durch das für nichtig Erklärte. Und WIR haben für die Kafir unter ihnen eine qualvolle Peinigung vorbereitet.\qt@no{161}}%
\qt@newcmd\qurantrans@de@dclv{Doch diejenigen unter ihnen, die über fundiertes Wissen verfügen, und die Mumin, sie verinnerlichen den Iman an das, was dir hinabgesandt wurde, und an das, was vor dir hinabgesandt wurde - und (insbesondere unter ihnen lobe ICH) diejenigen, die das rituelle Gebet ordnungsgemäß verrichten - ebenso diejenigen, die Zakat entrichten, und diejenigen, die den Iman an ALLAH und an den Jüngsten Tag verinnerlichen, diesen allen werden WIR übergroße Belohnung zuteil werden lassen.\qt@no{162}}%
\qt@newcmd\qurantrans@de@dclvi{Gewiß, WIR ließen dir Wahy zuteil werden, wie WIR Wahy Nuh und den Propheten nach ihm zuteil werden ließen. Ebenso ließen WIRWahy zuteil werden Ibrahim, Isma'il, Ishaq, Ya'qub, den Stämmen Israils, 'Isa, Ayyub, Yunus, Harun und Sulaiman. Auch ließen WIR Dawud Zabur zukommen.\qt@no{163}}%
\qt@newcmd\qurantrans@de@dclvii{Ebenfalls (ließen WIR Wahy zuteil werden) Gesandten, von deren Begebenheiten WIR dir bereits vorher berichteten, auch Gesandten, von deren Begebenheiten WIR dir nicht berichtet haben. Und ALLAH sprach doch zu Musa!\qt@no{164}}%
\qt@newcmd\qurantrans@de@dclviii{UndWIR entsandten) Gesandte, als Überbringer froher Botschaft und als Ermahner, damit die Menschen ALLAH gegenüber nach (der Entsendung) der Gesandten keine Ausflüchte machen können. Und ALLAH bleibt immer allwürdig, allweise.\qt@no{165}}%
\qt@newcmd\qurantrans@de@dclix{Doch ALLAH bestätigt das, was ER dir hinabgesandt hat. ER sandte es mit Seinem Wissen hinab, auch die Engel bezeugen es. Und ALLAH genügt als Zeuge.\qt@no{166}}%
\qt@newcmd\qurantrans@de@dclx{Gewiß, diejenigen, die Kufr betrieben und vom Wege ALLAHs abgebracht haben, diese sind bereits weit in die Irre irregegangen.\qt@no{167}}%
\qt@newcmd\qurantrans@de@dclxi{Gewiß, denjenigen, die Kufr betrieben und Unrecht begangen haben, wird ALLAH weder vergeben und keinen Weg zur Rechtleitung aufzeigen\qt@no{168}}%
\qt@newcmd\qurantrans@de@dclxii{außer einen Weg zu Dschahannam. Darin bleiben sie für immer und ewig. Und dies bleibt für ALLAH immer etwas Leichtes.\qt@no{169}}%
\qt@newcmd\qurantrans@de@dclxiii{Ihr Menschen! Bereits ist zu euch doch ein Gesandter mit der Wahrheit von eurem HERRN gekommen, so verinnerlicht den Iman, dies ist besser für euch. Doch solltet ihr Kufr betreiben, so gehört ALLAH gewiß, was in den Himmeln und was auf Erden ist. Und ALLAH bleibt immer allwissend, allweise.\qt@no{170}}%
\qt@newcmd\qurantrans@de@dclxiv{Ihr Schriftbesitzer! Übertreibt nicht in eurem Din und sagt nichts über ALLAH außer der Wahrheit! Almasih 'Isa Ibnu-Maryam, war doch nur ein Gesandter ALLAHs, Sein Wort, das ER Maryam zuteil werden ließ, und ein Ruhh von Ihm. So verinnerlicht den Iman an ALLAH und an Seine Gesandten. Und sagt nicht: ``(Gott sei) dreifaltig." Hört auf damit, dies ist besser für euch. Gewiß, ALLAH ist nur ein einziger Gott! Gepriesen-erhaben ist ER, daß ER einen Sohn hätte. Ihm gehört, was in den Himmeln und was auf Erden ist. Und ALLAH genügt als Wakil.\qt@no{171}}%
\qt@newcmd\qurantrans@de@dclxv{Almasih wird es niemals verweigern, ein Diener ALLAHs zu sein, ebenso die auserwählten Engel. Und wer sich verweigert, Ihm zu dienen, und sich in Arroganz erhebt, diese wird ER noch zu Sich allesamt versammeln.\qt@no{172}}%
\qt@newcmd\qurantrans@de@dclxvi{Hinsichtlich derjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, diesen wird ER ihre Belohnung zuteil werden lassen und von Seiner Gunst noch mehr geben. Doch hinsichtlich derjenigen, die sich verweigert und sich in Arroganz erhoben haben, diese wird ER mit einer qualvollen Peinigung peinigen. Und sie werden anstelle von ALLAH weder einen Wali, noch einen Beistehenden finden.\qt@no{173}}%
\qt@newcmd\qurantrans@de@dclxvii{Ihr Menschen! Bereits ist zu euch doch eine Bestätigung von eurem HERRN gekommen. Und WIR haben euch ein klares Licht hinabgesandt.\qt@no{174}}%
\qt@newcmd\qurantrans@de@dclxviii{Hinsichtlich derjenigen, die den Iman an ALLAH verinnerlicht und sich an Ihm festgehalten haben, diese wird ER in eine Gnade von Ihm und Gunst eintreten lassen und zu Sich auf einem geradlinigen Weg rechtleiten.\qt@no{175}}%
\qt@newcmd\qurantrans@de@dclxix{Und sie fragen dich nach einer Fatwa, sag: ``ALLAH gibt euch Fatwa hinsichtlich der Kalala. Wenn ein Mensch stirbt, ohne daß er Kinder (oder Eltern), aber eine Schwester (väterlicherseits) hinterläßt, so bekommt sie die Hälfte dessen, was er hinterläßt, und er beerbt sie, wenn sie keine Kinder (oder Eltern) hinterläßt. Und wenn sie zwei Schwestern waren, dann bekommen sie zwei Drittel dessen, was er hinterläßt. Und wenn sie Geschwister sind, Männer und Frauen, so bekommt der Mann das Gleiche wie zwei Frauen." ALLAH erläutert euch, damit ihr nicht irrt. Und ALLAH ist über alles allwissend.\qt@no{176}}%
\qt@newcmd\qurantrans@de@dclxx{\basmalahde Ihr, die den Iman verinnerlicht habt! Haltet die Verträge ein! Für halal wurden euch erklärt Bahimatul-an'am außer dem, was euch noch vorgetragen wird, ohne daß ihr die Jagd für halal erklärt, während ihr im Ihram-Zustand seid. Gewiß, ALLAH verfügt, was ER will.\qt@no{1}}%
\qt@newcmd\qurantrans@de@dclxxi{Ihr, die den Iman verinnerlicht habt! Erklärt nicht für halal (die Verletzung) weder von ALLAHs Scha'ira, noch von dem Haram-Monat, noch von den Hadyi- Opfertieren, noch von den mit Halsbändern gekennzeichneten Opfertieren, noch von den Besuchern von Al-baitul-haram, die nach Gunst von ihrem HERRN und Wohlgefallen streben! Und wenn ihr den Ihram beendet, dann dürft ihr wieder auf die Jagd gehen. Und die Abneigung gegenüber Leuten, die euch (vom Besuch) von Almasdschidul-haram abgehalten haben, darf euch nicht dazu veranlassen, daß ihr eine Aggression begeht. Und kooperiert miteinander beim gottgefällig Gebotenen und bei Taqwa. Doch kooperiert nicht bei dem Verwerflichen und bei der Übertretung! Und handelt Taqwa gemäß ALLAH gegenüber. Gewiß, ALLAH ist hart im Strafen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@dclxxii{Für haram wurde euch erklärt das Verendete, das (vergoßene) Blut, Schweinefleisch, das, was für andere als ALLAH geschächtet wurde, das Erwürgte, das Erschlagene, das zu Tode Gestürzte, das tödlich (von anderen Tieren) Gestoßene, das von Raubtieren Angefressene, außer dem, was ihr (noch vor dem Verenden) durch Schächten gereinigt habt, das, was auf den Kultsteinen geschlachtet wurde, und daß ihr eure Entscheidungen durch Al-azlam trefft. Dies ist Fisq! An diesem Tag haben die Kafir (es) aufgegeben, (euch) von eurem Din (abzubringen), so fürchtet sie nicht, sondern (nur) Mich! An diesem Tag habe ICH euch (die Gebote) eures Din vervollständigt, euch Meine Gabe (Rechtleitung) vollendet und euch den Islam als Din zugestimmt. Wer jedoch durch Hungersnot, ohne dabei eine Verfehlung zu beabsichtigen, gezwungen wird, (dies zu übertreten), so ist ALLAH gewiß allvergebend, allgnädig.\qt@no{3}}%
\qt@newcmd\qurantrans@de@dclxxiii{Sie fragen dich, was für sie (an Speisen) halal sei. Sag: ``Für halal wurden euch erklärt die Tay-yibat und (das Halten) von dem, was ihr an trainierten Jagdtieren abgerichtet habt. Ihr lehrt sie von dem, was ALLAH euch gelehrt hat. So eßt von dem, was sie für euch fangen, und sprecht ALLAHs Namen über ihnen aus. Und handelt Taqwa gemäß ALLAH gegenüber! Gewiß, ALLAH ist schnell im Zur- Rechenschaft-Ziehen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@dclxxiv{An diesem Tag wurden euch die Tay-yibat für halal erklärt. Auch die Speisen derjenigen, denen die Schrift zuteil wurde, gilt für euch als halal, ebenfalls gelten eure Speisen für sie als halal. Und (zur Heirat gelten für euch als halal) die Tugendhaften von den Mumin-Frauen und die tugendhaften Frauen von denjenigen, denen die Schrift vor euch zuteil wurde, wenn ihr ihnen ihre Morgengabe gegeben habt als tugendhafte Ehemänner, nicht als Unzucht-Treibende und nicht als solche, die sich Geliebte nehmen. Und wer Kufr gegenüber dem Iman betreibt, dessen Handeln ist zweifellos nichtig, und er gehört im Jenseits zu den Verlierern.\qt@no{5}}%
\qt@newcmd\qurantrans@de@dclxxv{Ihr, die den Iman verinnerlicht habt! Wenn ihr zum rituellen Gebet aufstehen wollt, dann wascht (vorher) eure Gesichter, eure Hände und Arme bis zu den Ellenbogen, benetzt eure Köpfe und (wascht) eure Füße bis zu den Knöcheln. Und wenn ihr dschunub seid, dann stellt die rituelle Reinheit wieder her. Und wenn ihr krank oder auf Reisen seid, oder von der Notdurft kommt oder eure Frauen (intim) berührt und kein Wasser findet, dann sucht reine Erde auf und überstreicht (mit bestäubten Händen) eure Gesichter und eure Hände. ALLAH will euch nichts Unangenehmes gebieten, sondern euch nur reinigen und euch Seine Gabe vervollständigen, damit ihr euch dankbar erweist.\qt@no{6}}%
\qt@newcmd\qurantrans@de@dclxxvi{Und entsinnt euch ALLAHs Gabe euch gegenüber und Dessen Gelöbnis, das ER von euch entgegennahm, als ihr sagtet: ``Wir haben zugehört und gehorcht." Und handelt Taqwa gemäß ALLAH gegenüber! Gewiß, ALLAH ist allwissend über das, was in den Brüsten ist.\qt@no{7}}%
\qt@newcmd\qurantrans@de@dclxxvii{Ihr, die den Iman verinnerlicht habt! Bemüht euch eurer Verpflichtung ALLAH gegenüber nachzukommen und seid Zeugen in gerechter Weise! Und die Abneigung einer Gemeinschaft gegenüber darf euch nicht dazu veranlassen, ungerecht zu sein. Seid gerecht, dies ist näher zur Taqwa, und handelt Taqwa gemäß ALLAH gegenüber! Gewiß, ALLAH ist dessen allkundig, was ihr tut.\qt@no{8}}%
\qt@newcmd\qurantrans@de@dclxxviii{ALLAH hat den Mumin und denjenigen, die gottgefällig Gutes tun, versprochen, daß für sie Vergebung und übergroße Belohnung bestimmt ist.\qt@no{9}}%
\qt@newcmd\qurantrans@de@dclxxix{Und diejenigen, die Kufr betreiben und Unsere Ayat verleugnen, diese sind die Weggenossen der Hölle.\qt@no{10}}%
\qt@newcmd\qurantrans@de@dclxxx{Ihr, die den Iman verinnerlicht habt! Entsinnt euch der Gabe ALLAHs euch gegenüber, als ein Volk euch Schaden zufügen wollte und ER sie von euch abgehalten hat. Und handelt Taqwa gemäß ALLAH gegenüber. Und ALLAH gegenüber sollen die Mumin Tawakkul üben!\qt@no{11}}%
\qt@newcmd\qurantrans@de@dclxxxi{Und gewiß, bereits nahm ALLAH das Gelöbnis der Kinder Israils entgegen. Und WIR setzten von ihnen zwölf Verantwortliche ein, und ALLAH sagte: ``Gewiß, ICH unterstütze euch." Wenn ihr das rituelle Gebet ordnungsgemäß verrichtet, die Zakat entrichtet, den Iman an Meine Gesandten verinnerlicht und sie unterstützt und um ALLAHs Willen eine Hasan-Anleihe macht, dann werde ICH euch mit Gewißheit eure gottmißfälligen Taten erlassen und euch in Dschannat eintreten lassen, die von Flüssen durchflossen sind. Wer danach von euch Kufr betreibt, der ist bereits vom rechten Weg abgeirrt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@dclxxxii{Infolge ihrer Verletzung ihres Gelöbnisses haben WIR sie verflucht und ihre Herzen hart gemacht. Sie verdrehen die Worte (der Schrift) und haben einen Teil von dem vergessen, woran sie erinnert wurden. Und du wirst immer wieder einen Verrat von ihnen aufdecken, außer von wenigen von ihnen, so verzeihe ihnen und sei nachsichtig! Gewiß, ALLAH liebt die Muhsin.\qt@no{13}}%
\qt@newcmd\qurantrans@de@dclxxxiii{Auch von denjenigen, die sagten: ``Wir sind Nazarener.", nahmen WIR ihr Gelöbnis entgegen, dann haben sie einen Teil dessen vergessen, woran sie erinnert wurden, so ließen WIR zwischen ihnen Feindschaft und Haß bis zum Tag der Auferstehung entstehen. Und ALLAH wird sie über das unterrichten, was sie zu bewerkstelligen pflegten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@dclxxxiv{Ihr Schriftbesitzer! Bereits kam zu euch Unser Gesandter, um euch vieles von dem zu enthüllen, was ihr von der Schrift verborgen habt, und vieles unenthüllt zu lassen. Bereits kam doch zu euch von ALLAH Licht und deutliche Schrift!\qt@no{15}}%
\qt@newcmd\qurantrans@de@dclxxxv{Damit leitet ALLAH jeden, der nach Seinem Wohlgefallen strebt, auf den Salam- Wegen recht und bringt sie von den Finsternissen ins Licht mit Seiner Zustimmung heraus und leitet sie auf den geradlinigen Weg recht.\qt@no{16}}%
\qt@newcmd\qurantrans@de@dclxxxvi{Gewiß, bereits betreiben Kufr diejenigen, die sagten: ``Gewiß, ALLAH ist selbst Almasih Ibnu-Maryam." Sag: ``Wer kann in irgendeiner Weise ALLAH daran hindern, wenn ER Almasih Ibnu-Maryam, seine Mutter und alle, die auf Erden sind, sterben lassen will?!" Und ALLAH gehören die Himmel und Erde und alles, was zwischen ihnen ist. ER erschafft, was ER will. Und ALLAH ist über alles allmächtig.\qt@no{17}}%
\qt@newcmd\qurantrans@de@dclxxxvii{Die Juden und die Nazarener sagten: ``Wir sind ALLAHs Kinder und Seine Lieblinge." Sag: ``Weshalb peinigt ER euch denn wegen eurer Verfehlungen?!" Nein, sondern ihr seid nur Menschen von denjenigen, die ER erschaffen hat. ER vergibt, wem ER will, und peinigt, wen ER will. Und ALLAH gehören die Himmel und Erde und alles, was zwischen ihnen ist. Und zu Ihm ist das Werden.\qt@no{18}}%
\qt@newcmd\qurantrans@de@dclxxxviii{Ihr Schriftbesitzer! Bereits kam zu euch Unser Gesandter, um euch nach einer (langen) Zeit, in der Gesandte ausgeblieben sind, (die Schrift) zu erklären, damit ihr nicht sagt: ``Zu uns kam weder ein Überbringer froher Botschaft, noch ein Ermahner." So kam zu euch bereits sowohl ein Überbringer froher Botschaft als auch ein Ermahner. Und ALLAH ist über alles allmächtig.\qt@no{19}}%
\qt@newcmd\qurantrans@de@dclxxxix{Und (erinnere daran), als Musa zu seinen Leuten sagte: ``Meine Leute! Entsinnt euch ALLAHs Gabe euch gegenüber, als ER unter euch Propheten entsandt, euch zu Königen gemacht und euch das hat zuteil werden lassen, was ER keinem von den Menschen gab.\qt@no{20}}%
\qt@newcmd\qurantrans@de@dcxc{Meine Leute! Tretet ein in das gereinigte Land, das ALLAH für euch bestimmt hat, und kehrt nicht zum Vorherigen zurück, damit ihr nicht als Verlierer umkehrt."\qt@no{21}}%
\qt@newcmd\qurantrans@de@dcxci{Sie sagten: ``Musa! Gewiß, darin sind sehr mächtige Leute und wir werden es zweifelsohne nicht betreten, bis sie es verlassen haben, und wenn sie es verlassen haben, erst dann werden wir es betreten."\qt@no{22}}%
\qt@newcmd\qurantrans@de@dcxcii{Zwei Männer von denjenigen, die Ehrfurcht haben und denen ALLAH von Seinen Gaben gewährte, sagten: ``Tretet zu ihnen durch das Tor ein! Und wenn ihr es durchschritten habt, werdet ihr dann doch Sieger sein. Und ALLAH gegenüber sollt ihr Tawakkul üben, wenn ihr Mumin seid."\qt@no{23}}%
\qt@newcmd\qurantrans@de@dcxciii{Sie sagten: ``Musa! Gewiß, wir werden es niemals betreten, solange sie darin sind. Also geh du mit deinem HERRN hin und kämpft. Gewiß, wir werden uns nicht von der Stelle rühren."\qt@no{24}}%
\qt@newcmd\qurantrans@de@dcxciv{Er sagte: ``HERR! Ich verfüge nur über mich und meinen Bruder, so trenne zwischen uns und den fisq-betreibenden Leuten!"\qt@no{25}}%
\qt@newcmd\qurantrans@de@dcxcv{ER sagte: ``Es gilt für sie als haram vierzig Jahre lang, sie werden im Lande umherirren. Also gräme dich nicht wegen der fisq-betreibenden Leute!"\qt@no{26}}%
\qt@newcmd\qurantrans@de@dcxcvi{Und trage ihnen vor die Begebenheit von beiden Söhnen Adams - mit der Wahrheit. Als beide (jeweils) eine Darbringung darbrachten, dann wurde sie von einem von ihnen angenommen, während diese vom anderen nicht angenommen wurde. Er (der andere) sagte: ``Ich werde dich zweifelsohne töten!" Er (der Erste) sagte: ``ALLAH nimmt doch nur von den Muttaqi an.\qt@no{27}}%
\qt@newcmd\qurantrans@de@dcxcvii{Solltest du deine Hand erheben, um mich zu töten, so werde ich meine Hand nicht erheben, um dich zu töten. Gewiß, ich habe Ehrfurcht vor ALLAH, Dem HERRN aller Schöpfung!\qt@no{28}}%
\qt@newcmd\qurantrans@de@dcxcviii{Ich will doch, daß du (allein) mit meiner und mit deiner Verfehlung zurückkehrst, damit du von den Weggenossen des Feuers wirst." Und dies ist die Vergeltung für die Unrecht-Begehenden.\qt@no{29}}%
\qt@newcmd\qurantrans@de@dcxcix{Und sein Ego erleichterte ihm das Töten seines Bruders, so hat er ihn getötet, so gehörte er zu den Verlierern.\qt@no{30}}%
\qt@newcmd\qurantrans@de@dcc{Dann schickte ALLAH einen Raben, der in der Erde scharrte, um ihm zu zeigen, wie er die Leiche seines Bruder begräbt. ER sagte: ``Mein Untergang! Konnte ich nicht wie dieser Rabe sein und die Leiche meines Bruders begraben?!" so wurde er von den Bereuenden.\qt@no{31}}%
\qt@newcmd\qurantrans@de@dcci{Deshalb haben WIR den Kindern Israils geboten, daß, wer einen Menschen tötet - nicht als Vergeltung für einen getöteten Menschen und nicht wegen Anrichten von Verderben auf Erden -, es so sei, als hätte er alle Menschen getötet, und wer ihm das Leben erhält, es so sei, als ob er der ganzen Menschheit das Leben erhalten hätte. Und gewiß, bereits kamen zu ihnen doch Unsere Gesandten mit den deutlichen Zeichen, dann sind viele von ihnen danach auf Erden gewiß des Maßes Überschreitende!\qt@no{32}}%
\qt@newcmd\qurantrans@de@dccii{Die Bestrafung für diejenigen, die ALLAHs (Din) und Seinen Gesandten durch Muharaba bekämpfen und Verderben auf Erden anrichten, ist, daß sie getötet oder gekreuzigt oder daß ihre Hände und Füße wechselseitig abgetrennt oder des Landes verwiesen werden. Dies ist für sie Schmach im Diesseits. Und im Jenseits ist für sie übergroße Peinigung bestimmt.\qt@no{33}}%
\qt@newcmd\qurantrans@de@dcciii{Ausgenommen sind diejenigen, die bereut haben, bevor ihr ihrer habhaft geworden seid. Also wißt, daß ALLAH gewiß allvergebend, allgnädig ist.\qt@no{34}}%
\qt@newcmd\qurantrans@de@dcciv{Ihr, die den Iman verinnerlicht habt! Handelt Taqwa gemäß ALLAH gegenüber, strebt nach dem, womit ihr Sein Wohlgefallen erreicht und leistet Dschihad fi-sabilillah, damit ihr erfolgreich werdet.\qt@no{35}}%
\qt@newcmd\qurantrans@de@dccv{Gewiß, diejenigen, die Kufr betrieben haben, selbst wenn sie alles, was es auf Erden gibt, besäßen und noch dazu seinesgleichen, um sich damit vor der Peinigung am Tag der Auferstehung freizukaufen, dies wird von ihnen nicht angenommen. Und für sie ist eine qualvolle Peinigung bestimmt.\qt@no{36}}%
\qt@newcmd\qurantrans@de@dccvi{Sie werden wollen, aus dem Feuer herauszukommen, aber sie werden daraus nicht herauskommen können. Und für sie ist eine bleibende Peinigung bestimmt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@dccvii{Dem Dieb und der Diebin trennt die Hand ab, als Vergeltung für das, was sie sich erworben haben - eine Strafe von ALLAH. Und ALLAH ist allwürdig, allweise.\qt@no{38}}%
\qt@newcmd\qurantrans@de@dccviii{Und wer nach seiner Übertretung bereut und gottgefällig Gutes tut, von dem wird ALLAH zweifellos die Reue annehmen. Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{39}}%
\qt@newcmd\qurantrans@de@dccix{Weißt du etwa nicht, daß ALLAH doch die Herrschaft der Himmel und der Erde gehört?! ER peinigt, wen ER will, und vergibt, wem ER will. Und ALLAH ist über alles allmächtig.\qt@no{40}}%
\qt@newcmd\qurantrans@de@dccx{Gesandter! Sei nicht traurig wegen denjenigen, die Kufr schnell bekunden, von denjenigen, die mit ihren Mündern sagten: ``Wir haben den Iman verinnerlicht.", während ihre Herzen den Iman nicht verinnerlicht haben, sowie von denjenigen, die Juden sind, sie schenken dem Lügen Gehör und sie schenken Gehör anderen Leute, die nicht zu dir kommen. Sie verdrehen die Worte (der Schrift). Sie sagen: ``Wenn ihr nach dem (veränderten Text) verurteilt werdet, dann akzeptiert es. Doch wenn ihr nicht danach verurteilt werdet, dann haltet euch nicht daran." Und wen ALLAH der Fitna aussetzt, dem kannst du vor ALLAH in Nichts helfen. Diese sind diejenigen, deren Herzen ALLAH nicht reinigen wollte. Für sie ist im Diesseits Schmach vorgesehen. Und im Jenseits ist für sie übergroße Peinigung bestimmt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@dccxi{Sie schenken dem Lügen Gehör und eignen sich Verbotenes an. Sollten sie zu dir kommen, so richte zwischen ihnen oder wende dich von ihnen weg! Und wenn du dich von ihnen wegwendest, werden sie dir in Nichts schaden können. Und wenn du zwischen ihnen richtest, dann nur mit Gerechtigkeit. Gewiß, ALLAH liebt die Gerechten.\qt@no{42}}%
\qt@newcmd\qurantrans@de@dccxii{Wie können sie nur dich (zwischen ihnen) richten lassen, während sie At-taurat besitzen, in der ALLAHs Gebot ist, und danach sich davon abwenden?! Und diese sind bestimmt keine Mumin.\qt@no{43}}%
\qt@newcmd\qurantrans@de@dccxiii{Gewiß, WIR sandten At-taurat hinab, darin ist Rechtleitung und Erleuchtung enthalten. Danach haben die Propheten, die den Islam praktizierten, zwischen den Juden gerichtet, ebenso die Gelehrten und die Rabbiner, aufgrund dessen, was ihnen von ALLAHs Schrift anvertraut wurde und (weil) sie deren Zeugen waren. So habt keine Ehrfurcht vor den Menschen und habt Ehrfurcht vor Mir alleine, und erkauft euch nicht mit Meinen Ayat Minderwertiges! Und wer nicht danach richtet, was ALLAH hinabgesandt hat, diese sind die wirklichen Kafir.\qt@no{44}}%
\qt@newcmd\qurantrans@de@dccxiv{Und WIR haben ihnen darin geboten: ``Leben um Leben, Auge um Auge, Nase um Nase, Ohr um Ohr, Zahn um Zahn und (sonstige) Verletzungen durch Qisas (zu vergelten). Und wer darauf verzichtet, wird dies eine Kaffara für ihn sein. Und wer nicht danach richtet, was ALLAH hinabgesandt hat, diese sind die wirklichen Unrecht-Begehenden.\qt@no{45}}%
\qt@newcmd\qurantrans@de@dccxv{Und WIR ließen auf ihren Spuren 'Isa Ibnu-Maryam folgen (als) Bestätiger dessen, was ihm in At-taurat vorangegangen war. UndWIR ließen ihm Alindschil zuteil werden, darin ist Rechtleitung, Licht und Bestätigung dessen, was ihm in At-taurat voranging, und (als) Rechtleitung und Ermahnung für die Muttaqi.\qt@no{46}}%
\qt@newcmd\qurantrans@de@dccxvi{Und die Leute von Alindschil sollen danach richten, was ALLAH darin hinabgesandt hat. Und wer nicht danach richtet, was ALLAH hinabgesandt hat, diese sind die wirklichen Fasiq.\qt@no{47}}%
\qt@newcmd\qurantrans@de@dccxvii{Und WIR haben dir die Schrift mit der Wahrheit hinabgesandt als Bestätigung dessen, was ihr von der Schrift voranging, und als Hüter über sie. So richte zwischen ihnen nach dem, was ALLAH hinabgesandt hat, und folge nicht ihren Neigungen anstelle dessen, was zu dir von der Wahrheit kam. Für jeden von euch haben WIR eine Schari'a und eine Lebensweise bestimmt. Und wenn ALLAH gewollt hätte, hätte ER euch doch zu einer einzigen Umma gemacht, jedoch ER prüft euch in alledem, was ER euch zuteil werden ließ. So wetteifert um die gottgefällig guten Taten! Zu ALLAH wird eure Rückkehr allesamt sein, dann wird ER euch darüber in Kenntnis setzen, worüber ihr uneins zu sein pflegtet.\qt@no{48}}%
\qt@newcmd\qurantrans@de@dccxviii{Und richte zwischen ihnen nur nach dem, was ALLAH hinabgesandt hat, und folge nicht ihren Neigungen und nimm dich vor ihnen in Acht, damit sie dich nicht durch Fitna von etwas abbringen, was ALLAH dir hinabgesandt hat. Und sollten sie sich abwenden, dann wisse, daß ALLAH ihnen nur wegen einiger Ihrer Verfehlungen vergelten will. Und viele von den Menschen sind gewiß Fasiq.\qt@no{49}}%
\qt@newcmd\qurantrans@de@dccxix{Erstreben sie etwa das Richten nach der Dschahiliya?! Und wer ist besser im Richten als ALLAH für Menschen, die über Gewißheit verfügen?!\qt@no{50}}%
\qt@newcmd\qurantrans@de@dccxx{Ihr, die den Iman verinnerlicht habt! Nehmt euch die Juden und die Nazarener nicht als Wali. Die einen von ihnen sind Wali der anderen. Und wer von euch sie als Wali nimmt, der gehört doch zu ihnen. Gewiß, ALLAH leitet die unrechtbegehende Gemeinschaft nicht recht.\qt@no{51}}%
\qt@newcmd\qurantrans@de@dccxxi{Und du siehst, daß diejenigen, in deren Herzen Krankheit ist, zu ihrer (Unterstützung) eilen. Sie sagen: ``Wir fürchten, daß wir unterlegen sein werden." ALLAH wird jedoch entweder den Sieg oder von Sich aus eine Entscheidung herbeiführen, so werden sie bereuen aufgrund dessen, was sie bei sich verheimlicht haben.\qt@no{52}}%
\qt@newcmd\qurantrans@de@dccxxii{Und diejenigen, die den Iman verinnerlicht haben, sagen: ``Sind diese etwa diejenigen, die ihre Eide bei ALLAH mit Nachdruck beschworen haben, daß sie doch mit euch seien?!" Ihre Handlungen sind nichtig geworden, so wurden sie dann zu Verlierern.\qt@no{53}}%
\qt@newcmd\qurantrans@de@dccxxiii{Ihr, die den Iman verinnerlicht habt! Wer von euch sich von seinem Din abwendet, so wird ALLAH Menschen bringen, die ER liebt und welche Ihn lieben. Sie sind demütig den Mumin gegenüber, aber stolz den Kafir gegenüber, sie leisten Dschihad fi-sabilillah und fürchten keine Mißbilligung irgendeines Mißbilligenden. Dies ist ALLAHs Gunst, die ER zuteil werden läßt, wem ER will. Und ALLAH ist allumfassend, allwissend.\qt@no{54}}%
\qt@newcmd\qurantrans@de@dccxxiv{Euer Wali ist nur ALLAH, Sein Gesandter und diejenigen Mumin, die das rituelle Gebet ordnungsgemäß verrichten und die Zakat entrichten, während sie Ruku'-Vollziehende sind.\qt@no{55}}%
\qt@newcmd\qurantrans@de@dccxxv{Und wer ALLAH, Seinen Gesandten und die Mumin als Wali nimmt, so sind die Vereinten um ALLAH gewiß die Sieger.\qt@no{56}}%
\qt@newcmd\qurantrans@de@dccxxvi{Ihr, die den Iman verinnerlicht habt! Nehmt diejenigen, die euren Din zum Spott und zur Belustigung mißbrauchen von denjenigen, denen die Schrift vor euch zuteil wurde, und von den Kafir, nicht als Wali! Und handelt Taqwa gemäß ALLAH gegenüber, solltet ihr Mumin sein.\qt@no{57}}%
\qt@newcmd\qurantrans@de@dccxxvii{Und wenn ihr zum rituellen Gebet ruft, treiben sie darüber Spott und Belustigung. Dies ist so, weil sie Menschen sind, die sich nicht besinnen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@dccxxviii{Sag: ``Ihr Schriftbesitzer! Nehmt ihr uns etwas anderes übel als daß wir den Iman an ALLAH verinnerlicht haben und an das, was uns hinabgesandt wurde, und an das, was vorher hinabgesandt wurde, und daß die meisten von euch Fasiq sind?!"\qt@no{59}}%
\qt@newcmd\qurantrans@de@dccxxix{Sag: ``Soll ich euch in Kenntnis setzen über schlimmere Vergeltung als für dies, (was ihr uns verübelt), bei ALLAH?: Es ist (die Vergeltung für) denjenigen, wen ALLAH verflucht, über den ER zürnt und aus denen ER Affen, Schweine und Taghut-Anbeter machte." Diese haben die niedrigere Stellung und sind vom rechten Weg noch mehr verirrt.\qt@no{60}}%
\qt@newcmd\qurantrans@de@dccxxx{Und wenn sie zu euch kommen, sagen sie: ``Wir haben den Iman bekundet.", während sie selbst doch mit dem Kufr eingetreten und mit diesem fortgegangen sind. Und ALLAH kennt besser das, was sie zu verbergen pflegten.\qt@no{61}}%
\qt@newcmd\qurantrans@de@dccxxxi{Und du siehst viele von ihnen, wie sie sich beeilen, Verfehlungen und Übertretungen zu begehen und sich Verbotenes anzueignen. Und erbärmlich ist dies, was sie zu tun pflegten.\qt@no{62}}%
\qt@newcmd\qurantrans@de@dccxxxii{Würden die Gelehrten und Rabbiner sie doch nur abhalten, Verwerfliches zu bekunden und sich Verbotenes anzueignen. Und gewiß erbärmlich ist dies, was sie zu bewerkstelligen pflegten.\qt@no{63}}%
\qt@newcmd\qurantrans@de@dccxxxiii{Die Juden sagten, ALLAH sei geizig! Ihre Hände mögen gefesselt werden und sie selbst verflucht sein für das, was sie gesagt haben. Nein, sondern ER ist übergroßzügig. ER gibt, wie ER will. Und viele von ihnen wird das, was dir von deinem HERRN hinabgesandt wurde, gewiß zu noch mehr Übertretung und Kufr veranlassen. Und WIR haben zwischen ihnen die (gegenseitige) Feindschaft und Abneigung bis zum Tag der Auferstehung gesetzt. Jedesmal, als sie ein Feuer zum Krieg angezündet haben, hat ALLAH es ausgelöscht. Und sie bemühen sich auf der Erde um Verderben. Doch ALLAH liebt nicht die Verderben-Anrichtenden.\qt@no{64}}%
\qt@newcmd\qurantrans@de@dccxxxiv{Und hätten die Schriftbesitzer doch den Iman verinnerlicht und die Taqwa gemäß gehandelt, gewiß hätten WIR ihnen ihre gottmißfälligen Taten vergeben und gewiß hätten WIR sie in Dschannat des Wohlergehens eintreten lassen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@dccxxxv{Und hätten sie doch eingehalten At-taurat, Alindschil und das, was ihnen von ihrem HERRN hinabgesandt wurde, gewiß hätten sie reichlich von über ihnen und von unter ihren Füßen zu speisen bekommen. Unter ihnen ist eine gemäßigte Umma - doch viele von ihnen (den anderen) - es ist schlimm, was sie tun.\qt@no{66}}%
\qt@newcmd\qurantrans@de@dccxxxvi{Gesandter! Verkünde, was dir von deinem HERRN hinabgesandt wurde! Und solltest du dies nicht tun, dann hast du Seine Botschaft nicht verkündet. Und ALLAH schützt dich vor den Menschen. Gewiß, ALLAH leitet die kufrbetreibende Gemeinschaft nicht recht.\qt@no{67}}%
\qt@newcmd\qurantrans@de@dccxxxvii{Sag: ``Ihr Schriftbesitzer! Ihr habt nichts (vom Din), bis ihr At-taurat und Alindschil und das einhaltet, was euch von eurem HERRN hinabgesandt wurde. Und viele von ihnen wird das, was dir von deinem HERRN hinabgesandt wurde, gewiß zu noch mehr Übertretung und Kufr veranlassen. So sei nicht traurig wegen der kufr-betreibenden Gemeinschaft!\qt@no{68}}%
\qt@newcmd\qurantrans@de@dccxxxviii{Gewiß, diejenigen, die den Iman bekundet haben, diejenigen, die Juden wurden, die Sabia und die Nazarener, wer von ihnen den Iman an ALLAH und den Jüngsten Tag verinnerlicht und gottgefällig Gutes getan hat  um sie gibt es weder Angst noch werden sie traurig sein.\qt@no{69}}%
\qt@newcmd\qurantrans@de@dccxxxix{Gewiß, bereits haben WIR das Gelöbnis der Kinder Israils entgegengenommen und ihnen Gesandte entsandt. Jedesmal, wenn zu ihnen ein Gesandter nicht mit dem kam, wozu ihre Seelen geneigt haben, haben sie einen Teil (der Gesandten) der Lüge bezichtigt und einen Teil getötet.\qt@no{70}}%
\qt@newcmd\qurantrans@de@dccxl{Und sie dachten, daß es keine Fitna sein würde. So waren sie (der Wahrheit gegenüber) blind und taub, dann hat ALLAH ihnen vergeben, dann wurden sie wieder blind und taub - viele von ihnen. Und ALLAH ist dessen allsehend, was sie tun.\qt@no{71}}%
\qt@newcmd\qurantrans@de@dccxli{Gewiß, bereits betreiben Kufr diejenigen, die sagten: ``ALLAH ist doch selbst Almasih Ibnu-Maryam." Doch Almasih sagte: ``Kinder Israils! Dient ALLAH, meinem HERRN und eurem HERRN. Gewiß, wer ALLAH gegenüber Schirk betreibt, dem hat ALLAH bereits die Dschanna für haram erklärt und sein Aufenthaltsort ist das Feuer." Und für die Unrecht-Begehenden gibt es keine Helfer.\qt@no{72}}%
\qt@newcmd\qurantrans@de@dccxlii{Gewiß, bereits betreiben Kufr diejenigen, die sagten: ``ALLAH ist einer von Dreien." Doch es gibt keine Gottheit außer einer einzigen Gottheit! Und sollten sie das nicht unterlassen, was sie sagen, wird gewiß diejenigen von ihnen, die Kufr betrieben, qualvolle Peinigung überkommen.\qt@no{73}}%
\qt@newcmd\qurantrans@de@dccxliii{Wollen sie etwa keine Reue ALLAH gegenüber zeigen und Ihn um Vergebung bitten?! Und ALLAH ist allvergebend, allgnädig.\qt@no{74}}%
\qt@newcmd\qurantrans@de@dccxliv{Almasih Ibnu-Maryam war gewiß nur ein Gesandter, vor dem andere Gesandte bereits dahingingen, und seine Mutter war eine Wahrheitsliebende. Beide haben die Speisen gegessen. Siehe, wie WIR ihnen die Ayat verdeutlichen, dann siehe, wie sie (von der Rechtleitung) abgebracht werden!\qt@no{75}}%
\qt@newcmd\qurantrans@de@dccxlv{Sag: ``Wollt ihr etwas anstelle ALLAH dienen, was euch weder Schaden noch Nutzen bringt?!" Und ALLAH ist Der Allhörende, Der Allwissende.\qt@no{76}}%
\qt@newcmd\qurantrans@de@dccxlvi{Sag: ``Ihr Schriftbesitzer! Übertreibt nicht in eurem Din (und sagt nichts) außer der Wahrheit und folgt nicht den Neigungen von Leuten, die bereits vorher irregegangen sind, viele in die Irre verleitet haben und vom rechten Weg abgekommen sind."\qt@no{77}}%
\qt@newcmd\qurantrans@de@dccxlvii{Diejenigen von den Kindern Israils, die Kufr betrieben haben, wurden aus dem Munde von Dawud und von 'Isa Ibnu-Maryam verflucht. Dies war wegen dem, was sie sich widersetzt haben, und dem, was sie zu übertreten pflegten.\qt@no{78}}%
\qt@newcmd\qurantrans@de@dccxlviii{Sie pflegten nicht sich gegenseitig vom Mißbilligten, was sie getan haben, abzuhalten. Gewiß, erbärmlich war dies, was sie zu tun pflegten.\qt@no{79}}%
\qt@newcmd\qurantrans@de@dccxlix{Du siehst viele von ihnen diejenigen als Wali nehmen, die Kufr betrieben haben. Gewiß, erbärmlich ist dies, was sie für sich selbst vorausschicken, daß ALLAH über sie erzürnt ist und daß sie in der Peinigung ewig bleiben werden.\qt@no{80}}%
\qt@newcmd\qurantrans@de@dccl{Und hätten sie den Iman verinnerlicht an ALLAH, an den Propheten und an das, was ihm hinabgesandt wurde, würden sie diese (die Kafir) nicht als Wali nehmen. Doch viele von ihnen sind Fasiq.\qt@no{81}}%
\qt@newcmd\qurantrans@de@dccli{Du wirst gewiß finden, daß unter den Menschen die erbittertsten Feinde für diejenigen, die den Iman verinnerlicht haben, die Juden und diejenigen sind, die Schirk betreiben. Und du wirst gewiß finden, daß die am engsten mit denjenigen, die den Iman verinnerlicht haben verbunden sind, diejenigen sind, welche sagen: ``Wir sind Nazarener." Dies ist so, weil doch unter ihnen Priester und Mönche sind und weil sie doch sich nicht in Arroganz erheben.\qt@no{82}}%
\qt@newcmd\qurantrans@de@dcclii{Auch wenn sie hören, was dem Gesandten hinabgesandt wurde, siehst du ihre Augen Tränen über das vergießen, was sie von der Wahrheit erkannt haben. Sie sagen: ``Unser HERR! Wir haben den Iman verinnerlicht, so trage uns unter den Bezeugenden ein!\qt@no{83}}%
\qt@newcmd\qurantrans@de@dccliii{Und weshalb sollen wir nicht den Iman an ALLAH verinnerlichen und an das, was zu uns von der Wahrheit kam, und nicht hoffen, daß unser HERR uns mit den gottgefällig guttuenden Leuten (in die Dschanna) eintreten läßt?!"\qt@no{84}}%
\qt@newcmd\qurantrans@de@dccliv{So belohnte ALLAH sie für das, was sie sagten, mit Dschannat, die von Flüssen durchflossen sind, in denen sie ewig bleiben. Und dies ist die Belohnung für die Muhsin.\qt@no{85}}%
\qt@newcmd\qurantrans@de@dcclv{Und diejenigen, die Kufr betrieben und Unsere Ayat verleugnet haben. Diese sind die Weggenossen der Hölle.\qt@no{86}}%
\qt@newcmd\qurantrans@de@dcclvi{Ihr, die den Iman verinnerlicht habt! Erklärt nicht für haram die Tay-yibat dessen, was ALLAH euch für halal erklärte, und übertretet nicht! Gewiß, ALLAH liebt nicht die Übertretenden.\qt@no{87}}%
\qt@newcmd\qurantrans@de@dcclvii{Und speist von dem Rizq, das ALLAH euch als gutes Halal gewährt hat und handelt Taqwa gemäß ALLAH gegenüber, an Den ihr den Iman verinnerlicht habt!\qt@no{88}}%
\qt@newcmd\qurantrans@de@dcclviii{ALLAH belangt euch nicht für Unbedachtes in euren Eiden. Doch ER belangt euch für Eide, die ihr mit Bedacht geschworen habt. Die Kaffara dafür ist die Speisung von zehn Bedürftigen in jenem Maß, wie ihr die Eurigen im Durchschnitt speist, oder Ihre Bekleidung oder die Befreiung eines Unfreien. Wer dies aber nicht kann, fastet drei Tage. Dies ist die Kaffara für eure Eide, wenn ihr sie geschworen habt. Und achtet auf eure Eide! Solcherart verdeutlicht ALLAH euch Seine Ayat, damit ihr euch dankbar erweist.\qt@no{89}}%
\qt@newcmd\qurantrans@de@dcclix{Ihr, die den Iman verinnerlicht habt! Khamr, Glücksspiel, Opfersteine und Alazlam sind doch nur Unreinheiten aus dem Werke des Satans, so meidet sie, damit ihr erfolgreich werdet.\qt@no{90}}%
\qt@newcmd\qurantrans@de@dcclx{Der Satan will doch zwischen euch Feindschaft und Haß durch Khamr und Glücksspiel säen und euch vom Gedenken an ALLAH und vom rituellen Gebet abhalten. Werdet ihr denn damit aufhören!\qt@no{91}}%
\qt@newcmd\qurantrans@de@dcclxi{Und gehorcht ALLAH, gehorcht dem Gesandten und nehmt euch in Acht! Doch solltet ihr euch abwenden, so wisst, daß Unserem Gesandten nur das deutliche Verkünden obliegt.\qt@no{92}}%
\qt@newcmd\qurantrans@de@dcclxii{Es trifft diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, keine Verfehlung für das, was sie (vor dem Verbot) zu sich genommen haben, wenn sie Taqwa gemäß handelten, den Iman verinnerlichten und gottgefällig Gutes taten, dann Taqwa gemäß handelten und Iman verinnerlichten, dann Taqwa und Ihsan gemäß handelten. Und ALLAH liebt die Muhsin.\qt@no{93}}%
\qt@newcmd\qurantrans@de@dcclxiii{Ihr, die den Iman verinnerlicht habt! ALLAH wird euch bestimmt mit Einigem an Jagdwild prüfen, das eure Hände und eure Speere fangen könnten, damit ALLAH kenntlich macht, wer Ihm im Verborgenen Ehrfurcht entgegenbringt. Wer dann danach übertritt, für den ist eine qualvolle Peinigung bestimmt.\qt@no{94}}%
\qt@newcmd\qurantrans@de@dcclxiv{Ihr, die den Iman verinnerlicht habt! Tötet kein Wild, während ihr im Ihram-Zustand seid. Und wer von euch es vorsätzlich tötet, so ist die Vergeltung (dafür) Gleiches von den An'am wie das, was man getötet hat, - darüber befinden zwei Redliche von euch - als Hady-Opfertier, das bei der Ka'ba geopfert wird. Oder als Kaffara ist die Speisung von Bedürftigen oder Gleiches an Fasten, damit er die bösen Konsequenzen seiner Tat erfährt. ALLAH hat schon das Vorangegangene vergeben. Aber wer es wieder tut, den wird ALLAH der Vergeltung aussetzen. Und ALLAH ist allwürdig und vergeltungs-übend.\qt@no{95}}%
\qt@newcmd\qurantrans@de@dcclxv{Für halal wurden euch der Meeresfang und sein Genuß erklärt - eine Versorgung für euch und für die Reisenden. Doch für haram wurde euch das Jagdwild des Festlandes erklärt, solange ihr im Ihram-Zustand seid. Und handelt Taqwa gemäß ALLAH gegenüber, zu Ihm ihr versammelt werdet.\qt@no{96}}%
\qt@newcmd\qurantrans@de@dcclxvi{ALLAH machte die Ka'ba, Al-baitul-haram, als Hilfeleistung für die Menschen, ebenfalls den Haram-Monat, die Hady-Opfertiere und die mit Halsbändern gekennzeichneten Opfertiere. Dies damit ihr wisst, daß ALLAH alles kennt, was in den Himmeln und was auf Erden ist, und daß ALLAH über alles allwissend ist.\qt@no{97}}%
\qt@newcmd\qurantrans@de@dcclxvii{Wisst, daß ALLAH gewiß hart im Strafen ist und daß ALLAH gewiß allvergebend und allgnädig ist.\qt@no{98}}%
\qt@newcmd\qurantrans@de@dcclxviii{Dem Gesandten obliegt nur das Verkünden. Und ALLAH weiß, was ihr offenbart und was ihr verbergt.\qt@no{99}}%
\qt@newcmd\qurantrans@de@dcclxix{Sag: ``Das Schlechte und das Gute sind nicht gleich, selbst dann nicht, solltest du an dem großen Ausmaß des Schlechten Gefallen finden." So handelt Taqwa gemäß ALLAH gegenüber - ihr Verständige! -, damit ihr erfolgreich werdet.\qt@no{100}}%
\qt@newcmd\qurantrans@de@dcclxx{Ihr, die den Iman verinnerlicht habt! Fragt nicht nach Dingen, die, wenn sie euch verdeutlicht würden, euch unangenehm wären. Doch wenn ihr danach fragt, während der Quran hinabgesandt wird, werden sie euch verdeutlicht. ALLAH hat sie euch vergeben. Und ALLAH ist allvergebend, allnachsichtig.\qt@no{101}}%
\qt@newcmd\qurantrans@de@dcclxxi{Bereits haben danach Leute vor euch gefragt, dann wurden sie ihnen gegenüber kufr-betreibend.\qt@no{102}}%
\qt@newcmd\qurantrans@de@dcclxxii{ALLAH hat weder Bahira noch Saiba noch Wasila noch Haam verordnet. Doch diejenigen, die Kufr betrieben haben, erfinden Lügen im Namen ALLAHs, und viele besinnen sich nicht.\qt@no{103}}%
\qt@newcmd\qurantrans@de@dcclxxiii{Und wenn ihnen gesagt wird: ``Fügt euch dem, was ALLAH hinabgesandt hat, und dem Gesandten", sagten sie: ``Es genügt uns, was wir bei unseren Ahnen vorgefunden haben." Selbst dann, wenn ihre Ahnen nichts wußten und keiner Rechtleitung folgten?!\qt@no{104}}%
\qt@newcmd\qurantrans@de@dcclxxiv{Ihr, die den Iman verinnerlicht habt! Kümmert euch um euch selbst! Euch wird nicht schaden, wer irregeht, wenn ihr der Rechtleitung folgt. Zu ALLAH ist eure Rückkehr allesamt, dann wird ER euch darüber in Kenntnis setzen, was ihr zu tun pflegtet.\qt@no{105}}%
\qt@newcmd\qurantrans@de@dcclxxv{Ihr, die den Iman verinnerlicht habt! (Zum Gebotenen gehört) ein Zeugnis unter euch, wenn ihr im Sterben liegt und das Vermächtnis regelt. (Es sollen es bezeugen) zwei Redliche von euch oder zwei andere von denen, die nicht zu euch gehören, wenn ihr auf Reisen seid und das Unglück des Todes euch ereilt. Ihr haltet sie (beide Zeugen) nach dem rituellen Gebet auf, dann schwören sie bei ALLAH - falls ihr Zweifel hegen solltet -: ``Wir erkaufen uns damit nichts Geringfügiges, selbst dann nicht, wenn derjenige, (für den wir Zeugnis ablegen), ein Verwandter ist, und wir verbergen nicht das von ALLAH gebotene Zeugnis, sonst gehören wir doch zu den Verfehlenden."\qt@no{106}}%
\qt@newcmd\qurantrans@de@dcclxxvi{Doch sollte es sich herausstellen, daß sie (beide Zeugen) eine Verfehlung (Meineid) begangen haben, dann sollen zwei andere an ihrer Stelle herangezogen werden von denjenigen, die an erster Stelle erbberechtigt sind, dann schwören beide bei ALLAH: ``Unser Zeugnis ist doch wahrhaftiger als ihr Zeugnis und wir haben nicht übertreten, sonst gehören wir doch zu den Unrecht-Begehenden."\qt@no{107}}%
\qt@newcmd\qurantrans@de@dcclxxvii{Dies kommt dem am Nächsten, daß sie das Zeugnis wahrheitsgemäß ablegen oder daß sie fürchten, daß (andere) Eide ihre Eide annullieren. Und handelt Taqwa gemäß ALLAH gegenüber und hört zu! ALLAH leitet die fisq-betreibende Gemeinschaft nicht recht.\qt@no{108}}%
\qt@newcmd\qurantrans@de@dcclxxviii{Erwähne) den Tag, an dem ALLAH die Gesandten versammelt, dann sagt: ``Welche Antwort wurde euch gegeben?!" Sie sagten: ``Wir verfügen über kein Wissen darüber, DU bist doch Der Allwissende über das Verborgene."\qt@no{109}}%
\qt@newcmd\qurantrans@de@dcclxxix{Und erinnere daran), als ALLAH sagte: ``'Isa Ibnu-Maryam! Gedenke Meiner Gabe an dich und an deine Mutter, als ICH dich mit Ruhhul-qudus stärkte: du hast zu den Menschen sowohl (als Kind) in der Wiege als auch als Erwachsener gesprochen, auch als ICH dich die Schrift, die Weisheit, At-taurat und Alindschil lehrte, auch als du mit Meiner Zustimmung aus Ton Gleiches wie Vögel geformt hast und in dieses dann eingehaucht hast, dann wurde dieses mit Meiner Zustimmung zu einem (wirklichen) Vogel, und du hast die Blinden und die Aussätzigen geheilt mit Meiner Zustimmung, auch als du die Toten mit Meiner Zustimmung erweckt hast, und als ICH die Kinder Israils von dir abgehalten habe, als du zu ihnen mit den deutlichen Zeichen kamst und diejenigen Kafir unter ihnen sagten: ``Dies ist nichts anderes als mächtige Magie."\qt@no{110}}%
\qt@newcmd\qurantrans@de@dcclxxx{Und erinnere daran), als ICH den Jüngern durch Wahy zuteil werden ließ: ``Verinnerlicht den Iman an Mich und Meinen Gesandten!" Sie sagten: ``Wir haben den Iman verinnerlicht und bezeuge, daß wir gewiß Muslime sind."\qt@no{111}}%
\qt@newcmd\qurantrans@de@dcclxxxi{Und erinnere daran), als die Jünger sagten: ``'Isa Ibnu-Maryam! Kann dein HERR für uns einen mit Speisen gedeckten Tisch vom Himmel nach und nach hinabsenden?" Er sagte: ``Handelt Taqwa gemäß ALLAH gegenüber, solltet ihr Mumin sein."\qt@no{112}}%
\qt@newcmd\qurantrans@de@dcclxxxii{Sie sagten: ``Wir wollen davon speisen, auch damit unsere Herzen Ruhe finden und damit wir wissen, daß du uns bereits die Wahrheit übermittelt hast und wir dafür Zeugen sein werden."\qt@no{113}}%
\qt@newcmd\qurantrans@de@dcclxxxiii{'Isa Ibnu-Maryam sagte: ``ALLAH, unser HERR! Sende uns einen mit Speisen gedeckten Tisch vom Himmel herab, damit dieser ein immer wiederkehrendes Fest für uns wird - für die Ersten und für die Letzten von uns, und als Aya von Dir, und gewähre uns Rizq, denn DU bist gewiß Der Beste, Der Rizq gewährt."\qt@no{114}}%
\qt@newcmd\qurantrans@de@dcclxxxiv{ALLAH sagte: ``Gewiß, ICH lasse ihn zu euch hinabsenden. Wer aber von euch danach Kufr betreibt, den werde ICH doch mit einer Peinigung peinigen, mit der ICH sonst keinen anderen der Menschen peinigen werde."\qt@no{115}}%
\qt@newcmd\qurantrans@de@dcclxxxv{Und erinnere daran), als ALLAH sagte: ``'Isa Ibnu-Maryam! Hast du etwa den Menschen gesagt: ``Nehmt euch mich und meine Mutter als zwei Gottheiten anstelle von ALLAH?!" Er sagte: ``Gepriesen-erhaben bist DU! Mir gebührt es nicht, das zu sagen, wozu ich keine Berechtigung habe. Hätte ich dies gesagt, so hättest DU es bereits gewußt. DU weißt, was in meinem Inneren ist, während ich nicht weiß, was bei Dir ist. Gewiß, DU bist Der Allwissende über alles Verborgene.\qt@no{116}}%
\qt@newcmd\qurantrans@de@dcclxxxvi{Ich habe ihnen nichts gesagt außer dem, womit DU mich beauftragt hast: Dient ALLAH, meinem HERRN und eurem HERRN. Und ich war Zeuge über sie, solange ich unter ihnen weilte. Doch als DU meine Frist hast ablaufen lassen, warst DU der Wachende über sie. Und DU bist über alles bezeugend.\qt@no{117}}%
\qt@newcmd\qurantrans@de@dcclxxxvii{Solltest DU sie peinigen, so sind sie doch Deine Diener. Doch solltest DU ihnen vergeben, so bist DU gewiß Der Allwürdige, Der Allweise. \qt@no{118}}%
\qt@newcmd\qurantrans@de@dcclxxxviii{ALLAH sagte: ``Dies ist ein Tag, an dem den Wahrhaftigen ihre Wahrhaftigkeit nützen wird. Sie haben Dschannat, die von Flüssen durchflossen sind, darin bleiben sie ewig und immer. ALLAH ist zufrieden mit ihnen und sie sind zufrieden mit Ihm. Und dies ist der übergroße Erfolg."\qt@no{119}}%
\qt@newcmd\qurantrans@de@dcclxxxix{ALLAH gehören die Himmeln und die Erde und das, was in ihnen ist. Und ER ist über alles allmächtig.\qt@no{120}}%
\qt@newcmd\qurantrans@de@dccxc{\basmalahde Alhamdulillah: Alles Lob gebührt ALLAH, Der die Himmel und die Erde erschaffen und die Finsternis und das Licht gemacht hat. Dann (trotzdem) stellen diejenigen, die Kufr betrieben haben, ihrem HERRN etwas gleich.\qt@no{1}}%
\qt@newcmd\qurantrans@de@dccxci{ER ist Derjenige, Der euch aus Lehm erschaffen hat, dann einen Zeitpunkt (für euren Tod) bestimmt hat und einen (anderen) bei Sich festgelegten Zeitpunkt (für eure Erweckung). Dann hegt ihr (Kafir) noch Zweifel daran.\qt@no{2}}%
\qt@newcmd\qurantrans@de@dccxcii{Und ER ist ALLAH, (Der Angebetete) in den Himmeln und auf Erden. ER kennt eure Geheimnisse und euer Offenkundiges und kennt das, was ihr euch erwerbt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@dccxciii{Und es kommt zu ihnen keine Aya von den Ayat ihres HERRN, ohne daß sie sich davon abwenden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@dccxciv{Und bereits haben sie die Wahrheit (den Quran) verleugnet, als diese zu ihnen kam, so werden ihnen noch Mitteilungen gemacht über das, was sie zu verspotten pflegten.\qt@no{5}}%
\qt@newcmd\qurantrans@de@dccxcv{Haben sie etwa nicht wahrgenommen, wie viele Generationen WIR vor ihnen zugrunde gerichtet haben, denen WIR auf Erden eine starke Stellung ermöglichten, was WIR euch nicht ermöglicht haben, und über denen WIR den Himmel in Strömen regnen und die Flüsse für sie durchfließen ließen, dann sie wegen ihrer Verfehlungen zugrunde gehen und nach ihnen andere Generationen entstehen ließen?!\qt@no{6}}%
\qt@newcmd\qurantrans@de@dccxcvi{Und würden WIR dir eine auf Schreibblättern geschriebene Schrift nach und nach hinabsenden, die sie mit eigenen Händen berühren könnten, dann würden diejenigen, die Kufr betrieben haben, gewiß sagen: ``Dies ist nichts anderes als starke Magie."\qt@no{7}}%
\qt@newcmd\qurantrans@de@dccxcvii{Und sie sagten: ``Würde ihm doch nur ein Engel herabgesandt!" Und würden WIR einen Engel hinabsenden, dann wäre die Angelegenheit doch abgeschlossen und dann würde ihnen keine Zeit mehr gewährt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@dccxcviii{Und hätten WIR ihn (den Gesandten) aus den Engeln auserwählt, gewiß hätten WIR ihm ebenfalls (die Gestalt) eines Mannes gegeben und gewiß hätten WIR sie genauso verwickeln lassen, wie sie (sich selbst jetzt) verwickeln.\qt@no{9}}%
\qt@newcmd\qurantrans@de@dccxcix{Und gewiß, bereits vor dir wurden Gesandte verspottet, dann überkam diejenigen, die sie verspottet haben, das (an Peinigung), worüber sie gespottet haben.\qt@no{10}}%
\qt@newcmd\qurantrans@de@dccc{Sag: ``Geht auf der Erde umher, dann seht, wie das Anschließende von den Leugnenden war."\qt@no{11}}%
\qt@newcmd\qurantrans@de@dccci{Sag: ``Wem gehört das, was in den Himmeln und auf Erden ist?" Sag: ``ALLAH!" ER hat Sich Selbst der Gnade verpflichtet. ER wird euch doch noch zu einem Tag der Auferstehung versammeln, an dem es keinerlei Zweifel gibt. Diejenigen, die sich selbst zum Scheitern gebracht haben, diese werden den Iman nicht verinnerlichen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@dcccii{Und Ihm gehört alles, was während der Nacht und des Tages ruht. Und ER ist Der Allhörende, Der Allwissende.\qt@no{13}}%
\qt@newcmd\qurantrans@de@dccciii{Sag: ``Soll ich etwa einen anderen als Wali anstelle von ALLAH nehmen, Dem Schöpfer der Himmel und der Erde, Der zu speisen gibt, während ER nicht gespeist wird?!" Sag: ``Gewiß, mir wurde geboten, der Erste zu sein, der den Islam annahm." Und gehöre nicht zu den Muschrik!\qt@no{14}}%
\qt@newcmd\qurantrans@de@dccciv{Sag: ``Gewiß, ich fürchte, sollte ich meinem HERRN widersprechen, die Peinigung eines gewaltigen Tages."\qt@no{15}}%
\qt@newcmd\qurantrans@de@dcccv{Wer davon an diesem Tag verschont wird, dem hat ER bereits Gnade erwiesen. Und dies ist der überragende Gewinn.\qt@no{16}}%
\qt@newcmd\qurantrans@de@dcccvi{Sollte ALLAH dich von einem Unglück treffen lassen, so wird keiner es beheben können außer Ihm. Und sollte ER dich von Gutem treffen lassen, so ist ER über alles allmächtig.\qt@no{17}}%
\qt@newcmd\qurantrans@de@dcccvii{Und ER ist Der Herrschende über Seine Diener. Und ER ist Der Allweise, Der Allkundige.\qt@no{18}}%
\qt@newcmd\qurantrans@de@dcccviii{Sag: ``Was wäre das von noch bedeutenderer Bezeugung (meiner Prophetenschaft)?" Sag: ``ALLAH ist der Bezeugende zwischen mir und euch. Und dieser Quran wurde mir durch Wahy zuteil, damit ich euch mit ihm ermahne sowie diejenigen, die von ihm Kenntnis haben. Bezeugt ihr etwa neben ALLAH noch zusätzliche Götter?!" Sag: ``Ich bezeuge es nicht. ER ist gewiß nur ein einziger Gott, und gewiß, ich sage mich los von dem, was ihr an Schirk betreibt."\qt@no{19}}%
\qt@newcmd\qurantrans@de@dcccix{Diejenigen, denen WIR die Schrift zuteil werden ließen, kennen ihn (den Gesandten), wie sie ihre eigenen Kinder kennen. Diejenigen, die sich selbst zum Scheitern brachten, diese werden den Iman nicht verinnerlichen\qt@no{20}}%
\qt@newcmd\qurantrans@de@dcccx{Und keiner begeht mehr Unrecht als derjenige, der im Namen ALLAHs Lügen erfindet oder Seine Ayat verleugnet. Gewiß, die Unrecht-Begehenden sind niemals erfolgreich.\qt@no{21}}%
\qt@newcmd\qurantrans@de@dcccxi{Und (erwähne) den Tag, an dem WIR sie allesamt versammeln werden, dann zu denjenigen, die Schirk betrieben haben, sagen: ``Wo sind denn die von euch (Mir beigesellten) Partner, die ihr (als solche) zu nennen pflegtet?!"\qt@no{22}}%
\qt@newcmd\qurantrans@de@dcccxii{Dann wird ihre Fitna nichts anderes sein, als daß sie sagen: ``Bei ALLAH, unser HERR! Wir waren keine Muschrik."\qt@no{23}}%
\qt@newcmd\qurantrans@de@dcccxiii{Siehe, wie sie sich selbst belogen haben. Und ihnen ist abhanden gekommen, was sie zu erlügen pflegten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@dcccxiv{Und manche von ihnen hören dir zu. Doch WIR haben ihre Herzen mit Bedeckung umhüllt, um ihn (den Quran) nicht zu begreifen, und ihre Ohren schwerhörig gemacht. Und selbst dann, wenn ihnen jede Aya gewahr würde, würden sie den Iman daran nicht verinnerlichen. Dann, als sie zu dir kamen, um mit dir zu disputieren, sagten diejenigen, die Kufr betrieben haben: ``Dies ist doch nichts anderes als Märchen der Vorfahren!"\qt@no{25}}%
\qt@newcmd\qurantrans@de@dcccxv{Während sie davon (vom Quran) abraten und sich davon abwenden. Und sie richten doch nur sich selbst zugrunde, ohne daß sie es bemerken.\qt@no{26}}%
\qt@newcmd\qurantrans@de@dcccxvi{Und würdest du doch nur sehen, als sie dem Feuer vorstellig wurden und dann sagten: ``Würden wir doch zurückgebracht, (würden wir) die Ayat unseres HERRN nicht verleugnen und mit den Mumin sein!"\qt@no{27}}%
\qt@newcmd\qurantrans@de@dcccxvii{Nein, sondern ihnen offenbarte sich das, was sie vorher zu verbergen pflegten. Und würden sie zurückgebracht, würden sie doch wiederholen, was ihnen untersagt wurde. Und gewiß, sie sind doch Lügner.\qt@no{28}}%
\qt@newcmd\qurantrans@de@dcccxviii{Sie sagten: ``Es gibt doch nur das unsere diesseitige Leben und wir werden niemals auferweckt."\qt@no{29}}%
\qt@newcmd\qurantrans@de@dcccxix{Und würdest du doch sehen, als sie ihrem HERRN vorstellig wurden! ER sagte: ``Ist dies nicht dieWahrheit?!" Sie sagten: ``Doch mit Sicherheit, bei unserem HERRN!" ER sagte: ``Dann erfahrt die Peinigung für das, was ihr an Kufr zu betreiben pflegtet."\qt@no{30}}%
\qt@newcmd\qurantrans@de@dcccxx{Bereits verloren haben diejenigen, die das Treffen mit ALLAH abgeleugnet haben. Dann, wenn die Stunde (der Jüngste Tag) sie unerwartet ereilt hat, werden sie sagen: ``Weh uns für das, was wir im Diesseits vernachlässigt haben.", während sie ihre Schulden auf ihren Rücken tragen. Doch erbärmlich ist das, was sie tragen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@dcccxxi{Doch das diesseitige Leben ist nichts anderes als Spiel und Vergnügen, und das Jenseits ist besser für diejenigen, die Taqwa gemäß handeln. Wollt ihr euch nicht besinnen?!\qt@no{32}}%
\qt@newcmd\qurantrans@de@dcccxxii{Bereits wissen WIR, daß dich das, was sie sagen, sicher traurig macht. Doch sie bezichtigen nicht dich der Lüge, sondern die Unrecht-Begehenden leugnen ALLAHs Ayat ab.\qt@no{33}}%
\qt@newcmd\qurantrans@de@dcccxxiii{Und gewiß, bereits vor dir wurden Gesandte der Lüge bezichtigt, so haben sie sich in Geduld geübt für das, weswegen sie der Lüge bezichtigt wurden und ihnen Schaden zugefügt wurde, bis Unser Sieg ihnen zuteil wurde. Und es gibt keinen, der ALLAHs Versprechen abändern kann. Und gewiß, bereits kam zu dir von den Nachrichten der Gesandten.\qt@no{34}}%
\qt@newcmd\qurantrans@de@dcccxxiv{Und sollte ihr Abwenden (vom Islam) dir unerträglich werden, dann (versuche), wenn es dir gelingt, einen Tunnel in die Erde oder eine Treppe in den Himmel zu finden, damit du ihnen eine Aya bringst. Und hätte ALLAH es gewollt, hätte ER sie doch alle in der Rechtleitung vereint, so gehöre nicht zu den Unwissenden!\qt@no{35}}%
\qt@newcmd\qurantrans@de@dcccxxv{Doch nur diejenigen, die zuhören, hören auf dich, während die Toten ALLAH erweckt. Dann werden sie zu Ihm zurückgebracht.\qt@no{36}}%
\qt@newcmd\qurantrans@de@dcccxxvi{Und sie sagten: ``Würde ihm doch eine Aya von seinem HERRN nach und nach hinabgesandt!" Sag: ``Gewiß, ALLAH ist immer im Stande, eine Aya nach und nach hinabzusenden." Doch die meisten von ihnen wissen es nicht.\qt@no{37}}%
\qt@newcmd\qurantrans@de@dcccxxvii{Es gibt weder ein sich (auf Erden) bewegendes Lebewesen, noch einen Vogel, welcher mit beiden eigenen Flügeln fliegt, die nicht Umam euresgleichen sind. WIR haben in der Schrift nichts weggelassen. Dann werden sie zu ihrem HERRN versammelt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@dcccxxviii{Doch diejenigen, die Unsere Ayat verleugnet haben, sind taub und stumm in den Finsternissen. Wen ALLAH will, den läßt ER abirren, und wen ER will, den bringt ER auf einen geraden Weg.\qt@no{39}}%
\qt@newcmd\qurantrans@de@dcccxxix{Sag: `` Wie seht ihr es? Sollte euch ALLAHs Peinigung treffen oder die Stunde (der Jüngste Tag) ereilen, würdet ihr jemanden außer ALLAH anflehen, solltet ihr wahrhaftig sein?"\qt@no{40}}%
\qt@newcmd\qurantrans@de@dcccxxx{Nein, sondern nur Ihn fleht ihr an! So befreit ER euch von dem, weswegen ihr Ihn angefleht habt, wenn ER will. Und ihr vergesst, womit ihr Schirk betrieben habt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@dcccxxxi{Und gewiß, bereits entsandten WIR (Gesandte) zu Umam vor dir, dann ließen WIR sie von Not und Krankheit befallen, damit sie (um Gnade) flehen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@dcccxxxii{Und hätten sie doch (um Gnade) gefleht, als unsere Peinigung sie ereilte! Doch ihre Herzen wurden hart und der Satan ließ ihnen das schön erscheinen, was sie zu tun pflegten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@dcccxxxiii{Und als sie das vergessen haben, womit sie ermahnt wurden, haben WIR ihnen die Türen zu allen (angenehmen) Dingen geöffnet. Doch nachdem sie dann mit dem Spaß hatten, was ihnen zuteil wurde, haben WIR sie unerwartet zugrunde gerichtet, sogleich waren sie regungslos niedergeschlagen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@dcccxxxiv{So wurden die Leute, die Unrecht begangen haben, ausgerottet. Und alhamdulillah: alles Lob gebührt ALLAH, Dem HERRN aller Schöpfung.\qt@no{45}}%
\qt@newcmd\qurantrans@de@dcccxxxv{Sag: `` Wie seht ihr es? Sollte ALLAH euch euer Gehör und euer Sehvermögen wegnehmen und eure Herzen versiegeln, welche Gottheit außer ALLAH wird euch dies zurückgeben?" Siehe, wie WIR die Ayat verdeutlichen, dann sie sich davon abwenden.\qt@no{46}}%
\qt@newcmd\qurantrans@de@dcccxxxvi{Sag: `` Wie seht ihr es? Sollte euch ALLAHs Peinigung unerwartet oder voraussehbar überkommen, werden etwa andere damit zugrunde gerichtet außer den unrecht-begehenden Leuten?"\qt@no{47}}%
\qt@newcmd\qurantrans@de@dcccxxxvii{Und WIR entsenden die Gesandten nur als Überbringer froher Botschaft und als Warner. Wer dann den Iman verinnerlicht und gottgefällig Gutes tut, diese überkommt keine Furcht und sie werden nicht traurig werden.\qt@no{48}}%
\qt@newcmd\qurantrans@de@dcccxxxviii{Doch diejenigen, die Unsere Ayat verleugnen, werden von der Peinigung erfaßt wegen dem, was sie an Fisq zu betreiben pflegten.\qt@no{49}}%
\qt@newcmd\qurantrans@de@dcccxxxix{Sag: ``Ich sage euch nicht, daß ich über dieMagazine von ALLAHs (Rizq) verfüge. Auch kenne ich das Verborgene nicht. Noch sage ich euch ebenfalls nicht, daß ich ein Engel sei. Ich folge doch nur dem, was mir an Wahy zuteil wird." Sag: ``Sind etwa der Blinde und der Sehende gleich? Denkt ihr etwa nicht nach?!"\qt@no{50}}%
\qt@newcmd\qurantrans@de@dcccxl{Und ermahne damit diejenigen, die fürchten, zu ihrem HERRN versammelt zu werden, während es für sie (dabei) anstelle von Ihm weder Wali noch Fürbittenden gibt, damit sie Taqwa gemäß handeln.\qt@no{51}}%
\qt@newcmd\qurantrans@de@dcccxli{Und schicke nicht diejenigen weg, die Bittgebete an ihren HERRN am Morgen und am Abend im Streben nach Seinem Wohlgefallen richten. DU verantwortest nichts von ihrer Rechenschaft und sie verantworten nichts von deiner Rechenschaft. Denn, wenn du sie wegschickst, so gehörst du zu den Unrecht-Begehenden.\qt@no{52}}%
\qt@newcmd\qurantrans@de@dcccxlii{Und solcherart haben WIR die einen von ihnen durch die anderen der Fitna ausgesetzt, damit sie sagen: ``Sind diese etwa diejenigen, die ALLAH unter uns auserwählt hat?" Weiß ALLAH etwa nicht besser Bescheid über die Dankbaren?!\qt@no{53}}%
\qt@newcmd\qurantrans@de@dcccxliii{Und wenn zu dir diejenigen kommen, die den Iman an Unsere Ayat verinnerlicht haben, so sag: ``Salam sei mit euch. Euer HERR hat sich selbst der Gnade verpflichtet: Wer von euch Verwerfliches aus Unwissenheit getan, dann danach bereut und gottgefällig Gutes vollbracht hat, so ist ER gewiß allvergebend, allgnädig."\qt@no{54}}%
\qt@newcmd\qurantrans@de@dcccxliv{Und solcherart verdeutlichen WIR die Ayat und damit du den Weg der schwer Verfehlenden erkennst.\qt@no{55}}%
\qt@newcmd\qurantrans@de@dcccxlv{Sag: ``Mir ist es untersagt, denen zu dienen, an die ihr anstelle von ALLAH Bittgebete richtet." Sag: ``Ich folge nicht euren Neigungen, sonst wäre ich bereits irregegangen und ich gehöre dann nicht mehr zu den Rechtgeleiteten!"\qt@no{56}}%
\qt@newcmd\qurantrans@de@dcccxlvi{Sag: ``Ich folge doch einem klaren Zeichen von meinem HERRN, während ihr Ihn verleugnet habt. Doch ich verfüge nicht über das, was ihr übereilt haben wollt. Denn die Entscheidung (darüber) unterliegt nur ALLAH. ER spricht dieWahrheit und ER ist Der Beste der Richtenden."\qt@no{57}}%
\qt@newcmd\qurantrans@de@dcccxlvii{Sag: ``Würde ich über das verfügen, was ihr übereilt haben wollt, so wäre die Angelegenheit zwischen mir und euch doch abgeschlossen. Und ALLAH ist allwissend über die Unrecht-Begehenden."\qt@no{58}}%
\qt@newcmd\qurantrans@de@dcccxlviii{Und bei Ihm liegen die Zugänge zum Verborgenen. Keiner kennt sie außer Ihm. Auch kennt ER, was auf dem Festland und was im Meer ist. Auch fällt weder ein Baumblatt, ohne daß ER es kennt, noch eine Saat in den Finsternissen der Erde, noch etwas Feuchtes, noch etwas Trockenes, ohne daß dies in einer deutlichen Schrift festgelegt wäre.\qt@no{59}}%
\qt@newcmd\qurantrans@de@dcccxlix{Und ER ist Derjenige, Der (eure Seelen) im Schlaf einzieht und weiß, was ihr am Tage an Verfehlungen begangen habt, dann läßt ER euch darin aufwecken, damit eine festgelegte Zeit abläuft, dann wird eure Rückkehr zu Ihm sein, dann wird ER euch in Kenntnis setzen über das, was ihr zu tun pflegtet.\qt@no{60}}%
\qt@newcmd\qurantrans@de@dcccl{Und ER ist Der Herrschende über Seine Diener. Und ER entsendet euch Bewahrende (Engel). Wenn dann der Tod den einen von euch ereilt, ziehen Unsere Entsandten (Engel) (seine Seele) ein, während sie nichts vernachlässigen.\qt@no{61}}%
\qt@newcmd\qurantrans@de@dcccli{Dann wurden sie zu ALLAH, ihrem wahren Maula zurückgeschickt. Ja! Ihm allein obliegt die Entscheidung und ER ist Der Schnellste aller Abrechnenden.\qt@no{62}}%
\qt@newcmd\qurantrans@de@dccclii{Sag: ``Wer rettet euch aus den Finsternissen des Festlandes und des Meeres? Ihr richtet an Ihn Bittgebete in Ergebenheit und im Verborgenen: ``Wenn ER uns von diesem (Unglück) errettet, so werden wir gewiß von den Dankbaren sein."\qt@no{63}}%
\qt@newcmd\qurantrans@de@dcccliii{Sag: ``ALLAH errettet euch von diesem und aus jeder Kümmernis, dennoch praktiziert ihr dann Schirk."\qt@no{64}}%
\qt@newcmd\qurantrans@de@dcccliv{Sag: ``ER ist derjenige, der in der Lage ist, auf euch eine Peinigung von oben über euch und unter euren Füßen zu schicken, oder daß ER euch in Gruppen zersplittert und die einen von euch die Gewalttätigkeit der anderen erfahren läßt." Siehe, wie WIR die Ayat verdeutlichen, damit sie begreifen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@dccclv{Doch deine Leute haben dies verleugnet, und dies ist die Wahrheit. Sag: ``Ich bin über euch kein Wakil."\qt@no{66}}%
\qt@newcmd\qurantrans@de@dccclvi{Jede Ankündigung geschieht zu ihrer Zeit, und ihr werdet es noch wissen.\qt@no{67}}%
\qt@newcmd\qurantrans@de@dccclvii{Und solltest du diejenigen sehen, die über Unsere Ayat lästern, so wende dich von ihnen (so lange) ab, bis sie zu einem anderen Thema übergehen. Und sollte der Satan dich dies vergessen lassen, so sitze nach der Erinnerung nicht länger mit den unrecht-begehenden Leuten!\qt@no{68}}%
\qt@newcmd\qurantrans@de@dccclviii{Diejenigen, die Taqwa gemäß handeln, verantworten von ihrer Rechenschaft nichts, doch (sie haben die Pflicht) zur Ermahnung, damit sie (die Lästerer) Taqwa suchen.\qt@no{69}}%
\qt@newcmd\qurantrans@de@dccclix{Und laß ab von denjenigen, die ihren Din als Spiel und Zeitvertreib nehmen und vom diesseitigen Leben eingenommen sind. Und ermahne mit ihm (dem Quran), damit keine Seele dem ausgeliefert wird, was sie sich erworben hat, und es dabei für sie anstelle von ALLAH weder einen Wali noch einen Fürbittenden gibt. Auch sollte sie sich mit irgendeiner Ablösung freikaufen wollen, wird dies von ihr nicht angenommen. Diese sind diejenigen, die dem ausgeliefert sind, was sie sich erworben haben, für sie ist ein Trank aus Siedendheißem und eine qualvolle Peinigung bestimmt wegen dem, was sie an Kufr zu betreiben pflegten.\qt@no{70}}%
\qt@newcmd\qurantrans@de@dccclx{Sag: ``Sollen wir etwa anstelle von ALLAH an etwas Bittgebete richten, das uns weder nützt noch schadet, damit wir auf unseren Fersen umgedreht werden, nachdem ALLAH uns rechtgeleitet hat, ähnlich wie derjenige, der von den Satanen im Lande weit in die Irre getrieben wird voller Zweifel, während er Intimi hat, die ihn zur Rechtleitung einladen: ``Komm doch zu uns"?! Sag: ``Gewiß, ALLAHs Rechtleitung ist die wahre Rechtleitung. Und uns wurde geboten, uns mit Islam Dem HERRN aller Schöpfung zu fügen,\qt@no{71}}%
\qt@newcmd\qurantrans@de@dccclxi{auch: Verrichtet ordnungsgemäß das rituelle Gebet und handelt Taqwa gemäß Ihm gegenüber!" Und ER ist Derjenige, zu Ihm ihr versammelt werdet.\qt@no{72}}%
\qt@newcmd\qurantrans@de@dccclxii{Und ER ist Derjenige, Der die Himmel und die Erde in Gesetzmäßigkeit erschuf. Und an jenem Tag, wenn ER sagt: ``Sei!" und es ist, ist Sein Wort die Wahrheit. Und nur Ihm gehört die Herrschaft an jenem Tag, wenn in As-sur geblasen wird. ER ist Der Allwissende über das Verborgene und das Sichtbare. Und ER ist Der Allweise, Der Allkundige.\qt@no{73}}%
\qt@newcmd\qurantrans@de@dccclxiii{Und erinnere daran), als Ibrahim seinem Vater Azar sagte: ``Nimmst du etwa Statuen als Götter? Gewiß, ich sehe dich mit deinen Leuten im deutlichen Abirren."\qt@no{74}}%
\qt@newcmd\qurantrans@de@dccclxiv{Und solcherart zeigten WIR Ibrahim die Zeichen der Himmel und der Erde und damit er zu denjenigen gehört, die über Gewißheit verfügen.\qt@no{75}}%
\qt@newcmd\qurantrans@de@dccclxv{Als über ihn dann die Nacht hereinbrach, sah er einen Planeten. ER sagte: ``Das ist mein HERR." Doch als dieser verschwand, sagte er: ``Ich mag nicht solche, die verschwinden."\qt@no{76}}%
\qt@newcmd\qurantrans@de@dccclxvi{Als er danach den aufgehenden Mond sah, sagte er: ``Das ist mein HERR." Doch als dieser verschwand, sagte er: ``Wenn mein HERR mich nicht rechtleitet, so werde ich gewiß zu den irregegangenen Leuten gehören."\qt@no{77}}%
\qt@newcmd\qurantrans@de@dccclxvii{Als er daraufhin die aufgehende Sonne sah, sagte er: ``Dies ist mein HERR, diese ist größer." Doch als sie verschwand, sagte er: ``Meine Leute! Ich sage mich los von dem, was ihr an Schirk betreibt.\qt@no{78}}%
\qt@newcmd\qurantrans@de@dccclxviii{Ich habe mich zu Dem hingewendet, Der die Himmel und die Erde erschuf, als Hanif. Und ich gehöre nicht zu den Muschrik."\qt@no{79}}%
\qt@newcmd\qurantrans@de@dccclxix{Seine Leute disputierten dann mit ihm. Er sagte: ``Wollt ihr etwa mit mir über ALLAH disputieren, nachdem ER mich bereits rechtgeleitet hat. Und ich fürchte mich nicht vor dem, was ihr neben Ihm mit Schirk verehrt, es sei denn, mein HERR will etwas anderes. Mein HERR umfaßt alles mit Allwissen. Wollt ihr euch nicht entsinnen?!\qt@no{80}}%
\qt@newcmd\qurantrans@de@dccclxx{Und wie kann ich mich vor dem fürchten, was ihr mit Schirk verehrt, während ihr nicht fürchtet, daß ihr neben ALLAH das an Schirk betreibt, wozu ER euch niemals eine Bestätigung hinabsandte?! Also welche von beiden Gruppen ist mehr berechtigt, sich sicher zu fühlen, solltet ihr es wissen?\qt@no{81}}%
\qt@newcmd\qurantrans@de@dccclxxi{Diejenigen, die den Iman verinnerlicht haben und ihren Iman nicht mit Übertretung vermischen, diese fühlen sich sicher und sie sind rechtgeleitet.\qt@no{82}}%
\qt@newcmd\qurantrans@de@dccclxxii{Das ist ein Argument von Uns, welches WIR Ibrahim seinen Leuten gegenüber gaben. WIR erhöhen um Stellungen, wen WIR wollen. Gewiß, dein HERR ist allweise, allwissend.\qt@no{83}}%
\qt@newcmd\qurantrans@de@dccclxxiii{Und WIR haben ihm Ishaq und Ya'qub geschenkt. Beide haben WIR rechtgeleitet. WIR leiteten auch vorher Nuh recht, ebenfalls von seiner Nachkommenschaft: Dawud, Sulaiman, Ayyub, Yusuf, Musa und Harun. So belohnen WIR die Muhsin.\qt@no{84}}%
\qt@newcmd\qurantrans@de@dccclxxiv{Ebenso (leiteten WIR recht) Zakaria, Yahia, 'Isa und Elias - sie alle gehörten zu den gottgefällig Guttuenden -\qt@no{85}}%
\qt@newcmd\qurantrans@de@dccclxxv{auch Isma'il, Alys', Yunus und Lut. Sie alle haben WIR vor den Menschen ausgezeichnet,\qt@no{86}}%
\qt@newcmd\qurantrans@de@dccclxxvi{so auch manche von ihren Vätern, Nachkommen und Brüdern. UndWIR erwählten sie aus und leiteten sie zum geraden Weg recht.\qt@no{87}}%
\qt@newcmd\qurantrans@de@dccclxxvii{Dies ist die Rechtleitung ALLAHs, damit leitet ER recht, wen ER will von Seinen Dienern. Und hätten sie Schirk betrieben, wäre ihnen gewiß verloren gegangen, was sie zu tun pflegten.\qt@no{88}}%
\qt@newcmd\qurantrans@de@dccclxxviii{Diese sind diejenigen, denen WIR die Schrift zuteil werden ließen, die Weisheit und das Prophetentum. Und sollten diese (deine Leute) dem gegenüber Kufr betreiben, werden WIR dann dafür eine Gemeinschaft als Wakil einsetzen, die dem gegenüber keinen Kufr betreibt.\qt@no{89}}%
\qt@newcmd\qurantrans@de@dccclxxix{Diese sind diejenigen, die ALLAH rechtleitete, so folge ihrer Rechtleitung! Sag: ``Ich erbitte von euch dafür keinen Lohn." Es ist doch nur eine Ermahnung für die Geschöpfe.\qt@no{90}}%
\qt@newcmd\qurantrans@de@dccclxxx{Keineswegs haben sie ALLAH verherrlicht, wie es Ihm gebührt, als sie sagten: ``ALLAH hat keinem Menschen irgend etwas hinabgesandt." Sag: ``Wer hat denn die Schrift hinabgesandt, mit der Musa als Licht und als Rechtleitung für die Menschen kam? Ihr macht sie zu Schreibblättern, die ihr offenlegt, aber auch vieles davon verheimlicht. Und euch wurde auch das gelehrt, was weder ihr gewußt habt noch eure Väter." Sag: ``ALLAH (sandte sie hinab)" Dann laß sie sich mit ihrem Lästern vergnügen!\qt@no{91}}%
\qt@newcmd\qurantrans@de@dccclxxxi{Und dieser (Quran) ist eine Schrift, die WIR hinabsandten, er ist voller Baraka und als Bestätigung für das, was vor ihm war, und damit du (mit ihm) Ummul-qura ermahnst, und was um sie herum ist. Und diejenigen, die den Iman an das Jenseits verinnerlichen, verinnerlichen den Iman an ihn und beachten ihre rituellen Gebete.\qt@no{92}}%
\qt@newcmd\qurantrans@de@dccclxxxii{Und keiner begeht mehr Unrecht als derjenige, der Lügen im Namen ALLAHs erfindet oder sagt: ``Mir wurde Wahy zuteil", obwohl ihm nichts von Wahy zuteil wurde, und derjenige, der sagt: ``Ich werde Ähnliches hinabsenden wie das, was ALLAH hinabsandte." Und würdest du nur sehen, wenn die Unrecht-Begehenden sich mitten im Todeskampf befinden, während die Engel auf sie einschlagen: ``Liefert eure Seelen aus! Heute werdet ihr mit der erniedrigenden Peinigung entlohnt für das, was ihr an Unwahrem über ALLAH gesprochen habt und weil ihr euch Seinen Ayat gegenüber hochmütig verhalten habt."\qt@no{93}}%
\qt@newcmd\qurantrans@de@dccclxxxiii{Gewiß, bereits seid ihr dann zu Uns einzeln gekommen, genauso wie WIR euch das erste Mal erschufen, und habt das hinter euch gelassen, was WIR euch zuteil werden ließen. Doch WIR sehen mit euch nicht eure Fürbittenden, die ihr unter euch als Partner (für ALLAH) angenommen habt. Gewiß, bereits ist (die Bindung) zwischen euch unterbrochen und euch ist das abhanden gekommen, was ihr angenommen habt.\qt@no{94}}%
\qt@newcmd\qurantrans@de@dccclxxxiv{Gewiß, ALLAH ist Derjenige, Der die Saat und den Kern aufgehen läßt. ER bringt das Lebendige aus dem Toten hervor und ER ist Der Hervorbringende des Toten aus dem Lebendigen. Dies ist ALLAH. Also wie wurdet ihr denn verleitet?!\qt@no{95}}%
\qt@newcmd\qurantrans@de@dccclxxxv{ER ist Derjenige, Der den Morgen aufbrechen läßt. ER hat die Nacht zur Ruhephase und die Sonne und den Mond für die Berechnungen gemacht. Dies ist die Bestimmung Des Allwürdigen, Des Allwissenden.\qt@no{96}}%
\qt@newcmd\qurantrans@de@dccclxxxvi{Auch ist ER Derjenige, Der euch die Sterne gemacht hat, damit ihr euch an ihnen in den Finsternissen des Festlandes und des Meeres orientieren könnt. Bereits haben WIR die Ayat verdeutlicht für Menschen, die über Wissen verfügen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@dccclxxxvii{Ebenso ist ER Derjenige, Der euch aus einem einzigen Wesen hervorbrachte, so gab es dann Niederlassungs- und Aufbewahrungsort. Bereits haben WIR die Ayat verdeutlicht für Menschen, die verständig sind.\qt@no{98}}%
\qt@newcmd\qurantrans@de@dccclxxxviii{Und ER ist Derjenige, Der vom Himmel Wasser hinabsenden ließ. So haben WIR damit Pflanzenwuchs jeder Art ermöglicht, so haben WIR daraus Grünes sprießen lassen, von dem WIR übereinander liegende Saatkörner hervorbrachten. Und aus den Dattelpalmen, aus ihrem Blütenstand (haben WIR hervorgebracht) herunterhängende Dattelbüschel und Dschannat mit Trauben, Oliven und Granatäpfeln - einander ähnelnd und nicht ähnelnd. Betrachtet ihre Früchte, wenn sie Früchte tragen und reif werden. Gewiß, darin sind zweifelsohne Ayat für Menschen, die den Iman verinnerlichen.\qt@no{99}}%
\qt@newcmd\qurantrans@de@dccclxxxix{Doch sie gesellten ALLAH Partner von Dschinn bei - und bereits erschuf ER sie. Und sie haben Ihm Söhne und Töchter angedichtet ohne Wissen. Gepriesenerhaben ist ER über das, was sie erdichten.\qt@no{100}}%
\qt@newcmd\qurantrans@de@dcccxc{ER ist Der Schöpfer der Himmel und der Erde. Wie kann Ihm ein Sohn gehören, wo ER niemals eine Partnerin hatte und doch alles erschuf?! Und ER ist über alles allwissend.\qt@no{101}}%
\qt@newcmd\qurantrans@de@dcccxci{Dies ist ALLAH euer HERR. Es gibt keine Gottheit außer Ihm. ER ist Der Schöpfer allen Seins, so dient Ihm. Und ER ist über allesWakil.\qt@no{102}}%
\qt@newcmd\qurantrans@de@dcccxcii{Das Sehvermögen erfaßt Ihn nicht, doch ER erfaßt das Sehvermögen. Und ER ist Der Allgütige, Der Allkundige.\qt@no{103}}%
\qt@newcmd\qurantrans@de@dcccxciii{Bereits wurden euch doch Einblick-Gewährende von eurem HERRN zuteil. Wer also diese wahrnimmt, so tut er dies (nur) für sich selbst. Doch wer blind bleibt, so tut er dies (nur) gegen sich selbst. Und ich bin kein Hüter über euch.\qt@no{104}}%
\qt@newcmd\qurantrans@de@dcccxciv{Und solcherart verdeutlichen WIR die Ayat. Und sie werden sagen: ``Du hast (dies) studiert ``. Doch WIR werden dies verdeutlichen für Menschen, die über Wissen verfügen.\qt@no{105}}%
\qt@newcmd\qurantrans@de@dcccxcv{Folge dem, was dir an Wahy von deinem HERRN zuteil wurde. Es ist keine Gottheit außer Ihm. Und wende dich weg von den Muschrik!\qt@no{106}}%
\qt@newcmd\qurantrans@de@dcccxcvi{Und hätte ALLAH es gewollt, hätten sie kein Schirk betrieben. Und WIR haben dich über sie nicht als Hüter eingesetzt und du bist über sie kein Wakil.\qt@no{107}}%
\qt@newcmd\qurantrans@de@dcccxcvii{Und verunglimpft nicht das, woran sie anstelle von ALLAH Bittgebete richten, sonst verunglimpfen sie ALLAH in übertretender Weise aus Unwissenheit! Solcherart ließen WIR jeder Umma ihr Handeln schön erscheinen, dann aber wird ihre Umkehr zu ihrem HERRN sein, dann wird ER sie darüber in Kenntnis setzen, was sie zu tun pflegten.\qt@no{108}}%
\qt@newcmd\qurantrans@de@dcccxcviii{Sie legten bei ALLAH nachdrückliche Eide ab, daß sie, wenn ihnen eine Aya zuteil würde, daran den Iman verinnerlichen würden. Sag: ``Über die Ayat verfügt nur ALLAH." Und was läßt euch annehmen, daß sie, wenn diese (Ayat) kämen, daran den Iman verinnerlichen würden?!\qt@no{109}}%
\qt@newcmd\qurantrans@de@dcccxcix{Und WIR lassen ihre Herzen und ihre Blicke durcheinanderbringen, wie sie den Iman daran das ersteMal nicht verinnerlichten, und WIR lassen sie in ihrer Maßlosigkeit sich verblenden.\qt@no{110}}%
\qt@newcmd\qurantrans@de@cm{Und würden WIR ihnen die Engel hinabsenden und die Toten zu ihnen sprechen lassen und ihnen gegenüber alles versammeln, würden sie den Iman nicht verinnerlichen, es sei denn, ALLAH will es. Doch die meisten unter ihnen sind unwissend.\qt@no{111}}%
\qt@newcmd\qurantrans@de@cmi{Und solcherart haben WIR jedem Gesandten einen Feind bestimmt - die Satane der Menschen und der Dschinn, die einen von ihnen geben den anderen schön erscheinendes Gerede als Täuschung ein. Und hätte dein HERR es gewollt, hätten sie es nicht getan. Also laß ab von ihnen und von dem, was sie erlügen.\qt@no{112}}%
\qt@newcmd\qurantrans@de@cmii{Und dazu sollen sich die Herzen derjenigen zuneigen, die den Iman an das Jenseits nicht verinnerlichen, und sie sollen damit zufrieden sein und sie sollen (an Verfehlungen) verüben, was (sie) verüben wollen.\qt@no{113}}%
\qt@newcmd\qurantrans@de@cmiii{Sag): ``Soll ich etwa außer ALLAH einen (anderen) Richtenden erstreben, und ER ist doch Derjenige, Der euch die Schrift hinabsandte, in Ausführlichkeit?" Und diejenigen, denen WIR die Schrift zuteil werden ließen, wissen, daß dies von deinem HERRN wahrheitsgemäß nach und nach hinabgesandt wurde. Also gehöre nicht zu den Zweifel-Hegenden!\qt@no{114}}%
\qt@newcmd\qurantrans@de@cmiv{Und die Worte deines HERRN sind in Wahrhaftigkeit und Gerechtigkeit vollständig geworden. Es gibt keinen, der Seine Worte verändern kann. Und ER ist Der Allhörende, Der Allwissende.\qt@no{115}}%
\qt@newcmd\qurantrans@de@cmv{Und wenn du auf die meisten derjenigen, die auf Erden sind, hören würdest, würden sie dich vom Wege ALLAHs verleiten. Sie folgen nur der Spekulation und sie verbreiten gewiß nur Lügen!\qt@no{116}}%
\qt@newcmd\qurantrans@de@cmvi{Gewiß, dein HERR ist Derjenige, Der bestens Bescheid darüber weiß, wer von Seinem Weg abirrt, und ER kennt am besten die Rechtgeleiteten.\qt@no{117}}%
\qt@newcmd\qurantrans@de@cmvii{So speist (nur) von dem, worauf ALLAHs Name (beim Schächten) erwähnt wurde, wenn ihr den Iman an Seine Ayat wirklich verinnerlicht habt.\qt@no{118}}%
\qt@newcmd\qurantrans@de@cmviii{Und weshalb speist ihr nicht von dem, worauf ALLAHs Name (beim Schächten) erwähnt wurde, nachdem ER euch deutlich das vorgetragen hat, was ER euch für haram erklärte - außer dem, wozu ihr gezwungen werdet?! Und viele (Menschen) verleiten (andere) mit ihren Neigungen ohne Wissen. Gewiß, dein HERR kennt die Übertretenden am besten.\qt@no{119}}%
\qt@newcmd\qurantrans@de@cmix{Auch enthaltet euch das Äußere der Verfehlung sowie deren Inneres! Gewiß, denjenigen, die sich Verfehlungen erwerben, wird mit dem vergolten, was sie sich zu erwerben pflegten.\qt@no{120}}%
\qt@newcmd\qurantrans@de@cmx{Und speist nicht von dem, worauf ALLAHs Name (beim Schächten) nicht erwähnt wurde, und gewiß, dies ist doch Fisq. Und gewiß, die Satane geben doch ihren Wali ein, um mit euch zu disputieren. Und solltet ihr auf sie hören, gewiß ihr seid dann doch Muschrik.\qt@no{121}}%
\qt@newcmd\qurantrans@de@cmxi{Ist denn derjenige, der tot war, und dem WIR dann Leben schenkten und Licht gaben, mit dem er unter den Menschen wandelt, dem gleich, der in Finsternissen verharrt und nicht daraus hervorzugehen vermag?! Solcherart wurde den Kafir schön gemacht, was sie zu tun pflegten.\qt@no{122}}%
\qt@newcmd\qurantrans@de@cmxii{Und solcherart ließen WIR in jedem Ort dessen größten Schwer-Verfehlenden darin (andere) täuschen. Doch sie täuschen nur sich selbst, ohne es zu merken.\qt@no{123}}%
\qt@newcmd\qurantrans@de@cmxiii{Und immer wieder als zu ihnen eine Aya kam, sagten sie: ``Wir werden den Iman nicht verinnerlichen, bis uns Ähnliches zuteil wird, wie es ALLAHs Gesandten zuteil wurde." Doch ALLAH weiß am besten, wen ER mit Seiner Botschaft beauftragt. Diejenigen, die schwere Verfehlungen begehen, wird bei ALLAH Er-niedrigung und qualvolle Peinigung treffen wegen dem, was sie an Täuschungen durchzuführen pflegten.\qt@no{124}}%
\qt@newcmd\qurantrans@de@cmxiv{Und wen ALLAH rechtleiten will, dessen Brust weitet ER für den Islam, und wen ER abirren lassen will, dessen Brust macht ER eng, zusammengepreßt, als ob er in den Himmel aufsteigen würde. Solcherart läßt ALLAH das Üble auf diejenigen kommen, die den Iman nicht verinnerlichen.\qt@no{125}}%
\qt@newcmd\qurantrans@de@cmxv{Und dies ist der Weg deines HERRN, ein gerader Weg. Bereits haben WIR die Ayat verdeutlicht für Menschen, die sich besinnen.\qt@no{126}}%
\qt@newcmd\qurantrans@de@cmxvi{Für sie ist Darus-salam bei ihrem HERRN bestimmt, und ER ist ihr Wali wegen dem, was sie zu tun pflegten.\qt@no{127}}%
\qt@newcmd\qurantrans@de@cmxvii{Und an jenem Tag, wenn ER sie alle versammelt, (sagt ER): ``Ihr Gesellschaft von Dschinn! Ihr habt bereits viele von den Menschen verleitet." Und ihre Wali von den Menschen sagten: ``Unser HERR! Die einen von uns haben den anderen Dienste erwiesen, und wir haben unsere Frist erreicht, die DU uns gesetzt hast." ER sagte: ``Das Feuer ist eure Unterkunft, dort werdet ihr ewig bleiben, es sei denn, was ALLAH will. Gewiß, dein HERR ist allweise, allwissend.\qt@no{128}}%
\qt@newcmd\qurantrans@de@cmxviii{Und solcherart lassen WIR die Übertretenden einander Wali sein wegen dem, was sie sich zu erwerben pflegten.\qt@no{129}}%
\qt@newcmd\qurantrans@de@cmxix{Ihr Gesellschaft von Dschinn und Menschen! Sind von euch keine Gesandten zu euch gekommen, um euch Meine Ayat vorzutragen und euch vor dem Zusammentreffen mit diesem eurigen Tag zu warnen?! Sie sagen: ``Wir bezeugen es gegen uns selbst." Und das diesseitige Leben hat sie getäuscht und sie haben gegen sich selbst bezeugt, daß sie doch Kafir waren.\qt@no{130}}%
\qt@newcmd\qurantrans@de@cmxx{Dies weil dein HERR niemals die Ortschaften wegen begangenem Unrecht vernichten wird, solange ihre Einwohner noch nicht ermahnt wurden.\qt@no{131}}%
\qt@newcmd\qurantrans@de@cmxxi{Und allen gebühren Stellungen für das, was sie taten. Und dein HERR ist gewiß nicht achtlos dem gegenüber, was sie tun.\qt@no{132}}%
\qt@newcmd\qurantrans@de@cmxxii{Und dein HERR ist Der absolut Autarke, Der Eigentümer der Gnade. Wenn ER wollte, hätte ER euch vergehen und nach euch andere als Nachfolger einsetzen lassen, die ER will, wie ER euch aus einer Nachkommenschaft anderer Menschen hervorgebracht hat.\qt@no{133}}%
\qt@newcmd\qurantrans@de@cmxxiii{Gewiß, was euch angedroht wird, wird doch eintreffen! Und ihr könnt (Uns) nicht zu schaffen machen.\qt@no{134}}%
\qt@newcmd\qurantrans@de@cmxxiv{Sag: ``Meine Leute! Handelt euren Standpunkten entsprechend, ich handle auch. Und ihr werdet noch wissen, wem das Anschließende vom Jenseits gehört." Die Unrecht-Begehenden werden gewiß keinen Erfolg haben.\qt@no{135}}%
\qt@newcmd\qurantrans@de@cmxxv{Und sie gesellten ALLAH einen Teil von dem bei, was ER an Äckern und An'am erschuf, und sagten: ``Dies gehört ALLAH - ihrer Behauptung nach - und dies gehört den von uns beigesellten Partnern." Und was den von ihnen beigesellten Partnern gehört, erreicht ALLAH nicht, aber was ALLAH gehört, dies erreicht die von ihnen beigesellten Partner. Erbärmlich ist das, wie sie richten.\qt@no{136}}%
\qt@newcmd\qurantrans@de@cmxxvi{Und solcherart haben ihre (von ihnen beigesellten) Partner vielen von den Muschrik das Töten ihrer Kinder schön erscheinen lassen, damit sie sie zugrunde richten können und ihnen ihren Din durcheinander bringen. Und hätte ALLAH es gewollt, hätten sie es nicht tun können. Also laß von ihnen und von dem ab, was sie erlügen.\qt@no{137}}%
\qt@newcmd\qurantrans@de@cmxxvii{Und sie sagten: ``Diese sind An'am und Äcker, die vorenthalten sind, von denen nur diejenigen speisen dürfen, denen wir dies erlauben - ihrer Behauptung nach. Auch diese sind An'am, deren Rücken für haram erklärt wurden, ebenfalls An'am, bei (deren Schächtung) ALLAHs Name nicht erwähnt wird - aus Erlogenheit in Seinem Namen." ER wird ihnen noch das vergelten, was sie zu erlügen pflegten.\qt@no{138}}%
\qt@newcmd\qurantrans@de@cmxxviii{Auch sagten sie: ``Alles in den Bäuchen dieser An'am ist ausschließlich unseren Männlichen bestimmt, aber es gilt als haram für unsere Partnerwesen, und sollte es als Verendetes (geboren werden), dann sind alle darin Partner." ER wird es ihnen für ihre Beschreibung vergelten. ER ist gewiß allweise, allwissend.\qt@no{139}}%
\qt@newcmd\qurantrans@de@cmxxix{Bereits verloren haben diejenigen, die ihre Kinder aus Torheit und Unwissen getötet und das vom Rizq für haram erklärt haben, das ALLAH ihnen gewährte, aus Erlogenheit in ALLAHs Namen. Bereits sind sie irregegangen und sie waren nicht rechtgeleitet.\qt@no{140}}%
\qt@newcmd\qurantrans@de@cmxxx{Und ER ist Derjenige, Der Dschannat mit und ohne Pfahlwerk hervorbrachte und die Dattelpalmen, die Pflanzen verschiedener Art, die Oliven und die Granatäpfel - einander ähnelnd und nicht ähnelnd. Esst von ihren Früchten, wenn sie Früchte tragen, doch gebt ihren Pflichtanteil am Tage der Ernte! Und seid nicht verschwenderisch, ER liebt gewiß nicht die Maßlosen.\qt@no{141}}%
\qt@newcmd\qurantrans@de@cmxxxi{Auch (brachte ER hervor) von den An'am Trag- und Schlachttiere. Esst von dem Rizq, das ALLAH euch gewährte und folgt nicht den Fußstapfen des Satans, gewiß, er ist für euch ein entschiedener Feind.\qt@no{142}}%
\qt@newcmd\qurantrans@de@cmxxxii{Ebenfalls (brachte ER hervor) acht Zweiheiten, von den Schafen zwei und von den Ziegen zwei (Zweiheiten). Sag: ``Erklärte ER etwa für haram beide Männchen oder beide Weibchen, oder das, was in den Gebärmüttern beider Weibchen enthalten ist? Setzt mich darüber in Kenntnis durch Wissen, solltet ihr wahrhaftig sein."\qt@no{143}}%
\qt@newcmd\qurantrans@de@cmxxxiii{Und (ER brachte hervor) von den Kamelen zwei und von den Kühen zwei (Zweiheiten). Sag: ``Erklärte ER etwa für haram beide Männchen oder beide Weibchen, oder das, was in den Gebärmüttern beider Weibchen enthalten ist? Oder waret ihr anwesend, als ER euch dies auferlegte?" Also keiner begeht mehr Unrecht als derjenige, der im Namen ALLAHs Lügen erfindet, um die Menschen, ohne über Wissen zu verfügen, irrezuführen. Gewiß, ALLAH leitet die unrecht-begehenden Leute nicht recht.\qt@no{144}}%
\qt@newcmd\qurantrans@de@cmxxxiv{Sag: ``Ich finde in dem, was mir an Wahy zuteil wurde, nichts als haram für den Speisenden, was er speist, außer wenn es Verendetes, vergossenes Blut oder Schweinefleisch ist - denn dies ist gewiß eine Unreinheit - oder Fisq, worauf (beim Schächten) anderes als ALLAHs Name ausgerufen wurde. Wer jedoch dazu gezwungen wurde, nicht aus Neigung dazu oder Übertretung, so ist dein HERR gewiß allvergebend, allgnädig.\qt@no{145}}%
\qt@newcmd\qurantrans@de@cmxxxv{Und denjenigen, die Juden wurden, erklärten WIR für haram alle (Tiere), die Krallen haben. Auch von den Kühen und den Schafen erklärten WIR ihnen für haram das Fett außer dem, was ihre Rücken und die Eingeweide tragen, und dem, was an den Knochen hängt. Dies haben WIR ihnen für ihre Übertretung vergolten. Und gewiß, WIR sind doch wahrhaftig.\qt@no{146}}%
\qt@newcmd\qurantrans@de@cmxxxvi{Und sollten sie dich des Lügens bezichtigen, dann sag: ``Euer HERR verfügt über allumfassende Gnade. Doch Seine Peinigung ist von den schwer verfehlenden Leuten nicht abhaltbar."\qt@no{147}}%
\qt@newcmd\qurantrans@de@cmxxxvii{Diejenigen, die Schirk betreiben, werden sagen: ``Hätte ALLAH es gewollt, hätten weder wir noch unsere Väter Schirk betrieben, auch hätten wir nichts (ohne Erlaubnis) für haram erklärt." Solcherart leugneten schon diejenigen vor ihnen, bis sie Unsere Peinigung erfuhren. Sag: ``Habt ihr irgendein Wissen darüber, das ihr uns offenlegt?" Doch ihr folgt nur dem Spekulieren, und ihr lügt ja nur.\qt@no{148}}%
\qt@newcmd\qurantrans@de@cmxxxviii{Sag: ``ALLAH verfügt über das eindeutige Argument, hätte ER es gewollt, so hätte ER euch allesamt doch rechtgeleitet."\qt@no{149}}%
\qt@newcmd\qurantrans@de@cmxxxix{Sag: ``Bringt eure Zeugen herbei, die bezeugen, daß ALLAH dies sicher für haram erklärte." Und sollten sie ein Zeugnis ablegen, so bezeuge es nicht mit ihnen, und folge nicht den Neigungen derjenigen, die Unsere Ayat verleugnen, und derjenigen, die den Iman an das Jenseits nicht verinnerlichen. Und diese setzen ihrem HERRN etwas Ebenbürtiges bei.\qt@no{150}}%
\qt@newcmd\qurantrans@de@cmxl{Sag: ``Kommt, damit ich euch vortrage, was euer HERR euch für haram erklärt hat, nämlich daß ihr Ihm gegenüber keinerlei Schirk betreibt, die Eltern Ihsan gemäß behandelt und eure Kinder nicht aufgrund von Bedürftigkeit tötet - denn WIR gewähren euch und ihnen Rizq, und daß ihr euch den Verwerflichkeiten nicht nähert - weder dem Offenkundigen davon noch dem Verborgenen, und daß ihr den Menschen, den ALLAH für haram erklärte, nicht tötet, es sei denn gemäß des Rechts. Dies ist, was ER euch geboten hat, damit ihr nachsinnt.\qt@no{151}}%
\qt@newcmd\qurantrans@de@cmxli{Auch nähert euch nicht dem Vermögen des Waisenkindes außer auf die bessere Art, bis es mündig wird, und gebt vollständigesMaß und Gewicht nach Gerechtigkeit - WIR erlegen keinem Menschen etwas auf außer dem, was er vermag. Und wenn ihr sprecht, dann seid gerecht, selbst dann, wenn es um einen Verwandten geht, und haltet das von ALLAH Auferlegte ein! Dies ist, was ER euch geboten hat, damit ihr euch entsinnt.\qt@no{152}}%
\qt@newcmd\qurantrans@de@cmxlii{Und das ist mein Weg - ein Gerader, so folgt ihm und folgt nicht den (anderen) Wegen, damit sie euch von Seinem Weg nicht abbringen. Dies ist, was ER euch geboten hat, damit ihr Taqwa gemäß handelt.\qt@no{153}}%
\qt@newcmd\qurantrans@de@cmxliii{Dann haben WIR Musa die Schrift zuteil werden lassen - als Vervollkommnung für das, was er an Gutem tat, als Erläuterung für jedes Ding, als Recht-leitung und als Gnade, damit sie den Iman an das Zusammentreffen mit ihrem HERRN verinnerlichen.\qt@no{154}}%
\qt@newcmd\qurantrans@de@cmxliv{Und dies ist eine Schrift, die WIR hinabsandten  voller Baraka, so folgt ihr und handelt Taqwa gemäß, damit euch Gnade erwiesen wird. -\qt@no{155}}%
\qt@newcmd\qurantrans@de@cmxlv{damit ihr nicht sagt: ``Die Schrift wurde nur zu zwei Gruppen vor uns hinabgesandt und wir waren deren Studium gegenüber achtlos",\qt@no{156}}%
\qt@newcmd\qurantrans@de@cmxlvi{oder ihr nicht sagt: ``Wäre doch zu uns die Schrift hinabgesandt worden, wären wir mit Sicherheit rechtgeleiteter gewesen als sie!" Euch ist bereits ein klares Zeichen von eurem HERRN, Rechtleitung und Gnade gekommen. Also keiner begeht mehr Unrecht als derjenige, der die Ayat ALLAHs verleugnet und sich davon abwendet. WIR werden denjenigen, die sich von Unseren Ayat abwenden, mit der Härte Unserer Peinigung vergelten wegen dem, wie sie sich abzuwenden pflegten.\qt@no{157}}%
\qt@newcmd\qurantrans@de@cmxlvii{Warten sie auf etwas anderes, außer daß die Engel zu ihnen kommen oder daß (die Bestimmung) deines HERRN kommt oder daß einige Ayat deines HERRN kommen?! An dem Tag, an dem einige Ayat deines HERRN eintreffen, wird keiner Seele ihr Iman nützen, wenn sie vorher den Iman nicht verinnerlichte oder mit ihrem Iman gottgefällig Gutes erwarb. Sag: ``Wartet nur! Gewiß, wir warten auch."\qt@no{158}}%
\qt@newcmd\qurantrans@de@cmxlviii{Mit denjenigen, die ihren Din zersplittert haben und zu Gruppen wurden, hast du mitnichten zu tun. Ihre Angelegenheit unterliegt nur ALLAH, dann wird ER ihnen das kundtun, was sie zu tun pflegten.\qt@no{159}}%
\qt@newcmd\qurantrans@de@cmxlix{Wer ein Einziges Gutes vollbringt, bekommt dafür das Zehnfache (an Belohnung), doch wer die Schlechtigkeit vollbringt, dem wird dafür mit Gleichem vergolten. Und ihnen wird kein Unrecht angetan.\qt@no{160}}%
\qt@newcmd\qurantrans@de@cml{Sag: ``Gewiß, mich leitete mein HERR recht auf einen geraden Weg  auf einen geradlinigen Din, auf die Gemeinschaft von Ibrahim, der hanif war. Und er gehörte nicht zu den Muschrik."\qt@no{161}}%
\qt@newcmd\qurantrans@de@cmli{Sag: ``Gewiß, mein rituelles Gebet, mein Opfern, mein Leben und mein Tod gehören ALLAH, Dem HERRN aller Schöpfung.\qt@no{162}}%
\qt@newcmd\qurantrans@de@cmlii{Es gibt keinen Partner für Ihn. Und dies wurde mir geboten, und ich bin der erste der Muslime."\qt@no{163}}%
\qt@newcmd\qurantrans@de@cmliii{Sag: ``Soll ich mir etwa einen anderen HERRN außer ALLAH nehmen, und ER ist Der HERR aller Dinge?!" Und keine Seele erwirbt etwas außer für sich selbst, und keine verfehlende Seele trägt die Verfehlung einer anderen. Dann wird eure Rückkehr zu eurem HERRN sein, dann wird ER euch das kundtun, worüber ihr uneins zu sein pflegtet.\qt@no{164}}%
\qt@newcmd\qurantrans@de@cmliv{Und ER ist Derjenige, Der euch als Nachfolger auf der Erde einsetzte und manchen von euch Vorrangstellung vor den anderen gewährt hat, damit ER euch darin prüft, was ER euch zuteil werden ließ. Gewiß, dein HERR ist schnell im Bestrafen. Und gewiß, ER ist doch allvergebend, allgnädig.\qt@no{165}}%
\qt@newcmd\qurantrans@de@cmlv{\basmalahde Alif-lam-mim-sad.\qt@no{1}}%
\qt@newcmd\qurantrans@de@cmlvi{Diese ist eine Schrift, die dir hinabgesandt wurde, so soll in deiner Brust kein Mißbehagen ihretwegen aufkommen, damit du mit ihr ermahnst. Auch ist es eine Erinnerung für die Mumin.\qt@no{2}}%
\qt@newcmd\qurantrans@de@cmlvii{Folgt dem, was zu euch von eurem HERRN hinabgesandt wurde, und folgt nicht anstelle von Ihm anderen Wali. Nur ein wenig entsinnt ihr euch.\qt@no{3}}%
\qt@newcmd\qurantrans@de@cmlviii{Und wie viele Ortschaften haben WIR doch zugrunde gehen lassen, so überraschte Unsere Peinigung sie entweder nachts, oder als sie sich bei der Mittagsruhe befanden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@cmlix{Dann hatten sie keine anderen Rufe, als Unsere Peinigung sie überraschte, außer daß sie sagten: ``Gewiß, wir waren Unrecht-Begehende."\qt@no{5}}%
\qt@newcmd\qurantrans@de@cmlx{So werden WIR diejenigen, zu denen Gesandte entsandt wurden, doch zur Rechenschaft ziehen, ebenfalls werden WIR die Gesandten gewiß befragen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@cmlxi{Zweifelsohne werden WIR sie auch mit Wissen (über ihre Taten) unterrichten. Und WIR waren doch nie abwesend.\qt@no{7}}%
\qt@newcmd\qurantrans@de@cmlxii{Und das Wägen (der Taten) an jenem Tag wird wahrhaftig sein. Diejenigen, deren Gewogenes dann schwer wiegt, diese sind die wirklichen Erfolgreichen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@cmlxiii{Doch diejenigen, deren Gewogenes leicht wiegt, diese sind diejenigen, die sich selbst verloren haben wegen dem, was sie Unseren Ayat gegenüber an Unrecht zu begehen pflegten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@cmlxiv{Und gewiß, bereits haben WIR euch über die Erde verfügen lassen und euch darauf Existenzgrundlage gewährt. Doch ihr erweist euch selten dankbar.\qt@no{10}}%
\qt@newcmd\qurantrans@de@cmlxv{Und gewiß, bereits erschufen WIR euch, dann formten WIR euch, dann sagten WIR den Engeln: ``Vollzieht Sudschud vor Adam!" So vollzogen sie (alle) Sudschud, außer Iblis, er war keiner der Sudschud-Vollziehenden.\qt@no{11}}%
\qt@newcmd\qurantrans@de@cmlxvi{ER sagte: ``Was hinderte dich daran, Sudschud zu vollziehen, als ICH es dir angewiesen habe?!" Er sagte: ``Ich besser bin als er. DU hast mich aus Feuer geschaffen, aber ihn hast DU aus Lehm geschaffen."\qt@no{12}}%
\qt@newcmd\qurantrans@de@cmlxvii{ER sagte: ``Dann steige davon hinab, denn du darfst dich darin nicht überheblich verhalten, gehe hinaus! Gewiß, du bist einer der Erniedrigten."\qt@no{13}}%
\qt@newcmd\qurantrans@de@cmlxviii{Er sagte: ``Gewähre mir Zeit bis zu dem Tag, an dem sie wieder erweckt werden."\qt@no{14}}%
\qt@newcmd\qurantrans@de@cmlxix{ER sagte: ``Du bist einer derjenigen, denen Zeit gewährt wird."\qt@no{15}}%
\qt@newcmd\qurantrans@de@cmlxx{Er sagte: ``Und weil DU mich (durch diese Prüfung) hast abirren lassen, werde ich ihnen auf Deinem geradenWeg auflauern,\qt@no{16}}%
\qt@newcmd\qurantrans@de@cmlxxi{dann werde ich ihnen von vorne, von hinten, von ihrer Rechten und von ihrer Linken entgegenkommen. Dann wirst DU die meisten von ihnen als Undankbare finden."\qt@no{17}}%
\qt@newcmd\qurantrans@de@cmlxxii{ER sagte: ``Verlasse es als ein Verachteter, Verfluchter. Wer von ihnen dir folgt - gewiß werde ICH Dschahannam mit euch allesamt füllen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@cmlxxiii{Und Adam! bewohne du mit deiner Frau die Dschanna und speist, wovon immer ihr wollt, aber nähert euch nicht diesem Baum, sonst wäret ihr von den Unrecht-Begehenden."\qt@no{19}}%
\qt@newcmd\qurantrans@de@cmlxxiv{Dann flüsterte ihnen der Satan ein, damit er ihnen das enthüllt, was ihnen von ihren Schamteilen verborgen war, und sagte: ``Euer HERR hat euch diesen Baum nur deswegen verboten, damit ihr keine Engel werdet oder zu den Ewig-Lebenden gehört!"\qt@no{20}}%
\qt@newcmd\qurantrans@de@cmlxxv{Auch schwur er ihnen: ``Gewiß, ich bin für euch doch von den Ratgebern!"\qt@no{21}}%
\qt@newcmd\qurantrans@de@cmlxxvi{So betrog er sie mit Täuschung. Als sie dann vom Baum gekostet haben, wurden ihnen ihre Schamteile sichtbar, und sie begannen eilends, sich mit den Blättern der Dschanna zu bedecken. Und ihr HERR hat sie rufen lassen: ``Habe ICH euch etwa nicht diesen Baum verboten und gesagt, daß der Satan für euch gewiß ein entschiedener Feind ist?!"\qt@no{22}}%
\qt@newcmd\qurantrans@de@cmlxxvii{Sie sagten: ``Unser HERR! Wir begingen Unrecht und wenn DU uns nicht vergibst und Gnade erweist, werden wir doch zu den Verlierern gehören."\qt@no{23}}%
\qt@newcmd\qurantrans@de@cmlxxviii{ER sagte: ``Steigt hinab! Die einen von euch (Menschen) sind den anderen (Satanen) Feind. Und auf der Erde habt ihr einen Aufenthaltsort und Nießbrauch bis zu einer bestimmten Zeit."\qt@no{24}}%
\qt@newcmd\qurantrans@de@cmlxxix{ER sagte: ``Darauf werdet ihr leben und darauf werdet ihr sterben und daraus werdet ihr hervorgebracht."\qt@no{25}}%
\qt@newcmd\qurantrans@de@cmlxxx{Kinder Adams! Bereits haben WIR euch Kleidung hinabgesandt, die eure Schamteile bedeckt, auch schmückende Kleidung. Doch die Kleidung der Taqwa ist besser. Dies ist von ALLAHs Ayat, damit ihr euch entsinnt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@cmlxxxi{Kinder Adams! Lasst den Satan euch nicht durch Fitna verführen, wie er eure Eltern aus der Dschanna hinausbrachte. Er hat ihnen ihre Kleidung ausgezogen, damit er ihnen ihre Schamteile sichtbar macht. Gewiß, er sieht euch - er mit seiner Sippe, während ihr sie nicht seht. Gewiß, WIR haben die Satane als Wali für diejenigen eingesetzt, die nicht den Iman verinnerlichen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@cmlxxxii{Und wenn sie eine Abscheulichkeit getan haben, sagten sie: ``Wir fanden unsere Ahnen diese praktizieren und ALLAH gebot sie uns." Sag: ``Gewiß, ALLAH gebietet keine Abscheulichkeit!" Wollt ihr von ALLAH etwas behaupten, was ihr nicht wisst?!\qt@no{28}}%
\qt@newcmd\qurantrans@de@cmlxxxiii{Sag: ``Mein HERR gebietet Gerechtigkeit. Und richtet euch zu Ihm bei jedem Sudschud und richtet Bittgebete an Ihn in lauterem Din. Wie ER euch das erste Mal erschuf, so werdet ihr nochmals zurückkehren."\qt@no{29}}%
\qt@newcmd\qurantrans@de@cmlxxxiv{Eine Gruppe leitete ER recht und eine Gruppe ließ ER mit Recht abirren, gewiß, diese haben die Satane als Wali anstelle von ALLAH und sie glauben, sie wären rechtgeleitet.\qt@no{30}}%
\qt@newcmd\qurantrans@de@cmlxxxv{Kinder Adams! Bekleidet euch gepflegt beim (Besuch) jeder Moschee, esst und trinkt, doch seid nicht verschwenderisch! Gewiß, ER liebt nicht die Verschwender.\qt@no{31}}%
\qt@newcmd\qurantrans@de@cmlxxxvi{Sag: ``Wer erklärte für haram die gepflegte Kleidung, die ALLAH für Seine Diener hervorbrachte, und die Tay-yibat vom Rizq?" Sag: ``Diese sind für die Mumin im diesseitigen Leben bestimmt und am Tag der Auferstehung ausschließlich ihnen vorbehalten." Solcherart verdeutlichen WIR die Ayat für Menschen, die über Wissen verfügen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@cmlxxxvii{Sag: ``Mein HERR hat nur für haram erklärt die Verwerflichkeiten, - alles, was davon offenkundig wird oder verborgen bleibt, - die bewußte Verfehlung, die Aggressivität zu Unrecht, daß ihr ALLAH gegenüber Schirk betreibt, wozu ER niemals eine Bestätigung hinabsandte, und daß ihr über ALLAH das sagt, was ihr nicht wisst."\qt@no{33}}%
\qt@newcmd\qurantrans@de@cmlxxxviii{Jede Umma unterliegt einer festgelegten Frist. Und wenn ihre Frist sich dem Ende nähert, können sie (diese) weder um eine Stunde hinauszögern noch vorverlegen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@cmlxxxix{Kinder Adams! Wenn zu euch Gesandte von euch kommen, die euch Meine Ayat vortragen, wer dann Taqwa gemäß handelt und gottgefällig Gutes tut, so gibt es um diese weder Angst noch werden sie traurig sein.\qt@no{35}}%
\qt@newcmd\qurantrans@de@cmxc{Doch diejenigen, die Unsere Ayat leugnen und sich darüber in Arroganz erheben, diese sind die Weggenossen des Feuers. Darin werden sie ewig bleiben.\qt@no{36}}%
\qt@newcmd\qurantrans@de@cmxci{Und keiner begeht mehr Unrecht als derjenige, der Lügen im Namen ALLAHs erfindet oder Seine Ayat ableugnet. Diese wird ihr Anteil von dem (für sie) Bestimmten erreichen. Wenn dann Unsere Gesandten (Engel) zu ihnen kommen, um (ihre Seelen) einzufordern, sagen sie (die Engel): ``Wo ist dies, woran ihr anstelle von ALLAH Bittgebete gerichtet habt?" Sie sagen: ``Sie sind uns abhanden gekommen." Und sie haben gegen sich selbst bezeugt, daß sie doch Kafir waren.\qt@no{37}}%
\qt@newcmd\qurantrans@de@cmxcii{ER sagte: ``Tretet ein ins Feuer mit Umam von Menschen und Dschinn, die vor euch vergingen." Immer wieder, wenn eine Umma eingetreten ist, hat sie ihre Vorgängerin verflucht. Und als sie sich darin (im Feuer), allesamt versammelten, sagte die Letzte (Umma) unter ihnen zur Ersten von ihnen: ``Unser HERR! Diese haben uns irregeleitet, so laß ihnen das Doppelte an Peinigung des Feuers zuteil werden!" ER sagte: ``Für jede das Doppelte." Doch ihr wisst nichts.\qt@no{38}}%
\qt@newcmd\qurantrans@de@cmxciii{Und die Erste (Umma) von ihnen sagte zur Letzten: ``Ihr seid uns gegenüber nicht besser gestellt, so erfahrt die Peinigung wegen dem, was ihr euch zu erwerben pflegtet."\qt@no{39}}%
\qt@newcmd\qurantrans@de@cmxciv{Gewiß, denjenigen, die Unsere Ayat leugneten und sich ihnen gegenüber in Arroganz erhoben, werden die Tore des Himmels nicht geöffnet und sie werden in die Dschanna nicht eintreten, bis das Kamel in das Nadelöhr hineingeht. Und solcherart vergelten WIR es den schwer Verfehlenden.\qt@no{40}}%
\qt@newcmd\qurantrans@de@cmxcv{Für sie wird aus Dschahannam eine Unterlage sein, und über ihnen eine Decke. Und solcherart vergelten WIR es den Unrecht-Begehenden.\qt@no{41}}%
\qt@newcmd\qurantrans@de@cmxcvi{Doch denjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, WIR gebieten einer Seele nur das, was sie vermag. Diese sind die Weggenossen der Dschanna. Darin werden sie ewig bleiben.\qt@no{42}}%
\qt@newcmd\qurantrans@de@cmxcvii{WIR haben aus ihren Brüsten allen Groll herausgenommen - unter ihnen fließen Flüsse, und sie sagten: ``Alhamdulillah: Alles Lob gebührt ALLAH, Der uns dazu rechtleitete, und WIR wären gewiß nicht rechtgeleitet, hätte ALLAH uns nicht rechtgeleitet. Gewiß, bereits sind die Gesandten unseres HERRN mit der Wahrheit gekommen." Und ihnen wurde zugerufen: ``Dies ist die Dschanna, euch wird er für das überlassen, was ihr zu tun pflegtet."\qt@no{43}}%
\qt@newcmd\qurantrans@de@cmxcviii{Und die Weggenossen der Dschanna haben den Weggenossen des Feuers zugerufen: ``Wir haben das, was unser HERR uns versprochen hat, bereits als Wahrhaftiges vorgefunden. Habt ihr etwa das, was euer HERR versprochen hat, als Wahrhaftiges vorgefunden?" Sie sagten: ``Ja!  Dann rief ein Ausrufender unter ihnen aus: ``ALLAHs Fluch laste auf den Unrecht-Begehenden! \qt@no{44}}%
\qt@newcmd\qurantrans@de@cmxcix{Diejenigen, die von ALLAHs Weg abhalten und diesen zu krümmen versuchen, und die dem Jenseits gegenüber Kufr betreiben.\qt@no{45}}%
\qt@newcmd\qurantrans@de@m{Und zwischen ihnen gibt es eine Abtrennung. Auch auf Al-a'raf sindMänner, die jeden an seiner Miene erkennen. Und sie haben den Weggenossen der Dschanna zugerufen: ``Salam sei mit euch." Sie selbst jedoch sind nicht hineingetreten, aber sie wünschen es sich sehr.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mi{Und wenn ihre Blicke auf die Weggenossen des Feuers gerichtet werden, sagten sie: ``Unser HERR! Führe uns nicht mit den unrecht-begehenden Leuten zusammen!"\qt@no{47}}%
\qt@newcmd\qurantrans@de@mii{Die Zugehörigen von Al-'araf haben dann (Kafir-)Männern, die sie an ihrer Miene erkannten, zugerufen, sie sagten (ihnen): ``Was hat euch eure Menge und das, was ihr an Überheblichkeit praktiziert habt, genützt?\qt@no{48}}%
\qt@newcmd\qurantrans@de@miii{Sind diese (Mumin) etwa diejenigen, von denen ihr geschworen habt, daß ALLAH ihnen keinerlei Gnade erweisen würde?! (Ihr, Mumin!) Tretet in die Dschanna ein und um euch gibt es weder Angst, noch werdet ihr traurig sein."\qt@no{49}}%
\qt@newcmd\qurantrans@de@miv{Und die Weggenossen des Feuers riefen den Weggenossen der Dschanna zu: ``Gebt uns etwas vom Wasser oder vom Rizq, das ALLAH euch gewährte." Sie sagten: ``Gewiß, ALLAH erklärte beides den Kafir für haram."\qt@no{50}}%
\qt@newcmd\qurantrans@de@mv{Diejenigen, die ihren Din als Zeitvertreib und als Spielchen nahmen und vom diesseitigen Leben getäuscht wurden. An diesem Tag lassen WIR sie (im Feuer verweilen) wie Vergessene, genauso wie sie die Begegnung mit diesem ihrem Tag vergessen haben und wegen dem, wie sie Unsere Ayat zu leugnen pflegten.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mvi{Und gewiß, bereits haben WIR ihnen doch eine Schrift zukommen lassen, die WIR mit Wissen verdeutlicht haben - als Rechtleitung und Gnade für Menschen, die den Iman verinnerlichen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mvii{Warten sie etwa auf etwas anderes außer auf die in ihr (angedrohten) Konsequenzen?! An dem Tage, wenn die in ihr (angedrohten) Konsequenzen eintreten, werden diejenigen, die sie vorher vergessen haben, sagen: ``Bereits kamen die Gesandten unseres HERRN doch mit der Wahrheit. Hätten wir doch Fürsprecher, die für uns fürsprechen, und würden wir doch (ins Diesseits) zurückgeschickt, damit wir anders handeln als das, was wir vorher zu tun pflegten." Doch bereits haben sie sich selbst verloren und ihnen ist abhanden gekommen, was sie zu erlügen pflegten.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mviii{Gewiß, euer HERR ist ALLAH, Der die Himmel und die Erde in sechs Ayyam erschuf, dann wandte ER sich dem Al'ahrsch zu. ER läßt die Nacht den Tag überdecken - sie bleibt ihm prompt auf den Fersen. Auch die Sonne, der Mond und die Sterne sind Seinem Gebot unterworfen. Ja! Ihm alleine gehören die Schöpfung und das Gebieten. Immer allerhabener ist ALLAH, Der HERR aller Schöpfung.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mix{Richtet Bittgebete an euren HERRN in Ergebenheit und im Verborgenen! Gewiß, ER liebt nicht die Übertretenden.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mx{Und richtet kein Verderben auf der Erde an, nachdem sie gut gemacht wurde, und richtet Bittgebete an Ihn in Furcht (vor Seiner Peinigung) und in Ersehnung (Seiner Belohnung)! Gewiß, ALLAHs Gnade liegt den Muhsin nahe.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mxi{Und ER ist Derjenige, Der die Winde als Überbringer froher Botschaft vor Seiner Gnade vorausschickt. Wenn diese dann dicke Regenwolken trugen, lassen WIR ihn (den Regen) zu einer toten Landschaft treiben, dann lassen WIR darauf das Wasser herab. Und damit lassen WIR von den verschiedenen Früchten sprießen. Solcherart erwecken WIR die Toten, damit ihr euch entsinnt.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mxii{Und die gute Landschaft läßt ihre Pflanzen mit der Zustimmung ihres HERRN sprießen. Doch diejenige (Landschaft), die karg ist, läßt (ihre Pflanzen) nur spärlich sprießen. Solcherart verdeutlichen WIR die Ayat für Menschen, die sich dankbar erweisen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mxiii{Gewiß, bereits haben WIR Nuh zu seinen Leuten entsandt, dann sagte er: ``Meine Leute! Dient ALLAH! Ihr habt keinen Gott außer Ihm. Ich fürchte für euch die Peinigung eines gewaltigen Tages."\qt@no{59}}%
\qt@newcmd\qurantrans@de@mxiv{Die Entscheidungsträger seiner Leute sagten: ``Gewiß, wir sehen dich doch im offenkundigen Irrtum."\qt@no{60}}%
\qt@newcmd\qurantrans@de@mxv{Er sagte: ``Meine Leute! Ich bin nicht im Irrtum, sondern ich bin ein Gesandter Des HERRN aller Schöpfung.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mxvi{Ich verkünde euch die Botschaft meines HERRN, rate euch und weiß von ALLAH, was ihr nicht wisst."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mxvii{Verwundert euch etwa, daß euch eine Ermahnung von eurem HERRN zuteil wurde durch einen Mann von euch, um euch zu warnen, damit ihr Taqwa gemäß handelt und damit euch Gnade erwiesen wird?!\qt@no{63}}%
\qt@newcmd\qurantrans@de@mxviii{So haben sie ihn der Lüge bezichtigt, dann haben WIR ihn und diejenigen, die mit ihm waren, in der Arche gerettet, und haben diejenigen, die Unsere Ayat leugneten, ertrinken lassen. Sie waren ja verblendete Menschen.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mxix{Und zu 'Aad (entsandten) WIR ihren Bruder Hud. Er sagte: ``Meine Leute! Dient ALLAH! Ihr habt keinen Gott außer Ihm. Wollt ihr etwa nicht Taqwa gemäß handeln?!"\qt@no{65}}%
\qt@newcmd\qurantrans@de@mxx{Die Entscheidungsträger seiner Leute, die Kufr betrieben, sagten: ``Gewiß, wir sehen dich doch in Sinnesverwirrung und gewiß, wir glauben doch, daß du zu den Lügnern gehörst."\qt@no{66}}%
\qt@newcmd\qurantrans@de@mxxi{Er sagte: ``Meine Leute! Ich leide nicht unter Sinnesverwirrung, sondern ich bin ein Gesandter Des HERRN aller Schöpfung.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mxxii{Ich verkünde euch die Botschaft meines HERRN und ich bin für euch ein treuer Ratgeber.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mxxiii{Verwundert ihr euch etwa, daß euch eine Ermahnung von eurem HERRN zuteil wurde durch einen Mann von euch, um euch zu warnen?! Und entsinnt euch, als ER euch zu Nachfolgern von Nuh-Leuten einsetzte und eure Körpergestalt noch kräftiger machte. So entsinnt euch der Wohltaten ALLAHs, damit ihr erfolgreich werdet."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mxxiv{Sie sagten: ``Bist du etwa zu uns gekommen, damit wir ALLAH alleine dienen und von dem ablassen, dem unsere Ahnen dienten?! Also lasse uns das zuteil werden, wovor du uns warnst, solltest du von den Wahrhaftigen sein."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mxxv{Er sagte: ``Bereits traf euch doch (dafür) von eurem HERRN Mißfallen und Zorn. Wollt ihr etwa mit mir über Namen (von Götzen) disputieren, die ihr und eure Ahnen (ihnen) gegeben habt, wozu ALLAH niemals eine Bestätigung hinabsandte?! So wartet nur ab! Ich warte ebenfalls mit euch ab."\qt@no{71}}%
\qt@newcmd\qurantrans@de@mxxvi{Dann haben WIR ihn und diejenigen mit ihm mit Gnade von Uns gerettet und diejenigen, die Unsere Ayat geleugnet haben, ausgerottet. Und sie waren keine Mumin.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mxxvii{Und zu Thamud (entsandten) WIR ihren Bruder Salih. Er sagte: ``Meine Leute! Dient ALLAH! Ihr habt keinen Gott außer Ihm. Bereits kam zu euch ein klares Zeichen von eurem HERRN: Dies ist die Kamelstute von ALLAH als Aya für euch, so lasst sie auf dem Lande ALLAHs weiden und fügt ihr keinen Schaden zu, sonst wird euch eine qualvolle Peinigung überkommen.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mxxviii{Auch entsinnt euch, als ER euch zu Nachfolgern von 'Aad einsetzte und euch im Lande wohnen ließ. Ihr habt auf ihren Auen Paläste gebaut und in die Felsengebirge Häuser gemeißelt. So entsinnt euch der Wohltaten ALLAHs und übertreibt nicht auf der Erde als Verderben-Anrichtende!"\qt@no{74}}%
\qt@newcmd\qurantrans@de@mxxix{Die Entscheidungsträger seiner Leute, die sich in Arroganz erhoben haben, sagten zu denjenigen, die unterdrückt wurden - zu denen von ihnen, die den Iman verinnerlicht haben: ``Wisst ihr etwa, daß Salih sicher ein Gesandter seines HERRN ist?" Sie sagten: ``Gewiß, wir verinnerlichen den Iman an das, womit er entsandt wurde."\qt@no{75}}%
\qt@newcmd\qurantrans@de@mxxx{Diejenigen, die sich in Arroganz erhoben haben, sagten: ``Gewiß, wir betreiben Kufr dem gegenüber, woran ihr den Iman verinnerlicht habt."\qt@no{76}}%
\qt@newcmd\qurantrans@de@mxxxi{Dann verletzten sie die Beine des Kamelweibchens, setzten sich über das Gebot ihres HERRN hinweg und sagten: ``Salih! Lasse uns nun zuteil werden, wovor du uns gewarnt hast, solltest du zu den Gesandten gehören."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mxxxii{Dann ergriff sie das Beben, dann wurden sie in ihren Wohnstätten zu leblosen Körpern.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mxxxiii{Dann hat er (Salih) sich von ihnen abgewendet und sagte: ``Meine Leute! Gewiß, bereits habe ich euch die Botschaft meines HERRN verkündet und euch Ratschläge erteilt, doch ihr mögt keine Ratgeber."\qt@no{79}}%
\qt@newcmd\qurantrans@de@mxxxiv{Und (erinnere an) Lut, als er zu seinen Leuten sagte: ``Begeht ihr das Abscheuliche, das keiner von den Menschen vor euch je beging?!\qt@no{80}}%
\qt@newcmd\qurantrans@de@mxxxv{Gewiß, ihr verkehrt doch intim aus Begierde mit Männern anstatt mit (euren) Ehefrauen. Nein, sondern ihr seid ausschweifende Leute!"\qt@no{81}}%
\qt@newcmd\qurantrans@de@mxxxvi{Die Antwort seiner Leute war nichts anderes als daß sie sagten: ``Treibt sie aus eurer Stadt hinaus! Gewiß, sie sind Menschen, die sich rein halten wollen."\qt@no{82}}%
\qt@newcmd\qurantrans@de@mxxxvii{Dann erretteten WIR ihn und seine Familie außer seiner Ehefrau, sie gehörte zu den zugrundegehenden Zurückgebliebenen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mxxxviii{Und WIR ließen Peinigungs- Regen auf sie niedergehen. So siehe, wie das Anschließende von den schwer Verfehlenden war.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mxxxix{Und zu Madyan (entsandten) WIR ihren Bruder Schu'aib. Er sagte: ``Meine Leute! Dient ALLAH! Ihr habt keinen Gott außer Ihm. Bereits kam zu euch ein deutliches Zeichen von eurem HERRN. So gebt vollständiges Maß und Gewicht und gebt den Menschen das ihnen Zustehende nicht inkomplett. Auch richtet auf der Erde kein Verderben an, nachdem sie gut gemacht wurde. Dies ist besser für euch, wenn ihr Mumin seid.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mxl{Und lauert nicht auf jedem Weg auf, um (Menschen) einzuschüchtern und von ALLAHs Weg diejenigen abzuhalten, die den Iman verinnerlicht haben - erstrebt nach dem Krümmen (dieses Weges). Und entsinnt euch, als ihr noch wenige wart, dann ließ ER euch mehr werden. Und nehmt wahr, wie das Anschließende von den Verderben-Anrichtenden war.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mxli{Und wenn eine Gruppe von euch den Iman daran verinnerlicht hat, womit ich entsandt wurde, und eine (andere) Gruppe den Iman nicht verinnerlicht hat, dann übt euch in Geduld, bis ALLAH zwischen uns richtet. Und ER ist Der Beste der Richtenden."\qt@no{87}}%
\qt@newcmd\qurantrans@de@mxlii{Die Entscheidungsträger seiner Leute, die sich in Arroganz erhoben haben, sagten: ``Gewiß, wir werden dich, Schu'aib, und diejenigen, die den Iman mit dir verinnerlicht haben, aus unserem Ort hinaustreiben, oder ihr kehrt zurück zu unserer Gemeinschaft!" Er sagte: ``Selbst dann (wollt ihr dies tun), sollten wir demgegenüber abgeneigt sein?!\qt@no{88}}%
\qt@newcmd\qurantrans@de@mxliii{Bereits hätte wir im Namen ALLAHs Lügen erfunden, würden wir in eure Gemeinschaft zurückkehren, nachdem ALLAH uns daraus errettete. Und uns gebührt nicht zu ihr (eurer Gemeinschaft) zurückzukehren, es sei denn, ALLAH, unser HERR will dies. Unser HERR umfaßt alles mit Wissen. (Nur) ALLAH gegenüber üben wir Tawakkul. Unser HERR! Decke (dies) zwischen uns und unseren Leuten mit der Wahrheit auf! Denn DU bist gewiß Der Beste der Aufdeckenden."\qt@no{89}}%
\qt@newcmd\qurantrans@de@mxliv{Die Entscheidungsträger seiner Leute, die Kufr betrieben haben, sagten: ``Solltet ihr Schu'aib folgen, gewiß wäret ihr dann doch Verlierer. \qt@no{90}}%
\qt@newcmd\qurantrans@de@mxlv{Dann ergriff sie das Beben und sie wurden in ihrem Wohnort zu leblosen Körpern.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mxlvi{Diejenigen, die Schu'aib der Lüge bezichtigt haben, als hätten sie dort nicht gewohnt. Diejenigen, die Schu'aib der Lüge bezichtigt haben, sie waren gewiß die wirklichen Verlierer.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mxlvii{Dann hat er (Schu'aib) sich von ihnen abgewendet und sagte: ``Meine Leute! Gewiß, bereits habe ich euch die Botschaft meines HERRN verkündet und euch Ratschläge gemacht, also weshalb soll ich kufr-betreibenden Leuten nachtrauern?!\qt@no{93}}%
\qt@newcmd\qurantrans@de@mxlviii{Und WIR haben in keine Ortschaft einen Propheten entsandt, ohne daß WIR ihre Bewohner durch Not und Krankheit geprüft haben, damit sie in Demut (um Hilfe) bitten.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mxlix{Dann ersetzten WIR anstelle des Schlechten das Gute, bis sie sich erholten und sagten: ``Bereits überkam unsere Ahnen sowohl das Schädliche als auch das Fröhliche." So richteten WIR sie unerwartet zugrunde, während sie es nicht merkten.\qt@no{95}}%
\qt@newcmd\qurantrans@de@ml{Und hätten die Bewohner der Ortschaften doch den Iman verinnerlicht und Taqwa gemäß gehandelt, gewiß hätten WIR ihnen dann Baraka von dem Himmel und der Erde gewährt! Doch sie leugneten, so richteten WIR sie zugrunde wegen dem, was sie sich (an Verfehlungen) zu erwerben pflegten.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mli{Fühlen sich etwa die Bewohner der Ortschaften davor sicher, daß Unsere Peinigung sie nachts überkommt, während sie schlafen?!\qt@no{97}}%
\qt@newcmd\qurantrans@de@mlii{Oder fühlen sich etwa die Bewohner der Ortschaften davor sicher, daß Unsere Peinigung sie morgens überkommt, während sie sich vergnügen?!\qt@no{98}}%
\qt@newcmd\qurantrans@de@mliii{Fühlen sie sich etwa sicher vor ALLAHs Versuchung?! Vor ALLAHs Versuchung fühlen sich doch nur die verlierenden Leute sicher.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mliv{Wurde etwa denjenigen, welche das Land nach seinen (vorherigen) Bewohnern beerben, noch nicht deutlich, daß WIR, hätten WIR es gewollt, sie wegen ihrer Verfehlungen zugrunde richten können?! Doch WIR versiegeln ihre Herzen, so vernehmen sie nichts mehr.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mlv{Dies sind die Ortschaften, von deren Begebenheiten WIR dir (einen Teil) berichtet haben. Und gewiß, bereits kamen zu ihnen ihre Gesandten mit den deutlichen Zeichen, so würden sie nicht den Iman an das verinnerlichen, was sie vorher abgeleugnet haben. Solcherart versiegelt ALLAH die Herzen der Kafir.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mlvi{Und WIR stellten bei den meisten von ihnen keine Vertragstreue fest. Jedoch stellten WIR die meisten von ihnen als Fasiq fest.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mlvii{Dann entsandten WIR nach ihnen Musa mit Unseren Ayat zu Pharao und seinen Entscheidungsträgern, so begingen sie ihnen gegenüber Unrecht. Also siehe, wie das Anschließende von den Verderben-Anrichtenden war.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mlviii{Musa sagte: ``Pharao! Gewiß, ich bin ein Gesandter vom HERRN aller Schöpfung.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mlix{Mir obliegt es, daß ich nichts über ALLAH außer der Wahrheit sage. Bereits brachte ich euch doch ein klares Zeichen von eurem HERRN mit, so schicke mit mir die Kinder Israils!"\qt@no{105}}%
\qt@newcmd\qurantrans@de@mlx{Er sagte: ``Solltest du eine Aya mitgebracht haben, so bringe sie vor, solltest du von den Wahrhaftigen sein."\qt@no{106}}%
\qt@newcmd\qurantrans@de@mlxi{So warf er seinen Stock und er wurde zu einer sichtbaren Schlange.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mlxii{Dann zog er seine Hand (aus dem Hemd) heraus und sie wurde hell für die Zuschauer.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mlxiii{Die Entscheidungsträger vom Pharaos Leuten sagten: ``Gewiß, dieser ist doch ein im Wissen fundierter Magier!\qt@no{109}}%
\qt@newcmd\qurantrans@de@mlxiv{Er will euch aus eurem Land heraustreiben. Was empfehlt ihr nun?"\qt@no{110}}%
\qt@newcmd\qurantrans@de@mlxv{Sie sagten: ``Halte ihn mit seinem Bruder hin, und schicke in die Städte Boten,\qt@no{111}}%
\qt@newcmd\qurantrans@de@mlxvi{die dir jeden im Wissen fundierten Magier holen."\qt@no{112}}%
\qt@newcmd\qurantrans@de@mlxvii{Und die Magier kamen zu Pharao, sie sagten: ``Gibt es für uns sicher eine Belohnung, wenn wir die Sieger werden?"\qt@no{113}}%
\qt@newcmd\qurantrans@de@mlxviii{Er sagte: ``Ja! Und gewiß ihr werdet doch von den engen Vertrauten sein."\qt@no{114}}%
\qt@newcmd\qurantrans@de@mlxix{Sie sagten: ``Musa! Entweder wirfst du oder wir sind die (ersten) Werfer."\qt@no{115}}%
\qt@newcmd\qurantrans@de@mlxx{Er sagte: ``Werft ihr!" Und als sie geworfen hatten, haben sie die Augen der Menschen verzaubert und ihnen Angst eingejagt. Und sie vollbrachten gewaltige Magie.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mlxxi{Dann ließen WIR Musa durch Wahy zuteil werden: ``Nun wirf deinen Stock!" Und prompt schluckte er alles, was sie (die Magier) vortäuschten.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mlxxii{Dann war die Wahrheit erkennbar. Und annulliert wurde, was sie zu tun pflegten.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mlxxiii{so wurden sie dort besiegt und kehrten danach erniedrigt um.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mlxxiv{Dann fielen die Magier in Sudschud nieder.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mlxxv{Sie sagten: ``Wir haben den Iman an Den HERRN aller Schöpfung verinnerlicht,\qt@no{121}}%
\qt@newcmd\qurantrans@de@mlxxvi{an Den HERRN von Musa und Harun!"\qt@no{122}}%
\qt@newcmd\qurantrans@de@mlxxvii{Pharao sagte: ``Habt ihr etwa den Iman verinnerlicht, bevor ich euch dies gestattet habe? Gewiß, dies ist doch nicht anderes als Täuschung, die ihr in dieser Stadt intrigiert habt, um ihre Einwohner daraus zu vertreiben. So werdet ihr es noch wissen.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mlxxviii{Zweifelsohne werde ich eure Hände und Füße wechselseitig abschneiden, dann werde ich euch doch allesamt kreuzigen lassen."\qt@no{124}}%
\qt@newcmd\qurantrans@de@mlxxix{Sie sagten: ``Wir werden (sowieso) zu unserem HERRN zurückkehren.\qt@no{125}}%
\qt@newcmd\qurantrans@de@mlxxx{Du bemängelst doch nichts an uns, außer daß wir den Iman an die Ayat unseres HERRN verinnerlicht haben, als diese zu uns kamen. Unser HERR! Verleihe uns Geduld und lasse uns als Muslime sterben!"\qt@no{126}}%
\qt@newcmd\qurantrans@de@mlxxxi{Und die Entscheidungsträger von Pharaos Leuten sagten: ``Willst du etwa von Musa und seinen Leuten ablassen, damit sie Verderben auf der Erde anrichten und damit sie von dir (deiner Verherrlichung) und von deinen Göttern ablassen." Er sagte: ``Wir werden ihre Söhne töten und (nur) ihre Frauen am Leben lassen. Und gewiß, wir beherrschen sie."\qt@no{127}}%
\qt@newcmd\qurantrans@de@mlxxxii{Musa sagte zu seinen Leuten: ``Fleht ALLAH um Hilfe an und übt euch in Geduld! Gewiß, die Erde gehört ALLAH und ER überläßt sie, wem ER will von Seinen Dienern. Und das Anschließende ist für die Muttaqi bestimmt."\qt@no{128}}%
\qt@newcmd\qurantrans@de@mlxxxiii{Sie sagten: ``Uns wurde Schaden zugefügt, bevor du zu uns kamst und nachdem du zu uns kamst." Er sagte: ``Möglich ist es, daß euer HERR euren Feind zugrunde richtet und euch als Nachfolger im Lande einsetzt. Dann sieht ER, wie ihr handelt."\qt@no{129}}%
\qt@newcmd\qurantrans@de@mlxxxiv{Und gewiß, bereits ließen WIR Dürre und Rückgang von Erzeugnissen Pharaos Leute überkommen, damit sie sich entsinnen.\qt@no{130}}%
\qt@newcmd\qurantrans@de@mlxxxv{Und als sie Gutes traf, sagten sie: ``Dies ist unser (Verdienst)!" Und wenn Schlechtes sie berührte, prophezeiten sie von Musa und von denen, die mit ihm waren, Unheilvolles. Ja! Das von euch (prophezeite) Unheilvolle ist doch nur von ALLAH, aber die meisten von ihnen wissen es nicht.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mlxxxvi{Und sie sagten: ``Egal welche Aya du uns vollbringst, um uns damit zu verzaubern, wir werden dir keinen Iman schenken."\qt@no{132}}%
\qt@newcmd\qurantrans@de@mlxxxvii{So schickten WIR ihnen Überschwemmung, Heuschrecken, Läuse, Frösche und Blut als deutliche Ayat, dann haben sie sich in Arroganz erhoben und waren schwer verfehlende Leute.\qt@no{133}}%
\qt@newcmd\qurantrans@de@mlxxxviii{Und als sie die Peinigung überkam, sagten sie: ``Musa! Richte für uns Bittgebete an deinen HERRN, wie ER es dir geboten hat. Wenn ER von uns die Peinigung wegnimmt, gewiß werden wir den Iman an dich verinnerlichen und mit dir Israils Kinder doch mitschicken.\qt@no{134}}%
\qt@newcmd\qurantrans@de@mlxxxix{Doch als WIR von ihnen die Peinigung - bis zu einer für sie festgelegten Frist, weggenommen haben, hielten sie (ihr Versprechen) nicht ein.\qt@no{135}}%
\qt@newcmd\qurantrans@de@mxc{So übten WIR an ihnen Vergeltung. Dann haben WIR sie im Meer ertrinken lassen, weil sie Unsere Ayat leugneten und ihnen gegenüber achtlos zu sein pflegten.\qt@no{136}}%
\qt@newcmd\qurantrans@de@mxci{Und WIR ließen diejenigen Leute, die unterdrückt wurden, erben das Östliche und das Westliche des Landes, das WIR mit Baraka erfüllt haben. Und das gute Versprechen deines HERRN für die Kinder Israils wurde vollständig eingelöst als Belohnung für die Geduld, die sie aufbrachten. Und WIR vernichteten das, was Pharao und seine Leute bewerkstelligten, und das, was sie an Bauten errichteten.\qt@no{137}}%
\qt@newcmd\qurantrans@de@mxcii{Und WIR ließen die Kinder Israils das Meer überqueren, dann kamen sie zu Leuten, die sich eigenen Götzen zuwendeten. Sie sagten: ``Musa! Richte für uns einen Gott wie ihre Götter ein!" Er sagte: ``Gewiß, ihr seid unwissende Leute!\qt@no{138}}%
\qt@newcmd\qurantrans@de@mxciii{Gewiß, für diese wird das, woran sie sind, zugrunde gerichtet, und nichtig ist das, was sie zu tun pflegten."\qt@no{139}}%
\qt@newcmd\qurantrans@de@mxciv{Er sagte: ``Soll ich euch etwa anstelle von ALLAH einen anderen Gott erstreben lassen, wo ER euch den anderen Menschen gegenüber ausgezeichnet hat?!"\qt@no{140}}%
\qt@newcmd\qurantrans@de@mxcv{Und erinnert euch daran), als WIR euch vor Pharaos Leuten errettet haben, nachdem sie euch das Üblste an Mißhandlung zugefügt hatten, sie schlachteten eure Söhne ab und ließen (nur) eure Frauen am Leben. Darin lag wahrhaftig eine schwere Prüfung von eurem HERRN!\qt@no{141}}%
\qt@newcmd\qurantrans@de@mxcvi{Und WIR legten für Musa eine Verabredung nach dreißig Tagen fest, dann erweiterten WIR sie um zehn Tage, so wurde die Verabredung, die sein HERR festlegte, nach vierzig Tagen vollständig. Musa sagte zu seinem Bruder Harun: ``Sei mein Khalifa bei meinen Leuten, tue Gutes und folge nicht dem Weg der Verderben-Anrichtenden!"\qt@no{142}}%
\qt@newcmd\qurantrans@de@mxcvii{Und als Musa zu der von Uns festgelegten Verabredung kam und sein HERR zu ihm sprach, sagte er: ``HERR! Lasse mich sehen, daß ich Dich anschaue." ER sagte: ``Du wirst Mich nicht sehen! Aber schau dir den Felsenberg an, sollte er an seiner Stelle bleiben, so wirst du Mich sehen." Doch als sein HERR sich dem Felsenberg offenbarte, hat ER ihn zu Staub gemacht, und Musa fiel bewußtlos nieder. Und als er erwachte, sagte er: ``Gepriesen-erhaben bist DU! Ich bereue es Dir gegenüber und ich bin der Erste der Mumin."\qt@no{143}}%
\qt@newcmd\qurantrans@de@mxcviii{ER sagte: ``Musa! Gewiß, ich habe dich vor allen Menschen für Meine Botschaft und für Meine Worte auserwählt, so nimm das entgegen, was ICH dir zuteil werden ließ, und sei von den Dankbaren!"\qt@no{144}}%
\qt@newcmd\qurantrans@de@mxcix{Und WIR schrieben ihm auf den Tafeln über alle (euch auferlegten) Dinge - als Ermahnung und als Erläuterung für jede (euch auferlegte) Sache. So setze diese mit Ernsthaftigkeit um und gebiete deinen Leuten, daß sie das Bessere davon umsetzen. ICH werde euch die Wohnstätten der Fasiq zeigen.\qt@no{145}}%
\qt@newcmd\qurantrans@de@mc{Meine Ayat werde ICH nicht wahrnehmen lassen diejenigen, die sich zu Unrecht auf der Erde in Arroganz erheben. Und wenn sie jegliche Aya sehen, verinnerlichen sie daran keinen Iman. Auch wenn sie den Weg der Rechtschaffenheit erkennen, folgen sie ihm nicht als (ihren) Weg, doch wenn sie den Weg des Fehlgehens erkennen, folgen sie ihm als (ihren) Weg, dies weil sie Unsere Ayat verleugnen und ihnen gegenüber achtlos zu sein pflegten.\qt@no{146}}%
\qt@newcmd\qurantrans@de@mci{Und diejenigen, die Unsere Ayat und das Treffen im Jenseits verleugnen, ihre (guten) Taten sind zunichte geworden. Wird ihnen etwa anderes vergolten als das, was sie zu tun pflegten?!\qt@no{147}}%
\qt@newcmd\qurantrans@de@mcii{Und die Leute von Musa nahmen sich nach dessen (Fortgang) aus ihrem Schmuck ein Kalb (als Götzen) - einen Körper, der muhte. Sahen sie etwa nicht, daß es weder mit ihnen spricht, noch sie auf den Weg rechtleiten kann?! Sie dienten ihm und waren Unrecht-Begehende.\qt@no{148}}%
\qt@newcmd\qurantrans@de@mciii{Und als sie bereut und erkannt haben, daß sie bereits fehlgegangen sind, sagten sie: ``Wenn unser HERR uns keine Gnade erweist und uns vergibt, werden wir gewiß von den Verlierern sein."\qt@no{149}}%
\qt@newcmd\qurantrans@de@mciv{Und als Musa zu seinen Leuten zurückkam - zornig und voller Trauer, sagte er: ``Erbärmlich ist das, wie ihr nach mir meine Nachfolge angetreten habt. Habt ihr etwa das Gebot eures HERRN nicht abwarten wollen?" Dann warf er die Tafeln und packte seinen Bruder am Kopf und zog ihn zu sich, dieser sagte: ``Sohn meiner Mutter! Die Leute haben mich gedemütigt und beinahe getötet, laß die Feinde nicht über mich triumphieren und setze mich nicht gleich mit den Unrecht- Begehenden!"\qt@no{150}}%
\qt@newcmd\qurantrans@de@mcv{Er sagte: ``HERR! Vergib mir und meinem Bruder und laß uns in Deine Gnade eintreten. DU bist der Gnädigste der Gnädigen!"\qt@no{151}}%
\qt@newcmd\qurantrans@de@mcvi{Gewiß, diejenigen, die sich das Kalb (als Götzen) nahmen, wird Zorn von ihrem HERRN und Erniedrigung im diesseitigen Leben treffen. Und solcherart vergelten WIR den Lügenerfindern.\qt@no{152}}%
\qt@newcmd\qurantrans@de@mcvii{Und für diejenigen, welche die gottmißfälligen Taten begingen, danach bereuten und den Iman verinnerlichten  gewiß, dein HERR ist danach doch allvergebend, allgnädig!\qt@no{153}}%
\qt@newcmd\qurantrans@de@mcviii{Und als der Zorn von Musa sich legte, nahm er die Tafeln. In ihrer Inschrift war Rechtleitung und Gnade für diejenigen, die vor ihrem HERRN Ehrfurcht empfinden.\qt@no{154}}%
\qt@newcmd\qurantrans@de@mcix{Musa wählte dann von seinen Leuten siebzig Männer für Unsere Verabredung aus. Und nachdem das Beben sie erfaßt hatte, sagte er: ``HERR! Hättest DU es gewollt, hättest DU sie vorher mit mir zugrunde gehen lassen. Willst DU uns etwa zugrunde gehen lassen für das, was die Beschränkten von uns getan haben?! Dies ist nichts anderes als Deine Fitna, damit läßt DU abirren, wen DU willst, und leitest recht, wen DU willst. DU bist unser Wali, so vergib uns und erweise uns Gnade! Und DU bist der Beste der Vergebenden.\qt@no{155}}%
\qt@newcmd\qurantrans@de@mcx{Und bestimme für uns in diesem Diesseits Gutes und auch im Jenseits, denn gewiß, wir bereuen Dir gegenüber. ER sagte: ``Mit Meiner Peinigung treffe ICH, wen ICH will. Doch Meine Gnade umfaßt alles. So werde ICH sie für diejenigen bestimmen, die Taqwa gemäß handeln und Zakat entrichten, und für diejenigen, die den Iman an Unsere Ayat verinnerlichen."\qt@no{156}}%
\qt@newcmd\qurantrans@de@mcxi{Es sind diejenigen, die dem Gesandten, dem lese- und schreibunkundigen Propheten folgen, über den sie bei sich in At-taurat und Alindschil geschrieben finden. Er ruft sie zum Gebilligten auf, rät ihnen vom Mißbilligten ab, erklärt ihnen die guten Dinge für halal und die schlechten Dinge für haram und erleichtert ihnen ihre schweren Gebote und die Einschränkungen, die ihnen auferlegt waren. Also diejenigen, die den Iman an ihn verinnerlicht, ihn geehrt, ihm zum Sieg verholfen haben und dem Licht (Quran) gefolgt sind, das ihm hinabgesandt wurde, diese sind die wirklichen Erfolgreichen.\qt@no{157}}%
\qt@newcmd\qurantrans@de@mcxii{Sag: ``Ihr Menschen! Gewiß, ich bin ALLAHs Gesandter zu euch allesamt, Demjenigen, Dem die Himmel und die Erde gehören. Es gibt keinen Gott außer Ihm, ER macht lebendig und läßt sterben. So verinnerlicht den Iman an ALLAH und an Seinen Gesandten, den lese- und schreibunkundigen Propheten, denjenigen, der den Iman an ALLAH und an Seine Worte verinnerlicht. Und folgt ihm, damit ihr Rechtleitung findet."\qt@no{158}}%
\qt@newcmd\qurantrans@de@mcxiii{Auch von Musas Leuten gibt es eine Umma, die mit der Wahrheit rechtleiten und danach Gerechtigkeit üben.\qt@no{159}}%
\qt@newcmd\qurantrans@de@mcxiv{Und WIR teilten sie ein in zwölf Stämme, Umam. Und WIR ließen Musa Wahy zuteil werden, als seine Leute ihn um Wasser baten: ``Schlage mit deinem Stock gegen den Felsen!" So entsprangen daraus zwölf Quellen. Jede Gruppe kannte bereits ihre Trinkstelle. WIR haben sie dann mit Wolken beschattet und ihnen Al-mann und As-salwa geschickt. Esst von den Tay-yibat dessen, wasWIR euch an Rizq gewährten. Doch Uns haben sie kein Unrecht angetan, sondern sie pflegten sich selbst Unrecht anzutun.\qt@no{160}}%
\qt@newcmd\qurantrans@de@mcxv{Und erinnere daran), als ihnen gesagt wurde: ``Bewohnt diesen Ort, esst darin allerorts, wo ihr wollt, sagt: ``Entlastung!" und tretet ein durch das Tor in Sudschud- Haltung, dann vergeben WIR euch eure Verfehlungen. Noch mehr werden WIR den Muhsin gewähren!"\qt@no{161}}%
\qt@newcmd\qurantrans@de@mcxvi{Diejenigen von ihnen, die Unrecht begangen haben, vertauschten dann ein anderes Wort als das, was ihnen bereits verkündet wurde. So schickten WIR ihnen Peinigung vom Himmel für das, was sie an Unrecht zu begehen pflegten.\qt@no{162}}%
\qt@newcmd\qurantrans@de@mcxvii{Und frage sie nach der Ortschaft, die in der Nähe des Meeres lag, als sie am Sabbat übertreten haben, als zu ihnen die Fische am Sabbat sichtbar (übers Wasser) kamen, während diese außerhalb des Sabbat nicht kamen. Solcherart prüfen wir sie wegen dem, was sie an Fisq zu betreiben pflegten.\qt@no{163}}%
\qt@newcmd\qurantrans@de@mcxviii{Und (erinnere daran), als eine Umma von ihnen sagte: ``Wozu ermahnt ihr Leute, die ALLAH zugrunde richten oder mit einer qualvollen Peinigung peinigen wird?" Sie sagten: ``(Unsere Ermahnung ist) eine Entschuldigung vor eurem HERRN und damit sie vielleicht Taqwa gemäß handeln."\qt@no{164}}%
\qt@newcmd\qurantrans@de@mcxix{Doch als sie vergessen haben, womit sie ermahnt wurden, erretteten WIR diejenigen, die das Verwerfliche verboten, und setzten diejenigen, die Unrecht begingen, einer qualvollen Peinigung aus für das, was sie an Fisq zu betreiben pflegten.\qt@no{165}}%
\qt@newcmd\qurantrans@de@mcxx{Und nachdem sie sich über das erhoben hatten, was ihnen untersagt wurde, sagten WIR ihnen: ``Seid erniedrigte Affen!"\qt@no{166}}%
\qt@newcmd\qurantrans@de@mcxxi{Und (erinnere daran), als dein HERR es bekanntmachte: ``ER wird ihnen doch bis zum Jüngsten Tag Menschen schicken, die ihnen das Härteste an Peinigung zufügen." Gewiß, Dein HERR ist zweifelsohne schnell im Strafen und gewiß, ER ist doch allvergebend, allgnädig.\qt@no{167}}%
\qt@newcmd\qurantrans@de@mcxxii{Und WIR teilten sie auf der Erde in Umam ein. Unter ihnen sind die gottgefällig Guttuenden und unter ihnen sind manche, die darunter sind. Und WIR prüften sie mit gottgefälligen guten Taten und mit den gottmißfälligen Taten, damit sie umkehren.\qt@no{168}}%
\qt@newcmd\qurantrans@de@mcxxiii{Dann folgten ihnen schlechte Nachfolger nach, welche die Schrift erbten. Sie eignen sich das Minderwertige dieses Vergänglichen an und sagen (dennoch): ``Es wird uns vergeben!" Doch wenn zu ihnen Ähnliches an Vergänglichem kommt, eignen sie es sich (wieder) an. Wurde von ihnen etwa nicht das Gelöbnis der Schrift entgegengenommen, daß sie im Namen ALLAHs nichts außer der Wahrheit sagen?! Und sie haben in ihr (der Schrift) studiert. Und die jenseitige Wohnstätte ist besser für diejenigen, die Taqwa gemäß handeln. Wollt ihr etwa nicht verständig sein?!\qt@no{169}}%
\qt@newcmd\qurantrans@de@mcxxiv{Und (für) diejenigen, welche die Schrift eingehalten und das rituelle Gebet ordnungsgemäß verrichtet haben, gewiß, WIR lassen die Belohnung der gottgefällig Guttuenden nicht verloren gehen.\qt@no{170}}%
\qt@newcmd\qurantrans@de@mcxxv{Und (erinnere daran), als WIR den Berg über sie erhoben haben, als wäre er eine Wolke, und sie dachten, daß er doch auf sie herabfallen würde. Nehmt mit Ernsthaftigkeit, was WIR euch zuteil werden ließen und entsinnt euch dessen, damit ihr Taqwa gemäß handelt.\qt@no{171}}%
\qt@newcmd\qurantrans@de@mcxxvi{Und (erinnere daran), als dein HERR von den Kindern Adams, aus ihren Rücken (Becken) ihre Nachkommenschaft entnahm und sie gegen sich selbst Zeugnis ablegen ließ: ``Bin ICH etwa nicht euer HERR?!" Sie sagten: ``Doch mit Sicherheit! Wir bezeugen es." Also, damit ihr am Tag der Auferstehung nicht sagt: ``Gewiß, wir waren demgegenüber achtlos",\qt@no{172}}%
\qt@newcmd\qurantrans@de@mcxxvii{oder sagt: ``Nur unsere Ahnen haben vorher Schirk betrieben und wir waren Nachkommenschaft nach ihnen! Willst DU uns etwa zugrunde richten für das, was die des Unwahren Verbreitenden taten?"\qt@no{173}}%
\qt@newcmd\qurantrans@de@mcxxviii{Und solcherart verdeutlichen WIR die Ayat und damit sie (zum Iman) zurückkehren.\qt@no{174}}%
\qt@newcmd\qurantrans@de@mcxxix{Und trage ihnen vor die Nachricht über denjenigen, dem WIR Unsere Ayat zuteil werden ließen, dann hat er sich von ihnen abgewendet, dann folgte ihm der Satan, so wurde er von den Verführten.\qt@no{175}}%
\qt@newcmd\qurantrans@de@mcxxx{Und hätten WIR es gewollt, hätten WIR ihm damit (den Ayat) doch eine hohe Stellung gewährt. Doch er neigte zur Erde (zum Diesseits) und folgte seinen Neigungen. So ähnelt sein Gleichnis dem Gleichnis eines Hundes: ``Wenn du ihn verjagst, hechelt er, und wenn du ihn in Ruhe läßt, hechelt er auch." Solcherart ist das Gleichnis von Leuten, die Unsere Ayat ableugneten. So erzähle die (dir offenbarten) Begebenheiten, damit sie sich entsinnen.\qt@no{176}}%
\qt@newcmd\qurantrans@de@mcxxxi{Erbärmlich ist das Gleichnis der Menschen, die Unsere Ayat ableugneten, und sich selbst Unrecht anzutun pflegten.\qt@no{177}}%
\qt@newcmd\qurantrans@de@mcxxxii{Wen ALLAH rechtleitet, ist gewiß der Rechtgeleitete. Wen ER aber abirren läßt, so sind diese die wirklichen Verlierer.\qt@no{178}}%
\qt@newcmd\qurantrans@de@mcxxxiii{Und gewiß, bereits erschufen WIR für Dschahannam viele von den Dschinn und den Menschen. Sie haben Herzen, mit denen sie keine Einsicht gewinnen, und Augen, mit denen sie (die Wahrheit) nicht erblicken, und Ohren, mit denen sie (die Ermahnung) nicht hören. Diese sind wie die An'am, nein, sondern sie sind noch abirrender! Diese sind die wirklichen Achtlosen.\qt@no{179}}%
\qt@newcmd\qurantrans@de@mcxxxiv{Und ALLAH hat die schönsten/besten Namen, so benennt Ihn damit. Und laßt ab von denjenigen, die Seine Namen entstellen. Ihnen wird das vergolten, was sie zu tun pflegten.\qt@no{180}}%
\qt@newcmd\qurantrans@de@mcxxxv{Und von denjenigen, die WIR erschaffen haben, gibt es eine Umma, sie leiten recht mit der Wahrheit und richten nach ihr.\qt@no{181}}%
\qt@newcmd\qurantrans@de@mcxxxvi{Und diejenigen, die Unsere Ayat ableugnen, diese werden WIR der Versuchung aussetzen da, wo sie es nicht wissen.\qt@no{182}}%
\qt@newcmd\qurantrans@de@mcxxxvii{Auch gewähre ICH ihnen noch mehr Zeit. Doch Meine Bestrafung ist hart.\qt@no{183}}%
\qt@newcmd\qurantrans@de@mcxxxviii{Besinnen sie sich etwa nicht?! Ihr Gefährte (Muhammad) leidet nicht unter Geistesgestörtheit, er ist doch nur ein deutlicher Warner.\qt@no{184}}%
\qt@newcmd\qurantrans@de@mcxxxix{Haben sie etwa nicht ALLAHs Herrschaft über die Himmel und die Erde versonnen erblickt und das, was ALLAH an Dingen erschuf, und daß vielleicht ihr Ende bereits sich genähert hat?! An welche Mitteilung werden sie denn nach ihm (Quran) den Iman verinnerlichen?!\qt@no{185}}%
\qt@newcmd\qurantrans@de@mcxl{Wen ALLAH in die Irre gehen läßt, der findet gewiß keinen Rechtleitenden. Und ER läßt sie in ihrer Maßlosigkeit sich verblenden.\qt@no{186}}%
\qt@newcmd\qurantrans@de@mcxli{Sie fragen dich nach der Stunde (dem Jüngsten Tag): ``Wann ereignet sie sich?" Sag: ``Das Wissen darüber ist nur bei meinem HERRN. Keiner kann sie zu ihrer Zeit offenlegen außer ER. Schwer lastet sie in den Himmeln und auf der Erde. Sie überkommt euch nur plötzlich." Sie fragen dich, als ob du davon genaueste Kenntnis besäßest. Sag: ``Das Wissen über sie ist nur bei ALLAH, doch die meisten Menschen wissen es nicht."\qt@no{187}}%
\qt@newcmd\qurantrans@de@mcxlii{Sag: ``Ich verfüge für mich selbst weder über Nützliches noch über Schädliches, es sei denn über das, was ALLAH will. Und hätte ich Kenntnis über das Verborgene gehabt, gewiß hätte ich mir vom Guten viel angeeignet, und Schlechtes würde mich nicht treffen! Ich bin doch nur ein Ermahner und ein Überbringer froher Botschaft für Leute, die den Iman verinnerlichen."\qt@no{188}}%
\qt@newcmd\qurantrans@de@mcxliii{ER ist Derjenige, Der euch aus einem einzigen Wesen erschuf und daraus sein Partnerwesen erschuf, damit es bei ihm Ruhe findet. Und nachdem er mit ihr intim gewesen war, hatte sie eine leichte Schwangerschaft, die sie bis zum Ende austrug. Und als die Entbindung sich näherte, richteten beide an ALLAH, ihren HERRN Bittgebete: ``Wenn DU uns ein gesundes Kind schenkst, gewiß werden wir von den Dankbaren sein!"\qt@no{189}}%
\qt@newcmd\qurantrans@de@mcxliv{Und nachdem ER ihnen ein gesundes Kind gegeben hatte, haben sie Ihm Partner mit dem beigesellt, was ER ihnen gegeben hat. Immer gepriesen-erhaben ist ALLAH über das, was sie an Schirk betreiben.\qt@no{190}}%
\qt@newcmd\qurantrans@de@mcxlv{Betreiben sie etwa Schirk mit dem, was Nichts schafft, während sie alle doch erschaffen werden?!\qt@no{191}}%
\qt@newcmd\qurantrans@de@mcxlvi{Und diese (Partner) können weder ihnen beistehen, noch sich selbst beistehen.\qt@no{192}}%
\qt@newcmd\qurantrans@de@mcxlvii{Auch dann solltet ihr sie um Rechtleitung bitten, entsprechen sie euch nicht. Für euch bleibt es gleich, ob ihr sie bittet oder ob ihr schweigt.\qt@no{193}}%
\qt@newcmd\qurantrans@de@mcxlviii{Gewiß, diejenigen, an die ihr anstelle von ALLAH Bittgebete richtet, sind nur Diener genauso wie ihr. Also richtet nur an sie Bittgebete und sie sollen euch erhören, solltet ihr wahrhaftig sein.\qt@no{194}}%
\qt@newcmd\qurantrans@de@mcxlix{Haben sie etwa Beine, mit denen sie gehen können, oder etwa Hände, mit denen sie zuschlagen können, oder etwa Augen, mit denen sie sehen können, oder etwa Ohren, mit denen sie hören können?! Sag: ``Richtet nur Bittgebete an eure Partner, dann führt eure List gegen mich durch und gewährt mir dabei keine Zeit!\qt@no{195}}%
\qt@newcmd\qurantrans@de@mcl{Gewiß, mein Wali ist ALLAH! ER ist Derjenige, Der die Schrift nach und nach hinabsandte und ER ist der Wali für die gottgefällig Guttuenden."\qt@no{196}}%
\qt@newcmd\qurantrans@de@mcli{Und diejenigen, an die ihr anstelle von Ihm Bittgebete richtet, können weder euch beistehen, noch sich selbst beistehen.\qt@no{197}}%
\qt@newcmd\qurantrans@de@mclii{Und solltest du sie um Rechtleitung bitten, hören sie bestimmt nicht. Und du siehst sie dich anstarren, während sie doch nichts sehen.\qt@no{198}}%
\qt@newcmd\qurantrans@de@mcliii{Nimm Al-'afu an, gebiete das Gebilligte und meide die Unwissenden.\qt@no{199}}%
\qt@newcmd\qurantrans@de@mcliv{Und sollte dich eine Versuchung des Satans (vom Wahren) ablenken, so suche Zuflucht bei ALLAH! Gewiß, ER ist allhörend, allwissend.\qt@no{200}}%
\qt@newcmd\qurantrans@de@mclv{Gewiß, diejenigen, die Taqwa gemäß handeln, wenn sie Unschönes durch Satan überkommt, entsinnen sie sich und sehen (ihre Verfehlung) wieder ein.\qt@no{201}}%
\qt@newcmd\qurantrans@de@mclvi{Doch ihre Brüder lassen sie (die Satane) in die Irre weit gehen, dann lassen sie von ihnen nicht mehr ab.\qt@no{202}}%
\qt@newcmd\qurantrans@de@mclvii{Und wenn du ihnen keine Ayat vollbringst, sagten sie: ``Hättest du sie doch zusammengestellt." Sag: ``Ich folge nur dem, was mir von meinem HERRN an Wahy zuteil wird." Dies ist (der Quran) etwas Einblick-Gewährendes von eurem HERRN, eine Rechtleitung und eine Gnade für Menschen, die den Iman verinnerlichen.\qt@no{203}}%
\qt@newcmd\qurantrans@de@mclviii{Und wenn der Quran vorgetragen wird, dann hört ihm zu und seid lautlos bedachtsam, damit euch Gnade erwiesen wird.\qt@no{204}}%
\qt@newcmd\qurantrans@de@mclix{Und lobpreise deinen HERRN in deinem Innern demütig, ehrfürchtig und in leiserer Art als das wahrnehmbare Sprechen - morgens und abends, und sei nicht von den Achtlosen!\qt@no{205}}%
\qt@newcmd\qurantrans@de@mclx{Gewiß, diejenigen bei deinem HERRN erheben sich nicht in Arroganz Seinem Dienen gegenüber, lobpreisen Ihn und vollziehen Ihm Sudschud.\qt@no{206}}%
\qt@newcmd\qurantrans@de@mclxi{\basmalahde Sie fragen dich nach den Al-anfal. Sag: ``Al-anfal gehören ALLAH und dem Gesandten, also handelt Taqwa gemäß ALLAH gegenüber, schlichtet den Streit untereinander wieder und gehorcht ALLAH und Seinem Gesandten, wenn ihr Mumin seid.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mclxii{Die Mumin sind ausschließlich diejenigen, deren Herzen Ehrfurcht empfinden, wenn ALLAH erwähnt wird, und wenn ihnen Seine Ayat vorgetragen werden, lassen diese ihren Iman zunehmen, und sie üben Tawakkul ihrem HERRN gegenüber.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mclxiii{Es sind diejenigen, die das rituelle Gebet ordnungsgemäß verrichten und vom Rizq gaben, dasWIR ihnen gewährten.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mclxiv{Diese sind die wahren Mumin. Für sie sind hohe Stellungen bei ihrem HERRN bestimmt, Vergebung und edles Rizq.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mclxv{Ihre Abneigung gegen diese Einteilung von Al-anfal ist) genauso (wie damals), als dein HERR dich aus deiner Wohnstätte mit der Wahrheit herausführen ließ, während eine Gruppe von den Mumin gewiß dem doch abgeneigt war.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mclxvi{Sie disputieren mit dir über das Wahre, nachdem es ihnen sichtbar wurde, als würden sie in den Tod hineingetrieben, während sie (ihn) sehen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mclxvii{Und (erinnere daran), als ALLAH euch versprochen hat, daß eine der beiden Gruppen euch gehören wird. Jedoch ihr wünschtet gerne, daß die nicht kampfbereite (Gruppe) euch gehörte. Doch ALLAH will dem Wahren mit Seinen Worten zur Geltung verhelfen und die Kafir ausrotten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mclxviii{ER wird gewiß dem Wahren zur Geltung verhelfen und das für nichtig Erklärte annullieren, selbst dann, wenn die schwer Verfehlenden dem abgeneigt sind.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mclxix{Und erinnere daran), als ihr euren HERRN um Hilfe gebeten habt und ER euch dann erhört hat: ``Gewiß, ICH werde euch mit tausend Engeln reihenweise Verstärkung gewähren."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mclxx{Und ALLAH hat dies ausschließlich als frohe Botschaft vollbracht und damit eure Herzen Ruhe finden. Und den Sieg verleiht nur ALLAH. Gewiß, ALLAH ist allwürdig, allweise.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mclxxi{Und erinnere daran), als ER die Schläfrigkeit euch überkommen ließ - (als Zeichen der) Sicherheit von Ihm, und das Wasser über euch vom Himmel fallen ließ, um euch damit zu reinigen, von euch das Einflüstern des Satans wegzunehmen, eure Herzen zu stärken und euch (im Kampf) standhaft zu machen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mclxxii{Als dein HERR den Engeln Wahy zuteil werden ließ: ``Gewiß, ICH bin mit euch, so festigt diejenigen, die den Iman verinnerlicht haben. In die Herzen derjenigen, die Kufr betrieben haben, werde ich Angst einjagen, so schlagt (sie) auf die Nacken und schlagt ihnen jeden Finger ab!"\qt@no{12}}%
\qt@newcmd\qurantrans@de@mclxxiii{Dies ist so, weil sie sich doch gegen ALLAH und Seinen Gesandten aufgelehnt haben. Und wer sich gegen ALLAH und Seinen Gesandten auflehnt, so ist ALLAH gewiß hart im Strafen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mclxxiv{Dies ist (die Peinigung), so erfahrt sie! Und für die Kafir ist gewiß noch die Peinigung des Feuers bestimmt.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mclxxv{Ihr, die den Iman verinnerlicht habt! Wenn ihr auf die Kafir im Kampf trefft, so kehrt ihnen nicht die Rücken!\qt@no{15}}%
\qt@newcmd\qurantrans@de@mclxxvi{Und wer an diesem Tag ihnen den Rücken kehrt - außer um sich an anderen Kampfhandlungen zu beteiligen oder um sich einer anderen Einheit anzuschließen, hat das Erzürnen ALLAHs auf sich geladen und seine Unterkunft ist Dschahannam. Und erbärmlich ist das Werden.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mclxxvii{Getötet habt nicht ihr sie, sondern ALLAH hat sie getötet. Und geworfen hast nicht du, als du geworfen hast, sondern ALLAH hat geworfen. Und damit ER die Mumin einer guten Prüfung unterzieht. Gewiß, ALLAH ist allhörend, allwissend.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mclxxviii{So war es. Und ALLAH läßt sicher die List der Kafir schwächen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mclxxix{Wenn ihr (Kafir) um den Sieg (als Zeichen der Rechtschaffenheit) bittet, so traf der Sieg euch bereits. Wenn ihr aber (vom Kufr) ablasst, so ist dies besser für euch. Wenn ihr jedoch wiederholtermaßen (zum Krieg) zurückkehrt, kehren WIR zurück, und dann wird eure Gruppe von euch nichts abwenden, selbst dann sollte sie noch so groß sein. Und ALLAH ist gewiß mit den Mumin.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mclxxx{Ihr, die den Iman verinnerlicht habt! Gehorcht ALLAHund Seinem Gesandten und kehrt ihm nicht den Rücken, während ihr hört.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mclxxxi{Und seid nicht wie diejenigen, die sagten: ``Wir haben gehört", während sie doch nichts hören.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mclxxxii{Gewiß, die Übelsten der sich (auf Erden) bewegenden Lebewesen, sind bei ALLAH die Stummen und die Tauben, die nicht verständig sind.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mclxxxiii{Und hätte ALLAH bei ihnen Gutes gekannt, hätte ER sie hören lassen. Und hätte ER sie hören lassen, hätten sie doch den Rücken gekehrt, während sie abgeneigt sind.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mclxxxiv{Ihr, die den Iman verinnerlicht habt! Fügt euch ALLAH und dem Gesandten, wenn er euch zu dem aufruft, was euch lebendig macht. Und wisst, daß ALLAH gewiß dem Menschen die Verfügung über sein Inneres verwehren kann und daß ihr gewiß zu Ihm versammelt werdet.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mclxxxv{Und sucht Taqwa einer Fitna gegenüber, die nicht nur diejenigen unter euch treffen wird, die Unrecht begingen. Und wisst, daß ALLAH zweifelsohne hart im Strafen ist.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mclxxxvi{Und bedenkt, als ihr Wenige wart, im Lande unterdrückt und Angst hattet, daß die (feindlichen)Menschen euch Gewalt antun könnten, dann gewährte ER euch Unterkunft, unterstützte euch mit Seinem Sieg und gewährte euch Rizq von den Tay-yibat, damit ihr euch dankbar erweist.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mclxxxvii{Ihr, die den Iman verinnerlicht habt! Begeht keinen Verrat an ALLAH und an den Gesandten, auch begeht keinen Verrat mit den euch anvertrauten Dingen, während ihr es wisst.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mclxxxviii{Und wisst, daß euer Vermögen und eure Kinder nur eine Fitna sind, und daß es bei ALLAH gewiß unermeßliche Belohnung gibt.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mclxxxix{Ihr, die den Iman verinnerlicht habt! Wenn ihr ALLAH gegenüber Taqwa gemäß handelt, gewährt ER euch Unterscheidungsvermögen, tilgt euch eure gottmißfälligen Taten und vergibt euch. Und ALLAH ist von unermeßlicher Gunst!\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcxc{Und erinnere daran), als diejenigen, die Kufr betrieben, gegen dich intrigiert haben, um dich entweder gefangen zu nehmen, oder zu töten, oder zu vertreiben. Und sie haben intrigiert und ALLAH entgegnete ihre Intrigen und ALLAHist Der Beste, Der Intrigen entgegnet.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcxci{Und als ihnen Unsere Ayat vorgetragen wurden, sagten sie: ``Bereits hörten wir es, hätten wir es gewollt, hätten wir doch Ähnliches wie dieses gesagt. Dies sind doch nur die Legenden der Vorfahren."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcxcii{Und (erinnere daran), als sie sagten: ``ALLAH! Wäre dies die Wahrheit von Dir, so lasse auf uns Steine vom Himmel regnen oder lasse uns eine qualvolle Peinigung zuteil werden!"\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcxciii{Doch ALLAH wird sie nicht strafen, solange du unter ihnen bist. Und ALLAH wird sie nicht strafen, solange sie Ihn um Vergebung bitten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcxciv{Und weshalb sollte ALLAH sie (ansonsten) nicht strafen, wo sie von Almasdschidil-haram abhalten?! Und sie waren niemals dessen Wali. Dessen Wali sind gewiß nur die Muttaqi. Doch viele von ihnen wissen es nicht.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcxcv{Und ihr Gebet bei Al-bait war nichts anderes als Pfeifen und Klatschen. Also erfahrt die Peinigung wegen dem, was ihr an Kufr zu betreiben pflegtet.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcxcvi{Gewiß, diejenigen, die Kufr betrieben haben, geben ihr Vermögen aus, um vom Wege ALLAHs abzuhalten. So werden sie es ausgeben, dann wird es für sie ein Jammer sein und dann werden sie besiegt werden. Und diejenigen, die Kufr betrieben haben, werden zu Dschahannam versammelt.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcxcvii{Zweifelsohne wird ALLAH das Verwerfliche vom Guten absondern, das Verwerfliche übereinander legen und dann alles anhäufen und in Dschahannam hineinschicken. Diese sind die wirklichen Verlierer.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcxcviii{Sag zu denjenigen, die Kufr betrieben haben: ``Wenn sie damit aufhören, wird ihnen das Vergangene vergeben, sollten sie jedoch (zum Kufr/Krieg) zurückkehren, so wird bereits nach dem Beispiel der Vorfahren vorgegangen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcxcix{Und führt gegen sie den bewaffneten Kampf, damit es keine Fitna mehr gibt und der Din insgesamt nur für ALLAH bleibt. Und wenn sie aufhören, so ist ALLAH gewiß dessen allsehend, was sie tun.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcc{Und wenn sie sich abwenden, dann wisst, daß ALLAH sicher euer Maula ist. ER ist Der beste Maula und Der beste Beistehende.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcci{Und wisst, daß von allem, was ihr an Beutegütern erlangt, ein Fünftel für ALLAHs (Weg), für den Gesandten, für die Verwandten, für die Waisen, für die Bedürftigen und für den in Not geratenen Reisenden ist, wenn ihr den Iman an ALLAH und an das verinnerlicht habt, was WIR Unserem Diener am Tag der Unterscheidung, als die beiden Gruppen aufeinander trafen, zuteil werden ließen. Und ALLAH ist über alles allmächtig.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mccii{Erinnere daran), als ihr am näheren Abhang des Tales (zu Madina) wart und sie am fernen Abhang waren und die Karawane am Ort unterhalb des Euren. Und hättet ihr euch da verabredet, hättet ihr die Verabredung gewiß nicht eingehalten, aber ALLAH wollte eine Angelegenheit verwirklichen, die sowieso bestimmt war, damit jeder, der zugrunde ging (im Kufr), nach einem eindeutigen Zeichen zugrunde geht, und jeder, der dann lebte (im Islam), nach einem eindeutigen Zeichen lebt. Und gewiß, ALLAH ist doch allhörend, allwissend!\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcciii{Und erinnere daran), als ALLAH sie (die Kafir) dir in deinem Traum als Wenige gezeigt hat. Und hätte ER sie euch als Viele gezeigt, hättet ihr gewiß versagt und unterschiedliche Ansicht in der Angelegenheit (Kampf) vertreten, doch ALLAH hat euch davor bewahrt. Gewiß, ER ist allwissend über das in den Brüsten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcciv{Und erinnere daran), als ER sie in euren Augen, als ihr aufeinander getroffen seid, als Wenige erscheinen ließ, und euch in ihren Augen als Wenige erscheinen ließ, damit ALLAH eine Angelegenheit verwirklicht, die sowieso bestimmt war. Und an ALLAH werden die Angelegenheiten gewandt.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mccv{Ihr, die den Iman verinnerlicht habt! Solltet ihr auf eine (feindliche) Gruppe treffen, so seid standhaft und gedenkt ALLAHs viel, damit ihr Erfolg erlangt.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mccvi{Und gehorcht ALLAH und Seinem Gesandten und streitet euch nicht, damit ihr nicht versagt und eure Macht verloren geht. Und übt euch in Geduld! Gewiß, ALLAH ist mit den sich in Geduld Übenden.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mccvii{Und seid nicht wie diejenigen, die ihre Wohnstätten verlassen haben aus Übermut und Riyaa den Menschen gegenüber und damit sie von ALLAHs Weg abzuhalten. ALLAH ist dessen, was sie tun, allumfassend.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mccviii{Und erinnere daran), als der Satan ihnen ihre Handlungen als schön erscheinen ließ und sagte: ``Keiner unter den Menschen kann euch heute bezwingen und gewiß ich bin für euch ein Unterstützer." Doch als die beiden Gruppen aufeinander trafen, kehrte er auf seinen Fersen um und sagte: ``Gewiß, ich sage mich los von euch. Ich sehe, was ihr nicht seht. Ich fürchte ALLAH." Und ALLAH ist hart im Strafen.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mccix{Als die Munafiq und diejenigen, in deren Herzen Krankheit ist, sagten: ``Ihr Din hat diese überheblich gemacht." Und wer Tawakkul ALLAH gegenüber übt, so ist ALLAH gewiß allwürdig, allweise.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mccx{Und würdest du nur sehen, wenn die Kafir von den Engeln in den Tod befördert werden: sie schlagen ihnen auf die Vorder- und Rückenpartie und (sagen): ``Kostet die Peinigung des Verbrennens!\qt@no{50}}%
\qt@newcmd\qurantrans@de@mccxi{Dies ist für das, was ihr euch erworben habt. Und gewiß ist ALLAH den Dienern in keiner Weise ein Unrecht- Zufügender."\qt@no{51}}%
\qt@newcmd\qurantrans@de@mccxii{Ihre Vorgehensweise) war wie die der Leute von Pharao und derjenigen vor ihnen: sie haben Kufr ALLAHs Ayat gegenüber betrieben, dann hat ER sie aufgrund ihrer Verfehlungen vernichtet. Gewiß, ALLAH ist allkraftvoll, hart im Strafen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mccxiii{Es ist so! Denn ALLAH wandelt keine Gabe (in Bestrafung) um, die ER Leuten beschert hat, bis sie sich selbst (zum Schlechten) verändern. Und ALLAH ist gewiß allhörend, allwissend.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mccxiv{Ihre Vorgehensweise) ist wie die der Leute von Pharao und derjenigen vor ihnen: sie haben ALLAHs Ayat verleugnet, dann haben WIR sie aufgrund ihrer Verfehlungen vernichtet. Und die Leute von Pharao haben WIR ertrinken lassen. Und allesamt waren Unrecht-Begehende.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mccxv{Gewiß, die Übelsten der sich bewegenden Lebewesen, sind bei ALLAH diejenigen, die Kufr betrieben haben, so verinnerlichen sie den Iman nicht.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mccxvi{Diese sind) diejenigen von ihnen, mit denen du einen Vertrag abgeschlossen hast, und sie dann ihren Vertrag ständig verletzen und nicht Taqwa gemäß handeln.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mccxvii{Und solltest du sie im Krieg unterwerfen, dann laß durch (die Härte gegen) sie diejenigen, die hinter ihnen stehen, auseinander stieben, damit sie sich besinnen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mccxviii{Und solltest du von Leuten Verrat fürchten, dann löse mit ihnen den Vertrag ebenfalls auf! Gewiß, ALLAH liebt nicht die Verräter.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mccxix{Und diejenigen, die Kufr betrieben haben, sollen nicht denken, daß sie (Uns) entkamen. Gewiß, sie machen (Uns) nicht zu schaffen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mccxx{Und bereitet euch ihnen gegenüber mit allem vor, was euch an Macht und an (für den Kampf vorgesehenen) Pferden zur Verfügung steht, um damit die Feinde ALLAHs, eure Feinde und andere außer ihnen, die ihr nicht kennt aber ALLAH kennt, abzuschrecken. Und was immer ihr fi-sabilillah spendet, wird euch zurückerstattet, und ihr werdet dabei keine Vorenthaltung erfahren.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mccxxi{Und sollten sie dem Salam zugeneigt sein, dann sei ihm ebenfalls zugeneigt und übe ALLAH gegenüber Tawakkul! ER ist gewiß Der Allhörende, Der Allwissende.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mccxxii{Und sollten sie dich damit betrügen wollen, so genügt dir doch ALLAH. ER war Derjenige, Der dich mit Seinem Sieg und mit den Mumin gestärkt hat.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mccxxiii{Und ER hat ihre Herzen vereint. Hättest du alles, was auf der Erde ist, dafür ausgegeben, hättest du ihre Herzen nicht vereint! Doch ALLAH hat sie vereint. Gewiß, ER ist allwürdig, allweise.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mccxxiv{Prophet! Dir genügen ALLAH und diejenigen, die dir von den Mumin gefolgt sind.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mccxxv{Prophet! Ermuntere die Mumin zum bewaffneten Kampf! Wenn es unter euch zwanzig gibt, die sich in Geduld üben, besiegen sie zweihundert. Und wenn es unter euch einhundert gibt, besiegen sie eintausend von denjenigen, die Kufr betrieben haben, weil sie Leute sind, die nicht begreifen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mccxxvi{Jetzt hat ALLAH euch Erleichterung gewährt. Und ER wußte, daß in euch Schwäche steckt. Wenn es unter euch einhundert gibt, die sich in Geduld üben, besiegen sie zweihundert. Und wenn es unter euch eintausend gibt, besiegen sie zweitausend - mit ALLAHs Zustimmung. Und ALLAH ist mit den sich in Geduld Übenden.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mccxxvii{Es gebührt keinem Propheten, Gefangene zu machen, bevor er (den Feinden) im Lande empfindliche Verluste zugefügt hat. Ihr strebt das Vergängliche vom Diesseits an und ALLAH will (für euch) das Jenseits. Und ALLAH ist allwürdig, allweise.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mccxxviii{Und gäbe es keine Bestimmung von ALLAH, die vorher bestand, hätte euch aufgrund eurer Annahme (des Lösegeldes) doch eine qualvolle Peinigung getroffen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mccxxix{So eignet euch von dem als gutes Halal an, was ihr an Beutegütern gemacht habt, und handelt Taqwa gemäß ALLAH gegenüber! Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mccxxx{Prophet! Sag denjenigen von den Gefangenen, über die ihr verfügt: ``Wenn ALLAH in euren Herzen Gutes feststellt, wird ER euch Besseres als das zuteil werden, was euch genommen wurde, und euch vergeben. Und ALLAH ist allvergebend, allgnädig."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mccxxxi{Doch sollten sie dich verraten wollen, so haben sie ALLAH bereits vorher verraten, dann hat ER sie unterwerfen lassen. Und ALLAH ist allwissend, allweise.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mccxxxii{Gewiß, diejenigen, die den Iman verinnerlicht, Hidschra unternommen, Dschihad mit ihrem Vermögen und mit sich selbst fi-sabilillah geleistet haben, und diejenigen, die (ihnen) Unterkunft gewährt und zum Sieg verholfen haben, diese sind einander Wali. Und diejenigen, die den Iman verinnerlicht, aber keine Hidschra unternommen haben, ihnen gegenüber habt ihr keinerlei Verpflichtungen alsWali, bis sie Hidschra unternommen haben. Sollten sie euch jedoch um Beistand in Din (-Angelegenheiten) bitten, dann habt ihr ihnen beizustehen, es sei denn gegen Leute, mit denen ihr einen Vertrag abgeschlossen habt. Und ALLAH ist allsehend dessen, was ihr tut.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mccxxxiii{Und diejenigen, die Kufr betrieben haben, sind einander Wali. Wenn ihr euch daran nicht haltet, wird es auf Erden eine Fitna und großes Übel geben.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mccxxxiv{Und diejenigen, die den Iman verinnerlicht, Hidschra unternommen, Dschihad fi-sabilillah geleistet haben, und diejenigen, die (ihnen) Unterkunft gewährt und zum Sieg verholfen haben, diese sind die wahren Mumin. Für sie gibt es Vergebung und edles Rizq.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mccxxxv{Auch diejenigen, die erst danach den Iman verinnerlicht, Hidschra unternommen und mit euch Dschihad geleistet haben, diese gehören zu euch. Und die Verwandten haben untereinander nach ALLAHs Schrift das Vorrecht (in der Erbschaft). Gewiß, ALLAH ist über alles allwissend.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mccxxxvi{\basmalahde (Dies ist) eine Lossagung durch ALLAH und Seinen Gesandten von denjenigen Muschrik, mit denen ihr ein Abkommen geschlossen habt.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mccxxxvii{Also bewegt euch (sicher) im Lande vier Monate lang, aber wißt, daß ihr ALLAH keineswegs zu schaffen macht und daß ALLAH gewiß die Kafir erniedrigen wird.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mccxxxviii{Und (dies ist) eine Ankündigung von ALLAH und Seinem Gesandten für alle Menschen am höchsten Hadsch-Tag, daß ALLAH Sich doch von den Muschrik losgesagt hat, sowie Sein Gesandter. Und wenn ihr bereut, so ist dies besser für euch. Doch solltet ihr euch abwenden, so wißt, daß ihr keineswegs ALLAH zu schaffen macht. Auch überbringe den Kafir die ``frohe Botschaft" einer qualvollen Peinigung!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mccxxxix{Ausgenommen davon sind diejenigen von den Muschrik, mit denen ihr ein Abkommen geschlossen habt, und die dann euch gegenüber in keiner Weise vertragsbrüchig wurden und niemanden (von euren Feinden) gegen euch unterstützt haben, für diese erfüllt ihr ihren Vertrag bis zu dem mit ihnen vereinbarten Ende. Gewiß, ALLAH liebt die Muttaqi.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mccxl{Wenn jedoch die Haram-Monate vergangen sind, dann tötet die (vertragsbrüchigen) Muschrik, überall wo ihr sie antrefft, nehmt sie gefangen, belagert sie und überwacht sie überall! Wenn sie jedoch bereut, das rituelle Gebet ordnungsgemäß verrichtet und die Zakat entrichtet haben, dann lasst von ihnen ab! Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mccxli{Und sollte einer von den (vertragsbrüchigen) Muschrik dich um Asyl bitten, dann gewähre ihm Asyl, bis er die Worte von ALLAH vernommen hat, dann geleite ihn zum Ort seiner Sicherheit. Dies ist so, weil sie doch Menschen sind, die nicht wissen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mccxlii{Weshalb sollten die Muschrik über einen (Schutz-)Vertrag bei ALLAH und bei Seinem Gesandten verfügen?! Ausgenommen sind diejenigen, mit denen ihr einen Vertrag bei Almasdschidil-haram abgeschlossen habt - so lange sie euch gegenüber vertragstreu sind, so seid ihnen gegenüber vertragstreu! Gewiß, ALLAH liebt die Muttaqi.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mccxliii{Weshalb (sollten sie einen Vertrag haben?!), denn wenn sie die Oberhand über euch gewinnen würden, würden sie euch gegenüber weder Verwandtschaft noch vertragliche Vereinbarungen beachten. Sie stellen euch mit ihrem Gerede zufrieden, während ihre Herzen dies ablehnen. Und die meisten von ihnen sind Fasiq.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mccxliv{Sie erkauften sich mit ALLAHs Ayat Minderwertiges, so haben sie damit von Seinem Weg abgehalten. Gewiß, es war schlimm, was sie zu tun pflegten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mccxlv{Sie beachten einem Mumin gegenüber weder Verwandtschaft noch vertragliche Vereinbarungen und diese sind die eigentlichen Übertretenden.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mccxlvi{Wenn sie dennoch bereut, das rituelle Gebet ordnungsgemäß verrichtet und die Zakat entrichtet haben, so sind sie eure Geschwister im Din. Und WIR erläutern die Ayat für Menschen, die wissen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mccxlvii{Doch sollten sie ihre Eide gebrochen haben, nachdem sie ihre Verträge abgeschlossen hatten, und eurem Din gegenüber Beleidigungen ausgesprochen haben, dann führt den bewaffneten Kampf gegen die Imame des Kufr - denn gewiß sie achten die Eide nicht -, damit sie aufhören.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mccxlviii{Wollt ihr etwa nicht den bewaffneten Kampf gegen Menschen führen, die ihre Eide gebrochen haben und bestrebt waren, den Gesandten zu vertreiben  und diejenigen waren, die euch zuerst angegriffen haben?! Fürchtet ihr sie etwa?! So ist ALLAH würdiger, von euch gefürchtet zu werden, wäret ihr Mumin.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mccxlix{Führt gegen sie den bewaffneten Kampf! ALLAH peinigt sie durch euch, erniedrigt sie, läßt euch über sie siegen und läßtMenschen, dieMumin sind, danach Genugtuung empfinden,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mccl{und läßt den Zorn ihrer Herzen sich legen. Und ALLAH läßt bereuen, wen ER will. Und ALLAH ist allwissend, allweise.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mccli{Oder habt ihr etwa gedacht, daß ihr in Ruhe gelassen würdet, ohne daß ALLAH diejenigen unter euch kenntlich macht, die Dschihad geleistet und außer ALLAH, Seinem Gesandten und den Mumin keine Vertraute genommen haben?! Und ALLAH ist dessen allkundig, was ihr tut.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mcclii{Es gebührt sich nicht für die Muschrik, daß sie die Moscheen ALLAHs zum ``Gottesdienst aufsuchen, während sie Kufr sich selbst gegenüber bezeugen. Diesen wurden ihre Werke zunichte gemacht und im Feuer werden sie ewig bleiben.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mccliii{Ausschließlich sucht die Moscheen ALLAHs zum Gottesdienst derjenige auf, der den Iman an ALLAH und an den Jüngsten Tag verinnerlicht, das Gebet ordnungsgemäß verrichtet, die Zakat entrichtet und keinen außer ALLAH fürchtet. Diese mögen unter den Rechtgeleiteten sein.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mccliv{Wollt ihr etwa (denjenigen, der sichert) die Versorgung der Wallfahrer mit Trinkwasser und das Aufsuchen von Almasdschidil-haram zum Gottesdienst, gleichsetzen mit demjenigen, der den Iman an ALLAH und an den Jüngsten Tag verinnerlicht und Dschihad fi-sabilillah leistet?! Sie sind bei ALLAH nicht gleich! ALLAH leitet nicht diejenigen Menschen recht, die Unrecht begehen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mcclv{Die-jenigen, die den Iman verinnerlicht, Hidschra unternommen und Dschihad fisabilillah mit ihrem Vermögen und mit sich selbst geleistet haben, genießen eine höhere Stellung bei ALLAH. Und diese sind die wirklichen Erfolgreichen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mcclvi{Ihr HERR überbringt ihnen frohe Botschaft: eine Gnade von Ihm, Zufriedenheit und Dschannat, in denen für sie beständige Wonne bestimmt ist.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mcclvii{Ewig verbleiben sie in ihnen - für immer. Gewiß, ALLAH hat bei sich unermeßliche Belohnung.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mcclviii{Ihr, die den Iman verinnerlicht habt! Nehmt euch eure Eltern und eure Geschwister nicht als Wali, wenn sie den Kufr dem Iman vorziehen. Und wer von euch sie als Wali nimmt, diese sind die wirklichen Unrecht-Begehenden.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mcclix{Sag: ``Sollten euch eure Eltern, eure Kinder, eure Geschwister, eure Ehepartner, euer Stamm, Vermögenswerte, die ihr erworben habt, Handel, dessen Stagnation ihr fürchtet, und von euch bevorzugte Wohnstätten lieber sein als ALLAH, Sein Gesandter und Dschihad für Ihn, so wartet ab, bis ALLAH Seine Bestimmung vollzieht." ALLAH leitet die fisq-betreibenden Leute nicht recht.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mcclx{Gewiß, bereits hat ALLAH euch den Sieg an vielen Kampfplätzen verliehen sowie am Kampftag von Hunain, als euch eure Überzahl stolz machte, aber euch nichts nützte und das Land trotz seiner Weite euch eng erschien und ihr dann flüchtend den Rücken kehrtet.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcclxi{Dann sandte ALLAH Seinem Gesandten und den Mumin Seine beruhigende Gnade hinab und sandte Kämpfer hinab, die ihr nicht gesehen habt, und peinigte diejenigen, die Kufr betrieben haben. Und dies ist die Vergeltung für die Kafir.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mcclxii{Dann läßt ALLAH danach bereuen wen ER will. Und ALLAH ist allvergebend, allgnädig.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mcclxiii{Ihr, die den Iman verinnerlicht habt! Die Muschrik sind wie die rituelle Unreinheit, so laßt sie, nach diesem ihrem Jahr, nicht in die Nähe von Almasdschidilharam kommen! Und wenn ihr (deswegen) Armut fürchtet, so wird ALLAH euch von Seiner Gunst genügen, wenn ER will. Gewiß, ALLAH ist allwissend, allweise.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mcclxiv{Führt den bewaffneten Kampf gegen diejenigen, die weder den Iman an ALLAH, noch an den Jüngsten Tag verinnerlichen, noch das für haram erklären, was ALLAH und Sein Gesandter für haram erklärten, noch sich dem wahren Din fügen von denjenigen, denen die Schrift zuteil wurde, bis sie die Dschizya in Ergebenheit entrichten, während sie unterwürfig sind.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcclxv{Und die Juden sagten: ``Uzair ist ALLAHs Sohn. Und die Nazarener sagten: ``Almasih ist ALLAHs Sohn." Dies ist lediglich ihre Aussage mit ihren Mündern. Sie ähneln (damit) der Aussage derjenigen, die vorher Kufr betrieben haben. ALLAH möge sie zugrunde richten, wie sie (andere) irreführen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcclxvi{Sie nahmen ihre Rabbiner und Priester als Herren anstelle von ALLAH sowie Almasih Ibnu- Maryam. Ihnen wurde jedoch nur geboten, einem einzigen Gott zu dienen. Es gibt keine Gottheit außer Ihm. Gepriesen-erhaben ist ER über das, was sie an Schirk betreiben.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcclxvii{Sie wollen ALLAHs Licht mit ihren Mündern auslöschen. Doch ALLAH lehnt es ab, Sein Licht nicht zu vervollständigen, selbst dann, würden die Kafir demgegenüber abgeneigt sein.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcclxviii{ER ist Derjenige, Der Seinen Gesandten mit der Rechtleitung und dem Din der Wahrheit entsandte, um ihn sämtliche Din übertreffen zu lassen, selbst dann, würden die Muschrik demgegenüber abgeneigt sein.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcclxix{Ihr, die den Iman verinnerlicht habt! Zweifelsohne eignen sich viele von den Rabbinern und Priestern Vermögenswerte von Menschen durch das für nichtig Erklärte an und halten vom Weg ALLAHs ab. Und denjenigen, die Gold und Silber horten und nicht fi-sabilillah ausgeben, überbringe die ``frohe Botschaft" einer qualvollen Peinigung!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcclxx{Am Tag, an dem sie (Gold und Silber) im Feuer von Dschahannam erhitzt werden, dann damit ihre Stirnen, ihre Seiten und ihre Rücken gebrandmarkt werden. Dies ist, was ihr für euch selbst gehortet habt, also erfahrt, was ihr gehortet habt.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcclxxi{Gewiß, die Anzahl der Monate beträgt in ALLAHs Schrift zwölf Monate, (so war es) am Tag, als ER die Himmel und die Erde erschuf - davon sind vier Haram-Monate. Dies ist der rechte Din. So begeht in ihnen kein Unrecht gegen euch selbst! Und führt den bewaffneten Kampf gegen die Muschrik allesamt, wie sie den bewaffneten Kampf gegen euch allesamt führen. Und wisst, daß ALLAH den Muttaqi gewiß beisteht.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcclxxii{Das Verschieben (eines Haram-Monats) ist eine Steigerung von Kufr. Damit werden diejenigen irregeleitet, die Kufr betrieben haben. Sie erklären ihn in einem Jahr für einen Nicht-Haram-Monat und in einem anderen Jahr für einen Haram-Monat, um der Anzahl der von ALLAH festgelegten Haram-Monate zu entsprechen und dann für halal zu erklären, was ALLAH für haram erklärte. Ihnen wurde das Schlechte ihrer Handlungen als schön dargeboten. ALLAH leitet die Kafir-Leute nicht recht.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcclxxiii{Ihr, die den Iman verinnerlicht habt! Weshalb, wenn euch gesagt wird: ``Brecht auf fi-sabilillah", werdet ihr träge an der Stelle haftend?! Seid ihr etwa zufrieden mit dem diesseitigen Leben anstelle des Jenseits?! Die Freuden und Gebrauchsgüter des diesseitigen Lebens sind neben dem Jenseits sicherlich unbedeutend.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcclxxiv{Wenn ihr nicht (fi-sabilillah) aufbrechen wollt, wird ER euch einer qualvollen Peinigung aussetzen und euch durch andere Leute ersetzen - und ihr werdet Ihm in keiner Weise schaden. Und ALLAH ist über alles allmächtig.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcclxxv{Wenn ihr ihm (dem Gesandten) nicht beistehen wollt, (so wird ALLAH ihm beistehen), wie ALLAH ihm beistand, als ihn diejenigen, die Kufr betrieben haben, vertrieben haben als einen von Zweien, als beide in der Höhle waren, als er seinem Gefährten sagte: ``Sei nicht traurig! Gewiß, ALLAH ist mit uns." Dann ließ ALLAH (ihnen) Seine beruhigende Gnade zuteil werden und unterstützte ihn mit Kämpfern, die ihr nicht gesehen habt, und hat das Wort derjenigen, die Kufr betrieben haben, als das Niedrige gemacht. Und das Wort ALLAHs ist das Erhobene! Und ALLAH ist allwürdig, allweise.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcclxxvi{Brecht auf als Leichte oder Schwere und leistet Dschihad fi-sabilillah mit eurem Vermögen und mit euch selbst! Dies ist besser für euch, solltet ihr wissen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mcclxxvii{Wäre dies ein demnächst verfügbarer Profit mit einer Reise von mittlerer Entfernung gewesen, wären sie dir gewiß gefolgt. Jedoch erschien ihnen die beschwerliche Reise zu weit. Und sie werden in ALLAHs Namen schwören: ``Hätten wir es nur vermocht, gewiß wären wir mit euch ausgezogen." Sie richten sich damit zugrunde und ALLAH weiß, daß sie gewiß doch Lügner sind.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcclxxviii{ALLAH hat dir vergeben! Weshalb hast du ihnen die Erlaubnis (nicht auszuziehen) erteilt? (Hättest du doch gewartet), bis dir diejenigen ersichtlich werden, welche die Wahrheit gesprochen haben und du die Lügner erkennst!\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcclxxix{Um Erlaubnis bitten dich nicht diejenigen, die den Iman an ALLAH und an den Jüngsten Tag verinnerlicht haben, um keinen Dschihad mit ihrem Vermögen und sich selbst zu leisten! Und ALLAH ist allwissend über die Muttaqi.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mcclxxx{Dich um Erlaubnis (zurückzubleiben) bitten nur diejenigen, die keinen Iman an ALLAH und an den Jüngsten Tag verinnerlicht haben und deren Herzen voller Zweifel sind, so daß sie durch ihren Zweifel hin- und herschwanken.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mcclxxxi{Und hätten sie den Aufbruch gewollt, gewiß hätten sie dafür das Notwendige vorbereitet. Doch ALLAH verabscheute ihren Aufbruch, so hat ER sie (noch mehr) entmutigt und es wurde gesagt: ``Bleibt zurück (als Drückeberger) mit den Zurückbleibenden!"\qt@no{46}}%
\qt@newcmd\qurantrans@de@mcclxxxii{Und wären sie mit euch aufgebrochen, hätten sie euch außer Schlechtem nichts hinzugefügt und sich beeilt, Fitna unter euch zu verbreiten. Und unter euch sind welche, die ihnen zuhören. Und ALLAH ist allwissend über die Unrecht- Begehenden.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mcclxxxiii{Gewiß, bereits waren sie vorher um Fitna bemüht und wollten deine Angelegenheiten (durch Intrigen) durcheinander bringen, bis dieWahrheit kam und die Sache ALLAHs siegte, während sie (diesem) abgeneigt sind.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mcclxxxiv{Und manch einer von ihnen sagt: ``Gewähre mir Erlaubnis (zurückzubleiben) und setze mich keiner Fitna aus!" Doch sie sind bereits in die Fitna gefallen. Und gewiß Dschahannam umfaßt unweigerlich die Kafir.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mcclxxxv{Wenn dich Gutes trifft, mißfällt es ihnen. Und wenn dich Unglück trifft, sagen sie: ``Bereits haben wir vorher für uns Vorkehrungen getroffen." Dann wenden sie sich ab, während sie glücklich sind.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mcclxxxvi{Sag: ``Uns wird nur das treffen, was ALLAH für uns niedergeschrieben hat. ER ist unser Maula!" Und ALLAH gegenüber sollen die Mumin Tawakkul üben!\qt@no{51}}%
\qt@newcmd\qurantrans@de@mcclxxxvii{Sag: ``Erwartet ihr für uns etwas anderes außer eines der beiden Schönen (Sieg oder Schahada fi-sabilillah)?" Und wir erwarten für euch, daß ALLAH euch mit einer Peinigung trifft entweder (direkt) von Sich oder durch uns. Also wartet ab, gewiß, wir warten ebenfalls mit euch ab.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mcclxxxviii{Sag: ``Spendet freiwillig oder widerwillig, dies wird von euch nie angenommen. Gewiß, ihr wart fisq-betreibende Leute."\qt@no{53}}%
\qt@newcmd\qurantrans@de@mcclxxxix{Und nichts stand ihnen im Wege, damit ihre Spenden angenommen werden, außer daß sie Kufr ALLAH und Seinem Gesandten gegenüber betrieben haben, zum rituellen Gebet nur widerstrebend kommen und nur spenden, während sie dem abgeneigt sind.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mccxc{Also finde keinen Gefallen weder an ihrem Vermögen noch an ihren Kindern! ALLAH will sie damit im diesseitigen Leben doch nur peinigen und (will), daß ihre Seelen (ihren Körper) beschwerlich verlassen, während sie noch Kafir sind.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mccxci{Und sie schwören bei ALLAH: ``Gewiß sie seien doch von euch", obwohl sie nicht zu euch gehören. Doch diese sind Leute, die Angst (vor euch) haben.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mccxcii{Würden sie einen Zufluchtsort, Höhlen oder Unterschlupf finden, gewiß würden sie sich dorthin begeben, während sie überhastig sind.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mccxciii{Und manche von ihnen bezichtigen dich (ungerechter Verteilung) der Sadaqa. Doch würde ihnen davon etwas gegeben, wären sie zufrieden. Aber wenn ihnen davon nichts gegeben wird, so sind sie wütend.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mccxciv{Und wären sie doch zufrieden damit, was ALLAH und Sein Gesandter ihnen zuteil werden ließen, und hätten sie doch gesagt: ``Uns genügt ALLAH. ALLAH wird uns von Seiner Gunst zuteil werden lassen sowie Sein Gesandter. Wir sind ALLAH gegenüber Bittsteller." (So wäre dies gewiß besser für sie).\qt@no{59}}%
\qt@newcmd\qurantrans@de@mccxcv{Die Zakat-Mittel sind ausschließlich bestimmt für die Mittellosen, für die Bedürftigen, für die Zakat-Beauftragten, für diejenigen, deren Herzen gewonnen werden sollen, für die (Befreiung von) Unfreien, für die (Entschuldung der) Schuldner, fi-sabilillah und für den (in Not geratenen) Reisenden. (Dies ist) eine Verpflichtung von ALLAH, und ALLAH ist allwissend, allweise.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mccxcvi{Und manche von ihnen sind diejenigen, die den Propheten belästigen und sagen: ``Er ist (wie) ein Ohr." Sag: ``Ja, ein Ohr des Guten für euch, er verinnerlicht den Iman an ALLAH und glaubt den Mumin und er ist eine Gnade für diejenigen von euch, die den Iman verinnerlicht haben." Und diejenigen, die ALLAHs Gesandten belästigen, für diese ist eine qualvolle Peinigung bestimmt.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mccxcvii{Sie schwören euch bei ALLAH, damit sie euch zufriedenstellen, während ALLAH und Sein Gesandter doch berechtigter sind, zufriedengestellt zu werden, sollten sie Mumin sein.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mccxcviii{Haben sie etwa nicht gewußt, daß für denjenigen, der ALLAH und Seinem Gesandten entgegenwirkt, doch das Feuer von Dschahannam bestimmt ist  als Ewiger darin?! Dies ist die übergroße Erniedrigung.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mccxcix{Die Munafiq fürchten, daß ihnen eine Sura hinabgesandt wird, die sie darüber in Kenntnis setzt, was in ihren Herzen ist. Sag: ``Spottet nur! Gewiß, ALLAH wird das hervorbringen, was ihr fürchtet."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mccc{Und würdest du sie (danach) fragen, würden sie sagen: ``Wir haben uns doch nur unterhalten und Unfug getrieben." Sag: ``Etwa über ALLAH, Seine Ayat und Seinen Gesandten pflegtet ihr zu spotten?!"\qt@no{65}}%
\qt@newcmd\qurantrans@de@mccci{Entschuldigt euch nicht, ihr habt bereits Kufr nach eurem Iman betrieben! Würden WIR einer Gruppe von euch vergeben, so würden WIR eine andere Gruppe peinigen, weil sie doch schwer Verfehlende zu sein pflegten.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mcccii{Die Munafiq-Männer und die Munafiq-Frauen, die einen von ihnen sind wie die anderen. Sie rufen zum Mißbilligten auf und raten vom Gebilligten ab und sind geizig. Sie haben ALLAH vergessen, so ließ ER sie vergessen werden. Gewiß, die Munafiq sind die wirklichen Fasiq.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mccciii{ALLAH versprach den Munafiq-Männern, den Munafiq-Frauen und den Kafir das Feuer von Dschahannam - darin werden sie ewig bleiben. Es genügt ihnen. Und ALLAH hat sie verflucht, und für sie ist eine bleibende Peinigung bestimmt.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mccciv{Ihr seid) wie diejenigen vor euch. Sie waren noch mächtiger als ihr und hatten noch mehr Vermögen und Kinder. So genossen sie, was ihnen bestimmt war, auch habt ihr genossen, was euch bestimmt war genauso, wie diejenigen vor euch das genossen haben, was ihnen bestimmt war. Auch habt ihr Unfug getrieben genauso, wie sie Unfug getrieben haben. Diesen sind ihre Werke im Diesseits und Jenseits zunichte geworden und diese sind die wirklichen Verlierer.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mcccv{Kam zu ihnen etwa nicht dieMitteilung über diejenigen vor ihnen: die Leute von Nuh, 'Aad, Thamud, die Leute von Ibrahim, die Bewohner von Madyan und von den Umgewandten?! Ihre Gesandten kamen zu ihnen mit den deutlichen Zeichen, so gebührt ALLAH niemals, ihnen Unrecht anzutun. Doch sie pflegten sich selbst Unrecht anzutun.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mcccvi{DieMumin-Männer und dieMumin-Frauen sind einander Wali. Sie rufen zum Gebilligten auf, raten vom Mißbilligten ab, verrichten ordnungsgemäß das rituelle Gebet, entrichten die Zakat und gehorchen ALLAH und Seinem Gesandten. Diesen wird ALLAH Gnade erweisen. Gewiß, ALLAH ist allwürdig, allweise.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mcccvii{ALLAH hat den Mumin-Männern und den Mumin-Frauen Dschannat versprochen, die von Flüssen durchflossen werden - darin werden sie ewig bleiben, und behagliche Wohnstätten in den Dschannat von 'Adn sowie Wohlgefallen von ALLAH, das noch größer ist. Dies ist der unermeßliche Erfolg.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mcccviii{Prophet! Leiste Dschihad gegen die Kafir und die Munafiq und begegne ihnen mit Härte. Ihre Unterkunft ist Dschahannam und erbärmlich ist dieses Werden.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mcccix{Sie schwören bei ALLAH, daß sie (es ) nicht sagten. Und gewiß, bereits sprachen sie das Kufr-Wort und betrieben Kufr, nachdem sie den Islam angenommen hatten, und leiteten etwas in die Wege, was ihnen nicht gelungen ist. Und nichts erfüllte sie mit Haß, außer daß ALLAH und Sein Gesandter sie von Seiner Gunst reich machte. Doch wenn sie bereuen, ist das besser für sie. Aber wenn sie sich abwenden, wird ALLAH sie einer qualvollen Peinigung im Diesseits und im Jenseits aussetzen. Und sie finden auf der Erde weder Wali noch Beistehenden.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mcccx{Und manche von ihnen versprachen ALLAH: ``Ließe ER uns von Seiner Gunst zuteil werden, so würden wir gewiß spenden und doch von den gottgefällig Guttuenden sein!"\qt@no{75}}%
\qt@newcmd\qurantrans@de@mcccxi{Und nachdem ER ihnen von Seiner Gunst hatte zuteil werden lassen, geizten sie damit und wandten sich ab, während sie ablehnend waren.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mcccxii{So hat ER in ihre Herzen Nifaq einziehen lassen - bis zu dem Tag, an dem sie Ihm begegnen werden -, weil sie ALLAH gegenüber das verletzten, was sie Ihm versprachen, und wegen dem, was sie zu lügen pflegten.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mcccxiii{Wußten sie etwa nicht, daß ALLAH gewiß ihre Heimlichkeiten und ihre Tuscheleien genau kennt, und daß ALLAH gewiß Der Allwissende über das Verborgene ist?!\qt@no{78}}%
\qt@newcmd\qurantrans@de@mcccxiv{Diejenigen, welche die freiwillig Spendenden unter den Mumin wegen ihrer Spenden schmähen - solche (Mumin), die nur das spenden, was sie vermögen, und deshalb diese sich über sie lustig machen - ALLAH machte sie lächerlich und für sie ist eine qualvolle Peinigung bestimmt.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mcccxv{Ob du für sie um Vergebung bittest oder nicht bittest, selbst dann, solltest du siebzig Mal für sie um Vergebung bitten, ALLAH wird ihnen sicher nie vergeben. Dies ist so, weil sie Kufr ALLAH und Seinem Gesandten gegenüber betrieben haben. Und ALLAH leitet die fisqbetreibenden Leute nicht recht.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mcccxvi{Die (als Drückeberger) Zurückgebliebenen freuten sich über ihr Zurückbleiben in Auflehnung gegen den Gesandten ALLAHs, lehnten ab, fi-sabilillah mit ihrem Vermögen und mit sich selbst Dschihad zu leisten und sagten: ``Brecht bei der Hitze nicht auf!" Sag: ``Das Feuer von Dschahannam ist noch heißer." Würden sie es doch nur begreifen!\qt@no{81}}%
\qt@newcmd\qurantrans@de@mcccxvii{So sollen sie sich ein wenig freuen, aber dann sollen sie viel weinen als Vergeltung für das, was sie (an Verfehlungen) zu erwerben pflegten.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mcccxviii{Und wenn ALLAH dich zu einer Gruppe von ihnen wiederkehren läßt und diese dich um Erlaubnis (zur Teilnahme) am Aufbruch bitten, dann sag: ``Ihr werdet mit mir nie und nimmer aufbrechen und den bewaffneten Kampf gegen keinen Feind mit mir aufnehmen. Ihr wart das erste Mal doch mit dem Zurückbleiben einverstanden, so bleibt (als Drückeberger) mit denjenigen zurück, die zurückbleiben!"\qt@no{83}}%
\qt@newcmd\qurantrans@de@mcccxix{Und verrichte niemals für einen von ihnen, der verstarb, das rituelle Totengebet und stehe nicht an seinem Grab! Gewiß, sie haben Kufr ALLAH und Seinem Gesandten gegenüber betrieben und sind verstorben, während sie noch Fasiq sind.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mcccxx{Auch finde keinen Gefallen an ihrem Vermögen und an ihren Kindern! ALLAH will sie damit im Diesseits nur peinigen und (will), daß ihre Seelen (ihren Körper) beschwerlich verlassen, während sie noch Kafir sind.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mcccxxi{Und als eine Sura hinabgesandt wurde: ``Verinnerlicht den Iman an ALLAH und leistet Dschihad mit Seinem Gesandten", baten dich die Vermögenden unter ihnen um Erlaubnis und sagten: ``Laß uns unter den Zurückbleibenden sein!"\qt@no{86}}%
\qt@newcmd\qurantrans@de@mcccxxii{Sie waren einverstanden, mit den Zurückgebliebenen zu sein. Und ihre Herzen wurden versiegelt, so begreifen sie nicht.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mcccxxiii{Doch der Gesandte und diejenigen, die den Iman mit ihm verinnerlichten, haben Dschihad mit ihrem Vermögen und mit sich selbst geleistet. Für diese sind die guten Dinge bestimmt und diese sind die wirklichen Erfolgreichen.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mcccxxiv{ALLAH hat für sie Dschannat vorbereitet, die von Flüssen durchflossen werden. Darin werden sie ewig bleiben. Dies ist der unermeßliche Gewinn.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mcccxxv{Und die mit einer Entschuldigung Täuschenden von den Wüstenarabern kamen, damit ihnen Erlaubnis erteilt wird. Und diejenigen, die ALLAH und Seinen Gesandten belogen, blieben (als Drückeberger) zurück. Diejenigen, die Kufr betrieben haben unter ihnen wird eine qualvolle Peinigung treffen.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mcccxxvi{Weder für die Schwachen, noch für die Kranken, noch für diejenigen, die nichts zum Spenden finden, ist es eine Verfehlung, wenn sie ALLAH und Seinem Gesandten gegenüber aufrichtig waren. Weder gegen die Muhsin gibt es etwas (zu mißbilligen) - und ALLAH ist allvergebend, allgnädig -\qt@no{91}}%
\qt@newcmd\qurantrans@de@mcccxxvii{noch gegen diejenigen, die zu dir kamen, damit du ihnen Transportmittel bereitstellst, (zu denen) du jedoch sagtest: ``Ich finde nichts (an Reittieren), worauf ich euch befördern könnte", dann sie umkehrten, während ihre Augen voller Tränen waren - aus Traurigkeit darüber, daß sie nichts finden, was sie spenden können.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mcccxxviii{Etwas (zu mißbilligen), gibt es ausschließlich gegen diejenigen, die dich um Erlaubnis (zum Zurückbleiben) bitten, während sie reich sind. Sie waren einverstanden damit, unter den Zurückbleibenden zu sein, und ALLAH hat ihre Herzen versiegelt, so wissen sie nicht.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mcccxxix{Sie tragen euch ihre Entschuldigungen vor, wenn ihr zu ihnen zurückkommt. Sag: ``Tragt keine Entschuldigungen vor! Wir glauben euch niemals. Bereits ALLAH hat uns von euren Begebenheiten kundgetan. Und ALLAH wird eure Taten sehen sowie Sein Gesandter, dann werdet ihr zum Allwissenden über das Verborgene und das Sichtbare zurückgebracht, dann wird ER euch das kundtun, was ihr zu tun pflegtet.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mcccxxx{Sie werden euch im Namen ALLAHs schwören, wenn ihr zu ihnen zurückkommt, damit ihr sie nicht behelligt, so behelligt sie nicht! Gewiß, sie sind (wie) die rituelle Unreinheit und ihre Unterkunft ist Dschahannam als Vergeltung für das, was sie zu tun pflegten.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mcccxxxi{Sie schwören euch, damit ihr mit ihnen zufrieden seid. Und solltet ihr mit ihnen zufrieden werden, so ist ALLAH mit den fisq-betreibenden Leuten nie zufrieden.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mcccxxxii{Die Wüstenaraber betreiben noch mehr Kufr und Nifaq und ihnen gebührt noch mehr, die Schranken dessen nicht zu kennen, was ALLAH Seinem Ge-sandten hinabsandte. Und ALLAH ist allwissend, allweise.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcccxxxiii{Und unter den Wüstenarabern ist manch einer, der das, was er (an Sadaqa ) abgibt, als Tribut ansieht und darauf wartet, daß euch Vernichtendes trifft, das Vernichtende möge sie treffen. Und ALLAH ist allhörend, allwissend.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mcccxxxiv{Und unter den Wüstenarabern ist manch einer, der den Iman an ALLAH und an den Jüngsten Tag verinnerlicht und das, was er abgibt, als etwas (dem Wohlgefallen von) ALLAH Näher- Bringendes ansieht sowie den Bittgebeten des Gesandten. Ja! Dies ist gewiß etwas (dem Wohlgefallen ALLAHs) Näher-Bringendes für sie. ER wird sie mit Seiner Gnade umgeben. Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mcccxxxv{Und die Pioniere, die ersten von den Muhadschir, die Ansar und diejenigen, die ihnen im gottgefällig Guten gefolgt sind, mit diesen ist ALLAH zufrieden und sie sind mit Ihm zufrieden. Und ER hat für sie Dschannat vorbereitet, die von Flüssen durchflossen werden, darin bleiben sie ewig - für immer. Dies ist der unermeßliche Gewinn.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mcccxxxvi{Und unter denjenigen von den Wüstenarabern um euch herum sind Munafiq sowie unter den Bewohnern von Madina. Sie beharren auf Nifaq. Du kennst sie nicht, doch WIR kennen sie. WIR werden sie zweimal peinigen, dann werden sie wieder in eine überharte Peinigung hineingeschickt.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mcccxxxvii{Und es gibt andere, die ihre Verfehlungen zugaben: sie vermischten gottgefällig gute Tat mit einer anderen Schlechten. ALLAH möge ihnen vergeben. Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mcccxxxviii{Nimm von ihrem Vermögen eine Sadaqa, mit der du sie reinigst und (ihre Belohnung) vermehrst, und sprich für sie Bittgebete, denn dein Bittgebet ist eine Beruhigung für sie. Und ALLAH ist allhörend, allwissend.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mcccxxxix{Wußten sie etwa nicht, daß ALLAH Derjenige ist, Der die Reue von Seinen Dienern akzeptiert und die Sadaqat annimmt, und daß ALLAH gewiß Der Reue- Annehmende, Der Allgnädige ist?!\qt@no{104}}%
\qt@newcmd\qurantrans@de@mcccxl{Und sag: ``Handelt, denn ALLAH wird euer Handeln sehen sowie Sein Gesandter und die Mumin. Und ihr werdet zum Allwissenden über das Verborgene und das Sichtbare zurückgebracht, dann wird ER euch das kundtun, was ihr zu tun pflegtet."\qt@no{105}}%
\qt@newcmd\qurantrans@de@mcccxli{Und es gibt andere, die (bis zum Eintreffen) der Entscheidung ALLAHs zurückgestellt werden - entweder peinigt ER sie oder nimmt ER ihre Reue an. Und ALLAH ist allwissend, allweise.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mcccxlii{Und (zu den Mißbilligten gehören) diejenigen, die eine Moschee errichteten als Schädigung (für die Mumin), als (Verstärkung von) Kufr, als Spaltung der Mumin und als Wachturm für denjenigen, der vorher ALLAH und Seinen Gesandten bekriegt hat. Und sie werden gewiß schwören: ``Wir erstrebten nur das Beste!" Und ALLAH bezeugt, daß sie bestimmt Lügner sind.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mcccxliii{Bete in ihr nie und nimmer! Sicher, einer Moschee, die vom ersten Tag an auf Taqwa funda-mentiert wurde, gebührt es mehr, daß du in ihr betest. In ihr sind Männer, die es lieben, sich zu reinigen. Und ALLAH liebt die Sich-Reinigenden.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mcccxliv{Ist etwa diejenige, deren Bauwerk auf Taqwa von ALLAH und Zufriedenheit fundamentiert, besser oder diejenige, deren Bauwerk auf der einbruchgefährdeten Uferböschung eines Sturzbaches fundamentiert, die dann mit ihr ins Feuer von Dschahannam einbricht?! Und ALLAH leitet nicht die Unrecht-Begehenden recht.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mcccxlv{Ihr Bauwerk, das sie errichteten, wird unaufhörlich Zweifel in ihren Herzen verursachen, bis ihre Herzen (in Stücke) brechen. Und ALLAH ist allwissend, allweise.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mcccxlvi{Gewiß, ALLAH erwarb von den Mumin ihre Person und ihr Vermögen dafür, daß ihnen doch die Dschanna zuteil wird. Sie führen den bewaffneten Kampf fisabilillah, wobei sie dann töten oder getötet werden. Dies ist ein von Ihm gemachtes Versprechen, ein Wahrhaftiges - (erwähnt) in At-taurat, Alindschil und im Quran. Und wer hält sein Versprechen besser als ALLAH ein?! Also seid froh über euren Handel, mit dem ihr gehandelt habt. Und dies ist der unermeßliche Gewinn.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mcccxlvii{Diese sind) die Bereuenden, die (ALLAH) Dienenden, die (ALLAH) Lobpreisenden, die (fi-sabilillah) Umherziehenden (bzw. die Fastenden), die Ruku'-Vollziehenden, die Sudschud-Vollziehenden, die zum Gebilligten Aufrufenden und vom Mißbilligten Abratenden und die ALLAHs Richtlinien Achtenden. Und überbringe den Mumin die frohe Botschaft!\qt@no{112}}%
\qt@newcmd\qurantrans@de@mcccxlviii{Es gebührt dem Gesandten und den Mumin nicht, daß sie für dieMuschrik um Vergebung bitten - selbst dann, sollten diese auch Verwandte sein, nachdem ihnen deutlich wurde, daß diese doch die Weggenossen der Hölle sind.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mcccxlix{Und das Bitten von Ibrahim um Vergebung für seinen Vater war nur aufgrund eines Versprechens, das er ihm gegeben hatte. Doch als ihm deutlich wurde, daß dieser ALLAH gegenüber doch ein Feind war, sagte er sich von ihm los. Gewiß, Ibrahim war doch äußerst anteilnehmend, langmütig.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mcccl{Und ALLAH würde keine Gemeinde fehlgehen lassen, nachdem ER sie rechtleitete, bis ER ihr verdeutlicht, wovor sie Taqwa suchen soll. Gewiß, ALLAH ist über alles allwissend.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mcccli{Gewiß, ALLAH gehört die Herrschaft über die Himmel und die Erde. ER belebt und läßt sterben. Und für euch gibt es außer ALLAH weder Wali noch Beistehenden!\qt@no{116}}%
\qt@newcmd\qurantrans@de@mccclii{Gewiß, bereits nahm ALLAH an die Reue des Gesandten, der Muhadschir und der Ansar, die ihm in der Stunde der Not gefolgt sind, nachdem beinahe die Herzen einer Gruppe von ihnen Abneigung empfanden, dann aber nahm ER ihre Reue an. ER ist ihnen gegenüber gewiß allerbarmend, allgnädig.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mcccliii{Auch (nahm ALLAH an die Reue) derjenigen Drei, die zurückgestellt wurden. Als das Land dann trotz seiner Weite ihnen eng erschien, ihre Seelen unter Beklemmung litten und sie wußten, daß es keine Zuflucht vor ALLAH gibt außer (der reuigen Umkehr) zu Ihm, dann verhalf ER ihnen zur Reue, damit sie bereuen. Gewiß, ALLAH ist Der Reue Annehmende, Der Allgnädige.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mcccliv{Ihr, die den Iman verinnerlicht habt! Handelt ALLAH gegenüber Taqwa gemäß und seid unter den Wahrhaftigen!\qt@no{119}}%
\qt@newcmd\qurantrans@de@mccclv{Es gebührt den Madina-Bewohnern und den Wüstenarabern um sie herum nicht, daß sie dem Gesandten ALLAHs nicht Folge leisten und sich selbst ihm gegenüber bevorzugen. Dies ist so, weil sie weder großen Durst, noch Strapazen, noch großen Hunger fi-sabilillah erleiden, und in kein Gebiet einmarschieren, was die Kafir lediglich erzürnen sollte, und keinem Feind Verluste zufügen, ohne daß dieses für sie als gottgefällig gute Tat registriert wird. Gewiß, ALLAHläßt keine Belohnung der Muhsin verloren gehen.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mccclvi{Und sie geben weder eine kleine noch eine große Spende, auch durchqueren sie kein Tal, ohne daß dies für sie registriert wird, damit ALLAH sie noch besser belohnt als das, was sie zu tun pflegten.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mccclvii{Es ist nicht angebracht, daß die Mumin - alle zusammen aufbrechen. Wäre doch aus jeder Gemeinde von ihnen eine Gruppe aufgebrochen, um sich im Din zu bilden und ihre Leute zu ermahnen, wenn sie zu ihnen zurückkehren, damit diese sich vorsehen!\qt@no{122}}%
\qt@newcmd\qurantrans@de@mccclviii{Ihr, die den Iman verinnerlicht habt! Führt den bewaffneten Kampf gegen die Kafir, die in eurem unmittelbaren Umkreis sind, und sie sollen von euch Härte erfahren. Und wisst, daß ALLAH gewiß mit den Muttaqi ist.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mccclix{Und wenn eine Sura hinabgesandt wurde, sagte manch einer von ihnen: ``Wen hat diese (Sura) im Iman verstärkt?" Also hinsichtlich derjenigen, die den Iman verinnerlicht haben, so ließ diese sie noch mehr Iman gewinnen, während sie zuversichtlich sind.\qt@no{124}}%
\qt@newcmd\qurantrans@de@mccclx{Doch hinsichtlich derjenigen, in deren Herzen Krankheit ist, so ließ diese sie noch mehr rituelle Unreinheit zu ihrer rituellen Unreinheit gewinnen. Und sie sind gestorben, während sie noch Kafir waren.\qt@no{125}}%
\qt@newcmd\qurantrans@de@mccclxi{Nehmen sie etwa nicht wahr, daß sie jedes Jahr ein- oder zweimal Fitna ausgesetzt werden, doch dann weder bereuen sie, noch besinnen sie sich?!\qt@no{126}}%
\qt@newcmd\qurantrans@de@mccclxii{Und wenn eine Sura hinabgesandt wurde, haben die einen mit den anderen Blicke gewechselt: ``Sieht euch jemand?" Dann entfernten sie sich. ALLAH möge ihre Herzen (vom Iman) entfernen, weil sie gewiß Leute sind, die nicht begreifen.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mccclxiii{Gewiß, bereits ist zu euch ein Gesandter von den Eurigen gekommen, ihn macht betroffen, was ihr erleidet, er achtet auf euch und ist den Mumin gegenüber erbarmend, gnädig.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mccclxiv{Doch sollten sie sich abwenden, dann sag: ``Mir genügt ALLAH, es gibt keine Gottheit außer Ihm! Ihm gegenüber übe ich Tawakkul, und ER ist Der HERR vom erhabenen Al'ahrsch.\qt@no{129}}%
\qt@newcmd\qurantrans@de@mccclxv{\basmalahde Alif-lam-raa. Diese sind die Ayat der weisen/vollendeten Schrift!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mccclxvi{War dies den Menschen etwa verwunderlich, daß WIR einem Mann von ihnen Wahy zuteil werden ließen: ``Ermahne die Menschen und überbringe den Mumin die frohe Botschaft, daß sie bei ihrem HERRN eine Vorrangstellung haben." Die Kafir sagten: ``Gewiß, dies ist wahrhaftig ein offenkundiger Magier!"\qt@no{2}}%
\qt@newcmd\qurantrans@de@mccclxvii{Gewiß, Euer HERR ist ALLAH, Der die Himmel und die Erde in sechs Ayyam erschuf, dann wandte ER sich dem Al'ahrsch zu. ER bestimmt die Angelegenheit. Es gibt keinen Fürbittenden außer mit Seiner Zustimmung. Dies ist ALLAH, euer HERR, also dient Ihm! Wollt ihr euch etwa nicht besinnen?!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mccclxviii{Zu Ihm ist eure Rückkehr, alle zusammen - ein wahrhaftiges Versprechen von ALLAH. Gewiß, ER leitet die Erschaffung ein, dann wiederholt ER sie, damit ER diejenigen, die den Iman verinnerlichten und gottgefällig Gutes taten, in gerechter Weise belohnt. Doch für diejenigen, die Kufr betrieben haben, gibt es einen Trank aus Siedendheißem und eine qualvolle Peinigung für das, was sie zu tun pflegten.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mccclxix{ER ist Derjenige, Der die Sonne als Leuchtende und den Mond als Licht machte und ihm Stufen bestimmte, damit ihr die Anzahl der Jahre und das Berechnen kennt. Nicht hat ALLAH dies außer wahrheitsgemäß erschaffen! ER verdeutlicht die Ayat für Menschen, die über Wissen verfügen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mccclxx{Gewiß, im Aufeinanderfolgen von Nacht und Tag und was ALLAH in den Himmeln und der Erde erschaffen hat, gibt es sicherlich Ayat für Menschen, die Taqwa gemäß handeln.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mccclxxi{Gewiß, diejenigen, die sich Unsere Begegnung nicht erhoffen, sich mit dem diesseitigen Leben zufriedengaben und an ihm Anlehnung fanden und die Unseren Ayat gegenüber achtlos sind,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mccclxxii{für diese - ihre Unterkunft ist das Feuer für das, was sie (an Verfehlungen) zu erwerben pflegten.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mccclxxiii{Gewiß, diejenigen, die den Iman verinnerlicht und das gottgefällig Gute getan haben, diese leitet ihr HERR mit ihrem Iman recht. Unter ihnen fließen Flüsse in den Dschannat des Wohlergehens.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mccclxxiv{Ihr Bittgebet darin ist: Subhanakal-lahumma und ihr Gruß ist darin Salam und der Abschluß ihres Bittgebets ist: Al-hamdulil-lahi rabil-'alamin.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mccclxxv{Und würde ALLAH den Forderungen der Menschen nach dem Unglück so schnell entsprechen wie ER ihren Forderungen nach dem Gutem entspricht, wäre ihre Zeit längst abgelaufen. Aber WIR lassen diejenigen, die Unsere Begegnung nicht erhoffen, in ihrer Maßlosigkeit verblenden.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mccclxxvi{Und wenn dem Menschen Unglück zustößt, richtet er an Uns Bittgebete im Liegen, im Sitzen oder im Stehen. Und als WIR von ihm sein Unglück wegnahmen, fuhr er fort, als richtete er an Uns niemals Bittgebete wegen des Unglücks, das ihm zustieß. Derart wird den Maßlosen schön gemacht, was sie zu tun pflegten.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mccclxxvii{Und gewiß, bereits haben WIR frühere Generationen vor euch zugrunde gerichtet, als sie Unrecht begingen und ihre Gesandten zu ihnen mit deutlichen Zeichen kamen, doch sie pflegten (trotzdem) den Iman nicht zu verinnerlichen. Derart vergelten WIR es den schwer verfehlenden Leuten.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mccclxxviii{Dann setzten WIR euch als Nachfolger im Lande nach ihnen ein, um zu sehen, wie ihr handelt.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mccclxxix{Und als ihnen Unsere Ayat verdeutlicht vorgetragen wurden, haben diejenigen, die Unsere Begegnung nicht erhoffen, gesagt: ``Bringe einen Quran außer diesem oder forme ihn um!" Sag: ``Mir steht es nicht zu, ihn nach meinem Gutdünken umzuformen. Ich folge nur dem, was mir an Wahy zuteil wird! Ich fürchte, sollte ich meinem HERRN widersprechen, die Peinigung eines gewaltigen Tages!"\qt@no{15}}%
\qt@newcmd\qurantrans@de@mccclxxx{Sag (weiter): ``Hätte ALLAH es gewollt, hätte ich ihn euch nicht vorgetragen und hätte ER euch über ihn nicht in Kenntnis gesetzt. Ich lebte doch unter euch lange Zeit vor ihm - wollt ihr euch etwa nicht besinnen?!"\qt@no{16}}%
\qt@newcmd\qurantrans@de@mccclxxxi{Und keiner begeht mehr Unrecht als derjenige, der im Namen ALLAHs Lügen verbreitet oder Seine Ayat verleugnet! Gewiß, die schwer Verfehlenden werden nicht erfolgreich sein.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mccclxxxii{Und sie dienen anstelle von ALLAH dem, was ihnen weder Schaden noch Nutzen bringen kann, und sagten: ``Diese sind unsere Fürbittenden bei ALLAH." Sag: ``Setzt ihr ALLAH etwa (damit) in Kenntnis über etwas, was ER weder in den Himmeln noch auf der Erde kennt?!" Gepriesen-erhaben sei ER über das, was sie an Schirk betreiben.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mccclxxxiii{Und die Menschen bildeten eigentlich nur eine einzige Umma, dann wurden sie uneins. Und gäbe es kein von deinem HERRN bereits gefälltes Wort, wäre zwischen ihnen gewiß über das gerichtet, worüber sie uneins sind.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mccclxxxiv{Und sie sagten: ``Wäre ihm doch eine Aya von seinem HERRN hinabgesandt worden!" Also sag: ``Das Verborgene kennt nur ALLAH, so wartet ab! Ich bin mit euch doch auch einer der Abwartenden."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mccclxxxv{Und wenn WIR die Menschen Gnade erfahren lassen nach einem Unglück, das ihnen zustieß, betreiben sie mit einemmal List Unseren Ayat gegenüber. Sag: ``ALLAH ist schneller im Entgegnen von List." Gewiß, Unsere Gesandten (Engel) registrieren, was ihr an List betreibt.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mccclxxxvi{ER ist Derjenige, Der euch auf dem Lande und auf dem Meer bewegen läßt! Dann, als ihr auf dem Schiff wart, und dieses sie (die Menschen) durch günstigen Wind fortbewegte und sie sich darüber freuten, kam ihm mit einemmal ein Sturmwind entgegen. Und Sturmwellen kamen ihm aus jeder Richtung entgegen und sie dachten, daß sie doch keinen Ausweg mehr haben, (da) richteten sie Bittgebete an ALLAH aufrichtig im Din Ihm gegenüber: ``Würdest DU uns daraus erretten, gewiß würden wir dann von den Dankbaren sein!"\qt@no{22}}%
\qt@newcmd\qurantrans@de@mccclxxxvii{Doch nachdem ER sie errettet hatte, begingen sie mit einemmal Übertretungen im Lande zu Unrecht. Ihr Menschen! (Die Folgen) eurer Übertretungen tragt nur ihr selbst. Dies sind (vergängliche) Freuden und Gebrauchsgüter des diesseitigen Lebens. Danach ist eure Rückkehr zu Uns, dann werden WIR euch über das in Kenntnis setzen, was ihr zu tun pflegtet.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mccclxxxviii{Das Gleichnis des diesseitigen Lebens ist doch wie Wasser, das WIR vom Himmel hinabsenden ließen, mit dem dann Pflanzenwuchs der Erde, von dem sich Menschen und die An'am ernähren, ineinander wucherte. Hierauf, als die Erde ihren Prunk anlegte und sich schmückte und ihre Bewohner dachten, daß sie die Macht über sie besäßen, kam zu ihr Unser Befehl in der Nacht oder am Tage, dann haben WIR sie wie abgeerntete (Flur) gemacht, als wäre vorher niemals etwas gewesen. Derart verdeutlichen WIR die Ayat für Menschen, die nachdenken.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mccclxxxix{Und ALLAH lädt zum Darus-salam ein und leitet recht, wen ER will, zum geraden Weg.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcccxc{Für diejenigen, die Ihsan gemäß handelten, gibt es Al-husna und etwas mehr! Und ihreMienen verfinstern weder Ruß noch Demütigung. Diese sind die Weggenossen der Dschanna. Darin bleiben sie ewig.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mcccxci{Doch für diejenigen, welche die gottmißfälligen Taten erwarben, ist die Vergeltung einer gottmißfälligen Tat mit Gleichem und sie überkommt Demütigung. Vor ALLAH haben sie keinen Schutz-Gewährenden, als wären ihre Gesichter mit Stücken einer stockfinsteren Nacht verschleiert. Diese sind die Weggenossen des Feuers. Darin bleiben sie ewig.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mcccxcii{Und (erinnere daran) an den Tag, an dem WIR sie alle zusammen versammeln, dann zu denjenigen, die Schirk betrieben haben, sagen: ``Rührt euch nicht von der Stelle, ihr und eure (ALLAH) beigesellten Partner!" Dann trennen WIR zwischen ihnen. Und ihre (ALLAH) beigesellten Partner sagten: ``Ihr pflegtet uns doch nie zu dienen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mcccxciii{ALLAH genügt als Zeuge zwischen uns und euch. Wir waren eurem Götzendienst (uns gegenüber) gewiß achtlos."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcccxciv{Dort überprüft jede Seele, was sie vorlegte. Und sie wurden zu ALLAH zurückgebracht, ihrem wahren Maula. Und ihnen ist abhanden gekommen, was sie zu erfinden pflegten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcccxcv{Sag: ``Wer gewährt euch denn Rizq vom Himmel und von der Erde?! Oder wer verfügt über das Hören und das Sehen?! Und wer bringt das Lebendige aus dem Toten hervor und bringt hervor das Tote aus dem Lebendigen?! Und wer lenkt die Angelegenheit?!" Sie werden dann sagen: ``ALLAH!" Dann sag: ``Wollt ihr denn nicht Taqwa gemäß handeln!"\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcccxcvi{Dies ist ALLAH, euer wahrer HERR. Was gibt es denn anstelle der Wahrheit außer dem Irregehen?! Wie werdet ihr denn davon abgebracht?!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcccxcvii{Derart bewahrheitete sich das Wort deines HERRN hinsichtlich derjenigen, die Fisq betrieben, daß sie den Iman nicht verinnerlichen werden.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcccxcviii{Sag: ``Gibt es unter den von euch (ALLAH) beigesellten Partnern einen, der die Erschaffung einleiten und dann wiederholen kann?" Sag: ``ALLAH leitet die Erschaffung ein, dann wiederholt ER sie!" Wie werdet ihr denn (davon) abgebracht?!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcccxcix{Sag: ``Gibt es unter den von euch (ALLAH) beigesellten Partnern einen, der zur Wahrheit rechtleitet?" Sag: ``ALLAH leitet zur Wahrheit recht!" Hat etwa derjenige, der zurWahrheit rechtleitet, mehr Anrecht darauf, daß ihm gefolgt wird, oder derjenige, der nicht rechtleitet, außer wenn er selbst rechtgeleitet wird?!" Was ist mit euch, wie urteilt ihr denn?!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcd{Und nichts folgen die meisten von ihnen außer einer Spekulation. Doch das Spekulieren ersetzt nichts von der Wahrheit. Gewiß, ALLAH ist allwissend über das, was sie tun.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcdi{Und dieser Quran könnte niemals im Namen ALLAHs erdichtet werden. Doch er ist eine Bestätigung dessen, was vor ihm war, und eine Erläuterung der Schrift - er ist erhaben über jeden Zweifel - von Dem HERRN aller Schöpfung.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcdii{Oder sagen sie etwa: ``Er (Muhammad) hat ihn erdichtet"?! Sag: ``Dann bringt eine Sura nach seinem Vorbild und bittet dabei um Unterstützung jeden, den ihr anstelle von ALLAH bitten könnt, solltet ihr wahrhaftig sein.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcdiii{Nein, sondern sie verleugneten das, dessen Wissen sie sich nicht aneigneten und dessen Exegese ihnen noch nicht zuteil wurde. Solcherart haben diejenigen vor ihnen geleugnet. So siehe, wie das Anschließende der Unrecht-Begehenden war.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcdiv{Und unter ihnen sind manche, die den Iman an ihn verinnerlichen, und unter ihnen sind manche, die an ihn den Iman nicht verinnerlichen. Doch dein HERR ist allwissend über die Unrecht-Begehenden.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcdv{Und sollten sie dich der Lüge bezichtigen, dann sag: ``Mir wird mein Handeln vergolten und euch wird euer Handeln vergolten. Ihr habt keine Schuld an dem, was ich tue, und ich habe keine Schuld an dem, was ihr tut."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mcdvi{Und unter ihnen sind manche, die dir zuhören. Möchtest du etwa den Tauben Gehör verleihen, selbst dann, sollten sie nicht verständig sind?!\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcdvii{Und unter ihnen sind manche, die ihre Blicke auf dich richten. Möchtest du etwa die Blinden rechtleiten, selbst dann, sollten sie nicht sehen können?!\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcdviii{Gewiß, ALLAH fügt den Menschen nichts an Unrecht zu, doch die Menschen fügen sich selbst Unrecht zu.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mcdix{Und an dem Tag, an dem ER sie versammelt - als hätten sie (im Diesseits) nur eine Stunde des Tages verbracht, erkennen sie sich gegenseitig. Bereits verloren haben diejenigen, die die Begegnung mit ALLAH leugneten, und (sie) waren nicht rechtgeleitet.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mcdx{Und entweder zeigen WIR dir etwas von dem, wasWIR ihnen androhten oder lassen dich vorher sterben. Zu Uns ist jedenfalls ihre Rückkehr! Dann ist ALLAH Zeuge über das, was sie tun.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mcdxi{Und für jede Umma gibt es einen Gesandten. Und wenn ihr Gesandter eintrifft, wird zwischen ihnen gerecht gerichtet und ihnen wird keinerlei Unrecht angetan.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mcdxii{Und sie sagen: ``Wann wird dieses Versprechen denn eingelöst, wenn ihr wahrhaftig seid?"\qt@no{48}}%
\qt@newcmd\qurantrans@de@mcdxiii{Sag: ``Ich kann mir selbst weder Schaden noch Nutzen zufügen außer dem, was ALLAH will." Für jede Umma ist eine Frist festgelegt. Wenn ihre Frist abgelaufen ist, können sie (diese) weder um eine Stunde verschieben, noch vorverlegen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mcdxiv{Sag: ``Wie seht ihr es? Sollte Seine Peinigung über euch nachts oder am Tage hereinbrechen, was wollen denn die schwer Verfehlenden davon beschleunigen?"\qt@no{50}}%
\qt@newcmd\qurantrans@de@mcdxv{Dann, wenn sie (die Peinigung) erfolgt, verinnerlicht ihr den Iman daran?! Jetzt erst, während ihr sie bereits immer wieder beschleunigt haben wolltet?!\qt@no{51}}%
\qt@newcmd\qurantrans@de@mcdxvi{Dann wurde zu denjenigen schwer Verfehlenden gesagt: ``Kostet die Peinigung der Ewigkeit! Wird euch etwa anderes vergolten als das, was ihr euch zu erwerben pflegtet?!"\qt@no{52}}%
\qt@newcmd\qurantrans@de@mcdxvii{Und sie holen bei dir die Auskunft ein: ``Ist es wahr?" Sag: ``Ja! Bei meinem HERRN gewiß, es ist doch wahr!" Und ihr könnt (Uns) nicht zu schaffen machen.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mcdxviii{Und würde jede Seele, die Unrecht beging, über alles auf der Erde verfügen, würde sie sich damit (von der Peinigung) freikaufen! Und sie verheimlichten die Reue, als sie die Peinigung sahen. Und unter ihnen wurde gerecht gerichtet. Und keinerlei Unrecht wird ihnen angetan.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mcdxix{Mit Sicherheit gehört ALLAH alles, was in den Himmeln und auf der Erde ist. Mit Sicherheit ist ALLAHs Versprechen wahr, doch die meisten wissen es nicht.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mcdxx{ER belebt und läßt sterben und zu Ihm werdet ihr zurückgebracht.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mcdxxi{Ihr Menschen! Bereits kam zu euch doch eine Ermahnung von eurem HERRN, eine Heilung von dem, was in den Brüsten (an Zweifel) ist, eine Rechtleitung und eine Gnade für die Mumin.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mcdxxii{Sag: ``Über ALLAHs Gunst und Seine Gnade, darüber sollen sie sich freuen. Dies ist besser als das, was sie anhäufen."\qt@no{58}}%
\qt@newcmd\qurantrans@de@mcdxxiii{Sag: ``Wie seht ihr das, was ALLAH euch vom Rizq hinabsandte, dann ihr es in Haram und Halal eingeteilt habt?" Sag (weiter): ``Erlaubte ALLAH es euch, oder erdichtet ihr im Namen ALLAHs?!"\qt@no{59}}%
\qt@newcmd\qurantrans@de@mcdxxiv{Und was denken sich diejenigen am Tag der Auferstehung, die im Namen ALLAHs Lügen erdichten?! Gewiß, ALLAH ist den Menschen gegenüber zweifelsohne Gunst erweisend. Doch die meisten von ihnen erweisen sich undankbar.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mcdxxv{Und du (Muhammad) unternimmst nichts und trägst davon nichts aus dem Quran vor und ihr tut nichts, ohne daß WIR über euch Zeugen sind, vor allem wenn ihr über ihn lästert. Und deinem HERRN bleibt auch nichts verborgen, weder das Gewicht eines Staubkorns auf Erden noch im Himmel, noch etwas Kleineres als dies noch etwas Größeres, ohne daß dies in einer deutlichen Schrift registriert ist.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mcdxxvi{Ja! Die Wali von ALLAH, um sie gibt es weder Angst noch werden sie traurig sein.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mcdxxvii{Es sind diejenigen, die den Iman verinnerlichten Taqwa gemäß handelten.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mcdxxviii{Für sie gibt es im diesseitigen Leben frohe Botschaft sowie im Jenseits. Keine Abänderung gibt es für ALLAHs Worte. Dies ist der unermeßliche Gewinn.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mcdxxix{Laß ihr Gerede dich nicht traurig machen! ALLAHgehört die Erhabenheit und Würde voll und ganz! ER ist Der Allhörende, Der Allwissende.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mcdxxx{Mit Sicherheit gehören ALLAH alle (Vernunftbegabten), die in den Himmeln und auf Erden sind. Und nicht folgen diejenigen, die anstelle von ALLAH andere anrufen, (in Wirklichkeit) Partnern. Sie folgen ausschließlich Spekulationen und sie lügen nur.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mcdxxxi{ER ist Derjenige, Der euch die Nacht einrichtete, um darin zu ruhen, sowie den Tag mit Licht. Gewiß, darin sind zweifelsohne Ayat für Menschen, die zuhören.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mcdxxxii{Sie sagten: ``ALLAH legte Sich ein Kind zu." Gepriesenerhaben bleibt ER. ER ist Der absolut Autarke! Ihm gehört alles, was in den Himmeln und auf Erden ist. Verfügt ihr etwa über Wissen darüber?! Sagt ihr etwa über ALLAH, was ihr kennt?!\qt@no{68}}%
\qt@newcmd\qurantrans@de@mcdxxxiii{Sag: ``Gewiß, diejenigen, die im Namen ALLAHs Lügen erdichten, werden keinen Erfolg haben."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mcdxxxiv{Sie haben (vergängliche) Freuden und Gebrauchsgüter im diesseitigen Leben, dann wird ihre Rückkehr zu Uns sein, dann werden WIR sie die qualvolle Peinigung erfahren lassen für das, was sie an Kufr zu betreiben pflegten.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mcdxxxv{Und trage ihnen die Geschichte von Nuh vor, als er zu seinen Leuten sagte: ``Meine Leute! Sollten mein (längerer) Aufenthalt und meine Ermahnung mit ALLAHs Ayat für euch unerträglich sein, so übe ich ALLAH gegenüber Tawakkul. Also trefft eure Entscheidung mit den von euch Beigesellten! Eure Entscheidung soll dann nicht geheim sein, danach unterrichtet mich darüber und nehmt auf mich dabei keine Rücksicht!\qt@no{71}}%
\qt@newcmd\qurantrans@de@mcdxxxvi{Solltet ihr euch abwenden, so habe ich euch nicht um Lohn gebeten. Mein Lohn obliegt nur ALLAH. Und mir wurde geboten, einer der Muslime zu sein."\qt@no{72}}%
\qt@newcmd\qurantrans@de@mcdxxxvii{Sie haben ihn jedoch des Lügens bezichtigt, so haben WIR ihn und diejenigen mit ihm im Schiff gerettet, diese als Nachfolger eingesetzt und diejenigen ertrinken lassen, die Unsere Ayat abgeleugnet haben. Also nimm wahr, wie das Anschließende von den Ermahnten war.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mcdxxxviii{Dann entsandten WIR nach ihm Gesandte zu ihren Leuten. Sie kamen zu ihnen mit deutlichen Zeichen. Doch es lag ihnen nicht, den Iman an das zu verinnerlichen, was sie vorher abgeleugnet haben. Solcherart versiegeln WIR die Herzen der Übertretenden.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mcdxxxix{Dann entsandten WIR nach ihnen Musa und Harun zu Pharao und seinen Entscheidungsträgern mit Unseren Ayat. Diese erhoben sich jedoch in Arroganz und waren schwer verfehlende Leute.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mcdxl{Und als zu ihnen die Wahrheit von Uns kam, sagten sie: ``Gewiß, dies ist zweifelsohne eine offenkundige Magie."\qt@no{76}}%
\qt@newcmd\qurantrans@de@mcdxli{Musa sagte: ``Sagt ihr etwa über die Wahrheit, als sie zu euch kam, dies sei Magie? Aber dieMagier werden nicht erfolgreich sein! \qt@no{77}}%
\qt@newcmd\qurantrans@de@mcdxlii{Sie sagten: ``Kamst du etwa, um uns von dem abzubringen, was wir bei unseren Ahnen vorfanden, und damit euch beiden (dir mit deinem Bruder) die Herrschaft im Lande gehört?! Wir werden euch gewiß nicht glauben."\qt@no{78}}%
\qt@newcmd\qurantrans@de@mcdxliii{Pharao sagte dann: ``Bringt mir jeden äußerst wissenden Magier!"\qt@no{79}}%
\qt@newcmd\qurantrans@de@mcdxliv{Und als die Magier dann kamen, sagte Musa zu ihnen: ``Werft, was ihr zu werfen habt!"\qt@no{80}}%
\qt@newcmd\qurantrans@de@mcdxlv{Und als sie warfen, sagteMusa: ``Was ihr vollbracht habt, ist die Magie. ALLAH wird es nichtig machen. Gewiß, ALLAH läßt dasWirken der Verderben-Anrichtenden nichtig machen.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mcdxlvi{ER läßt die Wahrheit durch Seine Worte zur Geltung kommen, selbst dann, sollten die schwer Verfehlenden dies verabscheuen."\qt@no{82}}%
\qt@newcmd\qurantrans@de@mcdxlvii{Doch mit Musa hat niemand den Iman verinnerlicht außer Nachkommenschaft von seinen Leuten  unter Angst vor Pharao und seinen Entscheidungsträgern, daß diese sie der Fitna aussetzen. Gewiß, Pharao war zweifellos ein Hochmütiger im Lande und gewiß, er war sicherlich einer der Maßlosen.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mcdxlviii{Musa sagte dann: ``Meine Leute! Solltet ihr den Iman an ALLAH verinnerlicht haben, so übt Ihm (alleine) gegenüber Tawakkul, wenn ihr Muslime seid."\qt@no{84}}%
\qt@newcmd\qurantrans@de@mcdxlix{Dann sagten sie: ``ALLAH gegenüber üben wir doch Tawakkul. Unser HERR! Laß uns nicht eine Fitna für die unrecht-begehenden Leute sein,\qt@no{85}}%
\qt@newcmd\qurantrans@de@mcdl{und rette uns durch Deine Gnade vor den Kafir-Leuten!"\qt@no{86}}%
\qt@newcmd\qurantrans@de@mcdli{Und WIR ließen Musa und seinem Bruder Wahy zuteil werden: ``Richtet ihr beide für eure Leute in Ägypten Häuser als Zentren ein, und macht aus euren Häusern Gebetsstätten und verrichtet ordnungsgemäß das rituelle Gebet! Und überbringe den Mumin frohe Botschaft!\qt@no{87}}%
\qt@newcmd\qurantrans@de@mcdlii{Und Musa sagte: ``Unser HERR! Gewiß, DU gabst Pharao und seinen Entscheidungsträgern Schmuck und Vermögenswerte im diesseitigen Leben, unser HERR! - um damit von Deinem Weg abzubringen. Unser HERR! Verwische ihr Vermögen und versiegele ihre Herzen, damit sie den Iman nicht verinnerlichen, bis sie die qualvolle Peinigung erleben."\qt@no{88}}%
\qt@newcmd\qurantrans@de@mcdliii{ER sagte: ``Bereits wurde der Bitte von euch beiden entsprochen, so bleibt standhaft und folgt nicht dem Weg derjenigen, die nicht wissen."\qt@no{89}}%
\qt@newcmd\qurantrans@de@mcdliv{Dann ließen WIR die Kinder Israils das Meer überqueren. Pharao und seine Soldaten sind ihnen dann aus Übertretung und Feindseligkeit gefolgt. Dann als er (Pharao) kurz vor dem Ertrinken war, sagte er: ``Ich bekunde den Iman, daß es gewiß keine Gottheit gibt außer Der, an Die die Kinder Israils den Iman bekundeten, und ich bin einer der Muslime."\qt@no{90}}%
\qt@newcmd\qurantrans@de@mcdlv{Jetzt erst! Wo du bereits vorher Verfehlungen begangen hast und einer von den Verderben-Anrichtenden warst?!\qt@no{91}}%
\qt@newcmd\qurantrans@de@mcdlvi{An diesem Tag werden WIR deinen Körper erretten, damit du denjenigen nach dir als Aya dienst. Gewiß, viele der Menschen sind Unseren Ayat gegenüber doch achtlos!\qt@no{92}}%
\qt@newcmd\qurantrans@de@mcdlvii{Und gewiß, bereits ließen WIR die Kinder Israils einen schönen Ort bewohnen und gewährten ihnen Rizq von den Tay-yibat. Und sie wurden nicht uneins, bis ihnen dasWissen zuteil wurde. Gewiß, dein HERR wird unter ihnen am Tag der Auferstehung das richten, worüber sie uneins zu sein pflegten.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mcdlviii{Und solltest du über das in Zweifel sein, was WIR dir zuteil werden ließen, dann frage diejenigen, welche die Schrift (At-taurat) vor dir lesen. Gewiß, bereits kam die Wahrheit zu dir von deinem HERRN, so sei nicht von den Zweifel- Hegenden!\qt@no{94}}%
\qt@newcmd\qurantrans@de@mcdlix{Und sei auf keinen Fall einer derjenigen, die ALLAHs Ayat ableugnen, sonst gehörst du zu den Verlierern!\qt@no{95}}%
\qt@newcmd\qurantrans@de@mcdlx{Gewiß, diejenigen, gegen sie das Wort deines HERRN rechtens wurde, verinnerlichten den Iman nicht,\qt@no{96}}%
\qt@newcmd\qurantrans@de@mcdlxi{selbst dann nicht, sollte zu ihnen jede Aya kommen - bis sie die qualvolle Peinigung erleben.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcdlxii{Gäbe es doch eine (andere Bevölkerung einer) Ortschaft, die den Iman verinnerlicht hat, und ihr Iman ihr nutzte! Ausgenommen sind die Leute von Yunus, als sie den Iman verinnerlicht haben, nahmen WIR von ihnen die Peinigung der Erniedrigung im diesseitigen Leben weg und gewährten ihnen Wohlergehen bis zu einer (bestimmten) Zeit.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mcdlxiii{Und hätte dein HERR es gewollt, bestimmt hätten alle, die auf Erden sind, alle, insgesamt, den Iman verinnerlicht. Willst du etwa die Menschen dazu zwingen, damit sie Mumin werden?!\qt@no{99}}%
\qt@newcmd\qurantrans@de@mcdlxiv{Und es gebührt keinem Menschen, den Iman zu verinnerlichen außer mit ALLAHs Zustimmung. Und ER läßt die Peinigung denjenigen zuteil werden, die nicht verständig sind.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mcdlxv{Sag: ``Denkt nach über das in den Himmeln und auf Erden!" Doch weder die Ayat noch die Ermahner nützen Menschen, die keinen Iman verinnerlichen wollen.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mcdlxvi{Warten sie denn auf etwas anderes als auf Ähnliches wie die Tage derjenigen, die vor ihnen vergangen sind?! Sag: ``Wartet nur ab! Gewiß, ich bin mit euch einer der Abwartenden."\qt@no{102}}%
\qt@newcmd\qurantrans@de@mcdlxvii{Dann erretten WIR Unsere Gesandten und diejenigen, die den Iman verinnerlicht haben. Solcherart obliegt es Uns, die Mumin zu erretten.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mcdlxviii{Sag: ``Ihr Menschen! Wenn ihr Zweifel meinem Din gegenüber habt, so diene ich nicht denjenigen, denen ihr anstelle von ALLAH dient, sondern ich diene ALLAH, Der euch sterben läßt! Und mir wurde geboten, unter den Mumin zu sein!"\qt@no{104}}%
\qt@newcmd\qurantrans@de@mcdlxix{Und richte dich dem Din als Hanif auf! Und sei nicht von den Muschrik!\qt@no{105}}%
\qt@newcmd\qurantrans@de@mcdlxx{Und richte nicht an das Bittgebete anstelle von ALLAH, was dir weder nutzt noch schadet. Und solltest du es tun, dann gehörst du gewiß zu den Unrecht-Begehenden.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mcdlxxi{Und sollte ALLAH dir Unglück zustoßen lassen, so kann niemand es beseitigen außer Ihm. Und sollte ER dir Gutes bestimmen, so kann niemand Seine Gunst abwenden. ER trifft damit von Seinen Dienern, wen ER will. Und ER ist Der Allvergebende, Der Allgnädige.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mcdlxxii{Sag: ``Ihr Menschen! Bereits kam die Wahrheit zu euch von eurem HERRN. Wer also der Rechtleitung folgt, der folgt der Rechtleitung nur für seinen eigenen Nutzen. Wer jedoch irregeht, der geht nur irre zu seinem eigenen Schaden. Und ich bin kein Wakil über euch."\qt@no{108}}%
\qt@newcmd\qurantrans@de@mcdlxxiii{Und folge dem, was dir an Wahy zuteil wurde, und übe dich in Geduld, bis ALLAH entscheidet. Und ER ist der beste der Entscheidenden.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mcdlxxiv{\basmalahde Alif-lam-raa. Dies ist eine Schrift, deren Ayat in vollendeter Weisheit ausformuliert wurden, dann erläutert wurden von Einem Allweisen, Allkundigen:\qt@no{1}}%
\qt@newcmd\qurantrans@de@mcdlxxv{``Dient nichts außer ALLAH! Gewiß, ich bin für euch von Ihm ein Warner und ein Überbringer froher Botschaft.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mcdlxxvi{Auch bittet euren HERRN um Vergebung, dann kehrt euch zu Ihm reuig um, so läßt ER euch schöne Versorgung bis zu einer festgelegten Frist versorgen und erweist jedem mit Verdiensten (im Islam) die ihm zustehende Belohnung. Doch solltet ihr euch abwenden, so fürchte ich doch für euch die Peinigung eines langen Tages.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mcdlxxvii{Zu ALLAH ist eure Rückkehr. Und ER ist über alles allmächtig."\qt@no{4}}%
\qt@newcmd\qurantrans@de@mcdlxxviii{Ja! Sie wenden sich ab, um sich vor Ihm zu verbergen. Doch selbst dann, wenn sie sich mit ihrer Kleidung verhüllen, ER kennt alles, was sie verheimlichen und was sie offenlegen. Gewiß, ER ist allwissend über das in den Brüsten.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mcdlxxix{Und es gibt kein Lebewesen, das sich bewegt, dessen Rizq nicht ALLAH obliegt. Und ER kennt dessen Aufenthalts- und Aufbewahrungsort. Alles steht in einem deutlichen Register.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mcdlxxx{Und ER ist Derjenige, Der die Himmel und die Erde in sechs Ay-yam erschuf - und Sein Al'ahrsch war über dem Wasser -, damit ER euch prüft, wer von euch besser handelt. Und wenn du sagst: ``Gewiß, ihr werdet nach dem Tod wieder erweckt, werden diejenigen, die Kufr betrieben haben, mit Sicherheit sagen: ``Dies ist doch nichts anderes als eindeutige Magie."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mcdlxxxi{Doch wenn WIR ihnen die Peinigung auf eine bestimmte Zeit hinauszögern, werden sie mit Sicherheit sagen: ``Was hält sie auf?" Ja! Wenn diese sie dann trifft, wird sie von ihnen nicht mehr abgewendet werden, und sie werden dann von dem umgeben, was sie zu verspotten pflegten.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mcdlxxxii{Und wenn WIR den Menschen eine Gnade von Uns erfahren lassen, dann diese von ihm wegnehmen, so ist er gewiß doch äußerst verzweifelt, äußerst kufr-betreibend!\qt@no{9}}%
\qt@newcmd\qurantrans@de@mcdlxxxiii{Und wenn WIR ihm eine Wohltat erweisen nach einem Unglück, das ihn traf, bestimmt sagt er: ``Die gottmißfälligen Taten sind von mir fortgegangen." Gewiß, ER ist doch überglücklich, stolz.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mcdlxxxiv{Ausgenommen sind diejenigen, die sich in Geduld geübt und gottgefällig Gutes getan haben. Für diese gibt es Vergebung und große Belohnung.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mcdlxxxv{Würdest du etwas von dem, was dir an Wahy zuteil wurde, auslassen und darüber bedrückt sein, daß sie sagen: ``Würde auf ihn doch ein Schatz niedergesandt oder käme mit ihm ein Engel!"?! DU bist nur ein Ermahner. Und ALLAH ist über alles Wakil.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mcdlxxxvi{Oder sagen sie etwa: ``Er hat ihn (den Quran) erdichtet!" Sag: ``Dann bringt doch zehn ähnlich erdichtete Suras und ruft dazu auf jeden, den ihr anstelle von ALLAH (rufen) könnt, solltet ihr wahrhaftig sein."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mcdlxxxvii{Und wenn sie eurer (Forderung) nicht nachkommen, dann wisst, daß er (der Quran) ausschließlich mit ALLAHs Wissen hinabgesandt wurde und daß es keine Gottheit außer Ihm gibt! Und werdet ihr nun Muslime werden?!\qt@no{14}}%
\qt@newcmd\qurantrans@de@mcdlxxxviii{Wer das diesseitige Leben und seinen Schmuck will, denen werden WIR darin ihre Werke vergelten und sie werden darin nicht benachteiligt!\qt@no{15}}%
\qt@newcmd\qurantrans@de@mcdlxxxix{Diese sind diejenigen, denen im Jenseits nur das Feuer bestimmt ist. Und zunichte wurde das, was sie darin (im Diesseits) bewerkstelligt haben, und nichtig ist das, was sie zu tun pflegten.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mcdxc{Ist derjenige, der über einen eindeutigen Beleg von seinem HERRN verfügt, (etwa mit einem ohne Beleg gleichzustellen?!). Und diesen trägt vor, sowohl ein Zeuge von ihm (vom Quran) selbst, als auch von vor ihm - die Schrift von Musa, die als Imam und Gnade war. Diese verinnerlichen den Iman an ihn. Und wer von Al-ahzab daran Kufr betreibt, dem ist das Feuer bestimmt. Also sei nicht im Zweifel daran! Gewiß, es ist dieWahrheit von deinem HERRN. Doch die meisten Menschen verinnerlichen den Iman nicht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mcdxci{Und keiner begeht mehr Unrecht als derjenige, der Lügen im Namen ALLAHs erdichtet. Diese werden ihrem HERRN vorgestellt werden, dann sagen die Zeugen: ``Diese sind diejenigen, die Lügen im Namen ihres HERRN erdichtet haben." Ja! ALLAHs Fluch möge auf den Unrecht-Begehenden lasten.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mcdxcii{Es sind diejenigen, die von ALLAHs Weg abhalten und diesen verunstalten wollen, und die dem Jenseits gegenüber Kufr betreiben,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mcdxciii{diese machten ALLAH auf Erden nicht zu schaffen und hatten anstelle von ALLAH keine Wali. Ihnen wird die Peinigung verzweifacht. Sie pflegten nicht zuzuhören und sie pflegten nicht einzublicken.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mcdxciv{Diese sind diejenigen, die sich selbst verloren haben und ihnen ist abhanden gekommen, was sie zu erdichten pflegten.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mcdxcv{Zweifellos eindeutig! Sie sind im Jenseits die wirklichen Verlierer.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mcdxcvi{Gewiß, diejenigen, die den Iman verinnerlicht, gottgefällig Gutes getan und sich ihrem HERRN gefügt haben, diese sind die Weggenossen der Dschanna. Darin werden sie ewig bleiben.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mcdxcvii{Das Gleichnis beider Parteien ähnelt einem Blinden und Tauben und einem Sehenden und Hörenden. Sind beide im Gleichnis etwa gleich?! Besinnt ihr euch etwa nicht?!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mcdxcviii{Und gewiß, bereits entsandten WIR Nuh zu seinen Leuten: ``Gewiß, ich bin für euch ein erklärender Ermahner,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcdxcix{daß ihr nichts außer ALLAH dient. Gewiß, ich fürchte um euch die Peinigung eines qualvollen Tages."\qt@no{26}}%
\qt@newcmd\qurantrans@de@md{Dann sagten die Entscheidungsträger unter seinen Leuten, die Kufr betrieben haben: ``Wir sehen dich nur als einen Menschen wie uns und sehen, daß dir nur diejenigen Unbedeutenden unter uns Hals über Kopf gefolgt sind, und wir sehen nicht, daß ihr uns gegenüber in irgendeiner Weise Verdienste habt. Nein, sondern wir glauben, daß ihr Lügner seid!"\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdi{Er sagte: ``Meine Leute! Wie seht ihr es? Wenn ich über ein eindeutiges Zeichen von meinem HERRN verfüge und ER mir Gnade von Ihm erwies, die euch verborgen blieb, sollen wir diese euch etwa zur Pflicht machen, wenn ihr demgegenüber abgeneigt seid?\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdii{Meine Leute! Ich verlange von euch dafür kein Vermögen. Mein Lohn obliegt doch nur ALLAH. Und ich werde diejenigen, die den Iman verinnerlicht haben, nicht vertreiben. Gewiß, sie werden ihrem HERRN begegnen. Doch ich sehe, daß ihr Leute seid, die unwissend sind.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdiii{Meine Leute! Wer steht mir bei gegen ALLAH, wenn ich sie vertreibe?! Besinnt ihr euch etwa nicht?!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdiv{Auch sage ich euch nicht, daß ich über ALLAHs Magazine verfüge, und ich kenne das Verborgene nicht, auch sage ich nicht, daß ich ein Engel sei, auch sage ich nicht zu denjenigen, die ihr verachtet, daß ALLAH ihnen nichts Gutes erweisen wird - ALLAH ist allwissend über das, was in ihren Seelen ist - ansonsten bin ich gewiß einer der Unrecht-Begehenden!"\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdv{Sie sagten: ``Nuh! Bereits hast du mit uns disputiert und uns gegenüber viel an Disput geführt, also laß uns das zuteil werden, was du uns androhst, wenn du von den Wahrhaftigen bist."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdvi{Er sagte: ``Nur ALLAH läßt es euch zuteil werden, wenn ER will, und ihr könnt (Ihm) nicht zu schaffen machen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdvii{Auch mein Rat wird euch nicht nützen - wenn ich euch raten will, wenn ALLAH euch verführen will. ER ist euer HERR und zu Ihm werdet ihr zurückgebracht."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdviii{Oder sagen sie etwa: ``Er hat ihn (den Quran) erdichtet." Sag: ``Sollte ich ihn erdichtet haben, so bleibt meine schwere Verfehlung an mir hängen, doch ich bin schuldlos dem gegenüber, was ihr an schweren Verfehlungen verübt! \qt@no{35}}%
\qt@newcmd\qurantrans@de@mdix{Und Nuh wurde als Wahy zuteil: ``Gewiß, von deinen Leuten wird niemand den Iman verinnerlichen, außer denjenigen, die bereits den Iman verinnerlicht haben, also sei nicht wehmütig wegen dem, was sie zu tun pflegten.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdx{Und baue das Schiff unter Unserer Aufsicht und nach Unserem Wahy und flehe Mich nicht an wegen denjenigen, die Unrecht begangen haben, gewiß, sie werden ertrinken."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdxi{Und er begann das Schiff zu bauen. Und immer wieder, wenn eine Gruppe seiner Leute bei ihm vorbeiging, verspottete sie ihn. ER sagte: ``Wenn ihr uns verspottet, so verspotten wir euch doch genauso, wie ihr uns verspottet!\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdxii{Ihr werdet noch wissen, wem eine erniedrigende Peinigung zuteil werden wird und wen eine dauernde Peinigung überkommen wird.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdxiii{Als dann Unsere Bestimmung eintraf und die Erdoberfläche sprudelte, sagten WIR: ``Nimm darauf mit von allen (Tieren) zwei Paare, deine Familie außer denjenigen, über welche die Bestimmung (zum Ertrinken) bereits ergangen ist, und alle, die den Iman verinnerlichten!" Und mit ihm haben nur Wenige den Iman verinnerlicht.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdxiv{Und er sagte: ``Steigt in es hinein mit dem Namen ALLAHs, Der es fahren und landen läßt. Gewiß, mein HERR ist zweifelsohne allvergebend, allgnädig."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdxv{Und es fuhr mit ihnen durch Wogen wie Berge, dann rief Nuh seinen Sohn, der abgetrennt war: ``Mein Söhnchen, steig mit uns hinein und sei nicht mit den Kafir!"\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdxvi{Er sagte: ``Ich werde einen Berg aufsuchen, der mich vor dem Wasser schützt." Er sagte: ``Heute gibt es keinen Schützenden vor ALLAHs Bestimmung außer für diejenigen, denen ER Gnade erwies." Die Wogen trennten sie jedoch, so war er von den Ertrunkenen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdxvii{Und es wurde gesagt: ``Erde! Verschlinge dein Wasser! Himmel! Höre auf (zu regnen)!" Und das Wasser wurde eingesickert und die Bestimmung erfüllt und es (das Schiff) stand auf Aldschudi und es wurde gesagt: ``Nieder mit den unrechtbegehenden Leuten!"\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdxviii{Nuh rief dann seinen HERRN und sagte: ``HERR! Mein Sohn gehört doch zu meiner Familie, Dein Versprechen ist gewiß das Wahre und DU bist Der Weiseste aller Richter."\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdxix{ER sagte: ``Nuh! Gewiß, er gehört nicht zu deiner Familie. Dies ist sicher keine gottgefällig gute Tat. Also bitteMich nicht um das, worüber du kein Wissen hast. ICH ermahne dich, einer der Unwissenden zu sein."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdxx{Er sagte: ``Mein HERR! Ich suche Schutz bei Dir, daß ich Dich um etwas bitte, worüber ich kein Wissen habe. Und würdest DU mir nicht vergeben und Gnade erweisen, würde ich bestimmt zu den Verlierern gehören."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdxxi{Es wurde gesagt: ``Nuh! Steige herunter in Salam von Uns und Baraka auf dich und auf Umam von denen, die mit dir sind, sowie auf Umam, denen WIR Wohlergehen gewähren werden, dann sie von Uns eine qualvolle Peinigung treffen wird."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdxxii{Diese sind von den Mitteilungen des Verborgenen, die WIR dir als Wahy zukommen lassen. Nicht gekannt hast du sie, auch nicht deine Leute vorher. Also übe dich in Geduld! Gewiß, das Anschließende gehört den Muttaqi.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdxxiii{Und zu 'Aad (entsandten WIR) ihren Bruder Hud. Er sagte: ``Meine Leute! Dient ALLAH, ihr habt keine Gottheit außer Ihm! Ihr seid doch nichts anderes als Erdichter!\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdxxiv{Meine Leute! Ich bitte euch nicht um eine Belohnung dafür. Meine Belohnung obliegt doch nur Demjenigen, Der mich erschuf. Besinnt ihr euch denn nicht?!\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdxxv{Meine Leute! Bittet euren HERRN um Vergebung, dann zeigt Ihm gegenüber Reue, so läßt ER den Himmel über euch reichlich regnen und verleiht euch zusätzliche Macht zu eurer Macht, und wendet euch nicht als schwer Verfehlende ab!"\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdxxvi{Sie sagten: ``Hud! Du hast uns kein eindeutiges Zeichen gebracht. Wir werden auch auf keinen Fall unsere Götter verlassen wegen deines Geredes und wir werden dir nicht glauben!\qt@no{53}}%
\qt@newcmd\qurantrans@de@mdxxvii{Wir sagen nur, daß manche unserer Götter dich einer Bosheit aussetzten." Er sagte: ``Ich bitte ALLAH doch, Zeuge zu sein, auch seid selbst Zeugen, daß ich keineswegs Schuld trage an dem, was ihr an Schirk betreibt\qt@no{54}}%
\qt@newcmd\qurantrans@de@mdxxviii{Ihm gegenüber! Also führt eure List gegen mich durch, allesamt, dann gewährt mir dabei keine Zeit.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mdxxix{Gewiß, ich übe Tawakkul ALLAH gegenüber, meinem HERRN und eurem HERRN, ohnehin gibt es kein sich bewegendes Lebewesen, ohne daß ER darüber verfügt. Gewiß, mein HERR (vergilt) nach einem rechten Weg.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mdxxx{Solltet ihr euch abwenden, so habe ich euch bereits das verkündet, womit ich zu euch entsandt wurde, und mein HERR wird als Nachfolger andere Leute als ihr einsetzen und ihr werdet Ihm dabei in keiner Weise schaden. Gewiß, mein HERR ist jeder Sache äußerst bewahrend."\qt@no{57}}%
\qt@newcmd\qurantrans@de@mdxxxi{Und als Unsere Bestimmung eintraf, erretteten WIR Hud und diejenigen, die mit ihm den Iman verinnerlicht haben - durch Gnade von Uns. UndWIR haben sie vor einer harten Peinigung errettet.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mdxxxii{Und dies ist 'Aad. Sie leugneten die Ayat ihres HERRN ab, widersetzten sich Seinen Gesandten und folgten der Anordnung jedes sturen Gewalttäters.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mdxxxiii{Und sie wurden in diesem Diesseits mit Fluch verfolgt sowie am Tag der Auferstehung. Ja! 'Aad betrieben zweifellos ihrem HERRN gegenüber Kufr. Ja! Nieder mit 'Aad, den Leuten von Hud.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mdxxxiv{Und zu Thamud (entsandten WIR) ihren Bruder Salih. Er sagte: ``Meine Leute! Dient ALLAH! Ihr habt doch keine Gottheit außer Ihm. ER hat euch aus der Erde hervorgebracht und euch sie bevölkern lassen, also bittet Ihn um Vergebung, dann seid Ihm gegenüber reuig. Gewiß, mein HERR ist nahe und erhörend."\qt@no{61}}%
\qt@newcmd\qurantrans@de@mdxxxv{Sie sagten: ``Salih! Bereits hast du unter uns Vertrauen vor dieser (Aufforderung) genossen. Verbietest du uns etwa, dem zu dienen, dem unsere Ahnen dienten? Gewiß, wir haben am dem, wozu du uns aufforderst, doch Verdacht schleichende Zweifel."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mdxxxvi{Er sagte: ``Meine Leute! Wie seht ihr es? Wenn ich über ein eindeutiges Zeichen von meinem HERRN verfüge und ER mir Gnade von Ihm erwies, wer wird mir gegen ALLAH beistehen können, sollte ich mich Ihm widersetzen?! Doch ihr könnt mir dann nichts außer Schaden hinzufügen!\qt@no{63}}%
\qt@newcmd\qurantrans@de@mdxxxvii{Und meine Leute! Dies ist eine Kamelstute von ALLAH als Aya für euch, so lasst sie auf ALLAHs Erde weiden und fügt ihr keinerlei Schaden zu, sonst wird euch eine bevorstehende Peinigung überkommen."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mdxxxviii{Doch sie verletzten ihre Beine. Dann sagte er: ``Vergnügt euch in euren Wohnstätten noch drei Tage, dies ist ein Versprechen, das nicht erlogen ist.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mdxxxix{Und als Unsere Bestimmung eintraf, erretteten WIR Salih und diejenigen, die mit ihm den Iman verinnerlichten - durch Gnade von Uns. Und vor der Erniedrigung des (Jüngsten) Tages (erretteten WIR sie). Gewiß, dein HERR ist Der Allkraftvolle, Der Allwürdige.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mdxl{Und der Schrei richtete zugrunde diejenigen, die Unrecht begingen, so wurden sie in ihren Wohnstätten zu leblosen Körpern.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mdxli{Als ob sie sich dort nicht aufhielten. Ohne Zweifel haben Thamud ihrem HERRN gegenüber doch Kufr betrieben. Ja! Nieder mit Thamud!\qt@no{68}}%
\qt@newcmd\qurantrans@de@mdxlii{Und gewiß, bereits kamen Unsere Entsandte (Engel) zu Ibrahim mit der frohen Botschaft, sie sagten: ``Salam (sei mit dir)", er erwiderte: ``Salam (sei mit euch)" Und er blieb nicht lange, bis er ein gegartes Kalb servierte.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mdxliii{Doch als er sah, daß ihre Hände nicht danach (nach dem Essen) langten, verübelte er es ihnen und verspürte Furcht vor ihnen. Sie sagten: ``Fürchte dich nicht! Wir wurden zu den Leuten Luts entsandt."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mdxliv{Und seine Frau stand (dabei), dann lachte sie, dann überbrachten WIR ihr die frohe Botschaft über Ishaq und nach Ishaq Ya'qub.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mdxlv{Sie sagte: ``Mein Untergang! Soll ich etwa gebären, während ich eine alte Frau bin und dieser mein Ehemann ein alter Mann ist?! Gewiß, dies ist zweifelsohne eine außergewöhnliche Sache."\qt@no{72}}%
\qt@newcmd\qurantrans@de@mdxlvi{Sie sagten: ``Wunderst du dich etwa über ALLAHs Bestimmung?! ALLAHs Gnade und Seine Baraka seien über euch, ihr Angehörige des Hauses (von Ibrahim). Gewiß, ER ist alllobenswürdig, herrlich.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mdxlvii{Und nachdem die Furcht von Ibrahim verflogen war und bei ihm die frohe Botschaft eintraf, disputierte er mit Uns über Luts Leute.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mdxlviii{Gewiß, Ibrahim war zweifelsohne langmütig, anteilnehmend, reumütig.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mdxlix{Ibrahim! Laß ab davon! Gewiß, die Bestimmung deines HERRN traf bereits ein, und sie (Luts Leute) wird doch eine Peinigung überkommen, die unabwendbar ist.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mdl{Und als Unsere Gesandten bei Lut ankamen, wurde er wegen ihnen mißgestimmt und fühlte sich damit überfordert und sagte: ``Dies ist ein schwerer Tag!"\qt@no{77}}%
\qt@newcmd\qurantrans@de@mdli{Dann kamen seine Leute eilig zu ihm - und vorher begingen sie die gottmißfälligen Taten. Er sagte: ``Da sind meine Töchter, sie sind doch für euch reiner. Also handelt Taqwa gemäß ALLAH gegenüber und stellt mich nicht bloß vor meinen Gästen! Gibt es unter euch denn keinen verständigen Mann?!"\qt@no{78}}%
\qt@newcmd\qurantrans@de@mdlii{Sie sagten: ``Gewiß, bereits weißt du doch, daß wir nach deinen Töchtern kein Verlangen haben, und du weißt sicher genau, was wir wollen."\qt@no{79}}%
\qt@newcmd\qurantrans@de@mdliii{Er sagte: ``Würde ich doch über Macht gegen euch verfügen oder könnte ich doch Beistand bei einem mächtigen Unterstützenden ersuchen!"\qt@no{80}}%
\qt@newcmd\qurantrans@de@mdliv{Sie (die Gäste) sagten: ``Lut! Wir sind doch Gesandte deines HERRN, sie werden dir nichts antun können. Also brich nachts mit deiner Familie auf und keiner von euch soll sich umsehen - außer deiner Ehefrau, denn gewiß, sie wird das treffen, was diese trifft. Gewiß, das ihnen Angedrohte trifft am Morgen ein. Kommt der Morgen etwa nicht bald?!"\qt@no{81}}%
\qt@newcmd\qurantrans@de@mdlv{Als dann Unsere Bestimmung eintraf, haben WIR ihr Oberstes zuunterst gekehrt und ließen darüber hageln unablässig Steine aus Sidsch-dschil,\qt@no{82}}%
\qt@newcmd\qurantrans@de@mdlvi{die bei deinem HERRN (für Peinigung) gekennzeichnet sind. Und diese sind nicht fern von den Unrecht-Begehenden.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mdlvii{Und zu Madyan (entsandten WIR) ihren Bruder Schu'aib. Er sagte: ``Meine Leute! Dient ALLAH! Ihr habt doch keine Gottheit außer Ihm und mindert nicht das Maß und das Wiegen. Ich sehe doch, daß es euch gut geht, auch gewiß, ich fürchte um euch die Peinigung eines alles umfassenden Tages.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mdlviii{Und meine Leute! Vervollständigt das Maß und das Wiegen in gerechter Weise, gebt den Menschen das ihnen Zustehende nicht inkomplett und übertreibt nicht auf der Erde als Verderben-Anrichtende!\qt@no{85}}%
\qt@newcmd\qurantrans@de@mdlix{Das nach ALLAHs Gebot Übrigbleibende ist besser für euch, würdet ihr den Iman verinnerlichen. Doch bin ich kein Wachender über euch."\qt@no{86}}%
\qt@newcmd\qurantrans@de@mdlx{Sie sagten: ``Schu'aib! Sind es etwa deine Gebete, die dir gebieten, daß wir das unterlassen, was unsere Ahnen angebetet haben, oder daß wir mit unserem Vermögen das tun, was wir wollen?! Gewiß, du bist doch der Nachsichtige, der Verständige.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mdlxi{Er sagte: ``Meine Leute! Wie seht ihr es? Wenn ich über ein eindeutiges Zeichen von meinem HERRN verfüge und ER mir schönes Rizq gewährte, (sollte ich dies mit Haram vermischen)?! Und ich will nicht das Gegenteil von dem tun, was ich euch verbiete. Ich strebe nur die Korrektur an, wie ich kann. Meinen Erfolg gewährt mir nur ALLAH! Ihm gegenüber übe ich Tawakkul und zu Ihm kehre ich um.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mdlxii{Meine Leute! Lasst euren Widerspruch mir gegenüber euch nicht dazu bringen, daß euch ähnliches trifft, wie es die Leute von Nuh, die Leute von Hud oder die Leute von Salih traf - auch (die Peinigung von) Luts Leuten liegt von euch doch nicht weit zurück.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mdlxiii{Und bittet euren HERRN um Vergebung, dann seid Ihm gegenüber reumütig! Zweifelsohne, mein HERR ist allgnädig, liebevoll."\qt@no{90}}%
\qt@newcmd\qurantrans@de@mdlxiv{Sie sagten: ``Schu'aib! Wir begreifen nicht viel von dem, was du sagst - außerdem gewiß sehen wir, daß du doch unter uns ein Schwacher bist. Und wäre dein Stamm nicht, hätten wir dich gewiß gesteinigt, und du bist für uns sicherlich nicht unbezwingbar."\qt@no{91}}%
\qt@newcmd\qurantrans@de@mdlxv{Er sagte: ``Meine Leute! Genießt etwa mein Stamm bei euch mehr Ansehen als ALLAH - und ihr stellt Ihn hinter euch?! Zweifelsohne, mein HERR ist allumfassend dessen, was ihr tut.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mdlxvi{Meine Leute! Handelt euren Standpunkten entsprechend, auch ich handle. Ihr werdet noch wissen, wen Peinigung überkommt, welche ihn erniedrigt, und wer Lügner ist. Wartet nur ab! Ich warte ebenfalls mit euch ab."\qt@no{93}}%
\qt@newcmd\qurantrans@de@mdlxvii{Als dann Unsere Bestimmung eintraf, erretteten WIR Schu'aib und diejenigen, die mit ihm den Iman verinnerlichten, durch Gnade von Uns. Und diejenigen, die Unrecht begingen, überkam der Schrei und sie wurden in ihren Wohnstätten zu leblosen Körpern,\qt@no{94}}%
\qt@newcmd\qurantrans@de@mdlxviii{als ob sie sich dort nicht aufhielten. Ja! Nieder mit Madyan, wie es nieder mit Thamud war.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mdlxix{Und gewiß, bereits entsandten WIR doch Musa mit Unseren Ayat und einem eindeutigen Beweis\qt@no{96}}%
\qt@newcmd\qurantrans@de@mdlxx{zu Pharao und seinen Entscheidungsträgern. So folgten sie der Anordnung Pharaos. Jedoch die Anordnung Pharaos war nicht vernünftig.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mdlxxi{Er geht seinen Leuten voraus am Tag der Auferstehung, so wird er sie ins Feuer führen. Und erbärmlich ist die (von ihnen) erreichte Quelle.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mdlxxii{Und sie werden in diesem (Diesseits) mit Fluch verfolgt sowie am Tage der Auferstehung. Erbärmlich ist die (ihnen) mitgegebene Gabe.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mdlxxiii{Diese sind ein Teil der Berichte von den Ortschaften, die WIR dir mitteilen. Manche von ihnen (den Ortschaften) stehen noch und andere sind längst vergangen.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mdlxxiv{Und WIR taten ihnen kein Unrecht an, sondern sie begingen gegen sich selbst Unrecht. So haben ihre Götter, an die sie anstelle von ALLAH Bittgebete richteten, ihnen nichts genützt, als die Bestimmung deines HERRN kam. Sie haben ihnen nichts außer Verderb hinzugefügt.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mdlxxv{Und solcherart ist die Bestrafung deines HERRN, wenn ER die (Bewohner der) Ortschaften bestraft, während sie Unrecht begingen. Gewiß, Seine Bestrafung ist qualvoll, hart.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mdlxxvi{Gewiß, darin ist zweifelsohne eine Aya für jeden, der die Peinigung im Jenseits fürchtet. Dies ist ein Tag, zu dem dieMenschen versammelt werden. Und dies ist ein ereignisreicher Tag.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mdlxxvii{Und WIR schieben ihn nur bis zu einer berechneten Frist hinaus.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mdlxxviii{Am Tag, wenn er eintrifft, wird keine Seele außer mit Seiner Zustimmung sprechen. Manche von ihnen sind dann voller Kummer und manche freudig.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mdlxxix{Also hinsichtlich derjenigen, die voller Kummer sind, diese sind im Feuer, darin stöhnen sie und atmen schwer wieder ein.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mdlxxx{Darin bleiben sie ewig, wie die Himmel und die Erde (des Jenseits) bleiben - außer was dein HERR will. Gewiß, dein HERR macht, was ER will.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mdlxxxi{Und hinsichtlich derjenigen, die freudig waren, diese sind in der Dschanna. Darin bleiben sie ewig, wie die Himmel und die Erde (des Jenseits) bleiben - außer was dein HERR will. Dies ist eine ununterbrochene Gabe.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mdlxxxii{So habe keine Zweifel wegen dem, was diese dienen. Sie dienen nur so, wie ihre Ahnen vorher dienten. Und gewiß, WIR werden ihnen doch ihren Anteil geben - unvermindert.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mdlxxxiii{Und gewiß, bereits ließen WIR Musa die Schrift zuteil werden, dann wurde darin Uneinigkeit erzielt. Und gäbe es kein von deinem HERRN bereits gefälltes Wort, wäre unter ihnen längst gerichtet worden. Und gewiß, sie hegen doch über ihn (den Quran) Verdacht schleichende Zweifel.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mdlxxxiv{Und gewiß, allen wird dein HERR zweifelsohne ihre Taten voll vergelten. ER ist gewiß dessen, was sie tun, allkundig.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mdlxxxv{Also benimm dich geradlinig, wie es dir geboten wurde - auch jeder, der mit dir reumütig wurde, und übertretet nicht! ER ist dessen, was ihr tut, allsehend.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mdlxxxvi{Und neigt nicht zu denjenigen zu, die Unrecht begingen, sonst wird euch das Feuer treffen. Und für euch gibt es anstelle von ALLAH keine Wali, dann wird euch nicht beigestanden.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mdlxxxvii{Und verrichte ordnungsgemäß das rituelle Gebet an beiden Enden der Tageszeit und an einem dem Tag näher gelegenen Teil der Nacht! Gewiß, die gottgefälligen guten Taten tilgen die gottmißfälligen Taten. Dies ist eine Ermahnung für diejenigen, die sich ermahnen lassen.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mdlxxxviii{Und übe dich in Geduld, denn ALLAH läßt die Belohnung für die Muhsin doch nicht verloren gehen.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mdlxxxix{Hätte es doch aus den Generationen vor euch manche gegeben, die einsichtig waren, welche Verderben auf Erden verboten! Ausgenommen davon sind die Wenigen aus denen, die WIR erretteten. Diejenigen, die Unrecht begingen, folgten jedoch dem, worin sie Luxus fanden, und sie waren schwer Verfehlende.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mdxc{Und deinem HERRN gebührt es nicht, eine Ortschaft wegen Unrecht zugrunde zu richten, solange ihre Bewohner gottgefällig Gutes tun.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mdxci{Und hätte dein HERR es gewollt, hätte ER doch die Menschen zu einer einzigen Umma gemacht - sie aber bleiben uneins\qt@no{118}}%
\qt@newcmd\qurantrans@de@mdxcii{außer denjenigen, welchen dein HERR Gnade erwies - und dafür erschuf ER sie. Und die Bestimmung deines HERRN ist vollendet: ``Gewiß, ICH werde Dschahannam mit Dschinn und Menschen, von beiden füllen.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mdxciii{Und alles berichten WIR dir von den Berichten über die Gesandten, um damit dein Herz zu festigen. Und darin kam zu dir die Wahrheit, Ermahnung und Erinnerung für dieMumin.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mdxciv{Und sag zu denjenigen, die den Iman nicht verinnerlichen: ``Handelt euren Standpunkten entsprechend, wir handeln doch auch!\qt@no{121}}%
\qt@newcmd\qurantrans@de@mdxcv{Und wartet nur ab, wir warten doch auch ab!"\qt@no{122}}%
\qt@newcmd\qurantrans@de@mdxcvi{Und ALLAH gehört das Verborgene der Himmel und der Erde und Ihm unterliegen die Angelegenheiten - allesamt. So diene Ihm und übe Tawakkul Ihm gegenüber. Dein HERR ist nicht achtlos dem gegenüber, was ihr tut.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mdxcvii{\basmalahde Alif-lam-raa. Diese sind die Ayat der deutlichen Schrift.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdxcviii{Gewiß, WIR sandten sie als einen arabischen Quran hinab, damit ihr euch besinnt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdxcix{WIR berichten dir von den schöneren Begebenheiten, indem WIR dir diesen Quran durch Wahy zuteil werden ließen. Und sicherlich warst du vor ihm einer der Achtlosen!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdc{Erinnere daran) als Yusuf seinem Vater sagte: ``Mein Vater! Gewiß, ich hatte ein Traumgesicht mit elf Planeten, der Sonne und dem Mond  ich sah sie alle - sie vollzogen vor mir Sudschud."\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdci{Er sagte: ``Mein Söhnchen! Berichte deinen Brüdern nicht über dein Traumgesicht, sonst werden sie gegen dich böse Intrigen schmieden. Gewiß, der Satan ist sicherlich dem Menschen gegenüber ein erklärter Feind.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdcii{Und solcherart erwählt dich dein HERR, lehrt dir von der Traumdeutung und vervollständigt dir und der Familie von Ya'qub Seine Gaben, wie ER sie deinen beiden Großvätern Ibrahim und Ishaq vorher vervollständigte. Gewiß, dein HERR ist allwissend, allweise."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdciii{Gewiß, bereits in (der Begebenheit von) Yusuf und seinen Brüdern waren Ayat für die Fragenden.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdciv{Als sie sagten: ``Zweifelsohne, Yusuf und sein Bruder sind unserem Vater lieber als wir, obgleich wir eine (größere) Gruppe sind. Gewiß, unser Vater ist doch eindeutig irriger Ansicht.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdcv{Tötet Yusuf oder setzt ihn in irgendeinem fernen Land aus, so wird euch die Zuneigung eures Vaters sicher sein und ihr werdet nach ihm (Yusuf) gottgefällig guttuende Leute sein."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdcvi{Alsdann sagte einer von ihnen: ``Tötet Yusuf nicht, sondern werft ihn in die Tiefe des Brunnens hinein, so finden ihn einige Reisende, solltet ihr dies tun wollen."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdcvii{Sie (gingen zum Vater und) sagten: ``Unser Vater! Weshalb bringst du uns kein Vertrauen entgegen, was Yusuf angeht? Gewiß, wir wollen ihm doch nur Gutes.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdcviii{Laß ihn morgen mit uns gehen, (die Herde) hüten und sich vergnügen! Gewiß, wir werden auf ihn doch achten."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdcix{Er sagte: ``Es betrübt mich doch, daß ihr mit ihm weggeht, und ich fürchte, daß ihn der Wolf frißt, während ihr ihm gegenüber achtlos seid."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdcx{Sie sagten: ``Sollte der Wolf ihn fressen, obwohl wir eine (starke) Gruppe sind, gewiß würden wir doch zu den Unterlegenen gehören."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdcxi{Alsdann nahmen sie ihn mit und entschlossen sich, ihn in der Tiefe des Brunnens zu lassen. Dann ließen WIR ihm Wahy zuteil werden: ``Du wirst ihnen über diese ihre Angelegenheit doch noch kundtun." Und sie merkten es nicht.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdcxii{Und sie kamen dann zu ihrem Vater abends weinend,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdcxiii{sie sagten: ``Unser Vater! Wir gingen Wettrennen machen und ließen Yusuf bei unseren Sachen zurück, dann hat ihn der Wolf gefressen. Und du wirst uns sowieso keinen Glauben schenken, egal wie wahrhaftig wir sind."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdcxiv{Und sie brachten sein Hemd befleckt mit Blut als Vortäuschung. Er sagte: ``Nein, sondern euer Ego machte euch etwas leicht! (Ich übe mich) in klagloser Geduld. Und ALLAH ist Derjenige, Den ich um Beistand bitte wegen dem, was ihr schildert."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdcxv{Dann kamen Reisende vorbei, sie schickten ihren Mann zum Wasserholen, dann ließ er seinen Wassereimer herunter, er sagte: ``Was für ein Glück! Dies ist ein Junge." Dann versteckten sie ihn als Handelsware. Und ALLAH ist allwissend über das, was sie tun.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdcxvi{Dann verkauften sie ihn gegen ein geringes Entgelt - wenige Silberstücke. Und sie hatten kein Interesse an ihm.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdcxvii{Und derjenige, der ihn aus Ägypten kaufte, sagte seiner Gattin: ``Pflege mit ihm einen gütigen Umgang! Vielleicht wird er uns von Nutzen sein oder wir nehmen ihn als Kind an." Und solcherart festigten WIR (die Stellung von) Yusuf im Lande, und um ihn von der Traumdeutung zu lehren. Und ALLAH erfüllt zweifelsohne Seine Bestimmung. Doch die meisten Menschen wissen es nicht.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdcxviii{Und als er das Mannesalter erreichte, ließen WIR ihm Urteilsvermögen und Wissen zuteil werden. Und solcherart vergelten WIR den Muhsin.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdcxix{Diejenige, in deren Haus er war, suchte ihn dann gegen seine Neigung zu verführen. Sie verschloß die Türen und sagte: ``Du, komm doch!" Er sagte: ``Nein, ALLAH schütze mich davor! Doch, mein Gebieter pflegte mit mir gütigen Umgang. Gewiß, die Unrecht-Begehenden werden niemals erfolgreich sein."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdcxx{Und gewiß, bereits war sie im Begriff es mit ihm zu tun, auch er wäre im Begriff es mit ihr zu tun, hätte er das Zeichen seines HERRN nicht wahrgenommen. Solcherart damit WIR von ihm das Verwerfliche und die Unzucht fernhalten. Zweifelsohne, er gehört zu Unseren auserwählten Dienern.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdcxxi{Beide rannten dann zur Tür und sie zerriß sein Hemd von hinten, dann stießen beide auf ihren Ehemann an der Tür. Sie sagte: ``Was wäre die Bestrafung für denjenigen, der Verwerfliches deiner Ehefrau antun wollte, außer daß er ins Gefängnis geht oder qualvoller Peinigung unterzogen wird?!"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdcxxii{Er sagte: ``Sie suchte mich gegen meine Neigung zu verführen." Ein Zeuge von ihrer Familie bezeugte dann: ``Wenn sein Hemd von vorne zerrissen wurde, so sagte sie dieWahrheit und er gehört zu den Lügnern.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdcxxiii{Doch sollte sein Hemd von hinten zerrissen sein, so log sie und er gehört zu den Wahrhaftigen."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdcxxiv{Als er dann sah, daß sein Hemd von hinten zerrissen wurde, sagte er: ``Gewiß, dies ist von eurer List. Gewiß, eure List ist gewaltig.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdcxxv{Yusuf, verschweige dies! Und du (Ehefrau), bitte um Vergebung für deine Sünde. Gewiß, du warst eine der bewußt Verfehlenden."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdcxxvi{Und manche Frauen in der Stadt sagten: ``Die Ehefrau von Al-'aziz suchte ihren Diener gegen seine Neigung zu verführen, er ließ sie unbändig flammende Liebe für ihn empfinden. Gewiß, wir sehen sie sicherlich im offenkundigen Irrtum."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdcxxvii{Als sie dann von ihrem arglistigen Gerede hörte, schickte sie nach ihnen und bereitete für sie ein Bankett, dann gab sie jeder Einzelnen von ihnen ein Messer und sagte (zu Yusuf): ``Gehe zu ihnen (hinein)!" Und als sie ihn sahen, bewunderten sie ihn sehr, schnitten sich die Hände und sagten: ``Makelloser ALLAH! Dies ist sicherlich kein Mensch. Nein! Er ist bestimmt nur ein ehrwürdiger Engel."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdcxxviii{Sie sagte: ``Ihr (Frauen), dies ist derjenige, dessentwegen ihr mir Vorwürfe gemacht habt. Und gewiß, bereits suchte ich ihn gegen seine Neigung zu verführen, jedoch weigerte er sich. Doch sollte er nicht das tun, was ich von ihm verlange, wird er unweigerlich ins Gefängnis geschickt und gewiß zu den Verachteten gehören."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdcxxix{Er sagte: ``HERR! Das Gefängnis ist mir lieber als das, wozu sie mich auffordern. Und wenn DU ihre List von mir nicht abwendest, neige ich sonst zu ihnen und gehöre dann zu den Unfug-Treibenden."\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdcxxx{Dann erhörte sein HERR seine Bittgebete und wandte ihre List von ihm ab. Gewiß, ER ist Der Allhörende, Der Allwissende.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdcxxxi{Danach ging es ihnen auf, nachdem sie die Ayat gesehen hatten, daß sie ihn doch für eine Weile ins Gefängnis schicken.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdcxxxii{Und mit ihm kamen ins Gefängnis zwei Diener. Einer von ihnen sagte: ``Ich hatte ein Traumgesicht, in dem ich Weintrauben gepreßt habe." Und der andere sagte: Ich hatte ein Traumgesicht, in dem ich auf meinem Kopf Brot getragen habe, von dem die Vögel gefressen haben." (Beide sagten): ``Teile uns deren Deutung mit! Gewiß, wir sehen dich als einen der Muhsin."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdcxxxiii{Er sagte: ``Es wird euch keine Speise als Rizq gewährt, ohne daß ich euch deren Deutung mitteile, bevor sie euch zuteil wird. Dies ist ein Teil dessen, was mich mein HERR lehrte. Gewiß, ich verließ die Gemeinschaft von Leuten, die den Iman an ALLAH nicht verinnerlichen, und die dem Jenseits gegenüber Kufr betreiben,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdcxxxiv{und folgte der Gemeinschaft meiner Väter Ibrahim, Ishaq und Ya'qub. Es ziemt sich für uns nicht, daß wir ALLAH gegenüber irgend etwas an Schirk betreiben. Dies ist ein Teil von ALLAHs Gunst uns und den Menschen gegenüber. Doch die meisten Menschen erweisen sich undankbar.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdcxxxv{Meine Mitgefangenen! Sind etwa differierende Herren besser oder ALLAH, Der Einzige, Der Allbezwingende?!\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdcxxxvi{Ihr dient anstelle von Ihm eigentlich nichts außer Namen, die ihr und eure Ahnen nannten, wozu ALLAH niemals eine Bestätigung hinabsandte. Die Entscheidung liegt doch nur bei ALLAH. ER hat geboten, daß ihr nichts außer Ihm dient. Dies ist der geradlinige Din, doch die meisten Menschen wissen es nicht.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdcxxxvii{Meine Mitgefangenen! Hinsichtlich des einen von euch, so wird er seinem Gebieter Wein servieren. Doch hinsichtlich des anderen, so wird er gekreuzigt, dann werden die Vögel von seinem Kopf fressen. Die Angelegenheit, worauf ihr um Antwort bittet, ist bereits bestimmt worden."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdcxxxviii{Demjenigen von beiden, von dem er dachte, daß er doch gerettet wird, sagte er dann: ``Erwähne mich vor deinem Gebieter!" Doch der Satan hat ihn die Erwähnung vor seinem Gebieter vergessen lassen. So blieb er im Gefängnis noch ein paar Jahre.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdcxxxix{Der König sagte: ``Ich hatte ein Traumgesicht, in dem sieben fette Kühe von sieben mageren Kühen gefressen werden und sieben grüne Ähren und (sieben) andere Vertrocknete waren. Ihr Entscheidungsträger! Gebt mir eine Antwort auf mein Traumgesicht, wenn ihr das Traumgesicht interpretieren könnt!"\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdcxl{Sie sagten: ``Das ist ein Durcheinander von Träumen und wir haben doch keinerlei Kenntnisse über Traumdeutung."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdcxli{Und derjenige von den Beiden, der gerettet war und sich nach langer Zeit erinnerte, sagte: ``Ich kann euch seine Deutung mitteilen, so schickt mich nur!"\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdcxlii{``Yusuf, du Wahrhaftiger! Gib uns eine Antwort über sieben fette Kühe, die von sieben mageren Kühen gefressen werden, und über sieben grüne Ähren und (sieben) andere Vertrocknete, vielleicht kehre ich zu den Menschen zurück, vielleicht wissen sie es dann."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdcxliii{Er sagte: ``Ihr werdet den Acker wie gewohnt bestellen. Was ihr dann erntet, so belasst es in seinen Ähren, außer ein wenig von dem, was ihr speist.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdcxliv{Dann kommen danach sieben dürre Jahre, welche (euch) das, was ihr für sie vorbereitet habt, verbrauchen lassen, außer ein wenig von dem, was ihr aufbewahrt habt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdcxlv{Dann kommt danach ein Jahr, in dem den Menschen (mit Regen) geholfen wird und in dem sie keltern.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdcxlvi{Der König sagte: ``Bringt ihn mir!" Als der Entsandte dann zu ihm kam, sagte er: ``Kehre zurück zu deinem Gebieter und frage ihn, was mit den Frauen war, die sich die Hände geschnitten hatten? Gewiß, mein HERR ist über ihre List allwissend."\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdcxlvii{Er sagte: ``Was war euer Bestreben, als ihr Yusuf gegen seine Neigung zu verführen suchtet?" Sie sagten: ``Makelloser ALLAH! Wir kannten von ihm nichts Verwerfliches." Die Frau von Al-'aziz sagte: ``Nun ist dieWahrheit erkennbar. Ich suchte ihn gegen seine Neigung zu verführen und gewiß, er ist doch von den Wahrhaftigen!\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdcxlviii{Dies damit er weiß, daß ich ihn in Abwesenheit doch nicht verraten habe, und daß ALLAH die List der Verräter doch nicht gelten läßt.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdcxlix{Und ich spreche meinem Ego keine Unschuld ab, gewiß, das Ego gebietet doch Verwerfliches außer denjenigen, denen ALLAH Gnade erwies. Gewiß, mein HERR ist allvergebend, allgnädig."\qt@no{53}}%
\qt@newcmd\qurantrans@de@mdcl{Und der König sagte: ``Bringt ihn mir! Ich möchte ihn in meinen persönlichen Dienst stellen." Und als er mit ihm sprach, sagte er: ``Gewiß, du genießt heute bei uns eine besondere Stellung und Vertrauen."\qt@no{54}}%
\qt@newcmd\qurantrans@de@mdcli{Er sagte: ``Setze mich für die Schatzkammer des Landes ein. Gewiß, ich bin achtgebend, wissend."\qt@no{55}}%
\qt@newcmd\qurantrans@de@mdclii{Und solcherart festigten WIR (die Stellung von) Yusuf im Lande, dort hält er sich auf, wo er will. WIR lassen Unsere Gnade zuteil werden, wem WIR wollen. Und WIR lassen die Belohnung der Muhsin nicht verloren gehen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mdcliii{Gewiß, die Belohnung des Jenseits ist besser für diejenigen, die den Iman verinnerlichten und Taqwa gemäß zu handeln pflegten.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mdcliv{Dann kamen die Brüder von Yusuf und traten zu ihm ein. Er erkannte sie, während sie jedoch ihn nicht erkannten.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mdclv{Und nachdem er ihnen ihre Bedarfsgüter zur Verfügung stellte, sagte er: ``Bringt mir euren Halbbruder väterlicherseits! Seht ihr nicht, daß ich doch vollständige Zumessung erteile, und daß ich der beste Gastgeber bin?\qt@no{59}}%
\qt@newcmd\qurantrans@de@mdclvi{Und solltet ihr ihn mir nicht bringen, so erhaltet ihr bei mir keine Zumessung (mehr) und kommt nicht in meine Nähe!"\qt@no{60}}%
\qt@newcmd\qurantrans@de@mdclvii{Sie sagten: ``Wir werden seinen Vater trotz seiner Zuneigung ihm gegenüber zu überreden suchen. Gewiß, wir werden es doch tun."\qt@no{61}}%
\qt@newcmd\qurantrans@de@mdclviii{Und er sagte seinen Dienern: ``Packt ihre Tauschware in ihre Satteltaschen, damit sie diese erkennen, wenn sie zu ihren Familien zurückkehren, damit sie zurückkommen."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mdclix{Und nachdem sie zu ihrem Vater zurückgekehrt waren, sagten sie: ``Unser Vater! Die weitere Zumessung wurde uns verweigert, so schicke mit uns unseren Bruder, damit wir Zumessung erhalten. Gewiß, wir werden auf ihn doch achten."\qt@no{63}}%
\qt@newcmd\qurantrans@de@mdclx{Er sagte: ``Kann ich ihn euch etwa anders anvertrauen, als ich euch seinen Bruder vorher anvertraut habe? Doch ALLAH ist Der beste Beschützer und ER ist Der Allgnädigste der Gnade-Erweisenden."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mdclxi{Und als sie ihre Satteltaschen öffneten, stellten sie fest, daß ihre Tauschware ihnen zurückgegeben wurde. Sie sagten: ``Unser Vater! Was wollen wir noch? Dies ist unsere Tauschware, sie wurde uns zurückgegeben. Damit können wir dann unsere Familien versorgen, auf unseren Bruder achten und eine zusätzliche Ladung eines Lasttieres erhalten. Dieses ist eine leicht zu erhaltende Zumessung."\qt@no{65}}%
\qt@newcmd\qurantrans@de@mdclxii{Er sagte: Ich werde ihn mit euch nicht schicken, bis ihr mir ein Versprechen vor ALLAH gebt, daß ihr ihn mir zurückbringt, es sei denn, ihr geht selbst zugrunde." Und nachdem sie ihm ihr Versprechen gaben, sagte er: ``ALLAH ist Wakil über das, was wir sagen."\qt@no{66}}%
\qt@newcmd\qurantrans@de@mdclxiii{Und er sagte: ``Meine Söhne! Tretet nicht durch ein einziges Tor ein, sondern tretet durch verschiedene Tore ein! Ich kann euch von ALLAH nichts abwenden. Die Bestimmung obliegt nur ALLAH. Ihm gegenüber übe ich Tawakkul und Ihm gegenüber sollen die Tawakkul-Übenden Tawakkul üben."\qt@no{67}}%
\qt@newcmd\qurantrans@de@mdclxiv{Als sie dann eintraten, von wo ihr Vater es ihnen bestimmte, konnte dies ihnen nichts von ALLAH abwenden. Aber es war ein Anliegen im Herzen von Ya'qub, das er äußerte. Und er verfügt sicherlich über Wissen, das WIR ihn lehrten, doch die meisten Menschen wissen es nicht.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mdclxv{Und als sie bei Yusuf eintraten, nahm er seinen Bruder zu sich. Er sagte: Gewiß, ich bin dein Bruder, so sei nicht verbittert über das, was sie zu tun pflegten."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mdclxvi{Und nachdem er ihnen ihre Bedarfsgüter zur Verfügung gestellt hatte, steckte er den Trinkbecher in die Satteltasche seines Bruders. Dann rief ein Ausrufer aus: ``Ihr von der Karawane! Ihr seid zweifelsohne Diebe."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mdclxvii{Sie sagten und gingen auf sie zu: ``Was vermisst ihr denn?"\qt@no{71}}%
\qt@newcmd\qurantrans@de@mdclxviii{Sie sagten: ``Wir vermissen den Meßbecher des Königs." Und wer ihn zurückbringt, erhält (als Belohnung) eine Kamelladung, und dafür verbürge ich mich.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mdclxix{Sie sagten: ``Bei ALLAH! Gewiß, bereits wisst ihr, daß wir nicht kamen, um Verderben im Lande anzurichten, und daß wir keine Diebe sind."\qt@no{73}}%
\qt@newcmd\qurantrans@de@mdclxx{Sie sagten: ``Welche Bestrafung erhält er (der Täter), wenn ihr gelogen habt?"\qt@no{74}}%
\qt@newcmd\qurantrans@de@mdclxxi{Sie sagten: ``Seine Bestrafung ist: Derjenige, in dessen Satteltasche er gefunden wird, er selbst ist dann die Vergeltung dafür." Und solcherart vergelten wir denjenigen, die Unrecht begehen.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mdclxxii{Dann begann er (das Durchsuchen) mit ihren Transportbehältern vor dem Transportbehälter seines Bruders, dann zog er ihn aus dem Transportbehälter seines Bruders. Solcherart haben WIR es für Yusuf geplant. Denn er hätte niemals seinen Bruder nach dem Din des Königs festnehmen können, hätte ALLAH es nicht gewollt. WIR erhöhen um Stellungen, wen WIR wollen. Und über jedem Wissenden gibt es einen noch mehr Wissenden.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mdclxxiii{Sie sagten: ``Wenn er stiehlt, so stahl bereits sein Bruder vor ihm." Yusuf behielt es jedoch bei sich und offenbarte es ihnen nicht, er sagte (bei sich): ``Ihr seid sicherlich niedriger in der Stellung. Und ALLAH ist allwissend über das, was ihr darstellt."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mdclxxiv{Sie sagten: ``Al-'aziz! Er hat doch einen Vater, der ein Greis ist, so nimm einen von uns an seiner Stelle fest. Wir sehen dich doch als einen der Muhsin."\qt@no{78}}%
\qt@newcmd\qurantrans@de@mdclxxv{Er sagte: ``ALLAH bewahre, daß wir jemanden festnehmen außer denjenigen, bei dem wir unser Gut fanden, sonst wären wir doch Unrecht- Begehende."\qt@no{79}}%
\qt@newcmd\qurantrans@de@mdclxxvi{Und als sie an ihm jede Hoffnung verloren, zogen sie sich zurück, um unter sich zu beraten. Der Älteste unter ihnen sagte: ``Wisst ihr etwa nicht, daß euer Vater von euch vor ALLAH ein Versprechen hat, und was ihr vorher Yusuf gegenüber an Mißachtung entgegengebracht habt?! So werde ich das Land nicht verlassen, bis mein Vater es mir erlaubt oder ALLAH mir eine Entscheidung macht. ER ist Der Beste der Entscheidenden."\qt@no{80}}%
\qt@newcmd\qurantrans@de@mdclxxvii{Kehrt zurück zu eurem Vater, dann sagt: ``Unser Vater! Dein Sohn hat gestohlen, und wir haben nur das bezeugt, was wir wußten. Und wir hatten über das Verborgene keine Kenntnis.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mdclxxviii{Und frage in der Ortschaft, in der wir waren, und bei der Karawane, mit der wir zurückkamen. Und gewiß, wir sind doch wahrhaftig."\qt@no{82}}%
\qt@newcmd\qurantrans@de@mdclxxix{Er sagte: ``Nein, sondern euer Ego machte euch etwas leicht! Also (übe ich mich) in klagloser Geduld, vielleicht wird ALLAH sie alle mir zurückbringen. Gewiß, ER ist Der Allwissende, Der Allweise!"\qt@no{83}}%
\qt@newcmd\qurantrans@de@mdclxxx{Dann wandte er sich von ihnen ab und sagte: ``Weh um Yusuf." Und seine Augen wurden weiß von der Trauer. Und er hat es still erduldet.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mdclxxxi{Sie sagten: ``Bei ALLAH! Du wirst nie aufhören, Yusuf zu gedenken, bis du todgeweiht oder selbst unter den Toten bist."\qt@no{85}}%
\qt@newcmd\qurantrans@de@mdclxxxii{Er sagte: ``Nur ALLAH klage ich meinen Kummer und meine Trauer. Und ich weiß von ALLAH, was ihr nicht wisst.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mdclxxxiii{Meine Söhne! Geht und erkundigt euch nach Yusuf und seinem Bruder und verzweifelt nicht an den Ruhh von ALLAH, denn es zweifeln an den Ruhh von ALLAH nur die Kafir-Leute."\qt@no{87}}%
\qt@newcmd\qurantrans@de@mdclxxxiv{Und als sie bei ihm eintraten, sagten sie: ``Oh Al-'aziz! Wir und unsere Familie litten Not und wir brachten minderwertige Handelsware mit. Bitte gib uns voll-ständiges Maß und gewähre uns Sadaqa. Gewiß, ALLAH vergilt den Sadaqa- Gewährenden.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mdclxxxv{Er sagte: ``Ist euch bewußt geworden, was ihr Yusuf und seinem Bruder angetan habt, als ihr noch Unwissende wart?"\qt@no{89}}%
\qt@newcmd\qurantrans@de@mdclxxxvi{Sie sagten: ``Bist du etwa doch selbst Yusuf?!" Er sagte: ``Ich bin Yusuf und dies ist mein Bruder. Bereits erwies ALLAH uns Wohltaten. Gewiß, wer Taqwa gemäß handelt und sich in Geduld übt, so läßt ALLAH zweifelsohne die Belohnung der Muhsin nicht verloren gehen.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mdclxxxvii{Sie sagten: ``Bei ALLAH! Gewiß, bereits hat ALLAH dich vor uns bevorzugt und wir waren doch Verfehlende."\qt@no{91}}%
\qt@newcmd\qurantrans@de@mdclxxxviii{Er sagte: ``Es werden euch heute keine Vorwürfe gemacht. ALLAH möge euch vergeben. Und ER ist Der Allgnädigste aller Gnädigen.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mdclxxxix{Nehmt dieses mein Hemd mit, dann legt es auf meines Vaters Gesicht, dann wird er wieder sehen können, und bringt mir eure ganze Familie mit!"\qt@no{93}}%
\qt@newcmd\qurantrans@de@mdcxc{Und als die Karawane aufbrach, sagte ihr Vater: ``Ich rieche den Duft von Yusuf. Würdet ihr mich der Faselei nicht bezichtigen, (würdet ihr mir glauben)."\qt@no{94}}%
\qt@newcmd\qurantrans@de@mdcxci{Sie sagten: ``Bei ALLAH! Gewiß, du bleibst doch bei deinem alten Irrtum."\qt@no{95}}%
\qt@newcmd\qurantrans@de@mdcxcii{Und als der Freudenbote kam, hat er es (das Hemd) auf sein Gesicht gelegt, so konnte er wieder sehen. Er sagte: ``Habe ich euch nicht gesagt, daß ich von ALLAH weiß, was ihr nicht wisst?!"\qt@no{96}}%
\qt@newcmd\qurantrans@de@mdcxciii{Sie sagten: ``Unser Vater! Bitte für uns um die Vergebung unserer Verfehlungen, denn wir waren gewiß Verfehlende."\qt@no{97}}%
\qt@newcmd\qurantrans@de@mdcxciv{Er sagte: ``Ich werde meinen HERRN für euch um Vergebung bitten. Gewiß, ER ist Der Allvergebende, Der Allgnädige."\qt@no{98}}%
\qt@newcmd\qurantrans@de@mdcxcv{Und als sie bei Yusuf eintraten, nahm er seine Eltern zu sich und sagte: ``Tretet in Ägypten ein, so ALLAH will, als Sicherheitsgenießende."\qt@no{99}}%
\qt@newcmd\qurantrans@de@mdcxcvi{Dann erhob er seine Eltern auf den Thron und sie (die Brüder) fielen vor ihm in Sudschud nieder. Er sagte: ``Mein Vater! Dies ist die Deutung meines Traumgesichts von vorher, bereits ließ mein HERR es bewahrheiten. Und bereits hat ER mir Gutes erwiesen, als ER mich vom Gefängnis entlassen und euch von der Wüste kommen ließ, nachdem der Satan Zwietracht zwischen mir und meinen Brüdern stiftete. Gewiß, mein HERR ist allgütig dem gegenüber, was ER will. Gewiß, ER ist Der Allwissende, Der Allweise.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mdcxcvii{Mein HERR! DU hast mir vom Königtum zuteil werden lassen und lehrtest mich von der Deutung der Träume - DU Schöpfer der Himmel und Erde -, DU bist mein Wali im Diesseits und im Jenseits, lasse mich als Muslim sterben und den gottgefällig Guttuenden folgen!"\qt@no{101}}%
\qt@newcmd\qurantrans@de@mdcxcviii{Dies ist von den Mitteilungen des Verborgenen, die WIR dir als Wahy zuteil werden lassen. Und du warst nicht bei ihnen, als sie sich entschlossen haben, während sie die List planten.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mdcxcix{Und die meisten Menschen - selbst dann, solltest du dich bemühen -, werden keine Mumin sein.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mdcc{Und du verlangst von ihnen dafür keine Belohnung. Es ist nur eine Ermahnung für die ganze Schöpfung.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mdcci{Und wie viele von den Ayat gibt es in den Himmeln und auf der Erde, bei denen sie vorbeigehen, und sie ihnen gegenüber uninteressiert bleiben.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mdccii{Und die meisten von ihnen bekunden nicht den Iman an ALLAH, es sei denn, sie bleiben Muschrik.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mdcciii{Fühlen sie sich etwa davor sicher, daß sie eine Peinigung von ALLAH überzieht, oder daß die Stunde sie überrascht, während sie es nicht merken?!\qt@no{107}}%
\qt@newcmd\qurantrans@de@mdcciv{Sag: ``Dies ist mein Weg, ich lade zu ALLAH mit Einblick ein - ich und jeder, der mir gefolgt ist. Und subhanallah und ich gehöre nicht zu den Muschrik."\qt@no{108}}%
\qt@newcmd\qurantrans@de@mdccv{Und WIR entsandten vor dir nur Männer, denen WIR Wahy zuteil werden ließen, die zu den Bewohnern der Ortschaften zählten. Sind sie etwa nicht durch das Land gezogen, damit sie wahrnehmen, wie das Anschließende von denjenigen war, die vor ihnen lebten?! Und das Jenseits ist gewiß besser für diejenigen, die Taqwa gemäß gehandelt haben. Wollt ihr euch etwa nicht besinnen?!\qt@no{109}}%
\qt@newcmd\qurantrans@de@mdccvi{Als die Gesandten die Hoffnung aufgaben und dachten, daß sie bereits getäuscht wurden, da kam zu ihnen unser Sieg. Dann wurde gerettet, wen WIR wollen. Und Unser Gewalt-Antun kann von den schwer verfehlenden Leuten niemals abgewendet werden.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mdccvii{Gewiß, bereits in ihren Begebenheiten war eine Lehre für diejenigen mit Verstand. Es war keine erfundene Erzählung, sondern er (der Quran) ist eine Bestätigung dessen, was vor ihm war, Erläuterung aller Dinge und Rechtleitung und Gnade für Menschen, die den Iman verinnerlichen.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mdccviii{\basmalahde Alif-lam-mim-raa. Diese sind die Ayat der Schrift. Und das, was dir von deinem HERRN hinabgesandt wurde, ist dieWahrheit! Doch diemeisten Menschen verinnerlichen den Iman nicht.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdccix{ALLAH ist Derjenige, Der die Himmel ohne von euch zu sichtende Stützen hochhielt, dann wandte ER Sich Al'ahrsch zu und machte die Sonne und den Mond gratis fügbar. Alles bewegt sich bis zu einem festgelegten Zeitpunkt. ER verwaltet die Angelegenheit (der Schöpfung)! ER erläutert die Ayat, damit ihr an der Begegnung eures HERRN Gewißheit habt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdccx{Und ER ist Derjenige, Der die Erde einebnete und in ihr Festigende (Berge) und Flüsse machte. Und von allen Früchten ließ ER auf ihr Zweiheiten sein. ER läßt die Nacht den Tag überdecken. Gewiß, darin sind sicherlich Ayat für Menschen, die nachdenken.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdccxi{Und auf der Erde sind benachbarte (aber unterschiedliche) Landstriche und Dschannat von Rebstöcken, Gewächs und Sinwan-Palmen und Nicht-Sinwan-Palmen, die mit dem gleichen Wasser bewässert werden, doch WIR machten manche von ihnen vor den anderen bevorzugt zum Verzehr. Gewiß, darin sind sicherlich Ayat für Menschen, die sich besinnen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdccxii{Und solltest du dich wundern, so ist ziemlich kurios ihre Aussage: ``Werden wir etwa, nachdem wir zur Erde wurden, doch neu erschaffen?" Diese sind diejenigen, die ihrem HERRN gegenüber Kufr betrieben haben. Und diese sind es, um deren Hälse Eisenringe sein werden. Und diese sind die Weggenossen des Feuers. Darin bleiben sie ewig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdccxiii{Und sie verlangen von dir zu beschleunigen das Übel vor dem Gutem. Doch bereits wurden vor ihnen Exempel statuiert. Gewiß, dein HERR ist sicherlich voller Vergebung den Menschen gegenüber trotz ihres Unrechttuns. Und gewiß, dein HERR ist ziemlich hart im Strafen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdccxiv{Und diejenigen, die Kufr betrieben haben, sagen: ``Würde ihm doch eine Aya von seinem HERRN hinabgesandt!" Du bist nur ein Warner, und für jedes Volk gibt es einen Rechtleitenden.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdccxv{ALLAH weiß, was jedeWeibliche (während der Schwangerschaft) in sich trägt, was die Mutterleibe verkürzen (von der Zeit der Schwangerschaft) und was sie dazu hinzufügen. Und alle Dinge sind bei Ihm nach Maß.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdccxvi{ER ist Der Allwissende über das Verborgene und das Sichtbare, Der Große, Der Überragende.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdccxvii{Es bleibt gleich für jeden von euch, ob man das Gesprochene bei sich behält oder hörbar sagt oder ob man sich in (der Dunkelheit) der Nacht verbirgt oder sich im Tageslicht bewegt,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdccxviii{für jeden gibt es Bewacher vor sich und hinter sich, die ihn mit ALLAHs Bestimmung bewahren. Gewiß, ALLAH ändert nichts von dem worin Menschen sind, bis sie das ändern, was sie in sich tragen. Und wenn ALLAH für Menschen Übel bestimmt, so kann es nicht aufgehalten werden. Und sie haben anstelle von Ihm keinen Wali.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdccxix{ER ist Derjenige, Der euch den Blitzstrahl sichtbar macht - furchterregend und hoffungserfüllend, und ER bildet die übersättigten Wolken.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdccxx{Und der Donner lobpreist mit Seinem Lob und die Engel lobpreisen aus Furcht vor Ihm. Und ER läßt die Donnerschläge schicken, um damit zu treffen, wen ER will. Sie disputieren jedoch über ALLAH, während ER gewaltig im Vernichten ist.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdccxxi{Ihm alleine gebührt das wahre Bittgebet. Und diejenigen, die sie anstelle von Ihm bitten, erfüllen ihnen nichts außer dem, was demjenigen (erfüllt wird), der seine Hände zum Wasser ausstreckt, damit es seinen Mund erreicht, wobei es ihn nicht erreicht. Und das Bittgebet der Kafir ist nur verfehlt.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdccxxii{Und für ALLAH vollziehen Sudschud alle, die in den Himmeln und auf Erden sind - aus freien Stücken und widerwillig - auch ihre Schatten morgens und abends.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdccxxiii{Sag: ``Wer ist Der HERR der Himmel und der Erde?" Sag: ``ALLAH!" Sag: ``Setzt ihr euch anstelle von Ihm etwa Wali ein, welche sich selbst weder nützen noch schaden können?!" Sag: ``Sind etwa der Blinde und der Sehende gleich? Oder sind etwa die Finsternis und das Licht gleich?" Oder haben sie ALLAH Partner beigesellt, welche Gleiches wie Seine Schöpfung erschufen, so daß ihnen die Schöpfung ähnlich zu sein schien?! Sag: ``ALLAH ist der Schöpfer allen Seins, und ER ist Der Einzigartige, Der Allbezwingende."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdccxxiv{ER ließ vom Himmel Wasser fallen, dann floß es durch Täler ihrer Größe nach, dann trug das strömende Wasser Schaum auf der Oberfläche. Auch von dem, was sie im Feuer schmelzen zur Herstellung von Schmuck und Utensilien, gibt es ihm ähnlichen Schaum. Solcherart prägt ALLAH Gleichnisse von der Wahrheit und von dem für nichtig Erklärten. Also hinsichtlich des Schaums, so wird er beiseite geworfen, doch hinsichtlich dessen, das den Menschen nützt, so verbleibt es in der Erde. Solcherart prägt ALLAH die Gleichnisse.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdccxxv{Für diejenigen, die sich ihrem HERRN fügten, gibt es das Beste. Und diejenigen, die sich Ihm nicht fügten - gehörte ihnen alles und noch einmal soviel dazu von dem, was auf der Erde ist, würden sie sich damit gewiß (vor der Peinigung) freikaufen. Für diese ist das Übel des Zur-Rechenschaft-Ziehens bestimmt, und ihre Unterkunft ist Dschahannam. Und was für ein erbärmlicher Aufenthaltsort!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdccxxvi{Ist derjenige, der weiß, daß dir von deinem HERRN nichts außer der Wahrheit hinabgesandt wurde, etwa gleich demjenigen, der blind ist?! Doch nur die verständigen Menschen besinnen sich.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdccxxvii{Es sind diejenigen, die das ihnen von ALLAH Auferlegte erfüllen und das Gelöbnis nicht verletzen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdccxxviii{Auch diejenigen, die das verbinden, dessen Verbindung ALLAH gebot, ihrem HERRN gegenüber ehrfürchtig sind und sich vor dem Übel des Zur-Rechenschaft-Ziehens fürchten.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdccxxix{Ebenso diejenigen, die sich in Geduld übten, um das Wohlgefallen ALLAHs zu erlangen, die das rituelle Gebet ordnungsgemäß verrichteten, die vom Rizq gaben, das WIR ihnen gewährten - heimlich und öffentlich - und die mit Gutem das Verwerfliche abwehren, für diese ist Belohnung nach dem Diesseits bestimmt.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdccxxx{Dies ist Dschannat von 'Adn, in die sie und die gottgefällig Guttuenden von ihren Ahnen, ihren Partnerwesen und ihrer Nachkommenschaft eintreten. Und die Engel treten zu ihnen durch jede Tür ein:\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdccxxxi{``Salam sei mit euch für das, was ihr an Geduld geübt habt. Was für eine erfreuliche Belohnung nach dem Diesseits!"\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdccxxxii{Und diejenigen, die das ihnen von ALLAH Auferlegte verletzen, nachdem sie es als verbindlich anerkannten, das abbrechen, dessen Verbindung ALLAH vorgeschrieben hat, und auf der Erde Verderben anrichten. Für diese ist Fluch vorgesehen, und für sie ist das Übel nach dem Diesseits bestimmt.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdccxxxiii{ALLAH gewährt viel Rizq, wem ER will, und läßt es wenig sein. Und sie freuten sich über das diesseitige Leben. Doch das diesseitige Leben ist im Jenseits nur ein (vergängliches) Gebrauchsgut.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdccxxxiv{Und diejenigen, die Kufr betrieben haben, sagen: ``Würde ihm doch eine Aya von seinem HERRN hinabgesandt!" Sag: ``Gewiß ALLAH läßt abirren, wen ER will, und leitet zu Sich denjenigen recht, der sich reumütig besinnt.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdccxxxv{Dies sind diejenigen, die den Iman verinnerlichten, und deren Herzen bei ALLAHs Gedenken Ruhe finden. Ja! Beim Gedenken ALLAHs finden die Herzen Ruhe.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdccxxxvi{Diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, für sie gibt es Glückseligkeit und schöne Rückkehr.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdccxxxvii{So entsandten WIR dich als einen Gesandten zu einer Umma, vor der bereits andere Umam vergingen, damit du ihnen das vorträgst, was WIR dir als Wahy zuteil werden ließen, während sie Dem Allgnade Erweisenden gegenüber Kufr betreiben. Sag: ``ER ist mein HERR, es gibt keine Gottheit außer Ihm! Ihm gegenüber übe ich Tawakkul und zu Ihm ist meine Umkehr.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdccxxxviii{Und (sie werden den Iman nicht verinnerlichen), auch dann (nicht), sollte ein Quran hinabgesandt werden, mit dem die Berge in Bewegung gesetzt würden oder die Landschaft gespalten würde oder mit den Toten gesprochen würde. Nein, sondern ALLAH gehört die Bestimmung insgesamt. Wollen diejenigen, die den Iman verinnerlichten, etwa nicht zur Kenntnis nehmen, daß wenn ALLAH wollte, ER alle Menschen rechtleitete?! Und diejenigen, die Kufr betrieben haben, wird immer wieder eine Katastrophe treffen für das, was sie bewerkstelligten, oder sie geschieht in der Nähe ihrer Heimat, bis das von ALLAH Versprochene eintrifft. Gewiß, ALLAH bricht nicht Sein Versprechen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdccxxxix{Und gewiß, bereits wurden vor dir Gesandte verspottet, dann gewährte ICH denjenigen, die Kufr betrieben haben, noch längere Zeit, dann beseitigte ICH sie. Und wie war Meine Bestrafung?!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdccxl{Ist etwa derjenige, der über jede Seele und das, was sie erwirbt, wacht, (gleich dem, der dies nicht kann)?! Und sie gesellten ALLAH (angebliche) Partner bei. Sag: ``Nennt sie doch!" Oder macht ihr Ihm etwa Mitteilung über das, was ER auf der Erde nicht kennt oder sind dies nur Worthülsen?! Nein, sondern denjenigen, die Kufr betrieben haben, wurde ihre List schön gemacht und sie wurden vom rechten (Weg) abgehalten. Und wen ALLAH abirren läßt, für ihn gibt es gewiß keinen Rechtleitenden.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdccxli{Für sie ist Peinigung im diesseitigen Leben bestimmt. Doch die Peinigung im Jenseits ist noch qualvoller. Und sie haben vor ALLAH keinen Schutz-Gewährenden.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdccxlii{Das Gleichnis der Dschanna, welche den Muttaqi versprochen wurde, ist (eine Dschanna), die von Flüssen durchflossen wird, ihre Früchte sind andauernd, ebenso ihre Schatten. Dies ist die Belohnung für diejenigen, die Taqwa gemäß handelten. Und die Vergeltung für die Kafir ist das Feuer.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdccxliii{Und diejenigen, denen WIR die Schrift zuteil werden ließen, freuen sich über das, was dir hinabgesandt wurde. Und unter Al-ahzab sind manche, die einen Teil von ihm abstreiten. Sag: ``Mir wurde geboten, daß ich nur ALLAH diene und Ihm gegenüber keinen Schirk betreibe. Zu Ihm richte ich meine Bittgebete und zu Ihm ist meine Rückkehr."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdccxliv{Und genauso haben WIR ihn als einen Richtenden auf Arabisch hinabgesandt. Und solltest du ihren Neigungen folgen nach dem Wissen, das dir zuteil wurde, so hast du vor ALLAH weder Wali noch Schutz-Gewährenden.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdccxlv{Und gewiß, bereits entsandten WIR vor dir Gesandte und gaben ihnen Ehefrauen und Nachwuchs. Und keinem Gesandten gebührt es, eine Aya zu bringen, außer mit ALLAHs Zustimmung. Für jede Zeitspanne gibt es eine Schrift.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdccxlvi{ALLAH löscht aus, was ER will, und bestätigt es. Und bei Ihm ist der Ursprung der Schrift.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdccxlvii{Und (ganz gleich) ob WIR dir einen Teil von dem zeigen, was WIR ihnen androhen, oder ob WIR dich sterben lassen, dir ist nur das Verkünden geboten, und Uns ist das Zur-Rechenschaft-Ziehen.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdccxlviii{Nehmen sie etwa nicht wahr, daß WIR Uns doch der Erde zuwenden und sie bei ihren Enden weniger machen. Und ALLAH bestimmt, es gibt niemanden, der Seine Bestimmung annulliert. Und ER ist schnell im Zur-Rechenschaft-Ziehen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdccxlix{Und bereits haben diejenigen vor ihnen intrigiert. Und ALLAH entgegnet alle Intrigen. ER weiß, was jede Seele erwirbt. Und die Kafir werden noch wissen, wem die Belohnung nach dem Diesseits gehört.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdccl{Und diejenigen, die Kufr betrieben haben, sagen: ``Du bist kein Gesandter." Sag: ``ALLAH genügt als Zeuge zwischen mir und euch, und jeder, der Wissen über die Schrift besitzt."\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdccli{\basmalahde Alif-lam-raa. Dies ist die Schrift, die WIR dir hinabgesandt haben, damit du die Menschen von der Finsternis ins Licht führst - mit der Zustimmung Ihres HERRN - auf den Weg Des Allwürdigen, Des Alllobenswürdigen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdcclii{ALLAH, Desjenigen, Dem alles gehört, was in den Himmeln und auf Erden ist. Und Untergang sei den Kafir wegen einer qualvollen Peinigung.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdccliii{Dies sind diejenigen, die das diesseitige Leben vor dem Jenseits bevorzugen, vom Wege ALLAHs abhalten und diesen verunstalten wollen. Diese sind im tiefen Irregehen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdccliv{Und WIR haben keinen Gesandten entsandt außer mit der Sprache seiner Leute, damit er ihnen (die Botschaft) erläutert. So läßt ALLAH abirren, wen ER will, und leitet recht, wen ER will. Und ER ist Der Allwürdige, Der Allweise.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdcclv{Und gewiß, bereits haben WIR Musa mit Unseren Ayat entsandt: ``Führe deine Leute von den Finsternissen ins Licht und erinnere sie an ALLAHs Wohltaten." Gewiß, darin sind zweifelsohne Ayat für jeden sich äußerst in Geduld Übenden, äußerst Dankbaren.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdcclvi{Und (erinnere daran), als Musa seinen Leuten sagte: ``Erinnert euch an ALLAHs Wohltat euch gegenüber, als ER euch vor Pharaos Leuten errettete, die euch das Übelste an Mißhandlung zufügten, eure Söhne abschlachteten und (nur) eure Frauen am Leben ließen. Darin lag wahrhaftig eine schwere Prüfung von eurem HERRN."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdcclvii{Und (erinnert euch daran), als euer HERR nachdrücklich ankündigen ließ: ``Wenn ihr euch dankbar erweist, so werde ICH euch noch mehr geben. Doch wenn ihr Kufr betreibt, so ist Meine Peinigung gewiß qualvoll."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdcclviii{Und Musa sagte: ``Solltet ihr und alle, die auf Erden sind, Kufr betreiben, so ist ALLAH gewiß doch autark, überreich belohnend."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdcclix{Wurde euch etwa keine Mitteilung zuteil über diejenigen, die vor euch waren - die Leute von Nuh, 'Aad und Thamud - und diejenigen nach ihnen, die nur ALLAH kennt?! Zu ihnen kamen ihre Gesandten mit deutlichen Zeichen, dann hielten sie ihre Hände vor ihre Münder und sagten: ``Gewiß, wir betreiben Kufr dem gegenüber, womit ihr entsandt wurdet, und wir haben am dem, wozu ihr uns auffordert, ja Verdacht schleichende Zweifel."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdcclx{Ihre Gesandten sagten: ``Gibt es etwa über ALLAH Zweifel, Den Schöpfer der Himmel und der Erde? ER ruft euch, damit ER euch von euren Verfehlungen vergibt und euch bis zu einer festgesetzten Frist Aufschub gewährt." Sie sagten: ``Ihr seid doch nichts anderes als Menschen wie wir. Ihr wollt uns davon abbringen, dem zu dienen, was unsere Ahnen zu dienen pflegten, also erbringt uns eine eindeutige Bestätigung!"\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdcclxi{Ihre Gesandten sagten ihnen: Wir sind bestimmt nur Menschen wie ihr. Doch ALLAH erweist Gnade, wem ER will von Seinen Dienern. Und uns steht es nicht zu, euch eine Bestätigung zu erbringen, es sei denn mit ALLAHs Zustimmung. Und ALLAH gegenüber sollen die Mumin Tawakkul üben.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdcclxii{Und weshalb sollen wir nicht ALLAH gegenüber Tawakkul üben, wo ER uns bereits auf unseren Wegen rechtleitete?! Und wir werden uns doch in Geduld üben dem gegenüber, was ihr uns an Kränkungen zufügt. Und ALLAH gegenüber sollen die Tawakkul-Übenden Tawakkul üben."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdcclxiii{Und diejenigen, die Kufr betrieben haben, sagten zu ihren Gesandten: ``Wir werden euch unweigerlich aus unserem Land vertreiben, oder ihr kehrt doch in unsere Glaubensgemeinschaft zurück." Dann hat ihr HERR ihnen Wahy zuteil werden lassen: ``Gewiß, WIR werden die Unrecht-Begehenden zugrunde richten.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdcclxiv{Und WIR werden euch bestimmt das Land nach ihnen bewohnen lassen. Dies ist für denjenigen, der Ehrfurcht vor der Vorstellung Mir gegenüber hat, und der Meine Androhung fürchtet."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdcclxv{Und sie baten um die Entscheidung, dann ging zugrunde jeder eigensinnige Gewalttäter.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdcclxvi{Und auf ihn wartet Dschahannam, und er wird mit Wasser voll mit Eiter getränkt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdcclxvii{Er nippt an ihm und kann es kaum herunterschlucken. Und der Tod umgibt ihn aus allen Richtungen, doch er wird nicht sterben. Und auf ihn wartet noch eine überharte Peinigung.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdcclxviii{Das Gleichnis derjenigen, die Kufr ihrem HERRN gegenüber betrieben haben, lautet: ``Ihre Werke ähneln Asche, über der sich der Wind an einem stürmischen Tag tummelt." Sie verfügen über nichts von dem, was sie erworben haben. Dies ist das weite Irregehen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdcclxix{Hast du etwa nicht wahrgenommen, daß ALLAH die Himmel und die Erde in Gesetzmäßigkeit erschaffen hat?! Wenn ER wollte, hätte ER euch vergehen lassen und eine neue Schöpfung hervorgebracht.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdcclxx{Und dies ist ALLAH nicht unmöglich.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdcclxxi{Und sie kamen vor ALLAH allesamt. Die Schwachen sagten dann zu denjenigen, die sich in Arroganz erhoben haben: ``Wir waren doch Gefolgsleute von euch! Könnt ihr von uns (nicht) etwas von ALLAHs Peinigung abwehren?" Sie sagten: ``Hätte ALLAH uns rechtleiten lassen, hätten wir euch gewiß rechtgeleitet. Es ist uns gleich, ob wir uns in Ungeduld oder in Geduld üben. Für uns gibt es kein Entrinnen mehr."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdcclxxii{Und der Satan sagte, nachdem die Angelegenheit bestimmt worden war: ``Zweifelsohne, ALLAH gab euch das wahre Versprechen! Doch ich habe euch etwas versprochen und nicht eingehalten. Aber ich hatte keine Verfügung über euch außer, daß ich euch (zum Kufr) aufforderte, und ihr mir gefolgt seid. So macht mir keine Vorwürfe, sondern macht euch selbst Vorwürfe! Weder ich kann euch behilflich sein, noch könnt ihr mir behilflich sein. Ich betreibe Kufr dem gegenüber, was ihr mit mir vorher an Schirk betrieben habt." Gewiß, für die Unrecht-Begehenden ist eine qualvolle Peinigung bestimmt.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdcclxxiii{Und diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, werden in Dschannat eintreten, die von Flüssen durchflossen sind - darin bleiben sie ewig - mit der Zustimmung ihres HERRN. Ihr Gruß ist darin: ``Salam (sei mit euch)."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdcclxxiv{Hast du etwa nicht gesehen, wie ALLAH ein Gleichnis prägte?! (ER setzte) ein gutes Wort einem guten Baum gleich, dessen Wurzelwerk fest ist und dessen Äste in den Himmel ragten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdcclxxv{Er trägt seine Früchte zu jeder Zeitspanne - mit der Zustimmung seines HERRN. Und ALLAH prägt den Menschen Gleichnisse, damit sie sich besinnen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdcclxxvi{Und ein schlechtes Wort ähnelt einem schlechten Baum, der vom Boden entwurzelt wird und nicht verankert war.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdcclxxvii{ALLAH gewährt Festigkeit denjenigen, die den Iman verinnerlicht haben, durch feststehende Worte im diesseitigen Leben und im Jenseits. Und ALLAH läßt die Unrecht-Begehenden abirren. Und ALLAH macht, was ER will.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdcclxxviii{Hast du etwa nicht gesehen diejenigen, welche die Gaben ALLAHs mit Kufr vertauschten und ihre Leute in die Stätte des Untergangs brachten?!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdcclxxix{Es ist Dschahannam, darin werden sie hineingeworfen. Und erbärmlich ist der Aufenthaltsort!\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdcclxxx{Und sie schrieben ALLAH etwas Ebenbürtiges zu, damit sie von Seinem Weg abhalten. Sag: ``Lasst euch nur vergnügen. Denn euer Werden führt unweigerlich ins Feuer."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdcclxxxi{Sag zu meinen Dienern, die den Iman verinnerlicht haben, daß sie das rituelle Gebet ordnungsgemäß verrichten und vom Rizq geben, das WIR ihnen gewährten - heimlich und öffentlich, bevor ein Tag eintrifft, an dem weder Kaufen noch Freundschaften gelten.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdcclxxxii{ALLAH ist Derjenige, Der die Himmel und die Erde erschuf und vom Himmel Wasser fallen ließ, und damit Früchte hervorbrachte als Rizq für euch. Und ER machte euch die Schiffe gratis fügbar, damit sie auf dem Meer nach Seiner Bestimmung fahren. Auch machte ER euch die Flüsse gratis fügbar.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdcclxxxiii{Ebenso machte ER euch die Sonne und den Mond gratis fügbar, in ständiger Bewegung. Auch machte ER euch die Nacht und den Tag gratis fügbar.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdcclxxxiv{ER gab euch noch von alledem, worum ihr Ihn gebeten habt. Und würdet ihr die Gaben ALLAHs aufzählen wollen, würdet ihr sie nie umfassend erfassen können. Gewiß, der Mensch ist doch äußerst unrecht-begehend, äußerst undankbar.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdcclxxxv{Und (erinnere daran), als Ibrahim sagte: ``Mein HERR! Mache diesen Ort sicher und lasse es mir und meinen Söhnen fern sein, daß wir die Götzen anbeten!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdcclxxxvi{Mein HERR! Gewiß, sie haben sehr viele von den Menschen in die Irre geführt. Also wer mir folgt, der gehört zweifelsohne zu mir, und wer mir widerspricht - so bist DU gewiß allvergebend, allgnädig.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdcclxxxvii{Unser HERR! Ich ließ von meiner Nachkommenschaft in einem unfruchtbaren Tal nieder - bei Deinem Muharram-Haus, unser HERR! - damit sie das rituelle Gebet ordnungsgemäß verrichten. So lasse Herzen von Menschen ihnen zugeneigt sein und gewähre ihnen Rizq von den Früchten, damit sie sich dankbar erweisen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdcclxxxviii{Unser HERR! DU weißt zweifellos, was wir verbergen und was wir offenlegen. Und ALLAH bleibt nichts verborgen, weder auf Erden, noch im Himmel.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdcclxxxix{Alhamdulillah: Alles Lob gebührt ALLAH, Der mir im hohen Alter Isma'il und Ishaq schenkte. Gewiß, mein HERR ist zweifelsohne des Bittgebets Allerhörend.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdccxc{Mein HERR! Laß mich des rituellen Gebets ordnungsgemäßer Verrichtender sein sowie von meiner Nachkommenschaft. Mein HERR! Und erfülle mein Bittgebet!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdccxci{Mein HERR! Vergib mir, meinen Eltern und den Mumin am Tag, wenn das Zur-Rechenschaft- Ziehen feststeht."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdccxcii{Und denke niemals, daß ALLAH dem gegenüber achtlos ist, was die Unrecht- Begehenden tun. ER hält sie nur für einen Tag zurück, an dem die Blicke erstarren werden.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdccxciii{Sie werden ängstlich hineilen mit hochgehaltenen Köpfen, ohne daß ihre Wimpern dabei auch nur zucken. Und sie sind verwirrt.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdccxciv{Und warne die Menschen vor einem Tag, an dem ihnen die Peinigung zuteil wird, dann werden diejenigen, die Unrecht begingen, sagen: ``Unser HERR! Halte uns für eine kurze Zeit zurück, dann werden wir Deiner Botschaft annehmen und den Gesandten folgen." Habt ihr etwa vorher nicht zu schwören gepflegt, daß ihr nicht vergehen würdet?!\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdccxcv{Und ihr habt euch in den Wohnstätten derjenigen niedergelassen, die gegen sich selbst Unrecht begingen, und euch wurde ersichtlich, was WIR mit ihnen machten, und WIR prägten euch noch die Gleichnisse.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdccxcvi{Und bereits haben sie ihre Intrigen geplant. Doch bei ALLAH sind ihre Intrigen bekannt, und obgleich ihre Intrigen so (heftig) waren, daß davon die Berge vergehen würden.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdccxcvii{Und denke niemals, daß ALLAH Sein Versprechen Seinen Gesandten gegenüber bricht. Gewiß, ALLAH ist allwürdig, vergeltungsübend.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdccxcviii{An dem Tag, wenn die Erde in eine andere Erde verwandelt wird sowie die Himmel. Dann werden sie treten vor ALLAH, Den Einzigen, Den Allbezwingenden.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdccxcix{Dann wirst du sehen die schwer Verfehlenden an diesem Tag in Fesseln gebündelt.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdccc{Ihre Kleidung wird aus Qatiran sein. Und das Feuer wird ihre Gesichter einhüllen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdccci{Zweifelsohne, ALLAH wird jeder Seele vergelten, was sie sich erworben hat. Gewiß, ALLAH ist schnell im Zur-Rechenschaft-Ziehen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdcccii{Dies ist eine Erklärung für dieMenschen, damit sie mit ihr gewarnt werden, damit sie wissen, daß ER nur ein einziger Gott ist, und damit die Verständigen sich entsinnen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdccciii{\basmalahde Alif-lam-raa. Dies sind die Ayat der Schrift und des erläuternden Quran.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mdccciv{Irgendwie wünschen sich diejenigen, die Kufr betrieben haben, daß sie Muslime wären.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mdcccv{Laß sie verspeisen und genießen und laß die Zuversicht sie beschäftigen, denn sie werden es noch wissen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mdcccvi{Und WIR haben keine Ortschaft zugrunde gerichtet, ohne daß es für sie eine bekannte Festlegung gab.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mdcccvii{Weder schneller als zu ihrer (bestimmten) Zeit vergeht eine Umma, noch überdauert sie (diese).\qt@no{5}}%
\qt@newcmd\qurantrans@de@mdcccviii{Und sie sagten: ``Du, derjenige, dem die Ermahnung (der Quran) nach und nach hinabgesandt wurde! Gewiß, du bist doch geistesgestört.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mdcccix{Würdest du doch uns die Engel bringen, solltest du einer der Wahrhaftigen sein!"\qt@no{7}}%
\qt@newcmd\qurantrans@de@mdcccx{WIR lassen die Engel nicht herabsteigen, es sei denn in Gesetzmäßigkeit. Und dann wird ihnen keine Zeit mehr gewährt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mdcccxi{Gewiß, WIR sind es Derjenige, Der die Ermahnung (den Quran) nach und nach hinabsandte, und gewiß, WIR sind ihm gegenüber zweifelsohne bewahrend.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mdcccxii{Und gewiß, bereits haben WIR vor dir (Gesandte) unter den früheren gleichgesinnten Gruppen entsandt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mdcccxiii{Und kein Gesandter kam zu ihnen, ohne daß sie ihn verspotteten.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mdcccxiv{Solcherart lassen WIR es in die Herzen der schwer Verfehlenden eindringen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mdcccxv{Sie verinnerlichen den Iman an ihn nicht. Und bereits verging das Beispiel der Früheren.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mdcccxvi{Und würden WIR über ihnen ein Tor vom Himmel öffnen, durch das sie im Tageslicht hinaufsteigen könnten,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mdcccxvii{würden sie gewiß sagen: ``Nur unsere Blicke wurden verwirrt. Nein, sondern wir sind verzauberte Leute!"\qt@no{15}}%
\qt@newcmd\qurantrans@de@mdcccxviii{Und gewiß, bereits haben WIR im Himmel Sternbilder eingesetzt und für die Betrachter schön gemacht.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mdcccxix{Auch haben WIR sie vor jedem verdammten Satan bewahrt,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mdcccxx{außer vor demjenigen, der lauschte, dem dann eine leuchtende erkennbare Feuerflamme folgte.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mdcccxxi{Und die Erde haben WIR eingeebnet und in ihr Berge eingesetzt. Und aus ihr haben WIR Allerlei nach Maß hervorsprießen lassen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mdcccxxii{Und auf ihr setzten WIR für euch Lebensmittel und alles andere, dem ihr kein Rizq gewährt.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mdcccxxiii{Und es gibt nichts, ohne daß WIR über dessen Magazine verfügen. Doch WIR lassen es nur nach einem bekannten Maß nach und nach hinabsenden.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mdcccxxiv{Und WIR schickten die Winde als Befruchtende. Dann ließen WIR vom Himmel Wasser fallen, das WIR euch zum Trinken gaben. Doch ihr seid keineswegs dessen Aufbewahrer.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mdcccxxv{Und gewiß, WIR sind doch Derjenige, Der beleben und sterben läßt. Und WIR sind die Bleibenden (nach dem Vergehen der Schöpfung).\qt@no{23}}%
\qt@newcmd\qurantrans@de@mdcccxxvi{Und gewiß, bereits wußten WIR Bescheid über die Vorangehenden von euch. Und gewiß, bereits wußten WIR Bescheid über die Nachfolgenden.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mdcccxxvii{Und gewiß, dein HERR, ER versammelt sie. Gewiß, ER ist allweise, allwissend.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mdcccxxviii{Und gewiß, bereits erschufen WIR den Menschen aus Salsal aus geformtem Hamaa.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mdcccxxix{Und die Dschinn erschufen WIR zuvor aus dem Samum-Feuer.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mdcccxxx{Und (erinnere daran), als dein HERR den Engeln sagte: ``ICH erschaffe einen Menschen aus Salsal aus geformtem Hamaa.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mdcccxxxi{Und nachdem ICH ihn vollendet und in ihn einen Ruhh von Mir eingehaucht habe, dann fallt nieder in Sudschud für ihn!"\qt@no{29}}%
\qt@newcmd\qurantrans@de@mdcccxxxii{So vollzogen alle Engel Sudschud, allesamt!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mdcccxxxiii{Nur Iblis lehnte ab, unter den Sudschud-Vollziehenden zu sein.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mdcccxxxiv{ER sagte: ``Iblis! Weshalb bist du nicht unter den Sudschud-Vollziehenden?!"\qt@no{32}}%
\qt@newcmd\qurantrans@de@mdcccxxxv{Er sagte: ``Ich würde nie vor einem Menschen Sudschud vollziehen, den DU aus Salsal aus geformtem Hamaa erschaffen hast!"\qt@no{33}}%
\qt@newcmd\qurantrans@de@mdcccxxxvi{ER sagte: ``Dann gehe aus ihr (der Dschanna) heraus, denn du bist gewiß verdammt.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mdcccxxxvii{Und gewiß, der Fluch lastet auf dir bis zum Tag des Din."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mdcccxxxviii{Er sagte: ``Mein HERR! Räume mir eine Frist ein bis zu dem Tag, an dem sie erweckt werden."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mdcccxxxix{ER sagte: ``Du gehörst zu denen, denen eine Frist eingeräumt wird,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mdcccxl{bis zu dem Tag der bekannten Zeit."\qt@no{38}}%
\qt@newcmd\qurantrans@de@mdcccxli{Er sagte: ``Mein HERR! Dafür, daß DU mich hast abirren lassen, werde ich es ihnen auf der Erde doch schön erscheinen lassen. Und ich werde sie alle zweifelsohne abirren lassen,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mdcccxlii{außer Deinen auserwählten Dienern unter ihnen."\qt@no{40}}%
\qt@newcmd\qurantrans@de@mdcccxliii{ER sagte: ``Das ist ein Mir verbindlicher, geradliniger Weg:\qt@no{41}}%
\qt@newcmd\qurantrans@de@mdcccxliv{Gewiß, über Meine Diener hast du keine Verfügung, außer über diejenigen von den Abirrenden, die dir folgen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mdcccxlv{Und sicherlich, Dschahannam ist doch ihre Verabredung, allesamt.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mdcccxlvi{Sie hat sieben Tore. Für jedes Tor von ihnen ist ein festgelegter Teil bestimmt.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mdcccxlvii{Gewiß, die Muttaqi sind in Dschannat und an Quellen.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mdcccxlviii{Tretet in sie in Salam ein als Sicherheit-Genießende.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mdcccxlix{Und WIR nahmen aus ihren Brüsten jeglichen Groll heraus. Sie sind als Geschwister auf Liegen einander gegenüber.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mdcccl{Keine Müdigkeit überkommt sie darin, und sie werden davon nicht herausgebracht.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mdcccli{TeileMeinen Dienern mit, daß ICH gewiß Der Allvergebende, Der Allgnädige bin,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mdccclii{und daß Meine Peinigung doch die qualvolle Peinigung ist.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mdcccliii{Und mache ihnen Mitteilung über die Gäste von Ibrahim.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mdcccliv{Als sie bei ihm eintraten, sagten sie: ``Salam (sei mit dir)!" Er sagte: ``Wir fürchten uns vor euch!"\qt@no{52}}%
\qt@newcmd\qurantrans@de@mdccclv{Sie sagten: ``Fürchte dich nicht! Wir bringen dir doch die frohe Botschaft über einen äußerst wissenden Sohn."\qt@no{53}}%
\qt@newcmd\qurantrans@de@mdccclvi{Er sagte: ``Bringt ihr mir etwa eine frohe Botschaft, wo ich vom Alter gezeichnet bin? Was für eine frohe Botschaft bringt ihr denn?!"\qt@no{54}}%
\qt@newcmd\qurantrans@de@mdccclvii{Sie sagten: ``Wir brachten dir eine frohe Botschaft wahrheitsgemäß, so sei nicht einer der Verzweifelnden!"\qt@no{55}}%
\qt@newcmd\qurantrans@de@mdccclviii{Er sagte: ``Und wer verzweifelt an der Gnade seines HERRN außer den Abirrenden?!"\qt@no{56}}%
\qt@newcmd\qurantrans@de@mdccclix{Er sagte: ``Was ist denn euer Bestreben, ihr Entsandte?"\qt@no{57}}%
\qt@newcmd\qurantrans@de@mdccclx{Sie sagten: ``Wir wurden doch zu schwer verfehlenden Leuten entsandt.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mdccclxi{Nur Luts Familie, die werden wir zweifelsohne erretten, allesamt,\qt@no{59}}%
\qt@newcmd\qurantrans@de@mdccclxii{außer seiner Ehefrau. Wir bestimmten: ``Gewiß, sie gehört doch zu den Vergehenden."\qt@no{60}}%
\qt@newcmd\qurantrans@de@mdccclxiii{Und als die Entsandten zur Familie Luts kamen,\qt@no{61}}%
\qt@newcmd\qurantrans@de@mdccclxiv{sagte er: ``Ihr seid ziemlich unbekannte Leute."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mdccclxv{Sie sagten: ``Nein, sondern wir brachten dir das, was sie anzuzweifeln pflegten.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mdccclxvi{Und wir kamen zu dir mit derWahrheit, und gewiß, wir sind doch wahrhaftig.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mdccclxvii{So brich mit deiner Familie am Ende der Nacht auf und bleibe hinter ihnen und keiner von euch wendet sich um. So geht dorthin, wohin es euch auferlegt wurde!"\qt@no{65}}%
\qt@newcmd\qurantrans@de@mdccclxviii{Und WIR haben ihm diese Bestimmung erteilt: ``Diese werden doch beim Morgengrauen bis auf den Letzten vernichtet."\qt@no{66}}%
\qt@newcmd\qurantrans@de@mdccclxix{Und die Stadtbewohner kamen voller Erwartung einer frohen Botschaft.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mdccclxx{Er sagte: ``Diese sind doch meine Gäste, so bringt mich nicht um die Ehre!\qt@no{68}}%
\qt@newcmd\qurantrans@de@mdccclxxi{Und handelt Taqwa gemäß ALLAH gegenüber und fügt mir keine Schmach zu!"\qt@no{69}}%
\qt@newcmd\qurantrans@de@mdccclxxii{Sie sagten: ``Haben wir dir etwa nicht verboten, Kontakte mit den Menschen zu pflegen?"\qt@no{70}}%
\qt@newcmd\qurantrans@de@mdccclxxiii{Er sagte: ``(Heiratet) diese meine Töchter, solltet ihr es tun wollen."\qt@no{71}}%
\qt@newcmd\qurantrans@de@mdccclxxiv{- Mein Schwur bei deinem Leben: sie sind durch ihr heftiges Verlangen verwirrt -\qt@no{72}}%
\qt@newcmd\qurantrans@de@mdccclxxv{So richtete sie ein Vernichtungsschrei beim Sonnenaufgang zugrunde.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mdccclxxvi{Dann kehrten WIR ihr Oberstes zuunterst und ließen über sie Steine von Sidsch-dschil hageln.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mdccclxxvii{Gewiß, darin sind zweifelsohne Ayat für die scharfsinnigen Beobachter.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mdccclxxviii{Und sie (die Ortschaft) liegt doch an einem noch bestehenden Weg.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mdccclxxix{Gewiß, darin ist zweifellos eine Aya für die Mumin.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mdccclxxx{Und die Bewohner von Al-aika waren zweifelsohne Unrecht-Begehende.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mdccclxxxi{So übten WIR an ihnen Vergeltung. Und beide (Ortschaften) lagen doch an einem bekannten Weg.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mdccclxxxii{Und gewiß, bereits haben die Bewohner von Al-hidschr den Gesandten der Lüge bezichtigt.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mdccclxxxiii{Und WIR ließen ihnen Unsere Ayat zuteil werden. So pflegten sie sich davon abzuwenden.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mdccclxxxiv{Und sie pflegten in die Felsenberge Wohnstätten zu meißeln, in voller Sicherheit.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mdccclxxxv{Dann richtete sie ein Vernichtungsschrei beim Sonnenaufgang zugrunde.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mdccclxxxvi{Und von ihnen hat nichts abwehren können das, was sie sich zu erwerben pflegten.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mdccclxxxvii{Und WIR erschufen die Himmel und die Erde und das, was zwischen ihnen ist, nur in Gesetzmäßigkeit. Und gewiß, die Stunde (der Jüngste Tag) wird anschlagen. So verzeihe in guter Verzeihung!\qt@no{85}}%
\qt@newcmd\qurantrans@de@mdccclxxxviii{Gewiß, dein HERR, ER ist Der Erschaffende, Der Allwissende.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mdccclxxxix{Und gewiß, bereits ließen WIR dir sieben von Al-mathani und den gigantischen Quran zuteil werden.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mdcccxc{Richte deine Augen nicht auf das, was WIR manchen von ihnen an vergänglichen Verbrauchsgütern zur Verfügung stellten. Und trauere nicht um sie, und behandle die Mumin milde!\qt@no{88}}%
\qt@newcmd\qurantrans@de@mdcccxci{Und sag: ``Gewiß, ich bin der erläuternde Warner!"\qt@no{89}}%
\qt@newcmd\qurantrans@de@mdcccxcii{WIR offenbarten dir) wie WIR (den Gesandten) derjenigen Einteilenden hinabsandten,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mdcccxciii{die den Quran in Teile einteilten.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mdcccxciv{Bei deinem HERRN! WIR werden sie doch zur Rechenschaft ziehen, allesamt,\qt@no{92}}%
\qt@newcmd\qurantrans@de@mdcccxcv{über das, was sie zu tun pflegten.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mdcccxcvi{So verkünde, was dir geboten wurde, und meide die Muschrik.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mdcccxcvii{Gewiß, WIR haben an deiner Stelle die Verspottenden abgewehrt,\qt@no{95}}%
\qt@newcmd\qurantrans@de@mdcccxcviii{diejenigen, die neben ALLAH eine andere Gottheit einsetzen. Diese werden es noch wissen.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mdcccxcix{Und gewiß, bereits wissen WIR, daß du von dem bedrückt bist, was sie sagen.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcm{So lobpreise mit dem Lob deines HERRN und sei einer der Sudschud- Vollziehenden,\qt@no{98}}%
\qt@newcmd\qurantrans@de@mcmi{und diene deinem HERRN, bis der Tod dich ereilt.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mcmii{\basmalahde ALLAHs Bestimmung trifft noch ein, so verlangt nicht eilig danach. Gepriesenerhaben ist ER über das, was sie an Schirk betreiben.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mcmiii{ER sendet die Engel nach und nach hinab mit dem Ruhh nach seiner Bestimmung zu demjenigen von Seinen Dienern, den ER will: ``Warnt, daß es zweifelsohne keine Gottheit gibt außer Mir, so handelt Taqwa gemäß Mir gegenüber!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mcmiv{ER erschuf die Himmel und die Erde in Gesetzmäßigkeit. Erhaben ist er über das, was sie an Schirk betreiben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mcmv{ER erschuf den Menschen aus Nutfa, so war er ein erklärter Widersacher.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mcmvi{Auch die An'am erschuf Er. Durch sie gewinnt ihr Wärme und (andere) Produkte. Und ihr ernährt euch von ihnen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mcmvii{Und durch sie genießt ihr Schönheit, wenn ihr sie abends eintreibt und wenn ihr sie morgens austreibt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mcmviii{Auch tragen sie eure Lasten zu einer Ortschaft, die ihr sonst nur mit großer Anstrengung erreichen würdet. Gewiß, euer HERR ist zweifelsohne allgütig, allgnädig.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mcmix{Auch die Pferde, die Maultiere und die Esel (erschuf ER), damit ihr sie reitet und als Schmuck. Und ER erschafft, was ihr nicht kennt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mcmx{Und ALLAH obliegt die Rechtleitung zum geraden Weg. Und manche der Wege sind abweichend. Und hätte ALLAH es gewollt, hätte ER euch doch rechtgeleitet, allesamt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mcmxi{ER ist Derjenige, Der vom Himmel Wasser fallen ließ. Von ihm habt ihr Trank und von ihm wachsen Pflanzen, auf denen ihr weiden laßt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mcmxii{Damit läßt ER für euch Gewächs, Oliven, Dattelpalmen, Trauben und von jeglichen Früchten hervorsprießen. Gewiß, darin ist zweifelsohne eine Aya für Leute, die sich besinnen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mcmxiii{Und ER machte euch die Nacht, das Tageslicht, die Sonne und den Mond gratis fügbar. Ebenfalls sind die Sterne Seiner Bestimmung nach gratis fügbar. Gewiß, darin sind zweifelsohne Ayat für Leute, die verständig sind.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mcmxiv{Und was ER für euch auf Erden an mannigfarbiger Vielfalt erschuf, gewiß, darin ist zweifelsohne eine Aya für Leute, die sich entsinnen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mcmxv{Und ER ist Derjenige, Der euch das Meer gratis fügbar machte, damit ihr von ihm saftiges Fleisch speist und aus ihm Schmuck herausfischt, den ihr anzieht - und du siehst, daß die Schiffe es durchpflügen - und damit ihr nach Seiner Gunst strebt, und damit ihr euch dankbar erweist.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mcmxvi{Und ER setzte auf Erden Felsenberge, damit sie mit euch nicht schwankt, sowie Flüsse und Wege, damit ihr Rechtleitung findet.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mcmxvii{Ebenso (setzte ER) Merkmale. Und mit dem Stern finden sie Rechtleitung.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mcmxviii{Ist Derjenige, Der erschafft, etwa demjenigen gleichzusetzen, der nichts erschafft?! Wollt ihr euch etwa nicht besinnen?!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mcmxix{Und wenn ihr ALLAHs Gaben aufzählen würdet, würdet ihr sie nie umfassend erfassen. Gewiß, ALLAH ist doch allvergebend, allgnädig.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mcmxx{Und ALLAH kennt das, was ihr verheimlicht und was ihr offenlegt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mcmxxi{Und diejenigen, an die sie anstelle von ALLAH Bittgebete richten, erschaffen nichts, sondern sie werden selbst erschaffen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mcmxxii{Sie sind Leblose, keine Lebendigen. Und sie merken nicht, wann sie erweckt werden.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mcmxxiii{Euere Gottheit ist eine einzige Gottheit. Und diejenigen, die keinen Iman an das Jenseits verinnerlichen - ihre Herzen leugnen es und sie sind sich in Arroganz Erhebende.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mcmxxiv{Zweifellos eindeutig ist es, daß ALLAH das kennt, was sie verheimlichen und was sie offenlegen. Gewiß, ER liebt nicht die sich in Arroganz Erhebenden.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mcmxxv{Und wenn ihnen gesagt wird: ``Was sandte euer HERR hinab?" Sagten sie: ``Die Legenden der Vorfahren."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mcmxxvi{Sie werden gewiß ihre Verfehlungen, vollständig am Tag der Auferstehung auf sich nehmen sowie von den Verfehlungen derjenigen, die sie verleitet haben ohne Wissen. Ja! Erbärmlich ist es, was sie an Verfehlungen auf sich nehmen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mcmxxvii{Bereits haben diejenigen vor ihnen Intrigen geplant, dann ließ ALLAH das von ihnen Aufgebaute von den Fundamenten herausreißen, so fiel das Dach über ihnen auf sie herunter, und die Peinigung überkam sie, von wo sie dies nicht merken konnten.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mcmxxviii{Dann wird ER sie am Tag der Auferstehung erniedrigen und sagen: ``Wo sind Meine angeblichen Partner, derentwegen ihr feindselig eingestellt wart?!" Diejenigen, denen Wissen zuteil wurde, sagten: ``Gewiß, die Erniedrigung und das Schlimme an diesem Tag sind auf den Kafir,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mcmxxix{denjenigen, welche die Engel einziehen, während sie gegen sich selbst noch Unrecht begingen." So fügten sie sich demütig: ``Wir pflegten doch nichts Schlechtes zu tun." Doch, mit Sicherheit! Gewiß, ALLAH ist allwissend über das, was ihr zu tun pflegtet.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mcmxxx{Also tretet ein in die Tore von Dschahannam, darin ewig bleibend. Und gewiß erbärmlich ist der Aufenthaltsort der Arroganten.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mcmxxxi{Und denjenigen, die Taqwa gemäß handelten, wurde gesagt: ``Was sandte euer HERR hinab?" Sie sagten: ``Nur Gutes!" Denjenigen, die in diesem Diesseits Ihsan gemäß handelten, ist Gutes bestimmt. Doch die Wohnstätte des Jenseits ist noch besser. Und zweifelsohne hervorragend ist die Wohnstätte der Muttaqi.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mcmxxxii{'Adn-Dschannat werden sie betreten, die von Flüssen durchflossen werden. In ihnen bekommen sie, was sie sich wünschen. Solcherart vergilt ALLAH den Muttaqi,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mcmxxxiii{denjenigen, deren Leben die Engel beenden, während sie rein sind. Sie (die Engel) sagen: ``Salam sei mit euch, tretet in Dschanna ein wegen dem, was ihr zu tun pflegtet."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mcmxxxiv{Warten sie etwa auf etwas anderes außer darauf, daß zu ihnen die Engel kommen oder daß die Bestimmung deines HERRN eintrifft?! Solcherart handelten diejenigen vor ihnen. Und ALLAH hat ihnen kein Unrecht angetan, sondern sie pflegten sich selbst Unrecht anzutun.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mcmxxxv{So trafen sie die Bosheiten dessen, was sie taten, und es umgab sie das, was sie zu verspotten pflegten.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mcmxxxvi{Und diejenigen, die Kufr betrieben haben, sagten: ``Hätte ALLAH anderes gewollt, hätten wir nichts anstelle von Ihm gedient, weder wir noch unsere Väter. Ebenfalls hätten wir nichts anstelle von Ihm für haram erklärt." Solcherart handelten diejenigen vor ihnen. Und obliegt den Gesandten etwas anderes außer dem deutlichen Verkünden?!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mcmxxxvii{Und gewiß, bereits entsandten WIR zu jeder Umma einen Gesandten: ``Dient ALLAH und bleibt dem Taghut fern!" Und unter denen waren einige, die ALLAH rechtgeleitet hat, und einige, für die das Irregehen recht war. Also zieht auf Erden umher, dann seht, wie das Anschließende von den Leugnern war.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mcmxxxviii{Selbst wenn du dich sorgfältig um ihre Rechtleitung bemühst, so leitet ALLAH doch keinen recht, den ER abirren läßt. Und für sie gibt es keine Beistehenden.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mcmxxxix{Und sie legten bei ALLAH nachdrückliche Eide ab: ``ALLAH erweckt nicht denjenigen, der stirbt." Doch mit Sicherheit! Dies ist ein von Ihm gemachtes Versprechen, ein Wahrhaftiges. Doch die meisten Menschen wissen es nicht.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mcmxl{Damit ER ihnen das erklärt, worüber sie uneins sind, und damit diejenigen wissen, die den Kufr betrieben haben, daß sie doch Lügner waren.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mcmxli{Unser Wort einer Sache gegenüber, wenn WIR sie wollen, ist ausschließlich, daß WIR ihr sagen: ``Sei!" Dann ist sie.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mcmxlii{Und denjenigen, die wegen ALLAH Hidschra unternahmen, nachdem ihnen Unrecht angetan worden ist, werden WIR gewiß im Diesseits einen schönen Aufenthaltsort gewähren. Doch die Belohnung des Jenseits ist noch größer, würden sie es nur wissen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mcmxliii{Dies sind diejenigen, die sich in Geduld übten und Tawakkul ihrem HERRN gegenüber praktizierten.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mcmxliv{Und vor dir entsandten WIR als Gesandte nur Männer, denen WIR Wahy zuteil werden ließen. So fragt die Leute von Adh-dhikr, wenn ihr selbst nicht wisst.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mcmxlv{WIR entsandten sie) mit klaren Zeichen und mit den Schriften, und WIR sandten dir die Ermahnung (den Quran) hinab, damit du den Menschen das erläuterst, was ihnen nach und nach hinabgesandt wurde, und damit sie sich besinnen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mcmxlvi{Fühlen sich diejenigen, welche die gottmißfälligen Taten intrigieren, etwa sicher davor, daß ALLAH mit ihnen die Erde nicht versenkt, oder daß die Peinigung sie ereilt, von wo sie nicht merken?!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mcmxlvii{Oder daß ER sie zugrunde richtet, während sie (ihren Beschäftigungen) nachgehen?! Denn sie können (Ihm) nicht zu schaffen machen.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mcmxlviii{Oder daß ER sie nach und nach zugrunde richtet?! So ist euer HERR bestimmt allgütig, allgnädig!\qt@no{47}}%
\qt@newcmd\qurantrans@de@mcmxlix{Haben sie etwa nicht das gesehen, was ALLAH an Sachen erschuf, wie ihre Schatten von rechts nach links in Sudschud für ALLAH abwechseln, während sie sich fügen?!\qt@no{48}}%
\qt@newcmd\qurantrans@de@mcml{Und für ALLAH vollzieht Sudschud alles, was in den Himmeln ist und was auf Erden ist, von den sich darauf bewegenden Lebewesen sowie die Engel, während sie sich darüber nicht erheben.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mcmli{Sie (die Engel) fürchten ihren HERRN über ihnen und tun, was ihnen geboten wird.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mcmlii{Und ALLAH sagte: ``Nehmt euch nicht zwei Gottheiten! ER ist gewiß nur ein einziger Gott, so habt vor Mir alleine Ehrfurcht!"\qt@no{51}}%
\qt@newcmd\qurantrans@de@mcmliii{Und Ihm gehört, was in den Himmeln und auf Erden ist. Und Ihm gehört der Din für immer. Handelt ihr etwa Taqwa gemäß (anderen) außer ALLAH gegenüber?!\qt@no{52}}%
\qt@newcmd\qurantrans@de@mcmliv{Und was ihr an Wohlergehen genießt, ist von ALLAH. Und dann, wenn Unglück euch trifft, zu Ihm alleine fleht ihr.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mcmlv{Dann, wenn ER das Unglück von euch beseitigt, gibt es eine Gruppe von euch, die ihrem HERRN gegenüber Schirk betreibt,\qt@no{54}}%
\qt@newcmd\qurantrans@de@mcmlvi{damit sie dem gegenüber, was WIR ihnen gaben, Kufr betreibt. Also vergnügt euch nur! Ihr werdet es noch wissen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mcmlvii{Und sie bestimmen für diejenigen (Götter), die nichts wissen, einen Anteil vom Rizq, das WIR ihnen gewährten. Bei ALLAH! Gewiß, Ihr werdet zur Rechenschaft gezogen für das, was ihr zu erdichten pflegtet.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mcmlviii{Und sie setzen für ALLAH Töchter ein  gepriesen-erhaben ist ER! - und für sich (nur) das, was sie sich wünschen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mcmlix{Und wenn jemandem von ihnen die frohe Nachricht über (die Geburt) einer Weiblichen überbracht wurde, blieb sein Gesicht verfinstert, während er voller Groll ist.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mcmlx{Er zieht sich vor den Leuten zurück wegen der Schlechtigkeit dessen, was ihm an froher Nachricht überbracht wurde: ``Soll er sie (das weibliche Neugeborene) trotz Demütigung behalten oder soll er sie in der Erde vergraben?" Ja! Erbärmlich ist das, was sie urteilen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mcmlxi{Denjenigen, die den Iman an das Jenseits nicht verinnerlichen, gebührt ein Gleichnis des Bösen. Doch ALLAH gebührt das allhöchste Gleichnis. Und ER ist Der Allwürdige, Der Allweise.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mcmlxii{Und würde ALLAH den Menschen das von ihnen begangene Unrecht vergelten, hätte ER kein einziges sich bewegendes Lebewesen auf ihr (der Erde) übriggelassen. Aber ER gewährt euch Aufschub bis zu einer festgesetzten Frist. Und wenn dann ihre Frist eintrifft, können sie (diese) weder um eine Stunde verschieben, noch vorverlegen.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mcmlxiii{Und sie schreiben ALLAH das zu, was sie verabscheuen, und ihre Zungen beschreiben die Lüge, daß ihnen unweigerlich das Beste gebührt. Zweifellos eindeutig ist es, daß für sie das Feuer bestimmt ist, und daß sie ohnehin hineingetrieben werden.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mcmlxiv{Bei ALLAH! Gewiß, bereits entsandten WIR (Gesandte) zu Umam vor dir, dann hat diesen der Satan ihre (schlechten) Taten schön erscheinen lassen, so ist er ihr Wali heute. Und für sie ist eine qualvolle Peinigung bestimmt.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mcmlxv{Und WIR sandten dir die Schrift nur hinab, damit du ihnen das erläuterst, worüber sie uneins sind, als Rechtleitung und als Gnade für Leute, die den Iman verinnerlichen.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mcmlxvi{Und ALLAH ließ vom Himmel Wasser fallen, dann belebte ER mit ihm die Landschaft nach ihrem Tod. Gewiß, dies ist zweifelsohne eine Aya für Leute, die zuhören.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mcmlxvii{Und gewiß, für euch liegt in den An'am bestimmt eine Lehre: WIR geben euch von dem in ihren Bäuchen zu trinken - aus Futterbrei und Blut - reine, leicht zu schluckende Milch für die Trinkenden.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mcmlxviii{Auch aus den Früchten der Dattelbäume und der Rebstöcke stellt ihr für euch Wein her, aber auch schönes Rizq. Gewiß, darin ist zweifelsohne eine Aya für Leute, die begreifen.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mcmlxix{Und dein HERR ließ der Biene Wahy zuteil werden: ``Unterhalte Behausungen in manchen der Berge, auf manchen der Bäume und in manchen von dem, was sie errichten.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mcmlxx{Dann ernähre dich von allen Früchten und verkehre auf den Wegen deines HERRN in Ergebenheit. Aus ihren Bäuchen kommt ein Trank, der unterschiedlich farbig ist, in ihm ist Heilung für die Menschen. Gewiß, darin ist zweifelsohne eine Aya für Leute, die nachdenken.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mcmlxxi{Und ALLAH hat euch erschaffen, dann läßt ER euch sterben. Und manch einer von euch wird in das faselnde Alter zurückgesetzt, damit er nichts mehr weiß, nachdem er über Wissen verfügte. Gewiß, ALLAH ist allwissend, allmächtig.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mcmlxxii{Und ALLAH begünstigte die einen von euch vor den anderen im Rizq. Und diejenigen, die begünstigt wurden, werden ihr Rizq denen, die ihnen gehören, gewiß nicht so zukommen lassen, daß alle dann darin (im Rizq) gleich werden! Wollen sie etwa ALLAHs Gaben leugnen?!\qt@no{71}}%
\qt@newcmd\qurantrans@de@mcmlxxiii{Und ALLAH machte euch aus euren eigenen Wesen Partnerwesen. Und ER machte euch aus euren Partnerwesen Kinder und Enkel und gewährte euch Rizq von den Tay-yibat. Bekunden sie etwa den Iman an das für nichtig Erklärte und betreiben ALLAHs Gaben gegenüber Kufr?!\qt@no{72}}%
\qt@newcmd\qurantrans@de@mcmlxxiv{Und sie dienen anstelle von ALLAH dem, das für sie über keinerlei Rizq in den Himmeln und auf Erden verfügt, und sie vermögen nichts.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mcmlxxv{Also prägt ALLAH keine Gleichnisse! Gewiß, ALLAH weiß und ihr wisst nicht.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mcmlxxvi{ALLAH prägte ein Gleichnis eines leibeigenen Sklaven, der nichts vermag, und desjenigen, dem WIR von Uns schönes Rizq gewährten, von dem er heimlich und offenkundig gibt, sind beide etwa gleich?! Alhamdulillah: Alles Lob gebührt ALLAH! Nein, sondern die meisten von ihnen wissen nicht.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mcmlxxvii{Und ALLAH prägt ein Gleichnis zweier Männer. Der eine von ihnen ist stumm, vermag nichts und ist eine Last für seinen Maula, egal wohin er ihn schickt, bringt er nichts Gutes. Ist er etwa dem gleich, der Gerechtigkeit gebietet und auf geradlinigem Weg ist?!\qt@no{76}}%
\qt@newcmd\qurantrans@de@mcmlxxviii{Und ALLAH verfügt über das Verborgene der Himmel und der Erde. Und die Angelegenheit der Stunde (des Jüngsten Tages) ähnelt nichts Anderem als einem Augenblick. Nein, sondern sie ist noch kürzer. Gewiß, ALLAH ist über alles allmächtig.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mcmlxxix{Und ALLAH brachte euch von den Bäuchen eurer Mütter hervor, während ihr nichts wisst. Und ER ließ euch über das Hören, das Sehen und den Verstand verfügen, damit ihr euch dankbar erweist.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mcmlxxx{Sehen sie etwa nicht die Vögel, im Luftraum des Himmels fügsam Gemachte, die nichts außer ALLAH hält?! Gewiß, darin sind zweifelsohne Ayat für Leute, die den Iman verinnerlichen.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mcmlxxxi{Und ALLAH machte für euch aus euren Wohnstätten Ruhestätten und machte für euch aus den An'am-Häuten Wohnstätten (Zelte), die ihr beim Tragen leicht findet, wenn ihr reist und wenn ihr euch niederlasst. Auch aus ihrer Wolle, ihren Fellen und ihren Haaren (machte ER für euch) Ausstattung und Utensilien bis zu einer bestimmten Zeit.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mcmlxxxii{Und ALLAH machte für euch Schatten von dem, was ER erschuf, und machte für euch aus den Felsenbergen ausgemeißelte Wohnungen. Und ER machte für euch Bekleidungen, die euch vor der Hitze schützen, und Bekleidungen, die euch vor eurer Gewalttätigkeit (gegeneinander) schützen. Solcherart vervollständigt ALLAH euch Seine Gabe, damit ihr den Islam praktiziert.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mcmlxxxiii{Sollten sie sich dann abwenden, dann obliegt dir ausschließlich das deutliche Verkünden.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mcmlxxxiv{Sie kennen ALLAHs Gabe, dann leugnen sie diese ab. Und die meisten von ihnen sind die Kafir.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mcmlxxxv{Und (erinnere an) den Tag, wenn WIR aus jeder Umma einen Zeugen erwecken, dann wird denjenigen, die Kufr betrieben haben, weder erlaubt (sich zu entschuldigen), noch werden sie zufrieden gestellt.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mcmlxxxvi{Und wenn diejenigen, die Unrecht begingen, die Peinigung gesehen haben, wird diese ihnen weder erleichtert, noch wird ihnen Zeitaufschub gewährt.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mcmlxxxvii{Und wenn diejenigen, die Schirk betrieben, die von ihnen beigesellten Partner gesehen haben, werden sie sagen: ``Unser HERR! Diese sind die von uns beigesellten Partner, an die wir an Stelle von Dir Bittgebete zu richten pflegten!" Dann werden diese ihnen die Rede erwidern: ``Gewiß, ihr seid zweifelsohne Lügner."\qt@no{86}}%
\qt@newcmd\qurantrans@de@mcmlxxxviii{Und sie werden an diesem Tag ihre Hingabe ALLAH gegenüber bekunden. Und ihnen ist abhanden gegangen, was sie zu erdichten pflegten.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mcmlxxxix{Denjenigen, die Kufr betrieben und von ALLAHs Weg abhielten, ihnen werden WIR noch zusätzliche Peinigung über die (eigentliche) Peinigung zufügen für das, was sie an Verderben anzurichten pflegten.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mcmxc{Und (erinnere an) den Tag, wenn WIR aus jeder Umma einen Zeugen gegen sie erwecken - von ihnen selber, und dich als Zeugen gegen diese bringen. Und WIR sandten dir die Schrift nach und nach hinab als Erläuterung für alle Dinge, als Rechtleitung, als Gnade und als frohe Botschaft für die Muslime.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mcmxci{Gewiß, ALLAH gebietet Gerechtigkeit, Ihsan und den Verwandten gegenüber Freigebigkeit, und verbietet Verwerfliches, Mißbilligtes und unrechtmäßige Aggression. ER ermahnt euch, damit ihr euch besinnt.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mcmxcii{Und erfüllt das ALLAH gegenüber gemachte Versprechen, wenn ihr etwas versprecht, und brecht die Eide nicht, nachdem ihr sie bestätigt habt und ihr ALLAH als Bürgen für euch bereits genommen habt. Gewiß, ALLAH weiß, was ihr tut.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mcmxciii{Und seid nicht wie diejenige, die ihr gesponnenes Garn zu Fasern auflöste, nachdem es fest gesponnen war, indem ihr eure Eide untereinander, wie Fiktionen betrachtet aus Sorge, daß eine Umma stärker als eine andere Umma sein könnte. ALLAH prüft euch damit nur. Und ER wird euch am Tag der Auferstehung eindeutig erläutern, worin ihr uneins wart.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mcmxciv{Und hätte ALLAH gewollt, gewiß hätte ER euch eine einzige Umma gemacht. Aber ER läßt abirren, wen ER will, und leitet recht, wen ER will. Und ihr werdet zweifellos zur Rechenschaft gezogen für das, was ihr getan habt.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mcmxcv{Und betrachtet eure Eide nicht wie Fiktionen untereinander, sonst entgleitet ein Fuß, nachdem er fest stand, und sonst erfahrt ihr das Unheil dafür, daß ihr vom Wege ALLAHs abgehalten habt, und für euch ist dann eine überharte Peinigung bestimmt.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mcmxcvi{Und erkauft euch nicht mit dem ALLAH gegenüber gemachten Versprechen etwas Minderwertiges. Was bei ALLAH ist, ist besser für euch, wenn ihr nur wüßtet.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mcmxcvii{Worüber ihr verfügt, geht zu Ende, und was ALLAH hat, dies bleibt! Und WIR werden denjenigen, die sich in Geduld übten, gewiß ihre Belohnung vergelten mit Besserem als das, was sie zu tun pflegten.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mcmxcviii{Und wer gottgefällig Gutes tut, sei es ein Männliches oder ein Weibliches, während es Mumin ist, diesem werden WIR gewiß schönes Leben gewähren. UndWIR werden ihnen ihre Belohnung gewiß vergelten mit Besserem als das, was sie zu tun pflegten.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mcmxcix{Wenn du den Quran rezitierst, dann sag: ``A'udhu billahi minasch-schaitanirradschim."\qt@no{98}}%
\qt@newcmd\qurantrans@de@mm{Gewiß, er hat keine Verfügung über diejenigen, die den Iman verinnerlichten und ihrem HERRN gegenüber Tawakkul üben.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmi{Seine Verfügung hat er ausschließlich über diejenigen, die ihn als Wali nehmen, und diejenigen, die durch ihn Schirk betreiben.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmii{Und wenn WIReine Aya anstelle einer anderen Aya austauschen - und ALLAH weiß besser, was ER hinabsendet - sagen sie: ``Du bist doch nur ein Erdichter!" Nein, sondern die meisten von ihnen wissen es nicht.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmiii{Sag: ``Nach und nach hinabgesandt hat ihn Ruhhul-qudus von deinem HERRN gemäß der Wahrheit, damit ER diejenigen festigt, die den Iman verinnerlichten, und als Rechtleitung und als frohe Botschaft für die Muslime."\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmiv{Und gewiß, bereits wissen WIR, daß sie sagen: ``Nur ein Mensch lehrt ihn." Die Sprache desjenigen, zu dem sie zugeneigt sind, ist eine Nichtarabische, doch dies ist eine klare arabische Sprache.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmv{Gewiß, diejenigen, die den Iman an die Ayat ALLAHs nicht verinnerlichen, leitet ALLAH nicht recht. Und für sie ist eine qualvolle Peinigung bestimmt.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmvi{Lügen erfinden nur diejenigen, die keinen Iman an die Ayat ALLAHs verinnerlichen. Und diese sind die eigentlichen Lügner.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmvii{Lügen erfindet ebenfalls), wer ALLAH gegenüber Kufr betrieb, nachdem er den Iman bekundete - außer dem, der (dazu) gezwungen wird, während sein Herz voller Iman ist. Jedoch wer die Brust dem Kufr öffnet, auf diesem lastet Zorn von ALLAH. Und für sie ist eine überharte Peinigung bestimmt.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmviii{Dies ist so, weil sie doch das diesseitige Leben vor dem Jenseits bevorzugt haben, und weil ALLAH die Kafir-Leute nicht rechtleitet.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmix{Diese sind diejenigen, deren Herzen, Gehör und Blicke versiegelt sind. Und diese sind die Achtlosen.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmx{Zweifellos, eindeutig ist es, daß sie im Jenseits diejenigen sind, welche die Verlierer sind.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmxi{Ohnedies, dein HERR ist gewiß für diejenigen, die Hidschra unternahmen, nachdem sie der Fitna ausgesetzt wurden, dann leisteten sie Dschihad und übten sich in Geduld. Gewiß, dein HERR ist danach zweifelsohne allvergebend, allgnädig.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmxii{An dem Tag, wenn jede Seele kommt, um sich selbst zu rechtfertigen. Dann wird jeder Seele vergolten, was sie getan hat. Und ihnen wird kein Unrecht widerfahren.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmxiii{Und ALLAH prägte ein Gleichnis (der Bewohner) einer Ortschaft. Sie war(en) sicher und unbesorgt. Ihr Rizq kam zu ihnen reichlich aus jedem Ort, dann betrieb(en) sie Kufr ALLAHs Gaben gegenüber, dann ließ ALLAH sie die Bedeckung von Hunger und Angst fühlen für das, was sie zu bewerkstelligen pflegten.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmxiv{Und gewiß, bereits kam zu ihnen ein Gesandter von ihnen, dann bezichtigten sie ihn des Lügens, dann richtete sie die Peinigung zugrunde, während sie Unrecht-Begehende waren.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmxv{Also esst vom Rizq, das ALLAH euch als gutes Halal gewährte. Und erweist euch den Gaben ALLAHs gegenüber dankbar, solltet ihr Ihm (wirklich) dienen.\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmxvi{Für haram wurde euch nur erklärt: das Verendete, das Blut, das Schweinefleisch und das, welches für Anderes als ALLAH geschächtet wurde. Wer jedoch in Not geriet, ohne daß er übertritt oder übertreibt, dann ist ALLAH gewiß allvergebend, allgnädig.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmxvii{Und sagt nicht zu dem, was eure Zungen an Lügen beschreiben: ``Dies ist halal und dies ist haram", damit ihr im Namen ALLAHs Lügen erfindet. Gewiß, diejenigen, die im Namen ALLAHs Lügen erfinden, werden keinen Erfolg haben.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmxviii{Es ist ein winziges Genußgut und für sie ist eine qualvolle Peinigung bestimmt.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmxix{Und den Juden erklärten WIR für haram all das, was WIR dir vorher berichteten. Und WIR fügten ihnen kein Unrecht zu, sondern sie pflegten, gegen sich selbst Unrecht zu begehen.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmxx{Ohnedies ist dein HERR denjenigen gegenüber, die das Schlechte aus Unwissen tun, dann danach es bereuen und gottgefällig Gutes tun, dein HERR ist (ihnen) danach gewiß allvergebend, allgnädig!\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmxxi{Gewiß, Ibrahim war eine Umma. Er war ALLAH gegenüber bewußt ergeben und ein der Wahrheit Verbundener und er war nicht von den Muschrik.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmxxii{Er war dankbar Seinen Gaben gegenüber. ER hat ihn dann auserwählt und auf einen geradlinigen Weg rechtgeleitet.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmxxiii{Und WIR ließen ihm im Diesseits Gutes zuteil werden und im Jenseits gehört er zweifelsohne zu den gottgefällig Guttuenden.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmxxiv{Dann ließen WIR dir Wahy zuteil werden, daß du der Gemeinschaft von Ibrahim, der hanif war, folgst. Und er war nicht von den Muschrik.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmxxv{Der Sabbat wurde nur denen auferlegt, die darüber uneins waren. Und gewiß, dein HERR wird doch am Tag der Auferstehung zwischen ihnen richten über das, worin sie uneins zu sein pflegten.\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmxxvi{Mache Da"wa zum Weg deines HERRN mit Weisheit und schöner Ermahnung, und disputiere mit ihnen auf die beste Art. Gewiß, dein HERR ist es, der Wissen darüber hat, wer von Seinem Weg abgekommen ist. Und ER ist allwissend über die Rechtgeleiteten.\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmxxvii{Und wenn ihr straft, dann straft mit dem Gleichen dessen, womit ihr bestraft wurdet. Und wenn ihr euch in Geduld übt, dies ist gewiß besser für die sich in Geduld Übenden.\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmxxviii{Und übe dich in Geduld. Und deine Geduld verleiht dir nur ALLAH. Und sei nicht traurig über sie. Und sei nicht bedrückt wegen dem, was sie an List planen.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmxxix{ALLAH ist gewiß mit denjenigen, die Taqwa gemäß handelten, und mit denjenigen, die Muhsin sind.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmxxx{\basmalahde Gepriesen-erhaben ist Derjenige, Der Seinen Diener nachts reisen ließ - in einem Teil der Nacht von Almasdschidil-haram zu der fernen Moschee, um die herum WIR mit Baraka versehen haben, um ihm von Unseren Ayat zu zeigen. Gewiß, ER ist Der Allhörende, Der Allsehende.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmxxxi{Und WIR ließen Musa die Schrift zuteil werden und machten sie zur Rechtleitung für die Kinder von Israil: ``Nehmt euch an Meiner Stelle keinen Wakil!"\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmxxxii{Ihr Nachkommen derer, dieWIR mit Nuh mittragen ließen! Gewiß, Er war ein äußerst dankbarer Diener.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmxxxiii{Und WIR gaben den Kindern von Israil in der Schrift ein: ``Gewiß werdet ihr zweimal im Lande Verderben anrichten, und gewiß werdet ihr euch in großer Überhebung überheben."\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmxxxiv{Und wenn die Zeit (für die Bestrafung) des Ersten beider Male kommt, werden WIR zu euch Unsere Diener schicken, die äußerst harte Kämpfer sind, so werden sie die Wohnstätten durchstöbern. Und dies ist ein zu erfüllendes Versprechen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmxxxv{Dann werden WIR euch wieder die Oberhand über sie geben und euch mit Vermögenswerten und Kindern Nachschub leisten und eure Bereitschaft zum Kampf ihre übertreffen lassen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmxxxvi{Wenn ihr Gutes tut, tut ihr ausschließlich euch selbst Gutes. Und wenn ihr schlecht handelt, so ist dies nur gegen diese selbst. Und wenn die Zeit (für die Bestrafung) des Letzten (beider Male) kommt, (schicken WIR sie), damit sie euch das Schlechte ins Gesicht schreiben lassen, und damit sie in die Moschee einmarschieren, wie sie in sie das ersteMal einmarschiert waren, und damit sie alles, worüber sie verfügen, vernichtend zugrunde richten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmxxxvii{Vielleicht wird euch euer HERR (danach) Gnade erweisen. Wenn ihr jedoch es wiederholt, werden WIR es auch wiederholen. Und WIR machten Dschahannam für die Kafir zum Gefängnis.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmxxxviii{Gewiß, dieser Quran leitet zu dem recht, was geradlinig ist, und überbringt frohe Botschaft zu den Mumin, die gottgefällig Gutes tun, daß für sie eine große Belohnung bestimmt ist,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmxxxix{und daß WIR denjenigen, die keinen Iman an das Jenseits verinnerlichen, eine qualvolle Peinigung vorbereitet haben.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmxl{Und der Mensch bittet (ALLAH) um das Böse, wie er um das Gute bittet. Und der Mensch ist voreilig.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmxli{Und WIR machten die Nacht und den Tag zu zwei Ayat, dann ließen WIR die Aya der Nacht verdunkeln und die Aya des Tages Sicht gewähren, damit ihr Gunst von eurem HERRN erstrebt, und damit ihr das Zählen der Jahre und das Rechnen kennt. Und jede Sache haben WIR in Ausführlichkeit genau dargelegt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmxlii{Und jedem Menschen ließen WIR sein (gutes und schlechtes) Wirken in seinem Nacken sitzen, und am Tag der Auferstehung bringen WIR ihm ein Register hervor, das er offengelegt vorfindet:\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmxliii{``Lies dein Register! Du selbst genügst heute, um über dich selbst ein Berechnender zu sein."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmxliv{Wer Rechtleitung findet, der findet für sich selbst Rechtleitung, und wer irregeht, der geht ausschließlich gegen sich selbst irre. Und keine verfehlende Seele trägt die Verfehlung einer anderen. Und WIR werden niemals peinigen, bis WIR einen Gesandten entsandt haben.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmxlv{Und wenn WIR eine Ortschaft zugrunde richten wollen, machen WIR ihren im Luxus Schwelgenden Gebote, dann betreiben sie Fisq dagegen, dann wird die Bestimmung gegen sie vollstreckt, dann lassen WIR sie vernichtend zugrunde gehen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmxlvi{Und wie viele Generationen richteten WIR nach Nuh zugrunde! Und dein HERR genügt über die Verfehlungen Seiner Diener als Ein Allkundiger, Allsehender.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmxlvii{Wer das schnell Vergängliche (Diesseits) erstrebt, dem werden WIR darin das prompt geben, was WIR wollen, wem WIR wollen. Danach haben WIR für ihn Dschahannam vorbereitet. Er wird es als ein Entehrter, Verdammter betreten.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmxlviii{Und wer das Jenseits will und sich dafür anstrengt mit der ihm (dem Jenseits) gebührenden Anstrengung und er dabei Mumin ist, diesen wird ihre Anstrengung reichlich belohnt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmxlix{Beide versorgen WIR, diese und diese mit den Gaben deines HERRN. Und die Gaben deines HERRN waren (niemandem) vorenthalten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mml{Siehe, wie WIR die einen von ihnen vor den anderen begünstigt haben. Doch das Jenseits hat noch höhere Ränge und größere Begünstigungen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmli{Geselle ALLAH keinen anderen Gott bei, sonst wirst du entehrt, erniedrigt.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmlii{Und dein HERR bestimmte, daß ihr niemandem außer Ihm dient, und daß ihr den Eltern Ihsan erweist. Sollten beide bei dir das hohe Alter erreichen - einer von ihnen oder beide, so sag ihnen niemals: ``Pfui!" Und schelte sie nicht und sag ihnen schöne Worte.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmliii{Und sei ihnen gegenüber demütig aus Barmherzigkeit und sage: ``Mein HERR! Erweise ihnen Gnade, so wie sie mich von klein an aufgezogen haben."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmliv{Euer HERR ist allwissend über das, was in euren Herzen ist. Wenn ihr gottgefällig Guttuende seid, dann ist ER gewiß immer für die stets Umkehrenden allvergebend.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmlv{Und gib dem Verwandten, was ihm zusteht, sowie dem Armen und dem in Not geratenen Reisenden. Und sei nicht verschwenderisch, vergeudend!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmlvi{Gewiß, die Verschwender sind die Gleichartigen der Satane. Und der Satan ist seinem HERRN gegenüber ein Extrem-Kafir.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmlvii{Und solltest du ihnen (den Armen) ausweichen, weil du in Erwartung einer Gnade (Rizq) von deinem HERRN bist, dann sage ihnen erleichternde Worte.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmlviii{Und halte deine Hand nicht (aus Geiz verschlossen, als wäre sie) an deinem Hals gebunden, und strecke sie nicht (verschwenderisch) total aus, sonst wirst du zu einem, dem Vorwürfe gemacht werden, und zu einem Selbstankläger.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmlix{Gewiß, dein HERR gewährt viel Rizq, wem ER will, und ER läßt es auch wenig sein. Gewiß, ER ist über Seine Diener allkundig, allsehend.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmlx{Und tötet eure Kinder nicht aus Furcht vor Verarmung! WIR gewähren ihnen Rizq und euch. Gewiß, ihre Tötung ist eine schwere Verfehlung.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmlxi{Und nähert euch nicht der Zina! Gewiß, sie ist eine schändliche Tat und erbärmlich ist dieser Weg.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmlxii{Und tötet nicht den Menschen, dessen (Tötung) ALLAH für haram erklärte, es sei denn nach dem Recht. Und wer zu Unrecht getötet wird, dessen Wali gaben WIR Verfügungsrecht, so darf er bei der Tötung (als Vergeltung) das Maß nicht überschreiten. Gewiß, er erhält Beistand.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmlxiii{Und nähert euch nicht dem Vermögen des Waisenkindes, es sei denn auf die bessere Art, (solange) bis es mündig wird. Und erfüllt eure Abmachungen! Gewiß, die Abmachung wird verantwortet.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmlxiv{Und gebt volles Maß, wenn ihr abmesst, und wägt mit der richtigen Waage! Dies ist gut und hat ein besseres Ende.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmlxv{Und folge nicht dem, worüber du über kein Wissen verfügst! Gewiß, das Gehör, das Sehen und der Verstand, all diese werden von ihm (vom Menschen) zweifellos verantwortet.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmlxvi{Und gehe auf Erden nicht freudenerregt! Gewiß, dabei wirst du weder die Erde zerlöchern können, noch wirst du die Berghöhen an Höhe erreichen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmlxvii{All dies, das Schlechte davon ist bei deinem HERRN verpönt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmlxviii{Dies ist von dem, was dir dein HERR von der Weisheit als Wahy zuteil werden ließ. Und geselle ALLAH keinen anderen Gott bei, sonst wirst du in Dschahannam hineingeworfen als einer, dem Vorwürfe gemacht werden, und als ein Verdammter.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmlxix{Begünstigte euch euer HERR etwa mit Söhnen und nahm sich von den Engeln Weibliche?! Gewiß, ihr macht bestimmt eine ungeheuerliche Aussage.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmlxx{Und gewiß, bereits erläuterten WIR vieles in diesem Quran, damit sie sich besinnen. Doch er bestärkt sie nur in der Abneigung.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmlxxi{Sag: ``Wären mit Ihm Götter, wie sie sagen, dann würden sie einen Weg zum Herrscher über Al'ahrsch anstreben."\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmlxxii{Gepriesen-erhaben und erhaben ist ER in unermeßlicher Erhabenheit über das, was sie sagen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmlxxiii{Die sieben Himmel, die Erde und wer in ihnen ist, preisen Ihn. Und es gibt nichts, das nicht mit Seinem Lob preist. Doch ihr begreift ihre Preisung nicht. Gewiß, ER ist immer allnachsichtig, allvergebend.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmlxxiv{Und jedesmal wenn du den Quran rezitiert hast, errichteten WIR zwischen dir und denjenigen, die keinen Iman an das Jenseits verinnerlichen, einen unsichtbaren Vorhang.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmlxxv{Und WIR umhüllten ihre Herzen mit Bedeckung, damit sie ihn nicht begreifen, und machten ihre Ohren schwerhörig. Und jedesmal wenn du deinen HERRN im Quran als Einen Einzigen erwähnt hast, kehrten sie ihre Rücken flüchtend in Abgeneigtheit.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmlxxvi{WIR wissen besser, weshalb sie zuhören, wenn sie dir zuhören, und wenn sie heimlich sich unterhalten, wenn die Unrecht-Begehenden sagen: ``Ihr folgt doch nur einem verzauberten Mann!"\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmlxxvii{Siehe, wie sie dir die Gleichnisse prägten, so sind sie irregegangen und fanden nicht zum Weg.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmlxxviii{Und sie sagten: ``Nachdem wir zu Knochen und Staub geworden sind, werden wir etwa sicher als neue Schöpfung erweckt?"\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmlxxix{Sag: ``Seid Steine oder Eisen,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmlxxx{oder eine Schöpfung, die eurem Verstand nach noch härter ist!" Dann werden sie sagen: ``Wer erweckt uns wieder?" Sag: ``Derjenige, Der euch das erste Mal erschuf." Sie werden dann dir gegenüber ihre Köpfe schütteln und sagen: ``Wann ist es?" Sag: ``Möglicherweise wird es bald sein."\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmlxxxi{An dem Tag, an dem ER euch ruft, dann werdet ihr mit Seinem Lob antworten. Und ihr werdet denken, daß ihr nur eine kurze Weile verweilt habt.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmlxxxii{Und sag Meinen Dienern, daß sie das bessere (Wort) sagen. Gewiß, der Satan stiftet Zwietracht zwischen ihnen. Gewiß, der Satan ist für den Menschen ein erklärter Feind.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmlxxxiii{Euer HERR weiß besser Bescheid über euch. Wenn ER will, erweist ER euch Gnade, und wenn ER will, läßt ER euch peinigen. Und WIR haben dich nicht als Wakil über sie entsandt.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmlxxxiv{Und dein HERR weiß besser Bescheid über diejenigen, die in den Himmeln und auf Erden sind. Und gewiß, bereits begünstigten WIR einige der Propheten vor anderen und ließen Dawud einen Zabur zuteil werden.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmlxxxv{Sag: ``Richtet nur Bittgebete an diejenigen, die ihr (als Götter) anstelle von Ihm ausgewiesen habt! Sie können den Schaden von euch weder wegnehmen, noch ableiten."\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmlxxxvi{Diese, an die sie Bittgebete richten, erstreben selbst die Annäherung zu ihrem HERRN, jeder will näher sein. Und sie erhoffen sich Seine Gnade und fürchten Seine Peinigung. Gewiß, der Peinigung deines HERRN gegenüber ist achtzugeben.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmlxxxvii{Und es gibt keine Ortschaft, ohne daß WIR sie vor dem Tag der Auferstehung zugrunde richten werden oder daß WIR sie einer harten Peinigung unterziehen werden. Dies ist in der Schrift längst niedergeschrieben.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmlxxxviii{Und es hielt Uns nichts zurück, Gesandte mit den Ayat zu entsenden, außer daß die Früheren sie als Lüge erklärten. Und WIR ließen Thamud das Kamelweibchen als Erläuterung zuteil werden, so begingen sie mit ihm Unrecht. Und WIR entsenden mit den Ayat nur Ermahnung.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmlxxxix{Und (erinnere daran), als WIR dir sagten: ``Gewiß, dein HERR wird die Menschen zugrunde richten!" Und WIR machten das Traumgesicht, dasWIR dich sehen ließen, nur als Fitna für die Menschen, so auch den verfluchten Baum im Quran. Und WIR ermahnen sie, doch dies bestärkt sie nur zu einer noch größeren Übertretung.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmxc{Und (erinnere daran), als WIR den Engeln sagten: ``Vollzieht Sudschud für Adam!" Und sie vollzogen Sudschud, außer Iblis, er sagte: ``Soll ich etwa Sudschud vor dem vollziehen, den DU aus Lehm erschaffen hast?"\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmxci{Er sagte: ``Wie siehst DU es mit diesem, den DU mehr als mich würdigst? Wenn DU mir bis zum Jüngsten Tag Aufschub gewährst, dann werde ich unweigerlich seine Nachkommenschaft zugrunde richten - außer Wenigen."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmxcii{ER sagte: ``Geh! Wer dir von ihnen folgt, dann ist Dschahannam gewiß eure Vergeltung - eine ausreichende Vergeltung!\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmxciii{Und wiegele mit deinem Aufruf auf, wen du von ihnen (aufwiegeln) kannst, mache sie scheu mit deinen Reitenden und deinen zu Fuß Gehenden, sei ihr Teilhaber an den Vermögenswerten und den Kindern, und mache ihnen Versprechungen. Und der Satan verspricht ihnen gewiß nur Fiktives.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmxciv{Gewiß, über Meine Diener hast du allerdings keinerlei Verfügung. Und dein HERR genügt als Wakil."\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmxcv{Euer HERR ist Derjenige, Der für euch die Schiffe auf dem Meer fahren läßt, damit ihr nach Seiner Gunst strebt. Gewiß, ER ist euch gegenüber allgnädig.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmxcvi{Und wenn euch der Schaden auf dem Meer trifft, gehen die verloren, an die ihr Bittgebete gerichtet habt, außer Ihm. Und nachdem ER euch an das Land errettet hat, wendet ihr euch ab. Der Mensch ist ein äußerst Kufr-Betreibender.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmxcvii{Fühlt ihr euch etwa sicher davor, daß ER einen Teil des Festlandes mit euch versenkt, oder daß ER über euch einen Sandsturm schickt? Dann findet ihr für euch sicher keinen Wakil.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmxcviii{Oder fühlt ihr euch sicher davor, daß ER euch dorthin (ins Meer) noch einmal zurückkehren läßt, und dann über euch einen Orkan schickt, dann euch ertrinken läßt für das, was ihr an Kufr betrieben habt? Dann findet ihr darin gegen Uns keinen Bestreitenden.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmxcix{Und gewiß, bereits verliehen WIR den Kindern Adams Würde, ließen sie auf dem Lande und auf dem Meer tragen, gewährten ihnen Rizq von den Tay-yibat und begünstigten sie gegenüber vielen von denjenigen, die WIR erschufen, in klarer Begünstigung.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmc{Erinnere an) den Tag, wenn WIR jede Menschengruppe mit ihrem Imam rufen. Wem sein Register in seine Rechte überreicht wird, diese lesen ihr Register und ihnen wird keinerlei Unrecht zugefügt.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmci{Und wer in diesem (Leben geistig) blind war, der bleibt im Jenseits blind und noch mehr vom Weg abirrend.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcii{Und beinahe hätten sie dich durch Fitna von dem abgebracht, was WIR dir als Wahy zuteil werden ließen, damit du in Unserem Namen anderes erdichtest. Und (hättest du es getan), dann hätten sie dich sicherlich als engen Freund genommen.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmciii{Und hätten WIR dich nicht gefestigt, gewiß, bereits wärest du beinahe ihnen ein klein wenig zugeneigt.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmciv{Dann hätten WIR dich doch das Zweifache (an Peinigung) im Leben und das Zweifache nach dem Tod erfahren lassen. Dann wirst du für dich gegen Uns keinen Beistehenden finden.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcv{Und beinahe hätten sie dich doch im Lande bedrängt, damit sie dich daraus vertreiben können. Und dann würden sie nach dir nur kurz (heil) zurückbleiben,\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcvi{nach dem Vorbild derjenigen, die WIR bereits vor dir von Unseren Gesandten entsandten. Und du findest kein Abweichen von dem von Uns statuierten Vorbild.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcvii{Verrichte ordnungsgemäß das rituelle Gebet, nachdem die Sonne den Zenit überschritten hat, bis die Nacht ganz dunkel wird, und (rezitiere) den Quran in der Fadschr-Zeit. Gewiß, der Quran in der Fadschr-Zeit wird erlebt.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmcviii{Und in einem Teil der Nacht verrichte damit (mit dem Quran) das Tahadsch-dschud-Gebet - ein freiwilliges Gebet für dich, damit dein HERR dir eine lobenswerte Stellung verleiht.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmcix{Und sag: ``HERR! Lasse mich in einen Eingang der Wahrhaftigkeit eintreten, und lasse mich aus einem Ausgang der Wahrhaftigkeit austreten, und gewähre mir von Dir aus eine beistehende Bestätigung."\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmcx{Und sag: ``Das Wahre ist gekommen und das für nichtig Erklärte ist vergangen." Gewiß, das für nichtig Erklärte ist immer vergänglich.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmcxi{Und WIR senden nach und nach vom Quran hinab, was eine Heilung und Gnade für die Mumin ist. Doch er bestärkt die Unrecht-Begehenden nur noch im Fehlgehen.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmcxii{Und wenn WIR dem Menschen gute Gaben gewähren, wendet er sich ab und weicht zur Seite. Und wenn ihn das Unheil trifft, ist er äußerst verzweifelt.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmcxiii{Sag: ``Jeder handelt nach seiner Art. Denn euer HERR weiß besser Bescheid über jeden, der den besseren Weg geht."\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmcxiv{Und sie fragen dich nach der Seele, sag: ``Die Seele ist eine Angelegenheit meines HERRN, und euch wurde vom Wissen nur ein wenig zuteil."\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmcxv{Und hätten WIR es gewollt, hätten WIR doch das vergehen lassen, was WIR dir als Wahy zuteil werden ließen, dann wirst du für dich niemanden finden, der dafür (für den Quran) alsWakil gegen Uns auftritt,\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmcxvi{es sei denn eine Gnade von deinem HERRN. Gewiß, Seine Gunst ist dir gegenüber unermeßlich.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmcxvii{Sag: ``Würden sich die Menschen und die Dschinn vereinigen, damit sie ein diesem Quran Gleiches hervorbringen, würden sie nicht Seinesgleichen hervorbringen, selbst dann nicht, sollten die einen den anderen beistehen."\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmcxviii{Und gewiß, bereits erläuterten WIR in diesem Quran von jedem Gleichnis. Doch die meisten Menschen lehnten alles außer dem äußersten Kufr ab.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmcxix{Und sie sagten: ``Wir werden dir keinen Iman schenken, bis du für uns von der Erde eine Quelle entspringen läßt,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmcxx{oder bis du über Dschanna von Dattelpalmen und Rebstöcken verfügst, in der die Flüsse in Strömen sprudeln,\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmcxxi{oder du läßt über uns den Himmel, wie du es behauptet hast, in Stücke fallen, oder du bringst ALLAH und die Engel uns gegenüber,\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmcxxii{oder du verfügst über ein Haus aus Gold, oder du fliegst in den Himmel, und wir werden keinen Iman an dein Fliegen verinnerlichen, bis du uns eine Schrift zuteil werden läßt, die wir lesen können." Sag: ``Gepriesen-erhaben ist mein HERR! Bin ich etwa etwas anderes als ein Mensch, ein Gesandter?!"\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmcxxiii{Und nichts hinderte die Menschen daran, den Iman zu verinnerlichen, als zu ihnen die Rechtleitung kam, außer daß sie sagten: ``Schickte ALLAH etwa einen Menschen als Gesandten?"\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmcxxiv{Sag: ``Wären auf der Erde Engel, die darauf sicher gehen würden, hätten WIR ihnen vom Himmel gewiß einen Engel als Gesandten herabgesandt."\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmcxxv{Sag: ``ALLAH genügt als Zeuge zwischen mir und euch! Gewiß, ER ist über Seine Diener allkundig, allsehend."\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmcxxvi{Und wen ALLAH rechtleitet, ist zweifellos der Rechtgeleitete, und wen ER abirren läßt, für die findest du gewiß keine Wali anstelle von Ihm! Und WIR werden sie am Tag der Auferstehung auf ihren Gesichtern versammeln - blind, stumm und taub. Ihre Unterkunft ist Dschahannam. Immer wieder wenn es verglimmt, fügen WIR ihnen mehr Flammen hinzu.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmcxxvii{Dies ist die ihnen bestimmte Vergeltung, weil sie gewiß Kufr Unseren Ayat gegenüber betrieben und sagten: ``Nachdem wir zu Knochen und Staub geworden sind, werden wir etwa sicher als neue Schöpfung erweckt?"\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmcxxviii{Haben sie etwa nicht gesehen, daß ALLAH, Der die Himmel und die Erde erschuf, im Stande ist, ihresgleichen zu schaffen?! Und ER setzte für sie eine Frist, über die es keinen Zweifel gibt. Doch die Unrecht-Begehenden lehnten alles außer dem äußersten Kufr ab.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmcxxix{Sag: ``Würdet ihr über die Magazine der Gnade meines HERRN verfügen, dann würdet ihr gewiß aus Furcht vor dem Aufgebraucht-Sein knausern." Der Mensch ist doch äußerst geizig.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmcxxx{Und gewiß, bereits ließen WIR Musa neun eindeutige Ayat zuteil werden. So frage die Kinder Israils, als er zu ihnen kam und Pharao dann ihm sagte: ``Gewiß, ich denke doch über dich, du Musa! - daß du verzaubert bist."\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmcxxxi{Er sagte: ``Gewiß, bereits weißt du doch, daß diese (Ayat) kein anderer hinabsandte außer Dem HERRN der Himmel und der Erde als etwas Einblick-Gewährendes. Und gewiß, ich denke doch über dich, du Pharao! - daß du verleitet bist."\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmcxxxii{Dann wollte er sie vom Lande durch Bedrängen vertreiben, dann ertränkten WIR ihn und alle, die mit ihm waren, allesamt.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmcxxxiii{UndWIR sagten nach ihm den Kindern Israils: ``Bewohnt das Land, und wenn das Versprechen des Letzteren kommt, werden WIR euch in vermischten Scharen bringen."\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmcxxxiv{Und gemäß der Wahrheit sandten WIR ihn hinab und gemäß der Wahrheit kam er herab. Und WIR schickten dich als Gesandten nur als Überbringer froher Botschaft und als Ermahnenden.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmcxxxv{Und ein Quran, den WIR in Teilen einteilten, damit du ihn den Menschen in Gemächlichkeit rezitierst. Und WIR sandten ihn doch nach und nach hinab.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmcxxxvi{Sag: ``Bekundet den Iman an ihn oder bekundet keinen Iman, (für Uns ist dies unerheblich)!" Gewiß, diejenigen, denen das Wissen vor ihm zuteil wurde, wenn er ihnen rezitiert wird, fallen auf ihr Angesicht in Sudschud nieder,\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmcxxxvii{und sagen: ``Gepriesen-erhaben ist unser HERR. Das Versprechen unseres HERRN wird gewiß erfüllt."\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmcxxxviii{Und sie fallen auf ihr Angesicht nieder, sie weinen, und er läßt sie noch mehr Ehrfurcht empfinden.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmcxxxix{Sag: ``Benennt (Ihn) ALLAH oder benennt (Ihn) den Allgnade Erweisenden! Mit welchem auch immer du (Ihn) benennst, Ihm gehören doch die Schönsten Namen!" Und sei bei deinem Gebet nicht zu laut, und sei damit nicht zu leise, und strebe dazwischen einen (mittleren) Weg an!\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmcxl{Und sag: ``Alhamdulillah: Alles Lob gebührt ALLAH, Der sich keinen Sohn nahm, Der nie einen Teilhaber in der Herrschaft hat, und Der nie einen Wali (als Schutz) vor Erniedrigung hat." Und verherrliche Ihn mit allahu-akbar!\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmcxli{\basmalahde Alhamdulillah: Alles Lob gebührt ALLAH, Der die Schrift Seinem Diener hinabsandte und in ihr nichts Gewundenes machte.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcxlii{Sie ist) geradlinig gemäßigt, damit sie vor einer harten Peinigung von Ihm warnt, und den Mumin, die gottgefällig Gutes tun, eine frohe Botschaft überbringt, daß für sie eine schöne Belohnung bestimmt ist,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcxliii{in der sie für immer bleiben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmcxliv{Und (damit) sie diejenigen warnt, die sagen: ``ALLAH hat sich einen Sohn genommen."\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcxlv{Weder sie verfügen über Wissen darüber, noch ihre Vorfahren. Es ist ein ungeheuerliches Wort, das aus ihren Mündern kommt. Sie sagen gewiß nur Lügenhaftes.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcxlvi{Und nicht etwa zugrunde richtest du dich selbst in Verdrießlichkeit über das von ihnen Übriggebliebene, sollten sie keinen Iman an diese Rede (den Quran) verinnerlichen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcxlvii{Gewiß, WIR machten das, was auf der Erde ist, als deren Schmuck, damit WIR sie prüfen, wer von ihnen der besser Handelnde ist.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcxlviii{Auch zweifelsohne werden WIR alles, was auf ihr ist, doch in staubigen, dürren Boden verwandeln.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcxlix{Oder denkst du etwa, daß diejenigen von der Höhle und von Ar-raqim zum Wundersamsten Unserer Ayat gehörten?!\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcl{Und erinnere daran), als die jungen Männer in der Höhle Unterkunft suchten und sagten: ``Unser HERR! laß uns von Dir aus Gnade zuteil werden und richte für uns aus unserer Angelegenheit eine richtige Haltung ab!"\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcli{Dann richteten WIR auf ihre Ohren (etwas Abtrennendes) in der Höhle ein, für abgezählte Jahre.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmclii{Dann erweckten WIR sie, damit WIR kenntlich machen, welche der beiden Gruppierungen die von ihnen verweilte Zeit genauer errechnete.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcliii{WIR berichten dir über ihre Begebenheit wahrheitsgemäß. Gewiß, sie waren jungeMänner, die den Iman an ihren HERRN verinnerlichten, undWIR ließen sie noch mehr Rechtleitung gewinnen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcliv{UndWIR gewährten ihren Herzen Mut, als sie sich erhoben und sagten: ``Unser HERR ist Der HERR der Himmel und der Erde. Wir werden anstelle von Ihm niemals an eine (andere Gottheit) Bittgebete richten, gewiß, bereits sagten wir etwas eindeutig Unwahres."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmclv{Diese sind unsere Leute, sie nahmen sich anstelle von Ihm Götter. Würden sie doch dafür eine klare Bestätigung vorlegen! Und keiner begeht noch mehr Unrecht als derjenige, der im Namen ALLAHs Lügen erfindet!\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmclvi{Und wenn ihr euch von ihnen und von dem, was sie anstelle von ALLAH dienen, absetzen wollt, dann sucht Unterschlupf in der Höhle, denn euer HERR wird für euch von Seiner Gnade angedeihen lassen und für euch aus eurer Angelegenheit etwas Nützliches abrichten."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmclvii{Und (würdest du sie sehen, so würdest) du die Sonne sehen: wenn sie aufging, umging sie ihre Höhle von der rechten Seite, und wenn sie unterging, überging sie diese von der linken Seite, während sie inmitten dieser (Höhle) waren. Dies ist von ALLAHs Ayat. Wen ALLAH rechtleitet, er ist der Rechtgeleitete, und wen ER abirren läßt, für den findest du keinen führenden Wali.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmclviii{Und du denkst: sie seien wach, während sie geschlafen haben. Und WIR wendeten sie auf die rechte und auf die linke Seite, und ihr Hund lag mit ausgestreckten Vorderpfoten an der Schwelle. Hättest du sie erblickt, hättest du dich gewiß vor ihnen flüchtend davon gemacht und wärest du ihretwegen unweigerlich erfüllt mit Schrecken.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmclix{Und solcherart erweckten WIR sie, damit sie sich gegenseitig fragen. Einer unter ihnen, der etwas zu sagen hatte, sagte: ``Wie lange habt ihr verweilt?" Sie sagten: ``Wir verweilten einen Tag oder einen Teil eines Tages." Sie sagten: ``Euer HERR weiß besser Bescheid darüber, wie lange ihr verweilt habt. Nun aber schickt jemanden von euch mit diesem eurem Silber in die Stadt. Er soll dann nachprüfen, welche der Speisen reiner ist, dann soll er euch davon Rizq mitbringen. Und er soll unauffällig sein und niemanden auf euch aufmerksam machen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmclx{Sollten sie über euch die Oberhand gewinnen, werden sie euch steinigen oder euch in ihre Gemeinschaft umkehren lassen. Und dann werdet ihr nimmer erfolgreich sein."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmclxi{Und solcherart ließen WIR sie entdecken, damit sie wissen, daß das Versprechen ALLAHs wahr ist, und daß es über die Stunde (den Jüngsten Tag) keinerlei Zweifel gibt, als sie untereinander in ihrer Angelegenheit stritten. Dann sagten sie: ``Errichtet über ihnen ein Gebäude, denn ihr HERR weiß besser Bescheid über sie." Diejenigen, die in ihrer Angelegenheit obsiegten, sagten: ``Wir errichten doch über ihnen eineMoschee."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmclxii{Sie werden sagen: ``Sie waren drei und ihr Vierter war ihr Hund." Und sie sagen: ``Sie waren fünf und ihr Sechster war ihr Hund." Dies ist nichts als Spekulation über das Verborgene. Und sie sagen: ``Sie waren sieben und ihr Achter war ihr Hund." Sag: ``Mein HERR weiß besser Bescheid über ihre Anzahl, und niemand kennt sie außer wenigen." Also disputiere nicht über sie, es sei denn ein Disputieren ohne Details, und befrage über sie niemanden von ihnen!\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmclxiii{Und sage zu keiner Sache: ``Ich vollbringe dies morgen."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmclxiv{Es sei denn, wenn ALLAH es will. Und erwähne (den Willen) deines HERRN (danach), solltest du es (vorher) vergessen haben. Und sag: ``Ich hoffe, daß mein HERR mich zu einer besseren Führung als dieses rechtleitet.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmclxv{Und sie verweilten in ihrer Höhle dreihundert Jahre und ihnen wurden neun noch hinzugefügt.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmclxvi{Sag: ``ALLAH weiß besser Bescheid darüber, wie lange sie verweilten." Ihm gehört das Verborgene der Himmel und der Erde. Wie allsehend ist ER und allhörend! Sie haben anstelle von Ihm keinen Wali, und ER beteiligt an Seiner Entscheidung niemanden.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmclxvii{Und rezitiere das, was dir als Wahy von der Schrift deines HERRN zuteil wurde. Es gibt keinen, der Seine Worte abändert. Und du findest anstelle von Ihm keinen Schutz-Gewährenden.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmclxviii{Und übe dich selbst in Geduld mit denjenigen, die an ihren HERRN morgens und abends Bittgebete richten und nach Seinem Wohlgefallen streben. Und übersieh sie nicht, während du das diesseitige Leben anstrebst. Und gehorche nicht demjenigen, dessen Herz WIR Unserem Gedenken gegenüber achtlos machten, der seinen Neigungen folgte und dessen Angelegenheit eine maßlose Übertretung war.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmclxix{Und sag: ``Die Wahrheit ist von eurem HERRN. Wer will, der soll den Iman verinnerlichen, und wer will, der soll Kufr betreiben." WIR bereiteten für die Unrecht-Begehenden ein Feuer, dessen Rauch sie umzingelt. Und wenn sie um Hilfe rufen, wird Ihrem Hilferuf entsprochen mit Wasser wie das Geschmolzene, das die Gesichter schmort. Erbärmlich ist der Trank und verhängnisvoll ist der Ort zum Anlehnen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmclxx{Gewiß, diejenigen, die den Iman verinnerlichten und gottgefällig Gutes taten, gewiß, WIR lassen die Belohnung dessen, der Gutes tat, auf keinen Fall verloren gehen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmclxxi{Für diese sind Dschannat von 'Adn bestimmt, die von Flüssen durchflossen sind. Darin werden sie mit Armbändern aus Gold geschmückt und sie werden grüne Kleidung aus Sundos und Istabraq tragen - angelehnt darin auf Liegen. Die beste Belohnung (ist dies) und schön gemacht ist der Ort zum Anlehnen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmclxxii{Und präge ihnen ein Gleichnis von zwei Männern. Einem von ihnen gaben WIR zwei Dschannat mit Rebstöcken, die WIR beide mit Dattelpalmen umgaben, und zwischen beiden ließen WIR Gewächs sein.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmclxxiii{Beide Dschannat brachten ihre Früchte hervor und verwuchsen nichts davon. Und WIR ließen darin einen Fluß entspringen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmclxxiv{Und er verfügte über Früchteertrag, dann sagte er seinem Gefährten, während er mit ihm einen Dialog führte: ``Ich verfüge über mehr Vermögen als du und bin dazu noch überlegener an Gefolge."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmclxxv{Und er trat in seine Dschanna ein, während er gegen sich selbst Unrecht beging, er sagte: ``Ich glaube nicht, daß diese jemals vergänglich sein wird.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmclxxvi{Und ich glaube nicht, daß die Stunde (der Jüngste Tag) zustande kommt. Und selbst wenn ich zu meinem HERRN zurückgebracht werde, werde ich auf jeden Fall eine bessere Rückkehr als diese vorfinden."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmclxxvii{Sein Gefährte sagte ihm, während er mit ihm den Dialog führte: ``Betreibst du etwa Kufr gegenüber Dem, Der dich aus Erde erschuf, dann aus einer Nutfa, dann machte (ER) dich zu einem Mann?!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmclxxviii{Doch ER ist ALLAH, mein HERR. Und ich geselle meinem HERRN niemanden bei.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmclxxix{Und würdest du sagen, wenn du in deine Dschanna eintrittst: ``Mascha-allah, la quwata illa bil-lah." (Und auch) wenn du mich siehst, daß ich über weniger Vermögen verfüge als du und weniger Kinder habe,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmclxxx{denn vielleicht läßt mir mein HERR etwas Besseres als deine Dschanna zuteil werden, und schickt über sie einen Blitzsturm aus dem Himmel, dann wird sie zu rutschigem Boden.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmclxxxi{Oder ihr Wasser versiegt, so daß du es nicht mehr wirst erreichen können."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmclxxxii{Und seine Früchte wurden komplett vernichtet. Dann begann er (aus Reue und Selbstanklage) abwechselnd die Handflächen aufeinander zu schlagen für das, was er darin investiert hatte - während ihre Gerüste (für die Reben) dem Erdboden gleichgemacht wurden, und er sagte: ``Hätte ich doch nur nicht meinem HERRN jemanden beigesellt!"\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmclxxxiii{Und für ihn gab es keine Gruppe, die ihm anstelle von ALLAH beisteht, und er war keineswegs ein Siegender.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmclxxxiv{Da ist die wahre Herrschaft für ALLAH. ER gewährt eine bessere Belohnung und ein besseres Ende.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmclxxxv{Und präge ihnen das Gleichnis des diesseitigen Lebens, wie Wasser, das WIR vom Himmel fallen ließen, dann gedieh mit ihm das Gewächs auf der Erde, dann wurde dies zu Vertrocknetem, das der Wind verweht. Und ALLAH ist über alles allvermögend.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmclxxxvi{Das Vermögen und die Kinder sind Schmuck des diesseitigen Lebens. Und die bleibenden gottgefällig guten Taten verdienen bei deinem HERRN mehr an Belohnung und mehr an Hoffnung.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmclxxxvii{Und (erinnere) an den Tag, wenn WIR die Felsenberge vergehen lassen, und du siehst die Erde hervorgehoben, und WIR werden sie versammeln und keinen von ihnen entkommen lassen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmclxxxviii{Und sie werden in Reihen deinem HERRN vorgestellt: ``Gewiß, bereits seid ihr zu Uns gekommen, wie WIR euch das erste Mal erschufen. Nein, sondern ihr habt behauptet, daß WIR für euch niemals einen Termin bestimmen werden."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmclxxxix{Und das Register wurde aufgelegt. Dann siehst du die schwer Verfehlenden voller Furcht vor dem, was in ihm ist, und sie sagen: ``Unser Untergang! Was ist das für ein Register, das weder kleine noch schwere Verfehlung wegläßt, ohne sie umfassend zu erfassen?!" Und sie fanden präsent, was sie taten, und dein HERR fügt keinem Unrecht zu.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcxc{Und (erinnere daran), als WIR den Engeln sagten: ``Vollzieht Sudschud für Adam!" Dann vollzogen sie Sudschud. Ausgenommen Iblis, er war von den Dschinn, so betrieb er Fisq dem Gebot seines HERRN gegenüber. Wollt ihr ihn und seine Nachkommenschaft als Wali an Meiner Stelle nehmen, während sie euch Feinde sind?! Erbärmlich ist für die Unrecht-Begehenden der Eintausch!\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcxci{ICH ließ sie weder dem Erschaffen der Himmel und der Erde, noch dem Erschaffen ihrer Selbst beiwohnen. Und ICH nehme die Fehlleitenden nie zur Verstärkung.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcxcii{Und erinnere an) den Tag, wenn ER sagt: ``Ruft die Mir beigesellten Partner, welche ihr erfunden habt!" Und sie haben sie gerufen, doch sie entsprachen ihnen nicht, und WIR bereiteten für sie einen Maubiq.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcxciii{Und die schwer Verfehlenden sahen das Feuer und wußten, daß sie zweifelsohne hineingeworfen werden, und sie fanden davon kein Entrinnen mehr.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcxciv{Und gewiß, bereits erläuterten WIR in diesem Quran für die Menschen von jedem Gleichnis, doch der Mensch ist immer der am meisten Disputierende.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcxcv{Und nichts hinderte die Menschen daran, den Iman zu verinnerlichen - nachdem zu ihnen die Rechtleitung kam - und ihren HERRN um Vergebung bitten, außer dem Warten darauf, daß ihnen (Peinigung nach) dem Vorbild ihrer Vorfahren zuteil wird oder daß die Peinigung sie in direkter Konfrontation überkommt.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcxcvi{Und WIR entsenden die Gesandten nur als Überbringer froher Botschaft und als Warner. Und diejenigen, die Kufr betrieben haben, argumentieren mit dem für nichtig Erklärten, damit sie mit ihm das Wahre annullieren. Und sie nahmen meine Ayat und das, wovor sie gewarnt wurden, zum Spott.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcxcvii{Und keiner begeht mehr Unrecht als derjenige, der an die Ayat seines HERRN erinnert wird, und dann sich von ihnen abwendet und vergißt, was er eigenhändig erworben hat. Gewiß, WIR umhüllten ihre Herzen doch mit Bedeckung, damit sie ihn (den Quran) nicht begreifen, und machten ihre Ohren schwerhörig. Selbst dann, wenn du sie zur Rechtleitung einladen würdest, werden sie nie einer Rechtleitung folgen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcxcviii{Und dein HERR ist Der Allvergebende, mit der Allgnade. Würde ER sie belangen für das, was sie sich erworben haben, hätte ER ihnen die Peinigung beschleunigt. Nein, sondern für sie ist ein Termin bestimmt, vor dem sie keine Zuflucht finden werden.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcxcix{Und diese Ortschaften haben WIR zugrunde gerichtet, nachdem sie Unrecht begingen. Und WIR setzten für ihr Zugrunde-Richten einen Termin fest.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcc{Und erinnere daran), als Musa seinem Dienstjungen sagte: ``Ich werde (das Reisen) nicht unterbrechen, bis ich die Vereinigung beider Meere erreiche, auch wenn ich doch für längere Zeit reisen muß.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcci{Und als sie die Vereinigung der beiden (Meere) erreichten, vergaßen sie ihren Fisch, dann nahm er seinen Weg ins Meer durch einen Hohlraum.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmccii{Und nachdem sie von da weiterzogen, sagte er seinem Dienstjungen: ``Bereite uns unser Mittagessen, gewiß, bereits haben wir auf dieser unserer Reise große Mühe gehabt.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcciii{Er sagte: ``Wie siehst du es? Als wir am Felsen rasteten, habe ich doch den Fisch vergessen. Und nichts hat mich ihn vergessen lassen außer dem Satan, damit ich mich daran nicht erinnere. Und er nahm auf seltsame Weise seinen Weg ins Meer.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcciv{Er sagte: ``Das ist genau, was wir anstreben." Dann kehrten beide ihren Spuren folgend zurück.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmccv{Dann fanden sie einen Diener von Unseren Dienern, dem WIR Gnade von Uns zuteil werden ließen und den WIRWissen von Uns lehrten.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmccvi{Musa sagte zu ihm: ``Darf ich dir folgen, so daß du mich von dem lehrst, was dir vom Guten gelehrt wurde?"\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmccvii{Er sagte: ``Gewiß, du wirst mit mir keine Geduld haben können.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmccviii{Und wie kannst du Geduld aufbringen für das, wofür du über keine Mitteilung verfügst?"\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmccix{Er sagte: ``Du wirst mich inscha-allah geduldig finden, und ich werde dir in keinerlei Anordnung widersprechen!"\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmccx{Er sagte: ``Solltest du mir folgen, dann frage mich nach nichts, bis ich dir darüber Auskunft gebe."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmccxi{Dann gingen beide los. Als die Beiden dann in das Schiff stiegen, schlug er ein Loch hinein. Er (Musa) sagte: ``Schlägst du etwa ein Loch hinein, damit du seine Passagiere ertränkst? Gewiß, bereits hast du eine verwerfliche Tat begangen."\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmccxii{Er sagte: ``Habe ich nicht gesagt, daß du gewiß mit mir keine Geduld haben kannst?"\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmccxiii{Er sagte: ``Mache mir keine Vorwürfe für das, was ich vergaß, und erlege mir in meiner Angelegenheit nicht noch Erschwernis auf."\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmccxiv{Dann gingen beide weiter. Als die Beiden dann auf einen Jungen trafen, tötete er ihn sofort. Er (Musa) sagte: ``Hast du etwa einen unschuldigen Menschen getötet, (grundlos und) nicht als Vergeltung für einen (anderen) Menschen?! Gewiß, bereits hast du eine ungeheuerliche Tat begangen."\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmccxv{Er sagte: ``Habe ich dir nicht gesagt, daß du gewiß mit mir keine Geduld haben kannst?"\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmccxvi{Er sagte: ``Sollte ich dich nach irgend etwas nach diesem fragen, dann laß mich nicht länger dein Begleiter sein, bereits hast du von mir aus eine Entschuldigung."\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmccxvii{Dann gingen beide weiter. Als die Beiden dann die Bewohner einer Ortschaft erreichten, fragten beide ihre Bewohner nach Speise, aber diese weigerten sich, ihnen Gastfreundschaft zu gewähren. Dann fanden beide eine Mauer, die dabei war, einzustürzen, und er richtete sie wieder auf. Er (Musa) sagte: ``Wenn du möchtest, könntest du dafür Lohn verlangen."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmccxviii{Er sagte: ``Dies ist das Trennende zwischen mir und dir! Ich werde dir jedoch die Deutung dessen mitteilen, wofür du keine Geduld aufbringen konntest."\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmccxix{Hinsichtlich des Schiffes: Dieses war im Besitz armer Menschen, welche auf dem Meere arbeiteten. Und ich wollte es mangelhaft machen, denn vor ihnen gab es einen König, der sich jedes (guten) Schiffs gewaltsam bemächtigte.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmccxx{Und hinsichtlich des Jungen: Dessen Eltern waren beide Mumin, und wir fürchteten, daß er beide mit Ausschweifung und Kufr überzieht.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmccxxi{Dann wollten wir, daß ihr HERR ihnen an seiner Stelle (ein Kind) gibt, welches lauterer und barmherziger als dieser ist.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmccxxii{Und hinsichtlich der Mauer: Sie gehörte zwei Waisenjungen in der Stadt. Und darunter lag ein Schatz, der für beide bestimmt war. Und ihr Vater war ein gottgefällig Guttuender. So wollte dein HERR, daß beide erwachsen werden und ihren Schatz ausgraben - aus Gnade von deinem HERRN. Und ich habe dies nicht aus eigenem Erwägen getan. Dies ist die Deutung dessen, wofür du keine Geduld aufbringen konntest.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmccxxiii{Und sie fragen dich nach Dhul-qarnain. Sag: ``Ich werde euch etwas über ihn vortragen."\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmccxxiv{Gewiß, WIR gewährten ihm Macht auf Erden und gaben ihm zu jeder Sache einen Zugang.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmccxxv{Dann folgte er einem Weg.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmccxxvi{Als er dann (den Ort) des Sonnenuntergangs erreichte, fand er sie so, daß sie in einer schlammigen Quelle untergehe, und er fand in ihrer Nähe Leute. WIR sagten: ``Dhul-qarnain! Entweder peinigst du sie, oder du gewährst ihnen Gutes."\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmccxxvii{Er sagte: ``Hinsichtlich dessen, der Unrecht beging, den werden wir peinigen, dann wird er zu seinem HERRN zurückgebracht, dann wird ER ihn einer ungeheuerlichen Peinigung unterziehen.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmccxxviii{Doch hinsichtlich dessen, der den Iman verinnerlichte und gottgefällig Gutes tat, für den ist die gute Belohnung bestimmt, und wir werden ihm Leichtes gebieten."\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmccxxix{Dann folgte er einem Weg.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmccxxx{Als er dann (den Ort) des Sonnenaufgangs erreichte, fand er sie so, daß sie über Leute aufgeht, gegen sie WIR ihnen keinen Schutz gaben.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmccxxxi{Solcherart war es. Und bereits hatten WIR umfassendes Wissen über das, worüber er verfügte.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmccxxxii{Dann folgte er einem Weg.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmccxxxiii{Als er dann (den Ort) zwischen den beiden Wällen erreichte, fand er hinter ihnen Leute, die kaum Gesagtes verstanden.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmccxxxiv{Sie sagten: ``Dhulqarnain! Gewiß, Yadschudsch und Madschudsch sind Verderben-Anrichtende auf Erden. Sollten wir dir eine Löhnung dafür ansetzen, daß du zwischen uns und ihnen einen Wall errichtest?"\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmccxxxv{Er sagte: ``Das, worin mich mein HERR festigte, ist besser. Also unterstützt mich durch Arbeitskraft, damit ich zwischen euch und ihnen einen Damm errichte.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmccxxxvi{Bringt mir Eisenstücke!" Als er dann (die Lücke) zwischen den beiden Wallabhängen auf gleicher Höhe zuschütten ließ, sagte er: ``Blast!" Als er dann es (das Eisen) erhitzte, sagte er: ``Bringt mir geschmolzenes Kupfer, das ich darüber gieße!"\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmccxxxvii{Dann konnten sie ihn weder erklimmen, noch darin ein Loch bohren.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmccxxxviii{Er sagte: ``Dies ist eine Gnade von meinem HERRN. Und wenn das Versprechen meines HERRN eintrifft, wird ER ihn dem Erdboden gleich machen. Und das Versprechen meines HERRN ist wahr."\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmccxxxix{Und WIR lassen die einen von ihnen an diesem Tag sich mit den anderen vermischen. Und nachdem es in As-sur gestoßen wird, werden WIR sie in einer Versammlung versammeln.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmccxl{Und WIR werden Dschahannam an diesem Tag den Kafir in einer Vorführung zeigen.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmccxli{Es sind diejenigen, deren Augen mit einer Blende gegen Meine Ermahnung umhüllt waren. Und sie vermochten nicht zu hören.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmccxlii{Dachten etwa diejenigen, die Kufr betrieben haben, daß Meine Diener sich an Meiner Stelle andere Wali nehmen. Gewiß, WIR bereiteten Dschahannam den Kafir als Unterkunft.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmccxliii{Sag: ``Sollen wir euch eine Mitteilung über die Erfolglosesten beim Handeln machen?!"\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmccxliv{Es sind diejenigen, deren Bestreben im diesseitigen Leben verfehlt war, während sie dachten, sie bewerkstelligten doch Gutes.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmccxlv{Es sind diese, die den Ayat ihres HERRN und Dessen Begegnung gegenüber Kufr betrieben haben, so wurden ihre Handlungen zunichte. Und am Tage der Auferstehung messen WIR ihnen keinerlei Bedeutung bei.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmccxlvi{Dies ist ihre Vergeltung - Dschahannam, für das, was sie an Kufr betrieben haben, und dafür, daß sie Meine Ayat und Meine Gesandten zu verspotten pflegten.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmccxlvii{Gewiß, diejenigen, die den Iman verinnerlichen und gottgefällig Gutes tun - für sie sind die Dschannat von Al-Firdaus als Unterkunft bestimmt.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmccxlviii{Darin werden sie ewig bleiben, und sich daraus keinen Umzug wünschen.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmccxlix{Sag: ``Wäre das Meer eine Schreibflüssigkeit für die Worte meines HERRN, würde das Meer auf jeden Fall ausgehen, bevor die Worte meines HERRN enden, auch dann sollten wir Seinesgleichen als Nachschub bringen."\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmccl{Sag: ``Ich bin nur ein Mensch euresgleichen, dem Wahy zuteil wird, daß euer Gott nur ein einziger Gott ist. Wer also die Begegnung seines HERRN erwartet, der soll gottgefällig gute Tat vollbringen und dem Dienen seines HERRN gegenüber keinerlei Schirk mit einem anderem betreiben.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmccli{\basmalahde Kaf-ha-ja-'ain-sad.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcclii{Dies ist die Mitteilung über die Gnade, die dein HERR Seinem Diener Zakaria erwiesen hat,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmccliii{als dieser seinen HERRN mit einem unauffälligen Ruf anrief.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmccliv{Er sagte: ``HERR! Gewiß, mir liegt die Schwäche in den Knochen und mein Kopf ist voll grauer Haare. Doch nie war ich mit dem Bittgebet zu Dir - HERR!  erfolglos.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcclv{Auch bin ich besorgt über das Verhalten der Erben nach meinem Tod, und meine Gattin war immer unfruchtbar, so schenke mir von Dir aus einen Wali,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcclvi{der mich beerbt und auch von der Familie Ya'qubs erbt, und lasse ihn - HERR! - ein Angenommener sein!"\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcclvii{Zakaria! Gewiß, WIR überbringen dir doch frohe Botschaft von einem Jungen Namens Yahya. Dessen Namen haben WIR niemandem zuvor gegeben.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcclviii{Er sagte: ``HERR! Wie kann ich einen Jungen haben, wo meine Gattin immer unfruchtbar war und ich bereits das Greisenalter erreicht habe?!"\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcclix{Er (der Engel) sagte: ``Es ist so! Dein HERR sagte: ``Dies ist Mir ein Leichtes! ICH erschuf bereits auch dich, obwohl du nichts warst."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcclx{Er sagte: ``HERR! Lege mir eine Aya fest!" ER sagte: ``Deine Aya sei, daß du mit den Menschen drei volle Nächte nicht reden kannst."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcclxi{Sogleich ging er heraus aus der Gebetsstätte zu seinen Leuten und gab ihnen zu verstehen: ``Lobpreist (ALLAH) morgens und abends!"\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmcclxii{Yahya! Halte dich mit Nachdruck an die Schrift! Und WIR ließen ihm als Kind die Weisheit zuteil werden,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcclxiii{sowie Mildherzigkeit von Uns und Reinheit. Und er war ein Muttaqi.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcclxiv{Er war auch gütig seinen Eltern gegenüber, und er war kein widerspenstiger Widersacher.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmcclxv{Und Salam sei auf ihm am Tage, als er geboren wurde, und am Tage, wenn er stirbt, und am Tage, wenn er lebendig erweckt wird.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmcclxvi{Und erwähne in der Schrift Maryam, als sie sich von ihrer Familie an einen östlichen Platz zurückzog,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmcclxvii{und vor ihnen einen Sichtschutz errichtete. Dann entsandten WIR ihr einen Ruhh von Uns und er erschien ihr als ein makelloser Mensch.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmcclxviii{Sie sagte: ``Ich suche Schutz beim Allgnade Erweisenden vor dir, wenn du ein Muttaqi bist!"\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmcclxix{Er erwiderte: ``Ich bin doch nur ein Gesandter deines HERRN, um dir einen reinen Jungen zu schenken!"\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmcclxx{Sie sagte: ``Wie kann ich einen Jungen haben, wo mich noch nie ein Mensch intim berührte und ich nie eine Unzüchtige war?"\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmcclxxi{Er sagte: ``Es ist so! Dein HERR sagte: ``Dies ist Mir ein Leichtes!" Und zweifellos werden WIR ihn zu einer Aya machen für die Menschen und zu einer Gnade von Uns! Und dies ist bereits eine beschlossene Sache.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmcclxxii{Dann wurde sie schwanger mit ihm, und zog sich mit ihm unmittelbar an einen entfernten Ort zurück.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmcclxxiii{Sogleich kamen ihr die Wehen am Stamm der Dattelpalme. Sie sagte: ``Wäre ich doch vordem tot gewesen und als etwas Vergessenes vergessen!"\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmcclxxiv{Da rief sie der Engel von unten her: ``Sei nicht traurig! Bereits ließ dein HERR unter dir ein Bächlein fließen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmcclxxv{Auch schüttele den Stamm der Dattelpalme zu dir! So läßt sie ausgereifte Datteln über dich fallen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmcclxxvi{Also iß und trink und sei fröhlich! Und solltest du einen der Menschen sehen, so sprich: ``Ich habe Dem Allgnade Erweisenden ein Fasten gelobt, deshalb werde ich heute mit keinem Menschen sprechen!"\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmcclxxvii{Und sie kam mit ihm zu ihren Leuten, sie trug ihn. Sie sagten: ``Maryam! Gewiß, bereits hast du eine ungeheuerliche Sache begangen!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmcclxxviii{Schwester Haruns! Weder war dein Vater ein Mensch des Bösen, noch war deineMutter eine Unzüchtige!"\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmcclxxix{Daraufhin zeigte sie nur auf ihn. Sie sagten: ``Wie sollen wir mit jemandem sprechen, der noch in der Wiege liegt?"\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmcclxxx{Er sagte: ``Ich bin ALLAHs Diener! ER ließ mir die Schrift zuteil werden und machte mich zum Propheten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmcclxxxi{ER machte mich auch zu einem mit Baraka Erfüllten, überall wo ich bin. Und ER hat mir das rituelle Gebet und die Zakat geboten, solange ich lebe.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmcclxxxii{ER machte mich) auch zu einem Gütigen meiner Mutter gegenüber. Doch ER machte mich nicht zu einem widerspenstigen Widersacher!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmcclxxxiii{Und Salam sei mit mir an dem Tag, als ich geboren wurde, und an dem Tag, wenn ich sterbe, und an dem Tag, wenn ich lebendig erweckt werde -\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmcclxxxiv{dies ist 'Isa Ibnu-Maryam. Er ist das Wort der Wahrheit, über den sie zweifeln.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmcclxxxv{ALLAH gebührt in keiner Weise, daß ER Sich Kinder nimmt, gepriesen-erhaben ist ER! Wenn ER eine Angelegenheit bestimmt, so sagt ER ihr nur: ``Sei!" Und sie ist -\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmcclxxxvi{Und gewiß, ALLAH ist mein HERR und euer HERR! So dient nur Ihm! Dies ist ein geradliniger Weg."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmcclxxxvii{Danach wurden die Parteien unter ihnen uneins. Und Untergang sei denjenigen, die Kufr betrieben haben, beim Erleben eines gewaltigen Tages!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmcclxxxviii{Wie sehend und hörend werden sie sein, wenn sie zu Uns kommen! Doch die Unrecht- Begehenden sind heute im offenkundigen Irrtum.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmcclxxxix{Und warne sie vor dem Tag der Selbstanklage, wenn die Angelegenheit beschlossen wird, während sie in Achtlosigkeit sind und keinen Iman verinnerlichen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmccxc{Gewiß, WIR sind Derjenige, Der die Erde und alles auf ihr wieder zu Sich nimmt, und zu Uns werden sie zurückgebracht.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmccxci{Und erwähne in der Schrift Ibrahim! Gewiß, er war ein äußerst Wahrhaftiger, ein Prophet!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmccxcii{Als er zu seinem Vater sagte: ``Mein Vater! Weshalb betest du etwas an, das weder sieht, noch hört, noch von dir irgend etwas abwenden kann?\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmccxciii{Mein Vater! Gewiß, mir wurde vom Wissen das zuteil, was dir nicht zuteil wurde, so folge mir, dann leite ich dich recht auf einen geraden Weg!\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmccxciv{Mein Vater! Diene nicht dem Satan! Gewiß, der Satan war Dem Allgnade Erweisenden immer ein Widersacher!\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmccxcv{Mein Vater! Gewiß, ich fürchte, daß dich eine Peinigung vom Allgnade Erweisenden trifft, und du ein Vertrauter Satans sein wirst."\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmccxcvi{Er sagte: ``Bist du etwa meinen Göttern gegenüber abgeneigt, du Ibrahim?! Wenn du damit nicht aufhörst, werde ich dich gewiß steinigen! Und bleibe fern von mir für längere Zeit!"\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmccxcvii{Er sagte: ``Salam sei mit dir. Ich werde für dich meinen HERRN um Vergebung bitten. Denn ER ist mir gegenüber immer großzügig gewesen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmccxcviii{Und ich ziehe mich von euch und von dem zurück, an das ihr anstelle von ALLAH Bittgebete richtet. Und ich richte Bittgebete an meinen HERRN! Ich hoffe, daß ich mit dem Bittgebet an meinen HERRN nicht erfolglos bin."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmccxcix{Nachdem er sich von ihnen und von dem zurückgezogen hatte, was sie anstelle von ALLAH anbeteten, haben WIR ihm Ishaq und Ya'qub geschenkt. Und beide machten WIR zu Propheten.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmccc{Und WIR haben ihnen von Unserer Gnade geschenkt und ihnen eine wahrhaftige, überragende Anerkennung zukommen lassen!\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmccci{Und erwähne in der Schrift Musa! Gewiß, er war ein Auserwählter, und war ein Gesandter, ein Prophet.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcccii{WIR ließen ihn von der rechten Seite des Tur- Bergs rufen, und Uns als Vertrauten näher bringen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmccciii{Und WIR schenkten ihm aus Unserer Gnade heraus seinen Bruder Harun als einen Propheten.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmccciv{Und erwähne in der Schrift Isma'il! Gewiß, er war wahrhaftig im Versprechen und war ein Gesandter, ein Prophet.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcccv{Und er pflegte, seiner Familie das rituelle Gebet und die Zakat zu gebieten. Und er war bei seinem HERRN ein Angenommener!\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcccvi{Und erwähne in der Schrift Idris! Gewiß, er war ein äußerst Wahrhaftiger, ein Prophet.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcccvii{Und WIR erhoben ihn zu einem hohen Ort.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcccviii{Diese sind diejenigen, denen ALLAH Gaben zuteil werden ließ von den Propheten aus der Nachkommenschaft Adams, und von denjenigen, die WIR mit Nuh trugen, und aus der Nachkommenschaft Ibrahims und Israils, und von denjenigen, dieWIR rechtgeleitet und auserwählt haben. Wenn ihnen die Ayat des Allgnade Erweisenden vorgetragen wurden, fielen sie sudschud-vollziehend und weinend nieder.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcccix{Doch dann folgten ihnen Nachfahren, die das rituelle Gebet vernachlässigten und ihren Neigungen folgten. Diese werden noch Böses finden,\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcccx{außer denjenigen, die reuig wurden, den Iman verinnerlicht und gottgefällig Gutes getan haben, denn diese werden in die Dschanna eintreten und ihnen wird in keiner Weise Unrecht zugefügt.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcccxi{Sie treten ein in) Dschannat von 'Adn, die Der Allgnade Erweisende Seinen Dienern von der Welt des Verborgenen versprochen hat. Sein Versprechen wird zweifelsohne erfüllt werden!\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcccxii{Darin hören sie nichts Unnützes - (sie hören) nur den Salam-Gruß! Und ihnen ist ihre Versorgung darin morgens und abends gesichert!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcccxiii{Diese ist die Dschanna, die WIR für diejenigen von Unseren Dienern bleiben lassen, die Muttaqi waren.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcccxiv{Und wir (die Engel) steigen nicht hinunter, es sei denn gemäß der Anweisung deines HERRN. Ihm gehört, was vor uns, was hinter uns und was dazwischen ist. Und dein HERR vergißt nie.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcccxv{ER ist Der HERR der Himmel, der Erde und dessen, was zwischen beiden ist, so diene Ihm und übe dich in Geduld bei seiner Verehrung. Kennst du etwa einen, der mit Ihm den Namen teilt?!\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcccxvi{Und der Mensch sagt: ``Wenn ich sterbe, werde ich dann etwa zum Leben erweckt werden?"\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcccxvii{Bedenkt der Mensch etwa nicht, daß WIR ihn gewiß vorher erschufen, nachdem er nichts gewesen war?!\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmcccxviii{Bei deinem HERRN! WIR werden sie doch versammeln, sowie die Satane. Dann werden WIR sie doch kniend um Dschahannam herum bringen lassen.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmcccxix{Dann werden WIR von jeder Gefolgschaft doch diejenigen herausgreifen, die gegenüber Dem Allgnade Erweisenden heftiger in der Übertretung waren.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmcccxx{Denn WIR wissen gewiß besser Bescheid über diejenigen, die eher verdienen, hineingeworfen zu werden!\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmcccxxi{Und jeder von euch wird es (das Feuer) gewiß ansteuern. Für deinen HERRN ist dies eine beschlossene Notwendigkeit.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmcccxxii{Dann erretten WIR diejenigen, die Taqwa gemäß handelten, und lassen die Unrecht-Begehenden darin kniend.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcccxxiii{Und wenn ihnen Unsere Ayat in Klarheit vorgetragen werden, sagen diejenigen, die Kufr betrieben haben, denjenigen, die den Iman verinnerlicht haben: ``Welche der beiden Gruppierungen hat den besseren Wohnort und die angenehmere Gesellschaft?"\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmcccxxiv{Wie viele Generationen vor ihnen haben WIR bereits zugrundegerichtet, die über noch schönere Ausstattungen und Sehenswürdigkeiten verfügten!\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmcccxxv{Sag also: ``Wer sich im Irrtum befindet, dem soll Der Allgnade Erweisende noch mehr davon gewähren!" Wenn sie dann sehen, was ihnen angedroht wurde: entweder die Peinigung oder die Stunde, dann werden sie wissen, wer über die schlimmere Stellung und die schwächere Streitmacht verfügt.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcccxxvi{Und ALLAH läßt diejenigen, die der Rechtleitung folgten, noch mehr an Rechtleitung gewinnen. Und die bleibenden gottgefälligen guten Taten haben bei deinem HERRN die bessere Belohnung und den besseren Zugewinn.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcccxxvii{Auch sollst du Mitteilung machen über denjenigen, der Unseren Ayat gegenüber Kufr betrieben und gesagt hat: ``Ganz gewiß werden mir Vermögen und Kinder zuteil!"\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcccxxviii{Hat er etwa Einsicht in das Verborgene gehabt oder etwa mit Dem Allgnade Erweisenden eine Abmachung getroffen?!\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmcccxxix{Bestimmt nicht! WIR werden registrieren lassen, was er sagt, und werden ihm noch viel mehr von der Peinigung zuteilen lassen.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmcccxxx{Und WIR werden ihm das wegnehmen, worüber er sprach. Auch wird er ja ohnehin alleine zu Uns kommen!\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmcccxxxi{Und sie nahmen sich Götter anstelle von ALLAH, damit diese ihnen Ansehen verleihen.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmcccxxxii{Bestimmt nicht! Sie werden ihrer Anbetung gegenüber Kufr betreiben und werden ihre Kontrahenten sein.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmcccxxxiii{Weißt du etwa nicht, daß WIR die Satane zu den Kafir schickten, damit sie diese ständig aufhetzen?!\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmcccxxxiv{So hab keine Eile mit ihnen! Denn WIR zählen ihnen nur noch ihre Tage auf.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmcccxxxv{Und erinnere) an den Tag, wenn WIR die Muttaqi vor Dem Allgnade Erweisenden in Delegation versammeln,\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmcccxxxvi{und die schwer Verfehlenden als Herde zu Dschahannam treiben.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmcccxxxvii{Sie verfügen über keine Fürbitte, sondern nur diejenigen, die eine Abmachung mit Dem Allgnade Erweisenden trafen.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmcccxxxviii{Und sie sagten: ``Der Allgnade Erweisende hat sich einen Sohn genommen."\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmcccxxxix{Gewiß, bereits habt ihr (damit) eine Ungeheuerlichkeit vollbracht!\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmcccxl{Beinahe würden davon die Himmel nach und nach aufreißen, die Erde sich spalten und die Felsengebirge in Trümmer auseinanderfallen,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmcccxli{daß sie Dem Allgnade Erweisenden einen Sohn zugeschrieben haben.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmcccxlii{Und in keiner Weise gebührt Dem Allgnade Erweisenden, daß ER sich einen Sohn nimmt!\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmcccxliii{Ganz gewiß gibt es nichts in den Himmeln und auf Erden, das nicht vor Dem Allgnade Erweisenden als Anbeter kommt!\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmcccxliv{Gewiß, bereits kennt ER sie umfassend und zählte sie genau auf!\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmcccxlv{Und alle kommen zu Ihm einzeln am Tag der Auferstehung.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmcccxlvi{Gewiß, diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, denen wird Der Allgnade Erweisende Liebe gewähren.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmcccxlvii{WIR haben ihn (den Quran) in deiner Sprache nur erleichtert, damit du mit ihm den Muttaqi frohe Botschaft überbringst und mit ihm die entschiedenen Gegner warnst.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmcccxlviii{Und wie viele Generationen vor ihnen haben WIR bereits zugrundegerichtet! Verspürst du etwa von ihnen irgendeinen, oder vernimmst du von ihnen etwa irgendein Geräusch?!\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmcccxlix{\basmalahde Ta-ha.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcccl{WIR sandten dir den Quran nicht hinab, damit du dich erschöpfst.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcccli{Es ist nur eine Erinnerung für diejenigen, die Ehrfurcht haben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmccclii{Er ist) eine sukzessive Hinabsendung von Demjenigen, Der die Erde und die hohen Himmel erschuf.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcccliii{ER ist Der Allgnade Erweisende, Der über Al'ahrsch komplett herrschte.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcccliv{Ihm gehört, was in den Himmeln, was auf Erden, was zwischen beiden und was unter der Erde ist.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmccclv{Und solltest du das Gesagte laut sprechen, so kennt ER doch das Geheimnis und das noch Verborgenere.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmccclvi{ALLAH, es gibt keine Gottheit außer Ihm! Ihm gebühren die schönsten Namen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmccclvii{Und wurde dir die Begebenheit von Musa etwa nicht zuteil?!\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmccclviii{Nachdem er Feuer gesehen hatte, sagte er seiner Familie: ``Bleibt! Ich sah Feuer, vielleicht bringe ich euch davon Feuerbrand oder finde beim Feuer (jemanden, der) mich rechtleitet.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmccclix{Und als er daran war, wurde gerufen: ``Musa!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmccclx{Gewiß, ICH bin dein HERR, so ziehe deine Schuhe aus, denn du bist gewiß im rein gehaltenen Tal Tuwa.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmccclxi{Und ICH habe dich erwählt, so höre dem zu, was als Wahy erteilt wird:\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmccclxii{Gewiß, ICH bin ALLAH. Keine Gottheit existiert, es sei denn ICH, so diene Mir und verrichte ordnungsgemäß das rituelle Gebet zu Meinem Gedenken.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmccclxiii{Zweifellos, die Stunde ist im Kommen. Beinahe werde ICH sie enthüllen, damit jeder Seele nach ihrem Bestreben vergolten wird.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmccclxiv{So soll dich von ihr nicht abwenden derjenige, der an sie keinen Iman verinnerlicht und seinen Neigungen folgt, sonst gehst du zugrunde.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmccclxv{Und was ist dies in deiner Rechten, Musa?!"\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmccclxvi{Er sagte: ``Es ist mein Stock, auf den stütze ich mich und mit ihm treibe ich meine Schafe zusammen. Und damit habe ich noch andere Verwendung."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmccclxvii{ER sagte: ``Wirf ihn hin, Musa!"\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmccclxviii{Sogleich warf er ihn hin, und er wurde zu einer Schlange, die umherglitt.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmccclxix{ER sagte: ``Nimm sie auf und fürchte dich nicht! WIR werden sie in ihre vorherige Gestalt zurückverwandeln.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmccclxx{Und führe deine Hand zu deiner Seite heran, kommt sie hell unversehrt heraus als eine zweite Aya,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmccclxxi{damit WIR dir manche von Unseren großen Ayat zeigen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmccclxxii{Gehe zu Pharao, gewiß beging er äußerstes Übertreten!"\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmccclxxiii{Er sagte: ``Mein HERR! Entspanne mir meine Brust,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmccclxxiv{erleichtere mir meine Angelegenheit\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmccclxxv{und entknote eines Knotens meine Zunge,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmccclxxvi{damit sie mein Gesagtes begreifen!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmccclxxvii{Und ernenne mir einen Assistenten von meiner Familie,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmccclxxviii{Harun, meinen Bruder,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmccclxxix{stärke mich mit ihm\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmccclxxx{und beteilige ihn an meiner Angelegenheit,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmccclxxxi{damit wir Dich viel lobpreisen\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmccclxxxii{und dir viel gedenken.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmccclxxxiii{Gewiß, DU bist uns immer allsehend."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmccclxxxiv{ER sagte: ``Bereits wurde dir deine Bitte gewährt, Musa!"\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmccclxxxv{Und gewiß, bereits erwiesen WIR dir ein anderes Mal Wohltaten,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmccclxxxvi{als WIR deiner Mutter Wahy zuteil werden ließen von dem, was als Wahy erteilt wird:\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmccclxxxvii{``Lege ihn in die Holzkiste, dann wirf sie in den Fluß, dann wird der Fluß sie ans Ufer spülen, dann nimmt ihn ein Feind von Mir und ein Feind von ihm." Und ICH machte dich beliebt und damit du unter Meiner Aufsicht erzogen wirst.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmccclxxxviii{Und erinnere daran), als deine Schwester vorbeiging und sagte: ``Soll ich euch jemanden zeigen, der ihn betreuen kann?" Dann brachten WIR dich zu deiner Mutter zurück, damit sie fröhlich und nicht traurig wird. Und du hast einen Menschen getötet, dann retteten WIR dich von der Betrübnis und ließen dich mehreren Fitnas unterziehen, dann bist du (einige) Jahre bei den Leuten von Madyan geblieben. Dann kamst du der Bestimmung nach, Musa!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmccclxxxix{Und ICH habe dich für Mich auserwählt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmcccxc{Gehe du und dein Bruder mitMeinen Ayat und laßt nicht mit Meinem Gedenken nach!\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmcccxci{Geht beide zu Pharao, denn gewiß, er beging äußerste Übertretungen,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmcccxcii{dann sagt ihm ein mildes Wort, damit er sich erinnert oder sich fürchtet.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmcccxciii{Beide sagten: ``Unser HERR! Gewiß, wir fürchten, daß er uns mit Bösem ereilt oder daß er äußerste Übertretungen begeht."\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmcccxciv{ER sagte: ``Fürchtet euch beide nicht! Gewiß, ICH bin mit euch beiden, ICH höre und sehe.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmcccxcv{So geht beide zu ihm, dann sagt beide: ``Wir sind beide die Gesandten deines HERRN, so schicke mit uns die Kinder Israils und peinige sie nicht! Bereits brachten wir dir eine Aya von deinem HERRN. Und Salam sei auf demjenigen, welcher der Rechtleitung folgt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmcccxcvi{Gewiß, uns wurde bereits als Wahy zuteil, daß die Peinigung doch dem bestimmt ist, der leugnet und sich abwendet."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmcccxcvii{Er sagte: ``Wer ist denn euer HERR, Musa?"\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcccxcviii{Er sagte: ``Unser HERR ist Derjenige, Der jeder Sache ihre Gestalt verliehen hat, dann Rechtleitung erwies."\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcccxcix{Er sagte: ``(Und) was ist denn mit den früheren Generationen?"\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcd{Er sagte: ``Das Wissen über sie ist bei meinem HERRN in einem Register. Weder irrt Mein HERR noch vergißt ER."\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcdi{ER ist Derjenige, Der euch die Erde als Unterlage machte, für euch darin Wege bahnte und vom Himmel Wasser fallen ließ, mit dem WIR Paare von vielfältigen Pflanzen hervorbrachten.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcdii{Esst und lasst eure An'am weiden! Gewiß, darin sind doch Ayat für die Verständigen.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcdiii{Aus ihr erschufen WIR euch und in sie bringen WIR euch zurück und aus ihr bringen WIR euch noch einmal hervor.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcdiv{Und gewiß, bereits zeigten WIR ihm alle Unsere Ayat, dann leugnete er und lehnte sie ab.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcdv{Er sagte: ``Bist du etwa zu uns gekommen, Musa! - mit deiner Magie, damit du uns von unserem Land vertreibst?\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcdvi{also sicher werden wir dir gleichwertige Magie entgegensetzen, so lege zwischen uns und dir einen Termin fest, den weder wir noch du versäumen, an einem (uns) bekannten Ort!"\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcdvii{Er sagte: ``Euer Termin ist der Tag der Festlichkeit. Und die Menschen sollen am Vormittag versammelt werden.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcdviii{Dann wandte Pharao sich ab und sammelte sein Listzeug zusammen, dann kam er.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcdix{Musa sagte ihnen: ``Euer Untergang! Erfindet im Namen ALLAHs keine Lügen, sonst richtet ER euch mit einer Peinigung zugrunde. Und bereits versagen wird derjenige, der etwas erdichtet!"\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcdx{Und sie waren in ihrer Angelegenheit uneins und verheimlichten ihre Unterredung.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcdxi{Sie sagten: ``Diese beiden sind sicherlich Magier, die euch aus eurem Land mit ihrer Magie vertreiben wollen und eure ideale Lebensweise annullieren.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcdxii{Also kommt in eurer List überein, dann kommt in Reihe. Und bereits ist erfolgreich an diesem Tag, wer die Oberhand gewinnt."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcdxiii{Sie sagten: ``Musa! Entweder wirfst du, oder werden wir die ersten sein, die werfen.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcdxiv{Er sagte: ``Nein, sondern werft (ihr)!" Und da erschienen ihm ihre Seile und Stöcke durch ihreMagie, als würden sie umhergleiten.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcdxv{Dann ängstigte sich insgeheim, Musa.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmcdxvi{WIR sagten: ``Fürchte dich nicht! Gewiß, du gewinnst die Oberhand.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmcdxvii{Und wirf das, was in deiner Rechten ist, es verschlingt alles, was sie bewerkstelligten! Sie bewerkstelligten doch nur eine List eines Magiers. Und der Magier wird nie erfolgreich sein, überall wo er hinkommt."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmcdxviii{So vollzogen die Magier Sudschud, sie sagten: ``Wir bekundeten den Iman an Den HERRN von Harun und Musa."\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmcdxix{Er (Pharao) sagte: ``Habt ihr etwa den Iman bekundet, bevor ich euch die Zustimmung dazu gab? Gewiß, er ist doch euer Lehrmeister, der euch die Magie beibrachte. Abschneiden werde ich doch eure Hände und Füße wechselseitig und kreuzigen werde ich euch gewiß an die Stämme der Dattelpalmen. Und ihr werdet zweifelsohne noch in Erfahrung bringen, welcher von uns über die qualvollere und dauerhaftere Peinigung verfügt."\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmcdxx{Sie sagten: ``Wir werden dich auf keinen Fall vor dem bevorzugen, was zu uns an eindeutigen Zeichen kam, und nicht vor Dem, Der uns erschuf. Also urteile, was du urteilst! Du fällst ein Urteil doch nur über dieses diesseitige Leben.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcdxxi{Gewiß, wir verinnerlichten den Iman an unseren HERRN, damit ER uns unsere Verfehlungen vergibt und das, wozu du uns an Magie gezwungen hast. Und ALLAH ist besser und bleibender!"\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmcdxxii{Gewiß, wer zu seinem HERRN als schwer Verfehlender kommt, für den ist Dschahannam bestimmt, darin weder stirbt, noch lebt er.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmcdxxiii{Und wer zu Ihm als Mumin kommt, wobei er gottgefällig Gutes tat, für diese sind die hohen Ränge bestimmt,\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcdxxiv{die 'Adn-Dschannat, die von Flüssen durchflossen sind. Darin werden sie ewig bleiben. Und dies ist die Belohnung für denjenigen, der sich rein hält.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcdxxv{Und gewiß, bereits ließen WIR Musa Wahy zuteil werden: ``Brich nachts auf mit Meinen Dienern, dann schlage ihnen einen getrockneten Weg ins Meer! Habe keine Angst vor Verfolgung und fürchte dich nicht!"\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcdxxvi{Dann verfolgte sie Pharao mit seinen Soldaten, dann überschwemmte sie vom Fluß, was sie überschwemmte.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmcdxxvii{Und Pharao verleitete seine Leute und leitete sie nicht recht.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmcdxxviii{Kinder Israils! WIR erretteten euch bereits von eurem Feind, verabredeten Uns mit euch an der rechten Seite des Tur-Bergs und ließen euch Al-mann und As-salwa zuteil werden.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmcdxxix{Esst von den Tay-yibat vom Rizq, das WIR euch gewährten, und begeht darin keine Übertretungen, sonst widerfährt euch Mein Zorn. Und wem Mein Zorn widerfährt, ist bereits zugrunde gerichtet.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmcdxxx{Und gewiß, ICH bin bestimmt allvergebend für denjenigen, der reuig wurde, den Iman verinnerlichte, gottgefällig Gutes tat und dann Rechtleitung fand.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmcdxxxi{Und was ließ dich hier vor deinen Leuten sein, Musa?\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmcdxxxii{Er sagte: ``Sie sind da, sie folgen meinen Spuren. Und ich eilte zu Dir, mein HERR! - damit du (mir) Wohlgefallen erweist."\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmcdxxxiii{ER sagte: ``WIR ließen doch deine Leute nach dir der Fitna aussetzen. Und As-samiri verleitete sie."\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmcdxxxiv{Dann kehrte Musa zu seinen Leuten zurück, wütend und traurig. Er sagte: ``Meine Leute! Machte euch euer HERR etwa nicht ein schönes Versprechen?! War euch die vereinbarte Zeit zu lange, oder wolltet ihr, daß euch Zorn von eurem HERRN widerfährt, dann habt ihr das Versprechen mir gegenüber gebrochen?!"\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmcdxxxv{Sie sagten: ``Wir brachen das Versprechen dir gegenüber nicht aus eigener Kraft. Auf uns wurden aber Schwerlastende von dem Schmuck der Leute gelegt, dann warfen wir sie (ins Feuer)." Und solcherart warf ebenfalls As-samiri.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmcdxxxvi{Dann brachte er ihnen hervor ein Kalb, einen Körper mit Gebrüll. Dann sagten sie: ``Dies ist euer Gott und Musas Gott. Er vergaß es jedoch."\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmcdxxxvii{Sehen sie etwa nicht, daß es ihnen nichts Gesagtes erwidert, und daß es ihnen weder zu schaden noch zu nützen vermag?!\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmcdxxxviii{Und gewiß, bereits sagte Harun zu ihnen vorher: ``Meine Leute! Ihr wurdet mit ihm nur der Fitna ausgesetzt. Und euer HERR ist gewiß Der Allgnade Erweisende. Also folgt mir und gehorcht meiner Anweisung!"\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmcdxxxix{Sie sagten: ``Wir lassen es nicht, ihm Zuwendung zu erweisen, bis Musa zu uns zurückkehrt."\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmcdxl{Er sagte: ``Harun! Was hinderte dich daran, nachdem du gesehen hattest, daß sie irregingen,\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmcdxli{mir zu folgen? Hast du etwa dich gegen meine Anweisung widersetzt?!"\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmcdxlii{Er sagte: ``Sohn meiner Mutter! Zerre weder an meinem Bart, noch an meinem Kopf! Gewiß, ich fürchtete, du würdest sagen: ``Du hast die Kinder Israils gespalten und nicht auf meine Anweisung gewartet."\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmcdxliii{Er sagte: ``Was ist dein Bestreben, Samiri?"\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmcdxliv{Er sagte: ``Ich nahm wahr, was sie nicht wahrnahmen, dann griff ich eine Handvoll von der Spur des Entsandten, dann warf ich sie hinein. Und solcherart machte es mir meine Seele leicht."\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmcdxlv{Er sagte: ``Geh! Denn dir ist in diesem Leben gewährt zu sagen: ``Keine Berührung!" Und gewiß, für dich ist noch ein Versprechen bestimmt, das dir gegenüber nicht gebrochen wird. Und schau deinen Gott an, dem du ständig Zuwendung erwiesen hast, wir werden es zweifelsohne verbrennen, dann werden wir es gewiß in den Fluß streuen."\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmcdxlvi{Euer Gott ist nur ALLAH, Derjenige, außer Ihm es keine Gottheit gibt. ER umfaßt alles mit Wissen.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmcdxlvii{Solcherart berichten WIR dir über Nachrichten von dem, was sich früher ereignete. Und WIR ließen dir von Uns aus eine Ermahnung (den Quran) zuteil werden.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmcdxlviii{Wer sich von ihm abwendet, dieser wird gewiß am Tag der Auferstehung eine Schuldenlast tragen.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmcdxlix{Darin werden sie ewig bleiben. Und erbärmlich ist es für sie am Tag der Auferstehung als Schuldenlast,\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmcdl{am Tag, wenn in As-sur geblasen wird. Und WIR versammeln die schwer Verfehlenden an diesem Tag als Zurq.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmcdli{Sie flüstern zueinander: ``Ihr habt doch nur zehn Tage verweilt!"\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmcdlii{WIR wissen besser, was sie sagen, wenn derjenige, der über die beste Denkart verfügt, sagt: ``Ihr habt doch nur einen Tag verweilt!"\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmcdliii{Und sie fragen dich nach den Felsenbergen, dann sag: ``Mein HERR wird sie in (kleinste) Teilchen zerkleinern.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmcdliv{Dann läßt ER sie zu einer ebenen Fläche werden,\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmcdlv{auf der du weder Unebenheit noch Erhebung siehst.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmcdlvi{An diesem Tag folgen sie dem Rufenden ohne Abweichung. Und die Stimmen verstummen vor Dem Allgnade Erweisenden, dann vernimmst du nichts außer Flüstern.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmcdlvii{An diesem Tag nützt keine Fürsprache, außer (der Fürsprache) desjenigen, dem Der Allgnade Erweisende die Zustimmung gab und von ihm das Gesagte annahm.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmcdlviii{ER kennt das, was vor ihnen und was hinter ihnen liegt. Sie jedoch haben kein umfassendes Wissen über Ihn.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmcdlix{Und die Gesichter sind geneigt vor Dem Lebendigen, Dem Allverantwortlichen. Und bereits versagte jeder, der Unrecht auf sich lud.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmcdlx{Doch wer von gottgefällig Gutem tut, während er Mumin ist, der fürchtet weder Ungerechtigkeit noch Minderung.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmcdlxi{Und solcherart sandten WIR ihn hinab als einen arabischen Quran und erläuterten in ihm von der Ermahnung, damit sie Taqwa gemäß handeln und damit er ihnen eine Besinnung bewirkt.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmcdlxii{Und hocherhaben ist ALLAH, Der Allherrschende, Der Wahrhaftigste! Und übereile dich nicht mit der Rezitation des Quran, bevor dir dessen Wahy komplett verkündet wurde. Und sag: ``Mein HERR! Lasse mir noch mehr Wissen zuteil werden!"\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmcdlxiii{Und gewiß, bereits wiesen WIR Adam vorher an, doch er vergaß, und WIR stellten bei ihm keinen Vorsatz fest.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmcdlxiv{Und erinnere daran), als WIR den Engeln sagten: ``Vollzieht Sudschud für Adam!" Sie vollzogen Sudschud, außer Iblis hat es abgelehnt.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmcdlxv{Dann sagten WIR: ``Adam! Gewiß, dieser ist ein Feind für dich und für deine Gattin. Also lasst ihn nicht euch beide aus der Dschanna herausbringen, sonst mußt du dich mühen.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmcdlxvi{Es wird dir gewiß gewährt, daß du darin weder hungerst noch nackt wirst.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmcdlxvii{Und daß du darin weder dürstest noch unter Sonnenhitze leidest."\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmcdlxviii{Dann flüsterte ihm der Satan ein, er sagte: ``Soll ich dir zeigen den Baum der Ewigkeit und eine Herrschaft, die nie vergeht?"\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmcdlxix{Dann aßen beide von ihm, dann wurde beiden ihre Blöße sichtbar. Und beide begannen, von den Blättern der Dschanna auf sich zu flechten. Und Adam widersetzte sich seinem HERRN, so irrte er.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmcdlxx{Dann erwählte ihn sein HERR, dann vergab ER ihm und leitete ihn recht.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmcdlxxi{ER sagte: ``Steigt von ihr, allesamt hinab! Die einen von euch (dieMenschen) sind für die anderen (Satan) Feind. Und wenn euch von Mir Rechtleitung zuteil wird, wer dann Meiner Rechtleitung folgt, der wird weder abirren noch unglücklich werden.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmcdlxxii{Und wer sich von Meiner Ermahnung abwendet, für den ist dann ein elendes Leben bestimmt, und WIR werden ihn am Tag der Auferstehung als Blinden zur Versammlung bringen."\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmcdlxxiii{Er sagte: ``Mein HERR! Weshalb bringst DU mich als Blinden zur Versammlung, wo ich doch bereits sehend war?"\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmcdlxxiv{ER sagte: ``Solcherart wurden dir Meine Ayat zuteil, dann hast du sie vergessen. Und solcherart wirst du heute in Vergessenheit geraten."\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmcdlxxv{Und solcherart vergelten WIR dem, der Übertretungen beging und den Iman an die Ayat seines HERRN nicht verinnerlichte. Und die Peinigung des Jenseits ist gewiß noch härter und andauernder.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmcdlxxvi{Wurde ihnen etwa noch nicht deutlich, wie viele an Generationen vor ihnen WIR zugrunde richteten, durch deren Wohnstätten sie gehen?! Gewiß, darin sind doch Ayat für die Verständigen.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmcdlxxvii{Und gäbe es kein von deinem HERRN bereits gefälltes Wort und keine festgelegte Frist, wäre es (das Bestrafen) sicherlich notwendig.\qt@no{129}}%
\qt@newcmd\qurantrans@de@mmcdlxxviii{So übe dich in Geduld hinsichtlich dessen, was sie sagen. Und lobpreise mit dem Lob deines HERRN vor Sonnenaufgang und vor ihrem Untergang! Auch während der Nacht lobpreise Ihn und an den Tagesenden, damit du zufrieden wirst.\qt@no{130}}%
\qt@newcmd\qurantrans@de@mmcdlxxix{Und richte deine Augen nicht auf das, was WIR manchen von ihnen an vergänglichen Verbrauchsgütern als Schmuck des diesseitigen Lebens zur Verfügung stellten, damit WIR sie darin einer Fitna unterziehen. Und das Rizq von deinem HERRN ist noch besser und andauernder.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mmcdlxxx{Und gebiete deiner Familie das rituelle Gebet und übe dich darin in Geduld. WIR bitten dich nicht um Rizq. WIR gewähren dir Rizq. Und das Anschließende ist für die Taqwa.\qt@no{132}}%
\qt@newcmd\qurantrans@de@mmcdlxxxi{Und sie sagten: ``Würde er doch uns eine Aya von seinem HERRN bringen!" Kam zu ihnen etwa nicht die Erklärung dessen, was in den früheren Schriften war?!\qt@no{133}}%
\qt@newcmd\qurantrans@de@mmcdlxxxii{Und hätten WIR sie mit einer Peinigung vor ihm zugrunde gerichtet, hätten sie gewiß gesagt: ``Unser HERR! Würdest DU uns doch einen Gesandten schicken, dann folgten wir Deinen Ayat, bevor wir erniedrigt und herabgewürdigt werden."\qt@no{134}}%
\qt@newcmd\qurantrans@de@mmcdlxxxiii{Sag: ``Jeder wartet, so wartet doch! Ihr werdet noch wissen, welche zu den Weggenossen des geraden Weges gehören und welche Rechtleitung fanden."\qt@no{135}}%
\qt@newcmd\qurantrans@de@mmcdlxxxiv{\basmalahde Angenähert hat sich den Menschen ihre Abrechnung, während sie in Achtlosigkeit abgeneigt sind.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcdlxxxv{Ihnen wird keine Ermahnung von ihrem HERRN zuteil, die erneuert wird, ohne daß sie diese vernehmen, während sie Sinnloses treiben.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcdlxxxvi{Nachlässig sind ihre Herzen. Und ihre heimliche Unterhaltung verschwiegen haben diejenigen, die Unrecht begingen: ``Ist dieser etwas anderes als ein Mensch wie ihr? Wollt ihr etwa dieMagie annehmen, während ihr es seht?"\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmcdlxxxvii{Sag: ``Mein HERR kennt das Gesagte im Himmel und auf Erden. Und ER ist Der Allhörende, Der Allwissende!"\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcdlxxxviii{Nein, sondern sie sagten: ``Es ist ein Durcheinander von Träumen. Nein, sondern er hat es erdichtet. Nein, sondern er ist ein Dichter. Dann soll er uns eine Aya bringen, wie es mit den Früheren entsandt wurde."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcdlxxxix{Keinen Iman verinnerlichten vor ihnen (die Bewohner) einer Ortschaft, die WIR zugrunde richteten, werden diese etwa den Iman verinnerlichen?!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcdxc{Und WIR entsandten vor dir als Gesandte nichts anderes alsMänner, denenWIRWahy zuteil werden ließen. So fragt die Leute von Adh-dhikr, wenn ihr selbst nicht wisst.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcdxci{Und WIR machten sie nicht zu Körpern, die keine Speisen aßen, und sie lebten nicht ewig.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcdxcii{Dann erfüllten WIR ihnen das Versprechen und erretteten sie und diejenigen, die WIR wollten, und richteten die Maßlosen zugrunde.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcdxciii{Gewiß, bereits sandten WIR euch eine Schrift hinab, mit der es für euch Würde gibt. Besinnt ihr euch etwa nicht?!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcdxciv{Und wie viele Ortschaften, (deren Bewohner) Unrecht begingen, WIR doch zerstörten, und nach ihnen andere Leute entstehen lassen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmcdxcv{Und als sie unser Gewalt-Antun wahrnahmen, da rannten sie davon weg.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcdxcvi{Rennt nicht weg und kehrt zurück zu dem, worin euch Luxus gewährt wurde, und zu euren Wohnstätten, damit ihr zur Rechenschaft gezogen werdet.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcdxcvii{Sie sagten: ``Unser Untergang! Wir waren doch Unrecht-Begehende."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmcdxcviii{Und es blieb dies ihr Ruf, bis WIR sie wie das Geerntete, regungslos machten.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmcdxcix{Und WIR erschufen den Himmel, die Erde und das, was zwischen ihnen ist, doch nicht aus sinnlosem Treiben.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmd{Und wollten WIR Uns Vergnüglichkeit nehmen, hätten WIR sie Uns von Uns aus genommen, wenn WIR dies tun würden.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdi{Nein, sondern werfen WIR mit der Wahrheit gegen das für nichtig Erklärte, dann annulliert sie es, da ist es dann vernichtet. Und für euch ist der Untergang bestimmt für das, was ihr behauptet.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdii{Und Ihm gehört, was in den Himmeln und auf Erden ist. Und diejenigen bei Ihm erheben sich nie über das Dienen Ihm gegenüber und sie werden dabei nie müde.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdiii{Sie lobpreisen (Ihn) die ganze Nacht und den ganzen Tag, sie lassen nicht nach.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdiv{Oder nahmen sie sich etwa Gottheiten aus der Erde, die lebendig machen?!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdv{Gäbe es in beiden (Himmel und Erde) Gottheiten außer ALLAH, gewiß wären beide verdorben. Also subhanallah, Der HERR von Al'ahrsch, über das, was sie erdichten.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdvi{ER wird nicht für das was ER tut zur Rechenschaft gezogen, jedoch sie werden zur Rechenschaft gezogen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdvii{Oder nahmen sie sich etwa anstelle von Ihm Gottheiten?! Sag: ``Bringt euren Beweis her! Dies ist die Ermahnung, über die ich verfüge, und die Ermahnung derer, die vor mir waren." Nein, sondern viele von ihnen kennen die Wahrheit nicht, so wenden sie sich ab.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdviii{Und WIR entsandten vor dir keinen Gesandten, ohne daß WIR ihm Wahy zuteil werden ließen: ``Gewiß, es gibt keine Gottheit außer Mir, so dient nur Mir!"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdix{Und sie sagten: ``Der Allgnade Erweisende hat Sich Kinder genommen." Gepriesen-erhaben ist ER! Nein, sondern (sie sind) nur ehrwürdige Diener.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdx{Sie kommen Ihm beim Sprechen nicht zuvor, und sie handeln gemäß Seiner Anweisung.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdxi{ER kennt das, was vor ihnen und was hinter ihnen ist. Und sie legen keine Fürbitte ein, außer für denjenigen, dem ER Wohlgefallen erwies. Und sie sind von der Ehrfurcht Ihm gegenüber zitternd.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdxii{Und wer von ihnen sagt: ``Ich bin ein Gott anstelle von Ihm." Dem vergelten WIR mit Dschahannam. Solcherart vergelten WIR den Unrecht-Begehenden.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdxiii{Haben diejenigen, die Kufr betrieben haben, etwa nicht wahrgenommen, daß die Himmel und die Erde gewiß ein Zusammengefügtes waren, dann rissen WIR sie auseinander, und daß WIR aus dem Wasser alles Lebendige machten?! Wollen sie etwa den Iman nicht verinnerlichen?!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdxiv{Und WIR setzten auf der Erde Felsengebirge, damit sie mit ihnen nicht schwankt, und WIR machten darin breite Wege, damit sie Rechtleitung finden.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdxv{Und WIR machten den Himmel als eine schützende Decke. Und sie wenden sich noch von Seinen Ayat ab.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdxvi{Und ER ist Derjenige, Der die Nacht, den Tag, die Sonne und den Mond erschuf. Alle schweben auf einer Flugbahn.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdxvii{Auch gewährten WIR keinem Menschen vor dir die Ewigkeit. Wenn du sterben wirst, werden sie etwa dann die Ewigen sein?!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdxviii{Jedes Lebewesen wird den Tod erfahren! Und WIR prüfen euch mit dem Bösen und mit dem Guten als Fitna. Und zu Uns werdet ihr zurückgebracht.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdxix{Und wenn dich diejenigen, die Kufr betrieben haben, sehen, machen sie dich zu nichts anderem als zum Spott: ``Ist dieser derjenige, der eure Gottheiten kritisiert?"Während sie selbst doch dem Gedenken an Den Allgnade Erweisenden gegenüber Kufr betreiben.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdxx{Der Mensch ist aus Eile erschaffen. ICH werde euch Meine Ayat zeigen, so fordert keine Eile von Mir.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdxxi{Und sie sagen: ``Wann wird dieses Versprechen erfüllt, solltet ihr wahrhaftig sein?"\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdxxii{Würden diejenigen, die Kufr betrieben haben, es doch nur wissen! (Es geschieht), wenn sie das Feuer weder von ihren vorderen Seiten noch von ihren hinteren Seiten zurückhalten können, noch ihnen beigestanden wird.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdxxiii{Nein, sondern sie (die Stunde) bricht über sie plötzlich herein, dann macht sie sie sprachlos verwundert, dann werden sie diese weder aufhalten können, noch wird ihnen eine Frist gewährt.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdxxiv{Und gewiß, bereits wurden vor dir Gesandte verspottet, dann umgab diejenigen, die sich über sie lustig machten, das, was sie zu verspotten pflegten.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdxxv{Sag: ``Wer kann euch in der Nacht und am Tage vor Dem Allgnade Erweisenden schützen?" Nein, sondern sie sind gegenüber dem Gedenken an ihren HERRN abwendend.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdxxvi{Oder haben sie etwa Gottheiten, die sie vor Uns verteidigen?! Sie können doch weder sich selbst beistehen, noch werden sie von Uns begleitet.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdxxvii{Nein, sondern WIR gewährten Verbrauchsgüter diesen und ihren Vorfahren, bis ihnen das Leben lang erschien. Nehmen sie etwa nicht wahr, daß WIR gewiß Uns der Erde zuwenden und sie von ihren Enden einengen?! Werden sie etwa die Sieger sein?!\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdxxviii{Sag: ``Ich warne euch ausschließlich mit dem Wahy." Doch die Tauben vernehmen keinen Ruf, wenn sie gewarnt werden.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdxxix{Und würde sie ein Hauch von der Peinigung deines HERRN treffen, würden sie bestimmt sagen: ``Unser Untergang! Gewiß, wir waren Unrecht-Begehende!"\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdxxx{Und WIR stellen die Waagen der Gerechtigkeit am Tag der Auferstehung auf. Dann wird keiner Seele etwas an Unrecht zugefügt. Und würde es (das Vollbrachte) das Gewicht eines Senfkorns haben, würden WIR es berücksichtigen. Und WIR genügen als Rechnende.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdxxxi{Und gewiß, bereits ließen WIR Musa und Harun zuteil werden Al-furqan, Erhellendes und eine Ermahnung für die Muttaqi,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdxxxii{die Ehrfurcht vor ihrem HERRN im Verborgenen haben, und die vor der Stunde zitternd sind.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdxxxiii{Und dies (der Quran) ist eine mit Baraka erfüllte Ermahnung, die WIR hinabsandten. Wollt ihr ihn etwa ableugnen?!\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdxxxiv{Und gewiß, bereits gewährten WIR Ibrahim seine Verständigkeit vorher und waren über ihn allwissend.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdxxxv{Als er seinem Vater und seinen Leuten sagte: ``Was sind diese Statuen, denen ihr Zuwendung erweist?"\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdxxxvi{Sie sagten: ``Wir fanden unsere Ahnen vor, sie dienten ihnen."\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdxxxvii{Er sagte: ``Gewiß, bereits wart ihr und eure Ahnen in einem eindeutigen Irregehen."\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdxxxviii{Sie sagten: ``Hast du uns die Wahrheit gebracht, oder bist du etwa einer der des Sinnlosen Treibenden?"\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdxxxix{Er sagte: ``Nein, sondern euer HERR ist Der HERR der Himmel und der Erde, Der sie erschuf. Und ich bin dafür einer der Zeugen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdxl{Und bei ALLAH! Ich werde eure Götzen auf jeden Fall überlisten, nachdem ihr abwendend zurückkehrt."\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdxli{Dann demolierte er sie zu Stücken außer einer Großen von ihnen, damit sie sich dann an sie wenden.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdxlii{Sie sagten: ``Wer tat dies unseren Göttern an? Gewiß, er ist zweifelsohne von den Unrecht-Begehenden."\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdxliii{Sie sagten: ``Wir hörten einen jungen Mann, der sie kritisierte, genannt wird er Ibrahim."\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdxliv{Sie sagten: ``Dann bringt ihn her vor die Augen der Menschen, vielleicht bezeugen sie es."\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdxlv{Sie sagten: ``Bist du derjenige, der dies unseren Göttern antat, Ibrahim?"\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdxlvi{Er sagte: ``Nein, sondern dies tat die Große (Statue) von ihnen, diese! Also fragt sie doch, sollten sie sprechen können!"\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdxlvii{Dann wandten sie sich zueinander, dann sagten sie: ``Gewiß, ihr seid selbst die Unrecht-Begehenden."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdxlviii{Dann wurden sie wieder auf den Kopf gestellt: ``Gewiß, bereits weißt du, daß diese nicht sprechen können."\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmdxlix{Er sagte: ``Dient ihr denn anstelle von ALLAH dem, das euch weder in irgend etwas nützt, noch schadet.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmdl{Pfui für euch und für das, dem ihr anstelle von ALLAH dient! Besinnt ihr euch nicht?"\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmdli{Sie sagten: ``Verbrennt ihn und unterstützt eure Götter, solltet ihr dies tun wollen."\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmdlii{WIR sagten: ``Feuer! Sei etwas Kühlendes und Salam für Ibrahim!"\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmdliii{Und sie wollten gegen ihn eine List planen, dann machten WIR sie zu den Verlierern.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmdliv{Und WIR erretteten ihn sowie Lut in das Land, das WIR mit Baraka für die ganze Schöpfung erfüllten.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmdlv{Und WIR schenkten ihm Ishaq und Ya'qub als zusätzliche Gabe. Und beide machten WIR zu gottgefällig Guttuenden.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmdlvi{Und WIR machten sie zu Imamen, die gemäß Unserer Anweisung (die Menschen) rechtleiten, und WIR ließen ihnen mit dem Wahy zuteil werden, das gottgefällig Gute zu tun, das rituelle Gebet ordnungsgemäß zu verrichten und die Zakat zu entrichten. Und sie waren Uns gegenüber Dienende.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmdlvii{Und Lut ließen WIR Weisheit und Wissen zuteil werden und erretteten ihn von der Ortschaft, (deren Einwohner) die Abscheulichkeiten zu tun pflegten. Gewiß, sie waren Leute der Schlechtigkeit, Fasiq.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmdlviii{Und WIR ließen ihn in Unsere Gnade eintreten. Gewiß, er war einer der gottgefällig Guttuenden.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmdlix{Ebenfalls Nuh, als er vorher rief, erhörten WIR ihn, dann erretteten WIR ihn und seine Familie von der gewaltigen Not.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmdlx{Und WIR schützten ihn vor den Leuten, die Unsere Ayat ableugneten. Gewiß, sie waren Leute der Schlechtigkeit, dann ertränkten WIR sie allesamt.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmdlxi{Ebenfalls Dawud und Sulaiman, als beide in der Angelegenheit des Ackerfeldes ein Urteil fällten, als darauf nachts die Schafe der Leute weideten. Und WIR waren gewiß ihrem Urteil gegenüber Zeugen.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmdlxii{Dann machten WIR sie (die Angelegenheit) Sulaiman klar. Und beiden ließen WIRWeisheit und Wissen zuteil werden. Und WIR machten mit Dawud die Felsenberge gratis fügbar, sie lobpreisen sowie die Vögel. Und WIR taten es wirklich.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmdlxiii{Und WIR brachten ihm die Herstellung der Panzerhemde für euch bei, damit sie euch Schutz vor eurer Gewalttätigkeit bieten. Seid ihr dann dankbar?!\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmdlxiv{Und für Sulaiman (machten WIR gratis fügbar) den Wind als Sturm, der gemäß seiner Anweisung zum Land weht, das WIR mit Baraka erfüllten. Und WIR waren über alles allwissend.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmdlxv{Und von den Satanen gab es welche, die für ihn tauchten und andere Arbeit als diese verrichten. Und WIR waren über sie Wachende.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmdlxvi{Ebenfalls Ayyub, als er seinem HERRN rief: ``Gewiß, mich traf das Schädigende und DU bist Derjenige, der mehr Gnade erweist als alle Gnade- Erweisenden."\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmdlxvii{Dann erhörten WIR ihn, so entfernten WIR von ihm, was ihn an Schädigendem traf. Und WIR gaben ihm seine Familie und ihresgleichen noch dazu als Gnade von Uns und als Ermahnung für die Dienenden.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmdlxviii{Ebenfalls Isma'il, Idris und Dhal-kifl - sie alle waren von den sich in Geduld Übenden.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmdlxix{Und WIR ließen sie in Unsere Gnade eintreten. Gewiß, sie waren von den gottgefällig Guttuenden.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmdlxx{Ebenfalls Dhan-nun, als er wütend fortging und dachte, daß WIR ihm dies nicht bestimmen werden, dann rief er in den Finsternissen: ``Es gibt keine Gottheit außer Dir, gepriesen-erhaben bist DU, ich gehörte gewiß zu den Unrecht- Begehenden."\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmdlxxi{Dann erhörten WIR ihn und erretteten ihn von der Not. Und solcherart erretten WIR die Mumin.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmdlxxii{Ebenfalls Zakaria, als er seinen HERRN rief: ``Mein HERR! Lasse mich nicht als Einzelnen und DU bist Der Beste der Nachbleibenden."\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmdlxxiii{Dann erhörten WIR ihn und schenkten ihm Yahya und machten ihm seine Frau fruchtbar. Gewiß, sie pflegten, zum Verrichten des gottgefällig Guten zu eilen und an Uns Bittgebete aus Liebe und aus Ehrfurcht zu richten. Und sie waren Uns gegenüber Demütige.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmdlxxiv{Ebenfalls diejenige, die ihre Keuschheit bewahrte, dann hauchten WIR in sie einen Ruhh von Uns hinein. Und WIR machten sie und ihren Sohn zu einer Aya für die ganze Schöpfung.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmdlxxv{Gewiß, diese ist eure Umma, eine einheitliche Umma! Und ICH bin euer HERR, so dient Mir!\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmdlxxvi{Und sie zerteilten ihre Angelegenheit untereinander. Sie alle werden jedoch zu Uns zurückkehren.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmdlxxvii{Wer dann von dem gottgefällig Guten tut, während er Mumin ist, für dessen Bestreben gibt es gewiß kein Kufran. Und WIR sind dafür gewiß Registrierende.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmdlxxviii{Und haram ist es für eine Ortschaft, die WIR zugrunde richteten, sie kehren nicht zurück.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmdlxxix{Wenn dann Yadschudsch und Madschudsch geöffnet wird, und sie von allen Anhöhen herunter eilen,\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmdlxxx{und das wahre Versprechen näherte sich, dann werden starr blicken die Augen derjenigen, die Kufr betrieben haben: ``Unser Untergang! wir waren demgegenüber achtlos. Nein, sondern wir waren Unrecht- Begehende!"\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmdlxxxi{Gewiß, ihr seid mit dem, was ihr anstelle von ALLAH dient, Brennstoff von Dschahannam. Ihr werdet es ansteuern.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmdlxxxii{Wären diese (wirklich) Gottheiten gewesen, hätten sie es nicht angesteuert. Und alle bleiben darin ewig.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmdlxxxiii{Darin stöhnen sie, und sie hören darin nicht.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmdlxxxiv{Gewiß, diejenigen, denen von Uns die schöne (Botschaft) im voraus ging, diese sind davon (vom Feuer) weit entfernt.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmdlxxxv{Sie hören sein Geräusch nicht. Und sie bleiben in dem, was sie sich wünschten, ewig.\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmdlxxxvi{Nicht betrübt sie der größte Schrecken, und die Engel empfangen sie: ``Dies ist euer Tag, der euch versprochen wurde."\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmdlxxxvii{Es ist an dem Tag, wenn WIR den Himmel falten wie das Falten des Registers mit dem Geschriebenen. Wie WIR die erste Schöpfung begannen, wiederholen wir sie. (Dies ist) ein Versprechen Uns gegenüber. WIR werden es gewiß tun!\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmdlxxxviii{Und gewiß, bereits schrieben WIR in Az-zabur nach der Ermahnung, daß die Erde doch Meine gottgefällig guttuenden Diener beerben.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmdlxxxix{Gewiß, in diesem ist doch eine Erklärung für dienende Leute.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmdxc{Und WIR entsandten dich nur als Gnade für die ganze Schöpfung.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmdxci{Sag: ``Es wird mir nur alsWahy zuteil, daß euer Gott nur ein einziger Gott ist. Werdet ihr dann nicht Muslime sein?"\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmdxcii{Und sollten sie sich abwenden, dann sag: ``Euch habe ich es in Gleichmäßigkeit mitgeteilt. Und ich weiß nicht, ob es nah oder fern, was euch angedroht wurde.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmdxciii{Gewiß, ER kennt das Verlautbarte vom Gesagten und ER kennt das, was ihr verschweigt.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmdxciv{Und ich weiß nicht, vielleicht ist es eine Fitna für euch, ein Vergnügen bis zu einer Frist."\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmdxcv{Sag: ``Mein HERR! Richte nach Gerechtigkeit! Und unser HERR, Der Allgnade Erweisende ist Der um Hilfe Gebetene gegen das, was ihr behauptet."\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmdxcvi{\basmalahde Ihr Menschen! Handelt Taqwa gemäß eurem HERRN gegenüber! Gewiß, das Beben der Stunde ist eine gewaltige Sache.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdxcvii{An dem Tag, wenn ihr seht, daß jede Stillende dem gegenüber, was sie stillte, zerstreut ist, und jede Schwangere das von ihr Getragene entbindet. Und du siehst die Menschen (wie) Trunkene, während sie keine Trunkenen sind. Doch die Peinigung von ALLAH ist hart.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdxcviii{Und unter den Menschen ist manch einer, der über ALLAH ohne Wissen disputiert und jedem äußerst widerspenstigen Satan folgt, dem\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdxcix{geschrieben wurde, daß wer ihn als Wali nimmt, den wird er doch verleiten und in die Peinigung der Gluthitze führen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdc{Ihr Menschen! Solltet ihr im Zweifel über die Auferweckung sein, so erschufen WIR euch doch aus Erdreich, dann aus Nutfa, dann aus 'Alaqa, dann aus proportionierter und unproportionierter Mudgha, damit WIR es euch erläutern. Und WIR lassen in den Mütterleibern ruhen, was WIR wollen, bis zu einer festgesetzten Frist. Dann lassen WIR euch als Kind herausbringen, dann (lassen WIR euch leben), damit ihr erwachsen werdet. Und von euch ist manch einer, der abberufen wird, und von euch ist manch einer, der in das faselnde Alter zurückgesetzt wird, damit er nichts mehr weiß, nachdem er über Wissen verfügte. Und du siehst die Erde leblos. Und wenn WIR über sie Wasser fallen lassen, regt sie sich, schwillt an und läßt von jeder erfreulichen Zweiheit hervorsprießen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdci{Dies (bezeugt), daß ALLAH gewiß Der Wahre ist, daß ER die Toten zweifelsohne lebendig macht und daß ER doch über alles allmächtig ist,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdcii{und daß die Stunde unweigerlich kommen wird, über die es keine Zweifel gibt, und daß ALLAH doch diejenigen erwecken wird, die in den Gräbern sind.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdciii{Und unter den Menschen ist manch einer, der über ALLAH disputiert ohne Wissen, ohne Rechtleitung und ohne erleuchtende Schrift -\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdciv{in Arroganz) mit dem Zur-Seite-drehen, damit er vom Weg ALLAHs ableitet. Für ihn ist im Diesseits Erniedrigung bestimmt, und WIR werden ihn am Tag der Auferstehung die Peinigung des Brandes erfahren lassen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdcv{Dies ist für das, was du eigenhändig erworben hast. Und ALLAH ist gewiß den Dienern gegenüber niemals ungerecht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdcvi{Und unter den Menschen ist manch einer, der mit dem Dienen ALLAHs auf der Kippe steht, wenn ihn Gutes trifft, findet er Ruhe dabei, und wenn ihn Fitna trifft, wendet er sich in die Gegenrichtung ab. ER verliert dann das Diesseits und auch das Jenseits. Dies ist gewiß der eindeutige Verlust.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdcvii{Er richtet Bittgebete an das, was ihm weder schadet noch nützt. Dies ist das weite Irregehen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdcviii{Er richtet Bittgebete an das, dessen Schaden näher liegt als dessen Nutzen. Doch erbärmlich ist der Maula und doch erbärmlich ist der Umgang-Pflegende.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdcix{Gewiß, ALLAH läßt diejenigen, die den Iman verinnerlicht und das gottgefällig Gute getan haben, in Dschannat eintreten, die von Flüssen durchflossen sind. Gewiß, ALLAH macht, was ER will.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdcx{Wer pflegte zu denken, daß ALLAH ihn (den Gesandten) im Diesseits und im Jenseits nicht unterstützen wird, so soll er ein Seil an der Decke anbringen, dann soll er sich erhängen, dann soll er sehen, ob seine List etwa das vergehen läßt, was er an Zorn hat.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdcxi{Und solcherart sandten WIR ihn (den Quran) hinab als klare Ayat, und ALLAH leitet recht, wen ER will.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdcxii{Gewiß, diejenigen, die den Iman bekundet haben, diejenigen, die Juden wurden, die Sabia, die Nazarener, die Madschus und diejenigen, die Schirk betrieben haben, ALLAH richtet zweifelsohne zwischen ihnen am Tag der Auferstehung. Gewiß, ALLAH ist über alles Zeuge.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdcxiii{Hast du etwa nicht wahrgenommen, daß ALLAH (Derjenige ist), für Ihn Sudschud vollziehen alle, die in den Himmeln und auf Erden sind, ebenfalls die Sonne, der Mond, die Sterne, die Felsenberge, die Bäume, die Tiere und viele von den Menschen?! Doch viele (andere) verdienen die Peinigung. Und wen ALLAH erniedrigt, für den findest du keinen Würdigenden. Gewiß, ALLAH macht, was ER will.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdcxiv{Diese sind zwei Gegner, die über ihren HERRN stritten. Für diejenigen, die Kufr betrieben haben, wurde dann Kleidung aus Feuer gefertigt. Gegossen wird über ihre Köpfe siedendes Wasser,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdcxv{damit wird das, was in ihren Bäuchen ist, geschmolzen, sowie die Häute.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdcxvi{Und für sie sind Hämmer aus Eisen bestimmt.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdcxvii{Immer wieder wenn sie aus ihr (Dschahannam) aus einer Drangsal herausgehen wollten, werden sie wieder zu ihr zurückgeschickt. Und erfahrt die Peinigung des Brandes!\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdcxviii{Gewiß, ALLAH wird diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, in Dschannat eintreten lassen, die von Flüssen durchflossen sind. Darin werden sie mit Armbändern aus Gold und Perlen geschmückt. Und ihre Kleidung ist darin aus Seide.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdcxix{Und sie wurden zum Guten des Gesagten rechtgeleitet. Und sie werden zum Weg Des Alllobenswürdigen rechtgeleitet.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdcxx{Gewiß, diejenigen, die Kufr betrieben und abhalten vom Wege ALLAHs und von Almasdschidil-haram, den WIR für die Menschen errichteten, gleich ob sie inländisch oder ausländisch sind, (diese verloren). Und wer darin etwas Abweichendes, Unrechtmäßiges anstrebt, den lassen WIR von qualvoller Peinigung erfahren.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdcxxi{Und (erinnere daran), als WIR Ibrahim den Ort des Hauses zuwiesen: ``Betreibe Mir gegenüber keinerlei Schirk und reinige das Meinige Haus für die Tawaf-, Qiyam-, und Ruku'-Sudschud-Vollziehenden."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdcxxii{Und rufe die Menschen zur Hadsch auf, so werden sie zu dir zu Fuß und (als Getragene) auf jedem schlanken (Transporttier) kommen, sie werden aus jeder fernen Gegend kommen,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdcxxiii{damit sie für sich Nützlichem beiwohnen und ALLAHs Namens an bestimmten Tagen gedenken für das, was ER ihnen an Rizq von den unverständigen An'am gewährte. So esst davon und speist den armen Notleidenden.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdcxxiv{Dann sollen sie ihren Schmutz entfernen, ihre Gelübde erfüllen und öfters Tawaf um das Ehrwürdige Haus vollziehen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdcxxv{Also und wer ALLAHs Hurumat hochachtet, dies ist gut für ihn bei seinem HERRN. Und für halal wurden euch die An'am erklärt außer dem, was euch vorgetragen wurde. So meidet das rituell Unreine von den Götzen, und meidet das Falschzeugnis!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdcxxvi{Seid) hanif für ALLAH, ohne an Ihn Schirk zu betreiben. Und wer Schirk an ALLAH betreibt, sei so, als ob er vom Himmel niederfiel, dann entweder reißen ihn die Vögel weg oder der Wind läßt ihn in einen tiefen Ort niederstürzen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdcxxvii{Also und wer ALLAHs Riten (über die Opfertiere) hochachtet, dies ist gewiß von der Taqwa der Herzen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdcxxviii{Damit habt ihr Nützlichkeiten bis zu einer festgesetzten Frist, dann ist ihr Opferort bei dem Ehrwürdigen Haus.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdcxxix{Und für jede Umma machten WIR einen Opferritus, damit sie den Namen ALLAHs auf das Rizq erwähnen, das ER ihnen von Bahimatul-an'am gewährte. Denn euer Gott ist ein einziger Gott, so praktiziert Ihm gegenüber den Islam! Und überbringe frohe Botschaft den Sich-Fügenden!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdcxxx{Diese sind) diejenigen, wenn ALLAH erwähnt wird, deren Herzen Ehrfurcht empfinden, die sich in Geduld Übenden über das, welches sie traf, die das rituelle Gebet ordnungsgemäß Verrichtenden und diejenigen, die vom Rizq geben, das WIR ihnen gewährten.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdcxxxi{Und die Budn machten WIR für euch von ALLAHs Riten. Für euch gibt es in ihnen Gutes. So erwähnt den Namen ALLAHs auf ihnen als sawaf. Und wenn sie auf ihre Seite fallen, dann esset davon und speist den Genügsamen und den Bittenden. Solcherart machten WIR sie für euch gratis fügbar, damit ihr euch dankbar erweist.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdcxxxii{ALLAH erhält weder ihr Fleisch, noch ihr Blut, sondern zu Ihm gelangt die Taqwa von euch. Solcherart machte ER sie für euch gratis fügbar, damit ihr ALLAH mit ``allahu-akbar" verherrlicht für das, was ER euch an Rechtleitung gewährte. Und überbringe den Muhsin frohe Botschaft.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdcxxxiii{Gewiß, ALLAH verteidigt diejenigen, die den Iman verinnerlicht haben. Gewiß, ALLAH liebt nicht alle äußerst kufr-betreibenden Verräter!\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdcxxxiv{Erlaubnis (zum bewaffneten Kampf) wurde denjenigen, die bekämpft werden, erteilt, weil ihnen Unrecht zugefügt wurde. Und gewiß, ALLAH ist doch allmächtig, ihnen zum Sieg zu verhelfen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdcxxxv{Es sind) diejenigen, die von ihren Häusern vertrieben wurden zu Unrecht, nur weil sie sagen: ``Unser HERR ist ALLAH!" Und würde ALLAH die Menschen, die einen durch die anderen, nicht zurückhalten, würden zweifelsohne Einsiedeleien, Synagogen, Kirchen und Moscheen zerstört werden, in denen des Namens ALLAHs häufig gedacht wurde. Und ALLAH wird doch demjenigen zum Sieg verhelfen, der Seiner zum Sieg verhilft. Gewiß, ALLAH ist doch allkraftvoll, allwürdig!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdcxxxvi{Es sind) diejenigen, wenn WIR sie im Lande festigen, dann verrichten sie ordnungsgemäß das rituelle Gebet, entrichten die Zakat, rufen zum Gebilligten auf und raten vomMißbilligten ab. Und ALLAH gehört das Anschließende von den Angelegenheiten.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdcxxxvii{Und sollten sie dich der Lüge bezichtigen, so leugneten vor ihnen die Leute von Nuh, 'Aad und Thamud ab.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdcxxxviii{Sowie die Leute von Ibrahim und die Leute von Lut.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdcxxxix{Sowie die Bewohner von Madyan. Auch Musa wurde der Lüge bezichtigt, so gewährte ICH den Kafir noch längere Zeit, dann richtete ICH sie zugrunde. Also wie war denn Meine Mißbilligung?!\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdcxl{Wie manche Ortschaft richteten WIR doch zugrunde, während sie (ihre Bewohner) Unrecht begingen. So wurden ihre Häuser dem Erdboden gleichgemacht, auch der Brunnen ist verlassen und das Schloß ist errichtet.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdcxli{Sind sie etwa nicht im Lande umhergereist, damit sie über Herzen verfügen, mit denen sie verständig werden, oder über Ohren, mit denen sie hören?! Denn gewiß, nicht die Augen werden blind, sondern blind werden die Herzen, die in den Brüsten sind.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdcxlii{Und sie fordern von dir Eile mit der Peinigung, und ALLAH wird doch nie Sein Versprechen brechen. Und zweifellos ist ein Yaum bei deinem HERRN wie tausend Jahre von dem, was ihr zählt.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdcxliii{Und wie manch eine Ortschaft, der ICH noch längere Zeit gewährte, während (ihre Bewohner) unrecht-begehend waren, dann beseitigte ICH sie, und zu Mir ist dasWerden.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdcxliv{Sag: ``Ihr Menschen! Ich bin für euch nur ein erläuternder Warner."\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdcxlv{Für diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, sind Vergebung und edles Rizq bestimmt.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdcxlvi{Doch diejenigen, die gegen Unsere Ayat in Widerspenstigkeit vorgehen, diese sind die Weggenossen der Hölle.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdcxlvii{Und WIR entsandten vor dir keinen Gesandten und keinen Propheten, ohne daß, wenn er vortrug, der Satan zu seinem Vortrag etwas hinzufügte. So eliminiert ALLAH das, was der Satan hinzufügte, dann konsolidiert ALLAH Seine Ayat. ALLAH ist allwissend, allweise.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdcxlviii{Damit ER das, was der Satan hinzufügt, als Fitna macht für diejenigen, in deren Herzen Krankheit ist, und die des harten Herzens sind. Und gewiß, die Unrecht-Begehenden befinden sich doch in einer unerbittlichen Feindseligkeit.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdcxlix{Und damit diejenigen, denen das Wissen zuteil wurde, wissen, daß er (der Quran) doch die Wahrheit von deinem HERRN ist, um den Iman an ihn zu verinnerlichen und ihre Herzen sich ihm gegenüber zu ergeben. Und gewiß, ALLAH ist zweifellos Der Rechtleitende derjenigen, die den Iman verinnerlichten, zu einem geradlinigen Weg.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdcl{Und diejenigen, die Kufr betrieben haben, bleiben ihm gegenüber in Zweifel, bis die Stunde plötzlich über sie hineinbricht oder die Peinigung eines unfruchtbaren Tages zu ihnen kommt.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdcli{Die Herrschaft an diesem Tag gehört ALLAH, ER richtet zwischen ihnen. So sind diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, in den Dschannat des Wohlergehens.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdclii{Und diejenigen, die Kufr betrieben und Unsere Ayat ableugnet haben, für diese ist eine erniedrigende Peinigung bestimmt.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdcliii{Und diejenigen, die fi-sabilillah Hidschra unternahmen, dann getötet wurden oder starben, ALLAH wird ihnen zweifelsohne ein schönes Rizq gewähren. Und gewiß, ALLAH ist doch Der Beste der Rizq-Gewährenden.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdcliv{ER wird sie doch in einen Eingang eingehen lassen, an dem sie Wohlgefallen finden werden. Und gewiß, ALLAH ist zweifelsohne allwissend, allnachsichtig.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdclv{Also und wer mit Gleichem straft, was ihm zugefügt wurde, dann gegen ihn Übertretung begangen wurde, ALLAH wird ihm doch beistehen. Gewiß, ALLAH ist bestimmt reue-annehmend, allvergebend.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdclvi{Dies ist so, da ALLAH die Nacht in den Tag einfließen läßt und den Tag in die Nacht einfließen läßt, und da ALLAH doch allhörend, allsehend ist.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdclvii{Dies ist so, da ALLAH Das Wahre ist, während das, woran sie anstelle von Ihm Bittgebete richten, doch das für nichtig Erklärte ist, und da ALLAH doch Der Allhohe, Der Allgroße ist.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdclviii{Hast du etwa nicht gesehen, daß ALLAH vom Himmel Wasser fallen ließ, dann wird die Erde grün?! Gewiß, ALLAH ist allgütig, allkundig.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdclix{Ihm gehört, was in den Himmeln und was auf Erden ist. Und gewiß, ALLAH ist doch Der absolut Autarke, Der Alllobenswürdige.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdclx{Hast du etwa nicht gesehen, daß ALLAH für euch das, was auf Erden ist, gratis fügbar machte, und daß die Schiffe auf dem Meer gemäß Seiner Anweisung fahren. Und er läßt den Himmel halten, damit er nicht auf die Erde herunterfällt, es sei denn nach Seiner Zustimmung. Gewiß, ALLAH ist den Menschen gegenüber doch allgütig, allgnädig.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmdclxi{Und ER ist Derjenige, Der euch lebendig machte, dann euch sterben läßt, dann lebendig macht. Gewiß, der Mensch ist doch äußerst kufr-betreibend.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmdclxii{Für jede Umma machten WIR einen Ritus, den sie durchführen. So laß sie mit dir nicht in der Angelegenheit streiten und mache Da'wa zu deinem HERRN. Gewiß, du folgst doch einer geraden Rechtleitung.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmdclxiii{Und sollten sie mit dir disputieren, dann sag: ``ALLAH weiß besser Bescheid über das, was ihr tut.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmdclxiv{ALLAH richtet zwischen euch am Tag der Auferstehung in dem, worüber ihr uneins zu sein pflegtet."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmdclxv{Weißt du etwa nicht, daß ALLAH alles kennt, was im Himmel und auf Erden ist?! Gewiß, dies ist in einem Register. Gewiß, dies ist für ALLAH leicht!\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmdclxvi{Und sie dienen anstelle von ALLAH dem, wozu ER niemals eine Bestätigung hinabsandte, und das, worüber sie über kein Wissen verfügen. Und für die Unrecht- Begehenden gibt es keinen Beistehenden.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmdclxvii{Und wenn ihnen Unsere Ayat in Klarheit vorgetragen werden, erkennst du die Mißbilligung in den Gesichtern derjenigen, die Kufr betrieben haben. Beinahe würden sie herfallen über diejenigen, die ihnen Unsere Ayat vortragen. Sag: ``Soll ich euch über Schlimmeres als dieses Mitteilung machen? Das Feuer versprach ALLAH denjenigen, die Kufr betrieben haben." Und erbärmlich ist das Werden.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmdclxviii{Ihr Menschen! Ein Gleichnis wurde geprägt, so hört dem zu! Gewiß, diejenigen, an die ihr Bittgebete anstelle von ALLAH richtet, werden niemals Fliegen erschaffen, selbst dann nicht, würden sie sich dafür versammeln. Und wenn die Fliegen ihnen etwas entwenden, können sie es von ihnen nicht mehr zurückholen. Schwach ist der Bittsteller und das Gebetene.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmdclxix{Doch sie achteten ALLAH nicht entsprechend der Ihm gebührenden Achtung. Gewiß, ALLAH ist doch allkraftvoll, allwürdig.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmdclxx{ALLAH erwählt aus den Engeln Gesandte und aus den Menschen. Gewiß, ALLAH ist allhörend, allsehend.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmdclxxi{ER weiß, was vor ihnen und was hinter ihnen ist. Und an ALLAH werden alle Angelegenheiten gewandt.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmdclxxii{Ihr, die den Iman verinnerlicht habt! Vollzieht Ruku' und Sudschud und dient eurem HERRN und tut gottgefällig Gutes, damit ihr erfolgreich werdet.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmdclxxiii{Und leistet Dschihad für ALLAH, wie Ihm die Dschihad-Leistung gebührt! ER erwählte euch und erlegte euch keine Unannehmlichkeit im Din auf. Es ist die Gemeinschaft eures Vaters Ibrahim. ER nannte euch Muslime vorher und in diesem (Quran), damit der Gesandte Zeuge über euch wird und ihr Zeugen über die Menschen werdet. So verrichtet ordnungsgemäß das rituelle Gebet, entrichtet die Zakat und haltet fest an ALLAH, ER ist euer Maula, Der besteMaula und Der beste Beistehende!\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmdclxxiv{\basmalahde Bereits sind die Mumin erfolgreich!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdclxxv{Diejenigen, die in ihrem rituellen Gebet voller Ehrfurcht sind.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdclxxvi{Und diejenigen, die dem unnützen Gerede abgeneigt sind.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdclxxvii{Und diejenigen, welche die Zakat entrichten.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdclxxviii{Und diejenigen, die ihre Intimbereiche behüten,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdclxxix{außer ihren Ehefrauen und denen gegenüber, die ihnen gehören. Denn dann sind sie gewiß nicht zu tadeln.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdclxxx{Wer dann anderes als dies erstrebt, diese sind die wirklichen Übertretenden.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdclxxxi{Und diejenigen, welche das ihnen Anvertraute und ihre Abmachungen wahren.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdclxxxii{Und diejenigen, die ihre rituellen Gebete beachten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdclxxxiii{Diese sind die Erben,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdclxxxiv{die Al-firdaus erben. Darin bleiben sie ewig.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdclxxxv{Und gewiß, bereits erschufen WIR den Menschen aus einem Auszug aus Lehm.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdclxxxvi{Dann machten WIR ihn als Nutfa in einem gefestigten Aufenthaltsort.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdclxxxvii{Dann ließen WIR die Nutfa zu einer 'Alaqa werden. So ließen WIR die 'Alaqa zu einer Mudgha werden, und WIR ließen die Mudgha zu Knochen werden, und die Knochen bedeckten WIR mit Fleisch, dann ließen WIR ihn als eine andere Schöpfung entstehen. Also immer allerhabener ist ALLAH, Der Beste aller Schöpfer.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdclxxxviii{Dann gewiß ihr werdet danach zweifelsohne sterben.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdclxxxix{Dann gewiß ihr werdet am Tag der Auferstehung erweckt werden.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdcxc{Und gewiß, bereits erschufen WIR über euch sieben übereinander Lagernde. Und WIR waren nie der Schöpfung gegenüber achtlos.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdcxci{Und WIR ließen vom Himmel Wasser in Abgewogenheit fallen, dann ließen WIR es in der Erde unterbringen. Und WIR sind, um es vergehen zu lassen, doch allmächtig.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdcxcii{Dann ließen WIR für euch mit ihm Dschannat von Dattelpalmen und Rebstöcken entstehen, darin habt ihr vieles an Obst und davon esst ihr.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdcxciii{Und (WIR ließen damit entstehen) einen Baum, der am Tur-Berg von Sainaa heraussprießt, er bringt das Öl und Soße für die Essenden hervor.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdcxciv{Und gewiß, für euch gibt es in den An'am zweifelsohne eine Lehre. WIR geben euch zu trinken von dem, was in ihren Bäuchen ist. Und für euch gibt es in ihnen noch andere Nützlichkeiten, und von ihnen esst ihr.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdcxcv{Und auf ihnen und auf den Schiffen werdet ihr getragen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdcxcvi{Und gewiß, bereits entsandten WIR Nuh zu seinen Leuten, er sagte: ``Meine Leute! Dient ALLAH! Denn für euch gibt es keinen anderen als Gott. Wollt ihr denn nicht Taqwa gemäß handeln?"\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdcxcvii{Dann sagten die Entscheidungsträger von seinen Leuten, die Kufr betrieben: ``Dieser ist nichts anderes als ein Mensch genau wie ihr, der sich nur über euch erheben will. Und sollte ALLAH wollen, hätte ER gewiß Engel herabgesandt. WIR hörten nichts darüber in den (Überlieferungen) unserer Vorfahren.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdcxcviii{Er ist nur ein Mann, der geistesgestört ist, also wartet mit ihm bis zu einer Frist ab."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdcxcix{Er sagte: ``Mein HERR! Stehe mir bei wegen dem, wie sie mich der Lüge bezichtigten."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdcc{Sogleich ließen WIR ihm Wahy zuteil werden: ``Baue das Schiff unter Unserer Aufsicht und nach Unserem Wahy! Und wenn dann Unsere Bestimmung eintrifft und die Erdoberfläche sprudelt, nimm darauf mit von allen (Dingen) genau Zweiheiten und deine Familie außer denjenigen von ihnen, über sie die Bestimmung (zum Ertrinken) bereits ergangen ist. Und sprich Mich nicht an wegen denjenigen, die Unrecht begingen, denn sie werden gewiß ertrinken."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdcci{Sogleich du mit denjenigen mit dir auf dem Schiff stehst, sag: ``Alhamdulillah: Alles Lob gebührt ALLAH, Der uns von den unrechtbegehenden Leuten errettete.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdccii{Und sag: ``Mein HERR! Bringe mich in einer Unterbringung voller Baraka unter. Und DU bist Der Beste der Unterbringenden."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdcciii{Gewiß, darin gibt sind zweifelsohne Ayat, und WIR waren doch Prüfende!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdcciv{Dann ließen WIR nach ihnen eine andere Generation entstehen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdccv{Sogleich entsandten WIR zu ihnen einen Gesandten von ihnen: ``Dient ALLAH! Denn für euch gibt es keinen anderen als Gott. Wollt ihr denn nicht Taqwa gemäß handeln?"\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdccvi{Dann sagten die Entscheidungsträger von seinen Leuten, die Kufr betrieben und das Treffen im Jenseits ableugnet haben, und denen WIR Luxus im diesseitigen Leben gewährten: ``Dieser ist nichts anderes als ein Mensch genau wie ihr, er ißt von dem, was ihr esst, und trinkt von dem, was ihr trinkt!\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdccvii{Und wenn ihr einem Menschen wie ihr (selbst) gehorcht, gewiß seid ihr dann doch Verlierer!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdccviii{Verspricht er euch etwa, daß wenn ihr sterbt und zu Erde und Knochen werdet, daß ihr doch dann hervorgebracht werdet?!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdccix{Unglaubwürdig, ziemlich unglaubwürdig ist das, was euch versprochen wird.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdccx{Es gibt nur unser diesseitiges Leben, wir sterben und leben und wir werden nie erweckt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdccxi{Er ist nur ein Mann, der im Namen ALLAHs Lügen erfindet, und wir werden an ihn den Iman nicht verinnerlichen."\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdccxii{Er sagte: ``Mein HERR! Stehe mir bei wegen dem, wie sie mich der Lüge bezichtigten."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdccxiii{ER sagte: ``In Kürze werden sie es doch noch bereuen."\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdccxiv{Dann richtete der Schrei sie in gerechter Weise zugrunde. Dann machten WIR sie als Überrest. Also nieder mit den unrecht-begehenden Leuten!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdccxv{Dann ließen WIR nach ihnen andere Generationen entstehen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdccxvi{Weder verlegt eine Umma ihre Frist vor, noch zögern sie diese hinaus.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdccxvii{Dann entsandten WIR Unsere Gesandten einen nach dem anderen. Immer wieder, wenn zu einer Umma ihr Gesandter kam, bezichtigten sie ihn der Lüge. Sogleich ließen WIR die einen den anderen (in der Vernichtung) folgen, und machten sie zum Objekt von Erzählungen. Also nieder mit Leuten, die den Iman nicht verinnerlichen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdccxviii{Dann entsandten WIR Musa und seinen Bruder Harun mit Unseren Ayat und einem eindeutigen Beweis\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdccxix{zu Pharao und seinen Entscheidungsträgern. Dann erhoben sie sich in Arroganz, und sie waren hochmütige Leute.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdccxx{Sie sagten dann: ``Sollen wir etwa den Iman an zwei Menschen wie wir verinnerlichen, deren Leute für uns Diener sind?!"\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdccxxi{Sie bezichtigten dann sie der Lüge, dann waren sie von den Zugrunde-Gerichteten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdccxxii{Und gewiß, bereits ließen WIR Musa die Schrift zuteil werden, damit sie Rechtleitung finden.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdccxxiii{Und WIR machten Ibnu- Maryam und seine Mutter zu einer Aya und gewährten ihnen Unterkunft bei einem Hügel mit Aufenthaltsmöglichkeit und fließendem Wasser.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdccxxiv{Ihr Gesandte! Esst von den Tay-yibat und tut gottgefällig Gutes! ICH bin gegenüber dem, was ihr tut, allwissend.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdccxxv{Und gewiß diese eure Umma, ist eine einheitliche Umma, und ICH bin euer HERR, so handelt Taqwa gemäß Mir gegenüber!\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdccxxvi{Dann zerteilten sie sich ihre Angelegenheit untereinander in Gruppen. Jede Partei findet Gefallen an dem, worüber sie verfügt.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdccxxvii{So laß sie in ihrer Achtlosigkeit bis zu einer Frist.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdccxxviii{Denken sie etwa, daß das, womit WIR ihnen Nachschub gewähren an Vermögen und Kindern,\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdccxxix{daß WIR ihnen damit die guten Dinge schnell zukommen lassen?! Nein, sondern sie merken es nicht.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdccxxx{Gewiß, diejenigen, die vor Ehrfurcht ihrem HERRN gegenüber ergeben sind,\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdccxxxi{und diejenigen, die den Iman an die Ayat ihres HERRN verinnerlichen,\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdccxxxii{und diejenigen, die ihrem HERRN gegenüber keinen Schirk betreiben,\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdccxxxiii{und diejenigen, die das geben, was sie gaben, während ihre Herzen ehrfurcht-erfüllt sind, da sie zu ihrem HERRN zurückkehren werden,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdccxxxiv{diese eilen initiativ zum (gottgefällig) Guten, und sind daran die ersten voran.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdccxxxv{Und WIR erlegen keinem Menschen auf außer dem, was er vermag. Und WIR haben ein Register, das die Wahrheit artikuliert. Und ihnen wird kein Unrecht zugefügt.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdccxxxvi{Nein, sondern ihre Herzen sind in Achtlosigkeit demgegenüber. Und sie haben andere Taten als diese, die sie immer wieder tun.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdccxxxvii{Wenn WIR dann ihre Luxurierenden mit der Peinigung zugrunde richten, dann schreien sie um Hilfe.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdccxxxviii{Schreit heute nicht um Hilfe! Gewiß, ihr werdet von Uns nicht unterstützt.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmdccxxxix{Bereits wurden euch Meine Ayat stets vorgetragen, dann pflegtet ihr auf euren Fersen (zum Kufr) umzukehren\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmdccxl{in äußerster Arroganz ihm gegenüber, und ihr habt nachts dagegen gelästert.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmdccxli{Haben sie etwa über das Vorgetragene nicht nachgedacht?! Oder kam etwa zu ihnen das, was zu ihren ersten Ahnen nicht kam?!\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmdccxlii{Oder kannten sie etwa ihren Gesandten nicht, so daß sie ihm gegenüber ableugnend waren?!\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmdccxliii{Oder sagen sie etwa: ``Er ist geistesgestört"?! Nein, sondern er kam zu ihnen mit der Wahrheit! Und die meisten von ihnen sind der Wahrheit gegenüber abgeneigt.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmdccxliv{Und würde die Wahrheit ihren Neigungen folgen, gewiß würden die Himmel und die Erde und alle, die in ihnen sind, ins Verderben fallen. Nein, sondern WIR ließen ihnen das von ihnen Erwähnte zuteil werden, dann waren sie dem von ihnen Erwähnten gegenüber abwendend.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmdccxlv{Oder bittest du sie etwa um Löhnung?! So ist die Löhnung deines HERRN besser. Und ER ist Der Beste der Rizq-Gewährenden.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmdccxlvi{Und gewiß, du machst für sie doch Da'wa zu einem geradlinigen Weg.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmdccxlvii{Doch diejenigen, die keinen Iman an das Jenseits verinnerlichen, sind vom Weg zweifelsohne abweichend.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmdccxlviii{Und würden WIR ihnen Gnade erweisen und ihnen das wegnehmen, was sie an Schädigendem traf, gewiß würden sie noch mehr in ihrer Maßlosigkeit verharren und abirren.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmdccxlix{Und gewiß, bereits richteten WIR sie mit der Peinigung zugrunde, dann weder gaben sie sich ihrem HERRN hin, noch baten sie unterwürfig um Hilfe.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmdccl{Als WIR dann für sie eine Tür mit harter Peinigung öffneten, da waren sie darüber verzweifelt.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmdccli{Und ER ist Derjenige, Der für euch das Hören, das Sehen und den Verstand entstehen ließ. Doch ihr seid selten dankbar.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmdcclii{Und ER ist Derjenige, Der euch auf der Erde vermehren ließ, und vor Ihm werdet ihr versammelt.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmdccliii{Und ER ist Derjenige, Der lebendig macht und sterben läßt. Und Ihm unterliegt die Aufeinanderfolge von Nacht und Tag. Besinnt ihr euch etwa nicht?!\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmdccliv{Nein, sondern sie sagten das Gleiche, was die Früheren sagten.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmdcclv{Sie sagten: ``Wenn wir starben und zu Erde und Knochen wurden, würden wir etwa doch erweckt werden?!\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmdcclvi{Gewiß, bereits wurde uns und unseren Vorfahren dieses vorher angedroht. Dies sind nichts anderes als die Legenden der Vorfahren."\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmdcclvii{Sag: ``Wem gehört die Erde und das, was auf ihr ist, solltet ihr wissen?"\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmdcclviii{Sie werden sagen: ``ALLAH!" Sag: ``Entsinnt ihr euch nicht?!"\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmdcclix{Sag: ``Wer ist Der HERR der sieben Himmeln und Der HERR vom gewaltigen Al'ahrsch?"\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmdcclx{Sie werden sagen: ``ALLAH!" Sag: ``Handelt ihr etwa nicht Taqwa gemäß?!"\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmdcclxi{Sag: ``Wem unterliegt die Herrschaft des Allerlei, und wer gewährt Schutz, während ihm kein Schutz gewährt wird, solltet ihr wissen?!"\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmdcclxii{Sie werden sagen: ``ALLAH!" Sag: ``Wie werdet ihr denn verblendet?!"\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmdcclxiii{Nein, sondern WIR ließen ihnen die Wahrheit zuteil werden! Doch sie sind zweifellos Lügner.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmdcclxiv{ALLAH nahm sich nie ein Kind, und mit Ihm war nie eine (andere) Gottheit, sonst nähme sich doch jede Gottheit, was sie erschuf, und es würden sich die einen von ihnen über die anderen erheben. Subhanallah über das, was sie erdichten.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmdcclxv{ER ist Der Allwissende über das Verborgene und das Sichtbare. Also erhaben ist ER über das, was sie an Schirk betreiben.\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmdcclxvi{Sag: ``Mein HERR! Wenn DU mich doch noch erleben läßt, was ihnen angedroht wurde -\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmdcclxvii{mein HERR!  lasse mich nicht unter den unrecht-begehenden Leuten sein!"\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmdcclxviii{Und gewiß, WIR sind doch allmächtig, dich noch erleben zu lassen, was WIR ihnen androhen.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmdcclxix{Wehre mit dem ab, was besser ist, die Verfehlung! WIR wissen besser Bescheid über das, was sie erdichten.\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmdcclxx{Und sag: ``Mein HERR! Ich suche bei Dir Schutz vor den Einflüsterungen der Satane.\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmdcclxxi{Und ich suche bei Dir Schutz, HERR! - daß sie anwesend werden."\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmdcclxxii{Wenn dann der Tod einen von ihnen ereilt, sagt er: ``Mein HERR! Schicke mich zurück,\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmdcclxxiii{Damit ich gottgefällig Gutes tue von dem, was ich vernachlässigt habe." Nein! Gewiß, dies ist nur eine Floskel, die er sagen wird. Und vor ihnen ist ein Barsakh bis zu dem Tag, an dem sie erweckt werden.\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmdcclxxiv{Und wenn in As-sur geblasen wird, dann gelten an diesem Tag keine Abstammungen mehr zwischen ihnen und sie fragen einander nicht.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmdcclxxv{Also jeder, dessen Abgewogenes schwer wiegt, so sind diese die wirklichen Erfolgreichen!\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmdcclxxvi{Und jeder, dessen Abgewogenes leicht wiegt, so sind diese diejenigen, die sich selbst verloren haben, in Dschahannam bleiben sie ewig.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmdcclxxvii{Das Feuer wird ihre Gesichter grillen. Und sie sind darin fletschend.\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmdcclxxviii{Wurden euch Meine Ayat etwa nicht stets vorgetragen, und ihr pflegtet sie als Lüge abzutun?!\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmdcclxxix{Sie sagten: ``Unser HERR! Unser Unglück hat uns überwältigt, und wir waren zweifelsohne abirrende Leute.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmdcclxxx{Unser HERR! Laß uns aus ihr (Dschahannam) herausbringen, und wenn wir es wiederholen, dann sind wir wirklich Unrecht-Begehende."\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmdcclxxxi{ER sagte: ``Bleibt darin erniedrigt und sprecht Mich nicht an!\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmdcclxxxii{Gewiß, es war eine Gruppe von Meinen Dienern, die sagten: ``Unser HERR! Wir verinnerlichten den Iman, so vergib uns und erweise uns Gnade, denn DU bist Der Allgnädigste der Gnädigen."\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmdcclxxxiii{So habt ihr sie zum Spott gemacht, bis sie euch das Gedenken an Mich vergessen ließen, und ihr pflegtet, euch über sie lächerlich zu machen.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmdcclxxxiv{Gewiß, ICH vergalt ihnen heute das, was sie an Geduld aufbrachten: Sie sind die Erfolgreichen!\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmdcclxxxv{ER sagte: ``Wie lange habt ihr auf der Erde an Anzahl von Jahren verweilt?!"\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmdcclxxxvi{Sie sagten: ``Wir verweilten einen Tag oder einen Teil eines Tages, so frage doch die Rechnenden!"\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmdcclxxxvii{Sag: ``Ihr habt nur ein wenig verweilt. Würdet ihr doch nur zu wissen gepflegt haben!\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmdcclxxxviii{Habt ihr etwa gedacht, daß WIR euch aus sinnlosem Treiben erschufen, und daß ihr zu Uns nicht zurückgebracht würdet?!\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmdcclxxxix{Also erhaben ist ALLAH, Der Wahre Herrscher! Es gibt keine Gottheit außer Ihm, Der HERR vom ehrwürdigen Al'ahrsch.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmdccxc{Und wer mit ALLAH einen anderen Gott anruft, für den er keinen Nachweis hat, dessen Abrechnung ist nur bei seinem HERRN. Gewiß, die Kafir werden nie erfolgreich sein.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmdccxci{Und sag: ``Mein HERR! Vergib und erweise Gnade! Und DU bist Der Beste der Gnade-Erweisenden."\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmdccxcii{\basmalahde Diese ist eine Sura, dieWIR hinabsandten und geboten, und darin sandten WIR deutliche Ayat hinab, damit ihr euch besinnt.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdccxciii{Hinsichtlich) der Zina-Treibenden und des Zina-Treibenden, so peitscht jeden von ihnen mit hundert Peitschenschlägen. Und lasst kein Erbarmen beiden gegenüber (bei der Erfüllung) von ALLAHs Din euch ergreifen, solltet ihr den Iman an ALLAH und an den Jüngsten Tag verinnerlichen. Und eine Gruppe von den Mumin soll ihrer Peinigung beiwohnen!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdccxciv{Der Zina-Treibende heiratet nur eine Zina-Treibende oder Muschrika. Und die Zina-Treibende heiratet nur einen Zina-Treibenden oder einen Muschrik. Und dies wurde für die Mumin für haram erklärt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdccxcv{Und diejenigen, die den keuschen Frauen (Zina) vorwerfen dann (dafür) keine vier (Augen-)Zeugen bringen, diese sollt ihr mit achtzig Peitschenschlägen peitschen und ihr Zeugnis niemals gelten lassen. Und diese sind die wirklichen Fasiq,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdccxcvi{außer denjenigen, die danach bereuten und gottgefällig Gutes taten, dann ist ALLAH gewiß allvergebend, allgnädig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdccxcvii{Und diejenigen, die ihren Ehefrauen (Zina) vorwerfen und dafür keine Zeugen außer sich selbst haben, dann ist das Zeugnis eines von ihnen vier Bezeugungen bei ALLAH: ``Gewiß, er ist zweifelsohne von den Wahrhaftigen."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdccxcviii{Und die fünfte (Bezeugung): ``ALLAHs Fluch laste doch auf ihm, sollte er von den Lügnern gewesen sein."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdccxcix{Und die Peinigung hält von ihr fern, daß sie bezeugt vier Bezeugungen bei ALLAH: ``Gewiß, er ist zweifelsohne von den Lügnern."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdccc{Und die fünfte (Bezeugung): ``ALLAH zürne ihr doch, sollte er von den Wahrhaftigen gewesen sein."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdccci{Und wären nicht ALLAHs Gunst euch gegenüber und Seine Gnade, und daß ALLAH doch reue-annehmend, allweise ist, (hätte ER euch bloßgestellt).\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdcccii{Gewiß, diejenigen, welche die Ifk-Lüge erfanden, sind ein Teil von euch. Denkt nicht, daß diese (Ifk-Lüge) etwas Böses für euch ist. Nein, sondern sie ist gut für euch. Für jeden Mann von ihnen gibt es das, was er sich von der Verfehlung erwarb. Und derjenige von ihnen, der das Meiste davon verantwortete, für ihn ist eine qualvolle Peinigung bestimmt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdccciii{Hätten doch, als ihr sie (die Ifk- Lüge) gehört habt, die Mumin-Männer und die Mumin-Frauen über sich selbst Gutes gedacht und gesagt: ``Dies ist eine eindeutige Ifk-Lüge!"\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdccciv{Hätten sie doch dafür vier Zeugen gebracht! Und sollten sie nun die Zeugen nicht bringen, dann sind diese selbst bei ALLAH die wirklichen Lügner.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdcccv{Und wären nicht ALLAHs Gunst euch gegenüber und Seine Gnade im Diesseits und im Jenseits, hätte euch wegen dem, mit dessen (Veröffentlichung) ihr eilig wart, gewiß eine überharte Peinigung getroffen,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdcccvi{als ihr sie (die Ifk-Lüge) mit euren Zungen einander weitergabt und mit euren Mündern das gesagt habt, worüber ihr kein Wissen habt. Und ihr erachtet sie als Kleinigkeit, während sie bei ALLAH etwas Ungeheuerliches ist.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdcccvii{Und hättet ihr, als ihr sie (die Ifk-Lüge) gehört habt, doch gesagt: ``Es gebührt uns nicht, solches zu sprechen. Gepriesen-erhaben bist DU! Diese (Ifk-Lüge) ist eine ungeheuerliche Unterstellung."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdcccviii{ALLAH ermahnt euch, daß ihr jemals Gleiches wiederholt, solltet ihr Mumin sein.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdcccix{Und ALLAH erläutert euch die Ayat. Und ALLAH ist allwissend, allweise.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdcccx{Gewiß, diejenigen, die es gerne haben, daß die Schändlichkeit sich verbreitet unter denjenigen, die den Iman verinnerlichten, für diese ist eine äußerst schmerzvolle Peinigung bestimmt im Diesseits und im Jenseits. Und ALLAH weiß, während ihr nicht wisst.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdcccxi{Und wären nicht ALLAHs Gunst euch gegenüber und Seine Gnade, und daß ALLAH doch allgütig, allgnädig ist, (hätte ER euch dafür gepeinigt).\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdcccxii{Ihr, die den Iman verinnerlicht habt! Folgt nicht den Schritten des Satans! Und wer den Schritten des Satans folgt, dieser fordert gewiß zur Schändlichkeit und zum Mißbilligten auf. Und wären nicht ALLAHs Gunst euch gegenüber und Seine Gnade, hätte sich keiner von euch jemals reinigen können. Doch ALLAH reinigt, wen ER will. Und ALLAH ist allhörend, allwissend.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdcccxiii{Und diejenigen von euch, die über Überfluß und Reichtum verfügen, sollen nicht schwören, daß sie denjenigen von der Verwandtschaft, den Armen und den Ausgewanderten fi-sabilillah nichts geben. Und sie sollen verzeihen und erlassen! Habt ihr es etwa nicht gerne, daß ALLAH euch vergibt?! Und ALLAH ist allvergebend, allgnädig.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdcccxiv{Gewiß, diejenigen, die den keuschen, ahnungslosen (der Unterstellung gegenüber) Mumin-Frauen (Zina) vorwerfen, wurden im Diesseits und im Jenseits verflucht. Und für sie ist eine überharte Peinigung bestimmt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdcccxv{An dem Tag, wenn ihre Zungen, ihre Hände und ihre Füße gegen sie Zeugnis ablegen über das, was sie zu tun pflegten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdcccxvi{An dem Tag wird ALLAH ihnen ihren wahren Din vergüten, und sie werden wissen, daß ALLAH gewiß Der Offenkundige, Der Wahrhaftige ist.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdcccxvii{Die weiblichen Schlechten sind für die männlichen Schlechten, und die männlichen Schlechten sind für die weiblichen Schlechten bestimmt. Und die weiblichen Guten sind für die männlichen Guten, und die männlichen Guten sind für die weiblichen Guten bestimmt, diese sind unschuldig hinsichtlich dessen, was sie (die anderen an Ifk-Lüge) sagen. Für sie ist Vergebung und edles Rizq bestimmt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdcccxviii{Ihr, die den Iman verinnerlicht habt! Betretet keine Häuser die nicht eure Häuser sind, bis ihr euch bemerkbar macht und deren Bewohner mit dem Salam- Gruß begrüßt. Dies ist besser für euch, damit ihr euch besinnt.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdcccxix{Und solltet ihr darin niemanden finden, dann betretet sie nicht, bis euch Erlaubnis gewährt wird. Und sollte euch gesagt werden: ``Kehrt zurück!" Dann kehrt zurück! Dies ist reiner für euch. Und ALLAH ist über das, was ihr tut, allwissend.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdcccxx{Euch trifft keine Verfehlung, wenn ihr unbewohnte Häuser betretet, in denen eure Gebrauchsgüter sind. Und ALLAH weiß, was ihr offenlegt, und was ihr verbergt.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdcccxxi{Sag zu den Mumin-Männern, daß sie von ihren Blicken niederschlagen und ihre Keuschheit bewahren. Dies ist reiner für sie. Gewiß, ALLAH ist dessen allkundig, was sie bewerkstelligen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdcccxxii{Und sag zu den Mumin-Frauen, daß sie von ihren Blicken niederschlagen, ihre Keuschheit bewahren und ihren Schmuck nicht sichtbar tragen außer dem, was davon sichtbar wird, und daß sie ihre Khumur bis über ihre Dschuyub ziehen, und daß sie ihren Schmuck vor niemandem sichtbar tragen außer vor ihren Ehemännern, ihren Vätern, den Vätern ihrer Ehemänner, ihren Söhnen, den Söhnen ihrer Ehemänner, ihren Brüdern, den Söhnen ihrer Brüder, den Söhnen ihrer Schwestern, ihren Frauen, denjenigen, die ihr gehören, den Bediensteten außer denjenigen mit Verlangen von den Männern oder den Kindern, die noch nichts über die Blößen der Frauen wissen, und daß sie mit ihren Füßen nicht (auf den Boden) stampfen, damit bekannt wird, was sie noch von ihrem Schmuck verbergen. Und bittet ALLAH um Vergebung allesamt - ihr Mumin! - damit ihr erfolgreich werdet.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdcccxxiii{Und verheiratet die Unverheirateten unter euch und die gottgefällig Guttuenden von euren Dienern und euren Dienerinnen! Sollten sie arm sein, wird ALLAH sie von Seiner Gunst reich machen. Und ALLAH ist allumfassend, allwissend.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdcccxxiv{Und diejenigen, die keine Heiratsmöglichkeit finden, sollen keusch bleiben, bis ALLAH sie von Seiner Gunst reich macht. Und diejenigen von denen, die euch gehören, wenn sieMukataba begehren, dann macht mit ihnen Mukataba, wenn ihr von ihnen Gutes wisst. Und gebt ihnen von ALLAHs Vermögen, das ER euch gab. Und zwingt eure Dienerinnen nicht zur Prostitution - wenn sie keusch bleiben wollen - um das Vergängliche vom diesseitigen Leben zu erstreben. Und wer sie zwingt, dann ist ALLAH gewiß nach dem Zwang ihnen gegenüber allvergebend, allgnädig.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdcccxxv{Und gewiß, bereits sandten WIR dir doch erläuternde Ayat hinab, ein Gleichnis von denjenigen, die vor euch vergingen, und eine Ermahnung für die Muttaqi.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdcccxxvi{ALLAH ist Der Erleuchtende von den Himmeln und der Erde. Das Gleichnis Seiner Erleuchtung ist wie eine Aushöhlung, in der eine Leuchte ist. Die Leuchte ist in einem Glas. Das Glas ähnelt einem leuchtenden Stern, das von einem Baum voller Baraka, einem Olivenbaum angezündet wird, der weder östlich noch westlich liegt. Sein Öl leuchtet beinahe, auch dann, würde es kein Feuer berühren, Licht über Licht! ALLAH leitet zum Seinem Licht recht, wen ER will. Und ALLAH prägt die Gleichnisse für die Menschen. Und ALLAH ist über alles allwissend.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdcccxxvii{Gepriesen wird ER) in Häusern, die ALLAH einzurichten erlaubte und in denen Seines Namens gedacht wird. ER wird darin nach Sonnenaufgang und vor Sonnenuntergang lobgepriesen von\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdcccxxviii{Männern, die weder durch Handel noch durch Verkauf vom Gedenken an ALLAH abgelenkt werden, sowie nicht vom ordnungsgemäßen Verrichten des rituellen Gebets und nicht vom Entrichten der Zakat. Sie fürchten einen Tag, an dem die Herzen und die Blicke sich ständig wenden werden.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdcccxxix{ALLAH wird ihnen gewiß vergelten Besseres als das, wie sie handelten, und wird ihnen noch mehr von Seiner Gunst geben. Und ALLAH gewährt Rizq, wem ER will, ohne Berechnung.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdcccxxx{Und diejenigen, die Kufr betrieben haben, ihre Handlungen ähneln einer Fata Morgana in einer Ebene. Der Durstige hält sie für Wasser, und wenn er es erreicht, findet er es als Nichts, aber er findet ALLAH bei sich, dann wird ER ihm seine Abrechnung vergüten. Und ALLAH ist schnell im Zur-Rechenschaft-Ziehen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdcccxxxi{Oder (sie ähnelt) den Finsternissen im einem tiefen Meer, das von einer Woge bedeckt wird, über der noch eine Woge liegt, und darüber eine Wolke ist. Es sind Finsternisse, die einen über den anderen, wenn er seine Hand ausstreckt, beinahe sieht er sie nicht. Und wem ALLAH kein Licht gewährt, für den gibt es kein Licht.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdcccxxxii{Hast du etwa nicht wahrgenommen, daß ALLAH alle lobpreisen, die in den Himmeln und auf Erden sind, und daß die Vögel (Flügel) ausstreckend sind?! Alle kennen bereits das rituelle Gebet und das Lobpreisen für Ihn. Und ALLAH ist allwissend über das, was sie machen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdcccxxxiii{Und ALLAH gehört die Herrschaft der Himmel und der Erde. Und zu ALLAH ist das Werden.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdcccxxxiv{Hast du etwa nicht wahrgenommen, daß ALLAH die Wolken ziehen läßt, dann fügt ER sie zusammen, dann macht ER sie zu Aufeinandergehäuften, dann siehst du den Regen, während er aus diesen herauskommt. Und ER läßt vom Himmel aus (Wolken-)Bergen fallen, in denen Hagel ist, dann trifft ER damit, wen ER will, und wendet ihn von dem ab, den ER will. Beinahe macht der Glanz seines Blitzes die Augen blind.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdcccxxxv{ALLAH läßt die Nacht und den Tag wenden. Gewiß, darin ist zweifelsohne eine Lehre für diejenigen von Einblick.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmdcccxxxvi{Und ALLAH erschuf alles sich bewegende Lebewesen aus Wasser. So geht manch einer von ihnen auf seinem Bauch, und manch einer von ihnen geht auf zwei Beinen, und manch einer von ihnen geht auf Vieren. ALLAH erschafft, was ER will! Gewiß, ALLAH ist über alles allmächtig.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmdcccxxxvii{Gewiß, bereits sandten WIR dir erläuternde Ayat hinab. Und ALLAH leitet recht, wen ER will, auf einen geradlinigen Weg.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmdcccxxxviii{Und sie sagen: ``Wir haben den Iman an ALLAH und an den Gesandten verinnerlicht und gehorcht." Dann wendet sich danach eine Gruppe von ihnen ab. Und diese sind doch keine Mumin.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmdcccxxxix{Und wenn sie zu ALLAH und zu Seinem Gesandten gerufen werden, damit er zwischen ihnen richtet, ist eine Gruppe von ihnen sich abwendend.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmdcccxl{Und wenn sie das Recht auf ihrer Seite haben, kommen sie zu ihm sich fügend.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmdcccxli{Ist etwa in ihren Herzen Krankheit, oder haben sie etwa Zweifel (an ihm), oder fürchten sie etwa, daß ALLAH und Sein Gesandter sie benachteiligen?! Nein, sondern diese sind die wirklichen Unrecht-Begehenden.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmdcccxlii{Die Worte der Mumin sind nur - wenn sie zu ALLAH und seinem Gesandten gerufen werden, um zwischen ihnen zu richten - daß sie sagen: ``Wir haben gehört und gehorcht." Und diese sind die eigentlichen Erfolgreichen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmdcccxliii{Und wer ALLAH und Seinem Gesandten gehorcht, ALLAH Ehrfurcht erweist und Ihm gegenüber Taqwa gemäß handelt, diese sind dann die eigentlichen Gewinner.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmdcccxliv{Auch legten sie bei ALLAH nachdrückliche Eide ab: ``Wenn du sie anweisen würdest, würden sie gewiß ausziehen." Sag: ``Schwört nicht! Ein sichtbarer Gehorsam (ist besser)." Gewiß, ALLAH ist dessen allkundig, was ihr tut.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmdcccxlv{Sag: ``Gehorcht ALLAH und gehorcht dem Gesandten!" Und sollten sie sich abwenden, dann verantwortet er, was ihm auferlegt wurde, und ihr verantwortet, was euch auferlegt wurde. Und wenn ihr ihm gehorcht, dann findet ihr Rechtleitung. Und dem Gesandten obliegt nichts außer dem Verkünden.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmdcccxlvi{ALLAH versprach denjenigen, die von euch den Iman verinnerlicht und gottgefällig Gutes getan haben, daß ER sie auf Erden die Nachfolge doch antreten lassen wird, wie ER diejenigen vor ihnen die Nachfolge antreten ließ, und daß ER ihnen ihren Din noch festigen wird, an welchem ER für sie Gefallen fand, und daß ER ihnen nach ihrer Angst doch Sicherheit eintauschen wird: Sie dienen Mir und betreiben Mir gegenüber keinerlei Schirk. Und wer danach Kufr betreibt, diese sind dann die wirklichen Fasiq.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmdcccxlvii{Und verrichtet ordnungsgemäß das rituelle Gebet, entrichtet die Zakat und gehorcht dem Gesandten, damit euch Gnade erwiesen wird.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmdcccxlviii{Denke auf keinen Fall über diejenigen, die Kufr betrieben haben, daß sie auf Erden (Uns) zu schaffen machen! Ihre Unterkunft ist das Feuer. Und doch erbärmlich ist das Werden.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmdcccxlix{Ihr, die den Iman verinnerlicht habt! Diejenigen, die euch gehören, und diejenigen von euch, die noch nicht die Pubertät erreichten, sollen euch dreimal (vor dem Eintreten) um Erlaubnis bitten: Vor dem Fadschr-Gebet, wenn ihr eure Kleidung mittags ablegt und nach dem 'Ischaa-Gebet. Diese sind drei (Zeiten) eurer Entblößung. Weder euch noch sie trifft Verfehlung nach ihnen (diesen Zeiten). Sie kommen öfters zu euch, die einen von euch zu den anderen. Solcherart erläutert ALLAH für euch die Ayat. Und ALLAH ist allwissend, allweise.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmdcccl{Und wenn die Kinder von euch die Pubertät erreichten, dann sollen sie um Erlaubnis bitten, wie diejenigen vor ihnen um Erlaubnis gebeten haben. Solcherart erläutert ALLAH für euch Seine Ayat. Und ALLAH ist allwissend, allweise.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmdcccli{Und die Alten (der Intimität abgeneigten) von den Frauen, die nicht auf Heirat hoffen, diese trifft keine Verfehlung, wenn sie ihre Kleidung ablegen, ohne einen Schmuck sichtbar zu tragen. Und daß sie es lassen, ist gewiß besser für sie. Und ALLAH ist allhörend, allwissend.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmdccclii{Weder für den Blinden ist es eine Unannehmlichkeit, noch ist es für den Lahmen eine Unannehmlichkeit, noch ist es für den Kranken eine Unannehmlichkeit, noch für euch selbst, daß ihr esst in euren Häusern, in den Häusern eurer Väter, in den Häusern eurer Mütter, in den Häusern eurer Brüder, in den Häusern eurer Schwestern, in den Häusern eurer Onkel väterlicherseits, in den Häusern eurer Tanten väterlicherseits, in den Häusern eurer Onkel mütterlicherseits, in den Häusern eurer Tanten mütterlicherseits, in denen, über deren Schlüssel ihr verfügt, oder im (Haus) eures Freundes. Es trifft euch keine Schuld, daß ihr gemeinsam oder einzeln esst. Wenn ihr Häuser betretet, dann grüßt euch gegenseitig mit einem guten Gruß von ALLAH voller Baraka. Solcherart erläutert ALLAH euch die Ayat, damit ihr verständig werdet.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmdcccliii{Die Mumin sind nur diejenigen, die den Iman an ALLAH und an Seinen Gesandten verinnerlicht haben und wenn sie mit ihm in einer versammelnden Angelegenheit waren, gehen sie nicht fort, bis sie ihn um Erlaubnis bitten. Gewiß, diejenigen, die dich um Erlaubnis bitten, diese sind diejenigen, die den Iman an ALLAH und an Seinen Gesandten verinnerlichen. Und sollten sie dich um Erlaubnis wegen mancher privaten Angelegenheiten bitten, dann gewähre Erlaubnis demjenigen von ihnen, dem du willst, und bitte ALLAH für sie um Vergebung. Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmdcccliv{Macht nicht das Rufen des Gesandten zwischen euch wie das Rufen von euch untereinander! Bereits kennt ALLAH diejenigen von euch, die sich angeschlossen (an andere) davonschleichen. Diejenigen, die von Seiner Anweisung abweichen, sollen sich (davor) in Acht nehmen, daß sie keine Fitna trifft oder daß sie keine qualvolle Peinigung trifft.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmdccclv{Ja! ALLAH gehört zweifelsohne alles, was in Himmeln und auf Erden ist. Bereits kennt ER genau das, woran ihr seid, und den Tag, an dem sie zu Ihm zurückgebracht werden, dann wird ER ihnen kundtun über das, was sie taten. Und ALLAH ist über alles allwissend.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmdccclvi{\basmalahde Immer allerhabener ist Derjenige, Der Al-furqan Seinem Diener nach und nach hinabsandte, damit er für die ganze Schöpfung ein Warner wird.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmdccclvii{Derjenige, Dem die Herrschaft der Himmel und der Erde gehören und Der nie Sich ein Kind nahm und für Den es nie einen Partner in der Herrschaft gab. Und ER erschuf alles und bestimmte es genaust.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmdccclviii{Und sie nahmen sich anstelle von Ihm Gottheiten, die nichts erschaffen, während sie erschaffen werden, und die sich selbst weder schaden noch nützen können und die weder über Tod noch über Leben noch über Erweckung verfügen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmdccclix{Und diejenigen, die Kufr betrieben haben, sagten: ``Dies ist nichts anderes als Ifk-Lüge, die er erfand, und ihm halfen dabei andere Leute." Bereits begingen sie Unrecht und Falschzeugnis.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmdccclx{Und sie sagten: ``Dies sind die Legenden der Vorfahren, die er schreiben ließ, so werden sie ihm morgens und abends diktiert."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmdccclxi{Sag: ``Den (Quran) sandte Derjenige hinab, Der das Geheimnis in den Himmeln und auf Erden kennt. Gewiß, ER ist immer allvergebend, allgnädig.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmdccclxii{Und sie sagten: ``Was ist los mit diesem Gesandten, er ißt die Speisen und geht zu den Märkten, würde zu ihm doch ein Engel herabgesandt, damit er mit ihm einWarner wird!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmdccclxiii{Oder würde ihm doch ein Schatz heruntergeworfen! Oder wäre für ihn doch eine Dschanna, von der er essen würde!" Und die Unrecht-Begehenden sagten: ``Ihr folgt doch nur einem verzauberten Mann."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmdccclxiv{Siehe, wie sie dir die Gleichnisse prägten, dann gingen sie irre, und sie vermögen zu keinem Weg.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmdccclxv{Immer allerhabener ist Derjenige, wenn ER will, ER gewährte dir noch Besseres als dies, nämlich Dschannat, die von Flüssen durchflossen werden, und ER gewährt dir Schlösser.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmdccclxvi{Nein, sondern sie leugneten die Stunde ab. Und WIR bereiteten für denjenigen, der die Stunde ableugnete, äußerst heftiges Feuer.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmdccclxvii{Wenn es (das Feuer) sie von einem fernen Ort sieht, hören sie von ihm (das Geräusch) seiner Wut und seines Anfachens.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmdccclxviii{Und wenn sie dort in einen engen Ort muqranin hineingeworfen werden, flehen sie dort um den Untergang.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmdccclxix{Fleht heute nicht nur um einen Untergang, sondern fleht um viele Untergänge!\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmdccclxx{Sag: ``Ist dies besser oder die Dschanna der Ewigkeit, die den Muttaqi versprochen wurde?!" Sie ist für sie die Belohnung und das Werden.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmdccclxxi{Darin haben sie, was sie begehren, als Ewige. Dies war von deinem HERRN ein Versprechen, um das (von den Mumin) gebeten wurde.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmdccclxxii{Und am Tag, wenn ER sie und das, dem sie anstelle von ALLAH dienten, versammelt und dann sagt: ``Habt ihr etwa diese Meine Diener irregeführt, oder sind sie selbst irregegangen?!"\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmdccclxxiii{Sie sagten: ``Gepriesen-erhaben bist DU! Uns gebührt nicht, daß wir uns anstelle von Dir irgendwelche Wali nehmen. Doch DU hast sie und ihre Ahnen vergnügen lassen, bis sie das Gedenken vergaßen und nutzlose Leute waren."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmdccclxxiv{Also sie wiesen, was ihr sagt, als Lüge zurück, so werdet ihr weder (die Peinigung) abwenden noch (euch) beistehen. Und wer von euch Unrecht begeht, den werden WIR eine große Peinigung erfahren lassen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmdccclxxv{Und WIR entsandten vor dir nur von den Gesandten, welche Speise aßen und zu den Märkten gingen. Und WIR machten die einen von euch für die anderen zur Fitna, ob ihr euch in Geduld übt?! Und dein HERR ist immer allsehend.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmdccclxxvi{Und diejenigen, die auf Unsere Begegnung nicht hoffen, sagten: ``Würden zu ihm doch Engel hinabgesandt werden! Oder würden wir doch unseren HERRN sehen!" Gewiß, bereits erhoben sie sich in Arroganz in ihren Inneren und übertrieben doch eine große Übertreibung.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmdccclxxvii{Am Tage, wenn sie die Engel sehen werden, gibt es keine frohe Botschaft an jenem Tag für die schwer Verfehlenden. Und sie (die Engel) sagen: ``(Die Dschanna ist für euch) gänzlich verboten."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmdccclxxviii{Und WIR wandten Uns zu dem, was sie an Handlungen vollbrachten, dann machten WIR sie zu verflogenem Staub.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmdccclxxix{Die Weggenossen der Dschanna haben an diesem Tag eine bessere Niederlassung und einen schöneren Ort zum Zurückziehen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmdccclxxx{Und an dem Tag, wenn der Himmel sich mit dicken Wolken spaltet, und die Engel wurden nach und nach hinabgesandt,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmdccclxxxi{die wahre Herrschaft an diesem Tag gehört Dem Allgnade Erweisenden. Und er ist für die Kafir ein harter Tag.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmdccclxxxii{Und an dem Tag, wenn der Unrecht-Begehende sich in die Hände beißt, sagt er: ``Hätte ich doch mit dem Gesandten einen (gemeinsamen) Weg eingeschlagen!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmdccclxxxiii{Mein Untergang! Hätte ich mir doch jene Person nicht zum engen Freund genommen!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmdccclxxxiv{Gewiß, bereits ließ er mich vom Gedenken (ALLAHs) abirren, nachdem es mir zuteil wurde." Und der Satan läßt den Menschen immer wieder im Stich.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmdccclxxxv{Und der Gesandte sagte: ``Mein HERR! Gewiß, meine Leute vernachlässigen den Quran."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmdccclxxxvi{Und solcherart machten WIR für jeden Propheten Erzfeinde von den schwer Verfehlenden. Und dein HERR genügt als Rechtleitender und als Beistehender.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmdccclxxxvii{Und diejenigen, die Kufr betrieben haben, sagten: ``Würde ihm der Quran doch auf einmal hinabgesandt!" Es ist so, damit WIR mit ihm dein Herz festigen. Und WIR haben ihn doch in Tartil-Form vorgetragen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmdccclxxxviii{Und sie bringen dir kein Gleichnis, ohne daß WIR dir die Wahrheit und eine bessere Erläuterung zukommen ließen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmdccclxxxix{Es sind diejenigen, die auf ihren Gesichtern vor Dschahannam versammelt werden, diese haben einen schlimmeren Ort und sind vom Weg weit abgeirrt.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmdcccxc{Und gewiß, bereits ließen WIR Musa die Schrift zuteil werden und machten mit ihm seinen Bruder Harun als einen Assistenten.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmdcccxci{Dann sagten WIR: ``Geht beide zu den Leuten, die Unsere Ayat ableugneten." Dann vernichteten WIR sie in kompletter Vernichtung.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmdcccxcii{Und die Leute von Nuh, als sie den Gesandten der Lüge bezichtigten, ertränkten WIR sie und machten sie zu einer Aya für die Menschen. Und WIR bereiteten für die Unrecht-Begehenden eine qualvolle Peinigung.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmdcccxciii{Sowie 'Aad, Thamud, die Weggenossen von Ar-rass und viele Generationen zwischen diesen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmdcccxciv{Und allen prägten WIR die Gleichnisse. Und alle haben WIR in Stückchen zerstückelt.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmdcccxcv{Und gewiß, bereits kamen sie doch zur Ortschaft, die mit dem Regen des Unheils beregnet wurde. Haben sie sie etwa noch nicht gesehen?! Nein, sondern sie pflegten auf keine Erweckung zu hoffen.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmdcccxcvi{Und wenn sie dich sehen, nehmen sie dich zu nichts außer zum Spott: ``Ist dieser derjenige, den ALLAH als Gesandten entsandte?\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmdcccxcvii{Beinahe hätte er uns doch von unseren Gottheiten abirren lassen, hätten wir uns nicht in Geduld ihnen gegenüber geübt!" Und sie werden wissen, wenn sie die Peinigung sehen, wer weiter vom Weg abgeirrt ist.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmdcccxcviii{Wie siehst du denjenigen, der seine Neigung zu seinem Gott machte?! Willst du etwa ein Wakil über ihn sein?!\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmdcccxcix{Oder denkst du etwa, daß die meisten von ihnen zuhören oder nachdenken?! Sie sind doch nur wie die An'am. Nein, sondern sie sind noch weiter vom Weg abgeirrt.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmcm{Siehst du etwa nicht (die Schöpfung) deines HERRN, wie ER den Schatten ausbreiten läßt?! Und hätte ER gewollt, hätte ER ihn stillstehend gemacht. Dann machten WIR die Sonne als Hinweis auf ihn.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmcmi{Dann verkürzten WIR ihn zu Uns mit leichter Verkürzung.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmcmii{Und ER ist Derjenige, Der für euch die Nacht als Kleidung machte und den Schlaf zum Ausruhen, und ER machte den Tag zur Auferstehung.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmcmiii{Und ER ist Derjenige, Der die Winde als frohe Botschaft vor Seiner Gnade schickte. Und WIR ließen vom Himmel reines Wasser fallen,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmcmiv{damit WIR mit ihm eine tote Landschaft lebendig machen, und damitWIR es zum Trinken denen geben, die WIR erschufen, An'am und vielen Menschen.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcmv{Und gewiß, bereits teilten WIR ihn zwischen ihnen auf, damit sie sich besinnen. Doch diemeisten Menschen lehnten dann ab, außer dem äußersten Kufr.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcmvi{Und hätten WIR es gewollt, hätten WIR doch zu jeder Ortschaft einen Warner entsandt!\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcmvii{So gehorche den Kafir nicht, und leiste mit ihm (dem Quran) gegen sie einen großen Dschihad!\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcmviii{Und ER ist Derjenige, Der beide Meere nebeneinander machte. Dieses ist wohlschmeckend, süß, und dieses ist salzig, bitter. Und ER machte zwischen ihnen eine Trennung und eine komplette Isolierung.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcmix{Und ER ist Derjenige, Der aus dem Wasser einen Menschen erschuf, dann machte ER ihn zur Abstammung und Verschwägerung. Und dein HERR ist immer allmächtig.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcmx{Und sie dienen anstelle von ALLAH dem, das ihnen weder nützt, noch schadet. Und der Kafir ist gegen seinen HERRN ein Beistehender.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcmxi{Und WIR entsandten dich nur als Überbringer froher Botschaft und als Warner.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcmxii{Sag: ``Ich verlange von euch dafür keinen Lohn! Nur wer zu seinem HERRN einen Weg einschlagen will, (soll es tun)."\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcmxiii{Und übe Tawakkul Dem Lebendigen, Der nie stirbt, und lobpreise mit Seinem Lob! Und ER genügt über die Verfehlungen Seiner Diener als Allkundiger.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcmxiv{Er ist Derjenige, Der die Himmel, die Erde und das, was zwischen ihnen ist, in sechs Ay-yam erschuf, dann wandte ER Sich Al'ahrsch zu. ER ist Der Allgnade Erweisende, so frage über Ihn einen Kundigen!\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcmxv{Und als ihnen gesagt wurde: ``Vollzieht Sudschud für den Allgnade Erweisenden!", sagten sie: ``Und was ist Der Allgnade Erweisende? Sollen wir etwa Sudschud zu dem vollziehen, den du uns anweist?!" Und es mehrte noch ihre Abneigung.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcmxvi{Immer allerhabener ist Derjenige, Der im Himmel Sternbilder einsetzte, und darin eine Leuchte und einen erhellten Mond machte.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcmxvii{Und ER ist Derjenige, Der die Nacht und den Tag als Nachfolge machte für denjenigen, der sich besinnen will oder sich dankbar erweisen will.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcmxviii{Und die Diener Des Allgnade Erweisenden sind diejenigen, die auf der Erde in Bescheidenheit gehen, und wenn die Unwissenden sie belästigen, sagen sie: ``Salam (sei mit euch)!".\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcmxix{Auch diejenigen, die ihre Nacht als Sudschud- und Qiyam-Vollziehende verbringen.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcmxx{Sowie diejenigen, die sagen: ``Unser HERR! Wende von uns die Peinigung von Dschahannam ab, denn ihre Peinigung ist gewiß eine bleibende Peinigung.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcmxxi{Gewiß, sie (Dschahannam) ist schlimm als Niederlassung und als Aufenthaltsort."\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcmxxii{Auch diejenigen, wenn sie ausgeben, weder verschwenden noch geizen, sondern es ist dazwischen mittelmäßig.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmcmxxiii{Und diejenigen, die weder neben ALLAH an einen anderen Gott Bittgebete richten, noch den Menschen töten, den ALLAH für haram erklärte, außer mit Berechtigung, noch Zina begehen. Und wer dieses tut, findet die Vergeltung.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmcmxxiv{Für ihn wird die Peinigung am Tag der Auferstehung verdoppelt, und er bleibt dort als Erniedrigter ewig\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmcmxxv{außer demjenigen, der bereut, den Iman verinnerlicht und gottgefällig Gutes getan hat, diesen tauscht ALLAH ihre gottmißfälligen Taten in gottgefällig gute Taten um. Und ALLAH bleibt immer allvergebend, allgnädig.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmcmxxvi{Und wer bereute und gottgefällig Gutes tat, dieser bereut ALLAH gegenüber wirklich.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmcmxxvii{Auch diejenigen, die das Lügnerische nicht bezeugen, und wenn sie beim sinnlosen Treiben vorbeigehen, gehen sie in Würde vorbei.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmcmxxviii{Und diejenigen, wenn sie mit den Ayat ihres HERRN ermahnt werden, fallen nicht vor ihnen taub und blind nieder.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmcmxxix{Ebenso diejenigen, die sagen: ``Unser HERR! Schenke uns von unseren Partnerwesen und unserer Nachkommenschaft Erfreuliches. Und mache uns für die Muttaqi als Imam!\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmcmxxx{Diesen wird das Paradies vergolten für das, was sie an Geduld aufbrachten. Und sie werden darin mit Gruß und Salam empfangen.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmcmxxxi{Darin bleiben sie ewig. Es ist schön als Niederlassung und Aufenthaltsort.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmcmxxxii{Sag: ``Mein HERR würde euch nicht beachten, wäre nicht euer Bittgebet. Ihr habt bereits doch abgeleugnet, so wird es (das Vergelten) unabdingbar sein."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmcmxxxiii{\basmalahde Ta-sin-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmcmxxxiv{Diese sind die Ayat der deutlichen Schrift.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmcmxxxv{Nicht etwa zugrunde richtest du dich selbst in Verdrießlichkeit, daß sie keine Mumin werden.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmcmxxxvi{Wenn WIR wollten, würden WIR über ihnen vom Himmel eine Aya nach und nach hinabsenden, vor der ihre Nacken stets gebeugt bleiben.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmcmxxxvii{Und es wird ihnen keine Ermahnung vom Allgnade Erweisenden zuteil, die erneuert wird, ohne daß sie ihr gegenüber abwendend waren.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmcmxxxviii{Also bereits leugneten sie ab, so wird ihnen noch dieMitteilung dessen zuteil, das sie zu verspotten pflegten.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmcmxxxix{Haben sie etwa nicht auf die Erde geschaut, wieviel WIR auf ihr von jeder edlen Zweiheit sprießen ließen?!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmcmxl{Gewiß, darin gibt es doch eine Aya. Und die meisten von ihnen waren keine Mumin.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmcmxli{Und gewiß, dein HERR ist doch Der Allwürdige, Der Allgnädige.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmcmxlii{Und (erinnere daran), als dein HERRMusa gerufen hat: ``Gehe zu den unrechtbegehenden Leuten,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmcmxliii{den Pharaos-Leuten! Wollen sie etwa nicht Taqwa gemäß handeln?!"\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmcmxliv{Er sagte: ``Mein HERR! Ich fürchte, daß sie mich der Lüge bezichtigen werden,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmcmxlv{daß meine Brust bedrückt wird und meine Zunge sich nicht löst, so mache aus Harun einen Gesandten!\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmcmxlvi{Auch schulde ich ihnen noch (die Sühne) einer Verfehlung, so fürchte ich, daß sie mich töten! \qt@no{14}}%
\qt@newcmd\qurantrans@de@mmcmxlvii{ER sagte: ``Nein, sicher nicht! Dann geht beide mit Unseren Ayat! Gewiß, WIR sind mit euch allhörend,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmcmxlviii{dann kommt bei Pharao an und sagt: ``Wir sind ein Gesandter Des HERRN aller Schöpfung!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmcmxlix{Schicke mit uns die Kinder Israils!"\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmcml{Er Pharao sagte: ``Haben wir dich etwa nicht unter uns als Neugeborenes erzogen?! Auch verweiltest du unter uns Jahre von deinem Leben,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmcmli{dann vollbrachtest du deine Tat, die du getan hast, und du bist von den Undankbaren."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmcmlii{Er sagte: ``Ich beging sie damals, während ich von den Irrenden war,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmcmliii{dann flüchtete ich vor euch, als ich euch fürchtete, dann schenkte mir mein HERRWeisheit und machte mich zu einem der Gesandten.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmcmliv{Und ist dies eine Wohltat, die du mir vorhältst, daß du die Kinder Israils zu Dienern machtest?!"\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmcmlv{Pharao sagte: ``Und was ist ``HERR aller Schöpfung"?"\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmcmlvi{Er sagte: ``ER ist Der HERR der Himmel, der Erde und dessen, was zwischen ihnen ist, solltet ihr Gewißheit erlangen wollen."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmcmlvii{Er (Pharao) sagte zu denjenigen um ihn herum: ``Hört ihr etwa nicht?!"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmcmlviii{Er (Musa) sagte: ``ER ist euer HERR und Der HERR eurer ersten Vorfahren!"\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmcmlix{Er (Pharao) sagte: ``Gewiß, euer Gesandter, der zu euch entsandt wurde, ist doch geistes-gestört."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmcmlx{Er (Musa) sagte: ``Er ist Der HERR vom Osten, Westen und dem, was zwischen ihnen ist, solltet ihr verständig sein!"\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmcmlxi{Er (Pharao) sagte: ``Wenn du dir einen Gott außer mir nimmst, dann werde ich dich gewiß einen von den Gefangenen machen."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmcmlxii{Er (Musa) sagte: ``Auch dann, wenn ich dir eine erklärende Sache bringe?!"\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmcmlxiii{Er (Pharao) sagte: ``Dann bringe sie, solltest du von den Wahrhaftigen sein."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmcmlxiv{Dann warf er seinen Stock, dann war er eine echte Schlange.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmcmlxv{Und er zog seine Hand heraus, dann war sie hell für die Zuschauer.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmcmlxvi{Er (Pharao) sagte zu den Entscheidungsträgern um ihn herum: ``Gewiß, dieser ist doch ein äußerst wissender Magier.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmcmlxvii{Er will euch von eurem Land mit seiner Magie vertreiben. Also was weist ihr nun an?"\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmcmlxviii{Sie sagten: ``Vertage (die Angelegenheit) von ihm und seinem Bruder, und schicke in die Städte Versammelnde,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmcmlxix{die dir jeden äußerst wissenden Magier holen."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmcmlxx{Dann versammelte er die Magier zu einem Termin an einem bekannten Tag.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmcmlxxi{Und es wurde den Menschen gesagt: ``Wollt ihr euch nicht versammeln,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmcmlxxii{damit wir den Magiern folgen, wenn sie die Sieger werden?"\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmcmlxxiii{Und als die Magier kamen, sagten sie zu Pharao: ``Bekommen wir sicher einen Lohn, wenn wir die Sieger werden?"\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmcmlxxiv{Er sagte: ``Ja! Und gewiß, ihr werdet dann doch von den Vertrauten sein."\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmcmlxxv{Musa sagte ihnen: ``Werft nun, was ihr werfen wollt."\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmcmlxxvi{Dann warfen sie ihre Seile und Stöcke und sagten: ``Bei der Würde von Pharao! Gewiß, wir sind doch die Sieger."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmcmlxxvii{Dann warf Musa seinen Stock, dann verschlang er, was sie vortäuschten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmcmlxxviii{Dann fielen die Magier in Sudschud nieder,\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmcmlxxix{sie sagten: ``Wir bekundeten den Iman an Den HERRN aller Schöpfung,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmcmlxxx{Den HERRN von Musa und Harun."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmcmlxxxi{Er (Pharao) sagte: ``Bekundet ihr etwa den Iman ihm gegenüber, bevor ich euch die Erlaubnis gab?! Gewiß, er ist doch euer Meister, der euch die Magie lehrte, und ihr werdet es doch noch wissen. Ich werde zweifelsohne eure Füße und eure Hände wechselseitig abtrennen und werde euch gewiß noch allesamt kreuzigen!"\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmcmlxxxii{Sie sagten: ``Kein Problem! Gewiß, wir kehren zu unserem HERRN zurück,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmcmlxxxiii{wir begehren doch, daß uns unser HERR unsere Verfehlungen vergibt, da wir die ersten Mumin waren."\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmcmlxxxiv{Und WIR ließen Musa Wahy zuteil werden: ``Ziehe mit Meinen Dienern nachts aus, denn gewiß, ihr werdet verfolgt."\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmcmlxxxv{Dann schickte Pharao in die Städte Versammelnde:\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmcmlxxxvi{``Gewiß, diese sind doch nur eine unbedeutende kleine Gruppe,\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmcmlxxxvii{und gewiß, sie sind für uns doch Verärgernde,\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmcmlxxxviii{und gewiß, wir sind alle doch Achtgebende."\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmcmlxxxix{Dann vertrieben WIR sie aus Dschannat, Quellen,\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmcmxc{Schätzen und edlem Aufenthalt.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmcmxci{Solcherart und WIR ließen sie die Kinder Israils beerben.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmcmxcii{Dann folgten sie ihnen beim Sonnenaufgang,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmcmxciii{und als beide Gruppierungen sich gegenseitig sahen, sagten die Weggenossen von Musa: ``Gewiß, wir werden doch noch eingeholt:"\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmcmxciv{Er (Musa) sagte: ``Nein, sicher nicht! Gewiß, mein HERR ist mit mir, ER wird mich rechtleiten."\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmcmxcv{Dann ließen WIR Musa Wahy zuteil werden: ``Schlage mit deinem Stock auf das Meer!" Dann spaltete es sich. Dann war jedes Stück wie ein gewaltiger Felsenberg.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmcmxcvi{Und WIR ließen dann die anderen näher kommen.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmcmxcvii{Und WIR erretteten Musa und alle, die mit ihm waren, allesamt.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmcmxcviii{Dann ertränkten WIR die anderen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmcmxcix{Gewiß, darin ist doch eine Aya. Und viele von ihnen waren keine Mumin.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmm{Und gewiß, dein HERR ist doch Der Allwürdige, Der Allgnädige.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmi{Und trage ihnen die Begebenheit von Ibrahim vor.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmii{Als er seinemVater und seinen Leuten sagte: ``Wem dient ihr denn?"\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmiii{Sie sagten: ``Wir dienen unseren Statuen, dann bleiben wir ihnen stets ergeben.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmiv{Er sagte: ``Hören sie euch, wenn ihr Bittgebete (an sie) richtet?\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmv{Oder bringen sie euch Schaden oder Nutzen?"\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmvi{Sie sagten: ``Nein, sondern wir fanden unsere Ahnen solcherart tun."\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmvii{Er sagte: ``Habt ihr etwa gesehen, wem ihr zu dienen pflegtet,\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmviii{ihr und eure ersten Ahnen?\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmix{Diese sind gewiß Feinde für mich, außer Dem HERRN aller Schöpfung,\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmx{Derjenige, Der mich erschuf. So wird ER mich rechtleiten.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmxi{Und Derjenige, Der mir zu essen und zu trinken gibt.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmxii{Und sollte ich krank werden, dann heilt ER mich.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmxiii{Und Derjenige, Der mich sterben läßt, dann mich lebendig macht.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmxiv{Und Derjenige, von Ihm ich begehre, daß ER mir meine Verfehlungen am Tag des Din vergibt.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmxv{Mein HERR! Schenke mir Weisheit und lasse mich den gottgefällig Guttuenden folgen!\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmxvi{Und gewähre mir eine Zunge der Wahrhaftigkeit bei den Nachkömmlingen!\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmxvii{Und mache mich von den Erben der Dschanna desWohlergehens!\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmxviii{Und vergib meinem Vater. Er war gewiß von den Irrenden.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmxix{Und lasse mich nicht erniedrigt werden an dem Tag, wenn sie erweckt werden.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmxx{An dem Tag, wenn weder Vermögen nützt noch Kinder,\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmxxi{außer dem, der zu ALLAH mit einem reinen Herz kam."\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmxxii{Und die Dschanna wurde den Muttaqi näher gebracht.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmxxiii{Und die Hölle wurde den Fehlgehenden hervorgebracht.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmxxiv{Und ihnen wurde gesagt: ``Wo ist das, dem ihr gedient habt\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmxxv{anstelle von ALLAH? Können sie euch beistehen oder gar siegen?!"\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmxxvi{Dann wurden sie in sie immer wieder hineingeworfen, so auch die Fehlgehenden\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmmxxvii{und die Gehilfen von Iblis, allesamt.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmmxxviii{Sie sagten, während sie darin stritten:\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmmxxix{``Bei ALLAH! Wir waren gewiß in einem eindeutigen Fehlgehen,\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmmxxx{als wir euch Dem HERRN aller Schöpfung gleichstellten,\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmmxxxi{und uns haben nur die schwer Verfehlenden irregeführt.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmmxxxii{So gibt es für uns weder Fürbittende\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmmxxxiii{noch einen engen Freund.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmmxxxiv{Würde uns doch nur eine Rückkehr gewährt, dann wären wir von den Mumin!"\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmmxxxv{Gewiß, darin ist doch eine Aya. Und viele von ihnen waren keine Mumin.\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmmxxxvi{Und gewiß, dein HERR ist doch Der Allwürdige, Der Allgnädige!\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmmxxxvii{Die Leute von Nuh bezichtigten den Gesandten der Lüge.\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmmxxxviii{Als ihnen ihr Bruder Nuh sagte: ``Wollt ihr nicht Taqwa gemäß handeln?\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmmxxxix{Ich bin für euch ein treuer Gesandter,\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmmxl{so handelt Taqwa gemäß ALLAH gegenüber und gehorcht mir.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmmxli{Und ich bitte euch dafür um keinen Lohn. Mein Lohn obliegt nur Dem HERRN aller Schöpfung.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmmxlii{So handelt Taqwa gemäß ALLAH gegenüber und gehorcht mir."\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmmxliii{Sie sagten: ``Sollen wir etwa dir gegenüber den Iman verinnerlichen, während dir die Nichtgeachteten folgten?!"\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmmxliv{Er sagte: ``Und was weiß ich von dem, was sie tun?\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmmxlv{Deren Abrechnung obliegt nur meinem HERRN, würdet ihr nur merken!\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmmxlvi{Und ich bin kein Vertreiber der Mumin!\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmmxlvii{Ich bin doch nur ein erläuternder Warner!"\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmmxlviii{Sie sagten: ``Solltest du nicht aufhören, du Nuh! - dann wirst du gewiß von den Gesteinigten sein."\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmmxlix{Er sagte: ``Mein HERR! Meine Leute bezichtigten mich der Lüge,\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmml{so urteile zwischen mir und ihnen eine Urteilung und errette mich und alle, die mit mir von den Mumin sind."\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmmli{Dann erretteten WIR ihn und alle, die mit ihm waren, im beladenen Schiff.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmmlii{Dann ertränkten WIR danach die Übriggebliebenen.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmmliii{Gewiß, darin ist doch eine Aya. Und viele von ihnen waren keine Mumin.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmmliv{Und gewiß, dein HERR ist doch Der Allwürdige, Der Allgnädige!.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmmlv{'Aad hat den Gesandten der Lüge bezichtigt.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmmlvi{Als ihnen ihr Bruder Hud sagte: ``Wollt ihr nicht Taqwa gemäß handeln?\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmmlvii{Ich bin für euch ein treuer Gesandter,\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmmlviii{so handelt Taqwa gemäß ALLAH gegenüber und gehorcht mir.\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmmlix{Und ich bitte euch dafür um keinen Lohn. Mein Lohn obliegt nur Dem HERRN aller Schöpfung.\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmmlx{Errichtet ihr auf jedem Hügel eine Aya, (mit der) ihr Sinnloses treibt?!\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmmlxi{Und ihr macht künstliche Seen, damit ihr vielleicht ewig lebt.\qt@no{129}}%
\qt@newcmd\qurantrans@de@mmmlxii{Und wenn ihr zuschlagt, dann schlagt ihr mitleidlos zu.\qt@no{130}}%
\qt@newcmd\qurantrans@de@mmmlxiii{So handelt Taqwa gemäß ALLAH gegenüber und gehorcht mir.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mmmlxiv{Und handelt Taqwa gemäß Dem gegenüber, Der euch mit dem versorgt, was ihr wisst!\qt@no{132}}%
\qt@newcmd\qurantrans@de@mmmlxv{Er versorgte euch mit An'am, Kindern,\qt@no{133}}%
\qt@newcmd\qurantrans@de@mmmlxvi{Dschannat und Quellen.\qt@no{134}}%
\qt@newcmd\qurantrans@de@mmmlxvii{Gewiß, ich fürchte um euch die Peinigung eines gewaltigen Tages."\qt@no{135}}%
\qt@newcmd\qurantrans@de@mmmlxviii{Sie sagten: ``Uns ist es gleich, ob du uns ermahnst, oder ob du nicht von den Ermahnern wärst.\qt@no{136}}%
\qt@newcmd\qurantrans@de@mmmlxix{Dies ist nichts anderes als die Moral der Früheren!\qt@no{137}}%
\qt@newcmd\qurantrans@de@mmmlxx{Und wir werden sicherlich nicht gepeinigt."\qt@no{138}}%
\qt@newcmd\qurantrans@de@mmmlxxi{Dann bezichtigten sie ihn der Lüge, dann richteten WIR sie zugrunde. Gewiß, darin ist doch eine Aya. Und viele von ihnen waren keineMumin.\qt@no{139}}%
\qt@newcmd\qurantrans@de@mmmlxxii{Und gewiß, dein HERR ist doch Der Allwürdige, Der Allgnädige.\qt@no{140}}%
\qt@newcmd\qurantrans@de@mmmlxxiii{Thamud hat den Gesandten der Lüge bezichtigt.\qt@no{141}}%
\qt@newcmd\qurantrans@de@mmmlxxiv{Als ihnen ihr Bruder Salih sagte: ``Wollt ihr nicht Taqwa gemäß handeln?\qt@no{142}}%
\qt@newcmd\qurantrans@de@mmmlxxv{Ich bin für euch ein treuer Gesandter,\qt@no{143}}%
\qt@newcmd\qurantrans@de@mmmlxxvi{so handelt Taqwa gemäß ALLAH gegenüber und gehorcht mir.\qt@no{144}}%
\qt@newcmd\qurantrans@de@mmmlxxvii{Und ich bitte euch dafür um keinen Lohn. Mein Lohn obliegt nur Dem HERRN aller Schöpfung.\qt@no{145}}%
\qt@newcmd\qurantrans@de@mmmlxxviii{ihr werdet hier gelassen, wo ihr sicher seid,\qt@no{146}}%
\qt@newcmd\qurantrans@de@mmmlxxix{in Dschannat und an Quellen,\qt@no{147}}%
\qt@newcmd\qurantrans@de@mmmlxxx{Gewächse und Dattelpalmen, deren Blütenstand fein ist.\qt@no{148}}%
\qt@newcmd\qurantrans@de@mmmlxxxi{Und ihr meißelt aus den Felsenbergen Häuser in Übermut.\qt@no{149}}%
\qt@newcmd\qurantrans@de@mmmlxxxii{So handelt Taqwa gemäß ALLAH gegenüber und gehorcht mir,\qt@no{150}}%
\qt@newcmd\qurantrans@de@mmmlxxxiii{und gehorcht nicht der Anweisung der Übertretenden,\qt@no{151}}%
\qt@newcmd\qurantrans@de@mmmlxxxiv{die auf Erden Verderben anrichten und nicht gottgefällig Gutes tun."\qt@no{152}}%
\qt@newcmd\qurantrans@de@mmmlxxxv{Sie sagten: ``Du bist nur von den Verzauberten!\qt@no{153}}%
\qt@newcmd\qurantrans@de@mmmlxxxvi{Du bist nichts anderes als ein Mensch wie wir. So bringe eine Aya, wenn du von den Wahrhaftigen bist."\qt@no{154}}%
\qt@newcmd\qurantrans@de@mmmlxxxvii{Er sagte: ``Dies ist eine Kamelstute! Sie hat einen Teil des Wassers und ihr habt einen Teil des Wassers an einem bekannten Tag.\qt@no{155}}%
\qt@newcmd\qurantrans@de@mmmlxxxviii{Und fügt ihr nichts Böses zu, sonst richtet euch die Peinigung eines gewaltigen Tages zugrunde."\qt@no{156}}%
\qt@newcmd\qurantrans@de@mmmlxxxix{Dann verletzten sie ihre Beine, dann standen sie morgens als Reuige auf.\qt@no{157}}%
\qt@newcmd\qurantrans@de@mmmxc{Dann richtet sie die Peinigung zugrunde. Gewiß, darin ist doch eine Aya. Und viele von ihnen waren keine Mumin.\qt@no{158}}%
\qt@newcmd\qurantrans@de@mmmxci{Und gewiß, dein HERR ist doch Der Allwürdige, Der Allgnädige.\qt@no{159}}%
\qt@newcmd\qurantrans@de@mmmxcii{Die Leute von Lut haben den Gesandten der Lüge bezichtigt.\qt@no{160}}%
\qt@newcmd\qurantrans@de@mmmxciii{Als ihnen ihr Bruder Lut sagte: ``Wollt ihr nicht Taqwa gemäß handeln?\qt@no{161}}%
\qt@newcmd\qurantrans@de@mmmxciv{Ich bin für euch ein treuer Gesandter,\qt@no{162}}%
\qt@newcmd\qurantrans@de@mmmxcv{so handelt Taqwa gemäß ALLAH gegenüber und gehorcht mir.\qt@no{163}}%
\qt@newcmd\qurantrans@de@mmmxcvi{Und ich bitte euch dafür um keinen Lohn. Mein Lohn obliegt nur Dem HERRN aller Schöpfung.\qt@no{164}}%
\qt@newcmd\qurantrans@de@mmmxcvii{Verkehrt ihr intim mit den Männlichen von den Menschen,\qt@no{165}}%
\qt@newcmd\qurantrans@de@mmmxcviii{und lasst bei Seite, was euer HERR euch an Partnerwesen erschuf. Nein, sondern ihr seid übertretende Leute!"\qt@no{166}}%
\qt@newcmd\qurantrans@de@mmmxcix{Sie sagten: ``Wenn du nicht aufhörst, Lut! - wirst du von den Vertriebenen sein."\qt@no{167}}%
\qt@newcmd\qurantrans@de@mmmc{Er sagte: ``Ich bin eurer Tat gegenüber von den Verabscheuenden.\qt@no{168}}%
\qt@newcmd\qurantrans@de@mmmci{Mein HERR! Errette mich und meine Familie vor dem, was sie tun."\qt@no{169}}%
\qt@newcmd\qurantrans@de@mmmcii{Dann erretteten WIR ihn und seine Familie, allesamt,\qt@no{170}}%
\qt@newcmd\qurantrans@de@mmmciii{außer einer Alten, sie war von den Untergehenden.\qt@no{171}}%
\qt@newcmd\qurantrans@de@mmmciv{Dann vernichteten WIR die anderen.\qt@no{172}}%
\qt@newcmd\qurantrans@de@mmmcv{Und WIR ließen sie mit Regen beregnen. Also schlimm ist der Regen der Gewarnten.\qt@no{173}}%
\qt@newcmd\qurantrans@de@mmmcvi{Gewiß, darin ist doch eine Aya. Und viele von ihnen waren keine Mumin.\qt@no{174}}%
\qt@newcmd\qurantrans@de@mmmcvii{Und gewiß, dein HERR ist doch Der Allwürdige, Der Allgnädige.\qt@no{175}}%
\qt@newcmd\qurantrans@de@mmmcviii{Die Weggenossen von Al-aika haben den Gesandten der Lüge bezichtigt.\qt@no{176}}%
\qt@newcmd\qurantrans@de@mmmcix{Als ihnen Schu'aib sagte: ``Wollt ihr nicht Taqwa gemäß handeln?\qt@no{177}}%
\qt@newcmd\qurantrans@de@mmmcx{Ich bin für euch ein treuer Gesandter,\qt@no{178}}%
\qt@newcmd\qurantrans@de@mmmcxi{so handelt Taqwa gemäß ALLAH gegenüber und gehorcht mir.\qt@no{179}}%
\qt@newcmd\qurantrans@de@mmmcxii{Und ich bitte euch dafür um keinen Lohn. Mein Lohn obliegt nur Dem HERRN aller Schöpfung.\qt@no{180}}%
\qt@newcmd\qurantrans@de@mmmcxiii{Und gebt volles Maß, und seid nicht von den Abziehenden!\qt@no{181}}%
\qt@newcmd\qurantrans@de@mmmcxiv{Und wägt mit der richtigen Waage!\qt@no{182}}%
\qt@newcmd\qurantrans@de@mmmcxv{Und gebt den Menschen das ihnen Zustehende nicht inkomplett und übertreibt nicht auf der Erde als Verderben-Anrichtende!\qt@no{183}}%
\qt@newcmd\qurantrans@de@mmmcxvi{Und handelt Taqwa gemäß Dem gegenüber, Der euch und die erste Schöpfung erschuf."\qt@no{184}}%
\qt@newcmd\qurantrans@de@mmmcxvii{Sie sagten: ``Du bist nur von den Verzauberten!\qt@no{185}}%
\qt@newcmd\qurantrans@de@mmmcxviii{Und du bist nicht anderes als ein Mensch wie wir. Auch wir glauben, daß du doch von den Lügnern bist!\qt@no{186}}%
\qt@newcmd\qurantrans@de@mmmcxix{Also laß über uns Stücke aus dem Himmel fallen, solltest du von den Wahrhaftigen sein."\qt@no{187}}%
\qt@newcmd\qurantrans@de@mmmcxx{Er sagte: ``Mein HERR weiß besser Bescheid über das, was ihr tut."\qt@no{188}}%
\qt@newcmd\qurantrans@de@mmmcxxi{Dann bezichtigten sie ihn der Lüge, dann richtete sie die Peinigung des Schirm-Tages. Gewiß, es war eine Peinigung eines gewaltigen Tages.\qt@no{189}}%
\qt@newcmd\qurantrans@de@mmmcxxii{Gewiß, darin ist doch eine Aya. Und viele von ihnen waren keine Mumin.\qt@no{190}}%
\qt@newcmd\qurantrans@de@mmmcxxiii{Und gewiß, dein HERR ist doch Der Allwürdige, Der Allgnädige.\qt@no{191}}%
\qt@newcmd\qurantrans@de@mmmcxxiv{Und gewiß, er (der Quran) ist eine Hinabsendung Des HERRN aller Schöpfung.\qt@no{192}}%
\qt@newcmd\qurantrans@de@mmmcxxv{Mit ihm kam Ar-ruhul-amin herunter\qt@no{193}}%
\qt@newcmd\qurantrans@de@mmmcxxvi{auf dein Herz, damit du von den Warnenden wirst,\qt@no{194}}%
\qt@newcmd\qurantrans@de@mmmcxxvii{mit einer erläuternden arabischen Sprache.\qt@no{195}}%
\qt@newcmd\qurantrans@de@mmmcxxviii{Und gewiß, er ist doch in den Schriften der Früheren.\qt@no{196}}%
\qt@newcmd\qurantrans@de@mmmcxxix{Gab es für sie etwa keine Aya, daß die Gelehrten der Kinder Israils ihn kennen?!\qt@no{197}}%
\qt@newcmd\qurantrans@de@mmmcxxx{Und hätten WIR ihn manchen der Nichtaraber hinabgesandt,\qt@no{198}}%
\qt@newcmd\qurantrans@de@mmmcxxxi{und hätte er ihnen ihn vorgetragen, würden sie keineMumin an ihn sein.\qt@no{199}}%
\qt@newcmd\qurantrans@de@mmmcxxxii{Solcherart ließen WIR ihn in die Herzen der schwer Verfehlenden eindringen,\qt@no{200}}%
\qt@newcmd\qurantrans@de@mmmcxxxiii{sie verinnerlichen den Iman an ihn nicht, bis sie die qualvolle Peinigung erfahren.\qt@no{201}}%
\qt@newcmd\qurantrans@de@mmmcxxxiv{Dann kommt sie zu ihnen plötzlich, während sie es nicht bemerken.\qt@no{202}}%
\qt@newcmd\qurantrans@de@mmmcxxxv{Dann sagen sie: ``Wird uns ein Aufschub gewährt?!"\qt@no{203}}%
\qt@newcmd\qurantrans@de@mmmcxxxvi{Wollen sie etwa Eile mit Unserer Peinigung fordern?!\qt@no{204}}%
\qt@newcmd\qurantrans@de@mmmcxxxvii{Wie siehst du es, würden WIR ihnen für Jahre Genuß gewähren,\qt@no{205}}%
\qt@newcmd\qurantrans@de@mmmcxxxviii{dann käme zu ihnen das, was ihnen angedroht wurde,\qt@no{206}}%
\qt@newcmd\qurantrans@de@mmmcxxxix{würde ihnen dann nützen, was sie zu genießen pflegten?!\qt@no{207}}%
\qt@newcmd\qurantrans@de@mmmcxl{Und WIR richteten keine Ortschaft, ohne daß es für sie Warner gab,\qt@no{208}}%
\qt@newcmd\qurantrans@de@mmmcxli{als Erinnerung. Und WIR waren nie Ungerechte!\qt@no{209}}%
\qt@newcmd\qurantrans@de@mmmcxlii{Und mit ihm kamen keine Satane herunter!\qt@no{210}}%
\qt@newcmd\qurantrans@de@mmmcxliii{Und es steht ihnen nicht zu und sie vermögen es nicht.\qt@no{211}}%
\qt@newcmd\qurantrans@de@mmmcxliv{Gewiß, sie sind vom Hören doch ausgeschlossen.\qt@no{212}}%
\qt@newcmd\qurantrans@de@mmmcxlv{Also richte deine Bittgebete an keinen anderen Gott neben ALLAH, sonst wirst du von den Gepeinigten.\qt@no{213}}%
\qt@newcmd\qurantrans@de@mmmcxlvi{Und warne deine nahen Verwandten!\qt@no{214}}%
\qt@newcmd\qurantrans@de@mmmcxlvii{Und behandle diejenigen der Mumin, die dir gefolgt sind, milde!\qt@no{215}}%
\qt@newcmd\qurantrans@de@mmmcxlviii{Und wenn sie sich dir widersetzten, dann sag: ``Ich sage mich los von dem, was ihr tut."\qt@no{216}}%
\qt@newcmd\qurantrans@de@mmmcxlix{Und übe Tawakkul Dem Allbezwingbaren, Dem Allgnädigen gegenüber!\qt@no{217}}%
\qt@newcmd\qurantrans@de@mmmcl{Derjenige, Der dich sieht, wenn du Qiyam vollziehst,\qt@no{218}}%
\qt@newcmd\qurantrans@de@mmmcli{sowie dein Wandeln unter den Sudschud-Vollziehenden.\qt@no{219}}%
\qt@newcmd\qurantrans@de@mmmclii{Gewiß, ER ist Der Allhörende, Der Allwissende!\qt@no{220}}%
\qt@newcmd\qurantrans@de@mmmcliii{Soll ich euch mitteilen, auf wen die Satane herunterkommen?\qt@no{221}}%
\qt@newcmd\qurantrans@de@mmmcliv{Sie kommen auf jeden äußerst verfehlenden Lügner herunter.\qt@no{222}}%
\qt@newcmd\qurantrans@de@mmmclv{Sie geben vom Gehörten weiter, und die meisten von ihnen sind Lügner.\qt@no{223}}%
\qt@newcmd\qurantrans@de@mmmclvi{Und die Dichter, denen folgen die Fehlgehenden.\qt@no{224}}%
\qt@newcmd\qurantrans@de@mmmclvii{Hast du etwa nicht gesehen, daß sie sich in jedem Tal verlieben,\qt@no{225}}%
\qt@newcmd\qurantrans@de@mmmclviii{und daß sie das sagen, was sie nicht tun?\qt@no{226}}%
\qt@newcmd\qurantrans@de@mmmclix{- außer denjenigen, die den Iman verinnerlichten, gottgefällig Gutes taten, ALLAHs öfters gedenken und kämpften, nachdem ihnen Unrecht zugefügt wurde. Und diejenigen, die Unrecht begingen, werden noch wissen, zu welcher Rückkehr sie zurückkehren werden.\qt@no{227}}%
\qt@newcmd\qurantrans@de@mmmclx{\basmalahde Ta-sin. Diese sind die Ayat des Quran und eine deutliche Schrift,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmclxi{eine Rechtleitung und frohe Botschaft für die Mumin,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmclxii{die das rituelle Gebet ordnungsgemäß verrichten und die Zakat entrichten, und sie haben Gewißheit an das Jenseits.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmclxiii{Gewiß, denjenigen, die den Iman an das Jenseits nicht verinnerlichen, haben WIR ihre Taten schön erscheinen lassen, so sind sie verwirrt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmclxiv{Diese sind diejenigen, für die das Schlimme der Peinigung bestimmt ist, und sie sind im Jenseits die wirklichen Verlierer.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmclxv{Und gewiß, du empfängst doch den Quran von Einem Allweisen, Allwissenden.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmclxvi{Und erinnere daran), als Musa seiner Familie sagte: ``Ich sah Feuer. Ich werde euch von dort eine Mitteilung bringen, oder ich bringe euch eine Fackel davon, damit ihr euch wärmt."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmclxvii{Und als er dort ankam, wurde gerufen: ``Baraka wurde verliehen dem, das im Feuer ist, sowie dem um es herum. Und subhanallah, Der HERR aller Schöpfung.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmclxviii{Musa! Gewiß, ICH bin ALLAH, Der Allwürdige, Der Allweise.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmclxix{Und wirf deinen Stock!" Und als er ihn sich schlängeln sah, als ob er eine flinke Schlange wäre, wandte er sich flüchtend ab und kehrte nicht mehr um: ``Musa! Fürchte dich nicht! Gewiß, bei Mir fürchten sich die Gesandten nicht,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmclxx{außer demjenigen, der Unrecht beging, dann aber Gutes nach dem Schlechten tauschte, dann bin ich doch allvergebend, allgnädig.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmclxxi{Und stecke deine Hand in den Ausschnitt deines (Kleides), kommt sie hell unversehrt heraus. (Geh!) mit neuen Ayat zu Pharao und seinen Leuten. Sie waren fisq-betreibende Leute."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmclxxii{Und als zu ihnen Unsere Einblick gewährenden Ayat kamen, sagten sie: ``Dies ist eine eindeutige Magie."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmclxxiii{Und sie verleugneten sie, während ihre Seelen daran Gewißheit fanden - aus Übertretung und Arroganz. Also siehe, wie das Anschließende von den Verderben-Anrichtenden war.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmclxxiv{Und gewiß, bereits ließen WIR Dawud und Sulaiman Wissen zuteil werden. Und beide sagten: ``Alhamdulillah: Alles Lob gebührt ALLAH, Der uns vor vielen Seiner Diener begünstigt hat."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmclxxv{Und Sulaiman beerbte Dawud. Und er sagte: ``Ihr Menschen! Uns wurde die Sprache der Vögel beigebracht und von allen Dingen zuteil. Gewiß, dies ist doch die eindeutige Gunst."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmclxxvi{Und für Sulaiman wurden seine Soldaten von den Dschinn, den Menschen und den Vögeln versammelt, so werden sie geordnet beisammengehalten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmclxxvii{Als sie dann beim Tal der Ameisen ankamen, sagte eine Ameise: ``Ihr Ameisen! Geht in eure Nester hinein, damit Sulaiman und seine Soldaten euch nicht zertreten, während sie es nicht merken."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmclxxviii{Dann lächelte er doch wegen ihrer Aussage und sagte: ``Mein HERR! Gib mir ein, daß ich deiner Wohltat, die DU mir und meinen Eltern gewährt hast, Dankbarkeit erweise, und daß ich gottgefällig Gutes tue, woran DU Wohlgefallen findest. Und laß mich mit Deiner Gnade unter Deinen gottgefällig guttuenden Dienern sein!"\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmclxxix{Und er inspizierte die Vögel, dann sagte er: ``Weshalb sehe ich den Wiedehopf nicht? Oder ist er etwa von den Abwesenden?\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmclxxx{Gewiß, ich werde ihn einer qualvollen Peinigung unterziehen oder ihn schlachten, oder er legt mir eine klare Entschuldigung vor."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmclxxxi{Dann blieb er nicht lang weg, dann sagte er: ``Ich kundschaftete aus, was du nicht auskundschaftest, und ich brachte dir aus Sabaa eine gewiße Mitteilung:\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmclxxxii{Ich fand eine Frau sie beherrschen, und ihr wurde von allem zuteil, und sie verfügt über einen gewaltigen Thron.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmclxxxiii{Ich fand sie und ihre Leute Sudschud der Sonne anstelle von ALLAH vollziehen, und der Satan hat ihnen ihre Taten schön erscheinen lassen, dann brachte er sie ab vom eigentlichen Weg, so finden sie keine Rechtleitung,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmclxxxiv{damit sie kein Sudschud für ALLAH vollziehen, Der das Verborgene in den Himmeln und auf Erden hervorbringt, und ER weiß, was ihr verbergt und was ihr offenlegt.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmclxxxv{ER ist ALLAH, es gibt keine Gottheit außer Ihm, Der HERR vom gewaltigen Al'ahrsch."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmclxxxvi{Er sagte: ``Wir werden sehen, ob du wahrhaftig oder von den Lügnern warst.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmclxxxvii{Fliege mit diesem meinem Schreiben (dorthin), wirf es über ihnen ab, dann wende dich von ihnen ab, dann warte ab, was sie entgegnen."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmclxxxviii{Sie sagte: ``Ihr Entscheidungsträger! Mir wurde ein würdevolles Schreiben zugeworfen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmclxxxix{Gewiß, es ist von Sulaiman. Und es (beginnt) doch mit Bismil-lahir-rahmanir-rahim:\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmcxc{Erhebt euch nicht über mich und kommt zu mir als Muslime!"\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmcxci{Sie sagte: ``Ihr Entscheidungsträger! Gebt mir eine Fatwa über meine Angelegenheit! Ich werde nie eine Entscheidung treffen, bis ihr dabei anwesend seid."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmcxcii{Sie sagten: ``Wir verfügen über Macht und über starke Kampfmoral. Also das Befehlen ist deins, so überlege dir, was du anordnest."\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmcxciii{Sie sagte: ``Gewiß, wenn die Könige eine Ortschaft einnehmen, richten sie in ihr Verderben an und machen die Einflußreichen ihrer Bewohner zu Erniedrigten. Und solcherart machen sie.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmcxciv{Und ich werde ihnen ein Geschenk zukommen lassen, dann werde ich sehen, womit die Entsandten zurückkommen."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmcxcv{Und als (der Entsandte) bei Sulaiman ankam, sagte er (Sulaiman): ``Bestärkt ihr mich mit Vermögenswerten?! Was ALLAH mir gewährte, ist besser als das, was ER euch gewährte! Nein, sondern ihr seid über euer Geschenk erfreut.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmcxcvi{Kehre zu ihnen zurück! Gewiß, wir werden zu ihnen mit Soldaten kommen, die sie nicht entgegnen können. Und wir werden sie doch daraus erniedrigt vertreiben, während sie unterwürfig sind."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmcxcvii{Er sagte: ``Ihr Entscheidungsträger! Wer von euch bringt mir ihren Thron, bevor sie zu mir als Muslime kommen?"\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmcxcviii{Ein 'Ifrit von den Dschinn sagte: ``Ich bringe ihn dir, bevor du von deinem Platz aufstehst! Und ich bin dafür gewiß mächtig und treu."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmcxcix{Derjenige, der über Wissen von der Schrift verfügte, sagte: ``Ich bringe ihn dir, bevor deine Wimper zuckt." Und als er ihn bei sich aufgestellt sah, sagte er: ``Dies ist etwas von der Gunst meines HERRN, damit ER mich prüft, ob ich mich dankbar oder undankbar erweise. Und wer sich dankbar erweist, erweist er sich doch dankbar nur zu seinem Nutzen. Und wer Kufr betreibt, so ist mein HERR absolut autark, allgroßzügig."\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmcc{Er sagte: ``Macht ihr ihren Thron unkenntlich, damit wir sehen, ob sie ihn erkennt, oder ob sie von denjenigen ist, die keine Rechtleitung finden?"\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmcci{Und als sie kam, wurde ihr gesagt: ``Ähnelt dein Thron diesem?" Sie sagte: ``Als ob er der Gleiche wäre!" Und uns wurde vor ihr das Wissen zuteil und wir waren Muslime.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmccii{Und das, dem sie anstelle von ALLAH zu dienen pflegte, hielt sie zurück. Gewiß, sie gehörte zu Kafir-Leuten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmcciii{Ihr wurde dann gesagt: ``Tritt ein in den Innenhof!" Und als sie ihn sah, vermutete sie ihn als Gewässer und enthüllte ihre Beine. Er sagte: ``Es ist ein aus Glas geglätteter Innenhof." Sie sagte: ``Mein HERR! Gewiß, ich fügte mir selbst Unrecht zu. Und ich nahm den Islam an mit Sulaiman ALLAH gegenüber, Dem HERRN aller Schöpfung."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmcciv{Und gewiß, bereits entsandten WIR zu Thamud ihren Bruder Salih: ``Dient ALLAH!" Dann wurden sie aber zwei Gruppen, die stritten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmccv{Er sagte: ``Meine Leute! Weshalb fordert ihr zur Eile mit dem Bösen vor dem Guten auf?! Würdet ihr doch ALLAH um Vergebung bitten, damit euch Gnade erwiesen wird!"\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmccvi{Sie sagten: ``Wir prophezeien etwas Unheilvolles mit dir und mit allen, die mit dir sind." Er sagte: ``Das von euch (prophezeite) Unheilvolle kommt von ALLAH. Nein, sondern ihr seid Leute, die geprüft werden."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmccvii{Und in der Stadt waren neun Menschen, die im Lande Verderben anrichteten und nichts gottgefällig Gutes taten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmccviii{Sie sagten: ``Legt einen Schwur bei ALLAH ab, daß wir ihn und seine Familie nachts überfallen. Dann sagen doch wir seinem Wali: ``Wir waren bei der Ermordung seiner Familie nicht anwesend, und gewiß, wir sind doch wahrhaftig!"\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmccix{Und sie planten eine Intrige und WIR entgegneten ihre Intrige, während sie nicht merkten.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmccx{Also siehe, wie das Anschließende von ihrer Intrige war, daß WIR sie und ihre Leute, allesamt vernichteten.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmccxi{Und diese ihre Wohnstätten sind leer wegen dem, womit sie Unrecht begingen. Gewiß, darin ist doch eine Aya für Leute, die wissen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmccxii{Und WIR erretteten diejenigen, die den Iman verinnerlichten und Taqwa gemäß zu handeln pflegten.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmccxiii{Auch Lut, als er seinen Leuten sagte: ``Begeht ihr das Abscheuliche, während ihr Einblick habt?\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmccxiv{Verkehrt ihr wirklich intim mit den Männern aus Begierde anstelle mit Frauen? Nein, sondern ihr seid Leute, die unwissend sind."\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmccxv{Und es gab keine Antwort von seinen Leuten, außer daß sie sagten: ``Vertreibt Luts Familie aus eurer Ortschaft! Gewiß, sie sind Leute, die sich rein halten."\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmccxvi{Dann erretteten WIR ihn und seine Familie außer seiner Ehefrau, sie bestimmten WIR unter den Untergehenden.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmccxvii{Und WIR ließen sie mit Regen beregnen. Also wirklich schlimm ist der Regen der Gewarnten.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmccxviii{Sag: ``Alhamdulillah: Alles Lob gebührt ALLAH! Und Salam sei über Seine Diener, die ER auserwählte." Ist ALLAH besser oder das, womit sie Schirk betreiben?!\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmccxix{Ist Derjenige, Der die Himmel und die Erde erschuf und für euch vom Himmel Wasser fallen ließ, mit dem WIR erfreuliche Gärten wachsen ließen - es gebührte euch nie, deren Bäume wachsen zu lassen, (besser oder euer Schirk)?! Gibt es etwa eine Gottheit neben ALLAH?! Nein, sondern sie sind Leute, die (vom Wahren) abweichen.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmccxx{Ist Derjenige, Der die Erde als Aufenthaltsort machte, durch sie Flüsse fließen ließ, für sie Festigende machte und zwischen beiden Meeren eine Trennung machte, (besser oder euer Schirk)?! Gibt es etwa eine Gottheit neben ALLAH?! Nein, sondern die meisten von ihnen wissen nicht!\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmccxxi{Ist Derjenige, Der den in Not Geratenen erhört, wenn er an Ihn Bittgebete richtet, und Der von ihm das Boshafte wegnimmt und euch die Nachfolge auf Erden antreten ließ, (besser oder euer Schirk)?! Gibt es etwa eine Gottheit neben ALLAH?! Nein, sondern es ist selten, daß ihr euch besinnt.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmccxxii{Ist Derjenige, Der euch in den Finsternissen vom Lande und vom Meer rechtleitet, und Der die Winde als frohe Botschaft vor Seiner Gnade (vor dem Regen) schickt, (besser oder euer Schirk)?! Gibt es etwa eine Gottheit neben ALLAH?! Immer allerhabener ist ALLAH über das, was sie an Schirk betreiben!\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmccxxiii{Ist Derjenige, Der das Erschaffen beginnt, dann es wiederholt, und Der euch Rizq von dem Himmel und der Erde gewährt, (besser oder euer Schirk)?! Gibt es etwa eine Gottheit neben ALLAH?! Sag: ``Bringt euren Beweis, solltet ihr wahrhaftig sein."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmccxxiv{Sag: ``Diejenigen in den Himmeln und auf Erden kennen das Verborgene nicht, nur ALLAH (kennt es)!" Und sie merken nicht, wann sie erweckt werden.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmccxxv{Nein, sondern ihr Wissen erfaßte auch noch das Jenseits?! Nein, sondern sie sind darüber im Zweifel! Nein, sondern sie sind ihm (dem Jenseits) gegenüber blind!\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmccxxvi{Und diejenigen, die Kufr betrieben haben, sagten: ``Wenn wir auch noch zu Erde wurden, so wie unsere Ahnen, werden wir etwa herausgebracht?!\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmccxxvii{Gewiß, bereits wurde uns dies angedroht sowie unseren Ahnen vorher. Dies ist nichts anderes außer Legenden der Früheren."\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmccxxviii{Sag: ``Zieht auf der Erde umher, dann seht, wie das Anschließende von den schwer Verfehlenden war."\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmccxxix{Und trauere ihnen nicht nach und sei nicht bedrückt wegen dem, was sie an List planen.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmccxxx{Und sie sagen: ``Wann erfüllt sich diese Androhung, solltet ihr wahrhaftig sein?"\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmccxxxi{Sag: ``Vielleicht nähert sich euch etwas von dem, womit ihr zur Eile fordert?"\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmccxxxii{Und gewiß, dein HERR erweist den Menschen Gunst! Doch die meisten erweisen sich nicht dankbar.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmccxxxiii{Und gewiß, dein HERR weiß doch, was ihre Brüste verbergen, und was sie offenlegen.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmccxxxiv{Und es gibt nichts Verborgenes im Himmel und auf Erden, ohne daß es in einer deutlichen Schrift ist.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmccxxxv{Gewiß, dieser Quran berichtet den Kindern Israils das Meiste von dem, worin sie uneins sind.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmccxxxvi{Und gewiß, er ist zweifelsohne eine Rechtleitung und Gnade für die Mumin.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmccxxxvii{Gewiß, dein HERR richtet zwischen ihnen nach Seiner Weisheit. Und ER ist Der Allwürdige, Der Allwissende.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmccxxxviii{So übe Tawakkul ALLAH gegenüber! Du verfügst zweifelsohne über die eindeutige Wahrheit.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmccxxxix{Gewiß, du kannst die Toten nicht hören lassen. Und du kannst die Tauben den Ruf nicht hören lassen, wenn sie sich flüchtend abwenden.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmccxl{Und du kannst die Blinden aus ihrem Irren nicht rechtleiten. Du kannst nur diejenigen hören lassen, die den Iman an Unsere Ayat verinnerlichen, die dann Muslime sind.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmccxli{Und wenn die Bestimmung gegen sie ergeht, werden WIR ihnen ein sich bewegendes Lebewesen aus der Erde hervorbringen, das zu ihnen spricht, daß die Menschen über Unsere Ayat keine Gewißheit zu verinnerlichen pflegten.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmccxlii{Und am Tag, wenn WIR von jeder Umma eine Gruppe von denen, die Unsere Ayat der Lüge bezichtigten, versammeln, so werden sie geordnet beisammengehalten.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmccxliii{Wenn sie dann kommen, sagt ER: ``Habt ihr Meine Ayat der Lüge bezichtigt und über sie kein Wissen erfaßt, oder was pflegtet ihr zu tun?!\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmccxliv{Und die Bestimmung ist gegen sie ergangen wegen dem, was sie an Unrecht taten, so geben sie von sich keinen Laut.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmccxlv{Sahen sie etwa nicht, daß WIR die Nacht machten, damit sie sich darin ausruhen, sowie den Tag hell?! Gewiß, darin sind doch Ayat für Leute, die den Iman verinnerlichen.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmccxlvi{Und am Tag, wenn in As-sur geblasen wird, werden dann alle, die in den Himmeln und auf Erden sind, sich erschrecken, außer denjenigen, die ALLAH will. Und alle kommen zu Ihm sich fügend.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmccxlvii{Und du siehst die Felsenberge, du denkst, sie stünden fest, während sie ziehen wie das Ziehen der Wolken. Es ist das Bewerkstelligen ALLAHs, Der alles bestens kann. Gewiß, ER ist allkundig dessen, was ihr macht.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmccxlviii{Wer mit der guten Tat kommt, für ihn ist Besseres als dies bestimmt, und sie werden vor dem Erschrecken jenes Tages sicher sein.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmccxlix{Und wer mit der schlechten Tat kommt, diese werden auf ihre Gesichter ins Feuer geworfen. Wird euch anderes vergolten, als das, was ihr zu tun pflegtet?!\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmccl{Ich wurde nur angewiesen, daß ich Dem HERRN dieser Ortschaft diene, die ER für haram erklärte, und Ihm gehört alles. Und ich wurde angewiesen, von den Muslimen zu sein,\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmccli{und daß ich den Quran vortrage. Wer dann Rechtleitung findet, der findet Rechtleitung nur für sich selbst, und wer irregeht, dann sag: ``Ich bin nur von den Warnern!"\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmcclii{Und sag: ``Alhamdulillah: Alles Lob gebührt ALLAH! ER wird euch Seine Ayat zeigen, so werdet ihr sie kennen." Und dein HERR ist nicht achtlos dem gegenüber, was ihr tut.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmccliii{\basmalahde Ta-sin-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmccliv{Diese sind die Ayat der deutlichen Schrift.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcclv{WIR tragen dir vor von der Begebenheit von Musa und Pharao wahrheitsgemäß für Leute, die den Iman verinnerlichen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcclvi{Gewiß, Pharao erhob sich in Arroganz im Lande und machte dessen Bewohner zu Parteien. Er schwächte eine Gruppe von ihnen ab. Er ließ ihre Söhne abschlachten und ihre Frauen am Leben bleiben. Gewiß, er war von den Verderben-Anrichtenden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcclvii{Und WIR wollen denjenigen, die im Lande abgeschwächt wurden, Gnade erweisen, aus ihnen Imame machen, zu den Erben machen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcclviii{im Lande festigen und Pharao, Haman und ihren Soldaten von ihnen das zeigen, vor dem sie sich in Acht zu nehmen pflegten.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcclix{Und WIR ließen der Mutter von Musa Wahy zuteil werden: ``Stille ihn! Und solltest du dich um ihn fürchten, dann lege ihn in den Fluß, und fürchte dich nicht und sei nicht traurig! Gewiß, WIR werden ihn dir zurückbringen und werden ihn von den Gesandten machen."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcclx{Dann las ihn Pharaos Familie auf, damit er für sie ein Feind und ein Grund zur Besorgnis wird. Gewiß, Pharao, Haman und ihre Soldaten waren Verfehlende.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcclxi{Und die Frau von Pharao sagte: ``Es ist eine Freude für mich und für dich. Tötet ihn nicht, vielleicht nützt er uns oder wir nehmen ihn zum Sohn." Und sie merkten nichts.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcclxii{Und das Herz der Mutter von Musa wurde leer. Sie hätte ihn beinahe verraten, hätten WIR ihrem Herzen keine Kraft gewährt, damit sie von den Mumin wird.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcclxiii{Und sie sagte zu seiner Schwester: ``Folge seinen Spuren!  Dann sah sie ihn von der Seite, während sie nichts merkten.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcclxiv{Und WIR machten für ihn die Stillenden vorher haram. Dann sagte sie (die Schwester): ``Soll ich euch zu einer Familie führen, die ihn für euch betreut? Und sie werden auf ihn aufpassen."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcclxv{So brachten WIR ihn zu seiner Mutter zurück, damit sie sich erfreut und nicht traurig wird und damit sie zweifelsohne weiß, daß ALLAHs Versprechen wahr ist. Doch die meisten von ihnen wissen es nicht.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcclxvi{Und nachdem er erwachsen und reif geworden ist, ließen WIR ihm Weisheit und Wissen zuteil werden. Und solcherart vergelten WIR es den Muhsin.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcclxvii{Und (eines Tages) trat er in die Stadt, ohne von ihren Einwohnern bemerkt zu werden, dann fand er zwei Männer, die gegeneinander kämpften, dieser gehörte seiner Partei an und jener war von seinen Feinden, dann bat ihn derjenige, der seiner Partei angehörte, um Hilfe gegen denjenigen, der von seinen Feinden war, dann schlug Musa mit der Faust auf ihn, dann tötete er ihn. Er sagte: ``Dies ist vom Werk des Satans, gewiß, er ist ein verleitender, entschiedener Feind."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcclxviii{Er sagte: ``Mein HERR! Gewiß, ich tat mir selbst Unrecht an, so vergib mir!" Dann vergab ER ihm. Gewiß, ER ist Der Allvergebende, Der Allgnädige.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcclxix{Er sagte: ``Mein HERR! Bei dem, was du mir an Wohltaten erwiesen hast, ich werde niemals ein Rückenstärkender für die schwer Verfehlenden sein."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcclxx{Dann wurde er in der Stadt angsterfüllt und beobachtete, da hat derjenige, der ihn tags zuvor um Beistand bat, laut um Hilfe gerufen. Musa sagte ihm: ``Gewiß, du bist ein eindeutiger Fehlgehender."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcclxxi{Und als er demjenigen, der ein Feind beider war, Gewalt antun wollte, sagte er: ``Musa! Willst du mich erschlagen, wie du einen Menschen gestern erschlugst? Du willst nur ein rücksichtsloser Gewalttäter im Lande sein, und du willst keineswegs von den gottgefällig Guttuenden sein."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcclxxii{Da kam ein Mann vom anderen Ende der Stadt angelaufen, er sagte: ``Musa! Gewiß, die Entscheidungsträger beraten sich gegenseitig wegen dir, um dich zu töten, also geh fort! Ich bin für dich doch einer von den guten Ratgebern."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcclxxiii{Dann ging er von ihr heraus angsterfüllt und beobachtete, er sagte: ``Mein HERR! Errette mich vor den unrecht-begehenden Leuten!"\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcclxxiv{Und als er auf Madyan zusteuerte, sagte er: ``Hoffentlich wird mein HERR mich auf den richtigen Weg rechtleiten."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcclxxv{Und als er beim Wasser von Madyan ankam, fand er da eine Umma von Menschen, die (ihre Tiere) tränkten, auch fand er da unter ihnen zwei Frauen, die (ihre Tiere) zurückhielten. Er sagte: ``Was ist euer Bestreben?" Beide sagten: ``Wir tränken nicht, bis die Hirten zurückgezogen sind. Und unser Vater ist ein Greis."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcclxxvi{Dann tränkte er für beide (ihre Tiere), dann wandte er sich ab (und ging) in den Schatten, dann sagte er: ``Mein HERR! Gewiß, ich bin dessen bedürftig, was DU mir an Gutem zuteil werden läßt."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcclxxvii{Dann kam eine von den beiden (Frauen), sie schritt in Schamhaftigkeit. Sie sagte: ``Ja! Mein Vater lädt dich ein, um dich doch dafür zu belohnen, daß du für uns die Tiere getränkt hast." Und nachdem er zu ihm gekommen war und ihm über die Begebenheiten berichtet hatte, sagte er: ``Fürchte dich nicht, du bist den unrechtbegehenden Leuten entkommen."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcclxxviii{Eine von den Beiden sagte: ``Mein Vater! Nimm ihn in deinen Dienst! Gewiß, der Beste, den du in Dienst nimmst, ist der Kraftvolle, der Anzuvertrauende."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcclxxix{Er sagte: ``Ich will dich mit einer von diesen meinen beiden Töchter verheiraten unter der Bedingung, daß du dich acht Jahre in meinen Dienst stellst. Und wenn du zehn (Jahre) vervollständigst, dann ist dies dir überlassen. Und ich will es dir nicht schwer machen. Du wirst mich inscha-allah von den gottgefällig Guttuenden finden."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcclxxx{Er sagte: ``Dies ist (abgemacht) zwischen mir und dir. Und egal welche der beiden Fristen ich erfülle, so habe ich unter keiner Übertretung zu leiden. Und ALLAH ist auf das, was wir sagen, Ein Wakil."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcclxxxi{Und nachdem Musa die Frist erfüllt hatte und mit seiner Familie nachts gezogen war, sah er an der Seite des Tur-Bergs ein Feuer. Er sagte zu seiner Familie: ``Bleibt hier! Ich sah doch ein Feuer. Vielleicht bringe ich euch davon eine Mitteilung oder eine Fackel vom Feuer, vielleicht wärmt ihr euch."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmcclxxxii{Und als er es erreichte, wurde vom Ufer des rechten Tales in der von Baraka erfüllten Landschaft aus dem Baum gerufen: ``Musa! Ja, ICH bin ALLAH, Der HERR aller Schöpfung.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmcclxxxiii{Und wirf deinen Stock! Und als er ihn sich schlängeln sah, als ob er eine flinke Schlange wäre, wandte er sich flüchtend ab und kehrte nicht mehr um: ``Musa! Kehre um und fürchte dich nicht, du bist doch einer der Sicheren.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmcclxxxiv{Führe deine Hand in deinen Kleidausschnitt hinein, kommt sie hell unbeschadet heraus, und ziehe deinen Arm an dich beim Fürchten heran! Also, diese sind zwei Beweise von deinem HERRN für Pharao und seine Entscheidungsträger. Sie sind doch fisq-betreibende Leute."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmcclxxxv{Er sagte: ``Mein HERR! Ich tötete doch von ihnen einen Menschen, so fürchte ich, daß sie mich töten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmcclxxxvi{Und mein Bruder Harun ist zungenfertiger als ich, so mache ihn zum Gesandten mit mir als Helfer, der mich glaubhaft macht. Ich fürchte, daß sie mich der Lüge bezichtigen."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmcclxxxvii{ER sagte: ``WIR werden dich mit deinem Bruder stärken und euch einen Beweis geben, so werden sie nicht in eure Nähe kommen. Mit Unseren Ayat seid ihr beide und diejenigen, die euch beiden folgen, die Sieger."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmcclxxxviii{Und als Musa zu ihnen mit Unseren erklärenden Ayat kam, sagten sie: ``Dies ist nichts als erfundene Magie, und wir hörten nie so etwas bei unseren ersten Vorfahren:"\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmcclxxxix{Und Musa sagte: ``Mein HERR weiß besser Bescheid über denjenigen, der mit der Rechtleitung von Ihm kam und wem das Anschließende vom Diesseits gehört. Gewiß, die Unrecht-Begehenden werden nie erfolgreich werden."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmccxc{Und Pharao sagte: ``Ihr Entscheidungsträger! Ich wußte für euch von keinem anderen Gott außer mir! Also Haman behandle meinetwegen die Erde mit Feuer und lasse für mich einen Turm errichten! Vielleicht schaue ich dann auf Musas Gott, und ich halte ihn zweifelsohne für einen von den Lügnern."\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmccxci{Und er erhob sich mit seinen Soldaten in Arroganz im Lande zu Unrecht, und sie dachten, daß sie zu Uns keineswegs zurückgebracht werden.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmccxcii{Dann richteten WIR ihn und seine Soldaten zugrunde, dann warfen WIR sie in den Fluß. Also siehe, wie das Anschließende von den Unrecht-Begehenden war.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmccxciii{Und WIR machten sie zu Imamen, die zum Feuer einladen. Und am Tag der Auferstehung wird ihnen nicht beigestanden.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmccxciv{Und WIR ließen ihnen in diesem Diesseits einen Fluch folgen, und am Tag der Auferstehung sind sie von den Verdammten.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmccxcv{Und gewiß, bereits ließen WIR Musa die Schrift zuteil werden, nachdem WIR die ersten Generationen zugrunde gerichtet hatten, als etwas Einblick-Gewährendes für die Menschen, eine Rechtleitung und eine Gnade, damit sie sich besinnen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmccxcvi{Und du warst nicht an der Seite des westlichen (Berges bzw. Tales), als WIR für Musa die Angelegenheit bestimmten. Und du warst nicht von den Bezeugenden.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmccxcvii{Aber WIR ließen Generationen entstehen, dann war ihnen das Leben lang. Und du hast dich nicht aufgehalten bei den Bewohnern von Madyan und ihnen Unsere Ayat vorgetragen, sondern WIR waren Entsendende.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmccxcviii{Und du warst nicht an der Seite vom Tur-Berg, als WIR riefen. Doch es ist eine Gnade von deinem HERRN, damit du Leute warnst, zu denen kein Warner vor dir kam, damit sie sich besinnen.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmccxcix{Und würde sie kein Unheil wegen dem von ihnen Zustandegebrachten treffen, (würden WIR dich nicht entsenden) - dann würden sie sagen: ``Unser HERR! Hättest DU uns doch einen Gesandten geschickt, damit wir Deinen Ayat folgen und von den Mumin werden!"\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmccc{Und als die Wahrheit von Uns zu ihnen kam, sagten sie: ``Würde ihm doch Gleiches zuteil, wie es Musa zuteil wurde!" Betrieben sie etwa kein Kufr dem gegenüber, was Musa zuteil wurde?! Sie sagten: ``Diese sind zwei Magier, die sich gegenseitig halfen." Und sie sagten: ``Gewiß, wir betreiben Kufr gegenüber Beiden."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmccci{Sag: ``Dann bringt eine Schrift von ALLAH, die rechtleitender als die Beiden ist, damit ich ihr folge, solltet ihr wahrhaftig sein."\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmcccii{Sollten sie dann (dieses) dir nicht erfüllen, so wisse, daß sie nur ihren Neigungen folgen. Und keiner ist abirrender als derjenige, der seinen Neigungen ohne Rechtleitung von ALLAH folgt! Gewiß, ALLAH leitet die unrecht-begehenden Leute nicht recht.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmccciii{Und gewiß, bereits brachten WIR ihnen das Mitgeteilte zuwege, damit sie sich besinnen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmccciv{Diejenigen, denen WIR die Schrift vor ihm (dem Quran) zuteil werden ließen, diese verinnerlichen den Iman an ihn.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmcccv{Und wenn er ihnen vorgetragen wird, sagen sie: ``Wir verinnerlichen den Iman an ihn. Er ist zweifelsohne dieWahrheit von unserem HERRN! Wir waren doch vor ihm Muslime."\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmcccvi{Diese bekommen ihre Belohnung zweimal für das, was sie an Geduld aufbrachten, und sie wehren mit dem Guten das Schlechte ab, und sie geben vom Rizq, das WIR ihnen gewährten.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmcccvii{Und wenn sie das Sinnlose hören, wenden sie sich davon ab und sagen: ``Uns gehören unsere Handlungen und euch gehören eure Handlungen, Salam sei mit euch! WIR möchten keine Unwissenden sein."\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmcccviii{Gewiß, du wirst nicht rechtleiten, wen du gerne hast. Doch ALLAH leitet recht, wen ER will! Und ER weiß besser Bescheid über die der Rechtleitung Folgenden.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmcccix{Und sie sagten: ``Wenn wir der Rechtleitung mit dir folgen, werden wir von unserem Land vertrieben." Festigten WIR ihnen etwa nicht ein sicheres Ha-ramm, zu dem die Früchte von Allerlei als Rizq von Uns gebracht werden? Doch die meisten wissen nicht.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmcccx{Und wie viele an Ortschaften richteten WIR doch zugrunde, als sie in ihrer Lebensweise Übertretungen begingen! Also diese sind ihre Wohnstätten, sie wurden nach ihnen nur ein wenig bewohnt. Und WIR sind die Bleibenden (nach deren Vergehen).\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmcccxi{Und dein HERR würde nie die Ortschaften zugrunde richten, bis ER zu ihrer Mutterstadt einen Gesandten schickt, der ihnen Unsere Ayat vorträgt. Und WIR würden die Ortschaften nie zugrunde richten, es sei denn, ihre Bewohner sind Unrecht-Begehende.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmcccxii{Und was euch an Dingen zuteil wurde, ist nur Verbrauchsgut des Diesseits und dessen Schmuck. Und was bei ALLAH ist, ist noch besser und bleibender. Besinnt ihr euch etwa nicht?!\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmcccxiii{Ist etwa derjenige, dem WIR ein schönes Versprechen machten, das er erfüllt bekommt, gleich dem, dem WIR Verbrauchsgüter des diesseitigen Lebens gewährten, und er dann am Tag der Auferstehung von den Ausgelieferten sein wird?!\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmcccxiv{Und an dem Tag, wenn ER sie ruft und dann sagt: ``Wo sind die Mir beigesellten Partner, die ihr zu behaupten pflegtet?!",\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmcccxv{sagen diejenigen, gegen die die Bestimmung Rechtens war: ``Unser HERR! Dies sind diejenigen, die wir irreführten. Wir führten sie irre, wie wir irregingen. Wir sagen uns (von ihnen) vor Dir los, sie pflegten uns keineswegs zu dienen.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmcccxvi{Und es wurde gesagt: ``Ruft die von euch beigesellten Partner! Dann riefen sie sie. Doch sie antworteten ihnen nicht und sahen die Peinigung. Hätten sie sich doch rechtleiten lassen!\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmcccxvii{Und an dem Tag, wenn ER sie ruft und sagt: ``Was habt ihr den Gesandten geantwortet?! ,\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmcccxviii{waren ihnen dann an diesem Tag die Mitteilungen verwirrend, so werden sie einander nicht fragen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmcccxix{Und hinsichtlich desjenigen, der bereut, den Iman verinnerlicht und gottgefällig Gutes tut, dieser wird dann von den Erfolgreichen sein.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmcccxx{Und dein HERR erschafft, was ER will, und ER wählt aus. Ihnen steht jedoch die Auswahl nicht zu. Subhanallah und immer allerhabener ist ER über das, was sie an Schirk betreiben.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmcccxxi{Und dein HERR weiß, was ihre Brüste verbergen, und was sie offenlegen.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmcccxxii{Und ER ist ALLAH! Es gibt keine Gottheit außer Ihm! Ihm gebührt alles Lob im Diesseits und im Jenseits. Und Ihm gebührt das Richten und zu Ihm werdet ihr zurückgebracht.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmcccxxiii{Sag: ``Wie seht ihr es? Wenn ALLAH euch die Nacht andauernd bis zum Tag der Auferstehung machen würde, welche Gottheit außer ALLAH, wird euch dann Helligkeit gewähren? Hört ihr etwa nicht?!"\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmcccxxiv{Sag: ``Wie seht ihr es? Wenn ALLAH euch den Tag andauernd bis zum Tag der Auferstehung machen würde, welche Gottheit außer ALLAH, wird euch dann Nacht gewähren, in der ihr euch ruht? Seht ihr etwa nicht?!"\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmcccxxv{Und wegen Seiner Gnade machte ER euch den Tag und die Nacht, damit ihr darin ruht und nach Seiner Gunst erstrebt. Und damit ihr euch dankbar erweist.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmcccxxvi{Und am Tag, wenn ER sie ruft und dann sagt: ``Wo sind die Mir beigesellten Partner, die ihr zu behaupten pflegtet?!" -\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmcccxxvii{Und WIR nehmen von jeder Umma einen Zeugen heraus, dann sagen WIR: ``Bringt euren Beweis her!" - Dann wußten sie, daß die Wahrheit gewiß ALLAH gehört. Und ihnen ist abhanden gegangen, was sie zu erfinden pflegten.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmcccxxviii{Gewiß, Qarun war von Musas Leuten, dann tat er ihnen Unrecht an. UndWIR gewährten ihm von den Schätzen, deren Schlüssel zu tragen sogar einer Gruppe Kraftvoller schwer fällt. Als zu ihm seine Leute sagten: ``Sei nicht freudenerregt! Gewiß, ALLAH liebt nicht die Freudenerregten.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmcccxxix{Und strebe mit dem, was ALLAH dir zuteil werden ließ, die jenseitige Wohnstätte an und vergiß nicht deinen Anteil am Diesseits, und tue Gutes, wie ALLAH dir Gutes tat, und strebe kein Verderben auf Erden an! Gewiß, ALLAH liebt nicht die Verderben- Anrichtenden.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmcccxxx{Er sagte: ``Zuteil wurde mir dies nur wegen dem Wissen, über das ich verfüge!" Wußte er etwa nicht, daß ALLAH bereits vor ihm von Generationen zugrunde richten ließ, die über noch mehr Macht verfügten als er und die noch mehr an Vermögen anhäuften?! Und die schwer Verfehlenden werden nach ihren Verfehlungen nicht gefragt.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmcccxxxi{Dann ging er zu seinen Leuten mit seinem Schmuck hinaus. Diejenigen, die das diesseitige Leben anstrebten, sagten: ``Würde uns doch Gleiches zuteil, wie es Qarun zuteil wurde! Er verfügt doch über Riesenglück."\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmcccxxxii{Und diejenigen, denen Wissen zuteil wurde, sagten: ``Euer Untergang! ALLAHs Belohnung ist besser für denjenigen, der den Iman verinnerlichte und gottgefällig Gutes tat! Und erhalten werden es nur die Duldsamen.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmcccxxxiii{Da ließen WIR mit ihm und mit seinem Haus die Erde versinken, so gab es für ihn keine Gruppe, die ihm anstelle von ALLAH beistand, und er war nicht von den Unterstützten.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmcccxxxiv{Und diejenigen, die sich wünschten, an seiner Stelle zu sein, begannen zu sagen: ``Erstaunlich! ALLAH gewährt viel Rizq, wem ER will von Seinen Dienern, und ER gewährt auch nur wenig. Hätte ALLAH uns keine Gnade gewährt, hätte ER mit uns die Erde versinken lassen. Erstaunlich! Die Kafir werden nicht erfolgreich sein."\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmcccxxxv{Diese ist die jenseitige Wohnstätte, WIR gewähren sie denjenigen, die auf Erden sich weder in Arroganz erheben, noch Verdeben anrichten. Und das Anschließende ist für die Muttaqi bestimmt!\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmcccxxxvi{Wer mit der gottgefälligen guten Tat kommt, für den ist Besseres als diese bestimmt. Doch wer mit der gottmißfälligen Tat kommt, so wird denjenigen, welche die gottmißfälligen Taten begingen, nichts außer dem vergolten, was sie zu tun pflegten.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmcccxxxvii{Gewiß, Derjenige, Der dir den Quran auferlegte, wird dich doch zum Versprochenen zurückkehren lassen. Sag: ``Mein HERR weiß besser Bescheid über denjenigen, der mit der Rechtleitung kam, und über denjenigen, der in einem klaren Fehlgehen ist.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmcccxxxviii{Und du hast nie erwartet, daß die Schrift dir gegeben würde - außer daß es eine Gnade von deinem HERRN war. So sei kein Rückenverstärker für die Kafir.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmcccxxxix{Und lasse sie niemals dich von ALLAHs Ayat abhalten, nachdem sie dir zuteil geworden sind. Und mache Da'wa zu deinem HERRN! Und sei nicht von den Muschrik!\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmcccxl{Und richte keine Bittgebete an eine andere Gottheit neben ALLAH! Es gibt keine Gottheit außer Ihm! Es vergeht alles außer Ihm. Ihm gehört das Richten, und zu Ihm werdet ihr zurückgebracht.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmcccxli{\basmalahde Alif-lam-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcccxlii{Dachten die Menschen etwa, daß sie in Ruhe gelassen werden, zu sagen: ``Wir verinnerlichten den Iman", ohne daß sie einer Fitna unterzogen werden?!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcccxliii{Gewiß, bereits setzten WIR diejenigen vor ihnen der Fitna aus. Und ALLAH wird gewiß diejenigen kenntlich machen, die wahrhaftig waren, und ER wird gewiß die Lügner kenntlich machen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcccxliv{Oder dachten etwa diejenigen, welche die gottmißfälligen Taten begehen, daß sie Uns entkommen werden?! Erbärmlich ist das, was sie urteilen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcccxlv{Wer auf die Begegnung ALLAHs hofft, so wird die Frist von ALLAH doch eintreffen! Und ER ist Der Allhörende, Der Allwissende.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcccxlvi{Und wer Dschihad leistet, der leistet Dschihad für sich selbst. Gewiß, ALLAH ist doch absolut autark aller Schöpfung gegenüber.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcccxlvii{Und denjenigen, die den Iman verinnerlichten und gottgefällig Gutes taten, werden WIR zweifelsohne ihre gottmißfälligen Taten tilgen und doch Besseres als das vergelten, was sie zu tun pflegten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcccxlviii{Und WIR wiesen dem Menschen zu, seinen Eltern Ihsan zu erweisen. Und sollten sie dich bedrängen, damit du Mir gegenüber Schirk betreibst, worüber du kein Wissen hast, dann gehorche ihnen nicht! Zu Mir ist eure Rückkehr, dann werde ICH euch Mitteilung über das machen, was ihr zu tun pflegtet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcccxlix{Und diejenigen, die den Iman verinnerlichten und gottgefällig Gutes taten, werden WIR doch unter die gottgefällig Guttuenden bringen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcccl{Und unter den Menschen gibt es manch einen, der sagt: ``Ich verinnerlichte den Iman", und wenn ihm dann wegen ALLAH Schaden zugefügt wurde, läßt er die Fitna der Menschen wie die Peinigung von ALLAH sein. Und würde ein Sieg von deinem HERRN kommen, würden sie doch sagen: ``Wir waren doch mit euch!" Weiß ALLAH nicht besser Bescheid über das, was in den Brüsten der Menschen ist?!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcccli{Und ALLAH macht gewiß kenntlich diejenigen, die den Iman verinnerlicht haben, und ER macht gewiß kenntlich die Munafiq.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmccclii{Und diejenigen, die Kufr betrieben haben, sagten zu denjenigen, die den Iman verinnerlicht haben: ``Folgt unserem Weg und wir werden euch doch eure Verfehlungen abnehmen." Doch sie werden ihnen von ihren Verfehlungen nichts abnehmen. Gewiß, sie sind doch Lügner.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcccliii{Und sie werden zweifelsohne ihre Lasten und noch (weitere) Lasten zu ihren Lasten tragen. Und am Tag der Auferstehung werden sie gewiß für das zur Rechenschaft gezogen, was sie zu erdichten pflegten.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcccliv{Und gewiß, bereits entsandten WIR Nuh zu seinen Leuten, dann blieb er unter ihnen tausend Jahre weniger fünfzig Jahre. Dann richtete sie die Sintflut zugrunde, während sie Unrecht-Begehende waren.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmccclv{Dann erretteten WIR ihn und diejenigen vom Schiff und machten es zur Aya für die ganze Schöpfung.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmccclvi{Und Ibrahim als er seinen Leuten sagte: ``Dient ALLAH und handelt Taqwa gemäß Ihm gegenüber! Dies ist besser für euch, solltet ihr es wissen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmccclvii{Ihr dient anstelle von ALLAH doch nur Götzen und ihr erfindet nur Ifk-Lüge. Gewiß, diejenigen, denen ihr anstelle von ALLAH dient, haben für euch kein Rizq, so strebt das Rizq bei ALLAH an, dient Ihm und erweist euch Ihm gegenüber dankbar! Zu Ihm werdet ihr zurückgebracht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmccclviii{Und solltet ihr ableugnen, so haben bereits viele Umam vor euch abgeleugnet." Und dem Gesandten obliegt nur das deutliche Verkünden.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmccclix{Haben sie etwa nicht wahrgenommen, wie ALLAH die Schöpfung beginnen läßt, dann sie wiederholen läßt?! Gewiß, dies ist für ALLAH etwas Leichtes.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmccclx{Sag: ``Zieht auf der Erde umher, dann seht nach, wie die Schöpfung begann. Dann wird ALLAH das letzte Entstehen entstehen lassen. Gewiß, ALLAH ist über alles allmächtig.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmccclxi{ER peinigt, wen ER will, und ER erweist Gnade, wem ER will. Und zu Ihm werdet ihr zurückgebracht.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmccclxii{Und ihr macht (Ihm) weder auf Erden noch im Himmel zu schaffen. Und ihr habt anstelle von ALLAH weder Wali noch Beistehenden.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmccclxiii{Und diejenigen, die Kufr ALLAHs Ayat und Seiner Begegnung gegenüber betrieben, diese verzweifeln an Meiner Gnade. Und für diese ist eine qualvolle Peinigung bestimmt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmccclxiv{Und die Antwort seiner Leute war nichts anderes als daß sie sagten: ``Tötet ihn oder verbrennt ihn!" Dann errettete ihn ALLAH von dem Feuer. Gewiß, darin sind zweifelsohne Ayat für Leute, die den Iman verinnerlichen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmccclxv{Und er sagte: ``Ihr habt euch anstelle von ALLAH nur Götzen als (Grund für) Zuneigung zueinander im diesseitigen Leben genommen. Dann am Tag der Auferstehung werden die einen von euch den anderen gegenüber Kufr betreiben und die einen von euch auf die anderen fluchen. Und eure Unterkunft ist dann das Feuer und für euch wird es keine Beistehenden geben.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmccclxvi{Dann verinnerlichte Lut den Iman an ihn. Und er sagte: ``Ich unternehme Hidschra zu meinem HERRN. Gewiß, ER ist Der Allwürdige, Der Allweise."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmccclxvii{Und WIR schenkten ihm Ishaq und Ya'qub, bestimmten die Prophetenschaft und die Schrift bei seiner Nachkommenschaft und ließen ihm seine Belohnung im Diesseits zuteil werden. Und gewiß, er ist im Jenseits doch unter den gottgefällig Guttuenden!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmccclxviii{Und Lut als er seinen Leuten sagte: ``Gewiß, Ihr begeht doch das Verabscheuenswürdige, das keiner von den Menschen vor euch je beging.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmccclxix{Ja! Ihr verkehrt doch intim mit den Männern, begeht Straßenraub und macht in euren Versammlungen das Mißbilligte." Und die Antwort seiner Leute war nichts anderes als daß sie sagten: ``Bringe uns ALLAHs Peinigung, solltest du von den Wahrhaftigen sein."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmccclxx{Er sagte: ``Mein HERR! Stehe mir gegen die verderbenanrichtenden Leute bei!"\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmccclxxi{Und als Unsere Entsandte zu Ibrahim mit der frohen Botschaft kamen, sagten sie: ``Wir werden die Bewohner dieser Ortschaft zugrunde richten. Ja! Ihre Bewohner pflegen unrecht-begehend zu sein.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmccclxxii{Er sagte: ``In ihr befindet sich doch Lut." Sie sagten: ``Wir wissen besser Bescheid darüber, wer sich in ihr befindet. Gewiß, wir werden ihn und seine Familie erretten, außer seiner Ehefrau, sie gehört zu den Untergehenden."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmccclxxiii{Und als Unsere Entsandten zu Lut kamen, ging es ihm schlecht wegen ihnen und überfordert war er mit ihnen. Und sie sagten: ``Fürchte dich nicht und sei nicht traurig! Wir werden dich und deine Familie sicher erretten, außer deiner Ehefrau, sie gehört zu den Untergehenden.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmccclxxiv{Ja! Wir werden auf die Bewohner dieser Ortschaft eine Peinigung vom Himmel hinabsenden für das, was sie an Fisq zu begehen pflegten."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmccclxxv{Und gewiß, bereits ließen WIR aus ihr eine eindeutige Aya übrig für Leute, die sich besinnen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmccclxxvi{Und zu Madyan (entsandten WIR) ihren Bruder Schu'aib. So sagte er: ``Meine Leute! Dient ALLAH, hofft auf den Jüngsten Tag und handelt auf Erden nicht als Verderben-Anrichtende!"\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmccclxxvii{Dann bezichtigten sie ihn der Lüge, dann richtete sie das Bebende zugrunde, so wurden sie in ihrer Wohnstätte zu leblosen Körpern.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmccclxxviii{Ebenfalls richteten WIR zugrunde) 'Aad und Thamud. Und bereits wurde (ihre Peinigung) euch sichtbar von ihren Wohnstätten. Und der Satan ließen ihnen ihr Taten schön erscheinen, dann hielt er sie vom Weg ab, während sie einblicken konnten.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmccclxxix{Ebenso war es mit) Qarun, Pharao und Haman. Gewiß, bereits kam Musa zu ihnen mit den deutlichen Zeichen, dann erhoben sie sich im Lande in Arroganz. Doch sie entkamen Uns nicht.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmccclxxx{Und jeden richteten WIR wegen seiner Verfehlung zugrunde. Manchen von ihnen schickten WIR über ihnen einen Wirbelsturm, manche von ihnen richtete der Schrei zugrunde, manche von ihnen ließen WIR mit der Erde versinken und manche von ihnen ließen WIR ertrinken. Und ALLAH gebührt es nicht, ihnen Unrecht anzutun, doch sie pflegten sich selbst Unrecht anzutun.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmccclxxxi{Das Gleichnis derjenigen, die sich anstelle von ALLAH Wali nahmen, ist wie das Gleichnis der Spinne, die sich ein Netz spinnt. Und gewiß, das Schwächste aller Netze ist zweifelsohne das Spinnennetz, würden sie es nur wissen!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmccclxxxii{Gewiß, ALLAH kennt alle diejenigen, an die sie anstelle von Ihm Bittgebete richten. Und ER ist Der Allwürdige, Der Allweise.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmccclxxxiii{Und diese Gleichnisse prägen WIR den Menschen. Und es begreifen sie nur die Wissenden.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmccclxxxiv{ALLAH erschuf die Himmel und die Erde mit der Gesetzmäßigkeit. Gewiß, darin ist doch eine Aya für die Mumin.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmccclxxxv{Trage vor, was dir von der Schrift an Wahy zuteil wurde, und verrichte ordnungsgemäß das rituelle Gebet! Gewiß, das rituelle Gebet hält vom Abscheulichen und vom Mißbilligten ab. Und das Gedenken an ALLAH ist doch noch größer. Und ALLAH weiß, was ihr bewerkstelligt.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmccclxxxvi{Und disputiert mit den Schriftbesitzern nicht, es sei denn auf die beste Art, außer mit denjenigen von ihnen, die Unrecht begingen, und sagt: ``Wir verinnerlichten den Iman an das, das uns hinabgesandt wurde und euch hinabgesandt wurde, und unser Gott und euer Gott ist einer, und wir sind Ihm gegenüber Muslime.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmccclxxxvii{Und solcherart sandten WIR die Schrift hinab. Also diejenigen, denen WIR die Schrift zuteil werden ließen, verinnerlichen den Iman an ihn. Und von diesen gibt es manche, die den Iman an ihn verinnerlichen werden. Und Unsere Ayat leugnen nur dieKafir ab!\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmccclxxxviii{Und vor ihm hast du weder eine Schrift lesen, noch diese mit deiner Rechten schreiben können, sonst hätten die des Unwahren Verbreitenden daran Zweifel gehegt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmccclxxxix{Nein, sondern er besteht aus klaren Ayat, die in den Brüsten derjenigen sind, denen das Wissen zuteil wurde. Und Unsere Ayat leugnen nur die Unrecht-Begehenden ab!\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmcccxc{Und sie sagten: ``Würden ihm doch Ayat von seinem HERRN hinabgesandt!" Sag: ``Die Ayat sind nur bei ALLAH und ich bin doch nur ein erläuternder Warner."\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmcccxci{War es ihnen denn nicht genug, daßWIR dir die Schrift hinabsandten, die ihnen vorgetragen wird?! Gewiß, darin ist doch Gnade und Ermahnung für Leute, die den Iman verinnerlichen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmcccxcii{Sag: ``ALLAH genügt zwischen mir und euch als Zeuge. ER weiß, was in den Himmeln und auf Erden ist." Und diejenigen, die den Iman an das für nichtig Erklärte verinnerlichten und Kufr ALLAH gegenüber betrieben, diese sind die eigentlichen Verlierer.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmcccxciii{Und sie fordern dich zur Eile mit der Peinigung auf. Und gäbe es keine festgelegte Frist, gewiß käme die Peinigung zu ihnen. Und sie wird doch zu ihnen unerwartet kommen, während sie es nicht merken.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmcccxciv{Sie fordern dich zur Eile mit der Peinigung auf. Und gewiß, Dschahannam wird die Kafir doch komplett umfassen\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmcccxcv{an dem Tag, wenn die Peinigung sie von über ihnen und von unter ihren Füßen umhüllt, und ER sagt: ``Kostet, was ihr zu tun pflegtet!"\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmcccxcvi{Meine Diener, die den Iman verinnerlichten! Gewiß, Meine Erde ist weit genug, so dient Mir alleine!\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmcccxcvii{Jedes Lebewesen wird den Tod erfahren, dann werdet ihr zu Uns zurückgebracht.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmcccxcviii{Und diejenigen, die den Iman verinnerlichten und gottgefällig Gutes taten, werden WIR doch in der Dschanna in Zimmern wohnen lassen, die von Flüssen durchflossen werden. Darin bleiben sie ewig. Und was für eine schöne Belohnung für die gottgefällig Guttuenden.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmcccxcix{Diejenigen, die sich in Geduld übten und ihrem HERRN gegenüber Tawakkul praktizierten.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmcd{Und wie viele der sich bewegenden Lebewesen gibt es, die ihr Rizq nicht halten können. ALLAH gewährt ihnen und euch Rizq. Und ER ist Der Allhörende, Der Allwissende.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmcdi{Und würdest du sie fragen: ``Wer die Himmel und die Erde erschuf und die Sonne und den Mond gratis fügbar machte?", gewiß würden sie sagen: ``ALLAH!" Also wie werden sie denn belogen?!\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmcdii{ALLAH gewährt viel Rizq, wem ER will von Seinen Dienern, und ER gewährt ihm auch nur wenig. Gewiß, ALLAH ist über alles allwissend.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmcdiii{Und würdest du sie fragen: ``Wer ließ vom Himmel Wasser fallen, dann mit ihm die Landschaft nach ihrem Tod beleben?", gewiß würden sie sagen: ``ALLAH!" Sag: ``Alhamdulillah: Alles Lob gebührt ALLAH!" Nein, sondern die meisten von ihnen begreifen es nicht.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmcdiv{Und dieses diesseitige Leben ist nichts anderes außer Vergnügen und sinnlosem Treiben. Und gewiß, die jenseitige Wohnstätte ist doch das wahre Leben, würden sie es nur wissen!\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmcdv{Und als sie das Schiff bestiegen haben, richteten sie Bittgebete an ALLAH aufrichtig im Din Ihm gegenüber, und nachdem ER sie auf das Land gerettet hatte, da betreiben sie Schirk.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmcdvi{Sie sollen nur Kufr dem gegenüber betreiben, was WIR ihnen zuteil werden ließen und sie sollen sich nur vergnügen! Denn sie werden es noch wissen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmcdvii{Sehen sie etwa nicht, daß WIR ein sicheres Ha-ramm errichteten, während die Menschen um sie herum weggerissen werden?! Verinnerlichen sie etwa den Iman an das für nichtig Erklärte und betreiben ALLAHs Wohltaten gegenüber Kufr?!\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmcdviii{Und keiner begeht mehr Unrecht als derjenige, der im Namen ALLAHs Lügen erdichtet oder das Wahre ableugnet, nachdem es zu ihm gekommen war! Gibt es etwa nicht in Dschahannam eine Bleibe für die Kafir?!\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmcdix{Und diejenigen, die Unseretwegen Dschihad leisteten, werden WIR doch zu Unserem Weg rechtleiten. Und gewiß, ALLAH ist doch mit den Muhsin.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmcdx{\basmalahde Alif-lam-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcdxi{Besiegt wurden die Römer\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcdxii{am unteren Teil des Landes. Und sie werden nach ihrer Niederlage siegen\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcdxiii{in wenigen Jahren. ALLAH unterliegt die Angelegenheit vorher und nachher. Und an diesem Tag freuen sich die Mumin\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcdxiv{über den Sieg von ALLAH. ER verhilft zum Sieg, wem ER will. Und ER ist Der Allwürdige, Der Allgnädige.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcdxv{Dies ist ein Versprechen von ALLAH. ALLAH verletzt nie Sein Versprechen. Doch die meisten Menschen wissen es nicht.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcdxvi{Sie kennen (nur) Oberflächliches vom diesseitigen Leben und sie sind dem Jenseits gegenüber achtlos.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcdxvii{Dachten sie etwa nicht über sich selbst nach?! ALLAH erschuf die Himmel, die Erde und das, was zwischen ihnen ist, nur nach Gesetzmäßigkeit und einer festgelegten Frist. Und gewiß, viele von den Menschen betreiben doch Kufr gegenüber der Begegnung mit ihrem HERRN.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcdxviii{Zogen sie etwa nicht auf der Erde umher, damit sie sehen, wie das Anschließende Derjenigen vor ihnen war?! Sie waren noch mächtiger als sie, sie pflügten die Erde und bebauten sie noch mehr als diese sie bebauten. Auch ihre Gesandten kamen zu ihnen mit den klaren Zeichen. Also es gebührt ALLAH nicht, ihnen Unrecht anzutun, doch sie pflegten sich selbst Unrecht anzutun.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcdxix{Dann war das Anschließende derjenigen, die schlecht handelten, das Schlechte, da sie ALLAHs Ayat ableugneten und sie zu verspotten pflegten.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcdxx{ALLAH beginnt die Erschaffung, dann wiederholt ER sie, dann werdet ihr zu Ihm zurückgebracht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcdxxi{Und an dem Tag, wenn die Stunde anschlägt, schweigen die schwer Verfehlenden.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcdxxii{Und für sie gibt es unter den von ihnen Beigesellten keine Fürbittenden. Und sie pflegen dann den von ihnen Beigesellten gegenüber Kufr zu betreiben.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcdxxiii{Und an dem Tag, wenn die Stunde anschlägt, an diesem Tag werden sie auseinandergehen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcdxxiv{Also hinsichtlich derjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, diese sind in einem Garten, sie freuen sich.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcdxxv{Doch hinsichtlich derjenigen, die Kufr betrieben und Unsere Ayat und die Begegnung im Jenseits abgeleugnet haben, diese werden der Peinigung ausgeliefert.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcdxxvi{Also subhanallah, wenn ihr in die Abendzeit und in die Morgenzeit hineingeht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcdxxvii{Und Ihm gebührt alles Lob in den Himmeln und auf Erden nachts und wenn ihr in die Mittagszeit hineingeht.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcdxxviii{ER bringt das Lebendige vom Toten heraus, das Tote vom Lebendigen und belebt die Landschaft nach ihrem Tod. Und solcherart werdet ihr hervorgebracht.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcdxxix{Auch zu Seinen Ayat zählt, daß ER euch aus Erde erschuf, dann - da seid ihr Menschen, die sich ausbreiten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcdxxx{Ebenso zu Seinen Ayat zählt, daß ER für euch von eurem Wesen Partnerwesen erschuf, damit ihr bei ihnen Geborgenheit findet. Und ER setzte zwischen euch Liebe und Barmherzigkeit. Gewiß, darin sind doch Ayat für Leute, die nachdenken.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcdxxxi{Ebenfalls zählt zu Seinen Ayat die Schöpfung der Himmel und der Erde und die Vielfalt an euren Sprachen und euren Farben. Gewiß, darin sind doch Ayat für die Wissenden.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcdxxxii{Und zu Seinen Ayat zählt euer Schlaf während der Nacht und am Tag und euer Anstreben nach Seiner Gunst. Gewiß, darin sind doch Ayat für Leute, die hören.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcdxxxiii{Auch zu Seinen Ayat zählt, daß ER euch den Blitz zum Furchterregen und zum Hoffungerwecken zeigt. Und ER läßt vom Himmel Wasser fallen, dann belebt ER damit die Landschaft nach ihrem Tod. Gewiß, darin sind doch Ayat für Leute, die begreifen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcdxxxiv{Auch zu Seinen Ayat zählt, daß der Himmel und die Erde nach Seiner Bestimmung feststehen. Dann, wenn ER euch mit einem Ruf ruft, da geht ihr heraus.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcdxxxv{Und Ihm gehört alles, was in den Himmeln und auf Erden ist. Alle sind Ihm gegenüber bewußt ergeben.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcdxxxvi{Und ER ist Derjenige, Der die Schöpfung beginnt und dann wiederholt, und dies ist für Ihn doch noch leichter. Und Ihm gehört das höchste Gleichnis in den Himmeln und auf Erden. Und ER ist Der Allwürdige, Der Allweise.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcdxxxvii{Er prägte euch ein Gleichnis von euch selbst. Habt ihr etwa unter denjenigen, die euch gehören, irgendwelche Partner im Rizq, das WIR euch gewährten, so daß ihr darin gleich seid, und ihr sie so fürchtet, wie ihr euch selbst fürchtet?! Solcherart erläutern WIR die Ayat für Leute, die begreifen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcdxxxviii{Nein, sondern diejenigen, die Unrecht begingen, folgten ihren Neigungen ohne Wissen. Und wer leitet denn denjenigen recht, den ALLAH abirren läßt?! Und für sie gibt es keine Beistehenden.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmcdxxxix{So richte dich nach dem Din als Hanif! Dies ist ALLAHs Veranlagung, womit ER die Menschen veranlagte. Es gibt kein Abändern für ALLAHs Schöpfung. Dies ist der äußerst geradlinige Din. Doch dies wissen die meisten Menschen nicht.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmcdxl{Seid Ihm gegenüber umkehrend, handelt Taqwa gemäß Ihm gegenüber, verrichtet ordnungsgemäß das rituelle Gebet und seid nicht von den Muschrik,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmcdxli{von denjenigen, die ihren Din spalteten und zu Parteien wurden. Jede Partei ist über das, worüber sie verfügt, äußerst erfreut.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmcdxlii{Und wenn die Menschen Schädliches trifft, richten sie Bittgebete an ihren HERRN umkehrend zu Ihm. Dann aber, wenn ER ihnen von Sich Gnade erweist, da ist eine Gruppe unter ihnen, die ihrem HERRN gegenüber Schirk betreiben.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmcdxliii{Sie sollen nur Kufr betreiben dem gegenüber, was WIR ihnen zuteil werden ließen! So vergnügt ihr euch nur! Ihr werdet noch gewiß wissen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmcdxliv{Oder sandten WIR ihnen etwa eine Bestätigung hinab, die das besagt, was sie an Schirk zu betreiben pflegten?!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmcdxlv{Und wenn WIR die Menschen eine Gnade von Uns erfahren ließen, freuen sie sich darüber, und wenn sie Schlechtes trifft wegen dem, was sie eigenhändig vollbrachten, da verzweifeln sie.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmcdxlvi{Haben sie etwa nicht wahrgenommen, daß ALLAH viel vom Rizq gewährt, wem ER will, und ER es auch wenig macht. Gewiß, darin sind doch Ayat für Leute, die den Iman verinnerlichen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmcdxlvii{Also gib dem Nahverwandten das ihm Zustehende, sowie dem Armen und dem in Not geratenen Reisenden. Dies ist besser für diejenigen, die (das Wohlgefallen) ALLAHs anstreben. Und diese sind die wirklichen Erfolgreichen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmcdxlviii{Und was ihr mit Riba verleiht, damit es sich durch die Vermögenswerte der Menschen vermehrt, dies vermehrt sich bei ALLAH nicht. Doch was ihr an Zakat entrichtet, mit der ihr (das Wohlgefallen) ALLAHs anstrebt, diese sind die wirklichen Vervielfachenden.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmcdxlix{ALLAH ist Derjenige, Der euch erschuf, dann euch Rizq gewährte, dann euch sterben läßt, dann euch lebendig macht. Gibt es von euren (ALLAH) beigesellten Partnern einen, der von diesem irgend etwas macht?! Gepriesen-erhaben ist ER, und immer allerhabener ist ER über das, was sie an Schirk betreiben.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmcdl{Das Verderben wurde sowohl auf dem Land als auch im Meer sichtbar wegen dem, was die Menschen eigenhändig erwarben. Dies, damit ER sie etwas von dem erfahren läßt, was sie taten, damit sie umkehren.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmcdli{Sag: ``Zieht auf der Erde herum, dann seht, wie das Anschließende derjenigen vor euch war." Die meisten von ihnen waren Muschrik.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmcdlii{So richte dich nach dem äußerst geradlinigen Din, bevor ein Tag von ALLAH kommt, der nicht abzuwehren ist. An diesem Tag werden sie auseinandergehen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmcdliii{Wer Kufr betrieb, so obliegt ihm sein Kufr. Und wer gottgefällig Gutes tat, diese treffen für sich Vorkehrungen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmcdliv{ER wird denjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, doch von Seiner Gunst vergelten. Gewiß, ER liebt nicht die Kafir.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmcdlv{Und zu Seinen Ayat zählt, daß ER die Winde als Überbringer froher Botschaft schickt, damit ER euch von Seiner Gnade erfahren läßt, damit die Schiffe nach Seiner Bestimmung fahren, damit ihr von Seiner Gunst anstrebt und damit ihr euch dankbar erweist.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmcdlvi{Und gewiß, bereits schickten WIR vor dir Gesandte zu ihren Leuten, dann kamen sie zu ihnen mit den klaren Zeichen, dann übten WIR Vergeltung an denjenigen, die schwere Verfehlungen begingen. Und es obliegt Uns die Unterstützung der Mumin.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmcdlvii{ALLAH ist Derjenige, Der die Winde schickt, die dann Wolken aufwirbeln, dann breitet ER diese am Himmel aus, wie ER will, und ER macht sie zu Stückteilen, dann siehst du den Regen aus ihnen herauskommen. Und wenn ER dann damit trifft, wen ER von Seinen Dienern will, da erwarten sie Erfreuliches,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmcdlviii{obgleich sie, bevor er auf sie fiel, vor ihm doch verzweifelt waren.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmcdlix{Also siehe die Spuren der Gnade deines HERRN, wie ER die Landschaft nach ihrem Tod belebt. Gewiß, Dieser wird doch die Toten lebendig machen. Und ER ist über alles allmächtig.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmcdlx{Und würden WIR einen Wind schicken, und sie es das Gewächs gelb werdend sehen, würden sie doch noch nach ihm Kufr-Betreibende bleiben!\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmcdlxi{Also du kannst die Toten nicht hören lassen, und du kannst die Tauben den Ruf nicht hören lassen, wenn sie flüchtend den Rücken umkehren.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmcdlxii{Und du wirst die Blinden von ihrem Abirren nicht rechtleiten können. Du kannst nur hören lassen diejenigen, die den Iman an Unsere Ayat verinnerlichen, die dann Muslime sind.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmcdlxiii{ALLAH ist Derjenige, Der euch von Schwäche erschuf, dann verlieh ER nach der Schwäche Stärke, dann verlieh ER nach der Stärke Schwäche und Grauhaare. ER erschafft, was ER will. Und ER ist Der Allwissende, Der Allmächtige.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmcdlxiv{Und an dem Tag, wenn die Stunde sich ereignet, schwören die schwer Verfehlenden, sie blieben nicht länger als eine Stunde. Solcherart wurden sie belogen.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmcdlxv{Und diejenigen, denen das Wissen und der Iman zuteil wurde, sagen: ``Gewiß, bereits seid ihr nach ALLAHs Schrift bis zum Tag des Erweckens geblieben. Also, dieser ist der Tag des Erweckens. Doch ihr pflegtet nicht zu wissen."\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmcdlxvi{Also, an diesem Tag nützt denjenigen, die Unrecht begingen, weder ihre Entschuldigung, noch werden sie zufrieden gestellt.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmcdlxvii{Und gewiß, bereits prägten WIR den Menschen in diesem Quran von jedem Gleichnis. Und würdest du ihnen eine Aya bringen, würden diejenigen, die Kufr betrieben haben, doch sagen: ``Ihr seid nichts anderes als Annullierung-Anstrebende."\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmcdlxviii{Solcherart versiegelt ALLAH die Herzen derjenigen, die nicht wissen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmcdlxix{So übe dich in Geduld! Gewiß, ALLAHs Versprechen ist wahr. Und laß diejenigen, die keine Gewißheit haben, dich nicht aus der Ruhe bringen!\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmcdlxx{\basmalahde Alif-lam-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcdlxxi{Diese sind die Ayat der weisen Schrift.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcdlxxii{Sie sind eine Rechtleitung und eine Gnade für die Muhsin,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcdlxxiii{diejenigen, die das rituelle Gebet ordnungsgemäß verrichten und die Zakat entrichten und die Gewißheit am Jenseits haben.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcdlxxiv{Diese folgen der Rechtleitung von ihrem HERRN. Und diese sind die Erfolgreichen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcdlxxv{Und unter den Menschen gibt es manch einen, der sich das Sinnlose vom Gerede ohne Wissen aneignet, damit er vom Wege ALLAHs abirren läßt, auch macht er ihn zum Spott. Für diese ist eine qualvolle Peinigung bestimmt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcdlxxvi{Und als ihm Unsere Ayat vorgetragen wurden, kehrte er den Rücken in Arroganz um, als ob er sie nicht hörte, als ob in seinen Ohren Schwerhörigkeit wäre. Also überbringe ihm die ``Frohe Botschaft" über eine qualvolle Peinigung!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcdlxxvii{Gewiß, diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, für diese ist die Dschannat des Wohlergehens bestimmt,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcdlxxviii{darin werden sie ewig bleiben. Es ist ALLAHs wahres Versprechen. Und ER ist Der Allwürdige, Der Allweise.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcdlxxix{ER erschuf die Himmel ohne von euch zu sichtende Stützen. Und ER setzte auf Erden Felsenberge, damit sie mit euch nicht schwankt, ER ließ auf ihr von jeglichen sich bewegenden Lebewesen ausbreiten. Und WIR ließen vom Himmel Wasser fallen, dann ließen WIR damit auf ihr von jeder edlen Zweiheit hervorsprießen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcdlxxx{Das ist die Schöpfung ALLAHs. Also zeigt Mir, was diejenigen erschufen die anstelle von Ihm sind. Nein, sondern die Unrecht- Begehenden sind im eindeutigen Abirren.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxi{Und gewiß, bereits ließen WIR Luqman die Weisheit zuteil werden: ``Erweise dich ALLAH gegenüber dankbar!" Und wer sich dankbar erweist, dieser erweist sich doch nur für sich selbst dankbar. Und wer Kufr betrieb, so ist ALLAH zweifelsohne absolut autark, alllobenswürdig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxii{Und (erinnere daran), als Luqman zu seinem Sohn sagte, während er ihn ermahnte: ``Mein Söhnchen! Betreibe ALLAH gegenüber keinen Schirk! Gewiß, Schirk ist doch ein ungeheuerliches Unrecht."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxiii{Und WIR wiesen dem Menschen hinsichtlich seiner Eltern zu: ``Seine Mutter war mit ihm schwanger in Schwäche über Schwäche, und seine Entwöhnung erfolgt in zwei Jahren. Erweise dich Mir und deinen Eltern gegenüber dankbar! Zu Mir ist das Werden.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxiv{Und sollten beide dich bedrängen, damit du Mir gegenüber das an Schirk betreibst, worüber du kein Wissen hast, dann gehorche ihnen nicht, doch pflege mit ihnen im Diesseits Kontakte nach Gebilligtem! Und folge dem Weg desjenigen, der zu Mir reuig umkehrt! Dann ist zu Mir eure Rückkehr, dann werde ICH euch Mitteilung über das machen, was ihr zu tun pflegtet."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxv{``Mein Söhnchen! Sollte es sich auch nur um das Gewicht eines Senfkorns handeln, das sich in einem Stein, in den Himmeln oder auf Erden befinden sollte, ALLAH bringt sie. Gewiß, ALLAH ist allgütig, allkundig.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxvi{Mein Söhnchen! Verrichte ordnungsgemäß das rituelle Gebet, fordere zum Gebilligten auf, rate vom Mißbilligten ab und übe dich in Geduld wegen dem, was dich traf. Gewiß, dies gehört zur Entschlossenheit den Angelegenheiten gegenüber.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxvii{Und wende dich nicht in Arroganz von den Leuten ab! Und gehe auf der Erde nicht freudenerregt! Gewiß, ALLAH liebt keinen stolzen Eingebildeten.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxviii{Und mäßige dich beim Gehen, und dämpfe von deiner Stimme! Gewiß, die abscheulichste aller Stimmen ist die Stimme der Esel."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcdlxxxix{Habt ihr etwa nicht gesehen, daß ALLAH für euch gratis fügbar machte, was in den Himmeln und was auf Erden ist, und für euch SeineWohltaten  Offenkundige und Verborgene, vollendete?! Und unter den Menschen gibt es manche, die über ALLAH weder mit Wissen, noch mit Rechtleitung, noch mit einer erleuchtenden Schrift disputieren.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcdxc{Und wenn ihnen gesagt wird: ``Folgt dem, was ALLAH hinabsandte." Sagen sie: ``Nein, sondern wir folgen dem, woran wir unsere Ahnen fanden." Auch dann wenn der Satan sie zur Peinigung der Gluthitze ruft?!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcdxci{Und wer für ALLAH den Islam praktiziert, während er Muhsin ist, dieser hielt sich bereits am wahrsten Halt fest. Und zu ALLAH ist das Anschließende aller Angelegenheiten.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcdxcii{Und wer Kufr betrieben hat, dessen Kufr soll dich nicht traurig machen. Zu Uns ist ihre Rückkehr, dann werden WIR ihnen Mitteilung machen über das, was sie taten. Gewiß, ALLAH ist allwissend über das, was in den Brüsten ist!\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcdxciii{WIR lassen sie sich ein wenig vergnügen, dann zwingen WIR sie in eine harte Peinigung hinein.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcdxciv{Und würdest du sie fragen: ``Wer erschuf die Himmel und die Erde?" Gewiß, würden sie sagen: ``ALLAH!" Sag: ``Alhamdulillah: Alles Lob gebührt ALLAH." Nein, sondern die meisten von ihnen wissen nicht.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcdxcv{ALLAH gehört, was in den Himmeln und auf Erden ist. Gewiß, ALLAH ist Der absolut Autarke, Der Alllobenswürdige.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcdxcvi{Und wäre alles an Bäumen, was auf Erden ist, Schreibgeräte gewesen, und das Meer (als Schreibflüssigkeit und) zusätzlich von noch sieben Meeren mit Nachschub versorgt, ALLAHs Worte würden nicht zu Ende gehen. Gewiß, ALLAH ist allwürdig, allweise.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcdxcvii{Und weder eure Erschaffung noch eure Erweckung ist etwas anderes als die einer einzigen Seele! Gewiß, ALLAH ist allhörend, allsehend.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcdxcviii{Hast du etwa nicht gesehen, daß ALLAH die Nacht in den Tag einfließen läßt und den Tag in die Nacht einfließen läßt, (daß) ER euch die Sonne und den Mond gratis fügbar machte - beide durchlaufen bis zu einer festgelegten Frist, und daß ALLAH dessen, was ihr tut, allkundig ist?!\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmcdxcix{Dies ist es, weil ALLAH gewiß Das Wahre ist, weil das, an das sie anstelle von Ihm Bittgebete richten, gewiß das für nichtig Erklärte ist, und weil ALLAH gewiß Der Allhöchste, Der Allgrößte ist.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmd{Hast etwa nicht gesehen, daß die Schiffe auf dem Meer mit ALLAHs Gabe fahren, damit ER euch von Seinen Ayat zeigt. Gewiß, darin sind doch Ayat für jeden dankbaren sich äußerst in Geduld Übenden.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdi{Und wenn Wogen wie Schattenwerfende sie umhüllen, richten sie Bittgebete an ALLAH als Aufrichtige im Din Ihm gegenüber, und nachdem ER sie ans Land errettet hatte, waren manche von ihnen gemäßigt. Und ALLAHs Ayat leugnet niemand außer jedem äußerst kufr-betreibenden Hinterhältigen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdii{Ihr Menschen! Handelt Taqwa gemäß eurem HERRN gegenüber, und habt Ehrfurcht vor einem Tag, an dem weder ein Gebärendes seinem Geborenen nützt, noch ein Geborenes seinem Gebärenden etwas nützen wird. Gewiß, ALLAHs Androhung ist wahr. So lasst das diesseitige Leben euch nicht täuschen, und lasst keineswegs das Täuschende euch in ALLAH täuschen!\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdiii{Gewiß ALLAH verfügt bei Sich über das Wissen der Stunde, ER läßt den helfenden (Regen) nieder und ER weiß, was in den Mutterschößen ist. Und kein Lebewesen weiß, was es morgen erwirbt, und kein Lebewesen weiß, in welchem Lande es stirbt. Gewiß, ALLAH ist allwissend, allkundig.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdiv{\basmalahde Alif-lam-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdv{Die sukzessive Hinabsendung der Schrift, über die keinerlei Zweifel ist, ist vom HERRN Aller Schöpfung.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdvi{Oder sagen sie etwa: ``Er hat sie erdichtet"?! Nein, sondern sie ist das Wahre von deinem HERRN, damit du Leute ermahnst, zu denen vor dir kein Warner kam, damit sie Rechtleitung finden.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdvii{ALLAH ist Derjenige, Der die Himmel, die Erde und das, was zwischen ihnen ist, in sechs Ay-yam erschuf, dann wandte ER Sich Al'ahrsch zu. Für euch gibt es anstelle von Ihm weder Wali noch Fürbittenden. Wollt ihr euch etwa nicht entsinnen?!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdviii{Er organisiert die Angelegenheit vom Himmel zur Erde, dann steigt sie zu Ihm in einem Yaum, dessen Länge eintausend Jahre ist von dem, was ihr zählt.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdix{Dieser ist Der Allwissende über das Verborgene und das Sichtbare, Der Allwürdige, Der Allgnädige.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdx{ER ist Derjenige, Der alles gut machte, das ER erschuf, und Der die Erschaffung des Menschen mit Lehm begann,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdxi{dann machte ER seine Nachkommenschaft aus einem Auszug einer geringgeschätzten Flüssigkeit.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdxii{Dann vollendete ER ihn und hauchte in ihn einen Ruhh von Ihm. Und ER machte euch das Gehör, die Augen und den Verstand. Doch nur ein wenig erweist ihr euch dankbar.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdxiii{Und sie sagten: ``Wenn wir auf Erden verloren gehen, werden wir etwa eine neue Erschaffung erfahren?" Nein, sondern sie sind gegenüber der Begegnung mit ihrem HERRN kufr-betreibend.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdxiv{Sag: ``Der Engel des Todes, der für euch zuständig gemacht wurde, wird eure (Seelen) vollständig einziehen, dann werdet ihr zu eurem HERRN zurückgebracht."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdxv{Und würdest du doch nur sehen, wenn die schwer Verfehlenden ihre Köpfe vor ihrem HERRN verneigen: ``Unser HERR! Wir sahen ein und hörten, so schicke uns zurück, um gottgefällig Gutes zu tun. Wir haben nun Gewißheit."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdxvi{Und würden WIR es wollen, hätten WIR jeder Seele ihre Rechtleitung zuteil werden lassen. Doch das Gesagte von Mir ist Rechtens: ``Gewiß, ICH werde Dschahannam mit Dschinn und Menschen, allesamt vollmachen."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdxvii{So erfahrt (die Peinigung) dafür, daß ihr die Begegnung dieses euren Tages vergessen habt. Gewiß, WIR berücksichtigen euch nicht. Und erfahrt die Peinigung in Ewigkeit für das, was ihr zu tun pflegtet.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdxviii{Den Iman an Unsere Ayat verinnerlichen nur diejenigen, die, wenn sie an sie erinnert wurden, in Sudschud niederfielen und mit dem Lob ihres HERRN lob-priesen, während sie sich nicht in Arroganz erheben.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdxix{Sie bleiben ihren Schlafplätzen öfters fern. Sie richten Bittgebete an ihren HERRN aus Ehrfurcht und aus Zuversicht, und vom Rizq, das WIR ihnen gewährten, geben sie.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdxx{So weiß kein Mensch, was für ihn an Freude verborgen wurde als Belohnung für das, was sie zu tun pflegten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdxxi{Ist etwa derjenige, der iman-erfüllt war, gleich demjenigen, der Fasiq war?! Sie werden bestimmt nicht gleich sein.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdxxii{Hinsichtlich derjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, für diese sind Dschannat der eigentlichen Unterkunft als Aufenthaltsort bestimmt für das, was zu tun pflegten.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdxxiii{Doch hinsichtlich derjenigen, die Fisq betrieben, deren Unterkunft ist das Feuer. Immer wieder wenn sie von ihm herausgehen wollen, werden sie darin zurückgeschickt und ihnen wird gesagt: ``Kostet die Peinigung des Feuers, das ihr abzuleugnen pflegtet."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdxxiv{Und WIR werden sie doch von der kleineren Peinigung vor der größeren Peinigung erfahren lassen, damit sie umkehren.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdxxv{Und keiner begeht mehr Unrecht als derjenige, der an die Ayat seines HERRN erinnert wird, dann sich von ihnen abwendet. Gewiß, WIR werden an den schwer Verfehlenden Vergeltung üben.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdxxvi{Und gewiß, bereits ließen WIR Musa die Schrift zuteil werden, so habe keine Zweifel an seiner Begegnung! Und WIR machten sie zur Rechtleitung für die Kinder Israils.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdxxvii{Und WIR machten von ihnen Imame, die nach Unserer Anweisung zur Rechtleitung aufriefen, als sie sich in Geduld übten. Und sie pflegten Gewißheit an Unseren Ayat zu haben.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdxxviii{Gewiß, dein HERR ist Derjenige, Der zwischen ihnen am Tag der Auferstehung richten wird über das, worüber sie uneins zu sein pflegten.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdxxix{Ist es ihnen etwa noch nicht wahrnehmbar geworden, wie vieleWIR vor ihnen an Generationen, durch deren Wohnstätten sie gehen, zugrunde richteten?! Gewiß, darin sind doch Ayat. Vernehmen sie etwa nichts?!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdxxx{Haben sie etwa nicht gesehen, daß WIR das Wasser in das dürre Land leiten lassen, dann lassen WIR mit ihm Gewächs hervorsprießen, von dem ihre An'am und sie selbst essen?! Sehen sie etwa nicht?!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdxxxi{Und sie sagen: ``Wann ist dieser Sieg, solltet ihr wahrhaftig sein?"\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdxxxii{Sag: ``Am Tag des Sieges wird denjenigen, die Kufr betrieben haben, weder ihr Iman nützen, noch wird ihnen Aufschub gewährt."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdxxxiii{So halte dich von ihnen fern, und warte, denn sie warten auch.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdxxxiv{\basmalahde Prophet! Handle Taqwa gemäß ALLAH gegenüber und höre nicht auf die Kafir und die Munafiq! Gewiß, ALLAH ist immer allwissend, allweise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdxxxv{Und folge dem, was dir von deinem HERRN an Wahy zuteil wurde. Gewiß, ALLAH ist immer dessen, was ihr tut, allkundig.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdxxxvi{Und übe Tawakkul ALLAH gegenüber! Und ALLAH genügt als Wakil.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdxxxvii{ALLAH setzte keinem Mann zwei Herzen in seinem Innern ein. Und ER machte eure Ehefrauen, denen ihr (die Formel von) Zihar aussprecht, nicht zu euren Müttern. Ebenfalls machte ER die nach euch Genannten nicht zu euren Söhnen. Dies ist euer Wort aus euren Mündern. Und ALLAH sagt dasWahre und ER leitet zum wahren Weg recht.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdxxxviii{Nennt sie nach ihren (leiblichen) Vätern! Dies ist gerechter vor ALLAH. Und solltet ihr für sie keine Väter kennen, so sind sie eure Brüder im Din und eure Maula. Und euch trifft keine Verfehlung in dem, wo ihr Fehler schon gemacht habt, sondern nur da, wo eure Herzen es beabsichtigten. Und ALLAH ist immer allvergebend, allgnädig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdxxxix{Der Prophet steht den Mumin näher als deren Selbst, und seine Ehefrauen sind ihre Mütter. Und diejenigen von der Verwandtschaft, die einen von ihnen stehen den anderen näher nach ALLAHs Schrift (hinsichtlich der Erbschaft) vor den (anderen) Mumin und den Hidschra- Unternehmenden, außer daß ihr euren Wali etwas Gebilligtes (im Testament) zukommen lassen wollt. Dies war in der Schrift niedergeschrieben.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdxl{Und (erinnere daran), als WIR von den Propheten ihr Gelöbnis entgegennahmen, von dir und von Nuh, Ibrahim, Musa und 'Isa Ibnu-Maryam - und WIR nahmen von ihnen ein äußerst verbindliches Gelöbnis entgegen -\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdxli{damit ER die Wahrhaftigen nach der Wahrhaftigkeit ihnen gegenüber fragt. Und ER bereitete für die Kafir eine qualvolle Peinigung.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdxlii{Ihr, die den Iman verinnerlicht habt! Gedenkt ALLAHs Wohltat euch gegenüber, als zu euch Soldaten kamen, und WIR dann gegen sie einen Wind und Soldaten entsandten, die ihr nicht gesehen habt. Und ALLAH ist immer dessen, was ihr tut, allsehend.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdxliii{Als sie zu euch von über euch und von unter euch kamen, und als die Blicke abgeirrt und die Herzen die Kehlen erreicht haben und ihr von ALLAH doch manches gedachtet habt,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdxliv{da wurden die Mumin geprüft und von einer heftigen Ergriffenheit ergriffen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdxlv{Und (erinnere daran), als die Munafiq und diejenigen, in deren Herzen Krankheit war, sagten: ``ALLAH und Sein Prophet versprachen uns nichts außer Täuschendem.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdxlvi{Ebenfalls als eine Gruppe von ihnen sagte: ``Bewohner von Yathrib! Es gibt keinen Platz für euch hier, so kehrt um!" Und eine Gruppe von ihnen baten den Gesandten um Erlaubnis und sagten: ``Unsere Häuser sind nicht befestigt." Während sie nicht unbefestigt sind. Sie wollten doch nur die Flucht ergreifen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdxlvii{Und würde sie (die Stadt) von ihren Seiten wider sie erstürmt, sie dann zur Fitna aufgefordert werden, würden sie ihr doch nachkommen und wären dabei nur ein wenig geblieben.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdxlviii{Und gewiß, bereits gelobten sie vorher ALLAH: ``Sie werden den Rücken nicht flüchtend umkehren." Und das ALLAH gegenüber gelobte Versprechen wird immer verantwortet.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdxlix{Sag: ``Die Flucht wird euch sicher nicht nützen, solltet ihr vor dem Tod oder dem Töten fliehen." Und dann werdet ihr nur ein wenig euch vergnügen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdl{Sag: ``Wer ist dieser, der euch vor ALLAH schützt, sollte ER euch Schaden zufügen wollen oder euch Gnade erweisen wollen?" Und sie finden für sich anstelle von ALLAH weder Wali noch Beistehenden.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdli{Bereits kennt ALLAH die Behindernden von euch und diejenigen, die ihren Brüdern sagen: ``Kommt zu uns!  Und sie nähern sich dem Kampf nur ein wenig.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdlii{Sie sind geizig euch gegenüber, und wenn die Angst kommt, siehst du sie dich anschauen, ihre Augen drehen sich wie derjenige, der in Ohnmacht vor dem Tod fällt. Und wenn die Angst weggeht, kränken sie euch mit scharfen Zungen wie Geizige dem Guten gegenüber. Diese verinnerlichten nicht den Iman, so ließ ALLAH ihre Handlungen nichtig werden. Und dies ist für ALLAH immer etwas Leichtes.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdliii{Sie denken, daß Al-ahzab noch nicht gegangen wären. Und sollten Al-ahzab wieder kommen, würden sie sich wünschen: sie wären zu den Wüstenarabern gegangen und hätten da nach euren Nachrichten gefragt. Und wären sie bei euch gewesen, würden sie doch nur ein wenig mitkämpfen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdliv{Gewiß, bereits gab es für euch mit dem Gesandten ALLAHs doch ein hervorragendes Vorbild für jeden, der auf ALLAH und den jenseitigen Tag hofft und ALLAHs öfters gedenkt.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdlv{Und als die Mumin Al-ahzab sahen, sagten sie: ``Dies ist das, was uns ALLAH und Seine Gesandten versprachen, und ALLAH und Sein Gesandter sagten die Wahrheit." Und dies ließ sie nur noch mehr Iman und Ergebenheit gewinnen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdlvi{Und unter den Mumin sind Männer, die wahrhaftig waren dem gegenüber, was sie ALLAH gelobten. Manche von ihnen starben und manche von ihnen warten noch darauf. Doch sie haben nichts (von ihrem Versprechen) abgeändert.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdlvii{Es geschah so), damit ALLAH den Wahrhaftigen für ihreWahrhaftigkeit vergilt und die Munafiq peinigt, wenn ER will, oder ER vergibt ihnen. Gewiß, ALLAH ist immer allvergebend, allgnädig.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdlviii{Und ALLAH wehrte diejenigen, die Kufr betrieben haben, mit ihrer Wut ab. Sie erreichten nichts Gutes. Und ALLAH ersparte den Mumin den Kampf. Und ALLAH ist immer allkraftvoll, allwürdig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdlix{Und ALLAH ließ diejenigen von den Schriftbesitzern, die ihnen (Al-ahzab) den Rücken stärkten, von ihren Burgen heruntersteigen, und ER jagte Schrecken in ihre Herzen ein - eine Gruppe habt ihr getötet und eine Gruppe habt ihr gefangen genommen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdlx{Und ER ließ euch ihr Land, ihre Wohnstätten und ihr Vermögen beerben sowie ein Land, das ihr noch nicht betreten habt. Und ALLAH ist immer über alles allmächtig.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdlxi{Prophet! Sag deinen Ehefrauen: ``Wenn ihr das diesseitige Leben und dessen Schmuck anstrebt, dann kommt, dann gebe ich euch von den Verbrauchsgütern und lasse mich von euch auf gütige Weise scheiden.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdlxii{Und wenn ihr ALLAH, Seinen Gesandten und die jenseitige Wohnstätte anstrebt, so bereitete ALLAH für die Muhsin unter euch eine gewaltige Belohnung.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdlxiii{Ehefrauen des Propheten! Welche von euch eine bewiesene Abscheulichkeit vollbringt, für diese wird die Peinigung verzweifacht. Und dies ist für ALLAH immer ein Leichtes.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdlxiv{Und wer von euch ALLAH und Seinem Gesandten gehorcht und gottgefällig Gutes tut, (dieser) gewähren WIR ihr ihre Belohnung zweifach, und WIR bereiteten für sie ein edles Rizq.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdlxv{Ehefrauen des Propheten! Ihr seid nicht wie irgendeine (Gruppe) der Frauen, wenn ihr Taqwa gemäß handelt. Also meidet Schmeichlerisches beim Sprechen, damit nicht derjenige, in dessen Herzen Krankheit ist, begierig wird. Und sagt nur gebilligte Worte!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdlxvi{Und haltet euch in euren Wohnstätten auf und wandelt nicht mit entblößten Reizen wie das reize-entblößte Wandeln der Dschahiliya-Zeit. Und verrichtet das rituelle Gebet ordnungsgemäß, entrichtet die Zakat und gehorcht ALLAH und Seinem Gesandten! ALLAH will doch nur von euch das Unreine wegnehmen - Ah-lalbait! - und ER will euch höchstmöglich reinigen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdlxvii{Und erinnert euch an das, was in euren Wohnstätten von ALLAHs Ayat und der Weisheit vorgetragen wird. Gewiß, ALLAH ist immer allgütig, allkundig.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdlxviii{Gewiß, die islam-praktizierenden Männer und die islam-praktizierenden Frauen, die iman-verinnerlichenden Männer und die iman-verinnerlichenden Frauen, die gehorchenden Männer und die gehorchenden Frauen, die wahrhaftigenMänner und die wahrhaftigen Frauen, die sich in Geduld übenden Männer und die sich in Geduld übenden Frauen, die sich hingebenden Männer und die sich hingebenden Frauen, die spendenden Männer und die spendenden Frauen, die fastenden Männer und die fastenden Frauen, die sich keusch haltenden Männer und die sich keusch haltenden Frauen und die ALLAHs viel gedenkenden Männer und die ALLAHs viel gedenkenden Frauen, für diese hat ALLAH Vergebung und ge-waltige Belohnung bereitet.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdlxix{Keineswegs gebührt es einem Mumin oder einer Mumina, wenn ALLAH und Sein Gesandter eine Angelegenheit bestimmten, daß es für sie noch eine Wahl in ihrer Angelegenheit gibt. Und wer sich ALLAH und Seinem Gesandten widersetzt, ist bereits mit eindeutigem Irrtum abgeirrt.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdlxx{Und (erinnere daran), als du demjenigen, dem ALLAH Wohltaten erwies und dem du Wohltaten erwiesen hast, sagtest: ``Behalte deine Ehefrau und handle Taqwa gemäß ALLAH gegenüber!" Doch du hast in deinem Inneren verheimlicht, was ALLAH offenlegen wird, und du hast dich vor den Menschen gefürchtet, während ALLAH doch mehr berechtigt ist, daß du dich vor Ihm fürchtest. Und nachdem Zaid an ihr kein Interesse mehr gehabt hatte (und sich von ihr geschieden hatte), ließen WIR dich sie heiraten, damit es keine Unannehmlichkeit für die Mumin wird, die (Ex-)Ehefrauen der nach ihnen Genannten zu heiraten, wenn diese kein Interesse mehr an ihnen haben (und sich von ihnen geschieden haben). Und ALLAHs Bestimmung wird immer umgesetzt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdlxxi{Keineswegs ist es für den Propheten eine Unannehmlichkeit in dem, was ALLAH ihm auferlegte. Dies ist ALLAHs Handlungsweise mit denjenigen, die vorher vergingen. Und ALLAHs Bestimmung ist eine festgelegte Bestimmung.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdlxxii{Diese sind diejenigen, die ALLAHs Botschaften verkünden und Ehrfurcht vor Ihm haben, und sie fürchten sich vor keinem außer vor ALLAH. Und ALLAH genügt als Schutz-Gewährender.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdlxxiii{Muhammad war nie der Vater eines eurer Männer, sondern er ist ALLAHs Gesandter und der Abschließende aller Propheten. Und ALLAH ist immer über alles allwissend.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdlxxiv{Ihr, die den Iman verinnerlicht habt! Gedenkt ALLAHs mit häufigem Gedenken,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdlxxv{und lobpreist ihn nach dem Sonnenaufgang und vor dem Sonnenuntergang!\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdlxxvi{ER ist Derjenige, Der euch Gnade erweist und Seine Engel erbitten sie für euch, damit ER euch von den Finsternissen ins Licht herausbringt. Und ER ist immer den Mumin gegenüber allgnädig.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdlxxvii{Ihr Gruß am Tag, wenn sie auf Ihn treffen, ist Salam. Und ER bereitete für sie eine edle Belohnung.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdlxxviii{Prophet! Gewiß, WIR entsandten dich als einen Zeugen, als Überbringer froher Botschaft, als Ermahner,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdlxxix{als Einladenden zu ALLAH nach Seiner Zustimmung und als leuchtende Lampe.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdlxxx{Und überbringe den Mumin eine frohe Botschaft, daß für sie von ALLAH unermeßliche Gunst bestimmt ist.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdlxxxi{Und höre nicht auf die Kafir und die Munafiq, laß ab von ihrer Schädigung und übe Tawakkul ALLAH gegenüber! Und ALLAH genügt als Wakil.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdlxxxii{Ihr, die den Iman verinnerlicht habt! Wenn ihr die iman-verinnerlichenden Frauen heiratet und dann euch von ihnen durch Talaq-Scheidung trennt, bevor ihr sie intim berührt habt, dann habt ihr sie zu keiner 'Idda zu verpflichten. Also gebt ihnen das ihnen Zustehende und scheidet euch von ihnen auf gütige Weise.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdlxxxiii{Prophet! Gewiß, WIR erklärten dir für halal deine Ehefrauen, denen du ihre Brautgabe gabst, und diejenigen, die dir gehören, von denen, die ALLAH dir durch den Krieg zuteil werden ließ, die Töchter deines Onkels väterlicherseits, die Töchter deiner Tanten väterlicherseits, die Töchter deines Onkels mütterlicherseits und die Töchter deiner Tanten mütterlicherseits, diejenigen, die mit dir Hidschra unternahmen, und eine iman-verinnerlichende Frau, wenn sie sich selbst dem Propheten schenkt, wenn der Prophet sie heiraten will, dieses gilt nur für dich alleine unter Ausschluß der Mumin  bereits wissen WIR, was WIR ihnen vorschrieben ihren Ehefrauen und denjenigen gegenüber, die ihnen gehören -, damit es für dich keine Unannehmlichkeit gibt. Und ALLAH ist immer allvergebend, allgnädig.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdlxxxiv{Du läßt von ihnen warten, die du willst, und du nimmst zu dir, die du willst. Und wenn du verlangst nach einer von denen, die du nicht mehr zu dir nahmst, trifft dich keine Verfehlung. Dies ist näher dazu, daß sie sich freuen, nicht traurig werden und mit dem zufrieden werden, was du ihnen allen zuteil werden ließt. Und ALLAH weiß, was in euren Herzen ist. Und ALLAH ist immer allwissend, allnachsichtig.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdlxxxv{Für dich sind die (anderen) Frauen nicht mehr halal danach, auch nicht, daß du an ihrer Stelle andere Gattinnen nimmst, selbst dann nicht, sollte ihre Schönheit dir gefallen außer denjenigen, die dir gehören. Und ALLAH ist immer über alles aufmerksam.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdlxxxvi{Ihr, die den Iman verinnerlicht habt! Betretet nicht die Wohnstätten des Propheten, es sei denn, es wird euch Erlaubnis zu einem Essen gegeben, ohne die Zeit seiner Zubereitung abzuwarten - doch wenn ihr eingeladen werdet, dann tretet ein, und wenn ihr gegessen habt, dann geht auseinander - und ohne es euch bei einer Unterhaltung gemütlich zu machen. Dies belästigt den Propheten, und er ist vor euch beschämt, aber ALLAH findet nichts Beschämendes an der Wahrheit. Und wenn ihr sie (die Ehefrauen des Gesandten) nach einem Verbrauchsgut fragt, dann fragt sie von hinter etwas Trennendem, dies ist reiner für eure Herzen und ihre Herzen. Und es gebührt euch keineswegs, daß ihr ALLAHs Gesandten belästigt, auch nicht daß ihr seine Ehefrauen nach ihm jemals heiratet. Gewiß, dies ist bei ALLAH eine Ungeheuerlichkeit.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdlxxxvii{Wenn ihr etwas offenlegt oder verheimlicht, da ist ALLAH gewiß über alles allwissend.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdlxxxviii{Es trifft sie keine Verfehlung weder hinsichtlich ihrer Väter, noch ihrer Söhne, noch ihrer Brüder, noch der Söhne ihrer Brüder, noch der Söhne ihrer Schwestern, noch ihrer Frauen, noch derjenigen, die ihnen gehören. Und handelt (ihr Frauen des Propheten) Taqwa gemäß ALLAH gegenüber! Gewiß, ALLAH ist immer über alles Zeuge.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmdlxxxix{Gewiß, ALLAH gewährt dem Propheten Gnade und die Engel erbitten sie für ihn. Ihr, die den Iman verinnerlicht habt! Macht für ihn Salah und begrüßt (ihn mit) einer Salam-Begrüßung!\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmdxc{Gewiß, diejenigen, die ALLAH und Seinen Gesandten kränken wollen, verfluchte ALLAH im Diesseits und im Jenseits und bereitet für sie eine erniedrigende Peinigung.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmdxci{Und diejenigen, welche die männlichen und die weiblichen Mumin kränken mit dem, was sie nicht bewirkt haben, diese luden auf sich bereits eine Verleumdung und eine klare Verfehlung.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmdxcii{Prophet! Sag zu deinen Ehefrauen, zu deinen Töchtern und zu den Frauen der Mumin, daß sie von ihren Dschilbab über sich ziehen. Dies ist eher daran, daß sie erkannt und dann nicht belästigt werden. Und ALLAH ist immer allvergebend, allgnädig.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmdxciii{Wenn die Munafiq, diejenigen, in denen Herzen Krankheit ist, und die Anti-Propagatoren in Madina nicht aufhören, dann werden WIR dich doch gegen sie aufbringen, dann werden sie darin (in Al-madina) nicht mehr mit dir benachbart sein, außer für kurze Zeit.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmdxciv{Sie werden verflucht sein. Überall, wo sie gefunden werden, werden sie ergriffen und gnadenlos getötet.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmdxcv{Dies ist ALLAHs Handlungsweise mit denjenigen, die vorher vergingen, und du wirst für ALLAHs Handlungsweise gewiß keine Änderung finden.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmdxcvi{Die Menschen fragen dich nach der Stunde. Sag: ``Das Wissen von ihr ist nur bei ALLAH." Und was läßt dich wissen, vielleicht liegt die Stunde nahe.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmdxcvii{Gewiß, ALLAH verfluchte die Kafir und bereitete für sie Gluthitze.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmdxcviii{Darin werden sie ewig für immer bleiben. Sie werden weder Wali, noch Beistehenden finden.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmdxcix{An dem Tag, wenn ihre Gesichter im Feuer ständig gedreht werden, werden sie sagen: ``Hätten wir doch ALLAH gehorcht, und hätten wir doch dem Gesandten gehorcht!"\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmdc{Und sie sagten: ``Unser HERR! Wir gehorchten unseren Herrschern und unseren Führern, so ließen sie uns vom Weg abirren.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmdci{Unser HERR! Gib ihnen Doppeltes von der Peinigung und verfluche sie mit einem großen Fluch!"\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmdcii{Ihr, die den Iman verinnerlicht habt! Seid nicht wie diejenigen, die Musa verleumdeten, dann erklärte ihn ALLAH von dem, was sie ihm unterstellten, für unschuldig. Und er war bei ALLAH hochgeschätzt.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmdciii{Ihr, die den Iman verinnerlicht habt! Handelt Taqwa gemäß ALLAH gegenüber und sagt ein zutreffendes Wort,\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmdciv{dann läßt ER für euch eure Taten gut machen und vergibt euch eure Verfehlungen. Und wer ALLAH und Seinem Gesandten gehorcht, der erzielte bereits einen gewaltigen Erfolg.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmdcv{Gewiß, würden WIR Al-amana den Himmeln, der Erde und den Bergen anbieten, würden sie sich weigern, es auf sich zu nehmen, und würden sich davor fürchten, doch der Mensch nahm es auf sich. Gewiß, er war äußerst unrecht-begehend, äußerst unwissend.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmdcvi{Es geschah), damit ALLAH die Munafiq-Männer, die Munafiq-Frauen, dieMuschrik-Männer und dieMuschrik-Frauen peinigt. Und ALLAH vergibt den Mumin-Männern und den Mumin-Frauen. Und ALLAH ist immer allvergebend, allgnädig.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmdcvii{\basmalahde Alles Lob gebührt ALLAH, Dem alles gehört, was in den Himmeln und was auf Erden ist. Auch Ihm gebührt alles Lob im Jenseits. Und ER ist Der Allweise, Der Allkundige.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdcviii{ER weiß, was in die Erde hineingeht, was aus ihr herauskommt, und was vom Himmel hinabkommt, und in ihn hinaufsteigt. Und ER ist Der Allgnädige, Der Allvergebende.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdcix{Und diejenigen, die Kufr betrieben haben, sagten: ``Die Stunde wird zu uns nicht kommen." Sag: ``Doch, mit Sicherheit, bei meinem HERRN! Sie wird doch zu euch kommen. ER ist Der Allwissende vom Verborgenen. Verborgen bleibt Ihm nicht das Gewicht eines Stäubchens weder in den Himmeln, noch auf Erden, auch weder kleiner als dies, noch größer als dies, außer daß es in einer deutlichen Schrift ist,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdcx{damit ER denjenigen vergilt, die den Iman verinnerlicht und gottgefällig Gutes getan haben. Für diese ist Vergebung und edles Rizq bestimmt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdcxi{Doch diejenigen, die gegen Unsere Ayat in Widerspenstigkeit vorgehen, für diese ist eine Peinigung aus qualvoller Bestrafung bestimmt.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdcxii{Und diejenigen, denen das Wissen zuteil wurde, sehen das, was dir von deinem HERRN hinabgesandt wurde, als das Wahre und daß es zum Wege Des Allwürdigen, Des Alllobenswürdigen rechtleitet.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdcxiii{Und diejenigen, die Kufr betrieben haben, sagten: ``Sollen wir euch zu einem Mann führen, der euch mitteilt, daß ihr, nachdem ihr in viele kleine Stücke auseinandergenommen wurdet, doch wieder in einer neuen Schöpfung seid?\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdcxiv{Erdichtete er etwa im Namen ALLAHs Lügen oder ist er geistesgestört?" Nein, sondern diejenigen, die den Iman an das Jenseits nicht verinnerlichen, sind in der Peinigung und im weiten Irregehen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdcxv{Sahen sie etwa nicht das, was vor ihnen und was hinter ihnen von dem Himmel und der Erde ist?! Wenn WIR wollen, lassen WIR mit ihnen die Erde versinken oder lassen WIR über ihnen Bruchteile vom Himmel fallen. Gewiß, darin ist doch eine Aya für jeden umkehrenden Diener.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdcxvi{Und gewiß, bereits WIR ließen Dawud von Uns Gunst zuteil werden: ``Ihr Berge! wiederholt nach ihm (die Lobpreisung), sowie ihr Vögel!" UndWIR ließen für ihn das Eisen weich werden:\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdcxvii{``Stelle Panzerhemden her und füge die Ringe nach Maß ineinander." Und tut gottgefällig Gutes! Gewiß, ICH bin dessen, was ihr tut, allsehend.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdcxviii{Auch für Sulaiman (machten WIR fügbar) den Wind, dessen Ausgang war nach der Morgendämmerung ein Monatsritt lang, und dessen Zurückkommen abends war ein Monatsritt lang. Und WIR ließen für ihn die Quelle des geschmolzenen Kupfers fließen. Und von den Dschinn waren manche, die für ihn mit der Zustimmung ALLAHs arbeiteten. Und wer von ihnen von unserer Anweisung abweicht, ihn lassen WIR von der Peinigung der Gluthitze erfahren.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdcxix{Sie stellten für ihn her, was er wollte von imponierenden Gebäuden, Statuen, Schüsseln wie große Teiche und unbewegliche Kochtöpfe. Vollzieht - Familie Dawuds! - Handlungen aus Dankbarkeit! Und nur Wenige von Meinen Dienern sind die sich Bedankenden.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdcxx{Und alsWIR für ihn den Tod bestimmten, hat sie niemand auf seinen Tod hingewiesen außer dem Holzwurm, der aus seinem Stock fraß. Und nachdem er niedergefallen war, wußten die Dschinn, daß - wenn sie das Verborgene kennen würden - sie gewiß nicht unter der erniedrigenden Peinigung blieben.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdcxxi{Gewiß, bereits gab es für Sabaa in ihrem Wohnort eine Aya: zwei Dschannat zur rechten und zur linken (Seite): ``Esst vom Rizq eures HERRN und seid Ihm gegenüber dankbar!" Dies ist eine gute Ortschaft und ER ist ein allvergebender HERR.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdcxxii{Dann wandten sie sich ab, dann entsandten WIR ihnen den Sturzbach des Damms und tauschten ihnen ihre beiden Dschannat ein gegen zwei Dschannat von bitteren Früchten, von Tamarisken und von ein wenig Lotosbäumen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdcxxiii{Dies vergalten WIR ihnen für das, was sie an Kufr betrieben. Und vergelten WIR es so außer dem äußerst Kufr-Betreibenden?!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdcxxiv{Und WIR ließen zwischen ihnen und den Ortschaften, die WIR mit Baraka erfüllten, bekannte Ortschaften sein und bestimmten die Reise durch sie. Reist durch sie wenige Nächte und Tage lang in Sicherheit!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdcxxv{Und sie sagten: ``Unser HERR! Laß große Entfernungen zwischen unseren Reisezielen sein! Und sie begingen Unrecht, so machten WIR sie zum Objekt von Erzählungen und ließen sie in vielen kleinen Gruppen auseinandergehen. Gewiß, darin sind doch Ayat für jeden äußerst sich Bedankenden, sich äußerst in Geduld Übenden.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdcxxvi{Und gewiß, bereits bewahrheitet hat sich sie betreffend die Vermutung von Iblis, so folgten sie ihm außer einer Gruppe von den Mumin.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdcxxvii{Und er hatte doch über sie keinerlei Verfügung. Nur damit WIR den kenntlich machen, der den Iman an das Jenseits verinnerlicht, gegenüber dem, der darüber im Zweifel ist. Und dein HERR ist gewiß jeder Sache äußerst bewahrend.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdcxxviii{Sag: ``Richtet nur Bittgebete an die, die ihr anstelle von ALLAH angebt! Sie verfügen nicht einmal über das Gewicht eines Stäubchens weder in den Himmeln, noch auf Erden. Sie haben an beiden keinerlei Beteiligung, und ER hat unter ihnen keinen Verstärkenden.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdcxxix{Die Fürbitte nützt bei Ihm nicht, es sei denn, von dem, dem ER Erlaubnis dazu gewährt. Dann nachdem von ihren Herzen der Schrecken weggenommen worden war, sagten sie: ``Was sagte denn euer HERR?" Sie sagten: ``Die Wahrheit!" Und ER ist Der Allhöchste, Der Allgrößte.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdcxxx{Sag: ``Wer gewährt euch Rizq von den Himmeln und der Erde?" Sag: ``ALLAH! Und gewiß, wir sind oder ihr seid doch entweder in Rechtleitung oder im eindeutigen Abirren."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdcxxxi{Sag: ``Weder Ihr wer-det nach dem zur Rechenschaft gezogen, was wir schwer verfehlten, noch wir werden nach dem zur Rechenschaft gezogen, was ihr tut."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdcxxxii{Sag: ``Unser HERR wird uns zusammenbringen, dann gemäß der Wahrheit zwischen uns richten. Und ER ist Der Richtende, Der Allwissende."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdcxxxiii{Sag: ``Zeigt mir diejenigen, die ihr Ihm an Partner beigeselltet?" Nein, mit Sicherheit! Nein, sondern ER ist ALLAH, Der Allwürdige, Der Allweise.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdcxxxiv{Und nicht entsandten WIR dich außer für alle Menschen als Überbringer froher Botschaft und als Warner. Doch die meisten Menschen wissen es nicht.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdcxxxv{Und sie sagen: ``Wann ist diese Androhung, solltet ihr wahrhaftig sein?"\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdcxxxvi{Sag: ``Für euch ist ein Termin eines Tages bestimmt, den ihr weder um eine Stunde hinauszögern könnt, noch vorverlegen."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdcxxxvii{Und diejenigen, die Kufr betrieben haben, sagten: ``Wir werden niemals den Iman verinnerlichen weder an diesen Quran, noch an das, was vor ihm war." Und (Erstaunliches wirst du erleben), würdest du nur sehen, wenn die Unrecht-Begehenden vor ihrem HERRN eingesperrt werden, während die einen den anderen das Gesagte beantworten. Diejenigen, die unterdrückt wurden, sagen denjenigen, die sich in Arroganz erhoben: ``Wäret ihr nicht, wären wir doch Mumin gewesen!"\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdcxxxviii{Diejenigen, die sich in Arroganz erhoben, sagten denjenigen, die unterdrückt wurden: ``Waren wir etwa diejenigen, die euch von der Rechtleitung abhielten, nachdem sie zu euch gekommen war?! Nein, sondern ihr wart schwer Verfehlende!"\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdcxxxix{Und diejenigen, die unterdrückt wurden, sagten denjenigen, die sich in Arroganz erhoben: ``Nein, sondern es war die List während der Nacht und des Tages, als ihr uns angewiesen habt, ALLAH gegenüber Kufr zu betreiben und Ihm etwas als Ebenbürtiges zuzuschreiben." Und sie verbargen die Reue, als sie die Peinigung sahen. Und WIR ließen Fesseln anbringen um die Nacken derjenigen, die Kufr betrieben haben. Wird ihnen etwa anderes vergolten außer dem, was sie zu tun pflegten?!\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdcxl{UndWIR entsandten zu keiner Ortschaft einen Warner, ohne daß ihre Luxurierenden sagten: ``Gewiß, wir sind dem gegenüber, womit ihr entsandt wurdet, Kufr- Betreibende!"\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdcxli{Und sie sagten: ``Wir haben mehr Kinder und Vermögen und wir werden nie gepeinigt!"\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdcxlii{Sag: ``Mein HERR gewährt viel Rizq, wem ER will, und ER gewährt wenig. Doch die meisten Menschen wissen nicht."\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdcxliii{Und weder euer Vermögen, noch eure Kinder sind solche, die euch Annäherung an Uns ermöglichen. Nur diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, für diese ist das Vielfache an Belohnung für das, was sie taten. Und sie sind in den Unterkünften sicher.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdcxliv{Doch diejenigen, die gegen Unsere Ayat in Widerspenstigkeit vorgehen, diese werden der Peinigung ausgeliefert.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdcxlv{Sag: ``Mein HERR gewährt viel Rizq, wem ER will von Seinen Dienern, und ER gewährt ihm wenig. Und was ihr auch immer gebt, ER ersetzt es. Und ER ist Der Beste der Rizq-Gewährenden."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdcxlvi{Und an dem Tag, wenn ER sie alle versammelt, dann zu den Engeln sagt: ``Sind diese etwa diejenigen, die euch zu dienen pflegten?!"\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdcxlvii{Sagen sie: ``Gepriesen-erhaben bist DU! DU bist unser Wali anstelle von ihnen. Nein, sondern sie pflegten den Dschinn zu dienen. Die meisten von ihnen verinnerlichen den Iman an sie."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdcxlviii{Also an diesem Tag können die einen von euch den anderen weder nützen, noch schaden. Und WIR sagen zu denjenigen, die Unrecht begingen: ``Erfahrt die Peinigung des Feuers, welches ihr zu verleugnen pflegtet."\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdcxlix{Und wenn ihnen Unsere Ayat in Erklärtheit vorgetragen werden, sagen sie: ``Dieser ist doch nichts anderes als ein Mann, der euch von dem abhalten will, was eure Vorfahren zu dienen pflegten." Und sie sagten: ``Dies ist doch nichts anderes als erdichtete Ifk-Lüge." Und diejenigen, die Kufr betrieben haben, sagten über die Wahrheit, als sie zu ihnen kam: ``Dies ist doch nichts anderes als eindeutige Magie."\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdcl{Und WIR ließen ihnen keine Schriften zuteil werden, die sie studieren, und WIR entsandten zu ihnen vor dir keinen Warner.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdcli{Und diejenigen vor ihnen verleugneten, und diese erreichten nicht einmal ein Tausendstel von dem, was WIR ihnen (den anderen) zuteil werden ließen, dann bezichtigten sie Meine Gesandten der Lüge. Also wie war denn Meine Mißbilligung?!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdclii{Sag: ``Ich ermahne euch nur eines, daß ihr euch um ALLAHs Willen zu zweit oder einzeln bemüht, dann nachdenkt." Euer Gefährte leidet sicher nicht unter Geistesgestörtheit. ER ist doch nur ein Warner für euch vor einer harten Peinigung.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdcliii{Sag: ``Sollte ich von euch Belohnung verlangt haben, so gehört dies euch. Denn meine Belohnung obliegt nur ALLAH. Und ER ist über alles ein Zeuge."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdcliv{Sag: ``Mein HERR zielt mit der Wahrheit, ER ist Der Allwissende über die Verborgenen."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdclv{Sag: ``Die Wahrheit ist gekommen. Und das für nichtig Erklärte kann weder etwas beginnen, noch etwas wiederholen."\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdclvi{Sag: ``Sollte ich geirrt haben, so leite ich nur mich selbst irre. Und sollte ich Rechtleitung gefunden haben, so ist dies mit dem, was mir mein HERR an Wahy zuteil werden läßt. Gewiß, ER ist allhörend, nahe."\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdclvii{Und (du wirst Erstaunliches erleben), würdest du nur sehen, als sie sich erschreckten. Dann gibt es kein Entkommen. Und sie wurden aus einem nahen Ort ergriffen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdclviii{Und sie sagten: ``Wir verinnerlichten den Iman an ihn!" Und wie sollte ihnen das Nahekommen daran aus einem fernen Ort möglich sein,\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdclix{und bereits betrieben sie doch vorher Kufr an ihn?! Doch sie verwerfen das Verborgene aus einem fernen Ort.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdclx{Und es wurde getrennt zwischen ihnen und dem, was sie begehren, wie es mit ihren Parteifreunden vorher gemacht wurde. Gewiß, Sie pflegten im Verdacht schleichenden Zweifel zu sein.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdclxi{\basmalahde Alles Lob gebührt ALLAH, Dem Schöpfer der Himmel und der Erde, Der die Engel zu Gesandten mit zwei, drei und vier Flügeln machte. ER fügt der Schöpfung hinzu, was ER will. Gewiß, ALLAH ist über alles allmächtig.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdclxii{Was ALLAH den Menschen an Gnade freiläßt, so gibt es niemanden, der sie zurückhalten kann. Und was ER zurückhält, kann niemand nach Ihm freilassen. Und ER ist Der Allwürdige, Der Allweise.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdclxiii{Ihr Menschen! Erinnert euch an die Wohltaten ALLAHs euch gegenüber! Gibt es etwa einen Schöpfer außer ALLAH, der euch Rizq vom Himmel und auf Erden gewährt?! Es gibt keine Gottheit außer Ihm. Also wie werdet ihr denn belogen?!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdclxiv{Und sollten sie dich der Lüge bezichtigen, so wurden bereits vor dir Gesandte der Lüge bezichtigt. Und ALLAH unterliegen die Angelegenheiten.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdclxv{Ihr Menschen! Gewiß, ALLAHs Versprechen ist wahr. So lasst das diesseitige Leben euch nicht täuschen, und lasst keineswegs das Täuschende euch in ALLAH täuschen!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdclxvi{Gewiß, der Satan ist für euch ein Feind, so nehmt ihn als Feind! Er ruft seine Partei nur dazu, von den Weggenossen der Gluthitze zu sein.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdclxvii{Für diejenigen, die Kufr betrieben haben, ist eine harte Peinigung bestimmt. Und für diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, ist Vergebung und große Belohnung bestimmt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdclxviii{Ist derjenige, dem das Schlechte seines Tuns schön gemacht wurde, und dem es gefiel, (etwa mit dem Rechtgeleiteten gleich zu setzen)? Also gewiß, ALLAH läßt abirren, wen ER will, und leitet recht, wen ER will! So laß deine Seele nicht ihretwegen unter Trauer leiden. Gewiß, ALLAH ist allwissend über das, was sie bewerkstelligen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdclxix{Und ALLAH ist Derjenige, Der die Winde schickte, die dann Wolken aufwirbeln. Dann fuhren WIR sie zu einer toten Ortschaft, dann belebten WIR damit die Landschaft nach ihrem Tod. Solcherart ist die Erweckung.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdclxx{Wer die Würde anstrebt, so gehört ALLAH die Würde, insgesamt. Zu Ihm steigt das gute Wort, und das gottgefällig Guttun erhebt ER. Und für diejenigen, welche die gottmißfälligen Taten intrigieren, ist eine harte Peinigung bestimmt. Und die List dieser wird sicher nutzlos sein.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdclxxi{Und ALLAH erschuf euch aus Erde, dann aus einer Nutfa, dann machte euch zu Ehepaaren. Und keineWeibliche wird schwanger und sie entbindet nicht, es sei denn nach Seinem Wissen. Und weder das Greisenalter wird ein Greis erreichen, noch wird von Seinem Lebensalter abgezogen außer nach einem Register. Gewiß, dies ist für ALLAH leicht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdclxxii{Und einander nicht gleich sind die beiden Meere, dies ist süß, sehr angenehm, leicht beim Trinken, und dies ist salzig, bitter, und von beiden esst ihr zartes Fleisch und holt Schmuck heraus, den ihr tragt. Und du siehst die Schiffe darauf durchpflügend, damit ihr von Seiner Gunst anstrebt und damit ihr euch dankbar erweist.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdclxxiii{ER läßt die Nacht in den Tag einfließen und läßt den Tag in die Nacht einfließen. Und ER machte euch die Sonne und den Mond gratis fügbar, beide durchlaufen bis zu einer festgesetzten Frist. Dieser ist ALLAH, euer HERR. Ihm gehört die Herrschaft. Und diejenigen, an die ihr anstelle von Ihm Bittgebete richtet, verfügen nicht einmal über das Häutchen eines Dattelkerns.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdclxxiv{Wenn ihr sie ruft, hören sie euren Ruf nicht. Und würden sie hören, würden sie auf euch nicht reagieren. Und am Tag der Auferstehung werden sie eurem Schirk gegenüber Kufr betreiben. Und Mitteilungen bringt dir keiner wie Ein Allkundiger.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdclxxv{Ihr Menschen! Ihr seid die Bedürftigen ALLAH gegenüber. Und ALLAH ist Der absolut Autarke, Der Alllobenswürdige.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdclxxvi{Wenn ER will, läßt ER euch vergehen und bringt eine neue Schöpfung hervor.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdclxxvii{Und dies ist für ALLAHkeinesfalls unmöglich.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdclxxviii{Und keine verfehlende Seele trägt die Verfehlung einer anderen. Und sollte eine Schwerbelastete um (die Entlastung von) ihrer Last rufen, der (Gerufene) wird (ihr) davon nichts abnehmen, und selbst dann nicht, wäre er von den Nahverwandten. Eigentlich ermahnst du nur diejenigen, die Ehrfurcht vor ihrem HERRN im Verborgenen haben und das rituelle Gebet ordnungsgemäß verrichteten. Und wer sich rein hält, der hält sich doch nur für sich selbst rein. Und zu ALLAH ist das Werden.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdclxxix{Und einander nicht gleich sind der Blinde und der Sehende,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdclxxx{auch weder die Finsternisse, noch das Licht,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdclxxxi{auch weder der Schatten, noch die Hitze.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdclxxxii{Ebenfalls einander nicht gleich sind, weder die Lebendigen, noch die Toten. Gewiß, ALLAH läßt hören, wen ER will. Und du wirst kein Gehör verleihen denen, die in den Gräbern liegen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdclxxxiii{Du bist nichts anderes als ein Warner.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdclxxxiv{Gewiß, WIR entsandten dich mit der Wahrheit als Überbringer froher Botschaft und als Warner. Und es gibt keine Umma, außer daß bei ihr ein Warner war.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdclxxxv{Und sollten sie dich der Lüge bezichtigen, so verleugneten bereits diejenigen vor ihnen. Ihre Gesandten kamen zu ihnen mit den klaren Zeichen, mit den Zubur und mit der erleuchtenden Schrift.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdclxxxvi{Dann richtete ICH diejenigen, die Kufr betrieben haben, zugrunde. Also wie war denn Meine Mißbilligung?!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdclxxxvii{Hast du etwa nicht gesehen, daß ALLAH vom Himmel Wasser fallen ließ, dann brachten WIR damit Früchte hervor, deren Farben vielfältig sind, auch in den Felsenbergen weiße und rote Streifen, deren Farben vielfältig sind, sowie rabenschwarz?!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdclxxxviii{Auch (erschufen WIR) von den Menschen, den Tieren und den An'am, deren Farben vielfältig sind. Solcherart! Ehrfurcht ALLAH gegenüber haben unter Seinen Dienern nur dieWissenden. Gewiß, ALLAH ist allwürdig, allvergebend.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdclxxxix{Gewiß, diejenigen, die ALLAHs Schrift vortragen, das rituelle Gebet ordnungsgemäß verrichteten und vom Rizq, dasWIR ihnen gewährten - heimlich und öffentlich gaben, diese hoffen auf einen Handel, der nie stagniert.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdcxc{Damit ER ihnen ihre Belohnungen voll vergütet und von Seiner Gunst noch mehr gibt. ER ist allvergebend, reichlichst belohnend.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdcxci{Und das, was WIR dir an Wahy von der Schrift zuteil werden ließen, ist die Wahrheit, bestätigend für das vor ihm. Gewiß, ALLAH ist über Seine Diener doch allkundig, allsehend.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdcxcii{Dann ließen WIR die Schrift diejenigen erben, die WIR von Unseren Dienern auserwählten. So ist manch einer von ihnen sich selbst gegenüber ein Unrecht-Begehender, manch einer von ihnen ist gemäßigt und manch einer ist initiativ Eilender mit dem gottgefällig Guten mit ALLAHs Zustimmung. Dies ist die große Gunst.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdcxciii{'Adn-Dschannat werden sie betreten. Darin werden Sie mit Armbändern aus Gold und Perlen geschmückt. Und ihre Kleidung darin ist aus Seide.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdcxciv{Und sie sagten: ``Alhamdulillah: Alles Lob gebührt ALLAH, Der von Uns die Traurigkeit weggehen ließ. Gewiß, unser HERR ist doch allvergebend, reichlichst belohnend.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdcxcv{Derjenige, Der uns im Haus des eigentlichen Aufenthalts aus Seiner Gunst heraus wohnen ließ. Weder berührt uns darin Müdigkeit, noch berührt uns darin Abgespanntheit.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdcxcvi{Und für diejenigen, die Kufr betrieben haben, ist das Feuer von Dschahannam bestimmt. Weder werden sie verurteilt, so daß sie sterben, noch wird ihnen etwas von seiner Peinigung erleichtert. Solcherart vergelten WIR jedem äußerst Kufr-Betreibenden.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdcxcvii{Und sie schreien darin: ``Unser HERR! Laß uns herausbringen, damit wir gottgefällig Gutes tun, anderes als das, das wir zu tun pflegten." Haben WIR euch etwa nicht ein langes Lebensalter gewährt, während dessen derjenige sich erinnern konnte, der sich erinnern wollte?! Auch der Warner kam zu euch. Also erfahrt (die Peinigung)! Denn für die Unrecht-Begehenden gibt es keinen Beistehenden."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdcxcviii{Gewiß, ALLAH ist Der Allwissende über das Verborgene der Himmel und der Erde! Gewiß, ER ist allwissend über das in den Brüsten.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdcxcix{ER ist Derjenige, Der euch als Nachfolger auf Erden machte. Wer also Kufr betreibt, auf ihn fällt sein Kufr zurück. Und die Kafir mehrt ihr Kufr bei ihrem HERRN an Nichts außer an Zorn. Und dieKafir mehrt ihr Kufr an Nichts außer an Schaden.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdcc{Sag: ``Wie seht ihr eure beigesellten Partner, an die ihr anstelle von ALLAH Bittgebete richtet? Zeigt mir, was sie von der Erde erschufen! Oder haben sie etwa eine Beteiligung an den Himmeln? Oder ließen WIR ihnen eine Schrift zuteil werden, so daß sie Klarheit darüber haben?" Nein, sondern die Unrecht-Begehenden versprechen sich gegenseitig nichts anderes außer Täuschendem.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdcci{Gewiß, ALLAH läßt die Himmel und die Erde festhalten, daß beide nicht vergehen. Und wenn beide vergangen sind, wird keiner nach Ihm sie festhalten können. Gewiß, ER ist immer allnachsichtig, allvergebend.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdccii{Und sie schworen bei ALLAH ihre Eide mit Nachdruck, daß, wenn zu ihnen ein Warner käme, sie doch rechtgeleiteter als eine der Umam sein würden." Und als zu ihnen der Warner kam, dies mehrte sie an Nichts außer an Abneigung.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdcciii{Dies war aus Arroganz auf Erden und der Listplanung des Bösen. Und die böse Listplanung fängt niemanden ein außer ihre Leute. Warten sie etwa auf etwas anderes außer auf (ALLAHs) Handlungsweise gegen die Früheren?! Also du findest nie für ALLAHs Handlungsweise Änderung. Und du findest nie für ALLAHs Handlungsweise Abweichung.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdcciv{Sind sie etwa nicht auf der Erde umhergezogen, damit sie sehen, wie das Anschließende derjenigen vor ihnen war?! Auch sie waren stärker als sie. Und es gebührt ALLAH nicht, daß Ihm irgend etwas weder in den Himmeln, noch auf Erden zu schaffen macht. Gewiß, ER ist immer allwissend, allmächtig.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdccv{Und würde ALLAH die Menschen belangen für das, was sie sich erwarben, würde ER darauf (auf der Erde) keinerlei sich bewegende Lebewesen lassen. Doch ER gewährt ihnen Aufschub bis zu einer festgesetzten Frist. Und wenn ihre Frist eingetroffen ist, so ist ALLAH doch immer Seinen Dienern gegenüber allsehend.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdccvi{\basmalahde Ya-sin.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdccvii{Bei dem Weisheit enthaltenden Quran!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdccviii{Gewiß, du bist doch einer der Gesandten\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdccix{auf einem geradlinigen Weg.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdccx{Er (der Quran) ist eine sukzessive Hinabsendung Des Allwürdigen, Des Allgnädigen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdccxi{damit du Leute warnst, deren Vorfahren nicht gewarnt wurden, so sind sie achtlos.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdccxii{Gewiß, bereits ist das Wort gegen die Meisten von ihnen Rechtens, so verinnerlichen sie den Iman nicht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdccxiii{Gewiß, WIR legen um ihre Hälse Fesseln, so reichen sie bis an die Kinne, so richten sie ihre Köpfe mit niedergeschlagenen Blicken auf.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdccxiv{Und WIR errichteten vor ihnen Trennendes und hinter ihnen Trennendes, dann umhüllten wir sie, so sehen sie nicht.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdccxv{Und ihnen ist es gleich, ob du sie warnst oder ob du sie nicht warnst, sie verinnerlichen den Iman nicht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdccxvi{Eigentlich warnst du nur denjenigen, welcher der Ermahnung folgt und Ehrfurcht vor Dem Allgnade Erweisenden im Verborgenen hat. So überbringe ihm die frohe Botschaft über Vergebung und edle Belohnung.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdccxvii{Gewiß, WIR sind Derjenige, Der die Toten belebt und Der registrieren läßt das, was sie vollbrachten, und ihre Hinterlassenschaften. Und alles erfaßten WIR umfassend im einem eindeutigen Register.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdccxviii{Und präge ihnen ein Gleichnis über die Bewohner einer Ortschaft, als zu ihr die Gesandten kamen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdccxix{Als WIR zu ihnen zwei entsandten, dann bezichtigten sie sie der Lüge, dann verstärkten WIR sie mit einem Dritten, dann sagten sie: ``Gewiß, wir wurden zu euch entsandt."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdccxx{Sie sagten: ``Ihr seid doch nichts anderes als Menschen wie wir, und Der Allgnade Erweisende hat nichts hinabgesandt. Ihr lügt ja nur!"\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdccxxi{Sie sagten: ``Unser HERR weiß: Gewiß, wir wurden doch zu euch entsandt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdccxxii{Und uns obliegt nichts außer dem deutlichen Verkünden."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdccxxiii{Sie sagten: ``Gewiß, wir prophezeien etwas Unheilvolles mit euch. Wenn ihr nicht aufhören würdet, würden wir euch doch steinigen und euch wird gewiß von uns eine qualvolle Peinigung treffen."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdccxxiv{Sie sagten: ``Das von euch (prophezeite) Unheilvolle liegt bei euch. Nur weil ihr ermahnt wurdet?! Nein, sondern ihr seid Leute, die übertreibend sind."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdccxxv{Und aus dem entferntesten Teil der Stadt kam ein Mann im Laufschritt, er sagte: ``Meine Leute! Folgt den Gesandten,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdccxxvi{folgt denjenigen, die von euch keinen Lohn verlangen, und sie sind rechtgeleitet.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdccxxvii{Und weshalb soll ich nicht Demjenigen dienen, Der mich erschuf und zu Dem ihr zurückgebracht werdet?!\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdccxxviii{Soll ich etwa anstelle von Ihm Gottheiten nehmen?! Sollte Der Allgnade Erweisende mir Schaden zufügen, würden weder ihre Fürbitte mir etwas nützen, noch würden sie mich erretten.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdccxxix{Gewiß, ich bin dann doch in einem eindeutigen Irrtum.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdccxxx{Gewiß, ich verinnerlichte den Iman an euren HERRN, so hört mir doch zu!"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdccxxxi{Es wurde gesagt: ``Tritt in die Dschanna ein!" Er sagte: ``Würden meine Leute doch über das Kenntnis haben,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdccxxxii{was mir mein HERR vergab, und daß ER mich zu einem der Gewürdigten machte!"\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdccxxxiii{Doch WIR sandten zu seinen Leuten nach ihm keinerlei Heerschar vom Himmel hinab, und WIR würden sicherlich Keine Hinabsendenden sein.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdccxxxiv{Es war nichts anderes außer einem einzigen Schrei, sogleich waren sie erloschen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdccxxxv{Trauer um die Menschen! Niemals kam zu ihnen ein Gesandter, ohne daß sie ihn verspotteten.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdccxxxvi{Sahen sie etwa nicht, wie vieleWIR vor ihnen von den Generationen zugrunde richteten, daß sie zu ihnen nicht zurückkehren?!\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdccxxxvii{Und ausnahmslos gewiß werden allesamt Uns vorgeführt.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdccxxxviii{Und eine Aya für sie ist die tote Landschaft, die WIR belebten und dann aus ihr Körner heraussprießen ließen, von denen sie essen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdccxxxix{Und WIR machten auf ihr Dschannat von Dattelpalmen und Rebstöcken und ließen in ihr Quellen entspringen,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdccxl{damit sie von deren Früchten essen. Und doch nicht eigenhändig machten sie es. Erweisen sie sich nicht dankbar?!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdccxli{Gepriesen-erhaben ist Derjenige, Der die Zweiheiten allesamt erschuf von dem, was die Erde wachsen läßt, von ihrem Selbst und von dem, worüber sie keine Kenntnis haben.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdccxlii{Auch eine Aya für sie ist die Nacht, von der WIR den Tag entnehmen, sogleich haben sie Dunkelheit.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdccxliii{Und die Sonne durchläuft einen ihr zugehörigen Aufenthaltsort. Dies ist die Bestimmung Des Allwürdigen, Des Allwissenden.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdccxliv{Und den Mond bestimmten WIR in Phasen, bis er wieder wie die alte (dünne, krumme und gelbliche) Rispe (am Dattelbaum) wird.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdccxlv{Weder die Sonne darf den Mond einholen, noch die Nacht den Tag ereilen. Und alle schweben in (bestimmter) Bahn.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdccxlvi{Auch eine Aya für sie ist, daß WIR ihre Nachkommenschaft auf dem voll beladenen Schiff tragen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdccxlvii{Und WIR erschufen ihnen Gleiches wie dies, worauf sie steigen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdccxlviii{Und wenn WIR wollten, ertränken WIR sie, so gibt es für sie dann weder einen Helfer, noch werden sie errettet,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdccxlix{außer durch Gnade von Uns und als Genießen-Lassen bis zu einer Frist.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdccl{Und als ihnen gesagt wurde: ``Handelt Taqwa gemäß dem gegenüber, was vor euch und was hinter euch ist, damit euch Gnade erwiesen wird, (wandten sie sich ab).\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdccli{Und niemals kam zu ihnen irgendeine Aya von den Ayat ihres HERRN, ohne daß sie sich von ihr abwandten.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdcclii{Und als ihnen gesagt wurde: ``Gebt vom Rizq, das ALLAH euch gewährte." Sagten diejenigen, die Kufr betrieben haben, zu denjenigen, die den Iman verinnerlichten: ``Sollen wir etwa denjenigen speisen, welchen ALLAH gespeist hätte, wenn ER wollte?! Ihr befindet euch doch nur in einem eindeutigen Irrtum."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdccliii{Und sie sagen: ``Wann ist diese Androhung, solltet ihr wahrhaftig sein?"\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdccliv{Sie warten nur auf einen einzigen Schrei, der sie zugrunde richtet, während sie noch disputieren.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdcclv{Dann können sie weder ein Vermächtnis machen, noch zu ihren Nahverwandten zurückkehren.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdcclvi{Und es wurde in As-sur geblasen, sogleich eilen sie von den Gräbern zu ihrem HERRN.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdcclvii{Sie sagten: ``Unser Untergang! Wer erweckte uns von unserer Liegestätte auf?! Dies ist, was Der Allgnade Erweisende androhte. Und die Gesandten waren wahrhaftig."\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdcclviii{Es war nichts anderes als ein einziger Schrei, sogleich wurden allesamt Uns vorgeführt.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdcclix{An dem Tag wird keiner Seele etwas unterschlagen, und euch wird nur das vergolten, was ihr zu tun pflegtet.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdcclx{Gewiß, die Weggenossen der Dschanna sind heute beschäftigt, freudenerregt.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmdcclxi{Sie und ihre Partnerwesen sind im Schatten auf Liegen angelehnt.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmdcclxii{Für sie gibt es darin Obst. Und für sie gibt es, was sie sich wünschen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmdcclxiii{Salam (sei mit euch)! Ein Wort (wird ihnen gesagt) von einem allgnädigen HERRN.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmdcclxiv{Und: ``Sondert euch an diesem Tag ab, ihr schwer Verfehlende!"\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmdcclxv{Machte ICH euch etwa nicht zur Verpflichtung - Kinder Adams! -, daß ihr dem Satan nicht dient - gewiß er ist für euch ein entschiedener Feind -\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmdcclxvi{und daß ihr Mir dient, dies ist ein geradliniger Weg?!\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmdcclxvii{Und gewiß, bereits verleitete er von euch eine Riesenmenge. Habt ihr euch damals nicht besinnen können?!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmdcclxviii{Dies ist Dschahannam, die euch stets angedroht wurde.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmdcclxix{Tretet in es heute hinein für das, was ihr an Kufr zu betreiben pflegtet!"\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmdcclxx{Heute versiegeln WIR ihnen ihre Münder, und zu Uns sprechen dann ihre Hände, auch ihre Beine legen Zeugnis ab für das, was sie zu erwerben pflegten.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmdcclxxi{Und wollten WIR es, würden WIR ihr Augenlicht auslöschen, dann eilen sie zum Weg, doch wie sähen sie denn?!\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmdcclxxii{Und wollten WIR es, würden WIR sie auf ihrer Stelle (in eine andere Schöpfung) doch verwandeln, so würden sie weder weitergehen, noch zurückkehren können.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmdcclxxiii{Und wem WIR langes Lebensalter gewähren, den kehren WIR in der (Entwicklung der) Schöpfung um. Besinnen sie sich nicht?!\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmdcclxxiv{Und WIR lehrten ihn nicht das Dichten und dies gebührt ihm nicht! Er ist doch nichts anderes außer Ermahnung und ein klarer Quran,\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmdcclxxv{damit er diejenigen warnt, die lebendig sind, und das Gesagte über die Kafir sich verwirklicht.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmdcclxxvi{Sahen sie etwa nicht, daß WIR für sie von dem, was WIR eigenständig machten, An'am erschufen, so verfügen sie über sie?!\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmdcclxxvii{Und WIR machten sie für sie fügsam. so sind manche von ihnen ihre Reittiere, und von ihnen essen sie.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmdcclxxviii{Und in ihnen gibt es für sie Nützliches und zum Trinken. Wollen sie sich nicht dankbar erweisen?!\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmdcclxxix{Und sie nahmen sich anstelle von ALLAH Gottheiten, damit ihnen möglicherweise beigestanden wird.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmdcclxxx{Sie können ihnen nicht beistehen. Und sie (die Muschrik) sind für sie (ihre Gottheiten) vorgeführte Gehilfen.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxi{So laß ihre Worte dich nicht traurig machen! Gewiß, WIR wissen, was sie verheimlichen und was sie offenlegen.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxii{Hat der Mensch etwa nicht wahrgenommen, daß WIR ihn doch aus Nutfa erschufen, sogleich ist er ein entschiedener Disputierender?!\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxiii{Und er prägte Uns ein Gleichnis und vergaß seine Erschaffung, er sagte: ``Wer belebt denn die Knochen, während sie etwas Verwestes sind?"\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxiv{Sag: ``Es belebt sie Derjenige, Der sie erstmals erschuf, und ER ist über alle Schöpfung allwissend.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxv{ER ist Derjenige, Der für euch von den grünen Bäumen Feuer machte, da zündet ihr es davon an.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxvi{Ist etwa Derjenige, Der die Himmel und die Erde erschuf, nicht mächtig, ihresgleichen zu erschaffen? Doch, mit Sicherheit! Und ER ist Der Erschaffende, Der Allwissende.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxvii{Seine Anweisung ist nur, wenn ER irgend etwas will, daß ER diesem sagt: ``Sei!" Und es ist.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxviii{Also gepriesen-erhaben ist Derjenige, Dem die Herrschaft über alles unterliegt! Und zu Ihm werdet ihr zurückgebracht."\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmdcclxxxix{\basmalahde Bei den sich in einer Reihe Reihenden,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmdccxc{dann den mit Antreiben Antreibenden,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmdccxci{dann den eine Ermahnung Vortragenden!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmdccxcii{Gewiß, euer HERR ist doch einer.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmdccxciii{ER ist der HERR der Himmel, der Erde und das, was zwischen ihnen ist, sowie der HERR der Orte der Sonnenaufgänge.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmdccxciv{WIR schmückten den nächsten Himmel mit dem Schmuck der Gestirne,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmdccxcv{und mit Bewahrendem vor jedem äußerst übertretenden Satan.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmdccxcvi{Sie belauschen nicht die hohen Angesehenen. Und sie werden von jeder Seite beworfen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmdccxcvii{ausgetrieben. Und für sie ist eine andauernde Peinigung bestimmt,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmdccxcviii{außer demjenigen, der etwas Aufzuschnappendes aufschnappt, dann ihm eine durchdringende Flamme folgte.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmdccxcix{So frage sie nach einer Fatwa, ob ihre Schöpfung schwieriger ist, oder das, was WIR sonst erschufen? Gewiß, WIR erschufen sie aus klebrigem Lehm.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmdccc{Nein, sondern du bist verwundert, und sie verspotten,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmdccci{und wenn sie ermahnt werden, besinnen sie sich nicht,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmdcccii{und wenn sie eine Aya sehen, machen sie sich äußerst lustig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmdccciii{Und sie sagten: ``Dies ist nichts anderes als eindeutige Magie:\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmdccciv{Wenn wir starben und zu Erde und Knochen wurden, werden wir etwa doch erweckt?\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmdcccv{Auch unsere ersten Ahnen?"\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmdcccvi{Sag: ``Ja, doch! Und ihr seid dann erniedrigt."\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmdcccvii{Und es ist nichts anderes als ein einziger Schrei, sogleich sehen sie.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmdcccviii{Und sie sagten: ``Unser Untergang! Dies ist der Tag des Din."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmdcccix{``Dies ist der Tag des Richtens, den ihr zu verleugnen pflegtet.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmdcccx{Versammelt diejenigen, die Unrecht begingen, ihre Ehepartner und das, dem sie dienten\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmdcccxi{anstelle von ALLAH. Dann führt sie auf den Weg der Hölle.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmdcccxii{Und sperrt sie ein! Sie werden zur Rechenschaft gezogen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmdcccxiii{Weshalb steht ihr einander nicht bei?!\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmdcccxiv{Nein, sondern sie sind heute Kapitulanten."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmdcccxv{Und die einen von ihnen wandten sich zu den anderen hin und fragten,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmdcccxvi{sie sagten: ``Gewiß, ihr pflegtet an uns von der Rechten heranzukommen."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmdcccxvii{Sie sagten: ``Nein, sondern ihr wart keine Mumin!\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmdcccxviii{Und wir hatten über euch keine Verfügung. Nein, sondern ihr wart übertretende Leute!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmdcccxix{So war Rechtens das Gesagte gegen uns von unserem HERRN. Gewiß, wir werden es doch erfahren!\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmdcccxx{So verleiteten wir euch. Gewiß, wir waren auch Verleitete."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmdcccxxi{So sind sie an diesem Tag in der Peinigung Beteiligte.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmdcccxxii{Gewiß, WIR machen es solcherart mit den schwer Verfehlenden.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmdcccxxiii{Sie pflegten, wenn ihnen gesagt wurde: ``Es gibt keine Gottheit außer ALLAH", sich in Arroganz zu erheben\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmdcccxxiv{und zu sagen: ``Sollen wir etwa unsere Gottheiten wegen eines geistesgestörten Dichters lassen?!"\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmdcccxxv{Nein, sondern er kam mit der Wahrheit! Und er bestätigte die (anderen) Gesandten.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmdcccxxvi{Gewiß, ihr werdet doch die qualvolle Peinigung erfahren.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmdcccxxvii{Und euch wird nur vergolten, was ihr zu tun pflegtet.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmdcccxxviii{Ausgenommen sind ALLAHs auserwählte Diener,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmdcccxxix{für die ist ein bekanntes Rizq bestimmt:\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmdcccxxx{Obst! Und sie sind Gewürdigte\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxi{in den Dschannat des Wohlergehens\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxii{auf Liegen einander gegenüber.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxiii{Und ihnen wird Wein aus einer Quelle serviert,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxiv{weiß und wohlschmeckend für die Trinkenden.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxv{Weder enthält er (der Wein) Betäubendes, noch werden sie trunken sein.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxvi{Und bei ihnen sind des Blickes Zurückhaltende mit großen Augen,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxvii{als wären sie im Nest aufbewahrte Eier.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxviii{Und die einen von ihnen wandten sich zu den anderen hin und fragten,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmdcccxxxix{einer von ihnen sagte: ``Gewiß, ich hatte einen engen Freund,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmdcccxl{der sagte: ``Bist du etwa doch von den Glauben Schenkenden:\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmdcccxli{Wenn wir starben und zu Erde und Knochen wurden, wird uns etwa doch vergolten?"\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmdcccxlii{Er sagte: ``Würdet ihr hinschauen!"\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmdcccxliii{Dann schaute er hin, dann sah er ihn inmitten der Hölle.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmdcccxliv{Er sagte: ``Bei ALLAH! Du hättest mich beinahe doch zugrunde gerichtet.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmdcccxlv{Und wäre nicht die Gnade meines HERRN, wäre ich doch von den Ausgelieferten.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmdcccxlvi{Werden wir etwa nicht sterben\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmdcccxlvii{außer unserem ersten Sterben, und werden wir nicht gepeinigt?!\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmdcccxlviii{Gewiß, dies ist doch der eigentliche gewaltige Erfolg.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmdcccxlix{Für genau dieses sollen die Tuenden tun."\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmdcccl{Ist dies eine bessere Unterkunft oder der Zaqqum-Baum?!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmdcccli{Gewiß, WIR machten ihn als Fitna für die Unrecht-Begehenden.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmdccclii{Gewiß, er ist ein Baum, der aus dem Abgrund der Hölle hervorsprießt.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmdcccliii{Seine Blütenstände sind so, als wären sie Köpfe der Satane.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmdcccliv{Dann gewiß, sie werden doch davon essen, dann werden sie davon die Bäuche voll füllen.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmdccclv{Gewiß, dann ist für sie darauf doch eine Mischung mit siedendem Wasser bestimmt.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmdccclvi{Dann ist ihre Rückkehr doch zur Gluthitze.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmdccclvii{Gewiß, sie fanden ihre Ahnen abirrend vor,\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmdccclviii{so stützen sie sich auf ihren Hinterlassenschaften.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmdccclix{Und gewiß, bereits gingen doch die Meisten der Früheren irre.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmdccclx{Und gewiß, bereits entsandten WIR unter ihnen Warner.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmdccclxi{So siehe, wie das Anschließende der Gewarnten war!\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmdccclxii{Ausgenommen waren ALLAHs auserwählte Diener.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmdccclxiii{Und gewiß, bereits rief Uns Nuh, und wie schön ist der Erhörende!\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmdccclxiv{Und WIR erretteten ihn und seine Familie aus der riesengroßen Not.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmdccclxv{Und WIR machten seine Nachkommenschaft die Übriggebliebenen.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmdccclxvi{Und WIR ließen über ihn (Lob) bei den Letzten.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmdccclxvii{Salam sei über Nuh unter den Menschen.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmdccclxviii{Gewiß, WIR vergelten solcherart den Muhsin.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmdccclxix{Gewiß, er ist von unseren imanverinnerlichenden Dienern.\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmdccclxx{Dann ertränkten WIR die anderen.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmdccclxxi{Und gewiß, von seiner Gefolgschaft ist doch Ibrahim.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmdccclxxii{Als er zu seinem HERRN mit einem aufrichtigen Herz kam.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmdccclxxiii{Als er zu seinem Vater und zu seinen Leuten sagte: ``Wem dient ihr?!\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmdccclxxiv{Wollt ihr etwa erdichtete Lüge, Gottheiten anstelle von ALLAH?!\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmdccclxxv{Also was denkt ihr über den HERRN aller Schöpfung?!\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmdccclxxvi{Dann schaute er kurz auf die Sterne,\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmdccclxxvii{dann sagte er: ``Ich bin krank."\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmdccclxxviii{Dann wandten sie sich von ihm flüchtend ab.\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmdccclxxix{Dann schlich er sich zu ihren Göttern, dann sagte er: ``Wollt ihr nicht essen?!\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmdccclxxx{Wieso gebt ihr von euch keinen Laut?!"\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxi{Dann begann er heimlich sie mit der Rechten zu schlagen.\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxii{Dann wandten sie ihm eilend hin.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxiii{Er sagte: ``Dient ihr etwa, was ihr meißelt?!\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxiv{Und ALLAH erschuf euch und das, was ihr tut."\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxv{Sie sagten: ``Errichtet für ihn einen Bau, dann werft ihn ins glühende Feuer."\qt@no{97}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxvi{Sie wollten ihm Böses tun, dann ließen WIR sie die Niedrigen sein.\qt@no{98}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxvii{Und er sagte: ``Gewiß, ich gehe zu meinem HERRN, ER wird mich rechtleiten.\qt@no{99}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxviii{Mein HERR schenke mir von den gottgefällig Guttuenden."\qt@no{100}}%
\qt@newcmd\qurantrans@de@mmmdccclxxxix{Dann ließen WIR ihm eine frohe Botschaft überbringen über einen nachsichtigen Jungen.\qt@no{101}}%
\qt@newcmd\qurantrans@de@mmmdcccxc{Und als dieser soweit wurde, mit ihm dem Erwerb nachzugehen, sagte er: ``Mein Söhnchen! Gewiß, ich sah im Schlaf, daß ich dich opfere, also schau, was du meinst." Er sagte: ``Mein Vater! Tu, was dir geboten wurde. Du wirst mich - inscha-allah - einer der sich in Geduld Übenden finden."\qt@no{102}}%
\qt@newcmd\qurantrans@de@mmmdcccxci{Und als beide sich hingaben, und er ihn auf die Schläfe lagerte,\qt@no{103}}%
\qt@newcmd\qurantrans@de@mmmdcccxcii{da riefen WIR ihn: ``Ibrahim!\qt@no{104}}%
\qt@newcmd\qurantrans@de@mmmdcccxciii{Bereits hast du das Traumgesicht verwirklicht! Gewiß, solcherart vergelten WIR es den Muhsin."\qt@no{105}}%
\qt@newcmd\qurantrans@de@mmmdcccxciv{Gewiß, dies war doch die eindeutige Prüfung.\qt@no{106}}%
\qt@newcmd\qurantrans@de@mmmdcccxcv{Und WIR lösten ihn mit einem riesengroßen Opfertier aus.\qt@no{107}}%
\qt@newcmd\qurantrans@de@mmmdcccxcvi{Und WIR ließen über ihn (Lob) bei den Letzten.\qt@no{108}}%
\qt@newcmd\qurantrans@de@mmmdcccxcvii{Salam sei über Ibrahim.\qt@no{109}}%
\qt@newcmd\qurantrans@de@mmmdcccxcviii{Solcherart vergelten WIR den Muhsin.\qt@no{110}}%
\qt@newcmd\qurantrans@de@mmmdcccxcix{Gewiß, er gehörte zu Unseren iman-verinnerlichenden Dienern.\qt@no{111}}%
\qt@newcmd\qurantrans@de@mmmcm{Und WIR ließen ihm eine frohe Botschaft über Ishaq überbringen, einen Gesandten von den gottgefällig Guttuenden.\qt@no{112}}%
\qt@newcmd\qurantrans@de@mmmcmi{Und WIR gewährten Baraka ihm sowie Ishaq. Und unter ihrer Nachkommenschaft gibt es einen Muhsin und einen eindeutigen gegen sich selbst Unrecht-Begehenden.\qt@no{113}}%
\qt@newcmd\qurantrans@de@mmmcmii{Und gewiß, bereits erwiesen WIR Musa und Harun Gnade\qt@no{114}}%
\qt@newcmd\qurantrans@de@mmmcmiii{und erretteten beide und ihre Leute von der riesengroßen Not.\qt@no{115}}%
\qt@newcmd\qurantrans@de@mmmcmiv{Und WIR verhalfen ihnen zum Sieg, so waren sie die Siegenden.\qt@no{116}}%
\qt@newcmd\qurantrans@de@mmmcmv{Und WIR ließen beiden die bedeutungsreiche Schrift zuteil werden.\qt@no{117}}%
\qt@newcmd\qurantrans@de@mmmcmvi{Und WIR leiteten sie auf den geradlinigen Weg recht.\qt@no{118}}%
\qt@newcmd\qurantrans@de@mmmcmvii{Und WIR ließen über beide (Lob) bei den Letzten.\qt@no{119}}%
\qt@newcmd\qurantrans@de@mmmcmviii{Salam sei über Musa und Harun.\qt@no{120}}%
\qt@newcmd\qurantrans@de@mmmcmix{Gewiß, solcherart vergelten WIR den Muhsin.\qt@no{121}}%
\qt@newcmd\qurantrans@de@mmmcmx{Gewiß, beide sind von Unseren iman-verinnerlichenden Dienern.\qt@no{122}}%
\qt@newcmd\qurantrans@de@mmmcmxi{Und gewiß, Ilyas ist doch von den Gesandten.\qt@no{123}}%
\qt@newcmd\qurantrans@de@mmmcmxii{Erinnere daran), als er zu seinen Leuten sagte: ``Wollt ihr nicht Taqwa gemäß handeln?!\qt@no{124}}%
\qt@newcmd\qurantrans@de@mmmcmxiii{Richtet ihr Bittgebete an Ba'l und vernachlässigt Den Besten aller Schöpfer?!\qt@no{125}}%
\qt@newcmd\qurantrans@de@mmmcmxiv{ALLAH ist euer HERR und Der HERR eurer ersten Ahnen."\qt@no{126}}%
\qt@newcmd\qurantrans@de@mmmcmxv{Dann bezichtigten sie ihn der Lüge, gewiß, so werden sie doch ausgeliefert,\qt@no{127}}%
\qt@newcmd\qurantrans@de@mmmcmxvi{ausgenommen sind ALLAHs auserwählte Diener.\qt@no{128}}%
\qt@newcmd\qurantrans@de@mmmcmxvii{Und WIR ließen über ihn (Lob) bei den Letzten.\qt@no{129}}%
\qt@newcmd\qurantrans@de@mmmcmxviii{Salam sei über Ilyas.\qt@no{130}}%
\qt@newcmd\qurantrans@de@mmmcmxix{Gewiß, solcherart vergelten WIR den Muhsin.\qt@no{131}}%
\qt@newcmd\qurantrans@de@mmmcmxx{Gewiß, er ist von Unseren iman-verinnerlichenden Dienern.\qt@no{132}}%
\qt@newcmd\qurantrans@de@mmmcmxxi{Und gewiß, Lut ist doch von den Gesandten.\qt@no{133}}%
\qt@newcmd\qurantrans@de@mmmcmxxii{Erinnere daran), als WIR ihn und seine Familie allesamt erretteten\qt@no{134}}%
\qt@newcmd\qurantrans@de@mmmcmxxiii{außer einer Alten, sie war unter den Vergangenen.\qt@no{135}}%
\qt@newcmd\qurantrans@de@mmmcmxxiv{Dann vernichteten WIR die anderen.\qt@no{136}}%
\qt@newcmd\qurantrans@de@mmmcmxxv{Und gewiß, ihr geht doch bei ihnen vorbei morgens\qt@no{137}}%
\qt@newcmd\qurantrans@de@mmmcmxxvi{und nachts. Besinnt ihr euch nicht?!\qt@no{138}}%
\qt@newcmd\qurantrans@de@mmmcmxxvii{Und gewiß, Yunus ist doch von den Gesandten.\qt@no{139}}%
\qt@newcmd\qurantrans@de@mmmcmxxviii{Erinnere daran), als er zum voll beladenen Schiff floh.\qt@no{140}}%
\qt@newcmd\qurantrans@de@mmmcmxxix{Dann loste er, dann war er von den Verlierern,\qt@no{141}}%
\qt@newcmd\qurantrans@de@mmmcmxxx{dann verschlang ihn der Fisch, während er tadelnswert war.\qt@no{142}}%
\qt@newcmd\qurantrans@de@mmmcmxxxi{Also wäre er doch nicht von den Lobpreisenden gewesen,\qt@no{143}}%
\qt@newcmd\qurantrans@de@mmmcmxxxii{dann wäre er in seinem Bauch geblieben bis zum Tag, an dem sie erweckt werden.\qt@no{144}}%
\qt@newcmd\qurantrans@de@mmmcmxxxiii{Dann ließen WIR ihn ans unbewohnte Land werfen, während er krank war.\qt@no{145}}%
\qt@newcmd\qurantrans@de@mmmcmxxxiv{Und WIR ließen über ihm einen Kürbisbaum wachsen.\qt@no{146}}%
\qt@newcmd\qurantrans@de@mmmcmxxxv{Und WIR entsandten ihn zu Hunderttausend, sondern noch mehr waren sie.\qt@no{147}}%
\qt@newcmd\qurantrans@de@mmmcmxxxvi{Dann verinnerlichten sie den Iman, dann gewährten WIR ihnen Verbrauchsgüter bis zur einer Zeit.\qt@no{148}}%
\qt@newcmd\qurantrans@de@mmmcmxxxvii{Also frage sie nach einer Fatwa: ``Gehören etwa deinem HERRN die Töchter und ihnen die Söhne?!"\qt@no{149}}%
\qt@newcmd\qurantrans@de@mmmcmxxxviii{Oder erschufen WIR die Engel als Weibliche, während sie Zeugen waren.\qt@no{150}}%
\qt@newcmd\qurantrans@de@mmmcmxxxix{Ja! Sie sagen doch aufgrund ihrer Ifk-Lüge:\qt@no{151}}%
\qt@newcmd\qurantrans@de@mmmcmxl{``ALLAH zeugte." Und gewiß, sie sind zweifelsohne Lügner.\qt@no{152}}%
\qt@newcmd\qurantrans@de@mmmcmxli{Erwählte ER etwa die Töchter vor den Söhnen aus?!\qt@no{153}}%
\qt@newcmd\qurantrans@de@mmmcmxlii{Was ist los mit euch, wie urteilt ihr denn?!\qt@no{154}}%
\qt@newcmd\qurantrans@de@mmmcmxliii{Besinnt ihr euch nicht?!\qt@no{155}}%
\qt@newcmd\qurantrans@de@mmmcmxliv{Oder habt ihr eine eindeutige Bestätigung?!\qt@no{156}}%
\qt@newcmd\qurantrans@de@mmmcmxlv{So bringt eure Schrift, solltet ihr wahrhaftig sein.\qt@no{157}}%
\qt@newcmd\qurantrans@de@mmmcmxlvi{Und sie machten zwischen Ihm und den Dschinn Abstammungsverwandtschaft. Gewiß, bereits wissen die Dschinn, daß sie zweifelsohne ausgeliefert werden -\qt@no{158}}%
\qt@newcmd\qurantrans@de@mmmcmxlvii{Subhanallah über das, was sie erdichten -\qt@no{159}}%
\qt@newcmd\qurantrans@de@mmmcmxlviii{ausgenommen sind ALLAHs auserwählte Diener.\qt@no{160}}%
\qt@newcmd\qurantrans@de@mmmcmxlix{Also ihr und das, wem ihr dient,\qt@no{161}}%
\qt@newcmd\qurantrans@de@mmmcml{niemanden werdet ihr von Ihm durch Fitna abbringen\qt@no{162}}%
\qt@newcmd\qurantrans@de@mmmcmli{außer demjenigen, der in die Hölle hineingeworfen wird.\qt@no{163}}%
\qt@newcmd\qurantrans@de@mmmcmlii{Und es ist keiner unter uns (Engeln), außer daß es für ihn eine bekannte Stellung gibt.\qt@no{164}}%
\qt@newcmd\qurantrans@de@mmmcmliii{Und gewiß, wir sind doch die Reihenden.\qt@no{165}}%
\qt@newcmd\qurantrans@de@mmmcmliv{Und gewiß, wir sind doch die Lobpreisenden.\qt@no{166}}%
\qt@newcmd\qurantrans@de@mmmcmlv{Und sie pflegten doch zu sagen:\qt@no{167}}%
\qt@newcmd\qurantrans@de@mmmcmlvi{``Verfügten wir doch über eine Ermahnung von den Früheren,\qt@no{168}}%
\qt@newcmd\qurantrans@de@mmmcmlvii{so wären wir gewiß ALLAHs auserwählte Diener gewesen.\qt@no{169}}%
\qt@newcmd\qurantrans@de@mmmcmlviii{So betrieben sie Kufr an ihn. So werden sie noch wissen.\qt@no{170}}%
\qt@newcmd\qurantrans@de@mmmcmlix{Und gewiß, bereits erging Unser Wort zu Unseren entsandten Dienern:\qt@no{171}}%
\qt@newcmd\qurantrans@de@mmmcmlx{``Gewiß, sie sind doch die Unterstützer.\qt@no{172}}%
\qt@newcmd\qurantrans@de@mmmcmlxi{Und gewiß, Unsere Schar sind doch die Sieger."\qt@no{173}}%
\qt@newcmd\qurantrans@de@mmmcmlxii{So wende dich von ihnen bis zu einer Zeit ab!\qt@no{174}}%
\qt@newcmd\qurantrans@de@mmmcmlxiii{Und gewähre ihnen Einblick, denn sie werden den Einblick haben.\qt@no{175}}%
\qt@newcmd\qurantrans@de@mmmcmlxiv{Fordern sie etwa mit Unserer Peinigung zur Eile auf?!\qt@no{176}}%
\qt@newcmd\qurantrans@de@mmmcmlxv{Und käme sie auf ihre Landschaft herunter, so wäre erbärmlich der Morgen der Gewarnten.\qt@no{177}}%
\qt@newcmd\qurantrans@de@mmmcmlxvi{So wende dich von ihnen bis zu einer Zeit ab!\qt@no{178}}%
\qt@newcmd\qurantrans@de@mmmcmlxvii{Und blicke ein, denn sie werden den Einblick haben.\qt@no{179}}%
\qt@newcmd\qurantrans@de@mmmcmlxviii{Gepriesen-erhaben ist dein HERR, Der HERR der Würde, über das, was sie erdichten,\qt@no{180}}%
\qt@newcmd\qurantrans@de@mmmcmlxix{und Salam sei über die Gesandten\qt@no{181}}%
\qt@newcmd\qurantrans@de@mmmcmlxx{und alhamdulillah: Alles Lob gebührt ALLAH, Dem HERRN aller Schöpfung.\qt@no{182}}%
\qt@newcmd\qurantrans@de@mmmcmlxxi{\basmalahde Sad. Bei dem Quran mit der Ermahnung!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmcmlxxii{Nein, sondern diejenigen, die Kufr betrieben haben, sind in Arroganz und Widerspenstigkeit.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmcmlxxiii{Wie manche Generation haben WIR vor ihnen zugrunde gerichtet, dann riefen sie (um Hilfe). Doch gäbe es zu der Zeit noch Errettung!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmcmlxxiv{Und sie wunderten sich darüber, daß zu ihnen ein Warner von ihnen kam, und die Kafir sagten: ``Dieser ist ein lügnerischer Magier.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmcmlxxv{Machte er etwa die Gottheiten zu einer einzigen Gottheit?! Gewiß, dies ist doch eine sehr erstaunliche Sache."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmcmlxxvi{Und die Entscheidungsträger von ihnen gingen los: ``Geht weiter so, und seid duldsam in eurer Gottheiten Sache! Gewiß, dies ist eine Sache, die doch erstrebt wird.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmcmlxxvii{Wir hörten davon nicht von der letzten Gemeinschaft. Dies ist doch nichts anderes als eine Erfindung!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmcmlxxviii{Wurde etwa die Ermahnung (ausgerechnet) ihm unter uns hinabgesandt?" Nein, sondern sie haben Zweifel über Meine Erinnerung! Nein, sondern sie erfuhren noch nicht Meine Peinigung!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmcmlxxix{Oder verfügen sie etwa über die Magazine der Gnade deines HERRN, Des Allwürdigen, Des Schenkenden?!\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmcmlxxx{Oder gehört ihnen etwa die Herrschaft der Himmel, der Erde und dessen, was zwischen ihnen ist?! Also, dann sollen sie mit ihren Verbindungen hochsteigen!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxi{Die Soldaten von Al-ahzab werden dort besiegt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxii{Vor ihnen leugneten ab die Leute Nuhs, 'Aads und Pharaos von den Pyramiden,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxiii{sowie Thamuds und die Leute Luts und die Bewohner von Al-aika. Diese sind Al-ahzab.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxiv{Doch alle bezichtigten die Gesandten der Lüge, also verdient ist die Bestrafung.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxv{Und diese warten auf nichts anderes außer auf einen einzigen Schrei, der keine Wiederholung hat.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxvi{Und sie sagten: ``Unser HERR! Beschleunige uns doch unseren Anteil vor dem Tag der Abrechnung!"\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxvii{Übe dich in Geduld dem gegenüber, was sie sagen, und erinnere dich an Unseren Diener Dawud, den Kräftigen. Er war stets umkehrend.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxviii{WIR ließen mit ihm die Felsenberge gratis fügbar sein, sie lobpreisen abends und nach dem Sonnenaufgang.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmcmlxxxix{Auch die Vögel waren versammelt, alle waren zu Ihm umkehrend.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmcmxc{Und WIR festigten seine Herrschaft und ließen ihm die Weisheit zuteil werden sowie die Sprachgewandtheit.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmcmxci{Und wurde dir etwa zuteil die Nachricht über die Gegenparteien, als sie die Mauer um die Gebetsstätte erkletterten?!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmcmxcii{Als sie zu Dawud eintraten, dann erschreckte er sich vor ihnen. Sie sagten: ``Fürchte dich nicht! Wir sind zwei Gegenparteien, die einen von uns begingen Übertretungen gegen die anderen, so richte unter uns des Rechts gemäß, sei nicht ( ungerecht und leite uns zum rechten Weg!\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmcmxciii{Dies ist mein Bruder. Ihm gehören neunundneunzig Schafsweibchen und mir gehört ein einziges Schafsweibchen, dann sagte er: ``Überlasse es mir!" Und er unterwarf mich im Gespräch."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmcmxciv{Er sagte: ``Gewiß, bereits tat er dir Unrecht mit seiner Forderung zur Überlassung deines Schafsweibchen zu seinen Schafsweibchen. Und gewiß, die meisten der Beteiligten begehen doch Übertretungen die einen von ihnen gegen die anderen außer denjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben. Doch diese sind wenige." Und Dawud dachte, daß WIR ihn nur einer Fitna unterzogen haben. Dann bat er seinen HERRN um Vergebung, fiel in Ruku' nieder und war umkehrend.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmcmxcv{Dann vergaben WIR ihm dieses. Und gewiß, für ihn gibt es bei Uns doch nähere Stellung und schöne Rückkehr.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmcmxcvi{Dawud! Gewiß, WIR machten dich als Khalifa im Lande, so richte unter den Menschen des Rechts gemäß und folge nicht dem Zuneigen, sonst verleitet es dich von ALLAHs Weg. Gewiß, für diejenigen, die von ALLAHs Weg abirren, ist eine qualvolle Peinigung bestimmt dafür, daß sie den Tag der Abrechnung vergessen haben.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmcmxcvii{Und WIR erschufen den Himmel, die Erde und das, was zwischen ihnen ist, nicht sinnlos. Dies ist die Vermutung derjenigen, die Kufr betrieben haben. Also Niedergang sei denjenigen, die Kufr betrieben haben, durch das Feuer.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmcmxcviii{Oder setzen WIR etwa diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, den auf Erden Verderben-Anrichtenden gleich?! Oder setzen WIR etwa die Muttaqi den die Verfehlungen öffentlich Begehenden gleich?!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmcmxcix{Dies ist eine Schrift, die WIR dir hinabsandten, voller Baraka, damit sie über ihre Ayat nachdenken und damit diejenigen mit Verstand sich besinnen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmm{Und WIR schenkten Dawud Sulaiman. Was für einen guten Diener! Gewiß, er ist umkehrend.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmi{Als ihm abends die reinrassigen Araberpferde vorgeführt wurden,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmii{so sagte er: ``Gewiß, ich liebte die Neigung zum Guten nach dem Gedenken meines HERRN, bis sie (die Pferde) sich hinter dem Sichtschutz verbargen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmiii{``Bringt sie mir zurück!" Dann begann er sie über die Beine und die Hälse zu streicheln.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmiv{Und gewiß, bereits unterzogen WIR Sulaiman einer Fitna und ließen auf seinem Thron einen Körper, dann bereute er.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmv{Er sagte: ``Mein HERR! Vergib mir und schenke mir eine Herrschaft, die niemandem nach mir möglich wird. Gewiß, DU bist Der reichlich Schenkende."\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmvi{Dann machten WIR ihm den Wind gratis fügbar - er wehte nach seiner Anweisung sanft, wohin er wollte -\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmvii{sowie die Satane - jeden Bauenden und Tauchenden,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmviii{sowie andere, die in Ketten gefesselt sind.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmix{``Dies ist Unsere Gabe - also erweise Gaben oder halte zurück - ohne Rechnen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmx{Und gewiß, für ihn gibt es bei Uns doch nähere Stellung und schöne Rückkehr.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmxi{Und erwähne Unseren Diener Ayyub, als er seinen HERRN rief: ``Gewiß, mich traf der Satan mit Mühsal und Peinigung."\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmxii{``Schlage mit dem Bein auf den Boden!" Dies ist ein kaltes Bad und Getränk.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmxiii{Und WIR schenkten ihm seine Familie und mit ihnen ihresgleichen als Gnade von Uns und Ermahnung für diejenigen mit Verstand.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmxiv{``Und nimm mit deiner Hand ein kleines Bündel, dann schlage mit ihm und sei nicht eidbrüchig!" Gewiß, WIR fanden ihn einen sich in Geduld Übenden. Was für einen guten Diener. Gewiß, er war stets umkehrend.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmxv{Und erwähne Unsere Diener Ibrahim, Ishaq und Ya'qub, diemit Kraft und Einblick waren.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmxvi{Gewiß, WIR erwählten sie mit einer Auserwählenden aus: Die Erinnerung an die eigentliche Wohnstätte.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmxvii{Und gewiß, sie sind bei Uns zweifelsohne von den Auserwählten, von den Besten.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmxviii{Und erwähne Isma'il, Alys' und Dhal-kifl! Und alle sind von den Besten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmxix{Dies ist eine Ermahnung. Und gewiß, für die Muttaqi gibt es doch eine schöne Rückkehr,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmxx{'Adn-Dschannat! Für sie sind die Türen voll geöffnet.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmxxi{Angelehnt sind sie darin. Sie verlangen nach vielerlei Obst und Trank.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmxxii{Und bei ihnen sind die des Blickes Zurückhaltenden, Gleichaltrige.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmxxiii{Dies ist, was euch für den Tag der Abrechnung versprochen wird.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmxxiv{Gewiß, dies ist zweifelsohne Unser Rizq. Es gibt dafür kein Ende.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmxxv{Dies! Und gewiß für die Übertretenden gibt es doch eine böse Rückkehr.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmxxvi{In Dschahannam werden sie hineingeworfen. So erbärmlich ist die Unterlage.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmxxvii{Dies! So sollen sie es kosten - Siedendes und Eiter,\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmxxviii{sowie anderes seinesgleichen - paarweise.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmxxix{Dies ist eine Gruppe, die mit euch hineinstürmt. ``Kein Willkommen für sie. Gewiß, sie werden in das Feuer hineingeworfen.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmxxx{Sie sagten: ``Nein, sondern kein Willkommen für euch! Ihr habt es uns unterbreitet!  So erbärmlich ist der Aufenthalt.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmxxxi{Sie sagten: ``Unser HERR! Für denjenigen, der dieses uns unterbreitete, mehre Vielfaches an Peinigung vom Feuer!"\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmxxxii{Und sie sagten: ``Weshalb sehen wir keine Männer, die wir zu den Bösen zu zählen pflegten?!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmxxxiii{Wir machten sie zum Spotten. Oder verfehlten sie unsere Blicke?"\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmxxxiv{Gewiß, dies ist doch wahr - das Disputieren der Bewohner des Feuers.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmxxxv{Sag: ``Ich bin doch nur ein Warner. Und es gibt keine Gottheit außer ALLAH, Dem Einzigen, Dem Allbezwingenden.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmxxxvi{ER ist Der HERR der Himmel, der Erde und dessen, was zwischen ihnen ist, Der Allwürdige, Der Allvergebende."\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmxxxvii{Sag: ``Es ist eine gewaltige Mitteilung,\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmxxxviii{von der ihr euch abwendet.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmxxxix{Ich verfüge über kein Wissen von den höheren Gewichtigen, wenn sie disputieren.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmxl{Es wird mir nichts an Wahy zuteil, außer daß ich ein eindeutiger Warner bin!"\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmxli{Erinnere daran), als dein HERR zu den Engeln sagte: ``Gewiß, ICH werde einen Menschen aus Lehm erschaffen.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmxlii{Und nachdem ICH ihn vollendete und in ihn einen Ruhh von Mir einhauchte, dann fallt nieder in Sudschud für ihn!"\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmxliii{Dann vollzogen die Engel Sudschud, sie alle, allesamt.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmxliv{Außer Iblis, er hat sich in Arroganz erhoben und war von den Kafir.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmxlv{ER sagte: ``Iblis! Was hinderte dich daran, Sudschud vor dem zu vollziehen, den ICH eigenständig erschuf?! Hast du dich in Arroganz erhoben, oder warst du von den Hochmütigen?!"\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmxlvi{Er sagte: ``Ich bin besser als er! DU hast mich aus Feuer erschaffen und ihn aus Lehm erschaffen."\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmxlvii{ER sagte: ``Also verlasse sie (die Dschanna)! Denn gewiß, du bist verdammt.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmxlviii{Und unweigerlich, auf dir lastet Mein Fluch bis zum Tag des Din."\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmxlix{Er sagte: ``Mein HERR! Dann gewähre mir Aufschub bis zu dem Tag, an dem sie erweckt werden."\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmml{ER sagte: ``Du bist von denjenigen, denen Aufschub gewährt wird\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmli{bis zum Tag der bekannten Zeit."\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmlii{Er sagte: ``Bei Deiner Würde! Ich werde sie doch beirren, allesamt,\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmliii{außer Deinen auserwählten Dienern von ihnen."\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmliv{ER sagte: ``Der Wahrheit gemäß - und die Wahrheit ist, was ICH sage -\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmlv{werde ICH doch Dschahannam voll füllen mit dir und allen, die von ihnen dir folgen, allesamt."\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmlvi{Sag: ``Ich verlange von euch dafür keinen Lohn, und ich bin nicht von den Angebenden."\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmmlvii{Es ist nur eine Ermahnung für die Schöpfung!\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmmlviii{Und ihr werdet doch noch seine Mitteilung nach einer Zeit erkennen.\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmmlix{\basmalahde Die sukzessive Hinabsendung der Schrift ist von ALLAH, Dem Allwürdigen, Dem Allweisen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmlx{Gewiß, WIR sandten dir die Schrift mit der Wahrheit hinab, so diene ALLAH als Aufrichtiger im Din Ihm gegenüber!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmlxi{Ja! Nur ALLAH gehört der reine Din. Und diejenigen, die anstelle von Ihm sich Wali nehmen: ``Wir dienen ihnen nicht, außer damit sie uns zu ALLAH eine nähere Stellung verschaffen." Gewiß, ALLAH wird zwischen ihnen richten in dem, worüber sie uneins waren. Gewiß, ALLAH leitet nicht denjenigen recht, der ein äußerster Lügner und ein äußerster Kafir ist.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmlxii{Hätte ALLAH Sich ein Kind nehmen wollen, hätte ER gewiß von dem, was ER erschuf, das auserwählt, was ER will. Gepriesen-erhaben ist ER! ER ist ALLAH, Der Einzige, Der Allbezwingende.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmlxiii{ER erschuf die Himmel und die Erde mit der Gesetzmäßigkeit. ER windet die Nacht über den Tag und windet den Tag über die Nacht. Und ER machte die Sonne und den Mond gratis fügbar. Alle durchlaufen bis zu einer festgelegten Zeit. Ja! ER ist Der Allwürdige, Der Allvergebende.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmlxiv{ER erschuf euch aus einem einzigen Wesen, dann machte ER daraus sein Partnerwesen und ER sandte euch von den An'am acht Paare. ER erschafft euch in den Bäuchen eurer Mütter als eine Schöpfung nach einer Schöpfung in drei Finsternissen. Dieser ist ALLAH, euer HERR, Ihm gehört die Herrschaft. Es gibt keine Gottheit außer Ihm. Wie werdet ihr denn davon abgebracht?!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmlxv{Wenn ihr Kufr betreibt, so ist gewiß ALLAH autark euch gegenüber! Und ER stimmt Seinen Dienern das Kufr-Betreiben nicht zu. Doch wenn ihr euch dankbar erweist, dies stimmt ER euch zu. Und keine verfehlende Seele trägt die Verfehlung einer anderen. Dann ist zu eurem HERRN eure Rückkehr, dann wird ER euch Mitteilung machen über das, was ihr zu tun pflegtet. Gewiß, ER ist allwissend über das in den Brüsten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmlxvi{Und wenn den Menschen Schädigendes trifft, richtet er an seinen HERRN Bittgebete reuig zu Ihm, dann wenn ER ihm eine Wohltat von Ihm erweist, vergißt er das, was er an Ihn vorher an Bittgebeten richtete, und schreibt ALLAH etwas als Ebenbürtiges zu, damit er von Seinem Weg (andere) abbringt. Sag: ``Vergnüge dich nur mit deinem Kufr ein wenig! Gewiß, du bist von den Weggenossen des Feuers."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmlxvii{Ist derjenige, der qanit während der Nacht sudschud- und qiyam-vollziehend ist, der sich vor dem Jenseits in Acht nimmt und der sich die Gnade seines HERRN erhofft, (etwa mit dem Kafir gleich)?! Sag: ``Sind etwa diejenigen, die wissen, gleich denjenigen, die nicht wissen?" Doch nur diejenigen mit Verstand besinnen sich.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmlxviii{Sag (in Meinem Namen): ``Meine Diener, die den Iman verinnerlicht haben, handelt Taqwa gemäß ALLAH gegenüber!" Für diejenigen, die Ihsan tun, ist in diesem Diesseits Schönes bestimmt. Und ALLAHs Erde ist weit. Doch nur denjenigen sich in Geduld Übenden wird ihre Belohnung ohne Berechnung vergütet.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmlxix{Sag: ``Mir wurde geboten, ALLAH als Aufrichtiger im Din Ihm gegenüber zu dienen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmlxx{Und mir wurde geboten, der erste Muslim zu sein."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmlxxi{Sag: ``Ich fürchte - sollte ich meinem HERRN widersprechen - die Peinigung eines gewaltigen Tages."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmlxxii{Sag: ``(Nur) ALLAH diene ich als Aufrichtiger in meinem Din Ihm gegenüber.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmlxxiii{So dient, wem ihr wollt, anstelle von Ihm!" Sag: ``Gewiß, die wirklichen Verlierer sind diejenigen, die sich selbst und ihre Familie am Tag der Auferstehung verlieren. Ja! Dies ist das wahre Verlieren."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmlxxiv{Für sie gibt es von über ihnen Schatten aus dem Feuer und von unter ihnen ebenso Schatten. Dies ist es, ALLAH läßt damit Seine Diener sich fürchten. ``AlsoMeine Diener, handelt Taqwa gemäß Mir gegenüber!"\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmlxxv{Und für diejenigen, die sich vom Taghut fernhielten, daß sie ihnen dienen, und ALLAH gegenüber bereuten, ist die frohe Botschaft bestimmt. So überbringe Meinen Dienern frohe Botschaft,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmlxxvi{die das Gesagte vernehmen und dann dem Besten davon folgen. Diese sind diejenigen, die ALLAH rechtleitete, und diese sind diejenigen mit Verstand.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmlxxvii{Ist etwa derjenige, für den das Wort der Peinigung vollstreckt wurde, (noch zu retten)?! Bist du etwa derjenige, der denjenigen errettet, der im Feuer ist?!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmlxxviii{Doch für diejenigen, die Taqwa gemäß ihrem HERRN gegenüber handelten, sind Wohnstätten bestimmt, über die noch andere Wohnstätten errichtet sind, die von Flüssen durchflossen werden. Dies ist ALLAHs Versprechen. ALLAH bricht nie das Versprochene.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmlxxix{Hast du etwa nicht gesehen, daß ALLAH vom Himmel Wasser fallen ließ, dann es in die Erde als Quellen eindringen ließ, dann bringt ER damit Gewächs in vielfältigen Farben hervor, dann wird es trocken, dann siehst du es gelbwerdend, dann macht ER es Zermalmtes. Gewiß, darin ist doch eine Ermahnung für diejenigen mit Verstand.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmlxxx{Ist etwa derjenige, dessen Brust ALLAH für den Islam öffnete, so daß er ein Licht von seinem HERRN hat, (wie der Irrende)?! Also Niedergang sei denjenigen, deren Herzen dem Gedenken ALLAHs gegenüber hart sind. Diese sind im eindeutigen Irregehen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmlxxxi{ALLAH sandte das Schönste des Gesprochenen nach und nach hinab, eine sich ähnlich aussehende Schrift, die mathani ist, von der diejenigen Gänsehaut bekommen, die Ehrfurcht vor ihrem HERRN haben, dann erweichen sich ihre Häute und ihre Herzen nach dem Gedenken ALLAHs. Dies ist ALLAHs Rechtleitung. ER leitet damit recht, wen ER will. Und wen ALLAH abirren läßt, für den gibt es keinen Rechtleitenden!\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmlxxxii{Ist etwa derjenige, der sich (sogar) mit seinem Gesicht Schutz vor dem Schlechten der Peinigung am Tag der Auferstehung sucht, (gleich demjenigen, der nicht gepeinigt wird)?! Und den Unrecht-Begehenden wurde gesagt: ``Kostet, was ihr euch zu erwerben pflegtet."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmlxxxiii{Diejenigen vor ihnen leugneten, dann kam zu ihnen die Peinigung von da, wo sie nicht merkten.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmlxxxiv{Dann ließ ALLAH sie die Erniedrigung im diesseitigen Leben erfahren. Doch die Peinigung des Jenseits ist noch größer, würden sie doch nur wissen!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmlxxxv{Und gewiß, bereits prägten WIR den Menschen in diesem Quran von jedem Gleichnis, damit sie sich besinnen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmlxxxvi{Es ist ein arabischer Quran ohne jegliches Gewundenes, damit sie Taqwa gemäß handeln.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmlxxxvii{ALLAH prägte ein Gleichnis von einem Diener, über den sich Partner streiten, und von einem (anderen) Diener, der ausschließlich einem einzigen Mann gehört, gleichen beide sich etwa?! Alhamdulillah: Alles Lob gebührt ALLAH! Nein, sondern die meisten wissen es nicht.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmlxxxviii{Gewiß, du wirst sterben und sie werden zweifelsohne sterben.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmlxxxix{Dann werdet ihr gewiß am Tag der Auferstehung bei eurem HERRN disputieren!\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmxc{Und keiner begeht mehr Unrecht als derjenige, der im Namen ALLAHs gelogen hat und die Wahrhaftigkeit leugnete, als sie zu ihm kam. Gibt es etwa nicht in Dschahannam eine Bleibe für die Kafir?!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmxci{Und derjenige, der mit der Wahrhaftigkeit kam, auch der diese als wahr verinnerlichte, diese sind die Muttaqi.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmxcii{Für sie gibt es, was sie wollen, bei ihrem HERRN. Dies ist die Belohnung der Muhsin.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmxciii{ALLAH wird ihnen zweifelsohne vergeben das Schlechteste von dem, was sie taten, und ihnen ihre Belohnung vergelten mit Besserem als das, welches sie zu tun pflegten.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmxciv{Genügt ALLAH etwa nicht Seinem Diener?! Und sie machen dir Angst mit denjenigen anstelle von Ihm. Und wen ALLAH abirren läßt, für den gibt es keinen Rechtleitenden.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmxcv{Und wen ALLAH rechtleitet, für den gibt es keinen Irreführenden. Ist ALLAH etwa nicht allwürdig, vergeltend?!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmxcvi{Und würdest du sie fragen: ``Wer erschuf die Himmel und die Erde?", gewiß würden sie sagen: ``ALLAH!" Sag: ``Seht ihr, woran ihr anstelle von ALLAH Bitt-gebete richtet, sollte ALLAH mir Schädigendes wollen, könnten sie das Schädigende von Ihm wegnehmen? Oder sollte ER mir eine Gnade wollen, könnten diese Seine Gnade zurückhalten?" Sag: ``Mir genügt ALLAH." Ihm gegenüber üben die Tawakkul-Übenden Tawakkul.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmxcvii{Sag: ``Meine Leute! Handelt euren Standpunkten entsprechend! Auch ich handle. Ihr werdet doch noch wissen,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmxcviii{wen Peinigung überkommt, welche ihn erniedrigt, und bei wem sich eine ständige Peinigung aufhält."\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmxcix{Gewiß, WIR sandten dir die Schrift für die Menschen der Wahrheit gemäß hinab. Wer also Rechtleitung findet, (tut es allein) für sich selbst, und wer irregeht, (tut es ausschließlich) gegen sich selbst. Und du bist über sie kein Wakil.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmc{ALLAH zieht die Seelen (der Menschen) bei ihrem Tod ein, sowie von denen, die noch nicht den Tod erfuhren, während ihres Schlafs. Dann hält ER diejenigen zurück, für die ER den Tod bestimmte, und läßt die anderen bis zu einer festgelegten Frist frei. Gewiß, darin sind doch Ayat für Leute, die nachdenken.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmci{Oder nahmen sie sich etwa anstelle von ALLAH Fürbittende?! Sag: ``Selbst dann, wenn sie über nichts verfügen und nicht begreifen können?!"\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcii{Sag: ``ALLAH gehört die ganze Fürbitte. Ihm gehört die Herrschaft der Himmel und der Erde. Dann werdet ihr zu Ihm zurückgebracht."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmciii{Und wenn ALLAH alleine erwähnt wird, beklommen werden die Herzen derjenigen, die den Iman an das Jenseits nicht verinnerlichen. Und wenn diejenigen anstelle von Ihm erwähnt werden, da freuen sie sich richtig.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmciv{Sag: ``ALLAH, DU Schöpfer der Himmel und der Erde, Der Allwissende über das Verborgene und das Sichtbare! DU richtest unter Deinen Dienern in dem, worüber sie uneins zu sein pflegten."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcv{Und würden diejenigen, die Unrecht begingen, über alles verfügen, was auf Erden ist - allesamt, und mit ihm seinesgleichen, würden sie sich damit vom Schlechten der Peinigung am Tag der Auferstehung zweifelsohne freikaufen. Und ihnen erschien von ALLAH, womit sie nie zu rechnen pflegten.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcvi{Und ihnen wurden die Bosheiten dessen, was sie erwarben, sichtbar. Und sie umgab das, was sie zu verspotten pflegten.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcvii{Also wenn den Menschen Schädigendes trifft, richtet er an Uns Bittgebete, dann, wenn WIR ihm eine Wohltat von Uns erwiesen haben, sagt er: ``Zuteil wurde mir dies nur wegen dem Wissen, über das ich verfüge." Nein, sondern es ist eine Fitna. Doch die meisten wissen es nicht.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcviii{Bereits sagten dies doch diejenigen vor ihnen, dann nützte ihnen nicht, was sie zu erwerben pflegten.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcix{Dann trafen sie die Bosheiten dessen, was sie erwarben. Und diejenigen von diesen, die Unrecht begingen, werden die Bosheiten dessen, was sie erwarben, doch noch treffen, und sie machen (Uns) sicherlich nicht zu schaffen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcx{Wissen sie etwa noch nicht, daß ALLAH gewiß viel Rizq gewährt, wem ER will, und daß ER wenig gewährt?! Gewiß, darin sind doch Ayat für Leute, die den Iman verinnerlichen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcxi{Sag (in Meinem Namen): ``Meine Diener, die sich gegen sich selbst vergingen! Verzweifelt nicht an ALLAHs Gnade! Gewiß, ALLAH vergibt die Verfehlungen - allesamt. Gewiß, ER ist Der Allvergebende, Der Allgnädige."\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcxii{Und werdet reuig eurem HERRN gegenüber! Und praktiziert für Ihn den Islam, bevor zu euch die Peinigung kommt, dann wird euch nicht beigestanden.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcxiii{Und folgt dem Besten dessen, was euch von eurem HERRN hinabgesandt wurde, bevor die Peinigung zu euch plötzlich hineinbricht, und während ihr es nicht merkt.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcxiv{Damit keine Seele sagt: ``Weh mir für das, was ich ALLAH gegenüber vernachlässigte. Und ich war doch von den Spöttern."\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcxv{Oder daß sie sagt: ``Hätte ALLAH mich doch rechtgeleitet, wäre ich doch von den Muttaqi!"\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcxvi{Oder daß sie sagt, wenn sie die Peinigung sieht: ``Hätte ich doch nur noch eine Rückkehr, damit ich von den Muhsin würde."\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcxvii{``Doch, mit Sicherheit! Bereits kamen zu dir Meine Ayat, dann hast du sie geleugnet, dich in Arroganz erhoben und warst von den Kafir."\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcxviii{Und am Tag der Auferstehung siehst du diejenigen, die im Namen ALLAHs gelogen haben, ihre Gesichter sind schwarz geworden. Gibt es etwa in Dschahannam nicht eine Bleibe für die Arroganten?!\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmcxix{Und ALLAH errettet diejenigen, die Taqwa gemäß handelten, mit ihrem Erfolg. Weder trifft sie das Schlechte, noch werden sie traurig sein.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmcxx{ALLAH ist Der Schöpfer aller Dinge. Und ER ist über alles Wakil.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmcxxi{Ihm gehören die Schlüssel der Himmel und der Erde. Und diejenigen, die Kufr betrieben haben, diese sind die wirklichen Verlierer.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmcxxii{Sag: ``Gebietet ihr mir anderem zu dienen, ihr Unwissende?!"\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcxxiii{Und gewiß, bereits wurde dir und denjenigen vor dir Wahy zuteil: ``Würdest du Schirk ALLAH gegenüber betreiben, wird dein Tun unweigerlich zunichte werden, und du wirst doch von den Verlierern sein."\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcxxiv{Nein, sondern diene ALLAH, und sei von den Dankenden!\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcxxv{Und sie erwiesen ALLAH nicht die Ehrung, die Ihm gebührt. Und die Erde insgesamt ist wie eine Handvoll Ihm gegenüber am Tag der Auferstehung. Und die Himmel sind zusammengerollt mit Seiner Allmacht. Gepriesen-erhaben ist ER und immer höher über das, was sie an Schirk betreiben.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcxxvi{Und es wurde in As-sur geblasen, dann erschlagen wurden alle in den Himmeln und auf Erden außer denjenigen, die ALLAH will. Dann wurde darin ein zweites (Mal) hineingeblasen, sogleich waren sie stehend und schauten zu.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmcxxvii{Und die Erde erstrahlte mit dem Licht ihres HERRN, die Schrift wurde aufgelegt, die Propheten und die Schahids wurden gebracht und es wurde zwischen ihnen gerichtet. Und ihnen wird kein Unrecht angetan.\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcxxviii{Und jeder Seele wurde das vergütet, was sie tat. Und ER weiß besser Bescheid über das, was sie machen.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcxxix{Und diejenigen, die Kufr betrieben haben, wurden nach Dschahannam in Gruppen getrieben. Dann als sie diese erreichten, und deren Tore geöffnet wurden und ihre Hüter ihnen sagten: ``Kamen zu euch etwa keine Gesandten von euch, die euch die Ayat eures HERRN vortrugen und euch vor dem Treffen mit diesem eurigen Tage warnten?!" Sie sagten: ``Doch, mit Sicherheit!" Doch das Wort von der Peinigung wurde verwirklicht gegen die Kafir.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmcxxx{Es wurde gesagt: ``Betretet die Tore von Dschahannam als Ewige darin! Also erbärmlich ist die Bleibe der Arroganten."\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmcxxxi{Und diejenigen, die ihrem HERRN gegenüber Taqwa gemäß handelten, wurden in die Dschanna in Gruppen geführt. Dann als sie diese erreichten, und deren Türen geöffnet wurden und ihre Hüter ihnen sagten: ``Salam sei mit euch! Gut wart ihr, so tretet in sie als Ewige hinein!\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmcxxxii{Und sie sagten: ``Alhamdulillah: Alles Lob gebührt ALLAH, Der uns Sein Versprechen erfüllte und uns die Erde erben ließ. Wir lassen uns in der Dschanna nieder, wo wir wollen." Und was für eine erfreuliche Belohnung für die Tuenden.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmcxxxiii{Und du siehst die Engel umgebend um Al'ahrsch herum. Sie lobpreisen mit dem Lob ihres HERRN. Und unter ihnen wurde nach Gerechtigkeit gerichtet, und es wurde gesagt: ``Alhamdulillah: Alles Lob gebührt ALLAH, Dem HERRN aller Schöpfung!"\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmcxxxiv{\basmalahde Ha-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcxxxv{Die sukzessive Hinabsendung der Schrift ist von ALLAH, Dem Allwürdigen, Dem Allwissenden,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcxxxvi{Dem Allvergebenden der Verfehlung und Dem Annehmenden der Reue, Dem Harten im Strafen, Dem mit den Wohltaten. Es gibt keine Gottheit außer Ihm. Zu Ihm ist das Werden.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcxxxvii{Über ALLAHs Ayat disputieren nur diejenigen, die Kufr betrieben haben. So soll dich ihr Herumreisen (für Handelszwecke) in den Ländern nicht täuschen!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcxxxviii{Vor ihnen leugneten die Leute von Nuh ab, sowie die Al-ahzab nach ihnen. Und jede Umma war im Begriff ihren Gesandten zu ergreifen. Und sie disputierten mit dem für nichtig Erklärte, um damit das Wahre zu annullieren, dann vernichtete ICH sie. Und wie war Meine Bestrafung!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcxxxix{Und solcherart war Rechtens das Wort deines HERRN gegen diejenigen, die Kufr betrieben haben, daß sie gewiß die Weggenossen des Feuers sind.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcxl{Diejenigen, die Al'ahrsch tragen und die um ihn herum sind, lobpreisen mit dem Lob ihres HERRN, verinnerlichen den Iman an Ihn und erbitten Vergebung für diejenigen, die den Iman verinnerlichten: ``Unser HERR! DU umfaßt alles mit Gnade und Wissen, also vergib denjenigen, die bereuten und Deinem Weg folgten, und bewahre sie vor der Peinigung der Hölle.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcxli{Unser HERR! und laß sie eintreten in die 'Adn-Dschannat, die DU ihnen versprochen hast, sowie diejenigen, die gut wurden von ihren Eltern, ihren Ehepartnern und ihren Kindern. Gewiß, DU bist Der Allwürdige, Der Allweise.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcxlii{Und bewahre sie vor den gottmißfälligen Taten, und wen DU an jenem Tag vor den gottmißfälligen Taten bewahrst, dem hast DU bereits Gnade erwiesen. Und dieses ist der gewaltige Erfolg."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcxliii{Gewiß, denjenigen, die Kufr betrieben haben, wird zugerufen: ``ALLAHs Widerwille (gegen euch) ist doch größer als euer eigener Widerwille gegen euch selbst, als ihr zum Iman gerufen wurdet, dann aber ihr Kufr betreibt."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcxliv{Sie sagten: ``Unser HERR! DU ließest uns zwei mal sterben und zwei mal beleben, dann gaben wir unsere Verfehlungen zu. Gibt es denn zum Herausgehen einen Weg?"\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcxlv{Dies weil, als einzig zu ALLAH gerufen wurde, ihr Kufr betrieben habt, doch wenn Ihm gegenüber Schirk betrieben wird, schenkt ihr (ihnen) Iman. Also das Urteilen unterliegt ALLAH, Dem Allhöchsten, Dem Allgroßen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcxlvi{ER ist Derjenige, Der euch Seine Ayat zeigt und euch Rizq vom Himmel nach und nach hinabsendet. Und es besinnt sich niemand außer demjenigen, der reuig wird.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcxlvii{So richtet an ALLAH Bittgebete als Aufrichtiger im Din Ihm gegenüber, selbst dann, sollten die Kafir dies verabscheuen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcxlviii{ER ist Der Allhöchste in den Rängen, Der von Al'ahrsch. ER läßt den Ruhh nach Seiner Anweisung dem von Seinen Dienern zukommen, dem ER will, damit er vor dem Tag des Treffens warnt.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcxlix{An dem Tag, wenn sie aufgehen. Von ihnen bleibt ALLAH nichts verdeckt. ``Wem gehört die Herrschaft heute?!" (Sie gehört) ALLAH, Dem Einzigen, Dem Allbezwingenden.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcl{An diesem Tag wird jeder Seele das vergolten, was sie erwarb. Es gibt keine Ungerechtigkeit heute. Gewiß, ALLAH ist schnell im Zur-Rechenschaft-Ziehen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcli{Und warne sie vor dem Tag des Heranrückenden, wenn die Herzen an den Kehlen würgend sitzen. Für den Unrecht-Begehenden gibt es weder einen engen Freund, noch einen Fürbittenden, dem entsprochen wird.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmclii{ER kennt das Verräterische von den Augen und das, was die Brüste verbergen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcliii{Und ALLAH richtet des Rechts gemäß! Und diejenigen, an die sie Bittgebete richten, richten in Nichts. Gewiß, ALLAH ist Der Allhörende, Der Allsehende.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcliv{Zogen sie etwa nicht auf der Erde umher, damit sie sehen, wie das Anschließende derjenigen war, die vor ihnen waren?! Sie waren noch mächtiger als sie und hatten noch mehr Hinterlassenschaften auf Erden. Dann richtete ALLAH sie wegen ihrer Verfehlungen zugrunde. Und für sie gab es vor ALLAH keinen Schutz-Gewährenden.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmclv{Dies, weil zu ihnen ihre Gesandten mit den klaren Zeichen zu kommen pflegten, dann betrieben sie Kufr, dann richtete ALLAH sie zugrunde. Gewiß, ALLAH ist allkräftig, hart im Bestrafen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmclvi{Und gewiß, bereits entsandten WIR Musa mit Unseren Ayat und einem eindeutigen Beweis\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmclvii{zu Pharao, Haman und Qarun, dann sagten sie: ``Er ist ein lügnerischer Magier."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmclviii{Und als er zu ihnen mit der Wahrheit von Uns kam, sagten sie: ``Tötet die Söhne derjenigen, die mit ihm den Iman verinnerlichten, und lasst (nur) ihre Frauen leben." Doch die List der Kafir ist nur im Irregehen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmclix{Und Pharao sagte: ``Lasst mich Musa töten, und er soll Bittgebete an seinen HERRN richten. Ja! Ich fürchte, daß er euren Din verändert, oder daß er im Lande das Verderben hervorbringt."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmclx{Und Musa sagte: ``Gewiß, ich suchte Schutz bei meinem HERRN und eurem HERRN vor jedem Arroganten, der den Iman an den Tag der Abrechnung nicht verinnerlicht."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmclxi{Und ein iman-verinnerlichender Mann von der Pharao-Familie, der seinen Iman verbarg, sagte: ``Tötet ihr einen Mann, weil er sagt: ``Mein Gott ist ALLAH", obwohl er euch bereits die klaren Zeichen von eurem HERRN brachte?! Und sollte er ein Lügner sein, so fällt sein Lügen auf ihn zurück. Doch wenn er wahrhaftig ist, trifft euch etwas von dem, was er euch verspricht. Gewiß, ALLAH leitet nicht einen recht, der maßlos, lügnerisch ist.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmclxii{Meine Leute! Euch gehört heute die Herrschaft als Sieger im Lande, also wer könnte uns vor ALLAHs Peinigung beistehen, wenn sie zu uns käme?" Pharao sagte: ``Ich rate euch nichts anderes als das, was ich meine, und ich leite euch auf nichts anderes als auf den Weg der Richtigkeit!"\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmclxiii{Und derjenige, der den Iman verinnerlichte, sagte: ``Meine Leute! Ja! Ich fürchte für euch Gleiches wie den Tag von Al-ahzab,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmclxiv{wie die Gewohnheit der Leute von Nuh, von 'Aad und von Thamud und denjenigen nach ihnen. Und ALLAH will keine Ungerechtigkeit für die Diener!\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmclxv{Und meine Leute! Ich fürchte für euch doch den Tag des Rufens,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmclxvi{wenn ihr flüchtend den Rücken umkehrt. Für euch gibt es dann vor ALLAH keinen Schützenden." Und wen ALLAH abirren läßt, für den gibt es keinen Rechtleitenden.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmclxvii{Und gewiß, bereits kam Yusuf zu euch vorher mit den klaren Zeichen, so zweifelt ihr immer noch an dem, womit er zu euch kam. Dann als er starb, sagtet ihr: ``ALLAH wird nach ihm nie einen Gesandten entsenden." Solcherart läßt ALLAH denjenigen abirren, der maßlos, voller Zweifel ist.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmclxviii{Diese sind diejenigen, die über ALLAHs Ayat disputieren ohne Beweis, welcher ihnen zuteil wurde. Dieses ist etwas schwerwiegendes Verabscheutes bei ALLAH und bei denjenigen, die den Iman verinnerlichten. Solcherart versiegelt ALLAH jedes Herz eines Arroganten, Gewalttätigen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmclxix{Und Pharao sagte: ``Haman! Laß für mich einen Turm errichten, vielleicht erreiche ich die Wege -\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmclxx{die Wege des Himmels, dann schaue ich auf Musas Gott, und gewiß, ich denke, daß er ja ein Lügner ist." Solcherart wurde Pharao das Schlechte seines Tuns schön gemacht, und er wurde vom Weg abgehalten. Und Pharaos List wurde zu Nichts außer zu Verlust.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmclxxi{Und derjenige, der den Iman verinnerlichte, sagte: ``Meine Leute! Folgt mir, so leite ich euch auf den Weg der Richtigkeit.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmclxxii{Meine Leute! Dieses diesseitige Leben ist nichts anderes als Verbrauchsgut. Und gewiß, das Jenseits ist dieWohnstätte des wahren Niederlassens.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmclxxiii{Wer Schlechtigkeit tat, dem wird Nichts außer mit Gleichem vergolten. Und wer gottgefällig Gutes tat, obMann oder Frau, während er Mumin ist, diese treten in die Dschanna ein. Darin wird ihnen Rizq ohne Berechnen zuteil.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmclxxiv{Und meine Leute! Wieso rufe ich euch zur Rettung, während ihr mich zum Feuer ruft?!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmclxxv{Ihr ruft mich, um ALLAH gegenüber Kufr zu betreiben und Ihm gegenüber Schirk mit dem zu begehen, worüber ich über kein Wissen verfüge. Und ich rufe euch doch zu Dem Allwürdigen, Dem Allvergebenden.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmclxxvi{Zweifellos eindeutig ist es, daß das, wozu ihr mich ruft, über kein Bittgebet weder im Diesseits, noch im Jenseits verfügt, und daß unser Zurückgebracht- Sein zu ALLAH ist, und daß die Maßlosen doch die Weggenossen des Feuers sind.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmclxxvii{So werdet ihr euch doch noch daran erinnern, was ich euch sage. Und ich überlasse meine Angelegenheit ALLAH. Gewiß ALLAH ist allsehend die Diener."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmclxxviii{Dann schützte ALLAH ihn vor den Bosheiten dessen, was sie an List planten. Und Pharaos Leute umgab das Schlimme der Peinigung,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmclxxix{das Feuer. Sie werden ihm morgens und abends ausgesetzt. Und an dem Tag, wenn die Stunde anbricht: ``Lasst Pharaos Leute zur qualvollsten Peinigung eintreten."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmclxxx{Und (erinnere daran), wenn sie im Feuer miteinander disputieren, dann sagen die Schwachen zu denjenigen, die sich in Arroganz erhoben: ``Gewiß, wir waren für euch Folgende, werdet ihr uns einen Anteil vom Feuer abnehmen?"\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmclxxxi{Diejenigen, die sich in Arroganz erhoben, sagten: ``Gewiß, wir sind alle drinnen. ALLAH richtete bereits über die Diener!"\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmclxxxii{Und diejenigen im Feuer sagten zu den Hütern von Dschahannam: ``Richtet Bittgebete an euren HERRN, damit ER uns einen Tag von der Peinigung erleichtert."\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmclxxxiii{Sie sagten: ``Kamen zu euch eure Gesandten etwa nicht mit den klaren Zeichen?!" Sie sagten: ``Doch, mit Sicherheit!" Sie sagten: ``Dann richtet selbst Bittgebete!" Und das Richten der Bittgebete von den Kafir ist nur verfehlt.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmclxxxiv{Gewiß, WIR stehen doch bei Unseren Gesandten und denjenigen, die den Iman verinnerlichten sowohl in diesem diesseitigen Leben als auch am Tag, wenn die Zeugen auferstehen.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmclxxxv{An dem Tag, wenn den Unrecht-Begehenden ihre Entschuldigung nicht nützt. Und für sie ist der Fluch bestimmt, und für sie ist das Schlimme der Wohnstätte bestimmt.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmclxxxvi{Und gewiß, bereits ließen WIR Musa die Rechtleitung zuteil werden, und ließen die Kinder Israils die Schrift erben\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmclxxxvii{als Rechtleitung und Ermahnung für diejenigen mit Verstand.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmclxxxviii{Also übe dich in Geduld! Gewiß, ALLAHs Versprechen ist wahr. Und bitte um Vergebung für deine Verfehlung, und lobpreise mit dem Lob deines HERRN während der Abend- und der Morgenzeit!\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmclxxxix{Gewiß, diejenigen, die über ALLAHs Ayat disputieren ohne Beweis, der ihnen zuteil wurde, in ihren Brüsten gibt es nichts außer Stolz, den sie nicht erlangen werden. So richte Isti'adha an ALLAH! Gewiß, ER ist Der Allhörende, Der Allsehende.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcxc{Gewiß, die Erschaffung der Himmel und der Erde ist noch größer als die Erschaffung der Menschen! Doch die meisten Menschen wissen es nicht.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcxci{Nicht gleich sind der Blinde und der Sehende, Auch nicht diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, und der des Schlechten Tuende. Doch ihr erinnert euch nur ein wenig.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcxcii{Gewiß, die Stunde wird bestimmt kommen, darüber gibt es keinerlei Zweifel. Doch die meisten Menschen verinnerlichen den Iman nicht.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcxciii{Und euer HERR sagte: ``Richtet an Mich Bittgebete, erhöre ICH euch! Gewiß, diejenigen, die sich in Arroganz über Mein Dienen erheben, werden doch noch in Dschahannam in Erniedrigung eintreten."\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmcxciv{ALLAH ist Derjenige, Der euch die Nacht machte, damit ihr euch darin ausruht, sowie den Tag hell. Gewiß, ALLAH hat bestimmt den Menschen Gunst erwiesen. Doch die meisten Menschen erweisen sich nicht dankbar.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmcxcv{Dieser ist ALLAH, euer HERR, Der Schöpfer aller Dinge, es gibt keine Gottheit außer Ihm. Wie werdet ihr denn davon abgebracht?!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmcxcvi{Solcherart werden diejenigen (von ALLAHs Dienen) abgebracht, die ALLAHs Ayat zu verleugnen pflegten.\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmcxcvii{ALLAH machte euch die Erde als Niederlassungsort und den Himmel als Überdachung. Und ER gab euch Gestalt, dann machte ER eure Gestalten schön, und ER gewährte euch Rizq von den Tay-yibat. Dieser ist ALLAH, euer HERR. Also immer allerhabener ist ALLAH, Der HERR aller Geschöpfe.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcxcviii{ER ist Der Lebendige. Es gibt keine Gottheit außer Ihm, so richtet an Ihn Bittgebete als Aufrichtiger im Din Ihm gegenüber. Alhamdulillah: Alles Lob gebührt ALLAH, Dem HERRN aller Schöpfung.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcxcix{Sag: ``Gewiß, mir wurde verboten, dem zu dienen, woran ihr anstelle von ALLAH Bittgebete richtet, als die klaren Zeichen von meinem HERRN zu mir kamen, und mir wurde geboten, daß ich den Islam für Den HERRN aller Schöpfung praktiziere."\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcc{ER ist Derjenige, Der euch aus Erde erschuf, dann aus Nutfa, dann aus 'Alaqa, dann bringt ER euch als Kind heraus, dann damit ihr reif werdet, dann damit ihr Alte werdet - und von euch gibt es manche, deren (Seelen) vorher eingezogen werden - und damit ihr eine festgelegte Frist erreicht. Und damit ihr begreift.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcci{ER ist Derjenige, Der belebt und sterben läßt. Und wenn ER eine Angelegenheit bestimmt, so sagt ER zu ihr nur: ``Sei!", und sie ist.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmccii{Hast du etwa nicht zu denjenigen, die über ALLAHs Ayat disputieren, geschaut, wie sie davon abgebracht werden?!\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcciii{Diejenigen, welche die Schrift leugneten und das, womit WIR Unsere Gesandten entsandten. Also sie werden doch noch wissen,\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcciv{wenn die Fesseln um ihre Hälse sind, sowie die Ketten. Sie werden gezogen\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmccv{in die Gluthitze, dann werden sie im Feuer angezündet.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmccvi{Dann wird zu ihnen gesagt: ``Wo ist das, was ihr an Schirk betrieben habt\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmccvii{anstelle von ALLAH." Sie sagten: ``Sie verloren uns. Nein, sondern wir pflegten vorher nichts anzurufen." Solcherart läßt ALLAH die Kafir abirren.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmccviii{Dies dafür, daß ihr auf Erden mit dem Unwahren euch zu vergnügen pflegtet, und dafür, daß ihr freudenerregt zu sein pflegtet.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmccix{``Betretet die Tore von Dschahannam als Ewige darin!" Und erbärmlich ist die Bleibe der Arroganten.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmccx{Also übe dich in Geduld! Gewiß, ALLAHs Versprechen ist wahr. Und entweder zeigen WIR dir etwas von dem, was WIR ihnen androhen, oder WIR lassen deine (Seele) einziehen, so werden sie danach zu Uns zurückgebracht.\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmccxi{Und gewiß, bereits entsandten WIR Gesandte vor dir. Von ihnen sind manche, über die WIR dir berichteten, und von ihnen sind manche, über die WIR dir nicht berichteten. Und es steht keinem Gesandten zu, eine Aya zu bringen außer mit ALLAHs Zustimmung. Und wenn ALLAHs Bestimmung gekommen ist, wird mit der Gerechtigkeit gerichtet, und dort werden die des Unwahren Verbreitenden verlieren.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmccxii{ALLAH ist Derjenige, Der für euch An'am machte, damit ihr auf manchen von ihnen reitet, und von ihnen speist.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmmccxiii{Und für euch gibt es in ihnen Nutzen, und damit ihr durch sie ein Bedürfnis in euren Brüsten erfüllt, und auf ihnen und auf dem Schiff werdet ihr getragen.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmccxiv{ER zeigt euch Seine Ayat, also welche von ALLAHs Ayat leugnet ihr ab?!\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmccxv{Sind sie etwa nicht auf Erden umhergezogen, damit sie sehen, wie das Anschließende derjenigen vor ihnen war. Sie waren mehr als sie, noch mächtiger und hatten noch mehr Hinterlassenschaften auf Erden. So nützte ihnen nicht, was sie zu erwerben pflegten.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmccxvi{Und als zu ihnen ihre Gesandten mit den klaren Zeichen kamen, freuten sie sich über das, was sie vom Wissen hatten. Und sie umgab das, was sie zu verspotten pflegten.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmccxvii{Und als sie Unsere Peinigung sahen, sagten sie: ``Wir verinnerlichten den Iman an ALLAH allein, und betrieben Kufr dem gegenüber, womit wir Ihm gegenüber Schirk betrieben haben."\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmccxviii{Doch ihr Iman nützte ihnen nicht, als sie Unsere Peinigung sahen. Dies ist ALLAHs Handlungsweise, die bei Seinen Dienern bereits angewandt wurde. Und verloren haben dort die Kafir!\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmccxix{\basmalahde Ha-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmccxx{Dies ist eine sukzessive Hinabsendung von Dem Allgnade Erweisenden, Dem Allgnädigen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmccxxi{Es ist eine Schrift, deren Ayat verdeutlicht wurde, ein arabischer Quran für Leute, die wissen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmccxxii{Es ist ein Warner und ein Überbringer froher Botschaft. Doch dann wandten sich die meisten ab, sie hören nicht zu.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmccxxiii{Und sie sagten: ``Unsere Herzen sind unter Bedeckung dem gegenüber, wozu du uns rufst, und in unseren Ohren ist Schwerhörigkeit, und zwischen uns und dir gibt es einen Sichtschutz, also handle! Gewiß, wir handeln ebenfalls."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmccxxiv{Sag: ``Ich bin doch nur ein Mensch wie ihr, zu mir wird Wahy zuteil, daß euer Gott nur ein einziger Gott ist, so steht aufrichtig Ihm gegenüber und bittet Ihn um Vergebung!" Und Niedergang sei für die Muschrik!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmccxxv{Diejenigen, welche die Zakat nicht entrichten, und sie sind dem Jenseits gegenüber kufr-betreibend.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmccxxvi{Gewiß, für diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, ist eine nicht ausgehende Belohnung bestimmt.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmccxxvii{Sag: ``Betreibt ihr etwa Kufr Demjenigen gegenüber, Der die Erde in zwei Ayyam erschuf, und setzt ihr Ihm etwas als Ebenbürtiges bei?! Dieser ist doch Der HERR aller Schöpfung."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmccxxviii{Und ER machte in ihr Festigende (Berge) von über ihr, setzte Baraka in sie und bestimmte in ihr ihre Nahrungen in ganzen vier Ay-yam für die Benötigenden.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmccxxix{Dann wandte ER sich dem Himmel zu, während er noch Rauch war, dann sagte ER ihm und der Erde: ``Kommt beide freiwillig oder widerwillig!" Beide sagten: ``Wir kamen freiwillig."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmccxxx{Dann bestimmte ER sie zu sieben Himmeln in zwei Ay-yam, und ließ in jedem Himmel seine Anweisung als Wahy zuteil werden. Und WIR schmückten den untersten Himmel mit Lampen und Bewahrendem. Dies ist die Bestimmung Des Allwürdigen, Des Allwissenden.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmccxxxi{Und sollten sie sich abwenden, dann sag: ``Ich warnte euch vor einem Blitzschlag wie der Blitzschlag von 'Aad und Thamud."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmccxxxii{Als zu ihnen die Gesandten von vor ihnen und von hinter ihnen kamen: ``Dient niemandem außer ALLAH." Sie sagten: ``Hätte ALLAH gewollt, hätte ER doch Engel hinabgesandt. Also gewiß, wir sind dem gegenüber, womit ihr entsandt wurdet, kufr-betreibend."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmccxxxiii{Hinsichtlich 'Aad, sie erhoben sich in Arroganz auf Erden mit dem Unrecht und sagten: ``Wer ist noch mächtiger als wir?" Sahen sie etwa nicht, daß ALLAH, Der sie erschuf, noch mächtiger als sie ist?! Und sie pflegten Unsere Ayat abzuleugnen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmccxxxiv{Dann schickten WIR über ihnen einen Wirbelsturm an unheilvollen Tagen, damit WIR sie die Peinigung der Erniedrigung im diesseitigen Leben erfahren lassen. Doch die Peinigung des Jenseits ist noch erniedrigender. Und ihnen wird nicht beigestanden.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmccxxxv{Und hinsichtlich Thamud, so leiteten WIR sie recht, dann bevorzugten sie die Verblendung vor der Rechtleitung, dann richtete sie der Blitzschlag der erniedrigenden Peinigung für das, was sie zu erwerben pflegten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmccxxxvi{Und WIR erretteten diejenigen, die den Iman verinnerlichten und Taqwa gemäß zu handeln pflegten.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmccxxxvii{Und an dem Tag, wenn ALLAHs Feinde zum Feuer versammelt werden, so werden sie geordnet beisammengehalten.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmccxxxviii{Dann wenn sie bei ihm (beim Feuer) sind, werden ihr Gehör, ihre Augen und ihre Häute Zeugnis gegen sie ablegen über das, was sie zu tun pflegten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmccxxxix{Und sie sagten zu ihren Häuten: ``Wieso habt ihr gegen uns Zeugnis abgelegt?" Sie sagten: ``ALLAH ließ uns sprechen, Der jedes Ding sprechen ließ. Und ER erschuf euch das erste Mal, und zu Ihm werdet ihr zurückgebracht."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmccxl{Und ihr pflegtet nicht euch zu bedecken, daß weder euer Gehör, noch eure Augen, noch eure Häute Zeugnis gegen euch ablegen, sondern ihr dachtet, daß ALLAH nicht viel kennt von dem, was ihr tut.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmccxli{Und dieses euer Denken, was ihr von ALLAH dachtet, richtete euch zugrunde. So wurdet ihr von den Verlierern.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmccxlii{Und sollten sie sich in Geduld üben können, so ist das Feuer für sie eine Bleibe. Und sollten sie sich um die Zufriedenheit bemühen, so wird ihnen keine Zufriedenheit entgegengebracht.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmccxliii{Und WIR bereiteten für sie enge Begleiter, dann ließen sie ihnen schön erscheinen das, was vor ihnen und hinter ihnen ist. Und das Gesagte über sie wurde vollstreckt. (Sie sind) unter Umam von den Dschinn und den Menschen, die vor ihnen vergingen. Gewiß, sie waren Verlierer.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmccxliv{Und diejenigen, die Kufr betrieben haben, sagten: ``Hört nicht diesem Quran zu und redet irgend etwas dabei, vielleicht gewinnt ihr."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmccxlv{Gewiß, WIR werden diejenigen, die Kufr betrieben haben, eine harte Peinigung erfahren lassen, und gewiß, WIR werden ihnen das Schlimmere von dem vergelten, das sie zu tun pflegten.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmccxlvi{Dies ist die Vergeltung der Feinde ALLAHs, das Feuer! Für sie ist darin die Wohnstätte der Ewigkeit als Vergeltung dafür, daß sie Unsere Ayat zu verleugnen pflegten.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmccxlvii{Und diejenigen, die Kufr betrieben haben, sagten: ``Unser HERR, zeige uns diejenigen, die uns von den Dschinn und den Menschen verleiteten, damit wir beide unter unsere Füße tun, damit beide von den Niedrigsten sind."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmccxlviii{Gewiß, auf diejenigen, die sagten: ``Unser HERR ist ALLAH", dann geradlinig wurden, werden die Engel nach und nach hinabgesandt: ``Habt keine Angst, seid nicht traurig und freut euch über die Dschanna, die euch versprochen wurde.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmccxlix{Wir sind eure Wali im diesseitigen Leben und im Jenseits. Und für euch ist darin bestimmt, was eure Seelen sich wünschen. Und darin ist für euch bestimmt, was ihr verlangt.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmccl{Sie ist) als Unterkunft von Einem Allvergebenden, Allgnädigen."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmccli{Und wer ist besser im Gesagten als derjenige, der zu ALLAH ruft, gottgefällig Gutes tut und sagt: ``Gewiß, ich bin von den Muslimen."\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcclii{Und nicht gleich sind das Gute und das Schlechte. Wehre ab mit dem, was besser ist. Dann sogleich ist derjenige, zwischen dir und dem Feindschaft ist, als wäre er ein enger Wali.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmccliii{Und dies (diese Moral) wird nur denjenigen gewährt, die sich in Geduld übten. Und dies wird nur demjenigen gewährt, der einen gewaltigen Anteil (am Guten) hat.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmccliv{Und sollte dich etwas Provozierendes vom Satan provozieren, dann richte Isti'adha an ALLAH. Gewiß, ER ist Der Allhörende, Der Allwissende.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcclv{Und zu Seinen Ayat zählen die Nacht, der Tag, die Sonne und der Mond. Vollzieht kein Sudschud weder für die Sonne, noch für den Mond! Doch vollzieht Sudschud für ALLAH, Der sie erschuf, solltet ihr Ihm dienen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcclvi{Und sollten sie sich in Arroganz erheben, so sind diejenigen bei deinem HERRN, sie lobpreisen Ihn während der Nacht und am Tage, und ihnen wird es nicht langweilig.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcclvii{Und zu Seinen Ayat zählt, daß du die Erde leblos siehst, dann wenn WIR Wasser auf sie fallen lassen, regt sie sich und schwillt. Gewiß, Derjenige, Der sie belebte, wird doch die Toten beleben. Gewiß, ER ist über alles allmächtig.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcclviii{Gewiß, diejenigen, die Unsere Ayat entstellen, bleiben Uns nicht verborgen. Ist etwa derjenige, der ins Feuer geworfen wird, besser oder derjenige, der am Tage der Auferstehung sicher kommt?! Tut, was ihr wollt! Gewiß, ER ist dessen, was ihr tut, allsehend.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcclix{Gewiß, diejenigen, die Kufr der Ermahnung gegenüber betrieben, als er zu ihnen kam, (bleiben Uns nicht verborgen). Und gewiß, er ist doch eine würdige Schrift.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcclx{Das für nichtig Erklärte kommt an ihn weder von vor ihm, noch von hinter ihm heran. Er ist eine sukzessive Hinabsendung von Einem Allweisen, Alllobenswürdigen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcclxi{Es wird dir nichts gesagt außer dem, was den Gesandten vor dir gesagt wurde. Gewiß, dein HERR hat doch Vergebung aber auch qualvolle Bestrafung.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcclxii{Und hätten WIR ihn als nichtarabischen Quran gemacht, hätten sie gesagt: ``Würden seine Ayat doch verdeutlicht - ein Nichtarabischer für Araber?" Sag: ``Er ist für diejenigen, die den Iman verinnerlichten, Rechtleitung und Heilung." Doch diejenigen, die den Iman nicht verinnerlichen, in ihren Ohren ist Schwerhörigkeit, und er ist für sie eine Blindheit. Diese werden von einem fernen Ort gerufen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcclxiii{Und gewiß, bereits ließen WIR Musa die Schrift zuteil werden, dann wurde über sie gestritten. Und gäbe es kein von deinem HERRN bereits gefälltes Wort, würde unter ihnen bestimmt gerichtet. Und gewiß sie haben über sie Verdacht schleichenden Zweifel.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcclxiv{Wer gottgefällig Gutes tut, (tut es) für sich selbst, und wer Schlechtes tut, (tut es nur) gegen sich selbst. Und dein HERR ist den Dienern gegenüber niemals ungerecht.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcclxv{Auf Ihn wird hinsichtlich des Wissens über die Stunde verwiesen. Und nicht brechen von den Früchten aus ihren Blütenkelchen hervor und nicht wird eine Weibliche schwanger und nicht entbindet sie außer mit Seinem Wissen! Und an dem Tag, wenn ER sie ruft: ``Wo sind Meine (angeblichen) Partner?!" Sagen sie: ``Wir teilten Dir mit, keiner von uns ist Zeuge."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcclxvi{Und ihnen ist abhanden gegangen, woran sie vorher Bittgebete zu richten pflegten. Und sie glaubten, daß es für sie kein Entrinnen mehr gibt.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcclxvii{Dem Menschen wird es nicht langweilig, um das Gute zu bitten, und wenn ihn Böses trifft, so ist er hoffnungslos, verzweifelt.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcclxviii{Und wenn WIRihn Gnade von Uns erfahren lassen nach einem Unglück, das ihnen zustieß, wird er sicher sagen: ``Dies ist mir! Und ich denke nicht, daß die Stunde anbrechen wird. Und würde ich zu meinem HERRN zurückgebracht, so würde für mich bei Ihm sicher das Schöne bestimmt sein." Gewiß, WIR werden denjenigen, die Kufr betrieben haben, Mitteilung über das machen, was sie taten. Und WIR werden sie doch von einer überharten Peinigung kosten lassen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcclxix{Und wenn WIR dem Menschen eineWohltat gewährt haben, wendet er sich ab und weicht zur Seite. Und wenn ihn das Unheil trifft, so macht er ein intensives Bittgebet.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcclxx{Sag: ``Wie seht ihr es? Sollte er (der Quran) von ALLAH sein, dann ihr daran Kufr betrieben habt, wer ist noch irrender als derjenige, der sich in einer unerbittlichen Feindseligkeit befindet?!"\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcclxxi{WIR werden ihnen doch noch Unsere Ayat in allen Bereichen zeigen und an ihnen selber, damit ihnen deutlich wird, daß er (der Quran) gewiß dieWahrheit ist. Genügt es nicht mit deinem HERRN, daß ER doch über alles Zeuge ist?!\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcclxxii{Ja! Sie sind doch im Zweifel über die Begegnung mit ihrem HERRN. Ja! ER ist alles zweifelsohne allumfassend.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcclxxiii{\basmalahde Ha-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcclxxiv{'Ain-sin-qaf.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcclxxv{Solcherart läßt dir und denjenigen vor dir Wahy zuteil werden ALLAH, Der Allwürdige, Der Allweise.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcclxxvi{Ihm gehört, was in den Himmeln und was auf Erden ist. Und ER ist Der Allhöchste, Der Allerhabene.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcclxxvii{Beinahe werden die Himmel sich von über ihnen spalten. Und die Engel lobpreisen mit dem Lob ihres HERRN und bitten um Vergebung für diejenigen auf Erden. Gewiß, ALLAH ist Der Allvergebende, Der Allgnädige.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcclxxviii{Und über diejenigen, die sich anstelle von Ihm Wali nahmen, ist ALLAH äußerst bewahrend. Und du bist über sie kein Wakil.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcclxxix{Und solcherart ließen WIR dir einen arabischen Quran zuteil werden, damit du die Mutterortschaft und das, was um sie herum ist, warnst, und damit du vor dem Tag des Versammelns warnst, über den es keinerlei Zweifel gibt. Ein Teil ist in der Dschanna und ein Teil ist in der Gluthitze.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcclxxx{Und hätte ALLAH gewollt, gewiß hätte ER euch zu einer einzigen Umma gemacht. Doch ER läßt in Seine Gnade eintreten, wen ER will. Und für die Unrecht-Begehenden gibt es weder Wali, noch Beistehenden.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxi{Oder nahmen sie sich etwa anstelle von Ihm Wali?! Also ALLAH ist Der wahre Wali, und ER belebt die Toten, und ER ist über alles allmächtig.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxii{Und Jegliches, worin ihr uneins werdet, so unterliegt dessen Entscheidung ALLAH. Dieser ist ALLAH, mein HERR. Ihm gegenüber übe ich Tawakkul, und zu Ihm wende ich mich hin.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxiii{ER ist Der Schöpfer der Himmel und der Erde. ER machte für euch aus euch selbst Zweiheiten, sowie von den An'am Zweiheiten. ER läßt euch damit vermehren. Es gibt nichts Seinesgleichen! Und ER ist Der Allhörende, Der Allsehende.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxiv{Ihm gehören die Schlüssel der Himmel und der Erde. ER gewährt viel Rizq, wem ER will, und ER gewährt wenig. Gewiß, ER ist über alles allwissend.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxv{ER trug euch vom Din das auf, was ER Nuh zuwies, das, was WIR dir als Wahy zuteil werden ließen, und das, was WIR Ibrahim, Musa und 'Isa zuwiesen: ``Haltet den Din ein und spaltet euch darin nicht!" Etwas Schwerwiegendes für die Muschrik ist das, wozu du sie rufst. ALLAH bringt ihm (dem Din) nahe, wen ER will, und ER leitet zu ihm recht, wer sich hinwendet.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxvi{Und sie spalteten sich nicht, außer nachdem das Wissen zu ihnen gekommen war - aus Übertretung einander gegenüber. Und gäbe es kein von deinem HERRN bereits gefälltes Wort bis zu einer festgelegten Frist, würde unter ihnen bestimmt gerichtet. Und gewiß, diejenigen, denen die Schrift nach ihnen vererbt wurde, haben über sie doch einen Verdacht schleichenden Zweifel.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxvii{Dazu mache Da'wa, halte es ein, wie es dir geboten wurde, folge nicht ihren Neigungen und sage: ``Ich verinnerlichte den Iman an das, was ALLAH von der Schrift hinabsandte, und mir wurde geboten, unter euch Gerechtigkeit walten zu lassen. ALLAH ist unser HERR und euer HERR. Uns sind unsere Handlungen und euch sind eure Handlungen. Es gibt keinen Wortstreit zwischen uns und euch. ALLAH wird uns zusammenbringen, und zu Ihm ist das Werden."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxviii{Und diejenigen, die über ALLAH disputieren, nachdem Ihm gefolgt wurde, ihr Disput ist wertlos bei ihrem HERRN, auf ihnen ist Zorn und für sie ist eine harte Peinigung bestimmt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcclxxxix{ALLAH ist Derjenige, Der die Schrift wahrheitsgemäß hinabsandte sowie Almizan. Und was läßt dich wissen?! Vielleicht liegt die Stunde doch nahe.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmccxc{Eile sie betreffend fordern diejenigen, die den Iman an sie nicht verinnerlichen. Doch diejenigen, die den Iman verinnerlichten, fürchten sich vor ihr und wissen, daß sie zweifelsohne die Wahrheit ist. Ja! Gewiß, diejenigen, die über die Stunde streiten, sind doch in weitem Irregehen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmccxci{ALLAH ist Seinen Dienern gegenüber allgütig. ER gewährt Rizq, wem ER will. Und ER ist Der Allkräftige, Der Allwürdige.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmccxcii{Wer die Ernte des Jenseits anstrebt, dem vermehren WIR seine Ernte. Und wer die Ernte des Diesseits anstrebt, dem lassen WIR davon zuteil werden, und für ihn gibt es dann im Jenseits keinen Anteil.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmccxciii{Oder gibt es für sie etwa Partner, die ihnen vom Din das zuwiesen, wozu ALLAH keine Zustimmung gab? Und gäbe es nicht das Wort des Richtens, würde bestimmt unter ihnen gerichtet. Und gewiß, für die Unrecht-Begehenden ist eine qualvolle Peinigung bestimmt.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmccxciv{Und du siehst die Unrecht-Begehenden voller Angst vor dem, was sie erwarben, und es wird ihnen geschehen. Und diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, sind in den Gärten der Dschannat. Für sie gibt es bei ihrem HERRN, was sie wollen. Dies ist die große Gunst.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmccxcv{Dies ist es, das ALLAH Seinen Dienern als frohe Botschaft überbringen läßt, denjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben. Sag: ``Ich verlange von euch dafür keinen Lohn außer der Liebe zur Verwandtschaft." Und wer ein gottgefällig Gutes erwirbt, dem vermehren WIR darin Schönes. Gewiß, ALLAH ist allvergebend, reichlichst belohnend.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmccxcvi{Oder sagen sie etwa: ``Er erdichtete im Namen ALLAHs Lügen"?! Also wenn ALLAH wollte, ER versiegelte dein Herz. Doch ALLAH läßt das für nichtig Erklärte löschen und dasWahre sich mit SeinenWorten bewahrheiten. Gewiß, ER ist allwissend über das in den Brüsten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmccxcvii{Und ER ist Derjenige, Der die Reue von Seinen Dienern annimmt, die gottmißfälligen Taten vergibt und das weiß, was ihr macht.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmccxcviii{Und ER erhört diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, und gibt ihnen noch mehr von Seiner Gunst. Und für die Kafir ist eine harte Peinigung bestimmt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmccxcix{Und würde ALLAH Seinen Dienern viel Rizq gewähren, bestimmt würden sie Übertretungen auf Erden begehen. Doch ER läßt nach einer Bestimmung nach und nach hinabsenden das, was ER will. Gewiß, ER ist Seinen Dienern gegenüber allkundig, allsehend.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmccc{Und ER ist Derjenige, Der den Regen nach und nach hinabsendet, nachdem sie verzweifelt wurden, und ER streut Seine Gnade. Und ER ist Der Wali, Der Alllobenswürdige.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmccci{Und zu Seinen Ayat zählt die Erschaffung der Himmel und der Erde, und was ER in ihnen von jedem sich bewegenden Lebewesen ausbreiten ließ. Und ER ist ihres Zusammenbringens, wenn ER will, allmächtig.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmcccii{Und was euch an Unglück traf, ist wegen dem, was ihr eigenhändig erwarbt. Und ER vergibt vieles.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmccciii{Und nicht macht ihr (Uns) auf Erden zu schaffen. Und für euch gibt es anstelle von ALLAH weder Wali, noch Beistehenden.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmccciv{Und zu Seinen Ayat zählen die Fahrenden auf dem Meer wie die Berge.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmcccv{Wenn ER will, läßt ER den Wind ruhen, dann bleiben sie stillstehend auf seiner Oberfläche. Gewiß, darin sind zweifelsohne Ayat für jeden äußerst Dankenden äußerst sich in Geduld Übenden.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcccvi{Oder ER läßt sie zugrunde gehen wegen dem, was sie erwarben, Und ER vergibt vieles.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmcccvii{Und ER kennt diejenigen, die über Unsere Ayat disputieren. Für sie gibt es kein Entrinnen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmcccviii{Also was euch auch immer zuteil wurde, dies ist das Verbrauchsgut des diesseitigen Lebens. Und was bei ALLAH ist, ist besser und bleibender für diejenigen, die den Iman verinnerlichten und Tawakkul ihrem HERRN gegenüber üben,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcccix{auch für diejenigen, welche die schweren Verfehlungen und Verabscheuenswürdigkeiten meiden, und wenn sie erzürnen, verzeihen sie,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcccx{auch für diejenigen, die ihrem HERRN folgen, das rituelle Gebet ordnungsgemäß verrichten, ihre Angelegenheiten gemäß der Schura unter sich klären und vom Rizq geben, wasWIR ihnen gewährten,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcccxi{auch für diejenigen, die, wenn das Unrecht sie traf, zurückschlagen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcccxii{Und die Vergeltung für eine Schlechtigkeit ist eine ähnliche Schlechtigkeit. Und wer verziehen und sich versöhnt hat, so obliegt dessen Belohnung ALLAH. Gewiß, ER liebt nicht die Unrecht-Begehenden.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcccxiii{Und wer zurückschlägt, nachdem ihm Unrecht angetan wurde, gegen diese gibt es keinen Weg (zur Bestrafung).\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcccxiv{Den Weg (zur Bestrafung) gibt es nur gegen diejenigen, die den Menschen Unrecht antun und Verderben auf Erden zu Unrecht anrichten, für diese ist eine qualvolle Peinigung bestimmt.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcccxv{Und wer sich in Geduld übt und verzeiht, gewiß, dies gehört bestimmt zur Entschiedenheit in den Angelegenheiten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcccxvi{Und wen ALLAH abirren läßt, für den gibt es keinen Wali nach Ihm. Und du siehst die Unrecht-Begehenden, als sie die Peinigung sahen, sagen: ``Gibt es zu einer Rückkehr einen Weg?"\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcccxvii{Und du siehst sie ihm (dem Feuer) ausgesetzt sein als Gedemütigte von der Erniedrigung, sie schauen mit verstecktem Blick. Und diejenigen, die den Iman verinnerlichten, sagten: ``Gewiß, die wirklichen Verlierer sind diejenigen, die sich selbst und ihre Familie am Tag der Auferstehung verloren." Jedenfalls, die Unrecht-Begehenden sind in einer andauernden Peinigung.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcccxviii{Und es gibt für sie keinerlei Wali, die ihnen anstelle von ALLAH beistehen. Und wen ALLAH abirren läßt, für den gibt es keinen Weg (zur Rechtleitung).\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcccxix{Hört auf euren HERRN, bevor ein Tag kommt, für den es kein Abwenden von ALLAH gibt. Für euch gibt es an diesem Tag keinen Zufluchtsort, und für euch gibt es kein Leugnen.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcccxx{Und sollten sie sich abwenden, so entsandten WIR dich zu ihnen nicht als Bewahrenden. Dir obliegt nichts außer dem Verkünden. Und gewiß, wenn WIR den Menschen von Uns Gnade erfahren ließen, freute er sich darüber. Und wenn sie Schlechtigkeit trifft wegen dem, was sie eigenhändig machten, also gewiß, der Mensch ist äußerst kufr-betreibend.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcccxxi{ALLAH gehört die Herrschaft der Himmel und der Erde. ER erschafft, was ER will. ER schenkt, wem ER will, Weibliche, und ER schenkt, wem ER will, Männliche,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcccxxii{oder ER macht sie zu Zweitheiten - Männliche und Weibliche. Und ER macht, wen ER will, unfruchtbar. Gewiß, ER ist allwissend, allmächtig.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcccxxiii{Und es steht keinem Menschen zu, daß ALLAH zu ihm spricht außer durch Wahy oder von hinter einem Sichtschutz oder ER entsendet einen Gesandten, dann läßt ER Wahy mit Seiner Zustimmung zuteil werden, was ER will. Gewiß, ER ist allhöchst, allweise.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcccxxiv{Und solcherart ließen WIR dir als Wahy einen Ruhh von Unserer Anweisung zuteil werden. Nicht wußtest du, weder was die Schrift ist, noch der Iman. Doch WIR machten ihn als Licht, mit dem WIR diejenigen von Unseren Dienern, die WIR wollen, rechtleiten. Und gewiß, du leitest doch zu einem geradlinigen Weg recht,\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcccxxv{zum Weg ALLAHs, Dem gehört, was in den Himmeln und was auf Erden ist. Ja! Zu ALLAH werden die Angelegenheiten werden.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcccxxvi{\basmalahde Ha-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcccxxvii{Bei der deutlichen Schrift!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcccxxviii{Gewiß, WIR ließen sie einen arabischen Quran werden, damit ihr euch besinnt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcccxxix{Und gewiß, er ist in der Mutterschrift bei Uns zweifelsohne hoch, weise.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcccxxx{Lassen WIR von euch etwa die Ermahnung abwenden, weil ihr maßlose Leute wart?!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxi{Und wie viele von Propheten entsandten WIR unter die Früheren.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxii{Und es kam zu ihnen kein Prophet, ohne daß sie ihn zu verspotten pflegten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxiii{So richteten WIR diejenigen zugrunde, die noch gewalttätiger als sie waren. Und das Gleichnis der Früheren kam bereits vor.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxiv{Und würdest du sie fragen: ``Wer erschuf die Himmel und die Erde?", gewiß würden sie sagen: ``Sie erschuf Der Allwürdige, Der Allwissende."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxv{Derjenige, Der euch die Erde als Unterlage machte, und auf ihr machte ER euch Wege, damit ihr Rechtleitung findet.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxvi{Auch Derjenige, Der vom Himmel Wasser nach Bestimmung fallen ließ, dann ließen WIR damit eine tote Landschaft erstehen. Solcherart werdet ihr hervorgebracht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxvii{Auch Derjenige, Der die Zweiheiten allesamt erschuf, und für euch von den Schiffen und den An'am machte, was ihr besteigt,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxviii{damit ihr euch auf ihren Rücken/Decks zurechtsetzt, dann euch an die Wohltat eures HERRN erinnert, wenn ihr euch darauf zurechtgesetzt habt, und sagt: ``subhanal-ladhi sach-chara lana hadha wama kunna lahu muqrinin\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcccxxxix{wa inna ila rab-bina lamun-qalibun."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcccxl{Und sie schrieben Ihm von Seinen Dienern ein Teil zu. Gewiß, der Mensch ist doch ein deutlicher äußerst Kufr-Betreibender.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcccxli{Oder ER nahm sich von dem, was ER erschuf, Töchter und bevorzugt euch mit Söhnen?!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcccxlii{Und wenn einem von ihnen frohe Botschaft überbracht wird über das, was er Dem Allgnade Erweisenden als Gleichnis prägte, blieb sein Gesicht verfinstert, während er voller Groll ist.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcccxliii{Etwa diejenigen, die mit Schmuck aufwachsen, (schreiben sie Ihm zu), und die beim Disputieren nicht deutlich sind?!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcccxliv{Und sie machten die Engel, welche die Diener Des Allgnade Erweisenden sind, zu Weiblichen. Waren sie etwa bei ihrer Erschaffung Zeugen?! Ihr Zeugnis wird niedergeschrieben, und sie werden zur Rechenschaft gezogen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcccxlv{Und sie sagten: ``Hätte Der Allgnade Erweisende es nicht gewollt, hätten wir ihnen nicht gedient!" Darüber verfügen sie über keinerlei Wissen, gewiß, sie lügen nur.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcccxlvi{Oder ließen WIR ihnen etwa eine Schrift vor ihm (dem Quran), so daß sie an ihr festhalten?!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcccxlvii{Nein, sondern sie sagten: ``Gewiß, wir fanden unsere Ahnen auf einem Weg vor, und wir sind ihren Spuren doch nach geleitet."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcccxlviii{Und solcherart entsandten WIR nicht vor dir zu einer Ortschaft einen Warner, ohne daß ihre Luxurierenden sagten: ``Wir fanden unsere Ahnen auf einem Weg vor, und wir sind ihren Spuren folgend."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcccxlix{Sag: ``Selbst dann, wenn ich euch etwas Rechtleitenderes brachte als das, wobei ihr eure Ahnen vorgefunden habt?" Sie sagten: ``Gewiß, wir sind dem gegenüber, womit ihr entsandt wurdet, kufrbetreibend!"\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcccl{Dann übten WIR Vergeltung an ihnen. So siehe, wie das Anschließende der äußerst Lügenden war?!\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcccli{Und (erinnere daran), als Ibrahim zu seinem Vater und seinen Leuten sagte: ``Gewiß, ich sage mich los von dem, was ihr dient,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmccclii{außer von Demjenigen, Der mich erschuf, denn ER wird mich rechtleiten."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmcccliii{Und er machte es zu einem bleibenden Wort unter seinen Nachkommen, damit sie umkehren.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmcccliv{Nein, sondern ICH gewährte diesen und ihren Ahnen Verbrauchsgüter, bis zu ihnen die Wahrheit und ein deutlicher Gesandter kam.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmccclv{Und als zu ihnen die Wahrheit kam, sagten sie: ``Dies ist Magie, und wir sind doch dem gegenüber äußerst kufr-betreibend."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmccclvi{Und sie sagten: ``Wäre dieser Quran doch einem erhabenen Mann aus den beiden Ortschaften hinabgesandt worden!"\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmccclvii{Sind sie etwa diejenigen, welche die Gnade deines HERRN verteilen?! WIR verteilten unter ihnen ihren Lebensunterhalt im diesseitigen Leben. Und WIR erhöhten die einen von ihnen über die anderen um Stellungen, damit die einen von ihnen die anderen zu Beschäftigten nehmen. Doch die Gnade deines HERRN ist besser als das, was sie anhäufen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmccclviii{Und würden dieMenschen nicht zu einer Umma (in Kufr) werden, hätten WIR bestimmt denjenigen, die Dem Allgnade Erweisenden gegenüber Kufr betreiben, für ihre Häuser Dächer aus Silber gemacht sowie Treppen, auf denen sie aufsteigen,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmccclix{sowie für ihre Häuser Tore und Liegen, auf denen sie sich anlehnen,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmccclx{sowie Zierschmuck. Und dies alles ist nur das Verbrauchsgut des diesseitigen Lebens. Doch das Jenseits bei deinem HERRN gilt den Muttaqi.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmccclxi{Und wer die Ermahnung Des Allgnade Erweisenden ignoriert, für den bestimmen WIR einen Satan, so ist er für ihn ein Begleiter.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmccclxii{Und gewiß, sie halten sie vom wahren Weg ab und denken, daß sie Rechtgeleitete sind.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmccclxiii{Dann als er zu Uns kam, sagte er: ``Wäre zwischen mir und dir doch die Entfernung beider Sonnengänge. Also erbärmlich ist der Begleiter."\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmccclxiv{Und euch wird es heute nicht nützen, da ihr Unrecht begangen habt, daß ihr in der Peinigung Beteiligte seid.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmccclxv{Bist du etwa derjenige, der die Tauben hören läßt, oder gewährst du etwa den Blinden Leitung und denjenigen, die in einem weiten Irregehen sind?!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmccclxvi{Und entweder würden WIR dich weggehen lassen, dann werden WIR gewiß an ihnen Vergeltung üben,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmccclxvii{oder WIR würden dir das zeigen, was WIR ihnen androhten, so sind WIR gewiß ihnen gegenüber allmächtig.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmccclxviii{So halte richtig fest an dem, was dir alsWahy zuteil wurde. Gewiß, du bist auf einem geradlinigen Weg.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmccclxix{Und gewiß, er ist eine Ehre für dich und für deine Leute. Und ihr werdet zur Rechenschaft gezogen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmccclxx{Und frage diejenigen, die WIR vor dir von Unseren Gesandten entsandten: ``Machten WIR etwa anstelle von Dem Allgnade Erweisenden Gottheiten, denen gedient wurde?!"\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmccclxxi{Und gewiß, bereits entsandten WIR Musa mit Unseren Ayat zu Pharao und seinen Entscheidungsträgern, dann sagte er: ``Gewiß, ich bin der Gesandte Des HERRN aller Schöpfung."\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmccclxxii{Und als er zu ihnen mit Unseren Ayat kam, sogleich lachten sie ihn aus.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmccclxxiii{Und WIR zeigen ihnen nicht eine Aya außer daß sie noch größer als ihre Vorläuferin ist. Und WIR belangten sie mit der Peinigung, damit sie umkehren.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmccclxxiv{Und sie sagten: ``Du Magier! Richte Bittgebete an deinen HERRN mit dem, wozu ER dich verpflichtete, wir sind gewiß rechtgeleitet!"\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmccclxxv{Und als WIR ihnen die Peinigung wegnahmen, da waren sie wortbrüchig.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmccclxxvi{Und Pharao rief unter seinen Leuten, er sagte: ``Meine Leute! Habe ich nicht die Herrschaft über Ägypten und diese Flüsse fließen da unter mir? Habt ihr etwa keinen Einblick?!\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmccclxxvii{Doch ich bin besser als dieser, der verachtet ist und beinahe nicht deutlich erklärt.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmccclxxviii{Also würde doch ihm ein Armband aus Gold gegeben, oder kämen doch die Engel mit ihm einanderfolgend!"\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmccclxxix{So forderte er von seinen Leuten Unbesonnenheit, dann gehorchten sie ihm. Gewiß, sie waren fisqbetreibende Leute.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmccclxxx{Und als sie Unseren Zorn provozierten, übten WIR an ihnen Vergeltung, dann ließen WIR sie allesamt ertrinken.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxi{Dann machtenWIR sie zum Typus und zum Gleichnis für die anderen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxii{Und als Ibnu-Maryam als Gleichnis geprägt wurde, sogleich brachen sie in schallendes Gelächter aus.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxiii{Und sie sagten: ``Sind unsere Götter besser oder er?" Sie prägten es dir nur zum Disput. Nein, sondern sie sind streitsüchtige Leute.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxiv{Gewiß, er ist nur ein Diener, dem WIRWohltaten erwiesen und zum Gleichnis für die Kinder Israils gemacht.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxv{Und würden WIR es wollen, würden WIR aus euch Engel auf Erden machen, die (euch) nachfolgen.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxvi{Und gewiß er ist doch ein Wissen für die Stunde. So zweifelt sie nicht an und folgt Mir! Dies ist ein geradliniger Weg.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxvii{Und der Satan soll euch nicht davon abhalten! Gewiß, er ist für euch ein entschiedener Feind.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxviii{Und als 'Isa mit den klaren Zeichen kam, sagte er: ``Bereits brachte ich euch die Weisheit, und damit ich euch manches von dem erkläre, worin ihr uneins seid. So handelt Taqwa gemäß ALLAH gegenüber und gehorcht mir!\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmccclxxxix{Gewiß, ALLAH ist mein HERR und euer HERR, so dient Ihm! Dies ist ein geradliniger Weg."\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcccxc{Dann wurden die Parteien unter ihnen uneins. Also Niedergang ist für diejenigen, die Unrecht begingen, bei der Peinigung eines qualvollen Tages bestimmt.\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcccxci{Warten sie etwa außer auf die Stunde, daß sie zu ihnen plötzlich kommt, während sie nicht merken.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcccxcii{Die engen Freunde sind an diesem Tag die einen von ihnen doch den anderen Feind, außer den Muttaqi.\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcccxciii{Meine Diener! Es gibt weder Angst um euch an dem Tag, noch werdet ihr traurig sein.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmcccxciv{Diejenigen, die den Iman an Unsere Ayat verinnerlicht haben und Muslime waren,\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcccxcv{tretet in die Dschanna ein! Ihr und eure Partnerwesen freut euch.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcccxcvi{Ihnen wird serviert mit Schalen aus Gold und Bechern. Und darin ist das, was die Seelen begehren und die Augen genießen. Und ihr bleibt darin ewig.\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmcccxcvii{Und dies ist die Dschanna, die euch überlassen wurde für das, was ihr zu tun pflegtet.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmcccxcviii{Für euch ist darin viel Obst, von dem ihr speist.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmcccxcix{Gewiß, die schwer Verfehlenden bleiben in der Peinigung von Dschahannam ewig.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmcd{Ihnen wird sie nicht gemildert, und sie sind darin verzweifelt.\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmcdi{Und WIR taten ihnen kein Unrecht an, sondern sie pflegten, die Unrecht-Begehenden zu sein.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmcdii{Und sie riefen: ``Malik! Laß deinen HERRN uns vernichten!" Er sagte: ``Gewiß, ihr müsst bleiben."\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmcdiii{Gewiß, bereits kamen WIR zu euch mit der Wahrheit. Doch die meisten von euch sind der Wahrheit gegenüber abgeneigt.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmcdiv{Oder entschlossen sie sich zu einem Ding?! Gewiß, WIR entschlossen Uns auch.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmmcdv{Oder denken sie etwa, daß WIR ihr Geheimnis und ihre heimliche Unterredung nicht hören?! Doch, mit Sicherheit! Und Unsere Boten bei ihnen schreiben es auf.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmcdvi{Sag: ``Sollte Der Allgnade Erweisende ein Kind haben, so bin ich der Erste der Dienenden."\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmcdvii{Gepriesen-erhaben ist Der HERR der Himmel und der Erde, Der HERR von Al'ahrsch, über das, was sie (Ihm) zudichten.\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmcdviii{Also laß sie sich unterhalten und Unfug treiben, bis sie ihren Tag treffen, der ihnen angedroht wird.\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmcdix{Und ER ist Derjenige, Der im Himmel eine Gottheit ist und auf Erden eine Gottheit ist. Und ER ist Der Allweise, Der Allwissende.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmcdx{Und Baraka gewährt Derjenige, Dem die Herrschaft der Himmel, der Erde und dessen, was zwischen ihnen ist, gehört. Und bei Ihm ist das Wissen über die Stunde. Und zu Ihm werdet ihr zurückgebracht.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmcdxi{Und nicht verfügen diejenigen, an die sie anstelle von Ihm Bittgebete richten, über die Fürbitte außer denjenigen, die wahrheitsgemäß Zeugnis ablegen, während sie wissen.\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmmcdxii{Und würdest du sie fragen, wer sie erschuf, gewiß würden sie sagen: ``ALLAH!" Also wie wurden sie davon abgebracht?!\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmmcdxiii{Und bei seiner Rede: ``Mein HERR! Gewiß, diese sind Leute, die den Iman nicht verinnerlichen", (werde ICH sie strafen).\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmmcdxiv{So wende dich von ihnen ab und sag: ``Salam (mit euch)", denn sie werden noch wissen.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmmcdxv{\basmalahde Ha-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcdxvi{Bei der deutlichen Schrift!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcdxvii{Gewiß, WIR sandten sie in einer Nacht voller Baraka hinab - Gewiß, WIR waren Warnende! -\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcdxviii{in ihr wird jede weise Angelegenheit entschieden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcdxix{WIR sandten sie hinab), als Angelegenheit von Uns. Gewiß, WIR waren Entsendende\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcdxx{eine Gnade von deinem HERRN. Gewiß, ER ist Der Allhörende, Der Allwissende,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcdxxi{Der HERR der Himmel, der Erde und dessen, was dazwischen ist. Solltet ihr von den Gewißheit-Anstrebenden sein.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcdxxii{Es gibt keine Gottheit außer Ihm, ER belebt und läßt sterben. ER ist euer HERR und Der HERR eurer ersten Ahnen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcdxxiii{Nein, sondern sie sind im Zweifel, sie treiben Unfug.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcdxxiv{Also warte auf den Tag, wenn der Himmel mit deutlichem Rauch kommt,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcdxxv{der die Menschen umhüllt. Dies ist eine qualvolle Peinigung.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcdxxvi{``Unser HERR, nimm von uns die Peinigung weg! Gewiß, wir sind Mumin."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcdxxvii{Woher soll denn für sie die Ermahnung sein, und bereits kam zu ihnen ein deutlicher Gesandter?!\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcdxxviii{Dann wandten sie sich von ihm ab und sagten: ``Er ist belehrt, geistesgestört."\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcdxxix{Gewiß, WIR werden die Peinigung ein wenig wegnehmen. Gewiß, ihr seid Rückfällige.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcdxxx{An dem Tag, wenn WIR die größte Gewalt anwenden lassen, gewiß, WIR sind Vergeltung Übende.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxi{Und gewiß, bereits unterzogen vor ihnen Pharaos Leute eine Fitna und zu ihnen kam ein edler Gesandter:\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxii{``Gebt mir ALLAHs Diener! Gewiß, ich bin für euch ein vertrauenswürdiger Gesandter.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxiii{Und erhebt euch nicht in Arroganz ALLAH gegenüber! Gewiß, ich brachte euch einen deutlichen Beweis.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxiv{Und gewiß, ich suchte Schutz bei meinem HERRN und eurem HERRN, daß ihr mich steinigt.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxv{Und solltet ihr mir keinen Iman schenken, dann bleibt mir fern!"\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxvi{Dann richtete Bittgebet an seinen HERRN: ``Diese sind doch schwer verfehlende Leute."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxvii{``Also mache dich mit Meinen Dienern nachts auf den Weg! Gewiß, ihr werdet verfolgt.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxviii{Und laß das Meer still sein! Gewiß, sie sind ertränkte Soldaten."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcdxxxix{Wie viele Dschannat und Quellen sie hinterließen,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcdxl{auch Gewächs und edlen Aufenthalt\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmcdxli{sowie Wohlergehen, in dem sie freudenerregt waren.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmcdxlii{Solcherart! Und WIR überließen diese anderen Leuten zum Erbe.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmcdxliii{So weinten ihnen weder Himmel, noch Erde nach. Und ihnen wurde kein Aufschub gewährt.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmcdxliv{Und gewiß, bereits erretteten WIR die Kinder Israils von der erniedrigenden Peinigung,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmcdxlv{von Pharao. Gewiß, er war ein Arroganter von den Maßlosen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmcdxlvi{Und gewiß, bereits erwählten WIR sie nach Wissen vor den (anderen) Menschen aus.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmcdxlvii{Und WIR ließen ihnen von den Ayat zuteil werden, worin deutliche Prüfung ist.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcdxlviii{Gewiß, diese sagen doch:\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmcdxlix{``Es ist nur unser erster Tod und wir werden nie auferstehen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmcdl{So bringt unsere Ahnen, solltet ihr wahrhaftig sein!"\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcdli{Sind sie etwa besser oder die Leute von Tubba' und diejenigen vor ihnen?! WIR richteten sie zugrunde. Gewiß, sie waren schwer verfehlende Leute!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcdlii{Und WIR erschufen die Himmel, die Erde und das, was zwischen ihnen ist, nicht als Unfug-Treibende.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcdliii{WIR erschufen beide nur nach der Gesetzmäßigkeit. Doch die meisten wissen es nicht.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcdliv{Gewiß, der Tag des Richtens ist ihr Termin, allesamt.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcdlv{An dem Tag, wenn kein Maula einem Maula etwas nützen kann. Und ihnen wird nicht beigestanden,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcdlvi{außer denjenigen, denen ALLAH Gnade erwies. Gewiß, ER ist Der Allwürdige, Der Allgnädige.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcdlvii{Gewiß, der Zaqqum-Baum\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcdlviii{ist die Speise des Verfehlenden.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcdlix{Er ist wie das Verschmolzene, das in den Bäuchen kocht,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcdlx{wie das Sieden des Siedenden.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcdlxi{``Nehmt ihn, dann schleift ihn in Dschahannams Mitten,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcdlxii{dann gießt über seinen Kopf von der Peinigung des Siedenden."\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcdlxiii{``Koste! Gewiß, du bist der Würdige, der Edle."\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcdlxiv{Gewiß, dies ist das, was ihr zu bezweifeln pflegtet.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcdlxv{Gewiß, die Muttaqi sind an einem sicheren Aufenthaltsort,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcdlxvi{in Dschannat und an Quellen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcdlxvii{Sie tragen Kleidung aus Sundos und Istabraq einander gegenüber (sitzend).\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcdlxviii{Solcherart! Und WIR vermählten sie mit Hur-'in.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcdlxix{Sie verlangen darin nach jeglicher Art Obst als sich sicher Fühlende.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcdlxx{Nicht erfahren sie den Tod darin, außer dem ersten Tod. Und ER schützte sie vor der Peinigung der Hölle,\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxi{als Gunst von deinem HERRN. Dies ist der gewaltige Erfolg.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxii{Und WIR erleichterten ihn doch in deiner Sprache, damit sie sich besinnen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxiii{So warte! Gewiß, sie warten ebenfalls.\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxiv{\basmalahde Ha-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxv{Die sukzessive Hinabsendung der Schrift ist von ALLAH, Dem Allwürdigen, Dem Allweisen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxvi{Gewiß, in den Himmeln und auf Erden sind doch Ayat für die Mumin.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxvii{Auch in eurer Schöpfung und in jedem sich bewegenden Lebewesen, was ER ausbreiten ließ, sind Ayat für Leute, die Gewißheit anstreben.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxviii{Auch die Aufeinanderfolge von Nacht und Tag und Rizq (Regen), das ALLAH vom Himmel fallen ließ, dann ER damit die Landschaft nach ihrem Tod wieder belebte, und die Strömung der Winde sind Ayat für Leute, die sich besinnen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxix{Diese sind ALLAHs Ayat, die WIR dir wahrheitsgemäß vortragen. An welche Botschaft nach ALLAH und Seinen Ayat werden sie denn den Iman verinnerlichen?!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxx{Niedergang ist für jeden verfehlenden Ifk-Lügner,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxi{der ALLAHs Ayat hört, wie sie ihm vortragen wird, dann in Arroganz sich erhebend beharrt, als hätte er sie nicht gehört. So überbringe ihm die ``frohe Botschaft" über eine qualvolle Peinigung.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxii{Und wenn er von Unseren Ayat etwas kennt, nimmt er sie zum Spott. Für diese ist eine erniedrigende Peinigung bestimmt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxiii{Vor ihnen steht Dschahannam. Und weder das, was sie sich erwarben, wird ihnen nützen, noch das, was sie sich anstelle von ALLAH als Wali nahmen. Und für sie ist eine überharte Peinigung bestimmt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxiv{Dies (der Quran) ist eine Rechtleitung. Und für diejenigen, die Kufr den Ayat ihres HERRN gegenüber betreiben, ist eine Peinigung von äußerst qualvoller Strafe bestimmt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxv{ALLAH ist Derjenige, Der für euch das Meer gratis fügbar machte, damit die Schiffe darauf nach Seiner Anweisung fahren, damit ihr von Seiner Gunst anstrebt und damit ihr euch dankbar erweist.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxvi{Und ER machte für euch gratis fügbar das, was in den Himmeln und was auf Erden ist, allesamt sind von Ihm. Gewiß, darin sind Ayat für Leute, die nachdenken.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxvii{Sag denjenigen, die den Iman verinnerlichten, denjenigen zu verzeihen, die auf die siegreichen Schlachten für ALLAH nicht hoffen, damit ALLAH Leuten vergilt, mit dem, was sie zu erwerben pflegten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxviii{Wer gottgefällig Gutes tat, (dies tut er) für sich selbst, und wer Schlechtes tat, (dies tut er) gegen sich selbst. Dann werdet ihr zu eurem HERRN zurückgebracht.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcdlxxxix{Und gewiß, bereits ließen WIR den Kindern Israils die Schrift, die Weisheit und die Prophetenschaft zuteil werden, WIR gewährten ihnen Rizq von den Tayyibat, und WIR bevorzugten sie vor den (anderen) Menschen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcdxc{Auch ließen WIR ihnen klare Zeichen von der Angelegenheit zuteil werden. So wurden sie nicht uneins außer, nachdem zu ihnen das Wissen gekommen ist - aus Übertretung untereinander. Gewiß, dein HERR richtet unter ihnen am Tag der Auferstehung über das, worüber sie uneins zu sein pflegten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcdxci{Dann brachten WIR dich zu einer Schir'a von der Angelegenheit, so folge ihr und folge nicht den Neigungen derjenigen, die nicht wissen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcdxcii{Gewiß, sie werden dir vor ALLAH in Nichts nützen! Und gewiß, die Unrecht-Begehenden die einen von ihnen sind Wali für die anderen. Doch ALLAH ist der Wali für die Muttaqi.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcdxciii{Dieser (der Quran) ist etwas Einblick-Gewährendes für die Menschen, eine Rechtleitung und eine Gnade für Leute, die Gewißheit anstreben.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcdxciv{Oder dachten diejenigen, welche die gottmißfälligen Taten verübten, daß WIR sie denjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, gleich machen, in ihrem Leben und in ihrem Tod. Erbärmlich ist das, was sie urteilen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcdxcv{Und ALLAH erschuf die Himmel und die Erde in Gesetzmäßigkeit, und damit jeder Seele das vergolten wird, was sie erwarb. Und ihnen wird kein Unrecht angetan.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcdxcvi{Hast du etwa denjenigen gesehen, der sich seine Neigung als seine Gottheit nahm, und ALLAH ihn mit Wissen abirren ließ und sein Gehör und sein Herz versiegelte und auf seine Augen (Sicht-)Blende machte, wer kann ihn rechtleiten nach ALLAH?! Entsinnt ihr euch nicht?!\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcdxcvii{Und sie sagten: ``Es gibt Nichts außer unserem diesseitigen Leben, wir sterben und leben und uns läßt nicht sterben außer der Zeit." Und sie verfügen darüber über keinerlei Wissen. Gewiß, sie spekulieren nur.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcdxcviii{Und wenn ihnen Unsere Ayat als Deutliche vorgetragen werden, es war ihre Argumentation nichts außer, daß sie sagten: ``Bringt unsere Ahnen herbei, solltet ihr wahrhaftig sein!"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcdxcix{Sag: ``ALLAH belebt euch, dann läßt ER euch sterben, dann läßt ER euch zum Tag der Auferstehung, über den es keinen Zweifel gibt, versammeln." Doch die meisten Menschen wissen es nicht.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmd{Und ALLAH gehört die Herrschaft der Himmel und der Erde. Und an dem Tag, wenn die Stunde anbricht, an diesem Tag, verlieren die des Unwahren Verbreitenden.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdi{Du siehst jede Umma kniend. Jede Umma wird zu ihrer Schrift gerufen: ``An diesem Tag wird euch vergolten, was ihr zu tun pflegtet."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdii{Dies ist Unsere Schrift, sie sagt gegen euch wahrheitsgemäß aus. Gewiß, WIR ließen das abschreiben, was ihr zu tun pflegtet.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdiii{Also hinsichtlich derjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, diese läßt ihr HERR in Seine Gnade eintreten. Dies ist der eindeutige Erfolg.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdiv{Doch hinsichtlich derjenigen, die Kufr betrieben haben: ``Wurden euch Meine Ayat nicht vorgetragen, dann ihr euch in Arroganz erhoben habt und ihr schwer verfehlende Leute wart?!"\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdv{Und als gesagt wurde: ``Gewiß, ALLAHs Versprechen ist wahr sowie die Stunde, über die es keinen Zweifel gibt." Habt ihr gesagt: ``Wir wissen nicht, was die Stunde ist, wir spekulieren doch nur eine Spekulation und wir haben keine Gewißheit darüber."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdvi{Und ihnen wurden die Bosheiten dessen sichtbar, was sie taten. Und sie umgab das, was sie zu verspotten pflegten.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdvii{Und es wurde gesagt: ``An diesem Tag vernachlässigen WIR euch, wie ihr die Begegnung mit eurem diesem Tag vergessen habt, und eure Unterkunft ist das Feuer. Und für euch gibt es heute keinerlei Beistehende.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdviii{Dies, weil ihr ALLAHs Ayat doch zum Spott genommen habt und das diesseitige Leben euch täuschte! Also an diesem Tag, weder gehen sie aus ihm (Feuer) heraus, noch werden sie zufriedengestellt.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdix{So gebührt alles Lob ALLAH, Dem HERRN der Himmel und Dem HERRN der Erde, Dem HERRN aller Schöpfung.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdx{Und Ihm gehört die Herrlichkeit in den Himmeln und auf Erden. Und ER ist Der Allwürdige, Der Allweise.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdxi{\basmalahde Ha-mim.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdxii{Die sukzessive Hinabsendung der Schrift ist von ALLAH, Dem Allwürdigen, Dem Allweisen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdxiii{Nicht erschufen WIR die Himmel, die Erde und das, was zwischen ihnen ist, außer nach Gesetzmäßigkeit und festgelegter Frist. Und diejenigen, die Kufr betrieben haben, sind dem gegenüber, wovor sie gewarnt wurden, abwendend.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdxiv{Sag: ``Habt ihr gesehen, woran ihr anstelle von ALLAH Bittgebete richtet? Zeigt mir, was sie von der Erde erschufen! Oder haben sie etwa eine Beteiligung an den Himmeln? Bringt mir eine Schrift von vor diesem (Quran) oder etwas Hinterlassenes vom Wissen, solltet ihr wahrhaftig sein."\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdxv{Und keiner ist irrender als derjenige, der anstelle von ALLAH Bittgebet richtet an das, was ihn bis zum Tag der Auferstehung nicht erhört. Und sie sind ihrem Bittgebet gegenüber achtlos.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdxvi{Und wenn die Menschen versammelt werden, werden sie für sie Feinde sein, auch werden sie ihrem Dienen gegenüber kufrbetreibend sein.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdxvii{Und wenn ihnen Unsere Ayat als Deutliche vorgetragen werden, sagen diejenigen, die Kufr betrieben haben, zurWahrheit, nachdem sie zu ihnen kam: ``Dies ist eine eindeutige Magie."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdxviii{Oder sagen sie etwa: ``Er erdichtete ihn"?! Sag: ``Sollte ich ihn erdichtet haben, würdet ihr für mich vor ALLAH nichts machen können! ER weiß besser, was ihr über ihn lästert. ALLAH genügt als Zeuge zwischen mir und euch. Und ER ist Der Allvergebende, Der Allgnädige."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdxix{Sag: ``Ich war keine Neuartigkeit unter den Gesandten. Und ich weiß nicht, weder was mit mir gemacht wird, noch mit euch. Ich folge ausschließlich dem, was mir als Wahy zuteil wird. Und ich bin doch nichts anderes als ein deutlicher Warner."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdxx{Sag: ``Wie seht ihr es? Sollte er von ALLAH sein, und ihr betreibt ihm gegenüber Kufr." Und ein Zeuge von den Kindern Israils legte Zeugnis ab für seinesgleichen, dann verinnerlichte er den Iman, während ihr euch in Arroganz erhoben habt. Gewiß, ALLAH leitet die unrecht-begehenden Leute nicht recht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdxxi{Und diejenigen, die Kufr betrieben haben, sagten zu denjenigen, die den Iman verinnerlichten: ``Wäre er etwas Gutes, wären sie zu ihm uns nicht zuvorgekommen." Und da sie durch ihn keine Rechtleitung fanden, so werden sie sagen: ``Dies ist eine alte Ifk-Lüge."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdxxii{Und vor ihm war die Schrift von Musa ein Imam und eine Gnade. Und dies ist eine bestätigende Schrift in arabischer Sprache, damit er diejenigen, die Unrecht begingen, warnt und eine frohe Botschaft für die Muhsin ist.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdxxiii{Gewiß, diejenigen, die sagten: ``Unser HERR ist ALLAH", dann sich geradlinig benahmen, über sie gibt es weder Angst, noch werden sie traurig sein.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdxxiv{Diese sind die Weggenossen der Dschanna. Darin bleiben sie ewig als Vergeltung für das, was sie zu tun pflegten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdxxv{Und WIR wiesen dem Menschen zu, seinen Eltern Ihsan zu erweisen. Seine Mutter war schwanger mit ihm in Angestrengtheit und brachte ihn zur Welt in Angestrengtheit. Und (die Zeit) der Schwangerschaft mit ihm und sein Entwöhnen ist dreißig Monate. Dann als er seine Reife erreichte und vierzig Jahre alt wurde, sagte er: ``Mein HERR! Gib mir ein, daß ich mich Deiner Wohltat, die DU mir und meinen Eltern gewährt hast, dankbar erweise, und daß ich gottgefällig Gutes tue, woran DU Wohlgefallen findest. Und gewähre mir gottgefällig Gutes bei meiner Nachkommenschaft! Ja! Ich bereute Dir gegenüber und gewiß, ich bin von den Muslimen."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdxxvi{Diese sind diejenigen, von denen WIR das Beste von dem, was sie taten, annehmen und deren gottmißfälligen Taten WIR nachsehen mit den Weggenossen der Dschanna. Dies ist das Versprechen der Wahrheit, das ihnen stets versprochen wurde.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdxxvii{Und derjenige, der seinen Eltern sagte: ``Pfui für euch! Versprecht ihr mir, daß ich hervorgebracht werde, und bereits vergingen die Generationen vor mir." Und beide rufen ALLAH um Hilfe und sagen: ``Dein Niedergang! Verinnerliche den Iman! ALLAHs Versprechen ist gewiß wahr." Dann sagt er: ``Dies ist doch nichts anderes als die Legenden der Früheren."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdxxviii{Diese sind diejenigen, gegen sie das Gesagte Rechtens war, unter Umam, die bereits vor ihnen vergingen, von den Dschinn und den Menschen. Gewiß, sie waren Verlierer.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdxxix{Und für beide gibt es Stellungen für das, was sie taten. Und ER wird ihnen ihre Taten doch vergüten, und ihnen wird kein Unrecht angetan.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdxxx{Und an dem Tag, wenn diejenigen, die Kufr betrieben haben, dem Feuer ausgesetzt werden: ``Ihr habt eure Tay-yibat in eurem diesseitigen Leben verbraucht und habt sie genossen, so wird euch heute die Peinigung der Erniedrigung vergolten wegen dem, wie ihr euch auf Erden zu Unrecht in Arroganz zu erheben pflegtet, und wegen dem, wie ihr Fisq zu betreiben pflegtet."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdxxxi{Und erwähne den Bruder von 'Aad, als er seine Leute in Al-ahqaf warnte, und bereits vergingen die Warner vor ihm und hinter ihm: ``Dient nichts anderes als ALLAH! Ja! Ich fürchte um euch die Peinigung eines gewaltigen Tages."\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdxxxii{Sie sagten: ``Kamst du zu uns, um uns von unseren Göttern abzuhalten? Dann bringe uns, was du uns androhst, solltest du von den Wahrhaftigen sein."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdxxxiii{Er sagte: ``Das Wissen ist nur bei ALLAH. Und ich verkünde euch das, womit ich entsandt wurde. Doch ich sehe euch als Leute, die unwissend sind."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdxxxiv{Und als sie es als Regenwolke sahen, die über ihre Täler zieht, sagten sie: ``Dies ist eine Regenwolke, die über uns regnen wird." Nein, sondern sie ist das, womit ihr Eile gefordert habt, ein Wind, mit dem eine qualvolle Peinigung ist,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdxxxv{er vernichtet alles, mit der Anweisung seines HERRN. Dann wurden sie so, daß nichts anderes als ihre Wohnstätten gesehen wird. Und solcherart vergelten WIR den schwer verfehlenden Leuten.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdxxxvi{Und gewiß, bereits festigten WIR sie, worin WIR euch nicht gefestigt haben, und WIR machten ihnen Gehör, Augen und Verstand. Doch weder ihr Gehör, noch ihre Augen, noch ihr Verstand nützten ihnen etwas, da sie ALLAHs Ayat verleugneten. Und sie umgab das, was sie zu verspotten pflegten.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdxxxvii{Und gewiß, bereits richteten WIR das, was um euch herum an Ortschaften ist, zugrunde, und verdeutlichten die Ayat, damit sie umkehren.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdxxxviii{Und hätten ihnen diejenigen, die sie sich anstelle von ALLAH als Näherbringende als Gottheiten nahmen, doch Beistand geleistet! Nein, sondern sie sind ihnen abhanden gekommen. Und dies ist ihre Ifk-Lüge und das, was sie zu erfinden pflegten.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdxxxix{Und (erinnere daran), als WIR dir eine Gruppe von den Dschinn zuwandten, die dem Quran zuhörten. Und als sie bei ihm anwesend wurden, sagten sie: ``Hört zu!" Und dann, als damit aufgehört wurde, kehrten sie zu ihren Leuten als Warnende zurück.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdxl{Sie sagten: ``Unsere Leute! Gewiß, wir hörten eine Schrift, die nach Musa hinabgesandt wurde, bestätigend dessen, was vor ihr war. Sie leitet zur Wahrheit und zu einem geradlinigen Weg recht.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdxli{Unsere Leute! Folgt ALLAHs Da'wa-Machenden und verinnerlicht den Iman an Ihn. ER vergibt euch eure Verfehlungen und schützt euch vor qualvoller Peinigung.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdxlii{Und wer ALLAHs Da'wa-Machenden nicht folgt, der macht (ALLAH) auf Erden nicht zu schaffen, und er hat anstelle von Ihm keine Wali. Diese sind im eindeutigen Irregehen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdxliii{Sahen sie etwa nicht, daß ALLAH, Der die Himmel und die Erde erschuf und wegen ihrer Schöpfung nicht müde wurde, die Allmacht hat, um die Toten zu beleben?! Doch, mit Sicherheit! Gewiß, ER ist über alles allmächtig.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdxliv{Und an dem Tag, wenn diejenigen, die Kufr betrieben haben, dem Feuer ausgesetzt werden: ``Ist dies etwa nicht die Wahrheit?!" Sie sagten: ``Doch, mit Sicherheit, bei unserem HERRN!" ER sagte: ``So erfahrt die Peinigung für das, was ihr an Kufr zu betreiben pflegtet."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdxlv{So übe dich in Geduld, wie Ulul-'azm von den Gesandten sich in Geduld übten, und fordere keine Eile mit ihnen. Für sie ist es an dem Tag, wenn sie das sehen, was ihnen angedroht wird, als ob sie nicht mehr als eine Stunde während des Tages verweilt haben. Dies ist eine Mitteilung. Werden etwa andere zugrunde gerichtet als die fisq-betreibenden Leute?!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdxlvi{\basmalahde Denjenigen, die Kufr betrieben und von ALLAHs Weg abgehalten haben, ließ ER ihre Handlungen verloren gehen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdxlvii{Und denjenigen, die den Iman verinnerlicht, gottgefällig Gutes getan und den Iman an das verinnerlicht haben, das Muhammad hinabgesandt wurde - und es ist die Wahrheit von ihrem HERRN - tilgte ER ihre gottmißfälligen Taten und besserte ihre Lage.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdxlviii{Dies weil diejenigen, die Kufr betrieben haben, dem für nichtig Erklärten folgten, und diejenigen, die den Iman verinnerlichten, demWahren von ihrem HERRN folgten. Solcherart prägt ALLAH den Menschen ihre Gleichnisse.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdxlix{Und wenn ihr auf diejenigen, die Kufr betrieben haben, trefft, gibt es das Schlagen auf die Hälse. Dann wenn ihr ihnen schwere Verluste zugefügt habt, dann zieht die Fesseln fest. Danach ist entweder Begnadigung oder Auslösung, bis der Krieg seine Lasten ablegte. Es ist dies! Und wollte ALLAH, würde ER gewiß an ihnen Vergeltung üben. Doch damit ER die einen von euch durch die anderen prüft. Und diejenigen, die fi-sabilillah getötet wurden, ihre Handlungen läßt ER nie verloren gehen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdl{ER wird sie rechtleiten und ihre Lage bessern.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdli{Und ER wird sie in die Dschanna eintreten lassen, die ER ihnen bekannt machte.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdlii{Ihr, die den Iman verinnerlicht habt! Wenn ihr ALLAHs (Din) beisteht, steht ER euch bei und festigt eure Füße (in der Sache).\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdliii{Und für diejenigen, die Kufr betrieben haben, gibt es Verfall, und ER ließ ihre Handlungen verloren gehen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdliv{Dies, weil sie dem gegenüber abgeneigt waren, was ALLAH hinabsandte, so ließ ER ihre Handlungen zunichte werden.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdlv{Sind sie etwa auf Erden nicht umhergezogen, damit sie sehen, wie das Anschließende derjenigen vor ihnen war?! ALLAH ließ über ihnen Vernichtung fallen. Und für die Kafir gibt es Gleiches.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdlvi{Dies, weil ALLAH gewiß Der Maula derjenigen ist, die den Iman verinnerlichten, und es für die Kafir gewiß keinen Maula gibt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdlvii{Gewiß, ALLAH läßt diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, in Dschannat eintreten, die von Flüssen durchflossen sind. Und diejenigen, die Kufr betrieben haben, vergnügen sich und speisen, wie die An'am fressen. Und das Feuer ist eine Bleibe für sie.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdlviii{Und wie viele Ortschaften, die noch mächtiger waren als deine Ortschaft, die dich vertrieb, richteten WIR doch zugrunde! So gibt es für sie keinen Beistehenden.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdlix{Ist etwa derjenige, der Klarheit von seinem HERRN hat, gleich demjenigen, dem das Schlechte seines Tuns schön gemacht wurde?! Doch sie folgten ihren Neigungen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdlx{Ist etwa) das Gleichnis der Dschanna, die den Muttaqi versprochen wurde - darin sind Flüsse mit nicht verderblichem Wasser, Flüsse mit Milch, deren Geschmack sich nie ändert, Flüsse mit den Trinkenden wohlschmeckendemWein, und Flüsse mit filtriertem Honig. Und für sie gibt es darin von allen Früchten und Vergebung von ihrem HERRN - gleich demjenigen, der ewig im Feuer bleibt?! Und sie wurden mit siedendem Wasser getränkt, so zerriß es ihre Därme.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdlxi{Und unter ihnen sind manche, die dir zuhören, dann nachdem sie dich verlassen hatten, sagten sie zu denjenigen, denen das Wissen zuteil wurde: ``Was sagte er vorhin?" Diese sind diejenigen, deren Herzen ALLAH versiegelte. Und sie folgten ihren Neigungen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdlxii{Und diejenigen, die Rechtleitung fanden, mehrte ER an Rechtleitung und gewährte ihnen ihre Taqwa.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdlxiii{Warten sie etwa auf anderes als auf die Stunde, daß sie plötzlich zu ihnen kommt?! Also bereits kamen ihre Vorzeichen. Und wie werden sie sich erinnern, wenn sie (die Stunde so) zu ihnen kommt?!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdlxiv{So stelle es durch Wissen fest, daß es gewiß keine Gottheit außer ALLAH gibt, und bitte um Vergebung für deine Verfehlung und für die Mumin-Männer und Mumin-Frauen! Und ALLAH kennt euer Herumreisen und eure Unterkunft.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdlxv{Und diejenigen, die den Iman verinnerlichten, sagen: ``Würde doch eine Sura hinabgesandt!" Und wenn eine eindeutige Sura hinabgesandt wurde und darin der Kampf erwähnt wurde, sahst du diejenigen, in deren Herzen Krankheit ist, dich anschauen wie das Starren desjenigen, der während des Sterbens bewußtlos wird. Also Niedergang nahe sich ihnen!\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdlxvi{Gehorsam und gebilligtes Wort (ist besser für sie). Und wenn die Angelegenheit ernst wird, würden sie ALLAH gegenüber wahrhaftig werden, so wäre dies besser für sie.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdlxvii{Würdet ihr etwa, wenn ihr Verantwortung übernehmt, Verderben auf Erden anrichten und eure Verwandtschaftsbande brechen?!\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdlxviii{Diese sind diejenigen, die ALLAH verfluchte, so machte ER sie taub und blind machte ER ihre Augen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdlxix{Denken sie nicht über den Quran nach?! Oder gibt es Schlösser auf (manchen) Herzen?!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdlxx{Gewiß, denjenigen, die den Rücken kehrten, nachdem ihnen die Rechtleitung sichtbar geworden ist, der Satan macht es ihnen leicht und machte ihnen Hoffnungen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdlxxi{Dies, weil sie gewiß zu denjenigen, die dem gegenüber, was ALLAH nach und nach hinabsandte, abgeneigt waren, sagten: ``Wir werden euch in manchem von der Angelegenheit gehorchen." Und ALLAH kennt ihre Geheimnisse.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdlxxii{Und wie (sind sie denn), wenn die Engel ihre (Seelen) einzogen, sie schlagen auf ihre Gesichter und ihre Rücken?!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdlxxiii{Dies, weil sie dem folgten, was den Zorn ALLAHs provozierte. Und sie waren Seinem Wohlgefallen gegenüber abgeneigt, so ließ ER ihre Handlungen verfallen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdlxxiv{Oder dachten diejenigen, in deren Herzen Krankheit ist, daß ALLAH ihren Groll nie hervorbringen wird?!\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdlxxv{Und wenn WIR es wollten, gewiß hätten WIR dir sie gezeigt, dann wirst du sie sicher an ihren Merkmalen erkennen. Und du erkennst sie doch an der Andeutung mit dem Gesagten. Und ALLAH kennt eure Handlungen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdlxxvi{Und WIR werden euch doch prüfen, damit WIR die Dschihad-Leistenden unter euch und die sich in Geduld Übenden kenntlich machen, und damit WIR eure Angelegenheiten prüfen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdlxxvii{Gewiß, diejenigen, die Kufr betrieben, von ALLAHs Weg abgehalten und sich dem Gesandten widersetzt haben, nachdem ihnen die Rechtleitung sichtbar wurde, schaden ALLAH in Nichts. Und ER wird ihre Handlungen zunichte werden lassen.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdlxxviii{Ihr, die den Iman verinnerlicht habt! Gehorcht ALLAH und gehorcht dem Gesandten und laßt eure Handlungen nicht nichtig werden!\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdlxxix{Gewiß, denjenigen, die Kufr betrieben und von ALLAHs Weg abgehalten haben, dann starben, während sie Kafir waren, wird ALLAH ihnen nie vergeben.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdlxxx{So gebt nicht nach und ruft nicht zur Versöhnung! Und ihr seid die Überlegenen! Und ALLAH ist mit euch. Und ER wird euch eure Handlungen gewiß nie abmindern.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxi{Das diesseitige Leben ist doch nur Unfug-Treiben und Vergnügen! Und wenn ihr den Iman und Taqwa verinnerlicht, läßt er euch eure Belohnung zuteil werden und verlangt von euch nicht euer Vermögen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxii{Und wenn ER danach verlangt, und dann mit euch bis an die Grenze geht, werdet ihr geizen, und ER wird euren Groll hervorbringen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxiii{Da werdet ihr aufgerufen, um fi-sabilillah zu spenden. Und unter euch gibt es welche, die geizen werden. Doch wer geizt, der geizt nur sich selbst gegenüber. Und ALLAH ist Der absolut Autarke und ihr seid die Armen. Und wenn ihr den Rücken kehrt, tauscht ER euch gegen andere Leute aus, dann werden sie nicht euresgleichen sein.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxiv{\basmalahde Gewiß, WIR ließen dich einen klaren Sieg erringen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxv{damit ALLAH dir das vergibt, was von deinem Verfehlen sich ereignete und noch ereignen wird, dir Seine Wohltat vollendet, dich auf einen geradlinigen Weg rechtleitet\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxvi{und ALLAH dir mit einem würdigen Sieg beisteht.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxvii{ER ist Derjenige, Der Herzensruhe in die Herzen der Mumin hinabsandte, damit sie mehr Iman zu ihrem Iman gewinnen. Und ALLAH gehören die Gehilfen der Himmel und der Erde. Und ALLAH ist immer allwissend, allweise.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxviii{ER wird doch die Mumin-Männer und die Mumin-Frauen in Dschannat eintreten lassen, die von Flüssen durchflossen sind, als Ewige darin, und ihnen ihre gottmißfälligen Taten tilgen. Und dies ist bei ALLAH stets ein gewaltiger Erfolg.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdlxxxix{Und ER wird die Munafiq-Männer und die Munafiq-Frauen und die Muschrik- Männer und die Muschrik-Frauen, die über ALLAH das schlechte Denken denken. Vernichtendes treffe sie! Und ALLAH zürnte ihnen, verfluchte sie und bereitete für sie Dschahannam. Und erbärmlich ist sie als Werden.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdxc{Und ALLAH gehören die Gehilfen der Himmel und der Erde. Und ALLAH ist immer allwürdig, allweise.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdxci{Gewiß, WIR entsandten dich als Zeugen, Überbringer froher Botschaft und Warner,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdxcii{damit ihr den Iman an ALLAH und Seinen Gesandten verinnerlicht, Seinem (Din) beisteht, Ihn verherrlicht und Ihn lobpreist nach Sonnenaufgang und vor Sonnenuntergang.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdxciii{Gewiß, diejenigen, die dir gegenüber Bai'a leisten, leisten eigentlich Bai'a ALLAH gegenüber. ALLAH bekräftigt ihren Handschlag. Wer (ihn) bricht, der bricht (ihn) nur gegen sich selbst, und wer das erfüllt, wozu er sich ALLAH gegenüber verpflichtet hat, dem wird ER eine riesengroße Belohnung gewähren.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdxciv{Die Zurückgebliebenen von den Wüstenarabern werden dir sagen: ``Uns beschäftigten unser Vermögen und unsere Familien, so bitte für uns um Vergebung!" Sie sagen mit ihren Zungen, was nicht in ihren Herzen ist. Sag: ``Wer verfügt für euch über etwas von ALLAH, wenn ER für euch Schädigendes will, oder für euch Nützliches will?" Nein, sondern ALLAH ist dessen, was ihr tut, allkundig.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdxcv{Nein, sondern ihr dachtet, daß der Gesandte und die Mumin niemals zu ihren Familien zurückkehren werden, und dies wurde in euren Herzen schön gemacht, ihr habt das schlechte Denken gedacht, und ihr wart verlorene Leute.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdxcvi{Und wer den Iman an ALLAH und Seinen Gesandten nicht verinnerlicht, so bereiteten WIR doch für die Kafir Gluthitze.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdxcvii{Und ALLAH gehört die Herrschaft der Himmel und der Erde. ER vergibt, wem ER will, und ER peinigt, wen ER will. Und ALLAH ist immer allvergebend, allgnädig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdxcviii{Die Zurückgebliebenen werden sagen: ``Wenn ihr zu den Beutegütern aufbrecht, um sie einzunehmen, dann laßt uns euch folgen!" Sie wollen ALLAHs Worte ändern. Sag: ``Ihr folgt uns nicht! Solcherart sagte ALLAH vorher." Dann werden sie sagen: ``Nein, sondern ihr beneidet uns." Nein, sondern sie pflegten nicht zu verstehen außer ein wenig.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdxcix{Sage zu den Zurückgebliebenen von den Wüstenarabern: ``Ihr werdet zu Leuten von heftiger Gewalttätigkeit gerufen, ihr kämpft gegen sie oder sie werden Muslime. Und wenn ihr gehorcht, gewährt euch ALLAH eine schöne Belohnung. Und wenn ihr den Rücken kehrt, wie ihr vorher den Rücken gekehrt habt, unterzieht ER euch einer qualvollen Peinigung."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdc{Weder für den Blinden ist es eine Verfehlung, noch für den Lahmen ist es eine Verfehlung, noch für den Kranken ist es eine Verfehlung. Und wer ALLAH und Seinem Gesandten gehorcht, den wird ER in Dschannat eintreten lassen, die von Flüssen durchflossen sind. Und wer den Rücken kehrt, den peinigt ER eine qualvolle Peinigung.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdci{Gewiß, bereits ist ALLAH zufrieden mit den Mumin, als sie dir Bai'a unter dem Baum leisteten, dann wußte ER, was in ihren Herzen ist, dann sandte ER die Herzensruhe über sie und belohnte sie mit einem nahen Sieg\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcii{und vielen Beutegütern, die sie einnehmen werden. Und ALLAH ist immer allwürdig, allweise.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdciii{ALLAH versprach euch viele Beutegüter, die ihr einnehmt, so beschleunigte ER euch diese und hielt die Hände der Menschen von euch zurück, und damit dies eine Aya für die Mumin wird. Und ER euch zu einem geraden Weg rechtleitet.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdciv{Auch andere (versprach ER euch), über die ihr keine Macht hattet, die ALLAH bereits allumfassend kennt. Und ALLAH ist immer über alles allmächtig.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcv{Und würden diejenigen, die Kufr betrieben haben, gegen euch kämpfen, würden sie gewiß die Rücken flüchtend kehren, dann finden sie weder Wali, noch Beistehenden.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcvi{Dies ist ALLAHs Handlungsweise, die bereits angewandt wurde. Und du wirst von ALLAHs Handlungsweise keine Abweichung finden.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcvii{Und ER ist Derjenige, Der ihre Hände von euch zurückhielt sowie eure Hände von ihnen inmitten des Makka-Tales, nachdem ER euch über sie hatte siegen lassen. Und ALLAH ist immer über das, was ihr tut, allsehend.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcviii{Sie sind diejenigen, die Kufr betrieben und euch von Almasdschidil-haram abgehalten haben und die Opfertiere gehindert, ihren Bestimmungsort zu erreichen. Und gäbe es nicht Mumin-Männer und Mumin-Frauen - welche ihr nicht kennt, die ihr niedertreten würdet ohne Wissen und dann euch wegen ihnen Beschämendes trifft, (hielte ER eure Hände von ihnen nicht zurück). Damit ALLAH in Seine Gnade eintreten läßt, wen ER will. Und hätten sie sich auseinandergetrennt, hätten WIR gewiß diejenigen von ihnen, die Kufr betrieben haben, einer qualvollen Peinigung unterzogen.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcix{Und erinnere daran), als diejenigen, die Kufr betrieben haben, in ihren Herzen den ablehnenden Stolz hegten, den ablehnenden Stolz der Dschahiliya, dann sandte ALLAH Herzensruhe von Ihm auf Seinen Gesandten und auf die Mumin hinab und verpflichtete sie zum Wort der Taqwa, und sie hatten mehr Anrecht darauf und waren dessen Leute. Und ALLAH ist immer über alles allwissend.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdcx{Gewiß bewahrheitete ALLAH Seinem Gesandten das Traumgesicht wahrheitsgemäß. Ihr werdet doch, inscha-allah, Almasdschidil-haram betreten, sicher mit rasierten Häuptern oder mit kurzgeschnittenen (Haaren). Nicht werdet ihr euch fürchten. So wußte ER, was ihr nicht wisst, dann machte ER nach diesem einen nahen Sieg.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcxi{ER ist Derjenige, Der Seinen Gesandten mit der Rechtleitung und dem Din der Wahrheit entsandte, damit ER ihn sämtliche Din übertreffen läßt. Und ALLAH genügt als Zeuge.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcxii{Muhammad ist ALLAHs Gesandter! Und diejenigen mit ihm sind hart gegen die Kafir, barmherzig untereinander. Du siehst sie als Ruku'- und Sudschud- Vollziehende, sie streben Gunst von ALLAH und Wohlgefallen an. Ihre Merkmale sind in ihren Gesichtern von der Sudschud-Wirkung. Dies ist ihr Gleichnis in At-taurat, sowie ihr Gleichnis im Alindschil. Es ist wie ein Gewächs, das seine Schößlinge trieb, dann stärkte es sie, dann wurden sie fest, dann standen sie auf ihren Stengeln gerade, es erfreut die Säer, damit ER mit ihnen die Kafir aufregt. ALLAH versprach denjenigen von ihnen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, Vergebung und riesengroße Belohnung.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdcxiii{\basmalahde Ihr, die den Iman verinnerlicht habt! Greift ALLAH und Seinem Gesandten nicht vor, und handelt Taqwa gemäß ALLAH gegenüber! Gewiß, ALLAH ist allhörend, allwissend.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcxiv{Ihr, die den Iman verinnerlicht habt! Erhebt nicht eure Stimmen über die Stimme des Propheten, und seid ihm gegenüber nicht so laut mit dem Gesagten, wie die einen von euch den anderen gegenüber laut sind, sonst werden eure Handlungen zunichte, während ihr es nicht merkt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcxv{Gewiß, diejenigen, die ihre Stimmen vor ALLAHs Gesandten dämpfen, sind solche, deren Herzen ALLAH für die Taqwa prüfte. Für sie ist Vergebung und riesengroße Belohnung bestimmt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcxvi{Gewiß, diejenigen, die dich von jenseits der Hudschurat rufen, die meisten von ihnen begreifen nicht.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcxvii{Und hätten sie sich doch in Geduld geübt, bis du zu ihnen herauskommst, wäre dies sicher besser für sie. Und ALLAH ist allvergebend, allgnädig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdcxviii{Ihr, die den Iman verinnerlicht habt! Wenn zu euch ein Fasiq mit einer Nachricht kommt, so vergewissert euch, damit ihr nicht Leute durch Unwissen verletzt, und dann ihr reuig werdet für das, was ihr getan habt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdcxix{Und wisst, daß unter euch ALLAHs Gesandter ist. Würde er auf euch in vielen der Angelegenheiten hören, würdet ihr sicherlich unter Unannehmlichkeiten leiden. Doch ALLAH hat euch den Iman lieben lassen und ließ ihn in euren Herzen schön erscheinen, und hat euch das Kufr, das Fisq und die Widersetztheit verabscheuen lassen. Diese sind die wahren Vernünftigen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdcxx{Es ist eine Gunst von ALLAH und Wohltat. Und ALLAH ist allwissend, allweise.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdcxxi{Und sollten zwei Gruppen von den Mumin sich bekämpfen, dann versöhnt sie miteinander! Und sollte dann eine von ihnen Übertretung gegen die andere begehen, dann bekämpft diejenige, die übertritt, bis sie zu ALLAHs Anweisung zurückkehrt. Und wenn sie dann zurückkehrte, dann versöhnt beide miteinander nach Gerechtigkeit, und seid gerecht! Gewiß, ALLAH liebt die Gerechten.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdcxxii{Die Mumin sind doch Geschwister, so versöhnt eure Geschwister miteinander! Und handelt Taqwa gemäß ALLAH gegenüber, damit euch Gnade erwiesen wird.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdcxxiii{Ihr, die den Iman verinnerlicht habt! Es sollen weder Männer sich lustig über (andere) Männer machen - vielleicht sind sie besser als sie, noch Frauen über (andere) Frauen - vielleicht sind sie besser als sie. Und macht euch nicht gegenseitig schlecht und gebt euch keine Schimpfnamen! Erbärmlich ist der Fisq-Name nach dem Iman. Und wer nicht bereut, so sind diese die Unrecht-Begehenden.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdcxxiv{Ihr, die den Iman verinnerlicht habt! Meidet das Meiste vom Spekulieren! Gewiß, manches Spekulieren ist eine Verfehlung. Und spioniert nicht, und die einen von euch sollen keine Ghibah gegen die anderen begehen! Mag der eine von euch etwa vom Fleisch seines toten Bruders speisen? So habt ihr es verabscheut. Und handelt Taqwa gemäß ALLAH gegenüber! Gewiß, ALLAH ist reue-annehmend, allgnädig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdcxxv{Ihr Menschen! Gewiß, WIR erschufen euch aus einem Männlichen und einem Weiblichen und machten euch zu Völkern und Stämmen, damit ihr euch kennenlernt. Gewiß, der Würdigste von euch bei ALLAH ist derjenige mit am meisten Taqwa. Gewiß, ALLAH ist allwissend, allkundig.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdcxxvi{Die Wüstenaraber sagten: ``Wir verinnerlichten den Iman." Sag: ``Ihr habt den Iman nicht verinnerlicht, sondern sagt: ``Wir wurden Muslime." Doch der Iman ist noch nicht in eure Herzen eingedrungen. Und wenn ihr ALLAH und Seinem Gesandten gehorcht, unterschlägt ER euch von euren Handlungen nichts." Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdcxxvii{Die Mumin sind nur diejenigen, die den Iman an ALLAH und Seinen Gesandten verinnerlichten, dann keinen Zweifel hegten, und Dschihad mit ihrem Vermögen und sich selbst fi-sabilillah leisteten. Diese sind die Wahrhaftigen.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdcxxviii{Sag: ``Wollt ihr ALLAH über euren Din belehren? Und ALLAH weiß, was in den Himmeln und was auf Erden ist." Und ALLAH ist über alles allwissend.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcxxix{Sie machen dir Vorhaltungen, daß sie den Islam annahmen. Sag: ``Macht mir keine Vorhaltungen wegen eures Islam! Nein, sondern ALLAH erwies euch Gnade, daß er euch zum Iman rechtleitete, solltet ihr wahrhaftig sein."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcxxx{Gewiß, ALLAH kennt das Verborgene der Himmel und der Erde. Und ALLAH ist dessen allsehend, was ihr tut.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxi{\basmalahde Qaf. Beim herrlichen Quran!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxii{Nein, sondern sie staunten, daß zu ihnen ein Warner von ihnen kam, so sagten die Kafir: ``Dies ist etwas Erstaunliches!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxiii{Wenn wir etwa starben und zu Erde wurden, (werden wir wiederkehren)? Dies ist eine weit entfernte Wiederkehr."\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxiv{Bereits wußten WIR, was die Erde von ihnen wegnimmt. Und bei Uns ist eine bewahrende Schrift.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxv{Nein, sondern sie verleugneten das Wahre, als es zu ihnen kam, so sind sie in einer verworrenen Situation.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxvi{Haben sie etwa nicht auf den Himmel über ihnen geschaut, wie WIR ihn errichteten und schmückten, und er keine Risse hat?!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxvii{Und die Erde dehnten WIR aus, setzten in sie Festigende (Berge) und ließen auf ihr von jeder erfreulichen Zweiheit hervorsprießen\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxviii{als Einblick-Gewährendes und Ermahnung für jeden umkehrenden Diener.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdcxxxix{Und WIR ließen vom Himmel Wasser voller Baraka nach und nach fallen, dann ließen WIR damit Dschannat und erntbare Körner hervorsprießen\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdcxl{sowie hohe Dattelpalmen, die büschelartige Früchte tragen,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdcxli{als Rizq für die Diener. Und damit belebten WIR eine tote Ortschaft. Solcherart ist das Hervorkommen (aus den Gräbern).\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdcxlii{Verleugnet vor ihnen haben die Leute von Nuh, die Bewohner von Ar-rass und Thamud\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdcxliii{sowie 'Aad, Pharao und die Brüder von Lut\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdcxliv{sowie die Bewohner von Al-aika und die Leute von Tubba'. Alle bezichtigten die Gesandten der Lüge, so war Meine Androhung Rechtens.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdcxlv{Waren WIR etwa überfordert mit der ersten Schöpfung?! Nein, sondern sie sind in Verwirrung vor einer neuen Schöpfung.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdcxlvi{Und gewiß, bereits erschufen WIR doch den Menschen und WIR wissen, was ihm seine Seele flüstert. Und WIR sind ihm näher als seine Halsschlagader.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcxlvii{Wenn (ihn) die beiden Empfänger vom Rechten und vom Linken sitzend empfangen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcxlviii{Er spricht kein Wort aus, ohne daß bei ihm ein anwesender Beaufsichtigender ist.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcxlix{Und das Benommen-Machende des Sterbens kam wahrheitsgemäß. Dies ist es, wogegen du abgeneigt warst.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdcl{Und es wurde in As-sur gestoßen. Dies ist der Tag der Androhung.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcli{Und jede Seele kam, mit ihr ein Führer und ein Zeuge.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdclii{Gewiß, bereits warst du doch demgegenüber achtlos, dann nahmen wir von dir deine Bedeckung weg, dann ist dein Sehen heute scharf.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcliii{Und sein (enger) Begleiter sagte: ``Das ist was bei mir vorhanden ist."\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcliv{``Werft ihr beide in Dschahannam jeden äußerst sturen Kufr-Betreibenden,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdclv{der äußerst das Gute verhindernd, übertretend und zweifel-hegend war,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdclvi{derjenige, der neben ALLAH eine andere Gottheit beigesellte, so werft beide ihn in die harte Peinigung.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdclvii{Sein (enger) Begleiter sagte: ``Unser HERR! Ich ließ ihn nicht übertreten, sondern er war im weiten Irregehen."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdclviii{ER sagte: ``Streitet nicht vor Mir, wo ICH euch bereits die Androhung vorausschickte.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdclix{Das Gesagte wird bei Mir nie geändert. Und ICH bin kein Unrecht-Zufügender den Dienern gegenüber."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdclx{An dem Tag, wenn WIR zu Dschahannam sagen: ``Bist du voll geworden?!" Und sie sagt: ``Gibt es etwa nicht noch mehr?"\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdclxi{Und die Dschanna wurde den Muttaqi näher gebracht, ohne weit zu sein.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdclxii{Dies ist, was euch versprochen wurde, für jeden bewahrenden Umkehrenden,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdclxiii{der Ehrfurcht Dem Allgnade Erweisenden gegenüber hatte und mit einem bereuenden Herzen kam.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdclxiv{``Betretet sie mit Salam. Dies ist der Tag der Ewigkeit."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdclxv{Für sie ist darin bestimmt, was sie wollen. Und bei Uns ist noch mehr.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdclxvi{Und wie viele WIR vor ihnen an Generationen zugrunde richteten, die noch heftiger in der Gewalttätigkeit als sie waren, dann suchten sie in den Ländern, ob es ein Entrinnen gibt.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdclxvii{Gewiß, darin ist doch eine Ermahnung für denjenigen, für den es einen Verstand gab oder der zuhörte, während er Zeuge war.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdclxviii{Und gewiß, bereits erschufen WIR die Himmel, die Erde und das, was zwischen ihnen ist, in sechs Ay-yam, und Uns überkam keinerlei Müdigkeit.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdclxix{So übe dich in Geduld dem gegenüber, was sie sagen, und lobpreise mit dem Lob deines HERRN vor dem Sonnenaufgang und vor dem Sonnenuntergang!\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdclxx{Und in einem Teil der Nacht lobpreise Ihn und im Anschluß an Sudschud!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdclxxi{Und höre zu an dem Tag, wenn der Rufende aus einem nahen Ort ruft.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdclxxii{An dem Tag, wenn sie den Schrei wahrheitsgemäß vernehmen, dies ist der Tag des Hervorkommens.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdclxxiii{Gewiß, WIR sind es, Der belebt und sterben läßt, und zu Uns ist das Werden,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdclxxiv{an dem Tag, wenn die Erde über ihnen aufbricht, (während sie) eilend (hervorkommen). Dies ist für Uns ein leichtes Versammeln.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdclxxv{Gewiß, WIR wissen besser Bescheid über das, was sie sagen. Und du bist über sie kein Erzwingender. So ermahne mit dem Quran diejenigen, die sich vor Meiner Androhung fürchten.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdclxxvi{\basmalahde Bei den das Streuen Streuenden,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdclxxvii{dann den Schweres Tragenden,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdclxxviii{dann den in Leichtigkeit sich Fortbewegenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdclxxix{dann den die Angelegenheit Einteilenden!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdclxxx{Was euch versprochen wird, ist doch wahrhaftig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxi{Und gewiß, der Din wird sicher geschehen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxii{Bei dem Himmel von Kreisbahnen!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxiii{Gewiß, ihr seid doch verschiedener Ansichten,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxiv{abgebracht wird davon, wer abgebracht wurde.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxv{Tod sei den Lügnern,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxvi{die in den Tiefen (der Unwissenheit) achtlos bleiben.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxvii{Sie fragen: ``Wann geschieht der Tag des Din?"\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxviii{``Am Tag, wenn sie im Feuer der Fitna unterzogen werden."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdclxxxix{Erfahrt eure Fitna! Dies ist das, wozu ihr zur Eile aufzufordern pflegtet.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdcxc{Gewiß, dieMuttaqi sind in Dschannat und an Quellen,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdcxci{sie nehmen das, was ihr HERR ihnen zuteil werden ließ. Gewiß, sie pflegten vor diesem Muhsin zu sein.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcxcii{Sie pflegten ein wenig von der Nacht zu schlafen,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcxciii{und vor der Morgendämmerung baten sie um Vergebung.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdcxciv{Und in ihren Vermögensgütern ist ein Anrecht für den Bittenden und den Ausgeschlossenen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdcxcv{Und auf Erden sind Ayat für die Gewißheit-Anstrebenden\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcxcvi{sowie in euch selbst. Habt ihr etwa keinen Einblick?!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcxcvii{Und im Himmel ist euer Rizq und das, was euch versprochen wird.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcxcviii{Beim HERRN des Himmels und der Erde! Gewiß, es ist bestimmt wahr, genauso (wie es wahr ist), daß ihr sprecht.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcxcix{Wurde dir der Bericht über Ibrahims gewürdigte Gäste zuteil?!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcc{Als sie zu ihm eintraten und sagten: ``Salam (sei mit dir)!" Er sagte: ``Salam (sei mit euch), ihr unbekannte Leute!"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcci{Dann ging er unbemerkt zu seiner Familie, dann kam er mit einem wohlgenährten Kalb,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdccii{dann setzte er es ihnen vor, er sagte: ``Esst ihr etwa nicht?"\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcciii{Dann verbarg er Furcht vor ihnen. Sie sagten: ``Fürchte dich nicht!" Und sie überbrachten ihm frohe Botschaft über einen äußerst wissenden Jungen.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcciv{Dann kam seine Frau in lautem Überschwang, schlug sich auf ihr Gesicht und sagte: ``Ich bin doch eine unfruchtbare Alte!"\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdccv{Sie sagten: ``Solcherart sagte dein HERR. Gewiß, ER ist Der Allweise, Der Allwissende."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdccvi{Er sagte: ``Was ist euer Bestreben, ihr Entsandte?"\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdccvii{Sie sagten: ``Gewiß, wir wurden zu schwer verfehlenden Leuten entsandt,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdccviii{damit wir über sie Steine von Lehm schicken,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdccix{die bei deinem HERRN für die Maßlosen gekennzeichnet sind."\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdccx{Dann brachten WIR heraus, wer in ihr von den Mumin war.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdccxi{Dann fanden WIR in ihr nur ein Haus von Muslimen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdccxii{Und WIR ließen in ihr eine Aya für diejenigen, die sich vor der qualvollen Peinigung fürchten.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdccxiii{Und mit Musa (gab es eine Aya), als WIR ihn zu Pharao mit einem deutlichen Beweis entsandten.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdccxiv{Dann wandte er sich zu seinen Protektoren und sagte: ``(Er ist) ein Magier oder geistesgestört."\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdccxv{Dann bestraften WIR ihn mit seinen Soldaten, dann warfen WIR sie in den Fluß, während er tadelnswert war.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdccxvi{Und mit 'Aad (gab es eine Aya), als WIR über sie den unfruchtbar machenden Wind schickten,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdccxvii{der nichts von dem läßt, worüber er wehte, ohne daß er es zu Zermalmtem machte.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdccxviii{Und mit Thamud (gab es eine Aya), als ihnen gesagt wurde: ``Vergnügt euch nur bis zu einer Zeit!"\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdccxix{Dann erhoben sie sich über die Anweisung ihres HERRN, dann erschlug sie der Blitzschlag während sie zuschauten.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdccxx{Dann konnten sie weder aufstehen, noch wurde ihnen beigestanden.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdccxxi{Ebenso die Leute von Nuh vorher, gewiß, sie waren fisq-betreibende Leute.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdccxxii{Und den Himmel errichteten WIR mit Kraft, und gewiß, WIR sind doch Ausdehnende.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdccxxiii{Und die Erde breiteten wir aus. Wie schön sind die Ebnenden.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdccxxiv{Und von allen Dingen erschufen WIR Zweiheiten, damit ihr euch besinnt.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdccxxv{``Also entweicht zu ALLAH! Gewiß, ich bin für euch vor Ihm ein deutlicher Warner.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmdccxxvi{Und setzt neben ALLAH keinen anderen Gott ein! Gewiß, ich bin für euch vor Ihm ein deutlicher Warner."\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmdccxxvii{Solcherart, zu denjenigen vor ihnen kam kein Gesandter, ohne daß sie sagten: ``(Er ist) ein Magier oder geistesgestört."\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmdccxxviii{Vermachten sie es einander etwa?! Nein, sondern sie sind übertretende Leute.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmdccxxix{So wende dich von ihnen ab, denn du bist nicht tadelnswert.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmdccxxx{Und erinnere, denn die Erinnerung nutzt doch den Mumin.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxi{Und ICH erschuf die Dschinn und die Menschen nur, um Mir zu dienen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxii{ICH will von ihnen keinerlei Rizq, und ICH will nicht, daß sie Mich speisen.\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxiii{Gewiß, ALLAH ist Der wahre reichlich Rizq-Gewährende, Der mit der Allkraft, Der absolut Solide.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxiv{Denn gewiß, für diejenigen, die Unrecht begingen, ist ein Anteil, wie der Anteil ihrer Weggenossen, so sollen sie nicht zur Eile auffordern!\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxv{Also Niedergang sei denjenigen, die Kufr betrieben haben, an ihrem Tag, der ihnen angedroht wird.\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxvi{\basmalahde Bei At-tur!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxvii{Bei der in Zeilen geschriebenen Schrift,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxviii{auf einem entfalteten Pergament.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdccxxxix{Bei Al-bait-ilma'mur!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdccxl{Bei dem hochgehobenen Dach!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdccxli{Bei dem Masdschur-Meer!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdccxlii{Gewiß, die Peinigung deines HERRN wird bestimmt geschehen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdccxliii{Für sie gibt es keinen Abwehrenden.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdccxliv{Am Tag, wenn der Himmel in Hin-und-Her-Schwankung hin und her schwankt,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdccxlv{und die Berge in Fortbewegung sich fortbewegten.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdccxlvi{Also Niedergang an diesem Tag für die Verleugnenden,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdccxlvii{die im Unfug- Treiben herumspielen,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdccxlviii{am Tag, wenn sie im Stoßen zum Feuer von Dschahannam gestoßen werden.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdccxlix{``Dies ist das Feuer, das ihr abzuleugnen pflegtet.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdccl{Ist dies etwa Magie, oder seht ihr etwa nicht?!\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdccli{Tretet in es hinein, dann übt euch in Geduld oder übt euch nicht in Geduld, es ist für euch gleich. Euch wird nur vergolten, was ihr zu tun pflegtet."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdcclii{Gewiß, die Muttaqi sind in Dschannat und Wohlergehen,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdccliii{freudenerregt über das, was ihr HERR ihnen gewährte. Und ihr HERR schützte sie vor der Peinigung der Hölle.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdccliv{``Esst und trinkt wohlbekömmlich für das, was ihr zu tun pflegtet."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdcclv{Sie sind angelehnt auf aufgereihten Liegen. Und WIR ließen sie mit Hur-'in vermählen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcclvi{Und denjenigen, die den Iman verinnerlichten, und deren Nachkommenschaft ihnen mit Iman folgte, schickten WIR ihre Nachkommen nach, und WIR minderten ihnen von ihrem Tun nichts. Jeder Mensch ist dessen, was er erwarb, haftbar.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcclvii{Und WIR versorgten sie mit Obst und Fleisch von dem, was sie begehren.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcclviii{Sie reichen darin einander Becher, mit dem weder sinnloses Gerede noch Verfehlendes ist.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcclix{Und ihnen servieren ihre Dienstjungen, die wie in Muscheln aufbewahrte Perlen wären.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcclx{Und die einen von ihnen wandten sich den anderen zu und fragten,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcclxi{sie sagten: ``Gewiß, wir pflegten vorher bei unseren Familien ehrfurcht-erfüllt zu sein,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdcclxii{dann erwies ALLAH uns Gnade und schützte uns vor der Peinigung der Gluthitze.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcclxiii{Gewiß, wir pflegten vorher an Ihn Bittgebete zu richten. Gewiß, ER ist Der Gütigste, Der Allgnädige.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcclxiv{So erinnere! Denn du bist wegen der Wohltat deines HERRN weder ein Wahrsager noch ein Geistesgestörter.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdcclxv{Oder sagen sie etwa: ``Er ist ein Dichter, bei dem wir ein zweifelhaftes Ende abwarten."\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdcclxvi{Sag: ``Wartet nur ab! Denn ich bin mit euch von den Abwartenden."\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdcclxvii{Oder gebietet ihnen ihre Nachsicht dieses?! Oder sind sie etwa übertretende Leute?!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdcclxviii{Oder sagen sie etwa: ``Er hat ihn erfunden!" Nein, sondern sie verinnerlichen den Iman nicht.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdcclxix{So sollen sie einen Bericht Seinesgleichen bringen, sollten sie wahrhaftig sein.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdcclxx{Oder wurden sie etwa durch Nichts erschaffen?! Oder waren sie selbst etwa die Schöpfer?!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxi{Oder erschufen sie etwa die Himmel und die Erde?! Nein, sondern sie verfügen über keine Gewißheit.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxii{Oder sind bei ihnen etwa die Magazine deines HERRN?! Oder sind sie etwa die Kontrollierenden?!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxiii{Oder gibt es für sie etwa eine Leiter, über die sie zuhören?! So soll ihr Zuhörer mit einem deutlichen Beweis kommen.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxiv{Oder gehören Ihm etwa die Töchter und euch die Söhne?!\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxv{Oder verlangst du etwa von ihnen Lohn, so sie durch ihre Verpflichtung belastet sind?!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxvi{Oder ist bei ihnen etwa das Verborgene, so sie es niederschreiben?!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxvii{Oder wollen sie etwa List planen?! So sind diejenigen, die Kufr betrieben haben, diejenigen, gegen die List geplant wird.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxviii{Oder gibt es für sie etwa eine Gottheit außer ALLAH?! Subhanallah über das, was sie an Schirk betreiben.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxix{Und wenn sie Bruchstücke vom Himmel fallend sehen, sagen sie: ``Dies sind übereinandergeschichtete Wolken."\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxx{Dann laß sie, bis sie ihrem Tag begegnen, an dem sie erschlagen werden.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxi{An dem Tag, wenn ihnen weder ihre List nutzt, noch ihnen beigestanden wird.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxii{Und gewiß, für diejenigen, die Unrecht begingen, gibt es eine Peinigung nach diesem. Doch die Meisten wissen es nicht.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxiii{Und übe Geduld, der Anweisung deines HERRN gegenüber! Denn du bist sicherlich unter Unserer Aufsicht. Und lobpreise mit dem Lob deines HERRN, wenn du aufstehst,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxiv{und an einem Teil der Nacht lobpreise Ihn, und nach dem Erblassen der Sterne.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxv{\basmalahde Bei dem Stern, wenn er abstürzt!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxvi{Euer Weggenosse ist weder irregegangen, noch folgte er einer Verführung.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxvii{Und er spricht nicht aus eigenem Gutdünken.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxviii{Es ist nur ein Wahy, das als Wahy zuteil wird.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcclxxxix{Ihn lehrte einer von enormen Kräften,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdccxc{von Intelligenz, dann stand er gerade,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdccxci{während er am höchsten Horizont war,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdccxcii{dann näherte er sich, dann stieg er hinab,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdccxciii{so war er (so nahe) wie der Abstand beider Bogenenden oder noch näher,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdccxciv{dann ließ er Seinem Diener als Wahy das zuteil werden, was er als Wahy zuteil werden ließ.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdccxcv{Der Verstand erlog nicht, was er sah.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdccxcvi{Zweifelt ihr etwa an ihm das an, was er sieht?!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdccxcvii{Und gewiß, bereits sah er ihn gewiß schon einmal\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdccxcviii{bei der äußersten Sidra,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdccxcix{bei dieser ist die Dschanna der Unterkunft,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdccc{da zur Sidra die kommen, die kommen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdccci{Weder verfehlte der Blick, noch überschritt er.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdcccii{Gewiß, bereits sah er von den großen Ayat seines HERRN!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdccciii{Wie seht ihr es mit Al-lat und Al-'uz-za\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdccciv{sowie die andere Dritte Manat?!"\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdcccv{Ist euch etwa das Männliche und Sein ist das Weibliche?!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdcccvi{Dies ist sicher eine ungerechte Teilung.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdcccvii{Es sind nur Namen, die ihr und eure Ahnen gegeben habt, für die ALLAH keinen Beweis hinabsandte. Sie folgen doch nur dem Spekulieren und dem, was die Seelen begehren! Und gewiß, bereits kam zu ihnen von ihrem HERRN die Rechtleitung.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdcccviii{Oder gehört dem Menschen etwa das, was er sich wünscht?!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdcccix{So gehört ALLAH das Letzte und das Erste.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdcccx{Und wie viele Engel gibt es in den Himmeln, deren Fürbitte nichts nützt, es sei denn, nachdem ALLAH zustimmt, wem ER will und an dem ER Wohlgefallen hat.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdcccxi{Gewiß, diejenigen, die den Iman an das Jenseits nicht verinnerlichen, benennen ja die Engel wie die Benennung des Weiblichen.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdcccxii{Und sie verfügen darüber über keinerlei Wissen. Sie folgen doch nur dem Spekulieren. Und das Spekulieren ersetzt die Wahrheit in Nichts.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdcccxiii{So wende dich ab von dem, der sich von Unserer Ermahnung abwendet und ausschließlich das diesseitige Leben will.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdcccxiv{Dies ist ihr Erreichtes an Wissen. Gewiß, dein HERR weiß besser Bescheid über denjenigen, der von Seinem Weg abirrte, und ER weiß besser Bescheid über denjenigen, der Rechtleitung fand.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdcccxv{Und ALLAH gehört, was in den Himmeln und was auf Erden ist. ER wird denjenigen, die Schlechtes taten, doch das vergelten, was sie taten, und ER wird denjenigen, die Ihsan taten, mit dem Besseren vergelten.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdcccxvi{Es sind diejenigen, die das Schwere an Verfehlungen und die Abscheulichkeiten außer den Kleinigkeiten meiden. Gewiß, dein HERR ist von allumfassender Vergebung. ER weiß besser Bescheid über euch, als ER euch von der Erde hervorkommen ließ, und als ihr Embryos in den Bäuchen eurer Mütter wart, so lobt euer Selbst nicht! ER weiß besser Bescheid über denjenigen, der Taqwa gemäß handelt.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdcccxvii{Wie siehst du es mit demjenigen, der sich abwandte\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdcccxviii{und ein wenig gab, dann sich enthielt?!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdcccxix{Hat er etwa das Verborgene bei sich, so sieht er?!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdcccxx{Wurde ihm etwa nicht mitgeteilt über das, was in den Schriften von Musa ist\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxi{sowie von Ibrahim, der (das ihm Gebotene) einhielt?!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxii{Daß keine belastete Seele die Last einer anderen trägt,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxiii{und daß es dem Menschen nur das gibt, was er erstrebte,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxiv{und daß sein Streben doch gesehen werden wird,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxv{dann ihm dafür die vollständigste Vergeltung vergolten wird,\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxvi{und daß zu deinem HERRN doch das Endgültige ist,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxvii{und daß ER doch Derjenige ist, Der lachen und weinen ließ,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxviii{und daß ER doch Derjenige ist, Der sterben und beleben ließ,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxix{und daß ER doch die Zweiheit, das Männliche und das Weibliche erschuf,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxx{aus einer Nutfa, wenn sie ergossen wird,\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxi{und daß Ihm doch die andere Erweckung obliegt,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxii{und daß ER doch Derjenige ist, Der reich werden und besitzen ließ,\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxiii{und daß ER doch Derjenige ist, Der HERR von Asch-schi'ra ist,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxiv{und daß ER doch das erste 'Aad zugrunde richtete\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxv{sowie Thamud, dann nichts übrigließ,\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxvi{sowie die Leute von Nuh vorher, gewiß, sie pflegten diejenigen zu sein, die noch mehr Unrecht und noch mehr Übertretungen begingen.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxvii{Und das Umgewandte ließ ER abstürzen,\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxviii{so bedeckte sie, was sie bedeckte.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmdcccxxxix{An welchen der Wohltaten deines HERRN zweifelst du denn?!\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmdcccxl{Dies ist eine Warnung von den ersten Warnungen.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmdcccxli{Die Nahende nahte sich,\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlii{für die es anstelle von ALLAH nichts Aufdeckendes gibt.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmdcccxliii{Seid ihr etwa über diesen Bericht erstaunt,\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmdcccxliv{lacht und weint nicht,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlv{während ihr in Arroganz sich Vergnügende seid?!\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlvi{So vollzieht Sudschud für ALLAH und dient Ihm!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlvii{\basmalahde Die Stunde nahte sich und der Mond spaltete sich.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlviii{Und wenn sie eine Aya sehen, wenden sie sich ab und sagen: ``Dies ist eine andauernde Magie."\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmdcccxlix{Und sie leugneten ab und folgten ihren Neigungen. Und jede Angelegenheit hat einen Endpunkt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmdcccl{Und gewiß, bereits kamen zu ihnen von den Mitteilungen, worin es Warnung gab.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmdcccli{Es ist eine (sie) erreichende Weisheit. Und was nützen denn die Warnungen?!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmdccclii{So kehre dich von ihnen ab! An dem Tag, wenn der Rufende zu etwas Mißbilligtem ruft -\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmdcccliii{erniedrigt sind ihre Blicke, sie kommen aus den Gräbern heraus, als wären sie ausschwärmende Heuschrecken,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmdcccliv{eilend zum Rufenden. Die Kafir sagen: ``Dies ist ein ziemlich schwerer Tag."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmdccclv{Vor ihnen leugneten die Leute von Nuh ab, so bezichtigten sie Unseren Diener der Lüge und sagten: ``Er ist geistesgestört und wurde besessen."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmdccclvi{Dann richtete er Bittgebete an seinen HERRN: ``Ich bin unterlegen, so stehe bei!"\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmdccclvii{Dann öffneten WIR dir Tore des Himmels mit geschütteten Wassermassen,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmdccclviii{und WIR ließen aus der Erde Quellen entspringen, dann traf sich das Wasser für eine Angelegenheit, die bereits bestimmt wurde,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmdccclix{und WIR ließen ihn auf etwas von Planken und Nägeln tragen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmdccclx{Es fuhr unter Unserer Aufsicht als Vergeltung für denjenigen, der Kufr zu betreiben pflegte.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmdccclxi{Und gewiß, bereits ließen WIR es als eine Aya! Gibt es etwa einen sich Erinnernden?!\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmdccclxii{Also wie waren Meine Peinigung und Meine Ermahnungen?!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmdccclxiii{Und gewiß, bereits erleichterten WIR den Quran zur Ermahnung. Gibt es etwa einen sich Erinnernden?!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmdccclxiv{Abgeleugnet hat 'Aad. Also wie waren Meine Peinigung und Meine Ermahnungen?!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmdccclxv{Gewiß, WIR schickten über sie einen Wirbelsturm an einem unheilvollen, andauernden Tag,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmdccclxvi{er zog die Menschen heraus, als wären sie Stümpfe entwurzelter Dattelpalmen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmdccclxvii{Also wie waren Meine Peinigung und Meine Ermahnungen?!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmdccclxviii{Und gewiß, bereits erleichterten WIR den Quran zur Ermahnung! Gibt es etwa einen sich Erinnernden?!\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmdccclxix{Abgeleugnet hat Thamud die Warnungen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmdccclxx{Dann sagten sie: ``Etwa einem einzigen Menschen von uns folgen wir? Gewiß, wir sind dann doch im Irregehen und in Peinigung.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxi{Wurde ihm etwa von unter uns die Ermahnung zuteil? Nein, sondern er ist ein prahlerischer Lügner."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxii{Sie werden noch morgen wissen, wer der prahlerische Lügner ist.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxiii{Gewiß, WIR werden das Kamelweibchen als Fitna für sie entsenden, also warte auf sie und übe dich noch mehr in Geduld!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxiv{Und teile ihnen mit, daß das Wasser zwischen ihnen geteilt wird! Jeder Trinkanteil ist nur für die Anwesenden.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxv{Dann riefen sie ihren Weggenossen, dann nahm er (das Schwert), dann verletzte er ihre Beine.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxvi{Also wie waren Meine Peinigung und Meine Ermahnungen?!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxvii{Gewiß, WIR schickten über sie einen einzigen Schrei, dann waren sie wie die Stoppeln für den Stallinhaber.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxviii{Und gewiß, bereits erleichterten WIR den Quran zur Ermahnung! Gibt es etwa einen sich Erinnernden?!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxix{Abgeleugnet haben die Leute von Lut die Warnungen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxx{Gewiß, WIR schickten über sie einen Sandsturm außer der Familie von Lut, diese erretteten WIR vor der Morgendämmerung,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxi{als Wohltat von Uns. Solcherart vergelten WIR demjenigen, der sich dankbar erweist.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxii{Und gewiß, bereits warnten WIR sie vor Unserer Gewalttat, dann zweifelten sie die Warnungen an.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxiii{Und gewiß, bereits suchten sie ihn gegen seine Neigung und seine Gäste zu verführen, dann löschten WIR ihr Augenlicht aus. Also erfahrt Meine Peinigung und Meine Warnungen!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxiv{Und gewiß, bereits morgens, in der Morgenfrühe kam zu ihnen eine (über sie) niedergelassene Peinigung.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxv{Also erfahrt Meine Peinigung und Meine Warnungen!\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxvi{Und gewiß, bereits erleichterten WIR den Quran zur Ermahnung! Gibt es etwa einen sich Erinnernden?!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxvii{Und gewiß, bereits kamen zu Pharaos Leuten die Warnungen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxviii{Sie leugneten Unsere Ayat allesamt ab, dann bestraften WIR sie das Bestrafen Eines allmächtigen Allwürdigen.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmdccclxxxix{Sind etwa eure Kafir besser als diese?! Oder gibt es für euch etwa eine Unschuldserklärung in den Schriften?!\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmdcccxc{Oder sagen sie etwa: ``Wir sind alle zusammen Siegende"?!\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmdcccxci{Die Sammlung wird doch noch eine Niederlage erleiden und den Rücken flüchtend umkehren.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcii{Nein, sondern die Stunde ist ihr Termin. Und die Stunde ist noch schlimmer, noch bitterer.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmdcccxciii{Gewiß, die schwer Verfehlenden sind im Irregehen und in Hitzeglut,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmdcccxciv{an dem Tag, wenn sie im Feuer auf ihren Gesichtern geschleift werden. Kostet die Berührung von Saqar.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcv{Gewiß, jedes Ding erschufen WIR gemäß einer Bestimmung.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcvi{Und Unsere Anweisung ist nichts anderes als eine Einzige, wie der Augenblick.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcvii{Und gewiß, bereits richteten WIR eure Anhänger zugrunde. Gibt es etwa einen sich Erinnernden?!\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcviii{Und alles, was sie machten, ist in den Schriften.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmdcccxcix{Auch jedes Kleine und Große ist niedergeschrieben.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcm{Gewiß, die Muttaqi sind in Dschannat und an Flüssen,\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcmi{an einem Sitz des Wahrhaftigen, bei Einem allmächtigen Herrscher.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcmii{\basmalahde Der Allgnade Erweisende\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcmiii{lehrte den Quran,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcmiv{erschuf den Menschen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcmv{lehrte ihn das Artikulieren.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcmvi{Die Sonne und der Mond sind nach einer Berechnung.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcmvii{Und das Bodengewächs und die Bäume vollziehen Sudschud.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcmviii{Und den Himmel erhob ER und setzte Al-mizan ein:\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcmix{``Begeht keine Übertretungen hinsichtlich von Al-mizan\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcmx{Und haltet das (mit Al-mizan) Festgestellte nach Gerechtigkeit ein und mindert Al-mizan nicht!"\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcmxi{Und die Erde setzte ER für die Lebewesen ein.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcmxii{Auf ihr sind Obst und Dattelbäume mit Blütenkelchen\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcmxiii{sowie Körner mit Halmen und Blättern.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcmxiv{Also, welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcmxv{ER erschuf den Menschen aus Salsal wie Ton\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcmxvi{und erschuf die Dschinn aus rauchlosem Feuer.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcmxvii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcmxviii{ER ist der HERR beider Osten und beider Westen.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcmxix{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcmxx{ER ließ beide Meere aufeinandertreffen,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcmxxi{zwischen beiden gibt es eine Trennung, sie vermischen sich nicht.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmcmxxii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmcmxxiii{Aus beiden kommen Perlen und Korallen hervor.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmcmxxiv{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmcmxxv{Und Ihm gehören die gebauten Fahrenden auf dem Meer wie die Berge.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmcmxxvi{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmcmxxvii{Alles, was auf ihr (der Erde) ist, vergeht,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmcmxxviii{und es bleibt dein HERR, Der mit der Majestät und der Würde.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmcmxxix{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmcmxxx{Ihn bitten alle, die in den Himmeln und auf Erden sind. Jeden Yaum ist ER befaßt mit den Angelegenheiten.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxi{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxii{WIR werden euch Uns noch vornehmen, ihr Menschen und Dschinn.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxiii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxiv{Ihr Gruppe von Dschinn und Menschen! Wenn ihr die Gegenden der Himmel und der Erde durchdringen könnt, so durchdringt! Ihr durchdringt nur mit Macht.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxv{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxvi{ER schickt über euch beide Flammen von Feuer und Rauch, so könnt ihr euch nicht helfen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxvii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxviii{Und wenn der Himmel sich spaltete, dann rot wie rotes Leder wird.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmcmxxxix{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmcmxl{Also an diesem Tag wird nach seiner Verfehlung weder ein Mensch noch ein Dschinn befragt.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmcmxli{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmcmxlii{Die schwer Verfehlenden werden durch ihreMerkmale bekannt, dann wird an den Stirnhaaren und an den Füßen gezerrt.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmcmxliii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmcmxliv{Dies ist Dschahannam, welche die schwer Verfehlenden ableugnen.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmcmxlv{Sie machen die Runde in ihr (Dschahannam) und mit extrem hoher Temperatur Siedendem.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmcmxlvi{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmcmxlvii{Und für denjenigen, der sich vor der Vorstellung vor seinem HERRN fürchtet, gibt es zwei Dschanna -\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmcmxlviii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?! -\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmcmxlix{die von Bäumen sind.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmcml{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmcmli{In denen es zwei Quellen gibt, die fließen.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmcmlii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmcmliii{In denen es von jeder Obstart Zweiheit gibt.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmcmliv{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmcmlv{Angelehnt sind sie auf Liegen, deren Inlett aus Istabraq ist. Und die Ernte beider Dschannat ist nahe.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmcmlvi{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmcmlvii{In den beiden gibt es des Blicks Zurückhaltende, die weder ein Mensch, noch ein Dschinn vor ihnen deflorierte.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmcmlviii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmcmlix{Als wären sie wie Rubinen und Korallen.\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmcmlx{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmcmlxi{Gibt es etwa eine andere Belohnung für den Ihsan, als den Ihsan?!\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmcmlxii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmcmlxiii{Und außer ihnen sind noch zwei Dschannat.\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmcmlxiv{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmcmlxv{Sie sind dunkelgrün.\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmcmlxvi{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmcmlxvii{In ihnen gibt es zwei Quellen, die sprudeln.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmcmlxviii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmcmlxix{In ihnen gibt es Obst, Datteln und Granatäpfel.\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmcmlxx{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxi{In ihnen sind schöne Gute.\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxiii{Sie sind Hur-Wesen, die sich in den Zelten zurückgezogen haben.\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxiv{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxv{Die weder ein Mensch, noch ein Dschinn vor ihnen deflorierte.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxvi{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxvii{Angelehnt auf grünen Kissen und schönen Teppichen.\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxviii{Also welche von den Wohltaten eures HERRN leugnet ihr beide ab?!\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxix{Immer allerhabener ist der Name deines HERRN, Der mit der Majestät und der Würde.\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxx{\basmalahde Wenn das sich Ereignende sich ereignet,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxi{für dessen Ereignen gibt es dann kein Ableugnendes.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxii{Es ist niedermachend, hochhebend,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxiii{wenn die Erde mit Schütteln geschüttelt wird\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxiv{und die Berge mit Zertrümmerung zertrümmert werden,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxv{dann sie zu verfliegendem Staub werden,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxvi{und ihr zu drei Zweiheiten wurdet.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxvii{So gibt es die Weggenossen von der Rechten. Was sind die Weggenossen von der Rechten?!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxviii{Ebenso die Weggenossen des bösen Vorzeichens. Was sind die Weggenossen des bösen Vorzeichens?!\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmcmlxxxix{Auch gibt es die den Vorsprung habenden, die den Vorsprung haben,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmcmxc{diese sind die Nahegebrachten\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmcmxci{in den Dschannat des Wohlergehens,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmcmxcii{eine Gruppe von den Ersten\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmcmxciii{und ein wenig von den Letzten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmcmxciv{Sie sind auf durchwebten Liegen,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmcmxcv{angelehnt darauf einander gegenüber.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmcmxcvi{Ihnen servieren ewige Dienstjungen\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmcmxcvii{mit Bechern, Krügen und Trinkschalen aus (einer) Weinquelle.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmcmxcviii{Sie bekommen dadurch keine Kopfschmerzen und sie werden nicht berauscht.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmcmxcix{Und es gibt Obst von dem, was sie sich auswählen,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmm{sowie Geflügelfleisch von dem, was sie begehren,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmi{sowie Hur-'in,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmii{als wären sie in Muscheln bewahrte Perlen,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmiii{als Belohnung für das, was sie zu tun pflegten.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmiv{Weder hören sie darin sinnloses Gerede, noch Verfehltes\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmv{außer dem Wort: ``Salam, Salam (sei mit euch)".\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmvi{Und die Weggenossen von der Rechten. Was sind die Weggenossen von der Rechten?!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmvii{Sie sind unter entdornten Lotusbäumen\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmviii{und nebeneinanderstehenden Akazienbäumen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmix{Und (sie haben) einen ausgebreiteten Schatten\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmx{und gegossenes Wasser\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmxi{und viel Obst,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmxii{das weder zu Ende geht, noch verboten ist,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmxiii{und gehobene Polster.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmxiv{Gewiß ließen WIR sie im Erstehen erstehen,\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmxv{dann machten WIR sie zu Jungfräulichen\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmxvi{anschmiegsamen Gleichaltrigen\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmxvii{für dieWeggenossen von der Rechten,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmxviii{eine Gruppe von den Ersten\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmxix{und eine Gruppe von den Letzten.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmxx{Und die Weggenossen von der Linken. Was sind die Weggenossen von der Linken?!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmxxi{Sie sind in gluthitzigen Winden und Siedendem\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmxxii{und im schwarzqualmigen Schatten,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmxxiii{der weder kühl noch nützlich ist.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmxxiv{Gewiß, sie waren vor diesem luxurierend\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmxxv{und pflegten auf der gewaltigen Verfehlung zu beharren.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmxxvi{Auch pflegten sie zu sagen: ``Auch wenn wir starben und zu Erde und Knochen wurden, werden wir etwa noch erweckt,\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmxxvii{auch unsere ersten Ahnen?"\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmxxviii{Sag: ``Gewiß, die ersten und die letzten\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmxxix{werden zweifellos zur Verabredung eines bekannten Tages versammelt."\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmxxx{Dann werdet ihr ableugnende Abirrende\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmxxxi{gewiß von Bäumen von Zaqum speisen,\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmxxxii{dann davon die Bäuche vollmachen,\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmmxxxiii{dann darauf von Siedendem trinken,\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmmxxxiv{dann so trinken, wie das Trinken der durstig-kranken Kamele.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmmxxxv{Das ist ihre Unterkunft am Tag des Din.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmmxxxvi{WIR erschufen euch. Würdet ihr (dieser Aussage) doch Glauben schenken!\qt@no{57}}%
\qt@newcmd\qurantrans@de@mmmmmxxxvii{Wie seht ihr das, was ihr an Samen ergießt?!\qt@no{58}}%
\qt@newcmd\qurantrans@de@mmmmmxxxviii{Seid ihr diejenigen, die es erschaffen, oder sind WIR Der Schöpfer?!\qt@no{59}}%
\qt@newcmd\qurantrans@de@mmmmmxxxix{WIR bestimmten unter euch den Tod, und Uns wird nicht zuvorgekommen,\qt@no{60}}%
\qt@newcmd\qurantrans@de@mmmmmxl{daßWIR euresgleichen austauschen und euch in dem was ihr nicht kennt, erstehen lassen.\qt@no{61}}%
\qt@newcmd\qurantrans@de@mmmmmxli{Und gewiß, bereits wißt ihr Bescheid über die erste Erstehung, würdet ihr doch euch erinnern!\qt@no{62}}%
\qt@newcmd\qurantrans@de@mmmmmxlii{Wie seht ihr das, was ihr sät?!\qt@no{63}}%
\qt@newcmd\qurantrans@de@mmmmmxliii{Seid ihr diejenigen, die es wachsen lassen, oder sind WIR es, Der es wachsen läßt?!\qt@no{64}}%
\qt@newcmd\qurantrans@de@mmmmmxliv{Wenn WIR es wollten, gewiß würden WIR es zu Zermalmtem machen, dann würdet ihr nicht aufhören, euch zu bedauern:\qt@no{65}}%
\qt@newcmd\qurantrans@de@mmmmmxlv{``Gewiß, wir sind doch Verlust-Erleidende.\qt@no{66}}%
\qt@newcmd\qurantrans@de@mmmmmxlvi{Nein, sondern es wurde uns vorenthalten."\qt@no{67}}%
\qt@newcmd\qurantrans@de@mmmmmxlvii{Wie seht ihr das Wasser, das ihr trinkt?!\qt@no{68}}%
\qt@newcmd\qurantrans@de@mmmmmxlviii{Seid ihr diejenigen, die es aus den Wolken fallen lassen, oder sind WIR Der Hinabsendende?!\qt@no{69}}%
\qt@newcmd\qurantrans@de@mmmmmxlix{Wenn WIR wollten, würden WIR es sauer machen. Würdet ihr euch doch dankbar erweisen!\qt@no{70}}%
\qt@newcmd\qurantrans@de@mmmmml{Wie seht ihr das Feuer, das ihr entzündet?!\qt@no{71}}%
\qt@newcmd\qurantrans@de@mmmmmli{Seid ihr diejenigen, die dessen Baum entstehen ließen, oder sind WIR Der Hervorbringende?!\qt@no{72}}%
\qt@newcmd\qurantrans@de@mmmmmlii{WIR machten es als Ermahnung und Verbrauchsgut für die Umherziehenden.\qt@no{73}}%
\qt@newcmd\qurantrans@de@mmmmmliii{So lobpreise mit dem Namen deines allerhabenen HERRN.\qt@no{74}}%
\qt@newcmd\qurantrans@de@mmmmmliv{Also schwöre ICH bei den Phasen der Sterne,\qt@no{75}}%
\qt@newcmd\qurantrans@de@mmmmmlv{und gewiß, es ist zweifelsohne ein gewaltiger Schwur, würdet ihr nur wissen!\qt@no{76}}%
\qt@newcmd\qurantrans@de@mmmmmlvi{Gewiß, er ist doch ein edler Quran\qt@no{77}}%
\qt@newcmd\qurantrans@de@mmmmmlvii{in einer bewahrten Schrift,\qt@no{78}}%
\qt@newcmd\qurantrans@de@mmmmmlviii{den nur die Gereinigten berühren.\qt@no{79}}%
\qt@newcmd\qurantrans@de@mmmmmlix{Er ist die sukzessive Hinabsendung vom HERRN aller Schöpfung.\qt@no{80}}%
\qt@newcmd\qurantrans@de@mmmmmlx{Diesem Bericht gegenüber seid ihr etwa heuchelnd?!\qt@no{81}}%
\qt@newcmd\qurantrans@de@mmmmmlxi{Und ihr erweist euch dankbar für euer Rizq, daß ihr ihn doch ableugnet?!\qt@no{82}}%
\qt@newcmd\qurantrans@de@mmmmmlxii{Und hättet ihr (die Seele zurückgeholt), wenn sie die Kehle erreichte,\qt@no{83}}%
\qt@newcmd\qurantrans@de@mmmmmlxiii{während ihr in dieser Zeit zuschaut.\qt@no{84}}%
\qt@newcmd\qurantrans@de@mmmmmlxiv{Und WIR sind ihm näher als ihr. Doch ihr seht nicht.\qt@no{85}}%
\qt@newcmd\qurantrans@de@mmmmmlxv{Und würdet ihr nicht zu verurteilen sein,\qt@no{86}}%
\qt@newcmd\qurantrans@de@mmmmmlxvi{würdet ihr sie zurückholen, wenn ihr wahrhaftig wäret.\qt@no{87}}%
\qt@newcmd\qurantrans@de@mmmmmlxvii{Und wenn er zu den Nahegebrachten gehörte,\qt@no{88}}%
\qt@newcmd\qurantrans@de@mmmmmlxviii{dann gibt es Erholung, Gnade und eine Dschanna des Wohlergehens.\qt@no{89}}%
\qt@newcmd\qurantrans@de@mmmmmlxix{Und wenn er zu den Weggenossen von der Rechten gehörte,\qt@no{90}}%
\qt@newcmd\qurantrans@de@mmmmmlxx{dann Salam sei dir von den Weggenossen von der Rechten.\qt@no{91}}%
\qt@newcmd\qurantrans@de@mmmmmlxxi{Und wenn er zu den abirrenden Ableugnenden gehörte,\qt@no{92}}%
\qt@newcmd\qurantrans@de@mmmmmlxxii{dann gibt es eine Unterkunft von Siedendem\qt@no{93}}%
\qt@newcmd\qurantrans@de@mmmmmlxxiii{und Hineinwerfen in der Hölle.\qt@no{94}}%
\qt@newcmd\qurantrans@de@mmmmmlxxiv{Sicherlich, dies ist doch die wahre Gewißheit.\qt@no{95}}%
\qt@newcmd\qurantrans@de@mmmmmlxxv{So lobpreise mit dem Namen deines allerhabenen HERRN!\qt@no{96}}%
\qt@newcmd\qurantrans@de@mmmmmlxxvi{\basmalahde ALLAH lobpreisen alle, die in den Himmeln und auf Erden sind. Und ER ist Der Allwürdige, Der Allweise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmlxxvii{Ihm gehört die Herrschaft der Himmel und der Erde. ER belebt und läßt sterben. Und ER ist über alles allmächtig.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmlxxviii{ER ist Der immer Dagewesene, Der immer Bleibende, Der Offenkundige und Der Inwendige. Und ER ist über alles allwissend.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmlxxix{ER ist Derjenige, Der die Himmel und die Erde in sechs Ay-yam erschuf, dann Sich dem Al'ahrsch zuwandte. ER kennt das, was in die Erde hineingeht, was aus ihr herauskommt, was vom Himmel hinabsteigt und was zu ihm heraufsteigt. Und ER ist mit euch, wo ihr seid. Und ALLAH ist dessen, was ihr tut, allsehend.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmlxxx{Ihm gehört die Herrschaft der Himmel und der Erde. Und ALLAH unterliegen die Angelegenheiten.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxi{ER läßt die Nacht in den Tag einfließen und läßt den Tag in die Nacht einfließen. Und ER ist allwissend über das in den Brüsten.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxii{Verinnerlicht den Iman an ALLAH und Seinen Gesandten und spendet von dem, worin ER euch zu Sachwaltern machte. Für diejenigen von euch, die den Iman verinnerlichten und spendeten, ist eine große Belohnung bestimmt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxiii{Und weshalb verinnerlicht ihr den Iman nicht an ALLAH, während der Gesandte euch dazu aufruft, den Iman an euren HERRN zu verinnerlichen?! Und ER nahm bereits euer Gelöbnis entgegen, solltet ihr Mumin sein.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxiv{ER ist Derjenige, Der seinem Diener deutliche Ayat hinabgesandt, damit ER euch herausbringt von den Finsternissen zum Licht. Und gewiß, ALLAH ist euch gegenüber doch allgütig, allgnädig.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxv{Und weshalb spendet ihr nicht fi-sabilillah, und ALLAH gehört die Hinterlassenschaft der Himmel und der Erde?! Nicht gleich ist derjenige von euch, der vor der Einnahme (von Mekka) spendete und kämpfte, (dem, der dies nicht tat). Diese haben höhere Stellung als diejenigen, die danach spendeten und kämpften. Und beiden versprach ALLAH das Schöne. Und ALLAH ist dessen, was ihr tut, allkundig.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxvi{Wer ist dieser, der ALLAH (Seiner Sache) ein schönes Darlehen gibt?! Dann wird ER ihm dieses vervielfachen. Und für ihn ist eine edle Belohnung bestimmt.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxvii{An dem Tag, wenn du die Mumin-Männer und die Mumin-Frauen siehst, ihr Licht läuft vor ihnen und mit ihrer Rechten: ``Eine frohe Botschaft gibt es heute für euch - Dschannat, die von Flüssen durchflossen sind, als Ewige darin." Dies ist der gewaltige Erfolg.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxviii{An dem Tag, wenn die Munafiq-Männer und die Munafiq-Frauen zu denjenigen, die den Iman verinnerlichten, sagten: ``Wartet auf uns, damit wir von eurem Licht etwas profitieren", wird gesagt: ``Geht nach hinten zurück, dann sucht nach Licht." Dann wurde zwischen ihnen eine Trennmauer errichtet, die eine Tür hat, in deren Innenseite ist Gnade, während außerhalb Peinigung ist.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmlxxxix{Sie rufen sie: ``Waren wir nicht mit euch?" Sie sagten: ``Doch, mit Sicherheit! Aber ihr habt euch selbst der Fitna überlassen, abgewartet und angezweifelt, und euch täuschten die Hoffnungen, bis ALLAHs Anweisung kam. Und euch täuschte ALLAH gegenüber das Täuschende.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmxc{So wird heute weder von euch eine Ersatzleistung entgegengenommen, noch von denjenigen, die Kufr betrieben haben. Eure Unterkunft ist das Feuer. Es ist euer Maula. Und erbärmlich ist das Werden."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmxci{Ist es noch nicht Zeit für diejenigen, die den Iman verinnerlichten, daß ihre Herzen ALLAHs Gedenken und dem, was von der Wahrheit hinabgesandt wurde, Ehrfurcht erweisen, und daß sie nicht wie diejenigen werden, denen die Schrift vorher zuteil wurde, dann für sie die Zeit lang wurde, dann ihre Herzen hart wurden?! Und viele von ihnen sind Fasiq.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmxcii{Wißt, daß ALLAH gewiß die Erde nach ihrem Tod belebt. Bereits erläuterten WIR euch die Ayat, damit ihr begreift.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmxciii{Gewiß, die spendenden Männer, die spendenden Frauen und diejenigen, die ALLAH (Seiner Sache) eine Hasan-Anleihe geben, diesen wird es vervielfacht und für sie ist eine schöne Belohnung bestimmt.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmxciv{Und diejenigen, die den Iman an ALLAH und Seine Gesandten verinnerlichten, diese sind die äußerstWahrhaftigen und die Zeugen bei ihrem HERRN. Sie haben ihre Belohnung und ihr Licht. Und diejenigen, die Kufr betrieben und Unsere Ayat abgeleugnet haben, diese sind die Weggenossen der Hölle.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmxcv{Wißt, daß das diesseitige Leben doch nur Vergnügen, Ablenkung, Schmuck, Wichtigtuerei unter euch und Vermehrung an Vermögen und Kindern ist. Es gleicht reichlichem Regen, dessen Pflanzenwuchs den Kafir gefällt. Dann vertrocknet er (der Pflanzenwuchs), und du siehst ihn gelb werden, dann wird er Zermalmtes, und im Jenseits gibt es harte Peinigung und Vergebung von ALLAH und Wohlgefallen. Doch das diesseitige Leben ist nichts außer Verbrauchsgut der Täuschung.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmxcvi{Wetteifert um Vergebung von eurem HERRN und eine Dschanna, deren Breite wie die Breite von Himmel und Erde ist, sie wurde denjenigen, die den Iman an ALLAH und Seinen Gesandten verinnerlichten, vorbereitet. Dies ist ALLAHs Gunst, die ER dem gewährt, den ER will. Und ALLAH ist von unermeßlicher Gunst.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmxcvii{Weder trifft ein Unheil auf Erden, noch euch selbst, ohne daß es in einer Schrift ist, bevor WIR es erschufen. Gewiß, dies ist für ALLAH etwas Leichtes.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmxcviii{Damit ihr nicht trauert um das, was euch entgangen ist, und euch nicht übermäßig freut über das, was euch gewährt wurde. Und ALLAH liebt nicht jeden sich wichtigtuenden Prahler,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmxcix{diejenigen, die geizen und den Menschen Geiz gebieten. Und wer sich abwendet, so ist gewiß ALLAH Der absolut Autarke, Der Alllobenswürdige.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmc{Gewiß, bereits entsandten WIR Unsere Gesandten mit den klaren Zeichen und sandten mit ihnen die Schrift und Al-mizan hinab, damit dieMenschen Gerechtigkeit walten lassen. Und WIR sandten das Eisen hinab, in dem es heftige Gewalttätigkeit und Nützliches für die Menschen gibt, und damit ALLAH kenntlich macht, wer Seinem (Din) und Seinen Gesandten im Verborgenen beisteht. Gewiß, ALLAH ist allkräftig, allwürdig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmci{Und gewiß, bereits entsandten WIR Nuh und Ibrahim und ließen in ihrer Nachkommenschaft die Prophetenschaft und die Schrift sein. Also manche von ihnen sind rechtgeleitet, und viele von ihnen sind Fasiq.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcii{Dann schickten WIR Unsere Gesandten auf ihren Spuren nach und schickten 'Isa Ibnu-Maryam nach und ließen ihm Alindschil zuteil werden. Und WIR legten in die Herzen derjenigen, die ihm folgten, Barmherzigkeit und Gnade und Mönchtum, das sie erfanden, das WIR ihnen nicht auferlegten außer im Streben nach ALLAHs Wohlgefallen. Doch sie hielten es nicht ein, wie es ihm gebührt. Dann ließen WIR denjenigen von ihnen, die den Iman verinnerlichten, ihre Belohnung zuteil werden. Und viele von ihnen sind Fasiq.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmciii{Ihr, die den Iman verinnerlicht habt! Handelt Taqwa gemäß ALLAH gegenüber und verinnerlicht den Iman an Seinen Gesandten, dann gewährt ER euch zwei Anteile von Seiner Gnade, macht für euch Licht, mit dem ihr geht, und vergibt euch. Und ALLAH ist allvergebend, allgnädig.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmciv{Damit die Schriftbesitzer mit Sicherheit wissen, daß sie über keineMacht über etwas von ALLAHs Gunst verfügen, und daß die Gunst gewiß ALLAH unterliegt, ER läßt sie erweisen, wem ER will. Und ALLAH ist von unermeßlicher Gunst.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcv{\basmalahde Bereits hörte ALLAH die Äußerung derjenigen, die mit dir wegen ihres Ehemannes disputierte und sich vor ALLAH beklagte, und ALLAH hörte euren Dialog. Gewiß, ALLAH ist Allhörend, Allsehend.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcvi{Diejenigen von euch, die ihren Ehefrauen Zihar aussprachen, diese sind sicherlich nicht ihre Mütter. Gewiß, ihre Mütter sind nur diejenigen, die sie gebaren. Und gewiß, sie sagen doch etwas Mißbilligtes vom Gesagten und Falsches. Und gewiß, ALLAH ist doch reue-annehmend, allvergebend.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcvii{Und denjenigen, die ihren Ehefrauen Zihar aussprachen, dann zurücknehmen, was sie sagten, gilt (als Kaffara ) die Befreiung eines Unfreien, bevor beide sich intim berühren. Damit werdet ihr ermahnt. Und ALLAH ist dessen, was ihr tut, allkundig.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcviii{Also wer dieses nicht vermag, so fastet er zwei aufeinander folgende Monate lang, bevor beide sich intim berühren. Und wer es nicht kann, dann speist er sechzig Arme. Dies, damit ihr den Iman an ALLAH und Seinen Gesandten verinnerlicht. Und diese sind ALLAHs Richtlinien. Und für die Kafir ist eine qualvolle Peinigung bestimmt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcix{Gewiß, diejenigen, die sich ALLAH und Seinen Gesandten widersetzen, wurden niedergemacht, wie diejenigen vor ihnen niedergemacht wurden. Und bereits sandten WIR deutliche Ayat hinab. Und für die Kafir ist erniedrigende Peinigung bestimmt.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcx{An dem Tag, wenn sie allesamt erweckt werden, dann wird ER ihnen Mitteilung machen über das, was sie taten. ALLAH umfaßte es detailliert, während sie es vergaßen. Und ALLAH ist über alles Zeuge.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcxi{Hast du etwa nicht wahrgenommen, daß ALLAH alles kennt, was in den Himmeln und auf Erden ist?! Es gibt weder eine heimliche Unterredung von dreien, ohne daß ER ihr Vierter ist, noch von Fünfen, ohne daß ER ihr Sechster ist, noch von weniger als diesen oder noch mehr, ohne daß ER doch mit ihnen ist, wo sie sind. Dann wird ER ihnen Mitteilung machen über das, was sie taten, am Tag der Auferstehung. Gewiß, ALLAH ist über alles allwissend.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcxii{Sahst du etwa nicht diejenigen, denen die heimliche Unterredung verboten wurde, dann sie dennoch zu dem zurückkehren, was ihnen verboten wurde, und unterhalten sich heimlich über das Verfehlen, das Übertreten und das Widersprechen des Gesandten?! Und wenn sie zu dir kommen, begrüßen sie dich mit dem, womit ALLAH dich nicht grüßte, und sagen in ihrem Innern: ``Würde ALLAH uns doch peinigen für das, was wir sagen!" Ihnen genügt Dschahannam, sie werden darin hineingeworfen. Also erbärmlich ist dasWerden.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcxiii{Ihr, die den Iman verinnerlicht habt! Wenn ihr euch vertraulich unterhaltet, dann unterhaltet euch vertraulich nicht über das Verfehlen, das Übertreten und das Widersprechen des Gesandten! Und unterhaltet euch vertraulich (nur) über das gütige Handeln und die Taqwa! Und handelt Taqwa gemäß ALLAH gegenüber, vor Dem ihr versammelt werdet.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcxiv{Die heimliche Unterredung ist nur vom Satan, damit ihr diejenigen, die den Iman verinnerlichten, traurig macht. Doch er wird ihnen in nichts schaden außer mit ALLAHs Zustimmung. Und ALLAH gegenüber sollen die Mumin Tawakkul üben.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcxv{Ihr, die den Iman verinnerlicht habt! Wenn euch gesagt wird: ``Macht Platz beim Sitzen!", dann macht Platz! ALLAH möge euch Platz machen lassen. Und wenn euch gesagt wird: ``Steht auf!", dann steht auf! ALLAH erhöht diejenigen von euch, die den Iman verinnerlichten, und diejenigen, denen dasWissen zuteil wurde, um Stellungen. Und ALLAH ist dessen allkundig, was ihr tut.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcxvi{Ihr, die den Iman verinnerlicht habt! Wenn ihr euch mit dem Gesandten vertraulich unterhalten wollt, so gebt vor eurer vertraulichen Unterredung eine Sadaqa! Dies ist besser für euch und reiner. Und wenn ihr (es) nicht vermögt, also gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcxvii{Scheut ihr euch etwa, vor eurer vertraulichen Unterredung Sadaqa zugeben?! Und wenn ihr es nicht macht - und ALLAH nahm bereits eure Reue an, so verrichtet ordnungsgemäß das rituelle Gebet, entrichtet die Zakat und gehorcht ALLAH und Seinem Gesandten! Und ALLAH ist dessen allkundig, was ihr tut.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcxviii{Sahst du etwa nicht diejenigen, die sich Leute, über die ALLAH erzürnt ist, als Wali nahmen?! Sie sind weder von euch, noch von ihnen, und sie schwören auf die Lüge, während sie wissen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcxix{ALLAH bereitete ihnen eine qualvolle Peinigung vor. Gewiß, es war schlecht, was sie zu tun pflegten.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcxx{Sie nahmen sich ihre Eide als Schutzschild, dann hielten sie von ALLAHs Weg ab. So ist für sie erniedrigende Peinigung bestimmt.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcxxi{Ihnen werden weder ihr Vermögen, noch ihre Kinder vor ALLAH etwas nützen. Diese sind die Weggenossen des Feuers, sie bleiben darin ewig.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcxxii{An dem Tag, wenn ER sie allesamt erweckt, werden sie Ihm schwören, wie sie euch schwören, und sie denken, sie verfügen über etwas (Nützliches). Ja, zweifelsohne! Sie sind die eigentlichen Lügner.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcxxiii{Der Satan bemächtigte sich ihrer, dann ließ er sie ALLAHs Gedenken vergessen. Diese sind die Partei des Satans. Ja, zweifelsohne! Die Partei des Satans sind die wirklichen Verlierer.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcxxiv{Gewiß, diejenigen, die sich ALLAH und Seinem Gesandten widersetzen, diese sind unter den Erniedrigsten.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcxxv{ALLAH schrieb: ``Doch siegen werde ICH und Meine Gesandten." Gewiß, ALLAH ist allkräftig, allwürdig.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcxxvi{Du findest keine Leute, die den Iman an ALLAH und an den Jüngsten Tag verinnerlichen, diejenigen lieben, die sich ALLAH und Seinem Gesandten widersetzten, selbst dann nicht, wenn diese ihre Väter, ihre Söhne, ihre Brüder oder ihre Verwandten wären. Diese sind diejenigen, in deren Herzen ER den Iman festschrieb und die ER mit Ruhh von Ihm stärkte. Und ER läßt sie in Dschannat eintreten, die von Flüssen durchflossen werden, darin sind sie ewig. ALLAH hat Wohlgefallen an ihnen und sie sind mit Ihm zufrieden. Diese sind die ALLAH (loyale) Partei. Ja, zweifelsohne! Die ALLAH (loyale) Partei sind die wirklichen Erfolgreichen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcxxvii{\basmalahde ALLAH lobpreisen alle, die in den Himmeln und auf Erden sind. Und ER ist Der Allwürdige, Der Allweise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcxxviii{ER ist Derjenige, Der diejenigen von den Schriftbesitzern, die Kufr betrieben haben, von ihren Wohnstätten zur ersten Vertreibung heraustreiben ließ. Ihr dachtet nicht, daß sie herauskommen werden, und sie dachten, daß ihre Burgen sie vor ALLAH schützen würden. Dann kam ALLAH zu ihnen von da, womit sie nicht rechneten, und warf in ihre Herzen den Schrecken. Sie zerstörten ihre Häuser mit ihren eigenen Händen und den Händen der Mumin. So zieht (daraus) eine Lehre, ihr mit Einblick!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcxxix{Und hätte ALLAH ihnen die Deportation nicht bestimmt, hätte ER sie doch im Diesseits gepeinigt. Und für sie ist im Jenseits die Peinigung des Feuers bestimmt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcxxx{Dies, weil sie sich ALLAH und Seinem Gesandten widersetzten. Und wer sich ALLAH widersetzt, also gewiß, ALLAH ist hart im Strafen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxi{Und ihr habt nicht eine junge Dattelpalme gefällt oder sie auf ihren Wurzeln stehen lassen außer mit ALLAHs Zustimmung, und damit ER die Fasiq erniedrigt.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxii{Und das, was ALLAH Seinem Gesandten von ihnen zur Beute machte, dafür habt ihr weder Pferde noch Reitkamele rennen lassen. Doch ALLAH läßt Seine Gesandten gegen diejenigen vorgehen, die ER will. Und ALLAH ist über alles allmächtig.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxiii{Was ALLAH Seinem Gesandten von den Bewohnern der Ortschaften zur Beute machte, so ist dies für ALLAHs (Sache), für den Gesandten, für die von der Verwandtschaft, für die Waisen, für die Armen und für den in Not geratenen Reisenden, damit es (das Vermögen) nicht (nur) unter den Reichen von euch rotiere. Und was der Gesandte euch gab, so nehmt es! Und was er euch verbot, so laßt davon ab! Und handelt Taqwa gemäß ALLAH gegenüber! Gewiß, ALLAH ist hart im Strafen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxiv{Es ist für die hidschra-unternehmenden Armen, die von ihren Wohnstätten und Vermögensgütern vertrieben wurden, sie streben Gunst von ALLAH und Wohlgefallen an, und sie stehen ALLAHs (Din) und Seinem Gesandten bei. Diese sind die wirklichen Wahrhaftigen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxv{Und diejenigen, welche bereits die Wohnstätte (Madina) bewohnten und den Iman (verinnerlichten) vor ihnen, lieben diejenigen, die zu ihnen Hidschra unternahmen, und sie finden in ihren Brüsten kein Verlangen nach dem, was ihnen (den anderen) zuteil wurde. Und sie bevorzugen andere vor sich selbst, auch dann, wenn sie in Not sind. Und wer vor dem Geiz seines Ego geschützt wird, diese sind die wirklichen Erfolgreichen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxvi{Und diejenigen, die nach ihnen kamen, sagen: ``Unser HERR! Vergib uns und unseren Geschwistern, die vor uns den Iman verinnerlichten, und lasse in unseren Herzen keinen Groll sein für diejenigen, die den Iman verinnerlichten. Unser HERR! Gewiß, DU bist allgütig, allgnädig."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxvii{Hast du etwa nicht diejenigen gesehen, die Nifaq betrieben haben, sie sagen ihren Brüdern von den Schriftbesitzern, die Kufr betrieben haben: ``Wenn ihr vertrieben würdet, dann würden wir sicherlich mit euch weggehen und gegen euch überhaupt niemandem gehorchen. Und wenn ihr bekämpft würdet, würden wir euch doch beistehen!" Und ALLAH bezeugt, daß sie bestimmt Lügner sind.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxviii{Würden sie vertrieben, werden sie mit ihnen nicht weggehen, und würden sie bekämpft, werden sie ihnen nicht beistehen, und würden sie ihnen beistehen, würden sie doch den Rücken abkehren, dann wird ihnen nicht beigestanden.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcxxxix{Gewiß, ihr seid furchterregender in ihren Herzen als ALLAH. Dies, weil sie doch Leute sind, die nicht begreifen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcxl{Sie kämpfen gegen euch nicht gemeinsam außer in befestigten Ortschaften oder von hinter Mauern. Ihre Gewalttätigkeit untereinander ist heftig. Du vermutest sie vereint, während ihre Herzen verschieden sind. Dies, weil sie doch Leute sind, die sich nicht besinnen.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcxli{Gleich denjenigen vor ihnen vor kurzer Zeit, sie erfuhren die bösen Konsequenzen ihrer Angelegenheit, und für sie ist qualvolle Peinigung bestimmt.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcxlii{Gleich dem Satan, als er dem Menschen sagte: ``Betreibe Kufr!" Und nachdem er Kufr betrieben hatte, sagte er: ``Gewiß, ich sage mich los von dir. Ich fürchte mich vor ALLAH, Dem HERRN aller Schöpfung."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcxliii{So war das Anschließende von beiden, daß sie im Feuer darin ewig sind. Und dies ist die Vergeltung der Unrecht-Begehenden.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcxliv{Ihr, die den Iman verinnerlicht habt! Handelt Taqwa gemäß ALLAH gegenüber, und eine Seele soll danach schauen, was sie für morgen vorausschickte! Und handelt Taqwa gemäß ALLAH gegenüber! Gewiß, ALLAH ist dessen allkundig, was ihr tut.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcxlv{Und seid nicht wie diejenigen, die ALLAH vergaßen, dann ließ ER sie sich selbst vergessen. Diese sind die wirklichen Fasiq.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcxlvi{Nicht gleich sind die Weggenossen des Feuers und die Weggenossen der Dschanna. Die Weggenossen der Dschanna sind die wirklichen Gewinner.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcxlvii{Würden WIR diesen Quran auf einen Berg hinabsenden, gewiß, würdest du ihn demütig, sich aufspaltend von der Ehrfurcht ALLAH gegenüber sehen. Und diese Gleichnisse prägen WIR den Menschen, damit sie nachdenken.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcxlviii{ER ist ALLAH, Derjenige, außer Dem es keine Gottheit gibt, Der Allwissende über das Verborgene und das Sichtbare. ER ist Der Allgnade Erweisende, Der Allgnädige.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcxlix{ER ist ALLAH, Derjenige, außer Dem es keine Gottheit gibt, Der Herrscher, Der absolut Reine, Der absolut Makellose, Der Sicherheit Gewährende, Der über alles Verfügende, Der Allwürdige, Der Allbezwingende, Der über allem Stehende. Subhanallah über das, was sie an Schirk betreiben.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcl{ER ist ALLAH, Der Erschaffende, Der Existenz Verleihende, Der Formende. Ihm gehören die Schönsten Namen. Alles, was in den Himmeln und auf Erden ist, lobpreist Ihn. Und ER ist Der Allwürdige, Der Allweise.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcli{\basmalahde Ihr, die den Iman verinnerlicht habt! Nehmt euch Meinen Feind und euren Feind nicht als Wali. Ihr erweist ihnen Liebe, während sie bereits Kufr dem gegenüber betrieben haben, was zu euch von derWahrheit kam. Sie vertreiben den Gesandten und euch, weil ihr den Iman an ALLAH, euren HERRN verinnerlicht habt. Solltet ihr dschihad-leistend auf Meinem Weg und im Streben nach Meinem Wohlgefallen weggegangen sein, (so nehmt sie nicht alsWali). Ihr bekundet ihnen heimlich Liebe, und ICH weiß besser Bescheid über das, was ihr verheimlicht und was ihr offenlegt. Und wer es tut von euch, so verfehlte er bereits den richtigen Weg.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmclii{Wenn sie euch überwältigen, werden sie für euch Feinde sein, und ihre Hände und ihre Zungen mit dem Bösen gegen euch strecken. Und sie wünschten sich, daß ihr Kufr betreibt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcliii{Euch werden weder eure Verwandten noch eure Kinder nützen. Am Tag der Auferstehung richtet ALLAH unter euch. Und ALLAH ist dessen, was ihr tut, allsehend.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcliv{Bereits gab es für euch ein schönes Vorbild an Ibrahim und an denjenigen mit ihm, als sie zu ihren Leuten sagten: `` Gewiß, wir sagen uns los von euch und von dem, dem ihr anstelle von ALLAH dient. Wir betreiben Kufr euch gegenüber, und zwischen uns und euch sind sichtbar die Feindschaft und die Abneigung für immer, bis ihr den Iman an ALLAH alleine verinnerlicht - außer der Äußerung von Ibrahim seinem Vater gegenüber: ``Ich werde doch für dich um Vergebung bitten, aber ich verfüge für dich bei ALLAH über nichts." - Unser HERR! Dir gegenüber üben wir Tawakkul, zu Dir kehren wir um und zu Dir ist das Werden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmclv{Unser HERR! Lasse uns nicht eine Fitna sein, für diejenigen, die Kufr betrieben, und vergib uns - unser HERR! - gewiß, DU bist Der Allwürdige, Der Allweise!"\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmclvi{Gewiß, bereits gab es für euch in ihnen ein schönes Vorbild für jeden, der auf ALLAH und auf den Jüngsten Tag hofft. Und wer sich abwendet, also gewiß, ALLAH ist Der absolut Autarke, Der Alllobenswürdige.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmclvii{Vielleicht läßt ALLAH zwischen euch und denjenigen von ihnen, die ihr angefeindet habt, Liebe entstehen. Und ALLAH ist allmächtig, und ALLAH ist allvergebend, allgnädig.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmclviii{ALLAH verbietet euch nicht diejenigen, die euch nicht des Din wegen bekämpften und euch nicht aus euren Wohnstätten vertrieben haben, daß ihr ihnen mit Güte begegnet und sie gerecht behandelt. Gewiß, ALLAH liebt die Gerechten.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmclix{ALLAH verbietet euch nur diejenigen, die euch des Din wegen bekämpften, euch aus euren Wohnstätten vertrieben und bei eurer Vertreibung Beistand leisteten, daß ihr sie alsWali nehmt. Und wer sie alsWali nimmt, so sind diese die eigentlichen Unrecht-Begehenden.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmclx{Ihr, die den Iman verinnerlicht habt! Wenn zu euch die Mumin-Frauen hidschra-unternehmend kommen, dann prüft sie. ALLAH weiß besser Bescheid über ihren Iman. Und wenn ihr sie als Mumin erkennt, so schickt sie nicht zu den Kafir zurück! Weder sie (die Mumin-Frauen) sind halal für sie (die Kafir), noch sie sind halal für sie. Und gebt ihnen (den Kafir), was sie (als Brautgabe) ausgaben. Und es trifft euch keine Verfehlung, wenn ihr sie heiratet, wenn ihr ihnen ihre Brautgabe gebt. Und haltet nicht die Kafir-Ehepartner! Und verlangt, was ihr ausgabt, und sie sollen verlangen, was sie ausgaben. Dies ist ALLAHs Entscheidung. ER richtet unter euch. Und ALLAH ist allwissend, allweise.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmclxi{Und sollte eine von euren Ehepartnern zu den Kafir überlaufen und ihr übt Vergeltung, dann gebt denjenigen, deren Frauen gingen, Gleiches wie das, was sie ausgaben. Und handelt Taqwa gemäß ALLAH gegenüber, an Den ihr den Iman verinnerlicht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmclxii{Prophet! Wenn die Mumin-Frauen zu dir kommen, um dir Bai'a zu geben, daß sie weder ALLAH gegenüber etwas an Schirk betreiben, noch stehlen, noch Zina begehen, noch ihre Kinder töten, noch Falsches bringen, das sie zwischen ihren Händen und Beinen erdichten und noch sich dir im Gebilligten widersetzen, dann nimm von ihnen ihre Bai'a entgegen und bitte ALLAH für sie um Vergebung. Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmclxiii{Ihr, die den Iman verinnerlicht habt! Nehmt euch nicht Leute alsWali, über die ALLAH erzürnt ist. Bereits verzweifelten sie am Jenseits, wie die Kafir an den Weggenossen der Gräber verzweifelten.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmclxiv{\basmalahde ALLAH lobpreisen alle, die in den Himmeln und auf Erden sind. Und ER ist Der Allwürdige, Der Allweise.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmclxv{Ihr, die den Iman verinnerlicht habt! Weshalb sagt ihr, was ihr nicht macht?!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmclxvi{Es ist etwas schwerwiegendes Verabscheutes bei ALLAH, daß ihr sagt, was ihr nicht macht.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmclxvii{Gewiß, ALLAH liebt diejenigen, die auf Seinem Weg in Reihe kämpfen, als wären sie ein kompakter Bau.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmclxviii{Und (erinnere daran), als Musa zu seinen Leuten sagte: ``Meine Leute! Weshalb beleidigt ihr mich, wo ihr bereits wisst, daß ich doch ALLAHs Gesandter zu euch bin?" Und als sie abwichen, ließ ALLAH ihre Herzen abweichen. Und ALLAH leitet die fisq-betreibenden Leute nicht recht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmclxix{Und (erinnere daran), als 'Isa Ibnu-Maryam sagte: ``Kinder Israil! Gewiß, ich bin ALLAHs Gesandter zu euch bestätigend dessen, was vor mir von At-taurat war, und überbringend eine frohe Botschaft von einem Gesandten, der nach mir kommt, dessen Name Ahmad (Gelobter) ist." Und als er zu ihnen mit den klaren Zeichen kam, sagten sie: ``Dies ist eine eindeutige Magie."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmclxx{Und keiner begeht mehr Unrecht als derjenige, der im Namen ALLAHs Lügen erdichtete, während er zum Islam eingeladen wird. Und ALLAH leitet die unrecht-begehenden Leute nicht recht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmclxxi{Sie wollen doch ALLAHs Licht mit ihren Mündern auslöschen. Und ALLAH wird Sein Licht vollständig machen, selbst dann, würden die Kafir demgegenüber abgeneigt sein.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmclxxii{ER ist Derjenige, Der Seinen Gesandten mit der Rechtleitung und dem Din der Wahrheit entsandte, damit ER ihn sämtliche Din übertreffen läßt, selbst dann, würden die Muschrik demgegenüber abgeneigt sein.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmclxxiii{Ihr, die den Iman verinnerlicht habt! Soll ICH euch auf einen Handel hinweisen, der euch von einer qualvollen Peinigung errettet?!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmclxxiv{Ihr verinnerlicht den Iman an ALLAH und Seinen Gesandten und leistet Dschihad fi-sabilillah mit euren Vermögensgütern und euch selbst. Dies ist besser für euch, würdet ihr es wissen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmclxxv{ER vergibt euch eure Verfehlungen und läßt euch in Dschannat eintreten, die von Flüssen durchflossen sind, sowie in schöne Wohnstätten in 'Adn-Dschannat. Dies ist der gewaltige Gewinn.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmclxxvi{Und noch etwas anderes, welches ihr liebt - Beistand von ALLAH und ein naher Sieg. Und überbringe den Mumin frohe Botschaft!\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmclxxvii{Ihr, die den Iman verinnerlicht habt! Seid Beistehende für ALLAHs (Din), wie 'Isa Ibnu-Maryam zu den Jüngern sagte: ``Wer sind meine Beistehenden zu ALLAH?" Die Jünger sagten: ``Wir sind die Beistehenden für ALLAHs (Din)." Dann verinnerlichte eine Gruppe von den Kindern Israils den Iman, und es betrieb Kufr eine (andere) Gruppe. Dann unterstützten WIR diejenigen, die den Iman verinnerlichten, gegen ihren Feind, dann wurden sie Übertreffende.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmclxxviii{\basmalahde ALLAH lobpreisen alle, die in den Himmeln und auf Erden sind, Den Herrscher, Den absolut Reinen, Den Allwürdigen, Den Allweisen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmclxxix{ER ist Derjenige, Der zu den Analphabeten einen Gesandten von ihnen schickte, der ihnen Seine Ayat vorträgt, sie reinigt und sie die Schrift und die Weisheit lehrt. Und sie waren vorher doch in einem eindeutigen Irregehen.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmclxxx{Ebenfalls zu anderen von ihnen, die sich ihnen noch nicht anschlossen. Und ER ist Der Allwürdige, Der Allweise.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxi{Dies ist ALLAHs Gunst, die ER gewährt, wem ER will. Und ALLAH ist von unermeßlicher Gunst.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxii{Das Gleichnis derjenigen, denen At-taurat auferlegt wurde und sie dann nicht trugen, ist wie das Gleichnis des Esels, der Bücher trägt. Erbärmlich ist das Gleichnis der Leute, die ALLAHs Ayat ableugneten. Und ALLAH leitet die unrecht-begehenden Leute nicht recht.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxiii{Sag: ``Ihr, die Juden wurden! Wenn ihr behauptet, daß ihr Wali für ALLAH anstelle aller Menschen seid, dann wünscht euch den Tod, wäret ihr wahrhaftig."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxiv{Und sie wünschen sich ihn niemals wegen dem, was sie eigenhändig vorleisteten. Und ALLAH ist allwissend über die Unrecht-Begehenden.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxv{Sag: ``Gewiß, der Tod, vor dem ihr flieht, wird euch doch noch begegnen, dann werdet ihr zu Dem Allwissenden über das Verborgene und das Sichtbare zurückgeschickt, dann wird ER euch Mitteilung machen über das, was ihr zu tun pflegtet.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxvi{Ihr, die den Iman verinnerlicht habt! Wenn zum rituellen Gebet vom Freitag gerufen wurde, dann strebt zum Gedenken ALLAHs und lasst vom Geschäftsbetrieb ab. Das ist besser für euch, würdet ihr es wissen.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxvii{Und wenn das rituelle Gebet beendet wurde, dann zerstreut euch im Lande, strebt von ALLAHs Gunst an und gedenkt ALLAHs viel, damit ihr erfolgreich werdet.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxviii{Und als sie Handel oder Vergnügen sahen, gingen sie eilig dorthin und ließen dich stehend. Sag: ``Das, was bei ALLAH ist, ist besser als das Vergnügen und als der Handel. Und ALLAH ist Der Beste der Rizq-Gewährenden."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmclxxxix{\basmalahde Als die Munafiq zu dir kamen, sagten sie: ``Wir bezeugen, daß du doch ALLAHs Gesandter bist." Und ALLAH weiß, daß du doch Sein Gesandter bist. Und ALLAH bezeugt: ``Gewiß, die Munafiq sind doch Lügner."\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcxc{Sie nahmen sich ihre Eide als Schutzschild, dann hielten sie von ALLAHs Weg ab. Gewiß, es wurde schlimm, was sie zu tun pflegten.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcxci{Dies, weil sie den Iman bekundeten, dann Kufr betrieben, so wurden ihre Herzen versiegelt, so begreifen sie nicht.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcxcii{Und wenn du sie siehst, gefallen dir ihre Figuren, und wenn sie sprechen, hörst du ihrem Gesagten zu. Gewiß, sie ähneln gestützen Hölzern, sie vermuten jeden Schrei gegen sie. Sie sind der wahre Feind, so nimm dich vor ihnen in Acht! ALLAH bekämpfe sie. Wie werden sie denn abgebracht?!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcxciii{Und wenn ihnen gesagt wird: ``Kommt, damit ALLAHs Gesandter für euch um Vergebung bittet!", wenden sie ihre Köpfe weg. Und du siehst sie sich abwenden, während sie sich in Arroganz erheben.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcxciv{Es ist für sie gleich, ob du für sie um Vergebung bittest oder für sie nicht um Vergebung bittest. ALLAH wird ihnen nie vergeben! Gewiß, ALLAH leitet die fisq-betreibenden Leute nicht recht.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcxcv{Sie sind diejenigen, die sagen: ``Spendet nicht für diejenigen, die bei ALLAHs Gesandten sind, damit sie auseinandergehen." Und ALLAH gehören die Magazine der Himmel und der Erde. Doch die Munafiq begreifen es nicht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcxcvi{Sie sagen: ``Wenn wir zur Madina zurückkehren, dann wird doch der Würdigere von ihr den Niedrigeren vertreiben." Und die Würde gehört ALLAH, Seinem Gesandten und den Mumin. Doch die Munafiq wissen es nicht.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcxcvii{Ihr, die den Iman verinnerlicht habt! Lasst weder eure Vermögensgüter, noch eure Kinder euch von ALLAHs Gedenken ablenken! Und wer dieses tut, so sind diese die wirklichen Verlierer.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcxcviii{Und gebt vom Rizq, das WIR euch gewährten, bevor zum einem von euch der Tod kommt, dann sagt er: ``Mein HERR! Hättest DU mir doch bis zu einer nahen Frist Aufschub gewährt! - dann werde ich spenden und von den gottgefällig Guttuenden sein."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcxcix{Und ALLAH wird nie einer Seele Aufschub gewähren, wenn ihre Frist kam. Und ALLAH ist dessen allkundig, was ihr tut.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcc{\basmalahde ALLAH lobpreisen alle, die in den Himmeln und auf Erden sind. Ihm gehört die Herrschaft und Ihm gebührt alles Lob. Und ER ist über alles allmächtig.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcci{ER ist Derjenige, Der euch erschuf, so sind einige unter euch Kafir und einige unter euch Mumin. Und ALLAH ist dessen, was ihr tut, allsehend.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccii{ER erschuf die Himmel und die Erde in Gesetzmäßigkeit und gestaltete euch, dann machte eure Gestalten schön. Und zu Ihm ist das Werden.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcciii{ER kennt das in den Himmeln und auf Erden. Und ER weiß, was ihr verheimlicht und was ihr offenlegt. Und ALLAH ist allwissend über das in den Brüsten.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcciv{Kam zu euch etwa nicht die Mitteilung über diejenigen, die vorher Kufr betrieben?! Dann erfuhren sie die bösen Konsequenzen ihrer Angelegenheit, und für (sie) ist qualvolle Peinigung bestimmt.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccv{Dies, weil ihre Gesandten zu ihnen doch mit klaren Zeichen zu kommen pflegten, dann sagten sie: ``Menschen leiten uns recht?" Dann betrieben sie Kufr und wandten sich ab. Und ALLAH ist auf niemanden angewiesen. Und ALLAH ist absolut autark, alllobenswürdig.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccvi{Diejenigen, die Kufr betrieben haben, behaupteten, daß sie nicht erweckt werden. Sag: ``Doch, mit Sicherheit, bei meinem HERRN! Ihr werdet doch erweckt, dann wird euch zweifelsohne Mitteilung gemacht über das, was ihr getan habt. Und dies ist für ALLAH etwas Leichtes."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccvii{Also verinnerlicht den Iman an ALLAH, Seinen Gesandten und das Licht, das WIR hinabsandten. Und ALLAH ist dessen, was ihr tut, allkundig.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccviii{Erinnere) an den Tag, wenn er euch zum Tag des Zusammenbringens zusammenbringt. Dies ist der (Anklage-)Tag wegen Übervorteilung. Und wer den Iman verinnerlicht und gottgefällig guttut, dem vergibt ER seine gottmißfälligen Taten und läßt ihn in Dschannat eintreten, die von Flüssen durchflossen sind, ewig sind sie darin für immer. Dies ist der gewaltige Gewinn.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccix{Und diejenigen, die Kufr betrieben und Unsere Ayat ableugneten, diese sind die Weggenossen des Feuers, ewig darin sind sie. Und erbärmlich ist das Werden.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccx{Es traf nichts an Unglück außer mit ALLAHs Zustimmung. Und wer den Iman verinnerlicht, leitet ER sein Herz recht. Und ALLAH ist über alles allwissend.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccxi{Und gehorcht ALLAH und gehorcht dem Gesandten! Solltet ihr dann euch abwenden, so obliegt Unserem Gesandten nur das deutliche Verkünden.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccxii{ER ist ALLAH, es gibt keine Gottheit außer Ihm. Und ALLAH gegenüber sollen die Mumin Tawakkul üben!\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmccxiii{Ihr, die den Iman verinnerlicht habt! Gewiß, von euren Ehepartnern und euren Kindern gibt es Feinde für euch, so nehmt euch in Acht vor ihnen! Und wenn ihr es erlasst, nachseht und vergebt, also gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmccxiv{Eure Vermögensgüter und eure Kinder sind nur Fitna. Und bei ALLAH gibt es gewaltige Belohnung.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmccxv{Also handelt Taqwa gemäß ALLAH gegenüber, wie ihr könnt, hört zu, gehorcht und spendet Gutes für euch selbst! Und wer vor dem Geiz seines Ego geschützt wird, diese sind die wirklichen Erfolgreichen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmccxvi{Wenn ihr ALLAH (Seiner Sache) eine Hasan-Anleihe gebt, wird ER euch vervielfachen und euch vergeben. Und ALLAH ist reichlich belohnend, allnachsichtig.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmccxvii{ER ist Der Allwissende über das Verborgene und das Sichtbare, Der Allwürdige, Der Allweise.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmccxviii{\basmalahde Prophet! Wenn ihr die Talaq-Scheidung den Ehefrauen gegenüber vollzieht, dann vollzieht die Talaq-Scheidung ihnen gegenüber, wenn sie auf die Zeit ihrer 'Idda hinsteuern und berechnet die 'Idda genau und handelt Taqwa gemäß ALLAH gegenüber, eurem HERRN! Bringt sie nicht heraus aus ihren Wohnstätten! Und sie gehen nicht weg, außer wenn sie eine bewiesene Abscheulichkeit begehen. Und diese sind ALLAHs Richtlinien. Und wer ALLAHs Richtlinien verletzt, beging bereits Unrecht gegen sich selbst. Du weißt nicht, vielleicht läßt ALLAH danach etwas sich ereignen.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccxix{Und wenn sie ihre festgesetzte Frist erreichten, dann haltet sie in Billigkeit oder trennt euch von ihnen nach Gebilligtem, und lasst zwei Redliche von euch dies bezeugen, und haltet das Zeugnis um ALLAHs Willen ein! Mit diesem wird derjenige ermahnt, der den Iman an ALLAH und den Jüngsten Tag verinnerlicht. Und wer Taqwa gemäß ALLAH gegenüber handelt, für den macht ER einen Ausweg\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccxx{und gewährt ihm Rizq von da, womit er nicht rechnet. Und wer ALLAH gegenüber Tawakkul übt, so genügt ER ihm. Gewiß, ALLAH wird Seiner Anweisung zur Geltung verhelfen. Bereits machte ALLAH für jede Sache etwas Bestimmtes.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccxxi{Und diejenigen von euren Ehefrauen, die keineMenstruation mehr erwarten - solltet ihr Zweifel haben - so ist ihre 'Idda drei Monate lang, ebenso für diejenigen, die noch keine Menstruation hatten. Und für die Schwangeren - deren Frist endet, wenn sie entbunden haben. Und (wer) Taqwa gemäß ALLAH gegenüber handelt, dem gewährt ER in seiner Angelegenheit Erleichterung.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccxxii{Dies ist ALLAHs Anweisung, die ER euch hinabsandte. Und wer Taqwa gemäß ALLAH gegenüber handelt, dem vergibt ER seine gottmißfälligen Taten und macht für ihn die Belohnung gewaltig.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccxxiii{Lasst sie wohnen, wo ihr wohnt, eurem Vermögen nach! Und tut ihnen nichts Schädigendes an, um sie zu bedrängen. Und sollten sie schwanger sein, dann gewährt ihnen Unterhalt, bis sie entbunden haben. Und wenn sie für euch stillen, dann gebt ihnen ihren Lohn. Und ruft euch gegenseitig zum Gebilligten auf. Und wenn ihr Schwierigkeiten miteinander habt, so wird für ihn eine andere stillen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccxxiv{Der Vermögende soll seinem Vermögen nach Unterhalt gewähren! Und wem wenig Rizq gewährt wurde, der soll Unterhalt geben von dem, was ALLAH ihm gewährte. ALLAH obliegt keiner Seele außer, was ER ihr gewährte. ALLAH wird nach der Beengtheit Entspannung machen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccxxv{Und wie viele Ortschaften widersetzten sich der Anweisung ihres HERRN und Seiner Gesandten, dann rechneten WIR mit ihnen ab mit einer harten Abrechnung und peinigten WIR sie eine entsetzliche Peinigung.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccxxvi{Dann erfuhren sie die bösen Konsequenzen ihrer Angelegenheit. Und das Anschließende ihrer Angelegenheit war Verlust.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccxxvii{ALLAH bereitete ihnen eine harte Peinigung. So handelt Taqwa gemäß ALLAH gegenüber, ihr mit Verstand, die den Iman verinnerlichten! Bereits sandte ALLAH euch eine Ermahnung hinab,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccxxviii{einen Gesandten, der euch ALLAHs Ayat deutlich vorträgt, damit ER diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, von den Finsternissen ins Licht herausbringt. Und wer den Iman an ALLAH verinnerlicht und gottgefällig Gutes tut, den läßt ER in Dschannat eintreten, die von Flüssen durchflossen sind, ewig darin sind sie. Bereits schön machte ALLAH ihm das Rizq.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccxxix{ALLAH ist Derjenige, Der sieben Himmel erschuf und von der Erde ihresgleichen. Die Anweisung steigt zwischen ihnen nach und nach hinab, damit ihr wisst, daß ALLAH doch über alles allmächtig ist, und daß ALLAH doch bereits alles mit Wissen umfaßte.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccxxx{\basmalahde Prophet! Weshalb erklärst du für haram, was ALLAH dir für halal erklärte?! Strebst du etwa die Zufriedenheit deiner Ehefrauen an?! Und ALLAH ist allver-gebend, allgnädig.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxi{Bereits schrieb ALLAH euch das Entbinden von euren Eiden vor. Und ALLAH ist euer Maula. Und ER ist Der Allwissende, Der Allweise.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxii{Und (erinnere daran), als der Prophet einigen seiner Ehefrauen einen Bericht als Geheimnis anvertraute, dann sie ihn (anderen) mitteilte und ALLAH ihn darüber in Kenntnis setzte, machte er einiges davon bekannt und unterließ einiges. Und als er ihr davon Mitteilung machte, sagte sie: ``Wer teilte dir dies mit?" Er sagte: ``Mir machte Mitteilung Der Allwissende, Der Allkundige."\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxiii{Wenn ihr beide ALLAH gegenüber bereut, so wichen eure Herzen ab. Und wenn ihr euch gegen ihn verbindet, so ist ALLAH sein Maula, sowie Dschibril und die guten der Mumin, und die Engel sind danach einander folgend.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxiv{Vielleicht tauscht ihm sein HERR, wenn er euch gegenüber Talaq-Scheidung vollzieht, andere Gattinnen ein, die besser als ihr sind, die islam-praktizierend, iman-verinnerlichend, viel betend, bereuend, ALLAH dienend, fastend/hidschra-unternehmend, nicht jungfräulich und jungfräulich sind.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxv{Ihr, die den Iman verinnerlicht habt! Schützt euch selbst und die Eurigen vor einem Feuer, dessen Brennstoff die Menschen und die Steine sind, über ihm (dem Feuer) sind strenge, harte Engel, die sich ALLAH nicht in dem widersetzen, was ER ihnen gebot, und sie machen, was ihnen geboten wird.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxvi{Ihr, die Kufr betrieben habt! Entschuldigt euch heute nicht! Euch wird doch nur vergolten, was ihr zu tun pflegtet.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxvii{Ihr, die den Iman verinnerlicht habt! Bereut ALLAH gegenüber mit einer aufrichtigen Reue. Vielleicht wird euer HERR euch eure gottmißfälligen Taten tilgen und euch in Dschannat eintreten lassen, die von Flüssen durchflossen sind, an dem Tag, wenn ALLAH den Propheten und diejenigen, die mit ihm den Iman verinnerlichten, nicht erniedrigt. Ihr Licht läuft vor ihnen und an ihrer Rechten, sie sagen: ``Unser HERR! Mache uns unser Licht vollständig und vergib uns! DU bist über alles allmächtig."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxviii{Prophet! Leiste Dschihad gegen die Kafir und die Munafiq und sei streng mit ihnen! Und ihre Unterkunft ist Dschahannam. Und erbärmlich ist das Werden.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccxxxix{ALLAH prägte ein Gleichnis für diejenigen, die Kufr betrieben haben, die Ehefrau von Nuh und die Ehefrau von Lut. Beide waren bei zwei Dienern von Unseren Dienern, die gottgefällig Guttuende waren, dann begingen beide (Frauen) Verrat gegen sie, dann nutzten beide ihnen (ihren Ehefrauen) nichts vor ALLAH. Und es wurde gesagt: ``Tretet ins Feuer mit den Eintretenden ein!"\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccxl{Und ALLAH prägte ein Gleichnis für diejenigen, die den Iman verinnerlichten, die Ehefrau von Pharao, als sie sagte: ``Mein HERR! Errichte für mich bei dir ein Haus in der Dschanna und errette mich vor Pharao und seinem Tun und errette mich vor den unrechtbegehenden Leuten!"\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccxli{Ebenso Maryam Ibnatu-Imran, die ihre Keuschheit bewahrte, dann hauchten WIR in sie einen Ruhh von Uns hinein. Und sie hielt die Worte ihres HERRN und Seine Schriften für wahrhaftig und war eine der stets Dienenden.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccxlii{\basmalahde Immer allerhabener ist Derjenige, Der die Herrschaft innehat. Und ER ist über alles allmächtig,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccxliii{Der den Tod und das Leben erschuf, damit ER euch prüft, wer von euch besser im Tun ist. Und ER ist Der Allwürdige, Der Allvergebende.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccxliv{Der sieben übereinander geschichtete Himmel erschuf. Du siehst in der Schöpfung Des Allgnade Erweisende nichts an Disharmonie. So siehe wiederholt hin, siehst du irgend etwas an Rissen?!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccxlv{Dann siehe zweimal wiederholt hin, so kehrt zur dir der Blick abgewiesen zurück, während er übermüdet ist.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccxlvi{Und gewiß, bereits schmückten WIR den untersten Himmel mit Leuchten und machten sie zu Steinigungen für die Satane. Und WIR bereiteten ihnen die Peinigung der Gluthitze.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccxlvii{Und für diejenigen, die ihrem HERRN gegenüber Kufr betrieben haben, ist ebenso die Peinigung von Dschahannam bestimmt. Und erbärmlich ist das Werden.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccxlviii{Wenn sie in sie hineingeworfen werden, hören sie von ihr Geschrei, während sie überkocht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccxlix{Beinahe platzt sie vor Wut. Immer wieder wenn eine Gruppe in sie hineingeworfen wird, werden diese von ihren Beauftragten gefragt: ``Kam zu euch etwa kein Warner?!"\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccl{Sie sagten: ``Doch, mit Sicherheit! Bereits kam zu uns ein Warner, dann leugneten wir ab und sagten: ``ALLAH sandte nichts nach und nach hinab, ihr seid doch nur in einem großen Irregehen."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccli{Und sie sagten: ``Hätten wir doch zugehört oder uns besonnen, würden wir nicht unter den Weggenossen der Gluthitze sein."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcclii{So gaben sie ihre Verfehlung zu. Also Vernichtung sei den Weggenossen der Gluthitze!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccliii{Gewiß, diejenigen, die Ehrfurcht vor ihrem HERRN im Verborgenen haben, für diese ist Vergebung und großer Lohn bestimmt.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccliv{Und verheimlicht euere Äußerung oder legt sie offen, gewiß, ER ist allwissend über das in den Brüsten.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcclv{Verfügt etwa Derjenige, Der erschuf, nicht über Wissen, während ER Der Allgütige, Der Allkundige ist?!\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcclvi{ER ist Derjenige, Der für euch die Erde nachgiebig machte, so zieht in ihren Gegenden umher und esst von Seinem Rizq. Und zu Ihm ist das Auferwecken.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcclvii{Fühlt ihr euch etwa sicher vor Dem, Dessen (Anweisung) im Himmel ist, daß ER mit euch die Erde versinkt, und daß sie gleich bebt?!\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcclviii{Oder fühlt ihr euch etwa sicher vor Dem, Dessen (Anweisung) im Himmel ist, daß ER über euch Sandsturm schickt?! So werdet ihr noch wissen, wie Meine Warnung ist.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcclix{Und gewiß, bereits leugneten ab diejenigen, die vor ihnen waren. Also wie war denn Meine Mißbilligung?!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcclx{Sahen sie etwa nicht über ihnen die Vögel mit ausgestreckten Flügeln, und sie ziehen sie ein?! Niemand hält sie außer Dem Allgnade Erweisenden. Gewiß, ER ist über alles allsehend.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcclxi{Oder wer ist dieser, der für euch ein Helfer ist und euch anstelle von Dem Allgnade Erweisenden beisteht?! Die Kafir sind doch nur im Täuschen.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcclxii{Oder wer ist dieser, der euch Rizq gewährt, wenn ER Sein Rizq zurückhielt?! Nein, sondern sie übertreiben in Übertretung und Abgeneigtheit.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcclxiii{Ist etwa derjenige, der mit dem Gesicht zum Boden geht, rechtgeleiteter oder derjenige, der aufrecht auf einem geradlinigen Weg geht?!\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcclxiv{Sag: ``ER ist Derjenige, Der euch erstehen ließ, und ER machte euch das Gehör, die Augen und den Verstand." Nur ein wenig erweist ihr euch dankbar.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcclxv{Sag: ``ER ist Derjenige, Der euch auf der Erde vermehren ließ. Und zu Ihm werdet ihr versammelt."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcclxvi{Und sie sagen: ``Wann ist diese Androhung, solltet ihr wahrhaftig sein?"\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcclxvii{Sag: ``DasWissen ist doch nur bei ALLAH. Und ich bin doch nur ein deutlicher Warner."\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcclxviii{Und nachdem sie diese nahe gesehen hatten, betrübt wurden die Gesichter derjenigen, die Kufr betrieben haben, und es wurde gesagt: ``Dies ist das, wonach ihr zu verlangen pflegtet."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcclxix{Sag: ``Wie seht ihr es? Wenn ALLAH mich und diejenigen mit mir zugrunde richtet oder Gnade erweist, wer gewährt den Kafir einen Schutz vor einer qualvollen Peinigung?"\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcclxx{Sag: ``ER ist Der Allgnade Erweisende. An Ihn verinnerlichten wir den Iman und Ihm gegenüber üben wir Tawakkul." Also ihr werdet doch noch wissen, wer im eindeutigen Irregehen ist.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxi{Sag: ``Wie seht ihr es? Wenn euer Wasser versiegt, wer bringt euch denn fließendesWasser?"\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxii{\basmalahde Nuun. Bei Al-qalam und dem, was sie in Zeilen schreiben!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxiii{Du bist wegen der Wohltat deines HERRN kein Geistesgestörter.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxiv{Und gewiß, für dich ist doch eine nicht ausgehende Belohnung bestimmt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxv{Und gewiß, du verfügst doch über einen vortrefflichen Charakter.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxvi{Also du wirst noch Einblick haben, und sie haben Einblick\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxvii{darüber, wer von euch der Fitna-Geplagte ist.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxviii{Gewiß, dein HERR weiß besser Bescheid über denjenigen, der von Seinem Weg abirrte, und ER weiß besser Bescheid über die Rechtgeleiteten.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxix{So höre nicht auf die Ableugner.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxx{Sie wünschten sich, du würdest dich anbiedern, dann biedern sie sich an.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxi{Und höre nicht auf jeden erniedrigten Viel-Schwörenden,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxii{der) bloßstellend, Namima verbreitend,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxiii{das Gute verhindernd, Aggression begehend, viel verfehlend\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxiv{und flegelhaft ist, auch dazu noch ein Bastard ist.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxv{Dies nur), weil er über Vermögen und Kinder verfügte.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxvi{Wenn ihm Unsere Ayat vorgetragen wurden, sagte er: ``Es sind Legenden der Früheren."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxvii{WIR werden ihn auf dem Rüssel brandmarken.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxviii{Gewiß, WIR prüften sie, wie WIR die Besitzer der Dschanna prüften, als sie schworen, sie doch am Morgen abzuernten\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcclxxxix{und nichts auszulassen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmccxc{Dann wirbelte durch sie ein Wirbelndes von deinem HERRN, während sie schliefen,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmccxci{dann wurde sie wie das Abgeerntete.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmccxcii{Dann riefen sie einander am Morgen (zu):\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmccxciii{``Geht in der Frühe zu eurem Ackerland, solltet ihr Aberntende sein."\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmccxciv{Dann gingen sie los, während sie einander zuflüsterten:\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmccxcv{``Auf keinen Fall betritt sie heute zu euch ein Bedürftiger."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmccxcvi{Und sie gingen in der Frühe zum Fernhalten fähig.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmccxcvii{Und als sie sie sahen, sagten sie: ``Gewiß, wir haben uns doch geirrt.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmccxcviii{Nein, sondern wir sind Ausgeschlossene."\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmccxcix{Der Mittlere von ihnen sagte: ``Sagte ich euch nicht: würdet ihr doch nur lobpreisen?"\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmccc{Sie sagten: ``Gepriesen-erhaben ist unser HERR! Gewiß, wir waren Unrecht-Begehende."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmccci{Dann wandten sich die einen von ihnen den anderen zu, sie machten sich gegenseitig Vorwürfe,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcccii{sie sagten: ``Unser Niedergang! Gewiß, wir waren Übertretende.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmccciii{Hoffentlich tauscht uns unser HERR Besseres als sie ein. Gewiß, wir sind ALLAH gegenüber Bittende."\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmccciv{Solcherart ist die Peinigung. Und die Peinigung des Jenseits ist doch noch größer, würden sie es nur wissen.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmcccv{Gewiß, für die Muttaqi gibt es bei ihrem HERRN die Dschannat des Wohlergehens.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmcccvi{Lassen WIR etwa die Muslime wie die schwer Verfehlenden sein?!\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmcccvii{Was ist mit euch, wie urteilt ihr?!\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmcccviii{Oder verfügt ihr über eine Schrift, in der ihr lest,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmcccix{daß es euer ist, was ihr euch auswählt?!\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmcccx{Oder schulden WIR euch bis zum Tag der Auferstehung hinreichende Gelöbnisse, daß es euer ist, was ihr urteilt?!\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmcccxi{Frage sie, welcher von ihnen dies behauptet!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmcccxii{Oder haben sie etwa (ALLAH) Beigesellte?! Dann sollen sie ihre (ALLAH) Beigesellten bringen, sollten sie wahrhaftig sein!\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmcccxiii{An dem Tag, wenn etwas Schwerwiegendes aufgedeckt wird und sie zum Sudschud aufgefordert werden, dann es nicht können,\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmcccxiv{ihre Blicke sind (vor Ehrfurcht) gesenkt, und Demütigung überkommt sie. Und bereits wurden sie zum Sudschud aufgefordert, als sie heil waren.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmcccxv{Also laß Mich mit denjenigen, die diese Mitteilung ableugnen, WIR werden sie anlocken, von wo sie nicht wissen.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmcccxvi{Und ICH gewähre ihnen Aufschub. Gewiß, Meine Planung ist stark.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmcccxvii{Oder verlangst du von ihnen Lohn, so daß sie durch ihre Verpflichtung belastet sind?!\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmcccxviii{Oder haben sie das Verborgene bei sich, so schreiben sie?!\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmcccxix{Also übe dich in Geduld mit der Bestimmung deines HERRN, und sei nicht wie der Weggenosse des Fisches, als er rief, während er bedrängt war.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmcccxx{Hätte ihn eine Wohltat von seinem HERRN nicht ereilt, wäre er doch ans blanke Land geworfen, während er getadelt war.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxi{Dann erwählte ihn sein HERR, dann machte ER ihn von den gottgefällig Guttuenden.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxii{Und diejenigen, die Kufr betrieben haben, hätten dich beinahe mit ihren Blicken doch zu Fall gebracht, als sie die Ermahnung hörten, und sie sagen: ``Gewiß, er ist doch geistesgestört."\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxiii{Und er (der Quran) ist nichts anderes als Ermahnung für alle Schöpfung.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxiv{\basmalahde Al-haqqa,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxv{was ist Al-haqqa?!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxvi{Und was weißt du, was Al-haqqa ist?!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxvii{Thamud und 'Aad leugneten die Anschlagende ab.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxviii{Also hinsichtlich Thamud, so wurden sie durch die Überschreitende zugrunde gerichtet.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxix{Und hinsichtlich 'Aad, so wurden sie durch einen sehr heftigen Wirbelsturm zugrunde gerichtet.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxx{Ihn fügte ER ihnen sieben Nächte und acht Tage in Folge zu, so siehst du die Leute zu Boden niedergestreckt, als wären sie umgefallene Dattelpalmenstümpfe.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxi{Also siehst du von ihnen etwas Übriggebliebenes?!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxii{Und es kamen Pharao, diejenigen vor ihm und die Umgewandten mit dem Verwerflichen,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxiii{dann widersetzten sie sich dem Gesandten ihres HERRN, dann belangten WIR sie mit übermäßigem Belangen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxiv{Gewiß, als das Wasser überflutete, trugen WIR euch in der Schwimmenden,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxv{damit WIR sie für euch zu einer Ermahnung machen, und es ein bewußtes Ohr sich bewußt macht.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxvi{Also wenn in As-sur ein einziger Stoß gestoßen wird,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxvii{und die Erde und die Berge getragen werden, dann mit einem einzigen Stoß zerstampft werden,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxviii{an diesem Tag ereignete sich die Ereignende,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcccxxxix{und der Himmel spaltete sich, denn er ist an diesem Tag gebrechlich,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcccxl{und die Engel sind an seinen Rändern. Und Al'ahrsch von deinem HERRN tragen über ihnen an diesem Tag acht.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcccxli{An diesem Tag werdet ihr vorgestellt. Nicht ein Verborgenes bleibt von euch geheim.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlii{Also hinsichtlich desjenigen, dem sein Register mit der Rechten gegeben wird, so sagte er: ``Nehmt, lest mein Register!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcccxliii{Gewiß, ich wußte, daß ich sicherlich auf meine Abrechnung treffen werde."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcccxliv{So ist er in einem wohlgefälligen Leben,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlv{in einer hohen Dschanna,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlvi{deren Früchte nahe sind.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlvii{Esst und trinkt wohlbekömmlich für das, was ihr in den vergangenen Tagen vorgelegt habt.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlviii{Und hinsichtlich desjenigen, dem sein Register mit der Linken gegeben wird, so sagte er: ``Hätte ich doch mein Register nicht bekommen,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcccxlix{und nicht gewußt, was meine Abrechnung ist!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcccl{Wäre er (der Tod) doch das Endgültige gewesen!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcccli{Mir nützte mein Vermögen nicht.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmccclii{Meine Macht hat mich verlassen."\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcccliii{``Nehmen sollt ihr ihn, sogleich fesseln,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcccliv{dann in die Hölle hineinwerfen,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmccclv{dann in eine Kette, deren Länge siebzig Ellen ist, hineinschlüpfen lassen!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmccclvi{Gewiß, er pflegte den Iman an ALLAH, Den Allerhabenen nicht zu verinnerlichen\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmccclvii{und zur Speisung des Bedürftigen nicht anzuspornen.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmccclviii{So gibt es für ihn hier weder einen engen Freund\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmccclix{noch Speise außer von Wasser voller Eiter,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmccclx{die niemand ißt außer den Verfehlenden."\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmccclxi{Also schwöre ICH bei dem, worin ihr Einblick habt,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmccclxii{und bei dem, worin ihr keinen Einblick habt,\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmccclxiii{gewiß, er ist zweifelsohne das Wort eines edlen Entsandten,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmccclxiv{und er ist nicht das Wort eines Dichters. Ein wenig ist es, daß ihr den Iman verinnerlicht.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmccclxv{Er ist auch nicht das Wort eines Wahrsagers. Ein wenig ist es, daß ihr euch besinnt.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmccclxvi{Er ist die sukzessive Hinabsendung vom HERRN aller Schöpfung.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmccclxvii{Und hätte er in Unserem Namen etwas Erdichtetes erdichtet,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmccclxviii{gewiß hätten WIR ihm seine Rechte weggenommen,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmccclxix{dann ihm doch die Hauptschlagader durchgeschnitten.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmccclxx{So gibt es von euch keinen, der es von ihm zurückhält.\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxi{Und gewiß, er ist doch eine Ermahnung für die Muttaqi.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxii{Und gewiß, WIR wissen doch, daß unter euch Ableugnende sind.\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxiii{Und gewiß, er ist doch eine Selbstanklage für die Kafir.\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxiv{Und gewiß, er ist doch das Wahre der Gewißheit.\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxv{So lobpreise mit dem allerhabenen Namen deines HERRN!\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxvi{\basmalahde Ein Bittender bat um eine sich ereignende Peinigung\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxvii{für die Kafir, für die es keinen Zurückhaltenden gibt\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxviii{von ALLAH, Dem mit den Aufstiegen himmelswärts.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxix{Die Engel und der Ruhh steigen auf zu Ihm an einem Yaum, dessen Länge fünfzigtausend Jahre ist.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxx{So übe dich in schöner Geduld!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxi{Gewiß, sie sehen sie weit entfernt,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxii{und WIR sehen sie nahe,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxiii{am Tag, wenn der Himmel wie das Verschmolzene wird,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxiv{und die Berge wie die Wolle werden,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxv{und kein enger Freund einen engen Freund fragt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxvi{Und sie werden ihnen gezeigt. Der schwer Verfehlende wünscht sich, er könnte sich von der Peinigung an diesem Tag freikaufen mit seinen Söhnen,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxvii{mit seiner Gefährtin, mit seinem Bruder,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxviii{mit seiner Großfamilie, die ihn aufnimmt,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmccclxxxix{und mit allen auf der Erde, allesamt, dann daß dies ihn errettet.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcccxc{Nein! Es ist eine Feuerflamme,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcccxci{Abziehende der Kopfhäute,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcii{die denjenigen ruft, der den Rücken kehrte und sich abwandte\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcccxciii{und anhäufte, dann (nur für sich) behielt.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcccxciv{Gewiß, der Mensch wurde als ein ängstlich Ungeduldiger erschaffen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcv{Wenn ihn das Böse trifft, so ist er äußerst ungeduldig\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcvi{und wenn ihn das Gute trifft, ist er äußerst geizig,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcvii{außer den Betenden,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcviii{die ihr rituelles Gebet stets einhalten,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcccxcix{sowie denjenigen, in deren Vermögensgütern ein festgesetztes Recht ist\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcd{für den Bittenden und den Ausgeschlossenen,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcdi{sowie denjenigen, die den Tag des Din für wahrhaftig halten,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcdii{sowie denjenigen, die vor der Peinigung ihres HERRN ehrfurcht-erfüllt sind,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcdiii{gewiß die Peinigung ihres HERRN ist nicht absicherbar,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcdiv{sowie denjenigen, die ihre Keuschheit Bewahrende sind\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmcdv{außer vor ihren Ehepartnern oder vor denen, die ihnen gehören, denn dann gewiß sind sie nicht zu tadeln,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmcdvi{und wer darüber hinaus etwas anstrebt, so sind diese die wirklichen Übertretenden,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmcdvii{sowie denjenigen, die sich um das ihnen Anvertraute und ihre Verträge kümmern,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmcdviii{sowie denjenigen, die ihre Zeugnisse standesgemäß ablegen,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmcdix{sowie denjenigen, die ihre rituellen Gebete zu ihrer Zeit verrichten,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmcdx{diese sind in Dschannat gewürdigt.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmcdxi{Also was ist mit denjenigen, die Kufr betrieben haben, sie kommen zu dir eilend\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmcdxii{von der Rechten und von der Linken in Gruppen?!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmcdxiii{Hofft etwa jeder Mann von ihnen, daß er in eine Dschanna des Wohlergehens hineingelassen wird?!\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmcdxiv{Nein! Gewiß, WIR erschufen sie doch von dem, was sie wissen.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmcdxv{Also schwöre ICH bei dem HERRN der Osten und der Westen, gewiß, WIR sind mit Sicherheit dazu imstande,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmcdxvi{sie mit Besseren als sie eintauschen, und Uns wird nicht zuvorgekommen.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmcdxvii{So laß sie sich unterhalten und Unfug treiben, bis sie ihren Tag treffen, der ihnen angedroht wird.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmcdxviii{An dem Tag, wenn sie von den Gräbern eilend herauskommen, als würden sie zu Aufgestellten (Zeichen) spurten.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmcdxix{Ihre Blicke sind (vor Ehrfurcht) gesenkt, und Demütigung überkommt sie. Dies ist der Tag, der ihnen immer angedroht wurde.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmcdxx{\basmalahde Gewiß, WIR entsandten Nuh zu seinen Leuten: ``Warne deine Leute, bevor zu ihnen eine qualvolle Peinigung kommt."\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxi{Er sagte: ``Meine Leute! Gewiß, ich bin für euch ein deutlicher Warner.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxii{Dient ALLAH und handelt Taqwa gemäß Ihm gegenüber und gehorcht mir!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxiii{ER vergibt euch von euren Verfehlungen und gewährt euch Aufschub bis zu einer festgesetzten Frist. Gewiß, wenn ALLAHs Frist anbricht, wird nicht mehr vorgeschoben, würdet ihr doch nur wissen."\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxiv{Er sagte: ``Mein HERR! Gewiß, ich machte Da'wa unter meinen Leuten nachts und tagsüber.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxv{Doch meine Da'wa stärkte sie nur noch im Entweichen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxvi{Und immer wieder, wenn ich ihnen Da'wa machte, damit DU ihnen vergibst, steckten sie ihre Finger in ihre Ohren, umhüllten sich mit ihrer Kleidung, beharrten auf (der Verfehlung) und erhoben sich in ziemlicher Arroganz.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxvii{Dann machte ich doch ihnen Da'wa öffentlich,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxviii{dann legte ich es ihnen offen und teilte es ihnen aber auch heimlich mit.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxix{Dann sagte ich: ``Bittet euren HERRN um Vergebung! Gewiß, ER ist immer allvergebend,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxx{ER läßt den Himmel über euch reichlich regnen,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxi{versorgt euch mit Vermögensgütern und Kindern, läßt Dschannat für euch wachsen und läßt Flüsse für euch fließen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxii{Wieso interessiert ihr euch nicht dafür, ALLAH zu würdigen,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxiii{wo ER euch bereits doch in Phasen erschuf?\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxiv{Nahmt ihr etwa nicht wahr, wie ALLAH sieben übereinander geschichtete Himmel erschuf?\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxv{Und ER machte den Mond in ihnen als Licht und die Sonne als Leuchte.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxvi{Und ALLAH ließ euch von der Erde wirklich hervorbringen,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxvii{dann bringt ER euch in sie zurück und ER bringt euch dann wirklich heraus.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxviii{Und ALLAH machte euch die Erde wie das Entfaltete,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcdxxxix{damit ihr auf ihr weite Straßen baut."\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcdxl{Nuh sagte: ``Mein HERR! Gewiß, sie widersetzten sich mir und folgten demjenigen, dessen Vermögen und Kinder ihn nur am Verlust mehrten.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcdxli{Und sie planten eine äußerst argwöhnische List,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlii{und sagten: ``Verlasst eure Götter nie, auch verlasst weder Wadd, noch Suwa', noch Yaghuth, Ya'uq und Nasr!"\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcdxliii{Und bereits verleiteten sie viele. Und DU mehrst die Unrecht- Begehenden nur am Irregehen."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcdxliv{Wegen ihrer Verfehlungen wurden sie ertränkt, dann in ein Feuer hineingebracht, dann fanden sie für sich anstelle von ALLAH keine Beistehende.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlv{Und Nuh sagte: ``Mein HERR! Lasse auf der Erde von den Kafir niemanden.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlvi{Gewiß, wenn DU sie läßt, verleiten sie Deine Diener und zeugen nichts außer einem äußerst kufr-betreibenden die Verfehlungen öffentlich Begehenden.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlvii{Mein HERR! Vergib mir, meinen Eltern, jedem, der mein Haus als Mumin betritt, den Mumin-Männern und den Mumin-Frauen. Und mehre die Unrecht-Begehenden nicht außer an Vernichtung."\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlviii{\basmalahde Sag: ``Mir wurde als Wahy zuteil, daß doch eine Gruppe von den Dschinn zuhörten." Dann sagten sie: ``Gewiß, wir vernahmen einen wunderbaren Quran.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdxlix{Er leitet zur Vernunft recht. Dann verinnerlichten wir den Iman an ihn, und wir werden mit niemandem Schirk unserem HERRN gegenüber betreiben."\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdl{``Und gewiß, allerhaben ist die Herrlichkeit unseres HERRN, ER nahm Sich weder Gefährtin, noch Kinder."\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdli{``Und gewiß, der Beschränkte von uns pflegte über ALLAH nur Abweichendes zu sagen."\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcdlii{``Und gewiß, wir dachten, daß die Menschen und Dschinn nie über ALLAH Erlogenes sagen."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcdliii{``Und gewiß, einige Männer von den Menschen pflegten Schutz bei einigen Männern von den Dschinn zu suchen, dann mehrten diese sie an Unrecht."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcdliv{``Und gewiß, sie dachten wie ihr dachtet, daß ALLAH niemanden erwecken wird."\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcdlv{``Und gewiß, wir suchten den Himmel, dann fanden sie voll gefüllt mit starken Wächtern und Blitzflammen."\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcdlvi{``Und gewiß, wir pflegten auf manchen seiner Warten zum Lauschen zu sitzen, wer nun lauscht, der findet für sich eine lauernde Blitzflamme."\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcdlvii{``Und gewiß, wir wissen nicht, ob Böses für diejenigen auf der Erde gewollt wird, oder ob ihr HERR für sie Gutes wollte."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcdlviii{``Und gewiß, unter uns sind die gottgefällig Guttuenden und unter uns sind manche unter diesem (Niveau), wir gehörten verschiedenen Konfessionen an."\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcdlix{``Und gewiß, wir glaubten, daß wir ALLAH auf Erden nicht zu schaffen machen und daß wir Ihm durch Flucht nicht zu schaffen machen."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcdlx{``Und gewiß, als wir die Rechtleitung vernahmen, verinnerlichten wir den Iman daran. Und wer den Iman an seinen HERRN verinnerlicht, der fürchtet weder Minderung, noch Unrecht."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxi{``Und gewiß, unter uns sind Muslime und unter uns sind Ungerechte, und wer den Islam praktizierte, diese suchten das Vernünftige.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxii{Und hinsichtlich der Ungerechten, so sind sie für Dschahannam Brennholz."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxiii{Und hätten sie den Weg (ALLAHs) eingehalten, hätten WIR sie doch (mit) reichlich Wasser getränkt,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxiv{damit WIR sie darin der Fitna unterziehen. Und wer sich vom Gedenken seines HERRN abwendet, den läßt ER in eine aufsteigende Peinigung eintreten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxv{``Und gewiß, alle Moscheen sind für ALLAH. So richtet keine Bittgebete an irgendeinen neben ALLAH!"\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxvi{``Und gewiß, als ALLAHs Diener aufstand, an Ihn Bittgebete zu richten, beinahe wären sie ihn Niederdrückende."\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxvii{Sag: ``Ich richte Bittgebete nur an meinen HERRN und betreibe Ihm gegenüber mit Niemanden Schirk!"\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxviii{Sag: Gewiß, ich verfüge für euch weder über Schädigendes, noch über Gutes\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxix{- Sag: ``Mir wird keiner Schutz vor ALLAH gewähren, und ich werde außer bei Ihm keinen Zufluchtsort finden!" -\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxx{außer über Verkünden von ALLAH und Seiner Botschaften. Und wer sich ALLAH und Seinem Gesandten widersetzt, also gewiß ist Dschahannam für ihn bestimmt. Darin bleiben sie ewig, für immer.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxi{Dann wenn sie das sahen, was ihnen angedroht wurde, werden sie doch noch wissen, wer einen schwächeren Beistehenden hat und weniger an Anzahl ist.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxii{Sag: ``Ich weiß nicht, ob es nahe liegt, was euch angedroht wird, oder ob mein HERR dafür Zeit läßt.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxiii{ER ist Der Allwissende über das Verborgene, so zeigt ER Sein Verborgenes niemandem\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxiv{außer demjenigen, an dem ER als Gesandten Wohlgefallen findet, also gewiß läßt ER vor ihm und hinter ihm Wachende sein,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxv{damit er (der Gesandte) weiß, daß sie die Botschaft ihres HERRN bereits überbrachten. Und ER umfaßte, was sie haben, und zählte alles detailliert.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxvi{\basmalahde Muz-zamil!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxvii{Vollziehe Qiyam die ganze Nacht außer ein wenig,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxviii{die Hälfte davon oder verringere davon ein wenig,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxix{oder füge ihr etwas hinzu, und rezitiere den Quran in Tartil-Form!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxx{Gewiß, WIR werden dir ein gewichtiges Wort zukommen lassen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxi{Gewiß, das Aufstehen in der Nacht ist schwerer als Verpflichtung und besser für das (rezitierte) Wort.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxii{Gewiß, am Tage gehst du vielen Beschäftigungen nach.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxiii{Und gedenke des Namens deines HERRN und widme dich Ihm im (gottesdienstlichen) Widmen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxiv{ER ist Der HERR des Ostens und des Westens, es gibt keine Gottheit außer Ihm. So nimm Ihn als Wakil!\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxv{Und übe dich in Geduld dem gegenüber, was sie sagen und entferne dich von ihnen in schöner Entfernung!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxvi{Und laß Mich mit den reichen Ableugnern und gewähre ihnen ein wenig Zeit!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxvii{Gewiß, bei Uns gibt es schwere Fesseln und Hölle,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxviii{und erstickende Speise und qualvolle Peinigung,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcdlxxxix{an dem Tag, wenn die Erde und die Berge beben, und die Berge zu verstreuten Dünen werden.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcdxc{Gewiß, WIR entsandten zu euch einen gegen euch bezeugenden Gesandten, wie WIR zu Pharao einen Gesandten entsandten.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcdxci{Dann widersetzte sich Pharao den Gesandten, dann belangten WIR ihn mit einem schweren Belangen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcii{Denn wie schützt ihr euch, wenn ihr Kufr betrieben habt, vor einem Tag, der den Kindern graue Haare bringt?!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcdxciii{Der Himmel spaltet sich davon. Seine Androhung wird gewiß erfüllt.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcdxciv{Gewiß, dies ist eine Ermahnung. Wer will, schlägt einen Weg zu seinem HERRN ein.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcv{Gewiß, dein HERR weiß, daß du doch Qiyam weniger als zwei Drittel der Nacht, die Hälfte davon und ein Drittel davon vollziehst, sowie eine Gruppe von denjenigen mit dir. Und ALLAH bestimmt die Nacht und den Tag. ER wußte, daß ihr es nicht detailliert erfassen könnt, so verzieh ER euch. So rezitiert das vom Quran, wozu ihr imstande seid. ER wußte, daß unter euch Kranke sein werden sowie andere, die im Lande umherreisen, sie streben etwas von ALLAHs Gunst an, sowie andere, die fi-sabilillah kämpfen, so rezitiert davon, wozu ihr imstande seid. Und verrichtet ordnungsgemäß das rituelle Gebet, entrichtet die Zakat und gewährt für ALLAHs (Din) eine Hasan-Anleihe! Und was ihr euch selbst an Gutem vorlegt, findet ihr bei ALLAH besser und gewaltiger an Belohnung. Und bittet ALLAH um Vergebung! Gewiß, ALLAH ist allvergebend, allgnädig.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcvi{\basmalahde Mud-dath-thir!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcvii{Stehe auf, dann warne,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcviii{und deinen HERRN dann verherrliche mit Takbir,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcdxcix{und deine Kleidung dann mache rituell rein,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmd{und die Götzen dann meide!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdi{Und mache keine Vorhaltung, umMehrung zu erhalten.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdii{Und für deinen HERRN dann übe dich in Geduld!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdiii{Also wenn in den Naquur gestoßen wird,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdiv{so ist dies an diesem Tag ein schwerer Tag\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdv{für die Kafir, nicht leicht.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdvi{Lasst Mich mit demjenigen, den ICH als Einzelnen erschuf,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdvii{und dem ICH großes Vermögen gewährte\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdviii{sowie anwesende Kinder,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdix{und dem ICH (den Weg zum Ruhm) ebnete.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdx{Dann hofft er, daß ICH ihm noch mehr gebe.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdxi{Nein! Gewiß, er war Unseren Ayat gegenüber trotzig.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdxii{ICH werde ihn durch schwere Hürde ermatten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdxiii{Gewiß, er dachte und schätzte ein.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdxiv{Also Tod sei ihm, wie er einschätzte.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdxv{Dann Tod sei ihm, wie er einschätzte.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdxvi{Dann schaute er,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdxvii{dann runzelte er die Stirn und finster wurde sein Gesicht,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdxviii{dann kehrte er den Rücken und erhob sich in Arroganz,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdxix{dann sagte er: ``Dies ist doch nichts anderes als überlieferte Magie.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdxx{Dies ist doch nur Menschenwort."\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdxxi{ICH werde ihn in Saqar hineinwerfen.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdxxii{Und was weißt du, was Saqar ist?!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdxxiii{Sie läßt nichts übrig und läßt nichts aus.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdxxiv{Sie verbrennt die Oberfläche der Haut.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdxxv{Über ihr sind neunzehn.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdxxvi{Und WIR ließen die Verantwortlichen des Feuers nichts anderes als Engel sein. Und WIR ließen ihre Anzahl nichts anderes als Fitna für diejenigen sein, die Kufr betrieben haben. Damit diejenigen, denen die Schrift zuteil wurde, Gewißheit haben, und diejenigen, die den Iman verinnerlichten, noch mehr Iman haben, und diejenigen, denen die Schrift zuteil wurde, und die Mumin nicht zweifeln. Und damit diejenigen, in deren Herzen Krankheit ist, und diejenigen, die Kufr betrieben haben, sagen: ``Was wollte ALLAH damit als Gleichnis?" Solcherart läßt ALLAH abirren, wen ER will, und ER leitet recht, wen ER will. Und die Soldaten deines HERRN kennt niemand außer Ihm. Und es ist nichts anderes als eine Ermahnung für die Menschen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdxxvii{Nein! Bei dem Mond!\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdxxviii{Bei der Nacht, wenn sie vergeht!\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdxxix{Bei dem Morgen, wenn er strahlt!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdxxx{Sie (Saqar) ist eine der größten (Katastrophen),\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxi{als Warnung für die Menschen,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxii{für denjenigen von euch, der vorgehen oder zurückgehen will.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxiii{Jede Seele ist von dem, was sie erwarb abhängig\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxiv{außer den Weggenossen der Rechten.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxv{Sie sind in Dschannat und fragen einander\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxvi{nach den schwer Verfehlenden:\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxvii{``Was brachte euch in Saqar hinein?"\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxviii{Sie sagten: ``Wir pflegten nicht von den des rituellen Gebets Verrichtenden zu sein,\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmdxxxix{und wir pflegten nicht den Bedürftigen zu speisen,\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmdxl{und wir pflegten mit den des Sinnlosen Schwätzenden Sinnloses zu schwätzen,\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmdxli{und wir pflegten den Tag des Din abzuleugnen,\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmdxlii{bis zu uns die Gewißheit kam."\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmdxliii{So nützt ihnen nicht die Fürbitte der Fürbittenden.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmdxliv{Was ist mit ihnen, sich von der Ermahnung abwendend,\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmdxlv{als wären sie erschreckte Esel,\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmdxlvi{die vor einem Löwen flohen?!\qt@no{51}}%
\qt@newcmd\qurantrans@de@mmmmmdxlvii{Nein, sondern jeder Mann von ihnen will, daß ihm entfaltete Schriften zuteil werden.\qt@no{52}}%
\qt@newcmd\qurantrans@de@mmmmmdxlviii{Gewiß, nein! Sondern sie fürchten nicht das Jenseits.\qt@no{53}}%
\qt@newcmd\qurantrans@de@mmmmmdxlix{Nein! Gewiß, er ist eine Ermahnung.\qt@no{54}}%
\qt@newcmd\qurantrans@de@mmmmmdl{Wer will, erinnert sich daran.\qt@no{55}}%
\qt@newcmd\qurantrans@de@mmmmmdli{Und sie erinnern sich nicht, außer wenn ALLAH will. ER ist würdig für die Taqwa und würdig für die Vergebung.\qt@no{56}}%
\qt@newcmd\qurantrans@de@mmmmmdlii{\basmalahde Nein, ICH schwöre beim Tag der Auferstehung,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdliii{nein, auch schwöre ICH bei der tadelnden Seele.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdliv{Denkt der Mensch etwa, daß WIR seine Knochen nicht zusammenfügen werden?!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdlv{Doch, mit Sicherheit! WIR sind fähig, daß WIR seine Fingerkuppen in Ordnung bringen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdlvi{Nein, sondern der Mensch will, vor Ihm Verfehlungen öffentlich begehen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdlvii{Er fragt: ``Wann ist der Tag der Auferstehung?"\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdlviii{Also wenn die Augen strahlen,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdlix{der Mond verfinstert wird\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdlx{und die Sonne und der Mond zusammengefügt werden,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdlxi{sagt der Mensch an diesem Tag: ``Wo ist der Fluchtweg?"\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdlxii{Nein! Es gibt keinen Zufluchtsort.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdlxiii{Bei deinem HERRN ist an diesem Tag der Aufenthaltsort.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdlxiv{Dem Menschen wird an diesem Tag mitgeteilt, was er vorlegte und was er aufschob.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdlxv{Nein, sondern der Mensch hat über sich selbst Einblick,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdlxvi{auch dann, wenn er seine Entschuldigungen vorbringt.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdlxvii{Bewege damit nicht deine Zunge, damit du über ihn schneller verfügst.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdlxviii{Uns obliegt seine Sammlung und sein Rezitieren.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdlxix{Wenn WIR ihn dann rezitieren ließen, dann folge seiner Rezitation!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdlxx{Dann obliegt Uns seine Erläuterung.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxi{Gewiß, nein! Sondern ihr liebt das Gegenwärtige,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxii{und vernachlässigt das Jenseits.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxiii{Es sind an diesem Tag strahlende Gesichter,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxiv{die zu ihrem HERRN aufschauen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxv{und es sind an diesem Tag verfinsterte Gesichter.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxvi{Sie glauben, daß ihnen Vernichtendes angetan wird.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxvii{Gewiß, nein! Wenn sie (die Seele) die Schlüsselbeine erreicht,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxviii{und gesagt wird: ``Wer ist Heiler?"\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxix{Und er glaubte, daß es doch die Trennung ist,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxx{und das Bein über das (andere) Bein geschlagen wird,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxi{zu deinem HERRN ist an diesem Tag das Hinbringen.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxii{Also weder hielt er es für wahrhaftig, noch verrichtete er das rituelle Gebet,\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxiii{sondern er leugnete ab und kehrte den Rücken,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxiv{dann ging er zu seiner Familie in Überheblichkeit.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxv{Es (das Böse) kam dir näher, es kam näher,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxvi{ja! Es kam dir näher, es kam näher, (so nimm dich in Acht!).\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxvii{Denkt der Mensch etwa, daß er als etwas Vernachlässigtes bleibt?!\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxviii{War er etwa nicht eine Nutfa aus Samen, der ergossen wird,\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdlxxxix{dann war er eine 'Alaqa, dann gab ER (ihm) die Schöpfung, dann machte ER (ihn) gut,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdxc{dann machte ER daraus die Zweiheit, das Männliche und das Weibliche?!\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdxci{Ist Dieser etwa nicht fähig, daß ER die Toten belebt?!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdxcii{\basmalahde Überkam den Menschen eine Weile von der Zeit, wo er nichts Erwähnbares war?!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdxciii{Gewiß, WIR erschufen den Menschen aus vermischter Nutfa, um ihn zu prüfen, dann machten WIR ihn hörend, sehend.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdxciv{Gewiß, WIR leiteten ihn zum Weg recht, entweder dankend oder äußerst kufr-betreibend.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdxcv{Gewiß, WIR bereiteten den Kafir Ketten, Fesseln und Gluthitze vor.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdxcvi{Gewiß, die Gütigen trinken vom Wein, der mit Kampfer vermischt ist,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdxcvii{aus einer Quelle, aus der ALLAHs Diener trinken, sie lassen sie kräftig sprudeln.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdxcviii{Sie erfüllen das Gelübde und fürchten einen Tag, dessen Bosheit ausgebreitet ist.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdxcix{Und sie speisen die Speise trotz Liebe ihr gegenüber den Armen, den Waisen und den Gefangengehaltenen:\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdc{``Wir speisen euch doch nur um ALLAHs Willen. Wir wollen von euch weder Lohn noch Danksagung,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdci{gewiß, wir fürchten von unserem HERRN einen dunklen, unheilvollen Tag."\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcii{Dann schützte ALLAH sie vor der Bosheit diese Tages und gewährte ihnen Strahlen und Freude,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdciii{und vergalt ihnen dafür, daß sie sich in Geduld übten Dschanna und Seide,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdciv{angelehnt darin auf Liegen. Sie sehen darin weder Sonne noch Mond.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcv{Und nahe sind über ihnen ihre Schatten und ihre Früchte wurden sehr leicht verfügbar gemacht.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcvi{Und ihnen wird serviert mit Schalen aus Silber und Kelchen, die Gläserne sind,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcvii{Gläserne aus Silber, die sie richtig schätzten.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcviii{Und ihnen wird darin zu trinken gegeben Wein, der mit Ingwer gemischt ist,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcix{aus einer Quelle darin, die Sal-sabil heißt.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcx{Und ihnen servieren ewige Dienstjungen, wenn du sie siehst, denkst du, sie wären ausgestreute Perlen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcxi{Und wenn du dort siehst, siehst du Wohlergehen und großen Reichtum.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcxii{Sie bedeckt Kleidung aus grünem Sundos und Istabraq. Und sie wurden mit Armbändern aus Silber geschmückt und ihr HERR gab ihnen reines Getränk zu trinken.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcxiii{Gewiß, dies ist für euch eine Belohnung. Und euer Bestreben wurde reichlich belohnt.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcxiv{Gewiß, WIR sandten dir den Quran in sukzessivem Hinabsenden hinab.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcxv{So übe dich in Geduld dem Urteil deines HERRN gegenüber und höre auf keinen Verfehlenden oder äußerst Kufr-Betreibenden von ihnen!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcxvi{Und gedenke des Namens deines HERRN in der Morgenfrühe und vor Sonnenuntergang!\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcxvii{Und von der Nacht, so vollziehe Sudschud für Ihn und lobpreise Ihn lange Zeit in der Nacht!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcxviii{Gewiß, diese lieben das Gegenwärtige und hinter sich lassen sie unbeachtet einen schweren Tag.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcxix{WIR erschufen sie und verstärkten ihre Gelenke. Und wenn WIR wollten, hätten WIR sie mit ihresgleichen eingetauscht.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcxx{Gewiß, dies ist eine Ermahnung. Also wer will, schlägt einen Weg zu seinem HERRN ein.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxi{Und ihr wollt nicht außer, daß ALLAH will. Gewiß, ALLAH ist immer allwissend, allweise.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxii{ER läßt, wen ER will, in Seine Gnade eintreten, und für die Unrecht-Begehenden bereitete ER qualvolle Peinigung vor.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxiii{\basmalahde Bei den nacheinander Geschickten,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxiv{dann den im Stürmen Stürmenden!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxv{Bei den im Ausbreiten Ausbreitenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxvi{dann den im Unterscheiden Unterscheidenden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxvii{dann den Ermahnendes Überbringenden,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxviii{sei es eine Entschuldigung oder Warnung!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxix{Gewiß, das, was euch angedroht wird, wird sich doch ereignen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxx{Also wenn die Sterne verwischt werden,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxi{und wenn der Himmel mit Öffnung versehen wird,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxii{und wenn die Berge zertrümmert werden,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxiii{und wenn den Gesandten ein Termin festgelegt wird,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxiv{zum Tag, zu dem ihr Aufschub gewährt wurde,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxv{zum Tag des Richtens,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxvi{und was weißt du, was der Tag des Richtens ist?!\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxvii{Niedergang ist an diesem Tag für die Ableugnenden bestimmt!\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxviii{Richteten WIR etwa nicht die Früheren zugrunde,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcxxxix{dann ihnen die Letzten folgen ließen?!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcxl{Solcherart machen WIR mit den schwer Verfehlenden.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcxli{Niedergang ist an diesem Tag für die Ableugnenden bestimmt!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlii{Erschufen WIR euch etwa nicht aus einer geringgeschätzten Flüssigkeit?!\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcxliii{Dann machten WIR sie in einem gefestigten Aufenthaltsort,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcxliv{bis zu einer bekannten Frist,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlv{so vermochten WIR es. Wie schön ist Der Vermögende!\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlvi{Niedergang ist an diesem Tag für die Ableugnenden bestimmt!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlvii{Machten WIR die Erde etwa nicht zum Sammelplatz\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlviii{von Lebenden und Toten?!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcxlix{Und WIR machten auf ihr hohe befestigende (Berge). Und WIR gaben euch süßes Wasser zu trinken.\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcl{Niedergang ist an diesem Tag für die Ableugnenden bestimmt!\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcli{Geht los zu dem, was ihr abzuleugnen pflegtet.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdclii{Geht los zu einem Schatten mit drei Säulen,\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcliii{der weder schattig ist, noch vor den Flammen schützt.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdcliv{Sie wirft mit Funken wie ein Turm,\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdclv{als wären sie gelbe Kamele.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdclvi{Niedergang ist an diesem Tag für die Ableugnenden bestimmt!\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdclvii{Dies ist der Tag, an dem sie nicht sprechen,\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdclviii{und ihnen nicht erlaubt wird, damit sie sich entschuldigen.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdclix{Niedergang ist an diesem Tag für die Ableugnenden bestimmt!\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdclx{Dies ist der Tag des Richtens. WIR sammelten euch und die Früheren.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdclxi{Also wenn ihr über List verfügt, so plant List gegen Mich!\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdclxii{Niedergang ist an diesem Tag für die Ableugnenden bestimmt!\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdclxiii{Gewiß, die Muttaqi sind heute in Schatten und an Quellen\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdclxiv{sowie mit Obst von dem, was sie begehren.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdclxv{Esst und trinkt wohlbekömmlich für das, was ihr zu tun pflegtet.\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmdclxvi{Gewiß, solcherart belohnen WIR die Muhsin.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmdclxvii{Niedergang ist an diesem Tag für die Ableugnenden bestimmt!\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmdclxviii{Esst und vergnügt euch ein wenig, ihr seid schwer Verfehlende.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmdclxix{Niedergang ist an diesem Tag für die Ableugnenden bestimmt!\qt@no{47}}%
\qt@newcmd\qurantrans@de@mmmmmdclxx{Und wenn ihnen gesagt wird: Vollzieht Ruku'!, vollziehen sie kein Ruku'.\qt@no{48}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxi{Niedergang ist an diesem Tag für die Ableugnenden bestimmt!\qt@no{49}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxii{An welches Wort nach ihm (dem Quran) werden sie denn den Iman verinnerlichen?!\qt@no{50}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxiii{\basmalahde Wonach fragen sie einander?!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxiv{Nach der gewaltigen Mitteilung,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxv{über die sie uneins sind?!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxvi{Gewiß, nein! Sie werden es noch wissen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxvii{Dann gewiß, nein! Sie werden es noch wissen.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxviii{Machten WIR die Erde etwa nicht als Unterlage,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxix{und die Berge als Pflöcke?!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxx{Und WIR erschufen euch als Zweiheiten.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxi{Auch machten WIR euren Schlaf als Ausruhendes.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxii{und WIR machten die Nacht als Umhüllendes.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxiii{Auch machten WIR den Tag als Zeit des Lebenserwerbs.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxiv{Und WIR richteten über euch sieben robuste (Himmel) ein.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxv{Auch machten WIR eine leuchtende Lampe.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxvi{Und WIR ließen von Regenwolken reichlich fließendes Wasser fallen,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxvii{damit WIR mit ihm Getreide und Gewächs hervorsprießen lassen\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxviii{sowie dichtbewachsene Dschannat.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdclxxxix{Gewiß, der Tag des Richtens ist ein festgelegter Zeitpunkt,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcxc{an dem Tag, wenn in As-sur geblasen wird, und ihr dann in Gruppen kommt,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcxci{und der Himmel geöffnet wird, und er voller Tore wird,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcii{und die Berge vergangen werden, und sie dann zu Fata Morgana werden.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcxciii{Gewiß, Dschahannam ist eine Warte,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcxciv{für die Übertretenden ein Rückkehrort.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcv{Sie verweilen darin Zeitspannen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcvi{Sie kosten darin weder Kühles, noch Getränk\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcvii{außer Siedendem und Eiter.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcviii{Es ist eine entsprechende Vergeltung.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcxcix{Gewiß, sie pflegten auf keine Abrechnung zu hoffen,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcc{und leugneten Unsere Ayat mit Ableugnen ab.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcci{Und alles erfaßten WIR detailliert in einem Register.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdccii{So kostet! WIR werden euch dann nur an Peinigung mehren.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcciii{Gewiß, für die Muttaqi gibt es einen Ort des Gewinns:\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdcciv{Gärten und Rebstöcke\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccv{sowie gleichaltrige gut Bestückte\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccvi{und gefüllte Becher.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccvii{Sie hören darin weder sinnloses Gerede, noch Lügen.\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccviii{Es ist eine Belohnung von deinem HERRN, eine ausreichende Gabe,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccix{von) Dem HERRN der Himmel, der Erde und das, was zwischen ihnen ist, (ER ist) Der Allgnade Erweisende. Sie haben mit Ihm nicht zu sprechen.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdccx{An dem Tag, wenn der Ruhh und die Engel in Reihe stehen, sie sprechen nicht außer demjenigen, dem Der Allgnade Erweisenden erlaubt, und Richtiges sagt.\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdccxi{Dies ist der wahre Tag. Also wer will, nimmt zu seinem HERRN eine Rückkehr.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdccxii{Gewiß, WIR warnten euch vor einer nahen Peinigung an dem Tag, wenn der Mensch schaut, was er eigenhändig vorlegte, und der Kafir sagt: ``Wäre ich doch nur Erde gewesen!"\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdccxiii{\basmalahde Bei den gründlichst Herausziehenden!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdccxiv{Bei den mit behutsamem Herausnehmen Herausnehmenden!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdccxv{Bei den schwebend Schwebenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccxvi{dann den im Wettlauf Führenden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdccxvii{dann den die Angelegenheit Regelnden!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdccxviii{An dem Tag, wenn das Bebende bebt,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdccxix{dann ihr die Abhängige nachfolgt,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdccxx{die Herzen an diesem Tag sind voller Furcht,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxi{ihre Blicke sind vor Ehrfurcht gesenkt,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxii{sie sagen: ``Werden wir etwa sicher zum vorherigen Zustand zurückgeführt,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxiii{auch dann nachdem wir modernde Knochen wurden?"\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxiv{Sie sagten: ``Dies ist dann sicher eine verlustreiche Umkehr."\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxv{Es ist doch nur ein einziger Schrei,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxvi{sogleich sind sie auf der flachen Ebene.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxvii{Wurde dir die Mitteilung über Musa zuteil?!\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxviii{Als sein HERR ihn im gereinigten Tuwa-Tal rief:\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxix{Gehe zu Pharao. Denn gewiß, er überschritt die Grenze,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxx{so sag: ``Möchtest du, daß du dich läuterst,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxi{und daß ich dich zu deinem HERRN rechtleite, dann du ehrfürchtig wirst?"\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxii{Dann zeigte er ihm die große Aya.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxiii{So leugnete er ab und widersetzte sich,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxiv{dann kehrte er den Rücken und ging,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxv{so versammelte er (die Leute) und rief,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxvi{dann sagte er: ``Ich bin euer hoher HERR."\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxvii{Dann belangte ALLAH ihn mit der harten Bestrafung für das Letzte und Erste.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxviii{Gewiß, darin ist doch eine Lehre für jeden, der ehrfürchtig ist.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdccxxxix{Seid ihr etwa schwieriger in der Schöpfung oder der Himmel?! ER richtete ihn ein,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdccxl{ER erhöhte seine Dicke, dann ebnete ER ihn,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdccxli{und ER verfinsterte seine Nacht und ER brachte seine Morgendämmerung hervor.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlii{Und die Erde danach dehnte ER aus.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdccxliii{ER ließ aus ihr ihr Wasser und ihre Weide herauskommen.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdccxliv{Und die Berge ließ ER festigen\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlv{als Verbrauchsgut für euch und für eure An'am.\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlvi{Wenn dann die große Katastrophe kommt,\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlvii{an diesem Tag erinnert sich der Mensch dessen, was er erstrebte,\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlviii{und die Hölle wurde hervorgebracht für den, der sieht.\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccxlix{Also hinsichtlich desjenigen, der die Grenze überschritt\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdccl{und das diesseitige Leben bevorzugte,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdccli{so ist gewiß die Hölle die Unterkunft.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdcclii{Und hinsichtlich desjenigen, der die Vorstellung vor seinem HERRN fürchtet und die Seele der Zuneigung enthalten ließ,\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdccliii{so ist gewiß die Dschanna die Unterkunft.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdccliv{Sie fragen dich nach der Stunde: ``Wann ist ihr Anbrechen?"\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdcclv{Und was hast du mit ihrer Erwähnung zu tun?!\qt@no{43}}%
\qt@newcmd\qurantrans@de@mmmmmdcclvi{Bei deinem HERRN ist das Letzte über sie.\qt@no{44}}%
\qt@newcmd\qurantrans@de@mmmmmdcclvii{Du bist doch nur ein Warner für denjenigen, der vor ihr Ehrfurcht hat.\qt@no{45}}%
\qt@newcmd\qurantrans@de@mmmmmdcclviii{Als hätten sie, wenn sie sehen, nicht verweilt außer einer Nacht und ihrer Morgendämmerung.\qt@no{46}}%
\qt@newcmd\qurantrans@de@mmmmmdcclix{\basmalahde Er runzelte die Stirn und kehrte den Rücken,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcclx{da der Blinde zu ihm kam.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxi{Und was weißt du, vielleicht wird er sich läutern,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxii{oder sich erinnern, so dann die Erinnerung ihm nützt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxiii{Hinsichtlich desjenigen, der darauf verzichtet,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxiv{dem kommst du entgegen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxv{und dir obliegt es nicht, wenn er sich nicht läutert.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxvi{Und hinsichtlich desjenigen, der zu dir kommt, der anstrebt,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxvii{während er sich ehrfürchtig erweist,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxviii{den vernachlässigst du.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxix{Gewiß, nein! Gewiß, es ist eine Ermahnung -\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxx{also wer will, erinnert sich daran -\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxi{auf gewürdigten Blättern,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxii{die gehoben und gereinigt sind,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxiii{mit Händen von Botschaftern,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxiv{die würdig, gütig-gehorsam sind.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxv{Tod sei dem Menschen, wie kufr-betreibend er ist.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxvi{Woraus erschuf ER ihn?!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxvii{Aus Nutfa erschuf ER ihn, dann bestimmte ER ihn,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxviii{dann erleichterte ER ihm den Weg,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxix{dann ließ ER ihn sterben, dann ließ ER ihn begraben,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxx{dann wenn ER will, läßt ER ihn erwecken.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxi{Gewiß, nein! Er erledigte nicht, was ER ihm gebot.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxii{So soll der Mensch zu seinem Essen schauen!\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxiii{Gewiß, WIR ließen das Wasser im Gießen gießen,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxiv{dann zerrissen WIR die Erde in Rissen,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxv{dann ließen WIR auf ihr Getreide wachsen\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxvi{sowie Rebstöcke und Gemüse,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxvii{und Olivenbäume und Dattelpalmen\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxviii{und dicht bewachsene Gärten\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdcclxxxix{und Obst und Futter\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdccxc{als Verbrauchsgut für euch und eure An'am.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccxci{Und wenn der ohrenbetäubende Schrei kommt,\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcii{an dem Tag, wenn der Mann vor seinem Bruder flieht\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccxciii{sowie (vor) seiner Mutter und seinem Vater\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccxciv{und seiner Gefährtin und seinen Kindern,\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcv{für jeden Mann von ihnen an diesem Tag gibt es eine Angelegenheit, die ihn beschäftigt.\qt@no{37}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcvi{Strahlende Gesichter gibt es an diesem Tag,\qt@no{38}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcvii{lächelnd voller Zuversicht.\qt@no{39}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcviii{Und es gibt an diesem Tag Gesichter, auf denen Staub ist.\qt@no{40}}%
\qt@newcmd\qurantrans@de@mmmmmdccxcix{Etwas Schwarzes überdeckt sie.\qt@no{41}}%
\qt@newcmd\qurantrans@de@mmmmmdccc{Diese sind die öffentlich die Verfehlung begehenden Kufr- Betreibenden.\qt@no{42}}%
\qt@newcmd\qurantrans@de@mmmmmdccci{\basmalahde Wenn die Sonne umwunden wird,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcccii{und wenn die Sterne zerfallen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdccciii{und wenn die Berge vergangen werden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccciv{und wenn die hochträchtigen Kamelweibchen vernachlässigt werden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcccv{und wenn die wilden Tiere versammelt werden,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccvi{und wenn die Meere angezündet/überflutet werden,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcccvii{und wenn die Seelen (mit ihren Leibern) gepaart werden,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcccviii{und wenn das lebendig begrabene Mädchen gefragt wird,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcccix{für welche Verfehlung es getötet wurde,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcccx{und wenn die Blätter entfaltet werden,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxi{und wenn der Himmel abgetragen wird,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxii{und wenn die Hölle entfacht wird,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxiii{und wenn die Dschanna nahe gebracht wird,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxiv{dann weiß bereits jede Seele, was sie vollbrachte.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxv{Nein, ICH schwöre bei den Sternen,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxvi{die durchlaufend, untergehend sind,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxvii{und bei der Nacht, wenn sie die Dunkelheit heraufzieht,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxviii{und bei dem Morgen, wenn er beimWerden ist,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxix{gewiß, er ist doch dasWort eines edlen Entsandten,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxx{der Kraft und Ansehen bei Demjenigen von Al'ahrsch hat,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxi{ihm gegenüber wird Gehorsam entgegengebracht und er ist vertrauenswürdig.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxii{Und euer Weggenosse ist nicht geistesgestört.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxiii{Und gewiß bereits sah er ihn am deutlichen Horizont.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxiv{Und er ist mit dem Verborgenen (Wahy) nicht enthaltsam.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxv{Und er ist nicht das Wort eines verdammten Satans!\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxvi{Wo geht ihr denn hin?!\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxvii{Er ist gewiß nur eine Ermahnung für die Schöpfung,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxviii{für denjenigen von euch, der sich geradlinig verhalten will.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxix{Und ihr wollt nicht, außer daß ALLAH will, Der HERR aller Schöpfung.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxx{\basmalahde Wenn der Himmel sich spaltet,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxi{und wenn die Sterne zerstreut sind,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxii{und wenn die Meere gesprengt werden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxiii{und wenn die Gräber ausgehoben werden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxiv{dann weiß bereits jede Seele, was sie vorlegte und aufschob.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxv{Du, Mensch! Was täuschte dich in deinem allgroßzügigen HERRN,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxvi{Der dich erschuf, dann dich zurechtmachte, dann dich aufrichtete,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxvii{in jeder Gestalt, die ER wollte, dich zusammenfügte?!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxviii{Gewiß, nein! Sondern ihr leugnet den Din ab,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxxxix{und gewiß über euch sind doch Bewahrende,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxl{die ehrwürdig registrierend sind,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxli{sie wissen, was ihr macht.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlii{Gewiß, die Gütig-Gehorsamen sind doch im Wohlergehen.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxliii{Und gewiß, die öffentlich die Verfehlung Begehenden sind doch in (der) Hölle.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxliv{Sie werden in sie am Tag des Din hineingeworfen,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlv{und sie werden von ihr nie abwesend sein.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlvi{Und was weißt du, was der Tag des Din ist?!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlvii{Dann, was weißt du, was der Tag des Din ist?!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlviii{An dem Tag, wenn keine Seele für eine Seele etwas vermag, und die Angelegenheit an diesem Tag ALLAH unterliegt.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxlix{\basmalahde Niedergang sei den beim Abmessen Betrügenden,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdcccl{diejenigen, wenn sie sich von den Menschen zumessen lassen, über das Maß Hinausgehendes nehmen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdcccli{und wenn sie ihnen zumessen oder abwiegen, vermindern.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccclii{Denken diese etwa nicht, daß sie erweckt werden\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdcccliii{zu einem gewaltigen Tag,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccliv{wenn dieMenschen für den HERRN aller Schöpfung aufstehen?!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdccclv{Gewiß, nein! Gewiß, das Register der öffentlich die Verfehlung Begehenden ist doch in Sidsch-dschin!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdccclvi{Und was weißt du, was Sidsch-dschin ist?!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdccclvii{Es ist ein versiegeltes/gekennzeichnetes Register.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdccclviii{Niedergang ist an diesem Tag für die Ableugner bestimmt,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdccclix{diejenigen, die den Tag des Din ableugnen.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdccclx{Und ihn leugnet nur jeder verfehlende Grenzüberschreitende ab.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxi{Als ihm Unsere Ayat vorgetragen wurden, sagte er: ``Dies sind die Legenden der Früheren."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxii{Gewiß, nein! Sondern es überkam ihre Herzen, was sie zu erwerben pflegten.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxiii{Gewiß, nein! Gewiß, sie sind an diesem Tag von ihrem HERRN doch abgeschirmt.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxiv{Dann gewiß, sie werden doch in Dschahannam hineingeworfen.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxv{Dann wird ihnen gesagt: ``Dies ist das, was ihr abzuleugnen pflegtet."\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxvi{Gewiß, nein! Gewiß, das Register der Gütig-Gehorsamen ist doch in 'Illiyin,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxvii{und was weißt du, was 'Illiyin ist,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxviii{Es ist ein versiegeltes/gekennzeichnetes Register,\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxix{welches die Nahestehenden bezeugen.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxx{Gewiß, die Gütig-Gehorsamen sind im Wohlergehen.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxi{Sie sind auf Liegen, sie schauen.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxii{Du erkennst in ihren Gesichtern das Strahlen des Wohlergehens.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxiii{Ihnen wird von einem puren mit Aroma versehenen Getränk zu trinken gegeben,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxiv{dessen Aroma aus Moschus ist. Und darin sollen die Konkurrierenden konkurrieren,\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxv{Und dessen Mischung ist aus Tasniem,\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxvi{eine Quelle, aus der die Nahestehenden trinken.\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxvii{Gewiß, diejenigen, die schwere Verfehlungen begingen, pflegten sich über diejenigen, die den Iman verinnerlichten, lustig zu machen.\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxviii{Und als sie an ihnen vorbeigingen, zwinkerten sie einander (vielsagend) zu.\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxix{Und als sie zu ihren Familien zurückkehrten, kehrten sie freudenerregt zurück.\qt@no{31}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxx{Und als sie sie sahen, sagten sie: ``Gewiß, diese sind doch Abirrende.\qt@no{32}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxi{Und über ihnen wurden keine Bewahrende entsandt."\qt@no{33}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxii{An diesem Tag also machen sich diejenigen, die den Iman verinnerlichten, über die Kafir lustig.\qt@no{34}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxiii{Sie sind auf Liegen, sie schauen zu:\qt@no{35}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxiv{``Wurde den Kafir vergolten, was sie zu machen pflegten?"\qt@no{36}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxv{\basmalahde Wenn der Himmel zerreißt,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxvi{und seinem HERRN ganz Ohr wird, und ihm obliegt es,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxvii{und wenn die Erde ausgedehnt wird,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxviii{und herauswirft, was in ihr ist und darauf verzichtet,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmdccclxxxix{und ihrem HERRN ganz Ohr wird, und ihr obliegt es,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxc{Mensch! Gewiß, du bist ein sich Anstrengender zu deinem HERRN mit Anstrengung, auf die du treffen wirst.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxci{Also hinsichtlich desjenigen, dem sein Register mit seiner Rechten gegeben wird,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcii{so wird er einer leichten Abrechnung unterzogen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxciii{und wird zu seiner Familie glücklich zurückkehren.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxciv{Und hinsichtlich desjenigen, dem sein Register von hinter seinem Rücken gegeben wird,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcv{so wird er nach Vernichtung rufen,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcvi{und in Gluthitze hineingeworfen.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcvii{Gewiß, er war in seiner Familie glücklich.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcviii{Gewiß, er dachte, daß er nicht zurückkehrt.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmdcccxcix{Doch, mit Sicherheit! Gewiß, sein HERR war über ihn allsehend.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcm{Nein, ICH schwöre bei der Abendröte\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmi{und bei der Nacht und dem, was sie zusammenbringt,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmii{und bei dem Mond, wenn er voll wird,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmiii{ihr werdet doch von einem Zustand zum anderen aufsteigen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmiv{Weshalb verinnerlichen sie den Iman nicht?!\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcmv{Und wenn ihnen der Quran vorgetragen wird, vollziehen sie kein Sudschud.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcmvi{Nein, sondern diejenigen, die Kufr betrieben haben, leugnen ab,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcmvii{und ALLAH weiß besser Bescheid über das, was ihnen bewußt ist.\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcmviii{So überbringe ihnen die ``frohe Botschaft" über eine qualvolle Peinigung,\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcmix{außer denjenigen, die den Iman verinnerlichten und gottgefällig guttaten, für die ist eine nicht endende Belohnung bestimmt.\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcmx{\basmalahde Bei dem Himmel von Sternbildern,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmxi{und bei dem angedrohten Tag,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmxii{und bei dem Zeugen und Bezeugten,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmxiii{Tod sei den Weggenossen des Grabens.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmxiv{Das Feuer darin ist mit Brennstoff,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmxv{als sie um es herum saßen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmxvi{und sie sind über das, was sie mit den Mumin machen, Zeugen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmxvii{Und sie mißbilligten an ihnen nichts außer, daß sie den Iman verinnerlichten an ALLAH, Den Allwürdigen, Den Alllobenswürdigen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmxviii{Dem die Herrschaft der Himmel und der Erde gehört. Und ALLAH ist über alles Zeuge.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmxix{Gewiß, für diejenigen, die die Mumin-Männer und Mumin-Frauen der Fitna aussetzen, dann nicht bereuen, ist die Peinigung von Dschahannam und die Peinigung des Verbrennens bestimmt.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmxx{Gewiß, für diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, sind Dschannat bestimmt, die von Flüssen durchflossen sind. Dies ist der große Erfolg.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxi{Gewiß, das Gewalt-Antun deines HERRN ist doch heftig.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxii{Gewiß, ER ist derjenige, der beginnen und zurückkehren läßt.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxiii{Und ER ist Der Allvergebende, Der Äußerst Liebende,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxiv{Der von dem herrlichen Al'ahrsch,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxv{Der Allbewirkende dessen, was ER will.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxvi{Wurde dir die Mitteilung über die Soldaten zuteil,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxvii{über Pharao und Thamud?!\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxviii{Nein, sondern diejenigen, die Kufr betrieben haben, sind im Ableugnen.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxix{Und ALLAH ist vor ihnen allumfassend.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxx{Nein, sondern er ist ein herrlicher Quran,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxi{auf einer wohlverwahrten Tafel.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxii{\basmalahde Bei dem Himmel und dem At-tariq,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxiii{und was weißt du, was At-tariq ist?!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxiv{Es ist der strahlende/hohe Stern.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxv{Über jede Seele gibt es zweifelsohne einen Bewahrenden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxvi{So soll der Mensch schauen, woraus er erschaffen wurde.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxvii{Er wurde aus ergossener Flüssigkeit erschaffen,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxviii{die von zwischen Rückgrat und Rippen herauskommt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmxxxix{Gewiß, ER ist zu seinem Zurückbringen doch fähig,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmxl{an dem Tag, wenn das Innerste geprüft wird,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmxli{dann gibt es für ihn weder Kraft, noch Beistehenden.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlii{Bei dem Himmel mit Wiederkehr,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmxliii{und bei der Erde mit Zerreißen!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmxliv{Gewiß, er ist doch ein richtendes Wort,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlv{und er ist keine Unernsthaftigkeit.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlvi{Gewiß, sie planen List\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlvii{und ICH entgegne ihre List,\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlviii{so gewähre den Kafir Aufschub, gewähre ihnen kurzen Aufschub!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmxlix{\basmalahde Rühme den Namen deines allhöchsten HERRN,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcml{Desjenigen, Der erschuf und dann zurechtmachte,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmli{und Desjenigen, Der bestimmte, dann rechtleitete,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmlii{und Desjenigen, Der das Gegraste hervorbringen ließ,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmliii{und ER es dann zum grünschwarzen Vertrockneten machte.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmliv{WIR werden dich vortragen lassen, so vergiß nicht\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmlv{außer dem, was ALLAH will. Gewiß, ER kennt das Offenkundige und das, was verborgen ist.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmlvi{Und WIR führen dich leicht zur Erleichterten.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmlvii{So ermahne, wenn die Ermahnung nützt.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmlviii{Es wird sich ermahnen lassen derjenige, der Ehrfurcht hat,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmlix{und sie meidet der Unseligste.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmlx{Derjenige, der in das größte Feuer hineingeworfen wird,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxi{dann darin weder stirbt, noch lebt.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxii{Bereits erfolgreich ist derjenige, der sich läuterte\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxiii{und des Namens seines HERRN gedachte, dann das rituelle Gebet verrichtete.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxiv{Nein, sondern ihr bevorzugt das diesseitige Leben.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxv{Doch das Jenseits ist besser und bleibender.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxvi{Gewiß, dies ist doch in den ersten Schriftblättern,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxvii{den Schriftblättern von Ibrahim und Musa.\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxviii{\basmalahde Wurde dir die Mitteilung über die Umhüllende zuteil?!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxix{Es sind Gesichter an diesem Tag demütig,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxx{arbeitend, erschöpft,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxi{sie werden in ein sehr heißes Feuer hineingeworfen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxii{sie werden aus einer siedenden Quelle getränkt.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxiii{Für sie gibt es keine Speise außer von getrocknetem Dornengewächs,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxiv{das weder ernährt, noch den Hunger stillt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxv{Es sind Gesichter an diesem Tag wohlergehend,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxvi{mit ihrem Anstreben zufrieden,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxvii{in einer hohen Dschanna.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxviii{Du hörst darin kein sinnloses Gerede.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxix{Darin gibt es eine fließende Quelle.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxx{Darin sind gehobene Liegen\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxi{und bereitgestellte Kelche\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxii{und aufgereihte Kissen\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxiii{und verteilte Teppiche.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxiv{Schauen sie etwa nicht zu den Kamelen, wie sie erschaffen wurden,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxv{und zum Himmel, wie er gehoben wurde,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxvi{und zu den Bergen, wie sie hingestellt wurden,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxvii{und zu der Erde, wie sie ausgebreitet wurde?!\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxviii{So ermahne! Du bist doch nur ein Ermahner,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmcmlxxxix{du bist über sie kein Verfügender.\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmcmxc{Außer demjenigen, der den Rücken kehrte und Kufr betrieb,\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmcmxci{den wird ALLAH dann mit der größten Peinigung peinigen.\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcii{Gewiß, zu Uns ist ihre Rückkehr,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmcmxciii{dann gewiß, Uns obliegt ihre Abrechnung.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmcmxciv{\basmalahde Bei der Morgendämmerung,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcv{und bei zehn Nächten,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcvi{und bei dem Geraden und dem Ungeraden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcvii{und bei der Nacht, wenn sie vergeht!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcviii{Gibt es darin etwa einen Schwur für einen mit Verstand?!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmcmxcix{Sahst du etwa nicht, wie dein HERR verfuhr mit 'Aad,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmm{die von Iram mit den Hohen,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmi{ihresgleichen in den Ländern nicht erschaffen wurde,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmii{sowie mit Thamud, diejenigen, die den Fels im Tal ausmeißelten,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmiii{sowie mit Pharao, der von den Pyramiden,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmiv{diejenigen, die in den Städten die Grenze überschritten,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmv{dann darin das Verderben vermehrten,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmvi{dann über sie dein HERR eine Mischung von Peinigung ergoß?!\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmvii{Gewiß, dein HERR ist doch wachend.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmviii{Also hinsichtlich des Menschen, wenn ER ihn prüft und ihm Würde und Wohltat erweist, so sagt er: ``Mein HERR erwies mir Würde."\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmix{Und hinsichtlich dessen, wenn ER ihn prüft und ihm sein Rizq wenig macht, so sagt er: ``Mein HERR erniedrigte mich."\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmx{Gewiß, nein! Sondern ihr erweist dem Waisen keineWürde,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmxi{und ihr spornt zum Speisen des Bedürftigen nicht an,\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmxii{und ihr verzehrt die Erbschaft im vollständigen Verzehren,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmxiii{und ihr liebt das Vermögen in übergroßer Liebe.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmmxiv{Gewiß, nein! Wenn die Erde im Zerstampfen nach Zerstampfen zerstampft wird,\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmmxv{und (die Anweisung) deines HERRN und die Engel in Reihen gereiht kamen,\qt@no{22}}%
\qt@newcmd\qurantrans@de@mmmmmmxvi{und an diesem Tag Dschahannam herangeholt wird, an diesem Tag erinnert sich der Mensch, und wie soll ihm die Erinnerung nützen?!\qt@no{23}}%
\qt@newcmd\qurantrans@de@mmmmmmxvii{Er sagt: ``Hätte ich doch nur für mein Leben etwas vorgelegt!"\qt@no{24}}%
\qt@newcmd\qurantrans@de@mmmmmmxviii{Also an diesem Tag peinigt wie Seine Peinigung niemand,\qt@no{25}}%
\qt@newcmd\qurantrans@de@mmmmmmxix{und fesselt wie Sein Fesseln niemand.\qt@no{26}}%
\qt@newcmd\qurantrans@de@mmmmmmxx{Du, die bereits Ruhe findende Seele!\qt@no{27}}%
\qt@newcmd\qurantrans@de@mmmmmmxxi{Kehre zu deinem HERRN zufrieden und befriedigt zurück,\qt@no{28}}%
\qt@newcmd\qurantrans@de@mmmmmmxxii{so tritt unter Meinen Dienern ein,\qt@no{29}}%
\qt@newcmd\qurantrans@de@mmmmmmxxiii{und tritt in Meine Dschanna ein!\qt@no{30}}%
\qt@newcmd\qurantrans@de@mmmmmmxxiv{\basmalahde Nein, ICH schwöre bei dieser Stadt -\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmxxv{und du wohnst in dieser Stadt -\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmxxvi{und bei dem Zeugenden und dem, was er gezeugt hat!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmxxvii{Gewiß, bereits erschufen WIR den Menschen zum Anstrengen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmxxviii{Denkt er etwa, daß keiner ihn überwältigt?!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmxxix{Er sagt: ``Ich verbrauchte angehäuftes Vermögen."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmxxx{Denkt er etwa, daß keiner ihn sah?!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxi{Machten WIR ihm etwa nicht zwei Augen\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxii{sowie Zungen und zwei Lippen,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxiii{und leiteten WIR ihn auf beide Wege recht?!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxiv{So bestieg er Al-'aqaba nicht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxv{Und was weißt du, was Al-'aqaba ist?!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxvi{Es ist die Befreiung eines Unfreien,\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxvii{oder die Speisung an einem Tag von großer Hungersnot\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxviii{eines Waisen von der Verwandtschaft\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmxxxix{oder eines Bedürftigen von großer Not.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmxl{Dann war er von denjenigen, die den Iman verinnerlichten und einander zur Geduld ermahnten und einander zur Barmherzigkeit ermahnten.\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmxli{Diese sind die Weggenossen der Rechten.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmxlii{Und diejenigen, die unseren Ayat gegenüber Kufr betrieben haben, sind die Weggenossen des bösen Omens,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmxliii{über ihnen ist abgeschlossenes Feuer.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmmxliv{\basmalahde Bei der Sonne und ihrem Strahlen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmxlv{und bei dem Mond, wenn er ihr folgt,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmxlvi{und dem Tag, wenn er sie sichtbar machte,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmxlvii{und bei der Nacht, wenn sie sie umhüllt,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmxlviii{und bei dem Himmel und Dem, Der ihn einrichtete,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmxlix{und bei der Erde und Dem, Der sie ausbreitete,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmml{und bei der Seele und Dem, Der sie zurechtmachte,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmli{dann ihr sowohl Taqwa-Losigkeit als auch Taqwa eingab,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmlii{bereits erfolgreich ist derjenige, der sie rein hält,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmliii{und bereits verlor derjenige, der sie verdeckt (mit Taqwa-Losigkeit).\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmliv{Thamud leugnete wegen ihrer Grenzüberschreitung ab,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmlv{als ihr Unseligster losging,\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmlvi{dann sagte ihnen ALLAHs Gesandter: ``(Lasst) das Kamelweibchen von ALLAH und (meidet) ihre Tränke."\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmlvii{Dann bezichtigten sie ihn der Lüge, dann verletzten sie ihm (dem Kamelweibchen) die Beine, dann richtete ihr HERR sie wegen ihrer Verfehlung zugrunde, dann ließ ER es ihnen gleichmäßig sein.\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmlviii{Und ER hat vor ihrer Folge keine Furcht.\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmlix{\basmalahde Bei der Nacht, wenn sie umhüllt,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmlx{und bei dem Tag, wenn er sich enthüllt,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmlxi{und bei Dem, Der das Männliche und das Weibliche erschuf!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmlxii{Gewiß, euer Streben ist doch verschieden.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmlxiii{Also hinsichtlich desjenigen, der spendete und Taqwa gemäß handelte\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmlxiv{und das Schöne (Dschanna) für wahrhaftig hielt,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmlxv{diesem werden WIR es zum Erleichterten leicht machen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmlxvi{Und hinsichtlich desjenigen, der geizte und verzichtete (auf den Din)\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmlxvii{und das Schöne ableugnete,\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmlxviii{diesem werden WIR es zum Erschwerten leicht machen.\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmlxix{Und sein Vermögen nützt ihm nicht, wenn er zugrundegeht.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmlxx{Gewiß, Uns obliegt doch die Rechtleitung.\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxi{Und gewiß, Uns gehört doch sowohl das Letzte als auch das Erste.\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxii{Also ICH warnte euch vor einem Feuer, das lodert,\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxiii{in das niemand außer dem Unseligsten hineingeworfen wird,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxiv{derjenige, der ableugnete und den Rücken kehrte.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxv{Und es wird von ihm der Muttaqi ferngehalten,\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxvi{der sein Vermögen spendet und sich reinhält.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxvii{Und niemand hat bei ihm eine Wohltat, die zu vergelten ist,\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxviii{also (er tat es) nur im Streben nach seinem allhöchsten HERRN.\qt@no{20}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxix{Und gewiß, er wird zufrieden sein.\qt@no{21}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxx{\basmalahde Bei dem Vormittag\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxi{und bei der Nacht, wenn sie ruht!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxii{Weder verließ dich dein HERR, noch ist ER dir abgeneigt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxiii{Und gewiß, das Letzte ist für dich besser als das Erste.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxiv{Und gewiß, dir wird dein HERR (so viel) geben, so daß du zufrieden bist.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxv{Fand ER dich nicht als Waise, dann gewährte ER dir Geborgenheit?!\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxvi{Und ER fand dich irrend, so leitete ER dich recht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxvii{Und ER fand dich arm, dann machte ER dich reich.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxviii{Also hinsichtlich der Waise, so unterdrücke nicht!\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmlxxxix{Und hinsichtlich des Bittenden, so beschimpfe nicht!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmxc{Und hinsichtlich der Wohltat deines HERRN, so sprich darüber!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmxci{\basmalahde Haben WIR dir etwa nicht deine Brust erweitert,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmxcii{und dir deine Last abgenommen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmxciii{die deinen Rücken niederdrückte,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmxciv{und dein Ansehen erhöht?!\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmxcv{Also gewiß, es gibt mit der Erschwernis eine Erleichterung.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmxcvi{Gewiß, es gibt mit der Erschwernis eine Erleichterung.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmxcvii{Also wenn du entlastet bist, so strenge dich an,\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmxcviii{und deinem HERRN, so widme dich aufrichtig!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmxcix{\basmalahde Bei den Feigen und den Oliven,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmc{bei dem Tur-Berg von Sinin,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmci{bei dieser sicheren Stadt!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcii{Gewiß, bereits erschufen WIR den Menschen in bestem Aufrechtstehen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmciii{dann kehrten WIR ihn zum Untersten des Unteren zurück,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmciv{außer denjenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, so ist für sie eine nicht endende Belohnung bestimmt.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcv{Also was läßt dich noch den Din ableugnen?!\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcvi{Ist ALLAH etwa nicht Der am besten Urteilende der Urteilenden?!\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcvii{\basmalahde Rezitiere im Namen deines HERRN, der erschuf.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcviii{ER erschuf den Menschen aus 'Alaq.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcix{Rezitiere, und dein HERR ist Der Ehrwürdigste,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcx{Der (das Schreiben) mit dem Schreibgerät lehrte,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxi{ER lehrte den Menschen, was er nicht kannte.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxii{Gewiß, nein! Gewiß, der Mensch überschreitet doch die Grenze,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcxiii{wenn er sich reich sieht.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcxiv{Gewiß, zu deinem HERRN ist die Rückkehr.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcxv{Wie seht ihr denjenigen, der es verbietet\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmcxvi{einem Diener, wenn er das rituelle Gebet verrichtet?!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmcxvii{Wie seht ihr es? Sollte er rechtgeleitet sein,\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmcxviii{oder Taqwa gebieten?!\qt@no{12}}%
\qt@newcmd\qurantrans@de@mmmmmmcxix{Wie seht ihr es? Sollte er ableugnen und den Rücken kehren?!\qt@no{13}}%
\qt@newcmd\qurantrans@de@mmmmmmcxx{Wußte er etwa nicht, daß ALLAH sieht?!\qt@no{14}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxi{Gewiß, nein! Wenn er nicht aufhört, werden WIR doch die Stirnhaare zerren,\qt@no{15}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxii{Stirnhaare, die lügnerisch und verfehlend sind.\qt@no{16}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxiii{So soll er seine Vereinigung rufen!\qt@no{17}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxiv{WIR werden die strengen Engel rufen.\qt@no{18}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxv{Gewiß, nein! Höre nicht auf ihn, vollziehe Sudschud und nähere dich!\qt@no{19}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxvi{\basmalahde Gewiß, WIR sandten ihn in der Nacht von Al-qadr hinab.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxvii{Und was weißt du, was Al-qadr-Nacht ist?!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxviii{Die Al-qadr-Nacht ist besser als tausend Monate.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxix{Die Engel und der Ruhh werden in ihr mit der Zustimmung ihres HERRN wegen jeder Angelegenheit nach und nach hinabgesandt.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxx{Salam ist sie bis zum Anbruch der Morgendämmerung.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxi{\basmalahde Diejenigen von den Schriftbesitzern, die Kufr betrieben haben, und die Muschrik werden nicht loslassen, bis zu ihnen die Deutlichkeit kommt,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxii{ein Gesandter von ALLAH, der gereinigte Schriften vorträgt,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxiii{in denen wertvolle Kapitel sind.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxiv{Und diejenigen, denen die Schrift zuteil wurde, wurden nicht uneins außer, nachdem zu ihnen die Deutlichkeit gekommen war.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxv{Und ihnen wurde nicht geboten außer, daß sie ALLAH dienen - aufrichtig im Din Ihm gegenüber als Hanif - das rituelle Gebet ordnungsgemäß verrichten und die Zakat entrichten. Und dies ist der Din der Geradlinigkeit.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxvi{Gewiß, diejenigen von den Schriftbesitzern, die Kufr betrieben haben, und die Muschrik sind im Feuer von Dschahannam, sie sind darin ewig. Diese sind die Schlimmsten der Geschöpfe.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxvii{Gewiß, diejenigen, die den Iman verinnerlicht und gottgefällig Gutes getan haben, diese sind die Besten der Geschöpfe.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxviii{Ihre Belohnung bei ihrem HERRN ist 'Adn-Dschannat, die von Flüssen durchflossen sind, darin sind sie ewig, für immer. ALLAH fand Gefallen an ihnen und sie sind zufrieden mit Ihm. Dies ist für denjenigen, der seinen HERRN Ehrfurcht erweist.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcxxxix{\basmalahde Wenn die Erde mit ihrem Beben erschüttert wird,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxl{und die Erde ihre Lasten hervorbringt,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxli{und der Mensch sagt: ``Was ist los mit ihr?",\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlii{an diesem Tag teilt sie ihre Nachrichten mit,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxliii{daß dein HERR ihr Wahy zuteil werden ließ.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxliv{An diesem Tag kommen die Menschen zerstreut hervor, damit ihnen ihre Taten gezeigt werden.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlv{Also, wer das Gewicht eines Stäubchens Gutes tut, der wird es sehen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlvi{Und wer das Gewicht eines Stäubchens Böses tut, der wird es sehen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlvii{\basmalahde Bei den schnaubenden Rennenden,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlviii{dann den Funken-Schlagenden,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxlix{dann den am Morgen Anstürmenden,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcl{dann wirbelten sie damit Staub auf,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcli{dann drangen sie damit in die Mitte der Versammlung ein!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclii{Gewiß, der Mensch ist seinem HERRN gegenüber doch undankbar.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcliii{Und gewiß, er ist darüber doch ein Zeuge.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmcliv{Und gewiß, er ist doch der Liebe von Gut gegenüber überschwenglich.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclv{Weiß er etwa nicht, wenn das, was von den Gräbern aufgedeckt wird\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmclvi{und das in den Brüsten hervorgebracht wird,\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmclvii{daß gewiß ihr HERR über sie an diesem Tag doch allkundig ist?!\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmclviii{\basmalahde Es ist Al-qari'a.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclix{Was ist Al-qari'a?!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclx{Und was weißt du, was Al-qari'a ist?!\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxi{An dem Tag, wenn die Menschen wie verstreute Motten sind,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmclxii{und die Berge wie gezupfte Wolle sind.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclxiii{Also hinsichtlich desjenigen, dessen Gewogenes schwer ist,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmclxiv{so hat er ein angenehmes Leben.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmclxv{Und hinsichtlich desjenigen, dessen Gewogenes leicht ist,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclxvi{so ist seine Mutter Hawiya.\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmclxvii{Und was weißt du, was diese ist?!\qt@no{10}}%
\qt@newcmd\qurantrans@de@mmmmmmclxviii{Es ist ein glühendes Feuer.\qt@no{11}}%
\qt@newcmd\qurantrans@de@mmmmmmclxix{\basmalahde Euch abgelenkt hat (die Sucht) nach Vermehrung,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclxx{bis ihr die Gräber besucht habt.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxi{Gewiß, nein! Ihr werdet noch wissen.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxii{Dann gewiß, nein! Ihr werdet noch wissen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxiii{Gewiß, nein! Würdet ihr über dasWissen der Gewißheit verfügen,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxiv{würdet ihr bestimmt die Hölle erkennen.\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxv{Dann werdet ihr sie doch als die Gewißheit an sich sehen.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxvi{Dann werdet ihr an diesem Tag zweifelsohne für das Wohlergehen zur Rechenschaft gezogen.\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxvii{\basmalahde Bei Al-'asr!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxviii{Gewiß, der Mensch ist im Verlust\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxix{außer denjenigen, die den Iman verinnerlicht, gottgefällig Gutes getan, einander zur Wahrheit ermahnt und einander zur Geduld ermahnt haben.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxx{\basmalahde Niedergang sei jedem äußerst durch üble Nachrede Beleidigenden, äußerst durch Mimik Beleidigenden,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxi{der Vermögen anhäufte und immer wieder zählte.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxii{Er dachte, daß sein Vermögen ihn verewigt hätte.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxiii{Gewiß, nein! Er wird doch in Al-hutama geworfen.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxiv{Und was weißt du, was Al-hutama ist?!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxv{Es ist das entfachte Feuer von ALLAH,\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxvi{das über die Herzen schlägt.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxvii{Gewiß, es ist über ihnen abgeschlossen,\qt@no{8}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxviii{an ausgestreckten Säulen (gefesselt).\qt@no{9}}%
\qt@newcmd\qurantrans@de@mmmmmmclxxxix{\basmalahde Sahst du etwa nicht, was dein HERR mit den Weggenossen des Elefanten machte?!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxc{Hat ER ihre List etwa nicht fehlgehen lassen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxci{und über sie Vögel in Schwärmen geschickt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcii{die sie mit Ton-Steinen bewarfen,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxciii{dann sie wie angefressenes Laub gemacht?!\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcxciv{\basmalahde Wegen der Verbindung von Quraisch,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcv{ihrer Verbindung der Reise vom Winter und vom Sommer,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcvi{so sollen sie Dem HERRN dieses Hauses dienen,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcvii{Der sie nach Hungern speiste, und Sicherheit nach Furcht gewährte.\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcviii{\basmalahde Kennst du denjenigen, der den Din ableugnet?!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmcxcix{Also dieser ist derjenige, der die Waise wegstößt\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmcc{und zur Speisung des Bedürftigen nicht anspornt.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmcci{Also Niedergang sei den des rituellen Gebets Verrichtenden,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccii{denjenigen, die ihrem rituellen Gebet gegenüber achtlos sind,\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmcciii{denjenigen, die Riyaa betreiben\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmcciv{und die Utensilien verweigern.\qt@no{7}}%
\qt@newcmd\qurantrans@de@mmmmmmccv{\basmalahde Gewiß, WIR gaben dir Al-kauthar,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccvi{so verrichte das rituelle Gebet für deinen HERRN und schächte!\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccvii{Gewiß, dein Verleumder ist derjenige ohne Stammhalter.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccviii{\basmalahde Sag: ``Ihr Kafir!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccix{Ich diene nicht dem, was ihr dient,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccx{und ihr seid nicht Diener Dessen, Dem ich diene.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxi{Und ich bin nicht Diener Dessen, was ihr gedient habt,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxii{und ihr seid nicht Diener Dessen, Dem ich diene.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxiii{Euch ist euer Din und mir ist mein Din."\qt@no{6}}%
\qt@newcmd\qurantrans@de@mmmmmmccxiv{\basmalahde Wenn ALLAHs Beistand und der Sieg kommt\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxv{und du dieMenschen in Din von ALLAH in Scharen eintreten siehst,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxvi{so lobpreise mit deines HERRN Lob und bitte Ihn um Vergebung! Gewiß, ER ist immer äußerst reue-annehmend.\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxvii{\basmalahde Vernichtung sei Abu-lahab, Vernichtung sei ihm!\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxviii{Nicht nützte ihm sein Vermögen und nicht das, was er erwarb.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxix{Er wird in ein Feuer mit Flammen hineingeworfen\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxx{sowie seine Ehefrau, die Trägerin des Holzes,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxi{um ihren Hals ist ein Seil aus Palmenbast.\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxii{\basmalahde Sag: ``ER ist ALLAH, einzig.\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxiii{ALLAH ist As-samad.\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxiv{Nie zeugte ER und nie wurde ER gezeugt,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxv{und nie ist Ihm jemand ebenbürtig."\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxvi{\basmalahde Sag: ``Ich suche Schutz bei Dem HERRN des Morgengrauens\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxvii{vor dem Bösen dessen, was ER erschuf,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxviii{und vor dem Bösen eines Verfinsterten, wenn es anbricht,\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxix{und vor dem Bösen der Blasenden auf die Knoten,\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxx{und vor dem Bösen eines Neiders, wenn er neidet."\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxi{\basmalahde Sag: ``Ich suche Schutz beim HERRN der Menschen,\qt@no{1}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxii{bei Dem Herrscher über die Menschen,\qt@no{2}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxiii{bei Der Gottheit der Menschen\qt@no{3}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxiv{vor dem Bösen des sich versteckenden Einflüsterers -\qt@no{4}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxv{der in den Brüsten der Menschen einflüstert -\qt@no{5}}%
\qt@newcmd\qurantrans@de@mmmmmmccxxxvi{von den Dschinn und den Menschen."\qt@no{6}}%
%% End of file `qurantext-deiv.translation.def'.
